
########################################################
#
# Copyright (c) 2003-2012 by University of Queensland
# Earth Systems Science Computational Center (ESSCC)
# http://www.uq.edu.au/esscc
#
# Primary Business: Queensland, Australia
# Licensed under the Open Software License version 3.0
# http://www.opensource.org/licenses/osl-3.0.php
#
########################################################

__copyright__="""Copyright (c) 2003-2012 by University of Queensland
Earth Systems Science Computational Center (ESSCC)
http://www.uq.edu.au/esscc
Primary Business: Queensland, Australia"""
__license__="""Licensed under the Open Software License version 3.0
http://www.opensource.org/licenses/osl-3.0.php"""
__url__="https://launchpad.net/escript-finley"

"""
basic tests for util.py

:remark: use see `test_util`

:var __author__: name of author
:var __copyright__: copyrights
:var __license__: licence agreement
:var __url__: url entry point on documentation
:var __version__: version
:var __date__: date of the version
"""

__author__="Lutz Gross, l.gross@uq.edu.au"

import unittest
import numpy
import os
from esys.escript import *

try:
     ESCRIPT_WORKDIR=os.environ['ESCRIPT_WORKDIR']
except KeyError:
     ESCRIPT_WORKDIR='.'

class Test_util_base(unittest.TestCase):
   """
   basic tests on util.py
   """
   RES_TOL=1.e-7 # RES_TOLerance to compare results
   DIFF_TOL=1.e-7 # RES_TOLerance to derivatices
#=========================================================
#  File writer
#=========================================================
   def __checkContent(self,fn,ref_cont):
        cont=open(fn,'r').readlines()
        self.assertTrue(len(cont)==len(ref_cont),"wrong number of records")
        for i in range(len(cont)):
           self.assertTrue(cont[i].strip()==ref_cont[i],"wrong records %s"%i)
   def test_FileWriter_W(self):
        fn=os.path.join(ESCRIPT_WORKDIR, "filewriter_w.txt")
        self.assertRaises(IOError,FileWriter,fn="",append=False)
        f=FileWriter(fn,append=False)
        self.assertTrue(f.name==fn, "wrong file name.")
        self.assertTrue(f.mode=='w', "wrong mode")
        self.assertTrue(f.newlines==os.linesep, "wrong line seps")
        self.assertTrue(not f.closed,"file shuold not be closed.")
        f.write("line1"+f.newlines)
        f.flush()
        self.__checkContent(fn,["line1"])
        f.writelines(["line2"+f.newlines, "line3"+f.newlines])
        f.close()
        self.assertTrue(f.closed,"file shuold be closed.")
        self.__checkContent(fn,["line1", "line2", "line3"])

   def test_FileWriter_A(self):
        fn=os.path.join(ESCRIPT_WORKDIR, "filewriter_a.txt")
        if getMPIRankWorld()==0: open(fn,'w').write("line1"+os.linesep)
        self.assertRaises(IOError,FileWriter,fn="",append=True)
        f=FileWriter(fn,append=True)
        self.assertTrue(f.name==fn, "wrong file name.")
        self.assertTrue(f.mode=='a', "wrong mode")
        self.assertTrue(f.newlines==os.linesep, "wrong line seps")
        self.assertTrue(not f.closed,"file shuold not be closed.")
        f.write("line2"+f.newlines)
        f.flush()
        self.__checkContent(fn,["line1", "line2"])
        f.writelines(["line3"+f.newlines, "line4"+f.newlines])
        f.close()
        self.assertTrue(f.closed,"file shuold be closed.")
        self.__checkContent(fn,["line1", "line2", "line3", "line4"])

   def test_FileWriter_A_loc(self):
        fn=os.path.join(ESCRIPT_WORKDIR, "filewriter_a_loc.txt")
        if getMPIRankWorld()>0:
            fn2=fn+".%s"%getMPIRankWorld()
        else:
            fn2=fn
        open(fn2,'w').write("line1"+os.linesep)
        self.assertRaises(IOError,FileWriter,fn="",append=True, createLocalFiles=True)
        f=FileWriter(fn,append=True,createLocalFiles=True)
        self.assertTrue(f.name==fn, "wrong file name.")
        self.assertTrue(f.mode=='a', "wrong mode")
        self.assertTrue(f.newlines==os.linesep, "wrong line seps")
        self.assertTrue(not f.closed,"file shuold not be closed.")
        f.write("line2"+f.newlines)
        f.flush()
        self.__checkContent(fn2,["line1", "line2"])
        f.writelines(["line3"+f.newlines, "line4"+f.newlines])
        f.close()
        self.assertTrue(f.closed,"file shuold be closed.")
        self.__checkContent(fn2,["line1", "line2", "line3", "line4"])

   def test_FileWriter_W_loc(self):
        fn=os.path.join(ESCRIPT_WORKDIR, "filewriter_w_loc.txt")
        if getMPIRankWorld()>0:
            fn2=fn+".%s"%getMPIRankWorld()
        else:
            fn2=fn
        self.assertRaises(IOError,FileWriter,fn="",append=True, createLocalFiles=True)
        f=FileWriter(fn,append=False,createLocalFiles=True)
        self.assertTrue(f.name==fn, "wrong file name.")
        self.assertTrue(f.mode=='w', "wrong mode")
        self.assertTrue(f.newlines==os.linesep, "wrong line seps")
        self.assertTrue(not f.closed,"file shuold not be closed.")
        f.write("line1"+f.newlines)
        f.flush()
        self.__checkContent(fn2,["line1"])
        f.writelines(["line2"+f.newlines, "line3"+f.newlines])
        f.close()
        self.assertTrue(f.closed,"file shuold be closed.")
        self.__checkContent(fn2,["line1", "line2", "line3"])
#=========================================================
#  constants
#=========================================================
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_kronecker_1(self):
      val=kronecker(d=1)
      self.assertTrue(isinstance(val,numpy.ndarray),"wrong type of result.")
      self.assertEqual(val[0,0],1.0,"wrong value for (0,0)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_kronecker_2(self):
      val=kronecker(d=2)
      self.assertTrue(isinstance(val,numpy.ndarray),"wrong type of result.")
      self.assertEqual(val[0,0],1.0,"wrong value for (0,0)")
      self.assertEqual(val[0,1],0.0,"wrong value for (0,1)")
      self.assertEqual(val[1,0],0.0,"wrong value for (1,0)")
      self.assertEqual(val[1,1],1.0,"wrong value for (1,1)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_kronecker_3(self):
      val=kronecker(d=3)
      self.assertTrue(isinstance(val,numpy.ndarray),"wrong type of result.")
      self.assertEqual(val[0,0],1.0,"wrong value for (0,0)")
      self.assertEqual(val[0,1],0.0,"wrong value for (0,1)")
      self.assertEqual(val[0,2],0.0,"wrong value for (0,2)")
      self.assertEqual(val[1,0],0.0,"wrong value for (1,0)")
      self.assertEqual(val[1,1],1.0,"wrong value for (1,1)")
      self.assertEqual(val[1,2],0.0,"wrong value for (1,2)")
      self.assertEqual(val[2,0],0.0,"wrong value for (2,0)")
      self.assertEqual(val[2,1],0.0,"wrong value for (2,1)")
      self.assertEqual(val[2,2],1.0,"wrong value for (2,2)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_kronecker_domain(self):
      val=kronecker(d=self.domain)
      self.assertTrue(isinstance(val,numpy.ndarray),"wrong type of result.")
      self.assertEqual(val.shape,(self.domain.getDim(),self.domain.getDim()),"wrong shape.")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_kronecker_functionspace(self):
      val=kronecker(d=self.functionspace)
      self.assertTrue(isinstance(val,escript.Data),"wrong type of result.")
      self.assertEqual(val.getShape(),(self.functionspace.getDim(),self.functionspace.getDim()),"wrong shape.")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_identityTensor_1(self):
      val=identityTensor(d=1)
      self.assertTrue(isinstance(val,numpy.ndarray),"wrong type of result.")
      self.assertEqual(val[0,0],1.0,"wrong value for (0,0)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_identityTensor_2(self):
      val=identityTensor(d=2)
      self.assertTrue(isinstance(val,numpy.ndarray),"wrong type of result.")
      self.assertEqual(val[0,0],1.0,"wrong value for (0,0)")
      self.assertEqual(val[0,1],0.0,"wrong value for (0,1)")
      self.assertEqual(val[1,0],0.0,"wrong value for (1,0)")
      self.assertEqual(val[1,1],1.0,"wrong value for (1,1)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_identityTensor_3(self):
      val=identityTensor(d=3)
      self.assertTrue(isinstance(val,numpy.ndarray),"wrong type of result.")
      self.assertEqual(val[0,0],1.0,"wrong value for (0,0)")
      self.assertEqual(val[0,1],0.0,"wrong value for (0,1)")
      self.assertEqual(val[0,2],0.0,"wrong value for (0,2)")
      self.assertEqual(val[1,0],0.0,"wrong value for (1,0)")
      self.assertEqual(val[1,1],1.0,"wrong value for (1,1)")
      self.assertEqual(val[1,2],0.0,"wrong value for (1,2)")
      self.assertEqual(val[2,0],0.0,"wrong value for (2,0)")
      self.assertEqual(val[2,1],0.0,"wrong value for (2,1)")
      self.assertEqual(val[2,2],1.0,"wrong value for (2,2)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_identityTensor_domain(self):
      val=identityTensor(d=self.domain)
      self.assertTrue(isinstance(val,numpy.ndarray),"wrong type of result.")
      self.assertEqual(val.shape,(self.domain.getDim(),self.domain.getDim()),"wrong shape.")
      if self.domain.getDim()==2:
         self.assertEqual(val[0,0],1.0,"wrong value for (0,0)")
         self.assertEqual(val[0,1],0.0,"wrong value for (0,1)")
         self.assertEqual(val[1,0],0.0,"wrong value for (1,0)")
         self.assertEqual(val[1,1],1.0,"wrong value for (1,1)")
      if self.domain.getDim()==3:
         self.assertEqual(val[0,0],1.0,"wrong value for (0,0)")
         self.assertEqual(val[0,1],0.0,"wrong value for (0,1)")
         self.assertEqual(val[0,2],0.0,"wrong value for (0,2)")
         self.assertEqual(val[1,0],0.0,"wrong value for (1,0)")
         self.assertEqual(val[1,1],1.0,"wrong value for (1,1)")
         self.assertEqual(val[1,2],0.0,"wrong value for (1,2)")
         self.assertEqual(val[2,0],0.0,"wrong value for (2,0)")
         self.assertEqual(val[2,1],0.0,"wrong value for (2,1)")
         self.assertEqual(val[2,2],1.0,"wrong value for (2,2)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_identityTensor_functionspace(self):
      val=identityTensor(d=self.functionspace)
      self.assertTrue(isinstance(val,escript.Data),"wrong type of result.")
      self.assertEqual(val.getShape(),(self.functionspace.getDim(),self.functionspace.getDim()),"wrong shape.")
      if self.domain.getDim()==2:
         self.assertTrue(Lsup(val[0,0]-1.0)<=self.RES_TOL,"wrong value for (0,0)")
         self.assertTrue(Lsup(val[0,1]-0.0)<=self.RES_TOL,"wrong value for (0,1)")
         self.assertTrue(Lsup(val[1,0]-0.0)<=self.RES_TOL,"wrong value for (1,0)")
         self.assertTrue(Lsup(val[1,1]-1.0)<=self.RES_TOL,"wrong value for (1,1)")
      if self.domain.getDim()==3:
         self.assertTrue(Lsup(val[0,0]-1.0)<=self.RES_TOL,"wrong value for (0,0)")
         self.assertTrue(Lsup(val[0,1]-0.0)<=self.RES_TOL,"wrong value for (0,1)")
         self.assertTrue(Lsup(val[0,2]-0.0)<=self.RES_TOL,"wrong value for (0,2)")
         self.assertTrue(Lsup(val[1,0]-0.0)<=self.RES_TOL,"wrong value for (1,0)")
         self.assertTrue(Lsup(val[1,1]-1.0)<=self.RES_TOL,"wrong value for (1,1)")
         self.assertTrue(Lsup(val[1,2]-0.0)<=self.RES_TOL,"wrong value for (1,2)")
         self.assertTrue(Lsup(val[2,0]-0.0)<=self.RES_TOL,"wrong value for (2,0)")
         self.assertTrue(Lsup(val[2,1]-0.0)<=self.RES_TOL,"wrong value for (2,1)")
         self.assertTrue(Lsup(val[2,2]-1.0)<=self.RES_TOL,"wrong value for (2,2)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_identityTensor4_1(self):
      val=identityTensor4(d=1)
      self.assertTrue(isinstance(val,numpy.ndarray),"wrong type of result.")
      self.assertEqual(val[0,0,0,0],1.0,"wrong value for (0,0,0,0)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_identityTensor4_2(self):
      val=identityTensor4(d=2)
      self.assertTrue(isinstance(val,numpy.ndarray),"wrong type of result.")
      self.assertEqual(val[0,0,0,0],1.0,"wrong value for (0,0,0,0)")
      self.assertEqual(val[0,0,0,1],0.0,"wrong value for (0,0,0,1)")
      self.assertEqual(val[0,0,1,0],0.0,"wrong value for (0,0,1,0)")
      self.assertEqual(val[0,0,1,1],0.0,"wrong value for (0,0,1,1)")
      self.assertEqual(val[0,1,0,0],0.0,"wrong value for (0,1,0,0)")
      self.assertEqual(val[0,1,0,1],1.0,"wrong value for (0,1,0,1)")
      self.assertEqual(val[0,1,1,0],0.0,"wrong value for (0,1,1,0)")
      self.assertEqual(val[0,1,1,1],0.0,"wrong value for (0,1,1,1)")
      self.assertEqual(val[1,0,0,0],0.0,"wrong value for (1,0,0,0)")
      self.assertEqual(val[1,0,0,1],0.0,"wrong value for (1,0,0,1)")
      self.assertEqual(val[1,0,1,0],1.0,"wrong value for (1,0,1,0)")
      self.assertEqual(val[1,0,1,1],0.0,"wrong value for (1,0,1,1)")
      self.assertEqual(val[1,1,0,0],0.0,"wrong value for (1,1,0,0)")
      self.assertEqual(val[1,1,0,1],0.0,"wrong value for (1,1,0,1)")
      self.assertEqual(val[1,1,1,0],0.0,"wrong value for (1,1,1,0)")
      self.assertEqual(val[1,1,1,1],1.0,"wrong value for (1,1,1,1)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_identityTensor4_3(self):
      val=identityTensor4(d=3)
      self.assertTrue(isinstance(val,numpy.ndarray),"wrong type of result.")
      self.assertEqual(val[0,0,0,0],1.0,"wrong value for (0,0,0,0)")
      self.assertEqual(val[0,0,0,1],0.0,"wrong value for (0,0,0,1)")
      self.assertEqual(val[0,0,0,2],0.0,"wrong value for (0,0,0,2)")
      self.assertEqual(val[0,0,1,0],0.0,"wrong value for (0,0,1,0)")
      self.assertEqual(val[0,0,1,1],0.0,"wrong value for (0,0,1,1)")
      self.assertEqual(val[0,0,1,2],0.0,"wrong value for (0,0,1,2)")
      self.assertEqual(val[0,0,2,0],0.0,"wrong value for (0,0,2,0)")
      self.assertEqual(val[0,0,2,1],0.0,"wrong value for (0,0,2,1)")
      self.assertEqual(val[0,0,2,2],0.0,"wrong value for (0,0,2,2)")
      self.assertEqual(val[0,1,0,0],0.0,"wrong value for (0,1,0,0)")
      self.assertEqual(val[0,1,0,1],1.0,"wrong value for (0,1,0,1)")
      self.assertEqual(val[0,1,0,2],0.0,"wrong value for (0,1,0,2)")
      self.assertEqual(val[0,1,1,0],0.0,"wrong value for (0,1,1,0)")
      self.assertEqual(val[0,1,1,1],0.0,"wrong value for (0,1,1,1)")
      self.assertEqual(val[0,1,1,2],0.0,"wrong value for (0,1,1,2)")
      self.assertEqual(val[0,1,2,0],0.0,"wrong value for (0,1,2,0)")
      self.assertEqual(val[0,1,2,1],0.0,"wrong value for (0,1,2,1)")
      self.assertEqual(val[0,1,2,2],0.0,"wrong value for (0,1,2,2)")
      self.assertEqual(val[0,2,0,0],0.0,"wrong value for (0,2,0,0)")
      self.assertEqual(val[0,2,0,1],0.0,"wrong value for (0,2,0,1)")
      self.assertEqual(val[0,2,0,2],1.0,"wrong value for (0,2,0,2)")
      self.assertEqual(val[0,2,1,0],0.0,"wrong value for (0,2,1,0)")
      self.assertEqual(val[0,2,1,1],0.0,"wrong value for (0,2,1,1)")
      self.assertEqual(val[0,2,1,2],0.0,"wrong value for (0,2,1,2)")
      self.assertEqual(val[0,2,2,0],0.0,"wrong value for (0,2,2,0)")
      self.assertEqual(val[0,2,2,1],0.0,"wrong value for (0,2,2,1)")
      self.assertEqual(val[0,2,2,2],0.0,"wrong value for (0,2,2,2)")
      self.assertEqual(val[1,0,0,0],0.0,"wrong value for (1,0,0,0)")
      self.assertEqual(val[1,0,0,1],0.0,"wrong value for (1,0,0,1)")
      self.assertEqual(val[1,0,0,2],0.0,"wrong value for (1,0,0,2)")
      self.assertEqual(val[1,0,1,0],1.0,"wrong value for (1,0,1,0)")
      self.assertEqual(val[1,0,1,1],0.0,"wrong value for (1,0,1,1)")
      self.assertEqual(val[1,0,1,2],0.0,"wrong value for (1,0,1,2)")
      self.assertEqual(val[1,0,2,0],0.0,"wrong value for (1,0,2,0)")
      self.assertEqual(val[1,0,2,1],0.0,"wrong value for (1,0,2,1)")
      self.assertEqual(val[1,0,2,2],0.0,"wrong value for (1,0,2,2)")
      self.assertEqual(val[1,1,0,0],0.0,"wrong value for (1,1,0,0)")
      self.assertEqual(val[1,1,0,1],0.0,"wrong value for (1,1,0,1)")
      self.assertEqual(val[1,1,0,2],0.0,"wrong value for (1,1,0,2)")
      self.assertEqual(val[1,1,1,0],0.0,"wrong value for (1,1,1,0)")
      self.assertEqual(val[1,1,1,1],1.0,"wrong value for (1,1,1,1)")
      self.assertEqual(val[1,1,1,2],0.0,"wrong value for (1,1,1,2)")
      self.assertEqual(val[1,1,2,0],0.0,"wrong value for (1,1,2,0)")
      self.assertEqual(val[1,1,2,1],0.0,"wrong value for (1,1,2,1)")
      self.assertEqual(val[1,1,2,2],0.0,"wrong value for (1,1,2,2)")
      self.assertEqual(val[1,2,0,0],0.0,"wrong value for (1,2,0,0)")
      self.assertEqual(val[1,2,0,1],0.0,"wrong value for (1,2,0,1)")
      self.assertEqual(val[1,2,0,2],0.0,"wrong value for (1,2,0,2)")
      self.assertEqual(val[1,2,1,0],0.0,"wrong value for (1,2,1,0)")
      self.assertEqual(val[1,2,1,1],0.0,"wrong value for (1,2,1,1)")
      self.assertEqual(val[1,2,1,2],1.0,"wrong value for (1,2,1,2)")
      self.assertEqual(val[1,2,2,0],0.0,"wrong value for (1,2,2,0)")
      self.assertEqual(val[1,2,2,1],0.0,"wrong value for (1,2,2,1)")
      self.assertEqual(val[1,2,2,2],0.0,"wrong value for (1,2,2,2)")
      self.assertEqual(val[2,0,0,0],0.0,"wrong value for (2,0,0,0)")
      self.assertEqual(val[2,0,0,1],0.0,"wrong value for (2,0,0,1)")
      self.assertEqual(val[2,0,0,2],0.0,"wrong value for (2,0,0,2)")
      self.assertEqual(val[2,0,1,0],0.0,"wrong value for (2,0,1,0)")
      self.assertEqual(val[2,0,1,1],0.0,"wrong value for (2,0,1,1)")
      self.assertEqual(val[2,0,1,2],0.0,"wrong value for (2,0,1,2)")
      self.assertEqual(val[2,0,2,0],1.0,"wrong value for (2,0,2,0)")
      self.assertEqual(val[2,0,2,1],0.0,"wrong value for (2,0,2,1)")
      self.assertEqual(val[2,0,2,2],0.0,"wrong value for (2,0,2,2)")
      self.assertEqual(val[2,1,0,0],0.0,"wrong value for (2,1,0,0)")
      self.assertEqual(val[2,1,0,1],0.0,"wrong value for (2,1,0,1)")
      self.assertEqual(val[2,1,0,2],0.0,"wrong value for (2,1,0,2)")
      self.assertEqual(val[2,1,1,0],0.0,"wrong value for (2,1,1,0)")
      self.assertEqual(val[2,1,1,1],0.0,"wrong value for (2,1,1,1)")
      self.assertEqual(val[2,1,1,2],0.0,"wrong value for (2,1,1,2)")
      self.assertEqual(val[2,1,2,0],0.0,"wrong value for (2,1,2,0)")
      self.assertEqual(val[2,1,2,1],1.0,"wrong value for (2,1,2,1)")
      self.assertEqual(val[2,1,2,2],0.0,"wrong value for (2,1,2,2)")
      self.assertEqual(val[2,2,0,0],0.0,"wrong value for (2,2,0,0)")
      self.assertEqual(val[2,2,0,1],0.0,"wrong value for (2,2,0,1)")
      self.assertEqual(val[2,2,0,2],0.0,"wrong value for (2,2,0,2)")
      self.assertEqual(val[2,2,1,0],0.0,"wrong value for (2,2,1,0)")
      self.assertEqual(val[2,2,1,1],0.0,"wrong value for (2,2,1,1)")
      self.assertEqual(val[2,2,1,2],0.0,"wrong value for (2,2,1,2)")
      self.assertEqual(val[2,2,2,0],0.0,"wrong value for (2,2,2,0)")
      self.assertEqual(val[2,2,2,1],0.0,"wrong value for (2,2,2,1)")
      self.assertEqual(val[2,2,2,2],1.0,"wrong value for (2,2,2,2)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_identityTensor4_domain(self):
      val=identityTensor4(d=self.domain)
      self.assertTrue(isinstance(val,numpy.ndarray),"wrong type of result.")
      self.assertEqual(val.shape,(self.domain.getDim(),self.domain.getDim(),self.domain.getDim(),self.domain.getDim()),"wrong shape.")
      if self.domain.getDim()==2:
         self.assertEqual(val[0,0,0,0],1.0,"wrong value for (0,0,0,0)")
         self.assertEqual(val[0,0,0,1],0.0,"wrong value for (0,0,0,1)")
         self.assertEqual(val[0,0,1,0],0.0,"wrong value for (0,0,1,0)")
         self.assertEqual(val[0,0,1,1],0.0,"wrong value for (0,0,1,1)")
         self.assertEqual(val[0,1,0,0],0.0,"wrong value for (0,1,0,0)")
         self.assertEqual(val[0,1,0,1],1.0,"wrong value for (0,1,0,1)")
         self.assertEqual(val[0,1,1,0],0.0,"wrong value for (0,1,1,0)")
         self.assertEqual(val[0,1,1,1],0.0,"wrong value for (0,1,1,1)")
         self.assertEqual(val[1,0,0,0],0.0,"wrong value for (1,0,0,0)")
         self.assertEqual(val[1,0,0,1],0.0,"wrong value for (1,0,0,1)")
         self.assertEqual(val[1,0,1,0],1.0,"wrong value for (1,0,1,0)")
         self.assertEqual(val[1,0,1,1],0.0,"wrong value for (1,0,1,1)")
         self.assertEqual(val[1,1,0,0],0.0,"wrong value for (1,1,0,0)")
         self.assertEqual(val[1,1,0,1],0.0,"wrong value for (1,1,0,1)")
         self.assertEqual(val[1,1,1,0],0.0,"wrong value for (1,1,1,0)")
         self.assertEqual(val[1,1,1,1],1.0,"wrong value for (1,1,1,1)")
      if self.domain.getDim()==3:
         self.assertEqual(val[0,0,0,0],1.0,"wrong value for (0,0,0,0)")
         self.assertEqual(val[0,0,0,1],0.0,"wrong value for (0,0,0,1)")
         self.assertEqual(val[0,0,0,2],0.0,"wrong value for (0,0,0,2)")
         self.assertEqual(val[0,0,1,0],0.0,"wrong value for (0,0,1,0)")
         self.assertEqual(val[0,0,1,1],0.0,"wrong value for (0,0,1,1)")
         self.assertEqual(val[0,0,1,2],0.0,"wrong value for (0,0,1,2)")
         self.assertEqual(val[0,0,2,0],0.0,"wrong value for (0,0,2,0)")
         self.assertEqual(val[0,0,2,1],0.0,"wrong value for (0,0,2,1)")
         self.assertEqual(val[0,0,2,2],0.0,"wrong value for (0,0,2,2)")
         self.assertEqual(val[0,1,0,0],0.0,"wrong value for (0,1,0,0)")
         self.assertEqual(val[0,1,0,1],1.0,"wrong value for (0,1,0,1)")
         self.assertEqual(val[0,1,0,2],0.0,"wrong value for (0,1,0,2)")
         self.assertEqual(val[0,1,1,0],0.0,"wrong value for (0,1,1,0)")
         self.assertEqual(val[0,1,1,1],0.0,"wrong value for (0,1,1,1)")
         self.assertEqual(val[0,1,1,2],0.0,"wrong value for (0,1,1,2)")
         self.assertEqual(val[0,1,2,0],0.0,"wrong value for (0,1,2,0)")
         self.assertEqual(val[0,1,2,1],0.0,"wrong value for (0,1,2,1)")
         self.assertEqual(val[0,1,2,2],0.0,"wrong value for (0,1,2,2)")
         self.assertEqual(val[0,2,0,0],0.0,"wrong value for (0,2,0,0)")
         self.assertEqual(val[0,2,0,1],0.0,"wrong value for (0,2,0,1)")
         self.assertEqual(val[0,2,0,2],1.0,"wrong value for (0,2,0,2)")
         self.assertEqual(val[0,2,1,0],0.0,"wrong value for (0,2,1,0)")
         self.assertEqual(val[0,2,1,1],0.0,"wrong value for (0,2,1,1)")
         self.assertEqual(val[0,2,1,2],0.0,"wrong value for (0,2,1,2)")
         self.assertEqual(val[0,2,2,0],0.0,"wrong value for (0,2,2,0)")
         self.assertEqual(val[0,2,2,1],0.0,"wrong value for (0,2,2,1)")
         self.assertEqual(val[0,2,2,2],0.0,"wrong value for (0,2,2,2)")
         self.assertEqual(val[1,0,0,0],0.0,"wrong value for (1,0,0,0)")
         self.assertEqual(val[1,0,0,1],0.0,"wrong value for (1,0,0,1)")
         self.assertEqual(val[1,0,0,2],0.0,"wrong value for (1,0,0,2)")
         self.assertEqual(val[1,0,1,0],1.0,"wrong value for (1,0,1,0)")
         self.assertEqual(val[1,0,1,1],0.0,"wrong value for (1,0,1,1)")
         self.assertEqual(val[1,0,1,2],0.0,"wrong value for (1,0,1,2)")
         self.assertEqual(val[1,0,2,0],0.0,"wrong value for (1,0,2,0)")
         self.assertEqual(val[1,0,2,1],0.0,"wrong value for (1,0,2,1)")
         self.assertEqual(val[1,0,2,2],0.0,"wrong value for (1,0,2,2)")
         self.assertEqual(val[1,1,0,0],0.0,"wrong value for (1,1,0,0)")
         self.assertEqual(val[1,1,0,1],0.0,"wrong value for (1,1,0,1)")
         self.assertEqual(val[1,1,0,2],0.0,"wrong value for (1,1,0,2)")
         self.assertEqual(val[1,1,1,0],0.0,"wrong value for (1,1,1,0)")
         self.assertEqual(val[1,1,1,1],1.0,"wrong value for (1,1,1,1)")
         self.assertEqual(val[1,1,1,2],0.0,"wrong value for (1,1,1,2)")
         self.assertEqual(val[1,1,2,0],0.0,"wrong value for (1,1,2,0)")
         self.assertEqual(val[1,1,2,1],0.0,"wrong value for (1,1,2,1)")
         self.assertEqual(val[1,1,2,2],0.0,"wrong value for (1,1,2,2)")
         self.assertEqual(val[1,2,0,0],0.0,"wrong value for (1,2,0,0)")
         self.assertEqual(val[1,2,0,1],0.0,"wrong value for (1,2,0,1)")
         self.assertEqual(val[1,2,0,2],0.0,"wrong value for (1,2,0,2)")
         self.assertEqual(val[1,2,1,0],0.0,"wrong value for (1,2,1,0)")
         self.assertEqual(val[1,2,1,1],0.0,"wrong value for (1,2,1,1)")
         self.assertEqual(val[1,2,1,2],1.0,"wrong value for (1,2,1,2)")
         self.assertEqual(val[1,2,2,0],0.0,"wrong value for (1,2,2,0)")
         self.assertEqual(val[1,2,2,1],0.0,"wrong value for (1,2,2,1)")
         self.assertEqual(val[1,2,2,2],0.0,"wrong value for (1,2,2,2)")
         self.assertEqual(val[2,0,0,0],0.0,"wrong value for (2,0,0,0)")
         self.assertEqual(val[2,0,0,1],0.0,"wrong value for (2,0,0,1)")
         self.assertEqual(val[2,0,0,2],0.0,"wrong value for (2,0,0,2)")
         self.assertEqual(val[2,0,1,0],0.0,"wrong value for (2,0,1,0)")
         self.assertEqual(val[2,0,1,1],0.0,"wrong value for (2,0,1,1)")
         self.assertEqual(val[2,0,1,2],0.0,"wrong value for (2,0,1,2)")
         self.assertEqual(val[2,0,2,0],1.0,"wrong value for (2,0,2,0)")
         self.assertEqual(val[2,0,2,1],0.0,"wrong value for (2,0,2,1)")
         self.assertEqual(val[2,0,2,2],0.0,"wrong value for (2,0,2,2)")
         self.assertEqual(val[2,1,0,0],0.0,"wrong value for (2,1,0,0)")
         self.assertEqual(val[2,1,0,1],0.0,"wrong value for (2,1,0,1)")
         self.assertEqual(val[2,1,0,2],0.0,"wrong value for (2,1,0,2)")
         self.assertEqual(val[2,1,1,0],0.0,"wrong value for (2,1,1,0)")
         self.assertEqual(val[2,1,1,1],0.0,"wrong value for (2,1,1,1)")
         self.assertEqual(val[2,1,1,2],0.0,"wrong value for (2,1,1,2)")
         self.assertEqual(val[2,1,2,0],0.0,"wrong value for (2,1,2,0)")
         self.assertEqual(val[2,1,2,1],1.0,"wrong value for (2,1,2,1)")
         self.assertEqual(val[2,1,2,2],0.0,"wrong value for (2,1,2,2)")
         self.assertEqual(val[2,2,0,0],0.0,"wrong value for (2,2,0,0)")
         self.assertEqual(val[2,2,0,1],0.0,"wrong value for (2,2,0,1)")
         self.assertEqual(val[2,2,0,2],0.0,"wrong value for (2,2,0,2)")
         self.assertEqual(val[2,2,1,0],0.0,"wrong value for (2,2,1,0)")
         self.assertEqual(val[2,2,1,1],0.0,"wrong value for (2,2,1,1)")
         self.assertEqual(val[2,2,1,2],0.0,"wrong value for (2,2,1,2)")
         self.assertEqual(val[2,2,2,0],0.0,"wrong value for (2,2,2,0)")
         self.assertEqual(val[2,2,2,1],0.0,"wrong value for (2,2,2,1)")
         self.assertEqual(val[2,2,2,2],1.0,"wrong value for (2,2,2,2)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_identityTensor4_functionspace(self):
      val=identityTensor4(d=self.functionspace)
      self.assertTrue(isinstance(val,escript.Data),"wrong type of result.")
      self.assertEqual(val.getShape(),(self.functionspace.getDim(),self.functionspace.getDim(),self.functionspace.getDim(),self.functionspace.getDim()),"wrong shape.")
      if self.domain.getDim()==2:
         self.assertTrue(Lsup(val[0,0,0,0]-1.0)<=self.RES_TOL,"wrong value for (0,0,0,0)")
         self.assertTrue(Lsup(val[0,0,0,1]-0.0)<=self.RES_TOL,"wrong value for (0,0,0,1)")
         self.assertTrue(Lsup(val[0,0,1,0]-0.0)<=self.RES_TOL,"wrong value for (0,0,1,0)")
         self.assertTrue(Lsup(val[0,0,1,1]-0.0)<=self.RES_TOL,"wrong value for (0,0,1,1)")
         self.assertTrue(Lsup(val[0,1,0,0]-0.0)<=self.RES_TOL,"wrong value for (0,1,0,0)")
         self.assertTrue(Lsup(val[0,1,0,1]-1.0)<=self.RES_TOL,"wrong value for (0,1,0,1)")
         self.assertTrue(Lsup(val[0,1,1,0]-0.0)<=self.RES_TOL,"wrong value for (0,1,1,0)")
         self.assertTrue(Lsup(val[0,1,1,1]-0.0)<=self.RES_TOL,"wrong value for (0,1,1,1)")
         self.assertTrue(Lsup(val[1,0,0,0]-0.0)<=self.RES_TOL,"wrong value for (1,0,0,0)")
         self.assertTrue(Lsup(val[1,0,0,1]-0.0)<=self.RES_TOL,"wrong value for (1,0,0,1)")
         self.assertTrue(Lsup(val[1,0,1,0]-1.0)<=self.RES_TOL,"wrong value for (1,0,1,0)")
         self.assertTrue(Lsup(val[1,0,1,1]-0.0)<=self.RES_TOL,"wrong value for (1,0,1,1)")
         self.assertTrue(Lsup(val[1,1,0,0]-0.0)<=self.RES_TOL,"wrong value for (1,1,0,0)")
         self.assertTrue(Lsup(val[1,1,0,1]-0.0)<=self.RES_TOL,"wrong value for (1,1,0,1)")
         self.assertTrue(Lsup(val[1,1,1,0]-0.0)<=self.RES_TOL,"wrong value for (1,1,1,0)")
         self.assertTrue(Lsup(val[1,1,1,1]-1.0)<=self.RES_TOL,"wrong value for (1,1,1,1)")
      if self.domain.getDim()==3:
         self.assertTrue(Lsup(val[0,0,0,0]-1.0)<=self.RES_TOL,"wrong value for (0,0,0,0)")
         self.assertTrue(Lsup(val[0,0,0,1]-0.0)<=self.RES_TOL,"wrong value for (0,0,0,1)")
         self.assertTrue(Lsup(val[0,0,0,2]-0.0)<=self.RES_TOL,"wrong value for (0,0,0,2)")
         self.assertTrue(Lsup(val[0,0,1,0]-0.0)<=self.RES_TOL,"wrong value for (0,0,1,0)")
         self.assertTrue(Lsup(val[0,0,1,1]-0.0)<=self.RES_TOL,"wrong value for (0,0,1,1)")
         self.assertTrue(Lsup(val[0,0,1,2]-0.0)<=self.RES_TOL,"wrong value for (0,0,1,2)")
         self.assertTrue(Lsup(val[0,0,2,0]-0.0)<=self.RES_TOL,"wrong value for (0,0,2,0)")
         self.assertTrue(Lsup(val[0,0,2,1]-0.0)<=self.RES_TOL,"wrong value for (0,0,2,1)")
         self.assertTrue(Lsup(val[0,0,2,2]-0.0)<=self.RES_TOL,"wrong value for (0,0,2,2)")
         self.assertTrue(Lsup(val[0,1,0,0]-0.0)<=self.RES_TOL,"wrong value for (0,1,0,0)")
         self.assertTrue(Lsup(val[0,1,0,1]-1.0)<=self.RES_TOL,"wrong value for (0,1,0,1)")
         self.assertTrue(Lsup(val[0,1,0,2]-0.0)<=self.RES_TOL,"wrong value for (0,1,0,2)")
         self.assertTrue(Lsup(val[0,1,1,0]-0.0)<=self.RES_TOL,"wrong value for (0,1,1,0)")
         self.assertTrue(Lsup(val[0,1,1,1]-0.0)<=self.RES_TOL,"wrong value for (0,1,1,1)")
         self.assertTrue(Lsup(val[0,1,1,2]-0.0)<=self.RES_TOL,"wrong value for (0,1,1,2)")
         self.assertTrue(Lsup(val[0,1,2,0]-0.0)<=self.RES_TOL,"wrong value for (0,1,2,0)")
         self.assertTrue(Lsup(val[0,1,2,1]-0.0)<=self.RES_TOL,"wrong value for (0,1,2,1)")
         self.assertTrue(Lsup(val[0,1,2,2]-0.0)<=self.RES_TOL,"wrong value for (0,1,2,2)")
         self.assertTrue(Lsup(val[0,2,0,0]-0.0)<=self.RES_TOL,"wrong value for (0,2,0,0)")
         self.assertTrue(Lsup(val[0,2,0,1]-0.0)<=self.RES_TOL,"wrong value for (0,2,0,1)")
         self.assertTrue(Lsup(val[0,2,0,2]-1.0)<=self.RES_TOL,"wrong value for (0,2,0,2)")
         self.assertTrue(Lsup(val[0,2,1,0]-0.0)<=self.RES_TOL,"wrong value for (0,2,1,0)")
         self.assertTrue(Lsup(val[0,2,1,1]-0.0)<=self.RES_TOL,"wrong value for (0,2,1,1)")
         self.assertTrue(Lsup(val[0,2,1,2]-0.0)<=self.RES_TOL,"wrong value for (0,2,1,2)")
         self.assertTrue(Lsup(val[0,2,2,0]-0.0)<=self.RES_TOL,"wrong value for (0,2,2,0)")
         self.assertTrue(Lsup(val[0,2,2,1]-0.0)<=self.RES_TOL,"wrong value for (0,2,2,1)")
         self.assertTrue(Lsup(val[0,2,2,2]-0.0)<=self.RES_TOL,"wrong value for (0,2,2,2)")
         self.assertTrue(Lsup(val[1,0,0,0]-0.0)<=self.RES_TOL,"wrong value for (1,0,0,0)")
         self.assertTrue(Lsup(val[1,0,0,1]-0.0)<=self.RES_TOL,"wrong value for (1,0,0,1)")
         self.assertTrue(Lsup(val[1,0,0,2]-0.0)<=self.RES_TOL,"wrong value for (1,0,0,2)")
         self.assertTrue(Lsup(val[1,0,1,0]-1.0)<=self.RES_TOL,"wrong value for (1,0,1,0)")
         self.assertTrue(Lsup(val[1,0,1,1]-0.0)<=self.RES_TOL,"wrong value for (1,0,1,1)")
         self.assertTrue(Lsup(val[1,0,1,2]-0.0)<=self.RES_TOL,"wrong value for (1,0,1,2)")
         self.assertTrue(Lsup(val[1,0,2,0]-0.0)<=self.RES_TOL,"wrong value for (1,0,2,0)")
         self.assertTrue(Lsup(val[1,0,2,1]-0.0)<=self.RES_TOL,"wrong value for (1,0,2,1)")
         self.assertTrue(Lsup(val[1,0,2,2]-0.0)<=self.RES_TOL,"wrong value for (1,0,2,2)")
         self.assertTrue(Lsup(val[1,1,0,0]-0.0)<=self.RES_TOL,"wrong value for (1,1,0,0)")
         self.assertTrue(Lsup(val[1,1,0,1]-0.0)<=self.RES_TOL,"wrong value for (1,1,0,1)")
         self.assertTrue(Lsup(val[1,1,0,2]-0.0)<=self.RES_TOL,"wrong value for (1,1,0,2)")
         self.assertTrue(Lsup(val[1,1,1,0]-0.0)<=self.RES_TOL,"wrong value for (1,1,1,0)")
         self.assertTrue(Lsup(val[1,1,1,1]-1.0)<=self.RES_TOL,"wrong value for (1,1,1,1)")
         self.assertTrue(Lsup(val[1,1,1,2]-0.0)<=self.RES_TOL,"wrong value for (1,1,1,2)")
         self.assertTrue(Lsup(val[1,1,2,0]-0.0)<=self.RES_TOL,"wrong value for (1,1,2,0)")
         self.assertTrue(Lsup(val[1,1,2,1]-0.0)<=self.RES_TOL,"wrong value for (1,1,2,1)")
         self.assertTrue(Lsup(val[1,1,2,2]-0.0)<=self.RES_TOL,"wrong value for (1,1,2,2)")
         self.assertTrue(Lsup(val[1,2,0,0]-0.0)<=self.RES_TOL,"wrong value for (1,2,0,0)")
         self.assertTrue(Lsup(val[1,2,0,1]-0.0)<=self.RES_TOL,"wrong value for (1,2,0,1)")
         self.assertTrue(Lsup(val[1,2,0,2]-0.0)<=self.RES_TOL,"wrong value for (1,2,0,2)")
         self.assertTrue(Lsup(val[1,2,1,0]-0.0)<=self.RES_TOL,"wrong value for (1,2,1,0)")
         self.assertTrue(Lsup(val[1,2,1,1]-0.0)<=self.RES_TOL,"wrong value for (1,2,1,1)")
         self.assertTrue(Lsup(val[1,2,1,2]-1.0)<=self.RES_TOL,"wrong value for (1,2,1,2)")
         self.assertTrue(Lsup(val[1,2,2,0]-0.0)<=self.RES_TOL,"wrong value for (1,2,2,0)")
         self.assertTrue(Lsup(val[1,2,2,1]-0.0)<=self.RES_TOL,"wrong value for (1,2,2,1)")
         self.assertTrue(Lsup(val[1,2,2,2]-0.0)<=self.RES_TOL,"wrong value for (1,2,2,2)")
         self.assertTrue(Lsup(val[2,0,0,0]-0.0)<=self.RES_TOL,"wrong value for (2,0,0,0)")
         self.assertTrue(Lsup(val[2,0,0,1]-0.0)<=self.RES_TOL,"wrong value for (2,0,0,1)")
         self.assertTrue(Lsup(val[2,0,0,2]-0.0)<=self.RES_TOL,"wrong value for (2,0,0,2)")
         self.assertTrue(Lsup(val[2,0,1,0]-0.0)<=self.RES_TOL,"wrong value for (2,0,1,0)")
         self.assertTrue(Lsup(val[2,0,1,1]-0.0)<=self.RES_TOL,"wrong value for (2,0,1,1)")
         self.assertTrue(Lsup(val[2,0,1,2]-0.0)<=self.RES_TOL,"wrong value for (2,0,1,2)")
         self.assertTrue(Lsup(val[2,0,2,0]-1.0)<=self.RES_TOL,"wrong value for (2,0,2,0)")
         self.assertTrue(Lsup(val[2,0,2,1]-0.0)<=self.RES_TOL,"wrong value for (2,0,2,1)")
         self.assertTrue(Lsup(val[2,0,2,2]-0.0)<=self.RES_TOL,"wrong value for (2,0,2,2)")
         self.assertTrue(Lsup(val[2,1,0,0]-0.0)<=self.RES_TOL,"wrong value for (2,1,0,0)")
         self.assertTrue(Lsup(val[2,1,0,1]-0.0)<=self.RES_TOL,"wrong value for (2,1,0,1)")
         self.assertTrue(Lsup(val[2,1,0,2]-0.0)<=self.RES_TOL,"wrong value for (2,1,0,2)")
         self.assertTrue(Lsup(val[2,1,1,0]-0.0)<=self.RES_TOL,"wrong value for (2,1,1,0)")
         self.assertTrue(Lsup(val[2,1,1,1]-0.0)<=self.RES_TOL,"wrong value for (2,1,1,1)")
         self.assertTrue(Lsup(val[2,1,1,2]-0.0)<=self.RES_TOL,"wrong value for (2,1,1,2)")
         self.assertTrue(Lsup(val[2,1,2,0]-0.0)<=self.RES_TOL,"wrong value for (2,1,2,0)")
         self.assertTrue(Lsup(val[2,1,2,1]-1.0)<=self.RES_TOL,"wrong value for (2,1,2,1)")
         self.assertTrue(Lsup(val[2,1,2,2]-0.0)<=self.RES_TOL,"wrong value for (2,1,2,2)")
         self.assertTrue(Lsup(val[2,2,0,0]-0.0)<=self.RES_TOL,"wrong value for (2,2,0,0)")
         self.assertTrue(Lsup(val[2,2,0,1]-0.0)<=self.RES_TOL,"wrong value for (2,2,0,1)")
         self.assertTrue(Lsup(val[2,2,0,2]-0.0)<=self.RES_TOL,"wrong value for (2,2,0,2)")
         self.assertTrue(Lsup(val[2,2,1,0]-0.0)<=self.RES_TOL,"wrong value for (2,2,1,0)")
         self.assertTrue(Lsup(val[2,2,1,1]-0.0)<=self.RES_TOL,"wrong value for (2,2,1,1)")
         self.assertTrue(Lsup(val[2,2,1,2]-0.0)<=self.RES_TOL,"wrong value for (2,2,1,2)")
         self.assertTrue(Lsup(val[2,2,2,0]-0.0)<=self.RES_TOL,"wrong value for (2,2,2,0)")
         self.assertTrue(Lsup(val[2,2,2,1]-0.0)<=self.RES_TOL,"wrong value for (2,2,2,1)")
         self.assertTrue(Lsup(val[2,2,2,2]-1.0)<=self.RES_TOL,"wrong value for (2,2,2,2)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_unitVector_1(self):
      val=unitVector(i=0,d=1)
      self.assertEqual(val[0],1.0,"wrong value for 0 in the 0 vector")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_unitVector_2(self):
      val=unitVector(i=0,d=2)
      self.assertEqual(val[0],1.0,"wrong value for 0 in the 0 vector")
      self.assertEqual(val[1],0.0,"wrong value for 1 in the 0 vector")
      val=unitVector(i=1,d=2)
      self.assertEqual(val[0],0.0,"wrong value for 0 in the 1 vector")
      self.assertEqual(val[1],1.0,"wrong value for 1 in the 1 vector")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_unitVector_3(self):
      val=unitVector(i=0,d=3)
      self.assertEqual(val[0],1.0,"wrong value for 0 in the 0 vector")
      self.assertEqual(val[1],0.0,"wrong value for 1 in the 0 vector")
      self.assertEqual(val[2],0.0,"wrong value for 2 in the 0 vector")
      val=unitVector(i=1,d=3)
      self.assertEqual(val[0],0.0,"wrong value for 0 in the 1 vector")
      self.assertEqual(val[1],1.0,"wrong value for 1 in the 1 vector")
      self.assertEqual(val[2],0.0,"wrong value for 2 in the 1 vector")
      val=unitVector(i=2,d=3)
      self.assertEqual(val[0],0.0,"wrong value for 0 in the 2 vector")
      self.assertEqual(val[1],0.0,"wrong value for 1 in the 2 vector")
      self.assertEqual(val[2],1.0,"wrong value for 2 in the 2 vector")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_unitVector_domain(self):
      val=unitVector(i=0,d=self.domain)
      self.assertTrue(isinstance(val,numpy.ndarray),"wrong type of result.")
      self.assertEqual(val.shape,(self.domain.getDim(),),"wrong shape.")
      if self.domain.getDim()==2:
         self.assertEqual(val[0],1.0,"wrong value for 0 in the 0 vector")
         self.assertEqual(val[1],0.0,"wrong value for 1 in the 0 vector")
      if self.domain.getDim()==3:
         self.assertEqual(val[0],1.0,"wrong value for 0 in the 0 vector")
         self.assertEqual(val[1],0.0,"wrong value for 1 in the 0 vector")
         self.assertEqual(val[2],0.0,"wrong value for 2 in the 0 vector")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_unitVector_functionspace(self):
      val=unitVector(i=0,d=self.functionspace)
      self.assertTrue(isinstance(val,escript.Data),"wrong type of result.")
      self.assertEqual(val.getShape(),(self.functionspace.getDim(),),"wrong shape.")
      if self.domain.getDim()==2:
         self.assertTrue(Lsup(val[0]-1.0)<=self.RES_TOL,"wrong value for 0 in the 0 vector")
         self.assertTrue(Lsup(val[1]-0.0)<=self.RES_TOL,"wrong value for 1 in the 0 vector")
      if self.domain.getDim()==3:
         self.assertTrue(Lsup(val[0]-1.0)<=self.RES_TOL,"wrong value for 0 in the 0 vector")
         self.assertTrue(Lsup(val[1]-0.0)<=self.RES_TOL,"wrong value for 1 in the 0 vector")
         self.assertTrue(Lsup(val[2]-0.0)<=self.RES_TOL,"wrong value for 2 in the 0 vector")
#=========================================================
#  Symbols
#=========================================================
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_Symbol_rank0_dNone_nargs0(self):
      s2=Symbol(shape=())
      s=Symbol(shape=(),dim=None,args=[])
      self.assertEqual(s.getRank(),0,"wrong rank.")
      self.assertEqual(s.getShape(),(),"wrong shape.")
      self.assertEqual(s.getDim(),None,"wrong spatial dimension.")
      self.assertEqual(s.getArgument(),[],"wrong arguments.")

      ss=s.substitute({s:numpy.zeros(())})
      self.assertTrue(isinstance(ss,numpy.ndarray),"value after substitution is not ndarray.")
      self.assertEqual(ss.shape,(),"value after substitution has not expected shape")
      try:
        s.substitute({s:numpy.zeros((5,))})
        fail("illegal substition was successful")
      except TypeError:
        pass

      dsdarg=s.diff(Symbol(shape=()))
      self.assertTrue(isinstance(dsdarg,float),"ds/ds() has wrong type.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds() has wrong value.")
      dsdarg=s.diff(Symbol(shape=(2,)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(2,) has wrong type.")
      self.assertEqual(dsdarg.shape,(2,),"ds/ds(2,) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(2,) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(4, 5)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(4, 5) has wrong type.")
      self.assertEqual(dsdarg.shape,(4, 5),"ds/ds(4, 5) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(4, 5) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(6, 2, 2)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(6, 2, 2) has wrong type.")
      self.assertEqual(dsdarg.shape,(6, 2, 2),"ds/ds(6, 2, 2) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(6, 2, 2) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(3, 2, 3, 4)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(3, 2, 3, 4) has wrong type.")
      self.assertEqual(dsdarg.shape,(3, 2, 3, 4),"ds/ds(3, 2, 3, 4) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(3, 2, 3, 4) has wrong value.")

      dsds=s.diff(s)
      self.assertTrue(isinstance(dsds,float),"ds/ds has wrong type.")
      self.assertEqual(dsds,1.,"ds/ds has wrong value.")

      sa=s.getSubstitutedArguments({s2:-10})
      self.assertEqual(len(sa),0,"wrong number of substituted arguments")

      self.assertTrue(s.isAppropriateValue(10.),"10. is appropriate substitute")
      self.assertTrue(s.isAppropriateValue(10),"10 is appropriate substitute")
      self.assertTrue(s.isAppropriateValue(Symbol(shape=(),dim=None)),"Symbol is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(Symbol(shape=(5,))),"Symbol is not appropriate substitute (shape)")
      self.assertFalse(s.isAppropriateValue(escript.Data(0.,(5,),self.functionspace)),"Data is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(numpy.zeros(())),"ndarray is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(numpy.zeros((5,))),"ndarray is not appropriate substitute (shape)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_Symbol_rank0_dNone_nargs1(self):
      s2=Symbol(shape=())
      s=Symbol(shape=(),dim=None,args=[s2])
      self.assertEqual(s.getRank(),0,"wrong rank.")
      self.assertEqual(s.getShape(),(),"wrong shape.")
      self.assertEqual(s.getDim(),None,"wrong spatial dimension.")
      self.assertEqual(s.getArgument(),[s2],"wrong arguments.")

      ss=s.substitute({s:numpy.zeros(())})
      self.assertTrue(isinstance(ss,numpy.ndarray),"value after substitution is not ndarray.")
      self.assertEqual(ss.shape,(),"value after substitution has not expected shape")
      try:
        s.substitute({s:numpy.zeros((5,))})
        fail("illegal substition was successful")
      except TypeError:
        pass

      dsdarg=s.diff(Symbol(shape=()))
      self.assertTrue(isinstance(dsdarg,float),"ds/ds() has wrong type.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds() has wrong value.")
      dsdarg=s.diff(Symbol(shape=(2,)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(2,) has wrong type.")
      self.assertEqual(dsdarg.shape,(2,),"ds/ds(2,) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(2,) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(4, 5)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(4, 5) has wrong type.")
      self.assertEqual(dsdarg.shape,(4, 5),"ds/ds(4, 5) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(4, 5) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(6, 2, 2)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(6, 2, 2) has wrong type.")
      self.assertEqual(dsdarg.shape,(6, 2, 2),"ds/ds(6, 2, 2) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(6, 2, 2) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(3, 2, 3, 4)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(3, 2, 3, 4) has wrong type.")
      self.assertEqual(dsdarg.shape,(3, 2, 3, 4),"ds/ds(3, 2, 3, 4) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(3, 2, 3, 4) has wrong value.")

      dsds=s.diff(s)
      self.assertTrue(isinstance(dsds,float),"ds/ds has wrong type.")
      self.assertEqual(dsds,1.,"ds/ds has wrong value.")

      self.assertEqual(s.getArgument(0),s2,"wrong argument 0.")
      sa=s.getSubstitutedArguments({s2:-10})
      self.assertEqual(len(sa),1,"wrong number of substituted arguments")
      self.assertEqual(sa[0],-10,"wrongly substituted argument 0.")

      self.assertTrue(s.isAppropriateValue(10.),"10. is appropriate substitute")
      self.assertTrue(s.isAppropriateValue(10),"10 is appropriate substitute")
      self.assertTrue(s.isAppropriateValue(Symbol(shape=(),dim=None)),"Symbol is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(Symbol(shape=(5,))),"Symbol is not appropriate substitute (shape)")
      self.assertFalse(s.isAppropriateValue(escript.Data(0.,(5,),self.functionspace)),"Data is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(numpy.zeros(())),"ndarray is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(numpy.zeros((5,))),"ndarray is not appropriate substitute (shape)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_Symbol_rank0_dNone_nargs2(self):
      s2=Symbol(shape=())
      s=Symbol(shape=(),dim=None,args=[1, -1.0])
      self.assertEqual(s.getRank(),0,"wrong rank.")
      self.assertEqual(s.getShape(),(),"wrong shape.")
      self.assertEqual(s.getDim(),None,"wrong spatial dimension.")
      self.assertEqual(s.getArgument(),[1, -1.0],"wrong arguments.")

      ss=s.substitute({s:numpy.zeros(())})
      self.assertTrue(isinstance(ss,numpy.ndarray),"value after substitution is not ndarray.")
      self.assertEqual(ss.shape,(),"value after substitution has not expected shape")
      try:
        s.substitute({s:numpy.zeros((5,))})
        fail("illegal substition was successful")
      except TypeError:
        pass

      dsdarg=s.diff(Symbol(shape=()))
      self.assertTrue(isinstance(dsdarg,float),"ds/ds() has wrong type.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds() has wrong value.")
      dsdarg=s.diff(Symbol(shape=(2,)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(2,) has wrong type.")
      self.assertEqual(dsdarg.shape,(2,),"ds/ds(2,) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(2,) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(4, 5)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(4, 5) has wrong type.")
      self.assertEqual(dsdarg.shape,(4, 5),"ds/ds(4, 5) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(4, 5) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(6, 2, 2)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(6, 2, 2) has wrong type.")
      self.assertEqual(dsdarg.shape,(6, 2, 2),"ds/ds(6, 2, 2) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(6, 2, 2) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(3, 2, 3, 4)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(3, 2, 3, 4) has wrong type.")
      self.assertEqual(dsdarg.shape,(3, 2, 3, 4),"ds/ds(3, 2, 3, 4) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(3, 2, 3, 4) has wrong value.")

      dsds=s.diff(s)
      self.assertTrue(isinstance(dsds,float),"ds/ds has wrong type.")
      self.assertEqual(dsds,1.,"ds/ds has wrong value.")

      self.assertEqual(s.getArgument(0),1,"wrong argument 0.")
      self.assertEqual(s.getArgument(1),-1.0,"wrong argument 1.")
      sa=s.getSubstitutedArguments({s2:-10})
      self.assertEqual(len(sa),2,"wrong number of substituted arguments")
      self.assertEqual(sa[0],1,"wrongly substituted argument 0.")
      self.assertEqual(sa[1],-1.0,"wrongly substituted argument 1.")

      self.assertTrue(s.isAppropriateValue(10.),"10. is appropriate substitute")
      self.assertTrue(s.isAppropriateValue(10),"10 is appropriate substitute")
      self.assertTrue(s.isAppropriateValue(Symbol(shape=(),dim=None)),"Symbol is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(Symbol(shape=(5,))),"Symbol is not appropriate substitute (shape)")
      self.assertFalse(s.isAppropriateValue(escript.Data(0.,(5,),self.functionspace)),"Data is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(numpy.zeros(())),"ndarray is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(numpy.zeros((5,))),"ndarray is not appropriate substitute (shape)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_Symbol_rank0_dd_nargs0(self):
      s2=Symbol(shape=())
      s=Symbol(shape=(),dim=self.functionspace.getDim(),args=[])
      self.assertEqual(s.getRank(),0,"wrong rank.")
      self.assertEqual(s.getShape(),(),"wrong shape.")
      self.assertEqual(s.getDim(),self.functionspace.getDim(),"wrong spatial dimension.")
      self.assertEqual(s.getArgument(),[],"wrong arguments.")

      ss=s.substitute({s:numpy.zeros(())})
      self.assertTrue(isinstance(ss,numpy.ndarray),"value after substitution is not ndarray.")
      self.assertEqual(ss.shape,(),"value after substitution has not expected shape")
      try:
        s.substitute({s:numpy.zeros((5,))})
        fail("illegal substition was successful")
      except TypeError:
        pass

      dsdarg=s.diff(Symbol(shape=()))
      self.assertTrue(isinstance(dsdarg,float),"ds/ds() has wrong type.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds() has wrong value.")
      dsdarg=s.diff(Symbol(shape=(2,)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(2,) has wrong type.")
      self.assertEqual(dsdarg.shape,(2,),"ds/ds(2,) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(2,) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(4, 5)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(4, 5) has wrong type.")
      self.assertEqual(dsdarg.shape,(4, 5),"ds/ds(4, 5) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(4, 5) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(6, 2, 2)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(6, 2, 2) has wrong type.")
      self.assertEqual(dsdarg.shape,(6, 2, 2),"ds/ds(6, 2, 2) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(6, 2, 2) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(3, 2, 3, 4)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(3, 2, 3, 4) has wrong type.")
      self.assertEqual(dsdarg.shape,(3, 2, 3, 4),"ds/ds(3, 2, 3, 4) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(3, 2, 3, 4) has wrong value.")

      dsds=s.diff(s)
      self.assertTrue(isinstance(dsds,float),"ds/ds has wrong type.")
      self.assertEqual(dsds,1.,"ds/ds has wrong value.")

      sa=s.getSubstitutedArguments({s2:-10})
      self.assertEqual(len(sa),0,"wrong number of substituted arguments")

      self.assertTrue(s.isAppropriateValue(10.),"10. is appropriate substitute")
      self.assertTrue(s.isAppropriateValue(10),"10 is appropriate substitute")
      self.assertTrue(s.isAppropriateValue(Symbol(shape=(),dim=self.functionspace.getDim())),"Symbol is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(Symbol(shape=(5,))),"Symbol is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(escript.Data(0.,(),self.functionspace)),"Data is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(escript.Data(0.,(5,),self.functionspace)),"Data is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(numpy.zeros(())),"ndarray is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(numpy.zeros((5,))),"ndarray is not appropriate substitute (shape)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_Symbol_rank0_dd_nargs1(self):
      s2=Symbol(shape=())
      s=Symbol(shape=(),dim=self.functionspace.getDim(),args=[s2])
      self.assertEqual(s.getRank(),0,"wrong rank.")
      self.assertEqual(s.getShape(),(),"wrong shape.")
      self.assertEqual(s.getDim(),self.functionspace.getDim(),"wrong spatial dimension.")
      self.assertEqual(s.getArgument(),[s2],"wrong arguments.")

      ss=s.substitute({s:numpy.zeros(())})
      self.assertTrue(isinstance(ss,numpy.ndarray),"value after substitution is not ndarray.")
      self.assertEqual(ss.shape,(),"value after substitution has not expected shape")
      try:
        s.substitute({s:numpy.zeros((5,))})
        fail("illegal substition was successful")
      except TypeError:
        pass

      dsdarg=s.diff(Symbol(shape=()))
      self.assertTrue(isinstance(dsdarg,float),"ds/ds() has wrong type.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds() has wrong value.")
      dsdarg=s.diff(Symbol(shape=(2,)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(2,) has wrong type.")
      self.assertEqual(dsdarg.shape,(2,),"ds/ds(2,) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(2,) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(4, 5)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(4, 5) has wrong type.")
      self.assertEqual(dsdarg.shape,(4, 5),"ds/ds(4, 5) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(4, 5) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(6, 2, 2)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(6, 2, 2) has wrong type.")
      self.assertEqual(dsdarg.shape,(6, 2, 2),"ds/ds(6, 2, 2) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(6, 2, 2) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(3, 2, 3, 4)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(3, 2, 3, 4) has wrong type.")
      self.assertEqual(dsdarg.shape,(3, 2, 3, 4),"ds/ds(3, 2, 3, 4) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(3, 2, 3, 4) has wrong value.")

      dsds=s.diff(s)
      self.assertTrue(isinstance(dsds,float),"ds/ds has wrong type.")
      self.assertEqual(dsds,1.,"ds/ds has wrong value.")

      self.assertEqual(s.getArgument(0),s2,"wrong argument 0.")
      sa=s.getSubstitutedArguments({s2:-10})
      self.assertEqual(len(sa),1,"wrong number of substituted arguments")
      self.assertEqual(sa[0],-10,"wrongly substituted argument 0.")

      self.assertTrue(s.isAppropriateValue(10.),"10. is appropriate substitute")
      self.assertTrue(s.isAppropriateValue(10),"10 is appropriate substitute")
      self.assertTrue(s.isAppropriateValue(Symbol(shape=(),dim=self.functionspace.getDim())),"Symbol is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(Symbol(shape=(5,))),"Symbol is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(escript.Data(0.,(),self.functionspace)),"Data is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(escript.Data(0.,(5,),self.functionspace)),"Data is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(numpy.zeros(())),"ndarray is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(numpy.zeros((5,))),"ndarray is not appropriate substitute (shape)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_Symbol_rank0_dd_nargs2(self):
      s2=Symbol(shape=())
      s=Symbol(shape=(),dim=self.functionspace.getDim(),args=[1, -1.0])
      self.assertEqual(s.getRank(),0,"wrong rank.")
      self.assertEqual(s.getShape(),(),"wrong shape.")
      self.assertEqual(s.getDim(),self.functionspace.getDim(),"wrong spatial dimension.")
      self.assertEqual(s.getArgument(),[1, -1.0],"wrong arguments.")

      ss=s.substitute({s:numpy.zeros(())})
      self.assertTrue(isinstance(ss,numpy.ndarray),"value after substitution is not ndarray.")
      self.assertEqual(ss.shape,(),"value after substitution has not expected shape")
      try:
        s.substitute({s:numpy.zeros((5,))})
        fail("illegal substition was successful")
      except TypeError:
        pass

      dsdarg=s.diff(Symbol(shape=()))
      self.assertTrue(isinstance(dsdarg,float),"ds/ds() has wrong type.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds() has wrong value.")
      dsdarg=s.diff(Symbol(shape=(2,)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(2,) has wrong type.")
      self.assertEqual(dsdarg.shape,(2,),"ds/ds(2,) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(2,) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(4, 5)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(4, 5) has wrong type.")
      self.assertEqual(dsdarg.shape,(4, 5),"ds/ds(4, 5) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(4, 5) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(6, 2, 2)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(6, 2, 2) has wrong type.")
      self.assertEqual(dsdarg.shape,(6, 2, 2),"ds/ds(6, 2, 2) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(6, 2, 2) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(3, 2, 3, 4)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(3, 2, 3, 4) has wrong type.")
      self.assertEqual(dsdarg.shape,(3, 2, 3, 4),"ds/ds(3, 2, 3, 4) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(3, 2, 3, 4) has wrong value.")

      dsds=s.diff(s)
      self.assertTrue(isinstance(dsds,float),"ds/ds has wrong type.")
      self.assertEqual(dsds,1.,"ds/ds has wrong value.")

      self.assertEqual(s.getArgument(0),1,"wrong argument 0.")
      self.assertEqual(s.getArgument(1),-1.0,"wrong argument 1.")
      sa=s.getSubstitutedArguments({s2:-10})
      self.assertEqual(len(sa),2,"wrong number of substituted arguments")
      self.assertEqual(sa[0],1,"wrongly substituted argument 0.")
      self.assertEqual(sa[1],-1.0,"wrongly substituted argument 1.")

      self.assertTrue(s.isAppropriateValue(10.),"10. is appropriate substitute")
      self.assertTrue(s.isAppropriateValue(10),"10 is appropriate substitute")
      self.assertTrue(s.isAppropriateValue(Symbol(shape=(),dim=self.functionspace.getDim())),"Symbol is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(Symbol(shape=(5,))),"Symbol is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(escript.Data(0.,(),self.functionspace)),"Data is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(escript.Data(0.,(5,),self.functionspace)),"Data is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(numpy.zeros(())),"ndarray is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(numpy.zeros((5,))),"ndarray is not appropriate substitute (shape)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_Symbol_rank0_d1_nargs0(self):
      s2=Symbol(shape=())
      s=Symbol(shape=(),dim=1,args=[])
      self.assertEqual(s.getRank(),0,"wrong rank.")
      self.assertEqual(s.getShape(),(),"wrong shape.")
      self.assertEqual(s.getDim(),1,"wrong spatial dimension.")
      self.assertEqual(s.getArgument(),[],"wrong arguments.")

      ss=s.substitute({s:numpy.zeros(())})
      self.assertTrue(isinstance(ss,numpy.ndarray),"value after substitution is not ndarray.")
      self.assertEqual(ss.shape,(),"value after substitution has not expected shape")
      try:
        s.substitute({s:numpy.zeros((5,))})
        fail("illegal substition was successful")
      except TypeError:
        pass

      dsdarg=s.diff(Symbol(shape=()))
      self.assertTrue(isinstance(dsdarg,float),"ds/ds() has wrong type.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds() has wrong value.")
      dsdarg=s.diff(Symbol(shape=(2,)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(2,) has wrong type.")
      self.assertEqual(dsdarg.shape,(2,),"ds/ds(2,) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(2,) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(4, 5)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(4, 5) has wrong type.")
      self.assertEqual(dsdarg.shape,(4, 5),"ds/ds(4, 5) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(4, 5) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(6, 2, 2)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(6, 2, 2) has wrong type.")
      self.assertEqual(dsdarg.shape,(6, 2, 2),"ds/ds(6, 2, 2) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(6, 2, 2) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(3, 2, 3, 4)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(3, 2, 3, 4) has wrong type.")
      self.assertEqual(dsdarg.shape,(3, 2, 3, 4),"ds/ds(3, 2, 3, 4) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(3, 2, 3, 4) has wrong value.")

      dsds=s.diff(s)
      self.assertTrue(isinstance(dsds,float),"ds/ds has wrong type.")
      self.assertEqual(dsds,1.,"ds/ds has wrong value.")

      sa=s.getSubstitutedArguments({s2:-10})
      self.assertEqual(len(sa),0,"wrong number of substituted arguments")

      self.assertTrue(s.isAppropriateValue(10.),"10. is appropriate substitute")
      self.assertTrue(s.isAppropriateValue(10),"10 is appropriate substitute")
      self.assertTrue(s.isAppropriateValue(Symbol(shape=(),dim=1)),"Symbol is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(Symbol(shape=(),dim=2)),"Symbol is not appropriate substitute (dim)")
      self.assertFalse(s.isAppropriateValue(escript.Data(0.,(5,),self.functionspace)),"Data is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(numpy.zeros(())),"ndarray is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(numpy.zeros((5,))),"ndarray is not appropriate substitute (shape)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_Symbol_rank0_d1_nargs1(self):
      s2=Symbol(shape=())
      s=Symbol(shape=(),dim=1,args=[s2])
      self.assertEqual(s.getRank(),0,"wrong rank.")
      self.assertEqual(s.getShape(),(),"wrong shape.")
      self.assertEqual(s.getDim(),1,"wrong spatial dimension.")
      self.assertEqual(s.getArgument(),[s2],"wrong arguments.")

      ss=s.substitute({s:numpy.zeros(())})
      self.assertTrue(isinstance(ss,numpy.ndarray),"value after substitution is not ndarray.")
      self.assertEqual(ss.shape,(),"value after substitution has not expected shape")
      try:
        s.substitute({s:numpy.zeros((5,))})
        fail("illegal substition was successful")
      except TypeError:
        pass

      dsdarg=s.diff(Symbol(shape=()))
      self.assertTrue(isinstance(dsdarg,float),"ds/ds() has wrong type.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds() has wrong value.")
      dsdarg=s.diff(Symbol(shape=(2,)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(2,) has wrong type.")
      self.assertEqual(dsdarg.shape,(2,),"ds/ds(2,) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(2,) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(4, 5)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(4, 5) has wrong type.")
      self.assertEqual(dsdarg.shape,(4, 5),"ds/ds(4, 5) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(4, 5) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(6, 2, 2)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(6, 2, 2) has wrong type.")
      self.assertEqual(dsdarg.shape,(6, 2, 2),"ds/ds(6, 2, 2) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(6, 2, 2) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(3, 2, 3, 4)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(3, 2, 3, 4) has wrong type.")
      self.assertEqual(dsdarg.shape,(3, 2, 3, 4),"ds/ds(3, 2, 3, 4) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(3, 2, 3, 4) has wrong value.")

      dsds=s.diff(s)
      self.assertTrue(isinstance(dsds,float),"ds/ds has wrong type.")
      self.assertEqual(dsds,1.,"ds/ds has wrong value.")

      self.assertEqual(s.getArgument(0),s2,"wrong argument 0.")
      sa=s.getSubstitutedArguments({s2:-10})
      self.assertEqual(len(sa),1,"wrong number of substituted arguments")
      self.assertEqual(sa[0],-10,"wrongly substituted argument 0.")

      self.assertTrue(s.isAppropriateValue(10.),"10. is appropriate substitute")
      self.assertTrue(s.isAppropriateValue(10),"10 is appropriate substitute")
      self.assertTrue(s.isAppropriateValue(Symbol(shape=(),dim=1)),"Symbol is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(Symbol(shape=(),dim=2)),"Symbol is not appropriate substitute (dim)")
      self.assertFalse(s.isAppropriateValue(escript.Data(0.,(5,),self.functionspace)),"Data is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(numpy.zeros(())),"ndarray is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(numpy.zeros((5,))),"ndarray is not appropriate substitute (shape)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_Symbol_rank0_d1_nargs2(self):
      s2=Symbol(shape=())
      s=Symbol(shape=(),dim=1,args=[1, -1.0])
      self.assertEqual(s.getRank(),0,"wrong rank.")
      self.assertEqual(s.getShape(),(),"wrong shape.")
      self.assertEqual(s.getDim(),1,"wrong spatial dimension.")
      self.assertEqual(s.getArgument(),[1, -1.0],"wrong arguments.")

      ss=s.substitute({s:numpy.zeros(())})
      self.assertTrue(isinstance(ss,numpy.ndarray),"value after substitution is not ndarray.")
      self.assertEqual(ss.shape,(),"value after substitution has not expected shape")
      try:
        s.substitute({s:numpy.zeros((5,))})
        fail("illegal substition was successful")
      except TypeError:
        pass

      dsdarg=s.diff(Symbol(shape=()))
      self.assertTrue(isinstance(dsdarg,float),"ds/ds() has wrong type.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds() has wrong value.")
      dsdarg=s.diff(Symbol(shape=(2,)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(2,) has wrong type.")
      self.assertEqual(dsdarg.shape,(2,),"ds/ds(2,) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(2,) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(4, 5)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(4, 5) has wrong type.")
      self.assertEqual(dsdarg.shape,(4, 5),"ds/ds(4, 5) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(4, 5) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(6, 2, 2)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(6, 2, 2) has wrong type.")
      self.assertEqual(dsdarg.shape,(6, 2, 2),"ds/ds(6, 2, 2) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(6, 2, 2) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(3, 2, 3, 4)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(3, 2, 3, 4) has wrong type.")
      self.assertEqual(dsdarg.shape,(3, 2, 3, 4),"ds/ds(3, 2, 3, 4) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(3, 2, 3, 4) has wrong value.")

      dsds=s.diff(s)
      self.assertTrue(isinstance(dsds,float),"ds/ds has wrong type.")
      self.assertEqual(dsds,1.,"ds/ds has wrong value.")

      self.assertEqual(s.getArgument(0),1,"wrong argument 0.")
      self.assertEqual(s.getArgument(1),-1.0,"wrong argument 1.")
      sa=s.getSubstitutedArguments({s2:-10})
      self.assertEqual(len(sa),2,"wrong number of substituted arguments")
      self.assertEqual(sa[0],1,"wrongly substituted argument 0.")
      self.assertEqual(sa[1],-1.0,"wrongly substituted argument 1.")

      self.assertTrue(s.isAppropriateValue(10.),"10. is appropriate substitute")
      self.assertTrue(s.isAppropriateValue(10),"10 is appropriate substitute")
      self.assertTrue(s.isAppropriateValue(Symbol(shape=(),dim=1)),"Symbol is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(Symbol(shape=(),dim=2)),"Symbol is not appropriate substitute (dim)")
      self.assertFalse(s.isAppropriateValue(escript.Data(0.,(5,),self.functionspace)),"Data is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(numpy.zeros(())),"ndarray is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(numpy.zeros((5,))),"ndarray is not appropriate substitute (shape)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_Symbol_rank0_d2_nargs0(self):
      s2=Symbol(shape=())
      s=Symbol(shape=(),dim=2,args=[])
      self.assertEqual(s.getRank(),0,"wrong rank.")
      self.assertEqual(s.getShape(),(),"wrong shape.")
      self.assertEqual(s.getDim(),2,"wrong spatial dimension.")
      self.assertEqual(s.getArgument(),[],"wrong arguments.")

      ss=s.substitute({s:numpy.zeros(())})
      self.assertTrue(isinstance(ss,numpy.ndarray),"value after substitution is not ndarray.")
      self.assertEqual(ss.shape,(),"value after substitution has not expected shape")
      try:
        s.substitute({s:numpy.zeros((5,))})
        fail("illegal substition was successful")
      except TypeError:
        pass

      dsdarg=s.diff(Symbol(shape=()))
      self.assertTrue(isinstance(dsdarg,float),"ds/ds() has wrong type.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds() has wrong value.")
      dsdarg=s.diff(Symbol(shape=(2,)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(2,) has wrong type.")
      self.assertEqual(dsdarg.shape,(2,),"ds/ds(2,) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(2,) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(4, 5)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(4, 5) has wrong type.")
      self.assertEqual(dsdarg.shape,(4, 5),"ds/ds(4, 5) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(4, 5) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(6, 2, 2)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(6, 2, 2) has wrong type.")
      self.assertEqual(dsdarg.shape,(6, 2, 2),"ds/ds(6, 2, 2) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(6, 2, 2) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(3, 2, 3, 4)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(3, 2, 3, 4) has wrong type.")
      self.assertEqual(dsdarg.shape,(3, 2, 3, 4),"ds/ds(3, 2, 3, 4) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(3, 2, 3, 4) has wrong value.")

      dsds=s.diff(s)
      self.assertTrue(isinstance(dsds,float),"ds/ds has wrong type.")
      self.assertEqual(dsds,1.,"ds/ds has wrong value.")

      sa=s.getSubstitutedArguments({s2:-10})
      self.assertEqual(len(sa),0,"wrong number of substituted arguments")

      self.assertTrue(s.isAppropriateValue(10.),"10. is appropriate substitute")
      self.assertTrue(s.isAppropriateValue(10),"10 is appropriate substitute")
      self.assertTrue(s.isAppropriateValue(Symbol(shape=(),dim=2)),"Symbol is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(Symbol(shape=(),dim=3)),"Symbol is not appropriate substitute (dim)")
      self.assertFalse(s.isAppropriateValue(escript.Data(0.,(5,),self.functionspace)),"Data is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(numpy.zeros(())),"ndarray is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(numpy.zeros((5,))),"ndarray is not appropriate substitute (shape)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_Symbol_rank0_d2_nargs1(self):
      s2=Symbol(shape=())
      s=Symbol(shape=(),dim=2,args=[s2])
      self.assertEqual(s.getRank(),0,"wrong rank.")
      self.assertEqual(s.getShape(),(),"wrong shape.")
      self.assertEqual(s.getDim(),2,"wrong spatial dimension.")
      self.assertEqual(s.getArgument(),[s2],"wrong arguments.")

      ss=s.substitute({s:numpy.zeros(())})
      self.assertTrue(isinstance(ss,numpy.ndarray),"value after substitution is not ndarray.")
      self.assertEqual(ss.shape,(),"value after substitution has not expected shape")
      try:
        s.substitute({s:numpy.zeros((5,))})
        fail("illegal substition was successful")
      except TypeError:
        pass

      dsdarg=s.diff(Symbol(shape=()))
      self.assertTrue(isinstance(dsdarg,float),"ds/ds() has wrong type.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds() has wrong value.")
      dsdarg=s.diff(Symbol(shape=(2,)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(2,) has wrong type.")
      self.assertEqual(dsdarg.shape,(2,),"ds/ds(2,) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(2,) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(4, 5)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(4, 5) has wrong type.")
      self.assertEqual(dsdarg.shape,(4, 5),"ds/ds(4, 5) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(4, 5) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(6, 2, 2)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(6, 2, 2) has wrong type.")
      self.assertEqual(dsdarg.shape,(6, 2, 2),"ds/ds(6, 2, 2) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(6, 2, 2) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(3, 2, 3, 4)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(3, 2, 3, 4) has wrong type.")
      self.assertEqual(dsdarg.shape,(3, 2, 3, 4),"ds/ds(3, 2, 3, 4) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(3, 2, 3, 4) has wrong value.")

      dsds=s.diff(s)
      self.assertTrue(isinstance(dsds,float),"ds/ds has wrong type.")
      self.assertEqual(dsds,1.,"ds/ds has wrong value.")

      self.assertEqual(s.getArgument(0),s2,"wrong argument 0.")
      sa=s.getSubstitutedArguments({s2:-10})
      self.assertEqual(len(sa),1,"wrong number of substituted arguments")
      self.assertEqual(sa[0],-10,"wrongly substituted argument 0.")

      self.assertTrue(s.isAppropriateValue(10.),"10. is appropriate substitute")
      self.assertTrue(s.isAppropriateValue(10),"10 is appropriate substitute")
      self.assertTrue(s.isAppropriateValue(Symbol(shape=(),dim=2)),"Symbol is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(Symbol(shape=(),dim=3)),"Symbol is not appropriate substitute (dim)")
      self.assertFalse(s.isAppropriateValue(escript.Data(0.,(5,),self.functionspace)),"Data is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(numpy.zeros(())),"ndarray is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(numpy.zeros((5,))),"ndarray is not appropriate substitute (shape)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_Symbol_rank0_d2_nargs2(self):
      s2=Symbol(shape=())
      s=Symbol(shape=(),dim=2,args=[1, -1.0])
      self.assertEqual(s.getRank(),0,"wrong rank.")
      self.assertEqual(s.getShape(),(),"wrong shape.")
      self.assertEqual(s.getDim(),2,"wrong spatial dimension.")
      self.assertEqual(s.getArgument(),[1, -1.0],"wrong arguments.")

      ss=s.substitute({s:numpy.zeros(())})
      self.assertTrue(isinstance(ss,numpy.ndarray),"value after substitution is not ndarray.")
      self.assertEqual(ss.shape,(),"value after substitution has not expected shape")
      try:
        s.substitute({s:numpy.zeros((5,))})
        fail("illegal substition was successful")
      except TypeError:
        pass

      dsdarg=s.diff(Symbol(shape=()))
      self.assertTrue(isinstance(dsdarg,float),"ds/ds() has wrong type.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds() has wrong value.")
      dsdarg=s.diff(Symbol(shape=(2,)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(2,) has wrong type.")
      self.assertEqual(dsdarg.shape,(2,),"ds/ds(2,) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(2,) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(4, 5)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(4, 5) has wrong type.")
      self.assertEqual(dsdarg.shape,(4, 5),"ds/ds(4, 5) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(4, 5) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(6, 2, 2)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(6, 2, 2) has wrong type.")
      self.assertEqual(dsdarg.shape,(6, 2, 2),"ds/ds(6, 2, 2) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(6, 2, 2) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(3, 2, 3, 4)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(3, 2, 3, 4) has wrong type.")
      self.assertEqual(dsdarg.shape,(3, 2, 3, 4),"ds/ds(3, 2, 3, 4) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(3, 2, 3, 4) has wrong value.")

      dsds=s.diff(s)
      self.assertTrue(isinstance(dsds,float),"ds/ds has wrong type.")
      self.assertEqual(dsds,1.,"ds/ds has wrong value.")

      self.assertEqual(s.getArgument(0),1,"wrong argument 0.")
      self.assertEqual(s.getArgument(1),-1.0,"wrong argument 1.")
      sa=s.getSubstitutedArguments({s2:-10})
      self.assertEqual(len(sa),2,"wrong number of substituted arguments")
      self.assertEqual(sa[0],1,"wrongly substituted argument 0.")
      self.assertEqual(sa[1],-1.0,"wrongly substituted argument 1.")

      self.assertTrue(s.isAppropriateValue(10.),"10. is appropriate substitute")
      self.assertTrue(s.isAppropriateValue(10),"10 is appropriate substitute")
      self.assertTrue(s.isAppropriateValue(Symbol(shape=(),dim=2)),"Symbol is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(Symbol(shape=(),dim=3)),"Symbol is not appropriate substitute (dim)")
      self.assertFalse(s.isAppropriateValue(escript.Data(0.,(5,),self.functionspace)),"Data is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(numpy.zeros(())),"ndarray is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(numpy.zeros((5,))),"ndarray is not appropriate substitute (shape)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_Symbol_rank0_d3_nargs0(self):
      s2=Symbol(shape=())
      s=Symbol(shape=(),dim=3,args=[])
      self.assertEqual(s.getRank(),0,"wrong rank.")
      self.assertEqual(s.getShape(),(),"wrong shape.")
      self.assertEqual(s.getDim(),3,"wrong spatial dimension.")
      self.assertEqual(s.getArgument(),[],"wrong arguments.")

      ss=s.substitute({s:numpy.zeros(())})
      self.assertTrue(isinstance(ss,numpy.ndarray),"value after substitution is not ndarray.")
      self.assertEqual(ss.shape,(),"value after substitution has not expected shape")
      try:
        s.substitute({s:numpy.zeros((5,))})
        fail("illegal substition was successful")
      except TypeError:
        pass

      dsdarg=s.diff(Symbol(shape=()))
      self.assertTrue(isinstance(dsdarg,float),"ds/ds() has wrong type.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds() has wrong value.")
      dsdarg=s.diff(Symbol(shape=(2,)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(2,) has wrong type.")
      self.assertEqual(dsdarg.shape,(2,),"ds/ds(2,) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(2,) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(4, 5)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(4, 5) has wrong type.")
      self.assertEqual(dsdarg.shape,(4, 5),"ds/ds(4, 5) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(4, 5) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(6, 2, 2)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(6, 2, 2) has wrong type.")
      self.assertEqual(dsdarg.shape,(6, 2, 2),"ds/ds(6, 2, 2) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(6, 2, 2) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(3, 2, 3, 4)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(3, 2, 3, 4) has wrong type.")
      self.assertEqual(dsdarg.shape,(3, 2, 3, 4),"ds/ds(3, 2, 3, 4) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(3, 2, 3, 4) has wrong value.")

      dsds=s.diff(s)
      self.assertTrue(isinstance(dsds,float),"ds/ds has wrong type.")
      self.assertEqual(dsds,1.,"ds/ds has wrong value.")

      sa=s.getSubstitutedArguments({s2:-10})
      self.assertEqual(len(sa),0,"wrong number of substituted arguments")

      self.assertTrue(s.isAppropriateValue(10.),"10. is appropriate substitute")
      self.assertTrue(s.isAppropriateValue(10),"10 is appropriate substitute")
      self.assertTrue(s.isAppropriateValue(Symbol(shape=(),dim=3)),"Symbol is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(Symbol(shape=(),dim=4)),"Symbol is not appropriate substitute (dim)")
      self.assertFalse(s.isAppropriateValue(escript.Data(0.,(5,),self.functionspace)),"Data is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(numpy.zeros(())),"ndarray is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(numpy.zeros((5,))),"ndarray is not appropriate substitute (shape)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_Symbol_rank0_d3_nargs1(self):
      s2=Symbol(shape=())
      s=Symbol(shape=(),dim=3,args=[s2])
      self.assertEqual(s.getRank(),0,"wrong rank.")
      self.assertEqual(s.getShape(),(),"wrong shape.")
      self.assertEqual(s.getDim(),3,"wrong spatial dimension.")
      self.assertEqual(s.getArgument(),[s2],"wrong arguments.")

      ss=s.substitute({s:numpy.zeros(())})
      self.assertTrue(isinstance(ss,numpy.ndarray),"value after substitution is not ndarray.")
      self.assertEqual(ss.shape,(),"value after substitution has not expected shape")
      try:
        s.substitute({s:numpy.zeros((5,))})
        fail("illegal substition was successful")
      except TypeError:
        pass

      dsdarg=s.diff(Symbol(shape=()))
      self.assertTrue(isinstance(dsdarg,float),"ds/ds() has wrong type.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds() has wrong value.")
      dsdarg=s.diff(Symbol(shape=(2,)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(2,) has wrong type.")
      self.assertEqual(dsdarg.shape,(2,),"ds/ds(2,) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(2,) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(4, 5)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(4, 5) has wrong type.")
      self.assertEqual(dsdarg.shape,(4, 5),"ds/ds(4, 5) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(4, 5) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(6, 2, 2)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(6, 2, 2) has wrong type.")
      self.assertEqual(dsdarg.shape,(6, 2, 2),"ds/ds(6, 2, 2) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(6, 2, 2) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(3, 2, 3, 4)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(3, 2, 3, 4) has wrong type.")
      self.assertEqual(dsdarg.shape,(3, 2, 3, 4),"ds/ds(3, 2, 3, 4) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(3, 2, 3, 4) has wrong value.")

      dsds=s.diff(s)
      self.assertTrue(isinstance(dsds,float),"ds/ds has wrong type.")
      self.assertEqual(dsds,1.,"ds/ds has wrong value.")

      self.assertEqual(s.getArgument(0),s2,"wrong argument 0.")
      sa=s.getSubstitutedArguments({s2:-10})
      self.assertEqual(len(sa),1,"wrong number of substituted arguments")
      self.assertEqual(sa[0],-10,"wrongly substituted argument 0.")

      self.assertTrue(s.isAppropriateValue(10.),"10. is appropriate substitute")
      self.assertTrue(s.isAppropriateValue(10),"10 is appropriate substitute")
      self.assertTrue(s.isAppropriateValue(Symbol(shape=(),dim=3)),"Symbol is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(Symbol(shape=(),dim=4)),"Symbol is not appropriate substitute (dim)")
      self.assertFalse(s.isAppropriateValue(escript.Data(0.,(5,),self.functionspace)),"Data is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(numpy.zeros(())),"ndarray is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(numpy.zeros((5,))),"ndarray is not appropriate substitute (shape)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_Symbol_rank0_d3_nargs2(self):
      s2=Symbol(shape=())
      s=Symbol(shape=(),dim=3,args=[1, -1.0])
      self.assertEqual(s.getRank(),0,"wrong rank.")
      self.assertEqual(s.getShape(),(),"wrong shape.")
      self.assertEqual(s.getDim(),3,"wrong spatial dimension.")
      self.assertEqual(s.getArgument(),[1, -1.0],"wrong arguments.")

      ss=s.substitute({s:numpy.zeros(())})
      self.assertTrue(isinstance(ss,numpy.ndarray),"value after substitution is not ndarray.")
      self.assertEqual(ss.shape,(),"value after substitution has not expected shape")
      try:
        s.substitute({s:numpy.zeros((5,))})
        fail("illegal substition was successful")
      except TypeError:
        pass

      dsdarg=s.diff(Symbol(shape=()))
      self.assertTrue(isinstance(dsdarg,float),"ds/ds() has wrong type.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds() has wrong value.")
      dsdarg=s.diff(Symbol(shape=(2,)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(2,) has wrong type.")
      self.assertEqual(dsdarg.shape,(2,),"ds/ds(2,) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(2,) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(4, 5)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(4, 5) has wrong type.")
      self.assertEqual(dsdarg.shape,(4, 5),"ds/ds(4, 5) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(4, 5) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(6, 2, 2)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(6, 2, 2) has wrong type.")
      self.assertEqual(dsdarg.shape,(6, 2, 2),"ds/ds(6, 2, 2) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(6, 2, 2) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(3, 2, 3, 4)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(3, 2, 3, 4) has wrong type.")
      self.assertEqual(dsdarg.shape,(3, 2, 3, 4),"ds/ds(3, 2, 3, 4) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(3, 2, 3, 4) has wrong value.")

      dsds=s.diff(s)
      self.assertTrue(isinstance(dsds,float),"ds/ds has wrong type.")
      self.assertEqual(dsds,1.,"ds/ds has wrong value.")

      self.assertEqual(s.getArgument(0),1,"wrong argument 0.")
      self.assertEqual(s.getArgument(1),-1.0,"wrong argument 1.")
      sa=s.getSubstitutedArguments({s2:-10})
      self.assertEqual(len(sa),2,"wrong number of substituted arguments")
      self.assertEqual(sa[0],1,"wrongly substituted argument 0.")
      self.assertEqual(sa[1],-1.0,"wrongly substituted argument 1.")

      self.assertTrue(s.isAppropriateValue(10.),"10. is appropriate substitute")
      self.assertTrue(s.isAppropriateValue(10),"10 is appropriate substitute")
      self.assertTrue(s.isAppropriateValue(Symbol(shape=(),dim=3)),"Symbol is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(Symbol(shape=(),dim=4)),"Symbol is not appropriate substitute (dim)")
      self.assertFalse(s.isAppropriateValue(escript.Data(0.,(5,),self.functionspace)),"Data is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(numpy.zeros(())),"ndarray is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(numpy.zeros((5,))),"ndarray is not appropriate substitute (shape)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_Symbol_rank1_dNone_nargs0(self):
      s2=Symbol(shape=())
      s=Symbol(shape=(2,),dim=None,args=[])
      self.assertEqual(s.getRank(),1,"wrong rank.")
      self.assertEqual(s.getShape(),(2,),"wrong shape.")
      self.assertEqual(s.getDim(),None,"wrong spatial dimension.")
      self.assertEqual(s.getArgument(),[],"wrong arguments.")

      ss=s.substitute({s:numpy.zeros((2,))})
      self.assertTrue(isinstance(ss,numpy.ndarray),"value after substitution is not ndarray.")
      self.assertEqual(ss.shape,(2,),"value after substitution has not expected shape")
      try:
        s.substitute({s:numpy.zeros((5,))})
        fail("illegal substition was successful")
      except TypeError:
        pass

      dsdarg=s.diff(Symbol(shape=()))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds() has wrong type.")
      self.assertEqual(dsdarg.shape,(2,),"ds/ds() has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds() has wrong value.")
      dsdarg=s.diff(Symbol(shape=(2,)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(2,) has wrong type.")
      self.assertEqual(dsdarg.shape,(2, 2),"ds/ds(2,) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(2,) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(4, 5)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(4, 5) has wrong type.")
      self.assertEqual(dsdarg.shape,(2, 4, 5),"ds/ds(4, 5) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(4, 5) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(6, 2, 2)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(6, 2, 2) has wrong type.")
      self.assertEqual(dsdarg.shape,(2, 6, 2, 2),"ds/ds(6, 2, 2) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(6, 2, 2) has wrong value.")

      dsds=s.diff(s)
      self.assertTrue(isinstance(dsds,numpy.ndarray),"ds/ds has wrong type.")
      self.assertEqual(dsds.shape,(2, 2),"ds/ds has wrong shape.")
      self.assertEqual(dsds[0,0],1.0,"ds/ds has wrong value at (0,0).")
      self.assertEqual(dsds[0,1],0.0,"ds/ds has wrong value at (0,1).")
      self.assertEqual(dsds[1,0],0.0,"ds/ds has wrong value at (1,0).")
      self.assertEqual(dsds[1,1],1.0,"ds/ds has wrong value at (1,1).")

      sa=s.getSubstitutedArguments({s2:-10})
      self.assertEqual(len(sa),0,"wrong number of substituted arguments")

      self.assertFalse(s.isAppropriateValue(10.)," 10. is not appropriate substitute")
      self.assertFalse(s.isAppropriateValue(10)," 10 is not appropriate substitute")
      self.assertTrue(s.isAppropriateValue(Symbol(shape=(2,),dim=None)),"Symbol is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(Symbol(shape=(5,))),"Symbol is not appropriate substitute (shape)")
      self.assertFalse(s.isAppropriateValue(escript.Data(0.,(5,),self.functionspace)),"Data is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(numpy.zeros((2,))),"ndarray is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(numpy.zeros((5,))),"ndarray is not appropriate substitute (shape)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_Symbol_rank1_dNone_nargs1(self):
      s2=Symbol(shape=())
      s=Symbol(shape=(2,),dim=None,args=[s2])
      self.assertEqual(s.getRank(),1,"wrong rank.")
      self.assertEqual(s.getShape(),(2,),"wrong shape.")
      self.assertEqual(s.getDim(),None,"wrong spatial dimension.")
      self.assertEqual(s.getArgument(),[s2],"wrong arguments.")

      ss=s.substitute({s:numpy.zeros((2,))})
      self.assertTrue(isinstance(ss,numpy.ndarray),"value after substitution is not ndarray.")
      self.assertEqual(ss.shape,(2,),"value after substitution has not expected shape")
      try:
        s.substitute({s:numpy.zeros((5,))})
        fail("illegal substition was successful")
      except TypeError:
        pass

      dsdarg=s.diff(Symbol(shape=()))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds() has wrong type.")
      self.assertEqual(dsdarg.shape,(2,),"ds/ds() has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds() has wrong value.")
      dsdarg=s.diff(Symbol(shape=(2,)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(2,) has wrong type.")
      self.assertEqual(dsdarg.shape,(2, 2),"ds/ds(2,) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(2,) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(4, 5)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(4, 5) has wrong type.")
      self.assertEqual(dsdarg.shape,(2, 4, 5),"ds/ds(4, 5) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(4, 5) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(6, 2, 2)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(6, 2, 2) has wrong type.")
      self.assertEqual(dsdarg.shape,(2, 6, 2, 2),"ds/ds(6, 2, 2) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(6, 2, 2) has wrong value.")

      dsds=s.diff(s)
      self.assertTrue(isinstance(dsds,numpy.ndarray),"ds/ds has wrong type.")
      self.assertEqual(dsds.shape,(2, 2),"ds/ds has wrong shape.")
      self.assertEqual(dsds[0,0],1.0,"ds/ds has wrong value at (0,0).")
      self.assertEqual(dsds[0,1],0.0,"ds/ds has wrong value at (0,1).")
      self.assertEqual(dsds[1,0],0.0,"ds/ds has wrong value at (1,0).")
      self.assertEqual(dsds[1,1],1.0,"ds/ds has wrong value at (1,1).")

      self.assertEqual(s.getArgument(0),s2,"wrong argument 0.")
      sa=s.getSubstitutedArguments({s2:-10})
      self.assertEqual(len(sa),1,"wrong number of substituted arguments")
      self.assertEqual(sa[0],-10,"wrongly substituted argument 0.")

      self.assertFalse(s.isAppropriateValue(10.)," 10. is not appropriate substitute")
      self.assertFalse(s.isAppropriateValue(10)," 10 is not appropriate substitute")
      self.assertTrue(s.isAppropriateValue(Symbol(shape=(2,),dim=None)),"Symbol is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(Symbol(shape=(5,))),"Symbol is not appropriate substitute (shape)")
      self.assertFalse(s.isAppropriateValue(escript.Data(0.,(5,),self.functionspace)),"Data is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(numpy.zeros((2,))),"ndarray is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(numpy.zeros((5,))),"ndarray is not appropriate substitute (shape)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_Symbol_rank1_dNone_nargs2(self):
      s2=Symbol(shape=())
      s=Symbol(shape=(2,),dim=None,args=[1, -1.0])
      self.assertEqual(s.getRank(),1,"wrong rank.")
      self.assertEqual(s.getShape(),(2,),"wrong shape.")
      self.assertEqual(s.getDim(),None,"wrong spatial dimension.")
      self.assertEqual(s.getArgument(),[1, -1.0],"wrong arguments.")

      ss=s.substitute({s:numpy.zeros((2,))})
      self.assertTrue(isinstance(ss,numpy.ndarray),"value after substitution is not ndarray.")
      self.assertEqual(ss.shape,(2,),"value after substitution has not expected shape")
      try:
        s.substitute({s:numpy.zeros((5,))})
        fail("illegal substition was successful")
      except TypeError:
        pass

      dsdarg=s.diff(Symbol(shape=()))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds() has wrong type.")
      self.assertEqual(dsdarg.shape,(2,),"ds/ds() has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds() has wrong value.")
      dsdarg=s.diff(Symbol(shape=(2,)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(2,) has wrong type.")
      self.assertEqual(dsdarg.shape,(2, 2),"ds/ds(2,) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(2,) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(4, 5)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(4, 5) has wrong type.")
      self.assertEqual(dsdarg.shape,(2, 4, 5),"ds/ds(4, 5) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(4, 5) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(6, 2, 2)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(6, 2, 2) has wrong type.")
      self.assertEqual(dsdarg.shape,(2, 6, 2, 2),"ds/ds(6, 2, 2) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(6, 2, 2) has wrong value.")

      dsds=s.diff(s)
      self.assertTrue(isinstance(dsds,numpy.ndarray),"ds/ds has wrong type.")
      self.assertEqual(dsds.shape,(2, 2),"ds/ds has wrong shape.")
      self.assertEqual(dsds[0,0],1.0,"ds/ds has wrong value at (0,0).")
      self.assertEqual(dsds[0,1],0.0,"ds/ds has wrong value at (0,1).")
      self.assertEqual(dsds[1,0],0.0,"ds/ds has wrong value at (1,0).")
      self.assertEqual(dsds[1,1],1.0,"ds/ds has wrong value at (1,1).")

      self.assertEqual(s.getArgument(0),1,"wrong argument 0.")
      self.assertEqual(s.getArgument(1),-1.0,"wrong argument 1.")
      sa=s.getSubstitutedArguments({s2:-10})
      self.assertEqual(len(sa),2,"wrong number of substituted arguments")
      self.assertEqual(sa[0],1,"wrongly substituted argument 0.")
      self.assertEqual(sa[1],-1.0,"wrongly substituted argument 1.")

      self.assertFalse(s.isAppropriateValue(10.)," 10. is not appropriate substitute")
      self.assertFalse(s.isAppropriateValue(10)," 10 is not appropriate substitute")
      self.assertTrue(s.isAppropriateValue(Symbol(shape=(2,),dim=None)),"Symbol is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(Symbol(shape=(5,))),"Symbol is not appropriate substitute (shape)")
      self.assertFalse(s.isAppropriateValue(escript.Data(0.,(5,),self.functionspace)),"Data is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(numpy.zeros((2,))),"ndarray is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(numpy.zeros((5,))),"ndarray is not appropriate substitute (shape)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_Symbol_rank1_dd_nargs0(self):
      s2=Symbol(shape=())
      s=Symbol(shape=(2,),dim=self.functionspace.getDim(),args=[])
      self.assertEqual(s.getRank(),1,"wrong rank.")
      self.assertEqual(s.getShape(),(2,),"wrong shape.")
      self.assertEqual(s.getDim(),self.functionspace.getDim(),"wrong spatial dimension.")
      self.assertEqual(s.getArgument(),[],"wrong arguments.")

      ss=s.substitute({s:numpy.zeros((2,))})
      self.assertTrue(isinstance(ss,numpy.ndarray),"value after substitution is not ndarray.")
      self.assertEqual(ss.shape,(2,),"value after substitution has not expected shape")
      try:
        s.substitute({s:numpy.zeros((5,))})
        fail("illegal substition was successful")
      except TypeError:
        pass

      dsdarg=s.diff(Symbol(shape=()))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds() has wrong type.")
      self.assertEqual(dsdarg.shape,(2,),"ds/ds() has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds() has wrong value.")
      dsdarg=s.diff(Symbol(shape=(2,)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(2,) has wrong type.")
      self.assertEqual(dsdarg.shape,(2, 2),"ds/ds(2,) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(2,) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(4, 5)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(4, 5) has wrong type.")
      self.assertEqual(dsdarg.shape,(2, 4, 5),"ds/ds(4, 5) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(4, 5) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(6, 2, 2)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(6, 2, 2) has wrong type.")
      self.assertEqual(dsdarg.shape,(2, 6, 2, 2),"ds/ds(6, 2, 2) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(6, 2, 2) has wrong value.")

      dsds=s.diff(s)
      self.assertTrue(isinstance(dsds,numpy.ndarray),"ds/ds has wrong type.")
      self.assertEqual(dsds.shape,(2, 2),"ds/ds has wrong shape.")
      self.assertEqual(dsds[0,0],1.0,"ds/ds has wrong value at (0,0).")
      self.assertEqual(dsds[0,1],0.0,"ds/ds has wrong value at (0,1).")
      self.assertEqual(dsds[1,0],0.0,"ds/ds has wrong value at (1,0).")
      self.assertEqual(dsds[1,1],1.0,"ds/ds has wrong value at (1,1).")

      sa=s.getSubstitutedArguments({s2:-10})
      self.assertEqual(len(sa),0,"wrong number of substituted arguments")

      self.assertFalse(s.isAppropriateValue(10.)," 10. is not appropriate substitute")
      self.assertFalse(s.isAppropriateValue(10)," 10 is not appropriate substitute")
      self.assertTrue(s.isAppropriateValue(Symbol(shape=(2,),dim=self.functionspace.getDim())),"Symbol is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(Symbol(shape=(5,))),"Symbol is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(escript.Data(0.,(2,),self.functionspace)),"Data is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(escript.Data(0.,(5,),self.functionspace)),"Data is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(numpy.zeros((2,))),"ndarray is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(numpy.zeros((5,))),"ndarray is not appropriate substitute (shape)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_Symbol_rank1_dd_nargs1(self):
      s2=Symbol(shape=())
      s=Symbol(shape=(2,),dim=self.functionspace.getDim(),args=[s2])
      self.assertEqual(s.getRank(),1,"wrong rank.")
      self.assertEqual(s.getShape(),(2,),"wrong shape.")
      self.assertEqual(s.getDim(),self.functionspace.getDim(),"wrong spatial dimension.")
      self.assertEqual(s.getArgument(),[s2],"wrong arguments.")

      ss=s.substitute({s:numpy.zeros((2,))})
      self.assertTrue(isinstance(ss,numpy.ndarray),"value after substitution is not ndarray.")
      self.assertEqual(ss.shape,(2,),"value after substitution has not expected shape")
      try:
        s.substitute({s:numpy.zeros((5,))})
        fail("illegal substition was successful")
      except TypeError:
        pass

      dsdarg=s.diff(Symbol(shape=()))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds() has wrong type.")
      self.assertEqual(dsdarg.shape,(2,),"ds/ds() has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds() has wrong value.")
      dsdarg=s.diff(Symbol(shape=(2,)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(2,) has wrong type.")
      self.assertEqual(dsdarg.shape,(2, 2),"ds/ds(2,) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(2,) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(4, 5)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(4, 5) has wrong type.")
      self.assertEqual(dsdarg.shape,(2, 4, 5),"ds/ds(4, 5) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(4, 5) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(6, 2, 2)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(6, 2, 2) has wrong type.")
      self.assertEqual(dsdarg.shape,(2, 6, 2, 2),"ds/ds(6, 2, 2) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(6, 2, 2) has wrong value.")

      dsds=s.diff(s)
      self.assertTrue(isinstance(dsds,numpy.ndarray),"ds/ds has wrong type.")
      self.assertEqual(dsds.shape,(2, 2),"ds/ds has wrong shape.")
      self.assertEqual(dsds[0,0],1.0,"ds/ds has wrong value at (0,0).")
      self.assertEqual(dsds[0,1],0.0,"ds/ds has wrong value at (0,1).")
      self.assertEqual(dsds[1,0],0.0,"ds/ds has wrong value at (1,0).")
      self.assertEqual(dsds[1,1],1.0,"ds/ds has wrong value at (1,1).")

      self.assertEqual(s.getArgument(0),s2,"wrong argument 0.")
      sa=s.getSubstitutedArguments({s2:-10})
      self.assertEqual(len(sa),1,"wrong number of substituted arguments")
      self.assertEqual(sa[0],-10,"wrongly substituted argument 0.")

      self.assertFalse(s.isAppropriateValue(10.)," 10. is not appropriate substitute")
      self.assertFalse(s.isAppropriateValue(10)," 10 is not appropriate substitute")
      self.assertTrue(s.isAppropriateValue(Symbol(shape=(2,),dim=self.functionspace.getDim())),"Symbol is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(Symbol(shape=(5,))),"Symbol is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(escript.Data(0.,(2,),self.functionspace)),"Data is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(escript.Data(0.,(5,),self.functionspace)),"Data is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(numpy.zeros((2,))),"ndarray is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(numpy.zeros((5,))),"ndarray is not appropriate substitute (shape)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_Symbol_rank1_dd_nargs2(self):
      s2=Symbol(shape=())
      s=Symbol(shape=(2,),dim=self.functionspace.getDim(),args=[1, -1.0])
      self.assertEqual(s.getRank(),1,"wrong rank.")
      self.assertEqual(s.getShape(),(2,),"wrong shape.")
      self.assertEqual(s.getDim(),self.functionspace.getDim(),"wrong spatial dimension.")
      self.assertEqual(s.getArgument(),[1, -1.0],"wrong arguments.")

      ss=s.substitute({s:numpy.zeros((2,))})
      self.assertTrue(isinstance(ss,numpy.ndarray),"value after substitution is not ndarray.")
      self.assertEqual(ss.shape,(2,),"value after substitution has not expected shape")
      try:
        s.substitute({s:numpy.zeros((5,))})
        fail("illegal substition was successful")
      except TypeError:
        pass

      dsdarg=s.diff(Symbol(shape=()))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds() has wrong type.")
      self.assertEqual(dsdarg.shape,(2,),"ds/ds() has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds() has wrong value.")
      dsdarg=s.diff(Symbol(shape=(2,)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(2,) has wrong type.")
      self.assertEqual(dsdarg.shape,(2, 2),"ds/ds(2,) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(2,) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(4, 5)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(4, 5) has wrong type.")
      self.assertEqual(dsdarg.shape,(2, 4, 5),"ds/ds(4, 5) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(4, 5) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(6, 2, 2)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(6, 2, 2) has wrong type.")
      self.assertEqual(dsdarg.shape,(2, 6, 2, 2),"ds/ds(6, 2, 2) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(6, 2, 2) has wrong value.")

      dsds=s.diff(s)
      self.assertTrue(isinstance(dsds,numpy.ndarray),"ds/ds has wrong type.")
      self.assertEqual(dsds.shape,(2, 2),"ds/ds has wrong shape.")
      self.assertEqual(dsds[0,0],1.0,"ds/ds has wrong value at (0,0).")
      self.assertEqual(dsds[0,1],0.0,"ds/ds has wrong value at (0,1).")
      self.assertEqual(dsds[1,0],0.0,"ds/ds has wrong value at (1,0).")
      self.assertEqual(dsds[1,1],1.0,"ds/ds has wrong value at (1,1).")

      self.assertEqual(s.getArgument(0),1,"wrong argument 0.")
      self.assertEqual(s.getArgument(1),-1.0,"wrong argument 1.")
      sa=s.getSubstitutedArguments({s2:-10})
      self.assertEqual(len(sa),2,"wrong number of substituted arguments")
      self.assertEqual(sa[0],1,"wrongly substituted argument 0.")
      self.assertEqual(sa[1],-1.0,"wrongly substituted argument 1.")

      self.assertFalse(s.isAppropriateValue(10.)," 10. is not appropriate substitute")
      self.assertFalse(s.isAppropriateValue(10)," 10 is not appropriate substitute")
      self.assertTrue(s.isAppropriateValue(Symbol(shape=(2,),dim=self.functionspace.getDim())),"Symbol is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(Symbol(shape=(5,))),"Symbol is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(escript.Data(0.,(2,),self.functionspace)),"Data is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(escript.Data(0.,(5,),self.functionspace)),"Data is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(numpy.zeros((2,))),"ndarray is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(numpy.zeros((5,))),"ndarray is not appropriate substitute (shape)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_Symbol_rank1_d1_nargs0(self):
      s2=Symbol(shape=())
      s=Symbol(shape=(2,),dim=1,args=[])
      self.assertEqual(s.getRank(),1,"wrong rank.")
      self.assertEqual(s.getShape(),(2,),"wrong shape.")
      self.assertEqual(s.getDim(),1,"wrong spatial dimension.")
      self.assertEqual(s.getArgument(),[],"wrong arguments.")

      ss=s.substitute({s:numpy.zeros((2,))})
      self.assertTrue(isinstance(ss,numpy.ndarray),"value after substitution is not ndarray.")
      self.assertEqual(ss.shape,(2,),"value after substitution has not expected shape")
      try:
        s.substitute({s:numpy.zeros((5,))})
        fail("illegal substition was successful")
      except TypeError:
        pass

      dsdarg=s.diff(Symbol(shape=()))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds() has wrong type.")
      self.assertEqual(dsdarg.shape,(2,),"ds/ds() has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds() has wrong value.")
      dsdarg=s.diff(Symbol(shape=(2,)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(2,) has wrong type.")
      self.assertEqual(dsdarg.shape,(2, 2),"ds/ds(2,) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(2,) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(4, 5)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(4, 5) has wrong type.")
      self.assertEqual(dsdarg.shape,(2, 4, 5),"ds/ds(4, 5) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(4, 5) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(6, 2, 2)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(6, 2, 2) has wrong type.")
      self.assertEqual(dsdarg.shape,(2, 6, 2, 2),"ds/ds(6, 2, 2) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(6, 2, 2) has wrong value.")

      dsds=s.diff(s)
      self.assertTrue(isinstance(dsds,numpy.ndarray),"ds/ds has wrong type.")
      self.assertEqual(dsds.shape,(2, 2),"ds/ds has wrong shape.")
      self.assertEqual(dsds[0,0],1.0,"ds/ds has wrong value at (0,0).")
      self.assertEqual(dsds[0,1],0.0,"ds/ds has wrong value at (0,1).")
      self.assertEqual(dsds[1,0],0.0,"ds/ds has wrong value at (1,0).")
      self.assertEqual(dsds[1,1],1.0,"ds/ds has wrong value at (1,1).")

      sa=s.getSubstitutedArguments({s2:-10})
      self.assertEqual(len(sa),0,"wrong number of substituted arguments")

      self.assertFalse(s.isAppropriateValue(10.)," 10. is not appropriate substitute")
      self.assertFalse(s.isAppropriateValue(10)," 10 is not appropriate substitute")
      self.assertTrue(s.isAppropriateValue(Symbol(shape=(2,),dim=1)),"Symbol is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(Symbol(shape=(2,),dim=2)),"Symbol is not appropriate substitute (dim)")
      self.assertFalse(s.isAppropriateValue(escript.Data(0.,(5,),self.functionspace)),"Data is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(numpy.zeros((2,))),"ndarray is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(numpy.zeros((5,))),"ndarray is not appropriate substitute (shape)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_Symbol_rank1_d1_nargs1(self):
      s2=Symbol(shape=())
      s=Symbol(shape=(2,),dim=1,args=[s2])
      self.assertEqual(s.getRank(),1,"wrong rank.")
      self.assertEqual(s.getShape(),(2,),"wrong shape.")
      self.assertEqual(s.getDim(),1,"wrong spatial dimension.")
      self.assertEqual(s.getArgument(),[s2],"wrong arguments.")

      ss=s.substitute({s:numpy.zeros((2,))})
      self.assertTrue(isinstance(ss,numpy.ndarray),"value after substitution is not ndarray.")
      self.assertEqual(ss.shape,(2,),"value after substitution has not expected shape")
      try:
        s.substitute({s:numpy.zeros((5,))})
        fail("illegal substition was successful")
      except TypeError:
        pass

      dsdarg=s.diff(Symbol(shape=()))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds() has wrong type.")
      self.assertEqual(dsdarg.shape,(2,),"ds/ds() has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds() has wrong value.")
      dsdarg=s.diff(Symbol(shape=(2,)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(2,) has wrong type.")
      self.assertEqual(dsdarg.shape,(2, 2),"ds/ds(2,) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(2,) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(4, 5)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(4, 5) has wrong type.")
      self.assertEqual(dsdarg.shape,(2, 4, 5),"ds/ds(4, 5) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(4, 5) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(6, 2, 2)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(6, 2, 2) has wrong type.")
      self.assertEqual(dsdarg.shape,(2, 6, 2, 2),"ds/ds(6, 2, 2) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(6, 2, 2) has wrong value.")

      dsds=s.diff(s)
      self.assertTrue(isinstance(dsds,numpy.ndarray),"ds/ds has wrong type.")
      self.assertEqual(dsds.shape,(2, 2),"ds/ds has wrong shape.")
      self.assertEqual(dsds[0,0],1.0,"ds/ds has wrong value at (0,0).")
      self.assertEqual(dsds[0,1],0.0,"ds/ds has wrong value at (0,1).")
      self.assertEqual(dsds[1,0],0.0,"ds/ds has wrong value at (1,0).")
      self.assertEqual(dsds[1,1],1.0,"ds/ds has wrong value at (1,1).")

      self.assertEqual(s.getArgument(0),s2,"wrong argument 0.")
      sa=s.getSubstitutedArguments({s2:-10})
      self.assertEqual(len(sa),1,"wrong number of substituted arguments")
      self.assertEqual(sa[0],-10,"wrongly substituted argument 0.")

      self.assertFalse(s.isAppropriateValue(10.)," 10. is not appropriate substitute")
      self.assertFalse(s.isAppropriateValue(10)," 10 is not appropriate substitute")
      self.assertTrue(s.isAppropriateValue(Symbol(shape=(2,),dim=1)),"Symbol is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(Symbol(shape=(2,),dim=2)),"Symbol is not appropriate substitute (dim)")
      self.assertFalse(s.isAppropriateValue(escript.Data(0.,(5,),self.functionspace)),"Data is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(numpy.zeros((2,))),"ndarray is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(numpy.zeros((5,))),"ndarray is not appropriate substitute (shape)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_Symbol_rank1_d1_nargs2(self):
      s2=Symbol(shape=())
      s=Symbol(shape=(2,),dim=1,args=[1, -1.0])
      self.assertEqual(s.getRank(),1,"wrong rank.")
      self.assertEqual(s.getShape(),(2,),"wrong shape.")
      self.assertEqual(s.getDim(),1,"wrong spatial dimension.")
      self.assertEqual(s.getArgument(),[1, -1.0],"wrong arguments.")

      ss=s.substitute({s:numpy.zeros((2,))})
      self.assertTrue(isinstance(ss,numpy.ndarray),"value after substitution is not ndarray.")
      self.assertEqual(ss.shape,(2,),"value after substitution has not expected shape")
      try:
        s.substitute({s:numpy.zeros((5,))})
        fail("illegal substition was successful")
      except TypeError:
        pass

      dsdarg=s.diff(Symbol(shape=()))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds() has wrong type.")
      self.assertEqual(dsdarg.shape,(2,),"ds/ds() has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds() has wrong value.")
      dsdarg=s.diff(Symbol(shape=(2,)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(2,) has wrong type.")
      self.assertEqual(dsdarg.shape,(2, 2),"ds/ds(2,) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(2,) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(4, 5)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(4, 5) has wrong type.")
      self.assertEqual(dsdarg.shape,(2, 4, 5),"ds/ds(4, 5) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(4, 5) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(6, 2, 2)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(6, 2, 2) has wrong type.")
      self.assertEqual(dsdarg.shape,(2, 6, 2, 2),"ds/ds(6, 2, 2) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(6, 2, 2) has wrong value.")

      dsds=s.diff(s)
      self.assertTrue(isinstance(dsds,numpy.ndarray),"ds/ds has wrong type.")
      self.assertEqual(dsds.shape,(2, 2),"ds/ds has wrong shape.")
      self.assertEqual(dsds[0,0],1.0,"ds/ds has wrong value at (0,0).")
      self.assertEqual(dsds[0,1],0.0,"ds/ds has wrong value at (0,1).")
      self.assertEqual(dsds[1,0],0.0,"ds/ds has wrong value at (1,0).")
      self.assertEqual(dsds[1,1],1.0,"ds/ds has wrong value at (1,1).")

      self.assertEqual(s.getArgument(0),1,"wrong argument 0.")
      self.assertEqual(s.getArgument(1),-1.0,"wrong argument 1.")
      sa=s.getSubstitutedArguments({s2:-10})
      self.assertEqual(len(sa),2,"wrong number of substituted arguments")
      self.assertEqual(sa[0],1,"wrongly substituted argument 0.")
      self.assertEqual(sa[1],-1.0,"wrongly substituted argument 1.")

      self.assertFalse(s.isAppropriateValue(10.)," 10. is not appropriate substitute")
      self.assertFalse(s.isAppropriateValue(10)," 10 is not appropriate substitute")
      self.assertTrue(s.isAppropriateValue(Symbol(shape=(2,),dim=1)),"Symbol is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(Symbol(shape=(2,),dim=2)),"Symbol is not appropriate substitute (dim)")
      self.assertFalse(s.isAppropriateValue(escript.Data(0.,(5,),self.functionspace)),"Data is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(numpy.zeros((2,))),"ndarray is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(numpy.zeros((5,))),"ndarray is not appropriate substitute (shape)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_Symbol_rank1_d2_nargs0(self):
      s2=Symbol(shape=())
      s=Symbol(shape=(2,),dim=2,args=[])
      self.assertEqual(s.getRank(),1,"wrong rank.")
      self.assertEqual(s.getShape(),(2,),"wrong shape.")
      self.assertEqual(s.getDim(),2,"wrong spatial dimension.")
      self.assertEqual(s.getArgument(),[],"wrong arguments.")

      ss=s.substitute({s:numpy.zeros((2,))})
      self.assertTrue(isinstance(ss,numpy.ndarray),"value after substitution is not ndarray.")
      self.assertEqual(ss.shape,(2,),"value after substitution has not expected shape")
      try:
        s.substitute({s:numpy.zeros((5,))})
        fail("illegal substition was successful")
      except TypeError:
        pass

      dsdarg=s.diff(Symbol(shape=()))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds() has wrong type.")
      self.assertEqual(dsdarg.shape,(2,),"ds/ds() has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds() has wrong value.")
      dsdarg=s.diff(Symbol(shape=(2,)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(2,) has wrong type.")
      self.assertEqual(dsdarg.shape,(2, 2),"ds/ds(2,) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(2,) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(4, 5)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(4, 5) has wrong type.")
      self.assertEqual(dsdarg.shape,(2, 4, 5),"ds/ds(4, 5) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(4, 5) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(6, 2, 2)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(6, 2, 2) has wrong type.")
      self.assertEqual(dsdarg.shape,(2, 6, 2, 2),"ds/ds(6, 2, 2) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(6, 2, 2) has wrong value.")

      dsds=s.diff(s)
      self.assertTrue(isinstance(dsds,numpy.ndarray),"ds/ds has wrong type.")
      self.assertEqual(dsds.shape,(2, 2),"ds/ds has wrong shape.")
      self.assertEqual(dsds[0,0],1.0,"ds/ds has wrong value at (0,0).")
      self.assertEqual(dsds[0,1],0.0,"ds/ds has wrong value at (0,1).")
      self.assertEqual(dsds[1,0],0.0,"ds/ds has wrong value at (1,0).")
      self.assertEqual(dsds[1,1],1.0,"ds/ds has wrong value at (1,1).")

      sa=s.getSubstitutedArguments({s2:-10})
      self.assertEqual(len(sa),0,"wrong number of substituted arguments")

      self.assertFalse(s.isAppropriateValue(10.)," 10. is not appropriate substitute")
      self.assertFalse(s.isAppropriateValue(10)," 10 is not appropriate substitute")
      self.assertTrue(s.isAppropriateValue(Symbol(shape=(2,),dim=2)),"Symbol is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(Symbol(shape=(2,),dim=3)),"Symbol is not appropriate substitute (dim)")
      self.assertFalse(s.isAppropriateValue(escript.Data(0.,(5,),self.functionspace)),"Data is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(numpy.zeros((2,))),"ndarray is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(numpy.zeros((5,))),"ndarray is not appropriate substitute (shape)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_Symbol_rank1_d2_nargs1(self):
      s2=Symbol(shape=())
      s=Symbol(shape=(2,),dim=2,args=[s2])
      self.assertEqual(s.getRank(),1,"wrong rank.")
      self.assertEqual(s.getShape(),(2,),"wrong shape.")
      self.assertEqual(s.getDim(),2,"wrong spatial dimension.")
      self.assertEqual(s.getArgument(),[s2],"wrong arguments.")

      ss=s.substitute({s:numpy.zeros((2,))})
      self.assertTrue(isinstance(ss,numpy.ndarray),"value after substitution is not ndarray.")
      self.assertEqual(ss.shape,(2,),"value after substitution has not expected shape")
      try:
        s.substitute({s:numpy.zeros((5,))})
        fail("illegal substition was successful")
      except TypeError:
        pass

      dsdarg=s.diff(Symbol(shape=()))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds() has wrong type.")
      self.assertEqual(dsdarg.shape,(2,),"ds/ds() has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds() has wrong value.")
      dsdarg=s.diff(Symbol(shape=(2,)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(2,) has wrong type.")
      self.assertEqual(dsdarg.shape,(2, 2),"ds/ds(2,) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(2,) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(4, 5)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(4, 5) has wrong type.")
      self.assertEqual(dsdarg.shape,(2, 4, 5),"ds/ds(4, 5) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(4, 5) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(6, 2, 2)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(6, 2, 2) has wrong type.")
      self.assertEqual(dsdarg.shape,(2, 6, 2, 2),"ds/ds(6, 2, 2) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(6, 2, 2) has wrong value.")

      dsds=s.diff(s)
      self.assertTrue(isinstance(dsds,numpy.ndarray),"ds/ds has wrong type.")
      self.assertEqual(dsds.shape,(2, 2),"ds/ds has wrong shape.")
      self.assertEqual(dsds[0,0],1.0,"ds/ds has wrong value at (0,0).")
      self.assertEqual(dsds[0,1],0.0,"ds/ds has wrong value at (0,1).")
      self.assertEqual(dsds[1,0],0.0,"ds/ds has wrong value at (1,0).")
      self.assertEqual(dsds[1,1],1.0,"ds/ds has wrong value at (1,1).")

      self.assertEqual(s.getArgument(0),s2,"wrong argument 0.")
      sa=s.getSubstitutedArguments({s2:-10})
      self.assertEqual(len(sa),1,"wrong number of substituted arguments")
      self.assertEqual(sa[0],-10,"wrongly substituted argument 0.")

      self.assertFalse(s.isAppropriateValue(10.)," 10. is not appropriate substitute")
      self.assertFalse(s.isAppropriateValue(10)," 10 is not appropriate substitute")
      self.assertTrue(s.isAppropriateValue(Symbol(shape=(2,),dim=2)),"Symbol is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(Symbol(shape=(2,),dim=3)),"Symbol is not appropriate substitute (dim)")
      self.assertFalse(s.isAppropriateValue(escript.Data(0.,(5,),self.functionspace)),"Data is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(numpy.zeros((2,))),"ndarray is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(numpy.zeros((5,))),"ndarray is not appropriate substitute (shape)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_Symbol_rank1_d2_nargs2(self):
      s2=Symbol(shape=())
      s=Symbol(shape=(2,),dim=2,args=[1, -1.0])
      self.assertEqual(s.getRank(),1,"wrong rank.")
      self.assertEqual(s.getShape(),(2,),"wrong shape.")
      self.assertEqual(s.getDim(),2,"wrong spatial dimension.")
      self.assertEqual(s.getArgument(),[1, -1.0],"wrong arguments.")

      ss=s.substitute({s:numpy.zeros((2,))})
      self.assertTrue(isinstance(ss,numpy.ndarray),"value after substitution is not ndarray.")
      self.assertEqual(ss.shape,(2,),"value after substitution has not expected shape")
      try:
        s.substitute({s:numpy.zeros((5,))})
        fail("illegal substition was successful")
      except TypeError:
        pass

      dsdarg=s.diff(Symbol(shape=()))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds() has wrong type.")
      self.assertEqual(dsdarg.shape,(2,),"ds/ds() has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds() has wrong value.")
      dsdarg=s.diff(Symbol(shape=(2,)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(2,) has wrong type.")
      self.assertEqual(dsdarg.shape,(2, 2),"ds/ds(2,) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(2,) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(4, 5)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(4, 5) has wrong type.")
      self.assertEqual(dsdarg.shape,(2, 4, 5),"ds/ds(4, 5) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(4, 5) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(6, 2, 2)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(6, 2, 2) has wrong type.")
      self.assertEqual(dsdarg.shape,(2, 6, 2, 2),"ds/ds(6, 2, 2) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(6, 2, 2) has wrong value.")

      dsds=s.diff(s)
      self.assertTrue(isinstance(dsds,numpy.ndarray),"ds/ds has wrong type.")
      self.assertEqual(dsds.shape,(2, 2),"ds/ds has wrong shape.")
      self.assertEqual(dsds[0,0],1.0,"ds/ds has wrong value at (0,0).")
      self.assertEqual(dsds[0,1],0.0,"ds/ds has wrong value at (0,1).")
      self.assertEqual(dsds[1,0],0.0,"ds/ds has wrong value at (1,0).")
      self.assertEqual(dsds[1,1],1.0,"ds/ds has wrong value at (1,1).")

      self.assertEqual(s.getArgument(0),1,"wrong argument 0.")
      self.assertEqual(s.getArgument(1),-1.0,"wrong argument 1.")
      sa=s.getSubstitutedArguments({s2:-10})
      self.assertEqual(len(sa),2,"wrong number of substituted arguments")
      self.assertEqual(sa[0],1,"wrongly substituted argument 0.")
      self.assertEqual(sa[1],-1.0,"wrongly substituted argument 1.")

      self.assertFalse(s.isAppropriateValue(10.)," 10. is not appropriate substitute")
      self.assertFalse(s.isAppropriateValue(10)," 10 is not appropriate substitute")
      self.assertTrue(s.isAppropriateValue(Symbol(shape=(2,),dim=2)),"Symbol is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(Symbol(shape=(2,),dim=3)),"Symbol is not appropriate substitute (dim)")
      self.assertFalse(s.isAppropriateValue(escript.Data(0.,(5,),self.functionspace)),"Data is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(numpy.zeros((2,))),"ndarray is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(numpy.zeros((5,))),"ndarray is not appropriate substitute (shape)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_Symbol_rank1_d3_nargs0(self):
      s2=Symbol(shape=())
      s=Symbol(shape=(2,),dim=3,args=[])
      self.assertEqual(s.getRank(),1,"wrong rank.")
      self.assertEqual(s.getShape(),(2,),"wrong shape.")
      self.assertEqual(s.getDim(),3,"wrong spatial dimension.")
      self.assertEqual(s.getArgument(),[],"wrong arguments.")

      ss=s.substitute({s:numpy.zeros((2,))})
      self.assertTrue(isinstance(ss,numpy.ndarray),"value after substitution is not ndarray.")
      self.assertEqual(ss.shape,(2,),"value after substitution has not expected shape")
      try:
        s.substitute({s:numpy.zeros((5,))})
        fail("illegal substition was successful")
      except TypeError:
        pass

      dsdarg=s.diff(Symbol(shape=()))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds() has wrong type.")
      self.assertEqual(dsdarg.shape,(2,),"ds/ds() has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds() has wrong value.")
      dsdarg=s.diff(Symbol(shape=(2,)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(2,) has wrong type.")
      self.assertEqual(dsdarg.shape,(2, 2),"ds/ds(2,) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(2,) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(4, 5)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(4, 5) has wrong type.")
      self.assertEqual(dsdarg.shape,(2, 4, 5),"ds/ds(4, 5) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(4, 5) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(6, 2, 2)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(6, 2, 2) has wrong type.")
      self.assertEqual(dsdarg.shape,(2, 6, 2, 2),"ds/ds(6, 2, 2) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(6, 2, 2) has wrong value.")

      dsds=s.diff(s)
      self.assertTrue(isinstance(dsds,numpy.ndarray),"ds/ds has wrong type.")
      self.assertEqual(dsds.shape,(2, 2),"ds/ds has wrong shape.")
      self.assertEqual(dsds[0,0],1.0,"ds/ds has wrong value at (0,0).")
      self.assertEqual(dsds[0,1],0.0,"ds/ds has wrong value at (0,1).")
      self.assertEqual(dsds[1,0],0.0,"ds/ds has wrong value at (1,0).")
      self.assertEqual(dsds[1,1],1.0,"ds/ds has wrong value at (1,1).")

      sa=s.getSubstitutedArguments({s2:-10})
      self.assertEqual(len(sa),0,"wrong number of substituted arguments")

      self.assertFalse(s.isAppropriateValue(10.)," 10. is not appropriate substitute")
      self.assertFalse(s.isAppropriateValue(10)," 10 is not appropriate substitute")
      self.assertTrue(s.isAppropriateValue(Symbol(shape=(2,),dim=3)),"Symbol is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(Symbol(shape=(2,),dim=4)),"Symbol is not appropriate substitute (dim)")
      self.assertFalse(s.isAppropriateValue(escript.Data(0.,(5,),self.functionspace)),"Data is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(numpy.zeros((2,))),"ndarray is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(numpy.zeros((5,))),"ndarray is not appropriate substitute (shape)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_Symbol_rank1_d3_nargs1(self):
      s2=Symbol(shape=())
      s=Symbol(shape=(2,),dim=3,args=[s2])
      self.assertEqual(s.getRank(),1,"wrong rank.")
      self.assertEqual(s.getShape(),(2,),"wrong shape.")
      self.assertEqual(s.getDim(),3,"wrong spatial dimension.")
      self.assertEqual(s.getArgument(),[s2],"wrong arguments.")

      ss=s.substitute({s:numpy.zeros((2,))})
      self.assertTrue(isinstance(ss,numpy.ndarray),"value after substitution is not ndarray.")
      self.assertEqual(ss.shape,(2,),"value after substitution has not expected shape")
      try:
        s.substitute({s:numpy.zeros((5,))})
        fail("illegal substition was successful")
      except TypeError:
        pass

      dsdarg=s.diff(Symbol(shape=()))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds() has wrong type.")
      self.assertEqual(dsdarg.shape,(2,),"ds/ds() has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds() has wrong value.")
      dsdarg=s.diff(Symbol(shape=(2,)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(2,) has wrong type.")
      self.assertEqual(dsdarg.shape,(2, 2),"ds/ds(2,) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(2,) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(4, 5)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(4, 5) has wrong type.")
      self.assertEqual(dsdarg.shape,(2, 4, 5),"ds/ds(4, 5) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(4, 5) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(6, 2, 2)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(6, 2, 2) has wrong type.")
      self.assertEqual(dsdarg.shape,(2, 6, 2, 2),"ds/ds(6, 2, 2) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(6, 2, 2) has wrong value.")

      dsds=s.diff(s)
      self.assertTrue(isinstance(dsds,numpy.ndarray),"ds/ds has wrong type.")
      self.assertEqual(dsds.shape,(2, 2),"ds/ds has wrong shape.")
      self.assertEqual(dsds[0,0],1.0,"ds/ds has wrong value at (0,0).")
      self.assertEqual(dsds[0,1],0.0,"ds/ds has wrong value at (0,1).")
      self.assertEqual(dsds[1,0],0.0,"ds/ds has wrong value at (1,0).")
      self.assertEqual(dsds[1,1],1.0,"ds/ds has wrong value at (1,1).")

      self.assertEqual(s.getArgument(0),s2,"wrong argument 0.")
      sa=s.getSubstitutedArguments({s2:-10})
      self.assertEqual(len(sa),1,"wrong number of substituted arguments")
      self.assertEqual(sa[0],-10,"wrongly substituted argument 0.")

      self.assertFalse(s.isAppropriateValue(10.)," 10. is not appropriate substitute")
      self.assertFalse(s.isAppropriateValue(10)," 10 is not appropriate substitute")
      self.assertTrue(s.isAppropriateValue(Symbol(shape=(2,),dim=3)),"Symbol is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(Symbol(shape=(2,),dim=4)),"Symbol is not appropriate substitute (dim)")
      self.assertFalse(s.isAppropriateValue(escript.Data(0.,(5,),self.functionspace)),"Data is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(numpy.zeros((2,))),"ndarray is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(numpy.zeros((5,))),"ndarray is not appropriate substitute (shape)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_Symbol_rank1_d3_nargs2(self):
      s2=Symbol(shape=())
      s=Symbol(shape=(2,),dim=3,args=[1, -1.0])
      self.assertEqual(s.getRank(),1,"wrong rank.")
      self.assertEqual(s.getShape(),(2,),"wrong shape.")
      self.assertEqual(s.getDim(),3,"wrong spatial dimension.")
      self.assertEqual(s.getArgument(),[1, -1.0],"wrong arguments.")

      ss=s.substitute({s:numpy.zeros((2,))})
      self.assertTrue(isinstance(ss,numpy.ndarray),"value after substitution is not ndarray.")
      self.assertEqual(ss.shape,(2,),"value after substitution has not expected shape")
      try:
        s.substitute({s:numpy.zeros((5,))})
        fail("illegal substition was successful")
      except TypeError:
        pass

      dsdarg=s.diff(Symbol(shape=()))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds() has wrong type.")
      self.assertEqual(dsdarg.shape,(2,),"ds/ds() has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds() has wrong value.")
      dsdarg=s.diff(Symbol(shape=(2,)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(2,) has wrong type.")
      self.assertEqual(dsdarg.shape,(2, 2),"ds/ds(2,) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(2,) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(4, 5)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(4, 5) has wrong type.")
      self.assertEqual(dsdarg.shape,(2, 4, 5),"ds/ds(4, 5) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(4, 5) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(6, 2, 2)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(6, 2, 2) has wrong type.")
      self.assertEqual(dsdarg.shape,(2, 6, 2, 2),"ds/ds(6, 2, 2) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(6, 2, 2) has wrong value.")

      dsds=s.diff(s)
      self.assertTrue(isinstance(dsds,numpy.ndarray),"ds/ds has wrong type.")
      self.assertEqual(dsds.shape,(2, 2),"ds/ds has wrong shape.")
      self.assertEqual(dsds[0,0],1.0,"ds/ds has wrong value at (0,0).")
      self.assertEqual(dsds[0,1],0.0,"ds/ds has wrong value at (0,1).")
      self.assertEqual(dsds[1,0],0.0,"ds/ds has wrong value at (1,0).")
      self.assertEqual(dsds[1,1],1.0,"ds/ds has wrong value at (1,1).")

      self.assertEqual(s.getArgument(0),1,"wrong argument 0.")
      self.assertEqual(s.getArgument(1),-1.0,"wrong argument 1.")
      sa=s.getSubstitutedArguments({s2:-10})
      self.assertEqual(len(sa),2,"wrong number of substituted arguments")
      self.assertEqual(sa[0],1,"wrongly substituted argument 0.")
      self.assertEqual(sa[1],-1.0,"wrongly substituted argument 1.")

      self.assertFalse(s.isAppropriateValue(10.)," 10. is not appropriate substitute")
      self.assertFalse(s.isAppropriateValue(10)," 10 is not appropriate substitute")
      self.assertTrue(s.isAppropriateValue(Symbol(shape=(2,),dim=3)),"Symbol is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(Symbol(shape=(2,),dim=4)),"Symbol is not appropriate substitute (dim)")
      self.assertFalse(s.isAppropriateValue(escript.Data(0.,(5,),self.functionspace)),"Data is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(numpy.zeros((2,))),"ndarray is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(numpy.zeros((5,))),"ndarray is not appropriate substitute (shape)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_Symbol_rank2_dNone_nargs0(self):
      s2=Symbol(shape=())
      s=Symbol(shape=(4, 5),dim=None,args=[])
      self.assertEqual(s.getRank(),2,"wrong rank.")
      self.assertEqual(s.getShape(),(4, 5),"wrong shape.")
      self.assertEqual(s.getDim(),None,"wrong spatial dimension.")
      self.assertEqual(s.getArgument(),[],"wrong arguments.")

      ss=s.substitute({s:numpy.zeros((4, 5))})
      self.assertTrue(isinstance(ss,numpy.ndarray),"value after substitution is not ndarray.")
      self.assertEqual(ss.shape,(4, 5),"value after substitution has not expected shape")
      try:
        s.substitute({s:numpy.zeros((5,))})
        fail("illegal substition was successful")
      except TypeError:
        pass

      dsdarg=s.diff(Symbol(shape=()))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds() has wrong type.")
      self.assertEqual(dsdarg.shape,(4, 5),"ds/ds() has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds() has wrong value.")
      dsdarg=s.diff(Symbol(shape=(2,)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(2,) has wrong type.")
      self.assertEqual(dsdarg.shape,(4, 5, 2),"ds/ds(2,) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(2,) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(4, 5)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(4, 5) has wrong type.")
      self.assertEqual(dsdarg.shape,(4, 5, 4, 5),"ds/ds(4, 5) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(4, 5) has wrong value.")

      dsds=s.diff(s)
      self.assertTrue(isinstance(dsds,numpy.ndarray),"ds/ds has wrong type.")
      self.assertEqual(dsds.shape,(4, 5, 4, 5),"ds/ds has wrong shape.")
      self.assertEqual(dsds[0,0,0,0],1.0,"ds/ds has wrong value at (0,0,0,0).")
      self.assertEqual(dsds[0,0,0,1],0.0,"ds/ds has wrong value at (0,0,0,1).")
      self.assertEqual(dsds[0,0,0,2],0.0,"ds/ds has wrong value at (0,0,0,2).")
      self.assertEqual(dsds[0,0,0,3],0.0,"ds/ds has wrong value at (0,0,0,3).")
      self.assertEqual(dsds[0,0,0,4],0.0,"ds/ds has wrong value at (0,0,0,4).")
      self.assertEqual(dsds[0,0,1,0],0.0,"ds/ds has wrong value at (0,0,1,0).")
      self.assertEqual(dsds[0,0,1,1],0.0,"ds/ds has wrong value at (0,0,1,1).")
      self.assertEqual(dsds[0,0,1,2],0.0,"ds/ds has wrong value at (0,0,1,2).")
      self.assertEqual(dsds[0,0,1,3],0.0,"ds/ds has wrong value at (0,0,1,3).")
      self.assertEqual(dsds[0,0,1,4],0.0,"ds/ds has wrong value at (0,0,1,4).")
      self.assertEqual(dsds[0,0,2,0],0.0,"ds/ds has wrong value at (0,0,2,0).")
      self.assertEqual(dsds[0,0,2,1],0.0,"ds/ds has wrong value at (0,0,2,1).")
      self.assertEqual(dsds[0,0,2,2],0.0,"ds/ds has wrong value at (0,0,2,2).")
      self.assertEqual(dsds[0,0,2,3],0.0,"ds/ds has wrong value at (0,0,2,3).")
      self.assertEqual(dsds[0,0,2,4],0.0,"ds/ds has wrong value at (0,0,2,4).")
      self.assertEqual(dsds[0,0,3,0],0.0,"ds/ds has wrong value at (0,0,3,0).")
      self.assertEqual(dsds[0,0,3,1],0.0,"ds/ds has wrong value at (0,0,3,1).")
      self.assertEqual(dsds[0,0,3,2],0.0,"ds/ds has wrong value at (0,0,3,2).")
      self.assertEqual(dsds[0,0,3,3],0.0,"ds/ds has wrong value at (0,0,3,3).")
      self.assertEqual(dsds[0,0,3,4],0.0,"ds/ds has wrong value at (0,0,3,4).")
      self.assertEqual(dsds[0,1,0,0],0.0,"ds/ds has wrong value at (0,1,0,0).")
      self.assertEqual(dsds[0,1,0,1],1.0,"ds/ds has wrong value at (0,1,0,1).")
      self.assertEqual(dsds[0,1,0,2],0.0,"ds/ds has wrong value at (0,1,0,2).")
      self.assertEqual(dsds[0,1,0,3],0.0,"ds/ds has wrong value at (0,1,0,3).")
      self.assertEqual(dsds[0,1,0,4],0.0,"ds/ds has wrong value at (0,1,0,4).")
      self.assertEqual(dsds[0,1,1,0],0.0,"ds/ds has wrong value at (0,1,1,0).")
      self.assertEqual(dsds[0,1,1,1],0.0,"ds/ds has wrong value at (0,1,1,1).")
      self.assertEqual(dsds[0,1,1,2],0.0,"ds/ds has wrong value at (0,1,1,2).")
      self.assertEqual(dsds[0,1,1,3],0.0,"ds/ds has wrong value at (0,1,1,3).")
      self.assertEqual(dsds[0,1,1,4],0.0,"ds/ds has wrong value at (0,1,1,4).")
      self.assertEqual(dsds[0,1,2,0],0.0,"ds/ds has wrong value at (0,1,2,0).")
      self.assertEqual(dsds[0,1,2,1],0.0,"ds/ds has wrong value at (0,1,2,1).")
      self.assertEqual(dsds[0,1,2,2],0.0,"ds/ds has wrong value at (0,1,2,2).")
      self.assertEqual(dsds[0,1,2,3],0.0,"ds/ds has wrong value at (0,1,2,3).")
      self.assertEqual(dsds[0,1,2,4],0.0,"ds/ds has wrong value at (0,1,2,4).")
      self.assertEqual(dsds[0,1,3,0],0.0,"ds/ds has wrong value at (0,1,3,0).")
      self.assertEqual(dsds[0,1,3,1],0.0,"ds/ds has wrong value at (0,1,3,1).")
      self.assertEqual(dsds[0,1,3,2],0.0,"ds/ds has wrong value at (0,1,3,2).")
      self.assertEqual(dsds[0,1,3,3],0.0,"ds/ds has wrong value at (0,1,3,3).")
      self.assertEqual(dsds[0,1,3,4],0.0,"ds/ds has wrong value at (0,1,3,4).")
      self.assertEqual(dsds[0,2,0,0],0.0,"ds/ds has wrong value at (0,2,0,0).")
      self.assertEqual(dsds[0,2,0,1],0.0,"ds/ds has wrong value at (0,2,0,1).")
      self.assertEqual(dsds[0,2,0,2],1.0,"ds/ds has wrong value at (0,2,0,2).")
      self.assertEqual(dsds[0,2,0,3],0.0,"ds/ds has wrong value at (0,2,0,3).")
      self.assertEqual(dsds[0,2,0,4],0.0,"ds/ds has wrong value at (0,2,0,4).")
      self.assertEqual(dsds[0,2,1,0],0.0,"ds/ds has wrong value at (0,2,1,0).")
      self.assertEqual(dsds[0,2,1,1],0.0,"ds/ds has wrong value at (0,2,1,1).")
      self.assertEqual(dsds[0,2,1,2],0.0,"ds/ds has wrong value at (0,2,1,2).")
      self.assertEqual(dsds[0,2,1,3],0.0,"ds/ds has wrong value at (0,2,1,3).")
      self.assertEqual(dsds[0,2,1,4],0.0,"ds/ds has wrong value at (0,2,1,4).")
      self.assertEqual(dsds[0,2,2,0],0.0,"ds/ds has wrong value at (0,2,2,0).")
      self.assertEqual(dsds[0,2,2,1],0.0,"ds/ds has wrong value at (0,2,2,1).")
      self.assertEqual(dsds[0,2,2,2],0.0,"ds/ds has wrong value at (0,2,2,2).")
      self.assertEqual(dsds[0,2,2,3],0.0,"ds/ds has wrong value at (0,2,2,3).")
      self.assertEqual(dsds[0,2,2,4],0.0,"ds/ds has wrong value at (0,2,2,4).")
      self.assertEqual(dsds[0,2,3,0],0.0,"ds/ds has wrong value at (0,2,3,0).")
      self.assertEqual(dsds[0,2,3,1],0.0,"ds/ds has wrong value at (0,2,3,1).")
      self.assertEqual(dsds[0,2,3,2],0.0,"ds/ds has wrong value at (0,2,3,2).")
      self.assertEqual(dsds[0,2,3,3],0.0,"ds/ds has wrong value at (0,2,3,3).")
      self.assertEqual(dsds[0,2,3,4],0.0,"ds/ds has wrong value at (0,2,3,4).")
      self.assertEqual(dsds[0,3,0,0],0.0,"ds/ds has wrong value at (0,3,0,0).")
      self.assertEqual(dsds[0,3,0,1],0.0,"ds/ds has wrong value at (0,3,0,1).")
      self.assertEqual(dsds[0,3,0,2],0.0,"ds/ds has wrong value at (0,3,0,2).")
      self.assertEqual(dsds[0,3,0,3],1.0,"ds/ds has wrong value at (0,3,0,3).")
      self.assertEqual(dsds[0,3,0,4],0.0,"ds/ds has wrong value at (0,3,0,4).")
      self.assertEqual(dsds[0,3,1,0],0.0,"ds/ds has wrong value at (0,3,1,0).")
      self.assertEqual(dsds[0,3,1,1],0.0,"ds/ds has wrong value at (0,3,1,1).")
      self.assertEqual(dsds[0,3,1,2],0.0,"ds/ds has wrong value at (0,3,1,2).")
      self.assertEqual(dsds[0,3,1,3],0.0,"ds/ds has wrong value at (0,3,1,3).")
      self.assertEqual(dsds[0,3,1,4],0.0,"ds/ds has wrong value at (0,3,1,4).")
      self.assertEqual(dsds[0,3,2,0],0.0,"ds/ds has wrong value at (0,3,2,0).")
      self.assertEqual(dsds[0,3,2,1],0.0,"ds/ds has wrong value at (0,3,2,1).")
      self.assertEqual(dsds[0,3,2,2],0.0,"ds/ds has wrong value at (0,3,2,2).")
      self.assertEqual(dsds[0,3,2,3],0.0,"ds/ds has wrong value at (0,3,2,3).")
      self.assertEqual(dsds[0,3,2,4],0.0,"ds/ds has wrong value at (0,3,2,4).")
      self.assertEqual(dsds[0,3,3,0],0.0,"ds/ds has wrong value at (0,3,3,0).")
      self.assertEqual(dsds[0,3,3,1],0.0,"ds/ds has wrong value at (0,3,3,1).")
      self.assertEqual(dsds[0,3,3,2],0.0,"ds/ds has wrong value at (0,3,3,2).")
      self.assertEqual(dsds[0,3,3,3],0.0,"ds/ds has wrong value at (0,3,3,3).")
      self.assertEqual(dsds[0,3,3,4],0.0,"ds/ds has wrong value at (0,3,3,4).")
      self.assertEqual(dsds[0,4,0,0],0.0,"ds/ds has wrong value at (0,4,0,0).")
      self.assertEqual(dsds[0,4,0,1],0.0,"ds/ds has wrong value at (0,4,0,1).")
      self.assertEqual(dsds[0,4,0,2],0.0,"ds/ds has wrong value at (0,4,0,2).")
      self.assertEqual(dsds[0,4,0,3],0.0,"ds/ds has wrong value at (0,4,0,3).")
      self.assertEqual(dsds[0,4,0,4],1.0,"ds/ds has wrong value at (0,4,0,4).")
      self.assertEqual(dsds[0,4,1,0],0.0,"ds/ds has wrong value at (0,4,1,0).")
      self.assertEqual(dsds[0,4,1,1],0.0,"ds/ds has wrong value at (0,4,1,1).")
      self.assertEqual(dsds[0,4,1,2],0.0,"ds/ds has wrong value at (0,4,1,2).")
      self.assertEqual(dsds[0,4,1,3],0.0,"ds/ds has wrong value at (0,4,1,3).")
      self.assertEqual(dsds[0,4,1,4],0.0,"ds/ds has wrong value at (0,4,1,4).")
      self.assertEqual(dsds[0,4,2,0],0.0,"ds/ds has wrong value at (0,4,2,0).")
      self.assertEqual(dsds[0,4,2,1],0.0,"ds/ds has wrong value at (0,4,2,1).")
      self.assertEqual(dsds[0,4,2,2],0.0,"ds/ds has wrong value at (0,4,2,2).")
      self.assertEqual(dsds[0,4,2,3],0.0,"ds/ds has wrong value at (0,4,2,3).")
      self.assertEqual(dsds[0,4,2,4],0.0,"ds/ds has wrong value at (0,4,2,4).")
      self.assertEqual(dsds[0,4,3,0],0.0,"ds/ds has wrong value at (0,4,3,0).")
      self.assertEqual(dsds[0,4,3,1],0.0,"ds/ds has wrong value at (0,4,3,1).")
      self.assertEqual(dsds[0,4,3,2],0.0,"ds/ds has wrong value at (0,4,3,2).")
      self.assertEqual(dsds[0,4,3,3],0.0,"ds/ds has wrong value at (0,4,3,3).")
      self.assertEqual(dsds[0,4,3,4],0.0,"ds/ds has wrong value at (0,4,3,4).")
      self.assertEqual(dsds[1,0,0,0],0.0,"ds/ds has wrong value at (1,0,0,0).")
      self.assertEqual(dsds[1,0,0,1],0.0,"ds/ds has wrong value at (1,0,0,1).")
      self.assertEqual(dsds[1,0,0,2],0.0,"ds/ds has wrong value at (1,0,0,2).")
      self.assertEqual(dsds[1,0,0,3],0.0,"ds/ds has wrong value at (1,0,0,3).")
      self.assertEqual(dsds[1,0,0,4],0.0,"ds/ds has wrong value at (1,0,0,4).")
      self.assertEqual(dsds[1,0,1,0],1.0,"ds/ds has wrong value at (1,0,1,0).")
      self.assertEqual(dsds[1,0,1,1],0.0,"ds/ds has wrong value at (1,0,1,1).")
      self.assertEqual(dsds[1,0,1,2],0.0,"ds/ds has wrong value at (1,0,1,2).")
      self.assertEqual(dsds[1,0,1,3],0.0,"ds/ds has wrong value at (1,0,1,3).")
      self.assertEqual(dsds[1,0,1,4],0.0,"ds/ds has wrong value at (1,0,1,4).")
      self.assertEqual(dsds[1,0,2,0],0.0,"ds/ds has wrong value at (1,0,2,0).")
      self.assertEqual(dsds[1,0,2,1],0.0,"ds/ds has wrong value at (1,0,2,1).")
      self.assertEqual(dsds[1,0,2,2],0.0,"ds/ds has wrong value at (1,0,2,2).")
      self.assertEqual(dsds[1,0,2,3],0.0,"ds/ds has wrong value at (1,0,2,3).")
      self.assertEqual(dsds[1,0,2,4],0.0,"ds/ds has wrong value at (1,0,2,4).")
      self.assertEqual(dsds[1,0,3,0],0.0,"ds/ds has wrong value at (1,0,3,0).")
      self.assertEqual(dsds[1,0,3,1],0.0,"ds/ds has wrong value at (1,0,3,1).")
      self.assertEqual(dsds[1,0,3,2],0.0,"ds/ds has wrong value at (1,0,3,2).")
      self.assertEqual(dsds[1,0,3,3],0.0,"ds/ds has wrong value at (1,0,3,3).")
      self.assertEqual(dsds[1,0,3,4],0.0,"ds/ds has wrong value at (1,0,3,4).")
      self.assertEqual(dsds[1,1,0,0],0.0,"ds/ds has wrong value at (1,1,0,0).")
      self.assertEqual(dsds[1,1,0,1],0.0,"ds/ds has wrong value at (1,1,0,1).")
      self.assertEqual(dsds[1,1,0,2],0.0,"ds/ds has wrong value at (1,1,0,2).")
      self.assertEqual(dsds[1,1,0,3],0.0,"ds/ds has wrong value at (1,1,0,3).")
      self.assertEqual(dsds[1,1,0,4],0.0,"ds/ds has wrong value at (1,1,0,4).")
      self.assertEqual(dsds[1,1,1,0],0.0,"ds/ds has wrong value at (1,1,1,0).")
      self.assertEqual(dsds[1,1,1,1],1.0,"ds/ds has wrong value at (1,1,1,1).")
      self.assertEqual(dsds[1,1,1,2],0.0,"ds/ds has wrong value at (1,1,1,2).")
      self.assertEqual(dsds[1,1,1,3],0.0,"ds/ds has wrong value at (1,1,1,3).")
      self.assertEqual(dsds[1,1,1,4],0.0,"ds/ds has wrong value at (1,1,1,4).")
      self.assertEqual(dsds[1,1,2,0],0.0,"ds/ds has wrong value at (1,1,2,0).")
      self.assertEqual(dsds[1,1,2,1],0.0,"ds/ds has wrong value at (1,1,2,1).")
      self.assertEqual(dsds[1,1,2,2],0.0,"ds/ds has wrong value at (1,1,2,2).")
      self.assertEqual(dsds[1,1,2,3],0.0,"ds/ds has wrong value at (1,1,2,3).")
      self.assertEqual(dsds[1,1,2,4],0.0,"ds/ds has wrong value at (1,1,2,4).")
      self.assertEqual(dsds[1,1,3,0],0.0,"ds/ds has wrong value at (1,1,3,0).")
      self.assertEqual(dsds[1,1,3,1],0.0,"ds/ds has wrong value at (1,1,3,1).")
      self.assertEqual(dsds[1,1,3,2],0.0,"ds/ds has wrong value at (1,1,3,2).")
      self.assertEqual(dsds[1,1,3,3],0.0,"ds/ds has wrong value at (1,1,3,3).")
      self.assertEqual(dsds[1,1,3,4],0.0,"ds/ds has wrong value at (1,1,3,4).")
      self.assertEqual(dsds[1,2,0,0],0.0,"ds/ds has wrong value at (1,2,0,0).")
      self.assertEqual(dsds[1,2,0,1],0.0,"ds/ds has wrong value at (1,2,0,1).")
      self.assertEqual(dsds[1,2,0,2],0.0,"ds/ds has wrong value at (1,2,0,2).")
      self.assertEqual(dsds[1,2,0,3],0.0,"ds/ds has wrong value at (1,2,0,3).")
      self.assertEqual(dsds[1,2,0,4],0.0,"ds/ds has wrong value at (1,2,0,4).")
      self.assertEqual(dsds[1,2,1,0],0.0,"ds/ds has wrong value at (1,2,1,0).")
      self.assertEqual(dsds[1,2,1,1],0.0,"ds/ds has wrong value at (1,2,1,1).")
      self.assertEqual(dsds[1,2,1,2],1.0,"ds/ds has wrong value at (1,2,1,2).")
      self.assertEqual(dsds[1,2,1,3],0.0,"ds/ds has wrong value at (1,2,1,3).")
      self.assertEqual(dsds[1,2,1,4],0.0,"ds/ds has wrong value at (1,2,1,4).")
      self.assertEqual(dsds[1,2,2,0],0.0,"ds/ds has wrong value at (1,2,2,0).")
      self.assertEqual(dsds[1,2,2,1],0.0,"ds/ds has wrong value at (1,2,2,1).")
      self.assertEqual(dsds[1,2,2,2],0.0,"ds/ds has wrong value at (1,2,2,2).")
      self.assertEqual(dsds[1,2,2,3],0.0,"ds/ds has wrong value at (1,2,2,3).")
      self.assertEqual(dsds[1,2,2,4],0.0,"ds/ds has wrong value at (1,2,2,4).")
      self.assertEqual(dsds[1,2,3,0],0.0,"ds/ds has wrong value at (1,2,3,0).")
      self.assertEqual(dsds[1,2,3,1],0.0,"ds/ds has wrong value at (1,2,3,1).")
      self.assertEqual(dsds[1,2,3,2],0.0,"ds/ds has wrong value at (1,2,3,2).")
      self.assertEqual(dsds[1,2,3,3],0.0,"ds/ds has wrong value at (1,2,3,3).")
      self.assertEqual(dsds[1,2,3,4],0.0,"ds/ds has wrong value at (1,2,3,4).")
      self.assertEqual(dsds[1,3,0,0],0.0,"ds/ds has wrong value at (1,3,0,0).")
      self.assertEqual(dsds[1,3,0,1],0.0,"ds/ds has wrong value at (1,3,0,1).")
      self.assertEqual(dsds[1,3,0,2],0.0,"ds/ds has wrong value at (1,3,0,2).")
      self.assertEqual(dsds[1,3,0,3],0.0,"ds/ds has wrong value at (1,3,0,3).")
      self.assertEqual(dsds[1,3,0,4],0.0,"ds/ds has wrong value at (1,3,0,4).")
      self.assertEqual(dsds[1,3,1,0],0.0,"ds/ds has wrong value at (1,3,1,0).")
      self.assertEqual(dsds[1,3,1,1],0.0,"ds/ds has wrong value at (1,3,1,1).")
      self.assertEqual(dsds[1,3,1,2],0.0,"ds/ds has wrong value at (1,3,1,2).")
      self.assertEqual(dsds[1,3,1,3],1.0,"ds/ds has wrong value at (1,3,1,3).")
      self.assertEqual(dsds[1,3,1,4],0.0,"ds/ds has wrong value at (1,3,1,4).")
      self.assertEqual(dsds[1,3,2,0],0.0,"ds/ds has wrong value at (1,3,2,0).")
      self.assertEqual(dsds[1,3,2,1],0.0,"ds/ds has wrong value at (1,3,2,1).")
      self.assertEqual(dsds[1,3,2,2],0.0,"ds/ds has wrong value at (1,3,2,2).")
      self.assertEqual(dsds[1,3,2,3],0.0,"ds/ds has wrong value at (1,3,2,3).")
      self.assertEqual(dsds[1,3,2,4],0.0,"ds/ds has wrong value at (1,3,2,4).")
      self.assertEqual(dsds[1,3,3,0],0.0,"ds/ds has wrong value at (1,3,3,0).")
      self.assertEqual(dsds[1,3,3,1],0.0,"ds/ds has wrong value at (1,3,3,1).")
      self.assertEqual(dsds[1,3,3,2],0.0,"ds/ds has wrong value at (1,3,3,2).")
      self.assertEqual(dsds[1,3,3,3],0.0,"ds/ds has wrong value at (1,3,3,3).")
      self.assertEqual(dsds[1,3,3,4],0.0,"ds/ds has wrong value at (1,3,3,4).")
      self.assertEqual(dsds[1,4,0,0],0.0,"ds/ds has wrong value at (1,4,0,0).")
      self.assertEqual(dsds[1,4,0,1],0.0,"ds/ds has wrong value at (1,4,0,1).")
      self.assertEqual(dsds[1,4,0,2],0.0,"ds/ds has wrong value at (1,4,0,2).")
      self.assertEqual(dsds[1,4,0,3],0.0,"ds/ds has wrong value at (1,4,0,3).")
      self.assertEqual(dsds[1,4,0,4],0.0,"ds/ds has wrong value at (1,4,0,4).")
      self.assertEqual(dsds[1,4,1,0],0.0,"ds/ds has wrong value at (1,4,1,0).")
      self.assertEqual(dsds[1,4,1,1],0.0,"ds/ds has wrong value at (1,4,1,1).")
      self.assertEqual(dsds[1,4,1,2],0.0,"ds/ds has wrong value at (1,4,1,2).")
      self.assertEqual(dsds[1,4,1,3],0.0,"ds/ds has wrong value at (1,4,1,3).")
      self.assertEqual(dsds[1,4,1,4],1.0,"ds/ds has wrong value at (1,4,1,4).")
      self.assertEqual(dsds[1,4,2,0],0.0,"ds/ds has wrong value at (1,4,2,0).")
      self.assertEqual(dsds[1,4,2,1],0.0,"ds/ds has wrong value at (1,4,2,1).")
      self.assertEqual(dsds[1,4,2,2],0.0,"ds/ds has wrong value at (1,4,2,2).")
      self.assertEqual(dsds[1,4,2,3],0.0,"ds/ds has wrong value at (1,4,2,3).")
      self.assertEqual(dsds[1,4,2,4],0.0,"ds/ds has wrong value at (1,4,2,4).")
      self.assertEqual(dsds[1,4,3,0],0.0,"ds/ds has wrong value at (1,4,3,0).")
      self.assertEqual(dsds[1,4,3,1],0.0,"ds/ds has wrong value at (1,4,3,1).")
      self.assertEqual(dsds[1,4,3,2],0.0,"ds/ds has wrong value at (1,4,3,2).")
      self.assertEqual(dsds[1,4,3,3],0.0,"ds/ds has wrong value at (1,4,3,3).")
      self.assertEqual(dsds[1,4,3,4],0.0,"ds/ds has wrong value at (1,4,3,4).")
      self.assertEqual(dsds[2,0,0,0],0.0,"ds/ds has wrong value at (2,0,0,0).")
      self.assertEqual(dsds[2,0,0,1],0.0,"ds/ds has wrong value at (2,0,0,1).")
      self.assertEqual(dsds[2,0,0,2],0.0,"ds/ds has wrong value at (2,0,0,2).")
      self.assertEqual(dsds[2,0,0,3],0.0,"ds/ds has wrong value at (2,0,0,3).")
      self.assertEqual(dsds[2,0,0,4],0.0,"ds/ds has wrong value at (2,0,0,4).")
      self.assertEqual(dsds[2,0,1,0],0.0,"ds/ds has wrong value at (2,0,1,0).")
      self.assertEqual(dsds[2,0,1,1],0.0,"ds/ds has wrong value at (2,0,1,1).")
      self.assertEqual(dsds[2,0,1,2],0.0,"ds/ds has wrong value at (2,0,1,2).")
      self.assertEqual(dsds[2,0,1,3],0.0,"ds/ds has wrong value at (2,0,1,3).")
      self.assertEqual(dsds[2,0,1,4],0.0,"ds/ds has wrong value at (2,0,1,4).")
      self.assertEqual(dsds[2,0,2,0],1.0,"ds/ds has wrong value at (2,0,2,0).")
      self.assertEqual(dsds[2,0,2,1],0.0,"ds/ds has wrong value at (2,0,2,1).")
      self.assertEqual(dsds[2,0,2,2],0.0,"ds/ds has wrong value at (2,0,2,2).")
      self.assertEqual(dsds[2,0,2,3],0.0,"ds/ds has wrong value at (2,0,2,3).")
      self.assertEqual(dsds[2,0,2,4],0.0,"ds/ds has wrong value at (2,0,2,4).")
      self.assertEqual(dsds[2,0,3,0],0.0,"ds/ds has wrong value at (2,0,3,0).")
      self.assertEqual(dsds[2,0,3,1],0.0,"ds/ds has wrong value at (2,0,3,1).")
      self.assertEqual(dsds[2,0,3,2],0.0,"ds/ds has wrong value at (2,0,3,2).")
      self.assertEqual(dsds[2,0,3,3],0.0,"ds/ds has wrong value at (2,0,3,3).")
      self.assertEqual(dsds[2,0,3,4],0.0,"ds/ds has wrong value at (2,0,3,4).")
      self.assertEqual(dsds[2,1,0,0],0.0,"ds/ds has wrong value at (2,1,0,0).")
      self.assertEqual(dsds[2,1,0,1],0.0,"ds/ds has wrong value at (2,1,0,1).")
      self.assertEqual(dsds[2,1,0,2],0.0,"ds/ds has wrong value at (2,1,0,2).")
      self.assertEqual(dsds[2,1,0,3],0.0,"ds/ds has wrong value at (2,1,0,3).")
      self.assertEqual(dsds[2,1,0,4],0.0,"ds/ds has wrong value at (2,1,0,4).")
      self.assertEqual(dsds[2,1,1,0],0.0,"ds/ds has wrong value at (2,1,1,0).")
      self.assertEqual(dsds[2,1,1,1],0.0,"ds/ds has wrong value at (2,1,1,1).")
      self.assertEqual(dsds[2,1,1,2],0.0,"ds/ds has wrong value at (2,1,1,2).")
      self.assertEqual(dsds[2,1,1,3],0.0,"ds/ds has wrong value at (2,1,1,3).")
      self.assertEqual(dsds[2,1,1,4],0.0,"ds/ds has wrong value at (2,1,1,4).")
      self.assertEqual(dsds[2,1,2,0],0.0,"ds/ds has wrong value at (2,1,2,0).")
      self.assertEqual(dsds[2,1,2,1],1.0,"ds/ds has wrong value at (2,1,2,1).")
      self.assertEqual(dsds[2,1,2,2],0.0,"ds/ds has wrong value at (2,1,2,2).")
      self.assertEqual(dsds[2,1,2,3],0.0,"ds/ds has wrong value at (2,1,2,3).")
      self.assertEqual(dsds[2,1,2,4],0.0,"ds/ds has wrong value at (2,1,2,4).")
      self.assertEqual(dsds[2,1,3,0],0.0,"ds/ds has wrong value at (2,1,3,0).")
      self.assertEqual(dsds[2,1,3,1],0.0,"ds/ds has wrong value at (2,1,3,1).")
      self.assertEqual(dsds[2,1,3,2],0.0,"ds/ds has wrong value at (2,1,3,2).")
      self.assertEqual(dsds[2,1,3,3],0.0,"ds/ds has wrong value at (2,1,3,3).")
      self.assertEqual(dsds[2,1,3,4],0.0,"ds/ds has wrong value at (2,1,3,4).")
      self.assertEqual(dsds[2,2,0,0],0.0,"ds/ds has wrong value at (2,2,0,0).")
      self.assertEqual(dsds[2,2,0,1],0.0,"ds/ds has wrong value at (2,2,0,1).")
      self.assertEqual(dsds[2,2,0,2],0.0,"ds/ds has wrong value at (2,2,0,2).")
      self.assertEqual(dsds[2,2,0,3],0.0,"ds/ds has wrong value at (2,2,0,3).")
      self.assertEqual(dsds[2,2,0,4],0.0,"ds/ds has wrong value at (2,2,0,4).")
      self.assertEqual(dsds[2,2,1,0],0.0,"ds/ds has wrong value at (2,2,1,0).")
      self.assertEqual(dsds[2,2,1,1],0.0,"ds/ds has wrong value at (2,2,1,1).")
      self.assertEqual(dsds[2,2,1,2],0.0,"ds/ds has wrong value at (2,2,1,2).")
      self.assertEqual(dsds[2,2,1,3],0.0,"ds/ds has wrong value at (2,2,1,3).")
      self.assertEqual(dsds[2,2,1,4],0.0,"ds/ds has wrong value at (2,2,1,4).")
      self.assertEqual(dsds[2,2,2,0],0.0,"ds/ds has wrong value at (2,2,2,0).")
      self.assertEqual(dsds[2,2,2,1],0.0,"ds/ds has wrong value at (2,2,2,1).")
      self.assertEqual(dsds[2,2,2,2],1.0,"ds/ds has wrong value at (2,2,2,2).")
      self.assertEqual(dsds[2,2,2,3],0.0,"ds/ds has wrong value at (2,2,2,3).")
      self.assertEqual(dsds[2,2,2,4],0.0,"ds/ds has wrong value at (2,2,2,4).")
      self.assertEqual(dsds[2,2,3,0],0.0,"ds/ds has wrong value at (2,2,3,0).")
      self.assertEqual(dsds[2,2,3,1],0.0,"ds/ds has wrong value at (2,2,3,1).")
      self.assertEqual(dsds[2,2,3,2],0.0,"ds/ds has wrong value at (2,2,3,2).")
      self.assertEqual(dsds[2,2,3,3],0.0,"ds/ds has wrong value at (2,2,3,3).")
      self.assertEqual(dsds[2,2,3,4],0.0,"ds/ds has wrong value at (2,2,3,4).")
      self.assertEqual(dsds[2,3,0,0],0.0,"ds/ds has wrong value at (2,3,0,0).")
      self.assertEqual(dsds[2,3,0,1],0.0,"ds/ds has wrong value at (2,3,0,1).")
      self.assertEqual(dsds[2,3,0,2],0.0,"ds/ds has wrong value at (2,3,0,2).")
      self.assertEqual(dsds[2,3,0,3],0.0,"ds/ds has wrong value at (2,3,0,3).")
      self.assertEqual(dsds[2,3,0,4],0.0,"ds/ds has wrong value at (2,3,0,4).")
      self.assertEqual(dsds[2,3,1,0],0.0,"ds/ds has wrong value at (2,3,1,0).")
      self.assertEqual(dsds[2,3,1,1],0.0,"ds/ds has wrong value at (2,3,1,1).")
      self.assertEqual(dsds[2,3,1,2],0.0,"ds/ds has wrong value at (2,3,1,2).")
      self.assertEqual(dsds[2,3,1,3],0.0,"ds/ds has wrong value at (2,3,1,3).")
      self.assertEqual(dsds[2,3,1,4],0.0,"ds/ds has wrong value at (2,3,1,4).")
      self.assertEqual(dsds[2,3,2,0],0.0,"ds/ds has wrong value at (2,3,2,0).")
      self.assertEqual(dsds[2,3,2,1],0.0,"ds/ds has wrong value at (2,3,2,1).")
      self.assertEqual(dsds[2,3,2,2],0.0,"ds/ds has wrong value at (2,3,2,2).")
      self.assertEqual(dsds[2,3,2,3],1.0,"ds/ds has wrong value at (2,3,2,3).")
      self.assertEqual(dsds[2,3,2,4],0.0,"ds/ds has wrong value at (2,3,2,4).")
      self.assertEqual(dsds[2,3,3,0],0.0,"ds/ds has wrong value at (2,3,3,0).")
      self.assertEqual(dsds[2,3,3,1],0.0,"ds/ds has wrong value at (2,3,3,1).")
      self.assertEqual(dsds[2,3,3,2],0.0,"ds/ds has wrong value at (2,3,3,2).")
      self.assertEqual(dsds[2,3,3,3],0.0,"ds/ds has wrong value at (2,3,3,3).")
      self.assertEqual(dsds[2,3,3,4],0.0,"ds/ds has wrong value at (2,3,3,4).")
      self.assertEqual(dsds[2,4,0,0],0.0,"ds/ds has wrong value at (2,4,0,0).")
      self.assertEqual(dsds[2,4,0,1],0.0,"ds/ds has wrong value at (2,4,0,1).")
      self.assertEqual(dsds[2,4,0,2],0.0,"ds/ds has wrong value at (2,4,0,2).")
      self.assertEqual(dsds[2,4,0,3],0.0,"ds/ds has wrong value at (2,4,0,3).")
      self.assertEqual(dsds[2,4,0,4],0.0,"ds/ds has wrong value at (2,4,0,4).")
      self.assertEqual(dsds[2,4,1,0],0.0,"ds/ds has wrong value at (2,4,1,0).")
      self.assertEqual(dsds[2,4,1,1],0.0,"ds/ds has wrong value at (2,4,1,1).")
      self.assertEqual(dsds[2,4,1,2],0.0,"ds/ds has wrong value at (2,4,1,2).")
      self.assertEqual(dsds[2,4,1,3],0.0,"ds/ds has wrong value at (2,4,1,3).")
      self.assertEqual(dsds[2,4,1,4],0.0,"ds/ds has wrong value at (2,4,1,4).")
      self.assertEqual(dsds[2,4,2,0],0.0,"ds/ds has wrong value at (2,4,2,0).")
      self.assertEqual(dsds[2,4,2,1],0.0,"ds/ds has wrong value at (2,4,2,1).")
      self.assertEqual(dsds[2,4,2,2],0.0,"ds/ds has wrong value at (2,4,2,2).")
      self.assertEqual(dsds[2,4,2,3],0.0,"ds/ds has wrong value at (2,4,2,3).")
      self.assertEqual(dsds[2,4,2,4],1.0,"ds/ds has wrong value at (2,4,2,4).")
      self.assertEqual(dsds[2,4,3,0],0.0,"ds/ds has wrong value at (2,4,3,0).")
      self.assertEqual(dsds[2,4,3,1],0.0,"ds/ds has wrong value at (2,4,3,1).")
      self.assertEqual(dsds[2,4,3,2],0.0,"ds/ds has wrong value at (2,4,3,2).")
      self.assertEqual(dsds[2,4,3,3],0.0,"ds/ds has wrong value at (2,4,3,3).")
      self.assertEqual(dsds[2,4,3,4],0.0,"ds/ds has wrong value at (2,4,3,4).")
      self.assertEqual(dsds[3,0,0,0],0.0,"ds/ds has wrong value at (3,0,0,0).")
      self.assertEqual(dsds[3,0,0,1],0.0,"ds/ds has wrong value at (3,0,0,1).")
      self.assertEqual(dsds[3,0,0,2],0.0,"ds/ds has wrong value at (3,0,0,2).")
      self.assertEqual(dsds[3,0,0,3],0.0,"ds/ds has wrong value at (3,0,0,3).")
      self.assertEqual(dsds[3,0,0,4],0.0,"ds/ds has wrong value at (3,0,0,4).")
      self.assertEqual(dsds[3,0,1,0],0.0,"ds/ds has wrong value at (3,0,1,0).")
      self.assertEqual(dsds[3,0,1,1],0.0,"ds/ds has wrong value at (3,0,1,1).")
      self.assertEqual(dsds[3,0,1,2],0.0,"ds/ds has wrong value at (3,0,1,2).")
      self.assertEqual(dsds[3,0,1,3],0.0,"ds/ds has wrong value at (3,0,1,3).")
      self.assertEqual(dsds[3,0,1,4],0.0,"ds/ds has wrong value at (3,0,1,4).")
      self.assertEqual(dsds[3,0,2,0],0.0,"ds/ds has wrong value at (3,0,2,0).")
      self.assertEqual(dsds[3,0,2,1],0.0,"ds/ds has wrong value at (3,0,2,1).")
      self.assertEqual(dsds[3,0,2,2],0.0,"ds/ds has wrong value at (3,0,2,2).")
      self.assertEqual(dsds[3,0,2,3],0.0,"ds/ds has wrong value at (3,0,2,3).")
      self.assertEqual(dsds[3,0,2,4],0.0,"ds/ds has wrong value at (3,0,2,4).")
      self.assertEqual(dsds[3,0,3,0],1.0,"ds/ds has wrong value at (3,0,3,0).")
      self.assertEqual(dsds[3,0,3,1],0.0,"ds/ds has wrong value at (3,0,3,1).")
      self.assertEqual(dsds[3,0,3,2],0.0,"ds/ds has wrong value at (3,0,3,2).")
      self.assertEqual(dsds[3,0,3,3],0.0,"ds/ds has wrong value at (3,0,3,3).")
      self.assertEqual(dsds[3,0,3,4],0.0,"ds/ds has wrong value at (3,0,3,4).")
      self.assertEqual(dsds[3,1,0,0],0.0,"ds/ds has wrong value at (3,1,0,0).")
      self.assertEqual(dsds[3,1,0,1],0.0,"ds/ds has wrong value at (3,1,0,1).")
      self.assertEqual(dsds[3,1,0,2],0.0,"ds/ds has wrong value at (3,1,0,2).")
      self.assertEqual(dsds[3,1,0,3],0.0,"ds/ds has wrong value at (3,1,0,3).")
      self.assertEqual(dsds[3,1,0,4],0.0,"ds/ds has wrong value at (3,1,0,4).")
      self.assertEqual(dsds[3,1,1,0],0.0,"ds/ds has wrong value at (3,1,1,0).")
      self.assertEqual(dsds[3,1,1,1],0.0,"ds/ds has wrong value at (3,1,1,1).")
      self.assertEqual(dsds[3,1,1,2],0.0,"ds/ds has wrong value at (3,1,1,2).")
      self.assertEqual(dsds[3,1,1,3],0.0,"ds/ds has wrong value at (3,1,1,3).")
      self.assertEqual(dsds[3,1,1,4],0.0,"ds/ds has wrong value at (3,1,1,4).")
      self.assertEqual(dsds[3,1,2,0],0.0,"ds/ds has wrong value at (3,1,2,0).")
      self.assertEqual(dsds[3,1,2,1],0.0,"ds/ds has wrong value at (3,1,2,1).")
      self.assertEqual(dsds[3,1,2,2],0.0,"ds/ds has wrong value at (3,1,2,2).")
      self.assertEqual(dsds[3,1,2,3],0.0,"ds/ds has wrong value at (3,1,2,3).")
      self.assertEqual(dsds[3,1,2,4],0.0,"ds/ds has wrong value at (3,1,2,4).")
      self.assertEqual(dsds[3,1,3,0],0.0,"ds/ds has wrong value at (3,1,3,0).")
      self.assertEqual(dsds[3,1,3,1],1.0,"ds/ds has wrong value at (3,1,3,1).")
      self.assertEqual(dsds[3,1,3,2],0.0,"ds/ds has wrong value at (3,1,3,2).")
      self.assertEqual(dsds[3,1,3,3],0.0,"ds/ds has wrong value at (3,1,3,3).")
      self.assertEqual(dsds[3,1,3,4],0.0,"ds/ds has wrong value at (3,1,3,4).")
      self.assertEqual(dsds[3,2,0,0],0.0,"ds/ds has wrong value at (3,2,0,0).")
      self.assertEqual(dsds[3,2,0,1],0.0,"ds/ds has wrong value at (3,2,0,1).")
      self.assertEqual(dsds[3,2,0,2],0.0,"ds/ds has wrong value at (3,2,0,2).")
      self.assertEqual(dsds[3,2,0,3],0.0,"ds/ds has wrong value at (3,2,0,3).")
      self.assertEqual(dsds[3,2,0,4],0.0,"ds/ds has wrong value at (3,2,0,4).")
      self.assertEqual(dsds[3,2,1,0],0.0,"ds/ds has wrong value at (3,2,1,0).")
      self.assertEqual(dsds[3,2,1,1],0.0,"ds/ds has wrong value at (3,2,1,1).")
      self.assertEqual(dsds[3,2,1,2],0.0,"ds/ds has wrong value at (3,2,1,2).")
      self.assertEqual(dsds[3,2,1,3],0.0,"ds/ds has wrong value at (3,2,1,3).")
      self.assertEqual(dsds[3,2,1,4],0.0,"ds/ds has wrong value at (3,2,1,4).")
      self.assertEqual(dsds[3,2,2,0],0.0,"ds/ds has wrong value at (3,2,2,0).")
      self.assertEqual(dsds[3,2,2,1],0.0,"ds/ds has wrong value at (3,2,2,1).")
      self.assertEqual(dsds[3,2,2,2],0.0,"ds/ds has wrong value at (3,2,2,2).")
      self.assertEqual(dsds[3,2,2,3],0.0,"ds/ds has wrong value at (3,2,2,3).")
      self.assertEqual(dsds[3,2,2,4],0.0,"ds/ds has wrong value at (3,2,2,4).")
      self.assertEqual(dsds[3,2,3,0],0.0,"ds/ds has wrong value at (3,2,3,0).")
      self.assertEqual(dsds[3,2,3,1],0.0,"ds/ds has wrong value at (3,2,3,1).")
      self.assertEqual(dsds[3,2,3,2],1.0,"ds/ds has wrong value at (3,2,3,2).")
      self.assertEqual(dsds[3,2,3,3],0.0,"ds/ds has wrong value at (3,2,3,3).")
      self.assertEqual(dsds[3,2,3,4],0.0,"ds/ds has wrong value at (3,2,3,4).")
      self.assertEqual(dsds[3,3,0,0],0.0,"ds/ds has wrong value at (3,3,0,0).")
      self.assertEqual(dsds[3,3,0,1],0.0,"ds/ds has wrong value at (3,3,0,1).")
      self.assertEqual(dsds[3,3,0,2],0.0,"ds/ds has wrong value at (3,3,0,2).")
      self.assertEqual(dsds[3,3,0,3],0.0,"ds/ds has wrong value at (3,3,0,3).")
      self.assertEqual(dsds[3,3,0,4],0.0,"ds/ds has wrong value at (3,3,0,4).")
      self.assertEqual(dsds[3,3,1,0],0.0,"ds/ds has wrong value at (3,3,1,0).")
      self.assertEqual(dsds[3,3,1,1],0.0,"ds/ds has wrong value at (3,3,1,1).")
      self.assertEqual(dsds[3,3,1,2],0.0,"ds/ds has wrong value at (3,3,1,2).")
      self.assertEqual(dsds[3,3,1,3],0.0,"ds/ds has wrong value at (3,3,1,3).")
      self.assertEqual(dsds[3,3,1,4],0.0,"ds/ds has wrong value at (3,3,1,4).")
      self.assertEqual(dsds[3,3,2,0],0.0,"ds/ds has wrong value at (3,3,2,0).")
      self.assertEqual(dsds[3,3,2,1],0.0,"ds/ds has wrong value at (3,3,2,1).")
      self.assertEqual(dsds[3,3,2,2],0.0,"ds/ds has wrong value at (3,3,2,2).")
      self.assertEqual(dsds[3,3,2,3],0.0,"ds/ds has wrong value at (3,3,2,3).")
      self.assertEqual(dsds[3,3,2,4],0.0,"ds/ds has wrong value at (3,3,2,4).")
      self.assertEqual(dsds[3,3,3,0],0.0,"ds/ds has wrong value at (3,3,3,0).")
      self.assertEqual(dsds[3,3,3,1],0.0,"ds/ds has wrong value at (3,3,3,1).")
      self.assertEqual(dsds[3,3,3,2],0.0,"ds/ds has wrong value at (3,3,3,2).")
      self.assertEqual(dsds[3,3,3,3],1.0,"ds/ds has wrong value at (3,3,3,3).")
      self.assertEqual(dsds[3,3,3,4],0.0,"ds/ds has wrong value at (3,3,3,4).")
      self.assertEqual(dsds[3,4,0,0],0.0,"ds/ds has wrong value at (3,4,0,0).")
      self.assertEqual(dsds[3,4,0,1],0.0,"ds/ds has wrong value at (3,4,0,1).")
      self.assertEqual(dsds[3,4,0,2],0.0,"ds/ds has wrong value at (3,4,0,2).")
      self.assertEqual(dsds[3,4,0,3],0.0,"ds/ds has wrong value at (3,4,0,3).")
      self.assertEqual(dsds[3,4,0,4],0.0,"ds/ds has wrong value at (3,4,0,4).")
      self.assertEqual(dsds[3,4,1,0],0.0,"ds/ds has wrong value at (3,4,1,0).")
      self.assertEqual(dsds[3,4,1,1],0.0,"ds/ds has wrong value at (3,4,1,1).")
      self.assertEqual(dsds[3,4,1,2],0.0,"ds/ds has wrong value at (3,4,1,2).")
      self.assertEqual(dsds[3,4,1,3],0.0,"ds/ds has wrong value at (3,4,1,3).")
      self.assertEqual(dsds[3,4,1,4],0.0,"ds/ds has wrong value at (3,4,1,4).")
      self.assertEqual(dsds[3,4,2,0],0.0,"ds/ds has wrong value at (3,4,2,0).")
      self.assertEqual(dsds[3,4,2,1],0.0,"ds/ds has wrong value at (3,4,2,1).")
      self.assertEqual(dsds[3,4,2,2],0.0,"ds/ds has wrong value at (3,4,2,2).")
      self.assertEqual(dsds[3,4,2,3],0.0,"ds/ds has wrong value at (3,4,2,3).")
      self.assertEqual(dsds[3,4,2,4],0.0,"ds/ds has wrong value at (3,4,2,4).")
      self.assertEqual(dsds[3,4,3,0],0.0,"ds/ds has wrong value at (3,4,3,0).")
      self.assertEqual(dsds[3,4,3,1],0.0,"ds/ds has wrong value at (3,4,3,1).")
      self.assertEqual(dsds[3,4,3,2],0.0,"ds/ds has wrong value at (3,4,3,2).")
      self.assertEqual(dsds[3,4,3,3],0.0,"ds/ds has wrong value at (3,4,3,3).")
      self.assertEqual(dsds[3,4,3,4],1.0,"ds/ds has wrong value at (3,4,3,4).")

      sa=s.getSubstitutedArguments({s2:-10})
      self.assertEqual(len(sa),0,"wrong number of substituted arguments")

      self.assertFalse(s.isAppropriateValue(10.)," 10. is not appropriate substitute")
      self.assertFalse(s.isAppropriateValue(10)," 10 is not appropriate substitute")
      self.assertTrue(s.isAppropriateValue(Symbol(shape=(4, 5),dim=None)),"Symbol is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(Symbol(shape=(5,))),"Symbol is not appropriate substitute (shape)")
      self.assertFalse(s.isAppropriateValue(escript.Data(0.,(5,),self.functionspace)),"Data is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(numpy.zeros((4, 5))),"ndarray is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(numpy.zeros((5,))),"ndarray is not appropriate substitute (shape)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_Symbol_rank2_dNone_nargs1(self):
      s2=Symbol(shape=())
      s=Symbol(shape=(4, 5),dim=None,args=[s2])
      self.assertEqual(s.getRank(),2,"wrong rank.")
      self.assertEqual(s.getShape(),(4, 5),"wrong shape.")
      self.assertEqual(s.getDim(),None,"wrong spatial dimension.")
      self.assertEqual(s.getArgument(),[s2],"wrong arguments.")

      ss=s.substitute({s:numpy.zeros((4, 5))})
      self.assertTrue(isinstance(ss,numpy.ndarray),"value after substitution is not ndarray.")
      self.assertEqual(ss.shape,(4, 5),"value after substitution has not expected shape")
      try:
        s.substitute({s:numpy.zeros((5,))})
        fail("illegal substition was successful")
      except TypeError:
        pass

      dsdarg=s.diff(Symbol(shape=()))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds() has wrong type.")
      self.assertEqual(dsdarg.shape,(4, 5),"ds/ds() has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds() has wrong value.")
      dsdarg=s.diff(Symbol(shape=(2,)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(2,) has wrong type.")
      self.assertEqual(dsdarg.shape,(4, 5, 2),"ds/ds(2,) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(2,) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(4, 5)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(4, 5) has wrong type.")
      self.assertEqual(dsdarg.shape,(4, 5, 4, 5),"ds/ds(4, 5) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(4, 5) has wrong value.")

      dsds=s.diff(s)
      self.assertTrue(isinstance(dsds,numpy.ndarray),"ds/ds has wrong type.")
      self.assertEqual(dsds.shape,(4, 5, 4, 5),"ds/ds has wrong shape.")
      self.assertEqual(dsds[0,0,0,0],1.0,"ds/ds has wrong value at (0,0,0,0).")
      self.assertEqual(dsds[0,0,0,1],0.0,"ds/ds has wrong value at (0,0,0,1).")
      self.assertEqual(dsds[0,0,0,2],0.0,"ds/ds has wrong value at (0,0,0,2).")
      self.assertEqual(dsds[0,0,0,3],0.0,"ds/ds has wrong value at (0,0,0,3).")
      self.assertEqual(dsds[0,0,0,4],0.0,"ds/ds has wrong value at (0,0,0,4).")
      self.assertEqual(dsds[0,0,1,0],0.0,"ds/ds has wrong value at (0,0,1,0).")
      self.assertEqual(dsds[0,0,1,1],0.0,"ds/ds has wrong value at (0,0,1,1).")
      self.assertEqual(dsds[0,0,1,2],0.0,"ds/ds has wrong value at (0,0,1,2).")
      self.assertEqual(dsds[0,0,1,3],0.0,"ds/ds has wrong value at (0,0,1,3).")
      self.assertEqual(dsds[0,0,1,4],0.0,"ds/ds has wrong value at (0,0,1,4).")
      self.assertEqual(dsds[0,0,2,0],0.0,"ds/ds has wrong value at (0,0,2,0).")
      self.assertEqual(dsds[0,0,2,1],0.0,"ds/ds has wrong value at (0,0,2,1).")
      self.assertEqual(dsds[0,0,2,2],0.0,"ds/ds has wrong value at (0,0,2,2).")
      self.assertEqual(dsds[0,0,2,3],0.0,"ds/ds has wrong value at (0,0,2,3).")
      self.assertEqual(dsds[0,0,2,4],0.0,"ds/ds has wrong value at (0,0,2,4).")
      self.assertEqual(dsds[0,0,3,0],0.0,"ds/ds has wrong value at (0,0,3,0).")
      self.assertEqual(dsds[0,0,3,1],0.0,"ds/ds has wrong value at (0,0,3,1).")
      self.assertEqual(dsds[0,0,3,2],0.0,"ds/ds has wrong value at (0,0,3,2).")
      self.assertEqual(dsds[0,0,3,3],0.0,"ds/ds has wrong value at (0,0,3,3).")
      self.assertEqual(dsds[0,0,3,4],0.0,"ds/ds has wrong value at (0,0,3,4).")
      self.assertEqual(dsds[0,1,0,0],0.0,"ds/ds has wrong value at (0,1,0,0).")
      self.assertEqual(dsds[0,1,0,1],1.0,"ds/ds has wrong value at (0,1,0,1).")
      self.assertEqual(dsds[0,1,0,2],0.0,"ds/ds has wrong value at (0,1,0,2).")
      self.assertEqual(dsds[0,1,0,3],0.0,"ds/ds has wrong value at (0,1,0,3).")
      self.assertEqual(dsds[0,1,0,4],0.0,"ds/ds has wrong value at (0,1,0,4).")
      self.assertEqual(dsds[0,1,1,0],0.0,"ds/ds has wrong value at (0,1,1,0).")
      self.assertEqual(dsds[0,1,1,1],0.0,"ds/ds has wrong value at (0,1,1,1).")
      self.assertEqual(dsds[0,1,1,2],0.0,"ds/ds has wrong value at (0,1,1,2).")
      self.assertEqual(dsds[0,1,1,3],0.0,"ds/ds has wrong value at (0,1,1,3).")
      self.assertEqual(dsds[0,1,1,4],0.0,"ds/ds has wrong value at (0,1,1,4).")
      self.assertEqual(dsds[0,1,2,0],0.0,"ds/ds has wrong value at (0,1,2,0).")
      self.assertEqual(dsds[0,1,2,1],0.0,"ds/ds has wrong value at (0,1,2,1).")
      self.assertEqual(dsds[0,1,2,2],0.0,"ds/ds has wrong value at (0,1,2,2).")
      self.assertEqual(dsds[0,1,2,3],0.0,"ds/ds has wrong value at (0,1,2,3).")
      self.assertEqual(dsds[0,1,2,4],0.0,"ds/ds has wrong value at (0,1,2,4).")
      self.assertEqual(dsds[0,1,3,0],0.0,"ds/ds has wrong value at (0,1,3,0).")
      self.assertEqual(dsds[0,1,3,1],0.0,"ds/ds has wrong value at (0,1,3,1).")
      self.assertEqual(dsds[0,1,3,2],0.0,"ds/ds has wrong value at (0,1,3,2).")
      self.assertEqual(dsds[0,1,3,3],0.0,"ds/ds has wrong value at (0,1,3,3).")
      self.assertEqual(dsds[0,1,3,4],0.0,"ds/ds has wrong value at (0,1,3,4).")
      self.assertEqual(dsds[0,2,0,0],0.0,"ds/ds has wrong value at (0,2,0,0).")
      self.assertEqual(dsds[0,2,0,1],0.0,"ds/ds has wrong value at (0,2,0,1).")
      self.assertEqual(dsds[0,2,0,2],1.0,"ds/ds has wrong value at (0,2,0,2).")
      self.assertEqual(dsds[0,2,0,3],0.0,"ds/ds has wrong value at (0,2,0,3).")
      self.assertEqual(dsds[0,2,0,4],0.0,"ds/ds has wrong value at (0,2,0,4).")
      self.assertEqual(dsds[0,2,1,0],0.0,"ds/ds has wrong value at (0,2,1,0).")
      self.assertEqual(dsds[0,2,1,1],0.0,"ds/ds has wrong value at (0,2,1,1).")
      self.assertEqual(dsds[0,2,1,2],0.0,"ds/ds has wrong value at (0,2,1,2).")
      self.assertEqual(dsds[0,2,1,3],0.0,"ds/ds has wrong value at (0,2,1,3).")
      self.assertEqual(dsds[0,2,1,4],0.0,"ds/ds has wrong value at (0,2,1,4).")
      self.assertEqual(dsds[0,2,2,0],0.0,"ds/ds has wrong value at (0,2,2,0).")
      self.assertEqual(dsds[0,2,2,1],0.0,"ds/ds has wrong value at (0,2,2,1).")
      self.assertEqual(dsds[0,2,2,2],0.0,"ds/ds has wrong value at (0,2,2,2).")
      self.assertEqual(dsds[0,2,2,3],0.0,"ds/ds has wrong value at (0,2,2,3).")
      self.assertEqual(dsds[0,2,2,4],0.0,"ds/ds has wrong value at (0,2,2,4).")
      self.assertEqual(dsds[0,2,3,0],0.0,"ds/ds has wrong value at (0,2,3,0).")
      self.assertEqual(dsds[0,2,3,1],0.0,"ds/ds has wrong value at (0,2,3,1).")
      self.assertEqual(dsds[0,2,3,2],0.0,"ds/ds has wrong value at (0,2,3,2).")
      self.assertEqual(dsds[0,2,3,3],0.0,"ds/ds has wrong value at (0,2,3,3).")
      self.assertEqual(dsds[0,2,3,4],0.0,"ds/ds has wrong value at (0,2,3,4).")
      self.assertEqual(dsds[0,3,0,0],0.0,"ds/ds has wrong value at (0,3,0,0).")
      self.assertEqual(dsds[0,3,0,1],0.0,"ds/ds has wrong value at (0,3,0,1).")
      self.assertEqual(dsds[0,3,0,2],0.0,"ds/ds has wrong value at (0,3,0,2).")
      self.assertEqual(dsds[0,3,0,3],1.0,"ds/ds has wrong value at (0,3,0,3).")
      self.assertEqual(dsds[0,3,0,4],0.0,"ds/ds has wrong value at (0,3,0,4).")
      self.assertEqual(dsds[0,3,1,0],0.0,"ds/ds has wrong value at (0,3,1,0).")
      self.assertEqual(dsds[0,3,1,1],0.0,"ds/ds has wrong value at (0,3,1,1).")
      self.assertEqual(dsds[0,3,1,2],0.0,"ds/ds has wrong value at (0,3,1,2).")
      self.assertEqual(dsds[0,3,1,3],0.0,"ds/ds has wrong value at (0,3,1,3).")
      self.assertEqual(dsds[0,3,1,4],0.0,"ds/ds has wrong value at (0,3,1,4).")
      self.assertEqual(dsds[0,3,2,0],0.0,"ds/ds has wrong value at (0,3,2,0).")
      self.assertEqual(dsds[0,3,2,1],0.0,"ds/ds has wrong value at (0,3,2,1).")
      self.assertEqual(dsds[0,3,2,2],0.0,"ds/ds has wrong value at (0,3,2,2).")
      self.assertEqual(dsds[0,3,2,3],0.0,"ds/ds has wrong value at (0,3,2,3).")
      self.assertEqual(dsds[0,3,2,4],0.0,"ds/ds has wrong value at (0,3,2,4).")
      self.assertEqual(dsds[0,3,3,0],0.0,"ds/ds has wrong value at (0,3,3,0).")
      self.assertEqual(dsds[0,3,3,1],0.0,"ds/ds has wrong value at (0,3,3,1).")
      self.assertEqual(dsds[0,3,3,2],0.0,"ds/ds has wrong value at (0,3,3,2).")
      self.assertEqual(dsds[0,3,3,3],0.0,"ds/ds has wrong value at (0,3,3,3).")
      self.assertEqual(dsds[0,3,3,4],0.0,"ds/ds has wrong value at (0,3,3,4).")
      self.assertEqual(dsds[0,4,0,0],0.0,"ds/ds has wrong value at (0,4,0,0).")
      self.assertEqual(dsds[0,4,0,1],0.0,"ds/ds has wrong value at (0,4,0,1).")
      self.assertEqual(dsds[0,4,0,2],0.0,"ds/ds has wrong value at (0,4,0,2).")
      self.assertEqual(dsds[0,4,0,3],0.0,"ds/ds has wrong value at (0,4,0,3).")
      self.assertEqual(dsds[0,4,0,4],1.0,"ds/ds has wrong value at (0,4,0,4).")
      self.assertEqual(dsds[0,4,1,0],0.0,"ds/ds has wrong value at (0,4,1,0).")
      self.assertEqual(dsds[0,4,1,1],0.0,"ds/ds has wrong value at (0,4,1,1).")
      self.assertEqual(dsds[0,4,1,2],0.0,"ds/ds has wrong value at (0,4,1,2).")
      self.assertEqual(dsds[0,4,1,3],0.0,"ds/ds has wrong value at (0,4,1,3).")
      self.assertEqual(dsds[0,4,1,4],0.0,"ds/ds has wrong value at (0,4,1,4).")
      self.assertEqual(dsds[0,4,2,0],0.0,"ds/ds has wrong value at (0,4,2,0).")
      self.assertEqual(dsds[0,4,2,1],0.0,"ds/ds has wrong value at (0,4,2,1).")
      self.assertEqual(dsds[0,4,2,2],0.0,"ds/ds has wrong value at (0,4,2,2).")
      self.assertEqual(dsds[0,4,2,3],0.0,"ds/ds has wrong value at (0,4,2,3).")
      self.assertEqual(dsds[0,4,2,4],0.0,"ds/ds has wrong value at (0,4,2,4).")
      self.assertEqual(dsds[0,4,3,0],0.0,"ds/ds has wrong value at (0,4,3,0).")
      self.assertEqual(dsds[0,4,3,1],0.0,"ds/ds has wrong value at (0,4,3,1).")
      self.assertEqual(dsds[0,4,3,2],0.0,"ds/ds has wrong value at (0,4,3,2).")
      self.assertEqual(dsds[0,4,3,3],0.0,"ds/ds has wrong value at (0,4,3,3).")
      self.assertEqual(dsds[0,4,3,4],0.0,"ds/ds has wrong value at (0,4,3,4).")
      self.assertEqual(dsds[1,0,0,0],0.0,"ds/ds has wrong value at (1,0,0,0).")
      self.assertEqual(dsds[1,0,0,1],0.0,"ds/ds has wrong value at (1,0,0,1).")
      self.assertEqual(dsds[1,0,0,2],0.0,"ds/ds has wrong value at (1,0,0,2).")
      self.assertEqual(dsds[1,0,0,3],0.0,"ds/ds has wrong value at (1,0,0,3).")
      self.assertEqual(dsds[1,0,0,4],0.0,"ds/ds has wrong value at (1,0,0,4).")
      self.assertEqual(dsds[1,0,1,0],1.0,"ds/ds has wrong value at (1,0,1,0).")
      self.assertEqual(dsds[1,0,1,1],0.0,"ds/ds has wrong value at (1,0,1,1).")
      self.assertEqual(dsds[1,0,1,2],0.0,"ds/ds has wrong value at (1,0,1,2).")
      self.assertEqual(dsds[1,0,1,3],0.0,"ds/ds has wrong value at (1,0,1,3).")
      self.assertEqual(dsds[1,0,1,4],0.0,"ds/ds has wrong value at (1,0,1,4).")
      self.assertEqual(dsds[1,0,2,0],0.0,"ds/ds has wrong value at (1,0,2,0).")
      self.assertEqual(dsds[1,0,2,1],0.0,"ds/ds has wrong value at (1,0,2,1).")
      self.assertEqual(dsds[1,0,2,2],0.0,"ds/ds has wrong value at (1,0,2,2).")
      self.assertEqual(dsds[1,0,2,3],0.0,"ds/ds has wrong value at (1,0,2,3).")
      self.assertEqual(dsds[1,0,2,4],0.0,"ds/ds has wrong value at (1,0,2,4).")
      self.assertEqual(dsds[1,0,3,0],0.0,"ds/ds has wrong value at (1,0,3,0).")
      self.assertEqual(dsds[1,0,3,1],0.0,"ds/ds has wrong value at (1,0,3,1).")
      self.assertEqual(dsds[1,0,3,2],0.0,"ds/ds has wrong value at (1,0,3,2).")
      self.assertEqual(dsds[1,0,3,3],0.0,"ds/ds has wrong value at (1,0,3,3).")
      self.assertEqual(dsds[1,0,3,4],0.0,"ds/ds has wrong value at (1,0,3,4).")
      self.assertEqual(dsds[1,1,0,0],0.0,"ds/ds has wrong value at (1,1,0,0).")
      self.assertEqual(dsds[1,1,0,1],0.0,"ds/ds has wrong value at (1,1,0,1).")
      self.assertEqual(dsds[1,1,0,2],0.0,"ds/ds has wrong value at (1,1,0,2).")
      self.assertEqual(dsds[1,1,0,3],0.0,"ds/ds has wrong value at (1,1,0,3).")
      self.assertEqual(dsds[1,1,0,4],0.0,"ds/ds has wrong value at (1,1,0,4).")
      self.assertEqual(dsds[1,1,1,0],0.0,"ds/ds has wrong value at (1,1,1,0).")
      self.assertEqual(dsds[1,1,1,1],1.0,"ds/ds has wrong value at (1,1,1,1).")
      self.assertEqual(dsds[1,1,1,2],0.0,"ds/ds has wrong value at (1,1,1,2).")
      self.assertEqual(dsds[1,1,1,3],0.0,"ds/ds has wrong value at (1,1,1,3).")
      self.assertEqual(dsds[1,1,1,4],0.0,"ds/ds has wrong value at (1,1,1,4).")
      self.assertEqual(dsds[1,1,2,0],0.0,"ds/ds has wrong value at (1,1,2,0).")
      self.assertEqual(dsds[1,1,2,1],0.0,"ds/ds has wrong value at (1,1,2,1).")
      self.assertEqual(dsds[1,1,2,2],0.0,"ds/ds has wrong value at (1,1,2,2).")
      self.assertEqual(dsds[1,1,2,3],0.0,"ds/ds has wrong value at (1,1,2,3).")
      self.assertEqual(dsds[1,1,2,4],0.0,"ds/ds has wrong value at (1,1,2,4).")
      self.assertEqual(dsds[1,1,3,0],0.0,"ds/ds has wrong value at (1,1,3,0).")
      self.assertEqual(dsds[1,1,3,1],0.0,"ds/ds has wrong value at (1,1,3,1).")
      self.assertEqual(dsds[1,1,3,2],0.0,"ds/ds has wrong value at (1,1,3,2).")
      self.assertEqual(dsds[1,1,3,3],0.0,"ds/ds has wrong value at (1,1,3,3).")
      self.assertEqual(dsds[1,1,3,4],0.0,"ds/ds has wrong value at (1,1,3,4).")
      self.assertEqual(dsds[1,2,0,0],0.0,"ds/ds has wrong value at (1,2,0,0).")
      self.assertEqual(dsds[1,2,0,1],0.0,"ds/ds has wrong value at (1,2,0,1).")
      self.assertEqual(dsds[1,2,0,2],0.0,"ds/ds has wrong value at (1,2,0,2).")
      self.assertEqual(dsds[1,2,0,3],0.0,"ds/ds has wrong value at (1,2,0,3).")
      self.assertEqual(dsds[1,2,0,4],0.0,"ds/ds has wrong value at (1,2,0,4).")
      self.assertEqual(dsds[1,2,1,0],0.0,"ds/ds has wrong value at (1,2,1,0).")
      self.assertEqual(dsds[1,2,1,1],0.0,"ds/ds has wrong value at (1,2,1,1).")
      self.assertEqual(dsds[1,2,1,2],1.0,"ds/ds has wrong value at (1,2,1,2).")
      self.assertEqual(dsds[1,2,1,3],0.0,"ds/ds has wrong value at (1,2,1,3).")
      self.assertEqual(dsds[1,2,1,4],0.0,"ds/ds has wrong value at (1,2,1,4).")
      self.assertEqual(dsds[1,2,2,0],0.0,"ds/ds has wrong value at (1,2,2,0).")
      self.assertEqual(dsds[1,2,2,1],0.0,"ds/ds has wrong value at (1,2,2,1).")
      self.assertEqual(dsds[1,2,2,2],0.0,"ds/ds has wrong value at (1,2,2,2).")
      self.assertEqual(dsds[1,2,2,3],0.0,"ds/ds has wrong value at (1,2,2,3).")
      self.assertEqual(dsds[1,2,2,4],0.0,"ds/ds has wrong value at (1,2,2,4).")
      self.assertEqual(dsds[1,2,3,0],0.0,"ds/ds has wrong value at (1,2,3,0).")
      self.assertEqual(dsds[1,2,3,1],0.0,"ds/ds has wrong value at (1,2,3,1).")
      self.assertEqual(dsds[1,2,3,2],0.0,"ds/ds has wrong value at (1,2,3,2).")
      self.assertEqual(dsds[1,2,3,3],0.0,"ds/ds has wrong value at (1,2,3,3).")
      self.assertEqual(dsds[1,2,3,4],0.0,"ds/ds has wrong value at (1,2,3,4).")
      self.assertEqual(dsds[1,3,0,0],0.0,"ds/ds has wrong value at (1,3,0,0).")
      self.assertEqual(dsds[1,3,0,1],0.0,"ds/ds has wrong value at (1,3,0,1).")
      self.assertEqual(dsds[1,3,0,2],0.0,"ds/ds has wrong value at (1,3,0,2).")
      self.assertEqual(dsds[1,3,0,3],0.0,"ds/ds has wrong value at (1,3,0,3).")
      self.assertEqual(dsds[1,3,0,4],0.0,"ds/ds has wrong value at (1,3,0,4).")
      self.assertEqual(dsds[1,3,1,0],0.0,"ds/ds has wrong value at (1,3,1,0).")
      self.assertEqual(dsds[1,3,1,1],0.0,"ds/ds has wrong value at (1,3,1,1).")
      self.assertEqual(dsds[1,3,1,2],0.0,"ds/ds has wrong value at (1,3,1,2).")
      self.assertEqual(dsds[1,3,1,3],1.0,"ds/ds has wrong value at (1,3,1,3).")
      self.assertEqual(dsds[1,3,1,4],0.0,"ds/ds has wrong value at (1,3,1,4).")
      self.assertEqual(dsds[1,3,2,0],0.0,"ds/ds has wrong value at (1,3,2,0).")
      self.assertEqual(dsds[1,3,2,1],0.0,"ds/ds has wrong value at (1,3,2,1).")
      self.assertEqual(dsds[1,3,2,2],0.0,"ds/ds has wrong value at (1,3,2,2).")
      self.assertEqual(dsds[1,3,2,3],0.0,"ds/ds has wrong value at (1,3,2,3).")
      self.assertEqual(dsds[1,3,2,4],0.0,"ds/ds has wrong value at (1,3,2,4).")
      self.assertEqual(dsds[1,3,3,0],0.0,"ds/ds has wrong value at (1,3,3,0).")
      self.assertEqual(dsds[1,3,3,1],0.0,"ds/ds has wrong value at (1,3,3,1).")
      self.assertEqual(dsds[1,3,3,2],0.0,"ds/ds has wrong value at (1,3,3,2).")
      self.assertEqual(dsds[1,3,3,3],0.0,"ds/ds has wrong value at (1,3,3,3).")
      self.assertEqual(dsds[1,3,3,4],0.0,"ds/ds has wrong value at (1,3,3,4).")
      self.assertEqual(dsds[1,4,0,0],0.0,"ds/ds has wrong value at (1,4,0,0).")
      self.assertEqual(dsds[1,4,0,1],0.0,"ds/ds has wrong value at (1,4,0,1).")
      self.assertEqual(dsds[1,4,0,2],0.0,"ds/ds has wrong value at (1,4,0,2).")
      self.assertEqual(dsds[1,4,0,3],0.0,"ds/ds has wrong value at (1,4,0,3).")
      self.assertEqual(dsds[1,4,0,4],0.0,"ds/ds has wrong value at (1,4,0,4).")
      self.assertEqual(dsds[1,4,1,0],0.0,"ds/ds has wrong value at (1,4,1,0).")
      self.assertEqual(dsds[1,4,1,1],0.0,"ds/ds has wrong value at (1,4,1,1).")
      self.assertEqual(dsds[1,4,1,2],0.0,"ds/ds has wrong value at (1,4,1,2).")
      self.assertEqual(dsds[1,4,1,3],0.0,"ds/ds has wrong value at (1,4,1,3).")
      self.assertEqual(dsds[1,4,1,4],1.0,"ds/ds has wrong value at (1,4,1,4).")
      self.assertEqual(dsds[1,4,2,0],0.0,"ds/ds has wrong value at (1,4,2,0).")
      self.assertEqual(dsds[1,4,2,1],0.0,"ds/ds has wrong value at (1,4,2,1).")
      self.assertEqual(dsds[1,4,2,2],0.0,"ds/ds has wrong value at (1,4,2,2).")
      self.assertEqual(dsds[1,4,2,3],0.0,"ds/ds has wrong value at (1,4,2,3).")
      self.assertEqual(dsds[1,4,2,4],0.0,"ds/ds has wrong value at (1,4,2,4).")
      self.assertEqual(dsds[1,4,3,0],0.0,"ds/ds has wrong value at (1,4,3,0).")
      self.assertEqual(dsds[1,4,3,1],0.0,"ds/ds has wrong value at (1,4,3,1).")
      self.assertEqual(dsds[1,4,3,2],0.0,"ds/ds has wrong value at (1,4,3,2).")
      self.assertEqual(dsds[1,4,3,3],0.0,"ds/ds has wrong value at (1,4,3,3).")
      self.assertEqual(dsds[1,4,3,4],0.0,"ds/ds has wrong value at (1,4,3,4).")
      self.assertEqual(dsds[2,0,0,0],0.0,"ds/ds has wrong value at (2,0,0,0).")
      self.assertEqual(dsds[2,0,0,1],0.0,"ds/ds has wrong value at (2,0,0,1).")
      self.assertEqual(dsds[2,0,0,2],0.0,"ds/ds has wrong value at (2,0,0,2).")
      self.assertEqual(dsds[2,0,0,3],0.0,"ds/ds has wrong value at (2,0,0,3).")
      self.assertEqual(dsds[2,0,0,4],0.0,"ds/ds has wrong value at (2,0,0,4).")
      self.assertEqual(dsds[2,0,1,0],0.0,"ds/ds has wrong value at (2,0,1,0).")
      self.assertEqual(dsds[2,0,1,1],0.0,"ds/ds has wrong value at (2,0,1,1).")
      self.assertEqual(dsds[2,0,1,2],0.0,"ds/ds has wrong value at (2,0,1,2).")
      self.assertEqual(dsds[2,0,1,3],0.0,"ds/ds has wrong value at (2,0,1,3).")
      self.assertEqual(dsds[2,0,1,4],0.0,"ds/ds has wrong value at (2,0,1,4).")
      self.assertEqual(dsds[2,0,2,0],1.0,"ds/ds has wrong value at (2,0,2,0).")
      self.assertEqual(dsds[2,0,2,1],0.0,"ds/ds has wrong value at (2,0,2,1).")
      self.assertEqual(dsds[2,0,2,2],0.0,"ds/ds has wrong value at (2,0,2,2).")
      self.assertEqual(dsds[2,0,2,3],0.0,"ds/ds has wrong value at (2,0,2,3).")
      self.assertEqual(dsds[2,0,2,4],0.0,"ds/ds has wrong value at (2,0,2,4).")
      self.assertEqual(dsds[2,0,3,0],0.0,"ds/ds has wrong value at (2,0,3,0).")
      self.assertEqual(dsds[2,0,3,1],0.0,"ds/ds has wrong value at (2,0,3,1).")
      self.assertEqual(dsds[2,0,3,2],0.0,"ds/ds has wrong value at (2,0,3,2).")
      self.assertEqual(dsds[2,0,3,3],0.0,"ds/ds has wrong value at (2,0,3,3).")
      self.assertEqual(dsds[2,0,3,4],0.0,"ds/ds has wrong value at (2,0,3,4).")
      self.assertEqual(dsds[2,1,0,0],0.0,"ds/ds has wrong value at (2,1,0,0).")
      self.assertEqual(dsds[2,1,0,1],0.0,"ds/ds has wrong value at (2,1,0,1).")
      self.assertEqual(dsds[2,1,0,2],0.0,"ds/ds has wrong value at (2,1,0,2).")
      self.assertEqual(dsds[2,1,0,3],0.0,"ds/ds has wrong value at (2,1,0,3).")
      self.assertEqual(dsds[2,1,0,4],0.0,"ds/ds has wrong value at (2,1,0,4).")
      self.assertEqual(dsds[2,1,1,0],0.0,"ds/ds has wrong value at (2,1,1,0).")
      self.assertEqual(dsds[2,1,1,1],0.0,"ds/ds has wrong value at (2,1,1,1).")
      self.assertEqual(dsds[2,1,1,2],0.0,"ds/ds has wrong value at (2,1,1,2).")
      self.assertEqual(dsds[2,1,1,3],0.0,"ds/ds has wrong value at (2,1,1,3).")
      self.assertEqual(dsds[2,1,1,4],0.0,"ds/ds has wrong value at (2,1,1,4).")
      self.assertEqual(dsds[2,1,2,0],0.0,"ds/ds has wrong value at (2,1,2,0).")
      self.assertEqual(dsds[2,1,2,1],1.0,"ds/ds has wrong value at (2,1,2,1).")
      self.assertEqual(dsds[2,1,2,2],0.0,"ds/ds has wrong value at (2,1,2,2).")
      self.assertEqual(dsds[2,1,2,3],0.0,"ds/ds has wrong value at (2,1,2,3).")
      self.assertEqual(dsds[2,1,2,4],0.0,"ds/ds has wrong value at (2,1,2,4).")
      self.assertEqual(dsds[2,1,3,0],0.0,"ds/ds has wrong value at (2,1,3,0).")
      self.assertEqual(dsds[2,1,3,1],0.0,"ds/ds has wrong value at (2,1,3,1).")
      self.assertEqual(dsds[2,1,3,2],0.0,"ds/ds has wrong value at (2,1,3,2).")
      self.assertEqual(dsds[2,1,3,3],0.0,"ds/ds has wrong value at (2,1,3,3).")
      self.assertEqual(dsds[2,1,3,4],0.0,"ds/ds has wrong value at (2,1,3,4).")
      self.assertEqual(dsds[2,2,0,0],0.0,"ds/ds has wrong value at (2,2,0,0).")
      self.assertEqual(dsds[2,2,0,1],0.0,"ds/ds has wrong value at (2,2,0,1).")
      self.assertEqual(dsds[2,2,0,2],0.0,"ds/ds has wrong value at (2,2,0,2).")
      self.assertEqual(dsds[2,2,0,3],0.0,"ds/ds has wrong value at (2,2,0,3).")
      self.assertEqual(dsds[2,2,0,4],0.0,"ds/ds has wrong value at (2,2,0,4).")
      self.assertEqual(dsds[2,2,1,0],0.0,"ds/ds has wrong value at (2,2,1,0).")
      self.assertEqual(dsds[2,2,1,1],0.0,"ds/ds has wrong value at (2,2,1,1).")
      self.assertEqual(dsds[2,2,1,2],0.0,"ds/ds has wrong value at (2,2,1,2).")
      self.assertEqual(dsds[2,2,1,3],0.0,"ds/ds has wrong value at (2,2,1,3).")
      self.assertEqual(dsds[2,2,1,4],0.0,"ds/ds has wrong value at (2,2,1,4).")
      self.assertEqual(dsds[2,2,2,0],0.0,"ds/ds has wrong value at (2,2,2,0).")
      self.assertEqual(dsds[2,2,2,1],0.0,"ds/ds has wrong value at (2,2,2,1).")
      self.assertEqual(dsds[2,2,2,2],1.0,"ds/ds has wrong value at (2,2,2,2).")
      self.assertEqual(dsds[2,2,2,3],0.0,"ds/ds has wrong value at (2,2,2,3).")
      self.assertEqual(dsds[2,2,2,4],0.0,"ds/ds has wrong value at (2,2,2,4).")
      self.assertEqual(dsds[2,2,3,0],0.0,"ds/ds has wrong value at (2,2,3,0).")
      self.assertEqual(dsds[2,2,3,1],0.0,"ds/ds has wrong value at (2,2,3,1).")
      self.assertEqual(dsds[2,2,3,2],0.0,"ds/ds has wrong value at (2,2,3,2).")
      self.assertEqual(dsds[2,2,3,3],0.0,"ds/ds has wrong value at (2,2,3,3).")
      self.assertEqual(dsds[2,2,3,4],0.0,"ds/ds has wrong value at (2,2,3,4).")
      self.assertEqual(dsds[2,3,0,0],0.0,"ds/ds has wrong value at (2,3,0,0).")
      self.assertEqual(dsds[2,3,0,1],0.0,"ds/ds has wrong value at (2,3,0,1).")
      self.assertEqual(dsds[2,3,0,2],0.0,"ds/ds has wrong value at (2,3,0,2).")
      self.assertEqual(dsds[2,3,0,3],0.0,"ds/ds has wrong value at (2,3,0,3).")
      self.assertEqual(dsds[2,3,0,4],0.0,"ds/ds has wrong value at (2,3,0,4).")
      self.assertEqual(dsds[2,3,1,0],0.0,"ds/ds has wrong value at (2,3,1,0).")
      self.assertEqual(dsds[2,3,1,1],0.0,"ds/ds has wrong value at (2,3,1,1).")
      self.assertEqual(dsds[2,3,1,2],0.0,"ds/ds has wrong value at (2,3,1,2).")
      self.assertEqual(dsds[2,3,1,3],0.0,"ds/ds has wrong value at (2,3,1,3).")
      self.assertEqual(dsds[2,3,1,4],0.0,"ds/ds has wrong value at (2,3,1,4).")
      self.assertEqual(dsds[2,3,2,0],0.0,"ds/ds has wrong value at (2,3,2,0).")
      self.assertEqual(dsds[2,3,2,1],0.0,"ds/ds has wrong value at (2,3,2,1).")
      self.assertEqual(dsds[2,3,2,2],0.0,"ds/ds has wrong value at (2,3,2,2).")
      self.assertEqual(dsds[2,3,2,3],1.0,"ds/ds has wrong value at (2,3,2,3).")
      self.assertEqual(dsds[2,3,2,4],0.0,"ds/ds has wrong value at (2,3,2,4).")
      self.assertEqual(dsds[2,3,3,0],0.0,"ds/ds has wrong value at (2,3,3,0).")
      self.assertEqual(dsds[2,3,3,1],0.0,"ds/ds has wrong value at (2,3,3,1).")
      self.assertEqual(dsds[2,3,3,2],0.0,"ds/ds has wrong value at (2,3,3,2).")
      self.assertEqual(dsds[2,3,3,3],0.0,"ds/ds has wrong value at (2,3,3,3).")
      self.assertEqual(dsds[2,3,3,4],0.0,"ds/ds has wrong value at (2,3,3,4).")
      self.assertEqual(dsds[2,4,0,0],0.0,"ds/ds has wrong value at (2,4,0,0).")
      self.assertEqual(dsds[2,4,0,1],0.0,"ds/ds has wrong value at (2,4,0,1).")
      self.assertEqual(dsds[2,4,0,2],0.0,"ds/ds has wrong value at (2,4,0,2).")
      self.assertEqual(dsds[2,4,0,3],0.0,"ds/ds has wrong value at (2,4,0,3).")
      self.assertEqual(dsds[2,4,0,4],0.0,"ds/ds has wrong value at (2,4,0,4).")
      self.assertEqual(dsds[2,4,1,0],0.0,"ds/ds has wrong value at (2,4,1,0).")
      self.assertEqual(dsds[2,4,1,1],0.0,"ds/ds has wrong value at (2,4,1,1).")
      self.assertEqual(dsds[2,4,1,2],0.0,"ds/ds has wrong value at (2,4,1,2).")
      self.assertEqual(dsds[2,4,1,3],0.0,"ds/ds has wrong value at (2,4,1,3).")
      self.assertEqual(dsds[2,4,1,4],0.0,"ds/ds has wrong value at (2,4,1,4).")
      self.assertEqual(dsds[2,4,2,0],0.0,"ds/ds has wrong value at (2,4,2,0).")
      self.assertEqual(dsds[2,4,2,1],0.0,"ds/ds has wrong value at (2,4,2,1).")
      self.assertEqual(dsds[2,4,2,2],0.0,"ds/ds has wrong value at (2,4,2,2).")
      self.assertEqual(dsds[2,4,2,3],0.0,"ds/ds has wrong value at (2,4,2,3).")
      self.assertEqual(dsds[2,4,2,4],1.0,"ds/ds has wrong value at (2,4,2,4).")
      self.assertEqual(dsds[2,4,3,0],0.0,"ds/ds has wrong value at (2,4,3,0).")
      self.assertEqual(dsds[2,4,3,1],0.0,"ds/ds has wrong value at (2,4,3,1).")
      self.assertEqual(dsds[2,4,3,2],0.0,"ds/ds has wrong value at (2,4,3,2).")
      self.assertEqual(dsds[2,4,3,3],0.0,"ds/ds has wrong value at (2,4,3,3).")
      self.assertEqual(dsds[2,4,3,4],0.0,"ds/ds has wrong value at (2,4,3,4).")
      self.assertEqual(dsds[3,0,0,0],0.0,"ds/ds has wrong value at (3,0,0,0).")
      self.assertEqual(dsds[3,0,0,1],0.0,"ds/ds has wrong value at (3,0,0,1).")
      self.assertEqual(dsds[3,0,0,2],0.0,"ds/ds has wrong value at (3,0,0,2).")
      self.assertEqual(dsds[3,0,0,3],0.0,"ds/ds has wrong value at (3,0,0,3).")
      self.assertEqual(dsds[3,0,0,4],0.0,"ds/ds has wrong value at (3,0,0,4).")
      self.assertEqual(dsds[3,0,1,0],0.0,"ds/ds has wrong value at (3,0,1,0).")
      self.assertEqual(dsds[3,0,1,1],0.0,"ds/ds has wrong value at (3,0,1,1).")
      self.assertEqual(dsds[3,0,1,2],0.0,"ds/ds has wrong value at (3,0,1,2).")
      self.assertEqual(dsds[3,0,1,3],0.0,"ds/ds has wrong value at (3,0,1,3).")
      self.assertEqual(dsds[3,0,1,4],0.0,"ds/ds has wrong value at (3,0,1,4).")
      self.assertEqual(dsds[3,0,2,0],0.0,"ds/ds has wrong value at (3,0,2,0).")
      self.assertEqual(dsds[3,0,2,1],0.0,"ds/ds has wrong value at (3,0,2,1).")
      self.assertEqual(dsds[3,0,2,2],0.0,"ds/ds has wrong value at (3,0,2,2).")
      self.assertEqual(dsds[3,0,2,3],0.0,"ds/ds has wrong value at (3,0,2,3).")
      self.assertEqual(dsds[3,0,2,4],0.0,"ds/ds has wrong value at (3,0,2,4).")
      self.assertEqual(dsds[3,0,3,0],1.0,"ds/ds has wrong value at (3,0,3,0).")
      self.assertEqual(dsds[3,0,3,1],0.0,"ds/ds has wrong value at (3,0,3,1).")
      self.assertEqual(dsds[3,0,3,2],0.0,"ds/ds has wrong value at (3,0,3,2).")
      self.assertEqual(dsds[3,0,3,3],0.0,"ds/ds has wrong value at (3,0,3,3).")
      self.assertEqual(dsds[3,0,3,4],0.0,"ds/ds has wrong value at (3,0,3,4).")
      self.assertEqual(dsds[3,1,0,0],0.0,"ds/ds has wrong value at (3,1,0,0).")
      self.assertEqual(dsds[3,1,0,1],0.0,"ds/ds has wrong value at (3,1,0,1).")
      self.assertEqual(dsds[3,1,0,2],0.0,"ds/ds has wrong value at (3,1,0,2).")
      self.assertEqual(dsds[3,1,0,3],0.0,"ds/ds has wrong value at (3,1,0,3).")
      self.assertEqual(dsds[3,1,0,4],0.0,"ds/ds has wrong value at (3,1,0,4).")
      self.assertEqual(dsds[3,1,1,0],0.0,"ds/ds has wrong value at (3,1,1,0).")
      self.assertEqual(dsds[3,1,1,1],0.0,"ds/ds has wrong value at (3,1,1,1).")
      self.assertEqual(dsds[3,1,1,2],0.0,"ds/ds has wrong value at (3,1,1,2).")
      self.assertEqual(dsds[3,1,1,3],0.0,"ds/ds has wrong value at (3,1,1,3).")
      self.assertEqual(dsds[3,1,1,4],0.0,"ds/ds has wrong value at (3,1,1,4).")
      self.assertEqual(dsds[3,1,2,0],0.0,"ds/ds has wrong value at (3,1,2,0).")
      self.assertEqual(dsds[3,1,2,1],0.0,"ds/ds has wrong value at (3,1,2,1).")
      self.assertEqual(dsds[3,1,2,2],0.0,"ds/ds has wrong value at (3,1,2,2).")
      self.assertEqual(dsds[3,1,2,3],0.0,"ds/ds has wrong value at (3,1,2,3).")
      self.assertEqual(dsds[3,1,2,4],0.0,"ds/ds has wrong value at (3,1,2,4).")
      self.assertEqual(dsds[3,1,3,0],0.0,"ds/ds has wrong value at (3,1,3,0).")
      self.assertEqual(dsds[3,1,3,1],1.0,"ds/ds has wrong value at (3,1,3,1).")
      self.assertEqual(dsds[3,1,3,2],0.0,"ds/ds has wrong value at (3,1,3,2).")
      self.assertEqual(dsds[3,1,3,3],0.0,"ds/ds has wrong value at (3,1,3,3).")
      self.assertEqual(dsds[3,1,3,4],0.0,"ds/ds has wrong value at (3,1,3,4).")
      self.assertEqual(dsds[3,2,0,0],0.0,"ds/ds has wrong value at (3,2,0,0).")
      self.assertEqual(dsds[3,2,0,1],0.0,"ds/ds has wrong value at (3,2,0,1).")
      self.assertEqual(dsds[3,2,0,2],0.0,"ds/ds has wrong value at (3,2,0,2).")
      self.assertEqual(dsds[3,2,0,3],0.0,"ds/ds has wrong value at (3,2,0,3).")
      self.assertEqual(dsds[3,2,0,4],0.0,"ds/ds has wrong value at (3,2,0,4).")
      self.assertEqual(dsds[3,2,1,0],0.0,"ds/ds has wrong value at (3,2,1,0).")
      self.assertEqual(dsds[3,2,1,1],0.0,"ds/ds has wrong value at (3,2,1,1).")
      self.assertEqual(dsds[3,2,1,2],0.0,"ds/ds has wrong value at (3,2,1,2).")
      self.assertEqual(dsds[3,2,1,3],0.0,"ds/ds has wrong value at (3,2,1,3).")
      self.assertEqual(dsds[3,2,1,4],0.0,"ds/ds has wrong value at (3,2,1,4).")
      self.assertEqual(dsds[3,2,2,0],0.0,"ds/ds has wrong value at (3,2,2,0).")
      self.assertEqual(dsds[3,2,2,1],0.0,"ds/ds has wrong value at (3,2,2,1).")
      self.assertEqual(dsds[3,2,2,2],0.0,"ds/ds has wrong value at (3,2,2,2).")
      self.assertEqual(dsds[3,2,2,3],0.0,"ds/ds has wrong value at (3,2,2,3).")
      self.assertEqual(dsds[3,2,2,4],0.0,"ds/ds has wrong value at (3,2,2,4).")
      self.assertEqual(dsds[3,2,3,0],0.0,"ds/ds has wrong value at (3,2,3,0).")
      self.assertEqual(dsds[3,2,3,1],0.0,"ds/ds has wrong value at (3,2,3,1).")
      self.assertEqual(dsds[3,2,3,2],1.0,"ds/ds has wrong value at (3,2,3,2).")
      self.assertEqual(dsds[3,2,3,3],0.0,"ds/ds has wrong value at (3,2,3,3).")
      self.assertEqual(dsds[3,2,3,4],0.0,"ds/ds has wrong value at (3,2,3,4).")
      self.assertEqual(dsds[3,3,0,0],0.0,"ds/ds has wrong value at (3,3,0,0).")
      self.assertEqual(dsds[3,3,0,1],0.0,"ds/ds has wrong value at (3,3,0,1).")
      self.assertEqual(dsds[3,3,0,2],0.0,"ds/ds has wrong value at (3,3,0,2).")
      self.assertEqual(dsds[3,3,0,3],0.0,"ds/ds has wrong value at (3,3,0,3).")
      self.assertEqual(dsds[3,3,0,4],0.0,"ds/ds has wrong value at (3,3,0,4).")
      self.assertEqual(dsds[3,3,1,0],0.0,"ds/ds has wrong value at (3,3,1,0).")
      self.assertEqual(dsds[3,3,1,1],0.0,"ds/ds has wrong value at (3,3,1,1).")
      self.assertEqual(dsds[3,3,1,2],0.0,"ds/ds has wrong value at (3,3,1,2).")
      self.assertEqual(dsds[3,3,1,3],0.0,"ds/ds has wrong value at (3,3,1,3).")
      self.assertEqual(dsds[3,3,1,4],0.0,"ds/ds has wrong value at (3,3,1,4).")
      self.assertEqual(dsds[3,3,2,0],0.0,"ds/ds has wrong value at (3,3,2,0).")
      self.assertEqual(dsds[3,3,2,1],0.0,"ds/ds has wrong value at (3,3,2,1).")
      self.assertEqual(dsds[3,3,2,2],0.0,"ds/ds has wrong value at (3,3,2,2).")
      self.assertEqual(dsds[3,3,2,3],0.0,"ds/ds has wrong value at (3,3,2,3).")
      self.assertEqual(dsds[3,3,2,4],0.0,"ds/ds has wrong value at (3,3,2,4).")
      self.assertEqual(dsds[3,3,3,0],0.0,"ds/ds has wrong value at (3,3,3,0).")
      self.assertEqual(dsds[3,3,3,1],0.0,"ds/ds has wrong value at (3,3,3,1).")
      self.assertEqual(dsds[3,3,3,2],0.0,"ds/ds has wrong value at (3,3,3,2).")
      self.assertEqual(dsds[3,3,3,3],1.0,"ds/ds has wrong value at (3,3,3,3).")
      self.assertEqual(dsds[3,3,3,4],0.0,"ds/ds has wrong value at (3,3,3,4).")
      self.assertEqual(dsds[3,4,0,0],0.0,"ds/ds has wrong value at (3,4,0,0).")
      self.assertEqual(dsds[3,4,0,1],0.0,"ds/ds has wrong value at (3,4,0,1).")
      self.assertEqual(dsds[3,4,0,2],0.0,"ds/ds has wrong value at (3,4,0,2).")
      self.assertEqual(dsds[3,4,0,3],0.0,"ds/ds has wrong value at (3,4,0,3).")
      self.assertEqual(dsds[3,4,0,4],0.0,"ds/ds has wrong value at (3,4,0,4).")
      self.assertEqual(dsds[3,4,1,0],0.0,"ds/ds has wrong value at (3,4,1,0).")
      self.assertEqual(dsds[3,4,1,1],0.0,"ds/ds has wrong value at (3,4,1,1).")
      self.assertEqual(dsds[3,4,1,2],0.0,"ds/ds has wrong value at (3,4,1,2).")
      self.assertEqual(dsds[3,4,1,3],0.0,"ds/ds has wrong value at (3,4,1,3).")
      self.assertEqual(dsds[3,4,1,4],0.0,"ds/ds has wrong value at (3,4,1,4).")
      self.assertEqual(dsds[3,4,2,0],0.0,"ds/ds has wrong value at (3,4,2,0).")
      self.assertEqual(dsds[3,4,2,1],0.0,"ds/ds has wrong value at (3,4,2,1).")
      self.assertEqual(dsds[3,4,2,2],0.0,"ds/ds has wrong value at (3,4,2,2).")
      self.assertEqual(dsds[3,4,2,3],0.0,"ds/ds has wrong value at (3,4,2,3).")
      self.assertEqual(dsds[3,4,2,4],0.0,"ds/ds has wrong value at (3,4,2,4).")
      self.assertEqual(dsds[3,4,3,0],0.0,"ds/ds has wrong value at (3,4,3,0).")
      self.assertEqual(dsds[3,4,3,1],0.0,"ds/ds has wrong value at (3,4,3,1).")
      self.assertEqual(dsds[3,4,3,2],0.0,"ds/ds has wrong value at (3,4,3,2).")
      self.assertEqual(dsds[3,4,3,3],0.0,"ds/ds has wrong value at (3,4,3,3).")
      self.assertEqual(dsds[3,4,3,4],1.0,"ds/ds has wrong value at (3,4,3,4).")

      self.assertEqual(s.getArgument(0),s2,"wrong argument 0.")
      sa=s.getSubstitutedArguments({s2:-10})
      self.assertEqual(len(sa),1,"wrong number of substituted arguments")
      self.assertEqual(sa[0],-10,"wrongly substituted argument 0.")

      self.assertFalse(s.isAppropriateValue(10.)," 10. is not appropriate substitute")
      self.assertFalse(s.isAppropriateValue(10)," 10 is not appropriate substitute")
      self.assertTrue(s.isAppropriateValue(Symbol(shape=(4, 5),dim=None)),"Symbol is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(Symbol(shape=(5,))),"Symbol is not appropriate substitute (shape)")
      self.assertFalse(s.isAppropriateValue(escript.Data(0.,(5,),self.functionspace)),"Data is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(numpy.zeros((4, 5))),"ndarray is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(numpy.zeros((5,))),"ndarray is not appropriate substitute (shape)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_Symbol_rank2_dNone_nargs2(self):
      s2=Symbol(shape=())
      s=Symbol(shape=(4, 5),dim=None,args=[1, -1.0])
      self.assertEqual(s.getRank(),2,"wrong rank.")
      self.assertEqual(s.getShape(),(4, 5),"wrong shape.")
      self.assertEqual(s.getDim(),None,"wrong spatial dimension.")
      self.assertEqual(s.getArgument(),[1, -1.0],"wrong arguments.")

      ss=s.substitute({s:numpy.zeros((4, 5))})
      self.assertTrue(isinstance(ss,numpy.ndarray),"value after substitution is not ndarray.")
      self.assertEqual(ss.shape,(4, 5),"value after substitution has not expected shape")
      try:
        s.substitute({s:numpy.zeros((5,))})
        fail("illegal substition was successful")
      except TypeError:
        pass

      dsdarg=s.diff(Symbol(shape=()))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds() has wrong type.")
      self.assertEqual(dsdarg.shape,(4, 5),"ds/ds() has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds() has wrong value.")
      dsdarg=s.diff(Symbol(shape=(2,)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(2,) has wrong type.")
      self.assertEqual(dsdarg.shape,(4, 5, 2),"ds/ds(2,) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(2,) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(4, 5)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(4, 5) has wrong type.")
      self.assertEqual(dsdarg.shape,(4, 5, 4, 5),"ds/ds(4, 5) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(4, 5) has wrong value.")

      dsds=s.diff(s)
      self.assertTrue(isinstance(dsds,numpy.ndarray),"ds/ds has wrong type.")
      self.assertEqual(dsds.shape,(4, 5, 4, 5),"ds/ds has wrong shape.")
      self.assertEqual(dsds[0,0,0,0],1.0,"ds/ds has wrong value at (0,0,0,0).")
      self.assertEqual(dsds[0,0,0,1],0.0,"ds/ds has wrong value at (0,0,0,1).")
      self.assertEqual(dsds[0,0,0,2],0.0,"ds/ds has wrong value at (0,0,0,2).")
      self.assertEqual(dsds[0,0,0,3],0.0,"ds/ds has wrong value at (0,0,0,3).")
      self.assertEqual(dsds[0,0,0,4],0.0,"ds/ds has wrong value at (0,0,0,4).")
      self.assertEqual(dsds[0,0,1,0],0.0,"ds/ds has wrong value at (0,0,1,0).")
      self.assertEqual(dsds[0,0,1,1],0.0,"ds/ds has wrong value at (0,0,1,1).")
      self.assertEqual(dsds[0,0,1,2],0.0,"ds/ds has wrong value at (0,0,1,2).")
      self.assertEqual(dsds[0,0,1,3],0.0,"ds/ds has wrong value at (0,0,1,3).")
      self.assertEqual(dsds[0,0,1,4],0.0,"ds/ds has wrong value at (0,0,1,4).")
      self.assertEqual(dsds[0,0,2,0],0.0,"ds/ds has wrong value at (0,0,2,0).")
      self.assertEqual(dsds[0,0,2,1],0.0,"ds/ds has wrong value at (0,0,2,1).")
      self.assertEqual(dsds[0,0,2,2],0.0,"ds/ds has wrong value at (0,0,2,2).")
      self.assertEqual(dsds[0,0,2,3],0.0,"ds/ds has wrong value at (0,0,2,3).")
      self.assertEqual(dsds[0,0,2,4],0.0,"ds/ds has wrong value at (0,0,2,4).")
      self.assertEqual(dsds[0,0,3,0],0.0,"ds/ds has wrong value at (0,0,3,0).")
      self.assertEqual(dsds[0,0,3,1],0.0,"ds/ds has wrong value at (0,0,3,1).")
      self.assertEqual(dsds[0,0,3,2],0.0,"ds/ds has wrong value at (0,0,3,2).")
      self.assertEqual(dsds[0,0,3,3],0.0,"ds/ds has wrong value at (0,0,3,3).")
      self.assertEqual(dsds[0,0,3,4],0.0,"ds/ds has wrong value at (0,0,3,4).")
      self.assertEqual(dsds[0,1,0,0],0.0,"ds/ds has wrong value at (0,1,0,0).")
      self.assertEqual(dsds[0,1,0,1],1.0,"ds/ds has wrong value at (0,1,0,1).")
      self.assertEqual(dsds[0,1,0,2],0.0,"ds/ds has wrong value at (0,1,0,2).")
      self.assertEqual(dsds[0,1,0,3],0.0,"ds/ds has wrong value at (0,1,0,3).")
      self.assertEqual(dsds[0,1,0,4],0.0,"ds/ds has wrong value at (0,1,0,4).")
      self.assertEqual(dsds[0,1,1,0],0.0,"ds/ds has wrong value at (0,1,1,0).")
      self.assertEqual(dsds[0,1,1,1],0.0,"ds/ds has wrong value at (0,1,1,1).")
      self.assertEqual(dsds[0,1,1,2],0.0,"ds/ds has wrong value at (0,1,1,2).")
      self.assertEqual(dsds[0,1,1,3],0.0,"ds/ds has wrong value at (0,1,1,3).")
      self.assertEqual(dsds[0,1,1,4],0.0,"ds/ds has wrong value at (0,1,1,4).")
      self.assertEqual(dsds[0,1,2,0],0.0,"ds/ds has wrong value at (0,1,2,0).")
      self.assertEqual(dsds[0,1,2,1],0.0,"ds/ds has wrong value at (0,1,2,1).")
      self.assertEqual(dsds[0,1,2,2],0.0,"ds/ds has wrong value at (0,1,2,2).")
      self.assertEqual(dsds[0,1,2,3],0.0,"ds/ds has wrong value at (0,1,2,3).")
      self.assertEqual(dsds[0,1,2,4],0.0,"ds/ds has wrong value at (0,1,2,4).")
      self.assertEqual(dsds[0,1,3,0],0.0,"ds/ds has wrong value at (0,1,3,0).")
      self.assertEqual(dsds[0,1,3,1],0.0,"ds/ds has wrong value at (0,1,3,1).")
      self.assertEqual(dsds[0,1,3,2],0.0,"ds/ds has wrong value at (0,1,3,2).")
      self.assertEqual(dsds[0,1,3,3],0.0,"ds/ds has wrong value at (0,1,3,3).")
      self.assertEqual(dsds[0,1,3,4],0.0,"ds/ds has wrong value at (0,1,3,4).")
      self.assertEqual(dsds[0,2,0,0],0.0,"ds/ds has wrong value at (0,2,0,0).")
      self.assertEqual(dsds[0,2,0,1],0.0,"ds/ds has wrong value at (0,2,0,1).")
      self.assertEqual(dsds[0,2,0,2],1.0,"ds/ds has wrong value at (0,2,0,2).")
      self.assertEqual(dsds[0,2,0,3],0.0,"ds/ds has wrong value at (0,2,0,3).")
      self.assertEqual(dsds[0,2,0,4],0.0,"ds/ds has wrong value at (0,2,0,4).")
      self.assertEqual(dsds[0,2,1,0],0.0,"ds/ds has wrong value at (0,2,1,0).")
      self.assertEqual(dsds[0,2,1,1],0.0,"ds/ds has wrong value at (0,2,1,1).")
      self.assertEqual(dsds[0,2,1,2],0.0,"ds/ds has wrong value at (0,2,1,2).")
      self.assertEqual(dsds[0,2,1,3],0.0,"ds/ds has wrong value at (0,2,1,3).")
      self.assertEqual(dsds[0,2,1,4],0.0,"ds/ds has wrong value at (0,2,1,4).")
      self.assertEqual(dsds[0,2,2,0],0.0,"ds/ds has wrong value at (0,2,2,0).")
      self.assertEqual(dsds[0,2,2,1],0.0,"ds/ds has wrong value at (0,2,2,1).")
      self.assertEqual(dsds[0,2,2,2],0.0,"ds/ds has wrong value at (0,2,2,2).")
      self.assertEqual(dsds[0,2,2,3],0.0,"ds/ds has wrong value at (0,2,2,3).")
      self.assertEqual(dsds[0,2,2,4],0.0,"ds/ds has wrong value at (0,2,2,4).")
      self.assertEqual(dsds[0,2,3,0],0.0,"ds/ds has wrong value at (0,2,3,0).")
      self.assertEqual(dsds[0,2,3,1],0.0,"ds/ds has wrong value at (0,2,3,1).")
      self.assertEqual(dsds[0,2,3,2],0.0,"ds/ds has wrong value at (0,2,3,2).")
      self.assertEqual(dsds[0,2,3,3],0.0,"ds/ds has wrong value at (0,2,3,3).")
      self.assertEqual(dsds[0,2,3,4],0.0,"ds/ds has wrong value at (0,2,3,4).")
      self.assertEqual(dsds[0,3,0,0],0.0,"ds/ds has wrong value at (0,3,0,0).")
      self.assertEqual(dsds[0,3,0,1],0.0,"ds/ds has wrong value at (0,3,0,1).")
      self.assertEqual(dsds[0,3,0,2],0.0,"ds/ds has wrong value at (0,3,0,2).")
      self.assertEqual(dsds[0,3,0,3],1.0,"ds/ds has wrong value at (0,3,0,3).")
      self.assertEqual(dsds[0,3,0,4],0.0,"ds/ds has wrong value at (0,3,0,4).")
      self.assertEqual(dsds[0,3,1,0],0.0,"ds/ds has wrong value at (0,3,1,0).")
      self.assertEqual(dsds[0,3,1,1],0.0,"ds/ds has wrong value at (0,3,1,1).")
      self.assertEqual(dsds[0,3,1,2],0.0,"ds/ds has wrong value at (0,3,1,2).")
      self.assertEqual(dsds[0,3,1,3],0.0,"ds/ds has wrong value at (0,3,1,3).")
      self.assertEqual(dsds[0,3,1,4],0.0,"ds/ds has wrong value at (0,3,1,4).")
      self.assertEqual(dsds[0,3,2,0],0.0,"ds/ds has wrong value at (0,3,2,0).")
      self.assertEqual(dsds[0,3,2,1],0.0,"ds/ds has wrong value at (0,3,2,1).")
      self.assertEqual(dsds[0,3,2,2],0.0,"ds/ds has wrong value at (0,3,2,2).")
      self.assertEqual(dsds[0,3,2,3],0.0,"ds/ds has wrong value at (0,3,2,3).")
      self.assertEqual(dsds[0,3,2,4],0.0,"ds/ds has wrong value at (0,3,2,4).")
      self.assertEqual(dsds[0,3,3,0],0.0,"ds/ds has wrong value at (0,3,3,0).")
      self.assertEqual(dsds[0,3,3,1],0.0,"ds/ds has wrong value at (0,3,3,1).")
      self.assertEqual(dsds[0,3,3,2],0.0,"ds/ds has wrong value at (0,3,3,2).")
      self.assertEqual(dsds[0,3,3,3],0.0,"ds/ds has wrong value at (0,3,3,3).")
      self.assertEqual(dsds[0,3,3,4],0.0,"ds/ds has wrong value at (0,3,3,4).")
      self.assertEqual(dsds[0,4,0,0],0.0,"ds/ds has wrong value at (0,4,0,0).")
      self.assertEqual(dsds[0,4,0,1],0.0,"ds/ds has wrong value at (0,4,0,1).")
      self.assertEqual(dsds[0,4,0,2],0.0,"ds/ds has wrong value at (0,4,0,2).")
      self.assertEqual(dsds[0,4,0,3],0.0,"ds/ds has wrong value at (0,4,0,3).")
      self.assertEqual(dsds[0,4,0,4],1.0,"ds/ds has wrong value at (0,4,0,4).")
      self.assertEqual(dsds[0,4,1,0],0.0,"ds/ds has wrong value at (0,4,1,0).")
      self.assertEqual(dsds[0,4,1,1],0.0,"ds/ds has wrong value at (0,4,1,1).")
      self.assertEqual(dsds[0,4,1,2],0.0,"ds/ds has wrong value at (0,4,1,2).")
      self.assertEqual(dsds[0,4,1,3],0.0,"ds/ds has wrong value at (0,4,1,3).")
      self.assertEqual(dsds[0,4,1,4],0.0,"ds/ds has wrong value at (0,4,1,4).")
      self.assertEqual(dsds[0,4,2,0],0.0,"ds/ds has wrong value at (0,4,2,0).")
      self.assertEqual(dsds[0,4,2,1],0.0,"ds/ds has wrong value at (0,4,2,1).")
      self.assertEqual(dsds[0,4,2,2],0.0,"ds/ds has wrong value at (0,4,2,2).")
      self.assertEqual(dsds[0,4,2,3],0.0,"ds/ds has wrong value at (0,4,2,3).")
      self.assertEqual(dsds[0,4,2,4],0.0,"ds/ds has wrong value at (0,4,2,4).")
      self.assertEqual(dsds[0,4,3,0],0.0,"ds/ds has wrong value at (0,4,3,0).")
      self.assertEqual(dsds[0,4,3,1],0.0,"ds/ds has wrong value at (0,4,3,1).")
      self.assertEqual(dsds[0,4,3,2],0.0,"ds/ds has wrong value at (0,4,3,2).")
      self.assertEqual(dsds[0,4,3,3],0.0,"ds/ds has wrong value at (0,4,3,3).")
      self.assertEqual(dsds[0,4,3,4],0.0,"ds/ds has wrong value at (0,4,3,4).")
      self.assertEqual(dsds[1,0,0,0],0.0,"ds/ds has wrong value at (1,0,0,0).")
      self.assertEqual(dsds[1,0,0,1],0.0,"ds/ds has wrong value at (1,0,0,1).")
      self.assertEqual(dsds[1,0,0,2],0.0,"ds/ds has wrong value at (1,0,0,2).")
      self.assertEqual(dsds[1,0,0,3],0.0,"ds/ds has wrong value at (1,0,0,3).")
      self.assertEqual(dsds[1,0,0,4],0.0,"ds/ds has wrong value at (1,0,0,4).")
      self.assertEqual(dsds[1,0,1,0],1.0,"ds/ds has wrong value at (1,0,1,0).")
      self.assertEqual(dsds[1,0,1,1],0.0,"ds/ds has wrong value at (1,0,1,1).")
      self.assertEqual(dsds[1,0,1,2],0.0,"ds/ds has wrong value at (1,0,1,2).")
      self.assertEqual(dsds[1,0,1,3],0.0,"ds/ds has wrong value at (1,0,1,3).")
      self.assertEqual(dsds[1,0,1,4],0.0,"ds/ds has wrong value at (1,0,1,4).")
      self.assertEqual(dsds[1,0,2,0],0.0,"ds/ds has wrong value at (1,0,2,0).")
      self.assertEqual(dsds[1,0,2,1],0.0,"ds/ds has wrong value at (1,0,2,1).")
      self.assertEqual(dsds[1,0,2,2],0.0,"ds/ds has wrong value at (1,0,2,2).")
      self.assertEqual(dsds[1,0,2,3],0.0,"ds/ds has wrong value at (1,0,2,3).")
      self.assertEqual(dsds[1,0,2,4],0.0,"ds/ds has wrong value at (1,0,2,4).")
      self.assertEqual(dsds[1,0,3,0],0.0,"ds/ds has wrong value at (1,0,3,0).")
      self.assertEqual(dsds[1,0,3,1],0.0,"ds/ds has wrong value at (1,0,3,1).")
      self.assertEqual(dsds[1,0,3,2],0.0,"ds/ds has wrong value at (1,0,3,2).")
      self.assertEqual(dsds[1,0,3,3],0.0,"ds/ds has wrong value at (1,0,3,3).")
      self.assertEqual(dsds[1,0,3,4],0.0,"ds/ds has wrong value at (1,0,3,4).")
      self.assertEqual(dsds[1,1,0,0],0.0,"ds/ds has wrong value at (1,1,0,0).")
      self.assertEqual(dsds[1,1,0,1],0.0,"ds/ds has wrong value at (1,1,0,1).")
      self.assertEqual(dsds[1,1,0,2],0.0,"ds/ds has wrong value at (1,1,0,2).")
      self.assertEqual(dsds[1,1,0,3],0.0,"ds/ds has wrong value at (1,1,0,3).")
      self.assertEqual(dsds[1,1,0,4],0.0,"ds/ds has wrong value at (1,1,0,4).")
      self.assertEqual(dsds[1,1,1,0],0.0,"ds/ds has wrong value at (1,1,1,0).")
      self.assertEqual(dsds[1,1,1,1],1.0,"ds/ds has wrong value at (1,1,1,1).")
      self.assertEqual(dsds[1,1,1,2],0.0,"ds/ds has wrong value at (1,1,1,2).")
      self.assertEqual(dsds[1,1,1,3],0.0,"ds/ds has wrong value at (1,1,1,3).")
      self.assertEqual(dsds[1,1,1,4],0.0,"ds/ds has wrong value at (1,1,1,4).")
      self.assertEqual(dsds[1,1,2,0],0.0,"ds/ds has wrong value at (1,1,2,0).")
      self.assertEqual(dsds[1,1,2,1],0.0,"ds/ds has wrong value at (1,1,2,1).")
      self.assertEqual(dsds[1,1,2,2],0.0,"ds/ds has wrong value at (1,1,2,2).")
      self.assertEqual(dsds[1,1,2,3],0.0,"ds/ds has wrong value at (1,1,2,3).")
      self.assertEqual(dsds[1,1,2,4],0.0,"ds/ds has wrong value at (1,1,2,4).")
      self.assertEqual(dsds[1,1,3,0],0.0,"ds/ds has wrong value at (1,1,3,0).")
      self.assertEqual(dsds[1,1,3,1],0.0,"ds/ds has wrong value at (1,1,3,1).")
      self.assertEqual(dsds[1,1,3,2],0.0,"ds/ds has wrong value at (1,1,3,2).")
      self.assertEqual(dsds[1,1,3,3],0.0,"ds/ds has wrong value at (1,1,3,3).")
      self.assertEqual(dsds[1,1,3,4],0.0,"ds/ds has wrong value at (1,1,3,4).")
      self.assertEqual(dsds[1,2,0,0],0.0,"ds/ds has wrong value at (1,2,0,0).")
      self.assertEqual(dsds[1,2,0,1],0.0,"ds/ds has wrong value at (1,2,0,1).")
      self.assertEqual(dsds[1,2,0,2],0.0,"ds/ds has wrong value at (1,2,0,2).")
      self.assertEqual(dsds[1,2,0,3],0.0,"ds/ds has wrong value at (1,2,0,3).")
      self.assertEqual(dsds[1,2,0,4],0.0,"ds/ds has wrong value at (1,2,0,4).")
      self.assertEqual(dsds[1,2,1,0],0.0,"ds/ds has wrong value at (1,2,1,0).")
      self.assertEqual(dsds[1,2,1,1],0.0,"ds/ds has wrong value at (1,2,1,1).")
      self.assertEqual(dsds[1,2,1,2],1.0,"ds/ds has wrong value at (1,2,1,2).")
      self.assertEqual(dsds[1,2,1,3],0.0,"ds/ds has wrong value at (1,2,1,3).")
      self.assertEqual(dsds[1,2,1,4],0.0,"ds/ds has wrong value at (1,2,1,4).")
      self.assertEqual(dsds[1,2,2,0],0.0,"ds/ds has wrong value at (1,2,2,0).")
      self.assertEqual(dsds[1,2,2,1],0.0,"ds/ds has wrong value at (1,2,2,1).")
      self.assertEqual(dsds[1,2,2,2],0.0,"ds/ds has wrong value at (1,2,2,2).")
      self.assertEqual(dsds[1,2,2,3],0.0,"ds/ds has wrong value at (1,2,2,3).")
      self.assertEqual(dsds[1,2,2,4],0.0,"ds/ds has wrong value at (1,2,2,4).")
      self.assertEqual(dsds[1,2,3,0],0.0,"ds/ds has wrong value at (1,2,3,0).")
      self.assertEqual(dsds[1,2,3,1],0.0,"ds/ds has wrong value at (1,2,3,1).")
      self.assertEqual(dsds[1,2,3,2],0.0,"ds/ds has wrong value at (1,2,3,2).")
      self.assertEqual(dsds[1,2,3,3],0.0,"ds/ds has wrong value at (1,2,3,3).")
      self.assertEqual(dsds[1,2,3,4],0.0,"ds/ds has wrong value at (1,2,3,4).")
      self.assertEqual(dsds[1,3,0,0],0.0,"ds/ds has wrong value at (1,3,0,0).")
      self.assertEqual(dsds[1,3,0,1],0.0,"ds/ds has wrong value at (1,3,0,1).")
      self.assertEqual(dsds[1,3,0,2],0.0,"ds/ds has wrong value at (1,3,0,2).")
      self.assertEqual(dsds[1,3,0,3],0.0,"ds/ds has wrong value at (1,3,0,3).")
      self.assertEqual(dsds[1,3,0,4],0.0,"ds/ds has wrong value at (1,3,0,4).")
      self.assertEqual(dsds[1,3,1,0],0.0,"ds/ds has wrong value at (1,3,1,0).")
      self.assertEqual(dsds[1,3,1,1],0.0,"ds/ds has wrong value at (1,3,1,1).")
      self.assertEqual(dsds[1,3,1,2],0.0,"ds/ds has wrong value at (1,3,1,2).")
      self.assertEqual(dsds[1,3,1,3],1.0,"ds/ds has wrong value at (1,3,1,3).")
      self.assertEqual(dsds[1,3,1,4],0.0,"ds/ds has wrong value at (1,3,1,4).")
      self.assertEqual(dsds[1,3,2,0],0.0,"ds/ds has wrong value at (1,3,2,0).")
      self.assertEqual(dsds[1,3,2,1],0.0,"ds/ds has wrong value at (1,3,2,1).")
      self.assertEqual(dsds[1,3,2,2],0.0,"ds/ds has wrong value at (1,3,2,2).")
      self.assertEqual(dsds[1,3,2,3],0.0,"ds/ds has wrong value at (1,3,2,3).")
      self.assertEqual(dsds[1,3,2,4],0.0,"ds/ds has wrong value at (1,3,2,4).")
      self.assertEqual(dsds[1,3,3,0],0.0,"ds/ds has wrong value at (1,3,3,0).")
      self.assertEqual(dsds[1,3,3,1],0.0,"ds/ds has wrong value at (1,3,3,1).")
      self.assertEqual(dsds[1,3,3,2],0.0,"ds/ds has wrong value at (1,3,3,2).")
      self.assertEqual(dsds[1,3,3,3],0.0,"ds/ds has wrong value at (1,3,3,3).")
      self.assertEqual(dsds[1,3,3,4],0.0,"ds/ds has wrong value at (1,3,3,4).")
      self.assertEqual(dsds[1,4,0,0],0.0,"ds/ds has wrong value at (1,4,0,0).")
      self.assertEqual(dsds[1,4,0,1],0.0,"ds/ds has wrong value at (1,4,0,1).")
      self.assertEqual(dsds[1,4,0,2],0.0,"ds/ds has wrong value at (1,4,0,2).")
      self.assertEqual(dsds[1,4,0,3],0.0,"ds/ds has wrong value at (1,4,0,3).")
      self.assertEqual(dsds[1,4,0,4],0.0,"ds/ds has wrong value at (1,4,0,4).")
      self.assertEqual(dsds[1,4,1,0],0.0,"ds/ds has wrong value at (1,4,1,0).")
      self.assertEqual(dsds[1,4,1,1],0.0,"ds/ds has wrong value at (1,4,1,1).")
      self.assertEqual(dsds[1,4,1,2],0.0,"ds/ds has wrong value at (1,4,1,2).")
      self.assertEqual(dsds[1,4,1,3],0.0,"ds/ds has wrong value at (1,4,1,3).")
      self.assertEqual(dsds[1,4,1,4],1.0,"ds/ds has wrong value at (1,4,1,4).")
      self.assertEqual(dsds[1,4,2,0],0.0,"ds/ds has wrong value at (1,4,2,0).")
      self.assertEqual(dsds[1,4,2,1],0.0,"ds/ds has wrong value at (1,4,2,1).")
      self.assertEqual(dsds[1,4,2,2],0.0,"ds/ds has wrong value at (1,4,2,2).")
      self.assertEqual(dsds[1,4,2,3],0.0,"ds/ds has wrong value at (1,4,2,3).")
      self.assertEqual(dsds[1,4,2,4],0.0,"ds/ds has wrong value at (1,4,2,4).")
      self.assertEqual(dsds[1,4,3,0],0.0,"ds/ds has wrong value at (1,4,3,0).")
      self.assertEqual(dsds[1,4,3,1],0.0,"ds/ds has wrong value at (1,4,3,1).")
      self.assertEqual(dsds[1,4,3,2],0.0,"ds/ds has wrong value at (1,4,3,2).")
      self.assertEqual(dsds[1,4,3,3],0.0,"ds/ds has wrong value at (1,4,3,3).")
      self.assertEqual(dsds[1,4,3,4],0.0,"ds/ds has wrong value at (1,4,3,4).")
      self.assertEqual(dsds[2,0,0,0],0.0,"ds/ds has wrong value at (2,0,0,0).")
      self.assertEqual(dsds[2,0,0,1],0.0,"ds/ds has wrong value at (2,0,0,1).")
      self.assertEqual(dsds[2,0,0,2],0.0,"ds/ds has wrong value at (2,0,0,2).")
      self.assertEqual(dsds[2,0,0,3],0.0,"ds/ds has wrong value at (2,0,0,3).")
      self.assertEqual(dsds[2,0,0,4],0.0,"ds/ds has wrong value at (2,0,0,4).")
      self.assertEqual(dsds[2,0,1,0],0.0,"ds/ds has wrong value at (2,0,1,0).")
      self.assertEqual(dsds[2,0,1,1],0.0,"ds/ds has wrong value at (2,0,1,1).")
      self.assertEqual(dsds[2,0,1,2],0.0,"ds/ds has wrong value at (2,0,1,2).")
      self.assertEqual(dsds[2,0,1,3],0.0,"ds/ds has wrong value at (2,0,1,3).")
      self.assertEqual(dsds[2,0,1,4],0.0,"ds/ds has wrong value at (2,0,1,4).")
      self.assertEqual(dsds[2,0,2,0],1.0,"ds/ds has wrong value at (2,0,2,0).")
      self.assertEqual(dsds[2,0,2,1],0.0,"ds/ds has wrong value at (2,0,2,1).")
      self.assertEqual(dsds[2,0,2,2],0.0,"ds/ds has wrong value at (2,0,2,2).")
      self.assertEqual(dsds[2,0,2,3],0.0,"ds/ds has wrong value at (2,0,2,3).")
      self.assertEqual(dsds[2,0,2,4],0.0,"ds/ds has wrong value at (2,0,2,4).")
      self.assertEqual(dsds[2,0,3,0],0.0,"ds/ds has wrong value at (2,0,3,0).")
      self.assertEqual(dsds[2,0,3,1],0.0,"ds/ds has wrong value at (2,0,3,1).")
      self.assertEqual(dsds[2,0,3,2],0.0,"ds/ds has wrong value at (2,0,3,2).")
      self.assertEqual(dsds[2,0,3,3],0.0,"ds/ds has wrong value at (2,0,3,3).")
      self.assertEqual(dsds[2,0,3,4],0.0,"ds/ds has wrong value at (2,0,3,4).")
      self.assertEqual(dsds[2,1,0,0],0.0,"ds/ds has wrong value at (2,1,0,0).")
      self.assertEqual(dsds[2,1,0,1],0.0,"ds/ds has wrong value at (2,1,0,1).")
      self.assertEqual(dsds[2,1,0,2],0.0,"ds/ds has wrong value at (2,1,0,2).")
      self.assertEqual(dsds[2,1,0,3],0.0,"ds/ds has wrong value at (2,1,0,3).")
      self.assertEqual(dsds[2,1,0,4],0.0,"ds/ds has wrong value at (2,1,0,4).")
      self.assertEqual(dsds[2,1,1,0],0.0,"ds/ds has wrong value at (2,1,1,0).")
      self.assertEqual(dsds[2,1,1,1],0.0,"ds/ds has wrong value at (2,1,1,1).")
      self.assertEqual(dsds[2,1,1,2],0.0,"ds/ds has wrong value at (2,1,1,2).")
      self.assertEqual(dsds[2,1,1,3],0.0,"ds/ds has wrong value at (2,1,1,3).")
      self.assertEqual(dsds[2,1,1,4],0.0,"ds/ds has wrong value at (2,1,1,4).")
      self.assertEqual(dsds[2,1,2,0],0.0,"ds/ds has wrong value at (2,1,2,0).")
      self.assertEqual(dsds[2,1,2,1],1.0,"ds/ds has wrong value at (2,1,2,1).")
      self.assertEqual(dsds[2,1,2,2],0.0,"ds/ds has wrong value at (2,1,2,2).")
      self.assertEqual(dsds[2,1,2,3],0.0,"ds/ds has wrong value at (2,1,2,3).")
      self.assertEqual(dsds[2,1,2,4],0.0,"ds/ds has wrong value at (2,1,2,4).")
      self.assertEqual(dsds[2,1,3,0],0.0,"ds/ds has wrong value at (2,1,3,0).")
      self.assertEqual(dsds[2,1,3,1],0.0,"ds/ds has wrong value at (2,1,3,1).")
      self.assertEqual(dsds[2,1,3,2],0.0,"ds/ds has wrong value at (2,1,3,2).")
      self.assertEqual(dsds[2,1,3,3],0.0,"ds/ds has wrong value at (2,1,3,3).")
      self.assertEqual(dsds[2,1,3,4],0.0,"ds/ds has wrong value at (2,1,3,4).")
      self.assertEqual(dsds[2,2,0,0],0.0,"ds/ds has wrong value at (2,2,0,0).")
      self.assertEqual(dsds[2,2,0,1],0.0,"ds/ds has wrong value at (2,2,0,1).")
      self.assertEqual(dsds[2,2,0,2],0.0,"ds/ds has wrong value at (2,2,0,2).")
      self.assertEqual(dsds[2,2,0,3],0.0,"ds/ds has wrong value at (2,2,0,3).")
      self.assertEqual(dsds[2,2,0,4],0.0,"ds/ds has wrong value at (2,2,0,4).")
      self.assertEqual(dsds[2,2,1,0],0.0,"ds/ds has wrong value at (2,2,1,0).")
      self.assertEqual(dsds[2,2,1,1],0.0,"ds/ds has wrong value at (2,2,1,1).")
      self.assertEqual(dsds[2,2,1,2],0.0,"ds/ds has wrong value at (2,2,1,2).")
      self.assertEqual(dsds[2,2,1,3],0.0,"ds/ds has wrong value at (2,2,1,3).")
      self.assertEqual(dsds[2,2,1,4],0.0,"ds/ds has wrong value at (2,2,1,4).")
      self.assertEqual(dsds[2,2,2,0],0.0,"ds/ds has wrong value at (2,2,2,0).")
      self.assertEqual(dsds[2,2,2,1],0.0,"ds/ds has wrong value at (2,2,2,1).")
      self.assertEqual(dsds[2,2,2,2],1.0,"ds/ds has wrong value at (2,2,2,2).")
      self.assertEqual(dsds[2,2,2,3],0.0,"ds/ds has wrong value at (2,2,2,3).")
      self.assertEqual(dsds[2,2,2,4],0.0,"ds/ds has wrong value at (2,2,2,4).")
      self.assertEqual(dsds[2,2,3,0],0.0,"ds/ds has wrong value at (2,2,3,0).")
      self.assertEqual(dsds[2,2,3,1],0.0,"ds/ds has wrong value at (2,2,3,1).")
      self.assertEqual(dsds[2,2,3,2],0.0,"ds/ds has wrong value at (2,2,3,2).")
      self.assertEqual(dsds[2,2,3,3],0.0,"ds/ds has wrong value at (2,2,3,3).")
      self.assertEqual(dsds[2,2,3,4],0.0,"ds/ds has wrong value at (2,2,3,4).")
      self.assertEqual(dsds[2,3,0,0],0.0,"ds/ds has wrong value at (2,3,0,0).")
      self.assertEqual(dsds[2,3,0,1],0.0,"ds/ds has wrong value at (2,3,0,1).")
      self.assertEqual(dsds[2,3,0,2],0.0,"ds/ds has wrong value at (2,3,0,2).")
      self.assertEqual(dsds[2,3,0,3],0.0,"ds/ds has wrong value at (2,3,0,3).")
      self.assertEqual(dsds[2,3,0,4],0.0,"ds/ds has wrong value at (2,3,0,4).")
      self.assertEqual(dsds[2,3,1,0],0.0,"ds/ds has wrong value at (2,3,1,0).")
      self.assertEqual(dsds[2,3,1,1],0.0,"ds/ds has wrong value at (2,3,1,1).")
      self.assertEqual(dsds[2,3,1,2],0.0,"ds/ds has wrong value at (2,3,1,2).")
      self.assertEqual(dsds[2,3,1,3],0.0,"ds/ds has wrong value at (2,3,1,3).")
      self.assertEqual(dsds[2,3,1,4],0.0,"ds/ds has wrong value at (2,3,1,4).")
      self.assertEqual(dsds[2,3,2,0],0.0,"ds/ds has wrong value at (2,3,2,0).")
      self.assertEqual(dsds[2,3,2,1],0.0,"ds/ds has wrong value at (2,3,2,1).")
      self.assertEqual(dsds[2,3,2,2],0.0,"ds/ds has wrong value at (2,3,2,2).")
      self.assertEqual(dsds[2,3,2,3],1.0,"ds/ds has wrong value at (2,3,2,3).")
      self.assertEqual(dsds[2,3,2,4],0.0,"ds/ds has wrong value at (2,3,2,4).")
      self.assertEqual(dsds[2,3,3,0],0.0,"ds/ds has wrong value at (2,3,3,0).")
      self.assertEqual(dsds[2,3,3,1],0.0,"ds/ds has wrong value at (2,3,3,1).")
      self.assertEqual(dsds[2,3,3,2],0.0,"ds/ds has wrong value at (2,3,3,2).")
      self.assertEqual(dsds[2,3,3,3],0.0,"ds/ds has wrong value at (2,3,3,3).")
      self.assertEqual(dsds[2,3,3,4],0.0,"ds/ds has wrong value at (2,3,3,4).")
      self.assertEqual(dsds[2,4,0,0],0.0,"ds/ds has wrong value at (2,4,0,0).")
      self.assertEqual(dsds[2,4,0,1],0.0,"ds/ds has wrong value at (2,4,0,1).")
      self.assertEqual(dsds[2,4,0,2],0.0,"ds/ds has wrong value at (2,4,0,2).")
      self.assertEqual(dsds[2,4,0,3],0.0,"ds/ds has wrong value at (2,4,0,3).")
      self.assertEqual(dsds[2,4,0,4],0.0,"ds/ds has wrong value at (2,4,0,4).")
      self.assertEqual(dsds[2,4,1,0],0.0,"ds/ds has wrong value at (2,4,1,0).")
      self.assertEqual(dsds[2,4,1,1],0.0,"ds/ds has wrong value at (2,4,1,1).")
      self.assertEqual(dsds[2,4,1,2],0.0,"ds/ds has wrong value at (2,4,1,2).")
      self.assertEqual(dsds[2,4,1,3],0.0,"ds/ds has wrong value at (2,4,1,3).")
      self.assertEqual(dsds[2,4,1,4],0.0,"ds/ds has wrong value at (2,4,1,4).")
      self.assertEqual(dsds[2,4,2,0],0.0,"ds/ds has wrong value at (2,4,2,0).")
      self.assertEqual(dsds[2,4,2,1],0.0,"ds/ds has wrong value at (2,4,2,1).")
      self.assertEqual(dsds[2,4,2,2],0.0,"ds/ds has wrong value at (2,4,2,2).")
      self.assertEqual(dsds[2,4,2,3],0.0,"ds/ds has wrong value at (2,4,2,3).")
      self.assertEqual(dsds[2,4,2,4],1.0,"ds/ds has wrong value at (2,4,2,4).")
      self.assertEqual(dsds[2,4,3,0],0.0,"ds/ds has wrong value at (2,4,3,0).")
      self.assertEqual(dsds[2,4,3,1],0.0,"ds/ds has wrong value at (2,4,3,1).")
      self.assertEqual(dsds[2,4,3,2],0.0,"ds/ds has wrong value at (2,4,3,2).")
      self.assertEqual(dsds[2,4,3,3],0.0,"ds/ds has wrong value at (2,4,3,3).")
      self.assertEqual(dsds[2,4,3,4],0.0,"ds/ds has wrong value at (2,4,3,4).")
      self.assertEqual(dsds[3,0,0,0],0.0,"ds/ds has wrong value at (3,0,0,0).")
      self.assertEqual(dsds[3,0,0,1],0.0,"ds/ds has wrong value at (3,0,0,1).")
      self.assertEqual(dsds[3,0,0,2],0.0,"ds/ds has wrong value at (3,0,0,2).")
      self.assertEqual(dsds[3,0,0,3],0.0,"ds/ds has wrong value at (3,0,0,3).")
      self.assertEqual(dsds[3,0,0,4],0.0,"ds/ds has wrong value at (3,0,0,4).")
      self.assertEqual(dsds[3,0,1,0],0.0,"ds/ds has wrong value at (3,0,1,0).")
      self.assertEqual(dsds[3,0,1,1],0.0,"ds/ds has wrong value at (3,0,1,1).")
      self.assertEqual(dsds[3,0,1,2],0.0,"ds/ds has wrong value at (3,0,1,2).")
      self.assertEqual(dsds[3,0,1,3],0.0,"ds/ds has wrong value at (3,0,1,3).")
      self.assertEqual(dsds[3,0,1,4],0.0,"ds/ds has wrong value at (3,0,1,4).")
      self.assertEqual(dsds[3,0,2,0],0.0,"ds/ds has wrong value at (3,0,2,0).")
      self.assertEqual(dsds[3,0,2,1],0.0,"ds/ds has wrong value at (3,0,2,1).")
      self.assertEqual(dsds[3,0,2,2],0.0,"ds/ds has wrong value at (3,0,2,2).")
      self.assertEqual(dsds[3,0,2,3],0.0,"ds/ds has wrong value at (3,0,2,3).")
      self.assertEqual(dsds[3,0,2,4],0.0,"ds/ds has wrong value at (3,0,2,4).")
      self.assertEqual(dsds[3,0,3,0],1.0,"ds/ds has wrong value at (3,0,3,0).")
      self.assertEqual(dsds[3,0,3,1],0.0,"ds/ds has wrong value at (3,0,3,1).")
      self.assertEqual(dsds[3,0,3,2],0.0,"ds/ds has wrong value at (3,0,3,2).")
      self.assertEqual(dsds[3,0,3,3],0.0,"ds/ds has wrong value at (3,0,3,3).")
      self.assertEqual(dsds[3,0,3,4],0.0,"ds/ds has wrong value at (3,0,3,4).")
      self.assertEqual(dsds[3,1,0,0],0.0,"ds/ds has wrong value at (3,1,0,0).")
      self.assertEqual(dsds[3,1,0,1],0.0,"ds/ds has wrong value at (3,1,0,1).")
      self.assertEqual(dsds[3,1,0,2],0.0,"ds/ds has wrong value at (3,1,0,2).")
      self.assertEqual(dsds[3,1,0,3],0.0,"ds/ds has wrong value at (3,1,0,3).")
      self.assertEqual(dsds[3,1,0,4],0.0,"ds/ds has wrong value at (3,1,0,4).")
      self.assertEqual(dsds[3,1,1,0],0.0,"ds/ds has wrong value at (3,1,1,0).")
      self.assertEqual(dsds[3,1,1,1],0.0,"ds/ds has wrong value at (3,1,1,1).")
      self.assertEqual(dsds[3,1,1,2],0.0,"ds/ds has wrong value at (3,1,1,2).")
      self.assertEqual(dsds[3,1,1,3],0.0,"ds/ds has wrong value at (3,1,1,3).")
      self.assertEqual(dsds[3,1,1,4],0.0,"ds/ds has wrong value at (3,1,1,4).")
      self.assertEqual(dsds[3,1,2,0],0.0,"ds/ds has wrong value at (3,1,2,0).")
      self.assertEqual(dsds[3,1,2,1],0.0,"ds/ds has wrong value at (3,1,2,1).")
      self.assertEqual(dsds[3,1,2,2],0.0,"ds/ds has wrong value at (3,1,2,2).")
      self.assertEqual(dsds[3,1,2,3],0.0,"ds/ds has wrong value at (3,1,2,3).")
      self.assertEqual(dsds[3,1,2,4],0.0,"ds/ds has wrong value at (3,1,2,4).")
      self.assertEqual(dsds[3,1,3,0],0.0,"ds/ds has wrong value at (3,1,3,0).")
      self.assertEqual(dsds[3,1,3,1],1.0,"ds/ds has wrong value at (3,1,3,1).")
      self.assertEqual(dsds[3,1,3,2],0.0,"ds/ds has wrong value at (3,1,3,2).")
      self.assertEqual(dsds[3,1,3,3],0.0,"ds/ds has wrong value at (3,1,3,3).")
      self.assertEqual(dsds[3,1,3,4],0.0,"ds/ds has wrong value at (3,1,3,4).")
      self.assertEqual(dsds[3,2,0,0],0.0,"ds/ds has wrong value at (3,2,0,0).")
      self.assertEqual(dsds[3,2,0,1],0.0,"ds/ds has wrong value at (3,2,0,1).")
      self.assertEqual(dsds[3,2,0,2],0.0,"ds/ds has wrong value at (3,2,0,2).")
      self.assertEqual(dsds[3,2,0,3],0.0,"ds/ds has wrong value at (3,2,0,3).")
      self.assertEqual(dsds[3,2,0,4],0.0,"ds/ds has wrong value at (3,2,0,4).")
      self.assertEqual(dsds[3,2,1,0],0.0,"ds/ds has wrong value at (3,2,1,0).")
      self.assertEqual(dsds[3,2,1,1],0.0,"ds/ds has wrong value at (3,2,1,1).")
      self.assertEqual(dsds[3,2,1,2],0.0,"ds/ds has wrong value at (3,2,1,2).")
      self.assertEqual(dsds[3,2,1,3],0.0,"ds/ds has wrong value at (3,2,1,3).")
      self.assertEqual(dsds[3,2,1,4],0.0,"ds/ds has wrong value at (3,2,1,4).")
      self.assertEqual(dsds[3,2,2,0],0.0,"ds/ds has wrong value at (3,2,2,0).")
      self.assertEqual(dsds[3,2,2,1],0.0,"ds/ds has wrong value at (3,2,2,1).")
      self.assertEqual(dsds[3,2,2,2],0.0,"ds/ds has wrong value at (3,2,2,2).")
      self.assertEqual(dsds[3,2,2,3],0.0,"ds/ds has wrong value at (3,2,2,3).")
      self.assertEqual(dsds[3,2,2,4],0.0,"ds/ds has wrong value at (3,2,2,4).")
      self.assertEqual(dsds[3,2,3,0],0.0,"ds/ds has wrong value at (3,2,3,0).")
      self.assertEqual(dsds[3,2,3,1],0.0,"ds/ds has wrong value at (3,2,3,1).")
      self.assertEqual(dsds[3,2,3,2],1.0,"ds/ds has wrong value at (3,2,3,2).")
      self.assertEqual(dsds[3,2,3,3],0.0,"ds/ds has wrong value at (3,2,3,3).")
      self.assertEqual(dsds[3,2,3,4],0.0,"ds/ds has wrong value at (3,2,3,4).")
      self.assertEqual(dsds[3,3,0,0],0.0,"ds/ds has wrong value at (3,3,0,0).")
      self.assertEqual(dsds[3,3,0,1],0.0,"ds/ds has wrong value at (3,3,0,1).")
      self.assertEqual(dsds[3,3,0,2],0.0,"ds/ds has wrong value at (3,3,0,2).")
      self.assertEqual(dsds[3,3,0,3],0.0,"ds/ds has wrong value at (3,3,0,3).")
      self.assertEqual(dsds[3,3,0,4],0.0,"ds/ds has wrong value at (3,3,0,4).")
      self.assertEqual(dsds[3,3,1,0],0.0,"ds/ds has wrong value at (3,3,1,0).")
      self.assertEqual(dsds[3,3,1,1],0.0,"ds/ds has wrong value at (3,3,1,1).")
      self.assertEqual(dsds[3,3,1,2],0.0,"ds/ds has wrong value at (3,3,1,2).")
      self.assertEqual(dsds[3,3,1,3],0.0,"ds/ds has wrong value at (3,3,1,3).")
      self.assertEqual(dsds[3,3,1,4],0.0,"ds/ds has wrong value at (3,3,1,4).")
      self.assertEqual(dsds[3,3,2,0],0.0,"ds/ds has wrong value at (3,3,2,0).")
      self.assertEqual(dsds[3,3,2,1],0.0,"ds/ds has wrong value at (3,3,2,1).")
      self.assertEqual(dsds[3,3,2,2],0.0,"ds/ds has wrong value at (3,3,2,2).")
      self.assertEqual(dsds[3,3,2,3],0.0,"ds/ds has wrong value at (3,3,2,3).")
      self.assertEqual(dsds[3,3,2,4],0.0,"ds/ds has wrong value at (3,3,2,4).")
      self.assertEqual(dsds[3,3,3,0],0.0,"ds/ds has wrong value at (3,3,3,0).")
      self.assertEqual(dsds[3,3,3,1],0.0,"ds/ds has wrong value at (3,3,3,1).")
      self.assertEqual(dsds[3,3,3,2],0.0,"ds/ds has wrong value at (3,3,3,2).")
      self.assertEqual(dsds[3,3,3,3],1.0,"ds/ds has wrong value at (3,3,3,3).")
      self.assertEqual(dsds[3,3,3,4],0.0,"ds/ds has wrong value at (3,3,3,4).")
      self.assertEqual(dsds[3,4,0,0],0.0,"ds/ds has wrong value at (3,4,0,0).")
      self.assertEqual(dsds[3,4,0,1],0.0,"ds/ds has wrong value at (3,4,0,1).")
      self.assertEqual(dsds[3,4,0,2],0.0,"ds/ds has wrong value at (3,4,0,2).")
      self.assertEqual(dsds[3,4,0,3],0.0,"ds/ds has wrong value at (3,4,0,3).")
      self.assertEqual(dsds[3,4,0,4],0.0,"ds/ds has wrong value at (3,4,0,4).")
      self.assertEqual(dsds[3,4,1,0],0.0,"ds/ds has wrong value at (3,4,1,0).")
      self.assertEqual(dsds[3,4,1,1],0.0,"ds/ds has wrong value at (3,4,1,1).")
      self.assertEqual(dsds[3,4,1,2],0.0,"ds/ds has wrong value at (3,4,1,2).")
      self.assertEqual(dsds[3,4,1,3],0.0,"ds/ds has wrong value at (3,4,1,3).")
      self.assertEqual(dsds[3,4,1,4],0.0,"ds/ds has wrong value at (3,4,1,4).")
      self.assertEqual(dsds[3,4,2,0],0.0,"ds/ds has wrong value at (3,4,2,0).")
      self.assertEqual(dsds[3,4,2,1],0.0,"ds/ds has wrong value at (3,4,2,1).")
      self.assertEqual(dsds[3,4,2,2],0.0,"ds/ds has wrong value at (3,4,2,2).")
      self.assertEqual(dsds[3,4,2,3],0.0,"ds/ds has wrong value at (3,4,2,3).")
      self.assertEqual(dsds[3,4,2,4],0.0,"ds/ds has wrong value at (3,4,2,4).")
      self.assertEqual(dsds[3,4,3,0],0.0,"ds/ds has wrong value at (3,4,3,0).")
      self.assertEqual(dsds[3,4,3,1],0.0,"ds/ds has wrong value at (3,4,3,1).")
      self.assertEqual(dsds[3,4,3,2],0.0,"ds/ds has wrong value at (3,4,3,2).")
      self.assertEqual(dsds[3,4,3,3],0.0,"ds/ds has wrong value at (3,4,3,3).")
      self.assertEqual(dsds[3,4,3,4],1.0,"ds/ds has wrong value at (3,4,3,4).")

      self.assertEqual(s.getArgument(0),1,"wrong argument 0.")
      self.assertEqual(s.getArgument(1),-1.0,"wrong argument 1.")
      sa=s.getSubstitutedArguments({s2:-10})
      self.assertEqual(len(sa),2,"wrong number of substituted arguments")
      self.assertEqual(sa[0],1,"wrongly substituted argument 0.")
      self.assertEqual(sa[1],-1.0,"wrongly substituted argument 1.")

      self.assertFalse(s.isAppropriateValue(10.)," 10. is not appropriate substitute")
      self.assertFalse(s.isAppropriateValue(10)," 10 is not appropriate substitute")
      self.assertTrue(s.isAppropriateValue(Symbol(shape=(4, 5),dim=None)),"Symbol is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(Symbol(shape=(5,))),"Symbol is not appropriate substitute (shape)")
      self.assertFalse(s.isAppropriateValue(escript.Data(0.,(5,),self.functionspace)),"Data is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(numpy.zeros((4, 5))),"ndarray is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(numpy.zeros((5,))),"ndarray is not appropriate substitute (shape)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_Symbol_rank2_dd_nargs0(self):
      s2=Symbol(shape=())
      s=Symbol(shape=(4, 5),dim=self.functionspace.getDim(),args=[])
      self.assertEqual(s.getRank(),2,"wrong rank.")
      self.assertEqual(s.getShape(),(4, 5),"wrong shape.")
      self.assertEqual(s.getDim(),self.functionspace.getDim(),"wrong spatial dimension.")
      self.assertEqual(s.getArgument(),[],"wrong arguments.")

      ss=s.substitute({s:numpy.zeros((4, 5))})
      self.assertTrue(isinstance(ss,numpy.ndarray),"value after substitution is not ndarray.")
      self.assertEqual(ss.shape,(4, 5),"value after substitution has not expected shape")
      try:
        s.substitute({s:numpy.zeros((5,))})
        fail("illegal substition was successful")
      except TypeError:
        pass

      dsdarg=s.diff(Symbol(shape=()))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds() has wrong type.")
      self.assertEqual(dsdarg.shape,(4, 5),"ds/ds() has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds() has wrong value.")
      dsdarg=s.diff(Symbol(shape=(2,)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(2,) has wrong type.")
      self.assertEqual(dsdarg.shape,(4, 5, 2),"ds/ds(2,) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(2,) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(4, 5)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(4, 5) has wrong type.")
      self.assertEqual(dsdarg.shape,(4, 5, 4, 5),"ds/ds(4, 5) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(4, 5) has wrong value.")

      dsds=s.diff(s)
      self.assertTrue(isinstance(dsds,numpy.ndarray),"ds/ds has wrong type.")
      self.assertEqual(dsds.shape,(4, 5, 4, 5),"ds/ds has wrong shape.")
      self.assertEqual(dsds[0,0,0,0],1.0,"ds/ds has wrong value at (0,0,0,0).")
      self.assertEqual(dsds[0,0,0,1],0.0,"ds/ds has wrong value at (0,0,0,1).")
      self.assertEqual(dsds[0,0,0,2],0.0,"ds/ds has wrong value at (0,0,0,2).")
      self.assertEqual(dsds[0,0,0,3],0.0,"ds/ds has wrong value at (0,0,0,3).")
      self.assertEqual(dsds[0,0,0,4],0.0,"ds/ds has wrong value at (0,0,0,4).")
      self.assertEqual(dsds[0,0,1,0],0.0,"ds/ds has wrong value at (0,0,1,0).")
      self.assertEqual(dsds[0,0,1,1],0.0,"ds/ds has wrong value at (0,0,1,1).")
      self.assertEqual(dsds[0,0,1,2],0.0,"ds/ds has wrong value at (0,0,1,2).")
      self.assertEqual(dsds[0,0,1,3],0.0,"ds/ds has wrong value at (0,0,1,3).")
      self.assertEqual(dsds[0,0,1,4],0.0,"ds/ds has wrong value at (0,0,1,4).")
      self.assertEqual(dsds[0,0,2,0],0.0,"ds/ds has wrong value at (0,0,2,0).")
      self.assertEqual(dsds[0,0,2,1],0.0,"ds/ds has wrong value at (0,0,2,1).")
      self.assertEqual(dsds[0,0,2,2],0.0,"ds/ds has wrong value at (0,0,2,2).")
      self.assertEqual(dsds[0,0,2,3],0.0,"ds/ds has wrong value at (0,0,2,3).")
      self.assertEqual(dsds[0,0,2,4],0.0,"ds/ds has wrong value at (0,0,2,4).")
      self.assertEqual(dsds[0,0,3,0],0.0,"ds/ds has wrong value at (0,0,3,0).")
      self.assertEqual(dsds[0,0,3,1],0.0,"ds/ds has wrong value at (0,0,3,1).")
      self.assertEqual(dsds[0,0,3,2],0.0,"ds/ds has wrong value at (0,0,3,2).")
      self.assertEqual(dsds[0,0,3,3],0.0,"ds/ds has wrong value at (0,0,3,3).")
      self.assertEqual(dsds[0,0,3,4],0.0,"ds/ds has wrong value at (0,0,3,4).")
      self.assertEqual(dsds[0,1,0,0],0.0,"ds/ds has wrong value at (0,1,0,0).")
      self.assertEqual(dsds[0,1,0,1],1.0,"ds/ds has wrong value at (0,1,0,1).")
      self.assertEqual(dsds[0,1,0,2],0.0,"ds/ds has wrong value at (0,1,0,2).")
      self.assertEqual(dsds[0,1,0,3],0.0,"ds/ds has wrong value at (0,1,0,3).")
      self.assertEqual(dsds[0,1,0,4],0.0,"ds/ds has wrong value at (0,1,0,4).")
      self.assertEqual(dsds[0,1,1,0],0.0,"ds/ds has wrong value at (0,1,1,0).")
      self.assertEqual(dsds[0,1,1,1],0.0,"ds/ds has wrong value at (0,1,1,1).")
      self.assertEqual(dsds[0,1,1,2],0.0,"ds/ds has wrong value at (0,1,1,2).")
      self.assertEqual(dsds[0,1,1,3],0.0,"ds/ds has wrong value at (0,1,1,3).")
      self.assertEqual(dsds[0,1,1,4],0.0,"ds/ds has wrong value at (0,1,1,4).")
      self.assertEqual(dsds[0,1,2,0],0.0,"ds/ds has wrong value at (0,1,2,0).")
      self.assertEqual(dsds[0,1,2,1],0.0,"ds/ds has wrong value at (0,1,2,1).")
      self.assertEqual(dsds[0,1,2,2],0.0,"ds/ds has wrong value at (0,1,2,2).")
      self.assertEqual(dsds[0,1,2,3],0.0,"ds/ds has wrong value at (0,1,2,3).")
      self.assertEqual(dsds[0,1,2,4],0.0,"ds/ds has wrong value at (0,1,2,4).")
      self.assertEqual(dsds[0,1,3,0],0.0,"ds/ds has wrong value at (0,1,3,0).")
      self.assertEqual(dsds[0,1,3,1],0.0,"ds/ds has wrong value at (0,1,3,1).")
      self.assertEqual(dsds[0,1,3,2],0.0,"ds/ds has wrong value at (0,1,3,2).")
      self.assertEqual(dsds[0,1,3,3],0.0,"ds/ds has wrong value at (0,1,3,3).")
      self.assertEqual(dsds[0,1,3,4],0.0,"ds/ds has wrong value at (0,1,3,4).")
      self.assertEqual(dsds[0,2,0,0],0.0,"ds/ds has wrong value at (0,2,0,0).")
      self.assertEqual(dsds[0,2,0,1],0.0,"ds/ds has wrong value at (0,2,0,1).")
      self.assertEqual(dsds[0,2,0,2],1.0,"ds/ds has wrong value at (0,2,0,2).")
      self.assertEqual(dsds[0,2,0,3],0.0,"ds/ds has wrong value at (0,2,0,3).")
      self.assertEqual(dsds[0,2,0,4],0.0,"ds/ds has wrong value at (0,2,0,4).")
      self.assertEqual(dsds[0,2,1,0],0.0,"ds/ds has wrong value at (0,2,1,0).")
      self.assertEqual(dsds[0,2,1,1],0.0,"ds/ds has wrong value at (0,2,1,1).")
      self.assertEqual(dsds[0,2,1,2],0.0,"ds/ds has wrong value at (0,2,1,2).")
      self.assertEqual(dsds[0,2,1,3],0.0,"ds/ds has wrong value at (0,2,1,3).")
      self.assertEqual(dsds[0,2,1,4],0.0,"ds/ds has wrong value at (0,2,1,4).")
      self.assertEqual(dsds[0,2,2,0],0.0,"ds/ds has wrong value at (0,2,2,0).")
      self.assertEqual(dsds[0,2,2,1],0.0,"ds/ds has wrong value at (0,2,2,1).")
      self.assertEqual(dsds[0,2,2,2],0.0,"ds/ds has wrong value at (0,2,2,2).")
      self.assertEqual(dsds[0,2,2,3],0.0,"ds/ds has wrong value at (0,2,2,3).")
      self.assertEqual(dsds[0,2,2,4],0.0,"ds/ds has wrong value at (0,2,2,4).")
      self.assertEqual(dsds[0,2,3,0],0.0,"ds/ds has wrong value at (0,2,3,0).")
      self.assertEqual(dsds[0,2,3,1],0.0,"ds/ds has wrong value at (0,2,3,1).")
      self.assertEqual(dsds[0,2,3,2],0.0,"ds/ds has wrong value at (0,2,3,2).")
      self.assertEqual(dsds[0,2,3,3],0.0,"ds/ds has wrong value at (0,2,3,3).")
      self.assertEqual(dsds[0,2,3,4],0.0,"ds/ds has wrong value at (0,2,3,4).")
      self.assertEqual(dsds[0,3,0,0],0.0,"ds/ds has wrong value at (0,3,0,0).")
      self.assertEqual(dsds[0,3,0,1],0.0,"ds/ds has wrong value at (0,3,0,1).")
      self.assertEqual(dsds[0,3,0,2],0.0,"ds/ds has wrong value at (0,3,0,2).")
      self.assertEqual(dsds[0,3,0,3],1.0,"ds/ds has wrong value at (0,3,0,3).")
      self.assertEqual(dsds[0,3,0,4],0.0,"ds/ds has wrong value at (0,3,0,4).")
      self.assertEqual(dsds[0,3,1,0],0.0,"ds/ds has wrong value at (0,3,1,0).")
      self.assertEqual(dsds[0,3,1,1],0.0,"ds/ds has wrong value at (0,3,1,1).")
      self.assertEqual(dsds[0,3,1,2],0.0,"ds/ds has wrong value at (0,3,1,2).")
      self.assertEqual(dsds[0,3,1,3],0.0,"ds/ds has wrong value at (0,3,1,3).")
      self.assertEqual(dsds[0,3,1,4],0.0,"ds/ds has wrong value at (0,3,1,4).")
      self.assertEqual(dsds[0,3,2,0],0.0,"ds/ds has wrong value at (0,3,2,0).")
      self.assertEqual(dsds[0,3,2,1],0.0,"ds/ds has wrong value at (0,3,2,1).")
      self.assertEqual(dsds[0,3,2,2],0.0,"ds/ds has wrong value at (0,3,2,2).")
      self.assertEqual(dsds[0,3,2,3],0.0,"ds/ds has wrong value at (0,3,2,3).")
      self.assertEqual(dsds[0,3,2,4],0.0,"ds/ds has wrong value at (0,3,2,4).")
      self.assertEqual(dsds[0,3,3,0],0.0,"ds/ds has wrong value at (0,3,3,0).")
      self.assertEqual(dsds[0,3,3,1],0.0,"ds/ds has wrong value at (0,3,3,1).")
      self.assertEqual(dsds[0,3,3,2],0.0,"ds/ds has wrong value at (0,3,3,2).")
      self.assertEqual(dsds[0,3,3,3],0.0,"ds/ds has wrong value at (0,3,3,3).")
      self.assertEqual(dsds[0,3,3,4],0.0,"ds/ds has wrong value at (0,3,3,4).")
      self.assertEqual(dsds[0,4,0,0],0.0,"ds/ds has wrong value at (0,4,0,0).")
      self.assertEqual(dsds[0,4,0,1],0.0,"ds/ds has wrong value at (0,4,0,1).")
      self.assertEqual(dsds[0,4,0,2],0.0,"ds/ds has wrong value at (0,4,0,2).")
      self.assertEqual(dsds[0,4,0,3],0.0,"ds/ds has wrong value at (0,4,0,3).")
      self.assertEqual(dsds[0,4,0,4],1.0,"ds/ds has wrong value at (0,4,0,4).")
      self.assertEqual(dsds[0,4,1,0],0.0,"ds/ds has wrong value at (0,4,1,0).")
      self.assertEqual(dsds[0,4,1,1],0.0,"ds/ds has wrong value at (0,4,1,1).")
      self.assertEqual(dsds[0,4,1,2],0.0,"ds/ds has wrong value at (0,4,1,2).")
      self.assertEqual(dsds[0,4,1,3],0.0,"ds/ds has wrong value at (0,4,1,3).")
      self.assertEqual(dsds[0,4,1,4],0.0,"ds/ds has wrong value at (0,4,1,4).")
      self.assertEqual(dsds[0,4,2,0],0.0,"ds/ds has wrong value at (0,4,2,0).")
      self.assertEqual(dsds[0,4,2,1],0.0,"ds/ds has wrong value at (0,4,2,1).")
      self.assertEqual(dsds[0,4,2,2],0.0,"ds/ds has wrong value at (0,4,2,2).")
      self.assertEqual(dsds[0,4,2,3],0.0,"ds/ds has wrong value at (0,4,2,3).")
      self.assertEqual(dsds[0,4,2,4],0.0,"ds/ds has wrong value at (0,4,2,4).")
      self.assertEqual(dsds[0,4,3,0],0.0,"ds/ds has wrong value at (0,4,3,0).")
      self.assertEqual(dsds[0,4,3,1],0.0,"ds/ds has wrong value at (0,4,3,1).")
      self.assertEqual(dsds[0,4,3,2],0.0,"ds/ds has wrong value at (0,4,3,2).")
      self.assertEqual(dsds[0,4,3,3],0.0,"ds/ds has wrong value at (0,4,3,3).")
      self.assertEqual(dsds[0,4,3,4],0.0,"ds/ds has wrong value at (0,4,3,4).")
      self.assertEqual(dsds[1,0,0,0],0.0,"ds/ds has wrong value at (1,0,0,0).")
      self.assertEqual(dsds[1,0,0,1],0.0,"ds/ds has wrong value at (1,0,0,1).")
      self.assertEqual(dsds[1,0,0,2],0.0,"ds/ds has wrong value at (1,0,0,2).")
      self.assertEqual(dsds[1,0,0,3],0.0,"ds/ds has wrong value at (1,0,0,3).")
      self.assertEqual(dsds[1,0,0,4],0.0,"ds/ds has wrong value at (1,0,0,4).")
      self.assertEqual(dsds[1,0,1,0],1.0,"ds/ds has wrong value at (1,0,1,0).")
      self.assertEqual(dsds[1,0,1,1],0.0,"ds/ds has wrong value at (1,0,1,1).")
      self.assertEqual(dsds[1,0,1,2],0.0,"ds/ds has wrong value at (1,0,1,2).")
      self.assertEqual(dsds[1,0,1,3],0.0,"ds/ds has wrong value at (1,0,1,3).")
      self.assertEqual(dsds[1,0,1,4],0.0,"ds/ds has wrong value at (1,0,1,4).")
      self.assertEqual(dsds[1,0,2,0],0.0,"ds/ds has wrong value at (1,0,2,0).")
      self.assertEqual(dsds[1,0,2,1],0.0,"ds/ds has wrong value at (1,0,2,1).")
      self.assertEqual(dsds[1,0,2,2],0.0,"ds/ds has wrong value at (1,0,2,2).")
      self.assertEqual(dsds[1,0,2,3],0.0,"ds/ds has wrong value at (1,0,2,3).")
      self.assertEqual(dsds[1,0,2,4],0.0,"ds/ds has wrong value at (1,0,2,4).")
      self.assertEqual(dsds[1,0,3,0],0.0,"ds/ds has wrong value at (1,0,3,0).")
      self.assertEqual(dsds[1,0,3,1],0.0,"ds/ds has wrong value at (1,0,3,1).")
      self.assertEqual(dsds[1,0,3,2],0.0,"ds/ds has wrong value at (1,0,3,2).")
      self.assertEqual(dsds[1,0,3,3],0.0,"ds/ds has wrong value at (1,0,3,3).")
      self.assertEqual(dsds[1,0,3,4],0.0,"ds/ds has wrong value at (1,0,3,4).")
      self.assertEqual(dsds[1,1,0,0],0.0,"ds/ds has wrong value at (1,1,0,0).")
      self.assertEqual(dsds[1,1,0,1],0.0,"ds/ds has wrong value at (1,1,0,1).")
      self.assertEqual(dsds[1,1,0,2],0.0,"ds/ds has wrong value at (1,1,0,2).")
      self.assertEqual(dsds[1,1,0,3],0.0,"ds/ds has wrong value at (1,1,0,3).")
      self.assertEqual(dsds[1,1,0,4],0.0,"ds/ds has wrong value at (1,1,0,4).")
      self.assertEqual(dsds[1,1,1,0],0.0,"ds/ds has wrong value at (1,1,1,0).")
      self.assertEqual(dsds[1,1,1,1],1.0,"ds/ds has wrong value at (1,1,1,1).")
      self.assertEqual(dsds[1,1,1,2],0.0,"ds/ds has wrong value at (1,1,1,2).")
      self.assertEqual(dsds[1,1,1,3],0.0,"ds/ds has wrong value at (1,1,1,3).")
      self.assertEqual(dsds[1,1,1,4],0.0,"ds/ds has wrong value at (1,1,1,4).")
      self.assertEqual(dsds[1,1,2,0],0.0,"ds/ds has wrong value at (1,1,2,0).")
      self.assertEqual(dsds[1,1,2,1],0.0,"ds/ds has wrong value at (1,1,2,1).")
      self.assertEqual(dsds[1,1,2,2],0.0,"ds/ds has wrong value at (1,1,2,2).")
      self.assertEqual(dsds[1,1,2,3],0.0,"ds/ds has wrong value at (1,1,2,3).")
      self.assertEqual(dsds[1,1,2,4],0.0,"ds/ds has wrong value at (1,1,2,4).")
      self.assertEqual(dsds[1,1,3,0],0.0,"ds/ds has wrong value at (1,1,3,0).")
      self.assertEqual(dsds[1,1,3,1],0.0,"ds/ds has wrong value at (1,1,3,1).")
      self.assertEqual(dsds[1,1,3,2],0.0,"ds/ds has wrong value at (1,1,3,2).")
      self.assertEqual(dsds[1,1,3,3],0.0,"ds/ds has wrong value at (1,1,3,3).")
      self.assertEqual(dsds[1,1,3,4],0.0,"ds/ds has wrong value at (1,1,3,4).")
      self.assertEqual(dsds[1,2,0,0],0.0,"ds/ds has wrong value at (1,2,0,0).")
      self.assertEqual(dsds[1,2,0,1],0.0,"ds/ds has wrong value at (1,2,0,1).")
      self.assertEqual(dsds[1,2,0,2],0.0,"ds/ds has wrong value at (1,2,0,2).")
      self.assertEqual(dsds[1,2,0,3],0.0,"ds/ds has wrong value at (1,2,0,3).")
      self.assertEqual(dsds[1,2,0,4],0.0,"ds/ds has wrong value at (1,2,0,4).")
      self.assertEqual(dsds[1,2,1,0],0.0,"ds/ds has wrong value at (1,2,1,0).")
      self.assertEqual(dsds[1,2,1,1],0.0,"ds/ds has wrong value at (1,2,1,1).")
      self.assertEqual(dsds[1,2,1,2],1.0,"ds/ds has wrong value at (1,2,1,2).")
      self.assertEqual(dsds[1,2,1,3],0.0,"ds/ds has wrong value at (1,2,1,3).")
      self.assertEqual(dsds[1,2,1,4],0.0,"ds/ds has wrong value at (1,2,1,4).")
      self.assertEqual(dsds[1,2,2,0],0.0,"ds/ds has wrong value at (1,2,2,0).")
      self.assertEqual(dsds[1,2,2,1],0.0,"ds/ds has wrong value at (1,2,2,1).")
      self.assertEqual(dsds[1,2,2,2],0.0,"ds/ds has wrong value at (1,2,2,2).")
      self.assertEqual(dsds[1,2,2,3],0.0,"ds/ds has wrong value at (1,2,2,3).")
      self.assertEqual(dsds[1,2,2,4],0.0,"ds/ds has wrong value at (1,2,2,4).")
      self.assertEqual(dsds[1,2,3,0],0.0,"ds/ds has wrong value at (1,2,3,0).")
      self.assertEqual(dsds[1,2,3,1],0.0,"ds/ds has wrong value at (1,2,3,1).")
      self.assertEqual(dsds[1,2,3,2],0.0,"ds/ds has wrong value at (1,2,3,2).")
      self.assertEqual(dsds[1,2,3,3],0.0,"ds/ds has wrong value at (1,2,3,3).")
      self.assertEqual(dsds[1,2,3,4],0.0,"ds/ds has wrong value at (1,2,3,4).")
      self.assertEqual(dsds[1,3,0,0],0.0,"ds/ds has wrong value at (1,3,0,0).")
      self.assertEqual(dsds[1,3,0,1],0.0,"ds/ds has wrong value at (1,3,0,1).")
      self.assertEqual(dsds[1,3,0,2],0.0,"ds/ds has wrong value at (1,3,0,2).")
      self.assertEqual(dsds[1,3,0,3],0.0,"ds/ds has wrong value at (1,3,0,3).")
      self.assertEqual(dsds[1,3,0,4],0.0,"ds/ds has wrong value at (1,3,0,4).")
      self.assertEqual(dsds[1,3,1,0],0.0,"ds/ds has wrong value at (1,3,1,0).")
      self.assertEqual(dsds[1,3,1,1],0.0,"ds/ds has wrong value at (1,3,1,1).")
      self.assertEqual(dsds[1,3,1,2],0.0,"ds/ds has wrong value at (1,3,1,2).")
      self.assertEqual(dsds[1,3,1,3],1.0,"ds/ds has wrong value at (1,3,1,3).")
      self.assertEqual(dsds[1,3,1,4],0.0,"ds/ds has wrong value at (1,3,1,4).")
      self.assertEqual(dsds[1,3,2,0],0.0,"ds/ds has wrong value at (1,3,2,0).")
      self.assertEqual(dsds[1,3,2,1],0.0,"ds/ds has wrong value at (1,3,2,1).")
      self.assertEqual(dsds[1,3,2,2],0.0,"ds/ds has wrong value at (1,3,2,2).")
      self.assertEqual(dsds[1,3,2,3],0.0,"ds/ds has wrong value at (1,3,2,3).")
      self.assertEqual(dsds[1,3,2,4],0.0,"ds/ds has wrong value at (1,3,2,4).")
      self.assertEqual(dsds[1,3,3,0],0.0,"ds/ds has wrong value at (1,3,3,0).")
      self.assertEqual(dsds[1,3,3,1],0.0,"ds/ds has wrong value at (1,3,3,1).")
      self.assertEqual(dsds[1,3,3,2],0.0,"ds/ds has wrong value at (1,3,3,2).")
      self.assertEqual(dsds[1,3,3,3],0.0,"ds/ds has wrong value at (1,3,3,3).")
      self.assertEqual(dsds[1,3,3,4],0.0,"ds/ds has wrong value at (1,3,3,4).")
      self.assertEqual(dsds[1,4,0,0],0.0,"ds/ds has wrong value at (1,4,0,0).")
      self.assertEqual(dsds[1,4,0,1],0.0,"ds/ds has wrong value at (1,4,0,1).")
      self.assertEqual(dsds[1,4,0,2],0.0,"ds/ds has wrong value at (1,4,0,2).")
      self.assertEqual(dsds[1,4,0,3],0.0,"ds/ds has wrong value at (1,4,0,3).")
      self.assertEqual(dsds[1,4,0,4],0.0,"ds/ds has wrong value at (1,4,0,4).")
      self.assertEqual(dsds[1,4,1,0],0.0,"ds/ds has wrong value at (1,4,1,0).")
      self.assertEqual(dsds[1,4,1,1],0.0,"ds/ds has wrong value at (1,4,1,1).")
      self.assertEqual(dsds[1,4,1,2],0.0,"ds/ds has wrong value at (1,4,1,2).")
      self.assertEqual(dsds[1,4,1,3],0.0,"ds/ds has wrong value at (1,4,1,3).")
      self.assertEqual(dsds[1,4,1,4],1.0,"ds/ds has wrong value at (1,4,1,4).")
      self.assertEqual(dsds[1,4,2,0],0.0,"ds/ds has wrong value at (1,4,2,0).")
      self.assertEqual(dsds[1,4,2,1],0.0,"ds/ds has wrong value at (1,4,2,1).")
      self.assertEqual(dsds[1,4,2,2],0.0,"ds/ds has wrong value at (1,4,2,2).")
      self.assertEqual(dsds[1,4,2,3],0.0,"ds/ds has wrong value at (1,4,2,3).")
      self.assertEqual(dsds[1,4,2,4],0.0,"ds/ds has wrong value at (1,4,2,4).")
      self.assertEqual(dsds[1,4,3,0],0.0,"ds/ds has wrong value at (1,4,3,0).")
      self.assertEqual(dsds[1,4,3,1],0.0,"ds/ds has wrong value at (1,4,3,1).")
      self.assertEqual(dsds[1,4,3,2],0.0,"ds/ds has wrong value at (1,4,3,2).")
      self.assertEqual(dsds[1,4,3,3],0.0,"ds/ds has wrong value at (1,4,3,3).")
      self.assertEqual(dsds[1,4,3,4],0.0,"ds/ds has wrong value at (1,4,3,4).")
      self.assertEqual(dsds[2,0,0,0],0.0,"ds/ds has wrong value at (2,0,0,0).")
      self.assertEqual(dsds[2,0,0,1],0.0,"ds/ds has wrong value at (2,0,0,1).")
      self.assertEqual(dsds[2,0,0,2],0.0,"ds/ds has wrong value at (2,0,0,2).")
      self.assertEqual(dsds[2,0,0,3],0.0,"ds/ds has wrong value at (2,0,0,3).")
      self.assertEqual(dsds[2,0,0,4],0.0,"ds/ds has wrong value at (2,0,0,4).")
      self.assertEqual(dsds[2,0,1,0],0.0,"ds/ds has wrong value at (2,0,1,0).")
      self.assertEqual(dsds[2,0,1,1],0.0,"ds/ds has wrong value at (2,0,1,1).")
      self.assertEqual(dsds[2,0,1,2],0.0,"ds/ds has wrong value at (2,0,1,2).")
      self.assertEqual(dsds[2,0,1,3],0.0,"ds/ds has wrong value at (2,0,1,3).")
      self.assertEqual(dsds[2,0,1,4],0.0,"ds/ds has wrong value at (2,0,1,4).")
      self.assertEqual(dsds[2,0,2,0],1.0,"ds/ds has wrong value at (2,0,2,0).")
      self.assertEqual(dsds[2,0,2,1],0.0,"ds/ds has wrong value at (2,0,2,1).")
      self.assertEqual(dsds[2,0,2,2],0.0,"ds/ds has wrong value at (2,0,2,2).")
      self.assertEqual(dsds[2,0,2,3],0.0,"ds/ds has wrong value at (2,0,2,3).")
      self.assertEqual(dsds[2,0,2,4],0.0,"ds/ds has wrong value at (2,0,2,4).")
      self.assertEqual(dsds[2,0,3,0],0.0,"ds/ds has wrong value at (2,0,3,0).")
      self.assertEqual(dsds[2,0,3,1],0.0,"ds/ds has wrong value at (2,0,3,1).")
      self.assertEqual(dsds[2,0,3,2],0.0,"ds/ds has wrong value at (2,0,3,2).")
      self.assertEqual(dsds[2,0,3,3],0.0,"ds/ds has wrong value at (2,0,3,3).")
      self.assertEqual(dsds[2,0,3,4],0.0,"ds/ds has wrong value at (2,0,3,4).")
      self.assertEqual(dsds[2,1,0,0],0.0,"ds/ds has wrong value at (2,1,0,0).")
      self.assertEqual(dsds[2,1,0,1],0.0,"ds/ds has wrong value at (2,1,0,1).")
      self.assertEqual(dsds[2,1,0,2],0.0,"ds/ds has wrong value at (2,1,0,2).")
      self.assertEqual(dsds[2,1,0,3],0.0,"ds/ds has wrong value at (2,1,0,3).")
      self.assertEqual(dsds[2,1,0,4],0.0,"ds/ds has wrong value at (2,1,0,4).")
      self.assertEqual(dsds[2,1,1,0],0.0,"ds/ds has wrong value at (2,1,1,0).")
      self.assertEqual(dsds[2,1,1,1],0.0,"ds/ds has wrong value at (2,1,1,1).")
      self.assertEqual(dsds[2,1,1,2],0.0,"ds/ds has wrong value at (2,1,1,2).")
      self.assertEqual(dsds[2,1,1,3],0.0,"ds/ds has wrong value at (2,1,1,3).")
      self.assertEqual(dsds[2,1,1,4],0.0,"ds/ds has wrong value at (2,1,1,4).")
      self.assertEqual(dsds[2,1,2,0],0.0,"ds/ds has wrong value at (2,1,2,0).")
      self.assertEqual(dsds[2,1,2,1],1.0,"ds/ds has wrong value at (2,1,2,1).")
      self.assertEqual(dsds[2,1,2,2],0.0,"ds/ds has wrong value at (2,1,2,2).")
      self.assertEqual(dsds[2,1,2,3],0.0,"ds/ds has wrong value at (2,1,2,3).")
      self.assertEqual(dsds[2,1,2,4],0.0,"ds/ds has wrong value at (2,1,2,4).")
      self.assertEqual(dsds[2,1,3,0],0.0,"ds/ds has wrong value at (2,1,3,0).")
      self.assertEqual(dsds[2,1,3,1],0.0,"ds/ds has wrong value at (2,1,3,1).")
      self.assertEqual(dsds[2,1,3,2],0.0,"ds/ds has wrong value at (2,1,3,2).")
      self.assertEqual(dsds[2,1,3,3],0.0,"ds/ds has wrong value at (2,1,3,3).")
      self.assertEqual(dsds[2,1,3,4],0.0,"ds/ds has wrong value at (2,1,3,4).")
      self.assertEqual(dsds[2,2,0,0],0.0,"ds/ds has wrong value at (2,2,0,0).")
      self.assertEqual(dsds[2,2,0,1],0.0,"ds/ds has wrong value at (2,2,0,1).")
      self.assertEqual(dsds[2,2,0,2],0.0,"ds/ds has wrong value at (2,2,0,2).")
      self.assertEqual(dsds[2,2,0,3],0.0,"ds/ds has wrong value at (2,2,0,3).")
      self.assertEqual(dsds[2,2,0,4],0.0,"ds/ds has wrong value at (2,2,0,4).")
      self.assertEqual(dsds[2,2,1,0],0.0,"ds/ds has wrong value at (2,2,1,0).")
      self.assertEqual(dsds[2,2,1,1],0.0,"ds/ds has wrong value at (2,2,1,1).")
      self.assertEqual(dsds[2,2,1,2],0.0,"ds/ds has wrong value at (2,2,1,2).")
      self.assertEqual(dsds[2,2,1,3],0.0,"ds/ds has wrong value at (2,2,1,3).")
      self.assertEqual(dsds[2,2,1,4],0.0,"ds/ds has wrong value at (2,2,1,4).")
      self.assertEqual(dsds[2,2,2,0],0.0,"ds/ds has wrong value at (2,2,2,0).")
      self.assertEqual(dsds[2,2,2,1],0.0,"ds/ds has wrong value at (2,2,2,1).")
      self.assertEqual(dsds[2,2,2,2],1.0,"ds/ds has wrong value at (2,2,2,2).")
      self.assertEqual(dsds[2,2,2,3],0.0,"ds/ds has wrong value at (2,2,2,3).")
      self.assertEqual(dsds[2,2,2,4],0.0,"ds/ds has wrong value at (2,2,2,4).")
      self.assertEqual(dsds[2,2,3,0],0.0,"ds/ds has wrong value at (2,2,3,0).")
      self.assertEqual(dsds[2,2,3,1],0.0,"ds/ds has wrong value at (2,2,3,1).")
      self.assertEqual(dsds[2,2,3,2],0.0,"ds/ds has wrong value at (2,2,3,2).")
      self.assertEqual(dsds[2,2,3,3],0.0,"ds/ds has wrong value at (2,2,3,3).")
      self.assertEqual(dsds[2,2,3,4],0.0,"ds/ds has wrong value at (2,2,3,4).")
      self.assertEqual(dsds[2,3,0,0],0.0,"ds/ds has wrong value at (2,3,0,0).")
      self.assertEqual(dsds[2,3,0,1],0.0,"ds/ds has wrong value at (2,3,0,1).")
      self.assertEqual(dsds[2,3,0,2],0.0,"ds/ds has wrong value at (2,3,0,2).")
      self.assertEqual(dsds[2,3,0,3],0.0,"ds/ds has wrong value at (2,3,0,3).")
      self.assertEqual(dsds[2,3,0,4],0.0,"ds/ds has wrong value at (2,3,0,4).")
      self.assertEqual(dsds[2,3,1,0],0.0,"ds/ds has wrong value at (2,3,1,0).")
      self.assertEqual(dsds[2,3,1,1],0.0,"ds/ds has wrong value at (2,3,1,1).")
      self.assertEqual(dsds[2,3,1,2],0.0,"ds/ds has wrong value at (2,3,1,2).")
      self.assertEqual(dsds[2,3,1,3],0.0,"ds/ds has wrong value at (2,3,1,3).")
      self.assertEqual(dsds[2,3,1,4],0.0,"ds/ds has wrong value at (2,3,1,4).")
      self.assertEqual(dsds[2,3,2,0],0.0,"ds/ds has wrong value at (2,3,2,0).")
      self.assertEqual(dsds[2,3,2,1],0.0,"ds/ds has wrong value at (2,3,2,1).")
      self.assertEqual(dsds[2,3,2,2],0.0,"ds/ds has wrong value at (2,3,2,2).")
      self.assertEqual(dsds[2,3,2,3],1.0,"ds/ds has wrong value at (2,3,2,3).")
      self.assertEqual(dsds[2,3,2,4],0.0,"ds/ds has wrong value at (2,3,2,4).")
      self.assertEqual(dsds[2,3,3,0],0.0,"ds/ds has wrong value at (2,3,3,0).")
      self.assertEqual(dsds[2,3,3,1],0.0,"ds/ds has wrong value at (2,3,3,1).")
      self.assertEqual(dsds[2,3,3,2],0.0,"ds/ds has wrong value at (2,3,3,2).")
      self.assertEqual(dsds[2,3,3,3],0.0,"ds/ds has wrong value at (2,3,3,3).")
      self.assertEqual(dsds[2,3,3,4],0.0,"ds/ds has wrong value at (2,3,3,4).")
      self.assertEqual(dsds[2,4,0,0],0.0,"ds/ds has wrong value at (2,4,0,0).")
      self.assertEqual(dsds[2,4,0,1],0.0,"ds/ds has wrong value at (2,4,0,1).")
      self.assertEqual(dsds[2,4,0,2],0.0,"ds/ds has wrong value at (2,4,0,2).")
      self.assertEqual(dsds[2,4,0,3],0.0,"ds/ds has wrong value at (2,4,0,3).")
      self.assertEqual(dsds[2,4,0,4],0.0,"ds/ds has wrong value at (2,4,0,4).")
      self.assertEqual(dsds[2,4,1,0],0.0,"ds/ds has wrong value at (2,4,1,0).")
      self.assertEqual(dsds[2,4,1,1],0.0,"ds/ds has wrong value at (2,4,1,1).")
      self.assertEqual(dsds[2,4,1,2],0.0,"ds/ds has wrong value at (2,4,1,2).")
      self.assertEqual(dsds[2,4,1,3],0.0,"ds/ds has wrong value at (2,4,1,3).")
      self.assertEqual(dsds[2,4,1,4],0.0,"ds/ds has wrong value at (2,4,1,4).")
      self.assertEqual(dsds[2,4,2,0],0.0,"ds/ds has wrong value at (2,4,2,0).")
      self.assertEqual(dsds[2,4,2,1],0.0,"ds/ds has wrong value at (2,4,2,1).")
      self.assertEqual(dsds[2,4,2,2],0.0,"ds/ds has wrong value at (2,4,2,2).")
      self.assertEqual(dsds[2,4,2,3],0.0,"ds/ds has wrong value at (2,4,2,3).")
      self.assertEqual(dsds[2,4,2,4],1.0,"ds/ds has wrong value at (2,4,2,4).")
      self.assertEqual(dsds[2,4,3,0],0.0,"ds/ds has wrong value at (2,4,3,0).")
      self.assertEqual(dsds[2,4,3,1],0.0,"ds/ds has wrong value at (2,4,3,1).")
      self.assertEqual(dsds[2,4,3,2],0.0,"ds/ds has wrong value at (2,4,3,2).")
      self.assertEqual(dsds[2,4,3,3],0.0,"ds/ds has wrong value at (2,4,3,3).")
      self.assertEqual(dsds[2,4,3,4],0.0,"ds/ds has wrong value at (2,4,3,4).")
      self.assertEqual(dsds[3,0,0,0],0.0,"ds/ds has wrong value at (3,0,0,0).")
      self.assertEqual(dsds[3,0,0,1],0.0,"ds/ds has wrong value at (3,0,0,1).")
      self.assertEqual(dsds[3,0,0,2],0.0,"ds/ds has wrong value at (3,0,0,2).")
      self.assertEqual(dsds[3,0,0,3],0.0,"ds/ds has wrong value at (3,0,0,3).")
      self.assertEqual(dsds[3,0,0,4],0.0,"ds/ds has wrong value at (3,0,0,4).")
      self.assertEqual(dsds[3,0,1,0],0.0,"ds/ds has wrong value at (3,0,1,0).")
      self.assertEqual(dsds[3,0,1,1],0.0,"ds/ds has wrong value at (3,0,1,1).")
      self.assertEqual(dsds[3,0,1,2],0.0,"ds/ds has wrong value at (3,0,1,2).")
      self.assertEqual(dsds[3,0,1,3],0.0,"ds/ds has wrong value at (3,0,1,3).")
      self.assertEqual(dsds[3,0,1,4],0.0,"ds/ds has wrong value at (3,0,1,4).")
      self.assertEqual(dsds[3,0,2,0],0.0,"ds/ds has wrong value at (3,0,2,0).")
      self.assertEqual(dsds[3,0,2,1],0.0,"ds/ds has wrong value at (3,0,2,1).")
      self.assertEqual(dsds[3,0,2,2],0.0,"ds/ds has wrong value at (3,0,2,2).")
      self.assertEqual(dsds[3,0,2,3],0.0,"ds/ds has wrong value at (3,0,2,3).")
      self.assertEqual(dsds[3,0,2,4],0.0,"ds/ds has wrong value at (3,0,2,4).")
      self.assertEqual(dsds[3,0,3,0],1.0,"ds/ds has wrong value at (3,0,3,0).")
      self.assertEqual(dsds[3,0,3,1],0.0,"ds/ds has wrong value at (3,0,3,1).")
      self.assertEqual(dsds[3,0,3,2],0.0,"ds/ds has wrong value at (3,0,3,2).")
      self.assertEqual(dsds[3,0,3,3],0.0,"ds/ds has wrong value at (3,0,3,3).")
      self.assertEqual(dsds[3,0,3,4],0.0,"ds/ds has wrong value at (3,0,3,4).")
      self.assertEqual(dsds[3,1,0,0],0.0,"ds/ds has wrong value at (3,1,0,0).")
      self.assertEqual(dsds[3,1,0,1],0.0,"ds/ds has wrong value at (3,1,0,1).")
      self.assertEqual(dsds[3,1,0,2],0.0,"ds/ds has wrong value at (3,1,0,2).")
      self.assertEqual(dsds[3,1,0,3],0.0,"ds/ds has wrong value at (3,1,0,3).")
      self.assertEqual(dsds[3,1,0,4],0.0,"ds/ds has wrong value at (3,1,0,4).")
      self.assertEqual(dsds[3,1,1,0],0.0,"ds/ds has wrong value at (3,1,1,0).")
      self.assertEqual(dsds[3,1,1,1],0.0,"ds/ds has wrong value at (3,1,1,1).")
      self.assertEqual(dsds[3,1,1,2],0.0,"ds/ds has wrong value at (3,1,1,2).")
      self.assertEqual(dsds[3,1,1,3],0.0,"ds/ds has wrong value at (3,1,1,3).")
      self.assertEqual(dsds[3,1,1,4],0.0,"ds/ds has wrong value at (3,1,1,4).")
      self.assertEqual(dsds[3,1,2,0],0.0,"ds/ds has wrong value at (3,1,2,0).")
      self.assertEqual(dsds[3,1,2,1],0.0,"ds/ds has wrong value at (3,1,2,1).")
      self.assertEqual(dsds[3,1,2,2],0.0,"ds/ds has wrong value at (3,1,2,2).")
      self.assertEqual(dsds[3,1,2,3],0.0,"ds/ds has wrong value at (3,1,2,3).")
      self.assertEqual(dsds[3,1,2,4],0.0,"ds/ds has wrong value at (3,1,2,4).")
      self.assertEqual(dsds[3,1,3,0],0.0,"ds/ds has wrong value at (3,1,3,0).")
      self.assertEqual(dsds[3,1,3,1],1.0,"ds/ds has wrong value at (3,1,3,1).")
      self.assertEqual(dsds[3,1,3,2],0.0,"ds/ds has wrong value at (3,1,3,2).")
      self.assertEqual(dsds[3,1,3,3],0.0,"ds/ds has wrong value at (3,1,3,3).")
      self.assertEqual(dsds[3,1,3,4],0.0,"ds/ds has wrong value at (3,1,3,4).")
      self.assertEqual(dsds[3,2,0,0],0.0,"ds/ds has wrong value at (3,2,0,0).")
      self.assertEqual(dsds[3,2,0,1],0.0,"ds/ds has wrong value at (3,2,0,1).")
      self.assertEqual(dsds[3,2,0,2],0.0,"ds/ds has wrong value at (3,2,0,2).")
      self.assertEqual(dsds[3,2,0,3],0.0,"ds/ds has wrong value at (3,2,0,3).")
      self.assertEqual(dsds[3,2,0,4],0.0,"ds/ds has wrong value at (3,2,0,4).")
      self.assertEqual(dsds[3,2,1,0],0.0,"ds/ds has wrong value at (3,2,1,0).")
      self.assertEqual(dsds[3,2,1,1],0.0,"ds/ds has wrong value at (3,2,1,1).")
      self.assertEqual(dsds[3,2,1,2],0.0,"ds/ds has wrong value at (3,2,1,2).")
      self.assertEqual(dsds[3,2,1,3],0.0,"ds/ds has wrong value at (3,2,1,3).")
      self.assertEqual(dsds[3,2,1,4],0.0,"ds/ds has wrong value at (3,2,1,4).")
      self.assertEqual(dsds[3,2,2,0],0.0,"ds/ds has wrong value at (3,2,2,0).")
      self.assertEqual(dsds[3,2,2,1],0.0,"ds/ds has wrong value at (3,2,2,1).")
      self.assertEqual(dsds[3,2,2,2],0.0,"ds/ds has wrong value at (3,2,2,2).")
      self.assertEqual(dsds[3,2,2,3],0.0,"ds/ds has wrong value at (3,2,2,3).")
      self.assertEqual(dsds[3,2,2,4],0.0,"ds/ds has wrong value at (3,2,2,4).")
      self.assertEqual(dsds[3,2,3,0],0.0,"ds/ds has wrong value at (3,2,3,0).")
      self.assertEqual(dsds[3,2,3,1],0.0,"ds/ds has wrong value at (3,2,3,1).")
      self.assertEqual(dsds[3,2,3,2],1.0,"ds/ds has wrong value at (3,2,3,2).")
      self.assertEqual(dsds[3,2,3,3],0.0,"ds/ds has wrong value at (3,2,3,3).")
      self.assertEqual(dsds[3,2,3,4],0.0,"ds/ds has wrong value at (3,2,3,4).")
      self.assertEqual(dsds[3,3,0,0],0.0,"ds/ds has wrong value at (3,3,0,0).")
      self.assertEqual(dsds[3,3,0,1],0.0,"ds/ds has wrong value at (3,3,0,1).")
      self.assertEqual(dsds[3,3,0,2],0.0,"ds/ds has wrong value at (3,3,0,2).")
      self.assertEqual(dsds[3,3,0,3],0.0,"ds/ds has wrong value at (3,3,0,3).")
      self.assertEqual(dsds[3,3,0,4],0.0,"ds/ds has wrong value at (3,3,0,4).")
      self.assertEqual(dsds[3,3,1,0],0.0,"ds/ds has wrong value at (3,3,1,0).")
      self.assertEqual(dsds[3,3,1,1],0.0,"ds/ds has wrong value at (3,3,1,1).")
      self.assertEqual(dsds[3,3,1,2],0.0,"ds/ds has wrong value at (3,3,1,2).")
      self.assertEqual(dsds[3,3,1,3],0.0,"ds/ds has wrong value at (3,3,1,3).")
      self.assertEqual(dsds[3,3,1,4],0.0,"ds/ds has wrong value at (3,3,1,4).")
      self.assertEqual(dsds[3,3,2,0],0.0,"ds/ds has wrong value at (3,3,2,0).")
      self.assertEqual(dsds[3,3,2,1],0.0,"ds/ds has wrong value at (3,3,2,1).")
      self.assertEqual(dsds[3,3,2,2],0.0,"ds/ds has wrong value at (3,3,2,2).")
      self.assertEqual(dsds[3,3,2,3],0.0,"ds/ds has wrong value at (3,3,2,3).")
      self.assertEqual(dsds[3,3,2,4],0.0,"ds/ds has wrong value at (3,3,2,4).")
      self.assertEqual(dsds[3,3,3,0],0.0,"ds/ds has wrong value at (3,3,3,0).")
      self.assertEqual(dsds[3,3,3,1],0.0,"ds/ds has wrong value at (3,3,3,1).")
      self.assertEqual(dsds[3,3,3,2],0.0,"ds/ds has wrong value at (3,3,3,2).")
      self.assertEqual(dsds[3,3,3,3],1.0,"ds/ds has wrong value at (3,3,3,3).")
      self.assertEqual(dsds[3,3,3,4],0.0,"ds/ds has wrong value at (3,3,3,4).")
      self.assertEqual(dsds[3,4,0,0],0.0,"ds/ds has wrong value at (3,4,0,0).")
      self.assertEqual(dsds[3,4,0,1],0.0,"ds/ds has wrong value at (3,4,0,1).")
      self.assertEqual(dsds[3,4,0,2],0.0,"ds/ds has wrong value at (3,4,0,2).")
      self.assertEqual(dsds[3,4,0,3],0.0,"ds/ds has wrong value at (3,4,0,3).")
      self.assertEqual(dsds[3,4,0,4],0.0,"ds/ds has wrong value at (3,4,0,4).")
      self.assertEqual(dsds[3,4,1,0],0.0,"ds/ds has wrong value at (3,4,1,0).")
      self.assertEqual(dsds[3,4,1,1],0.0,"ds/ds has wrong value at (3,4,1,1).")
      self.assertEqual(dsds[3,4,1,2],0.0,"ds/ds has wrong value at (3,4,1,2).")
      self.assertEqual(dsds[3,4,1,3],0.0,"ds/ds has wrong value at (3,4,1,3).")
      self.assertEqual(dsds[3,4,1,4],0.0,"ds/ds has wrong value at (3,4,1,4).")
      self.assertEqual(dsds[3,4,2,0],0.0,"ds/ds has wrong value at (3,4,2,0).")
      self.assertEqual(dsds[3,4,2,1],0.0,"ds/ds has wrong value at (3,4,2,1).")
      self.assertEqual(dsds[3,4,2,2],0.0,"ds/ds has wrong value at (3,4,2,2).")
      self.assertEqual(dsds[3,4,2,3],0.0,"ds/ds has wrong value at (3,4,2,3).")
      self.assertEqual(dsds[3,4,2,4],0.0,"ds/ds has wrong value at (3,4,2,4).")
      self.assertEqual(dsds[3,4,3,0],0.0,"ds/ds has wrong value at (3,4,3,0).")
      self.assertEqual(dsds[3,4,3,1],0.0,"ds/ds has wrong value at (3,4,3,1).")
      self.assertEqual(dsds[3,4,3,2],0.0,"ds/ds has wrong value at (3,4,3,2).")
      self.assertEqual(dsds[3,4,3,3],0.0,"ds/ds has wrong value at (3,4,3,3).")
      self.assertEqual(dsds[3,4,3,4],1.0,"ds/ds has wrong value at (3,4,3,4).")

      sa=s.getSubstitutedArguments({s2:-10})
      self.assertEqual(len(sa),0,"wrong number of substituted arguments")

      self.assertFalse(s.isAppropriateValue(10.)," 10. is not appropriate substitute")
      self.assertFalse(s.isAppropriateValue(10)," 10 is not appropriate substitute")
      self.assertTrue(s.isAppropriateValue(Symbol(shape=(4, 5),dim=self.functionspace.getDim())),"Symbol is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(Symbol(shape=(5,))),"Symbol is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(escript.Data(0.,(4, 5),self.functionspace)),"Data is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(escript.Data(0.,(5,),self.functionspace)),"Data is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(numpy.zeros((4, 5))),"ndarray is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(numpy.zeros((5,))),"ndarray is not appropriate substitute (shape)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_Symbol_rank2_dd_nargs1(self):
      s2=Symbol(shape=())
      s=Symbol(shape=(4, 5),dim=self.functionspace.getDim(),args=[s2])
      self.assertEqual(s.getRank(),2,"wrong rank.")
      self.assertEqual(s.getShape(),(4, 5),"wrong shape.")
      self.assertEqual(s.getDim(),self.functionspace.getDim(),"wrong spatial dimension.")
      self.assertEqual(s.getArgument(),[s2],"wrong arguments.")

      ss=s.substitute({s:numpy.zeros((4, 5))})
      self.assertTrue(isinstance(ss,numpy.ndarray),"value after substitution is not ndarray.")
      self.assertEqual(ss.shape,(4, 5),"value after substitution has not expected shape")
      try:
        s.substitute({s:numpy.zeros((5,))})
        fail("illegal substition was successful")
      except TypeError:
        pass

      dsdarg=s.diff(Symbol(shape=()))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds() has wrong type.")
      self.assertEqual(dsdarg.shape,(4, 5),"ds/ds() has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds() has wrong value.")
      dsdarg=s.diff(Symbol(shape=(2,)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(2,) has wrong type.")
      self.assertEqual(dsdarg.shape,(4, 5, 2),"ds/ds(2,) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(2,) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(4, 5)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(4, 5) has wrong type.")
      self.assertEqual(dsdarg.shape,(4, 5, 4, 5),"ds/ds(4, 5) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(4, 5) has wrong value.")

      dsds=s.diff(s)
      self.assertTrue(isinstance(dsds,numpy.ndarray),"ds/ds has wrong type.")
      self.assertEqual(dsds.shape,(4, 5, 4, 5),"ds/ds has wrong shape.")
      self.assertEqual(dsds[0,0,0,0],1.0,"ds/ds has wrong value at (0,0,0,0).")
      self.assertEqual(dsds[0,0,0,1],0.0,"ds/ds has wrong value at (0,0,0,1).")
      self.assertEqual(dsds[0,0,0,2],0.0,"ds/ds has wrong value at (0,0,0,2).")
      self.assertEqual(dsds[0,0,0,3],0.0,"ds/ds has wrong value at (0,0,0,3).")
      self.assertEqual(dsds[0,0,0,4],0.0,"ds/ds has wrong value at (0,0,0,4).")
      self.assertEqual(dsds[0,0,1,0],0.0,"ds/ds has wrong value at (0,0,1,0).")
      self.assertEqual(dsds[0,0,1,1],0.0,"ds/ds has wrong value at (0,0,1,1).")
      self.assertEqual(dsds[0,0,1,2],0.0,"ds/ds has wrong value at (0,0,1,2).")
      self.assertEqual(dsds[0,0,1,3],0.0,"ds/ds has wrong value at (0,0,1,3).")
      self.assertEqual(dsds[0,0,1,4],0.0,"ds/ds has wrong value at (0,0,1,4).")
      self.assertEqual(dsds[0,0,2,0],0.0,"ds/ds has wrong value at (0,0,2,0).")
      self.assertEqual(dsds[0,0,2,1],0.0,"ds/ds has wrong value at (0,0,2,1).")
      self.assertEqual(dsds[0,0,2,2],0.0,"ds/ds has wrong value at (0,0,2,2).")
      self.assertEqual(dsds[0,0,2,3],0.0,"ds/ds has wrong value at (0,0,2,3).")
      self.assertEqual(dsds[0,0,2,4],0.0,"ds/ds has wrong value at (0,0,2,4).")
      self.assertEqual(dsds[0,0,3,0],0.0,"ds/ds has wrong value at (0,0,3,0).")
      self.assertEqual(dsds[0,0,3,1],0.0,"ds/ds has wrong value at (0,0,3,1).")
      self.assertEqual(dsds[0,0,3,2],0.0,"ds/ds has wrong value at (0,0,3,2).")
      self.assertEqual(dsds[0,0,3,3],0.0,"ds/ds has wrong value at (0,0,3,3).")
      self.assertEqual(dsds[0,0,3,4],0.0,"ds/ds has wrong value at (0,0,3,4).")
      self.assertEqual(dsds[0,1,0,0],0.0,"ds/ds has wrong value at (0,1,0,0).")
      self.assertEqual(dsds[0,1,0,1],1.0,"ds/ds has wrong value at (0,1,0,1).")
      self.assertEqual(dsds[0,1,0,2],0.0,"ds/ds has wrong value at (0,1,0,2).")
      self.assertEqual(dsds[0,1,0,3],0.0,"ds/ds has wrong value at (0,1,0,3).")
      self.assertEqual(dsds[0,1,0,4],0.0,"ds/ds has wrong value at (0,1,0,4).")
      self.assertEqual(dsds[0,1,1,0],0.0,"ds/ds has wrong value at (0,1,1,0).")
      self.assertEqual(dsds[0,1,1,1],0.0,"ds/ds has wrong value at (0,1,1,1).")
      self.assertEqual(dsds[0,1,1,2],0.0,"ds/ds has wrong value at (0,1,1,2).")
      self.assertEqual(dsds[0,1,1,3],0.0,"ds/ds has wrong value at (0,1,1,3).")
      self.assertEqual(dsds[0,1,1,4],0.0,"ds/ds has wrong value at (0,1,1,4).")
      self.assertEqual(dsds[0,1,2,0],0.0,"ds/ds has wrong value at (0,1,2,0).")
      self.assertEqual(dsds[0,1,2,1],0.0,"ds/ds has wrong value at (0,1,2,1).")
      self.assertEqual(dsds[0,1,2,2],0.0,"ds/ds has wrong value at (0,1,2,2).")
      self.assertEqual(dsds[0,1,2,3],0.0,"ds/ds has wrong value at (0,1,2,3).")
      self.assertEqual(dsds[0,1,2,4],0.0,"ds/ds has wrong value at (0,1,2,4).")
      self.assertEqual(dsds[0,1,3,0],0.0,"ds/ds has wrong value at (0,1,3,0).")
      self.assertEqual(dsds[0,1,3,1],0.0,"ds/ds has wrong value at (0,1,3,1).")
      self.assertEqual(dsds[0,1,3,2],0.0,"ds/ds has wrong value at (0,1,3,2).")
      self.assertEqual(dsds[0,1,3,3],0.0,"ds/ds has wrong value at (0,1,3,3).")
      self.assertEqual(dsds[0,1,3,4],0.0,"ds/ds has wrong value at (0,1,3,4).")
      self.assertEqual(dsds[0,2,0,0],0.0,"ds/ds has wrong value at (0,2,0,0).")
      self.assertEqual(dsds[0,2,0,1],0.0,"ds/ds has wrong value at (0,2,0,1).")
      self.assertEqual(dsds[0,2,0,2],1.0,"ds/ds has wrong value at (0,2,0,2).")
      self.assertEqual(dsds[0,2,0,3],0.0,"ds/ds has wrong value at (0,2,0,3).")
      self.assertEqual(dsds[0,2,0,4],0.0,"ds/ds has wrong value at (0,2,0,4).")
      self.assertEqual(dsds[0,2,1,0],0.0,"ds/ds has wrong value at (0,2,1,0).")
      self.assertEqual(dsds[0,2,1,1],0.0,"ds/ds has wrong value at (0,2,1,1).")
      self.assertEqual(dsds[0,2,1,2],0.0,"ds/ds has wrong value at (0,2,1,2).")
      self.assertEqual(dsds[0,2,1,3],0.0,"ds/ds has wrong value at (0,2,1,3).")
      self.assertEqual(dsds[0,2,1,4],0.0,"ds/ds has wrong value at (0,2,1,4).")
      self.assertEqual(dsds[0,2,2,0],0.0,"ds/ds has wrong value at (0,2,2,0).")
      self.assertEqual(dsds[0,2,2,1],0.0,"ds/ds has wrong value at (0,2,2,1).")
      self.assertEqual(dsds[0,2,2,2],0.0,"ds/ds has wrong value at (0,2,2,2).")
      self.assertEqual(dsds[0,2,2,3],0.0,"ds/ds has wrong value at (0,2,2,3).")
      self.assertEqual(dsds[0,2,2,4],0.0,"ds/ds has wrong value at (0,2,2,4).")
      self.assertEqual(dsds[0,2,3,0],0.0,"ds/ds has wrong value at (0,2,3,0).")
      self.assertEqual(dsds[0,2,3,1],0.0,"ds/ds has wrong value at (0,2,3,1).")
      self.assertEqual(dsds[0,2,3,2],0.0,"ds/ds has wrong value at (0,2,3,2).")
      self.assertEqual(dsds[0,2,3,3],0.0,"ds/ds has wrong value at (0,2,3,3).")
      self.assertEqual(dsds[0,2,3,4],0.0,"ds/ds has wrong value at (0,2,3,4).")
      self.assertEqual(dsds[0,3,0,0],0.0,"ds/ds has wrong value at (0,3,0,0).")
      self.assertEqual(dsds[0,3,0,1],0.0,"ds/ds has wrong value at (0,3,0,1).")
      self.assertEqual(dsds[0,3,0,2],0.0,"ds/ds has wrong value at (0,3,0,2).")
      self.assertEqual(dsds[0,3,0,3],1.0,"ds/ds has wrong value at (0,3,0,3).")
      self.assertEqual(dsds[0,3,0,4],0.0,"ds/ds has wrong value at (0,3,0,4).")
      self.assertEqual(dsds[0,3,1,0],0.0,"ds/ds has wrong value at (0,3,1,0).")
      self.assertEqual(dsds[0,3,1,1],0.0,"ds/ds has wrong value at (0,3,1,1).")
      self.assertEqual(dsds[0,3,1,2],0.0,"ds/ds has wrong value at (0,3,1,2).")
      self.assertEqual(dsds[0,3,1,3],0.0,"ds/ds has wrong value at (0,3,1,3).")
      self.assertEqual(dsds[0,3,1,4],0.0,"ds/ds has wrong value at (0,3,1,4).")
      self.assertEqual(dsds[0,3,2,0],0.0,"ds/ds has wrong value at (0,3,2,0).")
      self.assertEqual(dsds[0,3,2,1],0.0,"ds/ds has wrong value at (0,3,2,1).")
      self.assertEqual(dsds[0,3,2,2],0.0,"ds/ds has wrong value at (0,3,2,2).")
      self.assertEqual(dsds[0,3,2,3],0.0,"ds/ds has wrong value at (0,3,2,3).")
      self.assertEqual(dsds[0,3,2,4],0.0,"ds/ds has wrong value at (0,3,2,4).")
      self.assertEqual(dsds[0,3,3,0],0.0,"ds/ds has wrong value at (0,3,3,0).")
      self.assertEqual(dsds[0,3,3,1],0.0,"ds/ds has wrong value at (0,3,3,1).")
      self.assertEqual(dsds[0,3,3,2],0.0,"ds/ds has wrong value at (0,3,3,2).")
      self.assertEqual(dsds[0,3,3,3],0.0,"ds/ds has wrong value at (0,3,3,3).")
      self.assertEqual(dsds[0,3,3,4],0.0,"ds/ds has wrong value at (0,3,3,4).")
      self.assertEqual(dsds[0,4,0,0],0.0,"ds/ds has wrong value at (0,4,0,0).")
      self.assertEqual(dsds[0,4,0,1],0.0,"ds/ds has wrong value at (0,4,0,1).")
      self.assertEqual(dsds[0,4,0,2],0.0,"ds/ds has wrong value at (0,4,0,2).")
      self.assertEqual(dsds[0,4,0,3],0.0,"ds/ds has wrong value at (0,4,0,3).")
      self.assertEqual(dsds[0,4,0,4],1.0,"ds/ds has wrong value at (0,4,0,4).")
      self.assertEqual(dsds[0,4,1,0],0.0,"ds/ds has wrong value at (0,4,1,0).")
      self.assertEqual(dsds[0,4,1,1],0.0,"ds/ds has wrong value at (0,4,1,1).")
      self.assertEqual(dsds[0,4,1,2],0.0,"ds/ds has wrong value at (0,4,1,2).")
      self.assertEqual(dsds[0,4,1,3],0.0,"ds/ds has wrong value at (0,4,1,3).")
      self.assertEqual(dsds[0,4,1,4],0.0,"ds/ds has wrong value at (0,4,1,4).")
      self.assertEqual(dsds[0,4,2,0],0.0,"ds/ds has wrong value at (0,4,2,0).")
      self.assertEqual(dsds[0,4,2,1],0.0,"ds/ds has wrong value at (0,4,2,1).")
      self.assertEqual(dsds[0,4,2,2],0.0,"ds/ds has wrong value at (0,4,2,2).")
      self.assertEqual(dsds[0,4,2,3],0.0,"ds/ds has wrong value at (0,4,2,3).")
      self.assertEqual(dsds[0,4,2,4],0.0,"ds/ds has wrong value at (0,4,2,4).")
      self.assertEqual(dsds[0,4,3,0],0.0,"ds/ds has wrong value at (0,4,3,0).")
      self.assertEqual(dsds[0,4,3,1],0.0,"ds/ds has wrong value at (0,4,3,1).")
      self.assertEqual(dsds[0,4,3,2],0.0,"ds/ds has wrong value at (0,4,3,2).")
      self.assertEqual(dsds[0,4,3,3],0.0,"ds/ds has wrong value at (0,4,3,3).")
      self.assertEqual(dsds[0,4,3,4],0.0,"ds/ds has wrong value at (0,4,3,4).")
      self.assertEqual(dsds[1,0,0,0],0.0,"ds/ds has wrong value at (1,0,0,0).")
      self.assertEqual(dsds[1,0,0,1],0.0,"ds/ds has wrong value at (1,0,0,1).")
      self.assertEqual(dsds[1,0,0,2],0.0,"ds/ds has wrong value at (1,0,0,2).")
      self.assertEqual(dsds[1,0,0,3],0.0,"ds/ds has wrong value at (1,0,0,3).")
      self.assertEqual(dsds[1,0,0,4],0.0,"ds/ds has wrong value at (1,0,0,4).")
      self.assertEqual(dsds[1,0,1,0],1.0,"ds/ds has wrong value at (1,0,1,0).")
      self.assertEqual(dsds[1,0,1,1],0.0,"ds/ds has wrong value at (1,0,1,1).")
      self.assertEqual(dsds[1,0,1,2],0.0,"ds/ds has wrong value at (1,0,1,2).")
      self.assertEqual(dsds[1,0,1,3],0.0,"ds/ds has wrong value at (1,0,1,3).")
      self.assertEqual(dsds[1,0,1,4],0.0,"ds/ds has wrong value at (1,0,1,4).")
      self.assertEqual(dsds[1,0,2,0],0.0,"ds/ds has wrong value at (1,0,2,0).")
      self.assertEqual(dsds[1,0,2,1],0.0,"ds/ds has wrong value at (1,0,2,1).")
      self.assertEqual(dsds[1,0,2,2],0.0,"ds/ds has wrong value at (1,0,2,2).")
      self.assertEqual(dsds[1,0,2,3],0.0,"ds/ds has wrong value at (1,0,2,3).")
      self.assertEqual(dsds[1,0,2,4],0.0,"ds/ds has wrong value at (1,0,2,4).")
      self.assertEqual(dsds[1,0,3,0],0.0,"ds/ds has wrong value at (1,0,3,0).")
      self.assertEqual(dsds[1,0,3,1],0.0,"ds/ds has wrong value at (1,0,3,1).")
      self.assertEqual(dsds[1,0,3,2],0.0,"ds/ds has wrong value at (1,0,3,2).")
      self.assertEqual(dsds[1,0,3,3],0.0,"ds/ds has wrong value at (1,0,3,3).")
      self.assertEqual(dsds[1,0,3,4],0.0,"ds/ds has wrong value at (1,0,3,4).")
      self.assertEqual(dsds[1,1,0,0],0.0,"ds/ds has wrong value at (1,1,0,0).")
      self.assertEqual(dsds[1,1,0,1],0.0,"ds/ds has wrong value at (1,1,0,1).")
      self.assertEqual(dsds[1,1,0,2],0.0,"ds/ds has wrong value at (1,1,0,2).")
      self.assertEqual(dsds[1,1,0,3],0.0,"ds/ds has wrong value at (1,1,0,3).")
      self.assertEqual(dsds[1,1,0,4],0.0,"ds/ds has wrong value at (1,1,0,4).")
      self.assertEqual(dsds[1,1,1,0],0.0,"ds/ds has wrong value at (1,1,1,0).")
      self.assertEqual(dsds[1,1,1,1],1.0,"ds/ds has wrong value at (1,1,1,1).")
      self.assertEqual(dsds[1,1,1,2],0.0,"ds/ds has wrong value at (1,1,1,2).")
      self.assertEqual(dsds[1,1,1,3],0.0,"ds/ds has wrong value at (1,1,1,3).")
      self.assertEqual(dsds[1,1,1,4],0.0,"ds/ds has wrong value at (1,1,1,4).")
      self.assertEqual(dsds[1,1,2,0],0.0,"ds/ds has wrong value at (1,1,2,0).")
      self.assertEqual(dsds[1,1,2,1],0.0,"ds/ds has wrong value at (1,1,2,1).")
      self.assertEqual(dsds[1,1,2,2],0.0,"ds/ds has wrong value at (1,1,2,2).")
      self.assertEqual(dsds[1,1,2,3],0.0,"ds/ds has wrong value at (1,1,2,3).")
      self.assertEqual(dsds[1,1,2,4],0.0,"ds/ds has wrong value at (1,1,2,4).")
      self.assertEqual(dsds[1,1,3,0],0.0,"ds/ds has wrong value at (1,1,3,0).")
      self.assertEqual(dsds[1,1,3,1],0.0,"ds/ds has wrong value at (1,1,3,1).")
      self.assertEqual(dsds[1,1,3,2],0.0,"ds/ds has wrong value at (1,1,3,2).")
      self.assertEqual(dsds[1,1,3,3],0.0,"ds/ds has wrong value at (1,1,3,3).")
      self.assertEqual(dsds[1,1,3,4],0.0,"ds/ds has wrong value at (1,1,3,4).")
      self.assertEqual(dsds[1,2,0,0],0.0,"ds/ds has wrong value at (1,2,0,0).")
      self.assertEqual(dsds[1,2,0,1],0.0,"ds/ds has wrong value at (1,2,0,1).")
      self.assertEqual(dsds[1,2,0,2],0.0,"ds/ds has wrong value at (1,2,0,2).")
      self.assertEqual(dsds[1,2,0,3],0.0,"ds/ds has wrong value at (1,2,0,3).")
      self.assertEqual(dsds[1,2,0,4],0.0,"ds/ds has wrong value at (1,2,0,4).")
      self.assertEqual(dsds[1,2,1,0],0.0,"ds/ds has wrong value at (1,2,1,0).")
      self.assertEqual(dsds[1,2,1,1],0.0,"ds/ds has wrong value at (1,2,1,1).")
      self.assertEqual(dsds[1,2,1,2],1.0,"ds/ds has wrong value at (1,2,1,2).")
      self.assertEqual(dsds[1,2,1,3],0.0,"ds/ds has wrong value at (1,2,1,3).")
      self.assertEqual(dsds[1,2,1,4],0.0,"ds/ds has wrong value at (1,2,1,4).")
      self.assertEqual(dsds[1,2,2,0],0.0,"ds/ds has wrong value at (1,2,2,0).")
      self.assertEqual(dsds[1,2,2,1],0.0,"ds/ds has wrong value at (1,2,2,1).")
      self.assertEqual(dsds[1,2,2,2],0.0,"ds/ds has wrong value at (1,2,2,2).")
      self.assertEqual(dsds[1,2,2,3],0.0,"ds/ds has wrong value at (1,2,2,3).")
      self.assertEqual(dsds[1,2,2,4],0.0,"ds/ds has wrong value at (1,2,2,4).")
      self.assertEqual(dsds[1,2,3,0],0.0,"ds/ds has wrong value at (1,2,3,0).")
      self.assertEqual(dsds[1,2,3,1],0.0,"ds/ds has wrong value at (1,2,3,1).")
      self.assertEqual(dsds[1,2,3,2],0.0,"ds/ds has wrong value at (1,2,3,2).")
      self.assertEqual(dsds[1,2,3,3],0.0,"ds/ds has wrong value at (1,2,3,3).")
      self.assertEqual(dsds[1,2,3,4],0.0,"ds/ds has wrong value at (1,2,3,4).")
      self.assertEqual(dsds[1,3,0,0],0.0,"ds/ds has wrong value at (1,3,0,0).")
      self.assertEqual(dsds[1,3,0,1],0.0,"ds/ds has wrong value at (1,3,0,1).")
      self.assertEqual(dsds[1,3,0,2],0.0,"ds/ds has wrong value at (1,3,0,2).")
      self.assertEqual(dsds[1,3,0,3],0.0,"ds/ds has wrong value at (1,3,0,3).")
      self.assertEqual(dsds[1,3,0,4],0.0,"ds/ds has wrong value at (1,3,0,4).")
      self.assertEqual(dsds[1,3,1,0],0.0,"ds/ds has wrong value at (1,3,1,0).")
      self.assertEqual(dsds[1,3,1,1],0.0,"ds/ds has wrong value at (1,3,1,1).")
      self.assertEqual(dsds[1,3,1,2],0.0,"ds/ds has wrong value at (1,3,1,2).")
      self.assertEqual(dsds[1,3,1,3],1.0,"ds/ds has wrong value at (1,3,1,3).")
      self.assertEqual(dsds[1,3,1,4],0.0,"ds/ds has wrong value at (1,3,1,4).")
      self.assertEqual(dsds[1,3,2,0],0.0,"ds/ds has wrong value at (1,3,2,0).")
      self.assertEqual(dsds[1,3,2,1],0.0,"ds/ds has wrong value at (1,3,2,1).")
      self.assertEqual(dsds[1,3,2,2],0.0,"ds/ds has wrong value at (1,3,2,2).")
      self.assertEqual(dsds[1,3,2,3],0.0,"ds/ds has wrong value at (1,3,2,3).")
      self.assertEqual(dsds[1,3,2,4],0.0,"ds/ds has wrong value at (1,3,2,4).")
      self.assertEqual(dsds[1,3,3,0],0.0,"ds/ds has wrong value at (1,3,3,0).")
      self.assertEqual(dsds[1,3,3,1],0.0,"ds/ds has wrong value at (1,3,3,1).")
      self.assertEqual(dsds[1,3,3,2],0.0,"ds/ds has wrong value at (1,3,3,2).")
      self.assertEqual(dsds[1,3,3,3],0.0,"ds/ds has wrong value at (1,3,3,3).")
      self.assertEqual(dsds[1,3,3,4],0.0,"ds/ds has wrong value at (1,3,3,4).")
      self.assertEqual(dsds[1,4,0,0],0.0,"ds/ds has wrong value at (1,4,0,0).")
      self.assertEqual(dsds[1,4,0,1],0.0,"ds/ds has wrong value at (1,4,0,1).")
      self.assertEqual(dsds[1,4,0,2],0.0,"ds/ds has wrong value at (1,4,0,2).")
      self.assertEqual(dsds[1,4,0,3],0.0,"ds/ds has wrong value at (1,4,0,3).")
      self.assertEqual(dsds[1,4,0,4],0.0,"ds/ds has wrong value at (1,4,0,4).")
      self.assertEqual(dsds[1,4,1,0],0.0,"ds/ds has wrong value at (1,4,1,0).")
      self.assertEqual(dsds[1,4,1,1],0.0,"ds/ds has wrong value at (1,4,1,1).")
      self.assertEqual(dsds[1,4,1,2],0.0,"ds/ds has wrong value at (1,4,1,2).")
      self.assertEqual(dsds[1,4,1,3],0.0,"ds/ds has wrong value at (1,4,1,3).")
      self.assertEqual(dsds[1,4,1,4],1.0,"ds/ds has wrong value at (1,4,1,4).")
      self.assertEqual(dsds[1,4,2,0],0.0,"ds/ds has wrong value at (1,4,2,0).")
      self.assertEqual(dsds[1,4,2,1],0.0,"ds/ds has wrong value at (1,4,2,1).")
      self.assertEqual(dsds[1,4,2,2],0.0,"ds/ds has wrong value at (1,4,2,2).")
      self.assertEqual(dsds[1,4,2,3],0.0,"ds/ds has wrong value at (1,4,2,3).")
      self.assertEqual(dsds[1,4,2,4],0.0,"ds/ds has wrong value at (1,4,2,4).")
      self.assertEqual(dsds[1,4,3,0],0.0,"ds/ds has wrong value at (1,4,3,0).")
      self.assertEqual(dsds[1,4,3,1],0.0,"ds/ds has wrong value at (1,4,3,1).")
      self.assertEqual(dsds[1,4,3,2],0.0,"ds/ds has wrong value at (1,4,3,2).")
      self.assertEqual(dsds[1,4,3,3],0.0,"ds/ds has wrong value at (1,4,3,3).")
      self.assertEqual(dsds[1,4,3,4],0.0,"ds/ds has wrong value at (1,4,3,4).")
      self.assertEqual(dsds[2,0,0,0],0.0,"ds/ds has wrong value at (2,0,0,0).")
      self.assertEqual(dsds[2,0,0,1],0.0,"ds/ds has wrong value at (2,0,0,1).")
      self.assertEqual(dsds[2,0,0,2],0.0,"ds/ds has wrong value at (2,0,0,2).")
      self.assertEqual(dsds[2,0,0,3],0.0,"ds/ds has wrong value at (2,0,0,3).")
      self.assertEqual(dsds[2,0,0,4],0.0,"ds/ds has wrong value at (2,0,0,4).")
      self.assertEqual(dsds[2,0,1,0],0.0,"ds/ds has wrong value at (2,0,1,0).")
      self.assertEqual(dsds[2,0,1,1],0.0,"ds/ds has wrong value at (2,0,1,1).")
      self.assertEqual(dsds[2,0,1,2],0.0,"ds/ds has wrong value at (2,0,1,2).")
      self.assertEqual(dsds[2,0,1,3],0.0,"ds/ds has wrong value at (2,0,1,3).")
      self.assertEqual(dsds[2,0,1,4],0.0,"ds/ds has wrong value at (2,0,1,4).")
      self.assertEqual(dsds[2,0,2,0],1.0,"ds/ds has wrong value at (2,0,2,0).")
      self.assertEqual(dsds[2,0,2,1],0.0,"ds/ds has wrong value at (2,0,2,1).")
      self.assertEqual(dsds[2,0,2,2],0.0,"ds/ds has wrong value at (2,0,2,2).")
      self.assertEqual(dsds[2,0,2,3],0.0,"ds/ds has wrong value at (2,0,2,3).")
      self.assertEqual(dsds[2,0,2,4],0.0,"ds/ds has wrong value at (2,0,2,4).")
      self.assertEqual(dsds[2,0,3,0],0.0,"ds/ds has wrong value at (2,0,3,0).")
      self.assertEqual(dsds[2,0,3,1],0.0,"ds/ds has wrong value at (2,0,3,1).")
      self.assertEqual(dsds[2,0,3,2],0.0,"ds/ds has wrong value at (2,0,3,2).")
      self.assertEqual(dsds[2,0,3,3],0.0,"ds/ds has wrong value at (2,0,3,3).")
      self.assertEqual(dsds[2,0,3,4],0.0,"ds/ds has wrong value at (2,0,3,4).")
      self.assertEqual(dsds[2,1,0,0],0.0,"ds/ds has wrong value at (2,1,0,0).")
      self.assertEqual(dsds[2,1,0,1],0.0,"ds/ds has wrong value at (2,1,0,1).")
      self.assertEqual(dsds[2,1,0,2],0.0,"ds/ds has wrong value at (2,1,0,2).")
      self.assertEqual(dsds[2,1,0,3],0.0,"ds/ds has wrong value at (2,1,0,3).")
      self.assertEqual(dsds[2,1,0,4],0.0,"ds/ds has wrong value at (2,1,0,4).")
      self.assertEqual(dsds[2,1,1,0],0.0,"ds/ds has wrong value at (2,1,1,0).")
      self.assertEqual(dsds[2,1,1,1],0.0,"ds/ds has wrong value at (2,1,1,1).")
      self.assertEqual(dsds[2,1,1,2],0.0,"ds/ds has wrong value at (2,1,1,2).")
      self.assertEqual(dsds[2,1,1,3],0.0,"ds/ds has wrong value at (2,1,1,3).")
      self.assertEqual(dsds[2,1,1,4],0.0,"ds/ds has wrong value at (2,1,1,4).")
      self.assertEqual(dsds[2,1,2,0],0.0,"ds/ds has wrong value at (2,1,2,0).")
      self.assertEqual(dsds[2,1,2,1],1.0,"ds/ds has wrong value at (2,1,2,1).")
      self.assertEqual(dsds[2,1,2,2],0.0,"ds/ds has wrong value at (2,1,2,2).")
      self.assertEqual(dsds[2,1,2,3],0.0,"ds/ds has wrong value at (2,1,2,3).")
      self.assertEqual(dsds[2,1,2,4],0.0,"ds/ds has wrong value at (2,1,2,4).")
      self.assertEqual(dsds[2,1,3,0],0.0,"ds/ds has wrong value at (2,1,3,0).")
      self.assertEqual(dsds[2,1,3,1],0.0,"ds/ds has wrong value at (2,1,3,1).")
      self.assertEqual(dsds[2,1,3,2],0.0,"ds/ds has wrong value at (2,1,3,2).")
      self.assertEqual(dsds[2,1,3,3],0.0,"ds/ds has wrong value at (2,1,3,3).")
      self.assertEqual(dsds[2,1,3,4],0.0,"ds/ds has wrong value at (2,1,3,4).")
      self.assertEqual(dsds[2,2,0,0],0.0,"ds/ds has wrong value at (2,2,0,0).")
      self.assertEqual(dsds[2,2,0,1],0.0,"ds/ds has wrong value at (2,2,0,1).")
      self.assertEqual(dsds[2,2,0,2],0.0,"ds/ds has wrong value at (2,2,0,2).")
      self.assertEqual(dsds[2,2,0,3],0.0,"ds/ds has wrong value at (2,2,0,3).")
      self.assertEqual(dsds[2,2,0,4],0.0,"ds/ds has wrong value at (2,2,0,4).")
      self.assertEqual(dsds[2,2,1,0],0.0,"ds/ds has wrong value at (2,2,1,0).")
      self.assertEqual(dsds[2,2,1,1],0.0,"ds/ds has wrong value at (2,2,1,1).")
      self.assertEqual(dsds[2,2,1,2],0.0,"ds/ds has wrong value at (2,2,1,2).")
      self.assertEqual(dsds[2,2,1,3],0.0,"ds/ds has wrong value at (2,2,1,3).")
      self.assertEqual(dsds[2,2,1,4],0.0,"ds/ds has wrong value at (2,2,1,4).")
      self.assertEqual(dsds[2,2,2,0],0.0,"ds/ds has wrong value at (2,2,2,0).")
      self.assertEqual(dsds[2,2,2,1],0.0,"ds/ds has wrong value at (2,2,2,1).")
      self.assertEqual(dsds[2,2,2,2],1.0,"ds/ds has wrong value at (2,2,2,2).")
      self.assertEqual(dsds[2,2,2,3],0.0,"ds/ds has wrong value at (2,2,2,3).")
      self.assertEqual(dsds[2,2,2,4],0.0,"ds/ds has wrong value at (2,2,2,4).")
      self.assertEqual(dsds[2,2,3,0],0.0,"ds/ds has wrong value at (2,2,3,0).")
      self.assertEqual(dsds[2,2,3,1],0.0,"ds/ds has wrong value at (2,2,3,1).")
      self.assertEqual(dsds[2,2,3,2],0.0,"ds/ds has wrong value at (2,2,3,2).")
      self.assertEqual(dsds[2,2,3,3],0.0,"ds/ds has wrong value at (2,2,3,3).")
      self.assertEqual(dsds[2,2,3,4],0.0,"ds/ds has wrong value at (2,2,3,4).")
      self.assertEqual(dsds[2,3,0,0],0.0,"ds/ds has wrong value at (2,3,0,0).")
      self.assertEqual(dsds[2,3,0,1],0.0,"ds/ds has wrong value at (2,3,0,1).")
      self.assertEqual(dsds[2,3,0,2],0.0,"ds/ds has wrong value at (2,3,0,2).")
      self.assertEqual(dsds[2,3,0,3],0.0,"ds/ds has wrong value at (2,3,0,3).")
      self.assertEqual(dsds[2,3,0,4],0.0,"ds/ds has wrong value at (2,3,0,4).")
      self.assertEqual(dsds[2,3,1,0],0.0,"ds/ds has wrong value at (2,3,1,0).")
      self.assertEqual(dsds[2,3,1,1],0.0,"ds/ds has wrong value at (2,3,1,1).")
      self.assertEqual(dsds[2,3,1,2],0.0,"ds/ds has wrong value at (2,3,1,2).")
      self.assertEqual(dsds[2,3,1,3],0.0,"ds/ds has wrong value at (2,3,1,3).")
      self.assertEqual(dsds[2,3,1,4],0.0,"ds/ds has wrong value at (2,3,1,4).")
      self.assertEqual(dsds[2,3,2,0],0.0,"ds/ds has wrong value at (2,3,2,0).")
      self.assertEqual(dsds[2,3,2,1],0.0,"ds/ds has wrong value at (2,3,2,1).")
      self.assertEqual(dsds[2,3,2,2],0.0,"ds/ds has wrong value at (2,3,2,2).")
      self.assertEqual(dsds[2,3,2,3],1.0,"ds/ds has wrong value at (2,3,2,3).")
      self.assertEqual(dsds[2,3,2,4],0.0,"ds/ds has wrong value at (2,3,2,4).")
      self.assertEqual(dsds[2,3,3,0],0.0,"ds/ds has wrong value at (2,3,3,0).")
      self.assertEqual(dsds[2,3,3,1],0.0,"ds/ds has wrong value at (2,3,3,1).")
      self.assertEqual(dsds[2,3,3,2],0.0,"ds/ds has wrong value at (2,3,3,2).")
      self.assertEqual(dsds[2,3,3,3],0.0,"ds/ds has wrong value at (2,3,3,3).")
      self.assertEqual(dsds[2,3,3,4],0.0,"ds/ds has wrong value at (2,3,3,4).")
      self.assertEqual(dsds[2,4,0,0],0.0,"ds/ds has wrong value at (2,4,0,0).")
      self.assertEqual(dsds[2,4,0,1],0.0,"ds/ds has wrong value at (2,4,0,1).")
      self.assertEqual(dsds[2,4,0,2],0.0,"ds/ds has wrong value at (2,4,0,2).")
      self.assertEqual(dsds[2,4,0,3],0.0,"ds/ds has wrong value at (2,4,0,3).")
      self.assertEqual(dsds[2,4,0,4],0.0,"ds/ds has wrong value at (2,4,0,4).")
      self.assertEqual(dsds[2,4,1,0],0.0,"ds/ds has wrong value at (2,4,1,0).")
      self.assertEqual(dsds[2,4,1,1],0.0,"ds/ds has wrong value at (2,4,1,1).")
      self.assertEqual(dsds[2,4,1,2],0.0,"ds/ds has wrong value at (2,4,1,2).")
      self.assertEqual(dsds[2,4,1,3],0.0,"ds/ds has wrong value at (2,4,1,3).")
      self.assertEqual(dsds[2,4,1,4],0.0,"ds/ds has wrong value at (2,4,1,4).")
      self.assertEqual(dsds[2,4,2,0],0.0,"ds/ds has wrong value at (2,4,2,0).")
      self.assertEqual(dsds[2,4,2,1],0.0,"ds/ds has wrong value at (2,4,2,1).")
      self.assertEqual(dsds[2,4,2,2],0.0,"ds/ds has wrong value at (2,4,2,2).")
      self.assertEqual(dsds[2,4,2,3],0.0,"ds/ds has wrong value at (2,4,2,3).")
      self.assertEqual(dsds[2,4,2,4],1.0,"ds/ds has wrong value at (2,4,2,4).")
      self.assertEqual(dsds[2,4,3,0],0.0,"ds/ds has wrong value at (2,4,3,0).")
      self.assertEqual(dsds[2,4,3,1],0.0,"ds/ds has wrong value at (2,4,3,1).")
      self.assertEqual(dsds[2,4,3,2],0.0,"ds/ds has wrong value at (2,4,3,2).")
      self.assertEqual(dsds[2,4,3,3],0.0,"ds/ds has wrong value at (2,4,3,3).")
      self.assertEqual(dsds[2,4,3,4],0.0,"ds/ds has wrong value at (2,4,3,4).")
      self.assertEqual(dsds[3,0,0,0],0.0,"ds/ds has wrong value at (3,0,0,0).")
      self.assertEqual(dsds[3,0,0,1],0.0,"ds/ds has wrong value at (3,0,0,1).")
      self.assertEqual(dsds[3,0,0,2],0.0,"ds/ds has wrong value at (3,0,0,2).")
      self.assertEqual(dsds[3,0,0,3],0.0,"ds/ds has wrong value at (3,0,0,3).")
      self.assertEqual(dsds[3,0,0,4],0.0,"ds/ds has wrong value at (3,0,0,4).")
      self.assertEqual(dsds[3,0,1,0],0.0,"ds/ds has wrong value at (3,0,1,0).")
      self.assertEqual(dsds[3,0,1,1],0.0,"ds/ds has wrong value at (3,0,1,1).")
      self.assertEqual(dsds[3,0,1,2],0.0,"ds/ds has wrong value at (3,0,1,2).")
      self.assertEqual(dsds[3,0,1,3],0.0,"ds/ds has wrong value at (3,0,1,3).")
      self.assertEqual(dsds[3,0,1,4],0.0,"ds/ds has wrong value at (3,0,1,4).")
      self.assertEqual(dsds[3,0,2,0],0.0,"ds/ds has wrong value at (3,0,2,0).")
      self.assertEqual(dsds[3,0,2,1],0.0,"ds/ds has wrong value at (3,0,2,1).")
      self.assertEqual(dsds[3,0,2,2],0.0,"ds/ds has wrong value at (3,0,2,2).")
      self.assertEqual(dsds[3,0,2,3],0.0,"ds/ds has wrong value at (3,0,2,3).")
      self.assertEqual(dsds[3,0,2,4],0.0,"ds/ds has wrong value at (3,0,2,4).")
      self.assertEqual(dsds[3,0,3,0],1.0,"ds/ds has wrong value at (3,0,3,0).")
      self.assertEqual(dsds[3,0,3,1],0.0,"ds/ds has wrong value at (3,0,3,1).")
      self.assertEqual(dsds[3,0,3,2],0.0,"ds/ds has wrong value at (3,0,3,2).")
      self.assertEqual(dsds[3,0,3,3],0.0,"ds/ds has wrong value at (3,0,3,3).")
      self.assertEqual(dsds[3,0,3,4],0.0,"ds/ds has wrong value at (3,0,3,4).")
      self.assertEqual(dsds[3,1,0,0],0.0,"ds/ds has wrong value at (3,1,0,0).")
      self.assertEqual(dsds[3,1,0,1],0.0,"ds/ds has wrong value at (3,1,0,1).")
      self.assertEqual(dsds[3,1,0,2],0.0,"ds/ds has wrong value at (3,1,0,2).")
      self.assertEqual(dsds[3,1,0,3],0.0,"ds/ds has wrong value at (3,1,0,3).")
      self.assertEqual(dsds[3,1,0,4],0.0,"ds/ds has wrong value at (3,1,0,4).")
      self.assertEqual(dsds[3,1,1,0],0.0,"ds/ds has wrong value at (3,1,1,0).")
      self.assertEqual(dsds[3,1,1,1],0.0,"ds/ds has wrong value at (3,1,1,1).")
      self.assertEqual(dsds[3,1,1,2],0.0,"ds/ds has wrong value at (3,1,1,2).")
      self.assertEqual(dsds[3,1,1,3],0.0,"ds/ds has wrong value at (3,1,1,3).")
      self.assertEqual(dsds[3,1,1,4],0.0,"ds/ds has wrong value at (3,1,1,4).")
      self.assertEqual(dsds[3,1,2,0],0.0,"ds/ds has wrong value at (3,1,2,0).")
      self.assertEqual(dsds[3,1,2,1],0.0,"ds/ds has wrong value at (3,1,2,1).")
      self.assertEqual(dsds[3,1,2,2],0.0,"ds/ds has wrong value at (3,1,2,2).")
      self.assertEqual(dsds[3,1,2,3],0.0,"ds/ds has wrong value at (3,1,2,3).")
      self.assertEqual(dsds[3,1,2,4],0.0,"ds/ds has wrong value at (3,1,2,4).")
      self.assertEqual(dsds[3,1,3,0],0.0,"ds/ds has wrong value at (3,1,3,0).")
      self.assertEqual(dsds[3,1,3,1],1.0,"ds/ds has wrong value at (3,1,3,1).")
      self.assertEqual(dsds[3,1,3,2],0.0,"ds/ds has wrong value at (3,1,3,2).")
      self.assertEqual(dsds[3,1,3,3],0.0,"ds/ds has wrong value at (3,1,3,3).")
      self.assertEqual(dsds[3,1,3,4],0.0,"ds/ds has wrong value at (3,1,3,4).")
      self.assertEqual(dsds[3,2,0,0],0.0,"ds/ds has wrong value at (3,2,0,0).")
      self.assertEqual(dsds[3,2,0,1],0.0,"ds/ds has wrong value at (3,2,0,1).")
      self.assertEqual(dsds[3,2,0,2],0.0,"ds/ds has wrong value at (3,2,0,2).")
      self.assertEqual(dsds[3,2,0,3],0.0,"ds/ds has wrong value at (3,2,0,3).")
      self.assertEqual(dsds[3,2,0,4],0.0,"ds/ds has wrong value at (3,2,0,4).")
      self.assertEqual(dsds[3,2,1,0],0.0,"ds/ds has wrong value at (3,2,1,0).")
      self.assertEqual(dsds[3,2,1,1],0.0,"ds/ds has wrong value at (3,2,1,1).")
      self.assertEqual(dsds[3,2,1,2],0.0,"ds/ds has wrong value at (3,2,1,2).")
      self.assertEqual(dsds[3,2,1,3],0.0,"ds/ds has wrong value at (3,2,1,3).")
      self.assertEqual(dsds[3,2,1,4],0.0,"ds/ds has wrong value at (3,2,1,4).")
      self.assertEqual(dsds[3,2,2,0],0.0,"ds/ds has wrong value at (3,2,2,0).")
      self.assertEqual(dsds[3,2,2,1],0.0,"ds/ds has wrong value at (3,2,2,1).")
      self.assertEqual(dsds[3,2,2,2],0.0,"ds/ds has wrong value at (3,2,2,2).")
      self.assertEqual(dsds[3,2,2,3],0.0,"ds/ds has wrong value at (3,2,2,3).")
      self.assertEqual(dsds[3,2,2,4],0.0,"ds/ds has wrong value at (3,2,2,4).")
      self.assertEqual(dsds[3,2,3,0],0.0,"ds/ds has wrong value at (3,2,3,0).")
      self.assertEqual(dsds[3,2,3,1],0.0,"ds/ds has wrong value at (3,2,3,1).")
      self.assertEqual(dsds[3,2,3,2],1.0,"ds/ds has wrong value at (3,2,3,2).")
      self.assertEqual(dsds[3,2,3,3],0.0,"ds/ds has wrong value at (3,2,3,3).")
      self.assertEqual(dsds[3,2,3,4],0.0,"ds/ds has wrong value at (3,2,3,4).")
      self.assertEqual(dsds[3,3,0,0],0.0,"ds/ds has wrong value at (3,3,0,0).")
      self.assertEqual(dsds[3,3,0,1],0.0,"ds/ds has wrong value at (3,3,0,1).")
      self.assertEqual(dsds[3,3,0,2],0.0,"ds/ds has wrong value at (3,3,0,2).")
      self.assertEqual(dsds[3,3,0,3],0.0,"ds/ds has wrong value at (3,3,0,3).")
      self.assertEqual(dsds[3,3,0,4],0.0,"ds/ds has wrong value at (3,3,0,4).")
      self.assertEqual(dsds[3,3,1,0],0.0,"ds/ds has wrong value at (3,3,1,0).")
      self.assertEqual(dsds[3,3,1,1],0.0,"ds/ds has wrong value at (3,3,1,1).")
      self.assertEqual(dsds[3,3,1,2],0.0,"ds/ds has wrong value at (3,3,1,2).")
      self.assertEqual(dsds[3,3,1,3],0.0,"ds/ds has wrong value at (3,3,1,3).")
      self.assertEqual(dsds[3,3,1,4],0.0,"ds/ds has wrong value at (3,3,1,4).")
      self.assertEqual(dsds[3,3,2,0],0.0,"ds/ds has wrong value at (3,3,2,0).")
      self.assertEqual(dsds[3,3,2,1],0.0,"ds/ds has wrong value at (3,3,2,1).")
      self.assertEqual(dsds[3,3,2,2],0.0,"ds/ds has wrong value at (3,3,2,2).")
      self.assertEqual(dsds[3,3,2,3],0.0,"ds/ds has wrong value at (3,3,2,3).")
      self.assertEqual(dsds[3,3,2,4],0.0,"ds/ds has wrong value at (3,3,2,4).")
      self.assertEqual(dsds[3,3,3,0],0.0,"ds/ds has wrong value at (3,3,3,0).")
      self.assertEqual(dsds[3,3,3,1],0.0,"ds/ds has wrong value at (3,3,3,1).")
      self.assertEqual(dsds[3,3,3,2],0.0,"ds/ds has wrong value at (3,3,3,2).")
      self.assertEqual(dsds[3,3,3,3],1.0,"ds/ds has wrong value at (3,3,3,3).")
      self.assertEqual(dsds[3,3,3,4],0.0,"ds/ds has wrong value at (3,3,3,4).")
      self.assertEqual(dsds[3,4,0,0],0.0,"ds/ds has wrong value at (3,4,0,0).")
      self.assertEqual(dsds[3,4,0,1],0.0,"ds/ds has wrong value at (3,4,0,1).")
      self.assertEqual(dsds[3,4,0,2],0.0,"ds/ds has wrong value at (3,4,0,2).")
      self.assertEqual(dsds[3,4,0,3],0.0,"ds/ds has wrong value at (3,4,0,3).")
      self.assertEqual(dsds[3,4,0,4],0.0,"ds/ds has wrong value at (3,4,0,4).")
      self.assertEqual(dsds[3,4,1,0],0.0,"ds/ds has wrong value at (3,4,1,0).")
      self.assertEqual(dsds[3,4,1,1],0.0,"ds/ds has wrong value at (3,4,1,1).")
      self.assertEqual(dsds[3,4,1,2],0.0,"ds/ds has wrong value at (3,4,1,2).")
      self.assertEqual(dsds[3,4,1,3],0.0,"ds/ds has wrong value at (3,4,1,3).")
      self.assertEqual(dsds[3,4,1,4],0.0,"ds/ds has wrong value at (3,4,1,4).")
      self.assertEqual(dsds[3,4,2,0],0.0,"ds/ds has wrong value at (3,4,2,0).")
      self.assertEqual(dsds[3,4,2,1],0.0,"ds/ds has wrong value at (3,4,2,1).")
      self.assertEqual(dsds[3,4,2,2],0.0,"ds/ds has wrong value at (3,4,2,2).")
      self.assertEqual(dsds[3,4,2,3],0.0,"ds/ds has wrong value at (3,4,2,3).")
      self.assertEqual(dsds[3,4,2,4],0.0,"ds/ds has wrong value at (3,4,2,4).")
      self.assertEqual(dsds[3,4,3,0],0.0,"ds/ds has wrong value at (3,4,3,0).")
      self.assertEqual(dsds[3,4,3,1],0.0,"ds/ds has wrong value at (3,4,3,1).")
      self.assertEqual(dsds[3,4,3,2],0.0,"ds/ds has wrong value at (3,4,3,2).")
      self.assertEqual(dsds[3,4,3,3],0.0,"ds/ds has wrong value at (3,4,3,3).")
      self.assertEqual(dsds[3,4,3,4],1.0,"ds/ds has wrong value at (3,4,3,4).")

      self.assertEqual(s.getArgument(0),s2,"wrong argument 0.")
      sa=s.getSubstitutedArguments({s2:-10})
      self.assertEqual(len(sa),1,"wrong number of substituted arguments")
      self.assertEqual(sa[0],-10,"wrongly substituted argument 0.")

      self.assertFalse(s.isAppropriateValue(10.)," 10. is not appropriate substitute")
      self.assertFalse(s.isAppropriateValue(10)," 10 is not appropriate substitute")
      self.assertTrue(s.isAppropriateValue(Symbol(shape=(4, 5),dim=self.functionspace.getDim())),"Symbol is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(Symbol(shape=(5,))),"Symbol is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(escript.Data(0.,(4, 5),self.functionspace)),"Data is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(escript.Data(0.,(5,),self.functionspace)),"Data is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(numpy.zeros((4, 5))),"ndarray is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(numpy.zeros((5,))),"ndarray is not appropriate substitute (shape)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_Symbol_rank2_dd_nargs2(self):
      s2=Symbol(shape=())
      s=Symbol(shape=(4, 5),dim=self.functionspace.getDim(),args=[1, -1.0])
      self.assertEqual(s.getRank(),2,"wrong rank.")
      self.assertEqual(s.getShape(),(4, 5),"wrong shape.")
      self.assertEqual(s.getDim(),self.functionspace.getDim(),"wrong spatial dimension.")
      self.assertEqual(s.getArgument(),[1, -1.0],"wrong arguments.")

      ss=s.substitute({s:numpy.zeros((4, 5))})
      self.assertTrue(isinstance(ss,numpy.ndarray),"value after substitution is not ndarray.")
      self.assertEqual(ss.shape,(4, 5),"value after substitution has not expected shape")
      try:
        s.substitute({s:numpy.zeros((5,))})
        fail("illegal substition was successful")
      except TypeError:
        pass

      dsdarg=s.diff(Symbol(shape=()))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds() has wrong type.")
      self.assertEqual(dsdarg.shape,(4, 5),"ds/ds() has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds() has wrong value.")
      dsdarg=s.diff(Symbol(shape=(2,)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(2,) has wrong type.")
      self.assertEqual(dsdarg.shape,(4, 5, 2),"ds/ds(2,) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(2,) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(4, 5)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(4, 5) has wrong type.")
      self.assertEqual(dsdarg.shape,(4, 5, 4, 5),"ds/ds(4, 5) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(4, 5) has wrong value.")

      dsds=s.diff(s)
      self.assertTrue(isinstance(dsds,numpy.ndarray),"ds/ds has wrong type.")
      self.assertEqual(dsds.shape,(4, 5, 4, 5),"ds/ds has wrong shape.")
      self.assertEqual(dsds[0,0,0,0],1.0,"ds/ds has wrong value at (0,0,0,0).")
      self.assertEqual(dsds[0,0,0,1],0.0,"ds/ds has wrong value at (0,0,0,1).")
      self.assertEqual(dsds[0,0,0,2],0.0,"ds/ds has wrong value at (0,0,0,2).")
      self.assertEqual(dsds[0,0,0,3],0.0,"ds/ds has wrong value at (0,0,0,3).")
      self.assertEqual(dsds[0,0,0,4],0.0,"ds/ds has wrong value at (0,0,0,4).")
      self.assertEqual(dsds[0,0,1,0],0.0,"ds/ds has wrong value at (0,0,1,0).")
      self.assertEqual(dsds[0,0,1,1],0.0,"ds/ds has wrong value at (0,0,1,1).")
      self.assertEqual(dsds[0,0,1,2],0.0,"ds/ds has wrong value at (0,0,1,2).")
      self.assertEqual(dsds[0,0,1,3],0.0,"ds/ds has wrong value at (0,0,1,3).")
      self.assertEqual(dsds[0,0,1,4],0.0,"ds/ds has wrong value at (0,0,1,4).")
      self.assertEqual(dsds[0,0,2,0],0.0,"ds/ds has wrong value at (0,0,2,0).")
      self.assertEqual(dsds[0,0,2,1],0.0,"ds/ds has wrong value at (0,0,2,1).")
      self.assertEqual(dsds[0,0,2,2],0.0,"ds/ds has wrong value at (0,0,2,2).")
      self.assertEqual(dsds[0,0,2,3],0.0,"ds/ds has wrong value at (0,0,2,3).")
      self.assertEqual(dsds[0,0,2,4],0.0,"ds/ds has wrong value at (0,0,2,4).")
      self.assertEqual(dsds[0,0,3,0],0.0,"ds/ds has wrong value at (0,0,3,0).")
      self.assertEqual(dsds[0,0,3,1],0.0,"ds/ds has wrong value at (0,0,3,1).")
      self.assertEqual(dsds[0,0,3,2],0.0,"ds/ds has wrong value at (0,0,3,2).")
      self.assertEqual(dsds[0,0,3,3],0.0,"ds/ds has wrong value at (0,0,3,3).")
      self.assertEqual(dsds[0,0,3,4],0.0,"ds/ds has wrong value at (0,0,3,4).")
      self.assertEqual(dsds[0,1,0,0],0.0,"ds/ds has wrong value at (0,1,0,0).")
      self.assertEqual(dsds[0,1,0,1],1.0,"ds/ds has wrong value at (0,1,0,1).")
      self.assertEqual(dsds[0,1,0,2],0.0,"ds/ds has wrong value at (0,1,0,2).")
      self.assertEqual(dsds[0,1,0,3],0.0,"ds/ds has wrong value at (0,1,0,3).")
      self.assertEqual(dsds[0,1,0,4],0.0,"ds/ds has wrong value at (0,1,0,4).")
      self.assertEqual(dsds[0,1,1,0],0.0,"ds/ds has wrong value at (0,1,1,0).")
      self.assertEqual(dsds[0,1,1,1],0.0,"ds/ds has wrong value at (0,1,1,1).")
      self.assertEqual(dsds[0,1,1,2],0.0,"ds/ds has wrong value at (0,1,1,2).")
      self.assertEqual(dsds[0,1,1,3],0.0,"ds/ds has wrong value at (0,1,1,3).")
      self.assertEqual(dsds[0,1,1,4],0.0,"ds/ds has wrong value at (0,1,1,4).")
      self.assertEqual(dsds[0,1,2,0],0.0,"ds/ds has wrong value at (0,1,2,0).")
      self.assertEqual(dsds[0,1,2,1],0.0,"ds/ds has wrong value at (0,1,2,1).")
      self.assertEqual(dsds[0,1,2,2],0.0,"ds/ds has wrong value at (0,1,2,2).")
      self.assertEqual(dsds[0,1,2,3],0.0,"ds/ds has wrong value at (0,1,2,3).")
      self.assertEqual(dsds[0,1,2,4],0.0,"ds/ds has wrong value at (0,1,2,4).")
      self.assertEqual(dsds[0,1,3,0],0.0,"ds/ds has wrong value at (0,1,3,0).")
      self.assertEqual(dsds[0,1,3,1],0.0,"ds/ds has wrong value at (0,1,3,1).")
      self.assertEqual(dsds[0,1,3,2],0.0,"ds/ds has wrong value at (0,1,3,2).")
      self.assertEqual(dsds[0,1,3,3],0.0,"ds/ds has wrong value at (0,1,3,3).")
      self.assertEqual(dsds[0,1,3,4],0.0,"ds/ds has wrong value at (0,1,3,4).")
      self.assertEqual(dsds[0,2,0,0],0.0,"ds/ds has wrong value at (0,2,0,0).")
      self.assertEqual(dsds[0,2,0,1],0.0,"ds/ds has wrong value at (0,2,0,1).")
      self.assertEqual(dsds[0,2,0,2],1.0,"ds/ds has wrong value at (0,2,0,2).")
      self.assertEqual(dsds[0,2,0,3],0.0,"ds/ds has wrong value at (0,2,0,3).")
      self.assertEqual(dsds[0,2,0,4],0.0,"ds/ds has wrong value at (0,2,0,4).")
      self.assertEqual(dsds[0,2,1,0],0.0,"ds/ds has wrong value at (0,2,1,0).")
      self.assertEqual(dsds[0,2,1,1],0.0,"ds/ds has wrong value at (0,2,1,1).")
      self.assertEqual(dsds[0,2,1,2],0.0,"ds/ds has wrong value at (0,2,1,2).")
      self.assertEqual(dsds[0,2,1,3],0.0,"ds/ds has wrong value at (0,2,1,3).")
      self.assertEqual(dsds[0,2,1,4],0.0,"ds/ds has wrong value at (0,2,1,4).")
      self.assertEqual(dsds[0,2,2,0],0.0,"ds/ds has wrong value at (0,2,2,0).")
      self.assertEqual(dsds[0,2,2,1],0.0,"ds/ds has wrong value at (0,2,2,1).")
      self.assertEqual(dsds[0,2,2,2],0.0,"ds/ds has wrong value at (0,2,2,2).")
      self.assertEqual(dsds[0,2,2,3],0.0,"ds/ds has wrong value at (0,2,2,3).")
      self.assertEqual(dsds[0,2,2,4],0.0,"ds/ds has wrong value at (0,2,2,4).")
      self.assertEqual(dsds[0,2,3,0],0.0,"ds/ds has wrong value at (0,2,3,0).")
      self.assertEqual(dsds[0,2,3,1],0.0,"ds/ds has wrong value at (0,2,3,1).")
      self.assertEqual(dsds[0,2,3,2],0.0,"ds/ds has wrong value at (0,2,3,2).")
      self.assertEqual(dsds[0,2,3,3],0.0,"ds/ds has wrong value at (0,2,3,3).")
      self.assertEqual(dsds[0,2,3,4],0.0,"ds/ds has wrong value at (0,2,3,4).")
      self.assertEqual(dsds[0,3,0,0],0.0,"ds/ds has wrong value at (0,3,0,0).")
      self.assertEqual(dsds[0,3,0,1],0.0,"ds/ds has wrong value at (0,3,0,1).")
      self.assertEqual(dsds[0,3,0,2],0.0,"ds/ds has wrong value at (0,3,0,2).")
      self.assertEqual(dsds[0,3,0,3],1.0,"ds/ds has wrong value at (0,3,0,3).")
      self.assertEqual(dsds[0,3,0,4],0.0,"ds/ds has wrong value at (0,3,0,4).")
      self.assertEqual(dsds[0,3,1,0],0.0,"ds/ds has wrong value at (0,3,1,0).")
      self.assertEqual(dsds[0,3,1,1],0.0,"ds/ds has wrong value at (0,3,1,1).")
      self.assertEqual(dsds[0,3,1,2],0.0,"ds/ds has wrong value at (0,3,1,2).")
      self.assertEqual(dsds[0,3,1,3],0.0,"ds/ds has wrong value at (0,3,1,3).")
      self.assertEqual(dsds[0,3,1,4],0.0,"ds/ds has wrong value at (0,3,1,4).")
      self.assertEqual(dsds[0,3,2,0],0.0,"ds/ds has wrong value at (0,3,2,0).")
      self.assertEqual(dsds[0,3,2,1],0.0,"ds/ds has wrong value at (0,3,2,1).")
      self.assertEqual(dsds[0,3,2,2],0.0,"ds/ds has wrong value at (0,3,2,2).")
      self.assertEqual(dsds[0,3,2,3],0.0,"ds/ds has wrong value at (0,3,2,3).")
      self.assertEqual(dsds[0,3,2,4],0.0,"ds/ds has wrong value at (0,3,2,4).")
      self.assertEqual(dsds[0,3,3,0],0.0,"ds/ds has wrong value at (0,3,3,0).")
      self.assertEqual(dsds[0,3,3,1],0.0,"ds/ds has wrong value at (0,3,3,1).")
      self.assertEqual(dsds[0,3,3,2],0.0,"ds/ds has wrong value at (0,3,3,2).")
      self.assertEqual(dsds[0,3,3,3],0.0,"ds/ds has wrong value at (0,3,3,3).")
      self.assertEqual(dsds[0,3,3,4],0.0,"ds/ds has wrong value at (0,3,3,4).")
      self.assertEqual(dsds[0,4,0,0],0.0,"ds/ds has wrong value at (0,4,0,0).")
      self.assertEqual(dsds[0,4,0,1],0.0,"ds/ds has wrong value at (0,4,0,1).")
      self.assertEqual(dsds[0,4,0,2],0.0,"ds/ds has wrong value at (0,4,0,2).")
      self.assertEqual(dsds[0,4,0,3],0.0,"ds/ds has wrong value at (0,4,0,3).")
      self.assertEqual(dsds[0,4,0,4],1.0,"ds/ds has wrong value at (0,4,0,4).")
      self.assertEqual(dsds[0,4,1,0],0.0,"ds/ds has wrong value at (0,4,1,0).")
      self.assertEqual(dsds[0,4,1,1],0.0,"ds/ds has wrong value at (0,4,1,1).")
      self.assertEqual(dsds[0,4,1,2],0.0,"ds/ds has wrong value at (0,4,1,2).")
      self.assertEqual(dsds[0,4,1,3],0.0,"ds/ds has wrong value at (0,4,1,3).")
      self.assertEqual(dsds[0,4,1,4],0.0,"ds/ds has wrong value at (0,4,1,4).")
      self.assertEqual(dsds[0,4,2,0],0.0,"ds/ds has wrong value at (0,4,2,0).")
      self.assertEqual(dsds[0,4,2,1],0.0,"ds/ds has wrong value at (0,4,2,1).")
      self.assertEqual(dsds[0,4,2,2],0.0,"ds/ds has wrong value at (0,4,2,2).")
      self.assertEqual(dsds[0,4,2,3],0.0,"ds/ds has wrong value at (0,4,2,3).")
      self.assertEqual(dsds[0,4,2,4],0.0,"ds/ds has wrong value at (0,4,2,4).")
      self.assertEqual(dsds[0,4,3,0],0.0,"ds/ds has wrong value at (0,4,3,0).")
      self.assertEqual(dsds[0,4,3,1],0.0,"ds/ds has wrong value at (0,4,3,1).")
      self.assertEqual(dsds[0,4,3,2],0.0,"ds/ds has wrong value at (0,4,3,2).")
      self.assertEqual(dsds[0,4,3,3],0.0,"ds/ds has wrong value at (0,4,3,3).")
      self.assertEqual(dsds[0,4,3,4],0.0,"ds/ds has wrong value at (0,4,3,4).")
      self.assertEqual(dsds[1,0,0,0],0.0,"ds/ds has wrong value at (1,0,0,0).")
      self.assertEqual(dsds[1,0,0,1],0.0,"ds/ds has wrong value at (1,0,0,1).")
      self.assertEqual(dsds[1,0,0,2],0.0,"ds/ds has wrong value at (1,0,0,2).")
      self.assertEqual(dsds[1,0,0,3],0.0,"ds/ds has wrong value at (1,0,0,3).")
      self.assertEqual(dsds[1,0,0,4],0.0,"ds/ds has wrong value at (1,0,0,4).")
      self.assertEqual(dsds[1,0,1,0],1.0,"ds/ds has wrong value at (1,0,1,0).")
      self.assertEqual(dsds[1,0,1,1],0.0,"ds/ds has wrong value at (1,0,1,1).")
      self.assertEqual(dsds[1,0,1,2],0.0,"ds/ds has wrong value at (1,0,1,2).")
      self.assertEqual(dsds[1,0,1,3],0.0,"ds/ds has wrong value at (1,0,1,3).")
      self.assertEqual(dsds[1,0,1,4],0.0,"ds/ds has wrong value at (1,0,1,4).")
      self.assertEqual(dsds[1,0,2,0],0.0,"ds/ds has wrong value at (1,0,2,0).")
      self.assertEqual(dsds[1,0,2,1],0.0,"ds/ds has wrong value at (1,0,2,1).")
      self.assertEqual(dsds[1,0,2,2],0.0,"ds/ds has wrong value at (1,0,2,2).")
      self.assertEqual(dsds[1,0,2,3],0.0,"ds/ds has wrong value at (1,0,2,3).")
      self.assertEqual(dsds[1,0,2,4],0.0,"ds/ds has wrong value at (1,0,2,4).")
      self.assertEqual(dsds[1,0,3,0],0.0,"ds/ds has wrong value at (1,0,3,0).")
      self.assertEqual(dsds[1,0,3,1],0.0,"ds/ds has wrong value at (1,0,3,1).")
      self.assertEqual(dsds[1,0,3,2],0.0,"ds/ds has wrong value at (1,0,3,2).")
      self.assertEqual(dsds[1,0,3,3],0.0,"ds/ds has wrong value at (1,0,3,3).")
      self.assertEqual(dsds[1,0,3,4],0.0,"ds/ds has wrong value at (1,0,3,4).")
      self.assertEqual(dsds[1,1,0,0],0.0,"ds/ds has wrong value at (1,1,0,0).")
      self.assertEqual(dsds[1,1,0,1],0.0,"ds/ds has wrong value at (1,1,0,1).")
      self.assertEqual(dsds[1,1,0,2],0.0,"ds/ds has wrong value at (1,1,0,2).")
      self.assertEqual(dsds[1,1,0,3],0.0,"ds/ds has wrong value at (1,1,0,3).")
      self.assertEqual(dsds[1,1,0,4],0.0,"ds/ds has wrong value at (1,1,0,4).")
      self.assertEqual(dsds[1,1,1,0],0.0,"ds/ds has wrong value at (1,1,1,0).")
      self.assertEqual(dsds[1,1,1,1],1.0,"ds/ds has wrong value at (1,1,1,1).")
      self.assertEqual(dsds[1,1,1,2],0.0,"ds/ds has wrong value at (1,1,1,2).")
      self.assertEqual(dsds[1,1,1,3],0.0,"ds/ds has wrong value at (1,1,1,3).")
      self.assertEqual(dsds[1,1,1,4],0.0,"ds/ds has wrong value at (1,1,1,4).")
      self.assertEqual(dsds[1,1,2,0],0.0,"ds/ds has wrong value at (1,1,2,0).")
      self.assertEqual(dsds[1,1,2,1],0.0,"ds/ds has wrong value at (1,1,2,1).")
      self.assertEqual(dsds[1,1,2,2],0.0,"ds/ds has wrong value at (1,1,2,2).")
      self.assertEqual(dsds[1,1,2,3],0.0,"ds/ds has wrong value at (1,1,2,3).")
      self.assertEqual(dsds[1,1,2,4],0.0,"ds/ds has wrong value at (1,1,2,4).")
      self.assertEqual(dsds[1,1,3,0],0.0,"ds/ds has wrong value at (1,1,3,0).")
      self.assertEqual(dsds[1,1,3,1],0.0,"ds/ds has wrong value at (1,1,3,1).")
      self.assertEqual(dsds[1,1,3,2],0.0,"ds/ds has wrong value at (1,1,3,2).")
      self.assertEqual(dsds[1,1,3,3],0.0,"ds/ds has wrong value at (1,1,3,3).")
      self.assertEqual(dsds[1,1,3,4],0.0,"ds/ds has wrong value at (1,1,3,4).")
      self.assertEqual(dsds[1,2,0,0],0.0,"ds/ds has wrong value at (1,2,0,0).")
      self.assertEqual(dsds[1,2,0,1],0.0,"ds/ds has wrong value at (1,2,0,1).")
      self.assertEqual(dsds[1,2,0,2],0.0,"ds/ds has wrong value at (1,2,0,2).")
      self.assertEqual(dsds[1,2,0,3],0.0,"ds/ds has wrong value at (1,2,0,3).")
      self.assertEqual(dsds[1,2,0,4],0.0,"ds/ds has wrong value at (1,2,0,4).")
      self.assertEqual(dsds[1,2,1,0],0.0,"ds/ds has wrong value at (1,2,1,0).")
      self.assertEqual(dsds[1,2,1,1],0.0,"ds/ds has wrong value at (1,2,1,1).")
      self.assertEqual(dsds[1,2,1,2],1.0,"ds/ds has wrong value at (1,2,1,2).")
      self.assertEqual(dsds[1,2,1,3],0.0,"ds/ds has wrong value at (1,2,1,3).")
      self.assertEqual(dsds[1,2,1,4],0.0,"ds/ds has wrong value at (1,2,1,4).")
      self.assertEqual(dsds[1,2,2,0],0.0,"ds/ds has wrong value at (1,2,2,0).")
      self.assertEqual(dsds[1,2,2,1],0.0,"ds/ds has wrong value at (1,2,2,1).")
      self.assertEqual(dsds[1,2,2,2],0.0,"ds/ds has wrong value at (1,2,2,2).")
      self.assertEqual(dsds[1,2,2,3],0.0,"ds/ds has wrong value at (1,2,2,3).")
      self.assertEqual(dsds[1,2,2,4],0.0,"ds/ds has wrong value at (1,2,2,4).")
      self.assertEqual(dsds[1,2,3,0],0.0,"ds/ds has wrong value at (1,2,3,0).")
      self.assertEqual(dsds[1,2,3,1],0.0,"ds/ds has wrong value at (1,2,3,1).")
      self.assertEqual(dsds[1,2,3,2],0.0,"ds/ds has wrong value at (1,2,3,2).")
      self.assertEqual(dsds[1,2,3,3],0.0,"ds/ds has wrong value at (1,2,3,3).")
      self.assertEqual(dsds[1,2,3,4],0.0,"ds/ds has wrong value at (1,2,3,4).")
      self.assertEqual(dsds[1,3,0,0],0.0,"ds/ds has wrong value at (1,3,0,0).")
      self.assertEqual(dsds[1,3,0,1],0.0,"ds/ds has wrong value at (1,3,0,1).")
      self.assertEqual(dsds[1,3,0,2],0.0,"ds/ds has wrong value at (1,3,0,2).")
      self.assertEqual(dsds[1,3,0,3],0.0,"ds/ds has wrong value at (1,3,0,3).")
      self.assertEqual(dsds[1,3,0,4],0.0,"ds/ds has wrong value at (1,3,0,4).")
      self.assertEqual(dsds[1,3,1,0],0.0,"ds/ds has wrong value at (1,3,1,0).")
      self.assertEqual(dsds[1,3,1,1],0.0,"ds/ds has wrong value at (1,3,1,1).")
      self.assertEqual(dsds[1,3,1,2],0.0,"ds/ds has wrong value at (1,3,1,2).")
      self.assertEqual(dsds[1,3,1,3],1.0,"ds/ds has wrong value at (1,3,1,3).")
      self.assertEqual(dsds[1,3,1,4],0.0,"ds/ds has wrong value at (1,3,1,4).")
      self.assertEqual(dsds[1,3,2,0],0.0,"ds/ds has wrong value at (1,3,2,0).")
      self.assertEqual(dsds[1,3,2,1],0.0,"ds/ds has wrong value at (1,3,2,1).")
      self.assertEqual(dsds[1,3,2,2],0.0,"ds/ds has wrong value at (1,3,2,2).")
      self.assertEqual(dsds[1,3,2,3],0.0,"ds/ds has wrong value at (1,3,2,3).")
      self.assertEqual(dsds[1,3,2,4],0.0,"ds/ds has wrong value at (1,3,2,4).")
      self.assertEqual(dsds[1,3,3,0],0.0,"ds/ds has wrong value at (1,3,3,0).")
      self.assertEqual(dsds[1,3,3,1],0.0,"ds/ds has wrong value at (1,3,3,1).")
      self.assertEqual(dsds[1,3,3,2],0.0,"ds/ds has wrong value at (1,3,3,2).")
      self.assertEqual(dsds[1,3,3,3],0.0,"ds/ds has wrong value at (1,3,3,3).")
      self.assertEqual(dsds[1,3,3,4],0.0,"ds/ds has wrong value at (1,3,3,4).")
      self.assertEqual(dsds[1,4,0,0],0.0,"ds/ds has wrong value at (1,4,0,0).")
      self.assertEqual(dsds[1,4,0,1],0.0,"ds/ds has wrong value at (1,4,0,1).")
      self.assertEqual(dsds[1,4,0,2],0.0,"ds/ds has wrong value at (1,4,0,2).")
      self.assertEqual(dsds[1,4,0,3],0.0,"ds/ds has wrong value at (1,4,0,3).")
      self.assertEqual(dsds[1,4,0,4],0.0,"ds/ds has wrong value at (1,4,0,4).")
      self.assertEqual(dsds[1,4,1,0],0.0,"ds/ds has wrong value at (1,4,1,0).")
      self.assertEqual(dsds[1,4,1,1],0.0,"ds/ds has wrong value at (1,4,1,1).")
      self.assertEqual(dsds[1,4,1,2],0.0,"ds/ds has wrong value at (1,4,1,2).")
      self.assertEqual(dsds[1,4,1,3],0.0,"ds/ds has wrong value at (1,4,1,3).")
      self.assertEqual(dsds[1,4,1,4],1.0,"ds/ds has wrong value at (1,4,1,4).")
      self.assertEqual(dsds[1,4,2,0],0.0,"ds/ds has wrong value at (1,4,2,0).")
      self.assertEqual(dsds[1,4,2,1],0.0,"ds/ds has wrong value at (1,4,2,1).")
      self.assertEqual(dsds[1,4,2,2],0.0,"ds/ds has wrong value at (1,4,2,2).")
      self.assertEqual(dsds[1,4,2,3],0.0,"ds/ds has wrong value at (1,4,2,3).")
      self.assertEqual(dsds[1,4,2,4],0.0,"ds/ds has wrong value at (1,4,2,4).")
      self.assertEqual(dsds[1,4,3,0],0.0,"ds/ds has wrong value at (1,4,3,0).")
      self.assertEqual(dsds[1,4,3,1],0.0,"ds/ds has wrong value at (1,4,3,1).")
      self.assertEqual(dsds[1,4,3,2],0.0,"ds/ds has wrong value at (1,4,3,2).")
      self.assertEqual(dsds[1,4,3,3],0.0,"ds/ds has wrong value at (1,4,3,3).")
      self.assertEqual(dsds[1,4,3,4],0.0,"ds/ds has wrong value at (1,4,3,4).")
      self.assertEqual(dsds[2,0,0,0],0.0,"ds/ds has wrong value at (2,0,0,0).")
      self.assertEqual(dsds[2,0,0,1],0.0,"ds/ds has wrong value at (2,0,0,1).")
      self.assertEqual(dsds[2,0,0,2],0.0,"ds/ds has wrong value at (2,0,0,2).")
      self.assertEqual(dsds[2,0,0,3],0.0,"ds/ds has wrong value at (2,0,0,3).")
      self.assertEqual(dsds[2,0,0,4],0.0,"ds/ds has wrong value at (2,0,0,4).")
      self.assertEqual(dsds[2,0,1,0],0.0,"ds/ds has wrong value at (2,0,1,0).")
      self.assertEqual(dsds[2,0,1,1],0.0,"ds/ds has wrong value at (2,0,1,1).")
      self.assertEqual(dsds[2,0,1,2],0.0,"ds/ds has wrong value at (2,0,1,2).")
      self.assertEqual(dsds[2,0,1,3],0.0,"ds/ds has wrong value at (2,0,1,3).")
      self.assertEqual(dsds[2,0,1,4],0.0,"ds/ds has wrong value at (2,0,1,4).")
      self.assertEqual(dsds[2,0,2,0],1.0,"ds/ds has wrong value at (2,0,2,0).")
      self.assertEqual(dsds[2,0,2,1],0.0,"ds/ds has wrong value at (2,0,2,1).")
      self.assertEqual(dsds[2,0,2,2],0.0,"ds/ds has wrong value at (2,0,2,2).")
      self.assertEqual(dsds[2,0,2,3],0.0,"ds/ds has wrong value at (2,0,2,3).")
      self.assertEqual(dsds[2,0,2,4],0.0,"ds/ds has wrong value at (2,0,2,4).")
      self.assertEqual(dsds[2,0,3,0],0.0,"ds/ds has wrong value at (2,0,3,0).")
      self.assertEqual(dsds[2,0,3,1],0.0,"ds/ds has wrong value at (2,0,3,1).")
      self.assertEqual(dsds[2,0,3,2],0.0,"ds/ds has wrong value at (2,0,3,2).")
      self.assertEqual(dsds[2,0,3,3],0.0,"ds/ds has wrong value at (2,0,3,3).")
      self.assertEqual(dsds[2,0,3,4],0.0,"ds/ds has wrong value at (2,0,3,4).")
      self.assertEqual(dsds[2,1,0,0],0.0,"ds/ds has wrong value at (2,1,0,0).")
      self.assertEqual(dsds[2,1,0,1],0.0,"ds/ds has wrong value at (2,1,0,1).")
      self.assertEqual(dsds[2,1,0,2],0.0,"ds/ds has wrong value at (2,1,0,2).")
      self.assertEqual(dsds[2,1,0,3],0.0,"ds/ds has wrong value at (2,1,0,3).")
      self.assertEqual(dsds[2,1,0,4],0.0,"ds/ds has wrong value at (2,1,0,4).")
      self.assertEqual(dsds[2,1,1,0],0.0,"ds/ds has wrong value at (2,1,1,0).")
      self.assertEqual(dsds[2,1,1,1],0.0,"ds/ds has wrong value at (2,1,1,1).")
      self.assertEqual(dsds[2,1,1,2],0.0,"ds/ds has wrong value at (2,1,1,2).")
      self.assertEqual(dsds[2,1,1,3],0.0,"ds/ds has wrong value at (2,1,1,3).")
      self.assertEqual(dsds[2,1,1,4],0.0,"ds/ds has wrong value at (2,1,1,4).")
      self.assertEqual(dsds[2,1,2,0],0.0,"ds/ds has wrong value at (2,1,2,0).")
      self.assertEqual(dsds[2,1,2,1],1.0,"ds/ds has wrong value at (2,1,2,1).")
      self.assertEqual(dsds[2,1,2,2],0.0,"ds/ds has wrong value at (2,1,2,2).")
      self.assertEqual(dsds[2,1,2,3],0.0,"ds/ds has wrong value at (2,1,2,3).")
      self.assertEqual(dsds[2,1,2,4],0.0,"ds/ds has wrong value at (2,1,2,4).")
      self.assertEqual(dsds[2,1,3,0],0.0,"ds/ds has wrong value at (2,1,3,0).")
      self.assertEqual(dsds[2,1,3,1],0.0,"ds/ds has wrong value at (2,1,3,1).")
      self.assertEqual(dsds[2,1,3,2],0.0,"ds/ds has wrong value at (2,1,3,2).")
      self.assertEqual(dsds[2,1,3,3],0.0,"ds/ds has wrong value at (2,1,3,3).")
      self.assertEqual(dsds[2,1,3,4],0.0,"ds/ds has wrong value at (2,1,3,4).")
      self.assertEqual(dsds[2,2,0,0],0.0,"ds/ds has wrong value at (2,2,0,0).")
      self.assertEqual(dsds[2,2,0,1],0.0,"ds/ds has wrong value at (2,2,0,1).")
      self.assertEqual(dsds[2,2,0,2],0.0,"ds/ds has wrong value at (2,2,0,2).")
      self.assertEqual(dsds[2,2,0,3],0.0,"ds/ds has wrong value at (2,2,0,3).")
      self.assertEqual(dsds[2,2,0,4],0.0,"ds/ds has wrong value at (2,2,0,4).")
      self.assertEqual(dsds[2,2,1,0],0.0,"ds/ds has wrong value at (2,2,1,0).")
      self.assertEqual(dsds[2,2,1,1],0.0,"ds/ds has wrong value at (2,2,1,1).")
      self.assertEqual(dsds[2,2,1,2],0.0,"ds/ds has wrong value at (2,2,1,2).")
      self.assertEqual(dsds[2,2,1,3],0.0,"ds/ds has wrong value at (2,2,1,3).")
      self.assertEqual(dsds[2,2,1,4],0.0,"ds/ds has wrong value at (2,2,1,4).")
      self.assertEqual(dsds[2,2,2,0],0.0,"ds/ds has wrong value at (2,2,2,0).")
      self.assertEqual(dsds[2,2,2,1],0.0,"ds/ds has wrong value at (2,2,2,1).")
      self.assertEqual(dsds[2,2,2,2],1.0,"ds/ds has wrong value at (2,2,2,2).")
      self.assertEqual(dsds[2,2,2,3],0.0,"ds/ds has wrong value at (2,2,2,3).")
      self.assertEqual(dsds[2,2,2,4],0.0,"ds/ds has wrong value at (2,2,2,4).")
      self.assertEqual(dsds[2,2,3,0],0.0,"ds/ds has wrong value at (2,2,3,0).")
      self.assertEqual(dsds[2,2,3,1],0.0,"ds/ds has wrong value at (2,2,3,1).")
      self.assertEqual(dsds[2,2,3,2],0.0,"ds/ds has wrong value at (2,2,3,2).")
      self.assertEqual(dsds[2,2,3,3],0.0,"ds/ds has wrong value at (2,2,3,3).")
      self.assertEqual(dsds[2,2,3,4],0.0,"ds/ds has wrong value at (2,2,3,4).")
      self.assertEqual(dsds[2,3,0,0],0.0,"ds/ds has wrong value at (2,3,0,0).")
      self.assertEqual(dsds[2,3,0,1],0.0,"ds/ds has wrong value at (2,3,0,1).")
      self.assertEqual(dsds[2,3,0,2],0.0,"ds/ds has wrong value at (2,3,0,2).")
      self.assertEqual(dsds[2,3,0,3],0.0,"ds/ds has wrong value at (2,3,0,3).")
      self.assertEqual(dsds[2,3,0,4],0.0,"ds/ds has wrong value at (2,3,0,4).")
      self.assertEqual(dsds[2,3,1,0],0.0,"ds/ds has wrong value at (2,3,1,0).")
      self.assertEqual(dsds[2,3,1,1],0.0,"ds/ds has wrong value at (2,3,1,1).")
      self.assertEqual(dsds[2,3,1,2],0.0,"ds/ds has wrong value at (2,3,1,2).")
      self.assertEqual(dsds[2,3,1,3],0.0,"ds/ds has wrong value at (2,3,1,3).")
      self.assertEqual(dsds[2,3,1,4],0.0,"ds/ds has wrong value at (2,3,1,4).")
      self.assertEqual(dsds[2,3,2,0],0.0,"ds/ds has wrong value at (2,3,2,0).")
      self.assertEqual(dsds[2,3,2,1],0.0,"ds/ds has wrong value at (2,3,2,1).")
      self.assertEqual(dsds[2,3,2,2],0.0,"ds/ds has wrong value at (2,3,2,2).")
      self.assertEqual(dsds[2,3,2,3],1.0,"ds/ds has wrong value at (2,3,2,3).")
      self.assertEqual(dsds[2,3,2,4],0.0,"ds/ds has wrong value at (2,3,2,4).")
      self.assertEqual(dsds[2,3,3,0],0.0,"ds/ds has wrong value at (2,3,3,0).")
      self.assertEqual(dsds[2,3,3,1],0.0,"ds/ds has wrong value at (2,3,3,1).")
      self.assertEqual(dsds[2,3,3,2],0.0,"ds/ds has wrong value at (2,3,3,2).")
      self.assertEqual(dsds[2,3,3,3],0.0,"ds/ds has wrong value at (2,3,3,3).")
      self.assertEqual(dsds[2,3,3,4],0.0,"ds/ds has wrong value at (2,3,3,4).")
      self.assertEqual(dsds[2,4,0,0],0.0,"ds/ds has wrong value at (2,4,0,0).")
      self.assertEqual(dsds[2,4,0,1],0.0,"ds/ds has wrong value at (2,4,0,1).")
      self.assertEqual(dsds[2,4,0,2],0.0,"ds/ds has wrong value at (2,4,0,2).")
      self.assertEqual(dsds[2,4,0,3],0.0,"ds/ds has wrong value at (2,4,0,3).")
      self.assertEqual(dsds[2,4,0,4],0.0,"ds/ds has wrong value at (2,4,0,4).")
      self.assertEqual(dsds[2,4,1,0],0.0,"ds/ds has wrong value at (2,4,1,0).")
      self.assertEqual(dsds[2,4,1,1],0.0,"ds/ds has wrong value at (2,4,1,1).")
      self.assertEqual(dsds[2,4,1,2],0.0,"ds/ds has wrong value at (2,4,1,2).")
      self.assertEqual(dsds[2,4,1,3],0.0,"ds/ds has wrong value at (2,4,1,3).")
      self.assertEqual(dsds[2,4,1,4],0.0,"ds/ds has wrong value at (2,4,1,4).")
      self.assertEqual(dsds[2,4,2,0],0.0,"ds/ds has wrong value at (2,4,2,0).")
      self.assertEqual(dsds[2,4,2,1],0.0,"ds/ds has wrong value at (2,4,2,1).")
      self.assertEqual(dsds[2,4,2,2],0.0,"ds/ds has wrong value at (2,4,2,2).")
      self.assertEqual(dsds[2,4,2,3],0.0,"ds/ds has wrong value at (2,4,2,3).")
      self.assertEqual(dsds[2,4,2,4],1.0,"ds/ds has wrong value at (2,4,2,4).")
      self.assertEqual(dsds[2,4,3,0],0.0,"ds/ds has wrong value at (2,4,3,0).")
      self.assertEqual(dsds[2,4,3,1],0.0,"ds/ds has wrong value at (2,4,3,1).")
      self.assertEqual(dsds[2,4,3,2],0.0,"ds/ds has wrong value at (2,4,3,2).")
      self.assertEqual(dsds[2,4,3,3],0.0,"ds/ds has wrong value at (2,4,3,3).")
      self.assertEqual(dsds[2,4,3,4],0.0,"ds/ds has wrong value at (2,4,3,4).")
      self.assertEqual(dsds[3,0,0,0],0.0,"ds/ds has wrong value at (3,0,0,0).")
      self.assertEqual(dsds[3,0,0,1],0.0,"ds/ds has wrong value at (3,0,0,1).")
      self.assertEqual(dsds[3,0,0,2],0.0,"ds/ds has wrong value at (3,0,0,2).")
      self.assertEqual(dsds[3,0,0,3],0.0,"ds/ds has wrong value at (3,0,0,3).")
      self.assertEqual(dsds[3,0,0,4],0.0,"ds/ds has wrong value at (3,0,0,4).")
      self.assertEqual(dsds[3,0,1,0],0.0,"ds/ds has wrong value at (3,0,1,0).")
      self.assertEqual(dsds[3,0,1,1],0.0,"ds/ds has wrong value at (3,0,1,1).")
      self.assertEqual(dsds[3,0,1,2],0.0,"ds/ds has wrong value at (3,0,1,2).")
      self.assertEqual(dsds[3,0,1,3],0.0,"ds/ds has wrong value at (3,0,1,3).")
      self.assertEqual(dsds[3,0,1,4],0.0,"ds/ds has wrong value at (3,0,1,4).")
      self.assertEqual(dsds[3,0,2,0],0.0,"ds/ds has wrong value at (3,0,2,0).")
      self.assertEqual(dsds[3,0,2,1],0.0,"ds/ds has wrong value at (3,0,2,1).")
      self.assertEqual(dsds[3,0,2,2],0.0,"ds/ds has wrong value at (3,0,2,2).")
      self.assertEqual(dsds[3,0,2,3],0.0,"ds/ds has wrong value at (3,0,2,3).")
      self.assertEqual(dsds[3,0,2,4],0.0,"ds/ds has wrong value at (3,0,2,4).")
      self.assertEqual(dsds[3,0,3,0],1.0,"ds/ds has wrong value at (3,0,3,0).")
      self.assertEqual(dsds[3,0,3,1],0.0,"ds/ds has wrong value at (3,0,3,1).")
      self.assertEqual(dsds[3,0,3,2],0.0,"ds/ds has wrong value at (3,0,3,2).")
      self.assertEqual(dsds[3,0,3,3],0.0,"ds/ds has wrong value at (3,0,3,3).")
      self.assertEqual(dsds[3,0,3,4],0.0,"ds/ds has wrong value at (3,0,3,4).")
      self.assertEqual(dsds[3,1,0,0],0.0,"ds/ds has wrong value at (3,1,0,0).")
      self.assertEqual(dsds[3,1,0,1],0.0,"ds/ds has wrong value at (3,1,0,1).")
      self.assertEqual(dsds[3,1,0,2],0.0,"ds/ds has wrong value at (3,1,0,2).")
      self.assertEqual(dsds[3,1,0,3],0.0,"ds/ds has wrong value at (3,1,0,3).")
      self.assertEqual(dsds[3,1,0,4],0.0,"ds/ds has wrong value at (3,1,0,4).")
      self.assertEqual(dsds[3,1,1,0],0.0,"ds/ds has wrong value at (3,1,1,0).")
      self.assertEqual(dsds[3,1,1,1],0.0,"ds/ds has wrong value at (3,1,1,1).")
      self.assertEqual(dsds[3,1,1,2],0.0,"ds/ds has wrong value at (3,1,1,2).")
      self.assertEqual(dsds[3,1,1,3],0.0,"ds/ds has wrong value at (3,1,1,3).")
      self.assertEqual(dsds[3,1,1,4],0.0,"ds/ds has wrong value at (3,1,1,4).")
      self.assertEqual(dsds[3,1,2,0],0.0,"ds/ds has wrong value at (3,1,2,0).")
      self.assertEqual(dsds[3,1,2,1],0.0,"ds/ds has wrong value at (3,1,2,1).")
      self.assertEqual(dsds[3,1,2,2],0.0,"ds/ds has wrong value at (3,1,2,2).")
      self.assertEqual(dsds[3,1,2,3],0.0,"ds/ds has wrong value at (3,1,2,3).")
      self.assertEqual(dsds[3,1,2,4],0.0,"ds/ds has wrong value at (3,1,2,4).")
      self.assertEqual(dsds[3,1,3,0],0.0,"ds/ds has wrong value at (3,1,3,0).")
      self.assertEqual(dsds[3,1,3,1],1.0,"ds/ds has wrong value at (3,1,3,1).")
      self.assertEqual(dsds[3,1,3,2],0.0,"ds/ds has wrong value at (3,1,3,2).")
      self.assertEqual(dsds[3,1,3,3],0.0,"ds/ds has wrong value at (3,1,3,3).")
      self.assertEqual(dsds[3,1,3,4],0.0,"ds/ds has wrong value at (3,1,3,4).")
      self.assertEqual(dsds[3,2,0,0],0.0,"ds/ds has wrong value at (3,2,0,0).")
      self.assertEqual(dsds[3,2,0,1],0.0,"ds/ds has wrong value at (3,2,0,1).")
      self.assertEqual(dsds[3,2,0,2],0.0,"ds/ds has wrong value at (3,2,0,2).")
      self.assertEqual(dsds[3,2,0,3],0.0,"ds/ds has wrong value at (3,2,0,3).")
      self.assertEqual(dsds[3,2,0,4],0.0,"ds/ds has wrong value at (3,2,0,4).")
      self.assertEqual(dsds[3,2,1,0],0.0,"ds/ds has wrong value at (3,2,1,0).")
      self.assertEqual(dsds[3,2,1,1],0.0,"ds/ds has wrong value at (3,2,1,1).")
      self.assertEqual(dsds[3,2,1,2],0.0,"ds/ds has wrong value at (3,2,1,2).")
      self.assertEqual(dsds[3,2,1,3],0.0,"ds/ds has wrong value at (3,2,1,3).")
      self.assertEqual(dsds[3,2,1,4],0.0,"ds/ds has wrong value at (3,2,1,4).")
      self.assertEqual(dsds[3,2,2,0],0.0,"ds/ds has wrong value at (3,2,2,0).")
      self.assertEqual(dsds[3,2,2,1],0.0,"ds/ds has wrong value at (3,2,2,1).")
      self.assertEqual(dsds[3,2,2,2],0.0,"ds/ds has wrong value at (3,2,2,2).")
      self.assertEqual(dsds[3,2,2,3],0.0,"ds/ds has wrong value at (3,2,2,3).")
      self.assertEqual(dsds[3,2,2,4],0.0,"ds/ds has wrong value at (3,2,2,4).")
      self.assertEqual(dsds[3,2,3,0],0.0,"ds/ds has wrong value at (3,2,3,0).")
      self.assertEqual(dsds[3,2,3,1],0.0,"ds/ds has wrong value at (3,2,3,1).")
      self.assertEqual(dsds[3,2,3,2],1.0,"ds/ds has wrong value at (3,2,3,2).")
      self.assertEqual(dsds[3,2,3,3],0.0,"ds/ds has wrong value at (3,2,3,3).")
      self.assertEqual(dsds[3,2,3,4],0.0,"ds/ds has wrong value at (3,2,3,4).")
      self.assertEqual(dsds[3,3,0,0],0.0,"ds/ds has wrong value at (3,3,0,0).")
      self.assertEqual(dsds[3,3,0,1],0.0,"ds/ds has wrong value at (3,3,0,1).")
      self.assertEqual(dsds[3,3,0,2],0.0,"ds/ds has wrong value at (3,3,0,2).")
      self.assertEqual(dsds[3,3,0,3],0.0,"ds/ds has wrong value at (3,3,0,3).")
      self.assertEqual(dsds[3,3,0,4],0.0,"ds/ds has wrong value at (3,3,0,4).")
      self.assertEqual(dsds[3,3,1,0],0.0,"ds/ds has wrong value at (3,3,1,0).")
      self.assertEqual(dsds[3,3,1,1],0.0,"ds/ds has wrong value at (3,3,1,1).")
      self.assertEqual(dsds[3,3,1,2],0.0,"ds/ds has wrong value at (3,3,1,2).")
      self.assertEqual(dsds[3,3,1,3],0.0,"ds/ds has wrong value at (3,3,1,3).")
      self.assertEqual(dsds[3,3,1,4],0.0,"ds/ds has wrong value at (3,3,1,4).")
      self.assertEqual(dsds[3,3,2,0],0.0,"ds/ds has wrong value at (3,3,2,0).")
      self.assertEqual(dsds[3,3,2,1],0.0,"ds/ds has wrong value at (3,3,2,1).")
      self.assertEqual(dsds[3,3,2,2],0.0,"ds/ds has wrong value at (3,3,2,2).")
      self.assertEqual(dsds[3,3,2,3],0.0,"ds/ds has wrong value at (3,3,2,3).")
      self.assertEqual(dsds[3,3,2,4],0.0,"ds/ds has wrong value at (3,3,2,4).")
      self.assertEqual(dsds[3,3,3,0],0.0,"ds/ds has wrong value at (3,3,3,0).")
      self.assertEqual(dsds[3,3,3,1],0.0,"ds/ds has wrong value at (3,3,3,1).")
      self.assertEqual(dsds[3,3,3,2],0.0,"ds/ds has wrong value at (3,3,3,2).")
      self.assertEqual(dsds[3,3,3,3],1.0,"ds/ds has wrong value at (3,3,3,3).")
      self.assertEqual(dsds[3,3,3,4],0.0,"ds/ds has wrong value at (3,3,3,4).")
      self.assertEqual(dsds[3,4,0,0],0.0,"ds/ds has wrong value at (3,4,0,0).")
      self.assertEqual(dsds[3,4,0,1],0.0,"ds/ds has wrong value at (3,4,0,1).")
      self.assertEqual(dsds[3,4,0,2],0.0,"ds/ds has wrong value at (3,4,0,2).")
      self.assertEqual(dsds[3,4,0,3],0.0,"ds/ds has wrong value at (3,4,0,3).")
      self.assertEqual(dsds[3,4,0,4],0.0,"ds/ds has wrong value at (3,4,0,4).")
      self.assertEqual(dsds[3,4,1,0],0.0,"ds/ds has wrong value at (3,4,1,0).")
      self.assertEqual(dsds[3,4,1,1],0.0,"ds/ds has wrong value at (3,4,1,1).")
      self.assertEqual(dsds[3,4,1,2],0.0,"ds/ds has wrong value at (3,4,1,2).")
      self.assertEqual(dsds[3,4,1,3],0.0,"ds/ds has wrong value at (3,4,1,3).")
      self.assertEqual(dsds[3,4,1,4],0.0,"ds/ds has wrong value at (3,4,1,4).")
      self.assertEqual(dsds[3,4,2,0],0.0,"ds/ds has wrong value at (3,4,2,0).")
      self.assertEqual(dsds[3,4,2,1],0.0,"ds/ds has wrong value at (3,4,2,1).")
      self.assertEqual(dsds[3,4,2,2],0.0,"ds/ds has wrong value at (3,4,2,2).")
      self.assertEqual(dsds[3,4,2,3],0.0,"ds/ds has wrong value at (3,4,2,3).")
      self.assertEqual(dsds[3,4,2,4],0.0,"ds/ds has wrong value at (3,4,2,4).")
      self.assertEqual(dsds[3,4,3,0],0.0,"ds/ds has wrong value at (3,4,3,0).")
      self.assertEqual(dsds[3,4,3,1],0.0,"ds/ds has wrong value at (3,4,3,1).")
      self.assertEqual(dsds[3,4,3,2],0.0,"ds/ds has wrong value at (3,4,3,2).")
      self.assertEqual(dsds[3,4,3,3],0.0,"ds/ds has wrong value at (3,4,3,3).")
      self.assertEqual(dsds[3,4,3,4],1.0,"ds/ds has wrong value at (3,4,3,4).")

      self.assertEqual(s.getArgument(0),1,"wrong argument 0.")
      self.assertEqual(s.getArgument(1),-1.0,"wrong argument 1.")
      sa=s.getSubstitutedArguments({s2:-10})
      self.assertEqual(len(sa),2,"wrong number of substituted arguments")
      self.assertEqual(sa[0],1,"wrongly substituted argument 0.")
      self.assertEqual(sa[1],-1.0,"wrongly substituted argument 1.")

      self.assertFalse(s.isAppropriateValue(10.)," 10. is not appropriate substitute")
      self.assertFalse(s.isAppropriateValue(10)," 10 is not appropriate substitute")
      self.assertTrue(s.isAppropriateValue(Symbol(shape=(4, 5),dim=self.functionspace.getDim())),"Symbol is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(Symbol(shape=(5,))),"Symbol is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(escript.Data(0.,(4, 5),self.functionspace)),"Data is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(escript.Data(0.,(5,),self.functionspace)),"Data is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(numpy.zeros((4, 5))),"ndarray is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(numpy.zeros((5,))),"ndarray is not appropriate substitute (shape)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_Symbol_rank2_d1_nargs0(self):
      s2=Symbol(shape=())
      s=Symbol(shape=(4, 5),dim=1,args=[])
      self.assertEqual(s.getRank(),2,"wrong rank.")
      self.assertEqual(s.getShape(),(4, 5),"wrong shape.")
      self.assertEqual(s.getDim(),1,"wrong spatial dimension.")
      self.assertEqual(s.getArgument(),[],"wrong arguments.")

      ss=s.substitute({s:numpy.zeros((4, 5))})
      self.assertTrue(isinstance(ss,numpy.ndarray),"value after substitution is not ndarray.")
      self.assertEqual(ss.shape,(4, 5),"value after substitution has not expected shape")
      try:
        s.substitute({s:numpy.zeros((5,))})
        fail("illegal substition was successful")
      except TypeError:
        pass

      dsdarg=s.diff(Symbol(shape=()))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds() has wrong type.")
      self.assertEqual(dsdarg.shape,(4, 5),"ds/ds() has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds() has wrong value.")
      dsdarg=s.diff(Symbol(shape=(2,)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(2,) has wrong type.")
      self.assertEqual(dsdarg.shape,(4, 5, 2),"ds/ds(2,) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(2,) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(4, 5)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(4, 5) has wrong type.")
      self.assertEqual(dsdarg.shape,(4, 5, 4, 5),"ds/ds(4, 5) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(4, 5) has wrong value.")

      dsds=s.diff(s)
      self.assertTrue(isinstance(dsds,numpy.ndarray),"ds/ds has wrong type.")
      self.assertEqual(dsds.shape,(4, 5, 4, 5),"ds/ds has wrong shape.")
      self.assertEqual(dsds[0,0,0,0],1.0,"ds/ds has wrong value at (0,0,0,0).")
      self.assertEqual(dsds[0,0,0,1],0.0,"ds/ds has wrong value at (0,0,0,1).")
      self.assertEqual(dsds[0,0,0,2],0.0,"ds/ds has wrong value at (0,0,0,2).")
      self.assertEqual(dsds[0,0,0,3],0.0,"ds/ds has wrong value at (0,0,0,3).")
      self.assertEqual(dsds[0,0,0,4],0.0,"ds/ds has wrong value at (0,0,0,4).")
      self.assertEqual(dsds[0,0,1,0],0.0,"ds/ds has wrong value at (0,0,1,0).")
      self.assertEqual(dsds[0,0,1,1],0.0,"ds/ds has wrong value at (0,0,1,1).")
      self.assertEqual(dsds[0,0,1,2],0.0,"ds/ds has wrong value at (0,0,1,2).")
      self.assertEqual(dsds[0,0,1,3],0.0,"ds/ds has wrong value at (0,0,1,3).")
      self.assertEqual(dsds[0,0,1,4],0.0,"ds/ds has wrong value at (0,0,1,4).")
      self.assertEqual(dsds[0,0,2,0],0.0,"ds/ds has wrong value at (0,0,2,0).")
      self.assertEqual(dsds[0,0,2,1],0.0,"ds/ds has wrong value at (0,0,2,1).")
      self.assertEqual(dsds[0,0,2,2],0.0,"ds/ds has wrong value at (0,0,2,2).")
      self.assertEqual(dsds[0,0,2,3],0.0,"ds/ds has wrong value at (0,0,2,3).")
      self.assertEqual(dsds[0,0,2,4],0.0,"ds/ds has wrong value at (0,0,2,4).")
      self.assertEqual(dsds[0,0,3,0],0.0,"ds/ds has wrong value at (0,0,3,0).")
      self.assertEqual(dsds[0,0,3,1],0.0,"ds/ds has wrong value at (0,0,3,1).")
      self.assertEqual(dsds[0,0,3,2],0.0,"ds/ds has wrong value at (0,0,3,2).")
      self.assertEqual(dsds[0,0,3,3],0.0,"ds/ds has wrong value at (0,0,3,3).")
      self.assertEqual(dsds[0,0,3,4],0.0,"ds/ds has wrong value at (0,0,3,4).")
      self.assertEqual(dsds[0,1,0,0],0.0,"ds/ds has wrong value at (0,1,0,0).")
      self.assertEqual(dsds[0,1,0,1],1.0,"ds/ds has wrong value at (0,1,0,1).")
      self.assertEqual(dsds[0,1,0,2],0.0,"ds/ds has wrong value at (0,1,0,2).")
      self.assertEqual(dsds[0,1,0,3],0.0,"ds/ds has wrong value at (0,1,0,3).")
      self.assertEqual(dsds[0,1,0,4],0.0,"ds/ds has wrong value at (0,1,0,4).")
      self.assertEqual(dsds[0,1,1,0],0.0,"ds/ds has wrong value at (0,1,1,0).")
      self.assertEqual(dsds[0,1,1,1],0.0,"ds/ds has wrong value at (0,1,1,1).")
      self.assertEqual(dsds[0,1,1,2],0.0,"ds/ds has wrong value at (0,1,1,2).")
      self.assertEqual(dsds[0,1,1,3],0.0,"ds/ds has wrong value at (0,1,1,3).")
      self.assertEqual(dsds[0,1,1,4],0.0,"ds/ds has wrong value at (0,1,1,4).")
      self.assertEqual(dsds[0,1,2,0],0.0,"ds/ds has wrong value at (0,1,2,0).")
      self.assertEqual(dsds[0,1,2,1],0.0,"ds/ds has wrong value at (0,1,2,1).")
      self.assertEqual(dsds[0,1,2,2],0.0,"ds/ds has wrong value at (0,1,2,2).")
      self.assertEqual(dsds[0,1,2,3],0.0,"ds/ds has wrong value at (0,1,2,3).")
      self.assertEqual(dsds[0,1,2,4],0.0,"ds/ds has wrong value at (0,1,2,4).")
      self.assertEqual(dsds[0,1,3,0],0.0,"ds/ds has wrong value at (0,1,3,0).")
      self.assertEqual(dsds[0,1,3,1],0.0,"ds/ds has wrong value at (0,1,3,1).")
      self.assertEqual(dsds[0,1,3,2],0.0,"ds/ds has wrong value at (0,1,3,2).")
      self.assertEqual(dsds[0,1,3,3],0.0,"ds/ds has wrong value at (0,1,3,3).")
      self.assertEqual(dsds[0,1,3,4],0.0,"ds/ds has wrong value at (0,1,3,4).")
      self.assertEqual(dsds[0,2,0,0],0.0,"ds/ds has wrong value at (0,2,0,0).")
      self.assertEqual(dsds[0,2,0,1],0.0,"ds/ds has wrong value at (0,2,0,1).")
      self.assertEqual(dsds[0,2,0,2],1.0,"ds/ds has wrong value at (0,2,0,2).")
      self.assertEqual(dsds[0,2,0,3],0.0,"ds/ds has wrong value at (0,2,0,3).")
      self.assertEqual(dsds[0,2,0,4],0.0,"ds/ds has wrong value at (0,2,0,4).")
      self.assertEqual(dsds[0,2,1,0],0.0,"ds/ds has wrong value at (0,2,1,0).")
      self.assertEqual(dsds[0,2,1,1],0.0,"ds/ds has wrong value at (0,2,1,1).")
      self.assertEqual(dsds[0,2,1,2],0.0,"ds/ds has wrong value at (0,2,1,2).")
      self.assertEqual(dsds[0,2,1,3],0.0,"ds/ds has wrong value at (0,2,1,3).")
      self.assertEqual(dsds[0,2,1,4],0.0,"ds/ds has wrong value at (0,2,1,4).")
      self.assertEqual(dsds[0,2,2,0],0.0,"ds/ds has wrong value at (0,2,2,0).")
      self.assertEqual(dsds[0,2,2,1],0.0,"ds/ds has wrong value at (0,2,2,1).")
      self.assertEqual(dsds[0,2,2,2],0.0,"ds/ds has wrong value at (0,2,2,2).")
      self.assertEqual(dsds[0,2,2,3],0.0,"ds/ds has wrong value at (0,2,2,3).")
      self.assertEqual(dsds[0,2,2,4],0.0,"ds/ds has wrong value at (0,2,2,4).")
      self.assertEqual(dsds[0,2,3,0],0.0,"ds/ds has wrong value at (0,2,3,0).")
      self.assertEqual(dsds[0,2,3,1],0.0,"ds/ds has wrong value at (0,2,3,1).")
      self.assertEqual(dsds[0,2,3,2],0.0,"ds/ds has wrong value at (0,2,3,2).")
      self.assertEqual(dsds[0,2,3,3],0.0,"ds/ds has wrong value at (0,2,3,3).")
      self.assertEqual(dsds[0,2,3,4],0.0,"ds/ds has wrong value at (0,2,3,4).")
      self.assertEqual(dsds[0,3,0,0],0.0,"ds/ds has wrong value at (0,3,0,0).")
      self.assertEqual(dsds[0,3,0,1],0.0,"ds/ds has wrong value at (0,3,0,1).")
      self.assertEqual(dsds[0,3,0,2],0.0,"ds/ds has wrong value at (0,3,0,2).")
      self.assertEqual(dsds[0,3,0,3],1.0,"ds/ds has wrong value at (0,3,0,3).")
      self.assertEqual(dsds[0,3,0,4],0.0,"ds/ds has wrong value at (0,3,0,4).")
      self.assertEqual(dsds[0,3,1,0],0.0,"ds/ds has wrong value at (0,3,1,0).")
      self.assertEqual(dsds[0,3,1,1],0.0,"ds/ds has wrong value at (0,3,1,1).")
      self.assertEqual(dsds[0,3,1,2],0.0,"ds/ds has wrong value at (0,3,1,2).")
      self.assertEqual(dsds[0,3,1,3],0.0,"ds/ds has wrong value at (0,3,1,3).")
      self.assertEqual(dsds[0,3,1,4],0.0,"ds/ds has wrong value at (0,3,1,4).")
      self.assertEqual(dsds[0,3,2,0],0.0,"ds/ds has wrong value at (0,3,2,0).")
      self.assertEqual(dsds[0,3,2,1],0.0,"ds/ds has wrong value at (0,3,2,1).")
      self.assertEqual(dsds[0,3,2,2],0.0,"ds/ds has wrong value at (0,3,2,2).")
      self.assertEqual(dsds[0,3,2,3],0.0,"ds/ds has wrong value at (0,3,2,3).")
      self.assertEqual(dsds[0,3,2,4],0.0,"ds/ds has wrong value at (0,3,2,4).")
      self.assertEqual(dsds[0,3,3,0],0.0,"ds/ds has wrong value at (0,3,3,0).")
      self.assertEqual(dsds[0,3,3,1],0.0,"ds/ds has wrong value at (0,3,3,1).")
      self.assertEqual(dsds[0,3,3,2],0.0,"ds/ds has wrong value at (0,3,3,2).")
      self.assertEqual(dsds[0,3,3,3],0.0,"ds/ds has wrong value at (0,3,3,3).")
      self.assertEqual(dsds[0,3,3,4],0.0,"ds/ds has wrong value at (0,3,3,4).")
      self.assertEqual(dsds[0,4,0,0],0.0,"ds/ds has wrong value at (0,4,0,0).")
      self.assertEqual(dsds[0,4,0,1],0.0,"ds/ds has wrong value at (0,4,0,1).")
      self.assertEqual(dsds[0,4,0,2],0.0,"ds/ds has wrong value at (0,4,0,2).")
      self.assertEqual(dsds[0,4,0,3],0.0,"ds/ds has wrong value at (0,4,0,3).")
      self.assertEqual(dsds[0,4,0,4],1.0,"ds/ds has wrong value at (0,4,0,4).")
      self.assertEqual(dsds[0,4,1,0],0.0,"ds/ds has wrong value at (0,4,1,0).")
      self.assertEqual(dsds[0,4,1,1],0.0,"ds/ds has wrong value at (0,4,1,1).")
      self.assertEqual(dsds[0,4,1,2],0.0,"ds/ds has wrong value at (0,4,1,2).")
      self.assertEqual(dsds[0,4,1,3],0.0,"ds/ds has wrong value at (0,4,1,3).")
      self.assertEqual(dsds[0,4,1,4],0.0,"ds/ds has wrong value at (0,4,1,4).")
      self.assertEqual(dsds[0,4,2,0],0.0,"ds/ds has wrong value at (0,4,2,0).")
      self.assertEqual(dsds[0,4,2,1],0.0,"ds/ds has wrong value at (0,4,2,1).")
      self.assertEqual(dsds[0,4,2,2],0.0,"ds/ds has wrong value at (0,4,2,2).")
      self.assertEqual(dsds[0,4,2,3],0.0,"ds/ds has wrong value at (0,4,2,3).")
      self.assertEqual(dsds[0,4,2,4],0.0,"ds/ds has wrong value at (0,4,2,4).")
      self.assertEqual(dsds[0,4,3,0],0.0,"ds/ds has wrong value at (0,4,3,0).")
      self.assertEqual(dsds[0,4,3,1],0.0,"ds/ds has wrong value at (0,4,3,1).")
      self.assertEqual(dsds[0,4,3,2],0.0,"ds/ds has wrong value at (0,4,3,2).")
      self.assertEqual(dsds[0,4,3,3],0.0,"ds/ds has wrong value at (0,4,3,3).")
      self.assertEqual(dsds[0,4,3,4],0.0,"ds/ds has wrong value at (0,4,3,4).")
      self.assertEqual(dsds[1,0,0,0],0.0,"ds/ds has wrong value at (1,0,0,0).")
      self.assertEqual(dsds[1,0,0,1],0.0,"ds/ds has wrong value at (1,0,0,1).")
      self.assertEqual(dsds[1,0,0,2],0.0,"ds/ds has wrong value at (1,0,0,2).")
      self.assertEqual(dsds[1,0,0,3],0.0,"ds/ds has wrong value at (1,0,0,3).")
      self.assertEqual(dsds[1,0,0,4],0.0,"ds/ds has wrong value at (1,0,0,4).")
      self.assertEqual(dsds[1,0,1,0],1.0,"ds/ds has wrong value at (1,0,1,0).")
      self.assertEqual(dsds[1,0,1,1],0.0,"ds/ds has wrong value at (1,0,1,1).")
      self.assertEqual(dsds[1,0,1,2],0.0,"ds/ds has wrong value at (1,0,1,2).")
      self.assertEqual(dsds[1,0,1,3],0.0,"ds/ds has wrong value at (1,0,1,3).")
      self.assertEqual(dsds[1,0,1,4],0.0,"ds/ds has wrong value at (1,0,1,4).")
      self.assertEqual(dsds[1,0,2,0],0.0,"ds/ds has wrong value at (1,0,2,0).")
      self.assertEqual(dsds[1,0,2,1],0.0,"ds/ds has wrong value at (1,0,2,1).")
      self.assertEqual(dsds[1,0,2,2],0.0,"ds/ds has wrong value at (1,0,2,2).")
      self.assertEqual(dsds[1,0,2,3],0.0,"ds/ds has wrong value at (1,0,2,3).")
      self.assertEqual(dsds[1,0,2,4],0.0,"ds/ds has wrong value at (1,0,2,4).")
      self.assertEqual(dsds[1,0,3,0],0.0,"ds/ds has wrong value at (1,0,3,0).")
      self.assertEqual(dsds[1,0,3,1],0.0,"ds/ds has wrong value at (1,0,3,1).")
      self.assertEqual(dsds[1,0,3,2],0.0,"ds/ds has wrong value at (1,0,3,2).")
      self.assertEqual(dsds[1,0,3,3],0.0,"ds/ds has wrong value at (1,0,3,3).")
      self.assertEqual(dsds[1,0,3,4],0.0,"ds/ds has wrong value at (1,0,3,4).")
      self.assertEqual(dsds[1,1,0,0],0.0,"ds/ds has wrong value at (1,1,0,0).")
      self.assertEqual(dsds[1,1,0,1],0.0,"ds/ds has wrong value at (1,1,0,1).")
      self.assertEqual(dsds[1,1,0,2],0.0,"ds/ds has wrong value at (1,1,0,2).")
      self.assertEqual(dsds[1,1,0,3],0.0,"ds/ds has wrong value at (1,1,0,3).")
      self.assertEqual(dsds[1,1,0,4],0.0,"ds/ds has wrong value at (1,1,0,4).")
      self.assertEqual(dsds[1,1,1,0],0.0,"ds/ds has wrong value at (1,1,1,0).")
      self.assertEqual(dsds[1,1,1,1],1.0,"ds/ds has wrong value at (1,1,1,1).")
      self.assertEqual(dsds[1,1,1,2],0.0,"ds/ds has wrong value at (1,1,1,2).")
      self.assertEqual(dsds[1,1,1,3],0.0,"ds/ds has wrong value at (1,1,1,3).")
      self.assertEqual(dsds[1,1,1,4],0.0,"ds/ds has wrong value at (1,1,1,4).")
      self.assertEqual(dsds[1,1,2,0],0.0,"ds/ds has wrong value at (1,1,2,0).")
      self.assertEqual(dsds[1,1,2,1],0.0,"ds/ds has wrong value at (1,1,2,1).")
      self.assertEqual(dsds[1,1,2,2],0.0,"ds/ds has wrong value at (1,1,2,2).")
      self.assertEqual(dsds[1,1,2,3],0.0,"ds/ds has wrong value at (1,1,2,3).")
      self.assertEqual(dsds[1,1,2,4],0.0,"ds/ds has wrong value at (1,1,2,4).")
      self.assertEqual(dsds[1,1,3,0],0.0,"ds/ds has wrong value at (1,1,3,0).")
      self.assertEqual(dsds[1,1,3,1],0.0,"ds/ds has wrong value at (1,1,3,1).")
      self.assertEqual(dsds[1,1,3,2],0.0,"ds/ds has wrong value at (1,1,3,2).")
      self.assertEqual(dsds[1,1,3,3],0.0,"ds/ds has wrong value at (1,1,3,3).")
      self.assertEqual(dsds[1,1,3,4],0.0,"ds/ds has wrong value at (1,1,3,4).")
      self.assertEqual(dsds[1,2,0,0],0.0,"ds/ds has wrong value at (1,2,0,0).")
      self.assertEqual(dsds[1,2,0,1],0.0,"ds/ds has wrong value at (1,2,0,1).")
      self.assertEqual(dsds[1,2,0,2],0.0,"ds/ds has wrong value at (1,2,0,2).")
      self.assertEqual(dsds[1,2,0,3],0.0,"ds/ds has wrong value at (1,2,0,3).")
      self.assertEqual(dsds[1,2,0,4],0.0,"ds/ds has wrong value at (1,2,0,4).")
      self.assertEqual(dsds[1,2,1,0],0.0,"ds/ds has wrong value at (1,2,1,0).")
      self.assertEqual(dsds[1,2,1,1],0.0,"ds/ds has wrong value at (1,2,1,1).")
      self.assertEqual(dsds[1,2,1,2],1.0,"ds/ds has wrong value at (1,2,1,2).")
      self.assertEqual(dsds[1,2,1,3],0.0,"ds/ds has wrong value at (1,2,1,3).")
      self.assertEqual(dsds[1,2,1,4],0.0,"ds/ds has wrong value at (1,2,1,4).")
      self.assertEqual(dsds[1,2,2,0],0.0,"ds/ds has wrong value at (1,2,2,0).")
      self.assertEqual(dsds[1,2,2,1],0.0,"ds/ds has wrong value at (1,2,2,1).")
      self.assertEqual(dsds[1,2,2,2],0.0,"ds/ds has wrong value at (1,2,2,2).")
      self.assertEqual(dsds[1,2,2,3],0.0,"ds/ds has wrong value at (1,2,2,3).")
      self.assertEqual(dsds[1,2,2,4],0.0,"ds/ds has wrong value at (1,2,2,4).")
      self.assertEqual(dsds[1,2,3,0],0.0,"ds/ds has wrong value at (1,2,3,0).")
      self.assertEqual(dsds[1,2,3,1],0.0,"ds/ds has wrong value at (1,2,3,1).")
      self.assertEqual(dsds[1,2,3,2],0.0,"ds/ds has wrong value at (1,2,3,2).")
      self.assertEqual(dsds[1,2,3,3],0.0,"ds/ds has wrong value at (1,2,3,3).")
      self.assertEqual(dsds[1,2,3,4],0.0,"ds/ds has wrong value at (1,2,3,4).")
      self.assertEqual(dsds[1,3,0,0],0.0,"ds/ds has wrong value at (1,3,0,0).")
      self.assertEqual(dsds[1,3,0,1],0.0,"ds/ds has wrong value at (1,3,0,1).")
      self.assertEqual(dsds[1,3,0,2],0.0,"ds/ds has wrong value at (1,3,0,2).")
      self.assertEqual(dsds[1,3,0,3],0.0,"ds/ds has wrong value at (1,3,0,3).")
      self.assertEqual(dsds[1,3,0,4],0.0,"ds/ds has wrong value at (1,3,0,4).")
      self.assertEqual(dsds[1,3,1,0],0.0,"ds/ds has wrong value at (1,3,1,0).")
      self.assertEqual(dsds[1,3,1,1],0.0,"ds/ds has wrong value at (1,3,1,1).")
      self.assertEqual(dsds[1,3,1,2],0.0,"ds/ds has wrong value at (1,3,1,2).")
      self.assertEqual(dsds[1,3,1,3],1.0,"ds/ds has wrong value at (1,3,1,3).")
      self.assertEqual(dsds[1,3,1,4],0.0,"ds/ds has wrong value at (1,3,1,4).")
      self.assertEqual(dsds[1,3,2,0],0.0,"ds/ds has wrong value at (1,3,2,0).")
      self.assertEqual(dsds[1,3,2,1],0.0,"ds/ds has wrong value at (1,3,2,1).")
      self.assertEqual(dsds[1,3,2,2],0.0,"ds/ds has wrong value at (1,3,2,2).")
      self.assertEqual(dsds[1,3,2,3],0.0,"ds/ds has wrong value at (1,3,2,3).")
      self.assertEqual(dsds[1,3,2,4],0.0,"ds/ds has wrong value at (1,3,2,4).")
      self.assertEqual(dsds[1,3,3,0],0.0,"ds/ds has wrong value at (1,3,3,0).")
      self.assertEqual(dsds[1,3,3,1],0.0,"ds/ds has wrong value at (1,3,3,1).")
      self.assertEqual(dsds[1,3,3,2],0.0,"ds/ds has wrong value at (1,3,3,2).")
      self.assertEqual(dsds[1,3,3,3],0.0,"ds/ds has wrong value at (1,3,3,3).")
      self.assertEqual(dsds[1,3,3,4],0.0,"ds/ds has wrong value at (1,3,3,4).")
      self.assertEqual(dsds[1,4,0,0],0.0,"ds/ds has wrong value at (1,4,0,0).")
      self.assertEqual(dsds[1,4,0,1],0.0,"ds/ds has wrong value at (1,4,0,1).")
      self.assertEqual(dsds[1,4,0,2],0.0,"ds/ds has wrong value at (1,4,0,2).")
      self.assertEqual(dsds[1,4,0,3],0.0,"ds/ds has wrong value at (1,4,0,3).")
      self.assertEqual(dsds[1,4,0,4],0.0,"ds/ds has wrong value at (1,4,0,4).")
      self.assertEqual(dsds[1,4,1,0],0.0,"ds/ds has wrong value at (1,4,1,0).")
      self.assertEqual(dsds[1,4,1,1],0.0,"ds/ds has wrong value at (1,4,1,1).")
      self.assertEqual(dsds[1,4,1,2],0.0,"ds/ds has wrong value at (1,4,1,2).")
      self.assertEqual(dsds[1,4,1,3],0.0,"ds/ds has wrong value at (1,4,1,3).")
      self.assertEqual(dsds[1,4,1,4],1.0,"ds/ds has wrong value at (1,4,1,4).")
      self.assertEqual(dsds[1,4,2,0],0.0,"ds/ds has wrong value at (1,4,2,0).")
      self.assertEqual(dsds[1,4,2,1],0.0,"ds/ds has wrong value at (1,4,2,1).")
      self.assertEqual(dsds[1,4,2,2],0.0,"ds/ds has wrong value at (1,4,2,2).")
      self.assertEqual(dsds[1,4,2,3],0.0,"ds/ds has wrong value at (1,4,2,3).")
      self.assertEqual(dsds[1,4,2,4],0.0,"ds/ds has wrong value at (1,4,2,4).")
      self.assertEqual(dsds[1,4,3,0],0.0,"ds/ds has wrong value at (1,4,3,0).")
      self.assertEqual(dsds[1,4,3,1],0.0,"ds/ds has wrong value at (1,4,3,1).")
      self.assertEqual(dsds[1,4,3,2],0.0,"ds/ds has wrong value at (1,4,3,2).")
      self.assertEqual(dsds[1,4,3,3],0.0,"ds/ds has wrong value at (1,4,3,3).")
      self.assertEqual(dsds[1,4,3,4],0.0,"ds/ds has wrong value at (1,4,3,4).")
      self.assertEqual(dsds[2,0,0,0],0.0,"ds/ds has wrong value at (2,0,0,0).")
      self.assertEqual(dsds[2,0,0,1],0.0,"ds/ds has wrong value at (2,0,0,1).")
      self.assertEqual(dsds[2,0,0,2],0.0,"ds/ds has wrong value at (2,0,0,2).")
      self.assertEqual(dsds[2,0,0,3],0.0,"ds/ds has wrong value at (2,0,0,3).")
      self.assertEqual(dsds[2,0,0,4],0.0,"ds/ds has wrong value at (2,0,0,4).")
      self.assertEqual(dsds[2,0,1,0],0.0,"ds/ds has wrong value at (2,0,1,0).")
      self.assertEqual(dsds[2,0,1,1],0.0,"ds/ds has wrong value at (2,0,1,1).")
      self.assertEqual(dsds[2,0,1,2],0.0,"ds/ds has wrong value at (2,0,1,2).")
      self.assertEqual(dsds[2,0,1,3],0.0,"ds/ds has wrong value at (2,0,1,3).")
      self.assertEqual(dsds[2,0,1,4],0.0,"ds/ds has wrong value at (2,0,1,4).")
      self.assertEqual(dsds[2,0,2,0],1.0,"ds/ds has wrong value at (2,0,2,0).")
      self.assertEqual(dsds[2,0,2,1],0.0,"ds/ds has wrong value at (2,0,2,1).")
      self.assertEqual(dsds[2,0,2,2],0.0,"ds/ds has wrong value at (2,0,2,2).")
      self.assertEqual(dsds[2,0,2,3],0.0,"ds/ds has wrong value at (2,0,2,3).")
      self.assertEqual(dsds[2,0,2,4],0.0,"ds/ds has wrong value at (2,0,2,4).")
      self.assertEqual(dsds[2,0,3,0],0.0,"ds/ds has wrong value at (2,0,3,0).")
      self.assertEqual(dsds[2,0,3,1],0.0,"ds/ds has wrong value at (2,0,3,1).")
      self.assertEqual(dsds[2,0,3,2],0.0,"ds/ds has wrong value at (2,0,3,2).")
      self.assertEqual(dsds[2,0,3,3],0.0,"ds/ds has wrong value at (2,0,3,3).")
      self.assertEqual(dsds[2,0,3,4],0.0,"ds/ds has wrong value at (2,0,3,4).")
      self.assertEqual(dsds[2,1,0,0],0.0,"ds/ds has wrong value at (2,1,0,0).")
      self.assertEqual(dsds[2,1,0,1],0.0,"ds/ds has wrong value at (2,1,0,1).")
      self.assertEqual(dsds[2,1,0,2],0.0,"ds/ds has wrong value at (2,1,0,2).")
      self.assertEqual(dsds[2,1,0,3],0.0,"ds/ds has wrong value at (2,1,0,3).")
      self.assertEqual(dsds[2,1,0,4],0.0,"ds/ds has wrong value at (2,1,0,4).")
      self.assertEqual(dsds[2,1,1,0],0.0,"ds/ds has wrong value at (2,1,1,0).")
      self.assertEqual(dsds[2,1,1,1],0.0,"ds/ds has wrong value at (2,1,1,1).")
      self.assertEqual(dsds[2,1,1,2],0.0,"ds/ds has wrong value at (2,1,1,2).")
      self.assertEqual(dsds[2,1,1,3],0.0,"ds/ds has wrong value at (2,1,1,3).")
      self.assertEqual(dsds[2,1,1,4],0.0,"ds/ds has wrong value at (2,1,1,4).")
      self.assertEqual(dsds[2,1,2,0],0.0,"ds/ds has wrong value at (2,1,2,0).")
      self.assertEqual(dsds[2,1,2,1],1.0,"ds/ds has wrong value at (2,1,2,1).")
      self.assertEqual(dsds[2,1,2,2],0.0,"ds/ds has wrong value at (2,1,2,2).")
      self.assertEqual(dsds[2,1,2,3],0.0,"ds/ds has wrong value at (2,1,2,3).")
      self.assertEqual(dsds[2,1,2,4],0.0,"ds/ds has wrong value at (2,1,2,4).")
      self.assertEqual(dsds[2,1,3,0],0.0,"ds/ds has wrong value at (2,1,3,0).")
      self.assertEqual(dsds[2,1,3,1],0.0,"ds/ds has wrong value at (2,1,3,1).")
      self.assertEqual(dsds[2,1,3,2],0.0,"ds/ds has wrong value at (2,1,3,2).")
      self.assertEqual(dsds[2,1,3,3],0.0,"ds/ds has wrong value at (2,1,3,3).")
      self.assertEqual(dsds[2,1,3,4],0.0,"ds/ds has wrong value at (2,1,3,4).")
      self.assertEqual(dsds[2,2,0,0],0.0,"ds/ds has wrong value at (2,2,0,0).")
      self.assertEqual(dsds[2,2,0,1],0.0,"ds/ds has wrong value at (2,2,0,1).")
      self.assertEqual(dsds[2,2,0,2],0.0,"ds/ds has wrong value at (2,2,0,2).")
      self.assertEqual(dsds[2,2,0,3],0.0,"ds/ds has wrong value at (2,2,0,3).")
      self.assertEqual(dsds[2,2,0,4],0.0,"ds/ds has wrong value at (2,2,0,4).")
      self.assertEqual(dsds[2,2,1,0],0.0,"ds/ds has wrong value at (2,2,1,0).")
      self.assertEqual(dsds[2,2,1,1],0.0,"ds/ds has wrong value at (2,2,1,1).")
      self.assertEqual(dsds[2,2,1,2],0.0,"ds/ds has wrong value at (2,2,1,2).")
      self.assertEqual(dsds[2,2,1,3],0.0,"ds/ds has wrong value at (2,2,1,3).")
      self.assertEqual(dsds[2,2,1,4],0.0,"ds/ds has wrong value at (2,2,1,4).")
      self.assertEqual(dsds[2,2,2,0],0.0,"ds/ds has wrong value at (2,2,2,0).")
      self.assertEqual(dsds[2,2,2,1],0.0,"ds/ds has wrong value at (2,2,2,1).")
      self.assertEqual(dsds[2,2,2,2],1.0,"ds/ds has wrong value at (2,2,2,2).")
      self.assertEqual(dsds[2,2,2,3],0.0,"ds/ds has wrong value at (2,2,2,3).")
      self.assertEqual(dsds[2,2,2,4],0.0,"ds/ds has wrong value at (2,2,2,4).")
      self.assertEqual(dsds[2,2,3,0],0.0,"ds/ds has wrong value at (2,2,3,0).")
      self.assertEqual(dsds[2,2,3,1],0.0,"ds/ds has wrong value at (2,2,3,1).")
      self.assertEqual(dsds[2,2,3,2],0.0,"ds/ds has wrong value at (2,2,3,2).")
      self.assertEqual(dsds[2,2,3,3],0.0,"ds/ds has wrong value at (2,2,3,3).")
      self.assertEqual(dsds[2,2,3,4],0.0,"ds/ds has wrong value at (2,2,3,4).")
      self.assertEqual(dsds[2,3,0,0],0.0,"ds/ds has wrong value at (2,3,0,0).")
      self.assertEqual(dsds[2,3,0,1],0.0,"ds/ds has wrong value at (2,3,0,1).")
      self.assertEqual(dsds[2,3,0,2],0.0,"ds/ds has wrong value at (2,3,0,2).")
      self.assertEqual(dsds[2,3,0,3],0.0,"ds/ds has wrong value at (2,3,0,3).")
      self.assertEqual(dsds[2,3,0,4],0.0,"ds/ds has wrong value at (2,3,0,4).")
      self.assertEqual(dsds[2,3,1,0],0.0,"ds/ds has wrong value at (2,3,1,0).")
      self.assertEqual(dsds[2,3,1,1],0.0,"ds/ds has wrong value at (2,3,1,1).")
      self.assertEqual(dsds[2,3,1,2],0.0,"ds/ds has wrong value at (2,3,1,2).")
      self.assertEqual(dsds[2,3,1,3],0.0,"ds/ds has wrong value at (2,3,1,3).")
      self.assertEqual(dsds[2,3,1,4],0.0,"ds/ds has wrong value at (2,3,1,4).")
      self.assertEqual(dsds[2,3,2,0],0.0,"ds/ds has wrong value at (2,3,2,0).")
      self.assertEqual(dsds[2,3,2,1],0.0,"ds/ds has wrong value at (2,3,2,1).")
      self.assertEqual(dsds[2,3,2,2],0.0,"ds/ds has wrong value at (2,3,2,2).")
      self.assertEqual(dsds[2,3,2,3],1.0,"ds/ds has wrong value at (2,3,2,3).")
      self.assertEqual(dsds[2,3,2,4],0.0,"ds/ds has wrong value at (2,3,2,4).")
      self.assertEqual(dsds[2,3,3,0],0.0,"ds/ds has wrong value at (2,3,3,0).")
      self.assertEqual(dsds[2,3,3,1],0.0,"ds/ds has wrong value at (2,3,3,1).")
      self.assertEqual(dsds[2,3,3,2],0.0,"ds/ds has wrong value at (2,3,3,2).")
      self.assertEqual(dsds[2,3,3,3],0.0,"ds/ds has wrong value at (2,3,3,3).")
      self.assertEqual(dsds[2,3,3,4],0.0,"ds/ds has wrong value at (2,3,3,4).")
      self.assertEqual(dsds[2,4,0,0],0.0,"ds/ds has wrong value at (2,4,0,0).")
      self.assertEqual(dsds[2,4,0,1],0.0,"ds/ds has wrong value at (2,4,0,1).")
      self.assertEqual(dsds[2,4,0,2],0.0,"ds/ds has wrong value at (2,4,0,2).")
      self.assertEqual(dsds[2,4,0,3],0.0,"ds/ds has wrong value at (2,4,0,3).")
      self.assertEqual(dsds[2,4,0,4],0.0,"ds/ds has wrong value at (2,4,0,4).")
      self.assertEqual(dsds[2,4,1,0],0.0,"ds/ds has wrong value at (2,4,1,0).")
      self.assertEqual(dsds[2,4,1,1],0.0,"ds/ds has wrong value at (2,4,1,1).")
      self.assertEqual(dsds[2,4,1,2],0.0,"ds/ds has wrong value at (2,4,1,2).")
      self.assertEqual(dsds[2,4,1,3],0.0,"ds/ds has wrong value at (2,4,1,3).")
      self.assertEqual(dsds[2,4,1,4],0.0,"ds/ds has wrong value at (2,4,1,4).")
      self.assertEqual(dsds[2,4,2,0],0.0,"ds/ds has wrong value at (2,4,2,0).")
      self.assertEqual(dsds[2,4,2,1],0.0,"ds/ds has wrong value at (2,4,2,1).")
      self.assertEqual(dsds[2,4,2,2],0.0,"ds/ds has wrong value at (2,4,2,2).")
      self.assertEqual(dsds[2,4,2,3],0.0,"ds/ds has wrong value at (2,4,2,3).")
      self.assertEqual(dsds[2,4,2,4],1.0,"ds/ds has wrong value at (2,4,2,4).")
      self.assertEqual(dsds[2,4,3,0],0.0,"ds/ds has wrong value at (2,4,3,0).")
      self.assertEqual(dsds[2,4,3,1],0.0,"ds/ds has wrong value at (2,4,3,1).")
      self.assertEqual(dsds[2,4,3,2],0.0,"ds/ds has wrong value at (2,4,3,2).")
      self.assertEqual(dsds[2,4,3,3],0.0,"ds/ds has wrong value at (2,4,3,3).")
      self.assertEqual(dsds[2,4,3,4],0.0,"ds/ds has wrong value at (2,4,3,4).")
      self.assertEqual(dsds[3,0,0,0],0.0,"ds/ds has wrong value at (3,0,0,0).")
      self.assertEqual(dsds[3,0,0,1],0.0,"ds/ds has wrong value at (3,0,0,1).")
      self.assertEqual(dsds[3,0,0,2],0.0,"ds/ds has wrong value at (3,0,0,2).")
      self.assertEqual(dsds[3,0,0,3],0.0,"ds/ds has wrong value at (3,0,0,3).")
      self.assertEqual(dsds[3,0,0,4],0.0,"ds/ds has wrong value at (3,0,0,4).")
      self.assertEqual(dsds[3,0,1,0],0.0,"ds/ds has wrong value at (3,0,1,0).")
      self.assertEqual(dsds[3,0,1,1],0.0,"ds/ds has wrong value at (3,0,1,1).")
      self.assertEqual(dsds[3,0,1,2],0.0,"ds/ds has wrong value at (3,0,1,2).")
      self.assertEqual(dsds[3,0,1,3],0.0,"ds/ds has wrong value at (3,0,1,3).")
      self.assertEqual(dsds[3,0,1,4],0.0,"ds/ds has wrong value at (3,0,1,4).")
      self.assertEqual(dsds[3,0,2,0],0.0,"ds/ds has wrong value at (3,0,2,0).")
      self.assertEqual(dsds[3,0,2,1],0.0,"ds/ds has wrong value at (3,0,2,1).")
      self.assertEqual(dsds[3,0,2,2],0.0,"ds/ds has wrong value at (3,0,2,2).")
      self.assertEqual(dsds[3,0,2,3],0.0,"ds/ds has wrong value at (3,0,2,3).")
      self.assertEqual(dsds[3,0,2,4],0.0,"ds/ds has wrong value at (3,0,2,4).")
      self.assertEqual(dsds[3,0,3,0],1.0,"ds/ds has wrong value at (3,0,3,0).")
      self.assertEqual(dsds[3,0,3,1],0.0,"ds/ds has wrong value at (3,0,3,1).")
      self.assertEqual(dsds[3,0,3,2],0.0,"ds/ds has wrong value at (3,0,3,2).")
      self.assertEqual(dsds[3,0,3,3],0.0,"ds/ds has wrong value at (3,0,3,3).")
      self.assertEqual(dsds[3,0,3,4],0.0,"ds/ds has wrong value at (3,0,3,4).")
      self.assertEqual(dsds[3,1,0,0],0.0,"ds/ds has wrong value at (3,1,0,0).")
      self.assertEqual(dsds[3,1,0,1],0.0,"ds/ds has wrong value at (3,1,0,1).")
      self.assertEqual(dsds[3,1,0,2],0.0,"ds/ds has wrong value at (3,1,0,2).")
      self.assertEqual(dsds[3,1,0,3],0.0,"ds/ds has wrong value at (3,1,0,3).")
      self.assertEqual(dsds[3,1,0,4],0.0,"ds/ds has wrong value at (3,1,0,4).")
      self.assertEqual(dsds[3,1,1,0],0.0,"ds/ds has wrong value at (3,1,1,0).")
      self.assertEqual(dsds[3,1,1,1],0.0,"ds/ds has wrong value at (3,1,1,1).")
      self.assertEqual(dsds[3,1,1,2],0.0,"ds/ds has wrong value at (3,1,1,2).")
      self.assertEqual(dsds[3,1,1,3],0.0,"ds/ds has wrong value at (3,1,1,3).")
      self.assertEqual(dsds[3,1,1,4],0.0,"ds/ds has wrong value at (3,1,1,4).")
      self.assertEqual(dsds[3,1,2,0],0.0,"ds/ds has wrong value at (3,1,2,0).")
      self.assertEqual(dsds[3,1,2,1],0.0,"ds/ds has wrong value at (3,1,2,1).")
      self.assertEqual(dsds[3,1,2,2],0.0,"ds/ds has wrong value at (3,1,2,2).")
      self.assertEqual(dsds[3,1,2,3],0.0,"ds/ds has wrong value at (3,1,2,3).")
      self.assertEqual(dsds[3,1,2,4],0.0,"ds/ds has wrong value at (3,1,2,4).")
      self.assertEqual(dsds[3,1,3,0],0.0,"ds/ds has wrong value at (3,1,3,0).")
      self.assertEqual(dsds[3,1,3,1],1.0,"ds/ds has wrong value at (3,1,3,1).")
      self.assertEqual(dsds[3,1,3,2],0.0,"ds/ds has wrong value at (3,1,3,2).")
      self.assertEqual(dsds[3,1,3,3],0.0,"ds/ds has wrong value at (3,1,3,3).")
      self.assertEqual(dsds[3,1,3,4],0.0,"ds/ds has wrong value at (3,1,3,4).")
      self.assertEqual(dsds[3,2,0,0],0.0,"ds/ds has wrong value at (3,2,0,0).")
      self.assertEqual(dsds[3,2,0,1],0.0,"ds/ds has wrong value at (3,2,0,1).")
      self.assertEqual(dsds[3,2,0,2],0.0,"ds/ds has wrong value at (3,2,0,2).")
      self.assertEqual(dsds[3,2,0,3],0.0,"ds/ds has wrong value at (3,2,0,3).")
      self.assertEqual(dsds[3,2,0,4],0.0,"ds/ds has wrong value at (3,2,0,4).")
      self.assertEqual(dsds[3,2,1,0],0.0,"ds/ds has wrong value at (3,2,1,0).")
      self.assertEqual(dsds[3,2,1,1],0.0,"ds/ds has wrong value at (3,2,1,1).")
      self.assertEqual(dsds[3,2,1,2],0.0,"ds/ds has wrong value at (3,2,1,2).")
      self.assertEqual(dsds[3,2,1,3],0.0,"ds/ds has wrong value at (3,2,1,3).")
      self.assertEqual(dsds[3,2,1,4],0.0,"ds/ds has wrong value at (3,2,1,4).")
      self.assertEqual(dsds[3,2,2,0],0.0,"ds/ds has wrong value at (3,2,2,0).")
      self.assertEqual(dsds[3,2,2,1],0.0,"ds/ds has wrong value at (3,2,2,1).")
      self.assertEqual(dsds[3,2,2,2],0.0,"ds/ds has wrong value at (3,2,2,2).")
      self.assertEqual(dsds[3,2,2,3],0.0,"ds/ds has wrong value at (3,2,2,3).")
      self.assertEqual(dsds[3,2,2,4],0.0,"ds/ds has wrong value at (3,2,2,4).")
      self.assertEqual(dsds[3,2,3,0],0.0,"ds/ds has wrong value at (3,2,3,0).")
      self.assertEqual(dsds[3,2,3,1],0.0,"ds/ds has wrong value at (3,2,3,1).")
      self.assertEqual(dsds[3,2,3,2],1.0,"ds/ds has wrong value at (3,2,3,2).")
      self.assertEqual(dsds[3,2,3,3],0.0,"ds/ds has wrong value at (3,2,3,3).")
      self.assertEqual(dsds[3,2,3,4],0.0,"ds/ds has wrong value at (3,2,3,4).")
      self.assertEqual(dsds[3,3,0,0],0.0,"ds/ds has wrong value at (3,3,0,0).")
      self.assertEqual(dsds[3,3,0,1],0.0,"ds/ds has wrong value at (3,3,0,1).")
      self.assertEqual(dsds[3,3,0,2],0.0,"ds/ds has wrong value at (3,3,0,2).")
      self.assertEqual(dsds[3,3,0,3],0.0,"ds/ds has wrong value at (3,3,0,3).")
      self.assertEqual(dsds[3,3,0,4],0.0,"ds/ds has wrong value at (3,3,0,4).")
      self.assertEqual(dsds[3,3,1,0],0.0,"ds/ds has wrong value at (3,3,1,0).")
      self.assertEqual(dsds[3,3,1,1],0.0,"ds/ds has wrong value at (3,3,1,1).")
      self.assertEqual(dsds[3,3,1,2],0.0,"ds/ds has wrong value at (3,3,1,2).")
      self.assertEqual(dsds[3,3,1,3],0.0,"ds/ds has wrong value at (3,3,1,3).")
      self.assertEqual(dsds[3,3,1,4],0.0,"ds/ds has wrong value at (3,3,1,4).")
      self.assertEqual(dsds[3,3,2,0],0.0,"ds/ds has wrong value at (3,3,2,0).")
      self.assertEqual(dsds[3,3,2,1],0.0,"ds/ds has wrong value at (3,3,2,1).")
      self.assertEqual(dsds[3,3,2,2],0.0,"ds/ds has wrong value at (3,3,2,2).")
      self.assertEqual(dsds[3,3,2,3],0.0,"ds/ds has wrong value at (3,3,2,3).")
      self.assertEqual(dsds[3,3,2,4],0.0,"ds/ds has wrong value at (3,3,2,4).")
      self.assertEqual(dsds[3,3,3,0],0.0,"ds/ds has wrong value at (3,3,3,0).")
      self.assertEqual(dsds[3,3,3,1],0.0,"ds/ds has wrong value at (3,3,3,1).")
      self.assertEqual(dsds[3,3,3,2],0.0,"ds/ds has wrong value at (3,3,3,2).")
      self.assertEqual(dsds[3,3,3,3],1.0,"ds/ds has wrong value at (3,3,3,3).")
      self.assertEqual(dsds[3,3,3,4],0.0,"ds/ds has wrong value at (3,3,3,4).")
      self.assertEqual(dsds[3,4,0,0],0.0,"ds/ds has wrong value at (3,4,0,0).")
      self.assertEqual(dsds[3,4,0,1],0.0,"ds/ds has wrong value at (3,4,0,1).")
      self.assertEqual(dsds[3,4,0,2],0.0,"ds/ds has wrong value at (3,4,0,2).")
      self.assertEqual(dsds[3,4,0,3],0.0,"ds/ds has wrong value at (3,4,0,3).")
      self.assertEqual(dsds[3,4,0,4],0.0,"ds/ds has wrong value at (3,4,0,4).")
      self.assertEqual(dsds[3,4,1,0],0.0,"ds/ds has wrong value at (3,4,1,0).")
      self.assertEqual(dsds[3,4,1,1],0.0,"ds/ds has wrong value at (3,4,1,1).")
      self.assertEqual(dsds[3,4,1,2],0.0,"ds/ds has wrong value at (3,4,1,2).")
      self.assertEqual(dsds[3,4,1,3],0.0,"ds/ds has wrong value at (3,4,1,3).")
      self.assertEqual(dsds[3,4,1,4],0.0,"ds/ds has wrong value at (3,4,1,4).")
      self.assertEqual(dsds[3,4,2,0],0.0,"ds/ds has wrong value at (3,4,2,0).")
      self.assertEqual(dsds[3,4,2,1],0.0,"ds/ds has wrong value at (3,4,2,1).")
      self.assertEqual(dsds[3,4,2,2],0.0,"ds/ds has wrong value at (3,4,2,2).")
      self.assertEqual(dsds[3,4,2,3],0.0,"ds/ds has wrong value at (3,4,2,3).")
      self.assertEqual(dsds[3,4,2,4],0.0,"ds/ds has wrong value at (3,4,2,4).")
      self.assertEqual(dsds[3,4,3,0],0.0,"ds/ds has wrong value at (3,4,3,0).")
      self.assertEqual(dsds[3,4,3,1],0.0,"ds/ds has wrong value at (3,4,3,1).")
      self.assertEqual(dsds[3,4,3,2],0.0,"ds/ds has wrong value at (3,4,3,2).")
      self.assertEqual(dsds[3,4,3,3],0.0,"ds/ds has wrong value at (3,4,3,3).")
      self.assertEqual(dsds[3,4,3,4],1.0,"ds/ds has wrong value at (3,4,3,4).")

      sa=s.getSubstitutedArguments({s2:-10})
      self.assertEqual(len(sa),0,"wrong number of substituted arguments")

      self.assertFalse(s.isAppropriateValue(10.)," 10. is not appropriate substitute")
      self.assertFalse(s.isAppropriateValue(10)," 10 is not appropriate substitute")
      self.assertTrue(s.isAppropriateValue(Symbol(shape=(4, 5),dim=1)),"Symbol is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(Symbol(shape=(4, 5),dim=2)),"Symbol is not appropriate substitute (dim)")
      self.assertFalse(s.isAppropriateValue(escript.Data(0.,(5,),self.functionspace)),"Data is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(numpy.zeros((4, 5))),"ndarray is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(numpy.zeros((5,))),"ndarray is not appropriate substitute (shape)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_Symbol_rank2_d1_nargs1(self):
      s2=Symbol(shape=())
      s=Symbol(shape=(4, 5),dim=1,args=[s2])
      self.assertEqual(s.getRank(),2,"wrong rank.")
      self.assertEqual(s.getShape(),(4, 5),"wrong shape.")
      self.assertEqual(s.getDim(),1,"wrong spatial dimension.")
      self.assertEqual(s.getArgument(),[s2],"wrong arguments.")

      ss=s.substitute({s:numpy.zeros((4, 5))})
      self.assertTrue(isinstance(ss,numpy.ndarray),"value after substitution is not ndarray.")
      self.assertEqual(ss.shape,(4, 5),"value after substitution has not expected shape")
      try:
        s.substitute({s:numpy.zeros((5,))})
        fail("illegal substition was successful")
      except TypeError:
        pass

      dsdarg=s.diff(Symbol(shape=()))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds() has wrong type.")
      self.assertEqual(dsdarg.shape,(4, 5),"ds/ds() has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds() has wrong value.")
      dsdarg=s.diff(Symbol(shape=(2,)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(2,) has wrong type.")
      self.assertEqual(dsdarg.shape,(4, 5, 2),"ds/ds(2,) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(2,) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(4, 5)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(4, 5) has wrong type.")
      self.assertEqual(dsdarg.shape,(4, 5, 4, 5),"ds/ds(4, 5) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(4, 5) has wrong value.")

      dsds=s.diff(s)
      self.assertTrue(isinstance(dsds,numpy.ndarray),"ds/ds has wrong type.")
      self.assertEqual(dsds.shape,(4, 5, 4, 5),"ds/ds has wrong shape.")
      self.assertEqual(dsds[0,0,0,0],1.0,"ds/ds has wrong value at (0,0,0,0).")
      self.assertEqual(dsds[0,0,0,1],0.0,"ds/ds has wrong value at (0,0,0,1).")
      self.assertEqual(dsds[0,0,0,2],0.0,"ds/ds has wrong value at (0,0,0,2).")
      self.assertEqual(dsds[0,0,0,3],0.0,"ds/ds has wrong value at (0,0,0,3).")
      self.assertEqual(dsds[0,0,0,4],0.0,"ds/ds has wrong value at (0,0,0,4).")
      self.assertEqual(dsds[0,0,1,0],0.0,"ds/ds has wrong value at (0,0,1,0).")
      self.assertEqual(dsds[0,0,1,1],0.0,"ds/ds has wrong value at (0,0,1,1).")
      self.assertEqual(dsds[0,0,1,2],0.0,"ds/ds has wrong value at (0,0,1,2).")
      self.assertEqual(dsds[0,0,1,3],0.0,"ds/ds has wrong value at (0,0,1,3).")
      self.assertEqual(dsds[0,0,1,4],0.0,"ds/ds has wrong value at (0,0,1,4).")
      self.assertEqual(dsds[0,0,2,0],0.0,"ds/ds has wrong value at (0,0,2,0).")
      self.assertEqual(dsds[0,0,2,1],0.0,"ds/ds has wrong value at (0,0,2,1).")
      self.assertEqual(dsds[0,0,2,2],0.0,"ds/ds has wrong value at (0,0,2,2).")
      self.assertEqual(dsds[0,0,2,3],0.0,"ds/ds has wrong value at (0,0,2,3).")
      self.assertEqual(dsds[0,0,2,4],0.0,"ds/ds has wrong value at (0,0,2,4).")
      self.assertEqual(dsds[0,0,3,0],0.0,"ds/ds has wrong value at (0,0,3,0).")
      self.assertEqual(dsds[0,0,3,1],0.0,"ds/ds has wrong value at (0,0,3,1).")
      self.assertEqual(dsds[0,0,3,2],0.0,"ds/ds has wrong value at (0,0,3,2).")
      self.assertEqual(dsds[0,0,3,3],0.0,"ds/ds has wrong value at (0,0,3,3).")
      self.assertEqual(dsds[0,0,3,4],0.0,"ds/ds has wrong value at (0,0,3,4).")
      self.assertEqual(dsds[0,1,0,0],0.0,"ds/ds has wrong value at (0,1,0,0).")
      self.assertEqual(dsds[0,1,0,1],1.0,"ds/ds has wrong value at (0,1,0,1).")
      self.assertEqual(dsds[0,1,0,2],0.0,"ds/ds has wrong value at (0,1,0,2).")
      self.assertEqual(dsds[0,1,0,3],0.0,"ds/ds has wrong value at (0,1,0,3).")
      self.assertEqual(dsds[0,1,0,4],0.0,"ds/ds has wrong value at (0,1,0,4).")
      self.assertEqual(dsds[0,1,1,0],0.0,"ds/ds has wrong value at (0,1,1,0).")
      self.assertEqual(dsds[0,1,1,1],0.0,"ds/ds has wrong value at (0,1,1,1).")
      self.assertEqual(dsds[0,1,1,2],0.0,"ds/ds has wrong value at (0,1,1,2).")
      self.assertEqual(dsds[0,1,1,3],0.0,"ds/ds has wrong value at (0,1,1,3).")
      self.assertEqual(dsds[0,1,1,4],0.0,"ds/ds has wrong value at (0,1,1,4).")
      self.assertEqual(dsds[0,1,2,0],0.0,"ds/ds has wrong value at (0,1,2,0).")
      self.assertEqual(dsds[0,1,2,1],0.0,"ds/ds has wrong value at (0,1,2,1).")
      self.assertEqual(dsds[0,1,2,2],0.0,"ds/ds has wrong value at (0,1,2,2).")
      self.assertEqual(dsds[0,1,2,3],0.0,"ds/ds has wrong value at (0,1,2,3).")
      self.assertEqual(dsds[0,1,2,4],0.0,"ds/ds has wrong value at (0,1,2,4).")
      self.assertEqual(dsds[0,1,3,0],0.0,"ds/ds has wrong value at (0,1,3,0).")
      self.assertEqual(dsds[0,1,3,1],0.0,"ds/ds has wrong value at (0,1,3,1).")
      self.assertEqual(dsds[0,1,3,2],0.0,"ds/ds has wrong value at (0,1,3,2).")
      self.assertEqual(dsds[0,1,3,3],0.0,"ds/ds has wrong value at (0,1,3,3).")
      self.assertEqual(dsds[0,1,3,4],0.0,"ds/ds has wrong value at (0,1,3,4).")
      self.assertEqual(dsds[0,2,0,0],0.0,"ds/ds has wrong value at (0,2,0,0).")
      self.assertEqual(dsds[0,2,0,1],0.0,"ds/ds has wrong value at (0,2,0,1).")
      self.assertEqual(dsds[0,2,0,2],1.0,"ds/ds has wrong value at (0,2,0,2).")
      self.assertEqual(dsds[0,2,0,3],0.0,"ds/ds has wrong value at (0,2,0,3).")
      self.assertEqual(dsds[0,2,0,4],0.0,"ds/ds has wrong value at (0,2,0,4).")
      self.assertEqual(dsds[0,2,1,0],0.0,"ds/ds has wrong value at (0,2,1,0).")
      self.assertEqual(dsds[0,2,1,1],0.0,"ds/ds has wrong value at (0,2,1,1).")
      self.assertEqual(dsds[0,2,1,2],0.0,"ds/ds has wrong value at (0,2,1,2).")
      self.assertEqual(dsds[0,2,1,3],0.0,"ds/ds has wrong value at (0,2,1,3).")
      self.assertEqual(dsds[0,2,1,4],0.0,"ds/ds has wrong value at (0,2,1,4).")
      self.assertEqual(dsds[0,2,2,0],0.0,"ds/ds has wrong value at (0,2,2,0).")
      self.assertEqual(dsds[0,2,2,1],0.0,"ds/ds has wrong value at (0,2,2,1).")
      self.assertEqual(dsds[0,2,2,2],0.0,"ds/ds has wrong value at (0,2,2,2).")
      self.assertEqual(dsds[0,2,2,3],0.0,"ds/ds has wrong value at (0,2,2,3).")
      self.assertEqual(dsds[0,2,2,4],0.0,"ds/ds has wrong value at (0,2,2,4).")
      self.assertEqual(dsds[0,2,3,0],0.0,"ds/ds has wrong value at (0,2,3,0).")
      self.assertEqual(dsds[0,2,3,1],0.0,"ds/ds has wrong value at (0,2,3,1).")
      self.assertEqual(dsds[0,2,3,2],0.0,"ds/ds has wrong value at (0,2,3,2).")
      self.assertEqual(dsds[0,2,3,3],0.0,"ds/ds has wrong value at (0,2,3,3).")
      self.assertEqual(dsds[0,2,3,4],0.0,"ds/ds has wrong value at (0,2,3,4).")
      self.assertEqual(dsds[0,3,0,0],0.0,"ds/ds has wrong value at (0,3,0,0).")
      self.assertEqual(dsds[0,3,0,1],0.0,"ds/ds has wrong value at (0,3,0,1).")
      self.assertEqual(dsds[0,3,0,2],0.0,"ds/ds has wrong value at (0,3,0,2).")
      self.assertEqual(dsds[0,3,0,3],1.0,"ds/ds has wrong value at (0,3,0,3).")
      self.assertEqual(dsds[0,3,0,4],0.0,"ds/ds has wrong value at (0,3,0,4).")
      self.assertEqual(dsds[0,3,1,0],0.0,"ds/ds has wrong value at (0,3,1,0).")
      self.assertEqual(dsds[0,3,1,1],0.0,"ds/ds has wrong value at (0,3,1,1).")
      self.assertEqual(dsds[0,3,1,2],0.0,"ds/ds has wrong value at (0,3,1,2).")
      self.assertEqual(dsds[0,3,1,3],0.0,"ds/ds has wrong value at (0,3,1,3).")
      self.assertEqual(dsds[0,3,1,4],0.0,"ds/ds has wrong value at (0,3,1,4).")
      self.assertEqual(dsds[0,3,2,0],0.0,"ds/ds has wrong value at (0,3,2,0).")
      self.assertEqual(dsds[0,3,2,1],0.0,"ds/ds has wrong value at (0,3,2,1).")
      self.assertEqual(dsds[0,3,2,2],0.0,"ds/ds has wrong value at (0,3,2,2).")
      self.assertEqual(dsds[0,3,2,3],0.0,"ds/ds has wrong value at (0,3,2,3).")
      self.assertEqual(dsds[0,3,2,4],0.0,"ds/ds has wrong value at (0,3,2,4).")
      self.assertEqual(dsds[0,3,3,0],0.0,"ds/ds has wrong value at (0,3,3,0).")
      self.assertEqual(dsds[0,3,3,1],0.0,"ds/ds has wrong value at (0,3,3,1).")
      self.assertEqual(dsds[0,3,3,2],0.0,"ds/ds has wrong value at (0,3,3,2).")
      self.assertEqual(dsds[0,3,3,3],0.0,"ds/ds has wrong value at (0,3,3,3).")
      self.assertEqual(dsds[0,3,3,4],0.0,"ds/ds has wrong value at (0,3,3,4).")
      self.assertEqual(dsds[0,4,0,0],0.0,"ds/ds has wrong value at (0,4,0,0).")
      self.assertEqual(dsds[0,4,0,1],0.0,"ds/ds has wrong value at (0,4,0,1).")
      self.assertEqual(dsds[0,4,0,2],0.0,"ds/ds has wrong value at (0,4,0,2).")
      self.assertEqual(dsds[0,4,0,3],0.0,"ds/ds has wrong value at (0,4,0,3).")
      self.assertEqual(dsds[0,4,0,4],1.0,"ds/ds has wrong value at (0,4,0,4).")
      self.assertEqual(dsds[0,4,1,0],0.0,"ds/ds has wrong value at (0,4,1,0).")
      self.assertEqual(dsds[0,4,1,1],0.0,"ds/ds has wrong value at (0,4,1,1).")
      self.assertEqual(dsds[0,4,1,2],0.0,"ds/ds has wrong value at (0,4,1,2).")
      self.assertEqual(dsds[0,4,1,3],0.0,"ds/ds has wrong value at (0,4,1,3).")
      self.assertEqual(dsds[0,4,1,4],0.0,"ds/ds has wrong value at (0,4,1,4).")
      self.assertEqual(dsds[0,4,2,0],0.0,"ds/ds has wrong value at (0,4,2,0).")
      self.assertEqual(dsds[0,4,2,1],0.0,"ds/ds has wrong value at (0,4,2,1).")
      self.assertEqual(dsds[0,4,2,2],0.0,"ds/ds has wrong value at (0,4,2,2).")
      self.assertEqual(dsds[0,4,2,3],0.0,"ds/ds has wrong value at (0,4,2,3).")
      self.assertEqual(dsds[0,4,2,4],0.0,"ds/ds has wrong value at (0,4,2,4).")
      self.assertEqual(dsds[0,4,3,0],0.0,"ds/ds has wrong value at (0,4,3,0).")
      self.assertEqual(dsds[0,4,3,1],0.0,"ds/ds has wrong value at (0,4,3,1).")
      self.assertEqual(dsds[0,4,3,2],0.0,"ds/ds has wrong value at (0,4,3,2).")
      self.assertEqual(dsds[0,4,3,3],0.0,"ds/ds has wrong value at (0,4,3,3).")
      self.assertEqual(dsds[0,4,3,4],0.0,"ds/ds has wrong value at (0,4,3,4).")
      self.assertEqual(dsds[1,0,0,0],0.0,"ds/ds has wrong value at (1,0,0,0).")
      self.assertEqual(dsds[1,0,0,1],0.0,"ds/ds has wrong value at (1,0,0,1).")
      self.assertEqual(dsds[1,0,0,2],0.0,"ds/ds has wrong value at (1,0,0,2).")
      self.assertEqual(dsds[1,0,0,3],0.0,"ds/ds has wrong value at (1,0,0,3).")
      self.assertEqual(dsds[1,0,0,4],0.0,"ds/ds has wrong value at (1,0,0,4).")
      self.assertEqual(dsds[1,0,1,0],1.0,"ds/ds has wrong value at (1,0,1,0).")
      self.assertEqual(dsds[1,0,1,1],0.0,"ds/ds has wrong value at (1,0,1,1).")
      self.assertEqual(dsds[1,0,1,2],0.0,"ds/ds has wrong value at (1,0,1,2).")
      self.assertEqual(dsds[1,0,1,3],0.0,"ds/ds has wrong value at (1,0,1,3).")
      self.assertEqual(dsds[1,0,1,4],0.0,"ds/ds has wrong value at (1,0,1,4).")
      self.assertEqual(dsds[1,0,2,0],0.0,"ds/ds has wrong value at (1,0,2,0).")
      self.assertEqual(dsds[1,0,2,1],0.0,"ds/ds has wrong value at (1,0,2,1).")
      self.assertEqual(dsds[1,0,2,2],0.0,"ds/ds has wrong value at (1,0,2,2).")
      self.assertEqual(dsds[1,0,2,3],0.0,"ds/ds has wrong value at (1,0,2,3).")
      self.assertEqual(dsds[1,0,2,4],0.0,"ds/ds has wrong value at (1,0,2,4).")
      self.assertEqual(dsds[1,0,3,0],0.0,"ds/ds has wrong value at (1,0,3,0).")
      self.assertEqual(dsds[1,0,3,1],0.0,"ds/ds has wrong value at (1,0,3,1).")
      self.assertEqual(dsds[1,0,3,2],0.0,"ds/ds has wrong value at (1,0,3,2).")
      self.assertEqual(dsds[1,0,3,3],0.0,"ds/ds has wrong value at (1,0,3,3).")
      self.assertEqual(dsds[1,0,3,4],0.0,"ds/ds has wrong value at (1,0,3,4).")
      self.assertEqual(dsds[1,1,0,0],0.0,"ds/ds has wrong value at (1,1,0,0).")
      self.assertEqual(dsds[1,1,0,1],0.0,"ds/ds has wrong value at (1,1,0,1).")
      self.assertEqual(dsds[1,1,0,2],0.0,"ds/ds has wrong value at (1,1,0,2).")
      self.assertEqual(dsds[1,1,0,3],0.0,"ds/ds has wrong value at (1,1,0,3).")
      self.assertEqual(dsds[1,1,0,4],0.0,"ds/ds has wrong value at (1,1,0,4).")
      self.assertEqual(dsds[1,1,1,0],0.0,"ds/ds has wrong value at (1,1,1,0).")
      self.assertEqual(dsds[1,1,1,1],1.0,"ds/ds has wrong value at (1,1,1,1).")
      self.assertEqual(dsds[1,1,1,2],0.0,"ds/ds has wrong value at (1,1,1,2).")
      self.assertEqual(dsds[1,1,1,3],0.0,"ds/ds has wrong value at (1,1,1,3).")
      self.assertEqual(dsds[1,1,1,4],0.0,"ds/ds has wrong value at (1,1,1,4).")
      self.assertEqual(dsds[1,1,2,0],0.0,"ds/ds has wrong value at (1,1,2,0).")
      self.assertEqual(dsds[1,1,2,1],0.0,"ds/ds has wrong value at (1,1,2,1).")
      self.assertEqual(dsds[1,1,2,2],0.0,"ds/ds has wrong value at (1,1,2,2).")
      self.assertEqual(dsds[1,1,2,3],0.0,"ds/ds has wrong value at (1,1,2,3).")
      self.assertEqual(dsds[1,1,2,4],0.0,"ds/ds has wrong value at (1,1,2,4).")
      self.assertEqual(dsds[1,1,3,0],0.0,"ds/ds has wrong value at (1,1,3,0).")
      self.assertEqual(dsds[1,1,3,1],0.0,"ds/ds has wrong value at (1,1,3,1).")
      self.assertEqual(dsds[1,1,3,2],0.0,"ds/ds has wrong value at (1,1,3,2).")
      self.assertEqual(dsds[1,1,3,3],0.0,"ds/ds has wrong value at (1,1,3,3).")
      self.assertEqual(dsds[1,1,3,4],0.0,"ds/ds has wrong value at (1,1,3,4).")
      self.assertEqual(dsds[1,2,0,0],0.0,"ds/ds has wrong value at (1,2,0,0).")
      self.assertEqual(dsds[1,2,0,1],0.0,"ds/ds has wrong value at (1,2,0,1).")
      self.assertEqual(dsds[1,2,0,2],0.0,"ds/ds has wrong value at (1,2,0,2).")
      self.assertEqual(dsds[1,2,0,3],0.0,"ds/ds has wrong value at (1,2,0,3).")
      self.assertEqual(dsds[1,2,0,4],0.0,"ds/ds has wrong value at (1,2,0,4).")
      self.assertEqual(dsds[1,2,1,0],0.0,"ds/ds has wrong value at (1,2,1,0).")
      self.assertEqual(dsds[1,2,1,1],0.0,"ds/ds has wrong value at (1,2,1,1).")
      self.assertEqual(dsds[1,2,1,2],1.0,"ds/ds has wrong value at (1,2,1,2).")
      self.assertEqual(dsds[1,2,1,3],0.0,"ds/ds has wrong value at (1,2,1,3).")
      self.assertEqual(dsds[1,2,1,4],0.0,"ds/ds has wrong value at (1,2,1,4).")
      self.assertEqual(dsds[1,2,2,0],0.0,"ds/ds has wrong value at (1,2,2,0).")
      self.assertEqual(dsds[1,2,2,1],0.0,"ds/ds has wrong value at (1,2,2,1).")
      self.assertEqual(dsds[1,2,2,2],0.0,"ds/ds has wrong value at (1,2,2,2).")
      self.assertEqual(dsds[1,2,2,3],0.0,"ds/ds has wrong value at (1,2,2,3).")
      self.assertEqual(dsds[1,2,2,4],0.0,"ds/ds has wrong value at (1,2,2,4).")
      self.assertEqual(dsds[1,2,3,0],0.0,"ds/ds has wrong value at (1,2,3,0).")
      self.assertEqual(dsds[1,2,3,1],0.0,"ds/ds has wrong value at (1,2,3,1).")
      self.assertEqual(dsds[1,2,3,2],0.0,"ds/ds has wrong value at (1,2,3,2).")
      self.assertEqual(dsds[1,2,3,3],0.0,"ds/ds has wrong value at (1,2,3,3).")
      self.assertEqual(dsds[1,2,3,4],0.0,"ds/ds has wrong value at (1,2,3,4).")
      self.assertEqual(dsds[1,3,0,0],0.0,"ds/ds has wrong value at (1,3,0,0).")
      self.assertEqual(dsds[1,3,0,1],0.0,"ds/ds has wrong value at (1,3,0,1).")
      self.assertEqual(dsds[1,3,0,2],0.0,"ds/ds has wrong value at (1,3,0,2).")
      self.assertEqual(dsds[1,3,0,3],0.0,"ds/ds has wrong value at (1,3,0,3).")
      self.assertEqual(dsds[1,3,0,4],0.0,"ds/ds has wrong value at (1,3,0,4).")
      self.assertEqual(dsds[1,3,1,0],0.0,"ds/ds has wrong value at (1,3,1,0).")
      self.assertEqual(dsds[1,3,1,1],0.0,"ds/ds has wrong value at (1,3,1,1).")
      self.assertEqual(dsds[1,3,1,2],0.0,"ds/ds has wrong value at (1,3,1,2).")
      self.assertEqual(dsds[1,3,1,3],1.0,"ds/ds has wrong value at (1,3,1,3).")
      self.assertEqual(dsds[1,3,1,4],0.0,"ds/ds has wrong value at (1,3,1,4).")
      self.assertEqual(dsds[1,3,2,0],0.0,"ds/ds has wrong value at (1,3,2,0).")
      self.assertEqual(dsds[1,3,2,1],0.0,"ds/ds has wrong value at (1,3,2,1).")
      self.assertEqual(dsds[1,3,2,2],0.0,"ds/ds has wrong value at (1,3,2,2).")
      self.assertEqual(dsds[1,3,2,3],0.0,"ds/ds has wrong value at (1,3,2,3).")
      self.assertEqual(dsds[1,3,2,4],0.0,"ds/ds has wrong value at (1,3,2,4).")
      self.assertEqual(dsds[1,3,3,0],0.0,"ds/ds has wrong value at (1,3,3,0).")
      self.assertEqual(dsds[1,3,3,1],0.0,"ds/ds has wrong value at (1,3,3,1).")
      self.assertEqual(dsds[1,3,3,2],0.0,"ds/ds has wrong value at (1,3,3,2).")
      self.assertEqual(dsds[1,3,3,3],0.0,"ds/ds has wrong value at (1,3,3,3).")
      self.assertEqual(dsds[1,3,3,4],0.0,"ds/ds has wrong value at (1,3,3,4).")
      self.assertEqual(dsds[1,4,0,0],0.0,"ds/ds has wrong value at (1,4,0,0).")
      self.assertEqual(dsds[1,4,0,1],0.0,"ds/ds has wrong value at (1,4,0,1).")
      self.assertEqual(dsds[1,4,0,2],0.0,"ds/ds has wrong value at (1,4,0,2).")
      self.assertEqual(dsds[1,4,0,3],0.0,"ds/ds has wrong value at (1,4,0,3).")
      self.assertEqual(dsds[1,4,0,4],0.0,"ds/ds has wrong value at (1,4,0,4).")
      self.assertEqual(dsds[1,4,1,0],0.0,"ds/ds has wrong value at (1,4,1,0).")
      self.assertEqual(dsds[1,4,1,1],0.0,"ds/ds has wrong value at (1,4,1,1).")
      self.assertEqual(dsds[1,4,1,2],0.0,"ds/ds has wrong value at (1,4,1,2).")
      self.assertEqual(dsds[1,4,1,3],0.0,"ds/ds has wrong value at (1,4,1,3).")
      self.assertEqual(dsds[1,4,1,4],1.0,"ds/ds has wrong value at (1,4,1,4).")
      self.assertEqual(dsds[1,4,2,0],0.0,"ds/ds has wrong value at (1,4,2,0).")
      self.assertEqual(dsds[1,4,2,1],0.0,"ds/ds has wrong value at (1,4,2,1).")
      self.assertEqual(dsds[1,4,2,2],0.0,"ds/ds has wrong value at (1,4,2,2).")
      self.assertEqual(dsds[1,4,2,3],0.0,"ds/ds has wrong value at (1,4,2,3).")
      self.assertEqual(dsds[1,4,2,4],0.0,"ds/ds has wrong value at (1,4,2,4).")
      self.assertEqual(dsds[1,4,3,0],0.0,"ds/ds has wrong value at (1,4,3,0).")
      self.assertEqual(dsds[1,4,3,1],0.0,"ds/ds has wrong value at (1,4,3,1).")
      self.assertEqual(dsds[1,4,3,2],0.0,"ds/ds has wrong value at (1,4,3,2).")
      self.assertEqual(dsds[1,4,3,3],0.0,"ds/ds has wrong value at (1,4,3,3).")
      self.assertEqual(dsds[1,4,3,4],0.0,"ds/ds has wrong value at (1,4,3,4).")
      self.assertEqual(dsds[2,0,0,0],0.0,"ds/ds has wrong value at (2,0,0,0).")
      self.assertEqual(dsds[2,0,0,1],0.0,"ds/ds has wrong value at (2,0,0,1).")
      self.assertEqual(dsds[2,0,0,2],0.0,"ds/ds has wrong value at (2,0,0,2).")
      self.assertEqual(dsds[2,0,0,3],0.0,"ds/ds has wrong value at (2,0,0,3).")
      self.assertEqual(dsds[2,0,0,4],0.0,"ds/ds has wrong value at (2,0,0,4).")
      self.assertEqual(dsds[2,0,1,0],0.0,"ds/ds has wrong value at (2,0,1,0).")
      self.assertEqual(dsds[2,0,1,1],0.0,"ds/ds has wrong value at (2,0,1,1).")
      self.assertEqual(dsds[2,0,1,2],0.0,"ds/ds has wrong value at (2,0,1,2).")
      self.assertEqual(dsds[2,0,1,3],0.0,"ds/ds has wrong value at (2,0,1,3).")
      self.assertEqual(dsds[2,0,1,4],0.0,"ds/ds has wrong value at (2,0,1,4).")
      self.assertEqual(dsds[2,0,2,0],1.0,"ds/ds has wrong value at (2,0,2,0).")
      self.assertEqual(dsds[2,0,2,1],0.0,"ds/ds has wrong value at (2,0,2,1).")
      self.assertEqual(dsds[2,0,2,2],0.0,"ds/ds has wrong value at (2,0,2,2).")
      self.assertEqual(dsds[2,0,2,3],0.0,"ds/ds has wrong value at (2,0,2,3).")
      self.assertEqual(dsds[2,0,2,4],0.0,"ds/ds has wrong value at (2,0,2,4).")
      self.assertEqual(dsds[2,0,3,0],0.0,"ds/ds has wrong value at (2,0,3,0).")
      self.assertEqual(dsds[2,0,3,1],0.0,"ds/ds has wrong value at (2,0,3,1).")
      self.assertEqual(dsds[2,0,3,2],0.0,"ds/ds has wrong value at (2,0,3,2).")
      self.assertEqual(dsds[2,0,3,3],0.0,"ds/ds has wrong value at (2,0,3,3).")
      self.assertEqual(dsds[2,0,3,4],0.0,"ds/ds has wrong value at (2,0,3,4).")
      self.assertEqual(dsds[2,1,0,0],0.0,"ds/ds has wrong value at (2,1,0,0).")
      self.assertEqual(dsds[2,1,0,1],0.0,"ds/ds has wrong value at (2,1,0,1).")
      self.assertEqual(dsds[2,1,0,2],0.0,"ds/ds has wrong value at (2,1,0,2).")
      self.assertEqual(dsds[2,1,0,3],0.0,"ds/ds has wrong value at (2,1,0,3).")
      self.assertEqual(dsds[2,1,0,4],0.0,"ds/ds has wrong value at (2,1,0,4).")
      self.assertEqual(dsds[2,1,1,0],0.0,"ds/ds has wrong value at (2,1,1,0).")
      self.assertEqual(dsds[2,1,1,1],0.0,"ds/ds has wrong value at (2,1,1,1).")
      self.assertEqual(dsds[2,1,1,2],0.0,"ds/ds has wrong value at (2,1,1,2).")
      self.assertEqual(dsds[2,1,1,3],0.0,"ds/ds has wrong value at (2,1,1,3).")
      self.assertEqual(dsds[2,1,1,4],0.0,"ds/ds has wrong value at (2,1,1,4).")
      self.assertEqual(dsds[2,1,2,0],0.0,"ds/ds has wrong value at (2,1,2,0).")
      self.assertEqual(dsds[2,1,2,1],1.0,"ds/ds has wrong value at (2,1,2,1).")
      self.assertEqual(dsds[2,1,2,2],0.0,"ds/ds has wrong value at (2,1,2,2).")
      self.assertEqual(dsds[2,1,2,3],0.0,"ds/ds has wrong value at (2,1,2,3).")
      self.assertEqual(dsds[2,1,2,4],0.0,"ds/ds has wrong value at (2,1,2,4).")
      self.assertEqual(dsds[2,1,3,0],0.0,"ds/ds has wrong value at (2,1,3,0).")
      self.assertEqual(dsds[2,1,3,1],0.0,"ds/ds has wrong value at (2,1,3,1).")
      self.assertEqual(dsds[2,1,3,2],0.0,"ds/ds has wrong value at (2,1,3,2).")
      self.assertEqual(dsds[2,1,3,3],0.0,"ds/ds has wrong value at (2,1,3,3).")
      self.assertEqual(dsds[2,1,3,4],0.0,"ds/ds has wrong value at (2,1,3,4).")
      self.assertEqual(dsds[2,2,0,0],0.0,"ds/ds has wrong value at (2,2,0,0).")
      self.assertEqual(dsds[2,2,0,1],0.0,"ds/ds has wrong value at (2,2,0,1).")
      self.assertEqual(dsds[2,2,0,2],0.0,"ds/ds has wrong value at (2,2,0,2).")
      self.assertEqual(dsds[2,2,0,3],0.0,"ds/ds has wrong value at (2,2,0,3).")
      self.assertEqual(dsds[2,2,0,4],0.0,"ds/ds has wrong value at (2,2,0,4).")
      self.assertEqual(dsds[2,2,1,0],0.0,"ds/ds has wrong value at (2,2,1,0).")
      self.assertEqual(dsds[2,2,1,1],0.0,"ds/ds has wrong value at (2,2,1,1).")
      self.assertEqual(dsds[2,2,1,2],0.0,"ds/ds has wrong value at (2,2,1,2).")
      self.assertEqual(dsds[2,2,1,3],0.0,"ds/ds has wrong value at (2,2,1,3).")
      self.assertEqual(dsds[2,2,1,4],0.0,"ds/ds has wrong value at (2,2,1,4).")
      self.assertEqual(dsds[2,2,2,0],0.0,"ds/ds has wrong value at (2,2,2,0).")
      self.assertEqual(dsds[2,2,2,1],0.0,"ds/ds has wrong value at (2,2,2,1).")
      self.assertEqual(dsds[2,2,2,2],1.0,"ds/ds has wrong value at (2,2,2,2).")
      self.assertEqual(dsds[2,2,2,3],0.0,"ds/ds has wrong value at (2,2,2,3).")
      self.assertEqual(dsds[2,2,2,4],0.0,"ds/ds has wrong value at (2,2,2,4).")
      self.assertEqual(dsds[2,2,3,0],0.0,"ds/ds has wrong value at (2,2,3,0).")
      self.assertEqual(dsds[2,2,3,1],0.0,"ds/ds has wrong value at (2,2,3,1).")
      self.assertEqual(dsds[2,2,3,2],0.0,"ds/ds has wrong value at (2,2,3,2).")
      self.assertEqual(dsds[2,2,3,3],0.0,"ds/ds has wrong value at (2,2,3,3).")
      self.assertEqual(dsds[2,2,3,4],0.0,"ds/ds has wrong value at (2,2,3,4).")
      self.assertEqual(dsds[2,3,0,0],0.0,"ds/ds has wrong value at (2,3,0,0).")
      self.assertEqual(dsds[2,3,0,1],0.0,"ds/ds has wrong value at (2,3,0,1).")
      self.assertEqual(dsds[2,3,0,2],0.0,"ds/ds has wrong value at (2,3,0,2).")
      self.assertEqual(dsds[2,3,0,3],0.0,"ds/ds has wrong value at (2,3,0,3).")
      self.assertEqual(dsds[2,3,0,4],0.0,"ds/ds has wrong value at (2,3,0,4).")
      self.assertEqual(dsds[2,3,1,0],0.0,"ds/ds has wrong value at (2,3,1,0).")
      self.assertEqual(dsds[2,3,1,1],0.0,"ds/ds has wrong value at (2,3,1,1).")
      self.assertEqual(dsds[2,3,1,2],0.0,"ds/ds has wrong value at (2,3,1,2).")
      self.assertEqual(dsds[2,3,1,3],0.0,"ds/ds has wrong value at (2,3,1,3).")
      self.assertEqual(dsds[2,3,1,4],0.0,"ds/ds has wrong value at (2,3,1,4).")
      self.assertEqual(dsds[2,3,2,0],0.0,"ds/ds has wrong value at (2,3,2,0).")
      self.assertEqual(dsds[2,3,2,1],0.0,"ds/ds has wrong value at (2,3,2,1).")
      self.assertEqual(dsds[2,3,2,2],0.0,"ds/ds has wrong value at (2,3,2,2).")
      self.assertEqual(dsds[2,3,2,3],1.0,"ds/ds has wrong value at (2,3,2,3).")
      self.assertEqual(dsds[2,3,2,4],0.0,"ds/ds has wrong value at (2,3,2,4).")
      self.assertEqual(dsds[2,3,3,0],0.0,"ds/ds has wrong value at (2,3,3,0).")
      self.assertEqual(dsds[2,3,3,1],0.0,"ds/ds has wrong value at (2,3,3,1).")
      self.assertEqual(dsds[2,3,3,2],0.0,"ds/ds has wrong value at (2,3,3,2).")
      self.assertEqual(dsds[2,3,3,3],0.0,"ds/ds has wrong value at (2,3,3,3).")
      self.assertEqual(dsds[2,3,3,4],0.0,"ds/ds has wrong value at (2,3,3,4).")
      self.assertEqual(dsds[2,4,0,0],0.0,"ds/ds has wrong value at (2,4,0,0).")
      self.assertEqual(dsds[2,4,0,1],0.0,"ds/ds has wrong value at (2,4,0,1).")
      self.assertEqual(dsds[2,4,0,2],0.0,"ds/ds has wrong value at (2,4,0,2).")
      self.assertEqual(dsds[2,4,0,3],0.0,"ds/ds has wrong value at (2,4,0,3).")
      self.assertEqual(dsds[2,4,0,4],0.0,"ds/ds has wrong value at (2,4,0,4).")
      self.assertEqual(dsds[2,4,1,0],0.0,"ds/ds has wrong value at (2,4,1,0).")
      self.assertEqual(dsds[2,4,1,1],0.0,"ds/ds has wrong value at (2,4,1,1).")
      self.assertEqual(dsds[2,4,1,2],0.0,"ds/ds has wrong value at (2,4,1,2).")
      self.assertEqual(dsds[2,4,1,3],0.0,"ds/ds has wrong value at (2,4,1,3).")
      self.assertEqual(dsds[2,4,1,4],0.0,"ds/ds has wrong value at (2,4,1,4).")
      self.assertEqual(dsds[2,4,2,0],0.0,"ds/ds has wrong value at (2,4,2,0).")
      self.assertEqual(dsds[2,4,2,1],0.0,"ds/ds has wrong value at (2,4,2,1).")
      self.assertEqual(dsds[2,4,2,2],0.0,"ds/ds has wrong value at (2,4,2,2).")
      self.assertEqual(dsds[2,4,2,3],0.0,"ds/ds has wrong value at (2,4,2,3).")
      self.assertEqual(dsds[2,4,2,4],1.0,"ds/ds has wrong value at (2,4,2,4).")
      self.assertEqual(dsds[2,4,3,0],0.0,"ds/ds has wrong value at (2,4,3,0).")
      self.assertEqual(dsds[2,4,3,1],0.0,"ds/ds has wrong value at (2,4,3,1).")
      self.assertEqual(dsds[2,4,3,2],0.0,"ds/ds has wrong value at (2,4,3,2).")
      self.assertEqual(dsds[2,4,3,3],0.0,"ds/ds has wrong value at (2,4,3,3).")
      self.assertEqual(dsds[2,4,3,4],0.0,"ds/ds has wrong value at (2,4,3,4).")
      self.assertEqual(dsds[3,0,0,0],0.0,"ds/ds has wrong value at (3,0,0,0).")
      self.assertEqual(dsds[3,0,0,1],0.0,"ds/ds has wrong value at (3,0,0,1).")
      self.assertEqual(dsds[3,0,0,2],0.0,"ds/ds has wrong value at (3,0,0,2).")
      self.assertEqual(dsds[3,0,0,3],0.0,"ds/ds has wrong value at (3,0,0,3).")
      self.assertEqual(dsds[3,0,0,4],0.0,"ds/ds has wrong value at (3,0,0,4).")
      self.assertEqual(dsds[3,0,1,0],0.0,"ds/ds has wrong value at (3,0,1,0).")
      self.assertEqual(dsds[3,0,1,1],0.0,"ds/ds has wrong value at (3,0,1,1).")
      self.assertEqual(dsds[3,0,1,2],0.0,"ds/ds has wrong value at (3,0,1,2).")
      self.assertEqual(dsds[3,0,1,3],0.0,"ds/ds has wrong value at (3,0,1,3).")
      self.assertEqual(dsds[3,0,1,4],0.0,"ds/ds has wrong value at (3,0,1,4).")
      self.assertEqual(dsds[3,0,2,0],0.0,"ds/ds has wrong value at (3,0,2,0).")
      self.assertEqual(dsds[3,0,2,1],0.0,"ds/ds has wrong value at (3,0,2,1).")
      self.assertEqual(dsds[3,0,2,2],0.0,"ds/ds has wrong value at (3,0,2,2).")
      self.assertEqual(dsds[3,0,2,3],0.0,"ds/ds has wrong value at (3,0,2,3).")
      self.assertEqual(dsds[3,0,2,4],0.0,"ds/ds has wrong value at (3,0,2,4).")
      self.assertEqual(dsds[3,0,3,0],1.0,"ds/ds has wrong value at (3,0,3,0).")
      self.assertEqual(dsds[3,0,3,1],0.0,"ds/ds has wrong value at (3,0,3,1).")
      self.assertEqual(dsds[3,0,3,2],0.0,"ds/ds has wrong value at (3,0,3,2).")
      self.assertEqual(dsds[3,0,3,3],0.0,"ds/ds has wrong value at (3,0,3,3).")
      self.assertEqual(dsds[3,0,3,4],0.0,"ds/ds has wrong value at (3,0,3,4).")
      self.assertEqual(dsds[3,1,0,0],0.0,"ds/ds has wrong value at (3,1,0,0).")
      self.assertEqual(dsds[3,1,0,1],0.0,"ds/ds has wrong value at (3,1,0,1).")
      self.assertEqual(dsds[3,1,0,2],0.0,"ds/ds has wrong value at (3,1,0,2).")
      self.assertEqual(dsds[3,1,0,3],0.0,"ds/ds has wrong value at (3,1,0,3).")
      self.assertEqual(dsds[3,1,0,4],0.0,"ds/ds has wrong value at (3,1,0,4).")
      self.assertEqual(dsds[3,1,1,0],0.0,"ds/ds has wrong value at (3,1,1,0).")
      self.assertEqual(dsds[3,1,1,1],0.0,"ds/ds has wrong value at (3,1,1,1).")
      self.assertEqual(dsds[3,1,1,2],0.0,"ds/ds has wrong value at (3,1,1,2).")
      self.assertEqual(dsds[3,1,1,3],0.0,"ds/ds has wrong value at (3,1,1,3).")
      self.assertEqual(dsds[3,1,1,4],0.0,"ds/ds has wrong value at (3,1,1,4).")
      self.assertEqual(dsds[3,1,2,0],0.0,"ds/ds has wrong value at (3,1,2,0).")
      self.assertEqual(dsds[3,1,2,1],0.0,"ds/ds has wrong value at (3,1,2,1).")
      self.assertEqual(dsds[3,1,2,2],0.0,"ds/ds has wrong value at (3,1,2,2).")
      self.assertEqual(dsds[3,1,2,3],0.0,"ds/ds has wrong value at (3,1,2,3).")
      self.assertEqual(dsds[3,1,2,4],0.0,"ds/ds has wrong value at (3,1,2,4).")
      self.assertEqual(dsds[3,1,3,0],0.0,"ds/ds has wrong value at (3,1,3,0).")
      self.assertEqual(dsds[3,1,3,1],1.0,"ds/ds has wrong value at (3,1,3,1).")
      self.assertEqual(dsds[3,1,3,2],0.0,"ds/ds has wrong value at (3,1,3,2).")
      self.assertEqual(dsds[3,1,3,3],0.0,"ds/ds has wrong value at (3,1,3,3).")
      self.assertEqual(dsds[3,1,3,4],0.0,"ds/ds has wrong value at (3,1,3,4).")
      self.assertEqual(dsds[3,2,0,0],0.0,"ds/ds has wrong value at (3,2,0,0).")
      self.assertEqual(dsds[3,2,0,1],0.0,"ds/ds has wrong value at (3,2,0,1).")
      self.assertEqual(dsds[3,2,0,2],0.0,"ds/ds has wrong value at (3,2,0,2).")
      self.assertEqual(dsds[3,2,0,3],0.0,"ds/ds has wrong value at (3,2,0,3).")
      self.assertEqual(dsds[3,2,0,4],0.0,"ds/ds has wrong value at (3,2,0,4).")
      self.assertEqual(dsds[3,2,1,0],0.0,"ds/ds has wrong value at (3,2,1,0).")
      self.assertEqual(dsds[3,2,1,1],0.0,"ds/ds has wrong value at (3,2,1,1).")
      self.assertEqual(dsds[3,2,1,2],0.0,"ds/ds has wrong value at (3,2,1,2).")
      self.assertEqual(dsds[3,2,1,3],0.0,"ds/ds has wrong value at (3,2,1,3).")
      self.assertEqual(dsds[3,2,1,4],0.0,"ds/ds has wrong value at (3,2,1,4).")
      self.assertEqual(dsds[3,2,2,0],0.0,"ds/ds has wrong value at (3,2,2,0).")
      self.assertEqual(dsds[3,2,2,1],0.0,"ds/ds has wrong value at (3,2,2,1).")
      self.assertEqual(dsds[3,2,2,2],0.0,"ds/ds has wrong value at (3,2,2,2).")
      self.assertEqual(dsds[3,2,2,3],0.0,"ds/ds has wrong value at (3,2,2,3).")
      self.assertEqual(dsds[3,2,2,4],0.0,"ds/ds has wrong value at (3,2,2,4).")
      self.assertEqual(dsds[3,2,3,0],0.0,"ds/ds has wrong value at (3,2,3,0).")
      self.assertEqual(dsds[3,2,3,1],0.0,"ds/ds has wrong value at (3,2,3,1).")
      self.assertEqual(dsds[3,2,3,2],1.0,"ds/ds has wrong value at (3,2,3,2).")
      self.assertEqual(dsds[3,2,3,3],0.0,"ds/ds has wrong value at (3,2,3,3).")
      self.assertEqual(dsds[3,2,3,4],0.0,"ds/ds has wrong value at (3,2,3,4).")
      self.assertEqual(dsds[3,3,0,0],0.0,"ds/ds has wrong value at (3,3,0,0).")
      self.assertEqual(dsds[3,3,0,1],0.0,"ds/ds has wrong value at (3,3,0,1).")
      self.assertEqual(dsds[3,3,0,2],0.0,"ds/ds has wrong value at (3,3,0,2).")
      self.assertEqual(dsds[3,3,0,3],0.0,"ds/ds has wrong value at (3,3,0,3).")
      self.assertEqual(dsds[3,3,0,4],0.0,"ds/ds has wrong value at (3,3,0,4).")
      self.assertEqual(dsds[3,3,1,0],0.0,"ds/ds has wrong value at (3,3,1,0).")
      self.assertEqual(dsds[3,3,1,1],0.0,"ds/ds has wrong value at (3,3,1,1).")
      self.assertEqual(dsds[3,3,1,2],0.0,"ds/ds has wrong value at (3,3,1,2).")
      self.assertEqual(dsds[3,3,1,3],0.0,"ds/ds has wrong value at (3,3,1,3).")
      self.assertEqual(dsds[3,3,1,4],0.0,"ds/ds has wrong value at (3,3,1,4).")
      self.assertEqual(dsds[3,3,2,0],0.0,"ds/ds has wrong value at (3,3,2,0).")
      self.assertEqual(dsds[3,3,2,1],0.0,"ds/ds has wrong value at (3,3,2,1).")
      self.assertEqual(dsds[3,3,2,2],0.0,"ds/ds has wrong value at (3,3,2,2).")
      self.assertEqual(dsds[3,3,2,3],0.0,"ds/ds has wrong value at (3,3,2,3).")
      self.assertEqual(dsds[3,3,2,4],0.0,"ds/ds has wrong value at (3,3,2,4).")
      self.assertEqual(dsds[3,3,3,0],0.0,"ds/ds has wrong value at (3,3,3,0).")
      self.assertEqual(dsds[3,3,3,1],0.0,"ds/ds has wrong value at (3,3,3,1).")
      self.assertEqual(dsds[3,3,3,2],0.0,"ds/ds has wrong value at (3,3,3,2).")
      self.assertEqual(dsds[3,3,3,3],1.0,"ds/ds has wrong value at (3,3,3,3).")
      self.assertEqual(dsds[3,3,3,4],0.0,"ds/ds has wrong value at (3,3,3,4).")
      self.assertEqual(dsds[3,4,0,0],0.0,"ds/ds has wrong value at (3,4,0,0).")
      self.assertEqual(dsds[3,4,0,1],0.0,"ds/ds has wrong value at (3,4,0,1).")
      self.assertEqual(dsds[3,4,0,2],0.0,"ds/ds has wrong value at (3,4,0,2).")
      self.assertEqual(dsds[3,4,0,3],0.0,"ds/ds has wrong value at (3,4,0,3).")
      self.assertEqual(dsds[3,4,0,4],0.0,"ds/ds has wrong value at (3,4,0,4).")
      self.assertEqual(dsds[3,4,1,0],0.0,"ds/ds has wrong value at (3,4,1,0).")
      self.assertEqual(dsds[3,4,1,1],0.0,"ds/ds has wrong value at (3,4,1,1).")
      self.assertEqual(dsds[3,4,1,2],0.0,"ds/ds has wrong value at (3,4,1,2).")
      self.assertEqual(dsds[3,4,1,3],0.0,"ds/ds has wrong value at (3,4,1,3).")
      self.assertEqual(dsds[3,4,1,4],0.0,"ds/ds has wrong value at (3,4,1,4).")
      self.assertEqual(dsds[3,4,2,0],0.0,"ds/ds has wrong value at (3,4,2,0).")
      self.assertEqual(dsds[3,4,2,1],0.0,"ds/ds has wrong value at (3,4,2,1).")
      self.assertEqual(dsds[3,4,2,2],0.0,"ds/ds has wrong value at (3,4,2,2).")
      self.assertEqual(dsds[3,4,2,3],0.0,"ds/ds has wrong value at (3,4,2,3).")
      self.assertEqual(dsds[3,4,2,4],0.0,"ds/ds has wrong value at (3,4,2,4).")
      self.assertEqual(dsds[3,4,3,0],0.0,"ds/ds has wrong value at (3,4,3,0).")
      self.assertEqual(dsds[3,4,3,1],0.0,"ds/ds has wrong value at (3,4,3,1).")
      self.assertEqual(dsds[3,4,3,2],0.0,"ds/ds has wrong value at (3,4,3,2).")
      self.assertEqual(dsds[3,4,3,3],0.0,"ds/ds has wrong value at (3,4,3,3).")
      self.assertEqual(dsds[3,4,3,4],1.0,"ds/ds has wrong value at (3,4,3,4).")

      self.assertEqual(s.getArgument(0),s2,"wrong argument 0.")
      sa=s.getSubstitutedArguments({s2:-10})
      self.assertEqual(len(sa),1,"wrong number of substituted arguments")
      self.assertEqual(sa[0],-10,"wrongly substituted argument 0.")

      self.assertFalse(s.isAppropriateValue(10.)," 10. is not appropriate substitute")
      self.assertFalse(s.isAppropriateValue(10)," 10 is not appropriate substitute")
      self.assertTrue(s.isAppropriateValue(Symbol(shape=(4, 5),dim=1)),"Symbol is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(Symbol(shape=(4, 5),dim=2)),"Symbol is not appropriate substitute (dim)")
      self.assertFalse(s.isAppropriateValue(escript.Data(0.,(5,),self.functionspace)),"Data is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(numpy.zeros((4, 5))),"ndarray is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(numpy.zeros((5,))),"ndarray is not appropriate substitute (shape)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_Symbol_rank2_d1_nargs2(self):
      s2=Symbol(shape=())
      s=Symbol(shape=(4, 5),dim=1,args=[1, -1.0])
      self.assertEqual(s.getRank(),2,"wrong rank.")
      self.assertEqual(s.getShape(),(4, 5),"wrong shape.")
      self.assertEqual(s.getDim(),1,"wrong spatial dimension.")
      self.assertEqual(s.getArgument(),[1, -1.0],"wrong arguments.")

      ss=s.substitute({s:numpy.zeros((4, 5))})
      self.assertTrue(isinstance(ss,numpy.ndarray),"value after substitution is not ndarray.")
      self.assertEqual(ss.shape,(4, 5),"value after substitution has not expected shape")
      try:
        s.substitute({s:numpy.zeros((5,))})
        fail("illegal substition was successful")
      except TypeError:
        pass

      dsdarg=s.diff(Symbol(shape=()))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds() has wrong type.")
      self.assertEqual(dsdarg.shape,(4, 5),"ds/ds() has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds() has wrong value.")
      dsdarg=s.diff(Symbol(shape=(2,)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(2,) has wrong type.")
      self.assertEqual(dsdarg.shape,(4, 5, 2),"ds/ds(2,) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(2,) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(4, 5)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(4, 5) has wrong type.")
      self.assertEqual(dsdarg.shape,(4, 5, 4, 5),"ds/ds(4, 5) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(4, 5) has wrong value.")

      dsds=s.diff(s)
      self.assertTrue(isinstance(dsds,numpy.ndarray),"ds/ds has wrong type.")
      self.assertEqual(dsds.shape,(4, 5, 4, 5),"ds/ds has wrong shape.")
      self.assertEqual(dsds[0,0,0,0],1.0,"ds/ds has wrong value at (0,0,0,0).")
      self.assertEqual(dsds[0,0,0,1],0.0,"ds/ds has wrong value at (0,0,0,1).")
      self.assertEqual(dsds[0,0,0,2],0.0,"ds/ds has wrong value at (0,0,0,2).")
      self.assertEqual(dsds[0,0,0,3],0.0,"ds/ds has wrong value at (0,0,0,3).")
      self.assertEqual(dsds[0,0,0,4],0.0,"ds/ds has wrong value at (0,0,0,4).")
      self.assertEqual(dsds[0,0,1,0],0.0,"ds/ds has wrong value at (0,0,1,0).")
      self.assertEqual(dsds[0,0,1,1],0.0,"ds/ds has wrong value at (0,0,1,1).")
      self.assertEqual(dsds[0,0,1,2],0.0,"ds/ds has wrong value at (0,0,1,2).")
      self.assertEqual(dsds[0,0,1,3],0.0,"ds/ds has wrong value at (0,0,1,3).")
      self.assertEqual(dsds[0,0,1,4],0.0,"ds/ds has wrong value at (0,0,1,4).")
      self.assertEqual(dsds[0,0,2,0],0.0,"ds/ds has wrong value at (0,0,2,0).")
      self.assertEqual(dsds[0,0,2,1],0.0,"ds/ds has wrong value at (0,0,2,1).")
      self.assertEqual(dsds[0,0,2,2],0.0,"ds/ds has wrong value at (0,0,2,2).")
      self.assertEqual(dsds[0,0,2,3],0.0,"ds/ds has wrong value at (0,0,2,3).")
      self.assertEqual(dsds[0,0,2,4],0.0,"ds/ds has wrong value at (0,0,2,4).")
      self.assertEqual(dsds[0,0,3,0],0.0,"ds/ds has wrong value at (0,0,3,0).")
      self.assertEqual(dsds[0,0,3,1],0.0,"ds/ds has wrong value at (0,0,3,1).")
      self.assertEqual(dsds[0,0,3,2],0.0,"ds/ds has wrong value at (0,0,3,2).")
      self.assertEqual(dsds[0,0,3,3],0.0,"ds/ds has wrong value at (0,0,3,3).")
      self.assertEqual(dsds[0,0,3,4],0.0,"ds/ds has wrong value at (0,0,3,4).")
      self.assertEqual(dsds[0,1,0,0],0.0,"ds/ds has wrong value at (0,1,0,0).")
      self.assertEqual(dsds[0,1,0,1],1.0,"ds/ds has wrong value at (0,1,0,1).")
      self.assertEqual(dsds[0,1,0,2],0.0,"ds/ds has wrong value at (0,1,0,2).")
      self.assertEqual(dsds[0,1,0,3],0.0,"ds/ds has wrong value at (0,1,0,3).")
      self.assertEqual(dsds[0,1,0,4],0.0,"ds/ds has wrong value at (0,1,0,4).")
      self.assertEqual(dsds[0,1,1,0],0.0,"ds/ds has wrong value at (0,1,1,0).")
      self.assertEqual(dsds[0,1,1,1],0.0,"ds/ds has wrong value at (0,1,1,1).")
      self.assertEqual(dsds[0,1,1,2],0.0,"ds/ds has wrong value at (0,1,1,2).")
      self.assertEqual(dsds[0,1,1,3],0.0,"ds/ds has wrong value at (0,1,1,3).")
      self.assertEqual(dsds[0,1,1,4],0.0,"ds/ds has wrong value at (0,1,1,4).")
      self.assertEqual(dsds[0,1,2,0],0.0,"ds/ds has wrong value at (0,1,2,0).")
      self.assertEqual(dsds[0,1,2,1],0.0,"ds/ds has wrong value at (0,1,2,1).")
      self.assertEqual(dsds[0,1,2,2],0.0,"ds/ds has wrong value at (0,1,2,2).")
      self.assertEqual(dsds[0,1,2,3],0.0,"ds/ds has wrong value at (0,1,2,3).")
      self.assertEqual(dsds[0,1,2,4],0.0,"ds/ds has wrong value at (0,1,2,4).")
      self.assertEqual(dsds[0,1,3,0],0.0,"ds/ds has wrong value at (0,1,3,0).")
      self.assertEqual(dsds[0,1,3,1],0.0,"ds/ds has wrong value at (0,1,3,1).")
      self.assertEqual(dsds[0,1,3,2],0.0,"ds/ds has wrong value at (0,1,3,2).")
      self.assertEqual(dsds[0,1,3,3],0.0,"ds/ds has wrong value at (0,1,3,3).")
      self.assertEqual(dsds[0,1,3,4],0.0,"ds/ds has wrong value at (0,1,3,4).")
      self.assertEqual(dsds[0,2,0,0],0.0,"ds/ds has wrong value at (0,2,0,0).")
      self.assertEqual(dsds[0,2,0,1],0.0,"ds/ds has wrong value at (0,2,0,1).")
      self.assertEqual(dsds[0,2,0,2],1.0,"ds/ds has wrong value at (0,2,0,2).")
      self.assertEqual(dsds[0,2,0,3],0.0,"ds/ds has wrong value at (0,2,0,3).")
      self.assertEqual(dsds[0,2,0,4],0.0,"ds/ds has wrong value at (0,2,0,4).")
      self.assertEqual(dsds[0,2,1,0],0.0,"ds/ds has wrong value at (0,2,1,0).")
      self.assertEqual(dsds[0,2,1,1],0.0,"ds/ds has wrong value at (0,2,1,1).")
      self.assertEqual(dsds[0,2,1,2],0.0,"ds/ds has wrong value at (0,2,1,2).")
      self.assertEqual(dsds[0,2,1,3],0.0,"ds/ds has wrong value at (0,2,1,3).")
      self.assertEqual(dsds[0,2,1,4],0.0,"ds/ds has wrong value at (0,2,1,4).")
      self.assertEqual(dsds[0,2,2,0],0.0,"ds/ds has wrong value at (0,2,2,0).")
      self.assertEqual(dsds[0,2,2,1],0.0,"ds/ds has wrong value at (0,2,2,1).")
      self.assertEqual(dsds[0,2,2,2],0.0,"ds/ds has wrong value at (0,2,2,2).")
      self.assertEqual(dsds[0,2,2,3],0.0,"ds/ds has wrong value at (0,2,2,3).")
      self.assertEqual(dsds[0,2,2,4],0.0,"ds/ds has wrong value at (0,2,2,4).")
      self.assertEqual(dsds[0,2,3,0],0.0,"ds/ds has wrong value at (0,2,3,0).")
      self.assertEqual(dsds[0,2,3,1],0.0,"ds/ds has wrong value at (0,2,3,1).")
      self.assertEqual(dsds[0,2,3,2],0.0,"ds/ds has wrong value at (0,2,3,2).")
      self.assertEqual(dsds[0,2,3,3],0.0,"ds/ds has wrong value at (0,2,3,3).")
      self.assertEqual(dsds[0,2,3,4],0.0,"ds/ds has wrong value at (0,2,3,4).")
      self.assertEqual(dsds[0,3,0,0],0.0,"ds/ds has wrong value at (0,3,0,0).")
      self.assertEqual(dsds[0,3,0,1],0.0,"ds/ds has wrong value at (0,3,0,1).")
      self.assertEqual(dsds[0,3,0,2],0.0,"ds/ds has wrong value at (0,3,0,2).")
      self.assertEqual(dsds[0,3,0,3],1.0,"ds/ds has wrong value at (0,3,0,3).")
      self.assertEqual(dsds[0,3,0,4],0.0,"ds/ds has wrong value at (0,3,0,4).")
      self.assertEqual(dsds[0,3,1,0],0.0,"ds/ds has wrong value at (0,3,1,0).")
      self.assertEqual(dsds[0,3,1,1],0.0,"ds/ds has wrong value at (0,3,1,1).")
      self.assertEqual(dsds[0,3,1,2],0.0,"ds/ds has wrong value at (0,3,1,2).")
      self.assertEqual(dsds[0,3,1,3],0.0,"ds/ds has wrong value at (0,3,1,3).")
      self.assertEqual(dsds[0,3,1,4],0.0,"ds/ds has wrong value at (0,3,1,4).")
      self.assertEqual(dsds[0,3,2,0],0.0,"ds/ds has wrong value at (0,3,2,0).")
      self.assertEqual(dsds[0,3,2,1],0.0,"ds/ds has wrong value at (0,3,2,1).")
      self.assertEqual(dsds[0,3,2,2],0.0,"ds/ds has wrong value at (0,3,2,2).")
      self.assertEqual(dsds[0,3,2,3],0.0,"ds/ds has wrong value at (0,3,2,3).")
      self.assertEqual(dsds[0,3,2,4],0.0,"ds/ds has wrong value at (0,3,2,4).")
      self.assertEqual(dsds[0,3,3,0],0.0,"ds/ds has wrong value at (0,3,3,0).")
      self.assertEqual(dsds[0,3,3,1],0.0,"ds/ds has wrong value at (0,3,3,1).")
      self.assertEqual(dsds[0,3,3,2],0.0,"ds/ds has wrong value at (0,3,3,2).")
      self.assertEqual(dsds[0,3,3,3],0.0,"ds/ds has wrong value at (0,3,3,3).")
      self.assertEqual(dsds[0,3,3,4],0.0,"ds/ds has wrong value at (0,3,3,4).")
      self.assertEqual(dsds[0,4,0,0],0.0,"ds/ds has wrong value at (0,4,0,0).")
      self.assertEqual(dsds[0,4,0,1],0.0,"ds/ds has wrong value at (0,4,0,1).")
      self.assertEqual(dsds[0,4,0,2],0.0,"ds/ds has wrong value at (0,4,0,2).")
      self.assertEqual(dsds[0,4,0,3],0.0,"ds/ds has wrong value at (0,4,0,3).")
      self.assertEqual(dsds[0,4,0,4],1.0,"ds/ds has wrong value at (0,4,0,4).")
      self.assertEqual(dsds[0,4,1,0],0.0,"ds/ds has wrong value at (0,4,1,0).")
      self.assertEqual(dsds[0,4,1,1],0.0,"ds/ds has wrong value at (0,4,1,1).")
      self.assertEqual(dsds[0,4,1,2],0.0,"ds/ds has wrong value at (0,4,1,2).")
      self.assertEqual(dsds[0,4,1,3],0.0,"ds/ds has wrong value at (0,4,1,3).")
      self.assertEqual(dsds[0,4,1,4],0.0,"ds/ds has wrong value at (0,4,1,4).")
      self.assertEqual(dsds[0,4,2,0],0.0,"ds/ds has wrong value at (0,4,2,0).")
      self.assertEqual(dsds[0,4,2,1],0.0,"ds/ds has wrong value at (0,4,2,1).")
      self.assertEqual(dsds[0,4,2,2],0.0,"ds/ds has wrong value at (0,4,2,2).")
      self.assertEqual(dsds[0,4,2,3],0.0,"ds/ds has wrong value at (0,4,2,3).")
      self.assertEqual(dsds[0,4,2,4],0.0,"ds/ds has wrong value at (0,4,2,4).")
      self.assertEqual(dsds[0,4,3,0],0.0,"ds/ds has wrong value at (0,4,3,0).")
      self.assertEqual(dsds[0,4,3,1],0.0,"ds/ds has wrong value at (0,4,3,1).")
      self.assertEqual(dsds[0,4,3,2],0.0,"ds/ds has wrong value at (0,4,3,2).")
      self.assertEqual(dsds[0,4,3,3],0.0,"ds/ds has wrong value at (0,4,3,3).")
      self.assertEqual(dsds[0,4,3,4],0.0,"ds/ds has wrong value at (0,4,3,4).")
      self.assertEqual(dsds[1,0,0,0],0.0,"ds/ds has wrong value at (1,0,0,0).")
      self.assertEqual(dsds[1,0,0,1],0.0,"ds/ds has wrong value at (1,0,0,1).")
      self.assertEqual(dsds[1,0,0,2],0.0,"ds/ds has wrong value at (1,0,0,2).")
      self.assertEqual(dsds[1,0,0,3],0.0,"ds/ds has wrong value at (1,0,0,3).")
      self.assertEqual(dsds[1,0,0,4],0.0,"ds/ds has wrong value at (1,0,0,4).")
      self.assertEqual(dsds[1,0,1,0],1.0,"ds/ds has wrong value at (1,0,1,0).")
      self.assertEqual(dsds[1,0,1,1],0.0,"ds/ds has wrong value at (1,0,1,1).")
      self.assertEqual(dsds[1,0,1,2],0.0,"ds/ds has wrong value at (1,0,1,2).")
      self.assertEqual(dsds[1,0,1,3],0.0,"ds/ds has wrong value at (1,0,1,3).")
      self.assertEqual(dsds[1,0,1,4],0.0,"ds/ds has wrong value at (1,0,1,4).")
      self.assertEqual(dsds[1,0,2,0],0.0,"ds/ds has wrong value at (1,0,2,0).")
      self.assertEqual(dsds[1,0,2,1],0.0,"ds/ds has wrong value at (1,0,2,1).")
      self.assertEqual(dsds[1,0,2,2],0.0,"ds/ds has wrong value at (1,0,2,2).")
      self.assertEqual(dsds[1,0,2,3],0.0,"ds/ds has wrong value at (1,0,2,3).")
      self.assertEqual(dsds[1,0,2,4],0.0,"ds/ds has wrong value at (1,0,2,4).")
      self.assertEqual(dsds[1,0,3,0],0.0,"ds/ds has wrong value at (1,0,3,0).")
      self.assertEqual(dsds[1,0,3,1],0.0,"ds/ds has wrong value at (1,0,3,1).")
      self.assertEqual(dsds[1,0,3,2],0.0,"ds/ds has wrong value at (1,0,3,2).")
      self.assertEqual(dsds[1,0,3,3],0.0,"ds/ds has wrong value at (1,0,3,3).")
      self.assertEqual(dsds[1,0,3,4],0.0,"ds/ds has wrong value at (1,0,3,4).")
      self.assertEqual(dsds[1,1,0,0],0.0,"ds/ds has wrong value at (1,1,0,0).")
      self.assertEqual(dsds[1,1,0,1],0.0,"ds/ds has wrong value at (1,1,0,1).")
      self.assertEqual(dsds[1,1,0,2],0.0,"ds/ds has wrong value at (1,1,0,2).")
      self.assertEqual(dsds[1,1,0,3],0.0,"ds/ds has wrong value at (1,1,0,3).")
      self.assertEqual(dsds[1,1,0,4],0.0,"ds/ds has wrong value at (1,1,0,4).")
      self.assertEqual(dsds[1,1,1,0],0.0,"ds/ds has wrong value at (1,1,1,0).")
      self.assertEqual(dsds[1,1,1,1],1.0,"ds/ds has wrong value at (1,1,1,1).")
      self.assertEqual(dsds[1,1,1,2],0.0,"ds/ds has wrong value at (1,1,1,2).")
      self.assertEqual(dsds[1,1,1,3],0.0,"ds/ds has wrong value at (1,1,1,3).")
      self.assertEqual(dsds[1,1,1,4],0.0,"ds/ds has wrong value at (1,1,1,4).")
      self.assertEqual(dsds[1,1,2,0],0.0,"ds/ds has wrong value at (1,1,2,0).")
      self.assertEqual(dsds[1,1,2,1],0.0,"ds/ds has wrong value at (1,1,2,1).")
      self.assertEqual(dsds[1,1,2,2],0.0,"ds/ds has wrong value at (1,1,2,2).")
      self.assertEqual(dsds[1,1,2,3],0.0,"ds/ds has wrong value at (1,1,2,3).")
      self.assertEqual(dsds[1,1,2,4],0.0,"ds/ds has wrong value at (1,1,2,4).")
      self.assertEqual(dsds[1,1,3,0],0.0,"ds/ds has wrong value at (1,1,3,0).")
      self.assertEqual(dsds[1,1,3,1],0.0,"ds/ds has wrong value at (1,1,3,1).")
      self.assertEqual(dsds[1,1,3,2],0.0,"ds/ds has wrong value at (1,1,3,2).")
      self.assertEqual(dsds[1,1,3,3],0.0,"ds/ds has wrong value at (1,1,3,3).")
      self.assertEqual(dsds[1,1,3,4],0.0,"ds/ds has wrong value at (1,1,3,4).")
      self.assertEqual(dsds[1,2,0,0],0.0,"ds/ds has wrong value at (1,2,0,0).")
      self.assertEqual(dsds[1,2,0,1],0.0,"ds/ds has wrong value at (1,2,0,1).")
      self.assertEqual(dsds[1,2,0,2],0.0,"ds/ds has wrong value at (1,2,0,2).")
      self.assertEqual(dsds[1,2,0,3],0.0,"ds/ds has wrong value at (1,2,0,3).")
      self.assertEqual(dsds[1,2,0,4],0.0,"ds/ds has wrong value at (1,2,0,4).")
      self.assertEqual(dsds[1,2,1,0],0.0,"ds/ds has wrong value at (1,2,1,0).")
      self.assertEqual(dsds[1,2,1,1],0.0,"ds/ds has wrong value at (1,2,1,1).")
      self.assertEqual(dsds[1,2,1,2],1.0,"ds/ds has wrong value at (1,2,1,2).")
      self.assertEqual(dsds[1,2,1,3],0.0,"ds/ds has wrong value at (1,2,1,3).")
      self.assertEqual(dsds[1,2,1,4],0.0,"ds/ds has wrong value at (1,2,1,4).")
      self.assertEqual(dsds[1,2,2,0],0.0,"ds/ds has wrong value at (1,2,2,0).")
      self.assertEqual(dsds[1,2,2,1],0.0,"ds/ds has wrong value at (1,2,2,1).")
      self.assertEqual(dsds[1,2,2,2],0.0,"ds/ds has wrong value at (1,2,2,2).")
      self.assertEqual(dsds[1,2,2,3],0.0,"ds/ds has wrong value at (1,2,2,3).")
      self.assertEqual(dsds[1,2,2,4],0.0,"ds/ds has wrong value at (1,2,2,4).")
      self.assertEqual(dsds[1,2,3,0],0.0,"ds/ds has wrong value at (1,2,3,0).")
      self.assertEqual(dsds[1,2,3,1],0.0,"ds/ds has wrong value at (1,2,3,1).")
      self.assertEqual(dsds[1,2,3,2],0.0,"ds/ds has wrong value at (1,2,3,2).")
      self.assertEqual(dsds[1,2,3,3],0.0,"ds/ds has wrong value at (1,2,3,3).")
      self.assertEqual(dsds[1,2,3,4],0.0,"ds/ds has wrong value at (1,2,3,4).")
      self.assertEqual(dsds[1,3,0,0],0.0,"ds/ds has wrong value at (1,3,0,0).")
      self.assertEqual(dsds[1,3,0,1],0.0,"ds/ds has wrong value at (1,3,0,1).")
      self.assertEqual(dsds[1,3,0,2],0.0,"ds/ds has wrong value at (1,3,0,2).")
      self.assertEqual(dsds[1,3,0,3],0.0,"ds/ds has wrong value at (1,3,0,3).")
      self.assertEqual(dsds[1,3,0,4],0.0,"ds/ds has wrong value at (1,3,0,4).")
      self.assertEqual(dsds[1,3,1,0],0.0,"ds/ds has wrong value at (1,3,1,0).")
      self.assertEqual(dsds[1,3,1,1],0.0,"ds/ds has wrong value at (1,3,1,1).")
      self.assertEqual(dsds[1,3,1,2],0.0,"ds/ds has wrong value at (1,3,1,2).")
      self.assertEqual(dsds[1,3,1,3],1.0,"ds/ds has wrong value at (1,3,1,3).")
      self.assertEqual(dsds[1,3,1,4],0.0,"ds/ds has wrong value at (1,3,1,4).")
      self.assertEqual(dsds[1,3,2,0],0.0,"ds/ds has wrong value at (1,3,2,0).")
      self.assertEqual(dsds[1,3,2,1],0.0,"ds/ds has wrong value at (1,3,2,1).")
      self.assertEqual(dsds[1,3,2,2],0.0,"ds/ds has wrong value at (1,3,2,2).")
      self.assertEqual(dsds[1,3,2,3],0.0,"ds/ds has wrong value at (1,3,2,3).")
      self.assertEqual(dsds[1,3,2,4],0.0,"ds/ds has wrong value at (1,3,2,4).")
      self.assertEqual(dsds[1,3,3,0],0.0,"ds/ds has wrong value at (1,3,3,0).")
      self.assertEqual(dsds[1,3,3,1],0.0,"ds/ds has wrong value at (1,3,3,1).")
      self.assertEqual(dsds[1,3,3,2],0.0,"ds/ds has wrong value at (1,3,3,2).")
      self.assertEqual(dsds[1,3,3,3],0.0,"ds/ds has wrong value at (1,3,3,3).")
      self.assertEqual(dsds[1,3,3,4],0.0,"ds/ds has wrong value at (1,3,3,4).")
      self.assertEqual(dsds[1,4,0,0],0.0,"ds/ds has wrong value at (1,4,0,0).")
      self.assertEqual(dsds[1,4,0,1],0.0,"ds/ds has wrong value at (1,4,0,1).")
      self.assertEqual(dsds[1,4,0,2],0.0,"ds/ds has wrong value at (1,4,0,2).")
      self.assertEqual(dsds[1,4,0,3],0.0,"ds/ds has wrong value at (1,4,0,3).")
      self.assertEqual(dsds[1,4,0,4],0.0,"ds/ds has wrong value at (1,4,0,4).")
      self.assertEqual(dsds[1,4,1,0],0.0,"ds/ds has wrong value at (1,4,1,0).")
      self.assertEqual(dsds[1,4,1,1],0.0,"ds/ds has wrong value at (1,4,1,1).")
      self.assertEqual(dsds[1,4,1,2],0.0,"ds/ds has wrong value at (1,4,1,2).")
      self.assertEqual(dsds[1,4,1,3],0.0,"ds/ds has wrong value at (1,4,1,3).")
      self.assertEqual(dsds[1,4,1,4],1.0,"ds/ds has wrong value at (1,4,1,4).")
      self.assertEqual(dsds[1,4,2,0],0.0,"ds/ds has wrong value at (1,4,2,0).")
      self.assertEqual(dsds[1,4,2,1],0.0,"ds/ds has wrong value at (1,4,2,1).")
      self.assertEqual(dsds[1,4,2,2],0.0,"ds/ds has wrong value at (1,4,2,2).")
      self.assertEqual(dsds[1,4,2,3],0.0,"ds/ds has wrong value at (1,4,2,3).")
      self.assertEqual(dsds[1,4,2,4],0.0,"ds/ds has wrong value at (1,4,2,4).")
      self.assertEqual(dsds[1,4,3,0],0.0,"ds/ds has wrong value at (1,4,3,0).")
      self.assertEqual(dsds[1,4,3,1],0.0,"ds/ds has wrong value at (1,4,3,1).")
      self.assertEqual(dsds[1,4,3,2],0.0,"ds/ds has wrong value at (1,4,3,2).")
      self.assertEqual(dsds[1,4,3,3],0.0,"ds/ds has wrong value at (1,4,3,3).")
      self.assertEqual(dsds[1,4,3,4],0.0,"ds/ds has wrong value at (1,4,3,4).")
      self.assertEqual(dsds[2,0,0,0],0.0,"ds/ds has wrong value at (2,0,0,0).")
      self.assertEqual(dsds[2,0,0,1],0.0,"ds/ds has wrong value at (2,0,0,1).")
      self.assertEqual(dsds[2,0,0,2],0.0,"ds/ds has wrong value at (2,0,0,2).")
      self.assertEqual(dsds[2,0,0,3],0.0,"ds/ds has wrong value at (2,0,0,3).")
      self.assertEqual(dsds[2,0,0,4],0.0,"ds/ds has wrong value at (2,0,0,4).")
      self.assertEqual(dsds[2,0,1,0],0.0,"ds/ds has wrong value at (2,0,1,0).")
      self.assertEqual(dsds[2,0,1,1],0.0,"ds/ds has wrong value at (2,0,1,1).")
      self.assertEqual(dsds[2,0,1,2],0.0,"ds/ds has wrong value at (2,0,1,2).")
      self.assertEqual(dsds[2,0,1,3],0.0,"ds/ds has wrong value at (2,0,1,3).")
      self.assertEqual(dsds[2,0,1,4],0.0,"ds/ds has wrong value at (2,0,1,4).")
      self.assertEqual(dsds[2,0,2,0],1.0,"ds/ds has wrong value at (2,0,2,0).")
      self.assertEqual(dsds[2,0,2,1],0.0,"ds/ds has wrong value at (2,0,2,1).")
      self.assertEqual(dsds[2,0,2,2],0.0,"ds/ds has wrong value at (2,0,2,2).")
      self.assertEqual(dsds[2,0,2,3],0.0,"ds/ds has wrong value at (2,0,2,3).")
      self.assertEqual(dsds[2,0,2,4],0.0,"ds/ds has wrong value at (2,0,2,4).")
      self.assertEqual(dsds[2,0,3,0],0.0,"ds/ds has wrong value at (2,0,3,0).")
      self.assertEqual(dsds[2,0,3,1],0.0,"ds/ds has wrong value at (2,0,3,1).")
      self.assertEqual(dsds[2,0,3,2],0.0,"ds/ds has wrong value at (2,0,3,2).")
      self.assertEqual(dsds[2,0,3,3],0.0,"ds/ds has wrong value at (2,0,3,3).")
      self.assertEqual(dsds[2,0,3,4],0.0,"ds/ds has wrong value at (2,0,3,4).")
      self.assertEqual(dsds[2,1,0,0],0.0,"ds/ds has wrong value at (2,1,0,0).")
      self.assertEqual(dsds[2,1,0,1],0.0,"ds/ds has wrong value at (2,1,0,1).")
      self.assertEqual(dsds[2,1,0,2],0.0,"ds/ds has wrong value at (2,1,0,2).")
      self.assertEqual(dsds[2,1,0,3],0.0,"ds/ds has wrong value at (2,1,0,3).")
      self.assertEqual(dsds[2,1,0,4],0.0,"ds/ds has wrong value at (2,1,0,4).")
      self.assertEqual(dsds[2,1,1,0],0.0,"ds/ds has wrong value at (2,1,1,0).")
      self.assertEqual(dsds[2,1,1,1],0.0,"ds/ds has wrong value at (2,1,1,1).")
      self.assertEqual(dsds[2,1,1,2],0.0,"ds/ds has wrong value at (2,1,1,2).")
      self.assertEqual(dsds[2,1,1,3],0.0,"ds/ds has wrong value at (2,1,1,3).")
      self.assertEqual(dsds[2,1,1,4],0.0,"ds/ds has wrong value at (2,1,1,4).")
      self.assertEqual(dsds[2,1,2,0],0.0,"ds/ds has wrong value at (2,1,2,0).")
      self.assertEqual(dsds[2,1,2,1],1.0,"ds/ds has wrong value at (2,1,2,1).")
      self.assertEqual(dsds[2,1,2,2],0.0,"ds/ds has wrong value at (2,1,2,2).")
      self.assertEqual(dsds[2,1,2,3],0.0,"ds/ds has wrong value at (2,1,2,3).")
      self.assertEqual(dsds[2,1,2,4],0.0,"ds/ds has wrong value at (2,1,2,4).")
      self.assertEqual(dsds[2,1,3,0],0.0,"ds/ds has wrong value at (2,1,3,0).")
      self.assertEqual(dsds[2,1,3,1],0.0,"ds/ds has wrong value at (2,1,3,1).")
      self.assertEqual(dsds[2,1,3,2],0.0,"ds/ds has wrong value at (2,1,3,2).")
      self.assertEqual(dsds[2,1,3,3],0.0,"ds/ds has wrong value at (2,1,3,3).")
      self.assertEqual(dsds[2,1,3,4],0.0,"ds/ds has wrong value at (2,1,3,4).")
      self.assertEqual(dsds[2,2,0,0],0.0,"ds/ds has wrong value at (2,2,0,0).")
      self.assertEqual(dsds[2,2,0,1],0.0,"ds/ds has wrong value at (2,2,0,1).")
      self.assertEqual(dsds[2,2,0,2],0.0,"ds/ds has wrong value at (2,2,0,2).")
      self.assertEqual(dsds[2,2,0,3],0.0,"ds/ds has wrong value at (2,2,0,3).")
      self.assertEqual(dsds[2,2,0,4],0.0,"ds/ds has wrong value at (2,2,0,4).")
      self.assertEqual(dsds[2,2,1,0],0.0,"ds/ds has wrong value at (2,2,1,0).")
      self.assertEqual(dsds[2,2,1,1],0.0,"ds/ds has wrong value at (2,2,1,1).")
      self.assertEqual(dsds[2,2,1,2],0.0,"ds/ds has wrong value at (2,2,1,2).")
      self.assertEqual(dsds[2,2,1,3],0.0,"ds/ds has wrong value at (2,2,1,3).")
      self.assertEqual(dsds[2,2,1,4],0.0,"ds/ds has wrong value at (2,2,1,4).")
      self.assertEqual(dsds[2,2,2,0],0.0,"ds/ds has wrong value at (2,2,2,0).")
      self.assertEqual(dsds[2,2,2,1],0.0,"ds/ds has wrong value at (2,2,2,1).")
      self.assertEqual(dsds[2,2,2,2],1.0,"ds/ds has wrong value at (2,2,2,2).")
      self.assertEqual(dsds[2,2,2,3],0.0,"ds/ds has wrong value at (2,2,2,3).")
      self.assertEqual(dsds[2,2,2,4],0.0,"ds/ds has wrong value at (2,2,2,4).")
      self.assertEqual(dsds[2,2,3,0],0.0,"ds/ds has wrong value at (2,2,3,0).")
      self.assertEqual(dsds[2,2,3,1],0.0,"ds/ds has wrong value at (2,2,3,1).")
      self.assertEqual(dsds[2,2,3,2],0.0,"ds/ds has wrong value at (2,2,3,2).")
      self.assertEqual(dsds[2,2,3,3],0.0,"ds/ds has wrong value at (2,2,3,3).")
      self.assertEqual(dsds[2,2,3,4],0.0,"ds/ds has wrong value at (2,2,3,4).")
      self.assertEqual(dsds[2,3,0,0],0.0,"ds/ds has wrong value at (2,3,0,0).")
      self.assertEqual(dsds[2,3,0,1],0.0,"ds/ds has wrong value at (2,3,0,1).")
      self.assertEqual(dsds[2,3,0,2],0.0,"ds/ds has wrong value at (2,3,0,2).")
      self.assertEqual(dsds[2,3,0,3],0.0,"ds/ds has wrong value at (2,3,0,3).")
      self.assertEqual(dsds[2,3,0,4],0.0,"ds/ds has wrong value at (2,3,0,4).")
      self.assertEqual(dsds[2,3,1,0],0.0,"ds/ds has wrong value at (2,3,1,0).")
      self.assertEqual(dsds[2,3,1,1],0.0,"ds/ds has wrong value at (2,3,1,1).")
      self.assertEqual(dsds[2,3,1,2],0.0,"ds/ds has wrong value at (2,3,1,2).")
      self.assertEqual(dsds[2,3,1,3],0.0,"ds/ds has wrong value at (2,3,1,3).")
      self.assertEqual(dsds[2,3,1,4],0.0,"ds/ds has wrong value at (2,3,1,4).")
      self.assertEqual(dsds[2,3,2,0],0.0,"ds/ds has wrong value at (2,3,2,0).")
      self.assertEqual(dsds[2,3,2,1],0.0,"ds/ds has wrong value at (2,3,2,1).")
      self.assertEqual(dsds[2,3,2,2],0.0,"ds/ds has wrong value at (2,3,2,2).")
      self.assertEqual(dsds[2,3,2,3],1.0,"ds/ds has wrong value at (2,3,2,3).")
      self.assertEqual(dsds[2,3,2,4],0.0,"ds/ds has wrong value at (2,3,2,4).")
      self.assertEqual(dsds[2,3,3,0],0.0,"ds/ds has wrong value at (2,3,3,0).")
      self.assertEqual(dsds[2,3,3,1],0.0,"ds/ds has wrong value at (2,3,3,1).")
      self.assertEqual(dsds[2,3,3,2],0.0,"ds/ds has wrong value at (2,3,3,2).")
      self.assertEqual(dsds[2,3,3,3],0.0,"ds/ds has wrong value at (2,3,3,3).")
      self.assertEqual(dsds[2,3,3,4],0.0,"ds/ds has wrong value at (2,3,3,4).")
      self.assertEqual(dsds[2,4,0,0],0.0,"ds/ds has wrong value at (2,4,0,0).")
      self.assertEqual(dsds[2,4,0,1],0.0,"ds/ds has wrong value at (2,4,0,1).")
      self.assertEqual(dsds[2,4,0,2],0.0,"ds/ds has wrong value at (2,4,0,2).")
      self.assertEqual(dsds[2,4,0,3],0.0,"ds/ds has wrong value at (2,4,0,3).")
      self.assertEqual(dsds[2,4,0,4],0.0,"ds/ds has wrong value at (2,4,0,4).")
      self.assertEqual(dsds[2,4,1,0],0.0,"ds/ds has wrong value at (2,4,1,0).")
      self.assertEqual(dsds[2,4,1,1],0.0,"ds/ds has wrong value at (2,4,1,1).")
      self.assertEqual(dsds[2,4,1,2],0.0,"ds/ds has wrong value at (2,4,1,2).")
      self.assertEqual(dsds[2,4,1,3],0.0,"ds/ds has wrong value at (2,4,1,3).")
      self.assertEqual(dsds[2,4,1,4],0.0,"ds/ds has wrong value at (2,4,1,4).")
      self.assertEqual(dsds[2,4,2,0],0.0,"ds/ds has wrong value at (2,4,2,0).")
      self.assertEqual(dsds[2,4,2,1],0.0,"ds/ds has wrong value at (2,4,2,1).")
      self.assertEqual(dsds[2,4,2,2],0.0,"ds/ds has wrong value at (2,4,2,2).")
      self.assertEqual(dsds[2,4,2,3],0.0,"ds/ds has wrong value at (2,4,2,3).")
      self.assertEqual(dsds[2,4,2,4],1.0,"ds/ds has wrong value at (2,4,2,4).")
      self.assertEqual(dsds[2,4,3,0],0.0,"ds/ds has wrong value at (2,4,3,0).")
      self.assertEqual(dsds[2,4,3,1],0.0,"ds/ds has wrong value at (2,4,3,1).")
      self.assertEqual(dsds[2,4,3,2],0.0,"ds/ds has wrong value at (2,4,3,2).")
      self.assertEqual(dsds[2,4,3,3],0.0,"ds/ds has wrong value at (2,4,3,3).")
      self.assertEqual(dsds[2,4,3,4],0.0,"ds/ds has wrong value at (2,4,3,4).")
      self.assertEqual(dsds[3,0,0,0],0.0,"ds/ds has wrong value at (3,0,0,0).")
      self.assertEqual(dsds[3,0,0,1],0.0,"ds/ds has wrong value at (3,0,0,1).")
      self.assertEqual(dsds[3,0,0,2],0.0,"ds/ds has wrong value at (3,0,0,2).")
      self.assertEqual(dsds[3,0,0,3],0.0,"ds/ds has wrong value at (3,0,0,3).")
      self.assertEqual(dsds[3,0,0,4],0.0,"ds/ds has wrong value at (3,0,0,4).")
      self.assertEqual(dsds[3,0,1,0],0.0,"ds/ds has wrong value at (3,0,1,0).")
      self.assertEqual(dsds[3,0,1,1],0.0,"ds/ds has wrong value at (3,0,1,1).")
      self.assertEqual(dsds[3,0,1,2],0.0,"ds/ds has wrong value at (3,0,1,2).")
      self.assertEqual(dsds[3,0,1,3],0.0,"ds/ds has wrong value at (3,0,1,3).")
      self.assertEqual(dsds[3,0,1,4],0.0,"ds/ds has wrong value at (3,0,1,4).")
      self.assertEqual(dsds[3,0,2,0],0.0,"ds/ds has wrong value at (3,0,2,0).")
      self.assertEqual(dsds[3,0,2,1],0.0,"ds/ds has wrong value at (3,0,2,1).")
      self.assertEqual(dsds[3,0,2,2],0.0,"ds/ds has wrong value at (3,0,2,2).")
      self.assertEqual(dsds[3,0,2,3],0.0,"ds/ds has wrong value at (3,0,2,3).")
      self.assertEqual(dsds[3,0,2,4],0.0,"ds/ds has wrong value at (3,0,2,4).")
      self.assertEqual(dsds[3,0,3,0],1.0,"ds/ds has wrong value at (3,0,3,0).")
      self.assertEqual(dsds[3,0,3,1],0.0,"ds/ds has wrong value at (3,0,3,1).")
      self.assertEqual(dsds[3,0,3,2],0.0,"ds/ds has wrong value at (3,0,3,2).")
      self.assertEqual(dsds[3,0,3,3],0.0,"ds/ds has wrong value at (3,0,3,3).")
      self.assertEqual(dsds[3,0,3,4],0.0,"ds/ds has wrong value at (3,0,3,4).")
      self.assertEqual(dsds[3,1,0,0],0.0,"ds/ds has wrong value at (3,1,0,0).")
      self.assertEqual(dsds[3,1,0,1],0.0,"ds/ds has wrong value at (3,1,0,1).")
      self.assertEqual(dsds[3,1,0,2],0.0,"ds/ds has wrong value at (3,1,0,2).")
      self.assertEqual(dsds[3,1,0,3],0.0,"ds/ds has wrong value at (3,1,0,3).")
      self.assertEqual(dsds[3,1,0,4],0.0,"ds/ds has wrong value at (3,1,0,4).")
      self.assertEqual(dsds[3,1,1,0],0.0,"ds/ds has wrong value at (3,1,1,0).")
      self.assertEqual(dsds[3,1,1,1],0.0,"ds/ds has wrong value at (3,1,1,1).")
      self.assertEqual(dsds[3,1,1,2],0.0,"ds/ds has wrong value at (3,1,1,2).")
      self.assertEqual(dsds[3,1,1,3],0.0,"ds/ds has wrong value at (3,1,1,3).")
      self.assertEqual(dsds[3,1,1,4],0.0,"ds/ds has wrong value at (3,1,1,4).")
      self.assertEqual(dsds[3,1,2,0],0.0,"ds/ds has wrong value at (3,1,2,0).")
      self.assertEqual(dsds[3,1,2,1],0.0,"ds/ds has wrong value at (3,1,2,1).")
      self.assertEqual(dsds[3,1,2,2],0.0,"ds/ds has wrong value at (3,1,2,2).")
      self.assertEqual(dsds[3,1,2,3],0.0,"ds/ds has wrong value at (3,1,2,3).")
      self.assertEqual(dsds[3,1,2,4],0.0,"ds/ds has wrong value at (3,1,2,4).")
      self.assertEqual(dsds[3,1,3,0],0.0,"ds/ds has wrong value at (3,1,3,0).")
      self.assertEqual(dsds[3,1,3,1],1.0,"ds/ds has wrong value at (3,1,3,1).")
      self.assertEqual(dsds[3,1,3,2],0.0,"ds/ds has wrong value at (3,1,3,2).")
      self.assertEqual(dsds[3,1,3,3],0.0,"ds/ds has wrong value at (3,1,3,3).")
      self.assertEqual(dsds[3,1,3,4],0.0,"ds/ds has wrong value at (3,1,3,4).")
      self.assertEqual(dsds[3,2,0,0],0.0,"ds/ds has wrong value at (3,2,0,0).")
      self.assertEqual(dsds[3,2,0,1],0.0,"ds/ds has wrong value at (3,2,0,1).")
      self.assertEqual(dsds[3,2,0,2],0.0,"ds/ds has wrong value at (3,2,0,2).")
      self.assertEqual(dsds[3,2,0,3],0.0,"ds/ds has wrong value at (3,2,0,3).")
      self.assertEqual(dsds[3,2,0,4],0.0,"ds/ds has wrong value at (3,2,0,4).")
      self.assertEqual(dsds[3,2,1,0],0.0,"ds/ds has wrong value at (3,2,1,0).")
      self.assertEqual(dsds[3,2,1,1],0.0,"ds/ds has wrong value at (3,2,1,1).")
      self.assertEqual(dsds[3,2,1,2],0.0,"ds/ds has wrong value at (3,2,1,2).")
      self.assertEqual(dsds[3,2,1,3],0.0,"ds/ds has wrong value at (3,2,1,3).")
      self.assertEqual(dsds[3,2,1,4],0.0,"ds/ds has wrong value at (3,2,1,4).")
      self.assertEqual(dsds[3,2,2,0],0.0,"ds/ds has wrong value at (3,2,2,0).")
      self.assertEqual(dsds[3,2,2,1],0.0,"ds/ds has wrong value at (3,2,2,1).")
      self.assertEqual(dsds[3,2,2,2],0.0,"ds/ds has wrong value at (3,2,2,2).")
      self.assertEqual(dsds[3,2,2,3],0.0,"ds/ds has wrong value at (3,2,2,3).")
      self.assertEqual(dsds[3,2,2,4],0.0,"ds/ds has wrong value at (3,2,2,4).")
      self.assertEqual(dsds[3,2,3,0],0.0,"ds/ds has wrong value at (3,2,3,0).")
      self.assertEqual(dsds[3,2,3,1],0.0,"ds/ds has wrong value at (3,2,3,1).")
      self.assertEqual(dsds[3,2,3,2],1.0,"ds/ds has wrong value at (3,2,3,2).")
      self.assertEqual(dsds[3,2,3,3],0.0,"ds/ds has wrong value at (3,2,3,3).")
      self.assertEqual(dsds[3,2,3,4],0.0,"ds/ds has wrong value at (3,2,3,4).")
      self.assertEqual(dsds[3,3,0,0],0.0,"ds/ds has wrong value at (3,3,0,0).")
      self.assertEqual(dsds[3,3,0,1],0.0,"ds/ds has wrong value at (3,3,0,1).")
      self.assertEqual(dsds[3,3,0,2],0.0,"ds/ds has wrong value at (3,3,0,2).")
      self.assertEqual(dsds[3,3,0,3],0.0,"ds/ds has wrong value at (3,3,0,3).")
      self.assertEqual(dsds[3,3,0,4],0.0,"ds/ds has wrong value at (3,3,0,4).")
      self.assertEqual(dsds[3,3,1,0],0.0,"ds/ds has wrong value at (3,3,1,0).")
      self.assertEqual(dsds[3,3,1,1],0.0,"ds/ds has wrong value at (3,3,1,1).")
      self.assertEqual(dsds[3,3,1,2],0.0,"ds/ds has wrong value at (3,3,1,2).")
      self.assertEqual(dsds[3,3,1,3],0.0,"ds/ds has wrong value at (3,3,1,3).")
      self.assertEqual(dsds[3,3,1,4],0.0,"ds/ds has wrong value at (3,3,1,4).")
      self.assertEqual(dsds[3,3,2,0],0.0,"ds/ds has wrong value at (3,3,2,0).")
      self.assertEqual(dsds[3,3,2,1],0.0,"ds/ds has wrong value at (3,3,2,1).")
      self.assertEqual(dsds[3,3,2,2],0.0,"ds/ds has wrong value at (3,3,2,2).")
      self.assertEqual(dsds[3,3,2,3],0.0,"ds/ds has wrong value at (3,3,2,3).")
      self.assertEqual(dsds[3,3,2,4],0.0,"ds/ds has wrong value at (3,3,2,4).")
      self.assertEqual(dsds[3,3,3,0],0.0,"ds/ds has wrong value at (3,3,3,0).")
      self.assertEqual(dsds[3,3,3,1],0.0,"ds/ds has wrong value at (3,3,3,1).")
      self.assertEqual(dsds[3,3,3,2],0.0,"ds/ds has wrong value at (3,3,3,2).")
      self.assertEqual(dsds[3,3,3,3],1.0,"ds/ds has wrong value at (3,3,3,3).")
      self.assertEqual(dsds[3,3,3,4],0.0,"ds/ds has wrong value at (3,3,3,4).")
      self.assertEqual(dsds[3,4,0,0],0.0,"ds/ds has wrong value at (3,4,0,0).")
      self.assertEqual(dsds[3,4,0,1],0.0,"ds/ds has wrong value at (3,4,0,1).")
      self.assertEqual(dsds[3,4,0,2],0.0,"ds/ds has wrong value at (3,4,0,2).")
      self.assertEqual(dsds[3,4,0,3],0.0,"ds/ds has wrong value at (3,4,0,3).")
      self.assertEqual(dsds[3,4,0,4],0.0,"ds/ds has wrong value at (3,4,0,4).")
      self.assertEqual(dsds[3,4,1,0],0.0,"ds/ds has wrong value at (3,4,1,0).")
      self.assertEqual(dsds[3,4,1,1],0.0,"ds/ds has wrong value at (3,4,1,1).")
      self.assertEqual(dsds[3,4,1,2],0.0,"ds/ds has wrong value at (3,4,1,2).")
      self.assertEqual(dsds[3,4,1,3],0.0,"ds/ds has wrong value at (3,4,1,3).")
      self.assertEqual(dsds[3,4,1,4],0.0,"ds/ds has wrong value at (3,4,1,4).")
      self.assertEqual(dsds[3,4,2,0],0.0,"ds/ds has wrong value at (3,4,2,0).")
      self.assertEqual(dsds[3,4,2,1],0.0,"ds/ds has wrong value at (3,4,2,1).")
      self.assertEqual(dsds[3,4,2,2],0.0,"ds/ds has wrong value at (3,4,2,2).")
      self.assertEqual(dsds[3,4,2,3],0.0,"ds/ds has wrong value at (3,4,2,3).")
      self.assertEqual(dsds[3,4,2,4],0.0,"ds/ds has wrong value at (3,4,2,4).")
      self.assertEqual(dsds[3,4,3,0],0.0,"ds/ds has wrong value at (3,4,3,0).")
      self.assertEqual(dsds[3,4,3,1],0.0,"ds/ds has wrong value at (3,4,3,1).")
      self.assertEqual(dsds[3,4,3,2],0.0,"ds/ds has wrong value at (3,4,3,2).")
      self.assertEqual(dsds[3,4,3,3],0.0,"ds/ds has wrong value at (3,4,3,3).")
      self.assertEqual(dsds[3,4,3,4],1.0,"ds/ds has wrong value at (3,4,3,4).")

      self.assertEqual(s.getArgument(0),1,"wrong argument 0.")
      self.assertEqual(s.getArgument(1),-1.0,"wrong argument 1.")
      sa=s.getSubstitutedArguments({s2:-10})
      self.assertEqual(len(sa),2,"wrong number of substituted arguments")
      self.assertEqual(sa[0],1,"wrongly substituted argument 0.")
      self.assertEqual(sa[1],-1.0,"wrongly substituted argument 1.")

      self.assertFalse(s.isAppropriateValue(10.)," 10. is not appropriate substitute")
      self.assertFalse(s.isAppropriateValue(10)," 10 is not appropriate substitute")
      self.assertTrue(s.isAppropriateValue(Symbol(shape=(4, 5),dim=1)),"Symbol is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(Symbol(shape=(4, 5),dim=2)),"Symbol is not appropriate substitute (dim)")
      self.assertFalse(s.isAppropriateValue(escript.Data(0.,(5,),self.functionspace)),"Data is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(numpy.zeros((4, 5))),"ndarray is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(numpy.zeros((5,))),"ndarray is not appropriate substitute (shape)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_Symbol_rank2_d2_nargs0(self):
      s2=Symbol(shape=())
      s=Symbol(shape=(4, 5),dim=2,args=[])
      self.assertEqual(s.getRank(),2,"wrong rank.")
      self.assertEqual(s.getShape(),(4, 5),"wrong shape.")
      self.assertEqual(s.getDim(),2,"wrong spatial dimension.")
      self.assertEqual(s.getArgument(),[],"wrong arguments.")

      ss=s.substitute({s:numpy.zeros((4, 5))})
      self.assertTrue(isinstance(ss,numpy.ndarray),"value after substitution is not ndarray.")
      self.assertEqual(ss.shape,(4, 5),"value after substitution has not expected shape")
      try:
        s.substitute({s:numpy.zeros((5,))})
        fail("illegal substition was successful")
      except TypeError:
        pass

      dsdarg=s.diff(Symbol(shape=()))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds() has wrong type.")
      self.assertEqual(dsdarg.shape,(4, 5),"ds/ds() has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds() has wrong value.")
      dsdarg=s.diff(Symbol(shape=(2,)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(2,) has wrong type.")
      self.assertEqual(dsdarg.shape,(4, 5, 2),"ds/ds(2,) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(2,) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(4, 5)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(4, 5) has wrong type.")
      self.assertEqual(dsdarg.shape,(4, 5, 4, 5),"ds/ds(4, 5) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(4, 5) has wrong value.")

      dsds=s.diff(s)
      self.assertTrue(isinstance(dsds,numpy.ndarray),"ds/ds has wrong type.")
      self.assertEqual(dsds.shape,(4, 5, 4, 5),"ds/ds has wrong shape.")
      self.assertEqual(dsds[0,0,0,0],1.0,"ds/ds has wrong value at (0,0,0,0).")
      self.assertEqual(dsds[0,0,0,1],0.0,"ds/ds has wrong value at (0,0,0,1).")
      self.assertEqual(dsds[0,0,0,2],0.0,"ds/ds has wrong value at (0,0,0,2).")
      self.assertEqual(dsds[0,0,0,3],0.0,"ds/ds has wrong value at (0,0,0,3).")
      self.assertEqual(dsds[0,0,0,4],0.0,"ds/ds has wrong value at (0,0,0,4).")
      self.assertEqual(dsds[0,0,1,0],0.0,"ds/ds has wrong value at (0,0,1,0).")
      self.assertEqual(dsds[0,0,1,1],0.0,"ds/ds has wrong value at (0,0,1,1).")
      self.assertEqual(dsds[0,0,1,2],0.0,"ds/ds has wrong value at (0,0,1,2).")
      self.assertEqual(dsds[0,0,1,3],0.0,"ds/ds has wrong value at (0,0,1,3).")
      self.assertEqual(dsds[0,0,1,4],0.0,"ds/ds has wrong value at (0,0,1,4).")
      self.assertEqual(dsds[0,0,2,0],0.0,"ds/ds has wrong value at (0,0,2,0).")
      self.assertEqual(dsds[0,0,2,1],0.0,"ds/ds has wrong value at (0,0,2,1).")
      self.assertEqual(dsds[0,0,2,2],0.0,"ds/ds has wrong value at (0,0,2,2).")
      self.assertEqual(dsds[0,0,2,3],0.0,"ds/ds has wrong value at (0,0,2,3).")
      self.assertEqual(dsds[0,0,2,4],0.0,"ds/ds has wrong value at (0,0,2,4).")
      self.assertEqual(dsds[0,0,3,0],0.0,"ds/ds has wrong value at (0,0,3,0).")
      self.assertEqual(dsds[0,0,3,1],0.0,"ds/ds has wrong value at (0,0,3,1).")
      self.assertEqual(dsds[0,0,3,2],0.0,"ds/ds has wrong value at (0,0,3,2).")
      self.assertEqual(dsds[0,0,3,3],0.0,"ds/ds has wrong value at (0,0,3,3).")
      self.assertEqual(dsds[0,0,3,4],0.0,"ds/ds has wrong value at (0,0,3,4).")
      self.assertEqual(dsds[0,1,0,0],0.0,"ds/ds has wrong value at (0,1,0,0).")
      self.assertEqual(dsds[0,1,0,1],1.0,"ds/ds has wrong value at (0,1,0,1).")
      self.assertEqual(dsds[0,1,0,2],0.0,"ds/ds has wrong value at (0,1,0,2).")
      self.assertEqual(dsds[0,1,0,3],0.0,"ds/ds has wrong value at (0,1,0,3).")
      self.assertEqual(dsds[0,1,0,4],0.0,"ds/ds has wrong value at (0,1,0,4).")
      self.assertEqual(dsds[0,1,1,0],0.0,"ds/ds has wrong value at (0,1,1,0).")
      self.assertEqual(dsds[0,1,1,1],0.0,"ds/ds has wrong value at (0,1,1,1).")
      self.assertEqual(dsds[0,1,1,2],0.0,"ds/ds has wrong value at (0,1,1,2).")
      self.assertEqual(dsds[0,1,1,3],0.0,"ds/ds has wrong value at (0,1,1,3).")
      self.assertEqual(dsds[0,1,1,4],0.0,"ds/ds has wrong value at (0,1,1,4).")
      self.assertEqual(dsds[0,1,2,0],0.0,"ds/ds has wrong value at (0,1,2,0).")
      self.assertEqual(dsds[0,1,2,1],0.0,"ds/ds has wrong value at (0,1,2,1).")
      self.assertEqual(dsds[0,1,2,2],0.0,"ds/ds has wrong value at (0,1,2,2).")
      self.assertEqual(dsds[0,1,2,3],0.0,"ds/ds has wrong value at (0,1,2,3).")
      self.assertEqual(dsds[0,1,2,4],0.0,"ds/ds has wrong value at (0,1,2,4).")
      self.assertEqual(dsds[0,1,3,0],0.0,"ds/ds has wrong value at (0,1,3,0).")
      self.assertEqual(dsds[0,1,3,1],0.0,"ds/ds has wrong value at (0,1,3,1).")
      self.assertEqual(dsds[0,1,3,2],0.0,"ds/ds has wrong value at (0,1,3,2).")
      self.assertEqual(dsds[0,1,3,3],0.0,"ds/ds has wrong value at (0,1,3,3).")
      self.assertEqual(dsds[0,1,3,4],0.0,"ds/ds has wrong value at (0,1,3,4).")
      self.assertEqual(dsds[0,2,0,0],0.0,"ds/ds has wrong value at (0,2,0,0).")
      self.assertEqual(dsds[0,2,0,1],0.0,"ds/ds has wrong value at (0,2,0,1).")
      self.assertEqual(dsds[0,2,0,2],1.0,"ds/ds has wrong value at (0,2,0,2).")
      self.assertEqual(dsds[0,2,0,3],0.0,"ds/ds has wrong value at (0,2,0,3).")
      self.assertEqual(dsds[0,2,0,4],0.0,"ds/ds has wrong value at (0,2,0,4).")
      self.assertEqual(dsds[0,2,1,0],0.0,"ds/ds has wrong value at (0,2,1,0).")
      self.assertEqual(dsds[0,2,1,1],0.0,"ds/ds has wrong value at (0,2,1,1).")
      self.assertEqual(dsds[0,2,1,2],0.0,"ds/ds has wrong value at (0,2,1,2).")
      self.assertEqual(dsds[0,2,1,3],0.0,"ds/ds has wrong value at (0,2,1,3).")
      self.assertEqual(dsds[0,2,1,4],0.0,"ds/ds has wrong value at (0,2,1,4).")
      self.assertEqual(dsds[0,2,2,0],0.0,"ds/ds has wrong value at (0,2,2,0).")
      self.assertEqual(dsds[0,2,2,1],0.0,"ds/ds has wrong value at (0,2,2,1).")
      self.assertEqual(dsds[0,2,2,2],0.0,"ds/ds has wrong value at (0,2,2,2).")
      self.assertEqual(dsds[0,2,2,3],0.0,"ds/ds has wrong value at (0,2,2,3).")
      self.assertEqual(dsds[0,2,2,4],0.0,"ds/ds has wrong value at (0,2,2,4).")
      self.assertEqual(dsds[0,2,3,0],0.0,"ds/ds has wrong value at (0,2,3,0).")
      self.assertEqual(dsds[0,2,3,1],0.0,"ds/ds has wrong value at (0,2,3,1).")
      self.assertEqual(dsds[0,2,3,2],0.0,"ds/ds has wrong value at (0,2,3,2).")
      self.assertEqual(dsds[0,2,3,3],0.0,"ds/ds has wrong value at (0,2,3,3).")
      self.assertEqual(dsds[0,2,3,4],0.0,"ds/ds has wrong value at (0,2,3,4).")
      self.assertEqual(dsds[0,3,0,0],0.0,"ds/ds has wrong value at (0,3,0,0).")
      self.assertEqual(dsds[0,3,0,1],0.0,"ds/ds has wrong value at (0,3,0,1).")
      self.assertEqual(dsds[0,3,0,2],0.0,"ds/ds has wrong value at (0,3,0,2).")
      self.assertEqual(dsds[0,3,0,3],1.0,"ds/ds has wrong value at (0,3,0,3).")
      self.assertEqual(dsds[0,3,0,4],0.0,"ds/ds has wrong value at (0,3,0,4).")
      self.assertEqual(dsds[0,3,1,0],0.0,"ds/ds has wrong value at (0,3,1,0).")
      self.assertEqual(dsds[0,3,1,1],0.0,"ds/ds has wrong value at (0,3,1,1).")
      self.assertEqual(dsds[0,3,1,2],0.0,"ds/ds has wrong value at (0,3,1,2).")
      self.assertEqual(dsds[0,3,1,3],0.0,"ds/ds has wrong value at (0,3,1,3).")
      self.assertEqual(dsds[0,3,1,4],0.0,"ds/ds has wrong value at (0,3,1,4).")
      self.assertEqual(dsds[0,3,2,0],0.0,"ds/ds has wrong value at (0,3,2,0).")
      self.assertEqual(dsds[0,3,2,1],0.0,"ds/ds has wrong value at (0,3,2,1).")
      self.assertEqual(dsds[0,3,2,2],0.0,"ds/ds has wrong value at (0,3,2,2).")
      self.assertEqual(dsds[0,3,2,3],0.0,"ds/ds has wrong value at (0,3,2,3).")
      self.assertEqual(dsds[0,3,2,4],0.0,"ds/ds has wrong value at (0,3,2,4).")
      self.assertEqual(dsds[0,3,3,0],0.0,"ds/ds has wrong value at (0,3,3,0).")
      self.assertEqual(dsds[0,3,3,1],0.0,"ds/ds has wrong value at (0,3,3,1).")
      self.assertEqual(dsds[0,3,3,2],0.0,"ds/ds has wrong value at (0,3,3,2).")
      self.assertEqual(dsds[0,3,3,3],0.0,"ds/ds has wrong value at (0,3,3,3).")
      self.assertEqual(dsds[0,3,3,4],0.0,"ds/ds has wrong value at (0,3,3,4).")
      self.assertEqual(dsds[0,4,0,0],0.0,"ds/ds has wrong value at (0,4,0,0).")
      self.assertEqual(dsds[0,4,0,1],0.0,"ds/ds has wrong value at (0,4,0,1).")
      self.assertEqual(dsds[0,4,0,2],0.0,"ds/ds has wrong value at (0,4,0,2).")
      self.assertEqual(dsds[0,4,0,3],0.0,"ds/ds has wrong value at (0,4,0,3).")
      self.assertEqual(dsds[0,4,0,4],1.0,"ds/ds has wrong value at (0,4,0,4).")
      self.assertEqual(dsds[0,4,1,0],0.0,"ds/ds has wrong value at (0,4,1,0).")
      self.assertEqual(dsds[0,4,1,1],0.0,"ds/ds has wrong value at (0,4,1,1).")
      self.assertEqual(dsds[0,4,1,2],0.0,"ds/ds has wrong value at (0,4,1,2).")
      self.assertEqual(dsds[0,4,1,3],0.0,"ds/ds has wrong value at (0,4,1,3).")
      self.assertEqual(dsds[0,4,1,4],0.0,"ds/ds has wrong value at (0,4,1,4).")
      self.assertEqual(dsds[0,4,2,0],0.0,"ds/ds has wrong value at (0,4,2,0).")
      self.assertEqual(dsds[0,4,2,1],0.0,"ds/ds has wrong value at (0,4,2,1).")
      self.assertEqual(dsds[0,4,2,2],0.0,"ds/ds has wrong value at (0,4,2,2).")
      self.assertEqual(dsds[0,4,2,3],0.0,"ds/ds has wrong value at (0,4,2,3).")
      self.assertEqual(dsds[0,4,2,4],0.0,"ds/ds has wrong value at (0,4,2,4).")
      self.assertEqual(dsds[0,4,3,0],0.0,"ds/ds has wrong value at (0,4,3,0).")
      self.assertEqual(dsds[0,4,3,1],0.0,"ds/ds has wrong value at (0,4,3,1).")
      self.assertEqual(dsds[0,4,3,2],0.0,"ds/ds has wrong value at (0,4,3,2).")
      self.assertEqual(dsds[0,4,3,3],0.0,"ds/ds has wrong value at (0,4,3,3).")
      self.assertEqual(dsds[0,4,3,4],0.0,"ds/ds has wrong value at (0,4,3,4).")
      self.assertEqual(dsds[1,0,0,0],0.0,"ds/ds has wrong value at (1,0,0,0).")
      self.assertEqual(dsds[1,0,0,1],0.0,"ds/ds has wrong value at (1,0,0,1).")
      self.assertEqual(dsds[1,0,0,2],0.0,"ds/ds has wrong value at (1,0,0,2).")
      self.assertEqual(dsds[1,0,0,3],0.0,"ds/ds has wrong value at (1,0,0,3).")
      self.assertEqual(dsds[1,0,0,4],0.0,"ds/ds has wrong value at (1,0,0,4).")
      self.assertEqual(dsds[1,0,1,0],1.0,"ds/ds has wrong value at (1,0,1,0).")
      self.assertEqual(dsds[1,0,1,1],0.0,"ds/ds has wrong value at (1,0,1,1).")
      self.assertEqual(dsds[1,0,1,2],0.0,"ds/ds has wrong value at (1,0,1,2).")
      self.assertEqual(dsds[1,0,1,3],0.0,"ds/ds has wrong value at (1,0,1,3).")
      self.assertEqual(dsds[1,0,1,4],0.0,"ds/ds has wrong value at (1,0,1,4).")
      self.assertEqual(dsds[1,0,2,0],0.0,"ds/ds has wrong value at (1,0,2,0).")
      self.assertEqual(dsds[1,0,2,1],0.0,"ds/ds has wrong value at (1,0,2,1).")
      self.assertEqual(dsds[1,0,2,2],0.0,"ds/ds has wrong value at (1,0,2,2).")
      self.assertEqual(dsds[1,0,2,3],0.0,"ds/ds has wrong value at (1,0,2,3).")
      self.assertEqual(dsds[1,0,2,4],0.0,"ds/ds has wrong value at (1,0,2,4).")
      self.assertEqual(dsds[1,0,3,0],0.0,"ds/ds has wrong value at (1,0,3,0).")
      self.assertEqual(dsds[1,0,3,1],0.0,"ds/ds has wrong value at (1,0,3,1).")
      self.assertEqual(dsds[1,0,3,2],0.0,"ds/ds has wrong value at (1,0,3,2).")
      self.assertEqual(dsds[1,0,3,3],0.0,"ds/ds has wrong value at (1,0,3,3).")
      self.assertEqual(dsds[1,0,3,4],0.0,"ds/ds has wrong value at (1,0,3,4).")
      self.assertEqual(dsds[1,1,0,0],0.0,"ds/ds has wrong value at (1,1,0,0).")
      self.assertEqual(dsds[1,1,0,1],0.0,"ds/ds has wrong value at (1,1,0,1).")
      self.assertEqual(dsds[1,1,0,2],0.0,"ds/ds has wrong value at (1,1,0,2).")
      self.assertEqual(dsds[1,1,0,3],0.0,"ds/ds has wrong value at (1,1,0,3).")
      self.assertEqual(dsds[1,1,0,4],0.0,"ds/ds has wrong value at (1,1,0,4).")
      self.assertEqual(dsds[1,1,1,0],0.0,"ds/ds has wrong value at (1,1,1,0).")
      self.assertEqual(dsds[1,1,1,1],1.0,"ds/ds has wrong value at (1,1,1,1).")
      self.assertEqual(dsds[1,1,1,2],0.0,"ds/ds has wrong value at (1,1,1,2).")
      self.assertEqual(dsds[1,1,1,3],0.0,"ds/ds has wrong value at (1,1,1,3).")
      self.assertEqual(dsds[1,1,1,4],0.0,"ds/ds has wrong value at (1,1,1,4).")
      self.assertEqual(dsds[1,1,2,0],0.0,"ds/ds has wrong value at (1,1,2,0).")
      self.assertEqual(dsds[1,1,2,1],0.0,"ds/ds has wrong value at (1,1,2,1).")
      self.assertEqual(dsds[1,1,2,2],0.0,"ds/ds has wrong value at (1,1,2,2).")
      self.assertEqual(dsds[1,1,2,3],0.0,"ds/ds has wrong value at (1,1,2,3).")
      self.assertEqual(dsds[1,1,2,4],0.0,"ds/ds has wrong value at (1,1,2,4).")
      self.assertEqual(dsds[1,1,3,0],0.0,"ds/ds has wrong value at (1,1,3,0).")
      self.assertEqual(dsds[1,1,3,1],0.0,"ds/ds has wrong value at (1,1,3,1).")
      self.assertEqual(dsds[1,1,3,2],0.0,"ds/ds has wrong value at (1,1,3,2).")
      self.assertEqual(dsds[1,1,3,3],0.0,"ds/ds has wrong value at (1,1,3,3).")
      self.assertEqual(dsds[1,1,3,4],0.0,"ds/ds has wrong value at (1,1,3,4).")
      self.assertEqual(dsds[1,2,0,0],0.0,"ds/ds has wrong value at (1,2,0,0).")
      self.assertEqual(dsds[1,2,0,1],0.0,"ds/ds has wrong value at (1,2,0,1).")
      self.assertEqual(dsds[1,2,0,2],0.0,"ds/ds has wrong value at (1,2,0,2).")
      self.assertEqual(dsds[1,2,0,3],0.0,"ds/ds has wrong value at (1,2,0,3).")
      self.assertEqual(dsds[1,2,0,4],0.0,"ds/ds has wrong value at (1,2,0,4).")
      self.assertEqual(dsds[1,2,1,0],0.0,"ds/ds has wrong value at (1,2,1,0).")
      self.assertEqual(dsds[1,2,1,1],0.0,"ds/ds has wrong value at (1,2,1,1).")
      self.assertEqual(dsds[1,2,1,2],1.0,"ds/ds has wrong value at (1,2,1,2).")
      self.assertEqual(dsds[1,2,1,3],0.0,"ds/ds has wrong value at (1,2,1,3).")
      self.assertEqual(dsds[1,2,1,4],0.0,"ds/ds has wrong value at (1,2,1,4).")
      self.assertEqual(dsds[1,2,2,0],0.0,"ds/ds has wrong value at (1,2,2,0).")
      self.assertEqual(dsds[1,2,2,1],0.0,"ds/ds has wrong value at (1,2,2,1).")
      self.assertEqual(dsds[1,2,2,2],0.0,"ds/ds has wrong value at (1,2,2,2).")
      self.assertEqual(dsds[1,2,2,3],0.0,"ds/ds has wrong value at (1,2,2,3).")
      self.assertEqual(dsds[1,2,2,4],0.0,"ds/ds has wrong value at (1,2,2,4).")
      self.assertEqual(dsds[1,2,3,0],0.0,"ds/ds has wrong value at (1,2,3,0).")
      self.assertEqual(dsds[1,2,3,1],0.0,"ds/ds has wrong value at (1,2,3,1).")
      self.assertEqual(dsds[1,2,3,2],0.0,"ds/ds has wrong value at (1,2,3,2).")
      self.assertEqual(dsds[1,2,3,3],0.0,"ds/ds has wrong value at (1,2,3,3).")
      self.assertEqual(dsds[1,2,3,4],0.0,"ds/ds has wrong value at (1,2,3,4).")
      self.assertEqual(dsds[1,3,0,0],0.0,"ds/ds has wrong value at (1,3,0,0).")
      self.assertEqual(dsds[1,3,0,1],0.0,"ds/ds has wrong value at (1,3,0,1).")
      self.assertEqual(dsds[1,3,0,2],0.0,"ds/ds has wrong value at (1,3,0,2).")
      self.assertEqual(dsds[1,3,0,3],0.0,"ds/ds has wrong value at (1,3,0,3).")
      self.assertEqual(dsds[1,3,0,4],0.0,"ds/ds has wrong value at (1,3,0,4).")
      self.assertEqual(dsds[1,3,1,0],0.0,"ds/ds has wrong value at (1,3,1,0).")
      self.assertEqual(dsds[1,3,1,1],0.0,"ds/ds has wrong value at (1,3,1,1).")
      self.assertEqual(dsds[1,3,1,2],0.0,"ds/ds has wrong value at (1,3,1,2).")
      self.assertEqual(dsds[1,3,1,3],1.0,"ds/ds has wrong value at (1,3,1,3).")
      self.assertEqual(dsds[1,3,1,4],0.0,"ds/ds has wrong value at (1,3,1,4).")
      self.assertEqual(dsds[1,3,2,0],0.0,"ds/ds has wrong value at (1,3,2,0).")
      self.assertEqual(dsds[1,3,2,1],0.0,"ds/ds has wrong value at (1,3,2,1).")
      self.assertEqual(dsds[1,3,2,2],0.0,"ds/ds has wrong value at (1,3,2,2).")
      self.assertEqual(dsds[1,3,2,3],0.0,"ds/ds has wrong value at (1,3,2,3).")
      self.assertEqual(dsds[1,3,2,4],0.0,"ds/ds has wrong value at (1,3,2,4).")
      self.assertEqual(dsds[1,3,3,0],0.0,"ds/ds has wrong value at (1,3,3,0).")
      self.assertEqual(dsds[1,3,3,1],0.0,"ds/ds has wrong value at (1,3,3,1).")
      self.assertEqual(dsds[1,3,3,2],0.0,"ds/ds has wrong value at (1,3,3,2).")
      self.assertEqual(dsds[1,3,3,3],0.0,"ds/ds has wrong value at (1,3,3,3).")
      self.assertEqual(dsds[1,3,3,4],0.0,"ds/ds has wrong value at (1,3,3,4).")
      self.assertEqual(dsds[1,4,0,0],0.0,"ds/ds has wrong value at (1,4,0,0).")
      self.assertEqual(dsds[1,4,0,1],0.0,"ds/ds has wrong value at (1,4,0,1).")
      self.assertEqual(dsds[1,4,0,2],0.0,"ds/ds has wrong value at (1,4,0,2).")
      self.assertEqual(dsds[1,4,0,3],0.0,"ds/ds has wrong value at (1,4,0,3).")
      self.assertEqual(dsds[1,4,0,4],0.0,"ds/ds has wrong value at (1,4,0,4).")
      self.assertEqual(dsds[1,4,1,0],0.0,"ds/ds has wrong value at (1,4,1,0).")
      self.assertEqual(dsds[1,4,1,1],0.0,"ds/ds has wrong value at (1,4,1,1).")
      self.assertEqual(dsds[1,4,1,2],0.0,"ds/ds has wrong value at (1,4,1,2).")
      self.assertEqual(dsds[1,4,1,3],0.0,"ds/ds has wrong value at (1,4,1,3).")
      self.assertEqual(dsds[1,4,1,4],1.0,"ds/ds has wrong value at (1,4,1,4).")
      self.assertEqual(dsds[1,4,2,0],0.0,"ds/ds has wrong value at (1,4,2,0).")
      self.assertEqual(dsds[1,4,2,1],0.0,"ds/ds has wrong value at (1,4,2,1).")
      self.assertEqual(dsds[1,4,2,2],0.0,"ds/ds has wrong value at (1,4,2,2).")
      self.assertEqual(dsds[1,4,2,3],0.0,"ds/ds has wrong value at (1,4,2,3).")
      self.assertEqual(dsds[1,4,2,4],0.0,"ds/ds has wrong value at (1,4,2,4).")
      self.assertEqual(dsds[1,4,3,0],0.0,"ds/ds has wrong value at (1,4,3,0).")
      self.assertEqual(dsds[1,4,3,1],0.0,"ds/ds has wrong value at (1,4,3,1).")
      self.assertEqual(dsds[1,4,3,2],0.0,"ds/ds has wrong value at (1,4,3,2).")
      self.assertEqual(dsds[1,4,3,3],0.0,"ds/ds has wrong value at (1,4,3,3).")
      self.assertEqual(dsds[1,4,3,4],0.0,"ds/ds has wrong value at (1,4,3,4).")
      self.assertEqual(dsds[2,0,0,0],0.0,"ds/ds has wrong value at (2,0,0,0).")
      self.assertEqual(dsds[2,0,0,1],0.0,"ds/ds has wrong value at (2,0,0,1).")
      self.assertEqual(dsds[2,0,0,2],0.0,"ds/ds has wrong value at (2,0,0,2).")
      self.assertEqual(dsds[2,0,0,3],0.0,"ds/ds has wrong value at (2,0,0,3).")
      self.assertEqual(dsds[2,0,0,4],0.0,"ds/ds has wrong value at (2,0,0,4).")
      self.assertEqual(dsds[2,0,1,0],0.0,"ds/ds has wrong value at (2,0,1,0).")
      self.assertEqual(dsds[2,0,1,1],0.0,"ds/ds has wrong value at (2,0,1,1).")
      self.assertEqual(dsds[2,0,1,2],0.0,"ds/ds has wrong value at (2,0,1,2).")
      self.assertEqual(dsds[2,0,1,3],0.0,"ds/ds has wrong value at (2,0,1,3).")
      self.assertEqual(dsds[2,0,1,4],0.0,"ds/ds has wrong value at (2,0,1,4).")
      self.assertEqual(dsds[2,0,2,0],1.0,"ds/ds has wrong value at (2,0,2,0).")
      self.assertEqual(dsds[2,0,2,1],0.0,"ds/ds has wrong value at (2,0,2,1).")
      self.assertEqual(dsds[2,0,2,2],0.0,"ds/ds has wrong value at (2,0,2,2).")
      self.assertEqual(dsds[2,0,2,3],0.0,"ds/ds has wrong value at (2,0,2,3).")
      self.assertEqual(dsds[2,0,2,4],0.0,"ds/ds has wrong value at (2,0,2,4).")
      self.assertEqual(dsds[2,0,3,0],0.0,"ds/ds has wrong value at (2,0,3,0).")
      self.assertEqual(dsds[2,0,3,1],0.0,"ds/ds has wrong value at (2,0,3,1).")
      self.assertEqual(dsds[2,0,3,2],0.0,"ds/ds has wrong value at (2,0,3,2).")
      self.assertEqual(dsds[2,0,3,3],0.0,"ds/ds has wrong value at (2,0,3,3).")
      self.assertEqual(dsds[2,0,3,4],0.0,"ds/ds has wrong value at (2,0,3,4).")
      self.assertEqual(dsds[2,1,0,0],0.0,"ds/ds has wrong value at (2,1,0,0).")
      self.assertEqual(dsds[2,1,0,1],0.0,"ds/ds has wrong value at (2,1,0,1).")
      self.assertEqual(dsds[2,1,0,2],0.0,"ds/ds has wrong value at (2,1,0,2).")
      self.assertEqual(dsds[2,1,0,3],0.0,"ds/ds has wrong value at (2,1,0,3).")
      self.assertEqual(dsds[2,1,0,4],0.0,"ds/ds has wrong value at (2,1,0,4).")
      self.assertEqual(dsds[2,1,1,0],0.0,"ds/ds has wrong value at (2,1,1,0).")
      self.assertEqual(dsds[2,1,1,1],0.0,"ds/ds has wrong value at (2,1,1,1).")
      self.assertEqual(dsds[2,1,1,2],0.0,"ds/ds has wrong value at (2,1,1,2).")
      self.assertEqual(dsds[2,1,1,3],0.0,"ds/ds has wrong value at (2,1,1,3).")
      self.assertEqual(dsds[2,1,1,4],0.0,"ds/ds has wrong value at (2,1,1,4).")
      self.assertEqual(dsds[2,1,2,0],0.0,"ds/ds has wrong value at (2,1,2,0).")
      self.assertEqual(dsds[2,1,2,1],1.0,"ds/ds has wrong value at (2,1,2,1).")
      self.assertEqual(dsds[2,1,2,2],0.0,"ds/ds has wrong value at (2,1,2,2).")
      self.assertEqual(dsds[2,1,2,3],0.0,"ds/ds has wrong value at (2,1,2,3).")
      self.assertEqual(dsds[2,1,2,4],0.0,"ds/ds has wrong value at (2,1,2,4).")
      self.assertEqual(dsds[2,1,3,0],0.0,"ds/ds has wrong value at (2,1,3,0).")
      self.assertEqual(dsds[2,1,3,1],0.0,"ds/ds has wrong value at (2,1,3,1).")
      self.assertEqual(dsds[2,1,3,2],0.0,"ds/ds has wrong value at (2,1,3,2).")
      self.assertEqual(dsds[2,1,3,3],0.0,"ds/ds has wrong value at (2,1,3,3).")
      self.assertEqual(dsds[2,1,3,4],0.0,"ds/ds has wrong value at (2,1,3,4).")
      self.assertEqual(dsds[2,2,0,0],0.0,"ds/ds has wrong value at (2,2,0,0).")
      self.assertEqual(dsds[2,2,0,1],0.0,"ds/ds has wrong value at (2,2,0,1).")
      self.assertEqual(dsds[2,2,0,2],0.0,"ds/ds has wrong value at (2,2,0,2).")
      self.assertEqual(dsds[2,2,0,3],0.0,"ds/ds has wrong value at (2,2,0,3).")
      self.assertEqual(dsds[2,2,0,4],0.0,"ds/ds has wrong value at (2,2,0,4).")
      self.assertEqual(dsds[2,2,1,0],0.0,"ds/ds has wrong value at (2,2,1,0).")
      self.assertEqual(dsds[2,2,1,1],0.0,"ds/ds has wrong value at (2,2,1,1).")
      self.assertEqual(dsds[2,2,1,2],0.0,"ds/ds has wrong value at (2,2,1,2).")
      self.assertEqual(dsds[2,2,1,3],0.0,"ds/ds has wrong value at (2,2,1,3).")
      self.assertEqual(dsds[2,2,1,4],0.0,"ds/ds has wrong value at (2,2,1,4).")
      self.assertEqual(dsds[2,2,2,0],0.0,"ds/ds has wrong value at (2,2,2,0).")
      self.assertEqual(dsds[2,2,2,1],0.0,"ds/ds has wrong value at (2,2,2,1).")
      self.assertEqual(dsds[2,2,2,2],1.0,"ds/ds has wrong value at (2,2,2,2).")
      self.assertEqual(dsds[2,2,2,3],0.0,"ds/ds has wrong value at (2,2,2,3).")
      self.assertEqual(dsds[2,2,2,4],0.0,"ds/ds has wrong value at (2,2,2,4).")
      self.assertEqual(dsds[2,2,3,0],0.0,"ds/ds has wrong value at (2,2,3,0).")
      self.assertEqual(dsds[2,2,3,1],0.0,"ds/ds has wrong value at (2,2,3,1).")
      self.assertEqual(dsds[2,2,3,2],0.0,"ds/ds has wrong value at (2,2,3,2).")
      self.assertEqual(dsds[2,2,3,3],0.0,"ds/ds has wrong value at (2,2,3,3).")
      self.assertEqual(dsds[2,2,3,4],0.0,"ds/ds has wrong value at (2,2,3,4).")
      self.assertEqual(dsds[2,3,0,0],0.0,"ds/ds has wrong value at (2,3,0,0).")
      self.assertEqual(dsds[2,3,0,1],0.0,"ds/ds has wrong value at (2,3,0,1).")
      self.assertEqual(dsds[2,3,0,2],0.0,"ds/ds has wrong value at (2,3,0,2).")
      self.assertEqual(dsds[2,3,0,3],0.0,"ds/ds has wrong value at (2,3,0,3).")
      self.assertEqual(dsds[2,3,0,4],0.0,"ds/ds has wrong value at (2,3,0,4).")
      self.assertEqual(dsds[2,3,1,0],0.0,"ds/ds has wrong value at (2,3,1,0).")
      self.assertEqual(dsds[2,3,1,1],0.0,"ds/ds has wrong value at (2,3,1,1).")
      self.assertEqual(dsds[2,3,1,2],0.0,"ds/ds has wrong value at (2,3,1,2).")
      self.assertEqual(dsds[2,3,1,3],0.0,"ds/ds has wrong value at (2,3,1,3).")
      self.assertEqual(dsds[2,3,1,4],0.0,"ds/ds has wrong value at (2,3,1,4).")
      self.assertEqual(dsds[2,3,2,0],0.0,"ds/ds has wrong value at (2,3,2,0).")
      self.assertEqual(dsds[2,3,2,1],0.0,"ds/ds has wrong value at (2,3,2,1).")
      self.assertEqual(dsds[2,3,2,2],0.0,"ds/ds has wrong value at (2,3,2,2).")
      self.assertEqual(dsds[2,3,2,3],1.0,"ds/ds has wrong value at (2,3,2,3).")
      self.assertEqual(dsds[2,3,2,4],0.0,"ds/ds has wrong value at (2,3,2,4).")
      self.assertEqual(dsds[2,3,3,0],0.0,"ds/ds has wrong value at (2,3,3,0).")
      self.assertEqual(dsds[2,3,3,1],0.0,"ds/ds has wrong value at (2,3,3,1).")
      self.assertEqual(dsds[2,3,3,2],0.0,"ds/ds has wrong value at (2,3,3,2).")
      self.assertEqual(dsds[2,3,3,3],0.0,"ds/ds has wrong value at (2,3,3,3).")
      self.assertEqual(dsds[2,3,3,4],0.0,"ds/ds has wrong value at (2,3,3,4).")
      self.assertEqual(dsds[2,4,0,0],0.0,"ds/ds has wrong value at (2,4,0,0).")
      self.assertEqual(dsds[2,4,0,1],0.0,"ds/ds has wrong value at (2,4,0,1).")
      self.assertEqual(dsds[2,4,0,2],0.0,"ds/ds has wrong value at (2,4,0,2).")
      self.assertEqual(dsds[2,4,0,3],0.0,"ds/ds has wrong value at (2,4,0,3).")
      self.assertEqual(dsds[2,4,0,4],0.0,"ds/ds has wrong value at (2,4,0,4).")
      self.assertEqual(dsds[2,4,1,0],0.0,"ds/ds has wrong value at (2,4,1,0).")
      self.assertEqual(dsds[2,4,1,1],0.0,"ds/ds has wrong value at (2,4,1,1).")
      self.assertEqual(dsds[2,4,1,2],0.0,"ds/ds has wrong value at (2,4,1,2).")
      self.assertEqual(dsds[2,4,1,3],0.0,"ds/ds has wrong value at (2,4,1,3).")
      self.assertEqual(dsds[2,4,1,4],0.0,"ds/ds has wrong value at (2,4,1,4).")
      self.assertEqual(dsds[2,4,2,0],0.0,"ds/ds has wrong value at (2,4,2,0).")
      self.assertEqual(dsds[2,4,2,1],0.0,"ds/ds has wrong value at (2,4,2,1).")
      self.assertEqual(dsds[2,4,2,2],0.0,"ds/ds has wrong value at (2,4,2,2).")
      self.assertEqual(dsds[2,4,2,3],0.0,"ds/ds has wrong value at (2,4,2,3).")
      self.assertEqual(dsds[2,4,2,4],1.0,"ds/ds has wrong value at (2,4,2,4).")
      self.assertEqual(dsds[2,4,3,0],0.0,"ds/ds has wrong value at (2,4,3,0).")
      self.assertEqual(dsds[2,4,3,1],0.0,"ds/ds has wrong value at (2,4,3,1).")
      self.assertEqual(dsds[2,4,3,2],0.0,"ds/ds has wrong value at (2,4,3,2).")
      self.assertEqual(dsds[2,4,3,3],0.0,"ds/ds has wrong value at (2,4,3,3).")
      self.assertEqual(dsds[2,4,3,4],0.0,"ds/ds has wrong value at (2,4,3,4).")
      self.assertEqual(dsds[3,0,0,0],0.0,"ds/ds has wrong value at (3,0,0,0).")
      self.assertEqual(dsds[3,0,0,1],0.0,"ds/ds has wrong value at (3,0,0,1).")
      self.assertEqual(dsds[3,0,0,2],0.0,"ds/ds has wrong value at (3,0,0,2).")
      self.assertEqual(dsds[3,0,0,3],0.0,"ds/ds has wrong value at (3,0,0,3).")
      self.assertEqual(dsds[3,0,0,4],0.0,"ds/ds has wrong value at (3,0,0,4).")
      self.assertEqual(dsds[3,0,1,0],0.0,"ds/ds has wrong value at (3,0,1,0).")
      self.assertEqual(dsds[3,0,1,1],0.0,"ds/ds has wrong value at (3,0,1,1).")
      self.assertEqual(dsds[3,0,1,2],0.0,"ds/ds has wrong value at (3,0,1,2).")
      self.assertEqual(dsds[3,0,1,3],0.0,"ds/ds has wrong value at (3,0,1,3).")
      self.assertEqual(dsds[3,0,1,4],0.0,"ds/ds has wrong value at (3,0,1,4).")
      self.assertEqual(dsds[3,0,2,0],0.0,"ds/ds has wrong value at (3,0,2,0).")
      self.assertEqual(dsds[3,0,2,1],0.0,"ds/ds has wrong value at (3,0,2,1).")
      self.assertEqual(dsds[3,0,2,2],0.0,"ds/ds has wrong value at (3,0,2,2).")
      self.assertEqual(dsds[3,0,2,3],0.0,"ds/ds has wrong value at (3,0,2,3).")
      self.assertEqual(dsds[3,0,2,4],0.0,"ds/ds has wrong value at (3,0,2,4).")
      self.assertEqual(dsds[3,0,3,0],1.0,"ds/ds has wrong value at (3,0,3,0).")
      self.assertEqual(dsds[3,0,3,1],0.0,"ds/ds has wrong value at (3,0,3,1).")
      self.assertEqual(dsds[3,0,3,2],0.0,"ds/ds has wrong value at (3,0,3,2).")
      self.assertEqual(dsds[3,0,3,3],0.0,"ds/ds has wrong value at (3,0,3,3).")
      self.assertEqual(dsds[3,0,3,4],0.0,"ds/ds has wrong value at (3,0,3,4).")
      self.assertEqual(dsds[3,1,0,0],0.0,"ds/ds has wrong value at (3,1,0,0).")
      self.assertEqual(dsds[3,1,0,1],0.0,"ds/ds has wrong value at (3,1,0,1).")
      self.assertEqual(dsds[3,1,0,2],0.0,"ds/ds has wrong value at (3,1,0,2).")
      self.assertEqual(dsds[3,1,0,3],0.0,"ds/ds has wrong value at (3,1,0,3).")
      self.assertEqual(dsds[3,1,0,4],0.0,"ds/ds has wrong value at (3,1,0,4).")
      self.assertEqual(dsds[3,1,1,0],0.0,"ds/ds has wrong value at (3,1,1,0).")
      self.assertEqual(dsds[3,1,1,1],0.0,"ds/ds has wrong value at (3,1,1,1).")
      self.assertEqual(dsds[3,1,1,2],0.0,"ds/ds has wrong value at (3,1,1,2).")
      self.assertEqual(dsds[3,1,1,3],0.0,"ds/ds has wrong value at (3,1,1,3).")
      self.assertEqual(dsds[3,1,1,4],0.0,"ds/ds has wrong value at (3,1,1,4).")
      self.assertEqual(dsds[3,1,2,0],0.0,"ds/ds has wrong value at (3,1,2,0).")
      self.assertEqual(dsds[3,1,2,1],0.0,"ds/ds has wrong value at (3,1,2,1).")
      self.assertEqual(dsds[3,1,2,2],0.0,"ds/ds has wrong value at (3,1,2,2).")
      self.assertEqual(dsds[3,1,2,3],0.0,"ds/ds has wrong value at (3,1,2,3).")
      self.assertEqual(dsds[3,1,2,4],0.0,"ds/ds has wrong value at (3,1,2,4).")
      self.assertEqual(dsds[3,1,3,0],0.0,"ds/ds has wrong value at (3,1,3,0).")
      self.assertEqual(dsds[3,1,3,1],1.0,"ds/ds has wrong value at (3,1,3,1).")
      self.assertEqual(dsds[3,1,3,2],0.0,"ds/ds has wrong value at (3,1,3,2).")
      self.assertEqual(dsds[3,1,3,3],0.0,"ds/ds has wrong value at (3,1,3,3).")
      self.assertEqual(dsds[3,1,3,4],0.0,"ds/ds has wrong value at (3,1,3,4).")
      self.assertEqual(dsds[3,2,0,0],0.0,"ds/ds has wrong value at (3,2,0,0).")
      self.assertEqual(dsds[3,2,0,1],0.0,"ds/ds has wrong value at (3,2,0,1).")
      self.assertEqual(dsds[3,2,0,2],0.0,"ds/ds has wrong value at (3,2,0,2).")
      self.assertEqual(dsds[3,2,0,3],0.0,"ds/ds has wrong value at (3,2,0,3).")
      self.assertEqual(dsds[3,2,0,4],0.0,"ds/ds has wrong value at (3,2,0,4).")
      self.assertEqual(dsds[3,2,1,0],0.0,"ds/ds has wrong value at (3,2,1,0).")
      self.assertEqual(dsds[3,2,1,1],0.0,"ds/ds has wrong value at (3,2,1,1).")
      self.assertEqual(dsds[3,2,1,2],0.0,"ds/ds has wrong value at (3,2,1,2).")
      self.assertEqual(dsds[3,2,1,3],0.0,"ds/ds has wrong value at (3,2,1,3).")
      self.assertEqual(dsds[3,2,1,4],0.0,"ds/ds has wrong value at (3,2,1,4).")
      self.assertEqual(dsds[3,2,2,0],0.0,"ds/ds has wrong value at (3,2,2,0).")
      self.assertEqual(dsds[3,2,2,1],0.0,"ds/ds has wrong value at (3,2,2,1).")
      self.assertEqual(dsds[3,2,2,2],0.0,"ds/ds has wrong value at (3,2,2,2).")
      self.assertEqual(dsds[3,2,2,3],0.0,"ds/ds has wrong value at (3,2,2,3).")
      self.assertEqual(dsds[3,2,2,4],0.0,"ds/ds has wrong value at (3,2,2,4).")
      self.assertEqual(dsds[3,2,3,0],0.0,"ds/ds has wrong value at (3,2,3,0).")
      self.assertEqual(dsds[3,2,3,1],0.0,"ds/ds has wrong value at (3,2,3,1).")
      self.assertEqual(dsds[3,2,3,2],1.0,"ds/ds has wrong value at (3,2,3,2).")
      self.assertEqual(dsds[3,2,3,3],0.0,"ds/ds has wrong value at (3,2,3,3).")
      self.assertEqual(dsds[3,2,3,4],0.0,"ds/ds has wrong value at (3,2,3,4).")
      self.assertEqual(dsds[3,3,0,0],0.0,"ds/ds has wrong value at (3,3,0,0).")
      self.assertEqual(dsds[3,3,0,1],0.0,"ds/ds has wrong value at (3,3,0,1).")
      self.assertEqual(dsds[3,3,0,2],0.0,"ds/ds has wrong value at (3,3,0,2).")
      self.assertEqual(dsds[3,3,0,3],0.0,"ds/ds has wrong value at (3,3,0,3).")
      self.assertEqual(dsds[3,3,0,4],0.0,"ds/ds has wrong value at (3,3,0,4).")
      self.assertEqual(dsds[3,3,1,0],0.0,"ds/ds has wrong value at (3,3,1,0).")
      self.assertEqual(dsds[3,3,1,1],0.0,"ds/ds has wrong value at (3,3,1,1).")
      self.assertEqual(dsds[3,3,1,2],0.0,"ds/ds has wrong value at (3,3,1,2).")
      self.assertEqual(dsds[3,3,1,3],0.0,"ds/ds has wrong value at (3,3,1,3).")
      self.assertEqual(dsds[3,3,1,4],0.0,"ds/ds has wrong value at (3,3,1,4).")
      self.assertEqual(dsds[3,3,2,0],0.0,"ds/ds has wrong value at (3,3,2,0).")
      self.assertEqual(dsds[3,3,2,1],0.0,"ds/ds has wrong value at (3,3,2,1).")
      self.assertEqual(dsds[3,3,2,2],0.0,"ds/ds has wrong value at (3,3,2,2).")
      self.assertEqual(dsds[3,3,2,3],0.0,"ds/ds has wrong value at (3,3,2,3).")
      self.assertEqual(dsds[3,3,2,4],0.0,"ds/ds has wrong value at (3,3,2,4).")
      self.assertEqual(dsds[3,3,3,0],0.0,"ds/ds has wrong value at (3,3,3,0).")
      self.assertEqual(dsds[3,3,3,1],0.0,"ds/ds has wrong value at (3,3,3,1).")
      self.assertEqual(dsds[3,3,3,2],0.0,"ds/ds has wrong value at (3,3,3,2).")
      self.assertEqual(dsds[3,3,3,3],1.0,"ds/ds has wrong value at (3,3,3,3).")
      self.assertEqual(dsds[3,3,3,4],0.0,"ds/ds has wrong value at (3,3,3,4).")
      self.assertEqual(dsds[3,4,0,0],0.0,"ds/ds has wrong value at (3,4,0,0).")
      self.assertEqual(dsds[3,4,0,1],0.0,"ds/ds has wrong value at (3,4,0,1).")
      self.assertEqual(dsds[3,4,0,2],0.0,"ds/ds has wrong value at (3,4,0,2).")
      self.assertEqual(dsds[3,4,0,3],0.0,"ds/ds has wrong value at (3,4,0,3).")
      self.assertEqual(dsds[3,4,0,4],0.0,"ds/ds has wrong value at (3,4,0,4).")
      self.assertEqual(dsds[3,4,1,0],0.0,"ds/ds has wrong value at (3,4,1,0).")
      self.assertEqual(dsds[3,4,1,1],0.0,"ds/ds has wrong value at (3,4,1,1).")
      self.assertEqual(dsds[3,4,1,2],0.0,"ds/ds has wrong value at (3,4,1,2).")
      self.assertEqual(dsds[3,4,1,3],0.0,"ds/ds has wrong value at (3,4,1,3).")
      self.assertEqual(dsds[3,4,1,4],0.0,"ds/ds has wrong value at (3,4,1,4).")
      self.assertEqual(dsds[3,4,2,0],0.0,"ds/ds has wrong value at (3,4,2,0).")
      self.assertEqual(dsds[3,4,2,1],0.0,"ds/ds has wrong value at (3,4,2,1).")
      self.assertEqual(dsds[3,4,2,2],0.0,"ds/ds has wrong value at (3,4,2,2).")
      self.assertEqual(dsds[3,4,2,3],0.0,"ds/ds has wrong value at (3,4,2,3).")
      self.assertEqual(dsds[3,4,2,4],0.0,"ds/ds has wrong value at (3,4,2,4).")
      self.assertEqual(dsds[3,4,3,0],0.0,"ds/ds has wrong value at (3,4,3,0).")
      self.assertEqual(dsds[3,4,3,1],0.0,"ds/ds has wrong value at (3,4,3,1).")
      self.assertEqual(dsds[3,4,3,2],0.0,"ds/ds has wrong value at (3,4,3,2).")
      self.assertEqual(dsds[3,4,3,3],0.0,"ds/ds has wrong value at (3,4,3,3).")
      self.assertEqual(dsds[3,4,3,4],1.0,"ds/ds has wrong value at (3,4,3,4).")

      sa=s.getSubstitutedArguments({s2:-10})
      self.assertEqual(len(sa),0,"wrong number of substituted arguments")

      self.assertFalse(s.isAppropriateValue(10.)," 10. is not appropriate substitute")
      self.assertFalse(s.isAppropriateValue(10)," 10 is not appropriate substitute")
      self.assertTrue(s.isAppropriateValue(Symbol(shape=(4, 5),dim=2)),"Symbol is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(Symbol(shape=(4, 5),dim=3)),"Symbol is not appropriate substitute (dim)")
      self.assertFalse(s.isAppropriateValue(escript.Data(0.,(5,),self.functionspace)),"Data is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(numpy.zeros((4, 5))),"ndarray is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(numpy.zeros((5,))),"ndarray is not appropriate substitute (shape)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_Symbol_rank2_d2_nargs1(self):
      s2=Symbol(shape=())
      s=Symbol(shape=(4, 5),dim=2,args=[s2])
      self.assertEqual(s.getRank(),2,"wrong rank.")
      self.assertEqual(s.getShape(),(4, 5),"wrong shape.")
      self.assertEqual(s.getDim(),2,"wrong spatial dimension.")
      self.assertEqual(s.getArgument(),[s2],"wrong arguments.")

      ss=s.substitute({s:numpy.zeros((4, 5))})
      self.assertTrue(isinstance(ss,numpy.ndarray),"value after substitution is not ndarray.")
      self.assertEqual(ss.shape,(4, 5),"value after substitution has not expected shape")
      try:
        s.substitute({s:numpy.zeros((5,))})
        fail("illegal substition was successful")
      except TypeError:
        pass

      dsdarg=s.diff(Symbol(shape=()))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds() has wrong type.")
      self.assertEqual(dsdarg.shape,(4, 5),"ds/ds() has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds() has wrong value.")
      dsdarg=s.diff(Symbol(shape=(2,)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(2,) has wrong type.")
      self.assertEqual(dsdarg.shape,(4, 5, 2),"ds/ds(2,) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(2,) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(4, 5)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(4, 5) has wrong type.")
      self.assertEqual(dsdarg.shape,(4, 5, 4, 5),"ds/ds(4, 5) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(4, 5) has wrong value.")

      dsds=s.diff(s)
      self.assertTrue(isinstance(dsds,numpy.ndarray),"ds/ds has wrong type.")
      self.assertEqual(dsds.shape,(4, 5, 4, 5),"ds/ds has wrong shape.")
      self.assertEqual(dsds[0,0,0,0],1.0,"ds/ds has wrong value at (0,0,0,0).")
      self.assertEqual(dsds[0,0,0,1],0.0,"ds/ds has wrong value at (0,0,0,1).")
      self.assertEqual(dsds[0,0,0,2],0.0,"ds/ds has wrong value at (0,0,0,2).")
      self.assertEqual(dsds[0,0,0,3],0.0,"ds/ds has wrong value at (0,0,0,3).")
      self.assertEqual(dsds[0,0,0,4],0.0,"ds/ds has wrong value at (0,0,0,4).")
      self.assertEqual(dsds[0,0,1,0],0.0,"ds/ds has wrong value at (0,0,1,0).")
      self.assertEqual(dsds[0,0,1,1],0.0,"ds/ds has wrong value at (0,0,1,1).")
      self.assertEqual(dsds[0,0,1,2],0.0,"ds/ds has wrong value at (0,0,1,2).")
      self.assertEqual(dsds[0,0,1,3],0.0,"ds/ds has wrong value at (0,0,1,3).")
      self.assertEqual(dsds[0,0,1,4],0.0,"ds/ds has wrong value at (0,0,1,4).")
      self.assertEqual(dsds[0,0,2,0],0.0,"ds/ds has wrong value at (0,0,2,0).")
      self.assertEqual(dsds[0,0,2,1],0.0,"ds/ds has wrong value at (0,0,2,1).")
      self.assertEqual(dsds[0,0,2,2],0.0,"ds/ds has wrong value at (0,0,2,2).")
      self.assertEqual(dsds[0,0,2,3],0.0,"ds/ds has wrong value at (0,0,2,3).")
      self.assertEqual(dsds[0,0,2,4],0.0,"ds/ds has wrong value at (0,0,2,4).")
      self.assertEqual(dsds[0,0,3,0],0.0,"ds/ds has wrong value at (0,0,3,0).")
      self.assertEqual(dsds[0,0,3,1],0.0,"ds/ds has wrong value at (0,0,3,1).")
      self.assertEqual(dsds[0,0,3,2],0.0,"ds/ds has wrong value at (0,0,3,2).")
      self.assertEqual(dsds[0,0,3,3],0.0,"ds/ds has wrong value at (0,0,3,3).")
      self.assertEqual(dsds[0,0,3,4],0.0,"ds/ds has wrong value at (0,0,3,4).")
      self.assertEqual(dsds[0,1,0,0],0.0,"ds/ds has wrong value at (0,1,0,0).")
      self.assertEqual(dsds[0,1,0,1],1.0,"ds/ds has wrong value at (0,1,0,1).")
      self.assertEqual(dsds[0,1,0,2],0.0,"ds/ds has wrong value at (0,1,0,2).")
      self.assertEqual(dsds[0,1,0,3],0.0,"ds/ds has wrong value at (0,1,0,3).")
      self.assertEqual(dsds[0,1,0,4],0.0,"ds/ds has wrong value at (0,1,0,4).")
      self.assertEqual(dsds[0,1,1,0],0.0,"ds/ds has wrong value at (0,1,1,0).")
      self.assertEqual(dsds[0,1,1,1],0.0,"ds/ds has wrong value at (0,1,1,1).")
      self.assertEqual(dsds[0,1,1,2],0.0,"ds/ds has wrong value at (0,1,1,2).")
      self.assertEqual(dsds[0,1,1,3],0.0,"ds/ds has wrong value at (0,1,1,3).")
      self.assertEqual(dsds[0,1,1,4],0.0,"ds/ds has wrong value at (0,1,1,4).")
      self.assertEqual(dsds[0,1,2,0],0.0,"ds/ds has wrong value at (0,1,2,0).")
      self.assertEqual(dsds[0,1,2,1],0.0,"ds/ds has wrong value at (0,1,2,1).")
      self.assertEqual(dsds[0,1,2,2],0.0,"ds/ds has wrong value at (0,1,2,2).")
      self.assertEqual(dsds[0,1,2,3],0.0,"ds/ds has wrong value at (0,1,2,3).")
      self.assertEqual(dsds[0,1,2,4],0.0,"ds/ds has wrong value at (0,1,2,4).")
      self.assertEqual(dsds[0,1,3,0],0.0,"ds/ds has wrong value at (0,1,3,0).")
      self.assertEqual(dsds[0,1,3,1],0.0,"ds/ds has wrong value at (0,1,3,1).")
      self.assertEqual(dsds[0,1,3,2],0.0,"ds/ds has wrong value at (0,1,3,2).")
      self.assertEqual(dsds[0,1,3,3],0.0,"ds/ds has wrong value at (0,1,3,3).")
      self.assertEqual(dsds[0,1,3,4],0.0,"ds/ds has wrong value at (0,1,3,4).")
      self.assertEqual(dsds[0,2,0,0],0.0,"ds/ds has wrong value at (0,2,0,0).")
      self.assertEqual(dsds[0,2,0,1],0.0,"ds/ds has wrong value at (0,2,0,1).")
      self.assertEqual(dsds[0,2,0,2],1.0,"ds/ds has wrong value at (0,2,0,2).")
      self.assertEqual(dsds[0,2,0,3],0.0,"ds/ds has wrong value at (0,2,0,3).")
      self.assertEqual(dsds[0,2,0,4],0.0,"ds/ds has wrong value at (0,2,0,4).")
      self.assertEqual(dsds[0,2,1,0],0.0,"ds/ds has wrong value at (0,2,1,0).")
      self.assertEqual(dsds[0,2,1,1],0.0,"ds/ds has wrong value at (0,2,1,1).")
      self.assertEqual(dsds[0,2,1,2],0.0,"ds/ds has wrong value at (0,2,1,2).")
      self.assertEqual(dsds[0,2,1,3],0.0,"ds/ds has wrong value at (0,2,1,3).")
      self.assertEqual(dsds[0,2,1,4],0.0,"ds/ds has wrong value at (0,2,1,4).")
      self.assertEqual(dsds[0,2,2,0],0.0,"ds/ds has wrong value at (0,2,2,0).")
      self.assertEqual(dsds[0,2,2,1],0.0,"ds/ds has wrong value at (0,2,2,1).")
      self.assertEqual(dsds[0,2,2,2],0.0,"ds/ds has wrong value at (0,2,2,2).")
      self.assertEqual(dsds[0,2,2,3],0.0,"ds/ds has wrong value at (0,2,2,3).")
      self.assertEqual(dsds[0,2,2,4],0.0,"ds/ds has wrong value at (0,2,2,4).")
      self.assertEqual(dsds[0,2,3,0],0.0,"ds/ds has wrong value at (0,2,3,0).")
      self.assertEqual(dsds[0,2,3,1],0.0,"ds/ds has wrong value at (0,2,3,1).")
      self.assertEqual(dsds[0,2,3,2],0.0,"ds/ds has wrong value at (0,2,3,2).")
      self.assertEqual(dsds[0,2,3,3],0.0,"ds/ds has wrong value at (0,2,3,3).")
      self.assertEqual(dsds[0,2,3,4],0.0,"ds/ds has wrong value at (0,2,3,4).")
      self.assertEqual(dsds[0,3,0,0],0.0,"ds/ds has wrong value at (0,3,0,0).")
      self.assertEqual(dsds[0,3,0,1],0.0,"ds/ds has wrong value at (0,3,0,1).")
      self.assertEqual(dsds[0,3,0,2],0.0,"ds/ds has wrong value at (0,3,0,2).")
      self.assertEqual(dsds[0,3,0,3],1.0,"ds/ds has wrong value at (0,3,0,3).")
      self.assertEqual(dsds[0,3,0,4],0.0,"ds/ds has wrong value at (0,3,0,4).")
      self.assertEqual(dsds[0,3,1,0],0.0,"ds/ds has wrong value at (0,3,1,0).")
      self.assertEqual(dsds[0,3,1,1],0.0,"ds/ds has wrong value at (0,3,1,1).")
      self.assertEqual(dsds[0,3,1,2],0.0,"ds/ds has wrong value at (0,3,1,2).")
      self.assertEqual(dsds[0,3,1,3],0.0,"ds/ds has wrong value at (0,3,1,3).")
      self.assertEqual(dsds[0,3,1,4],0.0,"ds/ds has wrong value at (0,3,1,4).")
      self.assertEqual(dsds[0,3,2,0],0.0,"ds/ds has wrong value at (0,3,2,0).")
      self.assertEqual(dsds[0,3,2,1],0.0,"ds/ds has wrong value at (0,3,2,1).")
      self.assertEqual(dsds[0,3,2,2],0.0,"ds/ds has wrong value at (0,3,2,2).")
      self.assertEqual(dsds[0,3,2,3],0.0,"ds/ds has wrong value at (0,3,2,3).")
      self.assertEqual(dsds[0,3,2,4],0.0,"ds/ds has wrong value at (0,3,2,4).")
      self.assertEqual(dsds[0,3,3,0],0.0,"ds/ds has wrong value at (0,3,3,0).")
      self.assertEqual(dsds[0,3,3,1],0.0,"ds/ds has wrong value at (0,3,3,1).")
      self.assertEqual(dsds[0,3,3,2],0.0,"ds/ds has wrong value at (0,3,3,2).")
      self.assertEqual(dsds[0,3,3,3],0.0,"ds/ds has wrong value at (0,3,3,3).")
      self.assertEqual(dsds[0,3,3,4],0.0,"ds/ds has wrong value at (0,3,3,4).")
      self.assertEqual(dsds[0,4,0,0],0.0,"ds/ds has wrong value at (0,4,0,0).")
      self.assertEqual(dsds[0,4,0,1],0.0,"ds/ds has wrong value at (0,4,0,1).")
      self.assertEqual(dsds[0,4,0,2],0.0,"ds/ds has wrong value at (0,4,0,2).")
      self.assertEqual(dsds[0,4,0,3],0.0,"ds/ds has wrong value at (0,4,0,3).")
      self.assertEqual(dsds[0,4,0,4],1.0,"ds/ds has wrong value at (0,4,0,4).")
      self.assertEqual(dsds[0,4,1,0],0.0,"ds/ds has wrong value at (0,4,1,0).")
      self.assertEqual(dsds[0,4,1,1],0.0,"ds/ds has wrong value at (0,4,1,1).")
      self.assertEqual(dsds[0,4,1,2],0.0,"ds/ds has wrong value at (0,4,1,2).")
      self.assertEqual(dsds[0,4,1,3],0.0,"ds/ds has wrong value at (0,4,1,3).")
      self.assertEqual(dsds[0,4,1,4],0.0,"ds/ds has wrong value at (0,4,1,4).")
      self.assertEqual(dsds[0,4,2,0],0.0,"ds/ds has wrong value at (0,4,2,0).")
      self.assertEqual(dsds[0,4,2,1],0.0,"ds/ds has wrong value at (0,4,2,1).")
      self.assertEqual(dsds[0,4,2,2],0.0,"ds/ds has wrong value at (0,4,2,2).")
      self.assertEqual(dsds[0,4,2,3],0.0,"ds/ds has wrong value at (0,4,2,3).")
      self.assertEqual(dsds[0,4,2,4],0.0,"ds/ds has wrong value at (0,4,2,4).")
      self.assertEqual(dsds[0,4,3,0],0.0,"ds/ds has wrong value at (0,4,3,0).")
      self.assertEqual(dsds[0,4,3,1],0.0,"ds/ds has wrong value at (0,4,3,1).")
      self.assertEqual(dsds[0,4,3,2],0.0,"ds/ds has wrong value at (0,4,3,2).")
      self.assertEqual(dsds[0,4,3,3],0.0,"ds/ds has wrong value at (0,4,3,3).")
      self.assertEqual(dsds[0,4,3,4],0.0,"ds/ds has wrong value at (0,4,3,4).")
      self.assertEqual(dsds[1,0,0,0],0.0,"ds/ds has wrong value at (1,0,0,0).")
      self.assertEqual(dsds[1,0,0,1],0.0,"ds/ds has wrong value at (1,0,0,1).")
      self.assertEqual(dsds[1,0,0,2],0.0,"ds/ds has wrong value at (1,0,0,2).")
      self.assertEqual(dsds[1,0,0,3],0.0,"ds/ds has wrong value at (1,0,0,3).")
      self.assertEqual(dsds[1,0,0,4],0.0,"ds/ds has wrong value at (1,0,0,4).")
      self.assertEqual(dsds[1,0,1,0],1.0,"ds/ds has wrong value at (1,0,1,0).")
      self.assertEqual(dsds[1,0,1,1],0.0,"ds/ds has wrong value at (1,0,1,1).")
      self.assertEqual(dsds[1,0,1,2],0.0,"ds/ds has wrong value at (1,0,1,2).")
      self.assertEqual(dsds[1,0,1,3],0.0,"ds/ds has wrong value at (1,0,1,3).")
      self.assertEqual(dsds[1,0,1,4],0.0,"ds/ds has wrong value at (1,0,1,4).")
      self.assertEqual(dsds[1,0,2,0],0.0,"ds/ds has wrong value at (1,0,2,0).")
      self.assertEqual(dsds[1,0,2,1],0.0,"ds/ds has wrong value at (1,0,2,1).")
      self.assertEqual(dsds[1,0,2,2],0.0,"ds/ds has wrong value at (1,0,2,2).")
      self.assertEqual(dsds[1,0,2,3],0.0,"ds/ds has wrong value at (1,0,2,3).")
      self.assertEqual(dsds[1,0,2,4],0.0,"ds/ds has wrong value at (1,0,2,4).")
      self.assertEqual(dsds[1,0,3,0],0.0,"ds/ds has wrong value at (1,0,3,0).")
      self.assertEqual(dsds[1,0,3,1],0.0,"ds/ds has wrong value at (1,0,3,1).")
      self.assertEqual(dsds[1,0,3,2],0.0,"ds/ds has wrong value at (1,0,3,2).")
      self.assertEqual(dsds[1,0,3,3],0.0,"ds/ds has wrong value at (1,0,3,3).")
      self.assertEqual(dsds[1,0,3,4],0.0,"ds/ds has wrong value at (1,0,3,4).")
      self.assertEqual(dsds[1,1,0,0],0.0,"ds/ds has wrong value at (1,1,0,0).")
      self.assertEqual(dsds[1,1,0,1],0.0,"ds/ds has wrong value at (1,1,0,1).")
      self.assertEqual(dsds[1,1,0,2],0.0,"ds/ds has wrong value at (1,1,0,2).")
      self.assertEqual(dsds[1,1,0,3],0.0,"ds/ds has wrong value at (1,1,0,3).")
      self.assertEqual(dsds[1,1,0,4],0.0,"ds/ds has wrong value at (1,1,0,4).")
      self.assertEqual(dsds[1,1,1,0],0.0,"ds/ds has wrong value at (1,1,1,0).")
      self.assertEqual(dsds[1,1,1,1],1.0,"ds/ds has wrong value at (1,1,1,1).")
      self.assertEqual(dsds[1,1,1,2],0.0,"ds/ds has wrong value at (1,1,1,2).")
      self.assertEqual(dsds[1,1,1,3],0.0,"ds/ds has wrong value at (1,1,1,3).")
      self.assertEqual(dsds[1,1,1,4],0.0,"ds/ds has wrong value at (1,1,1,4).")
      self.assertEqual(dsds[1,1,2,0],0.0,"ds/ds has wrong value at (1,1,2,0).")
      self.assertEqual(dsds[1,1,2,1],0.0,"ds/ds has wrong value at (1,1,2,1).")
      self.assertEqual(dsds[1,1,2,2],0.0,"ds/ds has wrong value at (1,1,2,2).")
      self.assertEqual(dsds[1,1,2,3],0.0,"ds/ds has wrong value at (1,1,2,3).")
      self.assertEqual(dsds[1,1,2,4],0.0,"ds/ds has wrong value at (1,1,2,4).")
      self.assertEqual(dsds[1,1,3,0],0.0,"ds/ds has wrong value at (1,1,3,0).")
      self.assertEqual(dsds[1,1,3,1],0.0,"ds/ds has wrong value at (1,1,3,1).")
      self.assertEqual(dsds[1,1,3,2],0.0,"ds/ds has wrong value at (1,1,3,2).")
      self.assertEqual(dsds[1,1,3,3],0.0,"ds/ds has wrong value at (1,1,3,3).")
      self.assertEqual(dsds[1,1,3,4],0.0,"ds/ds has wrong value at (1,1,3,4).")
      self.assertEqual(dsds[1,2,0,0],0.0,"ds/ds has wrong value at (1,2,0,0).")
      self.assertEqual(dsds[1,2,0,1],0.0,"ds/ds has wrong value at (1,2,0,1).")
      self.assertEqual(dsds[1,2,0,2],0.0,"ds/ds has wrong value at (1,2,0,2).")
      self.assertEqual(dsds[1,2,0,3],0.0,"ds/ds has wrong value at (1,2,0,3).")
      self.assertEqual(dsds[1,2,0,4],0.0,"ds/ds has wrong value at (1,2,0,4).")
      self.assertEqual(dsds[1,2,1,0],0.0,"ds/ds has wrong value at (1,2,1,0).")
      self.assertEqual(dsds[1,2,1,1],0.0,"ds/ds has wrong value at (1,2,1,1).")
      self.assertEqual(dsds[1,2,1,2],1.0,"ds/ds has wrong value at (1,2,1,2).")
      self.assertEqual(dsds[1,2,1,3],0.0,"ds/ds has wrong value at (1,2,1,3).")
      self.assertEqual(dsds[1,2,1,4],0.0,"ds/ds has wrong value at (1,2,1,4).")
      self.assertEqual(dsds[1,2,2,0],0.0,"ds/ds has wrong value at (1,2,2,0).")
      self.assertEqual(dsds[1,2,2,1],0.0,"ds/ds has wrong value at (1,2,2,1).")
      self.assertEqual(dsds[1,2,2,2],0.0,"ds/ds has wrong value at (1,2,2,2).")
      self.assertEqual(dsds[1,2,2,3],0.0,"ds/ds has wrong value at (1,2,2,3).")
      self.assertEqual(dsds[1,2,2,4],0.0,"ds/ds has wrong value at (1,2,2,4).")
      self.assertEqual(dsds[1,2,3,0],0.0,"ds/ds has wrong value at (1,2,3,0).")
      self.assertEqual(dsds[1,2,3,1],0.0,"ds/ds has wrong value at (1,2,3,1).")
      self.assertEqual(dsds[1,2,3,2],0.0,"ds/ds has wrong value at (1,2,3,2).")
      self.assertEqual(dsds[1,2,3,3],0.0,"ds/ds has wrong value at (1,2,3,3).")
      self.assertEqual(dsds[1,2,3,4],0.0,"ds/ds has wrong value at (1,2,3,4).")
      self.assertEqual(dsds[1,3,0,0],0.0,"ds/ds has wrong value at (1,3,0,0).")
      self.assertEqual(dsds[1,3,0,1],0.0,"ds/ds has wrong value at (1,3,0,1).")
      self.assertEqual(dsds[1,3,0,2],0.0,"ds/ds has wrong value at (1,3,0,2).")
      self.assertEqual(dsds[1,3,0,3],0.0,"ds/ds has wrong value at (1,3,0,3).")
      self.assertEqual(dsds[1,3,0,4],0.0,"ds/ds has wrong value at (1,3,0,4).")
      self.assertEqual(dsds[1,3,1,0],0.0,"ds/ds has wrong value at (1,3,1,0).")
      self.assertEqual(dsds[1,3,1,1],0.0,"ds/ds has wrong value at (1,3,1,1).")
      self.assertEqual(dsds[1,3,1,2],0.0,"ds/ds has wrong value at (1,3,1,2).")
      self.assertEqual(dsds[1,3,1,3],1.0,"ds/ds has wrong value at (1,3,1,3).")
      self.assertEqual(dsds[1,3,1,4],0.0,"ds/ds has wrong value at (1,3,1,4).")
      self.assertEqual(dsds[1,3,2,0],0.0,"ds/ds has wrong value at (1,3,2,0).")
      self.assertEqual(dsds[1,3,2,1],0.0,"ds/ds has wrong value at (1,3,2,1).")
      self.assertEqual(dsds[1,3,2,2],0.0,"ds/ds has wrong value at (1,3,2,2).")
      self.assertEqual(dsds[1,3,2,3],0.0,"ds/ds has wrong value at (1,3,2,3).")
      self.assertEqual(dsds[1,3,2,4],0.0,"ds/ds has wrong value at (1,3,2,4).")
      self.assertEqual(dsds[1,3,3,0],0.0,"ds/ds has wrong value at (1,3,3,0).")
      self.assertEqual(dsds[1,3,3,1],0.0,"ds/ds has wrong value at (1,3,3,1).")
      self.assertEqual(dsds[1,3,3,2],0.0,"ds/ds has wrong value at (1,3,3,2).")
      self.assertEqual(dsds[1,3,3,3],0.0,"ds/ds has wrong value at (1,3,3,3).")
      self.assertEqual(dsds[1,3,3,4],0.0,"ds/ds has wrong value at (1,3,3,4).")
      self.assertEqual(dsds[1,4,0,0],0.0,"ds/ds has wrong value at (1,4,0,0).")
      self.assertEqual(dsds[1,4,0,1],0.0,"ds/ds has wrong value at (1,4,0,1).")
      self.assertEqual(dsds[1,4,0,2],0.0,"ds/ds has wrong value at (1,4,0,2).")
      self.assertEqual(dsds[1,4,0,3],0.0,"ds/ds has wrong value at (1,4,0,3).")
      self.assertEqual(dsds[1,4,0,4],0.0,"ds/ds has wrong value at (1,4,0,4).")
      self.assertEqual(dsds[1,4,1,0],0.0,"ds/ds has wrong value at (1,4,1,0).")
      self.assertEqual(dsds[1,4,1,1],0.0,"ds/ds has wrong value at (1,4,1,1).")
      self.assertEqual(dsds[1,4,1,2],0.0,"ds/ds has wrong value at (1,4,1,2).")
      self.assertEqual(dsds[1,4,1,3],0.0,"ds/ds has wrong value at (1,4,1,3).")
      self.assertEqual(dsds[1,4,1,4],1.0,"ds/ds has wrong value at (1,4,1,4).")
      self.assertEqual(dsds[1,4,2,0],0.0,"ds/ds has wrong value at (1,4,2,0).")
      self.assertEqual(dsds[1,4,2,1],0.0,"ds/ds has wrong value at (1,4,2,1).")
      self.assertEqual(dsds[1,4,2,2],0.0,"ds/ds has wrong value at (1,4,2,2).")
      self.assertEqual(dsds[1,4,2,3],0.0,"ds/ds has wrong value at (1,4,2,3).")
      self.assertEqual(dsds[1,4,2,4],0.0,"ds/ds has wrong value at (1,4,2,4).")
      self.assertEqual(dsds[1,4,3,0],0.0,"ds/ds has wrong value at (1,4,3,0).")
      self.assertEqual(dsds[1,4,3,1],0.0,"ds/ds has wrong value at (1,4,3,1).")
      self.assertEqual(dsds[1,4,3,2],0.0,"ds/ds has wrong value at (1,4,3,2).")
      self.assertEqual(dsds[1,4,3,3],0.0,"ds/ds has wrong value at (1,4,3,3).")
      self.assertEqual(dsds[1,4,3,4],0.0,"ds/ds has wrong value at (1,4,3,4).")
      self.assertEqual(dsds[2,0,0,0],0.0,"ds/ds has wrong value at (2,0,0,0).")
      self.assertEqual(dsds[2,0,0,1],0.0,"ds/ds has wrong value at (2,0,0,1).")
      self.assertEqual(dsds[2,0,0,2],0.0,"ds/ds has wrong value at (2,0,0,2).")
      self.assertEqual(dsds[2,0,0,3],0.0,"ds/ds has wrong value at (2,0,0,3).")
      self.assertEqual(dsds[2,0,0,4],0.0,"ds/ds has wrong value at (2,0,0,4).")
      self.assertEqual(dsds[2,0,1,0],0.0,"ds/ds has wrong value at (2,0,1,0).")
      self.assertEqual(dsds[2,0,1,1],0.0,"ds/ds has wrong value at (2,0,1,1).")
      self.assertEqual(dsds[2,0,1,2],0.0,"ds/ds has wrong value at (2,0,1,2).")
      self.assertEqual(dsds[2,0,1,3],0.0,"ds/ds has wrong value at (2,0,1,3).")
      self.assertEqual(dsds[2,0,1,4],0.0,"ds/ds has wrong value at (2,0,1,4).")
      self.assertEqual(dsds[2,0,2,0],1.0,"ds/ds has wrong value at (2,0,2,0).")
      self.assertEqual(dsds[2,0,2,1],0.0,"ds/ds has wrong value at (2,0,2,1).")
      self.assertEqual(dsds[2,0,2,2],0.0,"ds/ds has wrong value at (2,0,2,2).")
      self.assertEqual(dsds[2,0,2,3],0.0,"ds/ds has wrong value at (2,0,2,3).")
      self.assertEqual(dsds[2,0,2,4],0.0,"ds/ds has wrong value at (2,0,2,4).")
      self.assertEqual(dsds[2,0,3,0],0.0,"ds/ds has wrong value at (2,0,3,0).")
      self.assertEqual(dsds[2,0,3,1],0.0,"ds/ds has wrong value at (2,0,3,1).")
      self.assertEqual(dsds[2,0,3,2],0.0,"ds/ds has wrong value at (2,0,3,2).")
      self.assertEqual(dsds[2,0,3,3],0.0,"ds/ds has wrong value at (2,0,3,3).")
      self.assertEqual(dsds[2,0,3,4],0.0,"ds/ds has wrong value at (2,0,3,4).")
      self.assertEqual(dsds[2,1,0,0],0.0,"ds/ds has wrong value at (2,1,0,0).")
      self.assertEqual(dsds[2,1,0,1],0.0,"ds/ds has wrong value at (2,1,0,1).")
      self.assertEqual(dsds[2,1,0,2],0.0,"ds/ds has wrong value at (2,1,0,2).")
      self.assertEqual(dsds[2,1,0,3],0.0,"ds/ds has wrong value at (2,1,0,3).")
      self.assertEqual(dsds[2,1,0,4],0.0,"ds/ds has wrong value at (2,1,0,4).")
      self.assertEqual(dsds[2,1,1,0],0.0,"ds/ds has wrong value at (2,1,1,0).")
      self.assertEqual(dsds[2,1,1,1],0.0,"ds/ds has wrong value at (2,1,1,1).")
      self.assertEqual(dsds[2,1,1,2],0.0,"ds/ds has wrong value at (2,1,1,2).")
      self.assertEqual(dsds[2,1,1,3],0.0,"ds/ds has wrong value at (2,1,1,3).")
      self.assertEqual(dsds[2,1,1,4],0.0,"ds/ds has wrong value at (2,1,1,4).")
      self.assertEqual(dsds[2,1,2,0],0.0,"ds/ds has wrong value at (2,1,2,0).")
      self.assertEqual(dsds[2,1,2,1],1.0,"ds/ds has wrong value at (2,1,2,1).")
      self.assertEqual(dsds[2,1,2,2],0.0,"ds/ds has wrong value at (2,1,2,2).")
      self.assertEqual(dsds[2,1,2,3],0.0,"ds/ds has wrong value at (2,1,2,3).")
      self.assertEqual(dsds[2,1,2,4],0.0,"ds/ds has wrong value at (2,1,2,4).")
      self.assertEqual(dsds[2,1,3,0],0.0,"ds/ds has wrong value at (2,1,3,0).")
      self.assertEqual(dsds[2,1,3,1],0.0,"ds/ds has wrong value at (2,1,3,1).")
      self.assertEqual(dsds[2,1,3,2],0.0,"ds/ds has wrong value at (2,1,3,2).")
      self.assertEqual(dsds[2,1,3,3],0.0,"ds/ds has wrong value at (2,1,3,3).")
      self.assertEqual(dsds[2,1,3,4],0.0,"ds/ds has wrong value at (2,1,3,4).")
      self.assertEqual(dsds[2,2,0,0],0.0,"ds/ds has wrong value at (2,2,0,0).")
      self.assertEqual(dsds[2,2,0,1],0.0,"ds/ds has wrong value at (2,2,0,1).")
      self.assertEqual(dsds[2,2,0,2],0.0,"ds/ds has wrong value at (2,2,0,2).")
      self.assertEqual(dsds[2,2,0,3],0.0,"ds/ds has wrong value at (2,2,0,3).")
      self.assertEqual(dsds[2,2,0,4],0.0,"ds/ds has wrong value at (2,2,0,4).")
      self.assertEqual(dsds[2,2,1,0],0.0,"ds/ds has wrong value at (2,2,1,0).")
      self.assertEqual(dsds[2,2,1,1],0.0,"ds/ds has wrong value at (2,2,1,1).")
      self.assertEqual(dsds[2,2,1,2],0.0,"ds/ds has wrong value at (2,2,1,2).")
      self.assertEqual(dsds[2,2,1,3],0.0,"ds/ds has wrong value at (2,2,1,3).")
      self.assertEqual(dsds[2,2,1,4],0.0,"ds/ds has wrong value at (2,2,1,4).")
      self.assertEqual(dsds[2,2,2,0],0.0,"ds/ds has wrong value at (2,2,2,0).")
      self.assertEqual(dsds[2,2,2,1],0.0,"ds/ds has wrong value at (2,2,2,1).")
      self.assertEqual(dsds[2,2,2,2],1.0,"ds/ds has wrong value at (2,2,2,2).")
      self.assertEqual(dsds[2,2,2,3],0.0,"ds/ds has wrong value at (2,2,2,3).")
      self.assertEqual(dsds[2,2,2,4],0.0,"ds/ds has wrong value at (2,2,2,4).")
      self.assertEqual(dsds[2,2,3,0],0.0,"ds/ds has wrong value at (2,2,3,0).")
      self.assertEqual(dsds[2,2,3,1],0.0,"ds/ds has wrong value at (2,2,3,1).")
      self.assertEqual(dsds[2,2,3,2],0.0,"ds/ds has wrong value at (2,2,3,2).")
      self.assertEqual(dsds[2,2,3,3],0.0,"ds/ds has wrong value at (2,2,3,3).")
      self.assertEqual(dsds[2,2,3,4],0.0,"ds/ds has wrong value at (2,2,3,4).")
      self.assertEqual(dsds[2,3,0,0],0.0,"ds/ds has wrong value at (2,3,0,0).")
      self.assertEqual(dsds[2,3,0,1],0.0,"ds/ds has wrong value at (2,3,0,1).")
      self.assertEqual(dsds[2,3,0,2],0.0,"ds/ds has wrong value at (2,3,0,2).")
      self.assertEqual(dsds[2,3,0,3],0.0,"ds/ds has wrong value at (2,3,0,3).")
      self.assertEqual(dsds[2,3,0,4],0.0,"ds/ds has wrong value at (2,3,0,4).")
      self.assertEqual(dsds[2,3,1,0],0.0,"ds/ds has wrong value at (2,3,1,0).")
      self.assertEqual(dsds[2,3,1,1],0.0,"ds/ds has wrong value at (2,3,1,1).")
      self.assertEqual(dsds[2,3,1,2],0.0,"ds/ds has wrong value at (2,3,1,2).")
      self.assertEqual(dsds[2,3,1,3],0.0,"ds/ds has wrong value at (2,3,1,3).")
      self.assertEqual(dsds[2,3,1,4],0.0,"ds/ds has wrong value at (2,3,1,4).")
      self.assertEqual(dsds[2,3,2,0],0.0,"ds/ds has wrong value at (2,3,2,0).")
      self.assertEqual(dsds[2,3,2,1],0.0,"ds/ds has wrong value at (2,3,2,1).")
      self.assertEqual(dsds[2,3,2,2],0.0,"ds/ds has wrong value at (2,3,2,2).")
      self.assertEqual(dsds[2,3,2,3],1.0,"ds/ds has wrong value at (2,3,2,3).")
      self.assertEqual(dsds[2,3,2,4],0.0,"ds/ds has wrong value at (2,3,2,4).")
      self.assertEqual(dsds[2,3,3,0],0.0,"ds/ds has wrong value at (2,3,3,0).")
      self.assertEqual(dsds[2,3,3,1],0.0,"ds/ds has wrong value at (2,3,3,1).")
      self.assertEqual(dsds[2,3,3,2],0.0,"ds/ds has wrong value at (2,3,3,2).")
      self.assertEqual(dsds[2,3,3,3],0.0,"ds/ds has wrong value at (2,3,3,3).")
      self.assertEqual(dsds[2,3,3,4],0.0,"ds/ds has wrong value at (2,3,3,4).")
      self.assertEqual(dsds[2,4,0,0],0.0,"ds/ds has wrong value at (2,4,0,0).")
      self.assertEqual(dsds[2,4,0,1],0.0,"ds/ds has wrong value at (2,4,0,1).")
      self.assertEqual(dsds[2,4,0,2],0.0,"ds/ds has wrong value at (2,4,0,2).")
      self.assertEqual(dsds[2,4,0,3],0.0,"ds/ds has wrong value at (2,4,0,3).")
      self.assertEqual(dsds[2,4,0,4],0.0,"ds/ds has wrong value at (2,4,0,4).")
      self.assertEqual(dsds[2,4,1,0],0.0,"ds/ds has wrong value at (2,4,1,0).")
      self.assertEqual(dsds[2,4,1,1],0.0,"ds/ds has wrong value at (2,4,1,1).")
      self.assertEqual(dsds[2,4,1,2],0.0,"ds/ds has wrong value at (2,4,1,2).")
      self.assertEqual(dsds[2,4,1,3],0.0,"ds/ds has wrong value at (2,4,1,3).")
      self.assertEqual(dsds[2,4,1,4],0.0,"ds/ds has wrong value at (2,4,1,4).")
      self.assertEqual(dsds[2,4,2,0],0.0,"ds/ds has wrong value at (2,4,2,0).")
      self.assertEqual(dsds[2,4,2,1],0.0,"ds/ds has wrong value at (2,4,2,1).")
      self.assertEqual(dsds[2,4,2,2],0.0,"ds/ds has wrong value at (2,4,2,2).")
      self.assertEqual(dsds[2,4,2,3],0.0,"ds/ds has wrong value at (2,4,2,3).")
      self.assertEqual(dsds[2,4,2,4],1.0,"ds/ds has wrong value at (2,4,2,4).")
      self.assertEqual(dsds[2,4,3,0],0.0,"ds/ds has wrong value at (2,4,3,0).")
      self.assertEqual(dsds[2,4,3,1],0.0,"ds/ds has wrong value at (2,4,3,1).")
      self.assertEqual(dsds[2,4,3,2],0.0,"ds/ds has wrong value at (2,4,3,2).")
      self.assertEqual(dsds[2,4,3,3],0.0,"ds/ds has wrong value at (2,4,3,3).")
      self.assertEqual(dsds[2,4,3,4],0.0,"ds/ds has wrong value at (2,4,3,4).")
      self.assertEqual(dsds[3,0,0,0],0.0,"ds/ds has wrong value at (3,0,0,0).")
      self.assertEqual(dsds[3,0,0,1],0.0,"ds/ds has wrong value at (3,0,0,1).")
      self.assertEqual(dsds[3,0,0,2],0.0,"ds/ds has wrong value at (3,0,0,2).")
      self.assertEqual(dsds[3,0,0,3],0.0,"ds/ds has wrong value at (3,0,0,3).")
      self.assertEqual(dsds[3,0,0,4],0.0,"ds/ds has wrong value at (3,0,0,4).")
      self.assertEqual(dsds[3,0,1,0],0.0,"ds/ds has wrong value at (3,0,1,0).")
      self.assertEqual(dsds[3,0,1,1],0.0,"ds/ds has wrong value at (3,0,1,1).")
      self.assertEqual(dsds[3,0,1,2],0.0,"ds/ds has wrong value at (3,0,1,2).")
      self.assertEqual(dsds[3,0,1,3],0.0,"ds/ds has wrong value at (3,0,1,3).")
      self.assertEqual(dsds[3,0,1,4],0.0,"ds/ds has wrong value at (3,0,1,4).")
      self.assertEqual(dsds[3,0,2,0],0.0,"ds/ds has wrong value at (3,0,2,0).")
      self.assertEqual(dsds[3,0,2,1],0.0,"ds/ds has wrong value at (3,0,2,1).")
      self.assertEqual(dsds[3,0,2,2],0.0,"ds/ds has wrong value at (3,0,2,2).")
      self.assertEqual(dsds[3,0,2,3],0.0,"ds/ds has wrong value at (3,0,2,3).")
      self.assertEqual(dsds[3,0,2,4],0.0,"ds/ds has wrong value at (3,0,2,4).")
      self.assertEqual(dsds[3,0,3,0],1.0,"ds/ds has wrong value at (3,0,3,0).")
      self.assertEqual(dsds[3,0,3,1],0.0,"ds/ds has wrong value at (3,0,3,1).")
      self.assertEqual(dsds[3,0,3,2],0.0,"ds/ds has wrong value at (3,0,3,2).")
      self.assertEqual(dsds[3,0,3,3],0.0,"ds/ds has wrong value at (3,0,3,3).")
      self.assertEqual(dsds[3,0,3,4],0.0,"ds/ds has wrong value at (3,0,3,4).")
      self.assertEqual(dsds[3,1,0,0],0.0,"ds/ds has wrong value at (3,1,0,0).")
      self.assertEqual(dsds[3,1,0,1],0.0,"ds/ds has wrong value at (3,1,0,1).")
      self.assertEqual(dsds[3,1,0,2],0.0,"ds/ds has wrong value at (3,1,0,2).")
      self.assertEqual(dsds[3,1,0,3],0.0,"ds/ds has wrong value at (3,1,0,3).")
      self.assertEqual(dsds[3,1,0,4],0.0,"ds/ds has wrong value at (3,1,0,4).")
      self.assertEqual(dsds[3,1,1,0],0.0,"ds/ds has wrong value at (3,1,1,0).")
      self.assertEqual(dsds[3,1,1,1],0.0,"ds/ds has wrong value at (3,1,1,1).")
      self.assertEqual(dsds[3,1,1,2],0.0,"ds/ds has wrong value at (3,1,1,2).")
      self.assertEqual(dsds[3,1,1,3],0.0,"ds/ds has wrong value at (3,1,1,3).")
      self.assertEqual(dsds[3,1,1,4],0.0,"ds/ds has wrong value at (3,1,1,4).")
      self.assertEqual(dsds[3,1,2,0],0.0,"ds/ds has wrong value at (3,1,2,0).")
      self.assertEqual(dsds[3,1,2,1],0.0,"ds/ds has wrong value at (3,1,2,1).")
      self.assertEqual(dsds[3,1,2,2],0.0,"ds/ds has wrong value at (3,1,2,2).")
      self.assertEqual(dsds[3,1,2,3],0.0,"ds/ds has wrong value at (3,1,2,3).")
      self.assertEqual(dsds[3,1,2,4],0.0,"ds/ds has wrong value at (3,1,2,4).")
      self.assertEqual(dsds[3,1,3,0],0.0,"ds/ds has wrong value at (3,1,3,0).")
      self.assertEqual(dsds[3,1,3,1],1.0,"ds/ds has wrong value at (3,1,3,1).")
      self.assertEqual(dsds[3,1,3,2],0.0,"ds/ds has wrong value at (3,1,3,2).")
      self.assertEqual(dsds[3,1,3,3],0.0,"ds/ds has wrong value at (3,1,3,3).")
      self.assertEqual(dsds[3,1,3,4],0.0,"ds/ds has wrong value at (3,1,3,4).")
      self.assertEqual(dsds[3,2,0,0],0.0,"ds/ds has wrong value at (3,2,0,0).")
      self.assertEqual(dsds[3,2,0,1],0.0,"ds/ds has wrong value at (3,2,0,1).")
      self.assertEqual(dsds[3,2,0,2],0.0,"ds/ds has wrong value at (3,2,0,2).")
      self.assertEqual(dsds[3,2,0,3],0.0,"ds/ds has wrong value at (3,2,0,3).")
      self.assertEqual(dsds[3,2,0,4],0.0,"ds/ds has wrong value at (3,2,0,4).")
      self.assertEqual(dsds[3,2,1,0],0.0,"ds/ds has wrong value at (3,2,1,0).")
      self.assertEqual(dsds[3,2,1,1],0.0,"ds/ds has wrong value at (3,2,1,1).")
      self.assertEqual(dsds[3,2,1,2],0.0,"ds/ds has wrong value at (3,2,1,2).")
      self.assertEqual(dsds[3,2,1,3],0.0,"ds/ds has wrong value at (3,2,1,3).")
      self.assertEqual(dsds[3,2,1,4],0.0,"ds/ds has wrong value at (3,2,1,4).")
      self.assertEqual(dsds[3,2,2,0],0.0,"ds/ds has wrong value at (3,2,2,0).")
      self.assertEqual(dsds[3,2,2,1],0.0,"ds/ds has wrong value at (3,2,2,1).")
      self.assertEqual(dsds[3,2,2,2],0.0,"ds/ds has wrong value at (3,2,2,2).")
      self.assertEqual(dsds[3,2,2,3],0.0,"ds/ds has wrong value at (3,2,2,3).")
      self.assertEqual(dsds[3,2,2,4],0.0,"ds/ds has wrong value at (3,2,2,4).")
      self.assertEqual(dsds[3,2,3,0],0.0,"ds/ds has wrong value at (3,2,3,0).")
      self.assertEqual(dsds[3,2,3,1],0.0,"ds/ds has wrong value at (3,2,3,1).")
      self.assertEqual(dsds[3,2,3,2],1.0,"ds/ds has wrong value at (3,2,3,2).")
      self.assertEqual(dsds[3,2,3,3],0.0,"ds/ds has wrong value at (3,2,3,3).")
      self.assertEqual(dsds[3,2,3,4],0.0,"ds/ds has wrong value at (3,2,3,4).")
      self.assertEqual(dsds[3,3,0,0],0.0,"ds/ds has wrong value at (3,3,0,0).")
      self.assertEqual(dsds[3,3,0,1],0.0,"ds/ds has wrong value at (3,3,0,1).")
      self.assertEqual(dsds[3,3,0,2],0.0,"ds/ds has wrong value at (3,3,0,2).")
      self.assertEqual(dsds[3,3,0,3],0.0,"ds/ds has wrong value at (3,3,0,3).")
      self.assertEqual(dsds[3,3,0,4],0.0,"ds/ds has wrong value at (3,3,0,4).")
      self.assertEqual(dsds[3,3,1,0],0.0,"ds/ds has wrong value at (3,3,1,0).")
      self.assertEqual(dsds[3,3,1,1],0.0,"ds/ds has wrong value at (3,3,1,1).")
      self.assertEqual(dsds[3,3,1,2],0.0,"ds/ds has wrong value at (3,3,1,2).")
      self.assertEqual(dsds[3,3,1,3],0.0,"ds/ds has wrong value at (3,3,1,3).")
      self.assertEqual(dsds[3,3,1,4],0.0,"ds/ds has wrong value at (3,3,1,4).")
      self.assertEqual(dsds[3,3,2,0],0.0,"ds/ds has wrong value at (3,3,2,0).")
      self.assertEqual(dsds[3,3,2,1],0.0,"ds/ds has wrong value at (3,3,2,1).")
      self.assertEqual(dsds[3,3,2,2],0.0,"ds/ds has wrong value at (3,3,2,2).")
      self.assertEqual(dsds[3,3,2,3],0.0,"ds/ds has wrong value at (3,3,2,3).")
      self.assertEqual(dsds[3,3,2,4],0.0,"ds/ds has wrong value at (3,3,2,4).")
      self.assertEqual(dsds[3,3,3,0],0.0,"ds/ds has wrong value at (3,3,3,0).")
      self.assertEqual(dsds[3,3,3,1],0.0,"ds/ds has wrong value at (3,3,3,1).")
      self.assertEqual(dsds[3,3,3,2],0.0,"ds/ds has wrong value at (3,3,3,2).")
      self.assertEqual(dsds[3,3,3,3],1.0,"ds/ds has wrong value at (3,3,3,3).")
      self.assertEqual(dsds[3,3,3,4],0.0,"ds/ds has wrong value at (3,3,3,4).")
      self.assertEqual(dsds[3,4,0,0],0.0,"ds/ds has wrong value at (3,4,0,0).")
      self.assertEqual(dsds[3,4,0,1],0.0,"ds/ds has wrong value at (3,4,0,1).")
      self.assertEqual(dsds[3,4,0,2],0.0,"ds/ds has wrong value at (3,4,0,2).")
      self.assertEqual(dsds[3,4,0,3],0.0,"ds/ds has wrong value at (3,4,0,3).")
      self.assertEqual(dsds[3,4,0,4],0.0,"ds/ds has wrong value at (3,4,0,4).")
      self.assertEqual(dsds[3,4,1,0],0.0,"ds/ds has wrong value at (3,4,1,0).")
      self.assertEqual(dsds[3,4,1,1],0.0,"ds/ds has wrong value at (3,4,1,1).")
      self.assertEqual(dsds[3,4,1,2],0.0,"ds/ds has wrong value at (3,4,1,2).")
      self.assertEqual(dsds[3,4,1,3],0.0,"ds/ds has wrong value at (3,4,1,3).")
      self.assertEqual(dsds[3,4,1,4],0.0,"ds/ds has wrong value at (3,4,1,4).")
      self.assertEqual(dsds[3,4,2,0],0.0,"ds/ds has wrong value at (3,4,2,0).")
      self.assertEqual(dsds[3,4,2,1],0.0,"ds/ds has wrong value at (3,4,2,1).")
      self.assertEqual(dsds[3,4,2,2],0.0,"ds/ds has wrong value at (3,4,2,2).")
      self.assertEqual(dsds[3,4,2,3],0.0,"ds/ds has wrong value at (3,4,2,3).")
      self.assertEqual(dsds[3,4,2,4],0.0,"ds/ds has wrong value at (3,4,2,4).")
      self.assertEqual(dsds[3,4,3,0],0.0,"ds/ds has wrong value at (3,4,3,0).")
      self.assertEqual(dsds[3,4,3,1],0.0,"ds/ds has wrong value at (3,4,3,1).")
      self.assertEqual(dsds[3,4,3,2],0.0,"ds/ds has wrong value at (3,4,3,2).")
      self.assertEqual(dsds[3,4,3,3],0.0,"ds/ds has wrong value at (3,4,3,3).")
      self.assertEqual(dsds[3,4,3,4],1.0,"ds/ds has wrong value at (3,4,3,4).")

      self.assertEqual(s.getArgument(0),s2,"wrong argument 0.")
      sa=s.getSubstitutedArguments({s2:-10})
      self.assertEqual(len(sa),1,"wrong number of substituted arguments")
      self.assertEqual(sa[0],-10,"wrongly substituted argument 0.")

      self.assertFalse(s.isAppropriateValue(10.)," 10. is not appropriate substitute")
      self.assertFalse(s.isAppropriateValue(10)," 10 is not appropriate substitute")
      self.assertTrue(s.isAppropriateValue(Symbol(shape=(4, 5),dim=2)),"Symbol is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(Symbol(shape=(4, 5),dim=3)),"Symbol is not appropriate substitute (dim)")
      self.assertFalse(s.isAppropriateValue(escript.Data(0.,(5,),self.functionspace)),"Data is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(numpy.zeros((4, 5))),"ndarray is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(numpy.zeros((5,))),"ndarray is not appropriate substitute (shape)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_Symbol_rank2_d2_nargs2(self):
      s2=Symbol(shape=())
      s=Symbol(shape=(4, 5),dim=2,args=[1, -1.0])
      self.assertEqual(s.getRank(),2,"wrong rank.")
      self.assertEqual(s.getShape(),(4, 5),"wrong shape.")
      self.assertEqual(s.getDim(),2,"wrong spatial dimension.")
      self.assertEqual(s.getArgument(),[1, -1.0],"wrong arguments.")

      ss=s.substitute({s:numpy.zeros((4, 5))})
      self.assertTrue(isinstance(ss,numpy.ndarray),"value after substitution is not ndarray.")
      self.assertEqual(ss.shape,(4, 5),"value after substitution has not expected shape")
      try:
        s.substitute({s:numpy.zeros((5,))})
        fail("illegal substition was successful")
      except TypeError:
        pass

      dsdarg=s.diff(Symbol(shape=()))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds() has wrong type.")
      self.assertEqual(dsdarg.shape,(4, 5),"ds/ds() has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds() has wrong value.")
      dsdarg=s.diff(Symbol(shape=(2,)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(2,) has wrong type.")
      self.assertEqual(dsdarg.shape,(4, 5, 2),"ds/ds(2,) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(2,) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(4, 5)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(4, 5) has wrong type.")
      self.assertEqual(dsdarg.shape,(4, 5, 4, 5),"ds/ds(4, 5) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(4, 5) has wrong value.")

      dsds=s.diff(s)
      self.assertTrue(isinstance(dsds,numpy.ndarray),"ds/ds has wrong type.")
      self.assertEqual(dsds.shape,(4, 5, 4, 5),"ds/ds has wrong shape.")
      self.assertEqual(dsds[0,0,0,0],1.0,"ds/ds has wrong value at (0,0,0,0).")
      self.assertEqual(dsds[0,0,0,1],0.0,"ds/ds has wrong value at (0,0,0,1).")
      self.assertEqual(dsds[0,0,0,2],0.0,"ds/ds has wrong value at (0,0,0,2).")
      self.assertEqual(dsds[0,0,0,3],0.0,"ds/ds has wrong value at (0,0,0,3).")
      self.assertEqual(dsds[0,0,0,4],0.0,"ds/ds has wrong value at (0,0,0,4).")
      self.assertEqual(dsds[0,0,1,0],0.0,"ds/ds has wrong value at (0,0,1,0).")
      self.assertEqual(dsds[0,0,1,1],0.0,"ds/ds has wrong value at (0,0,1,1).")
      self.assertEqual(dsds[0,0,1,2],0.0,"ds/ds has wrong value at (0,0,1,2).")
      self.assertEqual(dsds[0,0,1,3],0.0,"ds/ds has wrong value at (0,0,1,3).")
      self.assertEqual(dsds[0,0,1,4],0.0,"ds/ds has wrong value at (0,0,1,4).")
      self.assertEqual(dsds[0,0,2,0],0.0,"ds/ds has wrong value at (0,0,2,0).")
      self.assertEqual(dsds[0,0,2,1],0.0,"ds/ds has wrong value at (0,0,2,1).")
      self.assertEqual(dsds[0,0,2,2],0.0,"ds/ds has wrong value at (0,0,2,2).")
      self.assertEqual(dsds[0,0,2,3],0.0,"ds/ds has wrong value at (0,0,2,3).")
      self.assertEqual(dsds[0,0,2,4],0.0,"ds/ds has wrong value at (0,0,2,4).")
      self.assertEqual(dsds[0,0,3,0],0.0,"ds/ds has wrong value at (0,0,3,0).")
      self.assertEqual(dsds[0,0,3,1],0.0,"ds/ds has wrong value at (0,0,3,1).")
      self.assertEqual(dsds[0,0,3,2],0.0,"ds/ds has wrong value at (0,0,3,2).")
      self.assertEqual(dsds[0,0,3,3],0.0,"ds/ds has wrong value at (0,0,3,3).")
      self.assertEqual(dsds[0,0,3,4],0.0,"ds/ds has wrong value at (0,0,3,4).")
      self.assertEqual(dsds[0,1,0,0],0.0,"ds/ds has wrong value at (0,1,0,0).")
      self.assertEqual(dsds[0,1,0,1],1.0,"ds/ds has wrong value at (0,1,0,1).")
      self.assertEqual(dsds[0,1,0,2],0.0,"ds/ds has wrong value at (0,1,0,2).")
      self.assertEqual(dsds[0,1,0,3],0.0,"ds/ds has wrong value at (0,1,0,3).")
      self.assertEqual(dsds[0,1,0,4],0.0,"ds/ds has wrong value at (0,1,0,4).")
      self.assertEqual(dsds[0,1,1,0],0.0,"ds/ds has wrong value at (0,1,1,0).")
      self.assertEqual(dsds[0,1,1,1],0.0,"ds/ds has wrong value at (0,1,1,1).")
      self.assertEqual(dsds[0,1,1,2],0.0,"ds/ds has wrong value at (0,1,1,2).")
      self.assertEqual(dsds[0,1,1,3],0.0,"ds/ds has wrong value at (0,1,1,3).")
      self.assertEqual(dsds[0,1,1,4],0.0,"ds/ds has wrong value at (0,1,1,4).")
      self.assertEqual(dsds[0,1,2,0],0.0,"ds/ds has wrong value at (0,1,2,0).")
      self.assertEqual(dsds[0,1,2,1],0.0,"ds/ds has wrong value at (0,1,2,1).")
      self.assertEqual(dsds[0,1,2,2],0.0,"ds/ds has wrong value at (0,1,2,2).")
      self.assertEqual(dsds[0,1,2,3],0.0,"ds/ds has wrong value at (0,1,2,3).")
      self.assertEqual(dsds[0,1,2,4],0.0,"ds/ds has wrong value at (0,1,2,4).")
      self.assertEqual(dsds[0,1,3,0],0.0,"ds/ds has wrong value at (0,1,3,0).")
      self.assertEqual(dsds[0,1,3,1],0.0,"ds/ds has wrong value at (0,1,3,1).")
      self.assertEqual(dsds[0,1,3,2],0.0,"ds/ds has wrong value at (0,1,3,2).")
      self.assertEqual(dsds[0,1,3,3],0.0,"ds/ds has wrong value at (0,1,3,3).")
      self.assertEqual(dsds[0,1,3,4],0.0,"ds/ds has wrong value at (0,1,3,4).")
      self.assertEqual(dsds[0,2,0,0],0.0,"ds/ds has wrong value at (0,2,0,0).")
      self.assertEqual(dsds[0,2,0,1],0.0,"ds/ds has wrong value at (0,2,0,1).")
      self.assertEqual(dsds[0,2,0,2],1.0,"ds/ds has wrong value at (0,2,0,2).")
      self.assertEqual(dsds[0,2,0,3],0.0,"ds/ds has wrong value at (0,2,0,3).")
      self.assertEqual(dsds[0,2,0,4],0.0,"ds/ds has wrong value at (0,2,0,4).")
      self.assertEqual(dsds[0,2,1,0],0.0,"ds/ds has wrong value at (0,2,1,0).")
      self.assertEqual(dsds[0,2,1,1],0.0,"ds/ds has wrong value at (0,2,1,1).")
      self.assertEqual(dsds[0,2,1,2],0.0,"ds/ds has wrong value at (0,2,1,2).")
      self.assertEqual(dsds[0,2,1,3],0.0,"ds/ds has wrong value at (0,2,1,3).")
      self.assertEqual(dsds[0,2,1,4],0.0,"ds/ds has wrong value at (0,2,1,4).")
      self.assertEqual(dsds[0,2,2,0],0.0,"ds/ds has wrong value at (0,2,2,0).")
      self.assertEqual(dsds[0,2,2,1],0.0,"ds/ds has wrong value at (0,2,2,1).")
      self.assertEqual(dsds[0,2,2,2],0.0,"ds/ds has wrong value at (0,2,2,2).")
      self.assertEqual(dsds[0,2,2,3],0.0,"ds/ds has wrong value at (0,2,2,3).")
      self.assertEqual(dsds[0,2,2,4],0.0,"ds/ds has wrong value at (0,2,2,4).")
      self.assertEqual(dsds[0,2,3,0],0.0,"ds/ds has wrong value at (0,2,3,0).")
      self.assertEqual(dsds[0,2,3,1],0.0,"ds/ds has wrong value at (0,2,3,1).")
      self.assertEqual(dsds[0,2,3,2],0.0,"ds/ds has wrong value at (0,2,3,2).")
      self.assertEqual(dsds[0,2,3,3],0.0,"ds/ds has wrong value at (0,2,3,3).")
      self.assertEqual(dsds[0,2,3,4],0.0,"ds/ds has wrong value at (0,2,3,4).")
      self.assertEqual(dsds[0,3,0,0],0.0,"ds/ds has wrong value at (0,3,0,0).")
      self.assertEqual(dsds[0,3,0,1],0.0,"ds/ds has wrong value at (0,3,0,1).")
      self.assertEqual(dsds[0,3,0,2],0.0,"ds/ds has wrong value at (0,3,0,2).")
      self.assertEqual(dsds[0,3,0,3],1.0,"ds/ds has wrong value at (0,3,0,3).")
      self.assertEqual(dsds[0,3,0,4],0.0,"ds/ds has wrong value at (0,3,0,4).")
      self.assertEqual(dsds[0,3,1,0],0.0,"ds/ds has wrong value at (0,3,1,0).")
      self.assertEqual(dsds[0,3,1,1],0.0,"ds/ds has wrong value at (0,3,1,1).")
      self.assertEqual(dsds[0,3,1,2],0.0,"ds/ds has wrong value at (0,3,1,2).")
      self.assertEqual(dsds[0,3,1,3],0.0,"ds/ds has wrong value at (0,3,1,3).")
      self.assertEqual(dsds[0,3,1,4],0.0,"ds/ds has wrong value at (0,3,1,4).")
      self.assertEqual(dsds[0,3,2,0],0.0,"ds/ds has wrong value at (0,3,2,0).")
      self.assertEqual(dsds[0,3,2,1],0.0,"ds/ds has wrong value at (0,3,2,1).")
      self.assertEqual(dsds[0,3,2,2],0.0,"ds/ds has wrong value at (0,3,2,2).")
      self.assertEqual(dsds[0,3,2,3],0.0,"ds/ds has wrong value at (0,3,2,3).")
      self.assertEqual(dsds[0,3,2,4],0.0,"ds/ds has wrong value at (0,3,2,4).")
      self.assertEqual(dsds[0,3,3,0],0.0,"ds/ds has wrong value at (0,3,3,0).")
      self.assertEqual(dsds[0,3,3,1],0.0,"ds/ds has wrong value at (0,3,3,1).")
      self.assertEqual(dsds[0,3,3,2],0.0,"ds/ds has wrong value at (0,3,3,2).")
      self.assertEqual(dsds[0,3,3,3],0.0,"ds/ds has wrong value at (0,3,3,3).")
      self.assertEqual(dsds[0,3,3,4],0.0,"ds/ds has wrong value at (0,3,3,4).")
      self.assertEqual(dsds[0,4,0,0],0.0,"ds/ds has wrong value at (0,4,0,0).")
      self.assertEqual(dsds[0,4,0,1],0.0,"ds/ds has wrong value at (0,4,0,1).")
      self.assertEqual(dsds[0,4,0,2],0.0,"ds/ds has wrong value at (0,4,0,2).")
      self.assertEqual(dsds[0,4,0,3],0.0,"ds/ds has wrong value at (0,4,0,3).")
      self.assertEqual(dsds[0,4,0,4],1.0,"ds/ds has wrong value at (0,4,0,4).")
      self.assertEqual(dsds[0,4,1,0],0.0,"ds/ds has wrong value at (0,4,1,0).")
      self.assertEqual(dsds[0,4,1,1],0.0,"ds/ds has wrong value at (0,4,1,1).")
      self.assertEqual(dsds[0,4,1,2],0.0,"ds/ds has wrong value at (0,4,1,2).")
      self.assertEqual(dsds[0,4,1,3],0.0,"ds/ds has wrong value at (0,4,1,3).")
      self.assertEqual(dsds[0,4,1,4],0.0,"ds/ds has wrong value at (0,4,1,4).")
      self.assertEqual(dsds[0,4,2,0],0.0,"ds/ds has wrong value at (0,4,2,0).")
      self.assertEqual(dsds[0,4,2,1],0.0,"ds/ds has wrong value at (0,4,2,1).")
      self.assertEqual(dsds[0,4,2,2],0.0,"ds/ds has wrong value at (0,4,2,2).")
      self.assertEqual(dsds[0,4,2,3],0.0,"ds/ds has wrong value at (0,4,2,3).")
      self.assertEqual(dsds[0,4,2,4],0.0,"ds/ds has wrong value at (0,4,2,4).")
      self.assertEqual(dsds[0,4,3,0],0.0,"ds/ds has wrong value at (0,4,3,0).")
      self.assertEqual(dsds[0,4,3,1],0.0,"ds/ds has wrong value at (0,4,3,1).")
      self.assertEqual(dsds[0,4,3,2],0.0,"ds/ds has wrong value at (0,4,3,2).")
      self.assertEqual(dsds[0,4,3,3],0.0,"ds/ds has wrong value at (0,4,3,3).")
      self.assertEqual(dsds[0,4,3,4],0.0,"ds/ds has wrong value at (0,4,3,4).")
      self.assertEqual(dsds[1,0,0,0],0.0,"ds/ds has wrong value at (1,0,0,0).")
      self.assertEqual(dsds[1,0,0,1],0.0,"ds/ds has wrong value at (1,0,0,1).")
      self.assertEqual(dsds[1,0,0,2],0.0,"ds/ds has wrong value at (1,0,0,2).")
      self.assertEqual(dsds[1,0,0,3],0.0,"ds/ds has wrong value at (1,0,0,3).")
      self.assertEqual(dsds[1,0,0,4],0.0,"ds/ds has wrong value at (1,0,0,4).")
      self.assertEqual(dsds[1,0,1,0],1.0,"ds/ds has wrong value at (1,0,1,0).")
      self.assertEqual(dsds[1,0,1,1],0.0,"ds/ds has wrong value at (1,0,1,1).")
      self.assertEqual(dsds[1,0,1,2],0.0,"ds/ds has wrong value at (1,0,1,2).")
      self.assertEqual(dsds[1,0,1,3],0.0,"ds/ds has wrong value at (1,0,1,3).")
      self.assertEqual(dsds[1,0,1,4],0.0,"ds/ds has wrong value at (1,0,1,4).")
      self.assertEqual(dsds[1,0,2,0],0.0,"ds/ds has wrong value at (1,0,2,0).")
      self.assertEqual(dsds[1,0,2,1],0.0,"ds/ds has wrong value at (1,0,2,1).")
      self.assertEqual(dsds[1,0,2,2],0.0,"ds/ds has wrong value at (1,0,2,2).")
      self.assertEqual(dsds[1,0,2,3],0.0,"ds/ds has wrong value at (1,0,2,3).")
      self.assertEqual(dsds[1,0,2,4],0.0,"ds/ds has wrong value at (1,0,2,4).")
      self.assertEqual(dsds[1,0,3,0],0.0,"ds/ds has wrong value at (1,0,3,0).")
      self.assertEqual(dsds[1,0,3,1],0.0,"ds/ds has wrong value at (1,0,3,1).")
      self.assertEqual(dsds[1,0,3,2],0.0,"ds/ds has wrong value at (1,0,3,2).")
      self.assertEqual(dsds[1,0,3,3],0.0,"ds/ds has wrong value at (1,0,3,3).")
      self.assertEqual(dsds[1,0,3,4],0.0,"ds/ds has wrong value at (1,0,3,4).")
      self.assertEqual(dsds[1,1,0,0],0.0,"ds/ds has wrong value at (1,1,0,0).")
      self.assertEqual(dsds[1,1,0,1],0.0,"ds/ds has wrong value at (1,1,0,1).")
      self.assertEqual(dsds[1,1,0,2],0.0,"ds/ds has wrong value at (1,1,0,2).")
      self.assertEqual(dsds[1,1,0,3],0.0,"ds/ds has wrong value at (1,1,0,3).")
      self.assertEqual(dsds[1,1,0,4],0.0,"ds/ds has wrong value at (1,1,0,4).")
      self.assertEqual(dsds[1,1,1,0],0.0,"ds/ds has wrong value at (1,1,1,0).")
      self.assertEqual(dsds[1,1,1,1],1.0,"ds/ds has wrong value at (1,1,1,1).")
      self.assertEqual(dsds[1,1,1,2],0.0,"ds/ds has wrong value at (1,1,1,2).")
      self.assertEqual(dsds[1,1,1,3],0.0,"ds/ds has wrong value at (1,1,1,3).")
      self.assertEqual(dsds[1,1,1,4],0.0,"ds/ds has wrong value at (1,1,1,4).")
      self.assertEqual(dsds[1,1,2,0],0.0,"ds/ds has wrong value at (1,1,2,0).")
      self.assertEqual(dsds[1,1,2,1],0.0,"ds/ds has wrong value at (1,1,2,1).")
      self.assertEqual(dsds[1,1,2,2],0.0,"ds/ds has wrong value at (1,1,2,2).")
      self.assertEqual(dsds[1,1,2,3],0.0,"ds/ds has wrong value at (1,1,2,3).")
      self.assertEqual(dsds[1,1,2,4],0.0,"ds/ds has wrong value at (1,1,2,4).")
      self.assertEqual(dsds[1,1,3,0],0.0,"ds/ds has wrong value at (1,1,3,0).")
      self.assertEqual(dsds[1,1,3,1],0.0,"ds/ds has wrong value at (1,1,3,1).")
      self.assertEqual(dsds[1,1,3,2],0.0,"ds/ds has wrong value at (1,1,3,2).")
      self.assertEqual(dsds[1,1,3,3],0.0,"ds/ds has wrong value at (1,1,3,3).")
      self.assertEqual(dsds[1,1,3,4],0.0,"ds/ds has wrong value at (1,1,3,4).")
      self.assertEqual(dsds[1,2,0,0],0.0,"ds/ds has wrong value at (1,2,0,0).")
      self.assertEqual(dsds[1,2,0,1],0.0,"ds/ds has wrong value at (1,2,0,1).")
      self.assertEqual(dsds[1,2,0,2],0.0,"ds/ds has wrong value at (1,2,0,2).")
      self.assertEqual(dsds[1,2,0,3],0.0,"ds/ds has wrong value at (1,2,0,3).")
      self.assertEqual(dsds[1,2,0,4],0.0,"ds/ds has wrong value at (1,2,0,4).")
      self.assertEqual(dsds[1,2,1,0],0.0,"ds/ds has wrong value at (1,2,1,0).")
      self.assertEqual(dsds[1,2,1,1],0.0,"ds/ds has wrong value at (1,2,1,1).")
      self.assertEqual(dsds[1,2,1,2],1.0,"ds/ds has wrong value at (1,2,1,2).")
      self.assertEqual(dsds[1,2,1,3],0.0,"ds/ds has wrong value at (1,2,1,3).")
      self.assertEqual(dsds[1,2,1,4],0.0,"ds/ds has wrong value at (1,2,1,4).")
      self.assertEqual(dsds[1,2,2,0],0.0,"ds/ds has wrong value at (1,2,2,0).")
      self.assertEqual(dsds[1,2,2,1],0.0,"ds/ds has wrong value at (1,2,2,1).")
      self.assertEqual(dsds[1,2,2,2],0.0,"ds/ds has wrong value at (1,2,2,2).")
      self.assertEqual(dsds[1,2,2,3],0.0,"ds/ds has wrong value at (1,2,2,3).")
      self.assertEqual(dsds[1,2,2,4],0.0,"ds/ds has wrong value at (1,2,2,4).")
      self.assertEqual(dsds[1,2,3,0],0.0,"ds/ds has wrong value at (1,2,3,0).")
      self.assertEqual(dsds[1,2,3,1],0.0,"ds/ds has wrong value at (1,2,3,1).")
      self.assertEqual(dsds[1,2,3,2],0.0,"ds/ds has wrong value at (1,2,3,2).")
      self.assertEqual(dsds[1,2,3,3],0.0,"ds/ds has wrong value at (1,2,3,3).")
      self.assertEqual(dsds[1,2,3,4],0.0,"ds/ds has wrong value at (1,2,3,4).")
      self.assertEqual(dsds[1,3,0,0],0.0,"ds/ds has wrong value at (1,3,0,0).")
      self.assertEqual(dsds[1,3,0,1],0.0,"ds/ds has wrong value at (1,3,0,1).")
      self.assertEqual(dsds[1,3,0,2],0.0,"ds/ds has wrong value at (1,3,0,2).")
      self.assertEqual(dsds[1,3,0,3],0.0,"ds/ds has wrong value at (1,3,0,3).")
      self.assertEqual(dsds[1,3,0,4],0.0,"ds/ds has wrong value at (1,3,0,4).")
      self.assertEqual(dsds[1,3,1,0],0.0,"ds/ds has wrong value at (1,3,1,0).")
      self.assertEqual(dsds[1,3,1,1],0.0,"ds/ds has wrong value at (1,3,1,1).")
      self.assertEqual(dsds[1,3,1,2],0.0,"ds/ds has wrong value at (1,3,1,2).")
      self.assertEqual(dsds[1,3,1,3],1.0,"ds/ds has wrong value at (1,3,1,3).")
      self.assertEqual(dsds[1,3,1,4],0.0,"ds/ds has wrong value at (1,3,1,4).")
      self.assertEqual(dsds[1,3,2,0],0.0,"ds/ds has wrong value at (1,3,2,0).")
      self.assertEqual(dsds[1,3,2,1],0.0,"ds/ds has wrong value at (1,3,2,1).")
      self.assertEqual(dsds[1,3,2,2],0.0,"ds/ds has wrong value at (1,3,2,2).")
      self.assertEqual(dsds[1,3,2,3],0.0,"ds/ds has wrong value at (1,3,2,3).")
      self.assertEqual(dsds[1,3,2,4],0.0,"ds/ds has wrong value at (1,3,2,4).")
      self.assertEqual(dsds[1,3,3,0],0.0,"ds/ds has wrong value at (1,3,3,0).")
      self.assertEqual(dsds[1,3,3,1],0.0,"ds/ds has wrong value at (1,3,3,1).")
      self.assertEqual(dsds[1,3,3,2],0.0,"ds/ds has wrong value at (1,3,3,2).")
      self.assertEqual(dsds[1,3,3,3],0.0,"ds/ds has wrong value at (1,3,3,3).")
      self.assertEqual(dsds[1,3,3,4],0.0,"ds/ds has wrong value at (1,3,3,4).")
      self.assertEqual(dsds[1,4,0,0],0.0,"ds/ds has wrong value at (1,4,0,0).")
      self.assertEqual(dsds[1,4,0,1],0.0,"ds/ds has wrong value at (1,4,0,1).")
      self.assertEqual(dsds[1,4,0,2],0.0,"ds/ds has wrong value at (1,4,0,2).")
      self.assertEqual(dsds[1,4,0,3],0.0,"ds/ds has wrong value at (1,4,0,3).")
      self.assertEqual(dsds[1,4,0,4],0.0,"ds/ds has wrong value at (1,4,0,4).")
      self.assertEqual(dsds[1,4,1,0],0.0,"ds/ds has wrong value at (1,4,1,0).")
      self.assertEqual(dsds[1,4,1,1],0.0,"ds/ds has wrong value at (1,4,1,1).")
      self.assertEqual(dsds[1,4,1,2],0.0,"ds/ds has wrong value at (1,4,1,2).")
      self.assertEqual(dsds[1,4,1,3],0.0,"ds/ds has wrong value at (1,4,1,3).")
      self.assertEqual(dsds[1,4,1,4],1.0,"ds/ds has wrong value at (1,4,1,4).")
      self.assertEqual(dsds[1,4,2,0],0.0,"ds/ds has wrong value at (1,4,2,0).")
      self.assertEqual(dsds[1,4,2,1],0.0,"ds/ds has wrong value at (1,4,2,1).")
      self.assertEqual(dsds[1,4,2,2],0.0,"ds/ds has wrong value at (1,4,2,2).")
      self.assertEqual(dsds[1,4,2,3],0.0,"ds/ds has wrong value at (1,4,2,3).")
      self.assertEqual(dsds[1,4,2,4],0.0,"ds/ds has wrong value at (1,4,2,4).")
      self.assertEqual(dsds[1,4,3,0],0.0,"ds/ds has wrong value at (1,4,3,0).")
      self.assertEqual(dsds[1,4,3,1],0.0,"ds/ds has wrong value at (1,4,3,1).")
      self.assertEqual(dsds[1,4,3,2],0.0,"ds/ds has wrong value at (1,4,3,2).")
      self.assertEqual(dsds[1,4,3,3],0.0,"ds/ds has wrong value at (1,4,3,3).")
      self.assertEqual(dsds[1,4,3,4],0.0,"ds/ds has wrong value at (1,4,3,4).")
      self.assertEqual(dsds[2,0,0,0],0.0,"ds/ds has wrong value at (2,0,0,0).")
      self.assertEqual(dsds[2,0,0,1],0.0,"ds/ds has wrong value at (2,0,0,1).")
      self.assertEqual(dsds[2,0,0,2],0.0,"ds/ds has wrong value at (2,0,0,2).")
      self.assertEqual(dsds[2,0,0,3],0.0,"ds/ds has wrong value at (2,0,0,3).")
      self.assertEqual(dsds[2,0,0,4],0.0,"ds/ds has wrong value at (2,0,0,4).")
      self.assertEqual(dsds[2,0,1,0],0.0,"ds/ds has wrong value at (2,0,1,0).")
      self.assertEqual(dsds[2,0,1,1],0.0,"ds/ds has wrong value at (2,0,1,1).")
      self.assertEqual(dsds[2,0,1,2],0.0,"ds/ds has wrong value at (2,0,1,2).")
      self.assertEqual(dsds[2,0,1,3],0.0,"ds/ds has wrong value at (2,0,1,3).")
      self.assertEqual(dsds[2,0,1,4],0.0,"ds/ds has wrong value at (2,0,1,4).")
      self.assertEqual(dsds[2,0,2,0],1.0,"ds/ds has wrong value at (2,0,2,0).")
      self.assertEqual(dsds[2,0,2,1],0.0,"ds/ds has wrong value at (2,0,2,1).")
      self.assertEqual(dsds[2,0,2,2],0.0,"ds/ds has wrong value at (2,0,2,2).")
      self.assertEqual(dsds[2,0,2,3],0.0,"ds/ds has wrong value at (2,0,2,3).")
      self.assertEqual(dsds[2,0,2,4],0.0,"ds/ds has wrong value at (2,0,2,4).")
      self.assertEqual(dsds[2,0,3,0],0.0,"ds/ds has wrong value at (2,0,3,0).")
      self.assertEqual(dsds[2,0,3,1],0.0,"ds/ds has wrong value at (2,0,3,1).")
      self.assertEqual(dsds[2,0,3,2],0.0,"ds/ds has wrong value at (2,0,3,2).")
      self.assertEqual(dsds[2,0,3,3],0.0,"ds/ds has wrong value at (2,0,3,3).")
      self.assertEqual(dsds[2,0,3,4],0.0,"ds/ds has wrong value at (2,0,3,4).")
      self.assertEqual(dsds[2,1,0,0],0.0,"ds/ds has wrong value at (2,1,0,0).")
      self.assertEqual(dsds[2,1,0,1],0.0,"ds/ds has wrong value at (2,1,0,1).")
      self.assertEqual(dsds[2,1,0,2],0.0,"ds/ds has wrong value at (2,1,0,2).")
      self.assertEqual(dsds[2,1,0,3],0.0,"ds/ds has wrong value at (2,1,0,3).")
      self.assertEqual(dsds[2,1,0,4],0.0,"ds/ds has wrong value at (2,1,0,4).")
      self.assertEqual(dsds[2,1,1,0],0.0,"ds/ds has wrong value at (2,1,1,0).")
      self.assertEqual(dsds[2,1,1,1],0.0,"ds/ds has wrong value at (2,1,1,1).")
      self.assertEqual(dsds[2,1,1,2],0.0,"ds/ds has wrong value at (2,1,1,2).")
      self.assertEqual(dsds[2,1,1,3],0.0,"ds/ds has wrong value at (2,1,1,3).")
      self.assertEqual(dsds[2,1,1,4],0.0,"ds/ds has wrong value at (2,1,1,4).")
      self.assertEqual(dsds[2,1,2,0],0.0,"ds/ds has wrong value at (2,1,2,0).")
      self.assertEqual(dsds[2,1,2,1],1.0,"ds/ds has wrong value at (2,1,2,1).")
      self.assertEqual(dsds[2,1,2,2],0.0,"ds/ds has wrong value at (2,1,2,2).")
      self.assertEqual(dsds[2,1,2,3],0.0,"ds/ds has wrong value at (2,1,2,3).")
      self.assertEqual(dsds[2,1,2,4],0.0,"ds/ds has wrong value at (2,1,2,4).")
      self.assertEqual(dsds[2,1,3,0],0.0,"ds/ds has wrong value at (2,1,3,0).")
      self.assertEqual(dsds[2,1,3,1],0.0,"ds/ds has wrong value at (2,1,3,1).")
      self.assertEqual(dsds[2,1,3,2],0.0,"ds/ds has wrong value at (2,1,3,2).")
      self.assertEqual(dsds[2,1,3,3],0.0,"ds/ds has wrong value at (2,1,3,3).")
      self.assertEqual(dsds[2,1,3,4],0.0,"ds/ds has wrong value at (2,1,3,4).")
      self.assertEqual(dsds[2,2,0,0],0.0,"ds/ds has wrong value at (2,2,0,0).")
      self.assertEqual(dsds[2,2,0,1],0.0,"ds/ds has wrong value at (2,2,0,1).")
      self.assertEqual(dsds[2,2,0,2],0.0,"ds/ds has wrong value at (2,2,0,2).")
      self.assertEqual(dsds[2,2,0,3],0.0,"ds/ds has wrong value at (2,2,0,3).")
      self.assertEqual(dsds[2,2,0,4],0.0,"ds/ds has wrong value at (2,2,0,4).")
      self.assertEqual(dsds[2,2,1,0],0.0,"ds/ds has wrong value at (2,2,1,0).")
      self.assertEqual(dsds[2,2,1,1],0.0,"ds/ds has wrong value at (2,2,1,1).")
      self.assertEqual(dsds[2,2,1,2],0.0,"ds/ds has wrong value at (2,2,1,2).")
      self.assertEqual(dsds[2,2,1,3],0.0,"ds/ds has wrong value at (2,2,1,3).")
      self.assertEqual(dsds[2,2,1,4],0.0,"ds/ds has wrong value at (2,2,1,4).")
      self.assertEqual(dsds[2,2,2,0],0.0,"ds/ds has wrong value at (2,2,2,0).")
      self.assertEqual(dsds[2,2,2,1],0.0,"ds/ds has wrong value at (2,2,2,1).")
      self.assertEqual(dsds[2,2,2,2],1.0,"ds/ds has wrong value at (2,2,2,2).")
      self.assertEqual(dsds[2,2,2,3],0.0,"ds/ds has wrong value at (2,2,2,3).")
      self.assertEqual(dsds[2,2,2,4],0.0,"ds/ds has wrong value at (2,2,2,4).")
      self.assertEqual(dsds[2,2,3,0],0.0,"ds/ds has wrong value at (2,2,3,0).")
      self.assertEqual(dsds[2,2,3,1],0.0,"ds/ds has wrong value at (2,2,3,1).")
      self.assertEqual(dsds[2,2,3,2],0.0,"ds/ds has wrong value at (2,2,3,2).")
      self.assertEqual(dsds[2,2,3,3],0.0,"ds/ds has wrong value at (2,2,3,3).")
      self.assertEqual(dsds[2,2,3,4],0.0,"ds/ds has wrong value at (2,2,3,4).")
      self.assertEqual(dsds[2,3,0,0],0.0,"ds/ds has wrong value at (2,3,0,0).")
      self.assertEqual(dsds[2,3,0,1],0.0,"ds/ds has wrong value at (2,3,0,1).")
      self.assertEqual(dsds[2,3,0,2],0.0,"ds/ds has wrong value at (2,3,0,2).")
      self.assertEqual(dsds[2,3,0,3],0.0,"ds/ds has wrong value at (2,3,0,3).")
      self.assertEqual(dsds[2,3,0,4],0.0,"ds/ds has wrong value at (2,3,0,4).")
      self.assertEqual(dsds[2,3,1,0],0.0,"ds/ds has wrong value at (2,3,1,0).")
      self.assertEqual(dsds[2,3,1,1],0.0,"ds/ds has wrong value at (2,3,1,1).")
      self.assertEqual(dsds[2,3,1,2],0.0,"ds/ds has wrong value at (2,3,1,2).")
      self.assertEqual(dsds[2,3,1,3],0.0,"ds/ds has wrong value at (2,3,1,3).")
      self.assertEqual(dsds[2,3,1,4],0.0,"ds/ds has wrong value at (2,3,1,4).")
      self.assertEqual(dsds[2,3,2,0],0.0,"ds/ds has wrong value at (2,3,2,0).")
      self.assertEqual(dsds[2,3,2,1],0.0,"ds/ds has wrong value at (2,3,2,1).")
      self.assertEqual(dsds[2,3,2,2],0.0,"ds/ds has wrong value at (2,3,2,2).")
      self.assertEqual(dsds[2,3,2,3],1.0,"ds/ds has wrong value at (2,3,2,3).")
      self.assertEqual(dsds[2,3,2,4],0.0,"ds/ds has wrong value at (2,3,2,4).")
      self.assertEqual(dsds[2,3,3,0],0.0,"ds/ds has wrong value at (2,3,3,0).")
      self.assertEqual(dsds[2,3,3,1],0.0,"ds/ds has wrong value at (2,3,3,1).")
      self.assertEqual(dsds[2,3,3,2],0.0,"ds/ds has wrong value at (2,3,3,2).")
      self.assertEqual(dsds[2,3,3,3],0.0,"ds/ds has wrong value at (2,3,3,3).")
      self.assertEqual(dsds[2,3,3,4],0.0,"ds/ds has wrong value at (2,3,3,4).")
      self.assertEqual(dsds[2,4,0,0],0.0,"ds/ds has wrong value at (2,4,0,0).")
      self.assertEqual(dsds[2,4,0,1],0.0,"ds/ds has wrong value at (2,4,0,1).")
      self.assertEqual(dsds[2,4,0,2],0.0,"ds/ds has wrong value at (2,4,0,2).")
      self.assertEqual(dsds[2,4,0,3],0.0,"ds/ds has wrong value at (2,4,0,3).")
      self.assertEqual(dsds[2,4,0,4],0.0,"ds/ds has wrong value at (2,4,0,4).")
      self.assertEqual(dsds[2,4,1,0],0.0,"ds/ds has wrong value at (2,4,1,0).")
      self.assertEqual(dsds[2,4,1,1],0.0,"ds/ds has wrong value at (2,4,1,1).")
      self.assertEqual(dsds[2,4,1,2],0.0,"ds/ds has wrong value at (2,4,1,2).")
      self.assertEqual(dsds[2,4,1,3],0.0,"ds/ds has wrong value at (2,4,1,3).")
      self.assertEqual(dsds[2,4,1,4],0.0,"ds/ds has wrong value at (2,4,1,4).")
      self.assertEqual(dsds[2,4,2,0],0.0,"ds/ds has wrong value at (2,4,2,0).")
      self.assertEqual(dsds[2,4,2,1],0.0,"ds/ds has wrong value at (2,4,2,1).")
      self.assertEqual(dsds[2,4,2,2],0.0,"ds/ds has wrong value at (2,4,2,2).")
      self.assertEqual(dsds[2,4,2,3],0.0,"ds/ds has wrong value at (2,4,2,3).")
      self.assertEqual(dsds[2,4,2,4],1.0,"ds/ds has wrong value at (2,4,2,4).")
      self.assertEqual(dsds[2,4,3,0],0.0,"ds/ds has wrong value at (2,4,3,0).")
      self.assertEqual(dsds[2,4,3,1],0.0,"ds/ds has wrong value at (2,4,3,1).")
      self.assertEqual(dsds[2,4,3,2],0.0,"ds/ds has wrong value at (2,4,3,2).")
      self.assertEqual(dsds[2,4,3,3],0.0,"ds/ds has wrong value at (2,4,3,3).")
      self.assertEqual(dsds[2,4,3,4],0.0,"ds/ds has wrong value at (2,4,3,4).")
      self.assertEqual(dsds[3,0,0,0],0.0,"ds/ds has wrong value at (3,0,0,0).")
      self.assertEqual(dsds[3,0,0,1],0.0,"ds/ds has wrong value at (3,0,0,1).")
      self.assertEqual(dsds[3,0,0,2],0.0,"ds/ds has wrong value at (3,0,0,2).")
      self.assertEqual(dsds[3,0,0,3],0.0,"ds/ds has wrong value at (3,0,0,3).")
      self.assertEqual(dsds[3,0,0,4],0.0,"ds/ds has wrong value at (3,0,0,4).")
      self.assertEqual(dsds[3,0,1,0],0.0,"ds/ds has wrong value at (3,0,1,0).")
      self.assertEqual(dsds[3,0,1,1],0.0,"ds/ds has wrong value at (3,0,1,1).")
      self.assertEqual(dsds[3,0,1,2],0.0,"ds/ds has wrong value at (3,0,1,2).")
      self.assertEqual(dsds[3,0,1,3],0.0,"ds/ds has wrong value at (3,0,1,3).")
      self.assertEqual(dsds[3,0,1,4],0.0,"ds/ds has wrong value at (3,0,1,4).")
      self.assertEqual(dsds[3,0,2,0],0.0,"ds/ds has wrong value at (3,0,2,0).")
      self.assertEqual(dsds[3,0,2,1],0.0,"ds/ds has wrong value at (3,0,2,1).")
      self.assertEqual(dsds[3,0,2,2],0.0,"ds/ds has wrong value at (3,0,2,2).")
      self.assertEqual(dsds[3,0,2,3],0.0,"ds/ds has wrong value at (3,0,2,3).")
      self.assertEqual(dsds[3,0,2,4],0.0,"ds/ds has wrong value at (3,0,2,4).")
      self.assertEqual(dsds[3,0,3,0],1.0,"ds/ds has wrong value at (3,0,3,0).")
      self.assertEqual(dsds[3,0,3,1],0.0,"ds/ds has wrong value at (3,0,3,1).")
      self.assertEqual(dsds[3,0,3,2],0.0,"ds/ds has wrong value at (3,0,3,2).")
      self.assertEqual(dsds[3,0,3,3],0.0,"ds/ds has wrong value at (3,0,3,3).")
      self.assertEqual(dsds[3,0,3,4],0.0,"ds/ds has wrong value at (3,0,3,4).")
      self.assertEqual(dsds[3,1,0,0],0.0,"ds/ds has wrong value at (3,1,0,0).")
      self.assertEqual(dsds[3,1,0,1],0.0,"ds/ds has wrong value at (3,1,0,1).")
      self.assertEqual(dsds[3,1,0,2],0.0,"ds/ds has wrong value at (3,1,0,2).")
      self.assertEqual(dsds[3,1,0,3],0.0,"ds/ds has wrong value at (3,1,0,3).")
      self.assertEqual(dsds[3,1,0,4],0.0,"ds/ds has wrong value at (3,1,0,4).")
      self.assertEqual(dsds[3,1,1,0],0.0,"ds/ds has wrong value at (3,1,1,0).")
      self.assertEqual(dsds[3,1,1,1],0.0,"ds/ds has wrong value at (3,1,1,1).")
      self.assertEqual(dsds[3,1,1,2],0.0,"ds/ds has wrong value at (3,1,1,2).")
      self.assertEqual(dsds[3,1,1,3],0.0,"ds/ds has wrong value at (3,1,1,3).")
      self.assertEqual(dsds[3,1,1,4],0.0,"ds/ds has wrong value at (3,1,1,4).")
      self.assertEqual(dsds[3,1,2,0],0.0,"ds/ds has wrong value at (3,1,2,0).")
      self.assertEqual(dsds[3,1,2,1],0.0,"ds/ds has wrong value at (3,1,2,1).")
      self.assertEqual(dsds[3,1,2,2],0.0,"ds/ds has wrong value at (3,1,2,2).")
      self.assertEqual(dsds[3,1,2,3],0.0,"ds/ds has wrong value at (3,1,2,3).")
      self.assertEqual(dsds[3,1,2,4],0.0,"ds/ds has wrong value at (3,1,2,4).")
      self.assertEqual(dsds[3,1,3,0],0.0,"ds/ds has wrong value at (3,1,3,0).")
      self.assertEqual(dsds[3,1,3,1],1.0,"ds/ds has wrong value at (3,1,3,1).")
      self.assertEqual(dsds[3,1,3,2],0.0,"ds/ds has wrong value at (3,1,3,2).")
      self.assertEqual(dsds[3,1,3,3],0.0,"ds/ds has wrong value at (3,1,3,3).")
      self.assertEqual(dsds[3,1,3,4],0.0,"ds/ds has wrong value at (3,1,3,4).")
      self.assertEqual(dsds[3,2,0,0],0.0,"ds/ds has wrong value at (3,2,0,0).")
      self.assertEqual(dsds[3,2,0,1],0.0,"ds/ds has wrong value at (3,2,0,1).")
      self.assertEqual(dsds[3,2,0,2],0.0,"ds/ds has wrong value at (3,2,0,2).")
      self.assertEqual(dsds[3,2,0,3],0.0,"ds/ds has wrong value at (3,2,0,3).")
      self.assertEqual(dsds[3,2,0,4],0.0,"ds/ds has wrong value at (3,2,0,4).")
      self.assertEqual(dsds[3,2,1,0],0.0,"ds/ds has wrong value at (3,2,1,0).")
      self.assertEqual(dsds[3,2,1,1],0.0,"ds/ds has wrong value at (3,2,1,1).")
      self.assertEqual(dsds[3,2,1,2],0.0,"ds/ds has wrong value at (3,2,1,2).")
      self.assertEqual(dsds[3,2,1,3],0.0,"ds/ds has wrong value at (3,2,1,3).")
      self.assertEqual(dsds[3,2,1,4],0.0,"ds/ds has wrong value at (3,2,1,4).")
      self.assertEqual(dsds[3,2,2,0],0.0,"ds/ds has wrong value at (3,2,2,0).")
      self.assertEqual(dsds[3,2,2,1],0.0,"ds/ds has wrong value at (3,2,2,1).")
      self.assertEqual(dsds[3,2,2,2],0.0,"ds/ds has wrong value at (3,2,2,2).")
      self.assertEqual(dsds[3,2,2,3],0.0,"ds/ds has wrong value at (3,2,2,3).")
      self.assertEqual(dsds[3,2,2,4],0.0,"ds/ds has wrong value at (3,2,2,4).")
      self.assertEqual(dsds[3,2,3,0],0.0,"ds/ds has wrong value at (3,2,3,0).")
      self.assertEqual(dsds[3,2,3,1],0.0,"ds/ds has wrong value at (3,2,3,1).")
      self.assertEqual(dsds[3,2,3,2],1.0,"ds/ds has wrong value at (3,2,3,2).")
      self.assertEqual(dsds[3,2,3,3],0.0,"ds/ds has wrong value at (3,2,3,3).")
      self.assertEqual(dsds[3,2,3,4],0.0,"ds/ds has wrong value at (3,2,3,4).")
      self.assertEqual(dsds[3,3,0,0],0.0,"ds/ds has wrong value at (3,3,0,0).")
      self.assertEqual(dsds[3,3,0,1],0.0,"ds/ds has wrong value at (3,3,0,1).")
      self.assertEqual(dsds[3,3,0,2],0.0,"ds/ds has wrong value at (3,3,0,2).")
      self.assertEqual(dsds[3,3,0,3],0.0,"ds/ds has wrong value at (3,3,0,3).")
      self.assertEqual(dsds[3,3,0,4],0.0,"ds/ds has wrong value at (3,3,0,4).")
      self.assertEqual(dsds[3,3,1,0],0.0,"ds/ds has wrong value at (3,3,1,0).")
      self.assertEqual(dsds[3,3,1,1],0.0,"ds/ds has wrong value at (3,3,1,1).")
      self.assertEqual(dsds[3,3,1,2],0.0,"ds/ds has wrong value at (3,3,1,2).")
      self.assertEqual(dsds[3,3,1,3],0.0,"ds/ds has wrong value at (3,3,1,3).")
      self.assertEqual(dsds[3,3,1,4],0.0,"ds/ds has wrong value at (3,3,1,4).")
      self.assertEqual(dsds[3,3,2,0],0.0,"ds/ds has wrong value at (3,3,2,0).")
      self.assertEqual(dsds[3,3,2,1],0.0,"ds/ds has wrong value at (3,3,2,1).")
      self.assertEqual(dsds[3,3,2,2],0.0,"ds/ds has wrong value at (3,3,2,2).")
      self.assertEqual(dsds[3,3,2,3],0.0,"ds/ds has wrong value at (3,3,2,3).")
      self.assertEqual(dsds[3,3,2,4],0.0,"ds/ds has wrong value at (3,3,2,4).")
      self.assertEqual(dsds[3,3,3,0],0.0,"ds/ds has wrong value at (3,3,3,0).")
      self.assertEqual(dsds[3,3,3,1],0.0,"ds/ds has wrong value at (3,3,3,1).")
      self.assertEqual(dsds[3,3,3,2],0.0,"ds/ds has wrong value at (3,3,3,2).")
      self.assertEqual(dsds[3,3,3,3],1.0,"ds/ds has wrong value at (3,3,3,3).")
      self.assertEqual(dsds[3,3,3,4],0.0,"ds/ds has wrong value at (3,3,3,4).")
      self.assertEqual(dsds[3,4,0,0],0.0,"ds/ds has wrong value at (3,4,0,0).")
      self.assertEqual(dsds[3,4,0,1],0.0,"ds/ds has wrong value at (3,4,0,1).")
      self.assertEqual(dsds[3,4,0,2],0.0,"ds/ds has wrong value at (3,4,0,2).")
      self.assertEqual(dsds[3,4,0,3],0.0,"ds/ds has wrong value at (3,4,0,3).")
      self.assertEqual(dsds[3,4,0,4],0.0,"ds/ds has wrong value at (3,4,0,4).")
      self.assertEqual(dsds[3,4,1,0],0.0,"ds/ds has wrong value at (3,4,1,0).")
      self.assertEqual(dsds[3,4,1,1],0.0,"ds/ds has wrong value at (3,4,1,1).")
      self.assertEqual(dsds[3,4,1,2],0.0,"ds/ds has wrong value at (3,4,1,2).")
      self.assertEqual(dsds[3,4,1,3],0.0,"ds/ds has wrong value at (3,4,1,3).")
      self.assertEqual(dsds[3,4,1,4],0.0,"ds/ds has wrong value at (3,4,1,4).")
      self.assertEqual(dsds[3,4,2,0],0.0,"ds/ds has wrong value at (3,4,2,0).")
      self.assertEqual(dsds[3,4,2,1],0.0,"ds/ds has wrong value at (3,4,2,1).")
      self.assertEqual(dsds[3,4,2,2],0.0,"ds/ds has wrong value at (3,4,2,2).")
      self.assertEqual(dsds[3,4,2,3],0.0,"ds/ds has wrong value at (3,4,2,3).")
      self.assertEqual(dsds[3,4,2,4],0.0,"ds/ds has wrong value at (3,4,2,4).")
      self.assertEqual(dsds[3,4,3,0],0.0,"ds/ds has wrong value at (3,4,3,0).")
      self.assertEqual(dsds[3,4,3,1],0.0,"ds/ds has wrong value at (3,4,3,1).")
      self.assertEqual(dsds[3,4,3,2],0.0,"ds/ds has wrong value at (3,4,3,2).")
      self.assertEqual(dsds[3,4,3,3],0.0,"ds/ds has wrong value at (3,4,3,3).")
      self.assertEqual(dsds[3,4,3,4],1.0,"ds/ds has wrong value at (3,4,3,4).")

      self.assertEqual(s.getArgument(0),1,"wrong argument 0.")
      self.assertEqual(s.getArgument(1),-1.0,"wrong argument 1.")
      sa=s.getSubstitutedArguments({s2:-10})
      self.assertEqual(len(sa),2,"wrong number of substituted arguments")
      self.assertEqual(sa[0],1,"wrongly substituted argument 0.")
      self.assertEqual(sa[1],-1.0,"wrongly substituted argument 1.")

      self.assertFalse(s.isAppropriateValue(10.)," 10. is not appropriate substitute")
      self.assertFalse(s.isAppropriateValue(10)," 10 is not appropriate substitute")
      self.assertTrue(s.isAppropriateValue(Symbol(shape=(4, 5),dim=2)),"Symbol is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(Symbol(shape=(4, 5),dim=3)),"Symbol is not appropriate substitute (dim)")
      self.assertFalse(s.isAppropriateValue(escript.Data(0.,(5,),self.functionspace)),"Data is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(numpy.zeros((4, 5))),"ndarray is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(numpy.zeros((5,))),"ndarray is not appropriate substitute (shape)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_Symbol_rank2_d3_nargs0(self):
      s2=Symbol(shape=())
      s=Symbol(shape=(4, 5),dim=3,args=[])
      self.assertEqual(s.getRank(),2,"wrong rank.")
      self.assertEqual(s.getShape(),(4, 5),"wrong shape.")
      self.assertEqual(s.getDim(),3,"wrong spatial dimension.")
      self.assertEqual(s.getArgument(),[],"wrong arguments.")

      ss=s.substitute({s:numpy.zeros((4, 5))})
      self.assertTrue(isinstance(ss,numpy.ndarray),"value after substitution is not ndarray.")
      self.assertEqual(ss.shape,(4, 5),"value after substitution has not expected shape")
      try:
        s.substitute({s:numpy.zeros((5,))})
        fail("illegal substition was successful")
      except TypeError:
        pass

      dsdarg=s.diff(Symbol(shape=()))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds() has wrong type.")
      self.assertEqual(dsdarg.shape,(4, 5),"ds/ds() has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds() has wrong value.")
      dsdarg=s.diff(Symbol(shape=(2,)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(2,) has wrong type.")
      self.assertEqual(dsdarg.shape,(4, 5, 2),"ds/ds(2,) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(2,) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(4, 5)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(4, 5) has wrong type.")
      self.assertEqual(dsdarg.shape,(4, 5, 4, 5),"ds/ds(4, 5) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(4, 5) has wrong value.")

      dsds=s.diff(s)
      self.assertTrue(isinstance(dsds,numpy.ndarray),"ds/ds has wrong type.")
      self.assertEqual(dsds.shape,(4, 5, 4, 5),"ds/ds has wrong shape.")
      self.assertEqual(dsds[0,0,0,0],1.0,"ds/ds has wrong value at (0,0,0,0).")
      self.assertEqual(dsds[0,0,0,1],0.0,"ds/ds has wrong value at (0,0,0,1).")
      self.assertEqual(dsds[0,0,0,2],0.0,"ds/ds has wrong value at (0,0,0,2).")
      self.assertEqual(dsds[0,0,0,3],0.0,"ds/ds has wrong value at (0,0,0,3).")
      self.assertEqual(dsds[0,0,0,4],0.0,"ds/ds has wrong value at (0,0,0,4).")
      self.assertEqual(dsds[0,0,1,0],0.0,"ds/ds has wrong value at (0,0,1,0).")
      self.assertEqual(dsds[0,0,1,1],0.0,"ds/ds has wrong value at (0,0,1,1).")
      self.assertEqual(dsds[0,0,1,2],0.0,"ds/ds has wrong value at (0,0,1,2).")
      self.assertEqual(dsds[0,0,1,3],0.0,"ds/ds has wrong value at (0,0,1,3).")
      self.assertEqual(dsds[0,0,1,4],0.0,"ds/ds has wrong value at (0,0,1,4).")
      self.assertEqual(dsds[0,0,2,0],0.0,"ds/ds has wrong value at (0,0,2,0).")
      self.assertEqual(dsds[0,0,2,1],0.0,"ds/ds has wrong value at (0,0,2,1).")
      self.assertEqual(dsds[0,0,2,2],0.0,"ds/ds has wrong value at (0,0,2,2).")
      self.assertEqual(dsds[0,0,2,3],0.0,"ds/ds has wrong value at (0,0,2,3).")
      self.assertEqual(dsds[0,0,2,4],0.0,"ds/ds has wrong value at (0,0,2,4).")
      self.assertEqual(dsds[0,0,3,0],0.0,"ds/ds has wrong value at (0,0,3,0).")
      self.assertEqual(dsds[0,0,3,1],0.0,"ds/ds has wrong value at (0,0,3,1).")
      self.assertEqual(dsds[0,0,3,2],0.0,"ds/ds has wrong value at (0,0,3,2).")
      self.assertEqual(dsds[0,0,3,3],0.0,"ds/ds has wrong value at (0,0,3,3).")
      self.assertEqual(dsds[0,0,3,4],0.0,"ds/ds has wrong value at (0,0,3,4).")
      self.assertEqual(dsds[0,1,0,0],0.0,"ds/ds has wrong value at (0,1,0,0).")
      self.assertEqual(dsds[0,1,0,1],1.0,"ds/ds has wrong value at (0,1,0,1).")
      self.assertEqual(dsds[0,1,0,2],0.0,"ds/ds has wrong value at (0,1,0,2).")
      self.assertEqual(dsds[0,1,0,3],0.0,"ds/ds has wrong value at (0,1,0,3).")
      self.assertEqual(dsds[0,1,0,4],0.0,"ds/ds has wrong value at (0,1,0,4).")
      self.assertEqual(dsds[0,1,1,0],0.0,"ds/ds has wrong value at (0,1,1,0).")
      self.assertEqual(dsds[0,1,1,1],0.0,"ds/ds has wrong value at (0,1,1,1).")
      self.assertEqual(dsds[0,1,1,2],0.0,"ds/ds has wrong value at (0,1,1,2).")
      self.assertEqual(dsds[0,1,1,3],0.0,"ds/ds has wrong value at (0,1,1,3).")
      self.assertEqual(dsds[0,1,1,4],0.0,"ds/ds has wrong value at (0,1,1,4).")
      self.assertEqual(dsds[0,1,2,0],0.0,"ds/ds has wrong value at (0,1,2,0).")
      self.assertEqual(dsds[0,1,2,1],0.0,"ds/ds has wrong value at (0,1,2,1).")
      self.assertEqual(dsds[0,1,2,2],0.0,"ds/ds has wrong value at (0,1,2,2).")
      self.assertEqual(dsds[0,1,2,3],0.0,"ds/ds has wrong value at (0,1,2,3).")
      self.assertEqual(dsds[0,1,2,4],0.0,"ds/ds has wrong value at (0,1,2,4).")
      self.assertEqual(dsds[0,1,3,0],0.0,"ds/ds has wrong value at (0,1,3,0).")
      self.assertEqual(dsds[0,1,3,1],0.0,"ds/ds has wrong value at (0,1,3,1).")
      self.assertEqual(dsds[0,1,3,2],0.0,"ds/ds has wrong value at (0,1,3,2).")
      self.assertEqual(dsds[0,1,3,3],0.0,"ds/ds has wrong value at (0,1,3,3).")
      self.assertEqual(dsds[0,1,3,4],0.0,"ds/ds has wrong value at (0,1,3,4).")
      self.assertEqual(dsds[0,2,0,0],0.0,"ds/ds has wrong value at (0,2,0,0).")
      self.assertEqual(dsds[0,2,0,1],0.0,"ds/ds has wrong value at (0,2,0,1).")
      self.assertEqual(dsds[0,2,0,2],1.0,"ds/ds has wrong value at (0,2,0,2).")
      self.assertEqual(dsds[0,2,0,3],0.0,"ds/ds has wrong value at (0,2,0,3).")
      self.assertEqual(dsds[0,2,0,4],0.0,"ds/ds has wrong value at (0,2,0,4).")
      self.assertEqual(dsds[0,2,1,0],0.0,"ds/ds has wrong value at (0,2,1,0).")
      self.assertEqual(dsds[0,2,1,1],0.0,"ds/ds has wrong value at (0,2,1,1).")
      self.assertEqual(dsds[0,2,1,2],0.0,"ds/ds has wrong value at (0,2,1,2).")
      self.assertEqual(dsds[0,2,1,3],0.0,"ds/ds has wrong value at (0,2,1,3).")
      self.assertEqual(dsds[0,2,1,4],0.0,"ds/ds has wrong value at (0,2,1,4).")
      self.assertEqual(dsds[0,2,2,0],0.0,"ds/ds has wrong value at (0,2,2,0).")
      self.assertEqual(dsds[0,2,2,1],0.0,"ds/ds has wrong value at (0,2,2,1).")
      self.assertEqual(dsds[0,2,2,2],0.0,"ds/ds has wrong value at (0,2,2,2).")
      self.assertEqual(dsds[0,2,2,3],0.0,"ds/ds has wrong value at (0,2,2,3).")
      self.assertEqual(dsds[0,2,2,4],0.0,"ds/ds has wrong value at (0,2,2,4).")
      self.assertEqual(dsds[0,2,3,0],0.0,"ds/ds has wrong value at (0,2,3,0).")
      self.assertEqual(dsds[0,2,3,1],0.0,"ds/ds has wrong value at (0,2,3,1).")
      self.assertEqual(dsds[0,2,3,2],0.0,"ds/ds has wrong value at (0,2,3,2).")
      self.assertEqual(dsds[0,2,3,3],0.0,"ds/ds has wrong value at (0,2,3,3).")
      self.assertEqual(dsds[0,2,3,4],0.0,"ds/ds has wrong value at (0,2,3,4).")
      self.assertEqual(dsds[0,3,0,0],0.0,"ds/ds has wrong value at (0,3,0,0).")
      self.assertEqual(dsds[0,3,0,1],0.0,"ds/ds has wrong value at (0,3,0,1).")
      self.assertEqual(dsds[0,3,0,2],0.0,"ds/ds has wrong value at (0,3,0,2).")
      self.assertEqual(dsds[0,3,0,3],1.0,"ds/ds has wrong value at (0,3,0,3).")
      self.assertEqual(dsds[0,3,0,4],0.0,"ds/ds has wrong value at (0,3,0,4).")
      self.assertEqual(dsds[0,3,1,0],0.0,"ds/ds has wrong value at (0,3,1,0).")
      self.assertEqual(dsds[0,3,1,1],0.0,"ds/ds has wrong value at (0,3,1,1).")
      self.assertEqual(dsds[0,3,1,2],0.0,"ds/ds has wrong value at (0,3,1,2).")
      self.assertEqual(dsds[0,3,1,3],0.0,"ds/ds has wrong value at (0,3,1,3).")
      self.assertEqual(dsds[0,3,1,4],0.0,"ds/ds has wrong value at (0,3,1,4).")
      self.assertEqual(dsds[0,3,2,0],0.0,"ds/ds has wrong value at (0,3,2,0).")
      self.assertEqual(dsds[0,3,2,1],0.0,"ds/ds has wrong value at (0,3,2,1).")
      self.assertEqual(dsds[0,3,2,2],0.0,"ds/ds has wrong value at (0,3,2,2).")
      self.assertEqual(dsds[0,3,2,3],0.0,"ds/ds has wrong value at (0,3,2,3).")
      self.assertEqual(dsds[0,3,2,4],0.0,"ds/ds has wrong value at (0,3,2,4).")
      self.assertEqual(dsds[0,3,3,0],0.0,"ds/ds has wrong value at (0,3,3,0).")
      self.assertEqual(dsds[0,3,3,1],0.0,"ds/ds has wrong value at (0,3,3,1).")
      self.assertEqual(dsds[0,3,3,2],0.0,"ds/ds has wrong value at (0,3,3,2).")
      self.assertEqual(dsds[0,3,3,3],0.0,"ds/ds has wrong value at (0,3,3,3).")
      self.assertEqual(dsds[0,3,3,4],0.0,"ds/ds has wrong value at (0,3,3,4).")
      self.assertEqual(dsds[0,4,0,0],0.0,"ds/ds has wrong value at (0,4,0,0).")
      self.assertEqual(dsds[0,4,0,1],0.0,"ds/ds has wrong value at (0,4,0,1).")
      self.assertEqual(dsds[0,4,0,2],0.0,"ds/ds has wrong value at (0,4,0,2).")
      self.assertEqual(dsds[0,4,0,3],0.0,"ds/ds has wrong value at (0,4,0,3).")
      self.assertEqual(dsds[0,4,0,4],1.0,"ds/ds has wrong value at (0,4,0,4).")
      self.assertEqual(dsds[0,4,1,0],0.0,"ds/ds has wrong value at (0,4,1,0).")
      self.assertEqual(dsds[0,4,1,1],0.0,"ds/ds has wrong value at (0,4,1,1).")
      self.assertEqual(dsds[0,4,1,2],0.0,"ds/ds has wrong value at (0,4,1,2).")
      self.assertEqual(dsds[0,4,1,3],0.0,"ds/ds has wrong value at (0,4,1,3).")
      self.assertEqual(dsds[0,4,1,4],0.0,"ds/ds has wrong value at (0,4,1,4).")
      self.assertEqual(dsds[0,4,2,0],0.0,"ds/ds has wrong value at (0,4,2,0).")
      self.assertEqual(dsds[0,4,2,1],0.0,"ds/ds has wrong value at (0,4,2,1).")
      self.assertEqual(dsds[0,4,2,2],0.0,"ds/ds has wrong value at (0,4,2,2).")
      self.assertEqual(dsds[0,4,2,3],0.0,"ds/ds has wrong value at (0,4,2,3).")
      self.assertEqual(dsds[0,4,2,4],0.0,"ds/ds has wrong value at (0,4,2,4).")
      self.assertEqual(dsds[0,4,3,0],0.0,"ds/ds has wrong value at (0,4,3,0).")
      self.assertEqual(dsds[0,4,3,1],0.0,"ds/ds has wrong value at (0,4,3,1).")
      self.assertEqual(dsds[0,4,3,2],0.0,"ds/ds has wrong value at (0,4,3,2).")
      self.assertEqual(dsds[0,4,3,3],0.0,"ds/ds has wrong value at (0,4,3,3).")
      self.assertEqual(dsds[0,4,3,4],0.0,"ds/ds has wrong value at (0,4,3,4).")
      self.assertEqual(dsds[1,0,0,0],0.0,"ds/ds has wrong value at (1,0,0,0).")
      self.assertEqual(dsds[1,0,0,1],0.0,"ds/ds has wrong value at (1,0,0,1).")
      self.assertEqual(dsds[1,0,0,2],0.0,"ds/ds has wrong value at (1,0,0,2).")
      self.assertEqual(dsds[1,0,0,3],0.0,"ds/ds has wrong value at (1,0,0,3).")
      self.assertEqual(dsds[1,0,0,4],0.0,"ds/ds has wrong value at (1,0,0,4).")
      self.assertEqual(dsds[1,0,1,0],1.0,"ds/ds has wrong value at (1,0,1,0).")
      self.assertEqual(dsds[1,0,1,1],0.0,"ds/ds has wrong value at (1,0,1,1).")
      self.assertEqual(dsds[1,0,1,2],0.0,"ds/ds has wrong value at (1,0,1,2).")
      self.assertEqual(dsds[1,0,1,3],0.0,"ds/ds has wrong value at (1,0,1,3).")
      self.assertEqual(dsds[1,0,1,4],0.0,"ds/ds has wrong value at (1,0,1,4).")
      self.assertEqual(dsds[1,0,2,0],0.0,"ds/ds has wrong value at (1,0,2,0).")
      self.assertEqual(dsds[1,0,2,1],0.0,"ds/ds has wrong value at (1,0,2,1).")
      self.assertEqual(dsds[1,0,2,2],0.0,"ds/ds has wrong value at (1,0,2,2).")
      self.assertEqual(dsds[1,0,2,3],0.0,"ds/ds has wrong value at (1,0,2,3).")
      self.assertEqual(dsds[1,0,2,4],0.0,"ds/ds has wrong value at (1,0,2,4).")
      self.assertEqual(dsds[1,0,3,0],0.0,"ds/ds has wrong value at (1,0,3,0).")
      self.assertEqual(dsds[1,0,3,1],0.0,"ds/ds has wrong value at (1,0,3,1).")
      self.assertEqual(dsds[1,0,3,2],0.0,"ds/ds has wrong value at (1,0,3,2).")
      self.assertEqual(dsds[1,0,3,3],0.0,"ds/ds has wrong value at (1,0,3,3).")
      self.assertEqual(dsds[1,0,3,4],0.0,"ds/ds has wrong value at (1,0,3,4).")
      self.assertEqual(dsds[1,1,0,0],0.0,"ds/ds has wrong value at (1,1,0,0).")
      self.assertEqual(dsds[1,1,0,1],0.0,"ds/ds has wrong value at (1,1,0,1).")
      self.assertEqual(dsds[1,1,0,2],0.0,"ds/ds has wrong value at (1,1,0,2).")
      self.assertEqual(dsds[1,1,0,3],0.0,"ds/ds has wrong value at (1,1,0,3).")
      self.assertEqual(dsds[1,1,0,4],0.0,"ds/ds has wrong value at (1,1,0,4).")
      self.assertEqual(dsds[1,1,1,0],0.0,"ds/ds has wrong value at (1,1,1,0).")
      self.assertEqual(dsds[1,1,1,1],1.0,"ds/ds has wrong value at (1,1,1,1).")
      self.assertEqual(dsds[1,1,1,2],0.0,"ds/ds has wrong value at (1,1,1,2).")
      self.assertEqual(dsds[1,1,1,3],0.0,"ds/ds has wrong value at (1,1,1,3).")
      self.assertEqual(dsds[1,1,1,4],0.0,"ds/ds has wrong value at (1,1,1,4).")
      self.assertEqual(dsds[1,1,2,0],0.0,"ds/ds has wrong value at (1,1,2,0).")
      self.assertEqual(dsds[1,1,2,1],0.0,"ds/ds has wrong value at (1,1,2,1).")
      self.assertEqual(dsds[1,1,2,2],0.0,"ds/ds has wrong value at (1,1,2,2).")
      self.assertEqual(dsds[1,1,2,3],0.0,"ds/ds has wrong value at (1,1,2,3).")
      self.assertEqual(dsds[1,1,2,4],0.0,"ds/ds has wrong value at (1,1,2,4).")
      self.assertEqual(dsds[1,1,3,0],0.0,"ds/ds has wrong value at (1,1,3,0).")
      self.assertEqual(dsds[1,1,3,1],0.0,"ds/ds has wrong value at (1,1,3,1).")
      self.assertEqual(dsds[1,1,3,2],0.0,"ds/ds has wrong value at (1,1,3,2).")
      self.assertEqual(dsds[1,1,3,3],0.0,"ds/ds has wrong value at (1,1,3,3).")
      self.assertEqual(dsds[1,1,3,4],0.0,"ds/ds has wrong value at (1,1,3,4).")
      self.assertEqual(dsds[1,2,0,0],0.0,"ds/ds has wrong value at (1,2,0,0).")
      self.assertEqual(dsds[1,2,0,1],0.0,"ds/ds has wrong value at (1,2,0,1).")
      self.assertEqual(dsds[1,2,0,2],0.0,"ds/ds has wrong value at (1,2,0,2).")
      self.assertEqual(dsds[1,2,0,3],0.0,"ds/ds has wrong value at (1,2,0,3).")
      self.assertEqual(dsds[1,2,0,4],0.0,"ds/ds has wrong value at (1,2,0,4).")
      self.assertEqual(dsds[1,2,1,0],0.0,"ds/ds has wrong value at (1,2,1,0).")
      self.assertEqual(dsds[1,2,1,1],0.0,"ds/ds has wrong value at (1,2,1,1).")
      self.assertEqual(dsds[1,2,1,2],1.0,"ds/ds has wrong value at (1,2,1,2).")
      self.assertEqual(dsds[1,2,1,3],0.0,"ds/ds has wrong value at (1,2,1,3).")
      self.assertEqual(dsds[1,2,1,4],0.0,"ds/ds has wrong value at (1,2,1,4).")
      self.assertEqual(dsds[1,2,2,0],0.0,"ds/ds has wrong value at (1,2,2,0).")
      self.assertEqual(dsds[1,2,2,1],0.0,"ds/ds has wrong value at (1,2,2,1).")
      self.assertEqual(dsds[1,2,2,2],0.0,"ds/ds has wrong value at (1,2,2,2).")
      self.assertEqual(dsds[1,2,2,3],0.0,"ds/ds has wrong value at (1,2,2,3).")
      self.assertEqual(dsds[1,2,2,4],0.0,"ds/ds has wrong value at (1,2,2,4).")
      self.assertEqual(dsds[1,2,3,0],0.0,"ds/ds has wrong value at (1,2,3,0).")
      self.assertEqual(dsds[1,2,3,1],0.0,"ds/ds has wrong value at (1,2,3,1).")
      self.assertEqual(dsds[1,2,3,2],0.0,"ds/ds has wrong value at (1,2,3,2).")
      self.assertEqual(dsds[1,2,3,3],0.0,"ds/ds has wrong value at (1,2,3,3).")
      self.assertEqual(dsds[1,2,3,4],0.0,"ds/ds has wrong value at (1,2,3,4).")
      self.assertEqual(dsds[1,3,0,0],0.0,"ds/ds has wrong value at (1,3,0,0).")
      self.assertEqual(dsds[1,3,0,1],0.0,"ds/ds has wrong value at (1,3,0,1).")
      self.assertEqual(dsds[1,3,0,2],0.0,"ds/ds has wrong value at (1,3,0,2).")
      self.assertEqual(dsds[1,3,0,3],0.0,"ds/ds has wrong value at (1,3,0,3).")
      self.assertEqual(dsds[1,3,0,4],0.0,"ds/ds has wrong value at (1,3,0,4).")
      self.assertEqual(dsds[1,3,1,0],0.0,"ds/ds has wrong value at (1,3,1,0).")
      self.assertEqual(dsds[1,3,1,1],0.0,"ds/ds has wrong value at (1,3,1,1).")
      self.assertEqual(dsds[1,3,1,2],0.0,"ds/ds has wrong value at (1,3,1,2).")
      self.assertEqual(dsds[1,3,1,3],1.0,"ds/ds has wrong value at (1,3,1,3).")
      self.assertEqual(dsds[1,3,1,4],0.0,"ds/ds has wrong value at (1,3,1,4).")
      self.assertEqual(dsds[1,3,2,0],0.0,"ds/ds has wrong value at (1,3,2,0).")
      self.assertEqual(dsds[1,3,2,1],0.0,"ds/ds has wrong value at (1,3,2,1).")
      self.assertEqual(dsds[1,3,2,2],0.0,"ds/ds has wrong value at (1,3,2,2).")
      self.assertEqual(dsds[1,3,2,3],0.0,"ds/ds has wrong value at (1,3,2,3).")
      self.assertEqual(dsds[1,3,2,4],0.0,"ds/ds has wrong value at (1,3,2,4).")
      self.assertEqual(dsds[1,3,3,0],0.0,"ds/ds has wrong value at (1,3,3,0).")
      self.assertEqual(dsds[1,3,3,1],0.0,"ds/ds has wrong value at (1,3,3,1).")
      self.assertEqual(dsds[1,3,3,2],0.0,"ds/ds has wrong value at (1,3,3,2).")
      self.assertEqual(dsds[1,3,3,3],0.0,"ds/ds has wrong value at (1,3,3,3).")
      self.assertEqual(dsds[1,3,3,4],0.0,"ds/ds has wrong value at (1,3,3,4).")
      self.assertEqual(dsds[1,4,0,0],0.0,"ds/ds has wrong value at (1,4,0,0).")
      self.assertEqual(dsds[1,4,0,1],0.0,"ds/ds has wrong value at (1,4,0,1).")
      self.assertEqual(dsds[1,4,0,2],0.0,"ds/ds has wrong value at (1,4,0,2).")
      self.assertEqual(dsds[1,4,0,3],0.0,"ds/ds has wrong value at (1,4,0,3).")
      self.assertEqual(dsds[1,4,0,4],0.0,"ds/ds has wrong value at (1,4,0,4).")
      self.assertEqual(dsds[1,4,1,0],0.0,"ds/ds has wrong value at (1,4,1,0).")
      self.assertEqual(dsds[1,4,1,1],0.0,"ds/ds has wrong value at (1,4,1,1).")
      self.assertEqual(dsds[1,4,1,2],0.0,"ds/ds has wrong value at (1,4,1,2).")
      self.assertEqual(dsds[1,4,1,3],0.0,"ds/ds has wrong value at (1,4,1,3).")
      self.assertEqual(dsds[1,4,1,4],1.0,"ds/ds has wrong value at (1,4,1,4).")
      self.assertEqual(dsds[1,4,2,0],0.0,"ds/ds has wrong value at (1,4,2,0).")
      self.assertEqual(dsds[1,4,2,1],0.0,"ds/ds has wrong value at (1,4,2,1).")
      self.assertEqual(dsds[1,4,2,2],0.0,"ds/ds has wrong value at (1,4,2,2).")
      self.assertEqual(dsds[1,4,2,3],0.0,"ds/ds has wrong value at (1,4,2,3).")
      self.assertEqual(dsds[1,4,2,4],0.0,"ds/ds has wrong value at (1,4,2,4).")
      self.assertEqual(dsds[1,4,3,0],0.0,"ds/ds has wrong value at (1,4,3,0).")
      self.assertEqual(dsds[1,4,3,1],0.0,"ds/ds has wrong value at (1,4,3,1).")
      self.assertEqual(dsds[1,4,3,2],0.0,"ds/ds has wrong value at (1,4,3,2).")
      self.assertEqual(dsds[1,4,3,3],0.0,"ds/ds has wrong value at (1,4,3,3).")
      self.assertEqual(dsds[1,4,3,4],0.0,"ds/ds has wrong value at (1,4,3,4).")
      self.assertEqual(dsds[2,0,0,0],0.0,"ds/ds has wrong value at (2,0,0,0).")
      self.assertEqual(dsds[2,0,0,1],0.0,"ds/ds has wrong value at (2,0,0,1).")
      self.assertEqual(dsds[2,0,0,2],0.0,"ds/ds has wrong value at (2,0,0,2).")
      self.assertEqual(dsds[2,0,0,3],0.0,"ds/ds has wrong value at (2,0,0,3).")
      self.assertEqual(dsds[2,0,0,4],0.0,"ds/ds has wrong value at (2,0,0,4).")
      self.assertEqual(dsds[2,0,1,0],0.0,"ds/ds has wrong value at (2,0,1,0).")
      self.assertEqual(dsds[2,0,1,1],0.0,"ds/ds has wrong value at (2,0,1,1).")
      self.assertEqual(dsds[2,0,1,2],0.0,"ds/ds has wrong value at (2,0,1,2).")
      self.assertEqual(dsds[2,0,1,3],0.0,"ds/ds has wrong value at (2,0,1,3).")
      self.assertEqual(dsds[2,0,1,4],0.0,"ds/ds has wrong value at (2,0,1,4).")
      self.assertEqual(dsds[2,0,2,0],1.0,"ds/ds has wrong value at (2,0,2,0).")
      self.assertEqual(dsds[2,0,2,1],0.0,"ds/ds has wrong value at (2,0,2,1).")
      self.assertEqual(dsds[2,0,2,2],0.0,"ds/ds has wrong value at (2,0,2,2).")
      self.assertEqual(dsds[2,0,2,3],0.0,"ds/ds has wrong value at (2,0,2,3).")
      self.assertEqual(dsds[2,0,2,4],0.0,"ds/ds has wrong value at (2,0,2,4).")
      self.assertEqual(dsds[2,0,3,0],0.0,"ds/ds has wrong value at (2,0,3,0).")
      self.assertEqual(dsds[2,0,3,1],0.0,"ds/ds has wrong value at (2,0,3,1).")
      self.assertEqual(dsds[2,0,3,2],0.0,"ds/ds has wrong value at (2,0,3,2).")
      self.assertEqual(dsds[2,0,3,3],0.0,"ds/ds has wrong value at (2,0,3,3).")
      self.assertEqual(dsds[2,0,3,4],0.0,"ds/ds has wrong value at (2,0,3,4).")
      self.assertEqual(dsds[2,1,0,0],0.0,"ds/ds has wrong value at (2,1,0,0).")
      self.assertEqual(dsds[2,1,0,1],0.0,"ds/ds has wrong value at (2,1,0,1).")
      self.assertEqual(dsds[2,1,0,2],0.0,"ds/ds has wrong value at (2,1,0,2).")
      self.assertEqual(dsds[2,1,0,3],0.0,"ds/ds has wrong value at (2,1,0,3).")
      self.assertEqual(dsds[2,1,0,4],0.0,"ds/ds has wrong value at (2,1,0,4).")
      self.assertEqual(dsds[2,1,1,0],0.0,"ds/ds has wrong value at (2,1,1,0).")
      self.assertEqual(dsds[2,1,1,1],0.0,"ds/ds has wrong value at (2,1,1,1).")
      self.assertEqual(dsds[2,1,1,2],0.0,"ds/ds has wrong value at (2,1,1,2).")
      self.assertEqual(dsds[2,1,1,3],0.0,"ds/ds has wrong value at (2,1,1,3).")
      self.assertEqual(dsds[2,1,1,4],0.0,"ds/ds has wrong value at (2,1,1,4).")
      self.assertEqual(dsds[2,1,2,0],0.0,"ds/ds has wrong value at (2,1,2,0).")
      self.assertEqual(dsds[2,1,2,1],1.0,"ds/ds has wrong value at (2,1,2,1).")
      self.assertEqual(dsds[2,1,2,2],0.0,"ds/ds has wrong value at (2,1,2,2).")
      self.assertEqual(dsds[2,1,2,3],0.0,"ds/ds has wrong value at (2,1,2,3).")
      self.assertEqual(dsds[2,1,2,4],0.0,"ds/ds has wrong value at (2,1,2,4).")
      self.assertEqual(dsds[2,1,3,0],0.0,"ds/ds has wrong value at (2,1,3,0).")
      self.assertEqual(dsds[2,1,3,1],0.0,"ds/ds has wrong value at (2,1,3,1).")
      self.assertEqual(dsds[2,1,3,2],0.0,"ds/ds has wrong value at (2,1,3,2).")
      self.assertEqual(dsds[2,1,3,3],0.0,"ds/ds has wrong value at (2,1,3,3).")
      self.assertEqual(dsds[2,1,3,4],0.0,"ds/ds has wrong value at (2,1,3,4).")
      self.assertEqual(dsds[2,2,0,0],0.0,"ds/ds has wrong value at (2,2,0,0).")
      self.assertEqual(dsds[2,2,0,1],0.0,"ds/ds has wrong value at (2,2,0,1).")
      self.assertEqual(dsds[2,2,0,2],0.0,"ds/ds has wrong value at (2,2,0,2).")
      self.assertEqual(dsds[2,2,0,3],0.0,"ds/ds has wrong value at (2,2,0,3).")
      self.assertEqual(dsds[2,2,0,4],0.0,"ds/ds has wrong value at (2,2,0,4).")
      self.assertEqual(dsds[2,2,1,0],0.0,"ds/ds has wrong value at (2,2,1,0).")
      self.assertEqual(dsds[2,2,1,1],0.0,"ds/ds has wrong value at (2,2,1,1).")
      self.assertEqual(dsds[2,2,1,2],0.0,"ds/ds has wrong value at (2,2,1,2).")
      self.assertEqual(dsds[2,2,1,3],0.0,"ds/ds has wrong value at (2,2,1,3).")
      self.assertEqual(dsds[2,2,1,4],0.0,"ds/ds has wrong value at (2,2,1,4).")
      self.assertEqual(dsds[2,2,2,0],0.0,"ds/ds has wrong value at (2,2,2,0).")
      self.assertEqual(dsds[2,2,2,1],0.0,"ds/ds has wrong value at (2,2,2,1).")
      self.assertEqual(dsds[2,2,2,2],1.0,"ds/ds has wrong value at (2,2,2,2).")
      self.assertEqual(dsds[2,2,2,3],0.0,"ds/ds has wrong value at (2,2,2,3).")
      self.assertEqual(dsds[2,2,2,4],0.0,"ds/ds has wrong value at (2,2,2,4).")
      self.assertEqual(dsds[2,2,3,0],0.0,"ds/ds has wrong value at (2,2,3,0).")
      self.assertEqual(dsds[2,2,3,1],0.0,"ds/ds has wrong value at (2,2,3,1).")
      self.assertEqual(dsds[2,2,3,2],0.0,"ds/ds has wrong value at (2,2,3,2).")
      self.assertEqual(dsds[2,2,3,3],0.0,"ds/ds has wrong value at (2,2,3,3).")
      self.assertEqual(dsds[2,2,3,4],0.0,"ds/ds has wrong value at (2,2,3,4).")
      self.assertEqual(dsds[2,3,0,0],0.0,"ds/ds has wrong value at (2,3,0,0).")
      self.assertEqual(dsds[2,3,0,1],0.0,"ds/ds has wrong value at (2,3,0,1).")
      self.assertEqual(dsds[2,3,0,2],0.0,"ds/ds has wrong value at (2,3,0,2).")
      self.assertEqual(dsds[2,3,0,3],0.0,"ds/ds has wrong value at (2,3,0,3).")
      self.assertEqual(dsds[2,3,0,4],0.0,"ds/ds has wrong value at (2,3,0,4).")
      self.assertEqual(dsds[2,3,1,0],0.0,"ds/ds has wrong value at (2,3,1,0).")
      self.assertEqual(dsds[2,3,1,1],0.0,"ds/ds has wrong value at (2,3,1,1).")
      self.assertEqual(dsds[2,3,1,2],0.0,"ds/ds has wrong value at (2,3,1,2).")
      self.assertEqual(dsds[2,3,1,3],0.0,"ds/ds has wrong value at (2,3,1,3).")
      self.assertEqual(dsds[2,3,1,4],0.0,"ds/ds has wrong value at (2,3,1,4).")
      self.assertEqual(dsds[2,3,2,0],0.0,"ds/ds has wrong value at (2,3,2,0).")
      self.assertEqual(dsds[2,3,2,1],0.0,"ds/ds has wrong value at (2,3,2,1).")
      self.assertEqual(dsds[2,3,2,2],0.0,"ds/ds has wrong value at (2,3,2,2).")
      self.assertEqual(dsds[2,3,2,3],1.0,"ds/ds has wrong value at (2,3,2,3).")
      self.assertEqual(dsds[2,3,2,4],0.0,"ds/ds has wrong value at (2,3,2,4).")
      self.assertEqual(dsds[2,3,3,0],0.0,"ds/ds has wrong value at (2,3,3,0).")
      self.assertEqual(dsds[2,3,3,1],0.0,"ds/ds has wrong value at (2,3,3,1).")
      self.assertEqual(dsds[2,3,3,2],0.0,"ds/ds has wrong value at (2,3,3,2).")
      self.assertEqual(dsds[2,3,3,3],0.0,"ds/ds has wrong value at (2,3,3,3).")
      self.assertEqual(dsds[2,3,3,4],0.0,"ds/ds has wrong value at (2,3,3,4).")
      self.assertEqual(dsds[2,4,0,0],0.0,"ds/ds has wrong value at (2,4,0,0).")
      self.assertEqual(dsds[2,4,0,1],0.0,"ds/ds has wrong value at (2,4,0,1).")
      self.assertEqual(dsds[2,4,0,2],0.0,"ds/ds has wrong value at (2,4,0,2).")
      self.assertEqual(dsds[2,4,0,3],0.0,"ds/ds has wrong value at (2,4,0,3).")
      self.assertEqual(dsds[2,4,0,4],0.0,"ds/ds has wrong value at (2,4,0,4).")
      self.assertEqual(dsds[2,4,1,0],0.0,"ds/ds has wrong value at (2,4,1,0).")
      self.assertEqual(dsds[2,4,1,1],0.0,"ds/ds has wrong value at (2,4,1,1).")
      self.assertEqual(dsds[2,4,1,2],0.0,"ds/ds has wrong value at (2,4,1,2).")
      self.assertEqual(dsds[2,4,1,3],0.0,"ds/ds has wrong value at (2,4,1,3).")
      self.assertEqual(dsds[2,4,1,4],0.0,"ds/ds has wrong value at (2,4,1,4).")
      self.assertEqual(dsds[2,4,2,0],0.0,"ds/ds has wrong value at (2,4,2,0).")
      self.assertEqual(dsds[2,4,2,1],0.0,"ds/ds has wrong value at (2,4,2,1).")
      self.assertEqual(dsds[2,4,2,2],0.0,"ds/ds has wrong value at (2,4,2,2).")
      self.assertEqual(dsds[2,4,2,3],0.0,"ds/ds has wrong value at (2,4,2,3).")
      self.assertEqual(dsds[2,4,2,4],1.0,"ds/ds has wrong value at (2,4,2,4).")
      self.assertEqual(dsds[2,4,3,0],0.0,"ds/ds has wrong value at (2,4,3,0).")
      self.assertEqual(dsds[2,4,3,1],0.0,"ds/ds has wrong value at (2,4,3,1).")
      self.assertEqual(dsds[2,4,3,2],0.0,"ds/ds has wrong value at (2,4,3,2).")
      self.assertEqual(dsds[2,4,3,3],0.0,"ds/ds has wrong value at (2,4,3,3).")
      self.assertEqual(dsds[2,4,3,4],0.0,"ds/ds has wrong value at (2,4,3,4).")
      self.assertEqual(dsds[3,0,0,0],0.0,"ds/ds has wrong value at (3,0,0,0).")
      self.assertEqual(dsds[3,0,0,1],0.0,"ds/ds has wrong value at (3,0,0,1).")
      self.assertEqual(dsds[3,0,0,2],0.0,"ds/ds has wrong value at (3,0,0,2).")
      self.assertEqual(dsds[3,0,0,3],0.0,"ds/ds has wrong value at (3,0,0,3).")
      self.assertEqual(dsds[3,0,0,4],0.0,"ds/ds has wrong value at (3,0,0,4).")
      self.assertEqual(dsds[3,0,1,0],0.0,"ds/ds has wrong value at (3,0,1,0).")
      self.assertEqual(dsds[3,0,1,1],0.0,"ds/ds has wrong value at (3,0,1,1).")
      self.assertEqual(dsds[3,0,1,2],0.0,"ds/ds has wrong value at (3,0,1,2).")
      self.assertEqual(dsds[3,0,1,3],0.0,"ds/ds has wrong value at (3,0,1,3).")
      self.assertEqual(dsds[3,0,1,4],0.0,"ds/ds has wrong value at (3,0,1,4).")
      self.assertEqual(dsds[3,0,2,0],0.0,"ds/ds has wrong value at (3,0,2,0).")
      self.assertEqual(dsds[3,0,2,1],0.0,"ds/ds has wrong value at (3,0,2,1).")
      self.assertEqual(dsds[3,0,2,2],0.0,"ds/ds has wrong value at (3,0,2,2).")
      self.assertEqual(dsds[3,0,2,3],0.0,"ds/ds has wrong value at (3,0,2,3).")
      self.assertEqual(dsds[3,0,2,4],0.0,"ds/ds has wrong value at (3,0,2,4).")
      self.assertEqual(dsds[3,0,3,0],1.0,"ds/ds has wrong value at (3,0,3,0).")
      self.assertEqual(dsds[3,0,3,1],0.0,"ds/ds has wrong value at (3,0,3,1).")
      self.assertEqual(dsds[3,0,3,2],0.0,"ds/ds has wrong value at (3,0,3,2).")
      self.assertEqual(dsds[3,0,3,3],0.0,"ds/ds has wrong value at (3,0,3,3).")
      self.assertEqual(dsds[3,0,3,4],0.0,"ds/ds has wrong value at (3,0,3,4).")
      self.assertEqual(dsds[3,1,0,0],0.0,"ds/ds has wrong value at (3,1,0,0).")
      self.assertEqual(dsds[3,1,0,1],0.0,"ds/ds has wrong value at (3,1,0,1).")
      self.assertEqual(dsds[3,1,0,2],0.0,"ds/ds has wrong value at (3,1,0,2).")
      self.assertEqual(dsds[3,1,0,3],0.0,"ds/ds has wrong value at (3,1,0,3).")
      self.assertEqual(dsds[3,1,0,4],0.0,"ds/ds has wrong value at (3,1,0,4).")
      self.assertEqual(dsds[3,1,1,0],0.0,"ds/ds has wrong value at (3,1,1,0).")
      self.assertEqual(dsds[3,1,1,1],0.0,"ds/ds has wrong value at (3,1,1,1).")
      self.assertEqual(dsds[3,1,1,2],0.0,"ds/ds has wrong value at (3,1,1,2).")
      self.assertEqual(dsds[3,1,1,3],0.0,"ds/ds has wrong value at (3,1,1,3).")
      self.assertEqual(dsds[3,1,1,4],0.0,"ds/ds has wrong value at (3,1,1,4).")
      self.assertEqual(dsds[3,1,2,0],0.0,"ds/ds has wrong value at (3,1,2,0).")
      self.assertEqual(dsds[3,1,2,1],0.0,"ds/ds has wrong value at (3,1,2,1).")
      self.assertEqual(dsds[3,1,2,2],0.0,"ds/ds has wrong value at (3,1,2,2).")
      self.assertEqual(dsds[3,1,2,3],0.0,"ds/ds has wrong value at (3,1,2,3).")
      self.assertEqual(dsds[3,1,2,4],0.0,"ds/ds has wrong value at (3,1,2,4).")
      self.assertEqual(dsds[3,1,3,0],0.0,"ds/ds has wrong value at (3,1,3,0).")
      self.assertEqual(dsds[3,1,3,1],1.0,"ds/ds has wrong value at (3,1,3,1).")
      self.assertEqual(dsds[3,1,3,2],0.0,"ds/ds has wrong value at (3,1,3,2).")
      self.assertEqual(dsds[3,1,3,3],0.0,"ds/ds has wrong value at (3,1,3,3).")
      self.assertEqual(dsds[3,1,3,4],0.0,"ds/ds has wrong value at (3,1,3,4).")
      self.assertEqual(dsds[3,2,0,0],0.0,"ds/ds has wrong value at (3,2,0,0).")
      self.assertEqual(dsds[3,2,0,1],0.0,"ds/ds has wrong value at (3,2,0,1).")
      self.assertEqual(dsds[3,2,0,2],0.0,"ds/ds has wrong value at (3,2,0,2).")
      self.assertEqual(dsds[3,2,0,3],0.0,"ds/ds has wrong value at (3,2,0,3).")
      self.assertEqual(dsds[3,2,0,4],0.0,"ds/ds has wrong value at (3,2,0,4).")
      self.assertEqual(dsds[3,2,1,0],0.0,"ds/ds has wrong value at (3,2,1,0).")
      self.assertEqual(dsds[3,2,1,1],0.0,"ds/ds has wrong value at (3,2,1,1).")
      self.assertEqual(dsds[3,2,1,2],0.0,"ds/ds has wrong value at (3,2,1,2).")
      self.assertEqual(dsds[3,2,1,3],0.0,"ds/ds has wrong value at (3,2,1,3).")
      self.assertEqual(dsds[3,2,1,4],0.0,"ds/ds has wrong value at (3,2,1,4).")
      self.assertEqual(dsds[3,2,2,0],0.0,"ds/ds has wrong value at (3,2,2,0).")
      self.assertEqual(dsds[3,2,2,1],0.0,"ds/ds has wrong value at (3,2,2,1).")
      self.assertEqual(dsds[3,2,2,2],0.0,"ds/ds has wrong value at (3,2,2,2).")
      self.assertEqual(dsds[3,2,2,3],0.0,"ds/ds has wrong value at (3,2,2,3).")
      self.assertEqual(dsds[3,2,2,4],0.0,"ds/ds has wrong value at (3,2,2,4).")
      self.assertEqual(dsds[3,2,3,0],0.0,"ds/ds has wrong value at (3,2,3,0).")
      self.assertEqual(dsds[3,2,3,1],0.0,"ds/ds has wrong value at (3,2,3,1).")
      self.assertEqual(dsds[3,2,3,2],1.0,"ds/ds has wrong value at (3,2,3,2).")
      self.assertEqual(dsds[3,2,3,3],0.0,"ds/ds has wrong value at (3,2,3,3).")
      self.assertEqual(dsds[3,2,3,4],0.0,"ds/ds has wrong value at (3,2,3,4).")
      self.assertEqual(dsds[3,3,0,0],0.0,"ds/ds has wrong value at (3,3,0,0).")
      self.assertEqual(dsds[3,3,0,1],0.0,"ds/ds has wrong value at (3,3,0,1).")
      self.assertEqual(dsds[3,3,0,2],0.0,"ds/ds has wrong value at (3,3,0,2).")
      self.assertEqual(dsds[3,3,0,3],0.0,"ds/ds has wrong value at (3,3,0,3).")
      self.assertEqual(dsds[3,3,0,4],0.0,"ds/ds has wrong value at (3,3,0,4).")
      self.assertEqual(dsds[3,3,1,0],0.0,"ds/ds has wrong value at (3,3,1,0).")
      self.assertEqual(dsds[3,3,1,1],0.0,"ds/ds has wrong value at (3,3,1,1).")
      self.assertEqual(dsds[3,3,1,2],0.0,"ds/ds has wrong value at (3,3,1,2).")
      self.assertEqual(dsds[3,3,1,3],0.0,"ds/ds has wrong value at (3,3,1,3).")
      self.assertEqual(dsds[3,3,1,4],0.0,"ds/ds has wrong value at (3,3,1,4).")
      self.assertEqual(dsds[3,3,2,0],0.0,"ds/ds has wrong value at (3,3,2,0).")
      self.assertEqual(dsds[3,3,2,1],0.0,"ds/ds has wrong value at (3,3,2,1).")
      self.assertEqual(dsds[3,3,2,2],0.0,"ds/ds has wrong value at (3,3,2,2).")
      self.assertEqual(dsds[3,3,2,3],0.0,"ds/ds has wrong value at (3,3,2,3).")
      self.assertEqual(dsds[3,3,2,4],0.0,"ds/ds has wrong value at (3,3,2,4).")
      self.assertEqual(dsds[3,3,3,0],0.0,"ds/ds has wrong value at (3,3,3,0).")
      self.assertEqual(dsds[3,3,3,1],0.0,"ds/ds has wrong value at (3,3,3,1).")
      self.assertEqual(dsds[3,3,3,2],0.0,"ds/ds has wrong value at (3,3,3,2).")
      self.assertEqual(dsds[3,3,3,3],1.0,"ds/ds has wrong value at (3,3,3,3).")
      self.assertEqual(dsds[3,3,3,4],0.0,"ds/ds has wrong value at (3,3,3,4).")
      self.assertEqual(dsds[3,4,0,0],0.0,"ds/ds has wrong value at (3,4,0,0).")
      self.assertEqual(dsds[3,4,0,1],0.0,"ds/ds has wrong value at (3,4,0,1).")
      self.assertEqual(dsds[3,4,0,2],0.0,"ds/ds has wrong value at (3,4,0,2).")
      self.assertEqual(dsds[3,4,0,3],0.0,"ds/ds has wrong value at (3,4,0,3).")
      self.assertEqual(dsds[3,4,0,4],0.0,"ds/ds has wrong value at (3,4,0,4).")
      self.assertEqual(dsds[3,4,1,0],0.0,"ds/ds has wrong value at (3,4,1,0).")
      self.assertEqual(dsds[3,4,1,1],0.0,"ds/ds has wrong value at (3,4,1,1).")
      self.assertEqual(dsds[3,4,1,2],0.0,"ds/ds has wrong value at (3,4,1,2).")
      self.assertEqual(dsds[3,4,1,3],0.0,"ds/ds has wrong value at (3,4,1,3).")
      self.assertEqual(dsds[3,4,1,4],0.0,"ds/ds has wrong value at (3,4,1,4).")
      self.assertEqual(dsds[3,4,2,0],0.0,"ds/ds has wrong value at (3,4,2,0).")
      self.assertEqual(dsds[3,4,2,1],0.0,"ds/ds has wrong value at (3,4,2,1).")
      self.assertEqual(dsds[3,4,2,2],0.0,"ds/ds has wrong value at (3,4,2,2).")
      self.assertEqual(dsds[3,4,2,3],0.0,"ds/ds has wrong value at (3,4,2,3).")
      self.assertEqual(dsds[3,4,2,4],0.0,"ds/ds has wrong value at (3,4,2,4).")
      self.assertEqual(dsds[3,4,3,0],0.0,"ds/ds has wrong value at (3,4,3,0).")
      self.assertEqual(dsds[3,4,3,1],0.0,"ds/ds has wrong value at (3,4,3,1).")
      self.assertEqual(dsds[3,4,3,2],0.0,"ds/ds has wrong value at (3,4,3,2).")
      self.assertEqual(dsds[3,4,3,3],0.0,"ds/ds has wrong value at (3,4,3,3).")
      self.assertEqual(dsds[3,4,3,4],1.0,"ds/ds has wrong value at (3,4,3,4).")

      sa=s.getSubstitutedArguments({s2:-10})
      self.assertEqual(len(sa),0,"wrong number of substituted arguments")

      self.assertFalse(s.isAppropriateValue(10.)," 10. is not appropriate substitute")
      self.assertFalse(s.isAppropriateValue(10)," 10 is not appropriate substitute")
      self.assertTrue(s.isAppropriateValue(Symbol(shape=(4, 5),dim=3)),"Symbol is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(Symbol(shape=(4, 5),dim=4)),"Symbol is not appropriate substitute (dim)")
      self.assertFalse(s.isAppropriateValue(escript.Data(0.,(5,),self.functionspace)),"Data is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(numpy.zeros((4, 5))),"ndarray is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(numpy.zeros((5,))),"ndarray is not appropriate substitute (shape)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_Symbol_rank2_d3_nargs1(self):
      s2=Symbol(shape=())
      s=Symbol(shape=(4, 5),dim=3,args=[s2])
      self.assertEqual(s.getRank(),2,"wrong rank.")
      self.assertEqual(s.getShape(),(4, 5),"wrong shape.")
      self.assertEqual(s.getDim(),3,"wrong spatial dimension.")
      self.assertEqual(s.getArgument(),[s2],"wrong arguments.")

      ss=s.substitute({s:numpy.zeros((4, 5))})
      self.assertTrue(isinstance(ss,numpy.ndarray),"value after substitution is not ndarray.")
      self.assertEqual(ss.shape,(4, 5),"value after substitution has not expected shape")
      try:
        s.substitute({s:numpy.zeros((5,))})
        fail("illegal substition was successful")
      except TypeError:
        pass

      dsdarg=s.diff(Symbol(shape=()))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds() has wrong type.")
      self.assertEqual(dsdarg.shape,(4, 5),"ds/ds() has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds() has wrong value.")
      dsdarg=s.diff(Symbol(shape=(2,)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(2,) has wrong type.")
      self.assertEqual(dsdarg.shape,(4, 5, 2),"ds/ds(2,) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(2,) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(4, 5)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(4, 5) has wrong type.")
      self.assertEqual(dsdarg.shape,(4, 5, 4, 5),"ds/ds(4, 5) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(4, 5) has wrong value.")

      dsds=s.diff(s)
      self.assertTrue(isinstance(dsds,numpy.ndarray),"ds/ds has wrong type.")
      self.assertEqual(dsds.shape,(4, 5, 4, 5),"ds/ds has wrong shape.")
      self.assertEqual(dsds[0,0,0,0],1.0,"ds/ds has wrong value at (0,0,0,0).")
      self.assertEqual(dsds[0,0,0,1],0.0,"ds/ds has wrong value at (0,0,0,1).")
      self.assertEqual(dsds[0,0,0,2],0.0,"ds/ds has wrong value at (0,0,0,2).")
      self.assertEqual(dsds[0,0,0,3],0.0,"ds/ds has wrong value at (0,0,0,3).")
      self.assertEqual(dsds[0,0,0,4],0.0,"ds/ds has wrong value at (0,0,0,4).")
      self.assertEqual(dsds[0,0,1,0],0.0,"ds/ds has wrong value at (0,0,1,0).")
      self.assertEqual(dsds[0,0,1,1],0.0,"ds/ds has wrong value at (0,0,1,1).")
      self.assertEqual(dsds[0,0,1,2],0.0,"ds/ds has wrong value at (0,0,1,2).")
      self.assertEqual(dsds[0,0,1,3],0.0,"ds/ds has wrong value at (0,0,1,3).")
      self.assertEqual(dsds[0,0,1,4],0.0,"ds/ds has wrong value at (0,0,1,4).")
      self.assertEqual(dsds[0,0,2,0],0.0,"ds/ds has wrong value at (0,0,2,0).")
      self.assertEqual(dsds[0,0,2,1],0.0,"ds/ds has wrong value at (0,0,2,1).")
      self.assertEqual(dsds[0,0,2,2],0.0,"ds/ds has wrong value at (0,0,2,2).")
      self.assertEqual(dsds[0,0,2,3],0.0,"ds/ds has wrong value at (0,0,2,3).")
      self.assertEqual(dsds[0,0,2,4],0.0,"ds/ds has wrong value at (0,0,2,4).")
      self.assertEqual(dsds[0,0,3,0],0.0,"ds/ds has wrong value at (0,0,3,0).")
      self.assertEqual(dsds[0,0,3,1],0.0,"ds/ds has wrong value at (0,0,3,1).")
      self.assertEqual(dsds[0,0,3,2],0.0,"ds/ds has wrong value at (0,0,3,2).")
      self.assertEqual(dsds[0,0,3,3],0.0,"ds/ds has wrong value at (0,0,3,3).")
      self.assertEqual(dsds[0,0,3,4],0.0,"ds/ds has wrong value at (0,0,3,4).")
      self.assertEqual(dsds[0,1,0,0],0.0,"ds/ds has wrong value at (0,1,0,0).")
      self.assertEqual(dsds[0,1,0,1],1.0,"ds/ds has wrong value at (0,1,0,1).")
      self.assertEqual(dsds[0,1,0,2],0.0,"ds/ds has wrong value at (0,1,0,2).")
      self.assertEqual(dsds[0,1,0,3],0.0,"ds/ds has wrong value at (0,1,0,3).")
      self.assertEqual(dsds[0,1,0,4],0.0,"ds/ds has wrong value at (0,1,0,4).")
      self.assertEqual(dsds[0,1,1,0],0.0,"ds/ds has wrong value at (0,1,1,0).")
      self.assertEqual(dsds[0,1,1,1],0.0,"ds/ds has wrong value at (0,1,1,1).")
      self.assertEqual(dsds[0,1,1,2],0.0,"ds/ds has wrong value at (0,1,1,2).")
      self.assertEqual(dsds[0,1,1,3],0.0,"ds/ds has wrong value at (0,1,1,3).")
      self.assertEqual(dsds[0,1,1,4],0.0,"ds/ds has wrong value at (0,1,1,4).")
      self.assertEqual(dsds[0,1,2,0],0.0,"ds/ds has wrong value at (0,1,2,0).")
      self.assertEqual(dsds[0,1,2,1],0.0,"ds/ds has wrong value at (0,1,2,1).")
      self.assertEqual(dsds[0,1,2,2],0.0,"ds/ds has wrong value at (0,1,2,2).")
      self.assertEqual(dsds[0,1,2,3],0.0,"ds/ds has wrong value at (0,1,2,3).")
      self.assertEqual(dsds[0,1,2,4],0.0,"ds/ds has wrong value at (0,1,2,4).")
      self.assertEqual(dsds[0,1,3,0],0.0,"ds/ds has wrong value at (0,1,3,0).")
      self.assertEqual(dsds[0,1,3,1],0.0,"ds/ds has wrong value at (0,1,3,1).")
      self.assertEqual(dsds[0,1,3,2],0.0,"ds/ds has wrong value at (0,1,3,2).")
      self.assertEqual(dsds[0,1,3,3],0.0,"ds/ds has wrong value at (0,1,3,3).")
      self.assertEqual(dsds[0,1,3,4],0.0,"ds/ds has wrong value at (0,1,3,4).")
      self.assertEqual(dsds[0,2,0,0],0.0,"ds/ds has wrong value at (0,2,0,0).")
      self.assertEqual(dsds[0,2,0,1],0.0,"ds/ds has wrong value at (0,2,0,1).")
      self.assertEqual(dsds[0,2,0,2],1.0,"ds/ds has wrong value at (0,2,0,2).")
      self.assertEqual(dsds[0,2,0,3],0.0,"ds/ds has wrong value at (0,2,0,3).")
      self.assertEqual(dsds[0,2,0,4],0.0,"ds/ds has wrong value at (0,2,0,4).")
      self.assertEqual(dsds[0,2,1,0],0.0,"ds/ds has wrong value at (0,2,1,0).")
      self.assertEqual(dsds[0,2,1,1],0.0,"ds/ds has wrong value at (0,2,1,1).")
      self.assertEqual(dsds[0,2,1,2],0.0,"ds/ds has wrong value at (0,2,1,2).")
      self.assertEqual(dsds[0,2,1,3],0.0,"ds/ds has wrong value at (0,2,1,3).")
      self.assertEqual(dsds[0,2,1,4],0.0,"ds/ds has wrong value at (0,2,1,4).")
      self.assertEqual(dsds[0,2,2,0],0.0,"ds/ds has wrong value at (0,2,2,0).")
      self.assertEqual(dsds[0,2,2,1],0.0,"ds/ds has wrong value at (0,2,2,1).")
      self.assertEqual(dsds[0,2,2,2],0.0,"ds/ds has wrong value at (0,2,2,2).")
      self.assertEqual(dsds[0,2,2,3],0.0,"ds/ds has wrong value at (0,2,2,3).")
      self.assertEqual(dsds[0,2,2,4],0.0,"ds/ds has wrong value at (0,2,2,4).")
      self.assertEqual(dsds[0,2,3,0],0.0,"ds/ds has wrong value at (0,2,3,0).")
      self.assertEqual(dsds[0,2,3,1],0.0,"ds/ds has wrong value at (0,2,3,1).")
      self.assertEqual(dsds[0,2,3,2],0.0,"ds/ds has wrong value at (0,2,3,2).")
      self.assertEqual(dsds[0,2,3,3],0.0,"ds/ds has wrong value at (0,2,3,3).")
      self.assertEqual(dsds[0,2,3,4],0.0,"ds/ds has wrong value at (0,2,3,4).")
      self.assertEqual(dsds[0,3,0,0],0.0,"ds/ds has wrong value at (0,3,0,0).")
      self.assertEqual(dsds[0,3,0,1],0.0,"ds/ds has wrong value at (0,3,0,1).")
      self.assertEqual(dsds[0,3,0,2],0.0,"ds/ds has wrong value at (0,3,0,2).")
      self.assertEqual(dsds[0,3,0,3],1.0,"ds/ds has wrong value at (0,3,0,3).")
      self.assertEqual(dsds[0,3,0,4],0.0,"ds/ds has wrong value at (0,3,0,4).")
      self.assertEqual(dsds[0,3,1,0],0.0,"ds/ds has wrong value at (0,3,1,0).")
      self.assertEqual(dsds[0,3,1,1],0.0,"ds/ds has wrong value at (0,3,1,1).")
      self.assertEqual(dsds[0,3,1,2],0.0,"ds/ds has wrong value at (0,3,1,2).")
      self.assertEqual(dsds[0,3,1,3],0.0,"ds/ds has wrong value at (0,3,1,3).")
      self.assertEqual(dsds[0,3,1,4],0.0,"ds/ds has wrong value at (0,3,1,4).")
      self.assertEqual(dsds[0,3,2,0],0.0,"ds/ds has wrong value at (0,3,2,0).")
      self.assertEqual(dsds[0,3,2,1],0.0,"ds/ds has wrong value at (0,3,2,1).")
      self.assertEqual(dsds[0,3,2,2],0.0,"ds/ds has wrong value at (0,3,2,2).")
      self.assertEqual(dsds[0,3,2,3],0.0,"ds/ds has wrong value at (0,3,2,3).")
      self.assertEqual(dsds[0,3,2,4],0.0,"ds/ds has wrong value at (0,3,2,4).")
      self.assertEqual(dsds[0,3,3,0],0.0,"ds/ds has wrong value at (0,3,3,0).")
      self.assertEqual(dsds[0,3,3,1],0.0,"ds/ds has wrong value at (0,3,3,1).")
      self.assertEqual(dsds[0,3,3,2],0.0,"ds/ds has wrong value at (0,3,3,2).")
      self.assertEqual(dsds[0,3,3,3],0.0,"ds/ds has wrong value at (0,3,3,3).")
      self.assertEqual(dsds[0,3,3,4],0.0,"ds/ds has wrong value at (0,3,3,4).")
      self.assertEqual(dsds[0,4,0,0],0.0,"ds/ds has wrong value at (0,4,0,0).")
      self.assertEqual(dsds[0,4,0,1],0.0,"ds/ds has wrong value at (0,4,0,1).")
      self.assertEqual(dsds[0,4,0,2],0.0,"ds/ds has wrong value at (0,4,0,2).")
      self.assertEqual(dsds[0,4,0,3],0.0,"ds/ds has wrong value at (0,4,0,3).")
      self.assertEqual(dsds[0,4,0,4],1.0,"ds/ds has wrong value at (0,4,0,4).")
      self.assertEqual(dsds[0,4,1,0],0.0,"ds/ds has wrong value at (0,4,1,0).")
      self.assertEqual(dsds[0,4,1,1],0.0,"ds/ds has wrong value at (0,4,1,1).")
      self.assertEqual(dsds[0,4,1,2],0.0,"ds/ds has wrong value at (0,4,1,2).")
      self.assertEqual(dsds[0,4,1,3],0.0,"ds/ds has wrong value at (0,4,1,3).")
      self.assertEqual(dsds[0,4,1,4],0.0,"ds/ds has wrong value at (0,4,1,4).")
      self.assertEqual(dsds[0,4,2,0],0.0,"ds/ds has wrong value at (0,4,2,0).")
      self.assertEqual(dsds[0,4,2,1],0.0,"ds/ds has wrong value at (0,4,2,1).")
      self.assertEqual(dsds[0,4,2,2],0.0,"ds/ds has wrong value at (0,4,2,2).")
      self.assertEqual(dsds[0,4,2,3],0.0,"ds/ds has wrong value at (0,4,2,3).")
      self.assertEqual(dsds[0,4,2,4],0.0,"ds/ds has wrong value at (0,4,2,4).")
      self.assertEqual(dsds[0,4,3,0],0.0,"ds/ds has wrong value at (0,4,3,0).")
      self.assertEqual(dsds[0,4,3,1],0.0,"ds/ds has wrong value at (0,4,3,1).")
      self.assertEqual(dsds[0,4,3,2],0.0,"ds/ds has wrong value at (0,4,3,2).")
      self.assertEqual(dsds[0,4,3,3],0.0,"ds/ds has wrong value at (0,4,3,3).")
      self.assertEqual(dsds[0,4,3,4],0.0,"ds/ds has wrong value at (0,4,3,4).")
      self.assertEqual(dsds[1,0,0,0],0.0,"ds/ds has wrong value at (1,0,0,0).")
      self.assertEqual(dsds[1,0,0,1],0.0,"ds/ds has wrong value at (1,0,0,1).")
      self.assertEqual(dsds[1,0,0,2],0.0,"ds/ds has wrong value at (1,0,0,2).")
      self.assertEqual(dsds[1,0,0,3],0.0,"ds/ds has wrong value at (1,0,0,3).")
      self.assertEqual(dsds[1,0,0,4],0.0,"ds/ds has wrong value at (1,0,0,4).")
      self.assertEqual(dsds[1,0,1,0],1.0,"ds/ds has wrong value at (1,0,1,0).")
      self.assertEqual(dsds[1,0,1,1],0.0,"ds/ds has wrong value at (1,0,1,1).")
      self.assertEqual(dsds[1,0,1,2],0.0,"ds/ds has wrong value at (1,0,1,2).")
      self.assertEqual(dsds[1,0,1,3],0.0,"ds/ds has wrong value at (1,0,1,3).")
      self.assertEqual(dsds[1,0,1,4],0.0,"ds/ds has wrong value at (1,0,1,4).")
      self.assertEqual(dsds[1,0,2,0],0.0,"ds/ds has wrong value at (1,0,2,0).")
      self.assertEqual(dsds[1,0,2,1],0.0,"ds/ds has wrong value at (1,0,2,1).")
      self.assertEqual(dsds[1,0,2,2],0.0,"ds/ds has wrong value at (1,0,2,2).")
      self.assertEqual(dsds[1,0,2,3],0.0,"ds/ds has wrong value at (1,0,2,3).")
      self.assertEqual(dsds[1,0,2,4],0.0,"ds/ds has wrong value at (1,0,2,4).")
      self.assertEqual(dsds[1,0,3,0],0.0,"ds/ds has wrong value at (1,0,3,0).")
      self.assertEqual(dsds[1,0,3,1],0.0,"ds/ds has wrong value at (1,0,3,1).")
      self.assertEqual(dsds[1,0,3,2],0.0,"ds/ds has wrong value at (1,0,3,2).")
      self.assertEqual(dsds[1,0,3,3],0.0,"ds/ds has wrong value at (1,0,3,3).")
      self.assertEqual(dsds[1,0,3,4],0.0,"ds/ds has wrong value at (1,0,3,4).")
      self.assertEqual(dsds[1,1,0,0],0.0,"ds/ds has wrong value at (1,1,0,0).")
      self.assertEqual(dsds[1,1,0,1],0.0,"ds/ds has wrong value at (1,1,0,1).")
      self.assertEqual(dsds[1,1,0,2],0.0,"ds/ds has wrong value at (1,1,0,2).")
      self.assertEqual(dsds[1,1,0,3],0.0,"ds/ds has wrong value at (1,1,0,3).")
      self.assertEqual(dsds[1,1,0,4],0.0,"ds/ds has wrong value at (1,1,0,4).")
      self.assertEqual(dsds[1,1,1,0],0.0,"ds/ds has wrong value at (1,1,1,0).")
      self.assertEqual(dsds[1,1,1,1],1.0,"ds/ds has wrong value at (1,1,1,1).")
      self.assertEqual(dsds[1,1,1,2],0.0,"ds/ds has wrong value at (1,1,1,2).")
      self.assertEqual(dsds[1,1,1,3],0.0,"ds/ds has wrong value at (1,1,1,3).")
      self.assertEqual(dsds[1,1,1,4],0.0,"ds/ds has wrong value at (1,1,1,4).")
      self.assertEqual(dsds[1,1,2,0],0.0,"ds/ds has wrong value at (1,1,2,0).")
      self.assertEqual(dsds[1,1,2,1],0.0,"ds/ds has wrong value at (1,1,2,1).")
      self.assertEqual(dsds[1,1,2,2],0.0,"ds/ds has wrong value at (1,1,2,2).")
      self.assertEqual(dsds[1,1,2,3],0.0,"ds/ds has wrong value at (1,1,2,3).")
      self.assertEqual(dsds[1,1,2,4],0.0,"ds/ds has wrong value at (1,1,2,4).")
      self.assertEqual(dsds[1,1,3,0],0.0,"ds/ds has wrong value at (1,1,3,0).")
      self.assertEqual(dsds[1,1,3,1],0.0,"ds/ds has wrong value at (1,1,3,1).")
      self.assertEqual(dsds[1,1,3,2],0.0,"ds/ds has wrong value at (1,1,3,2).")
      self.assertEqual(dsds[1,1,3,3],0.0,"ds/ds has wrong value at (1,1,3,3).")
      self.assertEqual(dsds[1,1,3,4],0.0,"ds/ds has wrong value at (1,1,3,4).")
      self.assertEqual(dsds[1,2,0,0],0.0,"ds/ds has wrong value at (1,2,0,0).")
      self.assertEqual(dsds[1,2,0,1],0.0,"ds/ds has wrong value at (1,2,0,1).")
      self.assertEqual(dsds[1,2,0,2],0.0,"ds/ds has wrong value at (1,2,0,2).")
      self.assertEqual(dsds[1,2,0,3],0.0,"ds/ds has wrong value at (1,2,0,3).")
      self.assertEqual(dsds[1,2,0,4],0.0,"ds/ds has wrong value at (1,2,0,4).")
      self.assertEqual(dsds[1,2,1,0],0.0,"ds/ds has wrong value at (1,2,1,0).")
      self.assertEqual(dsds[1,2,1,1],0.0,"ds/ds has wrong value at (1,2,1,1).")
      self.assertEqual(dsds[1,2,1,2],1.0,"ds/ds has wrong value at (1,2,1,2).")
      self.assertEqual(dsds[1,2,1,3],0.0,"ds/ds has wrong value at (1,2,1,3).")
      self.assertEqual(dsds[1,2,1,4],0.0,"ds/ds has wrong value at (1,2,1,4).")
      self.assertEqual(dsds[1,2,2,0],0.0,"ds/ds has wrong value at (1,2,2,0).")
      self.assertEqual(dsds[1,2,2,1],0.0,"ds/ds has wrong value at (1,2,2,1).")
      self.assertEqual(dsds[1,2,2,2],0.0,"ds/ds has wrong value at (1,2,2,2).")
      self.assertEqual(dsds[1,2,2,3],0.0,"ds/ds has wrong value at (1,2,2,3).")
      self.assertEqual(dsds[1,2,2,4],0.0,"ds/ds has wrong value at (1,2,2,4).")
      self.assertEqual(dsds[1,2,3,0],0.0,"ds/ds has wrong value at (1,2,3,0).")
      self.assertEqual(dsds[1,2,3,1],0.0,"ds/ds has wrong value at (1,2,3,1).")
      self.assertEqual(dsds[1,2,3,2],0.0,"ds/ds has wrong value at (1,2,3,2).")
      self.assertEqual(dsds[1,2,3,3],0.0,"ds/ds has wrong value at (1,2,3,3).")
      self.assertEqual(dsds[1,2,3,4],0.0,"ds/ds has wrong value at (1,2,3,4).")
      self.assertEqual(dsds[1,3,0,0],0.0,"ds/ds has wrong value at (1,3,0,0).")
      self.assertEqual(dsds[1,3,0,1],0.0,"ds/ds has wrong value at (1,3,0,1).")
      self.assertEqual(dsds[1,3,0,2],0.0,"ds/ds has wrong value at (1,3,0,2).")
      self.assertEqual(dsds[1,3,0,3],0.0,"ds/ds has wrong value at (1,3,0,3).")
      self.assertEqual(dsds[1,3,0,4],0.0,"ds/ds has wrong value at (1,3,0,4).")
      self.assertEqual(dsds[1,3,1,0],0.0,"ds/ds has wrong value at (1,3,1,0).")
      self.assertEqual(dsds[1,3,1,1],0.0,"ds/ds has wrong value at (1,3,1,1).")
      self.assertEqual(dsds[1,3,1,2],0.0,"ds/ds has wrong value at (1,3,1,2).")
      self.assertEqual(dsds[1,3,1,3],1.0,"ds/ds has wrong value at (1,3,1,3).")
      self.assertEqual(dsds[1,3,1,4],0.0,"ds/ds has wrong value at (1,3,1,4).")
      self.assertEqual(dsds[1,3,2,0],0.0,"ds/ds has wrong value at (1,3,2,0).")
      self.assertEqual(dsds[1,3,2,1],0.0,"ds/ds has wrong value at (1,3,2,1).")
      self.assertEqual(dsds[1,3,2,2],0.0,"ds/ds has wrong value at (1,3,2,2).")
      self.assertEqual(dsds[1,3,2,3],0.0,"ds/ds has wrong value at (1,3,2,3).")
      self.assertEqual(dsds[1,3,2,4],0.0,"ds/ds has wrong value at (1,3,2,4).")
      self.assertEqual(dsds[1,3,3,0],0.0,"ds/ds has wrong value at (1,3,3,0).")
      self.assertEqual(dsds[1,3,3,1],0.0,"ds/ds has wrong value at (1,3,3,1).")
      self.assertEqual(dsds[1,3,3,2],0.0,"ds/ds has wrong value at (1,3,3,2).")
      self.assertEqual(dsds[1,3,3,3],0.0,"ds/ds has wrong value at (1,3,3,3).")
      self.assertEqual(dsds[1,3,3,4],0.0,"ds/ds has wrong value at (1,3,3,4).")
      self.assertEqual(dsds[1,4,0,0],0.0,"ds/ds has wrong value at (1,4,0,0).")
      self.assertEqual(dsds[1,4,0,1],0.0,"ds/ds has wrong value at (1,4,0,1).")
      self.assertEqual(dsds[1,4,0,2],0.0,"ds/ds has wrong value at (1,4,0,2).")
      self.assertEqual(dsds[1,4,0,3],0.0,"ds/ds has wrong value at (1,4,0,3).")
      self.assertEqual(dsds[1,4,0,4],0.0,"ds/ds has wrong value at (1,4,0,4).")
      self.assertEqual(dsds[1,4,1,0],0.0,"ds/ds has wrong value at (1,4,1,0).")
      self.assertEqual(dsds[1,4,1,1],0.0,"ds/ds has wrong value at (1,4,1,1).")
      self.assertEqual(dsds[1,4,1,2],0.0,"ds/ds has wrong value at (1,4,1,2).")
      self.assertEqual(dsds[1,4,1,3],0.0,"ds/ds has wrong value at (1,4,1,3).")
      self.assertEqual(dsds[1,4,1,4],1.0,"ds/ds has wrong value at (1,4,1,4).")
      self.assertEqual(dsds[1,4,2,0],0.0,"ds/ds has wrong value at (1,4,2,0).")
      self.assertEqual(dsds[1,4,2,1],0.0,"ds/ds has wrong value at (1,4,2,1).")
      self.assertEqual(dsds[1,4,2,2],0.0,"ds/ds has wrong value at (1,4,2,2).")
      self.assertEqual(dsds[1,4,2,3],0.0,"ds/ds has wrong value at (1,4,2,3).")
      self.assertEqual(dsds[1,4,2,4],0.0,"ds/ds has wrong value at (1,4,2,4).")
      self.assertEqual(dsds[1,4,3,0],0.0,"ds/ds has wrong value at (1,4,3,0).")
      self.assertEqual(dsds[1,4,3,1],0.0,"ds/ds has wrong value at (1,4,3,1).")
      self.assertEqual(dsds[1,4,3,2],0.0,"ds/ds has wrong value at (1,4,3,2).")
      self.assertEqual(dsds[1,4,3,3],0.0,"ds/ds has wrong value at (1,4,3,3).")
      self.assertEqual(dsds[1,4,3,4],0.0,"ds/ds has wrong value at (1,4,3,4).")
      self.assertEqual(dsds[2,0,0,0],0.0,"ds/ds has wrong value at (2,0,0,0).")
      self.assertEqual(dsds[2,0,0,1],0.0,"ds/ds has wrong value at (2,0,0,1).")
      self.assertEqual(dsds[2,0,0,2],0.0,"ds/ds has wrong value at (2,0,0,2).")
      self.assertEqual(dsds[2,0,0,3],0.0,"ds/ds has wrong value at (2,0,0,3).")
      self.assertEqual(dsds[2,0,0,4],0.0,"ds/ds has wrong value at (2,0,0,4).")
      self.assertEqual(dsds[2,0,1,0],0.0,"ds/ds has wrong value at (2,0,1,0).")
      self.assertEqual(dsds[2,0,1,1],0.0,"ds/ds has wrong value at (2,0,1,1).")
      self.assertEqual(dsds[2,0,1,2],0.0,"ds/ds has wrong value at (2,0,1,2).")
      self.assertEqual(dsds[2,0,1,3],0.0,"ds/ds has wrong value at (2,0,1,3).")
      self.assertEqual(dsds[2,0,1,4],0.0,"ds/ds has wrong value at (2,0,1,4).")
      self.assertEqual(dsds[2,0,2,0],1.0,"ds/ds has wrong value at (2,0,2,0).")
      self.assertEqual(dsds[2,0,2,1],0.0,"ds/ds has wrong value at (2,0,2,1).")
      self.assertEqual(dsds[2,0,2,2],0.0,"ds/ds has wrong value at (2,0,2,2).")
      self.assertEqual(dsds[2,0,2,3],0.0,"ds/ds has wrong value at (2,0,2,3).")
      self.assertEqual(dsds[2,0,2,4],0.0,"ds/ds has wrong value at (2,0,2,4).")
      self.assertEqual(dsds[2,0,3,0],0.0,"ds/ds has wrong value at (2,0,3,0).")
      self.assertEqual(dsds[2,0,3,1],0.0,"ds/ds has wrong value at (2,0,3,1).")
      self.assertEqual(dsds[2,0,3,2],0.0,"ds/ds has wrong value at (2,0,3,2).")
      self.assertEqual(dsds[2,0,3,3],0.0,"ds/ds has wrong value at (2,0,3,3).")
      self.assertEqual(dsds[2,0,3,4],0.0,"ds/ds has wrong value at (2,0,3,4).")
      self.assertEqual(dsds[2,1,0,0],0.0,"ds/ds has wrong value at (2,1,0,0).")
      self.assertEqual(dsds[2,1,0,1],0.0,"ds/ds has wrong value at (2,1,0,1).")
      self.assertEqual(dsds[2,1,0,2],0.0,"ds/ds has wrong value at (2,1,0,2).")
      self.assertEqual(dsds[2,1,0,3],0.0,"ds/ds has wrong value at (2,1,0,3).")
      self.assertEqual(dsds[2,1,0,4],0.0,"ds/ds has wrong value at (2,1,0,4).")
      self.assertEqual(dsds[2,1,1,0],0.0,"ds/ds has wrong value at (2,1,1,0).")
      self.assertEqual(dsds[2,1,1,1],0.0,"ds/ds has wrong value at (2,1,1,1).")
      self.assertEqual(dsds[2,1,1,2],0.0,"ds/ds has wrong value at (2,1,1,2).")
      self.assertEqual(dsds[2,1,1,3],0.0,"ds/ds has wrong value at (2,1,1,3).")
      self.assertEqual(dsds[2,1,1,4],0.0,"ds/ds has wrong value at (2,1,1,4).")
      self.assertEqual(dsds[2,1,2,0],0.0,"ds/ds has wrong value at (2,1,2,0).")
      self.assertEqual(dsds[2,1,2,1],1.0,"ds/ds has wrong value at (2,1,2,1).")
      self.assertEqual(dsds[2,1,2,2],0.0,"ds/ds has wrong value at (2,1,2,2).")
      self.assertEqual(dsds[2,1,2,3],0.0,"ds/ds has wrong value at (2,1,2,3).")
      self.assertEqual(dsds[2,1,2,4],0.0,"ds/ds has wrong value at (2,1,2,4).")
      self.assertEqual(dsds[2,1,3,0],0.0,"ds/ds has wrong value at (2,1,3,0).")
      self.assertEqual(dsds[2,1,3,1],0.0,"ds/ds has wrong value at (2,1,3,1).")
      self.assertEqual(dsds[2,1,3,2],0.0,"ds/ds has wrong value at (2,1,3,2).")
      self.assertEqual(dsds[2,1,3,3],0.0,"ds/ds has wrong value at (2,1,3,3).")
      self.assertEqual(dsds[2,1,3,4],0.0,"ds/ds has wrong value at (2,1,3,4).")
      self.assertEqual(dsds[2,2,0,0],0.0,"ds/ds has wrong value at (2,2,0,0).")
      self.assertEqual(dsds[2,2,0,1],0.0,"ds/ds has wrong value at (2,2,0,1).")
      self.assertEqual(dsds[2,2,0,2],0.0,"ds/ds has wrong value at (2,2,0,2).")
      self.assertEqual(dsds[2,2,0,3],0.0,"ds/ds has wrong value at (2,2,0,3).")
      self.assertEqual(dsds[2,2,0,4],0.0,"ds/ds has wrong value at (2,2,0,4).")
      self.assertEqual(dsds[2,2,1,0],0.0,"ds/ds has wrong value at (2,2,1,0).")
      self.assertEqual(dsds[2,2,1,1],0.0,"ds/ds has wrong value at (2,2,1,1).")
      self.assertEqual(dsds[2,2,1,2],0.0,"ds/ds has wrong value at (2,2,1,2).")
      self.assertEqual(dsds[2,2,1,3],0.0,"ds/ds has wrong value at (2,2,1,3).")
      self.assertEqual(dsds[2,2,1,4],0.0,"ds/ds has wrong value at (2,2,1,4).")
      self.assertEqual(dsds[2,2,2,0],0.0,"ds/ds has wrong value at (2,2,2,0).")
      self.assertEqual(dsds[2,2,2,1],0.0,"ds/ds has wrong value at (2,2,2,1).")
      self.assertEqual(dsds[2,2,2,2],1.0,"ds/ds has wrong value at (2,2,2,2).")
      self.assertEqual(dsds[2,2,2,3],0.0,"ds/ds has wrong value at (2,2,2,3).")
      self.assertEqual(dsds[2,2,2,4],0.0,"ds/ds has wrong value at (2,2,2,4).")
      self.assertEqual(dsds[2,2,3,0],0.0,"ds/ds has wrong value at (2,2,3,0).")
      self.assertEqual(dsds[2,2,3,1],0.0,"ds/ds has wrong value at (2,2,3,1).")
      self.assertEqual(dsds[2,2,3,2],0.0,"ds/ds has wrong value at (2,2,3,2).")
      self.assertEqual(dsds[2,2,3,3],0.0,"ds/ds has wrong value at (2,2,3,3).")
      self.assertEqual(dsds[2,2,3,4],0.0,"ds/ds has wrong value at (2,2,3,4).")
      self.assertEqual(dsds[2,3,0,0],0.0,"ds/ds has wrong value at (2,3,0,0).")
      self.assertEqual(dsds[2,3,0,1],0.0,"ds/ds has wrong value at (2,3,0,1).")
      self.assertEqual(dsds[2,3,0,2],0.0,"ds/ds has wrong value at (2,3,0,2).")
      self.assertEqual(dsds[2,3,0,3],0.0,"ds/ds has wrong value at (2,3,0,3).")
      self.assertEqual(dsds[2,3,0,4],0.0,"ds/ds has wrong value at (2,3,0,4).")
      self.assertEqual(dsds[2,3,1,0],0.0,"ds/ds has wrong value at (2,3,1,0).")
      self.assertEqual(dsds[2,3,1,1],0.0,"ds/ds has wrong value at (2,3,1,1).")
      self.assertEqual(dsds[2,3,1,2],0.0,"ds/ds has wrong value at (2,3,1,2).")
      self.assertEqual(dsds[2,3,1,3],0.0,"ds/ds has wrong value at (2,3,1,3).")
      self.assertEqual(dsds[2,3,1,4],0.0,"ds/ds has wrong value at (2,3,1,4).")
      self.assertEqual(dsds[2,3,2,0],0.0,"ds/ds has wrong value at (2,3,2,0).")
      self.assertEqual(dsds[2,3,2,1],0.0,"ds/ds has wrong value at (2,3,2,1).")
      self.assertEqual(dsds[2,3,2,2],0.0,"ds/ds has wrong value at (2,3,2,2).")
      self.assertEqual(dsds[2,3,2,3],1.0,"ds/ds has wrong value at (2,3,2,3).")
      self.assertEqual(dsds[2,3,2,4],0.0,"ds/ds has wrong value at (2,3,2,4).")
      self.assertEqual(dsds[2,3,3,0],0.0,"ds/ds has wrong value at (2,3,3,0).")
      self.assertEqual(dsds[2,3,3,1],0.0,"ds/ds has wrong value at (2,3,3,1).")
      self.assertEqual(dsds[2,3,3,2],0.0,"ds/ds has wrong value at (2,3,3,2).")
      self.assertEqual(dsds[2,3,3,3],0.0,"ds/ds has wrong value at (2,3,3,3).")
      self.assertEqual(dsds[2,3,3,4],0.0,"ds/ds has wrong value at (2,3,3,4).")
      self.assertEqual(dsds[2,4,0,0],0.0,"ds/ds has wrong value at (2,4,0,0).")
      self.assertEqual(dsds[2,4,0,1],0.0,"ds/ds has wrong value at (2,4,0,1).")
      self.assertEqual(dsds[2,4,0,2],0.0,"ds/ds has wrong value at (2,4,0,2).")
      self.assertEqual(dsds[2,4,0,3],0.0,"ds/ds has wrong value at (2,4,0,3).")
      self.assertEqual(dsds[2,4,0,4],0.0,"ds/ds has wrong value at (2,4,0,4).")
      self.assertEqual(dsds[2,4,1,0],0.0,"ds/ds has wrong value at (2,4,1,0).")
      self.assertEqual(dsds[2,4,1,1],0.0,"ds/ds has wrong value at (2,4,1,1).")
      self.assertEqual(dsds[2,4,1,2],0.0,"ds/ds has wrong value at (2,4,1,2).")
      self.assertEqual(dsds[2,4,1,3],0.0,"ds/ds has wrong value at (2,4,1,3).")
      self.assertEqual(dsds[2,4,1,4],0.0,"ds/ds has wrong value at (2,4,1,4).")
      self.assertEqual(dsds[2,4,2,0],0.0,"ds/ds has wrong value at (2,4,2,0).")
      self.assertEqual(dsds[2,4,2,1],0.0,"ds/ds has wrong value at (2,4,2,1).")
      self.assertEqual(dsds[2,4,2,2],0.0,"ds/ds has wrong value at (2,4,2,2).")
      self.assertEqual(dsds[2,4,2,3],0.0,"ds/ds has wrong value at (2,4,2,3).")
      self.assertEqual(dsds[2,4,2,4],1.0,"ds/ds has wrong value at (2,4,2,4).")
      self.assertEqual(dsds[2,4,3,0],0.0,"ds/ds has wrong value at (2,4,3,0).")
      self.assertEqual(dsds[2,4,3,1],0.0,"ds/ds has wrong value at (2,4,3,1).")
      self.assertEqual(dsds[2,4,3,2],0.0,"ds/ds has wrong value at (2,4,3,2).")
      self.assertEqual(dsds[2,4,3,3],0.0,"ds/ds has wrong value at (2,4,3,3).")
      self.assertEqual(dsds[2,4,3,4],0.0,"ds/ds has wrong value at (2,4,3,4).")
      self.assertEqual(dsds[3,0,0,0],0.0,"ds/ds has wrong value at (3,0,0,0).")
      self.assertEqual(dsds[3,0,0,1],0.0,"ds/ds has wrong value at (3,0,0,1).")
      self.assertEqual(dsds[3,0,0,2],0.0,"ds/ds has wrong value at (3,0,0,2).")
      self.assertEqual(dsds[3,0,0,3],0.0,"ds/ds has wrong value at (3,0,0,3).")
      self.assertEqual(dsds[3,0,0,4],0.0,"ds/ds has wrong value at (3,0,0,4).")
      self.assertEqual(dsds[3,0,1,0],0.0,"ds/ds has wrong value at (3,0,1,0).")
      self.assertEqual(dsds[3,0,1,1],0.0,"ds/ds has wrong value at (3,0,1,1).")
      self.assertEqual(dsds[3,0,1,2],0.0,"ds/ds has wrong value at (3,0,1,2).")
      self.assertEqual(dsds[3,0,1,3],0.0,"ds/ds has wrong value at (3,0,1,3).")
      self.assertEqual(dsds[3,0,1,4],0.0,"ds/ds has wrong value at (3,0,1,4).")
      self.assertEqual(dsds[3,0,2,0],0.0,"ds/ds has wrong value at (3,0,2,0).")
      self.assertEqual(dsds[3,0,2,1],0.0,"ds/ds has wrong value at (3,0,2,1).")
      self.assertEqual(dsds[3,0,2,2],0.0,"ds/ds has wrong value at (3,0,2,2).")
      self.assertEqual(dsds[3,0,2,3],0.0,"ds/ds has wrong value at (3,0,2,3).")
      self.assertEqual(dsds[3,0,2,4],0.0,"ds/ds has wrong value at (3,0,2,4).")
      self.assertEqual(dsds[3,0,3,0],1.0,"ds/ds has wrong value at (3,0,3,0).")
      self.assertEqual(dsds[3,0,3,1],0.0,"ds/ds has wrong value at (3,0,3,1).")
      self.assertEqual(dsds[3,0,3,2],0.0,"ds/ds has wrong value at (3,0,3,2).")
      self.assertEqual(dsds[3,0,3,3],0.0,"ds/ds has wrong value at (3,0,3,3).")
      self.assertEqual(dsds[3,0,3,4],0.0,"ds/ds has wrong value at (3,0,3,4).")
      self.assertEqual(dsds[3,1,0,0],0.0,"ds/ds has wrong value at (3,1,0,0).")
      self.assertEqual(dsds[3,1,0,1],0.0,"ds/ds has wrong value at (3,1,0,1).")
      self.assertEqual(dsds[3,1,0,2],0.0,"ds/ds has wrong value at (3,1,0,2).")
      self.assertEqual(dsds[3,1,0,3],0.0,"ds/ds has wrong value at (3,1,0,3).")
      self.assertEqual(dsds[3,1,0,4],0.0,"ds/ds has wrong value at (3,1,0,4).")
      self.assertEqual(dsds[3,1,1,0],0.0,"ds/ds has wrong value at (3,1,1,0).")
      self.assertEqual(dsds[3,1,1,1],0.0,"ds/ds has wrong value at (3,1,1,1).")
      self.assertEqual(dsds[3,1,1,2],0.0,"ds/ds has wrong value at (3,1,1,2).")
      self.assertEqual(dsds[3,1,1,3],0.0,"ds/ds has wrong value at (3,1,1,3).")
      self.assertEqual(dsds[3,1,1,4],0.0,"ds/ds has wrong value at (3,1,1,4).")
      self.assertEqual(dsds[3,1,2,0],0.0,"ds/ds has wrong value at (3,1,2,0).")
      self.assertEqual(dsds[3,1,2,1],0.0,"ds/ds has wrong value at (3,1,2,1).")
      self.assertEqual(dsds[3,1,2,2],0.0,"ds/ds has wrong value at (3,1,2,2).")
      self.assertEqual(dsds[3,1,2,3],0.0,"ds/ds has wrong value at (3,1,2,3).")
      self.assertEqual(dsds[3,1,2,4],0.0,"ds/ds has wrong value at (3,1,2,4).")
      self.assertEqual(dsds[3,1,3,0],0.0,"ds/ds has wrong value at (3,1,3,0).")
      self.assertEqual(dsds[3,1,3,1],1.0,"ds/ds has wrong value at (3,1,3,1).")
      self.assertEqual(dsds[3,1,3,2],0.0,"ds/ds has wrong value at (3,1,3,2).")
      self.assertEqual(dsds[3,1,3,3],0.0,"ds/ds has wrong value at (3,1,3,3).")
      self.assertEqual(dsds[3,1,3,4],0.0,"ds/ds has wrong value at (3,1,3,4).")
      self.assertEqual(dsds[3,2,0,0],0.0,"ds/ds has wrong value at (3,2,0,0).")
      self.assertEqual(dsds[3,2,0,1],0.0,"ds/ds has wrong value at (3,2,0,1).")
      self.assertEqual(dsds[3,2,0,2],0.0,"ds/ds has wrong value at (3,2,0,2).")
      self.assertEqual(dsds[3,2,0,3],0.0,"ds/ds has wrong value at (3,2,0,3).")
      self.assertEqual(dsds[3,2,0,4],0.0,"ds/ds has wrong value at (3,2,0,4).")
      self.assertEqual(dsds[3,2,1,0],0.0,"ds/ds has wrong value at (3,2,1,0).")
      self.assertEqual(dsds[3,2,1,1],0.0,"ds/ds has wrong value at (3,2,1,1).")
      self.assertEqual(dsds[3,2,1,2],0.0,"ds/ds has wrong value at (3,2,1,2).")
      self.assertEqual(dsds[3,2,1,3],0.0,"ds/ds has wrong value at (3,2,1,3).")
      self.assertEqual(dsds[3,2,1,4],0.0,"ds/ds has wrong value at (3,2,1,4).")
      self.assertEqual(dsds[3,2,2,0],0.0,"ds/ds has wrong value at (3,2,2,0).")
      self.assertEqual(dsds[3,2,2,1],0.0,"ds/ds has wrong value at (3,2,2,1).")
      self.assertEqual(dsds[3,2,2,2],0.0,"ds/ds has wrong value at (3,2,2,2).")
      self.assertEqual(dsds[3,2,2,3],0.0,"ds/ds has wrong value at (3,2,2,3).")
      self.assertEqual(dsds[3,2,2,4],0.0,"ds/ds has wrong value at (3,2,2,4).")
      self.assertEqual(dsds[3,2,3,0],0.0,"ds/ds has wrong value at (3,2,3,0).")
      self.assertEqual(dsds[3,2,3,1],0.0,"ds/ds has wrong value at (3,2,3,1).")
      self.assertEqual(dsds[3,2,3,2],1.0,"ds/ds has wrong value at (3,2,3,2).")
      self.assertEqual(dsds[3,2,3,3],0.0,"ds/ds has wrong value at (3,2,3,3).")
      self.assertEqual(dsds[3,2,3,4],0.0,"ds/ds has wrong value at (3,2,3,4).")
      self.assertEqual(dsds[3,3,0,0],0.0,"ds/ds has wrong value at (3,3,0,0).")
      self.assertEqual(dsds[3,3,0,1],0.0,"ds/ds has wrong value at (3,3,0,1).")
      self.assertEqual(dsds[3,3,0,2],0.0,"ds/ds has wrong value at (3,3,0,2).")
      self.assertEqual(dsds[3,3,0,3],0.0,"ds/ds has wrong value at (3,3,0,3).")
      self.assertEqual(dsds[3,3,0,4],0.0,"ds/ds has wrong value at (3,3,0,4).")
      self.assertEqual(dsds[3,3,1,0],0.0,"ds/ds has wrong value at (3,3,1,0).")
      self.assertEqual(dsds[3,3,1,1],0.0,"ds/ds has wrong value at (3,3,1,1).")
      self.assertEqual(dsds[3,3,1,2],0.0,"ds/ds has wrong value at (3,3,1,2).")
      self.assertEqual(dsds[3,3,1,3],0.0,"ds/ds has wrong value at (3,3,1,3).")
      self.assertEqual(dsds[3,3,1,4],0.0,"ds/ds has wrong value at (3,3,1,4).")
      self.assertEqual(dsds[3,3,2,0],0.0,"ds/ds has wrong value at (3,3,2,0).")
      self.assertEqual(dsds[3,3,2,1],0.0,"ds/ds has wrong value at (3,3,2,1).")
      self.assertEqual(dsds[3,3,2,2],0.0,"ds/ds has wrong value at (3,3,2,2).")
      self.assertEqual(dsds[3,3,2,3],0.0,"ds/ds has wrong value at (3,3,2,3).")
      self.assertEqual(dsds[3,3,2,4],0.0,"ds/ds has wrong value at (3,3,2,4).")
      self.assertEqual(dsds[3,3,3,0],0.0,"ds/ds has wrong value at (3,3,3,0).")
      self.assertEqual(dsds[3,3,3,1],0.0,"ds/ds has wrong value at (3,3,3,1).")
      self.assertEqual(dsds[3,3,3,2],0.0,"ds/ds has wrong value at (3,3,3,2).")
      self.assertEqual(dsds[3,3,3,3],1.0,"ds/ds has wrong value at (3,3,3,3).")
      self.assertEqual(dsds[3,3,3,4],0.0,"ds/ds has wrong value at (3,3,3,4).")
      self.assertEqual(dsds[3,4,0,0],0.0,"ds/ds has wrong value at (3,4,0,0).")
      self.assertEqual(dsds[3,4,0,1],0.0,"ds/ds has wrong value at (3,4,0,1).")
      self.assertEqual(dsds[3,4,0,2],0.0,"ds/ds has wrong value at (3,4,0,2).")
      self.assertEqual(dsds[3,4,0,3],0.0,"ds/ds has wrong value at (3,4,0,3).")
      self.assertEqual(dsds[3,4,0,4],0.0,"ds/ds has wrong value at (3,4,0,4).")
      self.assertEqual(dsds[3,4,1,0],0.0,"ds/ds has wrong value at (3,4,1,0).")
      self.assertEqual(dsds[3,4,1,1],0.0,"ds/ds has wrong value at (3,4,1,1).")
      self.assertEqual(dsds[3,4,1,2],0.0,"ds/ds has wrong value at (3,4,1,2).")
      self.assertEqual(dsds[3,4,1,3],0.0,"ds/ds has wrong value at (3,4,1,3).")
      self.assertEqual(dsds[3,4,1,4],0.0,"ds/ds has wrong value at (3,4,1,4).")
      self.assertEqual(dsds[3,4,2,0],0.0,"ds/ds has wrong value at (3,4,2,0).")
      self.assertEqual(dsds[3,4,2,1],0.0,"ds/ds has wrong value at (3,4,2,1).")
      self.assertEqual(dsds[3,4,2,2],0.0,"ds/ds has wrong value at (3,4,2,2).")
      self.assertEqual(dsds[3,4,2,3],0.0,"ds/ds has wrong value at (3,4,2,3).")
      self.assertEqual(dsds[3,4,2,4],0.0,"ds/ds has wrong value at (3,4,2,4).")
      self.assertEqual(dsds[3,4,3,0],0.0,"ds/ds has wrong value at (3,4,3,0).")
      self.assertEqual(dsds[3,4,3,1],0.0,"ds/ds has wrong value at (3,4,3,1).")
      self.assertEqual(dsds[3,4,3,2],0.0,"ds/ds has wrong value at (3,4,3,2).")
      self.assertEqual(dsds[3,4,3,3],0.0,"ds/ds has wrong value at (3,4,3,3).")
      self.assertEqual(dsds[3,4,3,4],1.0,"ds/ds has wrong value at (3,4,3,4).")

      self.assertEqual(s.getArgument(0),s2,"wrong argument 0.")
      sa=s.getSubstitutedArguments({s2:-10})
      self.assertEqual(len(sa),1,"wrong number of substituted arguments")
      self.assertEqual(sa[0],-10,"wrongly substituted argument 0.")

      self.assertFalse(s.isAppropriateValue(10.)," 10. is not appropriate substitute")
      self.assertFalse(s.isAppropriateValue(10)," 10 is not appropriate substitute")
      self.assertTrue(s.isAppropriateValue(Symbol(shape=(4, 5),dim=3)),"Symbol is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(Symbol(shape=(4, 5),dim=4)),"Symbol is not appropriate substitute (dim)")
      self.assertFalse(s.isAppropriateValue(escript.Data(0.,(5,),self.functionspace)),"Data is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(numpy.zeros((4, 5))),"ndarray is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(numpy.zeros((5,))),"ndarray is not appropriate substitute (shape)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_Symbol_rank2_d3_nargs2(self):
      s2=Symbol(shape=())
      s=Symbol(shape=(4, 5),dim=3,args=[1, -1.0])
      self.assertEqual(s.getRank(),2,"wrong rank.")
      self.assertEqual(s.getShape(),(4, 5),"wrong shape.")
      self.assertEqual(s.getDim(),3,"wrong spatial dimension.")
      self.assertEqual(s.getArgument(),[1, -1.0],"wrong arguments.")

      ss=s.substitute({s:numpy.zeros((4, 5))})
      self.assertTrue(isinstance(ss,numpy.ndarray),"value after substitution is not ndarray.")
      self.assertEqual(ss.shape,(4, 5),"value after substitution has not expected shape")
      try:
        s.substitute({s:numpy.zeros((5,))})
        fail("illegal substition was successful")
      except TypeError:
        pass

      dsdarg=s.diff(Symbol(shape=()))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds() has wrong type.")
      self.assertEqual(dsdarg.shape,(4, 5),"ds/ds() has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds() has wrong value.")
      dsdarg=s.diff(Symbol(shape=(2,)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(2,) has wrong type.")
      self.assertEqual(dsdarg.shape,(4, 5, 2),"ds/ds(2,) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(2,) has wrong value.")
      dsdarg=s.diff(Symbol(shape=(4, 5)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(4, 5) has wrong type.")
      self.assertEqual(dsdarg.shape,(4, 5, 4, 5),"ds/ds(4, 5) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(4, 5) has wrong value.")

      dsds=s.diff(s)
      self.assertTrue(isinstance(dsds,numpy.ndarray),"ds/ds has wrong type.")
      self.assertEqual(dsds.shape,(4, 5, 4, 5),"ds/ds has wrong shape.")
      self.assertEqual(dsds[0,0,0,0],1.0,"ds/ds has wrong value at (0,0,0,0).")
      self.assertEqual(dsds[0,0,0,1],0.0,"ds/ds has wrong value at (0,0,0,1).")
      self.assertEqual(dsds[0,0,0,2],0.0,"ds/ds has wrong value at (0,0,0,2).")
      self.assertEqual(dsds[0,0,0,3],0.0,"ds/ds has wrong value at (0,0,0,3).")
      self.assertEqual(dsds[0,0,0,4],0.0,"ds/ds has wrong value at (0,0,0,4).")
      self.assertEqual(dsds[0,0,1,0],0.0,"ds/ds has wrong value at (0,0,1,0).")
      self.assertEqual(dsds[0,0,1,1],0.0,"ds/ds has wrong value at (0,0,1,1).")
      self.assertEqual(dsds[0,0,1,2],0.0,"ds/ds has wrong value at (0,0,1,2).")
      self.assertEqual(dsds[0,0,1,3],0.0,"ds/ds has wrong value at (0,0,1,3).")
      self.assertEqual(dsds[0,0,1,4],0.0,"ds/ds has wrong value at (0,0,1,4).")
      self.assertEqual(dsds[0,0,2,0],0.0,"ds/ds has wrong value at (0,0,2,0).")
      self.assertEqual(dsds[0,0,2,1],0.0,"ds/ds has wrong value at (0,0,2,1).")
      self.assertEqual(dsds[0,0,2,2],0.0,"ds/ds has wrong value at (0,0,2,2).")
      self.assertEqual(dsds[0,0,2,3],0.0,"ds/ds has wrong value at (0,0,2,3).")
      self.assertEqual(dsds[0,0,2,4],0.0,"ds/ds has wrong value at (0,0,2,4).")
      self.assertEqual(dsds[0,0,3,0],0.0,"ds/ds has wrong value at (0,0,3,0).")
      self.assertEqual(dsds[0,0,3,1],0.0,"ds/ds has wrong value at (0,0,3,1).")
      self.assertEqual(dsds[0,0,3,2],0.0,"ds/ds has wrong value at (0,0,3,2).")
      self.assertEqual(dsds[0,0,3,3],0.0,"ds/ds has wrong value at (0,0,3,3).")
      self.assertEqual(dsds[0,0,3,4],0.0,"ds/ds has wrong value at (0,0,3,4).")
      self.assertEqual(dsds[0,1,0,0],0.0,"ds/ds has wrong value at (0,1,0,0).")
      self.assertEqual(dsds[0,1,0,1],1.0,"ds/ds has wrong value at (0,1,0,1).")
      self.assertEqual(dsds[0,1,0,2],0.0,"ds/ds has wrong value at (0,1,0,2).")
      self.assertEqual(dsds[0,1,0,3],0.0,"ds/ds has wrong value at (0,1,0,3).")
      self.assertEqual(dsds[0,1,0,4],0.0,"ds/ds has wrong value at (0,1,0,4).")
      self.assertEqual(dsds[0,1,1,0],0.0,"ds/ds has wrong value at (0,1,1,0).")
      self.assertEqual(dsds[0,1,1,1],0.0,"ds/ds has wrong value at (0,1,1,1).")
      self.assertEqual(dsds[0,1,1,2],0.0,"ds/ds has wrong value at (0,1,1,2).")
      self.assertEqual(dsds[0,1,1,3],0.0,"ds/ds has wrong value at (0,1,1,3).")
      self.assertEqual(dsds[0,1,1,4],0.0,"ds/ds has wrong value at (0,1,1,4).")
      self.assertEqual(dsds[0,1,2,0],0.0,"ds/ds has wrong value at (0,1,2,0).")
      self.assertEqual(dsds[0,1,2,1],0.0,"ds/ds has wrong value at (0,1,2,1).")
      self.assertEqual(dsds[0,1,2,2],0.0,"ds/ds has wrong value at (0,1,2,2).")
      self.assertEqual(dsds[0,1,2,3],0.0,"ds/ds has wrong value at (0,1,2,3).")
      self.assertEqual(dsds[0,1,2,4],0.0,"ds/ds has wrong value at (0,1,2,4).")
      self.assertEqual(dsds[0,1,3,0],0.0,"ds/ds has wrong value at (0,1,3,0).")
      self.assertEqual(dsds[0,1,3,1],0.0,"ds/ds has wrong value at (0,1,3,1).")
      self.assertEqual(dsds[0,1,3,2],0.0,"ds/ds has wrong value at (0,1,3,2).")
      self.assertEqual(dsds[0,1,3,3],0.0,"ds/ds has wrong value at (0,1,3,3).")
      self.assertEqual(dsds[0,1,3,4],0.0,"ds/ds has wrong value at (0,1,3,4).")
      self.assertEqual(dsds[0,2,0,0],0.0,"ds/ds has wrong value at (0,2,0,0).")
      self.assertEqual(dsds[0,2,0,1],0.0,"ds/ds has wrong value at (0,2,0,1).")
      self.assertEqual(dsds[0,2,0,2],1.0,"ds/ds has wrong value at (0,2,0,2).")
      self.assertEqual(dsds[0,2,0,3],0.0,"ds/ds has wrong value at (0,2,0,3).")
      self.assertEqual(dsds[0,2,0,4],0.0,"ds/ds has wrong value at (0,2,0,4).")
      self.assertEqual(dsds[0,2,1,0],0.0,"ds/ds has wrong value at (0,2,1,0).")
      self.assertEqual(dsds[0,2,1,1],0.0,"ds/ds has wrong value at (0,2,1,1).")
      self.assertEqual(dsds[0,2,1,2],0.0,"ds/ds has wrong value at (0,2,1,2).")
      self.assertEqual(dsds[0,2,1,3],0.0,"ds/ds has wrong value at (0,2,1,3).")
      self.assertEqual(dsds[0,2,1,4],0.0,"ds/ds has wrong value at (0,2,1,4).")
      self.assertEqual(dsds[0,2,2,0],0.0,"ds/ds has wrong value at (0,2,2,0).")
      self.assertEqual(dsds[0,2,2,1],0.0,"ds/ds has wrong value at (0,2,2,1).")
      self.assertEqual(dsds[0,2,2,2],0.0,"ds/ds has wrong value at (0,2,2,2).")
      self.assertEqual(dsds[0,2,2,3],0.0,"ds/ds has wrong value at (0,2,2,3).")
      self.assertEqual(dsds[0,2,2,4],0.0,"ds/ds has wrong value at (0,2,2,4).")
      self.assertEqual(dsds[0,2,3,0],0.0,"ds/ds has wrong value at (0,2,3,0).")
      self.assertEqual(dsds[0,2,3,1],0.0,"ds/ds has wrong value at (0,2,3,1).")
      self.assertEqual(dsds[0,2,3,2],0.0,"ds/ds has wrong value at (0,2,3,2).")
      self.assertEqual(dsds[0,2,3,3],0.0,"ds/ds has wrong value at (0,2,3,3).")
      self.assertEqual(dsds[0,2,3,4],0.0,"ds/ds has wrong value at (0,2,3,4).")
      self.assertEqual(dsds[0,3,0,0],0.0,"ds/ds has wrong value at (0,3,0,0).")
      self.assertEqual(dsds[0,3,0,1],0.0,"ds/ds has wrong value at (0,3,0,1).")
      self.assertEqual(dsds[0,3,0,2],0.0,"ds/ds has wrong value at (0,3,0,2).")
      self.assertEqual(dsds[0,3,0,3],1.0,"ds/ds has wrong value at (0,3,0,3).")
      self.assertEqual(dsds[0,3,0,4],0.0,"ds/ds has wrong value at (0,3,0,4).")
      self.assertEqual(dsds[0,3,1,0],0.0,"ds/ds has wrong value at (0,3,1,0).")
      self.assertEqual(dsds[0,3,1,1],0.0,"ds/ds has wrong value at (0,3,1,1).")
      self.assertEqual(dsds[0,3,1,2],0.0,"ds/ds has wrong value at (0,3,1,2).")
      self.assertEqual(dsds[0,3,1,3],0.0,"ds/ds has wrong value at (0,3,1,3).")
      self.assertEqual(dsds[0,3,1,4],0.0,"ds/ds has wrong value at (0,3,1,4).")
      self.assertEqual(dsds[0,3,2,0],0.0,"ds/ds has wrong value at (0,3,2,0).")
      self.assertEqual(dsds[0,3,2,1],0.0,"ds/ds has wrong value at (0,3,2,1).")
      self.assertEqual(dsds[0,3,2,2],0.0,"ds/ds has wrong value at (0,3,2,2).")
      self.assertEqual(dsds[0,3,2,3],0.0,"ds/ds has wrong value at (0,3,2,3).")
      self.assertEqual(dsds[0,3,2,4],0.0,"ds/ds has wrong value at (0,3,2,4).")
      self.assertEqual(dsds[0,3,3,0],0.0,"ds/ds has wrong value at (0,3,3,0).")
      self.assertEqual(dsds[0,3,3,1],0.0,"ds/ds has wrong value at (0,3,3,1).")
      self.assertEqual(dsds[0,3,3,2],0.0,"ds/ds has wrong value at (0,3,3,2).")
      self.assertEqual(dsds[0,3,3,3],0.0,"ds/ds has wrong value at (0,3,3,3).")
      self.assertEqual(dsds[0,3,3,4],0.0,"ds/ds has wrong value at (0,3,3,4).")
      self.assertEqual(dsds[0,4,0,0],0.0,"ds/ds has wrong value at (0,4,0,0).")
      self.assertEqual(dsds[0,4,0,1],0.0,"ds/ds has wrong value at (0,4,0,1).")
      self.assertEqual(dsds[0,4,0,2],0.0,"ds/ds has wrong value at (0,4,0,2).")
      self.assertEqual(dsds[0,4,0,3],0.0,"ds/ds has wrong value at (0,4,0,3).")
      self.assertEqual(dsds[0,4,0,4],1.0,"ds/ds has wrong value at (0,4,0,4).")
      self.assertEqual(dsds[0,4,1,0],0.0,"ds/ds has wrong value at (0,4,1,0).")
      self.assertEqual(dsds[0,4,1,1],0.0,"ds/ds has wrong value at (0,4,1,1).")
      self.assertEqual(dsds[0,4,1,2],0.0,"ds/ds has wrong value at (0,4,1,2).")
      self.assertEqual(dsds[0,4,1,3],0.0,"ds/ds has wrong value at (0,4,1,3).")
      self.assertEqual(dsds[0,4,1,4],0.0,"ds/ds has wrong value at (0,4,1,4).")
      self.assertEqual(dsds[0,4,2,0],0.0,"ds/ds has wrong value at (0,4,2,0).")
      self.assertEqual(dsds[0,4,2,1],0.0,"ds/ds has wrong value at (0,4,2,1).")
      self.assertEqual(dsds[0,4,2,2],0.0,"ds/ds has wrong value at (0,4,2,2).")
      self.assertEqual(dsds[0,4,2,3],0.0,"ds/ds has wrong value at (0,4,2,3).")
      self.assertEqual(dsds[0,4,2,4],0.0,"ds/ds has wrong value at (0,4,2,4).")
      self.assertEqual(dsds[0,4,3,0],0.0,"ds/ds has wrong value at (0,4,3,0).")
      self.assertEqual(dsds[0,4,3,1],0.0,"ds/ds has wrong value at (0,4,3,1).")
      self.assertEqual(dsds[0,4,3,2],0.0,"ds/ds has wrong value at (0,4,3,2).")
      self.assertEqual(dsds[0,4,3,3],0.0,"ds/ds has wrong value at (0,4,3,3).")
      self.assertEqual(dsds[0,4,3,4],0.0,"ds/ds has wrong value at (0,4,3,4).")
      self.assertEqual(dsds[1,0,0,0],0.0,"ds/ds has wrong value at (1,0,0,0).")
      self.assertEqual(dsds[1,0,0,1],0.0,"ds/ds has wrong value at (1,0,0,1).")
      self.assertEqual(dsds[1,0,0,2],0.0,"ds/ds has wrong value at (1,0,0,2).")
      self.assertEqual(dsds[1,0,0,3],0.0,"ds/ds has wrong value at (1,0,0,3).")
      self.assertEqual(dsds[1,0,0,4],0.0,"ds/ds has wrong value at (1,0,0,4).")
      self.assertEqual(dsds[1,0,1,0],1.0,"ds/ds has wrong value at (1,0,1,0).")
      self.assertEqual(dsds[1,0,1,1],0.0,"ds/ds has wrong value at (1,0,1,1).")
      self.assertEqual(dsds[1,0,1,2],0.0,"ds/ds has wrong value at (1,0,1,2).")
      self.assertEqual(dsds[1,0,1,3],0.0,"ds/ds has wrong value at (1,0,1,3).")
      self.assertEqual(dsds[1,0,1,4],0.0,"ds/ds has wrong value at (1,0,1,4).")
      self.assertEqual(dsds[1,0,2,0],0.0,"ds/ds has wrong value at (1,0,2,0).")
      self.assertEqual(dsds[1,0,2,1],0.0,"ds/ds has wrong value at (1,0,2,1).")
      self.assertEqual(dsds[1,0,2,2],0.0,"ds/ds has wrong value at (1,0,2,2).")
      self.assertEqual(dsds[1,0,2,3],0.0,"ds/ds has wrong value at (1,0,2,3).")
      self.assertEqual(dsds[1,0,2,4],0.0,"ds/ds has wrong value at (1,0,2,4).")
      self.assertEqual(dsds[1,0,3,0],0.0,"ds/ds has wrong value at (1,0,3,0).")
      self.assertEqual(dsds[1,0,3,1],0.0,"ds/ds has wrong value at (1,0,3,1).")
      self.assertEqual(dsds[1,0,3,2],0.0,"ds/ds has wrong value at (1,0,3,2).")
      self.assertEqual(dsds[1,0,3,3],0.0,"ds/ds has wrong value at (1,0,3,3).")
      self.assertEqual(dsds[1,0,3,4],0.0,"ds/ds has wrong value at (1,0,3,4).")
      self.assertEqual(dsds[1,1,0,0],0.0,"ds/ds has wrong value at (1,1,0,0).")
      self.assertEqual(dsds[1,1,0,1],0.0,"ds/ds has wrong value at (1,1,0,1).")
      self.assertEqual(dsds[1,1,0,2],0.0,"ds/ds has wrong value at (1,1,0,2).")
      self.assertEqual(dsds[1,1,0,3],0.0,"ds/ds has wrong value at (1,1,0,3).")
      self.assertEqual(dsds[1,1,0,4],0.0,"ds/ds has wrong value at (1,1,0,4).")
      self.assertEqual(dsds[1,1,1,0],0.0,"ds/ds has wrong value at (1,1,1,0).")
      self.assertEqual(dsds[1,1,1,1],1.0,"ds/ds has wrong value at (1,1,1,1).")
      self.assertEqual(dsds[1,1,1,2],0.0,"ds/ds has wrong value at (1,1,1,2).")
      self.assertEqual(dsds[1,1,1,3],0.0,"ds/ds has wrong value at (1,1,1,3).")
      self.assertEqual(dsds[1,1,1,4],0.0,"ds/ds has wrong value at (1,1,1,4).")
      self.assertEqual(dsds[1,1,2,0],0.0,"ds/ds has wrong value at (1,1,2,0).")
      self.assertEqual(dsds[1,1,2,1],0.0,"ds/ds has wrong value at (1,1,2,1).")
      self.assertEqual(dsds[1,1,2,2],0.0,"ds/ds has wrong value at (1,1,2,2).")
      self.assertEqual(dsds[1,1,2,3],0.0,"ds/ds has wrong value at (1,1,2,3).")
      self.assertEqual(dsds[1,1,2,4],0.0,"ds/ds has wrong value at (1,1,2,4).")
      self.assertEqual(dsds[1,1,3,0],0.0,"ds/ds has wrong value at (1,1,3,0).")
      self.assertEqual(dsds[1,1,3,1],0.0,"ds/ds has wrong value at (1,1,3,1).")
      self.assertEqual(dsds[1,1,3,2],0.0,"ds/ds has wrong value at (1,1,3,2).")
      self.assertEqual(dsds[1,1,3,3],0.0,"ds/ds has wrong value at (1,1,3,3).")
      self.assertEqual(dsds[1,1,3,4],0.0,"ds/ds has wrong value at (1,1,3,4).")
      self.assertEqual(dsds[1,2,0,0],0.0,"ds/ds has wrong value at (1,2,0,0).")
      self.assertEqual(dsds[1,2,0,1],0.0,"ds/ds has wrong value at (1,2,0,1).")
      self.assertEqual(dsds[1,2,0,2],0.0,"ds/ds has wrong value at (1,2,0,2).")
      self.assertEqual(dsds[1,2,0,3],0.0,"ds/ds has wrong value at (1,2,0,3).")
      self.assertEqual(dsds[1,2,0,4],0.0,"ds/ds has wrong value at (1,2,0,4).")
      self.assertEqual(dsds[1,2,1,0],0.0,"ds/ds has wrong value at (1,2,1,0).")
      self.assertEqual(dsds[1,2,1,1],0.0,"ds/ds has wrong value at (1,2,1,1).")
      self.assertEqual(dsds[1,2,1,2],1.0,"ds/ds has wrong value at (1,2,1,2).")
      self.assertEqual(dsds[1,2,1,3],0.0,"ds/ds has wrong value at (1,2,1,3).")
      self.assertEqual(dsds[1,2,1,4],0.0,"ds/ds has wrong value at (1,2,1,4).")
      self.assertEqual(dsds[1,2,2,0],0.0,"ds/ds has wrong value at (1,2,2,0).")
      self.assertEqual(dsds[1,2,2,1],0.0,"ds/ds has wrong value at (1,2,2,1).")
      self.assertEqual(dsds[1,2,2,2],0.0,"ds/ds has wrong value at (1,2,2,2).")
      self.assertEqual(dsds[1,2,2,3],0.0,"ds/ds has wrong value at (1,2,2,3).")
      self.assertEqual(dsds[1,2,2,4],0.0,"ds/ds has wrong value at (1,2,2,4).")
      self.assertEqual(dsds[1,2,3,0],0.0,"ds/ds has wrong value at (1,2,3,0).")
      self.assertEqual(dsds[1,2,3,1],0.0,"ds/ds has wrong value at (1,2,3,1).")
      self.assertEqual(dsds[1,2,3,2],0.0,"ds/ds has wrong value at (1,2,3,2).")
      self.assertEqual(dsds[1,2,3,3],0.0,"ds/ds has wrong value at (1,2,3,3).")
      self.assertEqual(dsds[1,2,3,4],0.0,"ds/ds has wrong value at (1,2,3,4).")
      self.assertEqual(dsds[1,3,0,0],0.0,"ds/ds has wrong value at (1,3,0,0).")
      self.assertEqual(dsds[1,3,0,1],0.0,"ds/ds has wrong value at (1,3,0,1).")
      self.assertEqual(dsds[1,3,0,2],0.0,"ds/ds has wrong value at (1,3,0,2).")
      self.assertEqual(dsds[1,3,0,3],0.0,"ds/ds has wrong value at (1,3,0,3).")
      self.assertEqual(dsds[1,3,0,4],0.0,"ds/ds has wrong value at (1,3,0,4).")
      self.assertEqual(dsds[1,3,1,0],0.0,"ds/ds has wrong value at (1,3,1,0).")
      self.assertEqual(dsds[1,3,1,1],0.0,"ds/ds has wrong value at (1,3,1,1).")
      self.assertEqual(dsds[1,3,1,2],0.0,"ds/ds has wrong value at (1,3,1,2).")
      self.assertEqual(dsds[1,3,1,3],1.0,"ds/ds has wrong value at (1,3,1,3).")
      self.assertEqual(dsds[1,3,1,4],0.0,"ds/ds has wrong value at (1,3,1,4).")
      self.assertEqual(dsds[1,3,2,0],0.0,"ds/ds has wrong value at (1,3,2,0).")
      self.assertEqual(dsds[1,3,2,1],0.0,"ds/ds has wrong value at (1,3,2,1).")
      self.assertEqual(dsds[1,3,2,2],0.0,"ds/ds has wrong value at (1,3,2,2).")
      self.assertEqual(dsds[1,3,2,3],0.0,"ds/ds has wrong value at (1,3,2,3).")
      self.assertEqual(dsds[1,3,2,4],0.0,"ds/ds has wrong value at (1,3,2,4).")
      self.assertEqual(dsds[1,3,3,0],0.0,"ds/ds has wrong value at (1,3,3,0).")
      self.assertEqual(dsds[1,3,3,1],0.0,"ds/ds has wrong value at (1,3,3,1).")
      self.assertEqual(dsds[1,3,3,2],0.0,"ds/ds has wrong value at (1,3,3,2).")
      self.assertEqual(dsds[1,3,3,3],0.0,"ds/ds has wrong value at (1,3,3,3).")
      self.assertEqual(dsds[1,3,3,4],0.0,"ds/ds has wrong value at (1,3,3,4).")
      self.assertEqual(dsds[1,4,0,0],0.0,"ds/ds has wrong value at (1,4,0,0).")
      self.assertEqual(dsds[1,4,0,1],0.0,"ds/ds has wrong value at (1,4,0,1).")
      self.assertEqual(dsds[1,4,0,2],0.0,"ds/ds has wrong value at (1,4,0,2).")
      self.assertEqual(dsds[1,4,0,3],0.0,"ds/ds has wrong value at (1,4,0,3).")
      self.assertEqual(dsds[1,4,0,4],0.0,"ds/ds has wrong value at (1,4,0,4).")
      self.assertEqual(dsds[1,4,1,0],0.0,"ds/ds has wrong value at (1,4,1,0).")
      self.assertEqual(dsds[1,4,1,1],0.0,"ds/ds has wrong value at (1,4,1,1).")
      self.assertEqual(dsds[1,4,1,2],0.0,"ds/ds has wrong value at (1,4,1,2).")
      self.assertEqual(dsds[1,4,1,3],0.0,"ds/ds has wrong value at (1,4,1,3).")
      self.assertEqual(dsds[1,4,1,4],1.0,"ds/ds has wrong value at (1,4,1,4).")
      self.assertEqual(dsds[1,4,2,0],0.0,"ds/ds has wrong value at (1,4,2,0).")
      self.assertEqual(dsds[1,4,2,1],0.0,"ds/ds has wrong value at (1,4,2,1).")
      self.assertEqual(dsds[1,4,2,2],0.0,"ds/ds has wrong value at (1,4,2,2).")
      self.assertEqual(dsds[1,4,2,3],0.0,"ds/ds has wrong value at (1,4,2,3).")
      self.assertEqual(dsds[1,4,2,4],0.0,"ds/ds has wrong value at (1,4,2,4).")
      self.assertEqual(dsds[1,4,3,0],0.0,"ds/ds has wrong value at (1,4,3,0).")
      self.assertEqual(dsds[1,4,3,1],0.0,"ds/ds has wrong value at (1,4,3,1).")
      self.assertEqual(dsds[1,4,3,2],0.0,"ds/ds has wrong value at (1,4,3,2).")
      self.assertEqual(dsds[1,4,3,3],0.0,"ds/ds has wrong value at (1,4,3,3).")
      self.assertEqual(dsds[1,4,3,4],0.0,"ds/ds has wrong value at (1,4,3,4).")
      self.assertEqual(dsds[2,0,0,0],0.0,"ds/ds has wrong value at (2,0,0,0).")
      self.assertEqual(dsds[2,0,0,1],0.0,"ds/ds has wrong value at (2,0,0,1).")
      self.assertEqual(dsds[2,0,0,2],0.0,"ds/ds has wrong value at (2,0,0,2).")
      self.assertEqual(dsds[2,0,0,3],0.0,"ds/ds has wrong value at (2,0,0,3).")
      self.assertEqual(dsds[2,0,0,4],0.0,"ds/ds has wrong value at (2,0,0,4).")
      self.assertEqual(dsds[2,0,1,0],0.0,"ds/ds has wrong value at (2,0,1,0).")
      self.assertEqual(dsds[2,0,1,1],0.0,"ds/ds has wrong value at (2,0,1,1).")
      self.assertEqual(dsds[2,0,1,2],0.0,"ds/ds has wrong value at (2,0,1,2).")
      self.assertEqual(dsds[2,0,1,3],0.0,"ds/ds has wrong value at (2,0,1,3).")
      self.assertEqual(dsds[2,0,1,4],0.0,"ds/ds has wrong value at (2,0,1,4).")
      self.assertEqual(dsds[2,0,2,0],1.0,"ds/ds has wrong value at (2,0,2,0).")
      self.assertEqual(dsds[2,0,2,1],0.0,"ds/ds has wrong value at (2,0,2,1).")
      self.assertEqual(dsds[2,0,2,2],0.0,"ds/ds has wrong value at (2,0,2,2).")
      self.assertEqual(dsds[2,0,2,3],0.0,"ds/ds has wrong value at (2,0,2,3).")
      self.assertEqual(dsds[2,0,2,4],0.0,"ds/ds has wrong value at (2,0,2,4).")
      self.assertEqual(dsds[2,0,3,0],0.0,"ds/ds has wrong value at (2,0,3,0).")
      self.assertEqual(dsds[2,0,3,1],0.0,"ds/ds has wrong value at (2,0,3,1).")
      self.assertEqual(dsds[2,0,3,2],0.0,"ds/ds has wrong value at (2,0,3,2).")
      self.assertEqual(dsds[2,0,3,3],0.0,"ds/ds has wrong value at (2,0,3,3).")
      self.assertEqual(dsds[2,0,3,4],0.0,"ds/ds has wrong value at (2,0,3,4).")
      self.assertEqual(dsds[2,1,0,0],0.0,"ds/ds has wrong value at (2,1,0,0).")
      self.assertEqual(dsds[2,1,0,1],0.0,"ds/ds has wrong value at (2,1,0,1).")
      self.assertEqual(dsds[2,1,0,2],0.0,"ds/ds has wrong value at (2,1,0,2).")
      self.assertEqual(dsds[2,1,0,3],0.0,"ds/ds has wrong value at (2,1,0,3).")
      self.assertEqual(dsds[2,1,0,4],0.0,"ds/ds has wrong value at (2,1,0,4).")
      self.assertEqual(dsds[2,1,1,0],0.0,"ds/ds has wrong value at (2,1,1,0).")
      self.assertEqual(dsds[2,1,1,1],0.0,"ds/ds has wrong value at (2,1,1,1).")
      self.assertEqual(dsds[2,1,1,2],0.0,"ds/ds has wrong value at (2,1,1,2).")
      self.assertEqual(dsds[2,1,1,3],0.0,"ds/ds has wrong value at (2,1,1,3).")
      self.assertEqual(dsds[2,1,1,4],0.0,"ds/ds has wrong value at (2,1,1,4).")
      self.assertEqual(dsds[2,1,2,0],0.0,"ds/ds has wrong value at (2,1,2,0).")
      self.assertEqual(dsds[2,1,2,1],1.0,"ds/ds has wrong value at (2,1,2,1).")
      self.assertEqual(dsds[2,1,2,2],0.0,"ds/ds has wrong value at (2,1,2,2).")
      self.assertEqual(dsds[2,1,2,3],0.0,"ds/ds has wrong value at (2,1,2,3).")
      self.assertEqual(dsds[2,1,2,4],0.0,"ds/ds has wrong value at (2,1,2,4).")
      self.assertEqual(dsds[2,1,3,0],0.0,"ds/ds has wrong value at (2,1,3,0).")
      self.assertEqual(dsds[2,1,3,1],0.0,"ds/ds has wrong value at (2,1,3,1).")
      self.assertEqual(dsds[2,1,3,2],0.0,"ds/ds has wrong value at (2,1,3,2).")
      self.assertEqual(dsds[2,1,3,3],0.0,"ds/ds has wrong value at (2,1,3,3).")
      self.assertEqual(dsds[2,1,3,4],0.0,"ds/ds has wrong value at (2,1,3,4).")
      self.assertEqual(dsds[2,2,0,0],0.0,"ds/ds has wrong value at (2,2,0,0).")
      self.assertEqual(dsds[2,2,0,1],0.0,"ds/ds has wrong value at (2,2,0,1).")
      self.assertEqual(dsds[2,2,0,2],0.0,"ds/ds has wrong value at (2,2,0,2).")
      self.assertEqual(dsds[2,2,0,3],0.0,"ds/ds has wrong value at (2,2,0,3).")
      self.assertEqual(dsds[2,2,0,4],0.0,"ds/ds has wrong value at (2,2,0,4).")
      self.assertEqual(dsds[2,2,1,0],0.0,"ds/ds has wrong value at (2,2,1,0).")
      self.assertEqual(dsds[2,2,1,1],0.0,"ds/ds has wrong value at (2,2,1,1).")
      self.assertEqual(dsds[2,2,1,2],0.0,"ds/ds has wrong value at (2,2,1,2).")
      self.assertEqual(dsds[2,2,1,3],0.0,"ds/ds has wrong value at (2,2,1,3).")
      self.assertEqual(dsds[2,2,1,4],0.0,"ds/ds has wrong value at (2,2,1,4).")
      self.assertEqual(dsds[2,2,2,0],0.0,"ds/ds has wrong value at (2,2,2,0).")
      self.assertEqual(dsds[2,2,2,1],0.0,"ds/ds has wrong value at (2,2,2,1).")
      self.assertEqual(dsds[2,2,2,2],1.0,"ds/ds has wrong value at (2,2,2,2).")
      self.assertEqual(dsds[2,2,2,3],0.0,"ds/ds has wrong value at (2,2,2,3).")
      self.assertEqual(dsds[2,2,2,4],0.0,"ds/ds has wrong value at (2,2,2,4).")
      self.assertEqual(dsds[2,2,3,0],0.0,"ds/ds has wrong value at (2,2,3,0).")
      self.assertEqual(dsds[2,2,3,1],0.0,"ds/ds has wrong value at (2,2,3,1).")
      self.assertEqual(dsds[2,2,3,2],0.0,"ds/ds has wrong value at (2,2,3,2).")
      self.assertEqual(dsds[2,2,3,3],0.0,"ds/ds has wrong value at (2,2,3,3).")
      self.assertEqual(dsds[2,2,3,4],0.0,"ds/ds has wrong value at (2,2,3,4).")
      self.assertEqual(dsds[2,3,0,0],0.0,"ds/ds has wrong value at (2,3,0,0).")
      self.assertEqual(dsds[2,3,0,1],0.0,"ds/ds has wrong value at (2,3,0,1).")
      self.assertEqual(dsds[2,3,0,2],0.0,"ds/ds has wrong value at (2,3,0,2).")
      self.assertEqual(dsds[2,3,0,3],0.0,"ds/ds has wrong value at (2,3,0,3).")
      self.assertEqual(dsds[2,3,0,4],0.0,"ds/ds has wrong value at (2,3,0,4).")
      self.assertEqual(dsds[2,3,1,0],0.0,"ds/ds has wrong value at (2,3,1,0).")
      self.assertEqual(dsds[2,3,1,1],0.0,"ds/ds has wrong value at (2,3,1,1).")
      self.assertEqual(dsds[2,3,1,2],0.0,"ds/ds has wrong value at (2,3,1,2).")
      self.assertEqual(dsds[2,3,1,3],0.0,"ds/ds has wrong value at (2,3,1,3).")
      self.assertEqual(dsds[2,3,1,4],0.0,"ds/ds has wrong value at (2,3,1,4).")
      self.assertEqual(dsds[2,3,2,0],0.0,"ds/ds has wrong value at (2,3,2,0).")
      self.assertEqual(dsds[2,3,2,1],0.0,"ds/ds has wrong value at (2,3,2,1).")
      self.assertEqual(dsds[2,3,2,2],0.0,"ds/ds has wrong value at (2,3,2,2).")
      self.assertEqual(dsds[2,3,2,3],1.0,"ds/ds has wrong value at (2,3,2,3).")
      self.assertEqual(dsds[2,3,2,4],0.0,"ds/ds has wrong value at (2,3,2,4).")
      self.assertEqual(dsds[2,3,3,0],0.0,"ds/ds has wrong value at (2,3,3,0).")
      self.assertEqual(dsds[2,3,3,1],0.0,"ds/ds has wrong value at (2,3,3,1).")
      self.assertEqual(dsds[2,3,3,2],0.0,"ds/ds has wrong value at (2,3,3,2).")
      self.assertEqual(dsds[2,3,3,3],0.0,"ds/ds has wrong value at (2,3,3,3).")
      self.assertEqual(dsds[2,3,3,4],0.0,"ds/ds has wrong value at (2,3,3,4).")
      self.assertEqual(dsds[2,4,0,0],0.0,"ds/ds has wrong value at (2,4,0,0).")
      self.assertEqual(dsds[2,4,0,1],0.0,"ds/ds has wrong value at (2,4,0,1).")
      self.assertEqual(dsds[2,4,0,2],0.0,"ds/ds has wrong value at (2,4,0,2).")
      self.assertEqual(dsds[2,4,0,3],0.0,"ds/ds has wrong value at (2,4,0,3).")
      self.assertEqual(dsds[2,4,0,4],0.0,"ds/ds has wrong value at (2,4,0,4).")
      self.assertEqual(dsds[2,4,1,0],0.0,"ds/ds has wrong value at (2,4,1,0).")
      self.assertEqual(dsds[2,4,1,1],0.0,"ds/ds has wrong value at (2,4,1,1).")
      self.assertEqual(dsds[2,4,1,2],0.0,"ds/ds has wrong value at (2,4,1,2).")
      self.assertEqual(dsds[2,4,1,3],0.0,"ds/ds has wrong value at (2,4,1,3).")
      self.assertEqual(dsds[2,4,1,4],0.0,"ds/ds has wrong value at (2,4,1,4).")
      self.assertEqual(dsds[2,4,2,0],0.0,"ds/ds has wrong value at (2,4,2,0).")
      self.assertEqual(dsds[2,4,2,1],0.0,"ds/ds has wrong value at (2,4,2,1).")
      self.assertEqual(dsds[2,4,2,2],0.0,"ds/ds has wrong value at (2,4,2,2).")
      self.assertEqual(dsds[2,4,2,3],0.0,"ds/ds has wrong value at (2,4,2,3).")
      self.assertEqual(dsds[2,4,2,4],1.0,"ds/ds has wrong value at (2,4,2,4).")
      self.assertEqual(dsds[2,4,3,0],0.0,"ds/ds has wrong value at (2,4,3,0).")
      self.assertEqual(dsds[2,4,3,1],0.0,"ds/ds has wrong value at (2,4,3,1).")
      self.assertEqual(dsds[2,4,3,2],0.0,"ds/ds has wrong value at (2,4,3,2).")
      self.assertEqual(dsds[2,4,3,3],0.0,"ds/ds has wrong value at (2,4,3,3).")
      self.assertEqual(dsds[2,4,3,4],0.0,"ds/ds has wrong value at (2,4,3,4).")
      self.assertEqual(dsds[3,0,0,0],0.0,"ds/ds has wrong value at (3,0,0,0).")
      self.assertEqual(dsds[3,0,0,1],0.0,"ds/ds has wrong value at (3,0,0,1).")
      self.assertEqual(dsds[3,0,0,2],0.0,"ds/ds has wrong value at (3,0,0,2).")
      self.assertEqual(dsds[3,0,0,3],0.0,"ds/ds has wrong value at (3,0,0,3).")
      self.assertEqual(dsds[3,0,0,4],0.0,"ds/ds has wrong value at (3,0,0,4).")
      self.assertEqual(dsds[3,0,1,0],0.0,"ds/ds has wrong value at (3,0,1,0).")
      self.assertEqual(dsds[3,0,1,1],0.0,"ds/ds has wrong value at (3,0,1,1).")
      self.assertEqual(dsds[3,0,1,2],0.0,"ds/ds has wrong value at (3,0,1,2).")
      self.assertEqual(dsds[3,0,1,3],0.0,"ds/ds has wrong value at (3,0,1,3).")
      self.assertEqual(dsds[3,0,1,4],0.0,"ds/ds has wrong value at (3,0,1,4).")
      self.assertEqual(dsds[3,0,2,0],0.0,"ds/ds has wrong value at (3,0,2,0).")
      self.assertEqual(dsds[3,0,2,1],0.0,"ds/ds has wrong value at (3,0,2,1).")
      self.assertEqual(dsds[3,0,2,2],0.0,"ds/ds has wrong value at (3,0,2,2).")
      self.assertEqual(dsds[3,0,2,3],0.0,"ds/ds has wrong value at (3,0,2,3).")
      self.assertEqual(dsds[3,0,2,4],0.0,"ds/ds has wrong value at (3,0,2,4).")
      self.assertEqual(dsds[3,0,3,0],1.0,"ds/ds has wrong value at (3,0,3,0).")
      self.assertEqual(dsds[3,0,3,1],0.0,"ds/ds has wrong value at (3,0,3,1).")
      self.assertEqual(dsds[3,0,3,2],0.0,"ds/ds has wrong value at (3,0,3,2).")
      self.assertEqual(dsds[3,0,3,3],0.0,"ds/ds has wrong value at (3,0,3,3).")
      self.assertEqual(dsds[3,0,3,4],0.0,"ds/ds has wrong value at (3,0,3,4).")
      self.assertEqual(dsds[3,1,0,0],0.0,"ds/ds has wrong value at (3,1,0,0).")
      self.assertEqual(dsds[3,1,0,1],0.0,"ds/ds has wrong value at (3,1,0,1).")
      self.assertEqual(dsds[3,1,0,2],0.0,"ds/ds has wrong value at (3,1,0,2).")
      self.assertEqual(dsds[3,1,0,3],0.0,"ds/ds has wrong value at (3,1,0,3).")
      self.assertEqual(dsds[3,1,0,4],0.0,"ds/ds has wrong value at (3,1,0,4).")
      self.assertEqual(dsds[3,1,1,0],0.0,"ds/ds has wrong value at (3,1,1,0).")
      self.assertEqual(dsds[3,1,1,1],0.0,"ds/ds has wrong value at (3,1,1,1).")
      self.assertEqual(dsds[3,1,1,2],0.0,"ds/ds has wrong value at (3,1,1,2).")
      self.assertEqual(dsds[3,1,1,3],0.0,"ds/ds has wrong value at (3,1,1,3).")
      self.assertEqual(dsds[3,1,1,4],0.0,"ds/ds has wrong value at (3,1,1,4).")
      self.assertEqual(dsds[3,1,2,0],0.0,"ds/ds has wrong value at (3,1,2,0).")
      self.assertEqual(dsds[3,1,2,1],0.0,"ds/ds has wrong value at (3,1,2,1).")
      self.assertEqual(dsds[3,1,2,2],0.0,"ds/ds has wrong value at (3,1,2,2).")
      self.assertEqual(dsds[3,1,2,3],0.0,"ds/ds has wrong value at (3,1,2,3).")
      self.assertEqual(dsds[3,1,2,4],0.0,"ds/ds has wrong value at (3,1,2,4).")
      self.assertEqual(dsds[3,1,3,0],0.0,"ds/ds has wrong value at (3,1,3,0).")
      self.assertEqual(dsds[3,1,3,1],1.0,"ds/ds has wrong value at (3,1,3,1).")
      self.assertEqual(dsds[3,1,3,2],0.0,"ds/ds has wrong value at (3,1,3,2).")
      self.assertEqual(dsds[3,1,3,3],0.0,"ds/ds has wrong value at (3,1,3,3).")
      self.assertEqual(dsds[3,1,3,4],0.0,"ds/ds has wrong value at (3,1,3,4).")
      self.assertEqual(dsds[3,2,0,0],0.0,"ds/ds has wrong value at (3,2,0,0).")
      self.assertEqual(dsds[3,2,0,1],0.0,"ds/ds has wrong value at (3,2,0,1).")
      self.assertEqual(dsds[3,2,0,2],0.0,"ds/ds has wrong value at (3,2,0,2).")
      self.assertEqual(dsds[3,2,0,3],0.0,"ds/ds has wrong value at (3,2,0,3).")
      self.assertEqual(dsds[3,2,0,4],0.0,"ds/ds has wrong value at (3,2,0,4).")
      self.assertEqual(dsds[3,2,1,0],0.0,"ds/ds has wrong value at (3,2,1,0).")
      self.assertEqual(dsds[3,2,1,1],0.0,"ds/ds has wrong value at (3,2,1,1).")
      self.assertEqual(dsds[3,2,1,2],0.0,"ds/ds has wrong value at (3,2,1,2).")
      self.assertEqual(dsds[3,2,1,3],0.0,"ds/ds has wrong value at (3,2,1,3).")
      self.assertEqual(dsds[3,2,1,4],0.0,"ds/ds has wrong value at (3,2,1,4).")
      self.assertEqual(dsds[3,2,2,0],0.0,"ds/ds has wrong value at (3,2,2,0).")
      self.assertEqual(dsds[3,2,2,1],0.0,"ds/ds has wrong value at (3,2,2,1).")
      self.assertEqual(dsds[3,2,2,2],0.0,"ds/ds has wrong value at (3,2,2,2).")
      self.assertEqual(dsds[3,2,2,3],0.0,"ds/ds has wrong value at (3,2,2,3).")
      self.assertEqual(dsds[3,2,2,4],0.0,"ds/ds has wrong value at (3,2,2,4).")
      self.assertEqual(dsds[3,2,3,0],0.0,"ds/ds has wrong value at (3,2,3,0).")
      self.assertEqual(dsds[3,2,3,1],0.0,"ds/ds has wrong value at (3,2,3,1).")
      self.assertEqual(dsds[3,2,3,2],1.0,"ds/ds has wrong value at (3,2,3,2).")
      self.assertEqual(dsds[3,2,3,3],0.0,"ds/ds has wrong value at (3,2,3,3).")
      self.assertEqual(dsds[3,2,3,4],0.0,"ds/ds has wrong value at (3,2,3,4).")
      self.assertEqual(dsds[3,3,0,0],0.0,"ds/ds has wrong value at (3,3,0,0).")
      self.assertEqual(dsds[3,3,0,1],0.0,"ds/ds has wrong value at (3,3,0,1).")
      self.assertEqual(dsds[3,3,0,2],0.0,"ds/ds has wrong value at (3,3,0,2).")
      self.assertEqual(dsds[3,3,0,3],0.0,"ds/ds has wrong value at (3,3,0,3).")
      self.assertEqual(dsds[3,3,0,4],0.0,"ds/ds has wrong value at (3,3,0,4).")
      self.assertEqual(dsds[3,3,1,0],0.0,"ds/ds has wrong value at (3,3,1,0).")
      self.assertEqual(dsds[3,3,1,1],0.0,"ds/ds has wrong value at (3,3,1,1).")
      self.assertEqual(dsds[3,3,1,2],0.0,"ds/ds has wrong value at (3,3,1,2).")
      self.assertEqual(dsds[3,3,1,3],0.0,"ds/ds has wrong value at (3,3,1,3).")
      self.assertEqual(dsds[3,3,1,4],0.0,"ds/ds has wrong value at (3,3,1,4).")
      self.assertEqual(dsds[3,3,2,0],0.0,"ds/ds has wrong value at (3,3,2,0).")
      self.assertEqual(dsds[3,3,2,1],0.0,"ds/ds has wrong value at (3,3,2,1).")
      self.assertEqual(dsds[3,3,2,2],0.0,"ds/ds has wrong value at (3,3,2,2).")
      self.assertEqual(dsds[3,3,2,3],0.0,"ds/ds has wrong value at (3,3,2,3).")
      self.assertEqual(dsds[3,3,2,4],0.0,"ds/ds has wrong value at (3,3,2,4).")
      self.assertEqual(dsds[3,3,3,0],0.0,"ds/ds has wrong value at (3,3,3,0).")
      self.assertEqual(dsds[3,3,3,1],0.0,"ds/ds has wrong value at (3,3,3,1).")
      self.assertEqual(dsds[3,3,3,2],0.0,"ds/ds has wrong value at (3,3,3,2).")
      self.assertEqual(dsds[3,3,3,3],1.0,"ds/ds has wrong value at (3,3,3,3).")
      self.assertEqual(dsds[3,3,3,4],0.0,"ds/ds has wrong value at (3,3,3,4).")
      self.assertEqual(dsds[3,4,0,0],0.0,"ds/ds has wrong value at (3,4,0,0).")
      self.assertEqual(dsds[3,4,0,1],0.0,"ds/ds has wrong value at (3,4,0,1).")
      self.assertEqual(dsds[3,4,0,2],0.0,"ds/ds has wrong value at (3,4,0,2).")
      self.assertEqual(dsds[3,4,0,3],0.0,"ds/ds has wrong value at (3,4,0,3).")
      self.assertEqual(dsds[3,4,0,4],0.0,"ds/ds has wrong value at (3,4,0,4).")
      self.assertEqual(dsds[3,4,1,0],0.0,"ds/ds has wrong value at (3,4,1,0).")
      self.assertEqual(dsds[3,4,1,1],0.0,"ds/ds has wrong value at (3,4,1,1).")
      self.assertEqual(dsds[3,4,1,2],0.0,"ds/ds has wrong value at (3,4,1,2).")
      self.assertEqual(dsds[3,4,1,3],0.0,"ds/ds has wrong value at (3,4,1,3).")
      self.assertEqual(dsds[3,4,1,4],0.0,"ds/ds has wrong value at (3,4,1,4).")
      self.assertEqual(dsds[3,4,2,0],0.0,"ds/ds has wrong value at (3,4,2,0).")
      self.assertEqual(dsds[3,4,2,1],0.0,"ds/ds has wrong value at (3,4,2,1).")
      self.assertEqual(dsds[3,4,2,2],0.0,"ds/ds has wrong value at (3,4,2,2).")
      self.assertEqual(dsds[3,4,2,3],0.0,"ds/ds has wrong value at (3,4,2,3).")
      self.assertEqual(dsds[3,4,2,4],0.0,"ds/ds has wrong value at (3,4,2,4).")
      self.assertEqual(dsds[3,4,3,0],0.0,"ds/ds has wrong value at (3,4,3,0).")
      self.assertEqual(dsds[3,4,3,1],0.0,"ds/ds has wrong value at (3,4,3,1).")
      self.assertEqual(dsds[3,4,3,2],0.0,"ds/ds has wrong value at (3,4,3,2).")
      self.assertEqual(dsds[3,4,3,3],0.0,"ds/ds has wrong value at (3,4,3,3).")
      self.assertEqual(dsds[3,4,3,4],1.0,"ds/ds has wrong value at (3,4,3,4).")

      self.assertEqual(s.getArgument(0),1,"wrong argument 0.")
      self.assertEqual(s.getArgument(1),-1.0,"wrong argument 1.")
      sa=s.getSubstitutedArguments({s2:-10})
      self.assertEqual(len(sa),2,"wrong number of substituted arguments")
      self.assertEqual(sa[0],1,"wrongly substituted argument 0.")
      self.assertEqual(sa[1],-1.0,"wrongly substituted argument 1.")

      self.assertFalse(s.isAppropriateValue(10.)," 10. is not appropriate substitute")
      self.assertFalse(s.isAppropriateValue(10)," 10 is not appropriate substitute")
      self.assertTrue(s.isAppropriateValue(Symbol(shape=(4, 5),dim=3)),"Symbol is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(Symbol(shape=(4, 5),dim=4)),"Symbol is not appropriate substitute (dim)")
      self.assertFalse(s.isAppropriateValue(escript.Data(0.,(5,),self.functionspace)),"Data is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(numpy.zeros((4, 5))),"ndarray is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(numpy.zeros((5,))),"ndarray is not appropriate substitute (shape)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_Symbol_rank3_dNone_nargs0(self):
      s2=Symbol(shape=())
      s=Symbol(shape=(6, 2, 2),dim=None,args=[])
      self.assertEqual(s.getRank(),3,"wrong rank.")
      self.assertEqual(s.getShape(),(6, 2, 2),"wrong shape.")
      self.assertEqual(s.getDim(),None,"wrong spatial dimension.")
      self.assertEqual(s.getArgument(),[],"wrong arguments.")

      ss=s.substitute({s:numpy.zeros((6, 2, 2))})
      self.assertTrue(isinstance(ss,numpy.ndarray),"value after substitution is not ndarray.")
      self.assertEqual(ss.shape,(6, 2, 2),"value after substitution has not expected shape")
      try:
        s.substitute({s:numpy.zeros((5,))})
        fail("illegal substition was successful")
      except TypeError:
        pass

      dsdarg=s.diff(Symbol(shape=()))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds() has wrong type.")
      self.assertEqual(dsdarg.shape,(6, 2, 2),"ds/ds() has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds() has wrong value.")
      dsdarg=s.diff(Symbol(shape=(2,)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(2,) has wrong type.")
      self.assertEqual(dsdarg.shape,(6, 2, 2, 2),"ds/ds(2,) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(2,) has wrong value.")

      sa=s.getSubstitutedArguments({s2:-10})
      self.assertEqual(len(sa),0,"wrong number of substituted arguments")

      self.assertFalse(s.isAppropriateValue(10.)," 10. is not appropriate substitute")
      self.assertFalse(s.isAppropriateValue(10)," 10 is not appropriate substitute")
      self.assertTrue(s.isAppropriateValue(Symbol(shape=(6, 2, 2),dim=None)),"Symbol is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(Symbol(shape=(5,))),"Symbol is not appropriate substitute (shape)")
      self.assertFalse(s.isAppropriateValue(escript.Data(0.,(5,),self.functionspace)),"Data is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(numpy.zeros((6, 2, 2))),"ndarray is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(numpy.zeros((5,))),"ndarray is not appropriate substitute (shape)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_Symbol_rank3_dNone_nargs1(self):
      s2=Symbol(shape=())
      s=Symbol(shape=(6, 2, 2),dim=None,args=[s2])
      self.assertEqual(s.getRank(),3,"wrong rank.")
      self.assertEqual(s.getShape(),(6, 2, 2),"wrong shape.")
      self.assertEqual(s.getDim(),None,"wrong spatial dimension.")
      self.assertEqual(s.getArgument(),[s2],"wrong arguments.")

      ss=s.substitute({s:numpy.zeros((6, 2, 2))})
      self.assertTrue(isinstance(ss,numpy.ndarray),"value after substitution is not ndarray.")
      self.assertEqual(ss.shape,(6, 2, 2),"value after substitution has not expected shape")
      try:
        s.substitute({s:numpy.zeros((5,))})
        fail("illegal substition was successful")
      except TypeError:
        pass

      dsdarg=s.diff(Symbol(shape=()))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds() has wrong type.")
      self.assertEqual(dsdarg.shape,(6, 2, 2),"ds/ds() has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds() has wrong value.")
      dsdarg=s.diff(Symbol(shape=(2,)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(2,) has wrong type.")
      self.assertEqual(dsdarg.shape,(6, 2, 2, 2),"ds/ds(2,) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(2,) has wrong value.")

      self.assertEqual(s.getArgument(0),s2,"wrong argument 0.")
      sa=s.getSubstitutedArguments({s2:-10})
      self.assertEqual(len(sa),1,"wrong number of substituted arguments")
      self.assertEqual(sa[0],-10,"wrongly substituted argument 0.")

      self.assertFalse(s.isAppropriateValue(10.)," 10. is not appropriate substitute")
      self.assertFalse(s.isAppropriateValue(10)," 10 is not appropriate substitute")
      self.assertTrue(s.isAppropriateValue(Symbol(shape=(6, 2, 2),dim=None)),"Symbol is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(Symbol(shape=(5,))),"Symbol is not appropriate substitute (shape)")
      self.assertFalse(s.isAppropriateValue(escript.Data(0.,(5,),self.functionspace)),"Data is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(numpy.zeros((6, 2, 2))),"ndarray is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(numpy.zeros((5,))),"ndarray is not appropriate substitute (shape)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_Symbol_rank3_dNone_nargs2(self):
      s2=Symbol(shape=())
      s=Symbol(shape=(6, 2, 2),dim=None,args=[1, -1.0])
      self.assertEqual(s.getRank(),3,"wrong rank.")
      self.assertEqual(s.getShape(),(6, 2, 2),"wrong shape.")
      self.assertEqual(s.getDim(),None,"wrong spatial dimension.")
      self.assertEqual(s.getArgument(),[1, -1.0],"wrong arguments.")

      ss=s.substitute({s:numpy.zeros((6, 2, 2))})
      self.assertTrue(isinstance(ss,numpy.ndarray),"value after substitution is not ndarray.")
      self.assertEqual(ss.shape,(6, 2, 2),"value after substitution has not expected shape")
      try:
        s.substitute({s:numpy.zeros((5,))})
        fail("illegal substition was successful")
      except TypeError:
        pass

      dsdarg=s.diff(Symbol(shape=()))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds() has wrong type.")
      self.assertEqual(dsdarg.shape,(6, 2, 2),"ds/ds() has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds() has wrong value.")
      dsdarg=s.diff(Symbol(shape=(2,)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(2,) has wrong type.")
      self.assertEqual(dsdarg.shape,(6, 2, 2, 2),"ds/ds(2,) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(2,) has wrong value.")

      self.assertEqual(s.getArgument(0),1,"wrong argument 0.")
      self.assertEqual(s.getArgument(1),-1.0,"wrong argument 1.")
      sa=s.getSubstitutedArguments({s2:-10})
      self.assertEqual(len(sa),2,"wrong number of substituted arguments")
      self.assertEqual(sa[0],1,"wrongly substituted argument 0.")
      self.assertEqual(sa[1],-1.0,"wrongly substituted argument 1.")

      self.assertFalse(s.isAppropriateValue(10.)," 10. is not appropriate substitute")
      self.assertFalse(s.isAppropriateValue(10)," 10 is not appropriate substitute")
      self.assertTrue(s.isAppropriateValue(Symbol(shape=(6, 2, 2),dim=None)),"Symbol is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(Symbol(shape=(5,))),"Symbol is not appropriate substitute (shape)")
      self.assertFalse(s.isAppropriateValue(escript.Data(0.,(5,),self.functionspace)),"Data is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(numpy.zeros((6, 2, 2))),"ndarray is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(numpy.zeros((5,))),"ndarray is not appropriate substitute (shape)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_Symbol_rank3_dd_nargs0(self):
      s2=Symbol(shape=())
      s=Symbol(shape=(6, 2, 2),dim=self.functionspace.getDim(),args=[])
      self.assertEqual(s.getRank(),3,"wrong rank.")
      self.assertEqual(s.getShape(),(6, 2, 2),"wrong shape.")
      self.assertEqual(s.getDim(),self.functionspace.getDim(),"wrong spatial dimension.")
      self.assertEqual(s.getArgument(),[],"wrong arguments.")

      ss=s.substitute({s:numpy.zeros((6, 2, 2))})
      self.assertTrue(isinstance(ss,numpy.ndarray),"value after substitution is not ndarray.")
      self.assertEqual(ss.shape,(6, 2, 2),"value after substitution has not expected shape")
      try:
        s.substitute({s:numpy.zeros((5,))})
        fail("illegal substition was successful")
      except TypeError:
        pass

      dsdarg=s.diff(Symbol(shape=()))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds() has wrong type.")
      self.assertEqual(dsdarg.shape,(6, 2, 2),"ds/ds() has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds() has wrong value.")
      dsdarg=s.diff(Symbol(shape=(2,)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(2,) has wrong type.")
      self.assertEqual(dsdarg.shape,(6, 2, 2, 2),"ds/ds(2,) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(2,) has wrong value.")

      sa=s.getSubstitutedArguments({s2:-10})
      self.assertEqual(len(sa),0,"wrong number of substituted arguments")

      self.assertFalse(s.isAppropriateValue(10.)," 10. is not appropriate substitute")
      self.assertFalse(s.isAppropriateValue(10)," 10 is not appropriate substitute")
      self.assertTrue(s.isAppropriateValue(Symbol(shape=(6, 2, 2),dim=self.functionspace.getDim())),"Symbol is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(Symbol(shape=(5,))),"Symbol is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(escript.Data(0.,(6, 2, 2),self.functionspace)),"Data is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(escript.Data(0.,(5,),self.functionspace)),"Data is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(numpy.zeros((6, 2, 2))),"ndarray is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(numpy.zeros((5,))),"ndarray is not appropriate substitute (shape)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_Symbol_rank3_dd_nargs1(self):
      s2=Symbol(shape=())
      s=Symbol(shape=(6, 2, 2),dim=self.functionspace.getDim(),args=[s2])
      self.assertEqual(s.getRank(),3,"wrong rank.")
      self.assertEqual(s.getShape(),(6, 2, 2),"wrong shape.")
      self.assertEqual(s.getDim(),self.functionspace.getDim(),"wrong spatial dimension.")
      self.assertEqual(s.getArgument(),[s2],"wrong arguments.")

      ss=s.substitute({s:numpy.zeros((6, 2, 2))})
      self.assertTrue(isinstance(ss,numpy.ndarray),"value after substitution is not ndarray.")
      self.assertEqual(ss.shape,(6, 2, 2),"value after substitution has not expected shape")
      try:
        s.substitute({s:numpy.zeros((5,))})
        fail("illegal substition was successful")
      except TypeError:
        pass

      dsdarg=s.diff(Symbol(shape=()))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds() has wrong type.")
      self.assertEqual(dsdarg.shape,(6, 2, 2),"ds/ds() has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds() has wrong value.")
      dsdarg=s.diff(Symbol(shape=(2,)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(2,) has wrong type.")
      self.assertEqual(dsdarg.shape,(6, 2, 2, 2),"ds/ds(2,) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(2,) has wrong value.")

      self.assertEqual(s.getArgument(0),s2,"wrong argument 0.")
      sa=s.getSubstitutedArguments({s2:-10})
      self.assertEqual(len(sa),1,"wrong number of substituted arguments")
      self.assertEqual(sa[0],-10,"wrongly substituted argument 0.")

      self.assertFalse(s.isAppropriateValue(10.)," 10. is not appropriate substitute")
      self.assertFalse(s.isAppropriateValue(10)," 10 is not appropriate substitute")
      self.assertTrue(s.isAppropriateValue(Symbol(shape=(6, 2, 2),dim=self.functionspace.getDim())),"Symbol is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(Symbol(shape=(5,))),"Symbol is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(escript.Data(0.,(6, 2, 2),self.functionspace)),"Data is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(escript.Data(0.,(5,),self.functionspace)),"Data is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(numpy.zeros((6, 2, 2))),"ndarray is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(numpy.zeros((5,))),"ndarray is not appropriate substitute (shape)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_Symbol_rank3_dd_nargs2(self):
      s2=Symbol(shape=())
      s=Symbol(shape=(6, 2, 2),dim=self.functionspace.getDim(),args=[1, -1.0])
      self.assertEqual(s.getRank(),3,"wrong rank.")
      self.assertEqual(s.getShape(),(6, 2, 2),"wrong shape.")
      self.assertEqual(s.getDim(),self.functionspace.getDim(),"wrong spatial dimension.")
      self.assertEqual(s.getArgument(),[1, -1.0],"wrong arguments.")

      ss=s.substitute({s:numpy.zeros((6, 2, 2))})
      self.assertTrue(isinstance(ss,numpy.ndarray),"value after substitution is not ndarray.")
      self.assertEqual(ss.shape,(6, 2, 2),"value after substitution has not expected shape")
      try:
        s.substitute({s:numpy.zeros((5,))})
        fail("illegal substition was successful")
      except TypeError:
        pass

      dsdarg=s.diff(Symbol(shape=()))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds() has wrong type.")
      self.assertEqual(dsdarg.shape,(6, 2, 2),"ds/ds() has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds() has wrong value.")
      dsdarg=s.diff(Symbol(shape=(2,)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(2,) has wrong type.")
      self.assertEqual(dsdarg.shape,(6, 2, 2, 2),"ds/ds(2,) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(2,) has wrong value.")

      self.assertEqual(s.getArgument(0),1,"wrong argument 0.")
      self.assertEqual(s.getArgument(1),-1.0,"wrong argument 1.")
      sa=s.getSubstitutedArguments({s2:-10})
      self.assertEqual(len(sa),2,"wrong number of substituted arguments")
      self.assertEqual(sa[0],1,"wrongly substituted argument 0.")
      self.assertEqual(sa[1],-1.0,"wrongly substituted argument 1.")

      self.assertFalse(s.isAppropriateValue(10.)," 10. is not appropriate substitute")
      self.assertFalse(s.isAppropriateValue(10)," 10 is not appropriate substitute")
      self.assertTrue(s.isAppropriateValue(Symbol(shape=(6, 2, 2),dim=self.functionspace.getDim())),"Symbol is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(Symbol(shape=(5,))),"Symbol is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(escript.Data(0.,(6, 2, 2),self.functionspace)),"Data is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(escript.Data(0.,(5,),self.functionspace)),"Data is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(numpy.zeros((6, 2, 2))),"ndarray is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(numpy.zeros((5,))),"ndarray is not appropriate substitute (shape)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_Symbol_rank3_d1_nargs0(self):
      s2=Symbol(shape=())
      s=Symbol(shape=(6, 2, 2),dim=1,args=[])
      self.assertEqual(s.getRank(),3,"wrong rank.")
      self.assertEqual(s.getShape(),(6, 2, 2),"wrong shape.")
      self.assertEqual(s.getDim(),1,"wrong spatial dimension.")
      self.assertEqual(s.getArgument(),[],"wrong arguments.")

      ss=s.substitute({s:numpy.zeros((6, 2, 2))})
      self.assertTrue(isinstance(ss,numpy.ndarray),"value after substitution is not ndarray.")
      self.assertEqual(ss.shape,(6, 2, 2),"value after substitution has not expected shape")
      try:
        s.substitute({s:numpy.zeros((5,))})
        fail("illegal substition was successful")
      except TypeError:
        pass

      dsdarg=s.diff(Symbol(shape=()))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds() has wrong type.")
      self.assertEqual(dsdarg.shape,(6, 2, 2),"ds/ds() has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds() has wrong value.")
      dsdarg=s.diff(Symbol(shape=(2,)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(2,) has wrong type.")
      self.assertEqual(dsdarg.shape,(6, 2, 2, 2),"ds/ds(2,) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(2,) has wrong value.")

      sa=s.getSubstitutedArguments({s2:-10})
      self.assertEqual(len(sa),0,"wrong number of substituted arguments")

      self.assertFalse(s.isAppropriateValue(10.)," 10. is not appropriate substitute")
      self.assertFalse(s.isAppropriateValue(10)," 10 is not appropriate substitute")
      self.assertTrue(s.isAppropriateValue(Symbol(shape=(6, 2, 2),dim=1)),"Symbol is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(Symbol(shape=(6, 2, 2),dim=2)),"Symbol is not appropriate substitute (dim)")
      self.assertFalse(s.isAppropriateValue(escript.Data(0.,(5,),self.functionspace)),"Data is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(numpy.zeros((6, 2, 2))),"ndarray is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(numpy.zeros((5,))),"ndarray is not appropriate substitute (shape)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_Symbol_rank3_d1_nargs1(self):
      s2=Symbol(shape=())
      s=Symbol(shape=(6, 2, 2),dim=1,args=[s2])
      self.assertEqual(s.getRank(),3,"wrong rank.")
      self.assertEqual(s.getShape(),(6, 2, 2),"wrong shape.")
      self.assertEqual(s.getDim(),1,"wrong spatial dimension.")
      self.assertEqual(s.getArgument(),[s2],"wrong arguments.")

      ss=s.substitute({s:numpy.zeros((6, 2, 2))})
      self.assertTrue(isinstance(ss,numpy.ndarray),"value after substitution is not ndarray.")
      self.assertEqual(ss.shape,(6, 2, 2),"value after substitution has not expected shape")
      try:
        s.substitute({s:numpy.zeros((5,))})
        fail("illegal substition was successful")
      except TypeError:
        pass

      dsdarg=s.diff(Symbol(shape=()))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds() has wrong type.")
      self.assertEqual(dsdarg.shape,(6, 2, 2),"ds/ds() has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds() has wrong value.")
      dsdarg=s.diff(Symbol(shape=(2,)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(2,) has wrong type.")
      self.assertEqual(dsdarg.shape,(6, 2, 2, 2),"ds/ds(2,) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(2,) has wrong value.")

      self.assertEqual(s.getArgument(0),s2,"wrong argument 0.")
      sa=s.getSubstitutedArguments({s2:-10})
      self.assertEqual(len(sa),1,"wrong number of substituted arguments")
      self.assertEqual(sa[0],-10,"wrongly substituted argument 0.")

      self.assertFalse(s.isAppropriateValue(10.)," 10. is not appropriate substitute")
      self.assertFalse(s.isAppropriateValue(10)," 10 is not appropriate substitute")
      self.assertTrue(s.isAppropriateValue(Symbol(shape=(6, 2, 2),dim=1)),"Symbol is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(Symbol(shape=(6, 2, 2),dim=2)),"Symbol is not appropriate substitute (dim)")
      self.assertFalse(s.isAppropriateValue(escript.Data(0.,(5,),self.functionspace)),"Data is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(numpy.zeros((6, 2, 2))),"ndarray is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(numpy.zeros((5,))),"ndarray is not appropriate substitute (shape)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_Symbol_rank3_d1_nargs2(self):
      s2=Symbol(shape=())
      s=Symbol(shape=(6, 2, 2),dim=1,args=[1, -1.0])
      self.assertEqual(s.getRank(),3,"wrong rank.")
      self.assertEqual(s.getShape(),(6, 2, 2),"wrong shape.")
      self.assertEqual(s.getDim(),1,"wrong spatial dimension.")
      self.assertEqual(s.getArgument(),[1, -1.0],"wrong arguments.")

      ss=s.substitute({s:numpy.zeros((6, 2, 2))})
      self.assertTrue(isinstance(ss,numpy.ndarray),"value after substitution is not ndarray.")
      self.assertEqual(ss.shape,(6, 2, 2),"value after substitution has not expected shape")
      try:
        s.substitute({s:numpy.zeros((5,))})
        fail("illegal substition was successful")
      except TypeError:
        pass

      dsdarg=s.diff(Symbol(shape=()))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds() has wrong type.")
      self.assertEqual(dsdarg.shape,(6, 2, 2),"ds/ds() has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds() has wrong value.")
      dsdarg=s.diff(Symbol(shape=(2,)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(2,) has wrong type.")
      self.assertEqual(dsdarg.shape,(6, 2, 2, 2),"ds/ds(2,) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(2,) has wrong value.")

      self.assertEqual(s.getArgument(0),1,"wrong argument 0.")
      self.assertEqual(s.getArgument(1),-1.0,"wrong argument 1.")
      sa=s.getSubstitutedArguments({s2:-10})
      self.assertEqual(len(sa),2,"wrong number of substituted arguments")
      self.assertEqual(sa[0],1,"wrongly substituted argument 0.")
      self.assertEqual(sa[1],-1.0,"wrongly substituted argument 1.")

      self.assertFalse(s.isAppropriateValue(10.)," 10. is not appropriate substitute")
      self.assertFalse(s.isAppropriateValue(10)," 10 is not appropriate substitute")
      self.assertTrue(s.isAppropriateValue(Symbol(shape=(6, 2, 2),dim=1)),"Symbol is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(Symbol(shape=(6, 2, 2),dim=2)),"Symbol is not appropriate substitute (dim)")
      self.assertFalse(s.isAppropriateValue(escript.Data(0.,(5,),self.functionspace)),"Data is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(numpy.zeros((6, 2, 2))),"ndarray is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(numpy.zeros((5,))),"ndarray is not appropriate substitute (shape)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_Symbol_rank3_d2_nargs0(self):
      s2=Symbol(shape=())
      s=Symbol(shape=(6, 2, 2),dim=2,args=[])
      self.assertEqual(s.getRank(),3,"wrong rank.")
      self.assertEqual(s.getShape(),(6, 2, 2),"wrong shape.")
      self.assertEqual(s.getDim(),2,"wrong spatial dimension.")
      self.assertEqual(s.getArgument(),[],"wrong arguments.")

      ss=s.substitute({s:numpy.zeros((6, 2, 2))})
      self.assertTrue(isinstance(ss,numpy.ndarray),"value after substitution is not ndarray.")
      self.assertEqual(ss.shape,(6, 2, 2),"value after substitution has not expected shape")
      try:
        s.substitute({s:numpy.zeros((5,))})
        fail("illegal substition was successful")
      except TypeError:
        pass

      dsdarg=s.diff(Symbol(shape=()))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds() has wrong type.")
      self.assertEqual(dsdarg.shape,(6, 2, 2),"ds/ds() has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds() has wrong value.")
      dsdarg=s.diff(Symbol(shape=(2,)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(2,) has wrong type.")
      self.assertEqual(dsdarg.shape,(6, 2, 2, 2),"ds/ds(2,) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(2,) has wrong value.")

      sa=s.getSubstitutedArguments({s2:-10})
      self.assertEqual(len(sa),0,"wrong number of substituted arguments")

      self.assertFalse(s.isAppropriateValue(10.)," 10. is not appropriate substitute")
      self.assertFalse(s.isAppropriateValue(10)," 10 is not appropriate substitute")
      self.assertTrue(s.isAppropriateValue(Symbol(shape=(6, 2, 2),dim=2)),"Symbol is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(Symbol(shape=(6, 2, 2),dim=3)),"Symbol is not appropriate substitute (dim)")
      self.assertFalse(s.isAppropriateValue(escript.Data(0.,(5,),self.functionspace)),"Data is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(numpy.zeros((6, 2, 2))),"ndarray is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(numpy.zeros((5,))),"ndarray is not appropriate substitute (shape)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_Symbol_rank3_d2_nargs1(self):
      s2=Symbol(shape=())
      s=Symbol(shape=(6, 2, 2),dim=2,args=[s2])
      self.assertEqual(s.getRank(),3,"wrong rank.")
      self.assertEqual(s.getShape(),(6, 2, 2),"wrong shape.")
      self.assertEqual(s.getDim(),2,"wrong spatial dimension.")
      self.assertEqual(s.getArgument(),[s2],"wrong arguments.")

      ss=s.substitute({s:numpy.zeros((6, 2, 2))})
      self.assertTrue(isinstance(ss,numpy.ndarray),"value after substitution is not ndarray.")
      self.assertEqual(ss.shape,(6, 2, 2),"value after substitution has not expected shape")
      try:
        s.substitute({s:numpy.zeros((5,))})
        fail("illegal substition was successful")
      except TypeError:
        pass

      dsdarg=s.diff(Symbol(shape=()))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds() has wrong type.")
      self.assertEqual(dsdarg.shape,(6, 2, 2),"ds/ds() has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds() has wrong value.")
      dsdarg=s.diff(Symbol(shape=(2,)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(2,) has wrong type.")
      self.assertEqual(dsdarg.shape,(6, 2, 2, 2),"ds/ds(2,) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(2,) has wrong value.")

      self.assertEqual(s.getArgument(0),s2,"wrong argument 0.")
      sa=s.getSubstitutedArguments({s2:-10})
      self.assertEqual(len(sa),1,"wrong number of substituted arguments")
      self.assertEqual(sa[0],-10,"wrongly substituted argument 0.")

      self.assertFalse(s.isAppropriateValue(10.)," 10. is not appropriate substitute")
      self.assertFalse(s.isAppropriateValue(10)," 10 is not appropriate substitute")
      self.assertTrue(s.isAppropriateValue(Symbol(shape=(6, 2, 2),dim=2)),"Symbol is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(Symbol(shape=(6, 2, 2),dim=3)),"Symbol is not appropriate substitute (dim)")
      self.assertFalse(s.isAppropriateValue(escript.Data(0.,(5,),self.functionspace)),"Data is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(numpy.zeros((6, 2, 2))),"ndarray is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(numpy.zeros((5,))),"ndarray is not appropriate substitute (shape)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_Symbol_rank3_d2_nargs2(self):
      s2=Symbol(shape=())
      s=Symbol(shape=(6, 2, 2),dim=2,args=[1, -1.0])
      self.assertEqual(s.getRank(),3,"wrong rank.")
      self.assertEqual(s.getShape(),(6, 2, 2),"wrong shape.")
      self.assertEqual(s.getDim(),2,"wrong spatial dimension.")
      self.assertEqual(s.getArgument(),[1, -1.0],"wrong arguments.")

      ss=s.substitute({s:numpy.zeros((6, 2, 2))})
      self.assertTrue(isinstance(ss,numpy.ndarray),"value after substitution is not ndarray.")
      self.assertEqual(ss.shape,(6, 2, 2),"value after substitution has not expected shape")
      try:
        s.substitute({s:numpy.zeros((5,))})
        fail("illegal substition was successful")
      except TypeError:
        pass

      dsdarg=s.diff(Symbol(shape=()))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds() has wrong type.")
      self.assertEqual(dsdarg.shape,(6, 2, 2),"ds/ds() has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds() has wrong value.")
      dsdarg=s.diff(Symbol(shape=(2,)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(2,) has wrong type.")
      self.assertEqual(dsdarg.shape,(6, 2, 2, 2),"ds/ds(2,) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(2,) has wrong value.")

      self.assertEqual(s.getArgument(0),1,"wrong argument 0.")
      self.assertEqual(s.getArgument(1),-1.0,"wrong argument 1.")
      sa=s.getSubstitutedArguments({s2:-10})
      self.assertEqual(len(sa),2,"wrong number of substituted arguments")
      self.assertEqual(sa[0],1,"wrongly substituted argument 0.")
      self.assertEqual(sa[1],-1.0,"wrongly substituted argument 1.")

      self.assertFalse(s.isAppropriateValue(10.)," 10. is not appropriate substitute")
      self.assertFalse(s.isAppropriateValue(10)," 10 is not appropriate substitute")
      self.assertTrue(s.isAppropriateValue(Symbol(shape=(6, 2, 2),dim=2)),"Symbol is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(Symbol(shape=(6, 2, 2),dim=3)),"Symbol is not appropriate substitute (dim)")
      self.assertFalse(s.isAppropriateValue(escript.Data(0.,(5,),self.functionspace)),"Data is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(numpy.zeros((6, 2, 2))),"ndarray is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(numpy.zeros((5,))),"ndarray is not appropriate substitute (shape)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_Symbol_rank3_d3_nargs0(self):
      s2=Symbol(shape=())
      s=Symbol(shape=(6, 2, 2),dim=3,args=[])
      self.assertEqual(s.getRank(),3,"wrong rank.")
      self.assertEqual(s.getShape(),(6, 2, 2),"wrong shape.")
      self.assertEqual(s.getDim(),3,"wrong spatial dimension.")
      self.assertEqual(s.getArgument(),[],"wrong arguments.")

      ss=s.substitute({s:numpy.zeros((6, 2, 2))})
      self.assertTrue(isinstance(ss,numpy.ndarray),"value after substitution is not ndarray.")
      self.assertEqual(ss.shape,(6, 2, 2),"value after substitution has not expected shape")
      try:
        s.substitute({s:numpy.zeros((5,))})
        fail("illegal substition was successful")
      except TypeError:
        pass

      dsdarg=s.diff(Symbol(shape=()))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds() has wrong type.")
      self.assertEqual(dsdarg.shape,(6, 2, 2),"ds/ds() has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds() has wrong value.")
      dsdarg=s.diff(Symbol(shape=(2,)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(2,) has wrong type.")
      self.assertEqual(dsdarg.shape,(6, 2, 2, 2),"ds/ds(2,) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(2,) has wrong value.")

      sa=s.getSubstitutedArguments({s2:-10})
      self.assertEqual(len(sa),0,"wrong number of substituted arguments")

      self.assertFalse(s.isAppropriateValue(10.)," 10. is not appropriate substitute")
      self.assertFalse(s.isAppropriateValue(10)," 10 is not appropriate substitute")
      self.assertTrue(s.isAppropriateValue(Symbol(shape=(6, 2, 2),dim=3)),"Symbol is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(Symbol(shape=(6, 2, 2),dim=4)),"Symbol is not appropriate substitute (dim)")
      self.assertFalse(s.isAppropriateValue(escript.Data(0.,(5,),self.functionspace)),"Data is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(numpy.zeros((6, 2, 2))),"ndarray is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(numpy.zeros((5,))),"ndarray is not appropriate substitute (shape)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_Symbol_rank3_d3_nargs1(self):
      s2=Symbol(shape=())
      s=Symbol(shape=(6, 2, 2),dim=3,args=[s2])
      self.assertEqual(s.getRank(),3,"wrong rank.")
      self.assertEqual(s.getShape(),(6, 2, 2),"wrong shape.")
      self.assertEqual(s.getDim(),3,"wrong spatial dimension.")
      self.assertEqual(s.getArgument(),[s2],"wrong arguments.")

      ss=s.substitute({s:numpy.zeros((6, 2, 2))})
      self.assertTrue(isinstance(ss,numpy.ndarray),"value after substitution is not ndarray.")
      self.assertEqual(ss.shape,(6, 2, 2),"value after substitution has not expected shape")
      try:
        s.substitute({s:numpy.zeros((5,))})
        fail("illegal substition was successful")
      except TypeError:
        pass

      dsdarg=s.diff(Symbol(shape=()))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds() has wrong type.")
      self.assertEqual(dsdarg.shape,(6, 2, 2),"ds/ds() has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds() has wrong value.")
      dsdarg=s.diff(Symbol(shape=(2,)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(2,) has wrong type.")
      self.assertEqual(dsdarg.shape,(6, 2, 2, 2),"ds/ds(2,) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(2,) has wrong value.")

      self.assertEqual(s.getArgument(0),s2,"wrong argument 0.")
      sa=s.getSubstitutedArguments({s2:-10})
      self.assertEqual(len(sa),1,"wrong number of substituted arguments")
      self.assertEqual(sa[0],-10,"wrongly substituted argument 0.")

      self.assertFalse(s.isAppropriateValue(10.)," 10. is not appropriate substitute")
      self.assertFalse(s.isAppropriateValue(10)," 10 is not appropriate substitute")
      self.assertTrue(s.isAppropriateValue(Symbol(shape=(6, 2, 2),dim=3)),"Symbol is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(Symbol(shape=(6, 2, 2),dim=4)),"Symbol is not appropriate substitute (dim)")
      self.assertFalse(s.isAppropriateValue(escript.Data(0.,(5,),self.functionspace)),"Data is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(numpy.zeros((6, 2, 2))),"ndarray is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(numpy.zeros((5,))),"ndarray is not appropriate substitute (shape)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_Symbol_rank3_d3_nargs2(self):
      s2=Symbol(shape=())
      s=Symbol(shape=(6, 2, 2),dim=3,args=[1, -1.0])
      self.assertEqual(s.getRank(),3,"wrong rank.")
      self.assertEqual(s.getShape(),(6, 2, 2),"wrong shape.")
      self.assertEqual(s.getDim(),3,"wrong spatial dimension.")
      self.assertEqual(s.getArgument(),[1, -1.0],"wrong arguments.")

      ss=s.substitute({s:numpy.zeros((6, 2, 2))})
      self.assertTrue(isinstance(ss,numpy.ndarray),"value after substitution is not ndarray.")
      self.assertEqual(ss.shape,(6, 2, 2),"value after substitution has not expected shape")
      try:
        s.substitute({s:numpy.zeros((5,))})
        fail("illegal substition was successful")
      except TypeError:
        pass

      dsdarg=s.diff(Symbol(shape=()))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds() has wrong type.")
      self.assertEqual(dsdarg.shape,(6, 2, 2),"ds/ds() has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds() has wrong value.")
      dsdarg=s.diff(Symbol(shape=(2,)))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds(2,) has wrong type.")
      self.assertEqual(dsdarg.shape,(6, 2, 2, 2),"ds/ds(2,) has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds(2,) has wrong value.")

      self.assertEqual(s.getArgument(0),1,"wrong argument 0.")
      self.assertEqual(s.getArgument(1),-1.0,"wrong argument 1.")
      sa=s.getSubstitutedArguments({s2:-10})
      self.assertEqual(len(sa),2,"wrong number of substituted arguments")
      self.assertEqual(sa[0],1,"wrongly substituted argument 0.")
      self.assertEqual(sa[1],-1.0,"wrongly substituted argument 1.")

      self.assertFalse(s.isAppropriateValue(10.)," 10. is not appropriate substitute")
      self.assertFalse(s.isAppropriateValue(10)," 10 is not appropriate substitute")
      self.assertTrue(s.isAppropriateValue(Symbol(shape=(6, 2, 2),dim=3)),"Symbol is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(Symbol(shape=(6, 2, 2),dim=4)),"Symbol is not appropriate substitute (dim)")
      self.assertFalse(s.isAppropriateValue(escript.Data(0.,(5,),self.functionspace)),"Data is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(numpy.zeros((6, 2, 2))),"ndarray is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(numpy.zeros((5,))),"ndarray is not appropriate substitute (shape)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_Symbol_rank4_dNone_nargs0(self):
      s2=Symbol(shape=())
      s=Symbol(shape=(3, 2, 3, 4),dim=None,args=[])
      self.assertEqual(s.getRank(),4,"wrong rank.")
      self.assertEqual(s.getShape(),(3, 2, 3, 4),"wrong shape.")
      self.assertEqual(s.getDim(),None,"wrong spatial dimension.")
      self.assertEqual(s.getArgument(),[],"wrong arguments.")

      ss=s.substitute({s:numpy.zeros((3, 2, 3, 4))})
      self.assertTrue(isinstance(ss,numpy.ndarray),"value after substitution is not ndarray.")
      self.assertEqual(ss.shape,(3, 2, 3, 4),"value after substitution has not expected shape")
      try:
        s.substitute({s:numpy.zeros((5,))})
        fail("illegal substition was successful")
      except TypeError:
        pass

      dsdarg=s.diff(Symbol(shape=()))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds() has wrong type.")
      self.assertEqual(dsdarg.shape,(3, 2, 3, 4),"ds/ds() has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds() has wrong value.")

      sa=s.getSubstitutedArguments({s2:-10})
      self.assertEqual(len(sa),0,"wrong number of substituted arguments")

      self.assertFalse(s.isAppropriateValue(10.)," 10. is not appropriate substitute")
      self.assertFalse(s.isAppropriateValue(10)," 10 is not appropriate substitute")
      self.assertTrue(s.isAppropriateValue(Symbol(shape=(3, 2, 3, 4),dim=None)),"Symbol is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(Symbol(shape=(5,))),"Symbol is not appropriate substitute (shape)")
      self.assertFalse(s.isAppropriateValue(escript.Data(0.,(5,),self.functionspace)),"Data is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(numpy.zeros((3, 2, 3, 4))),"ndarray is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(numpy.zeros((5,))),"ndarray is not appropriate substitute (shape)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_Symbol_rank4_dNone_nargs1(self):
      s2=Symbol(shape=())
      s=Symbol(shape=(3, 2, 3, 4),dim=None,args=[s2])
      self.assertEqual(s.getRank(),4,"wrong rank.")
      self.assertEqual(s.getShape(),(3, 2, 3, 4),"wrong shape.")
      self.assertEqual(s.getDim(),None,"wrong spatial dimension.")
      self.assertEqual(s.getArgument(),[s2],"wrong arguments.")

      ss=s.substitute({s:numpy.zeros((3, 2, 3, 4))})
      self.assertTrue(isinstance(ss,numpy.ndarray),"value after substitution is not ndarray.")
      self.assertEqual(ss.shape,(3, 2, 3, 4),"value after substitution has not expected shape")
      try:
        s.substitute({s:numpy.zeros((5,))})
        fail("illegal substition was successful")
      except TypeError:
        pass

      dsdarg=s.diff(Symbol(shape=()))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds() has wrong type.")
      self.assertEqual(dsdarg.shape,(3, 2, 3, 4),"ds/ds() has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds() has wrong value.")

      self.assertEqual(s.getArgument(0),s2,"wrong argument 0.")
      sa=s.getSubstitutedArguments({s2:-10})
      self.assertEqual(len(sa),1,"wrong number of substituted arguments")
      self.assertEqual(sa[0],-10,"wrongly substituted argument 0.")

      self.assertFalse(s.isAppropriateValue(10.)," 10. is not appropriate substitute")
      self.assertFalse(s.isAppropriateValue(10)," 10 is not appropriate substitute")
      self.assertTrue(s.isAppropriateValue(Symbol(shape=(3, 2, 3, 4),dim=None)),"Symbol is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(Symbol(shape=(5,))),"Symbol is not appropriate substitute (shape)")
      self.assertFalse(s.isAppropriateValue(escript.Data(0.,(5,),self.functionspace)),"Data is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(numpy.zeros((3, 2, 3, 4))),"ndarray is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(numpy.zeros((5,))),"ndarray is not appropriate substitute (shape)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_Symbol_rank4_dNone_nargs2(self):
      s2=Symbol(shape=())
      s=Symbol(shape=(3, 2, 3, 4),dim=None,args=[1, -1.0])
      self.assertEqual(s.getRank(),4,"wrong rank.")
      self.assertEqual(s.getShape(),(3, 2, 3, 4),"wrong shape.")
      self.assertEqual(s.getDim(),None,"wrong spatial dimension.")
      self.assertEqual(s.getArgument(),[1, -1.0],"wrong arguments.")

      ss=s.substitute({s:numpy.zeros((3, 2, 3, 4))})
      self.assertTrue(isinstance(ss,numpy.ndarray),"value after substitution is not ndarray.")
      self.assertEqual(ss.shape,(3, 2, 3, 4),"value after substitution has not expected shape")
      try:
        s.substitute({s:numpy.zeros((5,))})
        fail("illegal substition was successful")
      except TypeError:
        pass

      dsdarg=s.diff(Symbol(shape=()))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds() has wrong type.")
      self.assertEqual(dsdarg.shape,(3, 2, 3, 4),"ds/ds() has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds() has wrong value.")

      self.assertEqual(s.getArgument(0),1,"wrong argument 0.")
      self.assertEqual(s.getArgument(1),-1.0,"wrong argument 1.")
      sa=s.getSubstitutedArguments({s2:-10})
      self.assertEqual(len(sa),2,"wrong number of substituted arguments")
      self.assertEqual(sa[0],1,"wrongly substituted argument 0.")
      self.assertEqual(sa[1],-1.0,"wrongly substituted argument 1.")

      self.assertFalse(s.isAppropriateValue(10.)," 10. is not appropriate substitute")
      self.assertFalse(s.isAppropriateValue(10)," 10 is not appropriate substitute")
      self.assertTrue(s.isAppropriateValue(Symbol(shape=(3, 2, 3, 4),dim=None)),"Symbol is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(Symbol(shape=(5,))),"Symbol is not appropriate substitute (shape)")
      self.assertFalse(s.isAppropriateValue(escript.Data(0.,(5,),self.functionspace)),"Data is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(numpy.zeros((3, 2, 3, 4))),"ndarray is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(numpy.zeros((5,))),"ndarray is not appropriate substitute (shape)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_Symbol_rank4_dd_nargs0(self):
      s2=Symbol(shape=())
      s=Symbol(shape=(3, 2, 3, 4),dim=self.functionspace.getDim(),args=[])
      self.assertEqual(s.getRank(),4,"wrong rank.")
      self.assertEqual(s.getShape(),(3, 2, 3, 4),"wrong shape.")
      self.assertEqual(s.getDim(),self.functionspace.getDim(),"wrong spatial dimension.")
      self.assertEqual(s.getArgument(),[],"wrong arguments.")

      ss=s.substitute({s:numpy.zeros((3, 2, 3, 4))})
      self.assertTrue(isinstance(ss,numpy.ndarray),"value after substitution is not ndarray.")
      self.assertEqual(ss.shape,(3, 2, 3, 4),"value after substitution has not expected shape")
      try:
        s.substitute({s:numpy.zeros((5,))})
        fail("illegal substition was successful")
      except TypeError:
        pass

      dsdarg=s.diff(Symbol(shape=()))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds() has wrong type.")
      self.assertEqual(dsdarg.shape,(3, 2, 3, 4),"ds/ds() has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds() has wrong value.")

      sa=s.getSubstitutedArguments({s2:-10})
      self.assertEqual(len(sa),0,"wrong number of substituted arguments")

      self.assertFalse(s.isAppropriateValue(10.)," 10. is not appropriate substitute")
      self.assertFalse(s.isAppropriateValue(10)," 10 is not appropriate substitute")
      self.assertTrue(s.isAppropriateValue(Symbol(shape=(3, 2, 3, 4),dim=self.functionspace.getDim())),"Symbol is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(Symbol(shape=(5,))),"Symbol is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(escript.Data(0.,(3, 2, 3, 4),self.functionspace)),"Data is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(escript.Data(0.,(5,),self.functionspace)),"Data is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(numpy.zeros((3, 2, 3, 4))),"ndarray is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(numpy.zeros((5,))),"ndarray is not appropriate substitute (shape)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_Symbol_rank4_dd_nargs1(self):
      s2=Symbol(shape=())
      s=Symbol(shape=(3, 2, 3, 4),dim=self.functionspace.getDim(),args=[s2])
      self.assertEqual(s.getRank(),4,"wrong rank.")
      self.assertEqual(s.getShape(),(3, 2, 3, 4),"wrong shape.")
      self.assertEqual(s.getDim(),self.functionspace.getDim(),"wrong spatial dimension.")
      self.assertEqual(s.getArgument(),[s2],"wrong arguments.")

      ss=s.substitute({s:numpy.zeros((3, 2, 3, 4))})
      self.assertTrue(isinstance(ss,numpy.ndarray),"value after substitution is not ndarray.")
      self.assertEqual(ss.shape,(3, 2, 3, 4),"value after substitution has not expected shape")
      try:
        s.substitute({s:numpy.zeros((5,))})
        fail("illegal substition was successful")
      except TypeError:
        pass

      dsdarg=s.diff(Symbol(shape=()))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds() has wrong type.")
      self.assertEqual(dsdarg.shape,(3, 2, 3, 4),"ds/ds() has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds() has wrong value.")

      self.assertEqual(s.getArgument(0),s2,"wrong argument 0.")
      sa=s.getSubstitutedArguments({s2:-10})
      self.assertEqual(len(sa),1,"wrong number of substituted arguments")
      self.assertEqual(sa[0],-10,"wrongly substituted argument 0.")

      self.assertFalse(s.isAppropriateValue(10.)," 10. is not appropriate substitute")
      self.assertFalse(s.isAppropriateValue(10)," 10 is not appropriate substitute")
      self.assertTrue(s.isAppropriateValue(Symbol(shape=(3, 2, 3, 4),dim=self.functionspace.getDim())),"Symbol is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(Symbol(shape=(5,))),"Symbol is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(escript.Data(0.,(3, 2, 3, 4),self.functionspace)),"Data is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(escript.Data(0.,(5,),self.functionspace)),"Data is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(numpy.zeros((3, 2, 3, 4))),"ndarray is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(numpy.zeros((5,))),"ndarray is not appropriate substitute (shape)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_Symbol_rank4_dd_nargs2(self):
      s2=Symbol(shape=())
      s=Symbol(shape=(3, 2, 3, 4),dim=self.functionspace.getDim(),args=[1, -1.0])
      self.assertEqual(s.getRank(),4,"wrong rank.")
      self.assertEqual(s.getShape(),(3, 2, 3, 4),"wrong shape.")
      self.assertEqual(s.getDim(),self.functionspace.getDim(),"wrong spatial dimension.")
      self.assertEqual(s.getArgument(),[1, -1.0],"wrong arguments.")

      ss=s.substitute({s:numpy.zeros((3, 2, 3, 4))})
      self.assertTrue(isinstance(ss,numpy.ndarray),"value after substitution is not ndarray.")
      self.assertEqual(ss.shape,(3, 2, 3, 4),"value after substitution has not expected shape")
      try:
        s.substitute({s:numpy.zeros((5,))})
        fail("illegal substition was successful")
      except TypeError:
        pass

      dsdarg=s.diff(Symbol(shape=()))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds() has wrong type.")
      self.assertEqual(dsdarg.shape,(3, 2, 3, 4),"ds/ds() has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds() has wrong value.")

      self.assertEqual(s.getArgument(0),1,"wrong argument 0.")
      self.assertEqual(s.getArgument(1),-1.0,"wrong argument 1.")
      sa=s.getSubstitutedArguments({s2:-10})
      self.assertEqual(len(sa),2,"wrong number of substituted arguments")
      self.assertEqual(sa[0],1,"wrongly substituted argument 0.")
      self.assertEqual(sa[1],-1.0,"wrongly substituted argument 1.")

      self.assertFalse(s.isAppropriateValue(10.)," 10. is not appropriate substitute")
      self.assertFalse(s.isAppropriateValue(10)," 10 is not appropriate substitute")
      self.assertTrue(s.isAppropriateValue(Symbol(shape=(3, 2, 3, 4),dim=self.functionspace.getDim())),"Symbol is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(Symbol(shape=(5,))),"Symbol is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(escript.Data(0.,(3, 2, 3, 4),self.functionspace)),"Data is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(escript.Data(0.,(5,),self.functionspace)),"Data is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(numpy.zeros((3, 2, 3, 4))),"ndarray is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(numpy.zeros((5,))),"ndarray is not appropriate substitute (shape)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_Symbol_rank4_d1_nargs0(self):
      s2=Symbol(shape=())
      s=Symbol(shape=(3, 2, 3, 4),dim=1,args=[])
      self.assertEqual(s.getRank(),4,"wrong rank.")
      self.assertEqual(s.getShape(),(3, 2, 3, 4),"wrong shape.")
      self.assertEqual(s.getDim(),1,"wrong spatial dimension.")
      self.assertEqual(s.getArgument(),[],"wrong arguments.")

      ss=s.substitute({s:numpy.zeros((3, 2, 3, 4))})
      self.assertTrue(isinstance(ss,numpy.ndarray),"value after substitution is not ndarray.")
      self.assertEqual(ss.shape,(3, 2, 3, 4),"value after substitution has not expected shape")
      try:
        s.substitute({s:numpy.zeros((5,))})
        fail("illegal substition was successful")
      except TypeError:
        pass

      dsdarg=s.diff(Symbol(shape=()))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds() has wrong type.")
      self.assertEqual(dsdarg.shape,(3, 2, 3, 4),"ds/ds() has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds() has wrong value.")

      sa=s.getSubstitutedArguments({s2:-10})
      self.assertEqual(len(sa),0,"wrong number of substituted arguments")

      self.assertFalse(s.isAppropriateValue(10.)," 10. is not appropriate substitute")
      self.assertFalse(s.isAppropriateValue(10)," 10 is not appropriate substitute")
      self.assertTrue(s.isAppropriateValue(Symbol(shape=(3, 2, 3, 4),dim=1)),"Symbol is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(Symbol(shape=(3, 2, 3, 4),dim=2)),"Symbol is not appropriate substitute (dim)")
      self.assertFalse(s.isAppropriateValue(escript.Data(0.,(5,),self.functionspace)),"Data is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(numpy.zeros((3, 2, 3, 4))),"ndarray is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(numpy.zeros((5,))),"ndarray is not appropriate substitute (shape)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_Symbol_rank4_d1_nargs1(self):
      s2=Symbol(shape=())
      s=Symbol(shape=(3, 2, 3, 4),dim=1,args=[s2])
      self.assertEqual(s.getRank(),4,"wrong rank.")
      self.assertEqual(s.getShape(),(3, 2, 3, 4),"wrong shape.")
      self.assertEqual(s.getDim(),1,"wrong spatial dimension.")
      self.assertEqual(s.getArgument(),[s2],"wrong arguments.")

      ss=s.substitute({s:numpy.zeros((3, 2, 3, 4))})
      self.assertTrue(isinstance(ss,numpy.ndarray),"value after substitution is not ndarray.")
      self.assertEqual(ss.shape,(3, 2, 3, 4),"value after substitution has not expected shape")
      try:
        s.substitute({s:numpy.zeros((5,))})
        fail("illegal substition was successful")
      except TypeError:
        pass

      dsdarg=s.diff(Symbol(shape=()))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds() has wrong type.")
      self.assertEqual(dsdarg.shape,(3, 2, 3, 4),"ds/ds() has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds() has wrong value.")

      self.assertEqual(s.getArgument(0),s2,"wrong argument 0.")
      sa=s.getSubstitutedArguments({s2:-10})
      self.assertEqual(len(sa),1,"wrong number of substituted arguments")
      self.assertEqual(sa[0],-10,"wrongly substituted argument 0.")

      self.assertFalse(s.isAppropriateValue(10.)," 10. is not appropriate substitute")
      self.assertFalse(s.isAppropriateValue(10)," 10 is not appropriate substitute")
      self.assertTrue(s.isAppropriateValue(Symbol(shape=(3, 2, 3, 4),dim=1)),"Symbol is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(Symbol(shape=(3, 2, 3, 4),dim=2)),"Symbol is not appropriate substitute (dim)")
      self.assertFalse(s.isAppropriateValue(escript.Data(0.,(5,),self.functionspace)),"Data is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(numpy.zeros((3, 2, 3, 4))),"ndarray is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(numpy.zeros((5,))),"ndarray is not appropriate substitute (shape)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_Symbol_rank4_d1_nargs2(self):
      s2=Symbol(shape=())
      s=Symbol(shape=(3, 2, 3, 4),dim=1,args=[1, -1.0])
      self.assertEqual(s.getRank(),4,"wrong rank.")
      self.assertEqual(s.getShape(),(3, 2, 3, 4),"wrong shape.")
      self.assertEqual(s.getDim(),1,"wrong spatial dimension.")
      self.assertEqual(s.getArgument(),[1, -1.0],"wrong arguments.")

      ss=s.substitute({s:numpy.zeros((3, 2, 3, 4))})
      self.assertTrue(isinstance(ss,numpy.ndarray),"value after substitution is not ndarray.")
      self.assertEqual(ss.shape,(3, 2, 3, 4),"value after substitution has not expected shape")
      try:
        s.substitute({s:numpy.zeros((5,))})
        fail("illegal substition was successful")
      except TypeError:
        pass

      dsdarg=s.diff(Symbol(shape=()))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds() has wrong type.")
      self.assertEqual(dsdarg.shape,(3, 2, 3, 4),"ds/ds() has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds() has wrong value.")

      self.assertEqual(s.getArgument(0),1,"wrong argument 0.")
      self.assertEqual(s.getArgument(1),-1.0,"wrong argument 1.")
      sa=s.getSubstitutedArguments({s2:-10})
      self.assertEqual(len(sa),2,"wrong number of substituted arguments")
      self.assertEqual(sa[0],1,"wrongly substituted argument 0.")
      self.assertEqual(sa[1],-1.0,"wrongly substituted argument 1.")

      self.assertFalse(s.isAppropriateValue(10.)," 10. is not appropriate substitute")
      self.assertFalse(s.isAppropriateValue(10)," 10 is not appropriate substitute")
      self.assertTrue(s.isAppropriateValue(Symbol(shape=(3, 2, 3, 4),dim=1)),"Symbol is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(Symbol(shape=(3, 2, 3, 4),dim=2)),"Symbol is not appropriate substitute (dim)")
      self.assertFalse(s.isAppropriateValue(escript.Data(0.,(5,),self.functionspace)),"Data is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(numpy.zeros((3, 2, 3, 4))),"ndarray is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(numpy.zeros((5,))),"ndarray is not appropriate substitute (shape)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_Symbol_rank4_d2_nargs0(self):
      s2=Symbol(shape=())
      s=Symbol(shape=(3, 2, 3, 4),dim=2,args=[])
      self.assertEqual(s.getRank(),4,"wrong rank.")
      self.assertEqual(s.getShape(),(3, 2, 3, 4),"wrong shape.")
      self.assertEqual(s.getDim(),2,"wrong spatial dimension.")
      self.assertEqual(s.getArgument(),[],"wrong arguments.")

      ss=s.substitute({s:numpy.zeros((3, 2, 3, 4))})
      self.assertTrue(isinstance(ss,numpy.ndarray),"value after substitution is not ndarray.")
      self.assertEqual(ss.shape,(3, 2, 3, 4),"value after substitution has not expected shape")
      try:
        s.substitute({s:numpy.zeros((5,))})
        fail("illegal substition was successful")
      except TypeError:
        pass

      dsdarg=s.diff(Symbol(shape=()))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds() has wrong type.")
      self.assertEqual(dsdarg.shape,(3, 2, 3, 4),"ds/ds() has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds() has wrong value.")

      sa=s.getSubstitutedArguments({s2:-10})
      self.assertEqual(len(sa),0,"wrong number of substituted arguments")

      self.assertFalse(s.isAppropriateValue(10.)," 10. is not appropriate substitute")
      self.assertFalse(s.isAppropriateValue(10)," 10 is not appropriate substitute")
      self.assertTrue(s.isAppropriateValue(Symbol(shape=(3, 2, 3, 4),dim=2)),"Symbol is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(Symbol(shape=(3, 2, 3, 4),dim=3)),"Symbol is not appropriate substitute (dim)")
      self.assertFalse(s.isAppropriateValue(escript.Data(0.,(5,),self.functionspace)),"Data is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(numpy.zeros((3, 2, 3, 4))),"ndarray is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(numpy.zeros((5,))),"ndarray is not appropriate substitute (shape)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_Symbol_rank4_d2_nargs1(self):
      s2=Symbol(shape=())
      s=Symbol(shape=(3, 2, 3, 4),dim=2,args=[s2])
      self.assertEqual(s.getRank(),4,"wrong rank.")
      self.assertEqual(s.getShape(),(3, 2, 3, 4),"wrong shape.")
      self.assertEqual(s.getDim(),2,"wrong spatial dimension.")
      self.assertEqual(s.getArgument(),[s2],"wrong arguments.")

      ss=s.substitute({s:numpy.zeros((3, 2, 3, 4))})
      self.assertTrue(isinstance(ss,numpy.ndarray),"value after substitution is not ndarray.")
      self.assertEqual(ss.shape,(3, 2, 3, 4),"value after substitution has not expected shape")
      try:
        s.substitute({s:numpy.zeros((5,))})
        fail("illegal substition was successful")
      except TypeError:
        pass

      dsdarg=s.diff(Symbol(shape=()))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds() has wrong type.")
      self.assertEqual(dsdarg.shape,(3, 2, 3, 4),"ds/ds() has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds() has wrong value.")

      self.assertEqual(s.getArgument(0),s2,"wrong argument 0.")
      sa=s.getSubstitutedArguments({s2:-10})
      self.assertEqual(len(sa),1,"wrong number of substituted arguments")
      self.assertEqual(sa[0],-10,"wrongly substituted argument 0.")

      self.assertFalse(s.isAppropriateValue(10.)," 10. is not appropriate substitute")
      self.assertFalse(s.isAppropriateValue(10)," 10 is not appropriate substitute")
      self.assertTrue(s.isAppropriateValue(Symbol(shape=(3, 2, 3, 4),dim=2)),"Symbol is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(Symbol(shape=(3, 2, 3, 4),dim=3)),"Symbol is not appropriate substitute (dim)")
      self.assertFalse(s.isAppropriateValue(escript.Data(0.,(5,),self.functionspace)),"Data is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(numpy.zeros((3, 2, 3, 4))),"ndarray is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(numpy.zeros((5,))),"ndarray is not appropriate substitute (shape)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_Symbol_rank4_d2_nargs2(self):
      s2=Symbol(shape=())
      s=Symbol(shape=(3, 2, 3, 4),dim=2,args=[1, -1.0])
      self.assertEqual(s.getRank(),4,"wrong rank.")
      self.assertEqual(s.getShape(),(3, 2, 3, 4),"wrong shape.")
      self.assertEqual(s.getDim(),2,"wrong spatial dimension.")
      self.assertEqual(s.getArgument(),[1, -1.0],"wrong arguments.")

      ss=s.substitute({s:numpy.zeros((3, 2, 3, 4))})
      self.assertTrue(isinstance(ss,numpy.ndarray),"value after substitution is not ndarray.")
      self.assertEqual(ss.shape,(3, 2, 3, 4),"value after substitution has not expected shape")
      try:
        s.substitute({s:numpy.zeros((5,))})
        fail("illegal substition was successful")
      except TypeError:
        pass

      dsdarg=s.diff(Symbol(shape=()))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds() has wrong type.")
      self.assertEqual(dsdarg.shape,(3, 2, 3, 4),"ds/ds() has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds() has wrong value.")

      self.assertEqual(s.getArgument(0),1,"wrong argument 0.")
      self.assertEqual(s.getArgument(1),-1.0,"wrong argument 1.")
      sa=s.getSubstitutedArguments({s2:-10})
      self.assertEqual(len(sa),2,"wrong number of substituted arguments")
      self.assertEqual(sa[0],1,"wrongly substituted argument 0.")
      self.assertEqual(sa[1],-1.0,"wrongly substituted argument 1.")

      self.assertFalse(s.isAppropriateValue(10.)," 10. is not appropriate substitute")
      self.assertFalse(s.isAppropriateValue(10)," 10 is not appropriate substitute")
      self.assertTrue(s.isAppropriateValue(Symbol(shape=(3, 2, 3, 4),dim=2)),"Symbol is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(Symbol(shape=(3, 2, 3, 4),dim=3)),"Symbol is not appropriate substitute (dim)")
      self.assertFalse(s.isAppropriateValue(escript.Data(0.,(5,),self.functionspace)),"Data is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(numpy.zeros((3, 2, 3, 4))),"ndarray is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(numpy.zeros((5,))),"ndarray is not appropriate substitute (shape)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_Symbol_rank4_d3_nargs0(self):
      s2=Symbol(shape=())
      s=Symbol(shape=(3, 2, 3, 4),dim=3,args=[])
      self.assertEqual(s.getRank(),4,"wrong rank.")
      self.assertEqual(s.getShape(),(3, 2, 3, 4),"wrong shape.")
      self.assertEqual(s.getDim(),3,"wrong spatial dimension.")
      self.assertEqual(s.getArgument(),[],"wrong arguments.")

      ss=s.substitute({s:numpy.zeros((3, 2, 3, 4))})
      self.assertTrue(isinstance(ss,numpy.ndarray),"value after substitution is not ndarray.")
      self.assertEqual(ss.shape,(3, 2, 3, 4),"value after substitution has not expected shape")
      try:
        s.substitute({s:numpy.zeros((5,))})
        fail("illegal substition was successful")
      except TypeError:
        pass

      dsdarg=s.diff(Symbol(shape=()))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds() has wrong type.")
      self.assertEqual(dsdarg.shape,(3, 2, 3, 4),"ds/ds() has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds() has wrong value.")

      sa=s.getSubstitutedArguments({s2:-10})
      self.assertEqual(len(sa),0,"wrong number of substituted arguments")

      self.assertFalse(s.isAppropriateValue(10.)," 10. is not appropriate substitute")
      self.assertFalse(s.isAppropriateValue(10)," 10 is not appropriate substitute")
      self.assertTrue(s.isAppropriateValue(Symbol(shape=(3, 2, 3, 4),dim=3)),"Symbol is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(Symbol(shape=(3, 2, 3, 4),dim=4)),"Symbol is not appropriate substitute (dim)")
      self.assertFalse(s.isAppropriateValue(escript.Data(0.,(5,),self.functionspace)),"Data is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(numpy.zeros((3, 2, 3, 4))),"ndarray is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(numpy.zeros((5,))),"ndarray is not appropriate substitute (shape)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_Symbol_rank4_d3_nargs1(self):
      s2=Symbol(shape=())
      s=Symbol(shape=(3, 2, 3, 4),dim=3,args=[s2])
      self.assertEqual(s.getRank(),4,"wrong rank.")
      self.assertEqual(s.getShape(),(3, 2, 3, 4),"wrong shape.")
      self.assertEqual(s.getDim(),3,"wrong spatial dimension.")
      self.assertEqual(s.getArgument(),[s2],"wrong arguments.")

      ss=s.substitute({s:numpy.zeros((3, 2, 3, 4))})
      self.assertTrue(isinstance(ss,numpy.ndarray),"value after substitution is not ndarray.")
      self.assertEqual(ss.shape,(3, 2, 3, 4),"value after substitution has not expected shape")
      try:
        s.substitute({s:numpy.zeros((5,))})
        fail("illegal substition was successful")
      except TypeError:
        pass

      dsdarg=s.diff(Symbol(shape=()))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds() has wrong type.")
      self.assertEqual(dsdarg.shape,(3, 2, 3, 4),"ds/ds() has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds() has wrong value.")

      self.assertEqual(s.getArgument(0),s2,"wrong argument 0.")
      sa=s.getSubstitutedArguments({s2:-10})
      self.assertEqual(len(sa),1,"wrong number of substituted arguments")
      self.assertEqual(sa[0],-10,"wrongly substituted argument 0.")

      self.assertFalse(s.isAppropriateValue(10.)," 10. is not appropriate substitute")
      self.assertFalse(s.isAppropriateValue(10)," 10 is not appropriate substitute")
      self.assertTrue(s.isAppropriateValue(Symbol(shape=(3, 2, 3, 4),dim=3)),"Symbol is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(Symbol(shape=(3, 2, 3, 4),dim=4)),"Symbol is not appropriate substitute (dim)")
      self.assertFalse(s.isAppropriateValue(escript.Data(0.,(5,),self.functionspace)),"Data is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(numpy.zeros((3, 2, 3, 4))),"ndarray is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(numpy.zeros((5,))),"ndarray is not appropriate substitute (shape)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_Symbol_rank4_d3_nargs2(self):
      s2=Symbol(shape=())
      s=Symbol(shape=(3, 2, 3, 4),dim=3,args=[1, -1.0])
      self.assertEqual(s.getRank(),4,"wrong rank.")
      self.assertEqual(s.getShape(),(3, 2, 3, 4),"wrong shape.")
      self.assertEqual(s.getDim(),3,"wrong spatial dimension.")
      self.assertEqual(s.getArgument(),[1, -1.0],"wrong arguments.")

      ss=s.substitute({s:numpy.zeros((3, 2, 3, 4))})
      self.assertTrue(isinstance(ss,numpy.ndarray),"value after substitution is not ndarray.")
      self.assertEqual(ss.shape,(3, 2, 3, 4),"value after substitution has not expected shape")
      try:
        s.substitute({s:numpy.zeros((5,))})
        fail("illegal substition was successful")
      except TypeError:
        pass
      dsdarg=s.diff(Symbol(shape=()))
      self.assertTrue(isinstance(dsdarg,numpy.ndarray),"ds/ds() has wrong type.")
      self.assertEqual(dsdarg.shape,(3, 2, 3, 4),"ds/ds() has wrong shape.")
      self.assertFalse(Lsup(dsdarg)>0.,"ds/ds() has wrong value.")

      self.assertEqual(s.getArgument(0),1,"wrong argument 0.")
      self.assertEqual(s.getArgument(1),-1.0,"wrong argument 1.")
      sa=s.getSubstitutedArguments({s2:-10})
      self.assertEqual(len(sa),2,"wrong number of substituted arguments")
      self.assertEqual(sa[0],1,"wrongly substituted argument 0.")
      self.assertEqual(sa[1],-1.0,"wrongly substituted argument 1.")

      self.assertFalse(s.isAppropriateValue(10.)," 10. is not appropriate substitute")
      self.assertFalse(s.isAppropriateValue(10)," 10 is not appropriate substitute")
      self.assertTrue(s.isAppropriateValue(Symbol(shape=(3, 2, 3, 4),dim=3)),"Symbol is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(Symbol(shape=(3, 2, 3, 4),dim=4)),"Symbol is not appropriate substitute (dim)")
      self.assertFalse(s.isAppropriateValue(escript.Data(0.,(5,),self.functionspace)),"Data is not appropriate substitute (shape)")
      self.assertTrue(s.isAppropriateValue(numpy.zeros((3, 2, 3, 4))),"ndarray is appropriate substitute")
      self.assertFalse(s.isAppropriateValue(numpy.zeros((5,))),"ndarray is not appropriate substitute (shape)")

   def test_DependendSymbol(self):
      class TestC(DependendSymbol):
          pass
      s=DependendSymbol(shape=(3, 2, 3, 4),dim=3,args=[1, -1.0])
      self.assertTrue(s==DependendSymbol(shape=(3, 2, 3, 4),dim=3,args=[1, -1.0]),"identical symbol not detected")
      self.assertTrue(s==DependendSymbol(shape=(3, 2, 3, 4),dim=None,args=[1, -1.0]),"identical symbol with undefined dimension not detected")
      self.assertFalse(s==Symbol(),"different object type")
      self.assertFalse(s==TestC(shape=(3, 2, 3, 4),dim=None,args=[1, -1.0]),"different DependendSymbol subclass not detected")
      self.assertFalse(s==DependendSymbol(shape=(3,),dim=3,args=[1, -1.0]),"different shape not detected.")
      self.assertFalse(s==DependendSymbol(shape=(3,2,3,4),dim=2,args=[1, -1.0]),"different dimension.")
      self.assertFalse(s==DependendSymbol(shape=(3,2,3,4),dim=3,args=[1, 1.0]),"different arguments not detected.")
