
########################################################
#
# Copyright (c) 2003-2012 by University of Queensland
# Earth Systems Science Computational Center (ESSCC)
# http://www.uq.edu.au/esscc
#
# Primary Business: Queensland, Australia
# Licensed under the Open Software License version 3.0
# http://www.opensource.org/licenses/osl-3.0.php
#
########################################################

__copyright__="""Copyright (c) 2003-2012 by University of Queensland
Earth Systems Science Computational Center (ESSCC)
http://www.uq.edu.au/esscc
Primary Business: Queensland, Australia"""
__license__="""Licensed under the Open Software License version 3.0
http://www.opensource.org/licenses/osl-3.0.php"""
__url__="https://launchpad.net/escript-finley"

"""
basic tests for functions in util.py effecting the spatial distribution 

it is assumed that the domain is the usint square/cube 

not all these test will run for all domains. check the doc string for the assumptions of a particular test

:var __author__: name of author
:var __copyright__: copyrights
:var __license__: licence agreement
:var __url__: url entry point on documentation
:var __version__: version
:var __date__: date of the version
"""
__author__="Lutz Gross, l.gross@uq.edu.au"

import unittest
from esys.escript import *
from numpy import array

from test_util_spatial_functions1 import Test_Util_SpatialFunctions_noGradOnBoundary_noContact

#begin

class Test_Util_SpatialFunctions_noGradOnBoundary(Test_Util_SpatialFunctions_noGradOnBoundary_noContact):
   RES_TOL=1.e-8

   def test_normal_onFunctionOnContactZero(self):
     """
     test getNormal() on contact side 0

     assumptions: FunctionOnContactZero(self.domain) exists
     """
     dim=self.domain.getDim()
     f=FunctionOnContactZero(self.domain)
     x=f.getX()
     ref=Vector(0.,what=f)
     if dim==3:
         ref+=whereZero(x[0]-0.5,tol=self.RES_TOL)*[1,0,0]
     else:
         ref+=whereZero(x[0]-0.5,tol=self.RES_TOL)*[1,0]

     res=f.getNormal()
     self.assertEqual(res.getShape(),(dim,),"wrong shape of result.")
     self.assertEqual(res.getFunctionSpace(),f,"wrong functionspace of result.")
     self.assertTrue(Lsup(length(res)-1)<=self.RES_TOL,"wrong length")
     self.assertTrue(Lsup(abs(inner(ref,res))-1)<=self.RES_TOL,"wrong direction")
   def test_normal_onReducedFunctionOnContactZero(self):
     """
     test getNormal() on contact side 0

     assumptions: FunctionOnContactZero(self.domain) exists
     """
     dim=self.domain.getDim()
     f=ReducedFunctionOnContactZero(self.domain)
     x=f.getX()
     ref=Vector(0.,what=f)
     if dim==3:
         ref+=whereZero(x[0]-0.5,tol=self.RES_TOL)*[1,0,0]
     else:
         ref+=whereZero(x[0]-0.5,tol=self.RES_TOL)*[1,0]

     res=f.getNormal()
     self.assertEqual(res.getShape(),(dim,),"wrong shape of result.")
     self.assertEqual(res.getFunctionSpace(),f,"wrong functionspace of result.")
     self.assertTrue(Lsup(length(res)-1)<=self.RES_TOL,"wrong length")
     self.assertTrue(Lsup(abs(inner(ref,res))-1)<=self.RES_TOL,"wrong direction")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_normal_onFunctionOnContactOne(self):
     """
     test getNormal() on contact side 1

     assumptions: FunctionOnContactOne(self.domain) exists
     """
     dim=self.domain.getDim()
     f=FunctionOnContactOne(self.domain)
     x=f.getX()
     ref=Vector(0.,what=f)
     if dim==3:
         ref+=whereZero(x[0]-0.5,tol=self.RES_TOL)*[-1,0,0]
     else:
         ref+=whereZero(x[0]-0.5,tol=self.RES_TOL)*[-1,0]

     res=f.getNormal()
     self.assertEqual(res.getShape(),(dim,),"wrong shape of result.")
     self.assertEqual(res.getFunctionSpace(),f,"wrong functionspace of result.")
     self.assertTrue(Lsup(length(res)-1)<=self.RES_TOL,"wrong length")
     self.assertTrue(Lsup(abs(inner(ref,res))-1)<=self.RES_TOL,"wrong direction")
   def test_normal_onReducedFunctionOnContactOne(self):
     """
     test getNormal() on contact side 1

     assumptions: FunctionOnContactOne(self.domain) exists
     """
     dim=self.domain.getDim()
     f=ReducedFunctionOnContactOne(self.domain)
     x=f.getX()
     ref=Vector(0.,what=f)
     if dim==3:
         ref+=whereZero(x[0]-0.5,tol=self.RES_TOL)*[-1,0,0]
     else:
         ref+=whereZero(x[0]-0.5,tol=self.RES_TOL)*[-1,0]

     res=f.getNormal()
     self.assertEqual(res.getShape(),(dim,),"wrong shape of result.")
     self.assertEqual(res.getFunctionSpace(),f,"wrong functionspace of result.")
     self.assertTrue(Lsup(length(res)-1)<=self.RES_TOL,"wrong length")
     self.assertTrue(Lsup(abs(inner(ref,res))-1)<=self.RES_TOL,"wrong direction")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactZero_fromData_ContinuousFunction_rank0(self):
      """
      tests integral of rank 0 Data on the FunctionOnContactZero

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports integral on FunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(),w)
      if dim==2:
        arg=(0.311774053155)*x[0]**o+(0.52292841627)*x[0]+(-0.518467126196)*x[1]**o+(-0.11329290159)*x[1]
        ref=(-0.518467126196)/(o+1.)+(0.20481775734)+(0.311774053155)*0.5**o
      else:
        arg=(0.233765319091)*x[0]**o+(-0.453551209762)*x[0]+(0.920140231713)*x[1]**o+(0.741695479084)*x[1]+(-0.255834461065)*x[2]**o+(-0.102580899053)*x[2]
        ref=(0.664305770648)/(o+1.)+(0.0927816851348)+(0.233765319091)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactZero_fromData_ContinuousFunction_rank1(self):
      """
      tests integral of rank 1 Data on the FunctionOnContactZero

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports integral on FunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(2,),w)
      ref=numpy.zeros((2,),numpy.float_)
      if dim==2:
        arg[0]=(-0.651678780872)*x[0]**o+(-0.749709447394)*x[0]+(0.311088209082)*x[1]**o+(0.553081401193)*x[1]
        arg[1]=(0.657887728987)*x[0]**o+(0.052270143604)*x[0]+(-0.15507994277)*x[1]**o+(-0.30302150082)*x[1]
        ref[0]=(0.311088209082)/(o+1.)+(-0.0983140231004)+(-0.651678780872)*0.5**o
        ref[1]=(-0.15507994277)/(o+1.)+(-0.125375678608)+(0.657887728987)*0.5**o
      else:
        arg[0]=(0.844909347644)*x[0]**o+(-0.342697697483)*x[0]+(0.126242548186)*x[1]**o+(-0.854010732849)*x[1]+(-0.307627587461)*x[2]**o+(-0.998566290754)*x[2]
        arg[1]=(0.900073726701)*x[0]**o+(-0.703841701859)*x[0]+(-0.651690896499)*x[1]**o+(0.828362322181)*x[1]+(0.284410420895)*x[2]**o+(-0.394606348974)*x[2]
        ref[0]=(-0.181385039275)/(o+1.)+(-1.09763736054)+(0.844909347644)*0.5**o
        ref[1]=(-0.367280475604)/(o+1.)+(-0.135042864326)+(0.900073726701)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactZero_fromData_ContinuousFunction_rank2(self):
      """
      tests integral of rank 2 Data on the FunctionOnContactZero

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports integral on FunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5),w)
      ref=numpy.zeros((4, 5),numpy.float_)
      if dim==2:
        arg[0,0]=(-0.263642681371)*x[0]**o+(0.668992571787)*x[0]+(0.873939846194)*x[1]**o+(0.784820355998)*x[1]
        arg[0,1]=(-0.389306088628)*x[0]**o+(-0.0549482389907)*x[0]+(-0.787101008457)*x[1]**o+(0.860345123812)*x[1]
        arg[0,2]=(-0.995721191961)*x[0]**o+(-0.587928136153)*x[0]+(-0.378312614832)*x[1]**o+(0.199765674281)*x[1]
        arg[0,3]=(-0.374679425145)*x[0]**o+(-0.503606183146)*x[0]+(0.23995738992)*x[1]**o+(-0.525359181744)*x[1]
        arg[0,4]=(-0.69923255665)*x[0]**o+(0.547634979941)*x[0]+(0.728792702739)*x[1]**o+(0.949888533828)*x[1]
        arg[1,0]=(-0.61856415528)*x[0]**o+(-0.0516239651093)*x[0]+(0.158516649979)*x[1]**o+(-0.0522288881011)*x[1]
        arg[1,1]=(0.0285524133857)*x[0]**o+(0.741357758307)*x[0]+(-0.0714860903847)*x[1]**o+(-0.539632750385)*x[1]
        arg[1,2]=(-0.291101401382)*x[0]**o+(-0.114900246437)*x[0]+(-0.982305031665)*x[1]**o+(0.0823478844055)*x[1]
        arg[1,3]=(-0.931305732525)*x[0]**o+(0.077332436417)*x[0]+(-0.880626154921)*x[1]**o+(-0.321222234507)*x[1]
        arg[1,4]=(-0.00778033208815)*x[0]**o+(-0.536626847957)*x[0]+(-0.599742714546)*x[1]**o+(0.454108299336)*x[1]
        arg[2,0]=(-0.575243090445)*x[0]**o+(-0.920247228564)*x[0]+(0.4569784462)*x[1]**o+(-0.105175798804)*x[1]
        arg[2,1]=(0.333985911748)*x[0]**o+(-0.017876213076)*x[0]+(0.475164778165)*x[1]**o+(0.561072178015)*x[1]
        arg[2,2]=(-0.559587786773)*x[0]**o+(-0.917134128672)*x[0]+(0.316654889327)*x[1]**o+(-0.640569595027)*x[1]
        arg[2,3]=(-0.140461324817)*x[0]**o+(0.259052969883)*x[0]+(0.397898509159)*x[1]**o+(0.932233331785)*x[1]
        arg[2,4]=(0.368626304985)*x[0]**o+(0.616310502384)*x[0]+(-0.756845293566)*x[1]**o+(-0.920088968132)*x[1]
        arg[3,0]=(-0.824398741087)*x[0]**o+(0.0498115894252)*x[0]+(-0.5256621655)*x[1]**o+(0.114806237108)*x[1]
        arg[3,1]=(-0.77671057316)*x[0]**o+(0.569249443162)*x[0]+(0.416628465801)*x[1]**o+(0.0603627487913)*x[1]
        arg[3,2]=(-0.247875220913)*x[0]**o+(0.19548828878)*x[0]+(0.643226101838)*x[1]**o+(-0.154558826499)*x[1]
        arg[3,3]=(-0.918937118963)*x[0]**o+(0.193777174417)*x[0]+(-0.906042347107)*x[1]**o+(0.872033230033)*x[1]
        arg[3,4]=(-0.628918336086)*x[0]**o+(0.823242191714)*x[0]+(-0.593691166712)*x[1]**o+(-0.661715840814)*x[1]
        ref[0,0]=(0.873939846194)/(o+1.)+(0.726906463892)+(-0.263642681371)*0.5**o
        ref[0,1]=(-0.787101008457)/(o+1.)+(0.402698442411)+(-0.389306088628)*0.5**o
        ref[0,2]=(-0.378312614832)/(o+1.)+(-0.194081230936)+(-0.995721191961)*0.5**o
        ref[0,3]=(0.23995738992)/(o+1.)+(-0.514482682445)+(-0.374679425145)*0.5**o
        ref[0,4]=(0.728792702739)/(o+1.)+(0.748761756885)+(-0.69923255665)*0.5**o
        ref[1,0]=(0.158516649979)/(o+1.)+(-0.0519264266052)+(-0.61856415528)*0.5**o
        ref[1,1]=(-0.0714860903847)/(o+1.)+(0.100862503961)+(0.0285524133857)*0.5**o
        ref[1,2]=(-0.982305031665)/(o+1.)+(-0.0162761810159)+(-0.291101401382)*0.5**o
        ref[1,3]=(-0.880626154921)/(o+1.)+(-0.121944899045)+(-0.931305732525)*0.5**o
        ref[1,4]=(-0.599742714546)/(o+1.)+(-0.0412592743102)+(-0.00778033208815)*0.5**o
        ref[2,0]=(0.4569784462)/(o+1.)+(-0.512711513684)+(-0.575243090445)*0.5**o
        ref[2,1]=(0.475164778165)/(o+1.)+(0.27159798247)+(0.333985911748)*0.5**o
        ref[2,2]=(0.316654889327)/(o+1.)+(-0.778851861849)+(-0.559587786773)*0.5**o
        ref[2,3]=(0.397898509159)/(o+1.)+(0.595643150834)+(-0.140461324817)*0.5**o
        ref[2,4]=(-0.756845293566)/(o+1.)+(-0.151889232874)+(0.368626304985)*0.5**o
        ref[3,0]=(-0.5256621655)/(o+1.)+(0.0823089132668)+(-0.824398741087)*0.5**o
        ref[3,1]=(0.416628465801)/(o+1.)+(0.314806095977)+(-0.77671057316)*0.5**o
        ref[3,2]=(0.643226101838)/(o+1.)+(0.0204647311405)+(-0.247875220913)*0.5**o
        ref[3,3]=(-0.906042347107)/(o+1.)+(0.532905202225)+(-0.918937118963)*0.5**o
        ref[3,4]=(-0.593691166712)/(o+1.)+(0.0807631754499)+(-0.628918336086)*0.5**o
      else:
        arg[0,0]=(0.748857269399)*x[0]**o+(-0.21982973469)*x[0]+(-0.765987921295)*x[1]**o+(0.556577019909)*x[1]+(-0.0438548079278)*x[2]**o+(0.543610874817)*x[2]
        arg[0,1]=(-0.784526004748)*x[0]**o+(-0.949423172035)*x[0]+(-0.29888188317)*x[1]**o+(-0.850638766848)*x[1]+(-0.853027558387)*x[2]**o+(-0.846378486588)*x[2]
        arg[0,2]=(0.53445882525)*x[0]**o+(-0.653655652989)*x[0]+(-0.968129875689)*x[1]**o+(-0.603598808323)*x[1]+(0.545841014272)*x[2]**o+(0.126131861082)*x[2]
        arg[0,3]=(0.181662498503)*x[0]**o+(-0.769571080865)*x[0]+(0.771109181078)*x[1]**o+(0.980677373303)*x[1]+(-0.625057940268)*x[2]**o+(0.0585672357346)*x[2]
        arg[0,4]=(-0.0320907540755)*x[0]**o+(-0.519517369917)*x[0]+(-0.365190231996)*x[1]**o+(-0.192392652661)*x[1]+(-0.693267159516)*x[2]**o+(0.106458640425)*x[2]
        arg[1,0]=(0.414286358161)*x[0]**o+(0.0223282279507)*x[0]+(0.215881279469)*x[1]**o+(0.68990977849)*x[1]+(-0.532075885581)*x[2]**o+(-0.269164904232)*x[2]
        arg[1,1]=(-0.0445879250589)*x[0]**o+(0.889985400599)*x[0]+(0.842050582947)*x[1]**o+(-0.0275307884642)*x[1]+(0.774650107671)*x[2]**o+(0.19446514971)*x[2]
        arg[1,2]=(0.913032880034)*x[0]**o+(-0.286537470008)*x[0]+(0.0902778267032)*x[1]**o+(0.548596933564)*x[1]+(-0.996100405074)*x[2]**o+(-0.509554019549)*x[2]
        arg[1,3]=(0.44308812765)*x[0]**o+(-0.0641227726924)*x[0]+(0.484249008075)*x[1]**o+(0.581496384894)*x[1]+(0.592538533183)*x[2]**o+(-0.821638541512)*x[2]
        arg[1,4]=(-0.898304735153)*x[0]**o+(0.333217967121)*x[0]+(-0.788218899371)*x[1]**o+(0.579723459039)*x[1]+(0.467006473159)*x[2]**o+(-0.277113860535)*x[2]
        arg[2,0]=(0.515308716564)*x[0]**o+(0.255947750577)*x[0]+(0.232578337945)*x[1]**o+(0.900611887592)*x[1]+(-0.247710965611)*x[2]**o+(-0.985523228221)*x[2]
        arg[2,1]=(0.611196756618)*x[0]**o+(0.146822999187)*x[0]+(-0.318793147278)*x[1]**o+(0.790772432028)*x[1]+(0.138036152668)*x[2]**o+(-0.800234207543)*x[2]
        arg[2,2]=(-0.293576196546)*x[0]**o+(-0.521059592737)*x[0]+(0.301178293957)*x[1]**o+(-0.141430900198)*x[1]+(-0.782381874467)*x[2]**o+(0.92221179729)*x[2]
        arg[2,3]=(0.673951778483)*x[0]**o+(-0.335068772752)*x[0]+(0.408793200723)*x[1]**o+(0.36906212176)*x[1]+(0.246292093855)*x[2]**o+(0.356147233675)*x[2]
        arg[2,4]=(-0.218899302236)*x[0]**o+(0.271285962886)*x[0]+(-0.16364707283)*x[1]**o+(0.177195607727)*x[1]+(-0.287331396699)*x[2]**o+(-0.465391460497)*x[2]
        arg[3,0]=(-0.369727856735)*x[0]**o+(-0.44351931666)*x[0]+(0.0375097686939)*x[1]**o+(0.276779121402)*x[1]+(0.662505672881)*x[2]**o+(-0.642709866174)*x[2]
        arg[3,1]=(0.444940917816)*x[0]**o+(-0.178926126454)*x[0]+(0.824660645994)*x[1]**o+(0.825996820355)*x[1]+(0.798260651848)*x[2]**o+(0.331744022239)*x[2]
        arg[3,2]=(-0.82489261483)*x[0]**o+(-0.340862079766)*x[0]+(-0.998375587383)*x[1]**o+(0.880222106116)*x[1]+(0.656096580473)*x[2]**o+(0.530854486127)*x[2]
        arg[3,3]=(0.732863411435)*x[0]**o+(-0.324252004147)*x[0]+(0.789472654884)*x[1]**o+(-0.207794796011)*x[1]+(0.50903180567)*x[2]**o+(0.724319112504)*x[2]
        arg[3,4]=(-0.980748139471)*x[0]**o+(0.0974331161906)*x[0]+(0.196275528452)*x[1]**o+(0.305231910003)*x[1]+(-0.0417825350947)*x[2]**o+(-0.2916834261)*x[2]
        ref[0,0]=(-0.809842729223)/(o+1.)+(0.440179080018)+(0.748857269399)*0.5**o
        ref[0,1]=(-1.15190944156)/(o+1.)+(-1.32322021274)+(-0.784526004748)*0.5**o
        ref[0,2]=(-0.422288861417)/(o+1.)+(-0.565561300115)+(0.53445882525)*0.5**o
        ref[0,3]=(0.146051240811)/(o+1.)+(0.134836764087)+(0.181662498503)*0.5**o
        ref[0,4]=(-1.05845739151)/(o+1.)+(-0.302725691077)+(-0.0320907540755)*0.5**o
        ref[1,0]=(-0.316194606113)/(o+1.)+(0.221536551104)+(0.414286358161)*0.5**o
        ref[1,1]=(1.61670069062)/(o+1.)+(0.528459880922)+(-0.0445879250589)*0.5**o
        ref[1,2]=(-0.905822578371)/(o+1.)+(-0.123747277996)+(0.913032880034)*0.5**o
        ref[1,3]=(1.07678754126)/(o+1.)+(-0.152132464655)+(0.44308812765)*0.5**o
        ref[1,4]=(-0.321212426212)/(o+1.)+(0.317913782812)+(-0.898304735153)*0.5**o
        ref[2,0]=(-0.015132627666)/(o+1.)+(0.085518204974)+(0.515308716564)*0.5**o
        ref[2,1]=(-0.18075699461)/(o+1.)+(0.0686806118359)+(0.611196756618)*0.5**o
        ref[2,2]=(-0.481203580509)/(o+1.)+(0.129860652178)+(-0.293576196546)*0.5**o
        ref[2,3]=(0.655085294578)/(o+1.)+(0.195070291341)+(0.673951778483)*0.5**o
        ref[2,4]=(-0.450978469529)/(o+1.)+(-0.00845494494169)+(-0.218899302236)*0.5**o
        ref[3,0]=(0.700015441575)/(o+1.)+(-0.404725030716)+(-0.369727856735)*0.5**o
        ref[3,1]=(1.62292129784)/(o+1.)+(0.48940735807)+(0.444940917816)*0.5**o
        ref[3,2]=(-0.342279006911)/(o+1.)+(0.535107256239)+(-0.82489261483)*0.5**o
        ref[3,3]=(1.29850446055)/(o+1.)+(0.0961361561734)+(0.732863411435)*0.5**o
        ref[3,4]=(0.154492993358)/(o+1.)+(0.055490800047)+(-0.980748139471)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactZero_fromData_ContinuousFunction_rank3(self):
      """
      tests integral of rank 3 Data on the FunctionOnContactZero

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports integral on FunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(6, 2, 2),w)
      ref=numpy.zeros((6, 2, 2),numpy.float_)
      if dim==2:
        arg[0,0,0]=(-0.33788666753)*x[0]**o+(0.0147671449643)*x[0]+(0.14066054662)*x[1]**o+(-0.675231899756)*x[1]
        arg[0,0,1]=(0.0100961441611)*x[0]**o+(0.401304171984)*x[0]+(-0.594786282119)*x[1]**o+(-0.561543495067)*x[1]
        arg[0,1,0]=(-0.118311175162)*x[0]**o+(0.62454992688)*x[0]+(-0.8163725079)*x[1]**o+(-0.756355655277)*x[1]
        arg[0,1,1]=(0.586991709411)*x[0]**o+(-0.600757941586)*x[0]+(0.0953383109011)*x[1]**o+(0.317827629801)*x[1]
        arg[1,0,0]=(-0.251125236294)*x[0]**o+(0.166603616954)*x[0]+(0.978704943828)*x[1]**o+(0.589215389731)*x[1]
        arg[1,0,1]=(0.0543182786738)*x[0]**o+(-0.334756305272)*x[0]+(-0.919571042987)*x[1]**o+(-0.0425734727213)*x[1]
        arg[1,1,0]=(-0.867177621518)*x[0]**o+(-0.0750659593232)*x[0]+(-0.227277029921)*x[1]**o+(0.361120763296)*x[1]
        arg[1,1,1]=(0.85129514889)*x[0]**o+(-0.441962000795)*x[0]+(0.425335668395)*x[1]**o+(0.204091109081)*x[1]
        arg[2,0,0]=(-0.537930401585)*x[0]**o+(-0.206510941636)*x[0]+(-0.247374421196)*x[1]**o+(0.67043841725)*x[1]
        arg[2,0,1]=(-0.101273864474)*x[0]**o+(0.283769727506)*x[0]+(-0.0850059833654)*x[1]**o+(-0.71291661379)*x[1]
        arg[2,1,0]=(0.684430088582)*x[0]**o+(0.107733116086)*x[0]+(0.40398578341)*x[1]**o+(-0.525004991854)*x[1]
        arg[2,1,1]=(-0.00801759014554)*x[0]**o+(-0.214858014851)*x[0]+(-0.427822011518)*x[1]**o+(0.0168962506945)*x[1]
        arg[3,0,0]=(0.528563052182)*x[0]**o+(-0.630485796029)*x[0]+(0.0525765317703)*x[1]**o+(-0.469639006097)*x[1]
        arg[3,0,1]=(0.178622695427)*x[0]**o+(0.0952996500383)*x[0]+(-0.785851578496)*x[1]**o+(0.856665398338)*x[1]
        arg[3,1,0]=(0.547391926072)*x[0]**o+(-0.0179195091852)*x[0]+(-0.831368961862)*x[1]**o+(-0.437229107453)*x[1]
        arg[3,1,1]=(-0.657011566453)*x[0]**o+(0.530176142378)*x[0]+(0.649145894746)*x[1]**o+(-0.755185337598)*x[1]
        arg[4,0,0]=(-0.188305872652)*x[0]**o+(0.0207245930982)*x[0]+(-0.590757998421)*x[1]**o+(-0.793662033952)*x[1]
        arg[4,0,1]=(0.417712824349)*x[0]**o+(-0.138327612527)*x[0]+(-0.381595921766)*x[1]**o+(0.0287368364725)*x[1]
        arg[4,1,0]=(-0.83750540403)*x[0]**o+(0.247261273008)*x[0]+(-0.0769629479034)*x[1]**o+(0.0693501875686)*x[1]
        arg[4,1,1]=(0.697434044074)*x[0]**o+(0.396582403702)*x[0]+(0.736132107034)*x[1]**o+(0.482245183242)*x[1]
        arg[5,0,0]=(0.769936158184)*x[0]**o+(0.808871896459)*x[0]+(-0.559205314785)*x[1]**o+(-0.258014827146)*x[1]
        arg[5,0,1]=(0.667132553969)*x[0]**o+(0.618184492083)*x[0]+(0.475852440157)*x[1]**o+(0.084499293443)*x[1]
        arg[5,1,0]=(0.910882712823)*x[0]**o+(0.153897139268)*x[0]+(-0.61197280039)*x[1]**o+(0.441331985295)*x[1]
        arg[5,1,1]=(-0.290188623857)*x[0]**o+(0.318266320131)*x[0]+(0.9587856247)*x[1]**o+(0.311981486508)*x[1]
        ref[0,0,0]=(0.14066054662)/(o+1.)+(-0.330232377396)+(-0.33788666753)*0.5**o
        ref[0,0,1]=(-0.594786282119)/(o+1.)+(-0.0801196615418)+(0.0100961441611)*0.5**o
        ref[0,1,0]=(-0.8163725079)/(o+1.)+(-0.0659028641985)+(-0.118311175162)*0.5**o
        ref[0,1,1]=(0.0953383109011)/(o+1.)+(-0.141465155893)+(0.586991709411)*0.5**o
        ref[1,0,0]=(0.978704943828)/(o+1.)+(0.377909503343)+(-0.251125236294)*0.5**o
        ref[1,0,1]=(-0.919571042987)/(o+1.)+(-0.188664888997)+(0.0543182786738)*0.5**o
        ref[1,1,0]=(-0.227277029921)/(o+1.)+(0.143027401987)+(-0.867177621518)*0.5**o
        ref[1,1,1]=(0.425335668395)/(o+1.)+(-0.118935445857)+(0.85129514889)*0.5**o
        ref[2,0,0]=(-0.247374421196)/(o+1.)+(0.231963737807)+(-0.537930401585)*0.5**o
        ref[2,0,1]=(-0.0850059833654)/(o+1.)+(-0.214573443142)+(-0.101273864474)*0.5**o
        ref[2,1,0]=(0.40398578341)/(o+1.)+(-0.208635937884)+(0.684430088582)*0.5**o
        ref[2,1,1]=(-0.427822011518)/(o+1.)+(-0.0989808820785)+(-0.00801759014554)*0.5**o
        ref[3,0,0]=(0.0525765317703)/(o+1.)+(-0.550062401063)+(0.528563052182)*0.5**o
        ref[3,0,1]=(-0.785851578496)/(o+1.)+(0.475982524188)+(0.178622695427)*0.5**o
        ref[3,1,0]=(-0.831368961862)/(o+1.)+(-0.227574308319)+(0.547391926072)*0.5**o
        ref[3,1,1]=(0.649145894746)/(o+1.)+(-0.11250459761)+(-0.657011566453)*0.5**o
        ref[4,0,0]=(-0.590757998421)/(o+1.)+(-0.386468720427)+(-0.188305872652)*0.5**o
        ref[4,0,1]=(-0.381595921766)/(o+1.)+(-0.0547953880271)+(0.417712824349)*0.5**o
        ref[4,1,0]=(-0.0769629479034)/(o+1.)+(0.158305730288)+(-0.83750540403)*0.5**o
        ref[4,1,1]=(0.736132107034)/(o+1.)+(0.439413793472)+(0.697434044074)*0.5**o
        ref[5,0,0]=(-0.559205314785)/(o+1.)+(0.275428534657)+(0.769936158184)*0.5**o
        ref[5,0,1]=(0.475852440157)/(o+1.)+(0.351341892763)+(0.667132553969)*0.5**o
        ref[5,1,0]=(-0.61197280039)/(o+1.)+(0.297614562282)+(0.910882712823)*0.5**o
        ref[5,1,1]=(0.9587856247)/(o+1.)+(0.315123903319)+(-0.290188623857)*0.5**o
      else:
        arg[0,0,0]=(0.292346424086)*x[0]**o+(0.105058465241)*x[0]+(-0.281467283372)*x[1]**o+(0.828345267021)*x[1]+(0.741242558638)*x[2]**o+(0.0858780765977)*x[2]
        arg[0,0,1]=(-0.460556250835)*x[0]**o+(-0.845199338719)*x[0]+(-0.547000154821)*x[1]**o+(-0.972236450512)*x[1]+(0.823031420765)*x[2]**o+(0.278285294464)*x[2]
        arg[0,1,0]=(0.380027894487)*x[0]**o+(-0.579359210385)*x[0]+(-0.760849798908)*x[1]**o+(0.0065870459398)*x[1]+(-0.869735580799)*x[2]**o+(-0.937378284335)*x[2]
        arg[0,1,1]=(0.948682597897)*x[0]**o+(-0.882044080711)*x[0]+(-0.694398833899)*x[1]**o+(0.883635235205)*x[1]+(-0.473761903609)*x[2]**o+(0.961453193379)*x[2]
        arg[1,0,0]=(0.357218603696)*x[0]**o+(-0.694504294144)*x[0]+(-0.187119222895)*x[1]**o+(0.727463485088)*x[1]+(0.150410254875)*x[2]**o+(-0.373430202907)*x[2]
        arg[1,0,1]=(-0.776893233002)*x[0]**o+(0.565238637388)*x[0]+(-0.155674628662)*x[1]**o+(-0.938226259914)*x[1]+(0.575183633137)*x[2]**o+(0.319871583783)*x[2]
        arg[1,1,0]=(0.593512911569)*x[0]**o+(-0.0578762085927)*x[0]+(-0.913450847289)*x[1]**o+(0.7074653217)*x[1]+(0.991175636143)*x[2]**o+(-0.535632452281)*x[2]
        arg[1,1,1]=(-0.154542349616)*x[0]**o+(-0.841642050346)*x[0]+(0.878833704431)*x[1]**o+(0.85568524402)*x[1]+(-0.710374336281)*x[2]**o+(0.346540187252)*x[2]
        arg[2,0,0]=(-0.765160864305)*x[0]**o+(0.608017010487)*x[0]+(0.841265469171)*x[1]**o+(-0.409854579584)*x[1]+(0.728931324538)*x[2]**o+(0.121315209213)*x[2]
        arg[2,0,1]=(0.0397067301375)*x[0]**o+(-0.0451925117802)*x[0]+(-0.412724345414)*x[1]**o+(-0.447675002107)*x[1]+(0.141728105561)*x[2]**o+(0.923970189305)*x[2]
        arg[2,1,0]=(-0.237044096923)*x[0]**o+(-0.826168294669)*x[0]+(0.19833520121)*x[1]**o+(-0.796076984382)*x[1]+(0.367399642334)*x[2]**o+(0.71172848801)*x[2]
        arg[2,1,1]=(-0.3881830464)*x[0]**o+(0.988354722657)*x[0]+(0.762756805728)*x[1]**o+(0.551778683619)*x[1]+(0.975932105591)*x[2]**o+(-0.173192098292)*x[2]
        arg[3,0,0]=(-0.226163777371)*x[0]**o+(-0.301094659695)*x[0]+(0.0682567814743)*x[1]**o+(-0.154914902824)*x[1]+(0.984669238436)*x[2]**o+(-0.431011897738)*x[2]
        arg[3,0,1]=(-0.47230847571)*x[0]**o+(-0.579315893476)*x[0]+(-0.989289806396)*x[1]**o+(-0.673951334134)*x[1]+(0.98143767614)*x[2]**o+(-0.311074531383)*x[2]
        arg[3,1,0]=(-0.276309910109)*x[0]**o+(-0.70244304164)*x[0]+(-0.963669800079)*x[1]**o+(-0.947723474952)*x[1]+(0.513451083144)*x[2]**o+(-0.0651803692263)*x[2]
        arg[3,1,1]=(0.65175527234)*x[0]**o+(-0.294569414615)*x[0]+(-0.32846386125)*x[1]**o+(-0.868822318527)*x[1]+(0.268414480892)*x[2]**o+(-0.866835042217)*x[2]
        arg[4,0,0]=(0.304002845434)*x[0]**o+(0.938467246002)*x[0]+(-0.106648262772)*x[1]**o+(0.227863374571)*x[1]+(-0.789305204045)*x[2]**o+(-0.171526142133)*x[2]
        arg[4,0,1]=(0.70982305774)*x[0]**o+(0.404228697442)*x[0]+(0.190029230022)*x[1]**o+(-0.825276083634)*x[1]+(-0.739774636921)*x[2]**o+(-0.726529211281)*x[2]
        arg[4,1,0]=(-0.238487290076)*x[0]**o+(-0.4901234628)*x[0]+(-0.836670757945)*x[1]**o+(0.0841493712661)*x[1]+(-0.685144435198)*x[2]**o+(-0.304978244786)*x[2]
        arg[4,1,1]=(0.264176905985)*x[0]**o+(-0.410582158319)*x[0]+(-0.271915471538)*x[1]**o+(-0.996082427599)*x[1]+(-0.466654446945)*x[2]**o+(-0.723243781429)*x[2]
        arg[5,0,0]=(0.379390860481)*x[0]**o+(-0.78556086955)*x[0]+(0.189126836375)*x[1]**o+(0.235947881164)*x[1]+(0.149142076852)*x[2]**o+(-0.944419890441)*x[2]
        arg[5,0,1]=(0.768867280189)*x[0]**o+(-0.728364777249)*x[0]+(0.60414049451)*x[1]**o+(-0.538876171991)*x[1]+(-0.0947560131508)*x[2]**o+(0.474986163327)*x[2]
        arg[5,1,0]=(-0.228069986734)*x[0]**o+(-0.680841588894)*x[0]+(-0.663806510587)*x[1]**o+(-0.852775021416)*x[1]+(-0.59539582999)*x[2]**o+(0.52369159839)*x[2]
        arg[5,1,1]=(0.177872029425)*x[0]**o+(0.93423742256)*x[0]+(-0.0369475988192)*x[1]**o+(0.242202072343)*x[1]+(0.526144455402)*x[2]**o+(0.0217547142691)*x[2]
        ref[0,0,0]=(0.459775275267)/(o+1.)+(0.50964090443)+(0.292346424086)*0.5**o
        ref[0,0,1]=(0.276031265944)/(o+1.)+(-0.769575247383)+(-0.460556250835)*0.5**o
        ref[0,1,0]=(-1.63058537971)/(o+1.)+(-0.75507522439)+(0.380027894487)*0.5**o
        ref[0,1,1]=(-1.16816073751)/(o+1.)+(0.481522173937)+(0.948682597897)*0.5**o
        ref[1,0,0]=(-0.0367089680209)/(o+1.)+(-0.170235505982)+(0.357218603696)*0.5**o
        ref[1,0,1]=(0.419509004475)/(o+1.)+(-0.0265580193719)+(-0.776893233002)*0.5**o
        ref[1,1,0]=(0.0777247888533)/(o+1.)+(0.0569783304129)+(0.593512911569)*0.5**o
        ref[1,1,1]=(0.168459368149)/(o+1.)+(0.180291690463)+(-0.154542349616)*0.5**o
        ref[2,0,0]=(1.57019679371)/(o+1.)+(0.159738820058)+(-0.765160864305)*0.5**o
        ref[2,0,1]=(-0.270996239853)/(o+1.)+(0.215551337709)+(0.0397067301375)*0.5**o
        ref[2,1,0]=(0.565734843545)/(o+1.)+(-0.455258395521)+(-0.237044096923)*0.5**o
        ref[2,1,1]=(1.73868891132)/(o+1.)+(0.683470653992)+(-0.3881830464)*0.5**o
        ref[3,0,0]=(1.05292601991)/(o+1.)+(-0.443510730129)+(-0.226163777371)*0.5**o
        ref[3,0,1]=(-0.00785213025503)/(o+1.)+(-0.782170879496)+(-0.47230847571)*0.5**o
        ref[3,1,0]=(-0.450218716935)/(o+1.)+(-0.857673442909)+(-0.276309910109)*0.5**o
        ref[3,1,1]=(-0.0600493803575)/(o+1.)+(-1.01511338768)+(0.65175527234)*0.5**o
        ref[4,0,0]=(-0.895953466817)/(o+1.)+(0.49740223922)+(0.304002845434)*0.5**o
        ref[4,0,1]=(-0.549745406899)/(o+1.)+(-0.573788298736)+(0.70982305774)*0.5**o
        ref[4,1,0]=(-1.52181519314)/(o+1.)+(-0.35547616816)+(-0.238487290076)*0.5**o
        ref[4,1,1]=(-0.738569918483)/(o+1.)+(-1.06495418367)+(0.264176905985)*0.5**o
        ref[5,0,0]=(0.338268913228)/(o+1.)+(-0.747016439414)+(0.379390860481)*0.5**o
        ref[5,0,1]=(0.50938448136)/(o+1.)+(-0.396127392956)+(0.768867280189)*0.5**o
        ref[5,1,0]=(-1.25920234058)/(o+1.)+(-0.50496250596)+(-0.228069986734)*0.5**o
        ref[5,1,1]=(0.489196856583)/(o+1.)+(0.599097104586)+(0.177872029425)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactZero_fromData_ContinuousFunction_rank4(self):
      """
      tests integral of rank 4 Data on the FunctionOnContactZero

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports integral on FunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5, 3, 2),w)
      ref=numpy.zeros((4, 5, 3, 2),numpy.float_)
      if dim==2:
        arg[0,0,0,0]=(0.423100205741)*x[0]**o+(-0.681307394941)*x[0]+(-0.848175950825)*x[1]**o+(0.559338235783)*x[1]
        arg[0,0,0,1]=(-0.739687474039)*x[0]**o+(-0.202202358237)*x[0]+(-0.838397517739)*x[1]**o+(-0.377580621185)*x[1]
        arg[0,0,1,0]=(-0.423462958438)*x[0]**o+(-0.335929684088)*x[0]+(-0.616976579504)*x[1]**o+(-0.77803694581)*x[1]
        arg[0,0,1,1]=(-0.432533121428)*x[0]**o+(0.0723865130937)*x[0]+(-0.953014661461)*x[1]**o+(0.92761707252)*x[1]
        arg[0,0,2,0]=(-0.891613775753)*x[0]**o+(0.780592229933)*x[0]+(-0.156191993934)*x[1]**o+(0.985464289737)*x[1]
        arg[0,0,2,1]=(0.909573147888)*x[0]**o+(-0.225603851475)*x[0]+(-0.68287286477)*x[1]**o+(0.330105905794)*x[1]
        arg[0,1,0,0]=(-0.39808363424)*x[0]**o+(-0.290872560992)*x[0]+(-0.631624427826)*x[1]**o+(-0.389239289826)*x[1]
        arg[0,1,0,1]=(-0.748658247224)*x[0]**o+(0.22314629362)*x[0]+(-0.0750972377036)*x[1]**o+(0.908914722147)*x[1]
        arg[0,1,1,0]=(0.877627912148)*x[0]**o+(-0.988434656301)*x[0]+(-0.163646238293)*x[1]**o+(-0.169410711361)*x[1]
        arg[0,1,1,1]=(-0.592580581615)*x[0]**o+(0.997989912544)*x[0]+(0.0667564973731)*x[1]**o+(-0.121580259121)*x[1]
        arg[0,1,2,0]=(-0.505569292541)*x[0]**o+(-0.534428841308)*x[0]+(-0.899276958155)*x[1]**o+(-0.833972415825)*x[1]
        arg[0,1,2,1]=(-0.215752366701)*x[0]**o+(-0.249993262232)*x[0]+(0.912452712698)*x[1]**o+(0.0601524115958)*x[1]
        arg[0,2,0,0]=(0.684112319204)*x[0]**o+(0.343018234122)*x[0]+(0.267409202785)*x[1]**o+(0.7781133929)*x[1]
        arg[0,2,0,1]=(0.552438501301)*x[0]**o+(-0.101558113559)*x[0]+(-0.152472725029)*x[1]**o+(-0.8771654517)*x[1]
        arg[0,2,1,0]=(-0.31213422297)*x[0]**o+(0.597042319279)*x[0]+(-0.873134889407)*x[1]**o+(-0.68920786345)*x[1]
        arg[0,2,1,1]=(0.883048532408)*x[0]**o+(-0.653997880306)*x[0]+(-0.761310589424)*x[1]**o+(-0.698342623962)*x[1]
        arg[0,2,2,0]=(-0.324489237508)*x[0]**o+(-0.717724692716)*x[0]+(-0.936381308418)*x[1]**o+(0.712416971077)*x[1]
        arg[0,2,2,1]=(-0.971735193469)*x[0]**o+(0.0943629094751)*x[0]+(0.419811305063)*x[1]**o+(0.125623316569)*x[1]
        arg[0,3,0,0]=(-0.305553454905)*x[0]**o+(-0.936112994503)*x[0]+(0.707279059071)*x[1]**o+(-0.794754809337)*x[1]
        arg[0,3,0,1]=(0.803934620926)*x[0]**o+(-0.632020033455)*x[0]+(0.819264129099)*x[1]**o+(0.770744069672)*x[1]
        arg[0,3,1,0]=(0.996632326055)*x[0]**o+(-0.537850178404)*x[0]+(-0.60262522048)*x[1]**o+(-0.431667704232)*x[1]
        arg[0,3,1,1]=(0.402799911553)*x[0]**o+(0.155798565063)*x[0]+(0.722837842981)*x[1]**o+(0.462463978227)*x[1]
        arg[0,3,2,0]=(0.937966588808)*x[0]**o+(-0.797151899551)*x[0]+(0.62363567076)*x[1]**o+(-0.60458214868)*x[1]
        arg[0,3,2,1]=(-0.0389713032301)*x[0]**o+(0.497820326557)*x[0]+(-0.950660964237)*x[1]**o+(0.0247372614337)*x[1]
        arg[0,4,0,0]=(-0.589529640529)*x[0]**o+(-0.71761078076)*x[0]+(-0.872148906918)*x[1]**o+(-0.666618835503)*x[1]
        arg[0,4,0,1]=(0.78615039967)*x[0]**o+(0.99287672137)*x[0]+(0.329194160571)*x[1]**o+(0.309370990805)*x[1]
        arg[0,4,1,0]=(-0.801187683466)*x[0]**o+(0.476957978386)*x[0]+(0.107007555422)*x[1]**o+(-0.211909676249)*x[1]
        arg[0,4,1,1]=(0.817551079419)*x[0]**o+(0.958544826763)*x[0]+(-0.720479146591)*x[1]**o+(0.0769824586505)*x[1]
        arg[0,4,2,0]=(-0.245391293539)*x[0]**o+(0.240588876322)*x[0]+(0.766214937295)*x[1]**o+(0.625282150348)*x[1]
        arg[0,4,2,1]=(-0.780163545255)*x[0]**o+(-0.84424180556)*x[0]+(-0.993826355778)*x[1]**o+(-0.708432857746)*x[1]
        arg[1,0,0,0]=(0.0998104654851)*x[0]**o+(-0.933870759023)*x[0]+(-0.00667060999489)*x[1]**o+(-0.304328472241)*x[1]
        arg[1,0,0,1]=(0.716858223574)*x[0]**o+(0.0907083966301)*x[0]+(0.118004968786)*x[1]**o+(-0.989434356769)*x[1]
        arg[1,0,1,0]=(0.433505460082)*x[0]**o+(0.927792393402)*x[0]+(0.10117572965)*x[1]**o+(-0.719175235748)*x[1]
        arg[1,0,1,1]=(0.93394463477)*x[0]**o+(0.30554551436)*x[0]+(-0.221528653572)*x[1]**o+(-0.965153365935)*x[1]
        arg[1,0,2,0]=(0.217926136552)*x[0]**o+(0.75089182057)*x[0]+(0.798438119547)*x[1]**o+(-0.131885152617)*x[1]
        arg[1,0,2,1]=(0.489885370386)*x[0]**o+(-0.958785443386)*x[0]+(-0.934023631551)*x[1]**o+(-0.120727978484)*x[1]
        arg[1,1,0,0]=(-0.104659311537)*x[0]**o+(0.323817119133)*x[0]+(0.207178629335)*x[1]**o+(0.132677138305)*x[1]
        arg[1,1,0,1]=(0.855593354128)*x[0]**o+(-0.319099268106)*x[0]+(-0.75522659926)*x[1]**o+(0.398648791668)*x[1]
        arg[1,1,1,0]=(0.165499959353)*x[0]**o+(0.025416591231)*x[0]+(-0.454711374202)*x[1]**o+(-0.149643847094)*x[1]
        arg[1,1,1,1]=(0.668195333828)*x[0]**o+(-0.954315351102)*x[0]+(-0.15549992542)*x[1]**o+(0.960203842246)*x[1]
        arg[1,1,2,0]=(-0.0830862028828)*x[0]**o+(0.104413343709)*x[0]+(0.0666836881524)*x[1]**o+(-0.909918510114)*x[1]
        arg[1,1,2,1]=(-0.614061685098)*x[0]**o+(-0.696536572273)*x[0]+(0.674867602585)*x[1]**o+(0.742233774001)*x[1]
        arg[1,2,0,0]=(-0.271025278576)*x[0]**o+(0.194146585691)*x[0]+(0.369545867263)*x[1]**o+(-0.444209773003)*x[1]
        arg[1,2,0,1]=(-0.646570016511)*x[0]**o+(-0.1181935343)*x[0]+(-0.473468176784)*x[1]**o+(0.980639666617)*x[1]
        arg[1,2,1,0]=(-0.274558546949)*x[0]**o+(-0.994564294804)*x[0]+(-0.710677848448)*x[1]**o+(-0.471981673909)*x[1]
        arg[1,2,1,1]=(0.920326662139)*x[0]**o+(-0.503477706392)*x[0]+(-0.488075258363)*x[1]**o+(-0.390701293427)*x[1]
        arg[1,2,2,0]=(-0.734779609501)*x[0]**o+(-0.211522964885)*x[0]+(-0.209704158218)*x[1]**o+(0.602143419316)*x[1]
        arg[1,2,2,1]=(-0.71042927509)*x[0]**o+(-0.320801535296)*x[0]+(-0.307645214006)*x[1]**o+(0.902857741355)*x[1]
        arg[1,3,0,0]=(0.430044544461)*x[0]**o+(-0.321809854969)*x[0]+(-0.202705286059)*x[1]**o+(-0.136245903236)*x[1]
        arg[1,3,0,1]=(-0.0215232394148)*x[0]**o+(0.560515199075)*x[0]+(0.183422356152)*x[1]**o+(0.0500950744933)*x[1]
        arg[1,3,1,0]=(0.956072319541)*x[0]**o+(-0.910407504398)*x[0]+(0.710351406673)*x[1]**o+(-0.673149941902)*x[1]
        arg[1,3,1,1]=(0.512105745894)*x[0]**o+(0.617656019287)*x[0]+(0.178878380134)*x[1]**o+(-0.304972517759)*x[1]
        arg[1,3,2,0]=(0.505194196756)*x[0]**o+(-0.588278604691)*x[0]+(0.657312673212)*x[1]**o+(-0.198666173753)*x[1]
        arg[1,3,2,1]=(0.562199962955)*x[0]**o+(-0.855080919048)*x[0]+(0.798106457622)*x[1]**o+(-0.462368594285)*x[1]
        arg[1,4,0,0]=(-0.95103819151)*x[0]**o+(0.996179304819)*x[0]+(0.720809353558)*x[1]**o+(-0.357882371835)*x[1]
        arg[1,4,0,1]=(-0.337212448334)*x[0]**o+(-0.647939092937)*x[0]+(0.432152553594)*x[1]**o+(0.284393585377)*x[1]
        arg[1,4,1,0]=(0.286170290079)*x[0]**o+(0.813168170391)*x[0]+(-0.178814496497)*x[1]**o+(0.414687546663)*x[1]
        arg[1,4,1,1]=(0.00961378883254)*x[0]**o+(-0.946718643529)*x[0]+(0.828780485661)*x[1]**o+(0.54461217692)*x[1]
        arg[1,4,2,0]=(0.784403348421)*x[0]**o+(0.430078584466)*x[0]+(-0.542741131881)*x[1]**o+(0.0290828265624)*x[1]
        arg[1,4,2,1]=(-0.246343667616)*x[0]**o+(-0.27406067089)*x[0]+(-0.938704878102)*x[1]**o+(0.907921504301)*x[1]
        arg[2,0,0,0]=(-0.551056874357)*x[0]**o+(-0.697006980356)*x[0]+(-0.733447125487)*x[1]**o+(-0.241902075189)*x[1]
        arg[2,0,0,1]=(-0.75556701159)*x[0]**o+(0.155303331948)*x[0]+(0.480546881289)*x[1]**o+(-0.420954472057)*x[1]
        arg[2,0,1,0]=(0.12990139506)*x[0]**o+(-0.0990388974263)*x[0]+(-0.913089275462)*x[1]**o+(-0.766316334455)*x[1]
        arg[2,0,1,1]=(-0.96760564527)*x[0]**o+(0.94596820894)*x[0]+(-0.208507053422)*x[1]**o+(0.82642580152)*x[1]
        arg[2,0,2,0]=(-0.801660576339)*x[0]**o+(0.724091208253)*x[0]+(-0.416451086696)*x[1]**o+(-0.255959329575)*x[1]
        arg[2,0,2,1]=(0.380233654687)*x[0]**o+(-0.27467061721)*x[0]+(-0.626861863133)*x[1]**o+(0.498725464891)*x[1]
        arg[2,1,0,0]=(0.733227197081)*x[0]**o+(-0.111622822078)*x[0]+(-0.199363221943)*x[1]**o+(-0.449664634046)*x[1]
        arg[2,1,0,1]=(0.381774304841)*x[0]**o+(0.914208421879)*x[0]+(-0.545958908798)*x[1]**o+(-0.344721917598)*x[1]
        arg[2,1,1,0]=(-0.861412661903)*x[0]**o+(-0.202264578976)*x[0]+(-0.225404313047)*x[1]**o+(0.716189374265)*x[1]
        arg[2,1,1,1]=(-0.180088205315)*x[0]**o+(0.133136657032)*x[0]+(-0.341650807958)*x[1]**o+(-0.570348715256)*x[1]
        arg[2,1,2,0]=(-0.626863150737)*x[0]**o+(-0.236854925766)*x[0]+(-0.271186155613)*x[1]**o+(-0.750500826476)*x[1]
        arg[2,1,2,1]=(-0.138580929054)*x[0]**o+(-0.925057205334)*x[0]+(0.404961686669)*x[1]**o+(0.542298342705)*x[1]
        arg[2,2,0,0]=(0.453186536783)*x[0]**o+(0.368065177204)*x[0]+(0.545211472015)*x[1]**o+(0.334551805933)*x[1]
        arg[2,2,0,1]=(0.0639699038803)*x[0]**o+(0.154192875522)*x[0]+(0.101347900441)*x[1]**o+(-0.668313576443)*x[1]
        arg[2,2,1,0]=(0.465173445732)*x[0]**o+(-0.475009875122)*x[0]+(-0.748444748361)*x[1]**o+(0.620845514745)*x[1]
        arg[2,2,1,1]=(0.720975965615)*x[0]**o+(-0.266631032721)*x[0]+(0.0996389916136)*x[1]**o+(-0.96886977517)*x[1]
        arg[2,2,2,0]=(-0.396391719953)*x[0]**o+(-0.240091626001)*x[0]+(-0.42411685734)*x[1]**o+(-0.458446010754)*x[1]
        arg[2,2,2,1]=(-0.662584035264)*x[0]**o+(0.23813614536)*x[0]+(0.810893160816)*x[1]**o+(-0.130143663026)*x[1]
        arg[2,3,0,0]=(-0.651550480594)*x[0]**o+(0.370844612999)*x[0]+(-0.808002027275)*x[1]**o+(0.531083477622)*x[1]
        arg[2,3,0,1]=(-0.287184182382)*x[0]**o+(0.928877646221)*x[0]+(-0.271563784371)*x[1]**o+(-0.46908309398)*x[1]
        arg[2,3,1,0]=(0.342080138259)*x[0]**o+(0.458994887529)*x[0]+(-0.544475387926)*x[1]**o+(-0.203224249914)*x[1]
        arg[2,3,1,1]=(-0.0791009979206)*x[0]**o+(0.536480501773)*x[0]+(-0.624008015173)*x[1]**o+(-0.380504135174)*x[1]
        arg[2,3,2,0]=(-0.707089360149)*x[0]**o+(0.217716900812)*x[0]+(0.020441009853)*x[1]**o+(0.680214150153)*x[1]
        arg[2,3,2,1]=(0.0368635610611)*x[0]**o+(0.646697307691)*x[0]+(0.36118713723)*x[1]**o+(-0.535295583008)*x[1]
        arg[2,4,0,0]=(0.806938680905)*x[0]**o+(0.449096825134)*x[0]+(-0.43507826254)*x[1]**o+(-0.300956286252)*x[1]
        arg[2,4,0,1]=(-0.268005914326)*x[0]**o+(0.583132947291)*x[0]+(-0.344067495236)*x[1]**o+(-0.526011968282)*x[1]
        arg[2,4,1,0]=(-0.0953663458348)*x[0]**o+(0.834765967023)*x[0]+(-0.0410345097335)*x[1]**o+(0.493198054713)*x[1]
        arg[2,4,1,1]=(0.964777812988)*x[0]**o+(-0.791397009733)*x[0]+(-0.962108127047)*x[1]**o+(0.378083818799)*x[1]
        arg[2,4,2,0]=(0.0977715857241)*x[0]**o+(-0.118265910205)*x[0]+(-0.0181736456148)*x[1]**o+(-0.480499109439)*x[1]
        arg[2,4,2,1]=(0.396936262802)*x[0]**o+(0.979807961416)*x[0]+(0.383251286227)*x[1]**o+(-0.921599329372)*x[1]
        arg[3,0,0,0]=(-0.193869568904)*x[0]**o+(0.335533150532)*x[0]+(-0.542087879921)*x[1]**o+(0.916531699685)*x[1]
        arg[3,0,0,1]=(0.732192438113)*x[0]**o+(0.610866210623)*x[0]+(0.232455584069)*x[1]**o+(0.109577342902)*x[1]
        arg[3,0,1,0]=(0.0487640958479)*x[0]**o+(-0.250229114846)*x[0]+(-0.539343546695)*x[1]**o+(-0.456485153733)*x[1]
        arg[3,0,1,1]=(0.951844805407)*x[0]**o+(-0.811442383281)*x[0]+(-0.537581471096)*x[1]**o+(0.350217063609)*x[1]
        arg[3,0,2,0]=(0.495490307938)*x[0]**o+(0.529667417659)*x[0]+(-0.853447043865)*x[1]**o+(0.97002519322)*x[1]
        arg[3,0,2,1]=(0.38188303469)*x[0]**o+(-0.75911672537)*x[0]+(-0.67563531103)*x[1]**o+(0.384002769031)*x[1]
        arg[3,1,0,0]=(0.44471228922)*x[0]**o+(-0.958995287488)*x[0]+(-0.356730659314)*x[1]**o+(0.853749081247)*x[1]
        arg[3,1,0,1]=(-0.674677862576)*x[0]**o+(-0.651945231545)*x[0]+(-0.869871613483)*x[1]**o+(0.518493292422)*x[1]
        arg[3,1,1,0]=(-0.803923033456)*x[0]**o+(0.916638584514)*x[0]+(-0.410056071233)*x[1]**o+(-0.313811798732)*x[1]
        arg[3,1,1,1]=(-0.677586574312)*x[0]**o+(0.101317771211)*x[0]+(-0.848629733526)*x[1]**o+(0.956935407827)*x[1]
        arg[3,1,2,0]=(0.723956969878)*x[0]**o+(-0.677929268987)*x[0]+(0.168694427833)*x[1]**o+(0.675664564011)*x[1]
        arg[3,1,2,1]=(-0.0961257584415)*x[0]**o+(0.870273004118)*x[0]+(0.0979731186867)*x[1]**o+(-0.561786099655)*x[1]
        arg[3,2,0,0]=(-0.506857925404)*x[0]**o+(0.950429186893)*x[0]+(-0.765686972231)*x[1]**o+(0.300787357163)*x[1]
        arg[3,2,0,1]=(-0.479464705052)*x[0]**o+(-0.814881390688)*x[0]+(-0.549033177495)*x[1]**o+(0.875164713771)*x[1]
        arg[3,2,1,0]=(0.984579775952)*x[0]**o+(0.822464384935)*x[0]+(-0.352937841183)*x[1]**o+(-0.389516730371)*x[1]
        arg[3,2,1,1]=(-0.75230207503)*x[0]**o+(0.921759892886)*x[0]+(0.683120790039)*x[1]**o+(0.470250085193)*x[1]
        arg[3,2,2,0]=(-0.257935272452)*x[0]**o+(-0.437370511818)*x[0]+(-0.350363036315)*x[1]**o+(-0.232415372202)*x[1]
        arg[3,2,2,1]=(0.0784702766009)*x[0]**o+(0.460959683819)*x[0]+(0.758201028024)*x[1]**o+(0.101597092257)*x[1]
        arg[3,3,0,0]=(0.292597181224)*x[0]**o+(-0.322992033991)*x[0]+(-0.536413055541)*x[1]**o+(-0.95609054732)*x[1]
        arg[3,3,0,1]=(0.174134715952)*x[0]**o+(0.811729326606)*x[0]+(0.89040777319)*x[1]**o+(0.893572419966)*x[1]
        arg[3,3,1,0]=(0.768234215847)*x[0]**o+(0.36391719033)*x[0]+(0.598889528719)*x[1]**o+(0.152035960251)*x[1]
        arg[3,3,1,1]=(-0.77938361663)*x[0]**o+(-0.448945619013)*x[0]+(0.471242869055)*x[1]**o+(0.112888214554)*x[1]
        arg[3,3,2,0]=(-0.00617753872016)*x[0]**o+(0.0155218100192)*x[0]+(0.674100983537)*x[1]**o+(0.211406831421)*x[1]
        arg[3,3,2,1]=(0.124470359441)*x[0]**o+(0.117393617733)*x[0]+(-0.297791856337)*x[1]**o+(-0.210405798241)*x[1]
        arg[3,4,0,0]=(0.139920132608)*x[0]**o+(-0.0073381380034)*x[0]+(-0.77267392403)*x[1]**o+(0.792299687107)*x[1]
        arg[3,4,0,1]=(0.328592839797)*x[0]**o+(-0.758899359217)*x[0]+(0.730856047416)*x[1]**o+(-0.389548922357)*x[1]
        arg[3,4,1,0]=(-0.239362896774)*x[0]**o+(0.513205289336)*x[0]+(-0.393781550386)*x[1]**o+(0.229968513732)*x[1]
        arg[3,4,1,1]=(0.777985150139)*x[0]**o+(0.273278793895)*x[0]+(0.17782134987)*x[1]**o+(0.491127331923)*x[1]
        arg[3,4,2,0]=(0.348482781008)*x[0]**o+(0.458696185739)*x[0]+(-0.77882422986)*x[1]**o+(0.455739446004)*x[1]
        arg[3,4,2,1]=(-0.396751338134)*x[0]**o+(0.449026376364)*x[0]+(0.308553967581)*x[1]**o+(-0.777653415416)*x[1]
        ref[0,0,0,0]=(-0.848175950825)/(o+1.)+(-0.060984579579)+(0.423100205741)*0.5**o
        ref[0,0,0,1]=(-0.838397517739)/(o+1.)+(-0.289891489711)+(-0.739687474039)*0.5**o
        ref[0,0,1,0]=(-0.616976579504)/(o+1.)+(-0.556983314949)+(-0.423462958438)*0.5**o
        ref[0,0,1,1]=(-0.953014661461)/(o+1.)+(0.500001792807)+(-0.432533121428)*0.5**o
        ref[0,0,2,0]=(-0.156191993934)/(o+1.)+(0.883028259835)+(-0.891613775753)*0.5**o
        ref[0,0,2,1]=(-0.68287286477)/(o+1.)+(0.0522510271596)+(0.909573147888)*0.5**o
        ref[0,1,0,0]=(-0.631624427826)/(o+1.)+(-0.340055925409)+(-0.39808363424)*0.5**o
        ref[0,1,0,1]=(-0.0750972377036)/(o+1.)+(0.566030507884)+(-0.748658247224)*0.5**o
        ref[0,1,1,0]=(-0.163646238293)/(o+1.)+(-0.578922683831)+(0.877627912148)*0.5**o
        ref[0,1,1,1]=(0.0667564973731)/(o+1.)+(0.438204826712)+(-0.592580581615)*0.5**o
        ref[0,1,2,0]=(-0.899276958155)/(o+1.)+(-0.684200628566)+(-0.505569292541)*0.5**o
        ref[0,1,2,1]=(0.912452712698)/(o+1.)+(-0.0949204253183)+(-0.215752366701)*0.5**o
        ref[0,2,0,0]=(0.267409202785)/(o+1.)+(0.560565813511)+(0.684112319204)*0.5**o
        ref[0,2,0,1]=(-0.152472725029)/(o+1.)+(-0.48936178263)+(0.552438501301)*0.5**o
        ref[0,2,1,0]=(-0.873134889407)/(o+1.)+(-0.0460827720855)+(-0.31213422297)*0.5**o
        ref[0,2,1,1]=(-0.761310589424)/(o+1.)+(-0.676170252134)+(0.883048532408)*0.5**o
        ref[0,2,2,0]=(-0.936381308418)/(o+1.)+(-0.00265386081956)+(-0.324489237508)*0.5**o
        ref[0,2,2,1]=(0.419811305063)/(o+1.)+(0.109993113022)+(-0.971735193469)*0.5**o
        ref[0,3,0,0]=(0.707279059071)/(o+1.)+(-0.86543390192)+(-0.305553454905)*0.5**o
        ref[0,3,0,1]=(0.819264129099)/(o+1.)+(0.0693620181083)+(0.803934620926)*0.5**o
        ref[0,3,1,0]=(-0.60262522048)/(o+1.)+(-0.484758941318)+(0.996632326055)*0.5**o
        ref[0,3,1,1]=(0.722837842981)/(o+1.)+(0.309131271645)+(0.402799911553)*0.5**o
        ref[0,3,2,0]=(0.62363567076)/(o+1.)+(-0.700867024115)+(0.937966588808)*0.5**o
        ref[0,3,2,1]=(-0.950660964237)/(o+1.)+(0.261278793995)+(-0.0389713032301)*0.5**o
        ref[0,4,0,0]=(-0.872148906918)/(o+1.)+(-0.692114808131)+(-0.589529640529)*0.5**o
        ref[0,4,0,1]=(0.329194160571)/(o+1.)+(0.651123856087)+(0.78615039967)*0.5**o
        ref[0,4,1,0]=(0.107007555422)/(o+1.)+(0.132524151069)+(-0.801187683466)*0.5**o
        ref[0,4,1,1]=(-0.720479146591)/(o+1.)+(0.517763642707)+(0.817551079419)*0.5**o
        ref[0,4,2,0]=(0.766214937295)/(o+1.)+(0.432935513335)+(-0.245391293539)*0.5**o
        ref[0,4,2,1]=(-0.993826355778)/(o+1.)+(-0.776337331653)+(-0.780163545255)*0.5**o
        ref[1,0,0,0]=(-0.00667060999489)/(o+1.)+(-0.619099615632)+(0.0998104654851)*0.5**o
        ref[1,0,0,1]=(0.118004968786)/(o+1.)+(-0.449362980069)+(0.716858223574)*0.5**o
        ref[1,0,1,0]=(0.10117572965)/(o+1.)+(0.104308578827)+(0.433505460082)*0.5**o
        ref[1,0,1,1]=(-0.221528653572)/(o+1.)+(-0.329803925788)+(0.93394463477)*0.5**o
        ref[1,0,2,0]=(0.798438119547)/(o+1.)+(0.309503333977)+(0.217926136552)*0.5**o
        ref[1,0,2,1]=(-0.934023631551)/(o+1.)+(-0.539756710935)+(0.489885370386)*0.5**o
        ref[1,1,0,0]=(0.207178629335)/(o+1.)+(0.228247128719)+(-0.104659311537)*0.5**o
        ref[1,1,0,1]=(-0.75522659926)/(o+1.)+(0.0397747617809)+(0.855593354128)*0.5**o
        ref[1,1,1,0]=(-0.454711374202)/(o+1.)+(-0.0621136279316)+(0.165499959353)*0.5**o
        ref[1,1,1,1]=(-0.15549992542)/(o+1.)+(0.00294424557217)+(0.668195333828)*0.5**o
        ref[1,1,2,0]=(0.0666836881524)/(o+1.)+(-0.402752583203)+(-0.0830862028828)*0.5**o
        ref[1,1,2,1]=(0.674867602585)/(o+1.)+(0.0228486008642)+(-0.614061685098)*0.5**o
        ref[1,2,0,0]=(0.369545867263)/(o+1.)+(-0.125031593656)+(-0.271025278576)*0.5**o
        ref[1,2,0,1]=(-0.473468176784)/(o+1.)+(0.431223066159)+(-0.646570016511)*0.5**o
        ref[1,2,1,0]=(-0.710677848448)/(o+1.)+(-0.733272984356)+(-0.274558546949)*0.5**o
        ref[1,2,1,1]=(-0.488075258363)/(o+1.)+(-0.44708949991)+(0.920326662139)*0.5**o
        ref[1,2,2,0]=(-0.209704158218)/(o+1.)+(0.195310227215)+(-0.734779609501)*0.5**o
        ref[1,2,2,1]=(-0.307645214006)/(o+1.)+(0.29102810303)+(-0.71042927509)*0.5**o
        ref[1,3,0,0]=(-0.202705286059)/(o+1.)+(-0.229027879102)+(0.430044544461)*0.5**o
        ref[1,3,0,1]=(0.183422356152)/(o+1.)+(0.305305136784)+(-0.0215232394148)*0.5**o
        ref[1,3,1,0]=(0.710351406673)/(o+1.)+(-0.79177872315)+(0.956072319541)*0.5**o
        ref[1,3,1,1]=(0.178878380134)/(o+1.)+(0.156341750764)+(0.512105745894)*0.5**o
        ref[1,3,2,0]=(0.657312673212)/(o+1.)+(-0.393472389222)+(0.505194196756)*0.5**o
        ref[1,3,2,1]=(0.798106457622)/(o+1.)+(-0.658724756667)+(0.562199962955)*0.5**o
        ref[1,4,0,0]=(0.720809353558)/(o+1.)+(0.319148466492)+(-0.95103819151)*0.5**o
        ref[1,4,0,1]=(0.432152553594)/(o+1.)+(-0.18177275378)+(-0.337212448334)*0.5**o
        ref[1,4,1,0]=(-0.178814496497)/(o+1.)+(0.613927858527)+(0.286170290079)*0.5**o
        ref[1,4,1,1]=(0.828780485661)/(o+1.)+(-0.201053233304)+(0.00961378883254)*0.5**o
        ref[1,4,2,0]=(-0.542741131881)/(o+1.)+(0.229580705514)+(0.784403348421)*0.5**o
        ref[1,4,2,1]=(-0.938704878102)/(o+1.)+(0.316930416706)+(-0.246343667616)*0.5**o
        ref[2,0,0,0]=(-0.733447125487)/(o+1.)+(-0.469454527772)+(-0.551056874357)*0.5**o
        ref[2,0,0,1]=(0.480546881289)/(o+1.)+(-0.132825570055)+(-0.75556701159)*0.5**o
        ref[2,0,1,0]=(-0.913089275462)/(o+1.)+(-0.432677615941)+(0.12990139506)*0.5**o
        ref[2,0,1,1]=(-0.208507053422)/(o+1.)+(0.88619700523)+(-0.96760564527)*0.5**o
        ref[2,0,2,0]=(-0.416451086696)/(o+1.)+(0.234065939339)+(-0.801660576339)*0.5**o
        ref[2,0,2,1]=(-0.626861863133)/(o+1.)+(0.112027423841)+(0.380233654687)*0.5**o
        ref[2,1,0,0]=(-0.199363221943)/(o+1.)+(-0.280643728062)+(0.733227197081)*0.5**o
        ref[2,1,0,1]=(-0.545958908798)/(o+1.)+(0.284743252141)+(0.381774304841)*0.5**o
        ref[2,1,1,0]=(-0.225404313047)/(o+1.)+(0.256962397644)+(-0.861412661903)*0.5**o
        ref[2,1,1,1]=(-0.341650807958)/(o+1.)+(-0.218606029112)+(-0.180088205315)*0.5**o
        ref[2,1,2,0]=(-0.271186155613)/(o+1.)+(-0.493677876121)+(-0.626863150737)*0.5**o
        ref[2,1,2,1]=(0.404961686669)/(o+1.)+(-0.191379431315)+(-0.138580929054)*0.5**o
        ref[2,2,0,0]=(0.545211472015)/(o+1.)+(0.351308491569)+(0.453186536783)*0.5**o
        ref[2,2,0,1]=(0.101347900441)/(o+1.)+(-0.257060350461)+(0.0639699038803)*0.5**o
        ref[2,2,1,0]=(-0.748444748361)/(o+1.)+(0.0729178198114)+(0.465173445732)*0.5**o
        ref[2,2,1,1]=(0.0996389916136)/(o+1.)+(-0.617750403946)+(0.720975965615)*0.5**o
        ref[2,2,2,0]=(-0.42411685734)/(o+1.)+(-0.349268818378)+(-0.396391719953)*0.5**o
        ref[2,2,2,1]=(0.810893160816)/(o+1.)+(0.053996241167)+(-0.662584035264)*0.5**o
        ref[2,3,0,0]=(-0.808002027275)/(o+1.)+(0.45096404531)+(-0.651550480594)*0.5**o
        ref[2,3,0,1]=(-0.271563784371)/(o+1.)+(0.22989727612)+(-0.287184182382)*0.5**o
        ref[2,3,1,0]=(-0.544475387926)/(o+1.)+(0.127885318807)+(0.342080138259)*0.5**o
        ref[2,3,1,1]=(-0.624008015173)/(o+1.)+(0.0779881832997)+(-0.0791009979206)*0.5**o
        ref[2,3,2,0]=(0.020441009853)/(o+1.)+(0.448965525483)+(-0.707089360149)*0.5**o
        ref[2,3,2,1]=(0.36118713723)/(o+1.)+(0.0557008623413)+(0.0368635610611)*0.5**o
        ref[2,4,0,0]=(-0.43507826254)/(o+1.)+(0.0740702694408)+(0.806938680905)*0.5**o
        ref[2,4,0,1]=(-0.344067495236)/(o+1.)+(0.0285604895043)+(-0.268005914326)*0.5**o
        ref[2,4,1,0]=(-0.0410345097335)/(o+1.)+(0.663982010868)+(-0.0953663458348)*0.5**o
        ref[2,4,1,1]=(-0.962108127047)/(o+1.)+(-0.206656595467)+(0.964777812988)*0.5**o
        ref[2,4,2,0]=(-0.0181736456148)/(o+1.)+(-0.299382509822)+(0.0977715857241)*0.5**o
        ref[2,4,2,1]=(0.383251286227)/(o+1.)+(0.0291043160221)+(0.396936262802)*0.5**o
        ref[3,0,0,0]=(-0.542087879921)/(o+1.)+(0.626032425109)+(-0.193869568904)*0.5**o
        ref[3,0,0,1]=(0.232455584069)/(o+1.)+(0.360221776762)+(0.732192438113)*0.5**o
        ref[3,0,1,0]=(-0.539343546695)/(o+1.)+(-0.353357134289)+(0.0487640958479)*0.5**o
        ref[3,0,1,1]=(-0.537581471096)/(o+1.)+(-0.230612659836)+(0.951844805407)*0.5**o
        ref[3,0,2,0]=(-0.853447043865)/(o+1.)+(0.74984630544)+(0.495490307938)*0.5**o
        ref[3,0,2,1]=(-0.67563531103)/(o+1.)+(-0.187556978169)+(0.38188303469)*0.5**o
        ref[3,1,0,0]=(-0.356730659314)/(o+1.)+(-0.0526231031206)+(0.44471228922)*0.5**o
        ref[3,1,0,1]=(-0.869871613483)/(o+1.)+(-0.0667259695618)+(-0.674677862576)*0.5**o
        ref[3,1,1,0]=(-0.410056071233)/(o+1.)+(0.301413392891)+(-0.803923033456)*0.5**o
        ref[3,1,1,1]=(-0.848629733526)/(o+1.)+(0.529126589519)+(-0.677586574312)*0.5**o
        ref[3,1,2,0]=(0.168694427833)/(o+1.)+(-0.00113235248793)+(0.723956969878)*0.5**o
        ref[3,1,2,1]=(0.0979731186867)/(o+1.)+(0.154243452232)+(-0.0961257584415)*0.5**o
        ref[3,2,0,0]=(-0.765686972231)/(o+1.)+(0.625608272028)+(-0.506857925404)*0.5**o
        ref[3,2,0,1]=(-0.549033177495)/(o+1.)+(0.0301416615418)+(-0.479464705052)*0.5**o
        ref[3,2,1,0]=(-0.352937841183)/(o+1.)+(0.216473827282)+(0.984579775952)*0.5**o
        ref[3,2,1,1]=(0.683120790039)/(o+1.)+(0.696004989039)+(-0.75230207503)*0.5**o
        ref[3,2,2,0]=(-0.350363036315)/(o+1.)+(-0.33489294201)+(-0.257935272452)*0.5**o
        ref[3,2,2,1]=(0.758201028024)/(o+1.)+(0.281278388038)+(0.0784702766009)*0.5**o
        ref[3,3,0,0]=(-0.536413055541)/(o+1.)+(-0.639541290655)+(0.292597181224)*0.5**o
        ref[3,3,0,1]=(0.89040777319)/(o+1.)+(0.852650873286)+(0.174134715952)*0.5**o
        ref[3,3,1,0]=(0.598889528719)/(o+1.)+(0.257976575291)+(0.768234215847)*0.5**o
        ref[3,3,1,1]=(0.471242869055)/(o+1.)+(-0.168028702229)+(-0.77938361663)*0.5**o
        ref[3,3,2,0]=(0.674100983537)/(o+1.)+(0.11346432072)+(-0.00617753872016)*0.5**o
        ref[3,3,2,1]=(-0.297791856337)/(o+1.)+(-0.0465060902539)+(0.124470359441)*0.5**o
        ref[3,4,0,0]=(-0.77267392403)/(o+1.)+(0.392480774552)+(0.139920132608)*0.5**o
        ref[3,4,0,1]=(0.730856047416)/(o+1.)+(-0.574224140787)+(0.328592839797)*0.5**o
        ref[3,4,1,0]=(-0.393781550386)/(o+1.)+(0.371586901534)+(-0.239362896774)*0.5**o
        ref[3,4,1,1]=(0.17782134987)/(o+1.)+(0.382203062909)+(0.777985150139)*0.5**o
        ref[3,4,2,0]=(-0.77882422986)/(o+1.)+(0.457217815872)+(0.348482781008)*0.5**o
        ref[3,4,2,1]=(0.308553967581)/(o+1.)+(-0.164313519526)+(-0.396751338134)*0.5**o
      else:
        arg[0,0,0,0]=(0.663998508168)*x[0]**o+(0.797340164931)*x[0]+(0.285182001599)*x[1]**o+(0.99720606542)*x[1]+(0.127610260108)*x[2]**o+(0.0141819362674)*x[2]
        arg[0,0,0,1]=(-0.010130464476)*x[0]**o+(-0.694140348803)*x[0]+(-0.554528445238)*x[1]**o+(0.192639775817)*x[1]+(-0.520164891934)*x[2]**o+(0.316830081989)*x[2]
        arg[0,0,1,0]=(-0.176542289385)*x[0]**o+(-0.312451025451)*x[0]+(0.33951668081)*x[1]**o+(-0.66674551066)*x[1]+(-0.0515929546862)*x[2]**o+(0.688180995863)*x[2]
        arg[0,0,1,1]=(-0.98152572229)*x[0]**o+(0.156411605196)*x[0]+(0.588065240125)*x[1]**o+(0.665830732172)*x[1]+(-0.834625744106)*x[2]**o+(-0.138276534671)*x[2]
        arg[0,0,2,0]=(0.555897422173)*x[0]**o+(0.0269431977874)*x[0]+(0.671161746561)*x[1]**o+(0.0857445285374)*x[1]+(0.578027911522)*x[2]**o+(0.217805007393)*x[2]
        arg[0,0,2,1]=(0.23127040923)*x[0]**o+(0.624353504617)*x[0]+(-0.433915231286)*x[1]**o+(-0.860173971996)*x[1]+(-0.801472434932)*x[2]**o+(-0.779575445432)*x[2]
        arg[0,1,0,0]=(0.0202445177842)*x[0]**o+(-0.0654289514039)*x[0]+(-0.877581892208)*x[1]**o+(-0.231833216639)*x[1]+(0.609752401432)*x[2]**o+(-0.802673983603)*x[2]
        arg[0,1,0,1]=(0.99935098551)*x[0]**o+(0.665639246142)*x[0]+(0.593510187831)*x[1]**o+(0.213336090684)*x[1]+(0.232667772738)*x[2]**o+(-0.651386262633)*x[2]
        arg[0,1,1,0]=(0.151799117419)*x[0]**o+(-0.34631043288)*x[0]+(-0.264898731671)*x[1]**o+(0.697527693137)*x[1]+(0.389030412719)*x[2]**o+(0.613338381577)*x[2]
        arg[0,1,1,1]=(-0.51422532626)*x[0]**o+(0.228040429268)*x[0]+(-0.276760984751)*x[1]**o+(0.379484010198)*x[1]+(0.896837565192)*x[2]**o+(0.195866552401)*x[2]
        arg[0,1,2,0]=(0.908244888839)*x[0]**o+(0.217098236162)*x[0]+(0.798725721675)*x[1]**o+(0.216635630826)*x[1]+(0.960227216319)*x[2]**o+(-0.885381182879)*x[2]
        arg[0,1,2,1]=(-0.872307766634)*x[0]**o+(0.843197737136)*x[0]+(-0.895220198791)*x[1]**o+(0.866433237518)*x[1]+(0.560344834226)*x[2]**o+(0.895797915384)*x[2]
        arg[0,2,0,0]=(0.00173619592035)*x[0]**o+(-0.209651858019)*x[0]+(0.323249367733)*x[1]**o+(-0.848445788048)*x[1]+(-0.747443572813)*x[2]**o+(0.342929929066)*x[2]
        arg[0,2,0,1]=(0.998184336552)*x[0]**o+(-0.669882578313)*x[0]+(-0.632794054696)*x[1]**o+(-0.779832061924)*x[1]+(0.0588815237456)*x[2]**o+(0.419538634163)*x[2]
        arg[0,2,1,0]=(-0.921234331968)*x[0]**o+(0.319724478943)*x[0]+(0.867192523265)*x[1]**o+(-0.183047755109)*x[1]+(-0.753047661507)*x[2]**o+(-0.797179040607)*x[2]
        arg[0,2,1,1]=(0.153578015941)*x[0]**o+(0.887820807812)*x[0]+(-0.607944647299)*x[1]**o+(-0.0487413965527)*x[1]+(-0.812369692315)*x[2]**o+(0.323102876844)*x[2]
        arg[0,2,2,0]=(0.29601674794)*x[0]**o+(-0.108738759628)*x[0]+(0.242837345895)*x[1]**o+(-0.324487554664)*x[1]+(0.584399369463)*x[2]**o+(0.683877139789)*x[2]
        arg[0,2,2,1]=(0.0580007647099)*x[0]**o+(0.214488016057)*x[0]+(0.669087265419)*x[1]**o+(-0.311999860513)*x[1]+(0.678940598195)*x[2]**o+(0.658847203441)*x[2]
        arg[0,3,0,0]=(-0.337999890505)*x[0]**o+(-0.842479530409)*x[0]+(0.786376708623)*x[1]**o+(0.597298234359)*x[1]+(0.495442668997)*x[2]**o+(-0.183828960943)*x[2]
        arg[0,3,0,1]=(-0.825813329629)*x[0]**o+(0.197040670392)*x[0]+(0.997784261354)*x[1]**o+(-0.355290279278)*x[1]+(0.657909516664)*x[2]**o+(-0.568281670001)*x[2]
        arg[0,3,1,0]=(0.699313512079)*x[0]**o+(-0.916350147519)*x[0]+(-0.311605808179)*x[1]**o+(0.222618888987)*x[1]+(0.872150798976)*x[2]**o+(0.139773150125)*x[2]
        arg[0,3,1,1]=(0.140764124286)*x[0]**o+(0.994013777309)*x[0]+(-0.0792107151558)*x[1]**o+(-0.285357856751)*x[1]+(-0.800285107223)*x[2]**o+(0.413029089978)*x[2]
        arg[0,3,2,0]=(-0.68177752688)*x[0]**o+(0.0339522871939)*x[0]+(-0.641399841911)*x[1]**o+(0.263279621719)*x[1]+(-0.406115766497)*x[2]**o+(-0.774639092638)*x[2]
        arg[0,3,2,1]=(0.655685968811)*x[0]**o+(0.20085918281)*x[0]+(-0.753491228829)*x[1]**o+(0.393669658887)*x[1]+(0.339469131428)*x[2]**o+(0.424803724709)*x[2]
        arg[0,4,0,0]=(-0.18027010068)*x[0]**o+(-0.773007683353)*x[0]+(0.767133020895)*x[1]**o+(-0.810449138606)*x[1]+(0.295421955604)*x[2]**o+(-0.955926880008)*x[2]
        arg[0,4,0,1]=(-0.914768182433)*x[0]**o+(-0.67024319498)*x[0]+(0.533244666277)*x[1]**o+(-0.705719628704)*x[1]+(-0.963633923897)*x[2]**o+(-0.0818895784023)*x[2]
        arg[0,4,1,0]=(0.274727423835)*x[0]**o+(0.838218676826)*x[0]+(0.269989900726)*x[1]**o+(0.650600179293)*x[1]+(-0.855210211935)*x[2]**o+(-0.800261683904)*x[2]
        arg[0,4,1,1]=(0.257574026756)*x[0]**o+(-0.787643878696)*x[0]+(-0.619136916008)*x[1]**o+(-0.242580269843)*x[1]+(0.0229959873555)*x[2]**o+(-0.685725009107)*x[2]
        arg[0,4,2,0]=(0.703185665611)*x[0]**o+(0.37686548395)*x[0]+(-0.414901304236)*x[1]**o+(0.310335568741)*x[1]+(-0.740764649572)*x[2]**o+(-0.389356987349)*x[2]
        arg[0,4,2,1]=(-0.153792811673)*x[0]**o+(-0.179777301012)*x[0]+(0.694402537357)*x[1]**o+(-0.928918851291)*x[1]+(-0.439307255129)*x[2]**o+(0.148671231214)*x[2]
        arg[1,0,0,0]=(0.298044833507)*x[0]**o+(0.538000859279)*x[0]+(-0.405432041063)*x[1]**o+(-0.8912548734)*x[1]+(0.929554154255)*x[2]**o+(-0.515905504512)*x[2]
        arg[1,0,0,1]=(-0.575485929806)*x[0]**o+(0.757951009672)*x[0]+(-0.815024122535)*x[1]**o+(0.846519632117)*x[1]+(0.594901198403)*x[2]**o+(-0.672034020351)*x[2]
        arg[1,0,1,0]=(0.998358005707)*x[0]**o+(-0.144729504804)*x[0]+(0.463445062889)*x[1]**o+(0.428918148475)*x[1]+(-0.172684103233)*x[2]**o+(-0.394742387769)*x[2]
        arg[1,0,1,1]=(0.783593367143)*x[0]**o+(-0.269882783638)*x[0]+(-0.880336283806)*x[1]**o+(0.337087191943)*x[1]+(-0.819512478233)*x[2]**o+(0.582924902797)*x[2]
        arg[1,0,2,0]=(-0.42345692756)*x[0]**o+(-0.251910598711)*x[0]+(-0.882193473121)*x[1]**o+(0.535405977525)*x[1]+(-0.117109433242)*x[2]**o+(0.99565775159)*x[2]
        arg[1,0,2,1]=(-0.790917746681)*x[0]**o+(-0.291814540513)*x[0]+(-0.0828604085821)*x[1]**o+(-0.0715382319561)*x[1]+(-0.188453557262)*x[2]**o+(0.297596604241)*x[2]
        arg[1,1,0,0]=(0.944343622558)*x[0]**o+(0.537893843235)*x[0]+(-0.370697218599)*x[1]**o+(-0.043709074747)*x[1]+(-0.803318845377)*x[2]**o+(0.814392946479)*x[2]
        arg[1,1,0,1]=(-0.246711023019)*x[0]**o+(-0.847902406307)*x[0]+(0.062135604814)*x[1]**o+(0.581341495124)*x[1]+(0.561484848132)*x[2]**o+(0.06820017559)*x[2]
        arg[1,1,1,0]=(-0.530782063061)*x[0]**o+(-0.951246029994)*x[0]+(-0.809647094351)*x[1]**o+(0.198165508669)*x[1]+(0.591475451485)*x[2]**o+(-0.0567469904903)*x[2]
        arg[1,1,1,1]=(-0.710844761085)*x[0]**o+(0.742146961382)*x[0]+(0.482950602277)*x[1]**o+(-0.701576649521)*x[1]+(0.752305355358)*x[2]**o+(0.658478074845)*x[2]
        arg[1,1,2,0]=(0.574657505707)*x[0]**o+(-0.268094098242)*x[0]+(-0.79745549318)*x[1]**o+(0.994732385826)*x[1]+(0.36752110393)*x[2]**o+(0.0733521344978)*x[2]
        arg[1,1,2,1]=(-0.365373199615)*x[0]**o+(0.295532428151)*x[0]+(-0.916049089517)*x[1]**o+(0.562876895658)*x[1]+(0.698818392781)*x[2]**o+(0.413166153315)*x[2]
        arg[1,2,0,0]=(0.544776615353)*x[0]**o+(-0.647889580869)*x[0]+(-0.0569263298125)*x[1]**o+(0.145083519829)*x[1]+(-0.0456368749942)*x[2]**o+(0.125915481669)*x[2]
        arg[1,2,0,1]=(0.11684721915)*x[0]**o+(-0.793592672443)*x[0]+(-0.428585283805)*x[1]**o+(0.280358391066)*x[1]+(0.72731741847)*x[2]**o+(-0.320459136831)*x[2]
        arg[1,2,1,0]=(0.561736948232)*x[0]**o+(0.833411538285)*x[0]+(0.361519222525)*x[1]**o+(-0.785948355356)*x[1]+(-0.595938528895)*x[2]**o+(0.720247127573)*x[2]
        arg[1,2,1,1]=(0.578630452142)*x[0]**o+(0.218238639275)*x[0]+(0.792496779641)*x[1]**o+(-0.818431209813)*x[1]+(-0.252522807953)*x[2]**o+(0.648505893606)*x[2]
        arg[1,2,2,0]=(-0.0269203575755)*x[0]**o+(-0.586454813205)*x[0]+(-0.239885637578)*x[1]**o+(0.186263805313)*x[1]+(0.585909451795)*x[2]**o+(-0.630731036615)*x[2]
        arg[1,2,2,1]=(-0.534642633404)*x[0]**o+(0.921034930282)*x[0]+(-0.326672365556)*x[1]**o+(0.690509146367)*x[1]+(-0.802567575176)*x[2]**o+(-0.235903818924)*x[2]
        arg[1,3,0,0]=(0.892302802156)*x[0]**o+(-0.528836075071)*x[0]+(-0.806520835766)*x[1]**o+(0.771321197577)*x[1]+(-0.545545197249)*x[2]**o+(-0.633185919719)*x[2]
        arg[1,3,0,1]=(0.883702447985)*x[0]**o+(0.698176356191)*x[0]+(-0.348189466787)*x[1]**o+(-0.390058291315)*x[1]+(0.805736008481)*x[2]**o+(0.861787745203)*x[2]
        arg[1,3,1,0]=(0.492527250981)*x[0]**o+(0.635712008949)*x[0]+(-0.464397346193)*x[1]**o+(-0.820952575379)*x[1]+(0.235015001164)*x[2]**o+(-0.846160845231)*x[2]
        arg[1,3,1,1]=(0.372499842901)*x[0]**o+(-0.524824414377)*x[0]+(0.553450359486)*x[1]**o+(-0.187372498756)*x[1]+(-0.389983195918)*x[2]**o+(-0.310421627605)*x[2]
        arg[1,3,2,0]=(0.533098362924)*x[0]**o+(0.917820078197)*x[0]+(-0.557683070597)*x[1]**o+(-0.388498248383)*x[1]+(-0.551866333066)*x[2]**o+(0.696873468511)*x[2]
        arg[1,3,2,1]=(-0.909038123273)*x[0]**o+(0.19323884412)*x[0]+(-0.00134861813725)*x[1]**o+(-0.858193032021)*x[1]+(0.0436856504635)*x[2]**o+(-0.646994265986)*x[2]
        arg[1,4,0,0]=(-0.974474210185)*x[0]**o+(-0.39786371112)*x[0]+(-0.829024534519)*x[1]**o+(0.836151642189)*x[1]+(0.679654655991)*x[2]**o+(0.750995377779)*x[2]
        arg[1,4,0,1]=(0.964745405431)*x[0]**o+(0.269189443297)*x[0]+(0.531636623506)*x[1]**o+(-0.522687387383)*x[1]+(0.463500385567)*x[2]**o+(0.160556224359)*x[2]
        arg[1,4,1,0]=(-0.714725894011)*x[0]**o+(0.0478807291181)*x[0]+(0.591494552904)*x[1]**o+(-0.240293119817)*x[1]+(-0.0463360360003)*x[2]**o+(-0.932835908096)*x[2]
        arg[1,4,1,1]=(-0.809072545597)*x[0]**o+(-0.963335650229)*x[0]+(-0.781885522838)*x[1]**o+(0.822395174213)*x[1]+(-0.42590899181)*x[2]**o+(-0.237357227214)*x[2]
        arg[1,4,2,0]=(-0.21281876112)*x[0]**o+(-0.131640536998)*x[0]+(-0.648680371617)*x[1]**o+(0.555759774031)*x[1]+(-0.911030244394)*x[2]**o+(0.962859403252)*x[2]
        arg[1,4,2,1]=(-0.247930702516)*x[0]**o+(0.509829884687)*x[0]+(0.0387816417333)*x[1]**o+(-0.605945140743)*x[1]+(0.187880415018)*x[2]**o+(-0.813242364046)*x[2]
        arg[2,0,0,0]=(0.616352869937)*x[0]**o+(-0.930708209966)*x[0]+(-0.874778661809)*x[1]**o+(-0.330225154904)*x[1]+(0.632020336392)*x[2]**o+(0.205631598228)*x[2]
        arg[2,0,0,1]=(-0.85704374519)*x[0]**o+(0.99958496465)*x[0]+(0.807095767473)*x[1]**o+(0.63097388144)*x[1]+(0.000665614692828)*x[2]**o+(0.418070858864)*x[2]
        arg[2,0,1,0]=(0.326870102902)*x[0]**o+(0.565855791221)*x[0]+(-0.39418644908)*x[1]**o+(-0.96081735968)*x[1]+(0.937255293725)*x[2]**o+(0.533015538451)*x[2]
        arg[2,0,1,1]=(-0.699939842176)*x[0]**o+(0.808055402204)*x[0]+(0.448948790025)*x[1]**o+(-0.720122241439)*x[1]+(0.0532200858319)*x[2]**o+(-0.204851171261)*x[2]
        arg[2,0,2,0]=(-0.985996827188)*x[0]**o+(-0.604602591167)*x[0]+(0.445903925662)*x[1]**o+(-0.163135114049)*x[1]+(-0.264281316158)*x[2]**o+(-0.381814212863)*x[2]
        arg[2,0,2,1]=(-0.195357773436)*x[0]**o+(0.73285122682)*x[0]+(-0.914846292757)*x[1]**o+(-0.190524885095)*x[1]+(-0.103440307801)*x[2]**o+(-0.0309647945217)*x[2]
        arg[2,1,0,0]=(0.601652939329)*x[0]**o+(-0.757171386065)*x[0]+(-0.91147090556)*x[1]**o+(-0.665258414941)*x[1]+(-0.0504245123191)*x[2]**o+(-0.274928427711)*x[2]
        arg[2,1,0,1]=(0.534038022604)*x[0]**o+(-0.977365399298)*x[0]+(0.884656682614)*x[1]**o+(0.0866421048629)*x[1]+(-0.70687555879)*x[2]**o+(-0.99928465036)*x[2]
        arg[2,1,1,0]=(-0.361196978257)*x[0]**o+(0.192559020092)*x[0]+(-0.0767715201942)*x[1]**o+(0.222254953069)*x[1]+(-0.128362555436)*x[2]**o+(-0.667485829139)*x[2]
        arg[2,1,1,1]=(0.00930987921229)*x[0]**o+(0.269207633273)*x[0]+(0.943798095916)*x[1]**o+(0.483123564952)*x[1]+(-0.231669365037)*x[2]**o+(-0.877005245489)*x[2]
        arg[2,1,2,0]=(-0.181479246679)*x[0]**o+(0.619551810132)*x[0]+(0.499400298722)*x[1]**o+(-0.307864349855)*x[1]+(0.19303040809)*x[2]**o+(0.523640816928)*x[2]
        arg[2,1,2,1]=(-0.99665050175)*x[0]**o+(-0.647775674514)*x[0]+(-0.0015481673821)*x[1]**o+(-0.749198451256)*x[1]+(0.652528193579)*x[2]**o+(-0.0194127861341)*x[2]
        arg[2,2,0,0]=(0.870069083779)*x[0]**o+(0.179747121592)*x[0]+(0.387523112512)*x[1]**o+(0.625711925024)*x[1]+(0.840141938096)*x[2]**o+(0.629709976556)*x[2]
        arg[2,2,0,1]=(-0.841259995642)*x[0]**o+(0.220858862445)*x[0]+(0.397334296484)*x[1]**o+(-0.036654179636)*x[1]+(0.235704322138)*x[2]**o+(-0.6299228864)*x[2]
        arg[2,2,1,0]=(-0.561184923307)*x[0]**o+(-0.643644162376)*x[0]+(0.728784994685)*x[1]**o+(-0.697418070844)*x[1]+(-0.56166463788)*x[2]**o+(-0.668760084534)*x[2]
        arg[2,2,1,1]=(0.92243000401)*x[0]**o+(0.96165833182)*x[0]+(0.540379002453)*x[1]**o+(0.630245078647)*x[1]+(-0.375873656541)*x[2]**o+(0.71424898756)*x[2]
        arg[2,2,2,0]=(0.331131296272)*x[0]**o+(-0.136167378345)*x[0]+(-0.375407637025)*x[1]**o+(-0.0890691004072)*x[1]+(-0.787306129385)*x[2]**o+(-0.447474749986)*x[2]
        arg[2,2,2,1]=(-0.47892171854)*x[0]**o+(0.341448171328)*x[0]+(0.662403724393)*x[1]**o+(0.581128263913)*x[1]+(0.00987835902932)*x[2]**o+(-0.884847129103)*x[2]
        arg[2,3,0,0]=(0.0219406022052)*x[0]**o+(-0.370001982912)*x[0]+(0.948162807268)*x[1]**o+(-0.870857624404)*x[1]+(0.580053532448)*x[2]**o+(-0.488657040041)*x[2]
        arg[2,3,0,1]=(-0.114799511617)*x[0]**o+(0.547853919356)*x[0]+(-0.224367707174)*x[1]**o+(0.551491902831)*x[1]+(-0.101845907889)*x[2]**o+(-0.793527438395)*x[2]
        arg[2,3,1,0]=(-0.103518223905)*x[0]**o+(-0.319840011093)*x[0]+(-0.856612861717)*x[1]**o+(0.0208578084364)*x[1]+(0.132248308449)*x[2]**o+(-0.580663554499)*x[2]
        arg[2,3,1,1]=(-0.958142224242)*x[0]**o+(-0.401299403423)*x[0]+(-0.289034019185)*x[1]**o+(-0.0913958572584)*x[1]+(-0.818515945817)*x[2]**o+(-0.918366425737)*x[2]
        arg[2,3,2,0]=(0.730554594734)*x[0]**o+(0.209964841318)*x[0]+(-0.4157212696)*x[1]**o+(-0.673375403124)*x[1]+(-0.409899056244)*x[2]**o+(0.124418003637)*x[2]
        arg[2,3,2,1]=(-0.930974589897)*x[0]**o+(0.256452400333)*x[0]+(0.778606258675)*x[1]**o+(0.23173182074)*x[1]+(-0.846439682798)*x[2]**o+(-0.408860675089)*x[2]
        arg[2,4,0,0]=(-0.246046780287)*x[0]**o+(-0.653766430943)*x[0]+(0.725379461155)*x[1]**o+(-0.305490823497)*x[1]+(0.0837637468427)*x[2]**o+(-0.458770631133)*x[2]
        arg[2,4,0,1]=(-0.612751147889)*x[0]**o+(0.504418327803)*x[0]+(-0.835051783805)*x[1]**o+(-0.86410559166)*x[1]+(0.928919612861)*x[2]**o+(0.303164594248)*x[2]
        arg[2,4,1,0]=(-0.26092972064)*x[0]**o+(-0.990592356395)*x[0]+(-0.48716443965)*x[1]**o+(0.659447608304)*x[1]+(0.429865593656)*x[2]**o+(-0.661470437053)*x[2]
        arg[2,4,1,1]=(0.325840618456)*x[0]**o+(-0.261974904995)*x[0]+(0.476982730267)*x[1]**o+(-0.738119182885)*x[1]+(0.903618858571)*x[2]**o+(-0.287442022624)*x[2]
        arg[2,4,2,0]=(0.655590135983)*x[0]**o+(-0.135505327297)*x[0]+(-0.38079131346)*x[1]**o+(0.663850751769)*x[1]+(-0.658568109867)*x[2]**o+(-0.267875066063)*x[2]
        arg[2,4,2,1]=(0.570712162768)*x[0]**o+(0.390833274898)*x[0]+(-0.97326103687)*x[1]**o+(-0.419300688467)*x[1]+(0.55695479018)*x[2]**o+(0.547271407555)*x[2]
        arg[3,0,0,0]=(-0.486023722572)*x[0]**o+(-0.790544923201)*x[0]+(0.101237181961)*x[1]**o+(0.23267779263)*x[1]+(-0.458580701132)*x[2]**o+(-0.479449733139)*x[2]
        arg[3,0,0,1]=(-0.705291625776)*x[0]**o+(0.144166702405)*x[0]+(-0.210596631253)*x[1]**o+(0.843721258425)*x[1]+(-0.716420627484)*x[2]**o+(0.145427688875)*x[2]
        arg[3,0,1,0]=(-0.838974120992)*x[0]**o+(0.824423674933)*x[0]+(-0.211643366873)*x[1]**o+(0.603255736693)*x[1]+(-0.119484817371)*x[2]**o+(-0.357272543813)*x[2]
        arg[3,0,1,1]=(0.140027541071)*x[0]**o+(0.373598174368)*x[0]+(-0.661831753147)*x[1]**o+(-0.973724961653)*x[1]+(-0.0738899100301)*x[2]**o+(0.0128579666074)*x[2]
        arg[3,0,2,0]=(-0.39083471939)*x[0]**o+(-0.00821070494132)*x[0]+(-0.315936839906)*x[1]**o+(-0.967556350778)*x[1]+(0.136991762476)*x[2]**o+(0.374041520875)*x[2]
        arg[3,0,2,1]=(0.0734743009776)*x[0]**o+(-0.103186084491)*x[0]+(0.426933353985)*x[1]**o+(-0.631917740007)*x[1]+(0.755296770294)*x[2]**o+(0.492171872216)*x[2]
        arg[3,1,0,0]=(-0.414502599055)*x[0]**o+(0.512031410722)*x[0]+(0.610677049483)*x[1]**o+(-0.075275839509)*x[1]+(0.853407100545)*x[2]**o+(-0.569731924126)*x[2]
        arg[3,1,0,1]=(0.886115095396)*x[0]**o+(0.811807969433)*x[0]+(0.550623354147)*x[1]**o+(-0.703728894875)*x[1]+(0.784318903356)*x[2]**o+(0.45891829089)*x[2]
        arg[3,1,1,0]=(0.312577129879)*x[0]**o+(0.0127110992422)*x[0]+(-0.174614340667)*x[1]**o+(0.744535847679)*x[1]+(0.991125452344)*x[2]**o+(-0.405595301834)*x[2]
        arg[3,1,1,1]=(0.784369258288)*x[0]**o+(-0.709503832235)*x[0]+(0.032963558933)*x[1]**o+(-0.471036543959)*x[1]+(-0.138022967476)*x[2]**o+(-0.38452297541)*x[2]
        arg[3,1,2,0]=(-0.337332295082)*x[0]**o+(-0.460457867002)*x[0]+(-0.102806204559)*x[1]**o+(-0.502889108364)*x[1]+(-0.641488011668)*x[2]**o+(0.359501036527)*x[2]
        arg[3,1,2,1]=(0.791575892417)*x[0]**o+(0.126853712619)*x[0]+(-0.408911942637)*x[1]**o+(0.557039825987)*x[1]+(-0.260719710621)*x[2]**o+(-0.648326395935)*x[2]
        arg[3,2,0,0]=(-0.0909007497318)*x[0]**o+(-0.337085519884)*x[0]+(-0.651987922715)*x[1]**o+(-0.130225558042)*x[1]+(0.926567188664)*x[2]**o+(0.764369275478)*x[2]
        arg[3,2,0,1]=(-0.764137934896)*x[0]**o+(0.443892891832)*x[0]+(0.688197384487)*x[1]**o+(0.485210058592)*x[1]+(0.267932741224)*x[2]**o+(0.0366064885582)*x[2]
        arg[3,2,1,0]=(0.49435832562)*x[0]**o+(0.860649583676)*x[0]+(0.667415091305)*x[1]**o+(0.144069385904)*x[1]+(-0.933847080974)*x[2]**o+(-0.378123304101)*x[2]
        arg[3,2,1,1]=(-0.460752644946)*x[0]**o+(0.491079753273)*x[0]+(-0.536055633506)*x[1]**o+(0.786378615881)*x[1]+(-0.190647453518)*x[2]**o+(-0.192868019034)*x[2]
        arg[3,2,2,0]=(0.0946687985697)*x[0]**o+(0.978197346579)*x[0]+(0.972747948856)*x[1]**o+(-0.575670550665)*x[1]+(-0.096229657916)*x[2]**o+(-0.772567430449)*x[2]
        arg[3,2,2,1]=(0.177521713734)*x[0]**o+(0.0683685252148)*x[0]+(0.423995112682)*x[1]**o+(-0.489247981968)*x[1]+(-0.537130434099)*x[2]**o+(-0.0228879131449)*x[2]
        arg[3,3,0,0]=(0.592201472175)*x[0]**o+(0.674304504132)*x[0]+(0.428414872879)*x[1]**o+(0.956414209094)*x[1]+(-0.272853242491)*x[2]**o+(-0.639858910901)*x[2]
        arg[3,3,0,1]=(0.572972211447)*x[0]**o+(0.943096858489)*x[0]+(-0.990948797302)*x[1]**o+(-0.475383526007)*x[1]+(0.542076985363)*x[2]**o+(-0.884809421862)*x[2]
        arg[3,3,1,0]=(0.775163727164)*x[0]**o+(0.101808074039)*x[0]+(0.575490330857)*x[1]**o+(0.199751068512)*x[1]+(0.538742155685)*x[2]**o+(-0.725822956214)*x[2]
        arg[3,3,1,1]=(-0.244586606882)*x[0]**o+(0.983712254807)*x[0]+(0.187370538052)*x[1]**o+(0.332611385073)*x[1]+(0.536377667373)*x[2]**o+(0.255272558489)*x[2]
        arg[3,3,2,0]=(-0.676040282048)*x[0]**o+(-0.172059284021)*x[0]+(0.25100451419)*x[1]**o+(0.303226061816)*x[1]+(0.138976177556)*x[2]**o+(-0.330104849626)*x[2]
        arg[3,3,2,1]=(-0.136437682299)*x[0]**o+(0.938786085213)*x[0]+(0.853018944962)*x[1]**o+(-0.628078903025)*x[1]+(-0.903567849919)*x[2]**o+(0.837621994591)*x[2]
        arg[3,4,0,0]=(0.28632587806)*x[0]**o+(-0.480626263522)*x[0]+(-0.0537965100699)*x[1]**o+(0.423252432505)*x[1]+(0.200777153039)*x[2]**o+(0.629713098071)*x[2]
        arg[3,4,0,1]=(0.326448964148)*x[0]**o+(0.809991245673)*x[0]+(-0.527362539073)*x[1]**o+(-0.0727890632651)*x[1]+(0.210806278508)*x[2]**o+(0.736535063786)*x[2]
        arg[3,4,1,0]=(0.760474271531)*x[0]**o+(0.378083396435)*x[0]+(-0.991296179693)*x[1]**o+(0.431965391269)*x[1]+(-0.12597704605)*x[2]**o+(-0.128017966575)*x[2]
        arg[3,4,1,1]=(-0.21508452109)*x[0]**o+(-0.583147117492)*x[0]+(0.779535489322)*x[1]**o+(0.601144982341)*x[1]+(-0.870362610456)*x[2]**o+(-0.318535391517)*x[2]
        arg[3,4,2,0]=(-0.195181632154)*x[0]**o+(0.237570596161)*x[0]+(-0.794725125363)*x[1]**o+(0.862030507166)*x[1]+(-0.612206533195)*x[2]**o+(0.494078406766)*x[2]
        arg[3,4,2,1]=(0.270278273932)*x[0]**o+(-0.115833233113)*x[0]+(0.754464364285)*x[1]**o+(-0.186069521903)*x[1]+(-0.864426716229)*x[2]**o+(-0.425054438064)*x[2]
        ref[0,0,0,0]=(0.412792261707)/(o+1.)+(0.904364083309)+(0.663998508168)*0.5**o
        ref[0,0,0,1]=(-1.07469333717)/(o+1.)+(-0.0923352454984)+(-0.010130464476)*0.5**o
        ref[0,0,1,0]=(0.287923726124)/(o+1.)+(-0.145507770124)+(-0.176542289385)*0.5**o
        ref[0,0,1,1]=(-0.24656050398)/(o+1.)+(0.341982901349)+(-0.98152572229)*0.5**o
        ref[0,0,2,0]=(1.24918965808)/(o+1.)+(0.165246366859)+(0.555897422173)*0.5**o
        ref[0,0,2,1]=(-1.23538766622)/(o+1.)+(-0.507697956406)+(0.23127040923)*0.5**o
        ref[0,1,0,0]=(-0.267829490776)/(o+1.)+(-0.549968075823)+(0.0202445177842)*0.5**o
        ref[0,1,0,1]=(0.826177960569)/(o+1.)+(0.113794537096)+(0.99935098551)*0.5**o
        ref[0,1,1,0]=(0.124131681048)/(o+1.)+(0.482277820918)+(0.151799117419)*0.5**o
        ref[0,1,1,1]=(0.620076580442)/(o+1.)+(0.401695495934)+(-0.51422532626)*0.5**o
        ref[0,1,2,0]=(1.75895293799)/(o+1.)+(-0.225823657945)+(0.908244888839)*0.5**o
        ref[0,1,2,1]=(-0.334875364565)/(o+1.)+(1.30271444502)+(-0.872307766634)*0.5**o
        ref[0,2,0,0]=(-0.42419420508)/(o+1.)+(-0.3575838585)+(0.00173619592035)*0.5**o
        ref[0,2,0,1]=(-0.57391253095)/(o+1.)+(-0.515088003037)+(0.998184336552)*0.5**o
        ref[0,2,1,0]=(0.114144861758)/(o+1.)+(-0.330251158386)+(-0.921234331968)*0.5**o
        ref[0,2,1,1]=(-1.42031433961)/(o+1.)+(0.581091144051)+(0.153578015941)*0.5**o
        ref[0,2,2,0]=(0.827236715358)/(o+1.)+(0.125325412749)+(0.29601674794)*0.5**o
        ref[0,2,2,1]=(1.34802786361)/(o+1.)+(0.280667679493)+(0.0580007647099)*0.5**o
        ref[0,3,0,0]=(1.28181937762)/(o+1.)+(-0.214505128496)+(-0.337999890505)*0.5**o
        ref[0,3,0,1]=(1.65569377802)/(o+1.)+(-0.363265639443)+(-0.825813329629)*0.5**o
        ref[0,3,1,0]=(0.560544990797)/(o+1.)+(-0.276979054204)+(0.699313512079)*0.5**o
        ref[0,3,1,1]=(-0.879495822379)/(o+1.)+(0.560842505268)+(0.140764124286)*0.5**o
        ref[0,3,2,0]=(-1.04751560841)/(o+1.)+(-0.238703591863)+(-0.68177752688)*0.5**o
        ref[0,3,2,1]=(-0.414022097401)/(o+1.)+(0.509666283203)+(0.655685968811)*0.5**o
        ref[0,4,0,0]=(1.0625549765)/(o+1.)+(-1.26969185098)+(-0.18027010068)*0.5**o
        ref[0,4,0,1]=(-0.430389257621)/(o+1.)+(-0.728926201043)+(-0.914768182433)*0.5**o
        ref[0,4,1,0]=(-0.585220311209)/(o+1.)+(0.344278586108)+(0.274727423835)*0.5**o
        ref[0,4,1,1]=(-0.596140928652)/(o+1.)+(-0.857974578823)+(0.257574026756)*0.5**o
        ref[0,4,2,0]=(-1.15566595381)/(o+1.)+(0.148922032671)+(0.703185665611)*0.5**o
        ref[0,4,2,1]=(0.255095282229)/(o+1.)+(-0.480012460544)+(-0.153792811673)*0.5**o
        ref[1,0,0,0]=(0.524122113192)/(o+1.)+(-0.434579759316)+(0.298044833507)*0.5**o
        ref[1,0,0,1]=(-0.220122924131)/(o+1.)+(0.466218310719)+(-0.575485929806)*0.5**o
        ref[1,0,1,0]=(0.290760959656)/(o+1.)+(-0.0552768720487)+(0.998358005707)*0.5**o
        ref[1,0,1,1]=(-1.69984876204)/(o+1.)+(0.325064655551)+(0.783593367143)*0.5**o
        ref[1,0,2,0]=(-0.999302906363)/(o+1.)+(0.639576565202)+(-0.42345692756)*0.5**o
        ref[1,0,2,1]=(-0.271313965844)/(o+1.)+(-0.0328780841139)+(-0.790917746681)*0.5**o
        ref[1,1,0,0]=(-1.17401606398)/(o+1.)+(0.654288857484)+(0.944343622558)*0.5**o
        ref[1,1,0,1]=(0.623620452946)/(o+1.)+(-0.0991803677964)+(-0.246711023019)*0.5**o
        ref[1,1,1,0]=(-0.218171642866)/(o+1.)+(-0.404913755908)+(-0.530782063061)*0.5**o
        ref[1,1,1,1]=(1.23525595763)/(o+1.)+(0.349524193353)+(-0.710844761085)*0.5**o
        ref[1,1,2,0]=(-0.42993438925)/(o+1.)+(0.399995211041)+(0.574657505707)*0.5**o
        ref[1,1,2,1]=(-0.217230696736)/(o+1.)+(0.635787738562)+(-0.365373199615)*0.5**o
        ref[1,2,0,0]=(-0.102563204807)/(o+1.)+(-0.188445289686)+(0.544776615353)*0.5**o
        ref[1,2,0,1]=(0.298732134665)/(o+1.)+(-0.416846709104)+(0.11684721915)*0.5**o
        ref[1,2,1,0]=(-0.234419306371)/(o+1.)+(0.383855155251)+(0.561736948232)*0.5**o
        ref[1,2,1,1]=(0.539973971687)/(o+1.)+(0.0241566615339)+(0.578630452142)*0.5**o
        ref[1,2,2,0]=(0.346023814217)/(o+1.)+(-0.515461022254)+(-0.0269203575755)*0.5**o
        ref[1,2,2,1]=(-1.12923994073)/(o+1.)+(0.687820128862)+(-0.534642633404)*0.5**o
        ref[1,3,0,0]=(-1.35206603301)/(o+1.)+(-0.195350398606)+(0.892302802156)*0.5**o
        ref[1,3,0,1]=(0.457546541694)/(o+1.)+(0.584952905039)+(0.883702447985)*0.5**o
        ref[1,3,1,0]=(-0.229382345029)/(o+1.)+(-0.51570070583)+(0.492527250981)*0.5**o
        ref[1,3,1,1]=(0.163467163568)/(o+1.)+(-0.511309270369)+(0.372499842901)*0.5**o
        ref[1,3,2,0]=(-1.10954940366)/(o+1.)+(0.613097649163)+(0.533098362924)*0.5**o
        ref[1,3,2,1]=(0.0423370323263)/(o+1.)+(-0.655974226943)+(-0.909038123273)*0.5**o
        ref[1,4,0,0]=(-0.149369878527)/(o+1.)+(0.594641654424)+(-0.974474210185)*0.5**o
        ref[1,4,0,1]=(0.995137009073)/(o+1.)+(-0.0464708598635)+(0.964745405431)*0.5**o
        ref[1,4,1,0]=(0.545158516903)/(o+1.)+(-0.562624149397)+(-0.714725894011)*0.5**o
        ref[1,4,1,1]=(-1.20779451465)/(o+1.)+(-0.189148851615)+(-0.809072545597)*0.5**o
        ref[1,4,2,0]=(-1.55971061601)/(o+1.)+(0.693489320143)+(-0.21281876112)*0.5**o
        ref[1,4,2,1]=(0.226662056752)/(o+1.)+(-0.454678810051)+(-0.247930702516)*0.5**o
        ref[2,0,0,0]=(-0.242758325417)/(o+1.)+(-0.527650883321)+(0.616352869937)*0.5**o
        ref[2,0,0,1]=(0.807761382166)/(o+1.)+(1.02431485248)+(-0.85704374519)*0.5**o
        ref[2,0,1,0]=(0.543068844645)/(o+1.)+(0.0690269849966)+(0.326870102902)*0.5**o
        ref[2,0,1,1]=(0.502168875857)/(o+1.)+(-0.0584590052478)+(-0.699939842176)*0.5**o
        ref[2,0,2,0]=(0.181622609504)/(o+1.)+(-0.57477595904)+(-0.985996827188)*0.5**o
        ref[2,0,2,1]=(-1.01828660056)/(o+1.)+(0.255680773601)+(-0.195357773436)*0.5**o
        ref[2,1,0,0]=(-0.961895417879)/(o+1.)+(-0.848679114358)+(0.601652939329)*0.5**o
        ref[2,1,0,1]=(0.177781123824)/(o+1.)+(-0.945003972397)+(0.534038022604)*0.5**o
        ref[2,1,1,0]=(-0.20513407563)/(o+1.)+(-0.126335927989)+(-0.361196978257)*0.5**o
        ref[2,1,1,1]=(0.712128730879)/(o+1.)+(-0.0623370236322)+(0.00930987921229)*0.5**o
        ref[2,1,2,0]=(0.692430706812)/(o+1.)+(0.417664138602)+(-0.181479246679)*0.5**o
        ref[2,1,2,1]=(0.650980026197)/(o+1.)+(-0.708193455952)+(-0.99665050175)*0.5**o
        ref[2,2,0,0]=(1.22766505061)/(o+1.)+(0.717584511586)+(0.870069083779)*0.5**o
        ref[2,2,0,1]=(0.633038618622)/(o+1.)+(-0.222859101796)+(-0.841259995642)*0.5**o
        ref[2,2,1,0]=(0.167120356805)/(o+1.)+(-1.00491115888)+(-0.561184923307)*0.5**o
        ref[2,2,1,1]=(0.164505345911)/(o+1.)+(1.15307619901)+(0.92243000401)*0.5**o
        ref[2,2,2,0]=(-1.16271376641)/(o+1.)+(-0.336355614369)+(0.331131296272)*0.5**o
        ref[2,2,2,1]=(0.672282083422)/(o+1.)+(0.018864653069)+(-0.47892171854)*0.5**o
        ref[2,3,0,0]=(1.52821633972)/(o+1.)+(-0.864758323678)+(0.0219406022052)*0.5**o
        ref[2,3,0,1]=(-0.326213615062)/(o+1.)+(0.152909191896)+(-0.114799511617)*0.5**o
        ref[2,3,1,0]=(-0.724364553267)/(o+1.)+(-0.439822878578)+(-0.103518223905)*0.5**o
        ref[2,3,1,1]=(-1.107549965)/(o+1.)+(-0.705530843209)+(-0.958142224242)*0.5**o
        ref[2,3,2,0]=(-0.825620325844)/(o+1.)+(-0.169496279085)+(0.730554594734)*0.5**o
        ref[2,3,2,1]=(-0.0678334241233)/(o+1.)+(0.0396617729923)+(-0.930974589897)*0.5**o
        ref[2,4,0,0]=(0.809143207998)/(o+1.)+(-0.709013942786)+(-0.246046780287)*0.5**o
        ref[2,4,0,1]=(0.0938678290563)/(o+1.)+(-0.0282613348041)+(-0.612751147889)*0.5**o
        ref[2,4,1,0]=(-0.0572988459938)/(o+1.)+(-0.496307592572)+(-0.26092972064)*0.5**o
        ref[2,4,1,1]=(1.38060158884)/(o+1.)+(-0.643768055252)+(0.325840618456)*0.5**o
        ref[2,4,2,0]=(-1.03935942333)/(o+1.)+(0.130235179204)+(0.655590135983)*0.5**o
        ref[2,4,2,1]=(-0.41630624669)/(o+1.)+(0.259401996992)+(0.570712162768)*0.5**o
        ref[3,0,0,0]=(-0.357343519171)/(o+1.)+(-0.518658431855)+(-0.486023722572)*0.5**o
        ref[3,0,0,1]=(-0.927017258737)/(o+1.)+(0.566657824853)+(-0.705291625776)*0.5**o
        ref[3,0,1,0]=(-0.331128184244)/(o+1.)+(0.535203433907)+(-0.838974120992)*0.5**o
        ref[3,0,1,1]=(-0.735721663177)/(o+1.)+(-0.293634410339)+(0.140027541071)*0.5**o
        ref[3,0,2,0]=(-0.178945077431)/(o+1.)+(-0.300862767422)+(-0.39083471939)*0.5**o
        ref[3,0,2,1]=(1.18223012428)/(o+1.)+(-0.121465976141)+(0.0734743009776)*0.5**o
        ref[3,1,0,0]=(1.46408415003)/(o+1.)+(-0.0664881764563)+(-0.414502599055)*0.5**o
        ref[3,1,0,1]=(1.3349422575)/(o+1.)+(0.283498682724)+(0.886115095396)*0.5**o
        ref[3,1,1,0]=(0.816511111677)/(o+1.)+(0.175825822544)+(0.312577129879)*0.5**o
        ref[3,1,1,1]=(-0.105059408543)/(o+1.)+(-0.782531675802)+(0.784369258288)*0.5**o
        ref[3,1,2,0]=(-0.744294216228)/(o+1.)+(-0.301922969419)+(-0.337332295082)*0.5**o
        ref[3,1,2,1]=(-0.669631653258)/(o+1.)+(0.0177835713351)+(0.791575892417)*0.5**o
        ref[3,2,0,0]=(0.274579265949)/(o+1.)+(0.148529098776)+(-0.0909007497318)*0.5**o
        ref[3,2,0,1]=(0.956130125711)/(o+1.)+(0.482854719491)+(-0.764137934896)*0.5**o
        ref[3,2,1,0]=(-0.266431989669)/(o+1.)+(0.313297832739)+(0.49435832562)*0.5**o
        ref[3,2,1,1]=(-0.726703087024)/(o+1.)+(0.542295175061)+(-0.460752644946)*0.5**o
        ref[3,2,2,0]=(0.87651829094)/(o+1.)+(-0.185020317268)+(0.0946687985697)*0.5**o
        ref[3,2,2,1]=(-0.113135321417)/(o+1.)+(-0.221883684949)+(0.177521713734)*0.5**o
        ref[3,3,0,0]=(0.155561630388)/(o+1.)+(0.495429901163)+(0.592201472175)*0.5**o
        ref[3,3,0,1]=(-0.44887181194)/(o+1.)+(-0.20854804469)+(0.572972211447)*0.5**o
        ref[3,3,1,0]=(1.11423248654)/(o+1.)+(-0.212131906831)+(0.775163727164)*0.5**o
        ref[3,3,1,1]=(0.723748205425)/(o+1.)+(0.785798099184)+(-0.244586606882)*0.5**o
        ref[3,3,2,0]=(0.389980691746)/(o+1.)+(-0.0994690359155)+(-0.676040282048)*0.5**o
        ref[3,3,2,1]=(-0.0505489049572)/(o+1.)+(0.57416458839)+(-0.136437682299)*0.5**o
        ref[3,4,0,0]=(0.14698064297)/(o+1.)+(0.286169633527)+(0.28632587806)*0.5**o
        ref[3,4,0,1]=(-0.316556260565)/(o+1.)+(0.736868623097)+(0.326448964148)*0.5**o
        ref[3,4,1,0]=(-1.11727322574)/(o+1.)+(0.341015410564)+(0.760474271531)*0.5**o
        ref[3,4,1,1]=(-0.0908271211344)/(o+1.)+(-0.150268763334)+(-0.21508452109)*0.5**o
        ref[3,4,2,0]=(-1.40693165856)/(o+1.)+(0.796839755046)+(-0.195181632154)*0.5**o
        ref[3,4,2,1]=(-0.109962351944)/(o+1.)+(-0.36347859654)+(0.270278273932)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactZero_fromData_Solution_rank0(self):
      """
      tests integral of rank 0 Data on the FunctionOnContactZero

      assumptions: Solution(self.domain) exists
                   self.domain supports integral on FunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      w=Solution(self.domain)
      x=w.getX()
      arg=Data(0,(),w)
      if dim==2:
        arg=(0.0433006726989)*x[0]**o+(0.819109331838)*x[0]+(0.150292245587)*x[1]**o+(0.949397165008)*x[1]
        ref=(0.150292245587)/(o+1.)+(0.884253248423)+(0.0433006726989)*0.5**o
      else:
        arg=(-0.765858486678)*x[0]**o+(-0.829061249978)*x[0]+(-0.0255273602735)*x[1]**o+(-0.93097352763)*x[1]+(-0.775974817954)*x[2]**o+(0.0414580763278)*x[2]
        ref=(-0.801502178228)/(o+1.)+(-0.85928835064)+(-0.765858486678)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactZero_fromData_Solution_rank1(self):
      """
      tests integral of rank 1 Data on the FunctionOnContactZero

      assumptions: Solution(self.domain) exists
                   self.domain supports integral on FunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      w=Solution(self.domain)
      x=w.getX()
      arg=Data(0,(2,),w)
      ref=numpy.zeros((2,),numpy.float_)
      if dim==2:
        arg[0]=(0.708418284738)*x[0]**o+(-0.265416249865)*x[0]+(0.0412873279676)*x[1]**o+(0.18827690539)*x[1]
        arg[1]=(-0.0639578348425)*x[0]**o+(-0.435408788697)*x[0]+(0.429134354603)*x[1]**o+(0.0391558318157)*x[1]
        ref[0]=(0.0412873279676)/(o+1.)+(-0.0385696722374)+(0.708418284738)*0.5**o
        ref[1]=(0.429134354603)/(o+1.)+(-0.198126478441)+(-0.0639578348425)*0.5**o
      else:
        arg[0]=(-0.516066856372)*x[0]**o+(-0.509677909948)*x[0]+(0.189594948337)*x[1]**o+(0.743432553688)*x[1]+(0.687864109046)*x[2]**o+(0.257280277379)*x[2]
        arg[1]=(0.948762785699)*x[0]**o+(0.372861763607)*x[0]+(-0.166460043702)*x[1]**o+(0.0474018901531)*x[1]+(-0.485380626595)*x[2]**o+(-0.341701937267)*x[2]
        ref[0]=(0.877459057383)/(o+1.)+(0.245517460559)+(-0.516066856372)*0.5**o
        ref[1]=(-0.651840670298)/(o+1.)+(0.0392808582465)+(0.948762785699)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactZero_fromData_Solution_rank2(self):
      """
      tests integral of rank 2 Data on the FunctionOnContactZero

      assumptions: Solution(self.domain) exists
                   self.domain supports integral on FunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      w=Solution(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5),w)
      ref=numpy.zeros((4, 5),numpy.float_)
      if dim==2:
        arg[0,0]=(-0.0540280486222)*x[0]**o+(-0.936629447756)*x[0]+(0.530867101627)*x[1]**o+(-0.404522355122)*x[1]
        arg[0,1]=(0.585051034599)*x[0]**o+(-0.899683342239)*x[0]+(0.61865830472)*x[1]**o+(0.491768251743)*x[1]
        arg[0,2]=(0.924131379905)*x[0]**o+(-0.394367734246)*x[0]+(0.432122525065)*x[1]**o+(0.79923634141)*x[1]
        arg[0,3]=(-0.41435196904)*x[0]**o+(-0.22039395628)*x[0]+(-0.866551833459)*x[1]**o+(-0.0966534238435)*x[1]
        arg[0,4]=(0.855785701017)*x[0]**o+(0.0879754354535)*x[0]+(-0.0727782016239)*x[1]**o+(0.746878225435)*x[1]
        arg[1,0]=(-0.868450306659)*x[0]**o+(-0.00845792225901)*x[0]+(0.227399876353)*x[1]**o+(-0.522196483799)*x[1]
        arg[1,1]=(0.53076931297)*x[0]**o+(0.165744140226)*x[0]+(0.14586398181)*x[1]**o+(-0.694169551043)*x[1]
        arg[1,2]=(-0.936146144862)*x[0]**o+(-0.198367580291)*x[0]+(0.848822682163)*x[1]**o+(0.360521711263)*x[1]
        arg[1,3]=(-0.395889451325)*x[0]**o+(0.510713345487)*x[0]+(-0.62440033386)*x[1]**o+(0.29695714683)*x[1]
        arg[1,4]=(0.847685553395)*x[0]**o+(-0.291266469606)*x[0]+(-0.517114839541)*x[1]**o+(-0.279948585383)*x[1]
        arg[2,0]=(-0.622481093435)*x[0]**o+(0.0911922519622)*x[0]+(0.216648336495)*x[1]**o+(0.663562516802)*x[1]
        arg[2,1]=(0.567833843918)*x[0]**o+(0.18116887359)*x[0]+(-0.0433012798352)*x[1]**o+(0.998332142285)*x[1]
        arg[2,2]=(-0.81729786652)*x[0]**o+(-0.634680104287)*x[0]+(0.991513806139)*x[1]**o+(-0.0721700196003)*x[1]
        arg[2,3]=(0.121181286827)*x[0]**o+(-0.677803464822)*x[0]+(0.870139639102)*x[1]**o+(-0.134662002755)*x[1]
        arg[2,4]=(-0.167376807346)*x[0]**o+(-0.500556007898)*x[0]+(-0.839256017197)*x[1]**o+(-0.82147890878)*x[1]
        arg[3,0]=(0.576521064132)*x[0]**o+(-0.510276372588)*x[0]+(0.92888227487)*x[1]**o+(-0.88293512036)*x[1]
        arg[3,1]=(-0.159048471862)*x[0]**o+(-0.0479500210678)*x[0]+(-0.0216438779517)*x[1]**o+(-0.108252189548)*x[1]
        arg[3,2]=(-0.483779576334)*x[0]**o+(-0.871829147209)*x[0]+(-0.22556218283)*x[1]**o+(0.451007116911)*x[1]
        arg[3,3]=(-0.358888369902)*x[0]**o+(0.436554685191)*x[0]+(-0.0359187908139)*x[1]**o+(-0.0148930474244)*x[1]
        arg[3,4]=(-0.995411478882)*x[0]**o+(-0.457206328785)*x[0]+(-0.833823947273)*x[1]**o+(-0.0517068976093)*x[1]
        ref[0,0]=(0.530867101627)/(o+1.)+(-0.670575901439)+(-0.0540280486222)*0.5**o
        ref[0,1]=(0.61865830472)/(o+1.)+(-0.203957545248)+(0.585051034599)*0.5**o
        ref[0,2]=(0.432122525065)/(o+1.)+(0.202434303582)+(0.924131379905)*0.5**o
        ref[0,3]=(-0.866551833459)/(o+1.)+(-0.158523690062)+(-0.41435196904)*0.5**o
        ref[0,4]=(-0.0727782016239)/(o+1.)+(0.417426830444)+(0.855785701017)*0.5**o
        ref[1,0]=(0.227399876353)/(o+1.)+(-0.265327203029)+(-0.868450306659)*0.5**o
        ref[1,1]=(0.14586398181)/(o+1.)+(-0.264212705408)+(0.53076931297)*0.5**o
        ref[1,2]=(0.848822682163)/(o+1.)+(0.0810770654864)+(-0.936146144862)*0.5**o
        ref[1,3]=(-0.62440033386)/(o+1.)+(0.403835246159)+(-0.395889451325)*0.5**o
        ref[1,4]=(-0.517114839541)/(o+1.)+(-0.285607527495)+(0.847685553395)*0.5**o
        ref[2,0]=(0.216648336495)/(o+1.)+(0.377377384382)+(-0.622481093435)*0.5**o
        ref[2,1]=(-0.0433012798352)/(o+1.)+(0.589750507938)+(0.567833843918)*0.5**o
        ref[2,2]=(0.991513806139)/(o+1.)+(-0.353425061944)+(-0.81729786652)*0.5**o
        ref[2,3]=(0.870139639102)/(o+1.)+(-0.406232733789)+(0.121181286827)*0.5**o
        ref[2,4]=(-0.839256017197)/(o+1.)+(-0.661017458339)+(-0.167376807346)*0.5**o
        ref[3,0]=(0.92888227487)/(o+1.)+(-0.696605746474)+(0.576521064132)*0.5**o
        ref[3,1]=(-0.0216438779517)/(o+1.)+(-0.078101105308)+(-0.159048471862)*0.5**o
        ref[3,2]=(-0.22556218283)/(o+1.)+(-0.210411015149)+(-0.483779576334)*0.5**o
        ref[3,3]=(-0.0359187908139)/(o+1.)+(0.210830818883)+(-0.358888369902)*0.5**o
        ref[3,4]=(-0.833823947273)/(o+1.)+(-0.254456613197)+(-0.995411478882)*0.5**o
      else:
        arg[0,0]=(-0.937858877179)*x[0]**o+(-0.41590845461)*x[0]+(0.317385444442)*x[1]**o+(-0.166284606371)*x[1]+(-0.846106994172)*x[2]**o+(0.227876218054)*x[2]
        arg[0,1]=(0.497785867084)*x[0]**o+(0.883557362705)*x[0]+(-0.846602726567)*x[1]**o+(-0.859091910548)*x[1]+(-0.295742503292)*x[2]**o+(-0.853931233266)*x[2]
        arg[0,2]=(0.744501879094)*x[0]**o+(-0.0181115125382)*x[0]+(-0.856099357899)*x[1]**o+(0.767449074734)*x[1]+(-0.858552187616)*x[2]**o+(-0.580142057156)*x[2]
        arg[0,3]=(0.210031467359)*x[0]**o+(-0.994426053833)*x[0]+(0.648522478175)*x[1]**o+(0.0355805603417)*x[1]+(-0.274347447112)*x[2]**o+(-0.280165537012)*x[2]
        arg[0,4]=(-0.323650660168)*x[0]**o+(-0.279592941595)*x[0]+(-0.478811425713)*x[1]**o+(0.19753271141)*x[1]+(0.476202875551)*x[2]**o+(0.657358100659)*x[2]
        arg[1,0]=(0.890659868437)*x[0]**o+(0.100235405989)*x[0]+(0.325177871418)*x[1]**o+(0.675379114656)*x[1]+(0.458473828724)*x[2]**o+(-0.449822467434)*x[2]
        arg[1,1]=(-0.467895039242)*x[0]**o+(-0.551300000422)*x[0]+(0.976111922801)*x[1]**o+(0.809389941274)*x[1]+(-0.693066235685)*x[2]**o+(0.595719115505)*x[2]
        arg[1,2]=(-0.727304617172)*x[0]**o+(-0.270682330445)*x[0]+(-0.32926621635)*x[1]**o+(0.0389068111165)*x[1]+(0.300096425532)*x[2]**o+(-0.964910767632)*x[2]
        arg[1,3]=(0.195012106644)*x[0]**o+(0.18137839607)*x[0]+(-0.823129279257)*x[1]**o+(-0.0440106889304)*x[1]+(-0.298898659898)*x[2]**o+(0.801623925974)*x[2]
        arg[1,4]=(0.590200326369)*x[0]**o+(-0.800085209049)*x[0]+(-0.323836133834)*x[1]**o+(-0.953413527643)*x[1]+(0.0257128844735)*x[2]**o+(0.652469516082)*x[2]
        arg[2,0]=(-0.467537281458)*x[0]**o+(-0.05673369912)*x[0]+(-0.813542619659)*x[1]**o+(-0.778596374878)*x[1]+(0.752073310476)*x[2]**o+(0.187824163254)*x[2]
        arg[2,1]=(-0.75708112983)*x[0]**o+(0.866111788737)*x[0]+(0.715424327432)*x[1]**o+(-0.297967739435)*x[1]+(0.420241611178)*x[2]**o+(-0.616867390634)*x[2]
        arg[2,2]=(0.00628628803839)*x[0]**o+(0.700962301988)*x[0]+(0.712979263919)*x[1]**o+(-0.414401132518)*x[1]+(-0.975498928756)*x[2]**o+(0.858116391039)*x[2]
        arg[2,3]=(0.948164399923)*x[0]**o+(-0.88227825142)*x[0]+(0.30703295864)*x[1]**o+(0.510607674984)*x[1]+(0.362361719392)*x[2]**o+(-0.208696380629)*x[2]
        arg[2,4]=(0.661545494115)*x[0]**o+(-0.122960355184)*x[0]+(-0.801367604574)*x[1]**o+(0.395708459987)*x[1]+(0.0197225340457)*x[2]**o+(-0.377005073637)*x[2]
        arg[3,0]=(0.733597350529)*x[0]**o+(0.339318978843)*x[0]+(0.317608635437)*x[1]**o+(0.147746051469)*x[1]+(-0.33413935488)*x[2]**o+(-0.793337674933)*x[2]
        arg[3,1]=(0.0918904281646)*x[0]**o+(-0.530786152204)*x[0]+(-0.587542750401)*x[1]**o+(-0.810331029621)*x[1]+(-0.40221065987)*x[2]**o+(-0.250472092093)*x[2]
        arg[3,2]=(0.966442952042)*x[0]**o+(-0.756649866552)*x[0]+(-0.36466223222)*x[1]**o+(-0.48639005055)*x[1]+(-0.831374462742)*x[2]**o+(0.503451143644)*x[2]
        arg[3,3]=(-0.322275644664)*x[0]**o+(-0.254067149982)*x[0]+(-0.440264794128)*x[1]**o+(0.226546045529)*x[1]+(-0.761870916789)*x[2]**o+(0.70024066379)*x[2]
        arg[3,4]=(0.595459342613)*x[0]**o+(-0.439436200258)*x[0]+(0.0932538180176)*x[1]**o+(-0.975450806707)*x[1]+(0.0461423323623)*x[2]**o+(0.0294352472154)*x[2]
        ref[0,0]=(-0.52872154973)/(o+1.)+(-0.177158421464)+(-0.937858877179)*0.5**o
        ref[0,1]=(-1.14234522986)/(o+1.)+(-0.414732890554)+(0.497785867084)*0.5**o
        ref[0,2]=(-1.71465154551)/(o+1.)+(0.0845977525197)+(0.744501879094)*0.5**o
        ref[0,3]=(0.374175031062)/(o+1.)+(-0.619505515252)+(0.210031467359)*0.5**o
        ref[0,4]=(-0.00260855016132)/(o+1.)+(0.287648935237)+(-0.323650660168)*0.5**o
        ref[1,0]=(0.783651700142)/(o+1.)+(0.162896026605)+(0.890659868437)*0.5**o
        ref[1,1]=(0.283045687117)/(o+1.)+(0.426904528178)+(-0.467895039242)*0.5**o
        ref[1,2]=(-0.0291697908188)/(o+1.)+(-0.59834314348)+(-0.727304617172)*0.5**o
        ref[1,3]=(-1.12202793916)/(o+1.)+(0.469495816557)+(0.195012106644)*0.5**o
        ref[1,4]=(-0.29812324936)/(o+1.)+(-0.550514610305)+(0.590200326369)*0.5**o
        ref[2,0]=(-0.0614693091826)/(o+1.)+(-0.323752955372)+(-0.467537281458)*0.5**o
        ref[2,1]=(1.13566593861)/(o+1.)+(-0.024361670666)+(-0.75708112983)*0.5**o
        ref[2,2]=(-0.262519664837)/(o+1.)+(0.572338780255)+(0.00628628803839)*0.5**o
        ref[2,3]=(0.669394678032)/(o+1.)+(-0.290183478532)+(0.948164399923)*0.5**o
        ref[2,4]=(-0.781645070528)/(o+1.)+(-0.0521284844165)+(0.661545494115)*0.5**o
        ref[3,0]=(-0.0165307194439)/(o+1.)+(-0.15313632231)+(0.733597350529)*0.5**o
        ref[3,1]=(-0.989753410271)/(o+1.)+(-0.795794636959)+(0.0918904281646)*0.5**o
        ref[3,2]=(-1.19603669496)/(o+1.)+(-0.369794386729)+(0.966442952042)*0.5**o
        ref[3,3]=(-1.20213571092)/(o+1.)+(0.336359779668)+(-0.322275644664)*0.5**o
        ref[3,4]=(0.13939615038)/(o+1.)+(-0.692725879875)+(0.595459342613)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactZero_fromData_Solution_rank3(self):
      """
      tests integral of rank 3 Data on the FunctionOnContactZero

      assumptions: Solution(self.domain) exists
                   self.domain supports integral on FunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      w=Solution(self.domain)
      x=w.getX()
      arg=Data(0,(6, 2, 2),w)
      ref=numpy.zeros((6, 2, 2),numpy.float_)
      if dim==2:
        arg[0,0,0]=(-0.12237588236)*x[0]**o+(-0.671341785528)*x[0]+(0.998910161099)*x[1]**o+(-0.552290246966)*x[1]
        arg[0,0,1]=(-0.90821257221)*x[0]**o+(-0.413391927764)*x[0]+(-0.428904099491)*x[1]**o+(-0.226766222189)*x[1]
        arg[0,1,0]=(-0.333819313127)*x[0]**o+(-0.458569899029)*x[0]+(-0.83776177332)*x[1]**o+(0.22951184631)*x[1]
        arg[0,1,1]=(-0.0571879315489)*x[0]**o+(-0.38056227895)*x[0]+(0.0784628125977)*x[1]**o+(-0.659353613638)*x[1]
        arg[1,0,0]=(0.645871468353)*x[0]**o+(-0.986005542006)*x[0]+(0.395704509276)*x[1]**o+(0.980666918726)*x[1]
        arg[1,0,1]=(0.337293036597)*x[0]**o+(-0.520809490378)*x[0]+(0.543657498632)*x[1]**o+(0.632777244118)*x[1]
        arg[1,1,0]=(-0.432349538121)*x[0]**o+(-0.579685247099)*x[0]+(0.331943072726)*x[1]**o+(0.25042188644)*x[1]
        arg[1,1,1]=(0.329861800715)*x[0]**o+(-0.48725090895)*x[0]+(-0.149357998414)*x[1]**o+(-0.580650134153)*x[1]
        arg[2,0,0]=(-0.75424658357)*x[0]**o+(-0.696352517414)*x[0]+(-0.00272313166917)*x[1]**o+(-0.488581343546)*x[1]
        arg[2,0,1]=(-0.136489901339)*x[0]**o+(-0.752587775699)*x[0]+(0.986413392791)*x[1]**o+(0.759344728162)*x[1]
        arg[2,1,0]=(-0.0832674877963)*x[0]**o+(0.765419402778)*x[0]+(0.958533611564)*x[1]**o+(-0.315461136294)*x[1]
        arg[2,1,1]=(0.362764663642)*x[0]**o+(-0.351519282456)*x[0]+(-0.115554433923)*x[1]**o+(-0.789365036398)*x[1]
        arg[3,0,0]=(0.0977752933224)*x[0]**o+(-0.838881001576)*x[0]+(0.275578737811)*x[1]**o+(-0.971208155806)*x[1]
        arg[3,0,1]=(0.0588243034654)*x[0]**o+(-0.141302234145)*x[0]+(-0.460044396206)*x[1]**o+(-0.776793718354)*x[1]
        arg[3,1,0]=(0.786922930047)*x[0]**o+(-0.187797924361)*x[0]+(0.971516347812)*x[1]**o+(-0.250547377602)*x[1]
        arg[3,1,1]=(-0.69240453335)*x[0]**o+(0.439425578473)*x[0]+(0.00864940781276)*x[1]**o+(-0.458898387231)*x[1]
        arg[4,0,0]=(0.0700104829681)*x[0]**o+(0.905031121846)*x[0]+(0.37676750477)*x[1]**o+(0.411190149331)*x[1]
        arg[4,0,1]=(-0.615737714754)*x[0]**o+(0.1945204765)*x[0]+(0.0889014902938)*x[1]**o+(0.650020022597)*x[1]
        arg[4,1,0]=(-0.883448519911)*x[0]**o+(0.00124188371434)*x[0]+(-0.501662684156)*x[1]**o+(0.290235036398)*x[1]
        arg[4,1,1]=(-0.723885968554)*x[0]**o+(-0.0112308623755)*x[0]+(0.679499427805)*x[1]**o+(-0.00261690404287)*x[1]
        arg[5,0,0]=(0.0834731365594)*x[0]**o+(-0.873857991293)*x[0]+(0.831158842487)*x[1]**o+(-0.585808323324)*x[1]
        arg[5,0,1]=(0.250830078938)*x[0]**o+(0.310347394372)*x[0]+(-0.412138455827)*x[1]**o+(0.974823718029)*x[1]
        arg[5,1,0]=(-0.865155065381)*x[0]**o+(0.72190541012)*x[0]+(0.72020941875)*x[1]**o+(-0.604997858133)*x[1]
        arg[5,1,1]=(0.57252234131)*x[0]**o+(-0.366230226771)*x[0]+(0.115400105646)*x[1]**o+(0.0485229323141)*x[1]
        ref[0,0,0]=(0.998910161099)/(o+1.)+(-0.611816016247)+(-0.12237588236)*0.5**o
        ref[0,0,1]=(-0.428904099491)/(o+1.)+(-0.320079074977)+(-0.90821257221)*0.5**o
        ref[0,1,0]=(-0.83776177332)/(o+1.)+(-0.11452902636)+(-0.333819313127)*0.5**o
        ref[0,1,1]=(0.0784628125977)/(o+1.)+(-0.519957946294)+(-0.0571879315489)*0.5**o
        ref[1,0,0]=(0.395704509276)/(o+1.)+(-0.00266931163989)+(0.645871468353)*0.5**o
        ref[1,0,1]=(0.543657498632)/(o+1.)+(0.0559838768697)+(0.337293036597)*0.5**o
        ref[1,1,0]=(0.331943072726)/(o+1.)+(-0.16463168033)+(-0.432349538121)*0.5**o
        ref[1,1,1]=(-0.149357998414)/(o+1.)+(-0.533950521552)+(0.329861800715)*0.5**o
        ref[2,0,0]=(-0.00272313166917)/(o+1.)+(-0.59246693048)+(-0.75424658357)*0.5**o
        ref[2,0,1]=(0.986413392791)/(o+1.)+(0.00337847623164)+(-0.136489901339)*0.5**o
        ref[2,1,0]=(0.958533611564)/(o+1.)+(0.224979133242)+(-0.0832674877963)*0.5**o
        ref[2,1,1]=(-0.115554433923)/(o+1.)+(-0.570442159427)+(0.362764663642)*0.5**o
        ref[3,0,0]=(0.275578737811)/(o+1.)+(-0.905044578691)+(0.0977752933224)*0.5**o
        ref[3,0,1]=(-0.460044396206)/(o+1.)+(-0.45904797625)+(0.0588243034654)*0.5**o
        ref[3,1,0]=(0.971516347812)/(o+1.)+(-0.219172650981)+(0.786922930047)*0.5**o
        ref[3,1,1]=(0.00864940781276)/(o+1.)+(-0.00973640437899)+(-0.69240453335)*0.5**o
        ref[4,0,0]=(0.37676750477)/(o+1.)+(0.658110635589)+(0.0700104829681)*0.5**o
        ref[4,0,1]=(0.0889014902938)/(o+1.)+(0.422270249549)+(-0.615737714754)*0.5**o
        ref[4,1,0]=(-0.501662684156)/(o+1.)+(0.145738460056)+(-0.883448519911)*0.5**o
        ref[4,1,1]=(0.679499427805)/(o+1.)+(-0.00692388320921)+(-0.723885968554)*0.5**o
        ref[5,0,0]=(0.831158842487)/(o+1.)+(-0.729833157309)+(0.0834731365594)*0.5**o
        ref[5,0,1]=(-0.412138455827)/(o+1.)+(0.642585556201)+(0.250830078938)*0.5**o
        ref[5,1,0]=(0.72020941875)/(o+1.)+(0.0584537759935)+(-0.865155065381)*0.5**o
        ref[5,1,1]=(0.115400105646)/(o+1.)+(-0.158853647228)+(0.57252234131)*0.5**o
      else:
        arg[0,0,0]=(0.962792187023)*x[0]**o+(0.799861778652)*x[0]+(-0.811367630134)*x[1]**o+(-0.553496861095)*x[1]+(-0.810089307714)*x[2]**o+(0.135449765364)*x[2]
        arg[0,0,1]=(0.644573004654)*x[0]**o+(-0.565823623798)*x[0]+(0.337026645127)*x[1]**o+(0.413612857799)*x[1]+(0.604366347987)*x[2]**o+(0.0867373815054)*x[2]
        arg[0,1,0]=(0.829914553754)*x[0]**o+(-0.462419132852)*x[0]+(0.889464473044)*x[1]**o+(-0.425594549042)*x[1]+(0.679251542389)*x[2]**o+(-0.501090535615)*x[2]
        arg[0,1,1]=(0.874166303558)*x[0]**o+(0.437595762221)*x[0]+(-0.836229998654)*x[1]**o+(-0.704020633222)*x[1]+(-0.279878325513)*x[2]**o+(0.697289463565)*x[2]
        arg[1,0,0]=(-0.963425479949)*x[0]**o+(0.399090631945)*x[0]+(-0.80371987894)*x[1]**o+(0.614102223184)*x[1]+(-0.821741166107)*x[2]**o+(-0.245209306654)*x[2]
        arg[1,0,1]=(-0.288791346674)*x[0]**o+(0.0125896799209)*x[0]+(0.0536290891541)*x[1]**o+(-0.127968165901)*x[1]+(-0.635591402779)*x[2]**o+(-0.723850350215)*x[2]
        arg[1,1,0]=(-0.554196270608)*x[0]**o+(-0.409293351346)*x[0]+(-0.759821124187)*x[1]**o+(-0.828913354317)*x[1]+(0.66900958614)*x[2]**o+(0.412403847239)*x[2]
        arg[1,1,1]=(-0.862735330269)*x[0]**o+(-0.406644646366)*x[0]+(-0.375399872184)*x[1]**o+(-0.10762837777)*x[1]+(0.540886844249)*x[2]**o+(0.97487281528)*x[2]
        arg[2,0,0]=(0.388899900771)*x[0]**o+(-0.692092360652)*x[0]+(0.794415524913)*x[1]**o+(0.369239170999)*x[1]+(0.482269641926)*x[2]**o+(0.58717167121)*x[2]
        arg[2,0,1]=(0.324795219208)*x[0]**o+(0.508631698719)*x[0]+(0.128456415737)*x[1]**o+(0.73678163559)*x[1]+(0.0527520660005)*x[2]**o+(-0.180289299372)*x[2]
        arg[2,1,0]=(-0.901972287405)*x[0]**o+(-0.635701120264)*x[0]+(0.298343193246)*x[1]**o+(-0.870902858159)*x[1]+(-0.979619200422)*x[2]**o+(0.708567755329)*x[2]
        arg[2,1,1]=(0.391663724321)*x[0]**o+(0.69891810611)*x[0]+(0.717069638885)*x[1]**o+(0.493487853359)*x[1]+(0.270536570073)*x[2]**o+(0.679115502666)*x[2]
        arg[3,0,0]=(0.830590943609)*x[0]**o+(-0.000320739222143)*x[0]+(-0.659146599349)*x[1]**o+(-0.858428982679)*x[1]+(-0.158070494634)*x[2]**o+(-0.364394913197)*x[2]
        arg[3,0,1]=(0.903615175916)*x[0]**o+(0.186562406505)*x[0]+(0.463860110418)*x[1]**o+(0.0382550628307)*x[1]+(0.835148180357)*x[2]**o+(-0.117637969912)*x[2]
        arg[3,1,0]=(-0.901874408714)*x[0]**o+(0.00493918499294)*x[0]+(0.180179475909)*x[1]**o+(-0.549428619271)*x[1]+(-0.392536708051)*x[2]**o+(0.68054148024)*x[2]
        arg[3,1,1]=(0.807693382033)*x[0]**o+(-0.47532617097)*x[0]+(-0.52499678021)*x[1]**o+(-0.0365395713918)*x[1]+(0.654881606829)*x[2]**o+(-0.63350736651)*x[2]
        arg[4,0,0]=(0.440830144989)*x[0]**o+(-0.454694325046)*x[0]+(0.403509667018)*x[1]**o+(0.360080283937)*x[1]+(0.164208775222)*x[2]**o+(0.203945755083)*x[2]
        arg[4,0,1]=(-0.836804887763)*x[0]**o+(-0.910979763109)*x[0]+(-0.629411293881)*x[1]**o+(0.0546020113997)*x[1]+(0.63435754491)*x[2]**o+(-0.323224122475)*x[2]
        arg[4,1,0]=(0.204216251019)*x[0]**o+(-0.372349067194)*x[0]+(-0.320768886225)*x[1]**o+(-0.139715856072)*x[1]+(0.62329130659)*x[2]**o+(0.626784461235)*x[2]
        arg[4,1,1]=(0.275074685897)*x[0]**o+(0.0664691742991)*x[0]+(0.949175504221)*x[1]**o+(-0.438858093561)*x[1]+(0.330489174462)*x[2]**o+(0.361586647545)*x[2]
        arg[5,0,0]=(0.370194889673)*x[0]**o+(-0.0549428893157)*x[0]+(0.0219932528442)*x[1]**o+(-0.699553487775)*x[1]+(0.0161212665659)*x[2]**o+(-0.268426682266)*x[2]
        arg[5,0,1]=(-0.527801267225)*x[0]**o+(0.151774276637)*x[0]+(-0.0623666836223)*x[1]**o+(-0.953572390482)*x[1]+(0.778912307756)*x[2]**o+(-0.851760269136)*x[2]
        arg[5,1,0]=(0.11418535486)*x[0]**o+(-0.299719975234)*x[0]+(-0.508303365848)*x[1]**o+(-0.856367302992)*x[1]+(0.311609370032)*x[2]**o+(0.4731881155)*x[2]
        arg[5,1,1]=(0.455947680647)*x[0]**o+(0.856811965126)*x[0]+(-0.176215162097)*x[1]**o+(0.581325693162)*x[1]+(-0.306514780798)*x[2]**o+(-0.005241787834)*x[2]
        ref[0,0,0]=(-1.62145693785)/(o+1.)+(0.19090734146)+(0.962792187023)*0.5**o
        ref[0,0,1]=(0.941392993115)/(o+1.)+(-0.0327366922471)+(0.644573004654)*0.5**o
        ref[0,1,0]=(1.56871601543)/(o+1.)+(-0.694552108754)+(0.829914553754)*0.5**o
        ref[0,1,1]=(-1.11610832417)/(o+1.)+(0.215432296282)+(0.874166303558)*0.5**o
        ref[1,0,0]=(-1.62546104505)/(o+1.)+(0.383991774237)+(-0.963425479949)*0.5**o
        ref[1,0,1]=(-0.581962313625)/(o+1.)+(-0.419614418097)+(-0.288791346674)*0.5**o
        ref[1,1,0]=(-0.0908115380472)/(o+1.)+(-0.412901429212)+(-0.554196270608)*0.5**o
        ref[1,1,1]=(0.165486972066)/(o+1.)+(0.230299895572)+(-0.862735330269)*0.5**o
        ref[2,0,0]=(1.27668516684)/(o+1.)+(0.132159240779)+(0.388899900771)*0.5**o
        ref[2,0,1]=(0.181208481738)/(o+1.)+(0.532562017469)+(0.324795219208)*0.5**o
        ref[2,1,0]=(-0.681276007176)/(o+1.)+(-0.399018111547)+(-0.901972287405)*0.5**o
        ref[2,1,1]=(0.987606208958)/(o+1.)+(0.935760731067)+(0.391663724321)*0.5**o
        ref[3,0,0]=(-0.817217093983)/(o+1.)+(-0.611572317549)+(0.830590943609)*0.5**o
        ref[3,0,1]=(1.29900829077)/(o+1.)+(0.0535897497116)+(0.903615175916)*0.5**o
        ref[3,1,0]=(-0.212357232141)/(o+1.)+(0.0680260229807)+(-0.901874408714)*0.5**o
        ref[3,1,1]=(0.12988482662)/(o+1.)+(-0.572686554436)+(0.807693382033)*0.5**o
        ref[4,0,0]=(0.567718442239)/(o+1.)+(0.0546658569873)+(0.440830144989)*0.5**o
        ref[4,0,1]=(0.00494625102884)/(o+1.)+(-0.589800937092)+(-0.836804887763)*0.5**o
        ref[4,1,0]=(0.302522420366)/(o+1.)+(0.0573597689848)+(0.204216251019)*0.5**o
        ref[4,1,1]=(1.27966467868)/(o+1.)+(-0.0054011358584)+(0.275074685897)*0.5**o
        ref[5,0,0]=(0.0381145194101)/(o+1.)+(-0.511461529678)+(0.370194889673)*0.5**o
        ref[5,0,1]=(0.716545624134)/(o+1.)+(-0.826779191491)+(-0.527801267225)*0.5**o
        ref[5,1,0]=(-0.196693995816)/(o+1.)+(-0.341449581363)+(0.11418535486)*0.5**o
        ref[5,1,1]=(-0.482729942895)/(o+1.)+(0.716447935227)+(0.455947680647)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactZero_fromData_Solution_rank4(self):
      """
      tests integral of rank 4 Data on the FunctionOnContactZero

      assumptions: Solution(self.domain) exists
                   self.domain supports integral on FunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      w=Solution(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5, 3, 2),w)
      ref=numpy.zeros((4, 5, 3, 2),numpy.float_)
      if dim==2:
        arg[0,0,0,0]=(0.415003066171)*x[0]**o+(-0.47441250281)*x[0]+(0.765391656872)*x[1]**o+(-0.170257448926)*x[1]
        arg[0,0,0,1]=(-0.928539602254)*x[0]**o+(0.792240012243)*x[0]+(-0.152730990885)*x[1]**o+(-0.667500454847)*x[1]
        arg[0,0,1,0]=(-0.937831078575)*x[0]**o+(0.963649346531)*x[0]+(0.505558859525)*x[1]**o+(-0.921354612243)*x[1]
        arg[0,0,1,1]=(0.909415074451)*x[0]**o+(0.199360871726)*x[0]+(0.194087349442)*x[1]**o+(0.875065381425)*x[1]
        arg[0,0,2,0]=(0.935190448625)*x[0]**o+(-0.109984767478)*x[0]+(0.518460161897)*x[1]**o+(-0.597684157349)*x[1]
        arg[0,0,2,1]=(-0.94062714999)*x[0]**o+(-0.308960574286)*x[0]+(0.578391007321)*x[1]**o+(0.864501040998)*x[1]
        arg[0,1,0,0]=(-0.197219678026)*x[0]**o+(-0.982899373778)*x[0]+(-0.603956105825)*x[1]**o+(-0.242717148913)*x[1]
        arg[0,1,0,1]=(-0.590715606611)*x[0]**o+(0.461185487788)*x[0]+(-0.270612249495)*x[1]**o+(-0.644742980654)*x[1]
        arg[0,1,1,0]=(-0.365516638755)*x[0]**o+(-0.255307471195)*x[0]+(0.0875989587033)*x[1]**o+(0.923189427818)*x[1]
        arg[0,1,1,1]=(0.315218558548)*x[0]**o+(-0.373070897551)*x[0]+(0.625272530385)*x[1]**o+(-0.174054543645)*x[1]
        arg[0,1,2,0]=(-0.502497884169)*x[0]**o+(-0.933785405889)*x[0]+(0.115500534905)*x[1]**o+(-0.864454092408)*x[1]
        arg[0,1,2,1]=(0.615837581258)*x[0]**o+(0.832538142367)*x[0]+(-0.0715650027129)*x[1]**o+(-0.775173801322)*x[1]
        arg[0,2,0,0]=(-0.898904577889)*x[0]**o+(-0.430072579352)*x[0]+(-0.532213103734)*x[1]**o+(0.970885652167)*x[1]
        arg[0,2,0,1]=(-0.0056483770747)*x[0]**o+(-0.566550554106)*x[0]+(0.751663691868)*x[1]**o+(0.677878661708)*x[1]
        arg[0,2,1,0]=(0.271196923039)*x[0]**o+(-0.291608730386)*x[0]+(-0.932579716748)*x[1]**o+(-0.437823650176)*x[1]
        arg[0,2,1,1]=(-0.788206773125)*x[0]**o+(0.18675055747)*x[0]+(0.747625244884)*x[1]**o+(0.654933611758)*x[1]
        arg[0,2,2,0]=(-0.787603701957)*x[0]**o+(-0.789784850865)*x[0]+(0.197684766271)*x[1]**o+(-0.988404958072)*x[1]
        arg[0,2,2,1]=(0.417538087399)*x[0]**o+(0.745832250585)*x[0]+(-0.525306115268)*x[1]**o+(0.134086803871)*x[1]
        arg[0,3,0,0]=(-0.230658480436)*x[0]**o+(-0.072983085881)*x[0]+(-0.149386157207)*x[1]**o+(-0.506870604002)*x[1]
        arg[0,3,0,1]=(-0.815859549425)*x[0]**o+(-0.962632495033)*x[0]+(-0.542257139282)*x[1]**o+(0.93598141472)*x[1]
        arg[0,3,1,0]=(0.0508974791413)*x[0]**o+(-0.722669867743)*x[0]+(0.714019527678)*x[1]**o+(0.710015283752)*x[1]
        arg[0,3,1,1]=(0.119419200166)*x[0]**o+(0.365663074152)*x[0]+(0.352455072404)*x[1]**o+(-0.0788953701144)*x[1]
        arg[0,3,2,0]=(-0.82075904747)*x[0]**o+(-0.224534176808)*x[0]+(-0.585094684368)*x[1]**o+(0.242958171035)*x[1]
        arg[0,3,2,1]=(-0.748061395205)*x[0]**o+(0.676995891547)*x[0]+(0.686644239408)*x[1]**o+(0.260350484065)*x[1]
        arg[0,4,0,0]=(0.128293903485)*x[0]**o+(0.226748407809)*x[0]+(0.129373873322)*x[1]**o+(0.988775321025)*x[1]
        arg[0,4,0,1]=(0.639638672694)*x[0]**o+(-0.817791454136)*x[0]+(-0.170677596319)*x[1]**o+(0.404161643274)*x[1]
        arg[0,4,1,0]=(0.43743097727)*x[0]**o+(-0.408230274495)*x[0]+(0.282544628333)*x[1]**o+(-0.542352697967)*x[1]
        arg[0,4,1,1]=(-0.995599965283)*x[0]**o+(-0.0512373236039)*x[0]+(0.218383977926)*x[1]**o+(0.929355335423)*x[1]
        arg[0,4,2,0]=(0.922609534479)*x[0]**o+(-0.166291574079)*x[0]+(-0.449331655188)*x[1]**o+(-0.0279347569009)*x[1]
        arg[0,4,2,1]=(0.950186933361)*x[0]**o+(-0.426615341822)*x[0]+(-0.167639663907)*x[1]**o+(0.219020502174)*x[1]
        arg[1,0,0,0]=(0.0573632292788)*x[0]**o+(0.543681035865)*x[0]+(-0.622510875807)*x[1]**o+(-0.171567467655)*x[1]
        arg[1,0,0,1]=(0.368975600444)*x[0]**o+(-0.754217532725)*x[0]+(0.469744047929)*x[1]**o+(-0.922445394777)*x[1]
        arg[1,0,1,0]=(-0.810194964715)*x[0]**o+(0.0861796117934)*x[0]+(0.625367069433)*x[1]**o+(-0.950906871025)*x[1]
        arg[1,0,1,1]=(0.744923275143)*x[0]**o+(0.465744045178)*x[0]+(0.359324627118)*x[1]**o+(0.273934205663)*x[1]
        arg[1,0,2,0]=(0.891959004941)*x[0]**o+(-0.498241249484)*x[0]+(0.318198109895)*x[1]**o+(0.0674564169104)*x[1]
        arg[1,0,2,1]=(0.842125439023)*x[0]**o+(0.731884782944)*x[0]+(-0.674805144805)*x[1]**o+(0.431782566319)*x[1]
        arg[1,1,0,0]=(-0.786719774805)*x[0]**o+(-0.55929144961)*x[0]+(-0.740263211102)*x[1]**o+(0.777914467998)*x[1]
        arg[1,1,0,1]=(0.831245977993)*x[0]**o+(-0.758991188073)*x[0]+(0.714032957488)*x[1]**o+(-0.879935556766)*x[1]
        arg[1,1,1,0]=(0.862796574462)*x[0]**o+(0.746875042565)*x[0]+(0.455949140326)*x[1]**o+(0.585491946591)*x[1]
        arg[1,1,1,1]=(-0.500409398892)*x[0]**o+(0.292753163538)*x[0]+(-0.62474682088)*x[1]**o+(0.583368504524)*x[1]
        arg[1,1,2,0]=(-0.402886961189)*x[0]**o+(-0.288733078165)*x[0]+(-0.770315341866)*x[1]**o+(-0.462764773299)*x[1]
        arg[1,1,2,1]=(0.358161520677)*x[0]**o+(-0.524052652951)*x[0]+(0.654162491323)*x[1]**o+(0.902382016998)*x[1]
        arg[1,2,0,0]=(-0.15539934232)*x[0]**o+(0.693162750731)*x[0]+(-0.0538737948754)*x[1]**o+(-0.025534990068)*x[1]
        arg[1,2,0,1]=(0.371159212282)*x[0]**o+(-0.751292369171)*x[0]+(-0.377617770006)*x[1]**o+(0.707647483842)*x[1]
        arg[1,2,1,0]=(0.117790556271)*x[0]**o+(0.848017682445)*x[0]+(-0.360936107893)*x[1]**o+(-0.564060873236)*x[1]
        arg[1,2,1,1]=(-0.524828128971)*x[0]**o+(0.737876348154)*x[0]+(0.183316445987)*x[1]**o+(-0.821207746237)*x[1]
        arg[1,2,2,0]=(0.300516177535)*x[0]**o+(-0.204187510383)*x[0]+(0.126601242737)*x[1]**o+(0.0794834423875)*x[1]
        arg[1,2,2,1]=(-0.29046926624)*x[0]**o+(-0.310057692431)*x[0]+(-0.810227948574)*x[1]**o+(-0.464128160277)*x[1]
        arg[1,3,0,0]=(-0.134275834453)*x[0]**o+(0.448577162559)*x[0]+(0.757723452446)*x[1]**o+(0.139202834725)*x[1]
        arg[1,3,0,1]=(0.882915728842)*x[0]**o+(-0.0750591947315)*x[0]+(0.503482692135)*x[1]**o+(-0.176350465386)*x[1]
        arg[1,3,1,0]=(-0.954792910945)*x[0]**o+(0.772215353622)*x[0]+(-0.861747803918)*x[1]**o+(-0.932727438775)*x[1]
        arg[1,3,1,1]=(0.747255393187)*x[0]**o+(0.0392661297355)*x[0]+(-0.322274864209)*x[1]**o+(-0.491462177972)*x[1]
        arg[1,3,2,0]=(0.139287057836)*x[0]**o+(0.312258185231)*x[0]+(-0.488288412308)*x[1]**o+(-0.185131700928)*x[1]
        arg[1,3,2,1]=(-0.861625516304)*x[0]**o+(0.681454010645)*x[0]+(0.206644380777)*x[1]**o+(-0.0344138929268)*x[1]
        arg[1,4,0,0]=(-0.714198122614)*x[0]**o+(-0.413113909739)*x[0]+(-0.971680855409)*x[1]**o+(0.602598611545)*x[1]
        arg[1,4,0,1]=(0.293578109913)*x[0]**o+(-0.495635685351)*x[0]+(0.347408878778)*x[1]**o+(-0.923814185786)*x[1]
        arg[1,4,1,0]=(0.843760126513)*x[0]**o+(-0.671075074679)*x[0]+(-0.284086603407)*x[1]**o+(0.437492233496)*x[1]
        arg[1,4,1,1]=(-0.342911357374)*x[0]**o+(-0.440232224635)*x[0]+(0.537701123084)*x[1]**o+(-0.525203729082)*x[1]
        arg[1,4,2,0]=(-0.71201360314)*x[0]**o+(-0.706332525171)*x[0]+(0.831438049852)*x[1]**o+(0.819141670055)*x[1]
        arg[1,4,2,1]=(-0.616403541928)*x[0]**o+(0.635973739636)*x[0]+(-0.0369972206543)*x[1]**o+(-0.08595692743)*x[1]
        arg[2,0,0,0]=(-0.959754423491)*x[0]**o+(-0.332473557912)*x[0]+(-0.464334304695)*x[1]**o+(0.597214020843)*x[1]
        arg[2,0,0,1]=(0.231051530608)*x[0]**o+(0.368524293555)*x[0]+(0.173044245392)*x[1]**o+(0.505867390994)*x[1]
        arg[2,0,1,0]=(0.376577055194)*x[0]**o+(-0.739990982509)*x[0]+(-0.816201949845)*x[1]**o+(-0.71233193641)*x[1]
        arg[2,0,1,1]=(-0.4891837531)*x[0]**o+(-0.121562226208)*x[0]+(-0.876294548464)*x[1]**o+(0.377763924093)*x[1]
        arg[2,0,2,0]=(-0.759548081841)*x[0]**o+(-0.118749434442)*x[0]+(0.602375475355)*x[1]**o+(-0.877213577079)*x[1]
        arg[2,0,2,1]=(0.114018259802)*x[0]**o+(-0.627149204501)*x[0]+(0.813648191424)*x[1]**o+(-0.612548616323)*x[1]
        arg[2,1,0,0]=(-0.572699800518)*x[0]**o+(0.507715829662)*x[0]+(-0.12122456959)*x[1]**o+(0.75522878228)*x[1]
        arg[2,1,0,1]=(0.0655268452799)*x[0]**o+(0.211191572823)*x[0]+(0.539760655497)*x[1]**o+(0.657397992057)*x[1]
        arg[2,1,1,0]=(0.358281119336)*x[0]**o+(0.119364032507)*x[0]+(-0.0242249415295)*x[1]**o+(-0.645986730609)*x[1]
        arg[2,1,1,1]=(0.523751704889)*x[0]**o+(-0.237670103222)*x[0]+(-0.995787126907)*x[1]**o+(-0.504129088442)*x[1]
        arg[2,1,2,0]=(-0.185380508646)*x[0]**o+(-0.218341933583)*x[0]+(0.149400154478)*x[1]**o+(0.00252616040715)*x[1]
        arg[2,1,2,1]=(0.81086932122)*x[0]**o+(0.387271527481)*x[0]+(0.908525646396)*x[1]**o+(-0.75383394204)*x[1]
        arg[2,2,0,0]=(-0.463416100256)*x[0]**o+(-0.891654021409)*x[0]+(-0.0495088159461)*x[1]**o+(0.398915277911)*x[1]
        arg[2,2,0,1]=(-0.200698149193)*x[0]**o+(0.127899540753)*x[0]+(-0.895280577411)*x[1]**o+(-0.197025422242)*x[1]
        arg[2,2,1,0]=(0.888683772867)*x[0]**o+(-0.84436563509)*x[0]+(-0.375709672588)*x[1]**o+(0.591336312818)*x[1]
        arg[2,2,1,1]=(-0.0167949020901)*x[0]**o+(0.145017501865)*x[0]+(0.868194479789)*x[1]**o+(-0.62439676328)*x[1]
        arg[2,2,2,0]=(0.441981698578)*x[0]**o+(-0.481156166596)*x[0]+(0.267937553127)*x[1]**o+(-0.37032246733)*x[1]
        arg[2,2,2,1]=(0.876701424579)*x[0]**o+(0.473798212378)*x[0]+(0.697645775876)*x[1]**o+(-0.798143376838)*x[1]
        arg[2,3,0,0]=(0.111231509217)*x[0]**o+(0.188121806171)*x[0]+(-0.0706155476337)*x[1]**o+(0.779423336221)*x[1]
        arg[2,3,0,1]=(-0.858913402812)*x[0]**o+(0.655459361097)*x[0]+(0.326282512878)*x[1]**o+(0.863386952614)*x[1]
        arg[2,3,1,0]=(-0.524551221953)*x[0]**o+(-0.807242415277)*x[0]+(0.575866251831)*x[1]**o+(-0.74503615846)*x[1]
        arg[2,3,1,1]=(0.391514075129)*x[0]**o+(0.477257809825)*x[0]+(-0.488263373008)*x[1]**o+(-0.29101442481)*x[1]
        arg[2,3,2,0]=(0.26532129161)*x[0]**o+(0.22794872765)*x[0]+(0.85094548724)*x[1]**o+(-0.369690624817)*x[1]
        arg[2,3,2,1]=(0.2339994493)*x[0]**o+(-0.901432708956)*x[0]+(0.309349741876)*x[1]**o+(0.132010344317)*x[1]
        arg[2,4,0,0]=(-0.713553783036)*x[0]**o+(0.791484993301)*x[0]+(0.339247480459)*x[1]**o+(0.158842307171)*x[1]
        arg[2,4,0,1]=(-0.733490022564)*x[0]**o+(0.631284830844)*x[0]+(0.340009298488)*x[1]**o+(-0.483162310574)*x[1]
        arg[2,4,1,0]=(0.709413437683)*x[0]**o+(0.896590124659)*x[0]+(0.680862759763)*x[1]**o+(-0.23579845663)*x[1]
        arg[2,4,1,1]=(-0.130761551034)*x[0]**o+(0.269095282394)*x[0]+(0.687201138134)*x[1]**o+(0.905183436376)*x[1]
        arg[2,4,2,0]=(0.256894911335)*x[0]**o+(0.706744915172)*x[0]+(0.910504833397)*x[1]**o+(-0.285088297895)*x[1]
        arg[2,4,2,1]=(0.95137247728)*x[0]**o+(-0.924669436646)*x[0]+(-0.146356272875)*x[1]**o+(-0.673326194162)*x[1]
        arg[3,0,0,0]=(0.663180444534)*x[0]**o+(0.894728781279)*x[0]+(0.894383747285)*x[1]**o+(0.969779862148)*x[1]
        arg[3,0,0,1]=(0.424556283891)*x[0]**o+(-0.97849198275)*x[0]+(0.367908872519)*x[1]**o+(0.624098355956)*x[1]
        arg[3,0,1,0]=(0.908698580251)*x[0]**o+(-0.899102004477)*x[0]+(0.509310001763)*x[1]**o+(-0.8399697646)*x[1]
        arg[3,0,1,1]=(0.523444225814)*x[0]**o+(0.410650511976)*x[0]+(-0.986699126405)*x[1]**o+(0.081494682537)*x[1]
        arg[3,0,2,0]=(-0.163604985314)*x[0]**o+(0.959338619715)*x[0]+(-0.843900271478)*x[1]**o+(-0.0218386544947)*x[1]
        arg[3,0,2,1]=(-0.490804271432)*x[0]**o+(0.123332672938)*x[0]+(0.902072835174)*x[1]**o+(-0.202340709957)*x[1]
        arg[3,1,0,0]=(-0.949731382685)*x[0]**o+(0.533695608969)*x[0]+(0.430512336347)*x[1]**o+(0.977434372776)*x[1]
        arg[3,1,0,1]=(0.945102298088)*x[0]**o+(-0.117807763531)*x[0]+(-0.203921504027)*x[1]**o+(-0.261266079346)*x[1]
        arg[3,1,1,0]=(0.762442770795)*x[0]**o+(-0.318456087863)*x[0]+(-0.365021977395)*x[1]**o+(0.0516932816311)*x[1]
        arg[3,1,1,1]=(-0.829061607399)*x[0]**o+(0.854900781365)*x[0]+(-0.397139314214)*x[1]**o+(-0.549679351555)*x[1]
        arg[3,1,2,0]=(-0.780040060687)*x[0]**o+(0.719393460042)*x[0]+(-0.435377419129)*x[1]**o+(0.655962063649)*x[1]
        arg[3,1,2,1]=(-0.853144390228)*x[0]**o+(0.0940222442927)*x[0]+(-0.642258121264)*x[1]**o+(-0.615386923437)*x[1]
        arg[3,2,0,0]=(0.377161814597)*x[0]**o+(0.349238572634)*x[0]+(0.361300611554)*x[1]**o+(0.554382639639)*x[1]
        arg[3,2,0,1]=(-0.51174671413)*x[0]**o+(-0.0586248632463)*x[0]+(0.083224329707)*x[1]**o+(0.506614699443)*x[1]
        arg[3,2,1,0]=(0.612695079311)*x[0]**o+(0.82053876378)*x[0]+(-0.132608876197)*x[1]**o+(0.701720563108)*x[1]
        arg[3,2,1,1]=(0.0928734170244)*x[0]**o+(-0.127564289866)*x[0]+(0.68468196965)*x[1]**o+(0.0168958305852)*x[1]
        arg[3,2,2,0]=(-0.863542893122)*x[0]**o+(0.0533007397457)*x[0]+(-0.0602194449612)*x[1]**o+(0.391122261927)*x[1]
        arg[3,2,2,1]=(-0.0942537498633)*x[0]**o+(-0.421741120475)*x[0]+(-0.0746938765947)*x[1]**o+(0.988111802567)*x[1]
        arg[3,3,0,0]=(-0.0588913579205)*x[0]**o+(0.439330145374)*x[0]+(-0.541714868844)*x[1]**o+(0.845344272279)*x[1]
        arg[3,3,0,1]=(0.221784155922)*x[0]**o+(0.983539923382)*x[0]+(-0.32332836636)*x[1]**o+(-0.892579743718)*x[1]
        arg[3,3,1,0]=(0.804097639203)*x[0]**o+(-0.342072604164)*x[0]+(0.755681096012)*x[1]**o+(0.432897246925)*x[1]
        arg[3,3,1,1]=(0.825696602707)*x[0]**o+(0.536999112965)*x[0]+(0.745033643035)*x[1]**o+(0.156806358776)*x[1]
        arg[3,3,2,0]=(-0.372114391352)*x[0]**o+(0.670417556114)*x[0]+(-0.569395510546)*x[1]**o+(-0.523784408534)*x[1]
        arg[3,3,2,1]=(0.439723359721)*x[0]**o+(0.809733189174)*x[0]+(0.781753556415)*x[1]**o+(0.375991547044)*x[1]
        arg[3,4,0,0]=(-0.429896412297)*x[0]**o+(-0.908820042346)*x[0]+(0.328787484151)*x[1]**o+(-0.610488251755)*x[1]
        arg[3,4,0,1]=(0.803567111353)*x[0]**o+(-0.487219379817)*x[0]+(-0.541538468561)*x[1]**o+(-0.692964506777)*x[1]
        arg[3,4,1,0]=(-0.963266835039)*x[0]**o+(0.667510333677)*x[0]+(0.308312799035)*x[1]**o+(0.686605377458)*x[1]
        arg[3,4,1,1]=(0.651198604373)*x[0]**o+(0.306465142929)*x[0]+(-0.923691546947)*x[1]**o+(-0.376240035765)*x[1]
        arg[3,4,2,0]=(-0.339406539705)*x[0]**o+(0.524709759622)*x[0]+(-0.0326564953763)*x[1]**o+(-0.658296780229)*x[1]
        arg[3,4,2,1]=(0.471355209305)*x[0]**o+(-0.473756698531)*x[0]+(0.193890658409)*x[1]**o+(-0.266632530307)*x[1]
        ref[0,0,0,0]=(0.765391656872)/(o+1.)+(-0.322334975868)+(0.415003066171)*0.5**o
        ref[0,0,0,1]=(-0.152730990885)/(o+1.)+(0.0623697786981)+(-0.928539602254)*0.5**o
        ref[0,0,1,0]=(0.505558859525)/(o+1.)+(0.0211473671443)+(-0.937831078575)*0.5**o
        ref[0,0,1,1]=(0.194087349442)/(o+1.)+(0.537213126576)+(0.909415074451)*0.5**o
        ref[0,0,2,0]=(0.518460161897)/(o+1.)+(-0.353834462413)+(0.935190448625)*0.5**o
        ref[0,0,2,1]=(0.578391007321)/(o+1.)+(0.277770233356)+(-0.94062714999)*0.5**o
        ref[0,1,0,0]=(-0.603956105825)/(o+1.)+(-0.612808261345)+(-0.197219678026)*0.5**o
        ref[0,1,0,1]=(-0.270612249495)/(o+1.)+(-0.0917787464331)+(-0.590715606611)*0.5**o
        ref[0,1,1,0]=(0.0875989587033)/(o+1.)+(0.333940978312)+(-0.365516638755)*0.5**o
        ref[0,1,1,1]=(0.625272530385)/(o+1.)+(-0.273562720598)+(0.315218558548)*0.5**o
        ref[0,1,2,0]=(0.115500534905)/(o+1.)+(-0.899119749149)+(-0.502497884169)*0.5**o
        ref[0,1,2,1]=(-0.0715650027129)/(o+1.)+(0.0286821705227)+(0.615837581258)*0.5**o
        ref[0,2,0,0]=(-0.532213103734)/(o+1.)+(0.270406536408)+(-0.898904577889)*0.5**o
        ref[0,2,0,1]=(0.751663691868)/(o+1.)+(0.0556640538013)+(-0.0056483770747)*0.5**o
        ref[0,2,1,0]=(-0.932579716748)/(o+1.)+(-0.364716190281)+(0.271196923039)*0.5**o
        ref[0,2,1,1]=(0.747625244884)/(o+1.)+(0.420842084614)+(-0.788206773125)*0.5**o
        ref[0,2,2,0]=(0.197684766271)/(o+1.)+(-0.889094904468)+(-0.787603701957)*0.5**o
        ref[0,2,2,1]=(-0.525306115268)/(o+1.)+(0.439959527228)+(0.417538087399)*0.5**o
        ref[0,3,0,0]=(-0.149386157207)/(o+1.)+(-0.289926844941)+(-0.230658480436)*0.5**o
        ref[0,3,0,1]=(-0.542257139282)/(o+1.)+(-0.0133255401565)+(-0.815859549425)*0.5**o
        ref[0,3,1,0]=(0.714019527678)/(o+1.)+(-0.00632729199566)+(0.0508974791413)*0.5**o
        ref[0,3,1,1]=(0.352455072404)/(o+1.)+(0.143383852019)+(0.119419200166)*0.5**o
        ref[0,3,2,0]=(-0.585094684368)/(o+1.)+(0.00921199711359)+(-0.82075904747)*0.5**o
        ref[0,3,2,1]=(0.686644239408)/(o+1.)+(0.468673187806)+(-0.748061395205)*0.5**o
        ref[0,4,0,0]=(0.129373873322)/(o+1.)+(0.607761864417)+(0.128293903485)*0.5**o
        ref[0,4,0,1]=(-0.170677596319)/(o+1.)+(-0.206814905431)+(0.639638672694)*0.5**o
        ref[0,4,1,0]=(0.282544628333)/(o+1.)+(-0.475291486231)+(0.43743097727)*0.5**o
        ref[0,4,1,1]=(0.218383977926)/(o+1.)+(0.43905900591)+(-0.995599965283)*0.5**o
        ref[0,4,2,0]=(-0.449331655188)/(o+1.)+(-0.0971131654902)+(0.922609534479)*0.5**o
        ref[0,4,2,1]=(-0.167639663907)/(o+1.)+(-0.103797419824)+(0.950186933361)*0.5**o
        ref[1,0,0,0]=(-0.622510875807)/(o+1.)+(0.186056784105)+(0.0573632292788)*0.5**o
        ref[1,0,0,1]=(0.469744047929)/(o+1.)+(-0.838331463751)+(0.368975600444)*0.5**o
        ref[1,0,1,0]=(0.625367069433)/(o+1.)+(-0.432363629616)+(-0.810194964715)*0.5**o
        ref[1,0,1,1]=(0.359324627118)/(o+1.)+(0.36983912542)+(0.744923275143)*0.5**o
        ref[1,0,2,0]=(0.318198109895)/(o+1.)+(-0.215392416287)+(0.891959004941)*0.5**o
        ref[1,0,2,1]=(-0.674805144805)/(o+1.)+(0.581833674632)+(0.842125439023)*0.5**o
        ref[1,1,0,0]=(-0.740263211102)/(o+1.)+(0.109311509194)+(-0.786719774805)*0.5**o
        ref[1,1,0,1]=(0.714032957488)/(o+1.)+(-0.81946337242)+(0.831245977993)*0.5**o
        ref[1,1,1,0]=(0.455949140326)/(o+1.)+(0.666183494578)+(0.862796574462)*0.5**o
        ref[1,1,1,1]=(-0.62474682088)/(o+1.)+(0.438060834031)+(-0.500409398892)*0.5**o
        ref[1,1,2,0]=(-0.770315341866)/(o+1.)+(-0.375748925732)+(-0.402886961189)*0.5**o
        ref[1,1,2,1]=(0.654162491323)/(o+1.)+(0.189164682023)+(0.358161520677)*0.5**o
        ref[1,2,0,0]=(-0.0538737948754)/(o+1.)+(0.333813880332)+(-0.15539934232)*0.5**o
        ref[1,2,0,1]=(-0.377617770006)/(o+1.)+(-0.0218224426646)+(0.371159212282)*0.5**o
        ref[1,2,1,0]=(-0.360936107893)/(o+1.)+(0.141978404604)+(0.117790556271)*0.5**o
        ref[1,2,1,1]=(0.183316445987)/(o+1.)+(-0.0416656990415)+(-0.524828128971)*0.5**o
        ref[1,2,2,0]=(0.126601242737)/(o+1.)+(-0.0623520339976)+(0.300516177535)*0.5**o
        ref[1,2,2,1]=(-0.810227948574)/(o+1.)+(-0.387092926354)+(-0.29046926624)*0.5**o
        ref[1,3,0,0]=(0.757723452446)/(o+1.)+(0.293889998642)+(-0.134275834453)*0.5**o
        ref[1,3,0,1]=(0.503482692135)/(o+1.)+(-0.125704830059)+(0.882915728842)*0.5**o
        ref[1,3,1,0]=(-0.861747803918)/(o+1.)+(-0.0802560425765)+(-0.954792910945)*0.5**o
        ref[1,3,1,1]=(-0.322274864209)/(o+1.)+(-0.226098024118)+(0.747255393187)*0.5**o
        ref[1,3,2,0]=(-0.488288412308)/(o+1.)+(0.0635632421511)+(0.139287057836)*0.5**o
        ref[1,3,2,1]=(0.206644380777)/(o+1.)+(0.323520058859)+(-0.861625516304)*0.5**o
        ref[1,4,0,0]=(-0.971680855409)/(o+1.)+(0.0947423509032)+(-0.714198122614)*0.5**o
        ref[1,4,0,1]=(0.347408878778)/(o+1.)+(-0.709724935569)+(0.293578109913)*0.5**o
        ref[1,4,1,0]=(-0.284086603407)/(o+1.)+(-0.116791420591)+(0.843760126513)*0.5**o
        ref[1,4,1,1]=(0.537701123084)/(o+1.)+(-0.482717976859)+(-0.342911357374)*0.5**o
        ref[1,4,2,0]=(0.831438049852)/(o+1.)+(0.056404572442)+(-0.71201360314)*0.5**o
        ref[1,4,2,1]=(-0.0369972206543)/(o+1.)+(0.275008406103)+(-0.616403541928)*0.5**o
        ref[2,0,0,0]=(-0.464334304695)/(o+1.)+(0.132370231466)+(-0.959754423491)*0.5**o
        ref[2,0,0,1]=(0.173044245392)/(o+1.)+(0.437195842274)+(0.231051530608)*0.5**o
        ref[2,0,1,0]=(-0.816201949845)/(o+1.)+(-0.72616145946)+(0.376577055194)*0.5**o
        ref[2,0,1,1]=(-0.876294548464)/(o+1.)+(0.128100848943)+(-0.4891837531)*0.5**o
        ref[2,0,2,0]=(0.602375475355)/(o+1.)+(-0.497981505761)+(-0.759548081841)*0.5**o
        ref[2,0,2,1]=(0.813648191424)/(o+1.)+(-0.619848910412)+(0.114018259802)*0.5**o
        ref[2,1,0,0]=(-0.12122456959)/(o+1.)+(0.631472305971)+(-0.572699800518)*0.5**o
        ref[2,1,0,1]=(0.539760655497)/(o+1.)+(0.43429478244)+(0.0655268452799)*0.5**o
        ref[2,1,1,0]=(-0.0242249415295)/(o+1.)+(-0.263311349051)+(0.358281119336)*0.5**o
        ref[2,1,1,1]=(-0.995787126907)/(o+1.)+(-0.370899595832)+(0.523751704889)*0.5**o
        ref[2,1,2,0]=(0.149400154478)/(o+1.)+(-0.107907886588)+(-0.185380508646)*0.5**o
        ref[2,1,2,1]=(0.908525646396)/(o+1.)+(-0.18328120728)+(0.81086932122)*0.5**o
        ref[2,2,0,0]=(-0.0495088159461)/(o+1.)+(-0.246369371749)+(-0.463416100256)*0.5**o
        ref[2,2,0,1]=(-0.895280577411)/(o+1.)+(-0.0345629407446)+(-0.200698149193)*0.5**o
        ref[2,2,1,0]=(-0.375709672588)/(o+1.)+(-0.126514661136)+(0.888683772867)*0.5**o
        ref[2,2,1,1]=(0.868194479789)/(o+1.)+(-0.239689630708)+(-0.0167949020901)*0.5**o
        ref[2,2,2,0]=(0.267937553127)/(o+1.)+(-0.425739316963)+(0.441981698578)*0.5**o
        ref[2,2,2,1]=(0.697645775876)/(o+1.)+(-0.16217258223)+(0.876701424579)*0.5**o
        ref[2,3,0,0]=(-0.0706155476337)/(o+1.)+(0.483772571196)+(0.111231509217)*0.5**o
        ref[2,3,0,1]=(0.326282512878)/(o+1.)+(0.759423156855)+(-0.858913402812)*0.5**o
        ref[2,3,1,0]=(0.575866251831)/(o+1.)+(-0.776139286869)+(-0.524551221953)*0.5**o
        ref[2,3,1,1]=(-0.488263373008)/(o+1.)+(0.0931216925076)+(0.391514075129)*0.5**o
        ref[2,3,2,0]=(0.85094548724)/(o+1.)+(-0.0708709485834)+(0.26532129161)*0.5**o
        ref[2,3,2,1]=(0.309349741876)/(o+1.)+(-0.384711182319)+(0.2339994493)*0.5**o
        ref[2,4,0,0]=(0.339247480459)/(o+1.)+(0.475163650236)+(-0.713553783036)*0.5**o
        ref[2,4,0,1]=(0.340009298488)/(o+1.)+(0.0740612601349)+(-0.733490022564)*0.5**o
        ref[2,4,1,0]=(0.680862759763)/(o+1.)+(0.330395834014)+(0.709413437683)*0.5**o
        ref[2,4,1,1]=(0.687201138134)/(o+1.)+(0.587139359385)+(-0.130761551034)*0.5**o
        ref[2,4,2,0]=(0.910504833397)/(o+1.)+(0.210828308638)+(0.256894911335)*0.5**o
        ref[2,4,2,1]=(-0.146356272875)/(o+1.)+(-0.798997815404)+(0.95137247728)*0.5**o
        ref[3,0,0,0]=(0.894383747285)/(o+1.)+(0.932254321713)+(0.663180444534)*0.5**o
        ref[3,0,0,1]=(0.367908872519)/(o+1.)+(-0.177196813397)+(0.424556283891)*0.5**o
        ref[3,0,1,0]=(0.509310001763)/(o+1.)+(-0.869535884539)+(0.908698580251)*0.5**o
        ref[3,0,1,1]=(-0.986699126405)/(o+1.)+(0.246072597257)+(0.523444225814)*0.5**o
        ref[3,0,2,0]=(-0.843900271478)/(o+1.)+(0.46874998261)+(-0.163604985314)*0.5**o
        ref[3,0,2,1]=(0.902072835174)/(o+1.)+(-0.0395040185096)+(-0.490804271432)*0.5**o
        ref[3,1,0,0]=(0.430512336347)/(o+1.)+(0.755564990872)+(-0.949731382685)*0.5**o
        ref[3,1,0,1]=(-0.203921504027)/(o+1.)+(-0.189536921439)+(0.945102298088)*0.5**o
        ref[3,1,1,0]=(-0.365021977395)/(o+1.)+(-0.133381403116)+(0.762442770795)*0.5**o
        ref[3,1,1,1]=(-0.397139314214)/(o+1.)+(0.152610714905)+(-0.829061607399)*0.5**o
        ref[3,1,2,0]=(-0.435377419129)/(o+1.)+(0.687677761846)+(-0.780040060687)*0.5**o
        ref[3,1,2,1]=(-0.642258121264)/(o+1.)+(-0.260682339572)+(-0.853144390228)*0.5**o
        ref[3,2,0,0]=(0.361300611554)/(o+1.)+(0.451810606136)+(0.377161814597)*0.5**o
        ref[3,2,0,1]=(0.083224329707)/(o+1.)+(0.223994918098)+(-0.51174671413)*0.5**o
        ref[3,2,1,0]=(-0.132608876197)/(o+1.)+(0.761129663444)+(0.612695079311)*0.5**o
        ref[3,2,1,1]=(0.68468196965)/(o+1.)+(-0.0553342296404)+(0.0928734170244)*0.5**o
        ref[3,2,2,0]=(-0.0602194449612)/(o+1.)+(0.222211500837)+(-0.863542893122)*0.5**o
        ref[3,2,2,1]=(-0.0746938765947)/(o+1.)+(0.283185341046)+(-0.0942537498633)*0.5**o
        ref[3,3,0,0]=(-0.541714868844)/(o+1.)+(0.642337208826)+(-0.0588913579205)*0.5**o
        ref[3,3,0,1]=(-0.32332836636)/(o+1.)+(0.0454800898319)+(0.221784155922)*0.5**o
        ref[3,3,1,0]=(0.755681096012)/(o+1.)+(0.0454123213802)+(0.804097639203)*0.5**o
        ref[3,3,1,1]=(0.745033643035)/(o+1.)+(0.34690273587)+(0.825696602707)*0.5**o
        ref[3,3,2,0]=(-0.569395510546)/(o+1.)+(0.0733165737898)+(-0.372114391352)*0.5**o
        ref[3,3,2,1]=(0.781753556415)/(o+1.)+(0.592862368109)+(0.439723359721)*0.5**o
        ref[3,4,0,0]=(0.328787484151)/(o+1.)+(-0.75965414705)+(-0.429896412297)*0.5**o
        ref[3,4,0,1]=(-0.541538468561)/(o+1.)+(-0.590091943297)+(0.803567111353)*0.5**o
        ref[3,4,1,0]=(0.308312799035)/(o+1.)+(0.677057855567)+(-0.963266835039)*0.5**o
        ref[3,4,1,1]=(-0.923691546947)/(o+1.)+(-0.0348874464182)+(0.651198604373)*0.5**o
        ref[3,4,2,0]=(-0.0326564953763)/(o+1.)+(-0.0667935103036)+(-0.339406539705)*0.5**o
        ref[3,4,2,1]=(0.193890658409)/(o+1.)+(-0.370194614419)+(0.471355209305)*0.5**o
      else:
        arg[0,0,0,0]=(0.0737627367843)*x[0]**o+(-0.897508234376)*x[0]+(0.499361502035)*x[1]**o+(-0.830793706142)*x[1]+(0.967672023686)*x[2]**o+(-0.94209897356)*x[2]
        arg[0,0,0,1]=(0.353968533247)*x[0]**o+(-0.828575104081)*x[0]+(0.111197799977)*x[1]**o+(0.475627052455)*x[1]+(0.10233739786)*x[2]**o+(-0.297090205985)*x[2]
        arg[0,0,1,0]=(0.315704665816)*x[0]**o+(0.450577419574)*x[0]+(-0.0225047225996)*x[1]**o+(0.221436098936)*x[1]+(-0.210708820992)*x[2]**o+(0.22813686037)*x[2]
        arg[0,0,1,1]=(-0.453912049362)*x[0]**o+(0.443266972459)*x[0]+(-0.166003491301)*x[1]**o+(0.325157034345)*x[1]+(0.975435329063)*x[2]**o+(0.516477372955)*x[2]
        arg[0,0,2,0]=(0.705225432895)*x[0]**o+(0.406870556161)*x[0]+(0.967131048725)*x[1]**o+(-0.31111693058)*x[1]+(0.915125425876)*x[2]**o+(-0.417407852906)*x[2]
        arg[0,0,2,1]=(-0.397162258761)*x[0]**o+(-0.49079016751)*x[0]+(-0.444905720899)*x[1]**o+(0.258998080457)*x[1]+(-0.406172174923)*x[2]**o+(0.166620074269)*x[2]
        arg[0,1,0,0]=(0.00444300957662)*x[0]**o+(0.59448685984)*x[0]+(-0.755596203939)*x[1]**o+(-0.587866578467)*x[1]+(-0.994620629569)*x[2]**o+(0.558539124849)*x[2]
        arg[0,1,0,1]=(0.726535641072)*x[0]**o+(-0.515254556008)*x[0]+(-0.737095044797)*x[1]**o+(0.396218854768)*x[1]+(-0.873340093434)*x[2]**o+(0.538194586759)*x[2]
        arg[0,1,1,0]=(-0.554579794515)*x[0]**o+(-0.754117891877)*x[0]+(0.242579027102)*x[1]**o+(0.368307663104)*x[1]+(0.308372932825)*x[2]**o+(-0.521440130826)*x[2]
        arg[0,1,1,1]=(0.294744137189)*x[0]**o+(0.44996935134)*x[0]+(0.118263237453)*x[1]**o+(-0.143566244715)*x[1]+(-0.927385432821)*x[2]**o+(0.912735237305)*x[2]
        arg[0,1,2,0]=(-0.30261020867)*x[0]**o+(-0.994883662031)*x[0]+(0.514662053738)*x[1]**o+(0.327770859964)*x[1]+(0.862452368458)*x[2]**o+(-0.739026842429)*x[2]
        arg[0,1,2,1]=(0.494855964972)*x[0]**o+(-0.358461686787)*x[0]+(0.406489619959)*x[1]**o+(0.417722618206)*x[1]+(0.679808195501)*x[2]**o+(-0.00502769522311)*x[2]
        arg[0,2,0,0]=(-0.914585215232)*x[0]**o+(0.305890296017)*x[0]+(-0.867826544668)*x[1]**o+(0.304580882671)*x[1]+(-0.401401956053)*x[2]**o+(0.842276423411)*x[2]
        arg[0,2,0,1]=(-0.260590397572)*x[0]**o+(0.541867335986)*x[0]+(0.0279475511153)*x[1]**o+(0.0255320376447)*x[1]+(-0.584584718877)*x[2]**o+(-0.640388032783)*x[2]
        arg[0,2,1,0]=(0.0455797513404)*x[0]**o+(-0.764984318339)*x[0]+(-0.257174640639)*x[1]**o+(0.745888292962)*x[1]+(0.964524202534)*x[2]**o+(0.435680191282)*x[2]
        arg[0,2,1,1]=(-0.300900136798)*x[0]**o+(-0.899164082043)*x[0]+(-0.63273853331)*x[1]**o+(0.56522562781)*x[1]+(-0.846482371222)*x[2]**o+(-0.909985752269)*x[2]
        arg[0,2,2,0]=(0.745711773093)*x[0]**o+(0.0627291150681)*x[0]+(-0.090340386706)*x[1]**o+(-0.0912293084958)*x[1]+(0.108613205226)*x[2]**o+(-0.28985444593)*x[2]
        arg[0,2,2,1]=(0.291340077334)*x[0]**o+(0.592856577974)*x[0]+(0.141857446535)*x[1]**o+(0.830526360843)*x[1]+(0.956840406828)*x[2]**o+(-0.717465405888)*x[2]
        arg[0,3,0,0]=(0.89242099254)*x[0]**o+(0.529348654364)*x[0]+(-0.074382350183)*x[1]**o+(0.874670648345)*x[1]+(-0.728104560168)*x[2]**o+(0.0781153963606)*x[2]
        arg[0,3,0,1]=(0.095390666498)*x[0]**o+(0.0124610436496)*x[0]+(0.278014506594)*x[1]**o+(0.923928359216)*x[1]+(-0.0905667357972)*x[2]**o+(-0.894031439498)*x[2]
        arg[0,3,1,0]=(-0.317750910722)*x[0]**o+(0.677406387747)*x[0]+(0.550094997541)*x[1]**o+(-0.30098732949)*x[1]+(0.501753868155)*x[2]**o+(-0.42859127752)*x[2]
        arg[0,3,1,1]=(-0.204455448392)*x[0]**o+(0.445237113438)*x[0]+(0.853596204887)*x[1]**o+(0.315701645178)*x[1]+(0.307192864939)*x[2]**o+(-0.708325809173)*x[2]
        arg[0,3,2,0]=(0.38581656579)*x[0]**o+(-0.79289714243)*x[0]+(-0.412226202311)*x[1]**o+(0.655534112291)*x[1]+(0.31352728014)*x[2]**o+(0.502089385396)*x[2]
        arg[0,3,2,1]=(-0.225571343976)*x[0]**o+(0.687765563016)*x[0]+(0.0421378888728)*x[1]**o+(0.54466301835)*x[1]+(-0.430288255786)*x[2]**o+(-0.117571440224)*x[2]
        arg[0,4,0,0]=(0.834083057621)*x[0]**o+(0.0292441566815)*x[0]+(-0.947154369416)*x[1]**o+(0.597153808995)*x[1]+(0.323706009194)*x[2]**o+(0.664093182644)*x[2]
        arg[0,4,0,1]=(-0.613152578007)*x[0]**o+(0.253155875339)*x[0]+(0.297138228345)*x[1]**o+(-0.604393247773)*x[1]+(-0.936718722281)*x[2]**o+(-0.410140848312)*x[2]
        arg[0,4,1,0]=(-0.373668614153)*x[0]**o+(-0.401240526097)*x[0]+(-0.417458139032)*x[1]**o+(0.456033864594)*x[1]+(-0.158434012225)*x[2]**o+(-0.33938136039)*x[2]
        arg[0,4,1,1]=(0.00943802199128)*x[0]**o+(-0.800756064115)*x[0]+(0.899199671858)*x[1]**o+(0.376830611387)*x[1]+(-0.930822199046)*x[2]**o+(0.192235807799)*x[2]
        arg[0,4,2,0]=(0.157099024354)*x[0]**o+(0.195942080381)*x[0]+(-0.554519054021)*x[1]**o+(0.871823312361)*x[1]+(-0.34803033039)*x[2]**o+(-0.646933331859)*x[2]
        arg[0,4,2,1]=(0.933762543076)*x[0]**o+(-0.169843985744)*x[0]+(0.971089176971)*x[1]**o+(-0.605176382286)*x[1]+(0.254369209427)*x[2]**o+(0.0314820745541)*x[2]
        arg[1,0,0,0]=(0.0417476479889)*x[0]**o+(-0.144293810851)*x[0]+(-0.677039420744)*x[1]**o+(-0.137097132377)*x[1]+(0.721082639859)*x[2]**o+(-0.32510577691)*x[2]
        arg[1,0,0,1]=(0.450986977446)*x[0]**o+(-0.921371106219)*x[0]+(0.536898979198)*x[1]**o+(-0.814951108186)*x[1]+(-0.733774506956)*x[2]**o+(0.938941747764)*x[2]
        arg[1,0,1,0]=(-0.23712795738)*x[0]**o+(0.450883240541)*x[0]+(0.101102533189)*x[1]**o+(-0.998253664232)*x[1]+(-0.843977488601)*x[2]**o+(-0.807025450448)*x[2]
        arg[1,0,1,1]=(-0.308721170682)*x[0]**o+(-0.546268734305)*x[0]+(0.643628083343)*x[1]**o+(0.206934468523)*x[1]+(-0.257667927436)*x[2]**o+(0.468860780891)*x[2]
        arg[1,0,2,0]=(-0.267232319593)*x[0]**o+(0.221737693042)*x[0]+(0.202218228468)*x[1]**o+(-0.560941556406)*x[1]+(0.910244160614)*x[2]**o+(0.361584784536)*x[2]
        arg[1,0,2,1]=(0.405254374432)*x[0]**o+(-0.102574362257)*x[0]+(0.527874164745)*x[1]**o+(0.0318395137256)*x[1]+(-0.448050293943)*x[2]**o+(-0.6401160036)*x[2]
        arg[1,1,0,0]=(-0.877224972777)*x[0]**o+(-0.544175806369)*x[0]+(0.528037913494)*x[1]**o+(-0.477854554671)*x[1]+(-0.394875438861)*x[2]**o+(-0.704656012962)*x[2]
        arg[1,1,0,1]=(-0.873928172379)*x[0]**o+(0.273455449944)*x[0]+(0.681090480547)*x[1]**o+(0.883243766093)*x[1]+(0.557357114631)*x[2]**o+(0.586148966774)*x[2]
        arg[1,1,1,0]=(0.552826305913)*x[0]**o+(-0.277269605193)*x[0]+(-0.681872908424)*x[1]**o+(-0.84578699459)*x[1]+(0.6543624385)*x[2]**o+(-0.355051513153)*x[2]
        arg[1,1,1,1]=(-0.71233607947)*x[0]**o+(-0.779861555179)*x[0]+(0.11496674995)*x[1]**o+(0.634901982779)*x[1]+(0.833044137166)*x[2]**o+(0.0443514408009)*x[2]
        arg[1,1,2,0]=(-0.411667274338)*x[0]**o+(0.218709061898)*x[0]+(0.132573928273)*x[1]**o+(-0.450677430962)*x[1]+(0.88420873514)*x[2]**o+(-0.618513305411)*x[2]
        arg[1,1,2,1]=(0.533005373575)*x[0]**o+(0.846343252936)*x[0]+(0.697076801609)*x[1]**o+(0.342260840742)*x[1]+(0.923380136498)*x[2]**o+(-0.727450493778)*x[2]
        arg[1,2,0,0]=(-0.637366803484)*x[0]**o+(-0.654649539562)*x[0]+(-0.443035194144)*x[1]**o+(0.4928304386)*x[1]+(0.889290653473)*x[2]**o+(0.208513517091)*x[2]
        arg[1,2,0,1]=(-0.404870452835)*x[0]**o+(-0.556431486625)*x[0]+(-0.428885935216)*x[1]**o+(0.208771213187)*x[1]+(0.706109349268)*x[2]**o+(0.726168991522)*x[2]
        arg[1,2,1,0]=(0.85574176569)*x[0]**o+(-0.371989246254)*x[0]+(0.0376283091444)*x[1]**o+(-0.164811219077)*x[1]+(0.842513532253)*x[2]**o+(-0.963454790343)*x[2]
        arg[1,2,1,1]=(0.576825533647)*x[0]**o+(0.866570909412)*x[0]+(-0.286228055595)*x[1]**o+(-0.509258889664)*x[1]+(0.358087406384)*x[2]**o+(0.2558870556)*x[2]
        arg[1,2,2,0]=(0.741411326194)*x[0]**o+(0.887444210472)*x[0]+(0.176659455323)*x[1]**o+(-0.292731470152)*x[1]+(0.179836617557)*x[2]**o+(-0.10894424483)*x[2]
        arg[1,2,2,1]=(0.386515069261)*x[0]**o+(-0.423389396097)*x[0]+(0.300384913804)*x[1]**o+(-0.569308908813)*x[1]+(0.845849686169)*x[2]**o+(-0.802483942058)*x[2]
        arg[1,3,0,0]=(0.848952965744)*x[0]**o+(0.113555873929)*x[0]+(0.132168122705)*x[1]**o+(0.464116031047)*x[1]+(-0.928746352905)*x[2]**o+(-0.176374657603)*x[2]
        arg[1,3,0,1]=(0.357870838328)*x[0]**o+(-0.294290586373)*x[0]+(0.395859596543)*x[1]**o+(-0.265635871788)*x[1]+(0.283887401402)*x[2]**o+(0.500378090749)*x[2]
        arg[1,3,1,0]=(-0.509333127149)*x[0]**o+(0.328856435768)*x[0]+(-0.377387784476)*x[1]**o+(-0.878505533565)*x[1]+(0.448760304667)*x[2]**o+(0.996692047048)*x[2]
        arg[1,3,1,1]=(-0.837885550386)*x[0]**o+(0.0608579436757)*x[0]+(-0.627132023609)*x[1]**o+(0.902538475491)*x[1]+(-0.449596037062)*x[2]**o+(-0.158786724549)*x[2]
        arg[1,3,2,0]=(0.205889276236)*x[0]**o+(0.576561726109)*x[0]+(-0.00281354319689)*x[1]**o+(0.214968600999)*x[1]+(0.811758703832)*x[2]**o+(0.260164242506)*x[2]
        arg[1,3,2,1]=(-0.331660060588)*x[0]**o+(0.713879758554)*x[0]+(0.804549392751)*x[1]**o+(-0.346397636822)*x[1]+(-0.364073503513)*x[2]**o+(0.338612348994)*x[2]
        arg[1,4,0,0]=(0.394201404247)*x[0]**o+(-0.416853395415)*x[0]+(-0.0581465874206)*x[1]**o+(0.105267182588)*x[1]+(0.791799718009)*x[2]**o+(0.0840172853234)*x[2]
        arg[1,4,0,1]=(-0.346144268453)*x[0]**o+(-0.661228785728)*x[0]+(0.326588395606)*x[1]**o+(0.766137046896)*x[1]+(0.566377202267)*x[2]**o+(0.613565461425)*x[2]
        arg[1,4,1,0]=(-0.907566140749)*x[0]**o+(-0.391612881392)*x[0]+(0.263322351365)*x[1]**o+(-0.4373853914)*x[1]+(0.248855925063)*x[2]**o+(0.973515668505)*x[2]
        arg[1,4,1,1]=(0.726194147182)*x[0]**o+(0.497593423927)*x[0]+(-0.951344923018)*x[1]**o+(0.55327321528)*x[1]+(-0.266195110749)*x[2]**o+(-0.310893832826)*x[2]
        arg[1,4,2,0]=(-0.940824546553)*x[0]**o+(-0.47838524024)*x[0]+(0.408728941942)*x[1]**o+(0.627300786998)*x[1]+(-0.0547545650101)*x[2]**o+(0.433290388525)*x[2]
        arg[1,4,2,1]=(0.728499572603)*x[0]**o+(-0.232088507059)*x[0]+(0.0550057703801)*x[1]**o+(0.159356057226)*x[1]+(0.111191390097)*x[2]**o+(-0.64498018454)*x[2]
        arg[2,0,0,0]=(0.467025642879)*x[0]**o+(-0.695379603222)*x[0]+(0.591732172092)*x[1]**o+(0.84325648142)*x[1]+(0.965696472916)*x[2]**o+(0.626196181241)*x[2]
        arg[2,0,0,1]=(-0.211129222294)*x[0]**o+(-0.501208172412)*x[0]+(-0.0162146409175)*x[1]**o+(-0.672364205306)*x[1]+(-0.851073479662)*x[2]**o+(-0.817568385627)*x[2]
        arg[2,0,1,0]=(-0.668065295797)*x[0]**o+(-0.90359857855)*x[0]+(0.040519772083)*x[1]**o+(-0.295766307198)*x[1]+(0.0491234828524)*x[2]**o+(-0.589557178985)*x[2]
        arg[2,0,1,1]=(0.928751918298)*x[0]**o+(-0.773787731767)*x[0]+(-0.744325160449)*x[1]**o+(-0.426987905682)*x[1]+(0.503339956851)*x[2]**o+(0.386756648382)*x[2]
        arg[2,0,2,0]=(-0.531818105049)*x[0]**o+(-0.586811490681)*x[0]+(0.780379352762)*x[1]**o+(-0.5575328003)*x[1]+(0.0354867101462)*x[2]**o+(0.581226069105)*x[2]
        arg[2,0,2,1]=(0.0150988200159)*x[0]**o+(0.56415563975)*x[0]+(0.540723773613)*x[1]**o+(0.0653377679637)*x[1]+(-0.52633696371)*x[2]**o+(-0.122811009405)*x[2]
        arg[2,1,0,0]=(-0.0829154151121)*x[0]**o+(-0.416192198737)*x[0]+(0.944232067474)*x[1]**o+(-0.159852455491)*x[1]+(-0.198397797188)*x[2]**o+(-0.684211447305)*x[2]
        arg[2,1,0,1]=(-0.834502772339)*x[0]**o+(-0.856817328109)*x[0]+(0.213303574315)*x[1]**o+(-0.283140791695)*x[1]+(-0.727061494352)*x[2]**o+(0.12871372765)*x[2]
        arg[2,1,1,0]=(0.247439924718)*x[0]**o+(-0.912464825903)*x[0]+(0.791177764168)*x[1]**o+(0.160047921728)*x[1]+(0.591086305304)*x[2]**o+(-0.774025941232)*x[2]
        arg[2,1,1,1]=(-0.120169534728)*x[0]**o+(-0.747254680906)*x[0]+(-0.394003570933)*x[1]**o+(-0.938198960695)*x[1]+(-0.0142776288517)*x[2]**o+(0.0627959740849)*x[2]
        arg[2,1,2,0]=(0.703841647274)*x[0]**o+(-0.946039767174)*x[0]+(-0.836717609846)*x[1]**o+(0.776573515772)*x[1]+(0.250502751027)*x[2]**o+(0.403896552902)*x[2]
        arg[2,1,2,1]=(-0.19760048131)*x[0]**o+(-0.250991189554)*x[0]+(0.278810924634)*x[1]**o+(-0.580148097047)*x[1]+(-0.197518781697)*x[2]**o+(0.116835806412)*x[2]
        arg[2,2,0,0]=(-0.507872937196)*x[0]**o+(-0.771841217512)*x[0]+(0.852666314762)*x[1]**o+(-0.482947444136)*x[1]+(0.805824662389)*x[2]**o+(-0.299255374349)*x[2]
        arg[2,2,0,1]=(-0.481403986338)*x[0]**o+(0.904322169038)*x[0]+(0.937553651477)*x[1]**o+(0.727763542606)*x[1]+(-0.69485539356)*x[2]**o+(0.547755372933)*x[2]
        arg[2,2,1,0]=(0.241675645258)*x[0]**o+(0.957681326492)*x[0]+(-0.305747218513)*x[1]**o+(-0.0393749799535)*x[1]+(-0.448892234759)*x[2]**o+(0.707429367761)*x[2]
        arg[2,2,1,1]=(0.114772627763)*x[0]**o+(-0.411147710203)*x[0]+(0.13389320737)*x[1]**o+(0.806309117051)*x[1]+(-0.572620769665)*x[2]**o+(0.0389925355751)*x[2]
        arg[2,2,2,0]=(0.887023209688)*x[0]**o+(0.158261816385)*x[0]+(0.687399231822)*x[1]**o+(-0.858491519094)*x[1]+(-0.00348150506922)*x[2]**o+(-0.863496570874)*x[2]
        arg[2,2,2,1]=(0.836731324893)*x[0]**o+(0.975472534225)*x[0]+(0.407275572226)*x[1]**o+(0.565132026147)*x[1]+(0.79110912675)*x[2]**o+(-0.0816802575667)*x[2]
        arg[2,3,0,0]=(0.196161020545)*x[0]**o+(0.254301467318)*x[0]+(0.759363127106)*x[1]**o+(0.536320544038)*x[1]+(0.522705219849)*x[2]**o+(0.376531450811)*x[2]
        arg[2,3,0,1]=(-0.862326385376)*x[0]**o+(0.766714365146)*x[0]+(0.279327620722)*x[1]**o+(-0.44416461578)*x[1]+(0.115982232563)*x[2]**o+(0.824064889151)*x[2]
        arg[2,3,1,0]=(0.571263839862)*x[0]**o+(-0.679998793927)*x[0]+(0.63548515376)*x[1]**o+(0.464171242163)*x[1]+(0.712124752567)*x[2]**o+(0.913131755622)*x[2]
        arg[2,3,1,1]=(0.272559456412)*x[0]**o+(0.0577748172306)*x[0]+(-0.610464803438)*x[1]**o+(0.233987628538)*x[1]+(0.904147555826)*x[2]**o+(-0.131245442847)*x[2]
        arg[2,3,2,0]=(0.3409363126)*x[0]**o+(0.40060221588)*x[0]+(-0.603551487822)*x[1]**o+(-0.697430353793)*x[1]+(-0.867009193249)*x[2]**o+(-0.178991284979)*x[2]
        arg[2,3,2,1]=(0.949170673623)*x[0]**o+(-0.0217541208215)*x[0]+(-0.775307373317)*x[1]**o+(0.421565612874)*x[1]+(0.419254478365)*x[2]**o+(-0.732203964657)*x[2]
        arg[2,4,0,0]=(0.876774885889)*x[0]**o+(0.441155536177)*x[0]+(-0.292113312365)*x[1]**o+(0.880590008772)*x[1]+(0.0732363738743)*x[2]**o+(-0.590855067487)*x[2]
        arg[2,4,0,1]=(0.347441329383)*x[0]**o+(-0.96566344519)*x[0]+(-0.0870932691958)*x[1]**o+(-0.793898415692)*x[1]+(0.911757004621)*x[2]**o+(0.623070769788)*x[2]
        arg[2,4,1,0]=(0.732123350436)*x[0]**o+(-0.707615856771)*x[0]+(-0.57969408421)*x[1]**o+(0.0643464469617)*x[1]+(-0.648380091597)*x[2]**o+(0.567100447405)*x[2]
        arg[2,4,1,1]=(-0.128872747083)*x[0]**o+(-0.463336399393)*x[0]+(-0.1679086419)*x[1]**o+(0.602218708561)*x[1]+(0.62788576637)*x[2]**o+(0.223548351914)*x[2]
        arg[2,4,2,0]=(0.619034818105)*x[0]**o+(0.167994824318)*x[0]+(-0.244892750219)*x[1]**o+(0.471338499791)*x[1]+(0.526950200066)*x[2]**o+(0.345611623572)*x[2]
        arg[2,4,2,1]=(0.645979236759)*x[0]**o+(0.79896884484)*x[0]+(-0.587486435644)*x[1]**o+(-0.467838428827)*x[1]+(0.672707985612)*x[2]**o+(-0.169680966884)*x[2]
        arg[3,0,0,0]=(-0.265836015694)*x[0]**o+(-0.383762854285)*x[0]+(0.420375720992)*x[1]**o+(0.739662661302)*x[1]+(-0.593559713248)*x[2]**o+(0.0743208592009)*x[2]
        arg[3,0,0,1]=(0.306246512933)*x[0]**o+(0.0357065903179)*x[0]+(0.213209244461)*x[1]**o+(-0.069836605971)*x[1]+(0.668535341179)*x[2]**o+(0.59870885392)*x[2]
        arg[3,0,1,0]=(0.509420593773)*x[0]**o+(0.995859415429)*x[0]+(-0.07734119959)*x[1]**o+(0.269006992126)*x[1]+(-0.465576902019)*x[2]**o+(0.318766487761)*x[2]
        arg[3,0,1,1]=(-0.609626065482)*x[0]**o+(0.970976299205)*x[0]+(-0.314734168176)*x[1]**o+(-0.536438958228)*x[1]+(0.900096595598)*x[2]**o+(-0.893816431641)*x[2]
        arg[3,0,2,0]=(0.269212290711)*x[0]**o+(-0.119897130167)*x[0]+(-0.937350065698)*x[1]**o+(0.913313769348)*x[1]+(0.311695335026)*x[2]**o+(0.58822688023)*x[2]
        arg[3,0,2,1]=(-0.445116541697)*x[0]**o+(-0.64630078598)*x[0]+(0.811173945666)*x[1]**o+(-0.910503175962)*x[1]+(0.160819266078)*x[2]**o+(-0.807050548571)*x[2]
        arg[3,1,0,0]=(0.25585673506)*x[0]**o+(-0.174293490388)*x[0]+(-0.510750951271)*x[1]**o+(-0.215780338673)*x[1]+(-0.531739311594)*x[2]**o+(0.479569840128)*x[2]
        arg[3,1,0,1]=(-0.159301665223)*x[0]**o+(0.88558738006)*x[0]+(-0.933255168276)*x[1]**o+(0.739621529924)*x[1]+(-0.371501556668)*x[2]**o+(-0.224043643673)*x[2]
        arg[3,1,1,0]=(0.673375237931)*x[0]**o+(0.29315421593)*x[0]+(0.378983835599)*x[1]**o+(-0.187220470447)*x[1]+(-0.122366431969)*x[2]**o+(-0.178182488821)*x[2]
        arg[3,1,1,1]=(0.273992606955)*x[0]**o+(-0.42302685869)*x[0]+(0.490650718142)*x[1]**o+(0.067822229282)*x[1]+(0.758676637186)*x[2]**o+(-0.840062442278)*x[2]
        arg[3,1,2,0]=(-0.533129977534)*x[0]**o+(0.743362501638)*x[0]+(0.0397932337285)*x[1]**o+(0.134261172942)*x[1]+(0.076467475088)*x[2]**o+(0.750124646154)*x[2]
        arg[3,1,2,1]=(0.977824314677)*x[0]**o+(0.322104772509)*x[0]+(0.478366489119)*x[1]**o+(0.139966919282)*x[1]+(0.823631645543)*x[2]**o+(0.29259439301)*x[2]
        arg[3,2,0,0]=(0.969349250234)*x[0]**o+(0.158604629065)*x[0]+(0.517280925739)*x[1]**o+(-0.32151445591)*x[1]+(0.516829269337)*x[2]**o+(0.935139450142)*x[2]
        arg[3,2,0,1]=(-0.673721580725)*x[0]**o+(0.1932869215)*x[0]+(0.47375679317)*x[1]**o+(0.377941367578)*x[1]+(-0.359306451576)*x[2]**o+(0.00498822119998)*x[2]
        arg[3,2,1,0]=(-0.432873699676)*x[0]**o+(0.191782167044)*x[0]+(0.191395777927)*x[1]**o+(-0.540088847941)*x[1]+(0.608451872364)*x[2]**o+(0.151589512705)*x[2]
        arg[3,2,1,1]=(0.633726614581)*x[0]**o+(-0.824133266602)*x[0]+(-0.380289061608)*x[1]**o+(-0.722136850408)*x[1]+(-0.807853184729)*x[2]**o+(0.152552828168)*x[2]
        arg[3,2,2,0]=(-0.182700878265)*x[0]**o+(0.105776705469)*x[0]+(0.837527799957)*x[1]**o+(0.120438395343)*x[1]+(-0.211142277471)*x[2]**o+(-0.0499206664041)*x[2]
        arg[3,2,2,1]=(0.999790796581)*x[0]**o+(0.523238189614)*x[0]+(0.108118557801)*x[1]**o+(-0.899694790145)*x[1]+(0.165089392648)*x[2]**o+(0.38599870475)*x[2]
        arg[3,3,0,0]=(-0.959143897162)*x[0]**o+(-0.968197835338)*x[0]+(-0.521866033091)*x[1]**o+(-0.560799919169)*x[1]+(0.604562365858)*x[2]**o+(0.195593261393)*x[2]
        arg[3,3,0,1]=(0.385513188376)*x[0]**o+(0.918485762628)*x[0]+(0.581126961809)*x[1]**o+(-0.365882492483)*x[1]+(-0.260563773993)*x[2]**o+(0.997016165521)*x[2]
        arg[3,3,1,0]=(0.809269330969)*x[0]**o+(0.472424328796)*x[0]+(-0.920680159926)*x[1]**o+(0.756205962059)*x[1]+(-0.886275742364)*x[2]**o+(-0.800132407145)*x[2]
        arg[3,3,1,1]=(-0.983474390608)*x[0]**o+(-0.436192301851)*x[0]+(0.653210945793)*x[1]**o+(-0.465357783291)*x[1]+(-0.799261076035)*x[2]**o+(-0.349990963152)*x[2]
        arg[3,3,2,0]=(0.710820041642)*x[0]**o+(-0.973768204693)*x[0]+(0.0408403957292)*x[1]**o+(0.61546181161)*x[1]+(0.915939389579)*x[2]**o+(0.634659938301)*x[2]
        arg[3,3,2,1]=(0.633364771689)*x[0]**o+(0.887553239682)*x[0]+(-0.64913373177)*x[1]**o+(0.557465560353)*x[1]+(-0.744072153243)*x[2]**o+(-0.301528624747)*x[2]
        arg[3,4,0,0]=(0.44039719913)*x[0]**o+(-0.0339525806656)*x[0]+(0.574166395893)*x[1]**o+(-0.810441051899)*x[1]+(0.254523432152)*x[2]**o+(-0.798403721953)*x[2]
        arg[3,4,0,1]=(0.253149910502)*x[0]**o+(-0.909846068796)*x[0]+(0.855107984234)*x[1]**o+(-0.973196168972)*x[1]+(0.646832394105)*x[2]**o+(0.811892995874)*x[2]
        arg[3,4,1,0]=(0.13027869782)*x[0]**o+(-0.424094152568)*x[0]+(-0.197473699021)*x[1]**o+(0.0795845331856)*x[1]+(-0.511301398362)*x[2]**o+(-0.331330168767)*x[2]
        arg[3,4,1,1]=(0.476554240918)*x[0]**o+(0.18800431479)*x[0]+(0.259031626216)*x[1]**o+(-0.0532533771273)*x[1]+(0.914847196458)*x[2]**o+(0.333851370508)*x[2]
        arg[3,4,2,0]=(0.800058524963)*x[0]**o+(-0.402028928896)*x[0]+(0.913881556051)*x[1]**o+(0.895516442873)*x[1]+(-0.0227700921994)*x[2]**o+(-0.0502319834692)*x[2]
        arg[3,4,2,1]=(-0.533834741079)*x[0]**o+(0.557954226232)*x[0]+(-0.121676666641)*x[1]**o+(0.781782385789)*x[1]+(0.0682870158781)*x[2]**o+(-0.236432552686)*x[2]
        ref[0,0,0,0]=(1.46703352572)/(o+1.)+(-1.33520045704)+(0.0737627367843)*0.5**o
        ref[0,0,0,1]=(0.213535197837)/(o+1.)+(-0.325019128805)+(0.353968533247)*0.5**o
        ref[0,0,1,0]=(-0.233213543591)/(o+1.)+(0.450075189439)+(0.315704665816)*0.5**o
        ref[0,0,1,1]=(0.809431837761)/(o+1.)+(0.642450689879)+(-0.453912049362)*0.5**o
        ref[0,0,2,0]=(1.8822564746)/(o+1.)+(-0.160827113662)+(0.705225432895)*0.5**o
        ref[0,0,2,1]=(-0.851077895823)/(o+1.)+(-0.032586006392)+(-0.397162258761)*0.5**o
        ref[0,1,0,0]=(-1.75021683351)/(o+1.)+(0.282579703111)+(0.00444300957662)*0.5**o
        ref[0,1,0,1]=(-1.61043513823)/(o+1.)+(0.209579442759)+(0.726535641072)*0.5**o
        ref[0,1,1,0]=(0.550951959927)/(o+1.)+(-0.453625179799)+(-0.554579794515)*0.5**o
        ref[0,1,1,1]=(-0.809122195368)/(o+1.)+(0.609569171964)+(0.294744137189)*0.5**o
        ref[0,1,2,0]=(1.3771144222)/(o+1.)+(-0.703069822248)+(-0.30261020867)*0.5**o
        ref[0,1,2,1]=(1.08629781546)/(o+1.)+(0.0271166180983)+(0.494855964972)*0.5**o
        ref[0,2,0,0]=(-1.26922850072)/(o+1.)+(0.72637380105)+(-0.914585215232)*0.5**o
        ref[0,2,0,1]=(-0.556637167762)/(o+1.)+(-0.036494329576)+(-0.260590397572)*0.5**o
        ref[0,2,1,0]=(0.707349561895)/(o+1.)+(0.208292082952)+(0.0455797513404)*0.5**o
        ref[0,2,1,1]=(-1.47922090453)/(o+1.)+(-0.621962103251)+(-0.300900136798)*0.5**o
        ref[0,2,2,0]=(0.01827281852)/(o+1.)+(-0.159177319679)+(0.745711773093)*0.5**o
        ref[0,2,2,1]=(1.09869785336)/(o+1.)+(0.352958766464)+(0.291340077334)*0.5**o
        ref[0,3,0,0]=(-0.802486910351)/(o+1.)+(0.741067349535)+(0.89242099254)*0.5**o
        ref[0,3,0,1]=(0.187447770797)/(o+1.)+(0.0211789816838)+(0.095390666498)*0.5**o
        ref[0,3,1,0]=(1.0518488657)/(o+1.)+(-0.0260861096315)+(-0.317750910722)*0.5**o
        ref[0,3,1,1]=(1.16078906983)/(o+1.)+(0.0263064747213)+(-0.204455448392)*0.5**o
        ref[0,3,2,0]=(-0.0986989221708)/(o+1.)+(0.182363177629)+(0.38581656579)*0.5**o
        ref[0,3,2,1]=(-0.388150366913)/(o+1.)+(0.557428570571)+(-0.225571343976)*0.5**o
        ref[0,4,0,0]=(-0.623448360222)/(o+1.)+(0.64524557416)+(0.834083057621)*0.5**o
        ref[0,4,0,1]=(-0.639580493935)/(o+1.)+(-0.380689110373)+(-0.613152578007)*0.5**o
        ref[0,4,1,0]=(-0.575892151257)/(o+1.)+(-0.142294010946)+(-0.373668614153)*0.5**o
        ref[0,4,1,1]=(-0.0316225271882)/(o+1.)+(-0.115844822465)+(0.00943802199128)*0.5**o
        ref[0,4,2,0]=(-0.902549384411)/(o+1.)+(0.210416030442)+(0.157099024354)*0.5**o
        ref[0,4,2,1]=(1.2254583864)/(o+1.)+(-0.371769146738)+(0.933762543076)*0.5**o
        ref[1,0,0,0]=(0.0440432191149)/(o+1.)+(-0.303248360069)+(0.0417476479889)*0.5**o
        ref[1,0,0,1]=(-0.196875527758)/(o+1.)+(-0.39869023332)+(0.450986977446)*0.5**o
        ref[1,0,1,0]=(-0.742874955412)/(o+1.)+(-0.677197937069)+(-0.23712795738)*0.5**o
        ref[1,0,1,1]=(0.385960155907)/(o+1.)+(0.0647632575546)+(-0.308721170682)*0.5**o
        ref[1,0,2,0]=(1.11246238908)/(o+1.)+(0.0111904605859)+(-0.267232319593)*0.5**o
        ref[1,0,2,1]=(0.0798238708014)/(o+1.)+(-0.355425426066)+(0.405254374432)*0.5**o
        ref[1,1,0,0]=(0.133162474633)/(o+1.)+(-0.863343187001)+(-0.877224972777)*0.5**o
        ref[1,1,0,1]=(1.23844759518)/(o+1.)+(0.871424091405)+(-0.873928172379)*0.5**o
        ref[1,1,1,0]=(-0.0275104699246)/(o+1.)+(-0.739054056468)+(0.552826305913)*0.5**o
        ref[1,1,1,1]=(0.948010887116)/(o+1.)+(-0.0503040657995)+(-0.71233607947)*0.5**o
        ref[1,1,2,0]=(1.01678266341)/(o+1.)+(-0.425240837237)+(-0.411667274338)*0.5**o
        ref[1,1,2,1]=(1.62045693811)/(o+1.)+(0.23057679995)+(0.533005373575)*0.5**o
        ref[1,2,0,0]=(0.44625545933)/(o+1.)+(0.0233472080645)+(-0.637366803484)*0.5**o
        ref[1,2,0,1]=(0.277223414052)/(o+1.)+(0.189254359042)+(-0.404870452835)*0.5**o
        ref[1,2,1,0]=(0.880141841398)/(o+1.)+(-0.750127627837)+(0.85574176569)*0.5**o
        ref[1,2,1,1]=(0.0718593507895)/(o+1.)+(0.306599537674)+(0.576825533647)*0.5**o
        ref[1,2,2,0]=(0.35649607288)/(o+1.)+(0.242884247745)+(0.741411326194)*0.5**o
        ref[1,2,2,1]=(1.14623459997)/(o+1.)+(-0.897591123484)+(0.386515069261)*0.5**o
        ref[1,3,0,0]=(-0.796578230199)/(o+1.)+(0.200648623687)+(0.848952965744)*0.5**o
        ref[1,3,0,1]=(0.679746997945)/(o+1.)+(-0.0297741837061)+(0.357870838328)*0.5**o
        ref[1,3,1,0]=(0.0713725201916)/(o+1.)+(0.223521474626)+(-0.509333127149)*0.5**o
        ref[1,3,1,1]=(-1.07672806067)/(o+1.)+(0.402304847309)+(-0.837885550386)*0.5**o
        ref[1,3,2,0]=(0.808945160636)/(o+1.)+(0.525847284807)+(0.205889276236)*0.5**o
        ref[1,3,2,1]=(0.440475889238)/(o+1.)+(0.353047235363)+(-0.331660060588)*0.5**o
        ref[1,4,0,0]=(0.733653130588)/(o+1.)+(-0.113784463751)+(0.394201404247)*0.5**o
        ref[1,4,0,1]=(0.892965597874)/(o+1.)+(0.359236861297)+(-0.346144268453)*0.5**o
        ref[1,4,1,0]=(0.512178276429)/(o+1.)+(0.0722586978563)+(-0.907566140749)*0.5**o
        ref[1,4,1,1]=(-1.21754003377)/(o+1.)+(0.369986403191)+(0.726194147182)*0.5**o
        ref[1,4,2,0]=(0.353974376932)/(o+1.)+(0.291102967642)+(-0.940824546553)*0.5**o
        ref[1,4,2,1]=(0.166197160477)/(o+1.)+(-0.358856317187)+(0.728499572603)*0.5**o
        ref[2,0,0,0]=(1.55742864501)/(o+1.)+(0.38703652972)+(0.467025642879)*0.5**o
        ref[2,0,0,1]=(-0.867288120579)/(o+1.)+(-0.995570381672)+(-0.211129222294)*0.5**o
        ref[2,0,1,0]=(0.0896432549354)/(o+1.)+(-0.894461032367)+(-0.668065295797)*0.5**o
        ref[2,0,1,1]=(-0.240985203598)/(o+1.)+(-0.407009494534)+(0.928751918298)*0.5**o
        ref[2,0,2,0]=(0.815866062908)/(o+1.)+(-0.281559110938)+(-0.531818105049)*0.5**o
        ref[2,0,2,1]=(0.0143868099026)/(o+1.)+(0.253341199154)+(0.0150988200159)*0.5**o
        ref[2,1,0,0]=(0.745834270286)/(o+1.)+(-0.630128050767)+(-0.0829154151121)*0.5**o
        ref[2,1,0,1]=(-0.513757920038)/(o+1.)+(-0.505622196077)+(-0.834502772339)*0.5**o
        ref[2,1,1,0]=(1.38226406947)/(o+1.)+(-0.763221422703)+(0.247439924718)*0.5**o
        ref[2,1,1,1]=(-0.408281199784)/(o+1.)+(-0.811328833758)+(-0.120169534728)*0.5**o
        ref[2,1,2,0]=(-0.586214858819)/(o+1.)+(0.11721515075)+(0.703841647274)*0.5**o
        ref[2,1,2,1]=(0.0812921429368)/(o+1.)+(-0.357151740094)+(-0.19760048131)*0.5**o
        ref[2,2,0,0]=(1.65849097715)/(o+1.)+(-0.777022017999)+(-0.507872937196)*0.5**o
        ref[2,2,0,1]=(0.242698257917)/(o+1.)+(1.08992054229)+(-0.481403986338)*0.5**o
        ref[2,2,1,0]=(-0.754639453272)/(o+1.)+(0.81286785715)+(0.241675645258)*0.5**o
        ref[2,2,1,1]=(-0.438727562295)/(o+1.)+(0.217076971211)+(0.114772627763)*0.5**o
        ref[2,2,2,0]=(0.683917726753)/(o+1.)+(-0.781863136791)+(0.887023209688)*0.5**o
        ref[2,2,2,1]=(1.19838469898)/(o+1.)+(0.729462151403)+(0.836731324893)*0.5**o
        ref[2,3,0,0]=(1.28206834695)/(o+1.)+(0.583576731084)+(0.196161020545)*0.5**o
        ref[2,3,0,1]=(0.395309853285)/(o+1.)+(0.573307319258)+(-0.862326385376)*0.5**o
        ref[2,3,1,0]=(1.34760990633)/(o+1.)+(0.348652101929)+(0.571263839862)*0.5**o
        ref[2,3,1,1]=(0.293682752388)/(o+1.)+(0.080258501461)+(0.272559456412)*0.5**o
        ref[2,3,2,0]=(-1.47056068107)/(o+1.)+(-0.237909711446)+(0.3409363126)*0.5**o
        ref[2,3,2,1]=(-0.356052894952)/(o+1.)+(-0.166196236302)+(0.949170673623)*0.5**o
        ref[2,4,0,0]=(-0.21887693849)/(o+1.)+(0.365445238731)+(0.876774885889)*0.5**o
        ref[2,4,0,1]=(0.824663735425)/(o+1.)+(-0.568245545547)+(0.347441329383)*0.5**o
        ref[2,4,1,0]=(-1.22807417581)/(o+1.)+(-0.0380844812022)+(0.732123350436)*0.5**o
        ref[2,4,1,1]=(0.45997712447)/(o+1.)+(0.181215330541)+(-0.128872747083)*0.5**o
        ref[2,4,2,0]=(0.282057449847)/(o+1.)+(0.49247247384)+(0.619034818105)*0.5**o
        ref[2,4,2,1]=(0.0852215499679)/(o+1.)+(0.0807247245641)+(0.645979236759)*0.5**o
        ref[3,0,0,0]=(-0.173183992256)/(o+1.)+(0.215110333109)+(-0.265836015694)*0.5**o
        ref[3,0,0,1]=(0.88174458564)/(o+1.)+(0.282289419133)+(0.306246512933)*0.5**o
        ref[3,0,1,0]=(-0.542918101609)/(o+1.)+(0.791816447658)+(0.509420593773)*0.5**o
        ref[3,0,1,1]=(0.585362427423)/(o+1.)+(-0.229639545331)+(-0.609626065482)*0.5**o
        ref[3,0,2,0]=(-0.625654730672)/(o+1.)+(0.690821759706)+(0.269212290711)*0.5**o
        ref[3,0,2,1]=(0.971993211744)/(o+1.)+(-1.18192725526)+(-0.445116541697)*0.5**o
        ref[3,1,0,0]=(-1.04249026287)/(o+1.)+(0.0447480055334)+(0.25585673506)*0.5**o
        ref[3,1,0,1]=(-1.30475672494)/(o+1.)+(0.700582633156)+(-0.159301665223)*0.5**o
        ref[3,1,1,0]=(0.25661740363)/(o+1.)+(-0.036124371669)+(0.673375237931)*0.5**o
        ref[3,1,1,1]=(1.24932735533)/(o+1.)+(-0.597633535843)+(0.273992606955)*0.5**o
        ref[3,1,2,0]=(0.116260708817)/(o+1.)+(0.813874160367)+(-0.533129977534)*0.5**o
        ref[3,1,2,1]=(1.30199813466)/(o+1.)+(0.377333042401)+(0.977824314677)*0.5**o
        ref[3,2,0,0]=(1.03411019508)/(o+1.)+(0.386114811649)+(0.969349250234)*0.5**o
        ref[3,2,0,1]=(0.114450341594)/(o+1.)+(0.288108255139)+(-0.673721580725)*0.5**o
        ref[3,2,1,0]=(0.799847650292)/(o+1.)+(-0.0983585840958)+(-0.432873699676)*0.5**o
        ref[3,2,1,1]=(-1.18814224634)/(o+1.)+(-0.696858644421)+(0.633726614581)*0.5**o
        ref[3,2,2,0]=(0.626385522485)/(o+1.)+(0.0881472172038)+(-0.182700878265)*0.5**o
        ref[3,2,2,1]=(0.27320795045)/(o+1.)+(0.0047710521094)+(0.999790796581)*0.5**o
        ref[3,3,0,0]=(0.0826963327674)/(o+1.)+(-0.666702246557)+(-0.959143897162)*0.5**o
        ref[3,3,0,1]=(0.320563187816)/(o+1.)+(0.774809717833)+(0.385513188376)*0.5**o
        ref[3,3,1,0]=(-1.80695590229)/(o+1.)+(0.214248941855)+(0.809269330969)*0.5**o
        ref[3,3,1,1]=(-0.146050130242)/(o+1.)+(-0.625770524147)+(-0.983474390608)*0.5**o
        ref[3,3,2,0]=(0.956779785308)/(o+1.)+(0.138176772609)+(0.710820041642)*0.5**o
        ref[3,3,2,1]=(-1.39320588501)/(o+1.)+(0.571745087644)+(0.633364771689)*0.5**o
        ref[3,4,0,0]=(0.828689828045)/(o+1.)+(-0.821398677259)+(0.44039719913)*0.5**o
        ref[3,4,0,1]=(1.50194037834)/(o+1.)+(-0.535574620947)+(0.253149910502)*0.5**o
        ref[3,4,1,0]=(-0.708775097383)/(o+1.)+(-0.337919894074)+(0.13027869782)*0.5**o
        ref[3,4,1,1]=(1.17387882267)/(o+1.)+(0.234301154085)+(0.476554240918)*0.5**o
        ref[3,4,2,0]=(0.891111463851)/(o+1.)+(0.221627765254)+(0.800058524963)*0.5**o
        ref[3,4,2,1]=(-0.0533896507631)/(o+1.)+(0.551652029667)+(-0.533834741079)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactZero_fromData_ReducedSolution_rank0(self):
      """
      tests integral of rank 0 Data on the FunctionOnContactZero

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports integral on FunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Data(0,(),w)
      if dim==2:
        arg=(-0.570707549273)*x[0]+(-0.726264032186)*x[1]
        ref=(-0.0423765208873)/(o+1.)+(-0.791542656673)+(0.328490252775)*0.5**o
      else:
        arg=(1.28222892931)*x[0]+(-0.798170675576)*x[1]+(1.5701208405)*x[2]
        ref=(0.721975372868)/(o+1.)+(0.344720905832)+(0.642761909705)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactZero_fromData_ReducedSolution_rank1(self):
      """
      tests integral of rank 1 Data on the FunctionOnContactZero

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports integral on FunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Data(0,(2,),w)
      ref=numpy.zeros((2,),numpy.float_)
      if dim==2:
        arg[0]=(0.575166906798)*x[0]+(-1.63698773418)*x[1]
        arg[1]=(-0.0535946584636)*x[0]+(0.105969101775)*x[1]
        ref[0]=(-0.751926426344)/(o+1.)+(-0.395800138888)+(0.481705876734)*0.5**o
        ref[1]=(0.698496252242)/(o+1.)+(-0.265927161866)+(-0.114267485199)*0.5**o
      else:
        arg[0]=(-0.17879254138)*x[0]+(-1.12752123792)*x[1]+(0.921297311605)*x[2]
        arg[1]=(1.33131824618)*x[0]+(-0.392175059176)*x[1]+(0.545754281465)*x[2]
        ref[0]=(-0.494870303061)/(o+1.)+(0.0697636237654)+(-0.0296734121631)*0.5**o
        ref[1]=(-0.394300983704)/(o+1.)+(0.722261312754)+(0.434675826665)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactZero_fromData_ReducedSolution_rank2(self):
      """
      tests integral of rank 2 Data on the FunctionOnContactZero

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports integral on FunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5),w)
      ref=numpy.zeros((4, 5),numpy.float_)
      if dim==2:
        arg[0,0]=(-0.338658191734)*x[0]+(1.46127609683)*x[1]
        arg[0,1]=(-0.629116867491)*x[0]+(1.40141569066)*x[1]
        arg[0,2]=(0.105569779347)*x[0]+(0.239936324967)*x[1]
        arg[0,3]=(-0.281718331718)*x[0]+(0.280323404866)*x[1]
        arg[0,4]=(0.467341113661)*x[0]+(-0.50452062075)*x[1]
        arg[1,0]=(-0.841295219186)*x[0]+(0.524038163723)*x[1]
        arg[1,1]=(1.40143704236)*x[0]+(-1.50467961637)*x[1]
        arg[1,2]=(1.21078790505)*x[0]+(-0.731453470292)*x[1]
        arg[1,3]=(0.469556922076)*x[0]+(0.0686293204391)*x[1]
        arg[1,4]=(-1.26394637585)*x[0]+(-0.0361395276585)*x[1]
        arg[2,0]=(-1.12850883148)*x[0]+(0.284296673326)*x[1]
        arg[2,1]=(-0.626362828813)*x[0]+(1.19459827573)*x[1]
        arg[2,2]=(-0.545457293083)*x[0]+(1.547337019)*x[1]
        arg[2,3]=(0.664461986629)*x[0]+(-1.69082198266)*x[1]
        arg[2,4]=(-0.100004763305)*x[0]+(0.152159044423)*x[1]
        arg[3,0]=(0.307574843061)*x[0]+(1.20209181301)*x[1]
        arg[3,1]=(-0.0663203356627)*x[0]+(-1.26669863527)*x[1]
        arg[3,2]=(-1.20436798894)*x[0]+(-0.0375693379661)*x[1]
        arg[3,3]=(0.753649087747)*x[0]+(0.69376223486)*x[1]
        arg[3,4]=(-0.0512732264834)*x[0]+(-0.658097606436)*x[1]
        ref[0,0]=(0.996821439952)/(o+1.)+(0.015088160779)+(0.0956201435879)*0.5**o
        ref[0,1]=(0.643092963726)/(o+1.)+(0.239390724844)+(-0.349575590244)*0.5**o
        ref[0,2]=(-0.198468008977)/(o+1.)+(0.00411201743342)+(0.535750078425)*0.5**o
        ref[0,3]=(0.642491201137)/(o+1.)+(-0.623413199652)+(0.602940271315)*0.5**o
        ref[0,4]=(-0.105706120549)/(o+1.)+(-0.353560054939)+(0.775646723338)*0.5**o
        ref[1,0]=(0.144540669221)/(o+1.)+(0.0228268215803)+(-0.507451367845)*0.5**o
        ref[1,1]=(-0.820379339897)/(o+1.)+(0.0951825678509)+(0.526771630185)*0.5**o
        ref[1,2]=(-0.21214585824)/(o+1.)+(-0.0889183062985)+(0.869316905591)*0.5**o
        ref[1,3]=(0.291228940674)/(o+1.)+(0.192623928703)+(-0.138290555565)*0.5**o
        ref[1,4]=(-0.0663718956434)/(o+1.)+(-0.309484335614)+(-0.614745336642)*0.5**o
        ref[2,0]=(0.484892962201)/(o+1.)+(-0.367129859793)+(-0.594845400763)*0.5**o
        ref[2,1]=(0.768457634672)/(o+1.)+(0.154447657562)+(-0.509117502876)*0.5**o
        ref[2,2]=(0.707320338363)/(o+1.)+(-0.0592149232183)+(0.412989233992)*0.5**o
        ref[2,3]=(-0.983684339171)/(o+1.)+(-0.495377645598)+(0.948079634337)*0.5**o
        ref[2,4]=(-0.0315947220465)/(o+1.)+(0.506833986759)+(-0.929918970353)*0.5**o
        ref[3,0]=(0.54141067098)/(o+1.)+(0.489049131493)+(-0.00984227789859)*0.5**o
        ref[3,1]=(-0.552335053051)/(o+1.)+(-0.383325638803)+(-0.0140326402734)*0.5**o
        ref[3,2]=(0.0821472412575)/(o+1.)+(-0.335827719718)+(-0.652429128733)*0.5**o
        ref[3,3]=(0.112917208924)/(o+1.)+(0.667772807338)+(-0.00105150099312)*0.5**o
        ref[3,4]=(-0.19331508157)/(o+1.)+(-0.102296385708)+(-0.311462979932)*0.5**o
      else:
        arg[0,0]=(0.069245963142)*x[0]+(-0.185727379494)*x[1]+(-0.887410889699)*x[2]
        arg[0,1]=(-0.816745668949)*x[0]+(-0.250435680662)*x[1]+(0.701607830582)*x[2]
        arg[0,2]=(-0.862433954308)*x[0]+(0.731874372527)*x[1]+(-0.115384982779)*x[2]
        arg[0,3]=(-0.265247631604)*x[0]+(-0.157059741958)*x[1]+(-0.368419335078)*x[2]
        arg[0,4]=(1.66720203567)*x[0]+(-0.759393708719)*x[1]+(1.36360850704)*x[2]
        arg[1,0]=(-0.192822813795)*x[0]+(0.0358776152527)*x[1]+(-0.722864520788)*x[2]
        arg[1,1]=(-0.480606215471)*x[0]+(1.13682774573)*x[1]+(0.17900467623)*x[2]
        arg[1,2]=(0.346760852584)*x[0]+(0.994833030281)*x[1]+(0.507055933913)*x[2]
        arg[1,3]=(-0.509961853478)*x[0]+(-0.852997575505)*x[1]+(0.586527030253)*x[2]
        arg[1,4]=(0.93957975846)*x[0]+(0.28878335152)*x[1]+(0.128070931738)*x[2]
        arg[2,0]=(0.338283306313)*x[0]+(0.0244500761608)*x[1]+(-0.606875554888)*x[2]
        arg[2,1]=(0.850782624552)*x[0]+(-0.836382038189)*x[1]+(-1.41808459947)*x[2]
        arg[2,2]=(-1.06176229613)*x[0]+(-0.259247046061)*x[1]+(-0.583455484089)*x[2]
        arg[2,3]=(1.17437923282)*x[0]+(1.31485601495)*x[1]+(1.3139743231)*x[2]
        arg[2,4]=(0.362851636275)*x[0]+(0.804049783156)*x[1]+(1.09205000214)*x[2]
        arg[3,0]=(0.343603837195)*x[0]+(-1.49082257981)*x[1]+(-0.628668674545)*x[2]
        arg[3,1]=(-0.676847711765)*x[0]+(1.32350560453)*x[1]+(1.07275185686)*x[2]
        arg[3,2]=(0.570784985469)*x[0]+(-1.01212360602)*x[1]+(-0.0808776667606)*x[2]
        arg[3,3]=(0.506392444236)*x[0]+(-1.00848318041)*x[1]+(0.558777901541)*x[2]
        arg[3,4]=(-0.60925738291)*x[0]+(-0.386755957809)*x[1]+(0.60930115434)*x[2]
        ref[0,0]=(-0.695230136419)/(o+1.)+(-0.388400729083)+(0.468139288533)*0.5**o
        ref[0,1]=(0.462098132187)/(o+1.)+(-0.215041893287)+(-0.397587864642)*0.5**o
        ref[0,2]=(-0.262411385363)/(o+1.)+(0.275729226874)+(-0.534991632946)*0.5**o
        ref[0,3]=(0.117822633378)/(o+1.)+(0.0286827897592)+(-0.965914921536)*0.5**o
        ref[0,4]=(0.293575681039)/(o+1.)+(0.584039692827)+(0.809761767299)*0.5**o
        ref[1,0]=(0.627951922463)/(o+1.)+(-0.994889162557)+(0.482016683321)*0.5**o
        ref[1,1]=(1.83482238662)/(o+1.)+(-0.205719558739)+(-0.588157062647)*0.5**o
        ref[1,2]=(1.13612020398)/(o+1.)+(0.0262824938134)+(0.659964625174)*0.5**o
        ref[1,3]=(0.325264487741)/(o+1.)+(-0.0752154813413)+(-0.951265923787)*0.5**o
        ref[1,4]=(-0.149919283302)/(o+1.)+(0.77701502585)+(-0.0476767266789)*0.5**o
        ref[2,0]=(0.491504686171)/(o+1.)+(-0.562534706926)+(0.389422555265)*0.5**o
        ref[2,1]=(-1.52704217417)/(o+1.)+(0.120525042098)+(-0.117691923134)*0.5**o
        ref[2,2]=(-0.779796772543)/(o+1.)+(-0.318864997267)+(-0.486938059201)*0.5**o
        ref[2,3]=(1.53343318682)/(o+1.)+(0.731149467308)+(0.807477449443)*0.5**o
        ref[2,4]=(1.15919207694)/(o+1.)+(0.113921448815)+(0.871916446996)*0.5**o
        ref[3,0]=(-1.75312920944)/(o+1.)+(-0.153798982308)+(0.284839756895)*0.5**o
        ref[3,1]=(1.30985814569)/(o+1.)+(0.438430874237)+(-0.467310144534)*0.5**o
        ref[3,2]=(-0.737480320972)/(o+1.)+(0.0604403094397)+(0.094383414777)*0.5**o
        ref[3,3]=(-0.688723266524)/(o+1.)+(0.43486027018)+(-0.124310108473)*0.5**o
        ref[3,4]=(0.128399262674)/(o+1.)+(-0.372445844122)+(0.229780239192)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactZero_fromData_ReducedSolution_rank3(self):
      """
      tests integral of rank 3 Data on the FunctionOnContactZero

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports integral on FunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Data(0,(6, 2, 2),w)
      ref=numpy.zeros((6, 2, 2),numpy.float_)
      if dim==2:
        arg[0,0,0]=(-0.993339371954)*x[0]+(-1.35308128985)*x[1]
        arg[0,0,1]=(-1.07886149134)*x[0]+(-1.38999283206)*x[1]
        arg[0,1,0]=(0.246468026972)*x[0]+(-1.37217464882)*x[1]
        arg[0,1,1]=(0.645672648599)*x[0]+(1.8070624103)*x[1]
        arg[1,0,0]=(-1.02278989388)*x[0]+(0.151634023103)*x[1]
        arg[1,0,1]=(-0.667332860541)*x[0]+(0.312742149193)*x[1]
        arg[1,1,0]=(-0.848647385832)*x[0]+(0.466438038516)*x[1]
        arg[1,1,1]=(-0.185495911372)*x[0]+(-0.137118786891)*x[1]
        arg[2,0,0]=(-0.0250555870731)*x[0]+(0.905553573283)*x[1]
        arg[2,0,1]=(-1.03940852939)*x[0]+(-1.57326016394)*x[1]
        arg[2,1,0]=(1.48834718811)*x[0]+(-0.340889253246)*x[1]
        arg[2,1,1]=(0.875721514739)*x[0]+(-0.310857412212)*x[1]
        arg[3,0,0]=(1.25129508831)*x[0]+(-1.69312043525)*x[1]
        arg[3,0,1]=(1.50124675312)*x[0]+(0.244195748654)*x[1]
        arg[3,1,0]=(0.292060478909)*x[0]+(-0.660005422696)*x[1]
        arg[3,1,1]=(0.791335231308)*x[0]+(0.466481573591)*x[1]
        arg[4,0,0]=(-0.361241947236)*x[0]+(-1.43591648587)*x[1]
        arg[4,0,1]=(-1.10385456695)*x[0]+(-0.325314195825)*x[1]
        arg[4,1,0]=(1.01004173033)*x[0]+(0.426824552857)*x[1]
        arg[4,1,1]=(-0.631382479612)*x[0]+(0.0603792190966)*x[1]
        arg[5,0,0]=(-0.30404368786)*x[0]+(-0.0102498522932)*x[1]
        arg[5,0,1]=(0.109188848746)*x[0]+(-1.16846160105)*x[1]
        arg[5,1,0]=(-0.351916879547)*x[0]+(-0.320714795448)*x[1]
        arg[5,1,1]=(-0.534138562652)*x[0]+(-0.256013589844)*x[1]
        ref[0,0,0]=(-0.506159375456)/(o+1.)+(-0.522895228783)+(-0.794470828783)*0.5**o
        ref[0,0,1]=(-0.930984367068)/(o+1.)+(-0.636851258685)+(-0.264167438968)*0.5**o
        ref[0,1,0]=(-0.674222792172)/(o+1.)+(-0.52131379048)+(0.591143751282)*0.5**o
        ref[0,1,1]=(0.953370751283)/(o+1.)+(0.377773224645)+(0.743817858326)*0.5**o
        ref[1,0,0]=(0.700508215428)/(o+1.)+(-0.303343530867)+(-0.964977024471)*0.5**o
        ref[1,0,1]=(-0.408185116452)/(o+1.)+(0.0668983569743)+(-0.0802023088445)*0.5**o
        ref[1,1,0]=(0.80742345527)/(o+1.)+(-0.405780705683)+(-0.37807139122)*0.5**o
        ref[1,1,1]=(0.0184615568095)/(o+1.)+(0.221921562266)+(-0.784919379605)*0.5**o
        ref[2,0,0]=(0.571906256883)/(o+1.)+(-0.0415588382133)+(0.391709405753)*0.5**o
        ref[2,0,1]=(-0.778176009451)/(o+1.)+(-0.761680990566)+(-0.311130702752)*0.5**o
        ref[2,1,0]=(0.498214686321)/(o+1.)+(-0.146173705483)+(0.941590659509)*0.5**o
        ref[2,1,1]=(-0.273747071818)/(o+1.)+(-0.0319184747153)+(0.902448123776)*0.5**o
        ref[3,0,0]=(-0.730029471178)/(o+1.)+(-0.288672747933)+(0.865549620109)*0.5**o
        ref[3,0,1]=(-0.371409186475)/(o+1.)+(0.623169375401)+(0.870512937445)*0.5**o
        ref[3,1,0]=(0.0611454391867)/(o+1.)+(-0.627723874214)+(0.826357365454)*0.5**o
        ref[3,1,1]=(0.785553447935)/(o+1.)+(0.246940213405)+(-0.021617069845)*0.5**o
        ref[4,0,0]=(-0.936201955907)/(o+1.)+(-0.484823720021)+(0.108690962842)*0.5**o
        ref[4,0,1]=(-0.277830390815)/(o+1.)+(-0.151786547071)+(-0.847765277819)*0.5**o
        ref[4,1,0]=(0.12293099533)/(o+1.)+(0.295620382054)+(0.722694523749)*0.5**o
        ref[4,1,1]=(-0.5517019185)/(o+1.)+(0.0850748937088)+(-0.189451129433)*0.5**o
        ref[5,0,0]=(-0.942533148805)/(o+1.)+(0.425680504872)+(-0.223121401092)*0.5**o
        ref[5,0,1]=(-0.377134228332)/(o+1.)+(-0.721019409569)+(0.759900295162)*0.5**o
        ref[5,1,0]=(-0.643828391942)/(o+1.)+(0.399974456311)+(-0.828752195674)*0.5**o
        ref[5,1,1]=(-0.118293731439)/(o+1.)+(-0.355568544431)+(0.0392786678054)*0.5**o
      else:
        arg[0,0,0]=(0.205446621454)*x[0]+(0.10254878667)*x[1]+(1.27673728979)*x[2]
        arg[0,0,1]=(0.758846313193)*x[0]+(0.63320336713)*x[1]+(-0.42638505807)*x[2]
        arg[0,1,0]=(-0.0127116561603)*x[0]+(-0.360359420027)*x[1]+(0.540508591493)*x[2]
        arg[0,1,1]=(-0.278918629538)*x[0]+(-0.0291534835772)*x[1]+(0.708655731565)*x[2]
        arg[1,0,0]=(-0.612965210483)*x[0]+(0.787787556912)*x[1]+(0.222384448462)*x[2]
        arg[1,0,1]=(1.07409083742)*x[0]+(1.19429173706)*x[1]+(-0.484979976779)*x[2]
        arg[1,1,0]=(-1.25714999157)*x[0]+(-0.321346019622)*x[1]+(0.278329320837)*x[2]
        arg[1,1,1]=(-0.512262689707)*x[0]+(1.6891992053)*x[1]+(-1.02351830068)*x[2]
        arg[2,0,0]=(-1.57911907773)*x[0]+(0.096124258122)*x[1]+(0.336619253521)*x[2]
        arg[2,0,1]=(0.0901144254199)*x[0]+(-1.36818186255)*x[1]+(1.2969609239)*x[2]
        arg[2,1,0]=(0.158329511499)*x[0]+(0.696997185217)*x[1]+(-0.400535235927)*x[2]
        arg[2,1,1]=(-1.44570853201)*x[0]+(0.237359587195)*x[1]+(-1.17323024801)*x[2]
        arg[3,0,0]=(-1.4325812824)*x[0]+(-0.733085265902)*x[1]+(0.125883186484)*x[2]
        arg[3,0,1]=(-0.0529889021435)*x[0]+(0.335154600942)*x[1]+(0.839337375209)*x[2]
        arg[3,1,0]=(-0.423552182963)*x[0]+(1.31813289315)*x[1]+(0.13235854915)*x[2]
        arg[3,1,1]=(0.53278572217)*x[0]+(0.159434073071)*x[1]+(-0.313621199136)*x[2]
        arg[4,0,0]=(0.747779887673)*x[0]+(0.30977483911)*x[1]+(-0.0760295896739)*x[2]
        arg[4,0,1]=(0.00295118909194)*x[0]+(-0.687914867842)*x[1]+(0.0386268422915)*x[2]
        arg[4,1,0]=(-0.276363532754)*x[0]+(0.52102466785)*x[1]+(-0.684840191917)*x[2]
        arg[4,1,1]=(0.0283138898538)*x[0]+(0.116042458294)*x[1]+(0.485578401528)*x[2]
        arg[5,0,0]=(-0.118402060192)*x[0]+(-1.18409809944)*x[1]+(-0.304048852342)*x[2]
        arg[5,0,1]=(0.174393790186)*x[0]+(-0.524912689263)*x[1]+(-0.648708392355)*x[2]
        arg[5,1,0]=(-0.254958756344)*x[0]+(-0.772071348728)*x[1]+(-0.0572494064027)*x[2]
        arg[5,1,1]=(-0.74777072018)*x[0]+(0.19109939364)*x[1]+(0.498973987814)*x[2]
        ref[0,0,0]=(0.986457444152)/(o+1.)+(0.258994989412)+(0.0802852749346)*0.5**o
        ref[0,0,1]=(-0.0286717143415)/(o+1.)+(0.554867138611)+(-0.115397940627)*0.5**o
        ref[0,1,0]=(-0.886482131138)/(o+1.)+(0.464293129749)+(0.125333386946)*0.5**o
        ref[0,1,1]=(0.199214164457)/(o+1.)+(0.336882900218)+(-0.472396346443)*0.5**o
        ref[1,0,0]=(0.36021680954)/(o+1.)+(0.408903924703)+(-0.780817864055)*0.5**o
        ref[1,0,1]=(0.44795581074)/(o+1.)+(0.472159267742)+(0.391128251477)*0.5**o
        ref[1,1,0]=(1.02665734893)/(o+1.)+(-0.71418625728)+(-0.898451524719)*0.5**o
        ref[1,1,1]=(0.0322676607777)/(o+1.)+(0.105378958385)+(-0.0896073626306)*0.5**o
        ref[2,0,0]=(1.15723247637)/(o+1.)+(-0.719604538702)+(-0.86439896505)*0.5**o
        ref[2,0,1]=(-0.254100181576)/(o+1.)+(-0.0387997401585)+(0.35059314866)*0.5**o
        ref[2,1,0]=(-0.636439167982)/(o+1.)+(0.563390681981)+(-0.0355507351918)*0.5**o
        ref[2,1,1]=(-0.265265225142)/(o+1.)+(-0.635620798949)+(-0.845072369786)*0.5**o
        ref[3,0,0]=(-0.062162800164)/(o+1.)+(-0.554499928282)+(-0.868620705087)*0.5**o
        ref[3,0,1]=(0.784681961813)/(o+1.)+(0.355202655131)+(-0.373584198067)*0.5**o
        ref[3,1,0]=(1.20272043429)/(o+1.)+(0.352677694103)+(-0.881136563162)*0.5**o
        ref[3,1,1]=(-0.311152797156)/(o+1.)+(0.313736563953)+(0.0622782653549)*0.5**o
        ref[4,0,0]=(-0.365517620238)/(o+1.)+(0.733502174253)+(-0.119961591158)*0.5**o
        ref[4,0,1]=(0.0155845896619)/(o+1.)+(-0.654963543839)+(0.648005661558)*0.5**o
        ref[4,1,0]=(0.405697975915)/(o+1.)+(-0.637805133954)+(0.429733235172)*0.5**o
        ref[4,1,1]=(-0.0469738326054)/(o+1.)+(0.0590721074184)+(0.558764367445)*0.5**o
        ref[5,0,0]=(-0.333825007954)/(o+1.)+(-0.46448520882)+(-0.343753586377)*0.5**o
        ref[5,0,1]=(-0.469125954795)/(o+1.)+(-0.147192309255)+(-0.235716718127)*0.5**o
        ref[5,1,0]=(-0.573275891777)/(o+1.)+(-0.478328163121)+(0.445652706545)*0.5**o
        ref[5,1,1]=(0.0545515234726)/(o+1.)+(-0.153919681346)+(0.195590500493)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactZero_fromData_ReducedSolution_rank4(self):
      """
      tests integral of rank 4 Data on the FunctionOnContactZero

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports integral on FunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5, 3, 2),w)
      ref=numpy.zeros((4, 5, 3, 2),numpy.float_)
      if dim==2:
        arg[0,0,0,0]=(-0.226142137728)*x[0]+(0.85681895183)*x[1]
        arg[0,0,0,1]=(-0.824407181762)*x[0]+(-0.290023144892)*x[1]
        arg[0,0,1,0]=(-1.25987816017)*x[0]+(0.824340990595)*x[1]
        arg[0,0,1,1]=(-0.0917811606765)*x[0]+(0.660211932964)*x[1]
        arg[0,0,2,0]=(-1.31073149178)*x[0]+(0.482324984918)*x[1]
        arg[0,0,2,1]=(0.256995458227)*x[0]+(0.79244254852)*x[1]
        arg[0,1,0,0]=(-0.0682147475407)*x[0]+(0.243620052061)*x[1]
        arg[0,1,0,1]=(0.071099982687)*x[0]+(-0.62986754897)*x[1]
        arg[0,1,1,0]=(-0.0716793733131)*x[0]+(-1.15235525814)*x[1]
        arg[0,1,1,1]=(0.193107575216)*x[0]+(-0.596149957543)*x[1]
        arg[0,1,2,0]=(-0.160142629886)*x[0]+(1.36581707067)*x[1]
        arg[0,1,2,1]=(-1.21477648376)*x[0]+(-0.732000298369)*x[1]
        arg[0,2,0,0]=(-1.02468971059)*x[0]+(0.330721595217)*x[1]
        arg[0,2,0,1]=(1.11414907391)*x[0]+(1.29386127812)*x[1]
        arg[0,2,1,0]=(1.1485850154)*x[0]+(0.387585977495)*x[1]
        arg[0,2,1,1]=(0.237382583072)*x[0]+(1.56248221279)*x[1]
        arg[0,2,2,0]=(-0.949129049229)*x[0]+(0.578823379063)*x[1]
        arg[0,2,2,1]=(-1.0667182537)*x[0]+(0.462227485959)*x[1]
        arg[0,3,0,0]=(1.37954377554)*x[0]+(0.541856880139)*x[1]
        arg[0,3,0,1]=(0.401584700319)*x[0]+(0.577102327147)*x[1]
        arg[0,3,1,0]=(-0.289796408108)*x[0]+(0.807853785325)*x[1]
        arg[0,3,1,1]=(0.0038467861822)*x[0]+(-0.931311510771)*x[1]
        arg[0,3,2,0]=(0.859036254821)*x[0]+(0.198884218516)*x[1]
        arg[0,3,2,1]=(-0.00310256070968)*x[0]+(0.511838475968)*x[1]
        arg[0,4,0,0]=(0.539686886802)*x[0]+(-0.324612924267)*x[1]
        arg[0,4,0,1]=(-0.0458655898565)*x[0]+(0.211810380966)*x[1]
        arg[0,4,1,0]=(0.302333533003)*x[0]+(0.0663033316831)*x[1]
        arg[0,4,1,1]=(-0.981519161639)*x[0]+(-0.00207756909914)*x[1]
        arg[0,4,2,0]=(0.283659447004)*x[0]+(-0.231526592979)*x[1]
        arg[0,4,2,1]=(-1.17679167875)*x[0]+(0.285698985756)*x[1]
        arg[1,0,0,0]=(-0.162900059582)*x[0]+(-0.877164340783)*x[1]
        arg[1,0,0,1]=(-0.50468605065)*x[0]+(-0.160440073709)*x[1]
        arg[1,0,1,0]=(-0.100901676644)*x[0]+(-0.296111526029)*x[1]
        arg[1,0,1,1]=(0.176107421869)*x[0]+(1.74393708745)*x[1]
        arg[1,0,2,0]=(-0.979807721703)*x[0]+(-0.229400841067)*x[1]
        arg[1,0,2,1]=(0.217472795645)*x[0]+(0.0881503243323)*x[1]
        arg[1,1,0,0]=(-0.127980123625)*x[0]+(0.0932095456541)*x[1]
        arg[1,1,0,1]=(0.829674410931)*x[0]+(-0.796196041601)*x[1]
        arg[1,1,1,0]=(1.28656110233)*x[0]+(-1.27850471392)*x[1]
        arg[1,1,1,1]=(1.159269884)*x[0]+(-0.973135433631)*x[1]
        arg[1,1,2,0]=(1.46274837856)*x[0]+(-1.81554270863)*x[1]
        arg[1,1,2,1]=(0.0973472511688)*x[0]+(0.551451261495)*x[1]
        arg[1,2,0,0]=(0.510667671293)*x[0]+(-0.51075982099)*x[1]
        arg[1,2,0,1]=(-1.41612093451)*x[0]+(0.614417669629)*x[1]
        arg[1,2,1,0]=(1.46595125796)*x[0]+(0.60374279122)*x[1]
        arg[1,2,1,1]=(-0.317185218187)*x[0]+(-0.0785532882696)*x[1]
        arg[1,2,2,0]=(-0.388504041993)*x[0]+(-0.838003398775)*x[1]
        arg[1,2,2,1]=(0.0497207129012)*x[0]+(-0.622155692247)*x[1]
        arg[1,3,0,0]=(-0.0471400179563)*x[0]+(1.3378948207)*x[1]
        arg[1,3,0,1]=(-1.22250822195)*x[0]+(0.629714783138)*x[1]
        arg[1,3,1,0]=(-1.28801080408)*x[0]+(-1.54079710038)*x[1]
        arg[1,3,1,1]=(-0.426412448469)*x[0]+(-1.09410474021)*x[1]
        arg[1,3,2,0]=(0.022834167351)*x[0]+(-0.497423389788)*x[1]
        arg[1,3,2,1]=(-0.787492124301)*x[0]+(-0.245621168419)*x[1]
        arg[1,4,0,0]=(-1.37814880919)*x[0]+(-0.704805848451)*x[1]
        arg[1,4,0,1]=(-1.12494770055)*x[0]+(1.44920024305)*x[1]
        arg[1,4,1,0]=(-0.809532572761)*x[0]+(1.16638587657)*x[1]
        arg[1,4,1,1]=(0.931784486816)*x[0]+(1.34629889453)*x[1]
        arg[1,4,2,0]=(-0.172430942793)*x[0]+(-0.584736998321)*x[1]
        arg[1,4,2,1]=(0.00542948736389)*x[0]+(0.43917597284)*x[1]
        arg[2,0,0,0]=(-0.946122337406)*x[0]+(-0.161940168137)*x[1]
        arg[2,0,0,1]=(-0.7856653779)*x[0]+(-0.75287701788)*x[1]
        arg[2,0,1,0]=(-1.08858367592)*x[0]+(-0.530380818694)*x[1]
        arg[2,0,1,1]=(-1.0573555095)*x[0]+(-0.854148765785)*x[1]
        arg[2,0,2,0]=(0.723412825328)*x[0]+(0.326761950437)*x[1]
        arg[2,0,2,1]=(-0.0143142161225)*x[0]+(-0.752724311876)*x[1]
        arg[2,1,0,0]=(-0.942548092355)*x[0]+(0.700861094897)*x[1]
        arg[2,1,0,1]=(-0.712286567957)*x[0]+(0.230406545738)*x[1]
        arg[2,1,1,0]=(1.27414839774)*x[0]+(0.724784649037)*x[1]
        arg[2,1,1,1]=(0.637494163002)*x[0]+(0.442489599192)*x[1]
        arg[2,1,2,0]=(1.81617631726)*x[0]+(1.56870218809)*x[1]
        arg[2,1,2,1]=(1.16419900666)*x[0]+(-0.044314729141)*x[1]
        arg[2,2,0,0]=(0.880267125196)*x[0]+(0.77124406259)*x[1]
        arg[2,2,0,1]=(0.482402867542)*x[0]+(0.0746924305017)*x[1]
        arg[2,2,1,0]=(0.105861259696)*x[0]+(0.484292207267)*x[1]
        arg[2,2,1,1]=(-0.107785838196)*x[0]+(0.701132243209)*x[1]
        arg[2,2,2,0]=(0.559717405844)*x[0]+(-1.5585942717)*x[1]
        arg[2,2,2,1]=(-0.989130712605)*x[0]+(0.221361137556)*x[1]
        arg[2,3,0,0]=(-0.045471818095)*x[0]+(-0.437941959792)*x[1]
        arg[2,3,0,1]=(1.25840468379)*x[0]+(-1.18872011057)*x[1]
        arg[2,3,1,0]=(1.70665829408)*x[0]+(0.705273921003)*x[1]
        arg[2,3,1,1]=(-0.0928709253845)*x[0]+(0.785115449756)*x[1]
        arg[2,3,2,0]=(-0.887243418838)*x[0]+(1.06790660642)*x[1]
        arg[2,3,2,1]=(-0.572644686872)*x[0]+(-0.672831948394)*x[1]
        arg[2,4,0,0]=(1.28282566801)*x[0]+(0.651985199447)*x[1]
        arg[2,4,0,1]=(0.0965663229335)*x[0]+(1.33819998052)*x[1]
        arg[2,4,1,0]=(0.205569041098)*x[0]+(-0.873095358733)*x[1]
        arg[2,4,1,1]=(-1.04452025745)*x[0]+(-0.204917033907)*x[1]
        arg[2,4,2,0]=(-0.181163938972)*x[0]+(0.0341993409272)*x[1]
        arg[2,4,2,1]=(1.68327027059)*x[0]+(-0.289436355642)*x[1]
        arg[3,0,0,0]=(1.05712810731)*x[0]+(0.915711819735)*x[1]
        arg[3,0,0,1]=(-0.171072039482)*x[0]+(-0.160656943623)*x[1]
        arg[3,0,1,0]=(-1.27063799869)*x[0]+(-0.605131642813)*x[1]
        arg[3,0,1,1]=(0.940512990835)*x[0]+(-0.116675577531)*x[1]
        arg[3,0,2,0]=(-0.194213869952)*x[0]+(0.364066752025)*x[1]
        arg[3,0,2,1]=(-0.328780465397)*x[0]+(-0.114689413673)*x[1]
        arg[3,1,0,0]=(-0.825914305895)*x[0]+(-0.167010056731)*x[1]
        arg[3,1,0,1]=(0.657779056207)*x[0]+(0.0666252451638)*x[1]
        arg[3,1,1,0]=(-0.464004426036)*x[0]+(-0.340932918409)*x[1]
        arg[3,1,1,1]=(-1.6698652667)*x[0]+(0.731870373141)*x[1]
        arg[3,1,2,0]=(1.39626250779)*x[0]+(-0.879152855557)*x[1]
        arg[3,1,2,1]=(1.34973919508)*x[0]+(-0.129753332761)*x[1]
        arg[3,2,0,0]=(1.03443450706)*x[0]+(0.807333477811)*x[1]
        arg[3,2,0,1]=(0.366011302596)*x[0]+(0.543804800926)*x[1]
        arg[3,2,1,0]=(0.262446846036)*x[0]+(-0.247786850991)*x[1]
        arg[3,2,1,1]=(-1.04657091196)*x[0]+(-0.485443426155)*x[1]
        arg[3,2,2,0]=(0.342434327538)*x[0]+(-0.148503585588)*x[1]
        arg[3,2,2,1]=(-0.189833429716)*x[0]+(0.424391874819)*x[1]
        arg[3,3,0,0]=(-0.118348838118)*x[0]+(0.397628264341)*x[1]
        arg[3,3,0,1]=(0.481440510084)*x[0]+(-0.393576221424)*x[1]
        arg[3,3,1,0]=(0.907533552208)*x[0]+(-0.668646549376)*x[1]
        arg[3,3,1,1]=(-1.49731714213)*x[0]+(-1.18710678264)*x[1]
        arg[3,3,2,0]=(-0.460660079207)*x[0]+(1.48078266444)*x[1]
        arg[3,3,2,1]=(-0.161073556988)*x[0]+(-0.486827042754)*x[1]
        arg[3,4,0,0]=(-0.668755455623)*x[0]+(0.345804059434)*x[1]
        arg[3,4,0,1]=(1.00528889193)*x[0]+(0.978507928191)*x[1]
        arg[3,4,1,0]=(0.98955856331)*x[0]+(1.0421852144)*x[1]
        arg[3,4,1,1]=(0.711688260873)*x[0]+(1.39962222322)*x[1]
        arg[3,4,2,0]=(1.08512429209)*x[0]+(-0.555249097743)*x[1]
        arg[3,4,2,1]=(-1.10443784007)*x[0]+(-0.782274429169)*x[1]
        ref[0,0,0,0]=(0.151153165564)/(o+1.)+(0.112609656739)+(0.254304335059)*0.5**o
        ref[0,0,0,1]=(-0.530602583511)/(o+1.)+(0.1817855378)+(-0.947398818743)*0.5**o
        ref[0,0,1,0]=(0.308747437738)/(o+1.)+(-0.0338129255643)+(-0.676658756184)*0.5**o
        ref[0,0,1,1]=(0.321026553427)/(o+1.)+(0.0692705367618)+(0.108863145337)*0.5**o
        ref[0,0,2,0]=(-0.0908034666655)/(o+1.)+(-0.1100205155)+(-0.517562009199)*0.5**o
        ref[0,0,2,1]=(0.303456040903)/(o+1.)+(0.500090875336)+(-0.254199784829)*0.5**o
        ref[0,1,0,0]=(-0.440641221831)/(o+1.)+(0.456218946873)+(-0.296391367396)*0.5**o
        ref[0,1,0,1]=(0.222628561762)/(o+1.)+(-0.162064412854)+(-0.457267302337)*0.5**o
        ref[0,1,1,0]=(-0.913361887408)/(o+1.)+(-0.356168674239)+(0.401664604431)*0.5**o
        ref[0,1,1,1]=(-0.543116639248)/(o+1.)+(-0.120902439422)+(0.381879135767)*0.5**o
        ref[0,1,2,0]=(0.993649262831)/(o+1.)+(0.384695858206)+(-0.55736653846)*0.5**o
        ref[0,1,2,1]=(-0.968077797451)/(o+1.)+(-0.187765942637)+(-0.603167099406)*0.5**o
        ref[0,2,0,0]=(0.873662019918)/(o+1.)+(-0.710635360033)+(-0.146359415223)*0.5**o
        ref[0,2,0,1]=(0.466408230693)/(o+1.)+(0.77626565285)+(0.389070815631)*0.5**o
        ref[0,2,1,0]=(-0.475010813579)/(o+1.)+(0.525435880006)+(0.960310046461)*0.5**o
        ref[0,2,1,1]=(0.697127520484)/(o+1.)+(0.357204276552)+(0.388328722275)*0.5**o
        ref[0,2,2,0]=(-0.108434738487)/(o+1.)+(0.356443300466)+(-0.97475753261)*0.5**o
        ref[0,2,2,1]=(0.952274964304)/(o+1.)+(-0.497379508976)+(-0.562006714089)*0.5**o
        ref[0,3,0,0]=(-0.0368012761671)/(o+1.)+(0.594000625518)+(0.770200680808)*0.5**o
        ref[0,3,0,1]=(0.450361861836)/(o+1.)+(0.182031052848)+(0.164263059934)*0.5**o
        ref[0,3,1,0]=(0.500714741956)/(o+1.)+(0.0479711343168)+(-0.0785996333735)*0.5**o
        ref[0,3,1,1]=(-0.949280938645)/(o+1.)+(0.452441874504)+(-0.88306753495)*0.5**o
        ref[0,3,2,0]=(0.186931712837)/(o+1.)+(0.222335175903)+(0.426318408694)*0.5**o
        ref[0,3,2,1]=(-0.288015428918)/(o+1.)+(0.80948657963)+(-0.822221815085)*0.5**o
        ref[0,4,0,0]=(0.670151184695)/(o+1.)+(-0.415883847013)+(0.376690471866)*0.5**o
        ref[0,4,0,1]=(-0.107106843992)/(o+1.)+(0.555375298435)+(-0.837698961768)*0.5**o
        ref[0,4,1,0]=(-0.299999096802)/(o+1.)+(0.668996782774)+(-0.669357604061)*0.5**o
        ref[0,4,1,1]=(0.778006183401)/(o+1.)+(-0.614473529032)+(-0.532655856076)*0.5**o
        ref[0,4,2,0]=(-0.250068389818)/(o+1.)+(0.156274314197)+(-0.0103473845504)*0.5**o
        ref[0,4,2,1]=(0.902125322216)/(o+1.)+(-0.616424567683)+(-0.560368879846)*0.5**o
        ref[1,0,0,0]=(-0.133690258295)/(o+1.)+(-0.715949360249)+(0.525524578427)*0.5**o
        ref[1,0,0,1]=(-0.788097689216)/(o+1.)+(0.0998405659256)+(-0.076709566994)*0.5**o
        ref[1,0,1,0]=(0.144280886187)/(o+1.)+(0.0775377290314)+(-0.696369546924)*0.5**o
        ref[1,0,1,1]=(0.824172196605)/(o+1.)+(0.561804029444)+(-0.0277357461773)*0.5**o
        ref[1,0,2,0]=(0.422564421802)/(o+1.)+(-0.60719307099)+(-0.417386842591)*0.5**o
        ref[1,0,2,1]=(0.778908672762)/(o+1.)+(0.0163446340765)+(-0.505974820938)*0.5**o
        ref[1,1,0,0]=(0.0574071047387)/(o+1.)+(-0.350514214742)+(0.608850746775)*0.5**o
        ref[1,1,0,1]=(-0.991660961792)/(o+1.)+(0.322084279455)+(0.380970772212)*0.5**o
        ref[1,1,1,0]=(-0.823761120268)/(o+1.)+(-0.0495419092898)+(0.930901327253)*0.5**o
        ref[1,1,1,1]=(-0.384729980569)/(o+1.)+(-0.0392583789163)+(0.649381188769)*0.5**o
        ref[1,1,2,0]=(-0.840365304069)/(o+1.)+(-0.0150512662777)+(0.517673506558)*0.5**o
        ref[1,1,2,1]=(0.555976660846)/(o+1.)+(-0.307265886898)+(0.707353625614)*0.5**o
        ref[1,2,0,0]=(-0.423056705491)/(o+1.)+(0.0559404393793)+(0.311083677036)*0.5**o
        ref[1,2,0,1]=(0.15409797664)/(o+1.)+(-0.0906637067519)+(-0.774473828018)*0.5**o
        ref[1,2,1,0]=(0.796718644088)/(o+1.)+(0.248012077098)+(0.776951250898)*0.5**o
        ref[1,2,1,1]=(-0.342830889263)/(o+1.)+(-0.282162737841)+(0.511417858487)*0.5**o
        ref[1,2,2,0]=(-0.722403016671)/(o+1.)+(-0.367059457954)+(0.230014491812)*0.5**o
        ref[1,2,2,1]=(-0.773685058881)/(o+1.)+(0.412807900451)+(-0.624365721366)*0.5**o
        ref[1,3,0,0]=(0.527481434758)/(o+1.)+(0.36602961117)+(0.031214145649)*0.5**o
        ref[1,3,0,1]=(0.0630927531585)/(o+1.)+(-0.164989701995)+(-0.325906787975)*0.5**o
        ref[1,3,1,0]=(-0.612390590593)/(o+1.)+(-0.78855996274)+(-0.63929738839)*0.5**o
        ref[1,3,1,1]=(-0.962656044265)/(o+1.)+(-0.00576535037253)+(-0.546330443673)*0.5**o
        ref[1,3,2,0]=(-0.947280922008)/(o+1.)+(-0.137449579996)+(0.747590859563)*0.5**o
        ref[1,3,2,1]=(-0.946623026202)/(o+1.)+(0.0787536550213)+(-0.24399757656)*0.5**o
        ref[1,4,0,0]=(-0.174471016943)/(o+1.)+(-0.636500556039)+(-0.635482528623)*0.5**o
        ref[1,4,0,1]=(0.806379391595)/(o+1.)+(-0.00304106577093)+(-0.476044717545)*0.5**o
        ref[1,4,1,0]=(0.729649125134)/(o+1.)+(0.0151477051616)+(-0.403091231646)*0.5**o
        ref[1,4,1,1]=(0.559976454745)/(o+1.)+(0.479295896421)+(0.75951513376)*0.5**o
        ref[1,4,2,0]=(0.272199270818)/(o+1.)+(-0.0929072274332)+(-0.843552757065)*0.5**o
        ref[1,4,2,1]=(-0.552663678377)/(o+1.)+(0.613784745778)+(-0.230300352975)*0.5**o
        ref[2,0,0,0]=(-0.174247908098)/(o+1.)+(-0.108873783282)+(-0.716067030882)*0.5**o
        ref[2,0,0,1]=(-0.300584841532)/(o+1.)+(-0.485363665493)+(-0.267230223262)*0.5**o
        ref[2,0,1,0]=(0.316261805154)/(o+1.)+(-0.523474106451)+(-0.888278086869)*0.5**o
        ref[2,0,1,1]=(-0.699622562861)/(o+1.)+(-0.151089538248)+(-0.909702635929)*0.5**o
        ref[2,0,2,0]=(0.728545639829)/(o+1.)+(0.192083148877)+(-0.0625371618174)*0.5**o
        ref[2,0,2,1]=(-0.22042267239)/(o+1.)+(-0.611974599324)+(0.677333343041)*0.5**o
        ref[2,1,0,0]=(-0.0734883811113)/(o+1.)+(0.36100831022)+(-0.890215236787)*0.5**o
        ref[2,1,0,1]=(0.121656829057)/(o+1.)+(-0.130801635799)+(-0.341933579678)*0.5**o
        ref[2,1,1,0]=(0.221353168629)/(o+1.)+(0.618635673483)+(0.540308531179)*0.5**o
        ref[2,1,1,1]=(0.0152580531273)/(o+1.)+(0.252173415542)+(0.560378877984)*0.5**o
        ref[2,1,2,0]=(0.580319777731)/(o+1.)+(0.90566612137)+(0.993226484879)*0.5**o
        ref[2,1,2,1]=(-0.710250320949)/(o+1.)+(0.742177021871)+(0.345780554724)*0.5**o
        ref[2,2,0,0]=(0.770701563507)/(o+1.)+(0.240823471881)+(0.399162680517)*0.5**o
        ref[2,2,0,1]=(0.76121761761)/(o+1.)+(0.128382656738)+(-0.460887633042)*0.5**o
        ref[2,2,1,0]=(-0.269854962057)/(o+1.)+(0.690854898294)+(-0.521701367567)*0.5**o
        ref[2,2,1,1]=(0.719518595712)/(o+1.)+(0.029596430244)+(-0.185365051187)*0.5**o
        ref[2,2,2,0]=(-0.671157761165)/(o+1.)+(-0.142688285012)+(-0.0423425346644)*0.5**o
        ref[2,2,2,1]=(-0.541156097112)/(o+1.)+(0.0765180918929)+(-0.379649661722)*0.5**o
        ref[2,3,0,0]=(-0.0604347109212)/(o+1.)+(-0.19403702073)+(-0.034905025507)*0.5**o
        ref[2,3,0,1]=(-0.793398507071)/(o+1.)+(0.255681280626)+(0.351720519037)*0.5**o
        ref[2,3,1,0]=(-0.267058413731)/(o+1.)+(0.852211426049)+(0.974567776717)*0.5**o
        ref[2,3,1,1]=(0.228396641207)/(o+1.)+(0.322922896735)+(-0.181997910305)*0.5**o
        ref[2,3,2,0]=(0.480987773937)/(o+1.)+(-0.197709028569)+(0.0950934707835)*0.5**o
        ref[2,3,2,1]=(-0.648934475027)/(o+1.)+(-0.418383297875)+(0.240224435511)*0.5**o
        ref[2,4,0,0]=(0.430020261528)/(o+1.)+(0.53421018733)+(0.436370231269)*0.5**o
        ref[2,4,0,1]=(0.906390660223)/(o+1.)+(-0.056057545856)+(0.640490734948)*0.5**o
        ref[2,4,1,0]=(-0.732099770943)/(o+1.)+(-0.0485027617732)+(0.161578976854)*0.5**o
        ref[2,4,1,1]=(-0.827495680848)/(o+1.)+(0.285044728936)+(-0.992031068384)*0.5**o
        ref[2,4,2,0]=(-0.919905471787)/(o+1.)+(0.656971386264)+(-0.541001898785)*0.5**o
        ref[2,4,2,1]=(0.161975875302)/(o+1.)+(0.272665206557)+(0.686527626533)*0.5**o
        ref[3,0,0,0]=(0.584120963259)/(o+1.)+(0.207930729804)+(0.972857504177)*0.5**o
        ref[3,0,0,1]=(0.315823141491)/(o+1.)+(-0.217561208386)+(-0.212429707823)*0.5**o
        ref[3,0,1,0]=(-0.732294996139)/(o+1.)+(-0.278653143596)+(-0.586168358175)*0.5**o
        ref[3,0,1,1]=(0.481321847225)/(o+1.)+(-0.070080648792)+(0.482676863662)*0.5**o
        ref[3,0,2,0]=(0.616679859076)/(o+1.)+(-0.418784546677)+(0.390742116352)*0.5**o
        ref[3,0,2,1]=(0.866778417448)/(o+1.)+(-0.686585656643)+(0.0629230167692)*0.5**o
        ref[3,1,0,0]=(0.531890054167)/(o+1.)+(-0.516187607633)+(-0.492439201527)*0.5**o
        ref[3,1,0,1]=(-0.512361639413)/(o+1.)+(0.499557182545)+(0.237651575694)*0.5**o
        ref[3,1,1,0]=(-0.67542965405)/(o+1.)+(0.274414249559)+(-0.678336189514)*0.5**o
        ref[3,1,1,1]=(0.371936242507)/(o+1.)+(-0.1636482466)+(-0.982634642862)*0.5**o
        ref[3,1,2,0]=(-0.799408831501)/(o+1.)+(0.458633399549)+(0.399251684639)*0.5**o
        ref[3,1,2,1]=(-0.0145253903686)/(o+1.)+(0.238472345905)+(0.757566560882)*0.5**o
        ref[3,2,0,0]=(-0.130519473112)/(o+1.)+(0.695277604837)+(0.581732248306)*0.5**o
        ref[3,2,0,1]=(0.632010536438)/(o+1.)+(0.123066205529)+(0.0316731560259)*0.5**o
        ref[3,2,1,0]=(-0.709926210356)/(o+1.)+(0.0926699972302)+(0.53924621094)*0.5**o
        ref[3,2,1,1]=(0.428094002247)/(o+1.)+(-0.841997926131)+(-0.276112488096)*0.5**o
        ref[3,2,2,0]=(0.708481148086)/(o+1.)+(-0.50498161534)+(0.495412824545)*0.5**o
        ref[3,2,2,1]=(0.782824622703)/(o+1.)+(-0.151509698687)+(-0.245246780227)*0.5**o
        ref[3,3,0,0]=(0.848460868418)/(o+1.)+(-0.104639248461)+(-0.359902945273)*0.5**o
        ref[3,3,0,1]=(0.0569151307298)/(o+1.)+(-0.0674224535469)+(0.165794065024)*0.5**o
        ref[3,3,1,0]=(-0.165683739908)/(o+1.)+(-0.183388699236)+(0.771348141212)*0.5**o
        ref[3,3,1,1]=(-0.932550058069)/(o+1.)+(-0.47828109001)+(-0.795311686688)*0.5**o
        ref[3,3,2,0]=(0.821542940923)/(o+1.)+(0.166853194487)+(-0.13512674466)*0.5**o
        ref[3,3,2,1]=(-0.46375871628)/(o+1.)+(0.132701267732)+(-0.449544418926)*0.5**o
        ref[3,4,0,0]=(-0.074530542168)/(o+1.)+(-0.0567877963797)+(-0.134845261261)*0.5**o
        ref[3,4,0,1]=(0.238671520479)/(o+1.)+(0.50689253572)+(0.731340228204)*0.5**o
        ref[3,4,1,0]=(0.981601513335)/(o+1.)+(0.400717209822)+(0.248707844726)*0.5**o
        ref[3,4,1,1]=(0.592304390107)/(o+1.)+(0.342408637328)+(0.834188819333)*0.5**o
        ref[3,4,2,0]=(-0.339991330909)/(o+1.)+(-0.0145922376156)+(0.899051000485)*0.5**o
        ref[3,4,2,1]=(-0.626610688164)/(o+1.)+(-0.417008165115)+(-0.426085250846)*0.5**o
      else:
        arg[0,0,0,0]=(0.697666818785)*x[0]+(1.00202295791)*x[1]+(0.617700157184)*x[2]
        arg[0,0,0,1]=(-0.465577819457)*x[0]+(-1.40054693355)*x[1]+(1.16912653295)*x[2]
        arg[0,0,1,0]=(-1.26696500634)*x[0]+(-0.0469756169975)*x[1]+(-0.445591738991)*x[2]
        arg[0,0,1,1]=(-1.61654321709)*x[0]+(-1.69522962467)*x[1]+(-1.06089094851)*x[2]
        arg[0,0,2,0]=(0.229468378506)*x[0]+(0.716813596991)*x[1]+(-0.237703937029)*x[2]
        arg[0,0,2,1]=(-1.12052051905)*x[0]+(0.835000004618)*x[1]+(-0.853004812404)*x[2]
        arg[0,1,0,0]=(-1.75500745705)*x[0]+(0.655055836882)*x[1]+(1.05207750735)*x[2]
        arg[0,1,0,1]=(-0.0746024824253)*x[0]+(-0.421596816958)*x[1]+(-0.375715855989)*x[2]
        arg[0,1,1,0]=(-0.963269684396)*x[0]+(0.431993248836)*x[1]+(-1.63061928649)*x[2]
        arg[0,1,1,1]=(-0.418757265018)*x[0]+(-0.665626168583)*x[1]+(-0.335734432507)*x[2]
        arg[0,1,2,0]=(0.980346404307)*x[0]+(1.45442049802)*x[1]+(-0.833420308612)*x[2]
        arg[0,1,2,1]=(-1.52536017847)*x[0]+(0.99484028681)*x[1]+(0.109754321663)*x[2]
        arg[0,2,0,0]=(1.697179262)*x[0]+(-0.928732802147)*x[1]+(-0.653379513626)*x[2]
        arg[0,2,0,1]=(0.682698902948)*x[0]+(0.829307959583)*x[1]+(-1.0300112276)*x[2]
        arg[0,2,1,0]=(0.026084323266)*x[0]+(1.28865784608)*x[1]+(0.830446580645)*x[2]
        arg[0,2,1,1]=(0.486291506032)*x[0]+(-0.519637742913)*x[1]+(0.81671330629)*x[2]
        arg[0,2,2,0]=(-0.761591624634)*x[0]+(0.370951131011)*x[1]+(-0.256748479312)*x[2]
        arg[0,2,2,1]=(-0.249974326513)*x[0]+(0.267323484635)*x[1]+(-1.09642130905)*x[2]
        arg[0,3,0,0]=(0.752057563255)*x[0]+(-1.2772262175)*x[1]+(-0.264569668983)*x[2]
        arg[0,3,0,1]=(-0.173167796447)*x[0]+(0.31873208356)*x[1]+(-0.599780701112)*x[2]
        arg[0,3,1,0]=(-0.912464014941)*x[0]+(-1.56452517166)*x[1]+(1.37651660499)*x[2]
        arg[0,3,1,1]=(0.302009667854)*x[0]+(0.524213064769)*x[1]+(0.316301522683)*x[2]
        arg[0,3,2,0]=(-1.2940154673)*x[0]+(-0.900061396992)*x[1]+(0.3860029847)*x[2]
        arg[0,3,2,1]=(-0.97354131341)*x[0]+(1.08221883964)*x[1]+(-0.739323076416)*x[2]
        arg[0,4,0,0]=(1.86535317192)*x[0]+(-0.478359589546)*x[1]+(-0.798014592302)*x[2]
        arg[0,4,0,1]=(-1.08518693851)*x[0]+(-0.186048654536)*x[1]+(1.04915573267)*x[2]
        arg[0,4,1,0]=(-1.09692811055)*x[0]+(1.01042920935)*x[1]+(-0.73978089555)*x[2]
        arg[0,4,1,1]=(0.654656964663)*x[0]+(0.279204596126)*x[1]+(0.886217781361)*x[2]
        arg[0,4,2,0]=(-0.54009886291)*x[0]+(0.378037248739)*x[1]+(1.09785568886)*x[2]
        arg[0,4,2,1]=(-1.43995958913)*x[0]+(-0.720157090994)*x[1]+(0.0676362028136)*x[2]
        arg[1,0,0,0]=(1.47735412608)*x[0]+(0.342355546228)*x[1]+(-0.0134502415508)*x[2]
        arg[1,0,0,1]=(-0.419536590369)*x[0]+(-1.61108470511)*x[1]+(1.51281979355)*x[2]
        arg[1,0,1,0]=(1.31765698245)*x[0]+(-0.7499566294)*x[1]+(0.934537503478)*x[2]
        arg[1,0,1,1]=(0.688467914883)*x[0]+(-1.32681280673)*x[1]+(-0.387405705058)*x[2]
        arg[1,0,2,0]=(1.37069587556)*x[0]+(0.540830111773)*x[1]+(0.477311068263)*x[2]
        arg[1,0,2,1]=(-0.572066213761)*x[0]+(-1.11950925494)*x[1]+(1.07939217872)*x[2]
        arg[1,1,0,0]=(0.564997105326)*x[0]+(-0.73130546389)*x[1]+(0.435910937304)*x[2]
        arg[1,1,0,1]=(0.513931739674)*x[0]+(-0.944043404826)*x[1]+(0.014114227468)*x[2]
        arg[1,1,1,0]=(-0.77850597256)*x[0]+(-0.445835136484)*x[1]+(0.157478357479)*x[2]
        arg[1,1,1,1]=(0.752431176866)*x[0]+(0.659759664199)*x[1]+(-0.859515433521)*x[2]
        arg[1,1,2,0]=(0.0693158730911)*x[0]+(-0.820997827204)*x[1]+(-0.714694436829)*x[2]
        arg[1,1,2,1]=(0.604674152847)*x[0]+(0.647261183132)*x[1]+(-1.21981352028)*x[2]
        arg[1,2,0,0]=(0.0997504695984)*x[0]+(-0.271549839083)*x[1]+(-0.573467835942)*x[2]
        arg[1,2,0,1]=(0.01861845581)*x[0]+(0.157410502406)*x[1]+(0.939257868776)*x[2]
        arg[1,2,1,0]=(1.44909894631)*x[0]+(0.116038517547)*x[1]+(-0.851940489233)*x[2]
        arg[1,2,1,1]=(1.34598841007)*x[0]+(1.79163125841)*x[1]+(0.719171044245)*x[2]
        arg[1,2,2,0]=(-0.214737791448)*x[0]+(-1.85810839095)*x[1]+(-0.240776599927)*x[2]
        arg[1,2,2,1]=(-0.574707645928)*x[0]+(-0.664173831956)*x[1]+(0.310262106696)*x[2]
        arg[1,3,0,0]=(0.296213453592)*x[0]+(-0.831112482555)*x[1]+(0.130075739128)*x[2]
        arg[1,3,0,1]=(-0.369510604457)*x[0]+(0.0373909697998)*x[1]+(1.6689129783)*x[2]
        arg[1,3,1,0]=(0.0922435881419)*x[0]+(-0.280519827004)*x[1]+(0.285933839232)*x[2]
        arg[1,3,1,1]=(-0.649253282272)*x[0]+(-0.460065044123)*x[1]+(-0.0711164948063)*x[2]
        arg[1,3,2,0]=(0.793313307581)*x[0]+(-0.0587956351149)*x[1]+(0.810487259739)*x[2]
        arg[1,3,2,1]=(-1.38668597055)*x[0]+(1.51396934198)*x[1]+(-1.13918550255)*x[2]
        arg[1,4,0,0]=(1.40078167737)*x[0]+(-1.61915436345)*x[1]+(0.421710091214)*x[2]
        arg[1,4,0,1]=(-1.37982015877)*x[0]+(1.30607371738)*x[1]+(-0.608888848419)*x[2]
        arg[1,4,1,0]=(-0.558087802805)*x[0]+(-0.214169499512)*x[1]+(-0.766242546503)*x[2]
        arg[1,4,1,1]=(-1.75503436584)*x[0]+(0.183604705451)*x[1]+(-0.148071484576)*x[2]
        arg[1,4,2,0]=(0.537942069142)*x[0]+(-0.00640749928252)*x[1]+(-0.277544629657)*x[2]
        arg[1,4,2,1]=(-0.652645833371)*x[0]+(-1.33439077026)*x[1]+(-0.226308197201)*x[2]
        arg[2,0,0,0]=(1.45646289279)*x[0]+(1.40842127675)*x[1]+(1.75078281495)*x[2]
        arg[2,0,0,1]=(0.867988651995)*x[0]+(1.35061812241)*x[1]+(-1.56520776323)*x[2]
        arg[2,0,1,0]=(0.549686523522)*x[0]+(-0.528085465813)*x[1]+(-0.994978820267)*x[2]
        arg[2,0,1,1]=(0.383595996552)*x[0]+(0.612989859087)*x[1]+(-0.560401247231)*x[2]
        arg[2,0,2,0]=(-0.341259919499)*x[0]+(-0.125491675042)*x[1]+(0.545572340757)*x[2]
        arg[2,0,2,1]=(-0.414269395434)*x[0]+(-0.433969881898)*x[1]+(-1.01820199328)*x[2]
        arg[2,1,0,0]=(-0.113575280389)*x[0]+(0.326916039942)*x[1]+(-0.576072890249)*x[2]
        arg[2,1,0,1]=(-0.296473969661)*x[0]+(0.195675099431)*x[1]+(-1.72378150831)*x[2]
        arg[2,1,1,0]=(-1.31575932758)*x[0]+(-0.122557423522)*x[1]+(0.53435027695)*x[2]
        arg[2,1,1,1]=(0.299435754608)*x[0]+(0.727140413706)*x[1]+(0.848931358475)*x[2]
        arg[2,1,2,0]=(0.2618965198)*x[0]+(-0.461672573339)*x[1]+(0.923592803172)*x[2]
        arg[2,1,2,1]=(-0.326817258523)*x[0]+(-0.0363300805442)*x[1]+(1.82049875918)*x[2]
        arg[2,2,0,0]=(-0.435113320888)*x[0]+(0.0257984353786)*x[1]+(0.287846577657)*x[2]
        arg[2,2,0,1]=(1.39178068279)*x[0]+(0.136781967539)*x[1]+(-1.38161590351)*x[2]
        arg[2,2,1,0]=(-0.887874199356)*x[0]+(-0.476769153654)*x[1]+(-0.618760485829)*x[2]
        arg[2,2,1,1]=(1.58883355098)*x[0]+(-0.889469047555)*x[1]+(-0.678486755843)*x[2]
        arg[2,2,2,0]=(0.237147029365)*x[0]+(-0.509883462401)*x[1]+(0.533163151551)*x[2]
        arg[2,2,2,1]=(0.66513913749)*x[0]+(1.30317030657)*x[1]+(0.804083365617)*x[2]
        arg[2,3,0,0]=(1.01531518403)*x[0]+(1.3753053143)*x[1]+(0.000877284235964)*x[2]
        arg[2,3,0,1]=(-0.40853642543)*x[0]+(0.154794268783)*x[1]+(-1.34306671117)*x[2]
        arg[2,3,1,0]=(0.43065284359)*x[0]+(-0.28965146009)*x[1]+(0.660248828789)*x[2]
        arg[2,3,1,1]=(-1.4334922681)*x[0]+(-0.0348802858609)*x[1]+(0.502051304974)*x[2]
        arg[2,3,2,0]=(-1.17399537487)*x[0]+(0.865787043946)*x[1]+(-0.0105949677478)*x[2]
        arg[2,3,2,1]=(0.63104158794)*x[0]+(0.436196079951)*x[1]+(0.498606623061)*x[2]
        arg[2,4,0,0]=(-1.44410177354)*x[0]+(0.513406326011)*x[1]+(0.699127199447)*x[2]
        arg[2,4,0,1]=(-0.981727023077)*x[0]+(0.484182120695)*x[1]+(0.667175832045)*x[2]
        arg[2,4,1,0]=(-1.3352602588)*x[0]+(0.626269023364)*x[1]+(0.813537646695)*x[2]
        arg[2,4,1,1]=(-0.932486930114)*x[0]+(0.299448807849)*x[1]+(0.195862014797)*x[2]
        arg[2,4,2,0]=(-0.0621489644806)*x[0]+(0.131979664158)*x[1]+(0.442029269778)*x[2]
        arg[2,4,2,1]=(-0.507508608633)*x[0]+(-1.28613226533)*x[1]+(-0.697920444023)*x[2]
        arg[3,0,0,0]=(-0.984942677585)*x[0]+(-1.58913299616)*x[1]+(-0.373026784822)*x[2]
        arg[3,0,0,1]=(0.616666427113)*x[0]+(-0.625162800471)*x[1]+(-0.796307290839)*x[2]
        arg[3,0,1,0]=(-0.48469203683)*x[0]+(-0.0572105166553)*x[1]+(0.809572383581)*x[2]
        arg[3,0,1,1]=(-0.487160309892)*x[0]+(0.77646454341)*x[1]+(-0.41947544573)*x[2]
        arg[3,0,2,0]=(-1.17902121554)*x[0]+(-0.244939532033)*x[1]+(-0.282994359476)*x[2]
        arg[3,0,2,1]=(0.33515909881)*x[0]+(0.146866561705)*x[1]+(1.81486098364)*x[2]
        arg[3,1,0,0]=(0.379977901341)*x[0]+(0.834531701517)*x[1]+(-0.0242027062166)*x[2]
        arg[3,1,0,1]=(0.184318292766)*x[0]+(1.08367911805)*x[1]+(-0.164531741594)*x[2]
        arg[3,1,1,0]=(-1.00967115996)*x[0]+(0.542699640149)*x[1]+(0.707616701259)*x[2]
        arg[3,1,1,1]=(0.856948233811)*x[0]+(-1.2186550297)*x[1]+(-0.034268798159)*x[2]
        arg[3,1,2,0]=(0.259470684477)*x[0]+(-1.03586983732)*x[1]+(-0.157275979341)*x[2]
        arg[3,1,2,1]=(-0.161954890658)*x[0]+(1.65853209885)*x[1]+(0.381113456719)*x[2]
        arg[3,2,0,0]=(-0.437413004681)*x[0]+(-0.730340273929)*x[1]+(-0.687689272494)*x[2]
        arg[3,2,0,1]=(-0.336618164118)*x[0]+(-1.24388666895)*x[1]+(-0.2198689254)*x[2]
        arg[3,2,1,0]=(0.0872793087887)*x[0]+(-0.336554573215)*x[1]+(-0.0597803523262)*x[2]
        arg[3,2,1,1]=(0.192805432018)*x[0]+(0.0540495582064)*x[1]+(0.846658042071)*x[2]
        arg[3,2,2,0]=(0.00794192604414)*x[0]+(-0.33504559053)*x[1]+(0.615701149787)*x[2]
        arg[3,2,2,1]=(0.210870961503)*x[0]+(1.14164095846)*x[1]+(-0.192785742791)*x[2]
        arg[3,3,0,0]=(-0.25895162041)*x[0]+(-0.225800436488)*x[1]+(-0.198007079838)*x[2]
        arg[3,3,0,1]=(-0.150363703231)*x[0]+(1.23870429535)*x[1]+(0.28635881247)*x[2]
        arg[3,3,1,0]=(-0.865980718216)*x[0]+(-0.0849853244335)*x[1]+(-0.292164217189)*x[2]
        arg[3,3,1,1]=(-0.350465968722)*x[0]+(0.09853736936)*x[1]+(-0.476711524665)*x[2]
        arg[3,3,2,0]=(0.311396300263)*x[0]+(-0.92437612034)*x[1]+(-0.757139962928)*x[2]
        arg[3,3,2,1]=(-0.912311919837)*x[0]+(0.267793631099)*x[1]+(0.4917345401)*x[2]
        arg[3,4,0,0]=(0.613662541753)*x[0]+(0.782757341361)*x[1]+(1.04209808158)*x[2]
        arg[3,4,0,1]=(0.492828613997)*x[0]+(1.11979095664)*x[1]+(-0.843545072354)*x[2]
        arg[3,4,1,0]=(-0.51939120556)*x[0]+(0.476004484286)*x[1]+(-0.516311785432)*x[2]
        arg[3,4,1,1]=(1.00423313567)*x[0]+(0.90776890388)*x[1]+(1.34423309497)*x[2]
        arg[3,4,2,0]=(-1.44069600826)*x[0]+(-1.39515290909)*x[1]+(1.19851590247)*x[2]
        arg[3,4,2,1]=(0.391851619877)*x[0]+(0.265529120553)*x[1]+(0.996610410913)*x[2]
        ref[0,0,0,0]=(1.19330263935)/(o+1.)+(0.55394275251)+(0.0162017895041)*0.5**o
        ref[0,0,0,1]=(-0.672351783684)/(o+1.)+(-0.187889622039)+(0.351132807711)*0.5**o
        ref[0,0,1,0]=(-1.27021996264)/(o+1.)+(0.236286530101)+(-0.961885459894)*0.5**o
        ref[0,0,1,1]=(-1.43442517107)/(o+1.)+(-1.00113374859)+(-0.935971122022)*0.5**o
        ref[0,0,2,0]=(-0.434813695127)/(o+1.)+(0.606673209583)+(-0.0699546855712)*0.5**o
        ref[0,0,2,1]=(-0.0434901247948)/(o+1.)+(-0.180911877983)+(-0.733211446073)*0.5**o
        ref[0,1,0,0]=(1.41514302182)/(o+1.)+(-0.247977413325)+(-0.967062307989)*0.5**o
        ref[0,1,0,1]=(0.69128100142)/(o+1.)+(-0.621609882923)+(-0.319976390946)*0.5**o
        ref[0,1,1,0]=(-0.881870429191)/(o+1.)+(-0.654075774082)+(0.0281262553057)*0.5**o
        ref[0,1,1,1]=(-1.27239116086)/(o+1.)+(-0.0854066429492)+(0.0230865806541)*0.5**o
        ref[0,1,2,0]=(-0.222663945351)/(o+1.)+(0.685312598747)+(0.453385341573)*0.5**o
        ref[0,1,2,1]=(0.101409038706)/(o+1.)+(0.151367946744)+(-0.824910502191)*0.5**o
        ref[0,2,0,0]=(-1.19509443892)/(o+1.)+(0.225437856866)+(0.85928567141)*0.5**o
        ref[0,2,0,1]=(0.285764347754)/(o+1.)+(-0.383432127133)+(0.963095541446)*0.5**o
        ref[0,2,1,0]=(0.728010227911)/(o+1.)+(0.983790919896)+(-0.550403317717)*0.5**o
        ref[0,2,1,1]=(0.563748259036)/(o+1.)+(0.165164845927)+(-0.110710881481)*0.5**o
        ref[0,2,2,0]=(-0.361502606429)/(o+1.)+(0.0276145482611)+(-0.341115463028)*0.5**o
        ref[0,2,2,1]=(-0.293320207933)/(o+1.)+(0.0336757304385)+(-0.853103403872)*0.5**o
        ref[0,3,0,0]=(-0.410233956527)/(o+1.)+(-0.474281126245)+(0.569057885785)*0.5**o
        ref[0,3,0,1]=(-0.835861970115)/(o+1.)+(0.0173715483175)+(0.346902459481)*0.5**o
        ref[0,3,1,0]=(0.0830466456187)/(o+1.)+(-0.288204681495)+(-0.607109864241)*0.5**o
        ref[0,3,1,1]=(1.07074130823)/(o+1.)+(-0.0437983012867)+(0.159379549652)*0.5**o
        ref[0,3,2,0]=(0.172301049609)/(o+1.)+(-0.491724874152)+(-0.996925180895)*0.5**o
        ref[0,3,2,1]=(-0.24891941221)/(o+1.)+(-0.03935856269)+(-0.3030090126)*0.5**o
        ref[0,4,0,0]=(-0.778575122429)/(o+1.)+(0.213085222078)+(0.941383668349)*0.5**o
        ref[0,4,0,1]=(0.880801627174)/(o+1.)+(-0.454451723401)+(-0.193978040747)*0.5**o
        ref[0,4,1,0]=(-0.835604724882)/(o+1.)+(0.127240834083)+(-0.245156740036)*0.5**o
        ref[0,4,1,1]=(1.62336716324)/(o+1.)+(-0.101085918464)+(0.398884015839)*0.5**o
        ref[0,4,2,0]=(1.16805827702)/(o+1.)+(0.211232591688)+(-0.654729385702)*0.5**o
        ref[0,4,2,1]=(-0.305424138957)/(o+1.)+(-0.431076572693)+(-0.924903192969)*0.5**o
        ref[1,0,0,0]=(-0.481596247529)/(o+1.)+(0.769902632559)+(0.748050413171)*0.5**o
        ref[1,0,0,1]=(0.177388661252)/(o+1.)+(-0.118766442613)+(-0.457657277952)*0.5**o
        ref[1,0,1,0]=(0.709890860372)/(o+1.)+(0.0668010230727)+(0.658744950012)*0.5**o
        ref[1,0,1,1]=(-1.45685781381)/(o+1.)+(-0.178674426089)+(0.788456069079)*0.5**o
        ref[1,0,2,0]=(0.194388185612)/(o+1.)+(0.649177543063)+(0.896093783855)*0.5**o
        ref[1,0,2,1]=(0.181561279507)/(o+1.)+(-0.200618291902)+(-0.392507985678)*0.5**o
        ref[1,1,0,0]=(-0.884376814027)/(o+1.)+(0.280382736159)+(0.593213920451)*0.5**o
        ref[1,1,0,1]=(-0.250709602218)/(o+1.)+(-0.492104211211)+(0.818920586956)*0.5**o
        ref[1,1,1,0]=(0.0529723694625)/(o+1.)+(-0.502047650638)+(-0.115739819752)*0.5**o
        ref[1,1,1,1]=(0.215812202957)/(o+1.)+(-0.308796450292)+(0.954456105171)*0.5**o
        ref[1,1,2,0]=(-0.991481050902)/(o+1.)+(0.162626252838)+(-0.800147845717)*0.5**o
        ref[1,1,2,1]=(-0.433989756951)/(o+1.)+(0.362776336712)+(-0.259441100777)*0.5**o
        ref[1,2,0,0]=(-0.930287052624)/(o+1.)+(-0.347752763108)+(0.880525373413)*0.5**o
        ref[1,2,0,1]=(0.698674852861)/(o+1.)+(-0.15119613196)+(0.719004238051)*0.5**o
        ref[1,2,1,0]=(-0.777216821365)/(o+1.)+(0.436336076297)+(0.617741643399)*0.5**o
        ref[1,2,1,1]=(1.09595223135)/(o+1.)+(0.978251130983)+(0.804336219412)*0.5**o
        ref[1,2,2,0]=(-0.651012630849)/(o+1.)+(-1.04894606664)+(0.43528198181)*0.5**o
        ref[1,2,2,1]=(-1.5227353883)/(o+1.)+(0.670019307557)+(-0.745922598001)*0.5**o
        ref[1,3,0,0]=(-0.750195637997)/(o+1.)+(0.231864245348)+(-0.118356142533)*0.5**o
        ref[1,3,0,1]=(1.17136226596)/(o+1.)+(-0.114976421389)+(0.395383920468)*0.5**o
        ref[1,3,1,0]=(-0.108070636769)/(o+1.)+(0.279145429234)+(-0.352562621327)*0.5**o
        ref[1,3,1,1]=(-0.144223481714)/(o+1.)+(-0.472954352493)+(-0.0903026345013)*0.5**o
        ref[1,3,2,0]=(-0.487564307786)/(o+1.)+(0.636973351449)+(0.758622537094)*0.5**o
        ref[1,3,2,1]=(0.390860965954)/(o+1.)+(-0.36647678165)+(-0.66980953377)*0.5**o
        ref[1,4,0,0]=(-1.04824984837)/(o+1.)+(0.213932231728)+(0.823722790054)*0.5**o
        ref[1,4,0,1]=(-0.0984653601336)/(o+1.)+(0.07867877705)+(-0.741527483773)*0.5**o
        ref[1,4,1,0]=(-0.146287867383)/(o+1.)+(-0.908951868158)+(0.425691754878)*0.5**o
        ref[1,4,1,1]=(-0.94160416554)/(o+1.)+(0.0459877010016)+(-0.869872381431)*0.5**o
        ref[1,4,2,0]=(0.0115742858952)/(o+1.)+(-0.222488399041)+(0.687392452389)*0.5**o
        ref[1,4,2,1]=(-0.956561936575)/(o+1.)+(-0.59770879042)+(-0.0613652834159)*0.5**o
        ref[2,0,0,0]=(1.83987381729)/(o+1.)+(0.956579317963)+(0.862634531274)*0.5**o
        ref[2,0,0,1]=(0.105446127421)/(o+1.)+(0.180661881191)+(0.186629121375)*0.5**o
        ref[2,0,1,0]=(0.0212167440766)/(o+1.)+(-0.337825588888)+(-0.318943328859)*0.5**o
        ref[2,0,1,1]=(0.744834715631)/(o+1.)+(-0.350058416663)+(0.391466726104)*0.5**o
        ref[2,0,2,0]=(-0.788776613856)/(o+1.)+(0.233776806014)+(0.400043748043)*0.5**o
        ref[2,0,2,1]=(-0.95346976307)/(o+1.)+(0.0274998652324)+(-0.967971238003)*0.5**o
        ref[2,1,0,0]=(-0.536051408942)/(o+1.)+(-0.208498016994)+(0.590315312233)*0.5**o
        ref[2,1,0,1]=(-1.04158508788)/(o+1.)+(-0.397885793101)+(0.0127762955396)*0.5**o
        ref[2,1,1,0]=(0.529914207253)/(o+1.)+(-0.547503443987)+(-0.338873793431)*0.5**o
        ref[2,1,1,1]=(0.62298670274)/(o+1.)+(0.449860048786)+(0.352800726478)*0.5**o
        ref[2,1,2,0]=(0.965292650595)/(o+1.)+(0.00980498639)+(-0.261085873742)*0.5**o
        ref[2,1,2,1]=(0.403285360803)/(o+1.)+(0.369160592269)+(0.315744874774)*0.5**o
        ref[2,2,0,0]=(-0.457764884724)/(o+1.)+(0.563657776706)+(-0.791018976541)*0.5**o
        ref[2,2,0,1]=(0.188151274044)/(o+1.)+(-0.476470545911)+(0.911736564588)*0.5**o
        ref[2,2,1,0]=(-0.558515085739)/(o+1.)+(-0.445583796553)+(-0.533721159994)*0.5**o
        ref[2,2,1,1]=(-0.431992337265)/(o+1.)+(-0.0957726221961)+(0.64441532924)*0.5**o
        ref[2,2,2,0]=(-0.159056461947)/(o+1.)+(-0.0326864536156)+(0.484856087693)*0.5**o
        ref[2,2,2,1]=(0.533116456037)/(o+1.)+(1.10197257725)+(0.0353311991352)*0.5**o
        ref[2,3,0,0]=(0.0570625147023)/(o+1.)+(0.74170779854)+(0.851019670777)*0.5**o
        ref[2,3,0,1]=(-0.495059904832)/(o+1.)+(-0.169954912181)+(-0.761839138623)*0.5**o
        ref[2,3,1,0]=(0.482375173314)/(o+1.)+(-0.19374838651)+(0.706371811996)*0.5**o
        ref[2,3,1,1]=(0.12928100721)/(o+1.)+(-0.317327862062)+(-0.460946532077)*0.5**o
        ref[2,3,2,0]=(-0.234362084763)/(o+1.)+(0.423486279181)+(-0.93141377227)*0.5**o
        ref[2,3,2,1]=(1.25586693002)/(o+1.)+(-0.127551329993)+(0.56508002092)*0.5**o
        ref[2,4,0,0]=(0.45235297749)/(o+1.)+(-0.0217537364788)+(-0.640413752615)*0.5**o
        ref[2,4,0,1]=(0.936809207384)/(o+1.)+(-0.151519610695)+(-0.464139056331)*0.5**o
        ref[2,4,1,0]=(0.867693208863)/(o+1.)+(0.105683574314)+(-0.974513946233)*0.5**o
        ref[2,4,1,1]=(-1.27821575101)/(o+1.)+(0.463163847311)+(-0.0852880510814)*0.5**o
        ref[2,4,2,0]=(-0.846224501366)/(o+1.)+(0.432619461409)+(0.492845548002)*0.5**o
        ref[2,4,2,1]=(-1.48923229995)/(o+1.)+(-0.505747396375)+(0.00916577472118)*0.5**o
        ref[3,0,0,0]=(-0.959215731121)/(o+1.)+(-0.904609108587)+(-0.178668510268)*0.5**o
        ref[3,0,0,1]=(-1.53181231865)/(o+1.)+(-0.0758543610408)+(0.878717376536)*0.5**o
        ref[3,0,1,0]=(1.18643238831)/(o+1.)+(-0.288825159097)+(-0.341112240015)*0.5**o
        ref[3,0,1,1]=(-0.0675694917031)/(o+1.)+(-0.168955703616)+(0.275309686724)*0.5**o
        ref[3,0,2,0]=(-0.663269860133)/(o+1.)+(-0.235102421885)+(-0.573480403148)*0.5**o
        ref[3,0,2,1]=(1.0969289778)/(o+1.)+(0.46603172821)+(0.267894209932)*0.5**o
        ref[3,1,0,0]=(-0.716870513559)/(o+1.)+(0.712039409283)+(0.483098591635)*0.5**o
        ref[3,1,0,1]=(0.0328531999411)/(o+1.)+(0.379319450988)+(0.311973567308)*0.5**o
        ref[3,1,1,0]=(1.08648208064)/(o+1.)+(-0.196831959338)+(-0.452172980519)*0.5**o
        ref[3,1,1,1]=(0.0813181502612)/(o+1.)+(-0.227364064527)+(-0.0225656152548)*0.5**o
        ref[3,1,2,0]=(-0.271000454408)/(o+1.)+(-0.765711687155)+(0.868748696531)*0.5**o
        ref[3,1,2,1]=(1.60569646253)/(o+1.)+(0.137437739736)+(-0.00288127709197)*0.5**o
        ref[3,2,0,0]=(-1.25887089059)/(o+1.)+(-0.525842290945)+(0.455112921376)*0.5**o
        ref[3,2,0,1]=(-1.63787880725)/(o+1.)+(0.00214706725405)+(-0.166789085722)*0.5**o
        ref[3,2,1,0]=(-1.82111307894)/(o+1.)+(0.561806270366)+(0.388444921454)*0.5**o
        ref[3,2,1,1]=(-0.489259906414)/(o+1.)+(0.582548398381)+(0.417676141947)*0.5**o
        ref[3,2,2,0]=(-0.0674375340391)/(o+1.)+(-0.00345549227349)+(0.362946003887)*0.5**o
        ref[3,2,2,1]=(0.662780682001)/(o+1.)+(-0.15453848218)+(0.806022459534)*0.5**o
        ref[3,3,0,0]=(-1.15795406515)/(o+1.)+(0.0926725746467)+(0.289849779118)*0.5**o
        ref[3,3,0,1]=(1.27520269547)/(o+1.)+(-0.286529818361)+(0.672556345847)*0.5**o
        ref[3,3,1,0]=(-0.343290651469)/(o+1.)+(-0.0269962100904)+(-0.845847188189)*0.5**o
        ref[3,3,1,1]=(0.497715053371)/(o+1.)+(-0.158577177766)+(-0.909200821866)*0.5**o
        ref[3,3,2,0]=(-0.952431538662)/(o+1.)+(0.0816482063045)+(-0.580984656952)*0.5**o
        ref[3,3,2,1]=(-0.176227469799)/(o+1.)+(0.0456157853688)+(-0.0677878495767)*0.5**o
        ref[3,4,0,0]=(0.976460483802)/(o+1.)+(0.347595506011)+(0.766866468871)*0.5**o
        ref[3,4,0,1]=(0.0358170327662)/(o+1.)+(-0.0991456259778)+(0.931548717473)*0.5**o
        ref[3,4,1,0]=(0.114544318514)/(o+1.)+(-0.201014865792)+(-0.272213093635)*0.5**o
        ref[3,4,1,1]=(0.554509925385)/(o+1.)+(1.23219332557)+(0.237338557994)*0.5**o
        ref[3,4,2,0]=(0.0803465444931)/(o+1.)+(-0.518257569997)+(-0.681164419371)*0.5**o
        ref[3,4,2,1]=(-0.0443609570522)/(o+1.)+(0.572813368096)+(0.552725372202)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactZero_fromData_ReducedContinuousFunction_rank0(self):
      """
      tests integral of rank 0 Data on the FunctionOnContactZero

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports integral on FunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(),w)
      if dim==2:
        arg=(-0.570707549273)*x[0]+(-0.726264032186)*x[1]
        ref=(-0.0423765208873)/(o+1.)+(-0.791542656673)+(0.328490252775)*0.5**o
      else:
        arg=(1.28222892931)*x[0]+(-0.798170675576)*x[1]+(1.5701208405)*x[2]
        ref=(0.721975372868)/(o+1.)+(0.344720905832)+(0.642761909705)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactZero_fromData_ReducedContinuousFunction_rank1(self):
      """
      tests integral of rank 1 Data on the FunctionOnContactZero

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports integral on FunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(2,),w)
      ref=numpy.zeros((2,),numpy.float_)
      if dim==2:
        arg[0]=(0.575166906798)*x[0]+(-1.63698773418)*x[1]
        arg[1]=(-0.0535946584636)*x[0]+(0.105969101775)*x[1]
        ref[0]=(-0.751926426344)/(o+1.)+(-0.395800138888)+(0.481705876734)*0.5**o
        ref[1]=(0.698496252242)/(o+1.)+(-0.265927161866)+(-0.114267485199)*0.5**o
      else:
        arg[0]=(-0.17879254138)*x[0]+(-1.12752123792)*x[1]+(0.921297311605)*x[2]
        arg[1]=(1.33131824618)*x[0]+(-0.392175059176)*x[1]+(0.545754281465)*x[2]
        ref[0]=(-0.494870303061)/(o+1.)+(0.0697636237654)+(-0.0296734121631)*0.5**o
        ref[1]=(-0.394300983704)/(o+1.)+(0.722261312754)+(0.434675826665)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactZero_fromData_ReducedContinuousFunction_rank2(self):
      """
      tests integral of rank 2 Data on the FunctionOnContactZero

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports integral on FunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5),w)
      ref=numpy.zeros((4, 5),numpy.float_)
      if dim==2:
        arg[0,0]=(-0.338658191734)*x[0]+(1.46127609683)*x[1]
        arg[0,1]=(-0.629116867491)*x[0]+(1.40141569066)*x[1]
        arg[0,2]=(0.105569779347)*x[0]+(0.239936324967)*x[1]
        arg[0,3]=(-0.281718331718)*x[0]+(0.280323404866)*x[1]
        arg[0,4]=(0.467341113661)*x[0]+(-0.50452062075)*x[1]
        arg[1,0]=(-0.841295219186)*x[0]+(0.524038163723)*x[1]
        arg[1,1]=(1.40143704236)*x[0]+(-1.50467961637)*x[1]
        arg[1,2]=(1.21078790505)*x[0]+(-0.731453470292)*x[1]
        arg[1,3]=(0.469556922076)*x[0]+(0.0686293204391)*x[1]
        arg[1,4]=(-1.26394637585)*x[0]+(-0.0361395276585)*x[1]
        arg[2,0]=(-1.12850883148)*x[0]+(0.284296673326)*x[1]
        arg[2,1]=(-0.626362828813)*x[0]+(1.19459827573)*x[1]
        arg[2,2]=(-0.545457293083)*x[0]+(1.547337019)*x[1]
        arg[2,3]=(0.664461986629)*x[0]+(-1.69082198266)*x[1]
        arg[2,4]=(-0.100004763305)*x[0]+(0.152159044423)*x[1]
        arg[3,0]=(0.307574843061)*x[0]+(1.20209181301)*x[1]
        arg[3,1]=(-0.0663203356627)*x[0]+(-1.26669863527)*x[1]
        arg[3,2]=(-1.20436798894)*x[0]+(-0.0375693379661)*x[1]
        arg[3,3]=(0.753649087747)*x[0]+(0.69376223486)*x[1]
        arg[3,4]=(-0.0512732264834)*x[0]+(-0.658097606436)*x[1]
        ref[0,0]=(0.996821439952)/(o+1.)+(0.015088160779)+(0.0956201435879)*0.5**o
        ref[0,1]=(0.643092963726)/(o+1.)+(0.239390724844)+(-0.349575590244)*0.5**o
        ref[0,2]=(-0.198468008977)/(o+1.)+(0.00411201743342)+(0.535750078425)*0.5**o
        ref[0,3]=(0.642491201137)/(o+1.)+(-0.623413199652)+(0.602940271315)*0.5**o
        ref[0,4]=(-0.105706120549)/(o+1.)+(-0.353560054939)+(0.775646723338)*0.5**o
        ref[1,0]=(0.144540669221)/(o+1.)+(0.0228268215803)+(-0.507451367845)*0.5**o
        ref[1,1]=(-0.820379339897)/(o+1.)+(0.0951825678509)+(0.526771630185)*0.5**o
        ref[1,2]=(-0.21214585824)/(o+1.)+(-0.0889183062985)+(0.869316905591)*0.5**o
        ref[1,3]=(0.291228940674)/(o+1.)+(0.192623928703)+(-0.138290555565)*0.5**o
        ref[1,4]=(-0.0663718956434)/(o+1.)+(-0.309484335614)+(-0.614745336642)*0.5**o
        ref[2,0]=(0.484892962201)/(o+1.)+(-0.367129859793)+(-0.594845400763)*0.5**o
        ref[2,1]=(0.768457634672)/(o+1.)+(0.154447657562)+(-0.509117502876)*0.5**o
        ref[2,2]=(0.707320338363)/(o+1.)+(-0.0592149232183)+(0.412989233992)*0.5**o
        ref[2,3]=(-0.983684339171)/(o+1.)+(-0.495377645598)+(0.948079634337)*0.5**o
        ref[2,4]=(-0.0315947220465)/(o+1.)+(0.506833986759)+(-0.929918970353)*0.5**o
        ref[3,0]=(0.54141067098)/(o+1.)+(0.489049131493)+(-0.00984227789859)*0.5**o
        ref[3,1]=(-0.552335053051)/(o+1.)+(-0.383325638803)+(-0.0140326402734)*0.5**o
        ref[3,2]=(0.0821472412575)/(o+1.)+(-0.335827719718)+(-0.652429128733)*0.5**o
        ref[3,3]=(0.112917208924)/(o+1.)+(0.667772807338)+(-0.00105150099312)*0.5**o
        ref[3,4]=(-0.19331508157)/(o+1.)+(-0.102296385708)+(-0.311462979932)*0.5**o
      else:
        arg[0,0]=(0.069245963142)*x[0]+(-0.185727379494)*x[1]+(-0.887410889699)*x[2]
        arg[0,1]=(-0.816745668949)*x[0]+(-0.250435680662)*x[1]+(0.701607830582)*x[2]
        arg[0,2]=(-0.862433954308)*x[0]+(0.731874372527)*x[1]+(-0.115384982779)*x[2]
        arg[0,3]=(-0.265247631604)*x[0]+(-0.157059741958)*x[1]+(-0.368419335078)*x[2]
        arg[0,4]=(1.66720203567)*x[0]+(-0.759393708719)*x[1]+(1.36360850704)*x[2]
        arg[1,0]=(-0.192822813795)*x[0]+(0.0358776152527)*x[1]+(-0.722864520788)*x[2]
        arg[1,1]=(-0.480606215471)*x[0]+(1.13682774573)*x[1]+(0.17900467623)*x[2]
        arg[1,2]=(0.346760852584)*x[0]+(0.994833030281)*x[1]+(0.507055933913)*x[2]
        arg[1,3]=(-0.509961853478)*x[0]+(-0.852997575505)*x[1]+(0.586527030253)*x[2]
        arg[1,4]=(0.93957975846)*x[0]+(0.28878335152)*x[1]+(0.128070931738)*x[2]
        arg[2,0]=(0.338283306313)*x[0]+(0.0244500761608)*x[1]+(-0.606875554888)*x[2]
        arg[2,1]=(0.850782624552)*x[0]+(-0.836382038189)*x[1]+(-1.41808459947)*x[2]
        arg[2,2]=(-1.06176229613)*x[0]+(-0.259247046061)*x[1]+(-0.583455484089)*x[2]
        arg[2,3]=(1.17437923282)*x[0]+(1.31485601495)*x[1]+(1.3139743231)*x[2]
        arg[2,4]=(0.362851636275)*x[0]+(0.804049783156)*x[1]+(1.09205000214)*x[2]
        arg[3,0]=(0.343603837195)*x[0]+(-1.49082257981)*x[1]+(-0.628668674545)*x[2]
        arg[3,1]=(-0.676847711765)*x[0]+(1.32350560453)*x[1]+(1.07275185686)*x[2]
        arg[3,2]=(0.570784985469)*x[0]+(-1.01212360602)*x[1]+(-0.0808776667606)*x[2]
        arg[3,3]=(0.506392444236)*x[0]+(-1.00848318041)*x[1]+(0.558777901541)*x[2]
        arg[3,4]=(-0.60925738291)*x[0]+(-0.386755957809)*x[1]+(0.60930115434)*x[2]
        ref[0,0]=(-0.695230136419)/(o+1.)+(-0.388400729083)+(0.468139288533)*0.5**o
        ref[0,1]=(0.462098132187)/(o+1.)+(-0.215041893287)+(-0.397587864642)*0.5**o
        ref[0,2]=(-0.262411385363)/(o+1.)+(0.275729226874)+(-0.534991632946)*0.5**o
        ref[0,3]=(0.117822633378)/(o+1.)+(0.0286827897592)+(-0.965914921536)*0.5**o
        ref[0,4]=(0.293575681039)/(o+1.)+(0.584039692827)+(0.809761767299)*0.5**o
        ref[1,0]=(0.627951922463)/(o+1.)+(-0.994889162557)+(0.482016683321)*0.5**o
        ref[1,1]=(1.83482238662)/(o+1.)+(-0.205719558739)+(-0.588157062647)*0.5**o
        ref[1,2]=(1.13612020398)/(o+1.)+(0.0262824938134)+(0.659964625174)*0.5**o
        ref[1,3]=(0.325264487741)/(o+1.)+(-0.0752154813413)+(-0.951265923787)*0.5**o
        ref[1,4]=(-0.149919283302)/(o+1.)+(0.77701502585)+(-0.0476767266789)*0.5**o
        ref[2,0]=(0.491504686171)/(o+1.)+(-0.562534706926)+(0.389422555265)*0.5**o
        ref[2,1]=(-1.52704217417)/(o+1.)+(0.120525042098)+(-0.117691923134)*0.5**o
        ref[2,2]=(-0.779796772543)/(o+1.)+(-0.318864997267)+(-0.486938059201)*0.5**o
        ref[2,3]=(1.53343318682)/(o+1.)+(0.731149467308)+(0.807477449443)*0.5**o
        ref[2,4]=(1.15919207694)/(o+1.)+(0.113921448815)+(0.871916446996)*0.5**o
        ref[3,0]=(-1.75312920944)/(o+1.)+(-0.153798982308)+(0.284839756895)*0.5**o
        ref[3,1]=(1.30985814569)/(o+1.)+(0.438430874237)+(-0.467310144534)*0.5**o
        ref[3,2]=(-0.737480320972)/(o+1.)+(0.0604403094397)+(0.094383414777)*0.5**o
        ref[3,3]=(-0.688723266524)/(o+1.)+(0.43486027018)+(-0.124310108473)*0.5**o
        ref[3,4]=(0.128399262674)/(o+1.)+(-0.372445844122)+(0.229780239192)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactZero_fromData_ReducedContinuousFunction_rank3(self):
      """
      tests integral of rank 3 Data on the FunctionOnContactZero

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports integral on FunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(6, 2, 2),w)
      ref=numpy.zeros((6, 2, 2),numpy.float_)
      if dim==2:
        arg[0,0,0]=(-0.993339371954)*x[0]+(-1.35308128985)*x[1]
        arg[0,0,1]=(-1.07886149134)*x[0]+(-1.38999283206)*x[1]
        arg[0,1,0]=(0.246468026972)*x[0]+(-1.37217464882)*x[1]
        arg[0,1,1]=(0.645672648599)*x[0]+(1.8070624103)*x[1]
        arg[1,0,0]=(-1.02278989388)*x[0]+(0.151634023103)*x[1]
        arg[1,0,1]=(-0.667332860541)*x[0]+(0.312742149193)*x[1]
        arg[1,1,0]=(-0.848647385832)*x[0]+(0.466438038516)*x[1]
        arg[1,1,1]=(-0.185495911372)*x[0]+(-0.137118786891)*x[1]
        arg[2,0,0]=(-0.0250555870731)*x[0]+(0.905553573283)*x[1]
        arg[2,0,1]=(-1.03940852939)*x[0]+(-1.57326016394)*x[1]
        arg[2,1,0]=(1.48834718811)*x[0]+(-0.340889253246)*x[1]
        arg[2,1,1]=(0.875721514739)*x[0]+(-0.310857412212)*x[1]
        arg[3,0,0]=(1.25129508831)*x[0]+(-1.69312043525)*x[1]
        arg[3,0,1]=(1.50124675312)*x[0]+(0.244195748654)*x[1]
        arg[3,1,0]=(0.292060478909)*x[0]+(-0.660005422696)*x[1]
        arg[3,1,1]=(0.791335231308)*x[0]+(0.466481573591)*x[1]
        arg[4,0,0]=(-0.361241947236)*x[0]+(-1.43591648587)*x[1]
        arg[4,0,1]=(-1.10385456695)*x[0]+(-0.325314195825)*x[1]
        arg[4,1,0]=(1.01004173033)*x[0]+(0.426824552857)*x[1]
        arg[4,1,1]=(-0.631382479612)*x[0]+(0.0603792190966)*x[1]
        arg[5,0,0]=(-0.30404368786)*x[0]+(-0.0102498522932)*x[1]
        arg[5,0,1]=(0.109188848746)*x[0]+(-1.16846160105)*x[1]
        arg[5,1,0]=(-0.351916879547)*x[0]+(-0.320714795448)*x[1]
        arg[5,1,1]=(-0.534138562652)*x[0]+(-0.256013589844)*x[1]
        ref[0,0,0]=(-0.506159375456)/(o+1.)+(-0.522895228783)+(-0.794470828783)*0.5**o
        ref[0,0,1]=(-0.930984367068)/(o+1.)+(-0.636851258685)+(-0.264167438968)*0.5**o
        ref[0,1,0]=(-0.674222792172)/(o+1.)+(-0.52131379048)+(0.591143751282)*0.5**o
        ref[0,1,1]=(0.953370751283)/(o+1.)+(0.377773224645)+(0.743817858326)*0.5**o
        ref[1,0,0]=(0.700508215428)/(o+1.)+(-0.303343530867)+(-0.964977024471)*0.5**o
        ref[1,0,1]=(-0.408185116452)/(o+1.)+(0.0668983569743)+(-0.0802023088445)*0.5**o
        ref[1,1,0]=(0.80742345527)/(o+1.)+(-0.405780705683)+(-0.37807139122)*0.5**o
        ref[1,1,1]=(0.0184615568095)/(o+1.)+(0.221921562266)+(-0.784919379605)*0.5**o
        ref[2,0,0]=(0.571906256883)/(o+1.)+(-0.0415588382133)+(0.391709405753)*0.5**o
        ref[2,0,1]=(-0.778176009451)/(o+1.)+(-0.761680990566)+(-0.311130702752)*0.5**o
        ref[2,1,0]=(0.498214686321)/(o+1.)+(-0.146173705483)+(0.941590659509)*0.5**o
        ref[2,1,1]=(-0.273747071818)/(o+1.)+(-0.0319184747153)+(0.902448123776)*0.5**o
        ref[3,0,0]=(-0.730029471178)/(o+1.)+(-0.288672747933)+(0.865549620109)*0.5**o
        ref[3,0,1]=(-0.371409186475)/(o+1.)+(0.623169375401)+(0.870512937445)*0.5**o
        ref[3,1,0]=(0.0611454391867)/(o+1.)+(-0.627723874214)+(0.826357365454)*0.5**o
        ref[3,1,1]=(0.785553447935)/(o+1.)+(0.246940213405)+(-0.021617069845)*0.5**o
        ref[4,0,0]=(-0.936201955907)/(o+1.)+(-0.484823720021)+(0.108690962842)*0.5**o
        ref[4,0,1]=(-0.277830390815)/(o+1.)+(-0.151786547071)+(-0.847765277819)*0.5**o
        ref[4,1,0]=(0.12293099533)/(o+1.)+(0.295620382054)+(0.722694523749)*0.5**o
        ref[4,1,1]=(-0.5517019185)/(o+1.)+(0.0850748937088)+(-0.189451129433)*0.5**o
        ref[5,0,0]=(-0.942533148805)/(o+1.)+(0.425680504872)+(-0.223121401092)*0.5**o
        ref[5,0,1]=(-0.377134228332)/(o+1.)+(-0.721019409569)+(0.759900295162)*0.5**o
        ref[5,1,0]=(-0.643828391942)/(o+1.)+(0.399974456311)+(-0.828752195674)*0.5**o
        ref[5,1,1]=(-0.118293731439)/(o+1.)+(-0.355568544431)+(0.0392786678054)*0.5**o
      else:
        arg[0,0,0]=(0.205446621454)*x[0]+(0.10254878667)*x[1]+(1.27673728979)*x[2]
        arg[0,0,1]=(0.758846313193)*x[0]+(0.63320336713)*x[1]+(-0.42638505807)*x[2]
        arg[0,1,0]=(-0.0127116561603)*x[0]+(-0.360359420027)*x[1]+(0.540508591493)*x[2]
        arg[0,1,1]=(-0.278918629538)*x[0]+(-0.0291534835772)*x[1]+(0.708655731565)*x[2]
        arg[1,0,0]=(-0.612965210483)*x[0]+(0.787787556912)*x[1]+(0.222384448462)*x[2]
        arg[1,0,1]=(1.07409083742)*x[0]+(1.19429173706)*x[1]+(-0.484979976779)*x[2]
        arg[1,1,0]=(-1.25714999157)*x[0]+(-0.321346019622)*x[1]+(0.278329320837)*x[2]
        arg[1,1,1]=(-0.512262689707)*x[0]+(1.6891992053)*x[1]+(-1.02351830068)*x[2]
        arg[2,0,0]=(-1.57911907773)*x[0]+(0.096124258122)*x[1]+(0.336619253521)*x[2]
        arg[2,0,1]=(0.0901144254199)*x[0]+(-1.36818186255)*x[1]+(1.2969609239)*x[2]
        arg[2,1,0]=(0.158329511499)*x[0]+(0.696997185217)*x[1]+(-0.400535235927)*x[2]
        arg[2,1,1]=(-1.44570853201)*x[0]+(0.237359587195)*x[1]+(-1.17323024801)*x[2]
        arg[3,0,0]=(-1.4325812824)*x[0]+(-0.733085265902)*x[1]+(0.125883186484)*x[2]
        arg[3,0,1]=(-0.0529889021435)*x[0]+(0.335154600942)*x[1]+(0.839337375209)*x[2]
        arg[3,1,0]=(-0.423552182963)*x[0]+(1.31813289315)*x[1]+(0.13235854915)*x[2]
        arg[3,1,1]=(0.53278572217)*x[0]+(0.159434073071)*x[1]+(-0.313621199136)*x[2]
        arg[4,0,0]=(0.747779887673)*x[0]+(0.30977483911)*x[1]+(-0.0760295896739)*x[2]
        arg[4,0,1]=(0.00295118909194)*x[0]+(-0.687914867842)*x[1]+(0.0386268422915)*x[2]
        arg[4,1,0]=(-0.276363532754)*x[0]+(0.52102466785)*x[1]+(-0.684840191917)*x[2]
        arg[4,1,1]=(0.0283138898538)*x[0]+(0.116042458294)*x[1]+(0.485578401528)*x[2]
        arg[5,0,0]=(-0.118402060192)*x[0]+(-1.18409809944)*x[1]+(-0.304048852342)*x[2]
        arg[5,0,1]=(0.174393790186)*x[0]+(-0.524912689263)*x[1]+(-0.648708392355)*x[2]
        arg[5,1,0]=(-0.254958756344)*x[0]+(-0.772071348728)*x[1]+(-0.0572494064027)*x[2]
        arg[5,1,1]=(-0.74777072018)*x[0]+(0.19109939364)*x[1]+(0.498973987814)*x[2]
        ref[0,0,0]=(0.986457444152)/(o+1.)+(0.258994989412)+(0.0802852749346)*0.5**o
        ref[0,0,1]=(-0.0286717143415)/(o+1.)+(0.554867138611)+(-0.115397940627)*0.5**o
        ref[0,1,0]=(-0.886482131138)/(o+1.)+(0.464293129749)+(0.125333386946)*0.5**o
        ref[0,1,1]=(0.199214164457)/(o+1.)+(0.336882900218)+(-0.472396346443)*0.5**o
        ref[1,0,0]=(0.36021680954)/(o+1.)+(0.408903924703)+(-0.780817864055)*0.5**o
        ref[1,0,1]=(0.44795581074)/(o+1.)+(0.472159267742)+(0.391128251477)*0.5**o
        ref[1,1,0]=(1.02665734893)/(o+1.)+(-0.71418625728)+(-0.898451524719)*0.5**o
        ref[1,1,1]=(0.0322676607777)/(o+1.)+(0.105378958385)+(-0.0896073626306)*0.5**o
        ref[2,0,0]=(1.15723247637)/(o+1.)+(-0.719604538702)+(-0.86439896505)*0.5**o
        ref[2,0,1]=(-0.254100181576)/(o+1.)+(-0.0387997401585)+(0.35059314866)*0.5**o
        ref[2,1,0]=(-0.636439167982)/(o+1.)+(0.563390681981)+(-0.0355507351918)*0.5**o
        ref[2,1,1]=(-0.265265225142)/(o+1.)+(-0.635620798949)+(-0.845072369786)*0.5**o
        ref[3,0,0]=(-0.062162800164)/(o+1.)+(-0.554499928282)+(-0.868620705087)*0.5**o
        ref[3,0,1]=(0.784681961813)/(o+1.)+(0.355202655131)+(-0.373584198067)*0.5**o
        ref[3,1,0]=(1.20272043429)/(o+1.)+(0.352677694103)+(-0.881136563162)*0.5**o
        ref[3,1,1]=(-0.311152797156)/(o+1.)+(0.313736563953)+(0.0622782653549)*0.5**o
        ref[4,0,0]=(-0.365517620238)/(o+1.)+(0.733502174253)+(-0.119961591158)*0.5**o
        ref[4,0,1]=(0.0155845896619)/(o+1.)+(-0.654963543839)+(0.648005661558)*0.5**o
        ref[4,1,0]=(0.405697975915)/(o+1.)+(-0.637805133954)+(0.429733235172)*0.5**o
        ref[4,1,1]=(-0.0469738326054)/(o+1.)+(0.0590721074184)+(0.558764367445)*0.5**o
        ref[5,0,0]=(-0.333825007954)/(o+1.)+(-0.46448520882)+(-0.343753586377)*0.5**o
        ref[5,0,1]=(-0.469125954795)/(o+1.)+(-0.147192309255)+(-0.235716718127)*0.5**o
        ref[5,1,0]=(-0.573275891777)/(o+1.)+(-0.478328163121)+(0.445652706545)*0.5**o
        ref[5,1,1]=(0.0545515234726)/(o+1.)+(-0.153919681346)+(0.195590500493)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactZero_fromData_ReducedContinuousFunction_rank4(self):
      """
      tests integral of rank 4 Data on the FunctionOnContactZero

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports integral on FunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5, 3, 2),w)
      ref=numpy.zeros((4, 5, 3, 2),numpy.float_)
      if dim==2:
        arg[0,0,0,0]=(-0.226142137728)*x[0]+(0.85681895183)*x[1]
        arg[0,0,0,1]=(-0.824407181762)*x[0]+(-0.290023144892)*x[1]
        arg[0,0,1,0]=(-1.25987816017)*x[0]+(0.824340990595)*x[1]
        arg[0,0,1,1]=(-0.0917811606765)*x[0]+(0.660211932964)*x[1]
        arg[0,0,2,0]=(-1.31073149178)*x[0]+(0.482324984918)*x[1]
        arg[0,0,2,1]=(0.256995458227)*x[0]+(0.79244254852)*x[1]
        arg[0,1,0,0]=(-0.0682147475407)*x[0]+(0.243620052061)*x[1]
        arg[0,1,0,1]=(0.071099982687)*x[0]+(-0.62986754897)*x[1]
        arg[0,1,1,0]=(-0.0716793733131)*x[0]+(-1.15235525814)*x[1]
        arg[0,1,1,1]=(0.193107575216)*x[0]+(-0.596149957543)*x[1]
        arg[0,1,2,0]=(-0.160142629886)*x[0]+(1.36581707067)*x[1]
        arg[0,1,2,1]=(-1.21477648376)*x[0]+(-0.732000298369)*x[1]
        arg[0,2,0,0]=(-1.02468971059)*x[0]+(0.330721595217)*x[1]
        arg[0,2,0,1]=(1.11414907391)*x[0]+(1.29386127812)*x[1]
        arg[0,2,1,0]=(1.1485850154)*x[0]+(0.387585977495)*x[1]
        arg[0,2,1,1]=(0.237382583072)*x[0]+(1.56248221279)*x[1]
        arg[0,2,2,0]=(-0.949129049229)*x[0]+(0.578823379063)*x[1]
        arg[0,2,2,1]=(-1.0667182537)*x[0]+(0.462227485959)*x[1]
        arg[0,3,0,0]=(1.37954377554)*x[0]+(0.541856880139)*x[1]
        arg[0,3,0,1]=(0.401584700319)*x[0]+(0.577102327147)*x[1]
        arg[0,3,1,0]=(-0.289796408108)*x[0]+(0.807853785325)*x[1]
        arg[0,3,1,1]=(0.0038467861822)*x[0]+(-0.931311510771)*x[1]
        arg[0,3,2,0]=(0.859036254821)*x[0]+(0.198884218516)*x[1]
        arg[0,3,2,1]=(-0.00310256070968)*x[0]+(0.511838475968)*x[1]
        arg[0,4,0,0]=(0.539686886802)*x[0]+(-0.324612924267)*x[1]
        arg[0,4,0,1]=(-0.0458655898565)*x[0]+(0.211810380966)*x[1]
        arg[0,4,1,0]=(0.302333533003)*x[0]+(0.0663033316831)*x[1]
        arg[0,4,1,1]=(-0.981519161639)*x[0]+(-0.00207756909914)*x[1]
        arg[0,4,2,0]=(0.283659447004)*x[0]+(-0.231526592979)*x[1]
        arg[0,4,2,1]=(-1.17679167875)*x[0]+(0.285698985756)*x[1]
        arg[1,0,0,0]=(-0.162900059582)*x[0]+(-0.877164340783)*x[1]
        arg[1,0,0,1]=(-0.50468605065)*x[0]+(-0.160440073709)*x[1]
        arg[1,0,1,0]=(-0.100901676644)*x[0]+(-0.296111526029)*x[1]
        arg[1,0,1,1]=(0.176107421869)*x[0]+(1.74393708745)*x[1]
        arg[1,0,2,0]=(-0.979807721703)*x[0]+(-0.229400841067)*x[1]
        arg[1,0,2,1]=(0.217472795645)*x[0]+(0.0881503243323)*x[1]
        arg[1,1,0,0]=(-0.127980123625)*x[0]+(0.0932095456541)*x[1]
        arg[1,1,0,1]=(0.829674410931)*x[0]+(-0.796196041601)*x[1]
        arg[1,1,1,0]=(1.28656110233)*x[0]+(-1.27850471392)*x[1]
        arg[1,1,1,1]=(1.159269884)*x[0]+(-0.973135433631)*x[1]
        arg[1,1,2,0]=(1.46274837856)*x[0]+(-1.81554270863)*x[1]
        arg[1,1,2,1]=(0.0973472511688)*x[0]+(0.551451261495)*x[1]
        arg[1,2,0,0]=(0.510667671293)*x[0]+(-0.51075982099)*x[1]
        arg[1,2,0,1]=(-1.41612093451)*x[0]+(0.614417669629)*x[1]
        arg[1,2,1,0]=(1.46595125796)*x[0]+(0.60374279122)*x[1]
        arg[1,2,1,1]=(-0.317185218187)*x[0]+(-0.0785532882696)*x[1]
        arg[1,2,2,0]=(-0.388504041993)*x[0]+(-0.838003398775)*x[1]
        arg[1,2,2,1]=(0.0497207129012)*x[0]+(-0.622155692247)*x[1]
        arg[1,3,0,0]=(-0.0471400179563)*x[0]+(1.3378948207)*x[1]
        arg[1,3,0,1]=(-1.22250822195)*x[0]+(0.629714783138)*x[1]
        arg[1,3,1,0]=(-1.28801080408)*x[0]+(-1.54079710038)*x[1]
        arg[1,3,1,1]=(-0.426412448469)*x[0]+(-1.09410474021)*x[1]
        arg[1,3,2,0]=(0.022834167351)*x[0]+(-0.497423389788)*x[1]
        arg[1,3,2,1]=(-0.787492124301)*x[0]+(-0.245621168419)*x[1]
        arg[1,4,0,0]=(-1.37814880919)*x[0]+(-0.704805848451)*x[1]
        arg[1,4,0,1]=(-1.12494770055)*x[0]+(1.44920024305)*x[1]
        arg[1,4,1,0]=(-0.809532572761)*x[0]+(1.16638587657)*x[1]
        arg[1,4,1,1]=(0.931784486816)*x[0]+(1.34629889453)*x[1]
        arg[1,4,2,0]=(-0.172430942793)*x[0]+(-0.584736998321)*x[1]
        arg[1,4,2,1]=(0.00542948736389)*x[0]+(0.43917597284)*x[1]
        arg[2,0,0,0]=(-0.946122337406)*x[0]+(-0.161940168137)*x[1]
        arg[2,0,0,1]=(-0.7856653779)*x[0]+(-0.75287701788)*x[1]
        arg[2,0,1,0]=(-1.08858367592)*x[0]+(-0.530380818694)*x[1]
        arg[2,0,1,1]=(-1.0573555095)*x[0]+(-0.854148765785)*x[1]
        arg[2,0,2,0]=(0.723412825328)*x[0]+(0.326761950437)*x[1]
        arg[2,0,2,1]=(-0.0143142161225)*x[0]+(-0.752724311876)*x[1]
        arg[2,1,0,0]=(-0.942548092355)*x[0]+(0.700861094897)*x[1]
        arg[2,1,0,1]=(-0.712286567957)*x[0]+(0.230406545738)*x[1]
        arg[2,1,1,0]=(1.27414839774)*x[0]+(0.724784649037)*x[1]
        arg[2,1,1,1]=(0.637494163002)*x[0]+(0.442489599192)*x[1]
        arg[2,1,2,0]=(1.81617631726)*x[0]+(1.56870218809)*x[1]
        arg[2,1,2,1]=(1.16419900666)*x[0]+(-0.044314729141)*x[1]
        arg[2,2,0,0]=(0.880267125196)*x[0]+(0.77124406259)*x[1]
        arg[2,2,0,1]=(0.482402867542)*x[0]+(0.0746924305017)*x[1]
        arg[2,2,1,0]=(0.105861259696)*x[0]+(0.484292207267)*x[1]
        arg[2,2,1,1]=(-0.107785838196)*x[0]+(0.701132243209)*x[1]
        arg[2,2,2,0]=(0.559717405844)*x[0]+(-1.5585942717)*x[1]
        arg[2,2,2,1]=(-0.989130712605)*x[0]+(0.221361137556)*x[1]
        arg[2,3,0,0]=(-0.045471818095)*x[0]+(-0.437941959792)*x[1]
        arg[2,3,0,1]=(1.25840468379)*x[0]+(-1.18872011057)*x[1]
        arg[2,3,1,0]=(1.70665829408)*x[0]+(0.705273921003)*x[1]
        arg[2,3,1,1]=(-0.0928709253845)*x[0]+(0.785115449756)*x[1]
        arg[2,3,2,0]=(-0.887243418838)*x[0]+(1.06790660642)*x[1]
        arg[2,3,2,1]=(-0.572644686872)*x[0]+(-0.672831948394)*x[1]
        arg[2,4,0,0]=(1.28282566801)*x[0]+(0.651985199447)*x[1]
        arg[2,4,0,1]=(0.0965663229335)*x[0]+(1.33819998052)*x[1]
        arg[2,4,1,0]=(0.205569041098)*x[0]+(-0.873095358733)*x[1]
        arg[2,4,1,1]=(-1.04452025745)*x[0]+(-0.204917033907)*x[1]
        arg[2,4,2,0]=(-0.181163938972)*x[0]+(0.0341993409272)*x[1]
        arg[2,4,2,1]=(1.68327027059)*x[0]+(-0.289436355642)*x[1]
        arg[3,0,0,0]=(1.05712810731)*x[0]+(0.915711819735)*x[1]
        arg[3,0,0,1]=(-0.171072039482)*x[0]+(-0.160656943623)*x[1]
        arg[3,0,1,0]=(-1.27063799869)*x[0]+(-0.605131642813)*x[1]
        arg[3,0,1,1]=(0.940512990835)*x[0]+(-0.116675577531)*x[1]
        arg[3,0,2,0]=(-0.194213869952)*x[0]+(0.364066752025)*x[1]
        arg[3,0,2,1]=(-0.328780465397)*x[0]+(-0.114689413673)*x[1]
        arg[3,1,0,0]=(-0.825914305895)*x[0]+(-0.167010056731)*x[1]
        arg[3,1,0,1]=(0.657779056207)*x[0]+(0.0666252451638)*x[1]
        arg[3,1,1,0]=(-0.464004426036)*x[0]+(-0.340932918409)*x[1]
        arg[3,1,1,1]=(-1.6698652667)*x[0]+(0.731870373141)*x[1]
        arg[3,1,2,0]=(1.39626250779)*x[0]+(-0.879152855557)*x[1]
        arg[3,1,2,1]=(1.34973919508)*x[0]+(-0.129753332761)*x[1]
        arg[3,2,0,0]=(1.03443450706)*x[0]+(0.807333477811)*x[1]
        arg[3,2,0,1]=(0.366011302596)*x[0]+(0.543804800926)*x[1]
        arg[3,2,1,0]=(0.262446846036)*x[0]+(-0.247786850991)*x[1]
        arg[3,2,1,1]=(-1.04657091196)*x[0]+(-0.485443426155)*x[1]
        arg[3,2,2,0]=(0.342434327538)*x[0]+(-0.148503585588)*x[1]
        arg[3,2,2,1]=(-0.189833429716)*x[0]+(0.424391874819)*x[1]
        arg[3,3,0,0]=(-0.118348838118)*x[0]+(0.397628264341)*x[1]
        arg[3,3,0,1]=(0.481440510084)*x[0]+(-0.393576221424)*x[1]
        arg[3,3,1,0]=(0.907533552208)*x[0]+(-0.668646549376)*x[1]
        arg[3,3,1,1]=(-1.49731714213)*x[0]+(-1.18710678264)*x[1]
        arg[3,3,2,0]=(-0.460660079207)*x[0]+(1.48078266444)*x[1]
        arg[3,3,2,1]=(-0.161073556988)*x[0]+(-0.486827042754)*x[1]
        arg[3,4,0,0]=(-0.668755455623)*x[0]+(0.345804059434)*x[1]
        arg[3,4,0,1]=(1.00528889193)*x[0]+(0.978507928191)*x[1]
        arg[3,4,1,0]=(0.98955856331)*x[0]+(1.0421852144)*x[1]
        arg[3,4,1,1]=(0.711688260873)*x[0]+(1.39962222322)*x[1]
        arg[3,4,2,0]=(1.08512429209)*x[0]+(-0.555249097743)*x[1]
        arg[3,4,2,1]=(-1.10443784007)*x[0]+(-0.782274429169)*x[1]
        ref[0,0,0,0]=(0.151153165564)/(o+1.)+(0.112609656739)+(0.254304335059)*0.5**o
        ref[0,0,0,1]=(-0.530602583511)/(o+1.)+(0.1817855378)+(-0.947398818743)*0.5**o
        ref[0,0,1,0]=(0.308747437738)/(o+1.)+(-0.0338129255643)+(-0.676658756184)*0.5**o
        ref[0,0,1,1]=(0.321026553427)/(o+1.)+(0.0692705367618)+(0.108863145337)*0.5**o
        ref[0,0,2,0]=(-0.0908034666655)/(o+1.)+(-0.1100205155)+(-0.517562009199)*0.5**o
        ref[0,0,2,1]=(0.303456040903)/(o+1.)+(0.500090875336)+(-0.254199784829)*0.5**o
        ref[0,1,0,0]=(-0.440641221831)/(o+1.)+(0.456218946873)+(-0.296391367396)*0.5**o
        ref[0,1,0,1]=(0.222628561762)/(o+1.)+(-0.162064412854)+(-0.457267302337)*0.5**o
        ref[0,1,1,0]=(-0.913361887408)/(o+1.)+(-0.356168674239)+(0.401664604431)*0.5**o
        ref[0,1,1,1]=(-0.543116639248)/(o+1.)+(-0.120902439422)+(0.381879135767)*0.5**o
        ref[0,1,2,0]=(0.993649262831)/(o+1.)+(0.384695858206)+(-0.55736653846)*0.5**o
        ref[0,1,2,1]=(-0.968077797451)/(o+1.)+(-0.187765942637)+(-0.603167099406)*0.5**o
        ref[0,2,0,0]=(0.873662019918)/(o+1.)+(-0.710635360033)+(-0.146359415223)*0.5**o
        ref[0,2,0,1]=(0.466408230693)/(o+1.)+(0.77626565285)+(0.389070815631)*0.5**o
        ref[0,2,1,0]=(-0.475010813579)/(o+1.)+(0.525435880006)+(0.960310046461)*0.5**o
        ref[0,2,1,1]=(0.697127520484)/(o+1.)+(0.357204276552)+(0.388328722275)*0.5**o
        ref[0,2,2,0]=(-0.108434738487)/(o+1.)+(0.356443300466)+(-0.97475753261)*0.5**o
        ref[0,2,2,1]=(0.952274964304)/(o+1.)+(-0.497379508976)+(-0.562006714089)*0.5**o
        ref[0,3,0,0]=(-0.0368012761671)/(o+1.)+(0.594000625518)+(0.770200680808)*0.5**o
        ref[0,3,0,1]=(0.450361861836)/(o+1.)+(0.182031052848)+(0.164263059934)*0.5**o
        ref[0,3,1,0]=(0.500714741956)/(o+1.)+(0.0479711343168)+(-0.0785996333735)*0.5**o
        ref[0,3,1,1]=(-0.949280938645)/(o+1.)+(0.452441874504)+(-0.88306753495)*0.5**o
        ref[0,3,2,0]=(0.186931712837)/(o+1.)+(0.222335175903)+(0.426318408694)*0.5**o
        ref[0,3,2,1]=(-0.288015428918)/(o+1.)+(0.80948657963)+(-0.822221815085)*0.5**o
        ref[0,4,0,0]=(0.670151184695)/(o+1.)+(-0.415883847013)+(0.376690471866)*0.5**o
        ref[0,4,0,1]=(-0.107106843992)/(o+1.)+(0.555375298435)+(-0.837698961768)*0.5**o
        ref[0,4,1,0]=(-0.299999096802)/(o+1.)+(0.668996782774)+(-0.669357604061)*0.5**o
        ref[0,4,1,1]=(0.778006183401)/(o+1.)+(-0.614473529032)+(-0.532655856076)*0.5**o
        ref[0,4,2,0]=(-0.250068389818)/(o+1.)+(0.156274314197)+(-0.0103473845504)*0.5**o
        ref[0,4,2,1]=(0.902125322216)/(o+1.)+(-0.616424567683)+(-0.560368879846)*0.5**o
        ref[1,0,0,0]=(-0.133690258295)/(o+1.)+(-0.715949360249)+(0.525524578427)*0.5**o
        ref[1,0,0,1]=(-0.788097689216)/(o+1.)+(0.0998405659256)+(-0.076709566994)*0.5**o
        ref[1,0,1,0]=(0.144280886187)/(o+1.)+(0.0775377290314)+(-0.696369546924)*0.5**o
        ref[1,0,1,1]=(0.824172196605)/(o+1.)+(0.561804029444)+(-0.0277357461773)*0.5**o
        ref[1,0,2,0]=(0.422564421802)/(o+1.)+(-0.60719307099)+(-0.417386842591)*0.5**o
        ref[1,0,2,1]=(0.778908672762)/(o+1.)+(0.0163446340765)+(-0.505974820938)*0.5**o
        ref[1,1,0,0]=(0.0574071047387)/(o+1.)+(-0.350514214742)+(0.608850746775)*0.5**o
        ref[1,1,0,1]=(-0.991660961792)/(o+1.)+(0.322084279455)+(0.380970772212)*0.5**o
        ref[1,1,1,0]=(-0.823761120268)/(o+1.)+(-0.0495419092898)+(0.930901327253)*0.5**o
        ref[1,1,1,1]=(-0.384729980569)/(o+1.)+(-0.0392583789163)+(0.649381188769)*0.5**o
        ref[1,1,2,0]=(-0.840365304069)/(o+1.)+(-0.0150512662777)+(0.517673506558)*0.5**o
        ref[1,1,2,1]=(0.555976660846)/(o+1.)+(-0.307265886898)+(0.707353625614)*0.5**o
        ref[1,2,0,0]=(-0.423056705491)/(o+1.)+(0.0559404393793)+(0.311083677036)*0.5**o
        ref[1,2,0,1]=(0.15409797664)/(o+1.)+(-0.0906637067519)+(-0.774473828018)*0.5**o
        ref[1,2,1,0]=(0.796718644088)/(o+1.)+(0.248012077098)+(0.776951250898)*0.5**o
        ref[1,2,1,1]=(-0.342830889263)/(o+1.)+(-0.282162737841)+(0.511417858487)*0.5**o
        ref[1,2,2,0]=(-0.722403016671)/(o+1.)+(-0.367059457954)+(0.230014491812)*0.5**o
        ref[1,2,2,1]=(-0.773685058881)/(o+1.)+(0.412807900451)+(-0.624365721366)*0.5**o
        ref[1,3,0,0]=(0.527481434758)/(o+1.)+(0.36602961117)+(0.031214145649)*0.5**o
        ref[1,3,0,1]=(0.0630927531585)/(o+1.)+(-0.164989701995)+(-0.325906787975)*0.5**o
        ref[1,3,1,0]=(-0.612390590593)/(o+1.)+(-0.78855996274)+(-0.63929738839)*0.5**o
        ref[1,3,1,1]=(-0.962656044265)/(o+1.)+(-0.00576535037253)+(-0.546330443673)*0.5**o
        ref[1,3,2,0]=(-0.947280922008)/(o+1.)+(-0.137449579996)+(0.747590859563)*0.5**o
        ref[1,3,2,1]=(-0.946623026202)/(o+1.)+(0.0787536550213)+(-0.24399757656)*0.5**o
        ref[1,4,0,0]=(-0.174471016943)/(o+1.)+(-0.636500556039)+(-0.635482528623)*0.5**o
        ref[1,4,0,1]=(0.806379391595)/(o+1.)+(-0.00304106577093)+(-0.476044717545)*0.5**o
        ref[1,4,1,0]=(0.729649125134)/(o+1.)+(0.0151477051616)+(-0.403091231646)*0.5**o
        ref[1,4,1,1]=(0.559976454745)/(o+1.)+(0.479295896421)+(0.75951513376)*0.5**o
        ref[1,4,2,0]=(0.272199270818)/(o+1.)+(-0.0929072274332)+(-0.843552757065)*0.5**o
        ref[1,4,2,1]=(-0.552663678377)/(o+1.)+(0.613784745778)+(-0.230300352975)*0.5**o
        ref[2,0,0,0]=(-0.174247908098)/(o+1.)+(-0.108873783282)+(-0.716067030882)*0.5**o
        ref[2,0,0,1]=(-0.300584841532)/(o+1.)+(-0.485363665493)+(-0.267230223262)*0.5**o
        ref[2,0,1,0]=(0.316261805154)/(o+1.)+(-0.523474106451)+(-0.888278086869)*0.5**o
        ref[2,0,1,1]=(-0.699622562861)/(o+1.)+(-0.151089538248)+(-0.909702635929)*0.5**o
        ref[2,0,2,0]=(0.728545639829)/(o+1.)+(0.192083148877)+(-0.0625371618174)*0.5**o
        ref[2,0,2,1]=(-0.22042267239)/(o+1.)+(-0.611974599324)+(0.677333343041)*0.5**o
        ref[2,1,0,0]=(-0.0734883811113)/(o+1.)+(0.36100831022)+(-0.890215236787)*0.5**o
        ref[2,1,0,1]=(0.121656829057)/(o+1.)+(-0.130801635799)+(-0.341933579678)*0.5**o
        ref[2,1,1,0]=(0.221353168629)/(o+1.)+(0.618635673483)+(0.540308531179)*0.5**o
        ref[2,1,1,1]=(0.0152580531273)/(o+1.)+(0.252173415542)+(0.560378877984)*0.5**o
        ref[2,1,2,0]=(0.580319777731)/(o+1.)+(0.90566612137)+(0.993226484879)*0.5**o
        ref[2,1,2,1]=(-0.710250320949)/(o+1.)+(0.742177021871)+(0.345780554724)*0.5**o
        ref[2,2,0,0]=(0.770701563507)/(o+1.)+(0.240823471881)+(0.399162680517)*0.5**o
        ref[2,2,0,1]=(0.76121761761)/(o+1.)+(0.128382656738)+(-0.460887633042)*0.5**o
        ref[2,2,1,0]=(-0.269854962057)/(o+1.)+(0.690854898294)+(-0.521701367567)*0.5**o
        ref[2,2,1,1]=(0.719518595712)/(o+1.)+(0.029596430244)+(-0.185365051187)*0.5**o
        ref[2,2,2,0]=(-0.671157761165)/(o+1.)+(-0.142688285012)+(-0.0423425346644)*0.5**o
        ref[2,2,2,1]=(-0.541156097112)/(o+1.)+(0.0765180918929)+(-0.379649661722)*0.5**o
        ref[2,3,0,0]=(-0.0604347109212)/(o+1.)+(-0.19403702073)+(-0.034905025507)*0.5**o
        ref[2,3,0,1]=(-0.793398507071)/(o+1.)+(0.255681280626)+(0.351720519037)*0.5**o
        ref[2,3,1,0]=(-0.267058413731)/(o+1.)+(0.852211426049)+(0.974567776717)*0.5**o
        ref[2,3,1,1]=(0.228396641207)/(o+1.)+(0.322922896735)+(-0.181997910305)*0.5**o
        ref[2,3,2,0]=(0.480987773937)/(o+1.)+(-0.197709028569)+(0.0950934707835)*0.5**o
        ref[2,3,2,1]=(-0.648934475027)/(o+1.)+(-0.418383297875)+(0.240224435511)*0.5**o
        ref[2,4,0,0]=(0.430020261528)/(o+1.)+(0.53421018733)+(0.436370231269)*0.5**o
        ref[2,4,0,1]=(0.906390660223)/(o+1.)+(-0.056057545856)+(0.640490734948)*0.5**o
        ref[2,4,1,0]=(-0.732099770943)/(o+1.)+(-0.0485027617732)+(0.161578976854)*0.5**o
        ref[2,4,1,1]=(-0.827495680848)/(o+1.)+(0.285044728936)+(-0.992031068384)*0.5**o
        ref[2,4,2,0]=(-0.919905471787)/(o+1.)+(0.656971386264)+(-0.541001898785)*0.5**o
        ref[2,4,2,1]=(0.161975875302)/(o+1.)+(0.272665206557)+(0.686527626533)*0.5**o
        ref[3,0,0,0]=(0.584120963259)/(o+1.)+(0.207930729804)+(0.972857504177)*0.5**o
        ref[3,0,0,1]=(0.315823141491)/(o+1.)+(-0.217561208386)+(-0.212429707823)*0.5**o
        ref[3,0,1,0]=(-0.732294996139)/(o+1.)+(-0.278653143596)+(-0.586168358175)*0.5**o
        ref[3,0,1,1]=(0.481321847225)/(o+1.)+(-0.070080648792)+(0.482676863662)*0.5**o
        ref[3,0,2,0]=(0.616679859076)/(o+1.)+(-0.418784546677)+(0.390742116352)*0.5**o
        ref[3,0,2,1]=(0.866778417448)/(o+1.)+(-0.686585656643)+(0.0629230167692)*0.5**o
        ref[3,1,0,0]=(0.531890054167)/(o+1.)+(-0.516187607633)+(-0.492439201527)*0.5**o
        ref[3,1,0,1]=(-0.512361639413)/(o+1.)+(0.499557182545)+(0.237651575694)*0.5**o
        ref[3,1,1,0]=(-0.67542965405)/(o+1.)+(0.274414249559)+(-0.678336189514)*0.5**o
        ref[3,1,1,1]=(0.371936242507)/(o+1.)+(-0.1636482466)+(-0.982634642862)*0.5**o
        ref[3,1,2,0]=(-0.799408831501)/(o+1.)+(0.458633399549)+(0.399251684639)*0.5**o
        ref[3,1,2,1]=(-0.0145253903686)/(o+1.)+(0.238472345905)+(0.757566560882)*0.5**o
        ref[3,2,0,0]=(-0.130519473112)/(o+1.)+(0.695277604837)+(0.581732248306)*0.5**o
        ref[3,2,0,1]=(0.632010536438)/(o+1.)+(0.123066205529)+(0.0316731560259)*0.5**o
        ref[3,2,1,0]=(-0.709926210356)/(o+1.)+(0.0926699972302)+(0.53924621094)*0.5**o
        ref[3,2,1,1]=(0.428094002247)/(o+1.)+(-0.841997926131)+(-0.276112488096)*0.5**o
        ref[3,2,2,0]=(0.708481148086)/(o+1.)+(-0.50498161534)+(0.495412824545)*0.5**o
        ref[3,2,2,1]=(0.782824622703)/(o+1.)+(-0.151509698687)+(-0.245246780227)*0.5**o
        ref[3,3,0,0]=(0.848460868418)/(o+1.)+(-0.104639248461)+(-0.359902945273)*0.5**o
        ref[3,3,0,1]=(0.0569151307298)/(o+1.)+(-0.0674224535469)+(0.165794065024)*0.5**o
        ref[3,3,1,0]=(-0.165683739908)/(o+1.)+(-0.183388699236)+(0.771348141212)*0.5**o
        ref[3,3,1,1]=(-0.932550058069)/(o+1.)+(-0.47828109001)+(-0.795311686688)*0.5**o
        ref[3,3,2,0]=(0.821542940923)/(o+1.)+(0.166853194487)+(-0.13512674466)*0.5**o
        ref[3,3,2,1]=(-0.46375871628)/(o+1.)+(0.132701267732)+(-0.449544418926)*0.5**o
        ref[3,4,0,0]=(-0.074530542168)/(o+1.)+(-0.0567877963797)+(-0.134845261261)*0.5**o
        ref[3,4,0,1]=(0.238671520479)/(o+1.)+(0.50689253572)+(0.731340228204)*0.5**o
        ref[3,4,1,0]=(0.981601513335)/(o+1.)+(0.400717209822)+(0.248707844726)*0.5**o
        ref[3,4,1,1]=(0.592304390107)/(o+1.)+(0.342408637328)+(0.834188819333)*0.5**o
        ref[3,4,2,0]=(-0.339991330909)/(o+1.)+(-0.0145922376156)+(0.899051000485)*0.5**o
        ref[3,4,2,1]=(-0.626610688164)/(o+1.)+(-0.417008165115)+(-0.426085250846)*0.5**o
      else:
        arg[0,0,0,0]=(0.697666818785)*x[0]+(1.00202295791)*x[1]+(0.617700157184)*x[2]
        arg[0,0,0,1]=(-0.465577819457)*x[0]+(-1.40054693355)*x[1]+(1.16912653295)*x[2]
        arg[0,0,1,0]=(-1.26696500634)*x[0]+(-0.0469756169975)*x[1]+(-0.445591738991)*x[2]
        arg[0,0,1,1]=(-1.61654321709)*x[0]+(-1.69522962467)*x[1]+(-1.06089094851)*x[2]
        arg[0,0,2,0]=(0.229468378506)*x[0]+(0.716813596991)*x[1]+(-0.237703937029)*x[2]
        arg[0,0,2,1]=(-1.12052051905)*x[0]+(0.835000004618)*x[1]+(-0.853004812404)*x[2]
        arg[0,1,0,0]=(-1.75500745705)*x[0]+(0.655055836882)*x[1]+(1.05207750735)*x[2]
        arg[0,1,0,1]=(-0.0746024824253)*x[0]+(-0.421596816958)*x[1]+(-0.375715855989)*x[2]
        arg[0,1,1,0]=(-0.963269684396)*x[0]+(0.431993248836)*x[1]+(-1.63061928649)*x[2]
        arg[0,1,1,1]=(-0.418757265018)*x[0]+(-0.665626168583)*x[1]+(-0.335734432507)*x[2]
        arg[0,1,2,0]=(0.980346404307)*x[0]+(1.45442049802)*x[1]+(-0.833420308612)*x[2]
        arg[0,1,2,1]=(-1.52536017847)*x[0]+(0.99484028681)*x[1]+(0.109754321663)*x[2]
        arg[0,2,0,0]=(1.697179262)*x[0]+(-0.928732802147)*x[1]+(-0.653379513626)*x[2]
        arg[0,2,0,1]=(0.682698902948)*x[0]+(0.829307959583)*x[1]+(-1.0300112276)*x[2]
        arg[0,2,1,0]=(0.026084323266)*x[0]+(1.28865784608)*x[1]+(0.830446580645)*x[2]
        arg[0,2,1,1]=(0.486291506032)*x[0]+(-0.519637742913)*x[1]+(0.81671330629)*x[2]
        arg[0,2,2,0]=(-0.761591624634)*x[0]+(0.370951131011)*x[1]+(-0.256748479312)*x[2]
        arg[0,2,2,1]=(-0.249974326513)*x[0]+(0.267323484635)*x[1]+(-1.09642130905)*x[2]
        arg[0,3,0,0]=(0.752057563255)*x[0]+(-1.2772262175)*x[1]+(-0.264569668983)*x[2]
        arg[0,3,0,1]=(-0.173167796447)*x[0]+(0.31873208356)*x[1]+(-0.599780701112)*x[2]
        arg[0,3,1,0]=(-0.912464014941)*x[0]+(-1.56452517166)*x[1]+(1.37651660499)*x[2]
        arg[0,3,1,1]=(0.302009667854)*x[0]+(0.524213064769)*x[1]+(0.316301522683)*x[2]
        arg[0,3,2,0]=(-1.2940154673)*x[0]+(-0.900061396992)*x[1]+(0.3860029847)*x[2]
        arg[0,3,2,1]=(-0.97354131341)*x[0]+(1.08221883964)*x[1]+(-0.739323076416)*x[2]
        arg[0,4,0,0]=(1.86535317192)*x[0]+(-0.478359589546)*x[1]+(-0.798014592302)*x[2]
        arg[0,4,0,1]=(-1.08518693851)*x[0]+(-0.186048654536)*x[1]+(1.04915573267)*x[2]
        arg[0,4,1,0]=(-1.09692811055)*x[0]+(1.01042920935)*x[1]+(-0.73978089555)*x[2]
        arg[0,4,1,1]=(0.654656964663)*x[0]+(0.279204596126)*x[1]+(0.886217781361)*x[2]
        arg[0,4,2,0]=(-0.54009886291)*x[0]+(0.378037248739)*x[1]+(1.09785568886)*x[2]
        arg[0,4,2,1]=(-1.43995958913)*x[0]+(-0.720157090994)*x[1]+(0.0676362028136)*x[2]
        arg[1,0,0,0]=(1.47735412608)*x[0]+(0.342355546228)*x[1]+(-0.0134502415508)*x[2]
        arg[1,0,0,1]=(-0.419536590369)*x[0]+(-1.61108470511)*x[1]+(1.51281979355)*x[2]
        arg[1,0,1,0]=(1.31765698245)*x[0]+(-0.7499566294)*x[1]+(0.934537503478)*x[2]
        arg[1,0,1,1]=(0.688467914883)*x[0]+(-1.32681280673)*x[1]+(-0.387405705058)*x[2]
        arg[1,0,2,0]=(1.37069587556)*x[0]+(0.540830111773)*x[1]+(0.477311068263)*x[2]
        arg[1,0,2,1]=(-0.572066213761)*x[0]+(-1.11950925494)*x[1]+(1.07939217872)*x[2]
        arg[1,1,0,0]=(0.564997105326)*x[0]+(-0.73130546389)*x[1]+(0.435910937304)*x[2]
        arg[1,1,0,1]=(0.513931739674)*x[0]+(-0.944043404826)*x[1]+(0.014114227468)*x[2]
        arg[1,1,1,0]=(-0.77850597256)*x[0]+(-0.445835136484)*x[1]+(0.157478357479)*x[2]
        arg[1,1,1,1]=(0.752431176866)*x[0]+(0.659759664199)*x[1]+(-0.859515433521)*x[2]
        arg[1,1,2,0]=(0.0693158730911)*x[0]+(-0.820997827204)*x[1]+(-0.714694436829)*x[2]
        arg[1,1,2,1]=(0.604674152847)*x[0]+(0.647261183132)*x[1]+(-1.21981352028)*x[2]
        arg[1,2,0,0]=(0.0997504695984)*x[0]+(-0.271549839083)*x[1]+(-0.573467835942)*x[2]
        arg[1,2,0,1]=(0.01861845581)*x[0]+(0.157410502406)*x[1]+(0.939257868776)*x[2]
        arg[1,2,1,0]=(1.44909894631)*x[0]+(0.116038517547)*x[1]+(-0.851940489233)*x[2]
        arg[1,2,1,1]=(1.34598841007)*x[0]+(1.79163125841)*x[1]+(0.719171044245)*x[2]
        arg[1,2,2,0]=(-0.214737791448)*x[0]+(-1.85810839095)*x[1]+(-0.240776599927)*x[2]
        arg[1,2,2,1]=(-0.574707645928)*x[0]+(-0.664173831956)*x[1]+(0.310262106696)*x[2]
        arg[1,3,0,0]=(0.296213453592)*x[0]+(-0.831112482555)*x[1]+(0.130075739128)*x[2]
        arg[1,3,0,1]=(-0.369510604457)*x[0]+(0.0373909697998)*x[1]+(1.6689129783)*x[2]
        arg[1,3,1,0]=(0.0922435881419)*x[0]+(-0.280519827004)*x[1]+(0.285933839232)*x[2]
        arg[1,3,1,1]=(-0.649253282272)*x[0]+(-0.460065044123)*x[1]+(-0.0711164948063)*x[2]
        arg[1,3,2,0]=(0.793313307581)*x[0]+(-0.0587956351149)*x[1]+(0.810487259739)*x[2]
        arg[1,3,2,1]=(-1.38668597055)*x[0]+(1.51396934198)*x[1]+(-1.13918550255)*x[2]
        arg[1,4,0,0]=(1.40078167737)*x[0]+(-1.61915436345)*x[1]+(0.421710091214)*x[2]
        arg[1,4,0,1]=(-1.37982015877)*x[0]+(1.30607371738)*x[1]+(-0.608888848419)*x[2]
        arg[1,4,1,0]=(-0.558087802805)*x[0]+(-0.214169499512)*x[1]+(-0.766242546503)*x[2]
        arg[1,4,1,1]=(-1.75503436584)*x[0]+(0.183604705451)*x[1]+(-0.148071484576)*x[2]
        arg[1,4,2,0]=(0.537942069142)*x[0]+(-0.00640749928252)*x[1]+(-0.277544629657)*x[2]
        arg[1,4,2,1]=(-0.652645833371)*x[0]+(-1.33439077026)*x[1]+(-0.226308197201)*x[2]
        arg[2,0,0,0]=(1.45646289279)*x[0]+(1.40842127675)*x[1]+(1.75078281495)*x[2]
        arg[2,0,0,1]=(0.867988651995)*x[0]+(1.35061812241)*x[1]+(-1.56520776323)*x[2]
        arg[2,0,1,0]=(0.549686523522)*x[0]+(-0.528085465813)*x[1]+(-0.994978820267)*x[2]
        arg[2,0,1,1]=(0.383595996552)*x[0]+(0.612989859087)*x[1]+(-0.560401247231)*x[2]
        arg[2,0,2,0]=(-0.341259919499)*x[0]+(-0.125491675042)*x[1]+(0.545572340757)*x[2]
        arg[2,0,2,1]=(-0.414269395434)*x[0]+(-0.433969881898)*x[1]+(-1.01820199328)*x[2]
        arg[2,1,0,0]=(-0.113575280389)*x[0]+(0.326916039942)*x[1]+(-0.576072890249)*x[2]
        arg[2,1,0,1]=(-0.296473969661)*x[0]+(0.195675099431)*x[1]+(-1.72378150831)*x[2]
        arg[2,1,1,0]=(-1.31575932758)*x[0]+(-0.122557423522)*x[1]+(0.53435027695)*x[2]
        arg[2,1,1,1]=(0.299435754608)*x[0]+(0.727140413706)*x[1]+(0.848931358475)*x[2]
        arg[2,1,2,0]=(0.2618965198)*x[0]+(-0.461672573339)*x[1]+(0.923592803172)*x[2]
        arg[2,1,2,1]=(-0.326817258523)*x[0]+(-0.0363300805442)*x[1]+(1.82049875918)*x[2]
        arg[2,2,0,0]=(-0.435113320888)*x[0]+(0.0257984353786)*x[1]+(0.287846577657)*x[2]
        arg[2,2,0,1]=(1.39178068279)*x[0]+(0.136781967539)*x[1]+(-1.38161590351)*x[2]
        arg[2,2,1,0]=(-0.887874199356)*x[0]+(-0.476769153654)*x[1]+(-0.618760485829)*x[2]
        arg[2,2,1,1]=(1.58883355098)*x[0]+(-0.889469047555)*x[1]+(-0.678486755843)*x[2]
        arg[2,2,2,0]=(0.237147029365)*x[0]+(-0.509883462401)*x[1]+(0.533163151551)*x[2]
        arg[2,2,2,1]=(0.66513913749)*x[0]+(1.30317030657)*x[1]+(0.804083365617)*x[2]
        arg[2,3,0,0]=(1.01531518403)*x[0]+(1.3753053143)*x[1]+(0.000877284235964)*x[2]
        arg[2,3,0,1]=(-0.40853642543)*x[0]+(0.154794268783)*x[1]+(-1.34306671117)*x[2]
        arg[2,3,1,0]=(0.43065284359)*x[0]+(-0.28965146009)*x[1]+(0.660248828789)*x[2]
        arg[2,3,1,1]=(-1.4334922681)*x[0]+(-0.0348802858609)*x[1]+(0.502051304974)*x[2]
        arg[2,3,2,0]=(-1.17399537487)*x[0]+(0.865787043946)*x[1]+(-0.0105949677478)*x[2]
        arg[2,3,2,1]=(0.63104158794)*x[0]+(0.436196079951)*x[1]+(0.498606623061)*x[2]
        arg[2,4,0,0]=(-1.44410177354)*x[0]+(0.513406326011)*x[1]+(0.699127199447)*x[2]
        arg[2,4,0,1]=(-0.981727023077)*x[0]+(0.484182120695)*x[1]+(0.667175832045)*x[2]
        arg[2,4,1,0]=(-1.3352602588)*x[0]+(0.626269023364)*x[1]+(0.813537646695)*x[2]
        arg[2,4,1,1]=(-0.932486930114)*x[0]+(0.299448807849)*x[1]+(0.195862014797)*x[2]
        arg[2,4,2,0]=(-0.0621489644806)*x[0]+(0.131979664158)*x[1]+(0.442029269778)*x[2]
        arg[2,4,2,1]=(-0.507508608633)*x[0]+(-1.28613226533)*x[1]+(-0.697920444023)*x[2]
        arg[3,0,0,0]=(-0.984942677585)*x[0]+(-1.58913299616)*x[1]+(-0.373026784822)*x[2]
        arg[3,0,0,1]=(0.616666427113)*x[0]+(-0.625162800471)*x[1]+(-0.796307290839)*x[2]
        arg[3,0,1,0]=(-0.48469203683)*x[0]+(-0.0572105166553)*x[1]+(0.809572383581)*x[2]
        arg[3,0,1,1]=(-0.487160309892)*x[0]+(0.77646454341)*x[1]+(-0.41947544573)*x[2]
        arg[3,0,2,0]=(-1.17902121554)*x[0]+(-0.244939532033)*x[1]+(-0.282994359476)*x[2]
        arg[3,0,2,1]=(0.33515909881)*x[0]+(0.146866561705)*x[1]+(1.81486098364)*x[2]
        arg[3,1,0,0]=(0.379977901341)*x[0]+(0.834531701517)*x[1]+(-0.0242027062166)*x[2]
        arg[3,1,0,1]=(0.184318292766)*x[0]+(1.08367911805)*x[1]+(-0.164531741594)*x[2]
        arg[3,1,1,0]=(-1.00967115996)*x[0]+(0.542699640149)*x[1]+(0.707616701259)*x[2]
        arg[3,1,1,1]=(0.856948233811)*x[0]+(-1.2186550297)*x[1]+(-0.034268798159)*x[2]
        arg[3,1,2,0]=(0.259470684477)*x[0]+(-1.03586983732)*x[1]+(-0.157275979341)*x[2]
        arg[3,1,2,1]=(-0.161954890658)*x[0]+(1.65853209885)*x[1]+(0.381113456719)*x[2]
        arg[3,2,0,0]=(-0.437413004681)*x[0]+(-0.730340273929)*x[1]+(-0.687689272494)*x[2]
        arg[3,2,0,1]=(-0.336618164118)*x[0]+(-1.24388666895)*x[1]+(-0.2198689254)*x[2]
        arg[3,2,1,0]=(0.0872793087887)*x[0]+(-0.336554573215)*x[1]+(-0.0597803523262)*x[2]
        arg[3,2,1,1]=(0.192805432018)*x[0]+(0.0540495582064)*x[1]+(0.846658042071)*x[2]
        arg[3,2,2,0]=(0.00794192604414)*x[0]+(-0.33504559053)*x[1]+(0.615701149787)*x[2]
        arg[3,2,2,1]=(0.210870961503)*x[0]+(1.14164095846)*x[1]+(-0.192785742791)*x[2]
        arg[3,3,0,0]=(-0.25895162041)*x[0]+(-0.225800436488)*x[1]+(-0.198007079838)*x[2]
        arg[3,3,0,1]=(-0.150363703231)*x[0]+(1.23870429535)*x[1]+(0.28635881247)*x[2]
        arg[3,3,1,0]=(-0.865980718216)*x[0]+(-0.0849853244335)*x[1]+(-0.292164217189)*x[2]
        arg[3,3,1,1]=(-0.350465968722)*x[0]+(0.09853736936)*x[1]+(-0.476711524665)*x[2]
        arg[3,3,2,0]=(0.311396300263)*x[0]+(-0.92437612034)*x[1]+(-0.757139962928)*x[2]
        arg[3,3,2,1]=(-0.912311919837)*x[0]+(0.267793631099)*x[1]+(0.4917345401)*x[2]
        arg[3,4,0,0]=(0.613662541753)*x[0]+(0.782757341361)*x[1]+(1.04209808158)*x[2]
        arg[3,4,0,1]=(0.492828613997)*x[0]+(1.11979095664)*x[1]+(-0.843545072354)*x[2]
        arg[3,4,1,0]=(-0.51939120556)*x[0]+(0.476004484286)*x[1]+(-0.516311785432)*x[2]
        arg[3,4,1,1]=(1.00423313567)*x[0]+(0.90776890388)*x[1]+(1.34423309497)*x[2]
        arg[3,4,2,0]=(-1.44069600826)*x[0]+(-1.39515290909)*x[1]+(1.19851590247)*x[2]
        arg[3,4,2,1]=(0.391851619877)*x[0]+(0.265529120553)*x[1]+(0.996610410913)*x[2]
        ref[0,0,0,0]=(1.19330263935)/(o+1.)+(0.55394275251)+(0.0162017895041)*0.5**o
        ref[0,0,0,1]=(-0.672351783684)/(o+1.)+(-0.187889622039)+(0.351132807711)*0.5**o
        ref[0,0,1,0]=(-1.27021996264)/(o+1.)+(0.236286530101)+(-0.961885459894)*0.5**o
        ref[0,0,1,1]=(-1.43442517107)/(o+1.)+(-1.00113374859)+(-0.935971122022)*0.5**o
        ref[0,0,2,0]=(-0.434813695127)/(o+1.)+(0.606673209583)+(-0.0699546855712)*0.5**o
        ref[0,0,2,1]=(-0.0434901247948)/(o+1.)+(-0.180911877983)+(-0.733211446073)*0.5**o
        ref[0,1,0,0]=(1.41514302182)/(o+1.)+(-0.247977413325)+(-0.967062307989)*0.5**o
        ref[0,1,0,1]=(0.69128100142)/(o+1.)+(-0.621609882923)+(-0.319976390946)*0.5**o
        ref[0,1,1,0]=(-0.881870429191)/(o+1.)+(-0.654075774082)+(0.0281262553057)*0.5**o
        ref[0,1,1,1]=(-1.27239116086)/(o+1.)+(-0.0854066429492)+(0.0230865806541)*0.5**o
        ref[0,1,2,0]=(-0.222663945351)/(o+1.)+(0.685312598747)+(0.453385341573)*0.5**o
        ref[0,1,2,1]=(0.101409038706)/(o+1.)+(0.151367946744)+(-0.824910502191)*0.5**o
        ref[0,2,0,0]=(-1.19509443892)/(o+1.)+(0.225437856866)+(0.85928567141)*0.5**o
        ref[0,2,0,1]=(0.285764347754)/(o+1.)+(-0.383432127133)+(0.963095541446)*0.5**o
        ref[0,2,1,0]=(0.728010227911)/(o+1.)+(0.983790919896)+(-0.550403317717)*0.5**o
        ref[0,2,1,1]=(0.563748259036)/(o+1.)+(0.165164845927)+(-0.110710881481)*0.5**o
        ref[0,2,2,0]=(-0.361502606429)/(o+1.)+(0.0276145482611)+(-0.341115463028)*0.5**o
        ref[0,2,2,1]=(-0.293320207933)/(o+1.)+(0.0336757304385)+(-0.853103403872)*0.5**o
        ref[0,3,0,0]=(-0.410233956527)/(o+1.)+(-0.474281126245)+(0.569057885785)*0.5**o
        ref[0,3,0,1]=(-0.835861970115)/(o+1.)+(0.0173715483175)+(0.346902459481)*0.5**o
        ref[0,3,1,0]=(0.0830466456187)/(o+1.)+(-0.288204681495)+(-0.607109864241)*0.5**o
        ref[0,3,1,1]=(1.07074130823)/(o+1.)+(-0.0437983012867)+(0.159379549652)*0.5**o
        ref[0,3,2,0]=(0.172301049609)/(o+1.)+(-0.491724874152)+(-0.996925180895)*0.5**o
        ref[0,3,2,1]=(-0.24891941221)/(o+1.)+(-0.03935856269)+(-0.3030090126)*0.5**o
        ref[0,4,0,0]=(-0.778575122429)/(o+1.)+(0.213085222078)+(0.941383668349)*0.5**o
        ref[0,4,0,1]=(0.880801627174)/(o+1.)+(-0.454451723401)+(-0.193978040747)*0.5**o
        ref[0,4,1,0]=(-0.835604724882)/(o+1.)+(0.127240834083)+(-0.245156740036)*0.5**o
        ref[0,4,1,1]=(1.62336716324)/(o+1.)+(-0.101085918464)+(0.398884015839)*0.5**o
        ref[0,4,2,0]=(1.16805827702)/(o+1.)+(0.211232591688)+(-0.654729385702)*0.5**o
        ref[0,4,2,1]=(-0.305424138957)/(o+1.)+(-0.431076572693)+(-0.924903192969)*0.5**o
        ref[1,0,0,0]=(-0.481596247529)/(o+1.)+(0.769902632559)+(0.748050413171)*0.5**o
        ref[1,0,0,1]=(0.177388661252)/(o+1.)+(-0.118766442613)+(-0.457657277952)*0.5**o
        ref[1,0,1,0]=(0.709890860372)/(o+1.)+(0.0668010230727)+(0.658744950012)*0.5**o
        ref[1,0,1,1]=(-1.45685781381)/(o+1.)+(-0.178674426089)+(0.788456069079)*0.5**o
        ref[1,0,2,0]=(0.194388185612)/(o+1.)+(0.649177543063)+(0.896093783855)*0.5**o
        ref[1,0,2,1]=(0.181561279507)/(o+1.)+(-0.200618291902)+(-0.392507985678)*0.5**o
        ref[1,1,0,0]=(-0.884376814027)/(o+1.)+(0.280382736159)+(0.593213920451)*0.5**o
        ref[1,1,0,1]=(-0.250709602218)/(o+1.)+(-0.492104211211)+(0.818920586956)*0.5**o
        ref[1,1,1,0]=(0.0529723694625)/(o+1.)+(-0.502047650638)+(-0.115739819752)*0.5**o
        ref[1,1,1,1]=(0.215812202957)/(o+1.)+(-0.308796450292)+(0.954456105171)*0.5**o
        ref[1,1,2,0]=(-0.991481050902)/(o+1.)+(0.162626252838)+(-0.800147845717)*0.5**o
        ref[1,1,2,1]=(-0.433989756951)/(o+1.)+(0.362776336712)+(-0.259441100777)*0.5**o
        ref[1,2,0,0]=(-0.930287052624)/(o+1.)+(-0.347752763108)+(0.880525373413)*0.5**o
        ref[1,2,0,1]=(0.698674852861)/(o+1.)+(-0.15119613196)+(0.719004238051)*0.5**o
        ref[1,2,1,0]=(-0.777216821365)/(o+1.)+(0.436336076297)+(0.617741643399)*0.5**o
        ref[1,2,1,1]=(1.09595223135)/(o+1.)+(0.978251130983)+(0.804336219412)*0.5**o
        ref[1,2,2,0]=(-0.651012630849)/(o+1.)+(-1.04894606664)+(0.43528198181)*0.5**o
        ref[1,2,2,1]=(-1.5227353883)/(o+1.)+(0.670019307557)+(-0.745922598001)*0.5**o
        ref[1,3,0,0]=(-0.750195637997)/(o+1.)+(0.231864245348)+(-0.118356142533)*0.5**o
        ref[1,3,0,1]=(1.17136226596)/(o+1.)+(-0.114976421389)+(0.395383920468)*0.5**o
        ref[1,3,1,0]=(-0.108070636769)/(o+1.)+(0.279145429234)+(-0.352562621327)*0.5**o
        ref[1,3,1,1]=(-0.144223481714)/(o+1.)+(-0.472954352493)+(-0.0903026345013)*0.5**o
        ref[1,3,2,0]=(-0.487564307786)/(o+1.)+(0.636973351449)+(0.758622537094)*0.5**o
        ref[1,3,2,1]=(0.390860965954)/(o+1.)+(-0.36647678165)+(-0.66980953377)*0.5**o
        ref[1,4,0,0]=(-1.04824984837)/(o+1.)+(0.213932231728)+(0.823722790054)*0.5**o
        ref[1,4,0,1]=(-0.0984653601336)/(o+1.)+(0.07867877705)+(-0.741527483773)*0.5**o
        ref[1,4,1,0]=(-0.146287867383)/(o+1.)+(-0.908951868158)+(0.425691754878)*0.5**o
        ref[1,4,1,1]=(-0.94160416554)/(o+1.)+(0.0459877010016)+(-0.869872381431)*0.5**o
        ref[1,4,2,0]=(0.0115742858952)/(o+1.)+(-0.222488399041)+(0.687392452389)*0.5**o
        ref[1,4,2,1]=(-0.956561936575)/(o+1.)+(-0.59770879042)+(-0.0613652834159)*0.5**o
        ref[2,0,0,0]=(1.83987381729)/(o+1.)+(0.956579317963)+(0.862634531274)*0.5**o
        ref[2,0,0,1]=(0.105446127421)/(o+1.)+(0.180661881191)+(0.186629121375)*0.5**o
        ref[2,0,1,0]=(0.0212167440766)/(o+1.)+(-0.337825588888)+(-0.318943328859)*0.5**o
        ref[2,0,1,1]=(0.744834715631)/(o+1.)+(-0.350058416663)+(0.391466726104)*0.5**o
        ref[2,0,2,0]=(-0.788776613856)/(o+1.)+(0.233776806014)+(0.400043748043)*0.5**o
        ref[2,0,2,1]=(-0.95346976307)/(o+1.)+(0.0274998652324)+(-0.967971238003)*0.5**o
        ref[2,1,0,0]=(-0.536051408942)/(o+1.)+(-0.208498016994)+(0.590315312233)*0.5**o
        ref[2,1,0,1]=(-1.04158508788)/(o+1.)+(-0.397885793101)+(0.0127762955396)*0.5**o
        ref[2,1,1,0]=(0.529914207253)/(o+1.)+(-0.547503443987)+(-0.338873793431)*0.5**o
        ref[2,1,1,1]=(0.62298670274)/(o+1.)+(0.449860048786)+(0.352800726478)*0.5**o
        ref[2,1,2,0]=(0.965292650595)/(o+1.)+(0.00980498639)+(-0.261085873742)*0.5**o
        ref[2,1,2,1]=(0.403285360803)/(o+1.)+(0.369160592269)+(0.315744874774)*0.5**o
        ref[2,2,0,0]=(-0.457764884724)/(o+1.)+(0.563657776706)+(-0.791018976541)*0.5**o
        ref[2,2,0,1]=(0.188151274044)/(o+1.)+(-0.476470545911)+(0.911736564588)*0.5**o
        ref[2,2,1,0]=(-0.558515085739)/(o+1.)+(-0.445583796553)+(-0.533721159994)*0.5**o
        ref[2,2,1,1]=(-0.431992337265)/(o+1.)+(-0.0957726221961)+(0.64441532924)*0.5**o
        ref[2,2,2,0]=(-0.159056461947)/(o+1.)+(-0.0326864536156)+(0.484856087693)*0.5**o
        ref[2,2,2,1]=(0.533116456037)/(o+1.)+(1.10197257725)+(0.0353311991352)*0.5**o
        ref[2,3,0,0]=(0.0570625147023)/(o+1.)+(0.74170779854)+(0.851019670777)*0.5**o
        ref[2,3,0,1]=(-0.495059904832)/(o+1.)+(-0.169954912181)+(-0.761839138623)*0.5**o
        ref[2,3,1,0]=(0.482375173314)/(o+1.)+(-0.19374838651)+(0.706371811996)*0.5**o
        ref[2,3,1,1]=(0.12928100721)/(o+1.)+(-0.317327862062)+(-0.460946532077)*0.5**o
        ref[2,3,2,0]=(-0.234362084763)/(o+1.)+(0.423486279181)+(-0.93141377227)*0.5**o
        ref[2,3,2,1]=(1.25586693002)/(o+1.)+(-0.127551329993)+(0.56508002092)*0.5**o
        ref[2,4,0,0]=(0.45235297749)/(o+1.)+(-0.0217537364788)+(-0.640413752615)*0.5**o
        ref[2,4,0,1]=(0.936809207384)/(o+1.)+(-0.151519610695)+(-0.464139056331)*0.5**o
        ref[2,4,1,0]=(0.867693208863)/(o+1.)+(0.105683574314)+(-0.974513946233)*0.5**o
        ref[2,4,1,1]=(-1.27821575101)/(o+1.)+(0.463163847311)+(-0.0852880510814)*0.5**o
        ref[2,4,2,0]=(-0.846224501366)/(o+1.)+(0.432619461409)+(0.492845548002)*0.5**o
        ref[2,4,2,1]=(-1.48923229995)/(o+1.)+(-0.505747396375)+(0.00916577472118)*0.5**o
        ref[3,0,0,0]=(-0.959215731121)/(o+1.)+(-0.904609108587)+(-0.178668510268)*0.5**o
        ref[3,0,0,1]=(-1.53181231865)/(o+1.)+(-0.0758543610408)+(0.878717376536)*0.5**o
        ref[3,0,1,0]=(1.18643238831)/(o+1.)+(-0.288825159097)+(-0.341112240015)*0.5**o
        ref[3,0,1,1]=(-0.0675694917031)/(o+1.)+(-0.168955703616)+(0.275309686724)*0.5**o
        ref[3,0,2,0]=(-0.663269860133)/(o+1.)+(-0.235102421885)+(-0.573480403148)*0.5**o
        ref[3,0,2,1]=(1.0969289778)/(o+1.)+(0.46603172821)+(0.267894209932)*0.5**o
        ref[3,1,0,0]=(-0.716870513559)/(o+1.)+(0.712039409283)+(0.483098591635)*0.5**o
        ref[3,1,0,1]=(0.0328531999411)/(o+1.)+(0.379319450988)+(0.311973567308)*0.5**o
        ref[3,1,1,0]=(1.08648208064)/(o+1.)+(-0.196831959338)+(-0.452172980519)*0.5**o
        ref[3,1,1,1]=(0.0813181502612)/(o+1.)+(-0.227364064527)+(-0.0225656152548)*0.5**o
        ref[3,1,2,0]=(-0.271000454408)/(o+1.)+(-0.765711687155)+(0.868748696531)*0.5**o
        ref[3,1,2,1]=(1.60569646253)/(o+1.)+(0.137437739736)+(-0.00288127709197)*0.5**o
        ref[3,2,0,0]=(-1.25887089059)/(o+1.)+(-0.525842290945)+(0.455112921376)*0.5**o
        ref[3,2,0,1]=(-1.63787880725)/(o+1.)+(0.00214706725405)+(-0.166789085722)*0.5**o
        ref[3,2,1,0]=(-1.82111307894)/(o+1.)+(0.561806270366)+(0.388444921454)*0.5**o
        ref[3,2,1,1]=(-0.489259906414)/(o+1.)+(0.582548398381)+(0.417676141947)*0.5**o
        ref[3,2,2,0]=(-0.0674375340391)/(o+1.)+(-0.00345549227349)+(0.362946003887)*0.5**o
        ref[3,2,2,1]=(0.662780682001)/(o+1.)+(-0.15453848218)+(0.806022459534)*0.5**o
        ref[3,3,0,0]=(-1.15795406515)/(o+1.)+(0.0926725746467)+(0.289849779118)*0.5**o
        ref[3,3,0,1]=(1.27520269547)/(o+1.)+(-0.286529818361)+(0.672556345847)*0.5**o
        ref[3,3,1,0]=(-0.343290651469)/(o+1.)+(-0.0269962100904)+(-0.845847188189)*0.5**o
        ref[3,3,1,1]=(0.497715053371)/(o+1.)+(-0.158577177766)+(-0.909200821866)*0.5**o
        ref[3,3,2,0]=(-0.952431538662)/(o+1.)+(0.0816482063045)+(-0.580984656952)*0.5**o
        ref[3,3,2,1]=(-0.176227469799)/(o+1.)+(0.0456157853688)+(-0.0677878495767)*0.5**o
        ref[3,4,0,0]=(0.976460483802)/(o+1.)+(0.347595506011)+(0.766866468871)*0.5**o
        ref[3,4,0,1]=(0.0358170327662)/(o+1.)+(-0.0991456259778)+(0.931548717473)*0.5**o
        ref[3,4,1,0]=(0.114544318514)/(o+1.)+(-0.201014865792)+(-0.272213093635)*0.5**o
        ref[3,4,1,1]=(0.554509925385)/(o+1.)+(1.23219332557)+(0.237338557994)*0.5**o
        ref[3,4,2,0]=(0.0803465444931)/(o+1.)+(-0.518257569997)+(-0.681164419371)*0.5**o
        ref[3,4,2,1]=(-0.0443609570522)/(o+1.)+(0.572813368096)+(0.552725372202)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactZero_fromData_FunctionOnContactZero_rank0(self):
      """
      tests integral of rank 0 Data on the FunctionOnContactZero

      assumptions: FunctionOnContactZero(self.domain) exists
                   self.domain supports integral on FunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      w=FunctionOnContactZero(self.domain)
      x=w.getX()
      arg=Data(0,(),w)
      if dim==2:
        arg=(-0.114918917602)*x[0]**o+(0.304834778607)*x[0]+(0.704057176139)*x[1]**o+(-0.0664624375434)*x[1]
        ref=(0.704057176139)/(o+1.)+(0.119186170532)+(-0.114918917602)*0.5**o
      else:
        arg=(-0.134961573162)*x[0]**o+(0.585433861814)*x[0]+(0.209596676649)*x[1]**o+(0.870867416311)*x[1]+(-0.902854033973)*x[2]**o+(-0.812135783993)*x[2]
        ref=(-0.693257357324)/(o+1.)+(0.322082747066)+(-0.134961573162)*0.5**o
      res=integrate(arg)
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactZero_fromData_FunctionOnContactZero_rank1(self):
      """
      tests integral of rank 1 Data on the FunctionOnContactZero

      assumptions: FunctionOnContactZero(self.domain) exists
                   self.domain supports integral on FunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      w=FunctionOnContactZero(self.domain)
      x=w.getX()
      arg=Data(0,(2,),w)
      ref=numpy.zeros((2,),numpy.float_)
      if dim==2:
        arg[0]=(0.441677901994)*x[0]**o+(0.0234076994082)*x[0]+(-0.520465368981)*x[1]**o+(-0.668717137452)*x[1]
        arg[1]=(0.0637451453664)*x[0]**o+(0.610543665309)*x[0]+(0.639008465664)*x[1]**o+(-0.460500756968)*x[1]
        ref[0]=(-0.520465368981)/(o+1.)+(-0.322654719022)+(0.441677901994)*0.5**o
        ref[1]=(0.639008465664)/(o+1.)+(0.0750214541708)+(0.0637451453664)*0.5**o
      else:
        arg[0]=(0.207042421638)*x[0]**o+(0.586067682859)*x[0]+(-0.731189752251)*x[1]**o+(0.439755444977)*x[1]+(-0.215027146515)*x[2]**o+(-0.22109699198)*x[2]
        arg[1]=(-0.482897746923)*x[0]**o+(-0.262848758384)*x[0]+(-0.130231246709)*x[1]**o+(0.494143899685)*x[1]+(0.871226664446)*x[2]**o+(-0.45136577521)*x[2]
        ref[0]=(-0.946216898767)/(o+1.)+(0.402363067928)+(0.207042421638)*0.5**o
        ref[1]=(0.740995417737)/(o+1.)+(-0.110035316955)+(-0.482897746923)*0.5**o
      res=integrate(arg)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactZero_fromData_FunctionOnContactZero_rank2(self):
      """
      tests integral of rank 2 Data on the FunctionOnContactZero

      assumptions: FunctionOnContactZero(self.domain) exists
                   self.domain supports integral on FunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      w=FunctionOnContactZero(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5),w)
      ref=numpy.zeros((4, 5),numpy.float_)
      if dim==2:
        arg[0,0]=(0.418217706031)*x[0]**o+(0.817101274679)*x[0]+(0.359227724634)*x[1]**o+(0.106952446872)*x[1]
        arg[0,1]=(0.735477258623)*x[0]**o+(-0.181692404492)*x[0]+(0.118248218763)*x[1]**o+(0.910033225874)*x[1]
        arg[0,2]=(-0.370301174151)*x[0]**o+(0.421589410528)*x[0]+(-0.91974297037)*x[1]**o+(0.676441845707)*x[1]
        arg[0,3]=(0.170535646217)*x[0]**o+(0.650458506701)*x[0]+(-0.431898202846)*x[1]**o+(-0.89133664224)*x[1]
        arg[0,4]=(-0.288793192482)*x[0]**o+(-0.752066662422)*x[0]+(0.279565184651)*x[1]**o+(-0.20569858505)*x[1]
        arg[1,0]=(-0.831478902729)*x[0]**o+(-0.545347589281)*x[0]+(-0.315738709444)*x[1]**o+(-0.407156023524)*x[1]
        arg[1,1]=(0.119396512736)*x[0]**o+(0.56655186098)*x[0]+(0.798206239231)*x[1]**o+(0.0342467197178)*x[1]
        arg[1,2]=(-0.264721761214)*x[0]**o+(-0.7583989175)*x[0]+(-0.350411882147)*x[1]**o+(-0.556879396863)*x[1]
        arg[1,3]=(0.976029513102)*x[0]**o+(-0.117830794452)*x[0]+(0.318067419468)*x[1]**o+(-0.496129688948)*x[1]
        arg[1,4]=(0.462797018845)*x[0]**o+(-0.214502973992)*x[0]+(0.155052684044)*x[1]**o+(0.264794267973)*x[1]
        arg[2,0]=(0.193892132778)*x[0]**o+(-0.440406597181)*x[0]+(0.527330868842)*x[1]**o+(0.856602716155)*x[1]
        arg[2,1]=(0.0418968976051)*x[0]**o+(0.944283628708)*x[0]+(0.923548919008)*x[1]**o+(0.958011594347)*x[1]
        arg[2,2]=(0.744026460715)*x[0]**o+(0.238896690247)*x[0]+(0.968891312128)*x[1]**o+(0.0636831438113)*x[1]
        arg[2,3]=(-0.289048983574)*x[0]**o+(0.897469203226)*x[0]+(0.948131139828)*x[1]**o+(0.821749060686)*x[1]
        arg[2,4]=(-0.903766144138)*x[0]**o+(-0.453139974489)*x[0]+(-0.715818671181)*x[1]**o+(-0.712367492971)*x[1]
        arg[3,0]=(0.315277263598)*x[0]**o+(-0.676815102073)*x[0]+(0.428704415911)*x[1]**o+(0.54703485039)*x[1]
        arg[3,1]=(-0.301882901745)*x[0]**o+(0.888822293672)*x[0]+(-0.496204235393)*x[1]**o+(0.210598259748)*x[1]
        arg[3,2]=(0.568071178492)*x[0]**o+(-0.552863936415)*x[0]+(0.758619958752)*x[1]**o+(0.520428540448)*x[1]
        arg[3,3]=(0.477425351692)*x[0]**o+(-0.427405588029)*x[0]+(0.197959279245)*x[1]**o+(0.328047998649)*x[1]
        arg[3,4]=(-0.869704203328)*x[0]**o+(-0.0825164966999)*x[0]+(0.779325592334)*x[1]**o+(0.419916607608)*x[1]
        ref[0,0]=(0.359227724634)/(o+1.)+(0.462026860776)+(0.418217706031)*0.5**o
        ref[0,1]=(0.118248218763)/(o+1.)+(0.364170410691)+(0.735477258623)*0.5**o
        ref[0,2]=(-0.91974297037)/(o+1.)+(0.549015628118)+(-0.370301174151)*0.5**o
        ref[0,3]=(-0.431898202846)/(o+1.)+(-0.120439067769)+(0.170535646217)*0.5**o
        ref[0,4]=(0.279565184651)/(o+1.)+(-0.478882623736)+(-0.288793192482)*0.5**o
        ref[1,0]=(-0.315738709444)/(o+1.)+(-0.476251806402)+(-0.831478902729)*0.5**o
        ref[1,1]=(0.798206239231)/(o+1.)+(0.300399290349)+(0.119396512736)*0.5**o
        ref[1,2]=(-0.350411882147)/(o+1.)+(-0.657639157182)+(-0.264721761214)*0.5**o
        ref[1,3]=(0.318067419468)/(o+1.)+(-0.3069802417)+(0.976029513102)*0.5**o
        ref[1,4]=(0.155052684044)/(o+1.)+(0.02514564699)+(0.462797018845)*0.5**o
        ref[2,0]=(0.527330868842)/(o+1.)+(0.208098059487)+(0.193892132778)*0.5**o
        ref[2,1]=(0.923548919008)/(o+1.)+(0.951147611528)+(0.0418968976051)*0.5**o
        ref[2,2]=(0.968891312128)/(o+1.)+(0.151289917029)+(0.744026460715)*0.5**o
        ref[2,3]=(0.948131139828)/(o+1.)+(0.859609131956)+(-0.289048983574)*0.5**o
        ref[2,4]=(-0.715818671181)/(o+1.)+(-0.58275373373)+(-0.903766144138)*0.5**o
        ref[3,0]=(0.428704415911)/(o+1.)+(-0.0648901258416)+(0.315277263598)*0.5**o
        ref[3,1]=(-0.496204235393)/(o+1.)+(0.54971027671)+(-0.301882901745)*0.5**o
        ref[3,2]=(0.758619958752)/(o+1.)+(-0.0162176979831)+(0.568071178492)*0.5**o
        ref[3,3]=(0.197959279245)/(o+1.)+(-0.0496787946897)+(0.477425351692)*0.5**o
        ref[3,4]=(0.779325592334)/(o+1.)+(0.168700055454)+(-0.869704203328)*0.5**o
      else:
        arg[0,0]=(0.204452789147)*x[0]**o+(0.411843111144)*x[0]+(-0.0502866367412)*x[1]**o+(0.957356261522)*x[1]+(0.173522462943)*x[2]**o+(-0.0194908382525)*x[2]
        arg[0,1]=(-0.49641528149)*x[0]**o+(0.158625922243)*x[0]+(0.670932733725)*x[1]**o+(-0.611829064862)*x[1]+(0.758373881782)*x[2]**o+(0.151236368375)*x[2]
        arg[0,2]=(0.577504765611)*x[0]**o+(0.872996499874)*x[0]+(-0.734000014722)*x[1]**o+(-0.890824538872)*x[1]+(-0.796786080271)*x[2]**o+(-0.389005562901)*x[2]
        arg[0,3]=(-0.933166619447)*x[0]**o+(-0.528627639194)*x[0]+(-0.828111326322)*x[1]**o+(0.528345547301)*x[1]+(0.99894455165)*x[2]**o+(0.472982583662)*x[2]
        arg[0,4]=(-0.495503045054)*x[0]**o+(0.629909217292)*x[0]+(0.901515784751)*x[1]**o+(-0.863668288904)*x[1]+(0.76892679355)*x[2]**o+(0.34730747389)*x[2]
        arg[1,0]=(0.303189803506)*x[0]**o+(-0.17543401741)*x[0]+(-0.283157585975)*x[1]**o+(0.950171529024)*x[1]+(0.0483247572497)*x[2]**o+(-0.604332298954)*x[2]
        arg[1,1]=(-0.225480407643)*x[0]**o+(-0.0521067696425)*x[0]+(-0.10259234894)*x[1]**o+(0.373551917005)*x[1]+(0.89047195372)*x[2]**o+(-0.292740890417)*x[2]
        arg[1,2]=(-0.476041156832)*x[0]**o+(0.916078681628)*x[0]+(-0.393887610676)*x[1]**o+(-0.651852504525)*x[1]+(0.611404175606)*x[2]**o+(0.336594647903)*x[2]
        arg[1,3]=(-0.415178048882)*x[0]**o+(0.145394549559)*x[0]+(0.680115054186)*x[1]**o+(0.261517386893)*x[1]+(0.781566117396)*x[2]**o+(-0.659634539391)*x[2]
        arg[1,4]=(0.874821341857)*x[0]**o+(0.317418381126)*x[0]+(0.051455923305)*x[1]**o+(0.670170143071)*x[1]+(-0.211902435336)*x[2]**o+(-0.175299130233)*x[2]
        arg[2,0]=(-0.372961546517)*x[0]**o+(0.199493150648)*x[0]+(-0.630239499285)*x[1]**o+(0.657412283442)*x[1]+(-0.82211915324)*x[2]**o+(-0.468612977158)*x[2]
        arg[2,1]=(-0.503168527887)*x[0]**o+(0.904406483201)*x[0]+(-0.206179628633)*x[1]**o+(0.0787026584997)*x[1]+(0.899590466467)*x[2]**o+(-0.893401768413)*x[2]
        arg[2,2]=(-0.860556408954)*x[0]**o+(0.39021140432)*x[0]+(-0.66425383352)*x[1]**o+(0.287711780417)*x[1]+(0.776884760126)*x[2]**o+(-0.115905192334)*x[2]
        arg[2,3]=(0.461359695993)*x[0]**o+(0.997877835019)*x[0]+(0.0513514112685)*x[1]**o+(-0.904812988908)*x[1]+(-0.903222331838)*x[2]**o+(-0.470185679041)*x[2]
        arg[2,4]=(-0.646560730528)*x[0]**o+(0.10038324067)*x[0]+(0.426753245517)*x[1]**o+(-0.109889624344)*x[1]+(-0.20158193753)*x[2]**o+(0.014258531224)*x[2]
        arg[3,0]=(0.364407704271)*x[0]**o+(0.840519040103)*x[0]+(0.528721642544)*x[1]**o+(-0.853679853614)*x[1]+(0.079836122575)*x[2]**o+(-0.439039178308)*x[2]
        arg[3,1]=(-0.0726491927955)*x[0]**o+(0.217804431494)*x[0]+(-0.454354285545)*x[1]**o+(0.390539936003)*x[1]+(-0.989667912688)*x[2]**o+(-0.0674079606774)*x[2]
        arg[3,2]=(0.62239422825)*x[0]**o+(0.998733716844)*x[0]+(0.796258161536)*x[1]**o+(0.618545466026)*x[1]+(-0.243188896911)*x[2]**o+(-0.677687584224)*x[2]
        arg[3,3]=(0.444073180258)*x[0]**o+(-0.764729326792)*x[0]+(-0.823491453567)*x[1]**o+(0.985393963131)*x[1]+(0.623800470977)*x[2]**o+(0.947328422513)*x[2]
        arg[3,4]=(0.686438920325)*x[0]**o+(0.789760134176)*x[0]+(-0.26011082782)*x[1]**o+(0.0692377043339)*x[1]+(-0.3712894236)*x[2]**o+(-0.542974191103)*x[2]
        ref[0,0]=(0.123235826202)/(o+1.)+(0.674854267207)+(0.204452789147)*0.5**o
        ref[0,1]=(1.42930661551)/(o+1.)+(-0.150983387122)+(-0.49641528149)*0.5**o
        ref[0,2]=(-1.53078609499)/(o+1.)+(-0.203416800949)+(0.577504765611)*0.5**o
        ref[0,3]=(0.170833225328)/(o+1.)+(0.236350245885)+(-0.933166619447)*0.5**o
        ref[0,4]=(1.6704425783)/(o+1.)+(0.0567742011389)+(-0.495503045054)*0.5**o
        ref[1,0]=(-0.234832828725)/(o+1.)+(0.0852026063299)+(0.303189803506)*0.5**o
        ref[1,1]=(0.787879604779)/(o+1.)+(0.0143521284725)+(-0.225480407643)*0.5**o
        ref[1,2]=(0.21751656493)/(o+1.)+(0.300410412503)+(-0.476041156832)*0.5**o
        ref[1,3]=(1.46168117158)/(o+1.)+(-0.126361301469)+(-0.415178048882)*0.5**o
        ref[1,4]=(-0.160446512031)/(o+1.)+(0.406144696982)+(0.874821341857)*0.5**o
        ref[2,0]=(-1.45235865253)/(o+1.)+(0.194146228466)+(-0.372961546517)*0.5**o
        ref[2,1]=(0.693410837835)/(o+1.)+(0.0448536866438)+(-0.503168527887)*0.5**o
        ref[2,2]=(0.112630926606)/(o+1.)+(0.281008996202)+(-0.860556408954)*0.5**o
        ref[2,3]=(-0.85187092057)/(o+1.)+(-0.188560416464)+(0.461359695993)*0.5**o
        ref[2,4]=(0.225171307987)/(o+1.)+(0.00237607377473)+(-0.646560730528)*0.5**o
        ref[3,0]=(0.608557765119)/(o+1.)+(-0.22609999591)+(0.364407704271)*0.5**o
        ref[3,1]=(-1.44402219823)/(o+1.)+(0.27046820341)+(-0.0726491927955)*0.5**o
        ref[3,2]=(0.553069264625)/(o+1.)+(0.469795799323)+(0.62239422825)*0.5**o
        ref[3,3]=(-0.19969098259)/(o+1.)+(0.583996529426)+(0.444073180258)*0.5**o
        ref[3,4]=(-0.631400251421)/(o+1.)+(0.158011823704)+(0.686438920325)*0.5**o
      res=integrate(arg)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactZero_fromData_FunctionOnContactZero_rank3(self):
      """
      tests integral of rank 3 Data on the FunctionOnContactZero

      assumptions: FunctionOnContactZero(self.domain) exists
                   self.domain supports integral on FunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      w=FunctionOnContactZero(self.domain)
      x=w.getX()
      arg=Data(0,(6, 2, 2),w)
      ref=numpy.zeros((6, 2, 2),numpy.float_)
      if dim==2:
        arg[0,0,0]=(-0.963014866001)*x[0]**o+(-0.592091539519)*x[0]+(0.397897510985)*x[1]**o+(0.485612885034)*x[1]
        arg[0,0,1]=(-0.764849027936)*x[0]**o+(0.936217923087)*x[0]+(-0.6256860991)*x[1]**o+(0.162359755936)*x[1]
        arg[0,1,0]=(-0.269607090458)*x[0]**o+(0.197100067821)*x[0]+(-0.216513043667)*x[1]**o+(0.143038940436)*x[1]
        arg[0,1,1]=(-0.432206914977)*x[0]**o+(0.177851208963)*x[0]+(0.0477652910843)*x[1]**o+(0.278333389583)*x[1]
        arg[1,0,0]=(-0.123940629628)*x[0]**o+(0.818553665062)*x[0]+(-0.168322144522)*x[1]**o+(0.485324994762)*x[1]
        arg[1,0,1]=(0.593609516922)*x[0]**o+(0.530508390905)*x[0]+(-0.938421636915)*x[1]**o+(-0.686459217917)*x[1]
        arg[1,1,0]=(-0.874022523162)*x[0]**o+(0.694500749056)*x[0]+(-0.650951516798)*x[1]**o+(0.45826621855)*x[1]
        arg[1,1,1]=(-0.259058106634)*x[0]**o+(0.0837282545694)*x[0]+(0.465734884341)*x[1]**o+(0.185558060279)*x[1]
        arg[2,0,0]=(0.346292866419)*x[0]**o+(0.982902010078)*x[0]+(0.0297606445334)*x[1]**o+(0.339043518987)*x[1]
        arg[2,0,1]=(0.749684718966)*x[0]**o+(-0.855799436867)*x[0]+(-0.186333913876)*x[1]**o+(-0.548968316245)*x[1]
        arg[2,1,0]=(-0.255759466052)*x[0]**o+(0.441882703647)*x[0]+(-0.0239807362322)*x[1]**o+(-0.236417762222)*x[1]
        arg[2,1,1]=(-0.0586694299212)*x[0]**o+(-0.912208422332)*x[0]+(-0.11610871626)*x[1]**o+(-0.512805786403)*x[1]
        arg[3,0,0]=(0.826027179564)*x[0]**o+(-0.589876767386)*x[0]+(-0.654858828463)*x[1]**o+(0.601656609023)*x[1]
        arg[3,0,1]=(-0.148714999203)*x[0]**o+(-0.409138795316)*x[0]+(0.290726299307)*x[1]**o+(-0.94009130345)*x[1]
        arg[3,1,0]=(0.63431415979)*x[0]**o+(-0.184553980231)*x[0]+(0.4087395755)*x[1]**o+(-0.694376406449)*x[1]
        arg[3,1,1]=(-0.000928514830108)*x[0]**o+(-0.782814501236)*x[0]+(0.177390244103)*x[1]**o+(0.698562703841)*x[1]
        arg[4,0,0]=(-0.939264907756)*x[0]**o+(0.191360117367)*x[0]+(-0.318750748728)*x[1]**o+(0.910713477162)*x[1]
        arg[4,0,1]=(-0.105186863713)*x[0]**o+(0.0319652211881)*x[0]+(0.118896230389)*x[1]**o+(-0.212677289177)*x[1]
        arg[4,1,0]=(-0.702838100428)*x[0]**o+(0.881695294345)*x[0]+(0.728231539571)*x[1]**o+(-0.192311604244)*x[1]
        arg[4,1,1]=(0.25197450886)*x[0]**o+(0.464857988125)*x[0]+(0.851996888259)*x[1]**o+(-0.408525083481)*x[1]
        arg[5,0,0]=(0.665627182446)*x[0]**o+(0.5367740452)*x[0]+(0.295800616406)*x[1]**o+(0.0620589956995)*x[1]
        arg[5,0,1]=(-0.37320168697)*x[0]**o+(0.072855481498)*x[0]+(0.539806161326)*x[1]**o+(-0.999091813027)*x[1]
        arg[5,1,0]=(-0.060845772709)*x[0]**o+(-0.931869215952)*x[0]+(-0.393477853475)*x[1]**o+(0.968288081996)*x[1]
        arg[5,1,1]=(-0.0768045781739)*x[0]**o+(-0.257318730887)*x[0]+(0.781473167319)*x[1]**o+(-0.458870794544)*x[1]
        ref[0,0,0]=(0.397897510985)/(o+1.)+(-0.0532393272425)+(-0.963014866001)*0.5**o
        ref[0,0,1]=(-0.6256860991)/(o+1.)+(0.549288839511)+(-0.764849027936)*0.5**o
        ref[0,1,0]=(-0.216513043667)/(o+1.)+(0.170069504128)+(-0.269607090458)*0.5**o
        ref[0,1,1]=(0.0477652910843)/(o+1.)+(0.228092299273)+(-0.432206914977)*0.5**o
        ref[1,0,0]=(-0.168322144522)/(o+1.)+(0.651939329912)+(-0.123940629628)*0.5**o
        ref[1,0,1]=(-0.938421636915)/(o+1.)+(-0.0779754135063)+(0.593609516922)*0.5**o
        ref[1,1,0]=(-0.650951516798)/(o+1.)+(0.576383483803)+(-0.874022523162)*0.5**o
        ref[1,1,1]=(0.465734884341)/(o+1.)+(0.134643157424)+(-0.259058106634)*0.5**o
        ref[2,0,0]=(0.0297606445334)/(o+1.)+(0.660972764533)+(0.346292866419)*0.5**o
        ref[2,0,1]=(-0.186333913876)/(o+1.)+(-0.702383876556)+(0.749684718966)*0.5**o
        ref[2,1,0]=(-0.0239807362322)/(o+1.)+(0.102732470713)+(-0.255759466052)*0.5**o
        ref[2,1,1]=(-0.11610871626)/(o+1.)+(-0.712507104367)+(-0.0586694299212)*0.5**o
        ref[3,0,0]=(-0.654858828463)/(o+1.)+(0.0058899208184)+(0.826027179564)*0.5**o
        ref[3,0,1]=(0.290726299307)/(o+1.)+(-0.674615049383)+(-0.148714999203)*0.5**o
        ref[3,1,0]=(0.4087395755)/(o+1.)+(-0.43946519334)+(0.63431415979)*0.5**o
        ref[3,1,1]=(0.177390244103)/(o+1.)+(-0.0421258986973)+(-0.000928514830108)*0.5**o
        ref[4,0,0]=(-0.318750748728)/(o+1.)+(0.551036797264)+(-0.939264907756)*0.5**o
        ref[4,0,1]=(0.118896230389)/(o+1.)+(-0.0903560339946)+(-0.105186863713)*0.5**o
        ref[4,1,0]=(0.728231539571)/(o+1.)+(0.344691845051)+(-0.702838100428)*0.5**o
        ref[4,1,1]=(0.851996888259)/(o+1.)+(0.0281664523221)+(0.25197450886)*0.5**o
        ref[5,0,0]=(0.295800616406)/(o+1.)+(0.29941652045)+(0.665627182446)*0.5**o
        ref[5,0,1]=(0.539806161326)/(o+1.)+(-0.463118165764)+(-0.37320168697)*0.5**o
        ref[5,1,0]=(-0.393477853475)/(o+1.)+(0.0182094330217)+(-0.060845772709)*0.5**o
        ref[5,1,1]=(0.781473167319)/(o+1.)+(-0.358094762716)+(-0.0768045781739)*0.5**o
      else:
        arg[0,0,0]=(0.638389367885)*x[0]**o+(-0.77703387527)*x[0]+(0.951496212227)*x[1]**o+(-0.484305156823)*x[1]+(-0.582437996275)*x[2]**o+(-0.460130065965)*x[2]
        arg[0,0,1]=(0.859011160063)*x[0]**o+(0.999372374168)*x[0]+(-0.946049724069)*x[1]**o+(0.522184535496)*x[1]+(0.00820405492191)*x[2]**o+(0.769305042211)*x[2]
        arg[0,1,0]=(-0.998681640062)*x[0]**o+(0.916063974594)*x[0]+(0.0801858148899)*x[1]**o+(0.475476401258)*x[1]+(-0.805247247843)*x[2]**o+(-0.809331278141)*x[2]
        arg[0,1,1]=(0.1340261052)*x[0]**o+(0.843204972218)*x[0]+(-0.809433597166)*x[1]**o+(-0.947798215446)*x[1]+(0.268796347717)*x[2]**o+(-0.896620567629)*x[2]
        arg[1,0,0]=(0.251201824008)*x[0]**o+(0.762438451805)*x[0]+(0.892722429031)*x[1]**o+(0.146944927124)*x[1]+(0.00953973207152)*x[2]**o+(0.423262196414)*x[2]
        arg[1,0,1]=(-0.258843593921)*x[0]**o+(-0.950574260099)*x[0]+(-0.49096256134)*x[1]**o+(0.446238902975)*x[1]+(-0.880706204083)*x[2]**o+(-0.400075002454)*x[2]
        arg[1,1,0]=(-0.854529145157)*x[0]**o+(0.118165533114)*x[0]+(-0.913218271084)*x[1]**o+(0.991977570414)*x[1]+(0.626241160161)*x[2]**o+(0.667843914102)*x[2]
        arg[1,1,1]=(0.636818650335)*x[0]**o+(-0.174996100829)*x[0]+(0.77317621688)*x[1]**o+(0.369280208456)*x[1]+(-0.0130124322187)*x[2]**o+(0.857098751007)*x[2]
        arg[2,0,0]=(-0.415847479141)*x[0]**o+(0.675986357495)*x[0]+(0.704926614111)*x[1]**o+(-0.166544295134)*x[1]+(-0.00917226654826)*x[2]**o+(-0.896432190739)*x[2]
        arg[2,0,1]=(-0.882383192316)*x[0]**o+(0.11122730635)*x[0]+(-0.693587089789)*x[1]**o+(-0.0905457091132)*x[1]+(-0.610526000157)*x[2]**o+(-0.992542379681)*x[2]
        arg[2,1,0]=(-0.476987072283)*x[0]**o+(-0.450296923812)*x[0]+(0.905270434608)*x[1]**o+(0.976270820516)*x[1]+(-0.233422717373)*x[2]**o+(-0.641253735386)*x[2]
        arg[2,1,1]=(0.873832514958)*x[0]**o+(-0.478059419217)*x[0]+(-0.941650455606)*x[1]**o+(0.307250152442)*x[1]+(0.865160620819)*x[2]**o+(0.961340749451)*x[2]
        arg[3,0,0]=(-0.443837945036)*x[0]**o+(-0.688773058425)*x[0]+(-0.654988618952)*x[1]**o+(-0.259511611372)*x[1]+(-0.931332326494)*x[2]**o+(0.939379631125)*x[2]
        arg[3,0,1]=(-0.181860312567)*x[0]**o+(-0.229066064127)*x[0]+(-0.359408642857)*x[1]**o+(-0.47945907585)*x[1]+(0.312345402782)*x[2]**o+(0.559298232285)*x[2]
        arg[3,1,0]=(-0.372379819015)*x[0]**o+(0.308179383085)*x[0]+(0.123613160006)*x[1]**o+(0.191944822984)*x[1]+(0.950869988808)*x[2]**o+(0.229360419984)*x[2]
        arg[3,1,1]=(-0.803126271269)*x[0]**o+(0.833324260042)*x[0]+(-0.917001746509)*x[1]**o+(-0.649776906898)*x[1]+(0.49624769362)*x[2]**o+(0.414080802594)*x[2]
        arg[4,0,0]=(-0.951747333891)*x[0]**o+(0.5919442178)*x[0]+(0.121310623341)*x[1]**o+(-0.321939845724)*x[1]+(0.358051737083)*x[2]**o+(-0.222801347833)*x[2]
        arg[4,0,1]=(-0.59462534105)*x[0]**o+(0.943860617698)*x[0]+(0.787822231831)*x[1]**o+(-0.0913435772425)*x[1]+(0.831651946729)*x[2]**o+(-0.903604465879)*x[2]
        arg[4,1,0]=(0.68655463207)*x[0]**o+(-0.98918689213)*x[0]+(0.251300089098)*x[1]**o+(-0.317828131349)*x[1]+(-0.105934216764)*x[2]**o+(-0.874506361945)*x[2]
        arg[4,1,1]=(-0.80544616293)*x[0]**o+(-0.321523684723)*x[0]+(0.874455533992)*x[1]**o+(0.200912811442)*x[1]+(-0.646776981862)*x[2]**o+(-0.988728118812)*x[2]
        arg[5,0,0]=(0.413387141328)*x[0]**o+(0.349210032211)*x[0]+(0.80870689028)*x[1]**o+(-0.633296311752)*x[1]+(0.24621029112)*x[2]**o+(0.853398479355)*x[2]
        arg[5,0,1]=(0.398787817372)*x[0]**o+(0.662875239272)*x[0]+(-0.719962467558)*x[1]**o+(0.49562860294)*x[1]+(-0.694859272211)*x[2]**o+(-0.792616661463)*x[2]
        arg[5,1,0]=(-0.298838035128)*x[0]**o+(0.121578680468)*x[0]+(-0.931062588633)*x[1]**o+(-0.0575832984853)*x[1]+(-0.696140085453)*x[2]**o+(0.836738246058)*x[2]
        arg[5,1,1]=(-0.949626960081)*x[0]**o+(-0.449843145228)*x[0]+(-0.304260073311)*x[1]**o+(-0.258880519466)*x[1]+(0.880466679751)*x[2]**o+(0.97505579364)*x[2]
        ref[0,0,0]=(0.369058215952)/(o+1.)+(-0.860734549029)+(0.638389367885)*0.5**o
        ref[0,0,1]=(-0.937845669147)/(o+1.)+(1.14543097594)+(0.859011160063)*0.5**o
        ref[0,1,0]=(-0.725061432954)/(o+1.)+(0.291104548855)+(-0.998681640062)*0.5**o
        ref[0,1,1]=(-0.540637249448)/(o+1.)+(-0.500606905428)+(0.1340261052)*0.5**o
        ref[1,0,0]=(0.902262161102)/(o+1.)+(0.666322787672)+(0.251201824008)*0.5**o
        ref[1,0,1]=(-1.37166876542)/(o+1.)+(-0.452205179789)+(-0.258843593921)*0.5**o
        ref[1,1,0]=(-0.286977110923)/(o+1.)+(0.888993508815)+(-0.854529145157)*0.5**o
        ref[1,1,1]=(0.760163784662)/(o+1.)+(0.525691429317)+(0.636818650335)*0.5**o
        ref[2,0,0]=(0.695754347563)/(o+1.)+(-0.193495064189)+(-0.415847479141)*0.5**o
        ref[2,0,1]=(-1.30411308995)/(o+1.)+(-0.485930391222)+(-0.882383192316)*0.5**o
        ref[2,1,0]=(0.671847717235)/(o+1.)+(-0.0576399193408)+(-0.476987072283)*0.5**o
        ref[2,1,1]=(-0.0764898347868)/(o+1.)+(0.395265741338)+(0.873832514958)*0.5**o
        ref[3,0,0]=(-1.58632094545)/(o+1.)+(-0.00445251933618)+(-0.443837945036)*0.5**o
        ref[3,0,1]=(-0.0470632400747)/(o+1.)+(-0.0746134538458)+(-0.181860312567)*0.5**o
        ref[3,1,0]=(1.07448314881)/(o+1.)+(0.364742313027)+(-0.372379819015)*0.5**o
        ref[3,1,1]=(-0.420754052889)/(o+1.)+(0.298814077869)+(-0.803126271269)*0.5**o
        ref[4,0,0]=(0.479362360424)/(o+1.)+(0.0236015121217)+(-0.951747333891)*0.5**o
        ref[4,0,1]=(1.61947417856)/(o+1.)+(-0.0255437127116)+(-0.59462534105)*0.5**o
        ref[4,1,0]=(0.145365872334)/(o+1.)+(-1.09076069271)+(0.68655463207)*0.5**o
        ref[4,1,1]=(0.22767855213)/(o+1.)+(-0.554669496046)+(-0.80544616293)*0.5**o
        ref[5,0,0]=(1.0549171814)/(o+1.)+(0.284656099907)+(0.413387141328)*0.5**o
        ref[5,0,1]=(-1.41482173977)/(o+1.)+(0.182943590374)+(0.398787817372)*0.5**o
        ref[5,1,0]=(-1.62720267409)/(o+1.)+(0.450366814021)+(-0.298838035128)*0.5**o
        ref[5,1,1]=(0.576206606439)/(o+1.)+(0.133166064473)+(-0.949626960081)*0.5**o
      res=integrate(arg)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactZero_fromData_FunctionOnContactZero_rank4(self):
      """
      tests integral of rank 4 Data on the FunctionOnContactZero

      assumptions: FunctionOnContactZero(self.domain) exists
                   self.domain supports integral on FunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      w=FunctionOnContactZero(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5, 3, 2),w)
      ref=numpy.zeros((4, 5, 3, 2),numpy.float_)
      if dim==2:
        arg[0,0,0,0]=(0.392628958003)*x[0]**o+(0.186084293395)*x[0]+(0.965300679335)*x[1]**o+(0.0986282244747)*x[1]
        arg[0,0,0,1]=(-0.503312425492)*x[0]**o+(-0.205791466802)*x[0]+(0.384782163488)*x[1]**o+(-0.608371097114)*x[1]
        arg[0,0,1,0]=(0.660162511699)*x[0]**o+(-0.263141686561)*x[0]+(-0.765235012795)*x[1]**o+(-0.371233144611)*x[1]
        arg[0,0,1,1]=(-0.986935412732)*x[0]**o+(0.645433351743)*x[0]+(-0.451665346257)*x[1]**o+(-0.713815759872)*x[1]
        arg[0,0,2,0]=(0.632875178502)*x[0]**o+(-0.257542208821)*x[0]+(-0.695553902133)*x[1]**o+(-0.677920876645)*x[1]
        arg[0,0,2,1]=(0.102625648731)*x[0]**o+(0.511896902175)*x[0]+(0.345909088853)*x[1]**o+(-0.0609413722414)*x[1]
        arg[0,1,0,0]=(-0.0108106931425)*x[0]**o+(-0.595497517075)*x[0]+(0.332480141972)*x[1]**o+(-0.18131109924)*x[1]
        arg[0,1,0,1]=(0.220826593491)*x[0]**o+(-0.202422451222)*x[0]+(0.328480565578)*x[1]**o+(0.851432750557)*x[1]
        arg[0,1,1,0]=(0.22735402466)*x[0]**o+(0.51437836535)*x[0]+(0.292927451477)*x[1]**o+(-0.148121856023)*x[1]
        arg[0,1,1,1]=(0.495079875966)*x[0]**o+(0.582755117761)*x[0]+(-0.322061131008)*x[1]**o+(0.953437449986)*x[1]
        arg[0,1,2,0]=(0.968583889554)*x[0]**o+(-0.593253483731)*x[0]+(0.286037351205)*x[1]**o+(0.997314411263)*x[1]
        arg[0,1,2,1]=(0.361498191707)*x[0]**o+(0.633275777697)*x[0]+(0.891305263219)*x[1]**o+(-0.194468605638)*x[1]
        arg[0,2,0,0]=(0.185647566516)*x[0]**o+(0.101448861247)*x[0]+(0.0870608077911)*x[1]**o+(-0.86477908948)*x[1]
        arg[0,2,0,1]=(0.227262327826)*x[0]**o+(0.812586702875)*x[0]+(-0.0229924234496)*x[1]**o+(-0.323581477982)*x[1]
        arg[0,2,1,0]=(0.0405753322301)*x[0]**o+(0.891289727227)*x[0]+(0.957377329048)*x[1]**o+(-0.982119724379)*x[1]
        arg[0,2,1,1]=(-0.546046301701)*x[0]**o+(0.628511534929)*x[0]+(-0.628262263897)*x[1]**o+(-0.454315708999)*x[1]
        arg[0,2,2,0]=(-0.828455793175)*x[0]**o+(0.190147126828)*x[0]+(0.499595322279)*x[1]**o+(-0.0882297087937)*x[1]
        arg[0,2,2,1]=(0.926138085222)*x[0]**o+(0.402659001583)*x[0]+(-0.678432418122)*x[1]**o+(0.236054704896)*x[1]
        arg[0,3,0,0]=(-0.473066058658)*x[0]**o+(0.57622727181)*x[0]+(0.62665109546)*x[1]**o+(0.948409659533)*x[1]
        arg[0,3,0,1]=(0.776790978673)*x[0]**o+(-0.0244752180934)*x[0]+(-0.459252843816)*x[1]**o+(0.616203849528)*x[1]
        arg[0,3,1,0]=(0.34783214571)*x[0]**o+(0.806528120142)*x[0]+(0.445864469556)*x[1]**o+(-0.221961922099)*x[1]
        arg[0,3,1,1]=(0.0717966037463)*x[0]**o+(-0.434117388366)*x[0]+(0.577708844669)*x[1]**o+(-0.548519157145)*x[1]
        arg[0,3,2,0]=(-0.548573971799)*x[0]**o+(-0.926704595492)*x[0]+(-0.33936340707)*x[1]**o+(0.0687290934591)*x[1]
        arg[0,3,2,1]=(-0.28309240461)*x[0]**o+(0.12914675768)*x[0]+(-0.833323497891)*x[1]**o+(-0.577727985559)*x[1]
        arg[0,4,0,0]=(-0.105416428922)*x[0]**o+(0.41564239907)*x[0]+(0.146657173319)*x[1]**o+(-0.0141417271845)*x[1]
        arg[0,4,0,1]=(-0.876111094069)*x[0]**o+(0.491957669628)*x[0]+(0.534941970665)*x[1]**o+(-0.995507289274)*x[1]
        arg[0,4,1,0]=(-0.678996394303)*x[0]**o+(0.183159947613)*x[0]+(-0.739368558349)*x[1]**o+(0.274319401135)*x[1]
        arg[0,4,1,1]=(0.0472145581152)*x[0]**o+(-0.669737608207)*x[0]+(-0.651647023142)*x[1]**o+(0.351208329111)*x[1]
        arg[0,4,2,0]=(0.114936857967)*x[0]**o+(0.564688790898)*x[0]+(-0.524234284672)*x[1]**o+(-0.127615425724)*x[1]
        arg[0,4,2,1]=(0.568834117773)*x[0]**o+(-0.936513147599)*x[0]+(0.133122220811)*x[1]**o+(0.31510090512)*x[1]
        arg[1,0,0,0]=(-0.821241886235)*x[0]**o+(0.779734235239)*x[0]+(-0.132449009841)*x[1]**o+(-0.0179700618848)*x[1]
        arg[1,0,0,1]=(0.624372422235)*x[0]**o+(-0.752831611955)*x[0]+(-0.573597121344)*x[1]**o+(0.376463665377)*x[1]
        arg[1,0,1,0]=(-0.568749808355)*x[0]**o+(-0.697521100346)*x[0]+(-0.249786553024)*x[1]**o+(-0.624593568423)*x[1]
        arg[1,0,1,1]=(0.0765557360835)*x[0]**o+(-0.744005823966)*x[0]+(0.59246455477)*x[1]**o+(0.231310687774)*x[1]
        arg[1,0,2,0]=(0.909614864187)*x[0]**o+(0.350543076144)*x[0]+(0.0689432766231)*x[1]**o+(-0.88465698758)*x[1]
        arg[1,0,2,1]=(0.0610206203468)*x[0]**o+(0.620401328094)*x[0]+(0.378273922837)*x[1]**o+(0.91988744934)*x[1]
        arg[1,1,0,0]=(0.917249169051)*x[0]**o+(0.756656473206)*x[0]+(0.728786835113)*x[1]**o+(0.116994159781)*x[1]
        arg[1,1,0,1]=(0.331129708654)*x[0]**o+(-0.14342302905)*x[0]+(-0.418402258918)*x[1]**o+(-0.808676850609)*x[1]
        arg[1,1,1,0]=(-0.527408405279)*x[0]**o+(-0.671440239997)*x[0]+(0.989603613532)*x[1]**o+(0.364280392731)*x[1]
        arg[1,1,1,1]=(-0.189543145217)*x[0]**o+(0.14762392216)*x[0]+(0.11459047938)*x[1]**o+(-0.760477267267)*x[1]
        arg[1,1,2,0]=(0.385274935809)*x[0]**o+(0.527744200995)*x[0]+(-0.862654310338)*x[1]**o+(0.904005467598)*x[1]
        arg[1,1,2,1]=(-0.380421555378)*x[0]**o+(0.598300323394)*x[0]+(-0.541819816686)*x[1]**o+(0.446934572528)*x[1]
        arg[1,2,0,0]=(0.446781075767)*x[0]**o+(0.906830344178)*x[0]+(0.158734254777)*x[1]**o+(-0.722982586936)*x[1]
        arg[1,2,0,1]=(0.927811104869)*x[0]**o+(-0.446795135388)*x[0]+(-0.90301726295)*x[1]**o+(0.289530807979)*x[1]
        arg[1,2,1,0]=(0.741527109588)*x[0]**o+(-0.370029728344)*x[0]+(0.157695458386)*x[1]**o+(-0.620804465293)*x[1]
        arg[1,2,1,1]=(0.193242816889)*x[0]**o+(0.455789913444)*x[0]+(0.0491001091271)*x[1]**o+(-0.942127264151)*x[1]
        arg[1,2,2,0]=(0.220709813709)*x[0]**o+(-0.713472965425)*x[0]+(-0.736262418061)*x[1]**o+(0.341347605023)*x[1]
        arg[1,2,2,1]=(0.0034506429721)*x[0]**o+(0.397289753453)*x[0]+(-0.167013881353)*x[1]**o+(0.390364445974)*x[1]
        arg[1,3,0,0]=(0.321471168172)*x[0]**o+(-0.237896433618)*x[0]+(-0.199843421009)*x[1]**o+(0.811319091471)*x[1]
        arg[1,3,0,1]=(-0.0799622908037)*x[0]**o+(-0.66155883346)*x[0]+(0.0303112997369)*x[1]**o+(-0.199256497975)*x[1]
        arg[1,3,1,0]=(-0.12051943489)*x[0]**o+(0.674374603779)*x[0]+(0.335933608833)*x[1]**o+(0.366671959553)*x[1]
        arg[1,3,1,1]=(-0.587242845986)*x[0]**o+(-0.495652656487)*x[0]+(0.10609614346)*x[1]**o+(0.177210588935)*x[1]
        arg[1,3,2,0]=(0.947782336638)*x[0]**o+(-0.327283099142)*x[0]+(-0.0280648836948)*x[1]**o+(0.133334543825)*x[1]
        arg[1,3,2,1]=(-0.0887414134508)*x[0]**o+(0.640694455701)*x[0]+(0.266916366237)*x[1]**o+(0.345886606493)*x[1]
        arg[1,4,0,0]=(-0.138184872938)*x[0]**o+(0.658635897645)*x[0]+(-0.00526240442626)*x[1]**o+(0.800443113371)*x[1]
        arg[1,4,0,1]=(-0.549831917928)*x[0]**o+(-0.666733703837)*x[0]+(0.400982699808)*x[1]**o+(0.739099662817)*x[1]
        arg[1,4,1,0]=(-0.137046722563)*x[0]**o+(-0.818450480933)*x[0]+(-0.827665726293)*x[1]**o+(-0.286585607624)*x[1]
        arg[1,4,1,1]=(-0.474156202572)*x[0]**o+(-0.855381707793)*x[0]+(0.113318564035)*x[1]**o+(0.218410511108)*x[1]
        arg[1,4,2,0]=(0.18476112439)*x[0]**o+(-0.964533442641)*x[0]+(-0.968611931598)*x[1]**o+(0.0973026144203)*x[1]
        arg[1,4,2,1]=(-0.372995418552)*x[0]**o+(0.998322146553)*x[0]+(-0.384672070744)*x[1]**o+(0.47536977268)*x[1]
        arg[2,0,0,0]=(0.882400857222)*x[0]**o+(0.455324640128)*x[0]+(0.764297272234)*x[1]**o+(0.498669000791)*x[1]
        arg[2,0,0,1]=(0.894017590794)*x[0]**o+(0.217984119634)*x[0]+(-0.0934000967188)*x[1]**o+(-0.886960376026)*x[1]
        arg[2,0,1,0]=(-0.382158057826)*x[0]**o+(-0.897639065996)*x[0]+(0.819002513335)*x[1]**o+(-0.0739374877518)*x[1]
        arg[2,0,1,1]=(-0.259771272131)*x[0]**o+(-0.533540086977)*x[0]+(0.366917477923)*x[1]**o+(-0.751904525722)*x[1]
        arg[2,0,2,0]=(-0.33946164111)*x[0]**o+(-0.317441178182)*x[0]+(-0.857625945147)*x[1]**o+(-0.84647892719)*x[1]
        arg[2,0,2,1]=(0.153353093148)*x[0]**o+(0.832883645984)*x[0]+(-0.359476461654)*x[1]**o+(0.332390899654)*x[1]
        arg[2,1,0,0]=(0.780980225775)*x[0]**o+(0.619330950566)*x[0]+(0.194312746335)*x[1]**o+(0.595311976655)*x[1]
        arg[2,1,0,1]=(-0.334149973283)*x[0]**o+(-0.670178805858)*x[0]+(-0.313038325156)*x[1]**o+(0.793149618246)*x[1]
        arg[2,1,1,0]=(-0.832852528577)*x[0]**o+(-0.606634692926)*x[0]+(-0.0643816133251)*x[1]**o+(-0.663765987285)*x[1]
        arg[2,1,1,1]=(0.300975732963)*x[0]**o+(0.965431406497)*x[0]+(-0.618456119184)*x[1]**o+(0.459530421885)*x[1]
        arg[2,1,2,0]=(0.122326105776)*x[0]**o+(0.0940802696378)*x[0]+(-0.986852501828)*x[1]**o+(0.370428043236)*x[1]
        arg[2,1,2,1]=(0.447916985436)*x[0]**o+(-0.161926311978)*x[0]+(-0.0933563424166)*x[1]**o+(0.836290263421)*x[1]
        arg[2,2,0,0]=(0.032076347277)*x[0]**o+(-0.447517143643)*x[0]+(-0.053769157756)*x[1]**o+(-0.0242981207801)*x[1]
        arg[2,2,0,1]=(-0.403935703163)*x[0]**o+(-0.960750093184)*x[0]+(-0.287956277998)*x[1]**o+(-0.168614701082)*x[1]
        arg[2,2,1,0]=(-0.205077009492)*x[0]**o+(-0.409536289697)*x[0]+(0.359071620541)*x[1]**o+(-0.466455759025)*x[1]
        arg[2,2,1,1]=(0.443658057141)*x[0]**o+(-0.748681939527)*x[0]+(0.2153167916)*x[1]**o+(-0.475899799738)*x[1]
        arg[2,2,2,0]=(-0.463879681251)*x[0]**o+(0.758317222041)*x[0]+(0.608221916958)*x[1]**o+(-0.237798399085)*x[1]
        arg[2,2,2,1]=(0.533981754168)*x[0]**o+(0.590125912931)*x[0]+(0.8957267808)*x[1]**o+(0.294612478767)*x[1]
        arg[2,3,0,0]=(0.499387701403)*x[0]**o+(0.0379132293822)*x[0]+(0.717384215206)*x[1]**o+(0.754497698503)*x[1]
        arg[2,3,0,1]=(0.866500398882)*x[0]**o+(0.23678788012)*x[0]+(0.395132533716)*x[1]**o+(-0.451619930593)*x[1]
        arg[2,3,1,0]=(-0.879663354176)*x[0]**o+(-0.328937922978)*x[0]+(0.688771917581)*x[1]**o+(-0.87392051489)*x[1]
        arg[2,3,1,1]=(0.291214284356)*x[0]**o+(-0.783159133488)*x[0]+(0.240960084233)*x[1]**o+(0.339043900275)*x[1]
        arg[2,3,2,0]=(0.47064755192)*x[0]**o+(-0.42618815064)*x[0]+(-0.505342655776)*x[1]**o+(0.408218280069)*x[1]
        arg[2,3,2,1]=(0.775731405735)*x[0]**o+(-0.768750178591)*x[0]+(-0.896333275063)*x[1]**o+(0.286558848957)*x[1]
        arg[2,4,0,0]=(-0.7221248688)*x[0]**o+(0.612509823967)*x[0]+(0.436744528143)*x[1]**o+(-0.761502758764)*x[1]
        arg[2,4,0,1]=(-0.742652222385)*x[0]**o+(0.214811997988)*x[0]+(-0.031320962337)*x[1]**o+(-0.320134664789)*x[1]
        arg[2,4,1,0]=(-0.529575527815)*x[0]**o+(0.623770482091)*x[0]+(-0.50790148262)*x[1]**o+(0.629874274271)*x[1]
        arg[2,4,1,1]=(0.535631957883)*x[0]**o+(0.439578618993)*x[0]+(-0.770747718564)*x[1]**o+(0.0798696116058)*x[1]
        arg[2,4,2,0]=(0.81688202721)*x[0]**o+(-0.0301054197201)*x[0]+(0.613495031968)*x[1]**o+(-0.153619455295)*x[1]
        arg[2,4,2,1]=(0.493471708612)*x[0]**o+(0.423626757568)*x[0]+(0.093174141001)*x[1]**o+(0.307452108722)*x[1]
        arg[3,0,0,0]=(-0.0668630243284)*x[0]**o+(-0.577709692979)*x[0]+(0.984307685082)*x[1]**o+(-0.171487030152)*x[1]
        arg[3,0,0,1]=(-0.210452484293)*x[0]**o+(-0.130476372055)*x[0]+(-0.693596086089)*x[1]**o+(-0.692143724329)*x[1]
        arg[3,0,1,0]=(-0.34853285523)*x[0]**o+(-0.369649709966)*x[0]+(0.824648362105)*x[1]**o+(0.733046240399)*x[1]
        arg[3,0,1,1]=(0.861903045965)*x[0]**o+(0.609535692452)*x[0]+(0.681514895836)*x[1]**o+(0.66773275418)*x[1]
        arg[3,0,2,0]=(0.0197670342844)*x[0]**o+(-0.251645539228)*x[0]+(-0.0390293468405)*x[1]**o+(-0.00955605638373)*x[1]
        arg[3,0,2,1]=(-0.499093940395)*x[0]**o+(0.660356623112)*x[0]+(0.969057635866)*x[1]**o+(-0.821377317197)*x[1]
        arg[3,1,0,0]=(-0.955568800095)*x[0]**o+(0.557877698917)*x[0]+(0.829599262959)*x[1]**o+(0.933736802562)*x[1]
        arg[3,1,0,1]=(-0.910307638058)*x[0]**o+(-0.3287646946)*x[0]+(-0.777548579252)*x[1]**o+(-0.971744353969)*x[1]
        arg[3,1,1,0]=(0.22353820834)*x[0]**o+(0.904479243377)*x[0]+(-0.108768166204)*x[1]**o+(0.823767232517)*x[1]
        arg[3,1,1,1]=(0.371995768061)*x[0]**o+(0.747630377273)*x[0]+(0.931152588498)*x[1]**o+(-0.0154330268188)*x[1]
        arg[3,1,2,0]=(0.4567417369)*x[0]**o+(0.424951293892)*x[0]+(-0.14032259215)*x[1]**o+(0.603329614924)*x[1]
        arg[3,1,2,1]=(-0.855100213731)*x[0]**o+(-0.667832882495)*x[0]+(0.942530922378)*x[1]**o+(0.55861156259)*x[1]
        arg[3,2,0,0]=(0.17190936572)*x[0]**o+(-0.904098615762)*x[0]+(0.991497679624)*x[1]**o+(0.373020358381)*x[1]
        arg[3,2,0,1]=(-0.683587206983)*x[0]**o+(0.718257814718)*x[0]+(-0.730243933765)*x[1]**o+(-0.583892189979)*x[1]
        arg[3,2,1,0]=(0.051672736629)*x[0]**o+(-0.991172563807)*x[0]+(0.36605487466)*x[1]**o+(0.680719877794)*x[1]
        arg[3,2,1,1]=(0.438014662973)*x[0]**o+(0.172969450125)*x[0]+(-0.424515284177)*x[1]**o+(0.615315132547)*x[1]
        arg[3,2,2,0]=(-0.709499532674)*x[0]**o+(0.985097212621)*x[0]+(0.326610298255)*x[1]**o+(0.994701669258)*x[1]
        arg[3,2,2,1]=(-0.657647707844)*x[0]**o+(-0.683954483768)*x[0]+(-0.846657945161)*x[1]**o+(0.852308904063)*x[1]
        arg[3,3,0,0]=(0.21346320453)*x[0]**o+(-0.181665119768)*x[0]+(0.549866100727)*x[1]**o+(-0.0858045316871)*x[1]
        arg[3,3,0,1]=(0.249957470685)*x[0]**o+(0.537464773638)*x[0]+(0.0224330511636)*x[1]**o+(0.114623595638)*x[1]
        arg[3,3,1,0]=(0.166507029533)*x[0]**o+(0.232150195873)*x[0]+(0.119453667263)*x[1]**o+(-0.261110960642)*x[1]
        arg[3,3,1,1]=(0.358248720077)*x[0]**o+(-0.871911261059)*x[0]+(0.57743453064)*x[1]**o+(0.950322066335)*x[1]
        arg[3,3,2,0]=(-0.896899724447)*x[0]**o+(-0.898884767716)*x[0]+(0.63832012742)*x[1]**o+(-0.833800375634)*x[1]
        arg[3,3,2,1]=(0.425194254583)*x[0]**o+(0.161328467295)*x[0]+(0.496219405916)*x[1]**o+(0.380935896093)*x[1]
        arg[3,4,0,0]=(0.793902581592)*x[0]**o+(-0.33428511164)*x[0]+(-0.283510438468)*x[1]**o+(0.228913046074)*x[1]
        arg[3,4,0,1]=(0.372028980539)*x[0]**o+(-0.388909011671)*x[0]+(-0.259885631206)*x[1]**o+(-0.950910172897)*x[1]
        arg[3,4,1,0]=(-0.80489673786)*x[0]**o+(-0.828175420038)*x[0]+(0.895070489116)*x[1]**o+(-0.742272132103)*x[1]
        arg[3,4,1,1]=(0.272535126101)*x[0]**o+(0.902668974891)*x[0]+(0.399816667015)*x[1]**o+(-0.953186908789)*x[1]
        arg[3,4,2,0]=(0.0341775288582)*x[0]**o+(0.962241492639)*x[0]+(0.906020184876)*x[1]**o+(-0.400536255726)*x[1]
        arg[3,4,2,1]=(-0.745016464485)*x[0]**o+(-0.85241829747)*x[0]+(-0.648936544722)*x[1]**o+(-0.997030074218)*x[1]
        ref[0,0,0,0]=(0.965300679335)/(o+1.)+(0.142356258935)+(0.392628958003)*0.5**o
        ref[0,0,0,1]=(0.384782163488)/(o+1.)+(-0.407081281958)+(-0.503312425492)*0.5**o
        ref[0,0,1,0]=(-0.765235012795)/(o+1.)+(-0.317187415586)+(0.660162511699)*0.5**o
        ref[0,0,1,1]=(-0.451665346257)/(o+1.)+(-0.0341912040643)+(-0.986935412732)*0.5**o
        ref[0,0,2,0]=(-0.695553902133)/(o+1.)+(-0.467731542733)+(0.632875178502)*0.5**o
        ref[0,0,2,1]=(0.345909088853)/(o+1.)+(0.225477764967)+(0.102625648731)*0.5**o
        ref[0,1,0,0]=(0.332480141972)/(o+1.)+(-0.388404308158)+(-0.0108106931425)*0.5**o
        ref[0,1,0,1]=(0.328480565578)/(o+1.)+(0.324505149668)+(0.220826593491)*0.5**o
        ref[0,1,1,0]=(0.292927451477)/(o+1.)+(0.183128254664)+(0.22735402466)*0.5**o
        ref[0,1,1,1]=(-0.322061131008)/(o+1.)+(0.768096283874)+(0.495079875966)*0.5**o
        ref[0,1,2,0]=(0.286037351205)/(o+1.)+(0.202030463766)+(0.968583889554)*0.5**o
        ref[0,1,2,1]=(0.891305263219)/(o+1.)+(0.219403586029)+(0.361498191707)*0.5**o
        ref[0,2,0,0]=(0.0870608077911)/(o+1.)+(-0.381665114117)+(0.185647566516)*0.5**o
        ref[0,2,0,1]=(-0.0229924234496)/(o+1.)+(0.244502612446)+(0.227262327826)*0.5**o
        ref[0,2,1,0]=(0.957377329048)/(o+1.)+(-0.0454149985758)+(0.0405753322301)*0.5**o
        ref[0,2,1,1]=(-0.628262263897)/(o+1.)+(0.0870979129647)+(-0.546046301701)*0.5**o
        ref[0,2,2,0]=(0.499595322279)/(o+1.)+(0.0509587090171)+(-0.828455793175)*0.5**o
        ref[0,2,2,1]=(-0.678432418122)/(o+1.)+(0.319356853239)+(0.926138085222)*0.5**o
        ref[0,3,0,0]=(0.62665109546)/(o+1.)+(0.762318465671)+(-0.473066058658)*0.5**o
        ref[0,3,0,1]=(-0.459252843816)/(o+1.)+(0.295864315717)+(0.776790978673)*0.5**o
        ref[0,3,1,0]=(0.445864469556)/(o+1.)+(0.292283099022)+(0.34783214571)*0.5**o
        ref[0,3,1,1]=(0.577708844669)/(o+1.)+(-0.491318272755)+(0.0717966037463)*0.5**o
        ref[0,3,2,0]=(-0.33936340707)/(o+1.)+(-0.428987751016)+(-0.548573971799)*0.5**o
        ref[0,3,2,1]=(-0.833323497891)/(o+1.)+(-0.224290613939)+(-0.28309240461)*0.5**o
        ref[0,4,0,0]=(0.146657173319)/(o+1.)+(0.200750335943)+(-0.105416428922)*0.5**o
        ref[0,4,0,1]=(0.534941970665)/(o+1.)+(-0.251774809823)+(-0.876111094069)*0.5**o
        ref[0,4,1,0]=(-0.739368558349)/(o+1.)+(0.228739674374)+(-0.678996394303)*0.5**o
        ref[0,4,1,1]=(-0.651647023142)/(o+1.)+(-0.159264639548)+(0.0472145581152)*0.5**o
        ref[0,4,2,0]=(-0.524234284672)/(o+1.)+(0.218536682587)+(0.114936857967)*0.5**o
        ref[0,4,2,1]=(0.133122220811)/(o+1.)+(-0.31070612124)+(0.568834117773)*0.5**o
        ref[1,0,0,0]=(-0.132449009841)/(o+1.)+(0.380882086677)+(-0.821241886235)*0.5**o
        ref[1,0,0,1]=(-0.573597121344)/(o+1.)+(-0.188183973289)+(0.624372422235)*0.5**o
        ref[1,0,1,0]=(-0.249786553024)/(o+1.)+(-0.661057334385)+(-0.568749808355)*0.5**o
        ref[1,0,1,1]=(0.59246455477)/(o+1.)+(-0.256347568096)+(0.0765557360835)*0.5**o
        ref[1,0,2,0]=(0.0689432766231)/(o+1.)+(-0.267056955718)+(0.909614864187)*0.5**o
        ref[1,0,2,1]=(0.378273922837)/(o+1.)+(0.770144388717)+(0.0610206203468)*0.5**o
        ref[1,1,0,0]=(0.728786835113)/(o+1.)+(0.436825316493)+(0.917249169051)*0.5**o
        ref[1,1,0,1]=(-0.418402258918)/(o+1.)+(-0.47604993983)+(0.331129708654)*0.5**o
        ref[1,1,1,0]=(0.989603613532)/(o+1.)+(-0.153579923633)+(-0.527408405279)*0.5**o
        ref[1,1,1,1]=(0.11459047938)/(o+1.)+(-0.306426672553)+(-0.189543145217)*0.5**o
        ref[1,1,2,0]=(-0.862654310338)/(o+1.)+(0.715874834296)+(0.385274935809)*0.5**o
        ref[1,1,2,1]=(-0.541819816686)/(o+1.)+(0.522617447961)+(-0.380421555378)*0.5**o
        ref[1,2,0,0]=(0.158734254777)/(o+1.)+(0.0919238786215)+(0.446781075767)*0.5**o
        ref[1,2,0,1]=(-0.90301726295)/(o+1.)+(-0.0786321637044)+(0.927811104869)*0.5**o
        ref[1,2,1,0]=(0.157695458386)/(o+1.)+(-0.495417096819)+(0.741527109588)*0.5**o
        ref[1,2,1,1]=(0.0491001091271)/(o+1.)+(-0.243168675353)+(0.193242816889)*0.5**o
        ref[1,2,2,0]=(-0.736262418061)/(o+1.)+(-0.186062680201)+(0.220709813709)*0.5**o
        ref[1,2,2,1]=(-0.167013881353)/(o+1.)+(0.393827099713)+(0.0034506429721)*0.5**o
        ref[1,3,0,0]=(-0.199843421009)/(o+1.)+(0.286711328927)+(0.321471168172)*0.5**o
        ref[1,3,0,1]=(0.0303112997369)/(o+1.)+(-0.430407665717)+(-0.0799622908037)*0.5**o
        ref[1,3,1,0]=(0.335933608833)/(o+1.)+(0.520523281666)+(-0.12051943489)*0.5**o
        ref[1,3,1,1]=(0.10609614346)/(o+1.)+(-0.159221033776)+(-0.587242845986)*0.5**o
        ref[1,3,2,0]=(-0.0280648836948)/(o+1.)+(-0.0969742776586)+(0.947782336638)*0.5**o
        ref[1,3,2,1]=(0.266916366237)/(o+1.)+(0.493290531097)+(-0.0887414134508)*0.5**o
        ref[1,4,0,0]=(-0.00526240442626)/(o+1.)+(0.729539505508)+(-0.138184872938)*0.5**o
        ref[1,4,0,1]=(0.400982699808)/(o+1.)+(0.0361829794896)+(-0.549831917928)*0.5**o
        ref[1,4,1,0]=(-0.827665726293)/(o+1.)+(-0.552518044279)+(-0.137046722563)*0.5**o
        ref[1,4,1,1]=(0.113318564035)/(o+1.)+(-0.318485598343)+(-0.474156202572)*0.5**o
        ref[1,4,2,0]=(-0.968611931598)/(o+1.)+(-0.433615414111)+(0.18476112439)*0.5**o
        ref[1,4,2,1]=(-0.384672070744)/(o+1.)+(0.736845959617)+(-0.372995418552)*0.5**o
        ref[2,0,0,0]=(0.764297272234)/(o+1.)+(0.476996820459)+(0.882400857222)*0.5**o
        ref[2,0,0,1]=(-0.0934000967188)/(o+1.)+(-0.334488128196)+(0.894017590794)*0.5**o
        ref[2,0,1,0]=(0.819002513335)/(o+1.)+(-0.485788276874)+(-0.382158057826)*0.5**o
        ref[2,0,1,1]=(0.366917477923)/(o+1.)+(-0.64272230635)+(-0.259771272131)*0.5**o
        ref[2,0,2,0]=(-0.857625945147)/(o+1.)+(-0.581960052686)+(-0.33946164111)*0.5**o
        ref[2,0,2,1]=(-0.359476461654)/(o+1.)+(0.582637272819)+(0.153353093148)*0.5**o
        ref[2,1,0,0]=(0.194312746335)/(o+1.)+(0.607321463611)+(0.780980225775)*0.5**o
        ref[2,1,0,1]=(-0.313038325156)/(o+1.)+(0.0614854061938)+(-0.334149973283)*0.5**o
        ref[2,1,1,0]=(-0.0643816133251)/(o+1.)+(-0.635200340106)+(-0.832852528577)*0.5**o
        ref[2,1,1,1]=(-0.618456119184)/(o+1.)+(0.712480914191)+(0.300975732963)*0.5**o
        ref[2,1,2,0]=(-0.986852501828)/(o+1.)+(0.232254156437)+(0.122326105776)*0.5**o
        ref[2,1,2,1]=(-0.0933563424166)/(o+1.)+(0.337181975722)+(0.447916985436)*0.5**o
        ref[2,2,0,0]=(-0.053769157756)/(o+1.)+(-0.235907632211)+(0.032076347277)*0.5**o
        ref[2,2,0,1]=(-0.287956277998)/(o+1.)+(-0.564682397133)+(-0.403935703163)*0.5**o
        ref[2,2,1,0]=(0.359071620541)/(o+1.)+(-0.437996024361)+(-0.205077009492)*0.5**o
        ref[2,2,1,1]=(0.2153167916)/(o+1.)+(-0.612290869632)+(0.443658057141)*0.5**o
        ref[2,2,2,0]=(0.608221916958)/(o+1.)+(0.260259411478)+(-0.463879681251)*0.5**o
        ref[2,2,2,1]=(0.8957267808)/(o+1.)+(0.442369195849)+(0.533981754168)*0.5**o
        ref[2,3,0,0]=(0.717384215206)/(o+1.)+(0.396205463942)+(0.499387701403)*0.5**o
        ref[2,3,0,1]=(0.395132533716)/(o+1.)+(-0.107416025237)+(0.866500398882)*0.5**o
        ref[2,3,1,0]=(0.688771917581)/(o+1.)+(-0.601429218934)+(-0.879663354176)*0.5**o
        ref[2,3,1,1]=(0.240960084233)/(o+1.)+(-0.222057616607)+(0.291214284356)*0.5**o
        ref[2,3,2,0]=(-0.505342655776)/(o+1.)+(-0.00898493528542)+(0.47064755192)*0.5**o
        ref[2,3,2,1]=(-0.896333275063)/(o+1.)+(-0.241095664817)+(0.775731405735)*0.5**o
        ref[2,4,0,0]=(0.436744528143)/(o+1.)+(-0.0744964673986)+(-0.7221248688)*0.5**o
        ref[2,4,0,1]=(-0.031320962337)/(o+1.)+(-0.0526613334003)+(-0.742652222385)*0.5**o
        ref[2,4,1,0]=(-0.50790148262)/(o+1.)+(0.626822378181)+(-0.529575527815)*0.5**o
        ref[2,4,1,1]=(-0.770747718564)/(o+1.)+(0.2597241153)+(0.535631957883)*0.5**o
        ref[2,4,2,0]=(0.613495031968)/(o+1.)+(-0.0918624375077)+(0.81688202721)*0.5**o
        ref[2,4,2,1]=(0.093174141001)/(o+1.)+(0.365539433145)+(0.493471708612)*0.5**o
        ref[3,0,0,0]=(0.984307685082)/(o+1.)+(-0.374598361566)+(-0.0668630243284)*0.5**o
        ref[3,0,0,1]=(-0.693596086089)/(o+1.)+(-0.411310048192)+(-0.210452484293)*0.5**o
        ref[3,0,1,0]=(0.824648362105)/(o+1.)+(0.181698265217)+(-0.34853285523)*0.5**o
        ref[3,0,1,1]=(0.681514895836)/(o+1.)+(0.638634223316)+(0.861903045965)*0.5**o
        ref[3,0,2,0]=(-0.0390293468405)/(o+1.)+(-0.130600797806)+(0.0197670342844)*0.5**o
        ref[3,0,2,1]=(0.969057635866)/(o+1.)+(-0.0805103470423)+(-0.499093940395)*0.5**o
        ref[3,1,0,0]=(0.829599262959)/(o+1.)+(0.745807250739)+(-0.955568800095)*0.5**o
        ref[3,1,0,1]=(-0.777548579252)/(o+1.)+(-0.650254524284)+(-0.910307638058)*0.5**o
        ref[3,1,1,0]=(-0.108768166204)/(o+1.)+(0.864123237947)+(0.22353820834)*0.5**o
        ref[3,1,1,1]=(0.931152588498)/(o+1.)+(0.366098675227)+(0.371995768061)*0.5**o
        ref[3,1,2,0]=(-0.14032259215)/(o+1.)+(0.514140454408)+(0.4567417369)*0.5**o
        ref[3,1,2,1]=(0.942530922378)/(o+1.)+(-0.0546106599524)+(-0.855100213731)*0.5**o
        ref[3,2,0,0]=(0.991497679624)/(o+1.)+(-0.26553912869)+(0.17190936572)*0.5**o
        ref[3,2,0,1]=(-0.730243933765)/(o+1.)+(0.0671828123695)+(-0.683587206983)*0.5**o
        ref[3,2,1,0]=(0.36605487466)/(o+1.)+(-0.155226343006)+(0.051672736629)*0.5**o
        ref[3,2,1,1]=(-0.424515284177)/(o+1.)+(0.394142291336)+(0.438014662973)*0.5**o
        ref[3,2,2,0]=(0.326610298255)/(o+1.)+(0.98989944094)+(-0.709499532674)*0.5**o
        ref[3,2,2,1]=(-0.846657945161)/(o+1.)+(0.0841772101477)+(-0.657647707844)*0.5**o
        ref[3,3,0,0]=(0.549866100727)/(o+1.)+(-0.133734825728)+(0.21346320453)*0.5**o
        ref[3,3,0,1]=(0.0224330511636)/(o+1.)+(0.326044184638)+(0.249957470685)*0.5**o
        ref[3,3,1,0]=(0.119453667263)/(o+1.)+(-0.0144803823844)+(0.166507029533)*0.5**o
        ref[3,3,1,1]=(0.57743453064)/(o+1.)+(0.0392054026384)+(0.358248720077)*0.5**o
        ref[3,3,2,0]=(0.63832012742)/(o+1.)+(-0.866342571675)+(-0.896899724447)*0.5**o
        ref[3,3,2,1]=(0.496219405916)/(o+1.)+(0.271132181694)+(0.425194254583)*0.5**o
        ref[3,4,0,0]=(-0.283510438468)/(o+1.)+(-0.0526860327828)+(0.793902581592)*0.5**o
        ref[3,4,0,1]=(-0.259885631206)/(o+1.)+(-0.669909592284)+(0.372028980539)*0.5**o
        ref[3,4,1,0]=(0.895070489116)/(o+1.)+(-0.78522377607)+(-0.80489673786)*0.5**o
        ref[3,4,1,1]=(0.399816667015)/(o+1.)+(-0.0252589669489)+(0.272535126101)*0.5**o
        ref[3,4,2,0]=(0.906020184876)/(o+1.)+(0.280852618457)+(0.0341775288582)*0.5**o
        ref[3,4,2,1]=(-0.648936544722)/(o+1.)+(-0.924724185844)+(-0.745016464485)*0.5**o
      else:
        arg[0,0,0,0]=(-0.891975745754)*x[0]**o+(0.678331209591)*x[0]+(-0.450022305538)*x[1]**o+(0.521272586024)*x[1]+(-0.28868708769)*x[2]**o+(-0.631617544339)*x[2]
        arg[0,0,0,1]=(0.428698930349)*x[0]**o+(-0.856251438077)*x[0]+(0.915766367729)*x[1]**o+(-0.35019463199)*x[1]+(-0.0029340113102)*x[2]**o+(-0.394458505835)*x[2]
        arg[0,0,1,0]=(0.974069118653)*x[0]**o+(-0.674028781962)*x[0]+(0.489718068973)*x[1]**o+(-0.471323892144)*x[1]+(0.11469271678)*x[2]**o+(-0.264584383354)*x[2]
        arg[0,0,1,1]=(0.813412823502)*x[0]**o+(0.739276532652)*x[0]+(-0.863216628203)*x[1]**o+(0.486539683681)*x[1]+(-0.771448803712)*x[2]**o+(-0.621526889056)*x[2]
        arg[0,0,2,0]=(-0.874502916672)*x[0]**o+(0.0708047915752)*x[0]+(0.917576817333)*x[1]**o+(-0.257626031306)*x[1]+(-0.161822453369)*x[2]**o+(0.488866912786)*x[2]
        arg[0,0,2,1]=(-0.258524075807)*x[0]**o+(0.206589504672)*x[0]+(0.232971263871)*x[1]**o+(-0.917991566808)*x[1]+(0.725943193794)*x[2]**o+(-0.170341108992)*x[2]
        arg[0,1,0,0]=(0.489505903018)*x[0]**o+(0.386376068728)*x[0]+(0.510797276159)*x[1]**o+(0.45356046467)*x[1]+(-0.664963144251)*x[2]**o+(0.128579947316)*x[2]
        arg[0,1,0,1]=(0.441567332096)*x[0]**o+(0.658686083904)*x[0]+(0.46601056348)*x[1]**o+(-0.130759928108)*x[1]+(0.324576589)*x[2]**o+(0.844700852997)*x[2]
        arg[0,1,1,0]=(0.388576628492)*x[0]**o+(0.41731399682)*x[0]+(-0.37037879729)*x[1]**o+(-0.00306376860454)*x[1]+(-0.464420131682)*x[2]**o+(0.570470055678)*x[2]
        arg[0,1,1,1]=(-0.0537780037583)*x[0]**o+(0.641854502644)*x[0]+(-0.475355608842)*x[1]**o+(-0.155501886306)*x[1]+(0.159833591997)*x[2]**o+(0.889951997355)*x[2]
        arg[0,1,2,0]=(0.58262140787)*x[0]**o+(0.487380368384)*x[0]+(-0.783228845976)*x[1]**o+(-0.683221249176)*x[1]+(0.98628901689)*x[2]**o+(0.591884616904)*x[2]
        arg[0,1,2,1]=(0.438472233911)*x[0]**o+(0.989088416794)*x[0]+(0.221688966923)*x[1]**o+(-0.119266918098)*x[1]+(-0.887421255415)*x[2]**o+(0.728082459156)*x[2]
        arg[0,2,0,0]=(-0.334110476968)*x[0]**o+(-0.877094545216)*x[0]+(-0.293135209996)*x[1]**o+(0.578110480224)*x[1]+(-0.898686340104)*x[2]**o+(0.528072783107)*x[2]
        arg[0,2,0,1]=(0.33041258354)*x[0]**o+(-0.70349630629)*x[0]+(0.349551161792)*x[1]**o+(-0.296687014146)*x[1]+(-0.272377440466)*x[2]**o+(-0.42221458274)*x[2]
        arg[0,2,1,0]=(-0.80021736912)*x[0]**o+(0.271727608951)*x[0]+(0.689351606243)*x[1]**o+(-0.95063104158)*x[1]+(0.447293221661)*x[2]**o+(0.962435109303)*x[2]
        arg[0,2,1,1]=(-0.773331228218)*x[0]**o+(0.569225340051)*x[0]+(-0.0330042729183)*x[1]**o+(-0.732867386826)*x[1]+(-0.399631128084)*x[2]**o+(0.235907094874)*x[2]
        arg[0,2,2,0]=(-0.943072868965)*x[0]**o+(-0.959364749946)*x[0]+(0.922934300445)*x[1]**o+(0.27257203243)*x[1]+(0.450453864748)*x[2]**o+(0.521284310775)*x[2]
        arg[0,2,2,1]=(-0.821201079286)*x[0]**o+(-0.801011107627)*x[0]+(0.554575277011)*x[1]**o+(-0.411361733513)*x[1]+(-0.858195570474)*x[2]**o+(0.420660103079)*x[2]
        arg[0,3,0,0]=(-0.0564164524098)*x[0]**o+(-0.865516143298)*x[0]+(-0.85754022189)*x[1]**o+(-0.0159800361584)*x[1]+(-0.891158469429)*x[2]**o+(-0.809909518115)*x[2]
        arg[0,3,0,1]=(-0.553858251068)*x[0]**o+(0.827558838148)*x[0]+(0.690081127613)*x[1]**o+(0.3475090336)*x[1]+(-0.190426828891)*x[2]**o+(-0.99589316535)*x[2]
        arg[0,3,1,0]=(-0.992317453163)*x[0]**o+(-0.992812531491)*x[0]+(-0.859683310869)*x[1]**o+(0.441779587739)*x[1]+(0.582457020078)*x[2]**o+(-0.959111461424)*x[2]
        arg[0,3,1,1]=(-0.713753922213)*x[0]**o+(0.504885148776)*x[0]+(-0.076988545634)*x[1]**o+(0.530828889101)*x[1]+(-0.335616242708)*x[2]**o+(-0.0421293420747)*x[2]
        arg[0,3,2,0]=(0.536024118289)*x[0]**o+(-0.527038052847)*x[0]+(-0.558392588656)*x[1]**o+(0.928618057856)*x[1]+(0.427507510033)*x[2]**o+(0.223570914141)*x[2]
        arg[0,3,2,1]=(-0.197968756117)*x[0]**o+(-0.58157606173)*x[0]+(-0.878630012825)*x[1]**o+(-0.380635552726)*x[1]+(-0.477570149125)*x[2]**o+(-0.642904015434)*x[2]
        arg[0,4,0,0]=(0.688763655127)*x[0]**o+(0.286011523933)*x[0]+(0.0933014250929)*x[1]**o+(0.355550563233)*x[1]+(0.121445291305)*x[2]**o+(-0.814538082974)*x[2]
        arg[0,4,0,1]=(-0.402717742839)*x[0]**o+(0.403946649835)*x[0]+(0.286978800579)*x[1]**o+(-0.363358056216)*x[1]+(-0.185233594102)*x[2]**o+(-0.902660250807)*x[2]
        arg[0,4,1,0]=(0.39352838659)*x[0]**o+(-0.430755011337)*x[0]+(-0.912518631162)*x[1]**o+(-0.634118795867)*x[1]+(-0.802445843416)*x[2]**o+(0.643395158048)*x[2]
        arg[0,4,1,1]=(-0.375507250175)*x[0]**o+(-0.609576942939)*x[0]+(0.35060243693)*x[1]**o+(0.384630130603)*x[1]+(0.165576820334)*x[2]**o+(-0.319012357341)*x[2]
        arg[0,4,2,0]=(0.0603590263818)*x[0]**o+(-0.435954775883)*x[0]+(-0.895329092913)*x[1]**o+(0.364809539103)*x[1]+(0.912864788565)*x[2]**o+(0.582044419963)*x[2]
        arg[0,4,2,1]=(-0.111089454685)*x[0]**o+(-0.247527778755)*x[0]+(0.897263901657)*x[1]**o+(0.168097655548)*x[1]+(-0.532273075067)*x[2]**o+(-0.158529268933)*x[2]
        arg[1,0,0,0]=(-0.940865155622)*x[0]**o+(0.639652431549)*x[0]+(0.204433172026)*x[1]**o+(-0.333435100662)*x[1]+(0.922286591304)*x[2]**o+(0.245091036189)*x[2]
        arg[1,0,0,1]=(-0.843207880309)*x[0]**o+(0.496567621809)*x[0]+(-0.747925907677)*x[1]**o+(-0.666195656311)*x[1]+(-0.836848401917)*x[2]**o+(-0.77301298659)*x[2]
        arg[1,0,1,0]=(-0.484987528212)*x[0]**o+(-0.208798022576)*x[0]+(-0.516295195217)*x[1]**o+(0.273733676249)*x[1]+(0.642865886081)*x[2]**o+(0.94556566928)*x[2]
        arg[1,0,1,1]=(0.285548625366)*x[0]**o+(-0.316870037241)*x[0]+(0.166055429845)*x[1]**o+(-0.907240066063)*x[1]+(-0.973602611383)*x[2]**o+(-0.232655535454)*x[2]
        arg[1,0,2,0]=(0.96797757731)*x[0]**o+(-0.0920831342923)*x[0]+(0.0367163548581)*x[1]**o+(0.372503137376)*x[1]+(0.114902128151)*x[2]**o+(0.147390259838)*x[2]
        arg[1,0,2,1]=(-0.486190283922)*x[0]**o+(-0.713417720041)*x[0]+(0.0212905465522)*x[1]**o+(-0.358636168795)*x[1]+(0.719370965272)*x[2]**o+(-0.506502687359)*x[2]
        arg[1,1,0,0]=(-0.216544472879)*x[0]**o+(-0.690540499698)*x[0]+(0.190155427044)*x[1]**o+(0.832206912182)*x[1]+(-0.108713936252)*x[2]**o+(-0.221373087337)*x[2]
        arg[1,1,0,1]=(0.982934604971)*x[0]**o+(0.604369826835)*x[0]+(-0.34365978148)*x[1]**o+(-0.167486779981)*x[1]+(-0.753609370582)*x[2]**o+(-0.220160888977)*x[2]
        arg[1,1,1,0]=(0.516177509446)*x[0]**o+(-0.474445723273)*x[0]+(0.813647087566)*x[1]**o+(-0.281201093802)*x[1]+(0.941465944108)*x[2]**o+(0.352728360059)*x[2]
        arg[1,1,1,1]=(-0.914732890633)*x[0]**o+(0.214962090901)*x[0]+(-0.710485058274)*x[1]**o+(-0.070756754056)*x[1]+(-0.449571590211)*x[2]**o+(0.353655711451)*x[2]
        arg[1,1,2,0]=(-0.96655555619)*x[0]**o+(-0.47460534732)*x[0]+(0.0610695053327)*x[1]**o+(0.815390116709)*x[1]+(-0.844916163552)*x[2]**o+(-0.433015304307)*x[2]
        arg[1,1,2,1]=(-0.611879453709)*x[0]**o+(0.849092848509)*x[0]+(0.0181322361566)*x[1]**o+(0.264173017827)*x[1]+(0.0452154806937)*x[2]**o+(0.0395300648465)*x[2]
        arg[1,2,0,0]=(0.558820946357)*x[0]**o+(-0.471519705598)*x[0]+(-0.200404864145)*x[1]**o+(0.452044160691)*x[1]+(-0.507559485963)*x[2]**o+(-0.993437841726)*x[2]
        arg[1,2,0,1]=(-0.944738664192)*x[0]**o+(-0.497536772117)*x[0]+(0.111399525675)*x[1]**o+(-0.93450294271)*x[1]+(-0.463702615737)*x[2]**o+(-0.330112672884)*x[2]
        arg[1,2,1,0]=(0.238225961058)*x[0]**o+(-0.471520173419)*x[0]+(0.483767084892)*x[1]**o+(-0.0359407284537)*x[1]+(-0.767379409753)*x[2]**o+(-0.283596371915)*x[2]
        arg[1,2,1,1]=(0.188699181238)*x[0]**o+(0.829593121989)*x[0]+(0.603377767593)*x[1]**o+(0.163334775468)*x[1]+(-0.695141594131)*x[2]**o+(-0.335208640181)*x[2]
        arg[1,2,2,0]=(-0.0953160757881)*x[0]**o+(0.179594592176)*x[0]+(0.95983902191)*x[1]**o+(-0.689622683527)*x[1]+(0.418442085936)*x[2]**o+(-0.271399861129)*x[2]
        arg[1,2,2,1]=(0.598691886555)*x[0]**o+(0.334120340646)*x[0]+(0.580763872443)*x[1]**o+(0.769016736836)*x[1]+(-0.442121483951)*x[2]**o+(0.0685264980499)*x[2]
        arg[1,3,0,0]=(-0.482944276372)*x[0]**o+(-0.0518057205519)*x[0]+(-0.878100108766)*x[1]**o+(-0.130957239599)*x[1]+(0.71270341547)*x[2]**o+(-0.313822116756)*x[2]
        arg[1,3,0,1]=(0.565128725562)*x[0]**o+(0.706149004661)*x[0]+(0.837747652328)*x[1]**o+(-0.429417376176)*x[1]+(0.179437627997)*x[2]**o+(-0.841831183522)*x[2]
        arg[1,3,1,0]=(0.608648887224)*x[0]**o+(0.118220851451)*x[0]+(0.492890967377)*x[1]**o+(-0.162344084552)*x[1]+(0.141203210671)*x[2]**o+(0.883457779687)*x[2]
        arg[1,3,1,1]=(0.0322521291544)*x[0]**o+(0.839748977403)*x[0]+(0.75094786544)*x[1]**o+(0.954833145672)*x[1]+(0.461457599013)*x[2]**o+(-0.761973385747)*x[2]
        arg[1,3,2,0]=(-0.757022609619)*x[0]**o+(-0.250893790099)*x[0]+(-0.973631035961)*x[1]**o+(0.506574565217)*x[1]+(0.103557430896)*x[2]**o+(0.881026337134)*x[2]
        arg[1,3,2,1]=(-0.395945302352)*x[0]**o+(-0.125428842065)*x[0]+(-0.339663004568)*x[1]**o+(-0.0272044592179)*x[1]+(0.75200506467)*x[2]**o+(-0.632922582454)*x[2]
        arg[1,4,0,0]=(0.0462127928202)*x[0]**o+(-0.745613488751)*x[0]+(0.357338297422)*x[1]**o+(0.739034094804)*x[1]+(-0.451247272253)*x[2]**o+(0.418030110847)*x[2]
        arg[1,4,0,1]=(0.914131238526)*x[0]**o+(-0.0280436066624)*x[0]+(0.861048515118)*x[1]**o+(0.148206807819)*x[1]+(-0.990856990354)*x[2]**o+(0.903615279172)*x[2]
        arg[1,4,1,0]=(0.189809824038)*x[0]**o+(-0.749477832258)*x[0]+(-0.802713794942)*x[1]**o+(0.0610727255572)*x[1]+(0.292940815327)*x[2]**o+(0.213443422205)*x[2]
        arg[1,4,1,1]=(0.792313292544)*x[0]**o+(0.321344675677)*x[0]+(-0.417516596097)*x[1]**o+(-0.63797359819)*x[1]+(-0.426112319872)*x[2]**o+(0.69281310288)*x[2]
        arg[1,4,2,0]=(-0.11017764073)*x[0]**o+(0.703037562344)*x[0]+(0.739591950873)*x[1]**o+(0.445612268331)*x[1]+(-0.071251825024)*x[2]**o+(0.00577350654185)*x[2]
        arg[1,4,2,1]=(-0.793584849832)*x[0]**o+(-0.546439256231)*x[0]+(-0.947194030803)*x[1]**o+(-0.0455700937677)*x[1]+(0.535368301918)*x[2]**o+(0.814435535521)*x[2]
        arg[2,0,0,0]=(0.249661949438)*x[0]**o+(-0.581533619092)*x[0]+(-0.674373505747)*x[1]**o+(0.464175602196)*x[1]+(-0.528086662834)*x[2]**o+(-0.212416176351)*x[2]
        arg[2,0,0,1]=(-0.588282108798)*x[0]**o+(0.761293399494)*x[0]+(0.99918874903)*x[1]**o+(0.542499992928)*x[1]+(0.433925746483)*x[2]**o+(0.972864491889)*x[2]
        arg[2,0,1,0]=(0.0572335623442)*x[0]**o+(0.444906443533)*x[0]+(-0.305538151235)*x[1]**o+(0.244530068532)*x[1]+(-0.20198975858)*x[2]**o+(-0.00490928477351)*x[2]
        arg[2,0,1,1]=(0.832496085695)*x[0]**o+(-0.797246004595)*x[0]+(0.873816520001)*x[1]**o+(0.628293733929)*x[1]+(-0.110922710624)*x[2]**o+(-0.0718221914062)*x[2]
        arg[2,0,2,0]=(0.140970192196)*x[0]**o+(0.582230323368)*x[0]+(-0.757764751267)*x[1]**o+(0.206061262231)*x[1]+(0.657237474788)*x[2]**o+(0.598632779121)*x[2]
        arg[2,0,2,1]=(-0.686218790734)*x[0]**o+(-0.157012470162)*x[0]+(0.448565721848)*x[1]**o+(0.0539087954076)*x[1]+(0.619821805007)*x[2]**o+(0.289144379283)*x[2]
        arg[2,1,0,0]=(0.787423884494)*x[0]**o+(-0.985959420171)*x[0]+(0.155584360983)*x[1]**o+(0.223292293778)*x[1]+(-0.437349719833)*x[2]**o+(0.989077189275)*x[2]
        arg[2,1,0,1]=(-0.597301631508)*x[0]**o+(-0.0414813766671)*x[0]+(-0.64883048443)*x[1]**o+(-0.18731615522)*x[1]+(0.735189823623)*x[2]**o+(-0.746633725819)*x[2]
        arg[2,1,1,0]=(-0.393486075452)*x[0]**o+(-0.480110060647)*x[0]+(0.559811870081)*x[1]**o+(0.0814282056993)*x[1]+(0.244899906416)*x[2]**o+(-0.884478480652)*x[2]
        arg[2,1,1,1]=(0.547747813175)*x[0]**o+(0.317890585903)*x[0]+(-0.70925099068)*x[1]**o+(-0.543227445702)*x[1]+(0.749693240223)*x[2]**o+(0.79984501038)*x[2]
        arg[2,1,2,0]=(-0.289583636666)*x[0]**o+(0.513926622265)*x[0]+(-0.00283544918865)*x[1]**o+(0.530724065811)*x[1]+(0.900479623243)*x[2]**o+(-0.146332301083)*x[2]
        arg[2,1,2,1]=(-0.487223003118)*x[0]**o+(0.161546483594)*x[0]+(-0.685211654772)*x[1]**o+(0.4803179816)*x[1]+(0.676120396917)*x[2]**o+(-0.174953085068)*x[2]
        arg[2,2,0,0]=(0.31746495834)*x[0]**o+(-0.513225182059)*x[0]+(0.33220677322)*x[1]**o+(-0.302371334854)*x[1]+(0.0964988251907)*x[2]**o+(-0.259532745762)*x[2]
        arg[2,2,0,1]=(0.313462846078)*x[0]**o+(0.079720902627)*x[0]+(-0.487932457563)*x[1]**o+(-0.786522786773)*x[1]+(0.357868214554)*x[2]**o+(0.251856050362)*x[2]
        arg[2,2,1,0]=(-0.180472378556)*x[0]**o+(0.345084802362)*x[0]+(0.641108557953)*x[1]**o+(0.198717742483)*x[1]+(0.0374114682326)*x[2]**o+(-0.915249907773)*x[2]
        arg[2,2,1,1]=(0.303787742688)*x[0]**o+(-0.941822487745)*x[0]+(-0.262567025842)*x[1]**o+(-0.197051445307)*x[1]+(0.381561965565)*x[2]**o+(-0.198549777232)*x[2]
        arg[2,2,2,0]=(-0.850284190461)*x[0]**o+(0.985224868544)*x[0]+(0.633868153422)*x[1]**o+(-0.57559927346)*x[1]+(0.68753726951)*x[2]**o+(-0.923696997656)*x[2]
        arg[2,2,2,1]=(0.559527478503)*x[0]**o+(-0.619764636475)*x[0]+(-0.0352163693945)*x[1]**o+(-0.37351931995)*x[1]+(0.0772333960938)*x[2]**o+(-0.353057755509)*x[2]
        arg[2,3,0,0]=(-0.274260374984)*x[0]**o+(0.365180349506)*x[0]+(-0.879564869545)*x[1]**o+(-0.0908162765131)*x[1]+(0.813491263836)*x[2]**o+(-0.396739585204)*x[2]
        arg[2,3,0,1]=(-0.793218219979)*x[0]**o+(0.227474218201)*x[0]+(0.97716610661)*x[1]**o+(0.317074032722)*x[1]+(0.076776446926)*x[2]**o+(0.069448617786)*x[2]
        arg[2,3,1,0]=(0.411400569812)*x[0]**o+(0.517743454926)*x[0]+(0.646697523033)*x[1]**o+(-0.973369313024)*x[1]+(-0.599741793926)*x[2]**o+(-0.743687020283)*x[2]
        arg[2,3,1,1]=(-0.491445354697)*x[0]**o+(-0.549840110396)*x[0]+(-0.260012915614)*x[1]**o+(0.671054626537)*x[1]+(0.0394966796969)*x[2]**o+(-0.701243931411)*x[2]
        arg[2,3,2,0]=(-0.806859723517)*x[0]**o+(-0.753940066715)*x[0]+(-0.664229274865)*x[1]**o+(0.0595484609931)*x[1]+(-0.921892262667)*x[2]**o+(-0.701770672395)*x[2]
        arg[2,3,2,1]=(-0.313979805844)*x[0]**o+(-0.542583864222)*x[0]+(-0.985990886553)*x[1]**o+(0.664926443979)*x[1]+(-0.99778334615)*x[2]**o+(0.268046810979)*x[2]
        arg[2,4,0,0]=(-0.41348240167)*x[0]**o+(0.473517720534)*x[0]+(-0.290880345559)*x[1]**o+(-0.607611624107)*x[1]+(-0.25537697471)*x[2]**o+(-0.472402477362)*x[2]
        arg[2,4,0,1]=(0.0222534371058)*x[0]**o+(-0.0739354104963)*x[0]+(-0.960057765718)*x[1]**o+(-0.772161403037)*x[1]+(-0.0356127352146)*x[2]**o+(-0.623605628308)*x[2]
        arg[2,4,1,0]=(-0.750995516053)*x[0]**o+(-0.289601799215)*x[0]+(0.294370180152)*x[1]**o+(0.956012147239)*x[1]+(-0.27958662106)*x[2]**o+(-0.370609593537)*x[2]
        arg[2,4,1,1]=(0.321767371741)*x[0]**o+(0.999223434136)*x[0]+(0.665865428241)*x[1]**o+(0.427942082269)*x[1]+(-0.135020552387)*x[2]**o+(0.692025479514)*x[2]
        arg[2,4,2,0]=(-0.519508249649)*x[0]**o+(0.0779688946471)*x[0]+(0.493489242872)*x[1]**o+(-0.0961136530093)*x[1]+(-0.0750243559261)*x[2]**o+(0.860970740104)*x[2]
        arg[2,4,2,1]=(0.807909049346)*x[0]**o+(0.388180877227)*x[0]+(-0.9044578433)*x[1]**o+(0.86765389355)*x[1]+(-0.196105779343)*x[2]**o+(0.448661151455)*x[2]
        arg[3,0,0,0]=(-0.780895567041)*x[0]**o+(-0.571253266808)*x[0]+(-0.904183399932)*x[1]**o+(-0.883296586722)*x[1]+(0.855945559487)*x[2]**o+(0.618533553098)*x[2]
        arg[3,0,0,1]=(-0.831076704037)*x[0]**o+(0.898347446798)*x[0]+(-0.716442620396)*x[1]**o+(0.761028369421)*x[1]+(-0.486505604444)*x[2]**o+(0.25474220145)*x[2]
        arg[3,0,1,0]=(-0.0127541044953)*x[0]**o+(0.759482315308)*x[0]+(-0.725655006866)*x[1]**o+(0.667310781285)*x[1]+(0.266075927439)*x[2]**o+(0.423300182252)*x[2]
        arg[3,0,1,1]=(-0.594972266882)*x[0]**o+(-0.842501149639)*x[0]+(-0.347686790781)*x[1]**o+(0.686403640474)*x[1]+(0.3452971966)*x[2]**o+(0.722258585706)*x[2]
        arg[3,0,2,0]=(-0.715711146058)*x[0]**o+(-0.835739963371)*x[0]+(0.966177361301)*x[1]**o+(-0.909111754995)*x[1]+(-0.175184576557)*x[2]**o+(0.695058239921)*x[2]
        arg[3,0,2,1]=(0.306251544915)*x[0]**o+(0.944775715803)*x[0]+(-0.740601754617)*x[1]**o+(0.434394890125)*x[1]+(-0.876527474971)*x[2]**o+(0.209321129689)*x[2]
        arg[3,1,0,0]=(-0.86788221871)*x[0]**o+(-0.833030657309)*x[0]+(0.84576621947)*x[1]**o+(-0.469186887277)*x[1]+(-0.527611740694)*x[2]**o+(-0.312054431001)*x[2]
        arg[3,1,0,1]=(-0.353678394652)*x[0]**o+(0.591287444611)*x[0]+(-0.213012375931)*x[1]**o+(-0.917259099251)*x[1]+(0.739016360622)*x[2]**o+(0.718754459378)*x[2]
        arg[3,1,1,0]=(-0.92646825221)*x[0]**o+(-0.870564973175)*x[0]+(-0.781805219404)*x[1]**o+(0.078530434642)*x[1]+(0.386018501087)*x[2]**o+(-0.120490492288)*x[2]
        arg[3,1,1,1]=(-0.824273737615)*x[0]**o+(0.401211543421)*x[0]+(0.360077381624)*x[1]**o+(-0.0599234883152)*x[1]+(0.246820480884)*x[2]**o+(-0.175313542134)*x[2]
        arg[3,1,2,0]=(0.807756928769)*x[0]**o+(0.889534807508)*x[0]+(0.633296189954)*x[1]**o+(-0.471821951252)*x[1]+(0.396011249874)*x[2]**o+(0.386800809575)*x[2]
        arg[3,1,2,1]=(-0.494145142543)*x[0]**o+(-0.514793985721)*x[0]+(0.825379848403)*x[1]**o+(-0.457860259786)*x[1]+(-0.599557868275)*x[2]**o+(-0.952704784672)*x[2]
        arg[3,2,0,0]=(0.970345295013)*x[0]**o+(-0.838468036663)*x[0]+(0.265634348738)*x[1]**o+(-0.873847124748)*x[1]+(0.416242648913)*x[2]**o+(0.59908438222)*x[2]
        arg[3,2,0,1]=(0.692377432429)*x[0]**o+(-0.338749041227)*x[0]+(-0.449383750398)*x[1]**o+(0.841669527559)*x[1]+(0.902482059741)*x[2]**o+(-0.655719647704)*x[2]
        arg[3,2,1,0]=(0.152417917522)*x[0]**o+(-0.78860144)*x[0]+(-0.881276170743)*x[1]**o+(0.692901550061)*x[1]+(-0.985377819322)*x[2]**o+(0.362747973601)*x[2]
        arg[3,2,1,1]=(0.238845792119)*x[0]**o+(-0.65838373309)*x[0]+(-0.659676514296)*x[1]**o+(0.870742876778)*x[1]+(-0.367345415947)*x[2]**o+(0.322353666541)*x[2]
        arg[3,2,2,0]=(0.456386394533)*x[0]**o+(-0.135575574979)*x[0]+(-0.731714323922)*x[1]**o+(0.936807859479)*x[1]+(0.511595594967)*x[2]**o+(0.570282464669)*x[2]
        arg[3,2,2,1]=(0.53299212624)*x[0]**o+(0.800998166434)*x[0]+(-0.431740087552)*x[1]**o+(-0.880092532661)*x[1]+(-0.418657988674)*x[2]**o+(-0.181008149338)*x[2]
        arg[3,3,0,0]=(0.60015312936)*x[0]**o+(-0.163125583081)*x[0]+(0.299668981953)*x[1]**o+(-0.761494422747)*x[1]+(0.404839953231)*x[2]**o+(-0.114039836538)*x[2]
        arg[3,3,0,1]=(-0.597481072133)*x[0]**o+(-0.506468582896)*x[0]+(0.406822348574)*x[1]**o+(0.247431348859)*x[1]+(-0.137941563908)*x[2]**o+(0.0503979259383)*x[2]
        arg[3,3,1,0]=(0.0921069092744)*x[0]**o+(-0.113960490794)*x[0]+(0.750059773909)*x[1]**o+(0.894945048248)*x[1]+(-0.154873449786)*x[2]**o+(-0.520229467775)*x[2]
        arg[3,3,1,1]=(0.349078696154)*x[0]**o+(0.141298430827)*x[0]+(0.249161007832)*x[1]**o+(0.0240310278591)*x[1]+(-0.18294527838)*x[2]**o+(-0.666698832632)*x[2]
        arg[3,3,2,0]=(-0.476684936147)*x[0]**o+(0.334093402615)*x[0]+(-0.326434684009)*x[1]**o+(0.91336431975)*x[1]+(0.483080537399)*x[2]**o+(0.439096732335)*x[2]
        arg[3,3,2,1]=(0.214713426286)*x[0]**o+(-0.973239849984)*x[0]+(0.984290637567)*x[1]**o+(-0.644668525299)*x[1]+(0.00985524319943)*x[2]**o+(0.68777403533)*x[2]
        arg[3,4,0,0]=(-0.88715423933)*x[0]**o+(0.642412222188)*x[0]+(-0.119667464132)*x[1]**o+(-0.201583116481)*x[1]+(-0.870921042381)*x[2]**o+(-0.00379039429884)*x[2]
        arg[3,4,0,1]=(0.511114986801)*x[0]**o+(-0.497617928997)*x[0]+(-0.189435233543)*x[1]**o+(-0.29288141421)*x[1]+(-0.476297664539)*x[2]**o+(-0.202915859137)*x[2]
        arg[3,4,1,0]=(0.0739878219028)*x[0]**o+(0.675050675951)*x[0]+(-0.45950101521)*x[1]**o+(0.306538598103)*x[1]+(-0.726069135031)*x[2]**o+(0.193290422951)*x[2]
        arg[3,4,1,1]=(-0.141324267565)*x[0]**o+(0.377343882109)*x[0]+(-0.16075604872)*x[1]**o+(0.263332274591)*x[1]+(0.886416182992)*x[2]**o+(0.931905996631)*x[2]
        arg[3,4,2,0]=(-0.76912732001)*x[0]**o+(-0.255011103121)*x[0]+(-0.372810348364)*x[1]**o+(0.108137766687)*x[1]+(-0.0761382180171)*x[2]**o+(0.46825331282)*x[2]
        arg[3,4,2,1]=(-0.515945940217)*x[0]**o+(-0.630017609554)*x[0]+(-0.815236764508)*x[1]**o+(0.636572471005)*x[1]+(0.244112465789)*x[2]**o+(-0.471661017589)*x[2]
        ref[0,0,0,0]=(-0.738709393227)/(o+1.)+(0.283993125638)+(-0.891975745754)*0.5**o
        ref[0,0,0,1]=(0.912832356419)/(o+1.)+(-0.800452287951)+(0.428698930349)*0.5**o
        ref[0,0,1,0]=(0.604410785754)/(o+1.)+(-0.70496852873)+(0.974069118653)*0.5**o
        ref[0,0,1,1]=(-1.63466543191)/(o+1.)+(0.302144663639)+(0.813412823502)*0.5**o
        ref[0,0,2,0]=(0.755754363963)/(o+1.)+(0.151022836528)+(-0.874502916672)*0.5**o
        ref[0,0,2,1]=(0.958914457666)/(o+1.)+(-0.440871585564)+(-0.258524075807)*0.5**o
        ref[0,1,0,0]=(-0.154165868092)/(o+1.)+(0.484258240357)+(0.489505903018)*0.5**o
        ref[0,1,0,1]=(0.79058715248)/(o+1.)+(0.686313504396)+(0.441567332096)*0.5**o
        ref[0,1,1,0]=(-0.834798928973)/(o+1.)+(0.492360141946)+(0.388576628492)*0.5**o
        ref[0,1,1,1]=(-0.315522016845)/(o+1.)+(0.688152306846)+(-0.0537780037583)*0.5**o
        ref[0,1,2,0]=(0.203060170914)/(o+1.)+(0.198021868056)+(0.58262140787)*0.5**o
        ref[0,1,2,1]=(-0.665732288492)/(o+1.)+(0.798951978927)+(0.438472233911)*0.5**o
        ref[0,2,0,0]=(-1.1918215501)/(o+1.)+(0.114544359058)+(-0.334110476968)*0.5**o
        ref[0,2,0,1]=(0.0771737213265)/(o+1.)+(-0.711198951588)+(0.33041258354)*0.5**o
        ref[0,2,1,0]=(1.1366448279)/(o+1.)+(0.141765838337)+(-0.80021736912)*0.5**o
        ref[0,2,1,1]=(-0.432635401002)/(o+1.)+(0.0361325240497)+(-0.773331228218)*0.5**o
        ref[0,2,2,0]=(1.37338816519)/(o+1.)+(-0.0827542033711)+(-0.943072868965)*0.5**o
        ref[0,2,2,1]=(-0.303620293463)/(o+1.)+(-0.39585636903)+(-0.821201079286)*0.5**o
        ref[0,3,0,0]=(-1.74869869132)/(o+1.)+(-0.845702848785)+(-0.0564164524098)*0.5**o
        ref[0,3,0,1]=(0.499654298722)/(o+1.)+(0.0895873531986)+(-0.553858251068)*0.5**o
        ref[0,3,1,0]=(-0.277226290791)/(o+1.)+(-0.755072202588)+(-0.992317453163)*0.5**o
        ref[0,3,1,1]=(-0.412604788342)/(o+1.)+(0.496792347901)+(-0.713753922213)*0.5**o
        ref[0,3,2,0]=(-0.130885078624)/(o+1.)+(0.312575459575)+(0.536024118289)*0.5**o
        ref[0,3,2,1]=(-1.35620016195)/(o+1.)+(-0.802557814945)+(-0.197968756117)*0.5**o
        ref[0,4,0,0]=(0.214746716397)/(o+1.)+(-0.0864879979042)+(0.688763655127)*0.5**o
        ref[0,4,0,1]=(0.101745206477)/(o+1.)+(-0.431035828594)+(-0.402717742839)*0.5**o
        ref[0,4,1,0]=(-1.71496447458)/(o+1.)+(-0.210739324578)+(0.39352838659)*0.5**o
        ref[0,4,1,1]=(0.516179257264)/(o+1.)+(-0.271979584839)+(-0.375507250175)*0.5**o
        ref[0,4,2,0]=(0.0175356956523)/(o+1.)+(0.255449591592)+(0.0603590263818)*0.5**o
        ref[0,4,2,1]=(0.36499082659)/(o+1.)+(-0.11897969607)+(-0.111089454685)*0.5**o
        ref[1,0,0,0]=(1.12671976333)/(o+1.)+(0.275654183538)+(-0.940865155622)*0.5**o
        ref[1,0,0,1]=(-1.58477430959)/(o+1.)+(-0.471320510546)+(-0.843207880309)*0.5**o
        ref[1,0,1,0]=(0.126570690864)/(o+1.)+(0.505250661477)+(-0.484987528212)*0.5**o
        ref[1,0,1,1]=(-0.807547181539)/(o+1.)+(-0.728382819379)+(0.285548625366)*0.5**o
        ref[1,0,2,0]=(0.151618483009)/(o+1.)+(0.213905131461)+(0.96797757731)*0.5**o
        ref[1,0,2,1]=(0.740661511824)/(o+1.)+(-0.789278288097)+(-0.486190283922)*0.5**o
        ref[1,1,0,0]=(0.0814414907926)/(o+1.)+(-0.0398533374263)+(-0.216544472879)*0.5**o
        ref[1,1,0,1]=(-1.09726915206)/(o+1.)+(0.108361078938)+(0.982934604971)*0.5**o
        ref[1,1,1,0]=(1.75511303167)/(o+1.)+(-0.201459228508)+(0.516177509446)*0.5**o
        ref[1,1,1,1]=(-1.16005664849)/(o+1.)+(0.248930524148)+(-0.914732890633)*0.5**o
        ref[1,1,2,0]=(-0.78384665822)/(o+1.)+(-0.0461152674588)+(-0.96655555619)*0.5**o
        ref[1,1,2,1]=(0.0633477168503)/(o+1.)+(0.576397965591)+(-0.611879453709)*0.5**o
        ref[1,2,0,0]=(-0.707964350108)/(o+1.)+(-0.506456693317)+(0.558820946357)*0.5**o
        ref[1,2,0,1]=(-0.352303090063)/(o+1.)+(-0.881076193856)+(-0.944738664192)*0.5**o
        ref[1,2,1,0]=(-0.283612324861)/(o+1.)+(-0.395528636894)+(0.238225961058)*0.5**o
        ref[1,2,1,1]=(-0.0917638265383)/(o+1.)+(0.328859628638)+(0.188699181238)*0.5**o
        ref[1,2,2,0]=(1.37828110785)/(o+1.)+(-0.39071397624)+(-0.0953160757881)*0.5**o
        ref[1,2,2,1]=(0.138642388492)/(o+1.)+(0.585831787766)+(0.598691886555)*0.5**o
        ref[1,3,0,0]=(-0.165396693296)/(o+1.)+(-0.248292538454)+(-0.482944276372)*0.5**o
        ref[1,3,0,1]=(1.01718528032)/(o+1.)+(-0.282549777519)+(0.565128725562)*0.5**o
        ref[1,3,1,0]=(0.634094178049)/(o+1.)+(0.419667273293)+(0.608648887224)*0.5**o
        ref[1,3,1,1]=(1.21240546445)/(o+1.)+(0.516304368664)+(0.0322521291544)*0.5**o
        ref[1,3,2,0]=(-0.870073605065)/(o+1.)+(0.568353556126)+(-0.757022609619)*0.5**o
        ref[1,3,2,1]=(0.412342060102)/(o+1.)+(-0.392777941868)+(-0.395945302352)*0.5**o
        ref[1,4,0,0]=(-0.0939089748304)/(o+1.)+(0.20572535845)+(0.0462127928202)*0.5**o
        ref[1,4,0,1]=(-0.129808475236)/(o+1.)+(0.511889240164)+(0.914131238526)*0.5**o
        ref[1,4,1,0]=(-0.509772979615)/(o+1.)+(-0.237480842248)+(0.189809824038)*0.5**o
        ref[1,4,1,1]=(-0.843628915969)/(o+1.)+(0.188092090183)+(0.792313292544)*0.5**o
        ref[1,4,2,0]=(0.668340125849)/(o+1.)+(0.577211668608)+(-0.11017764073)*0.5**o
        ref[1,4,2,1]=(-0.411825728886)/(o+1.)+(0.111213092761)+(-0.793584849832)*0.5**o
        ref[2,0,0,0]=(-1.20246016858)/(o+1.)+(-0.164887096623)+(0.249661949438)*0.5**o
        ref[2,0,0,1]=(1.43311449551)/(o+1.)+(1.13832894216)+(-0.588282108798)*0.5**o
        ref[2,0,1,0]=(-0.507527909815)/(o+1.)+(0.342263613646)+(0.0572335623442)*0.5**o
        ref[2,0,1,1]=(0.762893809377)/(o+1.)+(-0.120387231036)+(0.832496085695)*0.5**o
        ref[2,0,2,0]=(-0.100527276479)/(o+1.)+(0.69346218236)+(0.140970192196)*0.5**o
        ref[2,0,2,1]=(1.06838752686)/(o+1.)+(0.0930203522642)+(-0.686218790734)*0.5**o
        ref[2,1,0,0]=(-0.28176535885)/(o+1.)+(0.113205031441)+(0.787423884494)*0.5**o
        ref[2,1,0,1]=(0.0863593391922)/(o+1.)+(-0.487715628853)+(-0.597301631508)*0.5**o
        ref[2,1,1,0]=(0.804711776497)/(o+1.)+(-0.6415801678)+(-0.393486075452)*0.5**o
        ref[2,1,1,1]=(0.0404422495423)/(o+1.)+(0.28725407529)+(0.547747813175)*0.5**o
        ref[2,1,2,0]=(0.897644174054)/(o+1.)+(0.449159193497)+(-0.289583636666)*0.5**o
        ref[2,1,2,1]=(-0.00909125785473)/(o+1.)+(0.233455690063)+(-0.487223003118)*0.5**o
        ref[2,2,0,0]=(0.428705598411)/(o+1.)+(-0.537564631337)+(0.31746495834)*0.5**o
        ref[2,2,0,1]=(-0.13006424301)/(o+1.)+(-0.227472916892)+(0.313462846078)*0.5**o
        ref[2,2,1,0]=(0.678520026185)/(o+1.)+(-0.185723681464)+(-0.180472378556)*0.5**o
        ref[2,2,1,1]=(0.118994939723)/(o+1.)+(-0.668711855142)+(0.303787742688)*0.5**o
        ref[2,2,2,0]=(1.32140542293)/(o+1.)+(-0.257035701286)+(-0.850284190461)*0.5**o
        ref[2,2,2,1]=(0.0420170266993)/(o+1.)+(-0.673170855967)+(0.559527478503)*0.5**o
        ref[2,3,0,0]=(-0.0660736057092)/(o+1.)+(-0.0611877561056)+(-0.274260374984)*0.5**o
        ref[2,3,0,1]=(1.05394255354)/(o+1.)+(0.306998434354)+(-0.793218219979)*0.5**o
        ref[2,3,1,0]=(0.046955729107)/(o+1.)+(-0.599656439191)+(0.411400569812)*0.5**o
        ref[2,3,1,1]=(-0.220516235917)/(o+1.)+(-0.290014707635)+(-0.491445354697)*0.5**o
        ref[2,3,2,0]=(-1.58612153753)/(o+1.)+(-0.698081139059)+(-0.806859723517)*0.5**o
        ref[2,3,2,1]=(-1.9837742327)/(o+1.)+(0.195194695368)+(-0.313979805844)*0.5**o
        ref[2,4,0,0]=(-0.546257320269)/(o+1.)+(-0.303248190467)+(-0.41348240167)*0.5**o
        ref[2,4,0,1]=(-0.995670500932)/(o+1.)+(-0.734851220921)+(0.0222534371058)*0.5**o
        ref[2,4,1,0]=(0.0147835590923)/(o+1.)+(0.147900377243)+(-0.750995516053)*0.5**o
        ref[2,4,1,1]=(0.530844875854)/(o+1.)+(1.05959549796)+(0.321767371741)*0.5**o
        ref[2,4,2,0]=(0.418464886946)/(o+1.)+(0.421412990871)+(-0.519508249649)*0.5**o
        ref[2,4,2,1]=(-1.10056362264)/(o+1.)+(0.852247961116)+(0.807909049346)*0.5**o
        ref[3,0,0,0]=(-0.0482378404454)/(o+1.)+(-0.418008150216)+(-0.780895567041)*0.5**o
        ref[3,0,0,1]=(-1.20294822484)/(o+1.)+(0.957059008835)+(-0.831076704037)*0.5**o
        ref[3,0,1,0]=(-0.459579079427)/(o+1.)+(0.925046639422)+(-0.0127541044953)*0.5**o
        ref[3,0,1,1]=(-0.00238959418047)/(o+1.)+(0.28308053827)+(-0.594972266882)*0.5**o
        ref[3,0,2,0]=(0.790992784744)/(o+1.)+(-0.524896739222)+(-0.715711146058)*0.5**o
        ref[3,0,2,1]=(-1.61712922959)/(o+1.)+(0.794245867809)+(0.306251544915)*0.5**o
        ref[3,1,0,0]=(0.318154478777)/(o+1.)+(-0.807135987793)+(-0.86788221871)*0.5**o
        ref[3,1,0,1]=(0.526003984691)/(o+1.)+(0.196391402369)+(-0.353678394652)*0.5**o
        ref[3,1,1,0]=(-0.395786718317)/(o+1.)+(-0.456262515411)+(-0.92646825221)*0.5**o
        ref[3,1,1,1]=(0.606897862508)/(o+1.)+(0.082987256486)+(-0.824273737615)*0.5**o
        ref[3,1,2,0]=(1.02930743983)/(o+1.)+(0.402256832916)+(0.807756928769)*0.5**o
        ref[3,1,2,1]=(0.225821980128)/(o+1.)+(-0.96267951509)+(-0.494145142543)*0.5**o
        ref[3,2,0,0]=(0.681876997651)/(o+1.)+(-0.556615389596)+(0.970345295013)*0.5**o
        ref[3,2,0,1]=(0.453098309344)/(o+1.)+(-0.0763995806862)+(0.692377432429)*0.5**o
        ref[3,2,1,0]=(-1.86665399006)/(o+1.)+(0.133524041831)+(0.152417917522)*0.5**o
        ref[3,2,1,1]=(-1.02702193024)/(o+1.)+(0.267356405114)+(0.238845792119)*0.5**o
        ref[3,2,2,0]=(-0.220118728954)/(o+1.)+(0.685757374584)+(0.456386394533)*0.5**o
        ref[3,2,2,1]=(-0.850398076225)/(o+1.)+(-0.130051257782)+(0.53299212624)*0.5**o
        ref[3,3,0,0]=(0.704508935183)/(o+1.)+(-0.519329921183)+(0.60015312936)*0.5**o
        ref[3,3,0,1]=(0.268880784666)/(o+1.)+(-0.10431965405)+(-0.597481072133)*0.5**o
        ref[3,3,1,0]=(0.595186324123)/(o+1.)+(0.13037754484)+(0.0921069092744)*0.5**o
        ref[3,3,1,1]=(0.0662157294518)/(o+1.)+(-0.250684686973)+(0.349078696154)*0.5**o
        ref[3,3,2,0]=(0.156645853389)/(o+1.)+(0.84327722735)+(-0.476684936147)*0.5**o
        ref[3,3,2,1]=(0.994145880767)/(o+1.)+(-0.465067169976)+(0.214713426286)*0.5**o
        ref[3,4,0,0]=(-0.990588506513)/(o+1.)+(0.218519355704)+(-0.88715423933)*0.5**o
        ref[3,4,0,1]=(-0.665732898082)/(o+1.)+(-0.496707601172)+(0.511114986801)*0.5**o
        ref[3,4,1,0]=(-1.18557015024)/(o+1.)+(0.587439848502)+(0.0739878219028)*0.5**o
        ref[3,4,1,1]=(0.725660134271)/(o+1.)+(0.786291076666)+(-0.141324267565)*0.5**o
        ref[3,4,2,0]=(-0.448948566381)/(o+1.)+(0.160689988193)+(-0.76912732001)*0.5**o
        ref[3,4,2,1]=(-0.571124298719)/(o+1.)+(-0.232553078069)+(-0.515945940217)*0.5**o
      res=integrate(arg)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactZero_fromSymbol_ContinuousFunction_rank0(self):
      """
      tests integral of rank 0 Symbol on the FunctionOnContactZero

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports integral on FunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=())
      s=Data(0,(),w)
      if dim==2:
        s=(-0.239048993811)*x[0]**o+(0.519530081152)*x[0]+(0.0639850840479)*x[1]**o+(0.629919962063)*x[1]
        ref=(0.0639850840479)/(o+1.)+(0.574725021608)+(-0.239048993811)*0.5**o
      else:
        s=(0.129836073672)*x[0]**o+(0.77979793904)*x[0]+(0.153605570913)*x[1]**o+(0.182457449004)*x[1]+(-0.363742743784)*x[2]**o+(-0.826028359978)*x[2]
        ref=(-0.210137172871)/(o+1.)+(0.068113514033)+(0.129836073672)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactZero_fromSymbol_ContinuousFunction_rank1(self):
      """
      tests integral of rank 1 Symbol on the FunctionOnContactZero

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports integral on FunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(2,))
      s=Data(0,(2,),w)
      ref=numpy.zeros((2,),numpy.float_)
      if dim==2:
        s[0]=(-0.0535727754535)*x[0]**o+(-0.378911132266)*x[0]+(0.893236107106)*x[1]**o+(-0.133308761371)*x[1]
        s[1]=(0.816344010178)*x[0]**o+(-0.629551729403)*x[0]+(0.962635839824)*x[1]**o+(0.531091887731)*x[1]
        ref[0]=(0.893236107106)/(o+1.)+(-0.256109946819)+(-0.0535727754535)*0.5**o
        ref[1]=(0.962635839824)/(o+1.)+(-0.049229920836)+(0.816344010178)*0.5**o
      else:
        s[0]=(0.99651578956)*x[0]**o+(-0.450391117128)*x[0]+(-0.0521202081325)*x[1]**o+(0.409486612049)*x[1]+(0.951839775929)*x[2]**o+(0.0755155386131)*x[2]
        s[1]=(-0.226900637285)*x[0]**o+(-0.784888192711)*x[0]+(-0.475467162806)*x[1]**o+(0.536559857164)*x[1]+(0.346844399644)*x[2]**o+(-0.237869505536)*x[2]
        ref[0]=(0.899719567796)/(o+1.)+(0.0173055167672)+(0.99651578956)*0.5**o
        ref[1]=(-0.128622763161)/(o+1.)+(-0.243098920542)+(-0.226900637285)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactZero_fromSymbol_ContinuousFunction_rank2(self):
      """
      tests integral of rank 2 Symbol on the FunctionOnContactZero

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports integral on FunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5))
      s=Data(0,(4, 5),w)
      ref=numpy.zeros((4, 5),numpy.float_)
      if dim==2:
        s[0,0]=(-0.328426418082)*x[0]**o+(-0.966431499834)*x[0]+(0.982530526624)*x[1]**o+(0.475372684178)*x[1]
        s[0,1]=(0.61075753424)*x[0]**o+(-0.974986578257)*x[0]+(-0.0986707750313)*x[1]**o+(-0.490145139235)*x[1]
        s[0,2]=(-0.629292138329)*x[0]**o+(-0.784677771399)*x[0]+(0.662986672949)*x[1]**o+(-0.0634716139549)*x[1]
        s[0,3]=(0.74966960207)*x[0]**o+(-0.672137973696)*x[0]+(-0.104345950525)*x[1]**o+(-0.196646103769)*x[1]
        s[0,4]=(0.714327500249)*x[0]**o+(-0.454839918323)*x[0]+(-0.904749225664)*x[1]**o+(0.829036667249)*x[1]
        s[1,0]=(-0.290327744474)*x[0]**o+(-0.0432653521002)*x[0]+(-0.814637993515)*x[1]**o+(0.329873273442)*x[1]
        s[1,1]=(-0.256944455249)*x[0]**o+(-0.0374267372088)*x[0]+(0.136172411148)*x[1]**o+(-0.82944305625)*x[1]
        s[1,2]=(-0.260661151595)*x[0]**o+(-0.597199898207)*x[0]+(0.909049968542)*x[1]**o+(-0.262195958725)*x[1]
        s[1,3]=(-0.228035815224)*x[0]**o+(0.772135534405)*x[0]+(0.00192609783775)*x[1]**o+(-0.855606696367)*x[1]
        s[1,4]=(0.824519454106)*x[0]**o+(0.723545812848)*x[0]+(0.588395672453)*x[1]**o+(0.701489329605)*x[1]
        s[2,0]=(0.104767951487)*x[0]**o+(0.0937742673825)*x[0]+(-0.226378722473)*x[1]**o+(-0.239638024937)*x[1]
        s[2,1]=(0.922749632743)*x[0]**o+(0.380185381588)*x[0]+(-0.873869266649)*x[1]**o+(-0.995526870666)*x[1]
        s[2,2]=(0.459628441315)*x[0]**o+(-0.870827776883)*x[0]+(0.963527703404)*x[1]**o+(-0.380921690386)*x[1]
        s[2,3]=(0.0538635635094)*x[0]**o+(0.636269518871)*x[0]+(0.496494434174)*x[1]**o+(0.389503037841)*x[1]
        s[2,4]=(0.486584334747)*x[0]**o+(0.595962829728)*x[0]+(-0.0473827215975)*x[1]**o+(-0.0598623245737)*x[1]
        s[3,0]=(0.104809543499)*x[0]**o+(-0.103399959545)*x[0]+(0.46274253883)*x[1]**o+(0.00540100368281)*x[1]
        s[3,1]=(0.182366414916)*x[0]**o+(-0.454996760913)*x[0]+(0.195895079357)*x[1]**o+(-0.445919339196)*x[1]
        s[3,2]=(-0.270350002842)*x[0]**o+(0.0801494193318)*x[0]+(-0.192008845814)*x[1]**o+(0.20055215786)*x[1]
        s[3,3]=(0.946634950526)*x[0]**o+(0.794617946132)*x[0]+(0.266843824183)*x[1]**o+(0.431654105192)*x[1]
        s[3,4]=(-0.282961386823)*x[0]**o+(-0.659458559302)*x[0]+(-0.701137505881)*x[1]**o+(0.419244903032)*x[1]
        ref[0,0]=(0.982530526624)/(o+1.)+(-0.245529407828)+(-0.328426418082)*0.5**o
        ref[0,1]=(-0.0986707750313)/(o+1.)+(-0.732565858746)+(0.61075753424)*0.5**o
        ref[0,2]=(0.662986672949)/(o+1.)+(-0.424074692677)+(-0.629292138329)*0.5**o
        ref[0,3]=(-0.104345950525)/(o+1.)+(-0.434392038732)+(0.74966960207)*0.5**o
        ref[0,4]=(-0.904749225664)/(o+1.)+(0.187098374463)+(0.714327500249)*0.5**o
        ref[1,0]=(-0.814637993515)/(o+1.)+(0.143303960671)+(-0.290327744474)*0.5**o
        ref[1,1]=(0.136172411148)/(o+1.)+(-0.433434896729)+(-0.256944455249)*0.5**o
        ref[1,2]=(0.909049968542)/(o+1.)+(-0.429697928466)+(-0.260661151595)*0.5**o
        ref[1,3]=(0.00192609783775)/(o+1.)+(-0.0417355809806)+(-0.228035815224)*0.5**o
        ref[1,4]=(0.588395672453)/(o+1.)+(0.712517571227)+(0.824519454106)*0.5**o
        ref[2,0]=(-0.226378722473)/(o+1.)+(-0.0729318787773)+(0.104767951487)*0.5**o
        ref[2,1]=(-0.873869266649)/(o+1.)+(-0.307670744539)+(0.922749632743)*0.5**o
        ref[2,2]=(0.963527703404)/(o+1.)+(-0.625874733635)+(0.459628441315)*0.5**o
        ref[2,3]=(0.496494434174)/(o+1.)+(0.512886278356)+(0.0538635635094)*0.5**o
        ref[2,4]=(-0.0473827215975)/(o+1.)+(0.268050252577)+(0.486584334747)*0.5**o
        ref[3,0]=(0.46274253883)/(o+1.)+(-0.0489994779311)+(0.104809543499)*0.5**o
        ref[3,1]=(0.195895079357)/(o+1.)+(-0.450458050054)+(0.182366414916)*0.5**o
        ref[3,2]=(-0.192008845814)/(o+1.)+(0.140350788596)+(-0.270350002842)*0.5**o
        ref[3,3]=(0.266843824183)/(o+1.)+(0.613136025662)+(0.946634950526)*0.5**o
        ref[3,4]=(-0.701137505881)/(o+1.)+(-0.120106828135)+(-0.282961386823)*0.5**o
      else:
        s[0,0]=(-0.544243335601)*x[0]**o+(-0.307372154904)*x[0]+(0.244525574061)*x[1]**o+(-0.528694523489)*x[1]+(0.667705852103)*x[2]**o+(0.866378210585)*x[2]
        s[0,1]=(-0.397733775219)*x[0]**o+(0.195969196217)*x[0]+(-0.941371407192)*x[1]**o+(0.524704832845)*x[1]+(-0.973723558169)*x[2]**o+(0.0452111700317)*x[2]
        s[0,2]=(-0.0445896420815)*x[0]**o+(-0.730345503814)*x[0]+(-0.797148073969)*x[1]**o+(0.687229609119)*x[1]+(-0.905358686985)*x[2]**o+(-0.764008239213)*x[2]
        s[0,3]=(0.803277368887)*x[0]**o+(0.716534949109)*x[0]+(-0.138102366205)*x[1]**o+(-0.0536511450001)*x[1]+(-0.608245637044)*x[2]**o+(-0.922294317712)*x[2]
        s[0,4]=(-0.802690014092)*x[0]**o+(0.397998151759)*x[0]+(0.209143804619)*x[1]**o+(0.171210888095)*x[1]+(0.945371372346)*x[2]**o+(0.143657498886)*x[2]
        s[1,0]=(-0.737094926707)*x[0]**o+(-0.682774905121)*x[0]+(-0.605040082549)*x[1]**o+(0.913015529243)*x[1]+(-0.923053380199)*x[2]**o+(0.488575946117)*x[2]
        s[1,1]=(-0.19024610752)*x[0]**o+(0.112566519608)*x[0]+(0.631915593649)*x[1]**o+(0.613460770368)*x[1]+(-0.145279640801)*x[2]**o+(0.391904990063)*x[2]
        s[1,2]=(0.503053165032)*x[0]**o+(0.150421546144)*x[0]+(-0.975785035648)*x[1]**o+(0.358360240583)*x[1]+(-0.116051949436)*x[2]**o+(0.22565570175)*x[2]
        s[1,3]=(-0.183734715218)*x[0]**o+(0.397188558255)*x[0]+(0.621875746076)*x[1]**o+(-0.683761765859)*x[1]+(-0.964066487584)*x[2]**o+(0.572293215139)*x[2]
        s[1,4]=(0.308876282691)*x[0]**o+(0.163499825051)*x[0]+(-0.859132579068)*x[1]**o+(-0.133046304211)*x[1]+(-0.995751005727)*x[2]**o+(0.256192590636)*x[2]
        s[2,0]=(-0.823620775877)*x[0]**o+(-0.173531059768)*x[0]+(-0.428278048825)*x[1]**o+(-0.935056836732)*x[1]+(-0.16471171795)*x[2]**o+(-0.0746252552822)*x[2]
        s[2,1]=(0.692340272178)*x[0]**o+(-0.178862964433)*x[0]+(0.241112494059)*x[1]**o+(-0.0400607170557)*x[1]+(-0.131477945619)*x[2]**o+(-0.742303256353)*x[2]
        s[2,2]=(-0.213693977195)*x[0]**o+(0.829007036928)*x[0]+(0.269322769984)*x[1]**o+(0.988636255935)*x[1]+(0.145550246384)*x[2]**o+(0.53131223825)*x[2]
        s[2,3]=(0.337485288934)*x[0]**o+(-0.794235442944)*x[0]+(-0.862195558251)*x[1]**o+(-0.302947127951)*x[1]+(0.660574358405)*x[2]**o+(0.383147294118)*x[2]
        s[2,4]=(-0.0555202855446)*x[0]**o+(-0.298089725164)*x[0]+(-0.971009290556)*x[1]**o+(-0.867265062556)*x[1]+(0.669532302056)*x[2]**o+(-0.432375844991)*x[2]
        s[3,0]=(0.596097084223)*x[0]**o+(-0.388031691584)*x[0]+(-0.596718750677)*x[1]**o+(0.196346996307)*x[1]+(0.0230334505824)*x[2]**o+(-0.293224683649)*x[2]
        s[3,1]=(-0.855920365774)*x[0]**o+(-0.0363724823656)*x[0]+(-0.918160096067)*x[1]**o+(-0.275555163751)*x[1]+(-0.90266927)*x[2]**o+(-0.275537041793)*x[2]
        s[3,2]=(0.788923009274)*x[0]**o+(-0.182338499866)*x[0]+(0.649442441856)*x[1]**o+(-0.179319161548)*x[1]+(-0.330240080654)*x[2]**o+(0.45030421169)*x[2]
        s[3,3]=(-0.0731084424823)*x[0]**o+(0.903713856354)*x[0]+(0.749750641159)*x[1]**o+(-0.274337831625)*x[1]+(0.667970754626)*x[2]**o+(-0.369139994984)*x[2]
        s[3,4]=(0.0580656964124)*x[0]**o+(0.30223985624)*x[0]+(-0.0278202819133)*x[1]**o+(-0.214845008141)*x[1]+(0.683342885793)*x[2]**o+(0.920939676772)*x[2]
        ref[0,0]=(0.912231426164)/(o+1.)+(0.0151557660955)+(-0.544243335601)*0.5**o
        ref[0,1]=(-1.91509496536)/(o+1.)+(0.382942599547)+(-0.397733775219)*0.5**o
        ref[0,2]=(-1.70250676095)/(o+1.)+(-0.403562066955)+(-0.0445896420815)*0.5**o
        ref[0,3]=(-0.746348003249)/(o+1.)+(-0.129705256801)+(0.803277368887)*0.5**o
        ref[0,4]=(1.15451517696)/(o+1.)+(0.35643326937)+(-0.802690014092)*0.5**o
        ref[1,0]=(-1.52809346275)/(o+1.)+(0.359408285119)+(-0.737094926707)*0.5**o
        ref[1,1]=(0.486635952848)/(o+1.)+(0.558966140019)+(-0.19024610752)*0.5**o
        ref[1,2]=(-1.09183698508)/(o+1.)+(0.367218744238)+(0.503053165032)*0.5**o
        ref[1,3]=(-0.342190741508)/(o+1.)+(0.142860003768)+(-0.183734715218)*0.5**o
        ref[1,4]=(-1.8548835848)/(o+1.)+(0.143323055738)+(0.308876282691)*0.5**o
        ref[2,0]=(-0.592989766775)/(o+1.)+(-0.591606575891)+(-0.823620775877)*0.5**o
        ref[2,1]=(0.109634548439)/(o+1.)+(-0.480613468921)+(0.692340272178)*0.5**o
        ref[2,2]=(0.414873016368)/(o+1.)+(1.17447776556)+(-0.213693977195)*0.5**o
        ref[2,3]=(-0.201621199846)/(o+1.)+(-0.357017638388)+(0.337485288934)*0.5**o
        ref[2,4]=(-0.3014769885)/(o+1.)+(-0.798865316355)+(-0.0555202855446)*0.5**o
        ref[3,0]=(-0.573685300094)/(o+1.)+(-0.242454689463)+(0.596097084223)*0.5**o
        ref[3,1]=(-1.82082936607)/(o+1.)+(-0.293732343955)+(-0.855920365774)*0.5**o
        ref[3,2]=(0.319202361202)/(o+1.)+(0.044323275138)+(0.788923009274)*0.5**o
        ref[3,3]=(1.41772139578)/(o+1.)+(0.130118014873)+(-0.0731084424823)*0.5**o
        ref[3,4]=(0.65552260388)/(o+1.)+(0.504167262435)+(0.0580656964124)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactZero_fromSymbol_ContinuousFunction_rank3(self):
      """
      tests integral of rank 3 Symbol on the FunctionOnContactZero

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports integral on FunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(6, 2, 2))
      s=Data(0,(6, 2, 2),w)
      ref=numpy.zeros((6, 2, 2),numpy.float_)
      if dim==2:
        s[0,0,0]=(-0.2538014101)*x[0]**o+(0.422893022023)*x[0]+(-0.168009677754)*x[1]**o+(0.967205152872)*x[1]
        s[0,0,1]=(-0.42169068503)*x[0]**o+(-0.87035047637)*x[0]+(0.861757151221)*x[1]**o+(0.233579078048)*x[1]
        s[0,1,0]=(0.761004021853)*x[0]**o+(-0.406910518681)*x[0]+(-0.653627037953)*x[1]**o+(0.78163950747)*x[1]
        s[0,1,1]=(0.571752563364)*x[0]**o+(0.352647230887)*x[0]+(0.64023828193)*x[1]**o+(-0.906400256269)*x[1]
        s[1,0,0]=(-0.229152209793)*x[0]**o+(-0.164526860803)*x[0]+(-0.815301528075)*x[1]**o+(-0.375201173874)*x[1]
        s[1,0,1]=(0.497997942477)*x[0]**o+(-0.590120034514)*x[0]+(0.387642938322)*x[1]**o+(-0.441598030675)*x[1]
        s[1,1,0]=(-0.396407524403)*x[0]**o+(0.225115161973)*x[0]+(0.709660265053)*x[1]**o+(-0.723548390356)*x[1]
        s[1,1,1]=(-0.810340425309)*x[0]**o+(0.571580569899)*x[0]+(0.401013753168)*x[1]**o+(0.881936918624)*x[1]
        s[2,0,0]=(0.096907527902)*x[0]**o+(-0.329521007808)*x[0]+(0.872734305266)*x[1]**o+(-0.677632080143)*x[1]
        s[2,0,1]=(-0.657791763259)*x[0]**o+(-0.0622202818891)*x[0]+(0.838133724288)*x[1]**o+(0.679646081219)*x[1]
        s[2,1,0]=(-0.982546119133)*x[0]**o+(0.0982803558898)*x[0]+(0.188605842026)*x[1]**o+(-0.436341762577)*x[1]
        s[2,1,1]=(0.623965743731)*x[0]**o+(-0.470311605613)*x[0]+(-0.700736177085)*x[1]**o+(0.688725936352)*x[1]
        s[3,0,0]=(0.154576108414)*x[0]**o+(-0.114669097915)*x[0]+(-0.922751077939)*x[1]**o+(-0.0336084204309)*x[1]
        s[3,0,1]=(-0.360543465066)*x[0]**o+(0.369432311166)*x[0]+(0.56460991873)*x[1]**o+(-0.56949007527)*x[1]
        s[3,1,0]=(0.914890366289)*x[0]**o+(0.216556566085)*x[0]+(-0.720400002066)*x[1]**o+(-0.44569085463)*x[1]
        s[3,1,1]=(0.0855171738813)*x[0]**o+(-0.47318307368)*x[0]+(0.267072371169)*x[1]**o+(-0.861793396744)*x[1]
        s[4,0,0]=(-0.862522626201)*x[0]**o+(0.101759724363)*x[0]+(-0.843728984752)*x[1]**o+(-0.510998623672)*x[1]
        s[4,0,1]=(0.827712219176)*x[0]**o+(-0.83052404804)*x[0]+(-0.349511963621)*x[1]**o+(0.379838260239)*x[1]
        s[4,1,0]=(0.923120645339)*x[0]**o+(-0.310138654901)*x[0]+(-0.0364015485007)*x[1]**o+(-0.869695736822)*x[1]
        s[4,1,1]=(0.462952706773)*x[0]**o+(0.391666695595)*x[0]+(0.918559046828)*x[1]**o+(0.0448293018515)*x[1]
        s[5,0,0]=(0.708616542984)*x[0]**o+(-0.826033306645)*x[0]+(0.50975079739)*x[1]**o+(-0.213839768114)*x[1]
        s[5,0,1]=(0.874170725144)*x[0]**o+(0.128033874112)*x[0]+(-0.236784782085)*x[1]**o+(0.885355734442)*x[1]
        s[5,1,0]=(0.0315543075879)*x[0]**o+(-0.482419902735)*x[0]+(0.112741960603)*x[1]**o+(0.336157525634)*x[1]
        s[5,1,1]=(0.000481165869329)*x[0]**o+(-0.356220818813)*x[0]+(-0.466568234378)*x[1]**o+(0.851934710295)*x[1]
        ref[0,0,0]=(-0.168009677754)/(o+1.)+(0.695049087447)+(-0.2538014101)*0.5**o
        ref[0,0,1]=(0.861757151221)/(o+1.)+(-0.318385699161)+(-0.42169068503)*0.5**o
        ref[0,1,0]=(-0.653627037953)/(o+1.)+(0.187364494395)+(0.761004021853)*0.5**o
        ref[0,1,1]=(0.64023828193)/(o+1.)+(-0.276876512691)+(0.571752563364)*0.5**o
        ref[1,0,0]=(-0.815301528075)/(o+1.)+(-0.269864017338)+(-0.229152209793)*0.5**o
        ref[1,0,1]=(0.387642938322)/(o+1.)+(-0.515859032595)+(0.497997942477)*0.5**o
        ref[1,1,0]=(0.709660265053)/(o+1.)+(-0.249216614192)+(-0.396407524403)*0.5**o
        ref[1,1,1]=(0.401013753168)/(o+1.)+(0.726758744261)+(-0.810340425309)*0.5**o
        ref[2,0,0]=(0.872734305266)/(o+1.)+(-0.503576543976)+(0.096907527902)*0.5**o
        ref[2,0,1]=(0.838133724288)/(o+1.)+(0.308712899665)+(-0.657791763259)*0.5**o
        ref[2,1,0]=(0.188605842026)/(o+1.)+(-0.169030703344)+(-0.982546119133)*0.5**o
        ref[2,1,1]=(-0.700736177085)/(o+1.)+(0.10920716537)+(0.623965743731)*0.5**o
        ref[3,0,0]=(-0.922751077939)/(o+1.)+(-0.0741387591729)+(0.154576108414)*0.5**o
        ref[3,0,1]=(0.56460991873)/(o+1.)+(-0.100028882052)+(-0.360543465066)*0.5**o
        ref[3,1,0]=(-0.720400002066)/(o+1.)+(-0.114567144272)+(0.914890366289)*0.5**o
        ref[3,1,1]=(0.267072371169)/(o+1.)+(-0.667488235212)+(0.0855171738813)*0.5**o
        ref[4,0,0]=(-0.843728984752)/(o+1.)+(-0.204619449655)+(-0.862522626201)*0.5**o
        ref[4,0,1]=(-0.349511963621)/(o+1.)+(-0.2253428939)+(0.827712219176)*0.5**o
        ref[4,1,0]=(-0.0364015485007)/(o+1.)+(-0.589917195861)+(0.923120645339)*0.5**o
        ref[4,1,1]=(0.918559046828)/(o+1.)+(0.218247998723)+(0.462952706773)*0.5**o
        ref[5,0,0]=(0.50975079739)/(o+1.)+(-0.51993653738)+(0.708616542984)*0.5**o
        ref[5,0,1]=(-0.236784782085)/(o+1.)+(0.506694804277)+(0.874170725144)*0.5**o
        ref[5,1,0]=(0.112741960603)/(o+1.)+(-0.0731311885505)+(0.0315543075879)*0.5**o
        ref[5,1,1]=(-0.466568234378)/(o+1.)+(0.247856945741)+(0.000481165869329)*0.5**o
      else:
        s[0,0,0]=(0.712776413788)*x[0]**o+(-0.232827372726)*x[0]+(0.504361060322)*x[1]**o+(-0.646213742366)*x[1]+(-0.292404315781)*x[2]**o+(0.0934342710289)*x[2]
        s[0,0,1]=(-0.6231391885)*x[0]**o+(-0.201681226437)*x[0]+(0.295572186857)*x[1]**o+(0.490652427459)*x[1]+(-0.816764035717)*x[2]**o+(-0.920253719103)*x[2]
        s[0,1,0]=(0.464396400115)*x[0]**o+(0.256116225806)*x[0]+(0.00347318217819)*x[1]**o+(0.290749204508)*x[1]+(0.755673392263)*x[2]**o+(0.0296476243926)*x[2]
        s[0,1,1]=(-0.901282392808)*x[0]**o+(0.390894518003)*x[0]+(-0.638342109049)*x[1]**o+(0.325572670945)*x[1]+(0.011997038614)*x[2]**o+(-0.953246898018)*x[2]
        s[1,0,0]=(-0.234152998295)*x[0]**o+(-0.686879447381)*x[0]+(0.684513221814)*x[1]**o+(0.314162707683)*x[1]+(-0.140878044122)*x[2]**o+(0.588563624841)*x[2]
        s[1,0,1]=(0.00302398956565)*x[0]**o+(-0.255603279589)*x[0]+(-0.356254001427)*x[1]**o+(-0.00335409560896)*x[1]+(0.761527454384)*x[2]**o+(-0.803864763242)*x[2]
        s[1,1,0]=(0.350958444218)*x[0]**o+(0.626399553048)*x[0]+(-0.804525479633)*x[1]**o+(0.999024446128)*x[1]+(-0.747418498661)*x[2]**o+(0.16538209042)*x[2]
        s[1,1,1]=(0.830959844328)*x[0]**o+(-0.838517518014)*x[0]+(-0.815601516751)*x[1]**o+(-0.527962402292)*x[1]+(0.849695437437)*x[2]**o+(0.0461655137013)*x[2]
        s[2,0,0]=(-0.654485309003)*x[0]**o+(-0.922437021301)*x[0]+(0.0726219637303)*x[1]**o+(-0.27358467284)*x[1]+(0.965940347168)*x[2]**o+(0.136286409807)*x[2]
        s[2,0,1]=(-0.765669487668)*x[0]**o+(-0.381631759049)*x[0]+(-0.338604617382)*x[1]**o+(-0.026416557321)*x[1]+(-0.505094659996)*x[2]**o+(-0.109271565345)*x[2]
        s[2,1,0]=(-0.823307360412)*x[0]**o+(0.742603440907)*x[0]+(0.264688242655)*x[1]**o+(0.152850147645)*x[1]+(0.576103041841)*x[2]**o+(-0.5223182504)*x[2]
        s[2,1,1]=(-0.853043070027)*x[0]**o+(-0.841177435972)*x[0]+(-0.899106240107)*x[1]**o+(0.605850900198)*x[1]+(0.66690888915)*x[2]**o+(-0.294170728255)*x[2]
        s[3,0,0]=(0.570005513209)*x[0]**o+(0.823970182367)*x[0]+(-0.097784833249)*x[1]**o+(0.873052382528)*x[1]+(-0.0636165773808)*x[2]**o+(0.282310913291)*x[2]
        s[3,0,1]=(0.456449154672)*x[0]**o+(0.951161773651)*x[0]+(-0.373225942356)*x[1]**o+(-0.199594431547)*x[1]+(-0.484728313328)*x[2]**o+(-0.931817560358)*x[2]
        s[3,1,0]=(-0.41042836838)*x[0]**o+(0.771670661129)*x[0]+(-0.0646371268204)*x[1]**o+(0.881286289043)*x[1]+(-0.197285717492)*x[2]**o+(0.857484079398)*x[2]
        s[3,1,1]=(0.297402801452)*x[0]**o+(-0.147528679657)*x[0]+(0.0071150978913)*x[1]**o+(0.788738967825)*x[1]+(-0.191726852031)*x[2]**o+(-0.481649460835)*x[2]
        s[4,0,0]=(-0.808360899963)*x[0]**o+(-0.181264991463)*x[0]+(-0.667551398858)*x[1]**o+(0.289372159531)*x[1]+(0.732773385522)*x[2]**o+(-0.123945731785)*x[2]
        s[4,0,1]=(0.854370035772)*x[0]**o+(0.169945736996)*x[0]+(0.305712234323)*x[1]**o+(-0.60291919973)*x[1]+(0.0403778674069)*x[2]**o+(-0.0309870891437)*x[2]
        s[4,1,0]=(0.543462693144)*x[0]**o+(0.102574070937)*x[0]+(0.963469439068)*x[1]**o+(-0.985245165577)*x[1]+(-0.74408200708)*x[2]**o+(-0.146993364858)*x[2]
        s[4,1,1]=(-0.50927952968)*x[0]**o+(0.471050528827)*x[0]+(0.554105772637)*x[1]**o+(0.265977196333)*x[1]+(-0.447676667005)*x[2]**o+(-0.725761367739)*x[2]
        s[5,0,0]=(0.738549622917)*x[0]**o+(-0.300701131285)*x[0]+(0.770397993383)*x[1]**o+(0.604771638849)*x[1]+(-0.839284085987)*x[2]**o+(-0.636936520767)*x[2]
        s[5,0,1]=(0.546026750751)*x[0]**o+(-0.413469373764)*x[0]+(-0.246023997114)*x[1]**o+(0.207570581839)*x[1]+(0.447030203856)*x[2]**o+(0.0894542975251)*x[2]
        s[5,1,0]=(0.262793971997)*x[0]**o+(-0.814182314897)*x[0]+(-0.340418569268)*x[1]**o+(-0.483737889608)*x[1]+(0.287508080597)*x[2]**o+(-0.602853752221)*x[2]
        s[5,1,1]=(-0.417452810992)*x[0]**o+(-0.906502058242)*x[0]+(0.280357642399)*x[1]**o+(-0.452257960569)*x[1]+(0.239691800726)*x[2]**o+(0.0506798793624)*x[2]
        ref[0,0,0]=(0.211956744542)/(o+1.)+(-0.392803422032)+(0.712776413788)*0.5**o
        ref[0,0,1]=(-0.521191848861)/(o+1.)+(-0.31564125904)+(-0.6231391885)*0.5**o
        ref[0,1,0]=(0.759146574441)/(o+1.)+(0.288256527353)+(0.464396400115)*0.5**o
        ref[0,1,1]=(-0.626345070435)/(o+1.)+(-0.118389854535)+(-0.901282392808)*0.5**o
        ref[1,0,0]=(0.543635177691)/(o+1.)+(0.107923442572)+(-0.234152998295)*0.5**o
        ref[1,0,1]=(0.405273452958)/(o+1.)+(-0.53141106922)+(0.00302398956565)*0.5**o
        ref[1,1,0]=(-1.55194397829)/(o+1.)+(0.895403044797)+(0.350958444218)*0.5**o
        ref[1,1,1]=(0.0340939206865)/(o+1.)+(-0.660157203302)+(0.830959844328)*0.5**o
        ref[2,0,0]=(1.0385623109)/(o+1.)+(-0.529867642167)+(-0.654485309003)*0.5**o
        ref[2,0,1]=(-0.843699277378)/(o+1.)+(-0.258659940857)+(-0.765669487668)*0.5**o
        ref[2,1,0]=(0.840791284496)/(o+1.)+(0.186567669076)+(-0.823307360412)*0.5**o
        ref[2,1,1]=(-0.232197350956)/(o+1.)+(-0.264748632015)+(-0.853043070027)*0.5**o
        ref[3,0,0]=(-0.16140141063)/(o+1.)+(0.989666739093)+(0.570005513209)*0.5**o
        ref[3,0,1]=(-0.857954255684)/(o+1.)+(-0.0901251091271)+(0.456449154672)*0.5**o
        ref[3,1,0]=(-0.261922844312)/(o+1.)+(1.25522051478)+(-0.41042836838)*0.5**o
        ref[3,1,1]=(-0.18461175414)/(o+1.)+(0.0797804136672)+(0.297402801452)*0.5**o
        ref[4,0,0]=(0.0652219866638)/(o+1.)+(-0.00791928185857)+(-0.808360899963)*0.5**o
        ref[4,0,1]=(0.34609010173)/(o+1.)+(-0.231980275939)+(0.854370035772)*0.5**o
        ref[4,1,0]=(0.219387431988)/(o+1.)+(-0.514832229749)+(0.543462693144)*0.5**o
        ref[4,1,1]=(0.106429105632)/(o+1.)+(0.00563317871007)+(-0.50927952968)*0.5**o
        ref[5,0,0]=(-0.0688860926042)/(o+1.)+(-0.166433006601)+(0.738549622917)*0.5**o
        ref[5,0,1]=(0.201006206742)/(o+1.)+(-0.0582222472)+(0.546026750751)*0.5**o
        ref[5,1,0]=(-0.0529104886711)/(o+1.)+(-0.950386978363)+(0.262793971997)*0.5**o
        ref[5,1,1]=(0.520049443126)/(o+1.)+(-0.654040069724)+(-0.417452810992)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactZero_fromSymbol_ContinuousFunction_rank4(self):
      """
      tests integral of rank 4 Symbol on the FunctionOnContactZero

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports integral on FunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5, 3, 2))
      s=Data(0,(4, 5, 3, 2),w)
      ref=numpy.zeros((4, 5, 3, 2),numpy.float_)
      if dim==2:
        s[0,0,0,0]=(-0.829288219058)*x[0]**o+(-0.65217791272)*x[0]+(0.698846288116)*x[1]**o+(-0.0392705140136)*x[1]
        s[0,0,0,1]=(-0.456023711562)*x[0]**o+(0.9183516109)*x[0]+(0.927490172078)*x[1]**o+(-0.394686077168)*x[1]
        s[0,0,1,0]=(-0.497067376291)*x[0]**o+(0.237428033612)*x[0]+(-0.785186724625)*x[1]**o+(0.471101666176)*x[1]
        s[0,0,1,1]=(-0.290568226175)*x[0]**o+(0.893982019035)*x[0]+(0.203073455207)*x[1]**o+(0.78973496755)*x[1]
        s[0,0,2,0]=(0.00829962077999)*x[0]**o+(0.99718958928)*x[0]+(0.330466533936)*x[1]**o+(-0.619185067306)*x[1]
        s[0,0,2,1]=(-0.806092658508)*x[0]**o+(-0.307457723596)*x[0]+(0.847169623314)*x[1]**o+(-0.832290917062)*x[1]
        s[0,1,0,0]=(-0.140461069383)*x[0]**o+(0.760902362673)*x[0]+(0.0231278168453)*x[1]**o+(-0.508849645358)*x[1]
        s[0,1,0,1]=(0.587581627616)*x[0]**o+(-0.229546331489)*x[0]+(-0.659235919696)*x[1]**o+(0.80409204008)*x[1]
        s[0,1,1,0]=(-0.641654760801)*x[0]**o+(0.122524921907)*x[0]+(0.122921572782)*x[1]**o+(-0.23094495884)*x[1]
        s[0,1,1,1]=(-0.380132178505)*x[0]**o+(-0.769451293694)*x[0]+(-0.745741833491)*x[1]**o+(0.11284350488)*x[1]
        s[0,1,2,0]=(0.126835269214)*x[0]**o+(0.918739367312)*x[0]+(0.927407682128)*x[1]**o+(-0.273231826206)*x[1]
        s[0,1,2,1]=(0.608413549196)*x[0]**o+(-0.591552902664)*x[0]+(0.873696757566)*x[1]**o+(-0.900225496133)*x[1]
        s[0,2,0,0]=(0.0488258290791)*x[0]**o+(0.211067968473)*x[0]+(0.11529432626)*x[1]**o+(0.497957233179)*x[1]
        s[0,2,0,1]=(-0.780993161707)*x[0]**o+(0.746546880275)*x[0]+(-0.0997128168301)*x[1]**o+(-0.663371860066)*x[1]
        s[0,2,1,0]=(0.305794159864)*x[0]**o+(0.493723656879)*x[0]+(0.334264621015)*x[1]**o+(0.0579972251793)*x[1]
        s[0,2,1,1]=(0.71588138307)*x[0]**o+(-0.0866536386387)*x[0]+(-0.159835910161)*x[1]**o+(0.562258753174)*x[1]
        s[0,2,2,0]=(0.864605462382)*x[0]**o+(0.818516795215)*x[0]+(-0.0558026806854)*x[1]**o+(-0.225953364341)*x[1]
        s[0,2,2,1]=(-0.0798994114092)*x[0]**o+(-0.73636091403)*x[0]+(-0.478543932177)*x[1]**o+(0.875789857891)*x[1]
        s[0,3,0,0]=(0.998019190199)*x[0]**o+(-0.218007434364)*x[0]+(0.181378057097)*x[1]**o+(0.429126350197)*x[1]
        s[0,3,0,1]=(-0.724306549649)*x[0]**o+(-0.303805025896)*x[0]+(0.58603690984)*x[1]**o+(0.976611908964)*x[1]
        s[0,3,1,0]=(0.776728843377)*x[0]**o+(-0.874157618694)*x[0]+(-0.590753603973)*x[1]**o+(0.185288488913)*x[1]
        s[0,3,1,1]=(0.285035031516)*x[0]**o+(-0.0667711315126)*x[0]+(0.220692714675)*x[1]**o+(0.841408314724)*x[1]
        s[0,3,2,0]=(-0.00286941650137)*x[0]**o+(0.59339349489)*x[0]+(-0.158356936936)*x[1]**o+(-0.451137261062)*x[1]
        s[0,3,2,1]=(0.37992478387)*x[0]**o+(0.835018328571)*x[0]+(-0.446900268053)*x[1]**o+(-0.700751807162)*x[1]
        s[0,4,0,0]=(0.714232167791)*x[0]**o+(-0.688473756667)*x[0]+(0.0552121131111)*x[1]**o+(0.632003397091)*x[1]
        s[0,4,0,1]=(0.49556700372)*x[0]**o+(0.0630560952096)*x[0]+(0.321456277553)*x[1]**o+(0.685060833092)*x[1]
        s[0,4,1,0]=(0.841677614054)*x[0]**o+(0.886957973836)*x[0]+(0.0202839586672)*x[1]**o+(-0.923265656841)*x[1]
        s[0,4,1,1]=(-0.504028346474)*x[0]**o+(0.071280762432)*x[0]+(0.629772140166)*x[1]**o+(-0.195290635886)*x[1]
        s[0,4,2,0]=(0.434265669856)*x[0]**o+(0.376229784992)*x[0]+(0.919945289519)*x[1]**o+(-0.496219699959)*x[1]
        s[0,4,2,1]=(-0.86013960593)*x[0]**o+(0.140206694208)*x[0]+(-0.834647694686)*x[1]**o+(0.381383127993)*x[1]
        s[1,0,0,0]=(-0.437686950383)*x[0]**o+(0.385307277242)*x[0]+(-0.430841927673)*x[1]**o+(-0.556445729508)*x[1]
        s[1,0,0,1]=(0.323843044435)*x[0]**o+(-0.21417604806)*x[0]+(-0.172390658038)*x[1]**o+(-0.847272401994)*x[1]
        s[1,0,1,0]=(-0.845882465152)*x[0]**o+(-0.598486605858)*x[0]+(0.89008620109)*x[1]**o+(0.874381441062)*x[1]
        s[1,0,1,1]=(-0.405423425934)*x[0]**o+(-0.404267054882)*x[0]+(-0.63801489002)*x[1]**o+(-0.728771425845)*x[1]
        s[1,0,2,0]=(-0.216267078676)*x[0]**o+(0.956804195842)*x[0]+(-0.908451202669)*x[1]**o+(-0.322043280117)*x[1]
        s[1,0,2,1]=(-0.168907777516)*x[0]**o+(0.840516962339)*x[0]+(0.103177353457)*x[1]**o+(0.939190271608)*x[1]
        s[1,1,0,0]=(-0.884189035361)*x[0]**o+(-0.396863590748)*x[0]+(-0.380355614494)*x[1]**o+(0.913843764972)*x[1]
        s[1,1,0,1]=(-0.983514772022)*x[0]**o+(0.953676778353)*x[0]+(-0.141666532996)*x[1]**o+(-0.782763806635)*x[1]
        s[1,1,1,0]=(0.723588279961)*x[0]**o+(0.0698022948292)*x[0]+(-0.202960085361)*x[1]**o+(-0.0971108181392)*x[1]
        s[1,1,1,1]=(-0.809610385863)*x[0]**o+(0.78262340178)*x[0]+(-0.292492437285)*x[1]**o+(0.909658049109)*x[1]
        s[1,1,2,0]=(0.831333270285)*x[0]**o+(0.532412110902)*x[0]+(0.852180039142)*x[1]**o+(-0.126550447094)*x[1]
        s[1,1,2,1]=(0.889635940128)*x[0]**o+(-0.0146231223219)*x[0]+(-0.505300606168)*x[1]**o+(0.286596962758)*x[1]
        s[1,2,0,0]=(-0.479209867816)*x[0]**o+(-0.707176716613)*x[0]+(0.385223744345)*x[1]**o+(0.679335984842)*x[1]
        s[1,2,0,1]=(-0.690494514078)*x[0]**o+(-0.591659781424)*x[0]+(-0.111527008468)*x[1]**o+(0.73261078188)*x[1]
        s[1,2,1,0]=(-0.65110579618)*x[0]**o+(-0.861397125883)*x[0]+(-0.264211841662)*x[1]**o+(0.750122373459)*x[1]
        s[1,2,1,1]=(-0.495114108462)*x[0]**o+(-0.555683103062)*x[0]+(-0.0384737773817)*x[1]**o+(0.422372598888)*x[1]
        s[1,2,2,0]=(-0.846365789548)*x[0]**o+(0.247402342046)*x[0]+(-0.998046573914)*x[1]**o+(0.264003327979)*x[1]
        s[1,2,2,1]=(0.4479853512)*x[0]**o+(-0.502771753169)*x[0]+(-0.366626482676)*x[1]**o+(0.506404378836)*x[1]
        s[1,3,0,0]=(-0.677771809341)*x[0]**o+(-0.105232399291)*x[0]+(0.10440523429)*x[1]**o+(-0.780931426435)*x[1]
        s[1,3,0,1]=(0.549864901432)*x[0]**o+(0.79598502718)*x[0]+(-0.424985507693)*x[1]**o+(-0.649629583228)*x[1]
        s[1,3,1,0]=(0.795465752618)*x[0]**o+(-0.0630573960436)*x[0]+(-0.158966662533)*x[1]**o+(0.888189443186)*x[1]
        s[1,3,1,1]=(-0.395120222846)*x[0]**o+(-0.529551413114)*x[0]+(-0.351127736099)*x[1]**o+(-0.960771541871)*x[1]
        s[1,3,2,0]=(-0.244306249618)*x[0]**o+(0.925164172251)*x[0]+(-0.988402656417)*x[1]**o+(-0.0797097670335)*x[1]
        s[1,3,2,1]=(0.905866127894)*x[0]**o+(0.135673487702)*x[0]+(-0.873223112612)*x[1]**o+(0.598121374326)*x[1]
        s[1,4,0,0]=(0.171338765926)*x[0]**o+(-0.31796084081)*x[0]+(-0.914462477805)*x[1]**o+(0.923691484317)*x[1]
        s[1,4,0,1]=(-0.188491880655)*x[0]**o+(-0.67119846649)*x[0]+(0.488643815471)*x[1]**o+(-0.912992530018)*x[1]
        s[1,4,1,0]=(0.252606322081)*x[0]**o+(0.509944026633)*x[0]+(0.539656085813)*x[1]**o+(0.0286599718948)*x[1]
        s[1,4,1,1]=(0.148934160341)*x[0]**o+(0.50451378885)*x[0]+(0.878910174211)*x[1]**o+(-0.617480145541)*x[1]
        s[1,4,2,0]=(-0.235447116973)*x[0]**o+(0.825149714177)*x[0]+(0.300052096347)*x[1]**o+(-0.360690924033)*x[1]
        s[1,4,2,1]=(0.944445188093)*x[0]**o+(-0.170511625527)*x[0]+(-0.333492913384)*x[1]**o+(0.907577674324)*x[1]
        s[2,0,0,0]=(-0.942092595372)*x[0]**o+(-0.254482071505)*x[0]+(-0.0151488336686)*x[1]**o+(-0.0287652780384)*x[1]
        s[2,0,0,1]=(0.0655506752825)*x[0]**o+(0.898719259761)*x[0]+(0.383024592178)*x[1]**o+(0.0989444595983)*x[1]
        s[2,0,1,0]=(0.291304803618)*x[0]**o+(-0.720915295421)*x[0]+(0.138378684)*x[1]**o+(0.876226100573)*x[1]
        s[2,0,1,1]=(0.0625773477714)*x[0]**o+(-0.0294480886227)*x[0]+(0.736449427619)*x[1]**o+(0.371697786988)*x[1]
        s[2,0,2,0]=(-0.931044858876)*x[0]**o+(0.911308988417)*x[0]+(0.689775520125)*x[1]**o+(0.415135153045)*x[1]
        s[2,0,2,1]=(-0.732200781532)*x[0]**o+(0.395135482591)*x[0]+(-0.619692129874)*x[1]**o+(-0.912759567625)*x[1]
        s[2,1,0,0]=(-0.261557477097)*x[0]**o+(-0.933044351058)*x[0]+(0.0156599006787)*x[1]**o+(-0.082399909281)*x[1]
        s[2,1,0,1]=(0.723444855088)*x[0]**o+(-0.216911506016)*x[0]+(0.604446335087)*x[1]**o+(0.947367685426)*x[1]
        s[2,1,1,0]=(-0.606511927315)*x[0]**o+(0.825992890197)*x[0]+(0.757190663205)*x[1]**o+(-0.0991622858024)*x[1]
        s[2,1,1,1]=(0.45110092033)*x[0]**o+(0.0571087683245)*x[0]+(-0.825169962155)*x[1]**o+(0.557059283915)*x[1]
        s[2,1,2,0]=(-0.612806058565)*x[0]**o+(0.928772523716)*x[0]+(-0.288177463065)*x[1]**o+(0.341361092846)*x[1]
        s[2,1,2,1]=(-0.746890321439)*x[0]**o+(0.893018453839)*x[0]+(0.0308905730956)*x[1]**o+(0.291585488925)*x[1]
        s[2,2,0,0]=(-0.665223434533)*x[0]**o+(-0.997256499342)*x[0]+(-0.134118374176)*x[1]**o+(-0.930692055226)*x[1]
        s[2,2,0,1]=(-0.478607794082)*x[0]**o+(0.0372165046416)*x[0]+(-0.149775511078)*x[1]**o+(-0.412243756318)*x[1]
        s[2,2,1,0]=(-0.782692588953)*x[0]**o+(0.896152570987)*x[0]+(-0.265361365664)*x[1]**o+(0.0859566508122)*x[1]
        s[2,2,1,1]=(0.141591460973)*x[0]**o+(0.399004634853)*x[0]+(0.351166595901)*x[1]**o+(-0.230314460514)*x[1]
        s[2,2,2,0]=(-0.282452862833)*x[0]**o+(0.652348758087)*x[0]+(-0.006664462988)*x[1]**o+(0.115435922234)*x[1]
        s[2,2,2,1]=(-0.108570066423)*x[0]**o+(0.536881567792)*x[0]+(-0.967086006781)*x[1]**o+(0.964315151846)*x[1]
        s[2,3,0,0]=(-0.872481061987)*x[0]**o+(-0.152036048838)*x[0]+(0.714460711637)*x[1]**o+(-0.641192704763)*x[1]
        s[2,3,0,1]=(-0.314322497313)*x[0]**o+(0.766601974446)*x[0]+(-0.304360730035)*x[1]**o+(-0.445351381604)*x[1]
        s[2,3,1,0]=(0.364419198839)*x[0]**o+(-0.615166967343)*x[0]+(0.797973787987)*x[1]**o+(0.312408283718)*x[1]
        s[2,3,1,1]=(-0.781147212189)*x[0]**o+(-0.549359753018)*x[0]+(0.235681715573)*x[1]**o+(-0.231862286881)*x[1]
        s[2,3,2,0]=(-0.958493367772)*x[0]**o+(-0.775968770096)*x[0]+(-0.917090286139)*x[1]**o+(0.656115811881)*x[1]
        s[2,3,2,1]=(0.572779229598)*x[0]**o+(-0.875089280859)*x[0]+(0.575188283953)*x[1]**o+(-0.617895968371)*x[1]
        s[2,4,0,0]=(-0.178437300104)*x[0]**o+(-0.305311887897)*x[0]+(-0.362423234235)*x[1]**o+(-0.993155475703)*x[1]
        s[2,4,0,1]=(-0.842863324823)*x[0]**o+(-0.334505517678)*x[0]+(-0.233874335956)*x[1]**o+(-0.280499554274)*x[1]
        s[2,4,1,0]=(-0.396341438458)*x[0]**o+(0.605087475544)*x[0]+(-0.648099915852)*x[1]**o+(-0.338407917903)*x[1]
        s[2,4,1,1]=(-0.359913098139)*x[0]**o+(0.80935120431)*x[0]+(-0.969248694111)*x[1]**o+(0.884624495846)*x[1]
        s[2,4,2,0]=(0.965251612654)*x[0]**o+(-0.811340257859)*x[0]+(0.960012562229)*x[1]**o+(-0.825791870471)*x[1]
        s[2,4,2,1]=(0.43888464177)*x[0]**o+(0.126116195477)*x[0]+(0.0455614359233)*x[1]**o+(0.387001710336)*x[1]
        s[3,0,0,0]=(0.961744925179)*x[0]**o+(0.590267645729)*x[0]+(-0.657096083564)*x[1]**o+(-0.336931534761)*x[1]
        s[3,0,0,1]=(0.0372329050631)*x[0]**o+(0.82966683846)*x[0]+(-0.537423647089)*x[1]**o+(0.13780696077)*x[1]
        s[3,0,1,0]=(-0.762516609688)*x[0]**o+(-0.619904320036)*x[0]+(0.653234747183)*x[1]**o+(0.3188610791)*x[1]
        s[3,0,1,1]=(-0.749684385643)*x[0]**o+(-0.30857008835)*x[0]+(0.066895978166)*x[1]**o+(-0.603289571527)*x[1]
        s[3,0,2,0]=(-0.621186971872)*x[0]**o+(0.109237849031)*x[0]+(0.312234719711)*x[1]**o+(0.214139545654)*x[1]
        s[3,0,2,1]=(-0.479630544493)*x[0]**o+(0.936515248516)*x[0]+(0.135913071464)*x[1]**o+(0.536193827105)*x[1]
        s[3,1,0,0]=(-0.419606668612)*x[0]**o+(-0.825159184029)*x[0]+(-0.862279634673)*x[1]**o+(0.0205463968802)*x[1]
        s[3,1,0,1]=(0.35798046036)*x[0]**o+(-0.599162907156)*x[0]+(0.174478509759)*x[1]**o+(-0.779627181611)*x[1]
        s[3,1,1,0]=(-0.175201495388)*x[0]**o+(0.754090549688)*x[0]+(-0.131095703521)*x[1]**o+(-0.149892880631)*x[1]
        s[3,1,1,1]=(-0.718173244947)*x[0]**o+(0.387119862798)*x[0]+(-0.190359128182)*x[1]**o+(0.71904977881)*x[1]
        s[3,1,2,0]=(0.542104637274)*x[0]**o+(0.85151096271)*x[0]+(0.0402471065929)*x[1]**o+(-0.276985707209)*x[1]
        s[3,1,2,1]=(-0.612520892999)*x[0]**o+(0.475838856018)*x[0]+(-0.524069927205)*x[1]**o+(-0.314728620326)*x[1]
        s[3,2,0,0]=(0.853018031276)*x[0]**o+(0.783150840935)*x[0]+(-0.168512012086)*x[1]**o+(0.713965998504)*x[1]
        s[3,2,0,1]=(0.253413905488)*x[0]**o+(-0.433043547194)*x[0]+(-0.775710938403)*x[1]**o+(-0.906939451854)*x[1]
        s[3,2,1,0]=(0.6612655067)*x[0]**o+(0.959161354342)*x[0]+(-0.351925885901)*x[1]**o+(-0.542741000383)*x[1]
        s[3,2,1,1]=(-0.651886826012)*x[0]**o+(0.799653456288)*x[0]+(0.482387863091)*x[1]**o+(-0.628626395493)*x[1]
        s[3,2,2,0]=(-0.314962063276)*x[0]**o+(-0.237445058522)*x[0]+(0.983233001597)*x[1]**o+(0.105803386746)*x[1]
        s[3,2,2,1]=(0.786422364797)*x[0]**o+(0.437192400732)*x[0]+(-0.238461142143)*x[1]**o+(0.696286963331)*x[1]
        s[3,3,0,0]=(0.483733340742)*x[0]**o+(0.749644548963)*x[0]+(0.765700270847)*x[1]**o+(-0.470185526075)*x[1]
        s[3,3,0,1]=(0.680316641067)*x[0]**o+(0.952443208592)*x[0]+(-0.0164617510632)*x[1]**o+(-0.234575904406)*x[1]
        s[3,3,1,0]=(-0.993408262104)*x[0]**o+(-0.229008195245)*x[0]+(0.523753196917)*x[1]**o+(-0.318485891049)*x[1]
        s[3,3,1,1]=(-0.748894192494)*x[0]**o+(-0.879841134058)*x[0]+(-0.650074035674)*x[1]**o+(0.07564873608)*x[1]
        s[3,3,2,0]=(-0.0390185724072)*x[0]**o+(0.754358690288)*x[0]+(-0.961050878591)*x[1]**o+(-0.849404044511)*x[1]
        s[3,3,2,1]=(0.483849885635)*x[0]**o+(0.168666402938)*x[0]+(-0.948399681695)*x[1]**o+(-0.923760856309)*x[1]
        s[3,4,0,0]=(0.0487172550208)*x[0]**o+(-0.627393318222)*x[0]+(0.328198988132)*x[1]**o+(0.615434829581)*x[1]
        s[3,4,0,1]=(0.294964552743)*x[0]**o+(0.593632204587)*x[0]+(-0.922244339835)*x[1]**o+(0.712032424818)*x[1]
        s[3,4,1,0]=(-0.774393234162)*x[0]**o+(0.122655955494)*x[0]+(-0.993126331736)*x[1]**o+(0.553836798684)*x[1]
        s[3,4,1,1]=(-0.46041945879)*x[0]**o+(-0.676034356031)*x[0]+(0.273980415493)*x[1]**o+(-0.190839002618)*x[1]
        s[3,4,2,0]=(0.0439333449053)*x[0]**o+(-0.0796104621766)*x[0]+(0.479839558444)*x[1]**o+(-0.0221747154)*x[1]
        s[3,4,2,1]=(-0.2108539974)*x[0]**o+(-0.743095604828)*x[0]+(-0.531276902705)*x[1]**o+(0.286137673119)*x[1]
        ref[0,0,0,0]=(0.698846288116)/(o+1.)+(-0.345724213367)+(-0.829288219058)*0.5**o
        ref[0,0,0,1]=(0.927490172078)/(o+1.)+(0.261832766866)+(-0.456023711562)*0.5**o
        ref[0,0,1,0]=(-0.785186724625)/(o+1.)+(0.354264849894)+(-0.497067376291)*0.5**o
        ref[0,0,1,1]=(0.203073455207)/(o+1.)+(0.841858493293)+(-0.290568226175)*0.5**o
        ref[0,0,2,0]=(0.330466533936)/(o+1.)+(0.189002260987)+(0.00829962077999)*0.5**o
        ref[0,0,2,1]=(0.847169623314)/(o+1.)+(-0.569874320329)+(-0.806092658508)*0.5**o
        ref[0,1,0,0]=(0.0231278168453)/(o+1.)+(0.126026358657)+(-0.140461069383)*0.5**o
        ref[0,1,0,1]=(-0.659235919696)/(o+1.)+(0.287272854295)+(0.587581627616)*0.5**o
        ref[0,1,1,0]=(0.122921572782)/(o+1.)+(-0.0542100184667)+(-0.641654760801)*0.5**o
        ref[0,1,1,1]=(-0.745741833491)/(o+1.)+(-0.328303894407)+(-0.380132178505)*0.5**o
        ref[0,1,2,0]=(0.927407682128)/(o+1.)+(0.322753770553)+(0.126835269214)*0.5**o
        ref[0,1,2,1]=(0.873696757566)/(o+1.)+(-0.745889199398)+(0.608413549196)*0.5**o
        ref[0,2,0,0]=(0.11529432626)/(o+1.)+(0.354512600826)+(0.0488258290791)*0.5**o
        ref[0,2,0,1]=(-0.0997128168301)/(o+1.)+(0.0415875101043)+(-0.780993161707)*0.5**o
        ref[0,2,1,0]=(0.334264621015)/(o+1.)+(0.275860441029)+(0.305794159864)*0.5**o
        ref[0,2,1,1]=(-0.159835910161)/(o+1.)+(0.237802557268)+(0.71588138307)*0.5**o
        ref[0,2,2,0]=(-0.0558026806854)/(o+1.)+(0.296281715437)+(0.864605462382)*0.5**o
        ref[0,2,2,1]=(-0.478543932177)/(o+1.)+(0.0697144719306)+(-0.0798994114092)*0.5**o
        ref[0,3,0,0]=(0.181378057097)/(o+1.)+(0.105559457917)+(0.998019190199)*0.5**o
        ref[0,3,0,1]=(0.58603690984)/(o+1.)+(0.336403441534)+(-0.724306549649)*0.5**o
        ref[0,3,1,0]=(-0.590753603973)/(o+1.)+(-0.34443456489)+(0.776728843377)*0.5**o
        ref[0,3,1,1]=(0.220692714675)/(o+1.)+(0.387318591606)+(0.285035031516)*0.5**o
        ref[0,3,2,0]=(-0.158356936936)/(o+1.)+(0.0711281169143)+(-0.00286941650137)*0.5**o
        ref[0,3,2,1]=(-0.446900268053)/(o+1.)+(0.0671332607043)+(0.37992478387)*0.5**o
        ref[0,4,0,0]=(0.0552121131111)/(o+1.)+(-0.0282351797878)+(0.714232167791)*0.5**o
        ref[0,4,0,1]=(0.321456277553)/(o+1.)+(0.374058464151)+(0.49556700372)*0.5**o
        ref[0,4,1,0]=(0.0202839586672)/(o+1.)+(-0.0181538415026)+(0.841677614054)*0.5**o
        ref[0,4,1,1]=(0.629772140166)/(o+1.)+(-0.062004936727)+(-0.504028346474)*0.5**o
        ref[0,4,2,0]=(0.919945289519)/(o+1.)+(-0.0599949574834)+(0.434265669856)*0.5**o
        ref[0,4,2,1]=(-0.834647694686)/(o+1.)+(0.2607949111)+(-0.86013960593)*0.5**o
        ref[1,0,0,0]=(-0.430841927673)/(o+1.)+(-0.0855692261333)+(-0.437686950383)*0.5**o
        ref[1,0,0,1]=(-0.172390658038)/(o+1.)+(-0.530724225027)+(0.323843044435)*0.5**o
        ref[1,0,1,0]=(0.89008620109)/(o+1.)+(0.137947417602)+(-0.845882465152)*0.5**o
        ref[1,0,1,1]=(-0.63801489002)/(o+1.)+(-0.566519240363)+(-0.405423425934)*0.5**o
        ref[1,0,2,0]=(-0.908451202669)/(o+1.)+(0.317380457863)+(-0.216267078676)*0.5**o
        ref[1,0,2,1]=(0.103177353457)/(o+1.)+(0.889853616974)+(-0.168907777516)*0.5**o
        ref[1,1,0,0]=(-0.380355614494)/(o+1.)+(0.258490087112)+(-0.884189035361)*0.5**o
        ref[1,1,0,1]=(-0.141666532996)/(o+1.)+(0.0854564858592)+(-0.983514772022)*0.5**o
        ref[1,1,1,0]=(-0.202960085361)/(o+1.)+(-0.013654261655)+(0.723588279961)*0.5**o
        ref[1,1,1,1]=(-0.292492437285)/(o+1.)+(0.846140725444)+(-0.809610385863)*0.5**o
        ref[1,1,2,0]=(0.852180039142)/(o+1.)+(0.202930831904)+(0.831333270285)*0.5**o
        ref[1,1,2,1]=(-0.505300606168)/(o+1.)+(0.135986920218)+(0.889635940128)*0.5**o
        ref[1,2,0,0]=(0.385223744345)/(o+1.)+(-0.0139203658855)+(-0.479209867816)*0.5**o
        ref[1,2,0,1]=(-0.111527008468)/(o+1.)+(0.0704755002279)+(-0.690494514078)*0.5**o
        ref[1,2,1,0]=(-0.264211841662)/(o+1.)+(-0.055637376212)+(-0.65110579618)*0.5**o
        ref[1,2,1,1]=(-0.0384737773817)/(o+1.)+(-0.066655252087)+(-0.495114108462)*0.5**o
        ref[1,2,2,0]=(-0.998046573914)/(o+1.)+(0.255702835012)+(-0.846365789548)*0.5**o
        ref[1,2,2,1]=(-0.366626482676)/(o+1.)+(0.00181631283349)+(0.4479853512)*0.5**o
        ref[1,3,0,0]=(0.10440523429)/(o+1.)+(-0.443081912863)+(-0.677771809341)*0.5**o
        ref[1,3,0,1]=(-0.424985507693)/(o+1.)+(0.0731777219764)+(0.549864901432)*0.5**o
        ref[1,3,1,0]=(-0.158966662533)/(o+1.)+(0.412566023571)+(0.795465752618)*0.5**o
        ref[1,3,1,1]=(-0.351127736099)/(o+1.)+(-0.745161477493)+(-0.395120222846)*0.5**o
        ref[1,3,2,0]=(-0.988402656417)/(o+1.)+(0.422727202609)+(-0.244306249618)*0.5**o
        ref[1,3,2,1]=(-0.873223112612)/(o+1.)+(0.366897431014)+(0.905866127894)*0.5**o
        ref[1,4,0,0]=(-0.914462477805)/(o+1.)+(0.302865321753)+(0.171338765926)*0.5**o
        ref[1,4,0,1]=(0.488643815471)/(o+1.)+(-0.792095498254)+(-0.188491880655)*0.5**o
        ref[1,4,1,0]=(0.539656085813)/(o+1.)+(0.269301999264)+(0.252606322081)*0.5**o
        ref[1,4,1,1]=(0.878910174211)/(o+1.)+(-0.0564831783457)+(0.148934160341)*0.5**o
        ref[1,4,2,0]=(0.300052096347)/(o+1.)+(0.232229395072)+(-0.235447116973)*0.5**o
        ref[1,4,2,1]=(-0.333492913384)/(o+1.)+(0.368533024398)+(0.944445188093)*0.5**o
        ref[2,0,0,0]=(-0.0151488336686)/(o+1.)+(-0.141623674772)+(-0.942092595372)*0.5**o
        ref[2,0,0,1]=(0.383024592178)/(o+1.)+(0.498831859679)+(0.0655506752825)*0.5**o
        ref[2,0,1,0]=(0.138378684)/(o+1.)+(0.0776554025757)+(0.291304803618)*0.5**o
        ref[2,0,1,1]=(0.736449427619)/(o+1.)+(0.171124849182)+(0.0625773477714)*0.5**o
        ref[2,0,2,0]=(0.689775520125)/(o+1.)+(0.663222070731)+(-0.931044858876)*0.5**o
        ref[2,0,2,1]=(-0.619692129874)/(o+1.)+(-0.258812042517)+(-0.732200781532)*0.5**o
        ref[2,1,0,0]=(0.0156599006787)/(o+1.)+(-0.507722130169)+(-0.261557477097)*0.5**o
        ref[2,1,0,1]=(0.604446335087)/(o+1.)+(0.365228089705)+(0.723444855088)*0.5**o
        ref[2,1,1,0]=(0.757190663205)/(o+1.)+(0.363415302197)+(-0.606511927315)*0.5**o
        ref[2,1,1,1]=(-0.825169962155)/(o+1.)+(0.30708402612)+(0.45110092033)*0.5**o
        ref[2,1,2,0]=(-0.288177463065)/(o+1.)+(0.635066808281)+(-0.612806058565)*0.5**o
        ref[2,1,2,1]=(0.0308905730956)/(o+1.)+(0.592301971382)+(-0.746890321439)*0.5**o
        ref[2,2,0,0]=(-0.134118374176)/(o+1.)+(-0.963974277284)+(-0.665223434533)*0.5**o
        ref[2,2,0,1]=(-0.149775511078)/(o+1.)+(-0.187513625838)+(-0.478607794082)*0.5**o
        ref[2,2,1,0]=(-0.265361365664)/(o+1.)+(0.4910546109)+(-0.782692588953)*0.5**o
        ref[2,2,1,1]=(0.351166595901)/(o+1.)+(0.0843450871693)+(0.141591460973)*0.5**o
        ref[2,2,2,0]=(-0.006664462988)/(o+1.)+(0.383892340161)+(-0.282452862833)*0.5**o
        ref[2,2,2,1]=(-0.967086006781)/(o+1.)+(0.750598359819)+(-0.108570066423)*0.5**o
        ref[2,3,0,0]=(0.714460711637)/(o+1.)+(-0.3966143768)+(-0.872481061987)*0.5**o
        ref[2,3,0,1]=(-0.304360730035)/(o+1.)+(0.160625296421)+(-0.314322497313)*0.5**o
        ref[2,3,1,0]=(0.797973787987)/(o+1.)+(-0.151379341813)+(0.364419198839)*0.5**o
        ref[2,3,1,1]=(0.235681715573)/(o+1.)+(-0.390611019949)+(-0.781147212189)*0.5**o
        ref[2,3,2,0]=(-0.917090286139)/(o+1.)+(-0.0599264791076)+(-0.958493367772)*0.5**o
        ref[2,3,2,1]=(0.575188283953)/(o+1.)+(-0.746492624615)+(0.572779229598)*0.5**o
        ref[2,4,0,0]=(-0.362423234235)/(o+1.)+(-0.6492336818)+(-0.178437300104)*0.5**o
        ref[2,4,0,1]=(-0.233874335956)/(o+1.)+(-0.307502535976)+(-0.842863324823)*0.5**o
        ref[2,4,1,0]=(-0.648099915852)/(o+1.)+(0.133339778821)+(-0.396341438458)*0.5**o
        ref[2,4,1,1]=(-0.969248694111)/(o+1.)+(0.846987850078)+(-0.359913098139)*0.5**o
        ref[2,4,2,0]=(0.960012562229)/(o+1.)+(-0.818566064165)+(0.965251612654)*0.5**o
        ref[2,4,2,1]=(0.0455614359233)/(o+1.)+(0.256558952907)+(0.43888464177)*0.5**o
        ref[3,0,0,0]=(-0.657096083564)/(o+1.)+(0.126668055484)+(0.961744925179)*0.5**o
        ref[3,0,0,1]=(-0.537423647089)/(o+1.)+(0.483736899615)+(0.0372329050631)*0.5**o
        ref[3,0,1,0]=(0.653234747183)/(o+1.)+(-0.150521620468)+(-0.762516609688)*0.5**o
        ref[3,0,1,1]=(0.066895978166)/(o+1.)+(-0.455929829939)+(-0.749684385643)*0.5**o
        ref[3,0,2,0]=(0.312234719711)/(o+1.)+(0.161688697343)+(-0.621186971872)*0.5**o
        ref[3,0,2,1]=(0.135913071464)/(o+1.)+(0.73635453781)+(-0.479630544493)*0.5**o
        ref[3,1,0,0]=(-0.862279634673)/(o+1.)+(-0.402306393574)+(-0.419606668612)*0.5**o
        ref[3,1,0,1]=(0.174478509759)/(o+1.)+(-0.689395044384)+(0.35798046036)*0.5**o
        ref[3,1,1,0]=(-0.131095703521)/(o+1.)+(0.302098834529)+(-0.175201495388)*0.5**o
        ref[3,1,1,1]=(-0.190359128182)/(o+1.)+(0.553084820804)+(-0.718173244947)*0.5**o
        ref[3,1,2,0]=(0.0402471065929)/(o+1.)+(0.28726262775)+(0.542104637274)*0.5**o
        ref[3,1,2,1]=(-0.524069927205)/(o+1.)+(0.0805551178455)+(-0.612520892999)*0.5**o
        ref[3,2,0,0]=(-0.168512012086)/(o+1.)+(0.748558419719)+(0.853018031276)*0.5**o
        ref[3,2,0,1]=(-0.775710938403)/(o+1.)+(-0.669991499524)+(0.253413905488)*0.5**o
        ref[3,2,1,0]=(-0.351925885901)/(o+1.)+(0.20821017698)+(0.6612655067)*0.5**o
        ref[3,2,1,1]=(0.482387863091)/(o+1.)+(0.0855135303978)+(-0.651886826012)*0.5**o
        ref[3,2,2,0]=(0.983233001597)/(o+1.)+(-0.0658208358883)+(-0.314962063276)*0.5**o
        ref[3,2,2,1]=(-0.238461142143)/(o+1.)+(0.566739682031)+(0.786422364797)*0.5**o
        ref[3,3,0,0]=(0.765700270847)/(o+1.)+(0.139729511444)+(0.483733340742)*0.5**o
        ref[3,3,0,1]=(-0.0164617510632)/(o+1.)+(0.358933652093)+(0.680316641067)*0.5**o
        ref[3,3,1,0]=(0.523753196917)/(o+1.)+(-0.273747043147)+(-0.993408262104)*0.5**o
        ref[3,3,1,1]=(-0.650074035674)/(o+1.)+(-0.402096198989)+(-0.748894192494)*0.5**o
        ref[3,3,2,0]=(-0.961050878591)/(o+1.)+(-0.0475226771118)+(-0.0390185724072)*0.5**o
        ref[3,3,2,1]=(-0.948399681695)/(o+1.)+(-0.377547226685)+(0.483849885635)*0.5**o
        ref[3,4,0,0]=(0.328198988132)/(o+1.)+(-0.00597924432023)+(0.0487172550208)*0.5**o
        ref[3,4,0,1]=(-0.922244339835)/(o+1.)+(0.652832314702)+(0.294964552743)*0.5**o
        ref[3,4,1,0]=(-0.993126331736)/(o+1.)+(0.338246377089)+(-0.774393234162)*0.5**o
        ref[3,4,1,1]=(0.273980415493)/(o+1.)+(-0.433436679324)+(-0.46041945879)*0.5**o
        ref[3,4,2,0]=(0.479839558444)/(o+1.)+(-0.0508925887883)+(0.0439333449053)*0.5**o
        ref[3,4,2,1]=(-0.531276902705)/(o+1.)+(-0.228478965855)+(-0.2108539974)*0.5**o
      else:
        s[0,0,0,0]=(-0.48893267678)*x[0]**o+(-0.276480532077)*x[0]+(0.774386441775)*x[1]**o+(-0.649500271111)*x[1]+(-0.0349902316269)*x[2]**o+(0.170017460369)*x[2]
        s[0,0,0,1]=(0.271529548298)*x[0]**o+(-0.318825369645)*x[0]+(0.304421796509)*x[1]**o+(0.445685002224)*x[1]+(0.0335660627736)*x[2]**o+(-0.498125294129)*x[2]
        s[0,0,1,0]=(0.217379546035)*x[0]**o+(0.0217166724721)*x[0]+(0.845692481133)*x[1]**o+(0.991062940209)*x[1]+(-0.504574922644)*x[2]**o+(-0.442660078459)*x[2]
        s[0,0,1,1]=(0.042921886647)*x[0]**o+(0.223486944232)*x[0]+(0.0494176704553)*x[1]**o+(0.96519307605)*x[1]+(-0.138757753118)*x[2]**o+(0.0467978128441)*x[2]
        s[0,0,2,0]=(-0.386813522991)*x[0]**o+(-0.460337705352)*x[0]+(0.868148506432)*x[1]**o+(-0.639578197438)*x[1]+(0.344241610414)*x[2]**o+(-0.340649859294)*x[2]
        s[0,0,2,1]=(0.270966465259)*x[0]**o+(-0.921862313681)*x[0]+(0.419658509548)*x[1]**o+(0.382347312575)*x[1]+(-0.458089861176)*x[2]**o+(-0.680587978014)*x[2]
        s[0,1,0,0]=(0.341922361346)*x[0]**o+(0.992862705739)*x[0]+(0.664064252002)*x[1]**o+(-0.976539711563)*x[1]+(-0.336860627327)*x[2]**o+(0.0351493378936)*x[2]
        s[0,1,0,1]=(0.232432790902)*x[0]**o+(0.661576839972)*x[0]+(-0.0847723144646)*x[1]**o+(0.676452509633)*x[1]+(-0.523713339087)*x[2]**o+(-0.944537647834)*x[2]
        s[0,1,1,0]=(-0.0195600321657)*x[0]**o+(0.548788073658)*x[0]+(0.732671243649)*x[1]**o+(-0.559596045627)*x[1]+(-0.821347880719)*x[2]**o+(0.523044807739)*x[2]
        s[0,1,1,1]=(0.440682443808)*x[0]**o+(0.489252739473)*x[0]+(-0.0286407968844)*x[1]**o+(0.389892189545)*x[1]+(0.802377372742)*x[2]**o+(-0.848679176767)*x[2]
        s[0,1,2,0]=(0.0308644963019)*x[0]**o+(-0.838818939488)*x[0]+(-0.923147289871)*x[1]**o+(-0.898497578423)*x[1]+(0.352358085546)*x[2]**o+(-0.0551880704817)*x[2]
        s[0,1,2,1]=(0.964064258383)*x[0]**o+(0.889207476385)*x[0]+(-0.446058209666)*x[1]**o+(-0.410646592871)*x[1]+(-0.143876047927)*x[2]**o+(-0.231175626697)*x[2]
        s[0,2,0,0]=(-0.378594793341)*x[0]**o+(0.45713093072)*x[0]+(-0.0164617155479)*x[1]**o+(-0.852502076132)*x[1]+(-0.445860749107)*x[2]**o+(0.718303558387)*x[2]
        s[0,2,0,1]=(-0.24119818585)*x[0]**o+(-0.480829504364)*x[0]+(0.0483190702959)*x[1]**o+(-0.943891784533)*x[1]+(0.822005676991)*x[2]**o+(0.462965609893)*x[2]
        s[0,2,1,0]=(0.260426222503)*x[0]**o+(0.801642256974)*x[0]+(-0.344027586708)*x[1]**o+(0.609754775626)*x[1]+(-0.614003273478)*x[2]**o+(0.254286708707)*x[2]
        s[0,2,1,1]=(-0.585208459519)*x[0]**o+(0.409783580697)*x[0]+(0.675131095845)*x[1]**o+(0.962899493666)*x[1]+(-0.555202089686)*x[2]**o+(0.55800653582)*x[2]
        s[0,2,2,0]=(0.926784535292)*x[0]**o+(0.366810833854)*x[0]+(0.280209292502)*x[1]**o+(0.455075782363)*x[1]+(0.315470862333)*x[2]**o+(0.804139765801)*x[2]
        s[0,2,2,1]=(-0.182330163829)*x[0]**o+(0.137160802887)*x[0]+(0.50089868344)*x[1]**o+(0.457091587964)*x[1]+(0.661074305613)*x[2]**o+(-0.774463051929)*x[2]
        s[0,3,0,0]=(0.251053015385)*x[0]**o+(-0.00252354996738)*x[0]+(0.219814741857)*x[1]**o+(-0.788960481779)*x[1]+(-0.628563574142)*x[2]**o+(0.238508420102)*x[2]
        s[0,3,0,1]=(-0.885181589797)*x[0]**o+(0.524674512592)*x[0]+(-0.137981402257)*x[1]**o+(-0.57775224131)*x[1]+(-0.151536711237)*x[2]**o+(0.147483336195)*x[2]
        s[0,3,1,0]=(0.464501690982)*x[0]**o+(0.477393661278)*x[0]+(-0.913546513036)*x[1]**o+(0.209928665471)*x[1]+(0.552522027491)*x[2]**o+(-0.200580731655)*x[2]
        s[0,3,1,1]=(0.117929836206)*x[0]**o+(-0.973701486459)*x[0]+(-0.876411386461)*x[1]**o+(-0.511161888778)*x[1]+(0.213923028781)*x[2]**o+(0.620921035428)*x[2]
        s[0,3,2,0]=(0.228255426058)*x[0]**o+(-0.135554118227)*x[0]+(0.607714808043)*x[1]**o+(0.522943351003)*x[1]+(-0.686514740068)*x[2]**o+(-0.238689322489)*x[2]
        s[0,3,2,1]=(-0.155365105782)*x[0]**o+(0.28572413448)*x[0]+(-0.452657577548)*x[1]**o+(-0.884598572632)*x[1]+(0.279309317605)*x[2]**o+(-0.523804815006)*x[2]
        s[0,4,0,0]=(-0.913401118579)*x[0]**o+(0.141999285441)*x[0]+(-0.0373272930617)*x[1]**o+(-0.907472677652)*x[1]+(0.648778084537)*x[2]**o+(0.905348717002)*x[2]
        s[0,4,0,1]=(0.272596166342)*x[0]**o+(0.523354359959)*x[0]+(-0.597238366724)*x[1]**o+(-0.57955847031)*x[1]+(0.961129977909)*x[2]**o+(-0.539446933115)*x[2]
        s[0,4,1,0]=(0.469445737741)*x[0]**o+(0.272602303288)*x[0]+(-0.168715269045)*x[1]**o+(-0.508483179891)*x[1]+(-0.0542883418628)*x[2]**o+(-0.549230857829)*x[2]
        s[0,4,1,1]=(-0.727336869594)*x[0]**o+(0.501720827606)*x[0]+(0.0112875501961)*x[1]**o+(-0.195351938275)*x[1]+(0.477540723998)*x[2]**o+(0.314035857499)*x[2]
        s[0,4,2,0]=(-0.826661708611)*x[0]**o+(-0.453403968062)*x[0]+(-0.424265582853)*x[1]**o+(0.850699490929)*x[1]+(0.869873737156)*x[2]**o+(0.512188794374)*x[2]
        s[0,4,2,1]=(0.788019130522)*x[0]**o+(-0.38976086271)*x[0]+(0.450211292079)*x[1]**o+(-0.845896268738)*x[1]+(-0.944295444764)*x[2]**o+(0.49424688571)*x[2]
        s[1,0,0,0]=(0.251841546653)*x[0]**o+(-0.262117658277)*x[0]+(-0.756023145868)*x[1]**o+(0.562918802897)*x[1]+(0.999897367962)*x[2]**o+(-0.438732703957)*x[2]
        s[1,0,0,1]=(0.0702543485705)*x[0]**o+(0.553020799687)*x[0]+(0.129510978458)*x[1]**o+(-0.287781735231)*x[1]+(0.0962686272734)*x[2]**o+(-0.153713487101)*x[2]
        s[1,0,1,0]=(0.0132145791001)*x[0]**o+(-0.747224756368)*x[0]+(0.533216099372)*x[1]**o+(0.651501143279)*x[1]+(0.275246126663)*x[2]**o+(0.564330219864)*x[2]
        s[1,0,1,1]=(0.00555685773193)*x[0]**o+(-0.912265903466)*x[0]+(-0.165714475359)*x[1]**o+(-0.29550893963)*x[1]+(-0.834017179013)*x[2]**o+(-0.66051475999)*x[2]
        s[1,0,2,0]=(-0.937628377428)*x[0]**o+(0.670046440332)*x[0]+(-0.836211833803)*x[1]**o+(0.294767128956)*x[1]+(-0.0831464801716)*x[2]**o+(0.173538164187)*x[2]
        s[1,0,2,1]=(0.0853414413422)*x[0]**o+(0.332113726085)*x[0]+(-0.00785676889602)*x[1]**o+(-0.787517944282)*x[1]+(0.640692804663)*x[2]**o+(0.16190882793)*x[2]
        s[1,1,0,0]=(-0.197717271419)*x[0]**o+(0.0875673864383)*x[0]+(0.303400667777)*x[1]**o+(0.561096499643)*x[1]+(-0.544100296942)*x[2]**o+(-0.236967889808)*x[2]
        s[1,1,0,1]=(0.378319093831)*x[0]**o+(-0.173085814456)*x[0]+(-0.520972068858)*x[1]**o+(-0.0887121930691)*x[1]+(0.0219775149259)*x[2]**o+(-0.902192785902)*x[2]
        s[1,1,1,0]=(-0.0852354427742)*x[0]**o+(-0.868052438949)*x[0]+(-0.962245566851)*x[1]**o+(-0.785773600296)*x[1]+(0.871469592508)*x[2]**o+(0.970335434685)*x[2]
        s[1,1,1,1]=(0.543079304077)*x[0]**o+(0.563798729263)*x[0]+(-0.0684074924309)*x[1]**o+(-0.0846345417372)*x[1]+(-0.0409611114801)*x[2]**o+(0.064168518437)*x[2]
        s[1,1,2,0]=(0.163126135652)*x[0]**o+(0.550910259467)*x[0]+(-0.838398060114)*x[1]**o+(0.20704094128)*x[1]+(-0.988745842471)*x[2]**o+(-0.238812411091)*x[2]
        s[1,1,2,1]=(-0.4311680318)*x[0]**o+(-0.689268700879)*x[0]+(-0.478591190853)*x[1]**o+(-0.935785118221)*x[1]+(0.681916835809)*x[2]**o+(0.316911720884)*x[2]
        s[1,2,0,0]=(-0.823741968282)*x[0]**o+(0.337348902553)*x[0]+(-0.773958391461)*x[1]**o+(0.319484407204)*x[1]+(-0.806392888327)*x[2]**o+(0.784342104197)*x[2]
        s[1,2,0,1]=(-0.271373691068)*x[0]**o+(-0.674977315639)*x[0]+(-0.73119288706)*x[1]**o+(-0.822294334638)*x[1]+(-0.348777213792)*x[2]**o+(0.539498527057)*x[2]
        s[1,2,1,0]=(-0.197804966703)*x[0]**o+(-0.000545898061773)*x[0]+(-0.477781076232)*x[1]**o+(-0.726309620722)*x[1]+(0.33565493623)*x[2]**o+(-0.472018740475)*x[2]
        s[1,2,1,1]=(0.985473777724)*x[0]**o+(-0.905489956976)*x[0]+(0.542738966677)*x[1]**o+(0.405770678949)*x[1]+(0.742199205539)*x[2]**o+(-0.834198356515)*x[2]
        s[1,2,2,0]=(0.0963375474866)*x[0]**o+(-0.702341975163)*x[0]+(-0.719805210146)*x[1]**o+(-0.573969955879)*x[1]+(0.849036302763)*x[2]**o+(-0.050325972969)*x[2]
        s[1,2,2,1]=(-0.000809557040863)*x[0]**o+(-0.298554227357)*x[0]+(-0.257571070168)*x[1]**o+(0.902842431485)*x[1]+(-0.565532231171)*x[2]**o+(-0.0268976084038)*x[2]
        s[1,3,0,0]=(-0.450224087014)*x[0]**o+(0.644911723614)*x[0]+(0.585908753859)*x[1]**o+(0.192087613433)*x[1]+(0.37463835574)*x[2]**o+(-0.858368552725)*x[2]
        s[1,3,0,1]=(-0.211716554169)*x[0]**o+(0.133373638633)*x[0]+(-0.477897006462)*x[1]**o+(-0.856170108137)*x[1]+(0.709445201671)*x[2]**o+(0.999843178308)*x[2]
        s[1,3,1,0]=(0.469992449976)*x[0]**o+(-0.547601912734)*x[0]+(-0.652917936853)*x[1]**o+(-0.973841854379)*x[1]+(0.346404346699)*x[2]**o+(0.960819995744)*x[2]
        s[1,3,1,1]=(-0.9202597392)*x[0]**o+(0.476802978382)*x[0]+(-0.106899807732)*x[1]**o+(-0.704577566959)*x[1]+(-0.331549015764)*x[2]**o+(0.936043692204)*x[2]
        s[1,3,2,0]=(0.724283786459)*x[0]**o+(0.822984194337)*x[0]+(0.705848034385)*x[1]**o+(-0.59858805565)*x[1]+(0.350635957833)*x[2]**o+(0.882807237348)*x[2]
        s[1,3,2,1]=(-0.582421422441)*x[0]**o+(0.809035080665)*x[0]+(-0.457851306764)*x[1]**o+(-0.993328352221)*x[1]+(-0.576322826333)*x[2]**o+(-0.749512812669)*x[2]
        s[1,4,0,0]=(0.268654181239)*x[0]**o+(-0.902343324019)*x[0]+(0.638580048206)*x[1]**o+(-0.832694904209)*x[1]+(-0.305201732282)*x[2]**o+(-0.867754959197)*x[2]
        s[1,4,0,1]=(-0.526400834988)*x[0]**o+(-0.40198050654)*x[0]+(0.337289899423)*x[1]**o+(0.179314676284)*x[1]+(-0.931514578551)*x[2]**o+(-0.449197337162)*x[2]
        s[1,4,1,0]=(0.912742445536)*x[0]**o+(-0.815137545811)*x[0]+(-0.38425815792)*x[1]**o+(-0.713804511416)*x[1]+(-0.0940645853762)*x[2]**o+(-0.184482970408)*x[2]
        s[1,4,1,1]=(0.0912352323191)*x[0]**o+(0.990389316513)*x[0]+(-0.0350410457429)*x[1]**o+(0.80277947226)*x[1]+(0.35771085432)*x[2]**o+(0.704976740376)*x[2]
        s[1,4,2,0]=(-0.212374984853)*x[0]**o+(0.115911507956)*x[0]+(0.743357884785)*x[1]**o+(-0.351179816129)*x[1]+(0.363783848165)*x[2]**o+(0.0667503905017)*x[2]
        s[1,4,2,1]=(-0.192346087265)*x[0]**o+(-0.875505251594)*x[0]+(0.450972468787)*x[1]**o+(0.0441627318307)*x[1]+(-0.955445583779)*x[2]**o+(0.561351576194)*x[2]
        s[2,0,0,0]=(0.646959412154)*x[0]**o+(0.49125803323)*x[0]+(-0.380573298578)*x[1]**o+(0.5795381042)*x[1]+(0.988167811553)*x[2]**o+(-0.848010513202)*x[2]
        s[2,0,0,1]=(-0.116490832795)*x[0]**o+(-0.413497866427)*x[0]+(0.015263239692)*x[1]**o+(0.995022978956)*x[1]+(-0.606550379921)*x[2]**o+(0.480728880447)*x[2]
        s[2,0,1,0]=(-0.656776412474)*x[0]**o+(0.105101642124)*x[0]+(-0.152043267549)*x[1]**o+(0.0397914854834)*x[1]+(0.907598907718)*x[2]**o+(0.55003346668)*x[2]
        s[2,0,1,1]=(-0.244997314762)*x[0]**o+(-0.180839249852)*x[0]+(0.737654816515)*x[1]**o+(-0.316505218599)*x[1]+(-0.467254068865)*x[2]**o+(0.642411232963)*x[2]
        s[2,0,2,0]=(0.427359531608)*x[0]**o+(0.0694309336993)*x[0]+(0.567819273901)*x[1]**o+(0.302951042003)*x[1]+(-0.652475445869)*x[2]**o+(-0.0604300175636)*x[2]
        s[2,0,2,1]=(-0.28054117321)*x[0]**o+(-0.506627838493)*x[0]+(0.830518434434)*x[1]**o+(-0.878165796713)*x[1]+(-0.924137956967)*x[2]**o+(-0.542977024006)*x[2]
        s[2,1,0,0]=(0.832323847638)*x[0]**o+(0.788814796981)*x[0]+(0.443003510301)*x[1]**o+(-0.692364079852)*x[1]+(-0.107112423673)*x[2]**o+(-0.42975252335)*x[2]
        s[2,1,0,1]=(0.791743274331)*x[0]**o+(0.506185826171)*x[0]+(0.880323084905)*x[1]**o+(0.875445981105)*x[1]+(-0.988158636575)*x[2]**o+(0.0106467506338)*x[2]
        s[2,1,1,0]=(-0.0716642151005)*x[0]**o+(0.184121338207)*x[0]+(0.563163918743)*x[1]**o+(-0.0476664508964)*x[1]+(-0.779735247644)*x[2]**o+(0.0532353296963)*x[2]
        s[2,1,1,1]=(-0.487936865731)*x[0]**o+(0.910882234056)*x[0]+(0.0698617623832)*x[1]**o+(0.949481044342)*x[1]+(0.605781066129)*x[2]**o+(-0.961538763982)*x[2]
        s[2,1,2,0]=(-0.865760152137)*x[0]**o+(0.98792005348)*x[0]+(-0.266208542741)*x[1]**o+(0.724207675193)*x[1]+(-0.431135420169)*x[2]**o+(-0.719606907727)*x[2]
        s[2,1,2,1]=(-0.550134186201)*x[0]**o+(0.377678575292)*x[0]+(0.681637629077)*x[1]**o+(0.880510886804)*x[1]+(0.185916696615)*x[2]**o+(-0.561352445373)*x[2]
        s[2,2,0,0]=(-0.0191940498302)*x[0]**o+(0.92499736077)*x[0]+(-0.457609595358)*x[1]**o+(0.295953391548)*x[1]+(-0.60039707059)*x[2]**o+(0.439541541552)*x[2]
        s[2,2,0,1]=(-0.593353550722)*x[0]**o+(-0.262307597953)*x[0]+(0.0707406713996)*x[1]**o+(0.510104333108)*x[1]+(-0.325033325591)*x[2]**o+(-0.978979243989)*x[2]
        s[2,2,1,0]=(0.758386313418)*x[0]**o+(-0.904849079285)*x[0]+(0.517417240328)*x[1]**o+(0.662794014603)*x[1]+(-0.9181308007)*x[2]**o+(0.545716036213)*x[2]
        s[2,2,1,1]=(-0.0562030350565)*x[0]**o+(-0.187267436123)*x[0]+(-0.0666932679699)*x[1]**o+(-0.192661397505)*x[1]+(0.683795366538)*x[2]**o+(0.554493956469)*x[2]
        s[2,2,2,0]=(0.632035307709)*x[0]**o+(-0.352426434378)*x[0]+(-0.572996524284)*x[1]**o+(-0.260948177348)*x[1]+(0.0781523669039)*x[2]**o+(-0.755607733499)*x[2]
        s[2,2,2,1]=(0.877117460076)*x[0]**o+(-0.368983710962)*x[0]+(0.339499499347)*x[1]**o+(-0.74687226645)*x[1]+(-0.0236621043423)*x[2]**o+(-0.524200850201)*x[2]
        s[2,3,0,0]=(0.868438479199)*x[0]**o+(-0.490384020524)*x[0]+(-0.348975370347)*x[1]**o+(0.0689779831612)*x[1]+(0.993171286973)*x[2]**o+(0.238168596199)*x[2]
        s[2,3,0,1]=(0.58688914355)*x[0]**o+(-0.299012943187)*x[0]+(-0.475183593899)*x[1]**o+(-0.209284023343)*x[1]+(-0.128788417577)*x[2]**o+(0.88040263674)*x[2]
        s[2,3,1,0]=(-0.491821145506)*x[0]**o+(0.28222061488)*x[0]+(-0.432027482169)*x[1]**o+(0.0843542963065)*x[1]+(0.179895461516)*x[2]**o+(0.26281058218)*x[2]
        s[2,3,1,1]=(-0.68076808014)*x[0]**o+(0.966337829009)*x[0]+(-0.967714227169)*x[1]**o+(-0.511464770717)*x[1]+(-0.00173666009442)*x[2]**o+(0.418327208497)*x[2]
        s[2,3,2,0]=(-0.981204168827)*x[0]**o+(-0.481006793801)*x[0]+(0.223651529206)*x[1]**o+(0.0692948204072)*x[1]+(0.833168651519)*x[2]**o+(-0.747962032571)*x[2]
        s[2,3,2,1]=(-0.979594675181)*x[0]**o+(-0.793815945451)*x[0]+(-0.972489753584)*x[1]**o+(0.193666863184)*x[1]+(0.813999185145)*x[2]**o+(0.976350643493)*x[2]
        s[2,4,0,0]=(0.160095467562)*x[0]**o+(-0.0596430245423)*x[0]+(0.421111483086)*x[1]**o+(-0.50368497144)*x[1]+(0.295956709432)*x[2]**o+(-0.547225692085)*x[2]
        s[2,4,0,1]=(0.121233419707)*x[0]**o+(0.0796884300301)*x[0]+(0.63360038458)*x[1]**o+(-0.285793125165)*x[1]+(-0.322665608493)*x[2]**o+(0.144053221653)*x[2]
        s[2,4,1,0]=(-0.327390540088)*x[0]**o+(-0.158990478104)*x[0]+(0.0417451752863)*x[1]**o+(0.0577661706274)*x[1]+(0.608054602266)*x[2]**o+(0.876076509191)*x[2]
        s[2,4,1,1]=(-0.336090262681)*x[0]**o+(0.460831366224)*x[0]+(0.126375418917)*x[1]**o+(-0.0524791065196)*x[1]+(0.778197798934)*x[2]**o+(-0.938503586105)*x[2]
        s[2,4,2,0]=(-0.157686815732)*x[0]**o+(-0.240464045969)*x[0]+(-0.610957847036)*x[1]**o+(-0.591812526831)*x[1]+(0.683562885502)*x[2]**o+(-0.575699253807)*x[2]
        s[2,4,2,1]=(-0.591186612156)*x[0]**o+(0.952366931656)*x[0]+(0.563360859838)*x[1]**o+(-0.540973061762)*x[1]+(0.95638308424)*x[2]**o+(0.0810587285575)*x[2]
        s[3,0,0,0]=(-0.777152282839)*x[0]**o+(0.0462962198971)*x[0]+(-0.989861157039)*x[1]**o+(-0.895147454926)*x[1]+(0.496304718705)*x[2]**o+(0.724664844743)*x[2]
        s[3,0,0,1]=(-0.152045574171)*x[0]**o+(0.557429298004)*x[0]+(0.172100777056)*x[1]**o+(-0.762302216782)*x[1]+(0.0107560041439)*x[2]**o+(0.0984433240396)*x[2]
        s[3,0,1,0]=(-0.300572515767)*x[0]**o+(-0.776096822514)*x[0]+(-0.304519070945)*x[1]**o+(0.222039297615)*x[1]+(0.42051246201)*x[2]**o+(0.875929404536)*x[2]
        s[3,0,1,1]=(-0.832809662655)*x[0]**o+(-0.775869677688)*x[0]+(-0.548130481677)*x[1]**o+(-0.863910117253)*x[1]+(-0.708195298544)*x[2]**o+(-0.895809178957)*x[2]
        s[3,0,2,0]=(0.417580286092)*x[0]**o+(-0.320463746468)*x[0]+(-0.780326066905)*x[1]**o+(-0.397006341179)*x[1]+(0.575088222676)*x[2]**o+(0.621841867422)*x[2]
        s[3,0,2,1]=(0.266423263811)*x[0]**o+(0.782081751363)*x[0]+(-0.395723332933)*x[1]**o+(-0.474817183348)*x[1]+(0.483870682109)*x[2]**o+(-0.416708790303)*x[2]
        s[3,1,0,0]=(0.364336403912)*x[0]**o+(0.4746091186)*x[0]+(0.419745216537)*x[1]**o+(0.906218032598)*x[1]+(-0.561381612354)*x[2]**o+(0.252977998753)*x[2]
        s[3,1,0,1]=(-0.479065041103)*x[0]**o+(-0.727722678216)*x[0]+(0.298427986755)*x[1]**o+(0.391426108723)*x[1]+(-0.18371001783)*x[2]**o+(-0.298396916251)*x[2]
        s[3,1,1,0]=(-0.456720853948)*x[0]**o+(-0.137192756988)*x[0]+(0.0387577082772)*x[1]**o+(0.0342816278)*x[1]+(0.287825957753)*x[2]**o+(-0.416093408739)*x[2]
        s[3,1,1,1]=(-0.157773606009)*x[0]**o+(0.229634478596)*x[0]+(-0.100573203257)*x[1]**o+(-0.791674554355)*x[1]+(0.633139739302)*x[2]**o+(0.53643300369)*x[2]
        s[3,1,2,0]=(-0.977094714538)*x[0]**o+(-0.980511324919)*x[0]+(-0.996777041114)*x[1]**o+(-0.135014608241)*x[1]+(0.171228488823)*x[2]**o+(-0.826149942234)*x[2]
        s[3,1,2,1]=(0.40095895971)*x[0]**o+(-0.16005681048)*x[0]+(-0.110948052824)*x[1]**o+(-0.0657272460506)*x[1]+(-0.921326915659)*x[2]**o+(0.498246475577)*x[2]
        s[3,2,0,0]=(-0.521393633448)*x[0]**o+(-0.442461853948)*x[0]+(0.882466157567)*x[1]**o+(-0.318640196818)*x[1]+(-0.454978258143)*x[2]**o+(-0.146713474073)*x[2]
        s[3,2,0,1]=(-0.60584665741)*x[0]**o+(0.186330030473)*x[0]+(0.800412347134)*x[1]**o+(0.632355962829)*x[1]+(-0.577489166576)*x[2]**o+(0.288233513083)*x[2]
        s[3,2,1,0]=(0.645974743024)*x[0]**o+(-0.463548979592)*x[0]+(0.454659333323)*x[1]**o+(-0.147068470329)*x[1]+(0.621612597429)*x[2]**o+(0.309226445731)*x[2]
        s[3,2,1,1]=(0.142559408818)*x[0]**o+(0.568796102102)*x[0]+(-0.957487822488)*x[1]**o+(0.199695598234)*x[1]+(0.437309500275)*x[2]**o+(-0.608539956732)*x[2]
        s[3,2,2,0]=(0.814258898685)*x[0]**o+(-0.0657073512095)*x[0]+(-0.675703123952)*x[1]**o+(-0.826924419864)*x[1]+(-0.202583280241)*x[2]**o+(0.556708057646)*x[2]
        s[3,2,2,1]=(0.420183604639)*x[0]**o+(0.234120268268)*x[0]+(-0.690228330954)*x[1]**o+(0.0742691494912)*x[1]+(-0.542445114625)*x[2]**o+(-0.671138939877)*x[2]
        s[3,3,0,0]=(-0.327300600039)*x[0]**o+(-0.883120008711)*x[0]+(-0.588356952373)*x[1]**o+(0.964702848682)*x[1]+(0.0983852055922)*x[2]**o+(-0.517082121951)*x[2]
        s[3,3,0,1]=(0.808391338918)*x[0]**o+(0.413826885405)*x[0]+(-0.781923414495)*x[1]**o+(0.941400794201)*x[1]+(0.566387362748)*x[2]**o+(-0.361926999623)*x[2]
        s[3,3,1,0]=(0.0408315227595)*x[0]**o+(0.00354329999935)*x[0]+(0.29340210554)*x[1]**o+(-0.129271597771)*x[1]+(0.85418604558)*x[2]**o+(0.972437132596)*x[2]
        s[3,3,1,1]=(0.488110633719)*x[0]**o+(0.442243578548)*x[0]+(0.491915283822)*x[1]**o+(0.900808596947)*x[1]+(0.872340191832)*x[2]**o+(-0.821189941972)*x[2]
        s[3,3,2,0]=(0.698445367492)*x[0]**o+(0.235000428999)*x[0]+(0.0231665862971)*x[1]**o+(0.175887750008)*x[1]+(0.285424647457)*x[2]**o+(-0.0889092865387)*x[2]
        s[3,3,2,1]=(-0.619448487664)*x[0]**o+(-0.25845368795)*x[0]+(0.514713543216)*x[1]**o+(0.933603723018)*x[1]+(0.965553741692)*x[2]**o+(0.775325889348)*x[2]
        s[3,4,0,0]=(0.653712053876)*x[0]**o+(0.397899343408)*x[0]+(-0.980528220216)*x[1]**o+(0.192768638146)*x[1]+(-0.131961987222)*x[2]**o+(0.294990291948)*x[2]
        s[3,4,0,1]=(0.519872654154)*x[0]**o+(-0.523923706123)*x[0]+(-0.366056337857)*x[1]**o+(0.587286575656)*x[1]+(0.67858517405)*x[2]**o+(-0.528581733781)*x[2]
        s[3,4,1,0]=(-0.561163638499)*x[0]**o+(0.834141187099)*x[0]+(0.101344553848)*x[1]**o+(-0.630659721656)*x[1]+(0.177616824366)*x[2]**o+(-0.702449367192)*x[2]
        s[3,4,1,1]=(0.0594382754162)*x[0]**o+(-0.274398458709)*x[0]+(0.423874245449)*x[1]**o+(0.644709194846)*x[1]+(0.742592384867)*x[2]**o+(-0.00411951621579)*x[2]
        s[3,4,2,0]=(0.0680763742436)*x[0]**o+(0.718014295363)*x[0]+(0.10166016369)*x[1]**o+(-0.308308565274)*x[1]+(0.529471669819)*x[2]**o+(0.371855430993)*x[2]
        s[3,4,2,1]=(0.154883537954)*x[0]**o+(-0.727616949065)*x[0]+(0.685560277601)*x[1]**o+(0.0846298021044)*x[1]+(0.284159382541)*x[2]**o+(0.869297635219)*x[2]
        ref[0,0,0,0]=(0.739396210148)/(o+1.)+(-0.377981671409)+(-0.48893267678)*0.5**o
        ref[0,0,0,1]=(0.337987859283)/(o+1.)+(-0.185632830775)+(0.271529548298)*0.5**o
        ref[0,0,1,0]=(0.341117558489)/(o+1.)+(0.285059767111)+(0.217379546035)*0.5**o
        ref[0,0,1,1]=(-0.0893400826624)/(o+1.)+(0.617738916563)+(0.042921886647)*0.5**o
        ref[0,0,2,0]=(1.21239011685)/(o+1.)+(-0.720282881042)+(-0.386813522991)*0.5**o
        ref[0,0,2,1]=(-0.0384313516284)/(o+1.)+(-0.61005148956)+(0.270966465259)*0.5**o
        ref[0,1,0,0]=(0.327203624676)/(o+1.)+(0.0257361660344)+(0.341922361346)*0.5**o
        ref[0,1,0,1]=(-0.608485653551)/(o+1.)+(0.196745850885)+(0.232432790902)*0.5**o
        ref[0,1,1,0]=(-0.0886766370694)/(o+1.)+(0.256118417885)+(-0.0195600321657)*0.5**o
        ref[0,1,1,1]=(0.773736575857)/(o+1.)+(0.015232876126)+(0.440682443808)*0.5**o
        ref[0,1,2,0]=(-0.570789204325)/(o+1.)+(-0.896252294196)+(0.0308644963019)*0.5**o
        ref[0,1,2,1]=(-0.589934257593)/(o+1.)+(0.123692628409)+(0.964064258383)*0.5**o
        ref[0,2,0,0]=(-0.462322464655)/(o+1.)+(0.161466206488)+(-0.378594793341)*0.5**o
        ref[0,2,0,1]=(0.870324747287)/(o+1.)+(-0.480877839502)+(-0.24119818585)*0.5**o
        ref[0,2,1,0]=(-0.958030860186)/(o+1.)+(0.832841870654)+(0.260426222503)*0.5**o
        ref[0,2,1,1]=(0.119929006158)/(o+1.)+(0.965344805092)+(-0.585208459519)*0.5**o
        ref[0,2,2,0]=(0.595680154835)/(o+1.)+(0.813013191009)+(0.926784535292)*0.5**o
        ref[0,2,2,1]=(1.16197298905)/(o+1.)+(-0.0901053305394)+(-0.182330163829)*0.5**o
        ref[0,3,0,0]=(-0.408748832285)/(o+1.)+(-0.276487805822)+(0.251053015385)*0.5**o
        ref[0,3,0,1]=(-0.289518113494)/(o+1.)+(0.0472028037384)+(-0.885181589797)*0.5**o
        ref[0,3,1,0]=(-0.361024485545)/(o+1.)+(0.243370797547)+(0.464501690982)*0.5**o
        ref[0,3,1,1]=(-0.66248835768)/(o+1.)+(-0.431971169905)+(0.117929836206)*0.5**o
        ref[0,3,2,0]=(-0.0787999320256)/(o+1.)+(0.0743499551435)+(0.228255426058)*0.5**o
        ref[0,3,2,1]=(-0.173348259943)/(o+1.)+(-0.561339626579)+(-0.155365105782)*0.5**o
        ref[0,4,0,0]=(0.611450791476)/(o+1.)+(0.0699376623952)+(-0.913401118579)*0.5**o
        ref[0,4,0,1]=(0.363891611185)/(o+1.)+(-0.297825521733)+(0.272596166342)*0.5**o
        ref[0,4,1,0]=(-0.223003610908)/(o+1.)+(-0.392555867216)+(0.469445737741)*0.5**o
        ref[0,4,1,1]=(0.488828274194)/(o+1.)+(0.310202373415)+(-0.727336869594)*0.5**o
        ref[0,4,2,0]=(0.445608154304)/(o+1.)+(0.454742158621)+(-0.826661708611)*0.5**o
        ref[0,4,2,1]=(-0.494084152685)/(o+1.)+(-0.370705122869)+(0.788019130522)*0.5**o
        ref[1,0,0,0]=(0.243874222094)/(o+1.)+(-0.0689657796687)+(0.251841546653)*0.5**o
        ref[1,0,0,1]=(0.225779605731)/(o+1.)+(0.0557627886771)+(0.0702543485705)*0.5**o
        ref[1,0,1,0]=(0.808462226036)/(o+1.)+(0.234303303388)+(0.0132145791001)*0.5**o
        ref[1,0,1,1]=(-0.999731654372)/(o+1.)+(-0.934144801543)+(0.00555685773193)*0.5**o
        ref[1,0,2,0]=(-0.919358313974)/(o+1.)+(0.569175866738)+(-0.937628377428)*0.5**o
        ref[1,0,2,1]=(0.632836035767)/(o+1.)+(-0.146747695134)+(0.0853414413422)*0.5**o
        ref[1,1,0,0]=(-0.240699629165)/(o+1.)+(0.205847998136)+(-0.197717271419)*0.5**o
        ref[1,1,0,1]=(-0.498994553932)/(o+1.)+(-0.581995396714)+(0.378319093831)*0.5**o
        ref[1,1,1,0]=(-0.0907759743431)/(o+1.)+(-0.34174530228)+(-0.0852354427742)*0.5**o
        ref[1,1,1,1]=(-0.109368603911)/(o+1.)+(0.271666352981)+(0.543079304077)*0.5**o
        ref[1,1,2,0]=(-1.82714390258)/(o+1.)+(0.259569394828)+(0.163126135652)*0.5**o
        ref[1,1,2,1]=(0.203325644955)/(o+1.)+(-0.654071049108)+(-0.4311680318)*0.5**o
        ref[1,2,0,0]=(-1.58035127979)/(o+1.)+(0.720587706977)+(-0.823741968282)*0.5**o
        ref[1,2,0,1]=(-1.07997010085)/(o+1.)+(-0.47888656161)+(-0.271373691068)*0.5**o
        ref[1,2,1,0]=(-0.142126140002)/(o+1.)+(-0.599437129629)+(-0.197804966703)*0.5**o
        ref[1,2,1,1]=(1.28493817222)/(o+1.)+(-0.666958817271)+(0.985473777724)*0.5**o
        ref[1,2,2,0]=(0.129231092617)/(o+1.)+(-0.663318952006)+(0.0963375474866)*0.5**o
        ref[1,2,2,1]=(-0.823103301339)/(o+1.)+(0.288695297862)+(-0.000809557040863)*0.5**o
        ref[1,3,0,0]=(0.960547109599)/(o+1.)+(-0.010684607839)+(-0.450224087014)*0.5**o
        ref[1,3,0,1]=(0.231548195209)/(o+1.)+(0.138523354402)+(-0.211716554169)*0.5**o
        ref[1,3,1,0]=(-0.306513590154)/(o+1.)+(-0.280311885684)+(0.469992449976)*0.5**o
        ref[1,3,1,1]=(-0.438448823496)/(o+1.)+(0.354134551813)+(-0.9202597392)*0.5**o
        ref[1,3,2,0]=(1.05648399222)/(o+1.)+(0.553601688018)+(0.724283786459)*0.5**o
        ref[1,3,2,1]=(-1.0341741331)/(o+1.)+(-0.466903042112)+(-0.582421422441)*0.5**o
        ref[1,4,0,0]=(0.333378315924)/(o+1.)+(-1.30139659371)+(0.268654181239)*0.5**o
        ref[1,4,0,1]=(-0.594224679128)/(o+1.)+(-0.335931583709)+(-0.526400834988)*0.5**o
        ref[1,4,1,0]=(-0.478322743296)/(o+1.)+(-0.856712513818)+(0.912742445536)*0.5**o
        ref[1,4,1,1]=(0.322669808577)/(o+1.)+(1.24907276457)+(0.0912352323191)*0.5**o
        ref[1,4,2,0]=(1.10714173295)/(o+1.)+(-0.0842589588357)+(-0.212374984853)*0.5**o
        ref[1,4,2,1]=(-0.504473114993)/(o+1.)+(-0.134995471785)+(-0.192346087265)*0.5**o
        ref[2,0,0,0]=(0.607594512975)/(o+1.)+(0.111392812114)+(0.646959412154)*0.5**o
        ref[2,0,0,1]=(-0.59128714023)/(o+1.)+(0.531126996488)+(-0.116490832795)*0.5**o
        ref[2,0,1,0]=(0.755555640169)/(o+1.)+(0.347463297144)+(-0.656776412474)*0.5**o
        ref[2,0,1,1]=(0.27040074765)/(o+1.)+(0.0725333822557)+(-0.244997314762)*0.5**o
        ref[2,0,2,0]=(-0.0846561719678)/(o+1.)+(0.155975979069)+(0.427359531608)*0.5**o
        ref[2,0,2,1]=(-0.0936195225336)/(o+1.)+(-0.963885329606)+(-0.28054117321)*0.5**o
        ref[2,1,0,0]=(0.335891086628)/(o+1.)+(-0.166650903111)+(0.832323847638)*0.5**o
        ref[2,1,0,1]=(-0.10783555167)/(o+1.)+(0.696139278955)+(0.791743274331)*0.5**o
        ref[2,1,1,0]=(-0.2165713289)/(o+1.)+(0.0948451085032)+(-0.0716642151005)*0.5**o
        ref[2,1,1,1]=(0.675642828512)/(o+1.)+(0.449412257208)+(-0.487936865731)*0.5**o
        ref[2,1,2,0]=(-0.69734396291)/(o+1.)+(0.496260410473)+(-0.865760152137)*0.5**o
        ref[2,1,2,1]=(0.867554325692)/(o+1.)+(0.348418508362)+(-0.550134186201)*0.5**o
        ref[2,2,0,0]=(-1.05800666595)/(o+1.)+(0.830246146935)+(-0.0191940498302)*0.5**o
        ref[2,2,0,1]=(-0.254292654192)/(o+1.)+(-0.365591254417)+(-0.593353550722)*0.5**o
        ref[2,2,1,0]=(-0.400713560372)/(o+1.)+(0.151830485765)+(0.758386313418)*0.5**o
        ref[2,2,1,1]=(0.617102098568)/(o+1.)+(0.0872825614209)+(-0.0562030350565)*0.5**o
        ref[2,2,2,0]=(-0.49484415738)/(o+1.)+(-0.684491172613)+(0.632035307709)*0.5**o
        ref[2,2,2,1]=(0.315837395005)/(o+1.)+(-0.820028413806)+(0.877117460076)*0.5**o
        ref[2,3,0,0]=(0.644195916625)/(o+1.)+(-0.0916187205818)+(0.868438479199)*0.5**o
        ref[2,3,0,1]=(-0.603972011476)/(o+1.)+(0.186052835105)+(0.58688914355)*0.5**o
        ref[2,3,1,0]=(-0.252132020652)/(o+1.)+(0.314692746683)+(-0.491821145506)*0.5**o
        ref[2,3,1,1]=(-0.969450887263)/(o+1.)+(0.436600133394)+(-0.68076808014)*0.5**o
        ref[2,3,2,0]=(1.05682018072)/(o+1.)+(-0.579837002982)+(-0.981204168827)*0.5**o
        ref[2,3,2,1]=(-0.15849056844)/(o+1.)+(0.188100780613)+(-0.979594675181)*0.5**o
        ref[2,4,0,0]=(0.717068192518)/(o+1.)+(-0.555276844034)+(0.160095467562)*0.5**o
        ref[2,4,0,1]=(0.310934776087)/(o+1.)+(-0.031025736741)+(0.121233419707)*0.5**o
        ref[2,4,1,0]=(0.649799777552)/(o+1.)+(0.387426100857)+(-0.327390540088)*0.5**o
        ref[2,4,1,1]=(0.904573217851)/(o+1.)+(-0.2650756632)+(-0.336090262681)*0.5**o
        ref[2,4,2,0]=(0.0726050384664)/(o+1.)+(-0.703987913303)+(-0.157686815732)*0.5**o
        ref[2,4,2,1]=(1.51974394408)/(o+1.)+(0.246226299226)+(-0.591186612156)*0.5**o
        ref[3,0,0,0]=(-0.493556438334)/(o+1.)+(-0.0620931951433)+(-0.777152282839)*0.5**o
        ref[3,0,0,1]=(0.1828567812)/(o+1.)+(-0.0532147973691)+(-0.152045574171)*0.5**o
        ref[3,0,1,0]=(0.115993391065)/(o+1.)+(0.160935939818)+(-0.300572515767)*0.5**o
        ref[3,0,1,1]=(-1.25632578022)/(o+1.)+(-1.26779448695)+(-0.832809662655)*0.5**o
        ref[3,0,2,0]=(-0.205237844229)/(o+1.)+(-0.0478141101127)+(0.417580286092)*0.5**o
        ref[3,0,2,1]=(0.088147349176)/(o+1.)+(-0.0547221111442)+(0.266423263811)*0.5**o
        ref[3,1,0,0]=(-0.141636395817)/(o+1.)+(0.816902574975)+(0.364336403912)*0.5**o
        ref[3,1,0,1]=(0.114717968925)/(o+1.)+(-0.317346742872)+(-0.479065041103)*0.5**o
        ref[3,1,1,0]=(0.32658366603)/(o+1.)+(-0.259502268963)+(-0.456720853948)*0.5**o
        ref[3,1,1,1]=(0.532566536046)/(o+1.)+(-0.0128035360346)+(-0.157773606009)*0.5**o
        ref[3,1,2,0]=(-0.825548552291)/(o+1.)+(-0.970837937697)+(-0.977094714538)*0.5**o
        ref[3,1,2,1]=(-1.03227496848)/(o+1.)+(0.136231209523)+(0.40095895971)*0.5**o
        ref[3,2,0,0]=(0.427487899424)/(o+1.)+(-0.453907762419)+(-0.521393633448)*0.5**o
        ref[3,2,0,1]=(0.222923180558)/(o+1.)+(0.553459753192)+(-0.60584665741)*0.5**o
        ref[3,2,1,0]=(1.07627193075)/(o+1.)+(-0.150695502095)+(0.645974743024)*0.5**o
        ref[3,2,1,1]=(-0.520178322214)/(o+1.)+(0.0799758718026)+(0.142559408818)*0.5**o
        ref[3,2,2,0]=(-0.878286404193)/(o+1.)+(-0.167961856714)+(0.814258898685)*0.5**o
        ref[3,2,2,1]=(-1.23267344558)/(o+1.)+(-0.181374761059)+(0.420183604639)*0.5**o
        ref[3,3,0,0]=(-0.489971746781)/(o+1.)+(-0.21774964099)+(-0.327300600039)*0.5**o
        ref[3,3,0,1]=(-0.215536051747)/(o+1.)+(0.496650339992)+(0.808391338918)*0.5**o
        ref[3,3,1,0]=(1.14758815112)/(o+1.)+(0.423354417412)+(0.0408315227595)*0.5**o
        ref[3,3,1,1]=(1.36425547565)/(o+1.)+(0.260931116761)+(0.488110633719)*0.5**o
        ref[3,3,2,0]=(0.308591233754)/(o+1.)+(0.160989446234)+(0.698445367492)*0.5**o
        ref[3,3,2,1]=(1.48026728491)/(o+1.)+(0.725237962208)+(-0.619448487664)*0.5**o
        ref[3,4,0,0]=(-1.11249020744)/(o+1.)+(0.442829136751)+(0.653712053876)*0.5**o
        ref[3,4,0,1]=(0.312528836193)/(o+1.)+(-0.232609432124)+(0.519872654154)*0.5**o
        ref[3,4,1,0]=(0.278961378214)/(o+1.)+(-0.249483950874)+(-0.561163638499)*0.5**o
        ref[3,4,1,1]=(1.16646663032)/(o+1.)+(0.183095609961)+(0.0594382754162)*0.5**o
        ref[3,4,2,0]=(0.631131833509)/(o+1.)+(0.390780580541)+(0.0680763742436)*0.5**o
        ref[3,4,2,1]=(0.969719660142)/(o+1.)+(0.113155244129)+(0.154883537954)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactZero_fromSymbol_Solution_rank0(self):
      """
      tests integral of rank 0 Symbol on the FunctionOnContactZero

      assumptions: Solution(self.domain) exists
                   self.domain supports integral on FunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      w=Solution(self.domain)
      x=w.getX()
      arg=Symbol(shape=())
      s=Data(0,(),w)
      if dim==2:
        s=(0.551831061348)*x[0]**o+(-0.711823790733)*x[0]+(0.766098862021)*x[1]**o+(0.348439108965)*x[1]
        ref=(0.766098862021)/(o+1.)+(-0.181692340884)+(0.551831061348)*0.5**o
      else:
        s=(-0.534750207803)*x[0]**o+(0.67975160623)*x[0]+(0.747517712959)*x[1]**o+(0.392190678153)*x[1]+(-0.586558763208)*x[2]**o+(0.67325251011)*x[2]
        ref=(0.160958949751)/(o+1.)+(0.872597397246)+(-0.534750207803)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactZero_fromSymbol_Solution_rank1(self):
      """
      tests integral of rank 1 Symbol on the FunctionOnContactZero

      assumptions: Solution(self.domain) exists
                   self.domain supports integral on FunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      w=Solution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(2,))
      s=Data(0,(2,),w)
      ref=numpy.zeros((2,),numpy.float_)
      if dim==2:
        s[0]=(0.824614999579)*x[0]**o+(-0.92501072999)*x[0]+(-0.164852435473)*x[1]**o+(0.625787035398)*x[1]
        s[1]=(0.933187313904)*x[0]**o+(-0.76410941839)*x[0]+(-0.924559271586)*x[1]**o+(-0.951797707375)*x[1]
        ref[0]=(-0.164852435473)/(o+1.)+(-0.149611847296)+(0.824614999579)*0.5**o
        ref[1]=(-0.924559271586)/(o+1.)+(-0.857953562883)+(0.933187313904)*0.5**o
      else:
        s[0]=(-0.73247014084)*x[0]**o+(-0.447784692568)*x[0]+(-0.206145797624)*x[1]**o+(0.800549366557)*x[1]+(0.0980004782263)*x[2]**o+(0.68684151518)*x[2]
        s[1]=(0.734011170766)*x[0]**o+(-0.718028398303)*x[0]+(0.768940123369)*x[1]**o+(0.410156990187)*x[1]+(0.6562835776)*x[2]**o+(-0.899108897402)*x[2]
        ref[0]=(-0.108145319398)/(o+1.)+(0.519803094585)+(-0.73247014084)*0.5**o
        ref[1]=(1.42522370097)/(o+1.)+(-0.603490152759)+(0.734011170766)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactZero_fromSymbol_Solution_rank2(self):
      """
      tests integral of rank 2 Symbol on the FunctionOnContactZero

      assumptions: Solution(self.domain) exists
                   self.domain supports integral on FunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      w=Solution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5))
      s=Data(0,(4, 5),w)
      ref=numpy.zeros((4, 5),numpy.float_)
      if dim==2:
        s[0,0]=(0.860004136579)*x[0]**o+(-0.117366140441)*x[0]+(0.0876424200904)*x[1]**o+(0.0578427219547)*x[1]
        s[0,1]=(-0.934262599682)*x[0]**o+(-0.834391940774)*x[0]+(-0.37703046074)*x[1]**o+(0.571779357388)*x[1]
        s[0,2]=(-0.524417646775)*x[0]**o+(-0.562788330473)*x[0]+(-0.454146889642)*x[1]**o+(0.382147150671)*x[1]
        s[0,3]=(0.53752054491)*x[0]**o+(0.52118522583)*x[0]+(0.196117385249)*x[1]**o+(0.968096844135)*x[1]
        s[0,4]=(-0.346245303793)*x[0]**o+(0.169183126761)*x[0]+(0.35430553025)*x[1]**o+(-0.1953526461)*x[1]
        s[1,0]=(0.110897459128)*x[0]**o+(-0.189310140801)*x[0]+(-0.503153929692)*x[1]**o+(-0.776512889143)*x[1]
        s[1,1]=(0.0773234920212)*x[0]**o+(-0.323298685849)*x[0]+(0.956652842652)*x[1]**o+(-0.834584362886)*x[1]
        s[1,2]=(0.725330064304)*x[0]**o+(-0.761117907603)*x[0]+(-0.161152622185)*x[1]**o+(0.633185715895)*x[1]
        s[1,3]=(0.399871656297)*x[0]**o+(-0.831922720579)*x[0]+(0.125957093778)*x[1]**o+(0.909312599897)*x[1]
        s[1,4]=(0.450333632811)*x[0]**o+(-0.914467685845)*x[0]+(-0.71553040823)*x[1]**o+(-0.806957497854)*x[1]
        s[2,0]=(-0.596111663177)*x[0]**o+(-0.896500016447)*x[0]+(-0.175137342743)*x[1]**o+(-0.519618663869)*x[1]
        s[2,1]=(0.219789961005)*x[0]**o+(0.394778437788)*x[0]+(0.516359075868)*x[1]**o+(0.158719123393)*x[1]
        s[2,2]=(-0.202978715302)*x[0]**o+(0.326528455071)*x[0]+(-0.475802171068)*x[1]**o+(-0.470892970453)*x[1]
        s[2,3]=(0.34431665114)*x[0]**o+(0.847623303071)*x[0]+(-0.300788565801)*x[1]**o+(0.623946324835)*x[1]
        s[2,4]=(-0.482716534989)*x[0]**o+(0.610848202802)*x[0]+(-0.618220314271)*x[1]**o+(-0.270417044737)*x[1]
        s[3,0]=(0.323084711653)*x[0]**o+(0.870297617987)*x[0]+(-0.817231621484)*x[1]**o+(0.599137155405)*x[1]
        s[3,1]=(-0.464413485909)*x[0]**o+(-0.948296858226)*x[0]+(-0.872776322969)*x[1]**o+(0.0058301771792)*x[1]
        s[3,2]=(-0.920685763376)*x[0]**o+(-0.429364041758)*x[0]+(0.211630014364)*x[1]**o+(-0.311548404071)*x[1]
        s[3,3]=(-0.175827412744)*x[0]**o+(-0.687378086381)*x[0]+(0.265852072745)*x[1]**o+(0.799115907363)*x[1]
        s[3,4]=(-0.99517403847)*x[0]**o+(0.746397561862)*x[0]+(-0.747340816778)*x[1]**o+(-0.57474910244)*x[1]
        ref[0,0]=(0.0876424200904)/(o+1.)+(-0.0297617092432)+(0.860004136579)*0.5**o
        ref[0,1]=(-0.37703046074)/(o+1.)+(-0.131306291693)+(-0.934262599682)*0.5**o
        ref[0,2]=(-0.454146889642)/(o+1.)+(-0.0903205899012)+(-0.524417646775)*0.5**o
        ref[0,3]=(0.196117385249)/(o+1.)+(0.744641034982)+(0.53752054491)*0.5**o
        ref[0,4]=(0.35430553025)/(o+1.)+(-0.0130847596698)+(-0.346245303793)*0.5**o
        ref[1,0]=(-0.503153929692)/(o+1.)+(-0.482911514972)+(0.110897459128)*0.5**o
        ref[1,1]=(0.956652842652)/(o+1.)+(-0.578941524367)+(0.0773234920212)*0.5**o
        ref[1,2]=(-0.161152622185)/(o+1.)+(-0.063966095854)+(0.725330064304)*0.5**o
        ref[1,3]=(0.125957093778)/(o+1.)+(0.0386949396593)+(0.399871656297)*0.5**o
        ref[1,4]=(-0.71553040823)/(o+1.)+(-0.860712591849)+(0.450333632811)*0.5**o
        ref[2,0]=(-0.175137342743)/(o+1.)+(-0.708059340158)+(-0.596111663177)*0.5**o
        ref[2,1]=(0.516359075868)/(o+1.)+(0.27674878059)+(0.219789961005)*0.5**o
        ref[2,2]=(-0.475802171068)/(o+1.)+(-0.072182257691)+(-0.202978715302)*0.5**o
        ref[2,3]=(-0.300788565801)/(o+1.)+(0.735784813953)+(0.34431665114)*0.5**o
        ref[2,4]=(-0.618220314271)/(o+1.)+(0.170215579033)+(-0.482716534989)*0.5**o
        ref[3,0]=(-0.817231621484)/(o+1.)+(0.734717386696)+(0.323084711653)*0.5**o
        ref[3,1]=(-0.872776322969)/(o+1.)+(-0.471233340523)+(-0.464413485909)*0.5**o
        ref[3,2]=(0.211630014364)/(o+1.)+(-0.370456222914)+(-0.920685763376)*0.5**o
        ref[3,3]=(0.265852072745)/(o+1.)+(0.0558689104915)+(-0.175827412744)*0.5**o
        ref[3,4]=(-0.747340816778)/(o+1.)+(0.0858242297109)+(-0.99517403847)*0.5**o
      else:
        s[0,0]=(0.578101616539)*x[0]**o+(-0.941355465542)*x[0]+(-0.661840086733)*x[1]**o+(-0.574686372209)*x[1]+(-0.488980453488)*x[2]**o+(0.8057154356)*x[2]
        s[0,1]=(0.188043130009)*x[0]**o+(-0.53795144833)*x[0]+(0.585611542575)*x[1]**o+(-0.394354708911)*x[1]+(-0.753395938984)*x[2]**o+(-0.775038773873)*x[2]
        s[0,2]=(-0.0069454884707)*x[0]**o+(0.925798676533)*x[0]+(-0.617492289762)*x[1]**o+(-0.412950379597)*x[1]+(0.530650448811)*x[2]**o+(-0.882156765835)*x[2]
        s[0,3]=(-0.109411491583)*x[0]**o+(0.794417685565)*x[0]+(0.395199315447)*x[1]**o+(0.221963288858)*x[1]+(0.944618017853)*x[2]**o+(0.710576727198)*x[2]
        s[0,4]=(0.0376370205208)*x[0]**o+(-0.387469327476)*x[0]+(0.989435100596)*x[1]**o+(-0.697198283115)*x[1]+(0.705109597826)*x[2]**o+(0.00235357316422)*x[2]
        s[1,0]=(-0.861013305772)*x[0]**o+(-0.117175144313)*x[0]+(-0.851114880006)*x[1]**o+(0.830657298231)*x[1]+(0.897872709447)*x[2]**o+(-0.500434787953)*x[2]
        s[1,1]=(-0.659762659407)*x[0]**o+(-0.293584253789)*x[0]+(-0.549133974878)*x[1]**o+(0.124482383522)*x[1]+(-0.71415106775)*x[2]**o+(-0.452353669527)*x[2]
        s[1,2]=(-0.944798023011)*x[0]**o+(0.352545190549)*x[0]+(-0.950981266242)*x[1]**o+(-0.338339076366)*x[1]+(-0.385100481358)*x[2]**o+(-0.751511160424)*x[2]
        s[1,3]=(-0.514889491143)*x[0]**o+(-0.0573778568353)*x[0]+(0.189167569167)*x[1]**o+(-0.122152801538)*x[1]+(-0.075952059447)*x[2]**o+(-0.575271878121)*x[2]
        s[1,4]=(0.092128759086)*x[0]**o+(-0.391395765897)*x[0]+(0.621302034853)*x[1]**o+(-0.169193910488)*x[1]+(0.895953307228)*x[2]**o+(-0.171715374513)*x[2]
        s[2,0]=(-0.896341445352)*x[0]**o+(0.55427420625)*x[0]+(-0.68618123058)*x[1]**o+(0.586997200722)*x[1]+(-0.16193062027)*x[2]**o+(0.47614229102)*x[2]
        s[2,1]=(0.0173555065441)*x[0]**o+(-0.137706680345)*x[0]+(0.124390959958)*x[1]**o+(0.0817218208151)*x[1]+(-0.883487601024)*x[2]**o+(-0.99725370538)*x[2]
        s[2,2]=(0.653947795759)*x[0]**o+(-0.0561188253491)*x[0]+(-0.304869234336)*x[1]**o+(-0.5647971996)*x[1]+(0.842979371974)*x[2]**o+(0.271112357653)*x[2]
        s[2,3]=(-0.916870447792)*x[0]**o+(0.532760446169)*x[0]+(0.125531293566)*x[1]**o+(-0.772485238164)*x[1]+(-0.0249888509834)*x[2]**o+(-0.215774489527)*x[2]
        s[2,4]=(0.82957204564)*x[0]**o+(-0.536668870818)*x[0]+(0.18150637899)*x[1]**o+(0.213776071225)*x[1]+(-0.607190995474)*x[2]**o+(-0.0534051775785)*x[2]
        s[3,0]=(-0.0156695889369)*x[0]**o+(-0.920483023826)*x[0]+(0.173953779108)*x[1]**o+(0.723326965312)*x[1]+(-0.946445962586)*x[2]**o+(0.242480497946)*x[2]
        s[3,1]=(0.253546369583)*x[0]**o+(-0.455303700999)*x[0]+(0.265484731924)*x[1]**o+(-0.334855283665)*x[1]+(0.263585801553)*x[2]**o+(0.768099093921)*x[2]
        s[3,2]=(-0.0884797551735)*x[0]**o+(-0.744966141189)*x[0]+(0.685728482818)*x[1]**o+(-0.700119204761)*x[1]+(-0.996053069429)*x[2]**o+(-0.300758647065)*x[2]
        s[3,3]=(-0.265221319311)*x[0]**o+(0.119741272443)*x[0]+(0.0623093135308)*x[1]**o+(0.617188763307)*x[1]+(-0.14935010921)*x[2]**o+(0.315030342521)*x[2]
        s[3,4]=(0.634255080399)*x[0]**o+(0.0848130779284)*x[0]+(-0.262488356592)*x[1]**o+(-0.698599321231)*x[1]+(-0.411566742148)*x[2]**o+(0.307995972853)*x[2]
        ref[0,0]=(-1.15082054022)/(o+1.)+(-0.355163201075)+(0.578101616539)*0.5**o
        ref[0,1]=(-0.167784396409)/(o+1.)+(-0.853672465557)+(0.188043130009)*0.5**o
        ref[0,2]=(-0.0868418409506)/(o+1.)+(-0.18465423445)+(-0.0069454884707)*0.5**o
        ref[0,3]=(1.3398173333)/(o+1.)+(0.86347885081)+(-0.109411491583)*0.5**o
        ref[0,4]=(1.69454469842)/(o+1.)+(-0.541157018713)+(0.0376370205208)*0.5**o
        ref[1,0]=(0.0467578294411)/(o+1.)+(0.106523682982)+(-0.861013305772)*0.5**o
        ref[1,1]=(-1.26328504263)/(o+1.)+(-0.310727769897)+(-0.659762659407)*0.5**o
        ref[1,2]=(-1.3360817476)/(o+1.)+(-0.36865252312)+(-0.944798023011)*0.5**o
        ref[1,3]=(0.11321550972)/(o+1.)+(-0.377401268247)+(-0.514889491143)*0.5**o
        ref[1,4]=(1.51725534208)/(o+1.)+(-0.366152525449)+(0.092128759086)*0.5**o
        ref[2,0]=(-0.84811185085)/(o+1.)+(0.808706848996)+(-0.896341445352)*0.5**o
        ref[2,1]=(-0.759096641066)/(o+1.)+(-0.526619282455)+(0.0173555065441)*0.5**o
        ref[2,2]=(0.538110137638)/(o+1.)+(-0.174901833648)+(0.653947795759)*0.5**o
        ref[2,3]=(0.100542442583)/(o+1.)+(-0.227749640761)+(-0.916870447792)*0.5**o
        ref[2,4]=(-0.425684616484)/(o+1.)+(-0.188148988586)+(0.82957204564)*0.5**o
        ref[3,0]=(-0.772492183478)/(o+1.)+(0.0226622197163)+(-0.0156695889369)*0.5**o
        ref[3,1]=(0.529070533478)/(o+1.)+(-0.0110299453715)+(0.253546369583)*0.5**o
        ref[3,2]=(-0.310324586611)/(o+1.)+(-0.872921996508)+(-0.0884797551735)*0.5**o
        ref[3,3]=(-0.0870407956792)/(o+1.)+(0.525980189135)+(-0.265221319311)*0.5**o
        ref[3,4]=(-0.67405509874)/(o+1.)+(-0.152895135225)+(0.634255080399)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactZero_fromSymbol_Solution_rank3(self):
      """
      tests integral of rank 3 Symbol on the FunctionOnContactZero

      assumptions: Solution(self.domain) exists
                   self.domain supports integral on FunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      w=Solution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(6, 2, 2))
      s=Data(0,(6, 2, 2),w)
      ref=numpy.zeros((6, 2, 2),numpy.float_)
      if dim==2:
        s[0,0,0]=(-0.970014070821)*x[0]**o+(0.195828821364)*x[0]+(-0.329248582962)*x[1]**o+(0.765729760657)*x[1]
        s[0,0,1]=(-0.78480276531)*x[0]**o+(0.660249421528)*x[0]+(-0.112211565471)*x[1]**o+(-0.732753874188)*x[1]
        s[0,1,0]=(-0.610728226383)*x[0]**o+(-0.617391668915)*x[0]+(-0.755934195771)*x[1]**o+(-0.301037750184)*x[1]
        s[0,1,1]=(-0.271483777605)*x[0]**o+(-0.118144079798)*x[0]+(0.00595756313744)*x[1]**o+(0.00973544005055)*x[1]
        s[1,0,0]=(-0.880152279586)*x[0]**o+(0.173587205863)*x[0]+(-0.772256433861)*x[1]**o+(-0.929831194333)*x[1]
        s[1,0,1]=(0.663917147636)*x[0]**o+(0.602039234824)*x[0]+(-0.761663518039)*x[1]**o+(-0.794981650104)*x[1]
        s[1,1,0]=(0.669982500097)*x[0]**o+(-0.867340940384)*x[0]+(0.425166963846)*x[1]**o+(0.809034058423)*x[1]
        s[1,1,1]=(-0.748164617999)*x[0]**o+(0.820897422648)*x[0]+(0.282790963068)*x[1]**o+(0.418210593072)*x[1]
        s[2,0,0]=(-0.151105173252)*x[0]**o+(0.588856734796)*x[0]+(-0.587574498534)*x[1]**o+(0.214690992942)*x[1]
        s[2,0,1]=(0.846293738229)*x[0]**o+(0.453778324604)*x[0]+(-0.750479939889)*x[1]**o+(0.988318560374)*x[1]
        s[2,1,0]=(0.0520443817173)*x[0]**o+(0.66477304094)*x[0]+(0.220203729906)*x[1]**o+(0.807678486344)*x[1]
        s[2,1,1]=(-0.481828231947)*x[0]**o+(0.301163693914)*x[0]+(-0.240012695397)*x[1]**o+(-0.739330362354)*x[1]
        s[3,0,0]=(-0.464086304345)*x[0]**o+(0.204976865077)*x[0]+(0.360828971652)*x[1]**o+(0.179421383025)*x[1]
        s[3,0,1]=(0.524003162473)*x[0]**o+(0.241725193779)*x[0]+(0.137006148453)*x[1]**o+(0.122270180832)*x[1]
        s[3,1,0]=(-0.148537542555)*x[0]**o+(-0.87771789463)*x[0]+(0.776889116646)*x[1]**o+(0.72202598779)*x[1]
        s[3,1,1]=(-0.0984207735775)*x[0]**o+(-0.200545725506)*x[0]+(-0.422044149217)*x[1]**o+(-0.403374011642)*x[1]
        s[4,0,0]=(0.332893223127)*x[0]**o+(-0.238038393676)*x[0]+(-0.796376263577)*x[1]**o+(0.059346957422)*x[1]
        s[4,0,1]=(-0.877726930853)*x[0]**o+(0.287774676493)*x[0]+(-0.690097100077)*x[1]**o+(0.806523530334)*x[1]
        s[4,1,0]=(-0.466369745006)*x[0]**o+(0.511932688176)*x[0]+(-0.569648246707)*x[1]**o+(-0.180363948019)*x[1]
        s[4,1,1]=(0.82395338594)*x[0]**o+(0.0423959362879)*x[0]+(-0.508349189015)*x[1]**o+(0.0735939835821)*x[1]
        s[5,0,0]=(-0.631799410304)*x[0]**o+(0.0828019140935)*x[0]+(0.132222069751)*x[1]**o+(0.716700894256)*x[1]
        s[5,0,1]=(-0.871739775831)*x[0]**o+(-0.364312533128)*x[0]+(0.660932493381)*x[1]**o+(-0.638799141415)*x[1]
        s[5,1,0]=(-0.927257402325)*x[0]**o+(-0.745507370225)*x[0]+(0.70647119912)*x[1]**o+(0.583112912025)*x[1]
        s[5,1,1]=(-0.440675815785)*x[0]**o+(0.292262539618)*x[0]+(0.172815874747)*x[1]**o+(-0.854959570718)*x[1]
        ref[0,0,0]=(-0.329248582962)/(o+1.)+(0.480779291011)+(-0.970014070821)*0.5**o
        ref[0,0,1]=(-0.112211565471)/(o+1.)+(-0.0362522263304)+(-0.78480276531)*0.5**o
        ref[0,1,0]=(-0.755934195771)/(o+1.)+(-0.459214709549)+(-0.610728226383)*0.5**o
        ref[0,1,1]=(0.00595756313744)/(o+1.)+(-0.0542043198736)+(-0.271483777605)*0.5**o
        ref[1,0,0]=(-0.772256433861)/(o+1.)+(-0.378121994235)+(-0.880152279586)*0.5**o
        ref[1,0,1]=(-0.761663518039)/(o+1.)+(-0.0964712076398)+(0.663917147636)*0.5**o
        ref[1,1,0]=(0.425166963846)/(o+1.)+(-0.0291534409806)+(0.669982500097)*0.5**o
        ref[1,1,1]=(0.282790963068)/(o+1.)+(0.61955400786)+(-0.748164617999)*0.5**o
        ref[2,0,0]=(-0.587574498534)/(o+1.)+(0.401773863869)+(-0.151105173252)*0.5**o
        ref[2,0,1]=(-0.750479939889)/(o+1.)+(0.721048442489)+(0.846293738229)*0.5**o
        ref[2,1,0]=(0.220203729906)/(o+1.)+(0.736225763642)+(0.0520443817173)*0.5**o
        ref[2,1,1]=(-0.240012695397)/(o+1.)+(-0.21908333422)+(-0.481828231947)*0.5**o
        ref[3,0,0]=(0.360828971652)/(o+1.)+(0.192199124051)+(-0.464086304345)*0.5**o
        ref[3,0,1]=(0.137006148453)/(o+1.)+(0.181997687306)+(0.524003162473)*0.5**o
        ref[3,1,0]=(0.776889116646)/(o+1.)+(-0.0778459534199)+(-0.148537542555)*0.5**o
        ref[3,1,1]=(-0.422044149217)/(o+1.)+(-0.301959868574)+(-0.0984207735775)*0.5**o
        ref[4,0,0]=(-0.796376263577)/(o+1.)+(-0.0893457181268)+(0.332893223127)*0.5**o
        ref[4,0,1]=(-0.690097100077)/(o+1.)+(0.547149103413)+(-0.877726930853)*0.5**o
        ref[4,1,0]=(-0.569648246707)/(o+1.)+(0.165784370078)+(-0.466369745006)*0.5**o
        ref[4,1,1]=(-0.508349189015)/(o+1.)+(0.057994959935)+(0.82395338594)*0.5**o
        ref[5,0,0]=(0.132222069751)/(o+1.)+(0.399751404175)+(-0.631799410304)*0.5**o
        ref[5,0,1]=(0.660932493381)/(o+1.)+(-0.501555837272)+(-0.871739775831)*0.5**o
        ref[5,1,0]=(0.70647119912)/(o+1.)+(-0.0811972291003)+(-0.927257402325)*0.5**o
        ref[5,1,1]=(0.172815874747)/(o+1.)+(-0.28134851555)+(-0.440675815785)*0.5**o
      else:
        s[0,0,0]=(-0.565533896373)*x[0]**o+(-0.0872485111367)*x[0]+(-0.283353388539)*x[1]**o+(-0.152989536217)*x[1]+(-0.444002969358)*x[2]**o+(0.670423385061)*x[2]
        s[0,0,1]=(-0.0944154284688)*x[0]**o+(-0.884982028889)*x[0]+(0.79965600284)*x[1]**o+(-0.422245813699)*x[1]+(-0.479173724334)*x[2]**o+(-0.726826676767)*x[2]
        s[0,1,0]=(0.789722540135)*x[0]**o+(-0.0774752885776)*x[0]+(-0.95515742871)*x[1]**o+(0.300583198538)*x[1]+(0.485489410617)*x[2]**o+(-0.303592354814)*x[2]
        s[0,1,1]=(0.652206005345)*x[0]**o+(0.90387230638)*x[0]+(-0.943316098191)*x[1]**o+(-0.600569529668)*x[1]+(-0.157071495888)*x[2]**o+(0.289655248779)*x[2]
        s[1,0,0]=(0.963881471424)*x[0]**o+(-0.67090862423)*x[0]+(-0.56250637167)*x[1]**o+(0.432938287823)*x[1]+(0.632690957193)*x[2]**o+(0.437978271014)*x[2]
        s[1,0,1]=(0.211364301181)*x[0]**o+(-0.482725619403)*x[0]+(0.818668277765)*x[1]**o+(-0.202108275895)*x[1]+(0.38755788328)*x[2]**o+(0.315009480388)*x[2]
        s[1,1,0]=(-0.931623738361)*x[0]**o+(0.865171378595)*x[0]+(0.983563036962)*x[1]**o+(-0.674022850075)*x[1]+(-0.743850008639)*x[2]**o+(-0.666081693537)*x[2]
        s[1,1,1]=(0.00368798622506)*x[0]**o+(0.431757463365)*x[0]+(-0.348566468597)*x[1]**o+(0.451792881229)*x[1]+(-0.380511811466)*x[2]**o+(0.850391058252)*x[2]
        s[2,0,0]=(0.0418268780255)*x[0]**o+(0.749915304085)*x[0]+(0.155766457341)*x[1]**o+(0.782071885494)*x[1]+(-0.539808510551)*x[2]**o+(0.483245601178)*x[2]
        s[2,0,1]=(-0.460744056666)*x[0]**o+(0.124701348548)*x[0]+(-0.703875182049)*x[1]**o+(0.796527478381)*x[1]+(-0.221938800568)*x[2]**o+(0.971830341802)*x[2]
        s[2,1,0]=(0.53954548965)*x[0]**o+(0.0162996837218)*x[0]+(0.142104929682)*x[1]**o+(-0.971657840256)*x[1]+(-0.503179671457)*x[2]**o+(-0.542014263985)*x[2]
        s[2,1,1]=(0.908980902057)*x[0]**o+(-0.502995849973)*x[0]+(-0.306045266955)*x[1]**o+(0.663346607536)*x[1]+(-0.350114579334)*x[2]**o+(0.576410602991)*x[2]
        s[3,0,0]=(0.813242844535)*x[0]**o+(-0.373751247405)*x[0]+(-0.155529865713)*x[1]**o+(-0.757860853884)*x[1]+(0.834637852603)*x[2]**o+(0.909806811712)*x[2]
        s[3,0,1]=(-0.287829796913)*x[0]**o+(-0.521503208519)*x[0]+(0.373235117298)*x[1]**o+(0.820819656976)*x[1]+(0.666911247106)*x[2]**o+(-0.0367218914776)*x[2]
        s[3,1,0]=(0.432791246384)*x[0]**o+(0.958703389048)*x[0]+(0.964027641291)*x[1]**o+(0.546429571768)*x[1]+(-0.299474553396)*x[2]**o+(-0.699665640827)*x[2]
        s[3,1,1]=(0.728884851779)*x[0]**o+(0.650980056729)*x[0]+(0.133592857822)*x[1]**o+(0.693391334869)*x[1]+(-0.836602488593)*x[2]**o+(-0.442753441073)*x[2]
        s[4,0,0]=(0.833243436386)*x[0]**o+(0.980178093504)*x[0]+(-0.819050787144)*x[1]**o+(0.797889271257)*x[1]+(-0.723026474407)*x[2]**o+(0.783764698489)*x[2]
        s[4,0,1]=(-0.452251186626)*x[0]**o+(-0.627980945947)*x[0]+(-0.462394480861)*x[1]**o+(-0.591297748282)*x[1]+(-0.630289931985)*x[2]**o+(0.495939505391)*x[2]
        s[4,1,0]=(-0.774984503568)*x[0]**o+(0.635843202085)*x[0]+(0.835305935226)*x[1]**o+(-0.0209656725511)*x[1]+(0.257099035711)*x[2]**o+(-0.735192237148)*x[2]
        s[4,1,1]=(0.919373693068)*x[0]**o+(0.38072204474)*x[0]+(-0.92955685309)*x[1]**o+(-0.145126274743)*x[1]+(-0.50004893122)*x[2]**o+(-0.252139341208)*x[2]
        s[5,0,0]=(0.802245464817)*x[0]**o+(0.617146384769)*x[0]+(0.0802760805435)*x[1]**o+(-0.698667208547)*x[1]+(-0.531164225022)*x[2]**o+(-0.761354857435)*x[2]
        s[5,0,1]=(-0.400952829207)*x[0]**o+(0.605414258928)*x[0]+(0.563769114393)*x[1]**o+(0.952187363261)*x[1]+(-0.539409233701)*x[2]**o+(-0.680444521941)*x[2]
        s[5,1,0]=(0.0304057674653)*x[0]**o+(0.190919897349)*x[0]+(-0.812827460152)*x[1]**o+(0.822295166707)*x[1]+(0.678796730302)*x[2]**o+(-0.648229118268)*x[2]
        s[5,1,1]=(0.644695792666)*x[0]**o+(-0.0232924889471)*x[0]+(0.294470818979)*x[1]**o+(0.626411257894)*x[1]+(-0.453808003683)*x[2]**o+(0.89191417949)*x[2]
        ref[0,0,0]=(-0.727356357898)/(o+1.)+(0.215092668853)+(-0.565533896373)*0.5**o
        ref[0,0,1]=(0.320482278506)/(o+1.)+(-1.01702725968)+(-0.0944154284688)*0.5**o
        ref[0,1,0]=(-0.469668018093)/(o+1.)+(-0.0402422224272)+(0.789722540135)*0.5**o
        ref[0,1,1]=(-1.10038759408)/(o+1.)+(0.296479012746)+(0.652206005345)*0.5**o
        ref[1,0,0]=(0.0701845855234)/(o+1.)+(0.100003967304)+(0.963881471424)*0.5**o
        ref[1,0,1]=(1.20622616105)/(o+1.)+(-0.184912207455)+(0.211364301181)*0.5**o
        ref[1,1,0]=(0.239713028324)/(o+1.)+(-0.237466582508)+(-0.931623738361)*0.5**o
        ref[1,1,1]=(-0.729078280063)/(o+1.)+(0.866970701424)+(0.00368798622506)*0.5**o
        ref[2,0,0]=(-0.38404205321)/(o+1.)+(1.00761639538)+(0.0418268780255)*0.5**o
        ref[2,0,1]=(-0.925813982617)/(o+1.)+(0.946529584366)+(-0.460744056666)*0.5**o
        ref[2,1,0]=(-0.361074741775)/(o+1.)+(-0.74868621026)+(0.53954548965)*0.5**o
        ref[2,1,1]=(-0.656159846289)/(o+1.)+(0.368380680277)+(0.908980902057)*0.5**o
        ref[3,0,0]=(0.67910798689)/(o+1.)+(-0.110902644788)+(0.813242844535)*0.5**o
        ref[3,0,1]=(1.0401463644)/(o+1.)+(0.13129727849)+(-0.287829796913)*0.5**o
        ref[3,1,0]=(0.664553087895)/(o+1.)+(0.402733659994)+(0.432791246384)*0.5**o
        ref[3,1,1]=(-0.703009630771)/(o+1.)+(0.450808975263)+(0.728884851779)*0.5**o
        ref[4,0,0]=(-1.54207726155)/(o+1.)+(1.28091603163)+(0.833243436386)*0.5**o
        ref[4,0,1]=(-1.09268441285)/(o+1.)+(-0.361669594419)+(-0.452251186626)*0.5**o
        ref[4,1,0]=(1.09240497094)/(o+1.)+(-0.0601573538071)+(-0.774984503568)*0.5**o
        ref[4,1,1]=(-1.42960578431)/(o+1.)+(-0.00827178560514)+(0.919373693068)*0.5**o
        ref[5,0,0]=(-0.450888144479)/(o+1.)+(-0.421437840607)+(0.802245464817)*0.5**o
        ref[5,0,1]=(0.0243598806918)/(o+1.)+(0.438578550124)+(-0.400952829207)*0.5**o
        ref[5,1,0]=(-0.13403072985)/(o+1.)+(0.182492972894)+(0.0304057674653)*0.5**o
        ref[5,1,1]=(-0.159337184705)/(o+1.)+(0.747516474218)+(0.644695792666)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactZero_fromSymbol_Solution_rank4(self):
      """
      tests integral of rank 4 Symbol on the FunctionOnContactZero

      assumptions: Solution(self.domain) exists
                   self.domain supports integral on FunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      w=Solution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5, 3, 2))
      s=Data(0,(4, 5, 3, 2),w)
      ref=numpy.zeros((4, 5, 3, 2),numpy.float_)
      if dim==2:
        s[0,0,0,0]=(0.0390601545717)*x[0]**o+(-0.222613292224)*x[0]+(0.246692793786)*x[1]**o+(-0.0199554145217)*x[1]
        s[0,0,0,1]=(0.608816949634)*x[0]**o+(0.840495963273)*x[0]+(-0.986136095398)*x[1]**o+(0.16852755525)*x[1]
        s[0,0,1,0]=(0.580621174141)*x[0]**o+(-0.931460241997)*x[0]+(0.725705948245)*x[1]**o+(0.0239157549247)*x[1]
        s[0,0,1,1]=(0.388567471102)*x[0]**o+(0.311382975022)*x[0]+(-0.376543304375)*x[1]**o+(-0.859691633345)*x[1]
        s[0,0,2,0]=(-0.692988033922)*x[0]**o+(-0.059082135717)*x[0]+(-0.088525361736)*x[1]**o+(-0.435255796923)*x[1]
        s[0,0,2,1]=(0.274277434778)*x[0]**o+(0.783272547433)*x[0]+(-0.380234357131)*x[1]**o+(-0.355490082493)*x[1]
        s[0,1,0,0]=(-0.802394756916)*x[0]**o+(-0.352698691101)*x[0]+(0.335411545595)*x[1]**o+(0.558556090302)*x[1]
        s[0,1,0,1]=(0.174271017656)*x[0]**o+(-0.442785108924)*x[0]+(0.178161299328)*x[1]**o+(-0.18823731442)*x[1]
        s[0,1,1,0]=(-0.598879030318)*x[0]**o+(0.373869209105)*x[0]+(0.589565043365)*x[1]**o+(-0.62976640408)*x[1]
        s[0,1,1,1]=(-0.667493955405)*x[0]**o+(0.998759793079)*x[0]+(-0.743341570183)*x[1]**o+(-0.0859848525819)*x[1]
        s[0,1,2,0]=(0.47631386081)*x[0]**o+(0.0151722662021)*x[0]+(0.899075872977)*x[1]**o+(0.0678095622752)*x[1]
        s[0,1,2,1]=(-0.497833539416)*x[0]**o+(0.00600269705464)*x[0]+(-0.541353591572)*x[1]**o+(0.0705426009397)*x[1]
        s[0,2,0,0]=(-0.735607995943)*x[0]**o+(0.921942121227)*x[0]+(-0.697588210675)*x[1]**o+(-0.555902048922)*x[1]
        s[0,2,0,1]=(0.705776536011)*x[0]**o+(0.202303757789)*x[0]+(-0.630771158486)*x[1]**o+(-0.0166043126446)*x[1]
        s[0,2,1,0]=(-0.975134219189)*x[0]**o+(0.566584421811)*x[0]+(0.315121166158)*x[1]**o+(-0.952791171948)*x[1]
        s[0,2,1,1]=(-0.43751086407)*x[0]**o+(0.793023085525)*x[0]+(-0.471199351714)*x[1]**o+(0.227828159087)*x[1]
        s[0,2,2,0]=(0.526770705207)*x[0]**o+(-0.779856515281)*x[0]+(0.433608622434)*x[1]**o+(0.45679766746)*x[1]
        s[0,2,2,1]=(-0.418834628035)*x[0]**o+(0.325385358616)*x[0]+(0.0493546850323)*x[1]**o+(-0.611087977891)*x[1]
        s[0,3,0,0]=(0.302691082154)*x[0]**o+(-0.0840254373723)*x[0]+(0.235788766456)*x[1]**o+(0.00697145365732)*x[1]
        s[0,3,0,1]=(-0.400507082213)*x[0]**o+(0.922575813908)*x[0]+(0.653142282067)*x[1]**o+(-0.819060793296)*x[1]
        s[0,3,1,0]=(-0.542123872101)*x[0]**o+(-0.386997884801)*x[0]+(-0.0222151576861)*x[1]**o+(-0.914062036349)*x[1]
        s[0,3,1,1]=(0.0629454961983)*x[0]**o+(0.953558638015)*x[0]+(-0.977561971438)*x[1]**o+(-0.3034113517)*x[1]
        s[0,3,2,0]=(0.088224801816)*x[0]**o+(-0.0370702186771)*x[0]+(0.094320839406)*x[1]**o+(-0.342299582748)*x[1]
        s[0,3,2,1]=(0.976917465786)*x[0]**o+(-0.528174142655)*x[0]+(-0.387883052017)*x[1]**o+(-0.631752291799)*x[1]
        s[0,4,0,0]=(-0.468335475351)*x[0]**o+(-0.565555734167)*x[0]+(-0.308997447406)*x[1]**o+(-0.747826956462)*x[1]
        s[0,4,0,1]=(0.069233902558)*x[0]**o+(-0.646481367913)*x[0]+(-0.978900933671)*x[1]**o+(0.743358367572)*x[1]
        s[0,4,1,0]=(0.192177279681)*x[0]**o+(0.443905094115)*x[0]+(-0.687749538927)*x[1]**o+(-0.0130016040578)*x[1]
        s[0,4,1,1]=(0.336586630924)*x[0]**o+(0.952526894755)*x[0]+(-0.32776980948)*x[1]**o+(-0.46574155251)*x[1]
        s[0,4,2,0]=(0.635240845083)*x[0]**o+(-0.694619696795)*x[0]+(0.184107273061)*x[1]**o+(-0.102554433435)*x[1]
        s[0,4,2,1]=(0.392781256576)*x[0]**o+(0.962026661797)*x[0]+(0.657049172645)*x[1]**o+(0.462113010746)*x[1]
        s[1,0,0,0]=(0.188456710847)*x[0]**o+(0.835668677088)*x[0]+(0.379822929854)*x[1]**o+(-0.864958528812)*x[1]
        s[1,0,0,1]=(-0.910686394304)*x[0]**o+(-0.323019307138)*x[0]+(0.339165812738)*x[1]**o+(0.651547968842)*x[1]
        s[1,0,1,0]=(0.0133563994025)*x[0]**o+(-0.931707806441)*x[0]+(0.473665399577)*x[1]**o+(-0.626120577625)*x[1]
        s[1,0,1,1]=(0.19107514064)*x[0]**o+(-0.462202695026)*x[0]+(0.46369841557)*x[1]**o+(-0.253328976575)*x[1]
        s[1,0,2,0]=(0.179414794128)*x[0]**o+(-0.809074596595)*x[0]+(0.128452495995)*x[1]**o+(-0.501454759976)*x[1]
        s[1,0,2,1]=(0.507586792052)*x[0]**o+(-0.448027963861)*x[0]+(0.244339731125)*x[1]**o+(0.506838949931)*x[1]
        s[1,1,0,0]=(0.0515340520755)*x[0]**o+(-0.554979125841)*x[0]+(0.508215404195)*x[1]**o+(-0.880450798699)*x[1]
        s[1,1,0,1]=(0.149995895729)*x[0]**o+(0.207292493123)*x[0]+(-0.0719243226652)*x[1]**o+(-0.9540219098)*x[1]
        s[1,1,1,0]=(-0.298918981626)*x[0]**o+(0.90891772564)*x[0]+(0.0457980649267)*x[1]**o+(-0.266285753328)*x[1]
        s[1,1,1,1]=(-0.833495545848)*x[0]**o+(0.902967001807)*x[0]+(0.103572135419)*x[1]**o+(-0.262026617122)*x[1]
        s[1,1,2,0]=(0.381644351387)*x[0]**o+(0.77834766803)*x[0]+(-0.785664720197)*x[1]**o+(0.678219935866)*x[1]
        s[1,1,2,1]=(0.589726138962)*x[0]**o+(-0.794897130021)*x[0]+(0.0749818396161)*x[1]**o+(0.818217309223)*x[1]
        s[1,2,0,0]=(0.648262414915)*x[0]**o+(0.737500663109)*x[0]+(0.714248701071)*x[1]**o+(-0.306480036057)*x[1]
        s[1,2,0,1]=(0.154336156671)*x[0]**o+(0.988237826231)*x[0]+(0.186782608772)*x[1]**o+(-0.29179239454)*x[1]
        s[1,2,1,0]=(0.631907675013)*x[0]**o+(-0.297341493926)*x[0]+(-0.159116807039)*x[1]**o+(-0.435773119414)*x[1]
        s[1,2,1,1]=(-0.150281595818)*x[0]**o+(0.00593213086215)*x[0]+(0.995323329778)*x[1]**o+(0.978446533527)*x[1]
        s[1,2,2,0]=(0.15122501321)*x[0]**o+(0.781060971949)*x[0]+(0.945042808379)*x[1]**o+(-0.236073537251)*x[1]
        s[1,2,2,1]=(0.490853599647)*x[0]**o+(0.70560372132)*x[0]+(0.518228280852)*x[1]**o+(0.00359879318509)*x[1]
        s[1,3,0,0]=(-0.868069831372)*x[0]**o+(-0.808015513984)*x[0]+(-0.99302787628)*x[1]**o+(0.643430993188)*x[1]
        s[1,3,0,1]=(-0.540392753009)*x[0]**o+(0.412535909714)*x[0]+(-0.691622353639)*x[1]**o+(-0.726320722169)*x[1]
        s[1,3,1,0]=(-0.38171518034)*x[0]**o+(0.574401211179)*x[0]+(0.239758390058)*x[1]**o+(0.667920000788)*x[1]
        s[1,3,1,1]=(0.405015772479)*x[0]**o+(0.871607122253)*x[0]+(0.875356104271)*x[1]**o+(0.571089711154)*x[1]
        s[1,3,2,0]=(-0.859288591023)*x[0]**o+(0.0495016165977)*x[0]+(0.652544093354)*x[1]**o+(-0.950289461543)*x[1]
        s[1,3,2,1]=(-0.385753789824)*x[0]**o+(-0.929231955794)*x[0]+(-0.266920992105)*x[1]**o+(-0.170089958141)*x[1]
        s[1,4,0,0]=(0.248320342844)*x[0]**o+(-0.193177840651)*x[0]+(-0.974008625555)*x[1]**o+(0.19289653706)*x[1]
        s[1,4,0,1]=(-0.408988831595)*x[0]**o+(-0.753769725549)*x[0]+(-0.890410564837)*x[1]**o+(-0.574271841364)*x[1]
        s[1,4,1,0]=(-0.637267396746)*x[0]**o+(-0.270785389868)*x[0]+(0.272797445173)*x[1]**o+(-0.495088539306)*x[1]
        s[1,4,1,1]=(-0.711284122459)*x[0]**o+(0.635952871792)*x[0]+(-0.0849564635014)*x[1]**o+(-0.227419456586)*x[1]
        s[1,4,2,0]=(0.194861831786)*x[0]**o+(0.52267182066)*x[0]+(0.396913284632)*x[1]**o+(0.446135337457)*x[1]
        s[1,4,2,1]=(-0.111812571843)*x[0]**o+(0.919973056019)*x[0]+(0.954568821795)*x[1]**o+(-0.410846336745)*x[1]
        s[2,0,0,0]=(0.149006416722)*x[0]**o+(-0.254413162151)*x[0]+(0.66590243332)*x[1]**o+(0.28412642191)*x[1]
        s[2,0,0,1]=(-0.788828642343)*x[0]**o+(-0.384244929105)*x[0]+(-0.115430427081)*x[1]**o+(-0.670217327178)*x[1]
        s[2,0,1,0]=(0.598185590774)*x[0]**o+(-0.593162615988)*x[0]+(-0.247117069506)*x[1]**o+(-0.545724450495)*x[1]
        s[2,0,1,1]=(-0.945230420194)*x[0]**o+(-0.158557029333)*x[0]+(-0.628357637608)*x[1]**o+(-0.516938772751)*x[1]
        s[2,0,2,0]=(0.184549704327)*x[0]**o+(-0.753954168024)*x[0]+(-0.43734992018)*x[1]**o+(-0.767428538976)*x[1]
        s[2,0,2,1]=(-0.897896267578)*x[0]**o+(0.782159121394)*x[0]+(0.926088436529)*x[1]**o+(-0.922666911396)*x[1]
        s[2,1,0,0]=(-0.0375376932325)*x[0]**o+(-0.648996277693)*x[0]+(0.163113306006)*x[1]**o+(0.326391594556)*x[1]
        s[2,1,0,1]=(-0.603507455453)*x[0]**o+(-0.227212072795)*x[0]+(0.762838138451)*x[1]**o+(0.784904636664)*x[1]
        s[2,1,1,0]=(-0.250121932289)*x[0]**o+(0.820330134347)*x[0]+(0.954063036094)*x[1]**o+(-0.0944681705347)*x[1]
        s[2,1,1,1]=(0.0147284052497)*x[0]**o+(0.214839823184)*x[0]+(-0.932724945971)*x[1]**o+(0.723405207766)*x[1]
        s[2,1,2,0]=(-0.832131948289)*x[0]**o+(-0.361716015855)*x[0]+(-0.648739038507)*x[1]**o+(-0.825135489955)*x[1]
        s[2,1,2,1]=(-0.809460903823)*x[0]**o+(0.219025460406)*x[0]+(-0.399288944386)*x[1]**o+(-0.443923619258)*x[1]
        s[2,2,0,0]=(0.851821921349)*x[0]**o+(0.864789449232)*x[0]+(-0.146472829559)*x[1]**o+(-0.13802610172)*x[1]
        s[2,2,0,1]=(0.145872642324)*x[0]**o+(-0.53737681799)*x[0]+(-0.262458059774)*x[1]**o+(-0.865100768157)*x[1]
        s[2,2,1,0]=(-0.97150697777)*x[0]**o+(0.74271566101)*x[0]+(-0.941540320886)*x[1]**o+(-0.992417829758)*x[1]
        s[2,2,1,1]=(0.705001677542)*x[0]**o+(-0.464736083815)*x[0]+(0.725341857111)*x[1]**o+(-0.350205035943)*x[1]
        s[2,2,2,0]=(-0.405826604429)*x[0]**o+(0.741314043354)*x[0]+(-0.797706422949)*x[1]**o+(0.177373453057)*x[1]
        s[2,2,2,1]=(-0.338189348522)*x[0]**o+(-0.933697184604)*x[0]+(-0.381859891282)*x[1]**o+(0.0825590545614)*x[1]
        s[2,3,0,0]=(-0.532343324478)*x[0]**o+(-0.425963751562)*x[0]+(0.306014483173)*x[1]**o+(-0.886115500167)*x[1]
        s[2,3,0,1]=(0.384975538101)*x[0]**o+(0.198217241539)*x[0]+(0.939876357973)*x[1]**o+(-0.227463489114)*x[1]
        s[2,3,1,0]=(0.39221893978)*x[0]**o+(-0.173675431821)*x[0]+(0.929829286228)*x[1]**o+(0.643170164534)*x[1]
        s[2,3,1,1]=(-0.706521549808)*x[0]**o+(-0.6114595264)*x[0]+(0.207625492571)*x[1]**o+(0.262958323309)*x[1]
        s[2,3,2,0]=(0.464449153296)*x[0]**o+(-0.438225086666)*x[0]+(-0.463914786644)*x[1]**o+(0.532074329884)*x[1]
        s[2,3,2,1]=(-0.755612237619)*x[0]**o+(0.450659729492)*x[0]+(0.265758298887)*x[1]**o+(0.0352759835818)*x[1]
        s[2,4,0,0]=(-0.913101029308)*x[0]**o+(-0.244134313945)*x[0]+(0.347084253405)*x[1]**o+(0.84625762348)*x[1]
        s[2,4,0,1]=(-0.261381262076)*x[0]**o+(-0.0817994875895)*x[0]+(0.870579357546)*x[1]**o+(-0.917318949068)*x[1]
        s[2,4,1,0]=(-0.128568967272)*x[0]**o+(0.732248131756)*x[0]+(0.414206585164)*x[1]**o+(0.659941361833)*x[1]
        s[2,4,1,1]=(0.924901584611)*x[0]**o+(0.592074499154)*x[0]+(0.681628131999)*x[1]**o+(0.167953983361)*x[1]
        s[2,4,2,0]=(0.409615949394)*x[0]**o+(-0.902690757356)*x[0]+(0.577674551804)*x[1]**o+(0.0220011629429)*x[1]
        s[2,4,2,1]=(0.382356012027)*x[0]**o+(-0.423633937718)*x[0]+(-0.592574898322)*x[1]**o+(0.183184987952)*x[1]
        s[3,0,0,0]=(-0.254267267009)*x[0]**o+(-0.665177288859)*x[0]+(-0.666690913098)*x[1]**o+(-0.433935909161)*x[1]
        s[3,0,0,1]=(-0.672935968283)*x[0]**o+(-0.862578688926)*x[0]+(-0.431014198736)*x[1]**o+(-0.950634125485)*x[1]
        s[3,0,1,0]=(0.198586757881)*x[0]**o+(0.654459371747)*x[0]+(0.768197508919)*x[1]**o+(-0.911430651255)*x[1]
        s[3,0,1,1]=(0.835596691179)*x[0]**o+(0.662532614321)*x[0]+(0.64745394627)*x[1]**o+(-0.177665076446)*x[1]
        s[3,0,2,0]=(-0.60108209247)*x[0]**o+(0.104120238589)*x[0]+(0.478363486403)*x[1]**o+(0.60998776125)*x[1]
        s[3,0,2,1]=(-0.632653025461)*x[0]**o+(0.415693475381)*x[0]+(-0.0139161976239)*x[1]**o+(-0.486360398365)*x[1]
        s[3,1,0,0]=(-0.216238961155)*x[0]**o+(0.605030211804)*x[0]+(-0.894286771395)*x[1]**o+(-0.738005399203)*x[1]
        s[3,1,0,1]=(0.453465128819)*x[0]**o+(-0.958353730943)*x[0]+(-0.191303802435)*x[1]**o+(-0.403969790319)*x[1]
        s[3,1,1,0]=(-0.371601120889)*x[0]**o+(-0.000506485761004)*x[0]+(0.128347107042)*x[1]**o+(-0.617448247517)*x[1]
        s[3,1,1,1]=(-0.492741845463)*x[0]**o+(0.114070742237)*x[0]+(0.968707420077)*x[1]**o+(0.897709784012)*x[1]
        s[3,1,2,0]=(-0.774958662409)*x[0]**o+(0.712770979439)*x[0]+(0.825889537824)*x[1]**o+(0.73040987218)*x[1]
        s[3,1,2,1]=(0.412550829295)*x[0]**o+(-0.861820540337)*x[0]+(-0.64322990825)*x[1]**o+(0.262526540471)*x[1]
        s[3,2,0,0]=(-0.543691366072)*x[0]**o+(0.685553656644)*x[0]+(0.65968803389)*x[1]**o+(0.664750334284)*x[1]
        s[3,2,0,1]=(0.428045943755)*x[0]**o+(-0.140247368953)*x[0]+(0.380175178975)*x[1]**o+(-0.467518419036)*x[1]
        s[3,2,1,0]=(-0.389824098536)*x[0]**o+(-0.276525222589)*x[0]+(0.147155465185)*x[1]**o+(0.976584972227)*x[1]
        s[3,2,1,1]=(-0.801779406727)*x[0]**o+(-0.265379451805)*x[0]+(-0.515076806975)*x[1]**o+(0.436497310193)*x[1]
        s[3,2,2,0]=(0.484267483123)*x[0]**o+(0.17147902302)*x[0]+(0.386200975176)*x[1]**o+(-0.467212146867)*x[1]
        s[3,2,2,1]=(0.508871274459)*x[0]**o+(-0.819775442961)*x[0]+(0.662298902936)*x[1]**o+(-0.680488821518)*x[1]
        s[3,3,0,0]=(-0.268312881659)*x[0]**o+(0.879490107736)*x[0]+(-0.0144262822452)*x[1]**o+(-0.71519934726)*x[1]
        s[3,3,0,1]=(-0.790430004852)*x[0]**o+(-0.532618460762)*x[0]+(0.253908480901)*x[1]**o+(-0.909634079731)*x[1]
        s[3,3,1,0]=(0.0137346713747)*x[0]**o+(-0.385222519657)*x[0]+(-0.217309505922)*x[1]**o+(-0.488231851176)*x[1]
        s[3,3,1,1]=(-0.0284395764434)*x[0]**o+(0.604128820394)*x[0]+(0.459487518143)*x[1]**o+(0.0415640328609)*x[1]
        s[3,3,2,0]=(-0.162643181454)*x[0]**o+(0.998506598628)*x[0]+(0.772502162236)*x[1]**o+(0.0667163114862)*x[1]
        s[3,3,2,1]=(0.623630922274)*x[0]**o+(0.148711722927)*x[0]+(-0.161606838558)*x[1]**o+(0.156709490526)*x[1]
        s[3,4,0,0]=(-0.30946349425)*x[0]**o+(0.0987044449972)*x[0]+(0.620328824713)*x[1]**o+(0.212795150211)*x[1]
        s[3,4,0,1]=(-0.431480440685)*x[0]**o+(-0.000853814120948)*x[0]+(0.322994919175)*x[1]**o+(-0.444398394184)*x[1]
        s[3,4,1,0]=(0.768816881062)*x[0]**o+(-0.148697199679)*x[0]+(0.676778413386)*x[1]**o+(0.58721092116)*x[1]
        s[3,4,1,1]=(0.642322282841)*x[0]**o+(-0.863985751359)*x[0]+(-0.030305847597)*x[1]**o+(0.16420798613)*x[1]
        s[3,4,2,0]=(-0.652491180967)*x[0]**o+(-0.455511531309)*x[0]+(0.739317807968)*x[1]**o+(-0.299398746891)*x[1]
        s[3,4,2,1]=(-0.934623810452)*x[0]**o+(0.0755256471511)*x[0]+(-0.573554337332)*x[1]**o+(0.940012415898)*x[1]
        ref[0,0,0,0]=(0.246692793786)/(o+1.)+(-0.121284353373)+(0.0390601545717)*0.5**o
        ref[0,0,0,1]=(-0.986136095398)/(o+1.)+(0.504511759262)+(0.608816949634)*0.5**o
        ref[0,0,1,0]=(0.725705948245)/(o+1.)+(-0.453772243536)+(0.580621174141)*0.5**o
        ref[0,0,1,1]=(-0.376543304375)/(o+1.)+(-0.274154329162)+(0.388567471102)*0.5**o
        ref[0,0,2,0]=(-0.088525361736)/(o+1.)+(-0.24716896632)+(-0.692988033922)*0.5**o
        ref[0,0,2,1]=(-0.380234357131)/(o+1.)+(0.21389123247)+(0.274277434778)*0.5**o
        ref[0,1,0,0]=(0.335411545595)/(o+1.)+(0.1029286996)+(-0.802394756916)*0.5**o
        ref[0,1,0,1]=(0.178161299328)/(o+1.)+(-0.315511211672)+(0.174271017656)*0.5**o
        ref[0,1,1,0]=(0.589565043365)/(o+1.)+(-0.127948597487)+(-0.598879030318)*0.5**o
        ref[0,1,1,1]=(-0.743341570183)/(o+1.)+(0.456387470249)+(-0.667493955405)*0.5**o
        ref[0,1,2,0]=(0.899075872977)/(o+1.)+(0.0414909142386)+(0.47631386081)*0.5**o
        ref[0,1,2,1]=(-0.541353591572)/(o+1.)+(0.0382726489972)+(-0.497833539416)*0.5**o
        ref[0,2,0,0]=(-0.697588210675)/(o+1.)+(0.183020036152)+(-0.735607995943)*0.5**o
        ref[0,2,0,1]=(-0.630771158486)/(o+1.)+(0.0928497225723)+(0.705776536011)*0.5**o
        ref[0,2,1,0]=(0.315121166158)/(o+1.)+(-0.193103375068)+(-0.975134219189)*0.5**o
        ref[0,2,1,1]=(-0.471199351714)/(o+1.)+(0.510425622306)+(-0.43751086407)*0.5**o
        ref[0,2,2,0]=(0.433608622434)/(o+1.)+(-0.16152942391)+(0.526770705207)*0.5**o
        ref[0,2,2,1]=(0.0493546850323)/(o+1.)+(-0.142851309637)+(-0.418834628035)*0.5**o
        ref[0,3,0,0]=(0.235788766456)/(o+1.)+(-0.0385269918575)+(0.302691082154)*0.5**o
        ref[0,3,0,1]=(0.653142282067)/(o+1.)+(0.0517575103062)+(-0.400507082213)*0.5**o
        ref[0,3,1,0]=(-0.0222151576861)/(o+1.)+(-0.650529960575)+(-0.542123872101)*0.5**o
        ref[0,3,1,1]=(-0.977561971438)/(o+1.)+(0.325073643158)+(0.0629454961983)*0.5**o
        ref[0,3,2,0]=(0.094320839406)/(o+1.)+(-0.189684900712)+(0.088224801816)*0.5**o
        ref[0,3,2,1]=(-0.387883052017)/(o+1.)+(-0.579963217227)+(0.976917465786)*0.5**o
        ref[0,4,0,0]=(-0.308997447406)/(o+1.)+(-0.656691345314)+(-0.468335475351)*0.5**o
        ref[0,4,0,1]=(-0.978900933671)/(o+1.)+(0.0484384998298)+(0.069233902558)*0.5**o
        ref[0,4,1,0]=(-0.687749538927)/(o+1.)+(0.215451745029)+(0.192177279681)*0.5**o
        ref[0,4,1,1]=(-0.32776980948)/(o+1.)+(0.243392671122)+(0.336586630924)*0.5**o
        ref[0,4,2,0]=(0.184107273061)/(o+1.)+(-0.398587065115)+(0.635240845083)*0.5**o
        ref[0,4,2,1]=(0.657049172645)/(o+1.)+(0.712069836271)+(0.392781256576)*0.5**o
        ref[1,0,0,0]=(0.379822929854)/(o+1.)+(-0.014644925862)+(0.188456710847)*0.5**o
        ref[1,0,0,1]=(0.339165812738)/(o+1.)+(0.164264330852)+(-0.910686394304)*0.5**o
        ref[1,0,1,0]=(0.473665399577)/(o+1.)+(-0.778914192033)+(0.0133563994025)*0.5**o
        ref[1,0,1,1]=(0.46369841557)/(o+1.)+(-0.3577658358)+(0.19107514064)*0.5**o
        ref[1,0,2,0]=(0.128452495995)/(o+1.)+(-0.655264678285)+(0.179414794128)*0.5**o
        ref[1,0,2,1]=(0.244339731125)/(o+1.)+(0.029405493035)+(0.507586792052)*0.5**o
        ref[1,1,0,0]=(0.508215404195)/(o+1.)+(-0.71771496227)+(0.0515340520755)*0.5**o
        ref[1,1,0,1]=(-0.0719243226652)/(o+1.)+(-0.373364708338)+(0.149995895729)*0.5**o
        ref[1,1,1,0]=(0.0457980649267)/(o+1.)+(0.321315986156)+(-0.298918981626)*0.5**o
        ref[1,1,1,1]=(0.103572135419)/(o+1.)+(0.320470192342)+(-0.833495545848)*0.5**o
        ref[1,1,2,0]=(-0.785664720197)/(o+1.)+(0.728283801948)+(0.381644351387)*0.5**o
        ref[1,1,2,1]=(0.0749818396161)/(o+1.)+(0.0116600896011)+(0.589726138962)*0.5**o
        ref[1,2,0,0]=(0.714248701071)/(o+1.)+(0.215510313526)+(0.648262414915)*0.5**o
        ref[1,2,0,1]=(0.186782608772)/(o+1.)+(0.348222715845)+(0.154336156671)*0.5**o
        ref[1,2,1,0]=(-0.159116807039)/(o+1.)+(-0.36655730667)+(0.631907675013)*0.5**o
        ref[1,2,1,1]=(0.995323329778)/(o+1.)+(0.492189332195)+(-0.150281595818)*0.5**o
        ref[1,2,2,0]=(0.945042808379)/(o+1.)+(0.272493717349)+(0.15122501321)*0.5**o
        ref[1,2,2,1]=(0.518228280852)/(o+1.)+(0.354601257252)+(0.490853599647)*0.5**o
        ref[1,3,0,0]=(-0.99302787628)/(o+1.)+(-0.0822922603983)+(-0.868069831372)*0.5**o
        ref[1,3,0,1]=(-0.691622353639)/(o+1.)+(-0.156892406228)+(-0.540392753009)*0.5**o
        ref[1,3,1,0]=(0.239758390058)/(o+1.)+(0.621160605983)+(-0.38171518034)*0.5**o
        ref[1,3,1,1]=(0.875356104271)/(o+1.)+(0.721348416703)+(0.405015772479)*0.5**o
        ref[1,3,2,0]=(0.652544093354)/(o+1.)+(-0.450393922473)+(-0.859288591023)*0.5**o
        ref[1,3,2,1]=(-0.266920992105)/(o+1.)+(-0.549660956967)+(-0.385753789824)*0.5**o
        ref[1,4,0,0]=(-0.974008625555)/(o+1.)+(-0.000140651795392)+(0.248320342844)*0.5**o
        ref[1,4,0,1]=(-0.890410564837)/(o+1.)+(-0.664020783456)+(-0.408988831595)*0.5**o
        ref[1,4,1,0]=(0.272797445173)/(o+1.)+(-0.382936964587)+(-0.637267396746)*0.5**o
        ref[1,4,1,1]=(-0.0849564635014)/(o+1.)+(0.204266707603)+(-0.711284122459)*0.5**o
        ref[1,4,2,0]=(0.396913284632)/(o+1.)+(0.484403579059)+(0.194861831786)*0.5**o
        ref[1,4,2,1]=(0.954568821795)/(o+1.)+(0.254563359637)+(-0.111812571843)*0.5**o
        ref[2,0,0,0]=(0.66590243332)/(o+1.)+(0.0148566298797)+(0.149006416722)*0.5**o
        ref[2,0,0,1]=(-0.115430427081)/(o+1.)+(-0.527231128141)+(-0.788828642343)*0.5**o
        ref[2,0,1,0]=(-0.247117069506)/(o+1.)+(-0.569443533242)+(0.598185590774)*0.5**o
        ref[2,0,1,1]=(-0.628357637608)/(o+1.)+(-0.337747901042)+(-0.945230420194)*0.5**o
        ref[2,0,2,0]=(-0.43734992018)/(o+1.)+(-0.7606913535)+(0.184549704327)*0.5**o
        ref[2,0,2,1]=(0.926088436529)/(o+1.)+(-0.070253895001)+(-0.897896267578)*0.5**o
        ref[2,1,0,0]=(0.163113306006)/(o+1.)+(-0.161302341569)+(-0.0375376932325)*0.5**o
        ref[2,1,0,1]=(0.762838138451)/(o+1.)+(0.278846281935)+(-0.603507455453)*0.5**o
        ref[2,1,1,0]=(0.954063036094)/(o+1.)+(0.362930981906)+(-0.250121932289)*0.5**o
        ref[2,1,1,1]=(-0.932724945971)/(o+1.)+(0.469122515475)+(0.0147284052497)*0.5**o
        ref[2,1,2,0]=(-0.648739038507)/(o+1.)+(-0.593425752905)+(-0.832131948289)*0.5**o
        ref[2,1,2,1]=(-0.399288944386)/(o+1.)+(-0.112449079426)+(-0.809460903823)*0.5**o
        ref[2,2,0,0]=(-0.146472829559)/(o+1.)+(0.363381673756)+(0.851821921349)*0.5**o
        ref[2,2,0,1]=(-0.262458059774)/(o+1.)+(-0.701238793074)+(0.145872642324)*0.5**o
        ref[2,2,1,0]=(-0.941540320886)/(o+1.)+(-0.124851084374)+(-0.97150697777)*0.5**o
        ref[2,2,1,1]=(0.725341857111)/(o+1.)+(-0.407470559879)+(0.705001677542)*0.5**o
        ref[2,2,2,0]=(-0.797706422949)/(o+1.)+(0.459343748206)+(-0.405826604429)*0.5**o
        ref[2,2,2,1]=(-0.381859891282)/(o+1.)+(-0.425569065022)+(-0.338189348522)*0.5**o
        ref[2,3,0,0]=(0.306014483173)/(o+1.)+(-0.656039625864)+(-0.532343324478)*0.5**o
        ref[2,3,0,1]=(0.939876357973)/(o+1.)+(-0.0146231237877)+(0.384975538101)*0.5**o
        ref[2,3,1,0]=(0.929829286228)/(o+1.)+(0.234747366357)+(0.39221893978)*0.5**o
        ref[2,3,1,1]=(0.207625492571)/(o+1.)+(-0.174250601546)+(-0.706521549808)*0.5**o
        ref[2,3,2,0]=(-0.463914786644)/(o+1.)+(0.046924621609)+(0.464449153296)*0.5**o
        ref[2,3,2,1]=(0.265758298887)/(o+1.)+(0.242967856537)+(-0.755612237619)*0.5**o
        ref[2,4,0,0]=(0.347084253405)/(o+1.)+(0.301061654767)+(-0.913101029308)*0.5**o
        ref[2,4,0,1]=(0.870579357546)/(o+1.)+(-0.499559218329)+(-0.261381262076)*0.5**o
        ref[2,4,1,0]=(0.414206585164)/(o+1.)+(0.696094746795)+(-0.128568967272)*0.5**o
        ref[2,4,1,1]=(0.681628131999)/(o+1.)+(0.380014241257)+(0.924901584611)*0.5**o
        ref[2,4,2,0]=(0.577674551804)/(o+1.)+(-0.440344797207)+(0.409615949394)*0.5**o
        ref[2,4,2,1]=(-0.592574898322)/(o+1.)+(-0.120224474883)+(0.382356012027)*0.5**o
        ref[3,0,0,0]=(-0.666690913098)/(o+1.)+(-0.54955659901)+(-0.254267267009)*0.5**o
        ref[3,0,0,1]=(-0.431014198736)/(o+1.)+(-0.906606407206)+(-0.672935968283)*0.5**o
        ref[3,0,1,0]=(0.768197508919)/(o+1.)+(-0.128485639754)+(0.198586757881)*0.5**o
        ref[3,0,1,1]=(0.64745394627)/(o+1.)+(0.242433768937)+(0.835596691179)*0.5**o
        ref[3,0,2,0]=(0.478363486403)/(o+1.)+(0.35705399992)+(-0.60108209247)*0.5**o
        ref[3,0,2,1]=(-0.0139161976239)/(o+1.)+(-0.0353334614921)+(-0.632653025461)*0.5**o
        ref[3,1,0,0]=(-0.894286771395)/(o+1.)+(-0.0664875936996)+(-0.216238961155)*0.5**o
        ref[3,1,0,1]=(-0.191303802435)/(o+1.)+(-0.681161760631)+(0.453465128819)*0.5**o
        ref[3,1,1,0]=(0.128347107042)/(o+1.)+(-0.308977366639)+(-0.371601120889)*0.5**o
        ref[3,1,1,1]=(0.968707420077)/(o+1.)+(0.505890263125)+(-0.492741845463)*0.5**o
        ref[3,1,2,0]=(0.825889537824)/(o+1.)+(0.72159042581)+(-0.774958662409)*0.5**o
        ref[3,1,2,1]=(-0.64322990825)/(o+1.)+(-0.299646999933)+(0.412550829295)*0.5**o
        ref[3,2,0,0]=(0.65968803389)/(o+1.)+(0.675151995464)+(-0.543691366072)*0.5**o
        ref[3,2,0,1]=(0.380175178975)/(o+1.)+(-0.303882893994)+(0.428045943755)*0.5**o
        ref[3,2,1,0]=(0.147155465185)/(o+1.)+(0.350029874819)+(-0.389824098536)*0.5**o
        ref[3,2,1,1]=(-0.515076806975)/(o+1.)+(0.0855589291935)+(-0.801779406727)*0.5**o
        ref[3,2,2,0]=(0.386200975176)/(o+1.)+(-0.147866561923)+(0.484267483123)*0.5**o
        ref[3,2,2,1]=(0.662298902936)/(o+1.)+(-0.75013213224)+(0.508871274459)*0.5**o
        ref[3,3,0,0]=(-0.0144262822452)/(o+1.)+(0.0821453802379)+(-0.268312881659)*0.5**o
        ref[3,3,0,1]=(0.253908480901)/(o+1.)+(-0.721126270246)+(-0.790430004852)*0.5**o
        ref[3,3,1,0]=(-0.217309505922)/(o+1.)+(-0.436727185417)+(0.0137346713747)*0.5**o
        ref[3,3,1,1]=(0.459487518143)/(o+1.)+(0.322846426627)+(-0.0284395764434)*0.5**o
        ref[3,3,2,0]=(0.772502162236)/(o+1.)+(0.532611455057)+(-0.162643181454)*0.5**o
        ref[3,3,2,1]=(-0.161606838558)/(o+1.)+(0.152710606726)+(0.623630922274)*0.5**o
        ref[3,4,0,0]=(0.620328824713)/(o+1.)+(0.155749797604)+(-0.30946349425)*0.5**o
        ref[3,4,0,1]=(0.322994919175)/(o+1.)+(-0.222626104153)+(-0.431480440685)*0.5**o
        ref[3,4,1,0]=(0.676778413386)/(o+1.)+(0.21925686074)+(0.768816881062)*0.5**o
        ref[3,4,1,1]=(-0.030305847597)/(o+1.)+(-0.349888882614)+(0.642322282841)*0.5**o
        ref[3,4,2,0]=(0.739317807968)/(o+1.)+(-0.3774551391)+(-0.652491180967)*0.5**o
        ref[3,4,2,1]=(-0.573554337332)/(o+1.)+(0.507769031524)+(-0.934623810452)*0.5**o
      else:
        s[0,0,0,0]=(-0.284157309093)*x[0]**o+(0.849767107542)*x[0]+(-0.384722111658)*x[1]**o+(-0.422005634353)*x[1]+(-0.650336986527)*x[2]**o+(-0.512697943529)*x[2]
        s[0,0,0,1]=(-0.315769534489)*x[0]**o+(0.535386332325)*x[0]+(0.853804045467)*x[1]**o+(-0.547748632318)*x[1]+(0.346701407152)*x[2]**o+(0.37814849359)*x[2]
        s[0,0,1,0]=(-0.4535776613)*x[0]**o+(-0.510530838778)*x[0]+(0.276827299966)*x[1]**o+(-0.382689005507)*x[1]+(0.905088915885)*x[2]**o+(0.0949553680219)*x[2]
        s[0,0,1,1]=(-0.739956684686)*x[0]**o+(0.461075107945)*x[0]+(-0.669109047898)*x[1]**o+(0.894371361493)*x[1]+(-0.67186417627)*x[2]**o+(0.646865110437)*x[2]
        s[0,0,2,0]=(0.630514031324)*x[0]**o+(-0.200697710807)*x[0]+(-0.425713236098)*x[1]**o+(0.990076474527)*x[1]+(0.799964278476)*x[2]**o+(-0.11090726662)*x[2]
        s[0,0,2,1]=(-0.763048729267)*x[0]**o+(0.108312132849)*x[0]+(-0.0743278550192)*x[1]**o+(-0.00609761227579)*x[1]+(0.856275832244)*x[2]**o+(-0.271931694818)*x[2]
        s[0,1,0,0]=(-0.438048883736)*x[0]**o+(0.0995162169923)*x[0]+(0.991935495829)*x[1]**o+(0.10168563116)*x[1]+(0.61021228041)*x[2]**o+(0.935697636412)*x[2]
        s[0,1,0,1]=(0.169577859304)*x[0]**o+(-0.32382927444)*x[0]+(-0.416050245653)*x[1]**o+(-0.218068414951)*x[1]+(-0.828154819878)*x[2]**o+(0.510133764467)*x[2]
        s[0,1,1,0]=(-0.136877583438)*x[0]**o+(0.225702373679)*x[0]+(-0.982330481939)*x[1]**o+(-0.76035657944)*x[1]+(-0.75008234946)*x[2]**o+(0.34026332188)*x[2]
        s[0,1,1,1]=(0.21329287668)*x[0]**o+(-0.583080810655)*x[0]+(0.304728874482)*x[1]**o+(-0.513735666156)*x[1]+(0.678531278973)*x[2]**o+(0.229523450267)*x[2]
        s[0,1,2,0]=(-0.488721257593)*x[0]**o+(-0.623065480166)*x[0]+(-0.689240830032)*x[1]**o+(0.0487255710714)*x[1]+(-0.345860672549)*x[2]**o+(-0.839070113015)*x[2]
        s[0,1,2,1]=(0.0407718763863)*x[0]**o+(-0.73486237198)*x[0]+(-0.430375782223)*x[1]**o+(0.937703487997)*x[1]+(0.211905262532)*x[2]**o+(0.436342043473)*x[2]
        s[0,2,0,0]=(0.587345517354)*x[0]**o+(-0.592886902091)*x[0]+(0.552036449839)*x[1]**o+(0.623893456691)*x[1]+(0.0732753243278)*x[2]**o+(0.0126745042435)*x[2]
        s[0,2,0,1]=(-0.811084805765)*x[0]**o+(0.535274295321)*x[0]+(0.24492936339)*x[1]**o+(-0.800440152287)*x[1]+(0.631212521526)*x[2]**o+(-0.447237919067)*x[2]
        s[0,2,1,0]=(-0.222984108172)*x[0]**o+(0.666955781431)*x[0]+(-0.793761886587)*x[1]**o+(0.767105994377)*x[1]+(0.106607736355)*x[2]**o+(0.107909765632)*x[2]
        s[0,2,1,1]=(-0.351616621125)*x[0]**o+(0.918653285574)*x[0]+(0.374186423556)*x[1]**o+(0.00935909197596)*x[1]+(0.40967652737)*x[2]**o+(0.85098748057)*x[2]
        s[0,2,2,0]=(0.326986640454)*x[0]**o+(-0.922098818053)*x[0]+(0.180209899234)*x[1]**o+(-0.473771920593)*x[1]+(0.850114791793)*x[2]**o+(-0.0591994178493)*x[2]
        s[0,2,2,1]=(0.416408828986)*x[0]**o+(-0.802940087419)*x[0]+(-0.15695987973)*x[1]**o+(-0.212571438106)*x[1]+(-0.695411431279)*x[2]**o+(0.652970553889)*x[2]
        s[0,3,0,0]=(0.165546039968)*x[0]**o+(0.510754381868)*x[0]+(0.922367777435)*x[1]**o+(0.946960595929)*x[1]+(0.488750467594)*x[2]**o+(0.684670926921)*x[2]
        s[0,3,0,1]=(0.806943353866)*x[0]**o+(-0.0452688268514)*x[0]+(-0.915858505648)*x[1]**o+(-0.930357118958)*x[1]+(-0.915989552667)*x[2]**o+(-0.058763902318)*x[2]
        s[0,3,1,0]=(-0.0329058672137)*x[0]**o+(-0.347664127646)*x[0]+(-0.642805650766)*x[1]**o+(-0.772181465981)*x[1]+(-0.489006714362)*x[2]**o+(-0.92809698455)*x[2]
        s[0,3,1,1]=(0.927391314677)*x[0]**o+(-0.00818607467373)*x[0]+(-0.25019363274)*x[1]**o+(-0.944005917142)*x[1]+(0.55451200174)*x[2]**o+(0.97132400122)*x[2]
        s[0,3,2,0]=(-0.395900087683)*x[0]**o+(-0.373500614568)*x[0]+(-0.292146120482)*x[1]**o+(0.481366513649)*x[1]+(0.793324319035)*x[2]**o+(-0.0829523908121)*x[2]
        s[0,3,2,1]=(0.167004592732)*x[0]**o+(-0.412448920865)*x[0]+(-0.898556558084)*x[1]**o+(-0.490869248138)*x[1]+(-0.567143223313)*x[2]**o+(-0.732576896378)*x[2]
        s[0,4,0,0]=(-0.0559473269152)*x[0]**o+(0.927731889097)*x[0]+(-0.650788921118)*x[1]**o+(0.9760762788)*x[1]+(0.115672920967)*x[2]**o+(-0.276468719613)*x[2]
        s[0,4,0,1]=(-0.126102715426)*x[0]**o+(0.948204367434)*x[0]+(-0.337377224065)*x[1]**o+(-0.66690062976)*x[1]+(-0.0486873976343)*x[2]**o+(0.731627670267)*x[2]
        s[0,4,1,0]=(-0.465158745622)*x[0]**o+(0.825435774485)*x[0]+(-0.325537513647)*x[1]**o+(-0.514033669128)*x[1]+(0.561307474551)*x[2]**o+(0.703487324789)*x[2]
        s[0,4,1,1]=(0.433494572203)*x[0]**o+(-0.193734534196)*x[0]+(-0.285490592547)*x[1]**o+(0.7933219877)*x[1]+(-0.968093339339)*x[2]**o+(0.891238901252)*x[2]
        s[0,4,2,0]=(-0.226388318689)*x[0]**o+(0.354588341815)*x[0]+(-0.592835636922)*x[1]**o+(0.164285456556)*x[1]+(0.969440431522)*x[2]**o+(0.942251489525)*x[2]
        s[0,4,2,1]=(-0.487005215247)*x[0]**o+(-0.363562477474)*x[0]+(-0.968419416695)*x[1]**o+(0.0746396954211)*x[1]+(0.0996297479628)*x[2]**o+(-0.977105194209)*x[2]
        s[1,0,0,0]=(-0.599811357812)*x[0]**o+(0.379322984555)*x[0]+(-0.263738481625)*x[1]**o+(0.737645780912)*x[1]+(0.579734520525)*x[2]**o+(0.674309085344)*x[2]
        s[1,0,0,1]=(-0.849384033331)*x[0]**o+(0.835916113742)*x[0]+(-0.95744797963)*x[1]**o+(-0.140222671452)*x[1]+(-0.885202051612)*x[2]**o+(-0.214627624347)*x[2]
        s[1,0,1,0]=(0.560384598535)*x[0]**o+(0.521038386588)*x[0]+(0.405346505288)*x[1]**o+(0.325623245692)*x[1]+(-0.499493979476)*x[2]**o+(0.457353575559)*x[2]
        s[1,0,1,1]=(0.394452679675)*x[0]**o+(-0.335426315796)*x[0]+(0.366492975013)*x[1]**o+(-0.204908992368)*x[1]+(0.72928619323)*x[2]**o+(0.868856537052)*x[2]
        s[1,0,2,0]=(-0.286588291759)*x[0]**o+(-0.457077770862)*x[0]+(0.48947586799)*x[1]**o+(-0.483134695964)*x[1]+(-0.708198861531)*x[2]**o+(-0.185141468661)*x[2]
        s[1,0,2,1]=(0.587814763664)*x[0]**o+(0.369493163541)*x[0]+(0.973921527395)*x[1]**o+(0.792281782334)*x[1]+(-0.622350476528)*x[2]**o+(-0.0910812853654)*x[2]
        s[1,1,0,0]=(-0.0104862086098)*x[0]**o+(0.636242048755)*x[0]+(-0.831278249993)*x[1]**o+(0.789497876069)*x[1]+(-0.14996840164)*x[2]**o+(0.560466658811)*x[2]
        s[1,1,0,1]=(-0.442153331955)*x[0]**o+(0.259117434312)*x[0]+(0.624678092445)*x[1]**o+(-0.95353370493)*x[1]+(-0.647048223519)*x[2]**o+(-0.264328530894)*x[2]
        s[1,1,1,0]=(0.143778546111)*x[0]**o+(-0.0162780042157)*x[0]+(0.11561710021)*x[1]**o+(-0.750325462664)*x[1]+(0.0513768047109)*x[2]**o+(0.605731207893)*x[2]
        s[1,1,1,1]=(0.830078999256)*x[0]**o+(-0.977563707962)*x[0]+(-0.0694815703262)*x[1]**o+(-0.631908693739)*x[1]+(-0.385240583095)*x[2]**o+(-0.325723419389)*x[2]
        s[1,1,2,0]=(-0.818875997493)*x[0]**o+(0.552071308369)*x[0]+(0.42469064565)*x[1]**o+(-0.939401594531)*x[1]+(-0.704918072568)*x[2]**o+(-0.412588503126)*x[2]
        s[1,1,2,1]=(0.897735820104)*x[0]**o+(-0.229171182759)*x[0]+(-0.509740516596)*x[1]**o+(-0.182079029613)*x[1]+(-0.935655203228)*x[2]**o+(0.0659300001682)*x[2]
        s[1,2,0,0]=(-0.225661260024)*x[0]**o+(-0.705274531847)*x[0]+(0.851323519708)*x[1]**o+(0.112877579527)*x[1]+(-0.622263839274)*x[2]**o+(0.0332240292837)*x[2]
        s[1,2,0,1]=(0.188146461111)*x[0]**o+(-0.401949407666)*x[0]+(0.600435405952)*x[1]**o+(-0.205690272021)*x[1]+(-0.715745492382)*x[2]**o+(-0.842790491589)*x[2]
        s[1,2,1,0]=(-0.713236970029)*x[0]**o+(-0.261492144857)*x[0]+(-0.154448102633)*x[1]**o+(-0.970544645841)*x[1]+(-0.920289620236)*x[2]**o+(-0.97207996703)*x[2]
        s[1,2,1,1]=(0.338343830055)*x[0]**o+(0.245579659207)*x[0]+(0.0867267737353)*x[1]**o+(-0.898055915558)*x[1]+(-0.0942524838856)*x[2]**o+(-0.935035704882)*x[2]
        s[1,2,2,0]=(-0.857754750211)*x[0]**o+(-0.491056643232)*x[0]+(-0.869773943254)*x[1]**o+(-0.4760062246)*x[1]+(-0.684952853861)*x[2]**o+(-0.576097444445)*x[2]
        s[1,2,2,1]=(0.798299933694)*x[0]**o+(0.0610709183894)*x[0]+(-0.486420716864)*x[1]**o+(0.0416959278242)*x[1]+(-0.928888319279)*x[2]**o+(0.854902913513)*x[2]
        s[1,3,0,0]=(-0.22936580714)*x[0]**o+(0.708863403011)*x[0]+(0.0460624260266)*x[1]**o+(-0.49429349985)*x[1]+(-0.65165177736)*x[2]**o+(-0.640993879589)*x[2]
        s[1,3,0,1]=(0.272143747738)*x[0]**o+(-0.786035898193)*x[0]+(-0.907839432563)*x[1]**o+(-0.544282319144)*x[1]+(-0.295284311966)*x[2]**o+(-0.646933800599)*x[2]
        s[1,3,1,0]=(-0.371813536068)*x[0]**o+(-0.989821491808)*x[0]+(0.87663418945)*x[1]**o+(0.49557916163)*x[1]+(0.432075714902)*x[2]**o+(0.986679642413)*x[2]
        s[1,3,1,1]=(0.64355520606)*x[0]**o+(0.849290029179)*x[0]+(-0.588615332307)*x[1]**o+(-0.43586534156)*x[1]+(0.440278053071)*x[2]**o+(0.71924945995)*x[2]
        s[1,3,2,0]=(0.296777344281)*x[0]**o+(0.369261091311)*x[0]+(-0.542374725062)*x[1]**o+(-0.378169086763)*x[1]+(-0.376253518198)*x[2]**o+(-0.0261035492256)*x[2]
        s[1,3,2,1]=(-0.631360619387)*x[0]**o+(-0.447767234553)*x[0]+(-0.907807578651)*x[1]**o+(0.0949745118593)*x[1]+(-0.467696925918)*x[2]**o+(0.328688763502)*x[2]
        s[1,4,0,0]=(-0.864162318969)*x[0]**o+(0.0492392548816)*x[0]+(-0.443732835548)*x[1]**o+(0.216532788053)*x[1]+(-0.589754138443)*x[2]**o+(0.225238430912)*x[2]
        s[1,4,0,1]=(-0.0204110610831)*x[0]**o+(-0.361198283837)*x[0]+(-0.534453313822)*x[1]**o+(0.199660564502)*x[1]+(0.76073237233)*x[2]**o+(-0.397852751275)*x[2]
        s[1,4,1,0]=(-0.0714182037356)*x[0]**o+(-0.838354030583)*x[0]+(-0.690623231638)*x[1]**o+(0.718400117263)*x[1]+(0.81989629515)*x[2]**o+(-0.99784667739)*x[2]
        s[1,4,1,1]=(-0.789159638733)*x[0]**o+(0.0550782963202)*x[0]+(0.691693836327)*x[1]**o+(-0.995863220343)*x[1]+(0.489187641363)*x[2]**o+(-0.892540698263)*x[2]
        s[1,4,2,0]=(-0.545542033256)*x[0]**o+(0.534307451322)*x[0]+(0.164514904761)*x[1]**o+(-0.604639193552)*x[1]+(0.299777195543)*x[2]**o+(0.103353434856)*x[2]
        s[1,4,2,1]=(0.655205311592)*x[0]**o+(-0.806735930884)*x[0]+(-0.808306407454)*x[1]**o+(-0.842421999068)*x[1]+(0.071496057035)*x[2]**o+(0.327827774456)*x[2]
        s[2,0,0,0]=(-0.0509015638697)*x[0]**o+(0.122893306456)*x[0]+(-0.495264442266)*x[1]**o+(-0.700582978791)*x[1]+(0.912958594653)*x[2]**o+(0.396603396384)*x[2]
        s[2,0,0,1]=(-0.834927535726)*x[0]**o+(0.0299018772298)*x[0]+(0.950983785443)*x[1]**o+(-0.467301669211)*x[1]+(0.105169729215)*x[2]**o+(0.776869887731)*x[2]
        s[2,0,1,0]=(0.695941957197)*x[0]**o+(-0.0827202988544)*x[0]+(0.551038863364)*x[1]**o+(0.768922568953)*x[1]+(0.658453649199)*x[2]**o+(0.0845854555463)*x[2]
        s[2,0,1,1]=(-0.813515499086)*x[0]**o+(0.37498311883)*x[0]+(0.249460504439)*x[1]**o+(0.57859274713)*x[1]+(0.674876282575)*x[2]**o+(0.358162640602)*x[2]
        s[2,0,2,0]=(0.264441597574)*x[0]**o+(-0.773282762615)*x[0]+(0.548708830043)*x[1]**o+(-0.365859504864)*x[1]+(-0.971213462476)*x[2]**o+(0.341839165432)*x[2]
        s[2,0,2,1]=(0.839834410476)*x[0]**o+(0.73258814656)*x[0]+(0.0894113621016)*x[1]**o+(-0.561436557466)*x[1]+(-0.87929169767)*x[2]**o+(-0.673343626233)*x[2]
        s[2,1,0,0]=(-0.9032683394)*x[0]**o+(0.548037699973)*x[0]+(-0.275249518695)*x[1]**o+(-0.182173933698)*x[1]+(-0.797774902804)*x[2]**o+(-0.541398445737)*x[2]
        s[2,1,0,1]=(-0.563127063275)*x[0]**o+(0.0246839329162)*x[0]+(0.104775018182)*x[1]**o+(0.887831444063)*x[1]+(-0.21494247853)*x[2]**o+(-0.273454823602)*x[2]
        s[2,1,1,0]=(-0.0215129450025)*x[0]**o+(-0.388850056041)*x[0]+(0.286605493305)*x[1]**o+(0.0744061944979)*x[1]+(-0.745577158797)*x[2]**o+(0.465555807335)*x[2]
        s[2,1,1,1]=(0.862938112821)*x[0]**o+(0.440536762323)*x[0]+(0.708402486014)*x[1]**o+(-0.330067981061)*x[1]+(0.212443131768)*x[2]**o+(0.604427083557)*x[2]
        s[2,1,2,0]=(-0.151555342557)*x[0]**o+(-0.422137938709)*x[0]+(0.725618931526)*x[1]**o+(0.965446678276)*x[1]+(-0.782758787615)*x[2]**o+(-0.913406963513)*x[2]
        s[2,1,2,1]=(0.485262168703)*x[0]**o+(-0.155309809717)*x[0]+(0.346371477276)*x[1]**o+(-0.25747778813)*x[1]+(0.29003119075)*x[2]**o+(-0.994256278631)*x[2]
        s[2,2,0,0]=(0.597629111376)*x[0]**o+(-0.675075309746)*x[0]+(-0.276946615105)*x[1]**o+(0.474817826479)*x[1]+(0.384217707699)*x[2]**o+(0.370453233617)*x[2]
        s[2,2,0,1]=(0.0885418745402)*x[0]**o+(0.723684895162)*x[0]+(0.380828880196)*x[1]**o+(-0.200773722805)*x[1]+(-0.72048497776)*x[2]**o+(0.627781439124)*x[2]
        s[2,2,1,0]=(-0.7058457718)*x[0]**o+(0.605583709445)*x[0]+(-0.137932623446)*x[1]**o+(-0.355521140013)*x[1]+(-0.100629345222)*x[2]**o+(-0.0196746976828)*x[2]
        s[2,2,1,1]=(0.431391331275)*x[0]**o+(0.624577796816)*x[0]+(0.517581044445)*x[1]**o+(0.810575610489)*x[1]+(0.151026961294)*x[2]**o+(-0.116575349397)*x[2]
        s[2,2,2,0]=(-0.172742409612)*x[0]**o+(-0.525478587831)*x[0]+(-0.245244804561)*x[1]**o+(0.466265204552)*x[1]+(-0.42763207872)*x[2]**o+(-0.37841245626)*x[2]
        s[2,2,2,1]=(0.193762571076)*x[0]**o+(-0.443537851353)*x[0]+(0.703848629926)*x[1]**o+(-0.694597427026)*x[1]+(0.135855093753)*x[2]**o+(-0.845940858008)*x[2]
        s[2,3,0,0]=(-0.816708755757)*x[0]**o+(-0.181019603056)*x[0]+(-0.743213747688)*x[1]**o+(0.798697343825)*x[1]+(-0.941634970147)*x[2]**o+(0.942560075608)*x[2]
        s[2,3,0,1]=(0.557205351559)*x[0]**o+(0.573309419549)*x[0]+(0.285649088042)*x[1]**o+(-0.915772596908)*x[1]+(0.894469058714)*x[2]**o+(-0.093076259014)*x[2]
        s[2,3,1,0]=(-0.330158214652)*x[0]**o+(0.519069568809)*x[0]+(-0.584129234472)*x[1]**o+(0.508779523144)*x[1]+(-0.496105354274)*x[2]**o+(-0.119894247636)*x[2]
        s[2,3,1,1]=(0.0113498976079)*x[0]**o+(-0.817148495948)*x[0]+(-0.843322049813)*x[1]**o+(0.397509634966)*x[1]+(-0.210167599559)*x[2]**o+(0.452940815863)*x[2]
        s[2,3,2,0]=(0.548708545741)*x[0]**o+(0.701316458542)*x[0]+(0.541383513552)*x[1]**o+(0.818821487001)*x[1]+(0.0337473469103)*x[2]**o+(0.144942886826)*x[2]
        s[2,3,2,1]=(0.367699563724)*x[0]**o+(-0.558358600746)*x[0]+(0.02800962366)*x[1]**o+(0.408211655912)*x[1]+(0.808098902096)*x[2]**o+(0.406029406708)*x[2]
        s[2,4,0,0]=(0.742185577055)*x[0]**o+(0.346992242703)*x[0]+(-0.789510739504)*x[1]**o+(-0.433601148291)*x[1]+(-0.317572193313)*x[2]**o+(-0.0929823026668)*x[2]
        s[2,4,0,1]=(-0.540606064885)*x[0]**o+(0.959079111439)*x[0]+(-0.477472945875)*x[1]**o+(0.327457999632)*x[1]+(0.640838941005)*x[2]**o+(0.600082339571)*x[2]
        s[2,4,1,0]=(-0.893508919799)*x[0]**o+(0.277903035098)*x[0]+(-0.10177580179)*x[1]**o+(0.404076458963)*x[1]+(0.497606493909)*x[2]**o+(0.343868899215)*x[2]
        s[2,4,1,1]=(-0.0277942588293)*x[0]**o+(-0.341120495601)*x[0]+(0.622259478993)*x[1]**o+(0.73519393456)*x[1]+(0.660012187037)*x[2]**o+(0.399368466331)*x[2]
        s[2,4,2,0]=(0.712530777919)*x[0]**o+(0.0672439307577)*x[0]+(0.124381974602)*x[1]**o+(-0.406347903024)*x[1]+(0.0208315631234)*x[2]**o+(-0.791500976149)*x[2]
        s[2,4,2,1]=(0.0858696597238)*x[0]**o+(0.629028381896)*x[0]+(0.795442036928)*x[1]**o+(-0.764623844645)*x[1]+(-0.524125153324)*x[2]**o+(0.974625430391)*x[2]
        s[3,0,0,0]=(-0.124887079943)*x[0]**o+(-0.654604024941)*x[0]+(0.822406023517)*x[1]**o+(-0.429933379615)*x[1]+(-0.693754435109)*x[2]**o+(0.745961675595)*x[2]
        s[3,0,0,1]=(0.0781604618068)*x[0]**o+(0.481187957149)*x[0]+(0.923097062213)*x[1]**o+(0.732993925667)*x[1]+(0.0504012859604)*x[2]**o+(-0.115131599222)*x[2]
        s[3,0,1,0]=(0.822494278919)*x[0]**o+(-0.595704755199)*x[0]+(-0.194581886746)*x[1]**o+(-0.943337806993)*x[1]+(0.147117678978)*x[2]**o+(0.728101168665)*x[2]
        s[3,0,1,1]=(-0.61666199289)*x[0]**o+(-0.311296394607)*x[0]+(0.0936521091279)*x[1]**o+(0.588466331236)*x[1]+(-0.878463665908)*x[2]**o+(-0.675819143473)*x[2]
        s[3,0,2,0]=(-0.275356624889)*x[0]**o+(0.383489643489)*x[0]+(0.738708963797)*x[1]**o+(0.974422154657)*x[1]+(-0.580144378752)*x[2]**o+(-0.398505112717)*x[2]
        s[3,0,2,1]=(0.17925744427)*x[0]**o+(-0.681535969307)*x[0]+(0.902860156428)*x[1]**o+(0.871645168608)*x[1]+(-0.500401958703)*x[2]**o+(-0.817977653599)*x[2]
        s[3,1,0,0]=(0.524297360691)*x[0]**o+(0.984668939309)*x[0]+(-0.929867000718)*x[1]**o+(0.270757914061)*x[1]+(-0.259503975648)*x[2]**o+(-0.780346807445)*x[2]
        s[3,1,0,1]=(-0.261808400698)*x[0]**o+(0.0194613328439)*x[0]+(-0.113405534134)*x[1]**o+(0.734179483403)*x[1]+(0.802102544514)*x[2]**o+(0.565610343674)*x[2]
        s[3,1,1,0]=(0.134876119034)*x[0]**o+(0.431559636814)*x[0]+(0.420152427242)*x[1]**o+(-0.882232674232)*x[1]+(0.267000935164)*x[2]**o+(-0.827008708436)*x[2]
        s[3,1,1,1]=(-0.987212349574)*x[0]**o+(-0.342415633625)*x[0]+(0.639912484673)*x[1]**o+(0.790670494677)*x[1]+(0.15123106124)*x[2]**o+(0.983458455307)*x[2]
        s[3,1,2,0]=(0.742204537127)*x[0]**o+(0.886895993942)*x[0]+(-0.349125661787)*x[1]**o+(-0.626136843474)*x[1]+(0.847031359985)*x[2]**o+(0.802788970865)*x[2]
        s[3,1,2,1]=(-0.235349061749)*x[0]**o+(-0.621187306924)*x[0]+(0.552218669219)*x[1]**o+(0.790729433037)*x[1]+(0.444333479788)*x[2]**o+(0.632032201563)*x[2]
        s[3,2,0,0]=(-0.938490786208)*x[0]**o+(0.869095707989)*x[0]+(0.140982256523)*x[1]**o+(-0.878601697433)*x[1]+(0.184535860313)*x[2]**o+(0.719071828514)*x[2]
        s[3,2,0,1]=(-0.0180578829991)*x[0]**o+(0.0463975318478)*x[0]+(0.00393591018992)*x[1]**o+(0.950931246975)*x[1]+(0.615063351086)*x[2]**o+(-0.258806804322)*x[2]
        s[3,2,1,0]=(0.294998689081)*x[0]**o+(-0.625359797867)*x[0]+(-0.403789197497)*x[1]**o+(-0.999420678246)*x[1]+(0.719776724665)*x[2]**o+(0.66899881431)*x[2]
        s[3,2,1,1]=(-0.722157328967)*x[0]**o+(0.299062154378)*x[0]+(0.598625708687)*x[1]**o+(-0.951467257263)*x[1]+(0.344507050433)*x[2]**o+(0.0139170273087)*x[2]
        s[3,2,2,0]=(0.621065266638)*x[0]**o+(0.985212075414)*x[0]+(-0.698645841337)*x[1]**o+(0.660676253875)*x[1]+(-0.406319768107)*x[2]**o+(-0.307469107008)*x[2]
        s[3,2,2,1]=(0.0383194307525)*x[0]**o+(0.979670738016)*x[0]+(-0.928710294194)*x[1]**o+(0.610840841234)*x[1]+(0.707435385876)*x[2]**o+(-0.243157095062)*x[2]
        s[3,3,0,0]=(0.0357627225956)*x[0]**o+(0.708815172231)*x[0]+(0.065359531343)*x[1]**o+(-0.0182577227471)*x[1]+(-0.0210976208306)*x[2]**o+(0.0761117140757)*x[2]
        s[3,3,0,1]=(0.748296379412)*x[0]**o+(-0.807953754409)*x[0]+(-0.545605437765)*x[1]**o+(0.23683862911)*x[1]+(0.173607048395)*x[2]**o+(-0.44250075226)*x[2]
        s[3,3,1,0]=(-0.726945541384)*x[0]**o+(0.426848422718)*x[0]+(-0.92314597997)*x[1]**o+(0.648637342846)*x[1]+(-0.578392786152)*x[2]**o+(0.848674260689)*x[2]
        s[3,3,1,1]=(-0.644752429929)*x[0]**o+(0.950947546555)*x[0]+(0.0149417119272)*x[1]**o+(0.716391605858)*x[1]+(0.0490453676731)*x[2]**o+(0.0576413425128)*x[2]
        s[3,3,2,0]=(0.466006820631)*x[0]**o+(0.161941404819)*x[0]+(-0.957641267925)*x[1]**o+(0.0765544212389)*x[1]+(0.701408110369)*x[2]**o+(-0.657156965052)*x[2]
        s[3,3,2,1]=(-0.867176738598)*x[0]**o+(-0.449219272229)*x[0]+(0.324298582458)*x[1]**o+(-0.998730968715)*x[1]+(0.574803005179)*x[2]**o+(-0.243281577204)*x[2]
        s[3,4,0,0]=(-0.263021668111)*x[0]**o+(0.802820216492)*x[0]+(0.372317055664)*x[1]**o+(-0.907203003358)*x[1]+(-0.0875142912227)*x[2]**o+(-0.899457536279)*x[2]
        s[3,4,0,1]=(0.223529185934)*x[0]**o+(-0.112858652745)*x[0]+(0.112751409232)*x[1]**o+(-0.383168214822)*x[1]+(0.0253282930677)*x[2]**o+(-0.0727000650191)*x[2]
        s[3,4,1,0]=(0.402273303095)*x[0]**o+(0.577572690365)*x[0]+(0.157976741538)*x[1]**o+(-0.374155092812)*x[1]+(-0.899398318425)*x[2]**o+(0.56916282818)*x[2]
        s[3,4,1,1]=(-0.565673417227)*x[0]**o+(-0.0157754170468)*x[0]+(-0.896925597573)*x[1]**o+(-0.968592388923)*x[1]+(-0.694486144109)*x[2]**o+(-0.795948130804)*x[2]
        s[3,4,2,0]=(-0.962992173758)*x[0]**o+(-0.299086071473)*x[0]+(0.367929277599)*x[1]**o+(-0.139102532943)*x[1]+(-0.838141685104)*x[2]**o+(0.181200376172)*x[2]
        s[3,4,2,1]=(0.610198649476)*x[0]**o+(-0.747032724663)*x[0]+(0.546335594194)*x[1]**o+(0.21236923042)*x[1]+(0.028021042999)*x[2]**o+(0.469015322864)*x[2]
        ref[0,0,0,0]=(-1.03505909819)/(o+1.)+(-0.0424682351702)+(-0.284157309093)*0.5**o
        ref[0,0,0,1]=(1.20050545262)/(o+1.)+(0.182893096798)+(-0.315769534489)*0.5**o
        ref[0,0,1,0]=(1.18191621585)/(o+1.)+(-0.399132238132)+(-0.4535776613)*0.5**o
        ref[0,0,1,1]=(-1.34097322417)/(o+1.)+(1.00115578994)+(-0.739956684686)*0.5**o
        ref[0,0,2,0]=(0.374251042378)/(o+1.)+(0.33923574855)+(0.630514031324)*0.5**o
        ref[0,0,2,1]=(0.781947977225)/(o+1.)+(-0.0848585871226)+(-0.763048729267)*0.5**o
        ref[0,1,0,0]=(1.60214777624)/(o+1.)+(0.568449742282)+(-0.438048883736)*0.5**o
        ref[0,1,0,1]=(-1.24420506553)/(o+1.)+(-0.0158819624619)+(0.169577859304)*0.5**o
        ref[0,1,1,0]=(-1.7324128314)/(o+1.)+(-0.0971954419402)+(-0.136877583438)*0.5**o
        ref[0,1,1,1]=(0.983260153455)/(o+1.)+(-0.433646513272)+(0.21329287668)*0.5**o
        ref[0,1,2,0]=(-1.03510150258)/(o+1.)+(-0.706705011055)+(-0.488721257593)*0.5**o
        ref[0,1,2,1]=(-0.218470519691)/(o+1.)+(0.319591579745)+(0.0407718763863)*0.5**o
        ref[0,2,0,0]=(0.625311774166)/(o+1.)+(0.0218405294219)+(0.587345517354)*0.5**o
        ref[0,2,0,1]=(0.876141884916)/(o+1.)+(-0.356201888016)+(-0.811084805765)*0.5**o
        ref[0,2,1,0]=(-0.687154150232)/(o+1.)+(0.77098577072)+(-0.222984108172)*0.5**o
        ref[0,2,1,1]=(0.783862950926)/(o+1.)+(0.88949992906)+(-0.351616621125)*0.5**o
        ref[0,2,2,0]=(1.03032469103)/(o+1.)+(-0.727535078248)+(0.326986640454)*0.5**o
        ref[0,2,2,1]=(-0.852371311009)/(o+1.)+(-0.181270485818)+(0.416408828986)*0.5**o
        ref[0,3,0,0]=(1.41111824503)/(o+1.)+(1.07119295236)+(0.165546039968)*0.5**o
        ref[0,3,0,1]=(-1.83184805832)/(o+1.)+(-0.517194924063)+(0.806943353866)*0.5**o
        ref[0,3,1,0]=(-1.13181236513)/(o+1.)+(-1.02397128909)+(-0.0329058672137)*0.5**o
        ref[0,3,1,1]=(0.304318369001)/(o+1.)+(0.00956600470204)+(0.927391314677)*0.5**o
        ref[0,3,2,0]=(0.501178198552)/(o+1.)+(0.0124567541342)+(-0.395900087683)*0.5**o
        ref[0,3,2,1]=(-1.4656997814)/(o+1.)+(-0.817947532691)+(0.167004592732)*0.5**o
        ref[0,4,0,0]=(-0.535116000151)/(o+1.)+(0.813669724142)+(-0.0559473269152)*0.5**o
        ref[0,4,0,1]=(-0.386064621699)/(o+1.)+(0.506465703971)+(-0.126102715426)*0.5**o
        ref[0,4,1,0]=(0.235769960904)/(o+1.)+(0.507444715073)+(-0.465158745622)*0.5**o
        ref[0,4,1,1]=(-1.25358393189)/(o+1.)+(0.745413177378)+(0.433494572203)*0.5**o
        ref[0,4,2,0]=(0.3766047946)/(o+1.)+(0.730562643948)+(-0.226388318689)*0.5**o
        ref[0,4,2,1]=(-0.868789668733)/(o+1.)+(-0.633013988131)+(-0.487005215247)*0.5**o
        ref[1,0,0,0]=(0.315996038899)/(o+1.)+(0.895638925406)+(-0.599811357812)*0.5**o
        ref[1,0,0,1]=(-1.84265003124)/(o+1.)+(0.240532908971)+(-0.849384033331)*0.5**o
        ref[1,0,1,0]=(-0.0941474741887)/(o+1.)+(0.65200760392)+(0.560384598535)*0.5**o
        ref[1,0,1,1]=(1.09577916824)/(o+1.)+(0.164260614444)+(0.394452679675)*0.5**o
        ref[1,0,2,0]=(-0.218722993542)/(o+1.)+(-0.562676967744)+(-0.286588291759)*0.5**o
        ref[1,0,2,1]=(0.351571050868)/(o+1.)+(0.535346830254)+(0.587814763664)*0.5**o
        ref[1,1,0,0]=(-0.981246651633)/(o+1.)+(0.993103291818)+(-0.0104862086098)*0.5**o
        ref[1,1,0,1]=(-0.0223701310738)/(o+1.)+(-0.479372400756)+(-0.442153331955)*0.5**o
        ref[1,1,1,0]=(0.166993904921)/(o+1.)+(-0.0804361294934)+(0.143778546111)*0.5**o
        ref[1,1,1,1]=(-0.454722153421)/(o+1.)+(-0.967597910545)+(0.830078999256)*0.5**o
        ref[1,1,2,0]=(-0.280227426918)/(o+1.)+(-0.399959394644)+(-0.818875997493)*0.5**o
        ref[1,1,2,1]=(-1.44539571982)/(o+1.)+(-0.172660106102)+(0.897735820104)*0.5**o
        ref[1,2,0,0]=(0.229059680434)/(o+1.)+(-0.279586461518)+(-0.225661260024)*0.5**o
        ref[1,2,0,1]=(-0.11531008643)/(o+1.)+(-0.725215085638)+(0.188146461111)*0.5**o
        ref[1,2,1,0]=(-1.07473772287)/(o+1.)+(-1.10205837886)+(-0.713236970029)*0.5**o
        ref[1,2,1,1]=(-0.00752571015035)/(o+1.)+(-0.793755980616)+(0.338343830055)*0.5**o
        ref[1,2,2,0]=(-1.55472679711)/(o+1.)+(-0.771580156139)+(-0.857754750211)*0.5**o
        ref[1,2,2,1]=(-1.41530903614)/(o+1.)+(0.478834879863)+(0.798299933694)*0.5**o
        ref[1,3,0,0]=(-0.605589351334)/(o+1.)+(-0.213211988214)+(-0.22936580714)*0.5**o
        ref[1,3,0,1]=(-1.20312374453)/(o+1.)+(-0.988626008968)+(0.272143747738)*0.5**o
        ref[1,3,1,0]=(1.30870990435)/(o+1.)+(0.246218656117)+(-0.371813536068)*0.5**o
        ref[1,3,1,1]=(-0.148337279237)/(o+1.)+(0.566337073785)+(0.64355520606)*0.5**o
        ref[1,3,2,0]=(-0.91862824326)/(o+1.)+(-0.0175057723384)+(0.296777344281)*0.5**o
        ref[1,3,2,1]=(-1.37550450457)/(o+1.)+(-0.0120519795955)+(-0.631360619387)*0.5**o
        ref[1,4,0,0]=(-1.03348697399)/(o+1.)+(0.245505236923)+(-0.864162318969)*0.5**o
        ref[1,4,0,1]=(0.226279058508)/(o+1.)+(-0.279695235305)+(-0.0204110610831)*0.5**o
        ref[1,4,1,0]=(0.129273063512)/(o+1.)+(-0.558900295355)+(-0.0714182037356)*0.5**o
        ref[1,4,1,1]=(1.18088147769)/(o+1.)+(-0.916662811143)+(-0.789159638733)*0.5**o
        ref[1,4,2,0]=(0.464292100303)/(o+1.)+(0.0165108463129)+(-0.545542033256)*0.5**o
        ref[1,4,2,1]=(-0.736810350419)/(o+1.)+(-0.660665077748)+(0.655205311592)*0.5**o
        ref[2,0,0,0]=(0.417694152388)/(o+1.)+(-0.0905431379759)+(-0.0509015638697)*0.5**o
        ref[2,0,0,1]=(1.05615351466)/(o+1.)+(0.169735047875)+(-0.834927535726)*0.5**o
        ref[2,0,1,0]=(1.20949251256)/(o+1.)+(0.385393862823)+(0.695941957197)*0.5**o
        ref[2,0,1,1]=(0.924336787014)/(o+1.)+(0.655869253281)+(-0.813515499086)*0.5**o
        ref[2,0,2,0]=(-0.422504632433)/(o+1.)+(-0.398651551024)+(0.264441597574)*0.5**o
        ref[2,0,2,1]=(-0.789880335569)/(o+1.)+(-0.25109601857)+(0.839834410476)*0.5**o
        ref[2,1,0,0]=(-1.0730244215)/(o+1.)+(-0.0877673397313)+(-0.9032683394)*0.5**o
        ref[2,1,0,1]=(-0.110167460347)/(o+1.)+(0.319530276689)+(-0.563127063275)*0.5**o
        ref[2,1,1,0]=(-0.458971665492)/(o+1.)+(0.0755559728958)+(-0.0215129450025)*0.5**o
        ref[2,1,1,1]=(0.920845617782)/(o+1.)+(0.357447932409)+(0.862938112821)*0.5**o
        ref[2,1,2,0]=(-0.0571398560888)/(o+1.)+(-0.185049111973)+(-0.151555342557)*0.5**o
        ref[2,1,2,1]=(0.636402668026)/(o+1.)+(-0.703521938239)+(0.485262168703)*0.5**o
        ref[2,2,0,0]=(0.107271092594)/(o+1.)+(0.0850978751753)+(0.597629111376)*0.5**o
        ref[2,2,0,1]=(-0.339656097563)/(o+1.)+(0.57534630574)+(0.0885418745402)*0.5**o
        ref[2,2,1,0]=(-0.238561968668)/(o+1.)+(0.115193935875)+(-0.7058457718)*0.5**o
        ref[2,2,1,1]=(0.668608005739)/(o+1.)+(0.659289028954)+(0.431391331275)*0.5**o
        ref[2,2,2,0]=(-0.672876883281)/(o+1.)+(-0.21881291977)+(-0.172742409612)*0.5**o
        ref[2,2,2,1]=(0.83970372368)/(o+1.)+(-0.992038068194)+(0.193762571076)*0.5**o
        ref[2,3,0,0]=(-1.68484871783)/(o+1.)+(0.780118908189)+(-0.816708755757)*0.5**o
        ref[2,3,0,1]=(1.18011814676)/(o+1.)+(-0.217769718186)+(0.557205351559)*0.5**o
        ref[2,3,1,0]=(-1.08023458875)/(o+1.)+(0.453977422158)+(-0.330158214652)*0.5**o
        ref[2,3,1,1]=(-1.05348964937)/(o+1.)+(0.0166509774407)+(0.0113498976079)*0.5**o
        ref[2,3,2,0]=(0.575130860462)/(o+1.)+(0.832540416184)+(0.548708545741)*0.5**o
        ref[2,3,2,1]=(0.836108525756)/(o+1.)+(0.127941230937)+(0.367699563724)*0.5**o
        ref[2,4,0,0]=(-1.10708293282)/(o+1.)+(-0.0897956041276)+(0.742185577055)*0.5**o
        ref[2,4,0,1]=(0.16336599513)/(o+1.)+(0.943309725321)+(-0.540606064885)*0.5**o
        ref[2,4,1,0]=(0.395830692119)/(o+1.)+(0.512924196638)+(-0.893508919799)*0.5**o
        ref[2,4,1,1]=(1.28227166603)/(o+1.)+(0.396720952645)+(-0.0277942588293)*0.5**o
        ref[2,4,2,0]=(0.145213537726)/(o+1.)+(-0.565302474208)+(0.712530777919)*0.5**o
        ref[2,4,2,1]=(0.271316883603)/(o+1.)+(0.419514983821)+(0.0858696597238)*0.5**o
        ref[3,0,0,0]=(0.128651588408)/(o+1.)+(-0.16928786448)+(-0.124887079943)*0.5**o
        ref[3,0,0,1]=(0.973498348173)/(o+1.)+(0.549525141797)+(0.0781604618068)*0.5**o
        ref[3,0,1,0]=(-0.0474642077688)/(o+1.)+(-0.405470696764)+(0.822494278919)*0.5**o
        ref[3,0,1,1]=(-0.78481155678)/(o+1.)+(-0.199324603422)+(-0.61666199289)*0.5**o
        ref[3,0,2,0]=(0.158564585045)/(o+1.)+(0.479703342715)+(-0.275356624889)*0.5**o
        ref[3,0,2,1]=(0.402458197725)/(o+1.)+(-0.313934227149)+(0.17925744427)*0.5**o
        ref[3,1,0,0]=(-1.18937097637)/(o+1.)+(0.237540022962)+(0.524297360691)*0.5**o
        ref[3,1,0,1]=(0.68869701038)/(o+1.)+(0.65962557996)+(-0.261808400698)*0.5**o
        ref[3,1,1,0]=(0.687153362406)/(o+1.)+(-0.638840872927)+(0.134876119034)*0.5**o
        ref[3,1,1,1]=(0.791143545912)/(o+1.)+(0.71585665818)+(-0.987212349574)*0.5**o
        ref[3,1,2,0]=(0.497905698197)/(o+1.)+(0.531774060666)+(0.742204537127)*0.5**o
        ref[3,1,2,1]=(0.996552149008)/(o+1.)+(0.400787163838)+(-0.235349061749)*0.5**o
        ref[3,2,0,0]=(0.325518116836)/(o+1.)+(0.354782919535)+(-0.938490786208)*0.5**o
        ref[3,2,0,1]=(0.618999261276)/(o+1.)+(0.369260987251)+(-0.0180578829991)*0.5**o
        ref[3,2,1,0]=(0.315987527168)/(o+1.)+(-0.477890830901)+(0.294998689081)*0.5**o
        ref[3,2,1,1]=(0.94313275912)/(o+1.)+(-0.319244037788)+(-0.722157328967)*0.5**o
        ref[3,2,2,0]=(-1.10496560944)/(o+1.)+(0.669209611141)+(0.621065266638)*0.5**o
        ref[3,2,2,1]=(-0.221274908319)/(o+1.)+(0.673677242094)+(0.0383194307525)*0.5**o
        ref[3,3,0,0]=(0.0442619105124)/(o+1.)+(0.38333458178)+(0.0357627225956)*0.5**o
        ref[3,3,0,1]=(-0.371998389371)/(o+1.)+(-0.50680793878)+(0.748296379412)*0.5**o
        ref[3,3,1,0]=(-1.50153876612)/(o+1.)+(0.962080013126)+(-0.726945541384)*0.5**o
        ref[3,3,1,1]=(0.0639870796003)/(o+1.)+(0.862490247463)+(-0.644752429929)*0.5**o
        ref[3,3,2,0]=(-0.256233157556)/(o+1.)+(-0.209330569497)+(0.466006820631)*0.5**o
        ref[3,3,2,1]=(0.899101587637)/(o+1.)+(-0.845615909074)+(-0.867176738598)*0.5**o
        ref[3,4,0,0]=(0.284802764441)/(o+1.)+(-0.501920161572)+(-0.263021668111)*0.5**o
        ref[3,4,0,1]=(0.1380797023)/(o+1.)+(-0.284363466293)+(0.223529185934)*0.5**o
        ref[3,4,1,0]=(-0.741421576887)/(o+1.)+(0.386290212867)+(0.402273303095)*0.5**o
        ref[3,4,1,1]=(-1.59141174168)/(o+1.)+(-0.890157968387)+(-0.565673417227)*0.5**o
        ref[3,4,2,0]=(-0.470212407505)/(o+1.)+(-0.128494114122)+(-0.962992173758)*0.5**o
        ref[3,4,2,1]=(0.574356637193)/(o+1.)+(-0.0328240856895)+(0.610198649476)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactZero_fromSymbol_ReducedSolution_rank0(self):
      """
      tests integral of rank 0 Symbol on the FunctionOnContactZero

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports integral on FunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Symbol(shape=())
      s=Data(0,(),w)
      if dim==2:
        s=(1.70255303494)*x[0]+(0.300011314949)*x[1]
        ref=(0.345983004398)/(o+1.)+(0.363938720886)+(0.928703903719)*0.5**o
      else:
        s=(-1.68260770684)*x[0]+(-1.14729600003)*x[1]+(-1.35659279947)*x[2]
        ref=(-1.3902352024)/(o+1.)+(-1.03788235052)+(-0.720496602901)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactZero_fromSymbol_ReducedSolution_rank1(self):
      """
      tests integral of rank 1 Symbol on the FunctionOnContactZero

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports integral on FunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(2,))
      s=Data(0,(2,),w)
      ref=numpy.zeros((2,),numpy.float_)
      if dim==2:
        s[0]=(0.23958799559)*x[0]+(-0.0458757682877)*x[1]
        s[1]=(0.0897882061379)*x[0]+(-1.54974909413)*x[1]
        ref[0]=(0.0485704769529)/(o+1.)+(0.210351971249)+(-0.275562192148)*0.5**o
        ref[1]=(-0.821668980438)/(o+1.)+(-0.712207676061)+(0.786123444573)*0.5**o
      else:
        s[0]=(-1.44817804789)*x[0]+(0.670605136525)*x[1]+(-0.593548775192)*x[2]
        s[1]=(-0.585702105721)*x[0]+(0.0665105334471)*x[1]+(-0.228099551666)*x[2]
        ref[0]=(0.859021549917)/(o+1.)+(-0.759857813727)+(-0.710427609023)*0.5**o
        ref[1]=(-1.40806718776)/(o+1.)+(0.267420623854)+(0.125934816108)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactZero_fromSymbol_ReducedSolution_rank2(self):
      """
      tests integral of rank 2 Symbol on the FunctionOnContactZero

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports integral on FunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5))
      s=Data(0,(4, 5),w)
      ref=numpy.zeros((4, 5),numpy.float_)
      if dim==2:
        s[0,0]=(0.933177344575)*x[0]+(-0.643793974434)*x[1]
        s[0,1]=(-0.311019666872)*x[0]+(0.686195407652)*x[1]
        s[0,2]=(0.470203454822)*x[0]+(1.32374103685)*x[1]
        s[0,3]=(-0.653103411901)*x[0]+(0.477452857192)*x[1]
        s[0,4]=(1.12606394408)*x[0]+(0.0898490519564)*x[1]
        s[1,0]=(1.26644130664)*x[0]+(0.226895598587)*x[1]
        s[1,1]=(0.0271345466814)*x[0]+(0.435024033603)*x[1]
        s[1,2]=(-0.227055843926)*x[0]+(-0.144686134637)*x[1]
        s[1,3]=(0.466198287572)*x[0]+(-0.130830709376)*x[1]
        s[1,4]=(1.04066080781)*x[0]+(0.546282129071)*x[1]
        s[2,0]=(0.317469515041)*x[0]+(1.26366516514)*x[1]
        s[2,1]=(-1.20210768286)*x[0]+(-0.0533328989847)*x[1]
        s[2,2]=(0.278459393186)*x[0]+(-0.379156129627)*x[1]
        s[2,3]=(1.03459096792)*x[0]+(-0.610026027371)*x[1]
        s[2,4]=(1.288131278)*x[0]+(-0.309929698872)*x[1]
        s[3,0]=(0.32856946183)*x[0]+(0.225709965354)*x[1]
        s[3,1]=(-1.03183224237)*x[0]+(-0.0153171719792)*x[1]
        s[3,2]=(0.259516430572)*x[0]+(-0.776082702284)*x[1]
        s[3,3]=(-0.417978040907)*x[0]+(-0.0941785049007)*x[1]
        s[3,4]=(1.07527114074)*x[0]+(-0.0198401110946)*x[1]
        ref[0,0]=(-0.79959835955)/(o+1.)+(0.566758804494)+(-0.0445358792971)*0.5**o
        ref[0,1]=(0.583455543567)/(o+1.)+(0.0458448886173)+(-0.299969580021)*0.5**o
        ref[0,2]=(0.471614825642)/(o+1.)+(0.891760595232)+(-0.461191524434)*0.5**o
        ref[0,3]=(0.672045220068)/(o+1.)+(-0.56343583876)+(0.279175902744)*0.5**o
        ref[0,4]=(-0.127993678518)/(o+1.)+(0.52668612755)+(0.290534419455)*0.5**o
        ref[1,0]=(0.822774751438)/(o+1.)+(-0.011855097694)+(0.694272349175)*0.5**o
        ref[1,1]=(0.552626324952)/(o+1.)+(0.277929562319)+(-0.646326869305)*0.5**o
        ref[1,2]=(-0.565996176797)/(o+1.)+(-0.128057099164)+(0.450368396562)*0.5**o
        ref[1,3]=(0.627837338323)/(o+1.)+(-0.555359040353)+(0.818248320579)*0.5**o
        ref[1,4]=(0.722984603229)/(o+1.)+(0.0957452220354)+(0.672467889583)*0.5**o
        ref[2,0]=(0.664524759547)/(o+1.)+(0.678277833656)+(-0.439945746675)*0.5**o
        ref[2,1]=(0.908213473231)/(o+1.)+(-0.805606306623)+(-0.552441441832)*0.5**o
        ref[2,2]=(0.0908414991316)/(o+1.)+(0.131818510391)+(-0.455175256354)*0.5**o
        ref[2,3]=(-0.119095663088)/(o+1.)+(-0.223144373358)+(0.989949350348)*0.5**o
        ref[2,4]=(-0.0540728359185)/(o+1.)+(0.0748680956845)+(0.882538223681)*0.5**o
        ref[3,0]=(0.98273847633)/(o+1.)+(-0.421621406424)+(0.414783763702)*0.5**o
        ref[3,1]=(0.29285575515)/(o+1.)+(-0.462798732226)+(-0.41440770505)*0.5**o
        ref[3,2]=(-0.509767634157)/(o+1.)+(-0.480617720091)+(0.954436802627)*0.5**o
        ref[3,3]=(-0.762302422898)/(o+1.)+(0.100280539595)+(0.0495847978999)*0.5**o
        ref[3,4]=(-0.154196647936)/(o+1.)+(0.329602137701)+(0.550423402184)*0.5**o
      else:
        s[0,0]=(1.34601675616)*x[0]+(-0.451295005459)*x[1]+(-0.49622174234)*x[2]
        s[0,1]=(-0.327908015969)*x[0]+(-1.21019079469)*x[1]+(1.09622527656)*x[2]
        s[0,2]=(-0.0243507161566)*x[0]+(-0.016230100758)*x[1]+(-1.37311484633)*x[2]
        s[0,3]=(-0.445665346289)*x[0]+(-0.626824621259)*x[1]+(-0.196033254199)*x[2]
        s[0,4]=(-0.819703680211)*x[0]+(1.30953832902)*x[1]+(0.0778023021203)*x[2]
        s[1,0]=(0.219519741099)*x[0]+(-1.47335622754)*x[1]+(0.198662145648)*x[2]
        s[1,1]=(-0.65456993306)*x[0]+(-0.125758153194)*x[1]+(-1.53966098547)*x[2]
        s[1,2]=(-0.112157349953)*x[0]+(-0.459351490445)*x[1]+(1.6889234353)*x[2]
        s[1,3]=(0.102642361518)*x[0]+(1.10412193101)*x[1]+(-0.239843060928)*x[2]
        s[1,4]=(-0.232367456037)*x[0]+(-1.17836662288)*x[1]+(-0.814085860866)*x[2]
        s[2,0]=(-1.00510966212)*x[0]+(1.23960710585)*x[1]+(0.0848958213029)*x[2]
        s[2,1]=(-1.93901641139)*x[0]+(-0.0469703867512)*x[1]+(-0.21201338191)*x[2]
        s[2,2]=(1.42666776379)*x[0]+(1.04000263985)*x[1]+(0.151609334515)*x[2]
        s[2,3]=(-0.297049329338)*x[0]+(-0.103284668913)*x[1]+(-0.110261476034)*x[2]
        s[2,4]=(0.964711394219)*x[0]+(1.28218755108)*x[1]+(-1.2382220868)*x[2]
        s[3,0]=(-0.922061452692)*x[0]+(-1.36593539333)*x[1]+(1.11436583585)*x[2]
        s[3,1]=(0.284590223973)*x[0]+(1.08442017719)*x[1]+(1.5619949376)*x[2]
        s[3,2]=(-0.245207105356)*x[0]+(-1.77865635864)*x[1]+(-0.908570594143)*x[2]
        s[3,3]=(0.426334196419)*x[0]+(-0.501103031142)*x[1]+(0.199762115243)*x[2]
        s[3,4]=(1.33158896071)*x[0]+(0.44737802367)*x[1]+(-0.76095091762)*x[2]
        ref[0,0]=(-0.581088472202)/(o+1.)+(0.287163129514)+(0.405262221535)*0.5**o
        ref[0,1]=(0.695209950176)/(o+1.)+(-0.748195486472)+(0.359307488667)*0.5**o
        ref[0,2]=(0.313650504565)/(o+1.)+(-1.20553227172)+(0.683718375628)*0.5**o
        ref[0,3]=(-0.872265925416)/(o+1.)+(-0.216679466249)+(0.0371016361661)*0.5**o
        ref[0,4]=(1.07552646704)/(o+1.)+(-0.293195846094)+(0.0785021760824)*0.5**o
        ref[1,0]=(-1.22710752208)/(o+1.)+(0.409726431872)+(-0.647519682458)*0.5**o
        ref[1,1]=(-0.940490307845)/(o+1.)+(-0.82083756271)+(0.26217636154)*0.5**o
        ref[1,2]=(1.00054234642)/(o+1.)+(-0.0533842117578)+(0.223640672001)*0.5**o
        ref[1,3]=(0.0595140200914)/(o+1.)+(0.407850368171)+(0.0917064751695)*0.5**o
        ref[1,4]=(-0.50735295069)/(o+1.)+(-1.10564117596)+(0.493815362818)*0.5**o
        ref[2,0]=(0.786394742154)/(o+1.)+(-0.212261329287)+(-0.0424788185539)*0.5**o
        ref[2,1]=(-0.868240537836)/(o+1.)+(-0.186797194849)+(-0.956165252514)*0.5**o
        ref[2,2]=(-0.67113838962)/(o+1.)+(1.26208372573)+(0.765250676321)*0.5**o
        ref[2,3]=(-1.2390712788)/(o+1.)+(0.286327258603)+(0.15582128731)*0.5**o
        ref[2,4]=(-0.00831435188402)/(o+1.)+(0.305363694299)+(0.406263821787)*0.5**o
        ref[3,0]=(-0.123205370838)/(o+1.)+(-0.410740153717)+(-0.228945331902)*0.5**o
        ref[3,1]=(1.39199709306)/(o+1.)+(0.646045427731)+(0.246917390241)*0.5**o
        ref[3,2]=(-1.10740261108)/(o+1.)+(-0.860231147304)+(-0.104569152453)*0.5**o
        ref[3,3]=(-0.506214554892)/(o+1.)+(0.53544681941)+(-0.439685803409)*0.5**o
        ref[3,4]=(0.846470301583)/(o+1.)+(-0.134756583683)+(0.441058932543)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactZero_fromSymbol_ReducedSolution_rank3(self):
      """
      tests integral of rank 3 Symbol on the FunctionOnContactZero

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports integral on FunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(6, 2, 2))
      s=Data(0,(6, 2, 2),w)
      ref=numpy.zeros((6, 2, 2),numpy.float_)
      if dim==2:
        s[0,0,0]=(-0.907789262757)*x[0]+(1.43778352913)*x[1]
        s[0,0,1]=(-0.441541130755)*x[0]+(1.75922038312)*x[1]
        s[0,1,0]=(-0.387854312613)*x[0]+(1.22523701409)*x[1]
        s[0,1,1]=(-1.15521894576)*x[0]+(0.0289558035739)*x[1]
        s[1,0,0]=(-0.640400813271)*x[0]+(-1.69766965873)*x[1]
        s[1,0,1]=(-0.641296842318)*x[0]+(-1.35895483901)*x[1]
        s[1,1,0]=(-0.14735518066)*x[0]+(0.520543405774)*x[1]
        s[1,1,1]=(0.369918186173)*x[0]+(0.209781392053)*x[1]
        s[2,0,0]=(0.677808706359)*x[0]+(0.481042907356)*x[1]
        s[2,0,1]=(-0.661702781276)*x[0]+(1.86193178646)*x[1]
        s[2,1,0]=(0.323230036331)*x[0]+(-1.45705288353)*x[1]
        s[2,1,1]=(1.8644362516)*x[0]+(0.764691070666)*x[1]
        s[3,0,0]=(-1.33103240086)*x[0]+(1.29035953404)*x[1]
        s[3,0,1]=(-0.161074703654)*x[0]+(-0.127322745584)*x[1]
        s[3,1,0]=(-0.359913251027)*x[0]+(-0.0952192793759)*x[1]
        s[3,1,1]=(-0.34519536765)*x[0]+(0.483950473232)*x[1]
        s[4,0,0]=(0.981563058768)*x[0]+(-0.0744642367992)*x[1]
        s[4,0,1]=(-0.45912279344)*x[0]+(0.0011253607226)*x[1]
        s[4,1,0]=(-0.618322333127)*x[0]+(0.199638289627)*x[1]
        s[4,1,1]=(-0.371110449749)*x[0]+(-0.382992800899)*x[1]
        s[5,0,0]=(0.840829543072)*x[0]+(-0.925833368141)*x[1]
        s[5,0,1]=(-0.0348263534937)*x[0]+(0.267324292682)*x[1]
        s[5,1,0]=(0.0785260612832)*x[0]+(1.59479548515)*x[1]
        s[5,1,1]=(-0.203914067868)*x[0]+(1.28202597585)*x[1]
        ref[0,0,0]=(0.898417888273)/(o+1.)+(0.171058001117)+(-0.710539624134)*0.5**o
        ref[0,0,1]=(0.949672663046)/(o+1.)+(0.186488421776)+(-0.00497025423347)*0.5**o
        ref[0,1,0]=(0.434571276181)/(o+1.)+(0.106095788846)+(0.190619847602)*0.5**o
        ref[0,1,1]=(0.337711977059)/(o+1.)+(-0.313335995076)+(-0.837303129097)*0.5**o
        ref[1,0,0]=(-0.757199193222)/(o+1.)+(-0.36300260101)+(-0.854866076761)*0.5**o
        ref[1,0,1]=(-0.767144980208)/(o+1.)+(-0.215422826519)+(-0.802261048082)*0.5**o
        ref[1,1,0]=(-0.13190981045)/(o+1.)+(-0.0320289199048)+(0.569155875375)*0.5**o
        ref[1,1,1]=(0.0466405054813)/(o+1.)+(0.565229924314)+(-0.597400775882)*0.5**o
        ref[2,0,0]=(0.760951384359)/(o+1.)+(0.0600961970572)+(0.277707835242)*0.5**o
        ref[2,0,1]=(0.869894715615)/(o+1.)+(0.631323963868)+(-0.932313638168)*0.5**o
        ref[2,1,0]=(-0.666302256751)/(o+1.)+(-0.510864963778)+(0.554209337103)*0.5**o
        ref[2,1,1]=(0.934304180676)/(o+1.)+(0.350887235523)+(0.993048670542)*0.5**o
        ref[3,0,0]=(0.930749923608)/(o+1.)+(-0.283299908306)+(-0.404822973819)*0.5**o
        ref[3,0,1]=(-0.572270969539)/(o+1.)+(0.583432503768)+(-0.882991487235)*0.5**o
        ref[3,1,0]=(0.642892258995)/(o+1.)+(-0.0826293437351)+(-0.932766101927)*0.5**o
        ref[3,1,1]=(0.679591980627)/(o+1.)+(-0.379930907942)+(0.219024940838)*0.5**o
        ref[4,0,0]=(-0.0326170324069)/(o+1.)+(0.0903259186859)+(0.759064017004)*0.5**o
        ref[4,0,1]=(-0.129142887308)/(o+1.)+(0.279560607285)+(-0.887975759979)*0.5**o
        ref[4,1,0]=(0.752463858901)/(o+1.)+(-0.256945595979)+(-0.657256710443)*0.5**o
        ref[4,1,1]=(0.565307319705)/(o+1.)+(-0.898218881636)+(0.477027192919)*0.5**o
        ref[5,0,0]=(-0.28969980947)/(o+1.)+(0.0341656280661)+(0.136364728269)*0.5**o
        ref[5,0,1]=(-0.362525553087)/(o+1.)+(0.17876968499)+(0.237484122295)*0.5**o
        ref[5,1,0]=(0.856225350097)/(o+1.)+(0.375747136218)+(0.0656019238951)*0.5**o
        ref[5,1,1]=(0.358764765052)/(o+1.)+(0.783470816704)+(-0.847594490479)*0.5**o
      else:
        s[0,0,0]=(-0.18318998999)*x[0]+(-1.71372306128)*x[1]+(1.13788709107)*x[2]
        s[0,0,1]=(0.95308908228)*x[0]+(0.123844162735)*x[1]+(0.0868410957691)*x[2]
        s[0,1,0]=(-0.712762731169)*x[0]+(-1.86018372873)*x[1]+(-0.0783828227128)*x[2]
        s[0,1,1]=(-1.72172214895)*x[0]+(-1.25477573826)*x[1]+(1.28096692374)*x[2]
        s[1,0,0]=(0.86250397119)*x[0]+(1.44314824122)*x[1]+(0.763303582012)*x[2]
        s[1,0,1]=(1.05520701337)*x[0]+(1.04926637639)*x[1]+(-0.378977987156)*x[2]
        s[1,1,0]=(0.753550528625)*x[0]+(-1.63882951308)*x[1]+(0.259583964201)*x[2]
        s[1,1,1]=(0.778941098246)*x[0]+(-1.10564597288)*x[1]+(-1.42906071049)*x[2]
        s[2,0,0]=(-0.842438687373)*x[0]+(-0.202024764935)*x[1]+(0.289144880342)*x[2]
        s[2,0,1]=(-1.02319531994)*x[0]+(0.155875285165)*x[1]+(0.310920357117)*x[2]
        s[2,1,0]=(-1.45156693584)*x[0]+(0.12231441645)*x[1]+(0.722232222435)*x[2]
        s[2,1,1]=(-1.08553719113)*x[0]+(0.316521434206)*x[1]+(0.720555059931)*x[2]
        s[3,0,0]=(1.91846595467)*x[0]+(0.199356393766)*x[1]+(-0.438144879601)*x[2]
        s[3,0,1]=(-0.341123312011)*x[0]+(0.325154462361)*x[1]+(0.691743856063)*x[2]
        s[3,1,0]=(1.72547153856)*x[0]+(-0.822030080649)*x[1]+(-1.15169896539)*x[2]
        s[3,1,1]=(0.243194545088)*x[0]+(-0.718504738836)*x[1]+(0.560536413366)*x[2]
        s[4,0,0]=(0.248455459192)*x[0]+(0.415661600687)*x[1]+(0.21712561395)*x[2]
        s[4,0,1]=(-1.23171610721)*x[0]+(-0.443121697053)*x[1]+(0.724481514081)*x[2]
        s[4,1,0]=(-0.0327531013679)*x[0]+(0.385175130963)*x[1]+(-0.0484020348058)*x[2]
        s[4,1,1]=(-0.199960439132)*x[0]+(-0.751568103399)*x[1]+(-1.20119343449)*x[2]
        s[5,0,0]=(0.853262644583)*x[0]+(1.47653986554)*x[1]+(-0.0642737845052)*x[2]
        s[5,0,1]=(1.1703484139)*x[0]+(-0.848977140408)*x[1]+(-0.303628307573)*x[2]
        s[5,1,0]=(-0.719950626922)*x[0]+(0.192452589234)*x[1]+(0.315963976897)*x[2]
        s[5,1,1]=(0.331604591856)*x[0]+(0.428822102766)*x[1]+(-1.35258251093)*x[2]
        ref[0,0,0]=(-0.452131788567)/(o+1.)+(-0.313943872628)+(0.320993573616)*0.5**o
        ref[0,0,1]=(-1.4121302884)/(o+1.)+(0.934332411096)+(0.707239806995)*0.5**o
        ref[0,1,0]=(-0.371701930519)/(o+1.)+(-1.22914563984)+(0.178663927586)*0.5**o
        ref[0,1,1]=(-0.569427715149)/(o+1.)+(-0.102129307368)+(-0.921844633589)*0.5**o
        ref[1,0,0]=(1.07041067382)/(o+1.)+(1.04773121415)+(-0.0969173076836)*0.5**o
        ref[1,0,1]=(0.344908055221)/(o+1.)+(0.253322966116)+(0.873941415143)*0.5**o
        ref[1,1,0]=(-0.568409920368)/(o+1.)+(-0.376074272994)+(0.694863446104)*0.5**o
        ref[1,1,1]=(-1.26296359103)/(o+1.)+(-0.285972966041)+(0.0791439379881)*0.5**o
        ref[2,0,0]=(0.261580662311)/(o+1.)+(-0.0693472624675)+(-0.878204709343)*0.5**o
        ref[2,0,1]=(-0.0659237231253)/(o+1.)+(-0.101871484642)+(-0.286732985248)*0.5**o
        ref[2,1,0]=(-0.609626668188)/(o+1.)+(0.438552975912)+(-0.874499580587)*0.5**o
        ref[2,1,1]=(1.21427389191)/(o+1.)+(-0.396304653588)+(-0.470125281731)*0.5**o
        ref[3,0,0]=(0.466268456837)/(o+1.)+(0.109894182607)+(0.993620646782)*0.5**o
        ref[3,0,1]=(0.798648502977)/(o+1.)+(0.30293631931)+(-0.728746135183)*0.5**o
        ref[3,1,0]=(-0.443525593788)/(o+1.)+(-0.32762855249)+(0.85052519129)*0.5**o
        ref[3,1,1]=(0.520520221786)/(o+1.)+(-0.21037503556)+(-0.0145439310481)*0.5**o
        ref[4,0,0]=(1.51637897594)/(o+1.)+(0.0251054216628)+(-0.685347145435)*0.5**o
        ref[4,0,1]=(-0.0126278181421)/(o+1.)+(-0.255524438913)+(-0.426679594213)*0.5**o
        ref[4,1,0]=(-0.110252848596)/(o+1.)+(-0.270402535847)+(0.95507791508)*0.5**o
        ref[4,1,1]=(-0.965558516877)/(o+1.)+(-0.777022669388)+(0.366881878633)*0.5**o
        ref[5,0,0]=(1.47489510124)/(o+1.)+(0.344964232978)+(0.100705158424)*0.5**o
        ref[5,0,1]=(0.272180101244)/(o+1.)+(-0.219694112974)+(0.184951090624)*0.5**o
        ref[5,1,0]=(0.868944944511)/(o+1.)+(-0.492651110402)+(-0.0951767844975)*0.5**o
        ref[5,1,1]=(-0.112001801469)/(o+1.)+(-0.0274812221923)+(-0.425191570457)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactZero_fromSymbol_ReducedSolution_rank4(self):
      """
      tests integral of rank 4 Symbol on the FunctionOnContactZero

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports integral on FunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5, 3, 2))
      s=Data(0,(4, 5, 3, 2),w)
      ref=numpy.zeros((4, 5, 3, 2),numpy.float_)
      if dim==2:
        s[0,0,0,0]=(0.492266509376)*x[0]+(-0.374307474367)*x[1]
        s[0,0,0,1]=(0.379610803929)*x[0]+(1.26877290052)*x[1]
        s[0,0,1,0]=(0.489836401727)*x[0]+(0.0568437409848)*x[1]
        s[0,0,1,1]=(-0.0962394811856)*x[0]+(1.08297193296)*x[1]
        s[0,0,2,0]=(0.0083854747638)*x[0]+(1.12783902851)*x[1]
        s[0,0,2,1]=(-1.60969353709)*x[0]+(-1.15342435141)*x[1]
        s[0,1,0,0]=(0.718333685586)*x[0]+(-0.580409068311)*x[1]
        s[0,1,0,1]=(-0.471412983112)*x[0]+(-1.21523638921)*x[1]
        s[0,1,1,0]=(0.436952252413)*x[0]+(0.817968269393)*x[1]
        s[0,1,1,1]=(0.904303325207)*x[0]+(1.27944662005)*x[1]
        s[0,1,2,0]=(-0.646670575219)*x[0]+(-0.268471138891)*x[1]
        s[0,1,2,1]=(0.216944526796)*x[0]+(-1.44656748298)*x[1]
        s[0,2,0,0]=(-1.45710792844)*x[0]+(-1.37549168202)*x[1]
        s[0,2,0,1]=(-0.0209124548614)*x[0]+(-0.405359828227)*x[1]
        s[0,2,1,0]=(0.713696154808)*x[0]+(-0.283663569311)*x[1]
        s[0,2,1,1]=(0.0879024421072)*x[0]+(-1.74270037297)*x[1]
        s[0,2,2,0]=(-0.263019221744)*x[0]+(0.587637432145)*x[1]
        s[0,2,2,1]=(0.291949019915)*x[0]+(1.21942896685)*x[1]
        s[0,3,0,0]=(0.711503988663)*x[0]+(-0.0889987684621)*x[1]
        s[0,3,0,1]=(-0.473200478311)*x[0]+(-0.731689338365)*x[1]
        s[0,3,1,0]=(0.0347877428229)*x[0]+(0.436342776392)*x[1]
        s[0,3,1,1]=(1.33181675729)*x[0]+(0.796826790935)*x[1]
        s[0,3,2,0]=(0.975860401214)*x[0]+(-0.0388876329765)*x[1]
        s[0,3,2,1]=(0.336224987742)*x[0]+(1.40851963434)*x[1]
        s[0,4,0,0]=(-0.430668122172)*x[0]+(-0.210883679811)*x[1]
        s[0,4,0,1]=(-0.0947644917837)*x[0]+(-0.145045406149)*x[1]
        s[0,4,1,0]=(-0.975155718684)*x[0]+(0.370091078964)*x[1]
        s[0,4,1,1]=(1.59897077782)*x[0]+(-0.716757934974)*x[1]
        s[0,4,2,0]=(0.0995595315716)*x[0]+(-0.654667529963)*x[1]
        s[0,4,2,1]=(0.621429995817)*x[0]+(-0.830020781954)*x[1]
        s[1,0,0,0]=(0.460799425849)*x[0]+(1.37039134597)*x[1]
        s[1,0,0,1]=(0.484677718571)*x[0]+(-1.06817334128)*x[1]
        s[1,0,1,0]=(0.157323780788)*x[0]+(-0.816294918051)*x[1]
        s[1,0,1,1]=(0.275640307711)*x[0]+(-1.13585749962)*x[1]
        s[1,0,2,0]=(1.16452950894)*x[0]+(-1.17355094567)*x[1]
        s[1,0,2,1]=(-0.567786612292)*x[0]+(-0.78722481996)*x[1]
        s[1,1,0,0]=(-0.106381006132)*x[0]+(1.35734811758)*x[1]
        s[1,1,0,1]=(0.870362310422)*x[0]+(-0.701894980494)*x[1]
        s[1,1,1,0]=(0.463101808026)*x[0]+(-0.106531628533)*x[1]
        s[1,1,1,1]=(0.34927073998)*x[0]+(0.597282241285)*x[1]
        s[1,1,2,0]=(-1.08705596894)*x[0]+(-0.476312920631)*x[1]
        s[1,1,2,1]=(0.461228158094)*x[0]+(0.553430989166)*x[1]
        s[1,2,0,0]=(-0.565889081597)*x[0]+(0.847674515062)*x[1]
        s[1,2,0,1]=(1.22774040742)*x[0]+(0.532076088807)*x[1]
        s[1,2,1,0]=(-1.35738184718)*x[0]+(-0.010480885754)*x[1]
        s[1,2,1,1]=(0.0609383835609)*x[0]+(0.539183578894)*x[1]
        s[1,2,2,0]=(0.765013210057)*x[0]+(-0.0658319206215)*x[1]
        s[1,2,2,1]=(0.720241379555)*x[0]+(-1.29084845886)*x[1]
        s[1,3,0,0]=(1.37641602996)*x[0]+(-0.713391122816)*x[1]
        s[1,3,0,1]=(0.475350779553)*x[0]+(-0.0146303427199)*x[1]
        s[1,3,1,0]=(-0.498436122956)*x[0]+(0.446982991213)*x[1]
        s[1,3,1,1]=(-0.0198417666776)*x[0]+(-0.0330880581816)*x[1]
        s[1,3,2,0]=(-0.968203407981)*x[0]+(-1.43766997027)*x[1]
        s[1,3,2,1]=(-1.516108898)*x[0]+(0.154669696369)*x[1]
        s[1,4,0,0]=(0.107239334472)*x[0]+(-1.01814801515)*x[1]
        s[1,4,0,1]=(0.490733729058)*x[0]+(1.0617427681)*x[1]
        s[1,4,1,0]=(-0.579990484487)*x[0]+(-0.681146890855)*x[1]
        s[1,4,1,1]=(-0.539291982427)*x[0]+(1.40646560401)*x[1]
        s[1,4,2,0]=(-0.804686819612)*x[0]+(0.617047118706)*x[1]
        s[1,4,2,1]=(-0.913106387219)*x[0]+(-0.35453310971)*x[1]
        s[2,0,0,0]=(1.07945504868)*x[0]+(0.147695653661)*x[1]
        s[2,0,0,1]=(0.0321229519775)*x[0]+(-0.80726917422)*x[1]
        s[2,0,1,0]=(-0.276983245168)*x[0]+(-1.8208105745)*x[1]
        s[2,0,1,1]=(-0.390466636115)*x[0]+(-0.753787382303)*x[1]
        s[2,0,2,0]=(-0.0551239488574)*x[0]+(-0.691793238598)*x[1]
        s[2,0,2,1]=(0.455414076199)*x[0]+(1.19885245354)*x[1]
        s[2,1,0,0]=(-0.774122061304)*x[0]+(1.10934639101)*x[1]
        s[2,1,0,1]=(-1.60749547114)*x[0]+(0.305715741213)*x[1]
        s[2,1,1,0]=(0.0697234437299)*x[0]+(0.542820812428)*x[1]
        s[2,1,1,1]=(1.29397510328)*x[0]+(-0.804845682022)*x[1]
        s[2,1,2,0]=(0.478973101459)*x[0]+(1.07219480882)*x[1]
        s[2,1,2,1]=(-0.897309279985)*x[0]+(1.75363512874)*x[1]
        s[2,2,0,0]=(1.61737724977)*x[0]+(0.161191822716)*x[1]
        s[2,2,0,1]=(0.808798403137)*x[0]+(-0.00926345040032)*x[1]
        s[2,2,1,0]=(-0.903485178097)*x[0]+(-0.36599523945)*x[1]
        s[2,2,1,1]=(0.30686972857)*x[0]+(-1.62803935845)*x[1]
        s[2,2,2,0]=(1.48915760228)*x[0]+(0.579057819566)*x[1]
        s[2,2,2,1]=(-0.540786588929)*x[0]+(-0.402843340661)*x[1]
        s[2,3,0,0]=(0.964227135928)*x[0]+(0.0210110310059)*x[1]
        s[2,3,0,1]=(0.0266958261678)*x[0]+(0.830215139026)*x[1]
        s[2,3,1,0]=(0.930518946289)*x[0]+(0.66745827854)*x[1]
        s[2,3,1,1]=(0.0363390243751)*x[0]+(0.591657283448)*x[1]
        s[2,3,2,0]=(-0.433934955699)*x[0]+(-1.57946754208)*x[1]
        s[2,3,2,1]=(-0.890298505624)*x[0]+(0.623825002267)*x[1]
        s[2,4,0,0]=(0.838929180901)*x[0]+(0.699121739125)*x[1]
        s[2,4,0,1]=(1.3886192544)*x[0]+(-0.232981076843)*x[1]
        s[2,4,1,0]=(-0.566792750082)*x[0]+(-0.631000271451)*x[1]
        s[2,4,1,1]=(-0.989529672577)*x[0]+(0.0943919436553)*x[1]
        s[2,4,2,0]=(-1.85858526639)*x[0]+(0.257423736684)*x[1]
        s[2,4,2,1]=(-1.01848069496)*x[0]+(-1.08574972329)*x[1]
        s[3,0,0,0]=(0.135022656073)*x[0]+(-0.641428277939)*x[1]
        s[3,0,0,1]=(-0.613029646479)*x[0]+(0.158334902417)*x[1]
        s[3,0,1,0]=(-1.01838576339)*x[0]+(-0.788404867889)*x[1]
        s[3,0,1,1]=(0.133395871444)*x[0]+(-0.540018893528)*x[1]
        s[3,0,2,0]=(0.359599806799)*x[0]+(-0.163549980956)*x[1]
        s[3,0,2,1]=(0.537279834479)*x[0]+(1.43219657076)*x[1]
        s[3,1,0,0]=(0.699221501837)*x[0]+(0.58098090756)*x[1]
        s[3,1,0,1]=(-0.264653821282)*x[0]+(-0.297670973468)*x[1]
        s[3,1,1,0]=(-1.14805006965)*x[0]+(0.173713527619)*x[1]
        s[3,1,1,1]=(0.582167066862)*x[0]+(0.224704535757)*x[1]
        s[3,1,2,0]=(-0.013329661508)*x[0]+(0.439567816478)*x[1]
        s[3,1,2,1]=(-0.206657167829)*x[0]+(-0.900945403197)*x[1]
        s[3,2,0,0]=(-1.34123093943)*x[0]+(-0.905024065154)*x[1]
        s[3,2,0,1]=(-0.0677797983484)*x[0]+(0.863260908229)*x[1]
        s[3,2,1,0]=(0.0651406495709)*x[0]+(1.35122911589)*x[1]
        s[3,2,1,1]=(-0.00598531484168)*x[0]+(0.690487120132)*x[1]
        s[3,2,2,0]=(0.748701424535)*x[0]+(-1.0273152862)*x[1]
        s[3,2,2,1]=(-0.239064296585)*x[0]+(-0.213292951531)*x[1]
        s[3,3,0,0]=(-0.620272397992)*x[0]+(1.00106031112)*x[1]
        s[3,3,0,1]=(-0.803133841712)*x[0]+(0.873569080382)*x[1]
        s[3,3,1,0]=(-1.187617412)*x[0]+(0.422664418077)*x[1]
        s[3,3,1,1]=(-1.51637824714)*x[0]+(-1.5177161306)*x[1]
        s[3,3,2,0]=(1.41348390435)*x[0]+(1.2134182272)*x[1]
        s[3,3,2,1]=(0.99094246846)*x[0]+(0.34796551642)*x[1]
        s[3,4,0,0]=(0.580352675436)*x[0]+(-1.21782952097)*x[1]
        s[3,4,0,1]=(0.060030794007)*x[0]+(0.711473074059)*x[1]
        s[3,4,1,0]=(0.966745511848)*x[0]+(-0.550401614505)*x[1]
        s[3,4,1,1]=(-0.791329990237)*x[0]+(-0.205150506961)*x[1]
        s[3,4,2,0]=(0.482041855799)*x[0]+(-1.09392636319)*x[1]
        s[3,4,2,1]=(-0.698074703295)*x[0]+(0.510339675206)*x[1]
        ref[0,0,0,0]=(-0.953900694954)/(o+1.)+(0.442044844728)+(0.187770040507)*0.5**o
        ref[0,0,0,1]=(0.580869130103)/(o+1.)+(0.516362992147)+(0.03478859005)*0.5**o
        ref[0,0,1,0]=(-0.411645490601)/(o+1.)+(0.406943479905)+(0.144438673502)*0.5**o
        ref[0,0,1,1]=(0.204312991686)/(o+1.)+(0.383971959467)+(0.0144755411578)*0.5**o
        ref[0,0,2,0]=(0.743369450952)/(o+1.)+(0.0895404122051)+(0.213774227914)*0.5**o
        ref[0,0,2,1]=(-0.239858711457)/(o+1.)+(-0.809763839108)+(-0.903731498821)*0.5**o
        ref[0,1,0,0]=(-0.484413922256)/(o+1.)+(0.0508933433825)+(0.520551852766)*0.5**o
        ref[0,1,0,1]=(-0.590736935429)/(o+1.)+(-0.344843181585)+(-0.406226073725)*0.5**o
        ref[0,1,1,0]=(-0.125338697971)/(o+1.)+(0.369801286198)+(0.640656647382)*0.5**o
        ref[0,1,1,1]=(0.552504077477)/(o+1.)+(0.788979766736)+(0.0532863343083)*0.5**o
        ref[0,1,2,0]=(0.306194481363)/(o+1.)+(-0.445453307231)+(-0.33042958101)*0.5**o
        ref[0,1,2,1]=(-0.818898129415)/(o+1.)+(-0.0332482175532)+(-0.34422839166)*0.5**o
        ref[0,2,0,0]=(-0.878218012751)/(o+1.)+(-0.612244128212)+(-0.729893341289)*0.5**o
        ref[0,2,0,1]=(-0.105155854997)/(o+1.)+(0.281922249648)+(-0.884960927387)*0.5**o
        ref[0,2,1,0]=(0.565149481722)/(o+1.)+(0.0625165001878)+(-0.2601498966)*0.5**o
        ref[0,2,1,1]=(-0.983424704841)/(o+1.)+(0.0730690344226)+(-0.817511294863)*0.5**o
        ref[0,2,2,0]=(-0.258286255412)/(o+1.)+(0.308348049886)+(-0.0337916339596)*0.5**o
        ref[0,2,2,1]=(0.915004878119)/(o+1.)+(-0.166545843752)+(0.929464796149)*0.5**o
        ref[0,3,0,0]=(-0.433529018481)/(o+1.)+(0.627958762155)+(-0.199883285628)*0.5**o
        ref[0,3,0,1]=(-0.945419963966)/(o+1.)+(-0.336318858392)+(0.413167864075)*0.5**o
        ref[0,3,1,0]=(0.502119767083)/(o+1.)+(0.334993773063)+(-0.700976793994)*0.5**o
        ref[0,3,1,1]=(-0.186129206617)/(o+1.)+(0.891324041438)+(0.532124671966)*0.5**o
        ref[0,3,2,0]=(0.504051134603)/(o+1.)+(-0.200874672613)+(0.834670978862)*0.5**o
        ref[0,3,2,1]=(0.436498536789)/(o+1.)+(0.545640525497)+(0.216965034296)*0.5**o
        ref[0,4,0,0]=(-0.756349034292)/(o+1.)+(0.140738322652)+(-0.166679412995)*0.5**o
        ref[0,4,0,1]=(0.780437446749)/(o+1.)+(-0.599105577138)+(0.177963809594)*0.5**o
        ref[0,4,1,0]=(0.943566244729)/(o+1.)+(-0.441716458606)+(-0.665197967238)*0.5**o
        ref[0,4,1,1]=(-0.734101358227)/(o+1.)+(0.356355202125)+(0.903603796827)*0.5**o
        ref[0,4,2,0]=(-0.43430093723)/(o+1.)+(0.383496742595)+(-0.887800546351)*0.5**o
        ref[0,4,2,1]=(-0.227167732111)/(o+1.)+(-0.274598730532)+(0.567774407038)*0.5**o
        ref[1,0,0,0]=(0.391393429129)/(o+1.)+(0.353444577314)+(0.732908188056)*0.5**o
        ref[1,0,0,1]=(-0.171565730455)/(o+1.)+(-0.259205209862)+(0.106480527466)*0.5**o
        ref[1,0,1,0]=(-0.774601901321)/(o+1.)+(0.259483708314)+(-0.403336652569)*0.5**o
        ref[1,0,1,1]=(-0.726406904963)/(o+1.)+(-0.129278181762)+(0.124746076574)*0.5**o
        ref[1,0,2,0]=(-0.617577515377)/(o+1.)+(-0.152390726783)+(0.91333753221)*0.5**o
        ref[1,0,2,1]=(0.138202581628)/(o+1.)+(-0.396089249037)+(-0.701035515806)*0.5**o
        ref[1,1,0,0]=(0.729934336346)/(o+1.)+(-0.00268233232676)+(0.52639743976)*0.5**o
        ref[1,1,0,1]=(-0.158306695958)/(o+1.)+(-0.0162211686892)+(0.359216363265)*0.5**o
        ref[1,1,1,0]=(-0.978599129855)/(o+1.)+(0.71609215355)+(-0.0970149977525)*0.5**o
        ref[1,1,1,1]=(-0.349041967673)/(o+1.)+(0.780124912357)+(-0.264654875775)*0.5**o
        ref[1,1,2,0]=(-0.809194404709)/(o+1.)+(-0.268116096378)+(-0.217942292102)*0.5**o
        ref[1,1,2,1]=(0.455578593899)/(o+1.)+(0.428430852179)+(-0.297781150997)*0.5**o
        ref[1,2,0,0]=(0.116383337262)/(o+1.)+(0.228068268367)+(-0.290734440531)*0.5**o
        ref[1,2,0,1]=(0.977031820054)/(o+1.)+(0.137033678728)+(0.508717318715)*0.5**o
        ref[1,2,1,0]=(0.282937416713)/(o+1.)+(-0.615158071804)+(-0.420484006037)*0.5**o
        ref[1,2,1,1]=(0.904627062081)/(o+1.)+(0.018901292084)+(-0.342307683794)*0.5**o
        ref[1,2,2,0]=(0.755780696034)/(o+1.)+(-0.0362605587438)+(0.0159217108891)*0.5**o
        ref[1,2,2,1]=(-0.699866517547)/(o+1.)+(0.171062073116)+(-0.212864707989)*0.5**o
        ref[1,3,0,0]=(0.228867465112)/(o+1.)+(-0.0322172708178)+(0.498591983668)*0.5**o
        ref[1,3,0,1]=(-0.289542634768)/(o+1.)+(0.110436622855)+(0.529389825891)*0.5**o
        ref[1,3,1,0]=(0.358054041082)/(o+1.)+(0.165615637362)+(-0.74073844755)*0.5**o
        ref[1,3,1,1]=(-0.60359826352)/(o+1.)+(0.517588005735)+(-0.48450757281)*0.5**o
        ref[1,3,2,0]=(-0.545672858809)/(o+1.)+(-0.618462721543)+(-0.623275076358)*0.5**o
        ref[1,3,2,1]=(-0.73428128984)/(o+1.)+(-0.0497647291554)+(-0.527628453484)*0.5**o
        ref[1,4,0,0]=(-0.04306414004)/(o+1.)+(-0.298207200434)+(-0.271430139773)*0.5**o
        ref[1,4,0,1]=(0.971916388567)/(o+1.)+(-0.0691551012311)+(0.718870311054)*0.5**o
        ref[1,4,1,0]=(-0.445510592605)/(o+1.)+(-0.088724728164)+(-0.638177326409)*0.5**o
        ref[1,4,1,1]=(0.753412648385)/(o+1.)+(-0.0777097810015)+(0.269180535197)*0.5**o
        ref[1,4,2,0]=(-0.305234413894)/(o+1.)+(0.169600008291)+(-0.221605303593)*0.5**o
        ref[1,4,2,1]=(-0.286993085073)/(o+1.)+(-0.0493395569951)+(-0.881967297866)*0.5**o
        ref[2,0,0,0]=(-0.719751624491)/(o+1.)+(0.677226475278)+(0.59244937628)*0.5**o
        ref[2,0,0,1]=(-0.670829954915)/(o+1.)+(-0.139434936863)+(0.174553606398)*0.5**o
        ref[2,0,1,0]=(-0.916962595573)/(o+1.)+(-0.372659474123)+(-0.435512275853)*0.5**o
        ref[2,0,1,1]=(-0.893849803211)/(o+1.)+(-0.341833589876)+(0.433262964544)*0.5**o
        ref[2,0,2,0]=(-0.773257190107)/(o+1.)+(0.0495078796887)+(-0.0726757567263)*0.5**o
        ref[2,0,2,1]=(0.632070005323)/(o+1.)+(0.697198313452)+(-0.372200102483)*0.5**o
        ref[2,1,0,0]=(0.459329022417)/(o+1.)+(0.249194755087)+(-0.622494202881)*0.5**o
        ref[2,1,0,1]=(0.0269909119644)/(o+1.)+(-0.210708025265)+(-0.907354591365)*0.5**o
        ref[2,1,1,0]=(0.183761641828)/(o+1.)+(-0.207261190919)+(0.843304996169)*0.5**o
        ref[2,1,1,1]=(-0.49062931094)/(o+1.)+(0.2742786733)+(0.431201385598)*0.5**o
        ref[2,1,2,0]=(0.798332846753)/(o+1.)+(0.187074177535)+(0.378686708458)*0.5**o
        ref[2,1,2,1]=(0.942487410042)/(o+1.)+(0.0268513037692)+(-0.139864168826)*0.5**o
        ref[2,2,0,0]=(-0.306480663139)/(o+1.)+(0.706708669675)+(0.671632396277)*0.5**o
        ref[2,2,0,1]=(-0.68426521806)/(o+1.)+(0.658766824582)+(0.166266521632)*0.5**o
        ref[2,2,1,0]=(-0.0872551950734)/(o+1.)+(-0.212578991849)+(-0.757067238775)*0.5**o
        ref[2,2,1,1]=(-0.779515023363)/(o+1.)+(-0.165651803902)+(-0.210350998718)*0.5**o
        ref[2,2,2,0]=(0.420662034868)/(o+1.)+(0.456572874952)+(0.734407637073)*0.5**o
        ref[2,2,2,1]=(-0.0347308970891)/(o+1.)+(-0.550927478137)+(0.192955923773)*0.5**o
        ref[2,3,0,0]=(0.279479543192)/(o+1.)+(0.340981994922)+(0.0237946338994)*0.5**o
        ref[2,3,0,1]=(0.885111721906)/(o+1.)+(0.0386593998513)+(-0.105519556415)*0.5**o
        ref[2,3,1,0]=(-0.142745975965)/(o+1.)+(0.571228905184)+(0.598265390426)*0.5**o
        ref[2,3,1,1]=(0.676813945202)/(o+1.)+(0.301993597369)+(-0.652804832118)*0.5**o
        ref[2,3,2,0]=(-0.834072218834)/(o+1.)+(-0.836092513405)+(0.492854747868)*0.5**o
        ref[2,3,2,1]=(0.451195972296)/(o+1.)+(-0.330911551268)+(-0.0558463731179)*0.5**o
        ref[2,4,0,0]=(0.604480974758)/(o+1.)+(-0.0272834841321)+(0.988136913533)*0.5**o
        ref[2,4,0,1]=(-0.672293493189)/(o+1.)+(0.436472725256)+(0.954986220232)*0.5**o
        ref[2,4,1,0]=(-0.945983571254)/(o+1.)+(0.362615161801)+(-0.97703977388)*0.5**o
        ref[2,4,1,1]=(-0.778028939318)/(o+1.)+(0.423610287746)+(-0.964329365094)*0.5**o
        ref[2,4,2,0]=(-0.138477487464)/(o+1.)+(-0.290097108281)+(-0.882489825678)*0.5**o
        ref[2,4,2,1]=(-0.953853492601)/(o+1.)+(-0.179508678068)+(-0.791359569512)*0.5**o
        ref[3,0,0,0]=(-0.199022712971)/(o+1.)+(0.253562515321)+(-0.814507939536)*0.5**o
        ref[3,0,0,1]=(0.112777444671)/(o+1.)+(-0.304438182199)+(0.0414041756638)*0.5**o
        ref[3,0,1,0]=(-0.880947344135)/(o+1.)+(-0.194192553178)+(-0.537458180783)*0.5**o
        ref[3,0,1,1]=(0.39314163058)/(o+1.)+(-0.775542968704)+(0.751321284745)*0.5**o
        ref[3,0,2,0]=(-0.482557545787)/(o+1.)+(-0.0683141681573)+(0.815235707945)*0.5**o
        ref[3,0,2,1]=(0.689498951848)/(o+1.)+(0.278187448835)+(0.723602555722)*0.5**o
        ref[3,1,0,0]=(0.293238350118)/(o+1.)+(0.126767755198)+(0.733428548881)*0.5**o
        ref[3,1,0,1]=(-0.26577191789)/(o+1.)+(0.201156903364)+(-0.698866683588)*0.5**o
        ref[3,1,1,0]=(0.439591690299)/(o+1.)+(-0.288902429619)+(-0.836123373092)*0.5**o
        ref[3,1,1,1]=(0.8910765985)/(o+1.)+(0.0553222730519)+(-0.194849541985)*0.5**o
        ref[3,1,2,0]=(-0.0320019026134)/(o+1.)+(0.477772657371)+(-0.497305257159)*0.5**o
        ref[3,1,2,1]=(-0.0499399267727)/(o+1.)+(-0.47037968496)+(-0.116903274335)*0.5**o
        ref[3,2,0,0]=(-0.276422491301)/(o+1.)+(-0.812540687574)+(-0.344751138134)*0.5**o
        ref[3,2,0,1]=(-0.00141392846802)/(o+1.)+(0.102797761295)+(0.591299515759)*0.5**o
        ref[3,2,1,0]=(0.630823139203)/(o+1.)+(0.743132641412)+(-0.700718656569)*0.5**o
        ref[3,2,1,1]=(0.490401876791)/(o+1.)+(-0.0461798489229)+(0.286459626346)*0.5**o
        ref[3,2,2,0]=(-0.787385728473)/(o+1.)+(0.0658852261331)+(0.377001414541)*0.5**o
        ref[3,2,2,1]=(-0.158857143023)/(o+1.)+(-0.421186525234)+(0.548872945376)*0.5**o
        ref[3,3,0,0]=(0.029229133788)/(o+1.)+(0.069575027013)+(0.212408725319)*0.5**o
        ref[3,3,0,1]=(0.757333397205)/(o+1.)+(0.029368913907)+(-0.74563598635)*0.5**o
        ref[3,3,1,0]=(0.0943476349516)/(o+1.)+(0.0337288993927)+(-0.926758427661)*0.5**o
        ref[3,3,1,1]=(-0.769051069188)/(o+1.)+(-0.844682211294)+(-0.575678885962)*0.5**o
        ref[3,3,2,0]=(0.246931939289)/(o+1.)+(0.756380254609)+(0.867209683046)*0.5**o
        ref[3,3,2,1]=(0.535429280541)/(o+1.)+(-0.0329131735419)+(0.869305051424)*0.5**o
        ref[3,4,0,0]=(-0.405156907998)/(o+1.)+(-0.244492906569)+(0.256665875603)*0.5**o
        ref[3,4,0,1]=(-0.140892979474)/(o+1.)+(0.516304250401)+(-0.120211653262)*0.5**o
        ref[3,4,1,0]=(0.168500977091)/(o+1.)+(0.089683045067)+(0.0684768301186)*0.5**o
        ref[3,4,1,1]=(0.479212374255)/(o+1.)+(-0.626588026593)+(-0.222516818267)*0.5**o
        ref[3,4,2,0]=(-0.563625996633)/(o+1.)+(-0.120136908079)+(0.192015305399)*0.5**o
        ref[3,4,2,1]=(-0.031361929046)/(o+1.)+(0.332029567053)+(-0.820432233148)*0.5**o
      else:
        s[0,0,0,0]=(0.186907875567)*x[0]+(0.323825050263)*x[1]+(1.15648314531)*x[2]
        s[0,0,0,1]=(1.12367405158)*x[0]+(1.13427439957)*x[1]+(0.189106618663)*x[2]
        s[0,0,1,0]=(1.76772912309)*x[0]+(0.0616670242122)*x[1]+(-1.23257311969)*x[2]
        s[0,0,1,1]=(0.0497251821243)*x[0]+(-1.47406701801)*x[1]+(0.164922402015)*x[2]
        s[0,0,2,0]=(-1.40783401802)*x[0]+(0.684901383857)*x[1]+(0.283077416398)*x[2]
        s[0,0,2,1]=(-0.283194016281)*x[0]+(-1.60164790888)*x[1]+(0.412908024034)*x[2]
        s[0,1,0,0]=(0.957812695611)*x[0]+(-0.863209638895)*x[1]+(-0.609946161661)*x[2]
        s[0,1,0,1]=(-0.410080093222)*x[0]+(0.703061988282)*x[1]+(0.0546952979936)*x[2]
        s[0,1,1,0]=(-1.43265292137)*x[0]+(-1.28132882624)*x[1]+(-0.103305431528)*x[2]
        s[0,1,1,1]=(-0.973042964661)*x[0]+(0.118946651326)*x[1]+(0.0121414797284)*x[2]
        s[0,1,2,0]=(-1.31257233483)*x[0]+(-1.90572394465)*x[1]+(-0.426472675848)*x[2]
        s[0,1,2,1]=(1.25719983663)*x[0]+(-0.68807451729)*x[1]+(0.329745219822)*x[2]
        s[0,2,0,0]=(-0.317555693596)*x[0]+(0.464722269829)*x[1]+(-0.0253415732374)*x[2]
        s[0,2,0,1]=(-0.802071257989)*x[0]+(-1.07682010382)*x[1]+(-1.52372537155)*x[2]
        s[0,2,1,0]=(-0.697501763365)*x[0]+(-1.5918814652)*x[1]+(-1.03755334358)*x[2]
        s[0,2,1,1]=(0.292457959822)*x[0]+(-0.446371848663)*x[1]+(1.32282628125)*x[2]
        s[0,2,2,0]=(1.44590220139)*x[0]+(-0.556542973367)*x[1]+(0.999665797669)*x[2]
        s[0,2,2,1]=(-0.376272847083)*x[0]+(-0.0251287305789)*x[1]+(0.364915231203)*x[2]
        s[0,3,0,0]=(0.88046028761)*x[0]+(0.372647019255)*x[1]+(-1.52356563515)*x[2]
        s[0,3,0,1]=(0.0765111785452)*x[0]+(-0.947059538185)*x[1]+(0.332098361392)*x[2]
        s[0,3,1,0]=(-1.37435064568)*x[0]+(1.04159418667)*x[1]+(0.314113199294)*x[2]
        s[0,3,1,1]=(1.03796425672)*x[0]+(-0.139508518244)*x[1]+(-0.46214547857)*x[2]
        s[0,3,2,0]=(0.285674556261)*x[0]+(-0.915609582509)*x[1]+(0.683131361973)*x[2]
        s[0,3,2,1]=(0.703275850512)*x[0]+(-0.894071596437)*x[1]+(0.118084971572)*x[2]
        s[0,4,0,0]=(-1.29507182493)*x[0]+(-0.275563564467)*x[1]+(-1.29101960297)*x[2]
        s[0,4,0,1]=(0.726127419233)*x[0]+(-0.163731035409)*x[1]+(1.45773190781)*x[2]
        s[0,4,1,0]=(-0.321962778987)*x[0]+(-0.434741013545)*x[1]+(0.348257210746)*x[2]
        s[0,4,1,1]=(0.892154653814)*x[0]+(1.12188965959)*x[1]+(-1.42589350879)*x[2]
        s[0,4,2,0]=(-1.06019594336)*x[0]+(-1.5907947093)*x[1]+(-1.64125703099)*x[2]
        s[0,4,2,1]=(0.820572048651)*x[0]+(-0.528087471135)*x[1]+(-0.0823830365488)*x[2]
        s[1,0,0,0]=(0.823455925698)*x[0]+(0.456056899485)*x[1]+(0.708907347857)*x[2]
        s[1,0,0,1]=(0.013059728892)*x[0]+(-0.772283592797)*x[1]+(0.363920439078)*x[2]
        s[1,0,1,0]=(-0.569161155945)*x[0]+(-1.60837471571)*x[1]+(-1.12748440619)*x[2]
        s[1,0,1,1]=(-1.06458634572)*x[0]+(-1.10787449393)*x[1]+(1.43157821547)*x[2]
        s[1,0,2,0]=(1.42955732758)*x[0]+(-0.769967391917)*x[1]+(0.461386087764)*x[2]
        s[1,0,2,1]=(-1.12922605294)*x[0]+(-0.256069161837)*x[1]+(0.934117328474)*x[2]
        s[1,1,0,0]=(-0.296512978109)*x[0]+(0.0247034910192)*x[1]+(-0.0918640286973)*x[2]
        s[1,1,0,1]=(0.262151201281)*x[0]+(-0.556624983637)*x[1]+(-0.82641291041)*x[2]
        s[1,1,1,0]=(-0.909049007906)*x[0]+(0.578952420411)*x[1]+(0.0963761548489)*x[2]
        s[1,1,1,1]=(0.789884036316)*x[0]+(0.714044849036)*x[1]+(-0.348148665956)*x[2]
        s[1,1,2,0]=(-0.306802924795)*x[0]+(0.166007262622)*x[1]+(0.812933157357)*x[2]
        s[1,1,2,1]=(-0.23666871482)*x[0]+(0.0180126754613)*x[1]+(0.465074288631)*x[2]
        s[1,2,0,0]=(-1.14910199315)*x[0]+(0.930765031574)*x[1]+(1.63176081675)*x[2]
        s[1,2,0,1]=(-0.220753578398)*x[0]+(0.153024000962)*x[1]+(0.29171013278)*x[2]
        s[1,2,1,0]=(-0.571208521033)*x[0]+(1.18218013588)*x[1]+(-0.0281527591651)*x[2]
        s[1,2,1,1]=(-0.0212229342253)*x[0]+(0.577228434778)*x[1]+(-0.0934776488559)*x[2]
        s[1,2,2,0]=(-0.592424182282)*x[0]+(-0.0403519194823)*x[1]+(0.150970946422)*x[2]
        s[1,2,2,1]=(-1.33143395635)*x[0]+(-1.3121470647)*x[1]+(-0.78895334942)*x[2]
        s[1,3,0,0]=(0.626711601793)*x[0]+(0.0187262812242)*x[1]+(-0.392743911033)*x[2]
        s[1,3,0,1]=(0.0360431567199)*x[0]+(-0.921408398245)*x[1]+(-0.172097608208)*x[2]
        s[1,3,1,0]=(0.0223088597951)*x[0]+(0.139631235358)*x[1]+(-0.529170239405)*x[2]
        s[1,3,1,1]=(0.11834184642)*x[0]+(-0.754799078426)*x[1]+(0.136555856977)*x[2]
        s[1,3,2,0]=(-0.288966482257)*x[0]+(-0.227333376328)*x[1]+(-0.583704417546)*x[2]
        s[1,3,2,1]=(0.263525293937)*x[0]+(0.842984179305)*x[1]+(0.592096192935)*x[2]
        s[1,4,0,0]=(-0.0145931697196)*x[0]+(-0.839359041248)*x[1]+(-0.635230965652)*x[2]
        s[1,4,0,1]=(-0.452080234732)*x[0]+(-0.500376989587)*x[1]+(-0.27541187994)*x[2]
        s[1,4,1,0]=(-0.028441856276)*x[0]+(0.99292041207)*x[1]+(-0.555205406529)*x[2]
        s[1,4,1,1]=(0.670664493076)*x[0]+(0.562911949702)*x[1]+(-0.767721105918)*x[2]
        s[1,4,2,0]=(0.960781399312)*x[0]+(0.0966843021482)*x[1]+(-0.448822164554)*x[2]
        s[1,4,2,1]=(1.19002654689)*x[0]+(0.0423440255032)*x[1]+(-0.566501161163)*x[2]
        s[2,0,0,0]=(-1.21404047236)*x[0]+(0.246576983095)*x[1]+(-1.35965095619)*x[2]
        s[2,0,0,1]=(-0.183423338333)*x[0]+(0.490089064451)*x[1]+(-0.578583461456)*x[2]
        s[2,0,1,0]=(1.12402576003)*x[0]+(-0.175494364486)*x[1]+(-0.303488944097)*x[2]
        s[2,0,1,1]=(-1.65572015787)*x[0]+(-1.0020836407)*x[1]+(0.480903327702)*x[2]
        s[2,0,2,0]=(0.145764271661)*x[0]+(-0.263090439788)*x[1]+(-1.17377189982)*x[2]
        s[2,0,2,1]=(-0.837574918907)*x[0]+(1.48561229389)*x[1]+(0.967914367565)*x[2]
        s[2,1,0,0]=(0.509756439273)*x[0]+(-0.111493103227)*x[1]+(0.748245818033)*x[2]
        s[2,1,0,1]=(0.131007923179)*x[0]+(-0.128179547101)*x[1]+(-0.755140067557)*x[2]
        s[2,1,1,0]=(-0.635852982675)*x[0]+(-0.475693144737)*x[1]+(-0.331519420739)*x[2]
        s[2,1,1,1]=(-1.29780426286)*x[0]+(0.774138127446)*x[1]+(-1.74756722025)*x[2]
        s[2,1,2,0]=(-0.14015158014)*x[0]+(-0.875354388859)*x[1]+(0.791253726662)*x[2]
        s[2,1,2,1]=(-0.0772968366884)*x[0]+(-0.937984863796)*x[1]+(-0.0602294365875)*x[2]
        s[2,2,0,0]=(0.955209462705)*x[0]+(0.849659517825)*x[1]+(0.0983431115722)*x[2]
        s[2,2,0,1]=(1.72389105142)*x[0]+(-0.412358364777)*x[1]+(-1.3083297441)*x[2]
        s[2,2,1,0]=(-0.895927963754)*x[0]+(1.65404011945)*x[1]+(0.297503560372)*x[2]
        s[2,2,1,1]=(1.34506983158)*x[0]+(-0.552690724942)*x[1]+(0.0882447776246)*x[2]
        s[2,2,2,0]=(-0.603470575964)*x[0]+(1.77788342742)*x[1]+(-0.960079624369)*x[2]
        s[2,2,2,1]=(-0.547711891427)*x[0]+(-0.562393805421)*x[1]+(-0.183240255616)*x[2]
        s[2,3,0,0]=(1.01018738642)*x[0]+(1.4085839719)*x[1]+(-0.619343450789)*x[2]
        s[2,3,0,1]=(0.525036070827)*x[0]+(-0.760210492125)*x[1]+(0.621245864477)*x[2]
        s[2,3,1,0]=(-0.786180468736)*x[0]+(0.784642311709)*x[1]+(-0.377020976891)*x[2]
        s[2,3,1,1]=(-0.333203982307)*x[0]+(-0.044265604048)*x[1]+(-1.6196925017)*x[2]
        s[2,3,2,0]=(-0.885601870306)*x[0]+(0.650248271309)*x[1]+(-1.7206378066)*x[2]
        s[2,3,2,1]=(0.0941920781315)*x[0]+(-1.24650013731)*x[1]+(0.67065624977)*x[2]
        s[2,4,0,0]=(0.0693008855416)*x[0]+(-1.33764045153)*x[1]+(0.277137873936)*x[2]
        s[2,4,0,1]=(0.887968960137)*x[0]+(1.57875979002)*x[1]+(-0.103648235191)*x[2]
        s[2,4,1,0]=(-0.547070384787)*x[0]+(-0.376770947917)*x[1]+(-1.81372790514)*x[2]
        s[2,4,1,1]=(0.375805210805)*x[0]+(0.660242065798)*x[1]+(1.69413225283)*x[2]
        s[2,4,2,0]=(-0.914715181893)*x[0]+(1.27301677765)*x[1]+(0.974031227808)*x[2]
        s[2,4,2,1]=(-0.503816280288)*x[0]+(0.552217633496)*x[1]+(-0.90127280688)*x[2]
        s[3,0,0,0]=(0.768275498738)*x[0]+(-0.811563543342)*x[1]+(-0.69998185891)*x[2]
        s[3,0,0,1]=(-0.629451202094)*x[0]+(1.02346787018)*x[1]+(0.780161524988)*x[2]
        s[3,0,1,0]=(0.584052565724)*x[0]+(-0.069634462512)*x[1]+(-1.00341091459)*x[2]
        s[3,0,1,1]=(1.03073176869)*x[0]+(1.53263439484)*x[1]+(1.03753255807)*x[2]
        s[3,0,2,0]=(0.0522744193577)*x[0]+(0.308512837882)*x[1]+(-0.315970675533)*x[2]
        s[3,0,2,1]=(0.0747430001964)*x[0]+(0.863711676177)*x[1]+(1.07705766445)*x[2]
        s[3,1,0,0]=(0.114255376522)*x[0]+(1.47818231679)*x[1]+(0.0222267873076)*x[2]
        s[3,1,0,1]=(0.613783843499)*x[0]+(0.369289287936)*x[1]+(1.08226313005)*x[2]
        s[3,1,1,0]=(0.474121107413)*x[0]+(-0.284169623677)*x[1]+(1.26739419136)*x[2]
        s[3,1,1,1]=(0.703235040854)*x[0]+(0.340511678132)*x[1]+(0.314037803901)*x[2]
        s[3,1,2,0]=(-0.754512705698)*x[0]+(0.957288483514)*x[1]+(1.38853172146)*x[2]
        s[3,1,2,1]=(0.338933480448)*x[0]+(1.30797425088)*x[1]+(-0.492819385452)*x[2]
        s[3,2,0,0]=(0.477902330587)*x[0]+(0.658604196318)*x[1]+(-0.669039618554)*x[2]
        s[3,2,0,1]=(0.584833130533)*x[0]+(0.205626741315)*x[1]+(-0.449267859148)*x[2]
        s[3,2,1,0]=(0.392564971475)*x[0]+(-0.867191184183)*x[1]+(0.508221616556)*x[2]
        s[3,2,1,1]=(0.199023224346)*x[0]+(0.210018232234)*x[1]+(0.17332311179)*x[2]
        s[3,2,2,0]=(-0.530182002398)*x[0]+(-0.70410123336)*x[1]+(0.454418407328)*x[2]
        s[3,2,2,1]=(-1.00165228123)*x[0]+(0.253353462588)*x[1]+(1.07981964164)*x[2]
        s[3,3,0,0]=(-0.00388223127856)*x[0]+(-0.256156832173)*x[1]+(-1.22618259947)*x[2]
        s[3,3,0,1]=(1.04947612909)*x[0]+(-0.590938924785)*x[1]+(-0.80453902875)*x[2]
        s[3,3,1,0]=(0.395025775135)*x[0]+(-1.30382091375)*x[1]+(1.00251618393)*x[2]
        s[3,3,1,1]=(-0.601315067206)*x[0]+(-0.226958312276)*x[1]+(-0.69720544326)*x[2]
        s[3,3,2,0]=(0.748907154335)*x[0]+(-0.694932273918)*x[1]+(0.143655252827)*x[2]
        s[3,3,2,1]=(-0.228346281761)*x[0]+(0.0818793892413)*x[1]+(0.376376078012)*x[2]
        s[3,4,0,0]=(0.675502208069)*x[0]+(0.498131679631)*x[1]+(-0.606527858224)*x[2]
        s[3,4,0,1]=(0.230994211989)*x[0]+(-1.81768645462)*x[1]+(1.59488392513)*x[2]
        s[3,4,1,0]=(-1.35866095694)*x[0]+(-0.122601842831)*x[1]+(1.73437014104)*x[2]
        s[3,4,1,1]=(0.63210638436)*x[0]+(1.50387972886)*x[1]+(0.655743288533)*x[2]
        s[3,4,2,0]=(1.42156357373)*x[0]+(-0.701544596166)*x[1]+(0.88931540397)*x[2]
        s[3,4,2,1]=(-0.405515262733)*x[0]+(0.872922189212)*x[1]+(-1.13847750709)*x[2]
        ref[0,0,0,0]=(0.415231330279)/(o+1.)+(0.982601202296)+(-0.713217663735)*0.5**o
        ref[0,0,0,1]=(0.650505784971)/(o+1.)+(0.406482330235)+(0.983584624374)*0.5**o
        ref[0,0,1,0]=(-0.615926989595)/(o+1.)+(0.113540305186)+(0.98566940684)*0.5**o
        ref[0,0,1,1]=(-1.36210768774)/(o+1.)+(0.330430698616)+(-0.558173143358)*0.5**o
        ref[0,0,2,0]=(0.589319954581)/(o+1.)+(-0.146725890489)+(-0.735723391368)*0.5**o
        ref[0,0,2,1]=(-0.819641973816)/(o+1.)+(0.0696728799133)+(-0.791637687141)*0.5**o
        ref[0,1,0,0]=(-0.984505810214)/(o+1.)+(0.0623424750886)+(0.344477755092)*0.5**o
        ref[0,1,0,1]=(0.0372238318297)/(o+1.)+(0.201167151246)+(-0.0918809412683)*0.5**o
        ref[0,1,1,0]=(-0.988162693644)/(o+1.)+(-0.658809560166)+(-0.51150536516)*0.5**o
        ref[0,1,1,1]=(0.191409590473)/(o+1.)+(-0.426836466412)+(-0.179691491256)*0.5**o
        ref[0,1,2,0]=(-1.50151479209)/(o+1.)+(-0.648874443157)+(-0.84550527693)*0.5**o
        ref[0,1,2,1]=(-1.3172677321)/(o+1.)+(0.885092333538)+(0.445953604186)*0.5**o
        ref[0,2,0,0]=(0.1269682286)/(o+1.)+(-0.175772039094)+(0.346400852583)*0.5**o
        ref[0,2,0,1]=(-1.46726533796)/(o+1.)+(-1.05015066374)+(0.164949932075)*0.5**o
        ref[0,2,1,0]=(-1.23701559967)/(o+1.)+(-0.825973196415)+(-0.437974579642)*0.5**o
        ref[0,2,1,1]=(0.132084480709)/(o+1.)+(0.0364121752536)+(0.964003561191)*0.5**o
        ref[0,2,2,0]=(-0.0575514788048)/(o+1.)+(0.574131321284)+(0.798313861933)*0.5**o
        ref[0,2,2,1]=(-0.854422215721)/(o+1.)+(0.454172855359)+(-0.0904098414555)*0.5**o
        ref[0,3,0,0]=(-1.04634289994)/(o+1.)+(0.354533324899)+(0.0668179218583)*0.5**o
        ref[0,3,0,1]=(-0.415702586981)/(o+1.)+(0.126519101056)+(-0.375785613379)*0.5**o
        ref[0,3,1,0]=(-0.268120824236)/(o+1.)+(0.500467336081)+(-0.751457107646)*0.5**o
        ref[0,3,1,1]=(0.827791290739)/(o+1.)+(-0.347418978526)+(0.303356926219)*0.5**o
        ref[0,3,2,0]=(-0.1387759961)/(o+1.)+(-0.350616137823)+(0.893204607472)*0.5**o
        ref[0,3,2,1]=(0.129310667468)/(o+1.)+(-0.252264005651)+(0.302506569481)*0.5**o
        ref[0,4,0,0]=(-0.021050677621)/(o+1.)+(-0.980639199085)+(-0.879325916576)*0.5**o
        ref[0,4,0,1]=(-0.262715302419)/(o+1.)+(1.20073556064)+(-0.118627527238)*0.5**o
        ref[0,4,1,0]=(0.261035782917)/(o+1.)+(-0.449979981215)+(0.230477597727)*0.5**o
        ref[0,4,1,1]=(-0.47960276307)/(o+1.)+(0.219069456491)+(0.629614654702)*0.5**o
        ref[0,4,2,0]=(-1.27925873031)/(o+1.)+(-1.32602054802)+(-0.360947857308)*0.5**o
        ref[0,4,2,1]=(-1.34486930774)/(o+1.)+(0.546553886393)+(0.46186307592)*0.5**o
        ref[1,0,0,0]=(0.430196940607)/(o+1.)+(0.311302638938)+(0.935617954556)*0.5**o
        ref[1,0,0,1]=(0.0865234935036)/(o+1.)+(-0.268442519337)+(0.055058120343)*0.5**o
        ref[1,0,1,0]=(-1.48387607046)/(o+1.)+(-0.786378289878)+(-0.248387627621)*0.5**o
        ref[1,0,1,1]=(0.758534313728)/(o+1.)+(-0.265213280892)+(-0.968990376127)*0.5**o
        ref[1,0,2,0]=(-0.713142570031)/(o+1.)+(0.462618013893)+(0.908882565677)*0.5**o
        ref[1,0,2,1]=(-0.482219090892)/(o+1.)+(0.307689127346)+(-0.584337050105)*0.5**o
        ref[1,1,0,0]=(-1.17250881543)/(o+1.)+(0.113442845094)+(0.581949609453)*0.5**o
        ref[1,1,0,1]=(-1.3468375595)/(o+1.)+(0.395669021834)+(-0.565387176936)*0.5**o
        ref[1,1,1,0]=(0.2018351732)/(o+1.)+(-0.157830690125)+(-0.119894225596)*0.5**o
        ref[1,1,1,1]=(1.21543817764)/(o+1.)+(-0.0497756593583)+(0.0398933604741)*0.5**o
        ref[1,1,2,0]=(-0.162294483944)/(o+1.)+(0.85994581496)+(-0.885459650791)*0.5**o
        ref[1,1,2,1]=(1.24422084677)/(o+1.)+(-0.379646444866)+(-0.238509707766)*0.5**o
        ref[1,2,0,0]=(1.5435438681)/(o+1.)+(0.08540126524)+(-0.300922543405)*0.5**o
        ref[1,2,0,1]=(-0.112210840363)/(o+1.)+(0.480051249506)+(-0.623911103306)*0.5**o
        ref[1,2,1,0]=(0.285721711645)/(o+1.)+(0.0100191655774)+(0.27705881288)*0.5**o
        ref[1,2,1,1]=(0.66930209958)/(o+1.)+(-0.542529578578)+(0.878284909271)*0.5**o
        ref[1,2,2,0]=(1.12100699995)/(o+1.)+(-0.678457673316)+(-0.245896808662)*0.5**o
        ref[1,2,2,1]=(-0.88612973059)/(o+1.)+(-1.00088764314)+(-0.544629353594)*0.5**o
        ref[1,3,0,0]=(0.387350166894)/(o+1.)+(-0.499575950447)+(0.864495705983)*0.5**o
        ref[1,3,0,1]=(-0.942393535847)/(o+1.)+(0.0303050564575)+(-0.175679426801)*0.5**o
        ref[1,3,1,0]=(-0.798050539357)/(o+1.)+(0.126548837387)+(0.177722720331)*0.5**o
        ref[1,3,1,1]=(-1.50957773465)/(o+1.)+(0.172938183698)+(0.663799992229)*0.5**o
        ref[1,3,2,0]=(0.469444503978)/(o+1.)+(-0.647836043317)+(-0.273776693474)*0.5**o
        ref[1,3,2,1]=(0.310061780349)/(o+1.)+(1.05452404891)+(-0.720504211989)*0.5**o
        ref[1,4,0,0]=(-0.820548632752)/(o+1.)+(0.105943005381)+(-0.88052055463)*0.5**o
        ref[1,4,0,1]=(-0.158467564266)/(o+1.)+(-0.709811754591)+(0.35022196919)*0.5**o
        ref[1,4,1,0]=(0.00990374166591)/(o+1.)+(0.51839921597)+(-0.637429024341)*0.5**o
        ref[1,4,1,1]=(-0.246106620377)/(o+1.)+(0.33455634807)+(0.0428492610972)*0.5**o
        ref[1,4,2,0]=(0.472702988578)/(o+1.)+(0.0688149777117)+(-0.00168940709497)*0.5**o
        ref[1,4,2,1]=(0.284541065049)/(o+1.)+(-0.0548349320689)+(0.490998210318)*0.5**o
        ref[2,0,0,0]=(-0.733338280715)/(o+1.)+(-0.47189177308)+(-0.649992618571)*0.5**o
        ref[2,0,0,1]=(-0.948815264173)/(o+1.)+(0.837962467604)+(-0.999027406375)*0.5**o
        ref[2,0,1,0]=(-0.0944539750337)/(o+1.)+(0.0746201219713)+(0.590256182541)*0.5**o
        ref[2,0,1,1]=(0.044594970161)/(o+1.)+(-0.717673746264)+(-0.786147948501)*0.5**o
        ref[2,0,2,0]=(-0.387429911457)/(o+1.)+(-0.0765750118601)+(-0.750518132769)*0.5**o
        ref[2,0,2,1]=(1.28025192232)/(o+1.)+(0.425010917096)+(-0.514322013965)*0.5**o
        ref[2,1,0,0]=(0.85161480185)/(o+1.)+(-0.117153136076)+(0.52920062438)*0.5**o
        ref[2,1,0,1]=(-0.558914411305)/(o+1.)+(-0.229298513746)+(0.265199747318)*0.5**o
        ref[2,1,1,0]=(-0.33769433116)/(o+1.)+(-0.678382612728)+(0.251394008465)*0.5**o
        ref[2,1,1,1]=(-0.990803230186)/(o+1.)+(-0.413400680719)+(-0.453628764049)*0.5**o
        ref[2,1,2,0]=(-0.377834292093)/(o+1.)+(0.521893888533)+(-0.890205727309)*0.5**o
        ref[2,1,2,1]=(-1.33150035489)/(o+1.)+(-0.0586549531766)+(0.373299124171)*0.5**o
        ref[2,2,0,0]=(-0.166255483746)/(o+1.)+(0.816713521389)+(0.436040533071)*0.5**o
        ref[2,2,0,1]=(-1.39682071748)/(o+1.)+(0.21770833884)+(0.964606982344)*0.5**o
        ref[2,2,1,0]=(1.3247136685)/(o+1.)+(-0.0466914213895)+(-0.175715109647)*0.5**o
        ref[2,2,1,1]=(-0.522454725229)/(o+1.)+(0.291262422208)+(0.820553765077)*0.5**o
        ref[2,2,2,0]=(0.136169385992)/(o+1.)+(0.253426637378)+(-0.42868943366)*0.5**o
        ref[2,2,2,1]=(-0.131390793028)/(o+1.)+(-0.715512276073)+(0.269069392711)*0.5**o
        ref[2,3,0,0]=(0.1679740824)/(o+1.)+(0.653952560954)+(0.323548703221)*0.5**o
        ref[2,3,0,1]=(-0.0511907658967)/(o+1.)+(-0.236699539579)+(0.910661288233)*0.5**o
        ref[2,3,1,0]=(0.451520708483)/(o+1.)+(-0.065786465884)+(-0.698506910633)*0.5**o
        ref[2,3,1,1]=(-0.542162878453)/(o+1.)+(-0.237944135968)+(-0.979110937669)*0.5**o
        ref[2,3,2,0]=(-0.489308922223)/(o+1.)+(-0.563117809253)+(-0.34044686487)*0.5**o
        ref[2,3,2,1]=(0.382652422829)/(o+1.)+(-0.328188452455)+(-0.207927327325)*0.5**o
        ref[2,4,0,0]=(0.0347966753187)/(o+1.)+(-0.629864354262)+(0.233730341151)*0.5**o
        ref[2,4,0,1]=(0.123939573502)/(o+1.)+(1.03666652808)+(0.165807885309)*0.5**o
        ref[2,4,1,0]=(-0.829855747878)/(o+1.)+(-0.507284312041)+(-0.89314486588)*0.5**o
        ref[2,4,1,1]=(1.80556499882)/(o+1.)+(0.554771100606)+(-0.1849276706)*0.5**o
        ref[2,4,2,0]=(0.829439412646)/(o+1.)+(0.342620313927)+(-0.182347216931)*0.5**o
        ref[2,4,2,1]=(-0.824674529166)/(o+1.)+(-0.172483000807)+(0.316769077106)*0.5**o
        ref[3,0,0,0]=(-1.81534289679)/(o+1.)+(0.235921261477)+(0.600230470324)*0.5**o
        ref[3,0,0,1]=(0.758824131522)/(o+1.)+(0.540267308765)+(-0.665180555978)*0.5**o
        ref[3,0,1,0]=(-0.61930013537)/(o+1.)+(-0.0863636993976)+(0.303034722786)*0.5**o
        ref[3,0,1,1]=(1.09849962509)/(o+1.)+(0.780381688481)+(0.941635719543)*0.5**o
        ref[3,0,2,0]=(0.984167465846)/(o+1.)+(-0.471651730612)+(0.00395257708327)*0.5**o
        ref[3,0,2,1]=(-0.0216502142754)/(o+1.)+(1.25274987136)+(-0.468337187615)*0.5**o
        ref[3,1,0,0]=(1.21762392246)/(o+1.)+(0.0235896546035)+(0.349861248945)*0.5**o
        ref[3,1,0,1]=(0.681142299068)/(o+1.)+(0.586170195383)+(0.211853571648)*0.5**o
        ref[3,1,1,0]=(1.22842568638)/(o+1.)+(0.308050279545)+(-0.387180570377)*0.5**o
        ref[3,1,1,1]=(-0.708900188728)/(o+1.)+(1.04208633918)+(-0.0174879667472)*0.5**o
        ref[3,1,2,0]=(1.79188214766)/(o+1.)+(0.301719505956)+(-0.804013660301)*0.5**o
        ref[3,1,2,1]=(0.669297533033)/(o+1.)+(0.119247079862)+(0.246296653123)*0.5**o
        ref[3,2,0,0]=(-0.791486989477)/(o+1.)+(0.204677775698)+(0.849598346434)*0.5**o
        ref[3,2,0,1]=(-0.505948254664)/(o+1.)+(0.0688840300017)+(0.70937220736)*0.5**o
        ref[3,2,1,0]=(-0.691348115315)/(o+1.)+(0.437490279091)+(-0.150037039019)*0.5**o
        ref[3,2,1,1]=(0.708369627369)/(o+1.)+(0.176205102648)+(-0.478415264295)*0.5**o
        ref[3,2,2,0]=(0.297093119787)/(o+1.)+(-0.248132872678)+(-0.580692202861)*0.5**o
        ref[3,2,2,1]=(-0.476148841282)/(o+1.)+(0.443525381364)+(-0.0793810984529)*0.5**o
        ref[3,3,0,0]=(-0.901387033571)/(o+1.)+(-0.628090499737)+(0.671346370124)*0.5**o
        ref[3,3,0,1]=(-0.123005978892)/(o+1.)+(-0.59694482259)+(0.970893799626)*0.5**o
        ref[3,3,1,0]=(-0.784454504087)/(o+1.)+(0.436648462004)+(0.00487862539679)*0.5**o
        ref[3,3,1,1]=(-1.73297919322)/(o+1.)+(-0.09410977014)+(0.395719910762)*0.5**o
        ref[3,3,2,0]=(0.0989755292654)/(o+1.)+(0.062352403584)+(-0.0260502031901)*0.5**o
        ref[3,3,2,1]=(-0.690845491742)/(o+1.)+(0.678904082779)+(-0.437053488325)*0.5**o
        ref[3,4,0,0]=(-0.801695028055)/(o+1.)+(0.731880266747)+(-0.0949594759642)*0.5**o
        ref[3,4,0,1]=(-0.263446786535)/(o+1.)+(0.0496642554438)+(0.172309958154)*0.5**o
        ref[3,4,1,0]=(0.542595531974)/(o+1.)+(0.308327980561)+(-0.906144151831)*0.5**o
        ref[3,4,1,1]=(0.637885608826)/(o+1.)+(1.16403227519)+(-0.174220757444)*0.5**o
        ref[3,4,2,0]=(0.946926166426)/(o+1.)+(0.0242541266649)+(0.613899961774)*0.5**o
        ref[3,4,2,1]=(-0.0303035732231)/(o+1.)+(0.0375314231208)+(-0.715829853629)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactZero_fromSymbol_ReducedContinuousFunction_rank0(self):
      """
      tests integral of rank 0 Symbol on the FunctionOnContactZero

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports integral on FunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=())
      s=Data(0,(),w)
      if dim==2:
        s=(1.70255303494)*x[0]+(0.300011314949)*x[1]
        ref=(0.345983004398)/(o+1.)+(0.363938720886)+(0.928703903719)*0.5**o
      else:
        s=(-1.68260770684)*x[0]+(-1.14729600003)*x[1]+(-1.35659279947)*x[2]
        ref=(-1.3902352024)/(o+1.)+(-1.03788235052)+(-0.720496602901)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactZero_fromSymbol_ReducedContinuousFunction_rank1(self):
      """
      tests integral of rank 1 Symbol on the FunctionOnContactZero

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports integral on FunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(2,))
      s=Data(0,(2,),w)
      ref=numpy.zeros((2,),numpy.float_)
      if dim==2:
        s[0]=(0.23958799559)*x[0]+(-0.0458757682877)*x[1]
        s[1]=(0.0897882061379)*x[0]+(-1.54974909413)*x[1]
        ref[0]=(0.0485704769529)/(o+1.)+(0.210351971249)+(-0.275562192148)*0.5**o
        ref[1]=(-0.821668980438)/(o+1.)+(-0.712207676061)+(0.786123444573)*0.5**o
      else:
        s[0]=(-1.44817804789)*x[0]+(0.670605136525)*x[1]+(-0.593548775192)*x[2]
        s[1]=(-0.585702105721)*x[0]+(0.0665105334471)*x[1]+(-0.228099551666)*x[2]
        ref[0]=(0.859021549917)/(o+1.)+(-0.759857813727)+(-0.710427609023)*0.5**o
        ref[1]=(-1.40806718776)/(o+1.)+(0.267420623854)+(0.125934816108)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactZero_fromSymbol_ReducedContinuousFunction_rank2(self):
      """
      tests integral of rank 2 Symbol on the FunctionOnContactZero

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports integral on FunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5))
      s=Data(0,(4, 5),w)
      ref=numpy.zeros((4, 5),numpy.float_)
      if dim==2:
        s[0,0]=(0.933177344575)*x[0]+(-0.643793974434)*x[1]
        s[0,1]=(-0.311019666872)*x[0]+(0.686195407652)*x[1]
        s[0,2]=(0.470203454822)*x[0]+(1.32374103685)*x[1]
        s[0,3]=(-0.653103411901)*x[0]+(0.477452857192)*x[1]
        s[0,4]=(1.12606394408)*x[0]+(0.0898490519564)*x[1]
        s[1,0]=(1.26644130664)*x[0]+(0.226895598587)*x[1]
        s[1,1]=(0.0271345466814)*x[0]+(0.435024033603)*x[1]
        s[1,2]=(-0.227055843926)*x[0]+(-0.144686134637)*x[1]
        s[1,3]=(0.466198287572)*x[0]+(-0.130830709376)*x[1]
        s[1,4]=(1.04066080781)*x[0]+(0.546282129071)*x[1]
        s[2,0]=(0.317469515041)*x[0]+(1.26366516514)*x[1]
        s[2,1]=(-1.20210768286)*x[0]+(-0.0533328989847)*x[1]
        s[2,2]=(0.278459393186)*x[0]+(-0.379156129627)*x[1]
        s[2,3]=(1.03459096792)*x[0]+(-0.610026027371)*x[1]
        s[2,4]=(1.288131278)*x[0]+(-0.309929698872)*x[1]
        s[3,0]=(0.32856946183)*x[0]+(0.225709965354)*x[1]
        s[3,1]=(-1.03183224237)*x[0]+(-0.0153171719792)*x[1]
        s[3,2]=(0.259516430572)*x[0]+(-0.776082702284)*x[1]
        s[3,3]=(-0.417978040907)*x[0]+(-0.0941785049007)*x[1]
        s[3,4]=(1.07527114074)*x[0]+(-0.0198401110946)*x[1]
        ref[0,0]=(-0.79959835955)/(o+1.)+(0.566758804494)+(-0.0445358792971)*0.5**o
        ref[0,1]=(0.583455543567)/(o+1.)+(0.0458448886173)+(-0.299969580021)*0.5**o
        ref[0,2]=(0.471614825642)/(o+1.)+(0.891760595232)+(-0.461191524434)*0.5**o
        ref[0,3]=(0.672045220068)/(o+1.)+(-0.56343583876)+(0.279175902744)*0.5**o
        ref[0,4]=(-0.127993678518)/(o+1.)+(0.52668612755)+(0.290534419455)*0.5**o
        ref[1,0]=(0.822774751438)/(o+1.)+(-0.011855097694)+(0.694272349175)*0.5**o
        ref[1,1]=(0.552626324952)/(o+1.)+(0.277929562319)+(-0.646326869305)*0.5**o
        ref[1,2]=(-0.565996176797)/(o+1.)+(-0.128057099164)+(0.450368396562)*0.5**o
        ref[1,3]=(0.627837338323)/(o+1.)+(-0.555359040353)+(0.818248320579)*0.5**o
        ref[1,4]=(0.722984603229)/(o+1.)+(0.0957452220354)+(0.672467889583)*0.5**o
        ref[2,0]=(0.664524759547)/(o+1.)+(0.678277833656)+(-0.439945746675)*0.5**o
        ref[2,1]=(0.908213473231)/(o+1.)+(-0.805606306623)+(-0.552441441832)*0.5**o
        ref[2,2]=(0.0908414991316)/(o+1.)+(0.131818510391)+(-0.455175256354)*0.5**o
        ref[2,3]=(-0.119095663088)/(o+1.)+(-0.223144373358)+(0.989949350348)*0.5**o
        ref[2,4]=(-0.0540728359185)/(o+1.)+(0.0748680956845)+(0.882538223681)*0.5**o
        ref[3,0]=(0.98273847633)/(o+1.)+(-0.421621406424)+(0.414783763702)*0.5**o
        ref[3,1]=(0.29285575515)/(o+1.)+(-0.462798732226)+(-0.41440770505)*0.5**o
        ref[3,2]=(-0.509767634157)/(o+1.)+(-0.480617720091)+(0.954436802627)*0.5**o
        ref[3,3]=(-0.762302422898)/(o+1.)+(0.100280539595)+(0.0495847978999)*0.5**o
        ref[3,4]=(-0.154196647936)/(o+1.)+(0.329602137701)+(0.550423402184)*0.5**o
      else:
        s[0,0]=(1.34601675616)*x[0]+(-0.451295005459)*x[1]+(-0.49622174234)*x[2]
        s[0,1]=(-0.327908015969)*x[0]+(-1.21019079469)*x[1]+(1.09622527656)*x[2]
        s[0,2]=(-0.0243507161566)*x[0]+(-0.016230100758)*x[1]+(-1.37311484633)*x[2]
        s[0,3]=(-0.445665346289)*x[0]+(-0.626824621259)*x[1]+(-0.196033254199)*x[2]
        s[0,4]=(-0.819703680211)*x[0]+(1.30953832902)*x[1]+(0.0778023021203)*x[2]
        s[1,0]=(0.219519741099)*x[0]+(-1.47335622754)*x[1]+(0.198662145648)*x[2]
        s[1,1]=(-0.65456993306)*x[0]+(-0.125758153194)*x[1]+(-1.53966098547)*x[2]
        s[1,2]=(-0.112157349953)*x[0]+(-0.459351490445)*x[1]+(1.6889234353)*x[2]
        s[1,3]=(0.102642361518)*x[0]+(1.10412193101)*x[1]+(-0.239843060928)*x[2]
        s[1,4]=(-0.232367456037)*x[0]+(-1.17836662288)*x[1]+(-0.814085860866)*x[2]
        s[2,0]=(-1.00510966212)*x[0]+(1.23960710585)*x[1]+(0.0848958213029)*x[2]
        s[2,1]=(-1.93901641139)*x[0]+(-0.0469703867512)*x[1]+(-0.21201338191)*x[2]
        s[2,2]=(1.42666776379)*x[0]+(1.04000263985)*x[1]+(0.151609334515)*x[2]
        s[2,3]=(-0.297049329338)*x[0]+(-0.103284668913)*x[1]+(-0.110261476034)*x[2]
        s[2,4]=(0.964711394219)*x[0]+(1.28218755108)*x[1]+(-1.2382220868)*x[2]
        s[3,0]=(-0.922061452692)*x[0]+(-1.36593539333)*x[1]+(1.11436583585)*x[2]
        s[3,1]=(0.284590223973)*x[0]+(1.08442017719)*x[1]+(1.5619949376)*x[2]
        s[3,2]=(-0.245207105356)*x[0]+(-1.77865635864)*x[1]+(-0.908570594143)*x[2]
        s[3,3]=(0.426334196419)*x[0]+(-0.501103031142)*x[1]+(0.199762115243)*x[2]
        s[3,4]=(1.33158896071)*x[0]+(0.44737802367)*x[1]+(-0.76095091762)*x[2]
        ref[0,0]=(-0.581088472202)/(o+1.)+(0.287163129514)+(0.405262221535)*0.5**o
        ref[0,1]=(0.695209950176)/(o+1.)+(-0.748195486472)+(0.359307488667)*0.5**o
        ref[0,2]=(0.313650504565)/(o+1.)+(-1.20553227172)+(0.683718375628)*0.5**o
        ref[0,3]=(-0.872265925416)/(o+1.)+(-0.216679466249)+(0.0371016361661)*0.5**o
        ref[0,4]=(1.07552646704)/(o+1.)+(-0.293195846094)+(0.0785021760824)*0.5**o
        ref[1,0]=(-1.22710752208)/(o+1.)+(0.409726431872)+(-0.647519682458)*0.5**o
        ref[1,1]=(-0.940490307845)/(o+1.)+(-0.82083756271)+(0.26217636154)*0.5**o
        ref[1,2]=(1.00054234642)/(o+1.)+(-0.0533842117578)+(0.223640672001)*0.5**o
        ref[1,3]=(0.0595140200914)/(o+1.)+(0.407850368171)+(0.0917064751695)*0.5**o
        ref[1,4]=(-0.50735295069)/(o+1.)+(-1.10564117596)+(0.493815362818)*0.5**o
        ref[2,0]=(0.786394742154)/(o+1.)+(-0.212261329287)+(-0.0424788185539)*0.5**o
        ref[2,1]=(-0.868240537836)/(o+1.)+(-0.186797194849)+(-0.956165252514)*0.5**o
        ref[2,2]=(-0.67113838962)/(o+1.)+(1.26208372573)+(0.765250676321)*0.5**o
        ref[2,3]=(-1.2390712788)/(o+1.)+(0.286327258603)+(0.15582128731)*0.5**o
        ref[2,4]=(-0.00831435188402)/(o+1.)+(0.305363694299)+(0.406263821787)*0.5**o
        ref[3,0]=(-0.123205370838)/(o+1.)+(-0.410740153717)+(-0.228945331902)*0.5**o
        ref[3,1]=(1.39199709306)/(o+1.)+(0.646045427731)+(0.246917390241)*0.5**o
        ref[3,2]=(-1.10740261108)/(o+1.)+(-0.860231147304)+(-0.104569152453)*0.5**o
        ref[3,3]=(-0.506214554892)/(o+1.)+(0.53544681941)+(-0.439685803409)*0.5**o
        ref[3,4]=(0.846470301583)/(o+1.)+(-0.134756583683)+(0.441058932543)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactZero_fromSymbol_ReducedContinuousFunction_rank3(self):
      """
      tests integral of rank 3 Symbol on the FunctionOnContactZero

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports integral on FunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(6, 2, 2))
      s=Data(0,(6, 2, 2),w)
      ref=numpy.zeros((6, 2, 2),numpy.float_)
      if dim==2:
        s[0,0,0]=(-0.907789262757)*x[0]+(1.43778352913)*x[1]
        s[0,0,1]=(-0.441541130755)*x[0]+(1.75922038312)*x[1]
        s[0,1,0]=(-0.387854312613)*x[0]+(1.22523701409)*x[1]
        s[0,1,1]=(-1.15521894576)*x[0]+(0.0289558035739)*x[1]
        s[1,0,0]=(-0.640400813271)*x[0]+(-1.69766965873)*x[1]
        s[1,0,1]=(-0.641296842318)*x[0]+(-1.35895483901)*x[1]
        s[1,1,0]=(-0.14735518066)*x[0]+(0.520543405774)*x[1]
        s[1,1,1]=(0.369918186173)*x[0]+(0.209781392053)*x[1]
        s[2,0,0]=(0.677808706359)*x[0]+(0.481042907356)*x[1]
        s[2,0,1]=(-0.661702781276)*x[0]+(1.86193178646)*x[1]
        s[2,1,0]=(0.323230036331)*x[0]+(-1.45705288353)*x[1]
        s[2,1,1]=(1.8644362516)*x[0]+(0.764691070666)*x[1]
        s[3,0,0]=(-1.33103240086)*x[0]+(1.29035953404)*x[1]
        s[3,0,1]=(-0.161074703654)*x[0]+(-0.127322745584)*x[1]
        s[3,1,0]=(-0.359913251027)*x[0]+(-0.0952192793759)*x[1]
        s[3,1,1]=(-0.34519536765)*x[0]+(0.483950473232)*x[1]
        s[4,0,0]=(0.981563058768)*x[0]+(-0.0744642367992)*x[1]
        s[4,0,1]=(-0.45912279344)*x[0]+(0.0011253607226)*x[1]
        s[4,1,0]=(-0.618322333127)*x[0]+(0.199638289627)*x[1]
        s[4,1,1]=(-0.371110449749)*x[0]+(-0.382992800899)*x[1]
        s[5,0,0]=(0.840829543072)*x[0]+(-0.925833368141)*x[1]
        s[5,0,1]=(-0.0348263534937)*x[0]+(0.267324292682)*x[1]
        s[5,1,0]=(0.0785260612832)*x[0]+(1.59479548515)*x[1]
        s[5,1,1]=(-0.203914067868)*x[0]+(1.28202597585)*x[1]
        ref[0,0,0]=(0.898417888273)/(o+1.)+(0.171058001117)+(-0.710539624134)*0.5**o
        ref[0,0,1]=(0.949672663046)/(o+1.)+(0.186488421776)+(-0.00497025423347)*0.5**o
        ref[0,1,0]=(0.434571276181)/(o+1.)+(0.106095788846)+(0.190619847602)*0.5**o
        ref[0,1,1]=(0.337711977059)/(o+1.)+(-0.313335995076)+(-0.837303129097)*0.5**o
        ref[1,0,0]=(-0.757199193222)/(o+1.)+(-0.36300260101)+(-0.854866076761)*0.5**o
        ref[1,0,1]=(-0.767144980208)/(o+1.)+(-0.215422826519)+(-0.802261048082)*0.5**o
        ref[1,1,0]=(-0.13190981045)/(o+1.)+(-0.0320289199048)+(0.569155875375)*0.5**o
        ref[1,1,1]=(0.0466405054813)/(o+1.)+(0.565229924314)+(-0.597400775882)*0.5**o
        ref[2,0,0]=(0.760951384359)/(o+1.)+(0.0600961970572)+(0.277707835242)*0.5**o
        ref[2,0,1]=(0.869894715615)/(o+1.)+(0.631323963868)+(-0.932313638168)*0.5**o
        ref[2,1,0]=(-0.666302256751)/(o+1.)+(-0.510864963778)+(0.554209337103)*0.5**o
        ref[2,1,1]=(0.934304180676)/(o+1.)+(0.350887235523)+(0.993048670542)*0.5**o
        ref[3,0,0]=(0.930749923608)/(o+1.)+(-0.283299908306)+(-0.404822973819)*0.5**o
        ref[3,0,1]=(-0.572270969539)/(o+1.)+(0.583432503768)+(-0.882991487235)*0.5**o
        ref[3,1,0]=(0.642892258995)/(o+1.)+(-0.0826293437351)+(-0.932766101927)*0.5**o
        ref[3,1,1]=(0.679591980627)/(o+1.)+(-0.379930907942)+(0.219024940838)*0.5**o
        ref[4,0,0]=(-0.0326170324069)/(o+1.)+(0.0903259186859)+(0.759064017004)*0.5**o
        ref[4,0,1]=(-0.129142887308)/(o+1.)+(0.279560607285)+(-0.887975759979)*0.5**o
        ref[4,1,0]=(0.752463858901)/(o+1.)+(-0.256945595979)+(-0.657256710443)*0.5**o
        ref[4,1,1]=(0.565307319705)/(o+1.)+(-0.898218881636)+(0.477027192919)*0.5**o
        ref[5,0,0]=(-0.28969980947)/(o+1.)+(0.0341656280661)+(0.136364728269)*0.5**o
        ref[5,0,1]=(-0.362525553087)/(o+1.)+(0.17876968499)+(0.237484122295)*0.5**o
        ref[5,1,0]=(0.856225350097)/(o+1.)+(0.375747136218)+(0.0656019238951)*0.5**o
        ref[5,1,1]=(0.358764765052)/(o+1.)+(0.783470816704)+(-0.847594490479)*0.5**o
      else:
        s[0,0,0]=(-0.18318998999)*x[0]+(-1.71372306128)*x[1]+(1.13788709107)*x[2]
        s[0,0,1]=(0.95308908228)*x[0]+(0.123844162735)*x[1]+(0.0868410957691)*x[2]
        s[0,1,0]=(-0.712762731169)*x[0]+(-1.86018372873)*x[1]+(-0.0783828227128)*x[2]
        s[0,1,1]=(-1.72172214895)*x[0]+(-1.25477573826)*x[1]+(1.28096692374)*x[2]
        s[1,0,0]=(0.86250397119)*x[0]+(1.44314824122)*x[1]+(0.763303582012)*x[2]
        s[1,0,1]=(1.05520701337)*x[0]+(1.04926637639)*x[1]+(-0.378977987156)*x[2]
        s[1,1,0]=(0.753550528625)*x[0]+(-1.63882951308)*x[1]+(0.259583964201)*x[2]
        s[1,1,1]=(0.778941098246)*x[0]+(-1.10564597288)*x[1]+(-1.42906071049)*x[2]
        s[2,0,0]=(-0.842438687373)*x[0]+(-0.202024764935)*x[1]+(0.289144880342)*x[2]
        s[2,0,1]=(-1.02319531994)*x[0]+(0.155875285165)*x[1]+(0.310920357117)*x[2]
        s[2,1,0]=(-1.45156693584)*x[0]+(0.12231441645)*x[1]+(0.722232222435)*x[2]
        s[2,1,1]=(-1.08553719113)*x[0]+(0.316521434206)*x[1]+(0.720555059931)*x[2]
        s[3,0,0]=(1.91846595467)*x[0]+(0.199356393766)*x[1]+(-0.438144879601)*x[2]
        s[3,0,1]=(-0.341123312011)*x[0]+(0.325154462361)*x[1]+(0.691743856063)*x[2]
        s[3,1,0]=(1.72547153856)*x[0]+(-0.822030080649)*x[1]+(-1.15169896539)*x[2]
        s[3,1,1]=(0.243194545088)*x[0]+(-0.718504738836)*x[1]+(0.560536413366)*x[2]
        s[4,0,0]=(0.248455459192)*x[0]+(0.415661600687)*x[1]+(0.21712561395)*x[2]
        s[4,0,1]=(-1.23171610721)*x[0]+(-0.443121697053)*x[1]+(0.724481514081)*x[2]
        s[4,1,0]=(-0.0327531013679)*x[0]+(0.385175130963)*x[1]+(-0.0484020348058)*x[2]
        s[4,1,1]=(-0.199960439132)*x[0]+(-0.751568103399)*x[1]+(-1.20119343449)*x[2]
        s[5,0,0]=(0.853262644583)*x[0]+(1.47653986554)*x[1]+(-0.0642737845052)*x[2]
        s[5,0,1]=(1.1703484139)*x[0]+(-0.848977140408)*x[1]+(-0.303628307573)*x[2]
        s[5,1,0]=(-0.719950626922)*x[0]+(0.192452589234)*x[1]+(0.315963976897)*x[2]
        s[5,1,1]=(0.331604591856)*x[0]+(0.428822102766)*x[1]+(-1.35258251093)*x[2]
        ref[0,0,0]=(-0.452131788567)/(o+1.)+(-0.313943872628)+(0.320993573616)*0.5**o
        ref[0,0,1]=(-1.4121302884)/(o+1.)+(0.934332411096)+(0.707239806995)*0.5**o
        ref[0,1,0]=(-0.371701930519)/(o+1.)+(-1.22914563984)+(0.178663927586)*0.5**o
        ref[0,1,1]=(-0.569427715149)/(o+1.)+(-0.102129307368)+(-0.921844633589)*0.5**o
        ref[1,0,0]=(1.07041067382)/(o+1.)+(1.04773121415)+(-0.0969173076836)*0.5**o
        ref[1,0,1]=(0.344908055221)/(o+1.)+(0.253322966116)+(0.873941415143)*0.5**o
        ref[1,1,0]=(-0.568409920368)/(o+1.)+(-0.376074272994)+(0.694863446104)*0.5**o
        ref[1,1,1]=(-1.26296359103)/(o+1.)+(-0.285972966041)+(0.0791439379881)*0.5**o
        ref[2,0,0]=(0.261580662311)/(o+1.)+(-0.0693472624675)+(-0.878204709343)*0.5**o
        ref[2,0,1]=(-0.0659237231253)/(o+1.)+(-0.101871484642)+(-0.286732985248)*0.5**o
        ref[2,1,0]=(-0.609626668188)/(o+1.)+(0.438552975912)+(-0.874499580587)*0.5**o
        ref[2,1,1]=(1.21427389191)/(o+1.)+(-0.396304653588)+(-0.470125281731)*0.5**o
        ref[3,0,0]=(0.466268456837)/(o+1.)+(0.109894182607)+(0.993620646782)*0.5**o
        ref[3,0,1]=(0.798648502977)/(o+1.)+(0.30293631931)+(-0.728746135183)*0.5**o
        ref[3,1,0]=(-0.443525593788)/(o+1.)+(-0.32762855249)+(0.85052519129)*0.5**o
        ref[3,1,1]=(0.520520221786)/(o+1.)+(-0.21037503556)+(-0.0145439310481)*0.5**o
        ref[4,0,0]=(1.51637897594)/(o+1.)+(0.0251054216628)+(-0.685347145435)*0.5**o
        ref[4,0,1]=(-0.0126278181421)/(o+1.)+(-0.255524438913)+(-0.426679594213)*0.5**o
        ref[4,1,0]=(-0.110252848596)/(o+1.)+(-0.270402535847)+(0.95507791508)*0.5**o
        ref[4,1,1]=(-0.965558516877)/(o+1.)+(-0.777022669388)+(0.366881878633)*0.5**o
        ref[5,0,0]=(1.47489510124)/(o+1.)+(0.344964232978)+(0.100705158424)*0.5**o
        ref[5,0,1]=(0.272180101244)/(o+1.)+(-0.219694112974)+(0.184951090624)*0.5**o
        ref[5,1,0]=(0.868944944511)/(o+1.)+(-0.492651110402)+(-0.0951767844975)*0.5**o
        ref[5,1,1]=(-0.112001801469)/(o+1.)+(-0.0274812221923)+(-0.425191570457)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactZero_fromSymbol_ReducedContinuousFunction_rank4(self):
      """
      tests integral of rank 4 Symbol on the FunctionOnContactZero

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports integral on FunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5, 3, 2))
      s=Data(0,(4, 5, 3, 2),w)
      ref=numpy.zeros((4, 5, 3, 2),numpy.float_)
      if dim==2:
        s[0,0,0,0]=(0.492266509376)*x[0]+(-0.374307474367)*x[1]
        s[0,0,0,1]=(0.379610803929)*x[0]+(1.26877290052)*x[1]
        s[0,0,1,0]=(0.489836401727)*x[0]+(0.0568437409848)*x[1]
        s[0,0,1,1]=(-0.0962394811856)*x[0]+(1.08297193296)*x[1]
        s[0,0,2,0]=(0.0083854747638)*x[0]+(1.12783902851)*x[1]
        s[0,0,2,1]=(-1.60969353709)*x[0]+(-1.15342435141)*x[1]
        s[0,1,0,0]=(0.718333685586)*x[0]+(-0.580409068311)*x[1]
        s[0,1,0,1]=(-0.471412983112)*x[0]+(-1.21523638921)*x[1]
        s[0,1,1,0]=(0.436952252413)*x[0]+(0.817968269393)*x[1]
        s[0,1,1,1]=(0.904303325207)*x[0]+(1.27944662005)*x[1]
        s[0,1,2,0]=(-0.646670575219)*x[0]+(-0.268471138891)*x[1]
        s[0,1,2,1]=(0.216944526796)*x[0]+(-1.44656748298)*x[1]
        s[0,2,0,0]=(-1.45710792844)*x[0]+(-1.37549168202)*x[1]
        s[0,2,0,1]=(-0.0209124548614)*x[0]+(-0.405359828227)*x[1]
        s[0,2,1,0]=(0.713696154808)*x[0]+(-0.283663569311)*x[1]
        s[0,2,1,1]=(0.0879024421072)*x[0]+(-1.74270037297)*x[1]
        s[0,2,2,0]=(-0.263019221744)*x[0]+(0.587637432145)*x[1]
        s[0,2,2,1]=(0.291949019915)*x[0]+(1.21942896685)*x[1]
        s[0,3,0,0]=(0.711503988663)*x[0]+(-0.0889987684621)*x[1]
        s[0,3,0,1]=(-0.473200478311)*x[0]+(-0.731689338365)*x[1]
        s[0,3,1,0]=(0.0347877428229)*x[0]+(0.436342776392)*x[1]
        s[0,3,1,1]=(1.33181675729)*x[0]+(0.796826790935)*x[1]
        s[0,3,2,0]=(0.975860401214)*x[0]+(-0.0388876329765)*x[1]
        s[0,3,2,1]=(0.336224987742)*x[0]+(1.40851963434)*x[1]
        s[0,4,0,0]=(-0.430668122172)*x[0]+(-0.210883679811)*x[1]
        s[0,4,0,1]=(-0.0947644917837)*x[0]+(-0.145045406149)*x[1]
        s[0,4,1,0]=(-0.975155718684)*x[0]+(0.370091078964)*x[1]
        s[0,4,1,1]=(1.59897077782)*x[0]+(-0.716757934974)*x[1]
        s[0,4,2,0]=(0.0995595315716)*x[0]+(-0.654667529963)*x[1]
        s[0,4,2,1]=(0.621429995817)*x[0]+(-0.830020781954)*x[1]
        s[1,0,0,0]=(0.460799425849)*x[0]+(1.37039134597)*x[1]
        s[1,0,0,1]=(0.484677718571)*x[0]+(-1.06817334128)*x[1]
        s[1,0,1,0]=(0.157323780788)*x[0]+(-0.816294918051)*x[1]
        s[1,0,1,1]=(0.275640307711)*x[0]+(-1.13585749962)*x[1]
        s[1,0,2,0]=(1.16452950894)*x[0]+(-1.17355094567)*x[1]
        s[1,0,2,1]=(-0.567786612292)*x[0]+(-0.78722481996)*x[1]
        s[1,1,0,0]=(-0.106381006132)*x[0]+(1.35734811758)*x[1]
        s[1,1,0,1]=(0.870362310422)*x[0]+(-0.701894980494)*x[1]
        s[1,1,1,0]=(0.463101808026)*x[0]+(-0.106531628533)*x[1]
        s[1,1,1,1]=(0.34927073998)*x[0]+(0.597282241285)*x[1]
        s[1,1,2,0]=(-1.08705596894)*x[0]+(-0.476312920631)*x[1]
        s[1,1,2,1]=(0.461228158094)*x[0]+(0.553430989166)*x[1]
        s[1,2,0,0]=(-0.565889081597)*x[0]+(0.847674515062)*x[1]
        s[1,2,0,1]=(1.22774040742)*x[0]+(0.532076088807)*x[1]
        s[1,2,1,0]=(-1.35738184718)*x[0]+(-0.010480885754)*x[1]
        s[1,2,1,1]=(0.0609383835609)*x[0]+(0.539183578894)*x[1]
        s[1,2,2,0]=(0.765013210057)*x[0]+(-0.0658319206215)*x[1]
        s[1,2,2,1]=(0.720241379555)*x[0]+(-1.29084845886)*x[1]
        s[1,3,0,0]=(1.37641602996)*x[0]+(-0.713391122816)*x[1]
        s[1,3,0,1]=(0.475350779553)*x[0]+(-0.0146303427199)*x[1]
        s[1,3,1,0]=(-0.498436122956)*x[0]+(0.446982991213)*x[1]
        s[1,3,1,1]=(-0.0198417666776)*x[0]+(-0.0330880581816)*x[1]
        s[1,3,2,0]=(-0.968203407981)*x[0]+(-1.43766997027)*x[1]
        s[1,3,2,1]=(-1.516108898)*x[0]+(0.154669696369)*x[1]
        s[1,4,0,0]=(0.107239334472)*x[0]+(-1.01814801515)*x[1]
        s[1,4,0,1]=(0.490733729058)*x[0]+(1.0617427681)*x[1]
        s[1,4,1,0]=(-0.579990484487)*x[0]+(-0.681146890855)*x[1]
        s[1,4,1,1]=(-0.539291982427)*x[0]+(1.40646560401)*x[1]
        s[1,4,2,0]=(-0.804686819612)*x[0]+(0.617047118706)*x[1]
        s[1,4,2,1]=(-0.913106387219)*x[0]+(-0.35453310971)*x[1]
        s[2,0,0,0]=(1.07945504868)*x[0]+(0.147695653661)*x[1]
        s[2,0,0,1]=(0.0321229519775)*x[0]+(-0.80726917422)*x[1]
        s[2,0,1,0]=(-0.276983245168)*x[0]+(-1.8208105745)*x[1]
        s[2,0,1,1]=(-0.390466636115)*x[0]+(-0.753787382303)*x[1]
        s[2,0,2,0]=(-0.0551239488574)*x[0]+(-0.691793238598)*x[1]
        s[2,0,2,1]=(0.455414076199)*x[0]+(1.19885245354)*x[1]
        s[2,1,0,0]=(-0.774122061304)*x[0]+(1.10934639101)*x[1]
        s[2,1,0,1]=(-1.60749547114)*x[0]+(0.305715741213)*x[1]
        s[2,1,1,0]=(0.0697234437299)*x[0]+(0.542820812428)*x[1]
        s[2,1,1,1]=(1.29397510328)*x[0]+(-0.804845682022)*x[1]
        s[2,1,2,0]=(0.478973101459)*x[0]+(1.07219480882)*x[1]
        s[2,1,2,1]=(-0.897309279985)*x[0]+(1.75363512874)*x[1]
        s[2,2,0,0]=(1.61737724977)*x[0]+(0.161191822716)*x[1]
        s[2,2,0,1]=(0.808798403137)*x[0]+(-0.00926345040032)*x[1]
        s[2,2,1,0]=(-0.903485178097)*x[0]+(-0.36599523945)*x[1]
        s[2,2,1,1]=(0.30686972857)*x[0]+(-1.62803935845)*x[1]
        s[2,2,2,0]=(1.48915760228)*x[0]+(0.579057819566)*x[1]
        s[2,2,2,1]=(-0.540786588929)*x[0]+(-0.402843340661)*x[1]
        s[2,3,0,0]=(0.964227135928)*x[0]+(0.0210110310059)*x[1]
        s[2,3,0,1]=(0.0266958261678)*x[0]+(0.830215139026)*x[1]
        s[2,3,1,0]=(0.930518946289)*x[0]+(0.66745827854)*x[1]
        s[2,3,1,1]=(0.0363390243751)*x[0]+(0.591657283448)*x[1]
        s[2,3,2,0]=(-0.433934955699)*x[0]+(-1.57946754208)*x[1]
        s[2,3,2,1]=(-0.890298505624)*x[0]+(0.623825002267)*x[1]
        s[2,4,0,0]=(0.838929180901)*x[0]+(0.699121739125)*x[1]
        s[2,4,0,1]=(1.3886192544)*x[0]+(-0.232981076843)*x[1]
        s[2,4,1,0]=(-0.566792750082)*x[0]+(-0.631000271451)*x[1]
        s[2,4,1,1]=(-0.989529672577)*x[0]+(0.0943919436553)*x[1]
        s[2,4,2,0]=(-1.85858526639)*x[0]+(0.257423736684)*x[1]
        s[2,4,2,1]=(-1.01848069496)*x[0]+(-1.08574972329)*x[1]
        s[3,0,0,0]=(0.135022656073)*x[0]+(-0.641428277939)*x[1]
        s[3,0,0,1]=(-0.613029646479)*x[0]+(0.158334902417)*x[1]
        s[3,0,1,0]=(-1.01838576339)*x[0]+(-0.788404867889)*x[1]
        s[3,0,1,1]=(0.133395871444)*x[0]+(-0.540018893528)*x[1]
        s[3,0,2,0]=(0.359599806799)*x[0]+(-0.163549980956)*x[1]
        s[3,0,2,1]=(0.537279834479)*x[0]+(1.43219657076)*x[1]
        s[3,1,0,0]=(0.699221501837)*x[0]+(0.58098090756)*x[1]
        s[3,1,0,1]=(-0.264653821282)*x[0]+(-0.297670973468)*x[1]
        s[3,1,1,0]=(-1.14805006965)*x[0]+(0.173713527619)*x[1]
        s[3,1,1,1]=(0.582167066862)*x[0]+(0.224704535757)*x[1]
        s[3,1,2,0]=(-0.013329661508)*x[0]+(0.439567816478)*x[1]
        s[3,1,2,1]=(-0.206657167829)*x[0]+(-0.900945403197)*x[1]
        s[3,2,0,0]=(-1.34123093943)*x[0]+(-0.905024065154)*x[1]
        s[3,2,0,1]=(-0.0677797983484)*x[0]+(0.863260908229)*x[1]
        s[3,2,1,0]=(0.0651406495709)*x[0]+(1.35122911589)*x[1]
        s[3,2,1,1]=(-0.00598531484168)*x[0]+(0.690487120132)*x[1]
        s[3,2,2,0]=(0.748701424535)*x[0]+(-1.0273152862)*x[1]
        s[3,2,2,1]=(-0.239064296585)*x[0]+(-0.213292951531)*x[1]
        s[3,3,0,0]=(-0.620272397992)*x[0]+(1.00106031112)*x[1]
        s[3,3,0,1]=(-0.803133841712)*x[0]+(0.873569080382)*x[1]
        s[3,3,1,0]=(-1.187617412)*x[0]+(0.422664418077)*x[1]
        s[3,3,1,1]=(-1.51637824714)*x[0]+(-1.5177161306)*x[1]
        s[3,3,2,0]=(1.41348390435)*x[0]+(1.2134182272)*x[1]
        s[3,3,2,1]=(0.99094246846)*x[0]+(0.34796551642)*x[1]
        s[3,4,0,0]=(0.580352675436)*x[0]+(-1.21782952097)*x[1]
        s[3,4,0,1]=(0.060030794007)*x[0]+(0.711473074059)*x[1]
        s[3,4,1,0]=(0.966745511848)*x[0]+(-0.550401614505)*x[1]
        s[3,4,1,1]=(-0.791329990237)*x[0]+(-0.205150506961)*x[1]
        s[3,4,2,0]=(0.482041855799)*x[0]+(-1.09392636319)*x[1]
        s[3,4,2,1]=(-0.698074703295)*x[0]+(0.510339675206)*x[1]
        ref[0,0,0,0]=(-0.953900694954)/(o+1.)+(0.442044844728)+(0.187770040507)*0.5**o
        ref[0,0,0,1]=(0.580869130103)/(o+1.)+(0.516362992147)+(0.03478859005)*0.5**o
        ref[0,0,1,0]=(-0.411645490601)/(o+1.)+(0.406943479905)+(0.144438673502)*0.5**o
        ref[0,0,1,1]=(0.204312991686)/(o+1.)+(0.383971959467)+(0.0144755411578)*0.5**o
        ref[0,0,2,0]=(0.743369450952)/(o+1.)+(0.0895404122051)+(0.213774227914)*0.5**o
        ref[0,0,2,1]=(-0.239858711457)/(o+1.)+(-0.809763839108)+(-0.903731498821)*0.5**o
        ref[0,1,0,0]=(-0.484413922256)/(o+1.)+(0.0508933433825)+(0.520551852766)*0.5**o
        ref[0,1,0,1]=(-0.590736935429)/(o+1.)+(-0.344843181585)+(-0.406226073725)*0.5**o
        ref[0,1,1,0]=(-0.125338697971)/(o+1.)+(0.369801286198)+(0.640656647382)*0.5**o
        ref[0,1,1,1]=(0.552504077477)/(o+1.)+(0.788979766736)+(0.0532863343083)*0.5**o
        ref[0,1,2,0]=(0.306194481363)/(o+1.)+(-0.445453307231)+(-0.33042958101)*0.5**o
        ref[0,1,2,1]=(-0.818898129415)/(o+1.)+(-0.0332482175532)+(-0.34422839166)*0.5**o
        ref[0,2,0,0]=(-0.878218012751)/(o+1.)+(-0.612244128212)+(-0.729893341289)*0.5**o
        ref[0,2,0,1]=(-0.105155854997)/(o+1.)+(0.281922249648)+(-0.884960927387)*0.5**o
        ref[0,2,1,0]=(0.565149481722)/(o+1.)+(0.0625165001878)+(-0.2601498966)*0.5**o
        ref[0,2,1,1]=(-0.983424704841)/(o+1.)+(0.0730690344226)+(-0.817511294863)*0.5**o
        ref[0,2,2,0]=(-0.258286255412)/(o+1.)+(0.308348049886)+(-0.0337916339596)*0.5**o
        ref[0,2,2,1]=(0.915004878119)/(o+1.)+(-0.166545843752)+(0.929464796149)*0.5**o
        ref[0,3,0,0]=(-0.433529018481)/(o+1.)+(0.627958762155)+(-0.199883285628)*0.5**o
        ref[0,3,0,1]=(-0.945419963966)/(o+1.)+(-0.336318858392)+(0.413167864075)*0.5**o
        ref[0,3,1,0]=(0.502119767083)/(o+1.)+(0.334993773063)+(-0.700976793994)*0.5**o
        ref[0,3,1,1]=(-0.186129206617)/(o+1.)+(0.891324041438)+(0.532124671966)*0.5**o
        ref[0,3,2,0]=(0.504051134603)/(o+1.)+(-0.200874672613)+(0.834670978862)*0.5**o
        ref[0,3,2,1]=(0.436498536789)/(o+1.)+(0.545640525497)+(0.216965034296)*0.5**o
        ref[0,4,0,0]=(-0.756349034292)/(o+1.)+(0.140738322652)+(-0.166679412995)*0.5**o
        ref[0,4,0,1]=(0.780437446749)/(o+1.)+(-0.599105577138)+(0.177963809594)*0.5**o
        ref[0,4,1,0]=(0.943566244729)/(o+1.)+(-0.441716458606)+(-0.665197967238)*0.5**o
        ref[0,4,1,1]=(-0.734101358227)/(o+1.)+(0.356355202125)+(0.903603796827)*0.5**o
        ref[0,4,2,0]=(-0.43430093723)/(o+1.)+(0.383496742595)+(-0.887800546351)*0.5**o
        ref[0,4,2,1]=(-0.227167732111)/(o+1.)+(-0.274598730532)+(0.567774407038)*0.5**o
        ref[1,0,0,0]=(0.391393429129)/(o+1.)+(0.353444577314)+(0.732908188056)*0.5**o
        ref[1,0,0,1]=(-0.171565730455)/(o+1.)+(-0.259205209862)+(0.106480527466)*0.5**o
        ref[1,0,1,0]=(-0.774601901321)/(o+1.)+(0.259483708314)+(-0.403336652569)*0.5**o
        ref[1,0,1,1]=(-0.726406904963)/(o+1.)+(-0.129278181762)+(0.124746076574)*0.5**o
        ref[1,0,2,0]=(-0.617577515377)/(o+1.)+(-0.152390726783)+(0.91333753221)*0.5**o
        ref[1,0,2,1]=(0.138202581628)/(o+1.)+(-0.396089249037)+(-0.701035515806)*0.5**o
        ref[1,1,0,0]=(0.729934336346)/(o+1.)+(-0.00268233232676)+(0.52639743976)*0.5**o
        ref[1,1,0,1]=(-0.158306695958)/(o+1.)+(-0.0162211686892)+(0.359216363265)*0.5**o
        ref[1,1,1,0]=(-0.978599129855)/(o+1.)+(0.71609215355)+(-0.0970149977525)*0.5**o
        ref[1,1,1,1]=(-0.349041967673)/(o+1.)+(0.780124912357)+(-0.264654875775)*0.5**o
        ref[1,1,2,0]=(-0.809194404709)/(o+1.)+(-0.268116096378)+(-0.217942292102)*0.5**o
        ref[1,1,2,1]=(0.455578593899)/(o+1.)+(0.428430852179)+(-0.297781150997)*0.5**o
        ref[1,2,0,0]=(0.116383337262)/(o+1.)+(0.228068268367)+(-0.290734440531)*0.5**o
        ref[1,2,0,1]=(0.977031820054)/(o+1.)+(0.137033678728)+(0.508717318715)*0.5**o
        ref[1,2,1,0]=(0.282937416713)/(o+1.)+(-0.615158071804)+(-0.420484006037)*0.5**o
        ref[1,2,1,1]=(0.904627062081)/(o+1.)+(0.018901292084)+(-0.342307683794)*0.5**o
        ref[1,2,2,0]=(0.755780696034)/(o+1.)+(-0.0362605587438)+(0.0159217108891)*0.5**o
        ref[1,2,2,1]=(-0.699866517547)/(o+1.)+(0.171062073116)+(-0.212864707989)*0.5**o
        ref[1,3,0,0]=(0.228867465112)/(o+1.)+(-0.0322172708178)+(0.498591983668)*0.5**o
        ref[1,3,0,1]=(-0.289542634768)/(o+1.)+(0.110436622855)+(0.529389825891)*0.5**o
        ref[1,3,1,0]=(0.358054041082)/(o+1.)+(0.165615637362)+(-0.74073844755)*0.5**o
        ref[1,3,1,1]=(-0.60359826352)/(o+1.)+(0.517588005735)+(-0.48450757281)*0.5**o
        ref[1,3,2,0]=(-0.545672858809)/(o+1.)+(-0.618462721543)+(-0.623275076358)*0.5**o
        ref[1,3,2,1]=(-0.73428128984)/(o+1.)+(-0.0497647291554)+(-0.527628453484)*0.5**o
        ref[1,4,0,0]=(-0.04306414004)/(o+1.)+(-0.298207200434)+(-0.271430139773)*0.5**o
        ref[1,4,0,1]=(0.971916388567)/(o+1.)+(-0.0691551012311)+(0.718870311054)*0.5**o
        ref[1,4,1,0]=(-0.445510592605)/(o+1.)+(-0.088724728164)+(-0.638177326409)*0.5**o
        ref[1,4,1,1]=(0.753412648385)/(o+1.)+(-0.0777097810015)+(0.269180535197)*0.5**o
        ref[1,4,2,0]=(-0.305234413894)/(o+1.)+(0.169600008291)+(-0.221605303593)*0.5**o
        ref[1,4,2,1]=(-0.286993085073)/(o+1.)+(-0.0493395569951)+(-0.881967297866)*0.5**o
        ref[2,0,0,0]=(-0.719751624491)/(o+1.)+(0.677226475278)+(0.59244937628)*0.5**o
        ref[2,0,0,1]=(-0.670829954915)/(o+1.)+(-0.139434936863)+(0.174553606398)*0.5**o
        ref[2,0,1,0]=(-0.916962595573)/(o+1.)+(-0.372659474123)+(-0.435512275853)*0.5**o
        ref[2,0,1,1]=(-0.893849803211)/(o+1.)+(-0.341833589876)+(0.433262964544)*0.5**o
        ref[2,0,2,0]=(-0.773257190107)/(o+1.)+(0.0495078796887)+(-0.0726757567263)*0.5**o
        ref[2,0,2,1]=(0.632070005323)/(o+1.)+(0.697198313452)+(-0.372200102483)*0.5**o
        ref[2,1,0,0]=(0.459329022417)/(o+1.)+(0.249194755087)+(-0.622494202881)*0.5**o
        ref[2,1,0,1]=(0.0269909119644)/(o+1.)+(-0.210708025265)+(-0.907354591365)*0.5**o
        ref[2,1,1,0]=(0.183761641828)/(o+1.)+(-0.207261190919)+(0.843304996169)*0.5**o
        ref[2,1,1,1]=(-0.49062931094)/(o+1.)+(0.2742786733)+(0.431201385598)*0.5**o
        ref[2,1,2,0]=(0.798332846753)/(o+1.)+(0.187074177535)+(0.378686708458)*0.5**o
        ref[2,1,2,1]=(0.942487410042)/(o+1.)+(0.0268513037692)+(-0.139864168826)*0.5**o
        ref[2,2,0,0]=(-0.306480663139)/(o+1.)+(0.706708669675)+(0.671632396277)*0.5**o
        ref[2,2,0,1]=(-0.68426521806)/(o+1.)+(0.658766824582)+(0.166266521632)*0.5**o
        ref[2,2,1,0]=(-0.0872551950734)/(o+1.)+(-0.212578991849)+(-0.757067238775)*0.5**o
        ref[2,2,1,1]=(-0.779515023363)/(o+1.)+(-0.165651803902)+(-0.210350998718)*0.5**o
        ref[2,2,2,0]=(0.420662034868)/(o+1.)+(0.456572874952)+(0.734407637073)*0.5**o
        ref[2,2,2,1]=(-0.0347308970891)/(o+1.)+(-0.550927478137)+(0.192955923773)*0.5**o
        ref[2,3,0,0]=(0.279479543192)/(o+1.)+(0.340981994922)+(0.0237946338994)*0.5**o
        ref[2,3,0,1]=(0.885111721906)/(o+1.)+(0.0386593998513)+(-0.105519556415)*0.5**o
        ref[2,3,1,0]=(-0.142745975965)/(o+1.)+(0.571228905184)+(0.598265390426)*0.5**o
        ref[2,3,1,1]=(0.676813945202)/(o+1.)+(0.301993597369)+(-0.652804832118)*0.5**o
        ref[2,3,2,0]=(-0.834072218834)/(o+1.)+(-0.836092513405)+(0.492854747868)*0.5**o
        ref[2,3,2,1]=(0.451195972296)/(o+1.)+(-0.330911551268)+(-0.0558463731179)*0.5**o
        ref[2,4,0,0]=(0.604480974758)/(o+1.)+(-0.0272834841321)+(0.988136913533)*0.5**o
        ref[2,4,0,1]=(-0.672293493189)/(o+1.)+(0.436472725256)+(0.954986220232)*0.5**o
        ref[2,4,1,0]=(-0.945983571254)/(o+1.)+(0.362615161801)+(-0.97703977388)*0.5**o
        ref[2,4,1,1]=(-0.778028939318)/(o+1.)+(0.423610287746)+(-0.964329365094)*0.5**o
        ref[2,4,2,0]=(-0.138477487464)/(o+1.)+(-0.290097108281)+(-0.882489825678)*0.5**o
        ref[2,4,2,1]=(-0.953853492601)/(o+1.)+(-0.179508678068)+(-0.791359569512)*0.5**o
        ref[3,0,0,0]=(-0.199022712971)/(o+1.)+(0.253562515321)+(-0.814507939536)*0.5**o
        ref[3,0,0,1]=(0.112777444671)/(o+1.)+(-0.304438182199)+(0.0414041756638)*0.5**o
        ref[3,0,1,0]=(-0.880947344135)/(o+1.)+(-0.194192553178)+(-0.537458180783)*0.5**o
        ref[3,0,1,1]=(0.39314163058)/(o+1.)+(-0.775542968704)+(0.751321284745)*0.5**o
        ref[3,0,2,0]=(-0.482557545787)/(o+1.)+(-0.0683141681573)+(0.815235707945)*0.5**o
        ref[3,0,2,1]=(0.689498951848)/(o+1.)+(0.278187448835)+(0.723602555722)*0.5**o
        ref[3,1,0,0]=(0.293238350118)/(o+1.)+(0.126767755198)+(0.733428548881)*0.5**o
        ref[3,1,0,1]=(-0.26577191789)/(o+1.)+(0.201156903364)+(-0.698866683588)*0.5**o
        ref[3,1,1,0]=(0.439591690299)/(o+1.)+(-0.288902429619)+(-0.836123373092)*0.5**o
        ref[3,1,1,1]=(0.8910765985)/(o+1.)+(0.0553222730519)+(-0.194849541985)*0.5**o
        ref[3,1,2,0]=(-0.0320019026134)/(o+1.)+(0.477772657371)+(-0.497305257159)*0.5**o
        ref[3,1,2,1]=(-0.0499399267727)/(o+1.)+(-0.47037968496)+(-0.116903274335)*0.5**o
        ref[3,2,0,0]=(-0.276422491301)/(o+1.)+(-0.812540687574)+(-0.344751138134)*0.5**o
        ref[3,2,0,1]=(-0.00141392846802)/(o+1.)+(0.102797761295)+(0.591299515759)*0.5**o
        ref[3,2,1,0]=(0.630823139203)/(o+1.)+(0.743132641412)+(-0.700718656569)*0.5**o
        ref[3,2,1,1]=(0.490401876791)/(o+1.)+(-0.0461798489229)+(0.286459626346)*0.5**o
        ref[3,2,2,0]=(-0.787385728473)/(o+1.)+(0.0658852261331)+(0.377001414541)*0.5**o
        ref[3,2,2,1]=(-0.158857143023)/(o+1.)+(-0.421186525234)+(0.548872945376)*0.5**o
        ref[3,3,0,0]=(0.029229133788)/(o+1.)+(0.069575027013)+(0.212408725319)*0.5**o
        ref[3,3,0,1]=(0.757333397205)/(o+1.)+(0.029368913907)+(-0.74563598635)*0.5**o
        ref[3,3,1,0]=(0.0943476349516)/(o+1.)+(0.0337288993927)+(-0.926758427661)*0.5**o
        ref[3,3,1,1]=(-0.769051069188)/(o+1.)+(-0.844682211294)+(-0.575678885962)*0.5**o
        ref[3,3,2,0]=(0.246931939289)/(o+1.)+(0.756380254609)+(0.867209683046)*0.5**o
        ref[3,3,2,1]=(0.535429280541)/(o+1.)+(-0.0329131735419)+(0.869305051424)*0.5**o
        ref[3,4,0,0]=(-0.405156907998)/(o+1.)+(-0.244492906569)+(0.256665875603)*0.5**o
        ref[3,4,0,1]=(-0.140892979474)/(o+1.)+(0.516304250401)+(-0.120211653262)*0.5**o
        ref[3,4,1,0]=(0.168500977091)/(o+1.)+(0.089683045067)+(0.0684768301186)*0.5**o
        ref[3,4,1,1]=(0.479212374255)/(o+1.)+(-0.626588026593)+(-0.222516818267)*0.5**o
        ref[3,4,2,0]=(-0.563625996633)/(o+1.)+(-0.120136908079)+(0.192015305399)*0.5**o
        ref[3,4,2,1]=(-0.031361929046)/(o+1.)+(0.332029567053)+(-0.820432233148)*0.5**o
      else:
        s[0,0,0,0]=(0.186907875567)*x[0]+(0.323825050263)*x[1]+(1.15648314531)*x[2]
        s[0,0,0,1]=(1.12367405158)*x[0]+(1.13427439957)*x[1]+(0.189106618663)*x[2]
        s[0,0,1,0]=(1.76772912309)*x[0]+(0.0616670242122)*x[1]+(-1.23257311969)*x[2]
        s[0,0,1,1]=(0.0497251821243)*x[0]+(-1.47406701801)*x[1]+(0.164922402015)*x[2]
        s[0,0,2,0]=(-1.40783401802)*x[0]+(0.684901383857)*x[1]+(0.283077416398)*x[2]
        s[0,0,2,1]=(-0.283194016281)*x[0]+(-1.60164790888)*x[1]+(0.412908024034)*x[2]
        s[0,1,0,0]=(0.957812695611)*x[0]+(-0.863209638895)*x[1]+(-0.609946161661)*x[2]
        s[0,1,0,1]=(-0.410080093222)*x[0]+(0.703061988282)*x[1]+(0.0546952979936)*x[2]
        s[0,1,1,0]=(-1.43265292137)*x[0]+(-1.28132882624)*x[1]+(-0.103305431528)*x[2]
        s[0,1,1,1]=(-0.973042964661)*x[0]+(0.118946651326)*x[1]+(0.0121414797284)*x[2]
        s[0,1,2,0]=(-1.31257233483)*x[0]+(-1.90572394465)*x[1]+(-0.426472675848)*x[2]
        s[0,1,2,1]=(1.25719983663)*x[0]+(-0.68807451729)*x[1]+(0.329745219822)*x[2]
        s[0,2,0,0]=(-0.317555693596)*x[0]+(0.464722269829)*x[1]+(-0.0253415732374)*x[2]
        s[0,2,0,1]=(-0.802071257989)*x[0]+(-1.07682010382)*x[1]+(-1.52372537155)*x[2]
        s[0,2,1,0]=(-0.697501763365)*x[0]+(-1.5918814652)*x[1]+(-1.03755334358)*x[2]
        s[0,2,1,1]=(0.292457959822)*x[0]+(-0.446371848663)*x[1]+(1.32282628125)*x[2]
        s[0,2,2,0]=(1.44590220139)*x[0]+(-0.556542973367)*x[1]+(0.999665797669)*x[2]
        s[0,2,2,1]=(-0.376272847083)*x[0]+(-0.0251287305789)*x[1]+(0.364915231203)*x[2]
        s[0,3,0,0]=(0.88046028761)*x[0]+(0.372647019255)*x[1]+(-1.52356563515)*x[2]
        s[0,3,0,1]=(0.0765111785452)*x[0]+(-0.947059538185)*x[1]+(0.332098361392)*x[2]
        s[0,3,1,0]=(-1.37435064568)*x[0]+(1.04159418667)*x[1]+(0.314113199294)*x[2]
        s[0,3,1,1]=(1.03796425672)*x[0]+(-0.139508518244)*x[1]+(-0.46214547857)*x[2]
        s[0,3,2,0]=(0.285674556261)*x[0]+(-0.915609582509)*x[1]+(0.683131361973)*x[2]
        s[0,3,2,1]=(0.703275850512)*x[0]+(-0.894071596437)*x[1]+(0.118084971572)*x[2]
        s[0,4,0,0]=(-1.29507182493)*x[0]+(-0.275563564467)*x[1]+(-1.29101960297)*x[2]
        s[0,4,0,1]=(0.726127419233)*x[0]+(-0.163731035409)*x[1]+(1.45773190781)*x[2]
        s[0,4,1,0]=(-0.321962778987)*x[0]+(-0.434741013545)*x[1]+(0.348257210746)*x[2]
        s[0,4,1,1]=(0.892154653814)*x[0]+(1.12188965959)*x[1]+(-1.42589350879)*x[2]
        s[0,4,2,0]=(-1.06019594336)*x[0]+(-1.5907947093)*x[1]+(-1.64125703099)*x[2]
        s[0,4,2,1]=(0.820572048651)*x[0]+(-0.528087471135)*x[1]+(-0.0823830365488)*x[2]
        s[1,0,0,0]=(0.823455925698)*x[0]+(0.456056899485)*x[1]+(0.708907347857)*x[2]
        s[1,0,0,1]=(0.013059728892)*x[0]+(-0.772283592797)*x[1]+(0.363920439078)*x[2]
        s[1,0,1,0]=(-0.569161155945)*x[0]+(-1.60837471571)*x[1]+(-1.12748440619)*x[2]
        s[1,0,1,1]=(-1.06458634572)*x[0]+(-1.10787449393)*x[1]+(1.43157821547)*x[2]
        s[1,0,2,0]=(1.42955732758)*x[0]+(-0.769967391917)*x[1]+(0.461386087764)*x[2]
        s[1,0,2,1]=(-1.12922605294)*x[0]+(-0.256069161837)*x[1]+(0.934117328474)*x[2]
        s[1,1,0,0]=(-0.296512978109)*x[0]+(0.0247034910192)*x[1]+(-0.0918640286973)*x[2]
        s[1,1,0,1]=(0.262151201281)*x[0]+(-0.556624983637)*x[1]+(-0.82641291041)*x[2]
        s[1,1,1,0]=(-0.909049007906)*x[0]+(0.578952420411)*x[1]+(0.0963761548489)*x[2]
        s[1,1,1,1]=(0.789884036316)*x[0]+(0.714044849036)*x[1]+(-0.348148665956)*x[2]
        s[1,1,2,0]=(-0.306802924795)*x[0]+(0.166007262622)*x[1]+(0.812933157357)*x[2]
        s[1,1,2,1]=(-0.23666871482)*x[0]+(0.0180126754613)*x[1]+(0.465074288631)*x[2]
        s[1,2,0,0]=(-1.14910199315)*x[0]+(0.930765031574)*x[1]+(1.63176081675)*x[2]
        s[1,2,0,1]=(-0.220753578398)*x[0]+(0.153024000962)*x[1]+(0.29171013278)*x[2]
        s[1,2,1,0]=(-0.571208521033)*x[0]+(1.18218013588)*x[1]+(-0.0281527591651)*x[2]
        s[1,2,1,1]=(-0.0212229342253)*x[0]+(0.577228434778)*x[1]+(-0.0934776488559)*x[2]
        s[1,2,2,0]=(-0.592424182282)*x[0]+(-0.0403519194823)*x[1]+(0.150970946422)*x[2]
        s[1,2,2,1]=(-1.33143395635)*x[0]+(-1.3121470647)*x[1]+(-0.78895334942)*x[2]
        s[1,3,0,0]=(0.626711601793)*x[0]+(0.0187262812242)*x[1]+(-0.392743911033)*x[2]
        s[1,3,0,1]=(0.0360431567199)*x[0]+(-0.921408398245)*x[1]+(-0.172097608208)*x[2]
        s[1,3,1,0]=(0.0223088597951)*x[0]+(0.139631235358)*x[1]+(-0.529170239405)*x[2]
        s[1,3,1,1]=(0.11834184642)*x[0]+(-0.754799078426)*x[1]+(0.136555856977)*x[2]
        s[1,3,2,0]=(-0.288966482257)*x[0]+(-0.227333376328)*x[1]+(-0.583704417546)*x[2]
        s[1,3,2,1]=(0.263525293937)*x[0]+(0.842984179305)*x[1]+(0.592096192935)*x[2]
        s[1,4,0,0]=(-0.0145931697196)*x[0]+(-0.839359041248)*x[1]+(-0.635230965652)*x[2]
        s[1,4,0,1]=(-0.452080234732)*x[0]+(-0.500376989587)*x[1]+(-0.27541187994)*x[2]
        s[1,4,1,0]=(-0.028441856276)*x[0]+(0.99292041207)*x[1]+(-0.555205406529)*x[2]
        s[1,4,1,1]=(0.670664493076)*x[0]+(0.562911949702)*x[1]+(-0.767721105918)*x[2]
        s[1,4,2,0]=(0.960781399312)*x[0]+(0.0966843021482)*x[1]+(-0.448822164554)*x[2]
        s[1,4,2,1]=(1.19002654689)*x[0]+(0.0423440255032)*x[1]+(-0.566501161163)*x[2]
        s[2,0,0,0]=(-1.21404047236)*x[0]+(0.246576983095)*x[1]+(-1.35965095619)*x[2]
        s[2,0,0,1]=(-0.183423338333)*x[0]+(0.490089064451)*x[1]+(-0.578583461456)*x[2]
        s[2,0,1,0]=(1.12402576003)*x[0]+(-0.175494364486)*x[1]+(-0.303488944097)*x[2]
        s[2,0,1,1]=(-1.65572015787)*x[0]+(-1.0020836407)*x[1]+(0.480903327702)*x[2]
        s[2,0,2,0]=(0.145764271661)*x[0]+(-0.263090439788)*x[1]+(-1.17377189982)*x[2]
        s[2,0,2,1]=(-0.837574918907)*x[0]+(1.48561229389)*x[1]+(0.967914367565)*x[2]
        s[2,1,0,0]=(0.509756439273)*x[0]+(-0.111493103227)*x[1]+(0.748245818033)*x[2]
        s[2,1,0,1]=(0.131007923179)*x[0]+(-0.128179547101)*x[1]+(-0.755140067557)*x[2]
        s[2,1,1,0]=(-0.635852982675)*x[0]+(-0.475693144737)*x[1]+(-0.331519420739)*x[2]
        s[2,1,1,1]=(-1.29780426286)*x[0]+(0.774138127446)*x[1]+(-1.74756722025)*x[2]
        s[2,1,2,0]=(-0.14015158014)*x[0]+(-0.875354388859)*x[1]+(0.791253726662)*x[2]
        s[2,1,2,1]=(-0.0772968366884)*x[0]+(-0.937984863796)*x[1]+(-0.0602294365875)*x[2]
        s[2,2,0,0]=(0.955209462705)*x[0]+(0.849659517825)*x[1]+(0.0983431115722)*x[2]
        s[2,2,0,1]=(1.72389105142)*x[0]+(-0.412358364777)*x[1]+(-1.3083297441)*x[2]
        s[2,2,1,0]=(-0.895927963754)*x[0]+(1.65404011945)*x[1]+(0.297503560372)*x[2]
        s[2,2,1,1]=(1.34506983158)*x[0]+(-0.552690724942)*x[1]+(0.0882447776246)*x[2]
        s[2,2,2,0]=(-0.603470575964)*x[0]+(1.77788342742)*x[1]+(-0.960079624369)*x[2]
        s[2,2,2,1]=(-0.547711891427)*x[0]+(-0.562393805421)*x[1]+(-0.183240255616)*x[2]
        s[2,3,0,0]=(1.01018738642)*x[0]+(1.4085839719)*x[1]+(-0.619343450789)*x[2]
        s[2,3,0,1]=(0.525036070827)*x[0]+(-0.760210492125)*x[1]+(0.621245864477)*x[2]
        s[2,3,1,0]=(-0.786180468736)*x[0]+(0.784642311709)*x[1]+(-0.377020976891)*x[2]
        s[2,3,1,1]=(-0.333203982307)*x[0]+(-0.044265604048)*x[1]+(-1.6196925017)*x[2]
        s[2,3,2,0]=(-0.885601870306)*x[0]+(0.650248271309)*x[1]+(-1.7206378066)*x[2]
        s[2,3,2,1]=(0.0941920781315)*x[0]+(-1.24650013731)*x[1]+(0.67065624977)*x[2]
        s[2,4,0,0]=(0.0693008855416)*x[0]+(-1.33764045153)*x[1]+(0.277137873936)*x[2]
        s[2,4,0,1]=(0.887968960137)*x[0]+(1.57875979002)*x[1]+(-0.103648235191)*x[2]
        s[2,4,1,0]=(-0.547070384787)*x[0]+(-0.376770947917)*x[1]+(-1.81372790514)*x[2]
        s[2,4,1,1]=(0.375805210805)*x[0]+(0.660242065798)*x[1]+(1.69413225283)*x[2]
        s[2,4,2,0]=(-0.914715181893)*x[0]+(1.27301677765)*x[1]+(0.974031227808)*x[2]
        s[2,4,2,1]=(-0.503816280288)*x[0]+(0.552217633496)*x[1]+(-0.90127280688)*x[2]
        s[3,0,0,0]=(0.768275498738)*x[0]+(-0.811563543342)*x[1]+(-0.69998185891)*x[2]
        s[3,0,0,1]=(-0.629451202094)*x[0]+(1.02346787018)*x[1]+(0.780161524988)*x[2]
        s[3,0,1,0]=(0.584052565724)*x[0]+(-0.069634462512)*x[1]+(-1.00341091459)*x[2]
        s[3,0,1,1]=(1.03073176869)*x[0]+(1.53263439484)*x[1]+(1.03753255807)*x[2]
        s[3,0,2,0]=(0.0522744193577)*x[0]+(0.308512837882)*x[1]+(-0.315970675533)*x[2]
        s[3,0,2,1]=(0.0747430001964)*x[0]+(0.863711676177)*x[1]+(1.07705766445)*x[2]
        s[3,1,0,0]=(0.114255376522)*x[0]+(1.47818231679)*x[1]+(0.0222267873076)*x[2]
        s[3,1,0,1]=(0.613783843499)*x[0]+(0.369289287936)*x[1]+(1.08226313005)*x[2]
        s[3,1,1,0]=(0.474121107413)*x[0]+(-0.284169623677)*x[1]+(1.26739419136)*x[2]
        s[3,1,1,1]=(0.703235040854)*x[0]+(0.340511678132)*x[1]+(0.314037803901)*x[2]
        s[3,1,2,0]=(-0.754512705698)*x[0]+(0.957288483514)*x[1]+(1.38853172146)*x[2]
        s[3,1,2,1]=(0.338933480448)*x[0]+(1.30797425088)*x[1]+(-0.492819385452)*x[2]
        s[3,2,0,0]=(0.477902330587)*x[0]+(0.658604196318)*x[1]+(-0.669039618554)*x[2]
        s[3,2,0,1]=(0.584833130533)*x[0]+(0.205626741315)*x[1]+(-0.449267859148)*x[2]
        s[3,2,1,0]=(0.392564971475)*x[0]+(-0.867191184183)*x[1]+(0.508221616556)*x[2]
        s[3,2,1,1]=(0.199023224346)*x[0]+(0.210018232234)*x[1]+(0.17332311179)*x[2]
        s[3,2,2,0]=(-0.530182002398)*x[0]+(-0.70410123336)*x[1]+(0.454418407328)*x[2]
        s[3,2,2,1]=(-1.00165228123)*x[0]+(0.253353462588)*x[1]+(1.07981964164)*x[2]
        s[3,3,0,0]=(-0.00388223127856)*x[0]+(-0.256156832173)*x[1]+(-1.22618259947)*x[2]
        s[3,3,0,1]=(1.04947612909)*x[0]+(-0.590938924785)*x[1]+(-0.80453902875)*x[2]
        s[3,3,1,0]=(0.395025775135)*x[0]+(-1.30382091375)*x[1]+(1.00251618393)*x[2]
        s[3,3,1,1]=(-0.601315067206)*x[0]+(-0.226958312276)*x[1]+(-0.69720544326)*x[2]
        s[3,3,2,0]=(0.748907154335)*x[0]+(-0.694932273918)*x[1]+(0.143655252827)*x[2]
        s[3,3,2,1]=(-0.228346281761)*x[0]+(0.0818793892413)*x[1]+(0.376376078012)*x[2]
        s[3,4,0,0]=(0.675502208069)*x[0]+(0.498131679631)*x[1]+(-0.606527858224)*x[2]
        s[3,4,0,1]=(0.230994211989)*x[0]+(-1.81768645462)*x[1]+(1.59488392513)*x[2]
        s[3,4,1,0]=(-1.35866095694)*x[0]+(-0.122601842831)*x[1]+(1.73437014104)*x[2]
        s[3,4,1,1]=(0.63210638436)*x[0]+(1.50387972886)*x[1]+(0.655743288533)*x[2]
        s[3,4,2,0]=(1.42156357373)*x[0]+(-0.701544596166)*x[1]+(0.88931540397)*x[2]
        s[3,4,2,1]=(-0.405515262733)*x[0]+(0.872922189212)*x[1]+(-1.13847750709)*x[2]
        ref[0,0,0,0]=(0.415231330279)/(o+1.)+(0.982601202296)+(-0.713217663735)*0.5**o
        ref[0,0,0,1]=(0.650505784971)/(o+1.)+(0.406482330235)+(0.983584624374)*0.5**o
        ref[0,0,1,0]=(-0.615926989595)/(o+1.)+(0.113540305186)+(0.98566940684)*0.5**o
        ref[0,0,1,1]=(-1.36210768774)/(o+1.)+(0.330430698616)+(-0.558173143358)*0.5**o
        ref[0,0,2,0]=(0.589319954581)/(o+1.)+(-0.146725890489)+(-0.735723391368)*0.5**o
        ref[0,0,2,1]=(-0.819641973816)/(o+1.)+(0.0696728799133)+(-0.791637687141)*0.5**o
        ref[0,1,0,0]=(-0.984505810214)/(o+1.)+(0.0623424750886)+(0.344477755092)*0.5**o
        ref[0,1,0,1]=(0.0372238318297)/(o+1.)+(0.201167151246)+(-0.0918809412683)*0.5**o
        ref[0,1,1,0]=(-0.988162693644)/(o+1.)+(-0.658809560166)+(-0.51150536516)*0.5**o
        ref[0,1,1,1]=(0.191409590473)/(o+1.)+(-0.426836466412)+(-0.179691491256)*0.5**o
        ref[0,1,2,0]=(-1.50151479209)/(o+1.)+(-0.648874443157)+(-0.84550527693)*0.5**o
        ref[0,1,2,1]=(-1.3172677321)/(o+1.)+(0.885092333538)+(0.445953604186)*0.5**o
        ref[0,2,0,0]=(0.1269682286)/(o+1.)+(-0.175772039094)+(0.346400852583)*0.5**o
        ref[0,2,0,1]=(-1.46726533796)/(o+1.)+(-1.05015066374)+(0.164949932075)*0.5**o
        ref[0,2,1,0]=(-1.23701559967)/(o+1.)+(-0.825973196415)+(-0.437974579642)*0.5**o
        ref[0,2,1,1]=(0.132084480709)/(o+1.)+(0.0364121752536)+(0.964003561191)*0.5**o
        ref[0,2,2,0]=(-0.0575514788048)/(o+1.)+(0.574131321284)+(0.798313861933)*0.5**o
        ref[0,2,2,1]=(-0.854422215721)/(o+1.)+(0.454172855359)+(-0.0904098414555)*0.5**o
        ref[0,3,0,0]=(-1.04634289994)/(o+1.)+(0.354533324899)+(0.0668179218583)*0.5**o
        ref[0,3,0,1]=(-0.415702586981)/(o+1.)+(0.126519101056)+(-0.375785613379)*0.5**o
        ref[0,3,1,0]=(-0.268120824236)/(o+1.)+(0.500467336081)+(-0.751457107646)*0.5**o
        ref[0,3,1,1]=(0.827791290739)/(o+1.)+(-0.347418978526)+(0.303356926219)*0.5**o
        ref[0,3,2,0]=(-0.1387759961)/(o+1.)+(-0.350616137823)+(0.893204607472)*0.5**o
        ref[0,3,2,1]=(0.129310667468)/(o+1.)+(-0.252264005651)+(0.302506569481)*0.5**o
        ref[0,4,0,0]=(-0.021050677621)/(o+1.)+(-0.980639199085)+(-0.879325916576)*0.5**o
        ref[0,4,0,1]=(-0.262715302419)/(o+1.)+(1.20073556064)+(-0.118627527238)*0.5**o
        ref[0,4,1,0]=(0.261035782917)/(o+1.)+(-0.449979981215)+(0.230477597727)*0.5**o
        ref[0,4,1,1]=(-0.47960276307)/(o+1.)+(0.219069456491)+(0.629614654702)*0.5**o
        ref[0,4,2,0]=(-1.27925873031)/(o+1.)+(-1.32602054802)+(-0.360947857308)*0.5**o
        ref[0,4,2,1]=(-1.34486930774)/(o+1.)+(0.546553886393)+(0.46186307592)*0.5**o
        ref[1,0,0,0]=(0.430196940607)/(o+1.)+(0.311302638938)+(0.935617954556)*0.5**o
        ref[1,0,0,1]=(0.0865234935036)/(o+1.)+(-0.268442519337)+(0.055058120343)*0.5**o
        ref[1,0,1,0]=(-1.48387607046)/(o+1.)+(-0.786378289878)+(-0.248387627621)*0.5**o
        ref[1,0,1,1]=(0.758534313728)/(o+1.)+(-0.265213280892)+(-0.968990376127)*0.5**o
        ref[1,0,2,0]=(-0.713142570031)/(o+1.)+(0.462618013893)+(0.908882565677)*0.5**o
        ref[1,0,2,1]=(-0.482219090892)/(o+1.)+(0.307689127346)+(-0.584337050105)*0.5**o
        ref[1,1,0,0]=(-1.17250881543)/(o+1.)+(0.113442845094)+(0.581949609453)*0.5**o
        ref[1,1,0,1]=(-1.3468375595)/(o+1.)+(0.395669021834)+(-0.565387176936)*0.5**o
        ref[1,1,1,0]=(0.2018351732)/(o+1.)+(-0.157830690125)+(-0.119894225596)*0.5**o
        ref[1,1,1,1]=(1.21543817764)/(o+1.)+(-0.0497756593583)+(0.0398933604741)*0.5**o
        ref[1,1,2,0]=(-0.162294483944)/(o+1.)+(0.85994581496)+(-0.885459650791)*0.5**o
        ref[1,1,2,1]=(1.24422084677)/(o+1.)+(-0.379646444866)+(-0.238509707766)*0.5**o
        ref[1,2,0,0]=(1.5435438681)/(o+1.)+(0.08540126524)+(-0.300922543405)*0.5**o
        ref[1,2,0,1]=(-0.112210840363)/(o+1.)+(0.480051249506)+(-0.623911103306)*0.5**o
        ref[1,2,1,0]=(0.285721711645)/(o+1.)+(0.0100191655774)+(0.27705881288)*0.5**o
        ref[1,2,1,1]=(0.66930209958)/(o+1.)+(-0.542529578578)+(0.878284909271)*0.5**o
        ref[1,2,2,0]=(1.12100699995)/(o+1.)+(-0.678457673316)+(-0.245896808662)*0.5**o
        ref[1,2,2,1]=(-0.88612973059)/(o+1.)+(-1.00088764314)+(-0.544629353594)*0.5**o
        ref[1,3,0,0]=(0.387350166894)/(o+1.)+(-0.499575950447)+(0.864495705983)*0.5**o
        ref[1,3,0,1]=(-0.942393535847)/(o+1.)+(0.0303050564575)+(-0.175679426801)*0.5**o
        ref[1,3,1,0]=(-0.798050539357)/(o+1.)+(0.126548837387)+(0.177722720331)*0.5**o
        ref[1,3,1,1]=(-1.50957773465)/(o+1.)+(0.172938183698)+(0.663799992229)*0.5**o
        ref[1,3,2,0]=(0.469444503978)/(o+1.)+(-0.647836043317)+(-0.273776693474)*0.5**o
        ref[1,3,2,1]=(0.310061780349)/(o+1.)+(1.05452404891)+(-0.720504211989)*0.5**o
        ref[1,4,0,0]=(-0.820548632752)/(o+1.)+(0.105943005381)+(-0.88052055463)*0.5**o
        ref[1,4,0,1]=(-0.158467564266)/(o+1.)+(-0.709811754591)+(0.35022196919)*0.5**o
        ref[1,4,1,0]=(0.00990374166591)/(o+1.)+(0.51839921597)+(-0.637429024341)*0.5**o
        ref[1,4,1,1]=(-0.246106620377)/(o+1.)+(0.33455634807)+(0.0428492610972)*0.5**o
        ref[1,4,2,0]=(0.472702988578)/(o+1.)+(0.0688149777117)+(-0.00168940709497)*0.5**o
        ref[1,4,2,1]=(0.284541065049)/(o+1.)+(-0.0548349320689)+(0.490998210318)*0.5**o
        ref[2,0,0,0]=(-0.733338280715)/(o+1.)+(-0.47189177308)+(-0.649992618571)*0.5**o
        ref[2,0,0,1]=(-0.948815264173)/(o+1.)+(0.837962467604)+(-0.999027406375)*0.5**o
        ref[2,0,1,0]=(-0.0944539750337)/(o+1.)+(0.0746201219713)+(0.590256182541)*0.5**o
        ref[2,0,1,1]=(0.044594970161)/(o+1.)+(-0.717673746264)+(-0.786147948501)*0.5**o
        ref[2,0,2,0]=(-0.387429911457)/(o+1.)+(-0.0765750118601)+(-0.750518132769)*0.5**o
        ref[2,0,2,1]=(1.28025192232)/(o+1.)+(0.425010917096)+(-0.514322013965)*0.5**o
        ref[2,1,0,0]=(0.85161480185)/(o+1.)+(-0.117153136076)+(0.52920062438)*0.5**o
        ref[2,1,0,1]=(-0.558914411305)/(o+1.)+(-0.229298513746)+(0.265199747318)*0.5**o
        ref[2,1,1,0]=(-0.33769433116)/(o+1.)+(-0.678382612728)+(0.251394008465)*0.5**o
        ref[2,1,1,1]=(-0.990803230186)/(o+1.)+(-0.413400680719)+(-0.453628764049)*0.5**o
        ref[2,1,2,0]=(-0.377834292093)/(o+1.)+(0.521893888533)+(-0.890205727309)*0.5**o
        ref[2,1,2,1]=(-1.33150035489)/(o+1.)+(-0.0586549531766)+(0.373299124171)*0.5**o
        ref[2,2,0,0]=(-0.166255483746)/(o+1.)+(0.816713521389)+(0.436040533071)*0.5**o
        ref[2,2,0,1]=(-1.39682071748)/(o+1.)+(0.21770833884)+(0.964606982344)*0.5**o
        ref[2,2,1,0]=(1.3247136685)/(o+1.)+(-0.0466914213895)+(-0.175715109647)*0.5**o
        ref[2,2,1,1]=(-0.522454725229)/(o+1.)+(0.291262422208)+(0.820553765077)*0.5**o
        ref[2,2,2,0]=(0.136169385992)/(o+1.)+(0.253426637378)+(-0.42868943366)*0.5**o
        ref[2,2,2,1]=(-0.131390793028)/(o+1.)+(-0.715512276073)+(0.269069392711)*0.5**o
        ref[2,3,0,0]=(0.1679740824)/(o+1.)+(0.653952560954)+(0.323548703221)*0.5**o
        ref[2,3,0,1]=(-0.0511907658967)/(o+1.)+(-0.236699539579)+(0.910661288233)*0.5**o
        ref[2,3,1,0]=(0.451520708483)/(o+1.)+(-0.065786465884)+(-0.698506910633)*0.5**o
        ref[2,3,1,1]=(-0.542162878453)/(o+1.)+(-0.237944135968)+(-0.979110937669)*0.5**o
        ref[2,3,2,0]=(-0.489308922223)/(o+1.)+(-0.563117809253)+(-0.34044686487)*0.5**o
        ref[2,3,2,1]=(0.382652422829)/(o+1.)+(-0.328188452455)+(-0.207927327325)*0.5**o
        ref[2,4,0,0]=(0.0347966753187)/(o+1.)+(-0.629864354262)+(0.233730341151)*0.5**o
        ref[2,4,0,1]=(0.123939573502)/(o+1.)+(1.03666652808)+(0.165807885309)*0.5**o
        ref[2,4,1,0]=(-0.829855747878)/(o+1.)+(-0.507284312041)+(-0.89314486588)*0.5**o
        ref[2,4,1,1]=(1.80556499882)/(o+1.)+(0.554771100606)+(-0.1849276706)*0.5**o
        ref[2,4,2,0]=(0.829439412646)/(o+1.)+(0.342620313927)+(-0.182347216931)*0.5**o
        ref[2,4,2,1]=(-0.824674529166)/(o+1.)+(-0.172483000807)+(0.316769077106)*0.5**o
        ref[3,0,0,0]=(-1.81534289679)/(o+1.)+(0.235921261477)+(0.600230470324)*0.5**o
        ref[3,0,0,1]=(0.758824131522)/(o+1.)+(0.540267308765)+(-0.665180555978)*0.5**o
        ref[3,0,1,0]=(-0.61930013537)/(o+1.)+(-0.0863636993976)+(0.303034722786)*0.5**o
        ref[3,0,1,1]=(1.09849962509)/(o+1.)+(0.780381688481)+(0.941635719543)*0.5**o
        ref[3,0,2,0]=(0.984167465846)/(o+1.)+(-0.471651730612)+(0.00395257708327)*0.5**o
        ref[3,0,2,1]=(-0.0216502142754)/(o+1.)+(1.25274987136)+(-0.468337187615)*0.5**o
        ref[3,1,0,0]=(1.21762392246)/(o+1.)+(0.0235896546035)+(0.349861248945)*0.5**o
        ref[3,1,0,1]=(0.681142299068)/(o+1.)+(0.586170195383)+(0.211853571648)*0.5**o
        ref[3,1,1,0]=(1.22842568638)/(o+1.)+(0.308050279545)+(-0.387180570377)*0.5**o
        ref[3,1,1,1]=(-0.708900188728)/(o+1.)+(1.04208633918)+(-0.0174879667472)*0.5**o
        ref[3,1,2,0]=(1.79188214766)/(o+1.)+(0.301719505956)+(-0.804013660301)*0.5**o
        ref[3,1,2,1]=(0.669297533033)/(o+1.)+(0.119247079862)+(0.246296653123)*0.5**o
        ref[3,2,0,0]=(-0.791486989477)/(o+1.)+(0.204677775698)+(0.849598346434)*0.5**o
        ref[3,2,0,1]=(-0.505948254664)/(o+1.)+(0.0688840300017)+(0.70937220736)*0.5**o
        ref[3,2,1,0]=(-0.691348115315)/(o+1.)+(0.437490279091)+(-0.150037039019)*0.5**o
        ref[3,2,1,1]=(0.708369627369)/(o+1.)+(0.176205102648)+(-0.478415264295)*0.5**o
        ref[3,2,2,0]=(0.297093119787)/(o+1.)+(-0.248132872678)+(-0.580692202861)*0.5**o
        ref[3,2,2,1]=(-0.476148841282)/(o+1.)+(0.443525381364)+(-0.0793810984529)*0.5**o
        ref[3,3,0,0]=(-0.901387033571)/(o+1.)+(-0.628090499737)+(0.671346370124)*0.5**o
        ref[3,3,0,1]=(-0.123005978892)/(o+1.)+(-0.59694482259)+(0.970893799626)*0.5**o
        ref[3,3,1,0]=(-0.784454504087)/(o+1.)+(0.436648462004)+(0.00487862539679)*0.5**o
        ref[3,3,1,1]=(-1.73297919322)/(o+1.)+(-0.09410977014)+(0.395719910762)*0.5**o
        ref[3,3,2,0]=(0.0989755292654)/(o+1.)+(0.062352403584)+(-0.0260502031901)*0.5**o
        ref[3,3,2,1]=(-0.690845491742)/(o+1.)+(0.678904082779)+(-0.437053488325)*0.5**o
        ref[3,4,0,0]=(-0.801695028055)/(o+1.)+(0.731880266747)+(-0.0949594759642)*0.5**o
        ref[3,4,0,1]=(-0.263446786535)/(o+1.)+(0.0496642554438)+(0.172309958154)*0.5**o
        ref[3,4,1,0]=(0.542595531974)/(o+1.)+(0.308327980561)+(-0.906144151831)*0.5**o
        ref[3,4,1,1]=(0.637885608826)/(o+1.)+(1.16403227519)+(-0.174220757444)*0.5**o
        ref[3,4,2,0]=(0.946926166426)/(o+1.)+(0.0242541266649)+(0.613899961774)*0.5**o
        ref[3,4,2,1]=(-0.0303035732231)/(o+1.)+(0.0375314231208)+(-0.715829853629)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactZero_fromSymbol_FunctionOnContactZero_rank0(self):
      """
      tests integral of rank 0 Symbol on the FunctionOnContactZero

      assumptions: FunctionOnContactZero(self.domain) exists
                   self.domain supports integral on FunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      w=FunctionOnContactZero(self.domain)
      x=w.getX()
      arg=Symbol(shape=())
      s=Data(0,(),w)
      if dim==2:
        s=(0.793574201557)*x[0]**o+(0.516960263083)*x[0]+(-0.448328841896)*x[1]**o+(-0.142637802972)*x[1]
        ref=(-0.448328841896)/(o+1.)+(0.187161230055)+(0.793574201557)*0.5**o
      else:
        s=(-0.338096271642)*x[0]**o+(0.968157263648)*x[0]+(0.646364944878)*x[1]**o+(-0.80014418872)*x[1]+(0.0561188904819)*x[2]**o+(0.413390558956)*x[2]
        ref=(0.702483835359)/(o+1.)+(0.290701816942)+(-0.338096271642)*0.5**o
      res=integrate(arg)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactZero_fromSymbol_FunctionOnContactZero_rank1(self):
      """
      tests integral of rank 1 Symbol on the FunctionOnContactZero

      assumptions: FunctionOnContactZero(self.domain) exists
                   self.domain supports integral on FunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      w=FunctionOnContactZero(self.domain)
      x=w.getX()
      arg=Symbol(shape=(2,))
      s=Data(0,(2,),w)
      ref=numpy.zeros((2,),numpy.float_)
      if dim==2:
        s[0]=(-0.298300777435)*x[0]**o+(-0.656203877839)*x[0]+(0.706465159872)*x[1]**o+(-0.441355227209)*x[1]
        s[1]=(-0.323920359168)*x[0]**o+(0.167106922584)*x[0]+(-0.77484608349)*x[1]**o+(0.961265696466)*x[1]
        ref[0]=(0.706465159872)/(o+1.)+(-0.548779552524)+(-0.298300777435)*0.5**o
        ref[1]=(-0.77484608349)/(o+1.)+(0.564186309525)+(-0.323920359168)*0.5**o
      else:
        s[0]=(0.226749576468)*x[0]**o+(-0.794303615675)*x[0]+(-0.475688985171)*x[1]**o+(-0.502986687652)*x[1]+(0.620418017904)*x[2]**o+(-0.201211072609)*x[2]
        s[1]=(-0.623757173924)*x[0]**o+(0.408817385275)*x[0]+(0.732106423025)*x[1]**o+(-0.414680272664)*x[1]+(-0.37214615142)*x[2]**o+(0.0433104237128)*x[2]
        ref[0]=(0.144729032732)/(o+1.)+(-0.749250687968)+(0.226749576468)*0.5**o
        ref[1]=(0.359960271605)/(o+1.)+(0.0187237681618)+(-0.623757173924)*0.5**o
      res=integrate(arg)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactZero_fromSymbol_FunctionOnContactZero_rank2(self):
      """
      tests integral of rank 2 Symbol on the FunctionOnContactZero

      assumptions: FunctionOnContactZero(self.domain) exists
                   self.domain supports integral on FunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      w=FunctionOnContactZero(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5))
      s=Data(0,(4, 5),w)
      ref=numpy.zeros((4, 5),numpy.float_)
      if dim==2:
        s[0,0]=(0.111040625394)*x[0]**o+(-0.262719374505)*x[0]+(-0.629693203515)*x[1]**o+(-0.483036975224)*x[1]
        s[0,1]=(-0.710606684727)*x[0]**o+(-0.0460165503366)*x[0]+(-0.293965417547)*x[1]**o+(-0.495255124461)*x[1]
        s[0,2]=(-0.777570630133)*x[0]**o+(-0.116630316032)*x[0]+(0.137509413983)*x[1]**o+(0.819368394153)*x[1]
        s[0,3]=(-0.710576800354)*x[0]**o+(-0.580982318739)*x[0]+(0.88040631872)*x[1]**o+(0.906791051848)*x[1]
        s[0,4]=(-0.182839498651)*x[0]**o+(-0.73087599847)*x[0]+(0.343867628394)*x[1]**o+(0.430107584469)*x[1]
        s[1,0]=(0.69614992851)*x[0]**o+(-0.897687606816)*x[0]+(0.0635059035444)*x[1]**o+(-0.664283784877)*x[1]
        s[1,1]=(-0.838368623112)*x[0]**o+(0.235356849847)*x[0]+(-0.575094104243)*x[1]**o+(0.317897877318)*x[1]
        s[1,2]=(-0.466268810559)*x[0]**o+(-0.0899593004601)*x[0]+(-0.966257251835)*x[1]**o+(-0.144068791611)*x[1]
        s[1,3]=(0.0053613868128)*x[0]**o+(0.0318146753697)*x[0]+(0.679315501165)*x[1]**o+(-0.0722416861767)*x[1]
        s[1,4]=(0.933407356129)*x[0]**o+(-0.495175209275)*x[0]+(-0.6009471862)*x[1]**o+(-0.539903879411)*x[1]
        s[2,0]=(0.557935263092)*x[0]**o+(0.858947740632)*x[0]+(0.0661071238236)*x[1]**o+(-0.92607178646)*x[1]
        s[2,1]=(-0.992502095953)*x[0]**o+(0.725486544204)*x[0]+(0.966008980795)*x[1]**o+(-0.687474869885)*x[1]
        s[2,2]=(0.131446552365)*x[0]**o+(-0.23682138542)*x[0]+(0.762830166878)*x[1]**o+(0.154873065449)*x[1]
        s[2,3]=(-0.325278661025)*x[0]**o+(0.00830856266952)*x[0]+(-0.67055674604)*x[1]**o+(-0.200351789931)*x[1]
        s[2,4]=(-0.153591894122)*x[0]**o+(0.11301998318)*x[0]+(-0.931207923183)*x[1]**o+(-0.808854650606)*x[1]
        s[3,0]=(0.482659628909)*x[0]**o+(0.625042019107)*x[0]+(-0.0579000114037)*x[1]**o+(0.468282154942)*x[1]
        s[3,1]=(-0.324897146141)*x[0]**o+(0.107637712241)*x[0]+(-0.802284595948)*x[1]**o+(0.462102155569)*x[1]
        s[3,2]=(0.0324851257443)*x[0]**o+(0.88112504986)*x[0]+(0.364393243039)*x[1]**o+(0.279546598993)*x[1]
        s[3,3]=(0.362592102084)*x[0]**o+(0.531299996192)*x[0]+(-0.133406058793)*x[1]**o+(0.331414049109)*x[1]
        s[3,4]=(0.527090134663)*x[0]**o+(0.992717007892)*x[0]+(0.0417616039807)*x[1]**o+(-0.447250742864)*x[1]
        ref[0,0]=(-0.629693203515)/(o+1.)+(-0.372878174865)+(0.111040625394)*0.5**o
        ref[0,1]=(-0.293965417547)/(o+1.)+(-0.270635837399)+(-0.710606684727)*0.5**o
        ref[0,2]=(0.137509413983)/(o+1.)+(0.351369039061)+(-0.777570630133)*0.5**o
        ref[0,3]=(0.88040631872)/(o+1.)+(0.162904366555)+(-0.710576800354)*0.5**o
        ref[0,4]=(0.343867628394)/(o+1.)+(-0.150384207)+(-0.182839498651)*0.5**o
        ref[1,0]=(0.0635059035444)/(o+1.)+(-0.780985695846)+(0.69614992851)*0.5**o
        ref[1,1]=(-0.575094104243)/(o+1.)+(0.276627363583)+(-0.838368623112)*0.5**o
        ref[1,2]=(-0.966257251835)/(o+1.)+(-0.117014046036)+(-0.466268810559)*0.5**o
        ref[1,3]=(0.679315501165)/(o+1.)+(-0.0202135054035)+(0.0053613868128)*0.5**o
        ref[1,4]=(-0.6009471862)/(o+1.)+(-0.517539544343)+(0.933407356129)*0.5**o
        ref[2,0]=(0.0661071238236)/(o+1.)+(-0.0335620229141)+(0.557935263092)*0.5**o
        ref[2,1]=(0.966008980795)/(o+1.)+(0.0190058371594)+(-0.992502095953)*0.5**o
        ref[2,2]=(0.762830166878)/(o+1.)+(-0.0409741599856)+(0.131446552365)*0.5**o
        ref[2,3]=(-0.67055674604)/(o+1.)+(-0.0960216136307)+(-0.325278661025)*0.5**o
        ref[2,4]=(-0.931207923183)/(o+1.)+(-0.347917333713)+(-0.153591894122)*0.5**o
        ref[3,0]=(-0.0579000114037)/(o+1.)+(0.546662087024)+(0.482659628909)*0.5**o
        ref[3,1]=(-0.802284595948)/(o+1.)+(0.284869933905)+(-0.324897146141)*0.5**o
        ref[3,2]=(0.364393243039)/(o+1.)+(0.580335824427)+(0.0324851257443)*0.5**o
        ref[3,3]=(-0.133406058793)/(o+1.)+(0.431357022651)+(0.362592102084)*0.5**o
        ref[3,4]=(0.0417616039807)/(o+1.)+(0.272733132514)+(0.527090134663)*0.5**o
      else:
        s[0,0]=(-0.910274365459)*x[0]**o+(0.444602331451)*x[0]+(0.109320774249)*x[1]**o+(-0.340302571967)*x[1]+(-0.43434237629)*x[2]**o+(-0.943276146561)*x[2]
        s[0,1]=(0.290437327261)*x[0]**o+(0.936756757082)*x[0]+(0.169426804577)*x[1]**o+(-0.606271363634)*x[1]+(0.0955072153861)*x[2]**o+(0.500289189372)*x[2]
        s[0,2]=(0.857537342473)*x[0]**o+(-0.441518068791)*x[0]+(0.280884170231)*x[1]**o+(-0.712646882141)*x[1]+(-0.771612131879)*x[2]**o+(-0.191796438089)*x[2]
        s[0,3]=(0.171520107567)*x[0]**o+(0.92204336001)*x[0]+(-0.370919720606)*x[1]**o+(0.768895739632)*x[1]+(0.784096316797)*x[2]**o+(-0.687371757606)*x[2]
        s[0,4]=(-0.941363513955)*x[0]**o+(-0.715641149458)*x[0]+(0.309819925901)*x[1]**o+(0.807925106464)*x[1]+(0.283008337009)*x[2]**o+(-0.858020235388)*x[2]
        s[1,0]=(-0.368063286241)*x[0]**o+(-0.379538647626)*x[0]+(0.568731253476)*x[1]**o+(-0.129142499862)*x[1]+(0.664990905083)*x[2]**o+(0.72526514854)*x[2]
        s[1,1]=(0.609862576834)*x[0]**o+(0.168414778244)*x[0]+(0.705960386503)*x[1]**o+(0.414950596481)*x[1]+(-0.324853936067)*x[2]**o+(0.705193129155)*x[2]
        s[1,2]=(0.259927679551)*x[0]**o+(0.023313779932)*x[0]+(0.542815998551)*x[1]**o+(-0.980335573131)*x[1]+(-0.104377098432)*x[2]**o+(0.678830057048)*x[2]
        s[1,3]=(-0.592662443082)*x[0]**o+(0.887389713188)*x[0]+(-0.907629711077)*x[1]**o+(0.965433874409)*x[1]+(-0.0179359825534)*x[2]**o+(-0.768070640771)*x[2]
        s[1,4]=(0.0233212032451)*x[0]**o+(-0.0505203178694)*x[0]+(-0.465871409785)*x[1]**o+(-0.437317133846)*x[1]+(0.208545612303)*x[2]**o+(-0.0502465428942)*x[2]
        s[2,0]=(-0.273603162201)*x[0]**o+(0.192745661279)*x[0]+(0.296585107806)*x[1]**o+(0.46173411046)*x[1]+(0.673984995689)*x[2]**o+(-0.88275015509)*x[2]
        s[2,1]=(0.36805707552)*x[0]**o+(-0.33705395449)*x[0]+(0.0140175482223)*x[1]**o+(0.64760814101)*x[1]+(-0.91874701962)*x[2]**o+(0.383318293978)*x[2]
        s[2,2]=(-0.900273344488)*x[0]**o+(0.498842169958)*x[0]+(-0.498406294921)*x[1]**o+(-0.403932835154)*x[1]+(-0.156723389073)*x[2]**o+(0.214962470693)*x[2]
        s[2,3]=(-0.798208224472)*x[0]**o+(-0.746730197648)*x[0]+(-0.397704785717)*x[1]**o+(-0.970985500613)*x[1]+(0.750200448417)*x[2]**o+(0.974572822346)*x[2]
        s[2,4]=(-0.464282027917)*x[0]**o+(-0.754333918275)*x[0]+(0.332816676589)*x[1]**o+(-0.600568600942)*x[1]+(0.188339826223)*x[2]**o+(0.956665673953)*x[2]
        s[3,0]=(0.291526453393)*x[0]**o+(0.259245798805)*x[0]+(0.742708355753)*x[1]**o+(0.21348170853)*x[1]+(-0.983407115555)*x[2]**o+(-0.862966239373)*x[2]
        s[3,1]=(0.874250384958)*x[0]**o+(-0.542706319171)*x[0]+(0.513206274058)*x[1]**o+(-0.493584668769)*x[1]+(-0.990552049402)*x[2]**o+(-0.0611365067488)*x[2]
        s[3,2]=(-0.620092725842)*x[0]**o+(-0.0940854652025)*x[0]+(-0.788220564782)*x[1]**o+(0.952851796216)*x[1]+(-0.102879423616)*x[2]**o+(-0.559035384428)*x[2]
        s[3,3]=(-0.149822264616)*x[0]**o+(-0.113017264993)*x[0]+(-0.472046202247)*x[1]**o+(-0.21062255325)*x[1]+(-0.531066124816)*x[2]**o+(0.307214441703)*x[2]
        s[3,4]=(0.412388911019)*x[0]**o+(-0.0814495987981)*x[0]+(0.176049376352)*x[1]**o+(-0.660623309779)*x[1]+(0.573030010589)*x[2]**o+(-0.671602921264)*x[2]
        ref[0,0]=(-0.325021602041)/(o+1.)+(-0.419488193539)+(-0.910274365459)*0.5**o
        ref[0,1]=(0.264934019963)/(o+1.)+(0.41538729141)+(0.290437327261)*0.5**o
        ref[0,2]=(-0.490727961648)/(o+1.)+(-0.672980694511)+(0.857537342473)*0.5**o
        ref[0,3]=(0.413176596192)/(o+1.)+(0.501783671018)+(0.171520107567)*0.5**o
        ref[0,4]=(0.592828262909)/(o+1.)+(-0.382868139192)+(-0.941363513955)*0.5**o
        ref[1,0]=(1.23372215856)/(o+1.)+(0.108292000526)+(-0.368063286241)*0.5**o
        ref[1,1]=(0.381106450436)/(o+1.)+(0.64427925194)+(0.609862576834)*0.5**o
        ref[1,2]=(0.438438900119)/(o+1.)+(-0.139095868075)+(0.259927679551)*0.5**o
        ref[1,3]=(-0.92556569363)/(o+1.)+(0.542376473413)+(-0.592662443082)*0.5**o
        ref[1,4]=(-0.257325797482)/(o+1.)+(-0.269041997305)+(0.0233212032451)*0.5**o
        ref[2,0]=(0.970570103495)/(o+1.)+(-0.114135191676)+(-0.273603162201)*0.5**o
        ref[2,1]=(-0.904729471398)/(o+1.)+(0.346936240249)+(0.36805707552)*0.5**o
        ref[2,2]=(-0.655129683994)/(o+1.)+(0.154935902749)+(-0.900273344488)*0.5**o
        ref[2,3]=(0.352495662699)/(o+1.)+(-0.371571437957)+(-0.798208224472)*0.5**o
        ref[2,4]=(0.521156502812)/(o+1.)+(-0.199118422633)+(-0.464282027917)*0.5**o
        ref[3,0]=(-0.240698759801)/(o+1.)+(-0.195119366019)+(0.291526453393)*0.5**o
        ref[3,1]=(-0.477345775345)/(o+1.)+(-0.548713747345)+(0.874250384958)*0.5**o
        ref[3,2]=(-0.891099988398)/(o+1.)+(0.149865473293)+(-0.620092725842)*0.5**o
        ref[3,3]=(-1.00311232706)/(o+1.)+(-0.00821268827029)+(-0.149822264616)*0.5**o
        ref[3,4]=(0.749079386941)/(o+1.)+(-0.70683791492)+(0.412388911019)*0.5**o
      res=integrate(arg)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactZero_fromSymbol_FunctionOnContactZero_rank3(self):
      """
      tests integral of rank 3 Symbol on the FunctionOnContactZero

      assumptions: FunctionOnContactZero(self.domain) exists
                   self.domain supports integral on FunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      w=FunctionOnContactZero(self.domain)
      x=w.getX()
      arg=Symbol(shape=(6, 2, 2))
      s=Data(0,(6, 2, 2),w)
      ref=numpy.zeros((6, 2, 2),numpy.float_)
      if dim==2:
        s[0,0,0]=(0.467159870975)*x[0]**o+(-0.231363355451)*x[0]+(-0.522613470003)*x[1]**o+(-0.396872827596)*x[1]
        s[0,0,1]=(0.483392039154)*x[0]**o+(-0.0481934692208)*x[0]+(0.59831432517)*x[1]**o+(0.753587205182)*x[1]
        s[0,1,0]=(-0.642178324261)*x[0]**o+(0.580441968971)*x[0]+(0.19099567725)*x[1]**o+(-0.409684325755)*x[1]
        s[0,1,1]=(-0.963913869717)*x[0]**o+(-0.412074133404)*x[0]+(-0.272818985272)*x[1]**o+(0.187802732278)*x[1]
        s[1,0,0]=(0.377505937225)*x[0]**o+(0.51898033576)*x[0]+(-0.439566585731)*x[1]**o+(-0.998166796948)*x[1]
        s[1,0,1]=(0.389095949557)*x[0]**o+(-0.382477743743)*x[0]+(-0.960850005856)*x[1]**o+(-0.440393846867)*x[1]
        s[1,1,0]=(-0.70296977387)*x[0]**o+(0.315478264729)*x[0]+(-0.867044520578)*x[1]**o+(-0.760970516579)*x[1]
        s[1,1,1]=(0.706189511641)*x[0]**o+(-0.847541758712)*x[0]+(0.814254280461)*x[1]**o+(0.38248389532)*x[1]
        s[2,0,0]=(-0.651567091764)*x[0]**o+(-0.410805007914)*x[0]+(0.0538051169317)*x[1]**o+(0.534010791665)*x[1]
        s[2,0,1]=(0.487085829627)*x[0]**o+(0.774536856681)*x[0]+(-0.873566030375)*x[1]**o+(0.370185957857)*x[1]
        s[2,1,0]=(0.632552547114)*x[0]**o+(0.147565981324)*x[0]+(0.868108695886)*x[1]**o+(0.671762982739)*x[1]
        s[2,1,1]=(-0.259846254622)*x[0]**o+(0.631407506034)*x[0]+(-0.681905081868)*x[1]**o+(0.0452497065686)*x[1]
        s[3,0,0]=(0.057043606868)*x[0]**o+(-0.0902461813963)*x[0]+(-0.240745938715)*x[1]**o+(-0.831861045923)*x[1]
        s[3,0,1]=(0.699648897337)*x[0]**o+(0.640417545224)*x[0]+(-0.430848221186)*x[1]**o+(0.462058652596)*x[1]
        s[3,1,0]=(0.664926292599)*x[0]**o+(-0.891859906903)*x[0]+(0.775280390876)*x[1]**o+(0.614722294397)*x[1]
        s[3,1,1]=(-0.785736689922)*x[0]**o+(0.687423487421)*x[0]+(-0.573349590903)*x[1]**o+(-0.653598711028)*x[1]
        s[4,0,0]=(0.319805468591)*x[0]**o+(0.601962226985)*x[0]+(0.358802389875)*x[1]**o+(-0.468789714036)*x[1]
        s[4,0,1]=(0.975677606412)*x[0]**o+(-0.702436984042)*x[0]+(-0.720390079934)*x[1]**o+(-0.377014128023)*x[1]
        s[4,1,0]=(0.736756275381)*x[0]**o+(-0.257998474493)*x[0]+(0.5991398372)*x[1]**o+(-0.32399683906)*x[1]
        s[4,1,1]=(-0.215912723594)*x[0]**o+(0.326544778825)*x[0]+(-0.620761245415)*x[1]**o+(-0.498174781757)*x[1]
        s[5,0,0]=(-0.601667515189)*x[0]**o+(0.516733915255)*x[0]+(0.183410588351)*x[1]**o+(-0.792386589315)*x[1]
        s[5,0,1]=(-0.561245419581)*x[0]**o+(0.371706636352)*x[0]+(-0.795300794294)*x[1]**o+(0.41047673348)*x[1]
        s[5,1,0]=(0.466316419068)*x[0]**o+(-0.0802419079562)*x[0]+(0.569083019829)*x[1]**o+(-0.20883523199)*x[1]
        s[5,1,1]=(-0.546242876843)*x[0]**o+(-0.499594802846)*x[0]+(-0.945490356092)*x[1]**o+(0.938984415563)*x[1]
        ref[0,0,0]=(-0.522613470003)/(o+1.)+(-0.314118091524)+(0.467159870975)*0.5**o
        ref[0,0,1]=(0.59831432517)/(o+1.)+(0.35269686798)+(0.483392039154)*0.5**o
        ref[0,1,0]=(0.19099567725)/(o+1.)+(0.0853788216078)+(-0.642178324261)*0.5**o
        ref[0,1,1]=(-0.272818985272)/(o+1.)+(-0.112135700563)+(-0.963913869717)*0.5**o
        ref[1,0,0]=(-0.439566585731)/(o+1.)+(-0.239593230594)+(0.377505937225)*0.5**o
        ref[1,0,1]=(-0.960850005856)/(o+1.)+(-0.411435795305)+(0.389095949557)*0.5**o
        ref[1,1,0]=(-0.867044520578)/(o+1.)+(-0.222746125925)+(-0.70296977387)*0.5**o
        ref[1,1,1]=(0.814254280461)/(o+1.)+(-0.232528931696)+(0.706189511641)*0.5**o
        ref[2,0,0]=(0.0538051169317)/(o+1.)+(0.0616028918756)+(-0.651567091764)*0.5**o
        ref[2,0,1]=(-0.873566030375)/(o+1.)+(0.572361407269)+(0.487085829627)*0.5**o
        ref[2,1,0]=(0.868108695886)/(o+1.)+(0.409664482032)+(0.632552547114)*0.5**o
        ref[2,1,1]=(-0.681905081868)/(o+1.)+(0.338328606302)+(-0.259846254622)*0.5**o
        ref[3,0,0]=(-0.240745938715)/(o+1.)+(-0.46105361366)+(0.057043606868)*0.5**o
        ref[3,0,1]=(-0.430848221186)/(o+1.)+(0.55123809891)+(0.699648897337)*0.5**o
        ref[3,1,0]=(0.775280390876)/(o+1.)+(-0.138568806253)+(0.664926292599)*0.5**o
        ref[3,1,1]=(-0.573349590903)/(o+1.)+(0.0169123881966)+(-0.785736689922)*0.5**o
        ref[4,0,0]=(0.358802389875)/(o+1.)+(0.0665862564747)+(0.319805468591)*0.5**o
        ref[4,0,1]=(-0.720390079934)/(o+1.)+(-0.539725556033)+(0.975677606412)*0.5**o
        ref[4,1,0]=(0.5991398372)/(o+1.)+(-0.290997656776)+(0.736756275381)*0.5**o
        ref[4,1,1]=(-0.620761245415)/(o+1.)+(-0.0858150014665)+(-0.215912723594)*0.5**o
        ref[5,0,0]=(0.183410588351)/(o+1.)+(-0.13782633703)+(-0.601667515189)*0.5**o
        ref[5,0,1]=(-0.795300794294)/(o+1.)+(0.391091684916)+(-0.561245419581)*0.5**o
        ref[5,1,0]=(0.569083019829)/(o+1.)+(-0.144538569973)+(0.466316419068)*0.5**o
        ref[5,1,1]=(-0.945490356092)/(o+1.)+(0.219694806358)+(-0.546242876843)*0.5**o
      else:
        s[0,0,0]=(-0.669363049954)*x[0]**o+(0.94445770304)*x[0]+(-0.504461532785)*x[1]**o+(0.636064462988)*x[1]+(0.416245244177)*x[2]**o+(0.666397932744)*x[2]
        s[0,0,1]=(-0.778564625852)*x[0]**o+(-0.825187520549)*x[0]+(0.72174236037)*x[1]**o+(-0.884811347472)*x[1]+(-0.206727051888)*x[2]**o+(0.669175992728)*x[2]
        s[0,1,0]=(0.59083523386)*x[0]**o+(0.264891177321)*x[0]+(0.728529437979)*x[1]**o+(0.689869176288)*x[1]+(0.759759991708)*x[2]**o+(-0.768640416685)*x[2]
        s[0,1,1]=(-0.256994678625)*x[0]**o+(0.151952222597)*x[0]+(-0.615154134713)*x[1]**o+(0.705589259856)*x[1]+(-0.27400110493)*x[2]**o+(-0.042497975508)*x[2]
        s[1,0,0]=(0.577165945348)*x[0]**o+(-0.475277785249)*x[0]+(-0.106826355519)*x[1]**o+(0.0424930134605)*x[1]+(0.680372240013)*x[2]**o+(0.943809871598)*x[2]
        s[1,0,1]=(-0.309232479272)*x[0]**o+(0.195348662283)*x[0]+(0.833914381189)*x[1]**o+(0.262108970555)*x[1]+(-0.716673250709)*x[2]**o+(-0.168478923947)*x[2]
        s[1,1,0]=(-0.275625390067)*x[0]**o+(0.523842393012)*x[0]+(0.437231937115)*x[1]**o+(0.758417244685)*x[1]+(0.826527568677)*x[2]**o+(-0.998749362644)*x[2]
        s[1,1,1]=(0.148292586557)*x[0]**o+(0.405011487356)*x[0]+(0.4599949448)*x[1]**o+(-0.612512744054)*x[1]+(0.187004698026)*x[2]**o+(0.41423737326)*x[2]
        s[2,0,0]=(-0.0683880574768)*x[0]**o+(0.527974031235)*x[0]+(0.761878395348)*x[1]**o+(0.588459184489)*x[1]+(-0.587912848751)*x[2]**o+(0.305103889218)*x[2]
        s[2,0,1]=(0.520929962776)*x[0]**o+(0.33193799149)*x[0]+(-0.342126485854)*x[1]**o+(-0.14865315386)*x[1]+(0.310895900233)*x[2]**o+(-0.322869734717)*x[2]
        s[2,1,0]=(-0.321360677385)*x[0]**o+(0.252389514576)*x[0]+(-0.0393452094196)*x[1]**o+(0.700761849784)*x[1]+(-0.526926106563)*x[2]**o+(0.510264453258)*x[2]
        s[2,1,1]=(-0.417527085289)*x[0]**o+(-0.272942288011)*x[0]+(0.601765245586)*x[1]**o+(-0.790696366296)*x[1]+(-0.012638708702)*x[2]**o+(0.932434350334)*x[2]
        s[3,0,0]=(-0.511516099074)*x[0]**o+(0.41782462791)*x[0]+(0.0995516379245)*x[1]**o+(0.546046499872)*x[1]+(0.874011011955)*x[2]**o+(-0.788555434082)*x[2]
        s[3,0,1]=(0.327526313236)*x[0]**o+(-0.425896436338)*x[0]+(0.914336317958)*x[1]**o+(0.307045278546)*x[1]+(0.72678975804)*x[2]**o+(-0.752188240491)*x[2]
        s[3,1,0]=(0.396685657572)*x[0]**o+(-0.853213412456)*x[0]+(0.592696880548)*x[1]**o+(-0.300028961606)*x[1]+(0.876996858765)*x[2]**o+(-0.0882159006726)*x[2]
        s[3,1,1]=(-0.482121450588)*x[0]**o+(-0.112743359188)*x[0]+(0.328241708104)*x[1]**o+(-0.617997058589)*x[1]+(0.936876537283)*x[2]**o+(0.350080033532)*x[2]
        s[4,0,0]=(-0.200856211954)*x[0]**o+(-0.122991084849)*x[0]+(0.487659949467)*x[1]**o+(-0.425527004923)*x[1]+(-0.147553486772)*x[2]**o+(-0.32641504978)*x[2]
        s[4,0,1]=(0.333642453215)*x[0]**o+(0.954952509878)*x[0]+(-0.291340184543)*x[1]**o+(-0.628839479489)*x[1]+(0.67230688241)*x[2]**o+(0.84754444469)*x[2]
        s[4,1,0]=(0.0708849044002)*x[0]**o+(-0.53553180865)*x[0]+(-0.114931581103)*x[1]**o+(-0.374826425939)*x[1]+(-0.0919922579012)*x[2]**o+(-0.745351276697)*x[2]
        s[4,1,1]=(-0.224313591576)*x[0]**o+(0.329362356625)*x[0]+(-0.60957590052)*x[1]**o+(0.824884117846)*x[1]+(-0.695979917244)*x[2]**o+(-0.279844578235)*x[2]
        s[5,0,0]=(-0.431218311479)*x[0]**o+(0.349200792571)*x[0]+(0.840732425028)*x[1]**o+(-0.484758521346)*x[1]+(0.295545194625)*x[2]**o+(0.47040408229)*x[2]
        s[5,0,1]=(0.506935329965)*x[0]**o+(0.797912921589)*x[0]+(-0.143979832786)*x[1]**o+(-0.660815905482)*x[1]+(0.123849993986)*x[2]**o+(-0.877052796091)*x[2]
        s[5,1,0]=(0.500348066835)*x[0]**o+(-0.455430279521)*x[0]+(-0.569813267302)*x[1]**o+(-0.855668211999)*x[1]+(0.203279416274)*x[2]**o+(-0.112743061503)*x[2]
        s[5,1,1]=(0.993758177333)*x[0]**o+(-0.784317748398)*x[0]+(0.530003336163)*x[1]**o+(-0.857305987154)*x[1]+(0.391887947598)*x[2]**o+(0.933473063729)*x[2]
        ref[0,0,0]=(-0.0882162886075)/(o+1.)+(1.12346004939)+(-0.669363049954)*0.5**o
        ref[0,0,1]=(0.515015308482)/(o+1.)+(-0.520411437647)+(-0.778564625852)*0.5**o
        ref[0,1,0]=(1.48828942969)/(o+1.)+(0.0930599684618)+(0.59083523386)*0.5**o
        ref[0,1,1]=(-0.889155239643)/(o+1.)+(0.407521753473)+(-0.256994678625)*0.5**o
        ref[1,0,0]=(0.573545884494)/(o+1.)+(0.255512549905)+(0.577165945348)*0.5**o
        ref[1,0,1]=(0.11724113048)/(o+1.)+(0.144489354446)+(-0.309232479272)*0.5**o
        ref[1,1,0]=(1.26375950579)/(o+1.)+(0.141755137526)+(-0.275625390067)*0.5**o
        ref[1,1,1]=(0.646999642826)/(o+1.)+(0.103368058281)+(0.148292586557)*0.5**o
        ref[2,0,0]=(0.173965546596)/(o+1.)+(0.710768552471)+(-0.0683880574768)*0.5**o
        ref[2,0,1]=(-0.0312305856211)/(o+1.)+(-0.0697924485433)+(0.520929962776)*0.5**o
        ref[2,1,0]=(-0.566271315982)/(o+1.)+(0.731707908809)+(-0.321360677385)*0.5**o
        ref[2,1,1]=(0.589126536884)/(o+1.)+(-0.0656021519867)+(-0.417527085289)*0.5**o
        ref[3,0,0]=(0.973562649879)/(o+1.)+(0.0876578468496)+(-0.511516099074)*0.5**o
        ref[3,0,1]=(1.641126076)/(o+1.)+(-0.435519699141)+(0.327526313236)*0.5**o
        ref[3,1,0]=(1.46969373931)/(o+1.)+(-0.620729137367)+(0.396685657572)*0.5**o
        ref[3,1,1]=(1.26511824539)/(o+1.)+(-0.190330192122)+(-0.482121450588)*0.5**o
        ref[4,0,0]=(0.340106462695)/(o+1.)+(-0.437466569776)+(-0.200856211954)*0.5**o
        ref[4,0,1]=(0.380966697866)/(o+1.)+(0.586828737539)+(0.333642453215)*0.5**o
        ref[4,1,0]=(-0.206923839004)/(o+1.)+(-0.827854755643)+(0.0708849044002)*0.5**o
        ref[4,1,1]=(-1.30555581776)/(o+1.)+(0.437200948118)+(-0.224313591576)*0.5**o
        ref[5,0,0]=(1.13627761965)/(o+1.)+(0.167423176757)+(-0.431218311479)*0.5**o
        ref[5,0,1]=(-0.0201298387998)/(o+1.)+(-0.369977889992)+(0.506935329965)*0.5**o
        ref[5,1,0]=(-0.366533851028)/(o+1.)+(-0.711920776511)+(0.500348066835)*0.5**o
        ref[5,1,1]=(0.921891283761)/(o+1.)+(-0.354075335912)+(0.993758177333)*0.5**o
      res=integrate(arg)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactZero_fromSymbol_FunctionOnContactZero_rank4(self):
      """
      tests integral of rank 4 Symbol on the FunctionOnContactZero

      assumptions: FunctionOnContactZero(self.domain) exists
                   self.domain supports integral on FunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      w=FunctionOnContactZero(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5, 3, 2))
      s=Data(0,(4, 5, 3, 2),w)
      ref=numpy.zeros((4, 5, 3, 2),numpy.float_)
      if dim==2:
        s[0,0,0,0]=(-0.135767714785)*x[0]**o+(0.602721853735)*x[0]+(0.685908691403)*x[1]**o+(-0.278963806046)*x[1]
        s[0,0,0,1]=(0.548074276718)*x[0]**o+(-0.00153460595028)*x[0]+(0.964032895202)*x[1]**o+(0.145345601288)*x[1]
        s[0,0,1,0]=(0.00388957848949)*x[0]**o+(-0.00330645752363)*x[0]+(-0.398340925709)*x[1]**o+(0.544904812736)*x[1]
        s[0,0,1,1]=(0.896772134786)*x[0]**o+(-0.305756341328)*x[0]+(0.766835159085)*x[1]**o+(-0.185065752039)*x[1]
        s[0,0,2,0]=(0.455765211649)*x[0]**o+(-0.172084793069)*x[0]+(-0.745694059134)*x[1]**o+(0.467137037155)*x[1]
        s[0,0,2,1]=(-0.683685778458)*x[0]**o+(-0.526137358848)*x[0]+(0.499311653339)*x[1]**o+(-0.810057930627)*x[1]
        s[0,1,0,0]=(0.244426126288)*x[0]**o+(0.464096329887)*x[0]+(-0.565273192791)*x[1]**o+(0.341584391362)*x[1]
        s[0,1,0,1]=(-0.0873578866485)*x[0]**o+(-0.280704680398)*x[0]+(0.0551381505624)*x[1]**o+(0.224333075687)*x[1]
        s[0,1,1,0]=(0.0886184124686)*x[0]**o+(-0.577421322103)*x[0]+(-0.784267376644)*x[1]**o+(-0.220754022299)*x[1]
        s[0,1,1,1]=(-0.901203063708)*x[0]**o+(-0.791637322168)*x[0]+(-0.615643115793)*x[1]**o+(-0.0955674319946)*x[1]
        s[0,1,2,0]=(0.418411748143)*x[0]**o+(0.0825948292296)*x[0]+(0.00527147896827)*x[1]**o+(0.474507057947)*x[1]
        s[0,1,2,1]=(0.574446727172)*x[0]**o+(0.634382868948)*x[0]+(-0.8192829244)*x[1]**o+(0.0916815679535)*x[1]
        s[0,2,0,0]=(-0.324743695914)*x[0]**o+(-0.517660802435)*x[0]+(-0.694637110589)*x[1]**o+(-0.191013585315)*x[1]
        s[0,2,0,1]=(0.538239581284)*x[0]**o+(-0.793537232228)*x[0]+(-0.260263189341)*x[1]**o+(-0.232999585659)*x[1]
        s[0,2,1,0]=(-0.535444326182)*x[0]**o+(0.375771246892)*x[0]+(-0.351338268294)*x[1]**o+(-0.922383156347)*x[1]
        s[0,2,1,1]=(-0.754984712928)*x[0]**o+(0.844268402619)*x[0]+(-0.373848162528)*x[1]**o+(-0.0348415758383)*x[1]
        s[0,2,2,0]=(0.101157816757)*x[0]**o+(0.322178434764)*x[0]+(0.309106041559)*x[1]**o+(-0.720194799504)*x[1]
        s[0,2,2,1]=(-0.537633855745)*x[0]**o+(-0.149185820274)*x[0]+(-0.155113200571)*x[1]**o+(0.433800441998)*x[1]
        s[0,3,0,0]=(0.466579973711)*x[0]**o+(-0.924681979368)*x[0]+(0.491019303223)*x[1]**o+(0.994389212618)*x[1]
        s[0,3,0,1]=(0.202651138329)*x[0]**o+(0.114061296405)*x[0]+(-0.660374384585)*x[1]**o+(0.192752003014)*x[1]
        s[0,3,1,0]=(0.442171269159)*x[0]**o+(-0.0740165829499)*x[0]+(-0.952344452735)*x[1]**o+(0.734456419975)*x[1]
        s[0,3,1,1]=(0.733989617351)*x[0]**o+(0.157469674236)*x[0]+(-0.305478230795)*x[1]**o+(-0.416091191381)*x[1]
        s[0,3,2,0]=(-0.709003057494)*x[0]**o+(-0.554321605532)*x[0]+(-0.374724207394)*x[1]**o+(-0.799384819999)*x[1]
        s[0,3,2,1]=(-0.125623348142)*x[0]**o+(-0.695830039564)*x[0]+(0.0283773197868)*x[1]**o+(-0.459458049105)*x[1]
        s[0,4,0,0]=(0.490207583728)*x[0]**o+(0.302026020799)*x[0]+(0.582676151383)*x[1]**o+(-0.540047416714)*x[1]
        s[0,4,0,1]=(-0.605187460486)*x[0]**o+(-0.700102630541)*x[0]+(0.576928080618)*x[1]**o+(-0.753735278462)*x[1]
        s[0,4,1,0]=(-0.0850932164524)*x[0]**o+(-0.132436670102)*x[0]+(0.634879197271)*x[1]**o+(-0.40759333575)*x[1]
        s[0,4,1,1]=(0.258713682268)*x[0]**o+(0.941275860044)*x[0]+(-0.976313171743)*x[1]**o+(-0.817915320276)*x[1]
        s[0,4,2,0]=(-0.350506879421)*x[0]**o+(-0.935416620637)*x[0]+(0.153793647237)*x[1]**o+(0.384520330312)*x[1]
        s[0,4,2,1]=(0.671395932648)*x[0]**o+(-0.338473533885)*x[0]+(-0.769374193622)*x[1]**o+(-0.0867650576614)*x[1]
        s[1,0,0,0]=(0.188023378319)*x[0]**o+(0.662970173794)*x[0]+(0.307120506561)*x[1]**o+(-0.561302563336)*x[1]
        s[1,0,0,1]=(0.800293252159)*x[0]**o+(0.414126518273)*x[0]+(-0.411140279426)*x[1]**o+(-0.726050617156)*x[1]
        s[1,0,1,0]=(-0.515339883608)*x[0]**o+(-0.849896429803)*x[0]+(-0.404819325345)*x[1]**o+(-0.660381411211)*x[1]
        s[1,0,1,1]=(0.0512515899156)*x[0]**o+(-0.59411151972)*x[0]+(-0.586954384255)*x[1]**o+(-0.581246213363)*x[1]
        s[1,0,2,0]=(-0.581670807515)*x[0]**o+(-0.151580006917)*x[0]+(0.166288186202)*x[1]**o+(-0.90507298782)*x[1]
        s[1,0,2,1]=(-0.634327645938)*x[0]**o+(0.227054101481)*x[0]+(-0.698802363494)*x[1]**o+(-0.191787378024)*x[1]
        s[1,1,0,0]=(0.434696675884)*x[0]**o+(0.550165686473)*x[0]+(-0.381545673134)*x[1]**o+(0.0353703778505)*x[1]
        s[1,1,0,1]=(0.438065102364)*x[0]**o+(0.734419538756)*x[0]+(-0.654368291721)*x[1]**o+(-0.288248589993)*x[1]
        s[1,1,1,0]=(0.812437488024)*x[0]**o+(0.105098628605)*x[0]+(0.670405216194)*x[1]**o+(-0.932828342978)*x[1]
        s[1,1,1,1]=(0.52676928941)*x[0]**o+(0.34803390652)*x[0]+(0.0351984047833)*x[1]**o+(0.410003399979)*x[1]
        s[1,1,2,0]=(-0.0967009361026)*x[0]**o+(0.164660384961)*x[0]+(0.210182888846)*x[1]**o+(0.522971695817)*x[1]
        s[1,1,2,1]=(-0.664016192839)*x[0]**o+(0.452428778809)*x[0]+(-0.107282780635)*x[1]**o+(0.807095090624)*x[1]
        s[1,2,0,0]=(-0.289167783943)*x[0]**o+(0.667637278664)*x[0]+(0.615088613098)*x[1]**o+(-0.771365681637)*x[1]
        s[1,2,0,1]=(0.554991016918)*x[0]**o+(-0.0658224324202)*x[0]+(0.305877722926)*x[1]**o+(-0.410754491717)*x[1]
        s[1,2,1,0]=(0.131109587703)*x[0]**o+(-0.589856526298)*x[0]+(-0.457418380817)*x[1]**o+(-0.766026536542)*x[1]
        s[1,2,1,1]=(-0.339196055896)*x[0]**o+(0.0879106255633)*x[0]+(-0.300776404484)*x[1]**o+(-0.855785910082)*x[1]
        s[1,2,2,0]=(0.169433038681)*x[0]**o+(0.900452320033)*x[0]+(-0.515800574208)*x[1]**o+(0.0353282676014)*x[1]
        s[1,2,2,1]=(0.120477320496)*x[0]**o+(0.544656714229)*x[0]+(0.712210191539)*x[1]**o+(-0.835300413971)*x[1]
        s[1,3,0,0]=(-0.144180890917)*x[0]**o+(0.847389690669)*x[0]+(0.21796553543)*x[1]**o+(0.0580182644011)*x[1]
        s[1,3,0,1]=(0.0202759559404)*x[0]**o+(-0.832118697631)*x[0]+(0.860042691805)*x[1]**o+(0.954802209319)*x[1]
        s[1,3,1,0]=(-0.232327132843)*x[0]**o+(0.988485328327)*x[0]+(-0.941830133542)*x[1]**o+(0.796919796173)*x[1]
        s[1,3,1,1]=(-0.0528857772705)*x[0]**o+(0.24536410605)*x[0]+(-0.89565411974)*x[1]**o+(0.473315809996)*x[1]
        s[1,3,2,0]=(0.920179439848)*x[0]**o+(0.415154298944)*x[0]+(0.690915792422)*x[1]**o+(0.0956157306525)*x[1]
        s[1,3,2,1]=(-0.332235130954)*x[0]**o+(0.756254925963)*x[0]+(0.936945508353)*x[1]**o+(0.56286108057)*x[1]
        s[1,4,0,0]=(0.802947524473)*x[0]**o+(0.764953990189)*x[0]+(0.58992910698)*x[1]**o+(0.807094950185)*x[1]
        s[1,4,0,1]=(-0.309951166637)*x[0]**o+(-0.605960228686)*x[0]+(0.0223945031407)*x[1]**o+(-0.230764354345)*x[1]
        s[1,4,1,0]=(0.856801428802)*x[0]**o+(0.748897933626)*x[0]+(0.0648717388846)*x[1]**o+(-0.197803227055)*x[1]
        s[1,4,1,1]=(0.160470457446)*x[0]**o+(0.466029696285)*x[0]+(-0.808841434426)*x[1]**o+(-0.890661270209)*x[1]
        s[1,4,2,0]=(0.316160329076)*x[0]**o+(-0.616711166445)*x[0]+(0.176082750216)*x[1]**o+(-0.389156386122)*x[1]
        s[1,4,2,1]=(0.966674586588)*x[0]**o+(0.743418831809)*x[0]+(0.828814302078)*x[1]**o+(0.443124030799)*x[1]
        s[2,0,0,0]=(0.895605391818)*x[0]**o+(-0.712359772806)*x[0]+(-0.254420890563)*x[1]**o+(0.04060068125)*x[1]
        s[2,0,0,1]=(0.407669194399)*x[0]**o+(-0.561911820032)*x[0]+(0.759519426562)*x[1]**o+(-0.791065777521)*x[1]
        s[2,0,1,0]=(0.0220907964237)*x[0]**o+(0.764347460016)*x[0]+(-0.894211196035)*x[1]**o+(-0.766734523336)*x[1]
        s[2,0,1,1]=(0.276708235973)*x[0]**o+(0.902519459875)*x[0]+(0.957736485845)*x[1]**o+(0.861836071217)*x[1]
        s[2,0,2,0]=(0.209485180668)*x[0]**o+(0.463649158628)*x[0]+(-0.0521057130471)*x[1]**o+(-0.64350767543)*x[1]
        s[2,0,2,1]=(-0.605362744895)*x[0]**o+(-0.131055561812)*x[0]+(-0.0963734475683)*x[1]**o+(0.38191997821)*x[1]
        s[2,1,0,0]=(0.242435009983)*x[0]**o+(-0.0678120221956)*x[0]+(-0.981711153358)*x[1]**o+(0.799273232234)*x[1]
        s[2,1,0,1]=(-0.178432239175)*x[0]**o+(-0.895623369837)*x[0]+(-0.572639180822)*x[1]**o+(-0.83834422153)*x[1]
        s[2,1,1,0]=(0.993103298694)*x[0]**o+(0.728567708683)*x[0]+(-0.799812986792)*x[1]**o+(-0.057428598921)*x[1]
        s[2,1,1,1]=(-0.776745418541)*x[0]**o+(-0.338983048206)*x[0]+(-0.565511785334)*x[1]**o+(-0.308717365714)*x[1]
        s[2,1,2,0]=(-0.199225286482)*x[0]**o+(-0.158190421122)*x[0]+(0.150112974223)*x[1]**o+(0.954346438264)*x[1]
        s[2,1,2,1]=(-0.402303812441)*x[0]**o+(-0.874974485349)*x[0]+(0.680761341229)*x[1]**o+(-0.694296934266)*x[1]
        s[2,2,0,0]=(0.0582479020766)*x[0]**o+(-0.562861368267)*x[0]+(0.757525601337)*x[1]**o+(0.932167579204)*x[1]
        s[2,2,0,1]=(-0.379424441757)*x[0]**o+(0.177098262781)*x[0]+(-0.485467998718)*x[1]**o+(-0.17324576024)*x[1]
        s[2,2,1,0]=(-0.339388895182)*x[0]**o+(0.491140214169)*x[0]+(0.471276867115)*x[1]**o+(0.292216420243)*x[1]
        s[2,2,1,1]=(0.502027084477)*x[0]**o+(0.927860130016)*x[0]+(0.787575747782)*x[1]**o+(-0.470519713839)*x[1]
        s[2,2,2,0]=(-0.583469643173)*x[0]**o+(-0.765710060231)*x[0]+(-0.54135991724)*x[1]**o+(0.665415514934)*x[1]
        s[2,2,2,1]=(-0.472362804158)*x[0]**o+(-0.766567819149)*x[0]+(0.201380986075)*x[1]**o+(0.270890088749)*x[1]
        s[2,3,0,0]=(-0.547838738364)*x[0]**o+(0.373177565132)*x[0]+(-0.921379470525)*x[1]**o+(0.0837731390271)*x[1]
        s[2,3,0,1]=(-0.557990737044)*x[0]**o+(-0.561311443234)*x[0]+(-0.924235646481)*x[1]**o+(-0.746081692868)*x[1]
        s[2,3,1,0]=(0.566861818147)*x[0]**o+(-0.107222369398)*x[0]+(-0.53810139661)*x[1]**o+(-0.814508288518)*x[1]
        s[2,3,1,1]=(-0.0578464759524)*x[0]**o+(0.525704186493)*x[0]+(0.434855750568)*x[1]**o+(-0.761515114625)*x[1]
        s[2,3,2,0]=(0.388204865306)*x[0]**o+(-0.320298614451)*x[0]+(0.0228508430597)*x[1]**o+(-0.436245057461)*x[1]
        s[2,3,2,1]=(0.70625829226)*x[0]**o+(-0.459381956838)*x[0]+(-0.756416817999)*x[1]**o+(0.963958503587)*x[1]
        s[2,4,0,0]=(0.595101935764)*x[0]**o+(0.0182198304835)*x[0]+(0.389172632633)*x[1]**o+(0.571052531257)*x[1]
        s[2,4,0,1]=(-0.59261552462)*x[0]**o+(0.176171578479)*x[0]+(-0.481303115308)*x[1]**o+(0.476762838828)*x[1]
        s[2,4,1,0]=(-0.274360684371)*x[0]**o+(0.496939161979)*x[0]+(0.0982270602491)*x[1]**o+(0.0521672721299)*x[1]
        s[2,4,1,1]=(0.0022428239189)*x[0]**o+(-0.926985821019)*x[0]+(0.0530228030052)*x[1]**o+(0.403935928384)*x[1]
        s[2,4,2,0]=(-0.96226874392)*x[0]**o+(0.282033354362)*x[0]+(0.917864727482)*x[1]**o+(0.32541112383)*x[1]
        s[2,4,2,1]=(0.684506236166)*x[0]**o+(0.215015538573)*x[0]+(0.74280905237)*x[1]**o+(0.781203116718)*x[1]
        s[3,0,0,0]=(0.658782504284)*x[0]**o+(0.396806335664)*x[0]+(0.0780367521917)*x[1]**o+(0.533907837354)*x[1]
        s[3,0,0,1]=(0.386822554052)*x[0]**o+(-0.951475963073)*x[0]+(0.992735332775)*x[1]**o+(-0.158295718477)*x[1]
        s[3,0,1,0]=(-0.253862856162)*x[0]**o+(-0.0131419224769)*x[0]+(0.810452271623)*x[1]**o+(-0.116490242053)*x[1]
        s[3,0,1,1]=(-0.297107568873)*x[0]**o+(0.339277467727)*x[0]+(-0.0455924137238)*x[1]**o+(0.408214745405)*x[1]
        s[3,0,2,0]=(0.101103302929)*x[0]**o+(0.84974169826)*x[0]+(0.668273722779)*x[1]**o+(-0.134494535453)*x[1]
        s[3,0,2,1]=(0.144907392006)*x[0]**o+(0.59561768643)*x[0]+(0.808003987088)*x[1]**o+(0.442993444756)*x[1]
        s[3,1,0,0]=(0.050719218416)*x[0]**o+(-0.205363570154)*x[0]+(-0.267767797056)*x[1]**o+(0.473617859576)*x[1]
        s[3,1,0,1]=(-0.488804469299)*x[0]**o+(-0.108366154443)*x[0]+(0.754051956614)*x[1]**o+(0.601979535209)*x[1]
        s[3,1,1,0]=(0.665579804971)*x[0]**o+(-0.820977378045)*x[0]+(-0.99768277192)*x[1]**o+(-0.553278556669)*x[1]
        s[3,1,1,1]=(0.28421003692)*x[0]**o+(0.34932169662)*x[0]+(-0.942736776261)*x[1]**o+(-0.761313236403)*x[1]
        s[3,1,2,0]=(-0.271033791176)*x[0]**o+(-0.269316831976)*x[0]+(0.637503921768)*x[1]**o+(0.550986595411)*x[1]
        s[3,1,2,1]=(-0.314094366001)*x[0]**o+(0.771643852473)*x[0]+(-0.370702666003)*x[1]**o+(0.925989724825)*x[1]
        s[3,2,0,0]=(0.608693141558)*x[0]**o+(-0.865838490365)*x[0]+(0.390277717008)*x[1]**o+(0.651999980469)*x[1]
        s[3,2,0,1]=(-0.911099837761)*x[0]**o+(0.307504460716)*x[0]+(0.82234563191)*x[1]**o+(-0.565850042096)*x[1]
        s[3,2,1,0]=(-0.876634035391)*x[0]**o+(0.410910752799)*x[0]+(-0.578952272394)*x[1]**o+(0.204714780618)*x[1]
        s[3,2,1,1]=(0.06295832442)*x[0]**o+(0.441355368057)*x[0]+(0.767590856273)*x[1]**o+(-0.67580811237)*x[1]
        s[3,2,2,0]=(-0.186380273058)*x[0]**o+(-0.891590560039)*x[0]+(-0.957486087526)*x[1]**o+(0.198598656166)*x[1]
        s[3,2,2,1]=(-0.733041869578)*x[0]**o+(-0.369283091906)*x[0]+(-0.103339143054)*x[1]**o+(-0.5468960827)*x[1]
        s[3,3,0,0]=(0.381695999815)*x[0]**o+(-0.398360040383)*x[0]+(-0.00816783658019)*x[1]**o+(0.209734829036)*x[1]
        s[3,3,0,1]=(-0.458999917243)*x[0]**o+(0.693814798148)*x[0]+(0.988023235161)*x[1]**o+(0.249848645187)*x[1]
        s[3,3,1,0]=(0.35272081264)*x[0]**o+(-0.24196842916)*x[0]+(-0.376576617413)*x[1]**o+(-0.512660897623)*x[1]
        s[3,3,1,1]=(-0.880579928879)*x[0]**o+(-0.901266101944)*x[0]+(-0.6695966178)*x[1]**o+(0.81462165322)*x[1]
        s[3,3,2,0]=(0.179006063683)*x[0]**o+(-0.772199368214)*x[0]+(0.173717958996)*x[1]**o+(0.943451083071)*x[1]
        s[3,3,2,1]=(0.772059778005)*x[0]**o+(-0.816863867151)*x[0]+(0.931619586753)*x[1]**o+(0.618296473051)*x[1]
        s[3,4,0,0]=(0.609550545616)*x[0]**o+(-0.226822163333)*x[0]+(0.498238525714)*x[1]**o+(0.145755694026)*x[1]
        s[3,4,0,1]=(0.925097562117)*x[0]**o+(0.793877528837)*x[0]+(0.483353477991)*x[1]**o+(-0.547313214969)*x[1]
        s[3,4,1,0]=(-0.878072917122)*x[0]**o+(0.703236130488)*x[0]+(-0.511134933342)*x[1]**o+(0.676303306122)*x[1]
        s[3,4,1,1]=(0.0745582123775)*x[0]**o+(0.668129797141)*x[0]+(-0.493968471165)*x[1]**o+(-0.26653808146)*x[1]
        s[3,4,2,0]=(-0.736620884918)*x[0]**o+(0.0401872117)*x[0]+(-0.312111305637)*x[1]**o+(0.811490469583)*x[1]
        s[3,4,2,1]=(-0.663426736678)*x[0]**o+(-0.630806839416)*x[0]+(0.638515711599)*x[1]**o+(-0.386941335499)*x[1]
        ref[0,0,0,0]=(0.685908691403)/(o+1.)+(0.161879023844)+(-0.135767714785)*0.5**o
        ref[0,0,0,1]=(0.964032895202)/(o+1.)+(0.071905497669)+(0.548074276718)*0.5**o
        ref[0,0,1,0]=(-0.398340925709)/(o+1.)+(0.270799177606)+(0.00388957848949)*0.5**o
        ref[0,0,1,1]=(0.766835159085)/(o+1.)+(-0.245411046684)+(0.896772134786)*0.5**o
        ref[0,0,2,0]=(-0.745694059134)/(o+1.)+(0.147526122043)+(0.455765211649)*0.5**o
        ref[0,0,2,1]=(0.499311653339)/(o+1.)+(-0.668097644737)+(-0.683685778458)*0.5**o
        ref[0,1,0,0]=(-0.565273192791)/(o+1.)+(0.402840360625)+(0.244426126288)*0.5**o
        ref[0,1,0,1]=(0.0551381505624)/(o+1.)+(-0.0281858023555)+(-0.0873578866485)*0.5**o
        ref[0,1,1,0]=(-0.784267376644)/(o+1.)+(-0.399087672201)+(0.0886184124686)*0.5**o
        ref[0,1,1,1]=(-0.615643115793)/(o+1.)+(-0.443602377081)+(-0.901203063708)*0.5**o
        ref[0,1,2,0]=(0.00527147896827)/(o+1.)+(0.278550943588)+(0.418411748143)*0.5**o
        ref[0,1,2,1]=(-0.8192829244)/(o+1.)+(0.363032218451)+(0.574446727172)*0.5**o
        ref[0,2,0,0]=(-0.694637110589)/(o+1.)+(-0.354337193875)+(-0.324743695914)*0.5**o
        ref[0,2,0,1]=(-0.260263189341)/(o+1.)+(-0.513268408944)+(0.538239581284)*0.5**o
        ref[0,2,1,0]=(-0.351338268294)/(o+1.)+(-0.273305954728)+(-0.535444326182)*0.5**o
        ref[0,2,1,1]=(-0.373848162528)/(o+1.)+(0.40471341339)+(-0.754984712928)*0.5**o
        ref[0,2,2,0]=(0.309106041559)/(o+1.)+(-0.19900818237)+(0.101157816757)*0.5**o
        ref[0,2,2,1]=(-0.155113200571)/(o+1.)+(0.142307310862)+(-0.537633855745)*0.5**o
        ref[0,3,0,0]=(0.491019303223)/(o+1.)+(0.0348536166248)+(0.466579973711)*0.5**o
        ref[0,3,0,1]=(-0.660374384585)/(o+1.)+(0.153406649709)+(0.202651138329)*0.5**o
        ref[0,3,1,0]=(-0.952344452735)/(o+1.)+(0.330219918513)+(0.442171269159)*0.5**o
        ref[0,3,1,1]=(-0.305478230795)/(o+1.)+(-0.129310758573)+(0.733989617351)*0.5**o
        ref[0,3,2,0]=(-0.374724207394)/(o+1.)+(-0.676853212765)+(-0.709003057494)*0.5**o
        ref[0,3,2,1]=(0.0283773197868)/(o+1.)+(-0.577644044334)+(-0.125623348142)*0.5**o
        ref[0,4,0,0]=(0.582676151383)/(o+1.)+(-0.119010697958)+(0.490207583728)*0.5**o
        ref[0,4,0,1]=(0.576928080618)/(o+1.)+(-0.726918954502)+(-0.605187460486)*0.5**o
        ref[0,4,1,0]=(0.634879197271)/(o+1.)+(-0.270015002926)+(-0.0850932164524)*0.5**o
        ref[0,4,1,1]=(-0.976313171743)/(o+1.)+(0.061680269884)+(0.258713682268)*0.5**o
        ref[0,4,2,0]=(0.153793647237)/(o+1.)+(-0.275448145163)+(-0.350506879421)*0.5**o
        ref[0,4,2,1]=(-0.769374193622)/(o+1.)+(-0.212619295773)+(0.671395932648)*0.5**o
        ref[1,0,0,0]=(0.307120506561)/(o+1.)+(0.0508338052288)+(0.188023378319)*0.5**o
        ref[1,0,0,1]=(-0.411140279426)/(o+1.)+(-0.155962049441)+(0.800293252159)*0.5**o
        ref[1,0,1,0]=(-0.404819325345)/(o+1.)+(-0.755138920507)+(-0.515339883608)*0.5**o
        ref[1,0,1,1]=(-0.586954384255)/(o+1.)+(-0.587678866541)+(0.0512515899156)*0.5**o
        ref[1,0,2,0]=(0.166288186202)/(o+1.)+(-0.528326497368)+(-0.581670807515)*0.5**o
        ref[1,0,2,1]=(-0.698802363494)/(o+1.)+(0.0176333617287)+(-0.634327645938)*0.5**o
        ref[1,1,0,0]=(-0.381545673134)/(o+1.)+(0.292768032162)+(0.434696675884)*0.5**o
        ref[1,1,0,1]=(-0.654368291721)/(o+1.)+(0.223085474382)+(0.438065102364)*0.5**o
        ref[1,1,1,0]=(0.670405216194)/(o+1.)+(-0.413864857186)+(0.812437488024)*0.5**o
        ref[1,1,1,1]=(0.0351984047833)/(o+1.)+(0.379018653249)+(0.52676928941)*0.5**o
        ref[1,1,2,0]=(0.210182888846)/(o+1.)+(0.343816040389)+(-0.0967009361026)*0.5**o
        ref[1,1,2,1]=(-0.107282780635)/(o+1.)+(0.629761934716)+(-0.664016192839)*0.5**o
        ref[1,2,0,0]=(0.615088613098)/(o+1.)+(-0.0518642014867)+(-0.289167783943)*0.5**o
        ref[1,2,0,1]=(0.305877722926)/(o+1.)+(-0.238288462068)+(0.554991016918)*0.5**o
        ref[1,2,1,0]=(-0.457418380817)/(o+1.)+(-0.67794153142)+(0.131109587703)*0.5**o
        ref[1,2,1,1]=(-0.300776404484)/(o+1.)+(-0.38393764226)+(-0.339196055896)*0.5**o
        ref[1,2,2,0]=(-0.515800574208)/(o+1.)+(0.467890293817)+(0.169433038681)*0.5**o
        ref[1,2,2,1]=(0.712210191539)/(o+1.)+(-0.145321849871)+(0.120477320496)*0.5**o
        ref[1,3,0,0]=(0.21796553543)/(o+1.)+(0.452703977535)+(-0.144180890917)*0.5**o
        ref[1,3,0,1]=(0.860042691805)/(o+1.)+(0.0613417558442)+(0.0202759559404)*0.5**o
        ref[1,3,1,0]=(-0.941830133542)/(o+1.)+(0.89270256225)+(-0.232327132843)*0.5**o
        ref[1,3,1,1]=(-0.89565411974)/(o+1.)+(0.359339958023)+(-0.0528857772705)*0.5**o
        ref[1,3,2,0]=(0.690915792422)/(o+1.)+(0.255385014798)+(0.920179439848)*0.5**o
        ref[1,3,2,1]=(0.936945508353)/(o+1.)+(0.659558003266)+(-0.332235130954)*0.5**o
        ref[1,4,0,0]=(0.58992910698)/(o+1.)+(0.786024470187)+(0.802947524473)*0.5**o
        ref[1,4,0,1]=(0.0223945031407)/(o+1.)+(-0.418362291515)+(-0.309951166637)*0.5**o
        ref[1,4,1,0]=(0.0648717388846)/(o+1.)+(0.275547353285)+(0.856801428802)*0.5**o
        ref[1,4,1,1]=(-0.808841434426)/(o+1.)+(-0.212315786962)+(0.160470457446)*0.5**o
        ref[1,4,2,0]=(0.176082750216)/(o+1.)+(-0.502933776283)+(0.316160329076)*0.5**o
        ref[1,4,2,1]=(0.828814302078)/(o+1.)+(0.593271431304)+(0.966674586588)*0.5**o
        ref[2,0,0,0]=(-0.254420890563)/(o+1.)+(-0.335879545778)+(0.895605391818)*0.5**o
        ref[2,0,0,1]=(0.759519426562)/(o+1.)+(-0.676488798776)+(0.407669194399)*0.5**o
        ref[2,0,1,0]=(-0.894211196035)/(o+1.)+(-0.00119353166009)+(0.0220907964237)*0.5**o
        ref[2,0,1,1]=(0.957736485845)/(o+1.)+(0.882177765546)+(0.276708235973)*0.5**o
        ref[2,0,2,0]=(-0.0521057130471)/(o+1.)+(-0.0899292584008)+(0.209485180668)*0.5**o
        ref[2,0,2,1]=(-0.0963734475683)/(o+1.)+(0.125432208199)+(-0.605362744895)*0.5**o
        ref[2,1,0,0]=(-0.981711153358)/(o+1.)+(0.365730605019)+(0.242435009983)*0.5**o
        ref[2,1,0,1]=(-0.572639180822)/(o+1.)+(-0.866983795684)+(-0.178432239175)*0.5**o
        ref[2,1,1,0]=(-0.799812986792)/(o+1.)+(0.335569554881)+(0.993103298694)*0.5**o
        ref[2,1,1,1]=(-0.565511785334)/(o+1.)+(-0.32385020696)+(-0.776745418541)*0.5**o
        ref[2,1,2,0]=(0.150112974223)/(o+1.)+(0.398078008571)+(-0.199225286482)*0.5**o
        ref[2,1,2,1]=(0.680761341229)/(o+1.)+(-0.784635709807)+(-0.402303812441)*0.5**o
        ref[2,2,0,0]=(0.757525601337)/(o+1.)+(0.184653105468)+(0.0582479020766)*0.5**o
        ref[2,2,0,1]=(-0.485467998718)/(o+1.)+(0.00192625127016)+(-0.379424441757)*0.5**o
        ref[2,2,1,0]=(0.471276867115)/(o+1.)+(0.391678317206)+(-0.339388895182)*0.5**o
        ref[2,2,1,1]=(0.787575747782)/(o+1.)+(0.228670208089)+(0.502027084477)*0.5**o
        ref[2,2,2,0]=(-0.54135991724)/(o+1.)+(-0.0501472726484)+(-0.583469643173)*0.5**o
        ref[2,2,2,1]=(0.201380986075)/(o+1.)+(-0.2478388652)+(-0.472362804158)*0.5**o
        ref[2,3,0,0]=(-0.921379470525)/(o+1.)+(0.228475352079)+(-0.547838738364)*0.5**o
        ref[2,3,0,1]=(-0.924235646481)/(o+1.)+(-0.653696568051)+(-0.557990737044)*0.5**o
        ref[2,3,1,0]=(-0.53810139661)/(o+1.)+(-0.460865328958)+(0.566861818147)*0.5**o
        ref[2,3,1,1]=(0.434855750568)/(o+1.)+(-0.117905464066)+(-0.0578464759524)*0.5**o
        ref[2,3,2,0]=(0.0228508430597)/(o+1.)+(-0.378271835956)+(0.388204865306)*0.5**o
        ref[2,3,2,1]=(-0.756416817999)/(o+1.)+(0.252288273374)+(0.70625829226)*0.5**o
        ref[2,4,0,0]=(0.389172632633)/(o+1.)+(0.29463618087)+(0.595101935764)*0.5**o
        ref[2,4,0,1]=(-0.481303115308)/(o+1.)+(0.326467208654)+(-0.59261552462)*0.5**o
        ref[2,4,1,0]=(0.0982270602491)/(o+1.)+(0.274553217055)+(-0.274360684371)*0.5**o
        ref[2,4,1,1]=(0.0530228030052)/(o+1.)+(-0.261524946318)+(0.0022428239189)*0.5**o
        ref[2,4,2,0]=(0.917864727482)/(o+1.)+(0.303722239096)+(-0.96226874392)*0.5**o
        ref[2,4,2,1]=(0.74280905237)/(o+1.)+(0.498109327645)+(0.684506236166)*0.5**o
        ref[3,0,0,0]=(0.0780367521917)/(o+1.)+(0.465357086509)+(0.658782504284)*0.5**o
        ref[3,0,0,1]=(0.992735332775)/(o+1.)+(-0.554885840775)+(0.386822554052)*0.5**o
        ref[3,0,1,0]=(0.810452271623)/(o+1.)+(-0.0648160822652)+(-0.253862856162)*0.5**o
        ref[3,0,1,1]=(-0.0455924137238)/(o+1.)+(0.373746106566)+(-0.297107568873)*0.5**o
        ref[3,0,2,0]=(0.668273722779)/(o+1.)+(0.357623581403)+(0.101103302929)*0.5**o
        ref[3,0,2,1]=(0.808003987088)/(o+1.)+(0.519305565593)+(0.144907392006)*0.5**o
        ref[3,1,0,0]=(-0.267767797056)/(o+1.)+(0.134127144711)+(0.050719218416)*0.5**o
        ref[3,1,0,1]=(0.754051956614)/(o+1.)+(0.246806690383)+(-0.488804469299)*0.5**o
        ref[3,1,1,0]=(-0.99768277192)/(o+1.)+(-0.687127967357)+(0.665579804971)*0.5**o
        ref[3,1,1,1]=(-0.942736776261)/(o+1.)+(-0.205995769892)+(0.28421003692)*0.5**o
        ref[3,1,2,0]=(0.637503921768)/(o+1.)+(0.140834881718)+(-0.271033791176)*0.5**o
        ref[3,1,2,1]=(-0.370702666003)/(o+1.)+(0.848816788649)+(-0.314094366001)*0.5**o
        ref[3,2,0,0]=(0.390277717008)/(o+1.)+(-0.106919254948)+(0.608693141558)*0.5**o
        ref[3,2,0,1]=(0.82234563191)/(o+1.)+(-0.12917279069)+(-0.911099837761)*0.5**o
        ref[3,2,1,0]=(-0.578952272394)/(o+1.)+(0.307812766709)+(-0.876634035391)*0.5**o
        ref[3,2,1,1]=(0.767590856273)/(o+1.)+(-0.117226372156)+(0.06295832442)*0.5**o
        ref[3,2,2,0]=(-0.957486087526)/(o+1.)+(-0.346495951936)+(-0.186380273058)*0.5**o
        ref[3,2,2,1]=(-0.103339143054)/(o+1.)+(-0.458089587303)+(-0.733041869578)*0.5**o
        ref[3,3,0,0]=(-0.00816783658019)/(o+1.)+(-0.0943126056733)+(0.381695999815)*0.5**o
        ref[3,3,0,1]=(0.988023235161)/(o+1.)+(0.471831721667)+(-0.458999917243)*0.5**o
        ref[3,3,1,0]=(-0.376576617413)/(o+1.)+(-0.377314663391)+(0.35272081264)*0.5**o
        ref[3,3,1,1]=(-0.6695966178)/(o+1.)+(-0.043322224362)+(-0.880579928879)*0.5**o
        ref[3,3,2,0]=(0.173717958996)/(o+1.)+(0.0856258574287)+(0.179006063683)*0.5**o
        ref[3,3,2,1]=(0.931619586753)/(o+1.)+(-0.0992836970501)+(0.772059778005)*0.5**o
        ref[3,4,0,0]=(0.498238525714)/(o+1.)+(-0.0405332346536)+(0.609550545616)*0.5**o
        ref[3,4,0,1]=(0.483353477991)/(o+1.)+(0.123282156934)+(0.925097562117)*0.5**o
        ref[3,4,1,0]=(-0.511134933342)/(o+1.)+(0.689769718305)+(-0.878072917122)*0.5**o
        ref[3,4,1,1]=(-0.493968471165)/(o+1.)+(0.200795857841)+(0.0745582123775)*0.5**o
        ref[3,4,2,0]=(-0.312111305637)/(o+1.)+(0.425838840642)+(-0.736620884918)*0.5**o
        ref[3,4,2,1]=(0.638515711599)/(o+1.)+(-0.508874087457)+(-0.663426736678)*0.5**o
      else:
        s[0,0,0,0]=(-0.938544579906)*x[0]**o+(0.180067483394)*x[0]+(-0.719233529982)*x[1]**o+(0.788617757419)*x[1]+(0.556994205355)*x[2]**o+(-0.127037817454)*x[2]
        s[0,0,0,1]=(0.953579560099)*x[0]**o+(0.562347618447)*x[0]+(-0.100565718949)*x[1]**o+(-0.110294341288)*x[1]+(0.23833069113)*x[2]**o+(-0.762859899976)*x[2]
        s[0,0,1,0]=(-0.0146573390123)*x[0]**o+(0.610306989551)*x[0]+(-0.322740693558)*x[1]**o+(0.0287307185836)*x[1]+(0.464511767666)*x[2]**o+(0.609636002246)*x[2]
        s[0,0,1,1]=(-0.658403837278)*x[0]**o+(0.446799474387)*x[0]+(-0.240860101539)*x[1]**o+(-0.198397925626)*x[1]+(0.874003266701)*x[2]**o+(0.313133624565)*x[2]
        s[0,0,2,0]=(0.62578079836)*x[0]**o+(-0.0257593936528)*x[0]+(-0.60984452513)*x[1]**o+(0.635946424498)*x[1]+(-0.218596411854)*x[2]**o+(0.00763050290993)*x[2]
        s[0,0,2,1]=(0.314093680988)*x[0]**o+(-0.869278986994)*x[0]+(-0.651770479439)*x[1]**o+(0.202960060033)*x[1]+(0.382275480924)*x[2]**o+(-0.595512215137)*x[2]
        s[0,1,0,0]=(0.00620161785372)*x[0]**o+(0.876847803666)*x[0]+(0.520320206245)*x[1]**o+(0.228011364507)*x[1]+(0.634707030605)*x[2]**o+(0.750039227257)*x[2]
        s[0,1,0,1]=(0.0303982843122)*x[0]**o+(-0.843928008659)*x[0]+(0.175282997162)*x[1]**o+(0.628713373934)*x[1]+(-0.260785452479)*x[2]**o+(-0.768157994235)*x[2]
        s[0,1,1,0]=(0.310811455642)*x[0]**o+(-0.907217641071)*x[0]+(-0.982538210789)*x[1]**o+(-0.600525634532)*x[1]+(-0.864394932414)*x[2]**o+(0.458881514929)*x[2]
        s[0,1,1,1]=(-0.751411238653)*x[0]**o+(-0.640190422476)*x[0]+(-0.408377872461)*x[1]**o+(0.72944579088)*x[1]+(0.946023481982)*x[2]**o+(-0.778523105712)*x[2]
        s[0,1,2,0]=(-0.852982317135)*x[0]**o+(-0.770284090211)*x[0]+(-0.0651898089121)*x[1]**o+(0.570265822823)*x[1]+(0.28669660112)*x[2]**o+(0.0651543264368)*x[2]
        s[0,1,2,1]=(-0.973162329979)*x[0]**o+(-0.246891970345)*x[0]+(0.0766743752541)*x[1]**o+(-0.403798628829)*x[1]+(-0.643254181567)*x[2]**o+(-0.378598111893)*x[2]
        s[0,2,0,0]=(0.472595355875)*x[0]**o+(0.00120464758292)*x[0]+(-0.42244937935)*x[1]**o+(-0.9164663947)*x[1]+(-0.0845858448629)*x[2]**o+(0.72392919756)*x[2]
        s[0,2,0,1]=(0.486817890916)*x[0]**o+(-0.70489295232)*x[0]+(-0.817086734784)*x[1]**o+(0.645820907995)*x[1]+(-0.335350124515)*x[2]**o+(-0.32309401603)*x[2]
        s[0,2,1,0]=(0.832549390017)*x[0]**o+(-0.434882976553)*x[0]+(0.0796018608256)*x[1]**o+(-0.815177284988)*x[1]+(0.420723606982)*x[2]**o+(-0.996616855812)*x[2]
        s[0,2,1,1]=(0.152135328754)*x[0]**o+(0.119431209093)*x[0]+(-0.0565707514319)*x[1]**o+(-0.779402141878)*x[1]+(0.504116223295)*x[2]**o+(0.35421565275)*x[2]
        s[0,2,2,0]=(-0.358237320736)*x[0]**o+(0.486656376538)*x[0]+(-0.831122271401)*x[1]**o+(-0.0521486506858)*x[1]+(-0.782495078477)*x[2]**o+(0.856211020463)*x[2]
        s[0,2,2,1]=(-0.1707212422)*x[0]**o+(-0.798979986621)*x[0]+(0.882236543414)*x[1]**o+(0.377099798101)*x[1]+(0.489392175267)*x[2]**o+(0.930902331932)*x[2]
        s[0,3,0,0]=(-0.128307529775)*x[0]**o+(-0.846783963276)*x[0]+(0.946673346517)*x[1]**o+(-0.335187413972)*x[1]+(-0.408325612416)*x[2]**o+(0.629338432074)*x[2]
        s[0,3,0,1]=(-0.554435822439)*x[0]**o+(0.990524237381)*x[0]+(-0.537192696978)*x[1]**o+(-0.826994397531)*x[1]+(0.866744291726)*x[2]**o+(-0.285154443258)*x[2]
        s[0,3,1,0]=(-0.823501956971)*x[0]**o+(0.881638907149)*x[0]+(-0.0906956584345)*x[1]**o+(-0.902956378971)*x[1]+(-0.161189770505)*x[2]**o+(-0.160538443759)*x[2]
        s[0,3,1,1]=(0.430542618951)*x[0]**o+(-0.769516956782)*x[0]+(0.475479099202)*x[1]**o+(-0.917793889771)*x[1]+(0.763381631494)*x[2]**o+(0.346636926064)*x[2]
        s[0,3,2,0]=(-0.958502709531)*x[0]**o+(-0.177711289961)*x[0]+(0.586266935723)*x[1]**o+(0.890784623626)*x[1]+(-0.787544127664)*x[2]**o+(0.446360741993)*x[2]
        s[0,3,2,1]=(-0.792569155533)*x[0]**o+(0.863653363017)*x[0]+(0.308377143826)*x[1]**o+(-0.30341033471)*x[1]+(0.0641187823207)*x[2]**o+(0.52775945782)*x[2]
        s[0,4,0,0]=(-0.0338438096091)*x[0]**o+(-0.200503853968)*x[0]+(-0.540577100866)*x[1]**o+(-0.228756900341)*x[1]+(-0.388513893583)*x[2]**o+(0.134360460805)*x[2]
        s[0,4,0,1]=(-0.406152624356)*x[0]**o+(-0.925153078113)*x[0]+(-0.432893083261)*x[1]**o+(0.118318067951)*x[1]+(0.839662677949)*x[2]**o+(-0.0163758572356)*x[2]
        s[0,4,1,0]=(-0.799456787818)*x[0]**o+(-0.0974094140856)*x[0]+(-0.967532895407)*x[1]**o+(0.313503423815)*x[1]+(-0.255345608246)*x[2]**o+(0.0212515985458)*x[2]
        s[0,4,1,1]=(0.561141037244)*x[0]**o+(0.421885997959)*x[0]+(-0.951943006997)*x[1]**o+(-0.293583958606)*x[1]+(0.26979445144)*x[2]**o+(-0.678061968406)*x[2]
        s[0,4,2,0]=(-0.346440790531)*x[0]**o+(0.486394576884)*x[0]+(0.318188318547)*x[1]**o+(-0.635068217742)*x[1]+(-0.0718964652741)*x[2]**o+(-0.702552674876)*x[2]
        s[0,4,2,1]=(0.450982323218)*x[0]**o+(0.682381807031)*x[0]+(-0.808585522869)*x[1]**o+(0.0201338006052)*x[1]+(-0.480685256454)*x[2]**o+(0.137075879378)*x[2]
        s[1,0,0,0]=(0.00370747226485)*x[0]**o+(-0.0925055645922)*x[0]+(-0.917381168613)*x[1]**o+(-0.193114928297)*x[1]+(-0.00570424377166)*x[2]**o+(0.913135198646)*x[2]
        s[1,0,0,1]=(0.314513438122)*x[0]**o+(-0.479659124185)*x[0]+(0.42293180951)*x[1]**o+(-0.597525641033)*x[1]+(-0.417172553037)*x[2]**o+(0.499251735975)*x[2]
        s[1,0,1,0]=(0.904368787276)*x[0]**o+(-0.514051100643)*x[0]+(0.212361933482)*x[1]**o+(-0.445110428363)*x[1]+(0.221673520737)*x[2]**o+(-0.0710601569942)*x[2]
        s[1,0,1,1]=(-0.0645313731593)*x[0]**o+(0.263267052465)*x[0]+(-0.715349556062)*x[1]**o+(0.236872392267)*x[1]+(0.605241495355)*x[2]**o+(-0.152375732378)*x[2]
        s[1,0,2,0]=(0.721223990054)*x[0]**o+(-0.704265146199)*x[0]+(-0.586881850891)*x[1]**o+(0.425307854616)*x[1]+(0.33484446761)*x[2]**o+(-0.573115444223)*x[2]
        s[1,0,2,1]=(-0.928667989887)*x[0]**o+(-0.877606514031)*x[0]+(-0.403016486409)*x[1]**o+(-0.517530622207)*x[1]+(0.0254991827594)*x[2]**o+(-0.82134638059)*x[2]
        s[1,1,0,0]=(-0.598976144645)*x[0]**o+(-0.0568344921691)*x[0]+(0.944100507076)*x[1]**o+(0.636963899978)*x[1]+(0.253911413671)*x[2]**o+(-0.178511347672)*x[2]
        s[1,1,0,1]=(0.51758257139)*x[0]**o+(-0.826610314817)*x[0]+(0.0669491359674)*x[1]**o+(-0.236450271581)*x[1]+(-0.702832265335)*x[2]**o+(-0.268110228427)*x[2]
        s[1,1,1,0]=(-0.847292260359)*x[0]**o+(-0.382061283693)*x[0]+(0.85982617142)*x[1]**o+(0.0930902132719)*x[1]+(0.568918674)*x[2]**o+(-0.811808588084)*x[2]
        s[1,1,1,1]=(0.396422756052)*x[0]**o+(0.667551047328)*x[0]+(-0.174441000478)*x[1]**o+(-0.40295080705)*x[1]+(0.075193248527)*x[2]**o+(-0.258123308085)*x[2]
        s[1,1,2,0]=(0.391968381002)*x[0]**o+(0.277092735448)*x[0]+(0.72619079286)*x[1]**o+(0.336787672025)*x[1]+(0.622007882)*x[2]**o+(0.286619657146)*x[2]
        s[1,1,2,1]=(-0.212692240987)*x[0]**o+(0.786660459419)*x[0]+(-0.528445652722)*x[1]**o+(0.177147918886)*x[1]+(0.40734794393)*x[2]**o+(0.155376280493)*x[2]
        s[1,2,0,0]=(0.0157691022819)*x[0]**o+(-0.661679523157)*x[0]+(0.123711726993)*x[1]**o+(-0.0870910621738)*x[1]+(-0.865865892104)*x[2]**o+(0.999338641606)*x[2]
        s[1,2,0,1]=(-0.828402321641)*x[0]**o+(-0.528217338749)*x[0]+(0.723149264147)*x[1]**o+(0.752532155985)*x[1]+(0.443929080203)*x[2]**o+(-0.929781507182)*x[2]
        s[1,2,1,0]=(-0.375626403305)*x[0]**o+(-0.510977471867)*x[0]+(0.288802388225)*x[1]**o+(0.168184581445)*x[1]+(-0.985676398515)*x[2]**o+(-0.993246463175)*x[2]
        s[1,2,1,1]=(0.839254171086)*x[0]**o+(-0.663664452054)*x[0]+(0.990048916717)*x[1]**o+(-0.986318241383)*x[1]+(0.726312571574)*x[2]**o+(0.411365473711)*x[2]
        s[1,2,2,0]=(0.179198759222)*x[0]**o+(0.362349404368)*x[0]+(0.372503911575)*x[1]**o+(-0.885663720586)*x[1]+(-0.656319589417)*x[2]**o+(-0.1547676929)*x[2]
        s[1,2,2,1]=(0.431283737039)*x[0]**o+(0.573392031568)*x[0]+(-0.167580615772)*x[1]**o+(-0.427449288729)*x[1]+(-0.771868709235)*x[2]**o+(-0.272748901939)*x[2]
        s[1,3,0,0]=(-0.735181806951)*x[0]**o+(0.536012589912)*x[0]+(0.623248739525)*x[1]**o+(0.590921832149)*x[1]+(-0.21724838693)*x[2]**o+(0.95165954451)*x[2]
        s[1,3,0,1]=(0.953086533561)*x[0]**o+(0.855151309595)*x[0]+(-0.877649378409)*x[1]**o+(-0.79234800867)*x[1]+(-0.647423149686)*x[2]**o+(0.718784987888)*x[2]
        s[1,3,1,0]=(-0.723560393521)*x[0]**o+(-0.0864445384144)*x[0]+(0.852841011672)*x[1]**o+(-0.953082229559)*x[1]+(0.983938853476)*x[2]**o+(-0.195518521863)*x[2]
        s[1,3,1,1]=(-0.915440550473)*x[0]**o+(-0.92449299685)*x[0]+(-0.150518843387)*x[1]**o+(-0.66259291358)*x[1]+(-0.812332416698)*x[2]**o+(-0.362998001567)*x[2]
        s[1,3,2,0]=(-0.205303485441)*x[0]**o+(-0.44047413477)*x[0]+(0.563675490655)*x[1]**o+(-0.802613298805)*x[1]+(-0.384663506204)*x[2]**o+(0.881891732298)*x[2]
        s[1,3,2,1]=(0.542339109554)*x[0]**o+(0.459852398778)*x[0]+(0.531257436652)*x[1]**o+(-0.0918654697667)*x[1]+(0.728385543797)*x[2]**o+(-0.142579191699)*x[2]
        s[1,4,0,0]=(-0.444230035592)*x[0]**o+(0.0532350197552)*x[0]+(-0.280357698853)*x[1]**o+(0.00480903349516)*x[1]+(-0.147327835489)*x[2]**o+(-0.630977138596)*x[2]
        s[1,4,0,1]=(0.972127984006)*x[0]**o+(0.904605524232)*x[0]+(0.44361144766)*x[1]**o+(0.586027289354)*x[1]+(-0.194334042153)*x[2]**o+(0.477271167786)*x[2]
        s[1,4,1,0]=(-0.162531060958)*x[0]**o+(-0.186338052619)*x[0]+(-0.0696302540091)*x[1]**o+(0.143088503291)*x[1]+(0.0107879016639)*x[2]**o+(0.39899128749)*x[2]
        s[1,4,1,1]=(0.315463685116)*x[0]**o+(-0.290348790738)*x[0]+(-0.292829602611)*x[1]**o+(-0.272218595021)*x[1]+(-0.247363514071)*x[2]**o+(0.830943482915)*x[2]
        s[1,4,2,0]=(0.340729082172)*x[0]**o+(0.266345838644)*x[0]+(0.205349295082)*x[1]**o+(-0.0281621785636)*x[1]+(0.89948000824)*x[2]**o+(-0.764082368496)*x[2]
        s[1,4,2,1]=(0.7615004712)*x[0]**o+(-0.366790213041)*x[0]+(-0.347719278119)*x[1]**o+(-0.664644802127)*x[1]+(0.153340126952)*x[2]**o+(-0.699790837984)*x[2]
        s[2,0,0,0]=(0.0125409231833)*x[0]**o+(-0.497512034949)*x[0]+(0.688244767493)*x[1]**o+(-0.160005018143)*x[1]+(-0.119715348465)*x[2]**o+(-0.797969768194)*x[2]
        s[2,0,0,1]=(-0.0036047535766)*x[0]**o+(0.726289674137)*x[0]+(-0.110716355541)*x[1]**o+(-0.937117263903)*x[1]+(0.540476278776)*x[2]**o+(-0.0254788538753)*x[2]
        s[2,0,1,0]=(0.0494808240706)*x[0]**o+(-0.469224784419)*x[0]+(0.599591744744)*x[1]**o+(-0.694323071412)*x[1]+(-0.584890416602)*x[2]**o+(0.795168667963)*x[2]
        s[2,0,1,1]=(-0.53303893512)*x[0]**o+(0.012157220346)*x[0]+(-0.546109156126)*x[1]**o+(-0.719758078719)*x[1]+(-0.496492728702)*x[2]**o+(0.816565066681)*x[2]
        s[2,0,2,0]=(-0.306356882566)*x[0]**o+(0.501809420086)*x[0]+(0.171130235154)*x[1]**o+(-0.804095060824)*x[1]+(0.929639721152)*x[2]**o+(-0.516285706731)*x[2]
        s[2,0,2,1]=(0.337441903133)*x[0]**o+(-0.524705933088)*x[0]+(0.490789790623)*x[1]**o+(0.0420695613636)*x[1]+(-0.296688134663)*x[2]**o+(-0.508555097135)*x[2]
        s[2,1,0,0]=(0.984282580742)*x[0]**o+(0.692515911833)*x[0]+(0.610802160351)*x[1]**o+(0.174938187813)*x[1]+(0.777154457482)*x[2]**o+(-0.296265352251)*x[2]
        s[2,1,0,1]=(0.263434795622)*x[0]**o+(0.146263475693)*x[0]+(-0.135060978)*x[1]**o+(-0.612057700058)*x[1]+(0.401206478053)*x[2]**o+(0.614641881217)*x[2]
        s[2,1,1,0]=(0.839155090264)*x[0]**o+(0.893746351779)*x[0]+(0.652777744086)*x[1]**o+(-0.119538495003)*x[1]+(-0.116919455352)*x[2]**o+(-0.24237363241)*x[2]
        s[2,1,1,1]=(-0.626863009974)*x[0]**o+(-0.328592553389)*x[0]+(-0.483755510715)*x[1]**o+(0.506129888719)*x[1]+(-0.369314928449)*x[2]**o+(-0.974391328259)*x[2]
        s[2,1,2,0]=(-0.548834239805)*x[0]**o+(-0.137282729528)*x[0]+(0.980067336071)*x[1]**o+(-0.677019881527)*x[1]+(-0.633352248762)*x[2]**o+(0.394845574769)*x[2]
        s[2,1,2,1]=(0.304097820123)*x[0]**o+(0.205705406237)*x[0]+(0.431075221394)*x[1]**o+(0.965948087934)*x[1]+(0.797211333629)*x[2]**o+(-0.750481732005)*x[2]
        s[2,2,0,0]=(0.00294465132151)*x[0]**o+(-0.36531365965)*x[0]+(-0.477095178168)*x[1]**o+(0.640123142737)*x[1]+(0.307285555133)*x[2]**o+(-0.535459483965)*x[2]
        s[2,2,0,1]=(0.185831262234)*x[0]**o+(0.680621489598)*x[0]+(-0.606597858106)*x[1]**o+(0.605582792294)*x[1]+(0.67028817128)*x[2]**o+(0.0936479735813)*x[2]
        s[2,2,1,0]=(0.234717477502)*x[0]**o+(0.739313223999)*x[0]+(-0.765253630778)*x[1]**o+(-0.199364440863)*x[1]+(-0.10386639265)*x[2]**o+(-0.772307849674)*x[2]
        s[2,2,1,1]=(0.692290926695)*x[0]**o+(-0.841554910484)*x[0]+(-0.156803726414)*x[1]**o+(0.656797138877)*x[1]+(0.4668832013)*x[2]**o+(-0.320142791051)*x[2]
        s[2,2,2,0]=(-0.408729971023)*x[0]**o+(0.28023578811)*x[0]+(0.756339486697)*x[1]**o+(0.130791597709)*x[1]+(-0.388130026173)*x[2]**o+(-0.232133676528)*x[2]
        s[2,2,2,1]=(-0.30215609634)*x[0]**o+(-0.515622149267)*x[0]+(-0.979560898475)*x[1]**o+(0.424855014788)*x[1]+(-0.0265200291972)*x[2]**o+(0.244204914292)*x[2]
        s[2,3,0,0]=(-0.731783739254)*x[0]**o+(-0.910917581289)*x[0]+(-0.490859941251)*x[1]**o+(0.265521242704)*x[1]+(0.246576247389)*x[2]**o+(0.551033884684)*x[2]
        s[2,3,0,1]=(0.353316440436)*x[0]**o+(0.607852620987)*x[0]+(0.716634525835)*x[1]**o+(0.931269029696)*x[1]+(0.512042632855)*x[2]**o+(0.84647829057)*x[2]
        s[2,3,1,0]=(-0.682583926526)*x[0]**o+(0.0414372583894)*x[0]+(-0.368919279038)*x[1]**o+(0.263442044187)*x[1]+(0.651154272252)*x[2]**o+(0.625505230753)*x[2]
        s[2,3,1,1]=(-0.773410808141)*x[0]**o+(-0.308532872412)*x[0]+(0.484022099132)*x[1]**o+(0.748310916514)*x[1]+(0.484822489595)*x[2]**o+(0.0919219970104)*x[2]
        s[2,3,2,0]=(-0.856792343472)*x[0]**o+(0.475526910813)*x[0]+(0.858286241213)*x[1]**o+(0.765684964483)*x[1]+(0.842680693423)*x[2]**o+(0.759239366541)*x[2]
        s[2,3,2,1]=(0.208814714888)*x[0]**o+(-0.319616331598)*x[0]+(-0.786662106876)*x[1]**o+(-0.742419573118)*x[1]+(-0.671654532185)*x[2]**o+(-0.81098024788)*x[2]
        s[2,4,0,0]=(0.0581246014334)*x[0]**o+(-0.738212823784)*x[0]+(0.42828697377)*x[1]**o+(-0.795604474847)*x[1]+(-0.867930879138)*x[2]**o+(0.897777664383)*x[2]
        s[2,4,0,1]=(0.658005351748)*x[0]**o+(0.318056754281)*x[0]+(-0.241917242833)*x[1]**o+(0.321210074114)*x[1]+(0.341921510554)*x[2]**o+(-0.713222645312)*x[2]
        s[2,4,1,0]=(-0.865800829839)*x[0]**o+(-0.24789092495)*x[0]+(-0.765024475984)*x[1]**o+(0.246675200333)*x[1]+(0.63601593739)*x[2]**o+(0.758761609858)*x[2]
        s[2,4,1,1]=(-0.722774458642)*x[0]**o+(-0.640277272458)*x[0]+(-0.847467314954)*x[1]**o+(-0.676376490579)*x[1]+(-0.867510590824)*x[2]**o+(0.304980448789)*x[2]
        s[2,4,2,0]=(-0.556352556597)*x[0]**o+(-0.314836337315)*x[0]+(-0.172549538929)*x[1]**o+(-0.603791779732)*x[1]+(-0.482556973138)*x[2]**o+(0.155915646569)*x[2]
        s[2,4,2,1]=(-0.790402254477)*x[0]**o+(0.137494169657)*x[0]+(-0.546722957671)*x[1]**o+(-0.504092758637)*x[1]+(0.370205905665)*x[2]**o+(0.905182951166)*x[2]
        s[3,0,0,0]=(-0.393498907366)*x[0]**o+(0.792111044321)*x[0]+(0.160732029377)*x[1]**o+(0.531118521589)*x[1]+(0.616077214135)*x[2]**o+(0.0450299929995)*x[2]
        s[3,0,0,1]=(0.0490385695005)*x[0]**o+(-0.112630305424)*x[0]+(0.932194477491)*x[1]**o+(-0.692641363201)*x[1]+(0.0679516868523)*x[2]**o+(0.997252361895)*x[2]
        s[3,0,1,0]=(0.372518182464)*x[0]**o+(-0.390347141072)*x[0]+(-0.551584590493)*x[1]**o+(-0.762622043128)*x[1]+(-0.777489328164)*x[2]**o+(-0.306555622213)*x[2]
        s[3,0,1,1]=(-0.907039911971)*x[0]**o+(0.716176125684)*x[0]+(-0.0317846045066)*x[1]**o+(0.344154975432)*x[1]+(-0.581050833334)*x[2]**o+(-0.800597412247)*x[2]
        s[3,0,2,0]=(0.671061807362)*x[0]**o+(-0.632730364318)*x[0]+(0.784926714274)*x[1]**o+(-0.689167186471)*x[1]+(-0.828322284596)*x[2]**o+(-0.757384289233)*x[2]
        s[3,0,2,1]=(0.797200318087)*x[0]**o+(0.0748753590429)*x[0]+(0.98432669884)*x[1]**o+(0.19160698616)*x[1]+(-0.941839443214)*x[2]**o+(-0.51797940249)*x[2]
        s[3,1,0,0]=(0.396318123324)*x[0]**o+(0.553544474446)*x[0]+(-0.196123045916)*x[1]**o+(0.931142300652)*x[1]+(-0.893432985678)*x[2]**o+(0.61239614626)*x[2]
        s[3,1,0,1]=(-0.281320711026)*x[0]**o+(0.0272769480991)*x[0]+(-0.43587599777)*x[1]**o+(0.419822767553)*x[1]+(-0.453045711683)*x[2]**o+(0.47569249252)*x[2]
        s[3,1,1,0]=(-0.213862255449)*x[0]**o+(0.355661536766)*x[0]+(0.251742827449)*x[1]**o+(-0.807017120416)*x[1]+(-0.730339835476)*x[2]**o+(0.551573971157)*x[2]
        s[3,1,1,1]=(0.554657719161)*x[0]**o+(-0.912970059328)*x[0]+(0.800146831674)*x[1]**o+(-0.583227572533)*x[1]+(-0.0933072839888)*x[2]**o+(0.950038311476)*x[2]
        s[3,1,2,0]=(-0.485569875217)*x[0]**o+(-0.636350568501)*x[0]+(-0.560720680426)*x[1]**o+(0.126204185032)*x[1]+(-0.272414378167)*x[2]**o+(-0.888532404451)*x[2]
        s[3,1,2,1]=(-0.727409224907)*x[0]**o+(0.856672459636)*x[0]+(0.37848918095)*x[1]**o+(0.622080880799)*x[1]+(-0.228417584572)*x[2]**o+(0.562188460361)*x[2]
        s[3,2,0,0]=(-0.294609451063)*x[0]**o+(0.577588405163)*x[0]+(0.667703746348)*x[1]**o+(-0.910501998185)*x[1]+(0.843110506022)*x[2]**o+(-0.826714153775)*x[2]
        s[3,2,0,1]=(-0.193869261184)*x[0]**o+(0.76598264579)*x[0]+(-0.15832998161)*x[1]**o+(0.647800212602)*x[1]+(0.0578289377486)*x[2]**o+(0.00658825845732)*x[2]
        s[3,2,1,0]=(-0.567464737056)*x[0]**o+(-0.149652747243)*x[0]+(-0.55259439138)*x[1]**o+(-0.879859303137)*x[1]+(-0.618763037058)*x[2]**o+(-0.361881515471)*x[2]
        s[3,2,1,1]=(-0.331240639597)*x[0]**o+(-0.842702790198)*x[0]+(0.14240123238)*x[1]**o+(0.732696112887)*x[1]+(0.678517810911)*x[2]**o+(0.874215749104)*x[2]
        s[3,2,2,0]=(0.569105573686)*x[0]**o+(-0.671670416897)*x[0]+(0.809241806071)*x[1]**o+(-0.781683673317)*x[1]+(0.915497987962)*x[2]**o+(0.529423259835)*x[2]
        s[3,2,2,1]=(0.803926266415)*x[0]**o+(0.973851217854)*x[0]+(0.00110452925071)*x[1]**o+(-0.0258074727224)*x[1]+(0.397580796648)*x[2]**o+(-0.483394954284)*x[2]
        s[3,3,0,0]=(-0.239744952662)*x[0]**o+(0.0330238686771)*x[0]+(0.57663829323)*x[1]**o+(-0.835285980271)*x[1]+(-0.541211231475)*x[2]**o+(0.309919910162)*x[2]
        s[3,3,0,1]=(0.998856178514)*x[0]**o+(0.69963418829)*x[0]+(-0.590956618638)*x[1]**o+(0.437083139196)*x[1]+(0.24957499362)*x[2]**o+(-0.207180573624)*x[2]
        s[3,3,1,0]=(-0.654845715855)*x[0]**o+(0.960307610722)*x[0]+(0.615977294745)*x[1]**o+(-0.717345465881)*x[1]+(-0.0724170395954)*x[2]**o+(0.660888358757)*x[2]
        s[3,3,1,1]=(-0.283730169109)*x[0]**o+(0.712609062937)*x[0]+(0.922422201518)*x[1]**o+(0.941787071667)*x[1]+(-0.317440296596)*x[2]**o+(-0.776070611464)*x[2]
        s[3,3,2,0]=(-0.377933655183)*x[0]**o+(-0.946504205082)*x[0]+(-0.705852108222)*x[1]**o+(0.735471828851)*x[1]+(0.740387572185)*x[2]**o+(0.478923064569)*x[2]
        s[3,3,2,1]=(-0.74484862371)*x[0]**o+(-0.394147355587)*x[0]+(-0.961488506416)*x[1]**o+(-0.117956733646)*x[1]+(-0.45639006521)*x[2]**o+(0.800113732885)*x[2]
        s[3,4,0,0]=(-0.625504798604)*x[0]**o+(0.316567164198)*x[0]+(-0.616071176069)*x[1]**o+(0.882101858084)*x[1]+(0.734544640696)*x[2]**o+(-0.0269245549068)*x[2]
        s[3,4,0,1]=(-0.307431859474)*x[0]**o+(-0.0850940272398)*x[0]+(0.389650545234)*x[1]**o+(-0.0311288588201)*x[1]+(0.560636030796)*x[2]**o+(0.990554208909)*x[2]
        s[3,4,1,0]=(-0.85379517016)*x[0]**o+(0.768069325756)*x[0]+(0.0149807290805)*x[1]**o+(-0.87320355719)*x[1]+(-0.153198452162)*x[2]**o+(0.0809551506622)*x[2]
        s[3,4,1,1]=(-0.427217312348)*x[0]**o+(0.860261211584)*x[0]+(0.988421955689)*x[1]**o+(0.10533108572)*x[1]+(0.903327272548)*x[2]**o+(0.434209037137)*x[2]
        s[3,4,2,0]=(-0.264611936187)*x[0]**o+(0.0438724619932)*x[0]+(0.398401170965)*x[1]**o+(0.53328233455)*x[1]+(-0.437516039922)*x[2]**o+(0.926837033874)*x[2]
        s[3,4,2,1]=(-0.0213427465429)*x[0]**o+(-0.41324925677)*x[0]+(0.00876072281828)*x[1]**o+(0.640763350258)*x[1]+(-0.548829379894)*x[2]**o+(0.523108062556)*x[2]
        ref[0,0,0,0]=(-0.162239324627)/(o+1.)+(0.420823711679)+(-0.938544579906)*0.5**o
        ref[0,0,0,1]=(0.137764972182)/(o+1.)+(-0.155403311408)+(0.953579560099)*0.5**o
        ref[0,0,1,0]=(0.141771074109)/(o+1.)+(0.62433685519)+(-0.0146573390123)*0.5**o
        ref[0,0,1,1]=(0.633143165162)/(o+1.)+(0.280767586663)+(-0.658403837278)*0.5**o
        ref[0,0,2,0]=(-0.828440936984)/(o+1.)+(0.308908766878)+(0.62578079836)*0.5**o
        ref[0,0,2,1]=(-0.269494998515)/(o+1.)+(-0.630915571049)+(0.314093680988)*0.5**o
        ref[0,1,0,0]=(1.15502723685)/(o+1.)+(0.927449197715)+(0.00620161785372)*0.5**o
        ref[0,1,0,1]=(-0.0855024553167)/(o+1.)+(-0.49168631448)+(0.0303982843122)*0.5**o
        ref[0,1,1,0]=(-1.8469331432)/(o+1.)+(-0.524430880337)+(0.310811455642)*0.5**o
        ref[0,1,1,1]=(0.537645609521)/(o+1.)+(-0.344633868654)+(-0.751411238653)*0.5**o
        ref[0,1,2,0]=(0.221506792208)/(o+1.)+(-0.0674319704757)+(-0.852982317135)*0.5**o
        ref[0,1,2,1]=(-0.566579806313)/(o+1.)+(-0.514644355533)+(-0.973162329979)*0.5**o
        ref[0,2,0,0]=(-0.507035224213)/(o+1.)+(-0.0956662747788)+(0.472595355875)*0.5**o
        ref[0,2,0,1]=(-1.1524368593)/(o+1.)+(-0.191083030177)+(0.486817890916)*0.5**o
        ref[0,2,1,0]=(0.500325467807)/(o+1.)+(-1.12333855868)+(0.832549390017)*0.5**o
        ref[0,2,1,1]=(0.447545471863)/(o+1.)+(-0.152877640017)+(0.152135328754)*0.5**o
        ref[0,2,2,0]=(-1.61361734988)/(o+1.)+(0.645359373158)+(-0.358237320736)*0.5**o
        ref[0,2,2,1]=(1.37162871868)/(o+1.)+(0.254511071706)+(-0.1707212422)*0.5**o
        ref[0,3,0,0]=(0.538347734102)/(o+1.)+(-0.276316472587)+(-0.128307529775)*0.5**o
        ref[0,3,0,1]=(0.329551594748)/(o+1.)+(-0.0608123017038)+(-0.554435822439)*0.5**o
        ref[0,3,1,0]=(-0.251885428939)/(o+1.)+(-0.0909279577907)+(-0.823501956971)*0.5**o
        ref[0,3,1,1]=(1.2388607307)/(o+1.)+(-0.670336960245)+(0.430542618951)*0.5**o
        ref[0,3,2,0]=(-0.201277191941)/(o+1.)+(0.579717037829)+(-0.958502709531)*0.5**o
        ref[0,3,2,1]=(0.372495926146)/(o+1.)+(0.544001243064)+(-0.792569155533)*0.5**o
        ref[0,4,0,0]=(-0.929090994448)/(o+1.)+(-0.147450146752)+(-0.0338438096091)*0.5**o
        ref[0,4,0,1]=(0.406769594688)/(o+1.)+(-0.411605433699)+(-0.406152624356)*0.5**o
        ref[0,4,1,0]=(-1.22287850365)/(o+1.)+(0.118672804138)+(-0.799456787818)*0.5**o
        ref[0,4,1,1]=(-0.682148555557)/(o+1.)+(-0.274879964526)+(0.561141037244)*0.5**o
        ref[0,4,2,0]=(0.246291853273)/(o+1.)+(-0.425613157867)+(-0.346440790531)*0.5**o
        ref[0,4,2,1]=(-1.28927077932)/(o+1.)+(0.419795743507)+(0.450982323218)*0.5**o
        ref[1,0,0,0]=(-0.923085412384)/(o+1.)+(0.313757352879)+(0.00370747226485)*0.5**o
        ref[1,0,0,1]=(0.00575925647317)/(o+1.)+(-0.288966514622)+(0.314513438122)*0.5**o
        ref[1,0,1,0]=(0.434035454219)/(o+1.)+(-0.515110843)+(0.904368787276)*0.5**o
        ref[1,0,1,1]=(-0.110108060707)/(o+1.)+(0.173881856177)+(-0.0645313731593)*0.5**o
        ref[1,0,2,0]=(-0.252037383281)/(o+1.)+(-0.426036367903)+(0.721223990054)*0.5**o
        ref[1,0,2,1]=(-0.37751730365)/(o+1.)+(-1.10824175841)+(-0.928667989887)*0.5**o
        ref[1,1,0,0]=(1.19801192075)/(o+1.)+(0.200809030069)+(-0.598976144645)*0.5**o
        ref[1,1,0,1]=(-0.635883129368)/(o+1.)+(-0.665585407412)+(0.51758257139)*0.5**o
        ref[1,1,1,0]=(1.42874484542)/(o+1.)+(-0.550389829253)+(-0.847292260359)*0.5**o
        ref[1,1,1,1]=(-0.0992477519506)/(o+1.)+(0.00323846609633)+(0.396422756052)*0.5**o
        ref[1,1,2,0]=(1.34819867486)/(o+1.)+(0.450250032309)+(0.391968381002)*0.5**o
        ref[1,1,2,1]=(-0.121097708792)/(o+1.)+(0.559592329399)+(-0.212692240987)*0.5**o
        ref[1,2,0,0]=(-0.74215416511)/(o+1.)+(0.125284028138)+(0.0157691022819)*0.5**o
        ref[1,2,0,1]=(1.16707834435)/(o+1.)+(-0.352733344973)+(-0.828402321641)*0.5**o
        ref[1,2,1,0]=(-0.69687401029)/(o+1.)+(-0.668019676798)+(-0.375626403305)*0.5**o
        ref[1,2,1,1]=(1.71636148829)/(o+1.)+(-0.619308609863)+(0.839254171086)*0.5**o
        ref[1,2,2,0]=(-0.283815677841)/(o+1.)+(-0.339041004559)+(0.179198759222)*0.5**o
        ref[1,2,2,1]=(-0.939449325007)/(o+1.)+(-0.0634030795503)+(0.431283737039)*0.5**o
        ref[1,3,0,0]=(0.406000352595)/(o+1.)+(1.03929698329)+(-0.735181806951)*0.5**o
        ref[1,3,0,1]=(-1.52507252809)/(o+1.)+(0.390794144406)+(0.953086533561)*0.5**o
        ref[1,3,1,0]=(1.83677986515)/(o+1.)+(-0.617522644919)+(-0.723560393521)*0.5**o
        ref[1,3,1,1]=(-0.962851260085)/(o+1.)+(-0.975041955998)+(-0.915440550473)*0.5**o
        ref[1,3,2,0]=(0.179011984451)/(o+1.)+(-0.180597850638)+(-0.205303485441)*0.5**o
        ref[1,3,2,1]=(1.25964298045)/(o+1.)+(0.112703868656)+(0.542339109554)*0.5**o
        ref[1,4,0,0]=(-0.427685534341)/(o+1.)+(-0.286466542673)+(-0.444230035592)*0.5**o
        ref[1,4,0,1]=(0.249277405507)/(o+1.)+(0.983951990686)+(0.972127984006)*0.5**o
        ref[1,4,1,0]=(-0.0588423523453)/(o+1.)+(0.177870869081)+(-0.162531060958)*0.5**o
        ref[1,4,1,1]=(-0.540193116681)/(o+1.)+(0.134188048578)+(0.315463685116)*0.5**o
        ref[1,4,2,0]=(1.10482930332)/(o+1.)+(-0.262949354208)+(0.340729082172)*0.5**o
        ref[1,4,2,1]=(-0.194379151166)/(o+1.)+(-0.865612926576)+(0.7615004712)*0.5**o
        ref[2,0,0,0]=(0.568529419028)/(o+1.)+(-0.727743410643)+(0.0125409231833)*0.5**o
        ref[2,0,0,1]=(0.429759923235)/(o+1.)+(-0.118153221821)+(-0.0036047535766)*0.5**o
        ref[2,0,1,0]=(0.0147013281422)/(o+1.)+(-0.184189593934)+(0.0494808240706)*0.5**o
        ref[2,0,1,1]=(-1.04260188483)/(o+1.)+(0.0544821041543)+(-0.53303893512)*0.5**o
        ref[2,0,2,0]=(1.10076995631)/(o+1.)+(-0.409285673735)+(-0.306356882566)*0.5**o
        ref[2,0,2,1]=(0.19410165596)/(o+1.)+(-0.49559573443)+(0.337441903133)*0.5**o
        ref[2,1,0,0]=(1.38795661783)/(o+1.)+(0.285594373698)+(0.984282580742)*0.5**o
        ref[2,1,0,1]=(0.266145500053)/(o+1.)+(0.0744238284255)+(0.263434795622)*0.5**o
        ref[2,1,1,0]=(0.535858288734)/(o+1.)+(0.265917112183)+(0.839155090264)*0.5**o
        ref[2,1,1,1]=(-0.853070439164)/(o+1.)+(-0.398426996464)+(-0.626863009974)*0.5**o
        ref[2,1,2,0]=(0.346715087308)/(o+1.)+(-0.209728518143)+(-0.548834239805)*0.5**o
        ref[2,1,2,1]=(1.22828655502)/(o+1.)+(0.210585881083)+(0.304097820123)*0.5**o
        ref[2,2,0,0]=(-0.169809623035)/(o+1.)+(-0.13032500044)+(0.00294465132151)*0.5**o
        ref[2,2,0,1]=(0.0636903131735)/(o+1.)+(0.689926127736)+(0.185831262234)*0.5**o
        ref[2,2,1,0]=(-0.869120023429)/(o+1.)+(-0.116179533269)+(0.234717477502)*0.5**o
        ref[2,2,1,1]=(0.310079474886)/(o+1.)+(-0.252450281328)+(0.692290926695)*0.5**o
        ref[2,2,2,0]=(0.368209460524)/(o+1.)+(0.0894468546456)+(-0.408729971023)*0.5**o
        ref[2,2,2,1]=(-1.00608092767)/(o+1.)+(0.0767188899067)+(-0.30215609634)*0.5**o
        ref[2,3,0,0]=(-0.244283693862)/(o+1.)+(-0.0471812269507)+(-0.731783739254)*0.5**o
        ref[2,3,0,1]=(1.22867715869)/(o+1.)+(1.19279997063)+(0.353316440436)*0.5**o
        ref[2,3,1,0]=(0.282234993214)/(o+1.)+(0.465192266665)+(-0.682583926526)*0.5**o
        ref[2,3,1,1]=(0.968844588727)/(o+1.)+(0.265850020556)+(-0.773410808141)*0.5**o
        ref[2,3,2,0]=(1.70096693464)/(o+1.)+(1.00022562092)+(-0.856792343472)*0.5**o
        ref[2,3,2,1]=(-1.45831663906)/(o+1.)+(-0.936508076298)+(0.208814714888)*0.5**o
        ref[2,4,0,0]=(-0.439643905368)/(o+1.)+(-0.318019817124)+(0.0581246014334)*0.5**o
        ref[2,4,0,1]=(0.100004267721)/(o+1.)+(-0.0369779084584)+(0.658005351748)*0.5**o
        ref[2,4,1,0]=(-0.129008538594)/(o+1.)+(0.37877294262)+(-0.865800829839)*0.5**o
        ref[2,4,1,1]=(-1.71497790578)/(o+1.)+(-0.505836657124)+(-0.722774458642)*0.5**o
        ref[2,4,2,0]=(-0.655106512067)/(o+1.)+(-0.38135623524)+(-0.556352556597)*0.5**o
        ref[2,4,2,1]=(-0.176517052006)/(o+1.)+(0.269292181093)+(-0.790402254477)*0.5**o
        ref[3,0,0,0]=(0.776809243512)/(o+1.)+(0.684129779455)+(-0.393498907366)*0.5**o
        ref[3,0,0,1]=(1.00014616434)/(o+1.)+(0.0959903466349)+(0.0490385695005)*0.5**o
        ref[3,0,1,0]=(-1.32907391866)/(o+1.)+(-0.729762403206)+(0.372518182464)*0.5**o
        ref[3,0,1,1]=(-0.612835437841)/(o+1.)+(0.129866844435)+(-0.907039911971)*0.5**o
        ref[3,0,2,0]=(-0.0433955703218)/(o+1.)+(-1.03964092001)+(0.671061807362)*0.5**o
        ref[3,0,2,1]=(0.0424872556266)/(o+1.)+(-0.125748528644)+(0.797200318087)*0.5**o
        ref[3,1,0,0]=(-1.08955603159)/(o+1.)+(1.04854146068)+(0.396318123324)*0.5**o
        ref[3,1,0,1]=(-0.888921709452)/(o+1.)+(0.461396104086)+(-0.281320711026)*0.5**o
        ref[3,1,1,0]=(-0.478597008027)/(o+1.)+(0.0501091937538)+(-0.213862255449)*0.5**o
        ref[3,1,1,1]=(0.706839547685)/(o+1.)+(-0.273079660193)+(0.554657719161)*0.5**o
        ref[3,1,2,0]=(-0.833135058593)/(o+1.)+(-0.69933939396)+(-0.485569875217)*0.5**o
        ref[3,1,2,1]=(0.150071596378)/(o+1.)+(1.0204709004)+(-0.727409224907)*0.5**o
        ref[3,2,0,0]=(1.51081425237)/(o+1.)+(-0.579813873399)+(-0.294609451063)*0.5**o
        ref[3,2,0,1]=(-0.100501043862)/(o+1.)+(0.710185558425)+(-0.193869261184)*0.5**o
        ref[3,2,1,0]=(-1.17135742844)/(o+1.)+(-0.695696782926)+(-0.567464737056)*0.5**o
        ref[3,2,1,1]=(0.820919043291)/(o+1.)+(0.382104535896)+(-0.331240639597)*0.5**o
        ref[3,2,2,0]=(1.72473979403)/(o+1.)+(-0.461965415189)+(0.569105573686)*0.5**o
        ref[3,2,2,1]=(0.398685325899)/(o+1.)+(0.232324395424)+(0.803926266415)*0.5**o
        ref[3,3,0,0]=(0.0354270617556)/(o+1.)+(-0.246171100716)+(-0.239744952662)*0.5**o
        ref[3,3,0,1]=(-0.341381625018)/(o+1.)+(0.464768376931)+(0.998856178514)*0.5**o
        ref[3,3,1,0]=(0.543560255149)/(o+1.)+(0.451925251799)+(-0.654845715855)*0.5**o
        ref[3,3,1,1]=(0.604981904922)/(o+1.)+(0.43916276157)+(-0.283730169109)*0.5**o
        ref[3,3,2,0]=(0.0345354639628)/(o+1.)+(0.133945344169)+(-0.377933655183)*0.5**o
        ref[3,3,2,1]=(-1.41787857163)/(o+1.)+(0.144004821826)+(-0.74484862371)*0.5**o
        ref[3,4,0,0]=(0.118473464627)/(o+1.)+(0.585872233688)+(-0.625504798604)*0.5**o
        ref[3,4,0,1]=(0.95028657603)/(o+1.)+(0.437165661424)+(-0.307431859474)*0.5**o
        ref[3,4,1,0]=(-0.138217723081)/(o+1.)+(-0.0120895403861)+(-0.85379517016)*0.5**o
        ref[3,4,1,1]=(1.89174922824)/(o+1.)+(0.699900667221)+(-0.427217312348)*0.5**o
        ref[3,4,2,0]=(-0.0391148689569)/(o+1.)+(0.751995915209)+(-0.264611936187)*0.5**o
        ref[3,4,2,1]=(-0.540068657076)/(o+1.)+(0.375311078022)+(-0.0213427465429)*0.5**o
      res=integrate(arg)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")

   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactZero_fromData_ContinuousFunction_rank0(self):
      """
      tests integral of rank 0 Data on the ReducedFunctionOnContactZero

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(),w)
      if dim==2:
        arg=(0.311774053155)*x[0]**o+(0.52292841627)*x[0]+(-0.518467126196)*x[1]**o+(-0.11329290159)*x[1]
        ref=(-0.518467126196)/(o+1.)+(0.20481775734)+(0.311774053155)*0.5**o
      else:
        arg=(0.233765319091)*x[0]**o+(-0.453551209762)*x[0]+(0.920140231713)*x[1]**o+(0.741695479084)*x[1]+(-0.255834461065)*x[2]**o+(-0.102580899053)*x[2]
        ref=(0.664305770648)/(o+1.)+(0.0927816851348)+(0.233765319091)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactZero_fromData_ContinuousFunction_rank1(self):
      """
      tests integral of rank 1 Data on the ReducedFunctionOnContactZero

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(2,),w)
      ref=numpy.zeros((2,),numpy.float_)
      if dim==2:
        arg[0]=(-0.651678780872)*x[0]**o+(-0.749709447394)*x[0]+(0.311088209082)*x[1]**o+(0.553081401193)*x[1]
        arg[1]=(0.657887728987)*x[0]**o+(0.052270143604)*x[0]+(-0.15507994277)*x[1]**o+(-0.30302150082)*x[1]
        ref[0]=(0.311088209082)/(o+1.)+(-0.0983140231004)+(-0.651678780872)*0.5**o
        ref[1]=(-0.15507994277)/(o+1.)+(-0.125375678608)+(0.657887728987)*0.5**o
      else:
        arg[0]=(0.844909347644)*x[0]**o+(-0.342697697483)*x[0]+(0.126242548186)*x[1]**o+(-0.854010732849)*x[1]+(-0.307627587461)*x[2]**o+(-0.998566290754)*x[2]
        arg[1]=(0.900073726701)*x[0]**o+(-0.703841701859)*x[0]+(-0.651690896499)*x[1]**o+(0.828362322181)*x[1]+(0.284410420895)*x[2]**o+(-0.394606348974)*x[2]
        ref[0]=(-0.181385039275)/(o+1.)+(-1.09763736054)+(0.844909347644)*0.5**o
        ref[1]=(-0.367280475604)/(o+1.)+(-0.135042864326)+(0.900073726701)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactZero_fromData_ContinuousFunction_rank2(self):
      """
      tests integral of rank 2 Data on the ReducedFunctionOnContactZero

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5),w)
      ref=numpy.zeros((4, 5),numpy.float_)
      if dim==2:
        arg[0,0]=(-0.263642681371)*x[0]**o+(0.668992571787)*x[0]+(0.873939846194)*x[1]**o+(0.784820355998)*x[1]
        arg[0,1]=(-0.389306088628)*x[0]**o+(-0.0549482389907)*x[0]+(-0.787101008457)*x[1]**o+(0.860345123812)*x[1]
        arg[0,2]=(-0.995721191961)*x[0]**o+(-0.587928136153)*x[0]+(-0.378312614832)*x[1]**o+(0.199765674281)*x[1]
        arg[0,3]=(-0.374679425145)*x[0]**o+(-0.503606183146)*x[0]+(0.23995738992)*x[1]**o+(-0.525359181744)*x[1]
        arg[0,4]=(-0.69923255665)*x[0]**o+(0.547634979941)*x[0]+(0.728792702739)*x[1]**o+(0.949888533828)*x[1]
        arg[1,0]=(-0.61856415528)*x[0]**o+(-0.0516239651093)*x[0]+(0.158516649979)*x[1]**o+(-0.0522288881011)*x[1]
        arg[1,1]=(0.0285524133857)*x[0]**o+(0.741357758307)*x[0]+(-0.0714860903847)*x[1]**o+(-0.539632750385)*x[1]
        arg[1,2]=(-0.291101401382)*x[0]**o+(-0.114900246437)*x[0]+(-0.982305031665)*x[1]**o+(0.0823478844055)*x[1]
        arg[1,3]=(-0.931305732525)*x[0]**o+(0.077332436417)*x[0]+(-0.880626154921)*x[1]**o+(-0.321222234507)*x[1]
        arg[1,4]=(-0.00778033208815)*x[0]**o+(-0.536626847957)*x[0]+(-0.599742714546)*x[1]**o+(0.454108299336)*x[1]
        arg[2,0]=(-0.575243090445)*x[0]**o+(-0.920247228564)*x[0]+(0.4569784462)*x[1]**o+(-0.105175798804)*x[1]
        arg[2,1]=(0.333985911748)*x[0]**o+(-0.017876213076)*x[0]+(0.475164778165)*x[1]**o+(0.561072178015)*x[1]
        arg[2,2]=(-0.559587786773)*x[0]**o+(-0.917134128672)*x[0]+(0.316654889327)*x[1]**o+(-0.640569595027)*x[1]
        arg[2,3]=(-0.140461324817)*x[0]**o+(0.259052969883)*x[0]+(0.397898509159)*x[1]**o+(0.932233331785)*x[1]
        arg[2,4]=(0.368626304985)*x[0]**o+(0.616310502384)*x[0]+(-0.756845293566)*x[1]**o+(-0.920088968132)*x[1]
        arg[3,0]=(-0.824398741087)*x[0]**o+(0.0498115894252)*x[0]+(-0.5256621655)*x[1]**o+(0.114806237108)*x[1]
        arg[3,1]=(-0.77671057316)*x[0]**o+(0.569249443162)*x[0]+(0.416628465801)*x[1]**o+(0.0603627487913)*x[1]
        arg[3,2]=(-0.247875220913)*x[0]**o+(0.19548828878)*x[0]+(0.643226101838)*x[1]**o+(-0.154558826499)*x[1]
        arg[3,3]=(-0.918937118963)*x[0]**o+(0.193777174417)*x[0]+(-0.906042347107)*x[1]**o+(0.872033230033)*x[1]
        arg[3,4]=(-0.628918336086)*x[0]**o+(0.823242191714)*x[0]+(-0.593691166712)*x[1]**o+(-0.661715840814)*x[1]
        ref[0,0]=(0.873939846194)/(o+1.)+(0.726906463892)+(-0.263642681371)*0.5**o
        ref[0,1]=(-0.787101008457)/(o+1.)+(0.402698442411)+(-0.389306088628)*0.5**o
        ref[0,2]=(-0.378312614832)/(o+1.)+(-0.194081230936)+(-0.995721191961)*0.5**o
        ref[0,3]=(0.23995738992)/(o+1.)+(-0.514482682445)+(-0.374679425145)*0.5**o
        ref[0,4]=(0.728792702739)/(o+1.)+(0.748761756885)+(-0.69923255665)*0.5**o
        ref[1,0]=(0.158516649979)/(o+1.)+(-0.0519264266052)+(-0.61856415528)*0.5**o
        ref[1,1]=(-0.0714860903847)/(o+1.)+(0.100862503961)+(0.0285524133857)*0.5**o
        ref[1,2]=(-0.982305031665)/(o+1.)+(-0.0162761810159)+(-0.291101401382)*0.5**o
        ref[1,3]=(-0.880626154921)/(o+1.)+(-0.121944899045)+(-0.931305732525)*0.5**o
        ref[1,4]=(-0.599742714546)/(o+1.)+(-0.0412592743102)+(-0.00778033208815)*0.5**o
        ref[2,0]=(0.4569784462)/(o+1.)+(-0.512711513684)+(-0.575243090445)*0.5**o
        ref[2,1]=(0.475164778165)/(o+1.)+(0.27159798247)+(0.333985911748)*0.5**o
        ref[2,2]=(0.316654889327)/(o+1.)+(-0.778851861849)+(-0.559587786773)*0.5**o
        ref[2,3]=(0.397898509159)/(o+1.)+(0.595643150834)+(-0.140461324817)*0.5**o
        ref[2,4]=(-0.756845293566)/(o+1.)+(-0.151889232874)+(0.368626304985)*0.5**o
        ref[3,0]=(-0.5256621655)/(o+1.)+(0.0823089132668)+(-0.824398741087)*0.5**o
        ref[3,1]=(0.416628465801)/(o+1.)+(0.314806095977)+(-0.77671057316)*0.5**o
        ref[3,2]=(0.643226101838)/(o+1.)+(0.0204647311405)+(-0.247875220913)*0.5**o
        ref[3,3]=(-0.906042347107)/(o+1.)+(0.532905202225)+(-0.918937118963)*0.5**o
        ref[3,4]=(-0.593691166712)/(o+1.)+(0.0807631754499)+(-0.628918336086)*0.5**o
      else:
        arg[0,0]=(0.748857269399)*x[0]**o+(-0.21982973469)*x[0]+(-0.765987921295)*x[1]**o+(0.556577019909)*x[1]+(-0.0438548079278)*x[2]**o+(0.543610874817)*x[2]
        arg[0,1]=(-0.784526004748)*x[0]**o+(-0.949423172035)*x[0]+(-0.29888188317)*x[1]**o+(-0.850638766848)*x[1]+(-0.853027558387)*x[2]**o+(-0.846378486588)*x[2]
        arg[0,2]=(0.53445882525)*x[0]**o+(-0.653655652989)*x[0]+(-0.968129875689)*x[1]**o+(-0.603598808323)*x[1]+(0.545841014272)*x[2]**o+(0.126131861082)*x[2]
        arg[0,3]=(0.181662498503)*x[0]**o+(-0.769571080865)*x[0]+(0.771109181078)*x[1]**o+(0.980677373303)*x[1]+(-0.625057940268)*x[2]**o+(0.0585672357346)*x[2]
        arg[0,4]=(-0.0320907540755)*x[0]**o+(-0.519517369917)*x[0]+(-0.365190231996)*x[1]**o+(-0.192392652661)*x[1]+(-0.693267159516)*x[2]**o+(0.106458640425)*x[2]
        arg[1,0]=(0.414286358161)*x[0]**o+(0.0223282279507)*x[0]+(0.215881279469)*x[1]**o+(0.68990977849)*x[1]+(-0.532075885581)*x[2]**o+(-0.269164904232)*x[2]
        arg[1,1]=(-0.0445879250589)*x[0]**o+(0.889985400599)*x[0]+(0.842050582947)*x[1]**o+(-0.0275307884642)*x[1]+(0.774650107671)*x[2]**o+(0.19446514971)*x[2]
        arg[1,2]=(0.913032880034)*x[0]**o+(-0.286537470008)*x[0]+(0.0902778267032)*x[1]**o+(0.548596933564)*x[1]+(-0.996100405074)*x[2]**o+(-0.509554019549)*x[2]
        arg[1,3]=(0.44308812765)*x[0]**o+(-0.0641227726924)*x[0]+(0.484249008075)*x[1]**o+(0.581496384894)*x[1]+(0.592538533183)*x[2]**o+(-0.821638541512)*x[2]
        arg[1,4]=(-0.898304735153)*x[0]**o+(0.333217967121)*x[0]+(-0.788218899371)*x[1]**o+(0.579723459039)*x[1]+(0.467006473159)*x[2]**o+(-0.277113860535)*x[2]
        arg[2,0]=(0.515308716564)*x[0]**o+(0.255947750577)*x[0]+(0.232578337945)*x[1]**o+(0.900611887592)*x[1]+(-0.247710965611)*x[2]**o+(-0.985523228221)*x[2]
        arg[2,1]=(0.611196756618)*x[0]**o+(0.146822999187)*x[0]+(-0.318793147278)*x[1]**o+(0.790772432028)*x[1]+(0.138036152668)*x[2]**o+(-0.800234207543)*x[2]
        arg[2,2]=(-0.293576196546)*x[0]**o+(-0.521059592737)*x[0]+(0.301178293957)*x[1]**o+(-0.141430900198)*x[1]+(-0.782381874467)*x[2]**o+(0.92221179729)*x[2]
        arg[2,3]=(0.673951778483)*x[0]**o+(-0.335068772752)*x[0]+(0.408793200723)*x[1]**o+(0.36906212176)*x[1]+(0.246292093855)*x[2]**o+(0.356147233675)*x[2]
        arg[2,4]=(-0.218899302236)*x[0]**o+(0.271285962886)*x[0]+(-0.16364707283)*x[1]**o+(0.177195607727)*x[1]+(-0.287331396699)*x[2]**o+(-0.465391460497)*x[2]
        arg[3,0]=(-0.369727856735)*x[0]**o+(-0.44351931666)*x[0]+(0.0375097686939)*x[1]**o+(0.276779121402)*x[1]+(0.662505672881)*x[2]**o+(-0.642709866174)*x[2]
        arg[3,1]=(0.444940917816)*x[0]**o+(-0.178926126454)*x[0]+(0.824660645994)*x[1]**o+(0.825996820355)*x[1]+(0.798260651848)*x[2]**o+(0.331744022239)*x[2]
        arg[3,2]=(-0.82489261483)*x[0]**o+(-0.340862079766)*x[0]+(-0.998375587383)*x[1]**o+(0.880222106116)*x[1]+(0.656096580473)*x[2]**o+(0.530854486127)*x[2]
        arg[3,3]=(0.732863411435)*x[0]**o+(-0.324252004147)*x[0]+(0.789472654884)*x[1]**o+(-0.207794796011)*x[1]+(0.50903180567)*x[2]**o+(0.724319112504)*x[2]
        arg[3,4]=(-0.980748139471)*x[0]**o+(0.0974331161906)*x[0]+(0.196275528452)*x[1]**o+(0.305231910003)*x[1]+(-0.0417825350947)*x[2]**o+(-0.2916834261)*x[2]
        ref[0,0]=(-0.809842729223)/(o+1.)+(0.440179080018)+(0.748857269399)*0.5**o
        ref[0,1]=(-1.15190944156)/(o+1.)+(-1.32322021274)+(-0.784526004748)*0.5**o
        ref[0,2]=(-0.422288861417)/(o+1.)+(-0.565561300115)+(0.53445882525)*0.5**o
        ref[0,3]=(0.146051240811)/(o+1.)+(0.134836764087)+(0.181662498503)*0.5**o
        ref[0,4]=(-1.05845739151)/(o+1.)+(-0.302725691077)+(-0.0320907540755)*0.5**o
        ref[1,0]=(-0.316194606113)/(o+1.)+(0.221536551104)+(0.414286358161)*0.5**o
        ref[1,1]=(1.61670069062)/(o+1.)+(0.528459880922)+(-0.0445879250589)*0.5**o
        ref[1,2]=(-0.905822578371)/(o+1.)+(-0.123747277996)+(0.913032880034)*0.5**o
        ref[1,3]=(1.07678754126)/(o+1.)+(-0.152132464655)+(0.44308812765)*0.5**o
        ref[1,4]=(-0.321212426212)/(o+1.)+(0.317913782812)+(-0.898304735153)*0.5**o
        ref[2,0]=(-0.015132627666)/(o+1.)+(0.085518204974)+(0.515308716564)*0.5**o
        ref[2,1]=(-0.18075699461)/(o+1.)+(0.0686806118359)+(0.611196756618)*0.5**o
        ref[2,2]=(-0.481203580509)/(o+1.)+(0.129860652178)+(-0.293576196546)*0.5**o
        ref[2,3]=(0.655085294578)/(o+1.)+(0.195070291341)+(0.673951778483)*0.5**o
        ref[2,4]=(-0.450978469529)/(o+1.)+(-0.00845494494169)+(-0.218899302236)*0.5**o
        ref[3,0]=(0.700015441575)/(o+1.)+(-0.404725030716)+(-0.369727856735)*0.5**o
        ref[3,1]=(1.62292129784)/(o+1.)+(0.48940735807)+(0.444940917816)*0.5**o
        ref[3,2]=(-0.342279006911)/(o+1.)+(0.535107256239)+(-0.82489261483)*0.5**o
        ref[3,3]=(1.29850446055)/(o+1.)+(0.0961361561734)+(0.732863411435)*0.5**o
        ref[3,4]=(0.154492993358)/(o+1.)+(0.055490800047)+(-0.980748139471)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactZero_fromData_ContinuousFunction_rank3(self):
      """
      tests integral of rank 3 Data on the ReducedFunctionOnContactZero

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(6, 2, 2),w)
      ref=numpy.zeros((6, 2, 2),numpy.float_)
      if dim==2:
        arg[0,0,0]=(-0.33788666753)*x[0]**o+(0.0147671449643)*x[0]+(0.14066054662)*x[1]**o+(-0.675231899756)*x[1]
        arg[0,0,1]=(0.0100961441611)*x[0]**o+(0.401304171984)*x[0]+(-0.594786282119)*x[1]**o+(-0.561543495067)*x[1]
        arg[0,1,0]=(-0.118311175162)*x[0]**o+(0.62454992688)*x[0]+(-0.8163725079)*x[1]**o+(-0.756355655277)*x[1]
        arg[0,1,1]=(0.586991709411)*x[0]**o+(-0.600757941586)*x[0]+(0.0953383109011)*x[1]**o+(0.317827629801)*x[1]
        arg[1,0,0]=(-0.251125236294)*x[0]**o+(0.166603616954)*x[0]+(0.978704943828)*x[1]**o+(0.589215389731)*x[1]
        arg[1,0,1]=(0.0543182786738)*x[0]**o+(-0.334756305272)*x[0]+(-0.919571042987)*x[1]**o+(-0.0425734727213)*x[1]
        arg[1,1,0]=(-0.867177621518)*x[0]**o+(-0.0750659593232)*x[0]+(-0.227277029921)*x[1]**o+(0.361120763296)*x[1]
        arg[1,1,1]=(0.85129514889)*x[0]**o+(-0.441962000795)*x[0]+(0.425335668395)*x[1]**o+(0.204091109081)*x[1]
        arg[2,0,0]=(-0.537930401585)*x[0]**o+(-0.206510941636)*x[0]+(-0.247374421196)*x[1]**o+(0.67043841725)*x[1]
        arg[2,0,1]=(-0.101273864474)*x[0]**o+(0.283769727506)*x[0]+(-0.0850059833654)*x[1]**o+(-0.71291661379)*x[1]
        arg[2,1,0]=(0.684430088582)*x[0]**o+(0.107733116086)*x[0]+(0.40398578341)*x[1]**o+(-0.525004991854)*x[1]
        arg[2,1,1]=(-0.00801759014554)*x[0]**o+(-0.214858014851)*x[0]+(-0.427822011518)*x[1]**o+(0.0168962506945)*x[1]
        arg[3,0,0]=(0.528563052182)*x[0]**o+(-0.630485796029)*x[0]+(0.0525765317703)*x[1]**o+(-0.469639006097)*x[1]
        arg[3,0,1]=(0.178622695427)*x[0]**o+(0.0952996500383)*x[0]+(-0.785851578496)*x[1]**o+(0.856665398338)*x[1]
        arg[3,1,0]=(0.547391926072)*x[0]**o+(-0.0179195091852)*x[0]+(-0.831368961862)*x[1]**o+(-0.437229107453)*x[1]
        arg[3,1,1]=(-0.657011566453)*x[0]**o+(0.530176142378)*x[0]+(0.649145894746)*x[1]**o+(-0.755185337598)*x[1]
        arg[4,0,0]=(-0.188305872652)*x[0]**o+(0.0207245930982)*x[0]+(-0.590757998421)*x[1]**o+(-0.793662033952)*x[1]
        arg[4,0,1]=(0.417712824349)*x[0]**o+(-0.138327612527)*x[0]+(-0.381595921766)*x[1]**o+(0.0287368364725)*x[1]
        arg[4,1,0]=(-0.83750540403)*x[0]**o+(0.247261273008)*x[0]+(-0.0769629479034)*x[1]**o+(0.0693501875686)*x[1]
        arg[4,1,1]=(0.697434044074)*x[0]**o+(0.396582403702)*x[0]+(0.736132107034)*x[1]**o+(0.482245183242)*x[1]
        arg[5,0,0]=(0.769936158184)*x[0]**o+(0.808871896459)*x[0]+(-0.559205314785)*x[1]**o+(-0.258014827146)*x[1]
        arg[5,0,1]=(0.667132553969)*x[0]**o+(0.618184492083)*x[0]+(0.475852440157)*x[1]**o+(0.084499293443)*x[1]
        arg[5,1,0]=(0.910882712823)*x[0]**o+(0.153897139268)*x[0]+(-0.61197280039)*x[1]**o+(0.441331985295)*x[1]
        arg[5,1,1]=(-0.290188623857)*x[0]**o+(0.318266320131)*x[0]+(0.9587856247)*x[1]**o+(0.311981486508)*x[1]
        ref[0,0,0]=(0.14066054662)/(o+1.)+(-0.330232377396)+(-0.33788666753)*0.5**o
        ref[0,0,1]=(-0.594786282119)/(o+1.)+(-0.0801196615418)+(0.0100961441611)*0.5**o
        ref[0,1,0]=(-0.8163725079)/(o+1.)+(-0.0659028641985)+(-0.118311175162)*0.5**o
        ref[0,1,1]=(0.0953383109011)/(o+1.)+(-0.141465155893)+(0.586991709411)*0.5**o
        ref[1,0,0]=(0.978704943828)/(o+1.)+(0.377909503343)+(-0.251125236294)*0.5**o
        ref[1,0,1]=(-0.919571042987)/(o+1.)+(-0.188664888997)+(0.0543182786738)*0.5**o
        ref[1,1,0]=(-0.227277029921)/(o+1.)+(0.143027401987)+(-0.867177621518)*0.5**o
        ref[1,1,1]=(0.425335668395)/(o+1.)+(-0.118935445857)+(0.85129514889)*0.5**o
        ref[2,0,0]=(-0.247374421196)/(o+1.)+(0.231963737807)+(-0.537930401585)*0.5**o
        ref[2,0,1]=(-0.0850059833654)/(o+1.)+(-0.214573443142)+(-0.101273864474)*0.5**o
        ref[2,1,0]=(0.40398578341)/(o+1.)+(-0.208635937884)+(0.684430088582)*0.5**o
        ref[2,1,1]=(-0.427822011518)/(o+1.)+(-0.0989808820785)+(-0.00801759014554)*0.5**o
        ref[3,0,0]=(0.0525765317703)/(o+1.)+(-0.550062401063)+(0.528563052182)*0.5**o
        ref[3,0,1]=(-0.785851578496)/(o+1.)+(0.475982524188)+(0.178622695427)*0.5**o
        ref[3,1,0]=(-0.831368961862)/(o+1.)+(-0.227574308319)+(0.547391926072)*0.5**o
        ref[3,1,1]=(0.649145894746)/(o+1.)+(-0.11250459761)+(-0.657011566453)*0.5**o
        ref[4,0,0]=(-0.590757998421)/(o+1.)+(-0.386468720427)+(-0.188305872652)*0.5**o
        ref[4,0,1]=(-0.381595921766)/(o+1.)+(-0.0547953880271)+(0.417712824349)*0.5**o
        ref[4,1,0]=(-0.0769629479034)/(o+1.)+(0.158305730288)+(-0.83750540403)*0.5**o
        ref[4,1,1]=(0.736132107034)/(o+1.)+(0.439413793472)+(0.697434044074)*0.5**o
        ref[5,0,0]=(-0.559205314785)/(o+1.)+(0.275428534657)+(0.769936158184)*0.5**o
        ref[5,0,1]=(0.475852440157)/(o+1.)+(0.351341892763)+(0.667132553969)*0.5**o
        ref[5,1,0]=(-0.61197280039)/(o+1.)+(0.297614562282)+(0.910882712823)*0.5**o
        ref[5,1,1]=(0.9587856247)/(o+1.)+(0.315123903319)+(-0.290188623857)*0.5**o
      else:
        arg[0,0,0]=(0.292346424086)*x[0]**o+(0.105058465241)*x[0]+(-0.281467283372)*x[1]**o+(0.828345267021)*x[1]+(0.741242558638)*x[2]**o+(0.0858780765977)*x[2]
        arg[0,0,1]=(-0.460556250835)*x[0]**o+(-0.845199338719)*x[0]+(-0.547000154821)*x[1]**o+(-0.972236450512)*x[1]+(0.823031420765)*x[2]**o+(0.278285294464)*x[2]
        arg[0,1,0]=(0.380027894487)*x[0]**o+(-0.579359210385)*x[0]+(-0.760849798908)*x[1]**o+(0.0065870459398)*x[1]+(-0.869735580799)*x[2]**o+(-0.937378284335)*x[2]
        arg[0,1,1]=(0.948682597897)*x[0]**o+(-0.882044080711)*x[0]+(-0.694398833899)*x[1]**o+(0.883635235205)*x[1]+(-0.473761903609)*x[2]**o+(0.961453193379)*x[2]
        arg[1,0,0]=(0.357218603696)*x[0]**o+(-0.694504294144)*x[0]+(-0.187119222895)*x[1]**o+(0.727463485088)*x[1]+(0.150410254875)*x[2]**o+(-0.373430202907)*x[2]
        arg[1,0,1]=(-0.776893233002)*x[0]**o+(0.565238637388)*x[0]+(-0.155674628662)*x[1]**o+(-0.938226259914)*x[1]+(0.575183633137)*x[2]**o+(0.319871583783)*x[2]
        arg[1,1,0]=(0.593512911569)*x[0]**o+(-0.0578762085927)*x[0]+(-0.913450847289)*x[1]**o+(0.7074653217)*x[1]+(0.991175636143)*x[2]**o+(-0.535632452281)*x[2]
        arg[1,1,1]=(-0.154542349616)*x[0]**o+(-0.841642050346)*x[0]+(0.878833704431)*x[1]**o+(0.85568524402)*x[1]+(-0.710374336281)*x[2]**o+(0.346540187252)*x[2]
        arg[2,0,0]=(-0.765160864305)*x[0]**o+(0.608017010487)*x[0]+(0.841265469171)*x[1]**o+(-0.409854579584)*x[1]+(0.728931324538)*x[2]**o+(0.121315209213)*x[2]
        arg[2,0,1]=(0.0397067301375)*x[0]**o+(-0.0451925117802)*x[0]+(-0.412724345414)*x[1]**o+(-0.447675002107)*x[1]+(0.141728105561)*x[2]**o+(0.923970189305)*x[2]
        arg[2,1,0]=(-0.237044096923)*x[0]**o+(-0.826168294669)*x[0]+(0.19833520121)*x[1]**o+(-0.796076984382)*x[1]+(0.367399642334)*x[2]**o+(0.71172848801)*x[2]
        arg[2,1,1]=(-0.3881830464)*x[0]**o+(0.988354722657)*x[0]+(0.762756805728)*x[1]**o+(0.551778683619)*x[1]+(0.975932105591)*x[2]**o+(-0.173192098292)*x[2]
        arg[3,0,0]=(-0.226163777371)*x[0]**o+(-0.301094659695)*x[0]+(0.0682567814743)*x[1]**o+(-0.154914902824)*x[1]+(0.984669238436)*x[2]**o+(-0.431011897738)*x[2]
        arg[3,0,1]=(-0.47230847571)*x[0]**o+(-0.579315893476)*x[0]+(-0.989289806396)*x[1]**o+(-0.673951334134)*x[1]+(0.98143767614)*x[2]**o+(-0.311074531383)*x[2]
        arg[3,1,0]=(-0.276309910109)*x[0]**o+(-0.70244304164)*x[0]+(-0.963669800079)*x[1]**o+(-0.947723474952)*x[1]+(0.513451083144)*x[2]**o+(-0.0651803692263)*x[2]
        arg[3,1,1]=(0.65175527234)*x[0]**o+(-0.294569414615)*x[0]+(-0.32846386125)*x[1]**o+(-0.868822318527)*x[1]+(0.268414480892)*x[2]**o+(-0.866835042217)*x[2]
        arg[4,0,0]=(0.304002845434)*x[0]**o+(0.938467246002)*x[0]+(-0.106648262772)*x[1]**o+(0.227863374571)*x[1]+(-0.789305204045)*x[2]**o+(-0.171526142133)*x[2]
        arg[4,0,1]=(0.70982305774)*x[0]**o+(0.404228697442)*x[0]+(0.190029230022)*x[1]**o+(-0.825276083634)*x[1]+(-0.739774636921)*x[2]**o+(-0.726529211281)*x[2]
        arg[4,1,0]=(-0.238487290076)*x[0]**o+(-0.4901234628)*x[0]+(-0.836670757945)*x[1]**o+(0.0841493712661)*x[1]+(-0.685144435198)*x[2]**o+(-0.304978244786)*x[2]
        arg[4,1,1]=(0.264176905985)*x[0]**o+(-0.410582158319)*x[0]+(-0.271915471538)*x[1]**o+(-0.996082427599)*x[1]+(-0.466654446945)*x[2]**o+(-0.723243781429)*x[2]
        arg[5,0,0]=(0.379390860481)*x[0]**o+(-0.78556086955)*x[0]+(0.189126836375)*x[1]**o+(0.235947881164)*x[1]+(0.149142076852)*x[2]**o+(-0.944419890441)*x[2]
        arg[5,0,1]=(0.768867280189)*x[0]**o+(-0.728364777249)*x[0]+(0.60414049451)*x[1]**o+(-0.538876171991)*x[1]+(-0.0947560131508)*x[2]**o+(0.474986163327)*x[2]
        arg[5,1,0]=(-0.228069986734)*x[0]**o+(-0.680841588894)*x[0]+(-0.663806510587)*x[1]**o+(-0.852775021416)*x[1]+(-0.59539582999)*x[2]**o+(0.52369159839)*x[2]
        arg[5,1,1]=(0.177872029425)*x[0]**o+(0.93423742256)*x[0]+(-0.0369475988192)*x[1]**o+(0.242202072343)*x[1]+(0.526144455402)*x[2]**o+(0.0217547142691)*x[2]
        ref[0,0,0]=(0.459775275267)/(o+1.)+(0.50964090443)+(0.292346424086)*0.5**o
        ref[0,0,1]=(0.276031265944)/(o+1.)+(-0.769575247383)+(-0.460556250835)*0.5**o
        ref[0,1,0]=(-1.63058537971)/(o+1.)+(-0.75507522439)+(0.380027894487)*0.5**o
        ref[0,1,1]=(-1.16816073751)/(o+1.)+(0.481522173937)+(0.948682597897)*0.5**o
        ref[1,0,0]=(-0.0367089680209)/(o+1.)+(-0.170235505982)+(0.357218603696)*0.5**o
        ref[1,0,1]=(0.419509004475)/(o+1.)+(-0.0265580193719)+(-0.776893233002)*0.5**o
        ref[1,1,0]=(0.0777247888533)/(o+1.)+(0.0569783304129)+(0.593512911569)*0.5**o
        ref[1,1,1]=(0.168459368149)/(o+1.)+(0.180291690463)+(-0.154542349616)*0.5**o
        ref[2,0,0]=(1.57019679371)/(o+1.)+(0.159738820058)+(-0.765160864305)*0.5**o
        ref[2,0,1]=(-0.270996239853)/(o+1.)+(0.215551337709)+(0.0397067301375)*0.5**o
        ref[2,1,0]=(0.565734843545)/(o+1.)+(-0.455258395521)+(-0.237044096923)*0.5**o
        ref[2,1,1]=(1.73868891132)/(o+1.)+(0.683470653992)+(-0.3881830464)*0.5**o
        ref[3,0,0]=(1.05292601991)/(o+1.)+(-0.443510730129)+(-0.226163777371)*0.5**o
        ref[3,0,1]=(-0.00785213025503)/(o+1.)+(-0.782170879496)+(-0.47230847571)*0.5**o
        ref[3,1,0]=(-0.450218716935)/(o+1.)+(-0.857673442909)+(-0.276309910109)*0.5**o
        ref[3,1,1]=(-0.0600493803575)/(o+1.)+(-1.01511338768)+(0.65175527234)*0.5**o
        ref[4,0,0]=(-0.895953466817)/(o+1.)+(0.49740223922)+(0.304002845434)*0.5**o
        ref[4,0,1]=(-0.549745406899)/(o+1.)+(-0.573788298736)+(0.70982305774)*0.5**o
        ref[4,1,0]=(-1.52181519314)/(o+1.)+(-0.35547616816)+(-0.238487290076)*0.5**o
        ref[4,1,1]=(-0.738569918483)/(o+1.)+(-1.06495418367)+(0.264176905985)*0.5**o
        ref[5,0,0]=(0.338268913228)/(o+1.)+(-0.747016439414)+(0.379390860481)*0.5**o
        ref[5,0,1]=(0.50938448136)/(o+1.)+(-0.396127392956)+(0.768867280189)*0.5**o
        ref[5,1,0]=(-1.25920234058)/(o+1.)+(-0.50496250596)+(-0.228069986734)*0.5**o
        ref[5,1,1]=(0.489196856583)/(o+1.)+(0.599097104586)+(0.177872029425)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactZero_fromData_ContinuousFunction_rank4(self):
      """
      tests integral of rank 4 Data on the ReducedFunctionOnContactZero

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5, 3, 2),w)
      ref=numpy.zeros((4, 5, 3, 2),numpy.float_)
      if dim==2:
        arg[0,0,0,0]=(0.423100205741)*x[0]**o+(-0.681307394941)*x[0]+(-0.848175950825)*x[1]**o+(0.559338235783)*x[1]
        arg[0,0,0,1]=(-0.739687474039)*x[0]**o+(-0.202202358237)*x[0]+(-0.838397517739)*x[1]**o+(-0.377580621185)*x[1]
        arg[0,0,1,0]=(-0.423462958438)*x[0]**o+(-0.335929684088)*x[0]+(-0.616976579504)*x[1]**o+(-0.77803694581)*x[1]
        arg[0,0,1,1]=(-0.432533121428)*x[0]**o+(0.0723865130937)*x[0]+(-0.953014661461)*x[1]**o+(0.92761707252)*x[1]
        arg[0,0,2,0]=(-0.891613775753)*x[0]**o+(0.780592229933)*x[0]+(-0.156191993934)*x[1]**o+(0.985464289737)*x[1]
        arg[0,0,2,1]=(0.909573147888)*x[0]**o+(-0.225603851475)*x[0]+(-0.68287286477)*x[1]**o+(0.330105905794)*x[1]
        arg[0,1,0,0]=(-0.39808363424)*x[0]**o+(-0.290872560992)*x[0]+(-0.631624427826)*x[1]**o+(-0.389239289826)*x[1]
        arg[0,1,0,1]=(-0.748658247224)*x[0]**o+(0.22314629362)*x[0]+(-0.0750972377036)*x[1]**o+(0.908914722147)*x[1]
        arg[0,1,1,0]=(0.877627912148)*x[0]**o+(-0.988434656301)*x[0]+(-0.163646238293)*x[1]**o+(-0.169410711361)*x[1]
        arg[0,1,1,1]=(-0.592580581615)*x[0]**o+(0.997989912544)*x[0]+(0.0667564973731)*x[1]**o+(-0.121580259121)*x[1]
        arg[0,1,2,0]=(-0.505569292541)*x[0]**o+(-0.534428841308)*x[0]+(-0.899276958155)*x[1]**o+(-0.833972415825)*x[1]
        arg[0,1,2,1]=(-0.215752366701)*x[0]**o+(-0.249993262232)*x[0]+(0.912452712698)*x[1]**o+(0.0601524115958)*x[1]
        arg[0,2,0,0]=(0.684112319204)*x[0]**o+(0.343018234122)*x[0]+(0.267409202785)*x[1]**o+(0.7781133929)*x[1]
        arg[0,2,0,1]=(0.552438501301)*x[0]**o+(-0.101558113559)*x[0]+(-0.152472725029)*x[1]**o+(-0.8771654517)*x[1]
        arg[0,2,1,0]=(-0.31213422297)*x[0]**o+(0.597042319279)*x[0]+(-0.873134889407)*x[1]**o+(-0.68920786345)*x[1]
        arg[0,2,1,1]=(0.883048532408)*x[0]**o+(-0.653997880306)*x[0]+(-0.761310589424)*x[1]**o+(-0.698342623962)*x[1]
        arg[0,2,2,0]=(-0.324489237508)*x[0]**o+(-0.717724692716)*x[0]+(-0.936381308418)*x[1]**o+(0.712416971077)*x[1]
        arg[0,2,2,1]=(-0.971735193469)*x[0]**o+(0.0943629094751)*x[0]+(0.419811305063)*x[1]**o+(0.125623316569)*x[1]
        arg[0,3,0,0]=(-0.305553454905)*x[0]**o+(-0.936112994503)*x[0]+(0.707279059071)*x[1]**o+(-0.794754809337)*x[1]
        arg[0,3,0,1]=(0.803934620926)*x[0]**o+(-0.632020033455)*x[0]+(0.819264129099)*x[1]**o+(0.770744069672)*x[1]
        arg[0,3,1,0]=(0.996632326055)*x[0]**o+(-0.537850178404)*x[0]+(-0.60262522048)*x[1]**o+(-0.431667704232)*x[1]
        arg[0,3,1,1]=(0.402799911553)*x[0]**o+(0.155798565063)*x[0]+(0.722837842981)*x[1]**o+(0.462463978227)*x[1]
        arg[0,3,2,0]=(0.937966588808)*x[0]**o+(-0.797151899551)*x[0]+(0.62363567076)*x[1]**o+(-0.60458214868)*x[1]
        arg[0,3,2,1]=(-0.0389713032301)*x[0]**o+(0.497820326557)*x[0]+(-0.950660964237)*x[1]**o+(0.0247372614337)*x[1]
        arg[0,4,0,0]=(-0.589529640529)*x[0]**o+(-0.71761078076)*x[0]+(-0.872148906918)*x[1]**o+(-0.666618835503)*x[1]
        arg[0,4,0,1]=(0.78615039967)*x[0]**o+(0.99287672137)*x[0]+(0.329194160571)*x[1]**o+(0.309370990805)*x[1]
        arg[0,4,1,0]=(-0.801187683466)*x[0]**o+(0.476957978386)*x[0]+(0.107007555422)*x[1]**o+(-0.211909676249)*x[1]
        arg[0,4,1,1]=(0.817551079419)*x[0]**o+(0.958544826763)*x[0]+(-0.720479146591)*x[1]**o+(0.0769824586505)*x[1]
        arg[0,4,2,0]=(-0.245391293539)*x[0]**o+(0.240588876322)*x[0]+(0.766214937295)*x[1]**o+(0.625282150348)*x[1]
        arg[0,4,2,1]=(-0.780163545255)*x[0]**o+(-0.84424180556)*x[0]+(-0.993826355778)*x[1]**o+(-0.708432857746)*x[1]
        arg[1,0,0,0]=(0.0998104654851)*x[0]**o+(-0.933870759023)*x[0]+(-0.00667060999489)*x[1]**o+(-0.304328472241)*x[1]
        arg[1,0,0,1]=(0.716858223574)*x[0]**o+(0.0907083966301)*x[0]+(0.118004968786)*x[1]**o+(-0.989434356769)*x[1]
        arg[1,0,1,0]=(0.433505460082)*x[0]**o+(0.927792393402)*x[0]+(0.10117572965)*x[1]**o+(-0.719175235748)*x[1]
        arg[1,0,1,1]=(0.93394463477)*x[0]**o+(0.30554551436)*x[0]+(-0.221528653572)*x[1]**o+(-0.965153365935)*x[1]
        arg[1,0,2,0]=(0.217926136552)*x[0]**o+(0.75089182057)*x[0]+(0.798438119547)*x[1]**o+(-0.131885152617)*x[1]
        arg[1,0,2,1]=(0.489885370386)*x[0]**o+(-0.958785443386)*x[0]+(-0.934023631551)*x[1]**o+(-0.120727978484)*x[1]
        arg[1,1,0,0]=(-0.104659311537)*x[0]**o+(0.323817119133)*x[0]+(0.207178629335)*x[1]**o+(0.132677138305)*x[1]
        arg[1,1,0,1]=(0.855593354128)*x[0]**o+(-0.319099268106)*x[0]+(-0.75522659926)*x[1]**o+(0.398648791668)*x[1]
        arg[1,1,1,0]=(0.165499959353)*x[0]**o+(0.025416591231)*x[0]+(-0.454711374202)*x[1]**o+(-0.149643847094)*x[1]
        arg[1,1,1,1]=(0.668195333828)*x[0]**o+(-0.954315351102)*x[0]+(-0.15549992542)*x[1]**o+(0.960203842246)*x[1]
        arg[1,1,2,0]=(-0.0830862028828)*x[0]**o+(0.104413343709)*x[0]+(0.0666836881524)*x[1]**o+(-0.909918510114)*x[1]
        arg[1,1,2,1]=(-0.614061685098)*x[0]**o+(-0.696536572273)*x[0]+(0.674867602585)*x[1]**o+(0.742233774001)*x[1]
        arg[1,2,0,0]=(-0.271025278576)*x[0]**o+(0.194146585691)*x[0]+(0.369545867263)*x[1]**o+(-0.444209773003)*x[1]
        arg[1,2,0,1]=(-0.646570016511)*x[0]**o+(-0.1181935343)*x[0]+(-0.473468176784)*x[1]**o+(0.980639666617)*x[1]
        arg[1,2,1,0]=(-0.274558546949)*x[0]**o+(-0.994564294804)*x[0]+(-0.710677848448)*x[1]**o+(-0.471981673909)*x[1]
        arg[1,2,1,1]=(0.920326662139)*x[0]**o+(-0.503477706392)*x[0]+(-0.488075258363)*x[1]**o+(-0.390701293427)*x[1]
        arg[1,2,2,0]=(-0.734779609501)*x[0]**o+(-0.211522964885)*x[0]+(-0.209704158218)*x[1]**o+(0.602143419316)*x[1]
        arg[1,2,2,1]=(-0.71042927509)*x[0]**o+(-0.320801535296)*x[0]+(-0.307645214006)*x[1]**o+(0.902857741355)*x[1]
        arg[1,3,0,0]=(0.430044544461)*x[0]**o+(-0.321809854969)*x[0]+(-0.202705286059)*x[1]**o+(-0.136245903236)*x[1]
        arg[1,3,0,1]=(-0.0215232394148)*x[0]**o+(0.560515199075)*x[0]+(0.183422356152)*x[1]**o+(0.0500950744933)*x[1]
        arg[1,3,1,0]=(0.956072319541)*x[0]**o+(-0.910407504398)*x[0]+(0.710351406673)*x[1]**o+(-0.673149941902)*x[1]
        arg[1,3,1,1]=(0.512105745894)*x[0]**o+(0.617656019287)*x[0]+(0.178878380134)*x[1]**o+(-0.304972517759)*x[1]
        arg[1,3,2,0]=(0.505194196756)*x[0]**o+(-0.588278604691)*x[0]+(0.657312673212)*x[1]**o+(-0.198666173753)*x[1]
        arg[1,3,2,1]=(0.562199962955)*x[0]**o+(-0.855080919048)*x[0]+(0.798106457622)*x[1]**o+(-0.462368594285)*x[1]
        arg[1,4,0,0]=(-0.95103819151)*x[0]**o+(0.996179304819)*x[0]+(0.720809353558)*x[1]**o+(-0.357882371835)*x[1]
        arg[1,4,0,1]=(-0.337212448334)*x[0]**o+(-0.647939092937)*x[0]+(0.432152553594)*x[1]**o+(0.284393585377)*x[1]
        arg[1,4,1,0]=(0.286170290079)*x[0]**o+(0.813168170391)*x[0]+(-0.178814496497)*x[1]**o+(0.414687546663)*x[1]
        arg[1,4,1,1]=(0.00961378883254)*x[0]**o+(-0.946718643529)*x[0]+(0.828780485661)*x[1]**o+(0.54461217692)*x[1]
        arg[1,4,2,0]=(0.784403348421)*x[0]**o+(0.430078584466)*x[0]+(-0.542741131881)*x[1]**o+(0.0290828265624)*x[1]
        arg[1,4,2,1]=(-0.246343667616)*x[0]**o+(-0.27406067089)*x[0]+(-0.938704878102)*x[1]**o+(0.907921504301)*x[1]
        arg[2,0,0,0]=(-0.551056874357)*x[0]**o+(-0.697006980356)*x[0]+(-0.733447125487)*x[1]**o+(-0.241902075189)*x[1]
        arg[2,0,0,1]=(-0.75556701159)*x[0]**o+(0.155303331948)*x[0]+(0.480546881289)*x[1]**o+(-0.420954472057)*x[1]
        arg[2,0,1,0]=(0.12990139506)*x[0]**o+(-0.0990388974263)*x[0]+(-0.913089275462)*x[1]**o+(-0.766316334455)*x[1]
        arg[2,0,1,1]=(-0.96760564527)*x[0]**o+(0.94596820894)*x[0]+(-0.208507053422)*x[1]**o+(0.82642580152)*x[1]
        arg[2,0,2,0]=(-0.801660576339)*x[0]**o+(0.724091208253)*x[0]+(-0.416451086696)*x[1]**o+(-0.255959329575)*x[1]
        arg[2,0,2,1]=(0.380233654687)*x[0]**o+(-0.27467061721)*x[0]+(-0.626861863133)*x[1]**o+(0.498725464891)*x[1]
        arg[2,1,0,0]=(0.733227197081)*x[0]**o+(-0.111622822078)*x[0]+(-0.199363221943)*x[1]**o+(-0.449664634046)*x[1]
        arg[2,1,0,1]=(0.381774304841)*x[0]**o+(0.914208421879)*x[0]+(-0.545958908798)*x[1]**o+(-0.344721917598)*x[1]
        arg[2,1,1,0]=(-0.861412661903)*x[0]**o+(-0.202264578976)*x[0]+(-0.225404313047)*x[1]**o+(0.716189374265)*x[1]
        arg[2,1,1,1]=(-0.180088205315)*x[0]**o+(0.133136657032)*x[0]+(-0.341650807958)*x[1]**o+(-0.570348715256)*x[1]
        arg[2,1,2,0]=(-0.626863150737)*x[0]**o+(-0.236854925766)*x[0]+(-0.271186155613)*x[1]**o+(-0.750500826476)*x[1]
        arg[2,1,2,1]=(-0.138580929054)*x[0]**o+(-0.925057205334)*x[0]+(0.404961686669)*x[1]**o+(0.542298342705)*x[1]
        arg[2,2,0,0]=(0.453186536783)*x[0]**o+(0.368065177204)*x[0]+(0.545211472015)*x[1]**o+(0.334551805933)*x[1]
        arg[2,2,0,1]=(0.0639699038803)*x[0]**o+(0.154192875522)*x[0]+(0.101347900441)*x[1]**o+(-0.668313576443)*x[1]
        arg[2,2,1,0]=(0.465173445732)*x[0]**o+(-0.475009875122)*x[0]+(-0.748444748361)*x[1]**o+(0.620845514745)*x[1]
        arg[2,2,1,1]=(0.720975965615)*x[0]**o+(-0.266631032721)*x[0]+(0.0996389916136)*x[1]**o+(-0.96886977517)*x[1]
        arg[2,2,2,0]=(-0.396391719953)*x[0]**o+(-0.240091626001)*x[0]+(-0.42411685734)*x[1]**o+(-0.458446010754)*x[1]
        arg[2,2,2,1]=(-0.662584035264)*x[0]**o+(0.23813614536)*x[0]+(0.810893160816)*x[1]**o+(-0.130143663026)*x[1]
        arg[2,3,0,0]=(-0.651550480594)*x[0]**o+(0.370844612999)*x[0]+(-0.808002027275)*x[1]**o+(0.531083477622)*x[1]
        arg[2,3,0,1]=(-0.287184182382)*x[0]**o+(0.928877646221)*x[0]+(-0.271563784371)*x[1]**o+(-0.46908309398)*x[1]
        arg[2,3,1,0]=(0.342080138259)*x[0]**o+(0.458994887529)*x[0]+(-0.544475387926)*x[1]**o+(-0.203224249914)*x[1]
        arg[2,3,1,1]=(-0.0791009979206)*x[0]**o+(0.536480501773)*x[0]+(-0.624008015173)*x[1]**o+(-0.380504135174)*x[1]
        arg[2,3,2,0]=(-0.707089360149)*x[0]**o+(0.217716900812)*x[0]+(0.020441009853)*x[1]**o+(0.680214150153)*x[1]
        arg[2,3,2,1]=(0.0368635610611)*x[0]**o+(0.646697307691)*x[0]+(0.36118713723)*x[1]**o+(-0.535295583008)*x[1]
        arg[2,4,0,0]=(0.806938680905)*x[0]**o+(0.449096825134)*x[0]+(-0.43507826254)*x[1]**o+(-0.300956286252)*x[1]
        arg[2,4,0,1]=(-0.268005914326)*x[0]**o+(0.583132947291)*x[0]+(-0.344067495236)*x[1]**o+(-0.526011968282)*x[1]
        arg[2,4,1,0]=(-0.0953663458348)*x[0]**o+(0.834765967023)*x[0]+(-0.0410345097335)*x[1]**o+(0.493198054713)*x[1]
        arg[2,4,1,1]=(0.964777812988)*x[0]**o+(-0.791397009733)*x[0]+(-0.962108127047)*x[1]**o+(0.378083818799)*x[1]
        arg[2,4,2,0]=(0.0977715857241)*x[0]**o+(-0.118265910205)*x[0]+(-0.0181736456148)*x[1]**o+(-0.480499109439)*x[1]
        arg[2,4,2,1]=(0.396936262802)*x[0]**o+(0.979807961416)*x[0]+(0.383251286227)*x[1]**o+(-0.921599329372)*x[1]
        arg[3,0,0,0]=(-0.193869568904)*x[0]**o+(0.335533150532)*x[0]+(-0.542087879921)*x[1]**o+(0.916531699685)*x[1]
        arg[3,0,0,1]=(0.732192438113)*x[0]**o+(0.610866210623)*x[0]+(0.232455584069)*x[1]**o+(0.109577342902)*x[1]
        arg[3,0,1,0]=(0.0487640958479)*x[0]**o+(-0.250229114846)*x[0]+(-0.539343546695)*x[1]**o+(-0.456485153733)*x[1]
        arg[3,0,1,1]=(0.951844805407)*x[0]**o+(-0.811442383281)*x[0]+(-0.537581471096)*x[1]**o+(0.350217063609)*x[1]
        arg[3,0,2,0]=(0.495490307938)*x[0]**o+(0.529667417659)*x[0]+(-0.853447043865)*x[1]**o+(0.97002519322)*x[1]
        arg[3,0,2,1]=(0.38188303469)*x[0]**o+(-0.75911672537)*x[0]+(-0.67563531103)*x[1]**o+(0.384002769031)*x[1]
        arg[3,1,0,0]=(0.44471228922)*x[0]**o+(-0.958995287488)*x[0]+(-0.356730659314)*x[1]**o+(0.853749081247)*x[1]
        arg[3,1,0,1]=(-0.674677862576)*x[0]**o+(-0.651945231545)*x[0]+(-0.869871613483)*x[1]**o+(0.518493292422)*x[1]
        arg[3,1,1,0]=(-0.803923033456)*x[0]**o+(0.916638584514)*x[0]+(-0.410056071233)*x[1]**o+(-0.313811798732)*x[1]
        arg[3,1,1,1]=(-0.677586574312)*x[0]**o+(0.101317771211)*x[0]+(-0.848629733526)*x[1]**o+(0.956935407827)*x[1]
        arg[3,1,2,0]=(0.723956969878)*x[0]**o+(-0.677929268987)*x[0]+(0.168694427833)*x[1]**o+(0.675664564011)*x[1]
        arg[3,1,2,1]=(-0.0961257584415)*x[0]**o+(0.870273004118)*x[0]+(0.0979731186867)*x[1]**o+(-0.561786099655)*x[1]
        arg[3,2,0,0]=(-0.506857925404)*x[0]**o+(0.950429186893)*x[0]+(-0.765686972231)*x[1]**o+(0.300787357163)*x[1]
        arg[3,2,0,1]=(-0.479464705052)*x[0]**o+(-0.814881390688)*x[0]+(-0.549033177495)*x[1]**o+(0.875164713771)*x[1]
        arg[3,2,1,0]=(0.984579775952)*x[0]**o+(0.822464384935)*x[0]+(-0.352937841183)*x[1]**o+(-0.389516730371)*x[1]
        arg[3,2,1,1]=(-0.75230207503)*x[0]**o+(0.921759892886)*x[0]+(0.683120790039)*x[1]**o+(0.470250085193)*x[1]
        arg[3,2,2,0]=(-0.257935272452)*x[0]**o+(-0.437370511818)*x[0]+(-0.350363036315)*x[1]**o+(-0.232415372202)*x[1]
        arg[3,2,2,1]=(0.0784702766009)*x[0]**o+(0.460959683819)*x[0]+(0.758201028024)*x[1]**o+(0.101597092257)*x[1]
        arg[3,3,0,0]=(0.292597181224)*x[0]**o+(-0.322992033991)*x[0]+(-0.536413055541)*x[1]**o+(-0.95609054732)*x[1]
        arg[3,3,0,1]=(0.174134715952)*x[0]**o+(0.811729326606)*x[0]+(0.89040777319)*x[1]**o+(0.893572419966)*x[1]
        arg[3,3,1,0]=(0.768234215847)*x[0]**o+(0.36391719033)*x[0]+(0.598889528719)*x[1]**o+(0.152035960251)*x[1]
        arg[3,3,1,1]=(-0.77938361663)*x[0]**o+(-0.448945619013)*x[0]+(0.471242869055)*x[1]**o+(0.112888214554)*x[1]
        arg[3,3,2,0]=(-0.00617753872016)*x[0]**o+(0.0155218100192)*x[0]+(0.674100983537)*x[1]**o+(0.211406831421)*x[1]
        arg[3,3,2,1]=(0.124470359441)*x[0]**o+(0.117393617733)*x[0]+(-0.297791856337)*x[1]**o+(-0.210405798241)*x[1]
        arg[3,4,0,0]=(0.139920132608)*x[0]**o+(-0.0073381380034)*x[0]+(-0.77267392403)*x[1]**o+(0.792299687107)*x[1]
        arg[3,4,0,1]=(0.328592839797)*x[0]**o+(-0.758899359217)*x[0]+(0.730856047416)*x[1]**o+(-0.389548922357)*x[1]
        arg[3,4,1,0]=(-0.239362896774)*x[0]**o+(0.513205289336)*x[0]+(-0.393781550386)*x[1]**o+(0.229968513732)*x[1]
        arg[3,4,1,1]=(0.777985150139)*x[0]**o+(0.273278793895)*x[0]+(0.17782134987)*x[1]**o+(0.491127331923)*x[1]
        arg[3,4,2,0]=(0.348482781008)*x[0]**o+(0.458696185739)*x[0]+(-0.77882422986)*x[1]**o+(0.455739446004)*x[1]
        arg[3,4,2,1]=(-0.396751338134)*x[0]**o+(0.449026376364)*x[0]+(0.308553967581)*x[1]**o+(-0.777653415416)*x[1]
        ref[0,0,0,0]=(-0.848175950825)/(o+1.)+(-0.060984579579)+(0.423100205741)*0.5**o
        ref[0,0,0,1]=(-0.838397517739)/(o+1.)+(-0.289891489711)+(-0.739687474039)*0.5**o
        ref[0,0,1,0]=(-0.616976579504)/(o+1.)+(-0.556983314949)+(-0.423462958438)*0.5**o
        ref[0,0,1,1]=(-0.953014661461)/(o+1.)+(0.500001792807)+(-0.432533121428)*0.5**o
        ref[0,0,2,0]=(-0.156191993934)/(o+1.)+(0.883028259835)+(-0.891613775753)*0.5**o
        ref[0,0,2,1]=(-0.68287286477)/(o+1.)+(0.0522510271596)+(0.909573147888)*0.5**o
        ref[0,1,0,0]=(-0.631624427826)/(o+1.)+(-0.340055925409)+(-0.39808363424)*0.5**o
        ref[0,1,0,1]=(-0.0750972377036)/(o+1.)+(0.566030507884)+(-0.748658247224)*0.5**o
        ref[0,1,1,0]=(-0.163646238293)/(o+1.)+(-0.578922683831)+(0.877627912148)*0.5**o
        ref[0,1,1,1]=(0.0667564973731)/(o+1.)+(0.438204826712)+(-0.592580581615)*0.5**o
        ref[0,1,2,0]=(-0.899276958155)/(o+1.)+(-0.684200628566)+(-0.505569292541)*0.5**o
        ref[0,1,2,1]=(0.912452712698)/(o+1.)+(-0.0949204253183)+(-0.215752366701)*0.5**o
        ref[0,2,0,0]=(0.267409202785)/(o+1.)+(0.560565813511)+(0.684112319204)*0.5**o
        ref[0,2,0,1]=(-0.152472725029)/(o+1.)+(-0.48936178263)+(0.552438501301)*0.5**o
        ref[0,2,1,0]=(-0.873134889407)/(o+1.)+(-0.0460827720855)+(-0.31213422297)*0.5**o
        ref[0,2,1,1]=(-0.761310589424)/(o+1.)+(-0.676170252134)+(0.883048532408)*0.5**o
        ref[0,2,2,0]=(-0.936381308418)/(o+1.)+(-0.00265386081956)+(-0.324489237508)*0.5**o
        ref[0,2,2,1]=(0.419811305063)/(o+1.)+(0.109993113022)+(-0.971735193469)*0.5**o
        ref[0,3,0,0]=(0.707279059071)/(o+1.)+(-0.86543390192)+(-0.305553454905)*0.5**o
        ref[0,3,0,1]=(0.819264129099)/(o+1.)+(0.0693620181083)+(0.803934620926)*0.5**o
        ref[0,3,1,0]=(-0.60262522048)/(o+1.)+(-0.484758941318)+(0.996632326055)*0.5**o
        ref[0,3,1,1]=(0.722837842981)/(o+1.)+(0.309131271645)+(0.402799911553)*0.5**o
        ref[0,3,2,0]=(0.62363567076)/(o+1.)+(-0.700867024115)+(0.937966588808)*0.5**o
        ref[0,3,2,1]=(-0.950660964237)/(o+1.)+(0.261278793995)+(-0.0389713032301)*0.5**o
        ref[0,4,0,0]=(-0.872148906918)/(o+1.)+(-0.692114808131)+(-0.589529640529)*0.5**o
        ref[0,4,0,1]=(0.329194160571)/(o+1.)+(0.651123856087)+(0.78615039967)*0.5**o
        ref[0,4,1,0]=(0.107007555422)/(o+1.)+(0.132524151069)+(-0.801187683466)*0.5**o
        ref[0,4,1,1]=(-0.720479146591)/(o+1.)+(0.517763642707)+(0.817551079419)*0.5**o
        ref[0,4,2,0]=(0.766214937295)/(o+1.)+(0.432935513335)+(-0.245391293539)*0.5**o
        ref[0,4,2,1]=(-0.993826355778)/(o+1.)+(-0.776337331653)+(-0.780163545255)*0.5**o
        ref[1,0,0,0]=(-0.00667060999489)/(o+1.)+(-0.619099615632)+(0.0998104654851)*0.5**o
        ref[1,0,0,1]=(0.118004968786)/(o+1.)+(-0.449362980069)+(0.716858223574)*0.5**o
        ref[1,0,1,0]=(0.10117572965)/(o+1.)+(0.104308578827)+(0.433505460082)*0.5**o
        ref[1,0,1,1]=(-0.221528653572)/(o+1.)+(-0.329803925788)+(0.93394463477)*0.5**o
        ref[1,0,2,0]=(0.798438119547)/(o+1.)+(0.309503333977)+(0.217926136552)*0.5**o
        ref[1,0,2,1]=(-0.934023631551)/(o+1.)+(-0.539756710935)+(0.489885370386)*0.5**o
        ref[1,1,0,0]=(0.207178629335)/(o+1.)+(0.228247128719)+(-0.104659311537)*0.5**o
        ref[1,1,0,1]=(-0.75522659926)/(o+1.)+(0.0397747617809)+(0.855593354128)*0.5**o
        ref[1,1,1,0]=(-0.454711374202)/(o+1.)+(-0.0621136279316)+(0.165499959353)*0.5**o
        ref[1,1,1,1]=(-0.15549992542)/(o+1.)+(0.00294424557217)+(0.668195333828)*0.5**o
        ref[1,1,2,0]=(0.0666836881524)/(o+1.)+(-0.402752583203)+(-0.0830862028828)*0.5**o
        ref[1,1,2,1]=(0.674867602585)/(o+1.)+(0.0228486008642)+(-0.614061685098)*0.5**o
        ref[1,2,0,0]=(0.369545867263)/(o+1.)+(-0.125031593656)+(-0.271025278576)*0.5**o
        ref[1,2,0,1]=(-0.473468176784)/(o+1.)+(0.431223066159)+(-0.646570016511)*0.5**o
        ref[1,2,1,0]=(-0.710677848448)/(o+1.)+(-0.733272984356)+(-0.274558546949)*0.5**o
        ref[1,2,1,1]=(-0.488075258363)/(o+1.)+(-0.44708949991)+(0.920326662139)*0.5**o
        ref[1,2,2,0]=(-0.209704158218)/(o+1.)+(0.195310227215)+(-0.734779609501)*0.5**o
        ref[1,2,2,1]=(-0.307645214006)/(o+1.)+(0.29102810303)+(-0.71042927509)*0.5**o
        ref[1,3,0,0]=(-0.202705286059)/(o+1.)+(-0.229027879102)+(0.430044544461)*0.5**o
        ref[1,3,0,1]=(0.183422356152)/(o+1.)+(0.305305136784)+(-0.0215232394148)*0.5**o
        ref[1,3,1,0]=(0.710351406673)/(o+1.)+(-0.79177872315)+(0.956072319541)*0.5**o
        ref[1,3,1,1]=(0.178878380134)/(o+1.)+(0.156341750764)+(0.512105745894)*0.5**o
        ref[1,3,2,0]=(0.657312673212)/(o+1.)+(-0.393472389222)+(0.505194196756)*0.5**o
        ref[1,3,2,1]=(0.798106457622)/(o+1.)+(-0.658724756667)+(0.562199962955)*0.5**o
        ref[1,4,0,0]=(0.720809353558)/(o+1.)+(0.319148466492)+(-0.95103819151)*0.5**o
        ref[1,4,0,1]=(0.432152553594)/(o+1.)+(-0.18177275378)+(-0.337212448334)*0.5**o
        ref[1,4,1,0]=(-0.178814496497)/(o+1.)+(0.613927858527)+(0.286170290079)*0.5**o
        ref[1,4,1,1]=(0.828780485661)/(o+1.)+(-0.201053233304)+(0.00961378883254)*0.5**o
        ref[1,4,2,0]=(-0.542741131881)/(o+1.)+(0.229580705514)+(0.784403348421)*0.5**o
        ref[1,4,2,1]=(-0.938704878102)/(o+1.)+(0.316930416706)+(-0.246343667616)*0.5**o
        ref[2,0,0,0]=(-0.733447125487)/(o+1.)+(-0.469454527772)+(-0.551056874357)*0.5**o
        ref[2,0,0,1]=(0.480546881289)/(o+1.)+(-0.132825570055)+(-0.75556701159)*0.5**o
        ref[2,0,1,0]=(-0.913089275462)/(o+1.)+(-0.432677615941)+(0.12990139506)*0.5**o
        ref[2,0,1,1]=(-0.208507053422)/(o+1.)+(0.88619700523)+(-0.96760564527)*0.5**o
        ref[2,0,2,0]=(-0.416451086696)/(o+1.)+(0.234065939339)+(-0.801660576339)*0.5**o
        ref[2,0,2,1]=(-0.626861863133)/(o+1.)+(0.112027423841)+(0.380233654687)*0.5**o
        ref[2,1,0,0]=(-0.199363221943)/(o+1.)+(-0.280643728062)+(0.733227197081)*0.5**o
        ref[2,1,0,1]=(-0.545958908798)/(o+1.)+(0.284743252141)+(0.381774304841)*0.5**o
        ref[2,1,1,0]=(-0.225404313047)/(o+1.)+(0.256962397644)+(-0.861412661903)*0.5**o
        ref[2,1,1,1]=(-0.341650807958)/(o+1.)+(-0.218606029112)+(-0.180088205315)*0.5**o
        ref[2,1,2,0]=(-0.271186155613)/(o+1.)+(-0.493677876121)+(-0.626863150737)*0.5**o
        ref[2,1,2,1]=(0.404961686669)/(o+1.)+(-0.191379431315)+(-0.138580929054)*0.5**o
        ref[2,2,0,0]=(0.545211472015)/(o+1.)+(0.351308491569)+(0.453186536783)*0.5**o
        ref[2,2,0,1]=(0.101347900441)/(o+1.)+(-0.257060350461)+(0.0639699038803)*0.5**o
        ref[2,2,1,0]=(-0.748444748361)/(o+1.)+(0.0729178198114)+(0.465173445732)*0.5**o
        ref[2,2,1,1]=(0.0996389916136)/(o+1.)+(-0.617750403946)+(0.720975965615)*0.5**o
        ref[2,2,2,0]=(-0.42411685734)/(o+1.)+(-0.349268818378)+(-0.396391719953)*0.5**o
        ref[2,2,2,1]=(0.810893160816)/(o+1.)+(0.053996241167)+(-0.662584035264)*0.5**o
        ref[2,3,0,0]=(-0.808002027275)/(o+1.)+(0.45096404531)+(-0.651550480594)*0.5**o
        ref[2,3,0,1]=(-0.271563784371)/(o+1.)+(0.22989727612)+(-0.287184182382)*0.5**o
        ref[2,3,1,0]=(-0.544475387926)/(o+1.)+(0.127885318807)+(0.342080138259)*0.5**o
        ref[2,3,1,1]=(-0.624008015173)/(o+1.)+(0.0779881832997)+(-0.0791009979206)*0.5**o
        ref[2,3,2,0]=(0.020441009853)/(o+1.)+(0.448965525483)+(-0.707089360149)*0.5**o
        ref[2,3,2,1]=(0.36118713723)/(o+1.)+(0.0557008623413)+(0.0368635610611)*0.5**o
        ref[2,4,0,0]=(-0.43507826254)/(o+1.)+(0.0740702694408)+(0.806938680905)*0.5**o
        ref[2,4,0,1]=(-0.344067495236)/(o+1.)+(0.0285604895043)+(-0.268005914326)*0.5**o
        ref[2,4,1,0]=(-0.0410345097335)/(o+1.)+(0.663982010868)+(-0.0953663458348)*0.5**o
        ref[2,4,1,1]=(-0.962108127047)/(o+1.)+(-0.206656595467)+(0.964777812988)*0.5**o
        ref[2,4,2,0]=(-0.0181736456148)/(o+1.)+(-0.299382509822)+(0.0977715857241)*0.5**o
        ref[2,4,2,1]=(0.383251286227)/(o+1.)+(0.0291043160221)+(0.396936262802)*0.5**o
        ref[3,0,0,0]=(-0.542087879921)/(o+1.)+(0.626032425109)+(-0.193869568904)*0.5**o
        ref[3,0,0,1]=(0.232455584069)/(o+1.)+(0.360221776762)+(0.732192438113)*0.5**o
        ref[3,0,1,0]=(-0.539343546695)/(o+1.)+(-0.353357134289)+(0.0487640958479)*0.5**o
        ref[3,0,1,1]=(-0.537581471096)/(o+1.)+(-0.230612659836)+(0.951844805407)*0.5**o
        ref[3,0,2,0]=(-0.853447043865)/(o+1.)+(0.74984630544)+(0.495490307938)*0.5**o
        ref[3,0,2,1]=(-0.67563531103)/(o+1.)+(-0.187556978169)+(0.38188303469)*0.5**o
        ref[3,1,0,0]=(-0.356730659314)/(o+1.)+(-0.0526231031206)+(0.44471228922)*0.5**o
        ref[3,1,0,1]=(-0.869871613483)/(o+1.)+(-0.0667259695618)+(-0.674677862576)*0.5**o
        ref[3,1,1,0]=(-0.410056071233)/(o+1.)+(0.301413392891)+(-0.803923033456)*0.5**o
        ref[3,1,1,1]=(-0.848629733526)/(o+1.)+(0.529126589519)+(-0.677586574312)*0.5**o
        ref[3,1,2,0]=(0.168694427833)/(o+1.)+(-0.00113235248793)+(0.723956969878)*0.5**o
        ref[3,1,2,1]=(0.0979731186867)/(o+1.)+(0.154243452232)+(-0.0961257584415)*0.5**o
        ref[3,2,0,0]=(-0.765686972231)/(o+1.)+(0.625608272028)+(-0.506857925404)*0.5**o
        ref[3,2,0,1]=(-0.549033177495)/(o+1.)+(0.0301416615418)+(-0.479464705052)*0.5**o
        ref[3,2,1,0]=(-0.352937841183)/(o+1.)+(0.216473827282)+(0.984579775952)*0.5**o
        ref[3,2,1,1]=(0.683120790039)/(o+1.)+(0.696004989039)+(-0.75230207503)*0.5**o
        ref[3,2,2,0]=(-0.350363036315)/(o+1.)+(-0.33489294201)+(-0.257935272452)*0.5**o
        ref[3,2,2,1]=(0.758201028024)/(o+1.)+(0.281278388038)+(0.0784702766009)*0.5**o
        ref[3,3,0,0]=(-0.536413055541)/(o+1.)+(-0.639541290655)+(0.292597181224)*0.5**o
        ref[3,3,0,1]=(0.89040777319)/(o+1.)+(0.852650873286)+(0.174134715952)*0.5**o
        ref[3,3,1,0]=(0.598889528719)/(o+1.)+(0.257976575291)+(0.768234215847)*0.5**o
        ref[3,3,1,1]=(0.471242869055)/(o+1.)+(-0.168028702229)+(-0.77938361663)*0.5**o
        ref[3,3,2,0]=(0.674100983537)/(o+1.)+(0.11346432072)+(-0.00617753872016)*0.5**o
        ref[3,3,2,1]=(-0.297791856337)/(o+1.)+(-0.0465060902539)+(0.124470359441)*0.5**o
        ref[3,4,0,0]=(-0.77267392403)/(o+1.)+(0.392480774552)+(0.139920132608)*0.5**o
        ref[3,4,0,1]=(0.730856047416)/(o+1.)+(-0.574224140787)+(0.328592839797)*0.5**o
        ref[3,4,1,0]=(-0.393781550386)/(o+1.)+(0.371586901534)+(-0.239362896774)*0.5**o
        ref[3,4,1,1]=(0.17782134987)/(o+1.)+(0.382203062909)+(0.777985150139)*0.5**o
        ref[3,4,2,0]=(-0.77882422986)/(o+1.)+(0.457217815872)+(0.348482781008)*0.5**o
        ref[3,4,2,1]=(0.308553967581)/(o+1.)+(-0.164313519526)+(-0.396751338134)*0.5**o
      else:
        arg[0,0,0,0]=(0.663998508168)*x[0]**o+(0.797340164931)*x[0]+(0.285182001599)*x[1]**o+(0.99720606542)*x[1]+(0.127610260108)*x[2]**o+(0.0141819362674)*x[2]
        arg[0,0,0,1]=(-0.010130464476)*x[0]**o+(-0.694140348803)*x[0]+(-0.554528445238)*x[1]**o+(0.192639775817)*x[1]+(-0.520164891934)*x[2]**o+(0.316830081989)*x[2]
        arg[0,0,1,0]=(-0.176542289385)*x[0]**o+(-0.312451025451)*x[0]+(0.33951668081)*x[1]**o+(-0.66674551066)*x[1]+(-0.0515929546862)*x[2]**o+(0.688180995863)*x[2]
        arg[0,0,1,1]=(-0.98152572229)*x[0]**o+(0.156411605196)*x[0]+(0.588065240125)*x[1]**o+(0.665830732172)*x[1]+(-0.834625744106)*x[2]**o+(-0.138276534671)*x[2]
        arg[0,0,2,0]=(0.555897422173)*x[0]**o+(0.0269431977874)*x[0]+(0.671161746561)*x[1]**o+(0.0857445285374)*x[1]+(0.578027911522)*x[2]**o+(0.217805007393)*x[2]
        arg[0,0,2,1]=(0.23127040923)*x[0]**o+(0.624353504617)*x[0]+(-0.433915231286)*x[1]**o+(-0.860173971996)*x[1]+(-0.801472434932)*x[2]**o+(-0.779575445432)*x[2]
        arg[0,1,0,0]=(0.0202445177842)*x[0]**o+(-0.0654289514039)*x[0]+(-0.877581892208)*x[1]**o+(-0.231833216639)*x[1]+(0.609752401432)*x[2]**o+(-0.802673983603)*x[2]
        arg[0,1,0,1]=(0.99935098551)*x[0]**o+(0.665639246142)*x[0]+(0.593510187831)*x[1]**o+(0.213336090684)*x[1]+(0.232667772738)*x[2]**o+(-0.651386262633)*x[2]
        arg[0,1,1,0]=(0.151799117419)*x[0]**o+(-0.34631043288)*x[0]+(-0.264898731671)*x[1]**o+(0.697527693137)*x[1]+(0.389030412719)*x[2]**o+(0.613338381577)*x[2]
        arg[0,1,1,1]=(-0.51422532626)*x[0]**o+(0.228040429268)*x[0]+(-0.276760984751)*x[1]**o+(0.379484010198)*x[1]+(0.896837565192)*x[2]**o+(0.195866552401)*x[2]
        arg[0,1,2,0]=(0.908244888839)*x[0]**o+(0.217098236162)*x[0]+(0.798725721675)*x[1]**o+(0.216635630826)*x[1]+(0.960227216319)*x[2]**o+(-0.885381182879)*x[2]
        arg[0,1,2,1]=(-0.872307766634)*x[0]**o+(0.843197737136)*x[0]+(-0.895220198791)*x[1]**o+(0.866433237518)*x[1]+(0.560344834226)*x[2]**o+(0.895797915384)*x[2]
        arg[0,2,0,0]=(0.00173619592035)*x[0]**o+(-0.209651858019)*x[0]+(0.323249367733)*x[1]**o+(-0.848445788048)*x[1]+(-0.747443572813)*x[2]**o+(0.342929929066)*x[2]
        arg[0,2,0,1]=(0.998184336552)*x[0]**o+(-0.669882578313)*x[0]+(-0.632794054696)*x[1]**o+(-0.779832061924)*x[1]+(0.0588815237456)*x[2]**o+(0.419538634163)*x[2]
        arg[0,2,1,0]=(-0.921234331968)*x[0]**o+(0.319724478943)*x[0]+(0.867192523265)*x[1]**o+(-0.183047755109)*x[1]+(-0.753047661507)*x[2]**o+(-0.797179040607)*x[2]
        arg[0,2,1,1]=(0.153578015941)*x[0]**o+(0.887820807812)*x[0]+(-0.607944647299)*x[1]**o+(-0.0487413965527)*x[1]+(-0.812369692315)*x[2]**o+(0.323102876844)*x[2]
        arg[0,2,2,0]=(0.29601674794)*x[0]**o+(-0.108738759628)*x[0]+(0.242837345895)*x[1]**o+(-0.324487554664)*x[1]+(0.584399369463)*x[2]**o+(0.683877139789)*x[2]
        arg[0,2,2,1]=(0.0580007647099)*x[0]**o+(0.214488016057)*x[0]+(0.669087265419)*x[1]**o+(-0.311999860513)*x[1]+(0.678940598195)*x[2]**o+(0.658847203441)*x[2]
        arg[0,3,0,0]=(-0.337999890505)*x[0]**o+(-0.842479530409)*x[0]+(0.786376708623)*x[1]**o+(0.597298234359)*x[1]+(0.495442668997)*x[2]**o+(-0.183828960943)*x[2]
        arg[0,3,0,1]=(-0.825813329629)*x[0]**o+(0.197040670392)*x[0]+(0.997784261354)*x[1]**o+(-0.355290279278)*x[1]+(0.657909516664)*x[2]**o+(-0.568281670001)*x[2]
        arg[0,3,1,0]=(0.699313512079)*x[0]**o+(-0.916350147519)*x[0]+(-0.311605808179)*x[1]**o+(0.222618888987)*x[1]+(0.872150798976)*x[2]**o+(0.139773150125)*x[2]
        arg[0,3,1,1]=(0.140764124286)*x[0]**o+(0.994013777309)*x[0]+(-0.0792107151558)*x[1]**o+(-0.285357856751)*x[1]+(-0.800285107223)*x[2]**o+(0.413029089978)*x[2]
        arg[0,3,2,0]=(-0.68177752688)*x[0]**o+(0.0339522871939)*x[0]+(-0.641399841911)*x[1]**o+(0.263279621719)*x[1]+(-0.406115766497)*x[2]**o+(-0.774639092638)*x[2]
        arg[0,3,2,1]=(0.655685968811)*x[0]**o+(0.20085918281)*x[0]+(-0.753491228829)*x[1]**o+(0.393669658887)*x[1]+(0.339469131428)*x[2]**o+(0.424803724709)*x[2]
        arg[0,4,0,0]=(-0.18027010068)*x[0]**o+(-0.773007683353)*x[0]+(0.767133020895)*x[1]**o+(-0.810449138606)*x[1]+(0.295421955604)*x[2]**o+(-0.955926880008)*x[2]
        arg[0,4,0,1]=(-0.914768182433)*x[0]**o+(-0.67024319498)*x[0]+(0.533244666277)*x[1]**o+(-0.705719628704)*x[1]+(-0.963633923897)*x[2]**o+(-0.0818895784023)*x[2]
        arg[0,4,1,0]=(0.274727423835)*x[0]**o+(0.838218676826)*x[0]+(0.269989900726)*x[1]**o+(0.650600179293)*x[1]+(-0.855210211935)*x[2]**o+(-0.800261683904)*x[2]
        arg[0,4,1,1]=(0.257574026756)*x[0]**o+(-0.787643878696)*x[0]+(-0.619136916008)*x[1]**o+(-0.242580269843)*x[1]+(0.0229959873555)*x[2]**o+(-0.685725009107)*x[2]
        arg[0,4,2,0]=(0.703185665611)*x[0]**o+(0.37686548395)*x[0]+(-0.414901304236)*x[1]**o+(0.310335568741)*x[1]+(-0.740764649572)*x[2]**o+(-0.389356987349)*x[2]
        arg[0,4,2,1]=(-0.153792811673)*x[0]**o+(-0.179777301012)*x[0]+(0.694402537357)*x[1]**o+(-0.928918851291)*x[1]+(-0.439307255129)*x[2]**o+(0.148671231214)*x[2]
        arg[1,0,0,0]=(0.298044833507)*x[0]**o+(0.538000859279)*x[0]+(-0.405432041063)*x[1]**o+(-0.8912548734)*x[1]+(0.929554154255)*x[2]**o+(-0.515905504512)*x[2]
        arg[1,0,0,1]=(-0.575485929806)*x[0]**o+(0.757951009672)*x[0]+(-0.815024122535)*x[1]**o+(0.846519632117)*x[1]+(0.594901198403)*x[2]**o+(-0.672034020351)*x[2]
        arg[1,0,1,0]=(0.998358005707)*x[0]**o+(-0.144729504804)*x[0]+(0.463445062889)*x[1]**o+(0.428918148475)*x[1]+(-0.172684103233)*x[2]**o+(-0.394742387769)*x[2]
        arg[1,0,1,1]=(0.783593367143)*x[0]**o+(-0.269882783638)*x[0]+(-0.880336283806)*x[1]**o+(0.337087191943)*x[1]+(-0.819512478233)*x[2]**o+(0.582924902797)*x[2]
        arg[1,0,2,0]=(-0.42345692756)*x[0]**o+(-0.251910598711)*x[0]+(-0.882193473121)*x[1]**o+(0.535405977525)*x[1]+(-0.117109433242)*x[2]**o+(0.99565775159)*x[2]
        arg[1,0,2,1]=(-0.790917746681)*x[0]**o+(-0.291814540513)*x[0]+(-0.0828604085821)*x[1]**o+(-0.0715382319561)*x[1]+(-0.188453557262)*x[2]**o+(0.297596604241)*x[2]
        arg[1,1,0,0]=(0.944343622558)*x[0]**o+(0.537893843235)*x[0]+(-0.370697218599)*x[1]**o+(-0.043709074747)*x[1]+(-0.803318845377)*x[2]**o+(0.814392946479)*x[2]
        arg[1,1,0,1]=(-0.246711023019)*x[0]**o+(-0.847902406307)*x[0]+(0.062135604814)*x[1]**o+(0.581341495124)*x[1]+(0.561484848132)*x[2]**o+(0.06820017559)*x[2]
        arg[1,1,1,0]=(-0.530782063061)*x[0]**o+(-0.951246029994)*x[0]+(-0.809647094351)*x[1]**o+(0.198165508669)*x[1]+(0.591475451485)*x[2]**o+(-0.0567469904903)*x[2]
        arg[1,1,1,1]=(-0.710844761085)*x[0]**o+(0.742146961382)*x[0]+(0.482950602277)*x[1]**o+(-0.701576649521)*x[1]+(0.752305355358)*x[2]**o+(0.658478074845)*x[2]
        arg[1,1,2,0]=(0.574657505707)*x[0]**o+(-0.268094098242)*x[0]+(-0.79745549318)*x[1]**o+(0.994732385826)*x[1]+(0.36752110393)*x[2]**o+(0.0733521344978)*x[2]
        arg[1,1,2,1]=(-0.365373199615)*x[0]**o+(0.295532428151)*x[0]+(-0.916049089517)*x[1]**o+(0.562876895658)*x[1]+(0.698818392781)*x[2]**o+(0.413166153315)*x[2]
        arg[1,2,0,0]=(0.544776615353)*x[0]**o+(-0.647889580869)*x[0]+(-0.0569263298125)*x[1]**o+(0.145083519829)*x[1]+(-0.0456368749942)*x[2]**o+(0.125915481669)*x[2]
        arg[1,2,0,1]=(0.11684721915)*x[0]**o+(-0.793592672443)*x[0]+(-0.428585283805)*x[1]**o+(0.280358391066)*x[1]+(0.72731741847)*x[2]**o+(-0.320459136831)*x[2]
        arg[1,2,1,0]=(0.561736948232)*x[0]**o+(0.833411538285)*x[0]+(0.361519222525)*x[1]**o+(-0.785948355356)*x[1]+(-0.595938528895)*x[2]**o+(0.720247127573)*x[2]
        arg[1,2,1,1]=(0.578630452142)*x[0]**o+(0.218238639275)*x[0]+(0.792496779641)*x[1]**o+(-0.818431209813)*x[1]+(-0.252522807953)*x[2]**o+(0.648505893606)*x[2]
        arg[1,2,2,0]=(-0.0269203575755)*x[0]**o+(-0.586454813205)*x[0]+(-0.239885637578)*x[1]**o+(0.186263805313)*x[1]+(0.585909451795)*x[2]**o+(-0.630731036615)*x[2]
        arg[1,2,2,1]=(-0.534642633404)*x[0]**o+(0.921034930282)*x[0]+(-0.326672365556)*x[1]**o+(0.690509146367)*x[1]+(-0.802567575176)*x[2]**o+(-0.235903818924)*x[2]
        arg[1,3,0,0]=(0.892302802156)*x[0]**o+(-0.528836075071)*x[0]+(-0.806520835766)*x[1]**o+(0.771321197577)*x[1]+(-0.545545197249)*x[2]**o+(-0.633185919719)*x[2]
        arg[1,3,0,1]=(0.883702447985)*x[0]**o+(0.698176356191)*x[0]+(-0.348189466787)*x[1]**o+(-0.390058291315)*x[1]+(0.805736008481)*x[2]**o+(0.861787745203)*x[2]
        arg[1,3,1,0]=(0.492527250981)*x[0]**o+(0.635712008949)*x[0]+(-0.464397346193)*x[1]**o+(-0.820952575379)*x[1]+(0.235015001164)*x[2]**o+(-0.846160845231)*x[2]
        arg[1,3,1,1]=(0.372499842901)*x[0]**o+(-0.524824414377)*x[0]+(0.553450359486)*x[1]**o+(-0.187372498756)*x[1]+(-0.389983195918)*x[2]**o+(-0.310421627605)*x[2]
        arg[1,3,2,0]=(0.533098362924)*x[0]**o+(0.917820078197)*x[0]+(-0.557683070597)*x[1]**o+(-0.388498248383)*x[1]+(-0.551866333066)*x[2]**o+(0.696873468511)*x[2]
        arg[1,3,2,1]=(-0.909038123273)*x[0]**o+(0.19323884412)*x[0]+(-0.00134861813725)*x[1]**o+(-0.858193032021)*x[1]+(0.0436856504635)*x[2]**o+(-0.646994265986)*x[2]
        arg[1,4,0,0]=(-0.974474210185)*x[0]**o+(-0.39786371112)*x[0]+(-0.829024534519)*x[1]**o+(0.836151642189)*x[1]+(0.679654655991)*x[2]**o+(0.750995377779)*x[2]
        arg[1,4,0,1]=(0.964745405431)*x[0]**o+(0.269189443297)*x[0]+(0.531636623506)*x[1]**o+(-0.522687387383)*x[1]+(0.463500385567)*x[2]**o+(0.160556224359)*x[2]
        arg[1,4,1,0]=(-0.714725894011)*x[0]**o+(0.0478807291181)*x[0]+(0.591494552904)*x[1]**o+(-0.240293119817)*x[1]+(-0.0463360360003)*x[2]**o+(-0.932835908096)*x[2]
        arg[1,4,1,1]=(-0.809072545597)*x[0]**o+(-0.963335650229)*x[0]+(-0.781885522838)*x[1]**o+(0.822395174213)*x[1]+(-0.42590899181)*x[2]**o+(-0.237357227214)*x[2]
        arg[1,4,2,0]=(-0.21281876112)*x[0]**o+(-0.131640536998)*x[0]+(-0.648680371617)*x[1]**o+(0.555759774031)*x[1]+(-0.911030244394)*x[2]**o+(0.962859403252)*x[2]
        arg[1,4,2,1]=(-0.247930702516)*x[0]**o+(0.509829884687)*x[0]+(0.0387816417333)*x[1]**o+(-0.605945140743)*x[1]+(0.187880415018)*x[2]**o+(-0.813242364046)*x[2]
        arg[2,0,0,0]=(0.616352869937)*x[0]**o+(-0.930708209966)*x[0]+(-0.874778661809)*x[1]**o+(-0.330225154904)*x[1]+(0.632020336392)*x[2]**o+(0.205631598228)*x[2]
        arg[2,0,0,1]=(-0.85704374519)*x[0]**o+(0.99958496465)*x[0]+(0.807095767473)*x[1]**o+(0.63097388144)*x[1]+(0.000665614692828)*x[2]**o+(0.418070858864)*x[2]
        arg[2,0,1,0]=(0.326870102902)*x[0]**o+(0.565855791221)*x[0]+(-0.39418644908)*x[1]**o+(-0.96081735968)*x[1]+(0.937255293725)*x[2]**o+(0.533015538451)*x[2]
        arg[2,0,1,1]=(-0.699939842176)*x[0]**o+(0.808055402204)*x[0]+(0.448948790025)*x[1]**o+(-0.720122241439)*x[1]+(0.0532200858319)*x[2]**o+(-0.204851171261)*x[2]
        arg[2,0,2,0]=(-0.985996827188)*x[0]**o+(-0.604602591167)*x[0]+(0.445903925662)*x[1]**o+(-0.163135114049)*x[1]+(-0.264281316158)*x[2]**o+(-0.381814212863)*x[2]
        arg[2,0,2,1]=(-0.195357773436)*x[0]**o+(0.73285122682)*x[0]+(-0.914846292757)*x[1]**o+(-0.190524885095)*x[1]+(-0.103440307801)*x[2]**o+(-0.0309647945217)*x[2]
        arg[2,1,0,0]=(0.601652939329)*x[0]**o+(-0.757171386065)*x[0]+(-0.91147090556)*x[1]**o+(-0.665258414941)*x[1]+(-0.0504245123191)*x[2]**o+(-0.274928427711)*x[2]
        arg[2,1,0,1]=(0.534038022604)*x[0]**o+(-0.977365399298)*x[0]+(0.884656682614)*x[1]**o+(0.0866421048629)*x[1]+(-0.70687555879)*x[2]**o+(-0.99928465036)*x[2]
        arg[2,1,1,0]=(-0.361196978257)*x[0]**o+(0.192559020092)*x[0]+(-0.0767715201942)*x[1]**o+(0.222254953069)*x[1]+(-0.128362555436)*x[2]**o+(-0.667485829139)*x[2]
        arg[2,1,1,1]=(0.00930987921229)*x[0]**o+(0.269207633273)*x[0]+(0.943798095916)*x[1]**o+(0.483123564952)*x[1]+(-0.231669365037)*x[2]**o+(-0.877005245489)*x[2]
        arg[2,1,2,0]=(-0.181479246679)*x[0]**o+(0.619551810132)*x[0]+(0.499400298722)*x[1]**o+(-0.307864349855)*x[1]+(0.19303040809)*x[2]**o+(0.523640816928)*x[2]
        arg[2,1,2,1]=(-0.99665050175)*x[0]**o+(-0.647775674514)*x[0]+(-0.0015481673821)*x[1]**o+(-0.749198451256)*x[1]+(0.652528193579)*x[2]**o+(-0.0194127861341)*x[2]
        arg[2,2,0,0]=(0.870069083779)*x[0]**o+(0.179747121592)*x[0]+(0.387523112512)*x[1]**o+(0.625711925024)*x[1]+(0.840141938096)*x[2]**o+(0.629709976556)*x[2]
        arg[2,2,0,1]=(-0.841259995642)*x[0]**o+(0.220858862445)*x[0]+(0.397334296484)*x[1]**o+(-0.036654179636)*x[1]+(0.235704322138)*x[2]**o+(-0.6299228864)*x[2]
        arg[2,2,1,0]=(-0.561184923307)*x[0]**o+(-0.643644162376)*x[0]+(0.728784994685)*x[1]**o+(-0.697418070844)*x[1]+(-0.56166463788)*x[2]**o+(-0.668760084534)*x[2]
        arg[2,2,1,1]=(0.92243000401)*x[0]**o+(0.96165833182)*x[0]+(0.540379002453)*x[1]**o+(0.630245078647)*x[1]+(-0.375873656541)*x[2]**o+(0.71424898756)*x[2]
        arg[2,2,2,0]=(0.331131296272)*x[0]**o+(-0.136167378345)*x[0]+(-0.375407637025)*x[1]**o+(-0.0890691004072)*x[1]+(-0.787306129385)*x[2]**o+(-0.447474749986)*x[2]
        arg[2,2,2,1]=(-0.47892171854)*x[0]**o+(0.341448171328)*x[0]+(0.662403724393)*x[1]**o+(0.581128263913)*x[1]+(0.00987835902932)*x[2]**o+(-0.884847129103)*x[2]
        arg[2,3,0,0]=(0.0219406022052)*x[0]**o+(-0.370001982912)*x[0]+(0.948162807268)*x[1]**o+(-0.870857624404)*x[1]+(0.580053532448)*x[2]**o+(-0.488657040041)*x[2]
        arg[2,3,0,1]=(-0.114799511617)*x[0]**o+(0.547853919356)*x[0]+(-0.224367707174)*x[1]**o+(0.551491902831)*x[1]+(-0.101845907889)*x[2]**o+(-0.793527438395)*x[2]
        arg[2,3,1,0]=(-0.103518223905)*x[0]**o+(-0.319840011093)*x[0]+(-0.856612861717)*x[1]**o+(0.0208578084364)*x[1]+(0.132248308449)*x[2]**o+(-0.580663554499)*x[2]
        arg[2,3,1,1]=(-0.958142224242)*x[0]**o+(-0.401299403423)*x[0]+(-0.289034019185)*x[1]**o+(-0.0913958572584)*x[1]+(-0.818515945817)*x[2]**o+(-0.918366425737)*x[2]
        arg[2,3,2,0]=(0.730554594734)*x[0]**o+(0.209964841318)*x[0]+(-0.4157212696)*x[1]**o+(-0.673375403124)*x[1]+(-0.409899056244)*x[2]**o+(0.124418003637)*x[2]
        arg[2,3,2,1]=(-0.930974589897)*x[0]**o+(0.256452400333)*x[0]+(0.778606258675)*x[1]**o+(0.23173182074)*x[1]+(-0.846439682798)*x[2]**o+(-0.408860675089)*x[2]
        arg[2,4,0,0]=(-0.246046780287)*x[0]**o+(-0.653766430943)*x[0]+(0.725379461155)*x[1]**o+(-0.305490823497)*x[1]+(0.0837637468427)*x[2]**o+(-0.458770631133)*x[2]
        arg[2,4,0,1]=(-0.612751147889)*x[0]**o+(0.504418327803)*x[0]+(-0.835051783805)*x[1]**o+(-0.86410559166)*x[1]+(0.928919612861)*x[2]**o+(0.303164594248)*x[2]
        arg[2,4,1,0]=(-0.26092972064)*x[0]**o+(-0.990592356395)*x[0]+(-0.48716443965)*x[1]**o+(0.659447608304)*x[1]+(0.429865593656)*x[2]**o+(-0.661470437053)*x[2]
        arg[2,4,1,1]=(0.325840618456)*x[0]**o+(-0.261974904995)*x[0]+(0.476982730267)*x[1]**o+(-0.738119182885)*x[1]+(0.903618858571)*x[2]**o+(-0.287442022624)*x[2]
        arg[2,4,2,0]=(0.655590135983)*x[0]**o+(-0.135505327297)*x[0]+(-0.38079131346)*x[1]**o+(0.663850751769)*x[1]+(-0.658568109867)*x[2]**o+(-0.267875066063)*x[2]
        arg[2,4,2,1]=(0.570712162768)*x[0]**o+(0.390833274898)*x[0]+(-0.97326103687)*x[1]**o+(-0.419300688467)*x[1]+(0.55695479018)*x[2]**o+(0.547271407555)*x[2]
        arg[3,0,0,0]=(-0.486023722572)*x[0]**o+(-0.790544923201)*x[0]+(0.101237181961)*x[1]**o+(0.23267779263)*x[1]+(-0.458580701132)*x[2]**o+(-0.479449733139)*x[2]
        arg[3,0,0,1]=(-0.705291625776)*x[0]**o+(0.144166702405)*x[0]+(-0.210596631253)*x[1]**o+(0.843721258425)*x[1]+(-0.716420627484)*x[2]**o+(0.145427688875)*x[2]
        arg[3,0,1,0]=(-0.838974120992)*x[0]**o+(0.824423674933)*x[0]+(-0.211643366873)*x[1]**o+(0.603255736693)*x[1]+(-0.119484817371)*x[2]**o+(-0.357272543813)*x[2]
        arg[3,0,1,1]=(0.140027541071)*x[0]**o+(0.373598174368)*x[0]+(-0.661831753147)*x[1]**o+(-0.973724961653)*x[1]+(-0.0738899100301)*x[2]**o+(0.0128579666074)*x[2]
        arg[3,0,2,0]=(-0.39083471939)*x[0]**o+(-0.00821070494132)*x[0]+(-0.315936839906)*x[1]**o+(-0.967556350778)*x[1]+(0.136991762476)*x[2]**o+(0.374041520875)*x[2]
        arg[3,0,2,1]=(0.0734743009776)*x[0]**o+(-0.103186084491)*x[0]+(0.426933353985)*x[1]**o+(-0.631917740007)*x[1]+(0.755296770294)*x[2]**o+(0.492171872216)*x[2]
        arg[3,1,0,0]=(-0.414502599055)*x[0]**o+(0.512031410722)*x[0]+(0.610677049483)*x[1]**o+(-0.075275839509)*x[1]+(0.853407100545)*x[2]**o+(-0.569731924126)*x[2]
        arg[3,1,0,1]=(0.886115095396)*x[0]**o+(0.811807969433)*x[0]+(0.550623354147)*x[1]**o+(-0.703728894875)*x[1]+(0.784318903356)*x[2]**o+(0.45891829089)*x[2]
        arg[3,1,1,0]=(0.312577129879)*x[0]**o+(0.0127110992422)*x[0]+(-0.174614340667)*x[1]**o+(0.744535847679)*x[1]+(0.991125452344)*x[2]**o+(-0.405595301834)*x[2]
        arg[3,1,1,1]=(0.784369258288)*x[0]**o+(-0.709503832235)*x[0]+(0.032963558933)*x[1]**o+(-0.471036543959)*x[1]+(-0.138022967476)*x[2]**o+(-0.38452297541)*x[2]
        arg[3,1,2,0]=(-0.337332295082)*x[0]**o+(-0.460457867002)*x[0]+(-0.102806204559)*x[1]**o+(-0.502889108364)*x[1]+(-0.641488011668)*x[2]**o+(0.359501036527)*x[2]
        arg[3,1,2,1]=(0.791575892417)*x[0]**o+(0.126853712619)*x[0]+(-0.408911942637)*x[1]**o+(0.557039825987)*x[1]+(-0.260719710621)*x[2]**o+(-0.648326395935)*x[2]
        arg[3,2,0,0]=(-0.0909007497318)*x[0]**o+(-0.337085519884)*x[0]+(-0.651987922715)*x[1]**o+(-0.130225558042)*x[1]+(0.926567188664)*x[2]**o+(0.764369275478)*x[2]
        arg[3,2,0,1]=(-0.764137934896)*x[0]**o+(0.443892891832)*x[0]+(0.688197384487)*x[1]**o+(0.485210058592)*x[1]+(0.267932741224)*x[2]**o+(0.0366064885582)*x[2]
        arg[3,2,1,0]=(0.49435832562)*x[0]**o+(0.860649583676)*x[0]+(0.667415091305)*x[1]**o+(0.144069385904)*x[1]+(-0.933847080974)*x[2]**o+(-0.378123304101)*x[2]
        arg[3,2,1,1]=(-0.460752644946)*x[0]**o+(0.491079753273)*x[0]+(-0.536055633506)*x[1]**o+(0.786378615881)*x[1]+(-0.190647453518)*x[2]**o+(-0.192868019034)*x[2]
        arg[3,2,2,0]=(0.0946687985697)*x[0]**o+(0.978197346579)*x[0]+(0.972747948856)*x[1]**o+(-0.575670550665)*x[1]+(-0.096229657916)*x[2]**o+(-0.772567430449)*x[2]
        arg[3,2,2,1]=(0.177521713734)*x[0]**o+(0.0683685252148)*x[0]+(0.423995112682)*x[1]**o+(-0.489247981968)*x[1]+(-0.537130434099)*x[2]**o+(-0.0228879131449)*x[2]
        arg[3,3,0,0]=(0.592201472175)*x[0]**o+(0.674304504132)*x[0]+(0.428414872879)*x[1]**o+(0.956414209094)*x[1]+(-0.272853242491)*x[2]**o+(-0.639858910901)*x[2]
        arg[3,3,0,1]=(0.572972211447)*x[0]**o+(0.943096858489)*x[0]+(-0.990948797302)*x[1]**o+(-0.475383526007)*x[1]+(0.542076985363)*x[2]**o+(-0.884809421862)*x[2]
        arg[3,3,1,0]=(0.775163727164)*x[0]**o+(0.101808074039)*x[0]+(0.575490330857)*x[1]**o+(0.199751068512)*x[1]+(0.538742155685)*x[2]**o+(-0.725822956214)*x[2]
        arg[3,3,1,1]=(-0.244586606882)*x[0]**o+(0.983712254807)*x[0]+(0.187370538052)*x[1]**o+(0.332611385073)*x[1]+(0.536377667373)*x[2]**o+(0.255272558489)*x[2]
        arg[3,3,2,0]=(-0.676040282048)*x[0]**o+(-0.172059284021)*x[0]+(0.25100451419)*x[1]**o+(0.303226061816)*x[1]+(0.138976177556)*x[2]**o+(-0.330104849626)*x[2]
        arg[3,3,2,1]=(-0.136437682299)*x[0]**o+(0.938786085213)*x[0]+(0.853018944962)*x[1]**o+(-0.628078903025)*x[1]+(-0.903567849919)*x[2]**o+(0.837621994591)*x[2]
        arg[3,4,0,0]=(0.28632587806)*x[0]**o+(-0.480626263522)*x[0]+(-0.0537965100699)*x[1]**o+(0.423252432505)*x[1]+(0.200777153039)*x[2]**o+(0.629713098071)*x[2]
        arg[3,4,0,1]=(0.326448964148)*x[0]**o+(0.809991245673)*x[0]+(-0.527362539073)*x[1]**o+(-0.0727890632651)*x[1]+(0.210806278508)*x[2]**o+(0.736535063786)*x[2]
        arg[3,4,1,0]=(0.760474271531)*x[0]**o+(0.378083396435)*x[0]+(-0.991296179693)*x[1]**o+(0.431965391269)*x[1]+(-0.12597704605)*x[2]**o+(-0.128017966575)*x[2]
        arg[3,4,1,1]=(-0.21508452109)*x[0]**o+(-0.583147117492)*x[0]+(0.779535489322)*x[1]**o+(0.601144982341)*x[1]+(-0.870362610456)*x[2]**o+(-0.318535391517)*x[2]
        arg[3,4,2,0]=(-0.195181632154)*x[0]**o+(0.237570596161)*x[0]+(-0.794725125363)*x[1]**o+(0.862030507166)*x[1]+(-0.612206533195)*x[2]**o+(0.494078406766)*x[2]
        arg[3,4,2,1]=(0.270278273932)*x[0]**o+(-0.115833233113)*x[0]+(0.754464364285)*x[1]**o+(-0.186069521903)*x[1]+(-0.864426716229)*x[2]**o+(-0.425054438064)*x[2]
        ref[0,0,0,0]=(0.412792261707)/(o+1.)+(0.904364083309)+(0.663998508168)*0.5**o
        ref[0,0,0,1]=(-1.07469333717)/(o+1.)+(-0.0923352454984)+(-0.010130464476)*0.5**o
        ref[0,0,1,0]=(0.287923726124)/(o+1.)+(-0.145507770124)+(-0.176542289385)*0.5**o
        ref[0,0,1,1]=(-0.24656050398)/(o+1.)+(0.341982901349)+(-0.98152572229)*0.5**o
        ref[0,0,2,0]=(1.24918965808)/(o+1.)+(0.165246366859)+(0.555897422173)*0.5**o
        ref[0,0,2,1]=(-1.23538766622)/(o+1.)+(-0.507697956406)+(0.23127040923)*0.5**o
        ref[0,1,0,0]=(-0.267829490776)/(o+1.)+(-0.549968075823)+(0.0202445177842)*0.5**o
        ref[0,1,0,1]=(0.826177960569)/(o+1.)+(0.113794537096)+(0.99935098551)*0.5**o
        ref[0,1,1,0]=(0.124131681048)/(o+1.)+(0.482277820918)+(0.151799117419)*0.5**o
        ref[0,1,1,1]=(0.620076580442)/(o+1.)+(0.401695495934)+(-0.51422532626)*0.5**o
        ref[0,1,2,0]=(1.75895293799)/(o+1.)+(-0.225823657945)+(0.908244888839)*0.5**o
        ref[0,1,2,1]=(-0.334875364565)/(o+1.)+(1.30271444502)+(-0.872307766634)*0.5**o
        ref[0,2,0,0]=(-0.42419420508)/(o+1.)+(-0.3575838585)+(0.00173619592035)*0.5**o
        ref[0,2,0,1]=(-0.57391253095)/(o+1.)+(-0.515088003037)+(0.998184336552)*0.5**o
        ref[0,2,1,0]=(0.114144861758)/(o+1.)+(-0.330251158386)+(-0.921234331968)*0.5**o
        ref[0,2,1,1]=(-1.42031433961)/(o+1.)+(0.581091144051)+(0.153578015941)*0.5**o
        ref[0,2,2,0]=(0.827236715358)/(o+1.)+(0.125325412749)+(0.29601674794)*0.5**o
        ref[0,2,2,1]=(1.34802786361)/(o+1.)+(0.280667679493)+(0.0580007647099)*0.5**o
        ref[0,3,0,0]=(1.28181937762)/(o+1.)+(-0.214505128496)+(-0.337999890505)*0.5**o
        ref[0,3,0,1]=(1.65569377802)/(o+1.)+(-0.363265639443)+(-0.825813329629)*0.5**o
        ref[0,3,1,0]=(0.560544990797)/(o+1.)+(-0.276979054204)+(0.699313512079)*0.5**o
        ref[0,3,1,1]=(-0.879495822379)/(o+1.)+(0.560842505268)+(0.140764124286)*0.5**o
        ref[0,3,2,0]=(-1.04751560841)/(o+1.)+(-0.238703591863)+(-0.68177752688)*0.5**o
        ref[0,3,2,1]=(-0.414022097401)/(o+1.)+(0.509666283203)+(0.655685968811)*0.5**o
        ref[0,4,0,0]=(1.0625549765)/(o+1.)+(-1.26969185098)+(-0.18027010068)*0.5**o
        ref[0,4,0,1]=(-0.430389257621)/(o+1.)+(-0.728926201043)+(-0.914768182433)*0.5**o
        ref[0,4,1,0]=(-0.585220311209)/(o+1.)+(0.344278586108)+(0.274727423835)*0.5**o
        ref[0,4,1,1]=(-0.596140928652)/(o+1.)+(-0.857974578823)+(0.257574026756)*0.5**o
        ref[0,4,2,0]=(-1.15566595381)/(o+1.)+(0.148922032671)+(0.703185665611)*0.5**o
        ref[0,4,2,1]=(0.255095282229)/(o+1.)+(-0.480012460544)+(-0.153792811673)*0.5**o
        ref[1,0,0,0]=(0.524122113192)/(o+1.)+(-0.434579759316)+(0.298044833507)*0.5**o
        ref[1,0,0,1]=(-0.220122924131)/(o+1.)+(0.466218310719)+(-0.575485929806)*0.5**o
        ref[1,0,1,0]=(0.290760959656)/(o+1.)+(-0.0552768720487)+(0.998358005707)*0.5**o
        ref[1,0,1,1]=(-1.69984876204)/(o+1.)+(0.325064655551)+(0.783593367143)*0.5**o
        ref[1,0,2,0]=(-0.999302906363)/(o+1.)+(0.639576565202)+(-0.42345692756)*0.5**o
        ref[1,0,2,1]=(-0.271313965844)/(o+1.)+(-0.0328780841139)+(-0.790917746681)*0.5**o
        ref[1,1,0,0]=(-1.17401606398)/(o+1.)+(0.654288857484)+(0.944343622558)*0.5**o
        ref[1,1,0,1]=(0.623620452946)/(o+1.)+(-0.0991803677964)+(-0.246711023019)*0.5**o
        ref[1,1,1,0]=(-0.218171642866)/(o+1.)+(-0.404913755908)+(-0.530782063061)*0.5**o
        ref[1,1,1,1]=(1.23525595763)/(o+1.)+(0.349524193353)+(-0.710844761085)*0.5**o
        ref[1,1,2,0]=(-0.42993438925)/(o+1.)+(0.399995211041)+(0.574657505707)*0.5**o
        ref[1,1,2,1]=(-0.217230696736)/(o+1.)+(0.635787738562)+(-0.365373199615)*0.5**o
        ref[1,2,0,0]=(-0.102563204807)/(o+1.)+(-0.188445289686)+(0.544776615353)*0.5**o
        ref[1,2,0,1]=(0.298732134665)/(o+1.)+(-0.416846709104)+(0.11684721915)*0.5**o
        ref[1,2,1,0]=(-0.234419306371)/(o+1.)+(0.383855155251)+(0.561736948232)*0.5**o
        ref[1,2,1,1]=(0.539973971687)/(o+1.)+(0.0241566615339)+(0.578630452142)*0.5**o
        ref[1,2,2,0]=(0.346023814217)/(o+1.)+(-0.515461022254)+(-0.0269203575755)*0.5**o
        ref[1,2,2,1]=(-1.12923994073)/(o+1.)+(0.687820128862)+(-0.534642633404)*0.5**o
        ref[1,3,0,0]=(-1.35206603301)/(o+1.)+(-0.195350398606)+(0.892302802156)*0.5**o
        ref[1,3,0,1]=(0.457546541694)/(o+1.)+(0.584952905039)+(0.883702447985)*0.5**o
        ref[1,3,1,0]=(-0.229382345029)/(o+1.)+(-0.51570070583)+(0.492527250981)*0.5**o
        ref[1,3,1,1]=(0.163467163568)/(o+1.)+(-0.511309270369)+(0.372499842901)*0.5**o
        ref[1,3,2,0]=(-1.10954940366)/(o+1.)+(0.613097649163)+(0.533098362924)*0.5**o
        ref[1,3,2,1]=(0.0423370323263)/(o+1.)+(-0.655974226943)+(-0.909038123273)*0.5**o
        ref[1,4,0,0]=(-0.149369878527)/(o+1.)+(0.594641654424)+(-0.974474210185)*0.5**o
        ref[1,4,0,1]=(0.995137009073)/(o+1.)+(-0.0464708598635)+(0.964745405431)*0.5**o
        ref[1,4,1,0]=(0.545158516903)/(o+1.)+(-0.562624149397)+(-0.714725894011)*0.5**o
        ref[1,4,1,1]=(-1.20779451465)/(o+1.)+(-0.189148851615)+(-0.809072545597)*0.5**o
        ref[1,4,2,0]=(-1.55971061601)/(o+1.)+(0.693489320143)+(-0.21281876112)*0.5**o
        ref[1,4,2,1]=(0.226662056752)/(o+1.)+(-0.454678810051)+(-0.247930702516)*0.5**o
        ref[2,0,0,0]=(-0.242758325417)/(o+1.)+(-0.527650883321)+(0.616352869937)*0.5**o
        ref[2,0,0,1]=(0.807761382166)/(o+1.)+(1.02431485248)+(-0.85704374519)*0.5**o
        ref[2,0,1,0]=(0.543068844645)/(o+1.)+(0.0690269849966)+(0.326870102902)*0.5**o
        ref[2,0,1,1]=(0.502168875857)/(o+1.)+(-0.0584590052478)+(-0.699939842176)*0.5**o
        ref[2,0,2,0]=(0.181622609504)/(o+1.)+(-0.57477595904)+(-0.985996827188)*0.5**o
        ref[2,0,2,1]=(-1.01828660056)/(o+1.)+(0.255680773601)+(-0.195357773436)*0.5**o
        ref[2,1,0,0]=(-0.961895417879)/(o+1.)+(-0.848679114358)+(0.601652939329)*0.5**o
        ref[2,1,0,1]=(0.177781123824)/(o+1.)+(-0.945003972397)+(0.534038022604)*0.5**o
        ref[2,1,1,0]=(-0.20513407563)/(o+1.)+(-0.126335927989)+(-0.361196978257)*0.5**o
        ref[2,1,1,1]=(0.712128730879)/(o+1.)+(-0.0623370236322)+(0.00930987921229)*0.5**o
        ref[2,1,2,0]=(0.692430706812)/(o+1.)+(0.417664138602)+(-0.181479246679)*0.5**o
        ref[2,1,2,1]=(0.650980026197)/(o+1.)+(-0.708193455952)+(-0.99665050175)*0.5**o
        ref[2,2,0,0]=(1.22766505061)/(o+1.)+(0.717584511586)+(0.870069083779)*0.5**o
        ref[2,2,0,1]=(0.633038618622)/(o+1.)+(-0.222859101796)+(-0.841259995642)*0.5**o
        ref[2,2,1,0]=(0.167120356805)/(o+1.)+(-1.00491115888)+(-0.561184923307)*0.5**o
        ref[2,2,1,1]=(0.164505345911)/(o+1.)+(1.15307619901)+(0.92243000401)*0.5**o
        ref[2,2,2,0]=(-1.16271376641)/(o+1.)+(-0.336355614369)+(0.331131296272)*0.5**o
        ref[2,2,2,1]=(0.672282083422)/(o+1.)+(0.018864653069)+(-0.47892171854)*0.5**o
        ref[2,3,0,0]=(1.52821633972)/(o+1.)+(-0.864758323678)+(0.0219406022052)*0.5**o
        ref[2,3,0,1]=(-0.326213615062)/(o+1.)+(0.152909191896)+(-0.114799511617)*0.5**o
        ref[2,3,1,0]=(-0.724364553267)/(o+1.)+(-0.439822878578)+(-0.103518223905)*0.5**o
        ref[2,3,1,1]=(-1.107549965)/(o+1.)+(-0.705530843209)+(-0.958142224242)*0.5**o
        ref[2,3,2,0]=(-0.825620325844)/(o+1.)+(-0.169496279085)+(0.730554594734)*0.5**o
        ref[2,3,2,1]=(-0.0678334241233)/(o+1.)+(0.0396617729923)+(-0.930974589897)*0.5**o
        ref[2,4,0,0]=(0.809143207998)/(o+1.)+(-0.709013942786)+(-0.246046780287)*0.5**o
        ref[2,4,0,1]=(0.0938678290563)/(o+1.)+(-0.0282613348041)+(-0.612751147889)*0.5**o
        ref[2,4,1,0]=(-0.0572988459938)/(o+1.)+(-0.496307592572)+(-0.26092972064)*0.5**o
        ref[2,4,1,1]=(1.38060158884)/(o+1.)+(-0.643768055252)+(0.325840618456)*0.5**o
        ref[2,4,2,0]=(-1.03935942333)/(o+1.)+(0.130235179204)+(0.655590135983)*0.5**o
        ref[2,4,2,1]=(-0.41630624669)/(o+1.)+(0.259401996992)+(0.570712162768)*0.5**o
        ref[3,0,0,0]=(-0.357343519171)/(o+1.)+(-0.518658431855)+(-0.486023722572)*0.5**o
        ref[3,0,0,1]=(-0.927017258737)/(o+1.)+(0.566657824853)+(-0.705291625776)*0.5**o
        ref[3,0,1,0]=(-0.331128184244)/(o+1.)+(0.535203433907)+(-0.838974120992)*0.5**o
        ref[3,0,1,1]=(-0.735721663177)/(o+1.)+(-0.293634410339)+(0.140027541071)*0.5**o
        ref[3,0,2,0]=(-0.178945077431)/(o+1.)+(-0.300862767422)+(-0.39083471939)*0.5**o
        ref[3,0,2,1]=(1.18223012428)/(o+1.)+(-0.121465976141)+(0.0734743009776)*0.5**o
        ref[3,1,0,0]=(1.46408415003)/(o+1.)+(-0.0664881764563)+(-0.414502599055)*0.5**o
        ref[3,1,0,1]=(1.3349422575)/(o+1.)+(0.283498682724)+(0.886115095396)*0.5**o
        ref[3,1,1,0]=(0.816511111677)/(o+1.)+(0.175825822544)+(0.312577129879)*0.5**o
        ref[3,1,1,1]=(-0.105059408543)/(o+1.)+(-0.782531675802)+(0.784369258288)*0.5**o
        ref[3,1,2,0]=(-0.744294216228)/(o+1.)+(-0.301922969419)+(-0.337332295082)*0.5**o
        ref[3,1,2,1]=(-0.669631653258)/(o+1.)+(0.0177835713351)+(0.791575892417)*0.5**o
        ref[3,2,0,0]=(0.274579265949)/(o+1.)+(0.148529098776)+(-0.0909007497318)*0.5**o
        ref[3,2,0,1]=(0.956130125711)/(o+1.)+(0.482854719491)+(-0.764137934896)*0.5**o
        ref[3,2,1,0]=(-0.266431989669)/(o+1.)+(0.313297832739)+(0.49435832562)*0.5**o
        ref[3,2,1,1]=(-0.726703087024)/(o+1.)+(0.542295175061)+(-0.460752644946)*0.5**o
        ref[3,2,2,0]=(0.87651829094)/(o+1.)+(-0.185020317268)+(0.0946687985697)*0.5**o
        ref[3,2,2,1]=(-0.113135321417)/(o+1.)+(-0.221883684949)+(0.177521713734)*0.5**o
        ref[3,3,0,0]=(0.155561630388)/(o+1.)+(0.495429901163)+(0.592201472175)*0.5**o
        ref[3,3,0,1]=(-0.44887181194)/(o+1.)+(-0.20854804469)+(0.572972211447)*0.5**o
        ref[3,3,1,0]=(1.11423248654)/(o+1.)+(-0.212131906831)+(0.775163727164)*0.5**o
        ref[3,3,1,1]=(0.723748205425)/(o+1.)+(0.785798099184)+(-0.244586606882)*0.5**o
        ref[3,3,2,0]=(0.389980691746)/(o+1.)+(-0.0994690359155)+(-0.676040282048)*0.5**o
        ref[3,3,2,1]=(-0.0505489049572)/(o+1.)+(0.57416458839)+(-0.136437682299)*0.5**o
        ref[3,4,0,0]=(0.14698064297)/(o+1.)+(0.286169633527)+(0.28632587806)*0.5**o
        ref[3,4,0,1]=(-0.316556260565)/(o+1.)+(0.736868623097)+(0.326448964148)*0.5**o
        ref[3,4,1,0]=(-1.11727322574)/(o+1.)+(0.341015410564)+(0.760474271531)*0.5**o
        ref[3,4,1,1]=(-0.0908271211344)/(o+1.)+(-0.150268763334)+(-0.21508452109)*0.5**o
        ref[3,4,2,0]=(-1.40693165856)/(o+1.)+(0.796839755046)+(-0.195181632154)*0.5**o
        ref[3,4,2,1]=(-0.109962351944)/(o+1.)+(-0.36347859654)+(0.270278273932)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactZero_fromData_Solution_rank0(self):
      """
      tests integral of rank 0 Data on the ReducedFunctionOnContactZero

      assumptions: Solution(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      w=Solution(self.domain)
      x=w.getX()
      arg=Data(0,(),w)
      if dim==2:
        arg=(0.0433006726989)*x[0]**o+(0.819109331838)*x[0]+(0.150292245587)*x[1]**o+(0.949397165008)*x[1]
        ref=(0.150292245587)/(o+1.)+(0.884253248423)+(0.0433006726989)*0.5**o
      else:
        arg=(-0.765858486678)*x[0]**o+(-0.829061249978)*x[0]+(-0.0255273602735)*x[1]**o+(-0.93097352763)*x[1]+(-0.775974817954)*x[2]**o+(0.0414580763278)*x[2]
        ref=(-0.801502178228)/(o+1.)+(-0.85928835064)+(-0.765858486678)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactZero_fromData_Solution_rank1(self):
      """
      tests integral of rank 1 Data on the ReducedFunctionOnContactZero

      assumptions: Solution(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      w=Solution(self.domain)
      x=w.getX()
      arg=Data(0,(2,),w)
      ref=numpy.zeros((2,),numpy.float_)
      if dim==2:
        arg[0]=(0.708418284738)*x[0]**o+(-0.265416249865)*x[0]+(0.0412873279676)*x[1]**o+(0.18827690539)*x[1]
        arg[1]=(-0.0639578348425)*x[0]**o+(-0.435408788697)*x[0]+(0.429134354603)*x[1]**o+(0.0391558318157)*x[1]
        ref[0]=(0.0412873279676)/(o+1.)+(-0.0385696722374)+(0.708418284738)*0.5**o
        ref[1]=(0.429134354603)/(o+1.)+(-0.198126478441)+(-0.0639578348425)*0.5**o
      else:
        arg[0]=(-0.516066856372)*x[0]**o+(-0.509677909948)*x[0]+(0.189594948337)*x[1]**o+(0.743432553688)*x[1]+(0.687864109046)*x[2]**o+(0.257280277379)*x[2]
        arg[1]=(0.948762785699)*x[0]**o+(0.372861763607)*x[0]+(-0.166460043702)*x[1]**o+(0.0474018901531)*x[1]+(-0.485380626595)*x[2]**o+(-0.341701937267)*x[2]
        ref[0]=(0.877459057383)/(o+1.)+(0.245517460559)+(-0.516066856372)*0.5**o
        ref[1]=(-0.651840670298)/(o+1.)+(0.0392808582465)+(0.948762785699)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactZero_fromData_Solution_rank2(self):
      """
      tests integral of rank 2 Data on the ReducedFunctionOnContactZero

      assumptions: Solution(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      w=Solution(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5),w)
      ref=numpy.zeros((4, 5),numpy.float_)
      if dim==2:
        arg[0,0]=(-0.0540280486222)*x[0]**o+(-0.936629447756)*x[0]+(0.530867101627)*x[1]**o+(-0.404522355122)*x[1]
        arg[0,1]=(0.585051034599)*x[0]**o+(-0.899683342239)*x[0]+(0.61865830472)*x[1]**o+(0.491768251743)*x[1]
        arg[0,2]=(0.924131379905)*x[0]**o+(-0.394367734246)*x[0]+(0.432122525065)*x[1]**o+(0.79923634141)*x[1]
        arg[0,3]=(-0.41435196904)*x[0]**o+(-0.22039395628)*x[0]+(-0.866551833459)*x[1]**o+(-0.0966534238435)*x[1]
        arg[0,4]=(0.855785701017)*x[0]**o+(0.0879754354535)*x[0]+(-0.0727782016239)*x[1]**o+(0.746878225435)*x[1]
        arg[1,0]=(-0.868450306659)*x[0]**o+(-0.00845792225901)*x[0]+(0.227399876353)*x[1]**o+(-0.522196483799)*x[1]
        arg[1,1]=(0.53076931297)*x[0]**o+(0.165744140226)*x[0]+(0.14586398181)*x[1]**o+(-0.694169551043)*x[1]
        arg[1,2]=(-0.936146144862)*x[0]**o+(-0.198367580291)*x[0]+(0.848822682163)*x[1]**o+(0.360521711263)*x[1]
        arg[1,3]=(-0.395889451325)*x[0]**o+(0.510713345487)*x[0]+(-0.62440033386)*x[1]**o+(0.29695714683)*x[1]
        arg[1,4]=(0.847685553395)*x[0]**o+(-0.291266469606)*x[0]+(-0.517114839541)*x[1]**o+(-0.279948585383)*x[1]
        arg[2,0]=(-0.622481093435)*x[0]**o+(0.0911922519622)*x[0]+(0.216648336495)*x[1]**o+(0.663562516802)*x[1]
        arg[2,1]=(0.567833843918)*x[0]**o+(0.18116887359)*x[0]+(-0.0433012798352)*x[1]**o+(0.998332142285)*x[1]
        arg[2,2]=(-0.81729786652)*x[0]**o+(-0.634680104287)*x[0]+(0.991513806139)*x[1]**o+(-0.0721700196003)*x[1]
        arg[2,3]=(0.121181286827)*x[0]**o+(-0.677803464822)*x[0]+(0.870139639102)*x[1]**o+(-0.134662002755)*x[1]
        arg[2,4]=(-0.167376807346)*x[0]**o+(-0.500556007898)*x[0]+(-0.839256017197)*x[1]**o+(-0.82147890878)*x[1]
        arg[3,0]=(0.576521064132)*x[0]**o+(-0.510276372588)*x[0]+(0.92888227487)*x[1]**o+(-0.88293512036)*x[1]
        arg[3,1]=(-0.159048471862)*x[0]**o+(-0.0479500210678)*x[0]+(-0.0216438779517)*x[1]**o+(-0.108252189548)*x[1]
        arg[3,2]=(-0.483779576334)*x[0]**o+(-0.871829147209)*x[0]+(-0.22556218283)*x[1]**o+(0.451007116911)*x[1]
        arg[3,3]=(-0.358888369902)*x[0]**o+(0.436554685191)*x[0]+(-0.0359187908139)*x[1]**o+(-0.0148930474244)*x[1]
        arg[3,4]=(-0.995411478882)*x[0]**o+(-0.457206328785)*x[0]+(-0.833823947273)*x[1]**o+(-0.0517068976093)*x[1]
        ref[0,0]=(0.530867101627)/(o+1.)+(-0.670575901439)+(-0.0540280486222)*0.5**o
        ref[0,1]=(0.61865830472)/(o+1.)+(-0.203957545248)+(0.585051034599)*0.5**o
        ref[0,2]=(0.432122525065)/(o+1.)+(0.202434303582)+(0.924131379905)*0.5**o
        ref[0,3]=(-0.866551833459)/(o+1.)+(-0.158523690062)+(-0.41435196904)*0.5**o
        ref[0,4]=(-0.0727782016239)/(o+1.)+(0.417426830444)+(0.855785701017)*0.5**o
        ref[1,0]=(0.227399876353)/(o+1.)+(-0.265327203029)+(-0.868450306659)*0.5**o
        ref[1,1]=(0.14586398181)/(o+1.)+(-0.264212705408)+(0.53076931297)*0.5**o
        ref[1,2]=(0.848822682163)/(o+1.)+(0.0810770654864)+(-0.936146144862)*0.5**o
        ref[1,3]=(-0.62440033386)/(o+1.)+(0.403835246159)+(-0.395889451325)*0.5**o
        ref[1,4]=(-0.517114839541)/(o+1.)+(-0.285607527495)+(0.847685553395)*0.5**o
        ref[2,0]=(0.216648336495)/(o+1.)+(0.377377384382)+(-0.622481093435)*0.5**o
        ref[2,1]=(-0.0433012798352)/(o+1.)+(0.589750507938)+(0.567833843918)*0.5**o
        ref[2,2]=(0.991513806139)/(o+1.)+(-0.353425061944)+(-0.81729786652)*0.5**o
        ref[2,3]=(0.870139639102)/(o+1.)+(-0.406232733789)+(0.121181286827)*0.5**o
        ref[2,4]=(-0.839256017197)/(o+1.)+(-0.661017458339)+(-0.167376807346)*0.5**o
        ref[3,0]=(0.92888227487)/(o+1.)+(-0.696605746474)+(0.576521064132)*0.5**o
        ref[3,1]=(-0.0216438779517)/(o+1.)+(-0.078101105308)+(-0.159048471862)*0.5**o
        ref[3,2]=(-0.22556218283)/(o+1.)+(-0.210411015149)+(-0.483779576334)*0.5**o
        ref[3,3]=(-0.0359187908139)/(o+1.)+(0.210830818883)+(-0.358888369902)*0.5**o
        ref[3,4]=(-0.833823947273)/(o+1.)+(-0.254456613197)+(-0.995411478882)*0.5**o
      else:
        arg[0,0]=(-0.937858877179)*x[0]**o+(-0.41590845461)*x[0]+(0.317385444442)*x[1]**o+(-0.166284606371)*x[1]+(-0.846106994172)*x[2]**o+(0.227876218054)*x[2]
        arg[0,1]=(0.497785867084)*x[0]**o+(0.883557362705)*x[0]+(-0.846602726567)*x[1]**o+(-0.859091910548)*x[1]+(-0.295742503292)*x[2]**o+(-0.853931233266)*x[2]
        arg[0,2]=(0.744501879094)*x[0]**o+(-0.0181115125382)*x[0]+(-0.856099357899)*x[1]**o+(0.767449074734)*x[1]+(-0.858552187616)*x[2]**o+(-0.580142057156)*x[2]
        arg[0,3]=(0.210031467359)*x[0]**o+(-0.994426053833)*x[0]+(0.648522478175)*x[1]**o+(0.0355805603417)*x[1]+(-0.274347447112)*x[2]**o+(-0.280165537012)*x[2]
        arg[0,4]=(-0.323650660168)*x[0]**o+(-0.279592941595)*x[0]+(-0.478811425713)*x[1]**o+(0.19753271141)*x[1]+(0.476202875551)*x[2]**o+(0.657358100659)*x[2]
        arg[1,0]=(0.890659868437)*x[0]**o+(0.100235405989)*x[0]+(0.325177871418)*x[1]**o+(0.675379114656)*x[1]+(0.458473828724)*x[2]**o+(-0.449822467434)*x[2]
        arg[1,1]=(-0.467895039242)*x[0]**o+(-0.551300000422)*x[0]+(0.976111922801)*x[1]**o+(0.809389941274)*x[1]+(-0.693066235685)*x[2]**o+(0.595719115505)*x[2]
        arg[1,2]=(-0.727304617172)*x[0]**o+(-0.270682330445)*x[0]+(-0.32926621635)*x[1]**o+(0.0389068111165)*x[1]+(0.300096425532)*x[2]**o+(-0.964910767632)*x[2]
        arg[1,3]=(0.195012106644)*x[0]**o+(0.18137839607)*x[0]+(-0.823129279257)*x[1]**o+(-0.0440106889304)*x[1]+(-0.298898659898)*x[2]**o+(0.801623925974)*x[2]
        arg[1,4]=(0.590200326369)*x[0]**o+(-0.800085209049)*x[0]+(-0.323836133834)*x[1]**o+(-0.953413527643)*x[1]+(0.0257128844735)*x[2]**o+(0.652469516082)*x[2]
        arg[2,0]=(-0.467537281458)*x[0]**o+(-0.05673369912)*x[0]+(-0.813542619659)*x[1]**o+(-0.778596374878)*x[1]+(0.752073310476)*x[2]**o+(0.187824163254)*x[2]
        arg[2,1]=(-0.75708112983)*x[0]**o+(0.866111788737)*x[0]+(0.715424327432)*x[1]**o+(-0.297967739435)*x[1]+(0.420241611178)*x[2]**o+(-0.616867390634)*x[2]
        arg[2,2]=(0.00628628803839)*x[0]**o+(0.700962301988)*x[0]+(0.712979263919)*x[1]**o+(-0.414401132518)*x[1]+(-0.975498928756)*x[2]**o+(0.858116391039)*x[2]
        arg[2,3]=(0.948164399923)*x[0]**o+(-0.88227825142)*x[0]+(0.30703295864)*x[1]**o+(0.510607674984)*x[1]+(0.362361719392)*x[2]**o+(-0.208696380629)*x[2]
        arg[2,4]=(0.661545494115)*x[0]**o+(-0.122960355184)*x[0]+(-0.801367604574)*x[1]**o+(0.395708459987)*x[1]+(0.0197225340457)*x[2]**o+(-0.377005073637)*x[2]
        arg[3,0]=(0.733597350529)*x[0]**o+(0.339318978843)*x[0]+(0.317608635437)*x[1]**o+(0.147746051469)*x[1]+(-0.33413935488)*x[2]**o+(-0.793337674933)*x[2]
        arg[3,1]=(0.0918904281646)*x[0]**o+(-0.530786152204)*x[0]+(-0.587542750401)*x[1]**o+(-0.810331029621)*x[1]+(-0.40221065987)*x[2]**o+(-0.250472092093)*x[2]
        arg[3,2]=(0.966442952042)*x[0]**o+(-0.756649866552)*x[0]+(-0.36466223222)*x[1]**o+(-0.48639005055)*x[1]+(-0.831374462742)*x[2]**o+(0.503451143644)*x[2]
        arg[3,3]=(-0.322275644664)*x[0]**o+(-0.254067149982)*x[0]+(-0.440264794128)*x[1]**o+(0.226546045529)*x[1]+(-0.761870916789)*x[2]**o+(0.70024066379)*x[2]
        arg[3,4]=(0.595459342613)*x[0]**o+(-0.439436200258)*x[0]+(0.0932538180176)*x[1]**o+(-0.975450806707)*x[1]+(0.0461423323623)*x[2]**o+(0.0294352472154)*x[2]
        ref[0,0]=(-0.52872154973)/(o+1.)+(-0.177158421464)+(-0.937858877179)*0.5**o
        ref[0,1]=(-1.14234522986)/(o+1.)+(-0.414732890554)+(0.497785867084)*0.5**o
        ref[0,2]=(-1.71465154551)/(o+1.)+(0.0845977525197)+(0.744501879094)*0.5**o
        ref[0,3]=(0.374175031062)/(o+1.)+(-0.619505515252)+(0.210031467359)*0.5**o
        ref[0,4]=(-0.00260855016132)/(o+1.)+(0.287648935237)+(-0.323650660168)*0.5**o
        ref[1,0]=(0.783651700142)/(o+1.)+(0.162896026605)+(0.890659868437)*0.5**o
        ref[1,1]=(0.283045687117)/(o+1.)+(0.426904528178)+(-0.467895039242)*0.5**o
        ref[1,2]=(-0.0291697908188)/(o+1.)+(-0.59834314348)+(-0.727304617172)*0.5**o
        ref[1,3]=(-1.12202793916)/(o+1.)+(0.469495816557)+(0.195012106644)*0.5**o
        ref[1,4]=(-0.29812324936)/(o+1.)+(-0.550514610305)+(0.590200326369)*0.5**o
        ref[2,0]=(-0.0614693091826)/(o+1.)+(-0.323752955372)+(-0.467537281458)*0.5**o
        ref[2,1]=(1.13566593861)/(o+1.)+(-0.024361670666)+(-0.75708112983)*0.5**o
        ref[2,2]=(-0.262519664837)/(o+1.)+(0.572338780255)+(0.00628628803839)*0.5**o
        ref[2,3]=(0.669394678032)/(o+1.)+(-0.290183478532)+(0.948164399923)*0.5**o
        ref[2,4]=(-0.781645070528)/(o+1.)+(-0.0521284844165)+(0.661545494115)*0.5**o
        ref[3,0]=(-0.0165307194439)/(o+1.)+(-0.15313632231)+(0.733597350529)*0.5**o
        ref[3,1]=(-0.989753410271)/(o+1.)+(-0.795794636959)+(0.0918904281646)*0.5**o
        ref[3,2]=(-1.19603669496)/(o+1.)+(-0.369794386729)+(0.966442952042)*0.5**o
        ref[3,3]=(-1.20213571092)/(o+1.)+(0.336359779668)+(-0.322275644664)*0.5**o
        ref[3,4]=(0.13939615038)/(o+1.)+(-0.692725879875)+(0.595459342613)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactZero_fromData_Solution_rank3(self):
      """
      tests integral of rank 3 Data on the ReducedFunctionOnContactZero

      assumptions: Solution(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      w=Solution(self.domain)
      x=w.getX()
      arg=Data(0,(6, 2, 2),w)
      ref=numpy.zeros((6, 2, 2),numpy.float_)
      if dim==2:
        arg[0,0,0]=(-0.12237588236)*x[0]**o+(-0.671341785528)*x[0]+(0.998910161099)*x[1]**o+(-0.552290246966)*x[1]
        arg[0,0,1]=(-0.90821257221)*x[0]**o+(-0.413391927764)*x[0]+(-0.428904099491)*x[1]**o+(-0.226766222189)*x[1]
        arg[0,1,0]=(-0.333819313127)*x[0]**o+(-0.458569899029)*x[0]+(-0.83776177332)*x[1]**o+(0.22951184631)*x[1]
        arg[0,1,1]=(-0.0571879315489)*x[0]**o+(-0.38056227895)*x[0]+(0.0784628125977)*x[1]**o+(-0.659353613638)*x[1]
        arg[1,0,0]=(0.645871468353)*x[0]**o+(-0.986005542006)*x[0]+(0.395704509276)*x[1]**o+(0.980666918726)*x[1]
        arg[1,0,1]=(0.337293036597)*x[0]**o+(-0.520809490378)*x[0]+(0.543657498632)*x[1]**o+(0.632777244118)*x[1]
        arg[1,1,0]=(-0.432349538121)*x[0]**o+(-0.579685247099)*x[0]+(0.331943072726)*x[1]**o+(0.25042188644)*x[1]
        arg[1,1,1]=(0.329861800715)*x[0]**o+(-0.48725090895)*x[0]+(-0.149357998414)*x[1]**o+(-0.580650134153)*x[1]
        arg[2,0,0]=(-0.75424658357)*x[0]**o+(-0.696352517414)*x[0]+(-0.00272313166917)*x[1]**o+(-0.488581343546)*x[1]
        arg[2,0,1]=(-0.136489901339)*x[0]**o+(-0.752587775699)*x[0]+(0.986413392791)*x[1]**o+(0.759344728162)*x[1]
        arg[2,1,0]=(-0.0832674877963)*x[0]**o+(0.765419402778)*x[0]+(0.958533611564)*x[1]**o+(-0.315461136294)*x[1]
        arg[2,1,1]=(0.362764663642)*x[0]**o+(-0.351519282456)*x[0]+(-0.115554433923)*x[1]**o+(-0.789365036398)*x[1]
        arg[3,0,0]=(0.0977752933224)*x[0]**o+(-0.838881001576)*x[0]+(0.275578737811)*x[1]**o+(-0.971208155806)*x[1]
        arg[3,0,1]=(0.0588243034654)*x[0]**o+(-0.141302234145)*x[0]+(-0.460044396206)*x[1]**o+(-0.776793718354)*x[1]
        arg[3,1,0]=(0.786922930047)*x[0]**o+(-0.187797924361)*x[0]+(0.971516347812)*x[1]**o+(-0.250547377602)*x[1]
        arg[3,1,1]=(-0.69240453335)*x[0]**o+(0.439425578473)*x[0]+(0.00864940781276)*x[1]**o+(-0.458898387231)*x[1]
        arg[4,0,0]=(0.0700104829681)*x[0]**o+(0.905031121846)*x[0]+(0.37676750477)*x[1]**o+(0.411190149331)*x[1]
        arg[4,0,1]=(-0.615737714754)*x[0]**o+(0.1945204765)*x[0]+(0.0889014902938)*x[1]**o+(0.650020022597)*x[1]
        arg[4,1,0]=(-0.883448519911)*x[0]**o+(0.00124188371434)*x[0]+(-0.501662684156)*x[1]**o+(0.290235036398)*x[1]
        arg[4,1,1]=(-0.723885968554)*x[0]**o+(-0.0112308623755)*x[0]+(0.679499427805)*x[1]**o+(-0.00261690404287)*x[1]
        arg[5,0,0]=(0.0834731365594)*x[0]**o+(-0.873857991293)*x[0]+(0.831158842487)*x[1]**o+(-0.585808323324)*x[1]
        arg[5,0,1]=(0.250830078938)*x[0]**o+(0.310347394372)*x[0]+(-0.412138455827)*x[1]**o+(0.974823718029)*x[1]
        arg[5,1,0]=(-0.865155065381)*x[0]**o+(0.72190541012)*x[0]+(0.72020941875)*x[1]**o+(-0.604997858133)*x[1]
        arg[5,1,1]=(0.57252234131)*x[0]**o+(-0.366230226771)*x[0]+(0.115400105646)*x[1]**o+(0.0485229323141)*x[1]
        ref[0,0,0]=(0.998910161099)/(o+1.)+(-0.611816016247)+(-0.12237588236)*0.5**o
        ref[0,0,1]=(-0.428904099491)/(o+1.)+(-0.320079074977)+(-0.90821257221)*0.5**o
        ref[0,1,0]=(-0.83776177332)/(o+1.)+(-0.11452902636)+(-0.333819313127)*0.5**o
        ref[0,1,1]=(0.0784628125977)/(o+1.)+(-0.519957946294)+(-0.0571879315489)*0.5**o
        ref[1,0,0]=(0.395704509276)/(o+1.)+(-0.00266931163989)+(0.645871468353)*0.5**o
        ref[1,0,1]=(0.543657498632)/(o+1.)+(0.0559838768697)+(0.337293036597)*0.5**o
        ref[1,1,0]=(0.331943072726)/(o+1.)+(-0.16463168033)+(-0.432349538121)*0.5**o
        ref[1,1,1]=(-0.149357998414)/(o+1.)+(-0.533950521552)+(0.329861800715)*0.5**o
        ref[2,0,0]=(-0.00272313166917)/(o+1.)+(-0.59246693048)+(-0.75424658357)*0.5**o
        ref[2,0,1]=(0.986413392791)/(o+1.)+(0.00337847623164)+(-0.136489901339)*0.5**o
        ref[2,1,0]=(0.958533611564)/(o+1.)+(0.224979133242)+(-0.0832674877963)*0.5**o
        ref[2,1,1]=(-0.115554433923)/(o+1.)+(-0.570442159427)+(0.362764663642)*0.5**o
        ref[3,0,0]=(0.275578737811)/(o+1.)+(-0.905044578691)+(0.0977752933224)*0.5**o
        ref[3,0,1]=(-0.460044396206)/(o+1.)+(-0.45904797625)+(0.0588243034654)*0.5**o
        ref[3,1,0]=(0.971516347812)/(o+1.)+(-0.219172650981)+(0.786922930047)*0.5**o
        ref[3,1,1]=(0.00864940781276)/(o+1.)+(-0.00973640437899)+(-0.69240453335)*0.5**o
        ref[4,0,0]=(0.37676750477)/(o+1.)+(0.658110635589)+(0.0700104829681)*0.5**o
        ref[4,0,1]=(0.0889014902938)/(o+1.)+(0.422270249549)+(-0.615737714754)*0.5**o
        ref[4,1,0]=(-0.501662684156)/(o+1.)+(0.145738460056)+(-0.883448519911)*0.5**o
        ref[4,1,1]=(0.679499427805)/(o+1.)+(-0.00692388320921)+(-0.723885968554)*0.5**o
        ref[5,0,0]=(0.831158842487)/(o+1.)+(-0.729833157309)+(0.0834731365594)*0.5**o
        ref[5,0,1]=(-0.412138455827)/(o+1.)+(0.642585556201)+(0.250830078938)*0.5**o
        ref[5,1,0]=(0.72020941875)/(o+1.)+(0.0584537759935)+(-0.865155065381)*0.5**o
        ref[5,1,1]=(0.115400105646)/(o+1.)+(-0.158853647228)+(0.57252234131)*0.5**o
      else:
        arg[0,0,0]=(0.962792187023)*x[0]**o+(0.799861778652)*x[0]+(-0.811367630134)*x[1]**o+(-0.553496861095)*x[1]+(-0.810089307714)*x[2]**o+(0.135449765364)*x[2]
        arg[0,0,1]=(0.644573004654)*x[0]**o+(-0.565823623798)*x[0]+(0.337026645127)*x[1]**o+(0.413612857799)*x[1]+(0.604366347987)*x[2]**o+(0.0867373815054)*x[2]
        arg[0,1,0]=(0.829914553754)*x[0]**o+(-0.462419132852)*x[0]+(0.889464473044)*x[1]**o+(-0.425594549042)*x[1]+(0.679251542389)*x[2]**o+(-0.501090535615)*x[2]
        arg[0,1,1]=(0.874166303558)*x[0]**o+(0.437595762221)*x[0]+(-0.836229998654)*x[1]**o+(-0.704020633222)*x[1]+(-0.279878325513)*x[2]**o+(0.697289463565)*x[2]
        arg[1,0,0]=(-0.963425479949)*x[0]**o+(0.399090631945)*x[0]+(-0.80371987894)*x[1]**o+(0.614102223184)*x[1]+(-0.821741166107)*x[2]**o+(-0.245209306654)*x[2]
        arg[1,0,1]=(-0.288791346674)*x[0]**o+(0.0125896799209)*x[0]+(0.0536290891541)*x[1]**o+(-0.127968165901)*x[1]+(-0.635591402779)*x[2]**o+(-0.723850350215)*x[2]
        arg[1,1,0]=(-0.554196270608)*x[0]**o+(-0.409293351346)*x[0]+(-0.759821124187)*x[1]**o+(-0.828913354317)*x[1]+(0.66900958614)*x[2]**o+(0.412403847239)*x[2]
        arg[1,1,1]=(-0.862735330269)*x[0]**o+(-0.406644646366)*x[0]+(-0.375399872184)*x[1]**o+(-0.10762837777)*x[1]+(0.540886844249)*x[2]**o+(0.97487281528)*x[2]
        arg[2,0,0]=(0.388899900771)*x[0]**o+(-0.692092360652)*x[0]+(0.794415524913)*x[1]**o+(0.369239170999)*x[1]+(0.482269641926)*x[2]**o+(0.58717167121)*x[2]
        arg[2,0,1]=(0.324795219208)*x[0]**o+(0.508631698719)*x[0]+(0.128456415737)*x[1]**o+(0.73678163559)*x[1]+(0.0527520660005)*x[2]**o+(-0.180289299372)*x[2]
        arg[2,1,0]=(-0.901972287405)*x[0]**o+(-0.635701120264)*x[0]+(0.298343193246)*x[1]**o+(-0.870902858159)*x[1]+(-0.979619200422)*x[2]**o+(0.708567755329)*x[2]
        arg[2,1,1]=(0.391663724321)*x[0]**o+(0.69891810611)*x[0]+(0.717069638885)*x[1]**o+(0.493487853359)*x[1]+(0.270536570073)*x[2]**o+(0.679115502666)*x[2]
        arg[3,0,0]=(0.830590943609)*x[0]**o+(-0.000320739222143)*x[0]+(-0.659146599349)*x[1]**o+(-0.858428982679)*x[1]+(-0.158070494634)*x[2]**o+(-0.364394913197)*x[2]
        arg[3,0,1]=(0.903615175916)*x[0]**o+(0.186562406505)*x[0]+(0.463860110418)*x[1]**o+(0.0382550628307)*x[1]+(0.835148180357)*x[2]**o+(-0.117637969912)*x[2]
        arg[3,1,0]=(-0.901874408714)*x[0]**o+(0.00493918499294)*x[0]+(0.180179475909)*x[1]**o+(-0.549428619271)*x[1]+(-0.392536708051)*x[2]**o+(0.68054148024)*x[2]
        arg[3,1,1]=(0.807693382033)*x[0]**o+(-0.47532617097)*x[0]+(-0.52499678021)*x[1]**o+(-0.0365395713918)*x[1]+(0.654881606829)*x[2]**o+(-0.63350736651)*x[2]
        arg[4,0,0]=(0.440830144989)*x[0]**o+(-0.454694325046)*x[0]+(0.403509667018)*x[1]**o+(0.360080283937)*x[1]+(0.164208775222)*x[2]**o+(0.203945755083)*x[2]
        arg[4,0,1]=(-0.836804887763)*x[0]**o+(-0.910979763109)*x[0]+(-0.629411293881)*x[1]**o+(0.0546020113997)*x[1]+(0.63435754491)*x[2]**o+(-0.323224122475)*x[2]
        arg[4,1,0]=(0.204216251019)*x[0]**o+(-0.372349067194)*x[0]+(-0.320768886225)*x[1]**o+(-0.139715856072)*x[1]+(0.62329130659)*x[2]**o+(0.626784461235)*x[2]
        arg[4,1,1]=(0.275074685897)*x[0]**o+(0.0664691742991)*x[0]+(0.949175504221)*x[1]**o+(-0.438858093561)*x[1]+(0.330489174462)*x[2]**o+(0.361586647545)*x[2]
        arg[5,0,0]=(0.370194889673)*x[0]**o+(-0.0549428893157)*x[0]+(0.0219932528442)*x[1]**o+(-0.699553487775)*x[1]+(0.0161212665659)*x[2]**o+(-0.268426682266)*x[2]
        arg[5,0,1]=(-0.527801267225)*x[0]**o+(0.151774276637)*x[0]+(-0.0623666836223)*x[1]**o+(-0.953572390482)*x[1]+(0.778912307756)*x[2]**o+(-0.851760269136)*x[2]
        arg[5,1,0]=(0.11418535486)*x[0]**o+(-0.299719975234)*x[0]+(-0.508303365848)*x[1]**o+(-0.856367302992)*x[1]+(0.311609370032)*x[2]**o+(0.4731881155)*x[2]
        arg[5,1,1]=(0.455947680647)*x[0]**o+(0.856811965126)*x[0]+(-0.176215162097)*x[1]**o+(0.581325693162)*x[1]+(-0.306514780798)*x[2]**o+(-0.005241787834)*x[2]
        ref[0,0,0]=(-1.62145693785)/(o+1.)+(0.19090734146)+(0.962792187023)*0.5**o
        ref[0,0,1]=(0.941392993115)/(o+1.)+(-0.0327366922471)+(0.644573004654)*0.5**o
        ref[0,1,0]=(1.56871601543)/(o+1.)+(-0.694552108754)+(0.829914553754)*0.5**o
        ref[0,1,1]=(-1.11610832417)/(o+1.)+(0.215432296282)+(0.874166303558)*0.5**o
        ref[1,0,0]=(-1.62546104505)/(o+1.)+(0.383991774237)+(-0.963425479949)*0.5**o
        ref[1,0,1]=(-0.581962313625)/(o+1.)+(-0.419614418097)+(-0.288791346674)*0.5**o
        ref[1,1,0]=(-0.0908115380472)/(o+1.)+(-0.412901429212)+(-0.554196270608)*0.5**o
        ref[1,1,1]=(0.165486972066)/(o+1.)+(0.230299895572)+(-0.862735330269)*0.5**o
        ref[2,0,0]=(1.27668516684)/(o+1.)+(0.132159240779)+(0.388899900771)*0.5**o
        ref[2,0,1]=(0.181208481738)/(o+1.)+(0.532562017469)+(0.324795219208)*0.5**o
        ref[2,1,0]=(-0.681276007176)/(o+1.)+(-0.399018111547)+(-0.901972287405)*0.5**o
        ref[2,1,1]=(0.987606208958)/(o+1.)+(0.935760731067)+(0.391663724321)*0.5**o
        ref[3,0,0]=(-0.817217093983)/(o+1.)+(-0.611572317549)+(0.830590943609)*0.5**o
        ref[3,0,1]=(1.29900829077)/(o+1.)+(0.0535897497116)+(0.903615175916)*0.5**o
        ref[3,1,0]=(-0.212357232141)/(o+1.)+(0.0680260229807)+(-0.901874408714)*0.5**o
        ref[3,1,1]=(0.12988482662)/(o+1.)+(-0.572686554436)+(0.807693382033)*0.5**o
        ref[4,0,0]=(0.567718442239)/(o+1.)+(0.0546658569873)+(0.440830144989)*0.5**o
        ref[4,0,1]=(0.00494625102884)/(o+1.)+(-0.589800937092)+(-0.836804887763)*0.5**o
        ref[4,1,0]=(0.302522420366)/(o+1.)+(0.0573597689848)+(0.204216251019)*0.5**o
        ref[4,1,1]=(1.27966467868)/(o+1.)+(-0.0054011358584)+(0.275074685897)*0.5**o
        ref[5,0,0]=(0.0381145194101)/(o+1.)+(-0.511461529678)+(0.370194889673)*0.5**o
        ref[5,0,1]=(0.716545624134)/(o+1.)+(-0.826779191491)+(-0.527801267225)*0.5**o
        ref[5,1,0]=(-0.196693995816)/(o+1.)+(-0.341449581363)+(0.11418535486)*0.5**o
        ref[5,1,1]=(-0.482729942895)/(o+1.)+(0.716447935227)+(0.455947680647)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactZero_fromData_Solution_rank4(self):
      """
      tests integral of rank 4 Data on the ReducedFunctionOnContactZero

      assumptions: Solution(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      w=Solution(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5, 3, 2),w)
      ref=numpy.zeros((4, 5, 3, 2),numpy.float_)
      if dim==2:
        arg[0,0,0,0]=(0.415003066171)*x[0]**o+(-0.47441250281)*x[0]+(0.765391656872)*x[1]**o+(-0.170257448926)*x[1]
        arg[0,0,0,1]=(-0.928539602254)*x[0]**o+(0.792240012243)*x[0]+(-0.152730990885)*x[1]**o+(-0.667500454847)*x[1]
        arg[0,0,1,0]=(-0.937831078575)*x[0]**o+(0.963649346531)*x[0]+(0.505558859525)*x[1]**o+(-0.921354612243)*x[1]
        arg[0,0,1,1]=(0.909415074451)*x[0]**o+(0.199360871726)*x[0]+(0.194087349442)*x[1]**o+(0.875065381425)*x[1]
        arg[0,0,2,0]=(0.935190448625)*x[0]**o+(-0.109984767478)*x[0]+(0.518460161897)*x[1]**o+(-0.597684157349)*x[1]
        arg[0,0,2,1]=(-0.94062714999)*x[0]**o+(-0.308960574286)*x[0]+(0.578391007321)*x[1]**o+(0.864501040998)*x[1]
        arg[0,1,0,0]=(-0.197219678026)*x[0]**o+(-0.982899373778)*x[0]+(-0.603956105825)*x[1]**o+(-0.242717148913)*x[1]
        arg[0,1,0,1]=(-0.590715606611)*x[0]**o+(0.461185487788)*x[0]+(-0.270612249495)*x[1]**o+(-0.644742980654)*x[1]
        arg[0,1,1,0]=(-0.365516638755)*x[0]**o+(-0.255307471195)*x[0]+(0.0875989587033)*x[1]**o+(0.923189427818)*x[1]
        arg[0,1,1,1]=(0.315218558548)*x[0]**o+(-0.373070897551)*x[0]+(0.625272530385)*x[1]**o+(-0.174054543645)*x[1]
        arg[0,1,2,0]=(-0.502497884169)*x[0]**o+(-0.933785405889)*x[0]+(0.115500534905)*x[1]**o+(-0.864454092408)*x[1]
        arg[0,1,2,1]=(0.615837581258)*x[0]**o+(0.832538142367)*x[0]+(-0.0715650027129)*x[1]**o+(-0.775173801322)*x[1]
        arg[0,2,0,0]=(-0.898904577889)*x[0]**o+(-0.430072579352)*x[0]+(-0.532213103734)*x[1]**o+(0.970885652167)*x[1]
        arg[0,2,0,1]=(-0.0056483770747)*x[0]**o+(-0.566550554106)*x[0]+(0.751663691868)*x[1]**o+(0.677878661708)*x[1]
        arg[0,2,1,0]=(0.271196923039)*x[0]**o+(-0.291608730386)*x[0]+(-0.932579716748)*x[1]**o+(-0.437823650176)*x[1]
        arg[0,2,1,1]=(-0.788206773125)*x[0]**o+(0.18675055747)*x[0]+(0.747625244884)*x[1]**o+(0.654933611758)*x[1]
        arg[0,2,2,0]=(-0.787603701957)*x[0]**o+(-0.789784850865)*x[0]+(0.197684766271)*x[1]**o+(-0.988404958072)*x[1]
        arg[0,2,2,1]=(0.417538087399)*x[0]**o+(0.745832250585)*x[0]+(-0.525306115268)*x[1]**o+(0.134086803871)*x[1]
        arg[0,3,0,0]=(-0.230658480436)*x[0]**o+(-0.072983085881)*x[0]+(-0.149386157207)*x[1]**o+(-0.506870604002)*x[1]
        arg[0,3,0,1]=(-0.815859549425)*x[0]**o+(-0.962632495033)*x[0]+(-0.542257139282)*x[1]**o+(0.93598141472)*x[1]
        arg[0,3,1,0]=(0.0508974791413)*x[0]**o+(-0.722669867743)*x[0]+(0.714019527678)*x[1]**o+(0.710015283752)*x[1]
        arg[0,3,1,1]=(0.119419200166)*x[0]**o+(0.365663074152)*x[0]+(0.352455072404)*x[1]**o+(-0.0788953701144)*x[1]
        arg[0,3,2,0]=(-0.82075904747)*x[0]**o+(-0.224534176808)*x[0]+(-0.585094684368)*x[1]**o+(0.242958171035)*x[1]
        arg[0,3,2,1]=(-0.748061395205)*x[0]**o+(0.676995891547)*x[0]+(0.686644239408)*x[1]**o+(0.260350484065)*x[1]
        arg[0,4,0,0]=(0.128293903485)*x[0]**o+(0.226748407809)*x[0]+(0.129373873322)*x[1]**o+(0.988775321025)*x[1]
        arg[0,4,0,1]=(0.639638672694)*x[0]**o+(-0.817791454136)*x[0]+(-0.170677596319)*x[1]**o+(0.404161643274)*x[1]
        arg[0,4,1,0]=(0.43743097727)*x[0]**o+(-0.408230274495)*x[0]+(0.282544628333)*x[1]**o+(-0.542352697967)*x[1]
        arg[0,4,1,1]=(-0.995599965283)*x[0]**o+(-0.0512373236039)*x[0]+(0.218383977926)*x[1]**o+(0.929355335423)*x[1]
        arg[0,4,2,0]=(0.922609534479)*x[0]**o+(-0.166291574079)*x[0]+(-0.449331655188)*x[1]**o+(-0.0279347569009)*x[1]
        arg[0,4,2,1]=(0.950186933361)*x[0]**o+(-0.426615341822)*x[0]+(-0.167639663907)*x[1]**o+(0.219020502174)*x[1]
        arg[1,0,0,0]=(0.0573632292788)*x[0]**o+(0.543681035865)*x[0]+(-0.622510875807)*x[1]**o+(-0.171567467655)*x[1]
        arg[1,0,0,1]=(0.368975600444)*x[0]**o+(-0.754217532725)*x[0]+(0.469744047929)*x[1]**o+(-0.922445394777)*x[1]
        arg[1,0,1,0]=(-0.810194964715)*x[0]**o+(0.0861796117934)*x[0]+(0.625367069433)*x[1]**o+(-0.950906871025)*x[1]
        arg[1,0,1,1]=(0.744923275143)*x[0]**o+(0.465744045178)*x[0]+(0.359324627118)*x[1]**o+(0.273934205663)*x[1]
        arg[1,0,2,0]=(0.891959004941)*x[0]**o+(-0.498241249484)*x[0]+(0.318198109895)*x[1]**o+(0.0674564169104)*x[1]
        arg[1,0,2,1]=(0.842125439023)*x[0]**o+(0.731884782944)*x[0]+(-0.674805144805)*x[1]**o+(0.431782566319)*x[1]
        arg[1,1,0,0]=(-0.786719774805)*x[0]**o+(-0.55929144961)*x[0]+(-0.740263211102)*x[1]**o+(0.777914467998)*x[1]
        arg[1,1,0,1]=(0.831245977993)*x[0]**o+(-0.758991188073)*x[0]+(0.714032957488)*x[1]**o+(-0.879935556766)*x[1]
        arg[1,1,1,0]=(0.862796574462)*x[0]**o+(0.746875042565)*x[0]+(0.455949140326)*x[1]**o+(0.585491946591)*x[1]
        arg[1,1,1,1]=(-0.500409398892)*x[0]**o+(0.292753163538)*x[0]+(-0.62474682088)*x[1]**o+(0.583368504524)*x[1]
        arg[1,1,2,0]=(-0.402886961189)*x[0]**o+(-0.288733078165)*x[0]+(-0.770315341866)*x[1]**o+(-0.462764773299)*x[1]
        arg[1,1,2,1]=(0.358161520677)*x[0]**o+(-0.524052652951)*x[0]+(0.654162491323)*x[1]**o+(0.902382016998)*x[1]
        arg[1,2,0,0]=(-0.15539934232)*x[0]**o+(0.693162750731)*x[0]+(-0.0538737948754)*x[1]**o+(-0.025534990068)*x[1]
        arg[1,2,0,1]=(0.371159212282)*x[0]**o+(-0.751292369171)*x[0]+(-0.377617770006)*x[1]**o+(0.707647483842)*x[1]
        arg[1,2,1,0]=(0.117790556271)*x[0]**o+(0.848017682445)*x[0]+(-0.360936107893)*x[1]**o+(-0.564060873236)*x[1]
        arg[1,2,1,1]=(-0.524828128971)*x[0]**o+(0.737876348154)*x[0]+(0.183316445987)*x[1]**o+(-0.821207746237)*x[1]
        arg[1,2,2,0]=(0.300516177535)*x[0]**o+(-0.204187510383)*x[0]+(0.126601242737)*x[1]**o+(0.0794834423875)*x[1]
        arg[1,2,2,1]=(-0.29046926624)*x[0]**o+(-0.310057692431)*x[0]+(-0.810227948574)*x[1]**o+(-0.464128160277)*x[1]
        arg[1,3,0,0]=(-0.134275834453)*x[0]**o+(0.448577162559)*x[0]+(0.757723452446)*x[1]**o+(0.139202834725)*x[1]
        arg[1,3,0,1]=(0.882915728842)*x[0]**o+(-0.0750591947315)*x[0]+(0.503482692135)*x[1]**o+(-0.176350465386)*x[1]
        arg[1,3,1,0]=(-0.954792910945)*x[0]**o+(0.772215353622)*x[0]+(-0.861747803918)*x[1]**o+(-0.932727438775)*x[1]
        arg[1,3,1,1]=(0.747255393187)*x[0]**o+(0.0392661297355)*x[0]+(-0.322274864209)*x[1]**o+(-0.491462177972)*x[1]
        arg[1,3,2,0]=(0.139287057836)*x[0]**o+(0.312258185231)*x[0]+(-0.488288412308)*x[1]**o+(-0.185131700928)*x[1]
        arg[1,3,2,1]=(-0.861625516304)*x[0]**o+(0.681454010645)*x[0]+(0.206644380777)*x[1]**o+(-0.0344138929268)*x[1]
        arg[1,4,0,0]=(-0.714198122614)*x[0]**o+(-0.413113909739)*x[0]+(-0.971680855409)*x[1]**o+(0.602598611545)*x[1]
        arg[1,4,0,1]=(0.293578109913)*x[0]**o+(-0.495635685351)*x[0]+(0.347408878778)*x[1]**o+(-0.923814185786)*x[1]
        arg[1,4,1,0]=(0.843760126513)*x[0]**o+(-0.671075074679)*x[0]+(-0.284086603407)*x[1]**o+(0.437492233496)*x[1]
        arg[1,4,1,1]=(-0.342911357374)*x[0]**o+(-0.440232224635)*x[0]+(0.537701123084)*x[1]**o+(-0.525203729082)*x[1]
        arg[1,4,2,0]=(-0.71201360314)*x[0]**o+(-0.706332525171)*x[0]+(0.831438049852)*x[1]**o+(0.819141670055)*x[1]
        arg[1,4,2,1]=(-0.616403541928)*x[0]**o+(0.635973739636)*x[0]+(-0.0369972206543)*x[1]**o+(-0.08595692743)*x[1]
        arg[2,0,0,0]=(-0.959754423491)*x[0]**o+(-0.332473557912)*x[0]+(-0.464334304695)*x[1]**o+(0.597214020843)*x[1]
        arg[2,0,0,1]=(0.231051530608)*x[0]**o+(0.368524293555)*x[0]+(0.173044245392)*x[1]**o+(0.505867390994)*x[1]
        arg[2,0,1,0]=(0.376577055194)*x[0]**o+(-0.739990982509)*x[0]+(-0.816201949845)*x[1]**o+(-0.71233193641)*x[1]
        arg[2,0,1,1]=(-0.4891837531)*x[0]**o+(-0.121562226208)*x[0]+(-0.876294548464)*x[1]**o+(0.377763924093)*x[1]
        arg[2,0,2,0]=(-0.759548081841)*x[0]**o+(-0.118749434442)*x[0]+(0.602375475355)*x[1]**o+(-0.877213577079)*x[1]
        arg[2,0,2,1]=(0.114018259802)*x[0]**o+(-0.627149204501)*x[0]+(0.813648191424)*x[1]**o+(-0.612548616323)*x[1]
        arg[2,1,0,0]=(-0.572699800518)*x[0]**o+(0.507715829662)*x[0]+(-0.12122456959)*x[1]**o+(0.75522878228)*x[1]
        arg[2,1,0,1]=(0.0655268452799)*x[0]**o+(0.211191572823)*x[0]+(0.539760655497)*x[1]**o+(0.657397992057)*x[1]
        arg[2,1,1,0]=(0.358281119336)*x[0]**o+(0.119364032507)*x[0]+(-0.0242249415295)*x[1]**o+(-0.645986730609)*x[1]
        arg[2,1,1,1]=(0.523751704889)*x[0]**o+(-0.237670103222)*x[0]+(-0.995787126907)*x[1]**o+(-0.504129088442)*x[1]
        arg[2,1,2,0]=(-0.185380508646)*x[0]**o+(-0.218341933583)*x[0]+(0.149400154478)*x[1]**o+(0.00252616040715)*x[1]
        arg[2,1,2,1]=(0.81086932122)*x[0]**o+(0.387271527481)*x[0]+(0.908525646396)*x[1]**o+(-0.75383394204)*x[1]
        arg[2,2,0,0]=(-0.463416100256)*x[0]**o+(-0.891654021409)*x[0]+(-0.0495088159461)*x[1]**o+(0.398915277911)*x[1]
        arg[2,2,0,1]=(-0.200698149193)*x[0]**o+(0.127899540753)*x[0]+(-0.895280577411)*x[1]**o+(-0.197025422242)*x[1]
        arg[2,2,1,0]=(0.888683772867)*x[0]**o+(-0.84436563509)*x[0]+(-0.375709672588)*x[1]**o+(0.591336312818)*x[1]
        arg[2,2,1,1]=(-0.0167949020901)*x[0]**o+(0.145017501865)*x[0]+(0.868194479789)*x[1]**o+(-0.62439676328)*x[1]
        arg[2,2,2,0]=(0.441981698578)*x[0]**o+(-0.481156166596)*x[0]+(0.267937553127)*x[1]**o+(-0.37032246733)*x[1]
        arg[2,2,2,1]=(0.876701424579)*x[0]**o+(0.473798212378)*x[0]+(0.697645775876)*x[1]**o+(-0.798143376838)*x[1]
        arg[2,3,0,0]=(0.111231509217)*x[0]**o+(0.188121806171)*x[0]+(-0.0706155476337)*x[1]**o+(0.779423336221)*x[1]
        arg[2,3,0,1]=(-0.858913402812)*x[0]**o+(0.655459361097)*x[0]+(0.326282512878)*x[1]**o+(0.863386952614)*x[1]
        arg[2,3,1,0]=(-0.524551221953)*x[0]**o+(-0.807242415277)*x[0]+(0.575866251831)*x[1]**o+(-0.74503615846)*x[1]
        arg[2,3,1,1]=(0.391514075129)*x[0]**o+(0.477257809825)*x[0]+(-0.488263373008)*x[1]**o+(-0.29101442481)*x[1]
        arg[2,3,2,0]=(0.26532129161)*x[0]**o+(0.22794872765)*x[0]+(0.85094548724)*x[1]**o+(-0.369690624817)*x[1]
        arg[2,3,2,1]=(0.2339994493)*x[0]**o+(-0.901432708956)*x[0]+(0.309349741876)*x[1]**o+(0.132010344317)*x[1]
        arg[2,4,0,0]=(-0.713553783036)*x[0]**o+(0.791484993301)*x[0]+(0.339247480459)*x[1]**o+(0.158842307171)*x[1]
        arg[2,4,0,1]=(-0.733490022564)*x[0]**o+(0.631284830844)*x[0]+(0.340009298488)*x[1]**o+(-0.483162310574)*x[1]
        arg[2,4,1,0]=(0.709413437683)*x[0]**o+(0.896590124659)*x[0]+(0.680862759763)*x[1]**o+(-0.23579845663)*x[1]
        arg[2,4,1,1]=(-0.130761551034)*x[0]**o+(0.269095282394)*x[0]+(0.687201138134)*x[1]**o+(0.905183436376)*x[1]
        arg[2,4,2,0]=(0.256894911335)*x[0]**o+(0.706744915172)*x[0]+(0.910504833397)*x[1]**o+(-0.285088297895)*x[1]
        arg[2,4,2,1]=(0.95137247728)*x[0]**o+(-0.924669436646)*x[0]+(-0.146356272875)*x[1]**o+(-0.673326194162)*x[1]
        arg[3,0,0,0]=(0.663180444534)*x[0]**o+(0.894728781279)*x[0]+(0.894383747285)*x[1]**o+(0.969779862148)*x[1]
        arg[3,0,0,1]=(0.424556283891)*x[0]**o+(-0.97849198275)*x[0]+(0.367908872519)*x[1]**o+(0.624098355956)*x[1]
        arg[3,0,1,0]=(0.908698580251)*x[0]**o+(-0.899102004477)*x[0]+(0.509310001763)*x[1]**o+(-0.8399697646)*x[1]
        arg[3,0,1,1]=(0.523444225814)*x[0]**o+(0.410650511976)*x[0]+(-0.986699126405)*x[1]**o+(0.081494682537)*x[1]
        arg[3,0,2,0]=(-0.163604985314)*x[0]**o+(0.959338619715)*x[0]+(-0.843900271478)*x[1]**o+(-0.0218386544947)*x[1]
        arg[3,0,2,1]=(-0.490804271432)*x[0]**o+(0.123332672938)*x[0]+(0.902072835174)*x[1]**o+(-0.202340709957)*x[1]
        arg[3,1,0,0]=(-0.949731382685)*x[0]**o+(0.533695608969)*x[0]+(0.430512336347)*x[1]**o+(0.977434372776)*x[1]
        arg[3,1,0,1]=(0.945102298088)*x[0]**o+(-0.117807763531)*x[0]+(-0.203921504027)*x[1]**o+(-0.261266079346)*x[1]
        arg[3,1,1,0]=(0.762442770795)*x[0]**o+(-0.318456087863)*x[0]+(-0.365021977395)*x[1]**o+(0.0516932816311)*x[1]
        arg[3,1,1,1]=(-0.829061607399)*x[0]**o+(0.854900781365)*x[0]+(-0.397139314214)*x[1]**o+(-0.549679351555)*x[1]
        arg[3,1,2,0]=(-0.780040060687)*x[0]**o+(0.719393460042)*x[0]+(-0.435377419129)*x[1]**o+(0.655962063649)*x[1]
        arg[3,1,2,1]=(-0.853144390228)*x[0]**o+(0.0940222442927)*x[0]+(-0.642258121264)*x[1]**o+(-0.615386923437)*x[1]
        arg[3,2,0,0]=(0.377161814597)*x[0]**o+(0.349238572634)*x[0]+(0.361300611554)*x[1]**o+(0.554382639639)*x[1]
        arg[3,2,0,1]=(-0.51174671413)*x[0]**o+(-0.0586248632463)*x[0]+(0.083224329707)*x[1]**o+(0.506614699443)*x[1]
        arg[3,2,1,0]=(0.612695079311)*x[0]**o+(0.82053876378)*x[0]+(-0.132608876197)*x[1]**o+(0.701720563108)*x[1]
        arg[3,2,1,1]=(0.0928734170244)*x[0]**o+(-0.127564289866)*x[0]+(0.68468196965)*x[1]**o+(0.0168958305852)*x[1]
        arg[3,2,2,0]=(-0.863542893122)*x[0]**o+(0.0533007397457)*x[0]+(-0.0602194449612)*x[1]**o+(0.391122261927)*x[1]
        arg[3,2,2,1]=(-0.0942537498633)*x[0]**o+(-0.421741120475)*x[0]+(-0.0746938765947)*x[1]**o+(0.988111802567)*x[1]
        arg[3,3,0,0]=(-0.0588913579205)*x[0]**o+(0.439330145374)*x[0]+(-0.541714868844)*x[1]**o+(0.845344272279)*x[1]
        arg[3,3,0,1]=(0.221784155922)*x[0]**o+(0.983539923382)*x[0]+(-0.32332836636)*x[1]**o+(-0.892579743718)*x[1]
        arg[3,3,1,0]=(0.804097639203)*x[0]**o+(-0.342072604164)*x[0]+(0.755681096012)*x[1]**o+(0.432897246925)*x[1]
        arg[3,3,1,1]=(0.825696602707)*x[0]**o+(0.536999112965)*x[0]+(0.745033643035)*x[1]**o+(0.156806358776)*x[1]
        arg[3,3,2,0]=(-0.372114391352)*x[0]**o+(0.670417556114)*x[0]+(-0.569395510546)*x[1]**o+(-0.523784408534)*x[1]
        arg[3,3,2,1]=(0.439723359721)*x[0]**o+(0.809733189174)*x[0]+(0.781753556415)*x[1]**o+(0.375991547044)*x[1]
        arg[3,4,0,0]=(-0.429896412297)*x[0]**o+(-0.908820042346)*x[0]+(0.328787484151)*x[1]**o+(-0.610488251755)*x[1]
        arg[3,4,0,1]=(0.803567111353)*x[0]**o+(-0.487219379817)*x[0]+(-0.541538468561)*x[1]**o+(-0.692964506777)*x[1]
        arg[3,4,1,0]=(-0.963266835039)*x[0]**o+(0.667510333677)*x[0]+(0.308312799035)*x[1]**o+(0.686605377458)*x[1]
        arg[3,4,1,1]=(0.651198604373)*x[0]**o+(0.306465142929)*x[0]+(-0.923691546947)*x[1]**o+(-0.376240035765)*x[1]
        arg[3,4,2,0]=(-0.339406539705)*x[0]**o+(0.524709759622)*x[0]+(-0.0326564953763)*x[1]**o+(-0.658296780229)*x[1]
        arg[3,4,2,1]=(0.471355209305)*x[0]**o+(-0.473756698531)*x[0]+(0.193890658409)*x[1]**o+(-0.266632530307)*x[1]
        ref[0,0,0,0]=(0.765391656872)/(o+1.)+(-0.322334975868)+(0.415003066171)*0.5**o
        ref[0,0,0,1]=(-0.152730990885)/(o+1.)+(0.0623697786981)+(-0.928539602254)*0.5**o
        ref[0,0,1,0]=(0.505558859525)/(o+1.)+(0.0211473671443)+(-0.937831078575)*0.5**o
        ref[0,0,1,1]=(0.194087349442)/(o+1.)+(0.537213126576)+(0.909415074451)*0.5**o
        ref[0,0,2,0]=(0.518460161897)/(o+1.)+(-0.353834462413)+(0.935190448625)*0.5**o
        ref[0,0,2,1]=(0.578391007321)/(o+1.)+(0.277770233356)+(-0.94062714999)*0.5**o
        ref[0,1,0,0]=(-0.603956105825)/(o+1.)+(-0.612808261345)+(-0.197219678026)*0.5**o
        ref[0,1,0,1]=(-0.270612249495)/(o+1.)+(-0.0917787464331)+(-0.590715606611)*0.5**o
        ref[0,1,1,0]=(0.0875989587033)/(o+1.)+(0.333940978312)+(-0.365516638755)*0.5**o
        ref[0,1,1,1]=(0.625272530385)/(o+1.)+(-0.273562720598)+(0.315218558548)*0.5**o
        ref[0,1,2,0]=(0.115500534905)/(o+1.)+(-0.899119749149)+(-0.502497884169)*0.5**o
        ref[0,1,2,1]=(-0.0715650027129)/(o+1.)+(0.0286821705227)+(0.615837581258)*0.5**o
        ref[0,2,0,0]=(-0.532213103734)/(o+1.)+(0.270406536408)+(-0.898904577889)*0.5**o
        ref[0,2,0,1]=(0.751663691868)/(o+1.)+(0.0556640538013)+(-0.0056483770747)*0.5**o
        ref[0,2,1,0]=(-0.932579716748)/(o+1.)+(-0.364716190281)+(0.271196923039)*0.5**o
        ref[0,2,1,1]=(0.747625244884)/(o+1.)+(0.420842084614)+(-0.788206773125)*0.5**o
        ref[0,2,2,0]=(0.197684766271)/(o+1.)+(-0.889094904468)+(-0.787603701957)*0.5**o
        ref[0,2,2,1]=(-0.525306115268)/(o+1.)+(0.439959527228)+(0.417538087399)*0.5**o
        ref[0,3,0,0]=(-0.149386157207)/(o+1.)+(-0.289926844941)+(-0.230658480436)*0.5**o
        ref[0,3,0,1]=(-0.542257139282)/(o+1.)+(-0.0133255401565)+(-0.815859549425)*0.5**o
        ref[0,3,1,0]=(0.714019527678)/(o+1.)+(-0.00632729199566)+(0.0508974791413)*0.5**o
        ref[0,3,1,1]=(0.352455072404)/(o+1.)+(0.143383852019)+(0.119419200166)*0.5**o
        ref[0,3,2,0]=(-0.585094684368)/(o+1.)+(0.00921199711359)+(-0.82075904747)*0.5**o
        ref[0,3,2,1]=(0.686644239408)/(o+1.)+(0.468673187806)+(-0.748061395205)*0.5**o
        ref[0,4,0,0]=(0.129373873322)/(o+1.)+(0.607761864417)+(0.128293903485)*0.5**o
        ref[0,4,0,1]=(-0.170677596319)/(o+1.)+(-0.206814905431)+(0.639638672694)*0.5**o
        ref[0,4,1,0]=(0.282544628333)/(o+1.)+(-0.475291486231)+(0.43743097727)*0.5**o
        ref[0,4,1,1]=(0.218383977926)/(o+1.)+(0.43905900591)+(-0.995599965283)*0.5**o
        ref[0,4,2,0]=(-0.449331655188)/(o+1.)+(-0.0971131654902)+(0.922609534479)*0.5**o
        ref[0,4,2,1]=(-0.167639663907)/(o+1.)+(-0.103797419824)+(0.950186933361)*0.5**o
        ref[1,0,0,0]=(-0.622510875807)/(o+1.)+(0.186056784105)+(0.0573632292788)*0.5**o
        ref[1,0,0,1]=(0.469744047929)/(o+1.)+(-0.838331463751)+(0.368975600444)*0.5**o
        ref[1,0,1,0]=(0.625367069433)/(o+1.)+(-0.432363629616)+(-0.810194964715)*0.5**o
        ref[1,0,1,1]=(0.359324627118)/(o+1.)+(0.36983912542)+(0.744923275143)*0.5**o
        ref[1,0,2,0]=(0.318198109895)/(o+1.)+(-0.215392416287)+(0.891959004941)*0.5**o
        ref[1,0,2,1]=(-0.674805144805)/(o+1.)+(0.581833674632)+(0.842125439023)*0.5**o
        ref[1,1,0,0]=(-0.740263211102)/(o+1.)+(0.109311509194)+(-0.786719774805)*0.5**o
        ref[1,1,0,1]=(0.714032957488)/(o+1.)+(-0.81946337242)+(0.831245977993)*0.5**o
        ref[1,1,1,0]=(0.455949140326)/(o+1.)+(0.666183494578)+(0.862796574462)*0.5**o
        ref[1,1,1,1]=(-0.62474682088)/(o+1.)+(0.438060834031)+(-0.500409398892)*0.5**o
        ref[1,1,2,0]=(-0.770315341866)/(o+1.)+(-0.375748925732)+(-0.402886961189)*0.5**o
        ref[1,1,2,1]=(0.654162491323)/(o+1.)+(0.189164682023)+(0.358161520677)*0.5**o
        ref[1,2,0,0]=(-0.0538737948754)/(o+1.)+(0.333813880332)+(-0.15539934232)*0.5**o
        ref[1,2,0,1]=(-0.377617770006)/(o+1.)+(-0.0218224426646)+(0.371159212282)*0.5**o
        ref[1,2,1,0]=(-0.360936107893)/(o+1.)+(0.141978404604)+(0.117790556271)*0.5**o
        ref[1,2,1,1]=(0.183316445987)/(o+1.)+(-0.0416656990415)+(-0.524828128971)*0.5**o
        ref[1,2,2,0]=(0.126601242737)/(o+1.)+(-0.0623520339976)+(0.300516177535)*0.5**o
        ref[1,2,2,1]=(-0.810227948574)/(o+1.)+(-0.387092926354)+(-0.29046926624)*0.5**o
        ref[1,3,0,0]=(0.757723452446)/(o+1.)+(0.293889998642)+(-0.134275834453)*0.5**o
        ref[1,3,0,1]=(0.503482692135)/(o+1.)+(-0.125704830059)+(0.882915728842)*0.5**o
        ref[1,3,1,0]=(-0.861747803918)/(o+1.)+(-0.0802560425765)+(-0.954792910945)*0.5**o
        ref[1,3,1,1]=(-0.322274864209)/(o+1.)+(-0.226098024118)+(0.747255393187)*0.5**o
        ref[1,3,2,0]=(-0.488288412308)/(o+1.)+(0.0635632421511)+(0.139287057836)*0.5**o
        ref[1,3,2,1]=(0.206644380777)/(o+1.)+(0.323520058859)+(-0.861625516304)*0.5**o
        ref[1,4,0,0]=(-0.971680855409)/(o+1.)+(0.0947423509032)+(-0.714198122614)*0.5**o
        ref[1,4,0,1]=(0.347408878778)/(o+1.)+(-0.709724935569)+(0.293578109913)*0.5**o
        ref[1,4,1,0]=(-0.284086603407)/(o+1.)+(-0.116791420591)+(0.843760126513)*0.5**o
        ref[1,4,1,1]=(0.537701123084)/(o+1.)+(-0.482717976859)+(-0.342911357374)*0.5**o
        ref[1,4,2,0]=(0.831438049852)/(o+1.)+(0.056404572442)+(-0.71201360314)*0.5**o
        ref[1,4,2,1]=(-0.0369972206543)/(o+1.)+(0.275008406103)+(-0.616403541928)*0.5**o
        ref[2,0,0,0]=(-0.464334304695)/(o+1.)+(0.132370231466)+(-0.959754423491)*0.5**o
        ref[2,0,0,1]=(0.173044245392)/(o+1.)+(0.437195842274)+(0.231051530608)*0.5**o
        ref[2,0,1,0]=(-0.816201949845)/(o+1.)+(-0.72616145946)+(0.376577055194)*0.5**o
        ref[2,0,1,1]=(-0.876294548464)/(o+1.)+(0.128100848943)+(-0.4891837531)*0.5**o
        ref[2,0,2,0]=(0.602375475355)/(o+1.)+(-0.497981505761)+(-0.759548081841)*0.5**o
        ref[2,0,2,1]=(0.813648191424)/(o+1.)+(-0.619848910412)+(0.114018259802)*0.5**o
        ref[2,1,0,0]=(-0.12122456959)/(o+1.)+(0.631472305971)+(-0.572699800518)*0.5**o
        ref[2,1,0,1]=(0.539760655497)/(o+1.)+(0.43429478244)+(0.0655268452799)*0.5**o
        ref[2,1,1,0]=(-0.0242249415295)/(o+1.)+(-0.263311349051)+(0.358281119336)*0.5**o
        ref[2,1,1,1]=(-0.995787126907)/(o+1.)+(-0.370899595832)+(0.523751704889)*0.5**o
        ref[2,1,2,0]=(0.149400154478)/(o+1.)+(-0.107907886588)+(-0.185380508646)*0.5**o
        ref[2,1,2,1]=(0.908525646396)/(o+1.)+(-0.18328120728)+(0.81086932122)*0.5**o
        ref[2,2,0,0]=(-0.0495088159461)/(o+1.)+(-0.246369371749)+(-0.463416100256)*0.5**o
        ref[2,2,0,1]=(-0.895280577411)/(o+1.)+(-0.0345629407446)+(-0.200698149193)*0.5**o
        ref[2,2,1,0]=(-0.375709672588)/(o+1.)+(-0.126514661136)+(0.888683772867)*0.5**o
        ref[2,2,1,1]=(0.868194479789)/(o+1.)+(-0.239689630708)+(-0.0167949020901)*0.5**o
        ref[2,2,2,0]=(0.267937553127)/(o+1.)+(-0.425739316963)+(0.441981698578)*0.5**o
        ref[2,2,2,1]=(0.697645775876)/(o+1.)+(-0.16217258223)+(0.876701424579)*0.5**o
        ref[2,3,0,0]=(-0.0706155476337)/(o+1.)+(0.483772571196)+(0.111231509217)*0.5**o
        ref[2,3,0,1]=(0.326282512878)/(o+1.)+(0.759423156855)+(-0.858913402812)*0.5**o
        ref[2,3,1,0]=(0.575866251831)/(o+1.)+(-0.776139286869)+(-0.524551221953)*0.5**o
        ref[2,3,1,1]=(-0.488263373008)/(o+1.)+(0.0931216925076)+(0.391514075129)*0.5**o
        ref[2,3,2,0]=(0.85094548724)/(o+1.)+(-0.0708709485834)+(0.26532129161)*0.5**o
        ref[2,3,2,1]=(0.309349741876)/(o+1.)+(-0.384711182319)+(0.2339994493)*0.5**o
        ref[2,4,0,0]=(0.339247480459)/(o+1.)+(0.475163650236)+(-0.713553783036)*0.5**o
        ref[2,4,0,1]=(0.340009298488)/(o+1.)+(0.0740612601349)+(-0.733490022564)*0.5**o
        ref[2,4,1,0]=(0.680862759763)/(o+1.)+(0.330395834014)+(0.709413437683)*0.5**o
        ref[2,4,1,1]=(0.687201138134)/(o+1.)+(0.587139359385)+(-0.130761551034)*0.5**o
        ref[2,4,2,0]=(0.910504833397)/(o+1.)+(0.210828308638)+(0.256894911335)*0.5**o
        ref[2,4,2,1]=(-0.146356272875)/(o+1.)+(-0.798997815404)+(0.95137247728)*0.5**o
        ref[3,0,0,0]=(0.894383747285)/(o+1.)+(0.932254321713)+(0.663180444534)*0.5**o
        ref[3,0,0,1]=(0.367908872519)/(o+1.)+(-0.177196813397)+(0.424556283891)*0.5**o
        ref[3,0,1,0]=(0.509310001763)/(o+1.)+(-0.869535884539)+(0.908698580251)*0.5**o
        ref[3,0,1,1]=(-0.986699126405)/(o+1.)+(0.246072597257)+(0.523444225814)*0.5**o
        ref[3,0,2,0]=(-0.843900271478)/(o+1.)+(0.46874998261)+(-0.163604985314)*0.5**o
        ref[3,0,2,1]=(0.902072835174)/(o+1.)+(-0.0395040185096)+(-0.490804271432)*0.5**o
        ref[3,1,0,0]=(0.430512336347)/(o+1.)+(0.755564990872)+(-0.949731382685)*0.5**o
        ref[3,1,0,1]=(-0.203921504027)/(o+1.)+(-0.189536921439)+(0.945102298088)*0.5**o
        ref[3,1,1,0]=(-0.365021977395)/(o+1.)+(-0.133381403116)+(0.762442770795)*0.5**o
        ref[3,1,1,1]=(-0.397139314214)/(o+1.)+(0.152610714905)+(-0.829061607399)*0.5**o
        ref[3,1,2,0]=(-0.435377419129)/(o+1.)+(0.687677761846)+(-0.780040060687)*0.5**o
        ref[3,1,2,1]=(-0.642258121264)/(o+1.)+(-0.260682339572)+(-0.853144390228)*0.5**o
        ref[3,2,0,0]=(0.361300611554)/(o+1.)+(0.451810606136)+(0.377161814597)*0.5**o
        ref[3,2,0,1]=(0.083224329707)/(o+1.)+(0.223994918098)+(-0.51174671413)*0.5**o
        ref[3,2,1,0]=(-0.132608876197)/(o+1.)+(0.761129663444)+(0.612695079311)*0.5**o
        ref[3,2,1,1]=(0.68468196965)/(o+1.)+(-0.0553342296404)+(0.0928734170244)*0.5**o
        ref[3,2,2,0]=(-0.0602194449612)/(o+1.)+(0.222211500837)+(-0.863542893122)*0.5**o
        ref[3,2,2,1]=(-0.0746938765947)/(o+1.)+(0.283185341046)+(-0.0942537498633)*0.5**o
        ref[3,3,0,0]=(-0.541714868844)/(o+1.)+(0.642337208826)+(-0.0588913579205)*0.5**o
        ref[3,3,0,1]=(-0.32332836636)/(o+1.)+(0.0454800898319)+(0.221784155922)*0.5**o
        ref[3,3,1,0]=(0.755681096012)/(o+1.)+(0.0454123213802)+(0.804097639203)*0.5**o
        ref[3,3,1,1]=(0.745033643035)/(o+1.)+(0.34690273587)+(0.825696602707)*0.5**o
        ref[3,3,2,0]=(-0.569395510546)/(o+1.)+(0.0733165737898)+(-0.372114391352)*0.5**o
        ref[3,3,2,1]=(0.781753556415)/(o+1.)+(0.592862368109)+(0.439723359721)*0.5**o
        ref[3,4,0,0]=(0.328787484151)/(o+1.)+(-0.75965414705)+(-0.429896412297)*0.5**o
        ref[3,4,0,1]=(-0.541538468561)/(o+1.)+(-0.590091943297)+(0.803567111353)*0.5**o
        ref[3,4,1,0]=(0.308312799035)/(o+1.)+(0.677057855567)+(-0.963266835039)*0.5**o
        ref[3,4,1,1]=(-0.923691546947)/(o+1.)+(-0.0348874464182)+(0.651198604373)*0.5**o
        ref[3,4,2,0]=(-0.0326564953763)/(o+1.)+(-0.0667935103036)+(-0.339406539705)*0.5**o
        ref[3,4,2,1]=(0.193890658409)/(o+1.)+(-0.370194614419)+(0.471355209305)*0.5**o
      else:
        arg[0,0,0,0]=(0.0737627367843)*x[0]**o+(-0.897508234376)*x[0]+(0.499361502035)*x[1]**o+(-0.830793706142)*x[1]+(0.967672023686)*x[2]**o+(-0.94209897356)*x[2]
        arg[0,0,0,1]=(0.353968533247)*x[0]**o+(-0.828575104081)*x[0]+(0.111197799977)*x[1]**o+(0.475627052455)*x[1]+(0.10233739786)*x[2]**o+(-0.297090205985)*x[2]
        arg[0,0,1,0]=(0.315704665816)*x[0]**o+(0.450577419574)*x[0]+(-0.0225047225996)*x[1]**o+(0.221436098936)*x[1]+(-0.210708820992)*x[2]**o+(0.22813686037)*x[2]
        arg[0,0,1,1]=(-0.453912049362)*x[0]**o+(0.443266972459)*x[0]+(-0.166003491301)*x[1]**o+(0.325157034345)*x[1]+(0.975435329063)*x[2]**o+(0.516477372955)*x[2]
        arg[0,0,2,0]=(0.705225432895)*x[0]**o+(0.406870556161)*x[0]+(0.967131048725)*x[1]**o+(-0.31111693058)*x[1]+(0.915125425876)*x[2]**o+(-0.417407852906)*x[2]
        arg[0,0,2,1]=(-0.397162258761)*x[0]**o+(-0.49079016751)*x[0]+(-0.444905720899)*x[1]**o+(0.258998080457)*x[1]+(-0.406172174923)*x[2]**o+(0.166620074269)*x[2]
        arg[0,1,0,0]=(0.00444300957662)*x[0]**o+(0.59448685984)*x[0]+(-0.755596203939)*x[1]**o+(-0.587866578467)*x[1]+(-0.994620629569)*x[2]**o+(0.558539124849)*x[2]
        arg[0,1,0,1]=(0.726535641072)*x[0]**o+(-0.515254556008)*x[0]+(-0.737095044797)*x[1]**o+(0.396218854768)*x[1]+(-0.873340093434)*x[2]**o+(0.538194586759)*x[2]
        arg[0,1,1,0]=(-0.554579794515)*x[0]**o+(-0.754117891877)*x[0]+(0.242579027102)*x[1]**o+(0.368307663104)*x[1]+(0.308372932825)*x[2]**o+(-0.521440130826)*x[2]
        arg[0,1,1,1]=(0.294744137189)*x[0]**o+(0.44996935134)*x[0]+(0.118263237453)*x[1]**o+(-0.143566244715)*x[1]+(-0.927385432821)*x[2]**o+(0.912735237305)*x[2]
        arg[0,1,2,0]=(-0.30261020867)*x[0]**o+(-0.994883662031)*x[0]+(0.514662053738)*x[1]**o+(0.327770859964)*x[1]+(0.862452368458)*x[2]**o+(-0.739026842429)*x[2]
        arg[0,1,2,1]=(0.494855964972)*x[0]**o+(-0.358461686787)*x[0]+(0.406489619959)*x[1]**o+(0.417722618206)*x[1]+(0.679808195501)*x[2]**o+(-0.00502769522311)*x[2]
        arg[0,2,0,0]=(-0.914585215232)*x[0]**o+(0.305890296017)*x[0]+(-0.867826544668)*x[1]**o+(0.304580882671)*x[1]+(-0.401401956053)*x[2]**o+(0.842276423411)*x[2]
        arg[0,2,0,1]=(-0.260590397572)*x[0]**o+(0.541867335986)*x[0]+(0.0279475511153)*x[1]**o+(0.0255320376447)*x[1]+(-0.584584718877)*x[2]**o+(-0.640388032783)*x[2]
        arg[0,2,1,0]=(0.0455797513404)*x[0]**o+(-0.764984318339)*x[0]+(-0.257174640639)*x[1]**o+(0.745888292962)*x[1]+(0.964524202534)*x[2]**o+(0.435680191282)*x[2]
        arg[0,2,1,1]=(-0.300900136798)*x[0]**o+(-0.899164082043)*x[0]+(-0.63273853331)*x[1]**o+(0.56522562781)*x[1]+(-0.846482371222)*x[2]**o+(-0.909985752269)*x[2]
        arg[0,2,2,0]=(0.745711773093)*x[0]**o+(0.0627291150681)*x[0]+(-0.090340386706)*x[1]**o+(-0.0912293084958)*x[1]+(0.108613205226)*x[2]**o+(-0.28985444593)*x[2]
        arg[0,2,2,1]=(0.291340077334)*x[0]**o+(0.592856577974)*x[0]+(0.141857446535)*x[1]**o+(0.830526360843)*x[1]+(0.956840406828)*x[2]**o+(-0.717465405888)*x[2]
        arg[0,3,0,0]=(0.89242099254)*x[0]**o+(0.529348654364)*x[0]+(-0.074382350183)*x[1]**o+(0.874670648345)*x[1]+(-0.728104560168)*x[2]**o+(0.0781153963606)*x[2]
        arg[0,3,0,1]=(0.095390666498)*x[0]**o+(0.0124610436496)*x[0]+(0.278014506594)*x[1]**o+(0.923928359216)*x[1]+(-0.0905667357972)*x[2]**o+(-0.894031439498)*x[2]
        arg[0,3,1,0]=(-0.317750910722)*x[0]**o+(0.677406387747)*x[0]+(0.550094997541)*x[1]**o+(-0.30098732949)*x[1]+(0.501753868155)*x[2]**o+(-0.42859127752)*x[2]
        arg[0,3,1,1]=(-0.204455448392)*x[0]**o+(0.445237113438)*x[0]+(0.853596204887)*x[1]**o+(0.315701645178)*x[1]+(0.307192864939)*x[2]**o+(-0.708325809173)*x[2]
        arg[0,3,2,0]=(0.38581656579)*x[0]**o+(-0.79289714243)*x[0]+(-0.412226202311)*x[1]**o+(0.655534112291)*x[1]+(0.31352728014)*x[2]**o+(0.502089385396)*x[2]
        arg[0,3,2,1]=(-0.225571343976)*x[0]**o+(0.687765563016)*x[0]+(0.0421378888728)*x[1]**o+(0.54466301835)*x[1]+(-0.430288255786)*x[2]**o+(-0.117571440224)*x[2]
        arg[0,4,0,0]=(0.834083057621)*x[0]**o+(0.0292441566815)*x[0]+(-0.947154369416)*x[1]**o+(0.597153808995)*x[1]+(0.323706009194)*x[2]**o+(0.664093182644)*x[2]
        arg[0,4,0,1]=(-0.613152578007)*x[0]**o+(0.253155875339)*x[0]+(0.297138228345)*x[1]**o+(-0.604393247773)*x[1]+(-0.936718722281)*x[2]**o+(-0.410140848312)*x[2]
        arg[0,4,1,0]=(-0.373668614153)*x[0]**o+(-0.401240526097)*x[0]+(-0.417458139032)*x[1]**o+(0.456033864594)*x[1]+(-0.158434012225)*x[2]**o+(-0.33938136039)*x[2]
        arg[0,4,1,1]=(0.00943802199128)*x[0]**o+(-0.800756064115)*x[0]+(0.899199671858)*x[1]**o+(0.376830611387)*x[1]+(-0.930822199046)*x[2]**o+(0.192235807799)*x[2]
        arg[0,4,2,0]=(0.157099024354)*x[0]**o+(0.195942080381)*x[0]+(-0.554519054021)*x[1]**o+(0.871823312361)*x[1]+(-0.34803033039)*x[2]**o+(-0.646933331859)*x[2]
        arg[0,4,2,1]=(0.933762543076)*x[0]**o+(-0.169843985744)*x[0]+(0.971089176971)*x[1]**o+(-0.605176382286)*x[1]+(0.254369209427)*x[2]**o+(0.0314820745541)*x[2]
        arg[1,0,0,0]=(0.0417476479889)*x[0]**o+(-0.144293810851)*x[0]+(-0.677039420744)*x[1]**o+(-0.137097132377)*x[1]+(0.721082639859)*x[2]**o+(-0.32510577691)*x[2]
        arg[1,0,0,1]=(0.450986977446)*x[0]**o+(-0.921371106219)*x[0]+(0.536898979198)*x[1]**o+(-0.814951108186)*x[1]+(-0.733774506956)*x[2]**o+(0.938941747764)*x[2]
        arg[1,0,1,0]=(-0.23712795738)*x[0]**o+(0.450883240541)*x[0]+(0.101102533189)*x[1]**o+(-0.998253664232)*x[1]+(-0.843977488601)*x[2]**o+(-0.807025450448)*x[2]
        arg[1,0,1,1]=(-0.308721170682)*x[0]**o+(-0.546268734305)*x[0]+(0.643628083343)*x[1]**o+(0.206934468523)*x[1]+(-0.257667927436)*x[2]**o+(0.468860780891)*x[2]
        arg[1,0,2,0]=(-0.267232319593)*x[0]**o+(0.221737693042)*x[0]+(0.202218228468)*x[1]**o+(-0.560941556406)*x[1]+(0.910244160614)*x[2]**o+(0.361584784536)*x[2]
        arg[1,0,2,1]=(0.405254374432)*x[0]**o+(-0.102574362257)*x[0]+(0.527874164745)*x[1]**o+(0.0318395137256)*x[1]+(-0.448050293943)*x[2]**o+(-0.6401160036)*x[2]
        arg[1,1,0,0]=(-0.877224972777)*x[0]**o+(-0.544175806369)*x[0]+(0.528037913494)*x[1]**o+(-0.477854554671)*x[1]+(-0.394875438861)*x[2]**o+(-0.704656012962)*x[2]
        arg[1,1,0,1]=(-0.873928172379)*x[0]**o+(0.273455449944)*x[0]+(0.681090480547)*x[1]**o+(0.883243766093)*x[1]+(0.557357114631)*x[2]**o+(0.586148966774)*x[2]
        arg[1,1,1,0]=(0.552826305913)*x[0]**o+(-0.277269605193)*x[0]+(-0.681872908424)*x[1]**o+(-0.84578699459)*x[1]+(0.6543624385)*x[2]**o+(-0.355051513153)*x[2]
        arg[1,1,1,1]=(-0.71233607947)*x[0]**o+(-0.779861555179)*x[0]+(0.11496674995)*x[1]**o+(0.634901982779)*x[1]+(0.833044137166)*x[2]**o+(0.0443514408009)*x[2]
        arg[1,1,2,0]=(-0.411667274338)*x[0]**o+(0.218709061898)*x[0]+(0.132573928273)*x[1]**o+(-0.450677430962)*x[1]+(0.88420873514)*x[2]**o+(-0.618513305411)*x[2]
        arg[1,1,2,1]=(0.533005373575)*x[0]**o+(0.846343252936)*x[0]+(0.697076801609)*x[1]**o+(0.342260840742)*x[1]+(0.923380136498)*x[2]**o+(-0.727450493778)*x[2]
        arg[1,2,0,0]=(-0.637366803484)*x[0]**o+(-0.654649539562)*x[0]+(-0.443035194144)*x[1]**o+(0.4928304386)*x[1]+(0.889290653473)*x[2]**o+(0.208513517091)*x[2]
        arg[1,2,0,1]=(-0.404870452835)*x[0]**o+(-0.556431486625)*x[0]+(-0.428885935216)*x[1]**o+(0.208771213187)*x[1]+(0.706109349268)*x[2]**o+(0.726168991522)*x[2]
        arg[1,2,1,0]=(0.85574176569)*x[0]**o+(-0.371989246254)*x[0]+(0.0376283091444)*x[1]**o+(-0.164811219077)*x[1]+(0.842513532253)*x[2]**o+(-0.963454790343)*x[2]
        arg[1,2,1,1]=(0.576825533647)*x[0]**o+(0.866570909412)*x[0]+(-0.286228055595)*x[1]**o+(-0.509258889664)*x[1]+(0.358087406384)*x[2]**o+(0.2558870556)*x[2]
        arg[1,2,2,0]=(0.741411326194)*x[0]**o+(0.887444210472)*x[0]+(0.176659455323)*x[1]**o+(-0.292731470152)*x[1]+(0.179836617557)*x[2]**o+(-0.10894424483)*x[2]
        arg[1,2,2,1]=(0.386515069261)*x[0]**o+(-0.423389396097)*x[0]+(0.300384913804)*x[1]**o+(-0.569308908813)*x[1]+(0.845849686169)*x[2]**o+(-0.802483942058)*x[2]
        arg[1,3,0,0]=(0.848952965744)*x[0]**o+(0.113555873929)*x[0]+(0.132168122705)*x[1]**o+(0.464116031047)*x[1]+(-0.928746352905)*x[2]**o+(-0.176374657603)*x[2]
        arg[1,3,0,1]=(0.357870838328)*x[0]**o+(-0.294290586373)*x[0]+(0.395859596543)*x[1]**o+(-0.265635871788)*x[1]+(0.283887401402)*x[2]**o+(0.500378090749)*x[2]
        arg[1,3,1,0]=(-0.509333127149)*x[0]**o+(0.328856435768)*x[0]+(-0.377387784476)*x[1]**o+(-0.878505533565)*x[1]+(0.448760304667)*x[2]**o+(0.996692047048)*x[2]
        arg[1,3,1,1]=(-0.837885550386)*x[0]**o+(0.0608579436757)*x[0]+(-0.627132023609)*x[1]**o+(0.902538475491)*x[1]+(-0.449596037062)*x[2]**o+(-0.158786724549)*x[2]
        arg[1,3,2,0]=(0.205889276236)*x[0]**o+(0.576561726109)*x[0]+(-0.00281354319689)*x[1]**o+(0.214968600999)*x[1]+(0.811758703832)*x[2]**o+(0.260164242506)*x[2]
        arg[1,3,2,1]=(-0.331660060588)*x[0]**o+(0.713879758554)*x[0]+(0.804549392751)*x[1]**o+(-0.346397636822)*x[1]+(-0.364073503513)*x[2]**o+(0.338612348994)*x[2]
        arg[1,4,0,0]=(0.394201404247)*x[0]**o+(-0.416853395415)*x[0]+(-0.0581465874206)*x[1]**o+(0.105267182588)*x[1]+(0.791799718009)*x[2]**o+(0.0840172853234)*x[2]
        arg[1,4,0,1]=(-0.346144268453)*x[0]**o+(-0.661228785728)*x[0]+(0.326588395606)*x[1]**o+(0.766137046896)*x[1]+(0.566377202267)*x[2]**o+(0.613565461425)*x[2]
        arg[1,4,1,0]=(-0.907566140749)*x[0]**o+(-0.391612881392)*x[0]+(0.263322351365)*x[1]**o+(-0.4373853914)*x[1]+(0.248855925063)*x[2]**o+(0.973515668505)*x[2]
        arg[1,4,1,1]=(0.726194147182)*x[0]**o+(0.497593423927)*x[0]+(-0.951344923018)*x[1]**o+(0.55327321528)*x[1]+(-0.266195110749)*x[2]**o+(-0.310893832826)*x[2]
        arg[1,4,2,0]=(-0.940824546553)*x[0]**o+(-0.47838524024)*x[0]+(0.408728941942)*x[1]**o+(0.627300786998)*x[1]+(-0.0547545650101)*x[2]**o+(0.433290388525)*x[2]
        arg[1,4,2,1]=(0.728499572603)*x[0]**o+(-0.232088507059)*x[0]+(0.0550057703801)*x[1]**o+(0.159356057226)*x[1]+(0.111191390097)*x[2]**o+(-0.64498018454)*x[2]
        arg[2,0,0,0]=(0.467025642879)*x[0]**o+(-0.695379603222)*x[0]+(0.591732172092)*x[1]**o+(0.84325648142)*x[1]+(0.965696472916)*x[2]**o+(0.626196181241)*x[2]
        arg[2,0,0,1]=(-0.211129222294)*x[0]**o+(-0.501208172412)*x[0]+(-0.0162146409175)*x[1]**o+(-0.672364205306)*x[1]+(-0.851073479662)*x[2]**o+(-0.817568385627)*x[2]
        arg[2,0,1,0]=(-0.668065295797)*x[0]**o+(-0.90359857855)*x[0]+(0.040519772083)*x[1]**o+(-0.295766307198)*x[1]+(0.0491234828524)*x[2]**o+(-0.589557178985)*x[2]
        arg[2,0,1,1]=(0.928751918298)*x[0]**o+(-0.773787731767)*x[0]+(-0.744325160449)*x[1]**o+(-0.426987905682)*x[1]+(0.503339956851)*x[2]**o+(0.386756648382)*x[2]
        arg[2,0,2,0]=(-0.531818105049)*x[0]**o+(-0.586811490681)*x[0]+(0.780379352762)*x[1]**o+(-0.5575328003)*x[1]+(0.0354867101462)*x[2]**o+(0.581226069105)*x[2]
        arg[2,0,2,1]=(0.0150988200159)*x[0]**o+(0.56415563975)*x[0]+(0.540723773613)*x[1]**o+(0.0653377679637)*x[1]+(-0.52633696371)*x[2]**o+(-0.122811009405)*x[2]
        arg[2,1,0,0]=(-0.0829154151121)*x[0]**o+(-0.416192198737)*x[0]+(0.944232067474)*x[1]**o+(-0.159852455491)*x[1]+(-0.198397797188)*x[2]**o+(-0.684211447305)*x[2]
        arg[2,1,0,1]=(-0.834502772339)*x[0]**o+(-0.856817328109)*x[0]+(0.213303574315)*x[1]**o+(-0.283140791695)*x[1]+(-0.727061494352)*x[2]**o+(0.12871372765)*x[2]
        arg[2,1,1,0]=(0.247439924718)*x[0]**o+(-0.912464825903)*x[0]+(0.791177764168)*x[1]**o+(0.160047921728)*x[1]+(0.591086305304)*x[2]**o+(-0.774025941232)*x[2]
        arg[2,1,1,1]=(-0.120169534728)*x[0]**o+(-0.747254680906)*x[0]+(-0.394003570933)*x[1]**o+(-0.938198960695)*x[1]+(-0.0142776288517)*x[2]**o+(0.0627959740849)*x[2]
        arg[2,1,2,0]=(0.703841647274)*x[0]**o+(-0.946039767174)*x[0]+(-0.836717609846)*x[1]**o+(0.776573515772)*x[1]+(0.250502751027)*x[2]**o+(0.403896552902)*x[2]
        arg[2,1,2,1]=(-0.19760048131)*x[0]**o+(-0.250991189554)*x[0]+(0.278810924634)*x[1]**o+(-0.580148097047)*x[1]+(-0.197518781697)*x[2]**o+(0.116835806412)*x[2]
        arg[2,2,0,0]=(-0.507872937196)*x[0]**o+(-0.771841217512)*x[0]+(0.852666314762)*x[1]**o+(-0.482947444136)*x[1]+(0.805824662389)*x[2]**o+(-0.299255374349)*x[2]
        arg[2,2,0,1]=(-0.481403986338)*x[0]**o+(0.904322169038)*x[0]+(0.937553651477)*x[1]**o+(0.727763542606)*x[1]+(-0.69485539356)*x[2]**o+(0.547755372933)*x[2]
        arg[2,2,1,0]=(0.241675645258)*x[0]**o+(0.957681326492)*x[0]+(-0.305747218513)*x[1]**o+(-0.0393749799535)*x[1]+(-0.448892234759)*x[2]**o+(0.707429367761)*x[2]
        arg[2,2,1,1]=(0.114772627763)*x[0]**o+(-0.411147710203)*x[0]+(0.13389320737)*x[1]**o+(0.806309117051)*x[1]+(-0.572620769665)*x[2]**o+(0.0389925355751)*x[2]
        arg[2,2,2,0]=(0.887023209688)*x[0]**o+(0.158261816385)*x[0]+(0.687399231822)*x[1]**o+(-0.858491519094)*x[1]+(-0.00348150506922)*x[2]**o+(-0.863496570874)*x[2]
        arg[2,2,2,1]=(0.836731324893)*x[0]**o+(0.975472534225)*x[0]+(0.407275572226)*x[1]**o+(0.565132026147)*x[1]+(0.79110912675)*x[2]**o+(-0.0816802575667)*x[2]
        arg[2,3,0,0]=(0.196161020545)*x[0]**o+(0.254301467318)*x[0]+(0.759363127106)*x[1]**o+(0.536320544038)*x[1]+(0.522705219849)*x[2]**o+(0.376531450811)*x[2]
        arg[2,3,0,1]=(-0.862326385376)*x[0]**o+(0.766714365146)*x[0]+(0.279327620722)*x[1]**o+(-0.44416461578)*x[1]+(0.115982232563)*x[2]**o+(0.824064889151)*x[2]
        arg[2,3,1,0]=(0.571263839862)*x[0]**o+(-0.679998793927)*x[0]+(0.63548515376)*x[1]**o+(0.464171242163)*x[1]+(0.712124752567)*x[2]**o+(0.913131755622)*x[2]
        arg[2,3,1,1]=(0.272559456412)*x[0]**o+(0.0577748172306)*x[0]+(-0.610464803438)*x[1]**o+(0.233987628538)*x[1]+(0.904147555826)*x[2]**o+(-0.131245442847)*x[2]
        arg[2,3,2,0]=(0.3409363126)*x[0]**o+(0.40060221588)*x[0]+(-0.603551487822)*x[1]**o+(-0.697430353793)*x[1]+(-0.867009193249)*x[2]**o+(-0.178991284979)*x[2]
        arg[2,3,2,1]=(0.949170673623)*x[0]**o+(-0.0217541208215)*x[0]+(-0.775307373317)*x[1]**o+(0.421565612874)*x[1]+(0.419254478365)*x[2]**o+(-0.732203964657)*x[2]
        arg[2,4,0,0]=(0.876774885889)*x[0]**o+(0.441155536177)*x[0]+(-0.292113312365)*x[1]**o+(0.880590008772)*x[1]+(0.0732363738743)*x[2]**o+(-0.590855067487)*x[2]
        arg[2,4,0,1]=(0.347441329383)*x[0]**o+(-0.96566344519)*x[0]+(-0.0870932691958)*x[1]**o+(-0.793898415692)*x[1]+(0.911757004621)*x[2]**o+(0.623070769788)*x[2]
        arg[2,4,1,0]=(0.732123350436)*x[0]**o+(-0.707615856771)*x[0]+(-0.57969408421)*x[1]**o+(0.0643464469617)*x[1]+(-0.648380091597)*x[2]**o+(0.567100447405)*x[2]
        arg[2,4,1,1]=(-0.128872747083)*x[0]**o+(-0.463336399393)*x[0]+(-0.1679086419)*x[1]**o+(0.602218708561)*x[1]+(0.62788576637)*x[2]**o+(0.223548351914)*x[2]
        arg[2,4,2,0]=(0.619034818105)*x[0]**o+(0.167994824318)*x[0]+(-0.244892750219)*x[1]**o+(0.471338499791)*x[1]+(0.526950200066)*x[2]**o+(0.345611623572)*x[2]
        arg[2,4,2,1]=(0.645979236759)*x[0]**o+(0.79896884484)*x[0]+(-0.587486435644)*x[1]**o+(-0.467838428827)*x[1]+(0.672707985612)*x[2]**o+(-0.169680966884)*x[2]
        arg[3,0,0,0]=(-0.265836015694)*x[0]**o+(-0.383762854285)*x[0]+(0.420375720992)*x[1]**o+(0.739662661302)*x[1]+(-0.593559713248)*x[2]**o+(0.0743208592009)*x[2]
        arg[3,0,0,1]=(0.306246512933)*x[0]**o+(0.0357065903179)*x[0]+(0.213209244461)*x[1]**o+(-0.069836605971)*x[1]+(0.668535341179)*x[2]**o+(0.59870885392)*x[2]
        arg[3,0,1,0]=(0.509420593773)*x[0]**o+(0.995859415429)*x[0]+(-0.07734119959)*x[1]**o+(0.269006992126)*x[1]+(-0.465576902019)*x[2]**o+(0.318766487761)*x[2]
        arg[3,0,1,1]=(-0.609626065482)*x[0]**o+(0.970976299205)*x[0]+(-0.314734168176)*x[1]**o+(-0.536438958228)*x[1]+(0.900096595598)*x[2]**o+(-0.893816431641)*x[2]
        arg[3,0,2,0]=(0.269212290711)*x[0]**o+(-0.119897130167)*x[0]+(-0.937350065698)*x[1]**o+(0.913313769348)*x[1]+(0.311695335026)*x[2]**o+(0.58822688023)*x[2]
        arg[3,0,2,1]=(-0.445116541697)*x[0]**o+(-0.64630078598)*x[0]+(0.811173945666)*x[1]**o+(-0.910503175962)*x[1]+(0.160819266078)*x[2]**o+(-0.807050548571)*x[2]
        arg[3,1,0,0]=(0.25585673506)*x[0]**o+(-0.174293490388)*x[0]+(-0.510750951271)*x[1]**o+(-0.215780338673)*x[1]+(-0.531739311594)*x[2]**o+(0.479569840128)*x[2]
        arg[3,1,0,1]=(-0.159301665223)*x[0]**o+(0.88558738006)*x[0]+(-0.933255168276)*x[1]**o+(0.739621529924)*x[1]+(-0.371501556668)*x[2]**o+(-0.224043643673)*x[2]
        arg[3,1,1,0]=(0.673375237931)*x[0]**o+(0.29315421593)*x[0]+(0.378983835599)*x[1]**o+(-0.187220470447)*x[1]+(-0.122366431969)*x[2]**o+(-0.178182488821)*x[2]
        arg[3,1,1,1]=(0.273992606955)*x[0]**o+(-0.42302685869)*x[0]+(0.490650718142)*x[1]**o+(0.067822229282)*x[1]+(0.758676637186)*x[2]**o+(-0.840062442278)*x[2]
        arg[3,1,2,0]=(-0.533129977534)*x[0]**o+(0.743362501638)*x[0]+(0.0397932337285)*x[1]**o+(0.134261172942)*x[1]+(0.076467475088)*x[2]**o+(0.750124646154)*x[2]
        arg[3,1,2,1]=(0.977824314677)*x[0]**o+(0.322104772509)*x[0]+(0.478366489119)*x[1]**o+(0.139966919282)*x[1]+(0.823631645543)*x[2]**o+(0.29259439301)*x[2]
        arg[3,2,0,0]=(0.969349250234)*x[0]**o+(0.158604629065)*x[0]+(0.517280925739)*x[1]**o+(-0.32151445591)*x[1]+(0.516829269337)*x[2]**o+(0.935139450142)*x[2]
        arg[3,2,0,1]=(-0.673721580725)*x[0]**o+(0.1932869215)*x[0]+(0.47375679317)*x[1]**o+(0.377941367578)*x[1]+(-0.359306451576)*x[2]**o+(0.00498822119998)*x[2]
        arg[3,2,1,0]=(-0.432873699676)*x[0]**o+(0.191782167044)*x[0]+(0.191395777927)*x[1]**o+(-0.540088847941)*x[1]+(0.608451872364)*x[2]**o+(0.151589512705)*x[2]
        arg[3,2,1,1]=(0.633726614581)*x[0]**o+(-0.824133266602)*x[0]+(-0.380289061608)*x[1]**o+(-0.722136850408)*x[1]+(-0.807853184729)*x[2]**o+(0.152552828168)*x[2]
        arg[3,2,2,0]=(-0.182700878265)*x[0]**o+(0.105776705469)*x[0]+(0.837527799957)*x[1]**o+(0.120438395343)*x[1]+(-0.211142277471)*x[2]**o+(-0.0499206664041)*x[2]
        arg[3,2,2,1]=(0.999790796581)*x[0]**o+(0.523238189614)*x[0]+(0.108118557801)*x[1]**o+(-0.899694790145)*x[1]+(0.165089392648)*x[2]**o+(0.38599870475)*x[2]
        arg[3,3,0,0]=(-0.959143897162)*x[0]**o+(-0.968197835338)*x[0]+(-0.521866033091)*x[1]**o+(-0.560799919169)*x[1]+(0.604562365858)*x[2]**o+(0.195593261393)*x[2]
        arg[3,3,0,1]=(0.385513188376)*x[0]**o+(0.918485762628)*x[0]+(0.581126961809)*x[1]**o+(-0.365882492483)*x[1]+(-0.260563773993)*x[2]**o+(0.997016165521)*x[2]
        arg[3,3,1,0]=(0.809269330969)*x[0]**o+(0.472424328796)*x[0]+(-0.920680159926)*x[1]**o+(0.756205962059)*x[1]+(-0.886275742364)*x[2]**o+(-0.800132407145)*x[2]
        arg[3,3,1,1]=(-0.983474390608)*x[0]**o+(-0.436192301851)*x[0]+(0.653210945793)*x[1]**o+(-0.465357783291)*x[1]+(-0.799261076035)*x[2]**o+(-0.349990963152)*x[2]
        arg[3,3,2,0]=(0.710820041642)*x[0]**o+(-0.973768204693)*x[0]+(0.0408403957292)*x[1]**o+(0.61546181161)*x[1]+(0.915939389579)*x[2]**o+(0.634659938301)*x[2]
        arg[3,3,2,1]=(0.633364771689)*x[0]**o+(0.887553239682)*x[0]+(-0.64913373177)*x[1]**o+(0.557465560353)*x[1]+(-0.744072153243)*x[2]**o+(-0.301528624747)*x[2]
        arg[3,4,0,0]=(0.44039719913)*x[0]**o+(-0.0339525806656)*x[0]+(0.574166395893)*x[1]**o+(-0.810441051899)*x[1]+(0.254523432152)*x[2]**o+(-0.798403721953)*x[2]
        arg[3,4,0,1]=(0.253149910502)*x[0]**o+(-0.909846068796)*x[0]+(0.855107984234)*x[1]**o+(-0.973196168972)*x[1]+(0.646832394105)*x[2]**o+(0.811892995874)*x[2]
        arg[3,4,1,0]=(0.13027869782)*x[0]**o+(-0.424094152568)*x[0]+(-0.197473699021)*x[1]**o+(0.0795845331856)*x[1]+(-0.511301398362)*x[2]**o+(-0.331330168767)*x[2]
        arg[3,4,1,1]=(0.476554240918)*x[0]**o+(0.18800431479)*x[0]+(0.259031626216)*x[1]**o+(-0.0532533771273)*x[1]+(0.914847196458)*x[2]**o+(0.333851370508)*x[2]
        arg[3,4,2,0]=(0.800058524963)*x[0]**o+(-0.402028928896)*x[0]+(0.913881556051)*x[1]**o+(0.895516442873)*x[1]+(-0.0227700921994)*x[2]**o+(-0.0502319834692)*x[2]
        arg[3,4,2,1]=(-0.533834741079)*x[0]**o+(0.557954226232)*x[0]+(-0.121676666641)*x[1]**o+(0.781782385789)*x[1]+(0.0682870158781)*x[2]**o+(-0.236432552686)*x[2]
        ref[0,0,0,0]=(1.46703352572)/(o+1.)+(-1.33520045704)+(0.0737627367843)*0.5**o
        ref[0,0,0,1]=(0.213535197837)/(o+1.)+(-0.325019128805)+(0.353968533247)*0.5**o
        ref[0,0,1,0]=(-0.233213543591)/(o+1.)+(0.450075189439)+(0.315704665816)*0.5**o
        ref[0,0,1,1]=(0.809431837761)/(o+1.)+(0.642450689879)+(-0.453912049362)*0.5**o
        ref[0,0,2,0]=(1.8822564746)/(o+1.)+(-0.160827113662)+(0.705225432895)*0.5**o
        ref[0,0,2,1]=(-0.851077895823)/(o+1.)+(-0.032586006392)+(-0.397162258761)*0.5**o
        ref[0,1,0,0]=(-1.75021683351)/(o+1.)+(0.282579703111)+(0.00444300957662)*0.5**o
        ref[0,1,0,1]=(-1.61043513823)/(o+1.)+(0.209579442759)+(0.726535641072)*0.5**o
        ref[0,1,1,0]=(0.550951959927)/(o+1.)+(-0.453625179799)+(-0.554579794515)*0.5**o
        ref[0,1,1,1]=(-0.809122195368)/(o+1.)+(0.609569171964)+(0.294744137189)*0.5**o
        ref[0,1,2,0]=(1.3771144222)/(o+1.)+(-0.703069822248)+(-0.30261020867)*0.5**o
        ref[0,1,2,1]=(1.08629781546)/(o+1.)+(0.0271166180983)+(0.494855964972)*0.5**o
        ref[0,2,0,0]=(-1.26922850072)/(o+1.)+(0.72637380105)+(-0.914585215232)*0.5**o
        ref[0,2,0,1]=(-0.556637167762)/(o+1.)+(-0.036494329576)+(-0.260590397572)*0.5**o
        ref[0,2,1,0]=(0.707349561895)/(o+1.)+(0.208292082952)+(0.0455797513404)*0.5**o
        ref[0,2,1,1]=(-1.47922090453)/(o+1.)+(-0.621962103251)+(-0.300900136798)*0.5**o
        ref[0,2,2,0]=(0.01827281852)/(o+1.)+(-0.159177319679)+(0.745711773093)*0.5**o
        ref[0,2,2,1]=(1.09869785336)/(o+1.)+(0.352958766464)+(0.291340077334)*0.5**o
        ref[0,3,0,0]=(-0.802486910351)/(o+1.)+(0.741067349535)+(0.89242099254)*0.5**o
        ref[0,3,0,1]=(0.187447770797)/(o+1.)+(0.0211789816838)+(0.095390666498)*0.5**o
        ref[0,3,1,0]=(1.0518488657)/(o+1.)+(-0.0260861096315)+(-0.317750910722)*0.5**o
        ref[0,3,1,1]=(1.16078906983)/(o+1.)+(0.0263064747213)+(-0.204455448392)*0.5**o
        ref[0,3,2,0]=(-0.0986989221708)/(o+1.)+(0.182363177629)+(0.38581656579)*0.5**o
        ref[0,3,2,1]=(-0.388150366913)/(o+1.)+(0.557428570571)+(-0.225571343976)*0.5**o
        ref[0,4,0,0]=(-0.623448360222)/(o+1.)+(0.64524557416)+(0.834083057621)*0.5**o
        ref[0,4,0,1]=(-0.639580493935)/(o+1.)+(-0.380689110373)+(-0.613152578007)*0.5**o
        ref[0,4,1,0]=(-0.575892151257)/(o+1.)+(-0.142294010946)+(-0.373668614153)*0.5**o
        ref[0,4,1,1]=(-0.0316225271882)/(o+1.)+(-0.115844822465)+(0.00943802199128)*0.5**o
        ref[0,4,2,0]=(-0.902549384411)/(o+1.)+(0.210416030442)+(0.157099024354)*0.5**o
        ref[0,4,2,1]=(1.2254583864)/(o+1.)+(-0.371769146738)+(0.933762543076)*0.5**o
        ref[1,0,0,0]=(0.0440432191149)/(o+1.)+(-0.303248360069)+(0.0417476479889)*0.5**o
        ref[1,0,0,1]=(-0.196875527758)/(o+1.)+(-0.39869023332)+(0.450986977446)*0.5**o
        ref[1,0,1,0]=(-0.742874955412)/(o+1.)+(-0.677197937069)+(-0.23712795738)*0.5**o
        ref[1,0,1,1]=(0.385960155907)/(o+1.)+(0.0647632575546)+(-0.308721170682)*0.5**o
        ref[1,0,2,0]=(1.11246238908)/(o+1.)+(0.0111904605859)+(-0.267232319593)*0.5**o
        ref[1,0,2,1]=(0.0798238708014)/(o+1.)+(-0.355425426066)+(0.405254374432)*0.5**o
        ref[1,1,0,0]=(0.133162474633)/(o+1.)+(-0.863343187001)+(-0.877224972777)*0.5**o
        ref[1,1,0,1]=(1.23844759518)/(o+1.)+(0.871424091405)+(-0.873928172379)*0.5**o
        ref[1,1,1,0]=(-0.0275104699246)/(o+1.)+(-0.739054056468)+(0.552826305913)*0.5**o
        ref[1,1,1,1]=(0.948010887116)/(o+1.)+(-0.0503040657995)+(-0.71233607947)*0.5**o
        ref[1,1,2,0]=(1.01678266341)/(o+1.)+(-0.425240837237)+(-0.411667274338)*0.5**o
        ref[1,1,2,1]=(1.62045693811)/(o+1.)+(0.23057679995)+(0.533005373575)*0.5**o
        ref[1,2,0,0]=(0.44625545933)/(o+1.)+(0.0233472080645)+(-0.637366803484)*0.5**o
        ref[1,2,0,1]=(0.277223414052)/(o+1.)+(0.189254359042)+(-0.404870452835)*0.5**o
        ref[1,2,1,0]=(0.880141841398)/(o+1.)+(-0.750127627837)+(0.85574176569)*0.5**o
        ref[1,2,1,1]=(0.0718593507895)/(o+1.)+(0.306599537674)+(0.576825533647)*0.5**o
        ref[1,2,2,0]=(0.35649607288)/(o+1.)+(0.242884247745)+(0.741411326194)*0.5**o
        ref[1,2,2,1]=(1.14623459997)/(o+1.)+(-0.897591123484)+(0.386515069261)*0.5**o
        ref[1,3,0,0]=(-0.796578230199)/(o+1.)+(0.200648623687)+(0.848952965744)*0.5**o
        ref[1,3,0,1]=(0.679746997945)/(o+1.)+(-0.0297741837061)+(0.357870838328)*0.5**o
        ref[1,3,1,0]=(0.0713725201916)/(o+1.)+(0.223521474626)+(-0.509333127149)*0.5**o
        ref[1,3,1,1]=(-1.07672806067)/(o+1.)+(0.402304847309)+(-0.837885550386)*0.5**o
        ref[1,3,2,0]=(0.808945160636)/(o+1.)+(0.525847284807)+(0.205889276236)*0.5**o
        ref[1,3,2,1]=(0.440475889238)/(o+1.)+(0.353047235363)+(-0.331660060588)*0.5**o
        ref[1,4,0,0]=(0.733653130588)/(o+1.)+(-0.113784463751)+(0.394201404247)*0.5**o
        ref[1,4,0,1]=(0.892965597874)/(o+1.)+(0.359236861297)+(-0.346144268453)*0.5**o
        ref[1,4,1,0]=(0.512178276429)/(o+1.)+(0.0722586978563)+(-0.907566140749)*0.5**o
        ref[1,4,1,1]=(-1.21754003377)/(o+1.)+(0.369986403191)+(0.726194147182)*0.5**o
        ref[1,4,2,0]=(0.353974376932)/(o+1.)+(0.291102967642)+(-0.940824546553)*0.5**o
        ref[1,4,2,1]=(0.166197160477)/(o+1.)+(-0.358856317187)+(0.728499572603)*0.5**o
        ref[2,0,0,0]=(1.55742864501)/(o+1.)+(0.38703652972)+(0.467025642879)*0.5**o
        ref[2,0,0,1]=(-0.867288120579)/(o+1.)+(-0.995570381672)+(-0.211129222294)*0.5**o
        ref[2,0,1,0]=(0.0896432549354)/(o+1.)+(-0.894461032367)+(-0.668065295797)*0.5**o
        ref[2,0,1,1]=(-0.240985203598)/(o+1.)+(-0.407009494534)+(0.928751918298)*0.5**o
        ref[2,0,2,0]=(0.815866062908)/(o+1.)+(-0.281559110938)+(-0.531818105049)*0.5**o
        ref[2,0,2,1]=(0.0143868099026)/(o+1.)+(0.253341199154)+(0.0150988200159)*0.5**o
        ref[2,1,0,0]=(0.745834270286)/(o+1.)+(-0.630128050767)+(-0.0829154151121)*0.5**o
        ref[2,1,0,1]=(-0.513757920038)/(o+1.)+(-0.505622196077)+(-0.834502772339)*0.5**o
        ref[2,1,1,0]=(1.38226406947)/(o+1.)+(-0.763221422703)+(0.247439924718)*0.5**o
        ref[2,1,1,1]=(-0.408281199784)/(o+1.)+(-0.811328833758)+(-0.120169534728)*0.5**o
        ref[2,1,2,0]=(-0.586214858819)/(o+1.)+(0.11721515075)+(0.703841647274)*0.5**o
        ref[2,1,2,1]=(0.0812921429368)/(o+1.)+(-0.357151740094)+(-0.19760048131)*0.5**o
        ref[2,2,0,0]=(1.65849097715)/(o+1.)+(-0.777022017999)+(-0.507872937196)*0.5**o
        ref[2,2,0,1]=(0.242698257917)/(o+1.)+(1.08992054229)+(-0.481403986338)*0.5**o
        ref[2,2,1,0]=(-0.754639453272)/(o+1.)+(0.81286785715)+(0.241675645258)*0.5**o
        ref[2,2,1,1]=(-0.438727562295)/(o+1.)+(0.217076971211)+(0.114772627763)*0.5**o
        ref[2,2,2,0]=(0.683917726753)/(o+1.)+(-0.781863136791)+(0.887023209688)*0.5**o
        ref[2,2,2,1]=(1.19838469898)/(o+1.)+(0.729462151403)+(0.836731324893)*0.5**o
        ref[2,3,0,0]=(1.28206834695)/(o+1.)+(0.583576731084)+(0.196161020545)*0.5**o
        ref[2,3,0,1]=(0.395309853285)/(o+1.)+(0.573307319258)+(-0.862326385376)*0.5**o
        ref[2,3,1,0]=(1.34760990633)/(o+1.)+(0.348652101929)+(0.571263839862)*0.5**o
        ref[2,3,1,1]=(0.293682752388)/(o+1.)+(0.080258501461)+(0.272559456412)*0.5**o
        ref[2,3,2,0]=(-1.47056068107)/(o+1.)+(-0.237909711446)+(0.3409363126)*0.5**o
        ref[2,3,2,1]=(-0.356052894952)/(o+1.)+(-0.166196236302)+(0.949170673623)*0.5**o
        ref[2,4,0,0]=(-0.21887693849)/(o+1.)+(0.365445238731)+(0.876774885889)*0.5**o
        ref[2,4,0,1]=(0.824663735425)/(o+1.)+(-0.568245545547)+(0.347441329383)*0.5**o
        ref[2,4,1,0]=(-1.22807417581)/(o+1.)+(-0.0380844812022)+(0.732123350436)*0.5**o
        ref[2,4,1,1]=(0.45997712447)/(o+1.)+(0.181215330541)+(-0.128872747083)*0.5**o
        ref[2,4,2,0]=(0.282057449847)/(o+1.)+(0.49247247384)+(0.619034818105)*0.5**o
        ref[2,4,2,1]=(0.0852215499679)/(o+1.)+(0.0807247245641)+(0.645979236759)*0.5**o
        ref[3,0,0,0]=(-0.173183992256)/(o+1.)+(0.215110333109)+(-0.265836015694)*0.5**o
        ref[3,0,0,1]=(0.88174458564)/(o+1.)+(0.282289419133)+(0.306246512933)*0.5**o
        ref[3,0,1,0]=(-0.542918101609)/(o+1.)+(0.791816447658)+(0.509420593773)*0.5**o
        ref[3,0,1,1]=(0.585362427423)/(o+1.)+(-0.229639545331)+(-0.609626065482)*0.5**o
        ref[3,0,2,0]=(-0.625654730672)/(o+1.)+(0.690821759706)+(0.269212290711)*0.5**o
        ref[3,0,2,1]=(0.971993211744)/(o+1.)+(-1.18192725526)+(-0.445116541697)*0.5**o
        ref[3,1,0,0]=(-1.04249026287)/(o+1.)+(0.0447480055334)+(0.25585673506)*0.5**o
        ref[3,1,0,1]=(-1.30475672494)/(o+1.)+(0.700582633156)+(-0.159301665223)*0.5**o
        ref[3,1,1,0]=(0.25661740363)/(o+1.)+(-0.036124371669)+(0.673375237931)*0.5**o
        ref[3,1,1,1]=(1.24932735533)/(o+1.)+(-0.597633535843)+(0.273992606955)*0.5**o
        ref[3,1,2,0]=(0.116260708817)/(o+1.)+(0.813874160367)+(-0.533129977534)*0.5**o
        ref[3,1,2,1]=(1.30199813466)/(o+1.)+(0.377333042401)+(0.977824314677)*0.5**o
        ref[3,2,0,0]=(1.03411019508)/(o+1.)+(0.386114811649)+(0.969349250234)*0.5**o
        ref[3,2,0,1]=(0.114450341594)/(o+1.)+(0.288108255139)+(-0.673721580725)*0.5**o
        ref[3,2,1,0]=(0.799847650292)/(o+1.)+(-0.0983585840958)+(-0.432873699676)*0.5**o
        ref[3,2,1,1]=(-1.18814224634)/(o+1.)+(-0.696858644421)+(0.633726614581)*0.5**o
        ref[3,2,2,0]=(0.626385522485)/(o+1.)+(0.0881472172038)+(-0.182700878265)*0.5**o
        ref[3,2,2,1]=(0.27320795045)/(o+1.)+(0.0047710521094)+(0.999790796581)*0.5**o
        ref[3,3,0,0]=(0.0826963327674)/(o+1.)+(-0.666702246557)+(-0.959143897162)*0.5**o
        ref[3,3,0,1]=(0.320563187816)/(o+1.)+(0.774809717833)+(0.385513188376)*0.5**o
        ref[3,3,1,0]=(-1.80695590229)/(o+1.)+(0.214248941855)+(0.809269330969)*0.5**o
        ref[3,3,1,1]=(-0.146050130242)/(o+1.)+(-0.625770524147)+(-0.983474390608)*0.5**o
        ref[3,3,2,0]=(0.956779785308)/(o+1.)+(0.138176772609)+(0.710820041642)*0.5**o
        ref[3,3,2,1]=(-1.39320588501)/(o+1.)+(0.571745087644)+(0.633364771689)*0.5**o
        ref[3,4,0,0]=(0.828689828045)/(o+1.)+(-0.821398677259)+(0.44039719913)*0.5**o
        ref[3,4,0,1]=(1.50194037834)/(o+1.)+(-0.535574620947)+(0.253149910502)*0.5**o
        ref[3,4,1,0]=(-0.708775097383)/(o+1.)+(-0.337919894074)+(0.13027869782)*0.5**o
        ref[3,4,1,1]=(1.17387882267)/(o+1.)+(0.234301154085)+(0.476554240918)*0.5**o
        ref[3,4,2,0]=(0.891111463851)/(o+1.)+(0.221627765254)+(0.800058524963)*0.5**o
        ref[3,4,2,1]=(-0.0533896507631)/(o+1.)+(0.551652029667)+(-0.533834741079)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactZero_fromData_ReducedSolution_rank0(self):
      """
      tests integral of rank 0 Data on the ReducedFunctionOnContactZero

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Data(0,(),w)
      if dim==2:
        arg=(-0.570707549273)*x[0]+(-0.726264032186)*x[1]
        ref=(-0.0423765208873)/(o+1.)+(-0.791542656673)+(0.328490252775)*0.5**o
      else:
        arg=(1.28222892931)*x[0]+(-0.798170675576)*x[1]+(1.5701208405)*x[2]
        ref=(0.721975372868)/(o+1.)+(0.344720905832)+(0.642761909705)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactZero_fromData_ReducedSolution_rank1(self):
      """
      tests integral of rank 1 Data on the ReducedFunctionOnContactZero

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Data(0,(2,),w)
      ref=numpy.zeros((2,),numpy.float_)
      if dim==2:
        arg[0]=(0.575166906798)*x[0]+(-1.63698773418)*x[1]
        arg[1]=(-0.0535946584636)*x[0]+(0.105969101775)*x[1]
        ref[0]=(-0.751926426344)/(o+1.)+(-0.395800138888)+(0.481705876734)*0.5**o
        ref[1]=(0.698496252242)/(o+1.)+(-0.265927161866)+(-0.114267485199)*0.5**o
      else:
        arg[0]=(-0.17879254138)*x[0]+(-1.12752123792)*x[1]+(0.921297311605)*x[2]
        arg[1]=(1.33131824618)*x[0]+(-0.392175059176)*x[1]+(0.545754281465)*x[2]
        ref[0]=(-0.494870303061)/(o+1.)+(0.0697636237654)+(-0.0296734121631)*0.5**o
        ref[1]=(-0.394300983704)/(o+1.)+(0.722261312754)+(0.434675826665)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactZero_fromData_ReducedSolution_rank2(self):
      """
      tests integral of rank 2 Data on the ReducedFunctionOnContactZero

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5),w)
      ref=numpy.zeros((4, 5),numpy.float_)
      if dim==2:
        arg[0,0]=(-0.338658191734)*x[0]+(1.46127609683)*x[1]
        arg[0,1]=(-0.629116867491)*x[0]+(1.40141569066)*x[1]
        arg[0,2]=(0.105569779347)*x[0]+(0.239936324967)*x[1]
        arg[0,3]=(-0.281718331718)*x[0]+(0.280323404866)*x[1]
        arg[0,4]=(0.467341113661)*x[0]+(-0.50452062075)*x[1]
        arg[1,0]=(-0.841295219186)*x[0]+(0.524038163723)*x[1]
        arg[1,1]=(1.40143704236)*x[0]+(-1.50467961637)*x[1]
        arg[1,2]=(1.21078790505)*x[0]+(-0.731453470292)*x[1]
        arg[1,3]=(0.469556922076)*x[0]+(0.0686293204391)*x[1]
        arg[1,4]=(-1.26394637585)*x[0]+(-0.0361395276585)*x[1]
        arg[2,0]=(-1.12850883148)*x[0]+(0.284296673326)*x[1]
        arg[2,1]=(-0.626362828813)*x[0]+(1.19459827573)*x[1]
        arg[2,2]=(-0.545457293083)*x[0]+(1.547337019)*x[1]
        arg[2,3]=(0.664461986629)*x[0]+(-1.69082198266)*x[1]
        arg[2,4]=(-0.100004763305)*x[0]+(0.152159044423)*x[1]
        arg[3,0]=(0.307574843061)*x[0]+(1.20209181301)*x[1]
        arg[3,1]=(-0.0663203356627)*x[0]+(-1.26669863527)*x[1]
        arg[3,2]=(-1.20436798894)*x[0]+(-0.0375693379661)*x[1]
        arg[3,3]=(0.753649087747)*x[0]+(0.69376223486)*x[1]
        arg[3,4]=(-0.0512732264834)*x[0]+(-0.658097606436)*x[1]
        ref[0,0]=(0.996821439952)/(o+1.)+(0.015088160779)+(0.0956201435879)*0.5**o
        ref[0,1]=(0.643092963726)/(o+1.)+(0.239390724844)+(-0.349575590244)*0.5**o
        ref[0,2]=(-0.198468008977)/(o+1.)+(0.00411201743342)+(0.535750078425)*0.5**o
        ref[0,3]=(0.642491201137)/(o+1.)+(-0.623413199652)+(0.602940271315)*0.5**o
        ref[0,4]=(-0.105706120549)/(o+1.)+(-0.353560054939)+(0.775646723338)*0.5**o
        ref[1,0]=(0.144540669221)/(o+1.)+(0.0228268215803)+(-0.507451367845)*0.5**o
        ref[1,1]=(-0.820379339897)/(o+1.)+(0.0951825678509)+(0.526771630185)*0.5**o
        ref[1,2]=(-0.21214585824)/(o+1.)+(-0.0889183062985)+(0.869316905591)*0.5**o
        ref[1,3]=(0.291228940674)/(o+1.)+(0.192623928703)+(-0.138290555565)*0.5**o
        ref[1,4]=(-0.0663718956434)/(o+1.)+(-0.309484335614)+(-0.614745336642)*0.5**o
        ref[2,0]=(0.484892962201)/(o+1.)+(-0.367129859793)+(-0.594845400763)*0.5**o
        ref[2,1]=(0.768457634672)/(o+1.)+(0.154447657562)+(-0.509117502876)*0.5**o
        ref[2,2]=(0.707320338363)/(o+1.)+(-0.0592149232183)+(0.412989233992)*0.5**o
        ref[2,3]=(-0.983684339171)/(o+1.)+(-0.495377645598)+(0.948079634337)*0.5**o
        ref[2,4]=(-0.0315947220465)/(o+1.)+(0.506833986759)+(-0.929918970353)*0.5**o
        ref[3,0]=(0.54141067098)/(o+1.)+(0.489049131493)+(-0.00984227789859)*0.5**o
        ref[3,1]=(-0.552335053051)/(o+1.)+(-0.383325638803)+(-0.0140326402734)*0.5**o
        ref[3,2]=(0.0821472412575)/(o+1.)+(-0.335827719718)+(-0.652429128733)*0.5**o
        ref[3,3]=(0.112917208924)/(o+1.)+(0.667772807338)+(-0.00105150099312)*0.5**o
        ref[3,4]=(-0.19331508157)/(o+1.)+(-0.102296385708)+(-0.311462979932)*0.5**o
      else:
        arg[0,0]=(0.069245963142)*x[0]+(-0.185727379494)*x[1]+(-0.887410889699)*x[2]
        arg[0,1]=(-0.816745668949)*x[0]+(-0.250435680662)*x[1]+(0.701607830582)*x[2]
        arg[0,2]=(-0.862433954308)*x[0]+(0.731874372527)*x[1]+(-0.115384982779)*x[2]
        arg[0,3]=(-0.265247631604)*x[0]+(-0.157059741958)*x[1]+(-0.368419335078)*x[2]
        arg[0,4]=(1.66720203567)*x[0]+(-0.759393708719)*x[1]+(1.36360850704)*x[2]
        arg[1,0]=(-0.192822813795)*x[0]+(0.0358776152527)*x[1]+(-0.722864520788)*x[2]
        arg[1,1]=(-0.480606215471)*x[0]+(1.13682774573)*x[1]+(0.17900467623)*x[2]
        arg[1,2]=(0.346760852584)*x[0]+(0.994833030281)*x[1]+(0.507055933913)*x[2]
        arg[1,3]=(-0.509961853478)*x[0]+(-0.852997575505)*x[1]+(0.586527030253)*x[2]
        arg[1,4]=(0.93957975846)*x[0]+(0.28878335152)*x[1]+(0.128070931738)*x[2]
        arg[2,0]=(0.338283306313)*x[0]+(0.0244500761608)*x[1]+(-0.606875554888)*x[2]
        arg[2,1]=(0.850782624552)*x[0]+(-0.836382038189)*x[1]+(-1.41808459947)*x[2]
        arg[2,2]=(-1.06176229613)*x[0]+(-0.259247046061)*x[1]+(-0.583455484089)*x[2]
        arg[2,3]=(1.17437923282)*x[0]+(1.31485601495)*x[1]+(1.3139743231)*x[2]
        arg[2,4]=(0.362851636275)*x[0]+(0.804049783156)*x[1]+(1.09205000214)*x[2]
        arg[3,0]=(0.343603837195)*x[0]+(-1.49082257981)*x[1]+(-0.628668674545)*x[2]
        arg[3,1]=(-0.676847711765)*x[0]+(1.32350560453)*x[1]+(1.07275185686)*x[2]
        arg[3,2]=(0.570784985469)*x[0]+(-1.01212360602)*x[1]+(-0.0808776667606)*x[2]
        arg[3,3]=(0.506392444236)*x[0]+(-1.00848318041)*x[1]+(0.558777901541)*x[2]
        arg[3,4]=(-0.60925738291)*x[0]+(-0.386755957809)*x[1]+(0.60930115434)*x[2]
        ref[0,0]=(-0.695230136419)/(o+1.)+(-0.388400729083)+(0.468139288533)*0.5**o
        ref[0,1]=(0.462098132187)/(o+1.)+(-0.215041893287)+(-0.397587864642)*0.5**o
        ref[0,2]=(-0.262411385363)/(o+1.)+(0.275729226874)+(-0.534991632946)*0.5**o
        ref[0,3]=(0.117822633378)/(o+1.)+(0.0286827897592)+(-0.965914921536)*0.5**o
        ref[0,4]=(0.293575681039)/(o+1.)+(0.584039692827)+(0.809761767299)*0.5**o
        ref[1,0]=(0.627951922463)/(o+1.)+(-0.994889162557)+(0.482016683321)*0.5**o
        ref[1,1]=(1.83482238662)/(o+1.)+(-0.205719558739)+(-0.588157062647)*0.5**o
        ref[1,2]=(1.13612020398)/(o+1.)+(0.0262824938134)+(0.659964625174)*0.5**o
        ref[1,3]=(0.325264487741)/(o+1.)+(-0.0752154813413)+(-0.951265923787)*0.5**o
        ref[1,4]=(-0.149919283302)/(o+1.)+(0.77701502585)+(-0.0476767266789)*0.5**o
        ref[2,0]=(0.491504686171)/(o+1.)+(-0.562534706926)+(0.389422555265)*0.5**o
        ref[2,1]=(-1.52704217417)/(o+1.)+(0.120525042098)+(-0.117691923134)*0.5**o
        ref[2,2]=(-0.779796772543)/(o+1.)+(-0.318864997267)+(-0.486938059201)*0.5**o
        ref[2,3]=(1.53343318682)/(o+1.)+(0.731149467308)+(0.807477449443)*0.5**o
        ref[2,4]=(1.15919207694)/(o+1.)+(0.113921448815)+(0.871916446996)*0.5**o
        ref[3,0]=(-1.75312920944)/(o+1.)+(-0.153798982308)+(0.284839756895)*0.5**o
        ref[3,1]=(1.30985814569)/(o+1.)+(0.438430874237)+(-0.467310144534)*0.5**o
        ref[3,2]=(-0.737480320972)/(o+1.)+(0.0604403094397)+(0.094383414777)*0.5**o
        ref[3,3]=(-0.688723266524)/(o+1.)+(0.43486027018)+(-0.124310108473)*0.5**o
        ref[3,4]=(0.128399262674)/(o+1.)+(-0.372445844122)+(0.229780239192)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactZero_fromData_ReducedSolution_rank3(self):
      """
      tests integral of rank 3 Data on the ReducedFunctionOnContactZero

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Data(0,(6, 2, 2),w)
      ref=numpy.zeros((6, 2, 2),numpy.float_)
      if dim==2:
        arg[0,0,0]=(-0.993339371954)*x[0]+(-1.35308128985)*x[1]
        arg[0,0,1]=(-1.07886149134)*x[0]+(-1.38999283206)*x[1]
        arg[0,1,0]=(0.246468026972)*x[0]+(-1.37217464882)*x[1]
        arg[0,1,1]=(0.645672648599)*x[0]+(1.8070624103)*x[1]
        arg[1,0,0]=(-1.02278989388)*x[0]+(0.151634023103)*x[1]
        arg[1,0,1]=(-0.667332860541)*x[0]+(0.312742149193)*x[1]
        arg[1,1,0]=(-0.848647385832)*x[0]+(0.466438038516)*x[1]
        arg[1,1,1]=(-0.185495911372)*x[0]+(-0.137118786891)*x[1]
        arg[2,0,0]=(-0.0250555870731)*x[0]+(0.905553573283)*x[1]
        arg[2,0,1]=(-1.03940852939)*x[0]+(-1.57326016394)*x[1]
        arg[2,1,0]=(1.48834718811)*x[0]+(-0.340889253246)*x[1]
        arg[2,1,1]=(0.875721514739)*x[0]+(-0.310857412212)*x[1]
        arg[3,0,0]=(1.25129508831)*x[0]+(-1.69312043525)*x[1]
        arg[3,0,1]=(1.50124675312)*x[0]+(0.244195748654)*x[1]
        arg[3,1,0]=(0.292060478909)*x[0]+(-0.660005422696)*x[1]
        arg[3,1,1]=(0.791335231308)*x[0]+(0.466481573591)*x[1]
        arg[4,0,0]=(-0.361241947236)*x[0]+(-1.43591648587)*x[1]
        arg[4,0,1]=(-1.10385456695)*x[0]+(-0.325314195825)*x[1]
        arg[4,1,0]=(1.01004173033)*x[0]+(0.426824552857)*x[1]
        arg[4,1,1]=(-0.631382479612)*x[0]+(0.0603792190966)*x[1]
        arg[5,0,0]=(-0.30404368786)*x[0]+(-0.0102498522932)*x[1]
        arg[5,0,1]=(0.109188848746)*x[0]+(-1.16846160105)*x[1]
        arg[5,1,0]=(-0.351916879547)*x[0]+(-0.320714795448)*x[1]
        arg[5,1,1]=(-0.534138562652)*x[0]+(-0.256013589844)*x[1]
        ref[0,0,0]=(-0.506159375456)/(o+1.)+(-0.522895228783)+(-0.794470828783)*0.5**o
        ref[0,0,1]=(-0.930984367068)/(o+1.)+(-0.636851258685)+(-0.264167438968)*0.5**o
        ref[0,1,0]=(-0.674222792172)/(o+1.)+(-0.52131379048)+(0.591143751282)*0.5**o
        ref[0,1,1]=(0.953370751283)/(o+1.)+(0.377773224645)+(0.743817858326)*0.5**o
        ref[1,0,0]=(0.700508215428)/(o+1.)+(-0.303343530867)+(-0.964977024471)*0.5**o
        ref[1,0,1]=(-0.408185116452)/(o+1.)+(0.0668983569743)+(-0.0802023088445)*0.5**o
        ref[1,1,0]=(0.80742345527)/(o+1.)+(-0.405780705683)+(-0.37807139122)*0.5**o
        ref[1,1,1]=(0.0184615568095)/(o+1.)+(0.221921562266)+(-0.784919379605)*0.5**o
        ref[2,0,0]=(0.571906256883)/(o+1.)+(-0.0415588382133)+(0.391709405753)*0.5**o
        ref[2,0,1]=(-0.778176009451)/(o+1.)+(-0.761680990566)+(-0.311130702752)*0.5**o
        ref[2,1,0]=(0.498214686321)/(o+1.)+(-0.146173705483)+(0.941590659509)*0.5**o
        ref[2,1,1]=(-0.273747071818)/(o+1.)+(-0.0319184747153)+(0.902448123776)*0.5**o
        ref[3,0,0]=(-0.730029471178)/(o+1.)+(-0.288672747933)+(0.865549620109)*0.5**o
        ref[3,0,1]=(-0.371409186475)/(o+1.)+(0.623169375401)+(0.870512937445)*0.5**o
        ref[3,1,0]=(0.0611454391867)/(o+1.)+(-0.627723874214)+(0.826357365454)*0.5**o
        ref[3,1,1]=(0.785553447935)/(o+1.)+(0.246940213405)+(-0.021617069845)*0.5**o
        ref[4,0,0]=(-0.936201955907)/(o+1.)+(-0.484823720021)+(0.108690962842)*0.5**o
        ref[4,0,1]=(-0.277830390815)/(o+1.)+(-0.151786547071)+(-0.847765277819)*0.5**o
        ref[4,1,0]=(0.12293099533)/(o+1.)+(0.295620382054)+(0.722694523749)*0.5**o
        ref[4,1,1]=(-0.5517019185)/(o+1.)+(0.0850748937088)+(-0.189451129433)*0.5**o
        ref[5,0,0]=(-0.942533148805)/(o+1.)+(0.425680504872)+(-0.223121401092)*0.5**o
        ref[5,0,1]=(-0.377134228332)/(o+1.)+(-0.721019409569)+(0.759900295162)*0.5**o
        ref[5,1,0]=(-0.643828391942)/(o+1.)+(0.399974456311)+(-0.828752195674)*0.5**o
        ref[5,1,1]=(-0.118293731439)/(o+1.)+(-0.355568544431)+(0.0392786678054)*0.5**o
      else:
        arg[0,0,0]=(0.205446621454)*x[0]+(0.10254878667)*x[1]+(1.27673728979)*x[2]
        arg[0,0,1]=(0.758846313193)*x[0]+(0.63320336713)*x[1]+(-0.42638505807)*x[2]
        arg[0,1,0]=(-0.0127116561603)*x[0]+(-0.360359420027)*x[1]+(0.540508591493)*x[2]
        arg[0,1,1]=(-0.278918629538)*x[0]+(-0.0291534835772)*x[1]+(0.708655731565)*x[2]
        arg[1,0,0]=(-0.612965210483)*x[0]+(0.787787556912)*x[1]+(0.222384448462)*x[2]
        arg[1,0,1]=(1.07409083742)*x[0]+(1.19429173706)*x[1]+(-0.484979976779)*x[2]
        arg[1,1,0]=(-1.25714999157)*x[0]+(-0.321346019622)*x[1]+(0.278329320837)*x[2]
        arg[1,1,1]=(-0.512262689707)*x[0]+(1.6891992053)*x[1]+(-1.02351830068)*x[2]
        arg[2,0,0]=(-1.57911907773)*x[0]+(0.096124258122)*x[1]+(0.336619253521)*x[2]
        arg[2,0,1]=(0.0901144254199)*x[0]+(-1.36818186255)*x[1]+(1.2969609239)*x[2]
        arg[2,1,0]=(0.158329511499)*x[0]+(0.696997185217)*x[1]+(-0.400535235927)*x[2]
        arg[2,1,1]=(-1.44570853201)*x[0]+(0.237359587195)*x[1]+(-1.17323024801)*x[2]
        arg[3,0,0]=(-1.4325812824)*x[0]+(-0.733085265902)*x[1]+(0.125883186484)*x[2]
        arg[3,0,1]=(-0.0529889021435)*x[0]+(0.335154600942)*x[1]+(0.839337375209)*x[2]
        arg[3,1,0]=(-0.423552182963)*x[0]+(1.31813289315)*x[1]+(0.13235854915)*x[2]
        arg[3,1,1]=(0.53278572217)*x[0]+(0.159434073071)*x[1]+(-0.313621199136)*x[2]
        arg[4,0,0]=(0.747779887673)*x[0]+(0.30977483911)*x[1]+(-0.0760295896739)*x[2]
        arg[4,0,1]=(0.00295118909194)*x[0]+(-0.687914867842)*x[1]+(0.0386268422915)*x[2]
        arg[4,1,0]=(-0.276363532754)*x[0]+(0.52102466785)*x[1]+(-0.684840191917)*x[2]
        arg[4,1,1]=(0.0283138898538)*x[0]+(0.116042458294)*x[1]+(0.485578401528)*x[2]
        arg[5,0,0]=(-0.118402060192)*x[0]+(-1.18409809944)*x[1]+(-0.304048852342)*x[2]
        arg[5,0,1]=(0.174393790186)*x[0]+(-0.524912689263)*x[1]+(-0.648708392355)*x[2]
        arg[5,1,0]=(-0.254958756344)*x[0]+(-0.772071348728)*x[1]+(-0.0572494064027)*x[2]
        arg[5,1,1]=(-0.74777072018)*x[0]+(0.19109939364)*x[1]+(0.498973987814)*x[2]
        ref[0,0,0]=(0.986457444152)/(o+1.)+(0.258994989412)+(0.0802852749346)*0.5**o
        ref[0,0,1]=(-0.0286717143415)/(o+1.)+(0.554867138611)+(-0.115397940627)*0.5**o
        ref[0,1,0]=(-0.886482131138)/(o+1.)+(0.464293129749)+(0.125333386946)*0.5**o
        ref[0,1,1]=(0.199214164457)/(o+1.)+(0.336882900218)+(-0.472396346443)*0.5**o
        ref[1,0,0]=(0.36021680954)/(o+1.)+(0.408903924703)+(-0.780817864055)*0.5**o
        ref[1,0,1]=(0.44795581074)/(o+1.)+(0.472159267742)+(0.391128251477)*0.5**o
        ref[1,1,0]=(1.02665734893)/(o+1.)+(-0.71418625728)+(-0.898451524719)*0.5**o
        ref[1,1,1]=(0.0322676607777)/(o+1.)+(0.105378958385)+(-0.0896073626306)*0.5**o
        ref[2,0,0]=(1.15723247637)/(o+1.)+(-0.719604538702)+(-0.86439896505)*0.5**o
        ref[2,0,1]=(-0.254100181576)/(o+1.)+(-0.0387997401585)+(0.35059314866)*0.5**o
        ref[2,1,0]=(-0.636439167982)/(o+1.)+(0.563390681981)+(-0.0355507351918)*0.5**o
        ref[2,1,1]=(-0.265265225142)/(o+1.)+(-0.635620798949)+(-0.845072369786)*0.5**o
        ref[3,0,0]=(-0.062162800164)/(o+1.)+(-0.554499928282)+(-0.868620705087)*0.5**o
        ref[3,0,1]=(0.784681961813)/(o+1.)+(0.355202655131)+(-0.373584198067)*0.5**o
        ref[3,1,0]=(1.20272043429)/(o+1.)+(0.352677694103)+(-0.881136563162)*0.5**o
        ref[3,1,1]=(-0.311152797156)/(o+1.)+(0.313736563953)+(0.0622782653549)*0.5**o
        ref[4,0,0]=(-0.365517620238)/(o+1.)+(0.733502174253)+(-0.119961591158)*0.5**o
        ref[4,0,1]=(0.0155845896619)/(o+1.)+(-0.654963543839)+(0.648005661558)*0.5**o
        ref[4,1,0]=(0.405697975915)/(o+1.)+(-0.637805133954)+(0.429733235172)*0.5**o
        ref[4,1,1]=(-0.0469738326054)/(o+1.)+(0.0590721074184)+(0.558764367445)*0.5**o
        ref[5,0,0]=(-0.333825007954)/(o+1.)+(-0.46448520882)+(-0.343753586377)*0.5**o
        ref[5,0,1]=(-0.469125954795)/(o+1.)+(-0.147192309255)+(-0.235716718127)*0.5**o
        ref[5,1,0]=(-0.573275891777)/(o+1.)+(-0.478328163121)+(0.445652706545)*0.5**o
        ref[5,1,1]=(0.0545515234726)/(o+1.)+(-0.153919681346)+(0.195590500493)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactZero_fromData_ReducedSolution_rank4(self):
      """
      tests integral of rank 4 Data on the ReducedFunctionOnContactZero

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5, 3, 2),w)
      ref=numpy.zeros((4, 5, 3, 2),numpy.float_)
      if dim==2:
        arg[0,0,0,0]=(-0.226142137728)*x[0]+(0.85681895183)*x[1]
        arg[0,0,0,1]=(-0.824407181762)*x[0]+(-0.290023144892)*x[1]
        arg[0,0,1,0]=(-1.25987816017)*x[0]+(0.824340990595)*x[1]
        arg[0,0,1,1]=(-0.0917811606765)*x[0]+(0.660211932964)*x[1]
        arg[0,0,2,0]=(-1.31073149178)*x[0]+(0.482324984918)*x[1]
        arg[0,0,2,1]=(0.256995458227)*x[0]+(0.79244254852)*x[1]
        arg[0,1,0,0]=(-0.0682147475407)*x[0]+(0.243620052061)*x[1]
        arg[0,1,0,1]=(0.071099982687)*x[0]+(-0.62986754897)*x[1]
        arg[0,1,1,0]=(-0.0716793733131)*x[0]+(-1.15235525814)*x[1]
        arg[0,1,1,1]=(0.193107575216)*x[0]+(-0.596149957543)*x[1]
        arg[0,1,2,0]=(-0.160142629886)*x[0]+(1.36581707067)*x[1]
        arg[0,1,2,1]=(-1.21477648376)*x[0]+(-0.732000298369)*x[1]
        arg[0,2,0,0]=(-1.02468971059)*x[0]+(0.330721595217)*x[1]
        arg[0,2,0,1]=(1.11414907391)*x[0]+(1.29386127812)*x[1]
        arg[0,2,1,0]=(1.1485850154)*x[0]+(0.387585977495)*x[1]
        arg[0,2,1,1]=(0.237382583072)*x[0]+(1.56248221279)*x[1]
        arg[0,2,2,0]=(-0.949129049229)*x[0]+(0.578823379063)*x[1]
        arg[0,2,2,1]=(-1.0667182537)*x[0]+(0.462227485959)*x[1]
        arg[0,3,0,0]=(1.37954377554)*x[0]+(0.541856880139)*x[1]
        arg[0,3,0,1]=(0.401584700319)*x[0]+(0.577102327147)*x[1]
        arg[0,3,1,0]=(-0.289796408108)*x[0]+(0.807853785325)*x[1]
        arg[0,3,1,1]=(0.0038467861822)*x[0]+(-0.931311510771)*x[1]
        arg[0,3,2,0]=(0.859036254821)*x[0]+(0.198884218516)*x[1]
        arg[0,3,2,1]=(-0.00310256070968)*x[0]+(0.511838475968)*x[1]
        arg[0,4,0,0]=(0.539686886802)*x[0]+(-0.324612924267)*x[1]
        arg[0,4,0,1]=(-0.0458655898565)*x[0]+(0.211810380966)*x[1]
        arg[0,4,1,0]=(0.302333533003)*x[0]+(0.0663033316831)*x[1]
        arg[0,4,1,1]=(-0.981519161639)*x[0]+(-0.00207756909914)*x[1]
        arg[0,4,2,0]=(0.283659447004)*x[0]+(-0.231526592979)*x[1]
        arg[0,4,2,1]=(-1.17679167875)*x[0]+(0.285698985756)*x[1]
        arg[1,0,0,0]=(-0.162900059582)*x[0]+(-0.877164340783)*x[1]
        arg[1,0,0,1]=(-0.50468605065)*x[0]+(-0.160440073709)*x[1]
        arg[1,0,1,0]=(-0.100901676644)*x[0]+(-0.296111526029)*x[1]
        arg[1,0,1,1]=(0.176107421869)*x[0]+(1.74393708745)*x[1]
        arg[1,0,2,0]=(-0.979807721703)*x[0]+(-0.229400841067)*x[1]
        arg[1,0,2,1]=(0.217472795645)*x[0]+(0.0881503243323)*x[1]
        arg[1,1,0,0]=(-0.127980123625)*x[0]+(0.0932095456541)*x[1]
        arg[1,1,0,1]=(0.829674410931)*x[0]+(-0.796196041601)*x[1]
        arg[1,1,1,0]=(1.28656110233)*x[0]+(-1.27850471392)*x[1]
        arg[1,1,1,1]=(1.159269884)*x[0]+(-0.973135433631)*x[1]
        arg[1,1,2,0]=(1.46274837856)*x[0]+(-1.81554270863)*x[1]
        arg[1,1,2,1]=(0.0973472511688)*x[0]+(0.551451261495)*x[1]
        arg[1,2,0,0]=(0.510667671293)*x[0]+(-0.51075982099)*x[1]
        arg[1,2,0,1]=(-1.41612093451)*x[0]+(0.614417669629)*x[1]
        arg[1,2,1,0]=(1.46595125796)*x[0]+(0.60374279122)*x[1]
        arg[1,2,1,1]=(-0.317185218187)*x[0]+(-0.0785532882696)*x[1]
        arg[1,2,2,0]=(-0.388504041993)*x[0]+(-0.838003398775)*x[1]
        arg[1,2,2,1]=(0.0497207129012)*x[0]+(-0.622155692247)*x[1]
        arg[1,3,0,0]=(-0.0471400179563)*x[0]+(1.3378948207)*x[1]
        arg[1,3,0,1]=(-1.22250822195)*x[0]+(0.629714783138)*x[1]
        arg[1,3,1,0]=(-1.28801080408)*x[0]+(-1.54079710038)*x[1]
        arg[1,3,1,1]=(-0.426412448469)*x[0]+(-1.09410474021)*x[1]
        arg[1,3,2,0]=(0.022834167351)*x[0]+(-0.497423389788)*x[1]
        arg[1,3,2,1]=(-0.787492124301)*x[0]+(-0.245621168419)*x[1]
        arg[1,4,0,0]=(-1.37814880919)*x[0]+(-0.704805848451)*x[1]
        arg[1,4,0,1]=(-1.12494770055)*x[0]+(1.44920024305)*x[1]
        arg[1,4,1,0]=(-0.809532572761)*x[0]+(1.16638587657)*x[1]
        arg[1,4,1,1]=(0.931784486816)*x[0]+(1.34629889453)*x[1]
        arg[1,4,2,0]=(-0.172430942793)*x[0]+(-0.584736998321)*x[1]
        arg[1,4,2,1]=(0.00542948736389)*x[0]+(0.43917597284)*x[1]
        arg[2,0,0,0]=(-0.946122337406)*x[0]+(-0.161940168137)*x[1]
        arg[2,0,0,1]=(-0.7856653779)*x[0]+(-0.75287701788)*x[1]
        arg[2,0,1,0]=(-1.08858367592)*x[0]+(-0.530380818694)*x[1]
        arg[2,0,1,1]=(-1.0573555095)*x[0]+(-0.854148765785)*x[1]
        arg[2,0,2,0]=(0.723412825328)*x[0]+(0.326761950437)*x[1]
        arg[2,0,2,1]=(-0.0143142161225)*x[0]+(-0.752724311876)*x[1]
        arg[2,1,0,0]=(-0.942548092355)*x[0]+(0.700861094897)*x[1]
        arg[2,1,0,1]=(-0.712286567957)*x[0]+(0.230406545738)*x[1]
        arg[2,1,1,0]=(1.27414839774)*x[0]+(0.724784649037)*x[1]
        arg[2,1,1,1]=(0.637494163002)*x[0]+(0.442489599192)*x[1]
        arg[2,1,2,0]=(1.81617631726)*x[0]+(1.56870218809)*x[1]
        arg[2,1,2,1]=(1.16419900666)*x[0]+(-0.044314729141)*x[1]
        arg[2,2,0,0]=(0.880267125196)*x[0]+(0.77124406259)*x[1]
        arg[2,2,0,1]=(0.482402867542)*x[0]+(0.0746924305017)*x[1]
        arg[2,2,1,0]=(0.105861259696)*x[0]+(0.484292207267)*x[1]
        arg[2,2,1,1]=(-0.107785838196)*x[0]+(0.701132243209)*x[1]
        arg[2,2,2,0]=(0.559717405844)*x[0]+(-1.5585942717)*x[1]
        arg[2,2,2,1]=(-0.989130712605)*x[0]+(0.221361137556)*x[1]
        arg[2,3,0,0]=(-0.045471818095)*x[0]+(-0.437941959792)*x[1]
        arg[2,3,0,1]=(1.25840468379)*x[0]+(-1.18872011057)*x[1]
        arg[2,3,1,0]=(1.70665829408)*x[0]+(0.705273921003)*x[1]
        arg[2,3,1,1]=(-0.0928709253845)*x[0]+(0.785115449756)*x[1]
        arg[2,3,2,0]=(-0.887243418838)*x[0]+(1.06790660642)*x[1]
        arg[2,3,2,1]=(-0.572644686872)*x[0]+(-0.672831948394)*x[1]
        arg[2,4,0,0]=(1.28282566801)*x[0]+(0.651985199447)*x[1]
        arg[2,4,0,1]=(0.0965663229335)*x[0]+(1.33819998052)*x[1]
        arg[2,4,1,0]=(0.205569041098)*x[0]+(-0.873095358733)*x[1]
        arg[2,4,1,1]=(-1.04452025745)*x[0]+(-0.204917033907)*x[1]
        arg[2,4,2,0]=(-0.181163938972)*x[0]+(0.0341993409272)*x[1]
        arg[2,4,2,1]=(1.68327027059)*x[0]+(-0.289436355642)*x[1]
        arg[3,0,0,0]=(1.05712810731)*x[0]+(0.915711819735)*x[1]
        arg[3,0,0,1]=(-0.171072039482)*x[0]+(-0.160656943623)*x[1]
        arg[3,0,1,0]=(-1.27063799869)*x[0]+(-0.605131642813)*x[1]
        arg[3,0,1,1]=(0.940512990835)*x[0]+(-0.116675577531)*x[1]
        arg[3,0,2,0]=(-0.194213869952)*x[0]+(0.364066752025)*x[1]
        arg[3,0,2,1]=(-0.328780465397)*x[0]+(-0.114689413673)*x[1]
        arg[3,1,0,0]=(-0.825914305895)*x[0]+(-0.167010056731)*x[1]
        arg[3,1,0,1]=(0.657779056207)*x[0]+(0.0666252451638)*x[1]
        arg[3,1,1,0]=(-0.464004426036)*x[0]+(-0.340932918409)*x[1]
        arg[3,1,1,1]=(-1.6698652667)*x[0]+(0.731870373141)*x[1]
        arg[3,1,2,0]=(1.39626250779)*x[0]+(-0.879152855557)*x[1]
        arg[3,1,2,1]=(1.34973919508)*x[0]+(-0.129753332761)*x[1]
        arg[3,2,0,0]=(1.03443450706)*x[0]+(0.807333477811)*x[1]
        arg[3,2,0,1]=(0.366011302596)*x[0]+(0.543804800926)*x[1]
        arg[3,2,1,0]=(0.262446846036)*x[0]+(-0.247786850991)*x[1]
        arg[3,2,1,1]=(-1.04657091196)*x[0]+(-0.485443426155)*x[1]
        arg[3,2,2,0]=(0.342434327538)*x[0]+(-0.148503585588)*x[1]
        arg[3,2,2,1]=(-0.189833429716)*x[0]+(0.424391874819)*x[1]
        arg[3,3,0,0]=(-0.118348838118)*x[0]+(0.397628264341)*x[1]
        arg[3,3,0,1]=(0.481440510084)*x[0]+(-0.393576221424)*x[1]
        arg[3,3,1,0]=(0.907533552208)*x[0]+(-0.668646549376)*x[1]
        arg[3,3,1,1]=(-1.49731714213)*x[0]+(-1.18710678264)*x[1]
        arg[3,3,2,0]=(-0.460660079207)*x[0]+(1.48078266444)*x[1]
        arg[3,3,2,1]=(-0.161073556988)*x[0]+(-0.486827042754)*x[1]
        arg[3,4,0,0]=(-0.668755455623)*x[0]+(0.345804059434)*x[1]
        arg[3,4,0,1]=(1.00528889193)*x[0]+(0.978507928191)*x[1]
        arg[3,4,1,0]=(0.98955856331)*x[0]+(1.0421852144)*x[1]
        arg[3,4,1,1]=(0.711688260873)*x[0]+(1.39962222322)*x[1]
        arg[3,4,2,0]=(1.08512429209)*x[0]+(-0.555249097743)*x[1]
        arg[3,4,2,1]=(-1.10443784007)*x[0]+(-0.782274429169)*x[1]
        ref[0,0,0,0]=(0.151153165564)/(o+1.)+(0.112609656739)+(0.254304335059)*0.5**o
        ref[0,0,0,1]=(-0.530602583511)/(o+1.)+(0.1817855378)+(-0.947398818743)*0.5**o
        ref[0,0,1,0]=(0.308747437738)/(o+1.)+(-0.0338129255643)+(-0.676658756184)*0.5**o
        ref[0,0,1,1]=(0.321026553427)/(o+1.)+(0.0692705367618)+(0.108863145337)*0.5**o
        ref[0,0,2,0]=(-0.0908034666655)/(o+1.)+(-0.1100205155)+(-0.517562009199)*0.5**o
        ref[0,0,2,1]=(0.303456040903)/(o+1.)+(0.500090875336)+(-0.254199784829)*0.5**o
        ref[0,1,0,0]=(-0.440641221831)/(o+1.)+(0.456218946873)+(-0.296391367396)*0.5**o
        ref[0,1,0,1]=(0.222628561762)/(o+1.)+(-0.162064412854)+(-0.457267302337)*0.5**o
        ref[0,1,1,0]=(-0.913361887408)/(o+1.)+(-0.356168674239)+(0.401664604431)*0.5**o
        ref[0,1,1,1]=(-0.543116639248)/(o+1.)+(-0.120902439422)+(0.381879135767)*0.5**o
        ref[0,1,2,0]=(0.993649262831)/(o+1.)+(0.384695858206)+(-0.55736653846)*0.5**o
        ref[0,1,2,1]=(-0.968077797451)/(o+1.)+(-0.187765942637)+(-0.603167099406)*0.5**o
        ref[0,2,0,0]=(0.873662019918)/(o+1.)+(-0.710635360033)+(-0.146359415223)*0.5**o
        ref[0,2,0,1]=(0.466408230693)/(o+1.)+(0.77626565285)+(0.389070815631)*0.5**o
        ref[0,2,1,0]=(-0.475010813579)/(o+1.)+(0.525435880006)+(0.960310046461)*0.5**o
        ref[0,2,1,1]=(0.697127520484)/(o+1.)+(0.357204276552)+(0.388328722275)*0.5**o
        ref[0,2,2,0]=(-0.108434738487)/(o+1.)+(0.356443300466)+(-0.97475753261)*0.5**o
        ref[0,2,2,1]=(0.952274964304)/(o+1.)+(-0.497379508976)+(-0.562006714089)*0.5**o
        ref[0,3,0,0]=(-0.0368012761671)/(o+1.)+(0.594000625518)+(0.770200680808)*0.5**o
        ref[0,3,0,1]=(0.450361861836)/(o+1.)+(0.182031052848)+(0.164263059934)*0.5**o
        ref[0,3,1,0]=(0.500714741956)/(o+1.)+(0.0479711343168)+(-0.0785996333735)*0.5**o
        ref[0,3,1,1]=(-0.949280938645)/(o+1.)+(0.452441874504)+(-0.88306753495)*0.5**o
        ref[0,3,2,0]=(0.186931712837)/(o+1.)+(0.222335175903)+(0.426318408694)*0.5**o
        ref[0,3,2,1]=(-0.288015428918)/(o+1.)+(0.80948657963)+(-0.822221815085)*0.5**o
        ref[0,4,0,0]=(0.670151184695)/(o+1.)+(-0.415883847013)+(0.376690471866)*0.5**o
        ref[0,4,0,1]=(-0.107106843992)/(o+1.)+(0.555375298435)+(-0.837698961768)*0.5**o
        ref[0,4,1,0]=(-0.299999096802)/(o+1.)+(0.668996782774)+(-0.669357604061)*0.5**o
        ref[0,4,1,1]=(0.778006183401)/(o+1.)+(-0.614473529032)+(-0.532655856076)*0.5**o
        ref[0,4,2,0]=(-0.250068389818)/(o+1.)+(0.156274314197)+(-0.0103473845504)*0.5**o
        ref[0,4,2,1]=(0.902125322216)/(o+1.)+(-0.616424567683)+(-0.560368879846)*0.5**o
        ref[1,0,0,0]=(-0.133690258295)/(o+1.)+(-0.715949360249)+(0.525524578427)*0.5**o
        ref[1,0,0,1]=(-0.788097689216)/(o+1.)+(0.0998405659256)+(-0.076709566994)*0.5**o
        ref[1,0,1,0]=(0.144280886187)/(o+1.)+(0.0775377290314)+(-0.696369546924)*0.5**o
        ref[1,0,1,1]=(0.824172196605)/(o+1.)+(0.561804029444)+(-0.0277357461773)*0.5**o
        ref[1,0,2,0]=(0.422564421802)/(o+1.)+(-0.60719307099)+(-0.417386842591)*0.5**o
        ref[1,0,2,1]=(0.778908672762)/(o+1.)+(0.0163446340765)+(-0.505974820938)*0.5**o
        ref[1,1,0,0]=(0.0574071047387)/(o+1.)+(-0.350514214742)+(0.608850746775)*0.5**o
        ref[1,1,0,1]=(-0.991660961792)/(o+1.)+(0.322084279455)+(0.380970772212)*0.5**o
        ref[1,1,1,0]=(-0.823761120268)/(o+1.)+(-0.0495419092898)+(0.930901327253)*0.5**o
        ref[1,1,1,1]=(-0.384729980569)/(o+1.)+(-0.0392583789163)+(0.649381188769)*0.5**o
        ref[1,1,2,0]=(-0.840365304069)/(o+1.)+(-0.0150512662777)+(0.517673506558)*0.5**o
        ref[1,1,2,1]=(0.555976660846)/(o+1.)+(-0.307265886898)+(0.707353625614)*0.5**o
        ref[1,2,0,0]=(-0.423056705491)/(o+1.)+(0.0559404393793)+(0.311083677036)*0.5**o
        ref[1,2,0,1]=(0.15409797664)/(o+1.)+(-0.0906637067519)+(-0.774473828018)*0.5**o
        ref[1,2,1,0]=(0.796718644088)/(o+1.)+(0.248012077098)+(0.776951250898)*0.5**o
        ref[1,2,1,1]=(-0.342830889263)/(o+1.)+(-0.282162737841)+(0.511417858487)*0.5**o
        ref[1,2,2,0]=(-0.722403016671)/(o+1.)+(-0.367059457954)+(0.230014491812)*0.5**o
        ref[1,2,2,1]=(-0.773685058881)/(o+1.)+(0.412807900451)+(-0.624365721366)*0.5**o
        ref[1,3,0,0]=(0.527481434758)/(o+1.)+(0.36602961117)+(0.031214145649)*0.5**o
        ref[1,3,0,1]=(0.0630927531585)/(o+1.)+(-0.164989701995)+(-0.325906787975)*0.5**o
        ref[1,3,1,0]=(-0.612390590593)/(o+1.)+(-0.78855996274)+(-0.63929738839)*0.5**o
        ref[1,3,1,1]=(-0.962656044265)/(o+1.)+(-0.00576535037253)+(-0.546330443673)*0.5**o
        ref[1,3,2,0]=(-0.947280922008)/(o+1.)+(-0.137449579996)+(0.747590859563)*0.5**o
        ref[1,3,2,1]=(-0.946623026202)/(o+1.)+(0.0787536550213)+(-0.24399757656)*0.5**o
        ref[1,4,0,0]=(-0.174471016943)/(o+1.)+(-0.636500556039)+(-0.635482528623)*0.5**o
        ref[1,4,0,1]=(0.806379391595)/(o+1.)+(-0.00304106577093)+(-0.476044717545)*0.5**o
        ref[1,4,1,0]=(0.729649125134)/(o+1.)+(0.0151477051616)+(-0.403091231646)*0.5**o
        ref[1,4,1,1]=(0.559976454745)/(o+1.)+(0.479295896421)+(0.75951513376)*0.5**o
        ref[1,4,2,0]=(0.272199270818)/(o+1.)+(-0.0929072274332)+(-0.843552757065)*0.5**o
        ref[1,4,2,1]=(-0.552663678377)/(o+1.)+(0.613784745778)+(-0.230300352975)*0.5**o
        ref[2,0,0,0]=(-0.174247908098)/(o+1.)+(-0.108873783282)+(-0.716067030882)*0.5**o
        ref[2,0,0,1]=(-0.300584841532)/(o+1.)+(-0.485363665493)+(-0.267230223262)*0.5**o
        ref[2,0,1,0]=(0.316261805154)/(o+1.)+(-0.523474106451)+(-0.888278086869)*0.5**o
        ref[2,0,1,1]=(-0.699622562861)/(o+1.)+(-0.151089538248)+(-0.909702635929)*0.5**o
        ref[2,0,2,0]=(0.728545639829)/(o+1.)+(0.192083148877)+(-0.0625371618174)*0.5**o
        ref[2,0,2,1]=(-0.22042267239)/(o+1.)+(-0.611974599324)+(0.677333343041)*0.5**o
        ref[2,1,0,0]=(-0.0734883811113)/(o+1.)+(0.36100831022)+(-0.890215236787)*0.5**o
        ref[2,1,0,1]=(0.121656829057)/(o+1.)+(-0.130801635799)+(-0.341933579678)*0.5**o
        ref[2,1,1,0]=(0.221353168629)/(o+1.)+(0.618635673483)+(0.540308531179)*0.5**o
        ref[2,1,1,1]=(0.0152580531273)/(o+1.)+(0.252173415542)+(0.560378877984)*0.5**o
        ref[2,1,2,0]=(0.580319777731)/(o+1.)+(0.90566612137)+(0.993226484879)*0.5**o
        ref[2,1,2,1]=(-0.710250320949)/(o+1.)+(0.742177021871)+(0.345780554724)*0.5**o
        ref[2,2,0,0]=(0.770701563507)/(o+1.)+(0.240823471881)+(0.399162680517)*0.5**o
        ref[2,2,0,1]=(0.76121761761)/(o+1.)+(0.128382656738)+(-0.460887633042)*0.5**o
        ref[2,2,1,0]=(-0.269854962057)/(o+1.)+(0.690854898294)+(-0.521701367567)*0.5**o
        ref[2,2,1,1]=(0.719518595712)/(o+1.)+(0.029596430244)+(-0.185365051187)*0.5**o
        ref[2,2,2,0]=(-0.671157761165)/(o+1.)+(-0.142688285012)+(-0.0423425346644)*0.5**o
        ref[2,2,2,1]=(-0.541156097112)/(o+1.)+(0.0765180918929)+(-0.379649661722)*0.5**o
        ref[2,3,0,0]=(-0.0604347109212)/(o+1.)+(-0.19403702073)+(-0.034905025507)*0.5**o
        ref[2,3,0,1]=(-0.793398507071)/(o+1.)+(0.255681280626)+(0.351720519037)*0.5**o
        ref[2,3,1,0]=(-0.267058413731)/(o+1.)+(0.852211426049)+(0.974567776717)*0.5**o
        ref[2,3,1,1]=(0.228396641207)/(o+1.)+(0.322922896735)+(-0.181997910305)*0.5**o
        ref[2,3,2,0]=(0.480987773937)/(o+1.)+(-0.197709028569)+(0.0950934707835)*0.5**o
        ref[2,3,2,1]=(-0.648934475027)/(o+1.)+(-0.418383297875)+(0.240224435511)*0.5**o
        ref[2,4,0,0]=(0.430020261528)/(o+1.)+(0.53421018733)+(0.436370231269)*0.5**o
        ref[2,4,0,1]=(0.906390660223)/(o+1.)+(-0.056057545856)+(0.640490734948)*0.5**o
        ref[2,4,1,0]=(-0.732099770943)/(o+1.)+(-0.0485027617732)+(0.161578976854)*0.5**o
        ref[2,4,1,1]=(-0.827495680848)/(o+1.)+(0.285044728936)+(-0.992031068384)*0.5**o
        ref[2,4,2,0]=(-0.919905471787)/(o+1.)+(0.656971386264)+(-0.541001898785)*0.5**o
        ref[2,4,2,1]=(0.161975875302)/(o+1.)+(0.272665206557)+(0.686527626533)*0.5**o
        ref[3,0,0,0]=(0.584120963259)/(o+1.)+(0.207930729804)+(0.972857504177)*0.5**o
        ref[3,0,0,1]=(0.315823141491)/(o+1.)+(-0.217561208386)+(-0.212429707823)*0.5**o
        ref[3,0,1,0]=(-0.732294996139)/(o+1.)+(-0.278653143596)+(-0.586168358175)*0.5**o
        ref[3,0,1,1]=(0.481321847225)/(o+1.)+(-0.070080648792)+(0.482676863662)*0.5**o
        ref[3,0,2,0]=(0.616679859076)/(o+1.)+(-0.418784546677)+(0.390742116352)*0.5**o
        ref[3,0,2,1]=(0.866778417448)/(o+1.)+(-0.686585656643)+(0.0629230167692)*0.5**o
        ref[3,1,0,0]=(0.531890054167)/(o+1.)+(-0.516187607633)+(-0.492439201527)*0.5**o
        ref[3,1,0,1]=(-0.512361639413)/(o+1.)+(0.499557182545)+(0.237651575694)*0.5**o
        ref[3,1,1,0]=(-0.67542965405)/(o+1.)+(0.274414249559)+(-0.678336189514)*0.5**o
        ref[3,1,1,1]=(0.371936242507)/(o+1.)+(-0.1636482466)+(-0.982634642862)*0.5**o
        ref[3,1,2,0]=(-0.799408831501)/(o+1.)+(0.458633399549)+(0.399251684639)*0.5**o
        ref[3,1,2,1]=(-0.0145253903686)/(o+1.)+(0.238472345905)+(0.757566560882)*0.5**o
        ref[3,2,0,0]=(-0.130519473112)/(o+1.)+(0.695277604837)+(0.581732248306)*0.5**o
        ref[3,2,0,1]=(0.632010536438)/(o+1.)+(0.123066205529)+(0.0316731560259)*0.5**o
        ref[3,2,1,0]=(-0.709926210356)/(o+1.)+(0.0926699972302)+(0.53924621094)*0.5**o
        ref[3,2,1,1]=(0.428094002247)/(o+1.)+(-0.841997926131)+(-0.276112488096)*0.5**o
        ref[3,2,2,0]=(0.708481148086)/(o+1.)+(-0.50498161534)+(0.495412824545)*0.5**o
        ref[3,2,2,1]=(0.782824622703)/(o+1.)+(-0.151509698687)+(-0.245246780227)*0.5**o
        ref[3,3,0,0]=(0.848460868418)/(o+1.)+(-0.104639248461)+(-0.359902945273)*0.5**o
        ref[3,3,0,1]=(0.0569151307298)/(o+1.)+(-0.0674224535469)+(0.165794065024)*0.5**o
        ref[3,3,1,0]=(-0.165683739908)/(o+1.)+(-0.183388699236)+(0.771348141212)*0.5**o
        ref[3,3,1,1]=(-0.932550058069)/(o+1.)+(-0.47828109001)+(-0.795311686688)*0.5**o
        ref[3,3,2,0]=(0.821542940923)/(o+1.)+(0.166853194487)+(-0.13512674466)*0.5**o
        ref[3,3,2,1]=(-0.46375871628)/(o+1.)+(0.132701267732)+(-0.449544418926)*0.5**o
        ref[3,4,0,0]=(-0.074530542168)/(o+1.)+(-0.0567877963797)+(-0.134845261261)*0.5**o
        ref[3,4,0,1]=(0.238671520479)/(o+1.)+(0.50689253572)+(0.731340228204)*0.5**o
        ref[3,4,1,0]=(0.981601513335)/(o+1.)+(0.400717209822)+(0.248707844726)*0.5**o
        ref[3,4,1,1]=(0.592304390107)/(o+1.)+(0.342408637328)+(0.834188819333)*0.5**o
        ref[3,4,2,0]=(-0.339991330909)/(o+1.)+(-0.0145922376156)+(0.899051000485)*0.5**o
        ref[3,4,2,1]=(-0.626610688164)/(o+1.)+(-0.417008165115)+(-0.426085250846)*0.5**o
      else:
        arg[0,0,0,0]=(0.697666818785)*x[0]+(1.00202295791)*x[1]+(0.617700157184)*x[2]
        arg[0,0,0,1]=(-0.465577819457)*x[0]+(-1.40054693355)*x[1]+(1.16912653295)*x[2]
        arg[0,0,1,0]=(-1.26696500634)*x[0]+(-0.0469756169975)*x[1]+(-0.445591738991)*x[2]
        arg[0,0,1,1]=(-1.61654321709)*x[0]+(-1.69522962467)*x[1]+(-1.06089094851)*x[2]
        arg[0,0,2,0]=(0.229468378506)*x[0]+(0.716813596991)*x[1]+(-0.237703937029)*x[2]
        arg[0,0,2,1]=(-1.12052051905)*x[0]+(0.835000004618)*x[1]+(-0.853004812404)*x[2]
        arg[0,1,0,0]=(-1.75500745705)*x[0]+(0.655055836882)*x[1]+(1.05207750735)*x[2]
        arg[0,1,0,1]=(-0.0746024824253)*x[0]+(-0.421596816958)*x[1]+(-0.375715855989)*x[2]
        arg[0,1,1,0]=(-0.963269684396)*x[0]+(0.431993248836)*x[1]+(-1.63061928649)*x[2]
        arg[0,1,1,1]=(-0.418757265018)*x[0]+(-0.665626168583)*x[1]+(-0.335734432507)*x[2]
        arg[0,1,2,0]=(0.980346404307)*x[0]+(1.45442049802)*x[1]+(-0.833420308612)*x[2]
        arg[0,1,2,1]=(-1.52536017847)*x[0]+(0.99484028681)*x[1]+(0.109754321663)*x[2]
        arg[0,2,0,0]=(1.697179262)*x[0]+(-0.928732802147)*x[1]+(-0.653379513626)*x[2]
        arg[0,2,0,1]=(0.682698902948)*x[0]+(0.829307959583)*x[1]+(-1.0300112276)*x[2]
        arg[0,2,1,0]=(0.026084323266)*x[0]+(1.28865784608)*x[1]+(0.830446580645)*x[2]
        arg[0,2,1,1]=(0.486291506032)*x[0]+(-0.519637742913)*x[1]+(0.81671330629)*x[2]
        arg[0,2,2,0]=(-0.761591624634)*x[0]+(0.370951131011)*x[1]+(-0.256748479312)*x[2]
        arg[0,2,2,1]=(-0.249974326513)*x[0]+(0.267323484635)*x[1]+(-1.09642130905)*x[2]
        arg[0,3,0,0]=(0.752057563255)*x[0]+(-1.2772262175)*x[1]+(-0.264569668983)*x[2]
        arg[0,3,0,1]=(-0.173167796447)*x[0]+(0.31873208356)*x[1]+(-0.599780701112)*x[2]
        arg[0,3,1,0]=(-0.912464014941)*x[0]+(-1.56452517166)*x[1]+(1.37651660499)*x[2]
        arg[0,3,1,1]=(0.302009667854)*x[0]+(0.524213064769)*x[1]+(0.316301522683)*x[2]
        arg[0,3,2,0]=(-1.2940154673)*x[0]+(-0.900061396992)*x[1]+(0.3860029847)*x[2]
        arg[0,3,2,1]=(-0.97354131341)*x[0]+(1.08221883964)*x[1]+(-0.739323076416)*x[2]
        arg[0,4,0,0]=(1.86535317192)*x[0]+(-0.478359589546)*x[1]+(-0.798014592302)*x[2]
        arg[0,4,0,1]=(-1.08518693851)*x[0]+(-0.186048654536)*x[1]+(1.04915573267)*x[2]
        arg[0,4,1,0]=(-1.09692811055)*x[0]+(1.01042920935)*x[1]+(-0.73978089555)*x[2]
        arg[0,4,1,1]=(0.654656964663)*x[0]+(0.279204596126)*x[1]+(0.886217781361)*x[2]
        arg[0,4,2,0]=(-0.54009886291)*x[0]+(0.378037248739)*x[1]+(1.09785568886)*x[2]
        arg[0,4,2,1]=(-1.43995958913)*x[0]+(-0.720157090994)*x[1]+(0.0676362028136)*x[2]
        arg[1,0,0,0]=(1.47735412608)*x[0]+(0.342355546228)*x[1]+(-0.0134502415508)*x[2]
        arg[1,0,0,1]=(-0.419536590369)*x[0]+(-1.61108470511)*x[1]+(1.51281979355)*x[2]
        arg[1,0,1,0]=(1.31765698245)*x[0]+(-0.7499566294)*x[1]+(0.934537503478)*x[2]
        arg[1,0,1,1]=(0.688467914883)*x[0]+(-1.32681280673)*x[1]+(-0.387405705058)*x[2]
        arg[1,0,2,0]=(1.37069587556)*x[0]+(0.540830111773)*x[1]+(0.477311068263)*x[2]
        arg[1,0,2,1]=(-0.572066213761)*x[0]+(-1.11950925494)*x[1]+(1.07939217872)*x[2]
        arg[1,1,0,0]=(0.564997105326)*x[0]+(-0.73130546389)*x[1]+(0.435910937304)*x[2]
        arg[1,1,0,1]=(0.513931739674)*x[0]+(-0.944043404826)*x[1]+(0.014114227468)*x[2]
        arg[1,1,1,0]=(-0.77850597256)*x[0]+(-0.445835136484)*x[1]+(0.157478357479)*x[2]
        arg[1,1,1,1]=(0.752431176866)*x[0]+(0.659759664199)*x[1]+(-0.859515433521)*x[2]
        arg[1,1,2,0]=(0.0693158730911)*x[0]+(-0.820997827204)*x[1]+(-0.714694436829)*x[2]
        arg[1,1,2,1]=(0.604674152847)*x[0]+(0.647261183132)*x[1]+(-1.21981352028)*x[2]
        arg[1,2,0,0]=(0.0997504695984)*x[0]+(-0.271549839083)*x[1]+(-0.573467835942)*x[2]
        arg[1,2,0,1]=(0.01861845581)*x[0]+(0.157410502406)*x[1]+(0.939257868776)*x[2]
        arg[1,2,1,0]=(1.44909894631)*x[0]+(0.116038517547)*x[1]+(-0.851940489233)*x[2]
        arg[1,2,1,1]=(1.34598841007)*x[0]+(1.79163125841)*x[1]+(0.719171044245)*x[2]
        arg[1,2,2,0]=(-0.214737791448)*x[0]+(-1.85810839095)*x[1]+(-0.240776599927)*x[2]
        arg[1,2,2,1]=(-0.574707645928)*x[0]+(-0.664173831956)*x[1]+(0.310262106696)*x[2]
        arg[1,3,0,0]=(0.296213453592)*x[0]+(-0.831112482555)*x[1]+(0.130075739128)*x[2]
        arg[1,3,0,1]=(-0.369510604457)*x[0]+(0.0373909697998)*x[1]+(1.6689129783)*x[2]
        arg[1,3,1,0]=(0.0922435881419)*x[0]+(-0.280519827004)*x[1]+(0.285933839232)*x[2]
        arg[1,3,1,1]=(-0.649253282272)*x[0]+(-0.460065044123)*x[1]+(-0.0711164948063)*x[2]
        arg[1,3,2,0]=(0.793313307581)*x[0]+(-0.0587956351149)*x[1]+(0.810487259739)*x[2]
        arg[1,3,2,1]=(-1.38668597055)*x[0]+(1.51396934198)*x[1]+(-1.13918550255)*x[2]
        arg[1,4,0,0]=(1.40078167737)*x[0]+(-1.61915436345)*x[1]+(0.421710091214)*x[2]
        arg[1,4,0,1]=(-1.37982015877)*x[0]+(1.30607371738)*x[1]+(-0.608888848419)*x[2]
        arg[1,4,1,0]=(-0.558087802805)*x[0]+(-0.214169499512)*x[1]+(-0.766242546503)*x[2]
        arg[1,4,1,1]=(-1.75503436584)*x[0]+(0.183604705451)*x[1]+(-0.148071484576)*x[2]
        arg[1,4,2,0]=(0.537942069142)*x[0]+(-0.00640749928252)*x[1]+(-0.277544629657)*x[2]
        arg[1,4,2,1]=(-0.652645833371)*x[0]+(-1.33439077026)*x[1]+(-0.226308197201)*x[2]
        arg[2,0,0,0]=(1.45646289279)*x[0]+(1.40842127675)*x[1]+(1.75078281495)*x[2]
        arg[2,0,0,1]=(0.867988651995)*x[0]+(1.35061812241)*x[1]+(-1.56520776323)*x[2]
        arg[2,0,1,0]=(0.549686523522)*x[0]+(-0.528085465813)*x[1]+(-0.994978820267)*x[2]
        arg[2,0,1,1]=(0.383595996552)*x[0]+(0.612989859087)*x[1]+(-0.560401247231)*x[2]
        arg[2,0,2,0]=(-0.341259919499)*x[0]+(-0.125491675042)*x[1]+(0.545572340757)*x[2]
        arg[2,0,2,1]=(-0.414269395434)*x[0]+(-0.433969881898)*x[1]+(-1.01820199328)*x[2]
        arg[2,1,0,0]=(-0.113575280389)*x[0]+(0.326916039942)*x[1]+(-0.576072890249)*x[2]
        arg[2,1,0,1]=(-0.296473969661)*x[0]+(0.195675099431)*x[1]+(-1.72378150831)*x[2]
        arg[2,1,1,0]=(-1.31575932758)*x[0]+(-0.122557423522)*x[1]+(0.53435027695)*x[2]
        arg[2,1,1,1]=(0.299435754608)*x[0]+(0.727140413706)*x[1]+(0.848931358475)*x[2]
        arg[2,1,2,0]=(0.2618965198)*x[0]+(-0.461672573339)*x[1]+(0.923592803172)*x[2]
        arg[2,1,2,1]=(-0.326817258523)*x[0]+(-0.0363300805442)*x[1]+(1.82049875918)*x[2]
        arg[2,2,0,0]=(-0.435113320888)*x[0]+(0.0257984353786)*x[1]+(0.287846577657)*x[2]
        arg[2,2,0,1]=(1.39178068279)*x[0]+(0.136781967539)*x[1]+(-1.38161590351)*x[2]
        arg[2,2,1,0]=(-0.887874199356)*x[0]+(-0.476769153654)*x[1]+(-0.618760485829)*x[2]
        arg[2,2,1,1]=(1.58883355098)*x[0]+(-0.889469047555)*x[1]+(-0.678486755843)*x[2]
        arg[2,2,2,0]=(0.237147029365)*x[0]+(-0.509883462401)*x[1]+(0.533163151551)*x[2]
        arg[2,2,2,1]=(0.66513913749)*x[0]+(1.30317030657)*x[1]+(0.804083365617)*x[2]
        arg[2,3,0,0]=(1.01531518403)*x[0]+(1.3753053143)*x[1]+(0.000877284235964)*x[2]
        arg[2,3,0,1]=(-0.40853642543)*x[0]+(0.154794268783)*x[1]+(-1.34306671117)*x[2]
        arg[2,3,1,0]=(0.43065284359)*x[0]+(-0.28965146009)*x[1]+(0.660248828789)*x[2]
        arg[2,3,1,1]=(-1.4334922681)*x[0]+(-0.0348802858609)*x[1]+(0.502051304974)*x[2]
        arg[2,3,2,0]=(-1.17399537487)*x[0]+(0.865787043946)*x[1]+(-0.0105949677478)*x[2]
        arg[2,3,2,1]=(0.63104158794)*x[0]+(0.436196079951)*x[1]+(0.498606623061)*x[2]
        arg[2,4,0,0]=(-1.44410177354)*x[0]+(0.513406326011)*x[1]+(0.699127199447)*x[2]
        arg[2,4,0,1]=(-0.981727023077)*x[0]+(0.484182120695)*x[1]+(0.667175832045)*x[2]
        arg[2,4,1,0]=(-1.3352602588)*x[0]+(0.626269023364)*x[1]+(0.813537646695)*x[2]
        arg[2,4,1,1]=(-0.932486930114)*x[0]+(0.299448807849)*x[1]+(0.195862014797)*x[2]
        arg[2,4,2,0]=(-0.0621489644806)*x[0]+(0.131979664158)*x[1]+(0.442029269778)*x[2]
        arg[2,4,2,1]=(-0.507508608633)*x[0]+(-1.28613226533)*x[1]+(-0.697920444023)*x[2]
        arg[3,0,0,0]=(-0.984942677585)*x[0]+(-1.58913299616)*x[1]+(-0.373026784822)*x[2]
        arg[3,0,0,1]=(0.616666427113)*x[0]+(-0.625162800471)*x[1]+(-0.796307290839)*x[2]
        arg[3,0,1,0]=(-0.48469203683)*x[0]+(-0.0572105166553)*x[1]+(0.809572383581)*x[2]
        arg[3,0,1,1]=(-0.487160309892)*x[0]+(0.77646454341)*x[1]+(-0.41947544573)*x[2]
        arg[3,0,2,0]=(-1.17902121554)*x[0]+(-0.244939532033)*x[1]+(-0.282994359476)*x[2]
        arg[3,0,2,1]=(0.33515909881)*x[0]+(0.146866561705)*x[1]+(1.81486098364)*x[2]
        arg[3,1,0,0]=(0.379977901341)*x[0]+(0.834531701517)*x[1]+(-0.0242027062166)*x[2]
        arg[3,1,0,1]=(0.184318292766)*x[0]+(1.08367911805)*x[1]+(-0.164531741594)*x[2]
        arg[3,1,1,0]=(-1.00967115996)*x[0]+(0.542699640149)*x[1]+(0.707616701259)*x[2]
        arg[3,1,1,1]=(0.856948233811)*x[0]+(-1.2186550297)*x[1]+(-0.034268798159)*x[2]
        arg[3,1,2,0]=(0.259470684477)*x[0]+(-1.03586983732)*x[1]+(-0.157275979341)*x[2]
        arg[3,1,2,1]=(-0.161954890658)*x[0]+(1.65853209885)*x[1]+(0.381113456719)*x[2]
        arg[3,2,0,0]=(-0.437413004681)*x[0]+(-0.730340273929)*x[1]+(-0.687689272494)*x[2]
        arg[3,2,0,1]=(-0.336618164118)*x[0]+(-1.24388666895)*x[1]+(-0.2198689254)*x[2]
        arg[3,2,1,0]=(0.0872793087887)*x[0]+(-0.336554573215)*x[1]+(-0.0597803523262)*x[2]
        arg[3,2,1,1]=(0.192805432018)*x[0]+(0.0540495582064)*x[1]+(0.846658042071)*x[2]
        arg[3,2,2,0]=(0.00794192604414)*x[0]+(-0.33504559053)*x[1]+(0.615701149787)*x[2]
        arg[3,2,2,1]=(0.210870961503)*x[0]+(1.14164095846)*x[1]+(-0.192785742791)*x[2]
        arg[3,3,0,0]=(-0.25895162041)*x[0]+(-0.225800436488)*x[1]+(-0.198007079838)*x[2]
        arg[3,3,0,1]=(-0.150363703231)*x[0]+(1.23870429535)*x[1]+(0.28635881247)*x[2]
        arg[3,3,1,0]=(-0.865980718216)*x[0]+(-0.0849853244335)*x[1]+(-0.292164217189)*x[2]
        arg[3,3,1,1]=(-0.350465968722)*x[0]+(0.09853736936)*x[1]+(-0.476711524665)*x[2]
        arg[3,3,2,0]=(0.311396300263)*x[0]+(-0.92437612034)*x[1]+(-0.757139962928)*x[2]
        arg[3,3,2,1]=(-0.912311919837)*x[0]+(0.267793631099)*x[1]+(0.4917345401)*x[2]
        arg[3,4,0,0]=(0.613662541753)*x[0]+(0.782757341361)*x[1]+(1.04209808158)*x[2]
        arg[3,4,0,1]=(0.492828613997)*x[0]+(1.11979095664)*x[1]+(-0.843545072354)*x[2]
        arg[3,4,1,0]=(-0.51939120556)*x[0]+(0.476004484286)*x[1]+(-0.516311785432)*x[2]
        arg[3,4,1,1]=(1.00423313567)*x[0]+(0.90776890388)*x[1]+(1.34423309497)*x[2]
        arg[3,4,2,0]=(-1.44069600826)*x[0]+(-1.39515290909)*x[1]+(1.19851590247)*x[2]
        arg[3,4,2,1]=(0.391851619877)*x[0]+(0.265529120553)*x[1]+(0.996610410913)*x[2]
        ref[0,0,0,0]=(1.19330263935)/(o+1.)+(0.55394275251)+(0.0162017895041)*0.5**o
        ref[0,0,0,1]=(-0.672351783684)/(o+1.)+(-0.187889622039)+(0.351132807711)*0.5**o
        ref[0,0,1,0]=(-1.27021996264)/(o+1.)+(0.236286530101)+(-0.961885459894)*0.5**o
        ref[0,0,1,1]=(-1.43442517107)/(o+1.)+(-1.00113374859)+(-0.935971122022)*0.5**o
        ref[0,0,2,0]=(-0.434813695127)/(o+1.)+(0.606673209583)+(-0.0699546855712)*0.5**o
        ref[0,0,2,1]=(-0.0434901247948)/(o+1.)+(-0.180911877983)+(-0.733211446073)*0.5**o
        ref[0,1,0,0]=(1.41514302182)/(o+1.)+(-0.247977413325)+(-0.967062307989)*0.5**o
        ref[0,1,0,1]=(0.69128100142)/(o+1.)+(-0.621609882923)+(-0.319976390946)*0.5**o
        ref[0,1,1,0]=(-0.881870429191)/(o+1.)+(-0.654075774082)+(0.0281262553057)*0.5**o
        ref[0,1,1,1]=(-1.27239116086)/(o+1.)+(-0.0854066429492)+(0.0230865806541)*0.5**o
        ref[0,1,2,0]=(-0.222663945351)/(o+1.)+(0.685312598747)+(0.453385341573)*0.5**o
        ref[0,1,2,1]=(0.101409038706)/(o+1.)+(0.151367946744)+(-0.824910502191)*0.5**o
        ref[0,2,0,0]=(-1.19509443892)/(o+1.)+(0.225437856866)+(0.85928567141)*0.5**o
        ref[0,2,0,1]=(0.285764347754)/(o+1.)+(-0.383432127133)+(0.963095541446)*0.5**o
        ref[0,2,1,0]=(0.728010227911)/(o+1.)+(0.983790919896)+(-0.550403317717)*0.5**o
        ref[0,2,1,1]=(0.563748259036)/(o+1.)+(0.165164845927)+(-0.110710881481)*0.5**o
        ref[0,2,2,0]=(-0.361502606429)/(o+1.)+(0.0276145482611)+(-0.341115463028)*0.5**o
        ref[0,2,2,1]=(-0.293320207933)/(o+1.)+(0.0336757304385)+(-0.853103403872)*0.5**o
        ref[0,3,0,0]=(-0.410233956527)/(o+1.)+(-0.474281126245)+(0.569057885785)*0.5**o
        ref[0,3,0,1]=(-0.835861970115)/(o+1.)+(0.0173715483175)+(0.346902459481)*0.5**o
        ref[0,3,1,0]=(0.0830466456187)/(o+1.)+(-0.288204681495)+(-0.607109864241)*0.5**o
        ref[0,3,1,1]=(1.07074130823)/(o+1.)+(-0.0437983012867)+(0.159379549652)*0.5**o
        ref[0,3,2,0]=(0.172301049609)/(o+1.)+(-0.491724874152)+(-0.996925180895)*0.5**o
        ref[0,3,2,1]=(-0.24891941221)/(o+1.)+(-0.03935856269)+(-0.3030090126)*0.5**o
        ref[0,4,0,0]=(-0.778575122429)/(o+1.)+(0.213085222078)+(0.941383668349)*0.5**o
        ref[0,4,0,1]=(0.880801627174)/(o+1.)+(-0.454451723401)+(-0.193978040747)*0.5**o
        ref[0,4,1,0]=(-0.835604724882)/(o+1.)+(0.127240834083)+(-0.245156740036)*0.5**o
        ref[0,4,1,1]=(1.62336716324)/(o+1.)+(-0.101085918464)+(0.398884015839)*0.5**o
        ref[0,4,2,0]=(1.16805827702)/(o+1.)+(0.211232591688)+(-0.654729385702)*0.5**o
        ref[0,4,2,1]=(-0.305424138957)/(o+1.)+(-0.431076572693)+(-0.924903192969)*0.5**o
        ref[1,0,0,0]=(-0.481596247529)/(o+1.)+(0.769902632559)+(0.748050413171)*0.5**o
        ref[1,0,0,1]=(0.177388661252)/(o+1.)+(-0.118766442613)+(-0.457657277952)*0.5**o
        ref[1,0,1,0]=(0.709890860372)/(o+1.)+(0.0668010230727)+(0.658744950012)*0.5**o
        ref[1,0,1,1]=(-1.45685781381)/(o+1.)+(-0.178674426089)+(0.788456069079)*0.5**o
        ref[1,0,2,0]=(0.194388185612)/(o+1.)+(0.649177543063)+(0.896093783855)*0.5**o
        ref[1,0,2,1]=(0.181561279507)/(o+1.)+(-0.200618291902)+(-0.392507985678)*0.5**o
        ref[1,1,0,0]=(-0.884376814027)/(o+1.)+(0.280382736159)+(0.593213920451)*0.5**o
        ref[1,1,0,1]=(-0.250709602218)/(o+1.)+(-0.492104211211)+(0.818920586956)*0.5**o
        ref[1,1,1,0]=(0.0529723694625)/(o+1.)+(-0.502047650638)+(-0.115739819752)*0.5**o
        ref[1,1,1,1]=(0.215812202957)/(o+1.)+(-0.308796450292)+(0.954456105171)*0.5**o
        ref[1,1,2,0]=(-0.991481050902)/(o+1.)+(0.162626252838)+(-0.800147845717)*0.5**o
        ref[1,1,2,1]=(-0.433989756951)/(o+1.)+(0.362776336712)+(-0.259441100777)*0.5**o
        ref[1,2,0,0]=(-0.930287052624)/(o+1.)+(-0.347752763108)+(0.880525373413)*0.5**o
        ref[1,2,0,1]=(0.698674852861)/(o+1.)+(-0.15119613196)+(0.719004238051)*0.5**o
        ref[1,2,1,0]=(-0.777216821365)/(o+1.)+(0.436336076297)+(0.617741643399)*0.5**o
        ref[1,2,1,1]=(1.09595223135)/(o+1.)+(0.978251130983)+(0.804336219412)*0.5**o
        ref[1,2,2,0]=(-0.651012630849)/(o+1.)+(-1.04894606664)+(0.43528198181)*0.5**o
        ref[1,2,2,1]=(-1.5227353883)/(o+1.)+(0.670019307557)+(-0.745922598001)*0.5**o
        ref[1,3,0,0]=(-0.750195637997)/(o+1.)+(0.231864245348)+(-0.118356142533)*0.5**o
        ref[1,3,0,1]=(1.17136226596)/(o+1.)+(-0.114976421389)+(0.395383920468)*0.5**o
        ref[1,3,1,0]=(-0.108070636769)/(o+1.)+(0.279145429234)+(-0.352562621327)*0.5**o
        ref[1,3,1,1]=(-0.144223481714)/(o+1.)+(-0.472954352493)+(-0.0903026345013)*0.5**o
        ref[1,3,2,0]=(-0.487564307786)/(o+1.)+(0.636973351449)+(0.758622537094)*0.5**o
        ref[1,3,2,1]=(0.390860965954)/(o+1.)+(-0.36647678165)+(-0.66980953377)*0.5**o
        ref[1,4,0,0]=(-1.04824984837)/(o+1.)+(0.213932231728)+(0.823722790054)*0.5**o
        ref[1,4,0,1]=(-0.0984653601336)/(o+1.)+(0.07867877705)+(-0.741527483773)*0.5**o
        ref[1,4,1,0]=(-0.146287867383)/(o+1.)+(-0.908951868158)+(0.425691754878)*0.5**o
        ref[1,4,1,1]=(-0.94160416554)/(o+1.)+(0.0459877010016)+(-0.869872381431)*0.5**o
        ref[1,4,2,0]=(0.0115742858952)/(o+1.)+(-0.222488399041)+(0.687392452389)*0.5**o
        ref[1,4,2,1]=(-0.956561936575)/(o+1.)+(-0.59770879042)+(-0.0613652834159)*0.5**o
        ref[2,0,0,0]=(1.83987381729)/(o+1.)+(0.956579317963)+(0.862634531274)*0.5**o
        ref[2,0,0,1]=(0.105446127421)/(o+1.)+(0.180661881191)+(0.186629121375)*0.5**o
        ref[2,0,1,0]=(0.0212167440766)/(o+1.)+(-0.337825588888)+(-0.318943328859)*0.5**o
        ref[2,0,1,1]=(0.744834715631)/(o+1.)+(-0.350058416663)+(0.391466726104)*0.5**o
        ref[2,0,2,0]=(-0.788776613856)/(o+1.)+(0.233776806014)+(0.400043748043)*0.5**o
        ref[2,0,2,1]=(-0.95346976307)/(o+1.)+(0.0274998652324)+(-0.967971238003)*0.5**o
        ref[2,1,0,0]=(-0.536051408942)/(o+1.)+(-0.208498016994)+(0.590315312233)*0.5**o
        ref[2,1,0,1]=(-1.04158508788)/(o+1.)+(-0.397885793101)+(0.0127762955396)*0.5**o
        ref[2,1,1,0]=(0.529914207253)/(o+1.)+(-0.547503443987)+(-0.338873793431)*0.5**o
        ref[2,1,1,1]=(0.62298670274)/(o+1.)+(0.449860048786)+(0.352800726478)*0.5**o
        ref[2,1,2,0]=(0.965292650595)/(o+1.)+(0.00980498639)+(-0.261085873742)*0.5**o
        ref[2,1,2,1]=(0.403285360803)/(o+1.)+(0.369160592269)+(0.315744874774)*0.5**o
        ref[2,2,0,0]=(-0.457764884724)/(o+1.)+(0.563657776706)+(-0.791018976541)*0.5**o
        ref[2,2,0,1]=(0.188151274044)/(o+1.)+(-0.476470545911)+(0.911736564588)*0.5**o
        ref[2,2,1,0]=(-0.558515085739)/(o+1.)+(-0.445583796553)+(-0.533721159994)*0.5**o
        ref[2,2,1,1]=(-0.431992337265)/(o+1.)+(-0.0957726221961)+(0.64441532924)*0.5**o
        ref[2,2,2,0]=(-0.159056461947)/(o+1.)+(-0.0326864536156)+(0.484856087693)*0.5**o
        ref[2,2,2,1]=(0.533116456037)/(o+1.)+(1.10197257725)+(0.0353311991352)*0.5**o
        ref[2,3,0,0]=(0.0570625147023)/(o+1.)+(0.74170779854)+(0.851019670777)*0.5**o
        ref[2,3,0,1]=(-0.495059904832)/(o+1.)+(-0.169954912181)+(-0.761839138623)*0.5**o
        ref[2,3,1,0]=(0.482375173314)/(o+1.)+(-0.19374838651)+(0.706371811996)*0.5**o
        ref[2,3,1,1]=(0.12928100721)/(o+1.)+(-0.317327862062)+(-0.460946532077)*0.5**o
        ref[2,3,2,0]=(-0.234362084763)/(o+1.)+(0.423486279181)+(-0.93141377227)*0.5**o
        ref[2,3,2,1]=(1.25586693002)/(o+1.)+(-0.127551329993)+(0.56508002092)*0.5**o
        ref[2,4,0,0]=(0.45235297749)/(o+1.)+(-0.0217537364788)+(-0.640413752615)*0.5**o
        ref[2,4,0,1]=(0.936809207384)/(o+1.)+(-0.151519610695)+(-0.464139056331)*0.5**o
        ref[2,4,1,0]=(0.867693208863)/(o+1.)+(0.105683574314)+(-0.974513946233)*0.5**o
        ref[2,4,1,1]=(-1.27821575101)/(o+1.)+(0.463163847311)+(-0.0852880510814)*0.5**o
        ref[2,4,2,0]=(-0.846224501366)/(o+1.)+(0.432619461409)+(0.492845548002)*0.5**o
        ref[2,4,2,1]=(-1.48923229995)/(o+1.)+(-0.505747396375)+(0.00916577472118)*0.5**o
        ref[3,0,0,0]=(-0.959215731121)/(o+1.)+(-0.904609108587)+(-0.178668510268)*0.5**o
        ref[3,0,0,1]=(-1.53181231865)/(o+1.)+(-0.0758543610408)+(0.878717376536)*0.5**o
        ref[3,0,1,0]=(1.18643238831)/(o+1.)+(-0.288825159097)+(-0.341112240015)*0.5**o
        ref[3,0,1,1]=(-0.0675694917031)/(o+1.)+(-0.168955703616)+(0.275309686724)*0.5**o
        ref[3,0,2,0]=(-0.663269860133)/(o+1.)+(-0.235102421885)+(-0.573480403148)*0.5**o
        ref[3,0,2,1]=(1.0969289778)/(o+1.)+(0.46603172821)+(0.267894209932)*0.5**o
        ref[3,1,0,0]=(-0.716870513559)/(o+1.)+(0.712039409283)+(0.483098591635)*0.5**o
        ref[3,1,0,1]=(0.0328531999411)/(o+1.)+(0.379319450988)+(0.311973567308)*0.5**o
        ref[3,1,1,0]=(1.08648208064)/(o+1.)+(-0.196831959338)+(-0.452172980519)*0.5**o
        ref[3,1,1,1]=(0.0813181502612)/(o+1.)+(-0.227364064527)+(-0.0225656152548)*0.5**o
        ref[3,1,2,0]=(-0.271000454408)/(o+1.)+(-0.765711687155)+(0.868748696531)*0.5**o
        ref[3,1,2,1]=(1.60569646253)/(o+1.)+(0.137437739736)+(-0.00288127709197)*0.5**o
        ref[3,2,0,0]=(-1.25887089059)/(o+1.)+(-0.525842290945)+(0.455112921376)*0.5**o
        ref[3,2,0,1]=(-1.63787880725)/(o+1.)+(0.00214706725405)+(-0.166789085722)*0.5**o
        ref[3,2,1,0]=(-1.82111307894)/(o+1.)+(0.561806270366)+(0.388444921454)*0.5**o
        ref[3,2,1,1]=(-0.489259906414)/(o+1.)+(0.582548398381)+(0.417676141947)*0.5**o
        ref[3,2,2,0]=(-0.0674375340391)/(o+1.)+(-0.00345549227349)+(0.362946003887)*0.5**o
        ref[3,2,2,1]=(0.662780682001)/(o+1.)+(-0.15453848218)+(0.806022459534)*0.5**o
        ref[3,3,0,0]=(-1.15795406515)/(o+1.)+(0.0926725746467)+(0.289849779118)*0.5**o
        ref[3,3,0,1]=(1.27520269547)/(o+1.)+(-0.286529818361)+(0.672556345847)*0.5**o
        ref[3,3,1,0]=(-0.343290651469)/(o+1.)+(-0.0269962100904)+(-0.845847188189)*0.5**o
        ref[3,3,1,1]=(0.497715053371)/(o+1.)+(-0.158577177766)+(-0.909200821866)*0.5**o
        ref[3,3,2,0]=(-0.952431538662)/(o+1.)+(0.0816482063045)+(-0.580984656952)*0.5**o
        ref[3,3,2,1]=(-0.176227469799)/(o+1.)+(0.0456157853688)+(-0.0677878495767)*0.5**o
        ref[3,4,0,0]=(0.976460483802)/(o+1.)+(0.347595506011)+(0.766866468871)*0.5**o
        ref[3,4,0,1]=(0.0358170327662)/(o+1.)+(-0.0991456259778)+(0.931548717473)*0.5**o
        ref[3,4,1,0]=(0.114544318514)/(o+1.)+(-0.201014865792)+(-0.272213093635)*0.5**o
        ref[3,4,1,1]=(0.554509925385)/(o+1.)+(1.23219332557)+(0.237338557994)*0.5**o
        ref[3,4,2,0]=(0.0803465444931)/(o+1.)+(-0.518257569997)+(-0.681164419371)*0.5**o
        ref[3,4,2,1]=(-0.0443609570522)/(o+1.)+(0.572813368096)+(0.552725372202)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactZero_fromData_ReducedContinuousFunction_rank0(self):
      """
      tests integral of rank 0 Data on the ReducedFunctionOnContactZero

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(),w)
      if dim==2:
        arg=(-0.570707549273)*x[0]+(-0.726264032186)*x[1]
        ref=(-0.0423765208873)/(o+1.)+(-0.791542656673)+(0.328490252775)*0.5**o
      else:
        arg=(1.28222892931)*x[0]+(-0.798170675576)*x[1]+(1.5701208405)*x[2]
        ref=(0.721975372868)/(o+1.)+(0.344720905832)+(0.642761909705)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactZero_fromData_ReducedContinuousFunction_rank1(self):
      """
      tests integral of rank 1 Data on the ReducedFunctionOnContactZero

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(2,),w)
      ref=numpy.zeros((2,),numpy.float_)
      if dim==2:
        arg[0]=(0.575166906798)*x[0]+(-1.63698773418)*x[1]
        arg[1]=(-0.0535946584636)*x[0]+(0.105969101775)*x[1]
        ref[0]=(-0.751926426344)/(o+1.)+(-0.395800138888)+(0.481705876734)*0.5**o
        ref[1]=(0.698496252242)/(o+1.)+(-0.265927161866)+(-0.114267485199)*0.5**o
      else:
        arg[0]=(-0.17879254138)*x[0]+(-1.12752123792)*x[1]+(0.921297311605)*x[2]
        arg[1]=(1.33131824618)*x[0]+(-0.392175059176)*x[1]+(0.545754281465)*x[2]
        ref[0]=(-0.494870303061)/(o+1.)+(0.0697636237654)+(-0.0296734121631)*0.5**o
        ref[1]=(-0.394300983704)/(o+1.)+(0.722261312754)+(0.434675826665)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactZero_fromData_ReducedContinuousFunction_rank2(self):
      """
      tests integral of rank 2 Data on the ReducedFunctionOnContactZero

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5),w)
      ref=numpy.zeros((4, 5),numpy.float_)
      if dim==2:
        arg[0,0]=(-0.338658191734)*x[0]+(1.46127609683)*x[1]
        arg[0,1]=(-0.629116867491)*x[0]+(1.40141569066)*x[1]
        arg[0,2]=(0.105569779347)*x[0]+(0.239936324967)*x[1]
        arg[0,3]=(-0.281718331718)*x[0]+(0.280323404866)*x[1]
        arg[0,4]=(0.467341113661)*x[0]+(-0.50452062075)*x[1]
        arg[1,0]=(-0.841295219186)*x[0]+(0.524038163723)*x[1]
        arg[1,1]=(1.40143704236)*x[0]+(-1.50467961637)*x[1]
        arg[1,2]=(1.21078790505)*x[0]+(-0.731453470292)*x[1]
        arg[1,3]=(0.469556922076)*x[0]+(0.0686293204391)*x[1]
        arg[1,4]=(-1.26394637585)*x[0]+(-0.0361395276585)*x[1]
        arg[2,0]=(-1.12850883148)*x[0]+(0.284296673326)*x[1]
        arg[2,1]=(-0.626362828813)*x[0]+(1.19459827573)*x[1]
        arg[2,2]=(-0.545457293083)*x[0]+(1.547337019)*x[1]
        arg[2,3]=(0.664461986629)*x[0]+(-1.69082198266)*x[1]
        arg[2,4]=(-0.100004763305)*x[0]+(0.152159044423)*x[1]
        arg[3,0]=(0.307574843061)*x[0]+(1.20209181301)*x[1]
        arg[3,1]=(-0.0663203356627)*x[0]+(-1.26669863527)*x[1]
        arg[3,2]=(-1.20436798894)*x[0]+(-0.0375693379661)*x[1]
        arg[3,3]=(0.753649087747)*x[0]+(0.69376223486)*x[1]
        arg[3,4]=(-0.0512732264834)*x[0]+(-0.658097606436)*x[1]
        ref[0,0]=(0.996821439952)/(o+1.)+(0.015088160779)+(0.0956201435879)*0.5**o
        ref[0,1]=(0.643092963726)/(o+1.)+(0.239390724844)+(-0.349575590244)*0.5**o
        ref[0,2]=(-0.198468008977)/(o+1.)+(0.00411201743342)+(0.535750078425)*0.5**o
        ref[0,3]=(0.642491201137)/(o+1.)+(-0.623413199652)+(0.602940271315)*0.5**o
        ref[0,4]=(-0.105706120549)/(o+1.)+(-0.353560054939)+(0.775646723338)*0.5**o
        ref[1,0]=(0.144540669221)/(o+1.)+(0.0228268215803)+(-0.507451367845)*0.5**o
        ref[1,1]=(-0.820379339897)/(o+1.)+(0.0951825678509)+(0.526771630185)*0.5**o
        ref[1,2]=(-0.21214585824)/(o+1.)+(-0.0889183062985)+(0.869316905591)*0.5**o
        ref[1,3]=(0.291228940674)/(o+1.)+(0.192623928703)+(-0.138290555565)*0.5**o
        ref[1,4]=(-0.0663718956434)/(o+1.)+(-0.309484335614)+(-0.614745336642)*0.5**o
        ref[2,0]=(0.484892962201)/(o+1.)+(-0.367129859793)+(-0.594845400763)*0.5**o
        ref[2,1]=(0.768457634672)/(o+1.)+(0.154447657562)+(-0.509117502876)*0.5**o
        ref[2,2]=(0.707320338363)/(o+1.)+(-0.0592149232183)+(0.412989233992)*0.5**o
        ref[2,3]=(-0.983684339171)/(o+1.)+(-0.495377645598)+(0.948079634337)*0.5**o
        ref[2,4]=(-0.0315947220465)/(o+1.)+(0.506833986759)+(-0.929918970353)*0.5**o
        ref[3,0]=(0.54141067098)/(o+1.)+(0.489049131493)+(-0.00984227789859)*0.5**o
        ref[3,1]=(-0.552335053051)/(o+1.)+(-0.383325638803)+(-0.0140326402734)*0.5**o
        ref[3,2]=(0.0821472412575)/(o+1.)+(-0.335827719718)+(-0.652429128733)*0.5**o
        ref[3,3]=(0.112917208924)/(o+1.)+(0.667772807338)+(-0.00105150099312)*0.5**o
        ref[3,4]=(-0.19331508157)/(o+1.)+(-0.102296385708)+(-0.311462979932)*0.5**o
      else:
        arg[0,0]=(0.069245963142)*x[0]+(-0.185727379494)*x[1]+(-0.887410889699)*x[2]
        arg[0,1]=(-0.816745668949)*x[0]+(-0.250435680662)*x[1]+(0.701607830582)*x[2]
        arg[0,2]=(-0.862433954308)*x[0]+(0.731874372527)*x[1]+(-0.115384982779)*x[2]
        arg[0,3]=(-0.265247631604)*x[0]+(-0.157059741958)*x[1]+(-0.368419335078)*x[2]
        arg[0,4]=(1.66720203567)*x[0]+(-0.759393708719)*x[1]+(1.36360850704)*x[2]
        arg[1,0]=(-0.192822813795)*x[0]+(0.0358776152527)*x[1]+(-0.722864520788)*x[2]
        arg[1,1]=(-0.480606215471)*x[0]+(1.13682774573)*x[1]+(0.17900467623)*x[2]
        arg[1,2]=(0.346760852584)*x[0]+(0.994833030281)*x[1]+(0.507055933913)*x[2]
        arg[1,3]=(-0.509961853478)*x[0]+(-0.852997575505)*x[1]+(0.586527030253)*x[2]
        arg[1,4]=(0.93957975846)*x[0]+(0.28878335152)*x[1]+(0.128070931738)*x[2]
        arg[2,0]=(0.338283306313)*x[0]+(0.0244500761608)*x[1]+(-0.606875554888)*x[2]
        arg[2,1]=(0.850782624552)*x[0]+(-0.836382038189)*x[1]+(-1.41808459947)*x[2]
        arg[2,2]=(-1.06176229613)*x[0]+(-0.259247046061)*x[1]+(-0.583455484089)*x[2]
        arg[2,3]=(1.17437923282)*x[0]+(1.31485601495)*x[1]+(1.3139743231)*x[2]
        arg[2,4]=(0.362851636275)*x[0]+(0.804049783156)*x[1]+(1.09205000214)*x[2]
        arg[3,0]=(0.343603837195)*x[0]+(-1.49082257981)*x[1]+(-0.628668674545)*x[2]
        arg[3,1]=(-0.676847711765)*x[0]+(1.32350560453)*x[1]+(1.07275185686)*x[2]
        arg[3,2]=(0.570784985469)*x[0]+(-1.01212360602)*x[1]+(-0.0808776667606)*x[2]
        arg[3,3]=(0.506392444236)*x[0]+(-1.00848318041)*x[1]+(0.558777901541)*x[2]
        arg[3,4]=(-0.60925738291)*x[0]+(-0.386755957809)*x[1]+(0.60930115434)*x[2]
        ref[0,0]=(-0.695230136419)/(o+1.)+(-0.388400729083)+(0.468139288533)*0.5**o
        ref[0,1]=(0.462098132187)/(o+1.)+(-0.215041893287)+(-0.397587864642)*0.5**o
        ref[0,2]=(-0.262411385363)/(o+1.)+(0.275729226874)+(-0.534991632946)*0.5**o
        ref[0,3]=(0.117822633378)/(o+1.)+(0.0286827897592)+(-0.965914921536)*0.5**o
        ref[0,4]=(0.293575681039)/(o+1.)+(0.584039692827)+(0.809761767299)*0.5**o
        ref[1,0]=(0.627951922463)/(o+1.)+(-0.994889162557)+(0.482016683321)*0.5**o
        ref[1,1]=(1.83482238662)/(o+1.)+(-0.205719558739)+(-0.588157062647)*0.5**o
        ref[1,2]=(1.13612020398)/(o+1.)+(0.0262824938134)+(0.659964625174)*0.5**o
        ref[1,3]=(0.325264487741)/(o+1.)+(-0.0752154813413)+(-0.951265923787)*0.5**o
        ref[1,4]=(-0.149919283302)/(o+1.)+(0.77701502585)+(-0.0476767266789)*0.5**o
        ref[2,0]=(0.491504686171)/(o+1.)+(-0.562534706926)+(0.389422555265)*0.5**o
        ref[2,1]=(-1.52704217417)/(o+1.)+(0.120525042098)+(-0.117691923134)*0.5**o
        ref[2,2]=(-0.779796772543)/(o+1.)+(-0.318864997267)+(-0.486938059201)*0.5**o
        ref[2,3]=(1.53343318682)/(o+1.)+(0.731149467308)+(0.807477449443)*0.5**o
        ref[2,4]=(1.15919207694)/(o+1.)+(0.113921448815)+(0.871916446996)*0.5**o
        ref[3,0]=(-1.75312920944)/(o+1.)+(-0.153798982308)+(0.284839756895)*0.5**o
        ref[3,1]=(1.30985814569)/(o+1.)+(0.438430874237)+(-0.467310144534)*0.5**o
        ref[3,2]=(-0.737480320972)/(o+1.)+(0.0604403094397)+(0.094383414777)*0.5**o
        ref[3,3]=(-0.688723266524)/(o+1.)+(0.43486027018)+(-0.124310108473)*0.5**o
        ref[3,4]=(0.128399262674)/(o+1.)+(-0.372445844122)+(0.229780239192)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactZero_fromData_ReducedContinuousFunction_rank3(self):
      """
      tests integral of rank 3 Data on the ReducedFunctionOnContactZero

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(6, 2, 2),w)
      ref=numpy.zeros((6, 2, 2),numpy.float_)
      if dim==2:
        arg[0,0,0]=(-0.993339371954)*x[0]+(-1.35308128985)*x[1]
        arg[0,0,1]=(-1.07886149134)*x[0]+(-1.38999283206)*x[1]
        arg[0,1,0]=(0.246468026972)*x[0]+(-1.37217464882)*x[1]
        arg[0,1,1]=(0.645672648599)*x[0]+(1.8070624103)*x[1]
        arg[1,0,0]=(-1.02278989388)*x[0]+(0.151634023103)*x[1]
        arg[1,0,1]=(-0.667332860541)*x[0]+(0.312742149193)*x[1]
        arg[1,1,0]=(-0.848647385832)*x[0]+(0.466438038516)*x[1]
        arg[1,1,1]=(-0.185495911372)*x[0]+(-0.137118786891)*x[1]
        arg[2,0,0]=(-0.0250555870731)*x[0]+(0.905553573283)*x[1]
        arg[2,0,1]=(-1.03940852939)*x[0]+(-1.57326016394)*x[1]
        arg[2,1,0]=(1.48834718811)*x[0]+(-0.340889253246)*x[1]
        arg[2,1,1]=(0.875721514739)*x[0]+(-0.310857412212)*x[1]
        arg[3,0,0]=(1.25129508831)*x[0]+(-1.69312043525)*x[1]
        arg[3,0,1]=(1.50124675312)*x[0]+(0.244195748654)*x[1]
        arg[3,1,0]=(0.292060478909)*x[0]+(-0.660005422696)*x[1]
        arg[3,1,1]=(0.791335231308)*x[0]+(0.466481573591)*x[1]
        arg[4,0,0]=(-0.361241947236)*x[0]+(-1.43591648587)*x[1]
        arg[4,0,1]=(-1.10385456695)*x[0]+(-0.325314195825)*x[1]
        arg[4,1,0]=(1.01004173033)*x[0]+(0.426824552857)*x[1]
        arg[4,1,1]=(-0.631382479612)*x[0]+(0.0603792190966)*x[1]
        arg[5,0,0]=(-0.30404368786)*x[0]+(-0.0102498522932)*x[1]
        arg[5,0,1]=(0.109188848746)*x[0]+(-1.16846160105)*x[1]
        arg[5,1,0]=(-0.351916879547)*x[0]+(-0.320714795448)*x[1]
        arg[5,1,1]=(-0.534138562652)*x[0]+(-0.256013589844)*x[1]
        ref[0,0,0]=(-0.506159375456)/(o+1.)+(-0.522895228783)+(-0.794470828783)*0.5**o
        ref[0,0,1]=(-0.930984367068)/(o+1.)+(-0.636851258685)+(-0.264167438968)*0.5**o
        ref[0,1,0]=(-0.674222792172)/(o+1.)+(-0.52131379048)+(0.591143751282)*0.5**o
        ref[0,1,1]=(0.953370751283)/(o+1.)+(0.377773224645)+(0.743817858326)*0.5**o
        ref[1,0,0]=(0.700508215428)/(o+1.)+(-0.303343530867)+(-0.964977024471)*0.5**o
        ref[1,0,1]=(-0.408185116452)/(o+1.)+(0.0668983569743)+(-0.0802023088445)*0.5**o
        ref[1,1,0]=(0.80742345527)/(o+1.)+(-0.405780705683)+(-0.37807139122)*0.5**o
        ref[1,1,1]=(0.0184615568095)/(o+1.)+(0.221921562266)+(-0.784919379605)*0.5**o
        ref[2,0,0]=(0.571906256883)/(o+1.)+(-0.0415588382133)+(0.391709405753)*0.5**o
        ref[2,0,1]=(-0.778176009451)/(o+1.)+(-0.761680990566)+(-0.311130702752)*0.5**o
        ref[2,1,0]=(0.498214686321)/(o+1.)+(-0.146173705483)+(0.941590659509)*0.5**o
        ref[2,1,1]=(-0.273747071818)/(o+1.)+(-0.0319184747153)+(0.902448123776)*0.5**o
        ref[3,0,0]=(-0.730029471178)/(o+1.)+(-0.288672747933)+(0.865549620109)*0.5**o
        ref[3,0,1]=(-0.371409186475)/(o+1.)+(0.623169375401)+(0.870512937445)*0.5**o
        ref[3,1,0]=(0.0611454391867)/(o+1.)+(-0.627723874214)+(0.826357365454)*0.5**o
        ref[3,1,1]=(0.785553447935)/(o+1.)+(0.246940213405)+(-0.021617069845)*0.5**o
        ref[4,0,0]=(-0.936201955907)/(o+1.)+(-0.484823720021)+(0.108690962842)*0.5**o
        ref[4,0,1]=(-0.277830390815)/(o+1.)+(-0.151786547071)+(-0.847765277819)*0.5**o
        ref[4,1,0]=(0.12293099533)/(o+1.)+(0.295620382054)+(0.722694523749)*0.5**o
        ref[4,1,1]=(-0.5517019185)/(o+1.)+(0.0850748937088)+(-0.189451129433)*0.5**o
        ref[5,0,0]=(-0.942533148805)/(o+1.)+(0.425680504872)+(-0.223121401092)*0.5**o
        ref[5,0,1]=(-0.377134228332)/(o+1.)+(-0.721019409569)+(0.759900295162)*0.5**o
        ref[5,1,0]=(-0.643828391942)/(o+1.)+(0.399974456311)+(-0.828752195674)*0.5**o
        ref[5,1,1]=(-0.118293731439)/(o+1.)+(-0.355568544431)+(0.0392786678054)*0.5**o
      else:
        arg[0,0,0]=(0.205446621454)*x[0]+(0.10254878667)*x[1]+(1.27673728979)*x[2]
        arg[0,0,1]=(0.758846313193)*x[0]+(0.63320336713)*x[1]+(-0.42638505807)*x[2]
        arg[0,1,0]=(-0.0127116561603)*x[0]+(-0.360359420027)*x[1]+(0.540508591493)*x[2]
        arg[0,1,1]=(-0.278918629538)*x[0]+(-0.0291534835772)*x[1]+(0.708655731565)*x[2]
        arg[1,0,0]=(-0.612965210483)*x[0]+(0.787787556912)*x[1]+(0.222384448462)*x[2]
        arg[1,0,1]=(1.07409083742)*x[0]+(1.19429173706)*x[1]+(-0.484979976779)*x[2]
        arg[1,1,0]=(-1.25714999157)*x[0]+(-0.321346019622)*x[1]+(0.278329320837)*x[2]
        arg[1,1,1]=(-0.512262689707)*x[0]+(1.6891992053)*x[1]+(-1.02351830068)*x[2]
        arg[2,0,0]=(-1.57911907773)*x[0]+(0.096124258122)*x[1]+(0.336619253521)*x[2]
        arg[2,0,1]=(0.0901144254199)*x[0]+(-1.36818186255)*x[1]+(1.2969609239)*x[2]
        arg[2,1,0]=(0.158329511499)*x[0]+(0.696997185217)*x[1]+(-0.400535235927)*x[2]
        arg[2,1,1]=(-1.44570853201)*x[0]+(0.237359587195)*x[1]+(-1.17323024801)*x[2]
        arg[3,0,0]=(-1.4325812824)*x[0]+(-0.733085265902)*x[1]+(0.125883186484)*x[2]
        arg[3,0,1]=(-0.0529889021435)*x[0]+(0.335154600942)*x[1]+(0.839337375209)*x[2]
        arg[3,1,0]=(-0.423552182963)*x[0]+(1.31813289315)*x[1]+(0.13235854915)*x[2]
        arg[3,1,1]=(0.53278572217)*x[0]+(0.159434073071)*x[1]+(-0.313621199136)*x[2]
        arg[4,0,0]=(0.747779887673)*x[0]+(0.30977483911)*x[1]+(-0.0760295896739)*x[2]
        arg[4,0,1]=(0.00295118909194)*x[0]+(-0.687914867842)*x[1]+(0.0386268422915)*x[2]
        arg[4,1,0]=(-0.276363532754)*x[0]+(0.52102466785)*x[1]+(-0.684840191917)*x[2]
        arg[4,1,1]=(0.0283138898538)*x[0]+(0.116042458294)*x[1]+(0.485578401528)*x[2]
        arg[5,0,0]=(-0.118402060192)*x[0]+(-1.18409809944)*x[1]+(-0.304048852342)*x[2]
        arg[5,0,1]=(0.174393790186)*x[0]+(-0.524912689263)*x[1]+(-0.648708392355)*x[2]
        arg[5,1,0]=(-0.254958756344)*x[0]+(-0.772071348728)*x[1]+(-0.0572494064027)*x[2]
        arg[5,1,1]=(-0.74777072018)*x[0]+(0.19109939364)*x[1]+(0.498973987814)*x[2]
        ref[0,0,0]=(0.986457444152)/(o+1.)+(0.258994989412)+(0.0802852749346)*0.5**o
        ref[0,0,1]=(-0.0286717143415)/(o+1.)+(0.554867138611)+(-0.115397940627)*0.5**o
        ref[0,1,0]=(-0.886482131138)/(o+1.)+(0.464293129749)+(0.125333386946)*0.5**o
        ref[0,1,1]=(0.199214164457)/(o+1.)+(0.336882900218)+(-0.472396346443)*0.5**o
        ref[1,0,0]=(0.36021680954)/(o+1.)+(0.408903924703)+(-0.780817864055)*0.5**o
        ref[1,0,1]=(0.44795581074)/(o+1.)+(0.472159267742)+(0.391128251477)*0.5**o
        ref[1,1,0]=(1.02665734893)/(o+1.)+(-0.71418625728)+(-0.898451524719)*0.5**o
        ref[1,1,1]=(0.0322676607777)/(o+1.)+(0.105378958385)+(-0.0896073626306)*0.5**o
        ref[2,0,0]=(1.15723247637)/(o+1.)+(-0.719604538702)+(-0.86439896505)*0.5**o
        ref[2,0,1]=(-0.254100181576)/(o+1.)+(-0.0387997401585)+(0.35059314866)*0.5**o
        ref[2,1,0]=(-0.636439167982)/(o+1.)+(0.563390681981)+(-0.0355507351918)*0.5**o
        ref[2,1,1]=(-0.265265225142)/(o+1.)+(-0.635620798949)+(-0.845072369786)*0.5**o
        ref[3,0,0]=(-0.062162800164)/(o+1.)+(-0.554499928282)+(-0.868620705087)*0.5**o
        ref[3,0,1]=(0.784681961813)/(o+1.)+(0.355202655131)+(-0.373584198067)*0.5**o
        ref[3,1,0]=(1.20272043429)/(o+1.)+(0.352677694103)+(-0.881136563162)*0.5**o
        ref[3,1,1]=(-0.311152797156)/(o+1.)+(0.313736563953)+(0.0622782653549)*0.5**o
        ref[4,0,0]=(-0.365517620238)/(o+1.)+(0.733502174253)+(-0.119961591158)*0.5**o
        ref[4,0,1]=(0.0155845896619)/(o+1.)+(-0.654963543839)+(0.648005661558)*0.5**o
        ref[4,1,0]=(0.405697975915)/(o+1.)+(-0.637805133954)+(0.429733235172)*0.5**o
        ref[4,1,1]=(-0.0469738326054)/(o+1.)+(0.0590721074184)+(0.558764367445)*0.5**o
        ref[5,0,0]=(-0.333825007954)/(o+1.)+(-0.46448520882)+(-0.343753586377)*0.5**o
        ref[5,0,1]=(-0.469125954795)/(o+1.)+(-0.147192309255)+(-0.235716718127)*0.5**o
        ref[5,1,0]=(-0.573275891777)/(o+1.)+(-0.478328163121)+(0.445652706545)*0.5**o
        ref[5,1,1]=(0.0545515234726)/(o+1.)+(-0.153919681346)+(0.195590500493)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactZero_fromData_ReducedContinuousFunction_rank4(self):
      """
      tests integral of rank 4 Data on the ReducedFunctionOnContactZero

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5, 3, 2),w)
      ref=numpy.zeros((4, 5, 3, 2),numpy.float_)
      if dim==2:
        arg[0,0,0,0]=(-0.226142137728)*x[0]+(0.85681895183)*x[1]
        arg[0,0,0,1]=(-0.824407181762)*x[0]+(-0.290023144892)*x[1]
        arg[0,0,1,0]=(-1.25987816017)*x[0]+(0.824340990595)*x[1]
        arg[0,0,1,1]=(-0.0917811606765)*x[0]+(0.660211932964)*x[1]
        arg[0,0,2,0]=(-1.31073149178)*x[0]+(0.482324984918)*x[1]
        arg[0,0,2,1]=(0.256995458227)*x[0]+(0.79244254852)*x[1]
        arg[0,1,0,0]=(-0.0682147475407)*x[0]+(0.243620052061)*x[1]
        arg[0,1,0,1]=(0.071099982687)*x[0]+(-0.62986754897)*x[1]
        arg[0,1,1,0]=(-0.0716793733131)*x[0]+(-1.15235525814)*x[1]
        arg[0,1,1,1]=(0.193107575216)*x[0]+(-0.596149957543)*x[1]
        arg[0,1,2,0]=(-0.160142629886)*x[0]+(1.36581707067)*x[1]
        arg[0,1,2,1]=(-1.21477648376)*x[0]+(-0.732000298369)*x[1]
        arg[0,2,0,0]=(-1.02468971059)*x[0]+(0.330721595217)*x[1]
        arg[0,2,0,1]=(1.11414907391)*x[0]+(1.29386127812)*x[1]
        arg[0,2,1,0]=(1.1485850154)*x[0]+(0.387585977495)*x[1]
        arg[0,2,1,1]=(0.237382583072)*x[0]+(1.56248221279)*x[1]
        arg[0,2,2,0]=(-0.949129049229)*x[0]+(0.578823379063)*x[1]
        arg[0,2,2,1]=(-1.0667182537)*x[0]+(0.462227485959)*x[1]
        arg[0,3,0,0]=(1.37954377554)*x[0]+(0.541856880139)*x[1]
        arg[0,3,0,1]=(0.401584700319)*x[0]+(0.577102327147)*x[1]
        arg[0,3,1,0]=(-0.289796408108)*x[0]+(0.807853785325)*x[1]
        arg[0,3,1,1]=(0.0038467861822)*x[0]+(-0.931311510771)*x[1]
        arg[0,3,2,0]=(0.859036254821)*x[0]+(0.198884218516)*x[1]
        arg[0,3,2,1]=(-0.00310256070968)*x[0]+(0.511838475968)*x[1]
        arg[0,4,0,0]=(0.539686886802)*x[0]+(-0.324612924267)*x[1]
        arg[0,4,0,1]=(-0.0458655898565)*x[0]+(0.211810380966)*x[1]
        arg[0,4,1,0]=(0.302333533003)*x[0]+(0.0663033316831)*x[1]
        arg[0,4,1,1]=(-0.981519161639)*x[0]+(-0.00207756909914)*x[1]
        arg[0,4,2,0]=(0.283659447004)*x[0]+(-0.231526592979)*x[1]
        arg[0,4,2,1]=(-1.17679167875)*x[0]+(0.285698985756)*x[1]
        arg[1,0,0,0]=(-0.162900059582)*x[0]+(-0.877164340783)*x[1]
        arg[1,0,0,1]=(-0.50468605065)*x[0]+(-0.160440073709)*x[1]
        arg[1,0,1,0]=(-0.100901676644)*x[0]+(-0.296111526029)*x[1]
        arg[1,0,1,1]=(0.176107421869)*x[0]+(1.74393708745)*x[1]
        arg[1,0,2,0]=(-0.979807721703)*x[0]+(-0.229400841067)*x[1]
        arg[1,0,2,1]=(0.217472795645)*x[0]+(0.0881503243323)*x[1]
        arg[1,1,0,0]=(-0.127980123625)*x[0]+(0.0932095456541)*x[1]
        arg[1,1,0,1]=(0.829674410931)*x[0]+(-0.796196041601)*x[1]
        arg[1,1,1,0]=(1.28656110233)*x[0]+(-1.27850471392)*x[1]
        arg[1,1,1,1]=(1.159269884)*x[0]+(-0.973135433631)*x[1]
        arg[1,1,2,0]=(1.46274837856)*x[0]+(-1.81554270863)*x[1]
        arg[1,1,2,1]=(0.0973472511688)*x[0]+(0.551451261495)*x[1]
        arg[1,2,0,0]=(0.510667671293)*x[0]+(-0.51075982099)*x[1]
        arg[1,2,0,1]=(-1.41612093451)*x[0]+(0.614417669629)*x[1]
        arg[1,2,1,0]=(1.46595125796)*x[0]+(0.60374279122)*x[1]
        arg[1,2,1,1]=(-0.317185218187)*x[0]+(-0.0785532882696)*x[1]
        arg[1,2,2,0]=(-0.388504041993)*x[0]+(-0.838003398775)*x[1]
        arg[1,2,2,1]=(0.0497207129012)*x[0]+(-0.622155692247)*x[1]
        arg[1,3,0,0]=(-0.0471400179563)*x[0]+(1.3378948207)*x[1]
        arg[1,3,0,1]=(-1.22250822195)*x[0]+(0.629714783138)*x[1]
        arg[1,3,1,0]=(-1.28801080408)*x[0]+(-1.54079710038)*x[1]
        arg[1,3,1,1]=(-0.426412448469)*x[0]+(-1.09410474021)*x[1]
        arg[1,3,2,0]=(0.022834167351)*x[0]+(-0.497423389788)*x[1]
        arg[1,3,2,1]=(-0.787492124301)*x[0]+(-0.245621168419)*x[1]
        arg[1,4,0,0]=(-1.37814880919)*x[0]+(-0.704805848451)*x[1]
        arg[1,4,0,1]=(-1.12494770055)*x[0]+(1.44920024305)*x[1]
        arg[1,4,1,0]=(-0.809532572761)*x[0]+(1.16638587657)*x[1]
        arg[1,4,1,1]=(0.931784486816)*x[0]+(1.34629889453)*x[1]
        arg[1,4,2,0]=(-0.172430942793)*x[0]+(-0.584736998321)*x[1]
        arg[1,4,2,1]=(0.00542948736389)*x[0]+(0.43917597284)*x[1]
        arg[2,0,0,0]=(-0.946122337406)*x[0]+(-0.161940168137)*x[1]
        arg[2,0,0,1]=(-0.7856653779)*x[0]+(-0.75287701788)*x[1]
        arg[2,0,1,0]=(-1.08858367592)*x[0]+(-0.530380818694)*x[1]
        arg[2,0,1,1]=(-1.0573555095)*x[0]+(-0.854148765785)*x[1]
        arg[2,0,2,0]=(0.723412825328)*x[0]+(0.326761950437)*x[1]
        arg[2,0,2,1]=(-0.0143142161225)*x[0]+(-0.752724311876)*x[1]
        arg[2,1,0,0]=(-0.942548092355)*x[0]+(0.700861094897)*x[1]
        arg[2,1,0,1]=(-0.712286567957)*x[0]+(0.230406545738)*x[1]
        arg[2,1,1,0]=(1.27414839774)*x[0]+(0.724784649037)*x[1]
        arg[2,1,1,1]=(0.637494163002)*x[0]+(0.442489599192)*x[1]
        arg[2,1,2,0]=(1.81617631726)*x[0]+(1.56870218809)*x[1]
        arg[2,1,2,1]=(1.16419900666)*x[0]+(-0.044314729141)*x[1]
        arg[2,2,0,0]=(0.880267125196)*x[0]+(0.77124406259)*x[1]
        arg[2,2,0,1]=(0.482402867542)*x[0]+(0.0746924305017)*x[1]
        arg[2,2,1,0]=(0.105861259696)*x[0]+(0.484292207267)*x[1]
        arg[2,2,1,1]=(-0.107785838196)*x[0]+(0.701132243209)*x[1]
        arg[2,2,2,0]=(0.559717405844)*x[0]+(-1.5585942717)*x[1]
        arg[2,2,2,1]=(-0.989130712605)*x[0]+(0.221361137556)*x[1]
        arg[2,3,0,0]=(-0.045471818095)*x[0]+(-0.437941959792)*x[1]
        arg[2,3,0,1]=(1.25840468379)*x[0]+(-1.18872011057)*x[1]
        arg[2,3,1,0]=(1.70665829408)*x[0]+(0.705273921003)*x[1]
        arg[2,3,1,1]=(-0.0928709253845)*x[0]+(0.785115449756)*x[1]
        arg[2,3,2,0]=(-0.887243418838)*x[0]+(1.06790660642)*x[1]
        arg[2,3,2,1]=(-0.572644686872)*x[0]+(-0.672831948394)*x[1]
        arg[2,4,0,0]=(1.28282566801)*x[0]+(0.651985199447)*x[1]
        arg[2,4,0,1]=(0.0965663229335)*x[0]+(1.33819998052)*x[1]
        arg[2,4,1,0]=(0.205569041098)*x[0]+(-0.873095358733)*x[1]
        arg[2,4,1,1]=(-1.04452025745)*x[0]+(-0.204917033907)*x[1]
        arg[2,4,2,0]=(-0.181163938972)*x[0]+(0.0341993409272)*x[1]
        arg[2,4,2,1]=(1.68327027059)*x[0]+(-0.289436355642)*x[1]
        arg[3,0,0,0]=(1.05712810731)*x[0]+(0.915711819735)*x[1]
        arg[3,0,0,1]=(-0.171072039482)*x[0]+(-0.160656943623)*x[1]
        arg[3,0,1,0]=(-1.27063799869)*x[0]+(-0.605131642813)*x[1]
        arg[3,0,1,1]=(0.940512990835)*x[0]+(-0.116675577531)*x[1]
        arg[3,0,2,0]=(-0.194213869952)*x[0]+(0.364066752025)*x[1]
        arg[3,0,2,1]=(-0.328780465397)*x[0]+(-0.114689413673)*x[1]
        arg[3,1,0,0]=(-0.825914305895)*x[0]+(-0.167010056731)*x[1]
        arg[3,1,0,1]=(0.657779056207)*x[0]+(0.0666252451638)*x[1]
        arg[3,1,1,0]=(-0.464004426036)*x[0]+(-0.340932918409)*x[1]
        arg[3,1,1,1]=(-1.6698652667)*x[0]+(0.731870373141)*x[1]
        arg[3,1,2,0]=(1.39626250779)*x[0]+(-0.879152855557)*x[1]
        arg[3,1,2,1]=(1.34973919508)*x[0]+(-0.129753332761)*x[1]
        arg[3,2,0,0]=(1.03443450706)*x[0]+(0.807333477811)*x[1]
        arg[3,2,0,1]=(0.366011302596)*x[0]+(0.543804800926)*x[1]
        arg[3,2,1,0]=(0.262446846036)*x[0]+(-0.247786850991)*x[1]
        arg[3,2,1,1]=(-1.04657091196)*x[0]+(-0.485443426155)*x[1]
        arg[3,2,2,0]=(0.342434327538)*x[0]+(-0.148503585588)*x[1]
        arg[3,2,2,1]=(-0.189833429716)*x[0]+(0.424391874819)*x[1]
        arg[3,3,0,0]=(-0.118348838118)*x[0]+(0.397628264341)*x[1]
        arg[3,3,0,1]=(0.481440510084)*x[0]+(-0.393576221424)*x[1]
        arg[3,3,1,0]=(0.907533552208)*x[0]+(-0.668646549376)*x[1]
        arg[3,3,1,1]=(-1.49731714213)*x[0]+(-1.18710678264)*x[1]
        arg[3,3,2,0]=(-0.460660079207)*x[0]+(1.48078266444)*x[1]
        arg[3,3,2,1]=(-0.161073556988)*x[0]+(-0.486827042754)*x[1]
        arg[3,4,0,0]=(-0.668755455623)*x[0]+(0.345804059434)*x[1]
        arg[3,4,0,1]=(1.00528889193)*x[0]+(0.978507928191)*x[1]
        arg[3,4,1,0]=(0.98955856331)*x[0]+(1.0421852144)*x[1]
        arg[3,4,1,1]=(0.711688260873)*x[0]+(1.39962222322)*x[1]
        arg[3,4,2,0]=(1.08512429209)*x[0]+(-0.555249097743)*x[1]
        arg[3,4,2,1]=(-1.10443784007)*x[0]+(-0.782274429169)*x[1]
        ref[0,0,0,0]=(0.151153165564)/(o+1.)+(0.112609656739)+(0.254304335059)*0.5**o
        ref[0,0,0,1]=(-0.530602583511)/(o+1.)+(0.1817855378)+(-0.947398818743)*0.5**o
        ref[0,0,1,0]=(0.308747437738)/(o+1.)+(-0.0338129255643)+(-0.676658756184)*0.5**o
        ref[0,0,1,1]=(0.321026553427)/(o+1.)+(0.0692705367618)+(0.108863145337)*0.5**o
        ref[0,0,2,0]=(-0.0908034666655)/(o+1.)+(-0.1100205155)+(-0.517562009199)*0.5**o
        ref[0,0,2,1]=(0.303456040903)/(o+1.)+(0.500090875336)+(-0.254199784829)*0.5**o
        ref[0,1,0,0]=(-0.440641221831)/(o+1.)+(0.456218946873)+(-0.296391367396)*0.5**o
        ref[0,1,0,1]=(0.222628561762)/(o+1.)+(-0.162064412854)+(-0.457267302337)*0.5**o
        ref[0,1,1,0]=(-0.913361887408)/(o+1.)+(-0.356168674239)+(0.401664604431)*0.5**o
        ref[0,1,1,1]=(-0.543116639248)/(o+1.)+(-0.120902439422)+(0.381879135767)*0.5**o
        ref[0,1,2,0]=(0.993649262831)/(o+1.)+(0.384695858206)+(-0.55736653846)*0.5**o
        ref[0,1,2,1]=(-0.968077797451)/(o+1.)+(-0.187765942637)+(-0.603167099406)*0.5**o
        ref[0,2,0,0]=(0.873662019918)/(o+1.)+(-0.710635360033)+(-0.146359415223)*0.5**o
        ref[0,2,0,1]=(0.466408230693)/(o+1.)+(0.77626565285)+(0.389070815631)*0.5**o
        ref[0,2,1,0]=(-0.475010813579)/(o+1.)+(0.525435880006)+(0.960310046461)*0.5**o
        ref[0,2,1,1]=(0.697127520484)/(o+1.)+(0.357204276552)+(0.388328722275)*0.5**o
        ref[0,2,2,0]=(-0.108434738487)/(o+1.)+(0.356443300466)+(-0.97475753261)*0.5**o
        ref[0,2,2,1]=(0.952274964304)/(o+1.)+(-0.497379508976)+(-0.562006714089)*0.5**o
        ref[0,3,0,0]=(-0.0368012761671)/(o+1.)+(0.594000625518)+(0.770200680808)*0.5**o
        ref[0,3,0,1]=(0.450361861836)/(o+1.)+(0.182031052848)+(0.164263059934)*0.5**o
        ref[0,3,1,0]=(0.500714741956)/(o+1.)+(0.0479711343168)+(-0.0785996333735)*0.5**o
        ref[0,3,1,1]=(-0.949280938645)/(o+1.)+(0.452441874504)+(-0.88306753495)*0.5**o
        ref[0,3,2,0]=(0.186931712837)/(o+1.)+(0.222335175903)+(0.426318408694)*0.5**o
        ref[0,3,2,1]=(-0.288015428918)/(o+1.)+(0.80948657963)+(-0.822221815085)*0.5**o
        ref[0,4,0,0]=(0.670151184695)/(o+1.)+(-0.415883847013)+(0.376690471866)*0.5**o
        ref[0,4,0,1]=(-0.107106843992)/(o+1.)+(0.555375298435)+(-0.837698961768)*0.5**o
        ref[0,4,1,0]=(-0.299999096802)/(o+1.)+(0.668996782774)+(-0.669357604061)*0.5**o
        ref[0,4,1,1]=(0.778006183401)/(o+1.)+(-0.614473529032)+(-0.532655856076)*0.5**o
        ref[0,4,2,0]=(-0.250068389818)/(o+1.)+(0.156274314197)+(-0.0103473845504)*0.5**o
        ref[0,4,2,1]=(0.902125322216)/(o+1.)+(-0.616424567683)+(-0.560368879846)*0.5**o
        ref[1,0,0,0]=(-0.133690258295)/(o+1.)+(-0.715949360249)+(0.525524578427)*0.5**o
        ref[1,0,0,1]=(-0.788097689216)/(o+1.)+(0.0998405659256)+(-0.076709566994)*0.5**o
        ref[1,0,1,0]=(0.144280886187)/(o+1.)+(0.0775377290314)+(-0.696369546924)*0.5**o
        ref[1,0,1,1]=(0.824172196605)/(o+1.)+(0.561804029444)+(-0.0277357461773)*0.5**o
        ref[1,0,2,0]=(0.422564421802)/(o+1.)+(-0.60719307099)+(-0.417386842591)*0.5**o
        ref[1,0,2,1]=(0.778908672762)/(o+1.)+(0.0163446340765)+(-0.505974820938)*0.5**o
        ref[1,1,0,0]=(0.0574071047387)/(o+1.)+(-0.350514214742)+(0.608850746775)*0.5**o
        ref[1,1,0,1]=(-0.991660961792)/(o+1.)+(0.322084279455)+(0.380970772212)*0.5**o
        ref[1,1,1,0]=(-0.823761120268)/(o+1.)+(-0.0495419092898)+(0.930901327253)*0.5**o
        ref[1,1,1,1]=(-0.384729980569)/(o+1.)+(-0.0392583789163)+(0.649381188769)*0.5**o
        ref[1,1,2,0]=(-0.840365304069)/(o+1.)+(-0.0150512662777)+(0.517673506558)*0.5**o
        ref[1,1,2,1]=(0.555976660846)/(o+1.)+(-0.307265886898)+(0.707353625614)*0.5**o
        ref[1,2,0,0]=(-0.423056705491)/(o+1.)+(0.0559404393793)+(0.311083677036)*0.5**o
        ref[1,2,0,1]=(0.15409797664)/(o+1.)+(-0.0906637067519)+(-0.774473828018)*0.5**o
        ref[1,2,1,0]=(0.796718644088)/(o+1.)+(0.248012077098)+(0.776951250898)*0.5**o
        ref[1,2,1,1]=(-0.342830889263)/(o+1.)+(-0.282162737841)+(0.511417858487)*0.5**o
        ref[1,2,2,0]=(-0.722403016671)/(o+1.)+(-0.367059457954)+(0.230014491812)*0.5**o
        ref[1,2,2,1]=(-0.773685058881)/(o+1.)+(0.412807900451)+(-0.624365721366)*0.5**o
        ref[1,3,0,0]=(0.527481434758)/(o+1.)+(0.36602961117)+(0.031214145649)*0.5**o
        ref[1,3,0,1]=(0.0630927531585)/(o+1.)+(-0.164989701995)+(-0.325906787975)*0.5**o
        ref[1,3,1,0]=(-0.612390590593)/(o+1.)+(-0.78855996274)+(-0.63929738839)*0.5**o
        ref[1,3,1,1]=(-0.962656044265)/(o+1.)+(-0.00576535037253)+(-0.546330443673)*0.5**o
        ref[1,3,2,0]=(-0.947280922008)/(o+1.)+(-0.137449579996)+(0.747590859563)*0.5**o
        ref[1,3,2,1]=(-0.946623026202)/(o+1.)+(0.0787536550213)+(-0.24399757656)*0.5**o
        ref[1,4,0,0]=(-0.174471016943)/(o+1.)+(-0.636500556039)+(-0.635482528623)*0.5**o
        ref[1,4,0,1]=(0.806379391595)/(o+1.)+(-0.00304106577093)+(-0.476044717545)*0.5**o
        ref[1,4,1,0]=(0.729649125134)/(o+1.)+(0.0151477051616)+(-0.403091231646)*0.5**o
        ref[1,4,1,1]=(0.559976454745)/(o+1.)+(0.479295896421)+(0.75951513376)*0.5**o
        ref[1,4,2,0]=(0.272199270818)/(o+1.)+(-0.0929072274332)+(-0.843552757065)*0.5**o
        ref[1,4,2,1]=(-0.552663678377)/(o+1.)+(0.613784745778)+(-0.230300352975)*0.5**o
        ref[2,0,0,0]=(-0.174247908098)/(o+1.)+(-0.108873783282)+(-0.716067030882)*0.5**o
        ref[2,0,0,1]=(-0.300584841532)/(o+1.)+(-0.485363665493)+(-0.267230223262)*0.5**o
        ref[2,0,1,0]=(0.316261805154)/(o+1.)+(-0.523474106451)+(-0.888278086869)*0.5**o
        ref[2,0,1,1]=(-0.699622562861)/(o+1.)+(-0.151089538248)+(-0.909702635929)*0.5**o
        ref[2,0,2,0]=(0.728545639829)/(o+1.)+(0.192083148877)+(-0.0625371618174)*0.5**o
        ref[2,0,2,1]=(-0.22042267239)/(o+1.)+(-0.611974599324)+(0.677333343041)*0.5**o
        ref[2,1,0,0]=(-0.0734883811113)/(o+1.)+(0.36100831022)+(-0.890215236787)*0.5**o
        ref[2,1,0,1]=(0.121656829057)/(o+1.)+(-0.130801635799)+(-0.341933579678)*0.5**o
        ref[2,1,1,0]=(0.221353168629)/(o+1.)+(0.618635673483)+(0.540308531179)*0.5**o
        ref[2,1,1,1]=(0.0152580531273)/(o+1.)+(0.252173415542)+(0.560378877984)*0.5**o
        ref[2,1,2,0]=(0.580319777731)/(o+1.)+(0.90566612137)+(0.993226484879)*0.5**o
        ref[2,1,2,1]=(-0.710250320949)/(o+1.)+(0.742177021871)+(0.345780554724)*0.5**o
        ref[2,2,0,0]=(0.770701563507)/(o+1.)+(0.240823471881)+(0.399162680517)*0.5**o
        ref[2,2,0,1]=(0.76121761761)/(o+1.)+(0.128382656738)+(-0.460887633042)*0.5**o
        ref[2,2,1,0]=(-0.269854962057)/(o+1.)+(0.690854898294)+(-0.521701367567)*0.5**o
        ref[2,2,1,1]=(0.719518595712)/(o+1.)+(0.029596430244)+(-0.185365051187)*0.5**o
        ref[2,2,2,0]=(-0.671157761165)/(o+1.)+(-0.142688285012)+(-0.0423425346644)*0.5**o
        ref[2,2,2,1]=(-0.541156097112)/(o+1.)+(0.0765180918929)+(-0.379649661722)*0.5**o
        ref[2,3,0,0]=(-0.0604347109212)/(o+1.)+(-0.19403702073)+(-0.034905025507)*0.5**o
        ref[2,3,0,1]=(-0.793398507071)/(o+1.)+(0.255681280626)+(0.351720519037)*0.5**o
        ref[2,3,1,0]=(-0.267058413731)/(o+1.)+(0.852211426049)+(0.974567776717)*0.5**o
        ref[2,3,1,1]=(0.228396641207)/(o+1.)+(0.322922896735)+(-0.181997910305)*0.5**o
        ref[2,3,2,0]=(0.480987773937)/(o+1.)+(-0.197709028569)+(0.0950934707835)*0.5**o
        ref[2,3,2,1]=(-0.648934475027)/(o+1.)+(-0.418383297875)+(0.240224435511)*0.5**o
        ref[2,4,0,0]=(0.430020261528)/(o+1.)+(0.53421018733)+(0.436370231269)*0.5**o
        ref[2,4,0,1]=(0.906390660223)/(o+1.)+(-0.056057545856)+(0.640490734948)*0.5**o
        ref[2,4,1,0]=(-0.732099770943)/(o+1.)+(-0.0485027617732)+(0.161578976854)*0.5**o
        ref[2,4,1,1]=(-0.827495680848)/(o+1.)+(0.285044728936)+(-0.992031068384)*0.5**o
        ref[2,4,2,0]=(-0.919905471787)/(o+1.)+(0.656971386264)+(-0.541001898785)*0.5**o
        ref[2,4,2,1]=(0.161975875302)/(o+1.)+(0.272665206557)+(0.686527626533)*0.5**o
        ref[3,0,0,0]=(0.584120963259)/(o+1.)+(0.207930729804)+(0.972857504177)*0.5**o
        ref[3,0,0,1]=(0.315823141491)/(o+1.)+(-0.217561208386)+(-0.212429707823)*0.5**o
        ref[3,0,1,0]=(-0.732294996139)/(o+1.)+(-0.278653143596)+(-0.586168358175)*0.5**o
        ref[3,0,1,1]=(0.481321847225)/(o+1.)+(-0.070080648792)+(0.482676863662)*0.5**o
        ref[3,0,2,0]=(0.616679859076)/(o+1.)+(-0.418784546677)+(0.390742116352)*0.5**o
        ref[3,0,2,1]=(0.866778417448)/(o+1.)+(-0.686585656643)+(0.0629230167692)*0.5**o
        ref[3,1,0,0]=(0.531890054167)/(o+1.)+(-0.516187607633)+(-0.492439201527)*0.5**o
        ref[3,1,0,1]=(-0.512361639413)/(o+1.)+(0.499557182545)+(0.237651575694)*0.5**o
        ref[3,1,1,0]=(-0.67542965405)/(o+1.)+(0.274414249559)+(-0.678336189514)*0.5**o
        ref[3,1,1,1]=(0.371936242507)/(o+1.)+(-0.1636482466)+(-0.982634642862)*0.5**o
        ref[3,1,2,0]=(-0.799408831501)/(o+1.)+(0.458633399549)+(0.399251684639)*0.5**o
        ref[3,1,2,1]=(-0.0145253903686)/(o+1.)+(0.238472345905)+(0.757566560882)*0.5**o
        ref[3,2,0,0]=(-0.130519473112)/(o+1.)+(0.695277604837)+(0.581732248306)*0.5**o
        ref[3,2,0,1]=(0.632010536438)/(o+1.)+(0.123066205529)+(0.0316731560259)*0.5**o
        ref[3,2,1,0]=(-0.709926210356)/(o+1.)+(0.0926699972302)+(0.53924621094)*0.5**o
        ref[3,2,1,1]=(0.428094002247)/(o+1.)+(-0.841997926131)+(-0.276112488096)*0.5**o
        ref[3,2,2,0]=(0.708481148086)/(o+1.)+(-0.50498161534)+(0.495412824545)*0.5**o
        ref[3,2,2,1]=(0.782824622703)/(o+1.)+(-0.151509698687)+(-0.245246780227)*0.5**o
        ref[3,3,0,0]=(0.848460868418)/(o+1.)+(-0.104639248461)+(-0.359902945273)*0.5**o
        ref[3,3,0,1]=(0.0569151307298)/(o+1.)+(-0.0674224535469)+(0.165794065024)*0.5**o
        ref[3,3,1,0]=(-0.165683739908)/(o+1.)+(-0.183388699236)+(0.771348141212)*0.5**o
        ref[3,3,1,1]=(-0.932550058069)/(o+1.)+(-0.47828109001)+(-0.795311686688)*0.5**o
        ref[3,3,2,0]=(0.821542940923)/(o+1.)+(0.166853194487)+(-0.13512674466)*0.5**o
        ref[3,3,2,1]=(-0.46375871628)/(o+1.)+(0.132701267732)+(-0.449544418926)*0.5**o
        ref[3,4,0,0]=(-0.074530542168)/(o+1.)+(-0.0567877963797)+(-0.134845261261)*0.5**o
        ref[3,4,0,1]=(0.238671520479)/(o+1.)+(0.50689253572)+(0.731340228204)*0.5**o
        ref[3,4,1,0]=(0.981601513335)/(o+1.)+(0.400717209822)+(0.248707844726)*0.5**o
        ref[3,4,1,1]=(0.592304390107)/(o+1.)+(0.342408637328)+(0.834188819333)*0.5**o
        ref[3,4,2,0]=(-0.339991330909)/(o+1.)+(-0.0145922376156)+(0.899051000485)*0.5**o
        ref[3,4,2,1]=(-0.626610688164)/(o+1.)+(-0.417008165115)+(-0.426085250846)*0.5**o
      else:
        arg[0,0,0,0]=(0.697666818785)*x[0]+(1.00202295791)*x[1]+(0.617700157184)*x[2]
        arg[0,0,0,1]=(-0.465577819457)*x[0]+(-1.40054693355)*x[1]+(1.16912653295)*x[2]
        arg[0,0,1,0]=(-1.26696500634)*x[0]+(-0.0469756169975)*x[1]+(-0.445591738991)*x[2]
        arg[0,0,1,1]=(-1.61654321709)*x[0]+(-1.69522962467)*x[1]+(-1.06089094851)*x[2]
        arg[0,0,2,0]=(0.229468378506)*x[0]+(0.716813596991)*x[1]+(-0.237703937029)*x[2]
        arg[0,0,2,1]=(-1.12052051905)*x[0]+(0.835000004618)*x[1]+(-0.853004812404)*x[2]
        arg[0,1,0,0]=(-1.75500745705)*x[0]+(0.655055836882)*x[1]+(1.05207750735)*x[2]
        arg[0,1,0,1]=(-0.0746024824253)*x[0]+(-0.421596816958)*x[1]+(-0.375715855989)*x[2]
        arg[0,1,1,0]=(-0.963269684396)*x[0]+(0.431993248836)*x[1]+(-1.63061928649)*x[2]
        arg[0,1,1,1]=(-0.418757265018)*x[0]+(-0.665626168583)*x[1]+(-0.335734432507)*x[2]
        arg[0,1,2,0]=(0.980346404307)*x[0]+(1.45442049802)*x[1]+(-0.833420308612)*x[2]
        arg[0,1,2,1]=(-1.52536017847)*x[0]+(0.99484028681)*x[1]+(0.109754321663)*x[2]
        arg[0,2,0,0]=(1.697179262)*x[0]+(-0.928732802147)*x[1]+(-0.653379513626)*x[2]
        arg[0,2,0,1]=(0.682698902948)*x[0]+(0.829307959583)*x[1]+(-1.0300112276)*x[2]
        arg[0,2,1,0]=(0.026084323266)*x[0]+(1.28865784608)*x[1]+(0.830446580645)*x[2]
        arg[0,2,1,1]=(0.486291506032)*x[0]+(-0.519637742913)*x[1]+(0.81671330629)*x[2]
        arg[0,2,2,0]=(-0.761591624634)*x[0]+(0.370951131011)*x[1]+(-0.256748479312)*x[2]
        arg[0,2,2,1]=(-0.249974326513)*x[0]+(0.267323484635)*x[1]+(-1.09642130905)*x[2]
        arg[0,3,0,0]=(0.752057563255)*x[0]+(-1.2772262175)*x[1]+(-0.264569668983)*x[2]
        arg[0,3,0,1]=(-0.173167796447)*x[0]+(0.31873208356)*x[1]+(-0.599780701112)*x[2]
        arg[0,3,1,0]=(-0.912464014941)*x[0]+(-1.56452517166)*x[1]+(1.37651660499)*x[2]
        arg[0,3,1,1]=(0.302009667854)*x[0]+(0.524213064769)*x[1]+(0.316301522683)*x[2]
        arg[0,3,2,0]=(-1.2940154673)*x[0]+(-0.900061396992)*x[1]+(0.3860029847)*x[2]
        arg[0,3,2,1]=(-0.97354131341)*x[0]+(1.08221883964)*x[1]+(-0.739323076416)*x[2]
        arg[0,4,0,0]=(1.86535317192)*x[0]+(-0.478359589546)*x[1]+(-0.798014592302)*x[2]
        arg[0,4,0,1]=(-1.08518693851)*x[0]+(-0.186048654536)*x[1]+(1.04915573267)*x[2]
        arg[0,4,1,0]=(-1.09692811055)*x[0]+(1.01042920935)*x[1]+(-0.73978089555)*x[2]
        arg[0,4,1,1]=(0.654656964663)*x[0]+(0.279204596126)*x[1]+(0.886217781361)*x[2]
        arg[0,4,2,0]=(-0.54009886291)*x[0]+(0.378037248739)*x[1]+(1.09785568886)*x[2]
        arg[0,4,2,1]=(-1.43995958913)*x[0]+(-0.720157090994)*x[1]+(0.0676362028136)*x[2]
        arg[1,0,0,0]=(1.47735412608)*x[0]+(0.342355546228)*x[1]+(-0.0134502415508)*x[2]
        arg[1,0,0,1]=(-0.419536590369)*x[0]+(-1.61108470511)*x[1]+(1.51281979355)*x[2]
        arg[1,0,1,0]=(1.31765698245)*x[0]+(-0.7499566294)*x[1]+(0.934537503478)*x[2]
        arg[1,0,1,1]=(0.688467914883)*x[0]+(-1.32681280673)*x[1]+(-0.387405705058)*x[2]
        arg[1,0,2,0]=(1.37069587556)*x[0]+(0.540830111773)*x[1]+(0.477311068263)*x[2]
        arg[1,0,2,1]=(-0.572066213761)*x[0]+(-1.11950925494)*x[1]+(1.07939217872)*x[2]
        arg[1,1,0,0]=(0.564997105326)*x[0]+(-0.73130546389)*x[1]+(0.435910937304)*x[2]
        arg[1,1,0,1]=(0.513931739674)*x[0]+(-0.944043404826)*x[1]+(0.014114227468)*x[2]
        arg[1,1,1,0]=(-0.77850597256)*x[0]+(-0.445835136484)*x[1]+(0.157478357479)*x[2]
        arg[1,1,1,1]=(0.752431176866)*x[0]+(0.659759664199)*x[1]+(-0.859515433521)*x[2]
        arg[1,1,2,0]=(0.0693158730911)*x[0]+(-0.820997827204)*x[1]+(-0.714694436829)*x[2]
        arg[1,1,2,1]=(0.604674152847)*x[0]+(0.647261183132)*x[1]+(-1.21981352028)*x[2]
        arg[1,2,0,0]=(0.0997504695984)*x[0]+(-0.271549839083)*x[1]+(-0.573467835942)*x[2]
        arg[1,2,0,1]=(0.01861845581)*x[0]+(0.157410502406)*x[1]+(0.939257868776)*x[2]
        arg[1,2,1,0]=(1.44909894631)*x[0]+(0.116038517547)*x[1]+(-0.851940489233)*x[2]
        arg[1,2,1,1]=(1.34598841007)*x[0]+(1.79163125841)*x[1]+(0.719171044245)*x[2]
        arg[1,2,2,0]=(-0.214737791448)*x[0]+(-1.85810839095)*x[1]+(-0.240776599927)*x[2]
        arg[1,2,2,1]=(-0.574707645928)*x[0]+(-0.664173831956)*x[1]+(0.310262106696)*x[2]
        arg[1,3,0,0]=(0.296213453592)*x[0]+(-0.831112482555)*x[1]+(0.130075739128)*x[2]
        arg[1,3,0,1]=(-0.369510604457)*x[0]+(0.0373909697998)*x[1]+(1.6689129783)*x[2]
        arg[1,3,1,0]=(0.0922435881419)*x[0]+(-0.280519827004)*x[1]+(0.285933839232)*x[2]
        arg[1,3,1,1]=(-0.649253282272)*x[0]+(-0.460065044123)*x[1]+(-0.0711164948063)*x[2]
        arg[1,3,2,0]=(0.793313307581)*x[0]+(-0.0587956351149)*x[1]+(0.810487259739)*x[2]
        arg[1,3,2,1]=(-1.38668597055)*x[0]+(1.51396934198)*x[1]+(-1.13918550255)*x[2]
        arg[1,4,0,0]=(1.40078167737)*x[0]+(-1.61915436345)*x[1]+(0.421710091214)*x[2]
        arg[1,4,0,1]=(-1.37982015877)*x[0]+(1.30607371738)*x[1]+(-0.608888848419)*x[2]
        arg[1,4,1,0]=(-0.558087802805)*x[0]+(-0.214169499512)*x[1]+(-0.766242546503)*x[2]
        arg[1,4,1,1]=(-1.75503436584)*x[0]+(0.183604705451)*x[1]+(-0.148071484576)*x[2]
        arg[1,4,2,0]=(0.537942069142)*x[0]+(-0.00640749928252)*x[1]+(-0.277544629657)*x[2]
        arg[1,4,2,1]=(-0.652645833371)*x[0]+(-1.33439077026)*x[1]+(-0.226308197201)*x[2]
        arg[2,0,0,0]=(1.45646289279)*x[0]+(1.40842127675)*x[1]+(1.75078281495)*x[2]
        arg[2,0,0,1]=(0.867988651995)*x[0]+(1.35061812241)*x[1]+(-1.56520776323)*x[2]
        arg[2,0,1,0]=(0.549686523522)*x[0]+(-0.528085465813)*x[1]+(-0.994978820267)*x[2]
        arg[2,0,1,1]=(0.383595996552)*x[0]+(0.612989859087)*x[1]+(-0.560401247231)*x[2]
        arg[2,0,2,0]=(-0.341259919499)*x[0]+(-0.125491675042)*x[1]+(0.545572340757)*x[2]
        arg[2,0,2,1]=(-0.414269395434)*x[0]+(-0.433969881898)*x[1]+(-1.01820199328)*x[2]
        arg[2,1,0,0]=(-0.113575280389)*x[0]+(0.326916039942)*x[1]+(-0.576072890249)*x[2]
        arg[2,1,0,1]=(-0.296473969661)*x[0]+(0.195675099431)*x[1]+(-1.72378150831)*x[2]
        arg[2,1,1,0]=(-1.31575932758)*x[0]+(-0.122557423522)*x[1]+(0.53435027695)*x[2]
        arg[2,1,1,1]=(0.299435754608)*x[0]+(0.727140413706)*x[1]+(0.848931358475)*x[2]
        arg[2,1,2,0]=(0.2618965198)*x[0]+(-0.461672573339)*x[1]+(0.923592803172)*x[2]
        arg[2,1,2,1]=(-0.326817258523)*x[0]+(-0.0363300805442)*x[1]+(1.82049875918)*x[2]
        arg[2,2,0,0]=(-0.435113320888)*x[0]+(0.0257984353786)*x[1]+(0.287846577657)*x[2]
        arg[2,2,0,1]=(1.39178068279)*x[0]+(0.136781967539)*x[1]+(-1.38161590351)*x[2]
        arg[2,2,1,0]=(-0.887874199356)*x[0]+(-0.476769153654)*x[1]+(-0.618760485829)*x[2]
        arg[2,2,1,1]=(1.58883355098)*x[0]+(-0.889469047555)*x[1]+(-0.678486755843)*x[2]
        arg[2,2,2,0]=(0.237147029365)*x[0]+(-0.509883462401)*x[1]+(0.533163151551)*x[2]
        arg[2,2,2,1]=(0.66513913749)*x[0]+(1.30317030657)*x[1]+(0.804083365617)*x[2]
        arg[2,3,0,0]=(1.01531518403)*x[0]+(1.3753053143)*x[1]+(0.000877284235964)*x[2]
        arg[2,3,0,1]=(-0.40853642543)*x[0]+(0.154794268783)*x[1]+(-1.34306671117)*x[2]
        arg[2,3,1,0]=(0.43065284359)*x[0]+(-0.28965146009)*x[1]+(0.660248828789)*x[2]
        arg[2,3,1,1]=(-1.4334922681)*x[0]+(-0.0348802858609)*x[1]+(0.502051304974)*x[2]
        arg[2,3,2,0]=(-1.17399537487)*x[0]+(0.865787043946)*x[1]+(-0.0105949677478)*x[2]
        arg[2,3,2,1]=(0.63104158794)*x[0]+(0.436196079951)*x[1]+(0.498606623061)*x[2]
        arg[2,4,0,0]=(-1.44410177354)*x[0]+(0.513406326011)*x[1]+(0.699127199447)*x[2]
        arg[2,4,0,1]=(-0.981727023077)*x[0]+(0.484182120695)*x[1]+(0.667175832045)*x[2]
        arg[2,4,1,0]=(-1.3352602588)*x[0]+(0.626269023364)*x[1]+(0.813537646695)*x[2]
        arg[2,4,1,1]=(-0.932486930114)*x[0]+(0.299448807849)*x[1]+(0.195862014797)*x[2]
        arg[2,4,2,0]=(-0.0621489644806)*x[0]+(0.131979664158)*x[1]+(0.442029269778)*x[2]
        arg[2,4,2,1]=(-0.507508608633)*x[0]+(-1.28613226533)*x[1]+(-0.697920444023)*x[2]
        arg[3,0,0,0]=(-0.984942677585)*x[0]+(-1.58913299616)*x[1]+(-0.373026784822)*x[2]
        arg[3,0,0,1]=(0.616666427113)*x[0]+(-0.625162800471)*x[1]+(-0.796307290839)*x[2]
        arg[3,0,1,0]=(-0.48469203683)*x[0]+(-0.0572105166553)*x[1]+(0.809572383581)*x[2]
        arg[3,0,1,1]=(-0.487160309892)*x[0]+(0.77646454341)*x[1]+(-0.41947544573)*x[2]
        arg[3,0,2,0]=(-1.17902121554)*x[0]+(-0.244939532033)*x[1]+(-0.282994359476)*x[2]
        arg[3,0,2,1]=(0.33515909881)*x[0]+(0.146866561705)*x[1]+(1.81486098364)*x[2]
        arg[3,1,0,0]=(0.379977901341)*x[0]+(0.834531701517)*x[1]+(-0.0242027062166)*x[2]
        arg[3,1,0,1]=(0.184318292766)*x[0]+(1.08367911805)*x[1]+(-0.164531741594)*x[2]
        arg[3,1,1,0]=(-1.00967115996)*x[0]+(0.542699640149)*x[1]+(0.707616701259)*x[2]
        arg[3,1,1,1]=(0.856948233811)*x[0]+(-1.2186550297)*x[1]+(-0.034268798159)*x[2]
        arg[3,1,2,0]=(0.259470684477)*x[0]+(-1.03586983732)*x[1]+(-0.157275979341)*x[2]
        arg[3,1,2,1]=(-0.161954890658)*x[0]+(1.65853209885)*x[1]+(0.381113456719)*x[2]
        arg[3,2,0,0]=(-0.437413004681)*x[0]+(-0.730340273929)*x[1]+(-0.687689272494)*x[2]
        arg[3,2,0,1]=(-0.336618164118)*x[0]+(-1.24388666895)*x[1]+(-0.2198689254)*x[2]
        arg[3,2,1,0]=(0.0872793087887)*x[0]+(-0.336554573215)*x[1]+(-0.0597803523262)*x[2]
        arg[3,2,1,1]=(0.192805432018)*x[0]+(0.0540495582064)*x[1]+(0.846658042071)*x[2]
        arg[3,2,2,0]=(0.00794192604414)*x[0]+(-0.33504559053)*x[1]+(0.615701149787)*x[2]
        arg[3,2,2,1]=(0.210870961503)*x[0]+(1.14164095846)*x[1]+(-0.192785742791)*x[2]
        arg[3,3,0,0]=(-0.25895162041)*x[0]+(-0.225800436488)*x[1]+(-0.198007079838)*x[2]
        arg[3,3,0,1]=(-0.150363703231)*x[0]+(1.23870429535)*x[1]+(0.28635881247)*x[2]
        arg[3,3,1,0]=(-0.865980718216)*x[0]+(-0.0849853244335)*x[1]+(-0.292164217189)*x[2]
        arg[3,3,1,1]=(-0.350465968722)*x[0]+(0.09853736936)*x[1]+(-0.476711524665)*x[2]
        arg[3,3,2,0]=(0.311396300263)*x[0]+(-0.92437612034)*x[1]+(-0.757139962928)*x[2]
        arg[3,3,2,1]=(-0.912311919837)*x[0]+(0.267793631099)*x[1]+(0.4917345401)*x[2]
        arg[3,4,0,0]=(0.613662541753)*x[0]+(0.782757341361)*x[1]+(1.04209808158)*x[2]
        arg[3,4,0,1]=(0.492828613997)*x[0]+(1.11979095664)*x[1]+(-0.843545072354)*x[2]
        arg[3,4,1,0]=(-0.51939120556)*x[0]+(0.476004484286)*x[1]+(-0.516311785432)*x[2]
        arg[3,4,1,1]=(1.00423313567)*x[0]+(0.90776890388)*x[1]+(1.34423309497)*x[2]
        arg[3,4,2,0]=(-1.44069600826)*x[0]+(-1.39515290909)*x[1]+(1.19851590247)*x[2]
        arg[3,4,2,1]=(0.391851619877)*x[0]+(0.265529120553)*x[1]+(0.996610410913)*x[2]
        ref[0,0,0,0]=(1.19330263935)/(o+1.)+(0.55394275251)+(0.0162017895041)*0.5**o
        ref[0,0,0,1]=(-0.672351783684)/(o+1.)+(-0.187889622039)+(0.351132807711)*0.5**o
        ref[0,0,1,0]=(-1.27021996264)/(o+1.)+(0.236286530101)+(-0.961885459894)*0.5**o
        ref[0,0,1,1]=(-1.43442517107)/(o+1.)+(-1.00113374859)+(-0.935971122022)*0.5**o
        ref[0,0,2,0]=(-0.434813695127)/(o+1.)+(0.606673209583)+(-0.0699546855712)*0.5**o
        ref[0,0,2,1]=(-0.0434901247948)/(o+1.)+(-0.180911877983)+(-0.733211446073)*0.5**o
        ref[0,1,0,0]=(1.41514302182)/(o+1.)+(-0.247977413325)+(-0.967062307989)*0.5**o
        ref[0,1,0,1]=(0.69128100142)/(o+1.)+(-0.621609882923)+(-0.319976390946)*0.5**o
        ref[0,1,1,0]=(-0.881870429191)/(o+1.)+(-0.654075774082)+(0.0281262553057)*0.5**o
        ref[0,1,1,1]=(-1.27239116086)/(o+1.)+(-0.0854066429492)+(0.0230865806541)*0.5**o
        ref[0,1,2,0]=(-0.222663945351)/(o+1.)+(0.685312598747)+(0.453385341573)*0.5**o
        ref[0,1,2,1]=(0.101409038706)/(o+1.)+(0.151367946744)+(-0.824910502191)*0.5**o
        ref[0,2,0,0]=(-1.19509443892)/(o+1.)+(0.225437856866)+(0.85928567141)*0.5**o
        ref[0,2,0,1]=(0.285764347754)/(o+1.)+(-0.383432127133)+(0.963095541446)*0.5**o
        ref[0,2,1,0]=(0.728010227911)/(o+1.)+(0.983790919896)+(-0.550403317717)*0.5**o
        ref[0,2,1,1]=(0.563748259036)/(o+1.)+(0.165164845927)+(-0.110710881481)*0.5**o
        ref[0,2,2,0]=(-0.361502606429)/(o+1.)+(0.0276145482611)+(-0.341115463028)*0.5**o
        ref[0,2,2,1]=(-0.293320207933)/(o+1.)+(0.0336757304385)+(-0.853103403872)*0.5**o
        ref[0,3,0,0]=(-0.410233956527)/(o+1.)+(-0.474281126245)+(0.569057885785)*0.5**o
        ref[0,3,0,1]=(-0.835861970115)/(o+1.)+(0.0173715483175)+(0.346902459481)*0.5**o
        ref[0,3,1,0]=(0.0830466456187)/(o+1.)+(-0.288204681495)+(-0.607109864241)*0.5**o
        ref[0,3,1,1]=(1.07074130823)/(o+1.)+(-0.0437983012867)+(0.159379549652)*0.5**o
        ref[0,3,2,0]=(0.172301049609)/(o+1.)+(-0.491724874152)+(-0.996925180895)*0.5**o
        ref[0,3,2,1]=(-0.24891941221)/(o+1.)+(-0.03935856269)+(-0.3030090126)*0.5**o
        ref[0,4,0,0]=(-0.778575122429)/(o+1.)+(0.213085222078)+(0.941383668349)*0.5**o
        ref[0,4,0,1]=(0.880801627174)/(o+1.)+(-0.454451723401)+(-0.193978040747)*0.5**o
        ref[0,4,1,0]=(-0.835604724882)/(o+1.)+(0.127240834083)+(-0.245156740036)*0.5**o
        ref[0,4,1,1]=(1.62336716324)/(o+1.)+(-0.101085918464)+(0.398884015839)*0.5**o
        ref[0,4,2,0]=(1.16805827702)/(o+1.)+(0.211232591688)+(-0.654729385702)*0.5**o
        ref[0,4,2,1]=(-0.305424138957)/(o+1.)+(-0.431076572693)+(-0.924903192969)*0.5**o
        ref[1,0,0,0]=(-0.481596247529)/(o+1.)+(0.769902632559)+(0.748050413171)*0.5**o
        ref[1,0,0,1]=(0.177388661252)/(o+1.)+(-0.118766442613)+(-0.457657277952)*0.5**o
        ref[1,0,1,0]=(0.709890860372)/(o+1.)+(0.0668010230727)+(0.658744950012)*0.5**o
        ref[1,0,1,1]=(-1.45685781381)/(o+1.)+(-0.178674426089)+(0.788456069079)*0.5**o
        ref[1,0,2,0]=(0.194388185612)/(o+1.)+(0.649177543063)+(0.896093783855)*0.5**o
        ref[1,0,2,1]=(0.181561279507)/(o+1.)+(-0.200618291902)+(-0.392507985678)*0.5**o
        ref[1,1,0,0]=(-0.884376814027)/(o+1.)+(0.280382736159)+(0.593213920451)*0.5**o
        ref[1,1,0,1]=(-0.250709602218)/(o+1.)+(-0.492104211211)+(0.818920586956)*0.5**o
        ref[1,1,1,0]=(0.0529723694625)/(o+1.)+(-0.502047650638)+(-0.115739819752)*0.5**o
        ref[1,1,1,1]=(0.215812202957)/(o+1.)+(-0.308796450292)+(0.954456105171)*0.5**o
        ref[1,1,2,0]=(-0.991481050902)/(o+1.)+(0.162626252838)+(-0.800147845717)*0.5**o
        ref[1,1,2,1]=(-0.433989756951)/(o+1.)+(0.362776336712)+(-0.259441100777)*0.5**o
        ref[1,2,0,0]=(-0.930287052624)/(o+1.)+(-0.347752763108)+(0.880525373413)*0.5**o
        ref[1,2,0,1]=(0.698674852861)/(o+1.)+(-0.15119613196)+(0.719004238051)*0.5**o
        ref[1,2,1,0]=(-0.777216821365)/(o+1.)+(0.436336076297)+(0.617741643399)*0.5**o
        ref[1,2,1,1]=(1.09595223135)/(o+1.)+(0.978251130983)+(0.804336219412)*0.5**o
        ref[1,2,2,0]=(-0.651012630849)/(o+1.)+(-1.04894606664)+(0.43528198181)*0.5**o
        ref[1,2,2,1]=(-1.5227353883)/(o+1.)+(0.670019307557)+(-0.745922598001)*0.5**o
        ref[1,3,0,0]=(-0.750195637997)/(o+1.)+(0.231864245348)+(-0.118356142533)*0.5**o
        ref[1,3,0,1]=(1.17136226596)/(o+1.)+(-0.114976421389)+(0.395383920468)*0.5**o
        ref[1,3,1,0]=(-0.108070636769)/(o+1.)+(0.279145429234)+(-0.352562621327)*0.5**o
        ref[1,3,1,1]=(-0.144223481714)/(o+1.)+(-0.472954352493)+(-0.0903026345013)*0.5**o
        ref[1,3,2,0]=(-0.487564307786)/(o+1.)+(0.636973351449)+(0.758622537094)*0.5**o
        ref[1,3,2,1]=(0.390860965954)/(o+1.)+(-0.36647678165)+(-0.66980953377)*0.5**o
        ref[1,4,0,0]=(-1.04824984837)/(o+1.)+(0.213932231728)+(0.823722790054)*0.5**o
        ref[1,4,0,1]=(-0.0984653601336)/(o+1.)+(0.07867877705)+(-0.741527483773)*0.5**o
        ref[1,4,1,0]=(-0.146287867383)/(o+1.)+(-0.908951868158)+(0.425691754878)*0.5**o
        ref[1,4,1,1]=(-0.94160416554)/(o+1.)+(0.0459877010016)+(-0.869872381431)*0.5**o
        ref[1,4,2,0]=(0.0115742858952)/(o+1.)+(-0.222488399041)+(0.687392452389)*0.5**o
        ref[1,4,2,1]=(-0.956561936575)/(o+1.)+(-0.59770879042)+(-0.0613652834159)*0.5**o
        ref[2,0,0,0]=(1.83987381729)/(o+1.)+(0.956579317963)+(0.862634531274)*0.5**o
        ref[2,0,0,1]=(0.105446127421)/(o+1.)+(0.180661881191)+(0.186629121375)*0.5**o
        ref[2,0,1,0]=(0.0212167440766)/(o+1.)+(-0.337825588888)+(-0.318943328859)*0.5**o
        ref[2,0,1,1]=(0.744834715631)/(o+1.)+(-0.350058416663)+(0.391466726104)*0.5**o
        ref[2,0,2,0]=(-0.788776613856)/(o+1.)+(0.233776806014)+(0.400043748043)*0.5**o
        ref[2,0,2,1]=(-0.95346976307)/(o+1.)+(0.0274998652324)+(-0.967971238003)*0.5**o
        ref[2,1,0,0]=(-0.536051408942)/(o+1.)+(-0.208498016994)+(0.590315312233)*0.5**o
        ref[2,1,0,1]=(-1.04158508788)/(o+1.)+(-0.397885793101)+(0.0127762955396)*0.5**o
        ref[2,1,1,0]=(0.529914207253)/(o+1.)+(-0.547503443987)+(-0.338873793431)*0.5**o
        ref[2,1,1,1]=(0.62298670274)/(o+1.)+(0.449860048786)+(0.352800726478)*0.5**o
        ref[2,1,2,0]=(0.965292650595)/(o+1.)+(0.00980498639)+(-0.261085873742)*0.5**o
        ref[2,1,2,1]=(0.403285360803)/(o+1.)+(0.369160592269)+(0.315744874774)*0.5**o
        ref[2,2,0,0]=(-0.457764884724)/(o+1.)+(0.563657776706)+(-0.791018976541)*0.5**o
        ref[2,2,0,1]=(0.188151274044)/(o+1.)+(-0.476470545911)+(0.911736564588)*0.5**o
        ref[2,2,1,0]=(-0.558515085739)/(o+1.)+(-0.445583796553)+(-0.533721159994)*0.5**o
        ref[2,2,1,1]=(-0.431992337265)/(o+1.)+(-0.0957726221961)+(0.64441532924)*0.5**o
        ref[2,2,2,0]=(-0.159056461947)/(o+1.)+(-0.0326864536156)+(0.484856087693)*0.5**o
        ref[2,2,2,1]=(0.533116456037)/(o+1.)+(1.10197257725)+(0.0353311991352)*0.5**o
        ref[2,3,0,0]=(0.0570625147023)/(o+1.)+(0.74170779854)+(0.851019670777)*0.5**o
        ref[2,3,0,1]=(-0.495059904832)/(o+1.)+(-0.169954912181)+(-0.761839138623)*0.5**o
        ref[2,3,1,0]=(0.482375173314)/(o+1.)+(-0.19374838651)+(0.706371811996)*0.5**o
        ref[2,3,1,1]=(0.12928100721)/(o+1.)+(-0.317327862062)+(-0.460946532077)*0.5**o
        ref[2,3,2,0]=(-0.234362084763)/(o+1.)+(0.423486279181)+(-0.93141377227)*0.5**o
        ref[2,3,2,1]=(1.25586693002)/(o+1.)+(-0.127551329993)+(0.56508002092)*0.5**o
        ref[2,4,0,0]=(0.45235297749)/(o+1.)+(-0.0217537364788)+(-0.640413752615)*0.5**o
        ref[2,4,0,1]=(0.936809207384)/(o+1.)+(-0.151519610695)+(-0.464139056331)*0.5**o
        ref[2,4,1,0]=(0.867693208863)/(o+1.)+(0.105683574314)+(-0.974513946233)*0.5**o
        ref[2,4,1,1]=(-1.27821575101)/(o+1.)+(0.463163847311)+(-0.0852880510814)*0.5**o
        ref[2,4,2,0]=(-0.846224501366)/(o+1.)+(0.432619461409)+(0.492845548002)*0.5**o
        ref[2,4,2,1]=(-1.48923229995)/(o+1.)+(-0.505747396375)+(0.00916577472118)*0.5**o
        ref[3,0,0,0]=(-0.959215731121)/(o+1.)+(-0.904609108587)+(-0.178668510268)*0.5**o
        ref[3,0,0,1]=(-1.53181231865)/(o+1.)+(-0.0758543610408)+(0.878717376536)*0.5**o
        ref[3,0,1,0]=(1.18643238831)/(o+1.)+(-0.288825159097)+(-0.341112240015)*0.5**o
        ref[3,0,1,1]=(-0.0675694917031)/(o+1.)+(-0.168955703616)+(0.275309686724)*0.5**o
        ref[3,0,2,0]=(-0.663269860133)/(o+1.)+(-0.235102421885)+(-0.573480403148)*0.5**o
        ref[3,0,2,1]=(1.0969289778)/(o+1.)+(0.46603172821)+(0.267894209932)*0.5**o
        ref[3,1,0,0]=(-0.716870513559)/(o+1.)+(0.712039409283)+(0.483098591635)*0.5**o
        ref[3,1,0,1]=(0.0328531999411)/(o+1.)+(0.379319450988)+(0.311973567308)*0.5**o
        ref[3,1,1,0]=(1.08648208064)/(o+1.)+(-0.196831959338)+(-0.452172980519)*0.5**o
        ref[3,1,1,1]=(0.0813181502612)/(o+1.)+(-0.227364064527)+(-0.0225656152548)*0.5**o
        ref[3,1,2,0]=(-0.271000454408)/(o+1.)+(-0.765711687155)+(0.868748696531)*0.5**o
        ref[3,1,2,1]=(1.60569646253)/(o+1.)+(0.137437739736)+(-0.00288127709197)*0.5**o
        ref[3,2,0,0]=(-1.25887089059)/(o+1.)+(-0.525842290945)+(0.455112921376)*0.5**o
        ref[3,2,0,1]=(-1.63787880725)/(o+1.)+(0.00214706725405)+(-0.166789085722)*0.5**o
        ref[3,2,1,0]=(-1.82111307894)/(o+1.)+(0.561806270366)+(0.388444921454)*0.5**o
        ref[3,2,1,1]=(-0.489259906414)/(o+1.)+(0.582548398381)+(0.417676141947)*0.5**o
        ref[3,2,2,0]=(-0.0674375340391)/(o+1.)+(-0.00345549227349)+(0.362946003887)*0.5**o
        ref[3,2,2,1]=(0.662780682001)/(o+1.)+(-0.15453848218)+(0.806022459534)*0.5**o
        ref[3,3,0,0]=(-1.15795406515)/(o+1.)+(0.0926725746467)+(0.289849779118)*0.5**o
        ref[3,3,0,1]=(1.27520269547)/(o+1.)+(-0.286529818361)+(0.672556345847)*0.5**o
        ref[3,3,1,0]=(-0.343290651469)/(o+1.)+(-0.0269962100904)+(-0.845847188189)*0.5**o
        ref[3,3,1,1]=(0.497715053371)/(o+1.)+(-0.158577177766)+(-0.909200821866)*0.5**o
        ref[3,3,2,0]=(-0.952431538662)/(o+1.)+(0.0816482063045)+(-0.580984656952)*0.5**o
        ref[3,3,2,1]=(-0.176227469799)/(o+1.)+(0.0456157853688)+(-0.0677878495767)*0.5**o
        ref[3,4,0,0]=(0.976460483802)/(o+1.)+(0.347595506011)+(0.766866468871)*0.5**o
        ref[3,4,0,1]=(0.0358170327662)/(o+1.)+(-0.0991456259778)+(0.931548717473)*0.5**o
        ref[3,4,1,0]=(0.114544318514)/(o+1.)+(-0.201014865792)+(-0.272213093635)*0.5**o
        ref[3,4,1,1]=(0.554509925385)/(o+1.)+(1.23219332557)+(0.237338557994)*0.5**o
        ref[3,4,2,0]=(0.0803465444931)/(o+1.)+(-0.518257569997)+(-0.681164419371)*0.5**o
        ref[3,4,2,1]=(-0.0443609570522)/(o+1.)+(0.572813368096)+(0.552725372202)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactZero_fromData_ReducedFunctionOnContactZero_rank0(self):
      """
      tests integral of rank 0 Data on the ReducedFunctionOnContactZero

      assumptions: ReducedFunctionOnContactZero(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      w=ReducedFunctionOnContactZero(self.domain)
      x=w.getX()
      arg=Data(0,(),w)
      if dim==2:
        arg=(-0.114918917602)*x[0]**o+(0.304834778607)*x[0]+(0.704057176139)*x[1]**o+(-0.0664624375434)*x[1]
        ref=(0.704057176139)/(o+1.)+(0.119186170532)+(-0.114918917602)*0.5**o
      else:
        arg=(-0.134961573162)*x[0]**o+(0.585433861814)*x[0]+(0.209596676649)*x[1]**o+(0.870867416311)*x[1]+(-0.902854033973)*x[2]**o+(-0.812135783993)*x[2]
        ref=(-0.693257357324)/(o+1.)+(0.322082747066)+(-0.134961573162)*0.5**o
      res=integrate(arg)
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactZero_fromData_ReducedFunctionOnContactZero_rank1(self):
      """
      tests integral of rank 1 Data on the ReducedFunctionOnContactZero

      assumptions: ReducedFunctionOnContactZero(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      w=ReducedFunctionOnContactZero(self.domain)
      x=w.getX()
      arg=Data(0,(2,),w)
      ref=numpy.zeros((2,),numpy.float_)
      if dim==2:
        arg[0]=(0.441677901994)*x[0]**o+(0.0234076994082)*x[0]+(-0.520465368981)*x[1]**o+(-0.668717137452)*x[1]
        arg[1]=(0.0637451453664)*x[0]**o+(0.610543665309)*x[0]+(0.639008465664)*x[1]**o+(-0.460500756968)*x[1]
        ref[0]=(-0.520465368981)/(o+1.)+(-0.322654719022)+(0.441677901994)*0.5**o
        ref[1]=(0.639008465664)/(o+1.)+(0.0750214541708)+(0.0637451453664)*0.5**o
      else:
        arg[0]=(0.207042421638)*x[0]**o+(0.586067682859)*x[0]+(-0.731189752251)*x[1]**o+(0.439755444977)*x[1]+(-0.215027146515)*x[2]**o+(-0.22109699198)*x[2]
        arg[1]=(-0.482897746923)*x[0]**o+(-0.262848758384)*x[0]+(-0.130231246709)*x[1]**o+(0.494143899685)*x[1]+(0.871226664446)*x[2]**o+(-0.45136577521)*x[2]
        ref[0]=(-0.946216898767)/(o+1.)+(0.402363067928)+(0.207042421638)*0.5**o
        ref[1]=(0.740995417737)/(o+1.)+(-0.110035316955)+(-0.482897746923)*0.5**o
      res=integrate(arg)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactZero_fromData_ReducedFunctionOnContactZero_rank2(self):
      """
      tests integral of rank 2 Data on the ReducedFunctionOnContactZero

      assumptions: ReducedFunctionOnContactZero(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      w=ReducedFunctionOnContactZero(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5),w)
      ref=numpy.zeros((4, 5),numpy.float_)
      if dim==2:
        arg[0,0]=(0.418217706031)*x[0]**o+(0.817101274679)*x[0]+(0.359227724634)*x[1]**o+(0.106952446872)*x[1]
        arg[0,1]=(0.735477258623)*x[0]**o+(-0.181692404492)*x[0]+(0.118248218763)*x[1]**o+(0.910033225874)*x[1]
        arg[0,2]=(-0.370301174151)*x[0]**o+(0.421589410528)*x[0]+(-0.91974297037)*x[1]**o+(0.676441845707)*x[1]
        arg[0,3]=(0.170535646217)*x[0]**o+(0.650458506701)*x[0]+(-0.431898202846)*x[1]**o+(-0.89133664224)*x[1]
        arg[0,4]=(-0.288793192482)*x[0]**o+(-0.752066662422)*x[0]+(0.279565184651)*x[1]**o+(-0.20569858505)*x[1]
        arg[1,0]=(-0.831478902729)*x[0]**o+(-0.545347589281)*x[0]+(-0.315738709444)*x[1]**o+(-0.407156023524)*x[1]
        arg[1,1]=(0.119396512736)*x[0]**o+(0.56655186098)*x[0]+(0.798206239231)*x[1]**o+(0.0342467197178)*x[1]
        arg[1,2]=(-0.264721761214)*x[0]**o+(-0.7583989175)*x[0]+(-0.350411882147)*x[1]**o+(-0.556879396863)*x[1]
        arg[1,3]=(0.976029513102)*x[0]**o+(-0.117830794452)*x[0]+(0.318067419468)*x[1]**o+(-0.496129688948)*x[1]
        arg[1,4]=(0.462797018845)*x[0]**o+(-0.214502973992)*x[0]+(0.155052684044)*x[1]**o+(0.264794267973)*x[1]
        arg[2,0]=(0.193892132778)*x[0]**o+(-0.440406597181)*x[0]+(0.527330868842)*x[1]**o+(0.856602716155)*x[1]
        arg[2,1]=(0.0418968976051)*x[0]**o+(0.944283628708)*x[0]+(0.923548919008)*x[1]**o+(0.958011594347)*x[1]
        arg[2,2]=(0.744026460715)*x[0]**o+(0.238896690247)*x[0]+(0.968891312128)*x[1]**o+(0.0636831438113)*x[1]
        arg[2,3]=(-0.289048983574)*x[0]**o+(0.897469203226)*x[0]+(0.948131139828)*x[1]**o+(0.821749060686)*x[1]
        arg[2,4]=(-0.903766144138)*x[0]**o+(-0.453139974489)*x[0]+(-0.715818671181)*x[1]**o+(-0.712367492971)*x[1]
        arg[3,0]=(0.315277263598)*x[0]**o+(-0.676815102073)*x[0]+(0.428704415911)*x[1]**o+(0.54703485039)*x[1]
        arg[3,1]=(-0.301882901745)*x[0]**o+(0.888822293672)*x[0]+(-0.496204235393)*x[1]**o+(0.210598259748)*x[1]
        arg[3,2]=(0.568071178492)*x[0]**o+(-0.552863936415)*x[0]+(0.758619958752)*x[1]**o+(0.520428540448)*x[1]
        arg[3,3]=(0.477425351692)*x[0]**o+(-0.427405588029)*x[0]+(0.197959279245)*x[1]**o+(0.328047998649)*x[1]
        arg[3,4]=(-0.869704203328)*x[0]**o+(-0.0825164966999)*x[0]+(0.779325592334)*x[1]**o+(0.419916607608)*x[1]
        ref[0,0]=(0.359227724634)/(o+1.)+(0.462026860776)+(0.418217706031)*0.5**o
        ref[0,1]=(0.118248218763)/(o+1.)+(0.364170410691)+(0.735477258623)*0.5**o
        ref[0,2]=(-0.91974297037)/(o+1.)+(0.549015628118)+(-0.370301174151)*0.5**o
        ref[0,3]=(-0.431898202846)/(o+1.)+(-0.120439067769)+(0.170535646217)*0.5**o
        ref[0,4]=(0.279565184651)/(o+1.)+(-0.478882623736)+(-0.288793192482)*0.5**o
        ref[1,0]=(-0.315738709444)/(o+1.)+(-0.476251806402)+(-0.831478902729)*0.5**o
        ref[1,1]=(0.798206239231)/(o+1.)+(0.300399290349)+(0.119396512736)*0.5**o
        ref[1,2]=(-0.350411882147)/(o+1.)+(-0.657639157182)+(-0.264721761214)*0.5**o
        ref[1,3]=(0.318067419468)/(o+1.)+(-0.3069802417)+(0.976029513102)*0.5**o
        ref[1,4]=(0.155052684044)/(o+1.)+(0.02514564699)+(0.462797018845)*0.5**o
        ref[2,0]=(0.527330868842)/(o+1.)+(0.208098059487)+(0.193892132778)*0.5**o
        ref[2,1]=(0.923548919008)/(o+1.)+(0.951147611528)+(0.0418968976051)*0.5**o
        ref[2,2]=(0.968891312128)/(o+1.)+(0.151289917029)+(0.744026460715)*0.5**o
        ref[2,3]=(0.948131139828)/(o+1.)+(0.859609131956)+(-0.289048983574)*0.5**o
        ref[2,4]=(-0.715818671181)/(o+1.)+(-0.58275373373)+(-0.903766144138)*0.5**o
        ref[3,0]=(0.428704415911)/(o+1.)+(-0.0648901258416)+(0.315277263598)*0.5**o
        ref[3,1]=(-0.496204235393)/(o+1.)+(0.54971027671)+(-0.301882901745)*0.5**o
        ref[3,2]=(0.758619958752)/(o+1.)+(-0.0162176979831)+(0.568071178492)*0.5**o
        ref[3,3]=(0.197959279245)/(o+1.)+(-0.0496787946897)+(0.477425351692)*0.5**o
        ref[3,4]=(0.779325592334)/(o+1.)+(0.168700055454)+(-0.869704203328)*0.5**o
      else:
        arg[0,0]=(0.204452789147)*x[0]**o+(0.411843111144)*x[0]+(-0.0502866367412)*x[1]**o+(0.957356261522)*x[1]+(0.173522462943)*x[2]**o+(-0.0194908382525)*x[2]
        arg[0,1]=(-0.49641528149)*x[0]**o+(0.158625922243)*x[0]+(0.670932733725)*x[1]**o+(-0.611829064862)*x[1]+(0.758373881782)*x[2]**o+(0.151236368375)*x[2]
        arg[0,2]=(0.577504765611)*x[0]**o+(0.872996499874)*x[0]+(-0.734000014722)*x[1]**o+(-0.890824538872)*x[1]+(-0.796786080271)*x[2]**o+(-0.389005562901)*x[2]
        arg[0,3]=(-0.933166619447)*x[0]**o+(-0.528627639194)*x[0]+(-0.828111326322)*x[1]**o+(0.528345547301)*x[1]+(0.99894455165)*x[2]**o+(0.472982583662)*x[2]
        arg[0,4]=(-0.495503045054)*x[0]**o+(0.629909217292)*x[0]+(0.901515784751)*x[1]**o+(-0.863668288904)*x[1]+(0.76892679355)*x[2]**o+(0.34730747389)*x[2]
        arg[1,0]=(0.303189803506)*x[0]**o+(-0.17543401741)*x[0]+(-0.283157585975)*x[1]**o+(0.950171529024)*x[1]+(0.0483247572497)*x[2]**o+(-0.604332298954)*x[2]
        arg[1,1]=(-0.225480407643)*x[0]**o+(-0.0521067696425)*x[0]+(-0.10259234894)*x[1]**o+(0.373551917005)*x[1]+(0.89047195372)*x[2]**o+(-0.292740890417)*x[2]
        arg[1,2]=(-0.476041156832)*x[0]**o+(0.916078681628)*x[0]+(-0.393887610676)*x[1]**o+(-0.651852504525)*x[1]+(0.611404175606)*x[2]**o+(0.336594647903)*x[2]
        arg[1,3]=(-0.415178048882)*x[0]**o+(0.145394549559)*x[0]+(0.680115054186)*x[1]**o+(0.261517386893)*x[1]+(0.781566117396)*x[2]**o+(-0.659634539391)*x[2]
        arg[1,4]=(0.874821341857)*x[0]**o+(0.317418381126)*x[0]+(0.051455923305)*x[1]**o+(0.670170143071)*x[1]+(-0.211902435336)*x[2]**o+(-0.175299130233)*x[2]
        arg[2,0]=(-0.372961546517)*x[0]**o+(0.199493150648)*x[0]+(-0.630239499285)*x[1]**o+(0.657412283442)*x[1]+(-0.82211915324)*x[2]**o+(-0.468612977158)*x[2]
        arg[2,1]=(-0.503168527887)*x[0]**o+(0.904406483201)*x[0]+(-0.206179628633)*x[1]**o+(0.0787026584997)*x[1]+(0.899590466467)*x[2]**o+(-0.893401768413)*x[2]
        arg[2,2]=(-0.860556408954)*x[0]**o+(0.39021140432)*x[0]+(-0.66425383352)*x[1]**o+(0.287711780417)*x[1]+(0.776884760126)*x[2]**o+(-0.115905192334)*x[2]
        arg[2,3]=(0.461359695993)*x[0]**o+(0.997877835019)*x[0]+(0.0513514112685)*x[1]**o+(-0.904812988908)*x[1]+(-0.903222331838)*x[2]**o+(-0.470185679041)*x[2]
        arg[2,4]=(-0.646560730528)*x[0]**o+(0.10038324067)*x[0]+(0.426753245517)*x[1]**o+(-0.109889624344)*x[1]+(-0.20158193753)*x[2]**o+(0.014258531224)*x[2]
        arg[3,0]=(0.364407704271)*x[0]**o+(0.840519040103)*x[0]+(0.528721642544)*x[1]**o+(-0.853679853614)*x[1]+(0.079836122575)*x[2]**o+(-0.439039178308)*x[2]
        arg[3,1]=(-0.0726491927955)*x[0]**o+(0.217804431494)*x[0]+(-0.454354285545)*x[1]**o+(0.390539936003)*x[1]+(-0.989667912688)*x[2]**o+(-0.0674079606774)*x[2]
        arg[3,2]=(0.62239422825)*x[0]**o+(0.998733716844)*x[0]+(0.796258161536)*x[1]**o+(0.618545466026)*x[1]+(-0.243188896911)*x[2]**o+(-0.677687584224)*x[2]
        arg[3,3]=(0.444073180258)*x[0]**o+(-0.764729326792)*x[0]+(-0.823491453567)*x[1]**o+(0.985393963131)*x[1]+(0.623800470977)*x[2]**o+(0.947328422513)*x[2]
        arg[3,4]=(0.686438920325)*x[0]**o+(0.789760134176)*x[0]+(-0.26011082782)*x[1]**o+(0.0692377043339)*x[1]+(-0.3712894236)*x[2]**o+(-0.542974191103)*x[2]
        ref[0,0]=(0.123235826202)/(o+1.)+(0.674854267207)+(0.204452789147)*0.5**o
        ref[0,1]=(1.42930661551)/(o+1.)+(-0.150983387122)+(-0.49641528149)*0.5**o
        ref[0,2]=(-1.53078609499)/(o+1.)+(-0.203416800949)+(0.577504765611)*0.5**o
        ref[0,3]=(0.170833225328)/(o+1.)+(0.236350245885)+(-0.933166619447)*0.5**o
        ref[0,4]=(1.6704425783)/(o+1.)+(0.0567742011389)+(-0.495503045054)*0.5**o
        ref[1,0]=(-0.234832828725)/(o+1.)+(0.0852026063299)+(0.303189803506)*0.5**o
        ref[1,1]=(0.787879604779)/(o+1.)+(0.0143521284725)+(-0.225480407643)*0.5**o
        ref[1,2]=(0.21751656493)/(o+1.)+(0.300410412503)+(-0.476041156832)*0.5**o
        ref[1,3]=(1.46168117158)/(o+1.)+(-0.126361301469)+(-0.415178048882)*0.5**o
        ref[1,4]=(-0.160446512031)/(o+1.)+(0.406144696982)+(0.874821341857)*0.5**o
        ref[2,0]=(-1.45235865253)/(o+1.)+(0.194146228466)+(-0.372961546517)*0.5**o
        ref[2,1]=(0.693410837835)/(o+1.)+(0.0448536866438)+(-0.503168527887)*0.5**o
        ref[2,2]=(0.112630926606)/(o+1.)+(0.281008996202)+(-0.860556408954)*0.5**o
        ref[2,3]=(-0.85187092057)/(o+1.)+(-0.188560416464)+(0.461359695993)*0.5**o
        ref[2,4]=(0.225171307987)/(o+1.)+(0.00237607377473)+(-0.646560730528)*0.5**o
        ref[3,0]=(0.608557765119)/(o+1.)+(-0.22609999591)+(0.364407704271)*0.5**o
        ref[3,1]=(-1.44402219823)/(o+1.)+(0.27046820341)+(-0.0726491927955)*0.5**o
        ref[3,2]=(0.553069264625)/(o+1.)+(0.469795799323)+(0.62239422825)*0.5**o
        ref[3,3]=(-0.19969098259)/(o+1.)+(0.583996529426)+(0.444073180258)*0.5**o
        ref[3,4]=(-0.631400251421)/(o+1.)+(0.158011823704)+(0.686438920325)*0.5**o
      res=integrate(arg)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactZero_fromData_ReducedFunctionOnContactZero_rank3(self):
      """
      tests integral of rank 3 Data on the ReducedFunctionOnContactZero

      assumptions: ReducedFunctionOnContactZero(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      w=ReducedFunctionOnContactZero(self.domain)
      x=w.getX()
      arg=Data(0,(6, 2, 2),w)
      ref=numpy.zeros((6, 2, 2),numpy.float_)
      if dim==2:
        arg[0,0,0]=(-0.963014866001)*x[0]**o+(-0.592091539519)*x[0]+(0.397897510985)*x[1]**o+(0.485612885034)*x[1]
        arg[0,0,1]=(-0.764849027936)*x[0]**o+(0.936217923087)*x[0]+(-0.6256860991)*x[1]**o+(0.162359755936)*x[1]
        arg[0,1,0]=(-0.269607090458)*x[0]**o+(0.197100067821)*x[0]+(-0.216513043667)*x[1]**o+(0.143038940436)*x[1]
        arg[0,1,1]=(-0.432206914977)*x[0]**o+(0.177851208963)*x[0]+(0.0477652910843)*x[1]**o+(0.278333389583)*x[1]
        arg[1,0,0]=(-0.123940629628)*x[0]**o+(0.818553665062)*x[0]+(-0.168322144522)*x[1]**o+(0.485324994762)*x[1]
        arg[1,0,1]=(0.593609516922)*x[0]**o+(0.530508390905)*x[0]+(-0.938421636915)*x[1]**o+(-0.686459217917)*x[1]
        arg[1,1,0]=(-0.874022523162)*x[0]**o+(0.694500749056)*x[0]+(-0.650951516798)*x[1]**o+(0.45826621855)*x[1]
        arg[1,1,1]=(-0.259058106634)*x[0]**o+(0.0837282545694)*x[0]+(0.465734884341)*x[1]**o+(0.185558060279)*x[1]
        arg[2,0,0]=(0.346292866419)*x[0]**o+(0.982902010078)*x[0]+(0.0297606445334)*x[1]**o+(0.339043518987)*x[1]
        arg[2,0,1]=(0.749684718966)*x[0]**o+(-0.855799436867)*x[0]+(-0.186333913876)*x[1]**o+(-0.548968316245)*x[1]
        arg[2,1,0]=(-0.255759466052)*x[0]**o+(0.441882703647)*x[0]+(-0.0239807362322)*x[1]**o+(-0.236417762222)*x[1]
        arg[2,1,1]=(-0.0586694299212)*x[0]**o+(-0.912208422332)*x[0]+(-0.11610871626)*x[1]**o+(-0.512805786403)*x[1]
        arg[3,0,0]=(0.826027179564)*x[0]**o+(-0.589876767386)*x[0]+(-0.654858828463)*x[1]**o+(0.601656609023)*x[1]
        arg[3,0,1]=(-0.148714999203)*x[0]**o+(-0.409138795316)*x[0]+(0.290726299307)*x[1]**o+(-0.94009130345)*x[1]
        arg[3,1,0]=(0.63431415979)*x[0]**o+(-0.184553980231)*x[0]+(0.4087395755)*x[1]**o+(-0.694376406449)*x[1]
        arg[3,1,1]=(-0.000928514830108)*x[0]**o+(-0.782814501236)*x[0]+(0.177390244103)*x[1]**o+(0.698562703841)*x[1]
        arg[4,0,0]=(-0.939264907756)*x[0]**o+(0.191360117367)*x[0]+(-0.318750748728)*x[1]**o+(0.910713477162)*x[1]
        arg[4,0,1]=(-0.105186863713)*x[0]**o+(0.0319652211881)*x[0]+(0.118896230389)*x[1]**o+(-0.212677289177)*x[1]
        arg[4,1,0]=(-0.702838100428)*x[0]**o+(0.881695294345)*x[0]+(0.728231539571)*x[1]**o+(-0.192311604244)*x[1]
        arg[4,1,1]=(0.25197450886)*x[0]**o+(0.464857988125)*x[0]+(0.851996888259)*x[1]**o+(-0.408525083481)*x[1]
        arg[5,0,0]=(0.665627182446)*x[0]**o+(0.5367740452)*x[0]+(0.295800616406)*x[1]**o+(0.0620589956995)*x[1]
        arg[5,0,1]=(-0.37320168697)*x[0]**o+(0.072855481498)*x[0]+(0.539806161326)*x[1]**o+(-0.999091813027)*x[1]
        arg[5,1,0]=(-0.060845772709)*x[0]**o+(-0.931869215952)*x[0]+(-0.393477853475)*x[1]**o+(0.968288081996)*x[1]
        arg[5,1,1]=(-0.0768045781739)*x[0]**o+(-0.257318730887)*x[0]+(0.781473167319)*x[1]**o+(-0.458870794544)*x[1]
        ref[0,0,0]=(0.397897510985)/(o+1.)+(-0.0532393272425)+(-0.963014866001)*0.5**o
        ref[0,0,1]=(-0.6256860991)/(o+1.)+(0.549288839511)+(-0.764849027936)*0.5**o
        ref[0,1,0]=(-0.216513043667)/(o+1.)+(0.170069504128)+(-0.269607090458)*0.5**o
        ref[0,1,1]=(0.0477652910843)/(o+1.)+(0.228092299273)+(-0.432206914977)*0.5**o
        ref[1,0,0]=(-0.168322144522)/(o+1.)+(0.651939329912)+(-0.123940629628)*0.5**o
        ref[1,0,1]=(-0.938421636915)/(o+1.)+(-0.0779754135063)+(0.593609516922)*0.5**o
        ref[1,1,0]=(-0.650951516798)/(o+1.)+(0.576383483803)+(-0.874022523162)*0.5**o
        ref[1,1,1]=(0.465734884341)/(o+1.)+(0.134643157424)+(-0.259058106634)*0.5**o
        ref[2,0,0]=(0.0297606445334)/(o+1.)+(0.660972764533)+(0.346292866419)*0.5**o
        ref[2,0,1]=(-0.186333913876)/(o+1.)+(-0.702383876556)+(0.749684718966)*0.5**o
        ref[2,1,0]=(-0.0239807362322)/(o+1.)+(0.102732470713)+(-0.255759466052)*0.5**o
        ref[2,1,1]=(-0.11610871626)/(o+1.)+(-0.712507104367)+(-0.0586694299212)*0.5**o
        ref[3,0,0]=(-0.654858828463)/(o+1.)+(0.0058899208184)+(0.826027179564)*0.5**o
        ref[3,0,1]=(0.290726299307)/(o+1.)+(-0.674615049383)+(-0.148714999203)*0.5**o
        ref[3,1,0]=(0.4087395755)/(o+1.)+(-0.43946519334)+(0.63431415979)*0.5**o
        ref[3,1,1]=(0.177390244103)/(o+1.)+(-0.0421258986973)+(-0.000928514830108)*0.5**o
        ref[4,0,0]=(-0.318750748728)/(o+1.)+(0.551036797264)+(-0.939264907756)*0.5**o
        ref[4,0,1]=(0.118896230389)/(o+1.)+(-0.0903560339946)+(-0.105186863713)*0.5**o
        ref[4,1,0]=(0.728231539571)/(o+1.)+(0.344691845051)+(-0.702838100428)*0.5**o
        ref[4,1,1]=(0.851996888259)/(o+1.)+(0.0281664523221)+(0.25197450886)*0.5**o
        ref[5,0,0]=(0.295800616406)/(o+1.)+(0.29941652045)+(0.665627182446)*0.5**o
        ref[5,0,1]=(0.539806161326)/(o+1.)+(-0.463118165764)+(-0.37320168697)*0.5**o
        ref[5,1,0]=(-0.393477853475)/(o+1.)+(0.0182094330217)+(-0.060845772709)*0.5**o
        ref[5,1,1]=(0.781473167319)/(o+1.)+(-0.358094762716)+(-0.0768045781739)*0.5**o
      else:
        arg[0,0,0]=(0.638389367885)*x[0]**o+(-0.77703387527)*x[0]+(0.951496212227)*x[1]**o+(-0.484305156823)*x[1]+(-0.582437996275)*x[2]**o+(-0.460130065965)*x[2]
        arg[0,0,1]=(0.859011160063)*x[0]**o+(0.999372374168)*x[0]+(-0.946049724069)*x[1]**o+(0.522184535496)*x[1]+(0.00820405492191)*x[2]**o+(0.769305042211)*x[2]
        arg[0,1,0]=(-0.998681640062)*x[0]**o+(0.916063974594)*x[0]+(0.0801858148899)*x[1]**o+(0.475476401258)*x[1]+(-0.805247247843)*x[2]**o+(-0.809331278141)*x[2]
        arg[0,1,1]=(0.1340261052)*x[0]**o+(0.843204972218)*x[0]+(-0.809433597166)*x[1]**o+(-0.947798215446)*x[1]+(0.268796347717)*x[2]**o+(-0.896620567629)*x[2]
        arg[1,0,0]=(0.251201824008)*x[0]**o+(0.762438451805)*x[0]+(0.892722429031)*x[1]**o+(0.146944927124)*x[1]+(0.00953973207152)*x[2]**o+(0.423262196414)*x[2]
        arg[1,0,1]=(-0.258843593921)*x[0]**o+(-0.950574260099)*x[0]+(-0.49096256134)*x[1]**o+(0.446238902975)*x[1]+(-0.880706204083)*x[2]**o+(-0.400075002454)*x[2]
        arg[1,1,0]=(-0.854529145157)*x[0]**o+(0.118165533114)*x[0]+(-0.913218271084)*x[1]**o+(0.991977570414)*x[1]+(0.626241160161)*x[2]**o+(0.667843914102)*x[2]
        arg[1,1,1]=(0.636818650335)*x[0]**o+(-0.174996100829)*x[0]+(0.77317621688)*x[1]**o+(0.369280208456)*x[1]+(-0.0130124322187)*x[2]**o+(0.857098751007)*x[2]
        arg[2,0,0]=(-0.415847479141)*x[0]**o+(0.675986357495)*x[0]+(0.704926614111)*x[1]**o+(-0.166544295134)*x[1]+(-0.00917226654826)*x[2]**o+(-0.896432190739)*x[2]
        arg[2,0,1]=(-0.882383192316)*x[0]**o+(0.11122730635)*x[0]+(-0.693587089789)*x[1]**o+(-0.0905457091132)*x[1]+(-0.610526000157)*x[2]**o+(-0.992542379681)*x[2]
        arg[2,1,0]=(-0.476987072283)*x[0]**o+(-0.450296923812)*x[0]+(0.905270434608)*x[1]**o+(0.976270820516)*x[1]+(-0.233422717373)*x[2]**o+(-0.641253735386)*x[2]
        arg[2,1,1]=(0.873832514958)*x[0]**o+(-0.478059419217)*x[0]+(-0.941650455606)*x[1]**o+(0.307250152442)*x[1]+(0.865160620819)*x[2]**o+(0.961340749451)*x[2]
        arg[3,0,0]=(-0.443837945036)*x[0]**o+(-0.688773058425)*x[0]+(-0.654988618952)*x[1]**o+(-0.259511611372)*x[1]+(-0.931332326494)*x[2]**o+(0.939379631125)*x[2]
        arg[3,0,1]=(-0.181860312567)*x[0]**o+(-0.229066064127)*x[0]+(-0.359408642857)*x[1]**o+(-0.47945907585)*x[1]+(0.312345402782)*x[2]**o+(0.559298232285)*x[2]
        arg[3,1,0]=(-0.372379819015)*x[0]**o+(0.308179383085)*x[0]+(0.123613160006)*x[1]**o+(0.191944822984)*x[1]+(0.950869988808)*x[2]**o+(0.229360419984)*x[2]
        arg[3,1,1]=(-0.803126271269)*x[0]**o+(0.833324260042)*x[0]+(-0.917001746509)*x[1]**o+(-0.649776906898)*x[1]+(0.49624769362)*x[2]**o+(0.414080802594)*x[2]
        arg[4,0,0]=(-0.951747333891)*x[0]**o+(0.5919442178)*x[0]+(0.121310623341)*x[1]**o+(-0.321939845724)*x[1]+(0.358051737083)*x[2]**o+(-0.222801347833)*x[2]
        arg[4,0,1]=(-0.59462534105)*x[0]**o+(0.943860617698)*x[0]+(0.787822231831)*x[1]**o+(-0.0913435772425)*x[1]+(0.831651946729)*x[2]**o+(-0.903604465879)*x[2]
        arg[4,1,0]=(0.68655463207)*x[0]**o+(-0.98918689213)*x[0]+(0.251300089098)*x[1]**o+(-0.317828131349)*x[1]+(-0.105934216764)*x[2]**o+(-0.874506361945)*x[2]
        arg[4,1,1]=(-0.80544616293)*x[0]**o+(-0.321523684723)*x[0]+(0.874455533992)*x[1]**o+(0.200912811442)*x[1]+(-0.646776981862)*x[2]**o+(-0.988728118812)*x[2]
        arg[5,0,0]=(0.413387141328)*x[0]**o+(0.349210032211)*x[0]+(0.80870689028)*x[1]**o+(-0.633296311752)*x[1]+(0.24621029112)*x[2]**o+(0.853398479355)*x[2]
        arg[5,0,1]=(0.398787817372)*x[0]**o+(0.662875239272)*x[0]+(-0.719962467558)*x[1]**o+(0.49562860294)*x[1]+(-0.694859272211)*x[2]**o+(-0.792616661463)*x[2]
        arg[5,1,0]=(-0.298838035128)*x[0]**o+(0.121578680468)*x[0]+(-0.931062588633)*x[1]**o+(-0.0575832984853)*x[1]+(-0.696140085453)*x[2]**o+(0.836738246058)*x[2]
        arg[5,1,1]=(-0.949626960081)*x[0]**o+(-0.449843145228)*x[0]+(-0.304260073311)*x[1]**o+(-0.258880519466)*x[1]+(0.880466679751)*x[2]**o+(0.97505579364)*x[2]
        ref[0,0,0]=(0.369058215952)/(o+1.)+(-0.860734549029)+(0.638389367885)*0.5**o
        ref[0,0,1]=(-0.937845669147)/(o+1.)+(1.14543097594)+(0.859011160063)*0.5**o
        ref[0,1,0]=(-0.725061432954)/(o+1.)+(0.291104548855)+(-0.998681640062)*0.5**o
        ref[0,1,1]=(-0.540637249448)/(o+1.)+(-0.500606905428)+(0.1340261052)*0.5**o
        ref[1,0,0]=(0.902262161102)/(o+1.)+(0.666322787672)+(0.251201824008)*0.5**o
        ref[1,0,1]=(-1.37166876542)/(o+1.)+(-0.452205179789)+(-0.258843593921)*0.5**o
        ref[1,1,0]=(-0.286977110923)/(o+1.)+(0.888993508815)+(-0.854529145157)*0.5**o
        ref[1,1,1]=(0.760163784662)/(o+1.)+(0.525691429317)+(0.636818650335)*0.5**o
        ref[2,0,0]=(0.695754347563)/(o+1.)+(-0.193495064189)+(-0.415847479141)*0.5**o
        ref[2,0,1]=(-1.30411308995)/(o+1.)+(-0.485930391222)+(-0.882383192316)*0.5**o
        ref[2,1,0]=(0.671847717235)/(o+1.)+(-0.0576399193408)+(-0.476987072283)*0.5**o
        ref[2,1,1]=(-0.0764898347868)/(o+1.)+(0.395265741338)+(0.873832514958)*0.5**o
        ref[3,0,0]=(-1.58632094545)/(o+1.)+(-0.00445251933618)+(-0.443837945036)*0.5**o
        ref[3,0,1]=(-0.0470632400747)/(o+1.)+(-0.0746134538458)+(-0.181860312567)*0.5**o
        ref[3,1,0]=(1.07448314881)/(o+1.)+(0.364742313027)+(-0.372379819015)*0.5**o
        ref[3,1,1]=(-0.420754052889)/(o+1.)+(0.298814077869)+(-0.803126271269)*0.5**o
        ref[4,0,0]=(0.479362360424)/(o+1.)+(0.0236015121217)+(-0.951747333891)*0.5**o
        ref[4,0,1]=(1.61947417856)/(o+1.)+(-0.0255437127116)+(-0.59462534105)*0.5**o
        ref[4,1,0]=(0.145365872334)/(o+1.)+(-1.09076069271)+(0.68655463207)*0.5**o
        ref[4,1,1]=(0.22767855213)/(o+1.)+(-0.554669496046)+(-0.80544616293)*0.5**o
        ref[5,0,0]=(1.0549171814)/(o+1.)+(0.284656099907)+(0.413387141328)*0.5**o
        ref[5,0,1]=(-1.41482173977)/(o+1.)+(0.182943590374)+(0.398787817372)*0.5**o
        ref[5,1,0]=(-1.62720267409)/(o+1.)+(0.450366814021)+(-0.298838035128)*0.5**o
        ref[5,1,1]=(0.576206606439)/(o+1.)+(0.133166064473)+(-0.949626960081)*0.5**o
      res=integrate(arg)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactZero_fromData_ReducedFunctionOnContactZero_rank4(self):
      """
      tests integral of rank 4 Data on the ReducedFunctionOnContactZero

      assumptions: ReducedFunctionOnContactZero(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      w=ReducedFunctionOnContactZero(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5, 3, 2),w)
      ref=numpy.zeros((4, 5, 3, 2),numpy.float_)
      if dim==2:
        arg[0,0,0,0]=(0.392628958003)*x[0]**o+(0.186084293395)*x[0]+(0.965300679335)*x[1]**o+(0.0986282244747)*x[1]
        arg[0,0,0,1]=(-0.503312425492)*x[0]**o+(-0.205791466802)*x[0]+(0.384782163488)*x[1]**o+(-0.608371097114)*x[1]
        arg[0,0,1,0]=(0.660162511699)*x[0]**o+(-0.263141686561)*x[0]+(-0.765235012795)*x[1]**o+(-0.371233144611)*x[1]
        arg[0,0,1,1]=(-0.986935412732)*x[0]**o+(0.645433351743)*x[0]+(-0.451665346257)*x[1]**o+(-0.713815759872)*x[1]
        arg[0,0,2,0]=(0.632875178502)*x[0]**o+(-0.257542208821)*x[0]+(-0.695553902133)*x[1]**o+(-0.677920876645)*x[1]
        arg[0,0,2,1]=(0.102625648731)*x[0]**o+(0.511896902175)*x[0]+(0.345909088853)*x[1]**o+(-0.0609413722414)*x[1]
        arg[0,1,0,0]=(-0.0108106931425)*x[0]**o+(-0.595497517075)*x[0]+(0.332480141972)*x[1]**o+(-0.18131109924)*x[1]
        arg[0,1,0,1]=(0.220826593491)*x[0]**o+(-0.202422451222)*x[0]+(0.328480565578)*x[1]**o+(0.851432750557)*x[1]
        arg[0,1,1,0]=(0.22735402466)*x[0]**o+(0.51437836535)*x[0]+(0.292927451477)*x[1]**o+(-0.148121856023)*x[1]
        arg[0,1,1,1]=(0.495079875966)*x[0]**o+(0.582755117761)*x[0]+(-0.322061131008)*x[1]**o+(0.953437449986)*x[1]
        arg[0,1,2,0]=(0.968583889554)*x[0]**o+(-0.593253483731)*x[0]+(0.286037351205)*x[1]**o+(0.997314411263)*x[1]
        arg[0,1,2,1]=(0.361498191707)*x[0]**o+(0.633275777697)*x[0]+(0.891305263219)*x[1]**o+(-0.194468605638)*x[1]
        arg[0,2,0,0]=(0.185647566516)*x[0]**o+(0.101448861247)*x[0]+(0.0870608077911)*x[1]**o+(-0.86477908948)*x[1]
        arg[0,2,0,1]=(0.227262327826)*x[0]**o+(0.812586702875)*x[0]+(-0.0229924234496)*x[1]**o+(-0.323581477982)*x[1]
        arg[0,2,1,0]=(0.0405753322301)*x[0]**o+(0.891289727227)*x[0]+(0.957377329048)*x[1]**o+(-0.982119724379)*x[1]
        arg[0,2,1,1]=(-0.546046301701)*x[0]**o+(0.628511534929)*x[0]+(-0.628262263897)*x[1]**o+(-0.454315708999)*x[1]
        arg[0,2,2,0]=(-0.828455793175)*x[0]**o+(0.190147126828)*x[0]+(0.499595322279)*x[1]**o+(-0.0882297087937)*x[1]
        arg[0,2,2,1]=(0.926138085222)*x[0]**o+(0.402659001583)*x[0]+(-0.678432418122)*x[1]**o+(0.236054704896)*x[1]
        arg[0,3,0,0]=(-0.473066058658)*x[0]**o+(0.57622727181)*x[0]+(0.62665109546)*x[1]**o+(0.948409659533)*x[1]
        arg[0,3,0,1]=(0.776790978673)*x[0]**o+(-0.0244752180934)*x[0]+(-0.459252843816)*x[1]**o+(0.616203849528)*x[1]
        arg[0,3,1,0]=(0.34783214571)*x[0]**o+(0.806528120142)*x[0]+(0.445864469556)*x[1]**o+(-0.221961922099)*x[1]
        arg[0,3,1,1]=(0.0717966037463)*x[0]**o+(-0.434117388366)*x[0]+(0.577708844669)*x[1]**o+(-0.548519157145)*x[1]
        arg[0,3,2,0]=(-0.548573971799)*x[0]**o+(-0.926704595492)*x[0]+(-0.33936340707)*x[1]**o+(0.0687290934591)*x[1]
        arg[0,3,2,1]=(-0.28309240461)*x[0]**o+(0.12914675768)*x[0]+(-0.833323497891)*x[1]**o+(-0.577727985559)*x[1]
        arg[0,4,0,0]=(-0.105416428922)*x[0]**o+(0.41564239907)*x[0]+(0.146657173319)*x[1]**o+(-0.0141417271845)*x[1]
        arg[0,4,0,1]=(-0.876111094069)*x[0]**o+(0.491957669628)*x[0]+(0.534941970665)*x[1]**o+(-0.995507289274)*x[1]
        arg[0,4,1,0]=(-0.678996394303)*x[0]**o+(0.183159947613)*x[0]+(-0.739368558349)*x[1]**o+(0.274319401135)*x[1]
        arg[0,4,1,1]=(0.0472145581152)*x[0]**o+(-0.669737608207)*x[0]+(-0.651647023142)*x[1]**o+(0.351208329111)*x[1]
        arg[0,4,2,0]=(0.114936857967)*x[0]**o+(0.564688790898)*x[0]+(-0.524234284672)*x[1]**o+(-0.127615425724)*x[1]
        arg[0,4,2,1]=(0.568834117773)*x[0]**o+(-0.936513147599)*x[0]+(0.133122220811)*x[1]**o+(0.31510090512)*x[1]
        arg[1,0,0,0]=(-0.821241886235)*x[0]**o+(0.779734235239)*x[0]+(-0.132449009841)*x[1]**o+(-0.0179700618848)*x[1]
        arg[1,0,0,1]=(0.624372422235)*x[0]**o+(-0.752831611955)*x[0]+(-0.573597121344)*x[1]**o+(0.376463665377)*x[1]
        arg[1,0,1,0]=(-0.568749808355)*x[0]**o+(-0.697521100346)*x[0]+(-0.249786553024)*x[1]**o+(-0.624593568423)*x[1]
        arg[1,0,1,1]=(0.0765557360835)*x[0]**o+(-0.744005823966)*x[0]+(0.59246455477)*x[1]**o+(0.231310687774)*x[1]
        arg[1,0,2,0]=(0.909614864187)*x[0]**o+(0.350543076144)*x[0]+(0.0689432766231)*x[1]**o+(-0.88465698758)*x[1]
        arg[1,0,2,1]=(0.0610206203468)*x[0]**o+(0.620401328094)*x[0]+(0.378273922837)*x[1]**o+(0.91988744934)*x[1]
        arg[1,1,0,0]=(0.917249169051)*x[0]**o+(0.756656473206)*x[0]+(0.728786835113)*x[1]**o+(0.116994159781)*x[1]
        arg[1,1,0,1]=(0.331129708654)*x[0]**o+(-0.14342302905)*x[0]+(-0.418402258918)*x[1]**o+(-0.808676850609)*x[1]
        arg[1,1,1,0]=(-0.527408405279)*x[0]**o+(-0.671440239997)*x[0]+(0.989603613532)*x[1]**o+(0.364280392731)*x[1]
        arg[1,1,1,1]=(-0.189543145217)*x[0]**o+(0.14762392216)*x[0]+(0.11459047938)*x[1]**o+(-0.760477267267)*x[1]
        arg[1,1,2,0]=(0.385274935809)*x[0]**o+(0.527744200995)*x[0]+(-0.862654310338)*x[1]**o+(0.904005467598)*x[1]
        arg[1,1,2,1]=(-0.380421555378)*x[0]**o+(0.598300323394)*x[0]+(-0.541819816686)*x[1]**o+(0.446934572528)*x[1]
        arg[1,2,0,0]=(0.446781075767)*x[0]**o+(0.906830344178)*x[0]+(0.158734254777)*x[1]**o+(-0.722982586936)*x[1]
        arg[1,2,0,1]=(0.927811104869)*x[0]**o+(-0.446795135388)*x[0]+(-0.90301726295)*x[1]**o+(0.289530807979)*x[1]
        arg[1,2,1,0]=(0.741527109588)*x[0]**o+(-0.370029728344)*x[0]+(0.157695458386)*x[1]**o+(-0.620804465293)*x[1]
        arg[1,2,1,1]=(0.193242816889)*x[0]**o+(0.455789913444)*x[0]+(0.0491001091271)*x[1]**o+(-0.942127264151)*x[1]
        arg[1,2,2,0]=(0.220709813709)*x[0]**o+(-0.713472965425)*x[0]+(-0.736262418061)*x[1]**o+(0.341347605023)*x[1]
        arg[1,2,2,1]=(0.0034506429721)*x[0]**o+(0.397289753453)*x[0]+(-0.167013881353)*x[1]**o+(0.390364445974)*x[1]
        arg[1,3,0,0]=(0.321471168172)*x[0]**o+(-0.237896433618)*x[0]+(-0.199843421009)*x[1]**o+(0.811319091471)*x[1]
        arg[1,3,0,1]=(-0.0799622908037)*x[0]**o+(-0.66155883346)*x[0]+(0.0303112997369)*x[1]**o+(-0.199256497975)*x[1]
        arg[1,3,1,0]=(-0.12051943489)*x[0]**o+(0.674374603779)*x[0]+(0.335933608833)*x[1]**o+(0.366671959553)*x[1]
        arg[1,3,1,1]=(-0.587242845986)*x[0]**o+(-0.495652656487)*x[0]+(0.10609614346)*x[1]**o+(0.177210588935)*x[1]
        arg[1,3,2,0]=(0.947782336638)*x[0]**o+(-0.327283099142)*x[0]+(-0.0280648836948)*x[1]**o+(0.133334543825)*x[1]
        arg[1,3,2,1]=(-0.0887414134508)*x[0]**o+(0.640694455701)*x[0]+(0.266916366237)*x[1]**o+(0.345886606493)*x[1]
        arg[1,4,0,0]=(-0.138184872938)*x[0]**o+(0.658635897645)*x[0]+(-0.00526240442626)*x[1]**o+(0.800443113371)*x[1]
        arg[1,4,0,1]=(-0.549831917928)*x[0]**o+(-0.666733703837)*x[0]+(0.400982699808)*x[1]**o+(0.739099662817)*x[1]
        arg[1,4,1,0]=(-0.137046722563)*x[0]**o+(-0.818450480933)*x[0]+(-0.827665726293)*x[1]**o+(-0.286585607624)*x[1]
        arg[1,4,1,1]=(-0.474156202572)*x[0]**o+(-0.855381707793)*x[0]+(0.113318564035)*x[1]**o+(0.218410511108)*x[1]
        arg[1,4,2,0]=(0.18476112439)*x[0]**o+(-0.964533442641)*x[0]+(-0.968611931598)*x[1]**o+(0.0973026144203)*x[1]
        arg[1,4,2,1]=(-0.372995418552)*x[0]**o+(0.998322146553)*x[0]+(-0.384672070744)*x[1]**o+(0.47536977268)*x[1]
        arg[2,0,0,0]=(0.882400857222)*x[0]**o+(0.455324640128)*x[0]+(0.764297272234)*x[1]**o+(0.498669000791)*x[1]
        arg[2,0,0,1]=(0.894017590794)*x[0]**o+(0.217984119634)*x[0]+(-0.0934000967188)*x[1]**o+(-0.886960376026)*x[1]
        arg[2,0,1,0]=(-0.382158057826)*x[0]**o+(-0.897639065996)*x[0]+(0.819002513335)*x[1]**o+(-0.0739374877518)*x[1]
        arg[2,0,1,1]=(-0.259771272131)*x[0]**o+(-0.533540086977)*x[0]+(0.366917477923)*x[1]**o+(-0.751904525722)*x[1]
        arg[2,0,2,0]=(-0.33946164111)*x[0]**o+(-0.317441178182)*x[0]+(-0.857625945147)*x[1]**o+(-0.84647892719)*x[1]
        arg[2,0,2,1]=(0.153353093148)*x[0]**o+(0.832883645984)*x[0]+(-0.359476461654)*x[1]**o+(0.332390899654)*x[1]
        arg[2,1,0,0]=(0.780980225775)*x[0]**o+(0.619330950566)*x[0]+(0.194312746335)*x[1]**o+(0.595311976655)*x[1]
        arg[2,1,0,1]=(-0.334149973283)*x[0]**o+(-0.670178805858)*x[0]+(-0.313038325156)*x[1]**o+(0.793149618246)*x[1]
        arg[2,1,1,0]=(-0.832852528577)*x[0]**o+(-0.606634692926)*x[0]+(-0.0643816133251)*x[1]**o+(-0.663765987285)*x[1]
        arg[2,1,1,1]=(0.300975732963)*x[0]**o+(0.965431406497)*x[0]+(-0.618456119184)*x[1]**o+(0.459530421885)*x[1]
        arg[2,1,2,0]=(0.122326105776)*x[0]**o+(0.0940802696378)*x[0]+(-0.986852501828)*x[1]**o+(0.370428043236)*x[1]
        arg[2,1,2,1]=(0.447916985436)*x[0]**o+(-0.161926311978)*x[0]+(-0.0933563424166)*x[1]**o+(0.836290263421)*x[1]
        arg[2,2,0,0]=(0.032076347277)*x[0]**o+(-0.447517143643)*x[0]+(-0.053769157756)*x[1]**o+(-0.0242981207801)*x[1]
        arg[2,2,0,1]=(-0.403935703163)*x[0]**o+(-0.960750093184)*x[0]+(-0.287956277998)*x[1]**o+(-0.168614701082)*x[1]
        arg[2,2,1,0]=(-0.205077009492)*x[0]**o+(-0.409536289697)*x[0]+(0.359071620541)*x[1]**o+(-0.466455759025)*x[1]
        arg[2,2,1,1]=(0.443658057141)*x[0]**o+(-0.748681939527)*x[0]+(0.2153167916)*x[1]**o+(-0.475899799738)*x[1]
        arg[2,2,2,0]=(-0.463879681251)*x[0]**o+(0.758317222041)*x[0]+(0.608221916958)*x[1]**o+(-0.237798399085)*x[1]
        arg[2,2,2,1]=(0.533981754168)*x[0]**o+(0.590125912931)*x[0]+(0.8957267808)*x[1]**o+(0.294612478767)*x[1]
        arg[2,3,0,0]=(0.499387701403)*x[0]**o+(0.0379132293822)*x[0]+(0.717384215206)*x[1]**o+(0.754497698503)*x[1]
        arg[2,3,0,1]=(0.866500398882)*x[0]**o+(0.23678788012)*x[0]+(0.395132533716)*x[1]**o+(-0.451619930593)*x[1]
        arg[2,3,1,0]=(-0.879663354176)*x[0]**o+(-0.328937922978)*x[0]+(0.688771917581)*x[1]**o+(-0.87392051489)*x[1]
        arg[2,3,1,1]=(0.291214284356)*x[0]**o+(-0.783159133488)*x[0]+(0.240960084233)*x[1]**o+(0.339043900275)*x[1]
        arg[2,3,2,0]=(0.47064755192)*x[0]**o+(-0.42618815064)*x[0]+(-0.505342655776)*x[1]**o+(0.408218280069)*x[1]
        arg[2,3,2,1]=(0.775731405735)*x[0]**o+(-0.768750178591)*x[0]+(-0.896333275063)*x[1]**o+(0.286558848957)*x[1]
        arg[2,4,0,0]=(-0.7221248688)*x[0]**o+(0.612509823967)*x[0]+(0.436744528143)*x[1]**o+(-0.761502758764)*x[1]
        arg[2,4,0,1]=(-0.742652222385)*x[0]**o+(0.214811997988)*x[0]+(-0.031320962337)*x[1]**o+(-0.320134664789)*x[1]
        arg[2,4,1,0]=(-0.529575527815)*x[0]**o+(0.623770482091)*x[0]+(-0.50790148262)*x[1]**o+(0.629874274271)*x[1]
        arg[2,4,1,1]=(0.535631957883)*x[0]**o+(0.439578618993)*x[0]+(-0.770747718564)*x[1]**o+(0.0798696116058)*x[1]
        arg[2,4,2,0]=(0.81688202721)*x[0]**o+(-0.0301054197201)*x[0]+(0.613495031968)*x[1]**o+(-0.153619455295)*x[1]
        arg[2,4,2,1]=(0.493471708612)*x[0]**o+(0.423626757568)*x[0]+(0.093174141001)*x[1]**o+(0.307452108722)*x[1]
        arg[3,0,0,0]=(-0.0668630243284)*x[0]**o+(-0.577709692979)*x[0]+(0.984307685082)*x[1]**o+(-0.171487030152)*x[1]
        arg[3,0,0,1]=(-0.210452484293)*x[0]**o+(-0.130476372055)*x[0]+(-0.693596086089)*x[1]**o+(-0.692143724329)*x[1]
        arg[3,0,1,0]=(-0.34853285523)*x[0]**o+(-0.369649709966)*x[0]+(0.824648362105)*x[1]**o+(0.733046240399)*x[1]
        arg[3,0,1,1]=(0.861903045965)*x[0]**o+(0.609535692452)*x[0]+(0.681514895836)*x[1]**o+(0.66773275418)*x[1]
        arg[3,0,2,0]=(0.0197670342844)*x[0]**o+(-0.251645539228)*x[0]+(-0.0390293468405)*x[1]**o+(-0.00955605638373)*x[1]
        arg[3,0,2,1]=(-0.499093940395)*x[0]**o+(0.660356623112)*x[0]+(0.969057635866)*x[1]**o+(-0.821377317197)*x[1]
        arg[3,1,0,0]=(-0.955568800095)*x[0]**o+(0.557877698917)*x[0]+(0.829599262959)*x[1]**o+(0.933736802562)*x[1]
        arg[3,1,0,1]=(-0.910307638058)*x[0]**o+(-0.3287646946)*x[0]+(-0.777548579252)*x[1]**o+(-0.971744353969)*x[1]
        arg[3,1,1,0]=(0.22353820834)*x[0]**o+(0.904479243377)*x[0]+(-0.108768166204)*x[1]**o+(0.823767232517)*x[1]
        arg[3,1,1,1]=(0.371995768061)*x[0]**o+(0.747630377273)*x[0]+(0.931152588498)*x[1]**o+(-0.0154330268188)*x[1]
        arg[3,1,2,0]=(0.4567417369)*x[0]**o+(0.424951293892)*x[0]+(-0.14032259215)*x[1]**o+(0.603329614924)*x[1]
        arg[3,1,2,1]=(-0.855100213731)*x[0]**o+(-0.667832882495)*x[0]+(0.942530922378)*x[1]**o+(0.55861156259)*x[1]
        arg[3,2,0,0]=(0.17190936572)*x[0]**o+(-0.904098615762)*x[0]+(0.991497679624)*x[1]**o+(0.373020358381)*x[1]
        arg[3,2,0,1]=(-0.683587206983)*x[0]**o+(0.718257814718)*x[0]+(-0.730243933765)*x[1]**o+(-0.583892189979)*x[1]
        arg[3,2,1,0]=(0.051672736629)*x[0]**o+(-0.991172563807)*x[0]+(0.36605487466)*x[1]**o+(0.680719877794)*x[1]
        arg[3,2,1,1]=(0.438014662973)*x[0]**o+(0.172969450125)*x[0]+(-0.424515284177)*x[1]**o+(0.615315132547)*x[1]
        arg[3,2,2,0]=(-0.709499532674)*x[0]**o+(0.985097212621)*x[0]+(0.326610298255)*x[1]**o+(0.994701669258)*x[1]
        arg[3,2,2,1]=(-0.657647707844)*x[0]**o+(-0.683954483768)*x[0]+(-0.846657945161)*x[1]**o+(0.852308904063)*x[1]
        arg[3,3,0,0]=(0.21346320453)*x[0]**o+(-0.181665119768)*x[0]+(0.549866100727)*x[1]**o+(-0.0858045316871)*x[1]
        arg[3,3,0,1]=(0.249957470685)*x[0]**o+(0.537464773638)*x[0]+(0.0224330511636)*x[1]**o+(0.114623595638)*x[1]
        arg[3,3,1,0]=(0.166507029533)*x[0]**o+(0.232150195873)*x[0]+(0.119453667263)*x[1]**o+(-0.261110960642)*x[1]
        arg[3,3,1,1]=(0.358248720077)*x[0]**o+(-0.871911261059)*x[0]+(0.57743453064)*x[1]**o+(0.950322066335)*x[1]
        arg[3,3,2,0]=(-0.896899724447)*x[0]**o+(-0.898884767716)*x[0]+(0.63832012742)*x[1]**o+(-0.833800375634)*x[1]
        arg[3,3,2,1]=(0.425194254583)*x[0]**o+(0.161328467295)*x[0]+(0.496219405916)*x[1]**o+(0.380935896093)*x[1]
        arg[3,4,0,0]=(0.793902581592)*x[0]**o+(-0.33428511164)*x[0]+(-0.283510438468)*x[1]**o+(0.228913046074)*x[1]
        arg[3,4,0,1]=(0.372028980539)*x[0]**o+(-0.388909011671)*x[0]+(-0.259885631206)*x[1]**o+(-0.950910172897)*x[1]
        arg[3,4,1,0]=(-0.80489673786)*x[0]**o+(-0.828175420038)*x[0]+(0.895070489116)*x[1]**o+(-0.742272132103)*x[1]
        arg[3,4,1,1]=(0.272535126101)*x[0]**o+(0.902668974891)*x[0]+(0.399816667015)*x[1]**o+(-0.953186908789)*x[1]
        arg[3,4,2,0]=(0.0341775288582)*x[0]**o+(0.962241492639)*x[0]+(0.906020184876)*x[1]**o+(-0.400536255726)*x[1]
        arg[3,4,2,1]=(-0.745016464485)*x[0]**o+(-0.85241829747)*x[0]+(-0.648936544722)*x[1]**o+(-0.997030074218)*x[1]
        ref[0,0,0,0]=(0.965300679335)/(o+1.)+(0.142356258935)+(0.392628958003)*0.5**o
        ref[0,0,0,1]=(0.384782163488)/(o+1.)+(-0.407081281958)+(-0.503312425492)*0.5**o
        ref[0,0,1,0]=(-0.765235012795)/(o+1.)+(-0.317187415586)+(0.660162511699)*0.5**o
        ref[0,0,1,1]=(-0.451665346257)/(o+1.)+(-0.0341912040643)+(-0.986935412732)*0.5**o
        ref[0,0,2,0]=(-0.695553902133)/(o+1.)+(-0.467731542733)+(0.632875178502)*0.5**o
        ref[0,0,2,1]=(0.345909088853)/(o+1.)+(0.225477764967)+(0.102625648731)*0.5**o
        ref[0,1,0,0]=(0.332480141972)/(o+1.)+(-0.388404308158)+(-0.0108106931425)*0.5**o
        ref[0,1,0,1]=(0.328480565578)/(o+1.)+(0.324505149668)+(0.220826593491)*0.5**o
        ref[0,1,1,0]=(0.292927451477)/(o+1.)+(0.183128254664)+(0.22735402466)*0.5**o
        ref[0,1,1,1]=(-0.322061131008)/(o+1.)+(0.768096283874)+(0.495079875966)*0.5**o
        ref[0,1,2,0]=(0.286037351205)/(o+1.)+(0.202030463766)+(0.968583889554)*0.5**o
        ref[0,1,2,1]=(0.891305263219)/(o+1.)+(0.219403586029)+(0.361498191707)*0.5**o
        ref[0,2,0,0]=(0.0870608077911)/(o+1.)+(-0.381665114117)+(0.185647566516)*0.5**o
        ref[0,2,0,1]=(-0.0229924234496)/(o+1.)+(0.244502612446)+(0.227262327826)*0.5**o
        ref[0,2,1,0]=(0.957377329048)/(o+1.)+(-0.0454149985758)+(0.0405753322301)*0.5**o
        ref[0,2,1,1]=(-0.628262263897)/(o+1.)+(0.0870979129647)+(-0.546046301701)*0.5**o
        ref[0,2,2,0]=(0.499595322279)/(o+1.)+(0.0509587090171)+(-0.828455793175)*0.5**o
        ref[0,2,2,1]=(-0.678432418122)/(o+1.)+(0.319356853239)+(0.926138085222)*0.5**o
        ref[0,3,0,0]=(0.62665109546)/(o+1.)+(0.762318465671)+(-0.473066058658)*0.5**o
        ref[0,3,0,1]=(-0.459252843816)/(o+1.)+(0.295864315717)+(0.776790978673)*0.5**o
        ref[0,3,1,0]=(0.445864469556)/(o+1.)+(0.292283099022)+(0.34783214571)*0.5**o
        ref[0,3,1,1]=(0.577708844669)/(o+1.)+(-0.491318272755)+(0.0717966037463)*0.5**o
        ref[0,3,2,0]=(-0.33936340707)/(o+1.)+(-0.428987751016)+(-0.548573971799)*0.5**o
        ref[0,3,2,1]=(-0.833323497891)/(o+1.)+(-0.224290613939)+(-0.28309240461)*0.5**o
        ref[0,4,0,0]=(0.146657173319)/(o+1.)+(0.200750335943)+(-0.105416428922)*0.5**o
        ref[0,4,0,1]=(0.534941970665)/(o+1.)+(-0.251774809823)+(-0.876111094069)*0.5**o
        ref[0,4,1,0]=(-0.739368558349)/(o+1.)+(0.228739674374)+(-0.678996394303)*0.5**o
        ref[0,4,1,1]=(-0.651647023142)/(o+1.)+(-0.159264639548)+(0.0472145581152)*0.5**o
        ref[0,4,2,0]=(-0.524234284672)/(o+1.)+(0.218536682587)+(0.114936857967)*0.5**o
        ref[0,4,2,1]=(0.133122220811)/(o+1.)+(-0.31070612124)+(0.568834117773)*0.5**o
        ref[1,0,0,0]=(-0.132449009841)/(o+1.)+(0.380882086677)+(-0.821241886235)*0.5**o
        ref[1,0,0,1]=(-0.573597121344)/(o+1.)+(-0.188183973289)+(0.624372422235)*0.5**o
        ref[1,0,1,0]=(-0.249786553024)/(o+1.)+(-0.661057334385)+(-0.568749808355)*0.5**o
        ref[1,0,1,1]=(0.59246455477)/(o+1.)+(-0.256347568096)+(0.0765557360835)*0.5**o
        ref[1,0,2,0]=(0.0689432766231)/(o+1.)+(-0.267056955718)+(0.909614864187)*0.5**o
        ref[1,0,2,1]=(0.378273922837)/(o+1.)+(0.770144388717)+(0.0610206203468)*0.5**o
        ref[1,1,0,0]=(0.728786835113)/(o+1.)+(0.436825316493)+(0.917249169051)*0.5**o
        ref[1,1,0,1]=(-0.418402258918)/(o+1.)+(-0.47604993983)+(0.331129708654)*0.5**o
        ref[1,1,1,0]=(0.989603613532)/(o+1.)+(-0.153579923633)+(-0.527408405279)*0.5**o
        ref[1,1,1,1]=(0.11459047938)/(o+1.)+(-0.306426672553)+(-0.189543145217)*0.5**o
        ref[1,1,2,0]=(-0.862654310338)/(o+1.)+(0.715874834296)+(0.385274935809)*0.5**o
        ref[1,1,2,1]=(-0.541819816686)/(o+1.)+(0.522617447961)+(-0.380421555378)*0.5**o
        ref[1,2,0,0]=(0.158734254777)/(o+1.)+(0.0919238786215)+(0.446781075767)*0.5**o
        ref[1,2,0,1]=(-0.90301726295)/(o+1.)+(-0.0786321637044)+(0.927811104869)*0.5**o
        ref[1,2,1,0]=(0.157695458386)/(o+1.)+(-0.495417096819)+(0.741527109588)*0.5**o
        ref[1,2,1,1]=(0.0491001091271)/(o+1.)+(-0.243168675353)+(0.193242816889)*0.5**o
        ref[1,2,2,0]=(-0.736262418061)/(o+1.)+(-0.186062680201)+(0.220709813709)*0.5**o
        ref[1,2,2,1]=(-0.167013881353)/(o+1.)+(0.393827099713)+(0.0034506429721)*0.5**o
        ref[1,3,0,0]=(-0.199843421009)/(o+1.)+(0.286711328927)+(0.321471168172)*0.5**o
        ref[1,3,0,1]=(0.0303112997369)/(o+1.)+(-0.430407665717)+(-0.0799622908037)*0.5**o
        ref[1,3,1,0]=(0.335933608833)/(o+1.)+(0.520523281666)+(-0.12051943489)*0.5**o
        ref[1,3,1,1]=(0.10609614346)/(o+1.)+(-0.159221033776)+(-0.587242845986)*0.5**o
        ref[1,3,2,0]=(-0.0280648836948)/(o+1.)+(-0.0969742776586)+(0.947782336638)*0.5**o
        ref[1,3,2,1]=(0.266916366237)/(o+1.)+(0.493290531097)+(-0.0887414134508)*0.5**o
        ref[1,4,0,0]=(-0.00526240442626)/(o+1.)+(0.729539505508)+(-0.138184872938)*0.5**o
        ref[1,4,0,1]=(0.400982699808)/(o+1.)+(0.0361829794896)+(-0.549831917928)*0.5**o
        ref[1,4,1,0]=(-0.827665726293)/(o+1.)+(-0.552518044279)+(-0.137046722563)*0.5**o
        ref[1,4,1,1]=(0.113318564035)/(o+1.)+(-0.318485598343)+(-0.474156202572)*0.5**o
        ref[1,4,2,0]=(-0.968611931598)/(o+1.)+(-0.433615414111)+(0.18476112439)*0.5**o
        ref[1,4,2,1]=(-0.384672070744)/(o+1.)+(0.736845959617)+(-0.372995418552)*0.5**o
        ref[2,0,0,0]=(0.764297272234)/(o+1.)+(0.476996820459)+(0.882400857222)*0.5**o
        ref[2,0,0,1]=(-0.0934000967188)/(o+1.)+(-0.334488128196)+(0.894017590794)*0.5**o
        ref[2,0,1,0]=(0.819002513335)/(o+1.)+(-0.485788276874)+(-0.382158057826)*0.5**o
        ref[2,0,1,1]=(0.366917477923)/(o+1.)+(-0.64272230635)+(-0.259771272131)*0.5**o
        ref[2,0,2,0]=(-0.857625945147)/(o+1.)+(-0.581960052686)+(-0.33946164111)*0.5**o
        ref[2,0,2,1]=(-0.359476461654)/(o+1.)+(0.582637272819)+(0.153353093148)*0.5**o
        ref[2,1,0,0]=(0.194312746335)/(o+1.)+(0.607321463611)+(0.780980225775)*0.5**o
        ref[2,1,0,1]=(-0.313038325156)/(o+1.)+(0.0614854061938)+(-0.334149973283)*0.5**o
        ref[2,1,1,0]=(-0.0643816133251)/(o+1.)+(-0.635200340106)+(-0.832852528577)*0.5**o
        ref[2,1,1,1]=(-0.618456119184)/(o+1.)+(0.712480914191)+(0.300975732963)*0.5**o
        ref[2,1,2,0]=(-0.986852501828)/(o+1.)+(0.232254156437)+(0.122326105776)*0.5**o
        ref[2,1,2,1]=(-0.0933563424166)/(o+1.)+(0.337181975722)+(0.447916985436)*0.5**o
        ref[2,2,0,0]=(-0.053769157756)/(o+1.)+(-0.235907632211)+(0.032076347277)*0.5**o
        ref[2,2,0,1]=(-0.287956277998)/(o+1.)+(-0.564682397133)+(-0.403935703163)*0.5**o
        ref[2,2,1,0]=(0.359071620541)/(o+1.)+(-0.437996024361)+(-0.205077009492)*0.5**o
        ref[2,2,1,1]=(0.2153167916)/(o+1.)+(-0.612290869632)+(0.443658057141)*0.5**o
        ref[2,2,2,0]=(0.608221916958)/(o+1.)+(0.260259411478)+(-0.463879681251)*0.5**o
        ref[2,2,2,1]=(0.8957267808)/(o+1.)+(0.442369195849)+(0.533981754168)*0.5**o
        ref[2,3,0,0]=(0.717384215206)/(o+1.)+(0.396205463942)+(0.499387701403)*0.5**o
        ref[2,3,0,1]=(0.395132533716)/(o+1.)+(-0.107416025237)+(0.866500398882)*0.5**o
        ref[2,3,1,0]=(0.688771917581)/(o+1.)+(-0.601429218934)+(-0.879663354176)*0.5**o
        ref[2,3,1,1]=(0.240960084233)/(o+1.)+(-0.222057616607)+(0.291214284356)*0.5**o
        ref[2,3,2,0]=(-0.505342655776)/(o+1.)+(-0.00898493528542)+(0.47064755192)*0.5**o
        ref[2,3,2,1]=(-0.896333275063)/(o+1.)+(-0.241095664817)+(0.775731405735)*0.5**o
        ref[2,4,0,0]=(0.436744528143)/(o+1.)+(-0.0744964673986)+(-0.7221248688)*0.5**o
        ref[2,4,0,1]=(-0.031320962337)/(o+1.)+(-0.0526613334003)+(-0.742652222385)*0.5**o
        ref[2,4,1,0]=(-0.50790148262)/(o+1.)+(0.626822378181)+(-0.529575527815)*0.5**o
        ref[2,4,1,1]=(-0.770747718564)/(o+1.)+(0.2597241153)+(0.535631957883)*0.5**o
        ref[2,4,2,0]=(0.613495031968)/(o+1.)+(-0.0918624375077)+(0.81688202721)*0.5**o
        ref[2,4,2,1]=(0.093174141001)/(o+1.)+(0.365539433145)+(0.493471708612)*0.5**o
        ref[3,0,0,0]=(0.984307685082)/(o+1.)+(-0.374598361566)+(-0.0668630243284)*0.5**o
        ref[3,0,0,1]=(-0.693596086089)/(o+1.)+(-0.411310048192)+(-0.210452484293)*0.5**o
        ref[3,0,1,0]=(0.824648362105)/(o+1.)+(0.181698265217)+(-0.34853285523)*0.5**o
        ref[3,0,1,1]=(0.681514895836)/(o+1.)+(0.638634223316)+(0.861903045965)*0.5**o
        ref[3,0,2,0]=(-0.0390293468405)/(o+1.)+(-0.130600797806)+(0.0197670342844)*0.5**o
        ref[3,0,2,1]=(0.969057635866)/(o+1.)+(-0.0805103470423)+(-0.499093940395)*0.5**o
        ref[3,1,0,0]=(0.829599262959)/(o+1.)+(0.745807250739)+(-0.955568800095)*0.5**o
        ref[3,1,0,1]=(-0.777548579252)/(o+1.)+(-0.650254524284)+(-0.910307638058)*0.5**o
        ref[3,1,1,0]=(-0.108768166204)/(o+1.)+(0.864123237947)+(0.22353820834)*0.5**o
        ref[3,1,1,1]=(0.931152588498)/(o+1.)+(0.366098675227)+(0.371995768061)*0.5**o
        ref[3,1,2,0]=(-0.14032259215)/(o+1.)+(0.514140454408)+(0.4567417369)*0.5**o
        ref[3,1,2,1]=(0.942530922378)/(o+1.)+(-0.0546106599524)+(-0.855100213731)*0.5**o
        ref[3,2,0,0]=(0.991497679624)/(o+1.)+(-0.26553912869)+(0.17190936572)*0.5**o
        ref[3,2,0,1]=(-0.730243933765)/(o+1.)+(0.0671828123695)+(-0.683587206983)*0.5**o
        ref[3,2,1,0]=(0.36605487466)/(o+1.)+(-0.155226343006)+(0.051672736629)*0.5**o
        ref[3,2,1,1]=(-0.424515284177)/(o+1.)+(0.394142291336)+(0.438014662973)*0.5**o
        ref[3,2,2,0]=(0.326610298255)/(o+1.)+(0.98989944094)+(-0.709499532674)*0.5**o
        ref[3,2,2,1]=(-0.846657945161)/(o+1.)+(0.0841772101477)+(-0.657647707844)*0.5**o
        ref[3,3,0,0]=(0.549866100727)/(o+1.)+(-0.133734825728)+(0.21346320453)*0.5**o
        ref[3,3,0,1]=(0.0224330511636)/(o+1.)+(0.326044184638)+(0.249957470685)*0.5**o
        ref[3,3,1,0]=(0.119453667263)/(o+1.)+(-0.0144803823844)+(0.166507029533)*0.5**o
        ref[3,3,1,1]=(0.57743453064)/(o+1.)+(0.0392054026384)+(0.358248720077)*0.5**o
        ref[3,3,2,0]=(0.63832012742)/(o+1.)+(-0.866342571675)+(-0.896899724447)*0.5**o
        ref[3,3,2,1]=(0.496219405916)/(o+1.)+(0.271132181694)+(0.425194254583)*0.5**o
        ref[3,4,0,0]=(-0.283510438468)/(o+1.)+(-0.0526860327828)+(0.793902581592)*0.5**o
        ref[3,4,0,1]=(-0.259885631206)/(o+1.)+(-0.669909592284)+(0.372028980539)*0.5**o
        ref[3,4,1,0]=(0.895070489116)/(o+1.)+(-0.78522377607)+(-0.80489673786)*0.5**o
        ref[3,4,1,1]=(0.399816667015)/(o+1.)+(-0.0252589669489)+(0.272535126101)*0.5**o
        ref[3,4,2,0]=(0.906020184876)/(o+1.)+(0.280852618457)+(0.0341775288582)*0.5**o
        ref[3,4,2,1]=(-0.648936544722)/(o+1.)+(-0.924724185844)+(-0.745016464485)*0.5**o
      else:
        arg[0,0,0,0]=(-0.891975745754)*x[0]**o+(0.678331209591)*x[0]+(-0.450022305538)*x[1]**o+(0.521272586024)*x[1]+(-0.28868708769)*x[2]**o+(-0.631617544339)*x[2]
        arg[0,0,0,1]=(0.428698930349)*x[0]**o+(-0.856251438077)*x[0]+(0.915766367729)*x[1]**o+(-0.35019463199)*x[1]+(-0.0029340113102)*x[2]**o+(-0.394458505835)*x[2]
        arg[0,0,1,0]=(0.974069118653)*x[0]**o+(-0.674028781962)*x[0]+(0.489718068973)*x[1]**o+(-0.471323892144)*x[1]+(0.11469271678)*x[2]**o+(-0.264584383354)*x[2]
        arg[0,0,1,1]=(0.813412823502)*x[0]**o+(0.739276532652)*x[0]+(-0.863216628203)*x[1]**o+(0.486539683681)*x[1]+(-0.771448803712)*x[2]**o+(-0.621526889056)*x[2]
        arg[0,0,2,0]=(-0.874502916672)*x[0]**o+(0.0708047915752)*x[0]+(0.917576817333)*x[1]**o+(-0.257626031306)*x[1]+(-0.161822453369)*x[2]**o+(0.488866912786)*x[2]
        arg[0,0,2,1]=(-0.258524075807)*x[0]**o+(0.206589504672)*x[0]+(0.232971263871)*x[1]**o+(-0.917991566808)*x[1]+(0.725943193794)*x[2]**o+(-0.170341108992)*x[2]
        arg[0,1,0,0]=(0.489505903018)*x[0]**o+(0.386376068728)*x[0]+(0.510797276159)*x[1]**o+(0.45356046467)*x[1]+(-0.664963144251)*x[2]**o+(0.128579947316)*x[2]
        arg[0,1,0,1]=(0.441567332096)*x[0]**o+(0.658686083904)*x[0]+(0.46601056348)*x[1]**o+(-0.130759928108)*x[1]+(0.324576589)*x[2]**o+(0.844700852997)*x[2]
        arg[0,1,1,0]=(0.388576628492)*x[0]**o+(0.41731399682)*x[0]+(-0.37037879729)*x[1]**o+(-0.00306376860454)*x[1]+(-0.464420131682)*x[2]**o+(0.570470055678)*x[2]
        arg[0,1,1,1]=(-0.0537780037583)*x[0]**o+(0.641854502644)*x[0]+(-0.475355608842)*x[1]**o+(-0.155501886306)*x[1]+(0.159833591997)*x[2]**o+(0.889951997355)*x[2]
        arg[0,1,2,0]=(0.58262140787)*x[0]**o+(0.487380368384)*x[0]+(-0.783228845976)*x[1]**o+(-0.683221249176)*x[1]+(0.98628901689)*x[2]**o+(0.591884616904)*x[2]
        arg[0,1,2,1]=(0.438472233911)*x[0]**o+(0.989088416794)*x[0]+(0.221688966923)*x[1]**o+(-0.119266918098)*x[1]+(-0.887421255415)*x[2]**o+(0.728082459156)*x[2]
        arg[0,2,0,0]=(-0.334110476968)*x[0]**o+(-0.877094545216)*x[0]+(-0.293135209996)*x[1]**o+(0.578110480224)*x[1]+(-0.898686340104)*x[2]**o+(0.528072783107)*x[2]
        arg[0,2,0,1]=(0.33041258354)*x[0]**o+(-0.70349630629)*x[0]+(0.349551161792)*x[1]**o+(-0.296687014146)*x[1]+(-0.272377440466)*x[2]**o+(-0.42221458274)*x[2]
        arg[0,2,1,0]=(-0.80021736912)*x[0]**o+(0.271727608951)*x[0]+(0.689351606243)*x[1]**o+(-0.95063104158)*x[1]+(0.447293221661)*x[2]**o+(0.962435109303)*x[2]
        arg[0,2,1,1]=(-0.773331228218)*x[0]**o+(0.569225340051)*x[0]+(-0.0330042729183)*x[1]**o+(-0.732867386826)*x[1]+(-0.399631128084)*x[2]**o+(0.235907094874)*x[2]
        arg[0,2,2,0]=(-0.943072868965)*x[0]**o+(-0.959364749946)*x[0]+(0.922934300445)*x[1]**o+(0.27257203243)*x[1]+(0.450453864748)*x[2]**o+(0.521284310775)*x[2]
        arg[0,2,2,1]=(-0.821201079286)*x[0]**o+(-0.801011107627)*x[0]+(0.554575277011)*x[1]**o+(-0.411361733513)*x[1]+(-0.858195570474)*x[2]**o+(0.420660103079)*x[2]
        arg[0,3,0,0]=(-0.0564164524098)*x[0]**o+(-0.865516143298)*x[0]+(-0.85754022189)*x[1]**o+(-0.0159800361584)*x[1]+(-0.891158469429)*x[2]**o+(-0.809909518115)*x[2]
        arg[0,3,0,1]=(-0.553858251068)*x[0]**o+(0.827558838148)*x[0]+(0.690081127613)*x[1]**o+(0.3475090336)*x[1]+(-0.190426828891)*x[2]**o+(-0.99589316535)*x[2]
        arg[0,3,1,0]=(-0.992317453163)*x[0]**o+(-0.992812531491)*x[0]+(-0.859683310869)*x[1]**o+(0.441779587739)*x[1]+(0.582457020078)*x[2]**o+(-0.959111461424)*x[2]
        arg[0,3,1,1]=(-0.713753922213)*x[0]**o+(0.504885148776)*x[0]+(-0.076988545634)*x[1]**o+(0.530828889101)*x[1]+(-0.335616242708)*x[2]**o+(-0.0421293420747)*x[2]
        arg[0,3,2,0]=(0.536024118289)*x[0]**o+(-0.527038052847)*x[0]+(-0.558392588656)*x[1]**o+(0.928618057856)*x[1]+(0.427507510033)*x[2]**o+(0.223570914141)*x[2]
        arg[0,3,2,1]=(-0.197968756117)*x[0]**o+(-0.58157606173)*x[0]+(-0.878630012825)*x[1]**o+(-0.380635552726)*x[1]+(-0.477570149125)*x[2]**o+(-0.642904015434)*x[2]
        arg[0,4,0,0]=(0.688763655127)*x[0]**o+(0.286011523933)*x[0]+(0.0933014250929)*x[1]**o+(0.355550563233)*x[1]+(0.121445291305)*x[2]**o+(-0.814538082974)*x[2]
        arg[0,4,0,1]=(-0.402717742839)*x[0]**o+(0.403946649835)*x[0]+(0.286978800579)*x[1]**o+(-0.363358056216)*x[1]+(-0.185233594102)*x[2]**o+(-0.902660250807)*x[2]
        arg[0,4,1,0]=(0.39352838659)*x[0]**o+(-0.430755011337)*x[0]+(-0.912518631162)*x[1]**o+(-0.634118795867)*x[1]+(-0.802445843416)*x[2]**o+(0.643395158048)*x[2]
        arg[0,4,1,1]=(-0.375507250175)*x[0]**o+(-0.609576942939)*x[0]+(0.35060243693)*x[1]**o+(0.384630130603)*x[1]+(0.165576820334)*x[2]**o+(-0.319012357341)*x[2]
        arg[0,4,2,0]=(0.0603590263818)*x[0]**o+(-0.435954775883)*x[0]+(-0.895329092913)*x[1]**o+(0.364809539103)*x[1]+(0.912864788565)*x[2]**o+(0.582044419963)*x[2]
        arg[0,4,2,1]=(-0.111089454685)*x[0]**o+(-0.247527778755)*x[0]+(0.897263901657)*x[1]**o+(0.168097655548)*x[1]+(-0.532273075067)*x[2]**o+(-0.158529268933)*x[2]
        arg[1,0,0,0]=(-0.940865155622)*x[0]**o+(0.639652431549)*x[0]+(0.204433172026)*x[1]**o+(-0.333435100662)*x[1]+(0.922286591304)*x[2]**o+(0.245091036189)*x[2]
        arg[1,0,0,1]=(-0.843207880309)*x[0]**o+(0.496567621809)*x[0]+(-0.747925907677)*x[1]**o+(-0.666195656311)*x[1]+(-0.836848401917)*x[2]**o+(-0.77301298659)*x[2]
        arg[1,0,1,0]=(-0.484987528212)*x[0]**o+(-0.208798022576)*x[0]+(-0.516295195217)*x[1]**o+(0.273733676249)*x[1]+(0.642865886081)*x[2]**o+(0.94556566928)*x[2]
        arg[1,0,1,1]=(0.285548625366)*x[0]**o+(-0.316870037241)*x[0]+(0.166055429845)*x[1]**o+(-0.907240066063)*x[1]+(-0.973602611383)*x[2]**o+(-0.232655535454)*x[2]
        arg[1,0,2,0]=(0.96797757731)*x[0]**o+(-0.0920831342923)*x[0]+(0.0367163548581)*x[1]**o+(0.372503137376)*x[1]+(0.114902128151)*x[2]**o+(0.147390259838)*x[2]
        arg[1,0,2,1]=(-0.486190283922)*x[0]**o+(-0.713417720041)*x[0]+(0.0212905465522)*x[1]**o+(-0.358636168795)*x[1]+(0.719370965272)*x[2]**o+(-0.506502687359)*x[2]
        arg[1,1,0,0]=(-0.216544472879)*x[0]**o+(-0.690540499698)*x[0]+(0.190155427044)*x[1]**o+(0.832206912182)*x[1]+(-0.108713936252)*x[2]**o+(-0.221373087337)*x[2]
        arg[1,1,0,1]=(0.982934604971)*x[0]**o+(0.604369826835)*x[0]+(-0.34365978148)*x[1]**o+(-0.167486779981)*x[1]+(-0.753609370582)*x[2]**o+(-0.220160888977)*x[2]
        arg[1,1,1,0]=(0.516177509446)*x[0]**o+(-0.474445723273)*x[0]+(0.813647087566)*x[1]**o+(-0.281201093802)*x[1]+(0.941465944108)*x[2]**o+(0.352728360059)*x[2]
        arg[1,1,1,1]=(-0.914732890633)*x[0]**o+(0.214962090901)*x[0]+(-0.710485058274)*x[1]**o+(-0.070756754056)*x[1]+(-0.449571590211)*x[2]**o+(0.353655711451)*x[2]
        arg[1,1,2,0]=(-0.96655555619)*x[0]**o+(-0.47460534732)*x[0]+(0.0610695053327)*x[1]**o+(0.815390116709)*x[1]+(-0.844916163552)*x[2]**o+(-0.433015304307)*x[2]
        arg[1,1,2,1]=(-0.611879453709)*x[0]**o+(0.849092848509)*x[0]+(0.0181322361566)*x[1]**o+(0.264173017827)*x[1]+(0.0452154806937)*x[2]**o+(0.0395300648465)*x[2]
        arg[1,2,0,0]=(0.558820946357)*x[0]**o+(-0.471519705598)*x[0]+(-0.200404864145)*x[1]**o+(0.452044160691)*x[1]+(-0.507559485963)*x[2]**o+(-0.993437841726)*x[2]
        arg[1,2,0,1]=(-0.944738664192)*x[0]**o+(-0.497536772117)*x[0]+(0.111399525675)*x[1]**o+(-0.93450294271)*x[1]+(-0.463702615737)*x[2]**o+(-0.330112672884)*x[2]
        arg[1,2,1,0]=(0.238225961058)*x[0]**o+(-0.471520173419)*x[0]+(0.483767084892)*x[1]**o+(-0.0359407284537)*x[1]+(-0.767379409753)*x[2]**o+(-0.283596371915)*x[2]
        arg[1,2,1,1]=(0.188699181238)*x[0]**o+(0.829593121989)*x[0]+(0.603377767593)*x[1]**o+(0.163334775468)*x[1]+(-0.695141594131)*x[2]**o+(-0.335208640181)*x[2]
        arg[1,2,2,0]=(-0.0953160757881)*x[0]**o+(0.179594592176)*x[0]+(0.95983902191)*x[1]**o+(-0.689622683527)*x[1]+(0.418442085936)*x[2]**o+(-0.271399861129)*x[2]
        arg[1,2,2,1]=(0.598691886555)*x[0]**o+(0.334120340646)*x[0]+(0.580763872443)*x[1]**o+(0.769016736836)*x[1]+(-0.442121483951)*x[2]**o+(0.0685264980499)*x[2]
        arg[1,3,0,0]=(-0.482944276372)*x[0]**o+(-0.0518057205519)*x[0]+(-0.878100108766)*x[1]**o+(-0.130957239599)*x[1]+(0.71270341547)*x[2]**o+(-0.313822116756)*x[2]
        arg[1,3,0,1]=(0.565128725562)*x[0]**o+(0.706149004661)*x[0]+(0.837747652328)*x[1]**o+(-0.429417376176)*x[1]+(0.179437627997)*x[2]**o+(-0.841831183522)*x[2]
        arg[1,3,1,0]=(0.608648887224)*x[0]**o+(0.118220851451)*x[0]+(0.492890967377)*x[1]**o+(-0.162344084552)*x[1]+(0.141203210671)*x[2]**o+(0.883457779687)*x[2]
        arg[1,3,1,1]=(0.0322521291544)*x[0]**o+(0.839748977403)*x[0]+(0.75094786544)*x[1]**o+(0.954833145672)*x[1]+(0.461457599013)*x[2]**o+(-0.761973385747)*x[2]
        arg[1,3,2,0]=(-0.757022609619)*x[0]**o+(-0.250893790099)*x[0]+(-0.973631035961)*x[1]**o+(0.506574565217)*x[1]+(0.103557430896)*x[2]**o+(0.881026337134)*x[2]
        arg[1,3,2,1]=(-0.395945302352)*x[0]**o+(-0.125428842065)*x[0]+(-0.339663004568)*x[1]**o+(-0.0272044592179)*x[1]+(0.75200506467)*x[2]**o+(-0.632922582454)*x[2]
        arg[1,4,0,0]=(0.0462127928202)*x[0]**o+(-0.745613488751)*x[0]+(0.357338297422)*x[1]**o+(0.739034094804)*x[1]+(-0.451247272253)*x[2]**o+(0.418030110847)*x[2]
        arg[1,4,0,1]=(0.914131238526)*x[0]**o+(-0.0280436066624)*x[0]+(0.861048515118)*x[1]**o+(0.148206807819)*x[1]+(-0.990856990354)*x[2]**o+(0.903615279172)*x[2]
        arg[1,4,1,0]=(0.189809824038)*x[0]**o+(-0.749477832258)*x[0]+(-0.802713794942)*x[1]**o+(0.0610727255572)*x[1]+(0.292940815327)*x[2]**o+(0.213443422205)*x[2]
        arg[1,4,1,1]=(0.792313292544)*x[0]**o+(0.321344675677)*x[0]+(-0.417516596097)*x[1]**o+(-0.63797359819)*x[1]+(-0.426112319872)*x[2]**o+(0.69281310288)*x[2]
        arg[1,4,2,0]=(-0.11017764073)*x[0]**o+(0.703037562344)*x[0]+(0.739591950873)*x[1]**o+(0.445612268331)*x[1]+(-0.071251825024)*x[2]**o+(0.00577350654185)*x[2]
        arg[1,4,2,1]=(-0.793584849832)*x[0]**o+(-0.546439256231)*x[0]+(-0.947194030803)*x[1]**o+(-0.0455700937677)*x[1]+(0.535368301918)*x[2]**o+(0.814435535521)*x[2]
        arg[2,0,0,0]=(0.249661949438)*x[0]**o+(-0.581533619092)*x[0]+(-0.674373505747)*x[1]**o+(0.464175602196)*x[1]+(-0.528086662834)*x[2]**o+(-0.212416176351)*x[2]
        arg[2,0,0,1]=(-0.588282108798)*x[0]**o+(0.761293399494)*x[0]+(0.99918874903)*x[1]**o+(0.542499992928)*x[1]+(0.433925746483)*x[2]**o+(0.972864491889)*x[2]
        arg[2,0,1,0]=(0.0572335623442)*x[0]**o+(0.444906443533)*x[0]+(-0.305538151235)*x[1]**o+(0.244530068532)*x[1]+(-0.20198975858)*x[2]**o+(-0.00490928477351)*x[2]
        arg[2,0,1,1]=(0.832496085695)*x[0]**o+(-0.797246004595)*x[0]+(0.873816520001)*x[1]**o+(0.628293733929)*x[1]+(-0.110922710624)*x[2]**o+(-0.0718221914062)*x[2]
        arg[2,0,2,0]=(0.140970192196)*x[0]**o+(0.582230323368)*x[0]+(-0.757764751267)*x[1]**o+(0.206061262231)*x[1]+(0.657237474788)*x[2]**o+(0.598632779121)*x[2]
        arg[2,0,2,1]=(-0.686218790734)*x[0]**o+(-0.157012470162)*x[0]+(0.448565721848)*x[1]**o+(0.0539087954076)*x[1]+(0.619821805007)*x[2]**o+(0.289144379283)*x[2]
        arg[2,1,0,0]=(0.787423884494)*x[0]**o+(-0.985959420171)*x[0]+(0.155584360983)*x[1]**o+(0.223292293778)*x[1]+(-0.437349719833)*x[2]**o+(0.989077189275)*x[2]
        arg[2,1,0,1]=(-0.597301631508)*x[0]**o+(-0.0414813766671)*x[0]+(-0.64883048443)*x[1]**o+(-0.18731615522)*x[1]+(0.735189823623)*x[2]**o+(-0.746633725819)*x[2]
        arg[2,1,1,0]=(-0.393486075452)*x[0]**o+(-0.480110060647)*x[0]+(0.559811870081)*x[1]**o+(0.0814282056993)*x[1]+(0.244899906416)*x[2]**o+(-0.884478480652)*x[2]
        arg[2,1,1,1]=(0.547747813175)*x[0]**o+(0.317890585903)*x[0]+(-0.70925099068)*x[1]**o+(-0.543227445702)*x[1]+(0.749693240223)*x[2]**o+(0.79984501038)*x[2]
        arg[2,1,2,0]=(-0.289583636666)*x[0]**o+(0.513926622265)*x[0]+(-0.00283544918865)*x[1]**o+(0.530724065811)*x[1]+(0.900479623243)*x[2]**o+(-0.146332301083)*x[2]
        arg[2,1,2,1]=(-0.487223003118)*x[0]**o+(0.161546483594)*x[0]+(-0.685211654772)*x[1]**o+(0.4803179816)*x[1]+(0.676120396917)*x[2]**o+(-0.174953085068)*x[2]
        arg[2,2,0,0]=(0.31746495834)*x[0]**o+(-0.513225182059)*x[0]+(0.33220677322)*x[1]**o+(-0.302371334854)*x[1]+(0.0964988251907)*x[2]**o+(-0.259532745762)*x[2]
        arg[2,2,0,1]=(0.313462846078)*x[0]**o+(0.079720902627)*x[0]+(-0.487932457563)*x[1]**o+(-0.786522786773)*x[1]+(0.357868214554)*x[2]**o+(0.251856050362)*x[2]
        arg[2,2,1,0]=(-0.180472378556)*x[0]**o+(0.345084802362)*x[0]+(0.641108557953)*x[1]**o+(0.198717742483)*x[1]+(0.0374114682326)*x[2]**o+(-0.915249907773)*x[2]
        arg[2,2,1,1]=(0.303787742688)*x[0]**o+(-0.941822487745)*x[0]+(-0.262567025842)*x[1]**o+(-0.197051445307)*x[1]+(0.381561965565)*x[2]**o+(-0.198549777232)*x[2]
        arg[2,2,2,0]=(-0.850284190461)*x[0]**o+(0.985224868544)*x[0]+(0.633868153422)*x[1]**o+(-0.57559927346)*x[1]+(0.68753726951)*x[2]**o+(-0.923696997656)*x[2]
        arg[2,2,2,1]=(0.559527478503)*x[0]**o+(-0.619764636475)*x[0]+(-0.0352163693945)*x[1]**o+(-0.37351931995)*x[1]+(0.0772333960938)*x[2]**o+(-0.353057755509)*x[2]
        arg[2,3,0,0]=(-0.274260374984)*x[0]**o+(0.365180349506)*x[0]+(-0.879564869545)*x[1]**o+(-0.0908162765131)*x[1]+(0.813491263836)*x[2]**o+(-0.396739585204)*x[2]
        arg[2,3,0,1]=(-0.793218219979)*x[0]**o+(0.227474218201)*x[0]+(0.97716610661)*x[1]**o+(0.317074032722)*x[1]+(0.076776446926)*x[2]**o+(0.069448617786)*x[2]
        arg[2,3,1,0]=(0.411400569812)*x[0]**o+(0.517743454926)*x[0]+(0.646697523033)*x[1]**o+(-0.973369313024)*x[1]+(-0.599741793926)*x[2]**o+(-0.743687020283)*x[2]
        arg[2,3,1,1]=(-0.491445354697)*x[0]**o+(-0.549840110396)*x[0]+(-0.260012915614)*x[1]**o+(0.671054626537)*x[1]+(0.0394966796969)*x[2]**o+(-0.701243931411)*x[2]
        arg[2,3,2,0]=(-0.806859723517)*x[0]**o+(-0.753940066715)*x[0]+(-0.664229274865)*x[1]**o+(0.0595484609931)*x[1]+(-0.921892262667)*x[2]**o+(-0.701770672395)*x[2]
        arg[2,3,2,1]=(-0.313979805844)*x[0]**o+(-0.542583864222)*x[0]+(-0.985990886553)*x[1]**o+(0.664926443979)*x[1]+(-0.99778334615)*x[2]**o+(0.268046810979)*x[2]
        arg[2,4,0,0]=(-0.41348240167)*x[0]**o+(0.473517720534)*x[0]+(-0.290880345559)*x[1]**o+(-0.607611624107)*x[1]+(-0.25537697471)*x[2]**o+(-0.472402477362)*x[2]
        arg[2,4,0,1]=(0.0222534371058)*x[0]**o+(-0.0739354104963)*x[0]+(-0.960057765718)*x[1]**o+(-0.772161403037)*x[1]+(-0.0356127352146)*x[2]**o+(-0.623605628308)*x[2]
        arg[2,4,1,0]=(-0.750995516053)*x[0]**o+(-0.289601799215)*x[0]+(0.294370180152)*x[1]**o+(0.956012147239)*x[1]+(-0.27958662106)*x[2]**o+(-0.370609593537)*x[2]
        arg[2,4,1,1]=(0.321767371741)*x[0]**o+(0.999223434136)*x[0]+(0.665865428241)*x[1]**o+(0.427942082269)*x[1]+(-0.135020552387)*x[2]**o+(0.692025479514)*x[2]
        arg[2,4,2,0]=(-0.519508249649)*x[0]**o+(0.0779688946471)*x[0]+(0.493489242872)*x[1]**o+(-0.0961136530093)*x[1]+(-0.0750243559261)*x[2]**o+(0.860970740104)*x[2]
        arg[2,4,2,1]=(0.807909049346)*x[0]**o+(0.388180877227)*x[0]+(-0.9044578433)*x[1]**o+(0.86765389355)*x[1]+(-0.196105779343)*x[2]**o+(0.448661151455)*x[2]
        arg[3,0,0,0]=(-0.780895567041)*x[0]**o+(-0.571253266808)*x[0]+(-0.904183399932)*x[1]**o+(-0.883296586722)*x[1]+(0.855945559487)*x[2]**o+(0.618533553098)*x[2]
        arg[3,0,0,1]=(-0.831076704037)*x[0]**o+(0.898347446798)*x[0]+(-0.716442620396)*x[1]**o+(0.761028369421)*x[1]+(-0.486505604444)*x[2]**o+(0.25474220145)*x[2]
        arg[3,0,1,0]=(-0.0127541044953)*x[0]**o+(0.759482315308)*x[0]+(-0.725655006866)*x[1]**o+(0.667310781285)*x[1]+(0.266075927439)*x[2]**o+(0.423300182252)*x[2]
        arg[3,0,1,1]=(-0.594972266882)*x[0]**o+(-0.842501149639)*x[0]+(-0.347686790781)*x[1]**o+(0.686403640474)*x[1]+(0.3452971966)*x[2]**o+(0.722258585706)*x[2]
        arg[3,0,2,0]=(-0.715711146058)*x[0]**o+(-0.835739963371)*x[0]+(0.966177361301)*x[1]**o+(-0.909111754995)*x[1]+(-0.175184576557)*x[2]**o+(0.695058239921)*x[2]
        arg[3,0,2,1]=(0.306251544915)*x[0]**o+(0.944775715803)*x[0]+(-0.740601754617)*x[1]**o+(0.434394890125)*x[1]+(-0.876527474971)*x[2]**o+(0.209321129689)*x[2]
        arg[3,1,0,0]=(-0.86788221871)*x[0]**o+(-0.833030657309)*x[0]+(0.84576621947)*x[1]**o+(-0.469186887277)*x[1]+(-0.527611740694)*x[2]**o+(-0.312054431001)*x[2]
        arg[3,1,0,1]=(-0.353678394652)*x[0]**o+(0.591287444611)*x[0]+(-0.213012375931)*x[1]**o+(-0.917259099251)*x[1]+(0.739016360622)*x[2]**o+(0.718754459378)*x[2]
        arg[3,1,1,0]=(-0.92646825221)*x[0]**o+(-0.870564973175)*x[0]+(-0.781805219404)*x[1]**o+(0.078530434642)*x[1]+(0.386018501087)*x[2]**o+(-0.120490492288)*x[2]
        arg[3,1,1,1]=(-0.824273737615)*x[0]**o+(0.401211543421)*x[0]+(0.360077381624)*x[1]**o+(-0.0599234883152)*x[1]+(0.246820480884)*x[2]**o+(-0.175313542134)*x[2]
        arg[3,1,2,0]=(0.807756928769)*x[0]**o+(0.889534807508)*x[0]+(0.633296189954)*x[1]**o+(-0.471821951252)*x[1]+(0.396011249874)*x[2]**o+(0.386800809575)*x[2]
        arg[3,1,2,1]=(-0.494145142543)*x[0]**o+(-0.514793985721)*x[0]+(0.825379848403)*x[1]**o+(-0.457860259786)*x[1]+(-0.599557868275)*x[2]**o+(-0.952704784672)*x[2]
        arg[3,2,0,0]=(0.970345295013)*x[0]**o+(-0.838468036663)*x[0]+(0.265634348738)*x[1]**o+(-0.873847124748)*x[1]+(0.416242648913)*x[2]**o+(0.59908438222)*x[2]
        arg[3,2,0,1]=(0.692377432429)*x[0]**o+(-0.338749041227)*x[0]+(-0.449383750398)*x[1]**o+(0.841669527559)*x[1]+(0.902482059741)*x[2]**o+(-0.655719647704)*x[2]
        arg[3,2,1,0]=(0.152417917522)*x[0]**o+(-0.78860144)*x[0]+(-0.881276170743)*x[1]**o+(0.692901550061)*x[1]+(-0.985377819322)*x[2]**o+(0.362747973601)*x[2]
        arg[3,2,1,1]=(0.238845792119)*x[0]**o+(-0.65838373309)*x[0]+(-0.659676514296)*x[1]**o+(0.870742876778)*x[1]+(-0.367345415947)*x[2]**o+(0.322353666541)*x[2]
        arg[3,2,2,0]=(0.456386394533)*x[0]**o+(-0.135575574979)*x[0]+(-0.731714323922)*x[1]**o+(0.936807859479)*x[1]+(0.511595594967)*x[2]**o+(0.570282464669)*x[2]
        arg[3,2,2,1]=(0.53299212624)*x[0]**o+(0.800998166434)*x[0]+(-0.431740087552)*x[1]**o+(-0.880092532661)*x[1]+(-0.418657988674)*x[2]**o+(-0.181008149338)*x[2]
        arg[3,3,0,0]=(0.60015312936)*x[0]**o+(-0.163125583081)*x[0]+(0.299668981953)*x[1]**o+(-0.761494422747)*x[1]+(0.404839953231)*x[2]**o+(-0.114039836538)*x[2]
        arg[3,3,0,1]=(-0.597481072133)*x[0]**o+(-0.506468582896)*x[0]+(0.406822348574)*x[1]**o+(0.247431348859)*x[1]+(-0.137941563908)*x[2]**o+(0.0503979259383)*x[2]
        arg[3,3,1,0]=(0.0921069092744)*x[0]**o+(-0.113960490794)*x[0]+(0.750059773909)*x[1]**o+(0.894945048248)*x[1]+(-0.154873449786)*x[2]**o+(-0.520229467775)*x[2]
        arg[3,3,1,1]=(0.349078696154)*x[0]**o+(0.141298430827)*x[0]+(0.249161007832)*x[1]**o+(0.0240310278591)*x[1]+(-0.18294527838)*x[2]**o+(-0.666698832632)*x[2]
        arg[3,3,2,0]=(-0.476684936147)*x[0]**o+(0.334093402615)*x[0]+(-0.326434684009)*x[1]**o+(0.91336431975)*x[1]+(0.483080537399)*x[2]**o+(0.439096732335)*x[2]
        arg[3,3,2,1]=(0.214713426286)*x[0]**o+(-0.973239849984)*x[0]+(0.984290637567)*x[1]**o+(-0.644668525299)*x[1]+(0.00985524319943)*x[2]**o+(0.68777403533)*x[2]
        arg[3,4,0,0]=(-0.88715423933)*x[0]**o+(0.642412222188)*x[0]+(-0.119667464132)*x[1]**o+(-0.201583116481)*x[1]+(-0.870921042381)*x[2]**o+(-0.00379039429884)*x[2]
        arg[3,4,0,1]=(0.511114986801)*x[0]**o+(-0.497617928997)*x[0]+(-0.189435233543)*x[1]**o+(-0.29288141421)*x[1]+(-0.476297664539)*x[2]**o+(-0.202915859137)*x[2]
        arg[3,4,1,0]=(0.0739878219028)*x[0]**o+(0.675050675951)*x[0]+(-0.45950101521)*x[1]**o+(0.306538598103)*x[1]+(-0.726069135031)*x[2]**o+(0.193290422951)*x[2]
        arg[3,4,1,1]=(-0.141324267565)*x[0]**o+(0.377343882109)*x[0]+(-0.16075604872)*x[1]**o+(0.263332274591)*x[1]+(0.886416182992)*x[2]**o+(0.931905996631)*x[2]
        arg[3,4,2,0]=(-0.76912732001)*x[0]**o+(-0.255011103121)*x[0]+(-0.372810348364)*x[1]**o+(0.108137766687)*x[1]+(-0.0761382180171)*x[2]**o+(0.46825331282)*x[2]
        arg[3,4,2,1]=(-0.515945940217)*x[0]**o+(-0.630017609554)*x[0]+(-0.815236764508)*x[1]**o+(0.636572471005)*x[1]+(0.244112465789)*x[2]**o+(-0.471661017589)*x[2]
        ref[0,0,0,0]=(-0.738709393227)/(o+1.)+(0.283993125638)+(-0.891975745754)*0.5**o
        ref[0,0,0,1]=(0.912832356419)/(o+1.)+(-0.800452287951)+(0.428698930349)*0.5**o
        ref[0,0,1,0]=(0.604410785754)/(o+1.)+(-0.70496852873)+(0.974069118653)*0.5**o
        ref[0,0,1,1]=(-1.63466543191)/(o+1.)+(0.302144663639)+(0.813412823502)*0.5**o
        ref[0,0,2,0]=(0.755754363963)/(o+1.)+(0.151022836528)+(-0.874502916672)*0.5**o
        ref[0,0,2,1]=(0.958914457666)/(o+1.)+(-0.440871585564)+(-0.258524075807)*0.5**o
        ref[0,1,0,0]=(-0.154165868092)/(o+1.)+(0.484258240357)+(0.489505903018)*0.5**o
        ref[0,1,0,1]=(0.79058715248)/(o+1.)+(0.686313504396)+(0.441567332096)*0.5**o
        ref[0,1,1,0]=(-0.834798928973)/(o+1.)+(0.492360141946)+(0.388576628492)*0.5**o
        ref[0,1,1,1]=(-0.315522016845)/(o+1.)+(0.688152306846)+(-0.0537780037583)*0.5**o
        ref[0,1,2,0]=(0.203060170914)/(o+1.)+(0.198021868056)+(0.58262140787)*0.5**o
        ref[0,1,2,1]=(-0.665732288492)/(o+1.)+(0.798951978927)+(0.438472233911)*0.5**o
        ref[0,2,0,0]=(-1.1918215501)/(o+1.)+(0.114544359058)+(-0.334110476968)*0.5**o
        ref[0,2,0,1]=(0.0771737213265)/(o+1.)+(-0.711198951588)+(0.33041258354)*0.5**o
        ref[0,2,1,0]=(1.1366448279)/(o+1.)+(0.141765838337)+(-0.80021736912)*0.5**o
        ref[0,2,1,1]=(-0.432635401002)/(o+1.)+(0.0361325240497)+(-0.773331228218)*0.5**o
        ref[0,2,2,0]=(1.37338816519)/(o+1.)+(-0.0827542033711)+(-0.943072868965)*0.5**o
        ref[0,2,2,1]=(-0.303620293463)/(o+1.)+(-0.39585636903)+(-0.821201079286)*0.5**o
        ref[0,3,0,0]=(-1.74869869132)/(o+1.)+(-0.845702848785)+(-0.0564164524098)*0.5**o
        ref[0,3,0,1]=(0.499654298722)/(o+1.)+(0.0895873531986)+(-0.553858251068)*0.5**o
        ref[0,3,1,0]=(-0.277226290791)/(o+1.)+(-0.755072202588)+(-0.992317453163)*0.5**o
        ref[0,3,1,1]=(-0.412604788342)/(o+1.)+(0.496792347901)+(-0.713753922213)*0.5**o
        ref[0,3,2,0]=(-0.130885078624)/(o+1.)+(0.312575459575)+(0.536024118289)*0.5**o
        ref[0,3,2,1]=(-1.35620016195)/(o+1.)+(-0.802557814945)+(-0.197968756117)*0.5**o
        ref[0,4,0,0]=(0.214746716397)/(o+1.)+(-0.0864879979042)+(0.688763655127)*0.5**o
        ref[0,4,0,1]=(0.101745206477)/(o+1.)+(-0.431035828594)+(-0.402717742839)*0.5**o
        ref[0,4,1,0]=(-1.71496447458)/(o+1.)+(-0.210739324578)+(0.39352838659)*0.5**o
        ref[0,4,1,1]=(0.516179257264)/(o+1.)+(-0.271979584839)+(-0.375507250175)*0.5**o
        ref[0,4,2,0]=(0.0175356956523)/(o+1.)+(0.255449591592)+(0.0603590263818)*0.5**o
        ref[0,4,2,1]=(0.36499082659)/(o+1.)+(-0.11897969607)+(-0.111089454685)*0.5**o
        ref[1,0,0,0]=(1.12671976333)/(o+1.)+(0.275654183538)+(-0.940865155622)*0.5**o
        ref[1,0,0,1]=(-1.58477430959)/(o+1.)+(-0.471320510546)+(-0.843207880309)*0.5**o
        ref[1,0,1,0]=(0.126570690864)/(o+1.)+(0.505250661477)+(-0.484987528212)*0.5**o
        ref[1,0,1,1]=(-0.807547181539)/(o+1.)+(-0.728382819379)+(0.285548625366)*0.5**o
        ref[1,0,2,0]=(0.151618483009)/(o+1.)+(0.213905131461)+(0.96797757731)*0.5**o
        ref[1,0,2,1]=(0.740661511824)/(o+1.)+(-0.789278288097)+(-0.486190283922)*0.5**o
        ref[1,1,0,0]=(0.0814414907926)/(o+1.)+(-0.0398533374263)+(-0.216544472879)*0.5**o
        ref[1,1,0,1]=(-1.09726915206)/(o+1.)+(0.108361078938)+(0.982934604971)*0.5**o
        ref[1,1,1,0]=(1.75511303167)/(o+1.)+(-0.201459228508)+(0.516177509446)*0.5**o
        ref[1,1,1,1]=(-1.16005664849)/(o+1.)+(0.248930524148)+(-0.914732890633)*0.5**o
        ref[1,1,2,0]=(-0.78384665822)/(o+1.)+(-0.0461152674588)+(-0.96655555619)*0.5**o
        ref[1,1,2,1]=(0.0633477168503)/(o+1.)+(0.576397965591)+(-0.611879453709)*0.5**o
        ref[1,2,0,0]=(-0.707964350108)/(o+1.)+(-0.506456693317)+(0.558820946357)*0.5**o
        ref[1,2,0,1]=(-0.352303090063)/(o+1.)+(-0.881076193856)+(-0.944738664192)*0.5**o
        ref[1,2,1,0]=(-0.283612324861)/(o+1.)+(-0.395528636894)+(0.238225961058)*0.5**o
        ref[1,2,1,1]=(-0.0917638265383)/(o+1.)+(0.328859628638)+(0.188699181238)*0.5**o
        ref[1,2,2,0]=(1.37828110785)/(o+1.)+(-0.39071397624)+(-0.0953160757881)*0.5**o
        ref[1,2,2,1]=(0.138642388492)/(o+1.)+(0.585831787766)+(0.598691886555)*0.5**o
        ref[1,3,0,0]=(-0.165396693296)/(o+1.)+(-0.248292538454)+(-0.482944276372)*0.5**o
        ref[1,3,0,1]=(1.01718528032)/(o+1.)+(-0.282549777519)+(0.565128725562)*0.5**o
        ref[1,3,1,0]=(0.634094178049)/(o+1.)+(0.419667273293)+(0.608648887224)*0.5**o
        ref[1,3,1,1]=(1.21240546445)/(o+1.)+(0.516304368664)+(0.0322521291544)*0.5**o
        ref[1,3,2,0]=(-0.870073605065)/(o+1.)+(0.568353556126)+(-0.757022609619)*0.5**o
        ref[1,3,2,1]=(0.412342060102)/(o+1.)+(-0.392777941868)+(-0.395945302352)*0.5**o
        ref[1,4,0,0]=(-0.0939089748304)/(o+1.)+(0.20572535845)+(0.0462127928202)*0.5**o
        ref[1,4,0,1]=(-0.129808475236)/(o+1.)+(0.511889240164)+(0.914131238526)*0.5**o
        ref[1,4,1,0]=(-0.509772979615)/(o+1.)+(-0.237480842248)+(0.189809824038)*0.5**o
        ref[1,4,1,1]=(-0.843628915969)/(o+1.)+(0.188092090183)+(0.792313292544)*0.5**o
        ref[1,4,2,0]=(0.668340125849)/(o+1.)+(0.577211668608)+(-0.11017764073)*0.5**o
        ref[1,4,2,1]=(-0.411825728886)/(o+1.)+(0.111213092761)+(-0.793584849832)*0.5**o
        ref[2,0,0,0]=(-1.20246016858)/(o+1.)+(-0.164887096623)+(0.249661949438)*0.5**o
        ref[2,0,0,1]=(1.43311449551)/(o+1.)+(1.13832894216)+(-0.588282108798)*0.5**o
        ref[2,0,1,0]=(-0.507527909815)/(o+1.)+(0.342263613646)+(0.0572335623442)*0.5**o
        ref[2,0,1,1]=(0.762893809377)/(o+1.)+(-0.120387231036)+(0.832496085695)*0.5**o
        ref[2,0,2,0]=(-0.100527276479)/(o+1.)+(0.69346218236)+(0.140970192196)*0.5**o
        ref[2,0,2,1]=(1.06838752686)/(o+1.)+(0.0930203522642)+(-0.686218790734)*0.5**o
        ref[2,1,0,0]=(-0.28176535885)/(o+1.)+(0.113205031441)+(0.787423884494)*0.5**o
        ref[2,1,0,1]=(0.0863593391922)/(o+1.)+(-0.487715628853)+(-0.597301631508)*0.5**o
        ref[2,1,1,0]=(0.804711776497)/(o+1.)+(-0.6415801678)+(-0.393486075452)*0.5**o
        ref[2,1,1,1]=(0.0404422495423)/(o+1.)+(0.28725407529)+(0.547747813175)*0.5**o
        ref[2,1,2,0]=(0.897644174054)/(o+1.)+(0.449159193497)+(-0.289583636666)*0.5**o
        ref[2,1,2,1]=(-0.00909125785473)/(o+1.)+(0.233455690063)+(-0.487223003118)*0.5**o
        ref[2,2,0,0]=(0.428705598411)/(o+1.)+(-0.537564631337)+(0.31746495834)*0.5**o
        ref[2,2,0,1]=(-0.13006424301)/(o+1.)+(-0.227472916892)+(0.313462846078)*0.5**o
        ref[2,2,1,0]=(0.678520026185)/(o+1.)+(-0.185723681464)+(-0.180472378556)*0.5**o
        ref[2,2,1,1]=(0.118994939723)/(o+1.)+(-0.668711855142)+(0.303787742688)*0.5**o
        ref[2,2,2,0]=(1.32140542293)/(o+1.)+(-0.257035701286)+(-0.850284190461)*0.5**o
        ref[2,2,2,1]=(0.0420170266993)/(o+1.)+(-0.673170855967)+(0.559527478503)*0.5**o
        ref[2,3,0,0]=(-0.0660736057092)/(o+1.)+(-0.0611877561056)+(-0.274260374984)*0.5**o
        ref[2,3,0,1]=(1.05394255354)/(o+1.)+(0.306998434354)+(-0.793218219979)*0.5**o
        ref[2,3,1,0]=(0.046955729107)/(o+1.)+(-0.599656439191)+(0.411400569812)*0.5**o
        ref[2,3,1,1]=(-0.220516235917)/(o+1.)+(-0.290014707635)+(-0.491445354697)*0.5**o
        ref[2,3,2,0]=(-1.58612153753)/(o+1.)+(-0.698081139059)+(-0.806859723517)*0.5**o
        ref[2,3,2,1]=(-1.9837742327)/(o+1.)+(0.195194695368)+(-0.313979805844)*0.5**o
        ref[2,4,0,0]=(-0.546257320269)/(o+1.)+(-0.303248190467)+(-0.41348240167)*0.5**o
        ref[2,4,0,1]=(-0.995670500932)/(o+1.)+(-0.734851220921)+(0.0222534371058)*0.5**o
        ref[2,4,1,0]=(0.0147835590923)/(o+1.)+(0.147900377243)+(-0.750995516053)*0.5**o
        ref[2,4,1,1]=(0.530844875854)/(o+1.)+(1.05959549796)+(0.321767371741)*0.5**o
        ref[2,4,2,0]=(0.418464886946)/(o+1.)+(0.421412990871)+(-0.519508249649)*0.5**o
        ref[2,4,2,1]=(-1.10056362264)/(o+1.)+(0.852247961116)+(0.807909049346)*0.5**o
        ref[3,0,0,0]=(-0.0482378404454)/(o+1.)+(-0.418008150216)+(-0.780895567041)*0.5**o
        ref[3,0,0,1]=(-1.20294822484)/(o+1.)+(0.957059008835)+(-0.831076704037)*0.5**o
        ref[3,0,1,0]=(-0.459579079427)/(o+1.)+(0.925046639422)+(-0.0127541044953)*0.5**o
        ref[3,0,1,1]=(-0.00238959418047)/(o+1.)+(0.28308053827)+(-0.594972266882)*0.5**o
        ref[3,0,2,0]=(0.790992784744)/(o+1.)+(-0.524896739222)+(-0.715711146058)*0.5**o
        ref[3,0,2,1]=(-1.61712922959)/(o+1.)+(0.794245867809)+(0.306251544915)*0.5**o
        ref[3,1,0,0]=(0.318154478777)/(o+1.)+(-0.807135987793)+(-0.86788221871)*0.5**o
        ref[3,1,0,1]=(0.526003984691)/(o+1.)+(0.196391402369)+(-0.353678394652)*0.5**o
        ref[3,1,1,0]=(-0.395786718317)/(o+1.)+(-0.456262515411)+(-0.92646825221)*0.5**o
        ref[3,1,1,1]=(0.606897862508)/(o+1.)+(0.082987256486)+(-0.824273737615)*0.5**o
        ref[3,1,2,0]=(1.02930743983)/(o+1.)+(0.402256832916)+(0.807756928769)*0.5**o
        ref[3,1,2,1]=(0.225821980128)/(o+1.)+(-0.96267951509)+(-0.494145142543)*0.5**o
        ref[3,2,0,0]=(0.681876997651)/(o+1.)+(-0.556615389596)+(0.970345295013)*0.5**o
        ref[3,2,0,1]=(0.453098309344)/(o+1.)+(-0.0763995806862)+(0.692377432429)*0.5**o
        ref[3,2,1,0]=(-1.86665399006)/(o+1.)+(0.133524041831)+(0.152417917522)*0.5**o
        ref[3,2,1,1]=(-1.02702193024)/(o+1.)+(0.267356405114)+(0.238845792119)*0.5**o
        ref[3,2,2,0]=(-0.220118728954)/(o+1.)+(0.685757374584)+(0.456386394533)*0.5**o
        ref[3,2,2,1]=(-0.850398076225)/(o+1.)+(-0.130051257782)+(0.53299212624)*0.5**o
        ref[3,3,0,0]=(0.704508935183)/(o+1.)+(-0.519329921183)+(0.60015312936)*0.5**o
        ref[3,3,0,1]=(0.268880784666)/(o+1.)+(-0.10431965405)+(-0.597481072133)*0.5**o
        ref[3,3,1,0]=(0.595186324123)/(o+1.)+(0.13037754484)+(0.0921069092744)*0.5**o
        ref[3,3,1,1]=(0.0662157294518)/(o+1.)+(-0.250684686973)+(0.349078696154)*0.5**o
        ref[3,3,2,0]=(0.156645853389)/(o+1.)+(0.84327722735)+(-0.476684936147)*0.5**o
        ref[3,3,2,1]=(0.994145880767)/(o+1.)+(-0.465067169976)+(0.214713426286)*0.5**o
        ref[3,4,0,0]=(-0.990588506513)/(o+1.)+(0.218519355704)+(-0.88715423933)*0.5**o
        ref[3,4,0,1]=(-0.665732898082)/(o+1.)+(-0.496707601172)+(0.511114986801)*0.5**o
        ref[3,4,1,0]=(-1.18557015024)/(o+1.)+(0.587439848502)+(0.0739878219028)*0.5**o
        ref[3,4,1,1]=(0.725660134271)/(o+1.)+(0.786291076666)+(-0.141324267565)*0.5**o
        ref[3,4,2,0]=(-0.448948566381)/(o+1.)+(0.160689988193)+(-0.76912732001)*0.5**o
        ref[3,4,2,1]=(-0.571124298719)/(o+1.)+(-0.232553078069)+(-0.515945940217)*0.5**o
      res=integrate(arg)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactZero_fromSymbol_ContinuousFunction_rank0(self):
      """
      tests integral of rank 0 Symbol on the ReducedFunctionOnContactZero

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=())
      s=Data(0,(),w)
      if dim==2:
        s=(-0.239048993811)*x[0]**o+(0.519530081152)*x[0]+(0.0639850840479)*x[1]**o+(0.629919962063)*x[1]
        ref=(0.0639850840479)/(o+1.)+(0.574725021608)+(-0.239048993811)*0.5**o
      else:
        s=(0.129836073672)*x[0]**o+(0.77979793904)*x[0]+(0.153605570913)*x[1]**o+(0.182457449004)*x[1]+(-0.363742743784)*x[2]**o+(-0.826028359978)*x[2]
        ref=(-0.210137172871)/(o+1.)+(0.068113514033)+(0.129836073672)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactZero_fromSymbol_ContinuousFunction_rank1(self):
      """
      tests integral of rank 1 Symbol on the ReducedFunctionOnContactZero

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(2,))
      s=Data(0,(2,),w)
      ref=numpy.zeros((2,),numpy.float_)
      if dim==2:
        s[0]=(-0.0535727754535)*x[0]**o+(-0.378911132266)*x[0]+(0.893236107106)*x[1]**o+(-0.133308761371)*x[1]
        s[1]=(0.816344010178)*x[0]**o+(-0.629551729403)*x[0]+(0.962635839824)*x[1]**o+(0.531091887731)*x[1]
        ref[0]=(0.893236107106)/(o+1.)+(-0.256109946819)+(-0.0535727754535)*0.5**o
        ref[1]=(0.962635839824)/(o+1.)+(-0.049229920836)+(0.816344010178)*0.5**o
      else:
        s[0]=(0.99651578956)*x[0]**o+(-0.450391117128)*x[0]+(-0.0521202081325)*x[1]**o+(0.409486612049)*x[1]+(0.951839775929)*x[2]**o+(0.0755155386131)*x[2]
        s[1]=(-0.226900637285)*x[0]**o+(-0.784888192711)*x[0]+(-0.475467162806)*x[1]**o+(0.536559857164)*x[1]+(0.346844399644)*x[2]**o+(-0.237869505536)*x[2]
        ref[0]=(0.899719567796)/(o+1.)+(0.0173055167672)+(0.99651578956)*0.5**o
        ref[1]=(-0.128622763161)/(o+1.)+(-0.243098920542)+(-0.226900637285)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactZero_fromSymbol_ContinuousFunction_rank2(self):
      """
      tests integral of rank 2 Symbol on the ReducedFunctionOnContactZero

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5))
      s=Data(0,(4, 5),w)
      ref=numpy.zeros((4, 5),numpy.float_)
      if dim==2:
        s[0,0]=(-0.328426418082)*x[0]**o+(-0.966431499834)*x[0]+(0.982530526624)*x[1]**o+(0.475372684178)*x[1]
        s[0,1]=(0.61075753424)*x[0]**o+(-0.974986578257)*x[0]+(-0.0986707750313)*x[1]**o+(-0.490145139235)*x[1]
        s[0,2]=(-0.629292138329)*x[0]**o+(-0.784677771399)*x[0]+(0.662986672949)*x[1]**o+(-0.0634716139549)*x[1]
        s[0,3]=(0.74966960207)*x[0]**o+(-0.672137973696)*x[0]+(-0.104345950525)*x[1]**o+(-0.196646103769)*x[1]
        s[0,4]=(0.714327500249)*x[0]**o+(-0.454839918323)*x[0]+(-0.904749225664)*x[1]**o+(0.829036667249)*x[1]
        s[1,0]=(-0.290327744474)*x[0]**o+(-0.0432653521002)*x[0]+(-0.814637993515)*x[1]**o+(0.329873273442)*x[1]
        s[1,1]=(-0.256944455249)*x[0]**o+(-0.0374267372088)*x[0]+(0.136172411148)*x[1]**o+(-0.82944305625)*x[1]
        s[1,2]=(-0.260661151595)*x[0]**o+(-0.597199898207)*x[0]+(0.909049968542)*x[1]**o+(-0.262195958725)*x[1]
        s[1,3]=(-0.228035815224)*x[0]**o+(0.772135534405)*x[0]+(0.00192609783775)*x[1]**o+(-0.855606696367)*x[1]
        s[1,4]=(0.824519454106)*x[0]**o+(0.723545812848)*x[0]+(0.588395672453)*x[1]**o+(0.701489329605)*x[1]
        s[2,0]=(0.104767951487)*x[0]**o+(0.0937742673825)*x[0]+(-0.226378722473)*x[1]**o+(-0.239638024937)*x[1]
        s[2,1]=(0.922749632743)*x[0]**o+(0.380185381588)*x[0]+(-0.873869266649)*x[1]**o+(-0.995526870666)*x[1]
        s[2,2]=(0.459628441315)*x[0]**o+(-0.870827776883)*x[0]+(0.963527703404)*x[1]**o+(-0.380921690386)*x[1]
        s[2,3]=(0.0538635635094)*x[0]**o+(0.636269518871)*x[0]+(0.496494434174)*x[1]**o+(0.389503037841)*x[1]
        s[2,4]=(0.486584334747)*x[0]**o+(0.595962829728)*x[0]+(-0.0473827215975)*x[1]**o+(-0.0598623245737)*x[1]
        s[3,0]=(0.104809543499)*x[0]**o+(-0.103399959545)*x[0]+(0.46274253883)*x[1]**o+(0.00540100368281)*x[1]
        s[3,1]=(0.182366414916)*x[0]**o+(-0.454996760913)*x[0]+(0.195895079357)*x[1]**o+(-0.445919339196)*x[1]
        s[3,2]=(-0.270350002842)*x[0]**o+(0.0801494193318)*x[0]+(-0.192008845814)*x[1]**o+(0.20055215786)*x[1]
        s[3,3]=(0.946634950526)*x[0]**o+(0.794617946132)*x[0]+(0.266843824183)*x[1]**o+(0.431654105192)*x[1]
        s[3,4]=(-0.282961386823)*x[0]**o+(-0.659458559302)*x[0]+(-0.701137505881)*x[1]**o+(0.419244903032)*x[1]
        ref[0,0]=(0.982530526624)/(o+1.)+(-0.245529407828)+(-0.328426418082)*0.5**o
        ref[0,1]=(-0.0986707750313)/(o+1.)+(-0.732565858746)+(0.61075753424)*0.5**o
        ref[0,2]=(0.662986672949)/(o+1.)+(-0.424074692677)+(-0.629292138329)*0.5**o
        ref[0,3]=(-0.104345950525)/(o+1.)+(-0.434392038732)+(0.74966960207)*0.5**o
        ref[0,4]=(-0.904749225664)/(o+1.)+(0.187098374463)+(0.714327500249)*0.5**o
        ref[1,0]=(-0.814637993515)/(o+1.)+(0.143303960671)+(-0.290327744474)*0.5**o
        ref[1,1]=(0.136172411148)/(o+1.)+(-0.433434896729)+(-0.256944455249)*0.5**o
        ref[1,2]=(0.909049968542)/(o+1.)+(-0.429697928466)+(-0.260661151595)*0.5**o
        ref[1,3]=(0.00192609783775)/(o+1.)+(-0.0417355809806)+(-0.228035815224)*0.5**o
        ref[1,4]=(0.588395672453)/(o+1.)+(0.712517571227)+(0.824519454106)*0.5**o
        ref[2,0]=(-0.226378722473)/(o+1.)+(-0.0729318787773)+(0.104767951487)*0.5**o
        ref[2,1]=(-0.873869266649)/(o+1.)+(-0.307670744539)+(0.922749632743)*0.5**o
        ref[2,2]=(0.963527703404)/(o+1.)+(-0.625874733635)+(0.459628441315)*0.5**o
        ref[2,3]=(0.496494434174)/(o+1.)+(0.512886278356)+(0.0538635635094)*0.5**o
        ref[2,4]=(-0.0473827215975)/(o+1.)+(0.268050252577)+(0.486584334747)*0.5**o
        ref[3,0]=(0.46274253883)/(o+1.)+(-0.0489994779311)+(0.104809543499)*0.5**o
        ref[3,1]=(0.195895079357)/(o+1.)+(-0.450458050054)+(0.182366414916)*0.5**o
        ref[3,2]=(-0.192008845814)/(o+1.)+(0.140350788596)+(-0.270350002842)*0.5**o
        ref[3,3]=(0.266843824183)/(o+1.)+(0.613136025662)+(0.946634950526)*0.5**o
        ref[3,4]=(-0.701137505881)/(o+1.)+(-0.120106828135)+(-0.282961386823)*0.5**o
      else:
        s[0,0]=(-0.544243335601)*x[0]**o+(-0.307372154904)*x[0]+(0.244525574061)*x[1]**o+(-0.528694523489)*x[1]+(0.667705852103)*x[2]**o+(0.866378210585)*x[2]
        s[0,1]=(-0.397733775219)*x[0]**o+(0.195969196217)*x[0]+(-0.941371407192)*x[1]**o+(0.524704832845)*x[1]+(-0.973723558169)*x[2]**o+(0.0452111700317)*x[2]
        s[0,2]=(-0.0445896420815)*x[0]**o+(-0.730345503814)*x[0]+(-0.797148073969)*x[1]**o+(0.687229609119)*x[1]+(-0.905358686985)*x[2]**o+(-0.764008239213)*x[2]
        s[0,3]=(0.803277368887)*x[0]**o+(0.716534949109)*x[0]+(-0.138102366205)*x[1]**o+(-0.0536511450001)*x[1]+(-0.608245637044)*x[2]**o+(-0.922294317712)*x[2]
        s[0,4]=(-0.802690014092)*x[0]**o+(0.397998151759)*x[0]+(0.209143804619)*x[1]**o+(0.171210888095)*x[1]+(0.945371372346)*x[2]**o+(0.143657498886)*x[2]
        s[1,0]=(-0.737094926707)*x[0]**o+(-0.682774905121)*x[0]+(-0.605040082549)*x[1]**o+(0.913015529243)*x[1]+(-0.923053380199)*x[2]**o+(0.488575946117)*x[2]
        s[1,1]=(-0.19024610752)*x[0]**o+(0.112566519608)*x[0]+(0.631915593649)*x[1]**o+(0.613460770368)*x[1]+(-0.145279640801)*x[2]**o+(0.391904990063)*x[2]
        s[1,2]=(0.503053165032)*x[0]**o+(0.150421546144)*x[0]+(-0.975785035648)*x[1]**o+(0.358360240583)*x[1]+(-0.116051949436)*x[2]**o+(0.22565570175)*x[2]
        s[1,3]=(-0.183734715218)*x[0]**o+(0.397188558255)*x[0]+(0.621875746076)*x[1]**o+(-0.683761765859)*x[1]+(-0.964066487584)*x[2]**o+(0.572293215139)*x[2]
        s[1,4]=(0.308876282691)*x[0]**o+(0.163499825051)*x[0]+(-0.859132579068)*x[1]**o+(-0.133046304211)*x[1]+(-0.995751005727)*x[2]**o+(0.256192590636)*x[2]
        s[2,0]=(-0.823620775877)*x[0]**o+(-0.173531059768)*x[0]+(-0.428278048825)*x[1]**o+(-0.935056836732)*x[1]+(-0.16471171795)*x[2]**o+(-0.0746252552822)*x[2]
        s[2,1]=(0.692340272178)*x[0]**o+(-0.178862964433)*x[0]+(0.241112494059)*x[1]**o+(-0.0400607170557)*x[1]+(-0.131477945619)*x[2]**o+(-0.742303256353)*x[2]
        s[2,2]=(-0.213693977195)*x[0]**o+(0.829007036928)*x[0]+(0.269322769984)*x[1]**o+(0.988636255935)*x[1]+(0.145550246384)*x[2]**o+(0.53131223825)*x[2]
        s[2,3]=(0.337485288934)*x[0]**o+(-0.794235442944)*x[0]+(-0.862195558251)*x[1]**o+(-0.302947127951)*x[1]+(0.660574358405)*x[2]**o+(0.383147294118)*x[2]
        s[2,4]=(-0.0555202855446)*x[0]**o+(-0.298089725164)*x[0]+(-0.971009290556)*x[1]**o+(-0.867265062556)*x[1]+(0.669532302056)*x[2]**o+(-0.432375844991)*x[2]
        s[3,0]=(0.596097084223)*x[0]**o+(-0.388031691584)*x[0]+(-0.596718750677)*x[1]**o+(0.196346996307)*x[1]+(0.0230334505824)*x[2]**o+(-0.293224683649)*x[2]
        s[3,1]=(-0.855920365774)*x[0]**o+(-0.0363724823656)*x[0]+(-0.918160096067)*x[1]**o+(-0.275555163751)*x[1]+(-0.90266927)*x[2]**o+(-0.275537041793)*x[2]
        s[3,2]=(0.788923009274)*x[0]**o+(-0.182338499866)*x[0]+(0.649442441856)*x[1]**o+(-0.179319161548)*x[1]+(-0.330240080654)*x[2]**o+(0.45030421169)*x[2]
        s[3,3]=(-0.0731084424823)*x[0]**o+(0.903713856354)*x[0]+(0.749750641159)*x[1]**o+(-0.274337831625)*x[1]+(0.667970754626)*x[2]**o+(-0.369139994984)*x[2]
        s[3,4]=(0.0580656964124)*x[0]**o+(0.30223985624)*x[0]+(-0.0278202819133)*x[1]**o+(-0.214845008141)*x[1]+(0.683342885793)*x[2]**o+(0.920939676772)*x[2]
        ref[0,0]=(0.912231426164)/(o+1.)+(0.0151557660955)+(-0.544243335601)*0.5**o
        ref[0,1]=(-1.91509496536)/(o+1.)+(0.382942599547)+(-0.397733775219)*0.5**o
        ref[0,2]=(-1.70250676095)/(o+1.)+(-0.403562066955)+(-0.0445896420815)*0.5**o
        ref[0,3]=(-0.746348003249)/(o+1.)+(-0.129705256801)+(0.803277368887)*0.5**o
        ref[0,4]=(1.15451517696)/(o+1.)+(0.35643326937)+(-0.802690014092)*0.5**o
        ref[1,0]=(-1.52809346275)/(o+1.)+(0.359408285119)+(-0.737094926707)*0.5**o
        ref[1,1]=(0.486635952848)/(o+1.)+(0.558966140019)+(-0.19024610752)*0.5**o
        ref[1,2]=(-1.09183698508)/(o+1.)+(0.367218744238)+(0.503053165032)*0.5**o
        ref[1,3]=(-0.342190741508)/(o+1.)+(0.142860003768)+(-0.183734715218)*0.5**o
        ref[1,4]=(-1.8548835848)/(o+1.)+(0.143323055738)+(0.308876282691)*0.5**o
        ref[2,0]=(-0.592989766775)/(o+1.)+(-0.591606575891)+(-0.823620775877)*0.5**o
        ref[2,1]=(0.109634548439)/(o+1.)+(-0.480613468921)+(0.692340272178)*0.5**o
        ref[2,2]=(0.414873016368)/(o+1.)+(1.17447776556)+(-0.213693977195)*0.5**o
        ref[2,3]=(-0.201621199846)/(o+1.)+(-0.357017638388)+(0.337485288934)*0.5**o
        ref[2,4]=(-0.3014769885)/(o+1.)+(-0.798865316355)+(-0.0555202855446)*0.5**o
        ref[3,0]=(-0.573685300094)/(o+1.)+(-0.242454689463)+(0.596097084223)*0.5**o
        ref[3,1]=(-1.82082936607)/(o+1.)+(-0.293732343955)+(-0.855920365774)*0.5**o
        ref[3,2]=(0.319202361202)/(o+1.)+(0.044323275138)+(0.788923009274)*0.5**o
        ref[3,3]=(1.41772139578)/(o+1.)+(0.130118014873)+(-0.0731084424823)*0.5**o
        ref[3,4]=(0.65552260388)/(o+1.)+(0.504167262435)+(0.0580656964124)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactZero_fromSymbol_ContinuousFunction_rank3(self):
      """
      tests integral of rank 3 Symbol on the ReducedFunctionOnContactZero

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(6, 2, 2))
      s=Data(0,(6, 2, 2),w)
      ref=numpy.zeros((6, 2, 2),numpy.float_)
      if dim==2:
        s[0,0,0]=(-0.2538014101)*x[0]**o+(0.422893022023)*x[0]+(-0.168009677754)*x[1]**o+(0.967205152872)*x[1]
        s[0,0,1]=(-0.42169068503)*x[0]**o+(-0.87035047637)*x[0]+(0.861757151221)*x[1]**o+(0.233579078048)*x[1]
        s[0,1,0]=(0.761004021853)*x[0]**o+(-0.406910518681)*x[0]+(-0.653627037953)*x[1]**o+(0.78163950747)*x[1]
        s[0,1,1]=(0.571752563364)*x[0]**o+(0.352647230887)*x[0]+(0.64023828193)*x[1]**o+(-0.906400256269)*x[1]
        s[1,0,0]=(-0.229152209793)*x[0]**o+(-0.164526860803)*x[0]+(-0.815301528075)*x[1]**o+(-0.375201173874)*x[1]
        s[1,0,1]=(0.497997942477)*x[0]**o+(-0.590120034514)*x[0]+(0.387642938322)*x[1]**o+(-0.441598030675)*x[1]
        s[1,1,0]=(-0.396407524403)*x[0]**o+(0.225115161973)*x[0]+(0.709660265053)*x[1]**o+(-0.723548390356)*x[1]
        s[1,1,1]=(-0.810340425309)*x[0]**o+(0.571580569899)*x[0]+(0.401013753168)*x[1]**o+(0.881936918624)*x[1]
        s[2,0,0]=(0.096907527902)*x[0]**o+(-0.329521007808)*x[0]+(0.872734305266)*x[1]**o+(-0.677632080143)*x[1]
        s[2,0,1]=(-0.657791763259)*x[0]**o+(-0.0622202818891)*x[0]+(0.838133724288)*x[1]**o+(0.679646081219)*x[1]
        s[2,1,0]=(-0.982546119133)*x[0]**o+(0.0982803558898)*x[0]+(0.188605842026)*x[1]**o+(-0.436341762577)*x[1]
        s[2,1,1]=(0.623965743731)*x[0]**o+(-0.470311605613)*x[0]+(-0.700736177085)*x[1]**o+(0.688725936352)*x[1]
        s[3,0,0]=(0.154576108414)*x[0]**o+(-0.114669097915)*x[0]+(-0.922751077939)*x[1]**o+(-0.0336084204309)*x[1]
        s[3,0,1]=(-0.360543465066)*x[0]**o+(0.369432311166)*x[0]+(0.56460991873)*x[1]**o+(-0.56949007527)*x[1]
        s[3,1,0]=(0.914890366289)*x[0]**o+(0.216556566085)*x[0]+(-0.720400002066)*x[1]**o+(-0.44569085463)*x[1]
        s[3,1,1]=(0.0855171738813)*x[0]**o+(-0.47318307368)*x[0]+(0.267072371169)*x[1]**o+(-0.861793396744)*x[1]
        s[4,0,0]=(-0.862522626201)*x[0]**o+(0.101759724363)*x[0]+(-0.843728984752)*x[1]**o+(-0.510998623672)*x[1]
        s[4,0,1]=(0.827712219176)*x[0]**o+(-0.83052404804)*x[0]+(-0.349511963621)*x[1]**o+(0.379838260239)*x[1]
        s[4,1,0]=(0.923120645339)*x[0]**o+(-0.310138654901)*x[0]+(-0.0364015485007)*x[1]**o+(-0.869695736822)*x[1]
        s[4,1,1]=(0.462952706773)*x[0]**o+(0.391666695595)*x[0]+(0.918559046828)*x[1]**o+(0.0448293018515)*x[1]
        s[5,0,0]=(0.708616542984)*x[0]**o+(-0.826033306645)*x[0]+(0.50975079739)*x[1]**o+(-0.213839768114)*x[1]
        s[5,0,1]=(0.874170725144)*x[0]**o+(0.128033874112)*x[0]+(-0.236784782085)*x[1]**o+(0.885355734442)*x[1]
        s[5,1,0]=(0.0315543075879)*x[0]**o+(-0.482419902735)*x[0]+(0.112741960603)*x[1]**o+(0.336157525634)*x[1]
        s[5,1,1]=(0.000481165869329)*x[0]**o+(-0.356220818813)*x[0]+(-0.466568234378)*x[1]**o+(0.851934710295)*x[1]
        ref[0,0,0]=(-0.168009677754)/(o+1.)+(0.695049087447)+(-0.2538014101)*0.5**o
        ref[0,0,1]=(0.861757151221)/(o+1.)+(-0.318385699161)+(-0.42169068503)*0.5**o
        ref[0,1,0]=(-0.653627037953)/(o+1.)+(0.187364494395)+(0.761004021853)*0.5**o
        ref[0,1,1]=(0.64023828193)/(o+1.)+(-0.276876512691)+(0.571752563364)*0.5**o
        ref[1,0,0]=(-0.815301528075)/(o+1.)+(-0.269864017338)+(-0.229152209793)*0.5**o
        ref[1,0,1]=(0.387642938322)/(o+1.)+(-0.515859032595)+(0.497997942477)*0.5**o
        ref[1,1,0]=(0.709660265053)/(o+1.)+(-0.249216614192)+(-0.396407524403)*0.5**o
        ref[1,1,1]=(0.401013753168)/(o+1.)+(0.726758744261)+(-0.810340425309)*0.5**o
        ref[2,0,0]=(0.872734305266)/(o+1.)+(-0.503576543976)+(0.096907527902)*0.5**o
        ref[2,0,1]=(0.838133724288)/(o+1.)+(0.308712899665)+(-0.657791763259)*0.5**o
        ref[2,1,0]=(0.188605842026)/(o+1.)+(-0.169030703344)+(-0.982546119133)*0.5**o
        ref[2,1,1]=(-0.700736177085)/(o+1.)+(0.10920716537)+(0.623965743731)*0.5**o
        ref[3,0,0]=(-0.922751077939)/(o+1.)+(-0.0741387591729)+(0.154576108414)*0.5**o
        ref[3,0,1]=(0.56460991873)/(o+1.)+(-0.100028882052)+(-0.360543465066)*0.5**o
        ref[3,1,0]=(-0.720400002066)/(o+1.)+(-0.114567144272)+(0.914890366289)*0.5**o
        ref[3,1,1]=(0.267072371169)/(o+1.)+(-0.667488235212)+(0.0855171738813)*0.5**o
        ref[4,0,0]=(-0.843728984752)/(o+1.)+(-0.204619449655)+(-0.862522626201)*0.5**o
        ref[4,0,1]=(-0.349511963621)/(o+1.)+(-0.2253428939)+(0.827712219176)*0.5**o
        ref[4,1,0]=(-0.0364015485007)/(o+1.)+(-0.589917195861)+(0.923120645339)*0.5**o
        ref[4,1,1]=(0.918559046828)/(o+1.)+(0.218247998723)+(0.462952706773)*0.5**o
        ref[5,0,0]=(0.50975079739)/(o+1.)+(-0.51993653738)+(0.708616542984)*0.5**o
        ref[5,0,1]=(-0.236784782085)/(o+1.)+(0.506694804277)+(0.874170725144)*0.5**o
        ref[5,1,0]=(0.112741960603)/(o+1.)+(-0.0731311885505)+(0.0315543075879)*0.5**o
        ref[5,1,1]=(-0.466568234378)/(o+1.)+(0.247856945741)+(0.000481165869329)*0.5**o
      else:
        s[0,0,0]=(0.712776413788)*x[0]**o+(-0.232827372726)*x[0]+(0.504361060322)*x[1]**o+(-0.646213742366)*x[1]+(-0.292404315781)*x[2]**o+(0.0934342710289)*x[2]
        s[0,0,1]=(-0.6231391885)*x[0]**o+(-0.201681226437)*x[0]+(0.295572186857)*x[1]**o+(0.490652427459)*x[1]+(-0.816764035717)*x[2]**o+(-0.920253719103)*x[2]
        s[0,1,0]=(0.464396400115)*x[0]**o+(0.256116225806)*x[0]+(0.00347318217819)*x[1]**o+(0.290749204508)*x[1]+(0.755673392263)*x[2]**o+(0.0296476243926)*x[2]
        s[0,1,1]=(-0.901282392808)*x[0]**o+(0.390894518003)*x[0]+(-0.638342109049)*x[1]**o+(0.325572670945)*x[1]+(0.011997038614)*x[2]**o+(-0.953246898018)*x[2]
        s[1,0,0]=(-0.234152998295)*x[0]**o+(-0.686879447381)*x[0]+(0.684513221814)*x[1]**o+(0.314162707683)*x[1]+(-0.140878044122)*x[2]**o+(0.588563624841)*x[2]
        s[1,0,1]=(0.00302398956565)*x[0]**o+(-0.255603279589)*x[0]+(-0.356254001427)*x[1]**o+(-0.00335409560896)*x[1]+(0.761527454384)*x[2]**o+(-0.803864763242)*x[2]
        s[1,1,0]=(0.350958444218)*x[0]**o+(0.626399553048)*x[0]+(-0.804525479633)*x[1]**o+(0.999024446128)*x[1]+(-0.747418498661)*x[2]**o+(0.16538209042)*x[2]
        s[1,1,1]=(0.830959844328)*x[0]**o+(-0.838517518014)*x[0]+(-0.815601516751)*x[1]**o+(-0.527962402292)*x[1]+(0.849695437437)*x[2]**o+(0.0461655137013)*x[2]
        s[2,0,0]=(-0.654485309003)*x[0]**o+(-0.922437021301)*x[0]+(0.0726219637303)*x[1]**o+(-0.27358467284)*x[1]+(0.965940347168)*x[2]**o+(0.136286409807)*x[2]
        s[2,0,1]=(-0.765669487668)*x[0]**o+(-0.381631759049)*x[0]+(-0.338604617382)*x[1]**o+(-0.026416557321)*x[1]+(-0.505094659996)*x[2]**o+(-0.109271565345)*x[2]
        s[2,1,0]=(-0.823307360412)*x[0]**o+(0.742603440907)*x[0]+(0.264688242655)*x[1]**o+(0.152850147645)*x[1]+(0.576103041841)*x[2]**o+(-0.5223182504)*x[2]
        s[2,1,1]=(-0.853043070027)*x[0]**o+(-0.841177435972)*x[0]+(-0.899106240107)*x[1]**o+(0.605850900198)*x[1]+(0.66690888915)*x[2]**o+(-0.294170728255)*x[2]
        s[3,0,0]=(0.570005513209)*x[0]**o+(0.823970182367)*x[0]+(-0.097784833249)*x[1]**o+(0.873052382528)*x[1]+(-0.0636165773808)*x[2]**o+(0.282310913291)*x[2]
        s[3,0,1]=(0.456449154672)*x[0]**o+(0.951161773651)*x[0]+(-0.373225942356)*x[1]**o+(-0.199594431547)*x[1]+(-0.484728313328)*x[2]**o+(-0.931817560358)*x[2]
        s[3,1,0]=(-0.41042836838)*x[0]**o+(0.771670661129)*x[0]+(-0.0646371268204)*x[1]**o+(0.881286289043)*x[1]+(-0.197285717492)*x[2]**o+(0.857484079398)*x[2]
        s[3,1,1]=(0.297402801452)*x[0]**o+(-0.147528679657)*x[0]+(0.0071150978913)*x[1]**o+(0.788738967825)*x[1]+(-0.191726852031)*x[2]**o+(-0.481649460835)*x[2]
        s[4,0,0]=(-0.808360899963)*x[0]**o+(-0.181264991463)*x[0]+(-0.667551398858)*x[1]**o+(0.289372159531)*x[1]+(0.732773385522)*x[2]**o+(-0.123945731785)*x[2]
        s[4,0,1]=(0.854370035772)*x[0]**o+(0.169945736996)*x[0]+(0.305712234323)*x[1]**o+(-0.60291919973)*x[1]+(0.0403778674069)*x[2]**o+(-0.0309870891437)*x[2]
        s[4,1,0]=(0.543462693144)*x[0]**o+(0.102574070937)*x[0]+(0.963469439068)*x[1]**o+(-0.985245165577)*x[1]+(-0.74408200708)*x[2]**o+(-0.146993364858)*x[2]
        s[4,1,1]=(-0.50927952968)*x[0]**o+(0.471050528827)*x[0]+(0.554105772637)*x[1]**o+(0.265977196333)*x[1]+(-0.447676667005)*x[2]**o+(-0.725761367739)*x[2]
        s[5,0,0]=(0.738549622917)*x[0]**o+(-0.300701131285)*x[0]+(0.770397993383)*x[1]**o+(0.604771638849)*x[1]+(-0.839284085987)*x[2]**o+(-0.636936520767)*x[2]
        s[5,0,1]=(0.546026750751)*x[0]**o+(-0.413469373764)*x[0]+(-0.246023997114)*x[1]**o+(0.207570581839)*x[1]+(0.447030203856)*x[2]**o+(0.0894542975251)*x[2]
        s[5,1,0]=(0.262793971997)*x[0]**o+(-0.814182314897)*x[0]+(-0.340418569268)*x[1]**o+(-0.483737889608)*x[1]+(0.287508080597)*x[2]**o+(-0.602853752221)*x[2]
        s[5,1,1]=(-0.417452810992)*x[0]**o+(-0.906502058242)*x[0]+(0.280357642399)*x[1]**o+(-0.452257960569)*x[1]+(0.239691800726)*x[2]**o+(0.0506798793624)*x[2]
        ref[0,0,0]=(0.211956744542)/(o+1.)+(-0.392803422032)+(0.712776413788)*0.5**o
        ref[0,0,1]=(-0.521191848861)/(o+1.)+(-0.31564125904)+(-0.6231391885)*0.5**o
        ref[0,1,0]=(0.759146574441)/(o+1.)+(0.288256527353)+(0.464396400115)*0.5**o
        ref[0,1,1]=(-0.626345070435)/(o+1.)+(-0.118389854535)+(-0.901282392808)*0.5**o
        ref[1,0,0]=(0.543635177691)/(o+1.)+(0.107923442572)+(-0.234152998295)*0.5**o
        ref[1,0,1]=(0.405273452958)/(o+1.)+(-0.53141106922)+(0.00302398956565)*0.5**o
        ref[1,1,0]=(-1.55194397829)/(o+1.)+(0.895403044797)+(0.350958444218)*0.5**o
        ref[1,1,1]=(0.0340939206865)/(o+1.)+(-0.660157203302)+(0.830959844328)*0.5**o
        ref[2,0,0]=(1.0385623109)/(o+1.)+(-0.529867642167)+(-0.654485309003)*0.5**o
        ref[2,0,1]=(-0.843699277378)/(o+1.)+(-0.258659940857)+(-0.765669487668)*0.5**o
        ref[2,1,0]=(0.840791284496)/(o+1.)+(0.186567669076)+(-0.823307360412)*0.5**o
        ref[2,1,1]=(-0.232197350956)/(o+1.)+(-0.264748632015)+(-0.853043070027)*0.5**o
        ref[3,0,0]=(-0.16140141063)/(o+1.)+(0.989666739093)+(0.570005513209)*0.5**o
        ref[3,0,1]=(-0.857954255684)/(o+1.)+(-0.0901251091271)+(0.456449154672)*0.5**o
        ref[3,1,0]=(-0.261922844312)/(o+1.)+(1.25522051478)+(-0.41042836838)*0.5**o
        ref[3,1,1]=(-0.18461175414)/(o+1.)+(0.0797804136672)+(0.297402801452)*0.5**o
        ref[4,0,0]=(0.0652219866638)/(o+1.)+(-0.00791928185857)+(-0.808360899963)*0.5**o
        ref[4,0,1]=(0.34609010173)/(o+1.)+(-0.231980275939)+(0.854370035772)*0.5**o
        ref[4,1,0]=(0.219387431988)/(o+1.)+(-0.514832229749)+(0.543462693144)*0.5**o
        ref[4,1,1]=(0.106429105632)/(o+1.)+(0.00563317871007)+(-0.50927952968)*0.5**o
        ref[5,0,0]=(-0.0688860926042)/(o+1.)+(-0.166433006601)+(0.738549622917)*0.5**o
        ref[5,0,1]=(0.201006206742)/(o+1.)+(-0.0582222472)+(0.546026750751)*0.5**o
        ref[5,1,0]=(-0.0529104886711)/(o+1.)+(-0.950386978363)+(0.262793971997)*0.5**o
        ref[5,1,1]=(0.520049443126)/(o+1.)+(-0.654040069724)+(-0.417452810992)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactZero_fromSymbol_ContinuousFunction_rank4(self):
      """
      tests integral of rank 4 Symbol on the ReducedFunctionOnContactZero

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5, 3, 2))
      s=Data(0,(4, 5, 3, 2),w)
      ref=numpy.zeros((4, 5, 3, 2),numpy.float_)
      if dim==2:
        s[0,0,0,0]=(-0.829288219058)*x[0]**o+(-0.65217791272)*x[0]+(0.698846288116)*x[1]**o+(-0.0392705140136)*x[1]
        s[0,0,0,1]=(-0.456023711562)*x[0]**o+(0.9183516109)*x[0]+(0.927490172078)*x[1]**o+(-0.394686077168)*x[1]
        s[0,0,1,0]=(-0.497067376291)*x[0]**o+(0.237428033612)*x[0]+(-0.785186724625)*x[1]**o+(0.471101666176)*x[1]
        s[0,0,1,1]=(-0.290568226175)*x[0]**o+(0.893982019035)*x[0]+(0.203073455207)*x[1]**o+(0.78973496755)*x[1]
        s[0,0,2,0]=(0.00829962077999)*x[0]**o+(0.99718958928)*x[0]+(0.330466533936)*x[1]**o+(-0.619185067306)*x[1]
        s[0,0,2,1]=(-0.806092658508)*x[0]**o+(-0.307457723596)*x[0]+(0.847169623314)*x[1]**o+(-0.832290917062)*x[1]
        s[0,1,0,0]=(-0.140461069383)*x[0]**o+(0.760902362673)*x[0]+(0.0231278168453)*x[1]**o+(-0.508849645358)*x[1]
        s[0,1,0,1]=(0.587581627616)*x[0]**o+(-0.229546331489)*x[0]+(-0.659235919696)*x[1]**o+(0.80409204008)*x[1]
        s[0,1,1,0]=(-0.641654760801)*x[0]**o+(0.122524921907)*x[0]+(0.122921572782)*x[1]**o+(-0.23094495884)*x[1]
        s[0,1,1,1]=(-0.380132178505)*x[0]**o+(-0.769451293694)*x[0]+(-0.745741833491)*x[1]**o+(0.11284350488)*x[1]
        s[0,1,2,0]=(0.126835269214)*x[0]**o+(0.918739367312)*x[0]+(0.927407682128)*x[1]**o+(-0.273231826206)*x[1]
        s[0,1,2,1]=(0.608413549196)*x[0]**o+(-0.591552902664)*x[0]+(0.873696757566)*x[1]**o+(-0.900225496133)*x[1]
        s[0,2,0,0]=(0.0488258290791)*x[0]**o+(0.211067968473)*x[0]+(0.11529432626)*x[1]**o+(0.497957233179)*x[1]
        s[0,2,0,1]=(-0.780993161707)*x[0]**o+(0.746546880275)*x[0]+(-0.0997128168301)*x[1]**o+(-0.663371860066)*x[1]
        s[0,2,1,0]=(0.305794159864)*x[0]**o+(0.493723656879)*x[0]+(0.334264621015)*x[1]**o+(0.0579972251793)*x[1]
        s[0,2,1,1]=(0.71588138307)*x[0]**o+(-0.0866536386387)*x[0]+(-0.159835910161)*x[1]**o+(0.562258753174)*x[1]
        s[0,2,2,0]=(0.864605462382)*x[0]**o+(0.818516795215)*x[0]+(-0.0558026806854)*x[1]**o+(-0.225953364341)*x[1]
        s[0,2,2,1]=(-0.0798994114092)*x[0]**o+(-0.73636091403)*x[0]+(-0.478543932177)*x[1]**o+(0.875789857891)*x[1]
        s[0,3,0,0]=(0.998019190199)*x[0]**o+(-0.218007434364)*x[0]+(0.181378057097)*x[1]**o+(0.429126350197)*x[1]
        s[0,3,0,1]=(-0.724306549649)*x[0]**o+(-0.303805025896)*x[0]+(0.58603690984)*x[1]**o+(0.976611908964)*x[1]
        s[0,3,1,0]=(0.776728843377)*x[0]**o+(-0.874157618694)*x[0]+(-0.590753603973)*x[1]**o+(0.185288488913)*x[1]
        s[0,3,1,1]=(0.285035031516)*x[0]**o+(-0.0667711315126)*x[0]+(0.220692714675)*x[1]**o+(0.841408314724)*x[1]
        s[0,3,2,0]=(-0.00286941650137)*x[0]**o+(0.59339349489)*x[0]+(-0.158356936936)*x[1]**o+(-0.451137261062)*x[1]
        s[0,3,2,1]=(0.37992478387)*x[0]**o+(0.835018328571)*x[0]+(-0.446900268053)*x[1]**o+(-0.700751807162)*x[1]
        s[0,4,0,0]=(0.714232167791)*x[0]**o+(-0.688473756667)*x[0]+(0.0552121131111)*x[1]**o+(0.632003397091)*x[1]
        s[0,4,0,1]=(0.49556700372)*x[0]**o+(0.0630560952096)*x[0]+(0.321456277553)*x[1]**o+(0.685060833092)*x[1]
        s[0,4,1,0]=(0.841677614054)*x[0]**o+(0.886957973836)*x[0]+(0.0202839586672)*x[1]**o+(-0.923265656841)*x[1]
        s[0,4,1,1]=(-0.504028346474)*x[0]**o+(0.071280762432)*x[0]+(0.629772140166)*x[1]**o+(-0.195290635886)*x[1]
        s[0,4,2,0]=(0.434265669856)*x[0]**o+(0.376229784992)*x[0]+(0.919945289519)*x[1]**o+(-0.496219699959)*x[1]
        s[0,4,2,1]=(-0.86013960593)*x[0]**o+(0.140206694208)*x[0]+(-0.834647694686)*x[1]**o+(0.381383127993)*x[1]
        s[1,0,0,0]=(-0.437686950383)*x[0]**o+(0.385307277242)*x[0]+(-0.430841927673)*x[1]**o+(-0.556445729508)*x[1]
        s[1,0,0,1]=(0.323843044435)*x[0]**o+(-0.21417604806)*x[0]+(-0.172390658038)*x[1]**o+(-0.847272401994)*x[1]
        s[1,0,1,0]=(-0.845882465152)*x[0]**o+(-0.598486605858)*x[0]+(0.89008620109)*x[1]**o+(0.874381441062)*x[1]
        s[1,0,1,1]=(-0.405423425934)*x[0]**o+(-0.404267054882)*x[0]+(-0.63801489002)*x[1]**o+(-0.728771425845)*x[1]
        s[1,0,2,0]=(-0.216267078676)*x[0]**o+(0.956804195842)*x[0]+(-0.908451202669)*x[1]**o+(-0.322043280117)*x[1]
        s[1,0,2,1]=(-0.168907777516)*x[0]**o+(0.840516962339)*x[0]+(0.103177353457)*x[1]**o+(0.939190271608)*x[1]
        s[1,1,0,0]=(-0.884189035361)*x[0]**o+(-0.396863590748)*x[0]+(-0.380355614494)*x[1]**o+(0.913843764972)*x[1]
        s[1,1,0,1]=(-0.983514772022)*x[0]**o+(0.953676778353)*x[0]+(-0.141666532996)*x[1]**o+(-0.782763806635)*x[1]
        s[1,1,1,0]=(0.723588279961)*x[0]**o+(0.0698022948292)*x[0]+(-0.202960085361)*x[1]**o+(-0.0971108181392)*x[1]
        s[1,1,1,1]=(-0.809610385863)*x[0]**o+(0.78262340178)*x[0]+(-0.292492437285)*x[1]**o+(0.909658049109)*x[1]
        s[1,1,2,0]=(0.831333270285)*x[0]**o+(0.532412110902)*x[0]+(0.852180039142)*x[1]**o+(-0.126550447094)*x[1]
        s[1,1,2,1]=(0.889635940128)*x[0]**o+(-0.0146231223219)*x[0]+(-0.505300606168)*x[1]**o+(0.286596962758)*x[1]
        s[1,2,0,0]=(-0.479209867816)*x[0]**o+(-0.707176716613)*x[0]+(0.385223744345)*x[1]**o+(0.679335984842)*x[1]
        s[1,2,0,1]=(-0.690494514078)*x[0]**o+(-0.591659781424)*x[0]+(-0.111527008468)*x[1]**o+(0.73261078188)*x[1]
        s[1,2,1,0]=(-0.65110579618)*x[0]**o+(-0.861397125883)*x[0]+(-0.264211841662)*x[1]**o+(0.750122373459)*x[1]
        s[1,2,1,1]=(-0.495114108462)*x[0]**o+(-0.555683103062)*x[0]+(-0.0384737773817)*x[1]**o+(0.422372598888)*x[1]
        s[1,2,2,0]=(-0.846365789548)*x[0]**o+(0.247402342046)*x[0]+(-0.998046573914)*x[1]**o+(0.264003327979)*x[1]
        s[1,2,2,1]=(0.4479853512)*x[0]**o+(-0.502771753169)*x[0]+(-0.366626482676)*x[1]**o+(0.506404378836)*x[1]
        s[1,3,0,0]=(-0.677771809341)*x[0]**o+(-0.105232399291)*x[0]+(0.10440523429)*x[1]**o+(-0.780931426435)*x[1]
        s[1,3,0,1]=(0.549864901432)*x[0]**o+(0.79598502718)*x[0]+(-0.424985507693)*x[1]**o+(-0.649629583228)*x[1]
        s[1,3,1,0]=(0.795465752618)*x[0]**o+(-0.0630573960436)*x[0]+(-0.158966662533)*x[1]**o+(0.888189443186)*x[1]
        s[1,3,1,1]=(-0.395120222846)*x[0]**o+(-0.529551413114)*x[0]+(-0.351127736099)*x[1]**o+(-0.960771541871)*x[1]
        s[1,3,2,0]=(-0.244306249618)*x[0]**o+(0.925164172251)*x[0]+(-0.988402656417)*x[1]**o+(-0.0797097670335)*x[1]
        s[1,3,2,1]=(0.905866127894)*x[0]**o+(0.135673487702)*x[0]+(-0.873223112612)*x[1]**o+(0.598121374326)*x[1]
        s[1,4,0,0]=(0.171338765926)*x[0]**o+(-0.31796084081)*x[0]+(-0.914462477805)*x[1]**o+(0.923691484317)*x[1]
        s[1,4,0,1]=(-0.188491880655)*x[0]**o+(-0.67119846649)*x[0]+(0.488643815471)*x[1]**o+(-0.912992530018)*x[1]
        s[1,4,1,0]=(0.252606322081)*x[0]**o+(0.509944026633)*x[0]+(0.539656085813)*x[1]**o+(0.0286599718948)*x[1]
        s[1,4,1,1]=(0.148934160341)*x[0]**o+(0.50451378885)*x[0]+(0.878910174211)*x[1]**o+(-0.617480145541)*x[1]
        s[1,4,2,0]=(-0.235447116973)*x[0]**o+(0.825149714177)*x[0]+(0.300052096347)*x[1]**o+(-0.360690924033)*x[1]
        s[1,4,2,1]=(0.944445188093)*x[0]**o+(-0.170511625527)*x[0]+(-0.333492913384)*x[1]**o+(0.907577674324)*x[1]
        s[2,0,0,0]=(-0.942092595372)*x[0]**o+(-0.254482071505)*x[0]+(-0.0151488336686)*x[1]**o+(-0.0287652780384)*x[1]
        s[2,0,0,1]=(0.0655506752825)*x[0]**o+(0.898719259761)*x[0]+(0.383024592178)*x[1]**o+(0.0989444595983)*x[1]
        s[2,0,1,0]=(0.291304803618)*x[0]**o+(-0.720915295421)*x[0]+(0.138378684)*x[1]**o+(0.876226100573)*x[1]
        s[2,0,1,1]=(0.0625773477714)*x[0]**o+(-0.0294480886227)*x[0]+(0.736449427619)*x[1]**o+(0.371697786988)*x[1]
        s[2,0,2,0]=(-0.931044858876)*x[0]**o+(0.911308988417)*x[0]+(0.689775520125)*x[1]**o+(0.415135153045)*x[1]
        s[2,0,2,1]=(-0.732200781532)*x[0]**o+(0.395135482591)*x[0]+(-0.619692129874)*x[1]**o+(-0.912759567625)*x[1]
        s[2,1,0,0]=(-0.261557477097)*x[0]**o+(-0.933044351058)*x[0]+(0.0156599006787)*x[1]**o+(-0.082399909281)*x[1]
        s[2,1,0,1]=(0.723444855088)*x[0]**o+(-0.216911506016)*x[0]+(0.604446335087)*x[1]**o+(0.947367685426)*x[1]
        s[2,1,1,0]=(-0.606511927315)*x[0]**o+(0.825992890197)*x[0]+(0.757190663205)*x[1]**o+(-0.0991622858024)*x[1]
        s[2,1,1,1]=(0.45110092033)*x[0]**o+(0.0571087683245)*x[0]+(-0.825169962155)*x[1]**o+(0.557059283915)*x[1]
        s[2,1,2,0]=(-0.612806058565)*x[0]**o+(0.928772523716)*x[0]+(-0.288177463065)*x[1]**o+(0.341361092846)*x[1]
        s[2,1,2,1]=(-0.746890321439)*x[0]**o+(0.893018453839)*x[0]+(0.0308905730956)*x[1]**o+(0.291585488925)*x[1]
        s[2,2,0,0]=(-0.665223434533)*x[0]**o+(-0.997256499342)*x[0]+(-0.134118374176)*x[1]**o+(-0.930692055226)*x[1]
        s[2,2,0,1]=(-0.478607794082)*x[0]**o+(0.0372165046416)*x[0]+(-0.149775511078)*x[1]**o+(-0.412243756318)*x[1]
        s[2,2,1,0]=(-0.782692588953)*x[0]**o+(0.896152570987)*x[0]+(-0.265361365664)*x[1]**o+(0.0859566508122)*x[1]
        s[2,2,1,1]=(0.141591460973)*x[0]**o+(0.399004634853)*x[0]+(0.351166595901)*x[1]**o+(-0.230314460514)*x[1]
        s[2,2,2,0]=(-0.282452862833)*x[0]**o+(0.652348758087)*x[0]+(-0.006664462988)*x[1]**o+(0.115435922234)*x[1]
        s[2,2,2,1]=(-0.108570066423)*x[0]**o+(0.536881567792)*x[0]+(-0.967086006781)*x[1]**o+(0.964315151846)*x[1]
        s[2,3,0,0]=(-0.872481061987)*x[0]**o+(-0.152036048838)*x[0]+(0.714460711637)*x[1]**o+(-0.641192704763)*x[1]
        s[2,3,0,1]=(-0.314322497313)*x[0]**o+(0.766601974446)*x[0]+(-0.304360730035)*x[1]**o+(-0.445351381604)*x[1]
        s[2,3,1,0]=(0.364419198839)*x[0]**o+(-0.615166967343)*x[0]+(0.797973787987)*x[1]**o+(0.312408283718)*x[1]
        s[2,3,1,1]=(-0.781147212189)*x[0]**o+(-0.549359753018)*x[0]+(0.235681715573)*x[1]**o+(-0.231862286881)*x[1]
        s[2,3,2,0]=(-0.958493367772)*x[0]**o+(-0.775968770096)*x[0]+(-0.917090286139)*x[1]**o+(0.656115811881)*x[1]
        s[2,3,2,1]=(0.572779229598)*x[0]**o+(-0.875089280859)*x[0]+(0.575188283953)*x[1]**o+(-0.617895968371)*x[1]
        s[2,4,0,0]=(-0.178437300104)*x[0]**o+(-0.305311887897)*x[0]+(-0.362423234235)*x[1]**o+(-0.993155475703)*x[1]
        s[2,4,0,1]=(-0.842863324823)*x[0]**o+(-0.334505517678)*x[0]+(-0.233874335956)*x[1]**o+(-0.280499554274)*x[1]
        s[2,4,1,0]=(-0.396341438458)*x[0]**o+(0.605087475544)*x[0]+(-0.648099915852)*x[1]**o+(-0.338407917903)*x[1]
        s[2,4,1,1]=(-0.359913098139)*x[0]**o+(0.80935120431)*x[0]+(-0.969248694111)*x[1]**o+(0.884624495846)*x[1]
        s[2,4,2,0]=(0.965251612654)*x[0]**o+(-0.811340257859)*x[0]+(0.960012562229)*x[1]**o+(-0.825791870471)*x[1]
        s[2,4,2,1]=(0.43888464177)*x[0]**o+(0.126116195477)*x[0]+(0.0455614359233)*x[1]**o+(0.387001710336)*x[1]
        s[3,0,0,0]=(0.961744925179)*x[0]**o+(0.590267645729)*x[0]+(-0.657096083564)*x[1]**o+(-0.336931534761)*x[1]
        s[3,0,0,1]=(0.0372329050631)*x[0]**o+(0.82966683846)*x[0]+(-0.537423647089)*x[1]**o+(0.13780696077)*x[1]
        s[3,0,1,0]=(-0.762516609688)*x[0]**o+(-0.619904320036)*x[0]+(0.653234747183)*x[1]**o+(0.3188610791)*x[1]
        s[3,0,1,1]=(-0.749684385643)*x[0]**o+(-0.30857008835)*x[0]+(0.066895978166)*x[1]**o+(-0.603289571527)*x[1]
        s[3,0,2,0]=(-0.621186971872)*x[0]**o+(0.109237849031)*x[0]+(0.312234719711)*x[1]**o+(0.214139545654)*x[1]
        s[3,0,2,1]=(-0.479630544493)*x[0]**o+(0.936515248516)*x[0]+(0.135913071464)*x[1]**o+(0.536193827105)*x[1]
        s[3,1,0,0]=(-0.419606668612)*x[0]**o+(-0.825159184029)*x[0]+(-0.862279634673)*x[1]**o+(0.0205463968802)*x[1]
        s[3,1,0,1]=(0.35798046036)*x[0]**o+(-0.599162907156)*x[0]+(0.174478509759)*x[1]**o+(-0.779627181611)*x[1]
        s[3,1,1,0]=(-0.175201495388)*x[0]**o+(0.754090549688)*x[0]+(-0.131095703521)*x[1]**o+(-0.149892880631)*x[1]
        s[3,1,1,1]=(-0.718173244947)*x[0]**o+(0.387119862798)*x[0]+(-0.190359128182)*x[1]**o+(0.71904977881)*x[1]
        s[3,1,2,0]=(0.542104637274)*x[0]**o+(0.85151096271)*x[0]+(0.0402471065929)*x[1]**o+(-0.276985707209)*x[1]
        s[3,1,2,1]=(-0.612520892999)*x[0]**o+(0.475838856018)*x[0]+(-0.524069927205)*x[1]**o+(-0.314728620326)*x[1]
        s[3,2,0,0]=(0.853018031276)*x[0]**o+(0.783150840935)*x[0]+(-0.168512012086)*x[1]**o+(0.713965998504)*x[1]
        s[3,2,0,1]=(0.253413905488)*x[0]**o+(-0.433043547194)*x[0]+(-0.775710938403)*x[1]**o+(-0.906939451854)*x[1]
        s[3,2,1,0]=(0.6612655067)*x[0]**o+(0.959161354342)*x[0]+(-0.351925885901)*x[1]**o+(-0.542741000383)*x[1]
        s[3,2,1,1]=(-0.651886826012)*x[0]**o+(0.799653456288)*x[0]+(0.482387863091)*x[1]**o+(-0.628626395493)*x[1]
        s[3,2,2,0]=(-0.314962063276)*x[0]**o+(-0.237445058522)*x[0]+(0.983233001597)*x[1]**o+(0.105803386746)*x[1]
        s[3,2,2,1]=(0.786422364797)*x[0]**o+(0.437192400732)*x[0]+(-0.238461142143)*x[1]**o+(0.696286963331)*x[1]
        s[3,3,0,0]=(0.483733340742)*x[0]**o+(0.749644548963)*x[0]+(0.765700270847)*x[1]**o+(-0.470185526075)*x[1]
        s[3,3,0,1]=(0.680316641067)*x[0]**o+(0.952443208592)*x[0]+(-0.0164617510632)*x[1]**o+(-0.234575904406)*x[1]
        s[3,3,1,0]=(-0.993408262104)*x[0]**o+(-0.229008195245)*x[0]+(0.523753196917)*x[1]**o+(-0.318485891049)*x[1]
        s[3,3,1,1]=(-0.748894192494)*x[0]**o+(-0.879841134058)*x[0]+(-0.650074035674)*x[1]**o+(0.07564873608)*x[1]
        s[3,3,2,0]=(-0.0390185724072)*x[0]**o+(0.754358690288)*x[0]+(-0.961050878591)*x[1]**o+(-0.849404044511)*x[1]
        s[3,3,2,1]=(0.483849885635)*x[0]**o+(0.168666402938)*x[0]+(-0.948399681695)*x[1]**o+(-0.923760856309)*x[1]
        s[3,4,0,0]=(0.0487172550208)*x[0]**o+(-0.627393318222)*x[0]+(0.328198988132)*x[1]**o+(0.615434829581)*x[1]
        s[3,4,0,1]=(0.294964552743)*x[0]**o+(0.593632204587)*x[0]+(-0.922244339835)*x[1]**o+(0.712032424818)*x[1]
        s[3,4,1,0]=(-0.774393234162)*x[0]**o+(0.122655955494)*x[0]+(-0.993126331736)*x[1]**o+(0.553836798684)*x[1]
        s[3,4,1,1]=(-0.46041945879)*x[0]**o+(-0.676034356031)*x[0]+(0.273980415493)*x[1]**o+(-0.190839002618)*x[1]
        s[3,4,2,0]=(0.0439333449053)*x[0]**o+(-0.0796104621766)*x[0]+(0.479839558444)*x[1]**o+(-0.0221747154)*x[1]
        s[3,4,2,1]=(-0.2108539974)*x[0]**o+(-0.743095604828)*x[0]+(-0.531276902705)*x[1]**o+(0.286137673119)*x[1]
        ref[0,0,0,0]=(0.698846288116)/(o+1.)+(-0.345724213367)+(-0.829288219058)*0.5**o
        ref[0,0,0,1]=(0.927490172078)/(o+1.)+(0.261832766866)+(-0.456023711562)*0.5**o
        ref[0,0,1,0]=(-0.785186724625)/(o+1.)+(0.354264849894)+(-0.497067376291)*0.5**o
        ref[0,0,1,1]=(0.203073455207)/(o+1.)+(0.841858493293)+(-0.290568226175)*0.5**o
        ref[0,0,2,0]=(0.330466533936)/(o+1.)+(0.189002260987)+(0.00829962077999)*0.5**o
        ref[0,0,2,1]=(0.847169623314)/(o+1.)+(-0.569874320329)+(-0.806092658508)*0.5**o
        ref[0,1,0,0]=(0.0231278168453)/(o+1.)+(0.126026358657)+(-0.140461069383)*0.5**o
        ref[0,1,0,1]=(-0.659235919696)/(o+1.)+(0.287272854295)+(0.587581627616)*0.5**o
        ref[0,1,1,0]=(0.122921572782)/(o+1.)+(-0.0542100184667)+(-0.641654760801)*0.5**o
        ref[0,1,1,1]=(-0.745741833491)/(o+1.)+(-0.328303894407)+(-0.380132178505)*0.5**o
        ref[0,1,2,0]=(0.927407682128)/(o+1.)+(0.322753770553)+(0.126835269214)*0.5**o
        ref[0,1,2,1]=(0.873696757566)/(o+1.)+(-0.745889199398)+(0.608413549196)*0.5**o
        ref[0,2,0,0]=(0.11529432626)/(o+1.)+(0.354512600826)+(0.0488258290791)*0.5**o
        ref[0,2,0,1]=(-0.0997128168301)/(o+1.)+(0.0415875101043)+(-0.780993161707)*0.5**o
        ref[0,2,1,0]=(0.334264621015)/(o+1.)+(0.275860441029)+(0.305794159864)*0.5**o
        ref[0,2,1,1]=(-0.159835910161)/(o+1.)+(0.237802557268)+(0.71588138307)*0.5**o
        ref[0,2,2,0]=(-0.0558026806854)/(o+1.)+(0.296281715437)+(0.864605462382)*0.5**o
        ref[0,2,2,1]=(-0.478543932177)/(o+1.)+(0.0697144719306)+(-0.0798994114092)*0.5**o
        ref[0,3,0,0]=(0.181378057097)/(o+1.)+(0.105559457917)+(0.998019190199)*0.5**o
        ref[0,3,0,1]=(0.58603690984)/(o+1.)+(0.336403441534)+(-0.724306549649)*0.5**o
        ref[0,3,1,0]=(-0.590753603973)/(o+1.)+(-0.34443456489)+(0.776728843377)*0.5**o
        ref[0,3,1,1]=(0.220692714675)/(o+1.)+(0.387318591606)+(0.285035031516)*0.5**o
        ref[0,3,2,0]=(-0.158356936936)/(o+1.)+(0.0711281169143)+(-0.00286941650137)*0.5**o
        ref[0,3,2,1]=(-0.446900268053)/(o+1.)+(0.0671332607043)+(0.37992478387)*0.5**o
        ref[0,4,0,0]=(0.0552121131111)/(o+1.)+(-0.0282351797878)+(0.714232167791)*0.5**o
        ref[0,4,0,1]=(0.321456277553)/(o+1.)+(0.374058464151)+(0.49556700372)*0.5**o
        ref[0,4,1,0]=(0.0202839586672)/(o+1.)+(-0.0181538415026)+(0.841677614054)*0.5**o
        ref[0,4,1,1]=(0.629772140166)/(o+1.)+(-0.062004936727)+(-0.504028346474)*0.5**o
        ref[0,4,2,0]=(0.919945289519)/(o+1.)+(-0.0599949574834)+(0.434265669856)*0.5**o
        ref[0,4,2,1]=(-0.834647694686)/(o+1.)+(0.2607949111)+(-0.86013960593)*0.5**o
        ref[1,0,0,0]=(-0.430841927673)/(o+1.)+(-0.0855692261333)+(-0.437686950383)*0.5**o
        ref[1,0,0,1]=(-0.172390658038)/(o+1.)+(-0.530724225027)+(0.323843044435)*0.5**o
        ref[1,0,1,0]=(0.89008620109)/(o+1.)+(0.137947417602)+(-0.845882465152)*0.5**o
        ref[1,0,1,1]=(-0.63801489002)/(o+1.)+(-0.566519240363)+(-0.405423425934)*0.5**o
        ref[1,0,2,0]=(-0.908451202669)/(o+1.)+(0.317380457863)+(-0.216267078676)*0.5**o
        ref[1,0,2,1]=(0.103177353457)/(o+1.)+(0.889853616974)+(-0.168907777516)*0.5**o
        ref[1,1,0,0]=(-0.380355614494)/(o+1.)+(0.258490087112)+(-0.884189035361)*0.5**o
        ref[1,1,0,1]=(-0.141666532996)/(o+1.)+(0.0854564858592)+(-0.983514772022)*0.5**o
        ref[1,1,1,0]=(-0.202960085361)/(o+1.)+(-0.013654261655)+(0.723588279961)*0.5**o
        ref[1,1,1,1]=(-0.292492437285)/(o+1.)+(0.846140725444)+(-0.809610385863)*0.5**o
        ref[1,1,2,0]=(0.852180039142)/(o+1.)+(0.202930831904)+(0.831333270285)*0.5**o
        ref[1,1,2,1]=(-0.505300606168)/(o+1.)+(0.135986920218)+(0.889635940128)*0.5**o
        ref[1,2,0,0]=(0.385223744345)/(o+1.)+(-0.0139203658855)+(-0.479209867816)*0.5**o
        ref[1,2,0,1]=(-0.111527008468)/(o+1.)+(0.0704755002279)+(-0.690494514078)*0.5**o
        ref[1,2,1,0]=(-0.264211841662)/(o+1.)+(-0.055637376212)+(-0.65110579618)*0.5**o
        ref[1,2,1,1]=(-0.0384737773817)/(o+1.)+(-0.066655252087)+(-0.495114108462)*0.5**o
        ref[1,2,2,0]=(-0.998046573914)/(o+1.)+(0.255702835012)+(-0.846365789548)*0.5**o
        ref[1,2,2,1]=(-0.366626482676)/(o+1.)+(0.00181631283349)+(0.4479853512)*0.5**o
        ref[1,3,0,0]=(0.10440523429)/(o+1.)+(-0.443081912863)+(-0.677771809341)*0.5**o
        ref[1,3,0,1]=(-0.424985507693)/(o+1.)+(0.0731777219764)+(0.549864901432)*0.5**o
        ref[1,3,1,0]=(-0.158966662533)/(o+1.)+(0.412566023571)+(0.795465752618)*0.5**o
        ref[1,3,1,1]=(-0.351127736099)/(o+1.)+(-0.745161477493)+(-0.395120222846)*0.5**o
        ref[1,3,2,0]=(-0.988402656417)/(o+1.)+(0.422727202609)+(-0.244306249618)*0.5**o
        ref[1,3,2,1]=(-0.873223112612)/(o+1.)+(0.366897431014)+(0.905866127894)*0.5**o
        ref[1,4,0,0]=(-0.914462477805)/(o+1.)+(0.302865321753)+(0.171338765926)*0.5**o
        ref[1,4,0,1]=(0.488643815471)/(o+1.)+(-0.792095498254)+(-0.188491880655)*0.5**o
        ref[1,4,1,0]=(0.539656085813)/(o+1.)+(0.269301999264)+(0.252606322081)*0.5**o
        ref[1,4,1,1]=(0.878910174211)/(o+1.)+(-0.0564831783457)+(0.148934160341)*0.5**o
        ref[1,4,2,0]=(0.300052096347)/(o+1.)+(0.232229395072)+(-0.235447116973)*0.5**o
        ref[1,4,2,1]=(-0.333492913384)/(o+1.)+(0.368533024398)+(0.944445188093)*0.5**o
        ref[2,0,0,0]=(-0.0151488336686)/(o+1.)+(-0.141623674772)+(-0.942092595372)*0.5**o
        ref[2,0,0,1]=(0.383024592178)/(o+1.)+(0.498831859679)+(0.0655506752825)*0.5**o
        ref[2,0,1,0]=(0.138378684)/(o+1.)+(0.0776554025757)+(0.291304803618)*0.5**o
        ref[2,0,1,1]=(0.736449427619)/(o+1.)+(0.171124849182)+(0.0625773477714)*0.5**o
        ref[2,0,2,0]=(0.689775520125)/(o+1.)+(0.663222070731)+(-0.931044858876)*0.5**o
        ref[2,0,2,1]=(-0.619692129874)/(o+1.)+(-0.258812042517)+(-0.732200781532)*0.5**o
        ref[2,1,0,0]=(0.0156599006787)/(o+1.)+(-0.507722130169)+(-0.261557477097)*0.5**o
        ref[2,1,0,1]=(0.604446335087)/(o+1.)+(0.365228089705)+(0.723444855088)*0.5**o
        ref[2,1,1,0]=(0.757190663205)/(o+1.)+(0.363415302197)+(-0.606511927315)*0.5**o
        ref[2,1,1,1]=(-0.825169962155)/(o+1.)+(0.30708402612)+(0.45110092033)*0.5**o
        ref[2,1,2,0]=(-0.288177463065)/(o+1.)+(0.635066808281)+(-0.612806058565)*0.5**o
        ref[2,1,2,1]=(0.0308905730956)/(o+1.)+(0.592301971382)+(-0.746890321439)*0.5**o
        ref[2,2,0,0]=(-0.134118374176)/(o+1.)+(-0.963974277284)+(-0.665223434533)*0.5**o
        ref[2,2,0,1]=(-0.149775511078)/(o+1.)+(-0.187513625838)+(-0.478607794082)*0.5**o
        ref[2,2,1,0]=(-0.265361365664)/(o+1.)+(0.4910546109)+(-0.782692588953)*0.5**o
        ref[2,2,1,1]=(0.351166595901)/(o+1.)+(0.0843450871693)+(0.141591460973)*0.5**o
        ref[2,2,2,0]=(-0.006664462988)/(o+1.)+(0.383892340161)+(-0.282452862833)*0.5**o
        ref[2,2,2,1]=(-0.967086006781)/(o+1.)+(0.750598359819)+(-0.108570066423)*0.5**o
        ref[2,3,0,0]=(0.714460711637)/(o+1.)+(-0.3966143768)+(-0.872481061987)*0.5**o
        ref[2,3,0,1]=(-0.304360730035)/(o+1.)+(0.160625296421)+(-0.314322497313)*0.5**o
        ref[2,3,1,0]=(0.797973787987)/(o+1.)+(-0.151379341813)+(0.364419198839)*0.5**o
        ref[2,3,1,1]=(0.235681715573)/(o+1.)+(-0.390611019949)+(-0.781147212189)*0.5**o
        ref[2,3,2,0]=(-0.917090286139)/(o+1.)+(-0.0599264791076)+(-0.958493367772)*0.5**o
        ref[2,3,2,1]=(0.575188283953)/(o+1.)+(-0.746492624615)+(0.572779229598)*0.5**o
        ref[2,4,0,0]=(-0.362423234235)/(o+1.)+(-0.6492336818)+(-0.178437300104)*0.5**o
        ref[2,4,0,1]=(-0.233874335956)/(o+1.)+(-0.307502535976)+(-0.842863324823)*0.5**o
        ref[2,4,1,0]=(-0.648099915852)/(o+1.)+(0.133339778821)+(-0.396341438458)*0.5**o
        ref[2,4,1,1]=(-0.969248694111)/(o+1.)+(0.846987850078)+(-0.359913098139)*0.5**o
        ref[2,4,2,0]=(0.960012562229)/(o+1.)+(-0.818566064165)+(0.965251612654)*0.5**o
        ref[2,4,2,1]=(0.0455614359233)/(o+1.)+(0.256558952907)+(0.43888464177)*0.5**o
        ref[3,0,0,0]=(-0.657096083564)/(o+1.)+(0.126668055484)+(0.961744925179)*0.5**o
        ref[3,0,0,1]=(-0.537423647089)/(o+1.)+(0.483736899615)+(0.0372329050631)*0.5**o
        ref[3,0,1,0]=(0.653234747183)/(o+1.)+(-0.150521620468)+(-0.762516609688)*0.5**o
        ref[3,0,1,1]=(0.066895978166)/(o+1.)+(-0.455929829939)+(-0.749684385643)*0.5**o
        ref[3,0,2,0]=(0.312234719711)/(o+1.)+(0.161688697343)+(-0.621186971872)*0.5**o
        ref[3,0,2,1]=(0.135913071464)/(o+1.)+(0.73635453781)+(-0.479630544493)*0.5**o
        ref[3,1,0,0]=(-0.862279634673)/(o+1.)+(-0.402306393574)+(-0.419606668612)*0.5**o
        ref[3,1,0,1]=(0.174478509759)/(o+1.)+(-0.689395044384)+(0.35798046036)*0.5**o
        ref[3,1,1,0]=(-0.131095703521)/(o+1.)+(0.302098834529)+(-0.175201495388)*0.5**o
        ref[3,1,1,1]=(-0.190359128182)/(o+1.)+(0.553084820804)+(-0.718173244947)*0.5**o
        ref[3,1,2,0]=(0.0402471065929)/(o+1.)+(0.28726262775)+(0.542104637274)*0.5**o
        ref[3,1,2,1]=(-0.524069927205)/(o+1.)+(0.0805551178455)+(-0.612520892999)*0.5**o
        ref[3,2,0,0]=(-0.168512012086)/(o+1.)+(0.748558419719)+(0.853018031276)*0.5**o
        ref[3,2,0,1]=(-0.775710938403)/(o+1.)+(-0.669991499524)+(0.253413905488)*0.5**o
        ref[3,2,1,0]=(-0.351925885901)/(o+1.)+(0.20821017698)+(0.6612655067)*0.5**o
        ref[3,2,1,1]=(0.482387863091)/(o+1.)+(0.0855135303978)+(-0.651886826012)*0.5**o
        ref[3,2,2,0]=(0.983233001597)/(o+1.)+(-0.0658208358883)+(-0.314962063276)*0.5**o
        ref[3,2,2,1]=(-0.238461142143)/(o+1.)+(0.566739682031)+(0.786422364797)*0.5**o
        ref[3,3,0,0]=(0.765700270847)/(o+1.)+(0.139729511444)+(0.483733340742)*0.5**o
        ref[3,3,0,1]=(-0.0164617510632)/(o+1.)+(0.358933652093)+(0.680316641067)*0.5**o
        ref[3,3,1,0]=(0.523753196917)/(o+1.)+(-0.273747043147)+(-0.993408262104)*0.5**o
        ref[3,3,1,1]=(-0.650074035674)/(o+1.)+(-0.402096198989)+(-0.748894192494)*0.5**o
        ref[3,3,2,0]=(-0.961050878591)/(o+1.)+(-0.0475226771118)+(-0.0390185724072)*0.5**o
        ref[3,3,2,1]=(-0.948399681695)/(o+1.)+(-0.377547226685)+(0.483849885635)*0.5**o
        ref[3,4,0,0]=(0.328198988132)/(o+1.)+(-0.00597924432023)+(0.0487172550208)*0.5**o
        ref[3,4,0,1]=(-0.922244339835)/(o+1.)+(0.652832314702)+(0.294964552743)*0.5**o
        ref[3,4,1,0]=(-0.993126331736)/(o+1.)+(0.338246377089)+(-0.774393234162)*0.5**o
        ref[3,4,1,1]=(0.273980415493)/(o+1.)+(-0.433436679324)+(-0.46041945879)*0.5**o
        ref[3,4,2,0]=(0.479839558444)/(o+1.)+(-0.0508925887883)+(0.0439333449053)*0.5**o
        ref[3,4,2,1]=(-0.531276902705)/(o+1.)+(-0.228478965855)+(-0.2108539974)*0.5**o
      else:
        s[0,0,0,0]=(-0.48893267678)*x[0]**o+(-0.276480532077)*x[0]+(0.774386441775)*x[1]**o+(-0.649500271111)*x[1]+(-0.0349902316269)*x[2]**o+(0.170017460369)*x[2]
        s[0,0,0,1]=(0.271529548298)*x[0]**o+(-0.318825369645)*x[0]+(0.304421796509)*x[1]**o+(0.445685002224)*x[1]+(0.0335660627736)*x[2]**o+(-0.498125294129)*x[2]
        s[0,0,1,0]=(0.217379546035)*x[0]**o+(0.0217166724721)*x[0]+(0.845692481133)*x[1]**o+(0.991062940209)*x[1]+(-0.504574922644)*x[2]**o+(-0.442660078459)*x[2]
        s[0,0,1,1]=(0.042921886647)*x[0]**o+(0.223486944232)*x[0]+(0.0494176704553)*x[1]**o+(0.96519307605)*x[1]+(-0.138757753118)*x[2]**o+(0.0467978128441)*x[2]
        s[0,0,2,0]=(-0.386813522991)*x[0]**o+(-0.460337705352)*x[0]+(0.868148506432)*x[1]**o+(-0.639578197438)*x[1]+(0.344241610414)*x[2]**o+(-0.340649859294)*x[2]
        s[0,0,2,1]=(0.270966465259)*x[0]**o+(-0.921862313681)*x[0]+(0.419658509548)*x[1]**o+(0.382347312575)*x[1]+(-0.458089861176)*x[2]**o+(-0.680587978014)*x[2]
        s[0,1,0,0]=(0.341922361346)*x[0]**o+(0.992862705739)*x[0]+(0.664064252002)*x[1]**o+(-0.976539711563)*x[1]+(-0.336860627327)*x[2]**o+(0.0351493378936)*x[2]
        s[0,1,0,1]=(0.232432790902)*x[0]**o+(0.661576839972)*x[0]+(-0.0847723144646)*x[1]**o+(0.676452509633)*x[1]+(-0.523713339087)*x[2]**o+(-0.944537647834)*x[2]
        s[0,1,1,0]=(-0.0195600321657)*x[0]**o+(0.548788073658)*x[0]+(0.732671243649)*x[1]**o+(-0.559596045627)*x[1]+(-0.821347880719)*x[2]**o+(0.523044807739)*x[2]
        s[0,1,1,1]=(0.440682443808)*x[0]**o+(0.489252739473)*x[0]+(-0.0286407968844)*x[1]**o+(0.389892189545)*x[1]+(0.802377372742)*x[2]**o+(-0.848679176767)*x[2]
        s[0,1,2,0]=(0.0308644963019)*x[0]**o+(-0.838818939488)*x[0]+(-0.923147289871)*x[1]**o+(-0.898497578423)*x[1]+(0.352358085546)*x[2]**o+(-0.0551880704817)*x[2]
        s[0,1,2,1]=(0.964064258383)*x[0]**o+(0.889207476385)*x[0]+(-0.446058209666)*x[1]**o+(-0.410646592871)*x[1]+(-0.143876047927)*x[2]**o+(-0.231175626697)*x[2]
        s[0,2,0,0]=(-0.378594793341)*x[0]**o+(0.45713093072)*x[0]+(-0.0164617155479)*x[1]**o+(-0.852502076132)*x[1]+(-0.445860749107)*x[2]**o+(0.718303558387)*x[2]
        s[0,2,0,1]=(-0.24119818585)*x[0]**o+(-0.480829504364)*x[0]+(0.0483190702959)*x[1]**o+(-0.943891784533)*x[1]+(0.822005676991)*x[2]**o+(0.462965609893)*x[2]
        s[0,2,1,0]=(0.260426222503)*x[0]**o+(0.801642256974)*x[0]+(-0.344027586708)*x[1]**o+(0.609754775626)*x[1]+(-0.614003273478)*x[2]**o+(0.254286708707)*x[2]
        s[0,2,1,1]=(-0.585208459519)*x[0]**o+(0.409783580697)*x[0]+(0.675131095845)*x[1]**o+(0.962899493666)*x[1]+(-0.555202089686)*x[2]**o+(0.55800653582)*x[2]
        s[0,2,2,0]=(0.926784535292)*x[0]**o+(0.366810833854)*x[0]+(0.280209292502)*x[1]**o+(0.455075782363)*x[1]+(0.315470862333)*x[2]**o+(0.804139765801)*x[2]
        s[0,2,2,1]=(-0.182330163829)*x[0]**o+(0.137160802887)*x[0]+(0.50089868344)*x[1]**o+(0.457091587964)*x[1]+(0.661074305613)*x[2]**o+(-0.774463051929)*x[2]
        s[0,3,0,0]=(0.251053015385)*x[0]**o+(-0.00252354996738)*x[0]+(0.219814741857)*x[1]**o+(-0.788960481779)*x[1]+(-0.628563574142)*x[2]**o+(0.238508420102)*x[2]
        s[0,3,0,1]=(-0.885181589797)*x[0]**o+(0.524674512592)*x[0]+(-0.137981402257)*x[1]**o+(-0.57775224131)*x[1]+(-0.151536711237)*x[2]**o+(0.147483336195)*x[2]
        s[0,3,1,0]=(0.464501690982)*x[0]**o+(0.477393661278)*x[0]+(-0.913546513036)*x[1]**o+(0.209928665471)*x[1]+(0.552522027491)*x[2]**o+(-0.200580731655)*x[2]
        s[0,3,1,1]=(0.117929836206)*x[0]**o+(-0.973701486459)*x[0]+(-0.876411386461)*x[1]**o+(-0.511161888778)*x[1]+(0.213923028781)*x[2]**o+(0.620921035428)*x[2]
        s[0,3,2,0]=(0.228255426058)*x[0]**o+(-0.135554118227)*x[0]+(0.607714808043)*x[1]**o+(0.522943351003)*x[1]+(-0.686514740068)*x[2]**o+(-0.238689322489)*x[2]
        s[0,3,2,1]=(-0.155365105782)*x[0]**o+(0.28572413448)*x[0]+(-0.452657577548)*x[1]**o+(-0.884598572632)*x[1]+(0.279309317605)*x[2]**o+(-0.523804815006)*x[2]
        s[0,4,0,0]=(-0.913401118579)*x[0]**o+(0.141999285441)*x[0]+(-0.0373272930617)*x[1]**o+(-0.907472677652)*x[1]+(0.648778084537)*x[2]**o+(0.905348717002)*x[2]
        s[0,4,0,1]=(0.272596166342)*x[0]**o+(0.523354359959)*x[0]+(-0.597238366724)*x[1]**o+(-0.57955847031)*x[1]+(0.961129977909)*x[2]**o+(-0.539446933115)*x[2]
        s[0,4,1,0]=(0.469445737741)*x[0]**o+(0.272602303288)*x[0]+(-0.168715269045)*x[1]**o+(-0.508483179891)*x[1]+(-0.0542883418628)*x[2]**o+(-0.549230857829)*x[2]
        s[0,4,1,1]=(-0.727336869594)*x[0]**o+(0.501720827606)*x[0]+(0.0112875501961)*x[1]**o+(-0.195351938275)*x[1]+(0.477540723998)*x[2]**o+(0.314035857499)*x[2]
        s[0,4,2,0]=(-0.826661708611)*x[0]**o+(-0.453403968062)*x[0]+(-0.424265582853)*x[1]**o+(0.850699490929)*x[1]+(0.869873737156)*x[2]**o+(0.512188794374)*x[2]
        s[0,4,2,1]=(0.788019130522)*x[0]**o+(-0.38976086271)*x[0]+(0.450211292079)*x[1]**o+(-0.845896268738)*x[1]+(-0.944295444764)*x[2]**o+(0.49424688571)*x[2]
        s[1,0,0,0]=(0.251841546653)*x[0]**o+(-0.262117658277)*x[0]+(-0.756023145868)*x[1]**o+(0.562918802897)*x[1]+(0.999897367962)*x[2]**o+(-0.438732703957)*x[2]
        s[1,0,0,1]=(0.0702543485705)*x[0]**o+(0.553020799687)*x[0]+(0.129510978458)*x[1]**o+(-0.287781735231)*x[1]+(0.0962686272734)*x[2]**o+(-0.153713487101)*x[2]
        s[1,0,1,0]=(0.0132145791001)*x[0]**o+(-0.747224756368)*x[0]+(0.533216099372)*x[1]**o+(0.651501143279)*x[1]+(0.275246126663)*x[2]**o+(0.564330219864)*x[2]
        s[1,0,1,1]=(0.00555685773193)*x[0]**o+(-0.912265903466)*x[0]+(-0.165714475359)*x[1]**o+(-0.29550893963)*x[1]+(-0.834017179013)*x[2]**o+(-0.66051475999)*x[2]
        s[1,0,2,0]=(-0.937628377428)*x[0]**o+(0.670046440332)*x[0]+(-0.836211833803)*x[1]**o+(0.294767128956)*x[1]+(-0.0831464801716)*x[2]**o+(0.173538164187)*x[2]
        s[1,0,2,1]=(0.0853414413422)*x[0]**o+(0.332113726085)*x[0]+(-0.00785676889602)*x[1]**o+(-0.787517944282)*x[1]+(0.640692804663)*x[2]**o+(0.16190882793)*x[2]
        s[1,1,0,0]=(-0.197717271419)*x[0]**o+(0.0875673864383)*x[0]+(0.303400667777)*x[1]**o+(0.561096499643)*x[1]+(-0.544100296942)*x[2]**o+(-0.236967889808)*x[2]
        s[1,1,0,1]=(0.378319093831)*x[0]**o+(-0.173085814456)*x[0]+(-0.520972068858)*x[1]**o+(-0.0887121930691)*x[1]+(0.0219775149259)*x[2]**o+(-0.902192785902)*x[2]
        s[1,1,1,0]=(-0.0852354427742)*x[0]**o+(-0.868052438949)*x[0]+(-0.962245566851)*x[1]**o+(-0.785773600296)*x[1]+(0.871469592508)*x[2]**o+(0.970335434685)*x[2]
        s[1,1,1,1]=(0.543079304077)*x[0]**o+(0.563798729263)*x[0]+(-0.0684074924309)*x[1]**o+(-0.0846345417372)*x[1]+(-0.0409611114801)*x[2]**o+(0.064168518437)*x[2]
        s[1,1,2,0]=(0.163126135652)*x[0]**o+(0.550910259467)*x[0]+(-0.838398060114)*x[1]**o+(0.20704094128)*x[1]+(-0.988745842471)*x[2]**o+(-0.238812411091)*x[2]
        s[1,1,2,1]=(-0.4311680318)*x[0]**o+(-0.689268700879)*x[0]+(-0.478591190853)*x[1]**o+(-0.935785118221)*x[1]+(0.681916835809)*x[2]**o+(0.316911720884)*x[2]
        s[1,2,0,0]=(-0.823741968282)*x[0]**o+(0.337348902553)*x[0]+(-0.773958391461)*x[1]**o+(0.319484407204)*x[1]+(-0.806392888327)*x[2]**o+(0.784342104197)*x[2]
        s[1,2,0,1]=(-0.271373691068)*x[0]**o+(-0.674977315639)*x[0]+(-0.73119288706)*x[1]**o+(-0.822294334638)*x[1]+(-0.348777213792)*x[2]**o+(0.539498527057)*x[2]
        s[1,2,1,0]=(-0.197804966703)*x[0]**o+(-0.000545898061773)*x[0]+(-0.477781076232)*x[1]**o+(-0.726309620722)*x[1]+(0.33565493623)*x[2]**o+(-0.472018740475)*x[2]
        s[1,2,1,1]=(0.985473777724)*x[0]**o+(-0.905489956976)*x[0]+(0.542738966677)*x[1]**o+(0.405770678949)*x[1]+(0.742199205539)*x[2]**o+(-0.834198356515)*x[2]
        s[1,2,2,0]=(0.0963375474866)*x[0]**o+(-0.702341975163)*x[0]+(-0.719805210146)*x[1]**o+(-0.573969955879)*x[1]+(0.849036302763)*x[2]**o+(-0.050325972969)*x[2]
        s[1,2,2,1]=(-0.000809557040863)*x[0]**o+(-0.298554227357)*x[0]+(-0.257571070168)*x[1]**o+(0.902842431485)*x[1]+(-0.565532231171)*x[2]**o+(-0.0268976084038)*x[2]
        s[1,3,0,0]=(-0.450224087014)*x[0]**o+(0.644911723614)*x[0]+(0.585908753859)*x[1]**o+(0.192087613433)*x[1]+(0.37463835574)*x[2]**o+(-0.858368552725)*x[2]
        s[1,3,0,1]=(-0.211716554169)*x[0]**o+(0.133373638633)*x[0]+(-0.477897006462)*x[1]**o+(-0.856170108137)*x[1]+(0.709445201671)*x[2]**o+(0.999843178308)*x[2]
        s[1,3,1,0]=(0.469992449976)*x[0]**o+(-0.547601912734)*x[0]+(-0.652917936853)*x[1]**o+(-0.973841854379)*x[1]+(0.346404346699)*x[2]**o+(0.960819995744)*x[2]
        s[1,3,1,1]=(-0.9202597392)*x[0]**o+(0.476802978382)*x[0]+(-0.106899807732)*x[1]**o+(-0.704577566959)*x[1]+(-0.331549015764)*x[2]**o+(0.936043692204)*x[2]
        s[1,3,2,0]=(0.724283786459)*x[0]**o+(0.822984194337)*x[0]+(0.705848034385)*x[1]**o+(-0.59858805565)*x[1]+(0.350635957833)*x[2]**o+(0.882807237348)*x[2]
        s[1,3,2,1]=(-0.582421422441)*x[0]**o+(0.809035080665)*x[0]+(-0.457851306764)*x[1]**o+(-0.993328352221)*x[1]+(-0.576322826333)*x[2]**o+(-0.749512812669)*x[2]
        s[1,4,0,0]=(0.268654181239)*x[0]**o+(-0.902343324019)*x[0]+(0.638580048206)*x[1]**o+(-0.832694904209)*x[1]+(-0.305201732282)*x[2]**o+(-0.867754959197)*x[2]
        s[1,4,0,1]=(-0.526400834988)*x[0]**o+(-0.40198050654)*x[0]+(0.337289899423)*x[1]**o+(0.179314676284)*x[1]+(-0.931514578551)*x[2]**o+(-0.449197337162)*x[2]
        s[1,4,1,0]=(0.912742445536)*x[0]**o+(-0.815137545811)*x[0]+(-0.38425815792)*x[1]**o+(-0.713804511416)*x[1]+(-0.0940645853762)*x[2]**o+(-0.184482970408)*x[2]
        s[1,4,1,1]=(0.0912352323191)*x[0]**o+(0.990389316513)*x[0]+(-0.0350410457429)*x[1]**o+(0.80277947226)*x[1]+(0.35771085432)*x[2]**o+(0.704976740376)*x[2]
        s[1,4,2,0]=(-0.212374984853)*x[0]**o+(0.115911507956)*x[0]+(0.743357884785)*x[1]**o+(-0.351179816129)*x[1]+(0.363783848165)*x[2]**o+(0.0667503905017)*x[2]
        s[1,4,2,1]=(-0.192346087265)*x[0]**o+(-0.875505251594)*x[0]+(0.450972468787)*x[1]**o+(0.0441627318307)*x[1]+(-0.955445583779)*x[2]**o+(0.561351576194)*x[2]
        s[2,0,0,0]=(0.646959412154)*x[0]**o+(0.49125803323)*x[0]+(-0.380573298578)*x[1]**o+(0.5795381042)*x[1]+(0.988167811553)*x[2]**o+(-0.848010513202)*x[2]
        s[2,0,0,1]=(-0.116490832795)*x[0]**o+(-0.413497866427)*x[0]+(0.015263239692)*x[1]**o+(0.995022978956)*x[1]+(-0.606550379921)*x[2]**o+(0.480728880447)*x[2]
        s[2,0,1,0]=(-0.656776412474)*x[0]**o+(0.105101642124)*x[0]+(-0.152043267549)*x[1]**o+(0.0397914854834)*x[1]+(0.907598907718)*x[2]**o+(0.55003346668)*x[2]
        s[2,0,1,1]=(-0.244997314762)*x[0]**o+(-0.180839249852)*x[0]+(0.737654816515)*x[1]**o+(-0.316505218599)*x[1]+(-0.467254068865)*x[2]**o+(0.642411232963)*x[2]
        s[2,0,2,0]=(0.427359531608)*x[0]**o+(0.0694309336993)*x[0]+(0.567819273901)*x[1]**o+(0.302951042003)*x[1]+(-0.652475445869)*x[2]**o+(-0.0604300175636)*x[2]
        s[2,0,2,1]=(-0.28054117321)*x[0]**o+(-0.506627838493)*x[0]+(0.830518434434)*x[1]**o+(-0.878165796713)*x[1]+(-0.924137956967)*x[2]**o+(-0.542977024006)*x[2]
        s[2,1,0,0]=(0.832323847638)*x[0]**o+(0.788814796981)*x[0]+(0.443003510301)*x[1]**o+(-0.692364079852)*x[1]+(-0.107112423673)*x[2]**o+(-0.42975252335)*x[2]
        s[2,1,0,1]=(0.791743274331)*x[0]**o+(0.506185826171)*x[0]+(0.880323084905)*x[1]**o+(0.875445981105)*x[1]+(-0.988158636575)*x[2]**o+(0.0106467506338)*x[2]
        s[2,1,1,0]=(-0.0716642151005)*x[0]**o+(0.184121338207)*x[0]+(0.563163918743)*x[1]**o+(-0.0476664508964)*x[1]+(-0.779735247644)*x[2]**o+(0.0532353296963)*x[2]
        s[2,1,1,1]=(-0.487936865731)*x[0]**o+(0.910882234056)*x[0]+(0.0698617623832)*x[1]**o+(0.949481044342)*x[1]+(0.605781066129)*x[2]**o+(-0.961538763982)*x[2]
        s[2,1,2,0]=(-0.865760152137)*x[0]**o+(0.98792005348)*x[0]+(-0.266208542741)*x[1]**o+(0.724207675193)*x[1]+(-0.431135420169)*x[2]**o+(-0.719606907727)*x[2]
        s[2,1,2,1]=(-0.550134186201)*x[0]**o+(0.377678575292)*x[0]+(0.681637629077)*x[1]**o+(0.880510886804)*x[1]+(0.185916696615)*x[2]**o+(-0.561352445373)*x[2]
        s[2,2,0,0]=(-0.0191940498302)*x[0]**o+(0.92499736077)*x[0]+(-0.457609595358)*x[1]**o+(0.295953391548)*x[1]+(-0.60039707059)*x[2]**o+(0.439541541552)*x[2]
        s[2,2,0,1]=(-0.593353550722)*x[0]**o+(-0.262307597953)*x[0]+(0.0707406713996)*x[1]**o+(0.510104333108)*x[1]+(-0.325033325591)*x[2]**o+(-0.978979243989)*x[2]
        s[2,2,1,0]=(0.758386313418)*x[0]**o+(-0.904849079285)*x[0]+(0.517417240328)*x[1]**o+(0.662794014603)*x[1]+(-0.9181308007)*x[2]**o+(0.545716036213)*x[2]
        s[2,2,1,1]=(-0.0562030350565)*x[0]**o+(-0.187267436123)*x[0]+(-0.0666932679699)*x[1]**o+(-0.192661397505)*x[1]+(0.683795366538)*x[2]**o+(0.554493956469)*x[2]
        s[2,2,2,0]=(0.632035307709)*x[0]**o+(-0.352426434378)*x[0]+(-0.572996524284)*x[1]**o+(-0.260948177348)*x[1]+(0.0781523669039)*x[2]**o+(-0.755607733499)*x[2]
        s[2,2,2,1]=(0.877117460076)*x[0]**o+(-0.368983710962)*x[0]+(0.339499499347)*x[1]**o+(-0.74687226645)*x[1]+(-0.0236621043423)*x[2]**o+(-0.524200850201)*x[2]
        s[2,3,0,0]=(0.868438479199)*x[0]**o+(-0.490384020524)*x[0]+(-0.348975370347)*x[1]**o+(0.0689779831612)*x[1]+(0.993171286973)*x[2]**o+(0.238168596199)*x[2]
        s[2,3,0,1]=(0.58688914355)*x[0]**o+(-0.299012943187)*x[0]+(-0.475183593899)*x[1]**o+(-0.209284023343)*x[1]+(-0.128788417577)*x[2]**o+(0.88040263674)*x[2]
        s[2,3,1,0]=(-0.491821145506)*x[0]**o+(0.28222061488)*x[0]+(-0.432027482169)*x[1]**o+(0.0843542963065)*x[1]+(0.179895461516)*x[2]**o+(0.26281058218)*x[2]
        s[2,3,1,1]=(-0.68076808014)*x[0]**o+(0.966337829009)*x[0]+(-0.967714227169)*x[1]**o+(-0.511464770717)*x[1]+(-0.00173666009442)*x[2]**o+(0.418327208497)*x[2]
        s[2,3,2,0]=(-0.981204168827)*x[0]**o+(-0.481006793801)*x[0]+(0.223651529206)*x[1]**o+(0.0692948204072)*x[1]+(0.833168651519)*x[2]**o+(-0.747962032571)*x[2]
        s[2,3,2,1]=(-0.979594675181)*x[0]**o+(-0.793815945451)*x[0]+(-0.972489753584)*x[1]**o+(0.193666863184)*x[1]+(0.813999185145)*x[2]**o+(0.976350643493)*x[2]
        s[2,4,0,0]=(0.160095467562)*x[0]**o+(-0.0596430245423)*x[0]+(0.421111483086)*x[1]**o+(-0.50368497144)*x[1]+(0.295956709432)*x[2]**o+(-0.547225692085)*x[2]
        s[2,4,0,1]=(0.121233419707)*x[0]**o+(0.0796884300301)*x[0]+(0.63360038458)*x[1]**o+(-0.285793125165)*x[1]+(-0.322665608493)*x[2]**o+(0.144053221653)*x[2]
        s[2,4,1,0]=(-0.327390540088)*x[0]**o+(-0.158990478104)*x[0]+(0.0417451752863)*x[1]**o+(0.0577661706274)*x[1]+(0.608054602266)*x[2]**o+(0.876076509191)*x[2]
        s[2,4,1,1]=(-0.336090262681)*x[0]**o+(0.460831366224)*x[0]+(0.126375418917)*x[1]**o+(-0.0524791065196)*x[1]+(0.778197798934)*x[2]**o+(-0.938503586105)*x[2]
        s[2,4,2,0]=(-0.157686815732)*x[0]**o+(-0.240464045969)*x[0]+(-0.610957847036)*x[1]**o+(-0.591812526831)*x[1]+(0.683562885502)*x[2]**o+(-0.575699253807)*x[2]
        s[2,4,2,1]=(-0.591186612156)*x[0]**o+(0.952366931656)*x[0]+(0.563360859838)*x[1]**o+(-0.540973061762)*x[1]+(0.95638308424)*x[2]**o+(0.0810587285575)*x[2]
        s[3,0,0,0]=(-0.777152282839)*x[0]**o+(0.0462962198971)*x[0]+(-0.989861157039)*x[1]**o+(-0.895147454926)*x[1]+(0.496304718705)*x[2]**o+(0.724664844743)*x[2]
        s[3,0,0,1]=(-0.152045574171)*x[0]**o+(0.557429298004)*x[0]+(0.172100777056)*x[1]**o+(-0.762302216782)*x[1]+(0.0107560041439)*x[2]**o+(0.0984433240396)*x[2]
        s[3,0,1,0]=(-0.300572515767)*x[0]**o+(-0.776096822514)*x[0]+(-0.304519070945)*x[1]**o+(0.222039297615)*x[1]+(0.42051246201)*x[2]**o+(0.875929404536)*x[2]
        s[3,0,1,1]=(-0.832809662655)*x[0]**o+(-0.775869677688)*x[0]+(-0.548130481677)*x[1]**o+(-0.863910117253)*x[1]+(-0.708195298544)*x[2]**o+(-0.895809178957)*x[2]
        s[3,0,2,0]=(0.417580286092)*x[0]**o+(-0.320463746468)*x[0]+(-0.780326066905)*x[1]**o+(-0.397006341179)*x[1]+(0.575088222676)*x[2]**o+(0.621841867422)*x[2]
        s[3,0,2,1]=(0.266423263811)*x[0]**o+(0.782081751363)*x[0]+(-0.395723332933)*x[1]**o+(-0.474817183348)*x[1]+(0.483870682109)*x[2]**o+(-0.416708790303)*x[2]
        s[3,1,0,0]=(0.364336403912)*x[0]**o+(0.4746091186)*x[0]+(0.419745216537)*x[1]**o+(0.906218032598)*x[1]+(-0.561381612354)*x[2]**o+(0.252977998753)*x[2]
        s[3,1,0,1]=(-0.479065041103)*x[0]**o+(-0.727722678216)*x[0]+(0.298427986755)*x[1]**o+(0.391426108723)*x[1]+(-0.18371001783)*x[2]**o+(-0.298396916251)*x[2]
        s[3,1,1,0]=(-0.456720853948)*x[0]**o+(-0.137192756988)*x[0]+(0.0387577082772)*x[1]**o+(0.0342816278)*x[1]+(0.287825957753)*x[2]**o+(-0.416093408739)*x[2]
        s[3,1,1,1]=(-0.157773606009)*x[0]**o+(0.229634478596)*x[0]+(-0.100573203257)*x[1]**o+(-0.791674554355)*x[1]+(0.633139739302)*x[2]**o+(0.53643300369)*x[2]
        s[3,1,2,0]=(-0.977094714538)*x[0]**o+(-0.980511324919)*x[0]+(-0.996777041114)*x[1]**o+(-0.135014608241)*x[1]+(0.171228488823)*x[2]**o+(-0.826149942234)*x[2]
        s[3,1,2,1]=(0.40095895971)*x[0]**o+(-0.16005681048)*x[0]+(-0.110948052824)*x[1]**o+(-0.0657272460506)*x[1]+(-0.921326915659)*x[2]**o+(0.498246475577)*x[2]
        s[3,2,0,0]=(-0.521393633448)*x[0]**o+(-0.442461853948)*x[0]+(0.882466157567)*x[1]**o+(-0.318640196818)*x[1]+(-0.454978258143)*x[2]**o+(-0.146713474073)*x[2]
        s[3,2,0,1]=(-0.60584665741)*x[0]**o+(0.186330030473)*x[0]+(0.800412347134)*x[1]**o+(0.632355962829)*x[1]+(-0.577489166576)*x[2]**o+(0.288233513083)*x[2]
        s[3,2,1,0]=(0.645974743024)*x[0]**o+(-0.463548979592)*x[0]+(0.454659333323)*x[1]**o+(-0.147068470329)*x[1]+(0.621612597429)*x[2]**o+(0.309226445731)*x[2]
        s[3,2,1,1]=(0.142559408818)*x[0]**o+(0.568796102102)*x[0]+(-0.957487822488)*x[1]**o+(0.199695598234)*x[1]+(0.437309500275)*x[2]**o+(-0.608539956732)*x[2]
        s[3,2,2,0]=(0.814258898685)*x[0]**o+(-0.0657073512095)*x[0]+(-0.675703123952)*x[1]**o+(-0.826924419864)*x[1]+(-0.202583280241)*x[2]**o+(0.556708057646)*x[2]
        s[3,2,2,1]=(0.420183604639)*x[0]**o+(0.234120268268)*x[0]+(-0.690228330954)*x[1]**o+(0.0742691494912)*x[1]+(-0.542445114625)*x[2]**o+(-0.671138939877)*x[2]
        s[3,3,0,0]=(-0.327300600039)*x[0]**o+(-0.883120008711)*x[0]+(-0.588356952373)*x[1]**o+(0.964702848682)*x[1]+(0.0983852055922)*x[2]**o+(-0.517082121951)*x[2]
        s[3,3,0,1]=(0.808391338918)*x[0]**o+(0.413826885405)*x[0]+(-0.781923414495)*x[1]**o+(0.941400794201)*x[1]+(0.566387362748)*x[2]**o+(-0.361926999623)*x[2]
        s[3,3,1,0]=(0.0408315227595)*x[0]**o+(0.00354329999935)*x[0]+(0.29340210554)*x[1]**o+(-0.129271597771)*x[1]+(0.85418604558)*x[2]**o+(0.972437132596)*x[2]
        s[3,3,1,1]=(0.488110633719)*x[0]**o+(0.442243578548)*x[0]+(0.491915283822)*x[1]**o+(0.900808596947)*x[1]+(0.872340191832)*x[2]**o+(-0.821189941972)*x[2]
        s[3,3,2,0]=(0.698445367492)*x[0]**o+(0.235000428999)*x[0]+(0.0231665862971)*x[1]**o+(0.175887750008)*x[1]+(0.285424647457)*x[2]**o+(-0.0889092865387)*x[2]
        s[3,3,2,1]=(-0.619448487664)*x[0]**o+(-0.25845368795)*x[0]+(0.514713543216)*x[1]**o+(0.933603723018)*x[1]+(0.965553741692)*x[2]**o+(0.775325889348)*x[2]
        s[3,4,0,0]=(0.653712053876)*x[0]**o+(0.397899343408)*x[0]+(-0.980528220216)*x[1]**o+(0.192768638146)*x[1]+(-0.131961987222)*x[2]**o+(0.294990291948)*x[2]
        s[3,4,0,1]=(0.519872654154)*x[0]**o+(-0.523923706123)*x[0]+(-0.366056337857)*x[1]**o+(0.587286575656)*x[1]+(0.67858517405)*x[2]**o+(-0.528581733781)*x[2]
        s[3,4,1,0]=(-0.561163638499)*x[0]**o+(0.834141187099)*x[0]+(0.101344553848)*x[1]**o+(-0.630659721656)*x[1]+(0.177616824366)*x[2]**o+(-0.702449367192)*x[2]
        s[3,4,1,1]=(0.0594382754162)*x[0]**o+(-0.274398458709)*x[0]+(0.423874245449)*x[1]**o+(0.644709194846)*x[1]+(0.742592384867)*x[2]**o+(-0.00411951621579)*x[2]
        s[3,4,2,0]=(0.0680763742436)*x[0]**o+(0.718014295363)*x[0]+(0.10166016369)*x[1]**o+(-0.308308565274)*x[1]+(0.529471669819)*x[2]**o+(0.371855430993)*x[2]
        s[3,4,2,1]=(0.154883537954)*x[0]**o+(-0.727616949065)*x[0]+(0.685560277601)*x[1]**o+(0.0846298021044)*x[1]+(0.284159382541)*x[2]**o+(0.869297635219)*x[2]
        ref[0,0,0,0]=(0.739396210148)/(o+1.)+(-0.377981671409)+(-0.48893267678)*0.5**o
        ref[0,0,0,1]=(0.337987859283)/(o+1.)+(-0.185632830775)+(0.271529548298)*0.5**o
        ref[0,0,1,0]=(0.341117558489)/(o+1.)+(0.285059767111)+(0.217379546035)*0.5**o
        ref[0,0,1,1]=(-0.0893400826624)/(o+1.)+(0.617738916563)+(0.042921886647)*0.5**o
        ref[0,0,2,0]=(1.21239011685)/(o+1.)+(-0.720282881042)+(-0.386813522991)*0.5**o
        ref[0,0,2,1]=(-0.0384313516284)/(o+1.)+(-0.61005148956)+(0.270966465259)*0.5**o
        ref[0,1,0,0]=(0.327203624676)/(o+1.)+(0.0257361660344)+(0.341922361346)*0.5**o
        ref[0,1,0,1]=(-0.608485653551)/(o+1.)+(0.196745850885)+(0.232432790902)*0.5**o
        ref[0,1,1,0]=(-0.0886766370694)/(o+1.)+(0.256118417885)+(-0.0195600321657)*0.5**o
        ref[0,1,1,1]=(0.773736575857)/(o+1.)+(0.015232876126)+(0.440682443808)*0.5**o
        ref[0,1,2,0]=(-0.570789204325)/(o+1.)+(-0.896252294196)+(0.0308644963019)*0.5**o
        ref[0,1,2,1]=(-0.589934257593)/(o+1.)+(0.123692628409)+(0.964064258383)*0.5**o
        ref[0,2,0,0]=(-0.462322464655)/(o+1.)+(0.161466206488)+(-0.378594793341)*0.5**o
        ref[0,2,0,1]=(0.870324747287)/(o+1.)+(-0.480877839502)+(-0.24119818585)*0.5**o
        ref[0,2,1,0]=(-0.958030860186)/(o+1.)+(0.832841870654)+(0.260426222503)*0.5**o
        ref[0,2,1,1]=(0.119929006158)/(o+1.)+(0.965344805092)+(-0.585208459519)*0.5**o
        ref[0,2,2,0]=(0.595680154835)/(o+1.)+(0.813013191009)+(0.926784535292)*0.5**o
        ref[0,2,2,1]=(1.16197298905)/(o+1.)+(-0.0901053305394)+(-0.182330163829)*0.5**o
        ref[0,3,0,0]=(-0.408748832285)/(o+1.)+(-0.276487805822)+(0.251053015385)*0.5**o
        ref[0,3,0,1]=(-0.289518113494)/(o+1.)+(0.0472028037384)+(-0.885181589797)*0.5**o
        ref[0,3,1,0]=(-0.361024485545)/(o+1.)+(0.243370797547)+(0.464501690982)*0.5**o
        ref[0,3,1,1]=(-0.66248835768)/(o+1.)+(-0.431971169905)+(0.117929836206)*0.5**o
        ref[0,3,2,0]=(-0.0787999320256)/(o+1.)+(0.0743499551435)+(0.228255426058)*0.5**o
        ref[0,3,2,1]=(-0.173348259943)/(o+1.)+(-0.561339626579)+(-0.155365105782)*0.5**o
        ref[0,4,0,0]=(0.611450791476)/(o+1.)+(0.0699376623952)+(-0.913401118579)*0.5**o
        ref[0,4,0,1]=(0.363891611185)/(o+1.)+(-0.297825521733)+(0.272596166342)*0.5**o
        ref[0,4,1,0]=(-0.223003610908)/(o+1.)+(-0.392555867216)+(0.469445737741)*0.5**o
        ref[0,4,1,1]=(0.488828274194)/(o+1.)+(0.310202373415)+(-0.727336869594)*0.5**o
        ref[0,4,2,0]=(0.445608154304)/(o+1.)+(0.454742158621)+(-0.826661708611)*0.5**o
        ref[0,4,2,1]=(-0.494084152685)/(o+1.)+(-0.370705122869)+(0.788019130522)*0.5**o
        ref[1,0,0,0]=(0.243874222094)/(o+1.)+(-0.0689657796687)+(0.251841546653)*0.5**o
        ref[1,0,0,1]=(0.225779605731)/(o+1.)+(0.0557627886771)+(0.0702543485705)*0.5**o
        ref[1,0,1,0]=(0.808462226036)/(o+1.)+(0.234303303388)+(0.0132145791001)*0.5**o
        ref[1,0,1,1]=(-0.999731654372)/(o+1.)+(-0.934144801543)+(0.00555685773193)*0.5**o
        ref[1,0,2,0]=(-0.919358313974)/(o+1.)+(0.569175866738)+(-0.937628377428)*0.5**o
        ref[1,0,2,1]=(0.632836035767)/(o+1.)+(-0.146747695134)+(0.0853414413422)*0.5**o
        ref[1,1,0,0]=(-0.240699629165)/(o+1.)+(0.205847998136)+(-0.197717271419)*0.5**o
        ref[1,1,0,1]=(-0.498994553932)/(o+1.)+(-0.581995396714)+(0.378319093831)*0.5**o
        ref[1,1,1,0]=(-0.0907759743431)/(o+1.)+(-0.34174530228)+(-0.0852354427742)*0.5**o
        ref[1,1,1,1]=(-0.109368603911)/(o+1.)+(0.271666352981)+(0.543079304077)*0.5**o
        ref[1,1,2,0]=(-1.82714390258)/(o+1.)+(0.259569394828)+(0.163126135652)*0.5**o
        ref[1,1,2,1]=(0.203325644955)/(o+1.)+(-0.654071049108)+(-0.4311680318)*0.5**o
        ref[1,2,0,0]=(-1.58035127979)/(o+1.)+(0.720587706977)+(-0.823741968282)*0.5**o
        ref[1,2,0,1]=(-1.07997010085)/(o+1.)+(-0.47888656161)+(-0.271373691068)*0.5**o
        ref[1,2,1,0]=(-0.142126140002)/(o+1.)+(-0.599437129629)+(-0.197804966703)*0.5**o
        ref[1,2,1,1]=(1.28493817222)/(o+1.)+(-0.666958817271)+(0.985473777724)*0.5**o
        ref[1,2,2,0]=(0.129231092617)/(o+1.)+(-0.663318952006)+(0.0963375474866)*0.5**o
        ref[1,2,2,1]=(-0.823103301339)/(o+1.)+(0.288695297862)+(-0.000809557040863)*0.5**o
        ref[1,3,0,0]=(0.960547109599)/(o+1.)+(-0.010684607839)+(-0.450224087014)*0.5**o
        ref[1,3,0,1]=(0.231548195209)/(o+1.)+(0.138523354402)+(-0.211716554169)*0.5**o
        ref[1,3,1,0]=(-0.306513590154)/(o+1.)+(-0.280311885684)+(0.469992449976)*0.5**o
        ref[1,3,1,1]=(-0.438448823496)/(o+1.)+(0.354134551813)+(-0.9202597392)*0.5**o
        ref[1,3,2,0]=(1.05648399222)/(o+1.)+(0.553601688018)+(0.724283786459)*0.5**o
        ref[1,3,2,1]=(-1.0341741331)/(o+1.)+(-0.466903042112)+(-0.582421422441)*0.5**o
        ref[1,4,0,0]=(0.333378315924)/(o+1.)+(-1.30139659371)+(0.268654181239)*0.5**o
        ref[1,4,0,1]=(-0.594224679128)/(o+1.)+(-0.335931583709)+(-0.526400834988)*0.5**o
        ref[1,4,1,0]=(-0.478322743296)/(o+1.)+(-0.856712513818)+(0.912742445536)*0.5**o
        ref[1,4,1,1]=(0.322669808577)/(o+1.)+(1.24907276457)+(0.0912352323191)*0.5**o
        ref[1,4,2,0]=(1.10714173295)/(o+1.)+(-0.0842589588357)+(-0.212374984853)*0.5**o
        ref[1,4,2,1]=(-0.504473114993)/(o+1.)+(-0.134995471785)+(-0.192346087265)*0.5**o
        ref[2,0,0,0]=(0.607594512975)/(o+1.)+(0.111392812114)+(0.646959412154)*0.5**o
        ref[2,0,0,1]=(-0.59128714023)/(o+1.)+(0.531126996488)+(-0.116490832795)*0.5**o
        ref[2,0,1,0]=(0.755555640169)/(o+1.)+(0.347463297144)+(-0.656776412474)*0.5**o
        ref[2,0,1,1]=(0.27040074765)/(o+1.)+(0.0725333822557)+(-0.244997314762)*0.5**o
        ref[2,0,2,0]=(-0.0846561719678)/(o+1.)+(0.155975979069)+(0.427359531608)*0.5**o
        ref[2,0,2,1]=(-0.0936195225336)/(o+1.)+(-0.963885329606)+(-0.28054117321)*0.5**o
        ref[2,1,0,0]=(0.335891086628)/(o+1.)+(-0.166650903111)+(0.832323847638)*0.5**o
        ref[2,1,0,1]=(-0.10783555167)/(o+1.)+(0.696139278955)+(0.791743274331)*0.5**o
        ref[2,1,1,0]=(-0.2165713289)/(o+1.)+(0.0948451085032)+(-0.0716642151005)*0.5**o
        ref[2,1,1,1]=(0.675642828512)/(o+1.)+(0.449412257208)+(-0.487936865731)*0.5**o
        ref[2,1,2,0]=(-0.69734396291)/(o+1.)+(0.496260410473)+(-0.865760152137)*0.5**o
        ref[2,1,2,1]=(0.867554325692)/(o+1.)+(0.348418508362)+(-0.550134186201)*0.5**o
        ref[2,2,0,0]=(-1.05800666595)/(o+1.)+(0.830246146935)+(-0.0191940498302)*0.5**o
        ref[2,2,0,1]=(-0.254292654192)/(o+1.)+(-0.365591254417)+(-0.593353550722)*0.5**o
        ref[2,2,1,0]=(-0.400713560372)/(o+1.)+(0.151830485765)+(0.758386313418)*0.5**o
        ref[2,2,1,1]=(0.617102098568)/(o+1.)+(0.0872825614209)+(-0.0562030350565)*0.5**o
        ref[2,2,2,0]=(-0.49484415738)/(o+1.)+(-0.684491172613)+(0.632035307709)*0.5**o
        ref[2,2,2,1]=(0.315837395005)/(o+1.)+(-0.820028413806)+(0.877117460076)*0.5**o
        ref[2,3,0,0]=(0.644195916625)/(o+1.)+(-0.0916187205818)+(0.868438479199)*0.5**o
        ref[2,3,0,1]=(-0.603972011476)/(o+1.)+(0.186052835105)+(0.58688914355)*0.5**o
        ref[2,3,1,0]=(-0.252132020652)/(o+1.)+(0.314692746683)+(-0.491821145506)*0.5**o
        ref[2,3,1,1]=(-0.969450887263)/(o+1.)+(0.436600133394)+(-0.68076808014)*0.5**o
        ref[2,3,2,0]=(1.05682018072)/(o+1.)+(-0.579837002982)+(-0.981204168827)*0.5**o
        ref[2,3,2,1]=(-0.15849056844)/(o+1.)+(0.188100780613)+(-0.979594675181)*0.5**o
        ref[2,4,0,0]=(0.717068192518)/(o+1.)+(-0.555276844034)+(0.160095467562)*0.5**o
        ref[2,4,0,1]=(0.310934776087)/(o+1.)+(-0.031025736741)+(0.121233419707)*0.5**o
        ref[2,4,1,0]=(0.649799777552)/(o+1.)+(0.387426100857)+(-0.327390540088)*0.5**o
        ref[2,4,1,1]=(0.904573217851)/(o+1.)+(-0.2650756632)+(-0.336090262681)*0.5**o
        ref[2,4,2,0]=(0.0726050384664)/(o+1.)+(-0.703987913303)+(-0.157686815732)*0.5**o
        ref[2,4,2,1]=(1.51974394408)/(o+1.)+(0.246226299226)+(-0.591186612156)*0.5**o
        ref[3,0,0,0]=(-0.493556438334)/(o+1.)+(-0.0620931951433)+(-0.777152282839)*0.5**o
        ref[3,0,0,1]=(0.1828567812)/(o+1.)+(-0.0532147973691)+(-0.152045574171)*0.5**o
        ref[3,0,1,0]=(0.115993391065)/(o+1.)+(0.160935939818)+(-0.300572515767)*0.5**o
        ref[3,0,1,1]=(-1.25632578022)/(o+1.)+(-1.26779448695)+(-0.832809662655)*0.5**o
        ref[3,0,2,0]=(-0.205237844229)/(o+1.)+(-0.0478141101127)+(0.417580286092)*0.5**o
        ref[3,0,2,1]=(0.088147349176)/(o+1.)+(-0.0547221111442)+(0.266423263811)*0.5**o
        ref[3,1,0,0]=(-0.141636395817)/(o+1.)+(0.816902574975)+(0.364336403912)*0.5**o
        ref[3,1,0,1]=(0.114717968925)/(o+1.)+(-0.317346742872)+(-0.479065041103)*0.5**o
        ref[3,1,1,0]=(0.32658366603)/(o+1.)+(-0.259502268963)+(-0.456720853948)*0.5**o
        ref[3,1,1,1]=(0.532566536046)/(o+1.)+(-0.0128035360346)+(-0.157773606009)*0.5**o
        ref[3,1,2,0]=(-0.825548552291)/(o+1.)+(-0.970837937697)+(-0.977094714538)*0.5**o
        ref[3,1,2,1]=(-1.03227496848)/(o+1.)+(0.136231209523)+(0.40095895971)*0.5**o
        ref[3,2,0,0]=(0.427487899424)/(o+1.)+(-0.453907762419)+(-0.521393633448)*0.5**o
        ref[3,2,0,1]=(0.222923180558)/(o+1.)+(0.553459753192)+(-0.60584665741)*0.5**o
        ref[3,2,1,0]=(1.07627193075)/(o+1.)+(-0.150695502095)+(0.645974743024)*0.5**o
        ref[3,2,1,1]=(-0.520178322214)/(o+1.)+(0.0799758718026)+(0.142559408818)*0.5**o
        ref[3,2,2,0]=(-0.878286404193)/(o+1.)+(-0.167961856714)+(0.814258898685)*0.5**o
        ref[3,2,2,1]=(-1.23267344558)/(o+1.)+(-0.181374761059)+(0.420183604639)*0.5**o
        ref[3,3,0,0]=(-0.489971746781)/(o+1.)+(-0.21774964099)+(-0.327300600039)*0.5**o
        ref[3,3,0,1]=(-0.215536051747)/(o+1.)+(0.496650339992)+(0.808391338918)*0.5**o
        ref[3,3,1,0]=(1.14758815112)/(o+1.)+(0.423354417412)+(0.0408315227595)*0.5**o
        ref[3,3,1,1]=(1.36425547565)/(o+1.)+(0.260931116761)+(0.488110633719)*0.5**o
        ref[3,3,2,0]=(0.308591233754)/(o+1.)+(0.160989446234)+(0.698445367492)*0.5**o
        ref[3,3,2,1]=(1.48026728491)/(o+1.)+(0.725237962208)+(-0.619448487664)*0.5**o
        ref[3,4,0,0]=(-1.11249020744)/(o+1.)+(0.442829136751)+(0.653712053876)*0.5**o
        ref[3,4,0,1]=(0.312528836193)/(o+1.)+(-0.232609432124)+(0.519872654154)*0.5**o
        ref[3,4,1,0]=(0.278961378214)/(o+1.)+(-0.249483950874)+(-0.561163638499)*0.5**o
        ref[3,4,1,1]=(1.16646663032)/(o+1.)+(0.183095609961)+(0.0594382754162)*0.5**o
        ref[3,4,2,0]=(0.631131833509)/(o+1.)+(0.390780580541)+(0.0680763742436)*0.5**o
        ref[3,4,2,1]=(0.969719660142)/(o+1.)+(0.113155244129)+(0.154883537954)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactZero_fromSymbol_Solution_rank0(self):
      """
      tests integral of rank 0 Symbol on the ReducedFunctionOnContactZero

      assumptions: Solution(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      w=Solution(self.domain)
      x=w.getX()
      arg=Symbol(shape=())
      s=Data(0,(),w)
      if dim==2:
        s=(0.551831061348)*x[0]**o+(-0.711823790733)*x[0]+(0.766098862021)*x[1]**o+(0.348439108965)*x[1]
        ref=(0.766098862021)/(o+1.)+(-0.181692340884)+(0.551831061348)*0.5**o
      else:
        s=(-0.534750207803)*x[0]**o+(0.67975160623)*x[0]+(0.747517712959)*x[1]**o+(0.392190678153)*x[1]+(-0.586558763208)*x[2]**o+(0.67325251011)*x[2]
        ref=(0.160958949751)/(o+1.)+(0.872597397246)+(-0.534750207803)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactZero_fromSymbol_Solution_rank1(self):
      """
      tests integral of rank 1 Symbol on the ReducedFunctionOnContactZero

      assumptions: Solution(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      w=Solution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(2,))
      s=Data(0,(2,),w)
      ref=numpy.zeros((2,),numpy.float_)
      if dim==2:
        s[0]=(0.824614999579)*x[0]**o+(-0.92501072999)*x[0]+(-0.164852435473)*x[1]**o+(0.625787035398)*x[1]
        s[1]=(0.933187313904)*x[0]**o+(-0.76410941839)*x[0]+(-0.924559271586)*x[1]**o+(-0.951797707375)*x[1]
        ref[0]=(-0.164852435473)/(o+1.)+(-0.149611847296)+(0.824614999579)*0.5**o
        ref[1]=(-0.924559271586)/(o+1.)+(-0.857953562883)+(0.933187313904)*0.5**o
      else:
        s[0]=(-0.73247014084)*x[0]**o+(-0.447784692568)*x[0]+(-0.206145797624)*x[1]**o+(0.800549366557)*x[1]+(0.0980004782263)*x[2]**o+(0.68684151518)*x[2]
        s[1]=(0.734011170766)*x[0]**o+(-0.718028398303)*x[0]+(0.768940123369)*x[1]**o+(0.410156990187)*x[1]+(0.6562835776)*x[2]**o+(-0.899108897402)*x[2]
        ref[0]=(-0.108145319398)/(o+1.)+(0.519803094585)+(-0.73247014084)*0.5**o
        ref[1]=(1.42522370097)/(o+1.)+(-0.603490152759)+(0.734011170766)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactZero_fromSymbol_Solution_rank2(self):
      """
      tests integral of rank 2 Symbol on the ReducedFunctionOnContactZero

      assumptions: Solution(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      w=Solution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5))
      s=Data(0,(4, 5),w)
      ref=numpy.zeros((4, 5),numpy.float_)
      if dim==2:
        s[0,0]=(0.860004136579)*x[0]**o+(-0.117366140441)*x[0]+(0.0876424200904)*x[1]**o+(0.0578427219547)*x[1]
        s[0,1]=(-0.934262599682)*x[0]**o+(-0.834391940774)*x[0]+(-0.37703046074)*x[1]**o+(0.571779357388)*x[1]
        s[0,2]=(-0.524417646775)*x[0]**o+(-0.562788330473)*x[0]+(-0.454146889642)*x[1]**o+(0.382147150671)*x[1]
        s[0,3]=(0.53752054491)*x[0]**o+(0.52118522583)*x[0]+(0.196117385249)*x[1]**o+(0.968096844135)*x[1]
        s[0,4]=(-0.346245303793)*x[0]**o+(0.169183126761)*x[0]+(0.35430553025)*x[1]**o+(-0.1953526461)*x[1]
        s[1,0]=(0.110897459128)*x[0]**o+(-0.189310140801)*x[0]+(-0.503153929692)*x[1]**o+(-0.776512889143)*x[1]
        s[1,1]=(0.0773234920212)*x[0]**o+(-0.323298685849)*x[0]+(0.956652842652)*x[1]**o+(-0.834584362886)*x[1]
        s[1,2]=(0.725330064304)*x[0]**o+(-0.761117907603)*x[0]+(-0.161152622185)*x[1]**o+(0.633185715895)*x[1]
        s[1,3]=(0.399871656297)*x[0]**o+(-0.831922720579)*x[0]+(0.125957093778)*x[1]**o+(0.909312599897)*x[1]
        s[1,4]=(0.450333632811)*x[0]**o+(-0.914467685845)*x[0]+(-0.71553040823)*x[1]**o+(-0.806957497854)*x[1]
        s[2,0]=(-0.596111663177)*x[0]**o+(-0.896500016447)*x[0]+(-0.175137342743)*x[1]**o+(-0.519618663869)*x[1]
        s[2,1]=(0.219789961005)*x[0]**o+(0.394778437788)*x[0]+(0.516359075868)*x[1]**o+(0.158719123393)*x[1]
        s[2,2]=(-0.202978715302)*x[0]**o+(0.326528455071)*x[0]+(-0.475802171068)*x[1]**o+(-0.470892970453)*x[1]
        s[2,3]=(0.34431665114)*x[0]**o+(0.847623303071)*x[0]+(-0.300788565801)*x[1]**o+(0.623946324835)*x[1]
        s[2,4]=(-0.482716534989)*x[0]**o+(0.610848202802)*x[0]+(-0.618220314271)*x[1]**o+(-0.270417044737)*x[1]
        s[3,0]=(0.323084711653)*x[0]**o+(0.870297617987)*x[0]+(-0.817231621484)*x[1]**o+(0.599137155405)*x[1]
        s[3,1]=(-0.464413485909)*x[0]**o+(-0.948296858226)*x[0]+(-0.872776322969)*x[1]**o+(0.0058301771792)*x[1]
        s[3,2]=(-0.920685763376)*x[0]**o+(-0.429364041758)*x[0]+(0.211630014364)*x[1]**o+(-0.311548404071)*x[1]
        s[3,3]=(-0.175827412744)*x[0]**o+(-0.687378086381)*x[0]+(0.265852072745)*x[1]**o+(0.799115907363)*x[1]
        s[3,4]=(-0.99517403847)*x[0]**o+(0.746397561862)*x[0]+(-0.747340816778)*x[1]**o+(-0.57474910244)*x[1]
        ref[0,0]=(0.0876424200904)/(o+1.)+(-0.0297617092432)+(0.860004136579)*0.5**o
        ref[0,1]=(-0.37703046074)/(o+1.)+(-0.131306291693)+(-0.934262599682)*0.5**o
        ref[0,2]=(-0.454146889642)/(o+1.)+(-0.0903205899012)+(-0.524417646775)*0.5**o
        ref[0,3]=(0.196117385249)/(o+1.)+(0.744641034982)+(0.53752054491)*0.5**o
        ref[0,4]=(0.35430553025)/(o+1.)+(-0.0130847596698)+(-0.346245303793)*0.5**o
        ref[1,0]=(-0.503153929692)/(o+1.)+(-0.482911514972)+(0.110897459128)*0.5**o
        ref[1,1]=(0.956652842652)/(o+1.)+(-0.578941524367)+(0.0773234920212)*0.5**o
        ref[1,2]=(-0.161152622185)/(o+1.)+(-0.063966095854)+(0.725330064304)*0.5**o
        ref[1,3]=(0.125957093778)/(o+1.)+(0.0386949396593)+(0.399871656297)*0.5**o
        ref[1,4]=(-0.71553040823)/(o+1.)+(-0.860712591849)+(0.450333632811)*0.5**o
        ref[2,0]=(-0.175137342743)/(o+1.)+(-0.708059340158)+(-0.596111663177)*0.5**o
        ref[2,1]=(0.516359075868)/(o+1.)+(0.27674878059)+(0.219789961005)*0.5**o
        ref[2,2]=(-0.475802171068)/(o+1.)+(-0.072182257691)+(-0.202978715302)*0.5**o
        ref[2,3]=(-0.300788565801)/(o+1.)+(0.735784813953)+(0.34431665114)*0.5**o
        ref[2,4]=(-0.618220314271)/(o+1.)+(0.170215579033)+(-0.482716534989)*0.5**o
        ref[3,0]=(-0.817231621484)/(o+1.)+(0.734717386696)+(0.323084711653)*0.5**o
        ref[3,1]=(-0.872776322969)/(o+1.)+(-0.471233340523)+(-0.464413485909)*0.5**o
        ref[3,2]=(0.211630014364)/(o+1.)+(-0.370456222914)+(-0.920685763376)*0.5**o
        ref[3,3]=(0.265852072745)/(o+1.)+(0.0558689104915)+(-0.175827412744)*0.5**o
        ref[3,4]=(-0.747340816778)/(o+1.)+(0.0858242297109)+(-0.99517403847)*0.5**o
      else:
        s[0,0]=(0.578101616539)*x[0]**o+(-0.941355465542)*x[0]+(-0.661840086733)*x[1]**o+(-0.574686372209)*x[1]+(-0.488980453488)*x[2]**o+(0.8057154356)*x[2]
        s[0,1]=(0.188043130009)*x[0]**o+(-0.53795144833)*x[0]+(0.585611542575)*x[1]**o+(-0.394354708911)*x[1]+(-0.753395938984)*x[2]**o+(-0.775038773873)*x[2]
        s[0,2]=(-0.0069454884707)*x[0]**o+(0.925798676533)*x[0]+(-0.617492289762)*x[1]**o+(-0.412950379597)*x[1]+(0.530650448811)*x[2]**o+(-0.882156765835)*x[2]
        s[0,3]=(-0.109411491583)*x[0]**o+(0.794417685565)*x[0]+(0.395199315447)*x[1]**o+(0.221963288858)*x[1]+(0.944618017853)*x[2]**o+(0.710576727198)*x[2]
        s[0,4]=(0.0376370205208)*x[0]**o+(-0.387469327476)*x[0]+(0.989435100596)*x[1]**o+(-0.697198283115)*x[1]+(0.705109597826)*x[2]**o+(0.00235357316422)*x[2]
        s[1,0]=(-0.861013305772)*x[0]**o+(-0.117175144313)*x[0]+(-0.851114880006)*x[1]**o+(0.830657298231)*x[1]+(0.897872709447)*x[2]**o+(-0.500434787953)*x[2]
        s[1,1]=(-0.659762659407)*x[0]**o+(-0.293584253789)*x[0]+(-0.549133974878)*x[1]**o+(0.124482383522)*x[1]+(-0.71415106775)*x[2]**o+(-0.452353669527)*x[2]
        s[1,2]=(-0.944798023011)*x[0]**o+(0.352545190549)*x[0]+(-0.950981266242)*x[1]**o+(-0.338339076366)*x[1]+(-0.385100481358)*x[2]**o+(-0.751511160424)*x[2]
        s[1,3]=(-0.514889491143)*x[0]**o+(-0.0573778568353)*x[0]+(0.189167569167)*x[1]**o+(-0.122152801538)*x[1]+(-0.075952059447)*x[2]**o+(-0.575271878121)*x[2]
        s[1,4]=(0.092128759086)*x[0]**o+(-0.391395765897)*x[0]+(0.621302034853)*x[1]**o+(-0.169193910488)*x[1]+(0.895953307228)*x[2]**o+(-0.171715374513)*x[2]
        s[2,0]=(-0.896341445352)*x[0]**o+(0.55427420625)*x[0]+(-0.68618123058)*x[1]**o+(0.586997200722)*x[1]+(-0.16193062027)*x[2]**o+(0.47614229102)*x[2]
        s[2,1]=(0.0173555065441)*x[0]**o+(-0.137706680345)*x[0]+(0.124390959958)*x[1]**o+(0.0817218208151)*x[1]+(-0.883487601024)*x[2]**o+(-0.99725370538)*x[2]
        s[2,2]=(0.653947795759)*x[0]**o+(-0.0561188253491)*x[0]+(-0.304869234336)*x[1]**o+(-0.5647971996)*x[1]+(0.842979371974)*x[2]**o+(0.271112357653)*x[2]
        s[2,3]=(-0.916870447792)*x[0]**o+(0.532760446169)*x[0]+(0.125531293566)*x[1]**o+(-0.772485238164)*x[1]+(-0.0249888509834)*x[2]**o+(-0.215774489527)*x[2]
        s[2,4]=(0.82957204564)*x[0]**o+(-0.536668870818)*x[0]+(0.18150637899)*x[1]**o+(0.213776071225)*x[1]+(-0.607190995474)*x[2]**o+(-0.0534051775785)*x[2]
        s[3,0]=(-0.0156695889369)*x[0]**o+(-0.920483023826)*x[0]+(0.173953779108)*x[1]**o+(0.723326965312)*x[1]+(-0.946445962586)*x[2]**o+(0.242480497946)*x[2]
        s[3,1]=(0.253546369583)*x[0]**o+(-0.455303700999)*x[0]+(0.265484731924)*x[1]**o+(-0.334855283665)*x[1]+(0.263585801553)*x[2]**o+(0.768099093921)*x[2]
        s[3,2]=(-0.0884797551735)*x[0]**o+(-0.744966141189)*x[0]+(0.685728482818)*x[1]**o+(-0.700119204761)*x[1]+(-0.996053069429)*x[2]**o+(-0.300758647065)*x[2]
        s[3,3]=(-0.265221319311)*x[0]**o+(0.119741272443)*x[0]+(0.0623093135308)*x[1]**o+(0.617188763307)*x[1]+(-0.14935010921)*x[2]**o+(0.315030342521)*x[2]
        s[3,4]=(0.634255080399)*x[0]**o+(0.0848130779284)*x[0]+(-0.262488356592)*x[1]**o+(-0.698599321231)*x[1]+(-0.411566742148)*x[2]**o+(0.307995972853)*x[2]
        ref[0,0]=(-1.15082054022)/(o+1.)+(-0.355163201075)+(0.578101616539)*0.5**o
        ref[0,1]=(-0.167784396409)/(o+1.)+(-0.853672465557)+(0.188043130009)*0.5**o
        ref[0,2]=(-0.0868418409506)/(o+1.)+(-0.18465423445)+(-0.0069454884707)*0.5**o
        ref[0,3]=(1.3398173333)/(o+1.)+(0.86347885081)+(-0.109411491583)*0.5**o
        ref[0,4]=(1.69454469842)/(o+1.)+(-0.541157018713)+(0.0376370205208)*0.5**o
        ref[1,0]=(0.0467578294411)/(o+1.)+(0.106523682982)+(-0.861013305772)*0.5**o
        ref[1,1]=(-1.26328504263)/(o+1.)+(-0.310727769897)+(-0.659762659407)*0.5**o
        ref[1,2]=(-1.3360817476)/(o+1.)+(-0.36865252312)+(-0.944798023011)*0.5**o
        ref[1,3]=(0.11321550972)/(o+1.)+(-0.377401268247)+(-0.514889491143)*0.5**o
        ref[1,4]=(1.51725534208)/(o+1.)+(-0.366152525449)+(0.092128759086)*0.5**o
        ref[2,0]=(-0.84811185085)/(o+1.)+(0.808706848996)+(-0.896341445352)*0.5**o
        ref[2,1]=(-0.759096641066)/(o+1.)+(-0.526619282455)+(0.0173555065441)*0.5**o
        ref[2,2]=(0.538110137638)/(o+1.)+(-0.174901833648)+(0.653947795759)*0.5**o
        ref[2,3]=(0.100542442583)/(o+1.)+(-0.227749640761)+(-0.916870447792)*0.5**o
        ref[2,4]=(-0.425684616484)/(o+1.)+(-0.188148988586)+(0.82957204564)*0.5**o
        ref[3,0]=(-0.772492183478)/(o+1.)+(0.0226622197163)+(-0.0156695889369)*0.5**o
        ref[3,1]=(0.529070533478)/(o+1.)+(-0.0110299453715)+(0.253546369583)*0.5**o
        ref[3,2]=(-0.310324586611)/(o+1.)+(-0.872921996508)+(-0.0884797551735)*0.5**o
        ref[3,3]=(-0.0870407956792)/(o+1.)+(0.525980189135)+(-0.265221319311)*0.5**o
        ref[3,4]=(-0.67405509874)/(o+1.)+(-0.152895135225)+(0.634255080399)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactZero_fromSymbol_Solution_rank3(self):
      """
      tests integral of rank 3 Symbol on the ReducedFunctionOnContactZero

      assumptions: Solution(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      w=Solution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(6, 2, 2))
      s=Data(0,(6, 2, 2),w)
      ref=numpy.zeros((6, 2, 2),numpy.float_)
      if dim==2:
        s[0,0,0]=(-0.970014070821)*x[0]**o+(0.195828821364)*x[0]+(-0.329248582962)*x[1]**o+(0.765729760657)*x[1]
        s[0,0,1]=(-0.78480276531)*x[0]**o+(0.660249421528)*x[0]+(-0.112211565471)*x[1]**o+(-0.732753874188)*x[1]
        s[0,1,0]=(-0.610728226383)*x[0]**o+(-0.617391668915)*x[0]+(-0.755934195771)*x[1]**o+(-0.301037750184)*x[1]
        s[0,1,1]=(-0.271483777605)*x[0]**o+(-0.118144079798)*x[0]+(0.00595756313744)*x[1]**o+(0.00973544005055)*x[1]
        s[1,0,0]=(-0.880152279586)*x[0]**o+(0.173587205863)*x[0]+(-0.772256433861)*x[1]**o+(-0.929831194333)*x[1]
        s[1,0,1]=(0.663917147636)*x[0]**o+(0.602039234824)*x[0]+(-0.761663518039)*x[1]**o+(-0.794981650104)*x[1]
        s[1,1,0]=(0.669982500097)*x[0]**o+(-0.867340940384)*x[0]+(0.425166963846)*x[1]**o+(0.809034058423)*x[1]
        s[1,1,1]=(-0.748164617999)*x[0]**o+(0.820897422648)*x[0]+(0.282790963068)*x[1]**o+(0.418210593072)*x[1]
        s[2,0,0]=(-0.151105173252)*x[0]**o+(0.588856734796)*x[0]+(-0.587574498534)*x[1]**o+(0.214690992942)*x[1]
        s[2,0,1]=(0.846293738229)*x[0]**o+(0.453778324604)*x[0]+(-0.750479939889)*x[1]**o+(0.988318560374)*x[1]
        s[2,1,0]=(0.0520443817173)*x[0]**o+(0.66477304094)*x[0]+(0.220203729906)*x[1]**o+(0.807678486344)*x[1]
        s[2,1,1]=(-0.481828231947)*x[0]**o+(0.301163693914)*x[0]+(-0.240012695397)*x[1]**o+(-0.739330362354)*x[1]
        s[3,0,0]=(-0.464086304345)*x[0]**o+(0.204976865077)*x[0]+(0.360828971652)*x[1]**o+(0.179421383025)*x[1]
        s[3,0,1]=(0.524003162473)*x[0]**o+(0.241725193779)*x[0]+(0.137006148453)*x[1]**o+(0.122270180832)*x[1]
        s[3,1,0]=(-0.148537542555)*x[0]**o+(-0.87771789463)*x[0]+(0.776889116646)*x[1]**o+(0.72202598779)*x[1]
        s[3,1,1]=(-0.0984207735775)*x[0]**o+(-0.200545725506)*x[0]+(-0.422044149217)*x[1]**o+(-0.403374011642)*x[1]
        s[4,0,0]=(0.332893223127)*x[0]**o+(-0.238038393676)*x[0]+(-0.796376263577)*x[1]**o+(0.059346957422)*x[1]
        s[4,0,1]=(-0.877726930853)*x[0]**o+(0.287774676493)*x[0]+(-0.690097100077)*x[1]**o+(0.806523530334)*x[1]
        s[4,1,0]=(-0.466369745006)*x[0]**o+(0.511932688176)*x[0]+(-0.569648246707)*x[1]**o+(-0.180363948019)*x[1]
        s[4,1,1]=(0.82395338594)*x[0]**o+(0.0423959362879)*x[0]+(-0.508349189015)*x[1]**o+(0.0735939835821)*x[1]
        s[5,0,0]=(-0.631799410304)*x[0]**o+(0.0828019140935)*x[0]+(0.132222069751)*x[1]**o+(0.716700894256)*x[1]
        s[5,0,1]=(-0.871739775831)*x[0]**o+(-0.364312533128)*x[0]+(0.660932493381)*x[1]**o+(-0.638799141415)*x[1]
        s[5,1,0]=(-0.927257402325)*x[0]**o+(-0.745507370225)*x[0]+(0.70647119912)*x[1]**o+(0.583112912025)*x[1]
        s[5,1,1]=(-0.440675815785)*x[0]**o+(0.292262539618)*x[0]+(0.172815874747)*x[1]**o+(-0.854959570718)*x[1]
        ref[0,0,0]=(-0.329248582962)/(o+1.)+(0.480779291011)+(-0.970014070821)*0.5**o
        ref[0,0,1]=(-0.112211565471)/(o+1.)+(-0.0362522263304)+(-0.78480276531)*0.5**o
        ref[0,1,0]=(-0.755934195771)/(o+1.)+(-0.459214709549)+(-0.610728226383)*0.5**o
        ref[0,1,1]=(0.00595756313744)/(o+1.)+(-0.0542043198736)+(-0.271483777605)*0.5**o
        ref[1,0,0]=(-0.772256433861)/(o+1.)+(-0.378121994235)+(-0.880152279586)*0.5**o
        ref[1,0,1]=(-0.761663518039)/(o+1.)+(-0.0964712076398)+(0.663917147636)*0.5**o
        ref[1,1,0]=(0.425166963846)/(o+1.)+(-0.0291534409806)+(0.669982500097)*0.5**o
        ref[1,1,1]=(0.282790963068)/(o+1.)+(0.61955400786)+(-0.748164617999)*0.5**o
        ref[2,0,0]=(-0.587574498534)/(o+1.)+(0.401773863869)+(-0.151105173252)*0.5**o
        ref[2,0,1]=(-0.750479939889)/(o+1.)+(0.721048442489)+(0.846293738229)*0.5**o
        ref[2,1,0]=(0.220203729906)/(o+1.)+(0.736225763642)+(0.0520443817173)*0.5**o
        ref[2,1,1]=(-0.240012695397)/(o+1.)+(-0.21908333422)+(-0.481828231947)*0.5**o
        ref[3,0,0]=(0.360828971652)/(o+1.)+(0.192199124051)+(-0.464086304345)*0.5**o
        ref[3,0,1]=(0.137006148453)/(o+1.)+(0.181997687306)+(0.524003162473)*0.5**o
        ref[3,1,0]=(0.776889116646)/(o+1.)+(-0.0778459534199)+(-0.148537542555)*0.5**o
        ref[3,1,1]=(-0.422044149217)/(o+1.)+(-0.301959868574)+(-0.0984207735775)*0.5**o
        ref[4,0,0]=(-0.796376263577)/(o+1.)+(-0.0893457181268)+(0.332893223127)*0.5**o
        ref[4,0,1]=(-0.690097100077)/(o+1.)+(0.547149103413)+(-0.877726930853)*0.5**o
        ref[4,1,0]=(-0.569648246707)/(o+1.)+(0.165784370078)+(-0.466369745006)*0.5**o
        ref[4,1,1]=(-0.508349189015)/(o+1.)+(0.057994959935)+(0.82395338594)*0.5**o
        ref[5,0,0]=(0.132222069751)/(o+1.)+(0.399751404175)+(-0.631799410304)*0.5**o
        ref[5,0,1]=(0.660932493381)/(o+1.)+(-0.501555837272)+(-0.871739775831)*0.5**o
        ref[5,1,0]=(0.70647119912)/(o+1.)+(-0.0811972291003)+(-0.927257402325)*0.5**o
        ref[5,1,1]=(0.172815874747)/(o+1.)+(-0.28134851555)+(-0.440675815785)*0.5**o
      else:
        s[0,0,0]=(-0.565533896373)*x[0]**o+(-0.0872485111367)*x[0]+(-0.283353388539)*x[1]**o+(-0.152989536217)*x[1]+(-0.444002969358)*x[2]**o+(0.670423385061)*x[2]
        s[0,0,1]=(-0.0944154284688)*x[0]**o+(-0.884982028889)*x[0]+(0.79965600284)*x[1]**o+(-0.422245813699)*x[1]+(-0.479173724334)*x[2]**o+(-0.726826676767)*x[2]
        s[0,1,0]=(0.789722540135)*x[0]**o+(-0.0774752885776)*x[0]+(-0.95515742871)*x[1]**o+(0.300583198538)*x[1]+(0.485489410617)*x[2]**o+(-0.303592354814)*x[2]
        s[0,1,1]=(0.652206005345)*x[0]**o+(0.90387230638)*x[0]+(-0.943316098191)*x[1]**o+(-0.600569529668)*x[1]+(-0.157071495888)*x[2]**o+(0.289655248779)*x[2]
        s[1,0,0]=(0.963881471424)*x[0]**o+(-0.67090862423)*x[0]+(-0.56250637167)*x[1]**o+(0.432938287823)*x[1]+(0.632690957193)*x[2]**o+(0.437978271014)*x[2]
        s[1,0,1]=(0.211364301181)*x[0]**o+(-0.482725619403)*x[0]+(0.818668277765)*x[1]**o+(-0.202108275895)*x[1]+(0.38755788328)*x[2]**o+(0.315009480388)*x[2]
        s[1,1,0]=(-0.931623738361)*x[0]**o+(0.865171378595)*x[0]+(0.983563036962)*x[1]**o+(-0.674022850075)*x[1]+(-0.743850008639)*x[2]**o+(-0.666081693537)*x[2]
        s[1,1,1]=(0.00368798622506)*x[0]**o+(0.431757463365)*x[0]+(-0.348566468597)*x[1]**o+(0.451792881229)*x[1]+(-0.380511811466)*x[2]**o+(0.850391058252)*x[2]
        s[2,0,0]=(0.0418268780255)*x[0]**o+(0.749915304085)*x[0]+(0.155766457341)*x[1]**o+(0.782071885494)*x[1]+(-0.539808510551)*x[2]**o+(0.483245601178)*x[2]
        s[2,0,1]=(-0.460744056666)*x[0]**o+(0.124701348548)*x[0]+(-0.703875182049)*x[1]**o+(0.796527478381)*x[1]+(-0.221938800568)*x[2]**o+(0.971830341802)*x[2]
        s[2,1,0]=(0.53954548965)*x[0]**o+(0.0162996837218)*x[0]+(0.142104929682)*x[1]**o+(-0.971657840256)*x[1]+(-0.503179671457)*x[2]**o+(-0.542014263985)*x[2]
        s[2,1,1]=(0.908980902057)*x[0]**o+(-0.502995849973)*x[0]+(-0.306045266955)*x[1]**o+(0.663346607536)*x[1]+(-0.350114579334)*x[2]**o+(0.576410602991)*x[2]
        s[3,0,0]=(0.813242844535)*x[0]**o+(-0.373751247405)*x[0]+(-0.155529865713)*x[1]**o+(-0.757860853884)*x[1]+(0.834637852603)*x[2]**o+(0.909806811712)*x[2]
        s[3,0,1]=(-0.287829796913)*x[0]**o+(-0.521503208519)*x[0]+(0.373235117298)*x[1]**o+(0.820819656976)*x[1]+(0.666911247106)*x[2]**o+(-0.0367218914776)*x[2]
        s[3,1,0]=(0.432791246384)*x[0]**o+(0.958703389048)*x[0]+(0.964027641291)*x[1]**o+(0.546429571768)*x[1]+(-0.299474553396)*x[2]**o+(-0.699665640827)*x[2]
        s[3,1,1]=(0.728884851779)*x[0]**o+(0.650980056729)*x[0]+(0.133592857822)*x[1]**o+(0.693391334869)*x[1]+(-0.836602488593)*x[2]**o+(-0.442753441073)*x[2]
        s[4,0,0]=(0.833243436386)*x[0]**o+(0.980178093504)*x[0]+(-0.819050787144)*x[1]**o+(0.797889271257)*x[1]+(-0.723026474407)*x[2]**o+(0.783764698489)*x[2]
        s[4,0,1]=(-0.452251186626)*x[0]**o+(-0.627980945947)*x[0]+(-0.462394480861)*x[1]**o+(-0.591297748282)*x[1]+(-0.630289931985)*x[2]**o+(0.495939505391)*x[2]
        s[4,1,0]=(-0.774984503568)*x[0]**o+(0.635843202085)*x[0]+(0.835305935226)*x[1]**o+(-0.0209656725511)*x[1]+(0.257099035711)*x[2]**o+(-0.735192237148)*x[2]
        s[4,1,1]=(0.919373693068)*x[0]**o+(0.38072204474)*x[0]+(-0.92955685309)*x[1]**o+(-0.145126274743)*x[1]+(-0.50004893122)*x[2]**o+(-0.252139341208)*x[2]
        s[5,0,0]=(0.802245464817)*x[0]**o+(0.617146384769)*x[0]+(0.0802760805435)*x[1]**o+(-0.698667208547)*x[1]+(-0.531164225022)*x[2]**o+(-0.761354857435)*x[2]
        s[5,0,1]=(-0.400952829207)*x[0]**o+(0.605414258928)*x[0]+(0.563769114393)*x[1]**o+(0.952187363261)*x[1]+(-0.539409233701)*x[2]**o+(-0.680444521941)*x[2]
        s[5,1,0]=(0.0304057674653)*x[0]**o+(0.190919897349)*x[0]+(-0.812827460152)*x[1]**o+(0.822295166707)*x[1]+(0.678796730302)*x[2]**o+(-0.648229118268)*x[2]
        s[5,1,1]=(0.644695792666)*x[0]**o+(-0.0232924889471)*x[0]+(0.294470818979)*x[1]**o+(0.626411257894)*x[1]+(-0.453808003683)*x[2]**o+(0.89191417949)*x[2]
        ref[0,0,0]=(-0.727356357898)/(o+1.)+(0.215092668853)+(-0.565533896373)*0.5**o
        ref[0,0,1]=(0.320482278506)/(o+1.)+(-1.01702725968)+(-0.0944154284688)*0.5**o
        ref[0,1,0]=(-0.469668018093)/(o+1.)+(-0.0402422224272)+(0.789722540135)*0.5**o
        ref[0,1,1]=(-1.10038759408)/(o+1.)+(0.296479012746)+(0.652206005345)*0.5**o
        ref[1,0,0]=(0.0701845855234)/(o+1.)+(0.100003967304)+(0.963881471424)*0.5**o
        ref[1,0,1]=(1.20622616105)/(o+1.)+(-0.184912207455)+(0.211364301181)*0.5**o
        ref[1,1,0]=(0.239713028324)/(o+1.)+(-0.237466582508)+(-0.931623738361)*0.5**o
        ref[1,1,1]=(-0.729078280063)/(o+1.)+(0.866970701424)+(0.00368798622506)*0.5**o
        ref[2,0,0]=(-0.38404205321)/(o+1.)+(1.00761639538)+(0.0418268780255)*0.5**o
        ref[2,0,1]=(-0.925813982617)/(o+1.)+(0.946529584366)+(-0.460744056666)*0.5**o
        ref[2,1,0]=(-0.361074741775)/(o+1.)+(-0.74868621026)+(0.53954548965)*0.5**o
        ref[2,1,1]=(-0.656159846289)/(o+1.)+(0.368380680277)+(0.908980902057)*0.5**o
        ref[3,0,0]=(0.67910798689)/(o+1.)+(-0.110902644788)+(0.813242844535)*0.5**o
        ref[3,0,1]=(1.0401463644)/(o+1.)+(0.13129727849)+(-0.287829796913)*0.5**o
        ref[3,1,0]=(0.664553087895)/(o+1.)+(0.402733659994)+(0.432791246384)*0.5**o
        ref[3,1,1]=(-0.703009630771)/(o+1.)+(0.450808975263)+(0.728884851779)*0.5**o
        ref[4,0,0]=(-1.54207726155)/(o+1.)+(1.28091603163)+(0.833243436386)*0.5**o
        ref[4,0,1]=(-1.09268441285)/(o+1.)+(-0.361669594419)+(-0.452251186626)*0.5**o
        ref[4,1,0]=(1.09240497094)/(o+1.)+(-0.0601573538071)+(-0.774984503568)*0.5**o
        ref[4,1,1]=(-1.42960578431)/(o+1.)+(-0.00827178560514)+(0.919373693068)*0.5**o
        ref[5,0,0]=(-0.450888144479)/(o+1.)+(-0.421437840607)+(0.802245464817)*0.5**o
        ref[5,0,1]=(0.0243598806918)/(o+1.)+(0.438578550124)+(-0.400952829207)*0.5**o
        ref[5,1,0]=(-0.13403072985)/(o+1.)+(0.182492972894)+(0.0304057674653)*0.5**o
        ref[5,1,1]=(-0.159337184705)/(o+1.)+(0.747516474218)+(0.644695792666)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactZero_fromSymbol_Solution_rank4(self):
      """
      tests integral of rank 4 Symbol on the ReducedFunctionOnContactZero

      assumptions: Solution(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      w=Solution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5, 3, 2))
      s=Data(0,(4, 5, 3, 2),w)
      ref=numpy.zeros((4, 5, 3, 2),numpy.float_)
      if dim==2:
        s[0,0,0,0]=(0.0390601545717)*x[0]**o+(-0.222613292224)*x[0]+(0.246692793786)*x[1]**o+(-0.0199554145217)*x[1]
        s[0,0,0,1]=(0.608816949634)*x[0]**o+(0.840495963273)*x[0]+(-0.986136095398)*x[1]**o+(0.16852755525)*x[1]
        s[0,0,1,0]=(0.580621174141)*x[0]**o+(-0.931460241997)*x[0]+(0.725705948245)*x[1]**o+(0.0239157549247)*x[1]
        s[0,0,1,1]=(0.388567471102)*x[0]**o+(0.311382975022)*x[0]+(-0.376543304375)*x[1]**o+(-0.859691633345)*x[1]
        s[0,0,2,0]=(-0.692988033922)*x[0]**o+(-0.059082135717)*x[0]+(-0.088525361736)*x[1]**o+(-0.435255796923)*x[1]
        s[0,0,2,1]=(0.274277434778)*x[0]**o+(0.783272547433)*x[0]+(-0.380234357131)*x[1]**o+(-0.355490082493)*x[1]
        s[0,1,0,0]=(-0.802394756916)*x[0]**o+(-0.352698691101)*x[0]+(0.335411545595)*x[1]**o+(0.558556090302)*x[1]
        s[0,1,0,1]=(0.174271017656)*x[0]**o+(-0.442785108924)*x[0]+(0.178161299328)*x[1]**o+(-0.18823731442)*x[1]
        s[0,1,1,0]=(-0.598879030318)*x[0]**o+(0.373869209105)*x[0]+(0.589565043365)*x[1]**o+(-0.62976640408)*x[1]
        s[0,1,1,1]=(-0.667493955405)*x[0]**o+(0.998759793079)*x[0]+(-0.743341570183)*x[1]**o+(-0.0859848525819)*x[1]
        s[0,1,2,0]=(0.47631386081)*x[0]**o+(0.0151722662021)*x[0]+(0.899075872977)*x[1]**o+(0.0678095622752)*x[1]
        s[0,1,2,1]=(-0.497833539416)*x[0]**o+(0.00600269705464)*x[0]+(-0.541353591572)*x[1]**o+(0.0705426009397)*x[1]
        s[0,2,0,0]=(-0.735607995943)*x[0]**o+(0.921942121227)*x[0]+(-0.697588210675)*x[1]**o+(-0.555902048922)*x[1]
        s[0,2,0,1]=(0.705776536011)*x[0]**o+(0.202303757789)*x[0]+(-0.630771158486)*x[1]**o+(-0.0166043126446)*x[1]
        s[0,2,1,0]=(-0.975134219189)*x[0]**o+(0.566584421811)*x[0]+(0.315121166158)*x[1]**o+(-0.952791171948)*x[1]
        s[0,2,1,1]=(-0.43751086407)*x[0]**o+(0.793023085525)*x[0]+(-0.471199351714)*x[1]**o+(0.227828159087)*x[1]
        s[0,2,2,0]=(0.526770705207)*x[0]**o+(-0.779856515281)*x[0]+(0.433608622434)*x[1]**o+(0.45679766746)*x[1]
        s[0,2,2,1]=(-0.418834628035)*x[0]**o+(0.325385358616)*x[0]+(0.0493546850323)*x[1]**o+(-0.611087977891)*x[1]
        s[0,3,0,0]=(0.302691082154)*x[0]**o+(-0.0840254373723)*x[0]+(0.235788766456)*x[1]**o+(0.00697145365732)*x[1]
        s[0,3,0,1]=(-0.400507082213)*x[0]**o+(0.922575813908)*x[0]+(0.653142282067)*x[1]**o+(-0.819060793296)*x[1]
        s[0,3,1,0]=(-0.542123872101)*x[0]**o+(-0.386997884801)*x[0]+(-0.0222151576861)*x[1]**o+(-0.914062036349)*x[1]
        s[0,3,1,1]=(0.0629454961983)*x[0]**o+(0.953558638015)*x[0]+(-0.977561971438)*x[1]**o+(-0.3034113517)*x[1]
        s[0,3,2,0]=(0.088224801816)*x[0]**o+(-0.0370702186771)*x[0]+(0.094320839406)*x[1]**o+(-0.342299582748)*x[1]
        s[0,3,2,1]=(0.976917465786)*x[0]**o+(-0.528174142655)*x[0]+(-0.387883052017)*x[1]**o+(-0.631752291799)*x[1]
        s[0,4,0,0]=(-0.468335475351)*x[0]**o+(-0.565555734167)*x[0]+(-0.308997447406)*x[1]**o+(-0.747826956462)*x[1]
        s[0,4,0,1]=(0.069233902558)*x[0]**o+(-0.646481367913)*x[0]+(-0.978900933671)*x[1]**o+(0.743358367572)*x[1]
        s[0,4,1,0]=(0.192177279681)*x[0]**o+(0.443905094115)*x[0]+(-0.687749538927)*x[1]**o+(-0.0130016040578)*x[1]
        s[0,4,1,1]=(0.336586630924)*x[0]**o+(0.952526894755)*x[0]+(-0.32776980948)*x[1]**o+(-0.46574155251)*x[1]
        s[0,4,2,0]=(0.635240845083)*x[0]**o+(-0.694619696795)*x[0]+(0.184107273061)*x[1]**o+(-0.102554433435)*x[1]
        s[0,4,2,1]=(0.392781256576)*x[0]**o+(0.962026661797)*x[0]+(0.657049172645)*x[1]**o+(0.462113010746)*x[1]
        s[1,0,0,0]=(0.188456710847)*x[0]**o+(0.835668677088)*x[0]+(0.379822929854)*x[1]**o+(-0.864958528812)*x[1]
        s[1,0,0,1]=(-0.910686394304)*x[0]**o+(-0.323019307138)*x[0]+(0.339165812738)*x[1]**o+(0.651547968842)*x[1]
        s[1,0,1,0]=(0.0133563994025)*x[0]**o+(-0.931707806441)*x[0]+(0.473665399577)*x[1]**o+(-0.626120577625)*x[1]
        s[1,0,1,1]=(0.19107514064)*x[0]**o+(-0.462202695026)*x[0]+(0.46369841557)*x[1]**o+(-0.253328976575)*x[1]
        s[1,0,2,0]=(0.179414794128)*x[0]**o+(-0.809074596595)*x[0]+(0.128452495995)*x[1]**o+(-0.501454759976)*x[1]
        s[1,0,2,1]=(0.507586792052)*x[0]**o+(-0.448027963861)*x[0]+(0.244339731125)*x[1]**o+(0.506838949931)*x[1]
        s[1,1,0,0]=(0.0515340520755)*x[0]**o+(-0.554979125841)*x[0]+(0.508215404195)*x[1]**o+(-0.880450798699)*x[1]
        s[1,1,0,1]=(0.149995895729)*x[0]**o+(0.207292493123)*x[0]+(-0.0719243226652)*x[1]**o+(-0.9540219098)*x[1]
        s[1,1,1,0]=(-0.298918981626)*x[0]**o+(0.90891772564)*x[0]+(0.0457980649267)*x[1]**o+(-0.266285753328)*x[1]
        s[1,1,1,1]=(-0.833495545848)*x[0]**o+(0.902967001807)*x[0]+(0.103572135419)*x[1]**o+(-0.262026617122)*x[1]
        s[1,1,2,0]=(0.381644351387)*x[0]**o+(0.77834766803)*x[0]+(-0.785664720197)*x[1]**o+(0.678219935866)*x[1]
        s[1,1,2,1]=(0.589726138962)*x[0]**o+(-0.794897130021)*x[0]+(0.0749818396161)*x[1]**o+(0.818217309223)*x[1]
        s[1,2,0,0]=(0.648262414915)*x[0]**o+(0.737500663109)*x[0]+(0.714248701071)*x[1]**o+(-0.306480036057)*x[1]
        s[1,2,0,1]=(0.154336156671)*x[0]**o+(0.988237826231)*x[0]+(0.186782608772)*x[1]**o+(-0.29179239454)*x[1]
        s[1,2,1,0]=(0.631907675013)*x[0]**o+(-0.297341493926)*x[0]+(-0.159116807039)*x[1]**o+(-0.435773119414)*x[1]
        s[1,2,1,1]=(-0.150281595818)*x[0]**o+(0.00593213086215)*x[0]+(0.995323329778)*x[1]**o+(0.978446533527)*x[1]
        s[1,2,2,0]=(0.15122501321)*x[0]**o+(0.781060971949)*x[0]+(0.945042808379)*x[1]**o+(-0.236073537251)*x[1]
        s[1,2,2,1]=(0.490853599647)*x[0]**o+(0.70560372132)*x[0]+(0.518228280852)*x[1]**o+(0.00359879318509)*x[1]
        s[1,3,0,0]=(-0.868069831372)*x[0]**o+(-0.808015513984)*x[0]+(-0.99302787628)*x[1]**o+(0.643430993188)*x[1]
        s[1,3,0,1]=(-0.540392753009)*x[0]**o+(0.412535909714)*x[0]+(-0.691622353639)*x[1]**o+(-0.726320722169)*x[1]
        s[1,3,1,0]=(-0.38171518034)*x[0]**o+(0.574401211179)*x[0]+(0.239758390058)*x[1]**o+(0.667920000788)*x[1]
        s[1,3,1,1]=(0.405015772479)*x[0]**o+(0.871607122253)*x[0]+(0.875356104271)*x[1]**o+(0.571089711154)*x[1]
        s[1,3,2,0]=(-0.859288591023)*x[0]**o+(0.0495016165977)*x[0]+(0.652544093354)*x[1]**o+(-0.950289461543)*x[1]
        s[1,3,2,1]=(-0.385753789824)*x[0]**o+(-0.929231955794)*x[0]+(-0.266920992105)*x[1]**o+(-0.170089958141)*x[1]
        s[1,4,0,0]=(0.248320342844)*x[0]**o+(-0.193177840651)*x[0]+(-0.974008625555)*x[1]**o+(0.19289653706)*x[1]
        s[1,4,0,1]=(-0.408988831595)*x[0]**o+(-0.753769725549)*x[0]+(-0.890410564837)*x[1]**o+(-0.574271841364)*x[1]
        s[1,4,1,0]=(-0.637267396746)*x[0]**o+(-0.270785389868)*x[0]+(0.272797445173)*x[1]**o+(-0.495088539306)*x[1]
        s[1,4,1,1]=(-0.711284122459)*x[0]**o+(0.635952871792)*x[0]+(-0.0849564635014)*x[1]**o+(-0.227419456586)*x[1]
        s[1,4,2,0]=(0.194861831786)*x[0]**o+(0.52267182066)*x[0]+(0.396913284632)*x[1]**o+(0.446135337457)*x[1]
        s[1,4,2,1]=(-0.111812571843)*x[0]**o+(0.919973056019)*x[0]+(0.954568821795)*x[1]**o+(-0.410846336745)*x[1]
        s[2,0,0,0]=(0.149006416722)*x[0]**o+(-0.254413162151)*x[0]+(0.66590243332)*x[1]**o+(0.28412642191)*x[1]
        s[2,0,0,1]=(-0.788828642343)*x[0]**o+(-0.384244929105)*x[0]+(-0.115430427081)*x[1]**o+(-0.670217327178)*x[1]
        s[2,0,1,0]=(0.598185590774)*x[0]**o+(-0.593162615988)*x[0]+(-0.247117069506)*x[1]**o+(-0.545724450495)*x[1]
        s[2,0,1,1]=(-0.945230420194)*x[0]**o+(-0.158557029333)*x[0]+(-0.628357637608)*x[1]**o+(-0.516938772751)*x[1]
        s[2,0,2,0]=(0.184549704327)*x[0]**o+(-0.753954168024)*x[0]+(-0.43734992018)*x[1]**o+(-0.767428538976)*x[1]
        s[2,0,2,1]=(-0.897896267578)*x[0]**o+(0.782159121394)*x[0]+(0.926088436529)*x[1]**o+(-0.922666911396)*x[1]
        s[2,1,0,0]=(-0.0375376932325)*x[0]**o+(-0.648996277693)*x[0]+(0.163113306006)*x[1]**o+(0.326391594556)*x[1]
        s[2,1,0,1]=(-0.603507455453)*x[0]**o+(-0.227212072795)*x[0]+(0.762838138451)*x[1]**o+(0.784904636664)*x[1]
        s[2,1,1,0]=(-0.250121932289)*x[0]**o+(0.820330134347)*x[0]+(0.954063036094)*x[1]**o+(-0.0944681705347)*x[1]
        s[2,1,1,1]=(0.0147284052497)*x[0]**o+(0.214839823184)*x[0]+(-0.932724945971)*x[1]**o+(0.723405207766)*x[1]
        s[2,1,2,0]=(-0.832131948289)*x[0]**o+(-0.361716015855)*x[0]+(-0.648739038507)*x[1]**o+(-0.825135489955)*x[1]
        s[2,1,2,1]=(-0.809460903823)*x[0]**o+(0.219025460406)*x[0]+(-0.399288944386)*x[1]**o+(-0.443923619258)*x[1]
        s[2,2,0,0]=(0.851821921349)*x[0]**o+(0.864789449232)*x[0]+(-0.146472829559)*x[1]**o+(-0.13802610172)*x[1]
        s[2,2,0,1]=(0.145872642324)*x[0]**o+(-0.53737681799)*x[0]+(-0.262458059774)*x[1]**o+(-0.865100768157)*x[1]
        s[2,2,1,0]=(-0.97150697777)*x[0]**o+(0.74271566101)*x[0]+(-0.941540320886)*x[1]**o+(-0.992417829758)*x[1]
        s[2,2,1,1]=(0.705001677542)*x[0]**o+(-0.464736083815)*x[0]+(0.725341857111)*x[1]**o+(-0.350205035943)*x[1]
        s[2,2,2,0]=(-0.405826604429)*x[0]**o+(0.741314043354)*x[0]+(-0.797706422949)*x[1]**o+(0.177373453057)*x[1]
        s[2,2,2,1]=(-0.338189348522)*x[0]**o+(-0.933697184604)*x[0]+(-0.381859891282)*x[1]**o+(0.0825590545614)*x[1]
        s[2,3,0,0]=(-0.532343324478)*x[0]**o+(-0.425963751562)*x[0]+(0.306014483173)*x[1]**o+(-0.886115500167)*x[1]
        s[2,3,0,1]=(0.384975538101)*x[0]**o+(0.198217241539)*x[0]+(0.939876357973)*x[1]**o+(-0.227463489114)*x[1]
        s[2,3,1,0]=(0.39221893978)*x[0]**o+(-0.173675431821)*x[0]+(0.929829286228)*x[1]**o+(0.643170164534)*x[1]
        s[2,3,1,1]=(-0.706521549808)*x[0]**o+(-0.6114595264)*x[0]+(0.207625492571)*x[1]**o+(0.262958323309)*x[1]
        s[2,3,2,0]=(0.464449153296)*x[0]**o+(-0.438225086666)*x[0]+(-0.463914786644)*x[1]**o+(0.532074329884)*x[1]
        s[2,3,2,1]=(-0.755612237619)*x[0]**o+(0.450659729492)*x[0]+(0.265758298887)*x[1]**o+(0.0352759835818)*x[1]
        s[2,4,0,0]=(-0.913101029308)*x[0]**o+(-0.244134313945)*x[0]+(0.347084253405)*x[1]**o+(0.84625762348)*x[1]
        s[2,4,0,1]=(-0.261381262076)*x[0]**o+(-0.0817994875895)*x[0]+(0.870579357546)*x[1]**o+(-0.917318949068)*x[1]
        s[2,4,1,0]=(-0.128568967272)*x[0]**o+(0.732248131756)*x[0]+(0.414206585164)*x[1]**o+(0.659941361833)*x[1]
        s[2,4,1,1]=(0.924901584611)*x[0]**o+(0.592074499154)*x[0]+(0.681628131999)*x[1]**o+(0.167953983361)*x[1]
        s[2,4,2,0]=(0.409615949394)*x[0]**o+(-0.902690757356)*x[0]+(0.577674551804)*x[1]**o+(0.0220011629429)*x[1]
        s[2,4,2,1]=(0.382356012027)*x[0]**o+(-0.423633937718)*x[0]+(-0.592574898322)*x[1]**o+(0.183184987952)*x[1]
        s[3,0,0,0]=(-0.254267267009)*x[0]**o+(-0.665177288859)*x[0]+(-0.666690913098)*x[1]**o+(-0.433935909161)*x[1]
        s[3,0,0,1]=(-0.672935968283)*x[0]**o+(-0.862578688926)*x[0]+(-0.431014198736)*x[1]**o+(-0.950634125485)*x[1]
        s[3,0,1,0]=(0.198586757881)*x[0]**o+(0.654459371747)*x[0]+(0.768197508919)*x[1]**o+(-0.911430651255)*x[1]
        s[3,0,1,1]=(0.835596691179)*x[0]**o+(0.662532614321)*x[0]+(0.64745394627)*x[1]**o+(-0.177665076446)*x[1]
        s[3,0,2,0]=(-0.60108209247)*x[0]**o+(0.104120238589)*x[0]+(0.478363486403)*x[1]**o+(0.60998776125)*x[1]
        s[3,0,2,1]=(-0.632653025461)*x[0]**o+(0.415693475381)*x[0]+(-0.0139161976239)*x[1]**o+(-0.486360398365)*x[1]
        s[3,1,0,0]=(-0.216238961155)*x[0]**o+(0.605030211804)*x[0]+(-0.894286771395)*x[1]**o+(-0.738005399203)*x[1]
        s[3,1,0,1]=(0.453465128819)*x[0]**o+(-0.958353730943)*x[0]+(-0.191303802435)*x[1]**o+(-0.403969790319)*x[1]
        s[3,1,1,0]=(-0.371601120889)*x[0]**o+(-0.000506485761004)*x[0]+(0.128347107042)*x[1]**o+(-0.617448247517)*x[1]
        s[3,1,1,1]=(-0.492741845463)*x[0]**o+(0.114070742237)*x[0]+(0.968707420077)*x[1]**o+(0.897709784012)*x[1]
        s[3,1,2,0]=(-0.774958662409)*x[0]**o+(0.712770979439)*x[0]+(0.825889537824)*x[1]**o+(0.73040987218)*x[1]
        s[3,1,2,1]=(0.412550829295)*x[0]**o+(-0.861820540337)*x[0]+(-0.64322990825)*x[1]**o+(0.262526540471)*x[1]
        s[3,2,0,0]=(-0.543691366072)*x[0]**o+(0.685553656644)*x[0]+(0.65968803389)*x[1]**o+(0.664750334284)*x[1]
        s[3,2,0,1]=(0.428045943755)*x[0]**o+(-0.140247368953)*x[0]+(0.380175178975)*x[1]**o+(-0.467518419036)*x[1]
        s[3,2,1,0]=(-0.389824098536)*x[0]**o+(-0.276525222589)*x[0]+(0.147155465185)*x[1]**o+(0.976584972227)*x[1]
        s[3,2,1,1]=(-0.801779406727)*x[0]**o+(-0.265379451805)*x[0]+(-0.515076806975)*x[1]**o+(0.436497310193)*x[1]
        s[3,2,2,0]=(0.484267483123)*x[0]**o+(0.17147902302)*x[0]+(0.386200975176)*x[1]**o+(-0.467212146867)*x[1]
        s[3,2,2,1]=(0.508871274459)*x[0]**o+(-0.819775442961)*x[0]+(0.662298902936)*x[1]**o+(-0.680488821518)*x[1]
        s[3,3,0,0]=(-0.268312881659)*x[0]**o+(0.879490107736)*x[0]+(-0.0144262822452)*x[1]**o+(-0.71519934726)*x[1]
        s[3,3,0,1]=(-0.790430004852)*x[0]**o+(-0.532618460762)*x[0]+(0.253908480901)*x[1]**o+(-0.909634079731)*x[1]
        s[3,3,1,0]=(0.0137346713747)*x[0]**o+(-0.385222519657)*x[0]+(-0.217309505922)*x[1]**o+(-0.488231851176)*x[1]
        s[3,3,1,1]=(-0.0284395764434)*x[0]**o+(0.604128820394)*x[0]+(0.459487518143)*x[1]**o+(0.0415640328609)*x[1]
        s[3,3,2,0]=(-0.162643181454)*x[0]**o+(0.998506598628)*x[0]+(0.772502162236)*x[1]**o+(0.0667163114862)*x[1]
        s[3,3,2,1]=(0.623630922274)*x[0]**o+(0.148711722927)*x[0]+(-0.161606838558)*x[1]**o+(0.156709490526)*x[1]
        s[3,4,0,0]=(-0.30946349425)*x[0]**o+(0.0987044449972)*x[0]+(0.620328824713)*x[1]**o+(0.212795150211)*x[1]
        s[3,4,0,1]=(-0.431480440685)*x[0]**o+(-0.000853814120948)*x[0]+(0.322994919175)*x[1]**o+(-0.444398394184)*x[1]
        s[3,4,1,0]=(0.768816881062)*x[0]**o+(-0.148697199679)*x[0]+(0.676778413386)*x[1]**o+(0.58721092116)*x[1]
        s[3,4,1,1]=(0.642322282841)*x[0]**o+(-0.863985751359)*x[0]+(-0.030305847597)*x[1]**o+(0.16420798613)*x[1]
        s[3,4,2,0]=(-0.652491180967)*x[0]**o+(-0.455511531309)*x[0]+(0.739317807968)*x[1]**o+(-0.299398746891)*x[1]
        s[3,4,2,1]=(-0.934623810452)*x[0]**o+(0.0755256471511)*x[0]+(-0.573554337332)*x[1]**o+(0.940012415898)*x[1]
        ref[0,0,0,0]=(0.246692793786)/(o+1.)+(-0.121284353373)+(0.0390601545717)*0.5**o
        ref[0,0,0,1]=(-0.986136095398)/(o+1.)+(0.504511759262)+(0.608816949634)*0.5**o
        ref[0,0,1,0]=(0.725705948245)/(o+1.)+(-0.453772243536)+(0.580621174141)*0.5**o
        ref[0,0,1,1]=(-0.376543304375)/(o+1.)+(-0.274154329162)+(0.388567471102)*0.5**o
        ref[0,0,2,0]=(-0.088525361736)/(o+1.)+(-0.24716896632)+(-0.692988033922)*0.5**o
        ref[0,0,2,1]=(-0.380234357131)/(o+1.)+(0.21389123247)+(0.274277434778)*0.5**o
        ref[0,1,0,0]=(0.335411545595)/(o+1.)+(0.1029286996)+(-0.802394756916)*0.5**o
        ref[0,1,0,1]=(0.178161299328)/(o+1.)+(-0.315511211672)+(0.174271017656)*0.5**o
        ref[0,1,1,0]=(0.589565043365)/(o+1.)+(-0.127948597487)+(-0.598879030318)*0.5**o
        ref[0,1,1,1]=(-0.743341570183)/(o+1.)+(0.456387470249)+(-0.667493955405)*0.5**o
        ref[0,1,2,0]=(0.899075872977)/(o+1.)+(0.0414909142386)+(0.47631386081)*0.5**o
        ref[0,1,2,1]=(-0.541353591572)/(o+1.)+(0.0382726489972)+(-0.497833539416)*0.5**o
        ref[0,2,0,0]=(-0.697588210675)/(o+1.)+(0.183020036152)+(-0.735607995943)*0.5**o
        ref[0,2,0,1]=(-0.630771158486)/(o+1.)+(0.0928497225723)+(0.705776536011)*0.5**o
        ref[0,2,1,0]=(0.315121166158)/(o+1.)+(-0.193103375068)+(-0.975134219189)*0.5**o
        ref[0,2,1,1]=(-0.471199351714)/(o+1.)+(0.510425622306)+(-0.43751086407)*0.5**o
        ref[0,2,2,0]=(0.433608622434)/(o+1.)+(-0.16152942391)+(0.526770705207)*0.5**o
        ref[0,2,2,1]=(0.0493546850323)/(o+1.)+(-0.142851309637)+(-0.418834628035)*0.5**o
        ref[0,3,0,0]=(0.235788766456)/(o+1.)+(-0.0385269918575)+(0.302691082154)*0.5**o
        ref[0,3,0,1]=(0.653142282067)/(o+1.)+(0.0517575103062)+(-0.400507082213)*0.5**o
        ref[0,3,1,0]=(-0.0222151576861)/(o+1.)+(-0.650529960575)+(-0.542123872101)*0.5**o
        ref[0,3,1,1]=(-0.977561971438)/(o+1.)+(0.325073643158)+(0.0629454961983)*0.5**o
        ref[0,3,2,0]=(0.094320839406)/(o+1.)+(-0.189684900712)+(0.088224801816)*0.5**o
        ref[0,3,2,1]=(-0.387883052017)/(o+1.)+(-0.579963217227)+(0.976917465786)*0.5**o
        ref[0,4,0,0]=(-0.308997447406)/(o+1.)+(-0.656691345314)+(-0.468335475351)*0.5**o
        ref[0,4,0,1]=(-0.978900933671)/(o+1.)+(0.0484384998298)+(0.069233902558)*0.5**o
        ref[0,4,1,0]=(-0.687749538927)/(o+1.)+(0.215451745029)+(0.192177279681)*0.5**o
        ref[0,4,1,1]=(-0.32776980948)/(o+1.)+(0.243392671122)+(0.336586630924)*0.5**o
        ref[0,4,2,0]=(0.184107273061)/(o+1.)+(-0.398587065115)+(0.635240845083)*0.5**o
        ref[0,4,2,1]=(0.657049172645)/(o+1.)+(0.712069836271)+(0.392781256576)*0.5**o
        ref[1,0,0,0]=(0.379822929854)/(o+1.)+(-0.014644925862)+(0.188456710847)*0.5**o
        ref[1,0,0,1]=(0.339165812738)/(o+1.)+(0.164264330852)+(-0.910686394304)*0.5**o
        ref[1,0,1,0]=(0.473665399577)/(o+1.)+(-0.778914192033)+(0.0133563994025)*0.5**o
        ref[1,0,1,1]=(0.46369841557)/(o+1.)+(-0.3577658358)+(0.19107514064)*0.5**o
        ref[1,0,2,0]=(0.128452495995)/(o+1.)+(-0.655264678285)+(0.179414794128)*0.5**o
        ref[1,0,2,1]=(0.244339731125)/(o+1.)+(0.029405493035)+(0.507586792052)*0.5**o
        ref[1,1,0,0]=(0.508215404195)/(o+1.)+(-0.71771496227)+(0.0515340520755)*0.5**o
        ref[1,1,0,1]=(-0.0719243226652)/(o+1.)+(-0.373364708338)+(0.149995895729)*0.5**o
        ref[1,1,1,0]=(0.0457980649267)/(o+1.)+(0.321315986156)+(-0.298918981626)*0.5**o
        ref[1,1,1,1]=(0.103572135419)/(o+1.)+(0.320470192342)+(-0.833495545848)*0.5**o
        ref[1,1,2,0]=(-0.785664720197)/(o+1.)+(0.728283801948)+(0.381644351387)*0.5**o
        ref[1,1,2,1]=(0.0749818396161)/(o+1.)+(0.0116600896011)+(0.589726138962)*0.5**o
        ref[1,2,0,0]=(0.714248701071)/(o+1.)+(0.215510313526)+(0.648262414915)*0.5**o
        ref[1,2,0,1]=(0.186782608772)/(o+1.)+(0.348222715845)+(0.154336156671)*0.5**o
        ref[1,2,1,0]=(-0.159116807039)/(o+1.)+(-0.36655730667)+(0.631907675013)*0.5**o
        ref[1,2,1,1]=(0.995323329778)/(o+1.)+(0.492189332195)+(-0.150281595818)*0.5**o
        ref[1,2,2,0]=(0.945042808379)/(o+1.)+(0.272493717349)+(0.15122501321)*0.5**o
        ref[1,2,2,1]=(0.518228280852)/(o+1.)+(0.354601257252)+(0.490853599647)*0.5**o
        ref[1,3,0,0]=(-0.99302787628)/(o+1.)+(-0.0822922603983)+(-0.868069831372)*0.5**o
        ref[1,3,0,1]=(-0.691622353639)/(o+1.)+(-0.156892406228)+(-0.540392753009)*0.5**o
        ref[1,3,1,0]=(0.239758390058)/(o+1.)+(0.621160605983)+(-0.38171518034)*0.5**o
        ref[1,3,1,1]=(0.875356104271)/(o+1.)+(0.721348416703)+(0.405015772479)*0.5**o
        ref[1,3,2,0]=(0.652544093354)/(o+1.)+(-0.450393922473)+(-0.859288591023)*0.5**o
        ref[1,3,2,1]=(-0.266920992105)/(o+1.)+(-0.549660956967)+(-0.385753789824)*0.5**o
        ref[1,4,0,0]=(-0.974008625555)/(o+1.)+(-0.000140651795392)+(0.248320342844)*0.5**o
        ref[1,4,0,1]=(-0.890410564837)/(o+1.)+(-0.664020783456)+(-0.408988831595)*0.5**o
        ref[1,4,1,0]=(0.272797445173)/(o+1.)+(-0.382936964587)+(-0.637267396746)*0.5**o
        ref[1,4,1,1]=(-0.0849564635014)/(o+1.)+(0.204266707603)+(-0.711284122459)*0.5**o
        ref[1,4,2,0]=(0.396913284632)/(o+1.)+(0.484403579059)+(0.194861831786)*0.5**o
        ref[1,4,2,1]=(0.954568821795)/(o+1.)+(0.254563359637)+(-0.111812571843)*0.5**o
        ref[2,0,0,0]=(0.66590243332)/(o+1.)+(0.0148566298797)+(0.149006416722)*0.5**o
        ref[2,0,0,1]=(-0.115430427081)/(o+1.)+(-0.527231128141)+(-0.788828642343)*0.5**o
        ref[2,0,1,0]=(-0.247117069506)/(o+1.)+(-0.569443533242)+(0.598185590774)*0.5**o
        ref[2,0,1,1]=(-0.628357637608)/(o+1.)+(-0.337747901042)+(-0.945230420194)*0.5**o
        ref[2,0,2,0]=(-0.43734992018)/(o+1.)+(-0.7606913535)+(0.184549704327)*0.5**o
        ref[2,0,2,1]=(0.926088436529)/(o+1.)+(-0.070253895001)+(-0.897896267578)*0.5**o
        ref[2,1,0,0]=(0.163113306006)/(o+1.)+(-0.161302341569)+(-0.0375376932325)*0.5**o
        ref[2,1,0,1]=(0.762838138451)/(o+1.)+(0.278846281935)+(-0.603507455453)*0.5**o
        ref[2,1,1,0]=(0.954063036094)/(o+1.)+(0.362930981906)+(-0.250121932289)*0.5**o
        ref[2,1,1,1]=(-0.932724945971)/(o+1.)+(0.469122515475)+(0.0147284052497)*0.5**o
        ref[2,1,2,0]=(-0.648739038507)/(o+1.)+(-0.593425752905)+(-0.832131948289)*0.5**o
        ref[2,1,2,1]=(-0.399288944386)/(o+1.)+(-0.112449079426)+(-0.809460903823)*0.5**o
        ref[2,2,0,0]=(-0.146472829559)/(o+1.)+(0.363381673756)+(0.851821921349)*0.5**o
        ref[2,2,0,1]=(-0.262458059774)/(o+1.)+(-0.701238793074)+(0.145872642324)*0.5**o
        ref[2,2,1,0]=(-0.941540320886)/(o+1.)+(-0.124851084374)+(-0.97150697777)*0.5**o
        ref[2,2,1,1]=(0.725341857111)/(o+1.)+(-0.407470559879)+(0.705001677542)*0.5**o
        ref[2,2,2,0]=(-0.797706422949)/(o+1.)+(0.459343748206)+(-0.405826604429)*0.5**o
        ref[2,2,2,1]=(-0.381859891282)/(o+1.)+(-0.425569065022)+(-0.338189348522)*0.5**o
        ref[2,3,0,0]=(0.306014483173)/(o+1.)+(-0.656039625864)+(-0.532343324478)*0.5**o
        ref[2,3,0,1]=(0.939876357973)/(o+1.)+(-0.0146231237877)+(0.384975538101)*0.5**o
        ref[2,3,1,0]=(0.929829286228)/(o+1.)+(0.234747366357)+(0.39221893978)*0.5**o
        ref[2,3,1,1]=(0.207625492571)/(o+1.)+(-0.174250601546)+(-0.706521549808)*0.5**o
        ref[2,3,2,0]=(-0.463914786644)/(o+1.)+(0.046924621609)+(0.464449153296)*0.5**o
        ref[2,3,2,1]=(0.265758298887)/(o+1.)+(0.242967856537)+(-0.755612237619)*0.5**o
        ref[2,4,0,0]=(0.347084253405)/(o+1.)+(0.301061654767)+(-0.913101029308)*0.5**o
        ref[2,4,0,1]=(0.870579357546)/(o+1.)+(-0.499559218329)+(-0.261381262076)*0.5**o
        ref[2,4,1,0]=(0.414206585164)/(o+1.)+(0.696094746795)+(-0.128568967272)*0.5**o
        ref[2,4,1,1]=(0.681628131999)/(o+1.)+(0.380014241257)+(0.924901584611)*0.5**o
        ref[2,4,2,0]=(0.577674551804)/(o+1.)+(-0.440344797207)+(0.409615949394)*0.5**o
        ref[2,4,2,1]=(-0.592574898322)/(o+1.)+(-0.120224474883)+(0.382356012027)*0.5**o
        ref[3,0,0,0]=(-0.666690913098)/(o+1.)+(-0.54955659901)+(-0.254267267009)*0.5**o
        ref[3,0,0,1]=(-0.431014198736)/(o+1.)+(-0.906606407206)+(-0.672935968283)*0.5**o
        ref[3,0,1,0]=(0.768197508919)/(o+1.)+(-0.128485639754)+(0.198586757881)*0.5**o
        ref[3,0,1,1]=(0.64745394627)/(o+1.)+(0.242433768937)+(0.835596691179)*0.5**o
        ref[3,0,2,0]=(0.478363486403)/(o+1.)+(0.35705399992)+(-0.60108209247)*0.5**o
        ref[3,0,2,1]=(-0.0139161976239)/(o+1.)+(-0.0353334614921)+(-0.632653025461)*0.5**o
        ref[3,1,0,0]=(-0.894286771395)/(o+1.)+(-0.0664875936996)+(-0.216238961155)*0.5**o
        ref[3,1,0,1]=(-0.191303802435)/(o+1.)+(-0.681161760631)+(0.453465128819)*0.5**o
        ref[3,1,1,0]=(0.128347107042)/(o+1.)+(-0.308977366639)+(-0.371601120889)*0.5**o
        ref[3,1,1,1]=(0.968707420077)/(o+1.)+(0.505890263125)+(-0.492741845463)*0.5**o
        ref[3,1,2,0]=(0.825889537824)/(o+1.)+(0.72159042581)+(-0.774958662409)*0.5**o
        ref[3,1,2,1]=(-0.64322990825)/(o+1.)+(-0.299646999933)+(0.412550829295)*0.5**o
        ref[3,2,0,0]=(0.65968803389)/(o+1.)+(0.675151995464)+(-0.543691366072)*0.5**o
        ref[3,2,0,1]=(0.380175178975)/(o+1.)+(-0.303882893994)+(0.428045943755)*0.5**o
        ref[3,2,1,0]=(0.147155465185)/(o+1.)+(0.350029874819)+(-0.389824098536)*0.5**o
        ref[3,2,1,1]=(-0.515076806975)/(o+1.)+(0.0855589291935)+(-0.801779406727)*0.5**o
        ref[3,2,2,0]=(0.386200975176)/(o+1.)+(-0.147866561923)+(0.484267483123)*0.5**o
        ref[3,2,2,1]=(0.662298902936)/(o+1.)+(-0.75013213224)+(0.508871274459)*0.5**o
        ref[3,3,0,0]=(-0.0144262822452)/(o+1.)+(0.0821453802379)+(-0.268312881659)*0.5**o
        ref[3,3,0,1]=(0.253908480901)/(o+1.)+(-0.721126270246)+(-0.790430004852)*0.5**o
        ref[3,3,1,0]=(-0.217309505922)/(o+1.)+(-0.436727185417)+(0.0137346713747)*0.5**o
        ref[3,3,1,1]=(0.459487518143)/(o+1.)+(0.322846426627)+(-0.0284395764434)*0.5**o
        ref[3,3,2,0]=(0.772502162236)/(o+1.)+(0.532611455057)+(-0.162643181454)*0.5**o
        ref[3,3,2,1]=(-0.161606838558)/(o+1.)+(0.152710606726)+(0.623630922274)*0.5**o
        ref[3,4,0,0]=(0.620328824713)/(o+1.)+(0.155749797604)+(-0.30946349425)*0.5**o
        ref[3,4,0,1]=(0.322994919175)/(o+1.)+(-0.222626104153)+(-0.431480440685)*0.5**o
        ref[3,4,1,0]=(0.676778413386)/(o+1.)+(0.21925686074)+(0.768816881062)*0.5**o
        ref[3,4,1,1]=(-0.030305847597)/(o+1.)+(-0.349888882614)+(0.642322282841)*0.5**o
        ref[3,4,2,0]=(0.739317807968)/(o+1.)+(-0.3774551391)+(-0.652491180967)*0.5**o
        ref[3,4,2,1]=(-0.573554337332)/(o+1.)+(0.507769031524)+(-0.934623810452)*0.5**o
      else:
        s[0,0,0,0]=(-0.284157309093)*x[0]**o+(0.849767107542)*x[0]+(-0.384722111658)*x[1]**o+(-0.422005634353)*x[1]+(-0.650336986527)*x[2]**o+(-0.512697943529)*x[2]
        s[0,0,0,1]=(-0.315769534489)*x[0]**o+(0.535386332325)*x[0]+(0.853804045467)*x[1]**o+(-0.547748632318)*x[1]+(0.346701407152)*x[2]**o+(0.37814849359)*x[2]
        s[0,0,1,0]=(-0.4535776613)*x[0]**o+(-0.510530838778)*x[0]+(0.276827299966)*x[1]**o+(-0.382689005507)*x[1]+(0.905088915885)*x[2]**o+(0.0949553680219)*x[2]
        s[0,0,1,1]=(-0.739956684686)*x[0]**o+(0.461075107945)*x[0]+(-0.669109047898)*x[1]**o+(0.894371361493)*x[1]+(-0.67186417627)*x[2]**o+(0.646865110437)*x[2]
        s[0,0,2,0]=(0.630514031324)*x[0]**o+(-0.200697710807)*x[0]+(-0.425713236098)*x[1]**o+(0.990076474527)*x[1]+(0.799964278476)*x[2]**o+(-0.11090726662)*x[2]
        s[0,0,2,1]=(-0.763048729267)*x[0]**o+(0.108312132849)*x[0]+(-0.0743278550192)*x[1]**o+(-0.00609761227579)*x[1]+(0.856275832244)*x[2]**o+(-0.271931694818)*x[2]
        s[0,1,0,0]=(-0.438048883736)*x[0]**o+(0.0995162169923)*x[0]+(0.991935495829)*x[1]**o+(0.10168563116)*x[1]+(0.61021228041)*x[2]**o+(0.935697636412)*x[2]
        s[0,1,0,1]=(0.169577859304)*x[0]**o+(-0.32382927444)*x[0]+(-0.416050245653)*x[1]**o+(-0.218068414951)*x[1]+(-0.828154819878)*x[2]**o+(0.510133764467)*x[2]
        s[0,1,1,0]=(-0.136877583438)*x[0]**o+(0.225702373679)*x[0]+(-0.982330481939)*x[1]**o+(-0.76035657944)*x[1]+(-0.75008234946)*x[2]**o+(0.34026332188)*x[2]
        s[0,1,1,1]=(0.21329287668)*x[0]**o+(-0.583080810655)*x[0]+(0.304728874482)*x[1]**o+(-0.513735666156)*x[1]+(0.678531278973)*x[2]**o+(0.229523450267)*x[2]
        s[0,1,2,0]=(-0.488721257593)*x[0]**o+(-0.623065480166)*x[0]+(-0.689240830032)*x[1]**o+(0.0487255710714)*x[1]+(-0.345860672549)*x[2]**o+(-0.839070113015)*x[2]
        s[0,1,2,1]=(0.0407718763863)*x[0]**o+(-0.73486237198)*x[0]+(-0.430375782223)*x[1]**o+(0.937703487997)*x[1]+(0.211905262532)*x[2]**o+(0.436342043473)*x[2]
        s[0,2,0,0]=(0.587345517354)*x[0]**o+(-0.592886902091)*x[0]+(0.552036449839)*x[1]**o+(0.623893456691)*x[1]+(0.0732753243278)*x[2]**o+(0.0126745042435)*x[2]
        s[0,2,0,1]=(-0.811084805765)*x[0]**o+(0.535274295321)*x[0]+(0.24492936339)*x[1]**o+(-0.800440152287)*x[1]+(0.631212521526)*x[2]**o+(-0.447237919067)*x[2]
        s[0,2,1,0]=(-0.222984108172)*x[0]**o+(0.666955781431)*x[0]+(-0.793761886587)*x[1]**o+(0.767105994377)*x[1]+(0.106607736355)*x[2]**o+(0.107909765632)*x[2]
        s[0,2,1,1]=(-0.351616621125)*x[0]**o+(0.918653285574)*x[0]+(0.374186423556)*x[1]**o+(0.00935909197596)*x[1]+(0.40967652737)*x[2]**o+(0.85098748057)*x[2]
        s[0,2,2,0]=(0.326986640454)*x[0]**o+(-0.922098818053)*x[0]+(0.180209899234)*x[1]**o+(-0.473771920593)*x[1]+(0.850114791793)*x[2]**o+(-0.0591994178493)*x[2]
        s[0,2,2,1]=(0.416408828986)*x[0]**o+(-0.802940087419)*x[0]+(-0.15695987973)*x[1]**o+(-0.212571438106)*x[1]+(-0.695411431279)*x[2]**o+(0.652970553889)*x[2]
        s[0,3,0,0]=(0.165546039968)*x[0]**o+(0.510754381868)*x[0]+(0.922367777435)*x[1]**o+(0.946960595929)*x[1]+(0.488750467594)*x[2]**o+(0.684670926921)*x[2]
        s[0,3,0,1]=(0.806943353866)*x[0]**o+(-0.0452688268514)*x[0]+(-0.915858505648)*x[1]**o+(-0.930357118958)*x[1]+(-0.915989552667)*x[2]**o+(-0.058763902318)*x[2]
        s[0,3,1,0]=(-0.0329058672137)*x[0]**o+(-0.347664127646)*x[0]+(-0.642805650766)*x[1]**o+(-0.772181465981)*x[1]+(-0.489006714362)*x[2]**o+(-0.92809698455)*x[2]
        s[0,3,1,1]=(0.927391314677)*x[0]**o+(-0.00818607467373)*x[0]+(-0.25019363274)*x[1]**o+(-0.944005917142)*x[1]+(0.55451200174)*x[2]**o+(0.97132400122)*x[2]
        s[0,3,2,0]=(-0.395900087683)*x[0]**o+(-0.373500614568)*x[0]+(-0.292146120482)*x[1]**o+(0.481366513649)*x[1]+(0.793324319035)*x[2]**o+(-0.0829523908121)*x[2]
        s[0,3,2,1]=(0.167004592732)*x[0]**o+(-0.412448920865)*x[0]+(-0.898556558084)*x[1]**o+(-0.490869248138)*x[1]+(-0.567143223313)*x[2]**o+(-0.732576896378)*x[2]
        s[0,4,0,0]=(-0.0559473269152)*x[0]**o+(0.927731889097)*x[0]+(-0.650788921118)*x[1]**o+(0.9760762788)*x[1]+(0.115672920967)*x[2]**o+(-0.276468719613)*x[2]
        s[0,4,0,1]=(-0.126102715426)*x[0]**o+(0.948204367434)*x[0]+(-0.337377224065)*x[1]**o+(-0.66690062976)*x[1]+(-0.0486873976343)*x[2]**o+(0.731627670267)*x[2]
        s[0,4,1,0]=(-0.465158745622)*x[0]**o+(0.825435774485)*x[0]+(-0.325537513647)*x[1]**o+(-0.514033669128)*x[1]+(0.561307474551)*x[2]**o+(0.703487324789)*x[2]
        s[0,4,1,1]=(0.433494572203)*x[0]**o+(-0.193734534196)*x[0]+(-0.285490592547)*x[1]**o+(0.7933219877)*x[1]+(-0.968093339339)*x[2]**o+(0.891238901252)*x[2]
        s[0,4,2,0]=(-0.226388318689)*x[0]**o+(0.354588341815)*x[0]+(-0.592835636922)*x[1]**o+(0.164285456556)*x[1]+(0.969440431522)*x[2]**o+(0.942251489525)*x[2]
        s[0,4,2,1]=(-0.487005215247)*x[0]**o+(-0.363562477474)*x[0]+(-0.968419416695)*x[1]**o+(0.0746396954211)*x[1]+(0.0996297479628)*x[2]**o+(-0.977105194209)*x[2]
        s[1,0,0,0]=(-0.599811357812)*x[0]**o+(0.379322984555)*x[0]+(-0.263738481625)*x[1]**o+(0.737645780912)*x[1]+(0.579734520525)*x[2]**o+(0.674309085344)*x[2]
        s[1,0,0,1]=(-0.849384033331)*x[0]**o+(0.835916113742)*x[0]+(-0.95744797963)*x[1]**o+(-0.140222671452)*x[1]+(-0.885202051612)*x[2]**o+(-0.214627624347)*x[2]
        s[1,0,1,0]=(0.560384598535)*x[0]**o+(0.521038386588)*x[0]+(0.405346505288)*x[1]**o+(0.325623245692)*x[1]+(-0.499493979476)*x[2]**o+(0.457353575559)*x[2]
        s[1,0,1,1]=(0.394452679675)*x[0]**o+(-0.335426315796)*x[0]+(0.366492975013)*x[1]**o+(-0.204908992368)*x[1]+(0.72928619323)*x[2]**o+(0.868856537052)*x[2]
        s[1,0,2,0]=(-0.286588291759)*x[0]**o+(-0.457077770862)*x[0]+(0.48947586799)*x[1]**o+(-0.483134695964)*x[1]+(-0.708198861531)*x[2]**o+(-0.185141468661)*x[2]
        s[1,0,2,1]=(0.587814763664)*x[0]**o+(0.369493163541)*x[0]+(0.973921527395)*x[1]**o+(0.792281782334)*x[1]+(-0.622350476528)*x[2]**o+(-0.0910812853654)*x[2]
        s[1,1,0,0]=(-0.0104862086098)*x[0]**o+(0.636242048755)*x[0]+(-0.831278249993)*x[1]**o+(0.789497876069)*x[1]+(-0.14996840164)*x[2]**o+(0.560466658811)*x[2]
        s[1,1,0,1]=(-0.442153331955)*x[0]**o+(0.259117434312)*x[0]+(0.624678092445)*x[1]**o+(-0.95353370493)*x[1]+(-0.647048223519)*x[2]**o+(-0.264328530894)*x[2]
        s[1,1,1,0]=(0.143778546111)*x[0]**o+(-0.0162780042157)*x[0]+(0.11561710021)*x[1]**o+(-0.750325462664)*x[1]+(0.0513768047109)*x[2]**o+(0.605731207893)*x[2]
        s[1,1,1,1]=(0.830078999256)*x[0]**o+(-0.977563707962)*x[0]+(-0.0694815703262)*x[1]**o+(-0.631908693739)*x[1]+(-0.385240583095)*x[2]**o+(-0.325723419389)*x[2]
        s[1,1,2,0]=(-0.818875997493)*x[0]**o+(0.552071308369)*x[0]+(0.42469064565)*x[1]**o+(-0.939401594531)*x[1]+(-0.704918072568)*x[2]**o+(-0.412588503126)*x[2]
        s[1,1,2,1]=(0.897735820104)*x[0]**o+(-0.229171182759)*x[0]+(-0.509740516596)*x[1]**o+(-0.182079029613)*x[1]+(-0.935655203228)*x[2]**o+(0.0659300001682)*x[2]
        s[1,2,0,0]=(-0.225661260024)*x[0]**o+(-0.705274531847)*x[0]+(0.851323519708)*x[1]**o+(0.112877579527)*x[1]+(-0.622263839274)*x[2]**o+(0.0332240292837)*x[2]
        s[1,2,0,1]=(0.188146461111)*x[0]**o+(-0.401949407666)*x[0]+(0.600435405952)*x[1]**o+(-0.205690272021)*x[1]+(-0.715745492382)*x[2]**o+(-0.842790491589)*x[2]
        s[1,2,1,0]=(-0.713236970029)*x[0]**o+(-0.261492144857)*x[0]+(-0.154448102633)*x[1]**o+(-0.970544645841)*x[1]+(-0.920289620236)*x[2]**o+(-0.97207996703)*x[2]
        s[1,2,1,1]=(0.338343830055)*x[0]**o+(0.245579659207)*x[0]+(0.0867267737353)*x[1]**o+(-0.898055915558)*x[1]+(-0.0942524838856)*x[2]**o+(-0.935035704882)*x[2]
        s[1,2,2,0]=(-0.857754750211)*x[0]**o+(-0.491056643232)*x[0]+(-0.869773943254)*x[1]**o+(-0.4760062246)*x[1]+(-0.684952853861)*x[2]**o+(-0.576097444445)*x[2]
        s[1,2,2,1]=(0.798299933694)*x[0]**o+(0.0610709183894)*x[0]+(-0.486420716864)*x[1]**o+(0.0416959278242)*x[1]+(-0.928888319279)*x[2]**o+(0.854902913513)*x[2]
        s[1,3,0,0]=(-0.22936580714)*x[0]**o+(0.708863403011)*x[0]+(0.0460624260266)*x[1]**o+(-0.49429349985)*x[1]+(-0.65165177736)*x[2]**o+(-0.640993879589)*x[2]
        s[1,3,0,1]=(0.272143747738)*x[0]**o+(-0.786035898193)*x[0]+(-0.907839432563)*x[1]**o+(-0.544282319144)*x[1]+(-0.295284311966)*x[2]**o+(-0.646933800599)*x[2]
        s[1,3,1,0]=(-0.371813536068)*x[0]**o+(-0.989821491808)*x[0]+(0.87663418945)*x[1]**o+(0.49557916163)*x[1]+(0.432075714902)*x[2]**o+(0.986679642413)*x[2]
        s[1,3,1,1]=(0.64355520606)*x[0]**o+(0.849290029179)*x[0]+(-0.588615332307)*x[1]**o+(-0.43586534156)*x[1]+(0.440278053071)*x[2]**o+(0.71924945995)*x[2]
        s[1,3,2,0]=(0.296777344281)*x[0]**o+(0.369261091311)*x[0]+(-0.542374725062)*x[1]**o+(-0.378169086763)*x[1]+(-0.376253518198)*x[2]**o+(-0.0261035492256)*x[2]
        s[1,3,2,1]=(-0.631360619387)*x[0]**o+(-0.447767234553)*x[0]+(-0.907807578651)*x[1]**o+(0.0949745118593)*x[1]+(-0.467696925918)*x[2]**o+(0.328688763502)*x[2]
        s[1,4,0,0]=(-0.864162318969)*x[0]**o+(0.0492392548816)*x[0]+(-0.443732835548)*x[1]**o+(0.216532788053)*x[1]+(-0.589754138443)*x[2]**o+(0.225238430912)*x[2]
        s[1,4,0,1]=(-0.0204110610831)*x[0]**o+(-0.361198283837)*x[0]+(-0.534453313822)*x[1]**o+(0.199660564502)*x[1]+(0.76073237233)*x[2]**o+(-0.397852751275)*x[2]
        s[1,4,1,0]=(-0.0714182037356)*x[0]**o+(-0.838354030583)*x[0]+(-0.690623231638)*x[1]**o+(0.718400117263)*x[1]+(0.81989629515)*x[2]**o+(-0.99784667739)*x[2]
        s[1,4,1,1]=(-0.789159638733)*x[0]**o+(0.0550782963202)*x[0]+(0.691693836327)*x[1]**o+(-0.995863220343)*x[1]+(0.489187641363)*x[2]**o+(-0.892540698263)*x[2]
        s[1,4,2,0]=(-0.545542033256)*x[0]**o+(0.534307451322)*x[0]+(0.164514904761)*x[1]**o+(-0.604639193552)*x[1]+(0.299777195543)*x[2]**o+(0.103353434856)*x[2]
        s[1,4,2,1]=(0.655205311592)*x[0]**o+(-0.806735930884)*x[0]+(-0.808306407454)*x[1]**o+(-0.842421999068)*x[1]+(0.071496057035)*x[2]**o+(0.327827774456)*x[2]
        s[2,0,0,0]=(-0.0509015638697)*x[0]**o+(0.122893306456)*x[0]+(-0.495264442266)*x[1]**o+(-0.700582978791)*x[1]+(0.912958594653)*x[2]**o+(0.396603396384)*x[2]
        s[2,0,0,1]=(-0.834927535726)*x[0]**o+(0.0299018772298)*x[0]+(0.950983785443)*x[1]**o+(-0.467301669211)*x[1]+(0.105169729215)*x[2]**o+(0.776869887731)*x[2]
        s[2,0,1,0]=(0.695941957197)*x[0]**o+(-0.0827202988544)*x[0]+(0.551038863364)*x[1]**o+(0.768922568953)*x[1]+(0.658453649199)*x[2]**o+(0.0845854555463)*x[2]
        s[2,0,1,1]=(-0.813515499086)*x[0]**o+(0.37498311883)*x[0]+(0.249460504439)*x[1]**o+(0.57859274713)*x[1]+(0.674876282575)*x[2]**o+(0.358162640602)*x[2]
        s[2,0,2,0]=(0.264441597574)*x[0]**o+(-0.773282762615)*x[0]+(0.548708830043)*x[1]**o+(-0.365859504864)*x[1]+(-0.971213462476)*x[2]**o+(0.341839165432)*x[2]
        s[2,0,2,1]=(0.839834410476)*x[0]**o+(0.73258814656)*x[0]+(0.0894113621016)*x[1]**o+(-0.561436557466)*x[1]+(-0.87929169767)*x[2]**o+(-0.673343626233)*x[2]
        s[2,1,0,0]=(-0.9032683394)*x[0]**o+(0.548037699973)*x[0]+(-0.275249518695)*x[1]**o+(-0.182173933698)*x[1]+(-0.797774902804)*x[2]**o+(-0.541398445737)*x[2]
        s[2,1,0,1]=(-0.563127063275)*x[0]**o+(0.0246839329162)*x[0]+(0.104775018182)*x[1]**o+(0.887831444063)*x[1]+(-0.21494247853)*x[2]**o+(-0.273454823602)*x[2]
        s[2,1,1,0]=(-0.0215129450025)*x[0]**o+(-0.388850056041)*x[0]+(0.286605493305)*x[1]**o+(0.0744061944979)*x[1]+(-0.745577158797)*x[2]**o+(0.465555807335)*x[2]
        s[2,1,1,1]=(0.862938112821)*x[0]**o+(0.440536762323)*x[0]+(0.708402486014)*x[1]**o+(-0.330067981061)*x[1]+(0.212443131768)*x[2]**o+(0.604427083557)*x[2]
        s[2,1,2,0]=(-0.151555342557)*x[0]**o+(-0.422137938709)*x[0]+(0.725618931526)*x[1]**o+(0.965446678276)*x[1]+(-0.782758787615)*x[2]**o+(-0.913406963513)*x[2]
        s[2,1,2,1]=(0.485262168703)*x[0]**o+(-0.155309809717)*x[0]+(0.346371477276)*x[1]**o+(-0.25747778813)*x[1]+(0.29003119075)*x[2]**o+(-0.994256278631)*x[2]
        s[2,2,0,0]=(0.597629111376)*x[0]**o+(-0.675075309746)*x[0]+(-0.276946615105)*x[1]**o+(0.474817826479)*x[1]+(0.384217707699)*x[2]**o+(0.370453233617)*x[2]
        s[2,2,0,1]=(0.0885418745402)*x[0]**o+(0.723684895162)*x[0]+(0.380828880196)*x[1]**o+(-0.200773722805)*x[1]+(-0.72048497776)*x[2]**o+(0.627781439124)*x[2]
        s[2,2,1,0]=(-0.7058457718)*x[0]**o+(0.605583709445)*x[0]+(-0.137932623446)*x[1]**o+(-0.355521140013)*x[1]+(-0.100629345222)*x[2]**o+(-0.0196746976828)*x[2]
        s[2,2,1,1]=(0.431391331275)*x[0]**o+(0.624577796816)*x[0]+(0.517581044445)*x[1]**o+(0.810575610489)*x[1]+(0.151026961294)*x[2]**o+(-0.116575349397)*x[2]
        s[2,2,2,0]=(-0.172742409612)*x[0]**o+(-0.525478587831)*x[0]+(-0.245244804561)*x[1]**o+(0.466265204552)*x[1]+(-0.42763207872)*x[2]**o+(-0.37841245626)*x[2]
        s[2,2,2,1]=(0.193762571076)*x[0]**o+(-0.443537851353)*x[0]+(0.703848629926)*x[1]**o+(-0.694597427026)*x[1]+(0.135855093753)*x[2]**o+(-0.845940858008)*x[2]
        s[2,3,0,0]=(-0.816708755757)*x[0]**o+(-0.181019603056)*x[0]+(-0.743213747688)*x[1]**o+(0.798697343825)*x[1]+(-0.941634970147)*x[2]**o+(0.942560075608)*x[2]
        s[2,3,0,1]=(0.557205351559)*x[0]**o+(0.573309419549)*x[0]+(0.285649088042)*x[1]**o+(-0.915772596908)*x[1]+(0.894469058714)*x[2]**o+(-0.093076259014)*x[2]
        s[2,3,1,0]=(-0.330158214652)*x[0]**o+(0.519069568809)*x[0]+(-0.584129234472)*x[1]**o+(0.508779523144)*x[1]+(-0.496105354274)*x[2]**o+(-0.119894247636)*x[2]
        s[2,3,1,1]=(0.0113498976079)*x[0]**o+(-0.817148495948)*x[0]+(-0.843322049813)*x[1]**o+(0.397509634966)*x[1]+(-0.210167599559)*x[2]**o+(0.452940815863)*x[2]
        s[2,3,2,0]=(0.548708545741)*x[0]**o+(0.701316458542)*x[0]+(0.541383513552)*x[1]**o+(0.818821487001)*x[1]+(0.0337473469103)*x[2]**o+(0.144942886826)*x[2]
        s[2,3,2,1]=(0.367699563724)*x[0]**o+(-0.558358600746)*x[0]+(0.02800962366)*x[1]**o+(0.408211655912)*x[1]+(0.808098902096)*x[2]**o+(0.406029406708)*x[2]
        s[2,4,0,0]=(0.742185577055)*x[0]**o+(0.346992242703)*x[0]+(-0.789510739504)*x[1]**o+(-0.433601148291)*x[1]+(-0.317572193313)*x[2]**o+(-0.0929823026668)*x[2]
        s[2,4,0,1]=(-0.540606064885)*x[0]**o+(0.959079111439)*x[0]+(-0.477472945875)*x[1]**o+(0.327457999632)*x[1]+(0.640838941005)*x[2]**o+(0.600082339571)*x[2]
        s[2,4,1,0]=(-0.893508919799)*x[0]**o+(0.277903035098)*x[0]+(-0.10177580179)*x[1]**o+(0.404076458963)*x[1]+(0.497606493909)*x[2]**o+(0.343868899215)*x[2]
        s[2,4,1,1]=(-0.0277942588293)*x[0]**o+(-0.341120495601)*x[0]+(0.622259478993)*x[1]**o+(0.73519393456)*x[1]+(0.660012187037)*x[2]**o+(0.399368466331)*x[2]
        s[2,4,2,0]=(0.712530777919)*x[0]**o+(0.0672439307577)*x[0]+(0.124381974602)*x[1]**o+(-0.406347903024)*x[1]+(0.0208315631234)*x[2]**o+(-0.791500976149)*x[2]
        s[2,4,2,1]=(0.0858696597238)*x[0]**o+(0.629028381896)*x[0]+(0.795442036928)*x[1]**o+(-0.764623844645)*x[1]+(-0.524125153324)*x[2]**o+(0.974625430391)*x[2]
        s[3,0,0,0]=(-0.124887079943)*x[0]**o+(-0.654604024941)*x[0]+(0.822406023517)*x[1]**o+(-0.429933379615)*x[1]+(-0.693754435109)*x[2]**o+(0.745961675595)*x[2]
        s[3,0,0,1]=(0.0781604618068)*x[0]**o+(0.481187957149)*x[0]+(0.923097062213)*x[1]**o+(0.732993925667)*x[1]+(0.0504012859604)*x[2]**o+(-0.115131599222)*x[2]
        s[3,0,1,0]=(0.822494278919)*x[0]**o+(-0.595704755199)*x[0]+(-0.194581886746)*x[1]**o+(-0.943337806993)*x[1]+(0.147117678978)*x[2]**o+(0.728101168665)*x[2]
        s[3,0,1,1]=(-0.61666199289)*x[0]**o+(-0.311296394607)*x[0]+(0.0936521091279)*x[1]**o+(0.588466331236)*x[1]+(-0.878463665908)*x[2]**o+(-0.675819143473)*x[2]
        s[3,0,2,0]=(-0.275356624889)*x[0]**o+(0.383489643489)*x[0]+(0.738708963797)*x[1]**o+(0.974422154657)*x[1]+(-0.580144378752)*x[2]**o+(-0.398505112717)*x[2]
        s[3,0,2,1]=(0.17925744427)*x[0]**o+(-0.681535969307)*x[0]+(0.902860156428)*x[1]**o+(0.871645168608)*x[1]+(-0.500401958703)*x[2]**o+(-0.817977653599)*x[2]
        s[3,1,0,0]=(0.524297360691)*x[0]**o+(0.984668939309)*x[0]+(-0.929867000718)*x[1]**o+(0.270757914061)*x[1]+(-0.259503975648)*x[2]**o+(-0.780346807445)*x[2]
        s[3,1,0,1]=(-0.261808400698)*x[0]**o+(0.0194613328439)*x[0]+(-0.113405534134)*x[1]**o+(0.734179483403)*x[1]+(0.802102544514)*x[2]**o+(0.565610343674)*x[2]
        s[3,1,1,0]=(0.134876119034)*x[0]**o+(0.431559636814)*x[0]+(0.420152427242)*x[1]**o+(-0.882232674232)*x[1]+(0.267000935164)*x[2]**o+(-0.827008708436)*x[2]
        s[3,1,1,1]=(-0.987212349574)*x[0]**o+(-0.342415633625)*x[0]+(0.639912484673)*x[1]**o+(0.790670494677)*x[1]+(0.15123106124)*x[2]**o+(0.983458455307)*x[2]
        s[3,1,2,0]=(0.742204537127)*x[0]**o+(0.886895993942)*x[0]+(-0.349125661787)*x[1]**o+(-0.626136843474)*x[1]+(0.847031359985)*x[2]**o+(0.802788970865)*x[2]
        s[3,1,2,1]=(-0.235349061749)*x[0]**o+(-0.621187306924)*x[0]+(0.552218669219)*x[1]**o+(0.790729433037)*x[1]+(0.444333479788)*x[2]**o+(0.632032201563)*x[2]
        s[3,2,0,0]=(-0.938490786208)*x[0]**o+(0.869095707989)*x[0]+(0.140982256523)*x[1]**o+(-0.878601697433)*x[1]+(0.184535860313)*x[2]**o+(0.719071828514)*x[2]
        s[3,2,0,1]=(-0.0180578829991)*x[0]**o+(0.0463975318478)*x[0]+(0.00393591018992)*x[1]**o+(0.950931246975)*x[1]+(0.615063351086)*x[2]**o+(-0.258806804322)*x[2]
        s[3,2,1,0]=(0.294998689081)*x[0]**o+(-0.625359797867)*x[0]+(-0.403789197497)*x[1]**o+(-0.999420678246)*x[1]+(0.719776724665)*x[2]**o+(0.66899881431)*x[2]
        s[3,2,1,1]=(-0.722157328967)*x[0]**o+(0.299062154378)*x[0]+(0.598625708687)*x[1]**o+(-0.951467257263)*x[1]+(0.344507050433)*x[2]**o+(0.0139170273087)*x[2]
        s[3,2,2,0]=(0.621065266638)*x[0]**o+(0.985212075414)*x[0]+(-0.698645841337)*x[1]**o+(0.660676253875)*x[1]+(-0.406319768107)*x[2]**o+(-0.307469107008)*x[2]
        s[3,2,2,1]=(0.0383194307525)*x[0]**o+(0.979670738016)*x[0]+(-0.928710294194)*x[1]**o+(0.610840841234)*x[1]+(0.707435385876)*x[2]**o+(-0.243157095062)*x[2]
        s[3,3,0,0]=(0.0357627225956)*x[0]**o+(0.708815172231)*x[0]+(0.065359531343)*x[1]**o+(-0.0182577227471)*x[1]+(-0.0210976208306)*x[2]**o+(0.0761117140757)*x[2]
        s[3,3,0,1]=(0.748296379412)*x[0]**o+(-0.807953754409)*x[0]+(-0.545605437765)*x[1]**o+(0.23683862911)*x[1]+(0.173607048395)*x[2]**o+(-0.44250075226)*x[2]
        s[3,3,1,0]=(-0.726945541384)*x[0]**o+(0.426848422718)*x[0]+(-0.92314597997)*x[1]**o+(0.648637342846)*x[1]+(-0.578392786152)*x[2]**o+(0.848674260689)*x[2]
        s[3,3,1,1]=(-0.644752429929)*x[0]**o+(0.950947546555)*x[0]+(0.0149417119272)*x[1]**o+(0.716391605858)*x[1]+(0.0490453676731)*x[2]**o+(0.0576413425128)*x[2]
        s[3,3,2,0]=(0.466006820631)*x[0]**o+(0.161941404819)*x[0]+(-0.957641267925)*x[1]**o+(0.0765544212389)*x[1]+(0.701408110369)*x[2]**o+(-0.657156965052)*x[2]
        s[3,3,2,1]=(-0.867176738598)*x[0]**o+(-0.449219272229)*x[0]+(0.324298582458)*x[1]**o+(-0.998730968715)*x[1]+(0.574803005179)*x[2]**o+(-0.243281577204)*x[2]
        s[3,4,0,0]=(-0.263021668111)*x[0]**o+(0.802820216492)*x[0]+(0.372317055664)*x[1]**o+(-0.907203003358)*x[1]+(-0.0875142912227)*x[2]**o+(-0.899457536279)*x[2]
        s[3,4,0,1]=(0.223529185934)*x[0]**o+(-0.112858652745)*x[0]+(0.112751409232)*x[1]**o+(-0.383168214822)*x[1]+(0.0253282930677)*x[2]**o+(-0.0727000650191)*x[2]
        s[3,4,1,0]=(0.402273303095)*x[0]**o+(0.577572690365)*x[0]+(0.157976741538)*x[1]**o+(-0.374155092812)*x[1]+(-0.899398318425)*x[2]**o+(0.56916282818)*x[2]
        s[3,4,1,1]=(-0.565673417227)*x[0]**o+(-0.0157754170468)*x[0]+(-0.896925597573)*x[1]**o+(-0.968592388923)*x[1]+(-0.694486144109)*x[2]**o+(-0.795948130804)*x[2]
        s[3,4,2,0]=(-0.962992173758)*x[0]**o+(-0.299086071473)*x[0]+(0.367929277599)*x[1]**o+(-0.139102532943)*x[1]+(-0.838141685104)*x[2]**o+(0.181200376172)*x[2]
        s[3,4,2,1]=(0.610198649476)*x[0]**o+(-0.747032724663)*x[0]+(0.546335594194)*x[1]**o+(0.21236923042)*x[1]+(0.028021042999)*x[2]**o+(0.469015322864)*x[2]
        ref[0,0,0,0]=(-1.03505909819)/(o+1.)+(-0.0424682351702)+(-0.284157309093)*0.5**o
        ref[0,0,0,1]=(1.20050545262)/(o+1.)+(0.182893096798)+(-0.315769534489)*0.5**o
        ref[0,0,1,0]=(1.18191621585)/(o+1.)+(-0.399132238132)+(-0.4535776613)*0.5**o
        ref[0,0,1,1]=(-1.34097322417)/(o+1.)+(1.00115578994)+(-0.739956684686)*0.5**o
        ref[0,0,2,0]=(0.374251042378)/(o+1.)+(0.33923574855)+(0.630514031324)*0.5**o
        ref[0,0,2,1]=(0.781947977225)/(o+1.)+(-0.0848585871226)+(-0.763048729267)*0.5**o
        ref[0,1,0,0]=(1.60214777624)/(o+1.)+(0.568449742282)+(-0.438048883736)*0.5**o
        ref[0,1,0,1]=(-1.24420506553)/(o+1.)+(-0.0158819624619)+(0.169577859304)*0.5**o
        ref[0,1,1,0]=(-1.7324128314)/(o+1.)+(-0.0971954419402)+(-0.136877583438)*0.5**o
        ref[0,1,1,1]=(0.983260153455)/(o+1.)+(-0.433646513272)+(0.21329287668)*0.5**o
        ref[0,1,2,0]=(-1.03510150258)/(o+1.)+(-0.706705011055)+(-0.488721257593)*0.5**o
        ref[0,1,2,1]=(-0.218470519691)/(o+1.)+(0.319591579745)+(0.0407718763863)*0.5**o
        ref[0,2,0,0]=(0.625311774166)/(o+1.)+(0.0218405294219)+(0.587345517354)*0.5**o
        ref[0,2,0,1]=(0.876141884916)/(o+1.)+(-0.356201888016)+(-0.811084805765)*0.5**o
        ref[0,2,1,0]=(-0.687154150232)/(o+1.)+(0.77098577072)+(-0.222984108172)*0.5**o
        ref[0,2,1,1]=(0.783862950926)/(o+1.)+(0.88949992906)+(-0.351616621125)*0.5**o
        ref[0,2,2,0]=(1.03032469103)/(o+1.)+(-0.727535078248)+(0.326986640454)*0.5**o
        ref[0,2,2,1]=(-0.852371311009)/(o+1.)+(-0.181270485818)+(0.416408828986)*0.5**o
        ref[0,3,0,0]=(1.41111824503)/(o+1.)+(1.07119295236)+(0.165546039968)*0.5**o
        ref[0,3,0,1]=(-1.83184805832)/(o+1.)+(-0.517194924063)+(0.806943353866)*0.5**o
        ref[0,3,1,0]=(-1.13181236513)/(o+1.)+(-1.02397128909)+(-0.0329058672137)*0.5**o
        ref[0,3,1,1]=(0.304318369001)/(o+1.)+(0.00956600470204)+(0.927391314677)*0.5**o
        ref[0,3,2,0]=(0.501178198552)/(o+1.)+(0.0124567541342)+(-0.395900087683)*0.5**o
        ref[0,3,2,1]=(-1.4656997814)/(o+1.)+(-0.817947532691)+(0.167004592732)*0.5**o
        ref[0,4,0,0]=(-0.535116000151)/(o+1.)+(0.813669724142)+(-0.0559473269152)*0.5**o
        ref[0,4,0,1]=(-0.386064621699)/(o+1.)+(0.506465703971)+(-0.126102715426)*0.5**o
        ref[0,4,1,0]=(0.235769960904)/(o+1.)+(0.507444715073)+(-0.465158745622)*0.5**o
        ref[0,4,1,1]=(-1.25358393189)/(o+1.)+(0.745413177378)+(0.433494572203)*0.5**o
        ref[0,4,2,0]=(0.3766047946)/(o+1.)+(0.730562643948)+(-0.226388318689)*0.5**o
        ref[0,4,2,1]=(-0.868789668733)/(o+1.)+(-0.633013988131)+(-0.487005215247)*0.5**o
        ref[1,0,0,0]=(0.315996038899)/(o+1.)+(0.895638925406)+(-0.599811357812)*0.5**o
        ref[1,0,0,1]=(-1.84265003124)/(o+1.)+(0.240532908971)+(-0.849384033331)*0.5**o
        ref[1,0,1,0]=(-0.0941474741887)/(o+1.)+(0.65200760392)+(0.560384598535)*0.5**o
        ref[1,0,1,1]=(1.09577916824)/(o+1.)+(0.164260614444)+(0.394452679675)*0.5**o
        ref[1,0,2,0]=(-0.218722993542)/(o+1.)+(-0.562676967744)+(-0.286588291759)*0.5**o
        ref[1,0,2,1]=(0.351571050868)/(o+1.)+(0.535346830254)+(0.587814763664)*0.5**o
        ref[1,1,0,0]=(-0.981246651633)/(o+1.)+(0.993103291818)+(-0.0104862086098)*0.5**o
        ref[1,1,0,1]=(-0.0223701310738)/(o+1.)+(-0.479372400756)+(-0.442153331955)*0.5**o
        ref[1,1,1,0]=(0.166993904921)/(o+1.)+(-0.0804361294934)+(0.143778546111)*0.5**o
        ref[1,1,1,1]=(-0.454722153421)/(o+1.)+(-0.967597910545)+(0.830078999256)*0.5**o
        ref[1,1,2,0]=(-0.280227426918)/(o+1.)+(-0.399959394644)+(-0.818875997493)*0.5**o
        ref[1,1,2,1]=(-1.44539571982)/(o+1.)+(-0.172660106102)+(0.897735820104)*0.5**o
        ref[1,2,0,0]=(0.229059680434)/(o+1.)+(-0.279586461518)+(-0.225661260024)*0.5**o
        ref[1,2,0,1]=(-0.11531008643)/(o+1.)+(-0.725215085638)+(0.188146461111)*0.5**o
        ref[1,2,1,0]=(-1.07473772287)/(o+1.)+(-1.10205837886)+(-0.713236970029)*0.5**o
        ref[1,2,1,1]=(-0.00752571015035)/(o+1.)+(-0.793755980616)+(0.338343830055)*0.5**o
        ref[1,2,2,0]=(-1.55472679711)/(o+1.)+(-0.771580156139)+(-0.857754750211)*0.5**o
        ref[1,2,2,1]=(-1.41530903614)/(o+1.)+(0.478834879863)+(0.798299933694)*0.5**o
        ref[1,3,0,0]=(-0.605589351334)/(o+1.)+(-0.213211988214)+(-0.22936580714)*0.5**o
        ref[1,3,0,1]=(-1.20312374453)/(o+1.)+(-0.988626008968)+(0.272143747738)*0.5**o
        ref[1,3,1,0]=(1.30870990435)/(o+1.)+(0.246218656117)+(-0.371813536068)*0.5**o
        ref[1,3,1,1]=(-0.148337279237)/(o+1.)+(0.566337073785)+(0.64355520606)*0.5**o
        ref[1,3,2,0]=(-0.91862824326)/(o+1.)+(-0.0175057723384)+(0.296777344281)*0.5**o
        ref[1,3,2,1]=(-1.37550450457)/(o+1.)+(-0.0120519795955)+(-0.631360619387)*0.5**o
        ref[1,4,0,0]=(-1.03348697399)/(o+1.)+(0.245505236923)+(-0.864162318969)*0.5**o
        ref[1,4,0,1]=(0.226279058508)/(o+1.)+(-0.279695235305)+(-0.0204110610831)*0.5**o
        ref[1,4,1,0]=(0.129273063512)/(o+1.)+(-0.558900295355)+(-0.0714182037356)*0.5**o
        ref[1,4,1,1]=(1.18088147769)/(o+1.)+(-0.916662811143)+(-0.789159638733)*0.5**o
        ref[1,4,2,0]=(0.464292100303)/(o+1.)+(0.0165108463129)+(-0.545542033256)*0.5**o
        ref[1,4,2,1]=(-0.736810350419)/(o+1.)+(-0.660665077748)+(0.655205311592)*0.5**o
        ref[2,0,0,0]=(0.417694152388)/(o+1.)+(-0.0905431379759)+(-0.0509015638697)*0.5**o
        ref[2,0,0,1]=(1.05615351466)/(o+1.)+(0.169735047875)+(-0.834927535726)*0.5**o
        ref[2,0,1,0]=(1.20949251256)/(o+1.)+(0.385393862823)+(0.695941957197)*0.5**o
        ref[2,0,1,1]=(0.924336787014)/(o+1.)+(0.655869253281)+(-0.813515499086)*0.5**o
        ref[2,0,2,0]=(-0.422504632433)/(o+1.)+(-0.398651551024)+(0.264441597574)*0.5**o
        ref[2,0,2,1]=(-0.789880335569)/(o+1.)+(-0.25109601857)+(0.839834410476)*0.5**o
        ref[2,1,0,0]=(-1.0730244215)/(o+1.)+(-0.0877673397313)+(-0.9032683394)*0.5**o
        ref[2,1,0,1]=(-0.110167460347)/(o+1.)+(0.319530276689)+(-0.563127063275)*0.5**o
        ref[2,1,1,0]=(-0.458971665492)/(o+1.)+(0.0755559728958)+(-0.0215129450025)*0.5**o
        ref[2,1,1,1]=(0.920845617782)/(o+1.)+(0.357447932409)+(0.862938112821)*0.5**o
        ref[2,1,2,0]=(-0.0571398560888)/(o+1.)+(-0.185049111973)+(-0.151555342557)*0.5**o
        ref[2,1,2,1]=(0.636402668026)/(o+1.)+(-0.703521938239)+(0.485262168703)*0.5**o
        ref[2,2,0,0]=(0.107271092594)/(o+1.)+(0.0850978751753)+(0.597629111376)*0.5**o
        ref[2,2,0,1]=(-0.339656097563)/(o+1.)+(0.57534630574)+(0.0885418745402)*0.5**o
        ref[2,2,1,0]=(-0.238561968668)/(o+1.)+(0.115193935875)+(-0.7058457718)*0.5**o
        ref[2,2,1,1]=(0.668608005739)/(o+1.)+(0.659289028954)+(0.431391331275)*0.5**o
        ref[2,2,2,0]=(-0.672876883281)/(o+1.)+(-0.21881291977)+(-0.172742409612)*0.5**o
        ref[2,2,2,1]=(0.83970372368)/(o+1.)+(-0.992038068194)+(0.193762571076)*0.5**o
        ref[2,3,0,0]=(-1.68484871783)/(o+1.)+(0.780118908189)+(-0.816708755757)*0.5**o
        ref[2,3,0,1]=(1.18011814676)/(o+1.)+(-0.217769718186)+(0.557205351559)*0.5**o
        ref[2,3,1,0]=(-1.08023458875)/(o+1.)+(0.453977422158)+(-0.330158214652)*0.5**o
        ref[2,3,1,1]=(-1.05348964937)/(o+1.)+(0.0166509774407)+(0.0113498976079)*0.5**o
        ref[2,3,2,0]=(0.575130860462)/(o+1.)+(0.832540416184)+(0.548708545741)*0.5**o
        ref[2,3,2,1]=(0.836108525756)/(o+1.)+(0.127941230937)+(0.367699563724)*0.5**o
        ref[2,4,0,0]=(-1.10708293282)/(o+1.)+(-0.0897956041276)+(0.742185577055)*0.5**o
        ref[2,4,0,1]=(0.16336599513)/(o+1.)+(0.943309725321)+(-0.540606064885)*0.5**o
        ref[2,4,1,0]=(0.395830692119)/(o+1.)+(0.512924196638)+(-0.893508919799)*0.5**o
        ref[2,4,1,1]=(1.28227166603)/(o+1.)+(0.396720952645)+(-0.0277942588293)*0.5**o
        ref[2,4,2,0]=(0.145213537726)/(o+1.)+(-0.565302474208)+(0.712530777919)*0.5**o
        ref[2,4,2,1]=(0.271316883603)/(o+1.)+(0.419514983821)+(0.0858696597238)*0.5**o
        ref[3,0,0,0]=(0.128651588408)/(o+1.)+(-0.16928786448)+(-0.124887079943)*0.5**o
        ref[3,0,0,1]=(0.973498348173)/(o+1.)+(0.549525141797)+(0.0781604618068)*0.5**o
        ref[3,0,1,0]=(-0.0474642077688)/(o+1.)+(-0.405470696764)+(0.822494278919)*0.5**o
        ref[3,0,1,1]=(-0.78481155678)/(o+1.)+(-0.199324603422)+(-0.61666199289)*0.5**o
        ref[3,0,2,0]=(0.158564585045)/(o+1.)+(0.479703342715)+(-0.275356624889)*0.5**o
        ref[3,0,2,1]=(0.402458197725)/(o+1.)+(-0.313934227149)+(0.17925744427)*0.5**o
        ref[3,1,0,0]=(-1.18937097637)/(o+1.)+(0.237540022962)+(0.524297360691)*0.5**o
        ref[3,1,0,1]=(0.68869701038)/(o+1.)+(0.65962557996)+(-0.261808400698)*0.5**o
        ref[3,1,1,0]=(0.687153362406)/(o+1.)+(-0.638840872927)+(0.134876119034)*0.5**o
        ref[3,1,1,1]=(0.791143545912)/(o+1.)+(0.71585665818)+(-0.987212349574)*0.5**o
        ref[3,1,2,0]=(0.497905698197)/(o+1.)+(0.531774060666)+(0.742204537127)*0.5**o
        ref[3,1,2,1]=(0.996552149008)/(o+1.)+(0.400787163838)+(-0.235349061749)*0.5**o
        ref[3,2,0,0]=(0.325518116836)/(o+1.)+(0.354782919535)+(-0.938490786208)*0.5**o
        ref[3,2,0,1]=(0.618999261276)/(o+1.)+(0.369260987251)+(-0.0180578829991)*0.5**o
        ref[3,2,1,0]=(0.315987527168)/(o+1.)+(-0.477890830901)+(0.294998689081)*0.5**o
        ref[3,2,1,1]=(0.94313275912)/(o+1.)+(-0.319244037788)+(-0.722157328967)*0.5**o
        ref[3,2,2,0]=(-1.10496560944)/(o+1.)+(0.669209611141)+(0.621065266638)*0.5**o
        ref[3,2,2,1]=(-0.221274908319)/(o+1.)+(0.673677242094)+(0.0383194307525)*0.5**o
        ref[3,3,0,0]=(0.0442619105124)/(o+1.)+(0.38333458178)+(0.0357627225956)*0.5**o
        ref[3,3,0,1]=(-0.371998389371)/(o+1.)+(-0.50680793878)+(0.748296379412)*0.5**o
        ref[3,3,1,0]=(-1.50153876612)/(o+1.)+(0.962080013126)+(-0.726945541384)*0.5**o
        ref[3,3,1,1]=(0.0639870796003)/(o+1.)+(0.862490247463)+(-0.644752429929)*0.5**o
        ref[3,3,2,0]=(-0.256233157556)/(o+1.)+(-0.209330569497)+(0.466006820631)*0.5**o
        ref[3,3,2,1]=(0.899101587637)/(o+1.)+(-0.845615909074)+(-0.867176738598)*0.5**o
        ref[3,4,0,0]=(0.284802764441)/(o+1.)+(-0.501920161572)+(-0.263021668111)*0.5**o
        ref[3,4,0,1]=(0.1380797023)/(o+1.)+(-0.284363466293)+(0.223529185934)*0.5**o
        ref[3,4,1,0]=(-0.741421576887)/(o+1.)+(0.386290212867)+(0.402273303095)*0.5**o
        ref[3,4,1,1]=(-1.59141174168)/(o+1.)+(-0.890157968387)+(-0.565673417227)*0.5**o
        ref[3,4,2,0]=(-0.470212407505)/(o+1.)+(-0.128494114122)+(-0.962992173758)*0.5**o
        ref[3,4,2,1]=(0.574356637193)/(o+1.)+(-0.0328240856895)+(0.610198649476)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactZero_fromSymbol_ReducedSolution_rank0(self):
      """
      tests integral of rank 0 Symbol on the ReducedFunctionOnContactZero

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Symbol(shape=())
      s=Data(0,(),w)
      if dim==2:
        s=(1.70255303494)*x[0]+(0.300011314949)*x[1]
        ref=(0.345983004398)/(o+1.)+(0.363938720886)+(0.928703903719)*0.5**o
      else:
        s=(-1.68260770684)*x[0]+(-1.14729600003)*x[1]+(-1.35659279947)*x[2]
        ref=(-1.3902352024)/(o+1.)+(-1.03788235052)+(-0.720496602901)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactZero_fromSymbol_ReducedSolution_rank1(self):
      """
      tests integral of rank 1 Symbol on the ReducedFunctionOnContactZero

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(2,))
      s=Data(0,(2,),w)
      ref=numpy.zeros((2,),numpy.float_)
      if dim==2:
        s[0]=(0.23958799559)*x[0]+(-0.0458757682877)*x[1]
        s[1]=(0.0897882061379)*x[0]+(-1.54974909413)*x[1]
        ref[0]=(0.0485704769529)/(o+1.)+(0.210351971249)+(-0.275562192148)*0.5**o
        ref[1]=(-0.821668980438)/(o+1.)+(-0.712207676061)+(0.786123444573)*0.5**o
      else:
        s[0]=(-1.44817804789)*x[0]+(0.670605136525)*x[1]+(-0.593548775192)*x[2]
        s[1]=(-0.585702105721)*x[0]+(0.0665105334471)*x[1]+(-0.228099551666)*x[2]
        ref[0]=(0.859021549917)/(o+1.)+(-0.759857813727)+(-0.710427609023)*0.5**o
        ref[1]=(-1.40806718776)/(o+1.)+(0.267420623854)+(0.125934816108)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactZero_fromSymbol_ReducedSolution_rank2(self):
      """
      tests integral of rank 2 Symbol on the ReducedFunctionOnContactZero

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5))
      s=Data(0,(4, 5),w)
      ref=numpy.zeros((4, 5),numpy.float_)
      if dim==2:
        s[0,0]=(0.933177344575)*x[0]+(-0.643793974434)*x[1]
        s[0,1]=(-0.311019666872)*x[0]+(0.686195407652)*x[1]
        s[0,2]=(0.470203454822)*x[0]+(1.32374103685)*x[1]
        s[0,3]=(-0.653103411901)*x[0]+(0.477452857192)*x[1]
        s[0,4]=(1.12606394408)*x[0]+(0.0898490519564)*x[1]
        s[1,0]=(1.26644130664)*x[0]+(0.226895598587)*x[1]
        s[1,1]=(0.0271345466814)*x[0]+(0.435024033603)*x[1]
        s[1,2]=(-0.227055843926)*x[0]+(-0.144686134637)*x[1]
        s[1,3]=(0.466198287572)*x[0]+(-0.130830709376)*x[1]
        s[1,4]=(1.04066080781)*x[0]+(0.546282129071)*x[1]
        s[2,0]=(0.317469515041)*x[0]+(1.26366516514)*x[1]
        s[2,1]=(-1.20210768286)*x[0]+(-0.0533328989847)*x[1]
        s[2,2]=(0.278459393186)*x[0]+(-0.379156129627)*x[1]
        s[2,3]=(1.03459096792)*x[0]+(-0.610026027371)*x[1]
        s[2,4]=(1.288131278)*x[0]+(-0.309929698872)*x[1]
        s[3,0]=(0.32856946183)*x[0]+(0.225709965354)*x[1]
        s[3,1]=(-1.03183224237)*x[0]+(-0.0153171719792)*x[1]
        s[3,2]=(0.259516430572)*x[0]+(-0.776082702284)*x[1]
        s[3,3]=(-0.417978040907)*x[0]+(-0.0941785049007)*x[1]
        s[3,4]=(1.07527114074)*x[0]+(-0.0198401110946)*x[1]
        ref[0,0]=(-0.79959835955)/(o+1.)+(0.566758804494)+(-0.0445358792971)*0.5**o
        ref[0,1]=(0.583455543567)/(o+1.)+(0.0458448886173)+(-0.299969580021)*0.5**o
        ref[0,2]=(0.471614825642)/(o+1.)+(0.891760595232)+(-0.461191524434)*0.5**o
        ref[0,3]=(0.672045220068)/(o+1.)+(-0.56343583876)+(0.279175902744)*0.5**o
        ref[0,4]=(-0.127993678518)/(o+1.)+(0.52668612755)+(0.290534419455)*0.5**o
        ref[1,0]=(0.822774751438)/(o+1.)+(-0.011855097694)+(0.694272349175)*0.5**o
        ref[1,1]=(0.552626324952)/(o+1.)+(0.277929562319)+(-0.646326869305)*0.5**o
        ref[1,2]=(-0.565996176797)/(o+1.)+(-0.128057099164)+(0.450368396562)*0.5**o
        ref[1,3]=(0.627837338323)/(o+1.)+(-0.555359040353)+(0.818248320579)*0.5**o
        ref[1,4]=(0.722984603229)/(o+1.)+(0.0957452220354)+(0.672467889583)*0.5**o
        ref[2,0]=(0.664524759547)/(o+1.)+(0.678277833656)+(-0.439945746675)*0.5**o
        ref[2,1]=(0.908213473231)/(o+1.)+(-0.805606306623)+(-0.552441441832)*0.5**o
        ref[2,2]=(0.0908414991316)/(o+1.)+(0.131818510391)+(-0.455175256354)*0.5**o
        ref[2,3]=(-0.119095663088)/(o+1.)+(-0.223144373358)+(0.989949350348)*0.5**o
        ref[2,4]=(-0.0540728359185)/(o+1.)+(0.0748680956845)+(0.882538223681)*0.5**o
        ref[3,0]=(0.98273847633)/(o+1.)+(-0.421621406424)+(0.414783763702)*0.5**o
        ref[3,1]=(0.29285575515)/(o+1.)+(-0.462798732226)+(-0.41440770505)*0.5**o
        ref[3,2]=(-0.509767634157)/(o+1.)+(-0.480617720091)+(0.954436802627)*0.5**o
        ref[3,3]=(-0.762302422898)/(o+1.)+(0.100280539595)+(0.0495847978999)*0.5**o
        ref[3,4]=(-0.154196647936)/(o+1.)+(0.329602137701)+(0.550423402184)*0.5**o
      else:
        s[0,0]=(1.34601675616)*x[0]+(-0.451295005459)*x[1]+(-0.49622174234)*x[2]
        s[0,1]=(-0.327908015969)*x[0]+(-1.21019079469)*x[1]+(1.09622527656)*x[2]
        s[0,2]=(-0.0243507161566)*x[0]+(-0.016230100758)*x[1]+(-1.37311484633)*x[2]
        s[0,3]=(-0.445665346289)*x[0]+(-0.626824621259)*x[1]+(-0.196033254199)*x[2]
        s[0,4]=(-0.819703680211)*x[0]+(1.30953832902)*x[1]+(0.0778023021203)*x[2]
        s[1,0]=(0.219519741099)*x[0]+(-1.47335622754)*x[1]+(0.198662145648)*x[2]
        s[1,1]=(-0.65456993306)*x[0]+(-0.125758153194)*x[1]+(-1.53966098547)*x[2]
        s[1,2]=(-0.112157349953)*x[0]+(-0.459351490445)*x[1]+(1.6889234353)*x[2]
        s[1,3]=(0.102642361518)*x[0]+(1.10412193101)*x[1]+(-0.239843060928)*x[2]
        s[1,4]=(-0.232367456037)*x[0]+(-1.17836662288)*x[1]+(-0.814085860866)*x[2]
        s[2,0]=(-1.00510966212)*x[0]+(1.23960710585)*x[1]+(0.0848958213029)*x[2]
        s[2,1]=(-1.93901641139)*x[0]+(-0.0469703867512)*x[1]+(-0.21201338191)*x[2]
        s[2,2]=(1.42666776379)*x[0]+(1.04000263985)*x[1]+(0.151609334515)*x[2]
        s[2,3]=(-0.297049329338)*x[0]+(-0.103284668913)*x[1]+(-0.110261476034)*x[2]
        s[2,4]=(0.964711394219)*x[0]+(1.28218755108)*x[1]+(-1.2382220868)*x[2]
        s[3,0]=(-0.922061452692)*x[0]+(-1.36593539333)*x[1]+(1.11436583585)*x[2]
        s[3,1]=(0.284590223973)*x[0]+(1.08442017719)*x[1]+(1.5619949376)*x[2]
        s[3,2]=(-0.245207105356)*x[0]+(-1.77865635864)*x[1]+(-0.908570594143)*x[2]
        s[3,3]=(0.426334196419)*x[0]+(-0.501103031142)*x[1]+(0.199762115243)*x[2]
        s[3,4]=(1.33158896071)*x[0]+(0.44737802367)*x[1]+(-0.76095091762)*x[2]
        ref[0,0]=(-0.581088472202)/(o+1.)+(0.287163129514)+(0.405262221535)*0.5**o
        ref[0,1]=(0.695209950176)/(o+1.)+(-0.748195486472)+(0.359307488667)*0.5**o
        ref[0,2]=(0.313650504565)/(o+1.)+(-1.20553227172)+(0.683718375628)*0.5**o
        ref[0,3]=(-0.872265925416)/(o+1.)+(-0.216679466249)+(0.0371016361661)*0.5**o
        ref[0,4]=(1.07552646704)/(o+1.)+(-0.293195846094)+(0.0785021760824)*0.5**o
        ref[1,0]=(-1.22710752208)/(o+1.)+(0.409726431872)+(-0.647519682458)*0.5**o
        ref[1,1]=(-0.940490307845)/(o+1.)+(-0.82083756271)+(0.26217636154)*0.5**o
        ref[1,2]=(1.00054234642)/(o+1.)+(-0.0533842117578)+(0.223640672001)*0.5**o
        ref[1,3]=(0.0595140200914)/(o+1.)+(0.407850368171)+(0.0917064751695)*0.5**o
        ref[1,4]=(-0.50735295069)/(o+1.)+(-1.10564117596)+(0.493815362818)*0.5**o
        ref[2,0]=(0.786394742154)/(o+1.)+(-0.212261329287)+(-0.0424788185539)*0.5**o
        ref[2,1]=(-0.868240537836)/(o+1.)+(-0.186797194849)+(-0.956165252514)*0.5**o
        ref[2,2]=(-0.67113838962)/(o+1.)+(1.26208372573)+(0.765250676321)*0.5**o
        ref[2,3]=(-1.2390712788)/(o+1.)+(0.286327258603)+(0.15582128731)*0.5**o
        ref[2,4]=(-0.00831435188402)/(o+1.)+(0.305363694299)+(0.406263821787)*0.5**o
        ref[3,0]=(-0.123205370838)/(o+1.)+(-0.410740153717)+(-0.228945331902)*0.5**o
        ref[3,1]=(1.39199709306)/(o+1.)+(0.646045427731)+(0.246917390241)*0.5**o
        ref[3,2]=(-1.10740261108)/(o+1.)+(-0.860231147304)+(-0.104569152453)*0.5**o
        ref[3,3]=(-0.506214554892)/(o+1.)+(0.53544681941)+(-0.439685803409)*0.5**o
        ref[3,4]=(0.846470301583)/(o+1.)+(-0.134756583683)+(0.441058932543)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactZero_fromSymbol_ReducedSolution_rank3(self):
      """
      tests integral of rank 3 Symbol on the ReducedFunctionOnContactZero

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(6, 2, 2))
      s=Data(0,(6, 2, 2),w)
      ref=numpy.zeros((6, 2, 2),numpy.float_)
      if dim==2:
        s[0,0,0]=(-0.907789262757)*x[0]+(1.43778352913)*x[1]
        s[0,0,1]=(-0.441541130755)*x[0]+(1.75922038312)*x[1]
        s[0,1,0]=(-0.387854312613)*x[0]+(1.22523701409)*x[1]
        s[0,1,1]=(-1.15521894576)*x[0]+(0.0289558035739)*x[1]
        s[1,0,0]=(-0.640400813271)*x[0]+(-1.69766965873)*x[1]
        s[1,0,1]=(-0.641296842318)*x[0]+(-1.35895483901)*x[1]
        s[1,1,0]=(-0.14735518066)*x[0]+(0.520543405774)*x[1]
        s[1,1,1]=(0.369918186173)*x[0]+(0.209781392053)*x[1]
        s[2,0,0]=(0.677808706359)*x[0]+(0.481042907356)*x[1]
        s[2,0,1]=(-0.661702781276)*x[0]+(1.86193178646)*x[1]
        s[2,1,0]=(0.323230036331)*x[0]+(-1.45705288353)*x[1]
        s[2,1,1]=(1.8644362516)*x[0]+(0.764691070666)*x[1]
        s[3,0,0]=(-1.33103240086)*x[0]+(1.29035953404)*x[1]
        s[3,0,1]=(-0.161074703654)*x[0]+(-0.127322745584)*x[1]
        s[3,1,0]=(-0.359913251027)*x[0]+(-0.0952192793759)*x[1]
        s[3,1,1]=(-0.34519536765)*x[0]+(0.483950473232)*x[1]
        s[4,0,0]=(0.981563058768)*x[0]+(-0.0744642367992)*x[1]
        s[4,0,1]=(-0.45912279344)*x[0]+(0.0011253607226)*x[1]
        s[4,1,0]=(-0.618322333127)*x[0]+(0.199638289627)*x[1]
        s[4,1,1]=(-0.371110449749)*x[0]+(-0.382992800899)*x[1]
        s[5,0,0]=(0.840829543072)*x[0]+(-0.925833368141)*x[1]
        s[5,0,1]=(-0.0348263534937)*x[0]+(0.267324292682)*x[1]
        s[5,1,0]=(0.0785260612832)*x[0]+(1.59479548515)*x[1]
        s[5,1,1]=(-0.203914067868)*x[0]+(1.28202597585)*x[1]
        ref[0,0,0]=(0.898417888273)/(o+1.)+(0.171058001117)+(-0.710539624134)*0.5**o
        ref[0,0,1]=(0.949672663046)/(o+1.)+(0.186488421776)+(-0.00497025423347)*0.5**o
        ref[0,1,0]=(0.434571276181)/(o+1.)+(0.106095788846)+(0.190619847602)*0.5**o
        ref[0,1,1]=(0.337711977059)/(o+1.)+(-0.313335995076)+(-0.837303129097)*0.5**o
        ref[1,0,0]=(-0.757199193222)/(o+1.)+(-0.36300260101)+(-0.854866076761)*0.5**o
        ref[1,0,1]=(-0.767144980208)/(o+1.)+(-0.215422826519)+(-0.802261048082)*0.5**o
        ref[1,1,0]=(-0.13190981045)/(o+1.)+(-0.0320289199048)+(0.569155875375)*0.5**o
        ref[1,1,1]=(0.0466405054813)/(o+1.)+(0.565229924314)+(-0.597400775882)*0.5**o
        ref[2,0,0]=(0.760951384359)/(o+1.)+(0.0600961970572)+(0.277707835242)*0.5**o
        ref[2,0,1]=(0.869894715615)/(o+1.)+(0.631323963868)+(-0.932313638168)*0.5**o
        ref[2,1,0]=(-0.666302256751)/(o+1.)+(-0.510864963778)+(0.554209337103)*0.5**o
        ref[2,1,1]=(0.934304180676)/(o+1.)+(0.350887235523)+(0.993048670542)*0.5**o
        ref[3,0,0]=(0.930749923608)/(o+1.)+(-0.283299908306)+(-0.404822973819)*0.5**o
        ref[3,0,1]=(-0.572270969539)/(o+1.)+(0.583432503768)+(-0.882991487235)*0.5**o
        ref[3,1,0]=(0.642892258995)/(o+1.)+(-0.0826293437351)+(-0.932766101927)*0.5**o
        ref[3,1,1]=(0.679591980627)/(o+1.)+(-0.379930907942)+(0.219024940838)*0.5**o
        ref[4,0,0]=(-0.0326170324069)/(o+1.)+(0.0903259186859)+(0.759064017004)*0.5**o
        ref[4,0,1]=(-0.129142887308)/(o+1.)+(0.279560607285)+(-0.887975759979)*0.5**o
        ref[4,1,0]=(0.752463858901)/(o+1.)+(-0.256945595979)+(-0.657256710443)*0.5**o
        ref[4,1,1]=(0.565307319705)/(o+1.)+(-0.898218881636)+(0.477027192919)*0.5**o
        ref[5,0,0]=(-0.28969980947)/(o+1.)+(0.0341656280661)+(0.136364728269)*0.5**o
        ref[5,0,1]=(-0.362525553087)/(o+1.)+(0.17876968499)+(0.237484122295)*0.5**o
        ref[5,1,0]=(0.856225350097)/(o+1.)+(0.375747136218)+(0.0656019238951)*0.5**o
        ref[5,1,1]=(0.358764765052)/(o+1.)+(0.783470816704)+(-0.847594490479)*0.5**o
      else:
        s[0,0,0]=(-0.18318998999)*x[0]+(-1.71372306128)*x[1]+(1.13788709107)*x[2]
        s[0,0,1]=(0.95308908228)*x[0]+(0.123844162735)*x[1]+(0.0868410957691)*x[2]
        s[0,1,0]=(-0.712762731169)*x[0]+(-1.86018372873)*x[1]+(-0.0783828227128)*x[2]
        s[0,1,1]=(-1.72172214895)*x[0]+(-1.25477573826)*x[1]+(1.28096692374)*x[2]
        s[1,0,0]=(0.86250397119)*x[0]+(1.44314824122)*x[1]+(0.763303582012)*x[2]
        s[1,0,1]=(1.05520701337)*x[0]+(1.04926637639)*x[1]+(-0.378977987156)*x[2]
        s[1,1,0]=(0.753550528625)*x[0]+(-1.63882951308)*x[1]+(0.259583964201)*x[2]
        s[1,1,1]=(0.778941098246)*x[0]+(-1.10564597288)*x[1]+(-1.42906071049)*x[2]
        s[2,0,0]=(-0.842438687373)*x[0]+(-0.202024764935)*x[1]+(0.289144880342)*x[2]
        s[2,0,1]=(-1.02319531994)*x[0]+(0.155875285165)*x[1]+(0.310920357117)*x[2]
        s[2,1,0]=(-1.45156693584)*x[0]+(0.12231441645)*x[1]+(0.722232222435)*x[2]
        s[2,1,1]=(-1.08553719113)*x[0]+(0.316521434206)*x[1]+(0.720555059931)*x[2]
        s[3,0,0]=(1.91846595467)*x[0]+(0.199356393766)*x[1]+(-0.438144879601)*x[2]
        s[3,0,1]=(-0.341123312011)*x[0]+(0.325154462361)*x[1]+(0.691743856063)*x[2]
        s[3,1,0]=(1.72547153856)*x[0]+(-0.822030080649)*x[1]+(-1.15169896539)*x[2]
        s[3,1,1]=(0.243194545088)*x[0]+(-0.718504738836)*x[1]+(0.560536413366)*x[2]
        s[4,0,0]=(0.248455459192)*x[0]+(0.415661600687)*x[1]+(0.21712561395)*x[2]
        s[4,0,1]=(-1.23171610721)*x[0]+(-0.443121697053)*x[1]+(0.724481514081)*x[2]
        s[4,1,0]=(-0.0327531013679)*x[0]+(0.385175130963)*x[1]+(-0.0484020348058)*x[2]
        s[4,1,1]=(-0.199960439132)*x[0]+(-0.751568103399)*x[1]+(-1.20119343449)*x[2]
        s[5,0,0]=(0.853262644583)*x[0]+(1.47653986554)*x[1]+(-0.0642737845052)*x[2]
        s[5,0,1]=(1.1703484139)*x[0]+(-0.848977140408)*x[1]+(-0.303628307573)*x[2]
        s[5,1,0]=(-0.719950626922)*x[0]+(0.192452589234)*x[1]+(0.315963976897)*x[2]
        s[5,1,1]=(0.331604591856)*x[0]+(0.428822102766)*x[1]+(-1.35258251093)*x[2]
        ref[0,0,0]=(-0.452131788567)/(o+1.)+(-0.313943872628)+(0.320993573616)*0.5**o
        ref[0,0,1]=(-1.4121302884)/(o+1.)+(0.934332411096)+(0.707239806995)*0.5**o
        ref[0,1,0]=(-0.371701930519)/(o+1.)+(-1.22914563984)+(0.178663927586)*0.5**o
        ref[0,1,1]=(-0.569427715149)/(o+1.)+(-0.102129307368)+(-0.921844633589)*0.5**o
        ref[1,0,0]=(1.07041067382)/(o+1.)+(1.04773121415)+(-0.0969173076836)*0.5**o
        ref[1,0,1]=(0.344908055221)/(o+1.)+(0.253322966116)+(0.873941415143)*0.5**o
        ref[1,1,0]=(-0.568409920368)/(o+1.)+(-0.376074272994)+(0.694863446104)*0.5**o
        ref[1,1,1]=(-1.26296359103)/(o+1.)+(-0.285972966041)+(0.0791439379881)*0.5**o
        ref[2,0,0]=(0.261580662311)/(o+1.)+(-0.0693472624675)+(-0.878204709343)*0.5**o
        ref[2,0,1]=(-0.0659237231253)/(o+1.)+(-0.101871484642)+(-0.286732985248)*0.5**o
        ref[2,1,0]=(-0.609626668188)/(o+1.)+(0.438552975912)+(-0.874499580587)*0.5**o
        ref[2,1,1]=(1.21427389191)/(o+1.)+(-0.396304653588)+(-0.470125281731)*0.5**o
        ref[3,0,0]=(0.466268456837)/(o+1.)+(0.109894182607)+(0.993620646782)*0.5**o
        ref[3,0,1]=(0.798648502977)/(o+1.)+(0.30293631931)+(-0.728746135183)*0.5**o
        ref[3,1,0]=(-0.443525593788)/(o+1.)+(-0.32762855249)+(0.85052519129)*0.5**o
        ref[3,1,1]=(0.520520221786)/(o+1.)+(-0.21037503556)+(-0.0145439310481)*0.5**o
        ref[4,0,0]=(1.51637897594)/(o+1.)+(0.0251054216628)+(-0.685347145435)*0.5**o
        ref[4,0,1]=(-0.0126278181421)/(o+1.)+(-0.255524438913)+(-0.426679594213)*0.5**o
        ref[4,1,0]=(-0.110252848596)/(o+1.)+(-0.270402535847)+(0.95507791508)*0.5**o
        ref[4,1,1]=(-0.965558516877)/(o+1.)+(-0.777022669388)+(0.366881878633)*0.5**o
        ref[5,0,0]=(1.47489510124)/(o+1.)+(0.344964232978)+(0.100705158424)*0.5**o
        ref[5,0,1]=(0.272180101244)/(o+1.)+(-0.219694112974)+(0.184951090624)*0.5**o
        ref[5,1,0]=(0.868944944511)/(o+1.)+(-0.492651110402)+(-0.0951767844975)*0.5**o
        ref[5,1,1]=(-0.112001801469)/(o+1.)+(-0.0274812221923)+(-0.425191570457)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactZero_fromSymbol_ReducedSolution_rank4(self):
      """
      tests integral of rank 4 Symbol on the ReducedFunctionOnContactZero

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5, 3, 2))
      s=Data(0,(4, 5, 3, 2),w)
      ref=numpy.zeros((4, 5, 3, 2),numpy.float_)
      if dim==2:
        s[0,0,0,0]=(0.492266509376)*x[0]+(-0.374307474367)*x[1]
        s[0,0,0,1]=(0.379610803929)*x[0]+(1.26877290052)*x[1]
        s[0,0,1,0]=(0.489836401727)*x[0]+(0.0568437409848)*x[1]
        s[0,0,1,1]=(-0.0962394811856)*x[0]+(1.08297193296)*x[1]
        s[0,0,2,0]=(0.0083854747638)*x[0]+(1.12783902851)*x[1]
        s[0,0,2,1]=(-1.60969353709)*x[0]+(-1.15342435141)*x[1]
        s[0,1,0,0]=(0.718333685586)*x[0]+(-0.580409068311)*x[1]
        s[0,1,0,1]=(-0.471412983112)*x[0]+(-1.21523638921)*x[1]
        s[0,1,1,0]=(0.436952252413)*x[0]+(0.817968269393)*x[1]
        s[0,1,1,1]=(0.904303325207)*x[0]+(1.27944662005)*x[1]
        s[0,1,2,0]=(-0.646670575219)*x[0]+(-0.268471138891)*x[1]
        s[0,1,2,1]=(0.216944526796)*x[0]+(-1.44656748298)*x[1]
        s[0,2,0,0]=(-1.45710792844)*x[0]+(-1.37549168202)*x[1]
        s[0,2,0,1]=(-0.0209124548614)*x[0]+(-0.405359828227)*x[1]
        s[0,2,1,0]=(0.713696154808)*x[0]+(-0.283663569311)*x[1]
        s[0,2,1,1]=(0.0879024421072)*x[0]+(-1.74270037297)*x[1]
        s[0,2,2,0]=(-0.263019221744)*x[0]+(0.587637432145)*x[1]
        s[0,2,2,1]=(0.291949019915)*x[0]+(1.21942896685)*x[1]
        s[0,3,0,0]=(0.711503988663)*x[0]+(-0.0889987684621)*x[1]
        s[0,3,0,1]=(-0.473200478311)*x[0]+(-0.731689338365)*x[1]
        s[0,3,1,0]=(0.0347877428229)*x[0]+(0.436342776392)*x[1]
        s[0,3,1,1]=(1.33181675729)*x[0]+(0.796826790935)*x[1]
        s[0,3,2,0]=(0.975860401214)*x[0]+(-0.0388876329765)*x[1]
        s[0,3,2,1]=(0.336224987742)*x[0]+(1.40851963434)*x[1]
        s[0,4,0,0]=(-0.430668122172)*x[0]+(-0.210883679811)*x[1]
        s[0,4,0,1]=(-0.0947644917837)*x[0]+(-0.145045406149)*x[1]
        s[0,4,1,0]=(-0.975155718684)*x[0]+(0.370091078964)*x[1]
        s[0,4,1,1]=(1.59897077782)*x[0]+(-0.716757934974)*x[1]
        s[0,4,2,0]=(0.0995595315716)*x[0]+(-0.654667529963)*x[1]
        s[0,4,2,1]=(0.621429995817)*x[0]+(-0.830020781954)*x[1]
        s[1,0,0,0]=(0.460799425849)*x[0]+(1.37039134597)*x[1]
        s[1,0,0,1]=(0.484677718571)*x[0]+(-1.06817334128)*x[1]
        s[1,0,1,0]=(0.157323780788)*x[0]+(-0.816294918051)*x[1]
        s[1,0,1,1]=(0.275640307711)*x[0]+(-1.13585749962)*x[1]
        s[1,0,2,0]=(1.16452950894)*x[0]+(-1.17355094567)*x[1]
        s[1,0,2,1]=(-0.567786612292)*x[0]+(-0.78722481996)*x[1]
        s[1,1,0,0]=(-0.106381006132)*x[0]+(1.35734811758)*x[1]
        s[1,1,0,1]=(0.870362310422)*x[0]+(-0.701894980494)*x[1]
        s[1,1,1,0]=(0.463101808026)*x[0]+(-0.106531628533)*x[1]
        s[1,1,1,1]=(0.34927073998)*x[0]+(0.597282241285)*x[1]
        s[1,1,2,0]=(-1.08705596894)*x[0]+(-0.476312920631)*x[1]
        s[1,1,2,1]=(0.461228158094)*x[0]+(0.553430989166)*x[1]
        s[1,2,0,0]=(-0.565889081597)*x[0]+(0.847674515062)*x[1]
        s[1,2,0,1]=(1.22774040742)*x[0]+(0.532076088807)*x[1]
        s[1,2,1,0]=(-1.35738184718)*x[0]+(-0.010480885754)*x[1]
        s[1,2,1,1]=(0.0609383835609)*x[0]+(0.539183578894)*x[1]
        s[1,2,2,0]=(0.765013210057)*x[0]+(-0.0658319206215)*x[1]
        s[1,2,2,1]=(0.720241379555)*x[0]+(-1.29084845886)*x[1]
        s[1,3,0,0]=(1.37641602996)*x[0]+(-0.713391122816)*x[1]
        s[1,3,0,1]=(0.475350779553)*x[0]+(-0.0146303427199)*x[1]
        s[1,3,1,0]=(-0.498436122956)*x[0]+(0.446982991213)*x[1]
        s[1,3,1,1]=(-0.0198417666776)*x[0]+(-0.0330880581816)*x[1]
        s[1,3,2,0]=(-0.968203407981)*x[0]+(-1.43766997027)*x[1]
        s[1,3,2,1]=(-1.516108898)*x[0]+(0.154669696369)*x[1]
        s[1,4,0,0]=(0.107239334472)*x[0]+(-1.01814801515)*x[1]
        s[1,4,0,1]=(0.490733729058)*x[0]+(1.0617427681)*x[1]
        s[1,4,1,0]=(-0.579990484487)*x[0]+(-0.681146890855)*x[1]
        s[1,4,1,1]=(-0.539291982427)*x[0]+(1.40646560401)*x[1]
        s[1,4,2,0]=(-0.804686819612)*x[0]+(0.617047118706)*x[1]
        s[1,4,2,1]=(-0.913106387219)*x[0]+(-0.35453310971)*x[1]
        s[2,0,0,0]=(1.07945504868)*x[0]+(0.147695653661)*x[1]
        s[2,0,0,1]=(0.0321229519775)*x[0]+(-0.80726917422)*x[1]
        s[2,0,1,0]=(-0.276983245168)*x[0]+(-1.8208105745)*x[1]
        s[2,0,1,1]=(-0.390466636115)*x[0]+(-0.753787382303)*x[1]
        s[2,0,2,0]=(-0.0551239488574)*x[0]+(-0.691793238598)*x[1]
        s[2,0,2,1]=(0.455414076199)*x[0]+(1.19885245354)*x[1]
        s[2,1,0,0]=(-0.774122061304)*x[0]+(1.10934639101)*x[1]
        s[2,1,0,1]=(-1.60749547114)*x[0]+(0.305715741213)*x[1]
        s[2,1,1,0]=(0.0697234437299)*x[0]+(0.542820812428)*x[1]
        s[2,1,1,1]=(1.29397510328)*x[0]+(-0.804845682022)*x[1]
        s[2,1,2,0]=(0.478973101459)*x[0]+(1.07219480882)*x[1]
        s[2,1,2,1]=(-0.897309279985)*x[0]+(1.75363512874)*x[1]
        s[2,2,0,0]=(1.61737724977)*x[0]+(0.161191822716)*x[1]
        s[2,2,0,1]=(0.808798403137)*x[0]+(-0.00926345040032)*x[1]
        s[2,2,1,0]=(-0.903485178097)*x[0]+(-0.36599523945)*x[1]
        s[2,2,1,1]=(0.30686972857)*x[0]+(-1.62803935845)*x[1]
        s[2,2,2,0]=(1.48915760228)*x[0]+(0.579057819566)*x[1]
        s[2,2,2,1]=(-0.540786588929)*x[0]+(-0.402843340661)*x[1]
        s[2,3,0,0]=(0.964227135928)*x[0]+(0.0210110310059)*x[1]
        s[2,3,0,1]=(0.0266958261678)*x[0]+(0.830215139026)*x[1]
        s[2,3,1,0]=(0.930518946289)*x[0]+(0.66745827854)*x[1]
        s[2,3,1,1]=(0.0363390243751)*x[0]+(0.591657283448)*x[1]
        s[2,3,2,0]=(-0.433934955699)*x[0]+(-1.57946754208)*x[1]
        s[2,3,2,1]=(-0.890298505624)*x[0]+(0.623825002267)*x[1]
        s[2,4,0,0]=(0.838929180901)*x[0]+(0.699121739125)*x[1]
        s[2,4,0,1]=(1.3886192544)*x[0]+(-0.232981076843)*x[1]
        s[2,4,1,0]=(-0.566792750082)*x[0]+(-0.631000271451)*x[1]
        s[2,4,1,1]=(-0.989529672577)*x[0]+(0.0943919436553)*x[1]
        s[2,4,2,0]=(-1.85858526639)*x[0]+(0.257423736684)*x[1]
        s[2,4,2,1]=(-1.01848069496)*x[0]+(-1.08574972329)*x[1]
        s[3,0,0,0]=(0.135022656073)*x[0]+(-0.641428277939)*x[1]
        s[3,0,0,1]=(-0.613029646479)*x[0]+(0.158334902417)*x[1]
        s[3,0,1,0]=(-1.01838576339)*x[0]+(-0.788404867889)*x[1]
        s[3,0,1,1]=(0.133395871444)*x[0]+(-0.540018893528)*x[1]
        s[3,0,2,0]=(0.359599806799)*x[0]+(-0.163549980956)*x[1]
        s[3,0,2,1]=(0.537279834479)*x[0]+(1.43219657076)*x[1]
        s[3,1,0,0]=(0.699221501837)*x[0]+(0.58098090756)*x[1]
        s[3,1,0,1]=(-0.264653821282)*x[0]+(-0.297670973468)*x[1]
        s[3,1,1,0]=(-1.14805006965)*x[0]+(0.173713527619)*x[1]
        s[3,1,1,1]=(0.582167066862)*x[0]+(0.224704535757)*x[1]
        s[3,1,2,0]=(-0.013329661508)*x[0]+(0.439567816478)*x[1]
        s[3,1,2,1]=(-0.206657167829)*x[0]+(-0.900945403197)*x[1]
        s[3,2,0,0]=(-1.34123093943)*x[0]+(-0.905024065154)*x[1]
        s[3,2,0,1]=(-0.0677797983484)*x[0]+(0.863260908229)*x[1]
        s[3,2,1,0]=(0.0651406495709)*x[0]+(1.35122911589)*x[1]
        s[3,2,1,1]=(-0.00598531484168)*x[0]+(0.690487120132)*x[1]
        s[3,2,2,0]=(0.748701424535)*x[0]+(-1.0273152862)*x[1]
        s[3,2,2,1]=(-0.239064296585)*x[0]+(-0.213292951531)*x[1]
        s[3,3,0,0]=(-0.620272397992)*x[0]+(1.00106031112)*x[1]
        s[3,3,0,1]=(-0.803133841712)*x[0]+(0.873569080382)*x[1]
        s[3,3,1,0]=(-1.187617412)*x[0]+(0.422664418077)*x[1]
        s[3,3,1,1]=(-1.51637824714)*x[0]+(-1.5177161306)*x[1]
        s[3,3,2,0]=(1.41348390435)*x[0]+(1.2134182272)*x[1]
        s[3,3,2,1]=(0.99094246846)*x[0]+(0.34796551642)*x[1]
        s[3,4,0,0]=(0.580352675436)*x[0]+(-1.21782952097)*x[1]
        s[3,4,0,1]=(0.060030794007)*x[0]+(0.711473074059)*x[1]
        s[3,4,1,0]=(0.966745511848)*x[0]+(-0.550401614505)*x[1]
        s[3,4,1,1]=(-0.791329990237)*x[0]+(-0.205150506961)*x[1]
        s[3,4,2,0]=(0.482041855799)*x[0]+(-1.09392636319)*x[1]
        s[3,4,2,1]=(-0.698074703295)*x[0]+(0.510339675206)*x[1]
        ref[0,0,0,0]=(-0.953900694954)/(o+1.)+(0.442044844728)+(0.187770040507)*0.5**o
        ref[0,0,0,1]=(0.580869130103)/(o+1.)+(0.516362992147)+(0.03478859005)*0.5**o
        ref[0,0,1,0]=(-0.411645490601)/(o+1.)+(0.406943479905)+(0.144438673502)*0.5**o
        ref[0,0,1,1]=(0.204312991686)/(o+1.)+(0.383971959467)+(0.0144755411578)*0.5**o
        ref[0,0,2,0]=(0.743369450952)/(o+1.)+(0.0895404122051)+(0.213774227914)*0.5**o
        ref[0,0,2,1]=(-0.239858711457)/(o+1.)+(-0.809763839108)+(-0.903731498821)*0.5**o
        ref[0,1,0,0]=(-0.484413922256)/(o+1.)+(0.0508933433825)+(0.520551852766)*0.5**o
        ref[0,1,0,1]=(-0.590736935429)/(o+1.)+(-0.344843181585)+(-0.406226073725)*0.5**o
        ref[0,1,1,0]=(-0.125338697971)/(o+1.)+(0.369801286198)+(0.640656647382)*0.5**o
        ref[0,1,1,1]=(0.552504077477)/(o+1.)+(0.788979766736)+(0.0532863343083)*0.5**o
        ref[0,1,2,0]=(0.306194481363)/(o+1.)+(-0.445453307231)+(-0.33042958101)*0.5**o
        ref[0,1,2,1]=(-0.818898129415)/(o+1.)+(-0.0332482175532)+(-0.34422839166)*0.5**o
        ref[0,2,0,0]=(-0.878218012751)/(o+1.)+(-0.612244128212)+(-0.729893341289)*0.5**o
        ref[0,2,0,1]=(-0.105155854997)/(o+1.)+(0.281922249648)+(-0.884960927387)*0.5**o
        ref[0,2,1,0]=(0.565149481722)/(o+1.)+(0.0625165001878)+(-0.2601498966)*0.5**o
        ref[0,2,1,1]=(-0.983424704841)/(o+1.)+(0.0730690344226)+(-0.817511294863)*0.5**o
        ref[0,2,2,0]=(-0.258286255412)/(o+1.)+(0.308348049886)+(-0.0337916339596)*0.5**o
        ref[0,2,2,1]=(0.915004878119)/(o+1.)+(-0.166545843752)+(0.929464796149)*0.5**o
        ref[0,3,0,0]=(-0.433529018481)/(o+1.)+(0.627958762155)+(-0.199883285628)*0.5**o
        ref[0,3,0,1]=(-0.945419963966)/(o+1.)+(-0.336318858392)+(0.413167864075)*0.5**o
        ref[0,3,1,0]=(0.502119767083)/(o+1.)+(0.334993773063)+(-0.700976793994)*0.5**o
        ref[0,3,1,1]=(-0.186129206617)/(o+1.)+(0.891324041438)+(0.532124671966)*0.5**o
        ref[0,3,2,0]=(0.504051134603)/(o+1.)+(-0.200874672613)+(0.834670978862)*0.5**o
        ref[0,3,2,1]=(0.436498536789)/(o+1.)+(0.545640525497)+(0.216965034296)*0.5**o
        ref[0,4,0,0]=(-0.756349034292)/(o+1.)+(0.140738322652)+(-0.166679412995)*0.5**o
        ref[0,4,0,1]=(0.780437446749)/(o+1.)+(-0.599105577138)+(0.177963809594)*0.5**o
        ref[0,4,1,0]=(0.943566244729)/(o+1.)+(-0.441716458606)+(-0.665197967238)*0.5**o
        ref[0,4,1,1]=(-0.734101358227)/(o+1.)+(0.356355202125)+(0.903603796827)*0.5**o
        ref[0,4,2,0]=(-0.43430093723)/(o+1.)+(0.383496742595)+(-0.887800546351)*0.5**o
        ref[0,4,2,1]=(-0.227167732111)/(o+1.)+(-0.274598730532)+(0.567774407038)*0.5**o
        ref[1,0,0,0]=(0.391393429129)/(o+1.)+(0.353444577314)+(0.732908188056)*0.5**o
        ref[1,0,0,1]=(-0.171565730455)/(o+1.)+(-0.259205209862)+(0.106480527466)*0.5**o
        ref[1,0,1,0]=(-0.774601901321)/(o+1.)+(0.259483708314)+(-0.403336652569)*0.5**o
        ref[1,0,1,1]=(-0.726406904963)/(o+1.)+(-0.129278181762)+(0.124746076574)*0.5**o
        ref[1,0,2,0]=(-0.617577515377)/(o+1.)+(-0.152390726783)+(0.91333753221)*0.5**o
        ref[1,0,2,1]=(0.138202581628)/(o+1.)+(-0.396089249037)+(-0.701035515806)*0.5**o
        ref[1,1,0,0]=(0.729934336346)/(o+1.)+(-0.00268233232676)+(0.52639743976)*0.5**o
        ref[1,1,0,1]=(-0.158306695958)/(o+1.)+(-0.0162211686892)+(0.359216363265)*0.5**o
        ref[1,1,1,0]=(-0.978599129855)/(o+1.)+(0.71609215355)+(-0.0970149977525)*0.5**o
        ref[1,1,1,1]=(-0.349041967673)/(o+1.)+(0.780124912357)+(-0.264654875775)*0.5**o
        ref[1,1,2,0]=(-0.809194404709)/(o+1.)+(-0.268116096378)+(-0.217942292102)*0.5**o
        ref[1,1,2,1]=(0.455578593899)/(o+1.)+(0.428430852179)+(-0.297781150997)*0.5**o
        ref[1,2,0,0]=(0.116383337262)/(o+1.)+(0.228068268367)+(-0.290734440531)*0.5**o
        ref[1,2,0,1]=(0.977031820054)/(o+1.)+(0.137033678728)+(0.508717318715)*0.5**o
        ref[1,2,1,0]=(0.282937416713)/(o+1.)+(-0.615158071804)+(-0.420484006037)*0.5**o
        ref[1,2,1,1]=(0.904627062081)/(o+1.)+(0.018901292084)+(-0.342307683794)*0.5**o
        ref[1,2,2,0]=(0.755780696034)/(o+1.)+(-0.0362605587438)+(0.0159217108891)*0.5**o
        ref[1,2,2,1]=(-0.699866517547)/(o+1.)+(0.171062073116)+(-0.212864707989)*0.5**o
        ref[1,3,0,0]=(0.228867465112)/(o+1.)+(-0.0322172708178)+(0.498591983668)*0.5**o
        ref[1,3,0,1]=(-0.289542634768)/(o+1.)+(0.110436622855)+(0.529389825891)*0.5**o
        ref[1,3,1,0]=(0.358054041082)/(o+1.)+(0.165615637362)+(-0.74073844755)*0.5**o
        ref[1,3,1,1]=(-0.60359826352)/(o+1.)+(0.517588005735)+(-0.48450757281)*0.5**o
        ref[1,3,2,0]=(-0.545672858809)/(o+1.)+(-0.618462721543)+(-0.623275076358)*0.5**o
        ref[1,3,2,1]=(-0.73428128984)/(o+1.)+(-0.0497647291554)+(-0.527628453484)*0.5**o
        ref[1,4,0,0]=(-0.04306414004)/(o+1.)+(-0.298207200434)+(-0.271430139773)*0.5**o
        ref[1,4,0,1]=(0.971916388567)/(o+1.)+(-0.0691551012311)+(0.718870311054)*0.5**o
        ref[1,4,1,0]=(-0.445510592605)/(o+1.)+(-0.088724728164)+(-0.638177326409)*0.5**o
        ref[1,4,1,1]=(0.753412648385)/(o+1.)+(-0.0777097810015)+(0.269180535197)*0.5**o
        ref[1,4,2,0]=(-0.305234413894)/(o+1.)+(0.169600008291)+(-0.221605303593)*0.5**o
        ref[1,4,2,1]=(-0.286993085073)/(o+1.)+(-0.0493395569951)+(-0.881967297866)*0.5**o
        ref[2,0,0,0]=(-0.719751624491)/(o+1.)+(0.677226475278)+(0.59244937628)*0.5**o
        ref[2,0,0,1]=(-0.670829954915)/(o+1.)+(-0.139434936863)+(0.174553606398)*0.5**o
        ref[2,0,1,0]=(-0.916962595573)/(o+1.)+(-0.372659474123)+(-0.435512275853)*0.5**o
        ref[2,0,1,1]=(-0.893849803211)/(o+1.)+(-0.341833589876)+(0.433262964544)*0.5**o
        ref[2,0,2,0]=(-0.773257190107)/(o+1.)+(0.0495078796887)+(-0.0726757567263)*0.5**o
        ref[2,0,2,1]=(0.632070005323)/(o+1.)+(0.697198313452)+(-0.372200102483)*0.5**o
        ref[2,1,0,0]=(0.459329022417)/(o+1.)+(0.249194755087)+(-0.622494202881)*0.5**o
        ref[2,1,0,1]=(0.0269909119644)/(o+1.)+(-0.210708025265)+(-0.907354591365)*0.5**o
        ref[2,1,1,0]=(0.183761641828)/(o+1.)+(-0.207261190919)+(0.843304996169)*0.5**o
        ref[2,1,1,1]=(-0.49062931094)/(o+1.)+(0.2742786733)+(0.431201385598)*0.5**o
        ref[2,1,2,0]=(0.798332846753)/(o+1.)+(0.187074177535)+(0.378686708458)*0.5**o
        ref[2,1,2,1]=(0.942487410042)/(o+1.)+(0.0268513037692)+(-0.139864168826)*0.5**o
        ref[2,2,0,0]=(-0.306480663139)/(o+1.)+(0.706708669675)+(0.671632396277)*0.5**o
        ref[2,2,0,1]=(-0.68426521806)/(o+1.)+(0.658766824582)+(0.166266521632)*0.5**o
        ref[2,2,1,0]=(-0.0872551950734)/(o+1.)+(-0.212578991849)+(-0.757067238775)*0.5**o
        ref[2,2,1,1]=(-0.779515023363)/(o+1.)+(-0.165651803902)+(-0.210350998718)*0.5**o
        ref[2,2,2,0]=(0.420662034868)/(o+1.)+(0.456572874952)+(0.734407637073)*0.5**o
        ref[2,2,2,1]=(-0.0347308970891)/(o+1.)+(-0.550927478137)+(0.192955923773)*0.5**o
        ref[2,3,0,0]=(0.279479543192)/(o+1.)+(0.340981994922)+(0.0237946338994)*0.5**o
        ref[2,3,0,1]=(0.885111721906)/(o+1.)+(0.0386593998513)+(-0.105519556415)*0.5**o
        ref[2,3,1,0]=(-0.142745975965)/(o+1.)+(0.571228905184)+(0.598265390426)*0.5**o
        ref[2,3,1,1]=(0.676813945202)/(o+1.)+(0.301993597369)+(-0.652804832118)*0.5**o
        ref[2,3,2,0]=(-0.834072218834)/(o+1.)+(-0.836092513405)+(0.492854747868)*0.5**o
        ref[2,3,2,1]=(0.451195972296)/(o+1.)+(-0.330911551268)+(-0.0558463731179)*0.5**o
        ref[2,4,0,0]=(0.604480974758)/(o+1.)+(-0.0272834841321)+(0.988136913533)*0.5**o
        ref[2,4,0,1]=(-0.672293493189)/(o+1.)+(0.436472725256)+(0.954986220232)*0.5**o
        ref[2,4,1,0]=(-0.945983571254)/(o+1.)+(0.362615161801)+(-0.97703977388)*0.5**o
        ref[2,4,1,1]=(-0.778028939318)/(o+1.)+(0.423610287746)+(-0.964329365094)*0.5**o
        ref[2,4,2,0]=(-0.138477487464)/(o+1.)+(-0.290097108281)+(-0.882489825678)*0.5**o
        ref[2,4,2,1]=(-0.953853492601)/(o+1.)+(-0.179508678068)+(-0.791359569512)*0.5**o
        ref[3,0,0,0]=(-0.199022712971)/(o+1.)+(0.253562515321)+(-0.814507939536)*0.5**o
        ref[3,0,0,1]=(0.112777444671)/(o+1.)+(-0.304438182199)+(0.0414041756638)*0.5**o
        ref[3,0,1,0]=(-0.880947344135)/(o+1.)+(-0.194192553178)+(-0.537458180783)*0.5**o
        ref[3,0,1,1]=(0.39314163058)/(o+1.)+(-0.775542968704)+(0.751321284745)*0.5**o
        ref[3,0,2,0]=(-0.482557545787)/(o+1.)+(-0.0683141681573)+(0.815235707945)*0.5**o
        ref[3,0,2,1]=(0.689498951848)/(o+1.)+(0.278187448835)+(0.723602555722)*0.5**o
        ref[3,1,0,0]=(0.293238350118)/(o+1.)+(0.126767755198)+(0.733428548881)*0.5**o
        ref[3,1,0,1]=(-0.26577191789)/(o+1.)+(0.201156903364)+(-0.698866683588)*0.5**o
        ref[3,1,1,0]=(0.439591690299)/(o+1.)+(-0.288902429619)+(-0.836123373092)*0.5**o
        ref[3,1,1,1]=(0.8910765985)/(o+1.)+(0.0553222730519)+(-0.194849541985)*0.5**o
        ref[3,1,2,0]=(-0.0320019026134)/(o+1.)+(0.477772657371)+(-0.497305257159)*0.5**o
        ref[3,1,2,1]=(-0.0499399267727)/(o+1.)+(-0.47037968496)+(-0.116903274335)*0.5**o
        ref[3,2,0,0]=(-0.276422491301)/(o+1.)+(-0.812540687574)+(-0.344751138134)*0.5**o
        ref[3,2,0,1]=(-0.00141392846802)/(o+1.)+(0.102797761295)+(0.591299515759)*0.5**o
        ref[3,2,1,0]=(0.630823139203)/(o+1.)+(0.743132641412)+(-0.700718656569)*0.5**o
        ref[3,2,1,1]=(0.490401876791)/(o+1.)+(-0.0461798489229)+(0.286459626346)*0.5**o
        ref[3,2,2,0]=(-0.787385728473)/(o+1.)+(0.0658852261331)+(0.377001414541)*0.5**o
        ref[3,2,2,1]=(-0.158857143023)/(o+1.)+(-0.421186525234)+(0.548872945376)*0.5**o
        ref[3,3,0,0]=(0.029229133788)/(o+1.)+(0.069575027013)+(0.212408725319)*0.5**o
        ref[3,3,0,1]=(0.757333397205)/(o+1.)+(0.029368913907)+(-0.74563598635)*0.5**o
        ref[3,3,1,0]=(0.0943476349516)/(o+1.)+(0.0337288993927)+(-0.926758427661)*0.5**o
        ref[3,3,1,1]=(-0.769051069188)/(o+1.)+(-0.844682211294)+(-0.575678885962)*0.5**o
        ref[3,3,2,0]=(0.246931939289)/(o+1.)+(0.756380254609)+(0.867209683046)*0.5**o
        ref[3,3,2,1]=(0.535429280541)/(o+1.)+(-0.0329131735419)+(0.869305051424)*0.5**o
        ref[3,4,0,0]=(-0.405156907998)/(o+1.)+(-0.244492906569)+(0.256665875603)*0.5**o
        ref[3,4,0,1]=(-0.140892979474)/(o+1.)+(0.516304250401)+(-0.120211653262)*0.5**o
        ref[3,4,1,0]=(0.168500977091)/(o+1.)+(0.089683045067)+(0.0684768301186)*0.5**o
        ref[3,4,1,1]=(0.479212374255)/(o+1.)+(-0.626588026593)+(-0.222516818267)*0.5**o
        ref[3,4,2,0]=(-0.563625996633)/(o+1.)+(-0.120136908079)+(0.192015305399)*0.5**o
        ref[3,4,2,1]=(-0.031361929046)/(o+1.)+(0.332029567053)+(-0.820432233148)*0.5**o
      else:
        s[0,0,0,0]=(0.186907875567)*x[0]+(0.323825050263)*x[1]+(1.15648314531)*x[2]
        s[0,0,0,1]=(1.12367405158)*x[0]+(1.13427439957)*x[1]+(0.189106618663)*x[2]
        s[0,0,1,0]=(1.76772912309)*x[0]+(0.0616670242122)*x[1]+(-1.23257311969)*x[2]
        s[0,0,1,1]=(0.0497251821243)*x[0]+(-1.47406701801)*x[1]+(0.164922402015)*x[2]
        s[0,0,2,0]=(-1.40783401802)*x[0]+(0.684901383857)*x[1]+(0.283077416398)*x[2]
        s[0,0,2,1]=(-0.283194016281)*x[0]+(-1.60164790888)*x[1]+(0.412908024034)*x[2]
        s[0,1,0,0]=(0.957812695611)*x[0]+(-0.863209638895)*x[1]+(-0.609946161661)*x[2]
        s[0,1,0,1]=(-0.410080093222)*x[0]+(0.703061988282)*x[1]+(0.0546952979936)*x[2]
        s[0,1,1,0]=(-1.43265292137)*x[0]+(-1.28132882624)*x[1]+(-0.103305431528)*x[2]
        s[0,1,1,1]=(-0.973042964661)*x[0]+(0.118946651326)*x[1]+(0.0121414797284)*x[2]
        s[0,1,2,0]=(-1.31257233483)*x[0]+(-1.90572394465)*x[1]+(-0.426472675848)*x[2]
        s[0,1,2,1]=(1.25719983663)*x[0]+(-0.68807451729)*x[1]+(0.329745219822)*x[2]
        s[0,2,0,0]=(-0.317555693596)*x[0]+(0.464722269829)*x[1]+(-0.0253415732374)*x[2]
        s[0,2,0,1]=(-0.802071257989)*x[0]+(-1.07682010382)*x[1]+(-1.52372537155)*x[2]
        s[0,2,1,0]=(-0.697501763365)*x[0]+(-1.5918814652)*x[1]+(-1.03755334358)*x[2]
        s[0,2,1,1]=(0.292457959822)*x[0]+(-0.446371848663)*x[1]+(1.32282628125)*x[2]
        s[0,2,2,0]=(1.44590220139)*x[0]+(-0.556542973367)*x[1]+(0.999665797669)*x[2]
        s[0,2,2,1]=(-0.376272847083)*x[0]+(-0.0251287305789)*x[1]+(0.364915231203)*x[2]
        s[0,3,0,0]=(0.88046028761)*x[0]+(0.372647019255)*x[1]+(-1.52356563515)*x[2]
        s[0,3,0,1]=(0.0765111785452)*x[0]+(-0.947059538185)*x[1]+(0.332098361392)*x[2]
        s[0,3,1,0]=(-1.37435064568)*x[0]+(1.04159418667)*x[1]+(0.314113199294)*x[2]
        s[0,3,1,1]=(1.03796425672)*x[0]+(-0.139508518244)*x[1]+(-0.46214547857)*x[2]
        s[0,3,2,0]=(0.285674556261)*x[0]+(-0.915609582509)*x[1]+(0.683131361973)*x[2]
        s[0,3,2,1]=(0.703275850512)*x[0]+(-0.894071596437)*x[1]+(0.118084971572)*x[2]
        s[0,4,0,0]=(-1.29507182493)*x[0]+(-0.275563564467)*x[1]+(-1.29101960297)*x[2]
        s[0,4,0,1]=(0.726127419233)*x[0]+(-0.163731035409)*x[1]+(1.45773190781)*x[2]
        s[0,4,1,0]=(-0.321962778987)*x[0]+(-0.434741013545)*x[1]+(0.348257210746)*x[2]
        s[0,4,1,1]=(0.892154653814)*x[0]+(1.12188965959)*x[1]+(-1.42589350879)*x[2]
        s[0,4,2,0]=(-1.06019594336)*x[0]+(-1.5907947093)*x[1]+(-1.64125703099)*x[2]
        s[0,4,2,1]=(0.820572048651)*x[0]+(-0.528087471135)*x[1]+(-0.0823830365488)*x[2]
        s[1,0,0,0]=(0.823455925698)*x[0]+(0.456056899485)*x[1]+(0.708907347857)*x[2]
        s[1,0,0,1]=(0.013059728892)*x[0]+(-0.772283592797)*x[1]+(0.363920439078)*x[2]
        s[1,0,1,0]=(-0.569161155945)*x[0]+(-1.60837471571)*x[1]+(-1.12748440619)*x[2]
        s[1,0,1,1]=(-1.06458634572)*x[0]+(-1.10787449393)*x[1]+(1.43157821547)*x[2]
        s[1,0,2,0]=(1.42955732758)*x[0]+(-0.769967391917)*x[1]+(0.461386087764)*x[2]
        s[1,0,2,1]=(-1.12922605294)*x[0]+(-0.256069161837)*x[1]+(0.934117328474)*x[2]
        s[1,1,0,0]=(-0.296512978109)*x[0]+(0.0247034910192)*x[1]+(-0.0918640286973)*x[2]
        s[1,1,0,1]=(0.262151201281)*x[0]+(-0.556624983637)*x[1]+(-0.82641291041)*x[2]
        s[1,1,1,0]=(-0.909049007906)*x[0]+(0.578952420411)*x[1]+(0.0963761548489)*x[2]
        s[1,1,1,1]=(0.789884036316)*x[0]+(0.714044849036)*x[1]+(-0.348148665956)*x[2]
        s[1,1,2,0]=(-0.306802924795)*x[0]+(0.166007262622)*x[1]+(0.812933157357)*x[2]
        s[1,1,2,1]=(-0.23666871482)*x[0]+(0.0180126754613)*x[1]+(0.465074288631)*x[2]
        s[1,2,0,0]=(-1.14910199315)*x[0]+(0.930765031574)*x[1]+(1.63176081675)*x[2]
        s[1,2,0,1]=(-0.220753578398)*x[0]+(0.153024000962)*x[1]+(0.29171013278)*x[2]
        s[1,2,1,0]=(-0.571208521033)*x[0]+(1.18218013588)*x[1]+(-0.0281527591651)*x[2]
        s[1,2,1,1]=(-0.0212229342253)*x[0]+(0.577228434778)*x[1]+(-0.0934776488559)*x[2]
        s[1,2,2,0]=(-0.592424182282)*x[0]+(-0.0403519194823)*x[1]+(0.150970946422)*x[2]
        s[1,2,2,1]=(-1.33143395635)*x[0]+(-1.3121470647)*x[1]+(-0.78895334942)*x[2]
        s[1,3,0,0]=(0.626711601793)*x[0]+(0.0187262812242)*x[1]+(-0.392743911033)*x[2]
        s[1,3,0,1]=(0.0360431567199)*x[0]+(-0.921408398245)*x[1]+(-0.172097608208)*x[2]
        s[1,3,1,0]=(0.0223088597951)*x[0]+(0.139631235358)*x[1]+(-0.529170239405)*x[2]
        s[1,3,1,1]=(0.11834184642)*x[0]+(-0.754799078426)*x[1]+(0.136555856977)*x[2]
        s[1,3,2,0]=(-0.288966482257)*x[0]+(-0.227333376328)*x[1]+(-0.583704417546)*x[2]
        s[1,3,2,1]=(0.263525293937)*x[0]+(0.842984179305)*x[1]+(0.592096192935)*x[2]
        s[1,4,0,0]=(-0.0145931697196)*x[0]+(-0.839359041248)*x[1]+(-0.635230965652)*x[2]
        s[1,4,0,1]=(-0.452080234732)*x[0]+(-0.500376989587)*x[1]+(-0.27541187994)*x[2]
        s[1,4,1,0]=(-0.028441856276)*x[0]+(0.99292041207)*x[1]+(-0.555205406529)*x[2]
        s[1,4,1,1]=(0.670664493076)*x[0]+(0.562911949702)*x[1]+(-0.767721105918)*x[2]
        s[1,4,2,0]=(0.960781399312)*x[0]+(0.0966843021482)*x[1]+(-0.448822164554)*x[2]
        s[1,4,2,1]=(1.19002654689)*x[0]+(0.0423440255032)*x[1]+(-0.566501161163)*x[2]
        s[2,0,0,0]=(-1.21404047236)*x[0]+(0.246576983095)*x[1]+(-1.35965095619)*x[2]
        s[2,0,0,1]=(-0.183423338333)*x[0]+(0.490089064451)*x[1]+(-0.578583461456)*x[2]
        s[2,0,1,0]=(1.12402576003)*x[0]+(-0.175494364486)*x[1]+(-0.303488944097)*x[2]
        s[2,0,1,1]=(-1.65572015787)*x[0]+(-1.0020836407)*x[1]+(0.480903327702)*x[2]
        s[2,0,2,0]=(0.145764271661)*x[0]+(-0.263090439788)*x[1]+(-1.17377189982)*x[2]
        s[2,0,2,1]=(-0.837574918907)*x[0]+(1.48561229389)*x[1]+(0.967914367565)*x[2]
        s[2,1,0,0]=(0.509756439273)*x[0]+(-0.111493103227)*x[1]+(0.748245818033)*x[2]
        s[2,1,0,1]=(0.131007923179)*x[0]+(-0.128179547101)*x[1]+(-0.755140067557)*x[2]
        s[2,1,1,0]=(-0.635852982675)*x[0]+(-0.475693144737)*x[1]+(-0.331519420739)*x[2]
        s[2,1,1,1]=(-1.29780426286)*x[0]+(0.774138127446)*x[1]+(-1.74756722025)*x[2]
        s[2,1,2,0]=(-0.14015158014)*x[0]+(-0.875354388859)*x[1]+(0.791253726662)*x[2]
        s[2,1,2,1]=(-0.0772968366884)*x[0]+(-0.937984863796)*x[1]+(-0.0602294365875)*x[2]
        s[2,2,0,0]=(0.955209462705)*x[0]+(0.849659517825)*x[1]+(0.0983431115722)*x[2]
        s[2,2,0,1]=(1.72389105142)*x[0]+(-0.412358364777)*x[1]+(-1.3083297441)*x[2]
        s[2,2,1,0]=(-0.895927963754)*x[0]+(1.65404011945)*x[1]+(0.297503560372)*x[2]
        s[2,2,1,1]=(1.34506983158)*x[0]+(-0.552690724942)*x[1]+(0.0882447776246)*x[2]
        s[2,2,2,0]=(-0.603470575964)*x[0]+(1.77788342742)*x[1]+(-0.960079624369)*x[2]
        s[2,2,2,1]=(-0.547711891427)*x[0]+(-0.562393805421)*x[1]+(-0.183240255616)*x[2]
        s[2,3,0,0]=(1.01018738642)*x[0]+(1.4085839719)*x[1]+(-0.619343450789)*x[2]
        s[2,3,0,1]=(0.525036070827)*x[0]+(-0.760210492125)*x[1]+(0.621245864477)*x[2]
        s[2,3,1,0]=(-0.786180468736)*x[0]+(0.784642311709)*x[1]+(-0.377020976891)*x[2]
        s[2,3,1,1]=(-0.333203982307)*x[0]+(-0.044265604048)*x[1]+(-1.6196925017)*x[2]
        s[2,3,2,0]=(-0.885601870306)*x[0]+(0.650248271309)*x[1]+(-1.7206378066)*x[2]
        s[2,3,2,1]=(0.0941920781315)*x[0]+(-1.24650013731)*x[1]+(0.67065624977)*x[2]
        s[2,4,0,0]=(0.0693008855416)*x[0]+(-1.33764045153)*x[1]+(0.277137873936)*x[2]
        s[2,4,0,1]=(0.887968960137)*x[0]+(1.57875979002)*x[1]+(-0.103648235191)*x[2]
        s[2,4,1,0]=(-0.547070384787)*x[0]+(-0.376770947917)*x[1]+(-1.81372790514)*x[2]
        s[2,4,1,1]=(0.375805210805)*x[0]+(0.660242065798)*x[1]+(1.69413225283)*x[2]
        s[2,4,2,0]=(-0.914715181893)*x[0]+(1.27301677765)*x[1]+(0.974031227808)*x[2]
        s[2,4,2,1]=(-0.503816280288)*x[0]+(0.552217633496)*x[1]+(-0.90127280688)*x[2]
        s[3,0,0,0]=(0.768275498738)*x[0]+(-0.811563543342)*x[1]+(-0.69998185891)*x[2]
        s[3,0,0,1]=(-0.629451202094)*x[0]+(1.02346787018)*x[1]+(0.780161524988)*x[2]
        s[3,0,1,0]=(0.584052565724)*x[0]+(-0.069634462512)*x[1]+(-1.00341091459)*x[2]
        s[3,0,1,1]=(1.03073176869)*x[0]+(1.53263439484)*x[1]+(1.03753255807)*x[2]
        s[3,0,2,0]=(0.0522744193577)*x[0]+(0.308512837882)*x[1]+(-0.315970675533)*x[2]
        s[3,0,2,1]=(0.0747430001964)*x[0]+(0.863711676177)*x[1]+(1.07705766445)*x[2]
        s[3,1,0,0]=(0.114255376522)*x[0]+(1.47818231679)*x[1]+(0.0222267873076)*x[2]
        s[3,1,0,1]=(0.613783843499)*x[0]+(0.369289287936)*x[1]+(1.08226313005)*x[2]
        s[3,1,1,0]=(0.474121107413)*x[0]+(-0.284169623677)*x[1]+(1.26739419136)*x[2]
        s[3,1,1,1]=(0.703235040854)*x[0]+(0.340511678132)*x[1]+(0.314037803901)*x[2]
        s[3,1,2,0]=(-0.754512705698)*x[0]+(0.957288483514)*x[1]+(1.38853172146)*x[2]
        s[3,1,2,1]=(0.338933480448)*x[0]+(1.30797425088)*x[1]+(-0.492819385452)*x[2]
        s[3,2,0,0]=(0.477902330587)*x[0]+(0.658604196318)*x[1]+(-0.669039618554)*x[2]
        s[3,2,0,1]=(0.584833130533)*x[0]+(0.205626741315)*x[1]+(-0.449267859148)*x[2]
        s[3,2,1,0]=(0.392564971475)*x[0]+(-0.867191184183)*x[1]+(0.508221616556)*x[2]
        s[3,2,1,1]=(0.199023224346)*x[0]+(0.210018232234)*x[1]+(0.17332311179)*x[2]
        s[3,2,2,0]=(-0.530182002398)*x[0]+(-0.70410123336)*x[1]+(0.454418407328)*x[2]
        s[3,2,2,1]=(-1.00165228123)*x[0]+(0.253353462588)*x[1]+(1.07981964164)*x[2]
        s[3,3,0,0]=(-0.00388223127856)*x[0]+(-0.256156832173)*x[1]+(-1.22618259947)*x[2]
        s[3,3,0,1]=(1.04947612909)*x[0]+(-0.590938924785)*x[1]+(-0.80453902875)*x[2]
        s[3,3,1,0]=(0.395025775135)*x[0]+(-1.30382091375)*x[1]+(1.00251618393)*x[2]
        s[3,3,1,1]=(-0.601315067206)*x[0]+(-0.226958312276)*x[1]+(-0.69720544326)*x[2]
        s[3,3,2,0]=(0.748907154335)*x[0]+(-0.694932273918)*x[1]+(0.143655252827)*x[2]
        s[3,3,2,1]=(-0.228346281761)*x[0]+(0.0818793892413)*x[1]+(0.376376078012)*x[2]
        s[3,4,0,0]=(0.675502208069)*x[0]+(0.498131679631)*x[1]+(-0.606527858224)*x[2]
        s[3,4,0,1]=(0.230994211989)*x[0]+(-1.81768645462)*x[1]+(1.59488392513)*x[2]
        s[3,4,1,0]=(-1.35866095694)*x[0]+(-0.122601842831)*x[1]+(1.73437014104)*x[2]
        s[3,4,1,1]=(0.63210638436)*x[0]+(1.50387972886)*x[1]+(0.655743288533)*x[2]
        s[3,4,2,0]=(1.42156357373)*x[0]+(-0.701544596166)*x[1]+(0.88931540397)*x[2]
        s[3,4,2,1]=(-0.405515262733)*x[0]+(0.872922189212)*x[1]+(-1.13847750709)*x[2]
        ref[0,0,0,0]=(0.415231330279)/(o+1.)+(0.982601202296)+(-0.713217663735)*0.5**o
        ref[0,0,0,1]=(0.650505784971)/(o+1.)+(0.406482330235)+(0.983584624374)*0.5**o
        ref[0,0,1,0]=(-0.615926989595)/(o+1.)+(0.113540305186)+(0.98566940684)*0.5**o
        ref[0,0,1,1]=(-1.36210768774)/(o+1.)+(0.330430698616)+(-0.558173143358)*0.5**o
        ref[0,0,2,0]=(0.589319954581)/(o+1.)+(-0.146725890489)+(-0.735723391368)*0.5**o
        ref[0,0,2,1]=(-0.819641973816)/(o+1.)+(0.0696728799133)+(-0.791637687141)*0.5**o
        ref[0,1,0,0]=(-0.984505810214)/(o+1.)+(0.0623424750886)+(0.344477755092)*0.5**o
        ref[0,1,0,1]=(0.0372238318297)/(o+1.)+(0.201167151246)+(-0.0918809412683)*0.5**o
        ref[0,1,1,0]=(-0.988162693644)/(o+1.)+(-0.658809560166)+(-0.51150536516)*0.5**o
        ref[0,1,1,1]=(0.191409590473)/(o+1.)+(-0.426836466412)+(-0.179691491256)*0.5**o
        ref[0,1,2,0]=(-1.50151479209)/(o+1.)+(-0.648874443157)+(-0.84550527693)*0.5**o
        ref[0,1,2,1]=(-1.3172677321)/(o+1.)+(0.885092333538)+(0.445953604186)*0.5**o
        ref[0,2,0,0]=(0.1269682286)/(o+1.)+(-0.175772039094)+(0.346400852583)*0.5**o
        ref[0,2,0,1]=(-1.46726533796)/(o+1.)+(-1.05015066374)+(0.164949932075)*0.5**o
        ref[0,2,1,0]=(-1.23701559967)/(o+1.)+(-0.825973196415)+(-0.437974579642)*0.5**o
        ref[0,2,1,1]=(0.132084480709)/(o+1.)+(0.0364121752536)+(0.964003561191)*0.5**o
        ref[0,2,2,0]=(-0.0575514788048)/(o+1.)+(0.574131321284)+(0.798313861933)*0.5**o
        ref[0,2,2,1]=(-0.854422215721)/(o+1.)+(0.454172855359)+(-0.0904098414555)*0.5**o
        ref[0,3,0,0]=(-1.04634289994)/(o+1.)+(0.354533324899)+(0.0668179218583)*0.5**o
        ref[0,3,0,1]=(-0.415702586981)/(o+1.)+(0.126519101056)+(-0.375785613379)*0.5**o
        ref[0,3,1,0]=(-0.268120824236)/(o+1.)+(0.500467336081)+(-0.751457107646)*0.5**o
        ref[0,3,1,1]=(0.827791290739)/(o+1.)+(-0.347418978526)+(0.303356926219)*0.5**o
        ref[0,3,2,0]=(-0.1387759961)/(o+1.)+(-0.350616137823)+(0.893204607472)*0.5**o
        ref[0,3,2,1]=(0.129310667468)/(o+1.)+(-0.252264005651)+(0.302506569481)*0.5**o
        ref[0,4,0,0]=(-0.021050677621)/(o+1.)+(-0.980639199085)+(-0.879325916576)*0.5**o
        ref[0,4,0,1]=(-0.262715302419)/(o+1.)+(1.20073556064)+(-0.118627527238)*0.5**o
        ref[0,4,1,0]=(0.261035782917)/(o+1.)+(-0.449979981215)+(0.230477597727)*0.5**o
        ref[0,4,1,1]=(-0.47960276307)/(o+1.)+(0.219069456491)+(0.629614654702)*0.5**o
        ref[0,4,2,0]=(-1.27925873031)/(o+1.)+(-1.32602054802)+(-0.360947857308)*0.5**o
        ref[0,4,2,1]=(-1.34486930774)/(o+1.)+(0.546553886393)+(0.46186307592)*0.5**o
        ref[1,0,0,0]=(0.430196940607)/(o+1.)+(0.311302638938)+(0.935617954556)*0.5**o
        ref[1,0,0,1]=(0.0865234935036)/(o+1.)+(-0.268442519337)+(0.055058120343)*0.5**o
        ref[1,0,1,0]=(-1.48387607046)/(o+1.)+(-0.786378289878)+(-0.248387627621)*0.5**o
        ref[1,0,1,1]=(0.758534313728)/(o+1.)+(-0.265213280892)+(-0.968990376127)*0.5**o
        ref[1,0,2,0]=(-0.713142570031)/(o+1.)+(0.462618013893)+(0.908882565677)*0.5**o
        ref[1,0,2,1]=(-0.482219090892)/(o+1.)+(0.307689127346)+(-0.584337050105)*0.5**o
        ref[1,1,0,0]=(-1.17250881543)/(o+1.)+(0.113442845094)+(0.581949609453)*0.5**o
        ref[1,1,0,1]=(-1.3468375595)/(o+1.)+(0.395669021834)+(-0.565387176936)*0.5**o
        ref[1,1,1,0]=(0.2018351732)/(o+1.)+(-0.157830690125)+(-0.119894225596)*0.5**o
        ref[1,1,1,1]=(1.21543817764)/(o+1.)+(-0.0497756593583)+(0.0398933604741)*0.5**o
        ref[1,1,2,0]=(-0.162294483944)/(o+1.)+(0.85994581496)+(-0.885459650791)*0.5**o
        ref[1,1,2,1]=(1.24422084677)/(o+1.)+(-0.379646444866)+(-0.238509707766)*0.5**o
        ref[1,2,0,0]=(1.5435438681)/(o+1.)+(0.08540126524)+(-0.300922543405)*0.5**o
        ref[1,2,0,1]=(-0.112210840363)/(o+1.)+(0.480051249506)+(-0.623911103306)*0.5**o
        ref[1,2,1,0]=(0.285721711645)/(o+1.)+(0.0100191655774)+(0.27705881288)*0.5**o
        ref[1,2,1,1]=(0.66930209958)/(o+1.)+(-0.542529578578)+(0.878284909271)*0.5**o
        ref[1,2,2,0]=(1.12100699995)/(o+1.)+(-0.678457673316)+(-0.245896808662)*0.5**o
        ref[1,2,2,1]=(-0.88612973059)/(o+1.)+(-1.00088764314)+(-0.544629353594)*0.5**o
        ref[1,3,0,0]=(0.387350166894)/(o+1.)+(-0.499575950447)+(0.864495705983)*0.5**o
        ref[1,3,0,1]=(-0.942393535847)/(o+1.)+(0.0303050564575)+(-0.175679426801)*0.5**o
        ref[1,3,1,0]=(-0.798050539357)/(o+1.)+(0.126548837387)+(0.177722720331)*0.5**o
        ref[1,3,1,1]=(-1.50957773465)/(o+1.)+(0.172938183698)+(0.663799992229)*0.5**o
        ref[1,3,2,0]=(0.469444503978)/(o+1.)+(-0.647836043317)+(-0.273776693474)*0.5**o
        ref[1,3,2,1]=(0.310061780349)/(o+1.)+(1.05452404891)+(-0.720504211989)*0.5**o
        ref[1,4,0,0]=(-0.820548632752)/(o+1.)+(0.105943005381)+(-0.88052055463)*0.5**o
        ref[1,4,0,1]=(-0.158467564266)/(o+1.)+(-0.709811754591)+(0.35022196919)*0.5**o
        ref[1,4,1,0]=(0.00990374166591)/(o+1.)+(0.51839921597)+(-0.637429024341)*0.5**o
        ref[1,4,1,1]=(-0.246106620377)/(o+1.)+(0.33455634807)+(0.0428492610972)*0.5**o
        ref[1,4,2,0]=(0.472702988578)/(o+1.)+(0.0688149777117)+(-0.00168940709497)*0.5**o
        ref[1,4,2,1]=(0.284541065049)/(o+1.)+(-0.0548349320689)+(0.490998210318)*0.5**o
        ref[2,0,0,0]=(-0.733338280715)/(o+1.)+(-0.47189177308)+(-0.649992618571)*0.5**o
        ref[2,0,0,1]=(-0.948815264173)/(o+1.)+(0.837962467604)+(-0.999027406375)*0.5**o
        ref[2,0,1,0]=(-0.0944539750337)/(o+1.)+(0.0746201219713)+(0.590256182541)*0.5**o
        ref[2,0,1,1]=(0.044594970161)/(o+1.)+(-0.717673746264)+(-0.786147948501)*0.5**o
        ref[2,0,2,0]=(-0.387429911457)/(o+1.)+(-0.0765750118601)+(-0.750518132769)*0.5**o
        ref[2,0,2,1]=(1.28025192232)/(o+1.)+(0.425010917096)+(-0.514322013965)*0.5**o
        ref[2,1,0,0]=(0.85161480185)/(o+1.)+(-0.117153136076)+(0.52920062438)*0.5**o
        ref[2,1,0,1]=(-0.558914411305)/(o+1.)+(-0.229298513746)+(0.265199747318)*0.5**o
        ref[2,1,1,0]=(-0.33769433116)/(o+1.)+(-0.678382612728)+(0.251394008465)*0.5**o
        ref[2,1,1,1]=(-0.990803230186)/(o+1.)+(-0.413400680719)+(-0.453628764049)*0.5**o
        ref[2,1,2,0]=(-0.377834292093)/(o+1.)+(0.521893888533)+(-0.890205727309)*0.5**o
        ref[2,1,2,1]=(-1.33150035489)/(o+1.)+(-0.0586549531766)+(0.373299124171)*0.5**o
        ref[2,2,0,0]=(-0.166255483746)/(o+1.)+(0.816713521389)+(0.436040533071)*0.5**o
        ref[2,2,0,1]=(-1.39682071748)/(o+1.)+(0.21770833884)+(0.964606982344)*0.5**o
        ref[2,2,1,0]=(1.3247136685)/(o+1.)+(-0.0466914213895)+(-0.175715109647)*0.5**o
        ref[2,2,1,1]=(-0.522454725229)/(o+1.)+(0.291262422208)+(0.820553765077)*0.5**o
        ref[2,2,2,0]=(0.136169385992)/(o+1.)+(0.253426637378)+(-0.42868943366)*0.5**o
        ref[2,2,2,1]=(-0.131390793028)/(o+1.)+(-0.715512276073)+(0.269069392711)*0.5**o
        ref[2,3,0,0]=(0.1679740824)/(o+1.)+(0.653952560954)+(0.323548703221)*0.5**o
        ref[2,3,0,1]=(-0.0511907658967)/(o+1.)+(-0.236699539579)+(0.910661288233)*0.5**o
        ref[2,3,1,0]=(0.451520708483)/(o+1.)+(-0.065786465884)+(-0.698506910633)*0.5**o
        ref[2,3,1,1]=(-0.542162878453)/(o+1.)+(-0.237944135968)+(-0.979110937669)*0.5**o
        ref[2,3,2,0]=(-0.489308922223)/(o+1.)+(-0.563117809253)+(-0.34044686487)*0.5**o
        ref[2,3,2,1]=(0.382652422829)/(o+1.)+(-0.328188452455)+(-0.207927327325)*0.5**o
        ref[2,4,0,0]=(0.0347966753187)/(o+1.)+(-0.629864354262)+(0.233730341151)*0.5**o
        ref[2,4,0,1]=(0.123939573502)/(o+1.)+(1.03666652808)+(0.165807885309)*0.5**o
        ref[2,4,1,0]=(-0.829855747878)/(o+1.)+(-0.507284312041)+(-0.89314486588)*0.5**o
        ref[2,4,1,1]=(1.80556499882)/(o+1.)+(0.554771100606)+(-0.1849276706)*0.5**o
        ref[2,4,2,0]=(0.829439412646)/(o+1.)+(0.342620313927)+(-0.182347216931)*0.5**o
        ref[2,4,2,1]=(-0.824674529166)/(o+1.)+(-0.172483000807)+(0.316769077106)*0.5**o
        ref[3,0,0,0]=(-1.81534289679)/(o+1.)+(0.235921261477)+(0.600230470324)*0.5**o
        ref[3,0,0,1]=(0.758824131522)/(o+1.)+(0.540267308765)+(-0.665180555978)*0.5**o
        ref[3,0,1,0]=(-0.61930013537)/(o+1.)+(-0.0863636993976)+(0.303034722786)*0.5**o
        ref[3,0,1,1]=(1.09849962509)/(o+1.)+(0.780381688481)+(0.941635719543)*0.5**o
        ref[3,0,2,0]=(0.984167465846)/(o+1.)+(-0.471651730612)+(0.00395257708327)*0.5**o
        ref[3,0,2,1]=(-0.0216502142754)/(o+1.)+(1.25274987136)+(-0.468337187615)*0.5**o
        ref[3,1,0,0]=(1.21762392246)/(o+1.)+(0.0235896546035)+(0.349861248945)*0.5**o
        ref[3,1,0,1]=(0.681142299068)/(o+1.)+(0.586170195383)+(0.211853571648)*0.5**o
        ref[3,1,1,0]=(1.22842568638)/(o+1.)+(0.308050279545)+(-0.387180570377)*0.5**o
        ref[3,1,1,1]=(-0.708900188728)/(o+1.)+(1.04208633918)+(-0.0174879667472)*0.5**o
        ref[3,1,2,0]=(1.79188214766)/(o+1.)+(0.301719505956)+(-0.804013660301)*0.5**o
        ref[3,1,2,1]=(0.669297533033)/(o+1.)+(0.119247079862)+(0.246296653123)*0.5**o
        ref[3,2,0,0]=(-0.791486989477)/(o+1.)+(0.204677775698)+(0.849598346434)*0.5**o
        ref[3,2,0,1]=(-0.505948254664)/(o+1.)+(0.0688840300017)+(0.70937220736)*0.5**o
        ref[3,2,1,0]=(-0.691348115315)/(o+1.)+(0.437490279091)+(-0.150037039019)*0.5**o
        ref[3,2,1,1]=(0.708369627369)/(o+1.)+(0.176205102648)+(-0.478415264295)*0.5**o
        ref[3,2,2,0]=(0.297093119787)/(o+1.)+(-0.248132872678)+(-0.580692202861)*0.5**o
        ref[3,2,2,1]=(-0.476148841282)/(o+1.)+(0.443525381364)+(-0.0793810984529)*0.5**o
        ref[3,3,0,0]=(-0.901387033571)/(o+1.)+(-0.628090499737)+(0.671346370124)*0.5**o
        ref[3,3,0,1]=(-0.123005978892)/(o+1.)+(-0.59694482259)+(0.970893799626)*0.5**o
        ref[3,3,1,0]=(-0.784454504087)/(o+1.)+(0.436648462004)+(0.00487862539679)*0.5**o
        ref[3,3,1,1]=(-1.73297919322)/(o+1.)+(-0.09410977014)+(0.395719910762)*0.5**o
        ref[3,3,2,0]=(0.0989755292654)/(o+1.)+(0.062352403584)+(-0.0260502031901)*0.5**o
        ref[3,3,2,1]=(-0.690845491742)/(o+1.)+(0.678904082779)+(-0.437053488325)*0.5**o
        ref[3,4,0,0]=(-0.801695028055)/(o+1.)+(0.731880266747)+(-0.0949594759642)*0.5**o
        ref[3,4,0,1]=(-0.263446786535)/(o+1.)+(0.0496642554438)+(0.172309958154)*0.5**o
        ref[3,4,1,0]=(0.542595531974)/(o+1.)+(0.308327980561)+(-0.906144151831)*0.5**o
        ref[3,4,1,1]=(0.637885608826)/(o+1.)+(1.16403227519)+(-0.174220757444)*0.5**o
        ref[3,4,2,0]=(0.946926166426)/(o+1.)+(0.0242541266649)+(0.613899961774)*0.5**o
        ref[3,4,2,1]=(-0.0303035732231)/(o+1.)+(0.0375314231208)+(-0.715829853629)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactZero_fromSymbol_ReducedContinuousFunction_rank0(self):
      """
      tests integral of rank 0 Symbol on the ReducedFunctionOnContactZero

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=())
      s=Data(0,(),w)
      if dim==2:
        s=(1.70255303494)*x[0]+(0.300011314949)*x[1]
        ref=(0.345983004398)/(o+1.)+(0.363938720886)+(0.928703903719)*0.5**o
      else:
        s=(-1.68260770684)*x[0]+(-1.14729600003)*x[1]+(-1.35659279947)*x[2]
        ref=(-1.3902352024)/(o+1.)+(-1.03788235052)+(-0.720496602901)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactZero_fromSymbol_ReducedContinuousFunction_rank1(self):
      """
      tests integral of rank 1 Symbol on the ReducedFunctionOnContactZero

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(2,))
      s=Data(0,(2,),w)
      ref=numpy.zeros((2,),numpy.float_)
      if dim==2:
        s[0]=(0.23958799559)*x[0]+(-0.0458757682877)*x[1]
        s[1]=(0.0897882061379)*x[0]+(-1.54974909413)*x[1]
        ref[0]=(0.0485704769529)/(o+1.)+(0.210351971249)+(-0.275562192148)*0.5**o
        ref[1]=(-0.821668980438)/(o+1.)+(-0.712207676061)+(0.786123444573)*0.5**o
      else:
        s[0]=(-1.44817804789)*x[0]+(0.670605136525)*x[1]+(-0.593548775192)*x[2]
        s[1]=(-0.585702105721)*x[0]+(0.0665105334471)*x[1]+(-0.228099551666)*x[2]
        ref[0]=(0.859021549917)/(o+1.)+(-0.759857813727)+(-0.710427609023)*0.5**o
        ref[1]=(-1.40806718776)/(o+1.)+(0.267420623854)+(0.125934816108)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactZero_fromSymbol_ReducedContinuousFunction_rank2(self):
      """
      tests integral of rank 2 Symbol on the ReducedFunctionOnContactZero

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5))
      s=Data(0,(4, 5),w)
      ref=numpy.zeros((4, 5),numpy.float_)
      if dim==2:
        s[0,0]=(0.933177344575)*x[0]+(-0.643793974434)*x[1]
        s[0,1]=(-0.311019666872)*x[0]+(0.686195407652)*x[1]
        s[0,2]=(0.470203454822)*x[0]+(1.32374103685)*x[1]
        s[0,3]=(-0.653103411901)*x[0]+(0.477452857192)*x[1]
        s[0,4]=(1.12606394408)*x[0]+(0.0898490519564)*x[1]
        s[1,0]=(1.26644130664)*x[0]+(0.226895598587)*x[1]
        s[1,1]=(0.0271345466814)*x[0]+(0.435024033603)*x[1]
        s[1,2]=(-0.227055843926)*x[0]+(-0.144686134637)*x[1]
        s[1,3]=(0.466198287572)*x[0]+(-0.130830709376)*x[1]
        s[1,4]=(1.04066080781)*x[0]+(0.546282129071)*x[1]
        s[2,0]=(0.317469515041)*x[0]+(1.26366516514)*x[1]
        s[2,1]=(-1.20210768286)*x[0]+(-0.0533328989847)*x[1]
        s[2,2]=(0.278459393186)*x[0]+(-0.379156129627)*x[1]
        s[2,3]=(1.03459096792)*x[0]+(-0.610026027371)*x[1]
        s[2,4]=(1.288131278)*x[0]+(-0.309929698872)*x[1]
        s[3,0]=(0.32856946183)*x[0]+(0.225709965354)*x[1]
        s[3,1]=(-1.03183224237)*x[0]+(-0.0153171719792)*x[1]
        s[3,2]=(0.259516430572)*x[0]+(-0.776082702284)*x[1]
        s[3,3]=(-0.417978040907)*x[0]+(-0.0941785049007)*x[1]
        s[3,4]=(1.07527114074)*x[0]+(-0.0198401110946)*x[1]
        ref[0,0]=(-0.79959835955)/(o+1.)+(0.566758804494)+(-0.0445358792971)*0.5**o
        ref[0,1]=(0.583455543567)/(o+1.)+(0.0458448886173)+(-0.299969580021)*0.5**o
        ref[0,2]=(0.471614825642)/(o+1.)+(0.891760595232)+(-0.461191524434)*0.5**o
        ref[0,3]=(0.672045220068)/(o+1.)+(-0.56343583876)+(0.279175902744)*0.5**o
        ref[0,4]=(-0.127993678518)/(o+1.)+(0.52668612755)+(0.290534419455)*0.5**o
        ref[1,0]=(0.822774751438)/(o+1.)+(-0.011855097694)+(0.694272349175)*0.5**o
        ref[1,1]=(0.552626324952)/(o+1.)+(0.277929562319)+(-0.646326869305)*0.5**o
        ref[1,2]=(-0.565996176797)/(o+1.)+(-0.128057099164)+(0.450368396562)*0.5**o
        ref[1,3]=(0.627837338323)/(o+1.)+(-0.555359040353)+(0.818248320579)*0.5**o
        ref[1,4]=(0.722984603229)/(o+1.)+(0.0957452220354)+(0.672467889583)*0.5**o
        ref[2,0]=(0.664524759547)/(o+1.)+(0.678277833656)+(-0.439945746675)*0.5**o
        ref[2,1]=(0.908213473231)/(o+1.)+(-0.805606306623)+(-0.552441441832)*0.5**o
        ref[2,2]=(0.0908414991316)/(o+1.)+(0.131818510391)+(-0.455175256354)*0.5**o
        ref[2,3]=(-0.119095663088)/(o+1.)+(-0.223144373358)+(0.989949350348)*0.5**o
        ref[2,4]=(-0.0540728359185)/(o+1.)+(0.0748680956845)+(0.882538223681)*0.5**o
        ref[3,0]=(0.98273847633)/(o+1.)+(-0.421621406424)+(0.414783763702)*0.5**o
        ref[3,1]=(0.29285575515)/(o+1.)+(-0.462798732226)+(-0.41440770505)*0.5**o
        ref[3,2]=(-0.509767634157)/(o+1.)+(-0.480617720091)+(0.954436802627)*0.5**o
        ref[3,3]=(-0.762302422898)/(o+1.)+(0.100280539595)+(0.0495847978999)*0.5**o
        ref[3,4]=(-0.154196647936)/(o+1.)+(0.329602137701)+(0.550423402184)*0.5**o
      else:
        s[0,0]=(1.34601675616)*x[0]+(-0.451295005459)*x[1]+(-0.49622174234)*x[2]
        s[0,1]=(-0.327908015969)*x[0]+(-1.21019079469)*x[1]+(1.09622527656)*x[2]
        s[0,2]=(-0.0243507161566)*x[0]+(-0.016230100758)*x[1]+(-1.37311484633)*x[2]
        s[0,3]=(-0.445665346289)*x[0]+(-0.626824621259)*x[1]+(-0.196033254199)*x[2]
        s[0,4]=(-0.819703680211)*x[0]+(1.30953832902)*x[1]+(0.0778023021203)*x[2]
        s[1,0]=(0.219519741099)*x[0]+(-1.47335622754)*x[1]+(0.198662145648)*x[2]
        s[1,1]=(-0.65456993306)*x[0]+(-0.125758153194)*x[1]+(-1.53966098547)*x[2]
        s[1,2]=(-0.112157349953)*x[0]+(-0.459351490445)*x[1]+(1.6889234353)*x[2]
        s[1,3]=(0.102642361518)*x[0]+(1.10412193101)*x[1]+(-0.239843060928)*x[2]
        s[1,4]=(-0.232367456037)*x[0]+(-1.17836662288)*x[1]+(-0.814085860866)*x[2]
        s[2,0]=(-1.00510966212)*x[0]+(1.23960710585)*x[1]+(0.0848958213029)*x[2]
        s[2,1]=(-1.93901641139)*x[0]+(-0.0469703867512)*x[1]+(-0.21201338191)*x[2]
        s[2,2]=(1.42666776379)*x[0]+(1.04000263985)*x[1]+(0.151609334515)*x[2]
        s[2,3]=(-0.297049329338)*x[0]+(-0.103284668913)*x[1]+(-0.110261476034)*x[2]
        s[2,4]=(0.964711394219)*x[0]+(1.28218755108)*x[1]+(-1.2382220868)*x[2]
        s[3,0]=(-0.922061452692)*x[0]+(-1.36593539333)*x[1]+(1.11436583585)*x[2]
        s[3,1]=(0.284590223973)*x[0]+(1.08442017719)*x[1]+(1.5619949376)*x[2]
        s[3,2]=(-0.245207105356)*x[0]+(-1.77865635864)*x[1]+(-0.908570594143)*x[2]
        s[3,3]=(0.426334196419)*x[0]+(-0.501103031142)*x[1]+(0.199762115243)*x[2]
        s[3,4]=(1.33158896071)*x[0]+(0.44737802367)*x[1]+(-0.76095091762)*x[2]
        ref[0,0]=(-0.581088472202)/(o+1.)+(0.287163129514)+(0.405262221535)*0.5**o
        ref[0,1]=(0.695209950176)/(o+1.)+(-0.748195486472)+(0.359307488667)*0.5**o
        ref[0,2]=(0.313650504565)/(o+1.)+(-1.20553227172)+(0.683718375628)*0.5**o
        ref[0,3]=(-0.872265925416)/(o+1.)+(-0.216679466249)+(0.0371016361661)*0.5**o
        ref[0,4]=(1.07552646704)/(o+1.)+(-0.293195846094)+(0.0785021760824)*0.5**o
        ref[1,0]=(-1.22710752208)/(o+1.)+(0.409726431872)+(-0.647519682458)*0.5**o
        ref[1,1]=(-0.940490307845)/(o+1.)+(-0.82083756271)+(0.26217636154)*0.5**o
        ref[1,2]=(1.00054234642)/(o+1.)+(-0.0533842117578)+(0.223640672001)*0.5**o
        ref[1,3]=(0.0595140200914)/(o+1.)+(0.407850368171)+(0.0917064751695)*0.5**o
        ref[1,4]=(-0.50735295069)/(o+1.)+(-1.10564117596)+(0.493815362818)*0.5**o
        ref[2,0]=(0.786394742154)/(o+1.)+(-0.212261329287)+(-0.0424788185539)*0.5**o
        ref[2,1]=(-0.868240537836)/(o+1.)+(-0.186797194849)+(-0.956165252514)*0.5**o
        ref[2,2]=(-0.67113838962)/(o+1.)+(1.26208372573)+(0.765250676321)*0.5**o
        ref[2,3]=(-1.2390712788)/(o+1.)+(0.286327258603)+(0.15582128731)*0.5**o
        ref[2,4]=(-0.00831435188402)/(o+1.)+(0.305363694299)+(0.406263821787)*0.5**o
        ref[3,0]=(-0.123205370838)/(o+1.)+(-0.410740153717)+(-0.228945331902)*0.5**o
        ref[3,1]=(1.39199709306)/(o+1.)+(0.646045427731)+(0.246917390241)*0.5**o
        ref[3,2]=(-1.10740261108)/(o+1.)+(-0.860231147304)+(-0.104569152453)*0.5**o
        ref[3,3]=(-0.506214554892)/(o+1.)+(0.53544681941)+(-0.439685803409)*0.5**o
        ref[3,4]=(0.846470301583)/(o+1.)+(-0.134756583683)+(0.441058932543)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactZero_fromSymbol_ReducedContinuousFunction_rank3(self):
      """
      tests integral of rank 3 Symbol on the ReducedFunctionOnContactZero

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(6, 2, 2))
      s=Data(0,(6, 2, 2),w)
      ref=numpy.zeros((6, 2, 2),numpy.float_)
      if dim==2:
        s[0,0,0]=(-0.907789262757)*x[0]+(1.43778352913)*x[1]
        s[0,0,1]=(-0.441541130755)*x[0]+(1.75922038312)*x[1]
        s[0,1,0]=(-0.387854312613)*x[0]+(1.22523701409)*x[1]
        s[0,1,1]=(-1.15521894576)*x[0]+(0.0289558035739)*x[1]
        s[1,0,0]=(-0.640400813271)*x[0]+(-1.69766965873)*x[1]
        s[1,0,1]=(-0.641296842318)*x[0]+(-1.35895483901)*x[1]
        s[1,1,0]=(-0.14735518066)*x[0]+(0.520543405774)*x[1]
        s[1,1,1]=(0.369918186173)*x[0]+(0.209781392053)*x[1]
        s[2,0,0]=(0.677808706359)*x[0]+(0.481042907356)*x[1]
        s[2,0,1]=(-0.661702781276)*x[0]+(1.86193178646)*x[1]
        s[2,1,0]=(0.323230036331)*x[0]+(-1.45705288353)*x[1]
        s[2,1,1]=(1.8644362516)*x[0]+(0.764691070666)*x[1]
        s[3,0,0]=(-1.33103240086)*x[0]+(1.29035953404)*x[1]
        s[3,0,1]=(-0.161074703654)*x[0]+(-0.127322745584)*x[1]
        s[3,1,0]=(-0.359913251027)*x[0]+(-0.0952192793759)*x[1]
        s[3,1,1]=(-0.34519536765)*x[0]+(0.483950473232)*x[1]
        s[4,0,0]=(0.981563058768)*x[0]+(-0.0744642367992)*x[1]
        s[4,0,1]=(-0.45912279344)*x[0]+(0.0011253607226)*x[1]
        s[4,1,0]=(-0.618322333127)*x[0]+(0.199638289627)*x[1]
        s[4,1,1]=(-0.371110449749)*x[0]+(-0.382992800899)*x[1]
        s[5,0,0]=(0.840829543072)*x[0]+(-0.925833368141)*x[1]
        s[5,0,1]=(-0.0348263534937)*x[0]+(0.267324292682)*x[1]
        s[5,1,0]=(0.0785260612832)*x[0]+(1.59479548515)*x[1]
        s[5,1,1]=(-0.203914067868)*x[0]+(1.28202597585)*x[1]
        ref[0,0,0]=(0.898417888273)/(o+1.)+(0.171058001117)+(-0.710539624134)*0.5**o
        ref[0,0,1]=(0.949672663046)/(o+1.)+(0.186488421776)+(-0.00497025423347)*0.5**o
        ref[0,1,0]=(0.434571276181)/(o+1.)+(0.106095788846)+(0.190619847602)*0.5**o
        ref[0,1,1]=(0.337711977059)/(o+1.)+(-0.313335995076)+(-0.837303129097)*0.5**o
        ref[1,0,0]=(-0.757199193222)/(o+1.)+(-0.36300260101)+(-0.854866076761)*0.5**o
        ref[1,0,1]=(-0.767144980208)/(o+1.)+(-0.215422826519)+(-0.802261048082)*0.5**o
        ref[1,1,0]=(-0.13190981045)/(o+1.)+(-0.0320289199048)+(0.569155875375)*0.5**o
        ref[1,1,1]=(0.0466405054813)/(o+1.)+(0.565229924314)+(-0.597400775882)*0.5**o
        ref[2,0,0]=(0.760951384359)/(o+1.)+(0.0600961970572)+(0.277707835242)*0.5**o
        ref[2,0,1]=(0.869894715615)/(o+1.)+(0.631323963868)+(-0.932313638168)*0.5**o
        ref[2,1,0]=(-0.666302256751)/(o+1.)+(-0.510864963778)+(0.554209337103)*0.5**o
        ref[2,1,1]=(0.934304180676)/(o+1.)+(0.350887235523)+(0.993048670542)*0.5**o
        ref[3,0,0]=(0.930749923608)/(o+1.)+(-0.283299908306)+(-0.404822973819)*0.5**o
        ref[3,0,1]=(-0.572270969539)/(o+1.)+(0.583432503768)+(-0.882991487235)*0.5**o
        ref[3,1,0]=(0.642892258995)/(o+1.)+(-0.0826293437351)+(-0.932766101927)*0.5**o
        ref[3,1,1]=(0.679591980627)/(o+1.)+(-0.379930907942)+(0.219024940838)*0.5**o
        ref[4,0,0]=(-0.0326170324069)/(o+1.)+(0.0903259186859)+(0.759064017004)*0.5**o
        ref[4,0,1]=(-0.129142887308)/(o+1.)+(0.279560607285)+(-0.887975759979)*0.5**o
        ref[4,1,0]=(0.752463858901)/(o+1.)+(-0.256945595979)+(-0.657256710443)*0.5**o
        ref[4,1,1]=(0.565307319705)/(o+1.)+(-0.898218881636)+(0.477027192919)*0.5**o
        ref[5,0,0]=(-0.28969980947)/(o+1.)+(0.0341656280661)+(0.136364728269)*0.5**o
        ref[5,0,1]=(-0.362525553087)/(o+1.)+(0.17876968499)+(0.237484122295)*0.5**o
        ref[5,1,0]=(0.856225350097)/(o+1.)+(0.375747136218)+(0.0656019238951)*0.5**o
        ref[5,1,1]=(0.358764765052)/(o+1.)+(0.783470816704)+(-0.847594490479)*0.5**o
      else:
        s[0,0,0]=(-0.18318998999)*x[0]+(-1.71372306128)*x[1]+(1.13788709107)*x[2]
        s[0,0,1]=(0.95308908228)*x[0]+(0.123844162735)*x[1]+(0.0868410957691)*x[2]
        s[0,1,0]=(-0.712762731169)*x[0]+(-1.86018372873)*x[1]+(-0.0783828227128)*x[2]
        s[0,1,1]=(-1.72172214895)*x[0]+(-1.25477573826)*x[1]+(1.28096692374)*x[2]
        s[1,0,0]=(0.86250397119)*x[0]+(1.44314824122)*x[1]+(0.763303582012)*x[2]
        s[1,0,1]=(1.05520701337)*x[0]+(1.04926637639)*x[1]+(-0.378977987156)*x[2]
        s[1,1,0]=(0.753550528625)*x[0]+(-1.63882951308)*x[1]+(0.259583964201)*x[2]
        s[1,1,1]=(0.778941098246)*x[0]+(-1.10564597288)*x[1]+(-1.42906071049)*x[2]
        s[2,0,0]=(-0.842438687373)*x[0]+(-0.202024764935)*x[1]+(0.289144880342)*x[2]
        s[2,0,1]=(-1.02319531994)*x[0]+(0.155875285165)*x[1]+(0.310920357117)*x[2]
        s[2,1,0]=(-1.45156693584)*x[0]+(0.12231441645)*x[1]+(0.722232222435)*x[2]
        s[2,1,1]=(-1.08553719113)*x[0]+(0.316521434206)*x[1]+(0.720555059931)*x[2]
        s[3,0,0]=(1.91846595467)*x[0]+(0.199356393766)*x[1]+(-0.438144879601)*x[2]
        s[3,0,1]=(-0.341123312011)*x[0]+(0.325154462361)*x[1]+(0.691743856063)*x[2]
        s[3,1,0]=(1.72547153856)*x[0]+(-0.822030080649)*x[1]+(-1.15169896539)*x[2]
        s[3,1,1]=(0.243194545088)*x[0]+(-0.718504738836)*x[1]+(0.560536413366)*x[2]
        s[4,0,0]=(0.248455459192)*x[0]+(0.415661600687)*x[1]+(0.21712561395)*x[2]
        s[4,0,1]=(-1.23171610721)*x[0]+(-0.443121697053)*x[1]+(0.724481514081)*x[2]
        s[4,1,0]=(-0.0327531013679)*x[0]+(0.385175130963)*x[1]+(-0.0484020348058)*x[2]
        s[4,1,1]=(-0.199960439132)*x[0]+(-0.751568103399)*x[1]+(-1.20119343449)*x[2]
        s[5,0,0]=(0.853262644583)*x[0]+(1.47653986554)*x[1]+(-0.0642737845052)*x[2]
        s[5,0,1]=(1.1703484139)*x[0]+(-0.848977140408)*x[1]+(-0.303628307573)*x[2]
        s[5,1,0]=(-0.719950626922)*x[0]+(0.192452589234)*x[1]+(0.315963976897)*x[2]
        s[5,1,1]=(0.331604591856)*x[0]+(0.428822102766)*x[1]+(-1.35258251093)*x[2]
        ref[0,0,0]=(-0.452131788567)/(o+1.)+(-0.313943872628)+(0.320993573616)*0.5**o
        ref[0,0,1]=(-1.4121302884)/(o+1.)+(0.934332411096)+(0.707239806995)*0.5**o
        ref[0,1,0]=(-0.371701930519)/(o+1.)+(-1.22914563984)+(0.178663927586)*0.5**o
        ref[0,1,1]=(-0.569427715149)/(o+1.)+(-0.102129307368)+(-0.921844633589)*0.5**o
        ref[1,0,0]=(1.07041067382)/(o+1.)+(1.04773121415)+(-0.0969173076836)*0.5**o
        ref[1,0,1]=(0.344908055221)/(o+1.)+(0.253322966116)+(0.873941415143)*0.5**o
        ref[1,1,0]=(-0.568409920368)/(o+1.)+(-0.376074272994)+(0.694863446104)*0.5**o
        ref[1,1,1]=(-1.26296359103)/(o+1.)+(-0.285972966041)+(0.0791439379881)*0.5**o
        ref[2,0,0]=(0.261580662311)/(o+1.)+(-0.0693472624675)+(-0.878204709343)*0.5**o
        ref[2,0,1]=(-0.0659237231253)/(o+1.)+(-0.101871484642)+(-0.286732985248)*0.5**o
        ref[2,1,0]=(-0.609626668188)/(o+1.)+(0.438552975912)+(-0.874499580587)*0.5**o
        ref[2,1,1]=(1.21427389191)/(o+1.)+(-0.396304653588)+(-0.470125281731)*0.5**o
        ref[3,0,0]=(0.466268456837)/(o+1.)+(0.109894182607)+(0.993620646782)*0.5**o
        ref[3,0,1]=(0.798648502977)/(o+1.)+(0.30293631931)+(-0.728746135183)*0.5**o
        ref[3,1,0]=(-0.443525593788)/(o+1.)+(-0.32762855249)+(0.85052519129)*0.5**o
        ref[3,1,1]=(0.520520221786)/(o+1.)+(-0.21037503556)+(-0.0145439310481)*0.5**o
        ref[4,0,0]=(1.51637897594)/(o+1.)+(0.0251054216628)+(-0.685347145435)*0.5**o
        ref[4,0,1]=(-0.0126278181421)/(o+1.)+(-0.255524438913)+(-0.426679594213)*0.5**o
        ref[4,1,0]=(-0.110252848596)/(o+1.)+(-0.270402535847)+(0.95507791508)*0.5**o
        ref[4,1,1]=(-0.965558516877)/(o+1.)+(-0.777022669388)+(0.366881878633)*0.5**o
        ref[5,0,0]=(1.47489510124)/(o+1.)+(0.344964232978)+(0.100705158424)*0.5**o
        ref[5,0,1]=(0.272180101244)/(o+1.)+(-0.219694112974)+(0.184951090624)*0.5**o
        ref[5,1,0]=(0.868944944511)/(o+1.)+(-0.492651110402)+(-0.0951767844975)*0.5**o
        ref[5,1,1]=(-0.112001801469)/(o+1.)+(-0.0274812221923)+(-0.425191570457)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactZero_fromSymbol_ReducedContinuousFunction_rank4(self):
      """
      tests integral of rank 4 Symbol on the ReducedFunctionOnContactZero

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5, 3, 2))
      s=Data(0,(4, 5, 3, 2),w)
      ref=numpy.zeros((4, 5, 3, 2),numpy.float_)
      if dim==2:
        s[0,0,0,0]=(0.492266509376)*x[0]+(-0.374307474367)*x[1]
        s[0,0,0,1]=(0.379610803929)*x[0]+(1.26877290052)*x[1]
        s[0,0,1,0]=(0.489836401727)*x[0]+(0.0568437409848)*x[1]
        s[0,0,1,1]=(-0.0962394811856)*x[0]+(1.08297193296)*x[1]
        s[0,0,2,0]=(0.0083854747638)*x[0]+(1.12783902851)*x[1]
        s[0,0,2,1]=(-1.60969353709)*x[0]+(-1.15342435141)*x[1]
        s[0,1,0,0]=(0.718333685586)*x[0]+(-0.580409068311)*x[1]
        s[0,1,0,1]=(-0.471412983112)*x[0]+(-1.21523638921)*x[1]
        s[0,1,1,0]=(0.436952252413)*x[0]+(0.817968269393)*x[1]
        s[0,1,1,1]=(0.904303325207)*x[0]+(1.27944662005)*x[1]
        s[0,1,2,0]=(-0.646670575219)*x[0]+(-0.268471138891)*x[1]
        s[0,1,2,1]=(0.216944526796)*x[0]+(-1.44656748298)*x[1]
        s[0,2,0,0]=(-1.45710792844)*x[0]+(-1.37549168202)*x[1]
        s[0,2,0,1]=(-0.0209124548614)*x[0]+(-0.405359828227)*x[1]
        s[0,2,1,0]=(0.713696154808)*x[0]+(-0.283663569311)*x[1]
        s[0,2,1,1]=(0.0879024421072)*x[0]+(-1.74270037297)*x[1]
        s[0,2,2,0]=(-0.263019221744)*x[0]+(0.587637432145)*x[1]
        s[0,2,2,1]=(0.291949019915)*x[0]+(1.21942896685)*x[1]
        s[0,3,0,0]=(0.711503988663)*x[0]+(-0.0889987684621)*x[1]
        s[0,3,0,1]=(-0.473200478311)*x[0]+(-0.731689338365)*x[1]
        s[0,3,1,0]=(0.0347877428229)*x[0]+(0.436342776392)*x[1]
        s[0,3,1,1]=(1.33181675729)*x[0]+(0.796826790935)*x[1]
        s[0,3,2,0]=(0.975860401214)*x[0]+(-0.0388876329765)*x[1]
        s[0,3,2,1]=(0.336224987742)*x[0]+(1.40851963434)*x[1]
        s[0,4,0,0]=(-0.430668122172)*x[0]+(-0.210883679811)*x[1]
        s[0,4,0,1]=(-0.0947644917837)*x[0]+(-0.145045406149)*x[1]
        s[0,4,1,0]=(-0.975155718684)*x[0]+(0.370091078964)*x[1]
        s[0,4,1,1]=(1.59897077782)*x[0]+(-0.716757934974)*x[1]
        s[0,4,2,0]=(0.0995595315716)*x[0]+(-0.654667529963)*x[1]
        s[0,4,2,1]=(0.621429995817)*x[0]+(-0.830020781954)*x[1]
        s[1,0,0,0]=(0.460799425849)*x[0]+(1.37039134597)*x[1]
        s[1,0,0,1]=(0.484677718571)*x[0]+(-1.06817334128)*x[1]
        s[1,0,1,0]=(0.157323780788)*x[0]+(-0.816294918051)*x[1]
        s[1,0,1,1]=(0.275640307711)*x[0]+(-1.13585749962)*x[1]
        s[1,0,2,0]=(1.16452950894)*x[0]+(-1.17355094567)*x[1]
        s[1,0,2,1]=(-0.567786612292)*x[0]+(-0.78722481996)*x[1]
        s[1,1,0,0]=(-0.106381006132)*x[0]+(1.35734811758)*x[1]
        s[1,1,0,1]=(0.870362310422)*x[0]+(-0.701894980494)*x[1]
        s[1,1,1,0]=(0.463101808026)*x[0]+(-0.106531628533)*x[1]
        s[1,1,1,1]=(0.34927073998)*x[0]+(0.597282241285)*x[1]
        s[1,1,2,0]=(-1.08705596894)*x[0]+(-0.476312920631)*x[1]
        s[1,1,2,1]=(0.461228158094)*x[0]+(0.553430989166)*x[1]
        s[1,2,0,0]=(-0.565889081597)*x[0]+(0.847674515062)*x[1]
        s[1,2,0,1]=(1.22774040742)*x[0]+(0.532076088807)*x[1]
        s[1,2,1,0]=(-1.35738184718)*x[0]+(-0.010480885754)*x[1]
        s[1,2,1,1]=(0.0609383835609)*x[0]+(0.539183578894)*x[1]
        s[1,2,2,0]=(0.765013210057)*x[0]+(-0.0658319206215)*x[1]
        s[1,2,2,1]=(0.720241379555)*x[0]+(-1.29084845886)*x[1]
        s[1,3,0,0]=(1.37641602996)*x[0]+(-0.713391122816)*x[1]
        s[1,3,0,1]=(0.475350779553)*x[0]+(-0.0146303427199)*x[1]
        s[1,3,1,0]=(-0.498436122956)*x[0]+(0.446982991213)*x[1]
        s[1,3,1,1]=(-0.0198417666776)*x[0]+(-0.0330880581816)*x[1]
        s[1,3,2,0]=(-0.968203407981)*x[0]+(-1.43766997027)*x[1]
        s[1,3,2,1]=(-1.516108898)*x[0]+(0.154669696369)*x[1]
        s[1,4,0,0]=(0.107239334472)*x[0]+(-1.01814801515)*x[1]
        s[1,4,0,1]=(0.490733729058)*x[0]+(1.0617427681)*x[1]
        s[1,4,1,0]=(-0.579990484487)*x[0]+(-0.681146890855)*x[1]
        s[1,4,1,1]=(-0.539291982427)*x[0]+(1.40646560401)*x[1]
        s[1,4,2,0]=(-0.804686819612)*x[0]+(0.617047118706)*x[1]
        s[1,4,2,1]=(-0.913106387219)*x[0]+(-0.35453310971)*x[1]
        s[2,0,0,0]=(1.07945504868)*x[0]+(0.147695653661)*x[1]
        s[2,0,0,1]=(0.0321229519775)*x[0]+(-0.80726917422)*x[1]
        s[2,0,1,0]=(-0.276983245168)*x[0]+(-1.8208105745)*x[1]
        s[2,0,1,1]=(-0.390466636115)*x[0]+(-0.753787382303)*x[1]
        s[2,0,2,0]=(-0.0551239488574)*x[0]+(-0.691793238598)*x[1]
        s[2,0,2,1]=(0.455414076199)*x[0]+(1.19885245354)*x[1]
        s[2,1,0,0]=(-0.774122061304)*x[0]+(1.10934639101)*x[1]
        s[2,1,0,1]=(-1.60749547114)*x[0]+(0.305715741213)*x[1]
        s[2,1,1,0]=(0.0697234437299)*x[0]+(0.542820812428)*x[1]
        s[2,1,1,1]=(1.29397510328)*x[0]+(-0.804845682022)*x[1]
        s[2,1,2,0]=(0.478973101459)*x[0]+(1.07219480882)*x[1]
        s[2,1,2,1]=(-0.897309279985)*x[0]+(1.75363512874)*x[1]
        s[2,2,0,0]=(1.61737724977)*x[0]+(0.161191822716)*x[1]
        s[2,2,0,1]=(0.808798403137)*x[0]+(-0.00926345040032)*x[1]
        s[2,2,1,0]=(-0.903485178097)*x[0]+(-0.36599523945)*x[1]
        s[2,2,1,1]=(0.30686972857)*x[0]+(-1.62803935845)*x[1]
        s[2,2,2,0]=(1.48915760228)*x[0]+(0.579057819566)*x[1]
        s[2,2,2,1]=(-0.540786588929)*x[0]+(-0.402843340661)*x[1]
        s[2,3,0,0]=(0.964227135928)*x[0]+(0.0210110310059)*x[1]
        s[2,3,0,1]=(0.0266958261678)*x[0]+(0.830215139026)*x[1]
        s[2,3,1,0]=(0.930518946289)*x[0]+(0.66745827854)*x[1]
        s[2,3,1,1]=(0.0363390243751)*x[0]+(0.591657283448)*x[1]
        s[2,3,2,0]=(-0.433934955699)*x[0]+(-1.57946754208)*x[1]
        s[2,3,2,1]=(-0.890298505624)*x[0]+(0.623825002267)*x[1]
        s[2,4,0,0]=(0.838929180901)*x[0]+(0.699121739125)*x[1]
        s[2,4,0,1]=(1.3886192544)*x[0]+(-0.232981076843)*x[1]
        s[2,4,1,0]=(-0.566792750082)*x[0]+(-0.631000271451)*x[1]
        s[2,4,1,1]=(-0.989529672577)*x[0]+(0.0943919436553)*x[1]
        s[2,4,2,0]=(-1.85858526639)*x[0]+(0.257423736684)*x[1]
        s[2,4,2,1]=(-1.01848069496)*x[0]+(-1.08574972329)*x[1]
        s[3,0,0,0]=(0.135022656073)*x[0]+(-0.641428277939)*x[1]
        s[3,0,0,1]=(-0.613029646479)*x[0]+(0.158334902417)*x[1]
        s[3,0,1,0]=(-1.01838576339)*x[0]+(-0.788404867889)*x[1]
        s[3,0,1,1]=(0.133395871444)*x[0]+(-0.540018893528)*x[1]
        s[3,0,2,0]=(0.359599806799)*x[0]+(-0.163549980956)*x[1]
        s[3,0,2,1]=(0.537279834479)*x[0]+(1.43219657076)*x[1]
        s[3,1,0,0]=(0.699221501837)*x[0]+(0.58098090756)*x[1]
        s[3,1,0,1]=(-0.264653821282)*x[0]+(-0.297670973468)*x[1]
        s[3,1,1,0]=(-1.14805006965)*x[0]+(0.173713527619)*x[1]
        s[3,1,1,1]=(0.582167066862)*x[0]+(0.224704535757)*x[1]
        s[3,1,2,0]=(-0.013329661508)*x[0]+(0.439567816478)*x[1]
        s[3,1,2,1]=(-0.206657167829)*x[0]+(-0.900945403197)*x[1]
        s[3,2,0,0]=(-1.34123093943)*x[0]+(-0.905024065154)*x[1]
        s[3,2,0,1]=(-0.0677797983484)*x[0]+(0.863260908229)*x[1]
        s[3,2,1,0]=(0.0651406495709)*x[0]+(1.35122911589)*x[1]
        s[3,2,1,1]=(-0.00598531484168)*x[0]+(0.690487120132)*x[1]
        s[3,2,2,0]=(0.748701424535)*x[0]+(-1.0273152862)*x[1]
        s[3,2,2,1]=(-0.239064296585)*x[0]+(-0.213292951531)*x[1]
        s[3,3,0,0]=(-0.620272397992)*x[0]+(1.00106031112)*x[1]
        s[3,3,0,1]=(-0.803133841712)*x[0]+(0.873569080382)*x[1]
        s[3,3,1,0]=(-1.187617412)*x[0]+(0.422664418077)*x[1]
        s[3,3,1,1]=(-1.51637824714)*x[0]+(-1.5177161306)*x[1]
        s[3,3,2,0]=(1.41348390435)*x[0]+(1.2134182272)*x[1]
        s[3,3,2,1]=(0.99094246846)*x[0]+(0.34796551642)*x[1]
        s[3,4,0,0]=(0.580352675436)*x[0]+(-1.21782952097)*x[1]
        s[3,4,0,1]=(0.060030794007)*x[0]+(0.711473074059)*x[1]
        s[3,4,1,0]=(0.966745511848)*x[0]+(-0.550401614505)*x[1]
        s[3,4,1,1]=(-0.791329990237)*x[0]+(-0.205150506961)*x[1]
        s[3,4,2,0]=(0.482041855799)*x[0]+(-1.09392636319)*x[1]
        s[3,4,2,1]=(-0.698074703295)*x[0]+(0.510339675206)*x[1]
        ref[0,0,0,0]=(-0.953900694954)/(o+1.)+(0.442044844728)+(0.187770040507)*0.5**o
        ref[0,0,0,1]=(0.580869130103)/(o+1.)+(0.516362992147)+(0.03478859005)*0.5**o
        ref[0,0,1,0]=(-0.411645490601)/(o+1.)+(0.406943479905)+(0.144438673502)*0.5**o
        ref[0,0,1,1]=(0.204312991686)/(o+1.)+(0.383971959467)+(0.0144755411578)*0.5**o
        ref[0,0,2,0]=(0.743369450952)/(o+1.)+(0.0895404122051)+(0.213774227914)*0.5**o
        ref[0,0,2,1]=(-0.239858711457)/(o+1.)+(-0.809763839108)+(-0.903731498821)*0.5**o
        ref[0,1,0,0]=(-0.484413922256)/(o+1.)+(0.0508933433825)+(0.520551852766)*0.5**o
        ref[0,1,0,1]=(-0.590736935429)/(o+1.)+(-0.344843181585)+(-0.406226073725)*0.5**o
        ref[0,1,1,0]=(-0.125338697971)/(o+1.)+(0.369801286198)+(0.640656647382)*0.5**o
        ref[0,1,1,1]=(0.552504077477)/(o+1.)+(0.788979766736)+(0.0532863343083)*0.5**o
        ref[0,1,2,0]=(0.306194481363)/(o+1.)+(-0.445453307231)+(-0.33042958101)*0.5**o
        ref[0,1,2,1]=(-0.818898129415)/(o+1.)+(-0.0332482175532)+(-0.34422839166)*0.5**o
        ref[0,2,0,0]=(-0.878218012751)/(o+1.)+(-0.612244128212)+(-0.729893341289)*0.5**o
        ref[0,2,0,1]=(-0.105155854997)/(o+1.)+(0.281922249648)+(-0.884960927387)*0.5**o
        ref[0,2,1,0]=(0.565149481722)/(o+1.)+(0.0625165001878)+(-0.2601498966)*0.5**o
        ref[0,2,1,1]=(-0.983424704841)/(o+1.)+(0.0730690344226)+(-0.817511294863)*0.5**o
        ref[0,2,2,0]=(-0.258286255412)/(o+1.)+(0.308348049886)+(-0.0337916339596)*0.5**o
        ref[0,2,2,1]=(0.915004878119)/(o+1.)+(-0.166545843752)+(0.929464796149)*0.5**o
        ref[0,3,0,0]=(-0.433529018481)/(o+1.)+(0.627958762155)+(-0.199883285628)*0.5**o
        ref[0,3,0,1]=(-0.945419963966)/(o+1.)+(-0.336318858392)+(0.413167864075)*0.5**o
        ref[0,3,1,0]=(0.502119767083)/(o+1.)+(0.334993773063)+(-0.700976793994)*0.5**o
        ref[0,3,1,1]=(-0.186129206617)/(o+1.)+(0.891324041438)+(0.532124671966)*0.5**o
        ref[0,3,2,0]=(0.504051134603)/(o+1.)+(-0.200874672613)+(0.834670978862)*0.5**o
        ref[0,3,2,1]=(0.436498536789)/(o+1.)+(0.545640525497)+(0.216965034296)*0.5**o
        ref[0,4,0,0]=(-0.756349034292)/(o+1.)+(0.140738322652)+(-0.166679412995)*0.5**o
        ref[0,4,0,1]=(0.780437446749)/(o+1.)+(-0.599105577138)+(0.177963809594)*0.5**o
        ref[0,4,1,0]=(0.943566244729)/(o+1.)+(-0.441716458606)+(-0.665197967238)*0.5**o
        ref[0,4,1,1]=(-0.734101358227)/(o+1.)+(0.356355202125)+(0.903603796827)*0.5**o
        ref[0,4,2,0]=(-0.43430093723)/(o+1.)+(0.383496742595)+(-0.887800546351)*0.5**o
        ref[0,4,2,1]=(-0.227167732111)/(o+1.)+(-0.274598730532)+(0.567774407038)*0.5**o
        ref[1,0,0,0]=(0.391393429129)/(o+1.)+(0.353444577314)+(0.732908188056)*0.5**o
        ref[1,0,0,1]=(-0.171565730455)/(o+1.)+(-0.259205209862)+(0.106480527466)*0.5**o
        ref[1,0,1,0]=(-0.774601901321)/(o+1.)+(0.259483708314)+(-0.403336652569)*0.5**o
        ref[1,0,1,1]=(-0.726406904963)/(o+1.)+(-0.129278181762)+(0.124746076574)*0.5**o
        ref[1,0,2,0]=(-0.617577515377)/(o+1.)+(-0.152390726783)+(0.91333753221)*0.5**o
        ref[1,0,2,1]=(0.138202581628)/(o+1.)+(-0.396089249037)+(-0.701035515806)*0.5**o
        ref[1,1,0,0]=(0.729934336346)/(o+1.)+(-0.00268233232676)+(0.52639743976)*0.5**o
        ref[1,1,0,1]=(-0.158306695958)/(o+1.)+(-0.0162211686892)+(0.359216363265)*0.5**o
        ref[1,1,1,0]=(-0.978599129855)/(o+1.)+(0.71609215355)+(-0.0970149977525)*0.5**o
        ref[1,1,1,1]=(-0.349041967673)/(o+1.)+(0.780124912357)+(-0.264654875775)*0.5**o
        ref[1,1,2,0]=(-0.809194404709)/(o+1.)+(-0.268116096378)+(-0.217942292102)*0.5**o
        ref[1,1,2,1]=(0.455578593899)/(o+1.)+(0.428430852179)+(-0.297781150997)*0.5**o
        ref[1,2,0,0]=(0.116383337262)/(o+1.)+(0.228068268367)+(-0.290734440531)*0.5**o
        ref[1,2,0,1]=(0.977031820054)/(o+1.)+(0.137033678728)+(0.508717318715)*0.5**o
        ref[1,2,1,0]=(0.282937416713)/(o+1.)+(-0.615158071804)+(-0.420484006037)*0.5**o
        ref[1,2,1,1]=(0.904627062081)/(o+1.)+(0.018901292084)+(-0.342307683794)*0.5**o
        ref[1,2,2,0]=(0.755780696034)/(o+1.)+(-0.0362605587438)+(0.0159217108891)*0.5**o
        ref[1,2,2,1]=(-0.699866517547)/(o+1.)+(0.171062073116)+(-0.212864707989)*0.5**o
        ref[1,3,0,0]=(0.228867465112)/(o+1.)+(-0.0322172708178)+(0.498591983668)*0.5**o
        ref[1,3,0,1]=(-0.289542634768)/(o+1.)+(0.110436622855)+(0.529389825891)*0.5**o
        ref[1,3,1,0]=(0.358054041082)/(o+1.)+(0.165615637362)+(-0.74073844755)*0.5**o
        ref[1,3,1,1]=(-0.60359826352)/(o+1.)+(0.517588005735)+(-0.48450757281)*0.5**o
        ref[1,3,2,0]=(-0.545672858809)/(o+1.)+(-0.618462721543)+(-0.623275076358)*0.5**o
        ref[1,3,2,1]=(-0.73428128984)/(o+1.)+(-0.0497647291554)+(-0.527628453484)*0.5**o
        ref[1,4,0,0]=(-0.04306414004)/(o+1.)+(-0.298207200434)+(-0.271430139773)*0.5**o
        ref[1,4,0,1]=(0.971916388567)/(o+1.)+(-0.0691551012311)+(0.718870311054)*0.5**o
        ref[1,4,1,0]=(-0.445510592605)/(o+1.)+(-0.088724728164)+(-0.638177326409)*0.5**o
        ref[1,4,1,1]=(0.753412648385)/(o+1.)+(-0.0777097810015)+(0.269180535197)*0.5**o
        ref[1,4,2,0]=(-0.305234413894)/(o+1.)+(0.169600008291)+(-0.221605303593)*0.5**o
        ref[1,4,2,1]=(-0.286993085073)/(o+1.)+(-0.0493395569951)+(-0.881967297866)*0.5**o
        ref[2,0,0,0]=(-0.719751624491)/(o+1.)+(0.677226475278)+(0.59244937628)*0.5**o
        ref[2,0,0,1]=(-0.670829954915)/(o+1.)+(-0.139434936863)+(0.174553606398)*0.5**o
        ref[2,0,1,0]=(-0.916962595573)/(o+1.)+(-0.372659474123)+(-0.435512275853)*0.5**o
        ref[2,0,1,1]=(-0.893849803211)/(o+1.)+(-0.341833589876)+(0.433262964544)*0.5**o
        ref[2,0,2,0]=(-0.773257190107)/(o+1.)+(0.0495078796887)+(-0.0726757567263)*0.5**o
        ref[2,0,2,1]=(0.632070005323)/(o+1.)+(0.697198313452)+(-0.372200102483)*0.5**o
        ref[2,1,0,0]=(0.459329022417)/(o+1.)+(0.249194755087)+(-0.622494202881)*0.5**o
        ref[2,1,0,1]=(0.0269909119644)/(o+1.)+(-0.210708025265)+(-0.907354591365)*0.5**o
        ref[2,1,1,0]=(0.183761641828)/(o+1.)+(-0.207261190919)+(0.843304996169)*0.5**o
        ref[2,1,1,1]=(-0.49062931094)/(o+1.)+(0.2742786733)+(0.431201385598)*0.5**o
        ref[2,1,2,0]=(0.798332846753)/(o+1.)+(0.187074177535)+(0.378686708458)*0.5**o
        ref[2,1,2,1]=(0.942487410042)/(o+1.)+(0.0268513037692)+(-0.139864168826)*0.5**o
        ref[2,2,0,0]=(-0.306480663139)/(o+1.)+(0.706708669675)+(0.671632396277)*0.5**o
        ref[2,2,0,1]=(-0.68426521806)/(o+1.)+(0.658766824582)+(0.166266521632)*0.5**o
        ref[2,2,1,0]=(-0.0872551950734)/(o+1.)+(-0.212578991849)+(-0.757067238775)*0.5**o
        ref[2,2,1,1]=(-0.779515023363)/(o+1.)+(-0.165651803902)+(-0.210350998718)*0.5**o
        ref[2,2,2,0]=(0.420662034868)/(o+1.)+(0.456572874952)+(0.734407637073)*0.5**o
        ref[2,2,2,1]=(-0.0347308970891)/(o+1.)+(-0.550927478137)+(0.192955923773)*0.5**o
        ref[2,3,0,0]=(0.279479543192)/(o+1.)+(0.340981994922)+(0.0237946338994)*0.5**o
        ref[2,3,0,1]=(0.885111721906)/(o+1.)+(0.0386593998513)+(-0.105519556415)*0.5**o
        ref[2,3,1,0]=(-0.142745975965)/(o+1.)+(0.571228905184)+(0.598265390426)*0.5**o
        ref[2,3,1,1]=(0.676813945202)/(o+1.)+(0.301993597369)+(-0.652804832118)*0.5**o
        ref[2,3,2,0]=(-0.834072218834)/(o+1.)+(-0.836092513405)+(0.492854747868)*0.5**o
        ref[2,3,2,1]=(0.451195972296)/(o+1.)+(-0.330911551268)+(-0.0558463731179)*0.5**o
        ref[2,4,0,0]=(0.604480974758)/(o+1.)+(-0.0272834841321)+(0.988136913533)*0.5**o
        ref[2,4,0,1]=(-0.672293493189)/(o+1.)+(0.436472725256)+(0.954986220232)*0.5**o
        ref[2,4,1,0]=(-0.945983571254)/(o+1.)+(0.362615161801)+(-0.97703977388)*0.5**o
        ref[2,4,1,1]=(-0.778028939318)/(o+1.)+(0.423610287746)+(-0.964329365094)*0.5**o
        ref[2,4,2,0]=(-0.138477487464)/(o+1.)+(-0.290097108281)+(-0.882489825678)*0.5**o
        ref[2,4,2,1]=(-0.953853492601)/(o+1.)+(-0.179508678068)+(-0.791359569512)*0.5**o
        ref[3,0,0,0]=(-0.199022712971)/(o+1.)+(0.253562515321)+(-0.814507939536)*0.5**o
        ref[3,0,0,1]=(0.112777444671)/(o+1.)+(-0.304438182199)+(0.0414041756638)*0.5**o
        ref[3,0,1,0]=(-0.880947344135)/(o+1.)+(-0.194192553178)+(-0.537458180783)*0.5**o
        ref[3,0,1,1]=(0.39314163058)/(o+1.)+(-0.775542968704)+(0.751321284745)*0.5**o
        ref[3,0,2,0]=(-0.482557545787)/(o+1.)+(-0.0683141681573)+(0.815235707945)*0.5**o
        ref[3,0,2,1]=(0.689498951848)/(o+1.)+(0.278187448835)+(0.723602555722)*0.5**o
        ref[3,1,0,0]=(0.293238350118)/(o+1.)+(0.126767755198)+(0.733428548881)*0.5**o
        ref[3,1,0,1]=(-0.26577191789)/(o+1.)+(0.201156903364)+(-0.698866683588)*0.5**o
        ref[3,1,1,0]=(0.439591690299)/(o+1.)+(-0.288902429619)+(-0.836123373092)*0.5**o
        ref[3,1,1,1]=(0.8910765985)/(o+1.)+(0.0553222730519)+(-0.194849541985)*0.5**o
        ref[3,1,2,0]=(-0.0320019026134)/(o+1.)+(0.477772657371)+(-0.497305257159)*0.5**o
        ref[3,1,2,1]=(-0.0499399267727)/(o+1.)+(-0.47037968496)+(-0.116903274335)*0.5**o
        ref[3,2,0,0]=(-0.276422491301)/(o+1.)+(-0.812540687574)+(-0.344751138134)*0.5**o
        ref[3,2,0,1]=(-0.00141392846802)/(o+1.)+(0.102797761295)+(0.591299515759)*0.5**o
        ref[3,2,1,0]=(0.630823139203)/(o+1.)+(0.743132641412)+(-0.700718656569)*0.5**o
        ref[3,2,1,1]=(0.490401876791)/(o+1.)+(-0.0461798489229)+(0.286459626346)*0.5**o
        ref[3,2,2,0]=(-0.787385728473)/(o+1.)+(0.0658852261331)+(0.377001414541)*0.5**o
        ref[3,2,2,1]=(-0.158857143023)/(o+1.)+(-0.421186525234)+(0.548872945376)*0.5**o
        ref[3,3,0,0]=(0.029229133788)/(o+1.)+(0.069575027013)+(0.212408725319)*0.5**o
        ref[3,3,0,1]=(0.757333397205)/(o+1.)+(0.029368913907)+(-0.74563598635)*0.5**o
        ref[3,3,1,0]=(0.0943476349516)/(o+1.)+(0.0337288993927)+(-0.926758427661)*0.5**o
        ref[3,3,1,1]=(-0.769051069188)/(o+1.)+(-0.844682211294)+(-0.575678885962)*0.5**o
        ref[3,3,2,0]=(0.246931939289)/(o+1.)+(0.756380254609)+(0.867209683046)*0.5**o
        ref[3,3,2,1]=(0.535429280541)/(o+1.)+(-0.0329131735419)+(0.869305051424)*0.5**o
        ref[3,4,0,0]=(-0.405156907998)/(o+1.)+(-0.244492906569)+(0.256665875603)*0.5**o
        ref[3,4,0,1]=(-0.140892979474)/(o+1.)+(0.516304250401)+(-0.120211653262)*0.5**o
        ref[3,4,1,0]=(0.168500977091)/(o+1.)+(0.089683045067)+(0.0684768301186)*0.5**o
        ref[3,4,1,1]=(0.479212374255)/(o+1.)+(-0.626588026593)+(-0.222516818267)*0.5**o
        ref[3,4,2,0]=(-0.563625996633)/(o+1.)+(-0.120136908079)+(0.192015305399)*0.5**o
        ref[3,4,2,1]=(-0.031361929046)/(o+1.)+(0.332029567053)+(-0.820432233148)*0.5**o
      else:
        s[0,0,0,0]=(0.186907875567)*x[0]+(0.323825050263)*x[1]+(1.15648314531)*x[2]
        s[0,0,0,1]=(1.12367405158)*x[0]+(1.13427439957)*x[1]+(0.189106618663)*x[2]
        s[0,0,1,0]=(1.76772912309)*x[0]+(0.0616670242122)*x[1]+(-1.23257311969)*x[2]
        s[0,0,1,1]=(0.0497251821243)*x[0]+(-1.47406701801)*x[1]+(0.164922402015)*x[2]
        s[0,0,2,0]=(-1.40783401802)*x[0]+(0.684901383857)*x[1]+(0.283077416398)*x[2]
        s[0,0,2,1]=(-0.283194016281)*x[0]+(-1.60164790888)*x[1]+(0.412908024034)*x[2]
        s[0,1,0,0]=(0.957812695611)*x[0]+(-0.863209638895)*x[1]+(-0.609946161661)*x[2]
        s[0,1,0,1]=(-0.410080093222)*x[0]+(0.703061988282)*x[1]+(0.0546952979936)*x[2]
        s[0,1,1,0]=(-1.43265292137)*x[0]+(-1.28132882624)*x[1]+(-0.103305431528)*x[2]
        s[0,1,1,1]=(-0.973042964661)*x[0]+(0.118946651326)*x[1]+(0.0121414797284)*x[2]
        s[0,1,2,0]=(-1.31257233483)*x[0]+(-1.90572394465)*x[1]+(-0.426472675848)*x[2]
        s[0,1,2,1]=(1.25719983663)*x[0]+(-0.68807451729)*x[1]+(0.329745219822)*x[2]
        s[0,2,0,0]=(-0.317555693596)*x[0]+(0.464722269829)*x[1]+(-0.0253415732374)*x[2]
        s[0,2,0,1]=(-0.802071257989)*x[0]+(-1.07682010382)*x[1]+(-1.52372537155)*x[2]
        s[0,2,1,0]=(-0.697501763365)*x[0]+(-1.5918814652)*x[1]+(-1.03755334358)*x[2]
        s[0,2,1,1]=(0.292457959822)*x[0]+(-0.446371848663)*x[1]+(1.32282628125)*x[2]
        s[0,2,2,0]=(1.44590220139)*x[0]+(-0.556542973367)*x[1]+(0.999665797669)*x[2]
        s[0,2,2,1]=(-0.376272847083)*x[0]+(-0.0251287305789)*x[1]+(0.364915231203)*x[2]
        s[0,3,0,0]=(0.88046028761)*x[0]+(0.372647019255)*x[1]+(-1.52356563515)*x[2]
        s[0,3,0,1]=(0.0765111785452)*x[0]+(-0.947059538185)*x[1]+(0.332098361392)*x[2]
        s[0,3,1,0]=(-1.37435064568)*x[0]+(1.04159418667)*x[1]+(0.314113199294)*x[2]
        s[0,3,1,1]=(1.03796425672)*x[0]+(-0.139508518244)*x[1]+(-0.46214547857)*x[2]
        s[0,3,2,0]=(0.285674556261)*x[0]+(-0.915609582509)*x[1]+(0.683131361973)*x[2]
        s[0,3,2,1]=(0.703275850512)*x[0]+(-0.894071596437)*x[1]+(0.118084971572)*x[2]
        s[0,4,0,0]=(-1.29507182493)*x[0]+(-0.275563564467)*x[1]+(-1.29101960297)*x[2]
        s[0,4,0,1]=(0.726127419233)*x[0]+(-0.163731035409)*x[1]+(1.45773190781)*x[2]
        s[0,4,1,0]=(-0.321962778987)*x[0]+(-0.434741013545)*x[1]+(0.348257210746)*x[2]
        s[0,4,1,1]=(0.892154653814)*x[0]+(1.12188965959)*x[1]+(-1.42589350879)*x[2]
        s[0,4,2,0]=(-1.06019594336)*x[0]+(-1.5907947093)*x[1]+(-1.64125703099)*x[2]
        s[0,4,2,1]=(0.820572048651)*x[0]+(-0.528087471135)*x[1]+(-0.0823830365488)*x[2]
        s[1,0,0,0]=(0.823455925698)*x[0]+(0.456056899485)*x[1]+(0.708907347857)*x[2]
        s[1,0,0,1]=(0.013059728892)*x[0]+(-0.772283592797)*x[1]+(0.363920439078)*x[2]
        s[1,0,1,0]=(-0.569161155945)*x[0]+(-1.60837471571)*x[1]+(-1.12748440619)*x[2]
        s[1,0,1,1]=(-1.06458634572)*x[0]+(-1.10787449393)*x[1]+(1.43157821547)*x[2]
        s[1,0,2,0]=(1.42955732758)*x[0]+(-0.769967391917)*x[1]+(0.461386087764)*x[2]
        s[1,0,2,1]=(-1.12922605294)*x[0]+(-0.256069161837)*x[1]+(0.934117328474)*x[2]
        s[1,1,0,0]=(-0.296512978109)*x[0]+(0.0247034910192)*x[1]+(-0.0918640286973)*x[2]
        s[1,1,0,1]=(0.262151201281)*x[0]+(-0.556624983637)*x[1]+(-0.82641291041)*x[2]
        s[1,1,1,0]=(-0.909049007906)*x[0]+(0.578952420411)*x[1]+(0.0963761548489)*x[2]
        s[1,1,1,1]=(0.789884036316)*x[0]+(0.714044849036)*x[1]+(-0.348148665956)*x[2]
        s[1,1,2,0]=(-0.306802924795)*x[0]+(0.166007262622)*x[1]+(0.812933157357)*x[2]
        s[1,1,2,1]=(-0.23666871482)*x[0]+(0.0180126754613)*x[1]+(0.465074288631)*x[2]
        s[1,2,0,0]=(-1.14910199315)*x[0]+(0.930765031574)*x[1]+(1.63176081675)*x[2]
        s[1,2,0,1]=(-0.220753578398)*x[0]+(0.153024000962)*x[1]+(0.29171013278)*x[2]
        s[1,2,1,0]=(-0.571208521033)*x[0]+(1.18218013588)*x[1]+(-0.0281527591651)*x[2]
        s[1,2,1,1]=(-0.0212229342253)*x[0]+(0.577228434778)*x[1]+(-0.0934776488559)*x[2]
        s[1,2,2,0]=(-0.592424182282)*x[0]+(-0.0403519194823)*x[1]+(0.150970946422)*x[2]
        s[1,2,2,1]=(-1.33143395635)*x[0]+(-1.3121470647)*x[1]+(-0.78895334942)*x[2]
        s[1,3,0,0]=(0.626711601793)*x[0]+(0.0187262812242)*x[1]+(-0.392743911033)*x[2]
        s[1,3,0,1]=(0.0360431567199)*x[0]+(-0.921408398245)*x[1]+(-0.172097608208)*x[2]
        s[1,3,1,0]=(0.0223088597951)*x[0]+(0.139631235358)*x[1]+(-0.529170239405)*x[2]
        s[1,3,1,1]=(0.11834184642)*x[0]+(-0.754799078426)*x[1]+(0.136555856977)*x[2]
        s[1,3,2,0]=(-0.288966482257)*x[0]+(-0.227333376328)*x[1]+(-0.583704417546)*x[2]
        s[1,3,2,1]=(0.263525293937)*x[0]+(0.842984179305)*x[1]+(0.592096192935)*x[2]
        s[1,4,0,0]=(-0.0145931697196)*x[0]+(-0.839359041248)*x[1]+(-0.635230965652)*x[2]
        s[1,4,0,1]=(-0.452080234732)*x[0]+(-0.500376989587)*x[1]+(-0.27541187994)*x[2]
        s[1,4,1,0]=(-0.028441856276)*x[0]+(0.99292041207)*x[1]+(-0.555205406529)*x[2]
        s[1,4,1,1]=(0.670664493076)*x[0]+(0.562911949702)*x[1]+(-0.767721105918)*x[2]
        s[1,4,2,0]=(0.960781399312)*x[0]+(0.0966843021482)*x[1]+(-0.448822164554)*x[2]
        s[1,4,2,1]=(1.19002654689)*x[0]+(0.0423440255032)*x[1]+(-0.566501161163)*x[2]
        s[2,0,0,0]=(-1.21404047236)*x[0]+(0.246576983095)*x[1]+(-1.35965095619)*x[2]
        s[2,0,0,1]=(-0.183423338333)*x[0]+(0.490089064451)*x[1]+(-0.578583461456)*x[2]
        s[2,0,1,0]=(1.12402576003)*x[0]+(-0.175494364486)*x[1]+(-0.303488944097)*x[2]
        s[2,0,1,1]=(-1.65572015787)*x[0]+(-1.0020836407)*x[1]+(0.480903327702)*x[2]
        s[2,0,2,0]=(0.145764271661)*x[0]+(-0.263090439788)*x[1]+(-1.17377189982)*x[2]
        s[2,0,2,1]=(-0.837574918907)*x[0]+(1.48561229389)*x[1]+(0.967914367565)*x[2]
        s[2,1,0,0]=(0.509756439273)*x[0]+(-0.111493103227)*x[1]+(0.748245818033)*x[2]
        s[2,1,0,1]=(0.131007923179)*x[0]+(-0.128179547101)*x[1]+(-0.755140067557)*x[2]
        s[2,1,1,0]=(-0.635852982675)*x[0]+(-0.475693144737)*x[1]+(-0.331519420739)*x[2]
        s[2,1,1,1]=(-1.29780426286)*x[0]+(0.774138127446)*x[1]+(-1.74756722025)*x[2]
        s[2,1,2,0]=(-0.14015158014)*x[0]+(-0.875354388859)*x[1]+(0.791253726662)*x[2]
        s[2,1,2,1]=(-0.0772968366884)*x[0]+(-0.937984863796)*x[1]+(-0.0602294365875)*x[2]
        s[2,2,0,0]=(0.955209462705)*x[0]+(0.849659517825)*x[1]+(0.0983431115722)*x[2]
        s[2,2,0,1]=(1.72389105142)*x[0]+(-0.412358364777)*x[1]+(-1.3083297441)*x[2]
        s[2,2,1,0]=(-0.895927963754)*x[0]+(1.65404011945)*x[1]+(0.297503560372)*x[2]
        s[2,2,1,1]=(1.34506983158)*x[0]+(-0.552690724942)*x[1]+(0.0882447776246)*x[2]
        s[2,2,2,0]=(-0.603470575964)*x[0]+(1.77788342742)*x[1]+(-0.960079624369)*x[2]
        s[2,2,2,1]=(-0.547711891427)*x[0]+(-0.562393805421)*x[1]+(-0.183240255616)*x[2]
        s[2,3,0,0]=(1.01018738642)*x[0]+(1.4085839719)*x[1]+(-0.619343450789)*x[2]
        s[2,3,0,1]=(0.525036070827)*x[0]+(-0.760210492125)*x[1]+(0.621245864477)*x[2]
        s[2,3,1,0]=(-0.786180468736)*x[0]+(0.784642311709)*x[1]+(-0.377020976891)*x[2]
        s[2,3,1,1]=(-0.333203982307)*x[0]+(-0.044265604048)*x[1]+(-1.6196925017)*x[2]
        s[2,3,2,0]=(-0.885601870306)*x[0]+(0.650248271309)*x[1]+(-1.7206378066)*x[2]
        s[2,3,2,1]=(0.0941920781315)*x[0]+(-1.24650013731)*x[1]+(0.67065624977)*x[2]
        s[2,4,0,0]=(0.0693008855416)*x[0]+(-1.33764045153)*x[1]+(0.277137873936)*x[2]
        s[2,4,0,1]=(0.887968960137)*x[0]+(1.57875979002)*x[1]+(-0.103648235191)*x[2]
        s[2,4,1,0]=(-0.547070384787)*x[0]+(-0.376770947917)*x[1]+(-1.81372790514)*x[2]
        s[2,4,1,1]=(0.375805210805)*x[0]+(0.660242065798)*x[1]+(1.69413225283)*x[2]
        s[2,4,2,0]=(-0.914715181893)*x[0]+(1.27301677765)*x[1]+(0.974031227808)*x[2]
        s[2,4,2,1]=(-0.503816280288)*x[0]+(0.552217633496)*x[1]+(-0.90127280688)*x[2]
        s[3,0,0,0]=(0.768275498738)*x[0]+(-0.811563543342)*x[1]+(-0.69998185891)*x[2]
        s[3,0,0,1]=(-0.629451202094)*x[0]+(1.02346787018)*x[1]+(0.780161524988)*x[2]
        s[3,0,1,0]=(0.584052565724)*x[0]+(-0.069634462512)*x[1]+(-1.00341091459)*x[2]
        s[3,0,1,1]=(1.03073176869)*x[0]+(1.53263439484)*x[1]+(1.03753255807)*x[2]
        s[3,0,2,0]=(0.0522744193577)*x[0]+(0.308512837882)*x[1]+(-0.315970675533)*x[2]
        s[3,0,2,1]=(0.0747430001964)*x[0]+(0.863711676177)*x[1]+(1.07705766445)*x[2]
        s[3,1,0,0]=(0.114255376522)*x[0]+(1.47818231679)*x[1]+(0.0222267873076)*x[2]
        s[3,1,0,1]=(0.613783843499)*x[0]+(0.369289287936)*x[1]+(1.08226313005)*x[2]
        s[3,1,1,0]=(0.474121107413)*x[0]+(-0.284169623677)*x[1]+(1.26739419136)*x[2]
        s[3,1,1,1]=(0.703235040854)*x[0]+(0.340511678132)*x[1]+(0.314037803901)*x[2]
        s[3,1,2,0]=(-0.754512705698)*x[0]+(0.957288483514)*x[1]+(1.38853172146)*x[2]
        s[3,1,2,1]=(0.338933480448)*x[0]+(1.30797425088)*x[1]+(-0.492819385452)*x[2]
        s[3,2,0,0]=(0.477902330587)*x[0]+(0.658604196318)*x[1]+(-0.669039618554)*x[2]
        s[3,2,0,1]=(0.584833130533)*x[0]+(0.205626741315)*x[1]+(-0.449267859148)*x[2]
        s[3,2,1,0]=(0.392564971475)*x[0]+(-0.867191184183)*x[1]+(0.508221616556)*x[2]
        s[3,2,1,1]=(0.199023224346)*x[0]+(0.210018232234)*x[1]+(0.17332311179)*x[2]
        s[3,2,2,0]=(-0.530182002398)*x[0]+(-0.70410123336)*x[1]+(0.454418407328)*x[2]
        s[3,2,2,1]=(-1.00165228123)*x[0]+(0.253353462588)*x[1]+(1.07981964164)*x[2]
        s[3,3,0,0]=(-0.00388223127856)*x[0]+(-0.256156832173)*x[1]+(-1.22618259947)*x[2]
        s[3,3,0,1]=(1.04947612909)*x[0]+(-0.590938924785)*x[1]+(-0.80453902875)*x[2]
        s[3,3,1,0]=(0.395025775135)*x[0]+(-1.30382091375)*x[1]+(1.00251618393)*x[2]
        s[3,3,1,1]=(-0.601315067206)*x[0]+(-0.226958312276)*x[1]+(-0.69720544326)*x[2]
        s[3,3,2,0]=(0.748907154335)*x[0]+(-0.694932273918)*x[1]+(0.143655252827)*x[2]
        s[3,3,2,1]=(-0.228346281761)*x[0]+(0.0818793892413)*x[1]+(0.376376078012)*x[2]
        s[3,4,0,0]=(0.675502208069)*x[0]+(0.498131679631)*x[1]+(-0.606527858224)*x[2]
        s[3,4,0,1]=(0.230994211989)*x[0]+(-1.81768645462)*x[1]+(1.59488392513)*x[2]
        s[3,4,1,0]=(-1.35866095694)*x[0]+(-0.122601842831)*x[1]+(1.73437014104)*x[2]
        s[3,4,1,1]=(0.63210638436)*x[0]+(1.50387972886)*x[1]+(0.655743288533)*x[2]
        s[3,4,2,0]=(1.42156357373)*x[0]+(-0.701544596166)*x[1]+(0.88931540397)*x[2]
        s[3,4,2,1]=(-0.405515262733)*x[0]+(0.872922189212)*x[1]+(-1.13847750709)*x[2]
        ref[0,0,0,0]=(0.415231330279)/(o+1.)+(0.982601202296)+(-0.713217663735)*0.5**o
        ref[0,0,0,1]=(0.650505784971)/(o+1.)+(0.406482330235)+(0.983584624374)*0.5**o
        ref[0,0,1,0]=(-0.615926989595)/(o+1.)+(0.113540305186)+(0.98566940684)*0.5**o
        ref[0,0,1,1]=(-1.36210768774)/(o+1.)+(0.330430698616)+(-0.558173143358)*0.5**o
        ref[0,0,2,0]=(0.589319954581)/(o+1.)+(-0.146725890489)+(-0.735723391368)*0.5**o
        ref[0,0,2,1]=(-0.819641973816)/(o+1.)+(0.0696728799133)+(-0.791637687141)*0.5**o
        ref[0,1,0,0]=(-0.984505810214)/(o+1.)+(0.0623424750886)+(0.344477755092)*0.5**o
        ref[0,1,0,1]=(0.0372238318297)/(o+1.)+(0.201167151246)+(-0.0918809412683)*0.5**o
        ref[0,1,1,0]=(-0.988162693644)/(o+1.)+(-0.658809560166)+(-0.51150536516)*0.5**o
        ref[0,1,1,1]=(0.191409590473)/(o+1.)+(-0.426836466412)+(-0.179691491256)*0.5**o
        ref[0,1,2,0]=(-1.50151479209)/(o+1.)+(-0.648874443157)+(-0.84550527693)*0.5**o
        ref[0,1,2,1]=(-1.3172677321)/(o+1.)+(0.885092333538)+(0.445953604186)*0.5**o
        ref[0,2,0,0]=(0.1269682286)/(o+1.)+(-0.175772039094)+(0.346400852583)*0.5**o
        ref[0,2,0,1]=(-1.46726533796)/(o+1.)+(-1.05015066374)+(0.164949932075)*0.5**o
        ref[0,2,1,0]=(-1.23701559967)/(o+1.)+(-0.825973196415)+(-0.437974579642)*0.5**o
        ref[0,2,1,1]=(0.132084480709)/(o+1.)+(0.0364121752536)+(0.964003561191)*0.5**o
        ref[0,2,2,0]=(-0.0575514788048)/(o+1.)+(0.574131321284)+(0.798313861933)*0.5**o
        ref[0,2,2,1]=(-0.854422215721)/(o+1.)+(0.454172855359)+(-0.0904098414555)*0.5**o
        ref[0,3,0,0]=(-1.04634289994)/(o+1.)+(0.354533324899)+(0.0668179218583)*0.5**o
        ref[0,3,0,1]=(-0.415702586981)/(o+1.)+(0.126519101056)+(-0.375785613379)*0.5**o
        ref[0,3,1,0]=(-0.268120824236)/(o+1.)+(0.500467336081)+(-0.751457107646)*0.5**o
        ref[0,3,1,1]=(0.827791290739)/(o+1.)+(-0.347418978526)+(0.303356926219)*0.5**o
        ref[0,3,2,0]=(-0.1387759961)/(o+1.)+(-0.350616137823)+(0.893204607472)*0.5**o
        ref[0,3,2,1]=(0.129310667468)/(o+1.)+(-0.252264005651)+(0.302506569481)*0.5**o
        ref[0,4,0,0]=(-0.021050677621)/(o+1.)+(-0.980639199085)+(-0.879325916576)*0.5**o
        ref[0,4,0,1]=(-0.262715302419)/(o+1.)+(1.20073556064)+(-0.118627527238)*0.5**o
        ref[0,4,1,0]=(0.261035782917)/(o+1.)+(-0.449979981215)+(0.230477597727)*0.5**o
        ref[0,4,1,1]=(-0.47960276307)/(o+1.)+(0.219069456491)+(0.629614654702)*0.5**o
        ref[0,4,2,0]=(-1.27925873031)/(o+1.)+(-1.32602054802)+(-0.360947857308)*0.5**o
        ref[0,4,2,1]=(-1.34486930774)/(o+1.)+(0.546553886393)+(0.46186307592)*0.5**o
        ref[1,0,0,0]=(0.430196940607)/(o+1.)+(0.311302638938)+(0.935617954556)*0.5**o
        ref[1,0,0,1]=(0.0865234935036)/(o+1.)+(-0.268442519337)+(0.055058120343)*0.5**o
        ref[1,0,1,0]=(-1.48387607046)/(o+1.)+(-0.786378289878)+(-0.248387627621)*0.5**o
        ref[1,0,1,1]=(0.758534313728)/(o+1.)+(-0.265213280892)+(-0.968990376127)*0.5**o
        ref[1,0,2,0]=(-0.713142570031)/(o+1.)+(0.462618013893)+(0.908882565677)*0.5**o
        ref[1,0,2,1]=(-0.482219090892)/(o+1.)+(0.307689127346)+(-0.584337050105)*0.5**o
        ref[1,1,0,0]=(-1.17250881543)/(o+1.)+(0.113442845094)+(0.581949609453)*0.5**o
        ref[1,1,0,1]=(-1.3468375595)/(o+1.)+(0.395669021834)+(-0.565387176936)*0.5**o
        ref[1,1,1,0]=(0.2018351732)/(o+1.)+(-0.157830690125)+(-0.119894225596)*0.5**o
        ref[1,1,1,1]=(1.21543817764)/(o+1.)+(-0.0497756593583)+(0.0398933604741)*0.5**o
        ref[1,1,2,0]=(-0.162294483944)/(o+1.)+(0.85994581496)+(-0.885459650791)*0.5**o
        ref[1,1,2,1]=(1.24422084677)/(o+1.)+(-0.379646444866)+(-0.238509707766)*0.5**o
        ref[1,2,0,0]=(1.5435438681)/(o+1.)+(0.08540126524)+(-0.300922543405)*0.5**o
        ref[1,2,0,1]=(-0.112210840363)/(o+1.)+(0.480051249506)+(-0.623911103306)*0.5**o
        ref[1,2,1,0]=(0.285721711645)/(o+1.)+(0.0100191655774)+(0.27705881288)*0.5**o
        ref[1,2,1,1]=(0.66930209958)/(o+1.)+(-0.542529578578)+(0.878284909271)*0.5**o
        ref[1,2,2,0]=(1.12100699995)/(o+1.)+(-0.678457673316)+(-0.245896808662)*0.5**o
        ref[1,2,2,1]=(-0.88612973059)/(o+1.)+(-1.00088764314)+(-0.544629353594)*0.5**o
        ref[1,3,0,0]=(0.387350166894)/(o+1.)+(-0.499575950447)+(0.864495705983)*0.5**o
        ref[1,3,0,1]=(-0.942393535847)/(o+1.)+(0.0303050564575)+(-0.175679426801)*0.5**o
        ref[1,3,1,0]=(-0.798050539357)/(o+1.)+(0.126548837387)+(0.177722720331)*0.5**o
        ref[1,3,1,1]=(-1.50957773465)/(o+1.)+(0.172938183698)+(0.663799992229)*0.5**o
        ref[1,3,2,0]=(0.469444503978)/(o+1.)+(-0.647836043317)+(-0.273776693474)*0.5**o
        ref[1,3,2,1]=(0.310061780349)/(o+1.)+(1.05452404891)+(-0.720504211989)*0.5**o
        ref[1,4,0,0]=(-0.820548632752)/(o+1.)+(0.105943005381)+(-0.88052055463)*0.5**o
        ref[1,4,0,1]=(-0.158467564266)/(o+1.)+(-0.709811754591)+(0.35022196919)*0.5**o
        ref[1,4,1,0]=(0.00990374166591)/(o+1.)+(0.51839921597)+(-0.637429024341)*0.5**o
        ref[1,4,1,1]=(-0.246106620377)/(o+1.)+(0.33455634807)+(0.0428492610972)*0.5**o
        ref[1,4,2,0]=(0.472702988578)/(o+1.)+(0.0688149777117)+(-0.00168940709497)*0.5**o
        ref[1,4,2,1]=(0.284541065049)/(o+1.)+(-0.0548349320689)+(0.490998210318)*0.5**o
        ref[2,0,0,0]=(-0.733338280715)/(o+1.)+(-0.47189177308)+(-0.649992618571)*0.5**o
        ref[2,0,0,1]=(-0.948815264173)/(o+1.)+(0.837962467604)+(-0.999027406375)*0.5**o
        ref[2,0,1,0]=(-0.0944539750337)/(o+1.)+(0.0746201219713)+(0.590256182541)*0.5**o
        ref[2,0,1,1]=(0.044594970161)/(o+1.)+(-0.717673746264)+(-0.786147948501)*0.5**o
        ref[2,0,2,0]=(-0.387429911457)/(o+1.)+(-0.0765750118601)+(-0.750518132769)*0.5**o
        ref[2,0,2,1]=(1.28025192232)/(o+1.)+(0.425010917096)+(-0.514322013965)*0.5**o
        ref[2,1,0,0]=(0.85161480185)/(o+1.)+(-0.117153136076)+(0.52920062438)*0.5**o
        ref[2,1,0,1]=(-0.558914411305)/(o+1.)+(-0.229298513746)+(0.265199747318)*0.5**o
        ref[2,1,1,0]=(-0.33769433116)/(o+1.)+(-0.678382612728)+(0.251394008465)*0.5**o
        ref[2,1,1,1]=(-0.990803230186)/(o+1.)+(-0.413400680719)+(-0.453628764049)*0.5**o
        ref[2,1,2,0]=(-0.377834292093)/(o+1.)+(0.521893888533)+(-0.890205727309)*0.5**o
        ref[2,1,2,1]=(-1.33150035489)/(o+1.)+(-0.0586549531766)+(0.373299124171)*0.5**o
        ref[2,2,0,0]=(-0.166255483746)/(o+1.)+(0.816713521389)+(0.436040533071)*0.5**o
        ref[2,2,0,1]=(-1.39682071748)/(o+1.)+(0.21770833884)+(0.964606982344)*0.5**o
        ref[2,2,1,0]=(1.3247136685)/(o+1.)+(-0.0466914213895)+(-0.175715109647)*0.5**o
        ref[2,2,1,1]=(-0.522454725229)/(o+1.)+(0.291262422208)+(0.820553765077)*0.5**o
        ref[2,2,2,0]=(0.136169385992)/(o+1.)+(0.253426637378)+(-0.42868943366)*0.5**o
        ref[2,2,2,1]=(-0.131390793028)/(o+1.)+(-0.715512276073)+(0.269069392711)*0.5**o
        ref[2,3,0,0]=(0.1679740824)/(o+1.)+(0.653952560954)+(0.323548703221)*0.5**o
        ref[2,3,0,1]=(-0.0511907658967)/(o+1.)+(-0.236699539579)+(0.910661288233)*0.5**o
        ref[2,3,1,0]=(0.451520708483)/(o+1.)+(-0.065786465884)+(-0.698506910633)*0.5**o
        ref[2,3,1,1]=(-0.542162878453)/(o+1.)+(-0.237944135968)+(-0.979110937669)*0.5**o
        ref[2,3,2,0]=(-0.489308922223)/(o+1.)+(-0.563117809253)+(-0.34044686487)*0.5**o
        ref[2,3,2,1]=(0.382652422829)/(o+1.)+(-0.328188452455)+(-0.207927327325)*0.5**o
        ref[2,4,0,0]=(0.0347966753187)/(o+1.)+(-0.629864354262)+(0.233730341151)*0.5**o
        ref[2,4,0,1]=(0.123939573502)/(o+1.)+(1.03666652808)+(0.165807885309)*0.5**o
        ref[2,4,1,0]=(-0.829855747878)/(o+1.)+(-0.507284312041)+(-0.89314486588)*0.5**o
        ref[2,4,1,1]=(1.80556499882)/(o+1.)+(0.554771100606)+(-0.1849276706)*0.5**o
        ref[2,4,2,0]=(0.829439412646)/(o+1.)+(0.342620313927)+(-0.182347216931)*0.5**o
        ref[2,4,2,1]=(-0.824674529166)/(o+1.)+(-0.172483000807)+(0.316769077106)*0.5**o
        ref[3,0,0,0]=(-1.81534289679)/(o+1.)+(0.235921261477)+(0.600230470324)*0.5**o
        ref[3,0,0,1]=(0.758824131522)/(o+1.)+(0.540267308765)+(-0.665180555978)*0.5**o
        ref[3,0,1,0]=(-0.61930013537)/(o+1.)+(-0.0863636993976)+(0.303034722786)*0.5**o
        ref[3,0,1,1]=(1.09849962509)/(o+1.)+(0.780381688481)+(0.941635719543)*0.5**o
        ref[3,0,2,0]=(0.984167465846)/(o+1.)+(-0.471651730612)+(0.00395257708327)*0.5**o
        ref[3,0,2,1]=(-0.0216502142754)/(o+1.)+(1.25274987136)+(-0.468337187615)*0.5**o
        ref[3,1,0,0]=(1.21762392246)/(o+1.)+(0.0235896546035)+(0.349861248945)*0.5**o
        ref[3,1,0,1]=(0.681142299068)/(o+1.)+(0.586170195383)+(0.211853571648)*0.5**o
        ref[3,1,1,0]=(1.22842568638)/(o+1.)+(0.308050279545)+(-0.387180570377)*0.5**o
        ref[3,1,1,1]=(-0.708900188728)/(o+1.)+(1.04208633918)+(-0.0174879667472)*0.5**o
        ref[3,1,2,0]=(1.79188214766)/(o+1.)+(0.301719505956)+(-0.804013660301)*0.5**o
        ref[3,1,2,1]=(0.669297533033)/(o+1.)+(0.119247079862)+(0.246296653123)*0.5**o
        ref[3,2,0,0]=(-0.791486989477)/(o+1.)+(0.204677775698)+(0.849598346434)*0.5**o
        ref[3,2,0,1]=(-0.505948254664)/(o+1.)+(0.0688840300017)+(0.70937220736)*0.5**o
        ref[3,2,1,0]=(-0.691348115315)/(o+1.)+(0.437490279091)+(-0.150037039019)*0.5**o
        ref[3,2,1,1]=(0.708369627369)/(o+1.)+(0.176205102648)+(-0.478415264295)*0.5**o
        ref[3,2,2,0]=(0.297093119787)/(o+1.)+(-0.248132872678)+(-0.580692202861)*0.5**o
        ref[3,2,2,1]=(-0.476148841282)/(o+1.)+(0.443525381364)+(-0.0793810984529)*0.5**o
        ref[3,3,0,0]=(-0.901387033571)/(o+1.)+(-0.628090499737)+(0.671346370124)*0.5**o
        ref[3,3,0,1]=(-0.123005978892)/(o+1.)+(-0.59694482259)+(0.970893799626)*0.5**o
        ref[3,3,1,0]=(-0.784454504087)/(o+1.)+(0.436648462004)+(0.00487862539679)*0.5**o
        ref[3,3,1,1]=(-1.73297919322)/(o+1.)+(-0.09410977014)+(0.395719910762)*0.5**o
        ref[3,3,2,0]=(0.0989755292654)/(o+1.)+(0.062352403584)+(-0.0260502031901)*0.5**o
        ref[3,3,2,1]=(-0.690845491742)/(o+1.)+(0.678904082779)+(-0.437053488325)*0.5**o
        ref[3,4,0,0]=(-0.801695028055)/(o+1.)+(0.731880266747)+(-0.0949594759642)*0.5**o
        ref[3,4,0,1]=(-0.263446786535)/(o+1.)+(0.0496642554438)+(0.172309958154)*0.5**o
        ref[3,4,1,0]=(0.542595531974)/(o+1.)+(0.308327980561)+(-0.906144151831)*0.5**o
        ref[3,4,1,1]=(0.637885608826)/(o+1.)+(1.16403227519)+(-0.174220757444)*0.5**o
        ref[3,4,2,0]=(0.946926166426)/(o+1.)+(0.0242541266649)+(0.613899961774)*0.5**o
        ref[3,4,2,1]=(-0.0303035732231)/(o+1.)+(0.0375314231208)+(-0.715829853629)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactZero_fromSymbol_ReducedFunctionOnContactZero_rank0(self):
      """
      tests integral of rank 0 Symbol on the ReducedFunctionOnContactZero

      assumptions: ReducedFunctionOnContactZero(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      w=ReducedFunctionOnContactZero(self.domain)
      x=w.getX()
      arg=Symbol(shape=())
      s=Data(0,(),w)
      if dim==2:
        s=(0.793574201557)*x[0]**o+(0.516960263083)*x[0]+(-0.448328841896)*x[1]**o+(-0.142637802972)*x[1]
        ref=(-0.448328841896)/(o+1.)+(0.187161230055)+(0.793574201557)*0.5**o
      else:
        s=(-0.338096271642)*x[0]**o+(0.968157263648)*x[0]+(0.646364944878)*x[1]**o+(-0.80014418872)*x[1]+(0.0561188904819)*x[2]**o+(0.413390558956)*x[2]
        ref=(0.702483835359)/(o+1.)+(0.290701816942)+(-0.338096271642)*0.5**o
      res=integrate(arg)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactZero_fromSymbol_ReducedFunctionOnContactZero_rank1(self):
      """
      tests integral of rank 1 Symbol on the ReducedFunctionOnContactZero

      assumptions: ReducedFunctionOnContactZero(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      w=ReducedFunctionOnContactZero(self.domain)
      x=w.getX()
      arg=Symbol(shape=(2,))
      s=Data(0,(2,),w)
      ref=numpy.zeros((2,),numpy.float_)
      if dim==2:
        s[0]=(-0.298300777435)*x[0]**o+(-0.656203877839)*x[0]+(0.706465159872)*x[1]**o+(-0.441355227209)*x[1]
        s[1]=(-0.323920359168)*x[0]**o+(0.167106922584)*x[0]+(-0.77484608349)*x[1]**o+(0.961265696466)*x[1]
        ref[0]=(0.706465159872)/(o+1.)+(-0.548779552524)+(-0.298300777435)*0.5**o
        ref[1]=(-0.77484608349)/(o+1.)+(0.564186309525)+(-0.323920359168)*0.5**o
      else:
        s[0]=(0.226749576468)*x[0]**o+(-0.794303615675)*x[0]+(-0.475688985171)*x[1]**o+(-0.502986687652)*x[1]+(0.620418017904)*x[2]**o+(-0.201211072609)*x[2]
        s[1]=(-0.623757173924)*x[0]**o+(0.408817385275)*x[0]+(0.732106423025)*x[1]**o+(-0.414680272664)*x[1]+(-0.37214615142)*x[2]**o+(0.0433104237128)*x[2]
        ref[0]=(0.144729032732)/(o+1.)+(-0.749250687968)+(0.226749576468)*0.5**o
        ref[1]=(0.359960271605)/(o+1.)+(0.0187237681618)+(-0.623757173924)*0.5**o
      res=integrate(arg)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactZero_fromSymbol_ReducedFunctionOnContactZero_rank2(self):
      """
      tests integral of rank 2 Symbol on the ReducedFunctionOnContactZero

      assumptions: ReducedFunctionOnContactZero(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      w=ReducedFunctionOnContactZero(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5))
      s=Data(0,(4, 5),w)
      ref=numpy.zeros((4, 5),numpy.float_)
      if dim==2:
        s[0,0]=(0.111040625394)*x[0]**o+(-0.262719374505)*x[0]+(-0.629693203515)*x[1]**o+(-0.483036975224)*x[1]
        s[0,1]=(-0.710606684727)*x[0]**o+(-0.0460165503366)*x[0]+(-0.293965417547)*x[1]**o+(-0.495255124461)*x[1]
        s[0,2]=(-0.777570630133)*x[0]**o+(-0.116630316032)*x[0]+(0.137509413983)*x[1]**o+(0.819368394153)*x[1]
        s[0,3]=(-0.710576800354)*x[0]**o+(-0.580982318739)*x[0]+(0.88040631872)*x[1]**o+(0.906791051848)*x[1]
        s[0,4]=(-0.182839498651)*x[0]**o+(-0.73087599847)*x[0]+(0.343867628394)*x[1]**o+(0.430107584469)*x[1]
        s[1,0]=(0.69614992851)*x[0]**o+(-0.897687606816)*x[0]+(0.0635059035444)*x[1]**o+(-0.664283784877)*x[1]
        s[1,1]=(-0.838368623112)*x[0]**o+(0.235356849847)*x[0]+(-0.575094104243)*x[1]**o+(0.317897877318)*x[1]
        s[1,2]=(-0.466268810559)*x[0]**o+(-0.0899593004601)*x[0]+(-0.966257251835)*x[1]**o+(-0.144068791611)*x[1]
        s[1,3]=(0.0053613868128)*x[0]**o+(0.0318146753697)*x[0]+(0.679315501165)*x[1]**o+(-0.0722416861767)*x[1]
        s[1,4]=(0.933407356129)*x[0]**o+(-0.495175209275)*x[0]+(-0.6009471862)*x[1]**o+(-0.539903879411)*x[1]
        s[2,0]=(0.557935263092)*x[0]**o+(0.858947740632)*x[0]+(0.0661071238236)*x[1]**o+(-0.92607178646)*x[1]
        s[2,1]=(-0.992502095953)*x[0]**o+(0.725486544204)*x[0]+(0.966008980795)*x[1]**o+(-0.687474869885)*x[1]
        s[2,2]=(0.131446552365)*x[0]**o+(-0.23682138542)*x[0]+(0.762830166878)*x[1]**o+(0.154873065449)*x[1]
        s[2,3]=(-0.325278661025)*x[0]**o+(0.00830856266952)*x[0]+(-0.67055674604)*x[1]**o+(-0.200351789931)*x[1]
        s[2,4]=(-0.153591894122)*x[0]**o+(0.11301998318)*x[0]+(-0.931207923183)*x[1]**o+(-0.808854650606)*x[1]
        s[3,0]=(0.482659628909)*x[0]**o+(0.625042019107)*x[0]+(-0.0579000114037)*x[1]**o+(0.468282154942)*x[1]
        s[3,1]=(-0.324897146141)*x[0]**o+(0.107637712241)*x[0]+(-0.802284595948)*x[1]**o+(0.462102155569)*x[1]
        s[3,2]=(0.0324851257443)*x[0]**o+(0.88112504986)*x[0]+(0.364393243039)*x[1]**o+(0.279546598993)*x[1]
        s[3,3]=(0.362592102084)*x[0]**o+(0.531299996192)*x[0]+(-0.133406058793)*x[1]**o+(0.331414049109)*x[1]
        s[3,4]=(0.527090134663)*x[0]**o+(0.992717007892)*x[0]+(0.0417616039807)*x[1]**o+(-0.447250742864)*x[1]
        ref[0,0]=(-0.629693203515)/(o+1.)+(-0.372878174865)+(0.111040625394)*0.5**o
        ref[0,1]=(-0.293965417547)/(o+1.)+(-0.270635837399)+(-0.710606684727)*0.5**o
        ref[0,2]=(0.137509413983)/(o+1.)+(0.351369039061)+(-0.777570630133)*0.5**o
        ref[0,3]=(0.88040631872)/(o+1.)+(0.162904366555)+(-0.710576800354)*0.5**o
        ref[0,4]=(0.343867628394)/(o+1.)+(-0.150384207)+(-0.182839498651)*0.5**o
        ref[1,0]=(0.0635059035444)/(o+1.)+(-0.780985695846)+(0.69614992851)*0.5**o
        ref[1,1]=(-0.575094104243)/(o+1.)+(0.276627363583)+(-0.838368623112)*0.5**o
        ref[1,2]=(-0.966257251835)/(o+1.)+(-0.117014046036)+(-0.466268810559)*0.5**o
        ref[1,3]=(0.679315501165)/(o+1.)+(-0.0202135054035)+(0.0053613868128)*0.5**o
        ref[1,4]=(-0.6009471862)/(o+1.)+(-0.517539544343)+(0.933407356129)*0.5**o
        ref[2,0]=(0.0661071238236)/(o+1.)+(-0.0335620229141)+(0.557935263092)*0.5**o
        ref[2,1]=(0.966008980795)/(o+1.)+(0.0190058371594)+(-0.992502095953)*0.5**o
        ref[2,2]=(0.762830166878)/(o+1.)+(-0.0409741599856)+(0.131446552365)*0.5**o
        ref[2,3]=(-0.67055674604)/(o+1.)+(-0.0960216136307)+(-0.325278661025)*0.5**o
        ref[2,4]=(-0.931207923183)/(o+1.)+(-0.347917333713)+(-0.153591894122)*0.5**o
        ref[3,0]=(-0.0579000114037)/(o+1.)+(0.546662087024)+(0.482659628909)*0.5**o
        ref[3,1]=(-0.802284595948)/(o+1.)+(0.284869933905)+(-0.324897146141)*0.5**o
        ref[3,2]=(0.364393243039)/(o+1.)+(0.580335824427)+(0.0324851257443)*0.5**o
        ref[3,3]=(-0.133406058793)/(o+1.)+(0.431357022651)+(0.362592102084)*0.5**o
        ref[3,4]=(0.0417616039807)/(o+1.)+(0.272733132514)+(0.527090134663)*0.5**o
      else:
        s[0,0]=(-0.910274365459)*x[0]**o+(0.444602331451)*x[0]+(0.109320774249)*x[1]**o+(-0.340302571967)*x[1]+(-0.43434237629)*x[2]**o+(-0.943276146561)*x[2]
        s[0,1]=(0.290437327261)*x[0]**o+(0.936756757082)*x[0]+(0.169426804577)*x[1]**o+(-0.606271363634)*x[1]+(0.0955072153861)*x[2]**o+(0.500289189372)*x[2]
        s[0,2]=(0.857537342473)*x[0]**o+(-0.441518068791)*x[0]+(0.280884170231)*x[1]**o+(-0.712646882141)*x[1]+(-0.771612131879)*x[2]**o+(-0.191796438089)*x[2]
        s[0,3]=(0.171520107567)*x[0]**o+(0.92204336001)*x[0]+(-0.370919720606)*x[1]**o+(0.768895739632)*x[1]+(0.784096316797)*x[2]**o+(-0.687371757606)*x[2]
        s[0,4]=(-0.941363513955)*x[0]**o+(-0.715641149458)*x[0]+(0.309819925901)*x[1]**o+(0.807925106464)*x[1]+(0.283008337009)*x[2]**o+(-0.858020235388)*x[2]
        s[1,0]=(-0.368063286241)*x[0]**o+(-0.379538647626)*x[0]+(0.568731253476)*x[1]**o+(-0.129142499862)*x[1]+(0.664990905083)*x[2]**o+(0.72526514854)*x[2]
        s[1,1]=(0.609862576834)*x[0]**o+(0.168414778244)*x[0]+(0.705960386503)*x[1]**o+(0.414950596481)*x[1]+(-0.324853936067)*x[2]**o+(0.705193129155)*x[2]
        s[1,2]=(0.259927679551)*x[0]**o+(0.023313779932)*x[0]+(0.542815998551)*x[1]**o+(-0.980335573131)*x[1]+(-0.104377098432)*x[2]**o+(0.678830057048)*x[2]
        s[1,3]=(-0.592662443082)*x[0]**o+(0.887389713188)*x[0]+(-0.907629711077)*x[1]**o+(0.965433874409)*x[1]+(-0.0179359825534)*x[2]**o+(-0.768070640771)*x[2]
        s[1,4]=(0.0233212032451)*x[0]**o+(-0.0505203178694)*x[0]+(-0.465871409785)*x[1]**o+(-0.437317133846)*x[1]+(0.208545612303)*x[2]**o+(-0.0502465428942)*x[2]
        s[2,0]=(-0.273603162201)*x[0]**o+(0.192745661279)*x[0]+(0.296585107806)*x[1]**o+(0.46173411046)*x[1]+(0.673984995689)*x[2]**o+(-0.88275015509)*x[2]
        s[2,1]=(0.36805707552)*x[0]**o+(-0.33705395449)*x[0]+(0.0140175482223)*x[1]**o+(0.64760814101)*x[1]+(-0.91874701962)*x[2]**o+(0.383318293978)*x[2]
        s[2,2]=(-0.900273344488)*x[0]**o+(0.498842169958)*x[0]+(-0.498406294921)*x[1]**o+(-0.403932835154)*x[1]+(-0.156723389073)*x[2]**o+(0.214962470693)*x[2]
        s[2,3]=(-0.798208224472)*x[0]**o+(-0.746730197648)*x[0]+(-0.397704785717)*x[1]**o+(-0.970985500613)*x[1]+(0.750200448417)*x[2]**o+(0.974572822346)*x[2]
        s[2,4]=(-0.464282027917)*x[0]**o+(-0.754333918275)*x[0]+(0.332816676589)*x[1]**o+(-0.600568600942)*x[1]+(0.188339826223)*x[2]**o+(0.956665673953)*x[2]
        s[3,0]=(0.291526453393)*x[0]**o+(0.259245798805)*x[0]+(0.742708355753)*x[1]**o+(0.21348170853)*x[1]+(-0.983407115555)*x[2]**o+(-0.862966239373)*x[2]
        s[3,1]=(0.874250384958)*x[0]**o+(-0.542706319171)*x[0]+(0.513206274058)*x[1]**o+(-0.493584668769)*x[1]+(-0.990552049402)*x[2]**o+(-0.0611365067488)*x[2]
        s[3,2]=(-0.620092725842)*x[0]**o+(-0.0940854652025)*x[0]+(-0.788220564782)*x[1]**o+(0.952851796216)*x[1]+(-0.102879423616)*x[2]**o+(-0.559035384428)*x[2]
        s[3,3]=(-0.149822264616)*x[0]**o+(-0.113017264993)*x[0]+(-0.472046202247)*x[1]**o+(-0.21062255325)*x[1]+(-0.531066124816)*x[2]**o+(0.307214441703)*x[2]
        s[3,4]=(0.412388911019)*x[0]**o+(-0.0814495987981)*x[0]+(0.176049376352)*x[1]**o+(-0.660623309779)*x[1]+(0.573030010589)*x[2]**o+(-0.671602921264)*x[2]
        ref[0,0]=(-0.325021602041)/(o+1.)+(-0.419488193539)+(-0.910274365459)*0.5**o
        ref[0,1]=(0.264934019963)/(o+1.)+(0.41538729141)+(0.290437327261)*0.5**o
        ref[0,2]=(-0.490727961648)/(o+1.)+(-0.672980694511)+(0.857537342473)*0.5**o
        ref[0,3]=(0.413176596192)/(o+1.)+(0.501783671018)+(0.171520107567)*0.5**o
        ref[0,4]=(0.592828262909)/(o+1.)+(-0.382868139192)+(-0.941363513955)*0.5**o
        ref[1,0]=(1.23372215856)/(o+1.)+(0.108292000526)+(-0.368063286241)*0.5**o
        ref[1,1]=(0.381106450436)/(o+1.)+(0.64427925194)+(0.609862576834)*0.5**o
        ref[1,2]=(0.438438900119)/(o+1.)+(-0.139095868075)+(0.259927679551)*0.5**o
        ref[1,3]=(-0.92556569363)/(o+1.)+(0.542376473413)+(-0.592662443082)*0.5**o
        ref[1,4]=(-0.257325797482)/(o+1.)+(-0.269041997305)+(0.0233212032451)*0.5**o
        ref[2,0]=(0.970570103495)/(o+1.)+(-0.114135191676)+(-0.273603162201)*0.5**o
        ref[2,1]=(-0.904729471398)/(o+1.)+(0.346936240249)+(0.36805707552)*0.5**o
        ref[2,2]=(-0.655129683994)/(o+1.)+(0.154935902749)+(-0.900273344488)*0.5**o
        ref[2,3]=(0.352495662699)/(o+1.)+(-0.371571437957)+(-0.798208224472)*0.5**o
        ref[2,4]=(0.521156502812)/(o+1.)+(-0.199118422633)+(-0.464282027917)*0.5**o
        ref[3,0]=(-0.240698759801)/(o+1.)+(-0.195119366019)+(0.291526453393)*0.5**o
        ref[3,1]=(-0.477345775345)/(o+1.)+(-0.548713747345)+(0.874250384958)*0.5**o
        ref[3,2]=(-0.891099988398)/(o+1.)+(0.149865473293)+(-0.620092725842)*0.5**o
        ref[3,3]=(-1.00311232706)/(o+1.)+(-0.00821268827029)+(-0.149822264616)*0.5**o
        ref[3,4]=(0.749079386941)/(o+1.)+(-0.70683791492)+(0.412388911019)*0.5**o
      res=integrate(arg)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactZero_fromSymbol_ReducedFunctionOnContactZero_rank3(self):
      """
      tests integral of rank 3 Symbol on the ReducedFunctionOnContactZero

      assumptions: ReducedFunctionOnContactZero(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      w=ReducedFunctionOnContactZero(self.domain)
      x=w.getX()
      arg=Symbol(shape=(6, 2, 2))
      s=Data(0,(6, 2, 2),w)
      ref=numpy.zeros((6, 2, 2),numpy.float_)
      if dim==2:
        s[0,0,0]=(0.467159870975)*x[0]**o+(-0.231363355451)*x[0]+(-0.522613470003)*x[1]**o+(-0.396872827596)*x[1]
        s[0,0,1]=(0.483392039154)*x[0]**o+(-0.0481934692208)*x[0]+(0.59831432517)*x[1]**o+(0.753587205182)*x[1]
        s[0,1,0]=(-0.642178324261)*x[0]**o+(0.580441968971)*x[0]+(0.19099567725)*x[1]**o+(-0.409684325755)*x[1]
        s[0,1,1]=(-0.963913869717)*x[0]**o+(-0.412074133404)*x[0]+(-0.272818985272)*x[1]**o+(0.187802732278)*x[1]
        s[1,0,0]=(0.377505937225)*x[0]**o+(0.51898033576)*x[0]+(-0.439566585731)*x[1]**o+(-0.998166796948)*x[1]
        s[1,0,1]=(0.389095949557)*x[0]**o+(-0.382477743743)*x[0]+(-0.960850005856)*x[1]**o+(-0.440393846867)*x[1]
        s[1,1,0]=(-0.70296977387)*x[0]**o+(0.315478264729)*x[0]+(-0.867044520578)*x[1]**o+(-0.760970516579)*x[1]
        s[1,1,1]=(0.706189511641)*x[0]**o+(-0.847541758712)*x[0]+(0.814254280461)*x[1]**o+(0.38248389532)*x[1]
        s[2,0,0]=(-0.651567091764)*x[0]**o+(-0.410805007914)*x[0]+(0.0538051169317)*x[1]**o+(0.534010791665)*x[1]
        s[2,0,1]=(0.487085829627)*x[0]**o+(0.774536856681)*x[0]+(-0.873566030375)*x[1]**o+(0.370185957857)*x[1]
        s[2,1,0]=(0.632552547114)*x[0]**o+(0.147565981324)*x[0]+(0.868108695886)*x[1]**o+(0.671762982739)*x[1]
        s[2,1,1]=(-0.259846254622)*x[0]**o+(0.631407506034)*x[0]+(-0.681905081868)*x[1]**o+(0.0452497065686)*x[1]
        s[3,0,0]=(0.057043606868)*x[0]**o+(-0.0902461813963)*x[0]+(-0.240745938715)*x[1]**o+(-0.831861045923)*x[1]
        s[3,0,1]=(0.699648897337)*x[0]**o+(0.640417545224)*x[0]+(-0.430848221186)*x[1]**o+(0.462058652596)*x[1]
        s[3,1,0]=(0.664926292599)*x[0]**o+(-0.891859906903)*x[0]+(0.775280390876)*x[1]**o+(0.614722294397)*x[1]
        s[3,1,1]=(-0.785736689922)*x[0]**o+(0.687423487421)*x[0]+(-0.573349590903)*x[1]**o+(-0.653598711028)*x[1]
        s[4,0,0]=(0.319805468591)*x[0]**o+(0.601962226985)*x[0]+(0.358802389875)*x[1]**o+(-0.468789714036)*x[1]
        s[4,0,1]=(0.975677606412)*x[0]**o+(-0.702436984042)*x[0]+(-0.720390079934)*x[1]**o+(-0.377014128023)*x[1]
        s[4,1,0]=(0.736756275381)*x[0]**o+(-0.257998474493)*x[0]+(0.5991398372)*x[1]**o+(-0.32399683906)*x[1]
        s[4,1,1]=(-0.215912723594)*x[0]**o+(0.326544778825)*x[0]+(-0.620761245415)*x[1]**o+(-0.498174781757)*x[1]
        s[5,0,0]=(-0.601667515189)*x[0]**o+(0.516733915255)*x[0]+(0.183410588351)*x[1]**o+(-0.792386589315)*x[1]
        s[5,0,1]=(-0.561245419581)*x[0]**o+(0.371706636352)*x[0]+(-0.795300794294)*x[1]**o+(0.41047673348)*x[1]
        s[5,1,0]=(0.466316419068)*x[0]**o+(-0.0802419079562)*x[0]+(0.569083019829)*x[1]**o+(-0.20883523199)*x[1]
        s[5,1,1]=(-0.546242876843)*x[0]**o+(-0.499594802846)*x[0]+(-0.945490356092)*x[1]**o+(0.938984415563)*x[1]
        ref[0,0,0]=(-0.522613470003)/(o+1.)+(-0.314118091524)+(0.467159870975)*0.5**o
        ref[0,0,1]=(0.59831432517)/(o+1.)+(0.35269686798)+(0.483392039154)*0.5**o
        ref[0,1,0]=(0.19099567725)/(o+1.)+(0.0853788216078)+(-0.642178324261)*0.5**o
        ref[0,1,1]=(-0.272818985272)/(o+1.)+(-0.112135700563)+(-0.963913869717)*0.5**o
        ref[1,0,0]=(-0.439566585731)/(o+1.)+(-0.239593230594)+(0.377505937225)*0.5**o
        ref[1,0,1]=(-0.960850005856)/(o+1.)+(-0.411435795305)+(0.389095949557)*0.5**o
        ref[1,1,0]=(-0.867044520578)/(o+1.)+(-0.222746125925)+(-0.70296977387)*0.5**o
        ref[1,1,1]=(0.814254280461)/(o+1.)+(-0.232528931696)+(0.706189511641)*0.5**o
        ref[2,0,0]=(0.0538051169317)/(o+1.)+(0.0616028918756)+(-0.651567091764)*0.5**o
        ref[2,0,1]=(-0.873566030375)/(o+1.)+(0.572361407269)+(0.487085829627)*0.5**o
        ref[2,1,0]=(0.868108695886)/(o+1.)+(0.409664482032)+(0.632552547114)*0.5**o
        ref[2,1,1]=(-0.681905081868)/(o+1.)+(0.338328606302)+(-0.259846254622)*0.5**o
        ref[3,0,0]=(-0.240745938715)/(o+1.)+(-0.46105361366)+(0.057043606868)*0.5**o
        ref[3,0,1]=(-0.430848221186)/(o+1.)+(0.55123809891)+(0.699648897337)*0.5**o
        ref[3,1,0]=(0.775280390876)/(o+1.)+(-0.138568806253)+(0.664926292599)*0.5**o
        ref[3,1,1]=(-0.573349590903)/(o+1.)+(0.0169123881966)+(-0.785736689922)*0.5**o
        ref[4,0,0]=(0.358802389875)/(o+1.)+(0.0665862564747)+(0.319805468591)*0.5**o
        ref[4,0,1]=(-0.720390079934)/(o+1.)+(-0.539725556033)+(0.975677606412)*0.5**o
        ref[4,1,0]=(0.5991398372)/(o+1.)+(-0.290997656776)+(0.736756275381)*0.5**o
        ref[4,1,1]=(-0.620761245415)/(o+1.)+(-0.0858150014665)+(-0.215912723594)*0.5**o
        ref[5,0,0]=(0.183410588351)/(o+1.)+(-0.13782633703)+(-0.601667515189)*0.5**o
        ref[5,0,1]=(-0.795300794294)/(o+1.)+(0.391091684916)+(-0.561245419581)*0.5**o
        ref[5,1,0]=(0.569083019829)/(o+1.)+(-0.144538569973)+(0.466316419068)*0.5**o
        ref[5,1,1]=(-0.945490356092)/(o+1.)+(0.219694806358)+(-0.546242876843)*0.5**o
      else:
        s[0,0,0]=(-0.669363049954)*x[0]**o+(0.94445770304)*x[0]+(-0.504461532785)*x[1]**o+(0.636064462988)*x[1]+(0.416245244177)*x[2]**o+(0.666397932744)*x[2]
        s[0,0,1]=(-0.778564625852)*x[0]**o+(-0.825187520549)*x[0]+(0.72174236037)*x[1]**o+(-0.884811347472)*x[1]+(-0.206727051888)*x[2]**o+(0.669175992728)*x[2]
        s[0,1,0]=(0.59083523386)*x[0]**o+(0.264891177321)*x[0]+(0.728529437979)*x[1]**o+(0.689869176288)*x[1]+(0.759759991708)*x[2]**o+(-0.768640416685)*x[2]
        s[0,1,1]=(-0.256994678625)*x[0]**o+(0.151952222597)*x[0]+(-0.615154134713)*x[1]**o+(0.705589259856)*x[1]+(-0.27400110493)*x[2]**o+(-0.042497975508)*x[2]
        s[1,0,0]=(0.577165945348)*x[0]**o+(-0.475277785249)*x[0]+(-0.106826355519)*x[1]**o+(0.0424930134605)*x[1]+(0.680372240013)*x[2]**o+(0.943809871598)*x[2]
        s[1,0,1]=(-0.309232479272)*x[0]**o+(0.195348662283)*x[0]+(0.833914381189)*x[1]**o+(0.262108970555)*x[1]+(-0.716673250709)*x[2]**o+(-0.168478923947)*x[2]
        s[1,1,0]=(-0.275625390067)*x[0]**o+(0.523842393012)*x[0]+(0.437231937115)*x[1]**o+(0.758417244685)*x[1]+(0.826527568677)*x[2]**o+(-0.998749362644)*x[2]
        s[1,1,1]=(0.148292586557)*x[0]**o+(0.405011487356)*x[0]+(0.4599949448)*x[1]**o+(-0.612512744054)*x[1]+(0.187004698026)*x[2]**o+(0.41423737326)*x[2]
        s[2,0,0]=(-0.0683880574768)*x[0]**o+(0.527974031235)*x[0]+(0.761878395348)*x[1]**o+(0.588459184489)*x[1]+(-0.587912848751)*x[2]**o+(0.305103889218)*x[2]
        s[2,0,1]=(0.520929962776)*x[0]**o+(0.33193799149)*x[0]+(-0.342126485854)*x[1]**o+(-0.14865315386)*x[1]+(0.310895900233)*x[2]**o+(-0.322869734717)*x[2]
        s[2,1,0]=(-0.321360677385)*x[0]**o+(0.252389514576)*x[0]+(-0.0393452094196)*x[1]**o+(0.700761849784)*x[1]+(-0.526926106563)*x[2]**o+(0.510264453258)*x[2]
        s[2,1,1]=(-0.417527085289)*x[0]**o+(-0.272942288011)*x[0]+(0.601765245586)*x[1]**o+(-0.790696366296)*x[1]+(-0.012638708702)*x[2]**o+(0.932434350334)*x[2]
        s[3,0,0]=(-0.511516099074)*x[0]**o+(0.41782462791)*x[0]+(0.0995516379245)*x[1]**o+(0.546046499872)*x[1]+(0.874011011955)*x[2]**o+(-0.788555434082)*x[2]
        s[3,0,1]=(0.327526313236)*x[0]**o+(-0.425896436338)*x[0]+(0.914336317958)*x[1]**o+(0.307045278546)*x[1]+(0.72678975804)*x[2]**o+(-0.752188240491)*x[2]
        s[3,1,0]=(0.396685657572)*x[0]**o+(-0.853213412456)*x[0]+(0.592696880548)*x[1]**o+(-0.300028961606)*x[1]+(0.876996858765)*x[2]**o+(-0.0882159006726)*x[2]
        s[3,1,1]=(-0.482121450588)*x[0]**o+(-0.112743359188)*x[0]+(0.328241708104)*x[1]**o+(-0.617997058589)*x[1]+(0.936876537283)*x[2]**o+(0.350080033532)*x[2]
        s[4,0,0]=(-0.200856211954)*x[0]**o+(-0.122991084849)*x[0]+(0.487659949467)*x[1]**o+(-0.425527004923)*x[1]+(-0.147553486772)*x[2]**o+(-0.32641504978)*x[2]
        s[4,0,1]=(0.333642453215)*x[0]**o+(0.954952509878)*x[0]+(-0.291340184543)*x[1]**o+(-0.628839479489)*x[1]+(0.67230688241)*x[2]**o+(0.84754444469)*x[2]
        s[4,1,0]=(0.0708849044002)*x[0]**o+(-0.53553180865)*x[0]+(-0.114931581103)*x[1]**o+(-0.374826425939)*x[1]+(-0.0919922579012)*x[2]**o+(-0.745351276697)*x[2]
        s[4,1,1]=(-0.224313591576)*x[0]**o+(0.329362356625)*x[0]+(-0.60957590052)*x[1]**o+(0.824884117846)*x[1]+(-0.695979917244)*x[2]**o+(-0.279844578235)*x[2]
        s[5,0,0]=(-0.431218311479)*x[0]**o+(0.349200792571)*x[0]+(0.840732425028)*x[1]**o+(-0.484758521346)*x[1]+(0.295545194625)*x[2]**o+(0.47040408229)*x[2]
        s[5,0,1]=(0.506935329965)*x[0]**o+(0.797912921589)*x[0]+(-0.143979832786)*x[1]**o+(-0.660815905482)*x[1]+(0.123849993986)*x[2]**o+(-0.877052796091)*x[2]
        s[5,1,0]=(0.500348066835)*x[0]**o+(-0.455430279521)*x[0]+(-0.569813267302)*x[1]**o+(-0.855668211999)*x[1]+(0.203279416274)*x[2]**o+(-0.112743061503)*x[2]
        s[5,1,1]=(0.993758177333)*x[0]**o+(-0.784317748398)*x[0]+(0.530003336163)*x[1]**o+(-0.857305987154)*x[1]+(0.391887947598)*x[2]**o+(0.933473063729)*x[2]
        ref[0,0,0]=(-0.0882162886075)/(o+1.)+(1.12346004939)+(-0.669363049954)*0.5**o
        ref[0,0,1]=(0.515015308482)/(o+1.)+(-0.520411437647)+(-0.778564625852)*0.5**o
        ref[0,1,0]=(1.48828942969)/(o+1.)+(0.0930599684618)+(0.59083523386)*0.5**o
        ref[0,1,1]=(-0.889155239643)/(o+1.)+(0.407521753473)+(-0.256994678625)*0.5**o
        ref[1,0,0]=(0.573545884494)/(o+1.)+(0.255512549905)+(0.577165945348)*0.5**o
        ref[1,0,1]=(0.11724113048)/(o+1.)+(0.144489354446)+(-0.309232479272)*0.5**o
        ref[1,1,0]=(1.26375950579)/(o+1.)+(0.141755137526)+(-0.275625390067)*0.5**o
        ref[1,1,1]=(0.646999642826)/(o+1.)+(0.103368058281)+(0.148292586557)*0.5**o
        ref[2,0,0]=(0.173965546596)/(o+1.)+(0.710768552471)+(-0.0683880574768)*0.5**o
        ref[2,0,1]=(-0.0312305856211)/(o+1.)+(-0.0697924485433)+(0.520929962776)*0.5**o
        ref[2,1,0]=(-0.566271315982)/(o+1.)+(0.731707908809)+(-0.321360677385)*0.5**o
        ref[2,1,1]=(0.589126536884)/(o+1.)+(-0.0656021519867)+(-0.417527085289)*0.5**o
        ref[3,0,0]=(0.973562649879)/(o+1.)+(0.0876578468496)+(-0.511516099074)*0.5**o
        ref[3,0,1]=(1.641126076)/(o+1.)+(-0.435519699141)+(0.327526313236)*0.5**o
        ref[3,1,0]=(1.46969373931)/(o+1.)+(-0.620729137367)+(0.396685657572)*0.5**o
        ref[3,1,1]=(1.26511824539)/(o+1.)+(-0.190330192122)+(-0.482121450588)*0.5**o
        ref[4,0,0]=(0.340106462695)/(o+1.)+(-0.437466569776)+(-0.200856211954)*0.5**o
        ref[4,0,1]=(0.380966697866)/(o+1.)+(0.586828737539)+(0.333642453215)*0.5**o
        ref[4,1,0]=(-0.206923839004)/(o+1.)+(-0.827854755643)+(0.0708849044002)*0.5**o
        ref[4,1,1]=(-1.30555581776)/(o+1.)+(0.437200948118)+(-0.224313591576)*0.5**o
        ref[5,0,0]=(1.13627761965)/(o+1.)+(0.167423176757)+(-0.431218311479)*0.5**o
        ref[5,0,1]=(-0.0201298387998)/(o+1.)+(-0.369977889992)+(0.506935329965)*0.5**o
        ref[5,1,0]=(-0.366533851028)/(o+1.)+(-0.711920776511)+(0.500348066835)*0.5**o
        ref[5,1,1]=(0.921891283761)/(o+1.)+(-0.354075335912)+(0.993758177333)*0.5**o
      res=integrate(arg)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactZero_fromSymbol_ReducedFunctionOnContactZero_rank4(self):
      """
      tests integral of rank 4 Symbol on the ReducedFunctionOnContactZero

      assumptions: ReducedFunctionOnContactZero(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      w=ReducedFunctionOnContactZero(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5, 3, 2))
      s=Data(0,(4, 5, 3, 2),w)
      ref=numpy.zeros((4, 5, 3, 2),numpy.float_)
      if dim==2:
        s[0,0,0,0]=(-0.135767714785)*x[0]**o+(0.602721853735)*x[0]+(0.685908691403)*x[1]**o+(-0.278963806046)*x[1]
        s[0,0,0,1]=(0.548074276718)*x[0]**o+(-0.00153460595028)*x[0]+(0.964032895202)*x[1]**o+(0.145345601288)*x[1]
        s[0,0,1,0]=(0.00388957848949)*x[0]**o+(-0.00330645752363)*x[0]+(-0.398340925709)*x[1]**o+(0.544904812736)*x[1]
        s[0,0,1,1]=(0.896772134786)*x[0]**o+(-0.305756341328)*x[0]+(0.766835159085)*x[1]**o+(-0.185065752039)*x[1]
        s[0,0,2,0]=(0.455765211649)*x[0]**o+(-0.172084793069)*x[0]+(-0.745694059134)*x[1]**o+(0.467137037155)*x[1]
        s[0,0,2,1]=(-0.683685778458)*x[0]**o+(-0.526137358848)*x[0]+(0.499311653339)*x[1]**o+(-0.810057930627)*x[1]
        s[0,1,0,0]=(0.244426126288)*x[0]**o+(0.464096329887)*x[0]+(-0.565273192791)*x[1]**o+(0.341584391362)*x[1]
        s[0,1,0,1]=(-0.0873578866485)*x[0]**o+(-0.280704680398)*x[0]+(0.0551381505624)*x[1]**o+(0.224333075687)*x[1]
        s[0,1,1,0]=(0.0886184124686)*x[0]**o+(-0.577421322103)*x[0]+(-0.784267376644)*x[1]**o+(-0.220754022299)*x[1]
        s[0,1,1,1]=(-0.901203063708)*x[0]**o+(-0.791637322168)*x[0]+(-0.615643115793)*x[1]**o+(-0.0955674319946)*x[1]
        s[0,1,2,0]=(0.418411748143)*x[0]**o+(0.0825948292296)*x[0]+(0.00527147896827)*x[1]**o+(0.474507057947)*x[1]
        s[0,1,2,1]=(0.574446727172)*x[0]**o+(0.634382868948)*x[0]+(-0.8192829244)*x[1]**o+(0.0916815679535)*x[1]
        s[0,2,0,0]=(-0.324743695914)*x[0]**o+(-0.517660802435)*x[0]+(-0.694637110589)*x[1]**o+(-0.191013585315)*x[1]
        s[0,2,0,1]=(0.538239581284)*x[0]**o+(-0.793537232228)*x[0]+(-0.260263189341)*x[1]**o+(-0.232999585659)*x[1]
        s[0,2,1,0]=(-0.535444326182)*x[0]**o+(0.375771246892)*x[0]+(-0.351338268294)*x[1]**o+(-0.922383156347)*x[1]
        s[0,2,1,1]=(-0.754984712928)*x[0]**o+(0.844268402619)*x[0]+(-0.373848162528)*x[1]**o+(-0.0348415758383)*x[1]
        s[0,2,2,0]=(0.101157816757)*x[0]**o+(0.322178434764)*x[0]+(0.309106041559)*x[1]**o+(-0.720194799504)*x[1]
        s[0,2,2,1]=(-0.537633855745)*x[0]**o+(-0.149185820274)*x[0]+(-0.155113200571)*x[1]**o+(0.433800441998)*x[1]
        s[0,3,0,0]=(0.466579973711)*x[0]**o+(-0.924681979368)*x[0]+(0.491019303223)*x[1]**o+(0.994389212618)*x[1]
        s[0,3,0,1]=(0.202651138329)*x[0]**o+(0.114061296405)*x[0]+(-0.660374384585)*x[1]**o+(0.192752003014)*x[1]
        s[0,3,1,0]=(0.442171269159)*x[0]**o+(-0.0740165829499)*x[0]+(-0.952344452735)*x[1]**o+(0.734456419975)*x[1]
        s[0,3,1,1]=(0.733989617351)*x[0]**o+(0.157469674236)*x[0]+(-0.305478230795)*x[1]**o+(-0.416091191381)*x[1]
        s[0,3,2,0]=(-0.709003057494)*x[0]**o+(-0.554321605532)*x[0]+(-0.374724207394)*x[1]**o+(-0.799384819999)*x[1]
        s[0,3,2,1]=(-0.125623348142)*x[0]**o+(-0.695830039564)*x[0]+(0.0283773197868)*x[1]**o+(-0.459458049105)*x[1]
        s[0,4,0,0]=(0.490207583728)*x[0]**o+(0.302026020799)*x[0]+(0.582676151383)*x[1]**o+(-0.540047416714)*x[1]
        s[0,4,0,1]=(-0.605187460486)*x[0]**o+(-0.700102630541)*x[0]+(0.576928080618)*x[1]**o+(-0.753735278462)*x[1]
        s[0,4,1,0]=(-0.0850932164524)*x[0]**o+(-0.132436670102)*x[0]+(0.634879197271)*x[1]**o+(-0.40759333575)*x[1]
        s[0,4,1,1]=(0.258713682268)*x[0]**o+(0.941275860044)*x[0]+(-0.976313171743)*x[1]**o+(-0.817915320276)*x[1]
        s[0,4,2,0]=(-0.350506879421)*x[0]**o+(-0.935416620637)*x[0]+(0.153793647237)*x[1]**o+(0.384520330312)*x[1]
        s[0,4,2,1]=(0.671395932648)*x[0]**o+(-0.338473533885)*x[0]+(-0.769374193622)*x[1]**o+(-0.0867650576614)*x[1]
        s[1,0,0,0]=(0.188023378319)*x[0]**o+(0.662970173794)*x[0]+(0.307120506561)*x[1]**o+(-0.561302563336)*x[1]
        s[1,0,0,1]=(0.800293252159)*x[0]**o+(0.414126518273)*x[0]+(-0.411140279426)*x[1]**o+(-0.726050617156)*x[1]
        s[1,0,1,0]=(-0.515339883608)*x[0]**o+(-0.849896429803)*x[0]+(-0.404819325345)*x[1]**o+(-0.660381411211)*x[1]
        s[1,0,1,1]=(0.0512515899156)*x[0]**o+(-0.59411151972)*x[0]+(-0.586954384255)*x[1]**o+(-0.581246213363)*x[1]
        s[1,0,2,0]=(-0.581670807515)*x[0]**o+(-0.151580006917)*x[0]+(0.166288186202)*x[1]**o+(-0.90507298782)*x[1]
        s[1,0,2,1]=(-0.634327645938)*x[0]**o+(0.227054101481)*x[0]+(-0.698802363494)*x[1]**o+(-0.191787378024)*x[1]
        s[1,1,0,0]=(0.434696675884)*x[0]**o+(0.550165686473)*x[0]+(-0.381545673134)*x[1]**o+(0.0353703778505)*x[1]
        s[1,1,0,1]=(0.438065102364)*x[0]**o+(0.734419538756)*x[0]+(-0.654368291721)*x[1]**o+(-0.288248589993)*x[1]
        s[1,1,1,0]=(0.812437488024)*x[0]**o+(0.105098628605)*x[0]+(0.670405216194)*x[1]**o+(-0.932828342978)*x[1]
        s[1,1,1,1]=(0.52676928941)*x[0]**o+(0.34803390652)*x[0]+(0.0351984047833)*x[1]**o+(0.410003399979)*x[1]
        s[1,1,2,0]=(-0.0967009361026)*x[0]**o+(0.164660384961)*x[0]+(0.210182888846)*x[1]**o+(0.522971695817)*x[1]
        s[1,1,2,1]=(-0.664016192839)*x[0]**o+(0.452428778809)*x[0]+(-0.107282780635)*x[1]**o+(0.807095090624)*x[1]
        s[1,2,0,0]=(-0.289167783943)*x[0]**o+(0.667637278664)*x[0]+(0.615088613098)*x[1]**o+(-0.771365681637)*x[1]
        s[1,2,0,1]=(0.554991016918)*x[0]**o+(-0.0658224324202)*x[0]+(0.305877722926)*x[1]**o+(-0.410754491717)*x[1]
        s[1,2,1,0]=(0.131109587703)*x[0]**o+(-0.589856526298)*x[0]+(-0.457418380817)*x[1]**o+(-0.766026536542)*x[1]
        s[1,2,1,1]=(-0.339196055896)*x[0]**o+(0.0879106255633)*x[0]+(-0.300776404484)*x[1]**o+(-0.855785910082)*x[1]
        s[1,2,2,0]=(0.169433038681)*x[0]**o+(0.900452320033)*x[0]+(-0.515800574208)*x[1]**o+(0.0353282676014)*x[1]
        s[1,2,2,1]=(0.120477320496)*x[0]**o+(0.544656714229)*x[0]+(0.712210191539)*x[1]**o+(-0.835300413971)*x[1]
        s[1,3,0,0]=(-0.144180890917)*x[0]**o+(0.847389690669)*x[0]+(0.21796553543)*x[1]**o+(0.0580182644011)*x[1]
        s[1,3,0,1]=(0.0202759559404)*x[0]**o+(-0.832118697631)*x[0]+(0.860042691805)*x[1]**o+(0.954802209319)*x[1]
        s[1,3,1,0]=(-0.232327132843)*x[0]**o+(0.988485328327)*x[0]+(-0.941830133542)*x[1]**o+(0.796919796173)*x[1]
        s[1,3,1,1]=(-0.0528857772705)*x[0]**o+(0.24536410605)*x[0]+(-0.89565411974)*x[1]**o+(0.473315809996)*x[1]
        s[1,3,2,0]=(0.920179439848)*x[0]**o+(0.415154298944)*x[0]+(0.690915792422)*x[1]**o+(0.0956157306525)*x[1]
        s[1,3,2,1]=(-0.332235130954)*x[0]**o+(0.756254925963)*x[0]+(0.936945508353)*x[1]**o+(0.56286108057)*x[1]
        s[1,4,0,0]=(0.802947524473)*x[0]**o+(0.764953990189)*x[0]+(0.58992910698)*x[1]**o+(0.807094950185)*x[1]
        s[1,4,0,1]=(-0.309951166637)*x[0]**o+(-0.605960228686)*x[0]+(0.0223945031407)*x[1]**o+(-0.230764354345)*x[1]
        s[1,4,1,0]=(0.856801428802)*x[0]**o+(0.748897933626)*x[0]+(0.0648717388846)*x[1]**o+(-0.197803227055)*x[1]
        s[1,4,1,1]=(0.160470457446)*x[0]**o+(0.466029696285)*x[0]+(-0.808841434426)*x[1]**o+(-0.890661270209)*x[1]
        s[1,4,2,0]=(0.316160329076)*x[0]**o+(-0.616711166445)*x[0]+(0.176082750216)*x[1]**o+(-0.389156386122)*x[1]
        s[1,4,2,1]=(0.966674586588)*x[0]**o+(0.743418831809)*x[0]+(0.828814302078)*x[1]**o+(0.443124030799)*x[1]
        s[2,0,0,0]=(0.895605391818)*x[0]**o+(-0.712359772806)*x[0]+(-0.254420890563)*x[1]**o+(0.04060068125)*x[1]
        s[2,0,0,1]=(0.407669194399)*x[0]**o+(-0.561911820032)*x[0]+(0.759519426562)*x[1]**o+(-0.791065777521)*x[1]
        s[2,0,1,0]=(0.0220907964237)*x[0]**o+(0.764347460016)*x[0]+(-0.894211196035)*x[1]**o+(-0.766734523336)*x[1]
        s[2,0,1,1]=(0.276708235973)*x[0]**o+(0.902519459875)*x[0]+(0.957736485845)*x[1]**o+(0.861836071217)*x[1]
        s[2,0,2,0]=(0.209485180668)*x[0]**o+(0.463649158628)*x[0]+(-0.0521057130471)*x[1]**o+(-0.64350767543)*x[1]
        s[2,0,2,1]=(-0.605362744895)*x[0]**o+(-0.131055561812)*x[0]+(-0.0963734475683)*x[1]**o+(0.38191997821)*x[1]
        s[2,1,0,0]=(0.242435009983)*x[0]**o+(-0.0678120221956)*x[0]+(-0.981711153358)*x[1]**o+(0.799273232234)*x[1]
        s[2,1,0,1]=(-0.178432239175)*x[0]**o+(-0.895623369837)*x[0]+(-0.572639180822)*x[1]**o+(-0.83834422153)*x[1]
        s[2,1,1,0]=(0.993103298694)*x[0]**o+(0.728567708683)*x[0]+(-0.799812986792)*x[1]**o+(-0.057428598921)*x[1]
        s[2,1,1,1]=(-0.776745418541)*x[0]**o+(-0.338983048206)*x[0]+(-0.565511785334)*x[1]**o+(-0.308717365714)*x[1]
        s[2,1,2,0]=(-0.199225286482)*x[0]**o+(-0.158190421122)*x[0]+(0.150112974223)*x[1]**o+(0.954346438264)*x[1]
        s[2,1,2,1]=(-0.402303812441)*x[0]**o+(-0.874974485349)*x[0]+(0.680761341229)*x[1]**o+(-0.694296934266)*x[1]
        s[2,2,0,0]=(0.0582479020766)*x[0]**o+(-0.562861368267)*x[0]+(0.757525601337)*x[1]**o+(0.932167579204)*x[1]
        s[2,2,0,1]=(-0.379424441757)*x[0]**o+(0.177098262781)*x[0]+(-0.485467998718)*x[1]**o+(-0.17324576024)*x[1]
        s[2,2,1,0]=(-0.339388895182)*x[0]**o+(0.491140214169)*x[0]+(0.471276867115)*x[1]**o+(0.292216420243)*x[1]
        s[2,2,1,1]=(0.502027084477)*x[0]**o+(0.927860130016)*x[0]+(0.787575747782)*x[1]**o+(-0.470519713839)*x[1]
        s[2,2,2,0]=(-0.583469643173)*x[0]**o+(-0.765710060231)*x[0]+(-0.54135991724)*x[1]**o+(0.665415514934)*x[1]
        s[2,2,2,1]=(-0.472362804158)*x[0]**o+(-0.766567819149)*x[0]+(0.201380986075)*x[1]**o+(0.270890088749)*x[1]
        s[2,3,0,0]=(-0.547838738364)*x[0]**o+(0.373177565132)*x[0]+(-0.921379470525)*x[1]**o+(0.0837731390271)*x[1]
        s[2,3,0,1]=(-0.557990737044)*x[0]**o+(-0.561311443234)*x[0]+(-0.924235646481)*x[1]**o+(-0.746081692868)*x[1]
        s[2,3,1,0]=(0.566861818147)*x[0]**o+(-0.107222369398)*x[0]+(-0.53810139661)*x[1]**o+(-0.814508288518)*x[1]
        s[2,3,1,1]=(-0.0578464759524)*x[0]**o+(0.525704186493)*x[0]+(0.434855750568)*x[1]**o+(-0.761515114625)*x[1]
        s[2,3,2,0]=(0.388204865306)*x[0]**o+(-0.320298614451)*x[0]+(0.0228508430597)*x[1]**o+(-0.436245057461)*x[1]
        s[2,3,2,1]=(0.70625829226)*x[0]**o+(-0.459381956838)*x[0]+(-0.756416817999)*x[1]**o+(0.963958503587)*x[1]
        s[2,4,0,0]=(0.595101935764)*x[0]**o+(0.0182198304835)*x[0]+(0.389172632633)*x[1]**o+(0.571052531257)*x[1]
        s[2,4,0,1]=(-0.59261552462)*x[0]**o+(0.176171578479)*x[0]+(-0.481303115308)*x[1]**o+(0.476762838828)*x[1]
        s[2,4,1,0]=(-0.274360684371)*x[0]**o+(0.496939161979)*x[0]+(0.0982270602491)*x[1]**o+(0.0521672721299)*x[1]
        s[2,4,1,1]=(0.0022428239189)*x[0]**o+(-0.926985821019)*x[0]+(0.0530228030052)*x[1]**o+(0.403935928384)*x[1]
        s[2,4,2,0]=(-0.96226874392)*x[0]**o+(0.282033354362)*x[0]+(0.917864727482)*x[1]**o+(0.32541112383)*x[1]
        s[2,4,2,1]=(0.684506236166)*x[0]**o+(0.215015538573)*x[0]+(0.74280905237)*x[1]**o+(0.781203116718)*x[1]
        s[3,0,0,0]=(0.658782504284)*x[0]**o+(0.396806335664)*x[0]+(0.0780367521917)*x[1]**o+(0.533907837354)*x[1]
        s[3,0,0,1]=(0.386822554052)*x[0]**o+(-0.951475963073)*x[0]+(0.992735332775)*x[1]**o+(-0.158295718477)*x[1]
        s[3,0,1,0]=(-0.253862856162)*x[0]**o+(-0.0131419224769)*x[0]+(0.810452271623)*x[1]**o+(-0.116490242053)*x[1]
        s[3,0,1,1]=(-0.297107568873)*x[0]**o+(0.339277467727)*x[0]+(-0.0455924137238)*x[1]**o+(0.408214745405)*x[1]
        s[3,0,2,0]=(0.101103302929)*x[0]**o+(0.84974169826)*x[0]+(0.668273722779)*x[1]**o+(-0.134494535453)*x[1]
        s[3,0,2,1]=(0.144907392006)*x[0]**o+(0.59561768643)*x[0]+(0.808003987088)*x[1]**o+(0.442993444756)*x[1]
        s[3,1,0,0]=(0.050719218416)*x[0]**o+(-0.205363570154)*x[0]+(-0.267767797056)*x[1]**o+(0.473617859576)*x[1]
        s[3,1,0,1]=(-0.488804469299)*x[0]**o+(-0.108366154443)*x[0]+(0.754051956614)*x[1]**o+(0.601979535209)*x[1]
        s[3,1,1,0]=(0.665579804971)*x[0]**o+(-0.820977378045)*x[0]+(-0.99768277192)*x[1]**o+(-0.553278556669)*x[1]
        s[3,1,1,1]=(0.28421003692)*x[0]**o+(0.34932169662)*x[0]+(-0.942736776261)*x[1]**o+(-0.761313236403)*x[1]
        s[3,1,2,0]=(-0.271033791176)*x[0]**o+(-0.269316831976)*x[0]+(0.637503921768)*x[1]**o+(0.550986595411)*x[1]
        s[3,1,2,1]=(-0.314094366001)*x[0]**o+(0.771643852473)*x[0]+(-0.370702666003)*x[1]**o+(0.925989724825)*x[1]
        s[3,2,0,0]=(0.608693141558)*x[0]**o+(-0.865838490365)*x[0]+(0.390277717008)*x[1]**o+(0.651999980469)*x[1]
        s[3,2,0,1]=(-0.911099837761)*x[0]**o+(0.307504460716)*x[0]+(0.82234563191)*x[1]**o+(-0.565850042096)*x[1]
        s[3,2,1,0]=(-0.876634035391)*x[0]**o+(0.410910752799)*x[0]+(-0.578952272394)*x[1]**o+(0.204714780618)*x[1]
        s[3,2,1,1]=(0.06295832442)*x[0]**o+(0.441355368057)*x[0]+(0.767590856273)*x[1]**o+(-0.67580811237)*x[1]
        s[3,2,2,0]=(-0.186380273058)*x[0]**o+(-0.891590560039)*x[0]+(-0.957486087526)*x[1]**o+(0.198598656166)*x[1]
        s[3,2,2,1]=(-0.733041869578)*x[0]**o+(-0.369283091906)*x[0]+(-0.103339143054)*x[1]**o+(-0.5468960827)*x[1]
        s[3,3,0,0]=(0.381695999815)*x[0]**o+(-0.398360040383)*x[0]+(-0.00816783658019)*x[1]**o+(0.209734829036)*x[1]
        s[3,3,0,1]=(-0.458999917243)*x[0]**o+(0.693814798148)*x[0]+(0.988023235161)*x[1]**o+(0.249848645187)*x[1]
        s[3,3,1,0]=(0.35272081264)*x[0]**o+(-0.24196842916)*x[0]+(-0.376576617413)*x[1]**o+(-0.512660897623)*x[1]
        s[3,3,1,1]=(-0.880579928879)*x[0]**o+(-0.901266101944)*x[0]+(-0.6695966178)*x[1]**o+(0.81462165322)*x[1]
        s[3,3,2,0]=(0.179006063683)*x[0]**o+(-0.772199368214)*x[0]+(0.173717958996)*x[1]**o+(0.943451083071)*x[1]
        s[3,3,2,1]=(0.772059778005)*x[0]**o+(-0.816863867151)*x[0]+(0.931619586753)*x[1]**o+(0.618296473051)*x[1]
        s[3,4,0,0]=(0.609550545616)*x[0]**o+(-0.226822163333)*x[0]+(0.498238525714)*x[1]**o+(0.145755694026)*x[1]
        s[3,4,0,1]=(0.925097562117)*x[0]**o+(0.793877528837)*x[0]+(0.483353477991)*x[1]**o+(-0.547313214969)*x[1]
        s[3,4,1,0]=(-0.878072917122)*x[0]**o+(0.703236130488)*x[0]+(-0.511134933342)*x[1]**o+(0.676303306122)*x[1]
        s[3,4,1,1]=(0.0745582123775)*x[0]**o+(0.668129797141)*x[0]+(-0.493968471165)*x[1]**o+(-0.26653808146)*x[1]
        s[3,4,2,0]=(-0.736620884918)*x[0]**o+(0.0401872117)*x[0]+(-0.312111305637)*x[1]**o+(0.811490469583)*x[1]
        s[3,4,2,1]=(-0.663426736678)*x[0]**o+(-0.630806839416)*x[0]+(0.638515711599)*x[1]**o+(-0.386941335499)*x[1]
        ref[0,0,0,0]=(0.685908691403)/(o+1.)+(0.161879023844)+(-0.135767714785)*0.5**o
        ref[0,0,0,1]=(0.964032895202)/(o+1.)+(0.071905497669)+(0.548074276718)*0.5**o
        ref[0,0,1,0]=(-0.398340925709)/(o+1.)+(0.270799177606)+(0.00388957848949)*0.5**o
        ref[0,0,1,1]=(0.766835159085)/(o+1.)+(-0.245411046684)+(0.896772134786)*0.5**o
        ref[0,0,2,0]=(-0.745694059134)/(o+1.)+(0.147526122043)+(0.455765211649)*0.5**o
        ref[0,0,2,1]=(0.499311653339)/(o+1.)+(-0.668097644737)+(-0.683685778458)*0.5**o
        ref[0,1,0,0]=(-0.565273192791)/(o+1.)+(0.402840360625)+(0.244426126288)*0.5**o
        ref[0,1,0,1]=(0.0551381505624)/(o+1.)+(-0.0281858023555)+(-0.0873578866485)*0.5**o
        ref[0,1,1,0]=(-0.784267376644)/(o+1.)+(-0.399087672201)+(0.0886184124686)*0.5**o
        ref[0,1,1,1]=(-0.615643115793)/(o+1.)+(-0.443602377081)+(-0.901203063708)*0.5**o
        ref[0,1,2,0]=(0.00527147896827)/(o+1.)+(0.278550943588)+(0.418411748143)*0.5**o
        ref[0,1,2,1]=(-0.8192829244)/(o+1.)+(0.363032218451)+(0.574446727172)*0.5**o
        ref[0,2,0,0]=(-0.694637110589)/(o+1.)+(-0.354337193875)+(-0.324743695914)*0.5**o
        ref[0,2,0,1]=(-0.260263189341)/(o+1.)+(-0.513268408944)+(0.538239581284)*0.5**o
        ref[0,2,1,0]=(-0.351338268294)/(o+1.)+(-0.273305954728)+(-0.535444326182)*0.5**o
        ref[0,2,1,1]=(-0.373848162528)/(o+1.)+(0.40471341339)+(-0.754984712928)*0.5**o
        ref[0,2,2,0]=(0.309106041559)/(o+1.)+(-0.19900818237)+(0.101157816757)*0.5**o
        ref[0,2,2,1]=(-0.155113200571)/(o+1.)+(0.142307310862)+(-0.537633855745)*0.5**o
        ref[0,3,0,0]=(0.491019303223)/(o+1.)+(0.0348536166248)+(0.466579973711)*0.5**o
        ref[0,3,0,1]=(-0.660374384585)/(o+1.)+(0.153406649709)+(0.202651138329)*0.5**o
        ref[0,3,1,0]=(-0.952344452735)/(o+1.)+(0.330219918513)+(0.442171269159)*0.5**o
        ref[0,3,1,1]=(-0.305478230795)/(o+1.)+(-0.129310758573)+(0.733989617351)*0.5**o
        ref[0,3,2,0]=(-0.374724207394)/(o+1.)+(-0.676853212765)+(-0.709003057494)*0.5**o
        ref[0,3,2,1]=(0.0283773197868)/(o+1.)+(-0.577644044334)+(-0.125623348142)*0.5**o
        ref[0,4,0,0]=(0.582676151383)/(o+1.)+(-0.119010697958)+(0.490207583728)*0.5**o
        ref[0,4,0,1]=(0.576928080618)/(o+1.)+(-0.726918954502)+(-0.605187460486)*0.5**o
        ref[0,4,1,0]=(0.634879197271)/(o+1.)+(-0.270015002926)+(-0.0850932164524)*0.5**o
        ref[0,4,1,1]=(-0.976313171743)/(o+1.)+(0.061680269884)+(0.258713682268)*0.5**o
        ref[0,4,2,0]=(0.153793647237)/(o+1.)+(-0.275448145163)+(-0.350506879421)*0.5**o
        ref[0,4,2,1]=(-0.769374193622)/(o+1.)+(-0.212619295773)+(0.671395932648)*0.5**o
        ref[1,0,0,0]=(0.307120506561)/(o+1.)+(0.0508338052288)+(0.188023378319)*0.5**o
        ref[1,0,0,1]=(-0.411140279426)/(o+1.)+(-0.155962049441)+(0.800293252159)*0.5**o
        ref[1,0,1,0]=(-0.404819325345)/(o+1.)+(-0.755138920507)+(-0.515339883608)*0.5**o
        ref[1,0,1,1]=(-0.586954384255)/(o+1.)+(-0.587678866541)+(0.0512515899156)*0.5**o
        ref[1,0,2,0]=(0.166288186202)/(o+1.)+(-0.528326497368)+(-0.581670807515)*0.5**o
        ref[1,0,2,1]=(-0.698802363494)/(o+1.)+(0.0176333617287)+(-0.634327645938)*0.5**o
        ref[1,1,0,0]=(-0.381545673134)/(o+1.)+(0.292768032162)+(0.434696675884)*0.5**o
        ref[1,1,0,1]=(-0.654368291721)/(o+1.)+(0.223085474382)+(0.438065102364)*0.5**o
        ref[1,1,1,0]=(0.670405216194)/(o+1.)+(-0.413864857186)+(0.812437488024)*0.5**o
        ref[1,1,1,1]=(0.0351984047833)/(o+1.)+(0.379018653249)+(0.52676928941)*0.5**o
        ref[1,1,2,0]=(0.210182888846)/(o+1.)+(0.343816040389)+(-0.0967009361026)*0.5**o
        ref[1,1,2,1]=(-0.107282780635)/(o+1.)+(0.629761934716)+(-0.664016192839)*0.5**o
        ref[1,2,0,0]=(0.615088613098)/(o+1.)+(-0.0518642014867)+(-0.289167783943)*0.5**o
        ref[1,2,0,1]=(0.305877722926)/(o+1.)+(-0.238288462068)+(0.554991016918)*0.5**o
        ref[1,2,1,0]=(-0.457418380817)/(o+1.)+(-0.67794153142)+(0.131109587703)*0.5**o
        ref[1,2,1,1]=(-0.300776404484)/(o+1.)+(-0.38393764226)+(-0.339196055896)*0.5**o
        ref[1,2,2,0]=(-0.515800574208)/(o+1.)+(0.467890293817)+(0.169433038681)*0.5**o
        ref[1,2,2,1]=(0.712210191539)/(o+1.)+(-0.145321849871)+(0.120477320496)*0.5**o
        ref[1,3,0,0]=(0.21796553543)/(o+1.)+(0.452703977535)+(-0.144180890917)*0.5**o
        ref[1,3,0,1]=(0.860042691805)/(o+1.)+(0.0613417558442)+(0.0202759559404)*0.5**o
        ref[1,3,1,0]=(-0.941830133542)/(o+1.)+(0.89270256225)+(-0.232327132843)*0.5**o
        ref[1,3,1,1]=(-0.89565411974)/(o+1.)+(0.359339958023)+(-0.0528857772705)*0.5**o
        ref[1,3,2,0]=(0.690915792422)/(o+1.)+(0.255385014798)+(0.920179439848)*0.5**o
        ref[1,3,2,1]=(0.936945508353)/(o+1.)+(0.659558003266)+(-0.332235130954)*0.5**o
        ref[1,4,0,0]=(0.58992910698)/(o+1.)+(0.786024470187)+(0.802947524473)*0.5**o
        ref[1,4,0,1]=(0.0223945031407)/(o+1.)+(-0.418362291515)+(-0.309951166637)*0.5**o
        ref[1,4,1,0]=(0.0648717388846)/(o+1.)+(0.275547353285)+(0.856801428802)*0.5**o
        ref[1,4,1,1]=(-0.808841434426)/(o+1.)+(-0.212315786962)+(0.160470457446)*0.5**o
        ref[1,4,2,0]=(0.176082750216)/(o+1.)+(-0.502933776283)+(0.316160329076)*0.5**o
        ref[1,4,2,1]=(0.828814302078)/(o+1.)+(0.593271431304)+(0.966674586588)*0.5**o
        ref[2,0,0,0]=(-0.254420890563)/(o+1.)+(-0.335879545778)+(0.895605391818)*0.5**o
        ref[2,0,0,1]=(0.759519426562)/(o+1.)+(-0.676488798776)+(0.407669194399)*0.5**o
        ref[2,0,1,0]=(-0.894211196035)/(o+1.)+(-0.00119353166009)+(0.0220907964237)*0.5**o
        ref[2,0,1,1]=(0.957736485845)/(o+1.)+(0.882177765546)+(0.276708235973)*0.5**o
        ref[2,0,2,0]=(-0.0521057130471)/(o+1.)+(-0.0899292584008)+(0.209485180668)*0.5**o
        ref[2,0,2,1]=(-0.0963734475683)/(o+1.)+(0.125432208199)+(-0.605362744895)*0.5**o
        ref[2,1,0,0]=(-0.981711153358)/(o+1.)+(0.365730605019)+(0.242435009983)*0.5**o
        ref[2,1,0,1]=(-0.572639180822)/(o+1.)+(-0.866983795684)+(-0.178432239175)*0.5**o
        ref[2,1,1,0]=(-0.799812986792)/(o+1.)+(0.335569554881)+(0.993103298694)*0.5**o
        ref[2,1,1,1]=(-0.565511785334)/(o+1.)+(-0.32385020696)+(-0.776745418541)*0.5**o
        ref[2,1,2,0]=(0.150112974223)/(o+1.)+(0.398078008571)+(-0.199225286482)*0.5**o
        ref[2,1,2,1]=(0.680761341229)/(o+1.)+(-0.784635709807)+(-0.402303812441)*0.5**o
        ref[2,2,0,0]=(0.757525601337)/(o+1.)+(0.184653105468)+(0.0582479020766)*0.5**o
        ref[2,2,0,1]=(-0.485467998718)/(o+1.)+(0.00192625127016)+(-0.379424441757)*0.5**o
        ref[2,2,1,0]=(0.471276867115)/(o+1.)+(0.391678317206)+(-0.339388895182)*0.5**o
        ref[2,2,1,1]=(0.787575747782)/(o+1.)+(0.228670208089)+(0.502027084477)*0.5**o
        ref[2,2,2,0]=(-0.54135991724)/(o+1.)+(-0.0501472726484)+(-0.583469643173)*0.5**o
        ref[2,2,2,1]=(0.201380986075)/(o+1.)+(-0.2478388652)+(-0.472362804158)*0.5**o
        ref[2,3,0,0]=(-0.921379470525)/(o+1.)+(0.228475352079)+(-0.547838738364)*0.5**o
        ref[2,3,0,1]=(-0.924235646481)/(o+1.)+(-0.653696568051)+(-0.557990737044)*0.5**o
        ref[2,3,1,0]=(-0.53810139661)/(o+1.)+(-0.460865328958)+(0.566861818147)*0.5**o
        ref[2,3,1,1]=(0.434855750568)/(o+1.)+(-0.117905464066)+(-0.0578464759524)*0.5**o
        ref[2,3,2,0]=(0.0228508430597)/(o+1.)+(-0.378271835956)+(0.388204865306)*0.5**o
        ref[2,3,2,1]=(-0.756416817999)/(o+1.)+(0.252288273374)+(0.70625829226)*0.5**o
        ref[2,4,0,0]=(0.389172632633)/(o+1.)+(0.29463618087)+(0.595101935764)*0.5**o
        ref[2,4,0,1]=(-0.481303115308)/(o+1.)+(0.326467208654)+(-0.59261552462)*0.5**o
        ref[2,4,1,0]=(0.0982270602491)/(o+1.)+(0.274553217055)+(-0.274360684371)*0.5**o
        ref[2,4,1,1]=(0.0530228030052)/(o+1.)+(-0.261524946318)+(0.0022428239189)*0.5**o
        ref[2,4,2,0]=(0.917864727482)/(o+1.)+(0.303722239096)+(-0.96226874392)*0.5**o
        ref[2,4,2,1]=(0.74280905237)/(o+1.)+(0.498109327645)+(0.684506236166)*0.5**o
        ref[3,0,0,0]=(0.0780367521917)/(o+1.)+(0.465357086509)+(0.658782504284)*0.5**o
        ref[3,0,0,1]=(0.992735332775)/(o+1.)+(-0.554885840775)+(0.386822554052)*0.5**o
        ref[3,0,1,0]=(0.810452271623)/(o+1.)+(-0.0648160822652)+(-0.253862856162)*0.5**o
        ref[3,0,1,1]=(-0.0455924137238)/(o+1.)+(0.373746106566)+(-0.297107568873)*0.5**o
        ref[3,0,2,0]=(0.668273722779)/(o+1.)+(0.357623581403)+(0.101103302929)*0.5**o
        ref[3,0,2,1]=(0.808003987088)/(o+1.)+(0.519305565593)+(0.144907392006)*0.5**o
        ref[3,1,0,0]=(-0.267767797056)/(o+1.)+(0.134127144711)+(0.050719218416)*0.5**o
        ref[3,1,0,1]=(0.754051956614)/(o+1.)+(0.246806690383)+(-0.488804469299)*0.5**o
        ref[3,1,1,0]=(-0.99768277192)/(o+1.)+(-0.687127967357)+(0.665579804971)*0.5**o
        ref[3,1,1,1]=(-0.942736776261)/(o+1.)+(-0.205995769892)+(0.28421003692)*0.5**o
        ref[3,1,2,0]=(0.637503921768)/(o+1.)+(0.140834881718)+(-0.271033791176)*0.5**o
        ref[3,1,2,1]=(-0.370702666003)/(o+1.)+(0.848816788649)+(-0.314094366001)*0.5**o
        ref[3,2,0,0]=(0.390277717008)/(o+1.)+(-0.106919254948)+(0.608693141558)*0.5**o
        ref[3,2,0,1]=(0.82234563191)/(o+1.)+(-0.12917279069)+(-0.911099837761)*0.5**o
        ref[3,2,1,0]=(-0.578952272394)/(o+1.)+(0.307812766709)+(-0.876634035391)*0.5**o
        ref[3,2,1,1]=(0.767590856273)/(o+1.)+(-0.117226372156)+(0.06295832442)*0.5**o
        ref[3,2,2,0]=(-0.957486087526)/(o+1.)+(-0.346495951936)+(-0.186380273058)*0.5**o
        ref[3,2,2,1]=(-0.103339143054)/(o+1.)+(-0.458089587303)+(-0.733041869578)*0.5**o
        ref[3,3,0,0]=(-0.00816783658019)/(o+1.)+(-0.0943126056733)+(0.381695999815)*0.5**o
        ref[3,3,0,1]=(0.988023235161)/(o+1.)+(0.471831721667)+(-0.458999917243)*0.5**o
        ref[3,3,1,0]=(-0.376576617413)/(o+1.)+(-0.377314663391)+(0.35272081264)*0.5**o
        ref[3,3,1,1]=(-0.6695966178)/(o+1.)+(-0.043322224362)+(-0.880579928879)*0.5**o
        ref[3,3,2,0]=(0.173717958996)/(o+1.)+(0.0856258574287)+(0.179006063683)*0.5**o
        ref[3,3,2,1]=(0.931619586753)/(o+1.)+(-0.0992836970501)+(0.772059778005)*0.5**o
        ref[3,4,0,0]=(0.498238525714)/(o+1.)+(-0.0405332346536)+(0.609550545616)*0.5**o
        ref[3,4,0,1]=(0.483353477991)/(o+1.)+(0.123282156934)+(0.925097562117)*0.5**o
        ref[3,4,1,0]=(-0.511134933342)/(o+1.)+(0.689769718305)+(-0.878072917122)*0.5**o
        ref[3,4,1,1]=(-0.493968471165)/(o+1.)+(0.200795857841)+(0.0745582123775)*0.5**o
        ref[3,4,2,0]=(-0.312111305637)/(o+1.)+(0.425838840642)+(-0.736620884918)*0.5**o
        ref[3,4,2,1]=(0.638515711599)/(o+1.)+(-0.508874087457)+(-0.663426736678)*0.5**o
      else:
        s[0,0,0,0]=(-0.938544579906)*x[0]**o+(0.180067483394)*x[0]+(-0.719233529982)*x[1]**o+(0.788617757419)*x[1]+(0.556994205355)*x[2]**o+(-0.127037817454)*x[2]
        s[0,0,0,1]=(0.953579560099)*x[0]**o+(0.562347618447)*x[0]+(-0.100565718949)*x[1]**o+(-0.110294341288)*x[1]+(0.23833069113)*x[2]**o+(-0.762859899976)*x[2]
        s[0,0,1,0]=(-0.0146573390123)*x[0]**o+(0.610306989551)*x[0]+(-0.322740693558)*x[1]**o+(0.0287307185836)*x[1]+(0.464511767666)*x[2]**o+(0.609636002246)*x[2]
        s[0,0,1,1]=(-0.658403837278)*x[0]**o+(0.446799474387)*x[0]+(-0.240860101539)*x[1]**o+(-0.198397925626)*x[1]+(0.874003266701)*x[2]**o+(0.313133624565)*x[2]
        s[0,0,2,0]=(0.62578079836)*x[0]**o+(-0.0257593936528)*x[0]+(-0.60984452513)*x[1]**o+(0.635946424498)*x[1]+(-0.218596411854)*x[2]**o+(0.00763050290993)*x[2]
        s[0,0,2,1]=(0.314093680988)*x[0]**o+(-0.869278986994)*x[0]+(-0.651770479439)*x[1]**o+(0.202960060033)*x[1]+(0.382275480924)*x[2]**o+(-0.595512215137)*x[2]
        s[0,1,0,0]=(0.00620161785372)*x[0]**o+(0.876847803666)*x[0]+(0.520320206245)*x[1]**o+(0.228011364507)*x[1]+(0.634707030605)*x[2]**o+(0.750039227257)*x[2]
        s[0,1,0,1]=(0.0303982843122)*x[0]**o+(-0.843928008659)*x[0]+(0.175282997162)*x[1]**o+(0.628713373934)*x[1]+(-0.260785452479)*x[2]**o+(-0.768157994235)*x[2]
        s[0,1,1,0]=(0.310811455642)*x[0]**o+(-0.907217641071)*x[0]+(-0.982538210789)*x[1]**o+(-0.600525634532)*x[1]+(-0.864394932414)*x[2]**o+(0.458881514929)*x[2]
        s[0,1,1,1]=(-0.751411238653)*x[0]**o+(-0.640190422476)*x[0]+(-0.408377872461)*x[1]**o+(0.72944579088)*x[1]+(0.946023481982)*x[2]**o+(-0.778523105712)*x[2]
        s[0,1,2,0]=(-0.852982317135)*x[0]**o+(-0.770284090211)*x[0]+(-0.0651898089121)*x[1]**o+(0.570265822823)*x[1]+(0.28669660112)*x[2]**o+(0.0651543264368)*x[2]
        s[0,1,2,1]=(-0.973162329979)*x[0]**o+(-0.246891970345)*x[0]+(0.0766743752541)*x[1]**o+(-0.403798628829)*x[1]+(-0.643254181567)*x[2]**o+(-0.378598111893)*x[2]
        s[0,2,0,0]=(0.472595355875)*x[0]**o+(0.00120464758292)*x[0]+(-0.42244937935)*x[1]**o+(-0.9164663947)*x[1]+(-0.0845858448629)*x[2]**o+(0.72392919756)*x[2]
        s[0,2,0,1]=(0.486817890916)*x[0]**o+(-0.70489295232)*x[0]+(-0.817086734784)*x[1]**o+(0.645820907995)*x[1]+(-0.335350124515)*x[2]**o+(-0.32309401603)*x[2]
        s[0,2,1,0]=(0.832549390017)*x[0]**o+(-0.434882976553)*x[0]+(0.0796018608256)*x[1]**o+(-0.815177284988)*x[1]+(0.420723606982)*x[2]**o+(-0.996616855812)*x[2]
        s[0,2,1,1]=(0.152135328754)*x[0]**o+(0.119431209093)*x[0]+(-0.0565707514319)*x[1]**o+(-0.779402141878)*x[1]+(0.504116223295)*x[2]**o+(0.35421565275)*x[2]
        s[0,2,2,0]=(-0.358237320736)*x[0]**o+(0.486656376538)*x[0]+(-0.831122271401)*x[1]**o+(-0.0521486506858)*x[1]+(-0.782495078477)*x[2]**o+(0.856211020463)*x[2]
        s[0,2,2,1]=(-0.1707212422)*x[0]**o+(-0.798979986621)*x[0]+(0.882236543414)*x[1]**o+(0.377099798101)*x[1]+(0.489392175267)*x[2]**o+(0.930902331932)*x[2]
        s[0,3,0,0]=(-0.128307529775)*x[0]**o+(-0.846783963276)*x[0]+(0.946673346517)*x[1]**o+(-0.335187413972)*x[1]+(-0.408325612416)*x[2]**o+(0.629338432074)*x[2]
        s[0,3,0,1]=(-0.554435822439)*x[0]**o+(0.990524237381)*x[0]+(-0.537192696978)*x[1]**o+(-0.826994397531)*x[1]+(0.866744291726)*x[2]**o+(-0.285154443258)*x[2]
        s[0,3,1,0]=(-0.823501956971)*x[0]**o+(0.881638907149)*x[0]+(-0.0906956584345)*x[1]**o+(-0.902956378971)*x[1]+(-0.161189770505)*x[2]**o+(-0.160538443759)*x[2]
        s[0,3,1,1]=(0.430542618951)*x[0]**o+(-0.769516956782)*x[0]+(0.475479099202)*x[1]**o+(-0.917793889771)*x[1]+(0.763381631494)*x[2]**o+(0.346636926064)*x[2]
        s[0,3,2,0]=(-0.958502709531)*x[0]**o+(-0.177711289961)*x[0]+(0.586266935723)*x[1]**o+(0.890784623626)*x[1]+(-0.787544127664)*x[2]**o+(0.446360741993)*x[2]
        s[0,3,2,1]=(-0.792569155533)*x[0]**o+(0.863653363017)*x[0]+(0.308377143826)*x[1]**o+(-0.30341033471)*x[1]+(0.0641187823207)*x[2]**o+(0.52775945782)*x[2]
        s[0,4,0,0]=(-0.0338438096091)*x[0]**o+(-0.200503853968)*x[0]+(-0.540577100866)*x[1]**o+(-0.228756900341)*x[1]+(-0.388513893583)*x[2]**o+(0.134360460805)*x[2]
        s[0,4,0,1]=(-0.406152624356)*x[0]**o+(-0.925153078113)*x[0]+(-0.432893083261)*x[1]**o+(0.118318067951)*x[1]+(0.839662677949)*x[2]**o+(-0.0163758572356)*x[2]
        s[0,4,1,0]=(-0.799456787818)*x[0]**o+(-0.0974094140856)*x[0]+(-0.967532895407)*x[1]**o+(0.313503423815)*x[1]+(-0.255345608246)*x[2]**o+(0.0212515985458)*x[2]
        s[0,4,1,1]=(0.561141037244)*x[0]**o+(0.421885997959)*x[0]+(-0.951943006997)*x[1]**o+(-0.293583958606)*x[1]+(0.26979445144)*x[2]**o+(-0.678061968406)*x[2]
        s[0,4,2,0]=(-0.346440790531)*x[0]**o+(0.486394576884)*x[0]+(0.318188318547)*x[1]**o+(-0.635068217742)*x[1]+(-0.0718964652741)*x[2]**o+(-0.702552674876)*x[2]
        s[0,4,2,1]=(0.450982323218)*x[0]**o+(0.682381807031)*x[0]+(-0.808585522869)*x[1]**o+(0.0201338006052)*x[1]+(-0.480685256454)*x[2]**o+(0.137075879378)*x[2]
        s[1,0,0,0]=(0.00370747226485)*x[0]**o+(-0.0925055645922)*x[0]+(-0.917381168613)*x[1]**o+(-0.193114928297)*x[1]+(-0.00570424377166)*x[2]**o+(0.913135198646)*x[2]
        s[1,0,0,1]=(0.314513438122)*x[0]**o+(-0.479659124185)*x[0]+(0.42293180951)*x[1]**o+(-0.597525641033)*x[1]+(-0.417172553037)*x[2]**o+(0.499251735975)*x[2]
        s[1,0,1,0]=(0.904368787276)*x[0]**o+(-0.514051100643)*x[0]+(0.212361933482)*x[1]**o+(-0.445110428363)*x[1]+(0.221673520737)*x[2]**o+(-0.0710601569942)*x[2]
        s[1,0,1,1]=(-0.0645313731593)*x[0]**o+(0.263267052465)*x[0]+(-0.715349556062)*x[1]**o+(0.236872392267)*x[1]+(0.605241495355)*x[2]**o+(-0.152375732378)*x[2]
        s[1,0,2,0]=(0.721223990054)*x[0]**o+(-0.704265146199)*x[0]+(-0.586881850891)*x[1]**o+(0.425307854616)*x[1]+(0.33484446761)*x[2]**o+(-0.573115444223)*x[2]
        s[1,0,2,1]=(-0.928667989887)*x[0]**o+(-0.877606514031)*x[0]+(-0.403016486409)*x[1]**o+(-0.517530622207)*x[1]+(0.0254991827594)*x[2]**o+(-0.82134638059)*x[2]
        s[1,1,0,0]=(-0.598976144645)*x[0]**o+(-0.0568344921691)*x[0]+(0.944100507076)*x[1]**o+(0.636963899978)*x[1]+(0.253911413671)*x[2]**o+(-0.178511347672)*x[2]
        s[1,1,0,1]=(0.51758257139)*x[0]**o+(-0.826610314817)*x[0]+(0.0669491359674)*x[1]**o+(-0.236450271581)*x[1]+(-0.702832265335)*x[2]**o+(-0.268110228427)*x[2]
        s[1,1,1,0]=(-0.847292260359)*x[0]**o+(-0.382061283693)*x[0]+(0.85982617142)*x[1]**o+(0.0930902132719)*x[1]+(0.568918674)*x[2]**o+(-0.811808588084)*x[2]
        s[1,1,1,1]=(0.396422756052)*x[0]**o+(0.667551047328)*x[0]+(-0.174441000478)*x[1]**o+(-0.40295080705)*x[1]+(0.075193248527)*x[2]**o+(-0.258123308085)*x[2]
        s[1,1,2,0]=(0.391968381002)*x[0]**o+(0.277092735448)*x[0]+(0.72619079286)*x[1]**o+(0.336787672025)*x[1]+(0.622007882)*x[2]**o+(0.286619657146)*x[2]
        s[1,1,2,1]=(-0.212692240987)*x[0]**o+(0.786660459419)*x[0]+(-0.528445652722)*x[1]**o+(0.177147918886)*x[1]+(0.40734794393)*x[2]**o+(0.155376280493)*x[2]
        s[1,2,0,0]=(0.0157691022819)*x[0]**o+(-0.661679523157)*x[0]+(0.123711726993)*x[1]**o+(-0.0870910621738)*x[1]+(-0.865865892104)*x[2]**o+(0.999338641606)*x[2]
        s[1,2,0,1]=(-0.828402321641)*x[0]**o+(-0.528217338749)*x[0]+(0.723149264147)*x[1]**o+(0.752532155985)*x[1]+(0.443929080203)*x[2]**o+(-0.929781507182)*x[2]
        s[1,2,1,0]=(-0.375626403305)*x[0]**o+(-0.510977471867)*x[0]+(0.288802388225)*x[1]**o+(0.168184581445)*x[1]+(-0.985676398515)*x[2]**o+(-0.993246463175)*x[2]
        s[1,2,1,1]=(0.839254171086)*x[0]**o+(-0.663664452054)*x[0]+(0.990048916717)*x[1]**o+(-0.986318241383)*x[1]+(0.726312571574)*x[2]**o+(0.411365473711)*x[2]
        s[1,2,2,0]=(0.179198759222)*x[0]**o+(0.362349404368)*x[0]+(0.372503911575)*x[1]**o+(-0.885663720586)*x[1]+(-0.656319589417)*x[2]**o+(-0.1547676929)*x[2]
        s[1,2,2,1]=(0.431283737039)*x[0]**o+(0.573392031568)*x[0]+(-0.167580615772)*x[1]**o+(-0.427449288729)*x[1]+(-0.771868709235)*x[2]**o+(-0.272748901939)*x[2]
        s[1,3,0,0]=(-0.735181806951)*x[0]**o+(0.536012589912)*x[0]+(0.623248739525)*x[1]**o+(0.590921832149)*x[1]+(-0.21724838693)*x[2]**o+(0.95165954451)*x[2]
        s[1,3,0,1]=(0.953086533561)*x[0]**o+(0.855151309595)*x[0]+(-0.877649378409)*x[1]**o+(-0.79234800867)*x[1]+(-0.647423149686)*x[2]**o+(0.718784987888)*x[2]
        s[1,3,1,0]=(-0.723560393521)*x[0]**o+(-0.0864445384144)*x[0]+(0.852841011672)*x[1]**o+(-0.953082229559)*x[1]+(0.983938853476)*x[2]**o+(-0.195518521863)*x[2]
        s[1,3,1,1]=(-0.915440550473)*x[0]**o+(-0.92449299685)*x[0]+(-0.150518843387)*x[1]**o+(-0.66259291358)*x[1]+(-0.812332416698)*x[2]**o+(-0.362998001567)*x[2]
        s[1,3,2,0]=(-0.205303485441)*x[0]**o+(-0.44047413477)*x[0]+(0.563675490655)*x[1]**o+(-0.802613298805)*x[1]+(-0.384663506204)*x[2]**o+(0.881891732298)*x[2]
        s[1,3,2,1]=(0.542339109554)*x[0]**o+(0.459852398778)*x[0]+(0.531257436652)*x[1]**o+(-0.0918654697667)*x[1]+(0.728385543797)*x[2]**o+(-0.142579191699)*x[2]
        s[1,4,0,0]=(-0.444230035592)*x[0]**o+(0.0532350197552)*x[0]+(-0.280357698853)*x[1]**o+(0.00480903349516)*x[1]+(-0.147327835489)*x[2]**o+(-0.630977138596)*x[2]
        s[1,4,0,1]=(0.972127984006)*x[0]**o+(0.904605524232)*x[0]+(0.44361144766)*x[1]**o+(0.586027289354)*x[1]+(-0.194334042153)*x[2]**o+(0.477271167786)*x[2]
        s[1,4,1,0]=(-0.162531060958)*x[0]**o+(-0.186338052619)*x[0]+(-0.0696302540091)*x[1]**o+(0.143088503291)*x[1]+(0.0107879016639)*x[2]**o+(0.39899128749)*x[2]
        s[1,4,1,1]=(0.315463685116)*x[0]**o+(-0.290348790738)*x[0]+(-0.292829602611)*x[1]**o+(-0.272218595021)*x[1]+(-0.247363514071)*x[2]**o+(0.830943482915)*x[2]
        s[1,4,2,0]=(0.340729082172)*x[0]**o+(0.266345838644)*x[0]+(0.205349295082)*x[1]**o+(-0.0281621785636)*x[1]+(0.89948000824)*x[2]**o+(-0.764082368496)*x[2]
        s[1,4,2,1]=(0.7615004712)*x[0]**o+(-0.366790213041)*x[0]+(-0.347719278119)*x[1]**o+(-0.664644802127)*x[1]+(0.153340126952)*x[2]**o+(-0.699790837984)*x[2]
        s[2,0,0,0]=(0.0125409231833)*x[0]**o+(-0.497512034949)*x[0]+(0.688244767493)*x[1]**o+(-0.160005018143)*x[1]+(-0.119715348465)*x[2]**o+(-0.797969768194)*x[2]
        s[2,0,0,1]=(-0.0036047535766)*x[0]**o+(0.726289674137)*x[0]+(-0.110716355541)*x[1]**o+(-0.937117263903)*x[1]+(0.540476278776)*x[2]**o+(-0.0254788538753)*x[2]
        s[2,0,1,0]=(0.0494808240706)*x[0]**o+(-0.469224784419)*x[0]+(0.599591744744)*x[1]**o+(-0.694323071412)*x[1]+(-0.584890416602)*x[2]**o+(0.795168667963)*x[2]
        s[2,0,1,1]=(-0.53303893512)*x[0]**o+(0.012157220346)*x[0]+(-0.546109156126)*x[1]**o+(-0.719758078719)*x[1]+(-0.496492728702)*x[2]**o+(0.816565066681)*x[2]
        s[2,0,2,0]=(-0.306356882566)*x[0]**o+(0.501809420086)*x[0]+(0.171130235154)*x[1]**o+(-0.804095060824)*x[1]+(0.929639721152)*x[2]**o+(-0.516285706731)*x[2]
        s[2,0,2,1]=(0.337441903133)*x[0]**o+(-0.524705933088)*x[0]+(0.490789790623)*x[1]**o+(0.0420695613636)*x[1]+(-0.296688134663)*x[2]**o+(-0.508555097135)*x[2]
        s[2,1,0,0]=(0.984282580742)*x[0]**o+(0.692515911833)*x[0]+(0.610802160351)*x[1]**o+(0.174938187813)*x[1]+(0.777154457482)*x[2]**o+(-0.296265352251)*x[2]
        s[2,1,0,1]=(0.263434795622)*x[0]**o+(0.146263475693)*x[0]+(-0.135060978)*x[1]**o+(-0.612057700058)*x[1]+(0.401206478053)*x[2]**o+(0.614641881217)*x[2]
        s[2,1,1,0]=(0.839155090264)*x[0]**o+(0.893746351779)*x[0]+(0.652777744086)*x[1]**o+(-0.119538495003)*x[1]+(-0.116919455352)*x[2]**o+(-0.24237363241)*x[2]
        s[2,1,1,1]=(-0.626863009974)*x[0]**o+(-0.328592553389)*x[0]+(-0.483755510715)*x[1]**o+(0.506129888719)*x[1]+(-0.369314928449)*x[2]**o+(-0.974391328259)*x[2]
        s[2,1,2,0]=(-0.548834239805)*x[0]**o+(-0.137282729528)*x[0]+(0.980067336071)*x[1]**o+(-0.677019881527)*x[1]+(-0.633352248762)*x[2]**o+(0.394845574769)*x[2]
        s[2,1,2,1]=(0.304097820123)*x[0]**o+(0.205705406237)*x[0]+(0.431075221394)*x[1]**o+(0.965948087934)*x[1]+(0.797211333629)*x[2]**o+(-0.750481732005)*x[2]
        s[2,2,0,0]=(0.00294465132151)*x[0]**o+(-0.36531365965)*x[0]+(-0.477095178168)*x[1]**o+(0.640123142737)*x[1]+(0.307285555133)*x[2]**o+(-0.535459483965)*x[2]
        s[2,2,0,1]=(0.185831262234)*x[0]**o+(0.680621489598)*x[0]+(-0.606597858106)*x[1]**o+(0.605582792294)*x[1]+(0.67028817128)*x[2]**o+(0.0936479735813)*x[2]
        s[2,2,1,0]=(0.234717477502)*x[0]**o+(0.739313223999)*x[0]+(-0.765253630778)*x[1]**o+(-0.199364440863)*x[1]+(-0.10386639265)*x[2]**o+(-0.772307849674)*x[2]
        s[2,2,1,1]=(0.692290926695)*x[0]**o+(-0.841554910484)*x[0]+(-0.156803726414)*x[1]**o+(0.656797138877)*x[1]+(0.4668832013)*x[2]**o+(-0.320142791051)*x[2]
        s[2,2,2,0]=(-0.408729971023)*x[0]**o+(0.28023578811)*x[0]+(0.756339486697)*x[1]**o+(0.130791597709)*x[1]+(-0.388130026173)*x[2]**o+(-0.232133676528)*x[2]
        s[2,2,2,1]=(-0.30215609634)*x[0]**o+(-0.515622149267)*x[0]+(-0.979560898475)*x[1]**o+(0.424855014788)*x[1]+(-0.0265200291972)*x[2]**o+(0.244204914292)*x[2]
        s[2,3,0,0]=(-0.731783739254)*x[0]**o+(-0.910917581289)*x[0]+(-0.490859941251)*x[1]**o+(0.265521242704)*x[1]+(0.246576247389)*x[2]**o+(0.551033884684)*x[2]
        s[2,3,0,1]=(0.353316440436)*x[0]**o+(0.607852620987)*x[0]+(0.716634525835)*x[1]**o+(0.931269029696)*x[1]+(0.512042632855)*x[2]**o+(0.84647829057)*x[2]
        s[2,3,1,0]=(-0.682583926526)*x[0]**o+(0.0414372583894)*x[0]+(-0.368919279038)*x[1]**o+(0.263442044187)*x[1]+(0.651154272252)*x[2]**o+(0.625505230753)*x[2]
        s[2,3,1,1]=(-0.773410808141)*x[0]**o+(-0.308532872412)*x[0]+(0.484022099132)*x[1]**o+(0.748310916514)*x[1]+(0.484822489595)*x[2]**o+(0.0919219970104)*x[2]
        s[2,3,2,0]=(-0.856792343472)*x[0]**o+(0.475526910813)*x[0]+(0.858286241213)*x[1]**o+(0.765684964483)*x[1]+(0.842680693423)*x[2]**o+(0.759239366541)*x[2]
        s[2,3,2,1]=(0.208814714888)*x[0]**o+(-0.319616331598)*x[0]+(-0.786662106876)*x[1]**o+(-0.742419573118)*x[1]+(-0.671654532185)*x[2]**o+(-0.81098024788)*x[2]
        s[2,4,0,0]=(0.0581246014334)*x[0]**o+(-0.738212823784)*x[0]+(0.42828697377)*x[1]**o+(-0.795604474847)*x[1]+(-0.867930879138)*x[2]**o+(0.897777664383)*x[2]
        s[2,4,0,1]=(0.658005351748)*x[0]**o+(0.318056754281)*x[0]+(-0.241917242833)*x[1]**o+(0.321210074114)*x[1]+(0.341921510554)*x[2]**o+(-0.713222645312)*x[2]
        s[2,4,1,0]=(-0.865800829839)*x[0]**o+(-0.24789092495)*x[0]+(-0.765024475984)*x[1]**o+(0.246675200333)*x[1]+(0.63601593739)*x[2]**o+(0.758761609858)*x[2]
        s[2,4,1,1]=(-0.722774458642)*x[0]**o+(-0.640277272458)*x[0]+(-0.847467314954)*x[1]**o+(-0.676376490579)*x[1]+(-0.867510590824)*x[2]**o+(0.304980448789)*x[2]
        s[2,4,2,0]=(-0.556352556597)*x[0]**o+(-0.314836337315)*x[0]+(-0.172549538929)*x[1]**o+(-0.603791779732)*x[1]+(-0.482556973138)*x[2]**o+(0.155915646569)*x[2]
        s[2,4,2,1]=(-0.790402254477)*x[0]**o+(0.137494169657)*x[0]+(-0.546722957671)*x[1]**o+(-0.504092758637)*x[1]+(0.370205905665)*x[2]**o+(0.905182951166)*x[2]
        s[3,0,0,0]=(-0.393498907366)*x[0]**o+(0.792111044321)*x[0]+(0.160732029377)*x[1]**o+(0.531118521589)*x[1]+(0.616077214135)*x[2]**o+(0.0450299929995)*x[2]
        s[3,0,0,1]=(0.0490385695005)*x[0]**o+(-0.112630305424)*x[0]+(0.932194477491)*x[1]**o+(-0.692641363201)*x[1]+(0.0679516868523)*x[2]**o+(0.997252361895)*x[2]
        s[3,0,1,0]=(0.372518182464)*x[0]**o+(-0.390347141072)*x[0]+(-0.551584590493)*x[1]**o+(-0.762622043128)*x[1]+(-0.777489328164)*x[2]**o+(-0.306555622213)*x[2]
        s[3,0,1,1]=(-0.907039911971)*x[0]**o+(0.716176125684)*x[0]+(-0.0317846045066)*x[1]**o+(0.344154975432)*x[1]+(-0.581050833334)*x[2]**o+(-0.800597412247)*x[2]
        s[3,0,2,0]=(0.671061807362)*x[0]**o+(-0.632730364318)*x[0]+(0.784926714274)*x[1]**o+(-0.689167186471)*x[1]+(-0.828322284596)*x[2]**o+(-0.757384289233)*x[2]
        s[3,0,2,1]=(0.797200318087)*x[0]**o+(0.0748753590429)*x[0]+(0.98432669884)*x[1]**o+(0.19160698616)*x[1]+(-0.941839443214)*x[2]**o+(-0.51797940249)*x[2]
        s[3,1,0,0]=(0.396318123324)*x[0]**o+(0.553544474446)*x[0]+(-0.196123045916)*x[1]**o+(0.931142300652)*x[1]+(-0.893432985678)*x[2]**o+(0.61239614626)*x[2]
        s[3,1,0,1]=(-0.281320711026)*x[0]**o+(0.0272769480991)*x[0]+(-0.43587599777)*x[1]**o+(0.419822767553)*x[1]+(-0.453045711683)*x[2]**o+(0.47569249252)*x[2]
        s[3,1,1,0]=(-0.213862255449)*x[0]**o+(0.355661536766)*x[0]+(0.251742827449)*x[1]**o+(-0.807017120416)*x[1]+(-0.730339835476)*x[2]**o+(0.551573971157)*x[2]
        s[3,1,1,1]=(0.554657719161)*x[0]**o+(-0.912970059328)*x[0]+(0.800146831674)*x[1]**o+(-0.583227572533)*x[1]+(-0.0933072839888)*x[2]**o+(0.950038311476)*x[2]
        s[3,1,2,0]=(-0.485569875217)*x[0]**o+(-0.636350568501)*x[0]+(-0.560720680426)*x[1]**o+(0.126204185032)*x[1]+(-0.272414378167)*x[2]**o+(-0.888532404451)*x[2]
        s[3,1,2,1]=(-0.727409224907)*x[0]**o+(0.856672459636)*x[0]+(0.37848918095)*x[1]**o+(0.622080880799)*x[1]+(-0.228417584572)*x[2]**o+(0.562188460361)*x[2]
        s[3,2,0,0]=(-0.294609451063)*x[0]**o+(0.577588405163)*x[0]+(0.667703746348)*x[1]**o+(-0.910501998185)*x[1]+(0.843110506022)*x[2]**o+(-0.826714153775)*x[2]
        s[3,2,0,1]=(-0.193869261184)*x[0]**o+(0.76598264579)*x[0]+(-0.15832998161)*x[1]**o+(0.647800212602)*x[1]+(0.0578289377486)*x[2]**o+(0.00658825845732)*x[2]
        s[3,2,1,0]=(-0.567464737056)*x[0]**o+(-0.149652747243)*x[0]+(-0.55259439138)*x[1]**o+(-0.879859303137)*x[1]+(-0.618763037058)*x[2]**o+(-0.361881515471)*x[2]
        s[3,2,1,1]=(-0.331240639597)*x[0]**o+(-0.842702790198)*x[0]+(0.14240123238)*x[1]**o+(0.732696112887)*x[1]+(0.678517810911)*x[2]**o+(0.874215749104)*x[2]
        s[3,2,2,0]=(0.569105573686)*x[0]**o+(-0.671670416897)*x[0]+(0.809241806071)*x[1]**o+(-0.781683673317)*x[1]+(0.915497987962)*x[2]**o+(0.529423259835)*x[2]
        s[3,2,2,1]=(0.803926266415)*x[0]**o+(0.973851217854)*x[0]+(0.00110452925071)*x[1]**o+(-0.0258074727224)*x[1]+(0.397580796648)*x[2]**o+(-0.483394954284)*x[2]
        s[3,3,0,0]=(-0.239744952662)*x[0]**o+(0.0330238686771)*x[0]+(0.57663829323)*x[1]**o+(-0.835285980271)*x[1]+(-0.541211231475)*x[2]**o+(0.309919910162)*x[2]
        s[3,3,0,1]=(0.998856178514)*x[0]**o+(0.69963418829)*x[0]+(-0.590956618638)*x[1]**o+(0.437083139196)*x[1]+(0.24957499362)*x[2]**o+(-0.207180573624)*x[2]
        s[3,3,1,0]=(-0.654845715855)*x[0]**o+(0.960307610722)*x[0]+(0.615977294745)*x[1]**o+(-0.717345465881)*x[1]+(-0.0724170395954)*x[2]**o+(0.660888358757)*x[2]
        s[3,3,1,1]=(-0.283730169109)*x[0]**o+(0.712609062937)*x[0]+(0.922422201518)*x[1]**o+(0.941787071667)*x[1]+(-0.317440296596)*x[2]**o+(-0.776070611464)*x[2]
        s[3,3,2,0]=(-0.377933655183)*x[0]**o+(-0.946504205082)*x[0]+(-0.705852108222)*x[1]**o+(0.735471828851)*x[1]+(0.740387572185)*x[2]**o+(0.478923064569)*x[2]
        s[3,3,2,1]=(-0.74484862371)*x[0]**o+(-0.394147355587)*x[0]+(-0.961488506416)*x[1]**o+(-0.117956733646)*x[1]+(-0.45639006521)*x[2]**o+(0.800113732885)*x[2]
        s[3,4,0,0]=(-0.625504798604)*x[0]**o+(0.316567164198)*x[0]+(-0.616071176069)*x[1]**o+(0.882101858084)*x[1]+(0.734544640696)*x[2]**o+(-0.0269245549068)*x[2]
        s[3,4,0,1]=(-0.307431859474)*x[0]**o+(-0.0850940272398)*x[0]+(0.389650545234)*x[1]**o+(-0.0311288588201)*x[1]+(0.560636030796)*x[2]**o+(0.990554208909)*x[2]
        s[3,4,1,0]=(-0.85379517016)*x[0]**o+(0.768069325756)*x[0]+(0.0149807290805)*x[1]**o+(-0.87320355719)*x[1]+(-0.153198452162)*x[2]**o+(0.0809551506622)*x[2]
        s[3,4,1,1]=(-0.427217312348)*x[0]**o+(0.860261211584)*x[0]+(0.988421955689)*x[1]**o+(0.10533108572)*x[1]+(0.903327272548)*x[2]**o+(0.434209037137)*x[2]
        s[3,4,2,0]=(-0.264611936187)*x[0]**o+(0.0438724619932)*x[0]+(0.398401170965)*x[1]**o+(0.53328233455)*x[1]+(-0.437516039922)*x[2]**o+(0.926837033874)*x[2]
        s[3,4,2,1]=(-0.0213427465429)*x[0]**o+(-0.41324925677)*x[0]+(0.00876072281828)*x[1]**o+(0.640763350258)*x[1]+(-0.548829379894)*x[2]**o+(0.523108062556)*x[2]
        ref[0,0,0,0]=(-0.162239324627)/(o+1.)+(0.420823711679)+(-0.938544579906)*0.5**o
        ref[0,0,0,1]=(0.137764972182)/(o+1.)+(-0.155403311408)+(0.953579560099)*0.5**o
        ref[0,0,1,0]=(0.141771074109)/(o+1.)+(0.62433685519)+(-0.0146573390123)*0.5**o
        ref[0,0,1,1]=(0.633143165162)/(o+1.)+(0.280767586663)+(-0.658403837278)*0.5**o
        ref[0,0,2,0]=(-0.828440936984)/(o+1.)+(0.308908766878)+(0.62578079836)*0.5**o
        ref[0,0,2,1]=(-0.269494998515)/(o+1.)+(-0.630915571049)+(0.314093680988)*0.5**o
        ref[0,1,0,0]=(1.15502723685)/(o+1.)+(0.927449197715)+(0.00620161785372)*0.5**o
        ref[0,1,0,1]=(-0.0855024553167)/(o+1.)+(-0.49168631448)+(0.0303982843122)*0.5**o
        ref[0,1,1,0]=(-1.8469331432)/(o+1.)+(-0.524430880337)+(0.310811455642)*0.5**o
        ref[0,1,1,1]=(0.537645609521)/(o+1.)+(-0.344633868654)+(-0.751411238653)*0.5**o
        ref[0,1,2,0]=(0.221506792208)/(o+1.)+(-0.0674319704757)+(-0.852982317135)*0.5**o
        ref[0,1,2,1]=(-0.566579806313)/(o+1.)+(-0.514644355533)+(-0.973162329979)*0.5**o
        ref[0,2,0,0]=(-0.507035224213)/(o+1.)+(-0.0956662747788)+(0.472595355875)*0.5**o
        ref[0,2,0,1]=(-1.1524368593)/(o+1.)+(-0.191083030177)+(0.486817890916)*0.5**o
        ref[0,2,1,0]=(0.500325467807)/(o+1.)+(-1.12333855868)+(0.832549390017)*0.5**o
        ref[0,2,1,1]=(0.447545471863)/(o+1.)+(-0.152877640017)+(0.152135328754)*0.5**o
        ref[0,2,2,0]=(-1.61361734988)/(o+1.)+(0.645359373158)+(-0.358237320736)*0.5**o
        ref[0,2,2,1]=(1.37162871868)/(o+1.)+(0.254511071706)+(-0.1707212422)*0.5**o
        ref[0,3,0,0]=(0.538347734102)/(o+1.)+(-0.276316472587)+(-0.128307529775)*0.5**o
        ref[0,3,0,1]=(0.329551594748)/(o+1.)+(-0.0608123017038)+(-0.554435822439)*0.5**o
        ref[0,3,1,0]=(-0.251885428939)/(o+1.)+(-0.0909279577907)+(-0.823501956971)*0.5**o
        ref[0,3,1,1]=(1.2388607307)/(o+1.)+(-0.670336960245)+(0.430542618951)*0.5**o
        ref[0,3,2,0]=(-0.201277191941)/(o+1.)+(0.579717037829)+(-0.958502709531)*0.5**o
        ref[0,3,2,1]=(0.372495926146)/(o+1.)+(0.544001243064)+(-0.792569155533)*0.5**o
        ref[0,4,0,0]=(-0.929090994448)/(o+1.)+(-0.147450146752)+(-0.0338438096091)*0.5**o
        ref[0,4,0,1]=(0.406769594688)/(o+1.)+(-0.411605433699)+(-0.406152624356)*0.5**o
        ref[0,4,1,0]=(-1.22287850365)/(o+1.)+(0.118672804138)+(-0.799456787818)*0.5**o
        ref[0,4,1,1]=(-0.682148555557)/(o+1.)+(-0.274879964526)+(0.561141037244)*0.5**o
        ref[0,4,2,0]=(0.246291853273)/(o+1.)+(-0.425613157867)+(-0.346440790531)*0.5**o
        ref[0,4,2,1]=(-1.28927077932)/(o+1.)+(0.419795743507)+(0.450982323218)*0.5**o
        ref[1,0,0,0]=(-0.923085412384)/(o+1.)+(0.313757352879)+(0.00370747226485)*0.5**o
        ref[1,0,0,1]=(0.00575925647317)/(o+1.)+(-0.288966514622)+(0.314513438122)*0.5**o
        ref[1,0,1,0]=(0.434035454219)/(o+1.)+(-0.515110843)+(0.904368787276)*0.5**o
        ref[1,0,1,1]=(-0.110108060707)/(o+1.)+(0.173881856177)+(-0.0645313731593)*0.5**o
        ref[1,0,2,0]=(-0.252037383281)/(o+1.)+(-0.426036367903)+(0.721223990054)*0.5**o
        ref[1,0,2,1]=(-0.37751730365)/(o+1.)+(-1.10824175841)+(-0.928667989887)*0.5**o
        ref[1,1,0,0]=(1.19801192075)/(o+1.)+(0.200809030069)+(-0.598976144645)*0.5**o
        ref[1,1,0,1]=(-0.635883129368)/(o+1.)+(-0.665585407412)+(0.51758257139)*0.5**o
        ref[1,1,1,0]=(1.42874484542)/(o+1.)+(-0.550389829253)+(-0.847292260359)*0.5**o
        ref[1,1,1,1]=(-0.0992477519506)/(o+1.)+(0.00323846609633)+(0.396422756052)*0.5**o
        ref[1,1,2,0]=(1.34819867486)/(o+1.)+(0.450250032309)+(0.391968381002)*0.5**o
        ref[1,1,2,1]=(-0.121097708792)/(o+1.)+(0.559592329399)+(-0.212692240987)*0.5**o
        ref[1,2,0,0]=(-0.74215416511)/(o+1.)+(0.125284028138)+(0.0157691022819)*0.5**o
        ref[1,2,0,1]=(1.16707834435)/(o+1.)+(-0.352733344973)+(-0.828402321641)*0.5**o
        ref[1,2,1,0]=(-0.69687401029)/(o+1.)+(-0.668019676798)+(-0.375626403305)*0.5**o
        ref[1,2,1,1]=(1.71636148829)/(o+1.)+(-0.619308609863)+(0.839254171086)*0.5**o
        ref[1,2,2,0]=(-0.283815677841)/(o+1.)+(-0.339041004559)+(0.179198759222)*0.5**o
        ref[1,2,2,1]=(-0.939449325007)/(o+1.)+(-0.0634030795503)+(0.431283737039)*0.5**o
        ref[1,3,0,0]=(0.406000352595)/(o+1.)+(1.03929698329)+(-0.735181806951)*0.5**o
        ref[1,3,0,1]=(-1.52507252809)/(o+1.)+(0.390794144406)+(0.953086533561)*0.5**o
        ref[1,3,1,0]=(1.83677986515)/(o+1.)+(-0.617522644919)+(-0.723560393521)*0.5**o
        ref[1,3,1,1]=(-0.962851260085)/(o+1.)+(-0.975041955998)+(-0.915440550473)*0.5**o
        ref[1,3,2,0]=(0.179011984451)/(o+1.)+(-0.180597850638)+(-0.205303485441)*0.5**o
        ref[1,3,2,1]=(1.25964298045)/(o+1.)+(0.112703868656)+(0.542339109554)*0.5**o
        ref[1,4,0,0]=(-0.427685534341)/(o+1.)+(-0.286466542673)+(-0.444230035592)*0.5**o
        ref[1,4,0,1]=(0.249277405507)/(o+1.)+(0.983951990686)+(0.972127984006)*0.5**o
        ref[1,4,1,0]=(-0.0588423523453)/(o+1.)+(0.177870869081)+(-0.162531060958)*0.5**o
        ref[1,4,1,1]=(-0.540193116681)/(o+1.)+(0.134188048578)+(0.315463685116)*0.5**o
        ref[1,4,2,0]=(1.10482930332)/(o+1.)+(-0.262949354208)+(0.340729082172)*0.5**o
        ref[1,4,2,1]=(-0.194379151166)/(o+1.)+(-0.865612926576)+(0.7615004712)*0.5**o
        ref[2,0,0,0]=(0.568529419028)/(o+1.)+(-0.727743410643)+(0.0125409231833)*0.5**o
        ref[2,0,0,1]=(0.429759923235)/(o+1.)+(-0.118153221821)+(-0.0036047535766)*0.5**o
        ref[2,0,1,0]=(0.0147013281422)/(o+1.)+(-0.184189593934)+(0.0494808240706)*0.5**o
        ref[2,0,1,1]=(-1.04260188483)/(o+1.)+(0.0544821041543)+(-0.53303893512)*0.5**o
        ref[2,0,2,0]=(1.10076995631)/(o+1.)+(-0.409285673735)+(-0.306356882566)*0.5**o
        ref[2,0,2,1]=(0.19410165596)/(o+1.)+(-0.49559573443)+(0.337441903133)*0.5**o
        ref[2,1,0,0]=(1.38795661783)/(o+1.)+(0.285594373698)+(0.984282580742)*0.5**o
        ref[2,1,0,1]=(0.266145500053)/(o+1.)+(0.0744238284255)+(0.263434795622)*0.5**o
        ref[2,1,1,0]=(0.535858288734)/(o+1.)+(0.265917112183)+(0.839155090264)*0.5**o
        ref[2,1,1,1]=(-0.853070439164)/(o+1.)+(-0.398426996464)+(-0.626863009974)*0.5**o
        ref[2,1,2,0]=(0.346715087308)/(o+1.)+(-0.209728518143)+(-0.548834239805)*0.5**o
        ref[2,1,2,1]=(1.22828655502)/(o+1.)+(0.210585881083)+(0.304097820123)*0.5**o
        ref[2,2,0,0]=(-0.169809623035)/(o+1.)+(-0.13032500044)+(0.00294465132151)*0.5**o
        ref[2,2,0,1]=(0.0636903131735)/(o+1.)+(0.689926127736)+(0.185831262234)*0.5**o
        ref[2,2,1,0]=(-0.869120023429)/(o+1.)+(-0.116179533269)+(0.234717477502)*0.5**o
        ref[2,2,1,1]=(0.310079474886)/(o+1.)+(-0.252450281328)+(0.692290926695)*0.5**o
        ref[2,2,2,0]=(0.368209460524)/(o+1.)+(0.0894468546456)+(-0.408729971023)*0.5**o
        ref[2,2,2,1]=(-1.00608092767)/(o+1.)+(0.0767188899067)+(-0.30215609634)*0.5**o
        ref[2,3,0,0]=(-0.244283693862)/(o+1.)+(-0.0471812269507)+(-0.731783739254)*0.5**o
        ref[2,3,0,1]=(1.22867715869)/(o+1.)+(1.19279997063)+(0.353316440436)*0.5**o
        ref[2,3,1,0]=(0.282234993214)/(o+1.)+(0.465192266665)+(-0.682583926526)*0.5**o
        ref[2,3,1,1]=(0.968844588727)/(o+1.)+(0.265850020556)+(-0.773410808141)*0.5**o
        ref[2,3,2,0]=(1.70096693464)/(o+1.)+(1.00022562092)+(-0.856792343472)*0.5**o
        ref[2,3,2,1]=(-1.45831663906)/(o+1.)+(-0.936508076298)+(0.208814714888)*0.5**o
        ref[2,4,0,0]=(-0.439643905368)/(o+1.)+(-0.318019817124)+(0.0581246014334)*0.5**o
        ref[2,4,0,1]=(0.100004267721)/(o+1.)+(-0.0369779084584)+(0.658005351748)*0.5**o
        ref[2,4,1,0]=(-0.129008538594)/(o+1.)+(0.37877294262)+(-0.865800829839)*0.5**o
        ref[2,4,1,1]=(-1.71497790578)/(o+1.)+(-0.505836657124)+(-0.722774458642)*0.5**o
        ref[2,4,2,0]=(-0.655106512067)/(o+1.)+(-0.38135623524)+(-0.556352556597)*0.5**o
        ref[2,4,2,1]=(-0.176517052006)/(o+1.)+(0.269292181093)+(-0.790402254477)*0.5**o
        ref[3,0,0,0]=(0.776809243512)/(o+1.)+(0.684129779455)+(-0.393498907366)*0.5**o
        ref[3,0,0,1]=(1.00014616434)/(o+1.)+(0.0959903466349)+(0.0490385695005)*0.5**o
        ref[3,0,1,0]=(-1.32907391866)/(o+1.)+(-0.729762403206)+(0.372518182464)*0.5**o
        ref[3,0,1,1]=(-0.612835437841)/(o+1.)+(0.129866844435)+(-0.907039911971)*0.5**o
        ref[3,0,2,0]=(-0.0433955703218)/(o+1.)+(-1.03964092001)+(0.671061807362)*0.5**o
        ref[3,0,2,1]=(0.0424872556266)/(o+1.)+(-0.125748528644)+(0.797200318087)*0.5**o
        ref[3,1,0,0]=(-1.08955603159)/(o+1.)+(1.04854146068)+(0.396318123324)*0.5**o
        ref[3,1,0,1]=(-0.888921709452)/(o+1.)+(0.461396104086)+(-0.281320711026)*0.5**o
        ref[3,1,1,0]=(-0.478597008027)/(o+1.)+(0.0501091937538)+(-0.213862255449)*0.5**o
        ref[3,1,1,1]=(0.706839547685)/(o+1.)+(-0.273079660193)+(0.554657719161)*0.5**o
        ref[3,1,2,0]=(-0.833135058593)/(o+1.)+(-0.69933939396)+(-0.485569875217)*0.5**o
        ref[3,1,2,1]=(0.150071596378)/(o+1.)+(1.0204709004)+(-0.727409224907)*0.5**o
        ref[3,2,0,0]=(1.51081425237)/(o+1.)+(-0.579813873399)+(-0.294609451063)*0.5**o
        ref[3,2,0,1]=(-0.100501043862)/(o+1.)+(0.710185558425)+(-0.193869261184)*0.5**o
        ref[3,2,1,0]=(-1.17135742844)/(o+1.)+(-0.695696782926)+(-0.567464737056)*0.5**o
        ref[3,2,1,1]=(0.820919043291)/(o+1.)+(0.382104535896)+(-0.331240639597)*0.5**o
        ref[3,2,2,0]=(1.72473979403)/(o+1.)+(-0.461965415189)+(0.569105573686)*0.5**o
        ref[3,2,2,1]=(0.398685325899)/(o+1.)+(0.232324395424)+(0.803926266415)*0.5**o
        ref[3,3,0,0]=(0.0354270617556)/(o+1.)+(-0.246171100716)+(-0.239744952662)*0.5**o
        ref[3,3,0,1]=(-0.341381625018)/(o+1.)+(0.464768376931)+(0.998856178514)*0.5**o
        ref[3,3,1,0]=(0.543560255149)/(o+1.)+(0.451925251799)+(-0.654845715855)*0.5**o
        ref[3,3,1,1]=(0.604981904922)/(o+1.)+(0.43916276157)+(-0.283730169109)*0.5**o
        ref[3,3,2,0]=(0.0345354639628)/(o+1.)+(0.133945344169)+(-0.377933655183)*0.5**o
        ref[3,3,2,1]=(-1.41787857163)/(o+1.)+(0.144004821826)+(-0.74484862371)*0.5**o
        ref[3,4,0,0]=(0.118473464627)/(o+1.)+(0.585872233688)+(-0.625504798604)*0.5**o
        ref[3,4,0,1]=(0.95028657603)/(o+1.)+(0.437165661424)+(-0.307431859474)*0.5**o
        ref[3,4,1,0]=(-0.138217723081)/(o+1.)+(-0.0120895403861)+(-0.85379517016)*0.5**o
        ref[3,4,1,1]=(1.89174922824)/(o+1.)+(0.699900667221)+(-0.427217312348)*0.5**o
        ref[3,4,2,0]=(-0.0391148689569)/(o+1.)+(0.751995915209)+(-0.264611936187)*0.5**o
        ref[3,4,2,1]=(-0.540068657076)/(o+1.)+(0.375311078022)+(-0.0213427465429)*0.5**o
      res=integrate(arg)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactOne_fromData_ContinuousFunction_rank0(self):
      """
      tests integral of rank 0 Data on the FunctionOnContactOne

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports integral on FunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(),w)
      if dim==2:
        arg=(-0.0333010052777)*x[0]**o+(-0.242011298555)*x[0]+(0.686457578633)*x[1]**o+(0.438506741453)*x[1]
        ref=(0.686457578633)/(o+1.)+(0.0982477214491)+(-0.0333010052777)*0.5**o
      else:
        arg=(-0.365813216932)*x[0]**o+(0.969816151161)*x[0]+(0.349995764077)*x[1]**o+(0.867492181692)*x[1]+(-0.866177355796)*x[2]**o+(-0.203219031102)*x[2]
        ref=(-0.516181591719)/(o+1.)+(0.817044650875)+(-0.365813216932)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactOne_fromData_ContinuousFunction_rank1(self):
      """
      tests integral of rank 1 Data on the FunctionOnContactOne

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports integral on FunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(2,),w)
      ref=numpy.zeros((2,),numpy.float_)
      if dim==2:
        arg[0]=(0.038446371199)*x[0]**o+(0.323075821067)*x[0]+(0.770528210463)*x[1]**o+(0.0434528043023)*x[1]
        arg[1]=(-0.461876402566)*x[0]**o+(0.746965376214)*x[0]+(0.861234840159)*x[1]**o+(-0.568165009958)*x[1]
        ref[0]=(0.770528210463)/(o+1.)+(0.183264312685)+(0.038446371199)*0.5**o
        ref[1]=(0.861234840159)/(o+1.)+(0.0894001831281)+(-0.461876402566)*0.5**o
      else:
        arg[0]=(-0.472858663642)*x[0]**o+(-0.680210745369)*x[0]+(0.175425838082)*x[1]**o+(0.0189427039667)*x[1]+(0.0365198456146)*x[2]**o+(0.7840527457)*x[2]
        arg[1]=(-0.0191775972693)*x[0]**o+(0.19303923194)*x[0]+(0.775528295333)*x[1]**o+(-0.765500708496)*x[1]+(0.351369471606)*x[2]**o+(-0.351587727906)*x[2]
        ref[0]=(0.211945683696)/(o+1.)+(0.0613923521485)+(-0.472858663642)*0.5**o
        ref[1]=(1.12689776694)/(o+1.)+(-0.462024602231)+(-0.0191775972693)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactOne_fromData_ContinuousFunction_rank2(self):
      """
      tests integral of rank 2 Data on the FunctionOnContactOne

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports integral on FunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5),w)
      ref=numpy.zeros((4, 5),numpy.float_)
      if dim==2:
        arg[0,0]=(-0.994443255559)*x[0]**o+(-0.403427184912)*x[0]+(-0.889656446862)*x[1]**o+(-0.411934106034)*x[1]
        arg[0,1]=(-0.864086447061)*x[0]**o+(-0.277489357308)*x[0]+(0.977910864864)*x[1]**o+(-0.0505775488074)*x[1]
        arg[0,2]=(-0.397179114534)*x[0]**o+(-0.448046821439)*x[0]+(-0.340253609736)*x[1]**o+(0.624988837716)*x[1]
        arg[0,3]=(-0.9155191116)*x[0]**o+(0.133777450278)*x[0]+(-0.809567026065)*x[1]**o+(-0.815374059364)*x[1]
        arg[0,4]=(-0.416764684181)*x[0]**o+(0.0965164570638)*x[0]+(0.190191799833)*x[1]**o+(0.0970709424519)*x[1]
        arg[1,0]=(0.646527294535)*x[0]**o+(-0.337119506636)*x[0]+(0.42669563245)*x[1]**o+(-0.185228568706)*x[1]
        arg[1,1]=(-0.411532647934)*x[0]**o+(0.178830973832)*x[0]+(-0.904886020892)*x[1]**o+(-0.740623071275)*x[1]
        arg[1,2]=(0.523024622808)*x[0]**o+(-0.226298848077)*x[0]+(-0.707740568617)*x[1]**o+(0.998182796571)*x[1]
        arg[1,3]=(0.840649917357)*x[0]**o+(-0.736279151571)*x[0]+(0.996017554392)*x[1]**o+(-0.397833319931)*x[1]
        arg[1,4]=(0.76711764419)*x[0]**o+(-0.905452991298)*x[0]+(-0.925675237951)*x[1]**o+(-0.982712645237)*x[1]
        arg[2,0]=(0.198547103682)*x[0]**o+(0.674537682234)*x[0]+(-0.891195618193)*x[1]**o+(0.979109691481)*x[1]
        arg[2,1]=(0.171629921737)*x[0]**o+(0.0102727001751)*x[0]+(0.467578393374)*x[1]**o+(0.583960984645)*x[1]
        arg[2,2]=(-0.637526271497)*x[0]**o+(-0.900821073531)*x[0]+(0.61637733965)*x[1]**o+(0.43717505341)*x[1]
        arg[2,3]=(-0.205861963967)*x[0]**o+(-0.104621062003)*x[0]+(0.582584591946)*x[1]**o+(0.351739662419)*x[1]
        arg[2,4]=(-0.179355911316)*x[0]**o+(-0.188415282678)*x[0]+(0.598342895007)*x[1]**o+(0.364638908577)*x[1]
        arg[3,0]=(0.150471526119)*x[0]**o+(-0.940789873202)*x[0]+(-0.524275825864)*x[1]**o+(0.729004683023)*x[1]
        arg[3,1]=(0.123933417087)*x[0]**o+(0.192413160498)*x[0]+(0.203730760507)*x[1]**o+(0.852010172966)*x[1]
        arg[3,2]=(-0.297766950847)*x[0]**o+(0.269905131596)*x[0]+(-0.0830931256847)*x[1]**o+(0.26964712569)*x[1]
        arg[3,3]=(0.96961517265)*x[0]**o+(0.363312967302)*x[0]+(-0.811142762589)*x[1]**o+(-0.420442983537)*x[1]
        arg[3,4]=(-0.314185602206)*x[0]**o+(-0.0446515646101)*x[0]+(-0.884651038732)*x[1]**o+(-0.166386477611)*x[1]
        ref[0,0]=(-0.889656446862)/(o+1.)+(-0.407680645473)+(-0.994443255559)*0.5**o
        ref[0,1]=(0.977910864864)/(o+1.)+(-0.164033453058)+(-0.864086447061)*0.5**o
        ref[0,2]=(-0.340253609736)/(o+1.)+(0.0884710081389)+(-0.397179114534)*0.5**o
        ref[0,3]=(-0.809567026065)/(o+1.)+(-0.340798304543)+(-0.9155191116)*0.5**o
        ref[0,4]=(0.190191799833)/(o+1.)+(0.0967936997578)+(-0.416764684181)*0.5**o
        ref[1,0]=(0.42669563245)/(o+1.)+(-0.261174037671)+(0.646527294535)*0.5**o
        ref[1,1]=(-0.904886020892)/(o+1.)+(-0.280896048722)+(-0.411532647934)*0.5**o
        ref[1,2]=(-0.707740568617)/(o+1.)+(0.385941974247)+(0.523024622808)*0.5**o
        ref[1,3]=(0.996017554392)/(o+1.)+(-0.567056235751)+(0.840649917357)*0.5**o
        ref[1,4]=(-0.925675237951)/(o+1.)+(-0.944082818268)+(0.76711764419)*0.5**o
        ref[2,0]=(-0.891195618193)/(o+1.)+(0.826823686857)+(0.198547103682)*0.5**o
        ref[2,1]=(0.467578393374)/(o+1.)+(0.29711684241)+(0.171629921737)*0.5**o
        ref[2,2]=(0.61637733965)/(o+1.)+(-0.231823010061)+(-0.637526271497)*0.5**o
        ref[2,3]=(0.582584591946)/(o+1.)+(0.123559300208)+(-0.205861963967)*0.5**o
        ref[2,4]=(0.598342895007)/(o+1.)+(0.0881118129495)+(-0.179355911316)*0.5**o
        ref[3,0]=(-0.524275825864)/(o+1.)+(-0.105892595089)+(0.150471526119)*0.5**o
        ref[3,1]=(0.203730760507)/(o+1.)+(0.522211666732)+(0.123933417087)*0.5**o
        ref[3,2]=(-0.0830931256847)/(o+1.)+(0.269776128643)+(-0.297766950847)*0.5**o
        ref[3,3]=(-0.811142762589)/(o+1.)+(-0.0285650081177)+(0.96961517265)*0.5**o
        ref[3,4]=(-0.884651038732)/(o+1.)+(-0.105519021111)+(-0.314185602206)*0.5**o
      else:
        arg[0,0]=(0.264691789613)*x[0]**o+(0.816541340997)*x[0]+(-0.402990464667)*x[1]**o+(-0.57479619687)*x[1]+(0.336313666995)*x[2]**o+(-0.353125285536)*x[2]
        arg[0,1]=(-0.0526559162494)*x[0]**o+(-0.901336706583)*x[0]+(0.237482536727)*x[1]**o+(-0.458564275773)*x[1]+(-0.90715992057)*x[2]**o+(-0.352062856077)*x[2]
        arg[0,2]=(0.786349112412)*x[0]**o+(-0.395569856472)*x[0]+(-0.996593023157)*x[1]**o+(-0.277001350627)*x[1]+(0.435375913289)*x[2]**o+(0.00380742297736)*x[2]
        arg[0,3]=(0.3336751197)*x[0]**o+(0.592185657129)*x[0]+(0.148956266814)*x[1]**o+(-0.852264651722)*x[1]+(-0.680249245511)*x[2]**o+(0.238843544413)*x[2]
        arg[0,4]=(0.875206676786)*x[0]**o+(0.0420423017124)*x[0]+(-0.690488811412)*x[1]**o+(-0.191328696331)*x[1]+(-0.861423624177)*x[2]**o+(0.483925405411)*x[2]
        arg[1,0]=(-0.573216715158)*x[0]**o+(-0.0668114726198)*x[0]+(0.382097918101)*x[1]**o+(-0.126485626438)*x[1]+(0.37521336616)*x[2]**o+(0.42402066058)*x[2]
        arg[1,1]=(-0.0919326514467)*x[0]**o+(-0.791351880042)*x[0]+(0.772694542694)*x[1]**o+(-0.537902003787)*x[1]+(0.424413342503)*x[2]**o+(-0.285553230731)*x[2]
        arg[1,2]=(0.855490362389)*x[0]**o+(0.60428104628)*x[0]+(-0.846565232506)*x[1]**o+(-0.243727717758)*x[1]+(0.900077399935)*x[2]**o+(-0.974305489047)*x[2]
        arg[1,3]=(-0.90802343526)*x[0]**o+(0.00950587842259)*x[0]+(0.821234711321)*x[1]**o+(-0.0628912085092)*x[1]+(-0.773563396868)*x[2]**o+(-0.403955952419)*x[2]
        arg[1,4]=(-0.33139358787)*x[0]**o+(-0.901382943687)*x[0]+(-0.981921375707)*x[1]**o+(-0.0654519807133)*x[1]+(0.880069861158)*x[2]**o+(-0.0796172270076)*x[2]
        arg[2,0]=(-0.202818692518)*x[0]**o+(-0.155436611971)*x[0]+(0.739467305515)*x[1]**o+(-0.12096877475)*x[1]+(0.479644796804)*x[2]**o+(0.0580715322389)*x[2]
        arg[2,1]=(0.0461326006936)*x[0]**o+(0.122168224674)*x[0]+(0.986788073931)*x[1]**o+(-0.435933938764)*x[1]+(-0.312527130215)*x[2]**o+(0.726805187165)*x[2]
        arg[2,2]=(-0.539659153805)*x[0]**o+(-0.26082353681)*x[0]+(0.367739719819)*x[1]**o+(0.943584828354)*x[1]+(0.688225221556)*x[2]**o+(-0.121563525696)*x[2]
        arg[2,3]=(0.861717128203)*x[0]**o+(0.0666983176426)*x[0]+(0.984510848927)*x[1]**o+(-0.321865378029)*x[1]+(0.522584734)*x[2]**o+(0.589527116625)*x[2]
        arg[2,4]=(0.778575699703)*x[0]**o+(0.515802361664)*x[0]+(0.817655905624)*x[1]**o+(0.883824717911)*x[1]+(0.656833761714)*x[2]**o+(-0.991585145611)*x[2]
        arg[3,0]=(0.752875513075)*x[0]**o+(0.615136567082)*x[0]+(-0.205265870704)*x[1]**o+(0.483918142728)*x[1]+(-0.0495322147557)*x[2]**o+(-0.132474571954)*x[2]
        arg[3,1]=(0.0563875061408)*x[0]**o+(-0.603033440119)*x[0]+(-0.0944809376206)*x[1]**o+(0.725330686731)*x[1]+(0.00527671533343)*x[2]**o+(-0.434519675586)*x[2]
        arg[3,2]=(0.100078550382)*x[0]**o+(-0.340988639989)*x[0]+(-0.392591649097)*x[1]**o+(-0.924705734588)*x[1]+(0.420410192075)*x[2]**o+(-0.94741221593)*x[2]
        arg[3,3]=(0.536414145801)*x[0]**o+(0.142421640429)*x[0]+(0.213162255423)*x[1]**o+(-0.695193008147)*x[1]+(0.9625091599)*x[2]**o+(0.534409542153)*x[2]
        arg[3,4]=(0.305683606235)*x[0]**o+(0.340964210305)*x[0]+(0.757807973246)*x[1]**o+(0.11459551493)*x[1]+(-0.81395047506)*x[2]**o+(0.636392281491)*x[2]
        ref[0,0]=(-0.0666767976714)/(o+1.)+(-0.0556900707042)+(0.264691789613)*0.5**o
        ref[0,1]=(-0.669677383843)/(o+1.)+(-0.855981919217)+(-0.0526559162494)*0.5**o
        ref[0,2]=(-0.561217109869)/(o+1.)+(-0.334381892061)+(0.786349112412)*0.5**o
        ref[0,3]=(-0.531292978697)/(o+1.)+(-0.0106177250896)+(0.3336751197)*0.5**o
        ref[0,4]=(-1.55191243559)/(o+1.)+(0.167319505396)+(0.875206676786)*0.5**o
        ref[1,0]=(0.757311284261)/(o+1.)+(0.115361780761)+(-0.573216715158)*0.5**o
        ref[1,1]=(1.1971078852)/(o+1.)+(-0.80740355728)+(-0.0919326514467)*0.5**o
        ref[1,2]=(0.0535121674292)/(o+1.)+(-0.306876080263)+(0.855490362389)*0.5**o
        ref[1,3]=(0.0476713144526)/(o+1.)+(-0.228670641253)+(-0.90802343526)*0.5**o
        ref[1,4]=(-0.101851514548)/(o+1.)+(-0.523226075704)+(-0.33139358787)*0.5**o
        ref[2,0]=(1.21911210232)/(o+1.)+(-0.109166927241)+(-0.202818692518)*0.5**o
        ref[2,1]=(0.674260943716)/(o+1.)+(0.206519736538)+(0.0461326006936)*0.5**o
        ref[2,2]=(1.05596494137)/(o+1.)+(0.280598882924)+(-0.539659153805)*0.5**o
        ref[2,3]=(1.50709558293)/(o+1.)+(0.167180028119)+(0.861717128203)*0.5**o
        ref[2,4]=(1.47448966734)/(o+1.)+(0.204020966982)+(0.778575699703)*0.5**o
        ref[3,0]=(-0.25479808546)/(o+1.)+(0.483290068928)+(0.752875513075)*0.5**o
        ref[3,1]=(-0.0892042222872)/(o+1.)+(-0.156111214487)+(0.0563875061408)*0.5**o
        ref[3,2]=(0.0278185429781)/(o+1.)+(-1.10655329525)+(0.100078550382)*0.5**o
        ref[3,3]=(1.17567141532)/(o+1.)+(-0.00918091278228)+(0.536414145801)*0.5**o
        ref[3,4]=(-0.0561425018138)/(o+1.)+(0.545976003363)+(0.305683606235)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactOne_fromData_ContinuousFunction_rank3(self):
      """
      tests integral of rank 3 Data on the FunctionOnContactOne

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports integral on FunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(6, 2, 2),w)
      ref=numpy.zeros((6, 2, 2),numpy.float_)
      if dim==2:
        arg[0,0,0]=(0.448877659671)*x[0]**o+(0.130599173692)*x[0]+(-0.264185555629)*x[1]**o+(-0.722726040492)*x[1]
        arg[0,0,1]=(-0.543806308372)*x[0]**o+(-0.685242399166)*x[0]+(0.325336801501)*x[1]**o+(0.572960380228)*x[1]
        arg[0,1,0]=(-0.893080563297)*x[0]**o+(-0.251553981066)*x[0]+(0.256078388483)*x[1]**o+(-0.926701685259)*x[1]
        arg[0,1,1]=(0.346088914447)*x[0]**o+(0.166180418696)*x[0]+(0.353024178635)*x[1]**o+(0.731811935721)*x[1]
        arg[1,0,0]=(0.553083243483)*x[0]**o+(0.976153803522)*x[0]+(-0.260777317634)*x[1]**o+(-0.641936915628)*x[1]
        arg[1,0,1]=(-0.561076901068)*x[0]**o+(0.837832618302)*x[0]+(-0.638970151828)*x[1]**o+(-0.495169195543)*x[1]
        arg[1,1,0]=(-0.292805925132)*x[0]**o+(0.670472035586)*x[0]+(-0.752026521307)*x[1]**o+(-0.473087472861)*x[1]
        arg[1,1,1]=(-0.432857805275)*x[0]**o+(0.495387797249)*x[0]+(0.0579881469257)*x[1]**o+(0.831644440274)*x[1]
        arg[2,0,0]=(0.413433231384)*x[0]**o+(0.919831571878)*x[0]+(-0.373581626416)*x[1]**o+(0.16423318333)*x[1]
        arg[2,0,1]=(0.857566619293)*x[0]**o+(0.340454525569)*x[0]+(0.139298007219)*x[1]**o+(0.220542978265)*x[1]
        arg[2,1,0]=(-0.914184896115)*x[0]**o+(-0.0279186159835)*x[0]+(0.893417478351)*x[1]**o+(-0.108178714604)*x[1]
        arg[2,1,1]=(-0.749629612712)*x[0]**o+(-0.649957092299)*x[0]+(0.181944194719)*x[1]**o+(-0.988782927329)*x[1]
        arg[3,0,0]=(0.292013067678)*x[0]**o+(0.491680364317)*x[0]+(0.951144487798)*x[1]**o+(0.980949630108)*x[1]
        arg[3,0,1]=(0.89779245885)*x[0]**o+(-0.147546711846)*x[0]+(0.0902450198362)*x[1]**o+(-0.669984062969)*x[1]
        arg[3,1,0]=(0.62590631314)*x[0]**o+(0.329384454978)*x[0]+(-0.676917185409)*x[1]**o+(-0.0903173226074)*x[1]
        arg[3,1,1]=(-0.668268661271)*x[0]**o+(-0.847167879584)*x[0]+(0.329431843319)*x[1]**o+(-0.989253343115)*x[1]
        arg[4,0,0]=(-0.675111374291)*x[0]**o+(0.738030423939)*x[0]+(-0.413299452027)*x[1]**o+(0.36859580301)*x[1]
        arg[4,0,1]=(-0.207625666422)*x[0]**o+(-0.455639256266)*x[0]+(0.554118743878)*x[1]**o+(0.14044849621)*x[1]
        arg[4,1,0]=(-0.313337039275)*x[0]**o+(-0.451797308521)*x[0]+(0.201296275104)*x[1]**o+(-0.779869953962)*x[1]
        arg[4,1,1]=(-0.0308697706768)*x[0]**o+(0.214664850788)*x[0]+(-0.824157101269)*x[1]**o+(0.156012836486)*x[1]
        arg[5,0,0]=(0.277885566309)*x[0]**o+(-0.387820938776)*x[0]+(0.820602896312)*x[1]**o+(0.162715513355)*x[1]
        arg[5,0,1]=(-0.216951421349)*x[0]**o+(0.906019329549)*x[0]+(-0.330132336808)*x[1]**o+(0.416109194357)*x[1]
        arg[5,1,0]=(0.923496627826)*x[0]**o+(-0.177644464371)*x[0]+(0.482278717995)*x[1]**o+(-0.362003106154)*x[1]
        arg[5,1,1]=(0.9109111591)*x[0]**o+(-0.0919782899769)*x[0]+(0.103950387446)*x[1]**o+(0.447452648868)*x[1]
        ref[0,0,0]=(-0.264185555629)/(o+1.)+(-0.2960634334)+(0.448877659671)*0.5**o
        ref[0,0,1]=(0.325336801501)/(o+1.)+(-0.0561410094689)+(-0.543806308372)*0.5**o
        ref[0,1,0]=(0.256078388483)/(o+1.)+(-0.589127833162)+(-0.893080563297)*0.5**o
        ref[0,1,1]=(0.353024178635)/(o+1.)+(0.448996177208)+(0.346088914447)*0.5**o
        ref[1,0,0]=(-0.260777317634)/(o+1.)+(0.167108443947)+(0.553083243483)*0.5**o
        ref[1,0,1]=(-0.638970151828)/(o+1.)+(0.17133171138)+(-0.561076901068)*0.5**o
        ref[1,1,0]=(-0.752026521307)/(o+1.)+(0.0986922813626)+(-0.292805925132)*0.5**o
        ref[1,1,1]=(0.0579881469257)/(o+1.)+(0.663516118762)+(-0.432857805275)*0.5**o
        ref[2,0,0]=(-0.373581626416)/(o+1.)+(0.542032377604)+(0.413433231384)*0.5**o
        ref[2,0,1]=(0.139298007219)/(o+1.)+(0.280498751917)+(0.857566619293)*0.5**o
        ref[2,1,0]=(0.893417478351)/(o+1.)+(-0.0680486652939)+(-0.914184896115)*0.5**o
        ref[2,1,1]=(0.181944194719)/(o+1.)+(-0.819370009814)+(-0.749629612712)*0.5**o
        ref[3,0,0]=(0.951144487798)/(o+1.)+(0.736314997212)+(0.292013067678)*0.5**o
        ref[3,0,1]=(0.0902450198362)/(o+1.)+(-0.408765387408)+(0.89779245885)*0.5**o
        ref[3,1,0]=(-0.676917185409)/(o+1.)+(0.119533566185)+(0.62590631314)*0.5**o
        ref[3,1,1]=(0.329431843319)/(o+1.)+(-0.91821061135)+(-0.668268661271)*0.5**o
        ref[4,0,0]=(-0.413299452027)/(o+1.)+(0.553313113475)+(-0.675111374291)*0.5**o
        ref[4,0,1]=(0.554118743878)/(o+1.)+(-0.157595380028)+(-0.207625666422)*0.5**o
        ref[4,1,0]=(0.201296275104)/(o+1.)+(-0.615833631241)+(-0.313337039275)*0.5**o
        ref[4,1,1]=(-0.824157101269)/(o+1.)+(0.185338843637)+(-0.0308697706768)*0.5**o
        ref[5,0,0]=(0.820602896312)/(o+1.)+(-0.11255271271)+(0.277885566309)*0.5**o
        ref[5,0,1]=(-0.330132336808)/(o+1.)+(0.661064261953)+(-0.216951421349)*0.5**o
        ref[5,1,0]=(0.482278717995)/(o+1.)+(-0.269823785262)+(0.923496627826)*0.5**o
        ref[5,1,1]=(0.103950387446)/(o+1.)+(0.177737179446)+(0.9109111591)*0.5**o
      else:
        arg[0,0,0]=(0.843169691966)*x[0]**o+(0.252560741792)*x[0]+(0.0428740803857)*x[1]**o+(-0.702885246223)*x[1]+(0.175406156382)*x[2]**o+(-0.819206174419)*x[2]
        arg[0,0,1]=(-0.780678250187)*x[0]**o+(-0.628781977796)*x[0]+(0.345241549754)*x[1]**o+(-0.686783366275)*x[1]+(0.298501249797)*x[2]**o+(-0.592222734778)*x[2]
        arg[0,1,0]=(0.680331984783)*x[0]**o+(0.20945731105)*x[0]+(0.582220641498)*x[1]**o+(0.159144772508)*x[1]+(0.197544685245)*x[2]**o+(-0.542847499692)*x[2]
        arg[0,1,1]=(0.561649107089)*x[0]**o+(0.409538404439)*x[0]+(0.687461001448)*x[1]**o+(0.327986995905)*x[1]+(0.701007758697)*x[2]**o+(0.95919995454)*x[2]
        arg[1,0,0]=(0.0620287124894)*x[0]**o+(-0.946536581396)*x[0]+(-0.813637608115)*x[1]**o+(0.0808967031579)*x[1]+(0.545408819059)*x[2]**o+(0.250558134093)*x[2]
        arg[1,0,1]=(-0.0778300410758)*x[0]**o+(-0.852201314682)*x[0]+(0.712528858815)*x[1]**o+(-0.69009644973)*x[1]+(-0.0836140529567)*x[2]**o+(0.687579338803)*x[2]
        arg[1,1,0]=(-0.37648103863)*x[0]**o+(-0.264461846666)*x[0]+(-0.45921598716)*x[1]**o+(-0.516551658367)*x[1]+(0.512446750786)*x[2]**o+(0.850266522322)*x[2]
        arg[1,1,1]=(-0.951741356481)*x[0]**o+(-0.989023540683)*x[0]+(0.338787640816)*x[1]**o+(0.764043925298)*x[1]+(0.524009019713)*x[2]**o+(-0.912507794534)*x[2]
        arg[2,0,0]=(0.613468181683)*x[0]**o+(-0.357325179573)*x[0]+(0.153221256731)*x[1]**o+(-0.748686771579)*x[1]+(0.364085012368)*x[2]**o+(-0.134535056195)*x[2]
        arg[2,0,1]=(-0.728948376511)*x[0]**o+(-0.0657162843876)*x[0]+(-0.336013775703)*x[1]**o+(0.299044747397)*x[1]+(-0.712877779871)*x[2]**o+(0.229974926734)*x[2]
        arg[2,1,0]=(-0.213428159952)*x[0]**o+(0.680250111549)*x[0]+(-0.986942572737)*x[1]**o+(0.549006049647)*x[1]+(0.572921637402)*x[2]**o+(-0.241851127933)*x[2]
        arg[2,1,1]=(0.0367155018841)*x[0]**o+(-0.768111161527)*x[0]+(-0.748085216726)*x[1]**o+(-0.752536272228)*x[1]+(0.555945063877)*x[2]**o+(0.012687074005)*x[2]
        arg[3,0,0]=(-0.945391902075)*x[0]**o+(-0.581330133808)*x[0]+(-0.253390460088)*x[1]**o+(-0.885589795648)*x[1]+(-0.555900062171)*x[2]**o+(-0.241006607054)*x[2]
        arg[3,0,1]=(0.924110231188)*x[0]**o+(0.428520753545)*x[0]+(0.187773835144)*x[1]**o+(-0.53787305121)*x[1]+(0.0970284978901)*x[2]**o+(0.0580212350951)*x[2]
        arg[3,1,0]=(0.426389283701)*x[0]**o+(-0.0976148877161)*x[0]+(-0.364828466078)*x[1]**o+(-0.536398529197)*x[1]+(0.648605043498)*x[2]**o+(-0.234629706559)*x[2]
        arg[3,1,1]=(-0.736098543054)*x[0]**o+(0.985014182905)*x[0]+(-0.563513361433)*x[1]**o+(0.589434310221)*x[1]+(-0.360296462891)*x[2]**o+(-0.586591636182)*x[2]
        arg[4,0,0]=(0.953916732391)*x[0]**o+(-0.835620914249)*x[0]+(0.562803727423)*x[1]**o+(-0.904263689551)*x[1]+(-0.349815820142)*x[2]**o+(-0.156410988081)*x[2]
        arg[4,0,1]=(-0.236710267891)*x[0]**o+(0.958992060128)*x[0]+(0.711565908363)*x[1]**o+(-0.639771085045)*x[1]+(-0.919042661652)*x[2]**o+(-0.198907436333)*x[2]
        arg[4,1,0]=(0.275058935551)*x[0]**o+(0.563461500878)*x[0]+(0.438833998471)*x[1]**o+(0.686375712335)*x[1]+(-0.930322605963)*x[2]**o+(0.710777122765)*x[2]
        arg[4,1,1]=(-0.507845102241)*x[0]**o+(-0.390574086909)*x[0]+(-0.170020669232)*x[1]**o+(-0.107455242464)*x[1]+(-0.97056652533)*x[2]**o+(-0.720180192255)*x[2]
        arg[5,0,0]=(-0.922597172214)*x[0]**o+(0.229197305733)*x[0]+(-0.943214499543)*x[1]**o+(0.269256599046)*x[1]+(-0.11540537404)*x[2]**o+(0.956940487934)*x[2]
        arg[5,0,1]=(-0.0350868975578)*x[0]**o+(-0.547620412319)*x[0]+(-0.818344866429)*x[1]**o+(-0.0632082353078)*x[1]+(-0.490536679501)*x[2]**o+(-0.79641747407)*x[2]
        arg[5,1,0]=(0.877279477171)*x[0]**o+(-0.408752853807)*x[0]+(0.143432191997)*x[1]**o+(0.230679627667)*x[1]+(0.0346230515425)*x[2]**o+(0.357377660033)*x[2]
        arg[5,1,1]=(0.294375780312)*x[0]**o+(-0.688447065878)*x[0]+(0.213334642918)*x[1]**o+(0.664021137697)*x[1]+(-0.532996065961)*x[2]**o+(0.794315136407)*x[2]
        ref[0,0,0]=(0.218280236768)/(o+1.)+(-0.634765339425)+(0.843169691966)*0.5**o
        ref[0,0,1]=(0.64374279955)/(o+1.)+(-0.953894039424)+(-0.780678250187)*0.5**o
        ref[0,1,0]=(0.779765326742)/(o+1.)+(-0.0871227080671)+(0.680331984783)*0.5**o
        ref[0,1,1]=(1.38846876015)/(o+1.)+(0.848362677442)+(0.561649107089)*0.5**o
        ref[1,0,0]=(-0.268228789055)/(o+1.)+(-0.307540872072)+(0.0620287124894)*0.5**o
        ref[1,0,1]=(0.628914805859)/(o+1.)+(-0.427359212804)+(-0.0778300410758)*0.5**o
        ref[1,1,0]=(0.0532307636262)/(o+1.)+(0.0346265086445)+(-0.37648103863)*0.5**o
        ref[1,1,1]=(0.862796660529)/(o+1.)+(-0.56874370496)+(-0.951741356481)*0.5**o
        ref[2,0,0]=(0.5173062691)/(o+1.)+(-0.620273503673)+(0.613468181683)*0.5**o
        ref[2,0,1]=(-1.04889155557)/(o+1.)+(0.231651694871)+(-0.728948376511)*0.5**o
        ref[2,1,0]=(-0.414020935335)/(o+1.)+(0.493702516631)+(-0.213428159952)*0.5**o
        ref[2,1,1]=(-0.192140152849)/(o+1.)+(-0.753980179875)+(0.0367155018841)*0.5**o
        ref[3,0,0]=(-0.809290522259)/(o+1.)+(-0.853963268255)+(-0.945391902075)*0.5**o
        ref[3,0,1]=(0.284802333034)/(o+1.)+(-0.0256655312851)+(0.924110231188)*0.5**o
        ref[3,1,0]=(0.28377657742)/(o+1.)+(-0.434321561736)+(0.426389283701)*0.5**o
        ref[3,1,1]=(-0.923809824324)/(o+1.)+(0.493928428472)+(-0.736098543054)*0.5**o
        ref[4,0,0]=(0.212987907281)/(o+1.)+(-0.948147795941)+(0.953916732391)*0.5**o
        ref[4,0,1]=(-0.207476753289)/(o+1.)+(0.0601567693744)+(-0.236710267891)*0.5**o
        ref[4,1,0]=(-0.491488607492)/(o+1.)+(0.980307167989)+(0.275058935551)*0.5**o
        ref[4,1,1]=(-1.14058719456)/(o+1.)+(-0.609104760814)+(-0.507845102241)*0.5**o
        ref[5,0,0]=(-1.05861987358)/(o+1.)+(0.727697196357)+(-0.922597172214)*0.5**o
        ref[5,0,1]=(-1.30888154593)/(o+1.)+(-0.703623060848)+(-0.0350868975578)*0.5**o
        ref[5,1,0]=(0.17805524354)/(o+1.)+(0.0896522169463)+(0.877279477171)*0.5**o
        ref[5,1,1]=(-0.319661423044)/(o+1.)+(0.384944604113)+(0.294375780312)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactOne_fromData_ContinuousFunction_rank4(self):
      """
      tests integral of rank 4 Data on the FunctionOnContactOne

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports integral on FunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5, 3, 2),w)
      ref=numpy.zeros((4, 5, 3, 2),numpy.float_)
      if dim==2:
        arg[0,0,0,0]=(0.182956096072)*x[0]**o+(0.857065806542)*x[0]+(-0.0352052809673)*x[1]**o+(0.950661773214)*x[1]
        arg[0,0,0,1]=(-0.81342189211)*x[0]**o+(0.36137453013)*x[0]+(0.0610373691872)*x[1]**o+(-0.974859653337)*x[1]
        arg[0,0,1,0]=(-0.149103531408)*x[0]**o+(0.853517194849)*x[0]+(0.623943596759)*x[1]**o+(0.441477983782)*x[1]
        arg[0,0,1,1]=(0.74912087771)*x[0]**o+(0.935896347068)*x[0]+(0.136600247183)*x[1]**o+(0.915043476739)*x[1]
        arg[0,0,2,0]=(-0.289042694587)*x[0]**o+(-0.0324580134489)*x[0]+(0.966454801911)*x[1]**o+(0.158747002668)*x[1]
        arg[0,0,2,1]=(0.0809737838902)*x[0]**o+(0.483125829124)*x[0]+(-0.156074498516)*x[1]**o+(-0.387215102608)*x[1]
        arg[0,1,0,0]=(0.638822193145)*x[0]**o+(-0.211942445721)*x[0]+(-0.016865109839)*x[1]**o+(-0.596486509057)*x[1]
        arg[0,1,0,1]=(0.611604473356)*x[0]**o+(-0.330484367791)*x[0]+(-0.897448372147)*x[1]**o+(-0.0188556077191)*x[1]
        arg[0,1,1,0]=(-0.745407006471)*x[0]**o+(0.547985829362)*x[0]+(-0.229098133178)*x[1]**o+(0.339357994536)*x[1]
        arg[0,1,1,1]=(0.756528549281)*x[0]**o+(0.0968167016962)*x[0]+(0.481206893907)*x[1]**o+(-0.320299781291)*x[1]
        arg[0,1,2,0]=(0.335544368565)*x[0]**o+(0.143490409353)*x[0]+(0.367303506039)*x[1]**o+(-0.459001464112)*x[1]
        arg[0,1,2,1]=(0.669933676654)*x[0]**o+(0.0679909847194)*x[0]+(0.182483888026)*x[1]**o+(-0.382138842505)*x[1]
        arg[0,2,0,0]=(-0.929163949925)*x[0]**o+(0.670533855076)*x[0]+(0.587675819052)*x[1]**o+(0.221939557678)*x[1]
        arg[0,2,0,1]=(0.639896773528)*x[0]**o+(-0.166901547286)*x[0]+(-0.83934259298)*x[1]**o+(-0.522536841927)*x[1]
        arg[0,2,1,0]=(-0.399556357179)*x[0]**o+(0.860582177909)*x[0]+(0.675592435624)*x[1]**o+(0.77480796903)*x[1]
        arg[0,2,1,1]=(-0.281969675471)*x[0]**o+(-0.250831251067)*x[0]+(0.14396936899)*x[1]**o+(-0.65775670156)*x[1]
        arg[0,2,2,0]=(0.339204154215)*x[0]**o+(0.125319656279)*x[0]+(0.417557775604)*x[1]**o+(-0.985431777777)*x[1]
        arg[0,2,2,1]=(-0.655500794874)*x[0]**o+(-0.419889131296)*x[0]+(-0.0763188160723)*x[1]**o+(0.995600932757)*x[1]
        arg[0,3,0,0]=(-0.483746115201)*x[0]**o+(-0.256632816498)*x[0]+(-0.0533973517656)*x[1]**o+(-0.982541041474)*x[1]
        arg[0,3,0,1]=(0.446810992304)*x[0]**o+(-0.823523374302)*x[0]+(-0.607238533539)*x[1]**o+(-0.97562384609)*x[1]
        arg[0,3,1,0]=(-0.0235907627093)*x[0]**o+(-0.0720587626585)*x[0]+(0.930589323947)*x[1]**o+(0.327916365569)*x[1]
        arg[0,3,1,1]=(0.486342062258)*x[0]**o+(0.430279082913)*x[0]+(0.237445053125)*x[1]**o+(0.577650421809)*x[1]
        arg[0,3,2,0]=(-0.581025309937)*x[0]**o+(0.97026755344)*x[0]+(0.180505264557)*x[1]**o+(0.289585805796)*x[1]
        arg[0,3,2,1]=(-0.523356041281)*x[0]**o+(-0.502230606793)*x[0]+(0.915270822508)*x[1]**o+(0.622076448072)*x[1]
        arg[0,4,0,0]=(-0.584782530121)*x[0]**o+(-0.433761163729)*x[0]+(-0.449731522195)*x[1]**o+(0.392234342377)*x[1]
        arg[0,4,0,1]=(-0.0724106581167)*x[0]**o+(0.76896002907)*x[0]+(0.0374933582543)*x[1]**o+(-0.417790382045)*x[1]
        arg[0,4,1,0]=(0.0460360663465)*x[0]**o+(-0.0343116732267)*x[0]+(0.815864788848)*x[1]**o+(-0.922236546574)*x[1]
        arg[0,4,1,1]=(-0.978584425712)*x[0]**o+(0.249337867126)*x[0]+(-0.822961904166)*x[1]**o+(0.627473480274)*x[1]
        arg[0,4,2,0]=(-0.267227111945)*x[0]**o+(0.791320996051)*x[0]+(0.721009228711)*x[1]**o+(0.00199546728243)*x[1]
        arg[0,4,2,1]=(-0.141372777283)*x[0]**o+(-0.996912487736)*x[0]+(0.94281427253)*x[1]**o+(-0.780685413978)*x[1]
        arg[1,0,0,0]=(-0.721911758033)*x[0]**o+(0.0119756829508)*x[0]+(0.583083015744)*x[1]**o+(0.80826073507)*x[1]
        arg[1,0,0,1]=(0.929636966638)*x[0]**o+(-0.35107696413)*x[0]+(-0.656523378462)*x[1]**o+(0.610681274197)*x[1]
        arg[1,0,1,0]=(0.594847872723)*x[0]**o+(-0.15013091781)*x[0]+(-0.562311637909)*x[1]**o+(0.871283992407)*x[1]
        arg[1,0,1,1]=(0.529274819592)*x[0]**o+(0.998035152453)*x[0]+(-0.265309908189)*x[1]**o+(-0.703391843389)*x[1]
        arg[1,0,2,0]=(0.2321959905)*x[0]**o+(-0.781602927232)*x[0]+(-0.0509405210126)*x[1]**o+(0.839298348419)*x[1]
        arg[1,0,2,1]=(0.0811824999287)*x[0]**o+(0.916373714851)*x[0]+(0.817375877796)*x[1]**o+(-0.926274023089)*x[1]
        arg[1,1,0,0]=(0.418525096774)*x[0]**o+(0.064653235493)*x[0]+(0.15912570069)*x[1]**o+(0.47570489259)*x[1]
        arg[1,1,0,1]=(0.784138248952)*x[0]**o+(-0.715130508682)*x[0]+(-0.299421222048)*x[1]**o+(-0.816706522916)*x[1]
        arg[1,1,1,0]=(-0.911449050332)*x[0]**o+(-0.472182975049)*x[0]+(0.935180253062)*x[1]**o+(0.527208406385)*x[1]
        arg[1,1,1,1]=(-0.378301315644)*x[0]**o+(0.430065353544)*x[0]+(0.827581865806)*x[1]**o+(-0.355263817375)*x[1]
        arg[1,1,2,0]=(-0.882543004053)*x[0]**o+(0.503794963637)*x[0]+(-0.580086073626)*x[1]**o+(0.545676378283)*x[1]
        arg[1,1,2,1]=(-0.116197074531)*x[0]**o+(-0.0629871602139)*x[0]+(0.320343551712)*x[1]**o+(0.0552550885696)*x[1]
        arg[1,2,0,0]=(-0.870347384636)*x[0]**o+(-0.611439594673)*x[0]+(0.707766015783)*x[1]**o+(0.0653036268139)*x[1]
        arg[1,2,0,1]=(-0.219872666014)*x[0]**o+(-0.435867669656)*x[0]+(0.825489444837)*x[1]**o+(-0.737253859382)*x[1]
        arg[1,2,1,0]=(-0.963149287055)*x[0]**o+(0.964306345088)*x[0]+(0.502736178959)*x[1]**o+(0.998017404878)*x[1]
        arg[1,2,1,1]=(-0.586350249194)*x[0]**o+(-0.437455605038)*x[0]+(-0.182719660538)*x[1]**o+(-0.50797820305)*x[1]
        arg[1,2,2,0]=(-0.423818475166)*x[0]**o+(0.290902418459)*x[0]+(-0.304129806418)*x[1]**o+(-0.67571878331)*x[1]
        arg[1,2,2,1]=(0.901384945886)*x[0]**o+(0.0526906910209)*x[0]+(0.438755721196)*x[1]**o+(-0.774352941544)*x[1]
        arg[1,3,0,0]=(-0.797968602257)*x[0]**o+(0.856637297517)*x[0]+(0.427243495756)*x[1]**o+(0.212942988457)*x[1]
        arg[1,3,0,1]=(-0.696126104381)*x[0]**o+(0.616445534254)*x[0]+(-0.943037028688)*x[1]**o+(0.807265721375)*x[1]
        arg[1,3,1,0]=(-0.69028368426)*x[0]**o+(-0.652017674488)*x[0]+(-0.772965740874)*x[1]**o+(-0.726830598291)*x[1]
        arg[1,3,1,1]=(-0.534347181294)*x[0]**o+(-0.861020241321)*x[0]+(-0.250549859157)*x[1]**o+(-0.638289396201)*x[1]
        arg[1,3,2,0]=(-0.827981962115)*x[0]**o+(-0.427544131008)*x[0]+(-0.516514308221)*x[1]**o+(-0.101906062547)*x[1]
        arg[1,3,2,1]=(0.864701869194)*x[0]**o+(0.876327129524)*x[0]+(0.806201346091)*x[1]**o+(0.845259832272)*x[1]
        arg[1,4,0,0]=(-0.589044559231)*x[0]**o+(0.697104039773)*x[0]+(-0.288389617381)*x[1]**o+(0.422618969092)*x[1]
        arg[1,4,0,1]=(0.814971550967)*x[0]**o+(-0.665151459593)*x[0]+(-0.696145763418)*x[1]**o+(0.906949165457)*x[1]
        arg[1,4,1,0]=(0.760190450826)*x[0]**o+(-0.883964913626)*x[0]+(-0.507159320061)*x[1]**o+(-0.15707026975)*x[1]
        arg[1,4,1,1]=(-0.0416603697257)*x[0]**o+(0.560561779164)*x[0]+(0.0153574982122)*x[1]**o+(0.75228872946)*x[1]
        arg[1,4,2,0]=(0.105929120872)*x[0]**o+(0.130738968719)*x[0]+(-0.0991159921294)*x[1]**o+(0.972627650896)*x[1]
        arg[1,4,2,1]=(-0.503911541225)*x[0]**o+(-0.00357063650385)*x[0]+(0.377083955928)*x[1]**o+(0.343010153434)*x[1]
        arg[2,0,0,0]=(0.953733725947)*x[0]**o+(-0.988629842686)*x[0]+(0.196766730386)*x[1]**o+(0.0427993038279)*x[1]
        arg[2,0,0,1]=(0.491469820597)*x[0]**o+(0.708248604732)*x[0]+(-0.600347310418)*x[1]**o+(-0.0380188651191)*x[1]
        arg[2,0,1,0]=(0.615118477478)*x[0]**o+(0.832030863133)*x[0]+(0.783037278839)*x[1]**o+(-0.331824170309)*x[1]
        arg[2,0,1,1]=(-0.806125471265)*x[0]**o+(0.763233458333)*x[0]+(-0.381389710969)*x[1]**o+(0.652974431609)*x[1]
        arg[2,0,2,0]=(-0.831727679795)*x[0]**o+(0.239296785135)*x[0]+(-0.13737609858)*x[1]**o+(0.583210422723)*x[1]
        arg[2,0,2,1]=(-0.805843206194)*x[0]**o+(-0.523576669019)*x[0]+(0.159758297984)*x[1]**o+(-0.298308874886)*x[1]
        arg[2,1,0,0]=(0.589135530105)*x[0]**o+(0.272145879514)*x[0]+(0.325741074892)*x[1]**o+(-0.579782654647)*x[1]
        arg[2,1,0,1]=(0.106049062774)*x[0]**o+(0.790280056075)*x[0]+(0.0171600098883)*x[1]**o+(-0.419540445094)*x[1]
        arg[2,1,1,0]=(-0.181511670026)*x[0]**o+(-0.814531462508)*x[0]+(-0.576165158737)*x[1]**o+(0.679678310717)*x[1]
        arg[2,1,1,1]=(-0.709695453728)*x[0]**o+(-0.489487606257)*x[0]+(-0.424093531654)*x[1]**o+(0.535427169862)*x[1]
        arg[2,1,2,0]=(0.60069280023)*x[0]**o+(-0.909882731214)*x[0]+(0.640141428017)*x[1]**o+(-0.697396064954)*x[1]
        arg[2,1,2,1]=(0.896964594562)*x[0]**o+(0.976387328697)*x[0]+(0.144061490066)*x[1]**o+(0.378042869796)*x[1]
        arg[2,2,0,0]=(0.447633246258)*x[0]**o+(-0.531416367062)*x[0]+(0.199304002742)*x[1]**o+(0.85195706213)*x[1]
        arg[2,2,0,1]=(-0.683614706554)*x[0]**o+(0.933255565209)*x[0]+(-0.328047392082)*x[1]**o+(-0.0865769137182)*x[1]
        arg[2,2,1,0]=(0.22474720984)*x[0]**o+(0.0910500178874)*x[0]+(0.449220001922)*x[1]**o+(-0.0406214557829)*x[1]
        arg[2,2,1,1]=(-0.266419887395)*x[0]**o+(-0.48883285131)*x[0]+(-0.794933934805)*x[1]**o+(0.182439649969)*x[1]
        arg[2,2,2,0]=(0.949852186215)*x[0]**o+(-0.116121704259)*x[0]+(-0.977131144423)*x[1]**o+(0.698402625747)*x[1]
        arg[2,2,2,1]=(-0.0446180768654)*x[0]**o+(-0.742263614438)*x[0]+(-0.898675367333)*x[1]**o+(-0.260891146746)*x[1]
        arg[2,3,0,0]=(0.819774310005)*x[0]**o+(-0.0677371406664)*x[0]+(0.276747165681)*x[1]**o+(-0.42828783)*x[1]
        arg[2,3,0,1]=(-0.391001671385)*x[0]**o+(0.0558814994728)*x[0]+(-0.624786978129)*x[1]**o+(0.178742280566)*x[1]
        arg[2,3,1,0]=(0.904597103646)*x[0]**o+(-0.825472271707)*x[0]+(0.112221476422)*x[1]**o+(0.269349199527)*x[1]
        arg[2,3,1,1]=(-0.190955697638)*x[0]**o+(0.441698994606)*x[0]+(-0.26229840557)*x[1]**o+(0.148974540187)*x[1]
        arg[2,3,2,0]=(-0.873098622934)*x[0]**o+(-0.624915584376)*x[0]+(-0.151646578512)*x[1]**o+(-0.914878228283)*x[1]
        arg[2,3,2,1]=(-0.712569947477)*x[0]**o+(-0.72679361801)*x[0]+(-0.0946869166491)*x[1]**o+(-0.580512407933)*x[1]
        arg[2,4,0,0]=(-0.655752680768)*x[0]**o+(-0.409544975855)*x[0]+(0.865325438086)*x[1]**o+(-0.227397026937)*x[1]
        arg[2,4,0,1]=(0.670566711209)*x[0]**o+(-0.794343533883)*x[0]+(-0.127116396219)*x[1]**o+(-0.422419866647)*x[1]
        arg[2,4,1,0]=(0.46903441444)*x[0]**o+(0.0689129759715)*x[0]+(-0.549325647826)*x[1]**o+(0.893700020399)*x[1]
        arg[2,4,1,1]=(-0.0099246750652)*x[0]**o+(0.446742615509)*x[0]+(0.900251202513)*x[1]**o+(0.221599931055)*x[1]
        arg[2,4,2,0]=(0.218854380066)*x[0]**o+(0.877849336909)*x[0]+(-0.338211083089)*x[1]**o+(0.746950973063)*x[1]
        arg[2,4,2,1]=(-0.890762146523)*x[0]**o+(0.439653904008)*x[0]+(0.0802890292215)*x[1]**o+(-0.305233739845)*x[1]
        arg[3,0,0,0]=(0.284622209957)*x[0]**o+(0.983284338924)*x[0]+(-0.732588048234)*x[1]**o+(0.654066388109)*x[1]
        arg[3,0,0,1]=(0.379628076244)*x[0]**o+(-0.75362875047)*x[0]+(0.288422119968)*x[1]**o+(0.139197401407)*x[1]
        arg[3,0,1,0]=(0.0594472092467)*x[0]**o+(-0.391581973307)*x[0]+(-0.898996153568)*x[1]**o+(-2.75165483707e-05)*x[1]
        arg[3,0,1,1]=(-0.241252797172)*x[0]**o+(-0.87384982932)*x[0]+(0.811628957112)*x[1]**o+(0.0170539509618)*x[1]
        arg[3,0,2,0]=(0.255617459837)*x[0]**o+(-0.744071491889)*x[0]+(0.745552769828)*x[1]**o+(0.128612266243)*x[1]
        arg[3,0,2,1]=(0.0725082197254)*x[0]**o+(0.779602587374)*x[0]+(-0.751879287668)*x[1]**o+(0.838360641095)*x[1]
        arg[3,1,0,0]=(-0.444520674459)*x[0]**o+(-0.811442143542)*x[0]+(-0.485010478999)*x[1]**o+(0.727600464745)*x[1]
        arg[3,1,0,1]=(0.260061065042)*x[0]**o+(0.932320346965)*x[0]+(0.556377882844)*x[1]**o+(0.284348284894)*x[1]
        arg[3,1,1,0]=(0.561755002522)*x[0]**o+(-0.842983435746)*x[0]+(0.729209244377)*x[1]**o+(-0.254683846974)*x[1]
        arg[3,1,1,1]=(-0.949556399673)*x[0]**o+(-0.0804522142379)*x[0]+(0.066141483426)*x[1]**o+(0.78626145966)*x[1]
        arg[3,1,2,0]=(0.833198651881)*x[0]**o+(-0.297955946153)*x[0]+(-0.561247934789)*x[1]**o+(-0.645124432611)*x[1]
        arg[3,1,2,1]=(-0.641727656043)*x[0]**o+(0.32981201062)*x[0]+(-0.152588766563)*x[1]**o+(-0.289611460761)*x[1]
        arg[3,2,0,0]=(0.825643539598)*x[0]**o+(0.924286336436)*x[0]+(0.413089850944)*x[1]**o+(-0.640198886209)*x[1]
        arg[3,2,0,1]=(0.0855693847128)*x[0]**o+(-0.363881253576)*x[0]+(0.0397854550832)*x[1]**o+(-0.844391996875)*x[1]
        arg[3,2,1,0]=(-0.8248537795)*x[0]**o+(0.153070857979)*x[0]+(0.970009460355)*x[1]**o+(0.431000028243)*x[1]
        arg[3,2,1,1]=(-0.978024209918)*x[0]**o+(-0.527494257814)*x[0]+(-0.814691105217)*x[1]**o+(0.458334868681)*x[1]
        arg[3,2,2,0]=(0.944359157869)*x[0]**o+(-0.868016282191)*x[0]+(0.410849160429)*x[1]**o+(0.480177162607)*x[1]
        arg[3,2,2,1]=(0.109166287134)*x[0]**o+(0.940100154901)*x[0]+(0.630946061805)*x[1]**o+(0.347160020469)*x[1]
        arg[3,3,0,0]=(0.364585502975)*x[0]**o+(0.717282598047)*x[0]+(0.517805310585)*x[1]**o+(0.963452728824)*x[1]
        arg[3,3,0,1]=(0.0373453558524)*x[0]**o+(0.943887402198)*x[0]+(0.983624425071)*x[1]**o+(-0.396242794059)*x[1]
        arg[3,3,1,0]=(-0.563724031351)*x[0]**o+(0.959753988518)*x[0]+(0.118506885299)*x[1]**o+(-0.587748202342)*x[1]
        arg[3,3,1,1]=(0.460419807013)*x[0]**o+(0.237753399499)*x[0]+(0.585589833251)*x[1]**o+(0.935177937667)*x[1]
        arg[3,3,2,0]=(-0.750248086829)*x[0]**o+(-0.561629848994)*x[0]+(-0.919919077536)*x[1]**o+(0.239984179086)*x[1]
        arg[3,3,2,1]=(-0.0969897163659)*x[0]**o+(0.185433022889)*x[0]+(0.733111631837)*x[1]**o+(-0.508147081928)*x[1]
        arg[3,4,0,0]=(0.247054568876)*x[0]**o+(0.721751593201)*x[0]+(-0.684740924498)*x[1]**o+(0.28395202476)*x[1]
        arg[3,4,0,1]=(-0.0949248638736)*x[0]**o+(0.270285273316)*x[0]+(0.850836559464)*x[1]**o+(-0.771085947382)*x[1]
        arg[3,4,1,0]=(-0.965410502362)*x[0]**o+(-0.582448742001)*x[0]+(-0.90307507827)*x[1]**o+(0.914051445745)*x[1]
        arg[3,4,1,1]=(0.443340058617)*x[0]**o+(0.434462120769)*x[0]+(0.522015964301)*x[1]**o+(-0.520184389865)*x[1]
        arg[3,4,2,0]=(-0.961375562593)*x[0]**o+(-0.474630747481)*x[0]+(0.59574636341)*x[1]**o+(0.434190359663)*x[1]
        arg[3,4,2,1]=(0.174154210501)*x[0]**o+(-0.510858596799)*x[0]+(-0.0279700806065)*x[1]**o+(-0.0401612371912)*x[1]
        ref[0,0,0,0]=(-0.0352052809673)/(o+1.)+(0.903863789878)+(0.182956096072)*0.5**o
        ref[0,0,0,1]=(0.0610373691872)/(o+1.)+(-0.306742561603)+(-0.81342189211)*0.5**o
        ref[0,0,1,0]=(0.623943596759)/(o+1.)+(0.647497589315)+(-0.149103531408)*0.5**o
        ref[0,0,1,1]=(0.136600247183)/(o+1.)+(0.925469911903)+(0.74912087771)*0.5**o
        ref[0,0,2,0]=(0.966454801911)/(o+1.)+(0.0631444946098)+(-0.289042694587)*0.5**o
        ref[0,0,2,1]=(-0.156074498516)/(o+1.)+(0.047955363258)+(0.0809737838902)*0.5**o
        ref[0,1,0,0]=(-0.016865109839)/(o+1.)+(-0.404214477389)+(0.638822193145)*0.5**o
        ref[0,1,0,1]=(-0.897448372147)/(o+1.)+(-0.174669987755)+(0.611604473356)*0.5**o
        ref[0,1,1,0]=(-0.229098133178)/(o+1.)+(0.443671911949)+(-0.745407006471)*0.5**o
        ref[0,1,1,1]=(0.481206893907)/(o+1.)+(-0.111741539797)+(0.756528549281)*0.5**o
        ref[0,1,2,0]=(0.367303506039)/(o+1.)+(-0.157755527379)+(0.335544368565)*0.5**o
        ref[0,1,2,1]=(0.182483888026)/(o+1.)+(-0.157073928893)+(0.669933676654)*0.5**o
        ref[0,2,0,0]=(0.587675819052)/(o+1.)+(0.446236706377)+(-0.929163949925)*0.5**o
        ref[0,2,0,1]=(-0.83934259298)/(o+1.)+(-0.344719194607)+(0.639896773528)*0.5**o
        ref[0,2,1,0]=(0.675592435624)/(o+1.)+(0.81769507347)+(-0.399556357179)*0.5**o
        ref[0,2,1,1]=(0.14396936899)/(o+1.)+(-0.454293976313)+(-0.281969675471)*0.5**o
        ref[0,2,2,0]=(0.417557775604)/(o+1.)+(-0.430056060749)+(0.339204154215)*0.5**o
        ref[0,2,2,1]=(-0.0763188160723)/(o+1.)+(0.28785590073)+(-0.655500794874)*0.5**o
        ref[0,3,0,0]=(-0.0533973517656)/(o+1.)+(-0.619586928986)+(-0.483746115201)*0.5**o
        ref[0,3,0,1]=(-0.607238533539)/(o+1.)+(-0.899573610196)+(0.446810992304)*0.5**o
        ref[0,3,1,0]=(0.930589323947)/(o+1.)+(0.127928801455)+(-0.0235907627093)*0.5**o
        ref[0,3,1,1]=(0.237445053125)/(o+1.)+(0.503964752361)+(0.486342062258)*0.5**o
        ref[0,3,2,0]=(0.180505264557)/(o+1.)+(0.629926679618)+(-0.581025309937)*0.5**o
        ref[0,3,2,1]=(0.915270822508)/(o+1.)+(0.0599229206396)+(-0.523356041281)*0.5**o
        ref[0,4,0,0]=(-0.449731522195)/(o+1.)+(-0.0207634106763)+(-0.584782530121)*0.5**o
        ref[0,4,0,1]=(0.0374933582543)/(o+1.)+(0.175584823513)+(-0.0724106581167)*0.5**o
        ref[0,4,1,0]=(0.815864788848)/(o+1.)+(-0.4782741099)+(0.0460360663465)*0.5**o
        ref[0,4,1,1]=(-0.822961904166)/(o+1.)+(0.4384056737)+(-0.978584425712)*0.5**o
        ref[0,4,2,0]=(0.721009228711)/(o+1.)+(0.396658231667)+(-0.267227111945)*0.5**o
        ref[0,4,2,1]=(0.94281427253)/(o+1.)+(-0.888798950857)+(-0.141372777283)*0.5**o
        ref[1,0,0,0]=(0.583083015744)/(o+1.)+(0.41011820901)+(-0.721911758033)*0.5**o
        ref[1,0,0,1]=(-0.656523378462)/(o+1.)+(0.129802155034)+(0.929636966638)*0.5**o
        ref[1,0,1,0]=(-0.562311637909)/(o+1.)+(0.360576537299)+(0.594847872723)*0.5**o
        ref[1,0,1,1]=(-0.265309908189)/(o+1.)+(0.147321654532)+(0.529274819592)*0.5**o
        ref[1,0,2,0]=(-0.0509405210126)/(o+1.)+(0.0288477105933)+(0.2321959905)*0.5**o
        ref[1,0,2,1]=(0.817375877796)/(o+1.)+(-0.00495015411903)+(0.0811824999287)*0.5**o
        ref[1,1,0,0]=(0.15912570069)/(o+1.)+(0.270179064041)+(0.418525096774)*0.5**o
        ref[1,1,0,1]=(-0.299421222048)/(o+1.)+(-0.765918515799)+(0.784138248952)*0.5**o
        ref[1,1,1,0]=(0.935180253062)/(o+1.)+(0.0275127156677)+(-0.911449050332)*0.5**o
        ref[1,1,1,1]=(0.827581865806)/(o+1.)+(0.0374007680845)+(-0.378301315644)*0.5**o
        ref[1,1,2,0]=(-0.580086073626)/(o+1.)+(0.52473567096)+(-0.882543004053)*0.5**o
        ref[1,1,2,1]=(0.320343551712)/(o+1.)+(-0.00386603582212)+(-0.116197074531)*0.5**o
        ref[1,2,0,0]=(0.707766015783)/(o+1.)+(-0.273067983929)+(-0.870347384636)*0.5**o
        ref[1,2,0,1]=(0.825489444837)/(o+1.)+(-0.586560764519)+(-0.219872666014)*0.5**o
        ref[1,2,1,0]=(0.502736178959)/(o+1.)+(0.981161874983)+(-0.963149287055)*0.5**o
        ref[1,2,1,1]=(-0.182719660538)/(o+1.)+(-0.472716904044)+(-0.586350249194)*0.5**o
        ref[1,2,2,0]=(-0.304129806418)/(o+1.)+(-0.192408182425)+(-0.423818475166)*0.5**o
        ref[1,2,2,1]=(0.438755721196)/(o+1.)+(-0.360831125262)+(0.901384945886)*0.5**o
        ref[1,3,0,0]=(0.427243495756)/(o+1.)+(0.534790142987)+(-0.797968602257)*0.5**o
        ref[1,3,0,1]=(-0.943037028688)/(o+1.)+(0.711855627815)+(-0.696126104381)*0.5**o
        ref[1,3,1,0]=(-0.772965740874)/(o+1.)+(-0.689424136389)+(-0.69028368426)*0.5**o
        ref[1,3,1,1]=(-0.250549859157)/(o+1.)+(-0.749654818761)+(-0.534347181294)*0.5**o
        ref[1,3,2,0]=(-0.516514308221)/(o+1.)+(-0.264725096778)+(-0.827981962115)*0.5**o
        ref[1,3,2,1]=(0.806201346091)/(o+1.)+(0.860793480898)+(0.864701869194)*0.5**o
        ref[1,4,0,0]=(-0.288389617381)/(o+1.)+(0.559861504432)+(-0.589044559231)*0.5**o
        ref[1,4,0,1]=(-0.696145763418)/(o+1.)+(0.120898852932)+(0.814971550967)*0.5**o
        ref[1,4,1,0]=(-0.507159320061)/(o+1.)+(-0.520517591688)+(0.760190450826)*0.5**o
        ref[1,4,1,1]=(0.0153574982122)/(o+1.)+(0.656425254312)+(-0.0416603697257)*0.5**o
        ref[1,4,2,0]=(-0.0991159921294)/(o+1.)+(0.551683309807)+(0.105929120872)*0.5**o
        ref[1,4,2,1]=(0.377083955928)/(o+1.)+(0.169719758465)+(-0.503911541225)*0.5**o
        ref[2,0,0,0]=(0.196766730386)/(o+1.)+(-0.472915269429)+(0.953733725947)*0.5**o
        ref[2,0,0,1]=(-0.600347310418)/(o+1.)+(0.335114869806)+(0.491469820597)*0.5**o
        ref[2,0,1,0]=(0.783037278839)/(o+1.)+(0.250103346412)+(0.615118477478)*0.5**o
        ref[2,0,1,1]=(-0.381389710969)/(o+1.)+(0.708103944971)+(-0.806125471265)*0.5**o
        ref[2,0,2,0]=(-0.13737609858)/(o+1.)+(0.411253603929)+(-0.831727679795)*0.5**o
        ref[2,0,2,1]=(0.159758297984)/(o+1.)+(-0.410942771953)+(-0.805843206194)*0.5**o
        ref[2,1,0,0]=(0.325741074892)/(o+1.)+(-0.153818387566)+(0.589135530105)*0.5**o
        ref[2,1,0,1]=(0.0171600098883)/(o+1.)+(0.185369805491)+(0.106049062774)*0.5**o
        ref[2,1,1,0]=(-0.576165158737)/(o+1.)+(-0.0674265758955)+(-0.181511670026)*0.5**o
        ref[2,1,1,1]=(-0.424093531654)/(o+1.)+(0.0229697818029)+(-0.709695453728)*0.5**o
        ref[2,1,2,0]=(0.640141428017)/(o+1.)+(-0.803639398084)+(0.60069280023)*0.5**o
        ref[2,1,2,1]=(0.144061490066)/(o+1.)+(0.677215099246)+(0.896964594562)*0.5**o
        ref[2,2,0,0]=(0.199304002742)/(o+1.)+(0.160270347534)+(0.447633246258)*0.5**o
        ref[2,2,0,1]=(-0.328047392082)/(o+1.)+(0.423339325745)+(-0.683614706554)*0.5**o
        ref[2,2,1,0]=(0.449220001922)/(o+1.)+(0.0252142810523)+(0.22474720984)*0.5**o
        ref[2,2,1,1]=(-0.794933934805)/(o+1.)+(-0.153196600671)+(-0.266419887395)*0.5**o
        ref[2,2,2,0]=(-0.977131144423)/(o+1.)+(0.291140460744)+(0.949852186215)*0.5**o
        ref[2,2,2,1]=(-0.898675367333)/(o+1.)+(-0.501577380592)+(-0.0446180768654)*0.5**o
        ref[2,3,0,0]=(0.276747165681)/(o+1.)+(-0.248012485333)+(0.819774310005)*0.5**o
        ref[2,3,0,1]=(-0.624786978129)/(o+1.)+(0.11731189002)+(-0.391001671385)*0.5**o
        ref[2,3,1,0]=(0.112221476422)/(o+1.)+(-0.27806153609)+(0.904597103646)*0.5**o
        ref[2,3,1,1]=(-0.26229840557)/(o+1.)+(0.295336767397)+(-0.190955697638)*0.5**o
        ref[2,3,2,0]=(-0.151646578512)/(o+1.)+(-0.76989690633)+(-0.873098622934)*0.5**o
        ref[2,3,2,1]=(-0.0946869166491)/(o+1.)+(-0.653653012971)+(-0.712569947477)*0.5**o
        ref[2,4,0,0]=(0.865325438086)/(o+1.)+(-0.318471001396)+(-0.655752680768)*0.5**o
        ref[2,4,0,1]=(-0.127116396219)/(o+1.)+(-0.608381700265)+(0.670566711209)*0.5**o
        ref[2,4,1,0]=(-0.549325647826)/(o+1.)+(0.481306498185)+(0.46903441444)*0.5**o
        ref[2,4,1,1]=(0.900251202513)/(o+1.)+(0.334171273282)+(-0.0099246750652)*0.5**o
        ref[2,4,2,0]=(-0.338211083089)/(o+1.)+(0.812400154986)+(0.218854380066)*0.5**o
        ref[2,4,2,1]=(0.0802890292215)/(o+1.)+(0.0672100820814)+(-0.890762146523)*0.5**o
        ref[3,0,0,0]=(-0.732588048234)/(o+1.)+(0.818675363516)+(0.284622209957)*0.5**o
        ref[3,0,0,1]=(0.288422119968)/(o+1.)+(-0.307215674532)+(0.379628076244)*0.5**o
        ref[3,0,1,0]=(-0.898996153568)/(o+1.)+(-0.195804744928)+(0.0594472092467)*0.5**o
        ref[3,0,1,1]=(0.811628957112)/(o+1.)+(-0.428397939179)+(-0.241252797172)*0.5**o
        ref[3,0,2,0]=(0.745552769828)/(o+1.)+(-0.307729612823)+(0.255617459837)*0.5**o
        ref[3,0,2,1]=(-0.751879287668)/(o+1.)+(0.808981614234)+(0.0725082197254)*0.5**o
        ref[3,1,0,0]=(-0.485010478999)/(o+1.)+(-0.0419208393982)+(-0.444520674459)*0.5**o
        ref[3,1,0,1]=(0.556377882844)/(o+1.)+(0.60833431593)+(0.260061065042)*0.5**o
        ref[3,1,1,0]=(0.729209244377)/(o+1.)+(-0.54883364136)+(0.561755002522)*0.5**o
        ref[3,1,1,1]=(0.066141483426)/(o+1.)+(0.352904622711)+(-0.949556399673)*0.5**o
        ref[3,1,2,0]=(-0.561247934789)/(o+1.)+(-0.471540189382)+(0.833198651881)*0.5**o
        ref[3,1,2,1]=(-0.152588766563)/(o+1.)+(0.0201002749295)+(-0.641727656043)*0.5**o
        ref[3,2,0,0]=(0.413089850944)/(o+1.)+(0.142043725114)+(0.825643539598)*0.5**o
        ref[3,2,0,1]=(0.0397854550832)/(o+1.)+(-0.604136625225)+(0.0855693847128)*0.5**o
        ref[3,2,1,0]=(0.970009460355)/(o+1.)+(0.292035443111)+(-0.8248537795)*0.5**o
        ref[3,2,1,1]=(-0.814691105217)/(o+1.)+(-0.0345796945668)+(-0.978024209918)*0.5**o
        ref[3,2,2,0]=(0.410849160429)/(o+1.)+(-0.193919559792)+(0.944359157869)*0.5**o
        ref[3,2,2,1]=(0.630946061805)/(o+1.)+(0.643630087685)+(0.109166287134)*0.5**o
        ref[3,3,0,0]=(0.517805310585)/(o+1.)+(0.840367663435)+(0.364585502975)*0.5**o
        ref[3,3,0,1]=(0.983624425071)/(o+1.)+(0.273822304069)+(0.0373453558524)*0.5**o
        ref[3,3,1,0]=(0.118506885299)/(o+1.)+(0.186002893088)+(-0.563724031351)*0.5**o
        ref[3,3,1,1]=(0.585589833251)/(o+1.)+(0.586465668583)+(0.460419807013)*0.5**o
        ref[3,3,2,0]=(-0.919919077536)/(o+1.)+(-0.160822834954)+(-0.750248086829)*0.5**o
        ref[3,3,2,1]=(0.733111631837)/(o+1.)+(-0.161357029519)+(-0.0969897163659)*0.5**o
        ref[3,4,0,0]=(-0.684740924498)/(o+1.)+(0.50285180898)+(0.247054568876)*0.5**o
        ref[3,4,0,1]=(0.850836559464)/(o+1.)+(-0.250400337033)+(-0.0949248638736)*0.5**o
        ref[3,4,1,0]=(-0.90307507827)/(o+1.)+(0.165801351872)+(-0.965410502362)*0.5**o
        ref[3,4,1,1]=(0.522015964301)/(o+1.)+(-0.0428611345479)+(0.443340058617)*0.5**o
        ref[3,4,2,0]=(0.59574636341)/(o+1.)+(-0.0202201939092)+(-0.961375562593)*0.5**o
        ref[3,4,2,1]=(-0.0279700806065)/(o+1.)+(-0.275509916995)+(0.174154210501)*0.5**o
      else:
        arg[0,0,0,0]=(-0.793251694335)*x[0]**o+(0.616970271582)*x[0]+(-0.805609243189)*x[1]**o+(0.949515692474)*x[1]+(0.484069738431)*x[2]**o+(-0.0407533911672)*x[2]
        arg[0,0,0,1]=(-0.624206604764)*x[0]**o+(-0.992961148702)*x[0]+(-0.737741763879)*x[1]**o+(0.161544434398)*x[1]+(0.47292947772)*x[2]**o+(0.525538846945)*x[2]
        arg[0,0,1,0]=(0.115999561481)*x[0]**o+(-0.612694096044)*x[0]+(0.625539443386)*x[1]**o+(0.857353715882)*x[1]+(-0.0811256709155)*x[2]**o+(0.183474706394)*x[2]
        arg[0,0,1,1]=(-0.972466831671)*x[0]**o+(0.262980887655)*x[0]+(0.920924576059)*x[1]**o+(0.983517658667)*x[1]+(0.579443973505)*x[2]**o+(0.101139612823)*x[2]
        arg[0,0,2,0]=(0.708180260094)*x[0]**o+(-0.0950913766285)*x[0]+(0.361602191798)*x[1]**o+(-0.880757848201)*x[1]+(0.326272205058)*x[2]**o+(-0.997155694972)*x[2]
        arg[0,0,2,1]=(0.0282615573045)*x[0]**o+(-0.317851994728)*x[0]+(0.449330147388)*x[1]**o+(-0.156946318079)*x[1]+(0.344196894245)*x[2]**o+(-0.045082290904)*x[2]
        arg[0,1,0,0]=(0.231621820235)*x[0]**o+(0.0985804817846)*x[0]+(-0.343623333022)*x[1]**o+(0.115043475441)*x[1]+(-0.910742772023)*x[2]**o+(-0.663592068553)*x[2]
        arg[0,1,0,1]=(0.114056801155)*x[0]**o+(-0.463711315494)*x[0]+(0.380521147429)*x[1]**o+(0.294823684238)*x[1]+(0.510501672453)*x[2]**o+(-0.161234251875)*x[2]
        arg[0,1,1,0]=(-0.973115195017)*x[0]**o+(-0.0133290715609)*x[0]+(-0.642072931588)*x[1]**o+(-0.580655605735)*x[1]+(-0.191371539061)*x[2]**o+(-0.0513051439064)*x[2]
        arg[0,1,1,1]=(-0.304758579369)*x[0]**o+(-0.321904612871)*x[0]+(-0.389525877547)*x[1]**o+(-0.0602638359252)*x[1]+(0.199726168523)*x[2]**o+(-0.10313327614)*x[2]
        arg[0,1,2,0]=(0.0923844967157)*x[0]**o+(-0.369319307184)*x[0]+(-0.614761123101)*x[1]**o+(0.78817430449)*x[1]+(-0.783809995839)*x[2]**o+(-0.701522582246)*x[2]
        arg[0,1,2,1]=(0.793568921835)*x[0]**o+(0.352244892726)*x[0]+(0.185546719529)*x[1]**o+(0.145969792058)*x[1]+(0.85147546253)*x[2]**o+(-0.862069843156)*x[2]
        arg[0,2,0,0]=(0.570673248048)*x[0]**o+(0.607924942767)*x[0]+(-0.353952625913)*x[1]**o+(0.111796842318)*x[1]+(-0.765670542263)*x[2]**o+(0.589532724922)*x[2]
        arg[0,2,0,1]=(0.299061155215)*x[0]**o+(-0.542978954882)*x[0]+(0.0946151623892)*x[1]**o+(0.310266302534)*x[1]+(-0.00880229885823)*x[2]**o+(0.585508874849)*x[2]
        arg[0,2,1,0]=(0.694725244159)*x[0]**o+(0.530004132186)*x[0]+(-0.32588478629)*x[1]**o+(0.357278329148)*x[1]+(-0.93877612203)*x[2]**o+(0.34973013935)*x[2]
        arg[0,2,1,1]=(-0.83896527178)*x[0]**o+(0.487231460025)*x[0]+(0.493952010418)*x[1]**o+(0.247442509692)*x[1]+(-0.312456515075)*x[2]**o+(0.90248200924)*x[2]
        arg[0,2,2,0]=(0.2228052704)*x[0]**o+(0.901373696962)*x[0]+(0.149826948427)*x[1]**o+(0.234013159005)*x[1]+(0.717685535862)*x[2]**o+(-0.42865545543)*x[2]
        arg[0,2,2,1]=(0.63141216243)*x[0]**o+(0.448446314659)*x[0]+(0.379625538779)*x[1]**o+(-0.807448154272)*x[1]+(0.238724835223)*x[2]**o+(-0.478216911196)*x[2]
        arg[0,3,0,0]=(0.00698107166168)*x[0]**o+(-0.727207190747)*x[0]+(-0.78731647832)*x[1]**o+(0.885371753974)*x[1]+(-0.0146821479126)*x[2]**o+(-0.763310617205)*x[2]
        arg[0,3,0,1]=(0.243423742325)*x[0]**o+(0.0587322175837)*x[0]+(-0.453213790216)*x[1]**o+(0.861270481703)*x[1]+(0.867332526083)*x[2]**o+(-0.713187214416)*x[2]
        arg[0,3,1,0]=(-0.106629554393)*x[0]**o+(0.723693136654)*x[0]+(-0.0276604293602)*x[1]**o+(0.339456904686)*x[1]+(-0.102075459215)*x[2]**o+(-0.0348969356596)*x[2]
        arg[0,3,1,1]=(0.375143733111)*x[0]**o+(-0.490781234413)*x[0]+(0.168903339622)*x[1]**o+(0.594080859808)*x[1]+(-0.0802336915361)*x[2]**o+(0.472348979787)*x[2]
        arg[0,3,2,0]=(0.955685455438)*x[0]**o+(0.105142628533)*x[0]+(-0.446384800971)*x[1]**o+(-0.905998305229)*x[1]+(-0.30011637186)*x[2]**o+(0.0218195326037)*x[2]
        arg[0,3,2,1]=(0.353654445481)*x[0]**o+(-0.0477024594709)*x[0]+(-0.495075817497)*x[1]**o+(-0.425091398447)*x[1]+(-0.571207600446)*x[2]**o+(0.177935012163)*x[2]
        arg[0,4,0,0]=(0.974759684151)*x[0]**o+(-0.876975976179)*x[0]+(-0.472336406836)*x[1]**o+(0.991410874408)*x[1]+(0.297823444758)*x[2]**o+(-0.161436760659)*x[2]
        arg[0,4,0,1]=(-0.0799922241539)*x[0]**o+(0.975369691404)*x[0]+(0.302361880712)*x[1]**o+(0.171779641966)*x[1]+(0.878428630413)*x[2]**o+(0.814497247319)*x[2]
        arg[0,4,1,0]=(0.532633280629)*x[0]**o+(0.902436316468)*x[0]+(-0.676910177358)*x[1]**o+(-0.986029752926)*x[1]+(-0.519990783496)*x[2]**o+(0.430468215617)*x[2]
        arg[0,4,1,1]=(-0.353243658293)*x[0]**o+(-0.908938062877)*x[0]+(0.922933758125)*x[1]**o+(0.00500161721147)*x[1]+(0.189861558609)*x[2]**o+(-0.623307494462)*x[2]
        arg[0,4,2,0]=(-0.720840869801)*x[0]**o+(0.0349736577597)*x[0]+(-0.516608060915)*x[1]**o+(0.688756843577)*x[1]+(-0.302494303699)*x[2]**o+(-0.0703783111937)*x[2]
        arg[0,4,2,1]=(-0.179073400214)*x[0]**o+(0.128485402427)*x[0]+(-0.726789308786)*x[1]**o+(-0.691163482057)*x[1]+(0.329009412577)*x[2]**o+(-0.79293394396)*x[2]
        arg[1,0,0,0]=(-0.384011553333)*x[0]**o+(0.285271264922)*x[0]+(0.512105930181)*x[1]**o+(0.190321101872)*x[1]+(-0.509334982029)*x[2]**o+(0.128097742502)*x[2]
        arg[1,0,0,1]=(-0.808211570649)*x[0]**o+(-0.730450956462)*x[0]+(0.92309757237)*x[1]**o+(-0.703749565272)*x[1]+(0.301129716544)*x[2]**o+(0.244379489105)*x[2]
        arg[1,0,1,0]=(0.0404274840294)*x[0]**o+(-0.483903392253)*x[0]+(0.73680959705)*x[1]**o+(0.190850758522)*x[1]+(0.133136432605)*x[2]**o+(0.910055186498)*x[2]
        arg[1,0,1,1]=(0.302394139901)*x[0]**o+(0.802524687002)*x[0]+(0.970871728408)*x[1]**o+(0.207348092291)*x[1]+(0.666310912781)*x[2]**o+(0.510392805767)*x[2]
        arg[1,0,2,0]=(-0.981981997649)*x[0]**o+(-0.465269233707)*x[0]+(0.916146418639)*x[1]**o+(0.143602438774)*x[1]+(-0.947874263512)*x[2]**o+(0.0840394012229)*x[2]
        arg[1,0,2,1]=(-0.237479805943)*x[0]**o+(-0.89993757244)*x[0]+(-0.759650487895)*x[1]**o+(-0.807139164373)*x[1]+(-0.377055204014)*x[2]**o+(-0.57656201026)*x[2]
        arg[1,1,0,0]=(-0.543719871098)*x[0]**o+(-0.726308160246)*x[0]+(0.821244616742)*x[1]**o+(0.718112961487)*x[1]+(-0.655445553192)*x[2]**o+(-0.599104608292)*x[2]
        arg[1,1,0,1]=(-0.2624625471)*x[0]**o+(0.85250071151)*x[0]+(-0.665066081561)*x[1]**o+(-0.566502138196)*x[1]+(-0.572858653628)*x[2]**o+(-0.862890633323)*x[2]
        arg[1,1,1,0]=(0.88003513115)*x[0]**o+(-0.0150959972442)*x[0]+(-0.698880737368)*x[1]**o+(-0.236909810427)*x[1]+(-0.721315930706)*x[2]**o+(-0.219894530829)*x[2]
        arg[1,1,1,1]=(0.177746615088)*x[0]**o+(0.205790814183)*x[0]+(-0.216021190604)*x[1]**o+(0.384079010991)*x[1]+(0.61634008905)*x[2]**o+(0.940900776108)*x[2]
        arg[1,1,2,0]=(0.889070121473)*x[0]**o+(0.68131533527)*x[0]+(0.709207182081)*x[1]**o+(-0.765184991252)*x[1]+(0.506328018952)*x[2]**o+(-0.46041174473)*x[2]
        arg[1,1,2,1]=(0.951736215954)*x[0]**o+(-0.147499702642)*x[0]+(0.60348358384)*x[1]**o+(0.777404433567)*x[1]+(-0.482441452679)*x[2]**o+(0.714835256434)*x[2]
        arg[1,2,0,0]=(-0.954701036263)*x[0]**o+(-0.360901887475)*x[0]+(-0.644285028623)*x[1]**o+(0.169271524632)*x[1]+(-0.687047203668)*x[2]**o+(0.205115312108)*x[2]
        arg[1,2,0,1]=(0.256489282842)*x[0]**o+(0.17406448362)*x[0]+(0.97500243816)*x[1]**o+(0.614577997602)*x[1]+(0.0537536804583)*x[2]**o+(-0.211963274032)*x[2]
        arg[1,2,1,0]=(-0.94015914517)*x[0]**o+(0.169982490736)*x[0]+(0.482085495724)*x[1]**o+(-0.556994404153)*x[1]+(-0.809311296883)*x[2]**o+(0.719907503024)*x[2]
        arg[1,2,1,1]=(0.367156421931)*x[0]**o+(0.489648825153)*x[0]+(-0.0398225517816)*x[1]**o+(-0.495726296032)*x[1]+(0.359374061053)*x[2]**o+(0.231991952482)*x[2]
        arg[1,2,2,0]=(-0.812516694781)*x[0]**o+(-0.631181257391)*x[0]+(-0.723784471687)*x[1]**o+(0.178976877342)*x[1]+(-0.629293758013)*x[2]**o+(-0.313897724608)*x[2]
        arg[1,2,2,1]=(-0.497461944186)*x[0]**o+(0.268998525103)*x[0]+(-0.478017492026)*x[1]**o+(0.026395792841)*x[1]+(0.581256386137)*x[2]**o+(-0.476961644324)*x[2]
        arg[1,3,0,0]=(-0.227077550639)*x[0]**o+(0.400407424161)*x[0]+(-0.53529812303)*x[1]**o+(-0.418594299853)*x[1]+(-0.195190916346)*x[2]**o+(-0.624284483774)*x[2]
        arg[1,3,0,1]=(0.822091021645)*x[0]**o+(-0.31133140026)*x[0]+(0.0665130965049)*x[1]**o+(0.596697216047)*x[1]+(0.466195828646)*x[2]**o+(-0.477988391299)*x[2]
        arg[1,3,1,0]=(-0.0341577310045)*x[0]**o+(-0.77936644392)*x[0]+(0.954811382784)*x[1]**o+(0.837374830631)*x[1]+(0.738992234768)*x[2]**o+(-0.501187788985)*x[2]
        arg[1,3,1,1]=(-0.102867224088)*x[0]**o+(0.00829793194765)*x[0]+(0.081790556957)*x[1]**o+(-0.779351210472)*x[1]+(-0.735321115716)*x[2]**o+(0.126919542418)*x[2]
        arg[1,3,2,0]=(0.556301076488)*x[0]**o+(0.0923434629749)*x[0]+(-0.191048735713)*x[1]**o+(-0.218833028687)*x[1]+(-0.088867516146)*x[2]**o+(-0.247214200389)*x[2]
        arg[1,3,2,1]=(0.888436531887)*x[0]**o+(-0.79225495914)*x[0]+(0.0154788262219)*x[1]**o+(-0.210622890864)*x[1]+(0.782629134632)*x[2]**o+(0.350772185791)*x[2]
        arg[1,4,0,0]=(0.651898073685)*x[0]**o+(-0.84098616689)*x[0]+(0.674088959443)*x[1]**o+(-0.658824952898)*x[1]+(-0.423551541558)*x[2]**o+(0.985098530356)*x[2]
        arg[1,4,0,1]=(-0.149187282052)*x[0]**o+(0.0880370482988)*x[0]+(0.780809276595)*x[1]**o+(-0.610833366472)*x[1]+(0.794946384304)*x[2]**o+(-0.817031796429)*x[2]
        arg[1,4,1,0]=(-0.285938055108)*x[0]**o+(-0.106165154734)*x[0]+(-0.948376845189)*x[1]**o+(0.874607682056)*x[1]+(-0.100051825909)*x[2]**o+(-0.483640835026)*x[2]
        arg[1,4,1,1]=(-0.34015136472)*x[0]**o+(0.991331180352)*x[0]+(-0.925796518928)*x[1]**o+(-0.872775691287)*x[1]+(0.953898056819)*x[2]**o+(0.985025365955)*x[2]
        arg[1,4,2,0]=(-0.464687865356)*x[0]**o+(-0.439271850879)*x[0]+(0.302716649876)*x[1]**o+(0.235186289965)*x[1]+(0.672062761925)*x[2]**o+(-0.793689262392)*x[2]
        arg[1,4,2,1]=(-0.777967458564)*x[0]**o+(0.0565891925223)*x[0]+(-0.27539733623)*x[1]**o+(0.252329868046)*x[1]+(-0.269529792944)*x[2]**o+(-0.953051723483)*x[2]
        arg[2,0,0,0]=(0.665321754689)*x[0]**o+(0.00639681939987)*x[0]+(0.262193891905)*x[1]**o+(-0.312800735151)*x[1]+(0.8403244871)*x[2]**o+(-0.607084328204)*x[2]
        arg[2,0,0,1]=(0.876435018569)*x[0]**o+(-0.388250784543)*x[0]+(-0.799866349878)*x[1]**o+(0.114896594902)*x[1]+(-0.438064877186)*x[2]**o+(-0.458871344685)*x[2]
        arg[2,0,1,0]=(-0.344552466761)*x[0]**o+(0.967217640605)*x[0]+(-0.00183230845245)*x[1]**o+(-0.861206591247)*x[1]+(0.795588257851)*x[2]**o+(-0.333607767932)*x[2]
        arg[2,0,1,1]=(0.841069648591)*x[0]**o+(0.580162460873)*x[0]+(-0.0277035983747)*x[1]**o+(-0.404616536792)*x[1]+(-0.403195438454)*x[2]**o+(-0.202877947947)*x[2]
        arg[2,0,2,0]=(0.192691357169)*x[0]**o+(-0.519968251714)*x[0]+(-0.575397578091)*x[1]**o+(-0.029378989829)*x[1]+(-0.958861195684)*x[2]**o+(0.848705509543)*x[2]
        arg[2,0,2,1]=(0.556616365284)*x[0]**o+(0.679993071506)*x[0]+(-0.800652717274)*x[1]**o+(0.979182847258)*x[1]+(-0.501953582042)*x[2]**o+(0.127003985335)*x[2]
        arg[2,1,0,0]=(0.947750123445)*x[0]**o+(-0.0350691553359)*x[0]+(0.0650485999092)*x[1]**o+(-0.131216537555)*x[1]+(0.888782559831)*x[2]**o+(0.0338703818654)*x[2]
        arg[2,1,0,1]=(0.962918564559)*x[0]**o+(-0.119367421389)*x[0]+(0.43003549616)*x[1]**o+(-0.097883763913)*x[1]+(0.230422206305)*x[2]**o+(0.187273479711)*x[2]
        arg[2,1,1,0]=(-0.549909211973)*x[0]**o+(0.263443098451)*x[0]+(-0.476703606205)*x[1]**o+(0.216159301182)*x[1]+(-0.561829938323)*x[2]**o+(0.943016557203)*x[2]
        arg[2,1,1,1]=(0.382867749635)*x[0]**o+(0.404317006935)*x[0]+(0.786117847428)*x[1]**o+(-0.80719258612)*x[1]+(0.471144889777)*x[2]**o+(0.6478813349)*x[2]
        arg[2,1,2,0]=(0.355742024596)*x[0]**o+(-0.728812391059)*x[0]+(0.766039604734)*x[1]**o+(-0.532629598363)*x[1]+(-0.518284303077)*x[2]**o+(-0.972348067947)*x[2]
        arg[2,1,2,1]=(-0.882334243081)*x[0]**o+(-0.0940633993231)*x[0]+(-0.00166643832426)*x[1]**o+(-0.971243725495)*x[1]+(-0.0422614083707)*x[2]**o+(-0.547881520447)*x[2]
        arg[2,2,0,0]=(0.372164678051)*x[0]**o+(-0.0644361811234)*x[0]+(0.886078780084)*x[1]**o+(-0.459054628604)*x[1]+(-0.983880547097)*x[2]**o+(-0.669749873181)*x[2]
        arg[2,2,0,1]=(0.126650330827)*x[0]**o+(0.375111974063)*x[0]+(-0.759797654917)*x[1]**o+(-0.663216091322)*x[1]+(-0.683191373782)*x[2]**o+(-0.607048449469)*x[2]
        arg[2,2,1,0]=(-0.805920040628)*x[0]**o+(-0.0144685877057)*x[0]+(0.00875222056291)*x[1]**o+(-0.110000267929)*x[1]+(-0.478201664476)*x[2]**o+(-0.0537439708493)*x[2]
        arg[2,2,1,1]=(-0.550624517803)*x[0]**o+(0.570407090431)*x[0]+(0.661026826148)*x[1]**o+(0.905189813242)*x[1]+(0.00707531661352)*x[2]**o+(0.711072343064)*x[2]
        arg[2,2,2,0]=(-0.644276976064)*x[0]**o+(-0.151665972531)*x[0]+(-0.645369927259)*x[1]**o+(0.915967227715)*x[1]+(-0.211750817665)*x[2]**o+(-0.596646212118)*x[2]
        arg[2,2,2,1]=(0.564688999107)*x[0]**o+(0.635094410913)*x[0]+(0.281863287848)*x[1]**o+(-0.927529659408)*x[1]+(0.581071087484)*x[2]**o+(0.164399277305)*x[2]
        arg[2,3,0,0]=(0.64754765253)*x[0]**o+(0.176693707964)*x[0]+(0.172801737347)*x[1]**o+(-0.769408746261)*x[1]+(-0.176026486601)*x[2]**o+(0.187304474822)*x[2]
        arg[2,3,0,1]=(0.365315558093)*x[0]**o+(0.888853264239)*x[0]+(0.0541759434362)*x[1]**o+(0.241489226711)*x[1]+(0.441822201832)*x[2]**o+(0.075541840076)*x[2]
        arg[2,3,1,0]=(-0.984940830356)*x[0]**o+(0.521596714942)*x[0]+(-0.933671223761)*x[1]**o+(-0.132907654205)*x[1]+(-0.225804727932)*x[2]**o+(0.987334445626)*x[2]
        arg[2,3,1,1]=(-0.785516613303)*x[0]**o+(0.0324126413814)*x[0]+(-0.084233004699)*x[1]**o+(0.45131442059)*x[1]+(0.36498759129)*x[2]**o+(0.417036982561)*x[2]
        arg[2,3,2,0]=(-0.310575473358)*x[0]**o+(-0.769219454218)*x[0]+(-0.263431454577)*x[1]**o+(0.474143448264)*x[1]+(-0.490633240671)*x[2]**o+(0.435609988849)*x[2]
        arg[2,3,2,1]=(0.230960030802)*x[0]**o+(-0.480593443732)*x[0]+(0.547888534735)*x[1]**o+(-0.818842709599)*x[1]+(-0.857141410253)*x[2]**o+(-0.287898487847)*x[2]
        arg[2,4,0,0]=(-0.811008715357)*x[0]**o+(-0.608505768579)*x[0]+(-0.117642307757)*x[1]**o+(-0.658229045784)*x[1]+(0.246542958193)*x[2]**o+(-0.42783248265)*x[2]
        arg[2,4,0,1]=(0.817994599948)*x[0]**o+(0.394931907893)*x[0]+(-0.273993145403)*x[1]**o+(0.9572397013)*x[1]+(0.101025833331)*x[2]**o+(-0.925874548613)*x[2]
        arg[2,4,1,0]=(-0.626152000269)*x[0]**o+(0.821322294557)*x[0]+(0.543761841392)*x[1]**o+(0.112341361443)*x[1]+(-0.395657376418)*x[2]**o+(0.815481187683)*x[2]
        arg[2,4,1,1]=(0.97134483493)*x[0]**o+(0.303341541402)*x[0]+(-0.270102237344)*x[1]**o+(-0.982452271352)*x[1]+(0.435568243011)*x[2]**o+(-0.200051496445)*x[2]
        arg[2,4,2,0]=(-0.827528679064)*x[0]**o+(-0.559447627475)*x[0]+(-0.867397899368)*x[1]**o+(0.915539157819)*x[1]+(0.370118757042)*x[2]**o+(0.578366179408)*x[2]
        arg[2,4,2,1]=(0.685882920875)*x[0]**o+(0.830016468585)*x[0]+(-0.603607199824)*x[1]**o+(0.888619939961)*x[1]+(-0.264173896988)*x[2]**o+(-0.588065135532)*x[2]
        arg[3,0,0,0]=(0.480000662344)*x[0]**o+(-0.240072007989)*x[0]+(0.252669294205)*x[1]**o+(-0.657466732916)*x[1]+(0.945149322572)*x[2]**o+(0.340592627149)*x[2]
        arg[3,0,0,1]=(-0.819956304014)*x[0]**o+(0.910408628621)*x[0]+(-0.875714560314)*x[1]**o+(0.653323198674)*x[1]+(0.910127896386)*x[2]**o+(0.21382952806)*x[2]
        arg[3,0,1,0]=(0.736655857249)*x[0]**o+(-0.982636511328)*x[0]+(0.747965385081)*x[1]**o+(-0.952056699676)*x[1]+(0.143168283422)*x[2]**o+(0.891386576579)*x[2]
        arg[3,0,1,1]=(0.615723453617)*x[0]**o+(0.57376968582)*x[0]+(0.766802940979)*x[1]**o+(-0.294849095727)*x[1]+(-0.26718123792)*x[2]**o+(-0.979175148343)*x[2]
        arg[3,0,2,0]=(-0.299975789693)*x[0]**o+(-0.941879795797)*x[0]+(-0.332633604206)*x[1]**o+(0.887589360228)*x[1]+(0.570321438438)*x[2]**o+(0.107180499249)*x[2]
        arg[3,0,2,1]=(-0.77479605873)*x[0]**o+(0.928359807986)*x[0]+(-0.144170026638)*x[1]**o+(0.533835316696)*x[1]+(0.183938136052)*x[2]**o+(-0.87336340483)*x[2]
        arg[3,1,0,0]=(-0.121858163593)*x[0]**o+(-0.971122306623)*x[0]+(-0.351891554168)*x[1]**o+(-0.989754547278)*x[1]+(0.48106592456)*x[2]**o+(0.750009716044)*x[2]
        arg[3,1,0,1]=(-0.332320017719)*x[0]**o+(0.229814969323)*x[0]+(0.950038106614)*x[1]**o+(-0.364221636516)*x[1]+(-0.842684764717)*x[2]**o+(-0.430003876831)*x[2]
        arg[3,1,1,0]=(0.1036461168)*x[0]**o+(0.243047542195)*x[0]+(-0.189881740385)*x[1]**o+(-0.646948186795)*x[1]+(-0.246523431838)*x[2]**o+(0.0363222140239)*x[2]
        arg[3,1,1,1]=(0.629747493968)*x[0]**o+(0.257110968013)*x[0]+(-0.594332699815)*x[1]**o+(-0.74041867705)*x[1]+(0.656583120031)*x[2]**o+(0.392776534802)*x[2]
        arg[3,1,2,0]=(0.997368307137)*x[0]**o+(-0.709004475665)*x[0]+(-0.379447862403)*x[1]**o+(-0.368700793823)*x[1]+(0.293673648471)*x[2]**o+(-0.555545691082)*x[2]
        arg[3,1,2,1]=(-0.323027033928)*x[0]**o+(0.804640557762)*x[0]+(-0.199434780533)*x[1]**o+(0.604194682736)*x[1]+(0.945516011113)*x[2]**o+(-0.831838977373)*x[2]
        arg[3,2,0,0]=(0.255071922458)*x[0]**o+(0.261075290259)*x[0]+(0.198792431961)*x[1]**o+(0.987681568369)*x[1]+(-0.257626079249)*x[2]**o+(0.817801578613)*x[2]
        arg[3,2,0,1]=(-0.732615916833)*x[0]**o+(0.8114347023)*x[0]+(0.308896981216)*x[1]**o+(-0.164981244979)*x[1]+(0.724411853266)*x[2]**o+(-0.647969863039)*x[2]
        arg[3,2,1,0]=(0.126755528121)*x[0]**o+(0.0203968097627)*x[0]+(0.350888385037)*x[1]**o+(0.765584878761)*x[1]+(0.43521215637)*x[2]**o+(0.5821821143)*x[2]
        arg[3,2,1,1]=(0.795956472753)*x[0]**o+(0.363631111688)*x[0]+(-0.296417527594)*x[1]**o+(-0.0927953540926)*x[1]+(0.396041397741)*x[2]**o+(0.43932000066)*x[2]
        arg[3,2,2,0]=(-0.926754341344)*x[0]**o+(-0.688414966522)*x[0]+(-0.660984101932)*x[1]**o+(-0.627883344982)*x[1]+(0.403036784646)*x[2]**o+(0.818277068314)*x[2]
        arg[3,2,2,1]=(-0.358921307079)*x[0]**o+(0.636847962491)*x[0]+(-0.510050460541)*x[1]**o+(-0.758190944228)*x[1]+(-0.0319326931965)*x[2]**o+(-0.643422203077)*x[2]
        arg[3,3,0,0]=(-0.930730482483)*x[0]**o+(-0.883991363505)*x[0]+(0.156751809769)*x[1]**o+(0.669949596594)*x[1]+(0.697840698407)*x[2]**o+(0.721622061073)*x[2]
        arg[3,3,0,1]=(0.108814498453)*x[0]**o+(-0.164024398834)*x[0]+(-0.172084798233)*x[1]**o+(0.031530303081)*x[1]+(-0.985663868487)*x[2]**o+(-0.290224642955)*x[2]
        arg[3,3,1,0]=(0.690575802875)*x[0]**o+(-0.720428203968)*x[0]+(-0.837111899789)*x[1]**o+(0.140256279088)*x[1]+(0.532813462778)*x[2]**o+(-0.557885762558)*x[2]
        arg[3,3,1,1]=(-0.0785388159589)*x[0]**o+(-0.40595445804)*x[0]+(-0.691255332383)*x[1]**o+(0.54108326284)*x[1]+(0.737684932212)*x[2]**o+(0.423508847197)*x[2]
        arg[3,3,2,0]=(-0.589838770269)*x[0]**o+(-0.557133995504)*x[0]+(-0.297626023793)*x[1]**o+(-0.511809000987)*x[1]+(-0.933566600332)*x[2]**o+(-0.21434993446)*x[2]
        arg[3,3,2,1]=(0.19892263441)*x[0]**o+(0.633952342142)*x[0]+(0.0179400729177)*x[1]**o+(0.154020947355)*x[1]+(-0.620690264897)*x[2]**o+(0.524853027077)*x[2]
        arg[3,4,0,0]=(0.475231682303)*x[0]**o+(-0.48174419167)*x[0]+(-0.100854189221)*x[1]**o+(0.818166518054)*x[1]+(-0.922566309657)*x[2]**o+(-0.498745068755)*x[2]
        arg[3,4,0,1]=(0.613982633884)*x[0]**o+(-0.947323459945)*x[0]+(0.17395204013)*x[1]**o+(0.929669087853)*x[1]+(-0.617278089466)*x[2]**o+(0.900537470392)*x[2]
        arg[3,4,1,0]=(-0.403633510324)*x[0]**o+(0.954097943555)*x[0]+(-0.463422043607)*x[1]**o+(0.843916487223)*x[1]+(-0.5955912138)*x[2]**o+(-0.765444657727)*x[2]
        arg[3,4,1,1]=(-0.663412886802)*x[0]**o+(-0.547440885077)*x[0]+(-0.479993570508)*x[1]**o+(0.0620303034445)*x[1]+(-0.653759917895)*x[2]**o+(-0.681004441743)*x[2]
        arg[3,4,2,0]=(-0.994529375243)*x[0]**o+(0.546983479745)*x[0]+(0.17913912435)*x[1]**o+(0.660366383627)*x[1]+(0.0747943517387)*x[2]**o+(-0.321063660452)*x[2]
        arg[3,4,2,1]=(-0.538074654323)*x[0]**o+(-0.81330034965)*x[0]+(0.660614896398)*x[1]**o+(-0.918192909648)*x[1]+(0.553635166581)*x[2]**o+(-0.918131802189)*x[2]
        ref[0,0,0,0]=(-0.321539504757)/(o+1.)+(0.762866286444)+(-0.793251694335)*0.5**o
        ref[0,0,0,1]=(-0.264812286159)/(o+1.)+(-0.15293893368)+(-0.624206604764)*0.5**o
        ref[0,0,1,0]=(0.54441377247)/(o+1.)+(0.214067163116)+(0.115999561481)*0.5**o
        ref[0,0,1,1]=(1.50036854956)/(o+1.)+(0.673819079573)+(-0.972466831671)*0.5**o
        ref[0,0,2,0]=(0.687874396856)/(o+1.)+(-0.986502459901)+(0.708180260094)*0.5**o
        ref[0,0,2,1]=(0.793527041633)/(o+1.)+(-0.259940301856)+(0.0282615573045)*0.5**o
        ref[0,1,0,0]=(-1.25436610505)/(o+1.)+(-0.224984055664)+(0.231621820235)*0.5**o
        ref[0,1,0,1]=(0.891022819882)/(o+1.)+(-0.165060941565)+(0.114056801155)*0.5**o
        ref[0,1,1,0]=(-0.833444470649)/(o+1.)+(-0.322644910601)+(-0.973115195017)*0.5**o
        ref[0,1,1,1]=(-0.189799709023)/(o+1.)+(-0.242650862468)+(-0.304758579369)*0.5**o
        ref[0,1,2,0]=(-1.39857111894)/(o+1.)+(-0.14133379247)+(0.0923844967157)*0.5**o
        ref[0,1,2,1]=(1.03702218206)/(o+1.)+(-0.181927579186)+(0.793568921835)*0.5**o
        ref[0,2,0,0]=(-1.11962316818)/(o+1.)+(0.654627255004)+(0.570673248048)*0.5**o
        ref[0,2,0,1]=(0.085812863531)/(o+1.)+(0.17639811125)+(0.299061155215)*0.5**o
        ref[0,2,1,0]=(-1.26466090832)/(o+1.)+(0.618506300342)+(0.694725244159)*0.5**o
        ref[0,2,1,1]=(0.181495495342)/(o+1.)+(0.818577989479)+(-0.83896527178)*0.5**o
        ref[0,2,2,0]=(0.867512484289)/(o+1.)+(0.353365700269)+(0.2228052704)*0.5**o
        ref[0,2,2,1]=(0.618350374002)/(o+1.)+(-0.418609375405)+(0.63141216243)*0.5**o
        ref[0,3,0,0]=(-0.801998626232)/(o+1.)+(-0.302573026989)+(0.00698107166168)*0.5**o
        ref[0,3,0,1]=(0.414118735868)/(o+1.)+(0.103407742435)+(0.243423742325)*0.5**o
        ref[0,3,1,0]=(-0.129735888575)/(o+1.)+(0.51412655284)+(-0.106629554393)*0.5**o
        ref[0,3,1,1]=(0.0886696480864)/(o+1.)+(0.287824302591)+(0.375143733111)*0.5**o
        ref[0,3,2,0]=(-0.746501172831)/(o+1.)+(-0.389518072046)+(0.955685455438)*0.5**o
        ref[0,3,2,1]=(-1.06628341794)/(o+1.)+(-0.147429422878)+(0.353654445481)*0.5**o
        ref[0,4,0,0]=(-0.174512962078)/(o+1.)+(-0.023500931215)+(0.974759684151)*0.5**o
        ref[0,4,0,1]=(1.18079051113)/(o+1.)+(0.980823290345)+(-0.0799922241539)*0.5**o
        ref[0,4,1,0]=(-1.19690096085)/(o+1.)+(0.173437389579)+(0.532633280629)*0.5**o
        ref[0,4,1,1]=(1.11279531673)/(o+1.)+(-0.763621970064)+(-0.353243658293)*0.5**o
        ref[0,4,2,0]=(-0.819102364613)/(o+1.)+(0.326676095071)+(-0.720840869801)*0.5**o
        ref[0,4,2,1]=(-0.397779896209)/(o+1.)+(-0.677806011795)+(-0.179073400214)*0.5**o
        ref[1,0,0,0]=(0.00277094815158)/(o+1.)+(0.301845054648)+(-0.384011553333)*0.5**o
        ref[1,0,0,1]=(1.22422728891)/(o+1.)+(-0.594910516315)+(-0.808211570649)*0.5**o
        ref[1,0,1,0]=(0.869946029655)/(o+1.)+(0.308501276384)+(0.0404274840294)*0.5**o
        ref[1,0,1,1]=(1.63718264119)/(o+1.)+(0.76013279253)+(0.302394139901)*0.5**o
        ref[1,0,2,0]=(-0.0317278448724)/(o+1.)+(-0.118813696855)+(-0.981981997649)*0.5**o
        ref[1,0,2,1]=(-1.13670569191)/(o+1.)+(-1.14181937354)+(-0.237479805943)*0.5**o
        ref[1,1,0,0]=(0.16579906355)/(o+1.)+(-0.303649903525)+(-0.543719871098)*0.5**o
        ref[1,1,0,1]=(-1.23792473519)/(o+1.)+(-0.288446030005)+(-0.2624625471)*0.5**o
        ref[1,1,1,0]=(-1.42019666807)/(o+1.)+(-0.23595016925)+(0.88003513115)*0.5**o
        ref[1,1,1,1]=(0.400318898446)/(o+1.)+(0.765385300641)+(0.177746615088)*0.5**o
        ref[1,1,2,0]=(1.21553520103)/(o+1.)+(-0.272140700356)+(0.889070121473)*0.5**o
        ref[1,1,2,1]=(0.121042131161)/(o+1.)+(0.672369993679)+(0.951736215954)*0.5**o
        ref[1,2,0,0]=(-1.33133223229)/(o+1.)+(0.0067424746325)+(-0.954701036263)*0.5**o
        ref[1,2,0,1]=(1.02875611862)/(o+1.)+(0.288339603595)+(0.256489282842)*0.5**o
        ref[1,2,1,0]=(-0.32722580116)/(o+1.)+(0.166447794803)+(-0.94015914517)*0.5**o
        ref[1,2,1,1]=(0.319551509271)/(o+1.)+(0.112957240802)+(0.367156421931)*0.5**o
        ref[1,2,2,0]=(-1.3530782297)/(o+1.)+(-0.383051052329)+(-0.812516694781)*0.5**o
        ref[1,2,2,1]=(0.103238894111)/(o+1.)+(-0.09078366319)+(-0.497461944186)*0.5**o
        ref[1,3,0,0]=(-0.730489039376)/(o+1.)+(-0.321235679733)+(-0.227077550639)*0.5**o
        ref[1,3,0,1]=(0.532708925151)/(o+1.)+(-0.0963112877562)+(0.822091021645)*0.5**o
        ref[1,3,1,0]=(1.69380361755)/(o+1.)+(-0.221589701137)+(-0.0341577310045)*0.5**o
        ref[1,3,1,1]=(-0.653530558759)/(o+1.)+(-0.322066868053)+(-0.102867224088)*0.5**o
        ref[1,3,2,0]=(-0.279916251859)/(o+1.)+(-0.18685188305)+(0.556301076488)*0.5**o
        ref[1,3,2,1]=(0.798107960854)/(o+1.)+(-0.326052832107)+(0.888436531887)*0.5**o
        ref[1,4,0,0]=(0.250537417885)/(o+1.)+(-0.257356294716)+(0.651898073685)*0.5**o
        ref[1,4,0,1]=(1.5757556609)/(o+1.)+(-0.669914057301)+(-0.149187282052)*0.5**o
        ref[1,4,1,0]=(-1.0484286711)/(o+1.)+(0.142400846148)+(-0.285938055108)*0.5**o
        ref[1,4,1,1]=(0.028101537891)/(o+1.)+(0.55179042751)+(-0.34015136472)*0.5**o
        ref[1,4,2,0]=(0.974779411801)/(o+1.)+(-0.498887411653)+(-0.464687865356)*0.5**o
        ref[1,4,2,1]=(-0.544927129174)/(o+1.)+(-0.322066331457)+(-0.777967458564)*0.5**o
        ref[2,0,0,0]=(1.102518379)/(o+1.)+(-0.456744121978)+(0.665321754689)*0.5**o
        ref[2,0,0,1]=(-1.23793122706)/(o+1.)+(-0.366112767163)+(0.876435018569)*0.5**o
        ref[2,0,1,0]=(0.793755949398)/(o+1.)+(-0.113798359287)+(-0.344552466761)*0.5**o
        ref[2,0,1,1]=(-0.430899036829)/(o+1.)+(-0.0136660119329)+(0.841069648591)*0.5**o
        ref[2,0,2,0]=(-1.53425877377)/(o+1.)+(0.149679134)+(0.192691357169)*0.5**o
        ref[2,0,2,1]=(-1.30260629932)/(o+1.)+(0.89308995205)+(0.556616365284)*0.5**o
        ref[2,1,0,0]=(0.95383115974)/(o+1.)+(-0.0662076555126)+(0.947750123445)*0.5**o
        ref[2,1,0,1]=(0.660457702465)/(o+1.)+(-0.0149888527952)+(0.962918564559)*0.5**o
        ref[2,1,1,0]=(-1.03853354453)/(o+1.)+(0.711309478418)+(-0.549909211973)*0.5**o
        ref[2,1,1,1]=(1.25726273721)/(o+1.)+(0.122502877857)+(0.382867749635)*0.5**o
        ref[2,1,2,0]=(0.247755301657)/(o+1.)+(-1.11689502868)+(0.355742024596)*0.5**o
        ref[2,1,2,1]=(-0.043927846695)/(o+1.)+(-0.806594322632)+(-0.882334243081)*0.5**o
        ref[2,2,0,0]=(-0.0978017670124)/(o+1.)+(-0.596620341454)+(0.372164678051)*0.5**o
        ref[2,2,0,1]=(-1.4429890287)/(o+1.)+(-0.447576283364)+(0.126650330827)*0.5**o
        ref[2,2,1,0]=(-0.469449443913)/(o+1.)+(-0.0891064132419)+(-0.805920040628)*0.5**o
        ref[2,2,1,1]=(0.668102142762)/(o+1.)+(1.09333462337)+(-0.550624517803)*0.5**o
        ref[2,2,2,0]=(-0.857120744924)/(o+1.)+(0.0838275215328)+(-0.644276976064)*0.5**o
        ref[2,2,2,1]=(0.862934375332)/(o+1.)+(-0.0640179855954)+(0.564688999107)*0.5**o
        ref[2,3,0,0]=(-0.00322474925369)/(o+1.)+(-0.202705281737)+(0.64754765253)*0.5**o
        ref[2,3,0,1]=(0.495998145268)/(o+1.)+(0.602942165513)+(0.365315558093)*0.5**o
        ref[2,3,1,0]=(-1.15947595169)/(o+1.)+(0.688011753181)+(-0.984940830356)*0.5**o
        ref[2,3,1,1]=(0.280754586591)/(o+1.)+(0.450382022266)+(-0.785516613303)*0.5**o
        ref[2,3,2,0]=(-0.754064695248)/(o+1.)+(0.0702669914474)+(-0.310575473358)*0.5**o
        ref[2,3,2,1]=(-0.309252875518)/(o+1.)+(-0.793667320589)+(0.230960030802)*0.5**o
        ref[2,4,0,0]=(0.128900650436)/(o+1.)+(-0.847283648506)+(-0.811008715357)*0.5**o
        ref[2,4,0,1]=(-0.172967312072)/(o+1.)+(0.21314853029)+(0.817994599948)*0.5**o
        ref[2,4,1,0]=(0.148104464974)/(o+1.)+(0.874572421841)+(-0.626152000269)*0.5**o
        ref[2,4,1,1]=(0.165466005667)/(o+1.)+(-0.439581113198)+(0.97134483493)*0.5**o
        ref[2,4,2,0]=(-0.497279142326)/(o+1.)+(0.467228854876)+(-0.827528679064)*0.5**o
        ref[2,4,2,1]=(-0.867781096812)/(o+1.)+(0.565285636507)+(0.685882920875)*0.5**o
        ref[3,0,0,0]=(1.19781861678)/(o+1.)+(-0.278473056878)+(0.480000662344)*0.5**o
        ref[3,0,0,1]=(0.0344133360711)/(o+1.)+(0.888780677678)+(-0.819956304014)*0.5**o
        ref[3,0,1,0]=(0.891133668503)/(o+1.)+(-0.521653317213)+(0.736655857249)*0.5**o
        ref[3,0,1,1]=(0.499621703059)/(o+1.)+(-0.350127279125)+(0.615723453617)*0.5**o
        ref[3,0,2,0]=(0.237687834232)/(o+1.)+(0.0264450318399)+(-0.299975789693)*0.5**o
        ref[3,0,2,1]=(0.0397681094138)/(o+1.)+(0.294415859926)+(-0.77479605873)*0.5**o
        ref[3,1,0,0]=(0.129174370391)/(o+1.)+(-0.605433568929)+(-0.121858163593)*0.5**o
        ref[3,1,0,1]=(0.107353341897)/(o+1.)+(-0.282205272012)+(-0.332320017719)*0.5**o
        ref[3,1,1,0]=(-0.436405172223)/(o+1.)+(-0.183789215288)+(0.1036461168)*0.5**o
        ref[3,1,1,1]=(0.0622504202157)/(o+1.)+(-0.0452655871179)+(0.629747493968)*0.5**o
        ref[3,1,2,0]=(-0.0857742139322)/(o+1.)+(-0.816625480285)+(0.997368307137)*0.5**o
        ref[3,1,2,1]=(0.746081230581)/(o+1.)+(0.288498131562)+(-0.323027033928)*0.5**o
        ref[3,2,0,0]=(-0.0588336472879)/(o+1.)+(1.03327921862)+(0.255071922458)*0.5**o
        ref[3,2,0,1]=(1.03330883448)/(o+1.)+(-0.000758202858916)+(-0.732615916833)*0.5**o
        ref[3,2,1,0]=(0.786100541406)/(o+1.)+(0.684081901412)+(0.126755528121)*0.5**o
        ref[3,2,1,1]=(0.099623870147)/(o+1.)+(0.355077879128)+(0.795956472753)*0.5**o
        ref[3,2,2,0]=(-0.257947317287)/(o+1.)+(-0.249010621595)+(-0.926754341344)*0.5**o
        ref[3,2,2,1]=(-0.541983153738)/(o+1.)+(-0.382382592407)+(-0.358921307079)*0.5**o
        ref[3,3,0,0]=(0.854592508176)/(o+1.)+(0.253790147081)+(-0.930730482483)*0.5**o
        ref[3,3,0,1]=(-1.15774866672)/(o+1.)+(-0.211359369354)+(0.108814498453)*0.5**o
        ref[3,3,1,0]=(-0.304298437011)/(o+1.)+(-0.569028843719)+(0.690575802875)*0.5**o
        ref[3,3,1,1]=(0.0464295998283)/(o+1.)+(0.279318825998)+(-0.0785388159589)*0.5**o
        ref[3,3,2,0]=(-1.23119262412)/(o+1.)+(-0.641646465476)+(-0.589838770269)*0.5**o
        ref[3,3,2,1]=(-0.60275019198)/(o+1.)+(0.656413158287)+(0.19892263441)*0.5**o
        ref[3,4,0,0]=(-1.02342049888)/(o+1.)+(-0.0811613711857)+(0.475231682303)*0.5**o
        ref[3,4,0,1]=(-0.443326049337)/(o+1.)+(0.44144154915)+(0.613982633884)*0.5**o
        ref[3,4,1,0]=(-1.05901325741)/(o+1.)+(0.516284886526)+(-0.403633510324)*0.5**o
        ref[3,4,1,1]=(-1.1337534884)/(o+1.)+(-0.583207511688)+(-0.663412886802)*0.5**o
        ref[3,4,2,0]=(0.253933476088)/(o+1.)+(0.44314310146)+(-0.994529375243)*0.5**o
        ref[3,4,2,1]=(1.21425006298)/(o+1.)+(-1.32481253074)+(-0.538074654323)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactOne_fromData_Solution_rank0(self):
      """
      tests integral of rank 0 Data on the FunctionOnContactOne

      assumptions: Solution(self.domain) exists
                   self.domain supports integral on FunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      w=Solution(self.domain)
      x=w.getX()
      arg=Data(0,(),w)
      if dim==2:
        arg=(-0.441098788712)*x[0]**o+(0.082953252724)*x[0]+(-0.97005044508)*x[1]**o+(0.928239302707)*x[1]
        ref=(-0.97005044508)/(o+1.)+(0.505596277715)+(-0.441098788712)*0.5**o
      else:
        arg=(0.384995424392)*x[0]**o+(0.999298118235)*x[0]+(0.144745460948)*x[1]**o+(0.0545198589076)*x[1]+(-0.773375797864)*x[2]**o+(0.145524911495)*x[2]
        ref=(-0.628630336917)/(o+1.)+(0.599671444318)+(0.384995424392)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactOne_fromData_Solution_rank1(self):
      """
      tests integral of rank 1 Data on the FunctionOnContactOne

      assumptions: Solution(self.domain) exists
                   self.domain supports integral on FunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      w=Solution(self.domain)
      x=w.getX()
      arg=Data(0,(2,),w)
      ref=numpy.zeros((2,),numpy.float_)
      if dim==2:
        arg[0]=(0.0745889108767)*x[0]**o+(-0.694812761318)*x[0]+(-0.357791726239)*x[1]**o+(-0.785448350489)*x[1]
        arg[1]=(0.436006633056)*x[0]**o+(-0.797943632034)*x[0]+(-0.903321835716)*x[1]**o+(0.299527545727)*x[1]
        ref[0]=(-0.357791726239)/(o+1.)+(-0.740130555904)+(0.0745889108767)*0.5**o
        ref[1]=(-0.903321835716)/(o+1.)+(-0.249208043154)+(0.436006633056)*0.5**o
      else:
        arg[0]=(-0.292048194246)*x[0]**o+(-0.612402340241)*x[0]+(0.830253896674)*x[1]**o+(-0.0173245051245)*x[1]+(-0.105062444571)*x[2]**o+(0.968232377167)*x[2]
        arg[1]=(0.153118126553)*x[0]**o+(0.958628221601)*x[0]+(-0.326368437133)*x[1]**o+(0.878861879269)*x[1]+(-0.297398071379)*x[2]**o+(0.155202299471)*x[2]
        ref[0]=(0.725191452103)/(o+1.)+(0.169252765901)+(-0.292048194246)*0.5**o
        ref[1]=(-0.623766508512)/(o+1.)+(0.99634620017)+(0.153118126553)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactOne_fromData_Solution_rank2(self):
      """
      tests integral of rank 2 Data on the FunctionOnContactOne

      assumptions: Solution(self.domain) exists
                   self.domain supports integral on FunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      w=Solution(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5),w)
      ref=numpy.zeros((4, 5),numpy.float_)
      if dim==2:
        arg[0,0]=(0.410346849722)*x[0]**o+(0.86102012198)*x[0]+(-0.0143604505833)*x[1]**o+(-0.333268586255)*x[1]
        arg[0,1]=(-0.104991947387)*x[0]**o+(0.51540973662)*x[0]+(0.449773354197)*x[1]**o+(0.0397236625379)*x[1]
        arg[0,2]=(0.0010671797269)*x[0]**o+(0.165202831822)*x[0]+(-0.536444395178)*x[1]**o+(0.142112903729)*x[1]
        arg[0,3]=(0.0655247081801)*x[0]**o+(-0.59093644572)*x[0]+(-0.0135713515051)*x[1]**o+(0.665578431202)*x[1]
        arg[0,4]=(0.902382714765)*x[0]**o+(-0.358716290301)*x[0]+(-0.504792876433)*x[1]**o+(-0.760982991404)*x[1]
        arg[1,0]=(0.663815334483)*x[0]**o+(-0.967978720711)*x[0]+(-0.308953852146)*x[1]**o+(-0.250315651366)*x[1]
        arg[1,1]=(0.602976841368)*x[0]**o+(0.407346455361)*x[0]+(0.820076160142)*x[1]**o+(0.720380314572)*x[1]
        arg[1,2]=(0.0899286747557)*x[0]**o+(-0.636748133402)*x[0]+(0.615853620374)*x[1]**o+(0.389136375831)*x[1]
        arg[1,3]=(-0.695859848989)*x[0]**o+(-0.160892907203)*x[0]+(0.972252211687)*x[1]**o+(0.971653054313)*x[1]
        arg[1,4]=(-0.229141368314)*x[0]**o+(0.434757135281)*x[0]+(0.508751791725)*x[1]**o+(-0.593022987916)*x[1]
        arg[2,0]=(-0.732297666821)*x[0]**o+(-0.885623150041)*x[0]+(0.355264312146)*x[1]**o+(-0.203582600498)*x[1]
        arg[2,1]=(-0.0683706126862)*x[0]**o+(0.390953181924)*x[0]+(-0.104115827873)*x[1]**o+(-0.0586349400277)*x[1]
        arg[2,2]=(0.213052201915)*x[0]**o+(0.813758731408)*x[0]+(-0.927085393446)*x[1]**o+(-0.70884993627)*x[1]
        arg[2,3]=(-0.11792311156)*x[0]**o+(0.705053056049)*x[0]+(-0.988858028527)*x[1]**o+(-0.532493233644)*x[1]
        arg[2,4]=(0.643258167584)*x[0]**o+(0.991059864292)*x[0]+(0.466734669253)*x[1]**o+(-0.763209421816)*x[1]
        arg[3,0]=(-0.686036752646)*x[0]**o+(-0.0152278415398)*x[0]+(0.572818298467)*x[1]**o+(0.0883817883272)*x[1]
        arg[3,1]=(-0.498088836117)*x[0]**o+(0.413958831603)*x[0]+(-0.788338642889)*x[1]**o+(-0.219412111863)*x[1]
        arg[3,2]=(0.895103303091)*x[0]**o+(0.967224972282)*x[0]+(0.778129303031)*x[1]**o+(-0.239067231978)*x[1]
        arg[3,3]=(-0.715385920659)*x[0]**o+(-0.928446013585)*x[0]+(-0.919578735328)*x[1]**o+(0.735438220506)*x[1]
        arg[3,4]=(0.273825880641)*x[0]**o+(0.602304312788)*x[0]+(-0.788415866411)*x[1]**o+(0.841776933557)*x[1]
        ref[0,0]=(-0.0143604505833)/(o+1.)+(0.263875767862)+(0.410346849722)*0.5**o
        ref[0,1]=(0.449773354197)/(o+1.)+(0.277566699579)+(-0.104991947387)*0.5**o
        ref[0,2]=(-0.536444395178)/(o+1.)+(0.153657867776)+(0.0010671797269)*0.5**o
        ref[0,3]=(-0.0135713515051)/(o+1.)+(0.0373209927411)+(0.0655247081801)*0.5**o
        ref[0,4]=(-0.504792876433)/(o+1.)+(-0.559849640852)+(0.902382714765)*0.5**o
        ref[1,0]=(-0.308953852146)/(o+1.)+(-0.609147186038)+(0.663815334483)*0.5**o
        ref[1,1]=(0.820076160142)/(o+1.)+(0.563863384966)+(0.602976841368)*0.5**o
        ref[1,2]=(0.615853620374)/(o+1.)+(-0.123805878786)+(0.0899286747557)*0.5**o
        ref[1,3]=(0.972252211687)/(o+1.)+(0.405380073555)+(-0.695859848989)*0.5**o
        ref[1,4]=(0.508751791725)/(o+1.)+(-0.0791329263176)+(-0.229141368314)*0.5**o
        ref[2,0]=(0.355264312146)/(o+1.)+(-0.544602875269)+(-0.732297666821)*0.5**o
        ref[2,1]=(-0.104115827873)/(o+1.)+(0.166159120948)+(-0.0683706126862)*0.5**o
        ref[2,2]=(-0.927085393446)/(o+1.)+(0.0524543975689)+(0.213052201915)*0.5**o
        ref[2,3]=(-0.988858028527)/(o+1.)+(0.0862799112023)+(-0.11792311156)*0.5**o
        ref[2,4]=(0.466734669253)/(o+1.)+(0.113925221238)+(0.643258167584)*0.5**o
        ref[3,0]=(0.572818298467)/(o+1.)+(0.0365769733937)+(-0.686036752646)*0.5**o
        ref[3,1]=(-0.788338642889)/(o+1.)+(0.0972733598697)+(-0.498088836117)*0.5**o
        ref[3,2]=(0.778129303031)/(o+1.)+(0.364078870152)+(0.895103303091)*0.5**o
        ref[3,3]=(-0.919578735328)/(o+1.)+(-0.0965038965398)+(-0.715385920659)*0.5**o
        ref[3,4]=(-0.788415866411)/(o+1.)+(0.722040623173)+(0.273825880641)*0.5**o
      else:
        arg[0,0]=(-0.903197979191)*x[0]**o+(-0.497252911882)*x[0]+(0.5689188153)*x[1]**o+(-0.569434759407)*x[1]+(-0.204533481399)*x[2]**o+(-0.0255207088467)*x[2]
        arg[0,1]=(-0.0296606374337)*x[0]**o+(0.917552577919)*x[0]+(0.746132694062)*x[1]**o+(0.285568252958)*x[1]+(0.175571603104)*x[2]**o+(0.5701931124)*x[2]
        arg[0,2]=(0.657981464039)*x[0]**o+(-0.105816675859)*x[0]+(-0.210726712386)*x[1]**o+(-0.457303699577)*x[1]+(-0.225517453595)*x[2]**o+(0.61593257413)*x[2]
        arg[0,3]=(-0.652766009035)*x[0]**o+(0.538908603782)*x[0]+(0.226594150058)*x[1]**o+(0.691428293374)*x[1]+(-0.210896072508)*x[2]**o+(0.538061905948)*x[2]
        arg[0,4]=(0.175024193446)*x[0]**o+(0.433285241007)*x[0]+(0.928411521986)*x[1]**o+(-0.241175862661)*x[1]+(-0.779192552542)*x[2]**o+(0.428100531631)*x[2]
        arg[1,0]=(-0.094230207815)*x[0]**o+(-0.113255272088)*x[0]+(-0.880816596679)*x[1]**o+(0.56261116141)*x[1]+(-0.418606336511)*x[2]**o+(0.858058623759)*x[2]
        arg[1,1]=(0.08556853498)*x[0]**o+(0.95491440456)*x[0]+(-0.435275492231)*x[1]**o+(0.339105115812)*x[1]+(-0.634101556975)*x[2]**o+(0.357557194572)*x[2]
        arg[1,2]=(-0.692883812524)*x[0]**o+(0.673540357384)*x[0]+(0.240219465837)*x[1]**o+(0.139957925862)*x[1]+(0.135892430596)*x[2]**o+(0.383465818668)*x[2]
        arg[1,3]=(-0.892921935206)*x[0]**o+(0.0694647682287)*x[0]+(0.0530888770821)*x[1]**o+(-0.0735398003251)*x[1]+(-0.40702392212)*x[2]**o+(-0.223494881098)*x[2]
        arg[1,4]=(0.743112585337)*x[0]**o+(-0.12393012942)*x[0]+(-0.660616664197)*x[1]**o+(-0.0334489536139)*x[1]+(-0.0566394013558)*x[2]**o+(-0.0281273872158)*x[2]
        arg[2,0]=(-0.500500871517)*x[0]**o+(-0.326237506809)*x[0]+(-0.960480146386)*x[1]**o+(0.0901654083154)*x[1]+(0.466124845857)*x[2]**o+(0.6908848619)*x[2]
        arg[2,1]=(0.120233846989)*x[0]**o+(0.520609850629)*x[0]+(-0.407423400749)*x[1]**o+(0.59145151128)*x[1]+(0.981171407459)*x[2]**o+(-0.703987165108)*x[2]
        arg[2,2]=(0.811775583161)*x[0]**o+(-0.209650983138)*x[0]+(-0.286223486261)*x[1]**o+(-0.864122954179)*x[1]+(0.609725467363)*x[2]**o+(0.0311464898903)*x[2]
        arg[2,3]=(-0.325647787579)*x[0]**o+(0.956001955119)*x[0]+(0.637272546129)*x[1]**o+(-0.951109798103)*x[1]+(0.406445558466)*x[2]**o+(-0.930854597943)*x[2]
        arg[2,4]=(0.0621315789778)*x[0]**o+(-0.917033227802)*x[0]+(-0.179584904974)*x[1]**o+(0.283357518809)*x[1]+(-0.778863909832)*x[2]**o+(-0.524497331838)*x[2]
        arg[3,0]=(0.140605486894)*x[0]**o+(0.172882849759)*x[0]+(-0.883586756712)*x[1]**o+(-0.160461884399)*x[1]+(-0.718343713337)*x[2]**o+(0.196937721256)*x[2]
        arg[3,1]=(-0.660232764173)*x[0]**o+(-0.92091042631)*x[0]+(-0.568852134661)*x[1]**o+(-0.331441330509)*x[1]+(-0.220286922375)*x[2]**o+(-0.256407439105)*x[2]
        arg[3,2]=(0.593940468197)*x[0]**o+(0.527756437117)*x[0]+(0.943980933807)*x[1]**o+(0.749415167795)*x[1]+(0.484043332105)*x[2]**o+(0.40135030381)*x[2]
        arg[3,3]=(0.870508483705)*x[0]**o+(-0.234209640456)*x[0]+(-0.62692310882)*x[1]**o+(0.352628981854)*x[1]+(-0.24937844453)*x[2]**o+(0.73438735864)*x[2]
        arg[3,4]=(-0.638437449353)*x[0]**o+(0.308007229542)*x[0]+(0.537590844528)*x[1]**o+(-0.755595388361)*x[1]+(-0.152781228745)*x[2]**o+(-0.450122095362)*x[2]
        ref[0,0]=(0.364385333901)/(o+1.)+(-0.546104190068)+(-0.903197979191)*0.5**o
        ref[0,1]=(0.921704297167)/(o+1.)+(0.886656971638)+(-0.0296606374337)*0.5**o
        ref[0,2]=(-0.436244165981)/(o+1.)+(0.0264060993469)+(0.657981464039)*0.5**o
        ref[0,3]=(0.0156980775503)/(o+1.)+(0.884199401552)+(-0.652766009035)*0.5**o
        ref[0,4]=(0.149218969444)/(o+1.)+(0.310104954989)+(0.175024193446)*0.5**o
        ref[1,0]=(-1.29942293319)/(o+1.)+(0.653707256541)+(-0.094230207815)*0.5**o
        ref[1,1]=(-1.06937704921)/(o+1.)+(0.825788357473)+(0.08556853498)*0.5**o
        ref[1,2]=(0.376111896433)/(o+1.)+(0.598482050957)+(-0.692883812524)*0.5**o
        ref[1,3]=(-0.353935045038)/(o+1.)+(-0.113784956597)+(-0.892921935206)*0.5**o
        ref[1,4]=(-0.717256065552)/(o+1.)+(-0.0927532351247)+(0.743112585337)*0.5**o
        ref[2,0]=(-0.494355300528)/(o+1.)+(0.227406381703)+(-0.500500871517)*0.5**o
        ref[2,1]=(0.57374800671)/(o+1.)+(0.2040370984)+(0.120233846989)*0.5**o
        ref[2,2]=(0.323501981101)/(o+1.)+(-0.521313723714)+(0.811775583161)*0.5**o
        ref[2,3]=(1.04371810459)/(o+1.)+(-0.462981220464)+(-0.325647787579)*0.5**o
        ref[2,4]=(-0.958448814807)/(o+1.)+(-0.579086520416)+(0.0621315789778)*0.5**o
        ref[3,0]=(-1.60193047005)/(o+1.)+(0.104679343308)+(0.140605486894)*0.5**o
        ref[3,1]=(-0.789139057035)/(o+1.)+(-0.754379597962)+(-0.660232764173)*0.5**o
        ref[3,2]=(1.42802426591)/(o+1.)+(0.839260954361)+(0.593940468197)*0.5**o
        ref[3,3]=(-0.87630155335)/(o+1.)+(0.426403350019)+(0.870508483705)*0.5**o
        ref[3,4]=(0.384809615783)/(o+1.)+(-0.448855127091)+(-0.638437449353)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactOne_fromData_Solution_rank3(self):
      """
      tests integral of rank 3 Data on the FunctionOnContactOne

      assumptions: Solution(self.domain) exists
                   self.domain supports integral on FunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      w=Solution(self.domain)
      x=w.getX()
      arg=Data(0,(6, 2, 2),w)
      ref=numpy.zeros((6, 2, 2),numpy.float_)
      if dim==2:
        arg[0,0,0]=(-0.315756069525)*x[0]**o+(0.253240552004)*x[0]+(0.86540335823)*x[1]**o+(-0.616203780065)*x[1]
        arg[0,0,1]=(-0.948116617977)*x[0]**o+(0.892154336439)*x[0]+(0.136605864823)*x[1]**o+(-0.135599413854)*x[1]
        arg[0,1,0]=(0.249388669107)*x[0]**o+(-0.299043026617)*x[0]+(0.335963868826)*x[1]**o+(-0.147202749474)*x[1]
        arg[0,1,1]=(0.649003124445)*x[0]**o+(0.102344972673)*x[0]+(-0.795038258171)*x[1]**o+(0.149716983842)*x[1]
        arg[1,0,0]=(0.814606374941)*x[0]**o+(-0.949251236145)*x[0]+(0.723727126173)*x[1]**o+(-0.75419822708)*x[1]
        arg[1,0,1]=(0.043251358201)*x[0]**o+(-0.227748374318)*x[0]+(-0.231556019421)*x[1]**o+(-0.152524439909)*x[1]
        arg[1,1,0]=(0.565989211326)*x[0]**o+(-0.488248650798)*x[0]+(-0.353077780419)*x[1]**o+(-0.0139123365766)*x[1]
        arg[1,1,1]=(-0.106085432662)*x[0]**o+(0.627311916563)*x[0]+(-0.496301561324)*x[1]**o+(-0.648448849796)*x[1]
        arg[2,0,0]=(0.741868098354)*x[0]**o+(-0.486898566862)*x[0]+(0.0461464851414)*x[1]**o+(0.0133916949979)*x[1]
        arg[2,0,1]=(0.969108066213)*x[0]**o+(-0.688797899381)*x[0]+(0.897017408844)*x[1]**o+(0.807714585761)*x[1]
        arg[2,1,0]=(-0.138159486947)*x[0]**o+(-0.377759480876)*x[0]+(-0.764484355987)*x[1]**o+(-0.723912265361)*x[1]
        arg[2,1,1]=(-0.781167132076)*x[0]**o+(0.119512137054)*x[0]+(-0.535230596139)*x[1]**o+(0.441358968417)*x[1]
        arg[3,0,0]=(-0.663820836909)*x[0]**o+(0.907668749761)*x[0]+(-0.345114213135)*x[1]**o+(0.0580058218243)*x[1]
        arg[3,0,1]=(-0.359054701663)*x[0]**o+(0.0389188839076)*x[0]+(0.168050801441)*x[1]**o+(-0.695770024565)*x[1]
        arg[3,1,0]=(0.70118269628)*x[0]**o+(0.87976625188)*x[0]+(0.154503286758)*x[1]**o+(0.807878896951)*x[1]
        arg[3,1,1]=(-0.143516320927)*x[0]**o+(-0.369452129865)*x[0]+(-0.0392268449697)*x[1]**o+(-0.046104227955)*x[1]
        arg[4,0,0]=(0.197244254754)*x[0]**o+(-0.580444991466)*x[0]+(0.334036744845)*x[1]**o+(0.000791083620504)*x[1]
        arg[4,0,1]=(-0.892626226865)*x[0]**o+(0.140042741589)*x[0]+(0.158775259845)*x[1]**o+(-0.308266331028)*x[1]
        arg[4,1,0]=(0.650374542633)*x[0]**o+(0.586152174865)*x[0]+(0.0201324360301)*x[1]**o+(-0.813221777119)*x[1]
        arg[4,1,1]=(0.149656968314)*x[0]**o+(0.700641349434)*x[0]+(-0.529780404433)*x[1]**o+(0.512159137234)*x[1]
        arg[5,0,0]=(0.288664621864)*x[0]**o+(0.119652133593)*x[0]+(-0.599113573241)*x[1]**o+(0.879966724309)*x[1]
        arg[5,0,1]=(0.861443356689)*x[0]**o+(0.530607121787)*x[0]+(0.240306513701)*x[1]**o+(-0.575772515225)*x[1]
        arg[5,1,0]=(0.721635890008)*x[0]**o+(-0.109537583449)*x[0]+(0.162041613112)*x[1]**o+(0.229143033467)*x[1]
        arg[5,1,1]=(-0.876930820494)*x[0]**o+(0.749378811252)*x[0]+(0.886213554632)*x[1]**o+(0.433327797349)*x[1]
        ref[0,0,0]=(0.86540335823)/(o+1.)+(-0.18148161403)+(-0.315756069525)*0.5**o
        ref[0,0,1]=(0.136605864823)/(o+1.)+(0.378277461292)+(-0.948116617977)*0.5**o
        ref[0,1,0]=(0.335963868826)/(o+1.)+(-0.223122888045)+(0.249388669107)*0.5**o
        ref[0,1,1]=(-0.795038258171)/(o+1.)+(0.126030978258)+(0.649003124445)*0.5**o
        ref[1,0,0]=(0.723727126173)/(o+1.)+(-0.851724731613)+(0.814606374941)*0.5**o
        ref[1,0,1]=(-0.231556019421)/(o+1.)+(-0.190136407114)+(0.043251358201)*0.5**o
        ref[1,1,0]=(-0.353077780419)/(o+1.)+(-0.251080493687)+(0.565989211326)*0.5**o
        ref[1,1,1]=(-0.496301561324)/(o+1.)+(-0.0105684666165)+(-0.106085432662)*0.5**o
        ref[2,0,0]=(0.0461464851414)/(o+1.)+(-0.236753435932)+(0.741868098354)*0.5**o
        ref[2,0,1]=(0.897017408844)/(o+1.)+(0.05945834319)+(0.969108066213)*0.5**o
        ref[2,1,0]=(-0.764484355987)/(o+1.)+(-0.550835873118)+(-0.138159486947)*0.5**o
        ref[2,1,1]=(-0.535230596139)/(o+1.)+(0.280435552736)+(-0.781167132076)*0.5**o
        ref[3,0,0]=(-0.345114213135)/(o+1.)+(0.482837285793)+(-0.663820836909)*0.5**o
        ref[3,0,1]=(0.168050801441)/(o+1.)+(-0.328425570329)+(-0.359054701663)*0.5**o
        ref[3,1,0]=(0.154503286758)/(o+1.)+(0.843822574415)+(0.70118269628)*0.5**o
        ref[3,1,1]=(-0.0392268449697)/(o+1.)+(-0.20777817891)+(-0.143516320927)*0.5**o
        ref[4,0,0]=(0.334036744845)/(o+1.)+(-0.289826953923)+(0.197244254754)*0.5**o
        ref[4,0,1]=(0.158775259845)/(o+1.)+(-0.0841117947196)+(-0.892626226865)*0.5**o
        ref[4,1,0]=(0.0201324360301)/(o+1.)+(-0.113534801127)+(0.650374542633)*0.5**o
        ref[4,1,1]=(-0.529780404433)/(o+1.)+(0.606400243334)+(0.149656968314)*0.5**o
        ref[5,0,0]=(-0.599113573241)/(o+1.)+(0.499809428951)+(0.288664621864)*0.5**o
        ref[5,0,1]=(0.240306513701)/(o+1.)+(-0.022582696719)+(0.861443356689)*0.5**o
        ref[5,1,0]=(0.162041613112)/(o+1.)+(0.0598027250089)+(0.721635890008)*0.5**o
        ref[5,1,1]=(0.886213554632)/(o+1.)+(0.5913533043)+(-0.876930820494)*0.5**o
      else:
        arg[0,0,0]=(0.804252705823)*x[0]**o+(-0.215722921442)*x[0]+(-0.806316349529)*x[1]**o+(-0.984002993222)*x[1]+(-0.411946014731)*x[2]**o+(-0.526356191855)*x[2]
        arg[0,0,1]=(-0.845902068304)*x[0]**o+(0.0773568471048)*x[0]+(-0.746877301439)*x[1]**o+(-0.581914187059)*x[1]+(-0.0299873869697)*x[2]**o+(-0.133702230427)*x[2]
        arg[0,1,0]=(0.0113663840954)*x[0]**o+(-0.155702215771)*x[0]+(-0.134490626112)*x[1]**o+(0.973329917011)*x[1]+(-0.71585624135)*x[2]**o+(-0.840113198085)*x[2]
        arg[0,1,1]=(0.708589516593)*x[0]**o+(0.40282025998)*x[0]+(0.114498353117)*x[1]**o+(0.117244230433)*x[1]+(0.110135318853)*x[2]**o+(-0.0227879044898)*x[2]
        arg[1,0,0]=(0.836102144172)*x[0]**o+(0.212564417583)*x[0]+(0.744354739679)*x[1]**o+(-0.265633978841)*x[1]+(0.843108381264)*x[2]**o+(-0.288428427501)*x[2]
        arg[1,0,1]=(-0.0712209756059)*x[0]**o+(-0.186158140814)*x[0]+(0.184457099888)*x[1]**o+(0.381365494077)*x[1]+(-0.547874738232)*x[2]**o+(0.645124013254)*x[2]
        arg[1,1,0]=(-0.982992052158)*x[0]**o+(0.259627201342)*x[0]+(-0.885159598031)*x[1]**o+(-0.720754022982)*x[1]+(-0.762444117798)*x[2]**o+(0.737661938668)*x[2]
        arg[1,1,1]=(0.518426662725)*x[0]**o+(0.718331452733)*x[0]+(0.523870992899)*x[1]**o+(0.527831420124)*x[1]+(0.791634534316)*x[2]**o+(0.17320870773)*x[2]
        arg[2,0,0]=(-0.0368697940364)*x[0]**o+(-0.101718169515)*x[0]+(0.933712377315)*x[1]**o+(-0.465193151914)*x[1]+(-0.113637506449)*x[2]**o+(0.606619949802)*x[2]
        arg[2,0,1]=(-0.785633726304)*x[0]**o+(0.526518355588)*x[0]+(0.151859749675)*x[1]**o+(0.805094094642)*x[1]+(-0.881122119524)*x[2]**o+(0.85143600659)*x[2]
        arg[2,1,0]=(0.969244001518)*x[0]**o+(0.22600349917)*x[0]+(0.404826692149)*x[1]**o+(0.633039564539)*x[1]+(0.470567669552)*x[2]**o+(0.378348877016)*x[2]
        arg[2,1,1]=(-0.310035929524)*x[0]**o+(0.42632271287)*x[0]+(0.866930472029)*x[1]**o+(-0.124483457934)*x[1]+(0.291920406068)*x[2]**o+(0.573984377154)*x[2]
        arg[3,0,0]=(0.245553971829)*x[0]**o+(-0.567563766526)*x[0]+(0.326654705159)*x[1]**o+(0.642370867777)*x[1]+(-0.538450732704)*x[2]**o+(0.773349470647)*x[2]
        arg[3,0,1]=(0.427017913395)*x[0]**o+(0.885132021286)*x[0]+(0.0338747045378)*x[1]**o+(-0.978508839789)*x[1]+(-0.734559666249)*x[2]**o+(0.379991690191)*x[2]
        arg[3,1,0]=(-0.29533334088)*x[0]**o+(-0.894548834889)*x[0]+(0.537043269081)*x[1]**o+(-0.559173506199)*x[1]+(0.359216600837)*x[2]**o+(0.269563745611)*x[2]
        arg[3,1,1]=(-0.146152544907)*x[0]**o+(-0.95681567795)*x[0]+(-0.0323978120064)*x[1]**o+(-0.5351716012)*x[1]+(-0.548536561192)*x[2]**o+(0.489675460511)*x[2]
        arg[4,0,0]=(0.0552991386013)*x[0]**o+(0.772417012152)*x[0]+(-0.298212559744)*x[1]**o+(-0.189934984078)*x[1]+(0.74843296731)*x[2]**o+(-0.00412448772016)*x[2]
        arg[4,0,1]=(0.577046858044)*x[0]**o+(-0.804142676861)*x[0]+(-0.0703211154568)*x[1]**o+(-0.84092953544)*x[1]+(0.254119640557)*x[2]**o+(-0.765048199333)*x[2]
        arg[4,1,0]=(-0.358558179011)*x[0]**o+(-0.998072158283)*x[0]+(0.508152998513)*x[1]**o+(-0.0813296609683)*x[1]+(-0.534332816004)*x[2]**o+(-0.189623317277)*x[2]
        arg[4,1,1]=(0.840098089823)*x[0]**o+(0.966726062679)*x[0]+(0.920079609258)*x[1]**o+(-0.242532131681)*x[1]+(0.429713768025)*x[2]**o+(0.171223902751)*x[2]
        arg[5,0,0]=(0.695331559401)*x[0]**o+(0.819482211381)*x[0]+(0.550452400412)*x[1]**o+(0.518496496336)*x[1]+(-0.24727850431)*x[2]**o+(-0.796406493488)*x[2]
        arg[5,0,1]=(0.469488028534)*x[0]**o+(0.457530124732)*x[0]+(0.922206279456)*x[1]**o+(0.678758573006)*x[1]+(0.34102098746)*x[2]**o+(0.108616278675)*x[2]
        arg[5,1,0]=(0.444799933589)*x[0]**o+(-0.361568786934)*x[0]+(0.470174557338)*x[1]**o+(-0.344557214342)*x[1]+(0.894860467988)*x[2]**o+(-0.347027743579)*x[2]
        arg[5,1,1]=(0.0304781332872)*x[0]**o+(-0.69046475182)*x[0]+(0.873260975232)*x[1]**o+(-0.705100555993)*x[1]+(0.660057279013)*x[2]**o+(-0.910002146105)*x[2]
        ref[0,0,0]=(-1.21826236426)/(o+1.)+(-0.863041053259)+(0.804252705823)*0.5**o
        ref[0,0,1]=(-0.776864688409)/(o+1.)+(-0.31912978519)+(-0.845902068304)*0.5**o
        ref[0,1,0]=(-0.850346867462)/(o+1.)+(-0.0112427484226)+(0.0113663840954)*0.5**o
        ref[0,1,1]=(0.22463367197)/(o+1.)+(0.248638292962)+(0.708589516593)*0.5**o
        ref[1,0,0]=(1.58746312094)/(o+1.)+(-0.17074899438)+(0.836102144172)*0.5**o
        ref[1,0,1]=(-0.363417638344)/(o+1.)+(0.420165683258)+(-0.0712209756059)*0.5**o
        ref[1,1,0]=(-1.64760371583)/(o+1.)+(0.138267558514)+(-0.982992052158)*0.5**o
        ref[1,1,1]=(1.31550552722)/(o+1.)+(0.709685790293)+(0.518426662725)*0.5**o
        ref[2,0,0]=(0.820074870865)/(o+1.)+(0.0198543141864)+(-0.0368697940364)*0.5**o
        ref[2,0,1]=(-0.729262369848)/(o+1.)+(1.09152422841)+(-0.785633726304)*0.5**o
        ref[2,1,0]=(0.875394361701)/(o+1.)+(0.618695970363)+(0.969244001518)*0.5**o
        ref[2,1,1]=(1.1588508781)/(o+1.)+(0.437911816045)+(-0.310035929524)*0.5**o
        ref[3,0,0]=(-0.211796027546)/(o+1.)+(0.424078285949)+(0.245553971829)*0.5**o
        ref[3,0,1]=(-0.700684961711)/(o+1.)+(0.143307435844)+(0.427017913395)*0.5**o
        ref[3,1,0]=(0.896259869918)/(o+1.)+(-0.592079297738)+(-0.29533334088)*0.5**o
        ref[3,1,1]=(-0.580934373198)/(o+1.)+(-0.50115590932)+(-0.146152544907)*0.5**o
        ref[4,0,0]=(0.450220407566)/(o+1.)+(0.289178770177)+(0.0552991386013)*0.5**o
        ref[4,0,1]=(0.1837985251)/(o+1.)+(-1.20506020582)+(0.577046858044)*0.5**o
        ref[4,1,0]=(-0.0261798174917)/(o+1.)+(-0.634512568264)+(-0.358558179011)*0.5**o
        ref[4,1,1]=(1.34979337728)/(o+1.)+(0.447708916874)+(0.840098089823)*0.5**o
        ref[5,0,0]=(0.303173896103)/(o+1.)+(0.270786107114)+(0.695331559401)*0.5**o
        ref[5,0,1]=(1.26322726692)/(o+1.)+(0.622452488207)+(0.469488028534)*0.5**o
        ref[5,1,0]=(1.36503502533)/(o+1.)+(-0.526576872427)+(0.444799933589)*0.5**o
        ref[5,1,1]=(1.53331825424)/(o+1.)+(-1.15278372696)+(0.0304781332872)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactOne_fromData_Solution_rank4(self):
      """
      tests integral of rank 4 Data on the FunctionOnContactOne

      assumptions: Solution(self.domain) exists
                   self.domain supports integral on FunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      w=Solution(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5, 3, 2),w)
      ref=numpy.zeros((4, 5, 3, 2),numpy.float_)
      if dim==2:
        arg[0,0,0,0]=(-0.348972036919)*x[0]**o+(0.123542094911)*x[0]+(-0.596353540657)*x[1]**o+(0.717656729405)*x[1]
        arg[0,0,0,1]=(0.187733014896)*x[0]**o+(0.76244697815)*x[0]+(0.261409664458)*x[1]**o+(0.0213714031642)*x[1]
        arg[0,0,1,0]=(0.951655197513)*x[0]**o+(-0.865274104487)*x[0]+(0.59127040848)*x[1]**o+(-0.261622300136)*x[1]
        arg[0,0,1,1]=(-0.188127934304)*x[0]**o+(-0.513768204254)*x[0]+(0.77162116582)*x[1]**o+(-0.12621176914)*x[1]
        arg[0,0,2,0]=(0.770717242337)*x[0]**o+(-0.991038362841)*x[0]+(0.908566805373)*x[1]**o+(-0.24890064672)*x[1]
        arg[0,0,2,1]=(-0.543253359807)*x[0]**o+(-0.0504511921259)*x[0]+(-0.724584621389)*x[1]**o+(0.999965312141)*x[1]
        arg[0,1,0,0]=(0.986939140609)*x[0]**o+(0.236894127675)*x[0]+(0.520162431169)*x[1]**o+(0.306413415759)*x[1]
        arg[0,1,0,1]=(-0.171773212929)*x[0]**o+(-0.248436968201)*x[0]+(0.43818378458)*x[1]**o+(-0.495734982435)*x[1]
        arg[0,1,1,0]=(-0.369350989684)*x[0]**o+(0.59047357584)*x[0]+(0.422340945702)*x[1]**o+(0.326270009775)*x[1]
        arg[0,1,1,1]=(0.559606389467)*x[0]**o+(-0.0507064379501)*x[0]+(-0.0464190048363)*x[1]**o+(0.18964879235)*x[1]
        arg[0,1,2,0]=(-0.989408371198)*x[0]**o+(0.108750148268)*x[0]+(-0.347984444559)*x[1]**o+(-0.819590699683)*x[1]
        arg[0,1,2,1]=(-0.289295900777)*x[0]**o+(-0.579698633174)*x[0]+(-0.124928198101)*x[1]**o+(0.640901698875)*x[1]
        arg[0,2,0,0]=(0.132599667548)*x[0]**o+(-0.102095433147)*x[0]+(0.446169236302)*x[1]**o+(-0.844062525718)*x[1]
        arg[0,2,0,1]=(-0.452662917924)*x[0]**o+(-0.0497770418153)*x[0]+(-0.258525519083)*x[1]**o+(-0.0365197858914)*x[1]
        arg[0,2,1,0]=(0.318740871592)*x[0]**o+(0.562764515639)*x[0]+(-0.209089266173)*x[1]**o+(0.922024900771)*x[1]
        arg[0,2,1,1]=(-0.825757440563)*x[0]**o+(0.634009249217)*x[0]+(0.954836398565)*x[1]**o+(-0.910652766097)*x[1]
        arg[0,2,2,0]=(-0.640337007803)*x[0]**o+(0.862625623816)*x[0]+(-0.0327529254483)*x[1]**o+(-0.327305638603)*x[1]
        arg[0,2,2,1]=(-0.956300293885)*x[0]**o+(0.456116376134)*x[0]+(0.312023159058)*x[1]**o+(0.768546792276)*x[1]
        arg[0,3,0,0]=(-0.927040285049)*x[0]**o+(-0.614994799603)*x[0]+(-0.431859510779)*x[1]**o+(-0.088774748513)*x[1]
        arg[0,3,0,1]=(-0.807009561768)*x[0]**o+(0.00323409463398)*x[0]+(0.274814268019)*x[1]**o+(-0.132755038246)*x[1]
        arg[0,3,1,0]=(0.931774139846)*x[0]**o+(0.665327108449)*x[0]+(0.130437344004)*x[1]**o+(-0.00859025546518)*x[1]
        arg[0,3,1,1]=(-0.314188008746)*x[0]**o+(0.405251729437)*x[0]+(-0.313484462262)*x[1]**o+(-0.670765905491)*x[1]
        arg[0,3,2,0]=(0.748691505108)*x[0]**o+(0.717110963353)*x[0]+(-0.451892315269)*x[1]**o+(-0.103930621637)*x[1]
        arg[0,3,2,1]=(-0.124290384657)*x[0]**o+(0.0761550925153)*x[0]+(0.864166710803)*x[1]**o+(0.050297196468)*x[1]
        arg[0,4,0,0]=(-0.726606985714)*x[0]**o+(-0.304862755503)*x[0]+(-0.364735884875)*x[1]**o+(0.769999145518)*x[1]
        arg[0,4,0,1]=(-0.716457484159)*x[0]**o+(0.850185561874)*x[0]+(0.924535166786)*x[1]**o+(0.414539526778)*x[1]
        arg[0,4,1,0]=(-0.67154700238)*x[0]**o+(-0.0990079744678)*x[0]+(-0.738258048173)*x[1]**o+(-0.105333364414)*x[1]
        arg[0,4,1,1]=(-0.568569022142)*x[0]**o+(-0.865340476096)*x[0]+(0.592223896873)*x[1]**o+(0.545216501911)*x[1]
        arg[0,4,2,0]=(0.959619955514)*x[0]**o+(-0.625097978453)*x[0]+(0.447944032533)*x[1]**o+(-0.792670959618)*x[1]
        arg[0,4,2,1]=(-0.446578795272)*x[0]**o+(0.516341492325)*x[0]+(0.383143400712)*x[1]**o+(-0.339255681885)*x[1]
        arg[1,0,0,0]=(0.130600745253)*x[0]**o+(-0.225084866063)*x[0]+(-0.871963625443)*x[1]**o+(-0.0741251944735)*x[1]
        arg[1,0,0,1]=(-0.684630134211)*x[0]**o+(-0.298097256903)*x[0]+(0.511206122216)*x[1]**o+(-0.784104136535)*x[1]
        arg[1,0,1,0]=(0.624178362847)*x[0]**o+(0.161787148072)*x[0]+(-0.0906465041298)*x[1]**o+(-0.0302149132362)*x[1]
        arg[1,0,1,1]=(-0.403988173215)*x[0]**o+(-0.423895624882)*x[0]+(-0.0594261878161)*x[1]**o+(0.71753539369)*x[1]
        arg[1,0,2,0]=(0.411689797339)*x[0]**o+(0.0555768292842)*x[0]+(0.786614145981)*x[1]**o+(-0.701418967065)*x[1]
        arg[1,0,2,1]=(0.971933572797)*x[0]**o+(0.203165120814)*x[0]+(0.50340688009)*x[1]**o+(-0.443869607992)*x[1]
        arg[1,1,0,0]=(0.517782412737)*x[0]**o+(-0.233071827503)*x[0]+(0.310115588175)*x[1]**o+(-0.970604318783)*x[1]
        arg[1,1,0,1]=(0.250379154253)*x[0]**o+(-0.0914513321258)*x[0]+(0.466403827523)*x[1]**o+(-0.454369386353)*x[1]
        arg[1,1,1,0]=(-0.235520035004)*x[0]**o+(0.0932661387959)*x[0]+(0.0964933173918)*x[1]**o+(0.585245824454)*x[1]
        arg[1,1,1,1]=(-0.466145341257)*x[0]**o+(0.532910678544)*x[0]+(0.757545184326)*x[1]**o+(-0.488241176428)*x[1]
        arg[1,1,2,0]=(0.802070127079)*x[0]**o+(0.30227826497)*x[0]+(0.76818718588)*x[1]**o+(-0.966380392422)*x[1]
        arg[1,1,2,1]=(0.764411747219)*x[0]**o+(0.62151105703)*x[0]+(0.310875960956)*x[1]**o+(0.218417169467)*x[1]
        arg[1,2,0,0]=(0.627865841486)*x[0]**o+(0.328844171432)*x[0]+(0.47405718918)*x[1]**o+(0.238471462812)*x[1]
        arg[1,2,0,1]=(-0.890402471541)*x[0]**o+(0.0654874441479)*x[0]+(0.411199516291)*x[1]**o+(0.0669990208336)*x[1]
        arg[1,2,1,0]=(-0.789401901979)*x[0]**o+(-0.379725885915)*x[0]+(-0.103817842125)*x[1]**o+(0.358055283478)*x[1]
        arg[1,2,1,1]=(-0.619688531158)*x[0]**o+(0.314022911152)*x[0]+(-0.515963904961)*x[1]**o+(0.807912272743)*x[1]
        arg[1,2,2,0]=(0.696584946679)*x[0]**o+(-0.156405028983)*x[0]+(0.99374085906)*x[1]**o+(0.692362331558)*x[1]
        arg[1,2,2,1]=(0.463393460998)*x[0]**o+(-0.0302433372264)*x[0]+(0.243006695756)*x[1]**o+(0.934578685307)*x[1]
        arg[1,3,0,0]=(-0.760149691585)*x[0]**o+(0.305312987392)*x[0]+(0.478665319824)*x[1]**o+(-0.395734016727)*x[1]
        arg[1,3,0,1]=(-0.495638457166)*x[0]**o+(-0.352868042523)*x[0]+(0.0577132713041)*x[1]**o+(0.856507718332)*x[1]
        arg[1,3,1,0]=(-0.588160041725)*x[0]**o+(-0.941242199161)*x[0]+(-0.883563055172)*x[1]**o+(-0.474349426459)*x[1]
        arg[1,3,1,1]=(-0.822752260982)*x[0]**o+(0.284881595545)*x[0]+(-0.914198363943)*x[1]**o+(0.421107500666)*x[1]
        arg[1,3,2,0]=(-0.125099230328)*x[0]**o+(-0.0167977306674)*x[0]+(-0.880147366581)*x[1]**o+(0.947650728684)*x[1]
        arg[1,3,2,1]=(-0.770895899992)*x[0]**o+(-0.121705997003)*x[0]+(-0.0486858273347)*x[1]**o+(0.666179109672)*x[1]
        arg[1,4,0,0]=(-0.767476534476)*x[0]**o+(0.720013119195)*x[0]+(-0.540220631624)*x[1]**o+(-0.363702385125)*x[1]
        arg[1,4,0,1]=(0.90195509915)*x[0]**o+(0.104569298687)*x[0]+(0.901816813435)*x[1]**o+(0.421060929159)*x[1]
        arg[1,4,1,0]=(0.0901631681262)*x[0]**o+(-0.330570875574)*x[0]+(0.892147778885)*x[1]**o+(-0.102976747188)*x[1]
        arg[1,4,1,1]=(-0.336016437275)*x[0]**o+(-0.00913798206221)*x[0]+(-0.20211674106)*x[1]**o+(-0.0470039081275)*x[1]
        arg[1,4,2,0]=(0.194383418491)*x[0]**o+(-0.913993330791)*x[0]+(-0.0207457431142)*x[1]**o+(-0.728130242724)*x[1]
        arg[1,4,2,1]=(-0.285983741049)*x[0]**o+(-0.359824200302)*x[0]+(-0.286913656927)*x[1]**o+(-0.801477464894)*x[1]
        arg[2,0,0,0]=(-0.266085322677)*x[0]**o+(-0.477036323188)*x[0]+(0.967304544659)*x[1]**o+(-0.843721203165)*x[1]
        arg[2,0,0,1]=(-0.431692878995)*x[0]**o+(-0.326793076488)*x[0]+(0.478220893643)*x[1]**o+(-0.953607426852)*x[1]
        arg[2,0,1,0]=(0.800398295672)*x[0]**o+(-0.726923243756)*x[0]+(0.641194491232)*x[1]**o+(0.933755071699)*x[1]
        arg[2,0,1,1]=(0.778158607269)*x[0]**o+(0.192895772287)*x[0]+(0.160540751687)*x[1]**o+(0.132780133987)*x[1]
        arg[2,0,2,0]=(0.0700487728725)*x[0]**o+(0.366571342769)*x[0]+(0.213123464402)*x[1]**o+(0.0224970803568)*x[1]
        arg[2,0,2,1]=(-0.608321696505)*x[0]**o+(0.416761589908)*x[0]+(0.0499280394381)*x[1]**o+(-0.678150075911)*x[1]
        arg[2,1,0,0]=(-0.580987690899)*x[0]**o+(-0.625951863984)*x[0]+(0.0064094419355)*x[1]**o+(0.646045286912)*x[1]
        arg[2,1,0,1]=(-0.881255703221)*x[0]**o+(-0.153796152409)*x[0]+(0.320566497467)*x[1]**o+(-0.217827916224)*x[1]
        arg[2,1,1,0]=(0.981593086009)*x[0]**o+(0.659643450249)*x[0]+(0.824530802391)*x[1]**o+(0.109125926037)*x[1]
        arg[2,1,1,1]=(-0.79695617371)*x[0]**o+(-0.340030091407)*x[0]+(0.476648191338)*x[1]**o+(-0.751961375342)*x[1]
        arg[2,1,2,0]=(0.00785807621726)*x[0]**o+(-0.177037931946)*x[0]+(-0.322685883101)*x[1]**o+(0.639485466195)*x[1]
        arg[2,1,2,1]=(0.386667953213)*x[0]**o+(0.805115373435)*x[0]+(0.489529148574)*x[1]**o+(0.380650996708)*x[1]
        arg[2,2,0,0]=(-0.205678281473)*x[0]**o+(0.725798252716)*x[0]+(-0.884620261631)*x[1]**o+(0.737678769923)*x[1]
        arg[2,2,0,1]=(-0.859841848792)*x[0]**o+(0.215497396508)*x[0]+(-0.9231710768)*x[1]**o+(-0.504914651252)*x[1]
        arg[2,2,1,0]=(-0.0161889255216)*x[0]**o+(0.218150095097)*x[0]+(0.508505799895)*x[1]**o+(0.441270271663)*x[1]
        arg[2,2,1,1]=(0.645494826531)*x[0]**o+(0.812462584734)*x[0]+(0.498639261012)*x[1]**o+(0.695415642432)*x[1]
        arg[2,2,2,0]=(-0.855342734108)*x[0]**o+(0.174928383231)*x[0]+(-0.550636347213)*x[1]**o+(0.814295216919)*x[1]
        arg[2,2,2,1]=(-0.146152468476)*x[0]**o+(-0.810876776267)*x[0]+(0.77444775812)*x[1]**o+(0.758171250375)*x[1]
        arg[2,3,0,0]=(0.914771099535)*x[0]**o+(-0.801103658802)*x[0]+(-0.339761119169)*x[1]**o+(-0.0697409372498)*x[1]
        arg[2,3,0,1]=(-0.278017621273)*x[0]**o+(-0.916896227964)*x[0]+(0.0151205004002)*x[1]**o+(-0.36424189479)*x[1]
        arg[2,3,1,0]=(0.635845368235)*x[0]**o+(-0.286420165946)*x[0]+(-0.547314386502)*x[1]**o+(-0.297882680065)*x[1]
        arg[2,3,1,1]=(0.236860892332)*x[0]**o+(0.670289851551)*x[0]+(-0.437288046493)*x[1]**o+(0.0610684872034)*x[1]
        arg[2,3,2,0]=(0.096479176127)*x[0]**o+(-0.992608241155)*x[0]+(0.622650933171)*x[1]**o+(-0.171594052732)*x[1]
        arg[2,3,2,1]=(0.259913661654)*x[0]**o+(0.521353257934)*x[0]+(0.141212812345)*x[1]**o+(0.0311971708742)*x[1]
        arg[2,4,0,0]=(0.208498140469)*x[0]**o+(0.914285328223)*x[0]+(-0.368860713612)*x[1]**o+(0.744057640483)*x[1]
        arg[2,4,0,1]=(-0.0760701135919)*x[0]**o+(-0.0666410245297)*x[0]+(-0.442858585142)*x[1]**o+(-0.869273907698)*x[1]
        arg[2,4,1,0]=(0.694291134331)*x[0]**o+(0.153487646161)*x[0]+(-0.454726570824)*x[1]**o+(0.12533511793)*x[1]
        arg[2,4,1,1]=(0.505643124198)*x[0]**o+(0.935208927517)*x[0]+(0.120271053976)*x[1]**o+(0.472499209985)*x[1]
        arg[2,4,2,0]=(0.458890639035)*x[0]**o+(-0.38125264637)*x[0]+(-0.742525095759)*x[1]**o+(0.0727341692272)*x[1]
        arg[2,4,2,1]=(-0.228856512525)*x[0]**o+(0.215540806099)*x[0]+(-0.492076254398)*x[1]**o+(0.325121281149)*x[1]
        arg[3,0,0,0]=(0.791320435019)*x[0]**o+(0.264208069532)*x[0]+(0.105218141263)*x[1]**o+(-0.947073121631)*x[1]
        arg[3,0,0,1]=(0.384081306848)*x[0]**o+(-0.735420806741)*x[0]+(0.695451719819)*x[1]**o+(-0.104071285853)*x[1]
        arg[3,0,1,0]=(0.176280431402)*x[0]**o+(0.766248516714)*x[0]+(0.620920827635)*x[1]**o+(0.571778731591)*x[1]
        arg[3,0,1,1]=(0.895148060637)*x[0]**o+(0.120401904357)*x[0]+(0.500131556768)*x[1]**o+(-0.432523916076)*x[1]
        arg[3,0,2,0]=(-0.171375065793)*x[0]**o+(0.731777853738)*x[0]+(0.914774635339)*x[1]**o+(-0.329504614878)*x[1]
        arg[3,0,2,1]=(-0.65523706909)*x[0]**o+(-0.309487164889)*x[0]+(-0.244763521442)*x[1]**o+(0.790744116827)*x[1]
        arg[3,1,0,0]=(-0.515742288)*x[0]**o+(0.688021032329)*x[0]+(0.744558730681)*x[1]**o+(0.663410706435)*x[1]
        arg[3,1,0,1]=(0.299920493853)*x[0]**o+(0.221554378219)*x[0]+(-0.262111371211)*x[1]**o+(0.597219370531)*x[1]
        arg[3,1,1,0]=(-0.491648532317)*x[0]**o+(-0.0763921400417)*x[0]+(0.517994656474)*x[1]**o+(0.690649210498)*x[1]
        arg[3,1,1,1]=(0.291881729319)*x[0]**o+(0.388296683726)*x[0]+(0.304223657331)*x[1]**o+(-0.17249216188)*x[1]
        arg[3,1,2,0]=(0.309288981861)*x[0]**o+(-0.638172982762)*x[0]+(-0.566768084047)*x[1]**o+(-0.694439437371)*x[1]
        arg[3,1,2,1]=(-0.680391436031)*x[0]**o+(0.876096306554)*x[0]+(0.676979754874)*x[1]**o+(0.376507786465)*x[1]
        arg[3,2,0,0]=(-0.688464200232)*x[0]**o+(0.404140014906)*x[0]+(-0.389359624725)*x[1]**o+(0.218257423404)*x[1]
        arg[3,2,0,1]=(-0.321661287857)*x[0]**o+(0.305707842777)*x[0]+(0.0488067596906)*x[1]**o+(0.475071042874)*x[1]
        arg[3,2,1,0]=(-0.172172832131)*x[0]**o+(-0.0589598637728)*x[0]+(0.997578239775)*x[1]**o+(0.0919614884141)*x[1]
        arg[3,2,1,1]=(0.500590025777)*x[0]**o+(0.258724052781)*x[0]+(0.233819575609)*x[1]**o+(0.787949345995)*x[1]
        arg[3,2,2,0]=(-0.464931198743)*x[0]**o+(0.689246395117)*x[0]+(0.0701366334357)*x[1]**o+(-0.206391031385)*x[1]
        arg[3,2,2,1]=(-0.352492545609)*x[0]**o+(-0.160385273073)*x[0]+(0.876791526094)*x[1]**o+(0.748257988817)*x[1]
        arg[3,3,0,0]=(0.70346530112)*x[0]**o+(-0.351192123239)*x[0]+(-0.985298342079)*x[1]**o+(-0.286817451723)*x[1]
        arg[3,3,0,1]=(0.841285056577)*x[0]**o+(0.741986440026)*x[0]+(-0.758645627646)*x[1]**o+(-0.914494381783)*x[1]
        arg[3,3,1,0]=(-0.614293213484)*x[0]**o+(-0.746555667296)*x[0]+(-0.301781252786)*x[1]**o+(-0.721633195357)*x[1]
        arg[3,3,1,1]=(-0.287510519978)*x[0]**o+(-0.764960517935)*x[0]+(-0.948032248565)*x[1]**o+(-0.548676002552)*x[1]
        arg[3,3,2,0]=(0.145506943729)*x[0]**o+(-0.353925293653)*x[0]+(0.786904887352)*x[1]**o+(-0.36548133551)*x[1]
        arg[3,3,2,1]=(0.60782215735)*x[0]**o+(0.872444227513)*x[0]+(-0.237571902274)*x[1]**o+(-0.0559745367747)*x[1]
        arg[3,4,0,0]=(0.691107698337)*x[0]**o+(0.0800532750053)*x[0]+(-0.847456183316)*x[1]**o+(0.911068576288)*x[1]
        arg[3,4,0,1]=(-0.00167606578449)*x[0]**o+(-0.697603805697)*x[0]+(-0.362440886816)*x[1]**o+(-0.0604379772766)*x[1]
        arg[3,4,1,0]=(-0.74026325403)*x[0]**o+(-0.539076677004)*x[0]+(-0.317176913637)*x[1]**o+(0.759776288772)*x[1]
        arg[3,4,1,1]=(-0.230005454291)*x[0]**o+(-0.929557352119)*x[0]+(-0.435125210116)*x[1]**o+(0.106892135893)*x[1]
        arg[3,4,2,0]=(-0.638149620463)*x[0]**o+(-0.0511424128986)*x[0]+(0.460436795546)*x[1]**o+(-0.919535518181)*x[1]
        arg[3,4,2,1]=(-0.793537245531)*x[0]**o+(0.409768940347)*x[0]+(0.632560678557)*x[1]**o+(0.700596370851)*x[1]
        ref[0,0,0,0]=(-0.596353540657)/(o+1.)+(0.420599412158)+(-0.348972036919)*0.5**o
        ref[0,0,0,1]=(0.261409664458)/(o+1.)+(0.391909190657)+(0.187733014896)*0.5**o
        ref[0,0,1,0]=(0.59127040848)/(o+1.)+(-0.563448202311)+(0.951655197513)*0.5**o
        ref[0,0,1,1]=(0.77162116582)/(o+1.)+(-0.319989986697)+(-0.188127934304)*0.5**o
        ref[0,0,2,0]=(0.908566805373)/(o+1.)+(-0.619969504781)+(0.770717242337)*0.5**o
        ref[0,0,2,1]=(-0.724584621389)/(o+1.)+(0.474757060008)+(-0.543253359807)*0.5**o
        ref[0,1,0,0]=(0.520162431169)/(o+1.)+(0.271653771717)+(0.986939140609)*0.5**o
        ref[0,1,0,1]=(0.43818378458)/(o+1.)+(-0.372085975318)+(-0.171773212929)*0.5**o
        ref[0,1,1,0]=(0.422340945702)/(o+1.)+(0.458371792807)+(-0.369350989684)*0.5**o
        ref[0,1,1,1]=(-0.0464190048363)/(o+1.)+(0.0694711771999)+(0.559606389467)*0.5**o
        ref[0,1,2,0]=(-0.347984444559)/(o+1.)+(-0.355420275708)+(-0.989408371198)*0.5**o
        ref[0,1,2,1]=(-0.124928198101)/(o+1.)+(0.0306015328506)+(-0.289295900777)*0.5**o
        ref[0,2,0,0]=(0.446169236302)/(o+1.)+(-0.473078979432)+(0.132599667548)*0.5**o
        ref[0,2,0,1]=(-0.258525519083)/(o+1.)+(-0.0431484138534)+(-0.452662917924)*0.5**o
        ref[0,2,1,0]=(-0.209089266173)/(o+1.)+(0.742394708205)+(0.318740871592)*0.5**o
        ref[0,2,1,1]=(0.954836398565)/(o+1.)+(-0.13832175844)+(-0.825757440563)*0.5**o
        ref[0,2,2,0]=(-0.0327529254483)/(o+1.)+(0.267659992607)+(-0.640337007803)*0.5**o
        ref[0,2,2,1]=(0.312023159058)/(o+1.)+(0.612331584205)+(-0.956300293885)*0.5**o
        ref[0,3,0,0]=(-0.431859510779)/(o+1.)+(-0.351884774058)+(-0.927040285049)*0.5**o
        ref[0,3,0,1]=(0.274814268019)/(o+1.)+(-0.064760471806)+(-0.807009561768)*0.5**o
        ref[0,3,1,0]=(0.130437344004)/(o+1.)+(0.328368426492)+(0.931774139846)*0.5**o
        ref[0,3,1,1]=(-0.313484462262)/(o+1.)+(-0.132757088027)+(-0.314188008746)*0.5**o
        ref[0,3,2,0]=(-0.451892315269)/(o+1.)+(0.306590170858)+(0.748691505108)*0.5**o
        ref[0,3,2,1]=(0.864166710803)/(o+1.)+(0.0632261444916)+(-0.124290384657)*0.5**o
        ref[0,4,0,0]=(-0.364735884875)/(o+1.)+(0.232568195007)+(-0.726606985714)*0.5**o
        ref[0,4,0,1]=(0.924535166786)/(o+1.)+(0.632362544326)+(-0.716457484159)*0.5**o
        ref[0,4,1,0]=(-0.738258048173)/(o+1.)+(-0.102170669441)+(-0.67154700238)*0.5**o
        ref[0,4,1,1]=(0.592223896873)/(o+1.)+(-0.160061987092)+(-0.568569022142)*0.5**o
        ref[0,4,2,0]=(0.447944032533)/(o+1.)+(-0.708884469036)+(0.959619955514)*0.5**o
        ref[0,4,2,1]=(0.383143400712)/(o+1.)+(0.0885429052198)+(-0.446578795272)*0.5**o
        ref[1,0,0,0]=(-0.871963625443)/(o+1.)+(-0.149605030268)+(0.130600745253)*0.5**o
        ref[1,0,0,1]=(0.511206122216)/(o+1.)+(-0.541100696719)+(-0.684630134211)*0.5**o
        ref[1,0,1,0]=(-0.0906465041298)/(o+1.)+(0.065786117418)+(0.624178362847)*0.5**o
        ref[1,0,1,1]=(-0.0594261878161)/(o+1.)+(0.146819884404)+(-0.403988173215)*0.5**o
        ref[1,0,2,0]=(0.786614145981)/(o+1.)+(-0.32292106889)+(0.411689797339)*0.5**o
        ref[1,0,2,1]=(0.50340688009)/(o+1.)+(-0.120352243589)+(0.971933572797)*0.5**o
        ref[1,1,0,0]=(0.310115588175)/(o+1.)+(-0.601838073143)+(0.517782412737)*0.5**o
        ref[1,1,0,1]=(0.466403827523)/(o+1.)+(-0.27291035924)+(0.250379154253)*0.5**o
        ref[1,1,1,0]=(0.0964933173918)/(o+1.)+(0.339255981625)+(-0.235520035004)*0.5**o
        ref[1,1,1,1]=(0.757545184326)/(o+1.)+(0.0223347510579)+(-0.466145341257)*0.5**o
        ref[1,1,2,0]=(0.76818718588)/(o+1.)+(-0.332051063726)+(0.802070127079)*0.5**o
        ref[1,1,2,1]=(0.310875960956)/(o+1.)+(0.419964113249)+(0.764411747219)*0.5**o
        ref[1,2,0,0]=(0.47405718918)/(o+1.)+(0.283657817122)+(0.627865841486)*0.5**o
        ref[1,2,0,1]=(0.411199516291)/(o+1.)+(0.0662432324908)+(-0.890402471541)*0.5**o
        ref[1,2,1,0]=(-0.103817842125)/(o+1.)+(-0.0108353012186)+(-0.789401901979)*0.5**o
        ref[1,2,1,1]=(-0.515963904961)/(o+1.)+(0.560967591948)+(-0.619688531158)*0.5**o
        ref[1,2,2,0]=(0.99374085906)/(o+1.)+(0.267978651288)+(0.696584946679)*0.5**o
        ref[1,2,2,1]=(0.243006695756)/(o+1.)+(0.45216767404)+(0.463393460998)*0.5**o
        ref[1,3,0,0]=(0.478665319824)/(o+1.)+(-0.0452105146678)+(-0.760149691585)*0.5**o
        ref[1,3,0,1]=(0.0577132713041)/(o+1.)+(0.251819837904)+(-0.495638457166)*0.5**o
        ref[1,3,1,0]=(-0.883563055172)/(o+1.)+(-0.70779581281)+(-0.588160041725)*0.5**o
        ref[1,3,1,1]=(-0.914198363943)/(o+1.)+(0.352994548105)+(-0.822752260982)*0.5**o
        ref[1,3,2,0]=(-0.880147366581)/(o+1.)+(0.465426499008)+(-0.125099230328)*0.5**o
        ref[1,3,2,1]=(-0.0486858273347)/(o+1.)+(0.272236556335)+(-0.770895899992)*0.5**o
        ref[1,4,0,0]=(-0.540220631624)/(o+1.)+(0.178155367035)+(-0.767476534476)*0.5**o
        ref[1,4,0,1]=(0.901816813435)/(o+1.)+(0.262815113923)+(0.90195509915)*0.5**o
        ref[1,4,1,0]=(0.892147778885)/(o+1.)+(-0.216773811381)+(0.0901631681262)*0.5**o
        ref[1,4,1,1]=(-0.20211674106)/(o+1.)+(-0.0280709450949)+(-0.336016437275)*0.5**o
        ref[1,4,2,0]=(-0.0207457431142)/(o+1.)+(-0.821061786757)+(0.194383418491)*0.5**o
        ref[1,4,2,1]=(-0.286913656927)/(o+1.)+(-0.580650832598)+(-0.285983741049)*0.5**o
        ref[2,0,0,0]=(0.967304544659)/(o+1.)+(-0.660378763176)+(-0.266085322677)*0.5**o
        ref[2,0,0,1]=(0.478220893643)/(o+1.)+(-0.64020025167)+(-0.431692878995)*0.5**o
        ref[2,0,1,0]=(0.641194491232)/(o+1.)+(0.103415913972)+(0.800398295672)*0.5**o
        ref[2,0,1,1]=(0.160540751687)/(o+1.)+(0.162837953137)+(0.778158607269)*0.5**o
        ref[2,0,2,0]=(0.213123464402)/(o+1.)+(0.194534211563)+(0.0700487728725)*0.5**o
        ref[2,0,2,1]=(0.0499280394381)/(o+1.)+(-0.130694243001)+(-0.608321696505)*0.5**o
        ref[2,1,0,0]=(0.0064094419355)/(o+1.)+(0.0100467114639)+(-0.580987690899)*0.5**o
        ref[2,1,0,1]=(0.320566497467)/(o+1.)+(-0.185812034316)+(-0.881255703221)*0.5**o
        ref[2,1,1,0]=(0.824530802391)/(o+1.)+(0.384384688143)+(0.981593086009)*0.5**o
        ref[2,1,1,1]=(0.476648191338)/(o+1.)+(-0.545995733375)+(-0.79695617371)*0.5**o
        ref[2,1,2,0]=(-0.322685883101)/(o+1.)+(0.231223767125)+(0.00785807621726)*0.5**o
        ref[2,1,2,1]=(0.489529148574)/(o+1.)+(0.592883185071)+(0.386667953213)*0.5**o
        ref[2,2,0,0]=(-0.884620261631)/(o+1.)+(0.73173851132)+(-0.205678281473)*0.5**o
        ref[2,2,0,1]=(-0.9231710768)/(o+1.)+(-0.144708627372)+(-0.859841848792)*0.5**o
        ref[2,2,1,0]=(0.508505799895)/(o+1.)+(0.32971018338)+(-0.0161889255216)*0.5**o
        ref[2,2,1,1]=(0.498639261012)/(o+1.)+(0.753939113583)+(0.645494826531)*0.5**o
        ref[2,2,2,0]=(-0.550636347213)/(o+1.)+(0.494611800075)+(-0.855342734108)*0.5**o
        ref[2,2,2,1]=(0.77444775812)/(o+1.)+(-0.0263527629459)+(-0.146152468476)*0.5**o
        ref[2,3,0,0]=(-0.339761119169)/(o+1.)+(-0.435422298026)+(0.914771099535)*0.5**o
        ref[2,3,0,1]=(0.0151205004002)/(o+1.)+(-0.640569061377)+(-0.278017621273)*0.5**o
        ref[2,3,1,0]=(-0.547314386502)/(o+1.)+(-0.292151423006)+(0.635845368235)*0.5**o
        ref[2,3,1,1]=(-0.437288046493)/(o+1.)+(0.365679169377)+(0.236860892332)*0.5**o
        ref[2,3,2,0]=(0.622650933171)/(o+1.)+(-0.582101146943)+(0.096479176127)*0.5**o
        ref[2,3,2,1]=(0.141212812345)/(o+1.)+(0.276275214404)+(0.259913661654)*0.5**o
        ref[2,4,0,0]=(-0.368860713612)/(o+1.)+(0.829171484353)+(0.208498140469)*0.5**o
        ref[2,4,0,1]=(-0.442858585142)/(o+1.)+(-0.467957466114)+(-0.0760701135919)*0.5**o
        ref[2,4,1,0]=(-0.454726570824)/(o+1.)+(0.139411382046)+(0.694291134331)*0.5**o
        ref[2,4,1,1]=(0.120271053976)/(o+1.)+(0.703854068751)+(0.505643124198)*0.5**o
        ref[2,4,2,0]=(-0.742525095759)/(o+1.)+(-0.154259238572)+(0.458890639035)*0.5**o
        ref[2,4,2,1]=(-0.492076254398)/(o+1.)+(0.270331043624)+(-0.228856512525)*0.5**o
        ref[3,0,0,0]=(0.105218141263)/(o+1.)+(-0.34143252605)+(0.791320435019)*0.5**o
        ref[3,0,0,1]=(0.695451719819)/(o+1.)+(-0.419746046297)+(0.384081306848)*0.5**o
        ref[3,0,1,0]=(0.620920827635)/(o+1.)+(0.669013624153)+(0.176280431402)*0.5**o
        ref[3,0,1,1]=(0.500131556768)/(o+1.)+(-0.15606100586)+(0.895148060637)*0.5**o
        ref[3,0,2,0]=(0.914774635339)/(o+1.)+(0.20113661943)+(-0.171375065793)*0.5**o
        ref[3,0,2,1]=(-0.244763521442)/(o+1.)+(0.240628475969)+(-0.65523706909)*0.5**o
        ref[3,1,0,0]=(0.744558730681)/(o+1.)+(0.675715869382)+(-0.515742288)*0.5**o
        ref[3,1,0,1]=(-0.262111371211)/(o+1.)+(0.409386874375)+(0.299920493853)*0.5**o
        ref[3,1,1,0]=(0.517994656474)/(o+1.)+(0.307128535228)+(-0.491648532317)*0.5**o
        ref[3,1,1,1]=(0.304223657331)/(o+1.)+(0.107902260923)+(0.291881729319)*0.5**o
        ref[3,1,2,0]=(-0.566768084047)/(o+1.)+(-0.666306210066)+(0.309288981861)*0.5**o
        ref[3,1,2,1]=(0.676979754874)/(o+1.)+(0.62630204651)+(-0.680391436031)*0.5**o
        ref[3,2,0,0]=(-0.389359624725)/(o+1.)+(0.311198719155)+(-0.688464200232)*0.5**o
        ref[3,2,0,1]=(0.0488067596906)/(o+1.)+(0.390389442826)+(-0.321661287857)*0.5**o
        ref[3,2,1,0]=(0.997578239775)/(o+1.)+(0.0165008123207)+(-0.172172832131)*0.5**o
        ref[3,2,1,1]=(0.233819575609)/(o+1.)+(0.523336699388)+(0.500590025777)*0.5**o
        ref[3,2,2,0]=(0.0701366334357)/(o+1.)+(0.241427681866)+(-0.464931198743)*0.5**o
        ref[3,2,2,1]=(0.876791526094)/(o+1.)+(0.293936357872)+(-0.352492545609)*0.5**o
        ref[3,3,0,0]=(-0.985298342079)/(o+1.)+(-0.319004787481)+(0.70346530112)*0.5**o
        ref[3,3,0,1]=(-0.758645627646)/(o+1.)+(-0.0862539708788)+(0.841285056577)*0.5**o
        ref[3,3,1,0]=(-0.301781252786)/(o+1.)+(-0.734094431327)+(-0.614293213484)*0.5**o
        ref[3,3,1,1]=(-0.948032248565)/(o+1.)+(-0.656818260243)+(-0.287510519978)*0.5**o
        ref[3,3,2,0]=(0.786904887352)/(o+1.)+(-0.359703314582)+(0.145506943729)*0.5**o
        ref[3,3,2,1]=(-0.237571902274)/(o+1.)+(0.408234845369)+(0.60782215735)*0.5**o
        ref[3,4,0,0]=(-0.847456183316)/(o+1.)+(0.495560925646)+(0.691107698337)*0.5**o
        ref[3,4,0,1]=(-0.362440886816)/(o+1.)+(-0.379020891487)+(-0.00167606578449)*0.5**o
        ref[3,4,1,0]=(-0.317176913637)/(o+1.)+(0.110349805884)+(-0.74026325403)*0.5**o
        ref[3,4,1,1]=(-0.435125210116)/(o+1.)+(-0.411332608113)+(-0.230005454291)*0.5**o
        ref[3,4,2,0]=(0.460436795546)/(o+1.)+(-0.48533896554)+(-0.638149620463)*0.5**o
        ref[3,4,2,1]=(0.632560678557)/(o+1.)+(0.555182655599)+(-0.793537245531)*0.5**o
      else:
        arg[0,0,0,0]=(0.816458625566)*x[0]**o+(-0.438180707327)*x[0]+(-0.39836196112)*x[1]**o+(0.641701267855)*x[1]+(0.919064096208)*x[2]**o+(0.0155653082454)*x[2]
        arg[0,0,0,1]=(-0.709065020122)*x[0]**o+(-0.787938005558)*x[0]+(-0.459829890247)*x[1]**o+(-0.306587811189)*x[1]+(0.648348776519)*x[2]**o+(-0.782857657922)*x[2]
        arg[0,0,1,0]=(-0.09165636265)*x[0]**o+(-0.215464577259)*x[0]+(-0.445060581145)*x[1]**o+(0.541797069021)*x[1]+(-0.312096860473)*x[2]**o+(0.307061070632)*x[2]
        arg[0,0,1,1]=(0.900670566988)*x[0]**o+(-0.157226841228)*x[0]+(-0.00514019601556)*x[1]**o+(0.167616993369)*x[1]+(0.331077209212)*x[2]**o+(0.0797784869412)*x[2]
        arg[0,0,2,0]=(-0.883617789072)*x[0]**o+(-0.233507045302)*x[0]+(0.540311388635)*x[1]**o+(-0.0698351272775)*x[1]+(0.598059668701)*x[2]**o+(-0.842961707358)*x[2]
        arg[0,0,2,1]=(-0.770041366552)*x[0]**o+(-0.683157969301)*x[0]+(0.235580399076)*x[1]**o+(-0.994177047985)*x[1]+(-0.90074656484)*x[2]**o+(-0.0133600820046)*x[2]
        arg[0,1,0,0]=(0.90396926296)*x[0]**o+(0.0492056402145)*x[0]+(0.32744155595)*x[1]**o+(0.0162505824727)*x[1]+(0.386607661085)*x[2]**o+(0.759639829913)*x[2]
        arg[0,1,0,1]=(0.47956310963)*x[0]**o+(0.638067028609)*x[0]+(-0.520596788355)*x[1]**o+(0.476897948019)*x[1]+(0.950320232053)*x[2]**o+(-0.603034001297)*x[2]
        arg[0,1,1,0]=(-0.75177304495)*x[0]**o+(0.369014786745)*x[0]+(-0.332220011735)*x[1]**o+(-0.835059332186)*x[1]+(0.0494323092557)*x[2]**o+(-0.796200422375)*x[2]
        arg[0,1,1,1]=(-0.109937058453)*x[0]**o+(0.607637390544)*x[0]+(0.464147925564)*x[1]**o+(0.650325883743)*x[1]+(0.698196425121)*x[2]**o+(0.933002044297)*x[2]
        arg[0,1,2,0]=(-0.13801307581)*x[0]**o+(0.75768744715)*x[0]+(-0.35990643116)*x[1]**o+(-0.787975707318)*x[1]+(-0.259281382264)*x[2]**o+(0.971003238549)*x[2]
        arg[0,1,2,1]=(-0.518540484981)*x[0]**o+(0.197912267942)*x[0]+(0.481058074022)*x[1]**o+(0.918189335061)*x[1]+(0.160239271801)*x[2]**o+(0.75950986792)*x[2]
        arg[0,2,0,0]=(0.286922773164)*x[0]**o+(-0.874592150595)*x[0]+(0.416014232771)*x[1]**o+(-0.492379239759)*x[1]+(0.950640982197)*x[2]**o+(-0.10625158241)*x[2]
        arg[0,2,0,1]=(-0.116447648923)*x[0]**o+(-0.499481854319)*x[0]+(-0.556602044084)*x[1]**o+(0.753256358343)*x[1]+(0.280780022694)*x[2]**o+(-0.879728755507)*x[2]
        arg[0,2,1,0]=(-0.322803067618)*x[0]**o+(0.0974878233964)*x[0]+(0.420547949274)*x[1]**o+(0.304839957099)*x[1]+(-0.12605355567)*x[2]**o+(-0.166970509739)*x[2]
        arg[0,2,1,1]=(0.997823909059)*x[0]**o+(0.76749010767)*x[0]+(0.376351544673)*x[1]**o+(-0.316937252621)*x[1]+(-0.627670639733)*x[2]**o+(0.584121037987)*x[2]
        arg[0,2,2,0]=(0.516866019209)*x[0]**o+(0.413564799736)*x[0]+(-0.0730736747833)*x[1]**o+(-0.500984764568)*x[1]+(-0.989323894645)*x[2]**o+(-0.261458643035)*x[2]
        arg[0,2,2,1]=(0.777538237337)*x[0]**o+(-0.0623276952959)*x[0]+(0.167068351133)*x[1]**o+(-0.345550070545)*x[1]+(-0.28166577069)*x[2]**o+(-0.568650037569)*x[2]
        arg[0,3,0,0]=(-0.622067362782)*x[0]**o+(0.971384323624)*x[0]+(-0.099597286042)*x[1]**o+(-0.265680407306)*x[1]+(-0.446275631545)*x[2]**o+(-0.0637591060995)*x[2]
        arg[0,3,0,1]=(-0.49698112241)*x[0]**o+(0.766181263321)*x[0]+(-0.977014162391)*x[1]**o+(0.806444170647)*x[1]+(-0.441023882645)*x[2]**o+(-0.919279365683)*x[2]
        arg[0,3,1,0]=(0.917091542097)*x[0]**o+(-0.67016353228)*x[0]+(0.0468346020734)*x[1]**o+(0.81092419675)*x[1]+(0.174037096652)*x[2]**o+(0.564190407348)*x[2]
        arg[0,3,1,1]=(-0.437188100982)*x[0]**o+(-0.0857174204982)*x[0]+(-0.971592105442)*x[1]**o+(-0.122352019844)*x[1]+(-0.853225566811)*x[2]**o+(-0.861128476903)*x[2]
        arg[0,3,2,0]=(-0.841265485658)*x[0]**o+(0.553186811235)*x[0]+(0.414489259007)*x[1]**o+(-0.800997281522)*x[1]+(0.630269543743)*x[2]**o+(0.263584335528)*x[2]
        arg[0,3,2,1]=(0.383282548921)*x[0]**o+(0.260041562865)*x[0]+(-0.95180999378)*x[1]**o+(0.566954668715)*x[1]+(0.688843907846)*x[2]**o+(-0.166837115553)*x[2]
        arg[0,4,0,0]=(0.0385891845805)*x[0]**o+(0.324932244772)*x[0]+(-0.946997425504)*x[1]**o+(-0.745439090316)*x[1]+(-0.721657519976)*x[2]**o+(-0.714366519961)*x[2]
        arg[0,4,0,1]=(-0.387739686761)*x[0]**o+(-0.860915144676)*x[0]+(0.270056169204)*x[1]**o+(-0.914861208322)*x[1]+(-0.453514265988)*x[2]**o+(-0.495768314604)*x[2]
        arg[0,4,1,0]=(-0.329464483421)*x[0]**o+(0.313424566493)*x[0]+(0.185677682455)*x[1]**o+(-0.720200750426)*x[1]+(0.425742092834)*x[2]**o+(0.505209526425)*x[2]
        arg[0,4,1,1]=(0.63272559561)*x[0]**o+(-0.026429919604)*x[0]+(-0.14023723263)*x[1]**o+(-0.580805084228)*x[1]+(0.141232832837)*x[2]**o+(0.241327430534)*x[2]
        arg[0,4,2,0]=(-0.847582850604)*x[0]**o+(0.207270297356)*x[0]+(0.851958035148)*x[1]**o+(0.336594484304)*x[1]+(-0.221434386211)*x[2]**o+(-0.607602400875)*x[2]
        arg[0,4,2,1]=(-0.722990535113)*x[0]**o+(-0.379964881881)*x[0]+(0.825779173038)*x[1]**o+(-0.285509543358)*x[1]+(-0.518809494335)*x[2]**o+(-0.788507606636)*x[2]
        arg[1,0,0,0]=(0.0725951752952)*x[0]**o+(-0.282800153416)*x[0]+(-0.0228303688498)*x[1]**o+(-0.863127549259)*x[1]+(-0.383553228278)*x[2]**o+(-0.0151737809185)*x[2]
        arg[1,0,0,1]=(-0.254920010173)*x[0]**o+(0.160195189848)*x[0]+(0.216407190109)*x[1]**o+(-0.814599655672)*x[1]+(0.771746025302)*x[2]**o+(0.0636221760766)*x[2]
        arg[1,0,1,0]=(0.486342223622)*x[0]**o+(0.736242651359)*x[0]+(0.720722697611)*x[1]**o+(0.974853194395)*x[1]+(-0.834707965096)*x[2]**o+(-0.947169299038)*x[2]
        arg[1,0,1,1]=(0.777015150638)*x[0]**o+(-0.228082772046)*x[0]+(-0.51095285588)*x[1]**o+(-0.835057692508)*x[1]+(-0.564012944223)*x[2]**o+(-0.898040787549)*x[2]
        arg[1,0,2,0]=(0.240813275278)*x[0]**o+(-0.509566588932)*x[0]+(0.247803237451)*x[1]**o+(0.156442781059)*x[1]+(0.618367580757)*x[2]**o+(0.943435734207)*x[2]
        arg[1,0,2,1]=(-0.670257601629)*x[0]**o+(-0.470716006203)*x[0]+(-0.559207336128)*x[1]**o+(-0.81577896974)*x[1]+(0.0571093198914)*x[2]**o+(0.477103420898)*x[2]
        arg[1,1,0,0]=(0.985895581872)*x[0]**o+(-0.00359465828149)*x[0]+(0.00739312124399)*x[1]**o+(-0.931299968425)*x[1]+(-0.10047347666)*x[2]**o+(0.410589913002)*x[2]
        arg[1,1,0,1]=(-0.954132399275)*x[0]**o+(0.159222338169)*x[0]+(-0.865370419503)*x[1]**o+(-0.744696059283)*x[1]+(-0.794993976252)*x[2]**o+(0.961664562839)*x[2]
        arg[1,1,1,0]=(0.462810965625)*x[0]**o+(0.927378551017)*x[0]+(-0.105005418179)*x[1]**o+(0.282340667975)*x[1]+(-0.443931241323)*x[2]**o+(0.740071617715)*x[2]
        arg[1,1,1,1]=(0.645629353541)*x[0]**o+(0.848804271896)*x[0]+(-0.882497356826)*x[1]**o+(-0.74661619875)*x[1]+(-0.927133238344)*x[2]**o+(-0.502758459229)*x[2]
        arg[1,1,2,0]=(-0.554993653726)*x[0]**o+(-0.473286752336)*x[0]+(0.759624773306)*x[1]**o+(0.809169103932)*x[1]+(0.657766085675)*x[2]**o+(0.763913478843)*x[2]
        arg[1,1,2,1]=(0.1476587556)*x[0]**o+(0.385711162597)*x[0]+(-0.0767252754622)*x[1]**o+(-0.544968629318)*x[1]+(0.16751959089)*x[2]**o+(-0.742697412247)*x[2]
        arg[1,2,0,0]=(-0.715887407694)*x[0]**o+(0.273818352078)*x[0]+(0.0426629688432)*x[1]**o+(-0.808531265186)*x[1]+(-0.699839650334)*x[2]**o+(-0.332093073009)*x[2]
        arg[1,2,0,1]=(0.48440337855)*x[0]**o+(0.765443731546)*x[0]+(-0.0761968055499)*x[1]**o+(0.0516389293584)*x[1]+(-0.486224925751)*x[2]**o+(-0.796105709176)*x[2]
        arg[1,2,1,0]=(0.621562227153)*x[0]**o+(0.907213416636)*x[0]+(0.591392192921)*x[1]**o+(-0.825911160831)*x[1]+(0.690773959314)*x[2]**o+(-0.744144678185)*x[2]
        arg[1,2,1,1]=(0.472846565983)*x[0]**o+(-0.837395663533)*x[0]+(0.526438435448)*x[1]**o+(0.310053110281)*x[1]+(0.634301291566)*x[2]**o+(0.608270679412)*x[2]
        arg[1,2,2,0]=(-0.212000572333)*x[0]**o+(0.644218219058)*x[0]+(0.827779140409)*x[1]**o+(0.563968121281)*x[1]+(-0.0556319977635)*x[2]**o+(0.98126338415)*x[2]
        arg[1,2,2,1]=(-0.632752637148)*x[0]**o+(-0.999490611532)*x[0]+(0.0717950178432)*x[1]**o+(-0.0611949850049)*x[1]+(0.740177288895)*x[2]**o+(0.241010363236)*x[2]
        arg[1,3,0,0]=(0.390081216336)*x[0]**o+(-0.389460065845)*x[0]+(0.0812808695521)*x[1]**o+(0.0843698990866)*x[1]+(0.672844227186)*x[2]**o+(0.306345015188)*x[2]
        arg[1,3,0,1]=(-0.402887151891)*x[0]**o+(-0.908615473265)*x[0]+(-0.296307155209)*x[1]**o+(0.228858413043)*x[1]+(0.509401868055)*x[2]**o+(-0.793787805038)*x[2]
        arg[1,3,1,0]=(0.253947492129)*x[0]**o+(0.401465417831)*x[0]+(-0.942329176703)*x[1]**o+(-0.518371272083)*x[1]+(0.90142806242)*x[2]**o+(0.694084281869)*x[2]
        arg[1,3,1,1]=(-0.435656267291)*x[0]**o+(-0.743058541536)*x[0]+(-0.22434079396)*x[1]**o+(-0.414125965702)*x[1]+(0.743546448275)*x[2]**o+(0.323188772386)*x[2]
        arg[1,3,2,0]=(-0.069319863146)*x[0]**o+(-0.0749969565014)*x[0]+(0.29700943253)*x[1]**o+(0.972639470201)*x[1]+(-0.270592469279)*x[2]**o+(-0.355098058901)*x[2]
        arg[1,3,2,1]=(0.741825647399)*x[0]**o+(0.542055833087)*x[0]+(-0.0684409109262)*x[1]**o+(0.842094761143)*x[1]+(-0.777320219447)*x[2]**o+(0.758570478904)*x[2]
        arg[1,4,0,0]=(-0.66003338591)*x[0]**o+(-0.393755284573)*x[0]+(0.617154869065)*x[1]**o+(-0.480999198967)*x[1]+(0.350625972066)*x[2]**o+(0.733472524967)*x[2]
        arg[1,4,0,1]=(-0.0001759210396)*x[0]**o+(-0.0900574950432)*x[0]+(-0.592936627579)*x[1]**o+(-0.647214933825)*x[1]+(-0.43249743908)*x[2]**o+(-0.901387692551)*x[2]
        arg[1,4,1,0]=(0.052435899043)*x[0]**o+(0.163326392489)*x[0]+(0.433169876656)*x[1]**o+(0.977133611649)*x[1]+(-0.311959404527)*x[2]**o+(0.297218570651)*x[2]
        arg[1,4,1,1]=(0.00806522196579)*x[0]**o+(-0.121807466189)*x[0]+(-0.176557734776)*x[1]**o+(0.865146379005)*x[1]+(-0.304648040177)*x[2]**o+(0.269150163824)*x[2]
        arg[1,4,2,0]=(0.440212388104)*x[0]**o+(-0.590292628428)*x[0]+(-0.595359323633)*x[1]**o+(0.606734888384)*x[1]+(-0.951693103773)*x[2]**o+(-0.539741612207)*x[2]
        arg[1,4,2,1]=(0.346121727918)*x[0]**o+(0.188961570963)*x[0]+(0.402818736255)*x[1]**o+(-0.474029523974)*x[1]+(-0.95972237672)*x[2]**o+(0.983543070357)*x[2]
        arg[2,0,0,0]=(-0.801941188348)*x[0]**o+(-0.318215868754)*x[0]+(0.322031073259)*x[1]**o+(-0.00368402006147)*x[1]+(0.473846239338)*x[2]**o+(0.512752561753)*x[2]
        arg[2,0,0,1]=(0.786459471841)*x[0]**o+(0.941049841845)*x[0]+(0.064717660139)*x[1]**o+(-0.891518891268)*x[1]+(-0.317681565962)*x[2]**o+(-0.672310725168)*x[2]
        arg[2,0,1,0]=(0.813571093948)*x[0]**o+(-0.146551267883)*x[0]+(0.942106359835)*x[1]**o+(0.165543949281)*x[1]+(-0.0301736488709)*x[2]**o+(0.936915469225)*x[2]
        arg[2,0,1,1]=(-0.0298664722355)*x[0]**o+(-0.596644819792)*x[0]+(-0.306728572634)*x[1]**o+(0.770671782936)*x[1]+(-0.334606536003)*x[2]**o+(0.888940190159)*x[2]
        arg[2,0,2,0]=(0.688553055025)*x[0]**o+(-0.241541868482)*x[0]+(-0.694609526571)*x[1]**o+(-0.509896457784)*x[1]+(-0.0610272843414)*x[2]**o+(-0.251290389152)*x[2]
        arg[2,0,2,1]=(-0.959007707329)*x[0]**o+(-0.193856482901)*x[0]+(-0.138697126551)*x[1]**o+(0.768166305153)*x[1]+(0.951006645306)*x[2]**o+(-0.533986863953)*x[2]
        arg[2,1,0,0]=(0.648021743651)*x[0]**o+(-0.839074631201)*x[0]+(-0.445337118707)*x[1]**o+(0.997475278653)*x[1]+(-0.154475323496)*x[2]**o+(-0.45292992537)*x[2]
        arg[2,1,0,1]=(-0.134785884326)*x[0]**o+(-0.906738442209)*x[0]+(0.00509900741927)*x[1]**o+(0.489865232888)*x[1]+(0.620823018353)*x[2]**o+(-0.480127263607)*x[2]
        arg[2,1,1,0]=(-0.210371458935)*x[0]**o+(0.308453655681)*x[0]+(-0.510473100104)*x[1]**o+(-0.0776488834445)*x[1]+(0.58103002434)*x[2]**o+(0.039267622777)*x[2]
        arg[2,1,1,1]=(-0.124204683462)*x[0]**o+(0.429525935007)*x[0]+(0.183174600008)*x[1]**o+(-0.665914708196)*x[1]+(-0.576713549033)*x[2]**o+(-0.3788567811)*x[2]
        arg[2,1,2,0]=(-0.864065825608)*x[0]**o+(0.644046059464)*x[0]+(-0.265959280151)*x[1]**o+(0.989465634246)*x[1]+(0.536466956842)*x[2]**o+(0.689999246325)*x[2]
        arg[2,1,2,1]=(0.37622234855)*x[0]**o+(-0.905554688955)*x[0]+(-0.0258037833143)*x[1]**o+(-0.23415538846)*x[1]+(-0.64461795706)*x[2]**o+(0.262259040646)*x[2]
        arg[2,2,0,0]=(0.0541619627989)*x[0]**o+(-0.978918782799)*x[0]+(0.478333900479)*x[1]**o+(0.958652455667)*x[1]+(-0.746701804693)*x[2]**o+(-0.557405653804)*x[2]
        arg[2,2,0,1]=(0.611472347101)*x[0]**o+(-0.409754132769)*x[0]+(0.291274496706)*x[1]**o+(0.901983238042)*x[1]+(-0.102917678824)*x[2]**o+(-0.668205224505)*x[2]
        arg[2,2,1,0]=(-0.926062469523)*x[0]**o+(0.101032437783)*x[0]+(-0.385485330853)*x[1]**o+(0.0521756546126)*x[1]+(0.132320977758)*x[2]**o+(0.471619572767)*x[2]
        arg[2,2,1,1]=(0.0689111652346)*x[0]**o+(-0.517070173331)*x[0]+(-0.599184868344)*x[1]**o+(-0.814239996638)*x[1]+(0.97403158037)*x[2]**o+(-0.546843890524)*x[2]
        arg[2,2,2,0]=(-0.133251706696)*x[0]**o+(-0.0954736405212)*x[0]+(0.780584398494)*x[1]**o+(-0.505305860913)*x[1]+(-0.11518705364)*x[2]**o+(0.0855698333433)*x[2]
        arg[2,2,2,1]=(-0.567309977436)*x[0]**o+(-0.284182307402)*x[0]+(0.220302202269)*x[1]**o+(-0.252628588826)*x[1]+(-0.368390646272)*x[2]**o+(-0.577499260947)*x[2]
        arg[2,3,0,0]=(0.477968626266)*x[0]**o+(0.522764690241)*x[0]+(-0.0339165683557)*x[1]**o+(0.623897338329)*x[1]+(-0.169558925831)*x[2]**o+(-0.426663511213)*x[2]
        arg[2,3,0,1]=(0.185482762361)*x[0]**o+(0.496017413966)*x[0]+(-0.217189284009)*x[1]**o+(0.488205372581)*x[1]+(0.445762454794)*x[2]**o+(-0.423587587222)*x[2]
        arg[2,3,1,0]=(-0.489549524298)*x[0]**o+(-0.482299084307)*x[0]+(0.567729727254)*x[1]**o+(-0.341933025591)*x[1]+(-0.325429025641)*x[2]**o+(0.476030984261)*x[2]
        arg[2,3,1,1]=(-0.390754226646)*x[0]**o+(-0.980255652462)*x[0]+(0.109328516217)*x[1]**o+(-0.0141509875662)*x[1]+(0.486458644215)*x[2]**o+(-0.668018419371)*x[2]
        arg[2,3,2,0]=(0.352912006363)*x[0]**o+(-0.542969561227)*x[0]+(0.512067200296)*x[1]**o+(0.661110389562)*x[1]+(0.289744832231)*x[2]**o+(-0.176943484459)*x[2]
        arg[2,3,2,1]=(0.535262244127)*x[0]**o+(0.528325019892)*x[0]+(-0.106312276613)*x[1]**o+(-0.484575045858)*x[1]+(-0.407717587021)*x[2]**o+(-0.786366604025)*x[2]
        arg[2,4,0,0]=(-0.511771630262)*x[0]**o+(-0.590485876141)*x[0]+(0.493685168985)*x[1]**o+(-0.726700517739)*x[1]+(-0.213137223607)*x[2]**o+(-0.566923814015)*x[2]
        arg[2,4,0,1]=(0.421559088561)*x[0]**o+(0.838784944841)*x[0]+(-0.693654869402)*x[1]**o+(-0.0755493272975)*x[1]+(-0.174245799979)*x[2]**o+(0.242245946157)*x[2]
        arg[2,4,1,0]=(0.681235817735)*x[0]**o+(-0.313564199551)*x[0]+(0.663598935418)*x[1]**o+(-0.604327253895)*x[1]+(0.698985821444)*x[2]**o+(0.298899431146)*x[2]
        arg[2,4,1,1]=(0.432866305507)*x[0]**o+(-0.891501121013)*x[0]+(-0.290106178906)*x[1]**o+(0.268152357012)*x[1]+(0.75279498948)*x[2]**o+(0.211312203371)*x[2]
        arg[2,4,2,0]=(-0.956901622446)*x[0]**o+(0.742548241435)*x[0]+(-0.956209816095)*x[1]**o+(-0.539017080821)*x[1]+(-0.08062797007)*x[2]**o+(0.383843198102)*x[2]
        arg[2,4,2,1]=(-0.16700370288)*x[0]**o+(0.388837619298)*x[0]+(-0.190586726252)*x[1]**o+(0.916004535916)*x[1]+(0.310254387059)*x[2]**o+(-0.380371301717)*x[2]
        arg[3,0,0,0]=(0.64046380712)*x[0]**o+(0.254058760972)*x[0]+(-0.829050222873)*x[1]**o+(0.289942284927)*x[1]+(0.196268101069)*x[2]**o+(0.748880970446)*x[2]
        arg[3,0,0,1]=(-0.226057776579)*x[0]**o+(0.982685008224)*x[0]+(0.819680514589)*x[1]**o+(0.48005222662)*x[1]+(-0.615248069776)*x[2]**o+(-0.0923489923012)*x[2]
        arg[3,0,1,0]=(-0.358031084019)*x[0]**o+(-0.83941518843)*x[0]+(-0.0687218422525)*x[1]**o+(-0.648976147029)*x[1]+(0.711009285701)*x[2]**o+(0.360758898243)*x[2]
        arg[3,0,1,1]=(-0.312175299678)*x[0]**o+(0.360877864123)*x[0]+(0.653283960509)*x[1]**o+(0.323527948404)*x[1]+(-0.278454805659)*x[2]**o+(0.0585194127844)*x[2]
        arg[3,0,2,0]=(0.367207247645)*x[0]**o+(-0.0743547693027)*x[0]+(-0.473358544608)*x[1]**o+(0.332985041128)*x[1]+(-0.899612971909)*x[2]**o+(0.69409768473)*x[2]
        arg[3,0,2,1]=(-0.806787228991)*x[0]**o+(-0.110344204889)*x[0]+(-0.631610779885)*x[1]**o+(0.0239853435419)*x[1]+(0.83036063388)*x[2]**o+(0.623825256849)*x[2]
        arg[3,1,0,0]=(-0.268344201624)*x[0]**o+(-0.59230799177)*x[0]+(0.493719194706)*x[1]**o+(-0.128351291806)*x[1]+(0.0584636422077)*x[2]**o+(0.0859725996454)*x[2]
        arg[3,1,0,1]=(-0.870410307785)*x[0]**o+(-0.917370176071)*x[0]+(0.395701966434)*x[1]**o+(0.694856510628)*x[1]+(-0.748440730513)*x[2]**o+(0.748465450614)*x[2]
        arg[3,1,1,0]=(-0.178465888488)*x[0]**o+(-0.956884701156)*x[0]+(0.0472790389286)*x[1]**o+(0.946702274722)*x[1]+(-0.200668750878)*x[2]**o+(-0.0104023894447)*x[2]
        arg[3,1,1,1]=(0.113559334778)*x[0]**o+(0.270505514561)*x[0]+(-0.238008799138)*x[1]**o+(0.533067625871)*x[1]+(-0.586814102301)*x[2]**o+(0.045657074859)*x[2]
        arg[3,1,2,0]=(-0.574716699633)*x[0]**o+(-0.659499274639)*x[0]+(0.892605757655)*x[1]**o+(-0.39697940457)*x[1]+(0.363011145793)*x[2]**o+(-0.364722526091)*x[2]
        arg[3,1,2,1]=(-0.0193167048813)*x[0]**o+(0.761104649925)*x[0]+(0.490164183635)*x[1]**o+(0.10693267057)*x[1]+(0.635328989408)*x[2]**o+(-0.353114187925)*x[2]
        arg[3,2,0,0]=(0.0777423524941)*x[0]**o+(-0.431011355257)*x[0]+(-0.356762042134)*x[1]**o+(-0.94361679144)*x[1]+(-0.716191700456)*x[2]**o+(0.49906825221)*x[2]
        arg[3,2,0,1]=(0.80859948321)*x[0]**o+(0.542680788872)*x[0]+(-0.626897369633)*x[1]**o+(-0.821877255135)*x[1]+(-0.455575534023)*x[2]**o+(0.317480554025)*x[2]
        arg[3,2,1,0]=(-0.0144328272818)*x[0]**o+(-0.395367848628)*x[0]+(-0.601042089925)*x[1]**o+(-0.0822921879892)*x[1]+(0.896199761801)*x[2]**o+(-0.215055215166)*x[2]
        arg[3,2,1,1]=(0.377945289524)*x[0]**o+(-0.580064012583)*x[0]+(0.993926713349)*x[1]**o+(-0.736307101943)*x[1]+(-0.939628422411)*x[2]**o+(-0.923219473393)*x[2]
        arg[3,2,2,0]=(-0.733150287394)*x[0]**o+(0.692501827859)*x[0]+(0.904655347343)*x[1]**o+(0.310386512564)*x[1]+(-0.93795078156)*x[2]**o+(0.772534565721)*x[2]
        arg[3,2,2,1]=(-0.155229801255)*x[0]**o+(0.580356480314)*x[0]+(0.0673352927499)*x[1]**o+(-0.274975805243)*x[1]+(0.553818209037)*x[2]**o+(-0.835142674988)*x[2]
        arg[3,3,0,0]=(0.60752130978)*x[0]**o+(0.847750395856)*x[0]+(0.8635279539)*x[1]**o+(0.16341072973)*x[1]+(0.470011948821)*x[2]**o+(0.41529971393)*x[2]
        arg[3,3,0,1]=(0.755049219363)*x[0]**o+(0.66080573214)*x[0]+(-0.0256982576552)*x[1]**o+(-0.0311937109251)*x[1]+(-0.028843413794)*x[2]**o+(-0.19345503082)*x[2]
        arg[3,3,1,0]=(-0.39765237082)*x[0]**o+(-0.746359385218)*x[0]+(0.275396324626)*x[1]**o+(0.064571659107)*x[1]+(0.771289010752)*x[2]**o+(-6.69891594269e-05)*x[2]
        arg[3,3,1,1]=(0.350497253938)*x[0]**o+(-0.236419516561)*x[0]+(0.673483791847)*x[1]**o+(-0.934644138115)*x[1]+(0.994408125332)*x[2]**o+(-0.398423596854)*x[2]
        arg[3,3,2,0]=(-0.952544750261)*x[0]**o+(0.24256752245)*x[0]+(0.376782843601)*x[1]**o+(-0.349840168156)*x[1]+(-0.260325141903)*x[2]**o+(-0.452866344447)*x[2]
        arg[3,3,2,1]=(-0.0232941915849)*x[0]**o+(0.191625715695)*x[0]+(-0.490187753329)*x[1]**o+(-0.823976612466)*x[1]+(0.476804203752)*x[2]**o+(0.61057521184)*x[2]
        arg[3,4,0,0]=(0.0279130167137)*x[0]**o+(-0.134953133648)*x[0]+(0.0821614836141)*x[1]**o+(-0.894046763697)*x[1]+(-0.497197221983)*x[2]**o+(0.586297244251)*x[2]
        arg[3,4,0,1]=(0.0423746259802)*x[0]**o+(0.458075512165)*x[0]+(0.855157549678)*x[1]**o+(-0.607654212682)*x[1]+(-0.672693159955)*x[2]**o+(-0.300116892313)*x[2]
        arg[3,4,1,0]=(-0.695685046398)*x[0]**o+(-0.709807910289)*x[0]+(-0.167879145281)*x[1]**o+(0.524084329764)*x[1]+(0.763706690019)*x[2]**o+(-0.219502671318)*x[2]
        arg[3,4,1,1]=(0.864599640696)*x[0]**o+(-0.341125612414)*x[0]+(-0.548888211648)*x[1]**o+(0.918497762429)*x[1]+(-0.445600024463)*x[2]**o+(0.221658300325)*x[2]
        arg[3,4,2,0]=(-0.893419673448)*x[0]**o+(0.653718653341)*x[0]+(0.323868832441)*x[1]**o+(0.905933530129)*x[1]+(0.62045953406)*x[2]**o+(0.145474797235)*x[2]
        arg[3,4,2,1]=(0.480643160777)*x[0]**o+(-0.15896921521)*x[0]+(-0.329564714008)*x[1]**o+(-0.613303857496)*x[1]+(-0.483827450625)*x[2]**o+(-0.91023336048)*x[2]
        ref[0,0,0,0]=(0.520702135089)/(o+1.)+(0.109542934387)+(0.816458625566)*0.5**o
        ref[0,0,0,1]=(0.188518886272)/(o+1.)+(-0.938691737334)+(-0.709065020122)*0.5**o
        ref[0,0,1,0]=(-0.757157441618)/(o+1.)+(0.316696781197)+(-0.09165636265)*0.5**o
        ref[0,0,1,1]=(0.325937013197)/(o+1.)+(0.0450843195409)+(0.900670566988)*0.5**o
        ref[0,0,2,0]=(1.13837105734)/(o+1.)+(-0.573151939969)+(-0.883617789072)*0.5**o
        ref[0,0,2,1]=(-0.665166165764)/(o+1.)+(-0.845347549645)+(-0.770041366552)*0.5**o
        ref[0,1,0,0]=(0.714049217035)/(o+1.)+(0.4125480263)+(0.90396926296)*0.5**o
        ref[0,1,0,1]=(0.429723443697)/(o+1.)+(0.255965487666)+(0.47956310963)*0.5**o
        ref[0,1,1,0]=(-0.282787702479)/(o+1.)+(-0.631122483908)+(-0.75177304495)*0.5**o
        ref[0,1,1,1]=(1.16234435068)/(o+1.)+(1.09548265929)+(-0.109937058453)*0.5**o
        ref[0,1,2,0]=(-0.619187813423)/(o+1.)+(0.47035748919)+(-0.13801307581)*0.5**o
        ref[0,1,2,1]=(0.641297345823)/(o+1.)+(0.937805735461)+(-0.518540484981)*0.5**o
        ref[0,2,0,0]=(1.36665521497)/(o+1.)+(-0.736611486382)+(0.286922773164)*0.5**o
        ref[0,2,0,1]=(-0.27582202139)/(o+1.)+(-0.312977125741)+(-0.116447648923)*0.5**o
        ref[0,2,1,0]=(0.294494393603)/(o+1.)+(0.117678635379)+(-0.322803067618)*0.5**o
        ref[0,2,1,1]=(-0.251319095061)/(o+1.)+(0.517336946518)+(0.997823909059)*0.5**o
        ref[0,2,2,0]=(-1.06239756943)/(o+1.)+(-0.174439303934)+(0.516866019209)*0.5**o
        ref[0,2,2,1]=(-0.114597419557)/(o+1.)+(-0.488263901705)+(0.777538237337)*0.5**o
        ref[0,3,0,0]=(-0.545872917587)/(o+1.)+(0.320972405109)+(-0.622067362782)*0.5**o
        ref[0,3,0,1]=(-1.41803804504)/(o+1.)+(0.326673034142)+(-0.49698112241)*0.5**o
        ref[0,3,1,0]=(0.220871698725)/(o+1.)+(0.352475535909)+(0.917091542097)*0.5**o
        ref[0,3,1,1]=(-1.82481767225)/(o+1.)+(-0.534598958622)+(-0.437188100982)*0.5**o
        ref[0,3,2,0]=(1.04475880275)/(o+1.)+(0.00788693262039)+(-0.841265485658)*0.5**o
        ref[0,3,2,1]=(-0.262966085934)/(o+1.)+(0.330079558014)+(0.383282548921)*0.5**o
        ref[0,4,0,0]=(-1.66865494548)/(o+1.)+(-0.567436682752)+(0.0385891845805)*0.5**o
        ref[0,4,0,1]=(-0.183458096784)/(o+1.)+(-1.1357723338)+(-0.387739686761)*0.5**o
        ref[0,4,1,0]=(0.611419775289)/(o+1.)+(0.0492166712458)+(-0.329464483421)*0.5**o
        ref[0,4,1,1]=(0.000995600207189)/(o+1.)+(-0.182953786649)+(0.63272559561)*0.5**o
        ref[0,4,2,0]=(0.630523648936)/(o+1.)+(-0.0318688096074)+(-0.847582850604)*0.5**o
        ref[0,4,2,1]=(0.306969678703)/(o+1.)+(-0.726991015938)+(-0.722990535113)*0.5**o
        ref[1,0,0,0]=(-0.406383597128)/(o+1.)+(-0.580550741796)+(0.0725951752952)*0.5**o
        ref[1,0,0,1]=(0.988153215411)/(o+1.)+(-0.295391144874)+(-0.254920010173)*0.5**o
        ref[1,0,1,0]=(-0.113985267485)/(o+1.)+(0.381963273358)+(0.486342223622)*0.5**o
        ref[1,0,1,1]=(-1.0749658001)/(o+1.)+(-0.980590626052)+(0.777015150638)*0.5**o
        ref[1,0,2,0]=(0.866170818208)/(o+1.)+(0.295155963167)+(0.240813275278)*0.5**o
        ref[1,0,2,1]=(-0.502098016236)/(o+1.)+(-0.404695777523)+(-0.670257601629)*0.5**o
        ref[1,1,0,0]=(-0.0930803554155)/(o+1.)+(-0.262152356852)+(0.985895581872)*0.5**o
        ref[1,1,0,1]=(-1.66036439576)/(o+1.)+(0.188095420862)+(-0.954132399275)*0.5**o
        ref[1,1,1,0]=(-0.548936659502)/(o+1.)+(0.974895418354)+(0.462810965625)*0.5**o
        ref[1,1,1,1]=(-1.80963059517)/(o+1.)+(-0.200285193041)+(0.645629353541)*0.5**o
        ref[1,1,2,0]=(1.41739085898)/(o+1.)+(0.549897915219)+(-0.554993653726)*0.5**o
        ref[1,1,2,1]=(0.0907943154278)/(o+1.)+(-0.450977439484)+(0.1476587556)*0.5**o
        ref[1,2,0,0]=(-0.657176681491)/(o+1.)+(-0.433402993059)+(-0.715887407694)*0.5**o
        ref[1,2,0,1]=(-0.562421731301)/(o+1.)+(0.0104884758643)+(0.48440337855)*0.5**o
        ref[1,2,1,0]=(1.28216615223)/(o+1.)+(-0.33142121119)+(0.621562227153)*0.5**o
        ref[1,2,1,1]=(1.16073972701)/(o+1.)+(0.0404640630801)+(0.472846565983)*0.5**o
        ref[1,2,2,0]=(0.772147142645)/(o+1.)+(1.09472486224)+(-0.212000572333)*0.5**o
        ref[1,2,2,1]=(0.811972306738)/(o+1.)+(-0.409837616651)+(-0.632752637148)*0.5**o
        ref[1,3,0,0]=(0.754125096738)/(o+1.)+(0.000627424214716)+(0.390081216336)*0.5**o
        ref[1,3,0,1]=(0.213094712846)/(o+1.)+(-0.73677243263)+(-0.402887151891)*0.5**o
        ref[1,3,1,0]=(-0.0409011142831)/(o+1.)+(0.288589213809)+(0.253947492129)*0.5**o
        ref[1,3,1,1]=(0.519205654314)/(o+1.)+(-0.416997867426)+(-0.435656267291)*0.5**o
        ref[1,3,2,0]=(0.0264169632513)/(o+1.)+(0.271272227399)+(-0.069319863146)*0.5**o
        ref[1,3,2,1]=(-0.845761130374)/(o+1.)+(1.07136053657)+(0.741825647399)*0.5**o
        ref[1,4,0,0]=(0.967780841131)/(o+1.)+(-0.0706409792865)+(-0.66003338591)*0.5**o
        ref[1,4,0,1]=(-1.02543406666)/(o+1.)+(-0.81933006071)+(-0.0001759210396)*0.5**o
        ref[1,4,1,0]=(0.121210472128)/(o+1.)+(0.718839287394)+(0.052435899043)*0.5**o
        ref[1,4,1,1]=(-0.481205774953)/(o+1.)+(0.50624453832)+(0.00806522196579)*0.5**o
        ref[1,4,2,0]=(-1.54705242741)/(o+1.)+(-0.261649676126)+(0.440212388104)*0.5**o
        ref[1,4,2,1]=(-0.556903640465)/(o+1.)+(0.349237558673)+(0.346121727918)*0.5**o
        ref[2,0,0,0]=(0.795877312597)/(o+1.)+(0.0954263364687)+(-0.801941188348)*0.5**o
        ref[2,0,0,1]=(-0.252963905823)/(o+1.)+(-0.311389887295)+(0.786459471841)*0.5**o
        ref[2,0,1,0]=(0.911932710964)/(o+1.)+(0.477954075311)+(0.813571093948)*0.5**o
        ref[2,0,1,1]=(-0.641335108636)/(o+1.)+(0.531483576652)+(-0.0298664722355)*0.5**o
        ref[2,0,2,0]=(-0.755636810913)/(o+1.)+(-0.501364357709)+(0.688553055025)*0.5**o
        ref[2,0,2,1]=(0.812309518755)/(o+1.)+(0.0201614791497)+(-0.959007707329)*0.5**o
        ref[2,1,0,0]=(-0.599812442203)/(o+1.)+(-0.147264638959)+(0.648021743651)*0.5**o
        ref[2,1,0,1]=(0.625922025772)/(o+1.)+(-0.448500236464)+(-0.134785884326)*0.5**o
        ref[2,1,1,0]=(0.0705569242367)/(o+1.)+(0.135036197507)+(-0.210371458935)*0.5**o
        ref[2,1,1,1]=(-0.393538949024)/(o+1.)+(-0.307622777145)+(-0.124204683462)*0.5**o
        ref[2,1,2,0]=(0.270507676691)/(o+1.)+(1.16175547002)+(-0.864065825608)*0.5**o
        ref[2,1,2,1]=(-0.670421740374)/(o+1.)+(-0.438725518385)+(0.37622234855)*0.5**o
        ref[2,2,0,0]=(-0.268367904213)/(o+1.)+(-0.288835990468)+(0.0541619627989)*0.5**o
        ref[2,2,0,1]=(0.188356817882)/(o+1.)+(-0.0879880596161)+(0.611472347101)*0.5**o
        ref[2,2,1,0]=(-0.253164353095)/(o+1.)+(0.312413832582)+(-0.926062469523)*0.5**o
        ref[2,2,1,1]=(0.374846712027)/(o+1.)+(-0.939077030246)+(0.0689111652346)*0.5**o
        ref[2,2,2,0]=(0.665397344854)/(o+1.)+(-0.257604834045)+(-0.133251706696)*0.5**o
        ref[2,2,2,1]=(-0.148088444003)/(o+1.)+(-0.557155078588)+(-0.567309977436)*0.5**o
        ref[2,3,0,0]=(-0.203475494187)/(o+1.)+(0.359999258678)+(0.477968626266)*0.5**o
        ref[2,3,0,1]=(0.228573170785)/(o+1.)+(0.280317599662)+(0.185482762361)*0.5**o
        ref[2,3,1,0]=(0.242300701614)/(o+1.)+(-0.174100562819)+(-0.489549524298)*0.5**o
        ref[2,3,1,1]=(0.595787160431)/(o+1.)+(-0.8312125297)+(-0.390754226646)*0.5**o
        ref[2,3,2,0]=(0.801812032527)/(o+1.)+(-0.0294013280621)+(0.352912006363)*0.5**o
        ref[2,3,2,1]=(-0.514029863634)/(o+1.)+(-0.371308314996)+(0.535262244127)*0.5**o
        ref[2,4,0,0]=(0.280547945378)/(o+1.)+(-0.942055103948)+(-0.511771630262)*0.5**o
        ref[2,4,0,1]=(-0.867900669381)/(o+1.)+(0.50274078185)+(0.421559088561)*0.5**o
        ref[2,4,1,0]=(1.36258475686)/(o+1.)+(-0.30949601115)+(0.681235817735)*0.5**o
        ref[2,4,1,1]=(0.462688810574)/(o+1.)+(-0.206018280315)+(0.432866305507)*0.5**o
        ref[2,4,2,0]=(-1.03683778617)/(o+1.)+(0.293687179358)+(-0.956901622446)*0.5**o
        ref[2,4,2,1]=(0.119667660807)/(o+1.)+(0.462235426749)+(-0.16700370288)*0.5**o
        ref[3,0,0,0]=(-0.632782121804)/(o+1.)+(0.646441008172)+(0.64046380712)*0.5**o
        ref[3,0,0,1]=(0.204432444813)/(o+1.)+(0.685194121272)+(-0.226057776579)*0.5**o
        ref[3,0,1,0]=(0.642287443449)/(o+1.)+(-0.563816218608)+(-0.358031084019)*0.5**o
        ref[3,0,1,1]=(0.37482915485)/(o+1.)+(0.371462612656)+(-0.312175299678)*0.5**o
        ref[3,0,2,0]=(-1.37297151652)/(o+1.)+(0.476363978278)+(0.367207247645)*0.5**o
        ref[3,0,2,1]=(0.198749853995)/(o+1.)+(0.268733197751)+(-0.806787228991)*0.5**o
        ref[3,1,0,0]=(0.552182836914)/(o+1.)+(-0.317343341965)+(-0.268344201624)*0.5**o
        ref[3,1,0,1]=(-0.352738764079)/(o+1.)+(0.262975892585)+(-0.870410307785)*0.5**o
        ref[3,1,1,0]=(-0.153389711949)/(o+1.)+(-0.0102924079393)+(-0.178465888488)*0.5**o
        ref[3,1,1,1]=(-0.824822901439)/(o+1.)+(0.424615107645)+(0.113559334778)*0.5**o
        ref[3,1,2,0]=(1.25561690345)/(o+1.)+(-0.71060060265)+(-0.574716699633)*0.5**o
        ref[3,1,2,1]=(1.12549317304)/(o+1.)+(0.257461566285)+(-0.0193167048813)*0.5**o
        ref[3,2,0,0]=(-1.07295374259)/(o+1.)+(-0.437779947243)+(0.0777423524941)*0.5**o
        ref[3,2,0,1]=(-1.08247290366)/(o+1.)+(0.019142043881)+(0.80859948321)*0.5**o
        ref[3,2,1,0]=(0.295157671875)/(o+1.)+(-0.346357625892)+(-0.0144328272818)*0.5**o
        ref[3,2,1,1]=(0.054298290938)/(o+1.)+(-1.11979529396)+(0.377945289524)*0.5**o
        ref[3,2,2,0]=(-0.0332954342171)/(o+1.)+(0.887711453071)+(-0.733150287394)*0.5**o
        ref[3,2,2,1]=(0.621153501787)/(o+1.)+(-0.264880999958)+(-0.155229801255)*0.5**o
        ref[3,3,0,0]=(1.33353990272)/(o+1.)+(0.713230419758)+(0.60752130978)*0.5**o
        ref[3,3,0,1]=(-0.0545416714491)/(o+1.)+(0.218078495198)+(0.755049219363)*0.5**o
        ref[3,3,1,0]=(1.04668533538)/(o+1.)+(-0.340927357635)+(-0.39765237082)*0.5**o
        ref[3,3,1,1]=(1.66789191718)/(o+1.)+(-0.784743625765)+(0.350497253938)*0.5**o
        ref[3,3,2,0]=(0.116457701698)/(o+1.)+(-0.280069495076)+(-0.952544750261)*0.5**o
        ref[3,3,2,1]=(-0.0133835495768)/(o+1.)+(-0.0108878424651)+(-0.0232941915849)*0.5**o
        ref[3,4,0,0]=(-0.415035738369)/(o+1.)+(-0.221351326547)+(0.0279130167137)*0.5**o
        ref[3,4,0,1]=(0.182464389723)/(o+1.)+(-0.224847796415)+(0.0423746259802)*0.5**o
        ref[3,4,1,0]=(0.595827544738)/(o+1.)+(-0.202613125922)+(-0.695685046398)*0.5**o
        ref[3,4,1,1]=(-0.99448823611)/(o+1.)+(0.39951522517)+(0.864599640696)*0.5**o
        ref[3,4,2,0]=(0.944328366501)/(o+1.)+(0.852563490353)+(-0.893419673448)*0.5**o
        ref[3,4,2,1]=(-0.813392164634)/(o+1.)+(-0.841253216593)+(0.480643160777)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactOne_fromData_ReducedSolution_rank0(self):
      """
      tests integral of rank 0 Data on the FunctionOnContactOne

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports integral on FunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Data(0,(),w)
      if dim==2:
        arg=(0.241972460279)*x[0]+(0.539729925673)*x[1]
        ref=(-0.11832904353)/(o+1.)+(0.649911364492)+(-0.399791299503)*0.5**o
      else:
        arg=(0.53919797656)*x[0]+(0.379010664884)*x[1]+(0.274416064529)*x[2]
        ref=(0.307947301472)/(o+1.)+(0.120674157748)+(0.643329089005)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactOne_fromData_ReducedSolution_rank1(self):
      """
      tests integral of rank 1 Data on the FunctionOnContactOne

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports integral on FunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Data(0,(2,),w)
      ref=numpy.zeros((2,),numpy.float_)
      if dim==2:
        arg[0]=(0.84849017056)*x[0]+(-0.161962845202)*x[1]
        arg[1]=(-0.794717919135)*x[0]+(1.16003773358)*x[1]
        ref[0]=(0.780514448202)/(o+1.)+(-0.34628958021)+(0.598592037576)*0.5**o
        ref[1]=(0.570158156278)/(o+1.)+(0.241192316704)+(-0.687222975237)*0.5**o
      else:
        arg[0]=(-0.637352163634)*x[0]+(1.49477462643)*x[1]+(-0.284018740238)*x[2]
        arg[1]=(0.125937459904)*x[0]+(-0.206499869891)*x[1]+(-0.342856240345)*x[2]
        ref[0]=(-0.274486314451)/(o+1.)+(0.379737553853)+(0.0884149293071)*0.5**o
        ref[1]=(-0.78452814021)/(o+1.)+(-0.122017410137)+(0.605144310151)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactOne_fromData_ReducedSolution_rank2(self):
      """
      tests integral of rank 2 Data on the FunctionOnContactOne

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports integral on FunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5),w)
      ref=numpy.zeros((4, 5),numpy.float_)
      if dim==2:
        arg[0,0]=(-0.440868762581)*x[0]+(-0.216223666616)*x[1]
        arg[0,1]=(-1.78807355591)*x[0]+(-0.208278123838)*x[1]
        arg[0,2]=(-0.448066998103)*x[0]+(-0.819815507826)*x[1]
        arg[0,3]=(1.27247881689)*x[0]+(0.0653439567221)*x[1]
        arg[0,4]=(-1.00344990223)*x[0]+(-0.291263790583)*x[1]
        arg[1,0]=(0.440080423838)*x[0]+(0.0356836558113)*x[1]
        arg[1,1]=(-1.49545340091)*x[0]+(-0.133615590211)*x[1]
        arg[1,2]=(1.05947638385)*x[0]+(0.408685904238)*x[1]
        arg[1,3]=(-0.170309086605)*x[0]+(0.660523970285)*x[1]
        arg[1,4]=(0.764323678475)*x[0]+(-0.529415080303)*x[1]
        arg[2,0]=(0.55270081325)*x[0]+(-1.60630146231)*x[1]
        arg[2,1]=(1.57419330248)*x[0]+(-1.24644288706)*x[1]
        arg[2,2]=(0.0416757238363)*x[0]+(0.0247379738465)*x[1]
        arg[2,3]=(1.0200370393)*x[0]+(0.364209137693)*x[1]
        arg[2,4]=(-1.21858490389)*x[0]+(0.523056254546)*x[1]
        arg[3,0]=(0.0524205820842)*x[0]+(1.65953217854)*x[1]
        arg[3,1]=(1.21824883395)*x[0]+(0.905190834998)*x[1]
        arg[3,2]=(0.226344434525)*x[0]+(-0.735349531462)*x[1]
        arg[3,3]=(-1.00295245717)*x[0]+(-0.719605376499)*x[1]
        arg[3,4]=(-0.446228759693)*x[0]+(1.23441974277)*x[1]
        ref[0,0]=(0.347328582602)/(o+1.)+(-0.520484463519)+(0.0365479152384)*0.5**o
        ref[0,1]=(-0.274564442352)/(o+1.)+(-0.413557210678)+(-0.894672816039)*0.5**o
        ref[0,2]=(0.0720644688954)/(o+1.)+(-0.906636966188)+(0.473326957552)*0.5**o
        ref[0,3]=(-0.747749341777)/(o+1.)+(0.730249972378)+(0.625072170633)*0.5**o
        ref[0,4]=(-0.194031116169)/(o+1.)+(-0.432063325829)+(-0.236555924983)*0.5**o
        ref[1,0]=(0.807081538033)/(o+1.)+(-0.612266650007)+(0.893215841629)*0.5**o
        ref[1,1]=(-0.662516518957)/(o+1.)+(-0.176318925886)+(-0.61391462039)*0.5**o
        ref[1,2]=(-0.570345636156)/(o+1.)+(0.791877954963)+(0.454752014319)*0.5**o
        ref[1,3]=(-0.312400910795)/(o+1.)+(0.212935137466)+(0.376745519544)*0.5**o
        ref[1,4]=(-0.764680414264)/(o+1.)+(0.174215486579)+(0.651158039278)*0.5**o
        ref[2,0]=(-0.837942771642)/(o+1.)+(-0.187938486702)+(0.160219095985)*0.5**o
        ref[2,1]=(-0.672421039761)/(o+1.)+(0.17384002882)+(0.652491397549)*0.5**o
        ref[2,2]=(-0.656187949922)/(o+1.)+(0.559888736121)+(-0.397175824637)*0.5**o
        ref[2,3]=(0.259558368195)/(o+1.)+(0.55148644978)+(0.0217149092406)*0.5**o
        ref[2,4]=(-0.188397537901)/(o+1.)+(-0.0561706170512)+(-0.394789877345)*0.5**o
        ref[3,0]=(0.91055628617)/(o+1.)+(0.574211274098)+(-0.347026073739)*0.5**o
        ref[3,1]=(0.29225509237)/(o+1.)+(0.64447705482)+(0.542230466941)*0.5**o
        ref[3,2]=(0.121065624384)/(o+1.)+(0.0316495277836)+(-0.693369776888)*0.5**o
        ref[3,3]=(-0.943523334976)/(o+1.)+(-0.266302644031)+(-0.246429210629)*0.5**o
        ref[3,4]=(0.714915399455)/(o+1.)+(0.459963131994)+(-0.846650680369)*0.5**o
      else:
        arg[0,0]=(-0.493869713454)*x[0]+(-0.381969413515)*x[1]+(1.59862368848)*x[2]
        arg[0,1]=(-0.581245543547)*x[0]+(0.961117271889)*x[1]+(-0.158073515343)*x[2]
        arg[0,2]=(-0.635283143641)*x[0]+(0.695573236987)*x[1]+(-0.719180832278)*x[2]
        arg[0,3]=(-0.737780596733)*x[0]+(-1.11171283352)*x[1]+(0.459954346315)*x[2]
        arg[0,4]=(1.12522065074)*x[0]+(-1.08427904854)*x[1]+(-0.715679673556)*x[2]
        arg[1,0]=(0.972468186181)*x[0]+(-0.211014530435)*x[1]+(-0.734414099002)*x[2]
        arg[1,1]=(-1.67337704748)*x[0]+(0.871096213089)*x[1]+(1.51063313223)*x[2]
        arg[1,2]=(-0.222013007834)*x[0]+(1.46474508326)*x[1]+(-0.979243908862)*x[2]
        arg[1,3]=(1.21195281087)*x[0]+(0.274082389329)*x[1]+(1.56144436465)*x[2]
        arg[1,4]=(1.35812865884)*x[0]+(1.25819798853)*x[1]+(0.895334048395)*x[2]
        arg[2,0]=(-0.167404344437)*x[0]+(0.0710591934315)*x[1]+(-0.124469683665)*x[2]
        arg[2,1]=(-0.492240632996)*x[0]+(-0.317251117382)*x[1]+(0.0195461703529)*x[2]
        arg[2,2]=(-0.277590266225)*x[0]+(-1.65859476899)*x[1]+(-0.0100216405668)*x[2]
        arg[2,3]=(-0.0701281298455)*x[0]+(0.502137254703)*x[1]+(0.701220923443)*x[2]
        arg[2,4]=(-0.72186096407)*x[0]+(-0.900487568179)*x[1]+(-0.735573922826)*x[2]
        arg[3,0]=(-0.0677324474129)*x[0]+(-0.649963209718)*x[1]+(-0.770861099726)*x[2]
        arg[3,1]=(0.563588189481)*x[0]+(0.714614932251)*x[1]+(-0.0957764632886)*x[2]
        arg[3,2]=(1.13337395062)*x[0]+(0.507556316073)*x[1]+(-0.196759085949)*x[2]
        arg[3,3]=(-1.2636883168)*x[0]+(-0.811660344076)*x[1]+(0.645462052598)*x[2]
        arg[3,4]=(-0.104175186987)*x[0]+(-0.929176038155)*x[1]+(1.07530379856)*x[2]
        ref[0,0]=(0.84538674973)/(o+1.)+(0.0569713556632)+(-0.236544899547)*0.5**o
        ref[0,1]=(-0.307689589699)/(o+1.)+(0.245040337642)+(0.0394071274131)*0.5**o
        ref[0,2]=(0.955244736357)/(o+1.)+(-0.341771136504)+(-0.930593202281)*0.5**o
        ref[0,3]=(-0.353490461722)/(o+1.)+(-0.476228433901)+(-0.083591754415)*0.5**o
        ref[0,4]=(-1.31220485482)/(o+1.)+(0.0466426656676)+(0.544181452128)*0.5**o
        ref[1,0]=(0.22773475414)/(o+1.)+(-0.488454801654)+(0.776214405914)*0.5**o
        ref[1,1]=(1.50688678707)/(o+1.)+(0.0851415404695)+(-0.96881757017)*0.5**o
        ref[1,2]=(0.313471097102)/(o+1.)+(-0.0875594518156)+(0.125135973096)*0.5**o
        ref[1,3]=(0.269521220179)/(o+1.)+(1.1192007195)+(0.539556905661)*0.5**o
        ref[1,4]=(1.91499876064)/(o+1.)+(0.509063797893)+(0.578534339343)*0.5**o
        ref[2,0]=(0.941352651562)/(o+1.)+(-0.966199402186)+(0.770231318139)*0.5**o
        ref[2,1]=(-0.141490823562)/(o+1.)+(-0.20919987022)+(-0.230055016023)*0.5**o
        ref[2,2]=(-0.916148047971)/(o+1.)+(-0.371740600192)+(-0.286577427426)*0.5**o
        ref[2,3]=(0.229516219331)/(o+1.)+(0.274790532736)+(0.354132763498)*0.5**o
        ref[2,4]=(-1.19120612619)/(o+1.)+(-0.139859859385)+(-0.88699661011)*0.5**o
        ref[3,0]=(-1.48630121465)/(o+1.)+(0.0744166251063)+(-0.151088792417)*0.5**o
        ref[3,1]=(-0.335583560513)/(o+1.)+(0.277436410841)+(0.963137397276)*0.5**o
        ref[3,2]=(-0.673942688644)/(o+1.)+(0.70077509905)+(0.716563671283)*0.5**o
        ref[3,3]=(-0.148394092795)/(o+1.)+(-0.237190239072)+(-0.807112037336)*0.5**o
        ref[3,4]=(0.529983886211)/(o+1.)+(-0.371055083588)+(0.254078854382)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactOne_fromData_ReducedSolution_rank3(self):
      """
      tests integral of rank 3 Data on the FunctionOnContactOne

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports integral on FunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Data(0,(6, 2, 2),w)
      ref=numpy.zeros((6, 2, 2),numpy.float_)
      if dim==2:
        arg[0,0,0]=(0.661725715165)*x[0]+(0.219720717581)*x[1]
        arg[0,0,1]=(-0.642240082304)*x[0]+(-0.279652756776)*x[1]
        arg[0,1,0]=(0.0204235813151)*x[0]+(-0.374873249933)*x[1]
        arg[0,1,1]=(1.4418481252)*x[0]+(0.205656834772)*x[1]
        arg[1,0,0]=(0.65992179951)*x[0]+(-1.04887875458)*x[1]
        arg[1,0,1]=(-0.195480216199)*x[0]+(-0.373905275039)*x[1]
        arg[1,1,0]=(0.781251543352)*x[0]+(0.0416248109536)*x[1]
        arg[1,1,1]=(-0.925751319189)*x[0]+(-0.113992910046)*x[1]
        arg[2,0,0]=(-0.0767414065227)*x[0]+(-0.156682840973)*x[1]
        arg[2,0,1]=(0.688457272529)*x[0]+(0.524916336046)*x[1]
        arg[2,1,0]=(0.753722830084)*x[0]+(-0.890325839466)*x[1]
        arg[2,1,1]=(0.730789290145)*x[0]+(-0.278986703716)*x[1]
        arg[3,0,0]=(-0.287376796439)*x[0]+(-0.153987233224)*x[1]
        arg[3,0,1]=(-1.15687435255)*x[0]+(0.250005890238)*x[1]
        arg[3,1,0]=(-0.845312830629)*x[0]+(0.256211962352)*x[1]
        arg[3,1,1]=(-0.574311820519)*x[0]+(-0.440297035998)*x[1]
        arg[4,0,0]=(-0.360607647308)*x[0]+(-1.29380437557)*x[1]
        arg[4,0,1]=(1.32019433485)*x[0]+(0.295255956618)*x[1]
        arg[4,1,0]=(-1.64162941972)*x[0]+(0.117482372162)*x[1]
        arg[4,1,1]=(-0.303103646619)*x[0]+(-0.88458982681)*x[1]
        arg[5,0,0]=(-0.666761879038)*x[0]+(-0.735786278877)*x[1]
        arg[5,0,1]=(0.0892896382617)*x[0]+(-0.45478872773)*x[1]
        arg[5,1,0]=(-0.50985624415)*x[0]+(-0.400022939567)*x[1]
        arg[5,1,1]=(1.39240706368)*x[0]+(0.0149740822664)*x[1]
        ref[0,0,0]=(-0.114834538488)/(o+1.)+(0.240621006484)+(0.515038958264)*0.5**o
        ref[0,0,1]=(0.588698337077)/(o+1.)+(-0.702256377233)+(-0.10607842169)*0.5**o
        ref[0,1,0]=(0.13849818224)/(o+1.)+(-0.277555277991)+(0.0621627051247)*0.5**o
        ref[0,1,1]=(-0.0878009521163)/(o+1.)+(0.630168348947)+(0.474969214192)*0.5**o
        ref[1,0,0]=(-0.696600538111)/(o+1.)+(-0.343318260912)+(0.994280104861)*0.5**o
        ref[1,0,1]=(-0.0887026688527)/(o+1.)+(-0.385753227979)+(0.290823633572)*0.5**o
        ref[1,1,0]=(0.831408608956)/(o+1.)+(-0.242985671853)+(0.477439089057)*0.5**o
        ref[1,1,1]=(-0.621059053191)/(o+1.)+(-0.0700203481241)+(-0.278644479797)*0.5**o
        ref[2,0,0]=(0.7890720399)/(o+1.)+(-0.539721196393)+(0.0569461053905)*0.5**o
        ref[2,0,1]=(-0.228628151401)/(o+1.)+(0.826946074171)+(-0.211890388366)*0.5**o
        ref[2,1,0]=(-0.837617325493)/(o+1.)+(0.289656653751)+(0.121701008608)*0.5**o
        ref[2,1,1]=(0.246577988625)/(o+1.)+(0.0512227634376)+(0.102779070928)*0.5**o
        ref[3,0,0]=(-0.0109584684212)/(o+1.)+(-0.334888666171)+(0.239371771102)*0.5**o
        ref[3,0,1]=(-0.125987676419)/(o+1.)+(-0.170179004032)+(-0.440522777827)*0.5**o
        ref[3,1,0]=(-0.354093141636)/(o+1.)+(-0.177294275919)+(0.119580825198)*0.5**o
        ref[3,1,1]=(-0.746761075412)/(o+1.)+(0.192068760731)+(-0.651985302566)*0.5**o
        ref[4,0,0]=(-0.42411673219)/(o+1.)+(-0.579162944937)+(-0.0719694008158)*0.5**o
        ref[4,0,1]=(0.169881169584)/(o+1.)+(0.483614029132)+(0.478341063616)*0.5**o
        ref[4,1,0]=(0.943567836264)/(o+1.)+(-0.898782941264)+(-0.670149001297)*0.5**o
        ref[4,1,1]=(-0.703610807936)/(o+1.)+(0.225342628657)+(-0.934767922806)*0.5**o
        ref[5,0,0]=(-0.788792225213)/(o+1.)+(-0.1768857093)+(-0.259984514102)*0.5**o
        ref[5,0,1]=(-0.788708879899)/(o+1.)+(-0.202288491141)+(0.827786772711)*0.5**o
        ref[5,1,0]=(-0.823081809707)/(o+1.)+(0.274720963093)+(-0.636239300197)*0.5**o
        ref[5,1,1]=(0.498922549807)/(o+1.)+(0.0129388257746)+(0.882580944588)*0.5**o
      else:
        arg[0,0,0]=(-0.426613654844)*x[0]+(-1.01433980801)*x[1]+(0.564621874916)*x[2]
        arg[0,0,1]=(-0.438521968882)*x[0]+(1.6033775267)*x[1]+(0.691349995832)*x[2]
        arg[0,1,0]=(-0.04337513731)*x[0]+(0.142927668234)*x[1]+(0.506581723199)*x[2]
        arg[0,1,1]=(-0.120598868162)*x[0]+(-0.213666787813)*x[1]+(-1.04221679332)*x[2]
        arg[1,0,0]=(0.153696490712)*x[0]+(-0.122516529815)*x[1]+(-1.08837272712)*x[2]
        arg[1,0,1]=(0.410706464237)*x[0]+(0.720079363315)*x[1]+(-0.767790357539)*x[2]
        arg[1,1,0]=(-1.03726428173)*x[0]+(-0.316809853003)*x[1]+(-0.780405868082)*x[2]
        arg[1,1,1]=(-0.427547967216)*x[0]+(-0.897766643609)*x[1]+(-1.56913552404)*x[2]
        arg[2,0,0]=(-0.282789248898)*x[0]+(-0.110846936031)*x[1]+(-1.4329391906)*x[2]
        arg[2,0,1]=(1.46866588617)*x[0]+(-1.23022501959)*x[1]+(-0.3123390018)*x[2]
        arg[2,1,0]=(-0.314115718621)*x[0]+(1.7333934438)*x[1]+(0.833572700726)*x[2]
        arg[2,1,1]=(0.464246462638)*x[0]+(0.267484068646)*x[1]+(0.342009204787)*x[2]
        arg[3,0,0]=(0.578853099163)*x[0]+(-0.0453855508465)*x[1]+(-0.0751578475516)*x[2]
        arg[3,0,1]=(-0.526734485752)*x[0]+(-1.03286310016)*x[1]+(-0.398986583917)*x[2]
        arg[3,1,0]=(0.0239088277798)*x[0]+(0.205105470192)*x[1]+(-1.04986349166)*x[2]
        arg[3,1,1]=(0.130893744001)*x[0]+(-0.273702140669)*x[1]+(-0.0804222415128)*x[2]
        arg[4,0,0]=(-0.822661618584)*x[0]+(-0.267189204519)*x[1]+(-0.290013766971)*x[2]
        arg[4,0,1]=(-1.46759823734)*x[0]+(0.317465945316)*x[1]+(0.108284317752)*x[2]
        arg[4,1,0]=(-1.3920990386)*x[0]+(-1.17823778694)*x[1]+(-0.18366434307)*x[2]
        arg[4,1,1]=(-0.0915496209516)*x[0]+(-0.367624609832)*x[1]+(1.30630257323)*x[2]
        arg[5,0,0]=(0.117840830455)*x[0]+(-0.667875433194)*x[1]+(-1.59716688455)*x[2]
        arg[5,0,1]=(-0.818693227605)*x[0]+(0.483842075248)*x[1]+(-0.408373205535)*x[2]
        arg[5,1,0]=(1.36906770301)*x[0]+(0.494926179999)*x[1]+(0.717041707262)*x[2]
        arg[5,1,1]=(0.819757116081)*x[0]+(0.0982458749015)*x[1]+(0.275080391599)*x[2]
        ref[0,0,0]=(0.236560699949)/(o+1.)+(-0.392784388084)+(-0.327323511722)*0.5**o
        ref[0,0,1]=(1.32491522045)/(o+1.)+(0.525146863711)+(-0.519003394222)*0.5**o
        ref[0,1,0]=(0.433749865563)/(o+1.)+(0.23007126008)+(-0.287758131599)*0.5**o
        ref[0,1,1]=(-0.0563021060686)/(o+1.)+(-0.403921160703)+(-0.512338021825)*0.5**o
        ref[1,0,0]=(-0.170496965963)/(o+1.)+(-0.914421777259)+(0.942147754258)*0.5**o
        ref[1,0,1]=(-0.372707483726)/(o+1.)+(0.636041158872)+(-0.536379364005)*0.5**o
        ref[1,1,0]=(-0.211529878364)/(o+1.)+(-0.912151985776)+(-0.0986461529033)*0.5**o
        ref[1,1,1]=(-1.32533998437)/(o+1.)+(-0.685030198386)+(-0.19904975372)*0.5**o
        ref[2,0,0]=(0.152048902469)/(o+1.)+(-0.88880590209)+(-0.201012473813)*0.5**o
        ref[2,0,1]=(-0.748900052246)/(o+1.)+(-0.0226120471998)+(0.720226011425)*0.5**o
        ref[2,1,0]=(0.77797707941)/(o+1.)+(1.01635055442)+(-0.557827762351)*0.5**o
        ref[2,1,1]=(1.08969334716)/(o+1.)+(-0.404644728844)+(0.793335846595)*0.5**o
        ref[3,0,0]=(1.56498448669)/(o+1.)+(-0.699967188685)+(0.293259591449)*0.5**o
        ref[3,0,1]=(-0.620133427128)/(o+1.)+(-0.550037413688)+(-0.238375915329)*0.5**o
        ref[3,1,0]=(-0.156380586288)/(o+1.)+(-0.0154369973565)+(-0.633594612687)*0.5**o
        ref[3,1,1]=(-0.352989559326)/(o+1.)+(-0.290547031627)+(0.710852984398)*0.5**o
        ref[4,0,0]=(-0.147579361355)/(o+1.)+(-0.615173424765)+(-0.00193837918766)*0.5**o
        ref[4,0,1]=(0.598152345277)/(o+1.)+(-0.540248606426)+(-0.559503106696)*0.5**o
        ref[4,1,0]=(-1.17182448825)/(o+1.)+(-0.590551681576)+(-0.401073317208)*0.5**o
        ref[4,1,1]=(0.837504141173)/(o+1.)+(0.0807414267186)+(-0.15185865216)*0.5**o
        ref[5,0,0]=(-0.646601115533)/(o+1.)+(-1.20895265273)+(0.9173049337)*0.5**o
        ref[5,0,1]=(1.26626110052)/(o+1.)+(-0.56471923973)+(-0.880046978955)*0.5**o
        ref[5,1,0]=(-0.069063169298)/(o+1.)+(0.883536796964)+(0.883025165644)*0.5**o
        ref[5,1,1]=(1.05999338993)/(o+1.)+(-0.392519102893)+(0.918128198439)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactOne_fromData_ReducedSolution_rank4(self):
      """
      tests integral of rank 4 Data on the FunctionOnContactOne

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports integral on FunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5, 3, 2),w)
      ref=numpy.zeros((4, 5, 3, 2),numpy.float_)
      if dim==2:
        arg[0,0,0,0]=(0.257898226293)*x[0]+(-0.01230083314)*x[1]
        arg[0,0,0,1]=(0.0531183070647)*x[0]+(-0.170192287836)*x[1]
        arg[0,0,1,0]=(-0.364644940864)*x[0]+(1.04227651446)*x[1]
        arg[0,0,1,1]=(-0.894334108203)*x[0]+(0.151390331211)*x[1]
        arg[0,0,2,0]=(1.03825401708)*x[0]+(-0.882136097149)*x[1]
        arg[0,0,2,1]=(-1.05304910155)*x[0]+(1.06866311708)*x[1]
        arg[0,1,0,0]=(1.54742542159)*x[0]+(0.941778348482)*x[1]
        arg[0,1,0,1]=(1.1797380956)*x[0]+(-0.296417138234)*x[1]
        arg[0,1,1,0]=(-1.09196028027)*x[0]+(-0.783377624006)*x[1]
        arg[0,1,1,1]=(0.370952191443)*x[0]+(-0.0854141858991)*x[1]
        arg[0,1,2,0]=(-1.23552368139)*x[0]+(1.23848088087)*x[1]
        arg[0,1,2,1]=(0.216345350114)*x[0]+(-0.136524468427)*x[1]
        arg[0,2,0,0]=(-1.21206901263)*x[0]+(0.804219582144)*x[1]
        arg[0,2,0,1]=(0.192956526208)*x[0]+(0.588417678858)*x[1]
        arg[0,2,1,0]=(-0.940175603742)*x[0]+(0.329047260784)*x[1]
        arg[0,2,1,1]=(-0.138597387792)*x[0]+(0.398298326223)*x[1]
        arg[0,2,2,0]=(0.10089766824)*x[0]+(1.25459493298)*x[1]
        arg[0,2,2,1]=(-0.401163525883)*x[0]+(0.276542765282)*x[1]
        arg[0,3,0,0]=(0.252156795268)*x[0]+(-0.50223063539)*x[1]
        arg[0,3,0,1]=(0.466244610905)*x[0]+(-0.587025224301)*x[1]
        arg[0,3,1,0]=(-0.268082012713)*x[0]+(-1.28874675764)*x[1]
        arg[0,3,1,1]=(1.10867463519)*x[0]+(-0.165500912864)*x[1]
        arg[0,3,2,0]=(-1.94681908266)*x[0]+(0.166854108476)*x[1]
        arg[0,3,2,1]=(-0.537150417922)*x[0]+(0.0872851457131)*x[1]
        arg[0,4,0,0]=(-0.376548581807)*x[0]+(-0.245201133388)*x[1]
        arg[0,4,0,1]=(-0.135287690767)*x[0]+(0.750376238942)*x[1]
        arg[0,4,1,0]=(-0.0240491596907)*x[0]+(-1.21337088424)*x[1]
        arg[0,4,1,1]=(0.699215733397)*x[0]+(0.0596356032853)*x[1]
        arg[0,4,2,0]=(-0.521316010702)*x[0]+(-1.75309802615)*x[1]
        arg[0,4,2,1]=(0.403640172305)*x[0]+(1.41465827077)*x[1]
        arg[1,0,0,0]=(-1.01538089315)*x[0]+(-1.14785578909)*x[1]
        arg[1,0,0,1]=(1.10045201832)*x[0]+(0.630761728795)*x[1]
        arg[1,0,1,0]=(0.137673646147)*x[0]+(-0.387129090253)*x[1]
        arg[1,0,1,1]=(-1.04588287863)*x[0]+(-1.22433029859)*x[1]
        arg[1,0,2,0]=(-0.567830263535)*x[0]+(-0.405808616295)*x[1]
        arg[1,0,2,1]=(0.139326102495)*x[0]+(-1.93153384363)*x[1]
        arg[1,1,0,0]=(-0.638524762921)*x[0]+(0.506405751653)*x[1]
        arg[1,1,0,1]=(0.130775153144)*x[0]+(0.531317202208)*x[1]
        arg[1,1,1,0]=(0.557476206038)*x[0]+(0.447588676184)*x[1]
        arg[1,1,1,1]=(-0.538795916875)*x[0]+(-0.593869493531)*x[1]
        arg[1,1,2,0]=(-1.09827483942)*x[0]+(0.376431061876)*x[1]
        arg[1,1,2,1]=(-0.0074505620387)*x[0]+(-0.798645151486)*x[1]
        arg[1,2,0,0]=(0.251283911459)*x[0]+(0.194595071813)*x[1]
        arg[1,2,0,1]=(1.00482763799)*x[0]+(-1.00059863394)*x[1]
        arg[1,2,1,0]=(-0.453095434684)*x[0]+(-0.513922011418)*x[1]
        arg[1,2,1,1]=(0.558548837484)*x[0]+(0.0696328406825)*x[1]
        arg[1,2,2,0]=(-1.20625552545)*x[0]+(-0.151371710692)*x[1]
        arg[1,2,2,1]=(0.100262209656)*x[0]+(-0.314628804179)*x[1]
        arg[1,3,0,0]=(0.947063141882)*x[0]+(-0.297271200562)*x[1]
        arg[1,3,0,1]=(0.0760277251261)*x[0]+(-1.17982146387)*x[1]
        arg[1,3,1,0]=(-1.46946063969)*x[0]+(-0.615302663567)*x[1]
        arg[1,3,1,1]=(0.419747293729)*x[0]+(-0.629842531949)*x[1]
        arg[1,3,2,0]=(-1.44607954917)*x[0]+(0.520024093924)*x[1]
        arg[1,3,2,1]=(-1.01556392596)*x[0]+(-0.307173811354)*x[1]
        arg[1,4,0,0]=(-0.164339335244)*x[0]+(0.390736414982)*x[1]
        arg[1,4,0,1]=(-0.266932380288)*x[0]+(-0.731533848078)*x[1]
        arg[1,4,1,0]=(-0.903825489055)*x[0]+(-1.81564784421)*x[1]
        arg[1,4,1,1]=(-0.426008882665)*x[0]+(-1.09792948908)*x[1]
        arg[1,4,2,0]=(-1.32984927589)*x[0]+(1.58196302541)*x[1]
        arg[1,4,2,1]=(-0.0581032806145)*x[0]+(-0.323073355302)*x[1]
        arg[2,0,0,0]=(-0.744014251373)*x[0]+(-0.818253254872)*x[1]
        arg[2,0,0,1]=(0.368656965396)*x[0]+(-0.0864646380237)*x[1]
        arg[2,0,1,0]=(-0.112428845773)*x[0]+(0.109170104421)*x[1]
        arg[2,0,1,1]=(0.0682241548911)*x[0]+(-1.65999482283)*x[1]
        arg[2,0,2,0]=(0.212777058993)*x[0]+(0.127163480945)*x[1]
        arg[2,0,2,1]=(-0.256989443356)*x[0]+(0.183206687414)*x[1]
        arg[2,1,0,0]=(0.578997994373)*x[0]+(0.413087453689)*x[1]
        arg[2,1,0,1]=(0.886532090807)*x[0]+(0.330645525532)*x[1]
        arg[2,1,1,0]=(0.18224742814)*x[0]+(-0.713166591262)*x[1]
        arg[2,1,1,1]=(-0.25218296706)*x[0]+(0.736797897014)*x[1]
        arg[2,1,2,0]=(0.885270371408)*x[0]+(-1.35641397127)*x[1]
        arg[2,1,2,1]=(-0.963214290879)*x[0]+(0.348421598509)*x[1]
        arg[2,2,0,0]=(0.884120745351)*x[0]+(0.956857507884)*x[1]
        arg[2,2,0,1]=(-0.00795298517561)*x[0]+(0.51243740322)*x[1]
        arg[2,2,1,0]=(-0.563655252975)*x[0]+(-0.716305131721)*x[1]
        arg[2,2,1,1]=(0.648243491623)*x[0]+(-0.142239786485)*x[1]
        arg[2,2,2,0]=(1.50677848542)*x[0]+(-0.359722006818)*x[1]
        arg[2,2,2,1]=(-0.522592086302)*x[0]+(-0.18544927497)*x[1]
        arg[2,3,0,0]=(0.0822787762966)*x[0]+(0.196309662733)*x[1]
        arg[2,3,0,1]=(0.232573258627)*x[0]+(-0.0597976775432)*x[1]
        arg[2,3,1,0]=(1.50895737052)*x[0]+(0.546625934312)*x[1]
        arg[2,3,1,1]=(-1.64023373286)*x[0]+(0.220053261777)*x[1]
        arg[2,3,2,0]=(1.5897208842)*x[0]+(-0.0355885594992)*x[1]
        arg[2,3,2,1]=(-1.52822349982)*x[0]+(-0.0906934717656)*x[1]
        arg[2,4,0,0]=(0.496696582601)*x[0]+(-0.303888675059)*x[1]
        arg[2,4,0,1]=(-0.0819586112177)*x[0]+(-0.0449047294378)*x[1]
        arg[2,4,1,0]=(-0.678530758513)*x[0]+(-1.2173963447)*x[1]
        arg[2,4,1,1]=(-0.645450475268)*x[0]+(0.356771113426)*x[1]
        arg[2,4,2,0]=(-0.486643021867)*x[0]+(0.977763869689)*x[1]
        arg[2,4,2,1]=(-0.458991807397)*x[0]+(1.80038432923)*x[1]
        arg[3,0,0,0]=(0.270795948755)*x[0]+(-0.691512096338)*x[1]
        arg[3,0,0,1]=(-0.756717130099)*x[0]+(1.16189640534)*x[1]
        arg[3,0,1,0]=(0.967554346699)*x[0]+(-0.272119008963)*x[1]
        arg[3,0,1,1]=(1.14296491195)*x[0]+(-0.707175662379)*x[1]
        arg[3,0,2,0]=(1.15665030692)*x[0]+(0.414803389409)*x[1]
        arg[3,0,2,1]=(1.38997274955)*x[0]+(1.07065568192)*x[1]
        arg[3,1,0,0]=(-0.623998925511)*x[0]+(0.375545806421)*x[1]
        arg[3,1,0,1]=(-0.363431948398)*x[0]+(-0.183100417297)*x[1]
        arg[3,1,1,0]=(0.0435866301729)*x[0]+(-0.557137572068)*x[1]
        arg[3,1,1,1]=(-0.253225217468)*x[0]+(-0.0657932188329)*x[1]
        arg[3,1,2,0]=(0.147700921064)*x[0]+(0.681624563015)*x[1]
        arg[3,1,2,1]=(-0.075394297616)*x[0]+(0.271741245789)*x[1]
        arg[3,2,0,0]=(1.34803044812)*x[0]+(0.87991065609)*x[1]
        arg[3,2,0,1]=(0.0529487135353)*x[0]+(0.806480092352)*x[1]
        arg[3,2,1,0]=(-1.52038624166)*x[0]+(-0.992718138685)*x[1]
        arg[3,2,1,1]=(0.0859852188749)*x[0]+(1.63512596202)*x[1]
        arg[3,2,2,0]=(1.46925148575)*x[0]+(-1.60383564005)*x[1]
        arg[3,2,2,1]=(0.114463395473)*x[0]+(-1.0186377376)*x[1]
        arg[3,3,0,0]=(-1.02962502658)*x[0]+(-1.06788543146)*x[1]
        arg[3,3,0,1]=(-0.184391597652)*x[0]+(0.462603936483)*x[1]
        arg[3,3,1,0]=(0.611113335782)*x[0]+(0.282417397043)*x[1]
        arg[3,3,1,1]=(-0.193423873796)*x[0]+(-0.56108614964)*x[1]
        arg[3,3,2,0]=(-0.0290687598224)*x[0]+(-0.460221158417)*x[1]
        arg[3,3,2,1]=(0.784513522959)*x[0]+(-0.484813129206)*x[1]
        arg[3,4,0,0]=(0.423780748664)*x[0]+(-0.344020787894)*x[1]
        arg[3,4,0,1]=(1.10623024451)*x[0]+(-0.775008153301)*x[1]
        arg[3,4,1,0]=(0.554847534291)*x[0]+(-0.854749107999)*x[1]
        arg[3,4,1,1]=(-0.639595693597)*x[0]+(-0.966152813212)*x[1]
        arg[3,4,2,0]=(-0.0172621996023)*x[0]+(0.321815074269)*x[1]
        arg[3,4,2,1]=(0.39605156824)*x[0]+(-0.499138231749)*x[1]
        ref[0,0,0,0]=(0.169892838337)/(o+1.)+(-0.323261281472)+(0.722227117759)*0.5**o
        ref[0,0,0,1]=(-0.899797330597)/(o+1.)+(0.731242549464)+(-0.679761749104)*0.5**o
        ref[0,0,1,0]=(0.948969615551)/(o+1.)+(-0.173478478707)+(0.0756189154575)*0.5**o
        ref[0,0,1,1]=(-0.484419525942)/(o+1.)+(0.287761319651)+(-0.834046890353)*0.5**o
        ref[0,0,2,0]=(-0.275366085384)/(o+1.)+(-0.051636507926)+(0.534757021171)*0.5**o
        ref[0,0,2,1]=(0.338862631582)/(o+1.)+(0.076489529483)+(-0.47622767501)*0.5**o
        ref[0,1,0,0]=(0.960885993642)/(o+1.)+(0.36959166751)+(0.789134441414)*0.5**o
        ref[0,1,0,1]=(-0.101908900511)/(o+1.)+(0.343409131566)+(0.298411594744)*0.5**o
        ref[0,1,1,0]=(-0.50227382988)/(o+1.)+(-0.431549057432)+(-0.509965959533)*0.5**o
        ref[0,1,1,1]=(-0.294216689224)/(o+1.)+(0.293114907564)+(-0.00647512035965)*0.5**o
        ref[0,1,2,0]=(0.330313139948)/(o+1.)+(0.0601559902451)+(-0.447667920962)*0.5**o
        ref[0,1,2,1]=(-0.855325117375)/(o+1.)+(0.455706690618)+(0.0237326178252)*0.5**o
        ref[0,2,0,0]=(0.958948557218)/(o+1.)+(-0.307015973897)+(-0.752766039908)*0.5**o
        ref[0,2,0,1]=(0.622666086232)/(o+1.)+(0.417782845058)+(-0.676857571283)*0.5**o
        ref[0,2,1,0]=(-0.476269525636)/(o+1.)+(0.203518519787)+(-0.541895856898)*0.5**o
        ref[0,2,1,1]=(0.396862410812)/(o+1.)+(-0.318442063423)+(0.499722654464)*0.5**o
        ref[0,2,2,0]=(0.291546173382)/(o+1.)+(0.345688404218)+(0.372569619405)*0.5**o
        ref[0,2,2,1]=(-0.0863432016899)/(o+1.)+(0.296494638577)+(-0.631266836064)*0.5**o
        ref[0,3,0,0]=(0.437179077921)/(o+1.)+(-0.340515851088)+(-0.00622121586734)*0.5**o
        ref[0,3,0,1]=(-0.0670787919003)/(o+1.)+(-0.492735838895)+(0.931769856293)*0.5**o
        ref[0,3,1,0]=(-0.455620785864)/(o+1.)+(-0.866886545624)+(0.632565106759)*0.5**o
        ref[0,3,1,1]=(0.769395985412)/(o+1.)+(-0.303642587154)+(0.781062911223)*0.5**o
        ref[0,3,2,0]=(-0.338160377272)/(o+1.)+(-0.237545833658)+(-0.966712929597)*0.5**o
        ref[0,3,2,1]=(-0.796435534783)/(o+1.)+(0.544922318125)+(-0.743274373676)*0.5**o
        ref[0,4,0,0]=(0.452932120968)/(o+1.)+(-0.512874769603)+(-0.048932296957)*0.5**o
        ref[0,4,0,1]=(0.361579034112)/(o+1.)+(-0.0561290568189)+(0.365767627701)*0.5**o
        ref[0,4,1,0]=(-0.626053161593)/(o+1.)+(-0.761324946295)+(0.911283010254)*0.5**o
        ref[0,4,1,1]=(-0.0455977379835)/(o+1.)+(0.292122388548)+(0.22020429757)*0.5**o
        ref[0,4,2,0]=(-0.881052488675)/(o+1.)+(-0.210171400955)+(-0.973018746263)*0.5**o
        ref[0,4,2,1]=(0.955475719603)/(o+1.)+(0.367959093676)+(0.126904536122)*0.5**o
        ref[1,0,0,0]=(-0.314523133291)/(o+1.)+(-0.649561146707)+(-0.54959125554)*0.5**o
        ref[1,0,0,1]=(0.776726613012)/(o+1.)+(0.0747896120381)+(0.804907910029)*0.5**o
        ref[1,0,1,0]=(0.416309554773)/(o+1.)+(0.0650606132082)+(-0.795886225295)*0.5**o
        ref[1,0,1,1]=(-0.642083920221)/(o+1.)+(-0.518812997818)+(-0.590503261366)*0.5**o
        ref[1,0,2,0]=(0.209400082886)/(o+1.)+(-0.592256267485)+(0.00147357225359)*0.5**o
        ref[1,0,2,1]=(-0.969945221754)/(o+1.)+(-0.364559871358)+(-0.0931427766681)*0.5**o
        ref[1,1,0,0]=(0.941138659497)/(o+1.)+(-0.103485523138)+(-0.866286624489)*0.5**o
        ref[1,1,0,1]=(0.527395399401)/(o+1.)+(0.373358049173)+(-0.612019142395)*0.5**o
        ref[1,1,1,0]=(-0.276942103917)/(o+1.)+(0.624509599487)+(0.0329877871648)*0.5**o
        ref[1,1,1,1]=(-0.153477829221)/(o+1.)+(-0.0616268433725)+(-0.85593389444)*0.5**o
        ref[1,1,2,0]=(0.2685285297)/(o+1.)+(-0.301824551915)+(-0.386723203413)*0.5**o
        ref[1,1,2,1]=(-0.530960074652)/(o+1.)+(0.248621312801)+(-0.772378264474)*0.5**o
        ref[1,2,0,0]=(0.745071931955)/(o+1.)+(-0.149268301427)+(-0.000656345828701)*0.5**o
        ref[1,2,0,1]=(-0.581822428087)/(o+1.)+(0.2315691847)+(0.122913062738)*0.5**o
        ref[1,2,1,0]=(-0.325165099021)/(o+1.)+(-0.414758139139)+(0.187663931198)*0.5**o
        ref[1,2,1,1]=(-0.389683593964)/(o+1.)+(0.265985790556)+(0.485893691018)*0.5**o
        ref[1,2,2,0]=(0.661969003098)/(o+1.)+(-0.514584521701)+(-0.990427195841)*0.5**o
        ref[1,2,2,1]=(-0.0828384828188)/(o+1.)+(-0.482166570368)+(0.832805029032)*0.5**o
        ref[1,3,0,0]=(-0.756020671713)/(o+1.)+(0.627704058228)+(0.150404496577)*0.5**o
        ref[1,3,0,1]=(-0.93648010541)/(o+1.)+(0.11022703478)+(-0.387767702896)*0.5**o
        ref[1,3,1,0]=(-0.995823513997)/(o+1.)+(-0.242072805569)+(-0.604794178119)*0.5**o
        ref[1,3,1,1]=(-0.543071878541)/(o+1.)+(-0.160890620667)+(0.654757881656)*0.5**o
        ref[1,3,2,0]=(0.803184441579)/(o+1.)+(-0.612293875048)+(-0.504652146723)*0.5**o
        ref[1,3,2,1]=(-0.786714439322)/(o+1.)+(-0.175556007517)+(-0.18491128296)*0.5**o
        ref[1,4,0,0]=(-0.302928744298)/(o+1.)+(0.476360987754)+(-0.423396151472)*0.5**o
        ref[1,4,0,1]=(-0.410240602476)/(o+1.)+(-0.49551630458)+(0.40280698327)*0.5**o
        ref[1,4,1,0]=(-0.820677400346)/(o+1.)+(-0.573037933363)+(-0.752720066188)*0.5**o
        ref[1,4,1,1]=(-0.492748923658)/(o+1.)+(-0.600739806799)+(0.170290165507)*0.5**o
        ref[1,4,2,0]=(0.926654770824)/(o+1.)+(0.011282908181)+(-0.697106837662)*0.5**o
        ref[1,4,2,1]=(-0.958477382611)/(o+1.)+(0.618515858902)+(-0.65973097111)*0.5**o
        ref[2,0,0,0]=(-0.15636950624)/(o+1.)+(-0.520670547262)+(-0.36455690548)*0.5**o
        ref[2,0,0,1]=(-0.726424878286)/(o+1.)+(0.697757734621)+(-0.386898263585)*0.5**o
        ref[2,0,1,0]=(-0.055162393508)/(o+1.)+(0.247015646426)+(-0.442127640696)*0.5**o
        ref[2,0,1,1]=(-0.769332288875)/(o+1.)+(-0.0917175733456)+(-0.639003232373)*0.5**o
        ref[2,0,2,0]=(-0.196390038297)/(o+1.)+(0.0977907270548)+(0.340749124126)*0.5**o
        ref[2,0,2,1]=(0.841927481859)/(o+1.)+(-0.489185426146)+(0.0626606144906)*0.5**o
        ref[2,1,0,0]=(0.590018249148)/(o+1.)+(0.378476967182)+(-0.354886735451)*0.5**o
        ref[2,1,0,1]=(0.178495604742)/(o+1.)+(0.233073081069)+(0.572535849459)*0.5**o
        ref[2,1,1,0]=(-0.762671064919)/(o+1.)+(-0.108539686142)+(0.448831274081)*0.5**o
        ref[2,1,1,1]=(0.927591406535)/(o+1.)+(0.0441635703243)+(-0.53130361723)*0.5**o
        ref[2,1,2,0]=(-0.604360706259)/(o+1.)+(-0.134848102332)+(0.402913311063)*0.5**o
        ref[2,1,2,1]=(0.229727627441)/(o+1.)+(-0.323296604293)+(-0.197927111225)*0.5**o
        ref[2,2,0,0]=(0.287478356112)/(o+1.)+(0.762635506201)+(0.0282288847209)*0.5**o
        ref[2,2,0,1]=(0.243518182621)/(o+1.)+(-0.241502069199)+(0.743970373822)*0.5**o
        ref[2,2,1,0]=(-0.369049760378)/(o+1.)+(-0.21941778965)+(-0.472075045017)*0.5**o
        ref[2,2,1,1]=(-0.745343658666)/(o+1.)+(0.193375557775)+(0.864596248255)*0.5**o
        ref[2,2,2,0]=(-0.491769251494)/(o+1.)+(0.442742387618)+(0.753340954862)*0.5**o
        ref[2,2,2,1]=(0.551196377158)/(o+1.)+(-0.523535406613)+(-0.212166925203)*0.5**o
        ref[2,3,0,0]=(0.192271522168)/(o+1.)+(-0.0181953944687)+(0.122707705798)*0.5**o
        ref[2,3,0,1]=(-0.593768738594)/(o+1.)+(0.199088910545)+(0.368366498588)*0.5**o
        ref[2,3,1,0]=(0.0866211808983)/(o+1.)+(0.723512694127)+(0.521936735685)*0.5**o
        ref[2,3,1,1]=(-0.453288268254)/(o+1.)+(-0.15580970281)+(-0.655272797205)*0.5**o
        ref[2,3,2,0]=(0.614105802669)/(o+1.)+(0.0411427694898)+(0.85774098305)*0.5**o
        ref[2,3,2,1]=(0.263133533654)/(o+1.)+(-0.628776355867)+(-0.624497793507)*0.5**o
        ref[2,4,0,0]=(-0.809347914722)/(o+1.)+(0.367719412237)+(0.26671699779)*0.5**o
        ref[2,4,0,1]=(0.773124137566)/(o+1.)+(-0.442496395542)+(-0.0149946871367)*0.5**o
        ref[2,4,1,0]=(-0.927311525126)/(o+1.)+(-0.0125177466568)+(-0.943580084777)*0.5**o
        ref[2,4,1,1]=(0.306409665483)/(o+1.)+(0.128569553604)+(-0.852228134533)*0.5**o
        ref[2,4,2,0]=(0.926185522145)/(o+1.)+(0.0750538146738)+(-0.58517230367)*0.5**o
        ref[2,4,2,1]=(0.858245470922)/(o+1.)+(0.0125876600893)+(0.457971730734)*0.5**o
        ref[3,0,0,0]=(-0.74501947953)/(o+1.)+(0.343942243752)+(-0.363581155556)*0.5**o
        ref[3,0,0,1]=(0.850890737948)/(o+1.)+(0.157997002133)+(-0.761705466968)*0.5**o
        ref[3,0,1,0]=(-0.0468962248591)/(o+1.)+(0.0297980532763)+(0.682735456043)*0.5**o
        ref[3,0,1,1]=(0.0126326425456)/(o+1.)+(-0.076115296096)+(0.575387199213)*0.5**o
        ref[3,0,2,0]=(-0.410549752333)/(o+1.)+(0.85556811978)+(0.270867209104)*0.5**o
        ref[3,0,2,1]=(0.580487848545)/(o+1.)+(0.59069171729)+(0.698757148346)*0.5**o
        ref[3,1,0,0]=(0.139052775786)/(o+1.)+(-0.334747475889)+(0.281989056901)*0.5**o
        ref[3,1,0,1]=(0.591083610998)/(o+1.)+(-0.322680889668)+(-0.492254197357)*0.5**o
        ref[3,1,1,0]=(0.384713965427)/(o+1.)+(-0.522411606102)+(0.146558304882)*0.5**o
        ref[3,1,1,1]=(-0.827615582009)/(o+1.)+(-0.00295218575854)+(0.514501517225)*0.5**o
        ref[3,1,2,0]=(0.84762578765)/(o+1.)+(0.208361555685)+(-0.43502341494)*0.5**o
        ref[3,1,2,1]=(-0.554668615664)/(o+1.)+(0.825347835483)+(-0.899680107129)*0.5**o
        ref[3,2,0,0]=(0.652802712143)/(o+1.)+(0.397197582731)+(0.7807432266)*0.5**o
        ref[3,2,0,1]=(0.421495251937)/(o+1.)+(-0.119571710512)+(0.677076974975)*0.5**o
        ref[3,2,1,0]=(-0.230439630375)/(o+1.)+(-0.733165339017)+(-0.816334071941)*0.5**o
        ref[3,2,1,1]=(0.64153308098)/(o+1.)+(0.778374176432)+(-0.477170252948)*0.5**o
        ref[3,2,2,0]=(-0.987848717403)/(o+1.)+(-0.0606435888479)+(0.974551740798)*0.5**o
        ref[3,2,2,1]=(-0.318593711859)/(o+1.)+(-0.459904454409)+(0.334228278552)*0.5**o
        ref[3,3,0,0]=(-0.418115413351)/(o+1.)+(-0.348616063735)+(-0.982162917221)*0.5**o
        ref[3,3,0,1]=(-0.437684435015)/(o+1.)+(0.804145379606)+(-0.892393985366)*0.5**o
        ref[3,3,1,0]=(0.944100110135)/(o+1.)+(-0.059994562245)+(0.0694197471801)*0.5**o
        ref[3,3,1,1]=(-0.860656048613)/(o+1.)+(0.178136109535)+(-0.250126193893)*0.5**o
        ref[3,3,2,0]=(-0.363631991908)/(o+1.)+(-0.298384287282)+(0.471110648232)*0.5**o
        ref[3,3,2,1]=(-0.578021736479)/(o+1.)+(0.320913094365)+(0.235895941501)*0.5**o
        ref[3,4,0,0]=(-0.946995649946)/(o+1.)+(0.0451978887455)+(0.936359833225)*0.5**o
        ref[3,4,0,1]=(-0.636305818291)/(o+1.)+(0.258932088634)+(0.449663732234)*0.5**o
        ref[3,4,1,0]=(-0.823421937366)/(o+1.)+(0.112980674356)+(0.297559014947)*0.5**o
        ref[3,4,1,1]=(-0.441768022626)/(o+1.)+(-0.240840962617)+(-0.682298558949)*0.5**o
        ref[3,4,2,0]=(-0.258808936822)/(o+1.)+(0.510306577882)+(-0.457251344276)*0.5**o
        ref[3,4,2,1]=(-0.489365749016)/(o+1.)+(-0.296617385332)+(0.979513856171)*0.5**o
      else:
        arg[0,0,0,0]=(-0.474247989728)*x[0]+(-0.768689352235)*x[1]+(-1.08527900853)*x[2]
        arg[0,0,0,1]=(-0.237113797789)*x[0]+(0.0842735741018)*x[1]+(0.203858551029)*x[2]
        arg[0,0,1,0]=(0.24237127665)*x[0]+(0.421022955803)*x[1]+(-1.52818540927)*x[2]
        arg[0,0,1,1]=(1.34624865817)*x[0]+(-0.798943272818)*x[1]+(-0.823168386315)*x[2]
        arg[0,0,2,0]=(-0.906723771267)*x[0]+(0.328893123771)*x[1]+(-0.942770202463)*x[2]
        arg[0,0,2,1]=(1.54305197694)*x[0]+(0.813063941984)*x[1]+(0.205532727168)*x[2]
        arg[0,1,0,0]=(0.399159979872)*x[0]+(-1.52089837934)*x[1]+(-1.4064251406)*x[2]
        arg[0,1,0,1]=(-0.121139354731)*x[0]+(1.12768246422)*x[1]+(0.234299409411)*x[2]
        arg[0,1,1,0]=(-1.62513899388)*x[0]+(-0.173131700583)*x[1]+(1.24199996799)*x[2]
        arg[0,1,1,1]=(-0.371929434177)*x[0]+(-0.606278610409)*x[1]+(-0.00600790063066)*x[2]
        arg[0,1,2,0]=(-0.804531294423)*x[0]+(0.869392248002)*x[1]+(0.840417359943)*x[2]
        arg[0,1,2,1]=(0.329452946275)*x[0]+(1.1072495446)*x[1]+(-0.213517932323)*x[2]
        arg[0,2,0,0]=(1.02784887224)*x[0]+(0.435517042337)*x[1]+(1.58767206677)*x[2]
        arg[0,2,0,1]=(-0.0120997177)*x[0]+(0.430135803314)*x[1]+(-0.717473068956)*x[2]
        arg[0,2,1,0]=(0.955455517302)*x[0]+(0.712976918968)*x[1]+(-0.624804508641)*x[2]
        arg[0,2,1,1]=(-1.03293986594)*x[0]+(-0.539062834482)*x[1]+(-1.61172701428)*x[2]
        arg[0,2,2,0]=(-0.177662810138)*x[0]+(0.553352366127)*x[1]+(0.695417121111)*x[2]
        arg[0,2,2,1]=(0.318133474679)*x[0]+(-0.452528947926)*x[1]+(0.468602505091)*x[2]
        arg[0,3,0,0]=(-0.294467725916)*x[0]+(1.67314409894)*x[1]+(0.100736440007)*x[2]
        arg[0,3,0,1]=(-0.517162349887)*x[0]+(-1.57430191254)*x[1]+(1.24045877402)*x[2]
        arg[0,3,1,0]=(0.667393640368)*x[0]+(0.340993632212)*x[1]+(-1.09920535559)*x[2]
        arg[0,3,1,1]=(0.37451406727)*x[0]+(0.37350649326)*x[1]+(0.357440871322)*x[2]
        arg[0,3,2,0]=(-0.989366398194)*x[0]+(0.596179154759)*x[1]+(-0.573280308087)*x[2]
        arg[0,3,2,1]=(-0.25561190141)*x[0]+(1.42296905238)*x[1]+(-1.21032031399)*x[2]
        arg[0,4,0,0]=(-0.41394085285)*x[0]+(-0.751213929646)*x[1]+(-0.68470718293)*x[2]
        arg[0,4,0,1]=(0.702985204167)*x[0]+(-0.0375821582587)*x[1]+(0.290630803819)*x[2]
        arg[0,4,1,0]=(0.0558204653441)*x[0]+(1.61940057302)*x[1]+(-0.518863528354)*x[2]
        arg[0,4,1,1]=(1.25876303604)*x[0]+(-1.40923202908)*x[1]+(1.05202333762)*x[2]
        arg[0,4,2,0]=(0.0715295483294)*x[0]+(0.883869702529)*x[1]+(-0.463253474408)*x[2]
        arg[0,4,2,1]=(0.564563789615)*x[0]+(-0.135049742988)*x[1]+(-1.3118878567)*x[2]
        arg[1,0,0,0]=(0.707267335291)*x[0]+(-0.197244449872)*x[1]+(-0.114362773781)*x[2]
        arg[1,0,0,1]=(-0.144762815659)*x[0]+(0.434177706452)*x[1]+(0.0754859099691)*x[2]
        arg[1,0,1,0]=(-0.906934991359)*x[0]+(0.179011995627)*x[1]+(0.823936036469)*x[2]
        arg[1,0,1,1]=(1.11165724605)*x[0]+(0.48023223731)*x[1]+(0.981353808181)*x[2]
        arg[1,0,2,0]=(0.499460583481)*x[0]+(0.171586243561)*x[1]+(-0.252474775657)*x[2]
        arg[1,0,2,1]=(-0.604289370884)*x[0]+(1.11610914086)*x[1]+(0.747439062711)*x[2]
        arg[1,1,0,0]=(0.696191463882)*x[0]+(0.349547714404)*x[1]+(0.651361019042)*x[2]
        arg[1,1,0,1]=(0.152679923426)*x[0]+(-0.448442911425)*x[1]+(1.33964371501)*x[2]
        arg[1,1,1,0]=(-0.74612592015)*x[0]+(-1.27089239786)*x[1]+(-0.261725631736)*x[2]
        arg[1,1,1,1]=(1.50839493929)*x[0]+(1.93229724987)*x[1]+(-0.963996845831)*x[2]
        arg[1,1,2,0]=(0.867984447375)*x[0]+(-0.777493811854)*x[1]+(0.847847553133)*x[2]
        arg[1,1,2,1]=(1.34249389498)*x[0]+(0.776160804062)*x[1]+(-0.295393972199)*x[2]
        arg[1,2,0,0]=(-0.655771613243)*x[0]+(1.58220214455)*x[1]+(-0.406292477629)*x[2]
        arg[1,2,0,1]=(0.59294895986)*x[0]+(0.544961164723)*x[1]+(0.634279670948)*x[2]
        arg[1,2,1,0]=(1.15324179862)*x[0]+(0.645764835217)*x[1]+(0.670364725309)*x[2]
        arg[1,2,1,1]=(-0.819459106271)*x[0]+(0.024815625404)*x[1]+(0.769753650676)*x[2]
        arg[1,2,2,0]=(1.19042366161)*x[0]+(0.982020548972)*x[1]+(0.92730609738)*x[2]
        arg[1,2,2,1]=(-0.742441488349)*x[0]+(-1.3101429812)*x[1]+(-0.198104246324)*x[2]
        arg[1,3,0,0]=(0.349205030437)*x[0]+(-0.745240339596)*x[1]+(-0.744742092259)*x[2]
        arg[1,3,0,1]=(0.993101296711)*x[0]+(-1.48331355036)*x[1]+(-1.00616778677)*x[2]
        arg[1,3,1,0]=(-1.10586442824)*x[0]+(1.71922449779)*x[1]+(-0.943451153259)*x[2]
        arg[1,3,1,1]=(-1.72393763035)*x[0]+(-1.14113495956)*x[1]+(-0.211909615809)*x[2]
        arg[1,3,2,0]=(-0.378302963108)*x[0]+(-0.494705322002)*x[1]+(0.940164363332)*x[2]
        arg[1,3,2,1]=(0.850541148743)*x[0]+(-1.45461945468)*x[1]+(1.20025821328)*x[2]
        arg[1,4,0,0]=(1.33882463795)*x[0]+(0.475208133119)*x[1]+(0.722686059253)*x[2]
        arg[1,4,0,1]=(0.0750422155969)*x[0]+(1.29758288845)*x[1]+(-1.41431668738)*x[2]
        arg[1,4,1,0]=(1.36072344575)*x[0]+(-0.345743547563)*x[1]+(-0.63248018754)*x[2]
        arg[1,4,1,1]=(0.579049229913)*x[0]+(0.954245074078)*x[1]+(-0.170538926965)*x[2]
        arg[1,4,2,0]=(1.67615484634)*x[0]+(0.561067019023)*x[1]+(-1.26952326914)*x[2]
        arg[1,4,2,1]=(0.700910547934)*x[0]+(0.58717328764)*x[1]+(0.795138196965)*x[2]
        arg[2,0,0,0]=(0.763193592359)*x[0]+(-0.397549328381)*x[1]+(-0.230593559545)*x[2]
        arg[2,0,0,1]=(0.454026153468)*x[0]+(-1.12464443717)*x[1]+(-0.147462098246)*x[2]
        arg[2,0,1,0]=(-0.276457034935)*x[0]+(0.118173156654)*x[1]+(0.57296366275)*x[2]
        arg[2,0,1,1]=(1.31887812253)*x[0]+(-0.544644483254)*x[1]+(0.0013728350582)*x[2]
        arg[2,0,2,0]=(-0.257589384265)*x[0]+(-0.016005453203)*x[1]+(-0.508836430965)*x[2]
        arg[2,0,2,1]=(-0.259420301773)*x[0]+(0.122645293168)*x[1]+(0.381613885289)*x[2]
        arg[2,1,0,0]=(-1.64168512418)*x[0]+(0.025182006192)*x[1]+(0.0268780458975)*x[2]
        arg[2,1,0,1]=(-0.0885344200898)*x[0]+(0.408795219114)*x[1]+(0.635798801606)*x[2]
        arg[2,1,1,0]=(-0.959658503283)*x[0]+(1.034415556)*x[1]+(0.725681105748)*x[2]
        arg[2,1,1,1]=(1.59312707789)*x[0]+(-0.116514952101)*x[1]+(-0.86121633507)*x[2]
        arg[2,1,2,0]=(-1.44859572067)*x[0]+(-1.28344247275)*x[1]+(0.299296401869)*x[2]
        arg[2,1,2,1]=(-0.488424372501)*x[0]+(1.23481402288)*x[1]+(0.01702902463)*x[2]
        arg[2,2,0,0]=(-1.29986951244)*x[0]+(-1.20190912751)*x[1]+(-0.755848291051)*x[2]
        arg[2,2,0,1]=(-0.52421656312)*x[0]+(-0.335471509689)*x[1]+(-1.01264312499)*x[2]
        arg[2,2,1,0]=(-1.12066642609)*x[0]+(-1.02138558053)*x[1]+(-1.00777755552)*x[2]
        arg[2,2,1,1]=(-1.42127024744)*x[0]+(1.63480022479)*x[1]+(0.542244906643)*x[2]
        arg[2,2,2,0]=(-0.418852880297)*x[0]+(-0.983110436453)*x[1]+(-0.622603673805)*x[2]
        arg[2,2,2,1]=(0.727366446825)*x[0]+(-0.256324856361)*x[1]+(-0.20608740758)*x[2]
        arg[2,3,0,0]=(0.548268302883)*x[0]+(0.0102173127665)*x[1]+(0.869833576217)*x[2]
        arg[2,3,0,1]=(-1.70159091061)*x[0]+(1.15876875083)*x[1]+(-0.234757706773)*x[2]
        arg[2,3,1,0]=(-1.72817024817)*x[0]+(0.690055132218)*x[1]+(-0.149398998825)*x[2]
        arg[2,3,1,1]=(1.80829412387)*x[0]+(-0.217899654855)*x[1]+(-0.922672217822)*x[2]
        arg[2,3,2,0]=(1.23855200328)*x[0]+(0.412795411508)*x[1]+(0.960476191755)*x[2]
        arg[2,3,2,1]=(-1.43061904621)*x[0]+(-0.766039000745)*x[1]+(-1.25765397361)*x[2]
        arg[2,4,0,0]=(-0.660872016687)*x[0]+(-0.000405233718708)*x[1]+(-0.178507968219)*x[2]
        arg[2,4,0,1]=(-0.168526882871)*x[0]+(-1.06936802685)*x[1]+(1.54083857104)*x[2]
        arg[2,4,1,0]=(0.565807359529)*x[0]+(-1.41401297939)*x[1]+(-1.23254260537)*x[2]
        arg[2,4,1,1]=(-0.373920075802)*x[0]+(0.407330812903)*x[1]+(-0.204667001328)*x[2]
        arg[2,4,2,0]=(0.874100879885)*x[0]+(-0.191238094267)*x[1]+(1.54890969328)*x[2]
        arg[2,4,2,1]=(0.566428125313)*x[0]+(-1.60899595601)*x[1]+(-1.05112529768)*x[2]
        arg[3,0,0,0]=(-0.688037248111)*x[0]+(-1.56483484784)*x[1]+(0.26782504619)*x[2]
        arg[3,0,0,1]=(0.418788954459)*x[0]+(-0.670455226092)*x[1]+(-0.0562760591375)*x[2]
        arg[3,0,1,0]=(-0.2614362754)*x[0]+(0.271050519855)*x[1]+(0.227811982923)*x[2]
        arg[3,0,1,1]=(-0.873198928647)*x[0]+(-0.483763141395)*x[1]+(0.33532383979)*x[2]
        arg[3,0,2,0]=(-0.864161938176)*x[0]+(-1.4682754386)*x[1]+(-0.38126411162)*x[2]
        arg[3,0,2,1]=(0.142827521381)*x[0]+(0.464483940188)*x[1]+(0.214297576606)*x[2]
        arg[3,1,0,0]=(0.188747079113)*x[0]+(0.590821253713)*x[1]+(0.908850367108)*x[2]
        arg[3,1,0,1]=(0.0587541582624)*x[0]+(0.955494266151)*x[1]+(1.70822209849)*x[2]
        arg[3,1,1,0]=(-0.420356861523)*x[0]+(-0.551445918262)*x[1]+(-0.0187114754269)*x[2]
        arg[3,1,1,1]=(1.07071658401)*x[0]+(-0.589035602194)*x[1]+(0.133725074027)*x[2]
        arg[3,1,2,0]=(0.370477043767)*x[0]+(0.226262353618)*x[1]+(0.566087784489)*x[2]
        arg[3,1,2,1]=(1.77828660075)*x[0]+(0.607419742685)*x[1]+(0.0297054744582)*x[2]
        arg[3,2,0,0]=(-1.19326939474)*x[0]+(1.20875153887)*x[1]+(-0.124756222091)*x[2]
        arg[3,2,0,1]=(0.0645190914271)*x[0]+(0.357580475716)*x[1]+(-1.59461469718)*x[2]
        arg[3,2,1,0]=(1.4853619905)*x[0]+(0.751582301648)*x[1]+(0.859363177167)*x[2]
        arg[3,2,1,1]=(1.36658655075)*x[0]+(-0.776650996613)*x[1]+(0.654743994261)*x[2]
        arg[3,2,2,0]=(0.406696005677)*x[0]+(-0.528546876384)*x[1]+(-0.439775325396)*x[2]
        arg[3,2,2,1]=(-0.725198159461)*x[0]+(-0.0933136564416)*x[1]+(-0.720365141393)*x[2]
        arg[3,3,0,0]=(-0.428669112669)*x[0]+(0.626162511175)*x[1]+(-0.382776233923)*x[2]
        arg[3,3,0,1]=(0.340474939218)*x[0]+(-0.140550383925)*x[1]+(0.353836983746)*x[2]
        arg[3,3,1,0]=(0.851646816711)*x[0]+(0.0171684195949)*x[1]+(0.38173044322)*x[2]
        arg[3,3,1,1]=(0.359647722225)*x[0]+(0.425799941165)*x[1]+(-0.80174052212)*x[2]
        arg[3,3,2,0]=(0.13012568736)*x[0]+(-0.573735693651)*x[1]+(1.08843730088)*x[2]
        arg[3,3,2,1]=(-1.01372953294)*x[0]+(0.335282956856)*x[1]+(1.24570512628)*x[2]
        arg[3,4,0,0]=(-0.05674609039)*x[0]+(-0.814879575064)*x[1]+(-0.169789408264)*x[2]
        arg[3,4,0,1]=(1.5968230938)*x[0]+(0.0679819277023)*x[1]+(0.892284314512)*x[2]
        arg[3,4,1,0]=(1.22220298478)*x[0]+(-0.0258417926472)*x[1]+(0.440670138568)*x[2]
        arg[3,4,1,1]=(0.647072768716)*x[0]+(-0.177651087906)*x[1]+(-0.115553644646)*x[2]
        arg[3,4,2,0]=(1.29816363306)*x[0]+(-1.18656044167)*x[1]+(-0.465776533157)*x[2]
        arg[3,4,2,1]=(0.328907496091)*x[0]+(-0.179857937151)*x[1]+(-0.476306938306)*x[2]
        ref[0,0,0,0]=(-0.530378359482)/(o+1.)+(-0.935143327523)+(0.0724486640358)*0.5**o
        ref[0,0,0,1]=(1.22837847871)/(o+1.)+(-0.113350954728)+(-0.950658241915)*0.5**o
        ref[0,0,1,0]=(-0.000414683535449)/(o+1.)+(-0.183158731552)+(-0.49805903018)*0.5**o
        ref[0,0,1,1]=(-0.882730389132)/(o+1.)+(-0.112174958816)+(0.831217305802)*0.5**o
        ref[0,0,2,0]=(-1.1913557966)/(o+1.)+(-0.0615750891074)+(-0.206094875146)*0.5**o
        ref[0,0,2,1]=(0.634536492589)/(o+1.)+(0.649890059464)+(0.627332034578)*0.5**o
        ref[0,1,0,0]=(-1.38665625775)/(o+1.)+(-1.0210653524)+(0.900623422471)*0.5**o
        ref[0,1,0,1]=(1.20633678024)/(o+1.)+(0.0888193121856)+(-0.143132885707)*0.5**o
        ref[0,1,1,0]=(0.452225278035)/(o+1.)+(-0.148926943801)+(-0.710642116901)*0.5**o
        ref[0,1,1,1]=(-0.244223430204)/(o+1.)+(-0.619607191996)+(0.499221868978)*0.5**o
        ref[0,1,2,0]=(0.0383849036606)/(o+1.)+(0.759484971644)+(-0.652076533427)*0.5**o
        ref[0,1,2,1]=(0.207558055759)/(o+1.)+(0.502305382384)+(0.0110157380215)*0.5**o
        ref[0,2,0,0]=(0.761374280144)/(o+1.)+(0.777616611515)+(0.734430478174)*0.5**o
        ref[0,2,0,1]=(0.554502862028)/(o+1.)+(-0.409274602016)+(-0.0353906413371)*0.5**o
        ref[0,2,1,0]=(-0.252543841709)/(o+1.)+(0.386640377894)+(0.522891013551)*0.5**o
        ref[0,2,1,1]=(-1.52497235887)/(o+1.)+(-0.807603254401)+(-0.0435508470367)*0.5**o
        ref[0,2,2,0]=(0.172485606257)/(o+1.)+(0.350297526187)+(0.19802601847)*0.5**o
        ref[0,2,2,1]=(-0.141271359379)/(o+1.)+(0.0509198134622)+(0.373638764298)*0.5**o
        ref[0,3,0,0]=(1.31278836809)/(o+1.)+(-0.0918739323256)+(0.350372309595)*0.5**o
        ref[0,3,0,1]=(-0.396933928109)/(o+1.)+(-0.436137805999)+(0.418204051696)*0.5**o
        ref[0,3,1,0]=(-0.206619840099)/(o+1.)+(-0.118703289798)+(0.353208336689)*0.5**o
        ref[0,3,1,1]=(-0.595153371293)/(o+1.)+(0.615722581562)+(0.469169640021)*0.5**o
        ref[0,3,2,0]=(-0.680858997986)/(o+1.)+(-0.0393223937964)+(-0.206963765944)*0.5**o
        ref[0,3,2,1]=(0.012753264598)/(o+1.)+(0.246188741489)+(-0.548093910595)*0.5**o
        ref[0,4,0,0]=(-0.40478987122)/(o+1.)+(-0.890399681609)+(0.33572726901)*0.5**o
        ref[0,4,0,1]=(0.0280062557264)/(o+1.)+(0.0358049413228)+(0.856417711355)*0.5**o
        ref[0,4,1,0]=(0.181560004565)/(o+1.)+(0.329885369854)+(0.31502676574)*0.5**o
        ref[0,4,1,1]=(0.0552681276841)/(o+1.)+(0.00372919426319)+(0.838827828375)*0.5**o
        ref[0,4,2,0]=(0.656057162724)/(o+1.)+(0.179315527684)+(-0.522542441641)*0.5**o
        ref[0,4,2,1]=(0.126189389317)/(o+1.)+(-0.429452001643)+(-0.149659196108)*0.5**o
        ref[1,0,0,0]=(1.43443485527)/(o+1.)+(-0.55091754491)+(0.0630603461901)*0.5**o
        ref[1,0,0,1]=(-0.612191779306)/(o+1.)+(0.604025235381)+(-0.230957890695)*0.5**o
        ref[1,0,1,0]=(0.373085423681)/(o+1.)+(-0.0817391530697)+(-0.113594076805)*0.5**o
        ref[1,0,1,1]=(0.761862677657)/(o+1.)+(0.819524484428)+(0.172331645033)*0.5**o
        ref[1,0,2,0]=(0.929265450746)/(o+1.)+(-0.382585784938)+(0.254478170515)*0.5**o
        ref[1,0,2,1]=(1.58236203463)/(o+1.)+(-0.280641188811)+(0.238179175679)*0.5**o
        ref[1,1,0,0]=(-0.117055726312)/(o+1.)+(0.957914090571)+(-0.101672257501)*0.5**o
        ref[1,1,0,1]=(0.0849062553796)/(o+1.)+(0.0703564566846)+(0.818261558262)*0.5**o
        ref[1,1,1,0]=(-1.72219113622)/(o+1.)+(-0.357454724939)+(0.15835663635)*0.5**o
        ref[1,1,1,1]=(0.739078400468)/(o+1.)+(0.532292543169)+(0.673031856531)*0.5**o
        ref[1,1,2,0]=(0.255023909903)/(o+1.)+(0.229656917655)+(0.22400044344)*0.5**o
        ref[1,1,2,1]=(0.518856765199)/(o+1.)+(0.380552790125)+(0.543298381396)*0.5**o
        ref[1,2,0,0]=(0.0321563884831)/(o+1.)+(0.25981567762)+(-0.0316496900482)*0.5**o
        ref[1,2,0,1]=(1.09318176771)/(o+1.)+(0.0572442907603)+(0.564519446305)*0.5**o
        ref[1,2,1,0]=(0.756962791387)/(o+1.)+(0.69419429624)+(0.32401997528)*0.5**o
        ref[1,2,1,1]=(1.28480299241)/(o+1.)+(-0.201990593541)+(-0.905711635518)*0.5**o
        ref[1,2,2,0]=(1.04969566442)/(o+1.)+(0.72854097372)+(0.592972696101)*0.5**o
        ref[1,2,2,1]=(-1.77086264775)/(o+1.)+(-0.261914180361)+(0.0440022926066)*0.5**o
        ref[1,3,0,0]=(-0.477780531124)/(o+1.)+(-0.181354085163)+(-0.300288699967)*0.5**o
        ref[1,3,0,1]=(-0.797627013456)/(o+1.)+(-0.523163955197)+(0.347574883435)*0.5**o
        ref[1,3,1,0]=(0.428806106821)/(o+1.)+(-0.0514584302394)+(-0.655980330052)*0.5**o
        ref[1,3,1,1]=(-0.141740459457)/(o+1.)+(-1.05699960931)+(-0.821242527644)*0.5**o
        ref[1,3,2,0]=(0.0181810985873)/(o+1.)+(-0.180697139553)+(0.41036925874)*0.5**o
        ref[1,3,2,1]=(-0.061077367)/(o+1.)+(0.250077446763)+(0.157102380812)*0.5**o
        ref[1,4,0,0]=(1.14087212913)/(o+1.)+(0.368059029864)+(0.659728641463)*0.5**o
        ref[1,4,0,1]=(0.365658713316)/(o+1.)+(0.088533821044)+(-0.584417938734)*0.5**o
        ref[1,4,1,0]=(-1.11681308321)/(o+1.)+(0.438275560627)+(0.622761672604)*0.5**o
        ref[1,4,1,1]=(1.18893372018)/(o+1.)+(0.172934240923)+(-0.172046824999)*0.5**o
        ref[1,4,2,0]=(0.16240489493)/(o+1.)+(-0.0145923852256)+(0.834478471749)*0.5**o
        ref[1,4,2,1]=(0.571355829378)/(o+1.)+(0.526896417749)+(0.458073367663)*0.5**o
        ref[2,0,0,0]=(-0.489234572937)/(o+1.)+(-0.0135589093901)+(0.65140309615)*0.5**o
        ref[2,0,0,1]=(-1.55383491931)/(o+1.)+(0.0161644619972)+(0.703425613366)*0.5**o
        ref[2,0,1,0]=(-0.258604855723)/(o+1.)+(0.0827812678671)+(0.507722104457)*0.5**o
        ref[2,0,1,1]=(0.0504178529481)/(o+1.)+(0.182991850332)+(0.359204920726)*0.5**o
        ref[2,0,2,0]=(0.652043479841)/(o+1.)+(-0.443015450013)+(-0.548443848248)*0.5**o
        ref[2,0,2,1]=(0.340317839857)/(o+1.)+(-0.379565594545)+(0.663652225917)*0.5**o
        ref[2,1,0,0]=(-0.587863196667)/(o+1.)+(-0.0137797674284)+(-0.974202340562)*0.5**o
        ref[2,1,0,1]=(1.00030747039)/(o+1.)+(-0.399997754768)+(0.755747639771)*0.5**o
        ref[2,1,1,0]=(0.703413281036)/(o+1.)+(0.487016208736)+(-0.877007540039)*0.5**o
        ref[2,1,1,1]=(0.263166830572)/(o+1.)+(-0.157259452849)+(0.666747865841)*0.5**o
        ref[2,1,2,0]=(0.104532343004)/(o+1.)+(-1.02548001493)+(-0.486314104701)*0.5**o
        ref[2,1,2,1]=(0.933518655478)/(o+1.)+(0.0369928229139)+(-0.244085626298)*0.5**o
        ref[2,2,0,0]=(-1.50550384519)/(o+1.)+(-0.590996227594)+(-0.570130630622)*0.5**o
        ref[2,2,0,1]=(-0.60318683839)/(o+1.)+(-0.38308245528)+(-0.502979448853)*0.5**o
        ref[2,2,1,0]=(-1.11258910879)/(o+1.)+(-0.518777277969)+(-0.999685897413)*0.5**o
        ref[2,2,1,1]=(1.84366617107)/(o+1.)+(-0.232866738261)+(-0.622157810558)*0.5**o
        ref[2,2,2,0]=(-0.620181988585)/(o+1.)+(-0.836162444183)+(0.267939886396)*0.5**o
        ref[2,2,2,1]=(-0.826647389099)/(o+1.)+(0.0929536518109)+(0.905694268362)*0.5**o
        ref[2,3,0,0]=(0.184067674443)/(o+1.)+(0.259745858757)+(0.724759799909)*0.5**o
        ref[2,3,0,1]=(0.0175595303049)/(o+1.)+(0.0560117358912)+(-0.907162868639)*0.5**o
        ref[2,3,1,0]=(-0.609443154617)/(o+1.)+(0.186540819401)+(-0.951152598958)*0.5**o
        ref[2,3,1,1]=(-1.61858082094)/(o+1.)+(0.679439858578)+(0.927423354971)*0.5**o
        ref[2,3,2,0]=(0.44350652486)/(o+1.)+(0.81466177481)+(0.53899353207)*0.5**o
        ref[2,3,2,1]=(-0.877289862245)/(o+1.)+(-1.05742260849)+(-0.462176941342)*0.5**o
        ref[2,4,0,0]=(-0.736381416539)/(o+1.)+(-0.0139374259612)+(-0.0755289501641)*0.5**o
        ref[2,4,0,1]=(0.690429326999)/(o+1.)+(-0.0293644018251)+(-0.328756862036)*0.5**o
        ref[2,4,1,0]=(-1.57117034002)/(o+1.)+(-0.531463765387)+(0.553349645561)*0.5**o
        ref[2,4,1,1]=(1.24898175282)/(o+1.)+(-0.452273024959)+(-0.515691967125)*0.5**o
        ref[2,4,2,0]=(0.239126191643)/(o+1.)+(0.722504563932)+(0.547637159392)*0.5**o
        ref[2,4,2,1]=(-1.40763254209)/(o+1.)+(-0.809456441012)+(0.932852295739)*0.5**o
        ref[3,0,0,0]=(-0.416554297686)/(o+1.)+(-0.632193610911)+(-0.304105530258)*0.5**o
        ref[3,0,0,1]=(-1.02983448036)/(o+1.)+(0.579212231778)+(-0.436532313966)*0.5**o
        ref[3,0,1,0]=(0.42124133216)/(o+1.)+(0.368290418827)+(-0.920395942436)*0.5**o
        ref[3,0,1,1]=(0.158195474854)/(o+1.)+(-0.17459474514)+(-0.830644214826)*0.5**o
        ref[3,0,2,0]=(-1.18749647329)/(o+1.)+(-0.506173996178)+(-0.513857022748)*0.5**o
        ref[3,0,2,1]=(0.233437753018)/(o+1.)+(0.455705145371)+(-0.323239005586)*0.5**o
        ref[3,1,0,0]=(1.42991958566)/(o+1.)+(0.053769921022)+(0.150959272234)*0.5**o
        ref[3,1,0,1]=(1.48423731111)/(o+1.)+(0.964048315997)+(-0.689863420196)*0.5**o
        ref[3,1,1,0]=(0.868837676765)/(o+1.)+(-1.01261543254)+(0.165878933098)*0.5**o
        ref[3,1,1,1]=(0.820750307831)/(o+1.)+(-0.300930223821)+(0.396516195659)*0.5**o
        ref[3,1,2,0]=(0.477077782202)/(o+1.)+(-0.0416336989494)+(0.769016797571)*0.5**o
        ref[3,1,2,1]=(0.359895440587)/(o+1.)+(0.555322395811)+(0.944871585683)*0.5**o
        ref[3,2,0,0]=(0.657668315832)/(o+1.)+(-0.240321895174)+(-0.286298603451)*0.5**o
        ref[3,2,0,1]=(-0.941360568282)/(o+1.)+(-0.0246544136593)+(-0.181845734439)*0.5**o
        ref[3,2,1,0]=(1.60316834971)/(o+1.)+(0.252291602298)+(0.988555915005)*0.5**o
        ref[3,2,1,1]=(0.833811957678)/(o+1.)+(-0.111970621871)+(0.634808834458)*0.5**o
        ref[3,2,2,0]=(-1.38877073784)/(o+1.)+(0.538356636307)+(-0.249568730877)*0.5**o
        ref[3,2,2,1]=(-0.311839184097)/(o+1.)+(-0.698114103264)+(0.169190433329)*0.5**o
        ref[3,3,0,0]=(0.743170989696)/(o+1.)+(-0.499224746788)+(0.0699956684628)*0.5**o
        ref[3,3,0,1]=(-0.293538786051)/(o+1.)+(0.26667992866)+(0.313940467768)*0.5**o
        ref[3,3,1,0]=(0.951483461345)/(o+1.)+(0.148317904561)+(0.00242640905887)*0.5**o
        ref[3,3,1,1]=(-0.110753912776)/(o+1.)+(-0.44308689138)+(0.980634836805)*0.5**o
        ref[3,3,2,0]=(-0.337895223544)/(o+1.)+(0.358941646037)+(0.264839226057)*0.5**o
        ref[3,3,2,1]=(0.857823611315)/(o+1.)+(0.30404611512)+(-0.898657291359)*0.5**o
        ref[3,4,0,0]=(0.037944509376)/(o+1.)+(-0.364801700413)+(-0.349756182268)*0.5**o
        ref[3,4,0,1]=(1.01114399396)/(o+1.)+(0.457103279485)+(0.631738783076)*0.5**o
        ref[3,4,1,0]=(0.969931913888)/(o+1.)+(-0.165922085756)+(0.998943588321)*0.5**o
        ref[3,4,1,1]=(1.35441114248)/(o+1.)+(-0.502422727199)+(0.00430234808496)*0.5**o
        ref[3,4,2,0]=(-0.367201097566)/(o+1.)+(-0.337780780531)+(0.688589316866)*0.5**o
        ref[3,4,2,1]=(-0.0547905846061)/(o+1.)+(-0.293870420685)+(0.31527404661)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactOne_fromData_ReducedContinuousFunction_rank0(self):
      """
      tests integral of rank 0 Data on the FunctionOnContactOne

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports integral on FunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(),w)
      if dim==2:
        arg=(0.241972460279)*x[0]+(0.539729925673)*x[1]
        ref=(-0.11832904353)/(o+1.)+(0.649911364492)+(-0.399791299503)*0.5**o
      else:
        arg=(0.53919797656)*x[0]+(0.379010664884)*x[1]+(0.274416064529)*x[2]
        ref=(0.307947301472)/(o+1.)+(0.120674157748)+(0.643329089005)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactOne_fromData_ReducedContinuousFunction_rank1(self):
      """
      tests integral of rank 1 Data on the FunctionOnContactOne

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports integral on FunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(2,),w)
      ref=numpy.zeros((2,),numpy.float_)
      if dim==2:
        arg[0]=(0.84849017056)*x[0]+(-0.161962845202)*x[1]
        arg[1]=(-0.794717919135)*x[0]+(1.16003773358)*x[1]
        ref[0]=(0.780514448202)/(o+1.)+(-0.34628958021)+(0.598592037576)*0.5**o
        ref[1]=(0.570158156278)/(o+1.)+(0.241192316704)+(-0.687222975237)*0.5**o
      else:
        arg[0]=(-0.637352163634)*x[0]+(1.49477462643)*x[1]+(-0.284018740238)*x[2]
        arg[1]=(0.125937459904)*x[0]+(-0.206499869891)*x[1]+(-0.342856240345)*x[2]
        ref[0]=(-0.274486314451)/(o+1.)+(0.379737553853)+(0.0884149293071)*0.5**o
        ref[1]=(-0.78452814021)/(o+1.)+(-0.122017410137)+(0.605144310151)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactOne_fromData_ReducedContinuousFunction_rank2(self):
      """
      tests integral of rank 2 Data on the FunctionOnContactOne

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports integral on FunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5),w)
      ref=numpy.zeros((4, 5),numpy.float_)
      if dim==2:
        arg[0,0]=(-0.440868762581)*x[0]+(-0.216223666616)*x[1]
        arg[0,1]=(-1.78807355591)*x[0]+(-0.208278123838)*x[1]
        arg[0,2]=(-0.448066998103)*x[0]+(-0.819815507826)*x[1]
        arg[0,3]=(1.27247881689)*x[0]+(0.0653439567221)*x[1]
        arg[0,4]=(-1.00344990223)*x[0]+(-0.291263790583)*x[1]
        arg[1,0]=(0.440080423838)*x[0]+(0.0356836558113)*x[1]
        arg[1,1]=(-1.49545340091)*x[0]+(-0.133615590211)*x[1]
        arg[1,2]=(1.05947638385)*x[0]+(0.408685904238)*x[1]
        arg[1,3]=(-0.170309086605)*x[0]+(0.660523970285)*x[1]
        arg[1,4]=(0.764323678475)*x[0]+(-0.529415080303)*x[1]
        arg[2,0]=(0.55270081325)*x[0]+(-1.60630146231)*x[1]
        arg[2,1]=(1.57419330248)*x[0]+(-1.24644288706)*x[1]
        arg[2,2]=(0.0416757238363)*x[0]+(0.0247379738465)*x[1]
        arg[2,3]=(1.0200370393)*x[0]+(0.364209137693)*x[1]
        arg[2,4]=(-1.21858490389)*x[0]+(0.523056254546)*x[1]
        arg[3,0]=(0.0524205820842)*x[0]+(1.65953217854)*x[1]
        arg[3,1]=(1.21824883395)*x[0]+(0.905190834998)*x[1]
        arg[3,2]=(0.226344434525)*x[0]+(-0.735349531462)*x[1]
        arg[3,3]=(-1.00295245717)*x[0]+(-0.719605376499)*x[1]
        arg[3,4]=(-0.446228759693)*x[0]+(1.23441974277)*x[1]
        ref[0,0]=(0.347328582602)/(o+1.)+(-0.520484463519)+(0.0365479152384)*0.5**o
        ref[0,1]=(-0.274564442352)/(o+1.)+(-0.413557210678)+(-0.894672816039)*0.5**o
        ref[0,2]=(0.0720644688954)/(o+1.)+(-0.906636966188)+(0.473326957552)*0.5**o
        ref[0,3]=(-0.747749341777)/(o+1.)+(0.730249972378)+(0.625072170633)*0.5**o
        ref[0,4]=(-0.194031116169)/(o+1.)+(-0.432063325829)+(-0.236555924983)*0.5**o
        ref[1,0]=(0.807081538033)/(o+1.)+(-0.612266650007)+(0.893215841629)*0.5**o
        ref[1,1]=(-0.662516518957)/(o+1.)+(-0.176318925886)+(-0.61391462039)*0.5**o
        ref[1,2]=(-0.570345636156)/(o+1.)+(0.791877954963)+(0.454752014319)*0.5**o
        ref[1,3]=(-0.312400910795)/(o+1.)+(0.212935137466)+(0.376745519544)*0.5**o
        ref[1,4]=(-0.764680414264)/(o+1.)+(0.174215486579)+(0.651158039278)*0.5**o
        ref[2,0]=(-0.837942771642)/(o+1.)+(-0.187938486702)+(0.160219095985)*0.5**o
        ref[2,1]=(-0.672421039761)/(o+1.)+(0.17384002882)+(0.652491397549)*0.5**o
        ref[2,2]=(-0.656187949922)/(o+1.)+(0.559888736121)+(-0.397175824637)*0.5**o
        ref[2,3]=(0.259558368195)/(o+1.)+(0.55148644978)+(0.0217149092406)*0.5**o
        ref[2,4]=(-0.188397537901)/(o+1.)+(-0.0561706170512)+(-0.394789877345)*0.5**o
        ref[3,0]=(0.91055628617)/(o+1.)+(0.574211274098)+(-0.347026073739)*0.5**o
        ref[3,1]=(0.29225509237)/(o+1.)+(0.64447705482)+(0.542230466941)*0.5**o
        ref[3,2]=(0.121065624384)/(o+1.)+(0.0316495277836)+(-0.693369776888)*0.5**o
        ref[3,3]=(-0.943523334976)/(o+1.)+(-0.266302644031)+(-0.246429210629)*0.5**o
        ref[3,4]=(0.714915399455)/(o+1.)+(0.459963131994)+(-0.846650680369)*0.5**o
      else:
        arg[0,0]=(-0.493869713454)*x[0]+(-0.381969413515)*x[1]+(1.59862368848)*x[2]
        arg[0,1]=(-0.581245543547)*x[0]+(0.961117271889)*x[1]+(-0.158073515343)*x[2]
        arg[0,2]=(-0.635283143641)*x[0]+(0.695573236987)*x[1]+(-0.719180832278)*x[2]
        arg[0,3]=(-0.737780596733)*x[0]+(-1.11171283352)*x[1]+(0.459954346315)*x[2]
        arg[0,4]=(1.12522065074)*x[0]+(-1.08427904854)*x[1]+(-0.715679673556)*x[2]
        arg[1,0]=(0.972468186181)*x[0]+(-0.211014530435)*x[1]+(-0.734414099002)*x[2]
        arg[1,1]=(-1.67337704748)*x[0]+(0.871096213089)*x[1]+(1.51063313223)*x[2]
        arg[1,2]=(-0.222013007834)*x[0]+(1.46474508326)*x[1]+(-0.979243908862)*x[2]
        arg[1,3]=(1.21195281087)*x[0]+(0.274082389329)*x[1]+(1.56144436465)*x[2]
        arg[1,4]=(1.35812865884)*x[0]+(1.25819798853)*x[1]+(0.895334048395)*x[2]
        arg[2,0]=(-0.167404344437)*x[0]+(0.0710591934315)*x[1]+(-0.124469683665)*x[2]
        arg[2,1]=(-0.492240632996)*x[0]+(-0.317251117382)*x[1]+(0.0195461703529)*x[2]
        arg[2,2]=(-0.277590266225)*x[0]+(-1.65859476899)*x[1]+(-0.0100216405668)*x[2]
        arg[2,3]=(-0.0701281298455)*x[0]+(0.502137254703)*x[1]+(0.701220923443)*x[2]
        arg[2,4]=(-0.72186096407)*x[0]+(-0.900487568179)*x[1]+(-0.735573922826)*x[2]
        arg[3,0]=(-0.0677324474129)*x[0]+(-0.649963209718)*x[1]+(-0.770861099726)*x[2]
        arg[3,1]=(0.563588189481)*x[0]+(0.714614932251)*x[1]+(-0.0957764632886)*x[2]
        arg[3,2]=(1.13337395062)*x[0]+(0.507556316073)*x[1]+(-0.196759085949)*x[2]
        arg[3,3]=(-1.2636883168)*x[0]+(-0.811660344076)*x[1]+(0.645462052598)*x[2]
        arg[3,4]=(-0.104175186987)*x[0]+(-0.929176038155)*x[1]+(1.07530379856)*x[2]
        ref[0,0]=(0.84538674973)/(o+1.)+(0.0569713556632)+(-0.236544899547)*0.5**o
        ref[0,1]=(-0.307689589699)/(o+1.)+(0.245040337642)+(0.0394071274131)*0.5**o
        ref[0,2]=(0.955244736357)/(o+1.)+(-0.341771136504)+(-0.930593202281)*0.5**o
        ref[0,3]=(-0.353490461722)/(o+1.)+(-0.476228433901)+(-0.083591754415)*0.5**o
        ref[0,4]=(-1.31220485482)/(o+1.)+(0.0466426656676)+(0.544181452128)*0.5**o
        ref[1,0]=(0.22773475414)/(o+1.)+(-0.488454801654)+(0.776214405914)*0.5**o
        ref[1,1]=(1.50688678707)/(o+1.)+(0.0851415404695)+(-0.96881757017)*0.5**o
        ref[1,2]=(0.313471097102)/(o+1.)+(-0.0875594518156)+(0.125135973096)*0.5**o
        ref[1,3]=(0.269521220179)/(o+1.)+(1.1192007195)+(0.539556905661)*0.5**o
        ref[1,4]=(1.91499876064)/(o+1.)+(0.509063797893)+(0.578534339343)*0.5**o
        ref[2,0]=(0.941352651562)/(o+1.)+(-0.966199402186)+(0.770231318139)*0.5**o
        ref[2,1]=(-0.141490823562)/(o+1.)+(-0.20919987022)+(-0.230055016023)*0.5**o
        ref[2,2]=(-0.916148047971)/(o+1.)+(-0.371740600192)+(-0.286577427426)*0.5**o
        ref[2,3]=(0.229516219331)/(o+1.)+(0.274790532736)+(0.354132763498)*0.5**o
        ref[2,4]=(-1.19120612619)/(o+1.)+(-0.139859859385)+(-0.88699661011)*0.5**o
        ref[3,0]=(-1.48630121465)/(o+1.)+(0.0744166251063)+(-0.151088792417)*0.5**o
        ref[3,1]=(-0.335583560513)/(o+1.)+(0.277436410841)+(0.963137397276)*0.5**o
        ref[3,2]=(-0.673942688644)/(o+1.)+(0.70077509905)+(0.716563671283)*0.5**o
        ref[3,3]=(-0.148394092795)/(o+1.)+(-0.237190239072)+(-0.807112037336)*0.5**o
        ref[3,4]=(0.529983886211)/(o+1.)+(-0.371055083588)+(0.254078854382)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactOne_fromData_ReducedContinuousFunction_rank3(self):
      """
      tests integral of rank 3 Data on the FunctionOnContactOne

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports integral on FunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(6, 2, 2),w)
      ref=numpy.zeros((6, 2, 2),numpy.float_)
      if dim==2:
        arg[0,0,0]=(0.661725715165)*x[0]+(0.219720717581)*x[1]
        arg[0,0,1]=(-0.642240082304)*x[0]+(-0.279652756776)*x[1]
        arg[0,1,0]=(0.0204235813151)*x[0]+(-0.374873249933)*x[1]
        arg[0,1,1]=(1.4418481252)*x[0]+(0.205656834772)*x[1]
        arg[1,0,0]=(0.65992179951)*x[0]+(-1.04887875458)*x[1]
        arg[1,0,1]=(-0.195480216199)*x[0]+(-0.373905275039)*x[1]
        arg[1,1,0]=(0.781251543352)*x[0]+(0.0416248109536)*x[1]
        arg[1,1,1]=(-0.925751319189)*x[0]+(-0.113992910046)*x[1]
        arg[2,0,0]=(-0.0767414065227)*x[0]+(-0.156682840973)*x[1]
        arg[2,0,1]=(0.688457272529)*x[0]+(0.524916336046)*x[1]
        arg[2,1,0]=(0.753722830084)*x[0]+(-0.890325839466)*x[1]
        arg[2,1,1]=(0.730789290145)*x[0]+(-0.278986703716)*x[1]
        arg[3,0,0]=(-0.287376796439)*x[0]+(-0.153987233224)*x[1]
        arg[3,0,1]=(-1.15687435255)*x[0]+(0.250005890238)*x[1]
        arg[3,1,0]=(-0.845312830629)*x[0]+(0.256211962352)*x[1]
        arg[3,1,1]=(-0.574311820519)*x[0]+(-0.440297035998)*x[1]
        arg[4,0,0]=(-0.360607647308)*x[0]+(-1.29380437557)*x[1]
        arg[4,0,1]=(1.32019433485)*x[0]+(0.295255956618)*x[1]
        arg[4,1,0]=(-1.64162941972)*x[0]+(0.117482372162)*x[1]
        arg[4,1,1]=(-0.303103646619)*x[0]+(-0.88458982681)*x[1]
        arg[5,0,0]=(-0.666761879038)*x[0]+(-0.735786278877)*x[1]
        arg[5,0,1]=(0.0892896382617)*x[0]+(-0.45478872773)*x[1]
        arg[5,1,0]=(-0.50985624415)*x[0]+(-0.400022939567)*x[1]
        arg[5,1,1]=(1.39240706368)*x[0]+(0.0149740822664)*x[1]
        ref[0,0,0]=(-0.114834538488)/(o+1.)+(0.240621006484)+(0.515038958264)*0.5**o
        ref[0,0,1]=(0.588698337077)/(o+1.)+(-0.702256377233)+(-0.10607842169)*0.5**o
        ref[0,1,0]=(0.13849818224)/(o+1.)+(-0.277555277991)+(0.0621627051247)*0.5**o
        ref[0,1,1]=(-0.0878009521163)/(o+1.)+(0.630168348947)+(0.474969214192)*0.5**o
        ref[1,0,0]=(-0.696600538111)/(o+1.)+(-0.343318260912)+(0.994280104861)*0.5**o
        ref[1,0,1]=(-0.0887026688527)/(o+1.)+(-0.385753227979)+(0.290823633572)*0.5**o
        ref[1,1,0]=(0.831408608956)/(o+1.)+(-0.242985671853)+(0.477439089057)*0.5**o
        ref[1,1,1]=(-0.621059053191)/(o+1.)+(-0.0700203481241)+(-0.278644479797)*0.5**o
        ref[2,0,0]=(0.7890720399)/(o+1.)+(-0.539721196393)+(0.0569461053905)*0.5**o
        ref[2,0,1]=(-0.228628151401)/(o+1.)+(0.826946074171)+(-0.211890388366)*0.5**o
        ref[2,1,0]=(-0.837617325493)/(o+1.)+(0.289656653751)+(0.121701008608)*0.5**o
        ref[2,1,1]=(0.246577988625)/(o+1.)+(0.0512227634376)+(0.102779070928)*0.5**o
        ref[3,0,0]=(-0.0109584684212)/(o+1.)+(-0.334888666171)+(0.239371771102)*0.5**o
        ref[3,0,1]=(-0.125987676419)/(o+1.)+(-0.170179004032)+(-0.440522777827)*0.5**o
        ref[3,1,0]=(-0.354093141636)/(o+1.)+(-0.177294275919)+(0.119580825198)*0.5**o
        ref[3,1,1]=(-0.746761075412)/(o+1.)+(0.192068760731)+(-0.651985302566)*0.5**o
        ref[4,0,0]=(-0.42411673219)/(o+1.)+(-0.579162944937)+(-0.0719694008158)*0.5**o
        ref[4,0,1]=(0.169881169584)/(o+1.)+(0.483614029132)+(0.478341063616)*0.5**o
        ref[4,1,0]=(0.943567836264)/(o+1.)+(-0.898782941264)+(-0.670149001297)*0.5**o
        ref[4,1,1]=(-0.703610807936)/(o+1.)+(0.225342628657)+(-0.934767922806)*0.5**o
        ref[5,0,0]=(-0.788792225213)/(o+1.)+(-0.1768857093)+(-0.259984514102)*0.5**o
        ref[5,0,1]=(-0.788708879899)/(o+1.)+(-0.202288491141)+(0.827786772711)*0.5**o
        ref[5,1,0]=(-0.823081809707)/(o+1.)+(0.274720963093)+(-0.636239300197)*0.5**o
        ref[5,1,1]=(0.498922549807)/(o+1.)+(0.0129388257746)+(0.882580944588)*0.5**o
      else:
        arg[0,0,0]=(-0.426613654844)*x[0]+(-1.01433980801)*x[1]+(0.564621874916)*x[2]
        arg[0,0,1]=(-0.438521968882)*x[0]+(1.6033775267)*x[1]+(0.691349995832)*x[2]
        arg[0,1,0]=(-0.04337513731)*x[0]+(0.142927668234)*x[1]+(0.506581723199)*x[2]
        arg[0,1,1]=(-0.120598868162)*x[0]+(-0.213666787813)*x[1]+(-1.04221679332)*x[2]
        arg[1,0,0]=(0.153696490712)*x[0]+(-0.122516529815)*x[1]+(-1.08837272712)*x[2]
        arg[1,0,1]=(0.410706464237)*x[0]+(0.720079363315)*x[1]+(-0.767790357539)*x[2]
        arg[1,1,0]=(-1.03726428173)*x[0]+(-0.316809853003)*x[1]+(-0.780405868082)*x[2]
        arg[1,1,1]=(-0.427547967216)*x[0]+(-0.897766643609)*x[1]+(-1.56913552404)*x[2]
        arg[2,0,0]=(-0.282789248898)*x[0]+(-0.110846936031)*x[1]+(-1.4329391906)*x[2]
        arg[2,0,1]=(1.46866588617)*x[0]+(-1.23022501959)*x[1]+(-0.3123390018)*x[2]
        arg[2,1,0]=(-0.314115718621)*x[0]+(1.7333934438)*x[1]+(0.833572700726)*x[2]
        arg[2,1,1]=(0.464246462638)*x[0]+(0.267484068646)*x[1]+(0.342009204787)*x[2]
        arg[3,0,0]=(0.578853099163)*x[0]+(-0.0453855508465)*x[1]+(-0.0751578475516)*x[2]
        arg[3,0,1]=(-0.526734485752)*x[0]+(-1.03286310016)*x[1]+(-0.398986583917)*x[2]
        arg[3,1,0]=(0.0239088277798)*x[0]+(0.205105470192)*x[1]+(-1.04986349166)*x[2]
        arg[3,1,1]=(0.130893744001)*x[0]+(-0.273702140669)*x[1]+(-0.0804222415128)*x[2]
        arg[4,0,0]=(-0.822661618584)*x[0]+(-0.267189204519)*x[1]+(-0.290013766971)*x[2]
        arg[4,0,1]=(-1.46759823734)*x[0]+(0.317465945316)*x[1]+(0.108284317752)*x[2]
        arg[4,1,0]=(-1.3920990386)*x[0]+(-1.17823778694)*x[1]+(-0.18366434307)*x[2]
        arg[4,1,1]=(-0.0915496209516)*x[0]+(-0.367624609832)*x[1]+(1.30630257323)*x[2]
        arg[5,0,0]=(0.117840830455)*x[0]+(-0.667875433194)*x[1]+(-1.59716688455)*x[2]
        arg[5,0,1]=(-0.818693227605)*x[0]+(0.483842075248)*x[1]+(-0.408373205535)*x[2]
        arg[5,1,0]=(1.36906770301)*x[0]+(0.494926179999)*x[1]+(0.717041707262)*x[2]
        arg[5,1,1]=(0.819757116081)*x[0]+(0.0982458749015)*x[1]+(0.275080391599)*x[2]
        ref[0,0,0]=(0.236560699949)/(o+1.)+(-0.392784388084)+(-0.327323511722)*0.5**o
        ref[0,0,1]=(1.32491522045)/(o+1.)+(0.525146863711)+(-0.519003394222)*0.5**o
        ref[0,1,0]=(0.433749865563)/(o+1.)+(0.23007126008)+(-0.287758131599)*0.5**o
        ref[0,1,1]=(-0.0563021060686)/(o+1.)+(-0.403921160703)+(-0.512338021825)*0.5**o
        ref[1,0,0]=(-0.170496965963)/(o+1.)+(-0.914421777259)+(0.942147754258)*0.5**o
        ref[1,0,1]=(-0.372707483726)/(o+1.)+(0.636041158872)+(-0.536379364005)*0.5**o
        ref[1,1,0]=(-0.211529878364)/(o+1.)+(-0.912151985776)+(-0.0986461529033)*0.5**o
        ref[1,1,1]=(-1.32533998437)/(o+1.)+(-0.685030198386)+(-0.19904975372)*0.5**o
        ref[2,0,0]=(0.152048902469)/(o+1.)+(-0.88880590209)+(-0.201012473813)*0.5**o
        ref[2,0,1]=(-0.748900052246)/(o+1.)+(-0.0226120471998)+(0.720226011425)*0.5**o
        ref[2,1,0]=(0.77797707941)/(o+1.)+(1.01635055442)+(-0.557827762351)*0.5**o
        ref[2,1,1]=(1.08969334716)/(o+1.)+(-0.404644728844)+(0.793335846595)*0.5**o
        ref[3,0,0]=(1.56498448669)/(o+1.)+(-0.699967188685)+(0.293259591449)*0.5**o
        ref[3,0,1]=(-0.620133427128)/(o+1.)+(-0.550037413688)+(-0.238375915329)*0.5**o
        ref[3,1,0]=(-0.156380586288)/(o+1.)+(-0.0154369973565)+(-0.633594612687)*0.5**o
        ref[3,1,1]=(-0.352989559326)/(o+1.)+(-0.290547031627)+(0.710852984398)*0.5**o
        ref[4,0,0]=(-0.147579361355)/(o+1.)+(-0.615173424765)+(-0.00193837918766)*0.5**o
        ref[4,0,1]=(0.598152345277)/(o+1.)+(-0.540248606426)+(-0.559503106696)*0.5**o
        ref[4,1,0]=(-1.17182448825)/(o+1.)+(-0.590551681576)+(-0.401073317208)*0.5**o
        ref[4,1,1]=(0.837504141173)/(o+1.)+(0.0807414267186)+(-0.15185865216)*0.5**o
        ref[5,0,0]=(-0.646601115533)/(o+1.)+(-1.20895265273)+(0.9173049337)*0.5**o
        ref[5,0,1]=(1.26626110052)/(o+1.)+(-0.56471923973)+(-0.880046978955)*0.5**o
        ref[5,1,0]=(-0.069063169298)/(o+1.)+(0.883536796964)+(0.883025165644)*0.5**o
        ref[5,1,1]=(1.05999338993)/(o+1.)+(-0.392519102893)+(0.918128198439)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactOne_fromData_ReducedContinuousFunction_rank4(self):
      """
      tests integral of rank 4 Data on the FunctionOnContactOne

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports integral on FunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5, 3, 2),w)
      ref=numpy.zeros((4, 5, 3, 2),numpy.float_)
      if dim==2:
        arg[0,0,0,0]=(0.257898226293)*x[0]+(-0.01230083314)*x[1]
        arg[0,0,0,1]=(0.0531183070647)*x[0]+(-0.170192287836)*x[1]
        arg[0,0,1,0]=(-0.364644940864)*x[0]+(1.04227651446)*x[1]
        arg[0,0,1,1]=(-0.894334108203)*x[0]+(0.151390331211)*x[1]
        arg[0,0,2,0]=(1.03825401708)*x[0]+(-0.882136097149)*x[1]
        arg[0,0,2,1]=(-1.05304910155)*x[0]+(1.06866311708)*x[1]
        arg[0,1,0,0]=(1.54742542159)*x[0]+(0.941778348482)*x[1]
        arg[0,1,0,1]=(1.1797380956)*x[0]+(-0.296417138234)*x[1]
        arg[0,1,1,0]=(-1.09196028027)*x[0]+(-0.783377624006)*x[1]
        arg[0,1,1,1]=(0.370952191443)*x[0]+(-0.0854141858991)*x[1]
        arg[0,1,2,0]=(-1.23552368139)*x[0]+(1.23848088087)*x[1]
        arg[0,1,2,1]=(0.216345350114)*x[0]+(-0.136524468427)*x[1]
        arg[0,2,0,0]=(-1.21206901263)*x[0]+(0.804219582144)*x[1]
        arg[0,2,0,1]=(0.192956526208)*x[0]+(0.588417678858)*x[1]
        arg[0,2,1,0]=(-0.940175603742)*x[0]+(0.329047260784)*x[1]
        arg[0,2,1,1]=(-0.138597387792)*x[0]+(0.398298326223)*x[1]
        arg[0,2,2,0]=(0.10089766824)*x[0]+(1.25459493298)*x[1]
        arg[0,2,2,1]=(-0.401163525883)*x[0]+(0.276542765282)*x[1]
        arg[0,3,0,0]=(0.252156795268)*x[0]+(-0.50223063539)*x[1]
        arg[0,3,0,1]=(0.466244610905)*x[0]+(-0.587025224301)*x[1]
        arg[0,3,1,0]=(-0.268082012713)*x[0]+(-1.28874675764)*x[1]
        arg[0,3,1,1]=(1.10867463519)*x[0]+(-0.165500912864)*x[1]
        arg[0,3,2,0]=(-1.94681908266)*x[0]+(0.166854108476)*x[1]
        arg[0,3,2,1]=(-0.537150417922)*x[0]+(0.0872851457131)*x[1]
        arg[0,4,0,0]=(-0.376548581807)*x[0]+(-0.245201133388)*x[1]
        arg[0,4,0,1]=(-0.135287690767)*x[0]+(0.750376238942)*x[1]
        arg[0,4,1,0]=(-0.0240491596907)*x[0]+(-1.21337088424)*x[1]
        arg[0,4,1,1]=(0.699215733397)*x[0]+(0.0596356032853)*x[1]
        arg[0,4,2,0]=(-0.521316010702)*x[0]+(-1.75309802615)*x[1]
        arg[0,4,2,1]=(0.403640172305)*x[0]+(1.41465827077)*x[1]
        arg[1,0,0,0]=(-1.01538089315)*x[0]+(-1.14785578909)*x[1]
        arg[1,0,0,1]=(1.10045201832)*x[0]+(0.630761728795)*x[1]
        arg[1,0,1,0]=(0.137673646147)*x[0]+(-0.387129090253)*x[1]
        arg[1,0,1,1]=(-1.04588287863)*x[0]+(-1.22433029859)*x[1]
        arg[1,0,2,0]=(-0.567830263535)*x[0]+(-0.405808616295)*x[1]
        arg[1,0,2,1]=(0.139326102495)*x[0]+(-1.93153384363)*x[1]
        arg[1,1,0,0]=(-0.638524762921)*x[0]+(0.506405751653)*x[1]
        arg[1,1,0,1]=(0.130775153144)*x[0]+(0.531317202208)*x[1]
        arg[1,1,1,0]=(0.557476206038)*x[0]+(0.447588676184)*x[1]
        arg[1,1,1,1]=(-0.538795916875)*x[0]+(-0.593869493531)*x[1]
        arg[1,1,2,0]=(-1.09827483942)*x[0]+(0.376431061876)*x[1]
        arg[1,1,2,1]=(-0.0074505620387)*x[0]+(-0.798645151486)*x[1]
        arg[1,2,0,0]=(0.251283911459)*x[0]+(0.194595071813)*x[1]
        arg[1,2,0,1]=(1.00482763799)*x[0]+(-1.00059863394)*x[1]
        arg[1,2,1,0]=(-0.453095434684)*x[0]+(-0.513922011418)*x[1]
        arg[1,2,1,1]=(0.558548837484)*x[0]+(0.0696328406825)*x[1]
        arg[1,2,2,0]=(-1.20625552545)*x[0]+(-0.151371710692)*x[1]
        arg[1,2,2,1]=(0.100262209656)*x[0]+(-0.314628804179)*x[1]
        arg[1,3,0,0]=(0.947063141882)*x[0]+(-0.297271200562)*x[1]
        arg[1,3,0,1]=(0.0760277251261)*x[0]+(-1.17982146387)*x[1]
        arg[1,3,1,0]=(-1.46946063969)*x[0]+(-0.615302663567)*x[1]
        arg[1,3,1,1]=(0.419747293729)*x[0]+(-0.629842531949)*x[1]
        arg[1,3,2,0]=(-1.44607954917)*x[0]+(0.520024093924)*x[1]
        arg[1,3,2,1]=(-1.01556392596)*x[0]+(-0.307173811354)*x[1]
        arg[1,4,0,0]=(-0.164339335244)*x[0]+(0.390736414982)*x[1]
        arg[1,4,0,1]=(-0.266932380288)*x[0]+(-0.731533848078)*x[1]
        arg[1,4,1,0]=(-0.903825489055)*x[0]+(-1.81564784421)*x[1]
        arg[1,4,1,1]=(-0.426008882665)*x[0]+(-1.09792948908)*x[1]
        arg[1,4,2,0]=(-1.32984927589)*x[0]+(1.58196302541)*x[1]
        arg[1,4,2,1]=(-0.0581032806145)*x[0]+(-0.323073355302)*x[1]
        arg[2,0,0,0]=(-0.744014251373)*x[0]+(-0.818253254872)*x[1]
        arg[2,0,0,1]=(0.368656965396)*x[0]+(-0.0864646380237)*x[1]
        arg[2,0,1,0]=(-0.112428845773)*x[0]+(0.109170104421)*x[1]
        arg[2,0,1,1]=(0.0682241548911)*x[0]+(-1.65999482283)*x[1]
        arg[2,0,2,0]=(0.212777058993)*x[0]+(0.127163480945)*x[1]
        arg[2,0,2,1]=(-0.256989443356)*x[0]+(0.183206687414)*x[1]
        arg[2,1,0,0]=(0.578997994373)*x[0]+(0.413087453689)*x[1]
        arg[2,1,0,1]=(0.886532090807)*x[0]+(0.330645525532)*x[1]
        arg[2,1,1,0]=(0.18224742814)*x[0]+(-0.713166591262)*x[1]
        arg[2,1,1,1]=(-0.25218296706)*x[0]+(0.736797897014)*x[1]
        arg[2,1,2,0]=(0.885270371408)*x[0]+(-1.35641397127)*x[1]
        arg[2,1,2,1]=(-0.963214290879)*x[0]+(0.348421598509)*x[1]
        arg[2,2,0,0]=(0.884120745351)*x[0]+(0.956857507884)*x[1]
        arg[2,2,0,1]=(-0.00795298517561)*x[0]+(0.51243740322)*x[1]
        arg[2,2,1,0]=(-0.563655252975)*x[0]+(-0.716305131721)*x[1]
        arg[2,2,1,1]=(0.648243491623)*x[0]+(-0.142239786485)*x[1]
        arg[2,2,2,0]=(1.50677848542)*x[0]+(-0.359722006818)*x[1]
        arg[2,2,2,1]=(-0.522592086302)*x[0]+(-0.18544927497)*x[1]
        arg[2,3,0,0]=(0.0822787762966)*x[0]+(0.196309662733)*x[1]
        arg[2,3,0,1]=(0.232573258627)*x[0]+(-0.0597976775432)*x[1]
        arg[2,3,1,0]=(1.50895737052)*x[0]+(0.546625934312)*x[1]
        arg[2,3,1,1]=(-1.64023373286)*x[0]+(0.220053261777)*x[1]
        arg[2,3,2,0]=(1.5897208842)*x[0]+(-0.0355885594992)*x[1]
        arg[2,3,2,1]=(-1.52822349982)*x[0]+(-0.0906934717656)*x[1]
        arg[2,4,0,0]=(0.496696582601)*x[0]+(-0.303888675059)*x[1]
        arg[2,4,0,1]=(-0.0819586112177)*x[0]+(-0.0449047294378)*x[1]
        arg[2,4,1,0]=(-0.678530758513)*x[0]+(-1.2173963447)*x[1]
        arg[2,4,1,1]=(-0.645450475268)*x[0]+(0.356771113426)*x[1]
        arg[2,4,2,0]=(-0.486643021867)*x[0]+(0.977763869689)*x[1]
        arg[2,4,2,1]=(-0.458991807397)*x[0]+(1.80038432923)*x[1]
        arg[3,0,0,0]=(0.270795948755)*x[0]+(-0.691512096338)*x[1]
        arg[3,0,0,1]=(-0.756717130099)*x[0]+(1.16189640534)*x[1]
        arg[3,0,1,0]=(0.967554346699)*x[0]+(-0.272119008963)*x[1]
        arg[3,0,1,1]=(1.14296491195)*x[0]+(-0.707175662379)*x[1]
        arg[3,0,2,0]=(1.15665030692)*x[0]+(0.414803389409)*x[1]
        arg[3,0,2,1]=(1.38997274955)*x[0]+(1.07065568192)*x[1]
        arg[3,1,0,0]=(-0.623998925511)*x[0]+(0.375545806421)*x[1]
        arg[3,1,0,1]=(-0.363431948398)*x[0]+(-0.183100417297)*x[1]
        arg[3,1,1,0]=(0.0435866301729)*x[0]+(-0.557137572068)*x[1]
        arg[3,1,1,1]=(-0.253225217468)*x[0]+(-0.0657932188329)*x[1]
        arg[3,1,2,0]=(0.147700921064)*x[0]+(0.681624563015)*x[1]
        arg[3,1,2,1]=(-0.075394297616)*x[0]+(0.271741245789)*x[1]
        arg[3,2,0,0]=(1.34803044812)*x[0]+(0.87991065609)*x[1]
        arg[3,2,0,1]=(0.0529487135353)*x[0]+(0.806480092352)*x[1]
        arg[3,2,1,0]=(-1.52038624166)*x[0]+(-0.992718138685)*x[1]
        arg[3,2,1,1]=(0.0859852188749)*x[0]+(1.63512596202)*x[1]
        arg[3,2,2,0]=(1.46925148575)*x[0]+(-1.60383564005)*x[1]
        arg[3,2,2,1]=(0.114463395473)*x[0]+(-1.0186377376)*x[1]
        arg[3,3,0,0]=(-1.02962502658)*x[0]+(-1.06788543146)*x[1]
        arg[3,3,0,1]=(-0.184391597652)*x[0]+(0.462603936483)*x[1]
        arg[3,3,1,0]=(0.611113335782)*x[0]+(0.282417397043)*x[1]
        arg[3,3,1,1]=(-0.193423873796)*x[0]+(-0.56108614964)*x[1]
        arg[3,3,2,0]=(-0.0290687598224)*x[0]+(-0.460221158417)*x[1]
        arg[3,3,2,1]=(0.784513522959)*x[0]+(-0.484813129206)*x[1]
        arg[3,4,0,0]=(0.423780748664)*x[0]+(-0.344020787894)*x[1]
        arg[3,4,0,1]=(1.10623024451)*x[0]+(-0.775008153301)*x[1]
        arg[3,4,1,0]=(0.554847534291)*x[0]+(-0.854749107999)*x[1]
        arg[3,4,1,1]=(-0.639595693597)*x[0]+(-0.966152813212)*x[1]
        arg[3,4,2,0]=(-0.0172621996023)*x[0]+(0.321815074269)*x[1]
        arg[3,4,2,1]=(0.39605156824)*x[0]+(-0.499138231749)*x[1]
        ref[0,0,0,0]=(0.169892838337)/(o+1.)+(-0.323261281472)+(0.722227117759)*0.5**o
        ref[0,0,0,1]=(-0.899797330597)/(o+1.)+(0.731242549464)+(-0.679761749104)*0.5**o
        ref[0,0,1,0]=(0.948969615551)/(o+1.)+(-0.173478478707)+(0.0756189154575)*0.5**o
        ref[0,0,1,1]=(-0.484419525942)/(o+1.)+(0.287761319651)+(-0.834046890353)*0.5**o
        ref[0,0,2,0]=(-0.275366085384)/(o+1.)+(-0.051636507926)+(0.534757021171)*0.5**o
        ref[0,0,2,1]=(0.338862631582)/(o+1.)+(0.076489529483)+(-0.47622767501)*0.5**o
        ref[0,1,0,0]=(0.960885993642)/(o+1.)+(0.36959166751)+(0.789134441414)*0.5**o
        ref[0,1,0,1]=(-0.101908900511)/(o+1.)+(0.343409131566)+(0.298411594744)*0.5**o
        ref[0,1,1,0]=(-0.50227382988)/(o+1.)+(-0.431549057432)+(-0.509965959533)*0.5**o
        ref[0,1,1,1]=(-0.294216689224)/(o+1.)+(0.293114907564)+(-0.00647512035965)*0.5**o
        ref[0,1,2,0]=(0.330313139948)/(o+1.)+(0.0601559902451)+(-0.447667920962)*0.5**o
        ref[0,1,2,1]=(-0.855325117375)/(o+1.)+(0.455706690618)+(0.0237326178252)*0.5**o
        ref[0,2,0,0]=(0.958948557218)/(o+1.)+(-0.307015973897)+(-0.752766039908)*0.5**o
        ref[0,2,0,1]=(0.622666086232)/(o+1.)+(0.417782845058)+(-0.676857571283)*0.5**o
        ref[0,2,1,0]=(-0.476269525636)/(o+1.)+(0.203518519787)+(-0.541895856898)*0.5**o
        ref[0,2,1,1]=(0.396862410812)/(o+1.)+(-0.318442063423)+(0.499722654464)*0.5**o
        ref[0,2,2,0]=(0.291546173382)/(o+1.)+(0.345688404218)+(0.372569619405)*0.5**o
        ref[0,2,2,1]=(-0.0863432016899)/(o+1.)+(0.296494638577)+(-0.631266836064)*0.5**o
        ref[0,3,0,0]=(0.437179077921)/(o+1.)+(-0.340515851088)+(-0.00622121586734)*0.5**o
        ref[0,3,0,1]=(-0.0670787919003)/(o+1.)+(-0.492735838895)+(0.931769856293)*0.5**o
        ref[0,3,1,0]=(-0.455620785864)/(o+1.)+(-0.866886545624)+(0.632565106759)*0.5**o
        ref[0,3,1,1]=(0.769395985412)/(o+1.)+(-0.303642587154)+(0.781062911223)*0.5**o
        ref[0,3,2,0]=(-0.338160377272)/(o+1.)+(-0.237545833658)+(-0.966712929597)*0.5**o
        ref[0,3,2,1]=(-0.796435534783)/(o+1.)+(0.544922318125)+(-0.743274373676)*0.5**o
        ref[0,4,0,0]=(0.452932120968)/(o+1.)+(-0.512874769603)+(-0.048932296957)*0.5**o
        ref[0,4,0,1]=(0.361579034112)/(o+1.)+(-0.0561290568189)+(0.365767627701)*0.5**o
        ref[0,4,1,0]=(-0.626053161593)/(o+1.)+(-0.761324946295)+(0.911283010254)*0.5**o
        ref[0,4,1,1]=(-0.0455977379835)/(o+1.)+(0.292122388548)+(0.22020429757)*0.5**o
        ref[0,4,2,0]=(-0.881052488675)/(o+1.)+(-0.210171400955)+(-0.973018746263)*0.5**o
        ref[0,4,2,1]=(0.955475719603)/(o+1.)+(0.367959093676)+(0.126904536122)*0.5**o
        ref[1,0,0,0]=(-0.314523133291)/(o+1.)+(-0.649561146707)+(-0.54959125554)*0.5**o
        ref[1,0,0,1]=(0.776726613012)/(o+1.)+(0.0747896120381)+(0.804907910029)*0.5**o
        ref[1,0,1,0]=(0.416309554773)/(o+1.)+(0.0650606132082)+(-0.795886225295)*0.5**o
        ref[1,0,1,1]=(-0.642083920221)/(o+1.)+(-0.518812997818)+(-0.590503261366)*0.5**o
        ref[1,0,2,0]=(0.209400082886)/(o+1.)+(-0.592256267485)+(0.00147357225359)*0.5**o
        ref[1,0,2,1]=(-0.969945221754)/(o+1.)+(-0.364559871358)+(-0.0931427766681)*0.5**o
        ref[1,1,0,0]=(0.941138659497)/(o+1.)+(-0.103485523138)+(-0.866286624489)*0.5**o
        ref[1,1,0,1]=(0.527395399401)/(o+1.)+(0.373358049173)+(-0.612019142395)*0.5**o
        ref[1,1,1,0]=(-0.276942103917)/(o+1.)+(0.624509599487)+(0.0329877871648)*0.5**o
        ref[1,1,1,1]=(-0.153477829221)/(o+1.)+(-0.0616268433725)+(-0.85593389444)*0.5**o
        ref[1,1,2,0]=(0.2685285297)/(o+1.)+(-0.301824551915)+(-0.386723203413)*0.5**o
        ref[1,1,2,1]=(-0.530960074652)/(o+1.)+(0.248621312801)+(-0.772378264474)*0.5**o
        ref[1,2,0,0]=(0.745071931955)/(o+1.)+(-0.149268301427)+(-0.000656345828701)*0.5**o
        ref[1,2,0,1]=(-0.581822428087)/(o+1.)+(0.2315691847)+(0.122913062738)*0.5**o
        ref[1,2,1,0]=(-0.325165099021)/(o+1.)+(-0.414758139139)+(0.187663931198)*0.5**o
        ref[1,2,1,1]=(-0.389683593964)/(o+1.)+(0.265985790556)+(0.485893691018)*0.5**o
        ref[1,2,2,0]=(0.661969003098)/(o+1.)+(-0.514584521701)+(-0.990427195841)*0.5**o
        ref[1,2,2,1]=(-0.0828384828188)/(o+1.)+(-0.482166570368)+(0.832805029032)*0.5**o
        ref[1,3,0,0]=(-0.756020671713)/(o+1.)+(0.627704058228)+(0.150404496577)*0.5**o
        ref[1,3,0,1]=(-0.93648010541)/(o+1.)+(0.11022703478)+(-0.387767702896)*0.5**o
        ref[1,3,1,0]=(-0.995823513997)/(o+1.)+(-0.242072805569)+(-0.604794178119)*0.5**o
        ref[1,3,1,1]=(-0.543071878541)/(o+1.)+(-0.160890620667)+(0.654757881656)*0.5**o
        ref[1,3,2,0]=(0.803184441579)/(o+1.)+(-0.612293875048)+(-0.504652146723)*0.5**o
        ref[1,3,2,1]=(-0.786714439322)/(o+1.)+(-0.175556007517)+(-0.18491128296)*0.5**o
        ref[1,4,0,0]=(-0.302928744298)/(o+1.)+(0.476360987754)+(-0.423396151472)*0.5**o
        ref[1,4,0,1]=(-0.410240602476)/(o+1.)+(-0.49551630458)+(0.40280698327)*0.5**o
        ref[1,4,1,0]=(-0.820677400346)/(o+1.)+(-0.573037933363)+(-0.752720066188)*0.5**o
        ref[1,4,1,1]=(-0.492748923658)/(o+1.)+(-0.600739806799)+(0.170290165507)*0.5**o
        ref[1,4,2,0]=(0.926654770824)/(o+1.)+(0.011282908181)+(-0.697106837662)*0.5**o
        ref[1,4,2,1]=(-0.958477382611)/(o+1.)+(0.618515858902)+(-0.65973097111)*0.5**o
        ref[2,0,0,0]=(-0.15636950624)/(o+1.)+(-0.520670547262)+(-0.36455690548)*0.5**o
        ref[2,0,0,1]=(-0.726424878286)/(o+1.)+(0.697757734621)+(-0.386898263585)*0.5**o
        ref[2,0,1,0]=(-0.055162393508)/(o+1.)+(0.247015646426)+(-0.442127640696)*0.5**o
        ref[2,0,1,1]=(-0.769332288875)/(o+1.)+(-0.0917175733456)+(-0.639003232373)*0.5**o
        ref[2,0,2,0]=(-0.196390038297)/(o+1.)+(0.0977907270548)+(0.340749124126)*0.5**o
        ref[2,0,2,1]=(0.841927481859)/(o+1.)+(-0.489185426146)+(0.0626606144906)*0.5**o
        ref[2,1,0,0]=(0.590018249148)/(o+1.)+(0.378476967182)+(-0.354886735451)*0.5**o
        ref[2,1,0,1]=(0.178495604742)/(o+1.)+(0.233073081069)+(0.572535849459)*0.5**o
        ref[2,1,1,0]=(-0.762671064919)/(o+1.)+(-0.108539686142)+(0.448831274081)*0.5**o
        ref[2,1,1,1]=(0.927591406535)/(o+1.)+(0.0441635703243)+(-0.53130361723)*0.5**o
        ref[2,1,2,0]=(-0.604360706259)/(o+1.)+(-0.134848102332)+(0.402913311063)*0.5**o
        ref[2,1,2,1]=(0.229727627441)/(o+1.)+(-0.323296604293)+(-0.197927111225)*0.5**o
        ref[2,2,0,0]=(0.287478356112)/(o+1.)+(0.762635506201)+(0.0282288847209)*0.5**o
        ref[2,2,0,1]=(0.243518182621)/(o+1.)+(-0.241502069199)+(0.743970373822)*0.5**o
        ref[2,2,1,0]=(-0.369049760378)/(o+1.)+(-0.21941778965)+(-0.472075045017)*0.5**o
        ref[2,2,1,1]=(-0.745343658666)/(o+1.)+(0.193375557775)+(0.864596248255)*0.5**o
        ref[2,2,2,0]=(-0.491769251494)/(o+1.)+(0.442742387618)+(0.753340954862)*0.5**o
        ref[2,2,2,1]=(0.551196377158)/(o+1.)+(-0.523535406613)+(-0.212166925203)*0.5**o
        ref[2,3,0,0]=(0.192271522168)/(o+1.)+(-0.0181953944687)+(0.122707705798)*0.5**o
        ref[2,3,0,1]=(-0.593768738594)/(o+1.)+(0.199088910545)+(0.368366498588)*0.5**o
        ref[2,3,1,0]=(0.0866211808983)/(o+1.)+(0.723512694127)+(0.521936735685)*0.5**o
        ref[2,3,1,1]=(-0.453288268254)/(o+1.)+(-0.15580970281)+(-0.655272797205)*0.5**o
        ref[2,3,2,0]=(0.614105802669)/(o+1.)+(0.0411427694898)+(0.85774098305)*0.5**o
        ref[2,3,2,1]=(0.263133533654)/(o+1.)+(-0.628776355867)+(-0.624497793507)*0.5**o
        ref[2,4,0,0]=(-0.809347914722)/(o+1.)+(0.367719412237)+(0.26671699779)*0.5**o
        ref[2,4,0,1]=(0.773124137566)/(o+1.)+(-0.442496395542)+(-0.0149946871367)*0.5**o
        ref[2,4,1,0]=(-0.927311525126)/(o+1.)+(-0.0125177466568)+(-0.943580084777)*0.5**o
        ref[2,4,1,1]=(0.306409665483)/(o+1.)+(0.128569553604)+(-0.852228134533)*0.5**o
        ref[2,4,2,0]=(0.926185522145)/(o+1.)+(0.0750538146738)+(-0.58517230367)*0.5**o
        ref[2,4,2,1]=(0.858245470922)/(o+1.)+(0.0125876600893)+(0.457971730734)*0.5**o
        ref[3,0,0,0]=(-0.74501947953)/(o+1.)+(0.343942243752)+(-0.363581155556)*0.5**o
        ref[3,0,0,1]=(0.850890737948)/(o+1.)+(0.157997002133)+(-0.761705466968)*0.5**o
        ref[3,0,1,0]=(-0.0468962248591)/(o+1.)+(0.0297980532763)+(0.682735456043)*0.5**o
        ref[3,0,1,1]=(0.0126326425456)/(o+1.)+(-0.076115296096)+(0.575387199213)*0.5**o
        ref[3,0,2,0]=(-0.410549752333)/(o+1.)+(0.85556811978)+(0.270867209104)*0.5**o
        ref[3,0,2,1]=(0.580487848545)/(o+1.)+(0.59069171729)+(0.698757148346)*0.5**o
        ref[3,1,0,0]=(0.139052775786)/(o+1.)+(-0.334747475889)+(0.281989056901)*0.5**o
        ref[3,1,0,1]=(0.591083610998)/(o+1.)+(-0.322680889668)+(-0.492254197357)*0.5**o
        ref[3,1,1,0]=(0.384713965427)/(o+1.)+(-0.522411606102)+(0.146558304882)*0.5**o
        ref[3,1,1,1]=(-0.827615582009)/(o+1.)+(-0.00295218575854)+(0.514501517225)*0.5**o
        ref[3,1,2,0]=(0.84762578765)/(o+1.)+(0.208361555685)+(-0.43502341494)*0.5**o
        ref[3,1,2,1]=(-0.554668615664)/(o+1.)+(0.825347835483)+(-0.899680107129)*0.5**o
        ref[3,2,0,0]=(0.652802712143)/(o+1.)+(0.397197582731)+(0.7807432266)*0.5**o
        ref[3,2,0,1]=(0.421495251937)/(o+1.)+(-0.119571710512)+(0.677076974975)*0.5**o
        ref[3,2,1,0]=(-0.230439630375)/(o+1.)+(-0.733165339017)+(-0.816334071941)*0.5**o
        ref[3,2,1,1]=(0.64153308098)/(o+1.)+(0.778374176432)+(-0.477170252948)*0.5**o
        ref[3,2,2,0]=(-0.987848717403)/(o+1.)+(-0.0606435888479)+(0.974551740798)*0.5**o
        ref[3,2,2,1]=(-0.318593711859)/(o+1.)+(-0.459904454409)+(0.334228278552)*0.5**o
        ref[3,3,0,0]=(-0.418115413351)/(o+1.)+(-0.348616063735)+(-0.982162917221)*0.5**o
        ref[3,3,0,1]=(-0.437684435015)/(o+1.)+(0.804145379606)+(-0.892393985366)*0.5**o
        ref[3,3,1,0]=(0.944100110135)/(o+1.)+(-0.059994562245)+(0.0694197471801)*0.5**o
        ref[3,3,1,1]=(-0.860656048613)/(o+1.)+(0.178136109535)+(-0.250126193893)*0.5**o
        ref[3,3,2,0]=(-0.363631991908)/(o+1.)+(-0.298384287282)+(0.471110648232)*0.5**o
        ref[3,3,2,1]=(-0.578021736479)/(o+1.)+(0.320913094365)+(0.235895941501)*0.5**o
        ref[3,4,0,0]=(-0.946995649946)/(o+1.)+(0.0451978887455)+(0.936359833225)*0.5**o
        ref[3,4,0,1]=(-0.636305818291)/(o+1.)+(0.258932088634)+(0.449663732234)*0.5**o
        ref[3,4,1,0]=(-0.823421937366)/(o+1.)+(0.112980674356)+(0.297559014947)*0.5**o
        ref[3,4,1,1]=(-0.441768022626)/(o+1.)+(-0.240840962617)+(-0.682298558949)*0.5**o
        ref[3,4,2,0]=(-0.258808936822)/(o+1.)+(0.510306577882)+(-0.457251344276)*0.5**o
        ref[3,4,2,1]=(-0.489365749016)/(o+1.)+(-0.296617385332)+(0.979513856171)*0.5**o
      else:
        arg[0,0,0,0]=(-0.474247989728)*x[0]+(-0.768689352235)*x[1]+(-1.08527900853)*x[2]
        arg[0,0,0,1]=(-0.237113797789)*x[0]+(0.0842735741018)*x[1]+(0.203858551029)*x[2]
        arg[0,0,1,0]=(0.24237127665)*x[0]+(0.421022955803)*x[1]+(-1.52818540927)*x[2]
        arg[0,0,1,1]=(1.34624865817)*x[0]+(-0.798943272818)*x[1]+(-0.823168386315)*x[2]
        arg[0,0,2,0]=(-0.906723771267)*x[0]+(0.328893123771)*x[1]+(-0.942770202463)*x[2]
        arg[0,0,2,1]=(1.54305197694)*x[0]+(0.813063941984)*x[1]+(0.205532727168)*x[2]
        arg[0,1,0,0]=(0.399159979872)*x[0]+(-1.52089837934)*x[1]+(-1.4064251406)*x[2]
        arg[0,1,0,1]=(-0.121139354731)*x[0]+(1.12768246422)*x[1]+(0.234299409411)*x[2]
        arg[0,1,1,0]=(-1.62513899388)*x[0]+(-0.173131700583)*x[1]+(1.24199996799)*x[2]
        arg[0,1,1,1]=(-0.371929434177)*x[0]+(-0.606278610409)*x[1]+(-0.00600790063066)*x[2]
        arg[0,1,2,0]=(-0.804531294423)*x[0]+(0.869392248002)*x[1]+(0.840417359943)*x[2]
        arg[0,1,2,1]=(0.329452946275)*x[0]+(1.1072495446)*x[1]+(-0.213517932323)*x[2]
        arg[0,2,0,0]=(1.02784887224)*x[0]+(0.435517042337)*x[1]+(1.58767206677)*x[2]
        arg[0,2,0,1]=(-0.0120997177)*x[0]+(0.430135803314)*x[1]+(-0.717473068956)*x[2]
        arg[0,2,1,0]=(0.955455517302)*x[0]+(0.712976918968)*x[1]+(-0.624804508641)*x[2]
        arg[0,2,1,1]=(-1.03293986594)*x[0]+(-0.539062834482)*x[1]+(-1.61172701428)*x[2]
        arg[0,2,2,0]=(-0.177662810138)*x[0]+(0.553352366127)*x[1]+(0.695417121111)*x[2]
        arg[0,2,2,1]=(0.318133474679)*x[0]+(-0.452528947926)*x[1]+(0.468602505091)*x[2]
        arg[0,3,0,0]=(-0.294467725916)*x[0]+(1.67314409894)*x[1]+(0.100736440007)*x[2]
        arg[0,3,0,1]=(-0.517162349887)*x[0]+(-1.57430191254)*x[1]+(1.24045877402)*x[2]
        arg[0,3,1,0]=(0.667393640368)*x[0]+(0.340993632212)*x[1]+(-1.09920535559)*x[2]
        arg[0,3,1,1]=(0.37451406727)*x[0]+(0.37350649326)*x[1]+(0.357440871322)*x[2]
        arg[0,3,2,0]=(-0.989366398194)*x[0]+(0.596179154759)*x[1]+(-0.573280308087)*x[2]
        arg[0,3,2,1]=(-0.25561190141)*x[0]+(1.42296905238)*x[1]+(-1.21032031399)*x[2]
        arg[0,4,0,0]=(-0.41394085285)*x[0]+(-0.751213929646)*x[1]+(-0.68470718293)*x[2]
        arg[0,4,0,1]=(0.702985204167)*x[0]+(-0.0375821582587)*x[1]+(0.290630803819)*x[2]
        arg[0,4,1,0]=(0.0558204653441)*x[0]+(1.61940057302)*x[1]+(-0.518863528354)*x[2]
        arg[0,4,1,1]=(1.25876303604)*x[0]+(-1.40923202908)*x[1]+(1.05202333762)*x[2]
        arg[0,4,2,0]=(0.0715295483294)*x[0]+(0.883869702529)*x[1]+(-0.463253474408)*x[2]
        arg[0,4,2,1]=(0.564563789615)*x[0]+(-0.135049742988)*x[1]+(-1.3118878567)*x[2]
        arg[1,0,0,0]=(0.707267335291)*x[0]+(-0.197244449872)*x[1]+(-0.114362773781)*x[2]
        arg[1,0,0,1]=(-0.144762815659)*x[0]+(0.434177706452)*x[1]+(0.0754859099691)*x[2]
        arg[1,0,1,0]=(-0.906934991359)*x[0]+(0.179011995627)*x[1]+(0.823936036469)*x[2]
        arg[1,0,1,1]=(1.11165724605)*x[0]+(0.48023223731)*x[1]+(0.981353808181)*x[2]
        arg[1,0,2,0]=(0.499460583481)*x[0]+(0.171586243561)*x[1]+(-0.252474775657)*x[2]
        arg[1,0,2,1]=(-0.604289370884)*x[0]+(1.11610914086)*x[1]+(0.747439062711)*x[2]
        arg[1,1,0,0]=(0.696191463882)*x[0]+(0.349547714404)*x[1]+(0.651361019042)*x[2]
        arg[1,1,0,1]=(0.152679923426)*x[0]+(-0.448442911425)*x[1]+(1.33964371501)*x[2]
        arg[1,1,1,0]=(-0.74612592015)*x[0]+(-1.27089239786)*x[1]+(-0.261725631736)*x[2]
        arg[1,1,1,1]=(1.50839493929)*x[0]+(1.93229724987)*x[1]+(-0.963996845831)*x[2]
        arg[1,1,2,0]=(0.867984447375)*x[0]+(-0.777493811854)*x[1]+(0.847847553133)*x[2]
        arg[1,1,2,1]=(1.34249389498)*x[0]+(0.776160804062)*x[1]+(-0.295393972199)*x[2]
        arg[1,2,0,0]=(-0.655771613243)*x[0]+(1.58220214455)*x[1]+(-0.406292477629)*x[2]
        arg[1,2,0,1]=(0.59294895986)*x[0]+(0.544961164723)*x[1]+(0.634279670948)*x[2]
        arg[1,2,1,0]=(1.15324179862)*x[0]+(0.645764835217)*x[1]+(0.670364725309)*x[2]
        arg[1,2,1,1]=(-0.819459106271)*x[0]+(0.024815625404)*x[1]+(0.769753650676)*x[2]
        arg[1,2,2,0]=(1.19042366161)*x[0]+(0.982020548972)*x[1]+(0.92730609738)*x[2]
        arg[1,2,2,1]=(-0.742441488349)*x[0]+(-1.3101429812)*x[1]+(-0.198104246324)*x[2]
        arg[1,3,0,0]=(0.349205030437)*x[0]+(-0.745240339596)*x[1]+(-0.744742092259)*x[2]
        arg[1,3,0,1]=(0.993101296711)*x[0]+(-1.48331355036)*x[1]+(-1.00616778677)*x[2]
        arg[1,3,1,0]=(-1.10586442824)*x[0]+(1.71922449779)*x[1]+(-0.943451153259)*x[2]
        arg[1,3,1,1]=(-1.72393763035)*x[0]+(-1.14113495956)*x[1]+(-0.211909615809)*x[2]
        arg[1,3,2,0]=(-0.378302963108)*x[0]+(-0.494705322002)*x[1]+(0.940164363332)*x[2]
        arg[1,3,2,1]=(0.850541148743)*x[0]+(-1.45461945468)*x[1]+(1.20025821328)*x[2]
        arg[1,4,0,0]=(1.33882463795)*x[0]+(0.475208133119)*x[1]+(0.722686059253)*x[2]
        arg[1,4,0,1]=(0.0750422155969)*x[0]+(1.29758288845)*x[1]+(-1.41431668738)*x[2]
        arg[1,4,1,0]=(1.36072344575)*x[0]+(-0.345743547563)*x[1]+(-0.63248018754)*x[2]
        arg[1,4,1,1]=(0.579049229913)*x[0]+(0.954245074078)*x[1]+(-0.170538926965)*x[2]
        arg[1,4,2,0]=(1.67615484634)*x[0]+(0.561067019023)*x[1]+(-1.26952326914)*x[2]
        arg[1,4,2,1]=(0.700910547934)*x[0]+(0.58717328764)*x[1]+(0.795138196965)*x[2]
        arg[2,0,0,0]=(0.763193592359)*x[0]+(-0.397549328381)*x[1]+(-0.230593559545)*x[2]
        arg[2,0,0,1]=(0.454026153468)*x[0]+(-1.12464443717)*x[1]+(-0.147462098246)*x[2]
        arg[2,0,1,0]=(-0.276457034935)*x[0]+(0.118173156654)*x[1]+(0.57296366275)*x[2]
        arg[2,0,1,1]=(1.31887812253)*x[0]+(-0.544644483254)*x[1]+(0.0013728350582)*x[2]
        arg[2,0,2,0]=(-0.257589384265)*x[0]+(-0.016005453203)*x[1]+(-0.508836430965)*x[2]
        arg[2,0,2,1]=(-0.259420301773)*x[0]+(0.122645293168)*x[1]+(0.381613885289)*x[2]
        arg[2,1,0,0]=(-1.64168512418)*x[0]+(0.025182006192)*x[1]+(0.0268780458975)*x[2]
        arg[2,1,0,1]=(-0.0885344200898)*x[0]+(0.408795219114)*x[1]+(0.635798801606)*x[2]
        arg[2,1,1,0]=(-0.959658503283)*x[0]+(1.034415556)*x[1]+(0.725681105748)*x[2]
        arg[2,1,1,1]=(1.59312707789)*x[0]+(-0.116514952101)*x[1]+(-0.86121633507)*x[2]
        arg[2,1,2,0]=(-1.44859572067)*x[0]+(-1.28344247275)*x[1]+(0.299296401869)*x[2]
        arg[2,1,2,1]=(-0.488424372501)*x[0]+(1.23481402288)*x[1]+(0.01702902463)*x[2]
        arg[2,2,0,0]=(-1.29986951244)*x[0]+(-1.20190912751)*x[1]+(-0.755848291051)*x[2]
        arg[2,2,0,1]=(-0.52421656312)*x[0]+(-0.335471509689)*x[1]+(-1.01264312499)*x[2]
        arg[2,2,1,0]=(-1.12066642609)*x[0]+(-1.02138558053)*x[1]+(-1.00777755552)*x[2]
        arg[2,2,1,1]=(-1.42127024744)*x[0]+(1.63480022479)*x[1]+(0.542244906643)*x[2]
        arg[2,2,2,0]=(-0.418852880297)*x[0]+(-0.983110436453)*x[1]+(-0.622603673805)*x[2]
        arg[2,2,2,1]=(0.727366446825)*x[0]+(-0.256324856361)*x[1]+(-0.20608740758)*x[2]
        arg[2,3,0,0]=(0.548268302883)*x[0]+(0.0102173127665)*x[1]+(0.869833576217)*x[2]
        arg[2,3,0,1]=(-1.70159091061)*x[0]+(1.15876875083)*x[1]+(-0.234757706773)*x[2]
        arg[2,3,1,0]=(-1.72817024817)*x[0]+(0.690055132218)*x[1]+(-0.149398998825)*x[2]
        arg[2,3,1,1]=(1.80829412387)*x[0]+(-0.217899654855)*x[1]+(-0.922672217822)*x[2]
        arg[2,3,2,0]=(1.23855200328)*x[0]+(0.412795411508)*x[1]+(0.960476191755)*x[2]
        arg[2,3,2,1]=(-1.43061904621)*x[0]+(-0.766039000745)*x[1]+(-1.25765397361)*x[2]
        arg[2,4,0,0]=(-0.660872016687)*x[0]+(-0.000405233718708)*x[1]+(-0.178507968219)*x[2]
        arg[2,4,0,1]=(-0.168526882871)*x[0]+(-1.06936802685)*x[1]+(1.54083857104)*x[2]
        arg[2,4,1,0]=(0.565807359529)*x[0]+(-1.41401297939)*x[1]+(-1.23254260537)*x[2]
        arg[2,4,1,1]=(-0.373920075802)*x[0]+(0.407330812903)*x[1]+(-0.204667001328)*x[2]
        arg[2,4,2,0]=(0.874100879885)*x[0]+(-0.191238094267)*x[1]+(1.54890969328)*x[2]
        arg[2,4,2,1]=(0.566428125313)*x[0]+(-1.60899595601)*x[1]+(-1.05112529768)*x[2]
        arg[3,0,0,0]=(-0.688037248111)*x[0]+(-1.56483484784)*x[1]+(0.26782504619)*x[2]
        arg[3,0,0,1]=(0.418788954459)*x[0]+(-0.670455226092)*x[1]+(-0.0562760591375)*x[2]
        arg[3,0,1,0]=(-0.2614362754)*x[0]+(0.271050519855)*x[1]+(0.227811982923)*x[2]
        arg[3,0,1,1]=(-0.873198928647)*x[0]+(-0.483763141395)*x[1]+(0.33532383979)*x[2]
        arg[3,0,2,0]=(-0.864161938176)*x[0]+(-1.4682754386)*x[1]+(-0.38126411162)*x[2]
        arg[3,0,2,1]=(0.142827521381)*x[0]+(0.464483940188)*x[1]+(0.214297576606)*x[2]
        arg[3,1,0,0]=(0.188747079113)*x[0]+(0.590821253713)*x[1]+(0.908850367108)*x[2]
        arg[3,1,0,1]=(0.0587541582624)*x[0]+(0.955494266151)*x[1]+(1.70822209849)*x[2]
        arg[3,1,1,0]=(-0.420356861523)*x[0]+(-0.551445918262)*x[1]+(-0.0187114754269)*x[2]
        arg[3,1,1,1]=(1.07071658401)*x[0]+(-0.589035602194)*x[1]+(0.133725074027)*x[2]
        arg[3,1,2,0]=(0.370477043767)*x[0]+(0.226262353618)*x[1]+(0.566087784489)*x[2]
        arg[3,1,2,1]=(1.77828660075)*x[0]+(0.607419742685)*x[1]+(0.0297054744582)*x[2]
        arg[3,2,0,0]=(-1.19326939474)*x[0]+(1.20875153887)*x[1]+(-0.124756222091)*x[2]
        arg[3,2,0,1]=(0.0645190914271)*x[0]+(0.357580475716)*x[1]+(-1.59461469718)*x[2]
        arg[3,2,1,0]=(1.4853619905)*x[0]+(0.751582301648)*x[1]+(0.859363177167)*x[2]
        arg[3,2,1,1]=(1.36658655075)*x[0]+(-0.776650996613)*x[1]+(0.654743994261)*x[2]
        arg[3,2,2,0]=(0.406696005677)*x[0]+(-0.528546876384)*x[1]+(-0.439775325396)*x[2]
        arg[3,2,2,1]=(-0.725198159461)*x[0]+(-0.0933136564416)*x[1]+(-0.720365141393)*x[2]
        arg[3,3,0,0]=(-0.428669112669)*x[0]+(0.626162511175)*x[1]+(-0.382776233923)*x[2]
        arg[3,3,0,1]=(0.340474939218)*x[0]+(-0.140550383925)*x[1]+(0.353836983746)*x[2]
        arg[3,3,1,0]=(0.851646816711)*x[0]+(0.0171684195949)*x[1]+(0.38173044322)*x[2]
        arg[3,3,1,1]=(0.359647722225)*x[0]+(0.425799941165)*x[1]+(-0.80174052212)*x[2]
        arg[3,3,2,0]=(0.13012568736)*x[0]+(-0.573735693651)*x[1]+(1.08843730088)*x[2]
        arg[3,3,2,1]=(-1.01372953294)*x[0]+(0.335282956856)*x[1]+(1.24570512628)*x[2]
        arg[3,4,0,0]=(-0.05674609039)*x[0]+(-0.814879575064)*x[1]+(-0.169789408264)*x[2]
        arg[3,4,0,1]=(1.5968230938)*x[0]+(0.0679819277023)*x[1]+(0.892284314512)*x[2]
        arg[3,4,1,0]=(1.22220298478)*x[0]+(-0.0258417926472)*x[1]+(0.440670138568)*x[2]
        arg[3,4,1,1]=(0.647072768716)*x[0]+(-0.177651087906)*x[1]+(-0.115553644646)*x[2]
        arg[3,4,2,0]=(1.29816363306)*x[0]+(-1.18656044167)*x[1]+(-0.465776533157)*x[2]
        arg[3,4,2,1]=(0.328907496091)*x[0]+(-0.179857937151)*x[1]+(-0.476306938306)*x[2]
        ref[0,0,0,0]=(-0.530378359482)/(o+1.)+(-0.935143327523)+(0.0724486640358)*0.5**o
        ref[0,0,0,1]=(1.22837847871)/(o+1.)+(-0.113350954728)+(-0.950658241915)*0.5**o
        ref[0,0,1,0]=(-0.000414683535449)/(o+1.)+(-0.183158731552)+(-0.49805903018)*0.5**o
        ref[0,0,1,1]=(-0.882730389132)/(o+1.)+(-0.112174958816)+(0.831217305802)*0.5**o
        ref[0,0,2,0]=(-1.1913557966)/(o+1.)+(-0.0615750891074)+(-0.206094875146)*0.5**o
        ref[0,0,2,1]=(0.634536492589)/(o+1.)+(0.649890059464)+(0.627332034578)*0.5**o
        ref[0,1,0,0]=(-1.38665625775)/(o+1.)+(-1.0210653524)+(0.900623422471)*0.5**o
        ref[0,1,0,1]=(1.20633678024)/(o+1.)+(0.0888193121856)+(-0.143132885707)*0.5**o
        ref[0,1,1,0]=(0.452225278035)/(o+1.)+(-0.148926943801)+(-0.710642116901)*0.5**o
        ref[0,1,1,1]=(-0.244223430204)/(o+1.)+(-0.619607191996)+(0.499221868978)*0.5**o
        ref[0,1,2,0]=(0.0383849036606)/(o+1.)+(0.759484971644)+(-0.652076533427)*0.5**o
        ref[0,1,2,1]=(0.207558055759)/(o+1.)+(0.502305382384)+(0.0110157380215)*0.5**o
        ref[0,2,0,0]=(0.761374280144)/(o+1.)+(0.777616611515)+(0.734430478174)*0.5**o
        ref[0,2,0,1]=(0.554502862028)/(o+1.)+(-0.409274602016)+(-0.0353906413371)*0.5**o
        ref[0,2,1,0]=(-0.252543841709)/(o+1.)+(0.386640377894)+(0.522891013551)*0.5**o
        ref[0,2,1,1]=(-1.52497235887)/(o+1.)+(-0.807603254401)+(-0.0435508470367)*0.5**o
        ref[0,2,2,0]=(0.172485606257)/(o+1.)+(0.350297526187)+(0.19802601847)*0.5**o
        ref[0,2,2,1]=(-0.141271359379)/(o+1.)+(0.0509198134622)+(0.373638764298)*0.5**o
        ref[0,3,0,0]=(1.31278836809)/(o+1.)+(-0.0918739323256)+(0.350372309595)*0.5**o
        ref[0,3,0,1]=(-0.396933928109)/(o+1.)+(-0.436137805999)+(0.418204051696)*0.5**o
        ref[0,3,1,0]=(-0.206619840099)/(o+1.)+(-0.118703289798)+(0.353208336689)*0.5**o
        ref[0,3,1,1]=(-0.595153371293)/(o+1.)+(0.615722581562)+(0.469169640021)*0.5**o
        ref[0,3,2,0]=(-0.680858997986)/(o+1.)+(-0.0393223937964)+(-0.206963765944)*0.5**o
        ref[0,3,2,1]=(0.012753264598)/(o+1.)+(0.246188741489)+(-0.548093910595)*0.5**o
        ref[0,4,0,0]=(-0.40478987122)/(o+1.)+(-0.890399681609)+(0.33572726901)*0.5**o
        ref[0,4,0,1]=(0.0280062557264)/(o+1.)+(0.0358049413228)+(0.856417711355)*0.5**o
        ref[0,4,1,0]=(0.181560004565)/(o+1.)+(0.329885369854)+(0.31502676574)*0.5**o
        ref[0,4,1,1]=(0.0552681276841)/(o+1.)+(0.00372919426319)+(0.838827828375)*0.5**o
        ref[0,4,2,0]=(0.656057162724)/(o+1.)+(0.179315527684)+(-0.522542441641)*0.5**o
        ref[0,4,2,1]=(0.126189389317)/(o+1.)+(-0.429452001643)+(-0.149659196108)*0.5**o
        ref[1,0,0,0]=(1.43443485527)/(o+1.)+(-0.55091754491)+(0.0630603461901)*0.5**o
        ref[1,0,0,1]=(-0.612191779306)/(o+1.)+(0.604025235381)+(-0.230957890695)*0.5**o
        ref[1,0,1,0]=(0.373085423681)/(o+1.)+(-0.0817391530697)+(-0.113594076805)*0.5**o
        ref[1,0,1,1]=(0.761862677657)/(o+1.)+(0.819524484428)+(0.172331645033)*0.5**o
        ref[1,0,2,0]=(0.929265450746)/(o+1.)+(-0.382585784938)+(0.254478170515)*0.5**o
        ref[1,0,2,1]=(1.58236203463)/(o+1.)+(-0.280641188811)+(0.238179175679)*0.5**o
        ref[1,1,0,0]=(-0.117055726312)/(o+1.)+(0.957914090571)+(-0.101672257501)*0.5**o
        ref[1,1,0,1]=(0.0849062553796)/(o+1.)+(0.0703564566846)+(0.818261558262)*0.5**o
        ref[1,1,1,0]=(-1.72219113622)/(o+1.)+(-0.357454724939)+(0.15835663635)*0.5**o
        ref[1,1,1,1]=(0.739078400468)/(o+1.)+(0.532292543169)+(0.673031856531)*0.5**o
        ref[1,1,2,0]=(0.255023909903)/(o+1.)+(0.229656917655)+(0.22400044344)*0.5**o
        ref[1,1,2,1]=(0.518856765199)/(o+1.)+(0.380552790125)+(0.543298381396)*0.5**o
        ref[1,2,0,0]=(0.0321563884831)/(o+1.)+(0.25981567762)+(-0.0316496900482)*0.5**o
        ref[1,2,0,1]=(1.09318176771)/(o+1.)+(0.0572442907603)+(0.564519446305)*0.5**o
        ref[1,2,1,0]=(0.756962791387)/(o+1.)+(0.69419429624)+(0.32401997528)*0.5**o
        ref[1,2,1,1]=(1.28480299241)/(o+1.)+(-0.201990593541)+(-0.905711635518)*0.5**o
        ref[1,2,2,0]=(1.04969566442)/(o+1.)+(0.72854097372)+(0.592972696101)*0.5**o
        ref[1,2,2,1]=(-1.77086264775)/(o+1.)+(-0.261914180361)+(0.0440022926066)*0.5**o
        ref[1,3,0,0]=(-0.477780531124)/(o+1.)+(-0.181354085163)+(-0.300288699967)*0.5**o
        ref[1,3,0,1]=(-0.797627013456)/(o+1.)+(-0.523163955197)+(0.347574883435)*0.5**o
        ref[1,3,1,0]=(0.428806106821)/(o+1.)+(-0.0514584302394)+(-0.655980330052)*0.5**o
        ref[1,3,1,1]=(-0.141740459457)/(o+1.)+(-1.05699960931)+(-0.821242527644)*0.5**o
        ref[1,3,2,0]=(0.0181810985873)/(o+1.)+(-0.180697139553)+(0.41036925874)*0.5**o
        ref[1,3,2,1]=(-0.061077367)/(o+1.)+(0.250077446763)+(0.157102380812)*0.5**o
        ref[1,4,0,0]=(1.14087212913)/(o+1.)+(0.368059029864)+(0.659728641463)*0.5**o
        ref[1,4,0,1]=(0.365658713316)/(o+1.)+(0.088533821044)+(-0.584417938734)*0.5**o
        ref[1,4,1,0]=(-1.11681308321)/(o+1.)+(0.438275560627)+(0.622761672604)*0.5**o
        ref[1,4,1,1]=(1.18893372018)/(o+1.)+(0.172934240923)+(-0.172046824999)*0.5**o
        ref[1,4,2,0]=(0.16240489493)/(o+1.)+(-0.0145923852256)+(0.834478471749)*0.5**o
        ref[1,4,2,1]=(0.571355829378)/(o+1.)+(0.526896417749)+(0.458073367663)*0.5**o
        ref[2,0,0,0]=(-0.489234572937)/(o+1.)+(-0.0135589093901)+(0.65140309615)*0.5**o
        ref[2,0,0,1]=(-1.55383491931)/(o+1.)+(0.0161644619972)+(0.703425613366)*0.5**o
        ref[2,0,1,0]=(-0.258604855723)/(o+1.)+(0.0827812678671)+(0.507722104457)*0.5**o
        ref[2,0,1,1]=(0.0504178529481)/(o+1.)+(0.182991850332)+(0.359204920726)*0.5**o
        ref[2,0,2,0]=(0.652043479841)/(o+1.)+(-0.443015450013)+(-0.548443848248)*0.5**o
        ref[2,0,2,1]=(0.340317839857)/(o+1.)+(-0.379565594545)+(0.663652225917)*0.5**o
        ref[2,1,0,0]=(-0.587863196667)/(o+1.)+(-0.0137797674284)+(-0.974202340562)*0.5**o
        ref[2,1,0,1]=(1.00030747039)/(o+1.)+(-0.399997754768)+(0.755747639771)*0.5**o
        ref[2,1,1,0]=(0.703413281036)/(o+1.)+(0.487016208736)+(-0.877007540039)*0.5**o
        ref[2,1,1,1]=(0.263166830572)/(o+1.)+(-0.157259452849)+(0.666747865841)*0.5**o
        ref[2,1,2,0]=(0.104532343004)/(o+1.)+(-1.02548001493)+(-0.486314104701)*0.5**o
        ref[2,1,2,1]=(0.933518655478)/(o+1.)+(0.0369928229139)+(-0.244085626298)*0.5**o
        ref[2,2,0,0]=(-1.50550384519)/(o+1.)+(-0.590996227594)+(-0.570130630622)*0.5**o
        ref[2,2,0,1]=(-0.60318683839)/(o+1.)+(-0.38308245528)+(-0.502979448853)*0.5**o
        ref[2,2,1,0]=(-1.11258910879)/(o+1.)+(-0.518777277969)+(-0.999685897413)*0.5**o
        ref[2,2,1,1]=(1.84366617107)/(o+1.)+(-0.232866738261)+(-0.622157810558)*0.5**o
        ref[2,2,2,0]=(-0.620181988585)/(o+1.)+(-0.836162444183)+(0.267939886396)*0.5**o
        ref[2,2,2,1]=(-0.826647389099)/(o+1.)+(0.0929536518109)+(0.905694268362)*0.5**o
        ref[2,3,0,0]=(0.184067674443)/(o+1.)+(0.259745858757)+(0.724759799909)*0.5**o
        ref[2,3,0,1]=(0.0175595303049)/(o+1.)+(0.0560117358912)+(-0.907162868639)*0.5**o
        ref[2,3,1,0]=(-0.609443154617)/(o+1.)+(0.186540819401)+(-0.951152598958)*0.5**o
        ref[2,3,1,1]=(-1.61858082094)/(o+1.)+(0.679439858578)+(0.927423354971)*0.5**o
        ref[2,3,2,0]=(0.44350652486)/(o+1.)+(0.81466177481)+(0.53899353207)*0.5**o
        ref[2,3,2,1]=(-0.877289862245)/(o+1.)+(-1.05742260849)+(-0.462176941342)*0.5**o
        ref[2,4,0,0]=(-0.736381416539)/(o+1.)+(-0.0139374259612)+(-0.0755289501641)*0.5**o
        ref[2,4,0,1]=(0.690429326999)/(o+1.)+(-0.0293644018251)+(-0.328756862036)*0.5**o
        ref[2,4,1,0]=(-1.57117034002)/(o+1.)+(-0.531463765387)+(0.553349645561)*0.5**o
        ref[2,4,1,1]=(1.24898175282)/(o+1.)+(-0.452273024959)+(-0.515691967125)*0.5**o
        ref[2,4,2,0]=(0.239126191643)/(o+1.)+(0.722504563932)+(0.547637159392)*0.5**o
        ref[2,4,2,1]=(-1.40763254209)/(o+1.)+(-0.809456441012)+(0.932852295739)*0.5**o
        ref[3,0,0,0]=(-0.416554297686)/(o+1.)+(-0.632193610911)+(-0.304105530258)*0.5**o
        ref[3,0,0,1]=(-1.02983448036)/(o+1.)+(0.579212231778)+(-0.436532313966)*0.5**o
        ref[3,0,1,0]=(0.42124133216)/(o+1.)+(0.368290418827)+(-0.920395942436)*0.5**o
        ref[3,0,1,1]=(0.158195474854)/(o+1.)+(-0.17459474514)+(-0.830644214826)*0.5**o
        ref[3,0,2,0]=(-1.18749647329)/(o+1.)+(-0.506173996178)+(-0.513857022748)*0.5**o
        ref[3,0,2,1]=(0.233437753018)/(o+1.)+(0.455705145371)+(-0.323239005586)*0.5**o
        ref[3,1,0,0]=(1.42991958566)/(o+1.)+(0.053769921022)+(0.150959272234)*0.5**o
        ref[3,1,0,1]=(1.48423731111)/(o+1.)+(0.964048315997)+(-0.689863420196)*0.5**o
        ref[3,1,1,0]=(0.868837676765)/(o+1.)+(-1.01261543254)+(0.165878933098)*0.5**o
        ref[3,1,1,1]=(0.820750307831)/(o+1.)+(-0.300930223821)+(0.396516195659)*0.5**o
        ref[3,1,2,0]=(0.477077782202)/(o+1.)+(-0.0416336989494)+(0.769016797571)*0.5**o
        ref[3,1,2,1]=(0.359895440587)/(o+1.)+(0.555322395811)+(0.944871585683)*0.5**o
        ref[3,2,0,0]=(0.657668315832)/(o+1.)+(-0.240321895174)+(-0.286298603451)*0.5**o
        ref[3,2,0,1]=(-0.941360568282)/(o+1.)+(-0.0246544136593)+(-0.181845734439)*0.5**o
        ref[3,2,1,0]=(1.60316834971)/(o+1.)+(0.252291602298)+(0.988555915005)*0.5**o
        ref[3,2,1,1]=(0.833811957678)/(o+1.)+(-0.111970621871)+(0.634808834458)*0.5**o
        ref[3,2,2,0]=(-1.38877073784)/(o+1.)+(0.538356636307)+(-0.249568730877)*0.5**o
        ref[3,2,2,1]=(-0.311839184097)/(o+1.)+(-0.698114103264)+(0.169190433329)*0.5**o
        ref[3,3,0,0]=(0.743170989696)/(o+1.)+(-0.499224746788)+(0.0699956684628)*0.5**o
        ref[3,3,0,1]=(-0.293538786051)/(o+1.)+(0.26667992866)+(0.313940467768)*0.5**o
        ref[3,3,1,0]=(0.951483461345)/(o+1.)+(0.148317904561)+(0.00242640905887)*0.5**o
        ref[3,3,1,1]=(-0.110753912776)/(o+1.)+(-0.44308689138)+(0.980634836805)*0.5**o
        ref[3,3,2,0]=(-0.337895223544)/(o+1.)+(0.358941646037)+(0.264839226057)*0.5**o
        ref[3,3,2,1]=(0.857823611315)/(o+1.)+(0.30404611512)+(-0.898657291359)*0.5**o
        ref[3,4,0,0]=(0.037944509376)/(o+1.)+(-0.364801700413)+(-0.349756182268)*0.5**o
        ref[3,4,0,1]=(1.01114399396)/(o+1.)+(0.457103279485)+(0.631738783076)*0.5**o
        ref[3,4,1,0]=(0.969931913888)/(o+1.)+(-0.165922085756)+(0.998943588321)*0.5**o
        ref[3,4,1,1]=(1.35441114248)/(o+1.)+(-0.502422727199)+(0.00430234808496)*0.5**o
        ref[3,4,2,0]=(-0.367201097566)/(o+1.)+(-0.337780780531)+(0.688589316866)*0.5**o
        ref[3,4,2,1]=(-0.0547905846061)/(o+1.)+(-0.293870420685)+(0.31527404661)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactOne_fromData_FunctionOnContactOne_rank0(self):
      """
      tests integral of rank 0 Data on the FunctionOnContactOne

      assumptions: FunctionOnContactOne(self.domain) exists
                   self.domain supports integral on FunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      w=FunctionOnContactOne(self.domain)
      x=w.getX()
      arg=Data(0,(),w)
      if dim==2:
        arg=(0.311787641998)*x[0]**o+(0.161673282875)*x[0]+(0.32570909237)*x[1]**o+(-0.249404742764)*x[1]
        ref=(0.32570909237)/(o+1.)+(-0.0438657299447)+(0.311787641998)*0.5**o
      else:
        arg=(0.0675036998003)*x[0]**o+(-0.247751392136)*x[0]+(0.791676220029)*x[1]**o+(-0.269015236163)*x[1]+(0.265735156875)*x[2]**o+(0.820319192519)*x[2]
        ref=(1.0574113769)/(o+1.)+(0.15177628211)+(0.0675036998003)*0.5**o
      res=integrate(arg)
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactOne_fromData_FunctionOnContactOne_rank1(self):
      """
      tests integral of rank 1 Data on the FunctionOnContactOne

      assumptions: FunctionOnContactOne(self.domain) exists
                   self.domain supports integral on FunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      w=FunctionOnContactOne(self.domain)
      x=w.getX()
      arg=Data(0,(2,),w)
      ref=numpy.zeros((2,),numpy.float_)
      if dim==2:
        arg[0]=(0.99235950851)*x[0]**o+(0.00867392831518)*x[0]+(-0.598024390869)*x[1]**o+(0.00945232777857)*x[1]
        arg[1]=(-0.140261338996)*x[0]**o+(-0.166524016889)*x[0]+(0.59850076881)*x[1]**o+(-0.805756215354)*x[1]
        ref[0]=(-0.598024390869)/(o+1.)+(0.00906312804687)+(0.99235950851)*0.5**o
        ref[1]=(0.59850076881)/(o+1.)+(-0.486140116122)+(-0.140261338996)*0.5**o
      else:
        arg[0]=(-0.158686556523)*x[0]**o+(0.249193164275)*x[0]+(-0.801383648208)*x[1]**o+(-0.439021533504)*x[1]+(-0.112035191366)*x[2]**o+(-0.323385862625)*x[2]
        arg[1]=(0.392997873484)*x[0]**o+(-0.867468854104)*x[0]+(0.804459243476)*x[1]**o+(0.129028860314)*x[1]+(-0.252715502862)*x[2]**o+(0.457355923469)*x[2]
        ref[0]=(-0.913418839574)/(o+1.)+(-0.256607115927)+(-0.158686556523)*0.5**o
        ref[1]=(0.551743740614)/(o+1.)+(-0.140542035161)+(0.392997873484)*0.5**o
      res=integrate(arg)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactOne_fromData_FunctionOnContactOne_rank2(self):
      """
      tests integral of rank 2 Data on the FunctionOnContactOne

      assumptions: FunctionOnContactOne(self.domain) exists
                   self.domain supports integral on FunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      w=FunctionOnContactOne(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5),w)
      ref=numpy.zeros((4, 5),numpy.float_)
      if dim==2:
        arg[0,0]=(0.330520725992)*x[0]**o+(0.160118740246)*x[0]+(-0.178670895269)*x[1]**o+(-0.281660040648)*x[1]
        arg[0,1]=(-0.871801084852)*x[0]**o+(0.692549035429)*x[0]+(-0.265102051042)*x[1]**o+(-0.799752901882)*x[1]
        arg[0,2]=(-0.816985024574)*x[0]**o+(0.534242721856)*x[0]+(0.81492666918)*x[1]**o+(0.891189158633)*x[1]
        arg[0,3]=(-0.20728122917)*x[0]**o+(-0.656873565737)*x[0]+(-0.924660993906)*x[1]**o+(0.453699593426)*x[1]
        arg[0,4]=(-0.541377726375)*x[0]**o+(0.916287049432)*x[0]+(-0.15879052465)*x[1]**o+(-0.319875161126)*x[1]
        arg[1,0]=(0.296966289838)*x[0]**o+(0.170347587334)*x[0]+(0.755207250839)*x[1]**o+(0.677658805212)*x[1]
        arg[1,1]=(-0.507591010573)*x[0]**o+(0.299638021889)*x[0]+(-0.757702071557)*x[1]**o+(-0.194564860748)*x[1]
        arg[1,2]=(0.821972467197)*x[0]**o+(-0.75902704466)*x[0]+(0.19424332718)*x[1]**o+(-0.070182646035)*x[1]
        arg[1,3]=(-0.340805687409)*x[0]**o+(0.585961973082)*x[0]+(-0.602942732318)*x[1]**o+(-0.288941395094)*x[1]
        arg[1,4]=(0.0314374795024)*x[0]**o+(0.293710824013)*x[0]+(0.576509865762)*x[1]**o+(-0.481377812302)*x[1]
        arg[2,0]=(0.250117528833)*x[0]**o+(-0.318419344173)*x[0]+(-0.793830015706)*x[1]**o+(0.0445553063586)*x[1]
        arg[2,1]=(-0.347974394766)*x[0]**o+(-0.569347105354)*x[0]+(0.838687005759)*x[1]**o+(0.534306628908)*x[1]
        arg[2,2]=(0.53063272248)*x[0]**o+(0.426504428158)*x[0]+(0.215398162753)*x[1]**o+(0.324348187788)*x[1]
        arg[2,3]=(-0.336752121527)*x[0]**o+(0.666074768462)*x[0]+(-0.739123251932)*x[1]**o+(-0.538899653147)*x[1]
        arg[2,4]=(-0.832783810519)*x[0]**o+(0.558572116384)*x[0]+(0.119988536199)*x[1]**o+(-0.664170918761)*x[1]
        arg[3,0]=(0.4941356722)*x[0]**o+(-0.403591783006)*x[0]+(0.205050527245)*x[1]**o+(0.351642574939)*x[1]
        arg[3,1]=(0.12538190695)*x[0]**o+(-0.890722638937)*x[0]+(-0.897011486902)*x[1]**o+(-0.465522770696)*x[1]
        arg[3,2]=(-0.862829007552)*x[0]**o+(-0.950531038019)*x[0]+(-0.970016379643)*x[1]**o+(-0.500580755888)*x[1]
        arg[3,3]=(0.131451743264)*x[0]**o+(0.43845495141)*x[0]+(-0.929808474589)*x[1]**o+(-0.104714452616)*x[1]
        arg[3,4]=(-0.81935224348)*x[0]**o+(-0.53791483312)*x[0]+(-0.714512816761)*x[1]**o+(0.329859667225)*x[1]
        ref[0,0]=(-0.178670895269)/(o+1.)+(-0.0607706502011)+(0.330520725992)*0.5**o
        ref[0,1]=(-0.265102051042)/(o+1.)+(-0.0536019332266)+(-0.871801084852)*0.5**o
        ref[0,2]=(0.81492666918)/(o+1.)+(0.712715940244)+(-0.816985024574)*0.5**o
        ref[0,3]=(-0.924660993906)/(o+1.)+(-0.101586986156)+(-0.20728122917)*0.5**o
        ref[0,4]=(-0.15879052465)/(o+1.)+(0.298205944153)+(-0.541377726375)*0.5**o
        ref[1,0]=(0.755207250839)/(o+1.)+(0.424003196273)+(0.296966289838)*0.5**o
        ref[1,1]=(-0.757702071557)/(o+1.)+(0.0525365805706)+(-0.507591010573)*0.5**o
        ref[1,2]=(0.19424332718)/(o+1.)+(-0.414604845348)+(0.821972467197)*0.5**o
        ref[1,3]=(-0.602942732318)/(o+1.)+(0.148510288994)+(-0.340805687409)*0.5**o
        ref[1,4]=(0.576509865762)/(o+1.)+(-0.0938334941445)+(0.0314374795024)*0.5**o
        ref[2,0]=(-0.793830015706)/(o+1.)+(-0.136932018907)+(0.250117528833)*0.5**o
        ref[2,1]=(0.838687005759)/(o+1.)+(-0.0175202382229)+(-0.347974394766)*0.5**o
        ref[2,2]=(0.215398162753)/(o+1.)+(0.375426307973)+(0.53063272248)*0.5**o
        ref[2,3]=(-0.739123251932)/(o+1.)+(0.0635875576574)+(-0.336752121527)*0.5**o
        ref[2,4]=(0.119988536199)/(o+1.)+(-0.0527994011886)+(-0.832783810519)*0.5**o
        ref[3,0]=(0.205050527245)/(o+1.)+(-0.0259746040335)+(0.4941356722)*0.5**o
        ref[3,1]=(-0.897011486902)/(o+1.)+(-0.678122704817)+(0.12538190695)*0.5**o
        ref[3,2]=(-0.970016379643)/(o+1.)+(-0.725555896953)+(-0.862829007552)*0.5**o
        ref[3,3]=(-0.929808474589)/(o+1.)+(0.166870249397)+(0.131451743264)*0.5**o
        ref[3,4]=(-0.714512816761)/(o+1.)+(-0.104027582947)+(-0.81935224348)*0.5**o
      else:
        arg[0,0]=(-0.916251178344)*x[0]**o+(0.858306033367)*x[0]+(0.814515071272)*x[1]**o+(-0.31988121358)*x[1]+(-0.973228319626)*x[2]**o+(-0.448894628432)*x[2]
        arg[0,1]=(0.394307707909)*x[0]**o+(-0.270473398988)*x[0]+(0.904831123674)*x[1]**o+(-0.949869961069)*x[1]+(0.771889549168)*x[2]**o+(0.650652953462)*x[2]
        arg[0,2]=(-0.0387282383528)*x[0]**o+(0.0292919821249)*x[0]+(-0.165189629855)*x[1]**o+(-0.265182468614)*x[1]+(0.919042632361)*x[2]**o+(0.710368688769)*x[2]
        arg[0,3]=(0.680006301334)*x[0]**o+(0.96959940913)*x[0]+(0.773153544603)*x[1]**o+(-0.460435754764)*x[1]+(-0.202448514604)*x[2]**o+(-0.495739672831)*x[2]
        arg[0,4]=(-0.972790915366)*x[0]**o+(-0.14148426268)*x[0]+(0.511995390348)*x[1]**o+(-0.252881217719)*x[1]+(0.255711210131)*x[2]**o+(0.729991325048)*x[2]
        arg[1,0]=(-0.105824689194)*x[0]**o+(-0.201427239595)*x[0]+(-0.00550922402684)*x[1]**o+(-0.732794556814)*x[1]+(-0.0820953451459)*x[2]**o+(0.289904794134)*x[2]
        arg[1,1]=(-0.378707957559)*x[0]**o+(0.776904510463)*x[0]+(0.0406721415351)*x[1]**o+(0.159288221237)*x[1]+(0.961004037621)*x[2]**o+(-0.0866180279992)*x[2]
        arg[1,2]=(-0.0226058491089)*x[0]**o+(-0.569995151391)*x[0]+(-0.0675265986408)*x[1]**o+(-0.141280346236)*x[1]+(0.116098815262)*x[2]**o+(0.654630437206)*x[2]
        arg[1,3]=(-0.567270755897)*x[0]**o+(0.580139672878)*x[0]+(-0.411816626246)*x[1]**o+(-0.490525831459)*x[1]+(0.9322522728)*x[2]**o+(-0.948230068413)*x[2]
        arg[1,4]=(0.977251285351)*x[0]**o+(-0.457921996364)*x[0]+(-0.586207130671)*x[1]**o+(0.801370818191)*x[1]+(-0.667233775445)*x[2]**o+(0.277054304674)*x[2]
        arg[2,0]=(0.997778241906)*x[0]**o+(0.490242237175)*x[0]+(0.406384527224)*x[1]**o+(-0.00724475574186)*x[1]+(-0.59799957401)*x[2]**o+(0.520598391022)*x[2]
        arg[2,1]=(0.59766081214)*x[0]**o+(-0.144793824209)*x[0]+(-0.191121009265)*x[1]**o+(-0.178016091024)*x[1]+(0.168431462883)*x[2]**o+(-0.766200678925)*x[2]
        arg[2,2]=(0.633988266946)*x[0]**o+(-0.880495194817)*x[0]+(-0.217502420499)*x[1]**o+(-0.730159316723)*x[1]+(0.255599562732)*x[2]**o+(-0.510511087905)*x[2]
        arg[2,3]=(0.102332382261)*x[0]**o+(-0.370884332534)*x[0]+(-0.0926186085648)*x[1]**o+(0.86446607832)*x[1]+(0.0643112986026)*x[2]**o+(0.950118360785)*x[2]
        arg[2,4]=(0.928323090624)*x[0]**o+(0.942943574803)*x[0]+(0.162570981489)*x[1]**o+(-0.971092675635)*x[1]+(0.641489870001)*x[2]**o+(0.695989498774)*x[2]
        arg[3,0]=(-0.988813741403)*x[0]**o+(0.407960892494)*x[0]+(0.671572481461)*x[1]**o+(0.119756938854)*x[1]+(-0.923671788537)*x[2]**o+(-0.458931381612)*x[2]
        arg[3,1]=(-0.241202616681)*x[0]**o+(-0.369134099039)*x[0]+(-0.129164268709)*x[1]**o+(-0.469311903775)*x[1]+(0.633909205241)*x[2]**o+(0.564043934538)*x[2]
        arg[3,2]=(-0.397081811407)*x[0]**o+(0.909387986899)*x[0]+(0.260590665694)*x[1]**o+(-0.371472818152)*x[1]+(-0.0369834262513)*x[2]**o+(-0.887598789153)*x[2]
        arg[3,3]=(-0.304262529036)*x[0]**o+(0.741316478862)*x[0]+(-0.384317206475)*x[1]**o+(-0.854685760579)*x[1]+(0.502373892581)*x[2]**o+(-0.923561844368)*x[2]
        arg[3,4]=(-0.720555420423)*x[0]**o+(-0.115786593478)*x[0]+(0.832043243085)*x[1]**o+(-0.912631896773)*x[1]+(-0.583397798668)*x[2]**o+(-0.508174721594)*x[2]
        ref[0,0]=(-0.158713248354)/(o+1.)+(0.0447650956777)+(-0.916251178344)*0.5**o
        ref[0,1]=(1.67672067284)/(o+1.)+(-0.284845203298)+(0.394307707909)*0.5**o
        ref[0,2]=(0.753853002506)/(o+1.)+(0.23723910114)+(-0.0387282383528)*0.5**o
        ref[0,3]=(0.570705029999)/(o+1.)+(0.00671199076774)+(0.680006301334)*0.5**o
        ref[0,4]=(0.767706600479)/(o+1.)+(0.167812922325)+(-0.972790915366)*0.5**o
        ref[1,0]=(-0.0876045691728)/(o+1.)+(-0.322158501138)+(-0.105824689194)*0.5**o
        ref[1,1]=(1.00167617916)/(o+1.)+(0.42478735185)+(-0.378707957559)*0.5**o
        ref[1,2]=(0.0485722166212)/(o+1.)+(-0.0283225302105)+(-0.0226058491089)*0.5**o
        ref[1,3]=(0.520435646554)/(o+1.)+(-0.429308113497)+(-0.567270755897)*0.5**o
        ref[1,4]=(-1.25344090612)/(o+1.)+(0.31025156325)+(0.977251285351)*0.5**o
        ref[2,0]=(-0.191615046786)/(o+1.)+(0.501797936227)+(0.997778241906)*0.5**o
        ref[2,1]=(-0.0226895463818)/(o+1.)+(-0.544505297079)+(0.59766081214)*0.5**o
        ref[2,2]=(0.0380971422336)/(o+1.)+(-1.06058279972)+(0.633988266946)*0.5**o
        ref[2,3]=(-0.0283073099622)/(o+1.)+(0.721850053286)+(0.102332382261)*0.5**o
        ref[2,4]=(0.80406085149)/(o+1.)+(0.333920198971)+(0.928323090624)*0.5**o
        ref[3,0]=(-0.252099307076)/(o+1.)+(0.0343932248679)+(-0.988813741403)*0.5**o
        ref[3,1]=(0.504744936532)/(o+1.)+(-0.137201034138)+(-0.241202616681)*0.5**o
        ref[3,2]=(0.223607239442)/(o+1.)+(-0.174841810203)+(-0.397081811407)*0.5**o
        ref[3,3]=(0.118056686106)/(o+1.)+(-0.518465563043)+(-0.304262529036)*0.5**o
        ref[3,4]=(0.248645444416)/(o+1.)+(-0.768296605922)+(-0.720555420423)*0.5**o
      res=integrate(arg)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactOne_fromData_FunctionOnContactOne_rank3(self):
      """
      tests integral of rank 3 Data on the FunctionOnContactOne

      assumptions: FunctionOnContactOne(self.domain) exists
                   self.domain supports integral on FunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      w=FunctionOnContactOne(self.domain)
      x=w.getX()
      arg=Data(0,(6, 2, 2),w)
      ref=numpy.zeros((6, 2, 2),numpy.float_)
      if dim==2:
        arg[0,0,0]=(-0.347273720148)*x[0]**o+(0.325185727721)*x[0]+(-0.855515194735)*x[1]**o+(0.825807975597)*x[1]
        arg[0,0,1]=(0.842054669437)*x[0]**o+(0.353768311241)*x[0]+(0.0148900864182)*x[1]**o+(-0.402154654143)*x[1]
        arg[0,1,0]=(-0.191525791617)*x[0]**o+(-0.299181220926)*x[0]+(-0.696260869529)*x[1]**o+(0.285645399154)*x[1]
        arg[0,1,1]=(0.190151633835)*x[0]**o+(0.184074338131)*x[0]+(-0.497689504982)*x[1]**o+(-0.813410371313)*x[1]
        arg[1,0,0]=(-0.695114302024)*x[0]**o+(0.427133485674)*x[0]+(-0.711899825683)*x[1]**o+(-0.296004734911)*x[1]
        arg[1,0,1]=(0.37620789583)*x[0]**o+(-0.944850444544)*x[0]+(0.666194812839)*x[1]**o+(-0.0715430628168)*x[1]
        arg[1,1,0]=(-0.0312291891427)*x[0]**o+(0.403009732443)*x[0]+(-0.378486005359)*x[1]**o+(-0.992729541609)*x[1]
        arg[1,1,1]=(0.0675984132698)*x[0]**o+(0.241001129867)*x[0]+(0.340546237588)*x[1]**o+(-0.024830810543)*x[1]
        arg[2,0,0]=(-0.146870758676)*x[0]**o+(-0.170434897147)*x[0]+(-0.579618469996)*x[1]**o+(-0.770817574749)*x[1]
        arg[2,0,1]=(-0.3029068201)*x[0]**o+(-0.110636302601)*x[0]+(-0.905841665338)*x[1]**o+(-0.280977886629)*x[1]
        arg[2,1,0]=(0.0547810600344)*x[0]**o+(0.00763321517272)*x[0]+(0.0156822194106)*x[1]**o+(-0.528337857094)*x[1]
        arg[2,1,1]=(0.12016647407)*x[0]**o+(0.4590743733)*x[0]+(-0.574499998403)*x[1]**o+(0.698871172503)*x[1]
        arg[3,0,0]=(0.481811757276)*x[0]**o+(0.748356476205)*x[0]+(-0.300272959749)*x[1]**o+(-0.707230263391)*x[1]
        arg[3,0,1]=(-0.347858576311)*x[0]**o+(-0.177810365974)*x[0]+(0.384658344075)*x[1]**o+(-0.837385668636)*x[1]
        arg[3,1,0]=(0.943559460135)*x[0]**o+(0.178778474835)*x[0]+(-0.831384000346)*x[1]**o+(-0.474975888568)*x[1]
        arg[3,1,1]=(-0.589903969401)*x[0]**o+(0.271114795948)*x[0]+(-0.946594221253)*x[1]**o+(-0.964029170264)*x[1]
        arg[4,0,0]=(0.221077049797)*x[0]**o+(0.986052914095)*x[0]+(-0.392982812685)*x[1]**o+(0.370828365061)*x[1]
        arg[4,0,1]=(0.463121232737)*x[0]**o+(-0.91920434872)*x[0]+(0.830655891827)*x[1]**o+(0.193876462558)*x[1]
        arg[4,1,0]=(0.925740559506)*x[0]**o+(0.534342942418)*x[0]+(0.446730463706)*x[1]**o+(0.181095371535)*x[1]
        arg[4,1,1]=(-0.777093674353)*x[0]**o+(-0.435130879793)*x[0]+(0.134406853212)*x[1]**o+(0.861099956528)*x[1]
        arg[5,0,0]=(-0.443943796359)*x[0]**o+(0.359190127208)*x[0]+(0.0570788998709)*x[1]**o+(0.548070594378)*x[1]
        arg[5,0,1]=(0.762962865085)*x[0]**o+(0.969248002849)*x[0]+(0.729222816543)*x[1]**o+(-0.865271983926)*x[1]
        arg[5,1,0]=(-0.915609563739)*x[0]**o+(-0.338560068696)*x[0]+(0.972145654544)*x[1]**o+(0.0941277349395)*x[1]
        arg[5,1,1]=(-0.486023356984)*x[0]**o+(-0.426382776921)*x[0]+(-0.114606039743)*x[1]**o+(-0.992152486495)*x[1]
        ref[0,0,0]=(-0.855515194735)/(o+1.)+(0.575496851659)+(-0.347273720148)*0.5**o
        ref[0,0,1]=(0.0148900864182)/(o+1.)+(-0.0241931714509)+(0.842054669437)*0.5**o
        ref[0,1,0]=(-0.696260869529)/(o+1.)+(-0.00676791088595)+(-0.191525791617)*0.5**o
        ref[0,1,1]=(-0.497689504982)/(o+1.)+(-0.314668016591)+(0.190151633835)*0.5**o
        ref[1,0,0]=(-0.711899825683)/(o+1.)+(0.0655643753813)+(-0.695114302024)*0.5**o
        ref[1,0,1]=(0.666194812839)/(o+1.)+(-0.50819675368)+(0.37620789583)*0.5**o
        ref[1,1,0]=(-0.378486005359)/(o+1.)+(-0.294859904583)+(-0.0312291891427)*0.5**o
        ref[1,1,1]=(0.340546237588)/(o+1.)+(0.108085159662)+(0.0675984132698)*0.5**o
        ref[2,0,0]=(-0.579618469996)/(o+1.)+(-0.470626235948)+(-0.146870758676)*0.5**o
        ref[2,0,1]=(-0.905841665338)/(o+1.)+(-0.195807094615)+(-0.3029068201)*0.5**o
        ref[2,1,0]=(0.0156822194106)/(o+1.)+(-0.260352320961)+(0.0547810600344)*0.5**o
        ref[2,1,1]=(-0.574499998403)/(o+1.)+(0.578972772902)+(0.12016647407)*0.5**o
        ref[3,0,0]=(-0.300272959749)/(o+1.)+(0.0205631064069)+(0.481811757276)*0.5**o
        ref[3,0,1]=(0.384658344075)/(o+1.)+(-0.507598017305)+(-0.347858576311)*0.5**o
        ref[3,1,0]=(-0.831384000346)/(o+1.)+(-0.148098706867)+(0.943559460135)*0.5**o
        ref[3,1,1]=(-0.946594221253)/(o+1.)+(-0.346457187158)+(-0.589903969401)*0.5**o
        ref[4,0,0]=(-0.392982812685)/(o+1.)+(0.678440639578)+(0.221077049797)*0.5**o
        ref[4,0,1]=(0.830655891827)/(o+1.)+(-0.362663943081)+(0.463121232737)*0.5**o
        ref[4,1,0]=(0.446730463706)/(o+1.)+(0.357719156977)+(0.925740559506)*0.5**o
        ref[4,1,1]=(0.134406853212)/(o+1.)+(0.212984538367)+(-0.777093674353)*0.5**o
        ref[5,0,0]=(0.0570788998709)/(o+1.)+(0.453630360793)+(-0.443943796359)*0.5**o
        ref[5,0,1]=(0.729222816543)/(o+1.)+(0.0519880094615)+(0.762962865085)*0.5**o
        ref[5,1,0]=(0.972145654544)/(o+1.)+(-0.122216166878)+(-0.915609563739)*0.5**o
        ref[5,1,1]=(-0.114606039743)/(o+1.)+(-0.709267631708)+(-0.486023356984)*0.5**o
      else:
        arg[0,0,0]=(0.455440122065)*x[0]**o+(0.911340571887)*x[0]+(0.158409397783)*x[1]**o+(0.353755257124)*x[1]+(-0.392556456322)*x[2]**o+(0.707232271095)*x[2]
        arg[0,0,1]=(-0.816405087957)*x[0]**o+(0.0691299991887)*x[0]+(0.178677657085)*x[1]**o+(0.526176369634)*x[1]+(-0.159459641444)*x[2]**o+(-0.197907778155)*x[2]
        arg[0,1,0]=(-0.463418217546)*x[0]**o+(-0.485835567862)*x[0]+(0.760458548482)*x[1]**o+(0.902873590314)*x[1]+(-0.760679635511)*x[2]**o+(-0.483163544794)*x[2]
        arg[0,1,1]=(0.743029549626)*x[0]**o+(-0.39131913217)*x[0]+(0.488809682734)*x[1]**o+(-0.256115251135)*x[1]+(0.0918611991036)*x[2]**o+(-0.281291060967)*x[2]
        arg[1,0,0]=(0.794367740289)*x[0]**o+(0.727663222288)*x[0]+(-0.631739003283)*x[1]**o+(0.733862609049)*x[1]+(-0.721230552103)*x[2]**o+(0.787774790111)*x[2]
        arg[1,0,1]=(0.505578181889)*x[0]**o+(0.616497234291)*x[0]+(-0.616903198005)*x[1]**o+(-0.355969913594)*x[1]+(0.0105854597103)*x[2]**o+(0.510273945857)*x[2]
        arg[1,1,0]=(0.35370601878)*x[0]**o+(-0.0414232536827)*x[0]+(0.0546619209334)*x[1]**o+(0.0459241421788)*x[1]+(0.796266758386)*x[2]**o+(0.303998757002)*x[2]
        arg[1,1,1]=(-0.245326163282)*x[0]**o+(0.0893045737315)*x[0]+(0.019401528413)*x[1]**o+(-0.838249399348)*x[1]+(0.510320088517)*x[2]**o+(0.544558918049)*x[2]
        arg[2,0,0]=(-0.660208039977)*x[0]**o+(-0.224874460802)*x[0]+(-0.0467836584153)*x[1]**o+(-0.463886703951)*x[1]+(-0.0951679104966)*x[2]**o+(-0.735787783779)*x[2]
        arg[2,0,1]=(-0.588259372806)*x[0]**o+(-0.18950606147)*x[0]+(-0.373439432467)*x[1]**o+(-0.116285718036)*x[1]+(-0.944687791765)*x[2]**o+(-0.243001707318)*x[2]
        arg[2,1,0]=(0.798498386971)*x[0]**o+(0.500125941024)*x[0]+(0.197339376576)*x[1]**o+(-0.068777545221)*x[1]+(0.36649695624)*x[2]**o+(0.193956440608)*x[2]
        arg[2,1,1]=(0.27501104189)*x[0]**o+(0.203066488761)*x[0]+(0.379278873515)*x[1]**o+(-0.789154057614)*x[1]+(0.893889499095)*x[2]**o+(0.867881975803)*x[2]
        arg[3,0,0]=(0.0579414247006)*x[0]**o+(0.661826954985)*x[0]+(0.313969775635)*x[1]**o+(-0.125420102972)*x[1]+(-0.808513922748)*x[2]**o+(0.0301073398588)*x[2]
        arg[3,0,1]=(-0.254544981247)*x[0]**o+(0.755983567644)*x[0]+(0.129524533092)*x[1]**o+(0.562828535682)*x[1]+(0.712881323083)*x[2]**o+(0.375450604678)*x[2]
        arg[3,1,0]=(-0.100221176995)*x[0]**o+(-0.470987646782)*x[0]+(-0.444245257427)*x[1]**o+(-0.0112626639353)*x[1]+(-0.516917833845)*x[2]**o+(-0.664158938756)*x[2]
        arg[3,1,1]=(0.259306174564)*x[0]**o+(0.367178744389)*x[0]+(0.177897569676)*x[1]**o+(-0.355724319562)*x[1]+(-0.809141944951)*x[2]**o+(0.962429694607)*x[2]
        arg[4,0,0]=(0.351990799011)*x[0]**o+(-0.41657457182)*x[0]+(-0.769748341564)*x[1]**o+(-0.856355494001)*x[1]+(-0.777902936204)*x[2]**o+(-0.388980567423)*x[2]
        arg[4,0,1]=(0.875307372684)*x[0]**o+(0.061375057221)*x[0]+(-0.262062525469)*x[1]**o+(0.0950324968281)*x[1]+(-0.145145751811)*x[2]**o+(-0.246872045818)*x[2]
        arg[4,1,0]=(0.115340652181)*x[0]**o+(0.910414483949)*x[0]+(0.347463744624)*x[1]**o+(0.399961393031)*x[1]+(0.0491863598064)*x[2]**o+(-0.992304094864)*x[2]
        arg[4,1,1]=(-0.125763267852)*x[0]**o+(0.485867057874)*x[0]+(-0.414172783931)*x[1]**o+(0.164145362806)*x[1]+(-0.488738184728)*x[2]**o+(-0.232758639076)*x[2]
        arg[5,0,0]=(-0.748184063441)*x[0]**o+(-0.252715593115)*x[0]+(0.688609235584)*x[1]**o+(0.0645665122312)*x[1]+(0.365465805991)*x[2]**o+(-0.217347848328)*x[2]
        arg[5,0,1]=(-0.298018437594)*x[0]**o+(-0.39683066279)*x[0]+(-0.0609074331294)*x[1]**o+(0.146930937933)*x[1]+(-0.00737018012329)*x[2]**o+(-0.700002160047)*x[2]
        arg[5,1,0]=(0.974781669859)*x[0]**o+(0.0634072121163)*x[0]+(0.0194044697894)*x[1]**o+(-0.919375545742)*x[1]+(-0.779806244021)*x[2]**o+(0.641761171945)*x[2]
        arg[5,1,1]=(0.485215976154)*x[0]**o+(0.879804212343)*x[0]+(-0.765170516406)*x[1]**o+(-0.205158612954)*x[1]+(-0.586708070331)*x[2]**o+(0.506670980992)*x[2]
        ref[0,0,0]=(-0.23414705854)/(o+1.)+(0.986164050053)+(0.455440122065)*0.5**o
        ref[0,0,1]=(0.0192180156412)/(o+1.)+(0.198699295334)+(-0.816405087957)*0.5**o
        ref[0,1,0]=(-0.000221087029094)/(o+1.)+(-0.0330627611712)+(-0.463418217546)*0.5**o
        ref[0,1,1]=(0.580670881837)/(o+1.)+(-0.464362722135)+(0.743029549626)*0.5**o
        ref[1,0,0]=(-1.35296955539)/(o+1.)+(1.12465031072)+(0.794367740289)*0.5**o
        ref[1,0,1]=(-0.606317738295)/(o+1.)+(0.385400633277)+(0.505578181889)*0.5**o
        ref[1,1,0]=(0.85092867932)/(o+1.)+(0.154249822749)+(0.35370601878)*0.5**o
        ref[1,1,1]=(0.52972161693)/(o+1.)+(-0.102192953784)+(-0.245326163282)*0.5**o
        ref[2,0,0]=(-0.141951568912)/(o+1.)+(-0.712274474266)+(-0.660208039977)*0.5**o
        ref[2,0,1]=(-1.31812722423)/(o+1.)+(-0.274396743412)+(-0.588259372806)*0.5**o
        ref[2,1,0]=(0.563836332815)/(o+1.)+(0.312652418205)+(0.798498386971)*0.5**o
        ref[2,1,1]=(1.27316837261)/(o+1.)+(0.140897203475)+(0.27501104189)*0.5**o
        ref[3,0,0]=(-0.494544147113)/(o+1.)+(0.283257095936)+(0.0579414247006)*0.5**o
        ref[3,0,1]=(0.842405856175)/(o+1.)+(0.847131354002)+(-0.254544981247)*0.5**o
        ref[3,1,0]=(-0.961163091272)/(o+1.)+(-0.573204624737)+(-0.100221176995)*0.5**o
        ref[3,1,1]=(-0.631244375275)/(o+1.)+(0.486942059717)+(0.259306174564)*0.5**o
        ref[4,0,0]=(-1.54765127777)/(o+1.)+(-0.830955316622)+(0.351990799011)*0.5**o
        ref[4,0,1]=(-0.407208277279)/(o+1.)+(-0.0452322458845)+(0.875307372684)*0.5**o
        ref[4,1,0]=(0.39665010443)/(o+1.)+(0.159035891058)+(0.115340652181)*0.5**o
        ref[4,1,1]=(-0.902910968659)/(o+1.)+(0.208626890802)+(-0.125763267852)*0.5**o
        ref[5,0,0]=(1.05407504158)/(o+1.)+(-0.202748464606)+(-0.748184063441)*0.5**o
        ref[5,0,1]=(-0.0682776132527)/(o+1.)+(-0.474950942452)+(-0.298018437594)*0.5**o
        ref[5,1,0]=(-0.760401774232)/(o+1.)+(-0.10710358084)+(0.974781669859)*0.5**o
        ref[5,1,1]=(-1.35187858674)/(o+1.)+(0.590658290191)+(0.485215976154)*0.5**o
      res=integrate(arg)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactOne_fromData_FunctionOnContactOne_rank4(self):
      """
      tests integral of rank 4 Data on the FunctionOnContactOne

      assumptions: FunctionOnContactOne(self.domain) exists
                   self.domain supports integral on FunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      w=FunctionOnContactOne(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5, 3, 2),w)
      ref=numpy.zeros((4, 5, 3, 2),numpy.float_)
      if dim==2:
        arg[0,0,0,0]=(-0.177078428867)*x[0]**o+(0.338116995819)*x[0]+(-0.750823220445)*x[1]**o+(0.0630074420245)*x[1]
        arg[0,0,0,1]=(-0.682838671264)*x[0]**o+(-0.893797611147)*x[0]+(-0.0127659464437)*x[1]**o+(0.144120749888)*x[1]
        arg[0,0,1,0]=(0.352439319413)*x[0]**o+(0.380064866457)*x[0]+(-0.932452125435)*x[1]**o+(0.914519937443)*x[1]
        arg[0,0,1,1]=(-0.354033408023)*x[0]**o+(0.771069338542)*x[0]+(-0.702758889809)*x[1]**o+(-0.484485925087)*x[1]
        arg[0,0,2,0]=(0.706826713479)*x[0]**o+(0.0526230471775)*x[0]+(-0.535244256862)*x[1]**o+(-0.085929844746)*x[1]
        arg[0,0,2,1]=(-0.893497856528)*x[0]**o+(-0.892309840177)*x[0]+(0.742770093639)*x[1]**o+(0.987917060326)*x[1]
        arg[0,1,0,0]=(-0.536624329097)*x[0]**o+(-0.569536051343)*x[0]+(-0.7740511254)*x[1]**o+(0.873150177653)*x[1]
        arg[0,1,0,1]=(0.388396767346)*x[0]**o+(-0.902569510334)*x[0]+(0.196975646933)*x[1]**o+(0.184178333918)*x[1]
        arg[0,1,1,0]=(-0.125771297178)*x[0]**o+(-0.335093336446)*x[0]+(0.372752949038)*x[1]**o+(-0.883928748878)*x[1]
        arg[0,1,1,1]=(-0.313637256508)*x[0]**o+(-0.49607955109)*x[0]+(-0.726599229867)*x[1]**o+(0.360837115079)*x[1]
        arg[0,1,2,0]=(0.92425817047)*x[0]**o+(0.734728600171)*x[0]+(1.60583902129e-05)*x[1]**o+(-0.124600971339)*x[1]
        arg[0,1,2,1]=(-0.0981827607557)*x[0]**o+(-0.556005942436)*x[0]+(-0.888060617134)*x[1]**o+(0.857508807636)*x[1]
        arg[0,2,0,0]=(-0.871590768072)*x[0]**o+(-0.208546938841)*x[0]+(-0.228336105988)*x[1]**o+(0.444098195066)*x[1]
        arg[0,2,0,1]=(-0.628675127006)*x[0]**o+(0.834043549769)*x[0]+(-0.509662470381)*x[1]**o+(-0.801907226651)*x[1]
        arg[0,2,1,0]=(-0.767528433339)*x[0]**o+(0.305967685996)*x[0]+(-0.587902586916)*x[1]**o+(0.38652781193)*x[1]
        arg[0,2,1,1]=(0.388984380682)*x[0]**o+(0.490673256209)*x[0]+(-0.78470263647)*x[1]**o+(0.34909383158)*x[1]
        arg[0,2,2,0]=(-0.910514531543)*x[0]**o+(-0.418810366626)*x[0]+(-0.637159792471)*x[1]**o+(0.794653160897)*x[1]
        arg[0,2,2,1]=(0.570535552656)*x[0]**o+(-0.590385806831)*x[0]+(-0.298806908011)*x[1]**o+(-0.901531327347)*x[1]
        arg[0,3,0,0]=(0.916864925136)*x[0]**o+(-0.019733476459)*x[0]+(-0.91370863065)*x[1]**o+(0.291932128733)*x[1]
        arg[0,3,0,1]=(-0.851147484632)*x[0]**o+(-0.159304969602)*x[0]+(0.699832871063)*x[1]**o+(0.327932729929)*x[1]
        arg[0,3,1,0]=(0.461273862468)*x[0]**o+(0.40068516653)*x[0]+(0.0203170502559)*x[1]**o+(0.973716167207)*x[1]
        arg[0,3,1,1]=(0.63984877633)*x[0]**o+(0.580871491133)*x[0]+(-0.00587359808953)*x[1]**o+(-0.104805621161)*x[1]
        arg[0,3,2,0]=(-0.159416471436)*x[0]**o+(-0.585039922994)*x[0]+(0.689553978928)*x[1]**o+(0.569617375406)*x[1]
        arg[0,3,2,1]=(0.782397836312)*x[0]**o+(-0.581894025693)*x[0]+(0.0370277809126)*x[1]**o+(0.597309139432)*x[1]
        arg[0,4,0,0]=(-0.0063433205485)*x[0]**o+(-0.242291627085)*x[0]+(0.110218625254)*x[1]**o+(0.819656578408)*x[1]
        arg[0,4,0,1]=(-0.128215646666)*x[0]**o+(0.821312510893)*x[0]+(0.910723121611)*x[1]**o+(-0.0174623980585)*x[1]
        arg[0,4,1,0]=(-0.604528735513)*x[0]**o+(-0.358705990504)*x[0]+(-0.281954207008)*x[1]**o+(0.123469639952)*x[1]
        arg[0,4,1,1]=(0.65096807032)*x[0]**o+(0.882935341204)*x[0]+(-0.761226249914)*x[1]**o+(-0.830800218214)*x[1]
        arg[0,4,2,0]=(-0.710718948634)*x[0]**o+(0.452128874613)*x[0]+(-0.780671499903)*x[1]**o+(-0.942033435912)*x[1]
        arg[0,4,2,1]=(-0.0443187506467)*x[0]**o+(0.513943853321)*x[0]+(0.709902333411)*x[1]**o+(0.755430555318)*x[1]
        arg[1,0,0,0]=(0.731890221753)*x[0]**o+(-0.821835735249)*x[0]+(-0.713098084853)*x[1]**o+(0.875738077525)*x[1]
        arg[1,0,0,1]=(-0.130335125365)*x[0]**o+(0.580579210083)*x[0]+(-0.427105331069)*x[1]**o+(0.0435798068353)*x[1]
        arg[1,0,1,0]=(-0.203908509107)*x[0]**o+(-0.941974700448)*x[0]+(0.344459670502)*x[1]**o+(0.758336135081)*x[1]
        arg[1,0,1,1]=(0.186770929724)*x[0]**o+(0.348332058705)*x[0]+(-0.0683630778679)*x[1]**o+(0.480425212189)*x[1]
        arg[1,0,2,0]=(-0.891275312975)*x[0]**o+(0.260630881729)*x[0]+(0.127955472707)*x[1]**o+(-0.554992279729)*x[1]
        arg[1,0,2,1]=(-0.74620344519)*x[0]**o+(0.944025669845)*x[0]+(-0.052031562679)*x[1]**o+(0.966060927262)*x[1]
        arg[1,1,0,0]=(0.871423198418)*x[0]**o+(-0.0769723128617)*x[0]+(0.783714758356)*x[1]**o+(0.530209967997)*x[1]
        arg[1,1,0,1]=(0.943197824902)*x[0]**o+(-0.62624855429)*x[0]+(-0.00888785913795)*x[1]**o+(-0.0408857497082)*x[1]
        arg[1,1,1,0]=(-0.74971981056)*x[0]**o+(0.925175163834)*x[0]+(0.481063204532)*x[1]**o+(-0.476700447185)*x[1]
        arg[1,1,1,1]=(-0.431776603043)*x[0]**o+(-0.495358165507)*x[0]+(-0.0110588803484)*x[1]**o+(-0.575147325496)*x[1]
        arg[1,1,2,0]=(0.578105297721)*x[0]**o+(-0.88169100464)*x[0]+(-0.646224518073)*x[1]**o+(0.465619837803)*x[1]
        arg[1,1,2,1]=(0.442066689582)*x[0]**o+(0.0937839719519)*x[0]+(-0.101534585934)*x[1]**o+(-0.974003442605)*x[1]
        arg[1,2,0,0]=(-0.206702379228)*x[0]**o+(0.330161049866)*x[0]+(0.185493587693)*x[1]**o+(0.0391986992855)*x[1]
        arg[1,2,0,1]=(0.79092697362)*x[0]**o+(0.692054695304)*x[0]+(0.141622971369)*x[1]**o+(0.643869474153)*x[1]
        arg[1,2,1,0]=(0.914281902407)*x[0]**o+(0.936365302055)*x[0]+(0.249095964198)*x[1]**o+(-0.440892551355)*x[1]
        arg[1,2,1,1]=(-0.602926709108)*x[0]**o+(0.817198874641)*x[0]+(0.262528290838)*x[1]**o+(0.493607450371)*x[1]
        arg[1,2,2,0]=(-0.724435531493)*x[0]**o+(-0.455127093687)*x[0]+(0.144972402754)*x[1]**o+(-0.361255031527)*x[1]
        arg[1,2,2,1]=(-0.611538590545)*x[0]**o+(-0.780730652602)*x[0]+(-0.150439352532)*x[1]**o+(0.450875229163)*x[1]
        arg[1,3,0,0]=(-0.913553947133)*x[0]**o+(-0.930172924067)*x[0]+(-0.509426399026)*x[1]**o+(0.3953360754)*x[1]
        arg[1,3,0,1]=(-0.709834132519)*x[0]**o+(0.972632526254)*x[0]+(0.540358729462)*x[1]**o+(-0.0295254269136)*x[1]
        arg[1,3,1,0]=(-0.30788139278)*x[0]**o+(-0.362561853154)*x[0]+(0.0135015706177)*x[1]**o+(0.558034983013)*x[1]
        arg[1,3,1,1]=(-0.243327801257)*x[0]**o+(-0.156242539226)*x[0]+(0.076082439418)*x[1]**o+(0.818613998695)*x[1]
        arg[1,3,2,0]=(-0.0290765899224)*x[0]**o+(-0.108604693002)*x[0]+(-0.0741034599215)*x[1]**o+(0.44467098076)*x[1]
        arg[1,3,2,1]=(-0.751374584848)*x[0]**o+(0.568700665267)*x[0]+(-0.431665912498)*x[1]**o+(0.881337230634)*x[1]
        arg[1,4,0,0]=(-0.940864464113)*x[0]**o+(-0.204392585977)*x[0]+(-0.872729326815)*x[1]**o+(0.454069061239)*x[1]
        arg[1,4,0,1]=(0.620544898911)*x[0]**o+(-0.194050689442)*x[0]+(0.399921161647)*x[1]**o+(0.812456444893)*x[1]
        arg[1,4,1,0]=(-0.398096053023)*x[0]**o+(-0.521802671926)*x[0]+(-0.525522030503)*x[1]**o+(-0.795276120062)*x[1]
        arg[1,4,1,1]=(0.277239330261)*x[0]**o+(-0.376159942175)*x[0]+(0.785902674573)*x[1]**o+(-0.00686958652716)*x[1]
        arg[1,4,2,0]=(-0.469242672532)*x[0]**o+(0.289432797089)*x[0]+(0.728305062026)*x[1]**o+(0.821761742668)*x[1]
        arg[1,4,2,1]=(0.828113301834)*x[0]**o+(-0.149301261166)*x[0]+(0.279230829643)*x[1]**o+(-0.971944918618)*x[1]
        arg[2,0,0,0]=(-0.724500208669)*x[0]**o+(-0.761179166402)*x[0]+(0.302409732402)*x[1]**o+(0.509101646177)*x[1]
        arg[2,0,0,1]=(-0.133300605917)*x[0]**o+(-0.0299057843108)*x[0]+(0.190028790905)*x[1]**o+(-0.668097775221)*x[1]
        arg[2,0,1,0]=(0.828019154028)*x[0]**o+(-0.446721913762)*x[0]+(0.745204383075)*x[1]**o+(0.363968853359)*x[1]
        arg[2,0,1,1]=(-0.653962912288)*x[0]**o+(0.753529804971)*x[0]+(-0.104712073949)*x[1]**o+(-0.44152589042)*x[1]
        arg[2,0,2,0]=(0.21572653818)*x[0]**o+(-0.553685840495)*x[0]+(0.372296357092)*x[1]**o+(-0.23566755183)*x[1]
        arg[2,0,2,1]=(-0.0943992743407)*x[0]**o+(0.677836956763)*x[0]+(-0.33055202826)*x[1]**o+(0.444414877097)*x[1]
        arg[2,1,0,0]=(-0.812500024009)*x[0]**o+(0.4856670967)*x[0]+(-0.891481528328)*x[1]**o+(-0.244043957515)*x[1]
        arg[2,1,0,1]=(-0.958152880518)*x[0]**o+(0.494470945352)*x[0]+(-0.000696143847979)*x[1]**o+(-0.036680781152)*x[1]
        arg[2,1,1,0]=(-0.81109120479)*x[0]**o+(-0.929246624754)*x[0]+(0.128356074186)*x[1]**o+(0.0107006959497)*x[1]
        arg[2,1,1,1]=(-0.72470858095)*x[0]**o+(-0.202254265463)*x[0]+(-0.995176229907)*x[1]**o+(-0.752592973024)*x[1]
        arg[2,1,2,0]=(-0.191810147267)*x[0]**o+(-0.819397723879)*x[0]+(0.306860971474)*x[1]**o+(0.681274705052)*x[1]
        arg[2,1,2,1]=(-0.867447323913)*x[0]**o+(-0.279148200325)*x[0]+(-0.195907590382)*x[1]**o+(-0.305826952304)*x[1]
        arg[2,2,0,0]=(0.983782298896)*x[0]**o+(-0.284064487432)*x[0]+(-0.0928333646602)*x[1]**o+(-0.929708520581)*x[1]
        arg[2,2,0,1]=(0.121619221979)*x[0]**o+(-0.876309915691)*x[0]+(0.455030678927)*x[1]**o+(0.635763125356)*x[1]
        arg[2,2,1,0]=(0.378726440151)*x[0]**o+(-0.626329947078)*x[0]+(-0.0239503446136)*x[1]**o+(0.32696940295)*x[1]
        arg[2,2,1,1]=(0.442586981104)*x[0]**o+(-0.427714534417)*x[0]+(0.906235883406)*x[1]**o+(0.819595435756)*x[1]
        arg[2,2,2,0]=(-0.564560890159)*x[0]**o+(-0.115075498726)*x[0]+(-0.520596320159)*x[1]**o+(-0.368366112222)*x[1]
        arg[2,2,2,1]=(0.215858165597)*x[0]**o+(-0.575665537635)*x[0]+(0.792289256264)*x[1]**o+(0.215575294284)*x[1]
        arg[2,3,0,0]=(-0.596335978041)*x[0]**o+(-0.677983108509)*x[0]+(-0.551360906631)*x[1]**o+(-0.874453959198)*x[1]
        arg[2,3,0,1]=(-0.913806992124)*x[0]**o+(0.192343143898)*x[0]+(0.821070238869)*x[1]**o+(-0.188359117641)*x[1]
        arg[2,3,1,0]=(-0.0831972104302)*x[0]**o+(-0.927005811846)*x[0]+(-0.43240265235)*x[1]**o+(-0.144332400422)*x[1]
        arg[2,3,1,1]=(0.643878326697)*x[0]**o+(0.212328499471)*x[0]+(0.72072318435)*x[1]**o+(-0.146144387035)*x[1]
        arg[2,3,2,0]=(0.843652876743)*x[0]**o+(-0.0386075132797)*x[0]+(0.754511827443)*x[1]**o+(-0.26945353599)*x[1]
        arg[2,3,2,1]=(0.0413857115672)*x[0]**o+(0.682727309008)*x[0]+(0.261249117327)*x[1]**o+(-0.753970539308)*x[1]
        arg[2,4,0,0]=(-0.607477693828)*x[0]**o+(-0.5870344058)*x[0]+(-0.0678823396451)*x[1]**o+(0.201869223616)*x[1]
        arg[2,4,0,1]=(-0.618172945146)*x[0]**o+(0.685084114438)*x[0]+(0.0457149578306)*x[1]**o+(-0.877971038996)*x[1]
        arg[2,4,1,0]=(0.722668769621)*x[0]**o+(0.302142466253)*x[0]+(0.630735604709)*x[1]**o+(-0.427754766567)*x[1]
        arg[2,4,1,1]=(-0.847113860795)*x[0]**o+(-0.779747246887)*x[0]+(-0.693659242703)*x[1]**o+(0.897451884568)*x[1]
        arg[2,4,2,0]=(-0.437578500878)*x[0]**o+(0.399407036331)*x[0]+(0.705207362097)*x[1]**o+(-0.271060427211)*x[1]
        arg[2,4,2,1]=(0.329570005179)*x[0]**o+(-0.712089906899)*x[0]+(0.860707329294)*x[1]**o+(0.553218273352)*x[1]
        arg[3,0,0,0]=(0.932602317046)*x[0]**o+(-0.948445141026)*x[0]+(0.549409370786)*x[1]**o+(0.782964260926)*x[1]
        arg[3,0,0,1]=(0.975116319103)*x[0]**o+(-0.0131313829738)*x[0]+(0.222376331882)*x[1]**o+(0.545375534163)*x[1]
        arg[3,0,1,0]=(0.019317277942)*x[0]**o+(0.918003018679)*x[0]+(-0.481194397466)*x[1]**o+(0.762780571026)*x[1]
        arg[3,0,1,1]=(-0.393162543578)*x[0]**o+(0.984304464298)*x[0]+(0.737901448601)*x[1]**o+(-0.890589811376)*x[1]
        arg[3,0,2,0]=(0.458096711279)*x[0]**o+(0.0343270226524)*x[0]+(0.278242150139)*x[1]**o+(-0.158378755134)*x[1]
        arg[3,0,2,1]=(0.307111052767)*x[0]**o+(-0.294724312785)*x[0]+(-0.489134379327)*x[1]**o+(-0.211139185303)*x[1]
        arg[3,1,0,0]=(0.420707084879)*x[0]**o+(-0.998173294986)*x[0]+(0.508808683)*x[1]**o+(-0.189902487212)*x[1]
        arg[3,1,0,1]=(-0.745253594401)*x[0]**o+(-0.344809947215)*x[0]+(0.198212694401)*x[1]**o+(-0.125428245606)*x[1]
        arg[3,1,1,0]=(0.535215252931)*x[0]**o+(0.565625295858)*x[0]+(-0.0741524230667)*x[1]**o+(-0.898600294889)*x[1]
        arg[3,1,1,1]=(0.193417593556)*x[0]**o+(-0.80053921694)*x[0]+(-0.664372557476)*x[1]**o+(0.47786060257)*x[1]
        arg[3,1,2,0]=(0.553690237434)*x[0]**o+(0.970763166888)*x[0]+(0.408815221365)*x[1]**o+(0.143175119269)*x[1]
        arg[3,1,2,1]=(-0.211683249218)*x[0]**o+(0.0643055656631)*x[0]+(-0.111028404397)*x[1]**o+(0.506093872018)*x[1]
        arg[3,2,0,0]=(-0.866119051438)*x[0]**o+(-0.417310807983)*x[0]+(-0.228385234131)*x[1]**o+(0.554977378653)*x[1]
        arg[3,2,0,1]=(-0.146618266885)*x[0]**o+(0.817733306502)*x[0]+(-0.709059272627)*x[1]**o+(-0.271140815458)*x[1]
        arg[3,2,1,0]=(0.622973500199)*x[0]**o+(-0.568858605993)*x[0]+(-0.651609755171)*x[1]**o+(-0.866184190672)*x[1]
        arg[3,2,1,1]=(-0.538565077894)*x[0]**o+(-0.676237465393)*x[0]+(0.106659392436)*x[1]**o+(0.488735158765)*x[1]
        arg[3,2,2,0]=(-0.325897413686)*x[0]**o+(0.99502293321)*x[0]+(-0.339096900793)*x[1]**o+(0.238083096263)*x[1]
        arg[3,2,2,1]=(0.861430484293)*x[0]**o+(-0.761739126386)*x[0]+(-0.10651751912)*x[1]**o+(0.688993400068)*x[1]
        arg[3,3,0,0]=(0.0890838652267)*x[0]**o+(0.151733228604)*x[0]+(0.230063809846)*x[1]**o+(-0.135041740614)*x[1]
        arg[3,3,0,1]=(-0.969251937403)*x[0]**o+(0.345709865473)*x[0]+(-0.309971427898)*x[1]**o+(-0.262217856737)*x[1]
        arg[3,3,1,0]=(0.640075232231)*x[0]**o+(-0.00817560718155)*x[0]+(0.55773788655)*x[1]**o+(0.642205213407)*x[1]
        arg[3,3,1,1]=(-0.350002630494)*x[0]**o+(0.373270925939)*x[0]+(-0.655757122598)*x[1]**o+(-0.74598967873)*x[1]
        arg[3,3,2,0]=(-0.262399316954)*x[0]**o+(-0.37991094527)*x[0]+(0.310310045807)*x[1]**o+(0.838942372822)*x[1]
        arg[3,3,2,1]=(-0.977039760257)*x[0]**o+(-0.0804751651971)*x[0]+(-0.844329846245)*x[1]**o+(0.905827257908)*x[1]
        arg[3,4,0,0]=(0.749594099085)*x[0]**o+(0.382880202285)*x[0]+(0.39699250956)*x[1]**o+(-0.708672318437)*x[1]
        arg[3,4,0,1]=(0.913703789336)*x[0]**o+(-0.497792024208)*x[0]+(-0.34125011876)*x[1]**o+(-0.657159430634)*x[1]
        arg[3,4,1,0]=(-0.397439923687)*x[0]**o+(0.595274583577)*x[0]+(-0.292338728991)*x[1]**o+(-0.314101057271)*x[1]
        arg[3,4,1,1]=(-0.403584993513)*x[0]**o+(-0.758728905384)*x[0]+(-0.182501473325)*x[1]**o+(-0.0942108622967)*x[1]
        arg[3,4,2,0]=(-0.263608698751)*x[0]**o+(-0.591451983536)*x[0]+(0.620412677389)*x[1]**o+(0.600005456466)*x[1]
        arg[3,4,2,1]=(-0.633502278259)*x[0]**o+(0.733127573934)*x[0]+(-0.765468966987)*x[1]**o+(-0.24211874772)*x[1]
        ref[0,0,0,0]=(-0.750823220445)/(o+1.)+(0.200562218922)+(-0.177078428867)*0.5**o
        ref[0,0,0,1]=(-0.0127659464437)/(o+1.)+(-0.374838430629)+(-0.682838671264)*0.5**o
        ref[0,0,1,0]=(-0.932452125435)/(o+1.)+(0.64729240195)+(0.352439319413)*0.5**o
        ref[0,0,1,1]=(-0.702758889809)/(o+1.)+(0.143291706728)+(-0.354033408023)*0.5**o
        ref[0,0,2,0]=(-0.535244256862)/(o+1.)+(-0.0166533987843)+(0.706826713479)*0.5**o
        ref[0,0,2,1]=(0.742770093639)/(o+1.)+(0.0478036100749)+(-0.893497856528)*0.5**o
        ref[0,1,0,0]=(-0.7740511254)/(o+1.)+(0.151807063155)+(-0.536624329097)*0.5**o
        ref[0,1,0,1]=(0.196975646933)/(o+1.)+(-0.359195588208)+(0.388396767346)*0.5**o
        ref[0,1,1,0]=(0.372752949038)/(o+1.)+(-0.609511042662)+(-0.125771297178)*0.5**o
        ref[0,1,1,1]=(-0.726599229867)/(o+1.)+(-0.0676212180057)+(-0.313637256508)*0.5**o
        ref[0,1,2,0]=(1.60583902129e-05)/(o+1.)+(0.305063814416)+(0.92425817047)*0.5**o
        ref[0,1,2,1]=(-0.888060617134)/(o+1.)+(0.1507514326)+(-0.0981827607557)*0.5**o
        ref[0,2,0,0]=(-0.228336105988)/(o+1.)+(0.117775628113)+(-0.871590768072)*0.5**o
        ref[0,2,0,1]=(-0.509662470381)/(o+1.)+(0.0160681615592)+(-0.628675127006)*0.5**o
        ref[0,2,1,0]=(-0.587902586916)/(o+1.)+(0.346247748963)+(-0.767528433339)*0.5**o
        ref[0,2,1,1]=(-0.78470263647)/(o+1.)+(0.419883543895)+(0.388984380682)*0.5**o
        ref[0,2,2,0]=(-0.637159792471)/(o+1.)+(0.187921397135)+(-0.910514531543)*0.5**o
        ref[0,2,2,1]=(-0.298806908011)/(o+1.)+(-0.745958567089)+(0.570535552656)*0.5**o
        ref[0,3,0,0]=(-0.91370863065)/(o+1.)+(0.136099326137)+(0.916864925136)*0.5**o
        ref[0,3,0,1]=(0.699832871063)/(o+1.)+(0.0843138801635)+(-0.851147484632)*0.5**o
        ref[0,3,1,0]=(0.0203170502559)/(o+1.)+(0.687200666868)+(0.461273862468)*0.5**o
        ref[0,3,1,1]=(-0.00587359808953)/(o+1.)+(0.238032934986)+(0.63984877633)*0.5**o
        ref[0,3,2,0]=(0.689553978928)/(o+1.)+(-0.00771127379434)+(-0.159416471436)*0.5**o
        ref[0,3,2,1]=(0.0370277809126)/(o+1.)+(0.00770755686953)+(0.782397836312)*0.5**o
        ref[0,4,0,0]=(0.110218625254)/(o+1.)+(0.288682475662)+(-0.0063433205485)*0.5**o
        ref[0,4,0,1]=(0.910723121611)/(o+1.)+(0.401925056417)+(-0.128215646666)*0.5**o
        ref[0,4,1,0]=(-0.281954207008)/(o+1.)+(-0.117618175276)+(-0.604528735513)*0.5**o
        ref[0,4,1,1]=(-0.761226249914)/(o+1.)+(0.0260675614951)+(0.65096807032)*0.5**o
        ref[0,4,2,0]=(-0.780671499903)/(o+1.)+(-0.244952280649)+(-0.710718948634)*0.5**o
        ref[0,4,2,1]=(0.709902333411)/(o+1.)+(0.63468720432)+(-0.0443187506467)*0.5**o
        ref[1,0,0,0]=(-0.713098084853)/(o+1.)+(0.026951171138)+(0.731890221753)*0.5**o
        ref[1,0,0,1]=(-0.427105331069)/(o+1.)+(0.312079508459)+(-0.130335125365)*0.5**o
        ref[1,0,1,0]=(0.344459670502)/(o+1.)+(-0.0918192826838)+(-0.203908509107)*0.5**o
        ref[1,0,1,1]=(-0.0683630778679)/(o+1.)+(0.414378635447)+(0.186770929724)*0.5**o
        ref[1,0,2,0]=(0.127955472707)/(o+1.)+(-0.147180699)+(-0.891275312975)*0.5**o
        ref[1,0,2,1]=(-0.052031562679)/(o+1.)+(0.955043298554)+(-0.74620344519)*0.5**o
        ref[1,1,0,0]=(0.783714758356)/(o+1.)+(0.226618827568)+(0.871423198418)*0.5**o
        ref[1,1,0,1]=(-0.00888785913795)/(o+1.)+(-0.333567151999)+(0.943197824902)*0.5**o
        ref[1,1,1,0]=(0.481063204532)/(o+1.)+(0.224237358325)+(-0.74971981056)*0.5**o
        ref[1,1,1,1]=(-0.0110588803484)/(o+1.)+(-0.535252745501)+(-0.431776603043)*0.5**o
        ref[1,1,2,0]=(-0.646224518073)/(o+1.)+(-0.208035583419)+(0.578105297721)*0.5**o
        ref[1,1,2,1]=(-0.101534585934)/(o+1.)+(-0.440109735326)+(0.442066689582)*0.5**o
        ref[1,2,0,0]=(0.185493587693)/(o+1.)+(0.184679874576)+(-0.206702379228)*0.5**o
        ref[1,2,0,1]=(0.141622971369)/(o+1.)+(0.667962084728)+(0.79092697362)*0.5**o
        ref[1,2,1,0]=(0.249095964198)/(o+1.)+(0.24773637535)+(0.914281902407)*0.5**o
        ref[1,2,1,1]=(0.262528290838)/(o+1.)+(0.655403162506)+(-0.602926709108)*0.5**o
        ref[1,2,2,0]=(0.144972402754)/(o+1.)+(-0.408191062607)+(-0.724435531493)*0.5**o
        ref[1,2,2,1]=(-0.150439352532)/(o+1.)+(-0.16492771172)+(-0.611538590545)*0.5**o
        ref[1,3,0,0]=(-0.509426399026)/(o+1.)+(-0.267418424334)+(-0.913553947133)*0.5**o
        ref[1,3,0,1]=(0.540358729462)/(o+1.)+(0.47155354967)+(-0.709834132519)*0.5**o
        ref[1,3,1,0]=(0.0135015706177)/(o+1.)+(0.0977365649293)+(-0.30788139278)*0.5**o
        ref[1,3,1,1]=(0.076082439418)/(o+1.)+(0.331185729735)+(-0.243327801257)*0.5**o
        ref[1,3,2,0]=(-0.0741034599215)/(o+1.)+(0.168033143879)+(-0.0290765899224)*0.5**o
        ref[1,3,2,1]=(-0.431665912498)/(o+1.)+(0.72501894795)+(-0.751374584848)*0.5**o
        ref[1,4,0,0]=(-0.872729326815)/(o+1.)+(0.124838237631)+(-0.940864464113)*0.5**o
        ref[1,4,0,1]=(0.399921161647)/(o+1.)+(0.309202877726)+(0.620544898911)*0.5**o
        ref[1,4,1,0]=(-0.525522030503)/(o+1.)+(-0.658539395994)+(-0.398096053023)*0.5**o
        ref[1,4,1,1]=(0.785902674573)/(o+1.)+(-0.191514764351)+(0.277239330261)*0.5**o
        ref[1,4,2,0]=(0.728305062026)/(o+1.)+(0.555597269878)+(-0.469242672532)*0.5**o
        ref[1,4,2,1]=(0.279230829643)/(o+1.)+(-0.560623089892)+(0.828113301834)*0.5**o
        ref[2,0,0,0]=(0.302409732402)/(o+1.)+(-0.126038760113)+(-0.724500208669)*0.5**o
        ref[2,0,0,1]=(0.190028790905)/(o+1.)+(-0.349001779766)+(-0.133300605917)*0.5**o
        ref[2,0,1,0]=(0.745204383075)/(o+1.)+(-0.0413765302013)+(0.828019154028)*0.5**o
        ref[2,0,1,1]=(-0.104712073949)/(o+1.)+(0.156001957276)+(-0.653962912288)*0.5**o
        ref[2,0,2,0]=(0.372296357092)/(o+1.)+(-0.394676696163)+(0.21572653818)*0.5**o
        ref[2,0,2,1]=(-0.33055202826)/(o+1.)+(0.56112591693)+(-0.0943992743407)*0.5**o
        ref[2,1,0,0]=(-0.891481528328)/(o+1.)+(0.120811569592)+(-0.812500024009)*0.5**o
        ref[2,1,0,1]=(-0.000696143847979)/(o+1.)+(0.2288950821)+(-0.958152880518)*0.5**o
        ref[2,1,1,0]=(0.128356074186)/(o+1.)+(-0.459272964402)+(-0.81109120479)*0.5**o
        ref[2,1,1,1]=(-0.995176229907)/(o+1.)+(-0.477423619244)+(-0.72470858095)*0.5**o
        ref[2,1,2,0]=(0.306860971474)/(o+1.)+(-0.0690615094134)+(-0.191810147267)*0.5**o
        ref[2,1,2,1]=(-0.195907590382)/(o+1.)+(-0.292487576314)+(-0.867447323913)*0.5**o
        ref[2,2,0,0]=(-0.0928333646602)/(o+1.)+(-0.606886504006)+(0.983782298896)*0.5**o
        ref[2,2,0,1]=(0.455030678927)/(o+1.)+(-0.120273395167)+(0.121619221979)*0.5**o
        ref[2,2,1,0]=(-0.0239503446136)/(o+1.)+(-0.149680272064)+(0.378726440151)*0.5**o
        ref[2,2,1,1]=(0.906235883406)/(o+1.)+(0.19594045067)+(0.442586981104)*0.5**o
        ref[2,2,2,0]=(-0.520596320159)/(o+1.)+(-0.241720805474)+(-0.564560890159)*0.5**o
        ref[2,2,2,1]=(0.792289256264)/(o+1.)+(-0.180045121676)+(0.215858165597)*0.5**o
        ref[2,3,0,0]=(-0.551360906631)/(o+1.)+(-0.776218533854)+(-0.596335978041)*0.5**o
        ref[2,3,0,1]=(0.821070238869)/(o+1.)+(0.00199201312847)+(-0.913806992124)*0.5**o
        ref[2,3,1,0]=(-0.43240265235)/(o+1.)+(-0.535669106134)+(-0.0831972104302)*0.5**o
        ref[2,3,1,1]=(0.72072318435)/(o+1.)+(0.0330920562179)+(0.643878326697)*0.5**o
        ref[2,3,2,0]=(0.754511827443)/(o+1.)+(-0.154030524635)+(0.843652876743)*0.5**o
        ref[2,3,2,1]=(0.261249117327)/(o+1.)+(-0.03562161515)+(0.0413857115672)*0.5**o
        ref[2,4,0,0]=(-0.0678823396451)/(o+1.)+(-0.192582591092)+(-0.607477693828)*0.5**o
        ref[2,4,0,1]=(0.0457149578306)/(o+1.)+(-0.0964434622791)+(-0.618172945146)*0.5**o
        ref[2,4,1,0]=(0.630735604709)/(o+1.)+(-0.0628061501569)+(0.722668769621)*0.5**o
        ref[2,4,1,1]=(-0.693659242703)/(o+1.)+(0.0588523188405)+(-0.847113860795)*0.5**o
        ref[2,4,2,0]=(0.705207362097)/(o+1.)+(0.0641733045603)+(-0.437578500878)*0.5**o
        ref[2,4,2,1]=(0.860707329294)/(o+1.)+(-0.0794358167735)+(0.329570005179)*0.5**o
        ref[3,0,0,0]=(0.549409370786)/(o+1.)+(-0.0827404400499)+(0.932602317046)*0.5**o
        ref[3,0,0,1]=(0.222376331882)/(o+1.)+(0.266122075595)+(0.975116319103)*0.5**o
        ref[3,0,1,0]=(-0.481194397466)/(o+1.)+(0.840391794852)+(0.019317277942)*0.5**o
        ref[3,0,1,1]=(0.737901448601)/(o+1.)+(0.0468573264614)+(-0.393162543578)*0.5**o
        ref[3,0,2,0]=(0.278242150139)/(o+1.)+(-0.0620258662406)+(0.458096711279)*0.5**o
        ref[3,0,2,1]=(-0.489134379327)/(o+1.)+(-0.252931749044)+(0.307111052767)*0.5**o
        ref[3,1,0,0]=(0.508808683)/(o+1.)+(-0.594037891099)+(0.420707084879)*0.5**o
        ref[3,1,0,1]=(0.198212694401)/(o+1.)+(-0.23511909641)+(-0.745253594401)*0.5**o
        ref[3,1,1,0]=(-0.0741524230667)/(o+1.)+(-0.166487499516)+(0.535215252931)*0.5**o
        ref[3,1,1,1]=(-0.664372557476)/(o+1.)+(-0.161339307185)+(0.193417593556)*0.5**o
        ref[3,1,2,0]=(0.408815221365)/(o+1.)+(0.556969143078)+(0.553690237434)*0.5**o
        ref[3,1,2,1]=(-0.111028404397)/(o+1.)+(0.28519971884)+(-0.211683249218)*0.5**o
        ref[3,2,0,0]=(-0.228385234131)/(o+1.)+(0.068833285335)+(-0.866119051438)*0.5**o
        ref[3,2,0,1]=(-0.709059272627)/(o+1.)+(0.273296245522)+(-0.146618266885)*0.5**o
        ref[3,2,1,0]=(-0.651609755171)/(o+1.)+(-0.717521398332)+(0.622973500199)*0.5**o
        ref[3,2,1,1]=(0.106659392436)/(o+1.)+(-0.0937511533137)+(-0.538565077894)*0.5**o
        ref[3,2,2,0]=(-0.339096900793)/(o+1.)+(0.616553014737)+(-0.325897413686)*0.5**o
        ref[3,2,2,1]=(-0.10651751912)/(o+1.)+(-0.0363728631586)+(0.861430484293)*0.5**o
        ref[3,3,0,0]=(0.230063809846)/(o+1.)+(0.0083457439947)+(0.0890838652267)*0.5**o
        ref[3,3,0,1]=(-0.309971427898)/(o+1.)+(0.0417460043682)+(-0.969251937403)*0.5**o
        ref[3,3,1,0]=(0.55773788655)/(o+1.)+(0.317014803113)+(0.640075232231)*0.5**o
        ref[3,3,1,1]=(-0.655757122598)/(o+1.)+(-0.186359376395)+(-0.350002630494)*0.5**o
        ref[3,3,2,0]=(0.310310045807)/(o+1.)+(0.229515713776)+(-0.262399316954)*0.5**o
        ref[3,3,2,1]=(-0.844329846245)/(o+1.)+(0.412676046356)+(-0.977039760257)*0.5**o
        ref[3,4,0,0]=(0.39699250956)/(o+1.)+(-0.162896058076)+(0.749594099085)*0.5**o
        ref[3,4,0,1]=(-0.34125011876)/(o+1.)+(-0.577475727421)+(0.913703789336)*0.5**o
        ref[3,4,1,0]=(-0.292338728991)/(o+1.)+(0.140586763153)+(-0.397439923687)*0.5**o
        ref[3,4,1,1]=(-0.182501473325)/(o+1.)+(-0.42646988384)+(-0.403584993513)*0.5**o
        ref[3,4,2,0]=(0.620412677389)/(o+1.)+(0.00427673646471)+(-0.263608698751)*0.5**o
        ref[3,4,2,1]=(-0.765468966987)/(o+1.)+(0.245504413107)+(-0.633502278259)*0.5**o
      else:
        arg[0,0,0,0]=(0.480896043196)*x[0]**o+(0.113355160927)*x[0]+(0.498630339771)*x[1]**o+(0.0149514785179)*x[1]+(-0.944439525708)*x[2]**o+(-0.349299625744)*x[2]
        arg[0,0,0,1]=(-0.609632888023)*x[0]**o+(-0.576508904483)*x[0]+(-0.0610487008887)*x[1]**o+(-0.844087384492)*x[1]+(-0.715427733517)*x[2]**o+(-0.263424969452)*x[2]
        arg[0,0,1,0]=(0.205065167181)*x[0]**o+(-0.840225514826)*x[0]+(0.808161497555)*x[1]**o+(0.685902712814)*x[1]+(-0.625183952417)*x[2]**o+(-0.0947932775112)*x[2]
        arg[0,0,1,1]=(0.875716461075)*x[0]**o+(0.814425331995)*x[0]+(-0.805331622235)*x[1]**o+(-0.287737068993)*x[1]+(-0.365715978159)*x[2]**o+(0.689907502778)*x[2]
        arg[0,0,2,0]=(0.711372030289)*x[0]**o+(-0.393612062405)*x[0]+(0.642793863702)*x[1]**o+(-0.66636178423)*x[1]+(0.00488425355017)*x[2]**o+(-0.444523817626)*x[2]
        arg[0,0,2,1]=(-0.0536842545617)*x[0]**o+(-0.714279418499)*x[0]+(-0.487452457649)*x[1]**o+(0.917805820162)*x[1]+(0.445007135027)*x[2]**o+(0.101201310289)*x[2]
        arg[0,1,0,0]=(-0.710091748251)*x[0]**o+(0.470243562863)*x[0]+(-0.0178862343634)*x[1]**o+(0.393644042332)*x[1]+(-0.952839337577)*x[2]**o+(-0.152392013939)*x[2]
        arg[0,1,0,1]=(0.250989366649)*x[0]**o+(0.592158236555)*x[0]+(0.698185234877)*x[1]**o+(0.112154263635)*x[1]+(0.843915127813)*x[2]**o+(-0.825331842242)*x[2]
        arg[0,1,1,0]=(0.690360096543)*x[0]**o+(0.78906430602)*x[0]+(0.596189773531)*x[1]**o+(-0.293452572358)*x[1]+(0.737565980909)*x[2]**o+(-0.96988090057)*x[2]
        arg[0,1,1,1]=(-0.815220933301)*x[0]**o+(-0.875224889387)*x[0]+(-0.0562950344908)*x[1]**o+(0.56567431969)*x[1]+(0.133489969036)*x[2]**o+(0.299403989148)*x[2]
        arg[0,1,2,0]=(0.169442617085)*x[0]**o+(-0.413300800662)*x[0]+(0.580773652363)*x[1]**o+(-0.627706012397)*x[1]+(-0.730218498677)*x[2]**o+(0.587233195345)*x[2]
        arg[0,1,2,1]=(-0.110636656112)*x[0]**o+(-0.302213944624)*x[0]+(0.643229732019)*x[1]**o+(-0.651805494828)*x[1]+(-0.528276901455)*x[2]**o+(-0.272057635286)*x[2]
        arg[0,2,0,0]=(0.340661037085)*x[0]**o+(-0.672207039496)*x[0]+(0.676291394824)*x[1]**o+(0.596998737167)*x[1]+(-0.689800391841)*x[2]**o+(-0.134273731064)*x[2]
        arg[0,2,0,1]=(0.185282186176)*x[0]**o+(0.252489447393)*x[0]+(-0.0447305595755)*x[1]**o+(-0.558361084175)*x[1]+(0.922681220429)*x[2]**o+(-0.484588607971)*x[2]
        arg[0,2,1,0]=(0.238772787004)*x[0]**o+(0.987311136495)*x[0]+(0.215079265839)*x[1]**o+(-0.797762453244)*x[1]+(0.0329055379323)*x[2]**o+(-0.88999503049)*x[2]
        arg[0,2,1,1]=(-0.175884373587)*x[0]**o+(0.806690051494)*x[0]+(0.0086744009376)*x[1]**o+(0.286131063746)*x[1]+(0.536839534689)*x[2]**o+(-0.0946282917135)*x[2]
        arg[0,2,2,0]=(-0.615695203188)*x[0]**o+(0.600565627235)*x[0]+(-0.869759996675)*x[1]**o+(0.648192341374)*x[1]+(-0.952728999251)*x[2]**o+(0.564068973312)*x[2]
        arg[0,2,2,1]=(-0.0992379560876)*x[0]**o+(-0.897179656441)*x[0]+(-0.519600820875)*x[1]**o+(0.70507167679)*x[1]+(0.903640275297)*x[2]**o+(-0.645039734963)*x[2]
        arg[0,3,0,0]=(0.432415072474)*x[0]**o+(-0.748137008742)*x[0]+(0.875883324091)*x[1]**o+(0.706172632952)*x[1]+(-0.47486904255)*x[2]**o+(0.238106343647)*x[2]
        arg[0,3,0,1]=(-0.656397468061)*x[0]**o+(-0.530867058489)*x[0]+(-0.904148914114)*x[1]**o+(0.87213389632)*x[1]+(0.0612056123548)*x[2]**o+(0.288620109264)*x[2]
        arg[0,3,1,0]=(-0.472896985897)*x[0]**o+(0.83328574859)*x[0]+(0.744310819801)*x[1]**o+(0.0601035189502)*x[1]+(-0.198693260491)*x[2]**o+(-0.882304016543)*x[2]
        arg[0,3,1,1]=(-0.515568063685)*x[0]**o+(0.484765564236)*x[0]+(0.754304059424)*x[1]**o+(0.654790289063)*x[1]+(-0.102335538236)*x[2]**o+(-0.0705158755887)*x[2]
        arg[0,3,2,0]=(-0.709802317012)*x[0]**o+(-0.346551215683)*x[0]+(0.312943005623)*x[1]**o+(0.476828461567)*x[1]+(0.901171463566)*x[2]**o+(0.538981245022)*x[2]
        arg[0,3,2,1]=(-0.20057625974)*x[0]**o+(0.0458666109661)*x[0]+(0.909610730794)*x[1]**o+(0.145637065192)*x[1]+(0.666420073244)*x[2]**o+(0.0550124082656)*x[2]
        arg[0,4,0,0]=(-0.726921583373)*x[0]**o+(0.891306726707)*x[0]+(-0.189151461127)*x[1]**o+(0.479274047923)*x[1]+(-0.129077261233)*x[2]**o+(-0.873599130413)*x[2]
        arg[0,4,0,1]=(0.765272127757)*x[0]**o+(0.159307695759)*x[0]+(-0.569162341331)*x[1]**o+(-0.530158933095)*x[1]+(-0.216727788754)*x[2]**o+(-0.519551969741)*x[2]
        arg[0,4,1,0]=(-0.613269202518)*x[0]**o+(0.612846846793)*x[0]+(-0.187285796021)*x[1]**o+(-0.462603518143)*x[1]+(-0.739162067312)*x[2]**o+(0.0737393951189)*x[2]
        arg[0,4,1,1]=(-0.727485674691)*x[0]**o+(-0.461427895477)*x[0]+(0.917029475887)*x[1]**o+(0.265874989879)*x[1]+(0.602505162351)*x[2]**o+(-0.372206475027)*x[2]
        arg[0,4,2,0]=(-0.663071984573)*x[0]**o+(0.511217165998)*x[0]+(-0.472765672078)*x[1]**o+(0.814962161218)*x[1]+(-0.608890631297)*x[2]**o+(-0.284872941025)*x[2]
        arg[0,4,2,1]=(0.893396132015)*x[0]**o+(0.183468149957)*x[0]+(0.647812435076)*x[1]**o+(-0.398672312216)*x[1]+(0.6359553902)*x[2]**o+(0.485509522238)*x[2]
        arg[1,0,0,0]=(-0.0771632829864)*x[0]**o+(-0.723025089864)*x[0]+(0.486209418481)*x[1]**o+(0.0869704919133)*x[1]+(-0.365499197901)*x[2]**o+(-0.0796713538898)*x[2]
        arg[1,0,0,1]=(0.133617787097)*x[0]**o+(0.993872304226)*x[0]+(-0.898504795319)*x[1]**o+(0.656793416434)*x[1]+(-0.821841885906)*x[2]**o+(0.726972748423)*x[2]
        arg[1,0,1,0]=(0.174277385391)*x[0]**o+(-0.301857401564)*x[0]+(-0.846153302312)*x[1]**o+(-0.559076632516)*x[1]+(-0.89870762709)*x[2]**o+(0.640171641512)*x[2]
        arg[1,0,1,1]=(0.795443369588)*x[0]**o+(-0.346082013333)*x[0]+(-0.439713049567)*x[1]**o+(0.860680213906)*x[1]+(-0.466761099917)*x[2]**o+(-0.843656974958)*x[2]
        arg[1,0,2,0]=(0.482814141263)*x[0]**o+(0.142978643528)*x[0]+(-0.735291704767)*x[1]**o+(-0.925675892787)*x[1]+(0.747235439093)*x[2]**o+(-0.569454067374)*x[2]
        arg[1,0,2,1]=(0.619359374773)*x[0]**o+(-0.547172728307)*x[0]+(0.0829642963797)*x[1]**o+(-0.0180882163346)*x[1]+(0.328627445196)*x[2]**o+(-0.274608686384)*x[2]
        arg[1,1,0,0]=(-0.510375454058)*x[0]**o+(-0.779894165181)*x[0]+(0.656168894408)*x[1]**o+(-0.449484133421)*x[1]+(-0.147284453744)*x[2]**o+(-0.488384108271)*x[2]
        arg[1,1,0,1]=(-0.302705575836)*x[0]**o+(0.256670307547)*x[0]+(0.110465379068)*x[1]**o+(0.319291387545)*x[1]+(0.408832172454)*x[2]**o+(0.718793126931)*x[2]
        arg[1,1,1,0]=(0.520882033451)*x[0]**o+(0.412524861348)*x[0]+(0.761080123209)*x[1]**o+(0.401223078157)*x[1]+(-0.65945839721)*x[2]**o+(0.71349489395)*x[2]
        arg[1,1,1,1]=(0.624622427594)*x[0]**o+(0.847907716976)*x[0]+(0.855268296474)*x[1]**o+(-0.671685662164)*x[1]+(-0.407895715308)*x[2]**o+(-0.719054767733)*x[2]
        arg[1,1,2,0]=(-0.044254701409)*x[0]**o+(0.72911290175)*x[0]+(0.869467934211)*x[1]**o+(0.613006764744)*x[1]+(0.678602796496)*x[2]**o+(0.388923815748)*x[2]
        arg[1,1,2,1]=(-0.240571107795)*x[0]**o+(0.838382917252)*x[0]+(0.0342933298383)*x[1]**o+(0.595421789197)*x[1]+(-0.989088178853)*x[2]**o+(-0.20002339454)*x[2]
        arg[1,2,0,0]=(0.0596937021722)*x[0]**o+(0.196768537308)*x[0]+(0.0834459161806)*x[1]**o+(0.194049508373)*x[1]+(-0.097309631361)*x[2]**o+(-0.867555781902)*x[2]
        arg[1,2,0,1]=(-0.935541542647)*x[0]**o+(-0.00702430636955)*x[0]+(-0.384657002538)*x[1]**o+(0.186884376746)*x[1]+(0.97207387841)*x[2]**o+(0.452460932869)*x[2]
        arg[1,2,1,0]=(0.276741406646)*x[0]**o+(0.838348610585)*x[0]+(-0.403054578066)*x[1]**o+(0.655318135938)*x[1]+(0.37614761635)*x[2]**o+(0.82736662494)*x[2]
        arg[1,2,1,1]=(0.828731604537)*x[0]**o+(0.762067030749)*x[0]+(-0.881827056079)*x[1]**o+(0.250928989647)*x[1]+(-0.771040367979)*x[2]**o+(0.239950799138)*x[2]
        arg[1,2,2,0]=(-0.380797583569)*x[0]**o+(0.112805267222)*x[0]+(-0.454730562411)*x[1]**o+(0.926061735219)*x[1]+(0.348331256905)*x[2]**o+(-0.725641369775)*x[2]
        arg[1,2,2,1]=(0.257022926473)*x[0]**o+(0.0160557858816)*x[0]+(-0.291367922966)*x[1]**o+(-0.611341260765)*x[1]+(0.232472950268)*x[2]**o+(-0.738139185521)*x[2]
        arg[1,3,0,0]=(-0.645340088797)*x[0]**o+(0.480235460274)*x[0]+(0.135346246274)*x[1]**o+(-0.560730820351)*x[1]+(-0.635582025321)*x[2]**o+(-0.476912924392)*x[2]
        arg[1,3,0,1]=(0.124132693667)*x[0]**o+(-0.813993821983)*x[0]+(-0.108593659227)*x[1]**o+(-0.432186668568)*x[1]+(0.209382070426)*x[2]**o+(0.928912669667)*x[2]
        arg[1,3,1,0]=(-0.7660131868)*x[0]**o+(-0.30274186797)*x[0]+(0.140498262981)*x[1]**o+(0.993084314471)*x[1]+(0.684043026981)*x[2]**o+(-0.339724714081)*x[2]
        arg[1,3,1,1]=(-0.150578257351)*x[0]**o+(-0.674840941109)*x[0]+(0.469240334383)*x[1]**o+(0.729963338278)*x[1]+(-0.800829664334)*x[2]**o+(0.0467922871395)*x[2]
        arg[1,3,2,0]=(-0.336661555676)*x[0]**o+(0.705661383379)*x[0]+(-0.76993460355)*x[1]**o+(0.459144989218)*x[1]+(0.545901025663)*x[2]**o+(0.385988013614)*x[2]
        arg[1,3,2,1]=(0.165928745462)*x[0]**o+(0.540718600752)*x[0]+(0.172773708803)*x[1]**o+(0.253664803784)*x[1]+(-0.264896626947)*x[2]**o+(-0.478577475495)*x[2]
        arg[1,4,0,0]=(0.283586653542)*x[0]**o+(-0.104364884571)*x[0]+(-0.840555806526)*x[1]**o+(0.701868415915)*x[1]+(-0.290943689679)*x[2]**o+(-0.73024942052)*x[2]
        arg[1,4,0,1]=(-0.772849835334)*x[0]**o+(-0.432170589951)*x[0]+(-0.376099423267)*x[1]**o+(0.0215357491826)*x[1]+(-0.346787011464)*x[2]**o+(0.620411709813)*x[2]
        arg[1,4,1,0]=(-0.718524564964)*x[0]**o+(0.119936682579)*x[0]+(-0.994276256462)*x[1]**o+(-0.839783920473)*x[1]+(-0.454576336782)*x[2]**o+(-0.515489988915)*x[2]
        arg[1,4,1,1]=(-0.771664154234)*x[0]**o+(-0.584052299067)*x[0]+(-0.0233756210307)*x[1]**o+(-0.174442816636)*x[1]+(-0.702383477714)*x[2]**o+(-0.279952858144)*x[2]
        arg[1,4,2,0]=(-0.591902790978)*x[0]**o+(-0.151826578123)*x[0]+(0.923906788219)*x[1]**o+(0.301090558476)*x[1]+(-0.518314647811)*x[2]**o+(-0.784331717681)*x[2]
        arg[1,4,2,1]=(0.303567440092)*x[0]**o+(0.744948852961)*x[0]+(0.719939071212)*x[1]**o+(-0.161138638319)*x[1]+(0.924953261766)*x[2]**o+(-0.875565390131)*x[2]
        arg[2,0,0,0]=(0.197574550087)*x[0]**o+(-0.962018785397)*x[0]+(0.498378037582)*x[1]**o+(-0.334199169606)*x[1]+(0.852426750588)*x[2]**o+(-0.650173474618)*x[2]
        arg[2,0,0,1]=(0.0663499012532)*x[0]**o+(0.00812461763474)*x[0]+(0.237233309304)*x[1]**o+(-0.839377881658)*x[1]+(-0.67246872567)*x[2]**o+(0.677471261363)*x[2]
        arg[2,0,1,0]=(-0.789090971877)*x[0]**o+(-0.387724517273)*x[0]+(0.455954877641)*x[1]**o+(-0.356995254842)*x[1]+(-0.897019846935)*x[2]**o+(-0.0703543939474)*x[2]
        arg[2,0,1,1]=(0.222873274437)*x[0]**o+(0.405228982254)*x[0]+(-0.532697993554)*x[1]**o+(-0.836340143789)*x[1]+(-0.139525062661)*x[2]**o+(-0.916307770461)*x[2]
        arg[2,0,2,0]=(-0.300828039715)*x[0]**o+(-0.584457605313)*x[0]+(-0.443125109903)*x[1]**o+(0.292734004842)*x[1]+(-0.55875691535)*x[2]**o+(0.391237822545)*x[2]
        arg[2,0,2,1]=(-0.501507304569)*x[0]**o+(-0.203549089015)*x[0]+(-0.113050138121)*x[1]**o+(0.712978728546)*x[1]+(-0.642617274635)*x[2]**o+(-0.307119076865)*x[2]
        arg[2,1,0,0]=(-0.809237327867)*x[0]**o+(0.984684980548)*x[0]+(-0.643310231704)*x[1]**o+(0.759430082065)*x[1]+(-0.393902220205)*x[2]**o+(-0.900173371318)*x[2]
        arg[2,1,0,1]=(-0.722795722506)*x[0]**o+(0.433309366611)*x[0]+(-0.559871890844)*x[1]**o+(0.417301432603)*x[1]+(-0.364192390887)*x[2]**o+(0.64549736058)*x[2]
        arg[2,1,1,0]=(-0.551325611251)*x[0]**o+(-0.638520772682)*x[0]+(0.222655558982)*x[1]**o+(-0.230002147207)*x[1]+(-0.290821163957)*x[2]**o+(0.0647728628594)*x[2]
        arg[2,1,1,1]=(0.743453045588)*x[0]**o+(-0.901163108996)*x[0]+(-0.546031302921)*x[1]**o+(-0.512614476111)*x[1]+(-0.886430682715)*x[2]**o+(0.720650678026)*x[2]
        arg[2,1,2,0]=(-0.033787671958)*x[0]**o+(0.930927279443)*x[0]+(-0.530482747719)*x[1]**o+(0.714053784656)*x[1]+(-0.0978565618487)*x[2]**o+(-0.0423072209447)*x[2]
        arg[2,1,2,1]=(0.127121989623)*x[0]**o+(0.809654796091)*x[0]+(0.718533701795)*x[1]**o+(0.542738494557)*x[1]+(-0.342918607246)*x[2]**o+(-0.627991145352)*x[2]
        arg[2,2,0,0]=(-0.867530011926)*x[0]**o+(0.870295856967)*x[0]+(-0.13043222912)*x[1]**o+(0.2670058777)*x[1]+(0.395016123247)*x[2]**o+(-0.811207959629)*x[2]
        arg[2,2,0,1]=(-0.00858729113056)*x[0]**o+(-0.885551110851)*x[0]+(0.820441870015)*x[1]**o+(-0.348896120549)*x[1]+(0.295475751596)*x[2]**o+(-0.672688762902)*x[2]
        arg[2,2,1,0]=(-0.982123891355)*x[0]**o+(0.232770408062)*x[0]+(0.374641341895)*x[1]**o+(-0.21375197203)*x[1]+(0.436992590849)*x[2]**o+(-0.473399371298)*x[2]
        arg[2,2,1,1]=(-0.415533190881)*x[0]**o+(0.0945615271507)*x[0]+(-0.281959759072)*x[1]**o+(0.0807942449559)*x[1]+(0.225501997354)*x[2]**o+(0.305247648387)*x[2]
        arg[2,2,2,0]=(-0.559916632426)*x[0]**o+(-0.372679495574)*x[0]+(0.392857820158)*x[1]**o+(0.726824319268)*x[1]+(0.36923535014)*x[2]**o+(-0.208169256022)*x[2]
        arg[2,2,2,1]=(-0.869055045156)*x[0]**o+(0.359188490398)*x[0]+(0.702110073265)*x[1]**o+(0.668757858248)*x[1]+(-0.499300337779)*x[2]**o+(0.0471054271112)*x[2]
        arg[2,3,0,0]=(0.754089271004)*x[0]**o+(-0.0240479506036)*x[0]+(-0.0775830745475)*x[1]**o+(0.215564363064)*x[1]+(-0.791969192801)*x[2]**o+(-0.130308630692)*x[2]
        arg[2,3,0,1]=(0.448970570476)*x[0]**o+(0.510346880872)*x[0]+(-0.561996764085)*x[1]**o+(0.217897696493)*x[1]+(-0.0610826665756)*x[2]**o+(0.387896322636)*x[2]
        arg[2,3,1,0]=(0.737635683404)*x[0]**o+(-0.35395540781)*x[0]+(0.0419652573208)*x[1]**o+(-0.519735381528)*x[1]+(0.739523085214)*x[2]**o+(0.808529094094)*x[2]
        arg[2,3,1,1]=(0.551348499194)*x[0]**o+(-0.40145493368)*x[0]+(0.211723169473)*x[1]**o+(-0.762751144824)*x[1]+(-0.17710033067)*x[2]**o+(0.688961224051)*x[2]
        arg[2,3,2,0]=(-0.870721462645)*x[0]**o+(-0.0356869329313)*x[0]+(0.277142920424)*x[1]**o+(-0.154474999666)*x[1]+(0.837001124911)*x[2]**o+(-0.233028214567)*x[2]
        arg[2,3,2,1]=(-0.234260935528)*x[0]**o+(0.913718165996)*x[0]+(0.385678747942)*x[1]**o+(0.237966356277)*x[1]+(0.382072927695)*x[2]**o+(0.627666395757)*x[2]
        arg[2,4,0,0]=(-0.474674316363)*x[0]**o+(-0.575533683959)*x[0]+(0.751489557666)*x[1]**o+(-0.0623565445313)*x[1]+(-0.517274743843)*x[2]**o+(-0.712143034168)*x[2]
        arg[2,4,0,1]=(0.259723799797)*x[0]**o+(0.345952822815)*x[0]+(0.169940774832)*x[1]**o+(-0.185154259609)*x[1]+(0.418430344681)*x[2]**o+(0.861837327531)*x[2]
        arg[2,4,1,0]=(-0.960970748682)*x[0]**o+(0.842706511282)*x[0]+(-0.771779045108)*x[1]**o+(-0.63675621795)*x[1]+(-0.317416556239)*x[2]**o+(0.287317968341)*x[2]
        arg[2,4,1,1]=(-0.753944077832)*x[0]**o+(0.570245260805)*x[0]+(-0.0226797611003)*x[1]**o+(0.0776773843077)*x[1]+(0.629536717673)*x[2]**o+(0.271216337699)*x[2]
        arg[2,4,2,0]=(-0.233304828171)*x[0]**o+(-0.217436697538)*x[0]+(-0.373618121496)*x[1]**o+(-0.342584906659)*x[1]+(0.0114484228954)*x[2]**o+(-0.639753178531)*x[2]
        arg[2,4,2,1]=(0.341373328517)*x[0]**o+(-0.432795268788)*x[0]+(-0.871263113926)*x[1]**o+(0.843590946949)*x[1]+(-0.653058566267)*x[2]**o+(-0.894366240333)*x[2]
        arg[3,0,0,0]=(-0.472652289489)*x[0]**o+(0.697119499826)*x[0]+(-0.195781503945)*x[1]**o+(-0.716872496009)*x[1]+(-0.159070341485)*x[2]**o+(-0.700548071467)*x[2]
        arg[3,0,0,1]=(-0.398674023611)*x[0]**o+(-0.690683017472)*x[0]+(0.957112237189)*x[1]**o+(-0.782702815224)*x[1]+(-0.249336230999)*x[2]**o+(0.22734983387)*x[2]
        arg[3,0,1,0]=(-0.0216721345971)*x[0]**o+(0.283765576696)*x[0]+(-0.35531833417)*x[1]**o+(0.974402352836)*x[1]+(-0.568820213603)*x[2]**o+(0.957610279127)*x[2]
        arg[3,0,1,1]=(0.0606265274442)*x[0]**o+(-0.941169961629)*x[0]+(0.804902933622)*x[1]**o+(-0.857158317629)*x[1]+(-0.227167312547)*x[2]**o+(0.945175823328)*x[2]
        arg[3,0,2,0]=(0.630444608821)*x[0]**o+(-0.192343222551)*x[0]+(0.0342249337232)*x[1]**o+(0.970401334961)*x[1]+(-0.608568046213)*x[2]**o+(-0.539374299133)*x[2]
        arg[3,0,2,1]=(-0.348127940577)*x[0]**o+(-0.895424161569)*x[0]+(0.918173840926)*x[1]**o+(0.99658998821)*x[1]+(-0.28733372377)*x[2]**o+(-0.516441029544)*x[2]
        arg[3,1,0,0]=(0.735069593965)*x[0]**o+(-0.727899981109)*x[0]+(-0.325639355247)*x[1]**o+(0.751064830331)*x[1]+(-0.297126073245)*x[2]**o+(-0.673421779764)*x[2]
        arg[3,1,0,1]=(0.00281719106153)*x[0]**o+(0.246929980679)*x[0]+(-0.48091940116)*x[1]**o+(0.501927059357)*x[1]+(-0.520099802736)*x[2]**o+(-0.902706012118)*x[2]
        arg[3,1,1,0]=(-0.341732637339)*x[0]**o+(-0.169802234575)*x[0]+(0.156725375114)*x[1]**o+(0.739176104936)*x[1]+(-0.374572295196)*x[2]**o+(-0.436641001832)*x[2]
        arg[3,1,1,1]=(0.644746847185)*x[0]**o+(-0.992126325722)*x[0]+(0.551602368714)*x[1]**o+(-0.839306768893)*x[1]+(-0.298981617653)*x[2]**o+(0.089795271605)*x[2]
        arg[3,1,2,0]=(0.382425223801)*x[0]**o+(-0.949095041278)*x[0]+(-0.135997266858)*x[1]**o+(-0.871114033611)*x[1]+(-0.422984170709)*x[2]**o+(-0.698818784989)*x[2]
        arg[3,1,2,1]=(0.630558573276)*x[0]**o+(0.90644556976)*x[0]+(0.512953870249)*x[1]**o+(-0.58514271049)*x[1]+(-0.0587225891088)*x[2]**o+(-0.390796465925)*x[2]
        arg[3,2,0,0]=(-0.404884771697)*x[0]**o+(-0.178144326223)*x[0]+(-0.606503995405)*x[1]**o+(0.190025878248)*x[1]+(0.735831203504)*x[2]**o+(-0.567394607398)*x[2]
        arg[3,2,0,1]=(0.28819895431)*x[0]**o+(0.456573144203)*x[0]+(0.00856899200079)*x[1]**o+(0.547827005333)*x[1]+(0.087428132552)*x[2]**o+(0.759650060552)*x[2]
        arg[3,2,1,0]=(0.949790710961)*x[0]**o+(-0.685602682574)*x[0]+(0.75278809536)*x[1]**o+(-0.876689834021)*x[1]+(0.0116394079709)*x[2]**o+(0.319113513167)*x[2]
        arg[3,2,1,1]=(0.619659549755)*x[0]**o+(-0.397388124185)*x[0]+(0.460679661901)*x[1]**o+(-0.507122241204)*x[1]+(0.985741194076)*x[2]**o+(0.916231863626)*x[2]
        arg[3,2,2,0]=(-0.457269257294)*x[0]**o+(-0.47265466694)*x[0]+(0.900051837969)*x[1]**o+(-0.148802803439)*x[1]+(-0.195540487054)*x[2]**o+(0.0784524130754)*x[2]
        arg[3,2,2,1]=(0.933553130797)*x[0]**o+(0.617362831046)*x[0]+(0.263616954879)*x[1]**o+(-0.0658887625105)*x[1]+(0.48091174024)*x[2]**o+(0.452935484504)*x[2]
        arg[3,3,0,0]=(0.825594594128)*x[0]**o+(0.350203407373)*x[0]+(0.211125321741)*x[1]**o+(0.194914147746)*x[1]+(-0.639432220749)*x[2]**o+(-0.678718852935)*x[2]
        arg[3,3,0,1]=(0.514394313629)*x[0]**o+(-0.595261005308)*x[0]+(-0.0219208527235)*x[1]**o+(-0.77204975709)*x[1]+(-0.483587370025)*x[2]**o+(-0.559464911044)*x[2]
        arg[3,3,1,0]=(0.821240511475)*x[0]**o+(-0.403492177329)*x[0]+(0.486132478046)*x[1]**o+(-0.0187529457547)*x[1]+(0.73209003456)*x[2]**o+(0.226791335797)*x[2]
        arg[3,3,1,1]=(-0.644256768839)*x[0]**o+(0.0372588784686)*x[0]+(0.726561650796)*x[1]**o+(0.518003809017)*x[1]+(0.738146681601)*x[2]**o+(-0.256036433171)*x[2]
        arg[3,3,2,0]=(0.534862520097)*x[0]**o+(0.275559826342)*x[0]+(0.292383677513)*x[1]**o+(-0.448938027415)*x[1]+(0.281053776971)*x[2]**o+(0.0119964402657)*x[2]
        arg[3,3,2,1]=(0.722071427786)*x[0]**o+(0.150280744346)*x[0]+(-0.0354786178164)*x[1]**o+(-0.703984035201)*x[1]+(0.105365795033)*x[2]**o+(-0.556567650379)*x[2]
        arg[3,4,0,0]=(0.188625467698)*x[0]**o+(0.737108753179)*x[0]+(0.481952725951)*x[1]**o+(0.841958412185)*x[1]+(-0.329504392569)*x[2]**o+(0.228456367852)*x[2]
        arg[3,4,0,1]=(0.181430484096)*x[0]**o+(0.307078178179)*x[0]+(0.262079226853)*x[1]**o+(0.111656765153)*x[1]+(0.15340094176)*x[2]**o+(-0.859774252333)*x[2]
        arg[3,4,1,0]=(0.213491785855)*x[0]**o+(-0.479441582203)*x[0]+(0.0527576231613)*x[1]**o+(0.328465118263)*x[1]+(-0.645719217198)*x[2]**o+(-0.550569446193)*x[2]
        arg[3,4,1,1]=(-0.401744411188)*x[0]**o+(-0.626446165761)*x[0]+(-0.191670239416)*x[1]**o+(0.866888407009)*x[1]+(-0.341220740645)*x[2]**o+(-0.742685031762)*x[2]
        arg[3,4,2,0]=(-0.0813611687514)*x[0]**o+(0.318680684889)*x[0]+(-0.780434724843)*x[1]**o+(-0.0902735483921)*x[1]+(0.586108990767)*x[2]**o+(0.934041577436)*x[2]
        arg[3,4,2,1]=(-0.0991425040958)*x[0]**o+(0.99274076773)*x[0]+(0.924248544389)*x[1]**o+(0.290576034321)*x[1]+(0.452104871421)*x[2]**o+(0.774017129246)*x[2]
        ref[0,0,0,0]=(-0.445809185937)/(o+1.)+(-0.11049649315)+(0.480896043196)*0.5**o
        ref[0,0,0,1]=(-0.776476434405)/(o+1.)+(-0.842010629213)+(-0.609632888023)*0.5**o
        ref[0,0,1,0]=(0.182977545138)/(o+1.)+(-0.124558039762)+(0.205065167181)*0.5**o
        ref[0,0,1,1]=(-1.17104760039)/(o+1.)+(0.608297882891)+(0.875716461075)*0.5**o
        ref[0,0,2,0]=(0.647678117252)/(o+1.)+(-0.752248832131)+(0.711372030289)*0.5**o
        ref[0,0,2,1]=(-0.0424453226222)/(o+1.)+(0.152363855976)+(-0.0536842545617)*0.5**o
        ref[0,1,0,0]=(-0.97072557194)/(o+1.)+(0.355747795628)+(-0.710091748251)*0.5**o
        ref[0,1,0,1]=(1.54210036269)/(o+1.)+(-0.0605096710259)+(0.250989366649)*0.5**o
        ref[0,1,1,0]=(1.33375575444)/(o+1.)+(-0.237134583454)+(0.690360096543)*0.5**o
        ref[0,1,1,1]=(0.0771949345448)/(o+1.)+(-0.00507329027461)+(-0.815220933301)*0.5**o
        ref[0,1,2,0]=(-0.149444846314)/(o+1.)+(-0.226886808857)+(0.169442617085)*0.5**o
        ref[0,1,2,1]=(0.114952830564)/(o+1.)+(-0.613038537369)+(-0.110636656112)*0.5**o
        ref[0,2,0,0]=(-0.0135089970172)/(o+1.)+(-0.104741016697)+(0.340661037085)*0.5**o
        ref[0,2,0,1]=(0.877950660854)/(o+1.)+(-0.395230122376)+(0.185282186176)*0.5**o
        ref[0,2,1,0]=(0.247984803771)/(o+1.)+(-0.35022317362)+(0.238772787004)*0.5**o
        ref[0,2,1,1]=(0.545513935627)/(o+1.)+(0.499096411763)+(-0.175884373587)*0.5**o
        ref[0,2,2,0]=(-1.82248899593)/(o+1.)+(0.90641347096)+(-0.615695203188)*0.5**o
        ref[0,2,2,1]=(0.384039454422)/(o+1.)+(-0.418573857307)+(-0.0992379560876)*0.5**o
        ref[0,3,0,0]=(0.401014281541)/(o+1.)+(0.0980709839284)+(0.432415072474)*0.5**o
        ref[0,3,0,1]=(-0.842943301759)/(o+1.)+(0.314943473548)+(-0.656397468061)*0.5**o
        ref[0,3,1,0]=(0.54561755931)/(o+1.)+(0.00554262549843)+(-0.472896985897)*0.5**o
        ref[0,3,1,1]=(0.651968521189)/(o+1.)+(0.534519988855)+(-0.515568063685)*0.5**o
        ref[0,3,2,0]=(1.21411446919)/(o+1.)+(0.334629245453)+(-0.709802317012)*0.5**o
        ref[0,3,2,1]=(1.57603080404)/(o+1.)+(0.123258042212)+(-0.20057625974)*0.5**o
        ref[0,4,0,0]=(-0.318228722361)/(o+1.)+(0.248490822109)+(-0.726921583373)*0.5**o
        ref[0,4,0,1]=(-0.785890130085)/(o+1.)+(-0.445201603538)+(0.765272127757)*0.5**o
        ref[0,4,1,0]=(-0.926447863334)/(o+1.)+(0.111991361885)+(-0.613269202518)*0.5**o
        ref[0,4,1,1]=(1.51953463824)/(o+1.)+(-0.283879690312)+(-0.727485674691)*0.5**o
        ref[0,4,2,0]=(-1.08165630338)/(o+1.)+(0.520653193096)+(-0.663071984573)*0.5**o
        ref[0,4,2,1]=(1.28376782528)/(o+1.)+(0.135152679989)+(0.893396132015)*0.5**o
        ref[1,0,0,0]=(0.12071022058)/(o+1.)+(-0.35786297592)+(-0.0771632829864)*0.5**o
        ref[1,0,0,1]=(-1.72034668123)/(o+1.)+(1.18881923454)+(0.133617787097)*0.5**o
        ref[1,0,1,0]=(-1.7448609294)/(o+1.)+(-0.110381196284)+(0.174277385391)*0.5**o
        ref[1,0,1,1]=(-0.906474149485)/(o+1.)+(-0.164529387192)+(0.795443369588)*0.5**o
        ref[1,0,2,0]=(0.0119437343259)/(o+1.)+(-0.676075658316)+(0.482814141263)*0.5**o
        ref[1,0,2,1]=(0.411591741576)/(o+1.)+(-0.419934815512)+(0.619359374773)*0.5**o
        ref[1,1,0,0]=(0.508884440665)/(o+1.)+(-0.858881203437)+(-0.510375454058)*0.5**o
        ref[1,1,0,1]=(0.519297551522)/(o+1.)+(0.647377411012)+(-0.302705575836)*0.5**o
        ref[1,1,1,0]=(0.101621725999)/(o+1.)+(0.763621416728)+(0.520882033451)*0.5**o
        ref[1,1,1,1]=(0.447372581166)/(o+1.)+(-0.27141635646)+(0.624622427594)*0.5**o
        ref[1,1,2,0]=(1.54807073071)/(o+1.)+(0.865521741121)+(-0.044254701409)*0.5**o
        ref[1,1,2,1]=(-0.954794849015)/(o+1.)+(0.616890655954)+(-0.240571107795)*0.5**o
        ref[1,2,0,0]=(-0.0138637151804)/(o+1.)+(-0.23836886811)+(0.0596937021722)*0.5**o
        ref[1,2,0,1]=(0.587416875871)/(o+1.)+(0.316160501623)+(-0.935541542647)*0.5**o
        ref[1,2,1,0]=(-0.0269069617163)/(o+1.)+(1.16051668573)+(0.276741406646)*0.5**o
        ref[1,2,1,1]=(-1.65286742406)/(o+1.)+(0.626473409767)+(0.828731604537)*0.5**o
        ref[1,2,2,0]=(-0.106399305507)/(o+1.)+(0.156612816333)+(-0.380797583569)*0.5**o
        ref[1,2,2,1]=(-0.0588949726984)/(o+1.)+(-0.666712330202)+(0.257022926473)*0.5**o
        ref[1,3,0,0]=(-0.500235779047)/(o+1.)+(-0.278704142234)+(-0.645340088797)*0.5**o
        ref[1,3,0,1]=(0.100788411199)/(o+1.)+(-0.158633910442)+(0.124132693667)*0.5**o
        ref[1,3,1,0]=(0.824541289962)/(o+1.)+(0.17530886621)+(-0.7660131868)*0.5**o
        ref[1,3,1,1]=(-0.331589329951)/(o+1.)+(0.0509573421542)+(-0.150578257351)*0.5**o
        ref[1,3,2,0]=(-0.224033577887)/(o+1.)+(0.775397193106)+(-0.336661555676)*0.5**o
        ref[1,3,2,1]=(-0.0921229181433)/(o+1.)+(0.15790296452)+(0.165928745462)*0.5**o
        ref[1,4,0,0]=(-1.13149949621)/(o+1.)+(-0.0663729445879)+(0.283586653542)*0.5**o
        ref[1,4,0,1]=(-0.722886434731)/(o+1.)+(0.104888434522)+(-0.772849835334)*0.5**o
        ref[1,4,1,0]=(-1.44885259324)/(o+1.)+(-0.617668613405)+(-0.718524564964)*0.5**o
        ref[1,4,1,1]=(-0.725759098745)/(o+1.)+(-0.519223986923)+(-0.771664154234)*0.5**o
        ref[1,4,2,0]=(0.405592140408)/(o+1.)+(-0.317533868664)+(-0.591902790978)*0.5**o
        ref[1,4,2,1]=(1.64489233298)/(o+1.)+(-0.145877587744)+(0.303567440092)*0.5**o
        ref[2,0,0,0]=(1.35080478817)/(o+1.)+(-0.97319571481)+(0.197574550087)*0.5**o
        ref[2,0,0,1]=(-0.435235416366)/(o+1.)+(-0.0768910013301)+(0.0663499012532)*0.5**o
        ref[2,0,1,0]=(-0.441064969294)/(o+1.)+(-0.407537083031)+(-0.789090971877)*0.5**o
        ref[2,0,1,1]=(-0.672223056216)/(o+1.)+(-0.673709465998)+(0.222873274437)*0.5**o
        ref[2,0,2,0]=(-1.00188202525)/(o+1.)+(0.0497571110368)+(-0.300828039715)*0.5**o
        ref[2,0,2,1]=(-0.755667412756)/(o+1.)+(0.101155281333)+(-0.501507304569)*0.5**o
        ref[2,1,0,0]=(-1.03721245191)/(o+1.)+(0.421970845648)+(-0.809237327867)*0.5**o
        ref[2,1,0,1]=(-0.924064281731)/(o+1.)+(0.748054079897)+(-0.722795722506)*0.5**o
        ref[2,1,1,0]=(-0.0681656049743)/(o+1.)+(-0.401875028515)+(-0.551325611251)*0.5**o
        ref[2,1,1,1]=(-1.43246198564)/(o+1.)+(-0.346563453541)+(0.743453045588)*0.5**o
        ref[2,1,2,0]=(-0.628339309568)/(o+1.)+(0.801336921577)+(-0.033787671958)*0.5**o
        ref[2,1,2,1]=(0.375615094549)/(o+1.)+(0.362201072648)+(0.127121989623)*0.5**o
        ref[2,2,0,0]=(0.264583894127)/(o+1.)+(0.163046887519)+(-0.867530011926)*0.5**o
        ref[2,2,0,1]=(1.11591762161)/(o+1.)+(-0.953567997151)+(-0.00858729113056)*0.5**o
        ref[2,2,1,0]=(0.811633932744)/(o+1.)+(-0.227190467633)+(-0.982123891355)*0.5**o
        ref[2,2,1,1]=(-0.0564577617182)/(o+1.)+(0.240301710247)+(-0.415533190881)*0.5**o
        ref[2,2,2,0]=(0.762093170298)/(o+1.)+(0.072987783836)+(-0.559916632426)*0.5**o
        ref[2,2,2,1]=(0.202809735486)/(o+1.)+(0.537525887878)+(-0.869055045156)*0.5**o
        ref[2,3,0,0]=(-0.869552267348)/(o+1.)+(0.0306038908843)+(0.754089271004)*0.5**o
        ref[2,3,0,1]=(-0.62307943066)/(o+1.)+(0.55807045)+(0.448970570476)*0.5**o
        ref[2,3,1,0]=(0.781488342534)/(o+1.)+(-0.0325808476221)+(0.737635683404)*0.5**o
        ref[2,3,1,1]=(0.0346228388027)/(o+1.)+(-0.237622427226)+(0.551348499194)*0.5**o
        ref[2,3,2,0]=(1.11414404534)/(o+1.)+(-0.211595073582)+(-0.870721462645)*0.5**o
        ref[2,3,2,1]=(0.767751675637)/(o+1.)+(0.889675459015)+(-0.234260935528)*0.5**o
        ref[2,4,0,0]=(0.234214813823)/(o+1.)+(-0.675016631329)+(-0.474674316363)*0.5**o
        ref[2,4,0,1]=(0.588371119514)/(o+1.)+(0.511317945368)+(0.259723799797)*0.5**o
        ref[2,4,1,0]=(-1.08919560135)/(o+1.)+(0.246634130836)+(-0.960970748682)*0.5**o
        ref[2,4,1,1]=(0.606856956573)/(o+1.)+(0.459569491406)+(-0.753944077832)*0.5**o
        ref[2,4,2,0]=(-0.3621696986)/(o+1.)+(-0.599887391364)+(-0.233304828171)*0.5**o
        ref[2,4,2,1]=(-1.52432168019)/(o+1.)+(-0.241785281086)+(0.341373328517)*0.5**o
        ref[3,0,0,0]=(-0.35485184543)/(o+1.)+(-0.360150533825)+(-0.472652289489)*0.5**o
        ref[3,0,0,1]=(0.70777600619)/(o+1.)+(-0.623017999413)+(-0.398674023611)*0.5**o
        ref[3,0,1,0]=(-0.924138547773)/(o+1.)+(1.10788910433)+(-0.0216721345971)*0.5**o
        ref[3,0,1,1]=(0.577735621075)/(o+1.)+(-0.426576227965)+(0.0606265274442)*0.5**o
        ref[3,0,2,0]=(-0.57434311249)/(o+1.)+(0.119341906638)+(0.630444608821)*0.5**o
        ref[3,0,2,1]=(0.630840117156)/(o+1.)+(-0.207637601452)+(-0.348127940577)*0.5**o
        ref[3,1,0,0]=(-0.622765428492)/(o+1.)+(-0.32512846527)+(0.735069593965)*0.5**o
        ref[3,1,0,1]=(-1.0010192039)/(o+1.)+(-0.076924486041)+(0.00281719106153)*0.5**o
        ref[3,1,1,0]=(-0.217846920082)/(o+1.)+(0.0663664342643)+(-0.341732637339)*0.5**o
        ref[3,1,1,1]=(0.25262075106)/(o+1.)+(-0.870818911505)+(0.644746847185)*0.5**o
        ref[3,1,2,0]=(-0.558981437567)/(o+1.)+(-1.25951392994)+(0.382425223801)*0.5**o
        ref[3,1,2,1]=(0.454231281141)/(o+1.)+(-0.034746803327)+(0.630558573276)*0.5**o
        ref[3,2,0,0]=(0.129327208099)/(o+1.)+(-0.277756527686)+(-0.404884771697)*0.5**o
        ref[3,2,0,1]=(0.0959971245528)/(o+1.)+(0.882025105044)+(0.28819895431)*0.5**o
        ref[3,2,1,0]=(0.764427503331)/(o+1.)+(-0.621589501714)+(0.949790710961)*0.5**o
        ref[3,2,1,1]=(1.44642085598)/(o+1.)+(0.00586074911794)+(0.619659549755)*0.5**o
        ref[3,2,2,0]=(0.704511350915)/(o+1.)+(-0.271502528652)+(-0.457269257294)*0.5**o
        ref[3,2,2,1]=(0.744528695119)/(o+1.)+(0.50220477652)+(0.933553130797)*0.5**o
        ref[3,3,0,0]=(-0.428306899008)/(o+1.)+(-0.0668006489082)+(0.825594594128)*0.5**o
        ref[3,3,0,1]=(-0.505508222749)/(o+1.)+(-0.963387836721)+(0.514394313629)*0.5**o
        ref[3,3,1,0]=(1.21822251261)/(o+1.)+(-0.0977268936434)+(0.821240511475)*0.5**o
        ref[3,3,1,1]=(1.4647083324)/(o+1.)+(0.149613127157)+(-0.644256768839)*0.5**o
        ref[3,3,2,0]=(0.573437454483)/(o+1.)+(-0.0806908804035)+(0.534862520097)*0.5**o
        ref[3,3,2,1]=(0.069887177217)/(o+1.)+(-0.555135470617)+(0.722071427786)*0.5**o
        ref[3,4,0,0]=(0.152448333383)/(o+1.)+(0.903761766608)+(0.188625467698)*0.5**o
        ref[3,4,0,1]=(0.415480168613)/(o+1.)+(-0.220519654501)+(0.181430484096)*0.5**o
        ref[3,4,1,0]=(-0.592961594036)/(o+1.)+(-0.350772955066)+(0.213491785855)*0.5**o
        ref[3,4,1,1]=(-0.532890980061)/(o+1.)+(-0.251121395257)+(-0.401744411188)*0.5**o
        ref[3,4,2,0]=(-0.194325734076)/(o+1.)+(0.581224356966)+(-0.0813611687514)*0.5**o
        ref[3,4,2,1]=(1.37635341581)/(o+1.)+(1.02866696565)+(-0.0991425040958)*0.5**o
      res=integrate(arg)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactOne_fromSymbol_ContinuousFunction_rank0(self):
      """
      tests integral of rank 0 Symbol on the FunctionOnContactOne

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports integral on FunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=())
      s=Data(0,(),w)
      if dim==2:
        s=(0.88016247767)*x[0]**o+(0.546409254589)*x[0]+(-0.782935386141)*x[1]**o+(-0.21996752206)*x[1]
        ref=(-0.782935386141)/(o+1.)+(0.163220866264)+(0.88016247767)*0.5**o
      else:
        s=(0.436925914346)*x[0]**o+(0.18630264415)*x[0]+(-0.374230617371)*x[1]**o+(0.331719266581)*x[1]+(-0.327334599603)*x[2]**o+(-0.376893151193)*x[2]
        ref=(-0.701565216974)/(o+1.)+(0.0705643797689)+(0.436925914346)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactOne_fromSymbol_ContinuousFunction_rank1(self):
      """
      tests integral of rank 1 Symbol on the FunctionOnContactOne

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports integral on FunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(2,))
      s=Data(0,(2,),w)
      ref=numpy.zeros((2,),numpy.float_)
      if dim==2:
        s[0]=(-0.0725655727924)*x[0]**o+(-0.539202679596)*x[0]+(0.00383805967589)*x[1]**o+(-0.42164917219)*x[1]
        s[1]=(0.799189397651)*x[0]**o+(0.468822667976)*x[0]+(0.946068596129)*x[1]**o+(-0.75397954631)*x[1]
        ref[0]=(0.00383805967589)/(o+1.)+(-0.480425925893)+(-0.0725655727924)*0.5**o
        ref[1]=(0.946068596129)/(o+1.)+(-0.142578439167)+(0.799189397651)*0.5**o
      else:
        s[0]=(0.94482411471)*x[0]**o+(-0.308967528984)*x[0]+(-0.417912410083)*x[1]**o+(-0.363984940852)*x[1]+(0.991921856553)*x[2]**o+(-0.264144586743)*x[2]
        s[1]=(-0.190159227288)*x[0]**o+(0.774206651681)*x[0]+(-0.585389796317)*x[1]**o+(-0.8203250901)*x[1]+(-0.959196504133)*x[2]**o+(0.118981346607)*x[2]
        ref[0]=(0.57400944647)/(o+1.)+(-0.46854852829)+(0.94482411471)*0.5**o
        ref[1]=(-1.54458630045)/(o+1.)+(0.036431454094)+(-0.190159227288)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactOne_fromSymbol_ContinuousFunction_rank2(self):
      """
      tests integral of rank 2 Symbol on the FunctionOnContactOne

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports integral on FunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5))
      s=Data(0,(4, 5),w)
      ref=numpy.zeros((4, 5),numpy.float_)
      if dim==2:
        s[0,0]=(0.821123722738)*x[0]**o+(-0.953533567845)*x[0]+(0.761401252898)*x[1]**o+(0.560845798718)*x[1]
        s[0,1]=(0.388458475385)*x[0]**o+(0.787499438329)*x[0]+(0.13496554321)*x[1]**o+(0.558370810363)*x[1]
        s[0,2]=(-0.583597257844)*x[0]**o+(-0.203553852077)*x[0]+(0.400123717156)*x[1]**o+(-0.748904505246)*x[1]
        s[0,3]=(0.215629523788)*x[0]**o+(0.380761852867)*x[0]+(-0.656661093703)*x[1]**o+(0.707314164752)*x[1]
        s[0,4]=(-0.945147360815)*x[0]**o+(0.761197937419)*x[0]+(-0.242270565927)*x[1]**o+(0.197444941556)*x[1]
        s[1,0]=(-0.260643768467)*x[0]**o+(-0.429658355574)*x[0]+(0.832795677562)*x[1]**o+(0.0465882509708)*x[1]
        s[1,1]=(0.80280828918)*x[0]**o+(0.824378440768)*x[0]+(-0.0704713499903)*x[1]**o+(0.882600027718)*x[1]
        s[1,2]=(-0.188924905219)*x[0]**o+(-0.532960871866)*x[0]+(0.130210761298)*x[1]**o+(-0.847312599467)*x[1]
        s[1,3]=(-0.637355923048)*x[0]**o+(0.102559784731)*x[0]+(-0.00202798859835)*x[1]**o+(-0.275087027189)*x[1]
        s[1,4]=(-0.677415150613)*x[0]**o+(0.212870653829)*x[0]+(0.60045622057)*x[1]**o+(0.554319358843)*x[1]
        s[2,0]=(-0.758104298178)*x[0]**o+(0.122102590939)*x[0]+(0.82242042837)*x[1]**o+(0.877931466117)*x[1]
        s[2,1]=(0.295560509979)*x[0]**o+(0.0550082286163)*x[0]+(0.28149317542)*x[1]**o+(0.933306020272)*x[1]
        s[2,2]=(0.164458091911)*x[0]**o+(0.895923571884)*x[0]+(-0.89721569291)*x[1]**o+(-0.797286890655)*x[1]
        s[2,3]=(-0.455008349519)*x[0]**o+(-0.950347778241)*x[0]+(-0.52866158871)*x[1]**o+(0.784423551832)*x[1]
        s[2,4]=(0.0404391288743)*x[0]**o+(0.919895028488)*x[0]+(0.14131342368)*x[1]**o+(-0.534436226168)*x[1]
        s[3,0]=(0.313572834288)*x[0]**o+(-0.981579650669)*x[0]+(-0.105890320276)*x[1]**o+(0.184272310362)*x[1]
        s[3,1]=(0.517529943685)*x[0]**o+(-0.225899698465)*x[0]+(0.314725287596)*x[1]**o+(-0.520678188485)*x[1]
        s[3,2]=(0.457697644746)*x[0]**o+(0.580651564264)*x[0]+(0.0819245033577)*x[1]**o+(-0.962585666695)*x[1]
        s[3,3]=(-0.540006520671)*x[0]**o+(0.00341289905337)*x[0]+(-0.75656675994)*x[1]**o+(0.355040790858)*x[1]
        s[3,4]=(0.211995984801)*x[0]**o+(-0.907257558871)*x[0]+(0.736102007034)*x[1]**o+(-0.841283087071)*x[1]
        ref[0,0]=(0.761401252898)/(o+1.)+(-0.196343884564)+(0.821123722738)*0.5**o
        ref[0,1]=(0.13496554321)/(o+1.)+(0.672935124346)+(0.388458475385)*0.5**o
        ref[0,2]=(0.400123717156)/(o+1.)+(-0.476229178662)+(-0.583597257844)*0.5**o
        ref[0,3]=(-0.656661093703)/(o+1.)+(0.544038008809)+(0.215629523788)*0.5**o
        ref[0,4]=(-0.242270565927)/(o+1.)+(0.479321439487)+(-0.945147360815)*0.5**o
        ref[1,0]=(0.832795677562)/(o+1.)+(-0.191535052302)+(-0.260643768467)*0.5**o
        ref[1,1]=(-0.0704713499903)/(o+1.)+(0.853489234243)+(0.80280828918)*0.5**o
        ref[1,2]=(0.130210761298)/(o+1.)+(-0.690136735666)+(-0.188924905219)*0.5**o
        ref[1,3]=(-0.00202798859835)/(o+1.)+(-0.0862636212292)+(-0.637355923048)*0.5**o
        ref[1,4]=(0.60045622057)/(o+1.)+(0.383595006336)+(-0.677415150613)*0.5**o
        ref[2,0]=(0.82242042837)/(o+1.)+(0.500017028528)+(-0.758104298178)*0.5**o
        ref[2,1]=(0.28149317542)/(o+1.)+(0.494157124444)+(0.295560509979)*0.5**o
        ref[2,2]=(-0.89721569291)/(o+1.)+(0.0493183406148)+(0.164458091911)*0.5**o
        ref[2,3]=(-0.52866158871)/(o+1.)+(-0.0829621132044)+(-0.455008349519)*0.5**o
        ref[2,4]=(0.14131342368)/(o+1.)+(0.19272940116)+(0.0404391288743)*0.5**o
        ref[3,0]=(-0.105890320276)/(o+1.)+(-0.398653670153)+(0.313572834288)*0.5**o
        ref[3,1]=(0.314725287596)/(o+1.)+(-0.373288943475)+(0.517529943685)*0.5**o
        ref[3,2]=(0.0819245033577)/(o+1.)+(-0.190967051216)+(0.457697644746)*0.5**o
        ref[3,3]=(-0.75656675994)/(o+1.)+(0.179226844956)+(-0.540006520671)*0.5**o
        ref[3,4]=(0.736102007034)/(o+1.)+(-0.874270322971)+(0.211995984801)*0.5**o
      else:
        s[0,0]=(-0.26796664441)*x[0]**o+(0.412787191224)*x[0]+(0.291586980058)*x[1]**o+(0.363446579326)*x[1]+(-0.925503103962)*x[2]**o+(-0.815680149808)*x[2]
        s[0,1]=(0.816233130972)*x[0]**o+(-0.412368706967)*x[0]+(-0.633601565886)*x[1]**o+(-0.371336006063)*x[1]+(0.296754676336)*x[2]**o+(0.670055343075)*x[2]
        s[0,2]=(0.0115846578221)*x[0]**o+(0.0092074168084)*x[0]+(0.852395533346)*x[1]**o+(-0.536392264791)*x[1]+(-0.937066324616)*x[2]**o+(0.164445148469)*x[2]
        s[0,3]=(0.334112468463)*x[0]**o+(-0.197414532294)*x[0]+(0.32282859646)*x[1]**o+(-0.829489106617)*x[1]+(0.637135896229)*x[2]**o+(-0.385346619948)*x[2]
        s[0,4]=(0.633179246285)*x[0]**o+(-0.775108312054)*x[0]+(-0.0275404480368)*x[1]**o+(-0.119328918403)*x[1]+(0.56571174294)*x[2]**o+(-0.306269532057)*x[2]
        s[1,0]=(-0.440298361388)*x[0]**o+(-0.644698995147)*x[0]+(-0.815801747709)*x[1]**o+(-0.347258269595)*x[1]+(-0.614702701308)*x[2]**o+(0.439358767152)*x[2]
        s[1,1]=(-0.397657154377)*x[0]**o+(0.529856649499)*x[0]+(-0.737187187383)*x[1]**o+(-0.242702727135)*x[1]+(-0.823515970874)*x[2]**o+(0.609907664585)*x[2]
        s[1,2]=(0.438570722796)*x[0]**o+(0.699038412356)*x[0]+(-0.731471431635)*x[1]**o+(0.334652466823)*x[1]+(0.728626361708)*x[2]**o+(0.140706080335)*x[2]
        s[1,3]=(0.445813070034)*x[0]**o+(-0.223359077861)*x[0]+(0.607122403242)*x[1]**o+(0.684390142218)*x[1]+(-0.0448436266022)*x[2]**o+(0.0177789962153)*x[2]
        s[1,4]=(-0.618164233561)*x[0]**o+(0.174313138771)*x[0]+(-0.891694340174)*x[1]**o+(-0.210641679269)*x[1]+(0.961104454395)*x[2]**o+(-0.335141946858)*x[2]
        s[2,0]=(0.824078577041)*x[0]**o+(-0.206045431624)*x[0]+(0.055588699321)*x[1]**o+(0.350116387216)*x[1]+(0.607866274938)*x[2]**o+(0.407638105768)*x[2]
        s[2,1]=(-0.210261210166)*x[0]**o+(-0.0133233759756)*x[0]+(0.801022275664)*x[1]**o+(-0.267209121471)*x[1]+(0.535577530444)*x[2]**o+(-0.541166709451)*x[2]
        s[2,2]=(0.346542013177)*x[0]**o+(0.305602431422)*x[0]+(0.45213719675)*x[1]**o+(0.32765019418)*x[1]+(0.367619644833)*x[2]**o+(0.933760631839)*x[2]
        s[2,3]=(-0.244955187752)*x[0]**o+(-0.645641982024)*x[0]+(0.0261013017433)*x[1]**o+(-0.230403895757)*x[1]+(0.201233200314)*x[2]**o+(-0.783467696033)*x[2]
        s[2,4]=(0.783620590334)*x[0]**o+(0.325268485711)*x[0]+(-0.912841443404)*x[1]**o+(-0.500647833534)*x[1]+(-0.980126267268)*x[2]**o+(-0.425755185755)*x[2]
        s[3,0]=(-0.4690769573)*x[0]**o+(-0.547691242018)*x[0]+(0.996570266539)*x[1]**o+(-0.472334765569)*x[1]+(0.81903298756)*x[2]**o+(-0.654817201284)*x[2]
        s[3,1]=(0.617564682102)*x[0]**o+(-0.991757850114)*x[0]+(-0.34214251344)*x[1]**o+(-0.321397198526)*x[1]+(0.503434489432)*x[2]**o+(0.683522308565)*x[2]
        s[3,2]=(0.999035085465)*x[0]**o+(0.431261164559)*x[0]+(0.660309836881)*x[1]**o+(0.596465902975)*x[1]+(-0.575064993246)*x[2]**o+(-0.290933980474)*x[2]
        s[3,3]=(0.355357213484)*x[0]**o+(-0.638400362695)*x[0]+(0.596482099946)*x[1]**o+(0.861245903515)*x[1]+(0.877992083876)*x[2]**o+(0.221520840799)*x[2]
        s[3,4]=(-0.0280956096914)*x[0]**o+(0.880536602898)*x[0]+(0.00969665494423)*x[1]**o+(-0.182537740984)*x[1]+(0.84927089239)*x[2]**o+(0.11423223273)*x[2]
        ref[0,0]=(-0.633916123904)/(o+1.)+(-0.0197231896285)+(-0.26796664441)*0.5**o
        ref[0,1]=(-0.33684688955)/(o+1.)+(-0.0568246849779)+(0.816233130972)*0.5**o
        ref[0,2]=(-0.0846707912707)/(o+1.)+(-0.181369849757)+(0.0115846578221)*0.5**o
        ref[0,3]=(0.959964492689)/(o+1.)+(-0.70612512943)+(0.334112468463)*0.5**o
        ref[0,4]=(0.538171294903)/(o+1.)+(-0.600353381257)+(0.633179246285)*0.5**o
        ref[1,0]=(-1.43050444902)/(o+1.)+(-0.276299248795)+(-0.440298361388)*0.5**o
        ref[1,1]=(-1.56070315826)/(o+1.)+(0.448530793475)+(-0.397657154377)*0.5**o
        ref[1,2]=(-0.00284506992628)/(o+1.)+(0.587198479757)+(0.438570722796)*0.5**o
        ref[1,3]=(0.56227877664)/(o+1.)+(0.239405030286)+(0.445813070034)*0.5**o
        ref[1,4]=(0.0694101142218)/(o+1.)+(-0.185735243678)+(-0.618164233561)*0.5**o
        ref[2,0]=(0.663454974259)/(o+1.)+(0.27585453068)+(0.824078577041)*0.5**o
        ref[2,1]=(1.33659980611)/(o+1.)+(-0.410849603449)+(-0.210261210166)*0.5**o
        ref[2,2]=(0.819756841583)/(o+1.)+(0.78350662872)+(0.346542013177)*0.5**o
        ref[2,3]=(0.227334502058)/(o+1.)+(-0.829756786907)+(-0.244955187752)*0.5**o
        ref[2,4]=(-1.89296771067)/(o+1.)+(-0.300567266789)+(0.783620590334)*0.5**o
        ref[3,0]=(1.8156032541)/(o+1.)+(-0.837421604436)+(-0.4690769573)*0.5**o
        ref[3,1]=(0.161291975992)/(o+1.)+(-0.314816370037)+(0.617564682102)*0.5**o
        ref[3,2]=(0.0852448436355)/(o+1.)+(0.36839654353)+(0.999035085465)*0.5**o
        ref[3,3]=(1.47447418382)/(o+1.)+(0.222183190809)+(0.355357213484)*0.5**o
        ref[3,4]=(0.858967547335)/(o+1.)+(0.406115547322)+(-0.0280956096914)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactOne_fromSymbol_ContinuousFunction_rank3(self):
      """
      tests integral of rank 3 Symbol on the FunctionOnContactOne

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports integral on FunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(6, 2, 2))
      s=Data(0,(6, 2, 2),w)
      ref=numpy.zeros((6, 2, 2),numpy.float_)
      if dim==2:
        s[0,0,0]=(0.230469008444)*x[0]**o+(0.508027324248)*x[0]+(0.696459735501)*x[1]**o+(-0.195259592119)*x[1]
        s[0,0,1]=(0.135411134228)*x[0]**o+(-0.684964577317)*x[0]+(0.804062624988)*x[1]**o+(0.388383344617)*x[1]
        s[0,1,0]=(-0.965361972117)*x[0]**o+(-0.0740755184742)*x[0]+(0.00222395987317)*x[1]**o+(0.116566925701)*x[1]
        s[0,1,1]=(0.773749135888)*x[0]**o+(-0.236062924811)*x[0]+(0.824300232936)*x[1]**o+(0.58451198061)*x[1]
        s[1,0,0]=(0.213202511915)*x[0]**o+(-0.0164093785777)*x[0]+(-0.152629558111)*x[1]**o+(-0.663542824781)*x[1]
        s[1,0,1]=(-0.446430833827)*x[0]**o+(0.643410978113)*x[0]+(0.35649029144)*x[1]**o+(-0.914705065197)*x[1]
        s[1,1,0]=(0.616559079235)*x[0]**o+(0.347804225396)*x[0]+(-0.733945941231)*x[1]**o+(0.49771373099)*x[1]
        s[1,1,1]=(-0.398631849965)*x[0]**o+(-0.854719511493)*x[0]+(0.501641783941)*x[1]**o+(0.360567791644)*x[1]
        s[2,0,0]=(0.404312756516)*x[0]**o+(0.805967978591)*x[0]+(0.980468298218)*x[1]**o+(0.368530205154)*x[1]
        s[2,0,1]=(0.92908409018)*x[0]**o+(0.984893826849)*x[0]+(-0.706723295076)*x[1]**o+(-0.68485578941)*x[1]
        s[2,1,0]=(-0.395198069725)*x[0]**o+(-0.865731577936)*x[0]+(0.284248647254)*x[1]**o+(0.199510619169)*x[1]
        s[2,1,1]=(-0.383178260681)*x[0]**o+(-0.0753678998731)*x[0]+(0.68853605046)*x[1]**o+(0.693301872966)*x[1]
        s[3,0,0]=(-0.504525570613)*x[0]**o+(-0.295454443098)*x[0]+(-0.85334585793)*x[1]**o+(-0.450742110681)*x[1]
        s[3,0,1]=(0.0306385968872)*x[0]**o+(-0.0918977201143)*x[0]+(-0.106665376273)*x[1]**o+(0.158103530025)*x[1]
        s[3,1,0]=(-0.371403581207)*x[0]**o+(0.662079642408)*x[0]+(0.818746082332)*x[1]**o+(0.601651002406)*x[1]
        s[3,1,1]=(-0.93523720418)*x[0]**o+(0.871800352376)*x[0]+(-0.294513821338)*x[1]**o+(-0.947893176361)*x[1]
        s[4,0,0]=(-0.734908326587)*x[0]**o+(0.481209886891)*x[0]+(0.61811626923)*x[1]**o+(-0.21778354281)*x[1]
        s[4,0,1]=(-0.689233989565)*x[0]**o+(0.1273357199)*x[0]+(0.398820148577)*x[1]**o+(0.538945475692)*x[1]
        s[4,1,0]=(0.885178741071)*x[0]**o+(-0.351046930859)*x[0]+(0.492384326646)*x[1]**o+(0.584584261959)*x[1]
        s[4,1,1]=(-0.561791063188)*x[0]**o+(0.701763941021)*x[0]+(-0.242041214153)*x[1]**o+(0.297044144162)*x[1]
        s[5,0,0]=(-0.487786682467)*x[0]**o+(-0.69717117402)*x[0]+(-0.308038610282)*x[1]**o+(0.966752012177)*x[1]
        s[5,0,1]=(0.26556821402)*x[0]**o+(-0.983047207652)*x[0]+(0.276098865961)*x[1]**o+(0.840073469203)*x[1]
        s[5,1,0]=(-0.152606883933)*x[0]**o+(-0.481720281304)*x[0]+(0.417680077266)*x[1]**o+(-0.882323563007)*x[1]
        s[5,1,1]=(0.0674752363532)*x[0]**o+(0.727094741288)*x[0]+(-0.299947571476)*x[1]**o+(0.391301157504)*x[1]
        ref[0,0,0]=(0.696459735501)/(o+1.)+(0.156383866065)+(0.230469008444)*0.5**o
        ref[0,0,1]=(0.804062624988)/(o+1.)+(-0.14829061635)+(0.135411134228)*0.5**o
        ref[0,1,0]=(0.00222395987317)/(o+1.)+(0.0212457036136)+(-0.965361972117)*0.5**o
        ref[0,1,1]=(0.824300232936)/(o+1.)+(0.174224527899)+(0.773749135888)*0.5**o
        ref[1,0,0]=(-0.152629558111)/(o+1.)+(-0.339976101679)+(0.213202511915)*0.5**o
        ref[1,0,1]=(0.35649029144)/(o+1.)+(-0.135647043542)+(-0.446430833827)*0.5**o
        ref[1,1,0]=(-0.733945941231)/(o+1.)+(0.422758978193)+(0.616559079235)*0.5**o
        ref[1,1,1]=(0.501641783941)/(o+1.)+(-0.247075859924)+(-0.398631849965)*0.5**o
        ref[2,0,0]=(0.980468298218)/(o+1.)+(0.587249091872)+(0.404312756516)*0.5**o
        ref[2,0,1]=(-0.706723295076)/(o+1.)+(0.15001901872)+(0.92908409018)*0.5**o
        ref[2,1,0]=(0.284248647254)/(o+1.)+(-0.333110479383)+(-0.395198069725)*0.5**o
        ref[2,1,1]=(0.68853605046)/(o+1.)+(0.308966986547)+(-0.383178260681)*0.5**o
        ref[3,0,0]=(-0.85334585793)/(o+1.)+(-0.37309827689)+(-0.504525570613)*0.5**o
        ref[3,0,1]=(-0.106665376273)/(o+1.)+(0.0331029049554)+(0.0306385968872)*0.5**o
        ref[3,1,0]=(0.818746082332)/(o+1.)+(0.631865322407)+(-0.371403581207)*0.5**o
        ref[3,1,1]=(-0.294513821338)/(o+1.)+(-0.0380464119924)+(-0.93523720418)*0.5**o
        ref[4,0,0]=(0.61811626923)/(o+1.)+(0.131713172041)+(-0.734908326587)*0.5**o
        ref[4,0,1]=(0.398820148577)/(o+1.)+(0.333140597796)+(-0.689233989565)*0.5**o
        ref[4,1,0]=(0.492384326646)/(o+1.)+(0.11676866555)+(0.885178741071)*0.5**o
        ref[4,1,1]=(-0.242041214153)/(o+1.)+(0.499404042592)+(-0.561791063188)*0.5**o
        ref[5,0,0]=(-0.308038610282)/(o+1.)+(0.134790419079)+(-0.487786682467)*0.5**o
        ref[5,0,1]=(0.276098865961)/(o+1.)+(-0.0714868692245)+(0.26556821402)*0.5**o
        ref[5,1,0]=(0.417680077266)/(o+1.)+(-0.682021922156)+(-0.152606883933)*0.5**o
        ref[5,1,1]=(-0.299947571476)/(o+1.)+(0.559197949396)+(0.0674752363532)*0.5**o
      else:
        s[0,0,0]=(-0.226564859572)*x[0]**o+(0.759975882725)*x[0]+(0.659032148551)*x[1]**o+(0.0807090996868)*x[1]+(-0.586765501877)*x[2]**o+(0.0274224545344)*x[2]
        s[0,0,1]=(0.926463229691)*x[0]**o+(0.134519539954)*x[0]+(0.976918539353)*x[1]**o+(-0.352762024083)*x[1]+(0.132042653159)*x[2]**o+(-0.686161947886)*x[2]
        s[0,1,0]=(0.371916775894)*x[0]**o+(-0.586490002548)*x[0]+(-0.304695527165)*x[1]**o+(0.440748639515)*x[1]+(-0.277758803854)*x[2]**o+(0.933648409631)*x[2]
        s[0,1,1]=(-0.871254388888)*x[0]**o+(-0.373959360688)*x[0]+(0.613060994689)*x[1]**o+(0.261906429329)*x[1]+(-0.0255596243956)*x[2]**o+(-0.912700299076)*x[2]
        s[1,0,0]=(0.0075965229641)*x[0]**o+(-0.977596529897)*x[0]+(0.160602293774)*x[1]**o+(-0.407220178255)*x[1]+(0.722872196439)*x[2]**o+(0.96683722852)*x[2]
        s[1,0,1]=(0.197284854866)*x[0]**o+(0.689180749331)*x[0]+(0.476641393876)*x[1]**o+(-0.575644071726)*x[1]+(-0.988391730968)*x[2]**o+(0.6278575683)*x[2]
        s[1,1,0]=(-0.968464883049)*x[0]**o+(0.415629727234)*x[0]+(0.698016769139)*x[1]**o+(0.967345907027)*x[1]+(0.0278637129728)*x[2]**o+(-0.895746388886)*x[2]
        s[1,1,1]=(-0.240984840545)*x[0]**o+(-0.0680920485806)*x[0]+(-0.159143749001)*x[1]**o+(-0.0595387242977)*x[1]+(0.564402939251)*x[2]**o+(-0.701335469294)*x[2]
        s[2,0,0]=(0.239124234745)*x[0]**o+(-0.86570078201)*x[0]+(-0.331189286269)*x[1]**o+(0.141412115059)*x[1]+(-0.498257980574)*x[2]**o+(-0.560030264671)*x[2]
        s[2,0,1]=(-0.126152295994)*x[0]**o+(0.220733769906)*x[0]+(0.293598600333)*x[1]**o+(0.0232185863704)*x[1]+(-0.277149571127)*x[2]**o+(0.686042508229)*x[2]
        s[2,1,0]=(-0.415700364121)*x[0]**o+(-0.998036555412)*x[0]+(0.932589123491)*x[1]**o+(0.474376426544)*x[1]+(0.868363410527)*x[2]**o+(0.20412367755)*x[2]
        s[2,1,1]=(-0.699650926538)*x[0]**o+(-0.885390927226)*x[0]+(0.315114808594)*x[1]**o+(0.0964378966171)*x[1]+(0.208547711734)*x[2]**o+(-0.234035721141)*x[2]
        s[3,0,0]=(-0.876365887814)*x[0]**o+(-0.809219980908)*x[0]+(-0.956629553931)*x[1]**o+(0.879512507986)*x[1]+(0.263252429544)*x[2]**o+(-0.0839386837172)*x[2]
        s[3,0,1]=(-0.667178396657)*x[0]**o+(-0.16553182711)*x[0]+(0.118512822439)*x[1]**o+(0.855526624871)*x[1]+(0.367375405597)*x[2]**o+(0.809554378159)*x[2]
        s[3,1,0]=(-0.822091423901)*x[0]**o+(0.805358337724)*x[0]+(-0.0169105328354)*x[1]**o+(0.911509787417)*x[1]+(0.201376799793)*x[2]**o+(-0.421841209698)*x[2]
        s[3,1,1]=(-0.924137416522)*x[0]**o+(-0.513720649867)*x[0]+(0.997645049075)*x[1]**o+(0.0427298712226)*x[1]+(0.0973602746566)*x[2]**o+(-0.210094600037)*x[2]
        s[4,0,0]=(-0.900263073705)*x[0]**o+(-0.348978017479)*x[0]+(-0.670123763869)*x[1]**o+(0.842214206909)*x[1]+(0.487852589522)*x[2]**o+(-0.990296977407)*x[2]
        s[4,0,1]=(0.733939716764)*x[0]**o+(0.802871411295)*x[0]+(-0.72554972641)*x[1]**o+(-0.675299974439)*x[1]+(-0.851276375175)*x[2]**o+(0.185926545414)*x[2]
        s[4,1,0]=(-0.00700280887647)*x[0]**o+(-0.588965317678)*x[0]+(-0.709984221808)*x[1]**o+(-0.278713234487)*x[1]+(0.571081276655)*x[2]**o+(-0.226676185948)*x[2]
        s[4,1,1]=(0.227749111196)*x[0]**o+(-0.682960403376)*x[0]+(0.26793708046)*x[1]**o+(-0.366550488255)*x[1]+(-0.888857273365)*x[2]**o+(0.0688817602584)*x[2]
        s[5,0,0]=(-0.125200843744)*x[0]**o+(-0.629160785496)*x[0]+(0.899283261399)*x[1]**o+(0.550308867618)*x[1]+(0.768807415182)*x[2]**o+(0.443292284662)*x[2]
        s[5,0,1]=(-0.0304781411569)*x[0]**o+(-0.560459929762)*x[0]+(0.588807987213)*x[1]**o+(0.330516228598)*x[1]+(-0.620114930397)*x[2]**o+(-0.598508456872)*x[2]
        s[5,1,0]=(0.538580607824)*x[0]**o+(0.0303519877944)*x[0]+(-0.26997035534)*x[1]**o+(0.364935864108)*x[1]+(0.748685142241)*x[2]**o+(0.0962889009161)*x[2]
        s[5,1,1]=(-0.54994010911)*x[0]**o+(0.159394722103)*x[0]+(-0.21371135673)*x[1]**o+(-0.395979383594)*x[1]+(-0.314185117077)*x[2]**o+(-0.204275202783)*x[2]
        ref[0,0,0]=(0.0722666466735)/(o+1.)+(0.434053718473)+(-0.226564859572)*0.5**o
        ref[0,0,1]=(1.10896119251)/(o+1.)+(-0.452202216007)+(0.926463229691)*0.5**o
        ref[0,1,0]=(-0.58245433102)/(o+1.)+(0.393953523299)+(0.371916775894)*0.5**o
        ref[0,1,1]=(0.587501370293)/(o+1.)+(-0.512376615217)+(-0.871254388888)*0.5**o
        ref[1,0,0]=(0.883474490213)/(o+1.)+(-0.208989739816)+(0.0075965229641)*0.5**o
        ref[1,0,1]=(-0.511750337092)/(o+1.)+(0.370697122953)+(0.197284854866)*0.5**o
        ref[1,1,0]=(0.725880482112)/(o+1.)+(0.243614622688)+(-0.968464883049)*0.5**o
        ref[1,1,1]=(0.405259190249)/(o+1.)+(-0.414483121086)+(-0.240984840545)*0.5**o
        ref[2,0,0]=(-0.829447266843)/(o+1.)+(-0.642159465811)+(0.239124234745)*0.5**o
        ref[2,0,1]=(0.0164490292067)/(o+1.)+(0.464997432253)+(-0.126152295994)*0.5**o
        ref[2,1,0]=(1.80095253402)/(o+1.)+(-0.159768225659)+(-0.415700364121)*0.5**o
        ref[2,1,1]=(0.523662520328)/(o+1.)+(-0.511494375875)+(-0.699650926538)*0.5**o
        ref[3,0,0]=(-0.693377124388)/(o+1.)+(-0.00682307831984)+(-0.876365887814)*0.5**o
        ref[3,0,1]=(0.485888228036)/(o+1.)+(0.74977458796)+(-0.667178396657)*0.5**o
        ref[3,1,0]=(0.184466266958)/(o+1.)+(0.647513457721)+(-0.822091423901)*0.5**o
        ref[3,1,1]=(1.09500532373)/(o+1.)+(-0.340542689341)+(-0.924137416522)*0.5**o
        ref[4,0,0]=(-0.182271174347)/(o+1.)+(-0.248530393989)+(-0.900263073705)*0.5**o
        ref[4,0,1]=(-1.57682610159)/(o+1.)+(0.156748991135)+(0.733939716764)*0.5**o
        ref[4,1,0]=(-0.138902945153)/(o+1.)+(-0.547177369056)+(-0.00700280887647)*0.5**o
        ref[4,1,1]=(-0.620920192905)/(o+1.)+(-0.490314565686)+(0.227749111196)*0.5**o
        ref[5,0,0]=(1.66809067658)/(o+1.)+(0.182220183392)+(-0.125200843744)*0.5**o
        ref[5,0,1]=(-0.0313069431837)/(o+1.)+(-0.414226079018)+(-0.0304781411569)*0.5**o
        ref[5,1,0]=(0.478714786901)/(o+1.)+(0.245788376409)+(0.538580607824)*0.5**o
        ref[5,1,1]=(-0.527896473807)/(o+1.)+(-0.220429932137)+(-0.54994010911)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactOne_fromSymbol_ContinuousFunction_rank4(self):
      """
      tests integral of rank 4 Symbol on the FunctionOnContactOne

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports integral on FunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5, 3, 2))
      s=Data(0,(4, 5, 3, 2),w)
      ref=numpy.zeros((4, 5, 3, 2),numpy.float_)
      if dim==2:
        s[0,0,0,0]=(0.826151835054)*x[0]**o+(0.778522285868)*x[0]+(-0.798735149387)*x[1]**o+(-0.875242059866)*x[1]
        s[0,0,0,1]=(0.281853758764)*x[0]**o+(-0.194633585298)*x[0]+(-0.559216870357)*x[1]**o+(0.699122025971)*x[1]
        s[0,0,1,0]=(0.734893466243)*x[0]**o+(0.629132659777)*x[0]+(0.147096504378)*x[1]**o+(0.11035713758)*x[1]
        s[0,0,1,1]=(0.928300107915)*x[0]**o+(-0.334284397134)*x[0]+(-0.978653421141)*x[1]**o+(-0.523491002413)*x[1]
        s[0,0,2,0]=(-0.84104720017)*x[0]**o+(-0.977429111913)*x[0]+(0.975786707279)*x[1]**o+(0.202832002651)*x[1]
        s[0,0,2,1]=(0.505216717255)*x[0]**o+(0.893953504598)*x[0]+(-0.478609919678)*x[1]**o+(-0.0960789116423)*x[1]
        s[0,1,0,0]=(-0.588826769132)*x[0]**o+(-0.793581403019)*x[0]+(0.0325820539534)*x[1]**o+(0.479489246372)*x[1]
        s[0,1,0,1]=(0.201705696091)*x[0]**o+(0.807828768455)*x[0]+(0.943593904137)*x[1]**o+(0.805216955013)*x[1]
        s[0,1,1,0]=(0.458858343943)*x[0]**o+(0.687368390273)*x[0]+(0.106488644042)*x[1]**o+(-0.53370113771)*x[1]
        s[0,1,1,1]=(-0.445360193781)*x[0]**o+(-0.0579880476857)*x[0]+(0.164158781184)*x[1]**o+(0.610932454554)*x[1]
        s[0,1,2,0]=(0.631597276525)*x[0]**o+(-0.958557448004)*x[0]+(-0.484354639056)*x[1]**o+(0.151153827438)*x[1]
        s[0,1,2,1]=(-0.742804913722)*x[0]**o+(-0.841835209231)*x[0]+(-0.209463960361)*x[1]**o+(-0.858110011148)*x[1]
        s[0,2,0,0]=(-0.757921024916)*x[0]**o+(0.63659891937)*x[0]+(0.807935626935)*x[1]**o+(0.902142249374)*x[1]
        s[0,2,0,1]=(-0.256981783889)*x[0]**o+(-0.610855048181)*x[0]+(-0.574033583989)*x[1]**o+(0.776282494782)*x[1]
        s[0,2,1,0]=(-0.943218252718)*x[0]**o+(0.516037703505)*x[0]+(0.341543474661)*x[1]**o+(-0.240364780932)*x[1]
        s[0,2,1,1]=(0.792692625218)*x[0]**o+(-0.323976519304)*x[0]+(-0.0146665511353)*x[1]**o+(-0.735190429984)*x[1]
        s[0,2,2,0]=(-0.30799523168)*x[0]**o+(-0.679564713739)*x[0]+(0.459127847422)*x[1]**o+(0.232465201074)*x[1]
        s[0,2,2,1]=(-0.610374384606)*x[0]**o+(-0.953734773788)*x[0]+(0.548484423475)*x[1]**o+(-0.615367713658)*x[1]
        s[0,3,0,0]=(0.657511913357)*x[0]**o+(-0.145435401393)*x[0]+(0.994651694429)*x[1]**o+(0.384878777184)*x[1]
        s[0,3,0,1]=(0.583248456434)*x[0]**o+(-0.187182307545)*x[0]+(0.944791134312)*x[1]**o+(-0.186032769285)*x[1]
        s[0,3,1,0]=(-0.781508935766)*x[0]**o+(0.860022230884)*x[0]+(-0.101044667455)*x[1]**o+(-0.222316103336)*x[1]
        s[0,3,1,1]=(-0.867519386038)*x[0]**o+(0.929281502241)*x[0]+(-0.946297144806)*x[1]**o+(0.817224805708)*x[1]
        s[0,3,2,0]=(-0.281309467001)*x[0]**o+(-0.037739241916)*x[0]+(0.535177813023)*x[1]**o+(-0.729632726113)*x[1]
        s[0,3,2,1]=(-0.860996437102)*x[0]**o+(-0.0283243105523)*x[0]+(0.291253331071)*x[1]**o+(-0.49621519253)*x[1]
        s[0,4,0,0]=(-0.499079353771)*x[0]**o+(0.0512126982011)*x[0]+(-0.264697628051)*x[1]**o+(-0.557092064587)*x[1]
        s[0,4,0,1]=(0.886952704303)*x[0]**o+(0.835576589274)*x[0]+(0.436241831047)*x[1]**o+(0.205000756969)*x[1]
        s[0,4,1,0]=(-0.794113947126)*x[0]**o+(0.272838850139)*x[0]+(0.596844427835)*x[1]**o+(-0.0770378590458)*x[1]
        s[0,4,1,1]=(-0.0741729381021)*x[0]**o+(-0.352184991227)*x[0]+(-0.47050747773)*x[1]**o+(-0.0426137892879)*x[1]
        s[0,4,2,0]=(-0.574730110675)*x[0]**o+(0.72928953398)*x[0]+(0.570813169088)*x[1]**o+(-0.140526724895)*x[1]
        s[0,4,2,1]=(-0.956657753944)*x[0]**o+(0.83653343098)*x[0]+(0.156596161146)*x[1]**o+(0.565547857113)*x[1]
        s[1,0,0,0]=(-0.951078063957)*x[0]**o+(-0.76755461909)*x[0]+(0.889791199852)*x[1]**o+(-0.751303752794)*x[1]
        s[1,0,0,1]=(-0.601689692228)*x[0]**o+(-0.872977808857)*x[0]+(-0.216836641085)*x[1]**o+(-0.141390507272)*x[1]
        s[1,0,1,0]=(0.963503254167)*x[0]**o+(0.881676487866)*x[0]+(0.98286908038)*x[1]**o+(-0.387200882531)*x[1]
        s[1,0,1,1]=(-0.220998216482)*x[0]**o+(-0.942896953922)*x[0]+(-0.784318364616)*x[1]**o+(-0.117636798188)*x[1]
        s[1,0,2,0]=(-0.0629547087136)*x[0]**o+(0.375822895811)*x[0]+(0.297365047085)*x[1]**o+(-0.108671078054)*x[1]
        s[1,0,2,1]=(-0.898579538704)*x[0]**o+(-0.611755404412)*x[0]+(0.8150001419)*x[1]**o+(-0.956535007847)*x[1]
        s[1,1,0,0]=(-0.0983531790455)*x[0]**o+(0.399432233914)*x[0]+(0.455058141159)*x[1]**o+(-0.272367862823)*x[1]
        s[1,1,0,1]=(0.198516567418)*x[0]**o+(-0.599045119583)*x[0]+(-0.660954533474)*x[1]**o+(-0.138080345883)*x[1]
        s[1,1,1,0]=(-0.345947424795)*x[0]**o+(-0.112667218524)*x[0]+(0.389246060633)*x[1]**o+(0.554453113274)*x[1]
        s[1,1,1,1]=(-0.195677951352)*x[0]**o+(-0.926026514503)*x[0]+(0.206735465226)*x[1]**o+(0.789299584752)*x[1]
        s[1,1,2,0]=(0.286674162919)*x[0]**o+(-0.836060882258)*x[0]+(0.728613552854)*x[1]**o+(0.684001924365)*x[1]
        s[1,1,2,1]=(0.462677800728)*x[0]**o+(0.128558178852)*x[0]+(-0.234360929327)*x[1]**o+(-0.0906684049145)*x[1]
        s[1,2,0,0]=(-0.618359459011)*x[0]**o+(0.31224194045)*x[0]+(0.826946704031)*x[1]**o+(-0.383586867066)*x[1]
        s[1,2,0,1]=(0.79180929497)*x[0]**o+(-0.00403790010433)*x[0]+(-0.610232300686)*x[1]**o+(0.655034409375)*x[1]
        s[1,2,1,0]=(-0.454630490901)*x[0]**o+(-0.294433649415)*x[0]+(0.222616002742)*x[1]**o+(-0.880830082688)*x[1]
        s[1,2,1,1]=(-0.358323719668)*x[0]**o+(-0.506159090503)*x[0]+(-0.298070302518)*x[1]**o+(0.670788233813)*x[1]
        s[1,2,2,0]=(-0.953302373475)*x[0]**o+(0.262673942284)*x[0]+(0.293175442678)*x[1]**o+(0.327941364282)*x[1]
        s[1,2,2,1]=(0.0847099318446)*x[0]**o+(-0.378899343492)*x[0]+(-0.656552393979)*x[1]**o+(0.578336235439)*x[1]
        s[1,3,0,0]=(0.0792265074365)*x[0]**o+(-0.703381588828)*x[0]+(0.0644008346021)*x[1]**o+(-0.414604593416)*x[1]
        s[1,3,0,1]=(-0.312474679341)*x[0]**o+(0.117859249242)*x[0]+(-0.712500149358)*x[1]**o+(-0.160567536664)*x[1]
        s[1,3,1,0]=(0.0930078501772)*x[0]**o+(0.235241217821)*x[0]+(-0.74265516534)*x[1]**o+(-0.737722766295)*x[1]
        s[1,3,1,1]=(0.839924906913)*x[0]**o+(0.937666844502)*x[0]+(0.0794334270857)*x[1]**o+(0.863046562296)*x[1]
        s[1,3,2,0]=(0.154200394821)*x[0]**o+(-0.66290218047)*x[0]+(-0.206887854301)*x[1]**o+(0.504705955597)*x[1]
        s[1,3,2,1]=(0.427091397128)*x[0]**o+(0.646434479942)*x[0]+(-0.499236661621)*x[1]**o+(-0.97608178001)*x[1]
        s[1,4,0,0]=(0.993473425442)*x[0]**o+(-0.385478858515)*x[0]+(0.991785227798)*x[1]**o+(-0.317386719533)*x[1]
        s[1,4,0,1]=(0.905969952921)*x[0]**o+(0.580348604986)*x[0]+(-0.549435231471)*x[1]**o+(-0.864673820376)*x[1]
        s[1,4,1,0]=(-0.414114927962)*x[0]**o+(0.192124891008)*x[0]+(0.270145165409)*x[1]**o+(0.984593711748)*x[1]
        s[1,4,1,1]=(-0.537634564837)*x[0]**o+(-0.832898745236)*x[0]+(0.987067570291)*x[1]**o+(-0.829413594749)*x[1]
        s[1,4,2,0]=(0.298204375484)*x[0]**o+(0.595594979515)*x[0]+(0.691510051463)*x[1]**o+(0.404779663287)*x[1]
        s[1,4,2,1]=(0.924136889038)*x[0]**o+(-0.834498479792)*x[0]+(-0.0920482079502)*x[1]**o+(0.653266085415)*x[1]
        s[2,0,0,0]=(-0.623512868124)*x[0]**o+(-0.674266041327)*x[0]+(-0.0496779342723)*x[1]**o+(-0.0576390340208)*x[1]
        s[2,0,0,1]=(-0.675571390646)*x[0]**o+(0.212900825131)*x[0]+(-0.698331157728)*x[1]**o+(0.323039640381)*x[1]
        s[2,0,1,0]=(-0.137713646762)*x[0]**o+(0.678939423651)*x[0]+(0.724413391261)*x[1]**o+(-0.758331483567)*x[1]
        s[2,0,1,1]=(-0.790333229868)*x[0]**o+(0.0388624829814)*x[0]+(-0.163758448345)*x[1]**o+(0.749265467726)*x[1]
        s[2,0,2,0]=(0.401526106126)*x[0]**o+(-0.654255384177)*x[0]+(0.654866937965)*x[1]**o+(0.395353725816)*x[1]
        s[2,0,2,1]=(-0.691391545016)*x[0]**o+(0.576097089046)*x[0]+(-0.351703606836)*x[1]**o+(0.272891465932)*x[1]
        s[2,1,0,0]=(-0.994782416678)*x[0]**o+(0.417827176298)*x[0]+(0.933443347736)*x[1]**o+(0.729641898342)*x[1]
        s[2,1,0,1]=(0.888393462781)*x[0]**o+(0.0669727248876)*x[0]+(0.0751230852899)*x[1]**o+(0.79394522379)*x[1]
        s[2,1,1,0]=(-0.67951464573)*x[0]**o+(-0.398740440509)*x[0]+(-0.200110899369)*x[1]**o+(-0.0313062595594)*x[1]
        s[2,1,1,1]=(0.769634138219)*x[0]**o+(-0.873544326827)*x[0]+(-0.231357242392)*x[1]**o+(0.728827227861)*x[1]
        s[2,1,2,0]=(-0.740242586287)*x[0]**o+(0.779752812047)*x[0]+(-0.935858357069)*x[1]**o+(0.423733076889)*x[1]
        s[2,1,2,1]=(0.0522567442585)*x[0]**o+(-0.196246914879)*x[0]+(0.594009331267)*x[1]**o+(0.390510523467)*x[1]
        s[2,2,0,0]=(-0.202135247802)*x[0]**o+(0.406919371556)*x[0]+(0.336054961442)*x[1]**o+(-0.00656791632921)*x[1]
        s[2,2,0,1]=(-0.18797538396)*x[0]**o+(-0.502138169684)*x[0]+(0.728985032074)*x[1]**o+(-0.851899173542)*x[1]
        s[2,2,1,0]=(-0.1657521328)*x[0]**o+(-0.235483396897)*x[0]+(0.923860896997)*x[1]**o+(-0.534524775419)*x[1]
        s[2,2,1,1]=(0.318366946697)*x[0]**o+(-0.714726079393)*x[0]+(0.625778742556)*x[1]**o+(-0.364822812091)*x[1]
        s[2,2,2,0]=(0.796633973705)*x[0]**o+(0.20378716389)*x[0]+(-0.428464082609)*x[1]**o+(0.414097140984)*x[1]
        s[2,2,2,1]=(0.430909811076)*x[0]**o+(-0.711249875145)*x[0]+(0.234909867943)*x[1]**o+(0.469828473347)*x[1]
        s[2,3,0,0]=(0.0637185370545)*x[0]**o+(-0.347041553087)*x[0]+(-0.415901909163)*x[1]**o+(0.302973266512)*x[1]
        s[2,3,0,1]=(-0.0514048718664)*x[0]**o+(0.577688892083)*x[0]+(-0.847060157535)*x[1]**o+(-0.295910657349)*x[1]
        s[2,3,1,0]=(0.272384814885)*x[0]**o+(-0.457091092283)*x[0]+(0.294803721831)*x[1]**o+(0.863908100714)*x[1]
        s[2,3,1,1]=(-0.171649346026)*x[0]**o+(0.283285517302)*x[0]+(-0.525102967397)*x[1]**o+(0.517875061987)*x[1]
        s[2,3,2,0]=(-0.393961839836)*x[0]**o+(0.594690475758)*x[0]+(0.165742336314)*x[1]**o+(0.655933809047)*x[1]
        s[2,3,2,1]=(-0.911200880858)*x[0]**o+(0.182403795244)*x[0]+(0.489125521474)*x[1]**o+(0.0496056049896)*x[1]
        s[2,4,0,0]=(-0.159105772726)*x[0]**o+(0.625018347934)*x[0]+(-0.0170170509106)*x[1]**o+(0.302248488132)*x[1]
        s[2,4,0,1]=(0.898869898377)*x[0]**o+(-0.172540097096)*x[0]+(0.11416927686)*x[1]**o+(0.465614034585)*x[1]
        s[2,4,1,0]=(-0.72081868999)*x[0]**o+(-0.605232985848)*x[0]+(-0.889107167889)*x[1]**o+(-0.970389096931)*x[1]
        s[2,4,1,1]=(0.566591889778)*x[0]**o+(0.844421879775)*x[0]+(-0.68950638965)*x[1]**o+(-0.555658441668)*x[1]
        s[2,4,2,0]=(-0.677923523128)*x[0]**o+(-0.764792213508)*x[0]+(-0.940720806672)*x[1]**o+(-0.349132395924)*x[1]
        s[2,4,2,1]=(0.511357852175)*x[0]**o+(-0.688298821028)*x[0]+(0.260299341082)*x[1]**o+(-0.172719107959)*x[1]
        s[3,0,0,0]=(0.96250974067)*x[0]**o+(-0.126604059489)*x[0]+(-0.117880197012)*x[1]**o+(-0.914802074674)*x[1]
        s[3,0,0,1]=(0.149007631585)*x[0]**o+(-0.673349488321)*x[0]+(-0.993719500465)*x[1]**o+(-0.598353371432)*x[1]
        s[3,0,1,0]=(-0.808696105441)*x[0]**o+(-0.288375046884)*x[0]+(-0.770109544749)*x[1]**o+(0.830326211908)*x[1]
        s[3,0,1,1]=(-0.156477225552)*x[0]**o+(0.739349262635)*x[0]+(0.364144184522)*x[1]**o+(0.628195257168)*x[1]
        s[3,0,2,0]=(-0.447857061312)*x[0]**o+(0.974956110157)*x[0]+(-0.470610412287)*x[1]**o+(-0.576396903386)*x[1]
        s[3,0,2,1]=(-0.953524211957)*x[0]**o+(0.0172359158991)*x[0]+(-0.587968819301)*x[1]**o+(-0.0404250064791)*x[1]
        s[3,1,0,0]=(-0.438441555817)*x[0]**o+(-0.0863380014049)*x[0]+(-0.651726866234)*x[1]**o+(-0.757302907976)*x[1]
        s[3,1,0,1]=(0.365066315119)*x[0]**o+(0.252165659991)*x[0]+(-0.1682171231)*x[1]**o+(-0.597926603548)*x[1]
        s[3,1,1,0]=(-0.0608091331878)*x[0]**o+(-0.118607423591)*x[0]+(0.572219801453)*x[1]**o+(-0.620635555103)*x[1]
        s[3,1,1,1]=(-0.258140275212)*x[0]**o+(0.106650435513)*x[0]+(-0.739971594229)*x[1]**o+(0.595195330003)*x[1]
        s[3,1,2,0]=(-0.229810808135)*x[0]**o+(-0.891173708591)*x[0]+(0.257932505524)*x[1]**o+(0.561295616991)*x[1]
        s[3,1,2,1]=(-0.579302432005)*x[0]**o+(0.607984452716)*x[0]+(-0.155861940983)*x[1]**o+(0.388572816738)*x[1]
        s[3,2,0,0]=(0.87799491053)*x[0]**o+(0.434743191656)*x[0]+(0.219633220676)*x[1]**o+(-0.515493145923)*x[1]
        s[3,2,0,1]=(-0.855493053888)*x[0]**o+(0.173518924283)*x[0]+(0.0675155754173)*x[1]**o+(-0.700260220492)*x[1]
        s[3,2,1,0]=(-0.412519942294)*x[0]**o+(-0.0114933134972)*x[0]+(0.232846893376)*x[1]**o+(0.272202827241)*x[1]
        s[3,2,1,1]=(0.325402950163)*x[0]**o+(-0.455151259934)*x[0]+(0.88929658634)*x[1]**o+(0.147699225253)*x[1]
        s[3,2,2,0]=(-0.246804489391)*x[0]**o+(0.626746060036)*x[0]+(0.70088034171)*x[1]**o+(-0.492021577395)*x[1]
        s[3,2,2,1]=(0.161987385265)*x[0]**o+(-0.038894607114)*x[0]+(-0.326936847239)*x[1]**o+(0.181111852946)*x[1]
        s[3,3,0,0]=(-0.287388091741)*x[0]**o+(0.516752550941)*x[0]+(-0.941603336187)*x[1]**o+(0.549867040053)*x[1]
        s[3,3,0,1]=(-0.711983817048)*x[0]**o+(-0.257572806886)*x[0]+(-0.441118042147)*x[1]**o+(-0.672544376584)*x[1]
        s[3,3,1,0]=(0.27179221385)*x[0]**o+(0.722679194783)*x[0]+(0.154628962433)*x[1]**o+(0.406406592446)*x[1]
        s[3,3,1,1]=(-0.484359850969)*x[0]**o+(-0.179916115792)*x[0]+(-0.461756438449)*x[1]**o+(-0.484596755972)*x[1]
        s[3,3,2,0]=(0.06709022871)*x[0]**o+(0.79525514856)*x[0]+(0.815544226952)*x[1]**o+(-0.498264422355)*x[1]
        s[3,3,2,1]=(0.843256724396)*x[0]**o+(0.0796294835103)*x[0]+(0.301414364597)*x[1]**o+(0.790478221822)*x[1]
        s[3,4,0,0]=(0.723595119935)*x[0]**o+(-0.757921136678)*x[0]+(-0.823917708581)*x[1]**o+(0.0624785383724)*x[1]
        s[3,4,0,1]=(0.733680579182)*x[0]**o+(-0.383756889549)*x[0]+(0.161828894605)*x[1]**o+(-0.192838756613)*x[1]
        s[3,4,1,0]=(0.147016857318)*x[0]**o+(0.0580828047654)*x[0]+(-0.628782245085)*x[1]**o+(-0.607487083314)*x[1]
        s[3,4,1,1]=(-0.0882094727061)*x[0]**o+(0.790852741583)*x[0]+(0.47412676393)*x[1]**o+(-0.873733941999)*x[1]
        s[3,4,2,0]=(0.0294423769171)*x[0]**o+(0.522861977201)*x[0]+(-0.360682765428)*x[1]**o+(-0.460704794865)*x[1]
        s[3,4,2,1]=(0.143274846749)*x[0]**o+(-0.164306681884)*x[0]+(0.632228849654)*x[1]**o+(-0.488837677105)*x[1]
        ref[0,0,0,0]=(-0.798735149387)/(o+1.)+(-0.0483598869992)+(0.826151835054)*0.5**o
        ref[0,0,0,1]=(-0.559216870357)/(o+1.)+(0.252244220336)+(0.281853758764)*0.5**o
        ref[0,0,1,0]=(0.147096504378)/(o+1.)+(0.369744898679)+(0.734893466243)*0.5**o
        ref[0,0,1,1]=(-0.978653421141)/(o+1.)+(-0.428887699773)+(0.928300107915)*0.5**o
        ref[0,0,2,0]=(0.975786707279)/(o+1.)+(-0.387298554631)+(-0.84104720017)*0.5**o
        ref[0,0,2,1]=(-0.478609919678)/(o+1.)+(0.398937296478)+(0.505216717255)*0.5**o
        ref[0,1,0,0]=(0.0325820539534)/(o+1.)+(-0.157046078324)+(-0.588826769132)*0.5**o
        ref[0,1,0,1]=(0.943593904137)/(o+1.)+(0.806522861734)+(0.201705696091)*0.5**o
        ref[0,1,1,0]=(0.106488644042)/(o+1.)+(0.0768336262816)+(0.458858343943)*0.5**o
        ref[0,1,1,1]=(0.164158781184)/(o+1.)+(0.276472203434)+(-0.445360193781)*0.5**o
        ref[0,1,2,0]=(-0.484354639056)/(o+1.)+(-0.403701810283)+(0.631597276525)*0.5**o
        ref[0,1,2,1]=(-0.209463960361)/(o+1.)+(-0.84997261019)+(-0.742804913722)*0.5**o
        ref[0,2,0,0]=(0.807935626935)/(o+1.)+(0.769370584372)+(-0.757921024916)*0.5**o
        ref[0,2,0,1]=(-0.574033583989)/(o+1.)+(0.0827137233007)+(-0.256981783889)*0.5**o
        ref[0,2,1,0]=(0.341543474661)/(o+1.)+(0.137836461287)+(-0.943218252718)*0.5**o
        ref[0,2,1,1]=(-0.0146665511353)/(o+1.)+(-0.529583474644)+(0.792692625218)*0.5**o
        ref[0,2,2,0]=(0.459127847422)/(o+1.)+(-0.223549756332)+(-0.30799523168)*0.5**o
        ref[0,2,2,1]=(0.548484423475)/(o+1.)+(-0.784551243723)+(-0.610374384606)*0.5**o
        ref[0,3,0,0]=(0.994651694429)/(o+1.)+(0.119721687896)+(0.657511913357)*0.5**o
        ref[0,3,0,1]=(0.944791134312)/(o+1.)+(-0.186607538415)+(0.583248456434)*0.5**o
        ref[0,3,1,0]=(-0.101044667455)/(o+1.)+(0.318853063774)+(-0.781508935766)*0.5**o
        ref[0,3,1,1]=(-0.946297144806)/(o+1.)+(0.873253153974)+(-0.867519386038)*0.5**o
        ref[0,3,2,0]=(0.535177813023)/(o+1.)+(-0.383685984015)+(-0.281309467001)*0.5**o
        ref[0,3,2,1]=(0.291253331071)/(o+1.)+(-0.262269751541)+(-0.860996437102)*0.5**o
        ref[0,4,0,0]=(-0.264697628051)/(o+1.)+(-0.252939683193)+(-0.499079353771)*0.5**o
        ref[0,4,0,1]=(0.436241831047)/(o+1.)+(0.520288673122)+(0.886952704303)*0.5**o
        ref[0,4,1,0]=(0.596844427835)/(o+1.)+(0.0979004955464)+(-0.794113947126)*0.5**o
        ref[0,4,1,1]=(-0.47050747773)/(o+1.)+(-0.197399390257)+(-0.0741729381021)*0.5**o
        ref[0,4,2,0]=(0.570813169088)/(o+1.)+(0.294381404543)+(-0.574730110675)*0.5**o
        ref[0,4,2,1]=(0.156596161146)/(o+1.)+(0.701040644046)+(-0.956657753944)*0.5**o
        ref[1,0,0,0]=(0.889791199852)/(o+1.)+(-0.759429185942)+(-0.951078063957)*0.5**o
        ref[1,0,0,1]=(-0.216836641085)/(o+1.)+(-0.507184158064)+(-0.601689692228)*0.5**o
        ref[1,0,1,0]=(0.98286908038)/(o+1.)+(0.247237802667)+(0.963503254167)*0.5**o
        ref[1,0,1,1]=(-0.784318364616)/(o+1.)+(-0.530266876055)+(-0.220998216482)*0.5**o
        ref[1,0,2,0]=(0.297365047085)/(o+1.)+(0.133575908879)+(-0.0629547087136)*0.5**o
        ref[1,0,2,1]=(0.8150001419)/(o+1.)+(-0.784145206129)+(-0.898579538704)*0.5**o
        ref[1,1,0,0]=(0.455058141159)/(o+1.)+(0.0635321855459)+(-0.0983531790455)*0.5**o
        ref[1,1,0,1]=(-0.660954533474)/(o+1.)+(-0.368562732733)+(0.198516567418)*0.5**o
        ref[1,1,1,0]=(0.389246060633)/(o+1.)+(0.220892947375)+(-0.345947424795)*0.5**o
        ref[1,1,1,1]=(0.206735465226)/(o+1.)+(-0.0683634648758)+(-0.195677951352)*0.5**o
        ref[1,1,2,0]=(0.728613552854)/(o+1.)+(-0.0760294789467)+(0.286674162919)*0.5**o
        ref[1,1,2,1]=(-0.234360929327)/(o+1.)+(0.0189448869689)+(0.462677800728)*0.5**o
        ref[1,2,0,0]=(0.826946704031)/(o+1.)+(-0.035672463308)+(-0.618359459011)*0.5**o
        ref[1,2,0,1]=(-0.610232300686)/(o+1.)+(0.325498254635)+(0.79180929497)*0.5**o
        ref[1,2,1,0]=(0.222616002742)/(o+1.)+(-0.587631866051)+(-0.454630490901)*0.5**o
        ref[1,2,1,1]=(-0.298070302518)/(o+1.)+(0.082314571655)+(-0.358323719668)*0.5**o
        ref[1,2,2,0]=(0.293175442678)/(o+1.)+(0.295307653283)+(-0.953302373475)*0.5**o
        ref[1,2,2,1]=(-0.656552393979)/(o+1.)+(0.0997184459739)+(0.0847099318446)*0.5**o
        ref[1,3,0,0]=(0.0644008346021)/(o+1.)+(-0.558993091122)+(0.0792265074365)*0.5**o
        ref[1,3,0,1]=(-0.712500149358)/(o+1.)+(-0.021354143711)+(-0.312474679341)*0.5**o
        ref[1,3,1,0]=(-0.74265516534)/(o+1.)+(-0.251240774237)+(0.0930078501772)*0.5**o
        ref[1,3,1,1]=(0.0794334270857)/(o+1.)+(0.900356703399)+(0.839924906913)*0.5**o
        ref[1,3,2,0]=(-0.206887854301)/(o+1.)+(-0.0790981124364)+(0.154200394821)*0.5**o
        ref[1,3,2,1]=(-0.499236661621)/(o+1.)+(-0.164823650034)+(0.427091397128)*0.5**o
        ref[1,4,0,0]=(0.991785227798)/(o+1.)+(-0.351432789024)+(0.993473425442)*0.5**o
        ref[1,4,0,1]=(-0.549435231471)/(o+1.)+(-0.142162607695)+(0.905969952921)*0.5**o
        ref[1,4,1,0]=(0.270145165409)/(o+1.)+(0.588359301378)+(-0.414114927962)*0.5**o
        ref[1,4,1,1]=(0.987067570291)/(o+1.)+(-0.831156169992)+(-0.537634564837)*0.5**o
        ref[1,4,2,0]=(0.691510051463)/(o+1.)+(0.500187321401)+(0.298204375484)*0.5**o
        ref[1,4,2,1]=(-0.0920482079502)/(o+1.)+(-0.0906161971883)+(0.924136889038)*0.5**o
        ref[2,0,0,0]=(-0.0496779342723)/(o+1.)+(-0.365952537674)+(-0.623512868124)*0.5**o
        ref[2,0,0,1]=(-0.698331157728)/(o+1.)+(0.267970232756)+(-0.675571390646)*0.5**o
        ref[2,0,1,0]=(0.724413391261)/(o+1.)+(-0.0396960299582)+(-0.137713646762)*0.5**o
        ref[2,0,1,1]=(-0.163758448345)/(o+1.)+(0.394063975354)+(-0.790333229868)*0.5**o
        ref[2,0,2,0]=(0.654866937965)/(o+1.)+(-0.12945082918)+(0.401526106126)*0.5**o
        ref[2,0,2,1]=(-0.351703606836)/(o+1.)+(0.424494277489)+(-0.691391545016)*0.5**o
        ref[2,1,0,0]=(0.933443347736)/(o+1.)+(0.57373453732)+(-0.994782416678)*0.5**o
        ref[2,1,0,1]=(0.0751230852899)/(o+1.)+(0.430458974339)+(0.888393462781)*0.5**o
        ref[2,1,1,0]=(-0.200110899369)/(o+1.)+(-0.215023350034)+(-0.67951464573)*0.5**o
        ref[2,1,1,1]=(-0.231357242392)/(o+1.)+(-0.0723585494832)+(0.769634138219)*0.5**o
        ref[2,1,2,0]=(-0.935858357069)/(o+1.)+(0.601742944468)+(-0.740242586287)*0.5**o
        ref[2,1,2,1]=(0.594009331267)/(o+1.)+(0.0971318042938)+(0.0522567442585)*0.5**o
        ref[2,2,0,0]=(0.336054961442)/(o+1.)+(0.200175727613)+(-0.202135247802)*0.5**o
        ref[2,2,0,1]=(0.728985032074)/(o+1.)+(-0.677018671613)+(-0.18797538396)*0.5**o
        ref[2,2,1,0]=(0.923860896997)/(o+1.)+(-0.385004086158)+(-0.1657521328)*0.5**o
        ref[2,2,1,1]=(0.625778742556)/(o+1.)+(-0.539774445742)+(0.318366946697)*0.5**o
        ref[2,2,2,0]=(-0.428464082609)/(o+1.)+(0.308942152437)+(0.796633973705)*0.5**o
        ref[2,2,2,1]=(0.234909867943)/(o+1.)+(-0.120710700899)+(0.430909811076)*0.5**o
        ref[2,3,0,0]=(-0.415901909163)/(o+1.)+(-0.0220341432872)+(0.0637185370545)*0.5**o
        ref[2,3,0,1]=(-0.847060157535)/(o+1.)+(0.140889117367)+(-0.0514048718664)*0.5**o
        ref[2,3,1,0]=(0.294803721831)/(o+1.)+(0.203408504215)+(0.272384814885)*0.5**o
        ref[2,3,1,1]=(-0.525102967397)/(o+1.)+(0.400580289645)+(-0.171649346026)*0.5**o
        ref[2,3,2,0]=(0.165742336314)/(o+1.)+(0.625312142402)+(-0.393961839836)*0.5**o
        ref[2,3,2,1]=(0.489125521474)/(o+1.)+(0.116004700117)+(-0.911200880858)*0.5**o
        ref[2,4,0,0]=(-0.0170170509106)/(o+1.)+(0.463633418033)+(-0.159105772726)*0.5**o
        ref[2,4,0,1]=(0.11416927686)/(o+1.)+(0.146536968744)+(0.898869898377)*0.5**o
        ref[2,4,1,0]=(-0.889107167889)/(o+1.)+(-0.78781104139)+(-0.72081868999)*0.5**o
        ref[2,4,1,1]=(-0.68950638965)/(o+1.)+(0.144381719054)+(0.566591889778)*0.5**o
        ref[2,4,2,0]=(-0.940720806672)/(o+1.)+(-0.556962304716)+(-0.677923523128)*0.5**o
        ref[2,4,2,1]=(0.260299341082)/(o+1.)+(-0.430508964494)+(0.511357852175)*0.5**o
        ref[3,0,0,0]=(-0.117880197012)/(o+1.)+(-0.520703067081)+(0.96250974067)*0.5**o
        ref[3,0,0,1]=(-0.993719500465)/(o+1.)+(-0.635851429876)+(0.149007631585)*0.5**o
        ref[3,0,1,0]=(-0.770109544749)/(o+1.)+(0.270975582512)+(-0.808696105441)*0.5**o
        ref[3,0,1,1]=(0.364144184522)/(o+1.)+(0.683772259902)+(-0.156477225552)*0.5**o
        ref[3,0,2,0]=(-0.470610412287)/(o+1.)+(0.199279603385)+(-0.447857061312)*0.5**o
        ref[3,0,2,1]=(-0.587968819301)/(o+1.)+(-0.01159454529)+(-0.953524211957)*0.5**o
        ref[3,1,0,0]=(-0.651726866234)/(o+1.)+(-0.42182045469)+(-0.438441555817)*0.5**o
        ref[3,1,0,1]=(-0.1682171231)/(o+1.)+(-0.172880471778)+(0.365066315119)*0.5**o
        ref[3,1,1,0]=(0.572219801453)/(o+1.)+(-0.369621489347)+(-0.0608091331878)*0.5**o
        ref[3,1,1,1]=(-0.739971594229)/(o+1.)+(0.350922882758)+(-0.258140275212)*0.5**o
        ref[3,1,2,0]=(0.257932505524)/(o+1.)+(-0.1649390458)+(-0.229810808135)*0.5**o
        ref[3,1,2,1]=(-0.155861940983)/(o+1.)+(0.498278634727)+(-0.579302432005)*0.5**o
        ref[3,2,0,0]=(0.219633220676)/(o+1.)+(-0.0403749771337)+(0.87799491053)*0.5**o
        ref[3,2,0,1]=(0.0675155754173)/(o+1.)+(-0.263370648105)+(-0.855493053888)*0.5**o
        ref[3,2,1,0]=(0.232846893376)/(o+1.)+(0.130354756872)+(-0.412519942294)*0.5**o
        ref[3,2,1,1]=(0.88929658634)/(o+1.)+(-0.15372601734)+(0.325402950163)*0.5**o
        ref[3,2,2,0]=(0.70088034171)/(o+1.)+(0.0673622413205)+(-0.246804489391)*0.5**o
        ref[3,2,2,1]=(-0.326936847239)/(o+1.)+(0.071108622916)+(0.161987385265)*0.5**o
        ref[3,3,0,0]=(-0.941603336187)/(o+1.)+(0.533309795497)+(-0.287388091741)*0.5**o
        ref[3,3,0,1]=(-0.441118042147)/(o+1.)+(-0.465058591735)+(-0.711983817048)*0.5**o
        ref[3,3,1,0]=(0.154628962433)/(o+1.)+(0.564542893615)+(0.27179221385)*0.5**o
        ref[3,3,1,1]=(-0.461756438449)/(o+1.)+(-0.332256435882)+(-0.484359850969)*0.5**o
        ref[3,3,2,0]=(0.815544226952)/(o+1.)+(0.148495363102)+(0.06709022871)*0.5**o
        ref[3,3,2,1]=(0.301414364597)/(o+1.)+(0.435053852666)+(0.843256724396)*0.5**o
        ref[3,4,0,0]=(-0.823917708581)/(o+1.)+(-0.347721299153)+(0.723595119935)*0.5**o
        ref[3,4,0,1]=(0.161828894605)/(o+1.)+(-0.288297823081)+(0.733680579182)*0.5**o
        ref[3,4,1,0]=(-0.628782245085)/(o+1.)+(-0.274702139274)+(0.147016857318)*0.5**o
        ref[3,4,1,1]=(0.47412676393)/(o+1.)+(-0.0414406002081)+(-0.0882094727061)*0.5**o
        ref[3,4,2,0]=(-0.360682765428)/(o+1.)+(0.0310785911681)+(0.0294423769171)*0.5**o
        ref[3,4,2,1]=(0.632228849654)/(o+1.)+(-0.326572179494)+(0.143274846749)*0.5**o
      else:
        s[0,0,0,0]=(0.356314179645)*x[0]**o+(-0.32357850671)*x[0]+(-0.824656670425)*x[1]**o+(-0.627681417632)*x[1]+(-0.269037662166)*x[2]**o+(-0.506111289647)*x[2]
        s[0,0,0,1]=(0.209589534917)*x[0]**o+(-0.245850334755)*x[0]+(-0.933340495998)*x[1]**o+(-0.091551513676)*x[1]+(-0.903932207206)*x[2]**o+(-0.462176705079)*x[2]
        s[0,0,1,0]=(-0.242847848445)*x[0]**o+(-0.303798875175)*x[0]+(-0.89496979627)*x[1]**o+(-0.442220551)*x[1]+(0.686085734196)*x[2]**o+(-0.422307267905)*x[2]
        s[0,0,1,1]=(0.255545417045)*x[0]**o+(0.0456454977479)*x[0]+(-0.378619350633)*x[1]**o+(0.720981703656)*x[1]+(0.251875164005)*x[2]**o+(-0.220351133411)*x[2]
        s[0,0,2,0]=(-0.962070081459)*x[0]**o+(0.697584448738)*x[0]+(0.855321986438)*x[1]**o+(0.818489831335)*x[1]+(-0.892304050049)*x[2]**o+(0.609699703411)*x[2]
        s[0,0,2,1]=(0.636687005195)*x[0]**o+(-0.4086000867)*x[0]+(-0.0426817477748)*x[1]**o+(-0.842690837453)*x[1]+(-0.704599957019)*x[2]**o+(0.518333776253)*x[2]
        s[0,1,0,0]=(0.521790025796)*x[0]**o+(-0.987699503457)*x[0]+(0.0903316739912)*x[1]**o+(0.307827334512)*x[1]+(-0.401791463687)*x[2]**o+(-0.702368788162)*x[2]
        s[0,1,0,1]=(-0.642544941536)*x[0]**o+(-0.881565606872)*x[0]+(-0.863360075641)*x[1]**o+(0.927565055153)*x[1]+(0.286728291693)*x[2]**o+(0.0652315285371)*x[2]
        s[0,1,1,0]=(0.987946287729)*x[0]**o+(-0.95542434477)*x[0]+(0.876907614088)*x[1]**o+(0.917353337436)*x[1]+(-0.40420250407)*x[2]**o+(-0.984390334421)*x[2]
        s[0,1,1,1]=(-0.848091494031)*x[0]**o+(0.205692942903)*x[0]+(0.804893714353)*x[1]**o+(0.34369751042)*x[1]+(-0.948317185277)*x[2]**o+(-0.937737932921)*x[2]
        s[0,1,2,0]=(-0.411836604088)*x[0]**o+(-0.520147883853)*x[0]+(0.10615721253)*x[1]**o+(0.674921463188)*x[1]+(0.245733286882)*x[2]**o+(-0.653168166924)*x[2]
        s[0,1,2,1]=(0.0397981165279)*x[0]**o+(0.667676059311)*x[0]+(0.720734430282)*x[1]**o+(-0.232075762355)*x[1]+(-0.558842944595)*x[2]**o+(0.39581457339)*x[2]
        s[0,2,0,0]=(-0.043373970003)*x[0]**o+(0.797078438464)*x[0]+(-0.488567358998)*x[1]**o+(0.0955394876958)*x[1]+(0.469780932691)*x[2]**o+(0.665728903694)*x[2]
        s[0,2,0,1]=(0.204098937116)*x[0]**o+(0.16435985408)*x[0]+(-0.245228652593)*x[1]**o+(-0.423628218893)*x[1]+(-0.0691452707964)*x[2]**o+(-0.709704899728)*x[2]
        s[0,2,1,0]=(0.506878702079)*x[0]**o+(0.740560436601)*x[0]+(0.193885598321)*x[1]**o+(0.521852898085)*x[1]+(0.0368080590022)*x[2]**o+(0.986970659964)*x[2]
        s[0,2,1,1]=(-0.261656641804)*x[0]**o+(-0.407521171557)*x[0]+(0.966151887826)*x[1]**o+(-0.918908288322)*x[1]+(-0.370153374393)*x[2]**o+(0.236661704457)*x[2]
        s[0,2,2,0]=(-0.491629682103)*x[0]**o+(-0.593057448837)*x[0]+(-0.022126429921)*x[1]**o+(-0.509356725061)*x[1]+(-0.680259680283)*x[2]**o+(-0.889690558667)*x[2]
        s[0,2,2,1]=(-0.564089634025)*x[0]**o+(0.0574486294037)*x[0]+(0.0578539449865)*x[1]**o+(0.258278489918)*x[1]+(0.174461168842)*x[2]**o+(0.0842548610258)*x[2]
        s[0,3,0,0]=(0.0606363217868)*x[0]**o+(0.160338081583)*x[0]+(-0.0636204375814)*x[1]**o+(0.287489206156)*x[1]+(-0.592019780262)*x[2]**o+(0.508884293667)*x[2]
        s[0,3,0,1]=(-0.702879071079)*x[0]**o+(0.874961299195)*x[0]+(-0.00733354592216)*x[1]**o+(0.0223647905227)*x[1]+(0.956232905909)*x[2]**o+(0.369832655586)*x[2]
        s[0,3,1,0]=(-0.824855043544)*x[0]**o+(-0.529925913352)*x[0]+(-0.0897704543542)*x[1]**o+(0.655124384268)*x[1]+(-0.586221756018)*x[2]**o+(0.104165560368)*x[2]
        s[0,3,1,1]=(-0.890319589507)*x[0]**o+(0.791154767703)*x[0]+(-0.315134766255)*x[1]**o+(-0.384720188852)*x[1]+(0.130920946919)*x[2]**o+(0.894095912396)*x[2]
        s[0,3,2,0]=(0.366123401919)*x[0]**o+(0.806533885372)*x[0]+(-0.579579932468)*x[1]**o+(0.490534769858)*x[1]+(0.250581383716)*x[2]**o+(-0.326970590496)*x[2]
        s[0,3,2,1]=(-0.438035010616)*x[0]**o+(-0.630762254554)*x[0]+(0.0848919367225)*x[1]**o+(-0.872733528687)*x[1]+(-0.465429840653)*x[2]**o+(0.567524409476)*x[2]
        s[0,4,0,0]=(0.319555220189)*x[0]**o+(0.397272010924)*x[0]+(-0.827915669722)*x[1]**o+(-0.0370119836485)*x[1]+(0.232621621392)*x[2]**o+(0.376285172425)*x[2]
        s[0,4,0,1]=(-0.874144853872)*x[0]**o+(0.740754812769)*x[0]+(0.878074282473)*x[1]**o+(-0.168290826137)*x[1]+(-0.0466881676326)*x[2]**o+(0.997360121528)*x[2]
        s[0,4,1,0]=(0.622984208352)*x[0]**o+(-0.60223710711)*x[0]+(-0.684599543294)*x[1]**o+(-0.654758989528)*x[1]+(-0.209806071733)*x[2]**o+(0.0331737697413)*x[2]
        s[0,4,1,1]=(-0.175155841122)*x[0]**o+(-0.73030489546)*x[0]+(0.226809166193)*x[1]**o+(-0.242344848157)*x[1]+(-0.331283539462)*x[2]**o+(-0.392781337615)*x[2]
        s[0,4,2,0]=(0.111634467654)*x[0]**o+(-0.0622591150598)*x[0]+(-0.675485413951)*x[1]**o+(-0.0169689015565)*x[1]+(-0.373014699305)*x[2]**o+(-0.827576200434)*x[2]
        s[0,4,2,1]=(0.769210305419)*x[0]**o+(-0.380741842544)*x[0]+(-0.0136492374016)*x[1]**o+(0.231367701468)*x[1]+(-0.156422804273)*x[2]**o+(0.69522180005)*x[2]
        s[1,0,0,0]=(-0.907821074278)*x[0]**o+(-0.466564046295)*x[0]+(-0.0115096148313)*x[1]**o+(0.682571066758)*x[1]+(-0.188270659989)*x[2]**o+(0.769027925506)*x[2]
        s[1,0,0,1]=(-0.935692950448)*x[0]**o+(0.801782734141)*x[0]+(-0.694482272453)*x[1]**o+(-0.357697785902)*x[1]+(0.604605447034)*x[2]**o+(-0.484488572975)*x[2]
        s[1,0,1,0]=(0.958399972856)*x[0]**o+(0.684229220211)*x[0]+(0.952743494375)*x[1]**o+(0.247944736517)*x[1]+(-0.251674820491)*x[2]**o+(-0.948785335663)*x[2]
        s[1,0,1,1]=(0.480962582731)*x[0]**o+(-0.233075914633)*x[0]+(-0.407820663337)*x[1]**o+(0.74897610034)*x[1]+(0.869138425397)*x[2]**o+(-0.115144267319)*x[2]
        s[1,0,2,0]=(-0.459646484527)*x[0]**o+(-0.42415176628)*x[0]+(0.361277035435)*x[1]**o+(-0.894956929165)*x[1]+(-0.710531250988)*x[2]**o+(-0.0704645482372)*x[2]
        s[1,0,2,1]=(-0.663523285059)*x[0]**o+(0.42693709011)*x[0]+(-0.225209635066)*x[1]**o+(0.505033755104)*x[1]+(-0.612237573846)*x[2]**o+(-0.824405122543)*x[2]
        s[1,1,0,0]=(-0.963081878353)*x[0]**o+(-0.318420605399)*x[0]+(-0.841731614457)*x[1]**o+(0.0524113793424)*x[1]+(0.824353304271)*x[2]**o+(-0.347662639363)*x[2]
        s[1,1,0,1]=(-0.426340341727)*x[0]**o+(-0.621345618708)*x[0]+(-0.599191472774)*x[1]**o+(0.720942014806)*x[1]+(0.699997023602)*x[2]**o+(-0.0401845986151)*x[2]
        s[1,1,1,0]=(0.195081954019)*x[0]**o+(0.404513751671)*x[0]+(-0.691733548084)*x[1]**o+(0.494301087961)*x[1]+(0.660194160561)*x[2]**o+(-0.719750815548)*x[2]
        s[1,1,1,1]=(0.174243105183)*x[0]**o+(-0.490648510315)*x[0]+(-0.921695831048)*x[1]**o+(0.03988622749)*x[1]+(0.600722960619)*x[2]**o+(-0.190758484114)*x[2]
        s[1,1,2,0]=(0.16246409321)*x[0]**o+(-0.861115959612)*x[0]+(-0.813061871575)*x[1]**o+(0.335664223473)*x[1]+(0.0462405018327)*x[2]**o+(-0.187262892401)*x[2]
        s[1,1,2,1]=(0.0961690353732)*x[0]**o+(0.388828620097)*x[0]+(0.108950572002)*x[1]**o+(-0.163688546076)*x[1]+(-0.278121101224)*x[2]**o+(0.612009658813)*x[2]
        s[1,2,0,0]=(-0.641539480622)*x[0]**o+(-0.459587156004)*x[0]+(0.672390673512)*x[1]**o+(-0.306916192961)*x[1]+(-0.350373910022)*x[2]**o+(0.846575285764)*x[2]
        s[1,2,0,1]=(0.305867854526)*x[0]**o+(0.494447320742)*x[0]+(0.325369680551)*x[1]**o+(-0.698268159382)*x[1]+(0.88031161562)*x[2]**o+(0.049240112332)*x[2]
        s[1,2,1,0]=(0.0262323517248)*x[0]**o+(0.165055680035)*x[0]+(0.978941268257)*x[1]**o+(-0.166940249199)*x[1]+(0.169948801883)*x[2]**o+(0.521926242685)*x[2]
        s[1,2,1,1]=(-0.73494826134)*x[0]**o+(-0.908473852515)*x[0]+(0.19661832779)*x[1]**o+(-0.0112732740593)*x[1]+(0.931924432628)*x[2]**o+(-0.433369394703)*x[2]
        s[1,2,2,0]=(-0.405925175898)*x[0]**o+(0.190346680156)*x[0]+(-0.486067244548)*x[1]**o+(-0.745250360184)*x[1]+(0.367855074214)*x[2]**o+(-0.489383863963)*x[2]
        s[1,2,2,1]=(-0.552448017406)*x[0]**o+(-0.10269646845)*x[0]+(0.216638861349)*x[1]**o+(0.285051823646)*x[1]+(-0.230326588392)*x[2]**o+(-0.963919984207)*x[2]
        s[1,3,0,0]=(0.498736551815)*x[0]**o+(0.714104256637)*x[0]+(-0.919467236631)*x[1]**o+(0.435774954621)*x[1]+(0.984691072998)*x[2]**o+(-0.385931573529)*x[2]
        s[1,3,0,1]=(-0.570322022545)*x[0]**o+(-0.431865640646)*x[0]+(-0.670344645547)*x[1]**o+(-0.0350120918436)*x[1]+(-0.574585723354)*x[2]**o+(-0.223611830882)*x[2]
        s[1,3,1,0]=(-0.978507788576)*x[0]**o+(0.692133143224)*x[0]+(-0.914092231492)*x[1]**o+(-0.878609687469)*x[1]+(0.123811943713)*x[2]**o+(0.566089817653)*x[2]
        s[1,3,1,1]=(0.413054411011)*x[0]**o+(0.134306111827)*x[0]+(0.0881588523756)*x[1]**o+(-0.260874928976)*x[1]+(0.59908530007)*x[2]**o+(-0.790312441625)*x[2]
        s[1,3,2,0]=(-0.56020772952)*x[0]**o+(-0.821899156728)*x[0]+(0.657107279954)*x[1]**o+(0.714198844049)*x[1]+(-0.797822774901)*x[2]**o+(-0.683075322516)*x[2]
        s[1,3,2,1]=(-0.784712199154)*x[0]**o+(0.257785308806)*x[0]+(0.668249127767)*x[1]**o+(-0.175010396143)*x[1]+(-0.450998789628)*x[2]**o+(-0.496319680914)*x[2]
        s[1,4,0,0]=(-0.361383889488)*x[0]**o+(-0.507929033088)*x[0]+(0.326087065339)*x[1]**o+(-0.672507245637)*x[1]+(-0.933796910616)*x[2]**o+(-0.964546616709)*x[2]
        s[1,4,0,1]=(-0.359553116054)*x[0]**o+(0.0378595816875)*x[0]+(0.866729184315)*x[1]**o+(-0.109325131607)*x[1]+(-0.436395616824)*x[2]**o+(-0.0250686006967)*x[2]
        s[1,4,1,0]=(-0.822256610634)*x[0]**o+(0.717956881162)*x[0]+(0.325671939606)*x[1]**o+(0.678862239184)*x[1]+(-0.721071141214)*x[2]**o+(0.371297227578)*x[2]
        s[1,4,1,1]=(-0.193864272459)*x[0]**o+(-0.165902167768)*x[0]+(-0.0799495476929)*x[1]**o+(-0.205949634256)*x[1]+(-0.830680911165)*x[2]**o+(-0.152405450656)*x[2]
        s[1,4,2,0]=(0.613697794358)*x[0]**o+(0.912521661263)*x[0]+(-0.00649065024755)*x[1]**o+(-0.215763322626)*x[1]+(-0.263693770435)*x[2]**o+(-0.596480965279)*x[2]
        s[1,4,2,1]=(0.234126882167)*x[0]**o+(0.912840520614)*x[0]+(0.933840609517)*x[1]**o+(0.713211976776)*x[1]+(0.936197279948)*x[2]**o+(-0.293174979732)*x[2]
        s[2,0,0,0]=(0.892309367374)*x[0]**o+(0.633921503439)*x[0]+(0.349595598938)*x[1]**o+(0.461978252492)*x[1]+(-0.711515661832)*x[2]**o+(0.0488550856917)*x[2]
        s[2,0,0,1]=(-0.180492557153)*x[0]**o+(0.56511183613)*x[0]+(-0.665614227144)*x[1]**o+(-0.27485018599)*x[1]+(0.882474739326)*x[2]**o+(0.55427047278)*x[2]
        s[2,0,1,0]=(-0.757743026711)*x[0]**o+(0.940849781783)*x[0]+(-0.269186132914)*x[1]**o+(0.955046584823)*x[1]+(0.378760429217)*x[2]**o+(-0.496548644452)*x[2]
        s[2,0,1,1]=(-0.793137192102)*x[0]**o+(-0.86866411939)*x[0]+(-0.132179169569)*x[1]**o+(0.809599174407)*x[1]+(-0.106002493623)*x[2]**o+(0.109415507767)*x[2]
        s[2,0,2,0]=(0.0272436873719)*x[0]**o+(0.882059697054)*x[0]+(0.195295016692)*x[1]**o+(-0.569734997529)*x[1]+(-0.211564162771)*x[2]**o+(-0.372926144524)*x[2]
        s[2,0,2,1]=(-0.578202088804)*x[0]**o+(-0.466997144939)*x[0]+(0.113612650512)*x[1]**o+(-0.364257850046)*x[1]+(0.260193611289)*x[2]**o+(0.084672789632)*x[2]
        s[2,1,0,0]=(-0.768791448871)*x[0]**o+(-0.0283052260408)*x[0]+(-0.424122141665)*x[1]**o+(-0.568176781664)*x[1]+(0.57776948712)*x[2]**o+(0.974311520873)*x[2]
        s[2,1,0,1]=(-0.868527274026)*x[0]**o+(0.898580042728)*x[0]+(0.293187514185)*x[1]**o+(-0.966501893286)*x[1]+(-0.984906818414)*x[2]**o+(0.385872938659)*x[2]
        s[2,1,1,0]=(0.912855062225)*x[0]**o+(-0.153613990867)*x[0]+(-0.498660070196)*x[1]**o+(0.324379040976)*x[1]+(0.595621093076)*x[2]**o+(0.952723749417)*x[2]
        s[2,1,1,1]=(-0.332918724678)*x[0]**o+(-0.134970034861)*x[0]+(0.0260338270417)*x[1]**o+(0.19534841947)*x[1]+(-0.192624928148)*x[2]**o+(0.351094519153)*x[2]
        s[2,1,2,0]=(-0.130412878953)*x[0]**o+(0.31027939409)*x[0]+(-0.475936742043)*x[1]**o+(-0.304153006832)*x[1]+(-0.293242151796)*x[2]**o+(0.305389900042)*x[2]
        s[2,1,2,1]=(0.701755042286)*x[0]**o+(-0.141745871059)*x[0]+(-0.8205828496)*x[1]**o+(-0.198914354743)*x[1]+(0.785036948065)*x[2]**o+(0.938605909098)*x[2]
        s[2,2,0,0]=(0.551929105975)*x[0]**o+(-0.307038421321)*x[0]+(-0.599015821782)*x[1]**o+(0.0392829428763)*x[1]+(0.000979795616653)*x[2]**o+(-0.674643683481)*x[2]
        s[2,2,0,1]=(0.620978846723)*x[0]**o+(0.440011791598)*x[0]+(0.810951443299)*x[1]**o+(0.99070790399)*x[1]+(0.287374917435)*x[2]**o+(-0.336581470165)*x[2]
        s[2,2,1,0]=(0.772920441549)*x[0]**o+(-0.425704048392)*x[0]+(0.186213730232)*x[1]**o+(-0.69328085038)*x[1]+(-0.496606146226)*x[2]**o+(-0.344844378531)*x[2]
        s[2,2,1,1]=(-0.278910103316)*x[0]**o+(-0.674237998285)*x[0]+(0.83739091668)*x[1]**o+(-0.675256965617)*x[1]+(-0.0220815626466)*x[2]**o+(-0.0600167098977)*x[2]
        s[2,2,2,0]=(0.161601546272)*x[0]**o+(-0.51703043722)*x[0]+(0.192844187688)*x[1]**o+(-0.588393830312)*x[1]+(-0.496441908334)*x[2]**o+(-0.655211307185)*x[2]
        s[2,2,2,1]=(-0.0849641142284)*x[0]**o+(-0.354892880977)*x[0]+(0.990421698189)*x[1]**o+(-0.560596614092)*x[1]+(0.341655795654)*x[2]**o+(0.494914875774)*x[2]
        s[2,3,0,0]=(0.0118611095917)*x[0]**o+(-0.379145547458)*x[0]+(0.443702691226)*x[1]**o+(-0.162411128493)*x[1]+(-0.235962251228)*x[2]**o+(0.647389691482)*x[2]
        s[2,3,0,1]=(0.876617418633)*x[0]**o+(-0.212420835108)*x[0]+(0.931783450145)*x[1]**o+(0.618767260975)*x[1]+(0.865648987862)*x[2]**o+(0.274171020917)*x[2]
        s[2,3,1,0]=(0.0490082163721)*x[0]**o+(-0.719949810351)*x[0]+(0.623252286197)*x[1]**o+(0.797654854976)*x[1]+(-0.688042858141)*x[2]**o+(-0.832375529422)*x[2]
        s[2,3,1,1]=(-0.219830798217)*x[0]**o+(0.404707913477)*x[0]+(0.586989479036)*x[1]**o+(-0.189149824721)*x[1]+(0.439899446736)*x[2]**o+(0.830665884353)*x[2]
        s[2,3,2,0]=(0.176667646979)*x[0]**o+(0.251389955823)*x[0]+(-0.589715137652)*x[1]**o+(0.454775590379)*x[1]+(-0.931971045729)*x[2]**o+(0.688896927587)*x[2]
        s[2,3,2,1]=(0.990217880611)*x[0]**o+(0.813900249046)*x[0]+(0.223918634272)*x[1]**o+(-0.52004580243)*x[1]+(-0.0715278690435)*x[2]**o+(-0.904847118013)*x[2]
        s[2,4,0,0]=(0.64966558184)*x[0]**o+(0.368126307546)*x[0]+(-0.464732694831)*x[1]**o+(0.411720375551)*x[1]+(0.228411933233)*x[2]**o+(0.241400675638)*x[2]
        s[2,4,0,1]=(-0.549913707284)*x[0]**o+(-0.805046099696)*x[0]+(0.0672499044192)*x[1]**o+(-0.443313070041)*x[1]+(0.724365127624)*x[2]**o+(0.796281599759)*x[2]
        s[2,4,1,0]=(-0.849777254443)*x[0]**o+(-0.856417954364)*x[0]+(-0.972865529342)*x[1]**o+(-0.928514101686)*x[1]+(-0.913751924661)*x[2]**o+(-0.988656521899)*x[2]
        s[2,4,1,1]=(-0.911477188787)*x[0]**o+(-0.779986760795)*x[0]+(-0.5389591554)*x[1]**o+(-0.510754176334)*x[1]+(0.897140016867)*x[2]**o+(-0.693629592672)*x[2]
        s[2,4,2,0]=(-0.714385212743)*x[0]**o+(0.335762026267)*x[0]+(-0.402732468944)*x[1]**o+(0.928476377932)*x[1]+(-0.0142181788391)*x[2]**o+(-0.878381714054)*x[2]
        s[2,4,2,1]=(-0.67410120519)*x[0]**o+(-0.958305251297)*x[0]+(-0.380909627066)*x[1]**o+(0.211866670591)*x[1]+(-0.366711818102)*x[2]**o+(0.0984085882283)*x[2]
        s[3,0,0,0]=(0.155086211904)*x[0]**o+(-0.754156385746)*x[0]+(0.0473073793112)*x[1]**o+(0.219578851828)*x[1]+(-0.687387954807)*x[2]**o+(-0.0446886834524)*x[2]
        s[3,0,0,1]=(-0.123628783768)*x[0]**o+(-0.731797394112)*x[0]+(0.603667790885)*x[1]**o+(0.668012714955)*x[1]+(0.992297575688)*x[2]**o+(0.710188099523)*x[2]
        s[3,0,1,0]=(0.142848824593)*x[0]**o+(-0.573105206297)*x[0]+(-0.458611102127)*x[1]**o+(-0.513741139974)*x[1]+(0.184040207705)*x[2]**o+(-0.381655193731)*x[2]
        s[3,0,1,1]=(0.441456662454)*x[0]**o+(0.743195285557)*x[0]+(-0.636813637657)*x[1]**o+(0.579742356597)*x[1]+(-0.100949836318)*x[2]**o+(-0.180042447693)*x[2]
        s[3,0,2,0]=(0.565324561738)*x[0]**o+(-0.734887633464)*x[0]+(-0.70603014663)*x[1]**o+(0.482330325634)*x[1]+(0.0468851802149)*x[2]**o+(0.15022059502)*x[2]
        s[3,0,2,1]=(0.209254635781)*x[0]**o+(0.249109731308)*x[0]+(0.0674776536717)*x[1]**o+(-0.514230929025)*x[1]+(0.209025695623)*x[2]**o+(0.828692252433)*x[2]
        s[3,1,0,0]=(-0.0724637124382)*x[0]**o+(-0.777393915285)*x[0]+(-0.388956221906)*x[1]**o+(0.237031963641)*x[1]+(0.695616762278)*x[2]**o+(-0.990904158717)*x[2]
        s[3,1,0,1]=(0.682394453758)*x[0]**o+(0.971411939071)*x[0]+(0.578523591228)*x[1]**o+(-0.999316997575)*x[1]+(0.967221232651)*x[2]**o+(0.967839631913)*x[2]
        s[3,1,1,0]=(0.790017081833)*x[0]**o+(0.00541365718201)*x[0]+(0.326051127661)*x[1]**o+(-0.0256095948145)*x[1]+(0.723206050948)*x[2]**o+(0.859401412085)*x[2]
        s[3,1,1,1]=(0.632718697794)*x[0]**o+(0.357404127028)*x[0]+(0.74859722663)*x[1]**o+(-0.467286199965)*x[1]+(-0.689766250946)*x[2]**o+(0.143607079021)*x[2]
        s[3,1,2,0]=(0.189165561366)*x[0]**o+(-0.569471633508)*x[0]+(0.827816159248)*x[1]**o+(-0.755414119587)*x[1]+(0.735612525192)*x[2]**o+(0.960576961762)*x[2]
        s[3,1,2,1]=(-0.591990093184)*x[0]**o+(-0.397066100484)*x[0]+(-0.983328257627)*x[1]**o+(0.38188756521)*x[1]+(0.342986157929)*x[2]**o+(-0.654635950498)*x[2]
        s[3,2,0,0]=(-0.539663780822)*x[0]**o+(-0.616950865853)*x[0]+(-0.173236358465)*x[1]**o+(-0.697589613553)*x[1]+(0.881000305923)*x[2]**o+(-0.313131191688)*x[2]
        s[3,2,0,1]=(0.694379614088)*x[0]**o+(0.0482437007544)*x[0]+(-0.752154726269)*x[1]**o+(0.70487516283)*x[1]+(-0.679034313009)*x[2]**o+(-0.759378421017)*x[2]
        s[3,2,1,0]=(0.110285075533)*x[0]**o+(-0.937673509278)*x[0]+(0.588965619655)*x[1]**o+(-0.296106521794)*x[1]+(-0.251861128346)*x[2]**o+(0.0499356188571)*x[2]
        s[3,2,1,1]=(-0.237031027497)*x[0]**o+(-0.555130292547)*x[0]+(0.0266496695782)*x[1]**o+(-0.457831015699)*x[1]+(-0.525100370138)*x[2]**o+(-0.110106658247)*x[2]
        s[3,2,2,0]=(-0.346203985282)*x[0]**o+(0.452969593191)*x[0]+(0.366653289186)*x[1]**o+(-0.431403207447)*x[1]+(0.00416181323958)*x[2]**o+(0.948257819905)*x[2]
        s[3,2,2,1]=(-0.161866645182)*x[0]**o+(-0.223171463142)*x[0]+(0.0335615555403)*x[1]**o+(-0.031701739926)*x[1]+(-0.682660238537)*x[2]**o+(-0.201265341974)*x[2]
        s[3,3,0,0]=(0.600997603354)*x[0]**o+(-0.0794128467746)*x[0]+(0.850816035448)*x[1]**o+(0.987737939576)*x[1]+(0.445092093996)*x[2]**o+(0.209682549773)*x[2]
        s[3,3,0,1]=(-0.281229513599)*x[0]**o+(-0.671047123259)*x[0]+(0.727339757353)*x[1]**o+(0.00116796626742)*x[1]+(0.750903575671)*x[2]**o+(0.688698278099)*x[2]
        s[3,3,1,0]=(0.482788258833)*x[0]**o+(-0.763378332747)*x[0]+(0.958666605693)*x[1]**o+(0.815260251405)*x[1]+(-0.185679132639)*x[2]**o+(-0.978181554387)*x[2]
        s[3,3,1,1]=(0.422342206704)*x[0]**o+(0.92124205703)*x[0]+(0.717119813424)*x[1]**o+(0.358838278821)*x[1]+(-0.905962064034)*x[2]**o+(-0.110063748765)*x[2]
        s[3,3,2,0]=(0.638413147984)*x[0]**o+(0.150065454883)*x[0]+(-0.185782870659)*x[1]**o+(-0.617305598909)*x[1]+(-0.744983914507)*x[2]**o+(-0.557435768794)*x[2]
        s[3,3,2,1]=(-0.373080897783)*x[0]**o+(-0.11883999233)*x[0]+(0.918434263505)*x[1]**o+(0.190498804258)*x[1]+(0.145703720162)*x[2]**o+(0.760937920293)*x[2]
        s[3,4,0,0]=(-0.655458683507)*x[0]**o+(0.181703803374)*x[0]+(0.399180648629)*x[1]**o+(-0.252039523462)*x[1]+(-0.304219626464)*x[2]**o+(0.146502412947)*x[2]
        s[3,4,0,1]=(-0.967498004243)*x[0]**o+(-0.720977530602)*x[0]+(-0.197853229934)*x[1]**o+(-0.206241841259)*x[1]+(-0.842800053665)*x[2]**o+(0.248622158299)*x[2]
        s[3,4,1,0]=(-0.20852177925)*x[0]**o+(0.451565234256)*x[0]+(-0.195228349745)*x[1]**o+(-0.176510706654)*x[1]+(-0.446406234444)*x[2]**o+(-0.391189367221)*x[2]
        s[3,4,1,1]=(-0.0245191385208)*x[0]**o+(-0.4318092516)*x[0]+(0.434565500298)*x[1]**o+(-0.226869925672)*x[1]+(-0.541039601648)*x[2]**o+(-0.661612910117)*x[2]
        s[3,4,2,0]=(0.66840921186)*x[0]**o+(-0.50096681529)*x[0]+(0.584113673029)*x[1]**o+(0.624256279715)*x[1]+(-0.97424822131)*x[2]**o+(-0.436152188081)*x[2]
        s[3,4,2,1]=(-0.524152888193)*x[0]**o+(-0.631030243012)*x[0]+(0.712790078556)*x[1]**o+(0.444287995295)*x[1]+(-0.32562141887)*x[2]**o+(0.999485979189)*x[2]
        ref[0,0,0,0]=(-1.09369433259)/(o+1.)+(-0.728685606995)+(0.356314179645)*0.5**o
        ref[0,0,0,1]=(-1.8372727032)/(o+1.)+(-0.399789276755)+(0.209589534917)*0.5**o
        ref[0,0,1,0]=(-0.208884062074)/(o+1.)+(-0.58416334704)+(-0.242847848445)*0.5**o
        ref[0,0,1,1]=(-0.126744186628)/(o+1.)+(0.273138033997)+(0.255545417045)*0.5**o
        ref[0,0,2,0]=(-0.0369820636107)/(o+1.)+(1.06288699174)+(-0.962070081459)*0.5**o
        ref[0,0,2,1]=(-0.747281704793)/(o+1.)+(-0.36647857395)+(0.636687005195)*0.5**o
        ref[0,1,0,0]=(-0.311459789696)/(o+1.)+(-0.691120478554)+(0.521790025796)*0.5**o
        ref[0,1,0,1]=(-0.576631783947)/(o+1.)+(0.0556154884091)+(-0.642544941536)*0.5**o
        ref[0,1,1,0]=(0.472705110018)/(o+1.)+(-0.511230670877)+(0.987946287729)*0.5**o
        ref[0,1,1,1]=(-0.143423470924)/(o+1.)+(-0.194173739799)+(-0.848091494031)*0.5**o
        ref[0,1,2,0]=(0.351890499412)/(o+1.)+(-0.249197293795)+(-0.411836604088)*0.5**o
        ref[0,1,2,1]=(0.161891485687)/(o+1.)+(0.415707435173)+(0.0397981165279)*0.5**o
        ref[0,2,0,0]=(-0.0187864263063)/(o+1.)+(0.779173414927)+(-0.043373970003)*0.5**o
        ref[0,2,0,1]=(-0.31437392339)/(o+1.)+(-0.48448663227)+(0.204098937116)*0.5**o
        ref[0,2,1,0]=(0.230693657323)/(o+1.)+(1.12469199732)+(0.506878702079)*0.5**o
        ref[0,2,1,1]=(0.595998513433)/(o+1.)+(-0.544883877711)+(-0.261656641804)*0.5**o
        ref[0,2,2,0]=(-0.702386110204)/(o+1.)+(-0.996052366283)+(-0.491629682103)*0.5**o
        ref[0,2,2,1]=(0.232315113828)/(o+1.)+(0.199990990174)+(-0.564089634025)*0.5**o
        ref[0,3,0,0]=(-0.655640217843)/(o+1.)+(0.478355790703)+(0.0606363217868)*0.5**o
        ref[0,3,0,1]=(0.948899359986)/(o+1.)+(0.633579372651)+(-0.702879071079)*0.5**o
        ref[0,3,1,0]=(-0.675992210372)/(o+1.)+(0.114682015642)+(-0.824855043544)*0.5**o
        ref[0,3,1,1]=(-0.184213819336)/(o+1.)+(0.650265245624)+(-0.890319589507)*0.5**o
        ref[0,3,2,0]=(-0.328998548752)/(o+1.)+(0.485049032367)+(0.366123401919)*0.5**o
        ref[0,3,2,1]=(-0.380537903931)/(o+1.)+(-0.467985686882)+(-0.438035010616)*0.5**o
        ref[0,4,0,0]=(-0.59529404833)/(o+1.)+(0.36827259985)+(0.319555220189)*0.5**o
        ref[0,4,0,1]=(0.83138611484)/(o+1.)+(0.78491205408)+(-0.874144853872)*0.5**o
        ref[0,4,1,0]=(-0.894405615027)/(o+1.)+(-0.611911163449)+(0.622984208352)*0.5**o
        ref[0,4,1,1]=(-0.104474373269)/(o+1.)+(-0.682715540615)+(-0.175155841122)*0.5**o
        ref[0,4,2,0]=(-1.04850011326)/(o+1.)+(-0.453402108525)+(0.111634467654)*0.5**o
        ref[0,4,2,1]=(-0.170072041674)/(o+1.)+(0.272923829487)+(0.769210305419)*0.5**o
        ref[1,0,0,0]=(-0.19978027482)/(o+1.)+(0.492517472984)+(-0.907821074278)*0.5**o
        ref[1,0,0,1]=(-0.0898768254184)/(o+1.)+(-0.0202018123679)+(-0.935692950448)*0.5**o
        ref[1,0,1,0]=(0.701068673884)/(o+1.)+(-0.00830568946752)+(0.958399972856)*0.5**o
        ref[1,0,1,1]=(0.46131776206)/(o+1.)+(0.200377959194)+(0.480962582731)*0.5**o
        ref[1,0,2,0]=(-0.349254215553)/(o+1.)+(-0.694786621841)+(-0.459646484527)*0.5**o
        ref[1,0,2,1]=(-0.837447208911)/(o+1.)+(0.0537828613354)+(-0.663523285059)*0.5**o
        ref[1,1,0,0]=(-0.0173783101856)/(o+1.)+(-0.30683593271)+(-0.963081878353)*0.5**o
        ref[1,1,0,1]=(0.100805550828)/(o+1.)+(0.0297058987412)+(-0.426340341727)*0.5**o
        ref[1,1,1,0]=(-0.0315393875231)/(o+1.)+(0.0895320120418)+(0.195081954019)*0.5**o
        ref[1,1,1,1]=(-0.320972870429)/(o+1.)+(-0.320760383469)+(0.174243105183)*0.5**o
        ref[1,1,2,0]=(-0.766821369743)/(o+1.)+(-0.35635731427)+(0.16246409321)*0.5**o
        ref[1,1,2,1]=(-0.169170529222)/(o+1.)+(0.418574866417)+(0.0961690353732)*0.5**o
        ref[1,2,0,0]=(0.32201676349)/(o+1.)+(0.0400359683992)+(-0.641539480622)*0.5**o
        ref[1,2,0,1]=(1.20568129617)/(o+1.)+(-0.0772903631539)+(0.305867854526)*0.5**o
        ref[1,2,1,0]=(1.14889007014)/(o+1.)+(0.26002083676)+(0.0262323517248)*0.5**o
        ref[1,2,1,1]=(1.12854276042)/(o+1.)+(-0.676558260639)+(-0.73494826134)*0.5**o
        ref[1,2,2,0]=(-0.118212170334)/(o+1.)+(-0.522143771996)+(-0.405925175898)*0.5**o
        ref[1,2,2,1]=(-0.013687727043)/(o+1.)+(-0.390782314506)+(-0.552448017406)*0.5**o
        ref[1,3,0,0]=(0.0652238363667)/(o+1.)+(0.381973818864)+(0.498736551815)*0.5**o
        ref[1,3,0,1]=(-1.2449303689)/(o+1.)+(-0.345244781686)+(-0.570322022545)*0.5**o
        ref[1,3,1,0]=(-0.790280287779)/(o+1.)+(0.189806636704)+(-0.978507788576)*0.5**o
        ref[1,3,1,1]=(0.687244152446)/(o+1.)+(-0.458440629387)+(0.413054411011)*0.5**o
        ref[1,3,2,0]=(-0.140715494947)/(o+1.)+(-0.395387817597)+(-0.56020772952)*0.5**o
        ref[1,3,2,1]=(0.217250338139)/(o+1.)+(-0.206772384125)+(-0.784712199154)*0.5**o
        ref[1,4,0,0]=(-0.607709845277)/(o+1.)+(-1.07249144772)+(-0.361383889488)*0.5**o
        ref[1,4,0,1]=(0.430333567491)/(o+1.)+(-0.0482670753081)+(-0.359553116054)*0.5**o
        ref[1,4,1,0]=(-0.395399201609)/(o+1.)+(0.884058173962)+(-0.822256610634)*0.5**o
        ref[1,4,1,1]=(-0.910630458857)/(o+1.)+(-0.26212862634)+(-0.193864272459)*0.5**o
        ref[1,4,2,0]=(-0.270184420683)/(o+1.)+(0.0501386866791)+(0.613697794358)*0.5**o
        ref[1,4,2,1]=(1.87003788947)/(o+1.)+(0.666438758829)+(0.234126882167)*0.5**o
        ref[2,0,0,0]=(-0.361920062895)/(o+1.)+(0.572377420811)+(0.892309367374)*0.5**o
        ref[2,0,0,1]=(0.216860512182)/(o+1.)+(0.42226606146)+(-0.180492557153)*0.5**o
        ref[2,0,1,0]=(0.109574296303)/(o+1.)+(0.699673861077)+(-0.757743026711)*0.5**o
        ref[2,0,1,1]=(-0.238181663192)/(o+1.)+(0.0251752813916)+(-0.793137192102)*0.5**o
        ref[2,0,2,0]=(-0.0162691460785)/(o+1.)+(-0.0303007224997)+(0.0272436873719)*0.5**o
        ref[2,0,2,1]=(0.373806261801)/(o+1.)+(-0.373291102676)+(-0.578202088804)*0.5**o
        ref[2,1,0,0]=(0.153647345455)/(o+1.)+(0.188914756584)+(-0.768791448871)*0.5**o
        ref[2,1,0,1]=(-0.691719304229)/(o+1.)+(0.158975544051)+(-0.868527274026)*0.5**o
        ref[2,1,1,0]=(0.0969610228798)/(o+1.)+(0.561744399763)+(0.912855062225)*0.5**o
        ref[2,1,1,1]=(-0.166591101107)/(o+1.)+(0.205736451881)+(-0.332918724678)*0.5**o
        ref[2,1,2,0]=(-0.769178893839)/(o+1.)+(0.15575814365)+(-0.130412878953)*0.5**o
        ref[2,1,2,1]=(-0.0355459015346)/(o+1.)+(0.298972841648)+(0.701755042286)*0.5**o
        ref[2,2,0,0]=(-0.598036026165)/(o+1.)+(-0.471199580962)+(0.551929105975)*0.5**o
        ref[2,2,0,1]=(1.09832636073)/(o+1.)+(0.547069112712)+(0.620978846723)*0.5**o
        ref[2,2,1,0]=(-0.310392415994)/(o+1.)+(-0.731914638652)+(0.772920441549)*0.5**o
        ref[2,2,1,1]=(0.815309354033)/(o+1.)+(-0.7047558369)+(-0.278910103316)*0.5**o
        ref[2,2,2,0]=(-0.303597720645)/(o+1.)+(-0.880317787358)+(0.161601546272)*0.5**o
        ref[2,2,2,1]=(1.33207749384)/(o+1.)+(-0.210287309648)+(-0.0849641142284)*0.5**o
        ref[2,3,0,0]=(0.207740439998)/(o+1.)+(0.0529165077655)+(0.0118611095917)*0.5**o
        ref[2,3,0,1]=(1.79743243801)/(o+1.)+(0.340258723392)+(0.876617418633)*0.5**o
        ref[2,3,1,0]=(-0.0647905719439)/(o+1.)+(-0.377335242398)+(0.0490082163721)*0.5**o
        ref[2,3,1,1]=(1.02688892577)/(o+1.)+(0.523111986555)+(-0.219830798217)*0.5**o
        ref[2,3,2,0]=(-1.52168618338)/(o+1.)+(0.697531236895)+(0.176667646979)*0.5**o
        ref[2,3,2,1]=(0.152390765229)/(o+1.)+(-0.305496335698)+(0.990217880611)*0.5**o
        ref[2,4,0,0]=(-0.236320761598)/(o+1.)+(0.510623679368)+(0.64966558184)*0.5**o
        ref[2,4,0,1]=(0.791615032044)/(o+1.)+(-0.226038784989)+(-0.549913707284)*0.5**o
        ref[2,4,1,0]=(-1.886617454)/(o+1.)+(-1.38679428897)+(-0.849777254443)*0.5**o
        ref[2,4,1,1]=(0.358180861467)/(o+1.)+(-0.9921852649)+(-0.911477188787)*0.5**o
        ref[2,4,2,0]=(-0.416950647783)/(o+1.)+(0.192928345072)+(-0.714385212743)*0.5**o
        ref[2,4,2,1]=(-0.747621445168)/(o+1.)+(-0.324014996239)+(-0.67410120519)*0.5**o
        ref[3,0,0,0]=(-0.640080575496)/(o+1.)+(-0.289633108685)+(0.155086211904)*0.5**o
        ref[3,0,0,1]=(1.59596536657)/(o+1.)+(0.323201710183)+(-0.123628783768)*0.5**o
        ref[3,0,1,0]=(-0.274570894421)/(o+1.)+(-0.734250770001)+(0.142848824593)*0.5**o
        ref[3,0,1,1]=(-0.737763473975)/(o+1.)+(0.57144759723)+(0.441456662454)*0.5**o
        ref[3,0,2,0]=(-0.659144966415)/(o+1.)+(-0.0511683564048)+(0.565324561738)*0.5**o
        ref[3,0,2,1]=(0.276503349295)/(o+1.)+(0.281785527358)+(0.209254635781)*0.5**o
        ref[3,1,0,0]=(0.306660540372)/(o+1.)+(-0.76563305518)+(-0.0724637124382)*0.5**o
        ref[3,1,0,1]=(1.54574482388)/(o+1.)+(0.469967286704)+(0.682394453758)*0.5**o
        ref[3,1,1,0]=(1.04925717861)/(o+1.)+(0.419602737226)+(0.790017081833)*0.5**o
        ref[3,1,1,1]=(0.0588309756839)/(o+1.)+(0.016862503042)+(0.632718697794)*0.5**o
        ref[3,1,2,0]=(1.56342868444)/(o+1.)+(-0.182154395667)+(0.189165561366)*0.5**o
        ref[3,1,2,1]=(-0.640342099698)/(o+1.)+(-0.334907242886)+(-0.591990093184)*0.5**o
        ref[3,2,0,0]=(0.707763947458)/(o+1.)+(-0.813835835547)+(-0.539663780822)*0.5**o
        ref[3,2,0,1]=(-1.43118903928)/(o+1.)+(-0.00312977871602)+(0.694379614088)*0.5**o
        ref[3,2,1,0]=(0.33710449131)/(o+1.)+(-0.591922206108)+(0.110285075533)*0.5**o
        ref[3,2,1,1]=(-0.49845070056)/(o+1.)+(-0.561533983246)+(-0.237031027497)*0.5**o
        ref[3,2,2,0]=(0.370815102426)/(o+1.)+(0.484912102825)+(-0.346203985282)*0.5**o
        ref[3,2,2,1]=(-0.649098682997)/(o+1.)+(-0.228069272521)+(-0.161866645182)*0.5**o
        ref[3,3,0,0]=(1.29590812944)/(o+1.)+(0.559003821287)+(0.600997603354)*0.5**o
        ref[3,3,0,1]=(1.47824333302)/(o+1.)+(0.00940956055351)+(-0.281229513599)*0.5**o
        ref[3,3,1,0]=(0.772987473054)/(o+1.)+(-0.463149817864)+(0.482788258833)*0.5**o
        ref[3,3,1,1]=(-0.18884225061)/(o+1.)+(0.585008293543)+(0.422342206704)*0.5**o
        ref[3,3,2,0]=(-0.930766785165)/(o+1.)+(-0.51233795641)+(0.638413147984)*0.5**o
        ref[3,3,2,1]=(1.06413798367)/(o+1.)+(0.416298366111)+(-0.373080897783)*0.5**o
        ref[3,4,0,0]=(0.0949610221643)/(o+1.)+(0.0380833464296)+(-0.655458683507)*0.5**o
        ref[3,4,0,1]=(-1.0406532836)/(o+1.)+(-0.339298606781)+(-0.967498004243)*0.5**o
        ref[3,4,1,0]=(-0.641634584189)/(o+1.)+(-0.0580674198095)+(-0.20852177925)*0.5**o
        ref[3,4,1,1]=(-0.10647410135)/(o+1.)+(-0.660146043694)+(-0.0245191385208)*0.5**o
        ref[3,4,2,0]=(-0.390134548281)/(o+1.)+(-0.156431361828)+(0.66840921186)*0.5**o
        ref[3,4,2,1]=(0.387168659686)/(o+1.)+(0.406371865736)+(-0.524152888193)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactOne_fromSymbol_Solution_rank0(self):
      """
      tests integral of rank 0 Symbol on the FunctionOnContactOne

      assumptions: Solution(self.domain) exists
                   self.domain supports integral on FunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      w=Solution(self.domain)
      x=w.getX()
      arg=Symbol(shape=())
      s=Data(0,(),w)
      if dim==2:
        s=(0.353053645706)*x[0]**o+(0.664927675255)*x[0]+(-0.942226663664)*x[1]**o+(0.928445933833)*x[1]
        ref=(-0.942226663664)/(o+1.)+(0.796686804544)+(0.353053645706)*0.5**o
      else:
        s=(-0.488825085875)*x[0]**o+(0.0402522219678)*x[0]+(-0.741198644498)*x[1]**o+(0.683886232761)*x[1]+(0.988205530376)*x[2]**o+(0.448214857867)*x[2]
        ref=(0.247006885878)/(o+1.)+(0.586176656298)+(-0.488825085875)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactOne_fromSymbol_Solution_rank1(self):
      """
      tests integral of rank 1 Symbol on the FunctionOnContactOne

      assumptions: Solution(self.domain) exists
                   self.domain supports integral on FunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      w=Solution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(2,))
      s=Data(0,(2,),w)
      ref=numpy.zeros((2,),numpy.float_)
      if dim==2:
        s[0]=(-0.615767854828)*x[0]**o+(-0.986142825206)*x[0]+(-0.129819716269)*x[1]**o+(0.250395618525)*x[1]
        s[1]=(-0.958601580261)*x[0]**o+(0.991062188074)*x[0]+(-0.644161609496)*x[1]**o+(0.50196388747)*x[1]
        ref[0]=(-0.129819716269)/(o+1.)+(-0.367873603341)+(-0.615767854828)*0.5**o
        ref[1]=(-0.644161609496)/(o+1.)+(0.746513037772)+(-0.958601580261)*0.5**o
      else:
        s[0]=(0.878288543713)*x[0]**o+(-0.0231318363536)*x[0]+(0.800114568351)*x[1]**o+(-0.631838147181)*x[1]+(0.747070597667)*x[2]**o+(-0.73748208859)*x[2]
        s[1]=(-0.393659677321)*x[0]**o+(0.219319690151)*x[0]+(0.579665458601)*x[1]**o+(0.158199277855)*x[1]+(0.347253385647)*x[2]**o+(-0.581467925155)*x[2]
        ref[0]=(1.54718516602)/(o+1.)+(-0.696226036062)+(0.878288543713)*0.5**o
        ref[1]=(0.926918844249)/(o+1.)+(-0.101974478575)+(-0.393659677321)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactOne_fromSymbol_Solution_rank2(self):
      """
      tests integral of rank 2 Symbol on the FunctionOnContactOne

      assumptions: Solution(self.domain) exists
                   self.domain supports integral on FunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      w=Solution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5))
      s=Data(0,(4, 5),w)
      ref=numpy.zeros((4, 5),numpy.float_)
      if dim==2:
        s[0,0]=(0.387453655624)*x[0]**o+(-0.469658136898)*x[0]+(0.250257043285)*x[1]**o+(0.618837167268)*x[1]
        s[0,1]=(-0.698973184774)*x[0]**o+(0.439366278802)*x[0]+(0.365483171998)*x[1]**o+(-0.596967887336)*x[1]
        s[0,2]=(-0.789327020147)*x[0]**o+(-0.720992513079)*x[0]+(0.254736202401)*x[1]**o+(-0.793036130784)*x[1]
        s[0,3]=(0.401178787667)*x[0]**o+(0.272725219284)*x[0]+(0.406915869496)*x[1]**o+(0.650109741413)*x[1]
        s[0,4]=(0.872851752065)*x[0]**o+(0.247512120353)*x[0]+(-0.849447364813)*x[1]**o+(0.341218375884)*x[1]
        s[1,0]=(-0.686440933718)*x[0]**o+(-0.699753221902)*x[0]+(0.416449038712)*x[1]**o+(0.953196364155)*x[1]
        s[1,1]=(-0.445063641949)*x[0]**o+(0.96357651863)*x[0]+(0.928584143634)*x[1]**o+(0.0134608093458)*x[1]
        s[1,2]=(-0.818205311776)*x[0]**o+(0.270136549223)*x[0]+(-0.684209682916)*x[1]**o+(-0.932729815977)*x[1]
        s[1,3]=(-0.0620403511795)*x[0]**o+(0.059231247402)*x[0]+(-0.688733974701)*x[1]**o+(0.581759221954)*x[1]
        s[1,4]=(-0.592213327841)*x[0]**o+(-0.339248053705)*x[0]+(0.384679583558)*x[1]**o+(-0.244452373913)*x[1]
        s[2,0]=(0.602926131051)*x[0]**o+(0.756963449553)*x[0]+(0.764520163983)*x[1]**o+(0.79924075351)*x[1]
        s[2,1]=(0.134902654892)*x[0]**o+(-0.983297874332)*x[0]+(0.919649726121)*x[1]**o+(0.408073376003)*x[1]
        s[2,2]=(0.806897621747)*x[0]**o+(-0.976625920159)*x[0]+(0.478598025599)*x[1]**o+(0.516714148448)*x[1]
        s[2,3]=(0.507729761378)*x[0]**o+(0.701695604786)*x[0]+(0.888872787083)*x[1]**o+(-0.903571907502)*x[1]
        s[2,4]=(0.171628496764)*x[0]**o+(0.54301146383)*x[0]+(0.463317911232)*x[1]**o+(0.998895766351)*x[1]
        s[3,0]=(0.957039213706)*x[0]**o+(-0.740478587736)*x[0]+(-0.915271935195)*x[1]**o+(0.566376957715)*x[1]
        s[3,1]=(0.187513468884)*x[0]**o+(-0.929127411057)*x[0]+(-0.227725906511)*x[1]**o+(-0.807394073167)*x[1]
        s[3,2]=(-0.947087097369)*x[0]**o+(-0.612267326048)*x[0]+(-0.456362542424)*x[1]**o+(-0.353534087643)*x[1]
        s[3,3]=(0.310532071177)*x[0]**o+(-0.221968959306)*x[0]+(-0.349150016728)*x[1]**o+(-0.509141774122)*x[1]
        s[3,4]=(-0.210845769048)*x[0]**o+(0.193660848384)*x[0]+(0.776869628574)*x[1]**o+(-0.154145417271)*x[1]
        ref[0,0]=(0.250257043285)/(o+1.)+(0.074589515185)+(0.387453655624)*0.5**o
        ref[0,1]=(0.365483171998)/(o+1.)+(-0.0788008042673)+(-0.698973184774)*0.5**o
        ref[0,2]=(0.254736202401)/(o+1.)+(-0.757014321931)+(-0.789327020147)*0.5**o
        ref[0,3]=(0.406915869496)/(o+1.)+(0.461417480349)+(0.401178787667)*0.5**o
        ref[0,4]=(-0.849447364813)/(o+1.)+(0.294365248118)+(0.872851752065)*0.5**o
        ref[1,0]=(0.416449038712)/(o+1.)+(0.126721571126)+(-0.686440933718)*0.5**o
        ref[1,1]=(0.928584143634)/(o+1.)+(0.488518663988)+(-0.445063641949)*0.5**o
        ref[1,2]=(-0.684209682916)/(o+1.)+(-0.331296633377)+(-0.818205311776)*0.5**o
        ref[1,3]=(-0.688733974701)/(o+1.)+(0.320495234678)+(-0.0620403511795)*0.5**o
        ref[1,4]=(0.384679583558)/(o+1.)+(-0.291850213809)+(-0.592213327841)*0.5**o
        ref[2,0]=(0.764520163983)/(o+1.)+(0.778102101531)+(0.602926131051)*0.5**o
        ref[2,1]=(0.919649726121)/(o+1.)+(-0.287612249164)+(0.134902654892)*0.5**o
        ref[2,2]=(0.478598025599)/(o+1.)+(-0.229955885856)+(0.806897621747)*0.5**o
        ref[2,3]=(0.888872787083)/(o+1.)+(-0.100938151358)+(0.507729761378)*0.5**o
        ref[2,4]=(0.463317911232)/(o+1.)+(0.77095361509)+(0.171628496764)*0.5**o
        ref[3,0]=(-0.915271935195)/(o+1.)+(-0.0870508150107)+(0.957039213706)*0.5**o
        ref[3,1]=(-0.227725906511)/(o+1.)+(-0.868260742112)+(0.187513468884)*0.5**o
        ref[3,2]=(-0.456362542424)/(o+1.)+(-0.482900706845)+(-0.947087097369)*0.5**o
        ref[3,3]=(-0.349150016728)/(o+1.)+(-0.365555366714)+(0.310532071177)*0.5**o
        ref[3,4]=(0.776869628574)/(o+1.)+(0.0197577155566)+(-0.210845769048)*0.5**o
      else:
        s[0,0]=(-0.0355854650857)*x[0]**o+(0.508097047577)*x[0]+(-0.559809522597)*x[1]**o+(0.274518262337)*x[1]+(-0.346896675687)*x[2]**o+(-0.820202383679)*x[2]
        s[0,1]=(-0.611401860909)*x[0]**o+(-0.804858756979)*x[0]+(0.792251514945)*x[1]**o+(0.127533284836)*x[1]+(0.96691512797)*x[2]**o+(0.0831709389735)*x[2]
        s[0,2]=(-0.0563416061339)*x[0]**o+(0.0591734952104)*x[0]+(0.992243816319)*x[1]**o+(0.457585067795)*x[1]+(0.248903218205)*x[2]**o+(-0.702938956647)*x[2]
        s[0,3]=(0.66016331397)*x[0]**o+(-0.460404888792)*x[0]+(0.834756895572)*x[1]**o+(-0.723598073525)*x[1]+(0.0804592839349)*x[2]**o+(-0.612859372888)*x[2]
        s[0,4]=(0.948196944613)*x[0]**o+(-0.954326701869)*x[0]+(-0.974120082057)*x[1]**o+(0.57658599582)*x[1]+(0.18517252062)*x[2]**o+(0.404027673976)*x[2]
        s[1,0]=(0.096690428758)*x[0]**o+(-0.335528774766)*x[0]+(0.529021814282)*x[1]**o+(0.0944401555902)*x[1]+(-0.0106293914555)*x[2]**o+(-0.885415125356)*x[2]
        s[1,1]=(-0.187544326204)*x[0]**o+(-0.832847850387)*x[0]+(-0.273330813724)*x[1]**o+(-0.0875781530468)*x[1]+(-0.422135351424)*x[2]**o+(-0.44851049492)*x[2]
        s[1,2]=(-0.416290563688)*x[0]**o+(-0.60796418234)*x[0]+(-0.786874489548)*x[1]**o+(-0.83344429526)*x[1]+(-0.636021842684)*x[2]**o+(0.76569386339)*x[2]
        s[1,3]=(-0.888962254312)*x[0]**o+(0.705830842388)*x[0]+(-0.628888150094)*x[1]**o+(0.699724558139)*x[1]+(-0.407095551825)*x[2]**o+(0.796316782037)*x[2]
        s[1,4]=(0.29031515404)*x[0]**o+(0.896505881191)*x[0]+(-0.314937896718)*x[1]**o+(-0.749045267106)*x[1]+(-0.464412869164)*x[2]**o+(-0.357988323816)*x[2]
        s[2,0]=(0.881903169624)*x[0]**o+(-0.962653173961)*x[0]+(0.826332318664)*x[1]**o+(0.625644776113)*x[1]+(0.330804909101)*x[2]**o+(-0.17772544382)*x[2]
        s[2,1]=(-0.503232182675)*x[0]**o+(-0.957203265575)*x[0]+(0.145865955701)*x[1]**o+(0.0915745220394)*x[1]+(-0.470345130645)*x[2]**o+(0.5949845374)*x[2]
        s[2,2]=(0.72138938011)*x[0]**o+(-0.914048390135)*x[0]+(-0.798916451586)*x[1]**o+(-0.720041081497)*x[1]+(-0.292635838745)*x[2]**o+(0.335321951624)*x[2]
        s[2,3]=(-0.182060346819)*x[0]**o+(0.173777377621)*x[0]+(-0.690798771753)*x[1]**o+(-0.792633276338)*x[1]+(0.691527659288)*x[2]**o+(-0.187507542541)*x[2]
        s[2,4]=(-0.5702456536)*x[0]**o+(0.601033902695)*x[0]+(-0.761437618934)*x[1]**o+(-0.258358320383)*x[1]+(-0.718966730705)*x[2]**o+(0.300570256286)*x[2]
        s[3,0]=(0.445936563031)*x[0]**o+(0.859485429121)*x[0]+(-0.191921809371)*x[1]**o+(-0.552489927682)*x[1]+(0.236031566969)*x[2]**o+(0.224011130312)*x[2]
        s[3,1]=(0.383619361689)*x[0]**o+(0.850743712521)*x[0]+(-0.583234052125)*x[1]**o+(-0.957466724321)*x[1]+(0.117664170019)*x[2]**o+(-0.118273519288)*x[2]
        s[3,2]=(0.32727041247)*x[0]**o+(-0.273021170896)*x[0]+(-0.488888392431)*x[1]**o+(0.924810229071)*x[1]+(0.120181256008)*x[2]**o+(0.292893993789)*x[2]
        s[3,3]=(-0.684680337734)*x[0]**o+(-0.494182942675)*x[0]+(-0.0912106871209)*x[1]**o+(-0.483261802779)*x[1]+(0.743468674518)*x[2]**o+(-0.0453199629827)*x[2]
        s[3,4]=(-0.790516141209)*x[0]**o+(0.939271717684)*x[0]+(0.939581799691)*x[1]**o+(0.961123577915)*x[1]+(0.653210734978)*x[2]**o+(0.245098724735)*x[2]
        ref[0,0]=(-0.906706198284)/(o+1.)+(-0.0187935368825)+(-0.0355854650857)*0.5**o
        ref[0,1]=(1.75916664291)/(o+1.)+(-0.297077266585)+(-0.611401860909)*0.5**o
        ref[0,2]=(1.24114703452)/(o+1.)+(-0.0930901968209)+(-0.0563416061339)*0.5**o
        ref[0,3]=(0.915216179507)/(o+1.)+(-0.898431167602)+(0.66016331397)*0.5**o
        ref[0,4]=(-0.788947561438)/(o+1.)+(0.0131434839635)+(0.948196944613)*0.5**o
        ref[1,0]=(0.518392422826)/(o+1.)+(-0.563251872266)+(0.096690428758)*0.5**o
        ref[1,1]=(-0.695466165148)/(o+1.)+(-0.684468249177)+(-0.187544326204)*0.5**o
        ref[1,2]=(-1.42289633223)/(o+1.)+(-0.337857307105)+(-0.416290563688)*0.5**o
        ref[1,3]=(-1.03598370192)/(o+1.)+(1.10093609128)+(-0.888962254312)*0.5**o
        ref[1,4]=(-0.779350765882)/(o+1.)+(-0.105263854865)+(0.29031515404)*0.5**o
        ref[2,0]=(1.15713722776)/(o+1.)+(-0.257366920834)+(0.881903169624)*0.5**o
        ref[2,1]=(-0.324479174944)/(o+1.)+(-0.135322103068)+(-0.503232182675)*0.5**o
        ref[2,2]=(-1.09155229033)/(o+1.)+(-0.649383760004)+(0.72138938011)*0.5**o
        ref[2,3]=(0.000728887534846)/(o+1.)+(-0.403181720629)+(-0.182060346819)*0.5**o
        ref[2,4]=(-1.48040434964)/(o+1.)+(0.321622919299)+(-0.5702456536)*0.5**o
        ref[3,0]=(0.0441097575979)/(o+1.)+(0.265503315876)+(0.445936563031)*0.5**o
        ref[3,1]=(-0.465569882106)/(o+1.)+(-0.112498265544)+(0.383619361689)*0.5**o
        ref[3,2]=(-0.368707136422)/(o+1.)+(0.472341525982)+(0.32727041247)*0.5**o
        ref[3,3]=(0.652257987397)/(o+1.)+(-0.511382354218)+(-0.684680337734)*0.5**o
        ref[3,4]=(1.59279253467)/(o+1.)+(1.07274701017)+(-0.790516141209)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactOne_fromSymbol_Solution_rank3(self):
      """
      tests integral of rank 3 Symbol on the FunctionOnContactOne

      assumptions: Solution(self.domain) exists
                   self.domain supports integral on FunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      w=Solution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(6, 2, 2))
      s=Data(0,(6, 2, 2),w)
      ref=numpy.zeros((6, 2, 2),numpy.float_)
      if dim==2:
        s[0,0,0]=(0.692838611059)*x[0]**o+(0.701451067887)*x[0]+(-0.958627133469)*x[1]**o+(0.309968446224)*x[1]
        s[0,0,1]=(-0.601729371833)*x[0]**o+(-0.00949649239345)*x[0]+(0.178428915942)*x[1]**o+(-0.378187092112)*x[1]
        s[0,1,0]=(-0.911621902906)*x[0]**o+(0.699730287366)*x[0]+(0.849669560241)*x[1]**o+(-0.24390831078)*x[1]
        s[0,1,1]=(-0.772410377566)*x[0]**o+(-0.775571271165)*x[0]+(0.429781604307)*x[1]**o+(-0.258252436014)*x[1]
        s[1,0,0]=(0.573401470974)*x[0]**o+(0.292621928093)*x[0]+(-0.584549055271)*x[1]**o+(-0.660681435117)*x[1]
        s[1,0,1]=(-0.0867987965936)*x[0]**o+(0.195581072977)*x[0]+(-0.259345214698)*x[1]**o+(-0.431225825128)*x[1]
        s[1,1,0]=(0.170068866316)*x[0]**o+(-0.0665083899809)*x[0]+(0.814824812224)*x[1]**o+(0.278218338402)*x[1]
        s[1,1,1]=(-0.424226747289)*x[0]**o+(-0.0605087462286)*x[0]+(0.72215867299)*x[1]**o+(0.636823345898)*x[1]
        s[2,0,0]=(0.948475744468)*x[0]**o+(-0.256081956766)*x[0]+(-0.265551833564)*x[1]**o+(0.219079882965)*x[1]
        s[2,0,1]=(-0.620771197949)*x[0]**o+(0.203458256679)*x[0]+(-0.950417242448)*x[1]**o+(0.0830268150339)*x[1]
        s[2,1,0]=(-0.392672423199)*x[0]**o+(0.0910137955111)*x[0]+(-0.8417031543)*x[1]**o+(-0.456474678014)*x[1]
        s[2,1,1]=(-0.0312010236388)*x[0]**o+(0.307221450065)*x[0]+(-0.479938290822)*x[1]**o+(-0.689609404462)*x[1]
        s[3,0,0]=(0.124108191486)*x[0]**o+(-0.786546027037)*x[0]+(0.0449572653773)*x[1]**o+(-0.887765762037)*x[1]
        s[3,0,1]=(0.327690655208)*x[0]**o+(-0.651767483905)*x[0]+(-0.14328969555)*x[1]**o+(-0.574417616475)*x[1]
        s[3,1,0]=(0.375233560681)*x[0]**o+(-0.4952402604)*x[0]+(0.419976842971)*x[1]**o+(0.573510110207)*x[1]
        s[3,1,1]=(-0.61513881981)*x[0]**o+(-0.42349537427)*x[0]+(-0.513786717031)*x[1]**o+(-0.656407725495)*x[1]
        s[4,0,0]=(-0.212361872891)*x[0]**o+(-0.274380100495)*x[0]+(-0.115417216962)*x[1]**o+(-0.407361424262)*x[1]
        s[4,0,1]=(-0.26215416052)*x[0]**o+(0.560215913083)*x[0]+(0.949340402876)*x[1]**o+(-0.311511101847)*x[1]
        s[4,1,0]=(0.35481412373)*x[0]**o+(0.55002753014)*x[0]+(-0.427957479742)*x[1]**o+(-0.571074070608)*x[1]
        s[4,1,1]=(0.228369065715)*x[0]**o+(0.34597130297)*x[0]+(-0.099176239601)*x[1]**o+(0.0149309525659)*x[1]
        s[5,0,0]=(-0.230638577613)*x[0]**o+(-0.450379175116)*x[0]+(0.150498069811)*x[1]**o+(-0.26480588837)*x[1]
        s[5,0,1]=(-0.563920707349)*x[0]**o+(-0.678734552317)*x[0]+(0.499409088964)*x[1]**o+(0.636349121568)*x[1]
        s[5,1,0]=(-0.654320489103)*x[0]**o+(-0.759308190818)*x[0]+(0.638645835733)*x[1]**o+(-0.932431876846)*x[1]
        s[5,1,1]=(-0.067089857507)*x[0]**o+(-0.828076668167)*x[0]+(0.329157131175)*x[1]**o+(0.191945408984)*x[1]
        ref[0,0,0]=(-0.958627133469)/(o+1.)+(0.505709757056)+(0.692838611059)*0.5**o
        ref[0,0,1]=(0.178428915942)/(o+1.)+(-0.193841792253)+(-0.601729371833)*0.5**o
        ref[0,1,0]=(0.849669560241)/(o+1.)+(0.227910988293)+(-0.911621902906)*0.5**o
        ref[0,1,1]=(0.429781604307)/(o+1.)+(-0.51691185359)+(-0.772410377566)*0.5**o
        ref[1,0,0]=(-0.584549055271)/(o+1.)+(-0.184029753512)+(0.573401470974)*0.5**o
        ref[1,0,1]=(-0.259345214698)/(o+1.)+(-0.117822376075)+(-0.0867987965936)*0.5**o
        ref[1,1,0]=(0.814824812224)/(o+1.)+(0.10585497421)+(0.170068866316)*0.5**o
        ref[1,1,1]=(0.72215867299)/(o+1.)+(0.288157299835)+(-0.424226747289)*0.5**o
        ref[2,0,0]=(-0.265551833564)/(o+1.)+(-0.0185010369005)+(0.948475744468)*0.5**o
        ref[2,0,1]=(-0.950417242448)/(o+1.)+(0.143242535857)+(-0.620771197949)*0.5**o
        ref[2,1,0]=(-0.8417031543)/(o+1.)+(-0.182730441252)+(-0.392672423199)*0.5**o
        ref[2,1,1]=(-0.479938290822)/(o+1.)+(-0.191193977199)+(-0.0312010236388)*0.5**o
        ref[3,0,0]=(0.0449572653773)/(o+1.)+(-0.837155894537)+(0.124108191486)*0.5**o
        ref[3,0,1]=(-0.14328969555)/(o+1.)+(-0.61309255019)+(0.327690655208)*0.5**o
        ref[3,1,0]=(0.419976842971)/(o+1.)+(0.0391349249035)+(0.375233560681)*0.5**o
        ref[3,1,1]=(-0.513786717031)/(o+1.)+(-0.539951549882)+(-0.61513881981)*0.5**o
        ref[4,0,0]=(-0.115417216962)/(o+1.)+(-0.340870762378)+(-0.212361872891)*0.5**o
        ref[4,0,1]=(0.949340402876)/(o+1.)+(0.124352405618)+(-0.26215416052)*0.5**o
        ref[4,1,0]=(-0.427957479742)/(o+1.)+(-0.0105232702342)+(0.35481412373)*0.5**o
        ref[4,1,1]=(-0.099176239601)/(o+1.)+(0.180451127768)+(0.228369065715)*0.5**o
        ref[5,0,0]=(0.150498069811)/(o+1.)+(-0.357592531743)+(-0.230638577613)*0.5**o
        ref[5,0,1]=(0.499409088964)/(o+1.)+(-0.0211927153744)+(-0.563920707349)*0.5**o
        ref[5,1,0]=(0.638645835733)/(o+1.)+(-0.845870033832)+(-0.654320489103)*0.5**o
        ref[5,1,1]=(0.329157131175)/(o+1.)+(-0.318065629591)+(-0.067089857507)*0.5**o
      else:
        s[0,0,0]=(-0.746527687404)*x[0]**o+(-0.596594966324)*x[0]+(0.2810066152)*x[1]**o+(-0.811343974865)*x[1]+(-0.021161977844)*x[2]**o+(-0.955799066641)*x[2]
        s[0,0,1]=(-0.431368611796)*x[0]**o+(0.209275860867)*x[0]+(-0.744753738662)*x[1]**o+(0.427090838531)*x[1]+(-0.125264890706)*x[2]**o+(0.212116081598)*x[2]
        s[0,1,0]=(-0.262032384111)*x[0]**o+(-0.690463175517)*x[0]+(-0.942716909227)*x[1]**o+(-0.512385399849)*x[1]+(0.0838382201449)*x[2]**o+(-0.645258119863)*x[2]
        s[0,1,1]=(0.528688164571)*x[0]**o+(0.436957291986)*x[0]+(-0.537210963816)*x[1]**o+(-0.857589396313)*x[1]+(-0.748363024129)*x[2]**o+(-0.562281821776)*x[2]
        s[1,0,0]=(0.612146020485)*x[0]**o+(0.732593742961)*x[0]+(-0.0856859718674)*x[1]**o+(0.620805675371)*x[1]+(-0.675276046976)*x[2]**o+(0.768362539829)*x[2]
        s[1,0,1]=(0.953317188148)*x[0]**o+(0.501280748029)*x[0]+(0.613804663039)*x[1]**o+(-0.346135069339)*x[1]+(-0.429549388245)*x[2]**o+(-0.460492494281)*x[2]
        s[1,1,0]=(-0.800228112753)*x[0]**o+(-0.849501214281)*x[0]+(-0.397413958567)*x[1]**o+(0.126586313232)*x[1]+(0.117902029118)*x[2]**o+(-0.163407646867)*x[2]
        s[1,1,1]=(0.814833462883)*x[0]**o+(0.888315382151)*x[0]+(-0.729112863291)*x[1]**o+(-0.990948475544)*x[1]+(-0.407293718484)*x[2]**o+(0.0356918466558)*x[2]
        s[2,0,0]=(0.82229169437)*x[0]**o+(0.346827253309)*x[0]+(-0.747273461294)*x[1]**o+(-0.398145482826)*x[1]+(0.889222392384)*x[2]**o+(0.618983213439)*x[2]
        s[2,0,1]=(-0.829164090486)*x[0]**o+(0.101408063492)*x[0]+(-0.323588772419)*x[1]**o+(-0.513436504297)*x[1]+(0.603168070349)*x[2]**o+(0.469577367849)*x[2]
        s[2,1,0]=(-0.174002172236)*x[0]**o+(0.0457253546329)*x[0]+(0.0680387720249)*x[1]**o+(0.108246528319)*x[1]+(-0.378115826261)*x[2]**o+(0.320064337589)*x[2]
        s[2,1,1]=(0.526364662089)*x[0]**o+(0.715789167446)*x[0]+(0.938035509553)*x[1]**o+(0.811137499085)*x[1]+(0.0343661926035)*x[2]**o+(-0.134159658752)*x[2]
        s[3,0,0]=(-0.391083668074)*x[0]**o+(-0.286558934534)*x[0]+(-0.900015970878)*x[1]**o+(-0.44319367626)*x[1]+(0.97392981229)*x[2]**o+(-0.470626411857)*x[2]
        s[3,0,1]=(-0.733432908462)*x[0]**o+(-0.678130107598)*x[0]+(0.150603930952)*x[1]**o+(-0.598239006862)*x[1]+(-0.768485734549)*x[2]**o+(-0.970399443344)*x[2]
        s[3,1,0]=(-0.247608296411)*x[0]**o+(0.153504414444)*x[0]+(-0.990050628042)*x[1]**o+(-0.274052014666)*x[1]+(-0.614045325729)*x[2]**o+(0.742660852789)*x[2]
        s[3,1,1]=(-0.848161397237)*x[0]**o+(0.0389689968506)*x[0]+(-0.746088757211)*x[1]**o+(0.964788844721)*x[1]+(0.0365840578283)*x[2]**o+(0.0937959359018)*x[2]
        s[4,0,0]=(-0.998118542373)*x[0]**o+(-0.622412978573)*x[0]+(-0.23375496789)*x[1]**o+(0.713065960655)*x[1]+(-0.134547861774)*x[2]**o+(-0.65617589708)*x[2]
        s[4,0,1]=(-0.633127771386)*x[0]**o+(-0.621567651537)*x[0]+(-0.54904676147)*x[1]**o+(-0.868056268787)*x[1]+(0.234405832043)*x[2]**o+(0.737621357594)*x[2]
        s[4,1,0]=(0.862411543309)*x[0]**o+(-0.737128008238)*x[0]+(0.307586357348)*x[1]**o+(0.760442280183)*x[1]+(-0.56282765669)*x[2]**o+(-0.0147875945863)*x[2]
        s[4,1,1]=(0.314415379367)*x[0]**o+(0.626003051386)*x[0]+(0.997284366098)*x[1]**o+(-0.199517448031)*x[1]+(0.375147056596)*x[2]**o+(0.732546433867)*x[2]
        s[5,0,0]=(0.819256041498)*x[0]**o+(0.733896748535)*x[0]+(0.0839088873455)*x[1]**o+(0.515382312439)*x[1]+(-0.0194919927864)*x[2]**o+(0.0318260275007)*x[2]
        s[5,0,1]=(0.940274091937)*x[0]**o+(-0.735201721223)*x[0]+(-0.953504926957)*x[1]**o+(-0.945986665743)*x[1]+(-0.0220685469559)*x[2]**o+(0.622957541524)*x[2]
        s[5,1,0]=(-0.93927853219)*x[0]**o+(-0.886451123487)*x[0]+(0.448010704185)*x[1]**o+(-0.9971779094)*x[1]+(0.782692635361)*x[2]**o+(0.690868313501)*x[2]
        s[5,1,1]=(0.101661872917)*x[0]**o+(0.636433790255)*x[0]+(0.526460976779)*x[1]**o+(-0.603459918432)*x[1]+(0.107035082256)*x[2]**o+(0.252479495202)*x[2]
        ref[0,0,0]=(0.259844637356)/(o+1.)+(-1.18186900391)+(-0.746527687404)*0.5**o
        ref[0,0,1]=(-0.870018629367)/(o+1.)+(0.424241390497)+(-0.431368611796)*0.5**o
        ref[0,1,0]=(-0.858878689082)/(o+1.)+(-0.924053347614)+(-0.262032384111)*0.5**o
        ref[0,1,1]=(-1.28557398794)/(o+1.)+(-0.491456963051)+(0.528688164571)*0.5**o
        ref[1,0,0]=(-0.760962018843)/(o+1.)+(1.06088097908)+(0.612146020485)*0.5**o
        ref[1,0,1]=(0.184255274794)/(o+1.)+(-0.152673407796)+(0.953317188148)*0.5**o
        ref[1,1,0]=(-0.279511929449)/(o+1.)+(-0.443161273958)+(-0.800228112753)*0.5**o
        ref[1,1,1]=(-1.13640658178)/(o+1.)+(-0.0334706233686)+(0.814833462883)*0.5**o
        ref[2,0,0]=(0.14194893109)/(o+1.)+(0.283832491961)+(0.82229169437)*0.5**o
        ref[2,0,1]=(0.27957929793)/(o+1.)+(0.0287744635219)+(-0.829164090486)*0.5**o
        ref[2,1,0]=(-0.310077054236)/(o+1.)+(0.23701811027)+(-0.174002172236)*0.5**o
        ref[2,1,1]=(0.972401702157)/(o+1.)+(0.696383503889)+(0.526364662089)*0.5**o
        ref[3,0,0]=(0.0739138414121)/(o+1.)+(-0.600189511325)+(-0.391083668074)*0.5**o
        ref[3,0,1]=(-0.617881803597)/(o+1.)+(-1.1233842789)+(-0.733432908462)*0.5**o
        ref[3,1,0]=(-1.60409595377)/(o+1.)+(0.311056626283)+(-0.247608296411)*0.5**o
        ref[3,1,1]=(-0.709504699383)/(o+1.)+(0.548776888736)+(-0.848161397237)*0.5**o
        ref[4,0,0]=(-0.368302829664)/(o+1.)+(-0.282761457499)+(-0.998118542373)*0.5**o
        ref[4,0,1]=(-0.314640929426)/(o+1.)+(-0.376001281365)+(-0.633127771386)*0.5**o
        ref[4,1,0]=(-0.255241299343)/(o+1.)+(0.00426333867911)+(0.862411543309)*0.5**o
        ref[4,1,1]=(1.37243142269)/(o+1.)+(0.579516018611)+(0.314415379367)*0.5**o
        ref[5,0,0]=(0.0644168945591)/(o+1.)+(0.640552544238)+(0.819256041498)*0.5**o
        ref[5,0,1]=(-0.975573473912)/(o+1.)+(-0.529115422721)+(0.940274091937)*0.5**o
        ref[5,1,0]=(1.23070333955)/(o+1.)+(-0.596380359693)+(-0.93927853219)*0.5**o
        ref[5,1,1]=(0.633496059035)/(o+1.)+(0.142726683513)+(0.101661872917)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactOne_fromSymbol_Solution_rank4(self):
      """
      tests integral of rank 4 Symbol on the FunctionOnContactOne

      assumptions: Solution(self.domain) exists
                   self.domain supports integral on FunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      w=Solution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5, 3, 2))
      s=Data(0,(4, 5, 3, 2),w)
      ref=numpy.zeros((4, 5, 3, 2),numpy.float_)
      if dim==2:
        s[0,0,0,0]=(-0.686834873081)*x[0]**o+(0.894217065974)*x[0]+(0.464807422099)*x[1]**o+(0.117486893484)*x[1]
        s[0,0,0,1]=(0.423267211666)*x[0]**o+(-0.926368250674)*x[0]+(-0.00127389833363)*x[1]**o+(0.644912230378)*x[1]
        s[0,0,1,0]=(0.225761019463)*x[0]**o+(0.941520398103)*x[0]+(0.698674836081)*x[1]**o+(-0.545668525315)*x[1]
        s[0,0,1,1]=(-0.0578569008694)*x[0]**o+(0.886606073204)*x[0]+(-0.943532460794)*x[1]**o+(0.63316605865)*x[1]
        s[0,0,2,0]=(-0.955100804268)*x[0]**o+(-0.197482002011)*x[0]+(0.727953088594)*x[1]**o+(0.842355085503)*x[1]
        s[0,0,2,1]=(0.55641264625)*x[0]**o+(0.26186880746)*x[0]+(0.769897594142)*x[1]**o+(0.630360270402)*x[1]
        s[0,1,0,0]=(0.0417005921153)*x[0]**o+(-0.254303592363)*x[0]+(0.517125320203)*x[1]**o+(0.356198004099)*x[1]
        s[0,1,0,1]=(-0.975918028511)*x[0]**o+(-0.655984560339)*x[0]+(0.00235297753913)*x[1]**o+(-0.111765878548)*x[1]
        s[0,1,1,0]=(-0.16393042329)*x[0]**o+(0.420621371478)*x[0]+(0.367090269805)*x[1]**o+(0.610715442136)*x[1]
        s[0,1,1,1]=(-0.875851967836)*x[0]**o+(-0.816358619616)*x[0]+(0.80277045991)*x[1]**o+(0.717265655871)*x[1]
        s[0,1,2,0]=(0.871764357933)*x[0]**o+(-0.867731309803)*x[0]+(0.410055383976)*x[1]**o+(-0.722483045113)*x[1]
        s[0,1,2,1]=(-0.427119083496)*x[0]**o+(-0.833805789919)*x[0]+(-0.275401454293)*x[1]**o+(0.442220067398)*x[1]
        s[0,2,0,0]=(0.462551904284)*x[0]**o+(-0.0535451795861)*x[0]+(0.270504950569)*x[1]**o+(0.834360806219)*x[1]
        s[0,2,0,1]=(-0.0507590824972)*x[0]**o+(0.26574820566)*x[0]+(0.610544260364)*x[1]**o+(-0.890035786158)*x[1]
        s[0,2,1,0]=(-0.638445791035)*x[0]**o+(-0.305351157409)*x[0]+(0.965223780441)*x[1]**o+(0.982229276343)*x[1]
        s[0,2,1,1]=(-0.731025223911)*x[0]**o+(0.307228768356)*x[0]+(0.645857556893)*x[1]**o+(0.913645113546)*x[1]
        s[0,2,2,0]=(-0.196899050904)*x[0]**o+(0.150588444027)*x[0]+(0.603646143067)*x[1]**o+(0.811245615941)*x[1]
        s[0,2,2,1]=(0.546119342726)*x[0]**o+(0.204141566001)*x[0]+(0.114502748703)*x[1]**o+(-0.698961842363)*x[1]
        s[0,3,0,0]=(-0.0426650685874)*x[0]**o+(-0.335440677298)*x[0]+(-0.174118589494)*x[1]**o+(-0.851843219794)*x[1]
        s[0,3,0,1]=(-0.14536830456)*x[0]**o+(0.505179058635)*x[0]+(-0.942847104809)*x[1]**o+(0.518276559236)*x[1]
        s[0,3,1,0]=(-0.737548624771)*x[0]**o+(0.70571598977)*x[0]+(-0.746943114818)*x[1]**o+(-0.2127583186)*x[1]
        s[0,3,1,1]=(0.360491166663)*x[0]**o+(0.986095817206)*x[0]+(-0.918270622952)*x[1]**o+(0.617678170994)*x[1]
        s[0,3,2,0]=(-0.570338406694)*x[0]**o+(-0.513173501835)*x[0]+(0.572925790394)*x[1]**o+(0.130209822282)*x[1]
        s[0,3,2,1]=(0.112118675411)*x[0]**o+(0.561084973521)*x[0]+(0.598978537086)*x[1]**o+(-0.0313515131555)*x[1]
        s[0,4,0,0]=(-0.971043700474)*x[0]**o+(0.841925646149)*x[0]+(0.858700520274)*x[1]**o+(-0.304258365028)*x[1]
        s[0,4,0,1]=(-0.33210808558)*x[0]**o+(-0.453977822683)*x[0]+(-0.74202650883)*x[1]**o+(0.546471602539)*x[1]
        s[0,4,1,0]=(-0.622155482312)*x[0]**o+(-0.0779995859909)*x[0]+(0.221889403634)*x[1]**o+(0.342594640865)*x[1]
        s[0,4,1,1]=(-0.83327572159)*x[0]**o+(-0.736091414594)*x[0]+(-0.907324696118)*x[1]**o+(0.543711697923)*x[1]
        s[0,4,2,0]=(0.97730521508)*x[0]**o+(0.680289384194)*x[0]+(0.749574300046)*x[1]**o+(-0.279718517953)*x[1]
        s[0,4,2,1]=(-0.188385359119)*x[0]**o+(-0.517585952202)*x[0]+(-0.92321177387)*x[1]**o+(0.794132586536)*x[1]
        s[1,0,0,0]=(-0.691363379735)*x[0]**o+(-0.338291851496)*x[0]+(0.708824882363)*x[1]**o+(-0.154240182249)*x[1]
        s[1,0,0,1]=(0.391349048346)*x[0]**o+(0.10762051179)*x[0]+(0.971913362543)*x[1]**o+(-0.0532019503958)*x[1]
        s[1,0,1,0]=(-0.669458166029)*x[0]**o+(0.353354538692)*x[0]+(-0.441190419207)*x[1]**o+(-0.528430239493)*x[1]
        s[1,0,1,1]=(0.107848426266)*x[0]**o+(-0.076965091351)*x[0]+(0.745243283501)*x[1]**o+(-0.782833446988)*x[1]
        s[1,0,2,0]=(0.364065471931)*x[0]**o+(0.54771360956)*x[0]+(0.565373904163)*x[1]**o+(0.616558822792)*x[1]
        s[1,0,2,1]=(-0.379996842423)*x[0]**o+(0.392499507062)*x[0]+(-0.808821894825)*x[1]**o+(-0.615251700689)*x[1]
        s[1,1,0,0]=(-0.245969602554)*x[0]**o+(0.436762196801)*x[0]+(0.860717658702)*x[1]**o+(0.132856923997)*x[1]
        s[1,1,0,1]=(-0.461963226134)*x[0]**o+(0.620555910148)*x[0]+(-0.119798710079)*x[1]**o+(0.0221208440902)*x[1]
        s[1,1,1,0]=(-0.357540385682)*x[0]**o+(0.35759692542)*x[0]+(-0.542207154591)*x[1]**o+(-0.334151697958)*x[1]
        s[1,1,1,1]=(-0.507609494267)*x[0]**o+(0.298724176211)*x[0]+(-0.689016521358)*x[1]**o+(-0.574923042649)*x[1]
        s[1,1,2,0]=(-0.314205160643)*x[0]**o+(-0.855793612952)*x[0]+(0.245833632085)*x[1]**o+(0.65947369599)*x[1]
        s[1,1,2,1]=(0.92975337751)*x[0]**o+(0.981531336944)*x[0]+(0.889631731534)*x[1]**o+(0.96084291721)*x[1]
        s[1,2,0,0]=(-0.206928230723)*x[0]**o+(-0.0227833929279)*x[0]+(0.910908314307)*x[1]**o+(-0.293206125585)*x[1]
        s[1,2,0,1]=(0.655611121589)*x[0]**o+(-0.363559028874)*x[0]+(0.944323994431)*x[1]**o+(0.675785829867)*x[1]
        s[1,2,1,0]=(0.097163050774)*x[0]**o+(-0.306871139813)*x[0]+(-0.583236391219)*x[1]**o+(0.2082878001)*x[1]
        s[1,2,1,1]=(0.966536588817)*x[0]**o+(0.366298955916)*x[0]+(0.637843032912)*x[1]**o+(0.808769955133)*x[1]
        s[1,2,2,0]=(-0.688421104726)*x[0]**o+(-0.208613716503)*x[0]+(-0.264530414111)*x[1]**o+(-0.869467631827)*x[1]
        s[1,2,2,1]=(0.572370662453)*x[0]**o+(0.0176965600004)*x[0]+(-0.252518792378)*x[1]**o+(-0.696779486832)*x[1]
        s[1,3,0,0]=(-0.23128498)*x[0]**o+(0.0318637329932)*x[0]+(0.477807829207)*x[1]**o+(-0.512393411225)*x[1]
        s[1,3,0,1]=(-0.121822943652)*x[0]**o+(0.264705755391)*x[0]+(-0.477007018414)*x[1]**o+(0.991097887584)*x[1]
        s[1,3,1,0]=(-0.338674627679)*x[0]**o+(0.779182013477)*x[0]+(-0.0510727374236)*x[1]**o+(0.613462237402)*x[1]
        s[1,3,1,1]=(-0.842082566923)*x[0]**o+(0.659487619964)*x[0]+(-0.0616578458759)*x[1]**o+(0.691256407339)*x[1]
        s[1,3,2,0]=(0.462359358829)*x[0]**o+(-0.782666568422)*x[0]+(-0.00922728778114)*x[1]**o+(0.935329586498)*x[1]
        s[1,3,2,1]=(0.0965038514703)*x[0]**o+(0.937329942152)*x[0]+(-0.0289919380082)*x[1]**o+(0.453957829701)*x[1]
        s[1,4,0,0]=(-0.577911003192)*x[0]**o+(0.74727013652)*x[0]+(-0.970205360146)*x[1]**o+(-0.525871518563)*x[1]
        s[1,4,0,1]=(0.0798727685012)*x[0]**o+(0.181295188234)*x[0]+(0.321952537611)*x[1]**o+(-0.702604584966)*x[1]
        s[1,4,1,0]=(-0.132278278329)*x[0]**o+(0.975101058477)*x[0]+(0.236705098567)*x[1]**o+(-0.10937616935)*x[1]
        s[1,4,1,1]=(-0.361402070774)*x[0]**o+(-0.370364725782)*x[0]+(-0.610674940874)*x[1]**o+(-0.0962382083614)*x[1]
        s[1,4,2,0]=(0.813396048857)*x[0]**o+(0.292472923811)*x[0]+(-0.901067471391)*x[1]**o+(0.547614067474)*x[1]
        s[1,4,2,1]=(0.390285391184)*x[0]**o+(-0.385915061071)*x[0]+(-0.0402809003623)*x[1]**o+(-0.988197507151)*x[1]
        s[2,0,0,0]=(-0.249708353038)*x[0]**o+(-0.906354186065)*x[0]+(0.962323825393)*x[1]**o+(0.0966005473089)*x[1]
        s[2,0,0,1]=(0.815655802435)*x[0]**o+(0.59037691594)*x[0]+(-0.913553917647)*x[1]**o+(0.195455032055)*x[1]
        s[2,0,1,0]=(-0.041263589419)*x[0]**o+(-0.722404855046)*x[0]+(0.792396772777)*x[1]**o+(0.980090324531)*x[1]
        s[2,0,1,1]=(0.336059537191)*x[0]**o+(-0.953661694087)*x[0]+(-0.377727072925)*x[1]**o+(-0.00288817417099)*x[1]
        s[2,0,2,0]=(0.0637701377122)*x[0]**o+(0.185264659888)*x[0]+(-0.63229573614)*x[1]**o+(-0.743036413091)*x[1]
        s[2,0,2,1]=(-0.187859444818)*x[0]**o+(-0.513109580827)*x[0]+(-0.649825178883)*x[1]**o+(0.883502750572)*x[1]
        s[2,1,0,0]=(0.87708393158)*x[0]**o+(-0.868702717082)*x[0]+(-0.727725348725)*x[1]**o+(-0.977109996353)*x[1]
        s[2,1,0,1]=(-0.192407804254)*x[0]**o+(0.240131364352)*x[0]+(-0.907681129252)*x[1]**o+(0.558779298439)*x[1]
        s[2,1,1,0]=(0.153001662465)*x[0]**o+(0.795683040566)*x[0]+(0.0438148749584)*x[1]**o+(-0.0621663419181)*x[1]
        s[2,1,1,1]=(0.591374562403)*x[0]**o+(0.0792161473602)*x[0]+(0.692300008749)*x[1]**o+(0.497404274834)*x[1]
        s[2,1,2,0]=(-0.742751457969)*x[0]**o+(0.963865160953)*x[0]+(-0.217320783858)*x[1]**o+(-0.532084042794)*x[1]
        s[2,1,2,1]=(-0.834015860027)*x[0]**o+(0.154439983509)*x[0]+(0.497803852733)*x[1]**o+(0.205893944848)*x[1]
        s[2,2,0,0]=(-0.203653128753)*x[0]**o+(0.240132473537)*x[0]+(-0.655435581477)*x[1]**o+(-0.858878900716)*x[1]
        s[2,2,0,1]=(0.971807071325)*x[0]**o+(-0.373206373801)*x[0]+(0.140960526316)*x[1]**o+(-0.726655373182)*x[1]
        s[2,2,1,0]=(0.36337495614)*x[0]**o+(0.024734800399)*x[0]+(-0.87272068545)*x[1]**o+(-0.342758158473)*x[1]
        s[2,2,1,1]=(0.703205712066)*x[0]**o+(-0.701536561698)*x[0]+(0.849847759033)*x[1]**o+(0.436346580068)*x[1]
        s[2,2,2,0]=(0.814179476293)*x[0]**o+(-0.722566145563)*x[0]+(0.730674735708)*x[1]**o+(0.433386430731)*x[1]
        s[2,2,2,1]=(0.465577907694)*x[0]**o+(-0.0864038816161)*x[0]+(0.927807005045)*x[1]**o+(0.56469159985)*x[1]
        s[2,3,0,0]=(-0.0809501575869)*x[0]**o+(-0.618484411939)*x[0]+(0.737902566912)*x[1]**o+(0.599580560022)*x[1]
        s[2,3,0,1]=(-0.974260986562)*x[0]**o+(0.403417818779)*x[0]+(0.688538873306)*x[1]**o+(0.12462953818)*x[1]
        s[2,3,1,0]=(0.256567297589)*x[0]**o+(0.996662217474)*x[0]+(-0.892476705335)*x[1]**o+(-0.586859542133)*x[1]
        s[2,3,1,1]=(0.600281177478)*x[0]**o+(-0.884090855927)*x[0]+(0.724531635238)*x[1]**o+(-0.16428377776)*x[1]
        s[2,3,2,0]=(-0.316555549763)*x[0]**o+(0.654066898461)*x[0]+(-0.636613154487)*x[1]**o+(-0.0527299937889)*x[1]
        s[2,3,2,1]=(0.509096161125)*x[0]**o+(0.553056557071)*x[0]+(0.241657689068)*x[1]**o+(-0.988729739553)*x[1]
        s[2,4,0,0]=(-0.00190589099569)*x[0]**o+(-0.838680435068)*x[0]+(-0.531592666371)*x[1]**o+(0.26101901932)*x[1]
        s[2,4,0,1]=(0.727384767513)*x[0]**o+(-0.584151796751)*x[0]+(-0.32109862114)*x[1]**o+(-0.440158856065)*x[1]
        s[2,4,1,0]=(-0.190511998024)*x[0]**o+(0.695377622072)*x[0]+(0.815200064372)*x[1]**o+(-0.91046353217)*x[1]
        s[2,4,1,1]=(0.505564114308)*x[0]**o+(-0.139255460306)*x[0]+(0.323832915733)*x[1]**o+(-0.307972664581)*x[1]
        s[2,4,2,0]=(-0.199017289187)*x[0]**o+(-0.748501229072)*x[0]+(-0.378591546965)*x[1]**o+(0.862316455474)*x[1]
        s[2,4,2,1]=(-0.166203292304)*x[0]**o+(-0.0809782096656)*x[0]+(0.0866258209859)*x[1]**o+(0.443578824637)*x[1]
        s[3,0,0,0]=(0.264566793834)*x[0]**o+(0.811278578401)*x[0]+(0.42099538494)*x[1]**o+(0.199600034959)*x[1]
        s[3,0,0,1]=(0.235746402593)*x[0]**o+(-0.921869283354)*x[0]+(-0.566125315231)*x[1]**o+(0.0537623665249)*x[1]
        s[3,0,1,0]=(0.862345276865)*x[0]**o+(0.227476031253)*x[0]+(-0.448144780994)*x[1]**o+(-0.861358696525)*x[1]
        s[3,0,1,1]=(-0.896403179984)*x[0]**o+(0.431197708747)*x[0]+(-0.652497974262)*x[1]**o+(0.104540024804)*x[1]
        s[3,0,2,0]=(0.788731583728)*x[0]**o+(0.570545432128)*x[0]+(-0.455838556745)*x[1]**o+(0.629398526061)*x[1]
        s[3,0,2,1]=(-0.727929054337)*x[0]**o+(0.561826310816)*x[0]+(0.158791540215)*x[1]**o+(0.458507235686)*x[1]
        s[3,1,0,0]=(0.349297794591)*x[0]**o+(-0.179009101281)*x[0]+(-0.0535819231755)*x[1]**o+(-0.153059139984)*x[1]
        s[3,1,0,1]=(-0.759321371341)*x[0]**o+(-0.464845449228)*x[0]+(-0.800973035912)*x[1]**o+(0.905300621325)*x[1]
        s[3,1,1,0]=(-0.597928946862)*x[0]**o+(-0.621215873514)*x[0]+(0.521881881398)*x[1]**o+(0.570001614348)*x[1]
        s[3,1,1,1]=(-0.263124019379)*x[0]**o+(-0.970277462371)*x[0]+(0.445221632359)*x[1]**o+(-0.610952489733)*x[1]
        s[3,1,2,0]=(0.434074902924)*x[0]**o+(-0.34516630325)*x[0]+(0.812893197377)*x[1]**o+(-0.755651456445)*x[1]
        s[3,1,2,1]=(-0.724153308139)*x[0]**o+(-0.111288572288)*x[0]+(-0.40676151673)*x[1]**o+(0.547624797328)*x[1]
        s[3,2,0,0]=(0.915215698154)*x[0]**o+(-0.770595818702)*x[0]+(-0.88668061529)*x[1]**o+(0.37772776152)*x[1]
        s[3,2,0,1]=(-0.456607413762)*x[0]**o+(-0.972000077172)*x[0]+(0.453597269106)*x[1]**o+(-0.535557277591)*x[1]
        s[3,2,1,0]=(-0.428848816409)*x[0]**o+(-0.957711101461)*x[0]+(0.0771381998527)*x[1]**o+(0.578640765452)*x[1]
        s[3,2,1,1]=(-0.488115399257)*x[0]**o+(0.966283881858)*x[0]+(0.786396903927)*x[1]**o+(-0.427389628779)*x[1]
        s[3,2,2,0]=(-0.748108272191)*x[0]**o+(-0.411931818676)*x[0]+(-0.104450005208)*x[1]**o+(-0.548784991612)*x[1]
        s[3,2,2,1]=(-0.288896584199)*x[0]**o+(-0.599403262911)*x[0]+(-0.107194826319)*x[1]**o+(-0.714445846353)*x[1]
        s[3,3,0,0]=(-0.643859183999)*x[0]**o+(0.0290410872689)*x[0]+(0.184456668026)*x[1]**o+(0.62348637013)*x[1]
        s[3,3,0,1]=(-0.258292072759)*x[0]**o+(0.337782506924)*x[0]+(-0.653836797208)*x[1]**o+(-0.373955061219)*x[1]
        s[3,3,1,0]=(-0.659219139805)*x[0]**o+(0.39140958876)*x[0]+(0.684973413357)*x[1]**o+(-0.0642329755552)*x[1]
        s[3,3,1,1]=(-0.649682331365)*x[0]**o+(-0.00281746580006)*x[0]+(0.967988326693)*x[1]**o+(0.895373576808)*x[1]
        s[3,3,2,0]=(-0.0299199232233)*x[0]**o+(0.458695499665)*x[0]+(0.911862994231)*x[1]**o+(-0.748787338345)*x[1]
        s[3,3,2,1]=(0.197795840564)*x[0]**o+(-0.765596541276)*x[0]+(0.121310526207)*x[1]**o+(-0.300366665995)*x[1]
        s[3,4,0,0]=(0.278568700563)*x[0]**o+(-0.81691908238)*x[0]+(0.144475855023)*x[1]**o+(0.104006513674)*x[1]
        s[3,4,0,1]=(-0.608132003655)*x[0]**o+(0.847594429708)*x[0]+(-0.846906570194)*x[1]**o+(0.350085739363)*x[1]
        s[3,4,1,0]=(-0.490163043631)*x[0]**o+(0.929230880519)*x[0]+(-0.599983340725)*x[1]**o+(-0.252289176277)*x[1]
        s[3,4,1,1]=(-0.802228749858)*x[0]**o+(-0.835578269096)*x[0]+(-0.466727935821)*x[1]**o+(0.208416931397)*x[1]
        s[3,4,2,0]=(0.963914397515)*x[0]**o+(0.552034808675)*x[0]+(-0.0529537502248)*x[1]**o+(0.56193543593)*x[1]
        s[3,4,2,1]=(-0.201051432425)*x[0]**o+(0.720512883794)*x[0]+(0.942559524116)*x[1]**o+(-0.660858122132)*x[1]
        ref[0,0,0,0]=(0.464807422099)/(o+1.)+(0.505851979729)+(-0.686834873081)*0.5**o
        ref[0,0,0,1]=(-0.00127389833363)/(o+1.)+(-0.140728010148)+(0.423267211666)*0.5**o
        ref[0,0,1,0]=(0.698674836081)/(o+1.)+(0.197925936394)+(0.225761019463)*0.5**o
        ref[0,0,1,1]=(-0.943532460794)/(o+1.)+(0.759886065927)+(-0.0578569008694)*0.5**o
        ref[0,0,2,0]=(0.727953088594)/(o+1.)+(0.322436541746)+(-0.955100804268)*0.5**o
        ref[0,0,2,1]=(0.769897594142)/(o+1.)+(0.446114538931)+(0.55641264625)*0.5**o
        ref[0,1,0,0]=(0.517125320203)/(o+1.)+(0.0509472058681)+(0.0417005921153)*0.5**o
        ref[0,1,0,1]=(0.00235297753913)/(o+1.)+(-0.383875219444)+(-0.975918028511)*0.5**o
        ref[0,1,1,0]=(0.367090269805)/(o+1.)+(0.515668406807)+(-0.16393042329)*0.5**o
        ref[0,1,1,1]=(0.80277045991)/(o+1.)+(-0.0495464818722)+(-0.875851967836)*0.5**o
        ref[0,1,2,0]=(0.410055383976)/(o+1.)+(-0.795107177458)+(0.871764357933)*0.5**o
        ref[0,1,2,1]=(-0.275401454293)/(o+1.)+(-0.195792861261)+(-0.427119083496)*0.5**o
        ref[0,2,0,0]=(0.270504950569)/(o+1.)+(0.390407813316)+(0.462551904284)*0.5**o
        ref[0,2,0,1]=(0.610544260364)/(o+1.)+(-0.312143790249)+(-0.0507590824972)*0.5**o
        ref[0,2,1,0]=(0.965223780441)/(o+1.)+(0.338439059467)+(-0.638445791035)*0.5**o
        ref[0,2,1,1]=(0.645857556893)/(o+1.)+(0.610436940951)+(-0.731025223911)*0.5**o
        ref[0,2,2,0]=(0.603646143067)/(o+1.)+(0.480917029984)+(-0.196899050904)*0.5**o
        ref[0,2,2,1]=(0.114502748703)/(o+1.)+(-0.247410138181)+(0.546119342726)*0.5**o
        ref[0,3,0,0]=(-0.174118589494)/(o+1.)+(-0.593641948546)+(-0.0426650685874)*0.5**o
        ref[0,3,0,1]=(-0.942847104809)/(o+1.)+(0.511727808935)+(-0.14536830456)*0.5**o
        ref[0,3,1,0]=(-0.746943114818)/(o+1.)+(0.246478835585)+(-0.737548624771)*0.5**o
        ref[0,3,1,1]=(-0.918270622952)/(o+1.)+(0.8018869941)+(0.360491166663)*0.5**o
        ref[0,3,2,0]=(0.572925790394)/(o+1.)+(-0.191481839776)+(-0.570338406694)*0.5**o
        ref[0,3,2,1]=(0.598978537086)/(o+1.)+(0.264866730183)+(0.112118675411)*0.5**o
        ref[0,4,0,0]=(0.858700520274)/(o+1.)+(0.26883364056)+(-0.971043700474)*0.5**o
        ref[0,4,0,1]=(-0.74202650883)/(o+1.)+(0.0462468899283)+(-0.33210808558)*0.5**o
        ref[0,4,1,0]=(0.221889403634)/(o+1.)+(0.132297527437)+(-0.622155482312)*0.5**o
        ref[0,4,1,1]=(-0.907324696118)/(o+1.)+(-0.0961898583351)+(-0.83327572159)*0.5**o
        ref[0,4,2,0]=(0.749574300046)/(o+1.)+(0.200285433121)+(0.97730521508)*0.5**o
        ref[0,4,2,1]=(-0.92321177387)/(o+1.)+(0.138273317167)+(-0.188385359119)*0.5**o
        ref[1,0,0,0]=(0.708824882363)/(o+1.)+(-0.246266016873)+(-0.691363379735)*0.5**o
        ref[1,0,0,1]=(0.971913362543)/(o+1.)+(0.0272092806971)+(0.391349048346)*0.5**o
        ref[1,0,1,0]=(-0.441190419207)/(o+1.)+(-0.0875378504002)+(-0.669458166029)*0.5**o
        ref[1,0,1,1]=(0.745243283501)/(o+1.)+(-0.42989926917)+(0.107848426266)*0.5**o
        ref[1,0,2,0]=(0.565373904163)/(o+1.)+(0.582136216176)+(0.364065471931)*0.5**o
        ref[1,0,2,1]=(-0.808821894825)/(o+1.)+(-0.111376096814)+(-0.379996842423)*0.5**o
        ref[1,1,0,0]=(0.860717658702)/(o+1.)+(0.284809560399)+(-0.245969602554)*0.5**o
        ref[1,1,0,1]=(-0.119798710079)/(o+1.)+(0.321338377119)+(-0.461963226134)*0.5**o
        ref[1,1,1,0]=(-0.542207154591)/(o+1.)+(0.0117226137313)+(-0.357540385682)*0.5**o
        ref[1,1,1,1]=(-0.689016521358)/(o+1.)+(-0.138099433219)+(-0.507609494267)*0.5**o
        ref[1,1,2,0]=(0.245833632085)/(o+1.)+(-0.0981599584809)+(-0.314205160643)*0.5**o
        ref[1,1,2,1]=(0.889631731534)/(o+1.)+(0.971187127077)+(0.92975337751)*0.5**o
        ref[1,2,0,0]=(0.910908314307)/(o+1.)+(-0.157994759256)+(-0.206928230723)*0.5**o
        ref[1,2,0,1]=(0.944323994431)/(o+1.)+(0.156113400497)+(0.655611121589)*0.5**o
        ref[1,2,1,0]=(-0.583236391219)/(o+1.)+(-0.0492916698561)+(0.097163050774)*0.5**o
        ref[1,2,1,1]=(0.637843032912)/(o+1.)+(0.587534455525)+(0.966536588817)*0.5**o
        ref[1,2,2,0]=(-0.264530414111)/(o+1.)+(-0.539040674165)+(-0.688421104726)*0.5**o
        ref[1,2,2,1]=(-0.252518792378)/(o+1.)+(-0.339541463416)+(0.572370662453)*0.5**o
        ref[1,3,0,0]=(0.477807829207)/(o+1.)+(-0.240264839116)+(-0.23128498)*0.5**o
        ref[1,3,0,1]=(-0.477007018414)/(o+1.)+(0.627901821488)+(-0.121822943652)*0.5**o
        ref[1,3,1,0]=(-0.0510727374236)/(o+1.)+(0.69632212544)+(-0.338674627679)*0.5**o
        ref[1,3,1,1]=(-0.0616578458759)/(o+1.)+(0.675372013651)+(-0.842082566923)*0.5**o
        ref[1,3,2,0]=(-0.00922728778114)/(o+1.)+(0.0763315090377)+(0.462359358829)*0.5**o
        ref[1,3,2,1]=(-0.0289919380082)/(o+1.)+(0.695643885927)+(0.0965038514703)*0.5**o
        ref[1,4,0,0]=(-0.970205360146)/(o+1.)+(0.110699308979)+(-0.577911003192)*0.5**o
        ref[1,4,0,1]=(0.321952537611)/(o+1.)+(-0.260654698366)+(0.0798727685012)*0.5**o
        ref[1,4,1,0]=(0.236705098567)/(o+1.)+(0.432862444563)+(-0.132278278329)*0.5**o
        ref[1,4,1,1]=(-0.610674940874)/(o+1.)+(-0.233301467071)+(-0.361402070774)*0.5**o
        ref[1,4,2,0]=(-0.901067471391)/(o+1.)+(0.420043495642)+(0.813396048857)*0.5**o
        ref[1,4,2,1]=(-0.0402809003623)/(o+1.)+(-0.687056284111)+(0.390285391184)*0.5**o
        ref[2,0,0,0]=(0.962323825393)/(o+1.)+(-0.404876819378)+(-0.249708353038)*0.5**o
        ref[2,0,0,1]=(-0.913553917647)/(o+1.)+(0.392915973998)+(0.815655802435)*0.5**o
        ref[2,0,1,0]=(0.792396772777)/(o+1.)+(0.128842734742)+(-0.041263589419)*0.5**o
        ref[2,0,1,1]=(-0.377727072925)/(o+1.)+(-0.478274934129)+(0.336059537191)*0.5**o
        ref[2,0,2,0]=(-0.63229573614)/(o+1.)+(-0.278885876602)+(0.0637701377122)*0.5**o
        ref[2,0,2,1]=(-0.649825178883)/(o+1.)+(0.185196584873)+(-0.187859444818)*0.5**o
        ref[2,1,0,0]=(-0.727725348725)/(o+1.)+(-0.922906356717)+(0.87708393158)*0.5**o
        ref[2,1,0,1]=(-0.907681129252)/(o+1.)+(0.399455331395)+(-0.192407804254)*0.5**o
        ref[2,1,1,0]=(0.0438148749584)/(o+1.)+(0.366758349324)+(0.153001662465)*0.5**o
        ref[2,1,1,1]=(0.692300008749)/(o+1.)+(0.288310211097)+(0.591374562403)*0.5**o
        ref[2,1,2,0]=(-0.217320783858)/(o+1.)+(0.21589055908)+(-0.742751457969)*0.5**o
        ref[2,1,2,1]=(0.497803852733)/(o+1.)+(0.180166964178)+(-0.834015860027)*0.5**o
        ref[2,2,0,0]=(-0.655435581477)/(o+1.)+(-0.30937321359)+(-0.203653128753)*0.5**o
        ref[2,2,0,1]=(0.140960526316)/(o+1.)+(-0.549930873491)+(0.971807071325)*0.5**o
        ref[2,2,1,0]=(-0.87272068545)/(o+1.)+(-0.159011679037)+(0.36337495614)*0.5**o
        ref[2,2,1,1]=(0.849847759033)/(o+1.)+(-0.132594990815)+(0.703205712066)*0.5**o
        ref[2,2,2,0]=(0.730674735708)/(o+1.)+(-0.144589857416)+(0.814179476293)*0.5**o
        ref[2,2,2,1]=(0.927807005045)/(o+1.)+(0.239143859117)+(0.465577907694)*0.5**o
        ref[2,3,0,0]=(0.737902566912)/(o+1.)+(-0.00945192595853)+(-0.0809501575869)*0.5**o
        ref[2,3,0,1]=(0.688538873306)/(o+1.)+(0.26402367848)+(-0.974260986562)*0.5**o
        ref[2,3,1,0]=(-0.892476705335)/(o+1.)+(0.20490133767)+(0.256567297589)*0.5**o
        ref[2,3,1,1]=(0.724531635238)/(o+1.)+(-0.524187316843)+(0.600281177478)*0.5**o
        ref[2,3,2,0]=(-0.636613154487)/(o+1.)+(0.300668452336)+(-0.316555549763)*0.5**o
        ref[2,3,2,1]=(0.241657689068)/(o+1.)+(-0.217836591241)+(0.509096161125)*0.5**o
        ref[2,4,0,0]=(-0.531592666371)/(o+1.)+(-0.288830707874)+(-0.00190589099569)*0.5**o
        ref[2,4,0,1]=(-0.32109862114)/(o+1.)+(-0.512155326408)+(0.727384767513)*0.5**o
        ref[2,4,1,0]=(0.815200064372)/(o+1.)+(-0.107542955049)+(-0.190511998024)*0.5**o
        ref[2,4,1,1]=(0.323832915733)/(o+1.)+(-0.223614062444)+(0.505564114308)*0.5**o
        ref[2,4,2,0]=(-0.378591546965)/(o+1.)+(0.0569076132008)+(-0.199017289187)*0.5**o
        ref[2,4,2,1]=(0.0866258209859)/(o+1.)+(0.181300307486)+(-0.166203292304)*0.5**o
        ref[3,0,0,0]=(0.42099538494)/(o+1.)+(0.50543930668)+(0.264566793834)*0.5**o
        ref[3,0,0,1]=(-0.566125315231)/(o+1.)+(-0.434053458415)+(0.235746402593)*0.5**o
        ref[3,0,1,0]=(-0.448144780994)/(o+1.)+(-0.316941332636)+(0.862345276865)*0.5**o
        ref[3,0,1,1]=(-0.652497974262)/(o+1.)+(0.267868866775)+(-0.896403179984)*0.5**o
        ref[3,0,2,0]=(-0.455838556745)/(o+1.)+(0.599971979095)+(0.788731583728)*0.5**o
        ref[3,0,2,1]=(0.158791540215)/(o+1.)+(0.510166773251)+(-0.727929054337)*0.5**o
        ref[3,1,0,0]=(-0.0535819231755)/(o+1.)+(-0.166034120633)+(0.349297794591)*0.5**o
        ref[3,1,0,1]=(-0.800973035912)/(o+1.)+(0.220227586048)+(-0.759321371341)*0.5**o
        ref[3,1,1,0]=(0.521881881398)/(o+1.)+(-0.025607129583)+(-0.597928946862)*0.5**o
        ref[3,1,1,1]=(0.445221632359)/(o+1.)+(-0.790614976052)+(-0.263124019379)*0.5**o
        ref[3,1,2,0]=(0.812893197377)/(o+1.)+(-0.550408879848)+(0.434074902924)*0.5**o
        ref[3,1,2,1]=(-0.40676151673)/(o+1.)+(0.21816811252)+(-0.724153308139)*0.5**o
        ref[3,2,0,0]=(-0.88668061529)/(o+1.)+(-0.196434028591)+(0.915215698154)*0.5**o
        ref[3,2,0,1]=(0.453597269106)/(o+1.)+(-0.753778677381)+(-0.456607413762)*0.5**o
        ref[3,2,1,0]=(0.0771381998527)/(o+1.)+(-0.189535168005)+(-0.428848816409)*0.5**o
        ref[3,2,1,1]=(0.786396903927)/(o+1.)+(0.26944712654)+(-0.488115399257)*0.5**o
        ref[3,2,2,0]=(-0.104450005208)/(o+1.)+(-0.480358405144)+(-0.748108272191)*0.5**o
        ref[3,2,2,1]=(-0.107194826319)/(o+1.)+(-0.656924554632)+(-0.288896584199)*0.5**o
        ref[3,3,0,0]=(0.184456668026)/(o+1.)+(0.3262637287)+(-0.643859183999)*0.5**o
        ref[3,3,0,1]=(-0.653836797208)/(o+1.)+(-0.0180862771476)+(-0.258292072759)*0.5**o
        ref[3,3,1,0]=(0.684973413357)/(o+1.)+(0.163588306602)+(-0.659219139805)*0.5**o
        ref[3,3,1,1]=(0.967988326693)/(o+1.)+(0.446278055504)+(-0.649682331365)*0.5**o
        ref[3,3,2,0]=(0.911862994231)/(o+1.)+(-0.14504591934)+(-0.0299199232233)*0.5**o
        ref[3,3,2,1]=(0.121310526207)/(o+1.)+(-0.532981603636)+(0.197795840564)*0.5**o
        ref[3,4,0,0]=(0.144475855023)/(o+1.)+(-0.356456284353)+(0.278568700563)*0.5**o
        ref[3,4,0,1]=(-0.846906570194)/(o+1.)+(0.598840084536)+(-0.608132003655)*0.5**o
        ref[3,4,1,0]=(-0.599983340725)/(o+1.)+(0.338470852121)+(-0.490163043631)*0.5**o
        ref[3,4,1,1]=(-0.466727935821)/(o+1.)+(-0.31358066885)+(-0.802228749858)*0.5**o
        ref[3,4,2,0]=(-0.0529537502248)/(o+1.)+(0.556985122302)+(0.963914397515)*0.5**o
        ref[3,4,2,1]=(0.942559524116)/(o+1.)+(0.029827380831)+(-0.201051432425)*0.5**o
      else:
        s[0,0,0,0]=(-0.345724047005)*x[0]**o+(-0.625686769843)*x[0]+(0.783605206054)*x[1]**o+(0.333976064143)*x[1]+(-0.567769956785)*x[2]**o+(-0.275228142751)*x[2]
        s[0,0,0,1]=(0.867013285813)*x[0]**o+(-0.0918669275484)*x[0]+(-0.162282139173)*x[1]**o+(-0.263245701183)*x[1]+(-0.358346293782)*x[2]**o+(0.170233601974)*x[2]
        s[0,0,1,0]=(-0.450333878814)*x[0]**o+(0.122384873389)*x[0]+(0.590676593091)*x[1]**o+(-0.373630926322)*x[1]+(-0.566894605215)*x[2]**o+(0.172738889453)*x[2]
        s[0,0,1,1]=(0.056183164677)*x[0]**o+(0.30593250486)*x[0]+(0.460330185746)*x[1]**o+(0.0649603857353)*x[1]+(0.530774242761)*x[2]**o+(-0.854200502062)*x[2]
        s[0,0,2,0]=(0.49271509708)*x[0]**o+(-0.822365387684)*x[0]+(0.150952101807)*x[1]**o+(-0.833532543948)*x[1]+(0.223699495078)*x[2]**o+(0.407256610259)*x[2]
        s[0,0,2,1]=(0.827558639052)*x[0]**o+(-0.310957726073)*x[0]+(-0.675858834923)*x[1]**o+(0.902788148659)*x[1]+(-0.569665674677)*x[2]**o+(0.971281178693)*x[2]
        s[0,1,0,0]=(-0.272375589027)*x[0]**o+(-0.285740311002)*x[0]+(0.181997544124)*x[1]**o+(-0.658402113041)*x[1]+(-0.278568799963)*x[2]**o+(-0.41754973361)*x[2]
        s[0,1,0,1]=(0.156364531059)*x[0]**o+(0.813560862368)*x[0]+(0.705700363405)*x[1]**o+(-0.817696115885)*x[1]+(-0.205894570429)*x[2]**o+(0.0674942609577)*x[2]
        s[0,1,1,0]=(0.429893143403)*x[0]**o+(-0.965234368002)*x[0]+(-0.974790265682)*x[1]**o+(0.0680256806875)*x[1]+(-0.433563362185)*x[2]**o+(-0.0492745942903)*x[2]
        s[0,1,1,1]=(-0.298098272504)*x[0]**o+(0.869998704901)*x[0]+(-0.18901211086)*x[1]**o+(-0.51577699091)*x[1]+(-0.328314462204)*x[2]**o+(-0.890845857013)*x[2]
        s[0,1,2,0]=(-0.43467847673)*x[0]**o+(-0.147481947495)*x[0]+(-0.542466755031)*x[1]**o+(-0.495418577682)*x[1]+(-0.2950922494)*x[2]**o+(-0.937573117558)*x[2]
        s[0,1,2,1]=(0.58285783575)*x[0]**o+(-0.902747768997)*x[0]+(-0.639055427646)*x[1]**o+(0.964665499733)*x[1]+(0.797615111504)*x[2]**o+(0.751347091591)*x[2]
        s[0,2,0,0]=(-0.53571726132)*x[0]**o+(-0.26462757313)*x[0]+(-0.456233906167)*x[1]**o+(0.881775957626)*x[1]+(-0.00514091075029)*x[2]**o+(-0.933937418957)*x[2]
        s[0,2,0,1]=(0.651527417402)*x[0]**o+(0.236630784815)*x[0]+(0.932958220915)*x[1]**o+(-0.819866395436)*x[1]+(0.253538518366)*x[2]**o+(-0.0163262782643)*x[2]
        s[0,2,1,0]=(0.452240733592)*x[0]**o+(0.450162809474)*x[0]+(0.699968406858)*x[1]**o+(0.164739887321)*x[1]+(-0.834564901207)*x[2]**o+(0.0249125503308)*x[2]
        s[0,2,1,1]=(-0.227907378794)*x[0]**o+(0.887204637473)*x[0]+(-0.514717641924)*x[1]**o+(0.911423718912)*x[1]+(0.876658247977)*x[2]**o+(-0.371520030414)*x[2]
        s[0,2,2,0]=(0.619773382249)*x[0]**o+(0.541760616441)*x[0]+(0.788185482567)*x[1]**o+(0.0250030620125)*x[1]+(0.163917412766)*x[2]**o+(0.191995043846)*x[2]
        s[0,2,2,1]=(-0.690898841846)*x[0]**o+(-0.708275716521)*x[0]+(-0.164538994288)*x[1]**o+(-0.803752886405)*x[1]+(-0.814464457618)*x[2]**o+(-0.910750603852)*x[2]
        s[0,3,0,0]=(0.752428867688)*x[0]**o+(0.911742295221)*x[0]+(-0.223242212004)*x[1]**o+(-0.757553597062)*x[1]+(-0.473559676565)*x[2]**o+(-0.827199094093)*x[2]
        s[0,3,0,1]=(0.300816669101)*x[0]**o+(-0.199692557045)*x[0]+(0.0825396055332)*x[1]**o+(-0.0693834001394)*x[1]+(-0.382683371052)*x[2]**o+(-0.0699997449306)*x[2]
        s[0,3,1,0]=(-0.370042079378)*x[0]**o+(-0.884622307688)*x[0]+(0.264943777413)*x[1]**o+(0.404099727226)*x[1]+(0.374755570635)*x[2]**o+(0.588241231168)*x[2]
        s[0,3,1,1]=(0.289583633117)*x[0]**o+(0.449941678672)*x[0]+(-0.353365382802)*x[1]**o+(0.515120940717)*x[1]+(0.740445432416)*x[2]**o+(-0.879930463576)*x[2]
        s[0,3,2,0]=(-0.347713282112)*x[0]**o+(-0.162259810338)*x[0]+(-0.696042768288)*x[1]**o+(-0.162202814362)*x[1]+(0.716572044832)*x[2]**o+(0.50377518734)*x[2]
        s[0,3,2,1]=(0.292567121826)*x[0]**o+(-0.76570190516)*x[0]+(0.0867231138296)*x[1]**o+(0.306479775529)*x[1]+(-0.377616255939)*x[2]**o+(0.232957710694)*x[2]
        s[0,4,0,0]=(0.73220485775)*x[0]**o+(-0.159793133992)*x[0]+(-0.377832723802)*x[1]**o+(-0.756658738359)*x[1]+(0.96057353991)*x[2]**o+(0.411157040033)*x[2]
        s[0,4,0,1]=(-0.331390253762)*x[0]**o+(-0.560289084334)*x[0]+(-0.827417830024)*x[1]**o+(0.719380560747)*x[1]+(-0.822147291472)*x[2]**o+(0.860119669443)*x[2]
        s[0,4,1,0]=(-0.505838817009)*x[0]**o+(-0.879628389565)*x[0]+(0.488746892572)*x[1]**o+(0.613837247188)*x[1]+(-0.154303229874)*x[2]**o+(0.00570279899539)*x[2]
        s[0,4,1,1]=(-0.122955617948)*x[0]**o+(-0.953071253159)*x[0]+(0.844003480214)*x[1]**o+(-0.746851086453)*x[1]+(-0.108832545758)*x[2]**o+(0.434183611414)*x[2]
        s[0,4,2,0]=(0.789617751574)*x[0]**o+(0.760644604676)*x[0]+(0.909793043646)*x[1]**o+(-0.279682873326)*x[1]+(0.700690761413)*x[2]**o+(-0.760237637314)*x[2]
        s[0,4,2,1]=(-0.882408865196)*x[0]**o+(0.328073069067)*x[0]+(0.858835691096)*x[1]**o+(0.650470359577)*x[1]+(-0.8717493079)*x[2]**o+(-0.177563387753)*x[2]
        s[1,0,0,0]=(-0.0500679538311)*x[0]**o+(-0.137932236517)*x[0]+(-0.742651074891)*x[1]**o+(-0.427978749091)*x[1]+(0.0361372113862)*x[2]**o+(-0.558139606192)*x[2]
        s[1,0,0,1]=(-0.00386321275167)*x[0]**o+(-0.51227499671)*x[0]+(-0.295080251306)*x[1]**o+(-0.326209704618)*x[1]+(-0.705968952206)*x[2]**o+(-0.848585657427)*x[2]
        s[1,0,1,0]=(0.173864935883)*x[0]**o+(0.148289460355)*x[0]+(0.40615421494)*x[1]**o+(-0.715315862741)*x[1]+(0.455318023779)*x[2]**o+(-0.539497210174)*x[2]
        s[1,0,1,1]=(-0.0628270158133)*x[0]**o+(-0.659218788396)*x[0]+(-0.0956661811574)*x[1]**o+(0.980329463956)*x[1]+(0.0114144009766)*x[2]**o+(-0.827594275845)*x[2]
        s[1,0,2,0]=(0.696208347195)*x[0]**o+(0.0807576571591)*x[0]+(0.998089746929)*x[1]**o+(0.177352400068)*x[1]+(0.685074203515)*x[2]**o+(-0.0635353733634)*x[2]
        s[1,0,2,1]=(-0.0223157590993)*x[0]**o+(0.559991238653)*x[0]+(0.548345826656)*x[1]**o+(-0.741886839548)*x[1]+(0.114218185947)*x[2]**o+(0.825638317499)*x[2]
        s[1,1,0,0]=(0.163206474973)*x[0]**o+(0.254896515732)*x[0]+(0.447114620799)*x[1]**o+(-0.563952237815)*x[1]+(0.57961770388)*x[2]**o+(0.0819856064889)*x[2]
        s[1,1,0,1]=(-0.328504704157)*x[0]**o+(0.0868460228579)*x[0]+(0.816944360875)*x[1]**o+(-0.697748850943)*x[1]+(-0.809725796641)*x[2]**o+(0.932721918575)*x[2]
        s[1,1,1,0]=(-0.287904638148)*x[0]**o+(-0.013943513773)*x[0]+(-0.54196693436)*x[1]**o+(-0.388084049637)*x[1]+(-0.702953342435)*x[2]**o+(-0.830664911774)*x[2]
        s[1,1,1,1]=(0.684091346652)*x[0]**o+(0.806375097652)*x[0]+(0.476003712147)*x[1]**o+(-0.864998508113)*x[1]+(-0.497254896777)*x[2]**o+(0.781872111446)*x[2]
        s[1,1,2,0]=(-0.844264089035)*x[0]**o+(-0.679014107829)*x[0]+(-0.198479658612)*x[1]**o+(0.375191314697)*x[1]+(-0.575465869392)*x[2]**o+(0.530364062529)*x[2]
        s[1,1,2,1]=(0.676323957189)*x[0]**o+(-0.501144179827)*x[0]+(-0.962137734649)*x[1]**o+(-0.356432667657)*x[1]+(-0.940336010235)*x[2]**o+(-0.236428299429)*x[2]
        s[1,2,0,0]=(0.355428925779)*x[0]**o+(0.274208957908)*x[0]+(-0.932593631234)*x[1]**o+(-0.376131921686)*x[1]+(0.286983572479)*x[2]**o+(-0.330880385128)*x[2]
        s[1,2,0,1]=(0.266570372333)*x[0]**o+(0.921388865301)*x[0]+(-0.1720476903)*x[1]**o+(0.998284527237)*x[1]+(0.0238473892294)*x[2]**o+(0.854216047199)*x[2]
        s[1,2,1,0]=(0.441558473549)*x[0]**o+(0.373044064317)*x[0]+(0.649370981178)*x[1]**o+(0.510194580982)*x[1]+(0.371823975996)*x[2]**o+(0.665301207824)*x[2]
        s[1,2,1,1]=(0.754004050836)*x[0]**o+(0.50557389845)*x[0]+(-0.251967535648)*x[1]**o+(0.704262653029)*x[1]+(0.645159509055)*x[2]**o+(0.183884331443)*x[2]
        s[1,2,2,0]=(-0.0673953798558)*x[0]**o+(0.708470323283)*x[0]+(-0.817934961085)*x[1]**o+(0.270394037097)*x[1]+(0.761348716495)*x[2]**o+(0.265483378874)*x[2]
        s[1,2,2,1]=(0.569853744398)*x[0]**o+(0.612861422857)*x[0]+(0.655472955979)*x[1]**o+(-0.387384090661)*x[1]+(0.026712723359)*x[2]**o+(0.504626498333)*x[2]
        s[1,3,0,0]=(-0.706312246902)*x[0]**o+(-0.309993808911)*x[0]+(0.25462597295)*x[1]**o+(-0.709061516294)*x[1]+(0.886706821135)*x[2]**o+(0.946960487449)*x[2]
        s[1,3,0,1]=(0.315339476547)*x[0]**o+(0.706694261286)*x[0]+(0.536483540093)*x[1]**o+(0.420431748645)*x[1]+(-0.401936712588)*x[2]**o+(-0.0303951289428)*x[2]
        s[1,3,1,0]=(-0.830759344331)*x[0]**o+(-0.0138001332539)*x[0]+(0.684117354857)*x[1]**o+(-0.521483470147)*x[1]+(0.261449467808)*x[2]**o+(0.81578910141)*x[2]
        s[1,3,1,1]=(0.218558430087)*x[0]**o+(0.117511078171)*x[0]+(-0.367757102068)*x[1]**o+(-0.301875631785)*x[1]+(-0.933013656177)*x[2]**o+(0.517230809269)*x[2]
        s[1,3,2,0]=(-0.273720099467)*x[0]**o+(-0.00869721580633)*x[0]+(-0.796835310998)*x[1]**o+(0.587901955041)*x[1]+(-0.664296869655)*x[2]**o+(-0.896980337429)*x[2]
        s[1,3,2,1]=(-0.068913139717)*x[0]**o+(0.656400342524)*x[0]+(0.855587799259)*x[1]**o+(0.876438035564)*x[1]+(-0.0622439736181)*x[2]**o+(-0.0977398744709)*x[2]
        s[1,4,0,0]=(0.427481073197)*x[0]**o+(0.20467745629)*x[0]+(-0.125277072461)*x[1]**o+(0.274783610604)*x[1]+(-0.384842532213)*x[2]**o+(0.790482142388)*x[2]
        s[1,4,0,1]=(0.723515401941)*x[0]**o+(-0.0529425602105)*x[0]+(-0.818443886494)*x[1]**o+(0.786875708369)*x[1]+(-0.747927140569)*x[2]**o+(-0.731906020242)*x[2]
        s[1,4,1,0]=(0.9774352561)*x[0]**o+(0.0769002693408)*x[0]+(-0.576964776525)*x[1]**o+(-0.313768570439)*x[1]+(-0.896910128278)*x[2]**o+(-0.280920538425)*x[2]
        s[1,4,1,1]=(0.138312702528)*x[0]**o+(-0.458814456032)*x[0]+(0.163241743373)*x[1]**o+(-0.803420897174)*x[1]+(-0.140886706823)*x[2]**o+(-0.14390608176)*x[2]
        s[1,4,2,0]=(0.529048827233)*x[0]**o+(-0.0554100800212)*x[0]+(0.456015489421)*x[1]**o+(0.422262108301)*x[1]+(-0.313743004281)*x[2]**o+(-0.13975864585)*x[2]
        s[1,4,2,1]=(0.838760174947)*x[0]**o+(-0.223195641306)*x[0]+(0.191058937589)*x[1]**o+(-0.0383799908705)*x[1]+(-0.123273771781)*x[2]**o+(0.375902823133)*x[2]
        s[2,0,0,0]=(-0.376325501997)*x[0]**o+(-0.999695820134)*x[0]+(0.145774433039)*x[1]**o+(0.0848795062374)*x[1]+(-0.789276953104)*x[2]**o+(-0.936604606234)*x[2]
        s[2,0,0,1]=(-0.393633363986)*x[0]**o+(0.122599807309)*x[0]+(-0.308578856774)*x[1]**o+(-0.123177461702)*x[1]+(-0.135481435454)*x[2]**o+(0.583777567706)*x[2]
        s[2,0,1,0]=(-0.573207876072)*x[0]**o+(-0.194869976915)*x[0]+(0.611265046543)*x[1]**o+(0.218225055561)*x[1]+(-0.88953645988)*x[2]**o+(0.314028438499)*x[2]
        s[2,0,1,1]=(0.82764053208)*x[0]**o+(0.408293655023)*x[0]+(-0.158898571521)*x[1]**o+(0.557257506806)*x[1]+(0.22438907333)*x[2]**o+(-0.616052713022)*x[2]
        s[2,0,2,0]=(0.680002851599)*x[0]**o+(-0.833669903036)*x[0]+(0.0182560742948)*x[1]**o+(-0.401224011261)*x[1]+(-0.601050104604)*x[2]**o+(0.354512589668)*x[2]
        s[2,0,2,1]=(-0.99533923296)*x[0]**o+(-0.252792558615)*x[0]+(-0.417436606414)*x[1]**o+(-0.905526797068)*x[1]+(-0.323384139739)*x[2]**o+(0.217030918198)*x[2]
        s[2,1,0,0]=(-0.757427161409)*x[0]**o+(-0.0463730910958)*x[0]+(0.44512589496)*x[1]**o+(0.545180882409)*x[1]+(0.78683574265)*x[2]**o+(-0.684406722817)*x[2]
        s[2,1,0,1]=(-0.61819592214)*x[0]**o+(-0.797609085554)*x[0]+(0.983105716733)*x[1]**o+(0.322191127831)*x[1]+(0.453981669947)*x[2]**o+(-0.909192167062)*x[2]
        s[2,1,1,0]=(-0.898119938157)*x[0]**o+(0.311855250372)*x[0]+(0.505532351296)*x[1]**o+(0.139215174265)*x[1]+(0.772001036932)*x[2]**o+(-0.225670248604)*x[2]
        s[2,1,1,1]=(0.0316457581752)*x[0]**o+(-0.488943159802)*x[0]+(-0.773453487185)*x[1]**o+(0.815697082082)*x[1]+(0.206189975058)*x[2]**o+(-0.575428943874)*x[2]
        s[2,1,2,0]=(0.319688798055)*x[0]**o+(-0.0695712382178)*x[0]+(0.689876574974)*x[1]**o+(0.190028248325)*x[1]+(-0.766718072759)*x[2]**o+(-0.52180809158)*x[2]
        s[2,1,2,1]=(-0.865578912594)*x[0]**o+(0.460194992238)*x[0]+(-0.0025880450282)*x[1]**o+(-0.400524494504)*x[1]+(0.417535391073)*x[2]**o+(-0.0342670929868)*x[2]
        s[2,2,0,0]=(0.421599950602)*x[0]**o+(0.266747809147)*x[0]+(-0.169043748241)*x[1]**o+(-0.164542032135)*x[1]+(0.555070565368)*x[2]**o+(-0.0217786140345)*x[2]
        s[2,2,0,1]=(-0.625527951515)*x[0]**o+(-0.998518258266)*x[0]+(-0.0978678841265)*x[1]**o+(0.00726403245479)*x[1]+(-0.210296926513)*x[2]**o+(0.0169357807292)*x[2]
        s[2,2,1,0]=(-0.927097383803)*x[0]**o+(0.142344498674)*x[0]+(-0.565097374968)*x[1]**o+(-0.19883325806)*x[1]+(-0.409577428899)*x[2]**o+(-0.433045796687)*x[2]
        s[2,2,1,1]=(0.0900116992509)*x[0]**o+(0.907232745809)*x[0]+(0.164351943946)*x[1]**o+(0.831012029454)*x[1]+(0.659298356625)*x[2]**o+(0.658166177134)*x[2]
        s[2,2,2,0]=(-0.468765776872)*x[0]**o+(-0.56578531156)*x[0]+(-0.515309324035)*x[1]**o+(-0.891192406798)*x[1]+(-0.229751474081)*x[2]**o+(0.861043357121)*x[2]
        s[2,2,2,1]=(-0.102463609474)*x[0]**o+(0.795582169816)*x[0]+(0.473799689371)*x[1]**o+(-0.624789163959)*x[1]+(-0.120758299241)*x[2]**o+(-0.790110725507)*x[2]
        s[2,3,0,0]=(-0.61205272187)*x[0]**o+(-0.40060893961)*x[0]+(-0.789664972057)*x[1]**o+(0.515295029759)*x[1]+(-0.0475219131724)*x[2]**o+(-0.148006363455)*x[2]
        s[2,3,0,1]=(-0.866234572657)*x[0]**o+(-0.980788916374)*x[0]+(0.110830262867)*x[1]**o+(0.76938462126)*x[1]+(0.338028197458)*x[2]**o+(0.695510468586)*x[2]
        s[2,3,1,0]=(-0.61195493774)*x[0]**o+(0.418983087999)*x[0]+(0.698112113863)*x[1]**o+(-0.961155169263)*x[1]+(-0.237690733199)*x[2]**o+(0.160246107308)*x[2]
        s[2,3,1,1]=(0.186790790362)*x[0]**o+(0.726925157226)*x[0]+(-0.876208560721)*x[1]**o+(0.684333403771)*x[1]+(0.859789784415)*x[2]**o+(0.180765658997)*x[2]
        s[2,3,2,0]=(-0.107447528726)*x[0]**o+(0.23776834378)*x[0]+(0.203151310439)*x[1]**o+(0.939282834891)*x[1]+(-0.739341306594)*x[2]**o+(0.526784359658)*x[2]
        s[2,3,2,1]=(-0.659583948903)*x[0]**o+(-0.876524913083)*x[0]+(-0.140502393442)*x[1]**o+(0.220988067839)*x[1]+(-0.315594035706)*x[2]**o+(-0.337007362379)*x[2]
        s[2,4,0,0]=(0.824520364974)*x[0]**o+(-0.0700628701349)*x[0]+(-0.337899096069)*x[1]**o+(-0.969230762171)*x[1]+(0.451483693063)*x[2]**o+(0.560376952474)*x[2]
        s[2,4,0,1]=(0.19676925667)*x[0]**o+(-0.509702667102)*x[0]+(-0.560085912439)*x[1]**o+(0.850131177282)*x[1]+(0.0209473113715)*x[2]**o+(-0.524835181986)*x[2]
        s[2,4,1,0]=(0.696459080737)*x[0]**o+(-0.0300420621411)*x[0]+(0.872376074883)*x[1]**o+(0.721784659413)*x[1]+(0.0469660828352)*x[2]**o+(0.848375867902)*x[2]
        s[2,4,1,1]=(-0.0300055784957)*x[0]**o+(0.86110892248)*x[0]+(-0.0372850528438)*x[1]**o+(-0.626675690839)*x[1]+(-0.96746918975)*x[2]**o+(-0.118288468609)*x[2]
        s[2,4,2,0]=(-0.773844175596)*x[0]**o+(0.97042325748)*x[0]+(-0.709105396658)*x[1]**o+(-0.436388373122)*x[1]+(-0.841519653893)*x[2]**o+(-0.661541598363)*x[2]
        s[2,4,2,1]=(-0.289904092026)*x[0]**o+(0.838224779606)*x[0]+(0.272732151891)*x[1]**o+(-0.789836453998)*x[1]+(0.574134239472)*x[2]**o+(-0.196980303288)*x[2]
        s[3,0,0,0]=(0.921207531349)*x[0]**o+(-0.944820675428)*x[0]+(-0.632934426248)*x[1]**o+(-0.986209890868)*x[1]+(0.889736953978)*x[2]**o+(0.308722225546)*x[2]
        s[3,0,0,1]=(-0.349795723275)*x[0]**o+(-0.748469735322)*x[0]+(-0.581786511254)*x[1]**o+(0.696378145052)*x[1]+(0.619384687507)*x[2]**o+(-0.0953346757601)*x[2]
        s[3,0,1,0]=(0.539616405472)*x[0]**o+(-0.265315341201)*x[0]+(0.042248150432)*x[1]**o+(-0.115847796714)*x[1]+(-0.217838196299)*x[2]**o+(-0.0631408906436)*x[2]
        s[3,0,1,1]=(-0.823978915895)*x[0]**o+(-0.608867256385)*x[0]+(0.446515851959)*x[1]**o+(0.531432611215)*x[1]+(0.689019079647)*x[2]**o+(-0.0317787775133)*x[2]
        s[3,0,2,0]=(0.347425369617)*x[0]**o+(-0.739691856701)*x[0]+(-0.398130284368)*x[1]**o+(0.108805616885)*x[1]+(-0.0912940693547)*x[2]**o+(0.117214641864)*x[2]
        s[3,0,2,1]=(-0.383805789854)*x[0]**o+(-0.0456422576844)*x[0]+(0.533382056608)*x[1]**o+(-0.218226601069)*x[1]+(-0.676716489399)*x[2]**o+(-0.00199666729435)*x[2]
        s[3,1,0,0]=(-0.372478384483)*x[0]**o+(0.586299542288)*x[0]+(-0.454391251107)*x[1]**o+(0.0882830261313)*x[1]+(-0.623432789013)*x[2]**o+(0.218598634148)*x[2]
        s[3,1,0,1]=(0.578294223191)*x[0]**o+(-0.355620351739)*x[0]+(0.231898319431)*x[1]**o+(0.423862975521)*x[1]+(0.646375025433)*x[2]**o+(-0.213634599835)*x[2]
        s[3,1,1,0]=(0.0525838616186)*x[0]**o+(0.118311802828)*x[0]+(0.548544020336)*x[1]**o+(0.584362580997)*x[1]+(-0.234381266126)*x[2]**o+(-0.845553660597)*x[2]
        s[3,1,1,1]=(-0.677671170669)*x[0]**o+(-0.290465667864)*x[0]+(-0.0167835285288)*x[1]**o+(-0.100024088985)*x[1]+(-0.150243328686)*x[2]**o+(0.679119492133)*x[2]
        s[3,1,2,0]=(-0.121237844262)*x[0]**o+(-0.458196712924)*x[0]+(0.963853129183)*x[1]**o+(-0.211094701554)*x[1]+(0.868445433817)*x[2]**o+(0.902283926176)*x[2]
        s[3,1,2,1]=(0.504161603066)*x[0]**o+(-0.190472686354)*x[0]+(-0.0917827193891)*x[1]**o+(-0.352839230228)*x[1]+(-0.353813861163)*x[2]**o+(0.230960874753)*x[2]
        s[3,2,0,0]=(-0.34790984999)*x[0]**o+(-0.395507825603)*x[0]+(-0.6489230899)*x[1]**o+(0.712401749209)*x[1]+(-0.0821476018477)*x[2]**o+(0.690431614802)*x[2]
        s[3,2,0,1]=(-0.557444127335)*x[0]**o+(-0.834050224977)*x[0]+(-0.22666297073)*x[1]**o+(0.405731464147)*x[1]+(-0.671211465442)*x[2]**o+(-0.644382670803)*x[2]
        s[3,2,1,0]=(0.965420676359)*x[0]**o+(-0.228764004519)*x[0]+(-0.859916958223)*x[1]**o+(-0.452756903956)*x[1]+(-0.973849033231)*x[2]**o+(-0.24133735483)*x[2]
        s[3,2,1,1]=(-0.950755375408)*x[0]**o+(-0.851938171221)*x[0]+(0.0181318395336)*x[1]**o+(-0.499474780924)*x[1]+(0.910360613628)*x[2]**o+(0.199314643801)*x[2]
        s[3,2,2,0]=(0.339657614739)*x[0]**o+(0.145101826319)*x[0]+(0.0477774162796)*x[1]**o+(-0.883319372262)*x[1]+(0.297676781475)*x[2]**o+(0.375455002664)*x[2]
        s[3,2,2,1]=(0.427508981997)*x[0]**o+(0.735804891833)*x[0]+(-0.941349025764)*x[1]**o+(-0.848431716981)*x[1]+(0.878538549993)*x[2]**o+(0.0726603284636)*x[2]
        s[3,3,0,0]=(-0.973352015556)*x[0]**o+(0.836731359075)*x[0]+(0.665815761101)*x[1]**o+(0.255531920855)*x[1]+(0.782674767576)*x[2]**o+(-0.570574483199)*x[2]
        s[3,3,0,1]=(0.950365830796)*x[0]**o+(0.421946061345)*x[0]+(0.905170182227)*x[1]**o+(0.613401792858)*x[1]+(-0.442076185783)*x[2]**o+(-0.484925057701)*x[2]
        s[3,3,1,0]=(-0.00222827033475)*x[0]**o+(-0.531682094474)*x[0]+(0.458841364479)*x[1]**o+(-0.655184028959)*x[1]+(-0.22086141276)*x[2]**o+(-0.813839980108)*x[2]
        s[3,3,1,1]=(0.887884370757)*x[0]**o+(-0.0538573388546)*x[0]+(0.927878694132)*x[1]**o+(0.471741834858)*x[1]+(-0.313646320846)*x[2]**o+(0.311547715614)*x[2]
        s[3,3,2,0]=(-0.0214623472154)*x[0]**o+(-0.444460344328)*x[0]+(0.0788294611308)*x[1]**o+(0.41576519959)*x[1]+(-0.778008962903)*x[2]**o+(0.900765472759)*x[2]
        s[3,3,2,1]=(0.949491478772)*x[0]**o+(-0.471010680081)*x[0]+(-0.876305371565)*x[1]**o+(-0.120079570893)*x[1]+(0.0947323737758)*x[2]**o+(-0.361392549886)*x[2]
        s[3,4,0,0]=(0.354489762998)*x[0]**o+(-0.425422633745)*x[0]+(-0.566692257139)*x[1]**o+(0.504748998989)*x[1]+(0.226102951066)*x[2]**o+(-0.744917446958)*x[2]
        s[3,4,0,1]=(0.552196121696)*x[0]**o+(0.533127640573)*x[0]+(-0.147082768815)*x[1]**o+(-0.801997447891)*x[1]+(0.104711977816)*x[2]**o+(0.99454944575)*x[2]
        s[3,4,1,0]=(0.839982844034)*x[0]**o+(-0.779008776044)*x[0]+(-0.561078679707)*x[1]**o+(-0.0519865292547)*x[1]+(-0.566301067503)*x[2]**o+(-0.267957119508)*x[2]
        s[3,4,1,1]=(0.78962834926)*x[0]**o+(-0.93892858306)*x[0]+(-0.114227870549)*x[1]**o+(0.420485886976)*x[1]+(0.974178392694)*x[2]**o+(-0.421944523239)*x[2]
        s[3,4,2,0]=(-0.882936934355)*x[0]**o+(-0.441556030186)*x[0]+(0.0681795049997)*x[1]**o+(0.731317357905)*x[1]+(-0.965801964833)*x[2]**o+(-0.319270469244)*x[2]
        s[3,4,2,1]=(0.342579062975)*x[0]**o+(0.23386981431)*x[0]+(-0.571662448793)*x[1]**o+(-0.257739898874)*x[1]+(0.176906895006)*x[2]**o+(0.892856914788)*x[2]
        ref[0,0,0,0]=(0.215835249268)/(o+1.)+(-0.283469424226)+(-0.345724047005)*0.5**o
        ref[0,0,0,1]=(-0.520628432955)/(o+1.)+(-0.0924395133784)+(0.867013285813)*0.5**o
        ref[0,0,1,0]=(0.0237819878764)/(o+1.)+(-0.0392535817398)+(-0.450333878814)*0.5**o
        ref[0,0,1,1]=(0.991104428507)/(o+1.)+(-0.241653805734)+(0.056183164677)*0.5**o
        ref[0,0,2,0]=(0.374651596885)/(o+1.)+(-0.624320660687)+(0.49271509708)*0.5**o
        ref[0,0,2,1]=(-1.2455245096)/(o+1.)+(0.781555800639)+(0.827558639052)*0.5**o
        ref[0,1,0,0]=(-0.0965712558395)/(o+1.)+(-0.680846078827)+(-0.272375589027)*0.5**o
        ref[0,1,0,1]=(0.499805792977)/(o+1.)+(0.0316795037204)+(0.156364531059)*0.5**o
        ref[0,1,1,0]=(-1.40835362787)/(o+1.)+(-0.473241640802)+(0.429893143403)*0.5**o
        ref[0,1,1,1]=(-0.517326573063)/(o+1.)+(-0.268312071511)+(-0.298098272504)*0.5**o
        ref[0,1,2,0]=(-0.837559004431)/(o+1.)+(-0.790236821368)+(-0.43467847673)*0.5**o
        ref[0,1,2,1]=(0.158559683859)/(o+1.)+(0.406632411163)+(0.58285783575)*0.5**o
        ref[0,2,0,0]=(-0.461374816917)/(o+1.)+(-0.158394517231)+(-0.53571726132)*0.5**o
        ref[0,2,0,1]=(1.18649673928)/(o+1.)+(-0.299780944442)+(0.651527417402)*0.5**o
        ref[0,2,1,0]=(-0.13459649435)/(o+1.)+(0.319907623563)+(0.452240733592)*0.5**o
        ref[0,2,1,1]=(0.361940606053)/(o+1.)+(0.713554162985)+(-0.227907378794)*0.5**o
        ref[0,2,2,0]=(0.952102895333)/(o+1.)+(0.37937936115)+(0.619773382249)*0.5**o
        ref[0,2,2,1]=(-0.979003451906)/(o+1.)+(-1.21138960339)+(-0.690898841846)*0.5**o
        ref[0,3,0,0]=(-0.696801888569)/(o+1.)+(-0.336505197967)+(0.752428867688)*0.5**o
        ref[0,3,0,1]=(-0.300143765518)/(o+1.)+(-0.169537851057)+(0.300816669101)*0.5**o
        ref[0,3,1,0]=(0.639699348048)/(o+1.)+(0.0538593253528)+(-0.370042079378)*0.5**o
        ref[0,3,1,1]=(0.387080049614)/(o+1.)+(0.0425660779061)+(0.289583633117)*0.5**o
        ref[0,3,2,0]=(0.0205292765437)/(o+1.)+(0.0896562813201)+(-0.347713282112)*0.5**o
        ref[0,3,2,1]=(-0.29089314211)/(o+1.)+(-0.113132209469)+(0.292567121826)*0.5**o
        ref[0,4,0,0]=(0.582740816108)/(o+1.)+(-0.252647416159)+(0.73220485775)*0.5**o
        ref[0,4,0,1]=(-1.6495651215)/(o+1.)+(0.509605572928)+(-0.331390253762)*0.5**o
        ref[0,4,1,0]=(0.334443662697)/(o+1.)+(-0.130044171691)+(-0.505838817009)*0.5**o
        ref[0,4,1,1]=(0.735170934456)/(o+1.)+(-0.632869364099)+(-0.122955617948)*0.5**o
        ref[0,4,2,0]=(1.61048380506)/(o+1.)+(-0.139637952982)+(0.789617751574)*0.5**o
        ref[0,4,2,1]=(-0.012913616804)/(o+1.)+(0.400490020446)+(-0.882408865196)*0.5**o
        ref[1,0,0,0]=(-0.706513863505)/(o+1.)+(-0.5620252959)+(-0.0500679538311)*0.5**o
        ref[1,0,0,1]=(-1.00104920351)/(o+1.)+(-0.843535179378)+(-0.00386321275167)*0.5**o
        ref[1,0,1,0]=(0.861472238719)/(o+1.)+(-0.55326180628)+(0.173864935883)*0.5**o
        ref[1,0,1,1]=(-0.0842517801808)/(o+1.)+(-0.253241800142)+(-0.0628270158133)*0.5**o
        ref[1,0,2,0]=(1.68316395044)/(o+1.)+(0.097287341932)+(0.696208347195)*0.5**o
        ref[1,0,2,1]=(0.662564012602)/(o+1.)+(0.321871358302)+(-0.0223157590993)*0.5**o
        ref[1,1,0,0]=(1.02673232468)/(o+1.)+(-0.113535057797)+(0.163206474973)*0.5**o
        ref[1,1,0,1]=(0.00721856423403)/(o+1.)+(0.160909545245)+(-0.328504704157)*0.5**o
        ref[1,1,1,0]=(-1.24492027679)/(o+1.)+(-0.616346237592)+(-0.287904638148)*0.5**o
        ref[1,1,1,1]=(-0.0212511846297)/(o+1.)+(0.361624350493)+(0.684091346652)*0.5**o
        ref[1,1,2,0]=(-0.773945528004)/(o+1.)+(0.113270634698)+(-0.844264089035)*0.5**o
        ref[1,1,2,1]=(-1.90247374488)/(o+1.)+(-0.547002573456)+(0.676323957189)*0.5**o
        ref[1,2,0,0]=(-0.645610058755)/(o+1.)+(-0.216401674453)+(0.355428925779)*0.5**o
        ref[1,2,0,1]=(-0.148200301071)/(o+1.)+(1.38694471987)+(0.266570372333)*0.5**o
        ref[1,2,1,0]=(1.02119495717)/(o+1.)+(0.774269926562)+(0.441558473549)*0.5**o
        ref[1,2,1,1]=(0.393191973407)/(o+1.)+(0.696860441461)+(0.754004050836)*0.5**o
        ref[1,2,2,0]=(-0.0565862445904)/(o+1.)+(0.622173869627)+(-0.0673953798558)*0.5**o
        ref[1,2,2,1]=(0.682185679338)/(o+1.)+(0.365051915264)+(0.569853744398)*0.5**o
        ref[1,3,0,0]=(1.14133279409)/(o+1.)+(-0.0360474188784)+(-0.706312246902)*0.5**o
        ref[1,3,0,1]=(0.134546827505)/(o+1.)+(0.548365440494)+(0.315339476547)*0.5**o
        ref[1,3,1,0]=(0.945566822665)/(o+1.)+(0.140252749005)+(-0.830759344331)*0.5**o
        ref[1,3,1,1]=(-1.30077075824)/(o+1.)+(0.166433127827)+(0.218558430087)*0.5**o
        ref[1,3,2,0]=(-1.46113218065)/(o+1.)+(-0.158887799097)+(-0.273720099467)*0.5**o
        ref[1,3,2,1]=(0.793343825641)/(o+1.)+(0.717549251808)+(-0.068913139717)*0.5**o
        ref[1,4,0,0]=(-0.510119604674)/(o+1.)+(0.634971604641)+(0.427481073197)*0.5**o
        ref[1,4,0,1]=(-1.56637102706)/(o+1.)+(0.00101356395823)+(0.723515401941)*0.5**o
        ref[1,4,1,0]=(-1.4738749048)/(o+1.)+(-0.258894419762)+(0.9774352561)*0.5**o
        ref[1,4,1,1]=(0.0223550365494)/(o+1.)+(-0.703070717483)+(0.138312702528)*0.5**o
        ref[1,4,2,0]=(0.14227248514)/(o+1.)+(0.113546691215)+(0.529048827233)*0.5**o
        ref[1,4,2,1]=(0.0677851658086)/(o+1.)+(0.0571635954781)+(0.838760174947)*0.5**o
        ref[2,0,0,0]=(-0.643502520065)/(o+1.)+(-0.925710460066)+(-0.376325501997)*0.5**o
        ref[2,0,0,1]=(-0.444060292228)/(o+1.)+(0.291599956656)+(-0.393633363986)*0.5**o
        ref[2,0,1,0]=(-0.278271413338)/(o+1.)+(0.168691758573)+(-0.573207876072)*0.5**o
        ref[2,0,1,1]=(0.0654905018086)/(o+1.)+(0.174749224404)+(0.82764053208)*0.5**o
        ref[2,0,2,0]=(-0.582794030309)/(o+1.)+(-0.440190662315)+(0.680002851599)*0.5**o
        ref[2,0,2,1]=(-0.740820746153)/(o+1.)+(-0.470644218743)+(-0.99533923296)*0.5**o
        ref[2,1,0,0]=(1.23196163761)/(o+1.)+(-0.0927994657521)+(-0.757427161409)*0.5**o
        ref[2,1,0,1]=(1.43708738668)/(o+1.)+(-0.692305062393)+(-0.61819592214)*0.5**o
        ref[2,1,1,0]=(1.27753338823)/(o+1.)+(0.112700088017)+(-0.898119938157)*0.5**o
        ref[2,1,1,1]=(-0.567263512127)/(o+1.)+(-0.124337510797)+(0.0316457581752)*0.5**o
        ref[2,1,2,0]=(-0.0768414977849)/(o+1.)+(-0.200675540737)+(0.319688798055)*0.5**o
        ref[2,1,2,1]=(0.414947346045)/(o+1.)+(0.0127017023739)+(-0.865578912594)*0.5**o
        ref[2,2,0,0]=(0.386026817127)/(o+1.)+(0.0402135814888)+(0.421599950602)*0.5**o
        ref[2,2,0,1]=(-0.30816481064)/(o+1.)+(-0.487159222541)+(-0.625527951515)*0.5**o
        ref[2,2,1,0]=(-0.974674803867)/(o+1.)+(-0.244767278037)+(-0.927097383803)*0.5**o
        ref[2,2,1,1]=(0.82365030057)/(o+1.)+(1.1982054762)+(0.0900116992509)*0.5**o
        ref[2,2,2,0]=(-0.745060798116)/(o+1.)+(-0.297967180618)+(-0.468765776872)*0.5**o
        ref[2,2,2,1]=(0.353041390129)/(o+1.)+(-0.309658859825)+(-0.102463609474)*0.5**o
        ref[2,3,0,0]=(-0.83718688523)/(o+1.)+(-0.0166601366533)+(-0.61205272187)*0.5**o
        ref[2,3,0,1]=(0.448858460325)/(o+1.)+(0.242053086736)+(-0.866234572657)*0.5**o
        ref[2,3,1,0]=(0.460421380665)/(o+1.)+(-0.190962986978)+(-0.61195493774)*0.5**o
        ref[2,3,1,1]=(-0.0164187763059)/(o+1.)+(0.796012109997)+(0.186790790362)*0.5**o
        ref[2,3,2,0]=(-0.536189996155)/(o+1.)+(0.851917769165)+(-0.107447528726)*0.5**o
        ref[2,3,2,1]=(-0.456096429148)/(o+1.)+(-0.496272103811)+(-0.659583948903)*0.5**o
        ref[2,4,0,0]=(0.113584596994)/(o+1.)+(-0.239458339916)+(0.824520364974)*0.5**o
        ref[2,4,0,1]=(-0.539138601067)/(o+1.)+(-0.0922033359031)+(0.19676925667)*0.5**o
        ref[2,4,1,0]=(0.919342157718)/(o+1.)+(0.770059232587)+(0.696459080737)*0.5**o
        ref[2,4,1,1]=(-1.00475424259)/(o+1.)+(0.0580723815159)+(-0.0300055784957)*0.5**o
        ref[2,4,2,0]=(-1.55062505055)/(o+1.)+(-0.0637533570027)+(-0.773844175596)*0.5**o
        ref[2,4,2,1]=(0.846866391363)/(o+1.)+(-0.0742959888403)+(-0.289904092026)*0.5**o
        ref[3,0,0,0]=(0.25680252773)/(o+1.)+(-0.811154170375)+(0.921207531349)*0.5**o
        ref[3,0,0,1]=(0.037598176253)/(o+1.)+(-0.0737131330151)+(-0.349795723275)*0.5**o
        ref[3,0,1,0]=(-0.175590045867)/(o+1.)+(-0.222152014279)+(0.539616405472)*0.5**o
        ref[3,0,1,1]=(1.13553493161)/(o+1.)+(-0.0546067113414)+(-0.823978915895)*0.5**o
        ref[3,0,2,0]=(-0.489424353723)/(o+1.)+(-0.256835798975)+(0.347425369617)*0.5**o
        ref[3,0,2,1]=(-0.143334432791)/(o+1.)+(-0.132932763024)+(-0.383805789854)*0.5**o
        ref[3,1,0,0]=(-1.07782404012)/(o+1.)+(0.446590601283)+(-0.372478384483)*0.5**o
        ref[3,1,0,1]=(0.878273344865)/(o+1.)+(-0.0726959880264)+(0.578294223191)*0.5**o
        ref[3,1,1,0]=(0.314162754211)/(o+1.)+(-0.0714396383864)+(0.0525838616186)*0.5**o
        ref[3,1,1,1]=(-0.167026857215)/(o+1.)+(0.144314867642)+(-0.677671170669)*0.5**o
        ref[3,1,2,0]=(1.832298563)/(o+1.)+(0.116496255849)+(-0.121237844262)*0.5**o
        ref[3,1,2,1]=(-0.445596580552)/(o+1.)+(-0.156175520914)+(0.504161603066)*0.5**o
        ref[3,2,0,0]=(-0.731070691747)/(o+1.)+(0.503662769204)+(-0.34790984999)*0.5**o
        ref[3,2,0,1]=(-0.897874436171)/(o+1.)+(-0.536350715816)+(-0.557444127335)*0.5**o
        ref[3,2,1,0]=(-1.83376599145)/(o+1.)+(-0.461429131652)+(0.965420676359)*0.5**o
        ref[3,2,1,1]=(0.928492453162)/(o+1.)+(-0.576049154172)+(-0.950755375408)*0.5**o
        ref[3,2,2,0]=(0.345454197754)/(o+1.)+(-0.181381271639)+(0.339657614739)*0.5**o
        ref[3,2,2,1]=(-0.0628104757709)/(o+1.)+(-0.0199832483422)+(0.427508981997)*0.5**o
        ref[3,3,0,0]=(1.44849052868)/(o+1.)+(0.260844398365)+(-0.973352015556)*0.5**o
        ref[3,3,0,1]=(0.463093996444)/(o+1.)+(0.275211398251)+(0.950365830796)*0.5**o
        ref[3,3,1,0]=(0.237979951719)/(o+1.)+(-1.00035305177)+(-0.00222827033475)*0.5**o
        ref[3,3,1,1]=(0.614232373286)/(o+1.)+(0.364716105808)+(0.887884370757)*0.5**o
        ref[3,3,2,0]=(-0.699179501772)/(o+1.)+(0.436035164011)+(-0.0214623472154)*0.5**o
        ref[3,3,2,1]=(-0.781572997789)/(o+1.)+(-0.47624140043)+(0.949491478772)*0.5**o
        ref[3,4,0,0]=(-0.340589306073)/(o+1.)+(-0.332795540857)+(0.354489762998)*0.5**o
        ref[3,4,0,1]=(-0.0423707909991)/(o+1.)+(0.362839819216)+(0.552196121696)*0.5**o
        ref[3,4,1,0]=(-1.12737974721)/(o+1.)+(-0.549476212403)+(0.839982844034)*0.5**o
        ref[3,4,1,1]=(0.859950522145)/(o+1.)+(-0.470193609661)+(0.78962834926)*0.5**o
        ref[3,4,2,0]=(-0.897622459834)/(o+1.)+(-0.0147545707622)+(-0.882936934355)*0.5**o
        ref[3,4,2,1]=(-0.394755553787)/(o+1.)+(0.434493415112)+(0.342579062975)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactOne_fromSymbol_ReducedSolution_rank0(self):
      """
      tests integral of rank 0 Symbol on the FunctionOnContactOne

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports integral on FunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Symbol(shape=())
      s=Data(0,(),w)
      if dim==2:
        s=(-0.0123111028353)*x[0]+(-1.54593060503)*x[1]
        ref=(-0.883444958139)/(o+1.)+(-0.0326056141412)+(-0.609585521441)*0.5**o
      else:
        s=(0.465974344755)*x[0]+(-0.406015068796)*x[1]+(0.217033091571)*x[2]
        ref=(-0.964783366814)/(o+1.)+(0.787210317182)+(-0.332644900019)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactOne_fromSymbol_ReducedSolution_rank1(self):
      """
      tests integral of rank 1 Symbol on the FunctionOnContactOne

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports integral on FunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(2,))
      s=Data(0,(2,),w)
      ref=numpy.zeros((2,),numpy.float_)
      if dim==2:
        s[0]=(-1.01806458946)*x[0]+(-0.84209500794)*x[1]
        s[1]=(-0.372739669691)*x[0]+(0.123203057686)*x[1]
        ref[0]=(-0.910310319387)/(o+1.)+(-0.35457604)+(-0.240697198017)*0.5**o
        ref[1]=(0.817182634856)/(o+1.)+(-0.752992097502)+(0.439264948143)*0.5**o
      else:
        s[0]=(-0.262859740698)*x[0]+(0.398827301114)*x[1]+(-0.0121863377176)*x[2]
        s[1]=(0.533472917947)*x[0]+(-0.654130540619)*x[1]+(-0.155059686066)*x[2]
        ref[0]=(0.173661143447)/(o+1.)+(-0.0377591573443)+(0.0256383939398)*0.5**o
        ref[1]=(-1.0263552046)/(o+1.)+(0.265270569058)+(0.220096757743)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactOne_fromSymbol_ReducedSolution_rank2(self):
      """
      tests integral of rank 2 Symbol on the FunctionOnContactOne

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports integral on FunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5))
      s=Data(0,(4, 5),w)
      ref=numpy.zeros((4, 5),numpy.float_)
      if dim==2:
        s[0,0]=(0.0355078592116)*x[0]+(-1.45322051958)*x[1]
        s[0,1]=(0.940649610535)*x[0]+(1.03783851181)*x[1]
        s[0,2]=(-0.285465603086)*x[0]+(-1.0279436539)*x[1]
        s[0,3]=(-1.05683604808)*x[0]+(-0.606790848933)*x[1]
        s[0,4]=(0.155725423448)*x[0]+(-0.0350991646899)*x[1]
        s[1,0]=(0.288671804193)*x[0]+(0.427933920845)*x[1]
        s[1,1]=(-0.0117065079514)*x[0]+(0.96117958225)*x[1]
        s[1,2]=(0.209419996754)*x[0]+(0.21574344987)*x[1]
        s[1,3]=(0.885781216051)*x[0]+(-0.68144471245)*x[1]
        s[1,4]=(-1.08228199466)*x[0]+(-0.155862749309)*x[1]
        s[2,0]=(-0.680644329614)*x[0]+(-1.38778484373)*x[1]
        s[2,1]=(0.0400929370385)*x[0]+(0.415428765055)*x[1]
        s[2,2]=(-1.20113508704)*x[0]+(-0.876683252298)*x[1]
        s[2,3]=(-0.312684596061)*x[0]+(-0.70367059229)*x[1]
        s[2,4]=(0.214059442951)*x[0]+(1.42513598809)*x[1]
        s[3,0]=(-0.24713062126)*x[0]+(1.02991464042)*x[1]
        s[3,1]=(-0.83089702409)*x[0]+(0.587925745944)*x[1]
        s[3,2]=(0.950881339072)*x[0]+(0.051413472076)*x[1]
        s[3,3]=(0.182953617513)*x[0]+(0.94052327152)*x[1]
        s[3,4]=(-0.387816734481)*x[0]+(0.366589363018)*x[1]
        ref[0,0]=(-0.563896045541)/(o+1.)+(-0.436916928584)+(0.0200172423412)*0.5**o
        ref[0,1]=(0.583712973931)/(o+1.)+(0.277276607579)+(0.84022193326)*0.5**o
        ref[0,2]=(-0.381794683559)/(o+1.)+(-0.29471407057)+(-0.342186432287)*0.5**o
        ref[0,3]=(0.0937197621552)/(o+1.)+(-0.417759182193)+(-0.921828294785)*0.5**o
        ref[0,4]=(-0.00732587396541)/(o+1.)+(0.282507495385)+(-0.437062858047)*0.5**o
        ref[1,0]=(0.656328782568)/(o+1.)+(0.105444180081)+(-0.150611417692)*0.5**o
        ref[1,1]=(0.0862490471738)/(o+1.)+(-0.0358008007742)+(0.934825628673)*0.5**o
        ref[1,2]=(-0.681994265679)/(o+1.)+(0.52707424273)+(0.053009226843)*0.5**o
        ref[1,3]=(0.162048053458)/(o+1.)+(-0.131185707156)+(0.304659864456)*0.5**o
        ref[1,4]=(-0.321375806412)/(o+1.)+(-0.12650756243)+(-0.663753812697)*0.5**o
        ref[2,0]=(-0.706054805711)/(o+1.)+(-0.805432593593)+(0.248490819557)*0.5**o
        ref[2,1]=(0.587809899116)/(o+1.)+(-0.0542554861756)+(-0.0237772246706)*0.5**o
        ref[2,2]=(0.044861811334)/(o+1.)+(-0.933527479587)+(-0.255625191498)*0.5**o
        ref[2,3]=(0.0752678267397)/(o+1.)+(-0.281993375709)+(-0.527636263672)*0.5**o
        ref[2,4]=(0.684054536011)/(o+1.)+(0.209350429588)+(0.536440035851)*0.5**o
        ref[3,0]=(0.827817530316)/(o+1.)+(-0.211443979463)+(0.37785444777)*0.5**o
        ref[3,1]=(-0.410871022592)/(o+1.)+(0.0530892951706)+(0.061721154105)*0.5**o
        ref[3,2]=(0.199285164042)/(o+1.)+(0.122582366509)+(0.557844914088)*0.5**o
        ref[3,3]=(0.195462635242)/(o+1.)+(0.027388051617)+(0.873238150557)*0.5**o
        ref[3,4]=(0.231128200422)/(o+1.)+(0.0245220097267)+(-0.301399591338)*0.5**o
      else:
        s[0,0]=(0.995116002112)*x[0]+(0.591812853606)*x[1]+(0.436070205257)*x[2]
        s[0,1]=(-0.118607336615)*x[0]+(0.594691767995)*x[1]+(-0.56002370897)*x[2]
        s[0,2]=(1.14520644838)*x[0]+(0.18460040375)*x[1]+(-0.960923519429)*x[2]
        s[0,3]=(0.392881958399)*x[0]+(-1.48738000894)*x[1]+(1.04349887295)*x[2]
        s[0,4]=(0.232444347548)*x[0]+(-0.83025700349)*x[1]+(0.612703936854)*x[2]
        s[1,0]=(0.190581558342)*x[0]+(-0.619707808792)*x[1]+(1.05804982356)*x[2]
        s[1,1]=(-0.42811715218)*x[0]+(-0.447035862369)*x[1]+(-1.24096539043)*x[2]
        s[1,2]=(0.200357664023)*x[0]+(-0.936410127129)*x[1]+(0.750394727948)*x[2]
        s[1,3]=(-0.612730981377)*x[0]+(-0.653699725295)*x[1]+(0.0015189792328)*x[2]
        s[1,4]=(0.288644454156)*x[0]+(0.93337277237)*x[1]+(-0.602600880867)*x[2]
        s[2,0]=(-1.17236877273)*x[0]+(1.45496830635)*x[1]+(-1.12383998094)*x[2]
        s[2,1]=(-0.232230799241)*x[0]+(0.415001155159)*x[1]+(-0.60248885989)*x[2]
        s[2,2]=(-0.83830117983)*x[0]+(0.192640431036)*x[1]+(1.43556749653)*x[2]
        s[2,3]=(0.973424823298)*x[0]+(0.369914981625)*x[1]+(-1.40643757872)*x[2]
        s[2,4]=(-0.458923994338)*x[0]+(-1.04785209961)*x[1]+(-0.592533584766)*x[2]
        s[3,0]=(0.545327477107)*x[0]+(1.9382915447)*x[1]+(1.47179032667)*x[2]
        s[3,1]=(-0.415696009498)*x[0]+(-0.594177620783)*x[1]+(-1.35108058064)*x[2]
        s[3,2]=(0.831178664645)*x[0]+(-0.0275877269769)*x[1]+(0.302241965885)*x[2]
        s[3,3]=(-0.360815522042)*x[0]+(-0.826462125533)*x[1]+(0.530840397679)*x[2]
        s[3,4]=(0.135928452703)*x[0]+(-1.51122126726)*x[1]+(1.6120424041)*x[2]
        ref[0,0]=(1.24742647646)/(o+1.)+(0.269060816234)+(0.237450952046)*0.5**o
        ref[0,1]=(1.12585957806)/(o+1.)+(-0.755828198463)+(0.301857541273)*0.5**o
        ref[0,2]=(-0.435989664675)/(o+1.)+(0.295471183049)+(0.213930631278)*0.5**o
        ref[0,3]=(-0.488921466935)/(o+1.)+(0.294480448259)+(-0.151038607178)*0.5**o
        ref[0,4]=(0.516230612817)/(o+1.)+(-0.0787984362944)+(-0.343742459316)*0.5**o
        ref[1,0]=(-0.492183705773)/(o+1.)+(0.501230271461)+(0.118646735962)*0.5**o
        ref[1,1]=(-1.43995574312)/(o+1.)+(-0.28437529362)+(-0.107412074623)*0.5**o
        ref[1,2]=(-0.101017880825)/(o+1.)+(0.28028462641)+(-0.445209107154)*0.5**o
        ref[1,3]=(0.128471434955)/(o+1.)+(-0.384371968608)+(-0.62463922518)*0.5**o
        ref[1,4]=(-0.0918241851534)/(o+1.)+(-0.0317649881343)+(0.774770507081)*0.5**o
        ref[2,0]=(0.221291051677)/(o+1.)+(-0.302519614649)+(-0.457492269704)*0.5**o
        ref[2,1]=(0.100276041699)/(o+1.)+(-0.387935114184)+(0.255875682695)*0.5**o
        ref[2,2]=(0.67334801807)/(o+1.)+(0.363637415491)+(-0.610716101317)*0.5**o
        ref[2,3]=(-0.101257531168)/(o+1.)+(-0.225887466089)+(0.48993468955)*0.5**o
        ref[2,4]=(-0.451085705642)/(o+1.)+(-0.415483032516)+(-0.817257908036)*0.5**o
        ref[3,0]=(1.87076639925)/(o+1.)+(1.20828670015)+(-0.331930451075)*0.5**o
        ref[3,1]=(-1.43689409978)/(o+1.)+(0.0171611774403)+(-0.958382466017)*0.5**o
        ref[3,2]=(0.146538702473)/(o+1.)+(0.391890493041)+(0.175513214998)*0.5**o
        ref[3,3]=(-0.122934034211)/(o+1.)+(0.140609173665)+(-0.814721563016)*0.5**o
        ref[3,4]=(0.223164895319)/(o+1.)+(-0.31244215916)+(0.638469012551)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactOne_fromSymbol_ReducedSolution_rank3(self):
      """
      tests integral of rank 3 Symbol on the FunctionOnContactOne

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports integral on FunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(6, 2, 2))
      s=Data(0,(6, 2, 2),w)
      ref=numpy.zeros((6, 2, 2),numpy.float_)
      if dim==2:
        s[0,0,0]=(-0.35078097593)*x[0]+(0.306926522015)*x[1]
        s[0,0,1]=(-0.803534689121)*x[0]+(-0.0602880575367)*x[1]
        s[0,1,0]=(-0.903470951227)*x[0]+(0.374123966765)*x[1]
        s[0,1,1]=(-1.18641387512)*x[0]+(0.273430931834)*x[1]
        s[1,0,0]=(0.493295479958)*x[0]+(1.54668330013)*x[1]
        s[1,0,1]=(-1.27848534954)*x[0]+(0.0588492485708)*x[1]
        s[1,1,0]=(-1.42953381447)*x[0]+(0.717393314307)*x[1]
        s[1,1,1]=(-0.467730806598)*x[0]+(-0.0117815181086)*x[1]
        s[2,0,0]=(-0.602626661024)*x[0]+(0.533324435346)*x[1]
        s[2,0,1]=(-0.904855697799)*x[0]+(1.09112846759)*x[1]
        s[2,1,0]=(1.20360337188)*x[0]+(1.23097020284)*x[1]
        s[2,1,1]=(0.0885447539403)*x[0]+(0.405387510263)*x[1]
        s[3,0,0]=(-0.730436778413)*x[0]+(0.416740386984)*x[1]
        s[3,0,1]=(-1.04126166563)*x[0]+(0.41753641298)*x[1]
        s[3,1,0]=(-0.271195830488)*x[0]+(-0.11575841172)*x[1]
        s[3,1,1]=(1.16100889666)*x[0]+(-0.233929048364)*x[1]
        s[4,0,0]=(0.664549807695)*x[0]+(0.224807977257)*x[1]
        s[4,0,1]=(-0.796935211563)*x[0]+(0.365917637053)*x[1]
        s[4,1,0]=(0.0401977690429)*x[0]+(-0.612500495333)*x[1]
        s[4,1,1]=(-0.115172707709)*x[0]+(-0.280626474812)*x[1]
        s[5,0,0]=(0.844835935845)*x[0]+(-1.19344222698)*x[1]
        s[5,0,1]=(-1.81553680308)*x[0]+(-1.66095998384)*x[1]
        s[5,1,0]=(0.761230206372)*x[0]+(0.547313314376)*x[1]
        s[5,1,1]=(-0.246895651321)*x[0]+(0.391870038086)*x[1]
        ref[0,0,0]=(-0.600965801098)/(o+1.)+(0.338185808482)+(-0.119260269781)*0.5**o
        ref[0,0,1]=(-0.224867141426)/(o+1.)+(-0.334457471517)+(0.0299593378011)*0.5**o
        ref[0,1,0]=(0.0129821011627)/(o+1.)+(-0.0950564992708)+(-0.352216087084)*0.5**o
        ref[0,1,1]=(0.0332057375276)/(o+1.)+(-0.0551431610872)+(-0.835902358643)*0.5**o
        ref[1,0,0]=(0.553424856182)/(o+1.)+(0.848186123918)+(-0.209818323927)*0.5**o
        ref[1,0,1]=(-0.75127641037)/(o+1.)+(-0.0481529718983)+(-0.372053746804)*0.5**o
        ref[1,1,0]=(-0.218691700126)/(o+1.)+(0.0665842087159)+(-0.626617217469)*0.5**o
        ref[1,1,1]=(-0.751752416764)/(o+1.)+(0.165004967668)+(-0.0577698432779)*0.5**o
        ref[2,0,0]=(-0.0172781449053)/(o+1.)+(-0.197949307988)+(0.343874535203)*0.5**o
        ref[2,0,1]=(0.499625949394)/(o+1.)+(-0.0639621511173)+(-0.185428877371)*0.5**o
        ref[2,1,0]=(0.965002845499)/(o+1.)+(0.296405176523)+(0.87676037618)*0.5**o
        ref[2,1,1]=(0.571427043323)/(o+1.)+(0.355568778058)+(-0.788632335234)*0.5**o
        ref[3,0,0]=(0.643869395266)/(o+1.)+(-0.248603098629)+(-0.460359589438)*0.5**o
        ref[3,0,1]=(0.847552650465)/(o+1.)+(-0.471374980899)+(-0.528527941319)*0.5**o
        ref[3,1,0]=(0.792482290139)/(o+1.)+(-0.288835326975)+(-0.601765878397)*0.5**o
        ref[3,1,1]=(0.674315208514)/(o+1.)+(-0.301924186259)+(0.8566130123)*0.5**o
        ref[4,0,0]=(0.14482725218)/(o+1.)+(0.123667770392)+(0.497194991988)*0.5**o
        ref[4,0,1]=(0.207166197869)/(o+1.)+(-0.00140080206462)+(-0.63538216825)*0.5**o
        ref[4,1,0]=(0.134064796438)/(o+1.)+(-0.583980524751)+(0.461593526773)*0.5**o
        ref[4,1,1]=(0.329865203686)/(o+1.)+(-0.0277930553577)+(-0.670078275492)*0.5**o
        ref[5,0,0]=(-0.671043415594)/(o+1.)+(-0.0772895763368)+(0.477016277134)*0.5**o
        ref[5,0,1]=(-0.863660332362)/(o+1.)+(-0.862182891111)+(-0.888470672335)*0.5**o
        ref[5,1,0]=(0.734550839405)/(o+1.)+(-0.00320324272275)+(0.580399166789)*0.5**o
        ref[5,1,1]=(-0.186529971258)/(o+1.)+(0.409102503591)+(-0.486700649159)*0.5**o
      else:
        s[0,0,0]=(1.49896775273)*x[0]+(-1.78250968743)*x[1]+(1.41503337092)*x[2]
        s[0,0,1]=(-0.643818266916)*x[0]+(-0.375134170105)*x[1]+(0.650514899885)*x[2]
        s[0,1,0]=(-0.0528798419185)*x[0]+(0.665778795831)*x[1]+(1.21069663928)*x[2]
        s[0,1,1]=(-1.45464226351)*x[0]+(-0.19467401007)*x[1]+(0.442665314437)*x[2]
        s[1,0,0]=(-0.836797292596)*x[0]+(-0.0542158346848)*x[1]+(-0.54209479287)*x[2]
        s[1,0,1]=(-1.09189713442)*x[0]+(1.03696758011)*x[1]+(0.0722113737424)*x[2]
        s[1,1,0]=(-0.493099825558)*x[0]+(-0.914461955024)*x[1]+(1.50580577272)*x[2]
        s[1,1,1]=(-0.702256378373)*x[0]+(-1.55374444057)*x[1]+(0.187304514552)*x[2]
        s[2,0,0]=(-0.924126823838)*x[0]+(-1.20470503473)*x[1]+(-0.702166397011)*x[2]
        s[2,0,1]=(1.04661135244)*x[0]+(0.995774871402)*x[1]+(0.912661077993)*x[2]
        s[2,1,0]=(0.0694812274474)*x[0]+(-0.384400463672)*x[1]+(-1.27290008863)*x[2]
        s[2,1,1]=(-0.196348771836)*x[0]+(0.314419668134)*x[1]+(-0.147174148507)*x[2]
        s[3,0,0]=(0.555982166527)*x[0]+(1.26542592993)*x[1]+(1.94075238034)*x[2]
        s[3,0,1]=(-0.15925841434)*x[0]+(0.235963126853)*x[1]+(1.01719057653)*x[2]
        s[3,1,0]=(1.8766116496)*x[0]+(0.0416539079261)*x[1]+(0.692631820852)*x[2]
        s[3,1,1]=(-1.60594678123)*x[0]+(0.418185984317)*x[1]+(1.00380643717)*x[2]
        s[4,0,0]=(-0.496656665568)*x[0]+(-0.519100502539)*x[1]+(-0.402964765219)*x[2]
        s[4,0,1]=(0.810723701732)*x[0]+(-0.9121518445)*x[1]+(0.0693075114891)*x[2]
        s[4,1,0]=(-1.38980929277)*x[0]+(-0.135080762705)*x[1]+(-0.708040071312)*x[2]
        s[4,1,1]=(0.0366128814416)*x[0]+(-0.598343408328)*x[1]+(-1.05172726191)*x[2]
        s[5,0,0]=(0.626645586525)*x[0]+(0.368702379272)*x[1]+(1.06419625419)*x[2]
        s[5,0,1]=(0.823542930912)*x[0]+(-0.397291307792)*x[1]+(1.47677515709)*x[2]
        s[5,1,0]=(1.0044323164)*x[0]+(-0.400460024166)*x[1]+(0.0881220069594)*x[2]
        s[5,1,1]=(0.841418294962)*x[0]+(0.448350914459)*x[1]+(-0.535982888316)*x[2]
        ref[0,0,0]=(0.0352781393333)/(o+1.)+(0.227326591212)+(0.64156011446)*0.5**o
        ref[0,0,1]=(-0.122693252863)/(o+1.)+(-0.221154887636)+(0.196565490999)*0.5**o
        ref[0,1,0]=(0.784542061966)/(o+1.)+(0.731576295626)+(-0.424099060024)*0.5**o
        ref[0,1,1]=(0.118776006377)/(o+1.)+(-0.423066319495)+(-0.479294326528)*0.5**o
        ref[1,0,0]=(1.18086837157)/(o+1.)+(-1.14308640082)+(-0.327803490075)*0.5**o
        ref[1,0,1]=(1.11878511573)/(o+1.)+(-0.427948355911)+(-0.245606584475)*0.5**o
        ref[1,1,0]=(-0.256395599928)/(o+1.)+(0.22748609258)+(-0.100332593094)*0.5**o
        ref[1,1,1]=(-0.192853498607)/(o+1.)+(-0.925785926894)+(-0.0242709519946)*0.5**o
        ref[2,0,0]=(-0.415752649176)/(o+1.)+(-1.14572035003)+(-0.123804906338)*0.5**o
        ref[2,0,1]=(0.384199373535)/(o+1.)+(0.843162532691)+(0.884522862915)*0.5**o
        ref[2,1,0]=(-0.229595030341)/(o+1.)+(-1.14751601921)+(0.936807743902)*0.5**o
        ref[2,1,1]=(0.267255066685)/(o+1.)+(-0.491644195523)+(0.686930072154)*0.5**o
        ref[3,0,0]=(1.68424865452)/(o+1.)+(0.955116898624)+(0.167678025029)*0.5**o
        ref[3,0,1]=(0.0034339126924)/(o+1.)+(0.718824562093)+(-0.347187747836)*0.5**o
        ref[3,1,0]=(1.62164005566)/(o+1.)+(0.0562907078128)+(0.876675907094)*0.5**o
        ref[3,1,1]=(0.472424872079)/(o+1.)+(0.0308594354746)+(-0.718098102775)*0.5**o
        ref[4,0,0]=(-0.0285810697935)/(o+1.)+(-0.525693855415)+(-0.338753152702)*0.5**o
        ref[4,0,1]=(0.303057697082)/(o+1.)+(-0.350175453973)+(0.365172579586)*0.5**o
        ref[4,1,0]=(-1.04637813553)/(o+1.)+(-0.308024248014)+(-0.57050349523)*0.5**o
        ref[4,1,1]=(-0.500793745646)/(o+1.)+(-0.424344892355)+(-0.26397425844)*0.5**o
        ref[5,0,0]=(0.365122763119)/(o+1.)+(0.568706597744)+(0.557008261376)*0.5**o
        ref[5,0,1]=(0.510379226755)/(o+1.)+(0.436754108223)+(0.519139337008)*0.5**o
        ref[5,1,0]=(-0.200129774692)/(o+1.)+(0.274983568325)+(0.342256937234)*0.5**o
        ref[5,1,1]=(0.566120139584)/(o+1.)+(-0.312318326061)+(0.812302833644)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactOne_fromSymbol_ReducedSolution_rank4(self):
      """
      tests integral of rank 4 Symbol on the FunctionOnContactOne

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports integral on FunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5, 3, 2))
      s=Data(0,(4, 5, 3, 2),w)
      ref=numpy.zeros((4, 5, 3, 2),numpy.float_)
      if dim==2:
        s[0,0,0,0]=(-0.36635416566)*x[0]+(0.454278450476)*x[1]
        s[0,0,0,1]=(0.449553060759)*x[0]+(-0.226772438798)*x[1]
        s[0,0,1,0]=(0.648343714053)*x[0]+(0.233007941149)*x[1]
        s[0,0,1,1]=(-0.135283076943)*x[0]+(1.61027554283)*x[1]
        s[0,0,2,0]=(0.619161973877)*x[0]+(-0.112220917964)*x[1]
        s[0,0,2,1]=(0.857902398581)*x[0]+(1.51928597673)*x[1]
        s[0,1,0,0]=(0.331827670107)*x[0]+(1.40775973855)*x[1]
        s[0,1,0,1]=(0.421840987115)*x[0]+(0.118563659741)*x[1]
        s[0,1,1,0]=(0.447268593931)*x[0]+(0.484984994669)*x[1]
        s[0,1,1,1]=(-0.189018546367)*x[0]+(0.467542988869)*x[1]
        s[0,1,2,0]=(0.466123494496)*x[0]+(-0.713267013868)*x[1]
        s[0,1,2,1]=(1.14105709142)*x[0]+(-0.134715682223)*x[1]
        s[0,2,0,0]=(0.871597208182)*x[0]+(-1.49080786854)*x[1]
        s[0,2,0,1]=(-0.94888847468)*x[0]+(0.0275999018363)*x[1]
        s[0,2,1,0]=(0.77534514887)*x[0]+(1.67162256756)*x[1]
        s[0,2,1,1]=(-0.786790506628)*x[0]+(-0.0692869424346)*x[1]
        s[0,2,2,0]=(1.78371863881)*x[0]+(-0.0481919137551)*x[1]
        s[0,2,2,1]=(-0.992074926087)*x[0]+(-1.02372593432)*x[1]
        s[0,3,0,0]=(0.113042447336)*x[0]+(-0.0790352303856)*x[1]
        s[0,3,0,1]=(1.12030253557)*x[0]+(0.393484268041)*x[1]
        s[0,3,1,0]=(0.379936292181)*x[0]+(-1.53869802379)*x[1]
        s[0,3,1,1]=(-0.191443496207)*x[0]+(0.563534464341)*x[1]
        s[0,3,2,0]=(0.104758691419)*x[0]+(-0.99439187659)*x[1]
        s[0,3,2,1]=(-0.479774987108)*x[0]+(-0.142765773584)*x[1]
        s[0,4,0,0]=(-1.50474690932)*x[0]+(-0.507691997905)*x[1]
        s[0,4,0,1]=(-0.78875103976)*x[0]+(0.132625919742)*x[1]
        s[0,4,1,0]=(-0.692847502554)*x[0]+(0.437815890612)*x[1]
        s[0,4,1,1]=(-0.808194455413)*x[0]+(0.376369555557)*x[1]
        s[0,4,2,0]=(0.919158598696)*x[0]+(-1.18391426084)*x[1]
        s[0,4,2,1]=(0.42374001855)*x[0]+(0.812305143875)*x[1]
        s[1,0,0,0]=(0.218364606199)*x[0]+(-1.40515806247)*x[1]
        s[1,0,0,1]=(-0.351603222911)*x[0]+(1.45105178061)*x[1]
        s[1,0,1,0]=(1.39849984972)*x[0]+(-1.35867734934)*x[1]
        s[1,0,1,1]=(-0.00360307599185)*x[0]+(0.246277359726)*x[1]
        s[1,0,2,0]=(0.514944968042)*x[0]+(0.428253029894)*x[1]
        s[1,0,2,1]=(-0.786120897197)*x[0]+(0.920810959626)*x[1]
        s[1,1,0,0]=(-0.602179171574)*x[0]+(0.515213890072)*x[1]
        s[1,1,0,1]=(-0.0861102153588)*x[0]+(0.477359706262)*x[1]
        s[1,1,1,0]=(-0.688164903151)*x[0]+(-0.313549483397)*x[1]
        s[1,1,1,1]=(-0.429990373791)*x[0]+(-0.0543708786137)*x[1]
        s[1,1,2,0]=(-0.39223682603)*x[0]+(0.796504408378)*x[1]
        s[1,1,2,1]=(-1.69046328113)*x[0]+(-0.344952802268)*x[1]
        s[1,2,0,0]=(-0.528152430512)*x[0]+(-0.269147355)*x[1]
        s[1,2,0,1]=(-1.68741664082)*x[0]+(0.568028242365)*x[1]
        s[1,2,1,0]=(-0.076694122635)*x[0]+(0.512745790897)*x[1]
        s[1,2,1,1]=(-0.738719294653)*x[0]+(1.72566985283)*x[1]
        s[1,2,2,0]=(0.679026798185)*x[0]+(-0.766391547753)*x[1]
        s[1,2,2,1]=(-1.7766436376)*x[0]+(-0.626962152238)*x[1]
        s[1,3,0,0]=(-0.0629592766364)*x[0]+(-0.602623643371)*x[1]
        s[1,3,0,1]=(1.46050982285)*x[0]+(-1.29912311111)*x[1]
        s[1,3,1,0]=(0.565052310821)*x[0]+(0.451146130683)*x[1]
        s[1,3,1,1]=(-1.72352086869)*x[0]+(-0.729969658338)*x[1]
        s[1,3,2,0]=(-0.699562180077)*x[0]+(0.785838373001)*x[1]
        s[1,3,2,1]=(0.0504742963364)*x[0]+(-0.856596707973)*x[1]
        s[1,4,0,0]=(-0.220685313017)*x[0]+(1.49415344926)*x[1]
        s[1,4,0,1]=(0.965835473202)*x[0]+(-0.664523896748)*x[1]
        s[1,4,1,0]=(-0.643560593026)*x[0]+(-1.06826452446)*x[1]
        s[1,4,1,1]=(-0.347993256951)*x[0]+(0.240926576999)*x[1]
        s[1,4,2,0]=(-0.666305003293)*x[0]+(1.5567540195)*x[1]
        s[1,4,2,1]=(0.752122481773)*x[0]+(-0.940078899457)*x[1]
        s[2,0,0,0]=(-0.428935565836)*x[0]+(-0.471904780642)*x[1]
        s[2,0,0,1]=(0.157225979958)*x[0]+(-0.707126471203)*x[1]
        s[2,0,1,0]=(-0.127873936162)*x[0]+(-1.01452234653)*x[1]
        s[2,0,1,1]=(-1.43314905378)*x[0]+(1.09406218538)*x[1]
        s[2,0,2,0]=(0.125475529763)*x[0]+(-0.144317966895)*x[1]
        s[2,0,2,1]=(0.781409858349)*x[0]+(-0.27747495012)*x[1]
        s[2,1,0,0]=(-1.21917608112)*x[0]+(-0.785716389383)*x[1]
        s[2,1,0,1]=(-0.26082261117)*x[0]+(-0.376402142009)*x[1]
        s[2,1,1,0]=(-0.125232448715)*x[0]+(0.261783624866)*x[1]
        s[2,1,1,1]=(-1.25985888818)*x[0]+(-0.0711437757878)*x[1]
        s[2,1,2,0]=(-0.300629742217)*x[0]+(-0.46939686121)*x[1]
        s[2,1,2,1]=(-1.77973256304)*x[0]+(0.713178700308)*x[1]
        s[2,2,0,0]=(-0.900064650204)*x[0]+(0.507949604762)*x[1]
        s[2,2,0,1]=(-0.885252734166)*x[0]+(-0.154018162942)*x[1]
        s[2,2,1,0]=(-0.7944479569)*x[0]+(1.77390521085)*x[1]
        s[2,2,1,1]=(1.1725888609)*x[0]+(0.662244216625)*x[1]
        s[2,2,2,0]=(-0.236310280848)*x[0]+(-0.542661345078)*x[1]
        s[2,2,2,1]=(1.15526522118)*x[0]+(0.0247167490822)*x[1]
        s[2,3,0,0]=(0.175989767802)*x[0]+(1.43963679604)*x[1]
        s[2,3,0,1]=(-0.692773425778)*x[0]+(-0.297594800061)*x[1]
        s[2,3,1,0]=(-0.847486832282)*x[0]+(0.0810250645286)*x[1]
        s[2,3,1,1]=(-0.681973624741)*x[0]+(-1.24971452577)*x[1]
        s[2,3,2,0]=(-1.06756371509)*x[0]+(0.977349108627)*x[1]
        s[2,3,2,1]=(0.846764513647)*x[0]+(-0.547998096826)*x[1]
        s[2,4,0,0]=(0.634255041776)*x[0]+(0.339363455414)*x[1]
        s[2,4,0,1]=(-1.39093420159)*x[0]+(0.716421880025)*x[1]
        s[2,4,1,0]=(0.257472890363)*x[0]+(-0.197982551362)*x[1]
        s[2,4,1,1]=(-0.45529259808)*x[0]+(-0.761148165674)*x[1]
        s[2,4,2,0]=(0.955343907343)*x[0]+(-1.29615358423)*x[1]
        s[2,4,2,1]=(0.362935017626)*x[0]+(1.25503750319)*x[1]
        s[3,0,0,0]=(0.179219886497)*x[0]+(1.78329032848)*x[1]
        s[3,0,0,1]=(0.43735751276)*x[0]+(0.0403938192717)*x[1]
        s[3,0,1,0]=(1.11478662344)*x[0]+(-0.240063001264)*x[1]
        s[3,0,1,1]=(0.172583585916)*x[0]+(0.506428670417)*x[1]
        s[3,0,2,0]=(1.33494651584)*x[0]+(0.781224884171)*x[1]
        s[3,0,2,1]=(0.0432954454652)*x[0]+(-0.68034279444)*x[1]
        s[3,1,0,0]=(-0.856892482415)*x[0]+(0.836687116891)*x[1]
        s[3,1,0,1]=(0.487431626464)*x[0]+(-0.148084918499)*x[1]
        s[3,1,1,0]=(0.331122282784)*x[0]+(-1.38429248259)*x[1]
        s[3,1,1,1]=(-0.215232051361)*x[0]+(0.537089578691)*x[1]
        s[3,1,2,0]=(1.24485548633)*x[0]+(0.509639847837)*x[1]
        s[3,1,2,1]=(-0.321450199534)*x[0]+(0.211384054649)*x[1]
        s[3,2,0,0]=(0.355385603273)*x[0]+(0.928139739798)*x[1]
        s[3,2,0,1]=(0.927589198094)*x[0]+(1.38324753996)*x[1]
        s[3,2,1,0]=(-1.16347040084)*x[0]+(-0.351854201414)*x[1]
        s[3,2,1,1]=(0.717488068131)*x[0]+(-0.0977120861443)*x[1]
        s[3,2,2,0]=(-0.799786558433)*x[0]+(0.829503505995)*x[1]
        s[3,2,2,1]=(1.37899235577)*x[0]+(-1.77910476445)*x[1]
        s[3,3,0,0]=(0.763361907215)*x[0]+(-0.861336281243)*x[1]
        s[3,3,0,1]=(0.476903208654)*x[0]+(0.903897305941)*x[1]
        s[3,3,1,0]=(1.60374028575)*x[0]+(-0.315620315575)*x[1]
        s[3,3,1,1]=(0.972809255696)*x[0]+(0.363233068227)*x[1]
        s[3,3,2,0]=(0.602799894079)*x[0]+(1.20772970348)*x[1]
        s[3,3,2,1]=(-0.000443665606339)*x[0]+(-1.6279763946)*x[1]
        s[3,4,0,0]=(-1.26676161275)*x[0]+(0.148063159145)*x[1]
        s[3,4,0,1]=(0.322891919608)*x[0]+(-1.08096025287)*x[1]
        s[3,4,1,0]=(-0.0944277709182)*x[0]+(-0.824212826197)*x[1]
        s[3,4,1,1]=(1.13530096846)*x[0]+(0.217321895058)*x[1]
        s[3,4,2,0]=(1.25663262459)*x[0]+(-0.308861390599)*x[1]
        s[3,4,2,1]=(-0.379182493224)*x[0]+(0.331110509538)*x[1]
        ref[0,0,0,0]=(0.408958259232)/(o+1.)+(0.247137449101)+(-0.815308872619)*0.5**o
        ref[0,0,0,1]=(0.13228131757)/(o+1.)+(0.165819155398)+(-0.241139006405)*0.5**o
        ref[0,0,1,0]=(0.128399251556)/(o+1.)+(0.371321245227)+(0.0103099131928)*0.5**o
        ref[0,0,1,1]=(0.966281299065)/(o+1.)+(0.381611325047)+(-0.254511483272)*0.5**o
        ref[0,0,2,0]=(-0.200167961434)/(o+1.)+(-0.0928181112535)+(0.892745239853)*0.5**o
        ref[0,0,2,1]=(0.830787814329)/(o+1.)+(0.314595496988)+(0.917209567011)*0.5**o
        ref[0,1,0,0]=(0.612992604133)/(o+1.)+(0.66241770717)+(-0.19824060982)*0.5**o
        ref[0,1,0,1]=(-0.0163161609645)/(o+1.)+(0.2025662158)+(0.151588376221)*0.5**o
        ref[0,1,1,0]=(-0.126917561445)/(o+1.)+(0.630548380093)+(-0.201925610142)*0.5**o
        ref[0,1,1,1]=(0.819096391825)/(o+1.)+(-0.370043224043)+(0.199514498764)*0.5**o
        ref[0,1,2,0]=(-0.96777130286)/(o+1.)+(0.367817797326)+(-0.0150078111633)*0.5**o
        ref[0,1,2,1]=(0.045654631235)/(o+1.)+(0.396367346781)+(0.167952084398)*0.5**o
        ref[0,2,0,0]=(-0.784765277575)/(o+1.)+(-0.298358907675)+(0.762272432566)*0.5**o
        ref[0,2,0,1]=(0.386572441281)/(o+1.)+(-0.51566858578)+(-0.276523842565)*0.5**o
        ref[0,2,1,0]=(0.680861847658)/(o+1.)+(0.740328104187)+(0.285449660402)*0.5**o
        ref[0,2,1,1]=(0.325836705829)/(o+1.)+(-0.49316269795)+(-0.195588758992)*0.5**o
        ref[0,2,2,0]=(0.783589269167)/(o+1.)+(0.0810542535881)+(0.789828948707)*0.5**o
        ref[0,2,2,1]=(-0.370218192335)/(o+1.)+(-0.608350209702)+(-0.428882248665)*0.5**o
        ref[0,3,0,0]=(0.188865301872)/(o+1.)+(-0.465976959809)+(0.777095834696)*0.5**o
        ref[0,3,0,1]=(0.764156203059)/(o+1.)+(-0.0928154457191)+(0.935261491991)*0.5**o
        ref[0,3,1,0]=(-0.689450927985)/(o+1.)+(-0.664550854911)+(0.8597909062)*0.5**o
        ref[0,3,1,1]=(0.555793140817)/(o+1.)+(0.142885227944)+(-0.469472628571)*0.5**o
        ref[0,3,2,0]=(-0.625798760976)/(o+1.)+(-0.499725272457)+(0.735616120719)*0.5**o
        ref[0,3,2,1]=(-0.907491695474)/(o+1.)+(0.385181050937)+(-0.485411167091)*0.5**o
        ref[0,4,0,0]=(-0.879336661239)/(o+1.)+(-0.205953419637)+(-0.721195406712)*0.5**o
        ref[0,4,0,1]=(0.86693894952)/(o+1.)+(-0.819677383073)+(0.11629069661)*0.5**o
        ref[0,4,1,0]=(-0.310289123464)/(o+1.)+(0.401826760004)+(-0.748396008487)*0.5**o
        ref[0,4,1,1]=(0.931547956812)/(o+1.)+(-0.740019832824)+(0.116666808981)*0.5**o
        ref[0,4,2,0]=(-0.847165088314)/(o+1.)+(-0.124403947896)+(0.831217321961)*0.5**o
        ref[0,4,2,1]=(0.177734973756)/(o+1.)+(0.441537508054)+(0.175235172561)*0.5**o
        ref[1,0,0,0]=(-0.542365809895)/(o+1.)+(-0.0827972838196)+(-0.478833078736)*0.5**o
        ref[1,0,0,1]=(0.844666854947)/(o+1.)+(0.498180748967)+(-0.741579795179)*0.5**o
        ref[1,0,1,0]=(-0.491126713824)/(o+1.)+(0.0335621009267)+(0.463825012354)*0.5**o
        ref[1,0,1,1]=(-0.309670980539)/(o+1.)+(0.472409867826)+(-0.392474471379)*0.5**o
        ref[1,0,2,0]=(0.278634540091)/(o+1.)+(-0.121141274647)+(0.906846007138)*0.5**o
        ref[1,0,2,1]=(-0.00603476025805)/(o+1.)+(0.182935877942)+(-0.225146933197)*0.5**o
        ref[1,1,0,0]=(0.580918965968)/(o+1.)+(0.0504878873855)+(-0.768860022241)*0.5**o
        ref[1,1,0,1]=(0.631736797799)/(o+1.)+(-0.537539597912)+(0.834591888928)*0.5**o
        ref[1,1,1,0]=(0.291226625025)/(o+1.)+(-0.703641010727)+(0.11434100988)*0.5**o
        ref[1,1,1,1]=(-0.909671911939)/(o+1.)+(0.237461939347)+(-0.0496132191606)*0.5**o
        ref[1,1,2,0]=(0.21073170992)/(o+1.)+(0.325715980645)+(-0.457896088863)*0.5**o
        ref[1,1,2,1]=(-0.725096486574)/(o+1.)+(-0.296742654874)+(-0.716834287076)*0.5**o
        ref[1,2,0,0]=(-0.164851515918)/(o+1.)+(-0.100676749276)+(-0.431094771042)*0.5**o
        ref[1,2,0,1]=(0.294840872181)/(o+1.)+(-0.329953166746)+(-0.754322937148)*0.5**o
        ref[1,2,1,0]=(0.341767621978)/(o+1.)+(0.400623115147)+(-0.70696218401)*0.5**o
        ref[1,2,1,1]=(0.821671220024)/(o+1.)+(0.193132433528)+(-0.220985528909)*0.5**o
        ref[1,2,2,0]=(-0.135508676597)/(o+1.)+(-0.38093100269)+(0.81000593241)*0.5**o
        ref[1,2,2,1]=(0.101637058171)/(o+1.)+(-0.771937440463)+(-0.961367967078)*0.5**o
        ref[1,3,0,0]=(-0.354951988253)/(o+1.)+(-0.41833260094)+(0.526034270126)*0.5**o
        ref[1,3,0,1]=(-0.461724677474)/(o+1.)+(-0.082145170992)+(0.787401731204)*0.5**o
        ref[1,3,1,0]=(0.681652074623)/(o+1.)+(-0.127199759209)+(0.5889458853)*0.5**o
        ref[1,3,1,1]=(-0.816769655641)/(o+1.)+(-0.382750879464)+(-0.871219112459)*0.5**o
        ref[1,3,2,0]=(0.549856768471)/(o+1.)+(-0.136659604755)+(-0.190261366036)*0.5**o
        ref[1,3,2,1]=(-0.298435559802)/(o+1.)+(-0.364033867566)+(0.220380883298)*0.5**o
        ref[1,4,0,0]=(0.692982064919)/(o+1.)+(0.445763556713)+(-0.311041042098)*0.5**o
        ref[1,4,0,1]=(-0.0284802774195)/(o+1.)+(0.173564544909)+(-0.0173372359445)*0.5**o
        ref[1,4,1,0]=(-0.521190661286)/(o+1.)+(-0.19266811161)+(-0.805298232979)*0.5**o
        ref[1,4,1,1]=(-0.344391605752)/(o+1.)+(0.344434513106)+(-0.451544100412)*0.5**o
        ref[1,4,2,0]=(0.590871123139)/(o+1.)+(0.123309088073)+(0.0529597169219)*0.5**o
        ref[1,4,2,1]=(-0.00996351412971)/(o+1.)+(-0.328705279006)+(0.479417654458)*0.5**o
        ref[2,0,0,0]=(0.220674489827)/(o+1.)+(-0.618024232079)+(0.114533627854)*0.5**o
        ref[2,0,0,1]=(-0.0844798976424)/(o+1.)+(0.070811636225)+(-0.607043866052)*0.5**o
        ref[2,0,1,0]=(-0.445683449242)/(o+1.)+(-0.113440437004)+(-0.469831959437)*0.5**o
        ref[2,0,1,1]=(0.271633369975)/(o+1.)+(0.132535807211)+(-0.875791852797)*0.5**o
        ref[2,0,2,0]=(0.649542525852)/(o+1.)+(0.0590165361575)+(-0.786418035298)*0.5**o
        ref[2,0,2,1]=(-0.317621366014)/(o+1.)+(0.329998426635)+(0.161559420974)*0.5**o
        ref[2,1,0,0]=(-0.835806177468)/(o+1.)+(-0.214223776118)+(-0.7406387408)*0.5**o
        ref[2,1,0,1]=(0.429741516432)/(o+1.)+(-0.552565623152)+(0.0381649766927)*0.5**o
        ref[2,1,1,0]=(-0.0472916937886)/(o+1.)+(-0.00552013887145)+(0.194883147682)*0.5**o
        ref[2,1,1,1]=(-0.199584048679)/(o+1.)+(-0.27639819939)+(-0.578622216512)*0.5**o
        ref[2,1,2,0]=(-0.941086888016)/(o+1.)+(-0.0385375072603)+(0.24813529911)*0.5**o
        ref[2,1,2,1]=(-0.0868737022971)/(o+1.)+(-0.0970507102796)+(-0.785578739872)*0.5**o
        ref[2,2,0,0]=(-0.202279119459)/(o+1.)+(-0.0839554337907)+(-0.0219250584016)*0.5**o
        ref[2,2,0,1]=(-0.184000614671)/(o+1.)+(-0.477928353442)+(0.100586424448)*0.5**o
        ref[2,2,1,0]=(0.927675340201)/(o+1.)+(0.0186691089268)+(0.0144436958933)*0.5**o
        ref[2,2,1,1]=(0.304792611717)/(o+1.)+(0.459166286092)+(0.611707893628)*0.5**o
        ref[2,2,2,0]=(0.0815687138062)/(o+1.)+(-0.768726979684)+(0.676913619635)*0.5**o
        ref[2,2,2,1]=(0.849269282518)/(o+1.)+(-0.25478410885)+(0.840280905449)*0.5**o
        ref[2,3,0,0]=(0.820793959083)/(o+1.)+(0.575169789593)+(-0.355506974424)*0.5**o
        ref[2,3,0,1]=(0.283603638973)/(o+1.)+(-0.335480628352)+(-0.603010608107)*0.5**o
        ref[2,3,1,0]=(-0.381013144765)/(o+1.)+(0.190974767434)+(-0.767398157855)*0.5**o
        ref[2,3,1,1]=(-0.773551356194)/(o+1.)+(-0.620890671367)+(0.0836445484134)*0.5**o
        ref[2,3,2,0]=(0.570212356041)/(o+1.)+(-0.289714757955)+(-0.0809974465983)*0.5**o
        ref[2,3,2,1]=(0.21581143181)/(o+1.)+(-0.415400501134)+(0.913755987278)*0.5**o
        ref[2,4,0,0]=(0.323568653669)/(o+1.)+(0.0799583453359)+(0.490133152849)*0.5**o
        ref[2,4,0,1]=(0.532408915634)/(o+1.)+(-0.115548404995)+(-0.97582442721)*0.5**o
        ref[2,4,1,0]=(-0.82114168932)/(o+1.)+(0.702564400414)+(-0.524496772508)*0.5**o
        ref[2,4,1,1]=(-0.479224200985)/(o+1.)+(-0.49364246599)+(0.25006836921)*0.5**o
        ref[2,4,2,0]=(-0.389740913016)/(o+1.)+(-0.0884282111182)+(0.225787658364)*0.5**o
        ref[2,4,2,1]=(0.508843816312)/(o+1.)+(0.17685868642)+(0.755411331668)*0.5**o
        ref[3,0,0,0]=(0.954468971244)/(o+1.)+(0.671012220464)+(-0.333983197198)*0.5**o
        ref[3,0,0,1]=(-0.312850718368)/(o+1.)+(-0.0723748107028)+(0.935351671806)*0.5**o
        ref[3,0,1,0]=(0.0640122675963)/(o+1.)+(0.220683721613)+(0.36934391135)*0.5**o
        ref[3,0,1,1]=(0.855735115439)/(o+1.)+(-0.366272230844)+(0.555821602582)*0.5**o
        ref[3,0,2,0]=(0.0417837662591)/(o+1.)+(0.556307682679)+(0.961772268394)*0.5**o
        ref[3,0,2,1]=(-0.928375029285)/(o+1.)+(-0.133425494561)+(0.558178669432)*0.5**o
        ref[3,1,0,0]=(0.610808931498)/(o+1.)+(-0.00371515949037)+(-0.623583978041)*0.5**o
        ref[3,1,0,1]=(-0.274141402005)/(o+1.)+(0.180852020544)+(0.251784068882)*0.5**o
        ref[3,1,1,0]=(-0.481032880244)/(o+1.)+(-0.705203882431)+(0.838270445302)*0.5**o
        ref[3,1,1,1]=(0.389002730435)/(o+1.)+(0.371327956101)+(-0.809801115307)*0.5**o
        ref[3,1,2,0]=(-0.217052039031)/(o+1.)+(0.504898297656)+(0.961750777882)*0.5**o
        ref[3,1,2,1]=(0.983589747751)/(o+1.)+(-0.768560013671)+(0.443464134708)*0.5**o
        ref[3,2,0,0]=(0.0850720894356)/(o+1.)+(0.295360321829)+(0.607732609977)*0.5**o
        ref[3,2,0,1]=(0.812866987322)/(o+1.)+(0.672388816065)+(0.153192118599)*0.5**o
        ref[3,2,1,0]=(-0.947419039593)/(o+1.)+(-0.179746473508)+(-0.208412615647)*0.5**o
        ref[3,2,1,1]=(-0.694332599484)/(o+1.)+(0.574069301711)+(0.165969978048)*0.5**o
        ref[3,2,2,0]=(0.746797880687)/(o+1.)+(0.0731831278015)+(-0.863447188728)*0.5**o
        ref[3,2,2,1]=(-0.9955274799)/(o+1.)+(-0.100141136686)+(0.795697344588)*0.5**o
        ref[3,3,0,0]=(-0.874362496111)/(o+1.)+(0.185559038089)+(0.405270045904)*0.5**o
        ref[3,3,0,1]=(0.557583002677)/(o+1.)+(0.556451155279)+(-0.28968479864)*0.5**o
        ref[3,3,1,0]=(0.435966507789)/(o+1.)+(0.00561393027505)+(0.840925601832)*0.5**o
        ref[3,3,1,1]=(-0.277310702324)/(o+1.)+(0.808938926899)+(-0.00452482755147)*0.5**o
        ref[3,3,2,0]=(0.878069034303)/(o+1.)+(0.0250230902204)+(0.882414382818)*0.5**o
        ref[3,3,2,1]=(-0.953670613054)/(o+1.)+(-0.626136165517)+(0.577522883879)*0.5**o
        ref[3,4,0,0]=(-0.0572166464991)/(o+1.)+(-0.195764673671)+(-0.669952459767)*0.5**o
        ref[3,4,0,1]=(-0.611801368818)/(o+1.)+(0.23313248955)+(-0.612531943539)*0.5**o
        ref[3,4,1,0]=(0.0451584859358)/(o+1.)+(-0.693097871519)+(0.422396659987)*0.5**o
        ref[3,4,1,1]=(0.0616735716897)/(o+1.)+(0.360358324628)+(0.57023264257)*0.5**o
        ref[3,4,2,0]=(0.404353246317)/(o+1.)+(0.02838608211)+(0.486645823449)*0.5**o
        ref[3,4,2,1]=(0.0039199584997)/(o+1.)+(0.344800813187)+(-0.741593568559)*0.5**o
      else:
        s[0,0,0,0]=(-0.00215071746669)*x[0]+(-0.332788203774)*x[1]+(-1.01240104354)*x[2]
        s[0,0,0,1]=(0.534102846033)*x[0]+(0.432162550346)*x[1]+(0.382807104545)*x[2]
        s[0,0,1,0]=(-0.774071131493)*x[0]+(0.174280314187)*x[1]+(0.235599208299)*x[2]
        s[0,0,1,1]=(1.66560900439)*x[0]+(1.38276801197)*x[1]+(0.451912961751)*x[2]
        s[0,0,2,0]=(-0.308642285607)*x[0]+(0.925763055269)*x[1]+(1.30533127575)*x[2]
        s[0,0,2,1]=(-0.15617828399)*x[0]+(-0.405834652149)*x[1]+(-0.846888299486)*x[2]
        s[0,1,0,0]=(-0.0743207250799)*x[0]+(-0.0831917437929)*x[1]+(-1.78311394968)*x[2]
        s[0,1,0,1]=(1.24178762381)*x[0]+(-0.121417608169)*x[1]+(0.0528284855272)*x[2]
        s[0,1,1,0]=(-1.14437209159)*x[0]+(0.0618138540822)*x[1]+(0.19124990065)*x[2]
        s[0,1,1,1]=(0.854246128467)*x[0]+(1.56688838996)*x[1]+(1.30424035038)*x[2]
        s[0,1,2,0]=(1.3184765993)*x[0]+(-1.40883832059)*x[1]+(0.832393276179)*x[2]
        s[0,1,2,1]=(-1.1625604886)*x[0]+(-0.104956544515)*x[1]+(-1.52389901089)*x[2]
        s[0,2,0,0]=(-0.106003547387)*x[0]+(0.852941663718)*x[1]+(0.24003638122)*x[2]
        s[0,2,0,1]=(0.892312082297)*x[0]+(-0.156996856451)*x[1]+(-1.2674975556)*x[2]
        s[0,2,1,0]=(0.0300043771968)*x[0]+(-0.848942993281)*x[1]+(-0.322425526609)*x[2]
        s[0,2,1,1]=(-0.361971760465)*x[0]+(1.89870303113)*x[1]+(0.861574517617)*x[2]
        s[0,2,2,0]=(0.97819437229)*x[0]+(-0.132714911288)*x[1]+(-0.957702168325)*x[2]
        s[0,2,2,1]=(0.588644003365)*x[0]+(-0.568265700558)*x[1]+(0.150521572866)*x[2]
        s[0,3,0,0]=(0.700807304996)*x[0]+(-1.14422226798)*x[1]+(0.587223436541)*x[2]
        s[0,3,0,1]=(-0.862702931786)*x[0]+(0.423016472206)*x[1]+(-0.555427243683)*x[2]
        s[0,3,1,0]=(0.574966038173)*x[0]+(-0.487199028698)*x[1]+(-0.32389521042)*x[2]
        s[0,3,1,1]=(0.130862033386)*x[0]+(0.42056248135)*x[1]+(0.925321527155)*x[2]
        s[0,3,2,0]=(-0.735071923108)*x[0]+(-0.528993360596)*x[1]+(0.485622476133)*x[2]
        s[0,3,2,1]=(0.126763646072)*x[0]+(0.0323222279682)*x[1]+(1.42515358228)*x[2]
        s[0,4,0,0]=(-0.334189936412)*x[0]+(0.14123257798)*x[1]+(-0.487379116976)*x[2]
        s[0,4,0,1]=(-0.528967405148)*x[0]+(-0.0622502130673)*x[1]+(-1.22107834734)*x[2]
        s[0,4,1,0]=(-1.67869230303)*x[0]+(-0.427809293008)*x[1]+(0.35027727689)*x[2]
        s[0,4,1,1]=(-1.3023324224)*x[0]+(0.364963594938)*x[1]+(1.25798740064)*x[2]
        s[0,4,2,0]=(0.789575151987)*x[0]+(-1.08985193505)*x[1]+(0.286072474971)*x[2]
        s[0,4,2,1]=(0.0321971599533)*x[0]+(1.64269617809)*x[1]+(0.448773378519)*x[2]
        s[1,0,0,0]=(-0.206961758788)*x[0]+(0.188339936734)*x[1]+(-1.36499979129)*x[2]
        s[1,0,0,1]=(-0.439106473295)*x[0]+(-1.35694867727)*x[1]+(0.17565861577)*x[2]
        s[1,0,1,0]=(0.754962041339)*x[0]+(0.377298691237)*x[1]+(-0.225684064888)*x[2]
        s[1,0,1,1]=(-0.164606093877)*x[0]+(-1.12165840203)*x[1]+(0.475143697931)*x[2]
        s[1,0,2,0]=(-0.563561093763)*x[0]+(-0.605592098451)*x[1]+(-0.382814067375)*x[2]
        s[1,0,2,1]=(-0.529311859218)*x[0]+(-0.440753365779)*x[1]+(-0.213919010047)*x[2]
        s[1,1,0,0]=(-0.381937118975)*x[0]+(0.115015087)*x[1]+(-1.58208233129)*x[2]
        s[1,1,0,1]=(-0.898803250164)*x[0]+(1.21794054101)*x[1]+(0.000708470416828)*x[2]
        s[1,1,1,0]=(0.417980628363)*x[0]+(-0.89943521283)*x[1]+(1.18981658848)*x[2]
        s[1,1,1,1]=(0.400600150308)*x[0]+(1.59168041332)*x[1]+(0.526855874334)*x[2]
        s[1,1,2,0]=(-0.658036770065)*x[0]+(0.903572546571)*x[1]+(-0.0322095817377)*x[2]
        s[1,1,2,1]=(-0.487427224544)*x[0]+(0.25991247483)*x[1]+(1.71801479005)*x[2]
        s[1,2,0,0]=(-1.23648267242)*x[0]+(-1.14397314877)*x[1]+(1.51420721169)*x[2]
        s[1,2,0,1]=(1.76604962545)*x[0]+(0.183730664401)*x[1]+(0.119413404231)*x[2]
        s[1,2,1,0]=(-0.371372970825)*x[0]+(1.67376148418)*x[1]+(-1.4673890303)*x[2]
        s[1,2,1,1]=(-1.05767796861)*x[0]+(1.57674524903)*x[1]+(0.907317505281)*x[2]
        s[1,2,2,0]=(-0.560171709945)*x[0]+(0.932731419096)*x[1]+(-1.34845422611)*x[2]
        s[1,2,2,1]=(0.299743316165)*x[0]+(1.27560261018)*x[1]+(-0.835084977966)*x[2]
        s[1,3,0,0]=(-0.255337437533)*x[0]+(-0.597132326112)*x[1]+(-0.136201264537)*x[2]
        s[1,3,0,1]=(0.593835766145)*x[0]+(-0.799771329645)*x[1]+(-0.980158768361)*x[2]
        s[1,3,1,0]=(1.38678662193)*x[0]+(0.145756993166)*x[1]+(-0.687830261701)*x[2]
        s[1,3,1,1]=(-0.535937612709)*x[0]+(-0.0191554551358)*x[1]+(0.360231645604)*x[2]
        s[1,3,2,0]=(0.464446491173)*x[0]+(0.601115829679)*x[1]+(-1.23243462932)*x[2]
        s[1,3,2,1]=(-1.00252524049)*x[0]+(0.38542247592)*x[1]+(-0.786817371152)*x[2]
        s[1,4,0,0]=(0.127520587286)*x[0]+(0.0193180871908)*x[1]+(0.443880703635)*x[2]
        s[1,4,0,1]=(0.459087583966)*x[0]+(-0.0133413082011)*x[1]+(-0.469574441754)*x[2]
        s[1,4,1,0]=(-0.0408745863162)*x[0]+(-0.450729520372)*x[1]+(0.49494190957)*x[2]
        s[1,4,1,1]=(0.698205417361)*x[0]+(0.475698414334)*x[1]+(-0.738661347545)*x[2]
        s[1,4,2,0]=(1.20797915788)*x[0]+(-0.132247786402)*x[1]+(-0.282227294584)*x[2]
        s[1,4,2,1]=(-0.76998141325)*x[0]+(-0.0677377690684)*x[1]+(-0.0836477753317)*x[2]
        s[2,0,0,0]=(-1.10579544966)*x[0]+(1.40458332668)*x[1]+(0.666829139108)*x[2]
        s[2,0,0,1]=(0.316772911983)*x[0]+(-0.722315825552)*x[1]+(-0.420491453418)*x[2]
        s[2,0,1,0]=(1.28978684389)*x[0]+(-1.31024104856)*x[1]+(-0.909892235182)*x[2]
        s[2,0,1,1]=(0.516882102505)*x[0]+(0.137837613805)*x[1]+(-0.0746054680439)*x[2]
        s[2,0,2,0]=(-0.684536226359)*x[0]+(1.26314901343)*x[1]+(-0.80088731126)*x[2]
        s[2,0,2,1]=(1.04894156753)*x[0]+(-0.292514668329)*x[1]+(0.189452265734)*x[2]
        s[2,1,0,0]=(-1.89697799819)*x[0]+(0.842415025171)*x[1]+(1.50175587499)*x[2]
        s[2,1,0,1]=(-0.132472949872)*x[0]+(0.755809155321)*x[1]+(1.4210576902)*x[2]
        s[2,1,1,0]=(-1.20812267424)*x[0]+(0.776283189748)*x[1]+(0.466844337822)*x[2]
        s[2,1,1,1]=(-0.545404145415)*x[0]+(0.630064653938)*x[1]+(0.416024592778)*x[2]
        s[2,1,2,0]=(-1.25450160322)*x[0]+(0.712099345608)*x[1]+(-1.40448708108)*x[2]
        s[2,1,2,1]=(0.8675638655)*x[0]+(-0.694288742412)*x[1]+(0.423917775454)*x[2]
        s[2,2,0,0]=(0.611369389916)*x[0]+(-0.282854928933)*x[1]+(0.421814371036)*x[2]
        s[2,2,0,1]=(0.87143261059)*x[0]+(0.626319549038)*x[1]+(-0.247486033705)*x[2]
        s[2,2,1,0]=(-1.09640818879)*x[0]+(-0.625440705278)*x[1]+(1.06488909366)*x[2]
        s[2,2,1,1]=(-1.63825859868)*x[0]+(0.243702450075)*x[1]+(0.37689118934)*x[2]
        s[2,2,2,0]=(1.14554562168)*x[0]+(-0.656545182621)*x[1]+(0.122881381407)*x[2]
        s[2,2,2,1]=(1.04057428387)*x[0]+(0.605147014643)*x[1]+(-0.0624571196862)*x[2]
        s[2,3,0,0]=(-0.687211572887)*x[0]+(-1.33387647369)*x[1]+(1.56461600602)*x[2]
        s[2,3,0,1]=(-0.439360840188)*x[0]+(-0.175050010481)*x[1]+(-0.493088639437)*x[2]
        s[2,3,1,0]=(1.01647468205)*x[0]+(1.607854298)*x[1]+(0.154041122248)*x[2]
        s[2,3,1,1]=(-1.17279615871)*x[0]+(0.265464982509)*x[1]+(0.536445292976)*x[2]
        s[2,3,2,0]=(-0.370774714547)*x[0]+(-0.140305952222)*x[1]+(-0.27755251782)*x[2]
        s[2,3,2,1]=(-0.601589093641)*x[0]+(-0.0914485078099)*x[1]+(-0.246092622851)*x[2]
        s[2,4,0,0]=(1.16222773615)*x[0]+(-1.0556446667)*x[1]+(1.18749640554)*x[2]
        s[2,4,0,1]=(0.441185917779)*x[0]+(0.106062084171)*x[1]+(-0.150699537615)*x[2]
        s[2,4,1,0]=(1.24765736189)*x[0]+(-1.28446862056)*x[1]+(-1.15581931924)*x[2]
        s[2,4,1,1]=(0.751269018864)*x[0]+(1.03373528238)*x[1]+(0.953542233964)*x[2]
        s[2,4,2,0]=(-0.507055271742)*x[0]+(-0.139767333963)*x[1]+(0.369122007484)*x[2]
        s[2,4,2,1]=(1.75708904398)*x[0]+(0.229784892513)*x[1]+(0.138439119614)*x[2]
        s[3,0,0,0]=(0.488579497744)*x[0]+(0.227848351934)*x[1]+(-0.503567606843)*x[2]
        s[3,0,0,1]=(1.48513008903)*x[0]+(0.0119808614235)*x[1]+(0.50138290689)*x[2]
        s[3,0,1,0]=(1.89811058679)*x[0]+(0.936442809385)*x[1]+(0.618927531336)*x[2]
        s[3,0,1,1]=(0.520596564131)*x[0]+(-1.34097734027)*x[1]+(-1.24288925352)*x[2]
        s[3,0,2,0]=(-1.22679833631)*x[0]+(0.946723115385)*x[1]+(-0.444825501575)*x[2]
        s[3,0,2,1]=(-0.786039767784)*x[0]+(1.01276854439)*x[1]+(0.868428470305)*x[2]
        s[3,1,0,0]=(-1.14928294841)*x[0]+(1.83212625566)*x[1]+(-0.131295307828)*x[2]
        s[3,1,0,1]=(0.530939875091)*x[0]+(-0.00936464244878)*x[1]+(-0.54397285021)*x[2]
        s[3,1,1,0]=(0.776394250264)*x[0]+(-0.0432924713397)*x[1]+(-0.48992936123)*x[2]
        s[3,1,1,1]=(-0.295285841654)*x[0]+(1.60838322278)*x[1]+(-0.822857254214)*x[2]
        s[3,1,2,0]=(-0.445325691811)*x[0]+(0.143992033055)*x[1]+(-0.562336605856)*x[2]
        s[3,1,2,1]=(-0.424042508176)*x[0]+(-0.119602696473)*x[1]+(0.50936759192)*x[2]
        s[3,2,0,0]=(0.0376712305683)*x[0]+(-0.127442186223)*x[1]+(-0.98477458245)*x[2]
        s[3,2,0,1]=(-0.964194007245)*x[0]+(-0.951386585236)*x[1]+(-0.376434083193)*x[2]
        s[3,2,1,0]=(-0.645708363662)*x[0]+(0.729523237957)*x[1]+(-0.815980951689)*x[2]
        s[3,2,1,1]=(1.54280019708)*x[0]+(0.347142929119)*x[1]+(0.117050155931)*x[2]
        s[3,2,2,0]=(-0.70653056148)*x[0]+(0.658564301815)*x[1]+(1.5766896863)*x[2]
        s[3,2,2,1]=(1.28238648298)*x[0]+(0.415173079215)*x[1]+(1.25492135064)*x[2]
        s[3,3,0,0]=(-0.907422225702)*x[0]+(0.354252510498)*x[1]+(-0.617733757576)*x[2]
        s[3,3,0,1]=(-0.687653032786)*x[0]+(0.166147005719)*x[1]+(0.0668202207665)*x[2]
        s[3,3,1,0]=(-0.515227984328)*x[0]+(-1.20687483216)*x[1]+(0.325703771711)*x[2]
        s[3,3,1,1]=(-0.239992988401)*x[0]+(-0.625429153195)*x[1]+(-0.987302012149)*x[2]
        s[3,3,2,0]=(0.48686562504)*x[0]+(-0.687077509478)*x[1]+(0.630530432518)*x[2]
        s[3,3,2,1]=(0.575785040914)*x[0]+(0.978122193974)*x[1]+(0.0753481784122)*x[2]
        s[3,4,0,0]=(-0.201304266512)*x[0]+(-0.563373770577)*x[1]+(-0.0445045378284)*x[2]
        s[3,4,0,1]=(0.348899505512)*x[0]+(-0.983874875078)*x[1]+(0.561390754019)*x[2]
        s[3,4,1,0]=(0.687251502909)*x[0]+(0.806428209931)*x[1]+(-0.848431263257)*x[2]
        s[3,4,1,1]=(-0.0690481011866)*x[0]+(-0.637464706806)*x[1]+(1.32335195878)*x[2]
        s[3,4,2,0]=(0.640286948745)*x[0]+(-1.44112416361)*x[1]+(-0.724716414365)*x[2]
        s[3,4,2,1]=(-0.967100763601)*x[0]+(0.0347001879437)*x[1]+(-0.888183629447)*x[2]
        ref[0,0,0,0]=(-1.47634692775)/(o+1.)+(0.413823374944)+(-0.69863978692)*0.5**o
        ref[0,0,0,1]=(0.467438056347)/(o+1.)+(-0.049348895708)+(0.980332235993)*0.5**o
        ref[0,0,1,0]=(0.0324608468542)/(o+1.)+(0.227219571463)+(-0.851091598789)*0.5**o
        ref[0,0,1,1]=(1.24663216229)/(o+1.)+(0.680736038169)+(0.892185739473)*0.5**o
        ref[0,0,2,0]=(1.04128811669)/(o+1.)+(0.801526892617)+(-0.721889856517)*0.5**o
        ref[0,0,2,1]=(-1.57019640933)/(o+1.)+(-0.0149100811556)+(0.191115336015)*0.5**o
        ref[0,1,0,0]=(-1.06234362705)/(o+1.)+(-0.881380501065)+(0.88447821062)*0.5**o
        ref[0,1,0,1]=(0.267287676352)/(o+1.)+(0.0934259729578)+(0.719058878904)*0.5**o
        ref[0,1,1,0]=(-0.83257335756)/(o+1.)+(0.056558158197)+(-0.171851295688)*0.5**o
        ref[0,1,1,1]=(1.59747135641)/(o+1.)+(1.00837611714)+(0.111151278122)*0.5**o
        ref[0,1,2,0]=(0.181694306894)/(o+1.)+(-0.0366161862313)+(0.633569620457)*0.5**o
        ref[0,1,2,1]=(-0.116206059658)/(o+1.)+(-1.20967709253)+(-0.255855799289)*0.5**o
        ref[0,2,0,0]=(0.615063090083)/(o+1.)+(-0.0944203632267)+(0.560752133921)*0.5**o
        ref[0,2,0,1]=(-1.68718452323)/(o+1.)+(0.415922156486)+(0.323157880499)*0.5**o
        ref[0,2,1,0]=(-1.38436480215)/(o+1.)+(0.514201265835)+(-0.785401872214)*0.5**o
        ref[0,2,1,1]=(1.07260243754)/(o+1.)+(1.03208581185)+(-0.738468272962)*0.5**o
        ref[0,2,2,0]=(-0.619469154652)/(o+1.)+(0.156513881249)+(0.19421868483)*0.5**o
        ref[0,2,2,1]=(0.742851517381)/(o+1.)+(-0.577857666542)+(0.583763691375)*0.5**o
        ref[0,3,0,0]=(-0.0209287146839)/(o+1.)+(-0.0417684031135)+(0.248273994472)*0.5**o
        ref[0,3,0,1]=(-0.876757092127)/(o+1.)+(0.156654815475)+(-0.431666242085)*0.5**o
        ref[0,3,1,0]=(-1.40731912274)/(o+1.)+(0.295731262157)+(0.579728397479)*0.5**o
        ref[0,3,1,1]=(0.766713725705)/(o+1.)+(0.520952712789)+(-0.331873109391)*0.5**o
        ref[0,3,2,0]=(-1.37493161295)/(o+1.)+(0.626269025128)+(-0.656049244875)*0.5**o
        ref[0,3,2,1]=(0.738828256274)/(o+1.)+(0.299922102653)+(0.245566994744)*0.5**o
        ref[0,4,0,0]=(-0.220320428485)/(o+1.)+(-0.381104395776)+(0.30219274463)*0.5**o
        ref[0,4,0,1]=(-1.71138026305)/(o+1.)+(0.0858133974006)+(-0.272542497315)*0.5**o
        ref[0,4,1,0]=(-0.0512711854771)/(o+1.)+(-0.398429115874)+(-0.908094901926)*0.5**o
        ref[0,4,1,1]=(0.775088175251)/(o+1.)+(0.153496176444)+(-0.76146195497)*0.5**o
        ref[0,4,2,0]=(-0.188892896681)/(o+1.)+(-0.212757962174)+(0.600204512934)*0.5**o
        ref[0,4,2,1]=(0.786135983408)/(o+1.)+(0.709443217024)+(-0.0813557008918)*0.5**o
        ref[1,0,0,0]=(-0.682877215956)/(o+1.)+(0.0430800979484)+(-0.786904593287)*0.5**o
        ref[1,0,0,1]=(-0.209743447547)/(o+1.)+(-0.529335530226)+(-0.351982026793)*0.5**o
        ref[1,0,1,0]=(1.7467904951)/(o+1.)+(-0.522734985482)+(0.205256143552)*0.5**o
        ref[1,0,1,1]=(-0.681949654071)/(o+1.)+(0.430192257885)+(-0.989555659669)*0.5**o
        ref[1,0,2,0]=(0.00559076557551)/(o+1.)+(-0.355981322114)+(-0.845595380938)*0.5**o
        ref[1,0,2,1]=(-1.0795557316)/(o+1.)+(0.433501713489)+(-0.971431930418)*0.5**o
        ref[1,1,0,0]=(-1.41228679449)/(o+1.)+(0.0214610301511)+(-0.47963962908)*0.5**o
        ref[1,1,0,1]=(0.675271467105)/(o+1.)+(0.258295057037)+(-0.872015819915)*0.5**o
        ref[1,1,1,0]=(-0.0723449260888)/(o+1.)+(0.642648455626)+(-0.504589981145)*0.5**o
        ref[1,1,1,1]=(1.69082429043)/(o+1.)+(0.351194447364)+(0.125923252805)*0.5**o
        ref[1,1,2,0]=(1.04100385874)/(o+1.)+(-0.253396731491)+(-0.320884200985)*0.5**o
        ref[1,1,2,1]=(1.44000111068)/(o+1.)+(0.167426476075)+(-0.284354022497)*0.5**o
        ref[1,2,0,0]=(0.762931449029)/(o+1.)+(-0.397841053459)+(-0.833497951611)*0.5**o
        ref[1,2,0,1]=(-0.353193148804)/(o+1.)+(0.729759415974)+(0.962868010939)*0.5**o
        ref[1,2,1,0]=(-0.0142626154573)/(o+1.)+(0.205452893539)+(-0.561643688565)*0.5**o
        ref[1,2,1,1]=(1.31669689043)/(o+1.)+(0.322616901969)+(-0.535545908666)*0.5**o
        ref[1,2,2,0]=(-0.720144981522)/(o+1.)+(-0.253513298251)+(0.251277061063)*0.5**o
        ref[1,2,2,1]=(0.171627338105)/(o+1.)+(0.304508788985)+(-0.0403839677009)*0.5**o
        ref[1,3,0,0]=(-0.385245021569)/(o+1.)+(0.0533837214678)+(-0.710193449549)*0.5**o
        ref[1,3,0,1]=(-1.66770468862)/(o+1.)+(0.0386999300625)+(0.404210496633)*0.5**o
        ref[1,3,1,0]=(0.1536232451)/(o+1.)+(-0.114369222402)+(0.919828553095)*0.5**o
        ref[1,3,1,1]=(0.279969861456)/(o+1.)+(0.199789198624)+(-0.874409680945)*0.5**o
        ref[1,3,2,0]=(-0.29883793799)/(o+1.)+(-0.179133605171)+(0.490232839865)*0.5**o
        ref[1,3,2,1]=(-0.20045707503)/(o+1.)+(-0.419654937503)+(-0.364153185689)*0.5**o
        ref[1,4,0,0]=(0.328269609451)/(o+1.)+(0.0976736768568)+(0.0671024149466)*0.5**o
        ref[1,4,0,1]=(0.70998344827)/(o+1.)+(-0.831520044753)+(0.929228475247)*0.5**o
        ref[1,4,1,0]=(1.14467737453)/(o+1.)+(-0.309242254267)+(-0.522855063115)*0.5**o
        ref[1,4,1,1]=(0.386397027677)/(o+1.)+(-0.152706345419)+(0.354258147311)*0.5**o
        ref[1,4,2,0]=(0.565989417809)/(o+1.)+(-0.0912836510331)+(0.410081961152)*0.5**o
        ref[1,4,2,1]=(-0.108228050157)/(o+1.)+(-0.382022658574)+(-0.0490935903447)*0.5**o
        ref[2,0,0,0]=(1.30816654656)/(o+1.)+(0.218850490887)+(-0.780250512205)*0.5**o
        ref[2,0,0,1]=(-0.072210934601)/(o+1.)+(-0.376572060485)+(-0.000679311415873)*0.5**o
        ref[2,0,1,0]=(-1.03731005117)/(o+1.)+(-0.369008331776)+(0.844980274869)*0.5**o
        ref[2,0,1,1]=(-0.37673073433)/(o+1.)+(0.147779920342)+(0.661285141912)*0.5**o
        ref[2,0,2,0]=(-0.126456722513)/(o+1.)+(-0.173173731574)+(0.250529661474)*0.5**o
        ref[2,0,2,1]=(0.269921444109)/(o+1.)+(0.256112039487)+(0.163733641849)*0.5**o
        ref[2,1,0,0]=(1.02033624645)/(o+1.)+(0.162073420045)+(-0.897290184569)*0.5**o
        ref[2,1,0,1]=(0.673075377238)/(o+1.)+(0.377904898438)+(0.615508721534)*0.5**o
        ref[2,1,1,0]=(-0.536750510163)/(o+1.)+(0.687164638156)+(-0.802573912815)*0.5**o
        ref[2,1,1,1]=(1.39522327504)/(o+1.)+(-0.168020484927)+(-0.558497203882)*0.5**o
        ref[2,1,2,0]=(-1.21446181317)/(o+1.)+(-0.219360528348)+(-0.293706468828)*0.5**o
        ref[2,1,2,1]=(0.352877691113)/(o+1.)+(0.0397286675094)+(0.16485787241)*0.5**o
        ref[2,2,0,0]=(-0.927761522597)/(o+1.)+(1.03238921925)+(-0.386688083878)*0.5**o
        ref[2,2,0,1]=(0.875914163347)/(o+1.)+(-0.0805989643579)+(0.535549891292)*0.5**o
        ref[2,2,1,0]=(-0.268027052879)/(o+1.)+(0.24501270411)+(-0.878958155751)*0.5**o
        ref[2,2,1,1]=(0.579267469532)/(o+1.)+(-0.419360478349)+(-0.758211472099)*0.5**o
        ref[2,2,2,0]=(-0.407532480549)/(o+1.)+(0.0236731772421)+(0.972067946527)*0.5**o
        ref[2,2,2,1]=(-0.848446514529)/(o+1.)+(0.986605187547)+(0.458500318263)*0.5**o
        ref[2,3,0,0]=(0.0139531165204)/(o+1.)+(0.0612674782454)+(-0.592960113562)*0.5**o
        ref[2,3,0,1]=(-0.434410189326)/(o+1.)+(0.0699472934754)+(-0.81298388773)*0.5**o
        ref[2,3,1,0]=(1.51687712753)/(o+1.)+(0.29518962448)+(0.671113725811)*0.5**o
        ref[2,3,1,1]=(-0.825018275156)/(o+1.)+(0.348395434187)+(-0.242658476441)*0.5**o
        ref[2,3,2,0]=(-0.129038376537)/(o+1.)+(0.0407049218889)+(-0.74100465183)*0.5**o
        ref[2,3,2,1]=(-0.61436863216)/(o+1.)+(0.0569955443553)+(-0.438752680853)*0.5**o
        ref[2,4,0,0]=(-0.186811568504)/(o+1.)+(0.324468688913)+(0.831953665661)*0.5**o
        ref[2,4,0,1]=(1.01471089788)/(o+1.)+(-0.404804936557)+(0.191447439569)*0.5**o
        ref[2,4,1,0]=(-0.833336708992)/(o+1.)+(-0.465420260939)+(0.571546652968)*0.5**o
        ref[2,4,1,1]=(1.48092456289)/(o+1.)+(0.548316338846)+(0.16098929463)*0.5**o
        ref[2,4,2,0]=(-1.20254311473)/(o+1.)+(0.778023417549)+(-0.631204318592)*0.5**o
        ref[2,4,2,1]=(0.672521404448)/(o+1.)+(0.265167920765)+(0.922455810129)*0.5**o
        ref[3,0,0,0]=(-0.400146773167)/(o+1.)+(0.151226335458)+(0.310554345086)*0.5**o
        ref[3,0,0,1]=(0.580039994556)/(o+1.)+(0.287523081772)+(0.843407699243)*0.5**o
        ref[3,0,1,0]=(0.930972332962)/(o+1.)+(0.768302886889)+(0.985902820768)*0.5**o
        ref[3,0,1,1]=(-0.991140747279)/(o+1.)+(-0.914680203238)+(0.757231124095)*0.5**o
        ref[3,0,2,0]=(-0.458287257967)/(o+1.)+(0.237995705359)+(-0.742604875255)*0.5**o
        ref[3,0,2,1]=(1.14385326779)/(o+1.)+(0.243645854371)+(-0.535987729627)*0.5**o
        ref[3,1,0,0]=(0.322354058507)/(o+1.)+(0.341162149011)+(-0.453130357106)*0.5**o
        ref[3,1,0,1]=(-1.78201691527)/(o+1.)+(1.04199074308)+(-0.324362188462)*0.5**o
        ref[3,1,1,0]=(-0.0578982797037)/(o+1.)+(-0.110933167998)+(0.522937033394)*0.5**o
        ref[3,1,1,1]=(-0.276009095212)/(o+1.)+(0.517113652949)+(-0.267978083771)*0.5**o
        ref[3,1,2,0]=(-0.023345636799)/(o+1.)+(-0.66878791133)+(0.497251194848)*0.5**o
        ref[3,1,2,1]=(1.2604140706)/(o+1.)+(-0.878310666552)+(0.461929649774)*0.5**o
        ref[3,2,0,0]=(-0.00211014675007)/(o+1.)+(-0.814091029436)+(0.555746667517)*0.5**o
        ref[3,2,0,1]=(-1.07650025357)/(o+1.)+(-0.26199073554)+(-0.691532951025)*0.5**o
        ref[3,2,1,0]=(-0.0770650645465)/(o+1.)+(-0.307619169254)+(-0.0398626743392)*0.5**o
        ref[3,2,1,1]=(1.12398638253)/(o+1.)+(-0.00626411324864)+(0.895535126103)*0.5**o
        ref[3,2,2,0]=(0.822758496096)/(o+1.)+(0.448681597953)+(-0.191398265373)*0.5**o
        ref[3,2,2,1]=(1.38402514586)/(o+1.)+(0.489672391609)+(0.589110983761)*0.5**o
        ref[3,3,0,0]=(0.626837013941)/(o+1.)+(-0.698370737203)+(-0.400999012316)*0.5**o
        ref[3,3,0,1]=(-0.819551723109)/(o+1.)+(0.588359566557)+(-0.811853216306)*0.5**o
        ref[3,3,1,0]=(-0.472944922304)/(o+1.)+(0.024839136659)+(-0.973132395795)*0.5**o
        ref[3,3,1,1]=(-0.674351724226)/(o+1.)+(-0.596697469314)+(0.0150225091087)*0.5**o
        ref[3,3,2,0]=(-0.653245573198)/(o+1.)+(0.590003731898)+(-0.0964433425189)*0.5**o
        ref[3,3,2,1]=(1.15741134315)/(o+1.)+(-0.128540702232)+(0.728925474609)*0.5**o
        ref[3,4,0,0]=(-0.745079746498)/(o+1.)+(-0.313462120963)+(0.562821413507)*0.5**o
        ref[3,4,0,1]=(-0.585232841208)/(o+1.)+(0.464633596697)+(-0.417618967733)*0.5**o
        ref[3,4,1,0]=(0.792309614109)/(o+1.)+(-0.478065247211)+(0.809069329896)*0.5**o
        ref[3,4,1,1]=(0.0536562315177)/(o+1.)+(-0.128980351611)+(0.821143622493)*0.5**o
        ref[3,4,2,0]=(-1.20913408459)/(o+1.)+(-0.144016051074)+(-0.0283874424941)*0.5**o
        ref[3,4,2,1]=(-1.12139150964)/(o+1.)+(-0.135856270303)+(-0.427480154859)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactOne_fromSymbol_ReducedContinuousFunction_rank0(self):
      """
      tests integral of rank 0 Symbol on the FunctionOnContactOne

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports integral on FunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=())
      s=Data(0,(),w)
      if dim==2:
        s=(-0.0123111028353)*x[0]+(-1.54593060503)*x[1]
        ref=(-0.883444958139)/(o+1.)+(-0.0326056141412)+(-0.609585521441)*0.5**o
      else:
        s=(0.465974344755)*x[0]+(-0.406015068796)*x[1]+(0.217033091571)*x[2]
        ref=(-0.964783366814)/(o+1.)+(0.787210317182)+(-0.332644900019)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactOne_fromSymbol_ReducedContinuousFunction_rank1(self):
      """
      tests integral of rank 1 Symbol on the FunctionOnContactOne

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports integral on FunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(2,))
      s=Data(0,(2,),w)
      ref=numpy.zeros((2,),numpy.float_)
      if dim==2:
        s[0]=(-1.01806458946)*x[0]+(-0.84209500794)*x[1]
        s[1]=(-0.372739669691)*x[0]+(0.123203057686)*x[1]
        ref[0]=(-0.910310319387)/(o+1.)+(-0.35457604)+(-0.240697198017)*0.5**o
        ref[1]=(0.817182634856)/(o+1.)+(-0.752992097502)+(0.439264948143)*0.5**o
      else:
        s[0]=(-0.262859740698)*x[0]+(0.398827301114)*x[1]+(-0.0121863377176)*x[2]
        s[1]=(0.533472917947)*x[0]+(-0.654130540619)*x[1]+(-0.155059686066)*x[2]
        ref[0]=(0.173661143447)/(o+1.)+(-0.0377591573443)+(0.0256383939398)*0.5**o
        ref[1]=(-1.0263552046)/(o+1.)+(0.265270569058)+(0.220096757743)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactOne_fromSymbol_ReducedContinuousFunction_rank2(self):
      """
      tests integral of rank 2 Symbol on the FunctionOnContactOne

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports integral on FunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5))
      s=Data(0,(4, 5),w)
      ref=numpy.zeros((4, 5),numpy.float_)
      if dim==2:
        s[0,0]=(0.0355078592116)*x[0]+(-1.45322051958)*x[1]
        s[0,1]=(0.940649610535)*x[0]+(1.03783851181)*x[1]
        s[0,2]=(-0.285465603086)*x[0]+(-1.0279436539)*x[1]
        s[0,3]=(-1.05683604808)*x[0]+(-0.606790848933)*x[1]
        s[0,4]=(0.155725423448)*x[0]+(-0.0350991646899)*x[1]
        s[1,0]=(0.288671804193)*x[0]+(0.427933920845)*x[1]
        s[1,1]=(-0.0117065079514)*x[0]+(0.96117958225)*x[1]
        s[1,2]=(0.209419996754)*x[0]+(0.21574344987)*x[1]
        s[1,3]=(0.885781216051)*x[0]+(-0.68144471245)*x[1]
        s[1,4]=(-1.08228199466)*x[0]+(-0.155862749309)*x[1]
        s[2,0]=(-0.680644329614)*x[0]+(-1.38778484373)*x[1]
        s[2,1]=(0.0400929370385)*x[0]+(0.415428765055)*x[1]
        s[2,2]=(-1.20113508704)*x[0]+(-0.876683252298)*x[1]
        s[2,3]=(-0.312684596061)*x[0]+(-0.70367059229)*x[1]
        s[2,4]=(0.214059442951)*x[0]+(1.42513598809)*x[1]
        s[3,0]=(-0.24713062126)*x[0]+(1.02991464042)*x[1]
        s[3,1]=(-0.83089702409)*x[0]+(0.587925745944)*x[1]
        s[3,2]=(0.950881339072)*x[0]+(0.051413472076)*x[1]
        s[3,3]=(0.182953617513)*x[0]+(0.94052327152)*x[1]
        s[3,4]=(-0.387816734481)*x[0]+(0.366589363018)*x[1]
        ref[0,0]=(-0.563896045541)/(o+1.)+(-0.436916928584)+(0.0200172423412)*0.5**o
        ref[0,1]=(0.583712973931)/(o+1.)+(0.277276607579)+(0.84022193326)*0.5**o
        ref[0,2]=(-0.381794683559)/(o+1.)+(-0.29471407057)+(-0.342186432287)*0.5**o
        ref[0,3]=(0.0937197621552)/(o+1.)+(-0.417759182193)+(-0.921828294785)*0.5**o
        ref[0,4]=(-0.00732587396541)/(o+1.)+(0.282507495385)+(-0.437062858047)*0.5**o
        ref[1,0]=(0.656328782568)/(o+1.)+(0.105444180081)+(-0.150611417692)*0.5**o
        ref[1,1]=(0.0862490471738)/(o+1.)+(-0.0358008007742)+(0.934825628673)*0.5**o
        ref[1,2]=(-0.681994265679)/(o+1.)+(0.52707424273)+(0.053009226843)*0.5**o
        ref[1,3]=(0.162048053458)/(o+1.)+(-0.131185707156)+(0.304659864456)*0.5**o
        ref[1,4]=(-0.321375806412)/(o+1.)+(-0.12650756243)+(-0.663753812697)*0.5**o
        ref[2,0]=(-0.706054805711)/(o+1.)+(-0.805432593593)+(0.248490819557)*0.5**o
        ref[2,1]=(0.587809899116)/(o+1.)+(-0.0542554861756)+(-0.0237772246706)*0.5**o
        ref[2,2]=(0.044861811334)/(o+1.)+(-0.933527479587)+(-0.255625191498)*0.5**o
        ref[2,3]=(0.0752678267397)/(o+1.)+(-0.281993375709)+(-0.527636263672)*0.5**o
        ref[2,4]=(0.684054536011)/(o+1.)+(0.209350429588)+(0.536440035851)*0.5**o
        ref[3,0]=(0.827817530316)/(o+1.)+(-0.211443979463)+(0.37785444777)*0.5**o
        ref[3,1]=(-0.410871022592)/(o+1.)+(0.0530892951706)+(0.061721154105)*0.5**o
        ref[3,2]=(0.199285164042)/(o+1.)+(0.122582366509)+(0.557844914088)*0.5**o
        ref[3,3]=(0.195462635242)/(o+1.)+(0.027388051617)+(0.873238150557)*0.5**o
        ref[3,4]=(0.231128200422)/(o+1.)+(0.0245220097267)+(-0.301399591338)*0.5**o
      else:
        s[0,0]=(0.995116002112)*x[0]+(0.591812853606)*x[1]+(0.436070205257)*x[2]
        s[0,1]=(-0.118607336615)*x[0]+(0.594691767995)*x[1]+(-0.56002370897)*x[2]
        s[0,2]=(1.14520644838)*x[0]+(0.18460040375)*x[1]+(-0.960923519429)*x[2]
        s[0,3]=(0.392881958399)*x[0]+(-1.48738000894)*x[1]+(1.04349887295)*x[2]
        s[0,4]=(0.232444347548)*x[0]+(-0.83025700349)*x[1]+(0.612703936854)*x[2]
        s[1,0]=(0.190581558342)*x[0]+(-0.619707808792)*x[1]+(1.05804982356)*x[2]
        s[1,1]=(-0.42811715218)*x[0]+(-0.447035862369)*x[1]+(-1.24096539043)*x[2]
        s[1,2]=(0.200357664023)*x[0]+(-0.936410127129)*x[1]+(0.750394727948)*x[2]
        s[1,3]=(-0.612730981377)*x[0]+(-0.653699725295)*x[1]+(0.0015189792328)*x[2]
        s[1,4]=(0.288644454156)*x[0]+(0.93337277237)*x[1]+(-0.602600880867)*x[2]
        s[2,0]=(-1.17236877273)*x[0]+(1.45496830635)*x[1]+(-1.12383998094)*x[2]
        s[2,1]=(-0.232230799241)*x[0]+(0.415001155159)*x[1]+(-0.60248885989)*x[2]
        s[2,2]=(-0.83830117983)*x[0]+(0.192640431036)*x[1]+(1.43556749653)*x[2]
        s[2,3]=(0.973424823298)*x[0]+(0.369914981625)*x[1]+(-1.40643757872)*x[2]
        s[2,4]=(-0.458923994338)*x[0]+(-1.04785209961)*x[1]+(-0.592533584766)*x[2]
        s[3,0]=(0.545327477107)*x[0]+(1.9382915447)*x[1]+(1.47179032667)*x[2]
        s[3,1]=(-0.415696009498)*x[0]+(-0.594177620783)*x[1]+(-1.35108058064)*x[2]
        s[3,2]=(0.831178664645)*x[0]+(-0.0275877269769)*x[1]+(0.302241965885)*x[2]
        s[3,3]=(-0.360815522042)*x[0]+(-0.826462125533)*x[1]+(0.530840397679)*x[2]
        s[3,4]=(0.135928452703)*x[0]+(-1.51122126726)*x[1]+(1.6120424041)*x[2]
        ref[0,0]=(1.24742647646)/(o+1.)+(0.269060816234)+(0.237450952046)*0.5**o
        ref[0,1]=(1.12585957806)/(o+1.)+(-0.755828198463)+(0.301857541273)*0.5**o
        ref[0,2]=(-0.435989664675)/(o+1.)+(0.295471183049)+(0.213930631278)*0.5**o
        ref[0,3]=(-0.488921466935)/(o+1.)+(0.294480448259)+(-0.151038607178)*0.5**o
        ref[0,4]=(0.516230612817)/(o+1.)+(-0.0787984362944)+(-0.343742459316)*0.5**o
        ref[1,0]=(-0.492183705773)/(o+1.)+(0.501230271461)+(0.118646735962)*0.5**o
        ref[1,1]=(-1.43995574312)/(o+1.)+(-0.28437529362)+(-0.107412074623)*0.5**o
        ref[1,2]=(-0.101017880825)/(o+1.)+(0.28028462641)+(-0.445209107154)*0.5**o
        ref[1,3]=(0.128471434955)/(o+1.)+(-0.384371968608)+(-0.62463922518)*0.5**o
        ref[1,4]=(-0.0918241851534)/(o+1.)+(-0.0317649881343)+(0.774770507081)*0.5**o
        ref[2,0]=(0.221291051677)/(o+1.)+(-0.302519614649)+(-0.457492269704)*0.5**o
        ref[2,1]=(0.100276041699)/(o+1.)+(-0.387935114184)+(0.255875682695)*0.5**o
        ref[2,2]=(0.67334801807)/(o+1.)+(0.363637415491)+(-0.610716101317)*0.5**o
        ref[2,3]=(-0.101257531168)/(o+1.)+(-0.225887466089)+(0.48993468955)*0.5**o
        ref[2,4]=(-0.451085705642)/(o+1.)+(-0.415483032516)+(-0.817257908036)*0.5**o
        ref[3,0]=(1.87076639925)/(o+1.)+(1.20828670015)+(-0.331930451075)*0.5**o
        ref[3,1]=(-1.43689409978)/(o+1.)+(0.0171611774403)+(-0.958382466017)*0.5**o
        ref[3,2]=(0.146538702473)/(o+1.)+(0.391890493041)+(0.175513214998)*0.5**o
        ref[3,3]=(-0.122934034211)/(o+1.)+(0.140609173665)+(-0.814721563016)*0.5**o
        ref[3,4]=(0.223164895319)/(o+1.)+(-0.31244215916)+(0.638469012551)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactOne_fromSymbol_ReducedContinuousFunction_rank3(self):
      """
      tests integral of rank 3 Symbol on the FunctionOnContactOne

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports integral on FunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(6, 2, 2))
      s=Data(0,(6, 2, 2),w)
      ref=numpy.zeros((6, 2, 2),numpy.float_)
      if dim==2:
        s[0,0,0]=(-0.35078097593)*x[0]+(0.306926522015)*x[1]
        s[0,0,1]=(-0.803534689121)*x[0]+(-0.0602880575367)*x[1]
        s[0,1,0]=(-0.903470951227)*x[0]+(0.374123966765)*x[1]
        s[0,1,1]=(-1.18641387512)*x[0]+(0.273430931834)*x[1]
        s[1,0,0]=(0.493295479958)*x[0]+(1.54668330013)*x[1]
        s[1,0,1]=(-1.27848534954)*x[0]+(0.0588492485708)*x[1]
        s[1,1,0]=(-1.42953381447)*x[0]+(0.717393314307)*x[1]
        s[1,1,1]=(-0.467730806598)*x[0]+(-0.0117815181086)*x[1]
        s[2,0,0]=(-0.602626661024)*x[0]+(0.533324435346)*x[1]
        s[2,0,1]=(-0.904855697799)*x[0]+(1.09112846759)*x[1]
        s[2,1,0]=(1.20360337188)*x[0]+(1.23097020284)*x[1]
        s[2,1,1]=(0.0885447539403)*x[0]+(0.405387510263)*x[1]
        s[3,0,0]=(-0.730436778413)*x[0]+(0.416740386984)*x[1]
        s[3,0,1]=(-1.04126166563)*x[0]+(0.41753641298)*x[1]
        s[3,1,0]=(-0.271195830488)*x[0]+(-0.11575841172)*x[1]
        s[3,1,1]=(1.16100889666)*x[0]+(-0.233929048364)*x[1]
        s[4,0,0]=(0.664549807695)*x[0]+(0.224807977257)*x[1]
        s[4,0,1]=(-0.796935211563)*x[0]+(0.365917637053)*x[1]
        s[4,1,0]=(0.0401977690429)*x[0]+(-0.612500495333)*x[1]
        s[4,1,1]=(-0.115172707709)*x[0]+(-0.280626474812)*x[1]
        s[5,0,0]=(0.844835935845)*x[0]+(-1.19344222698)*x[1]
        s[5,0,1]=(-1.81553680308)*x[0]+(-1.66095998384)*x[1]
        s[5,1,0]=(0.761230206372)*x[0]+(0.547313314376)*x[1]
        s[5,1,1]=(-0.246895651321)*x[0]+(0.391870038086)*x[1]
        ref[0,0,0]=(-0.600965801098)/(o+1.)+(0.338185808482)+(-0.119260269781)*0.5**o
        ref[0,0,1]=(-0.224867141426)/(o+1.)+(-0.334457471517)+(0.0299593378011)*0.5**o
        ref[0,1,0]=(0.0129821011627)/(o+1.)+(-0.0950564992708)+(-0.352216087084)*0.5**o
        ref[0,1,1]=(0.0332057375276)/(o+1.)+(-0.0551431610872)+(-0.835902358643)*0.5**o
        ref[1,0,0]=(0.553424856182)/(o+1.)+(0.848186123918)+(-0.209818323927)*0.5**o
        ref[1,0,1]=(-0.75127641037)/(o+1.)+(-0.0481529718983)+(-0.372053746804)*0.5**o
        ref[1,1,0]=(-0.218691700126)/(o+1.)+(0.0665842087159)+(-0.626617217469)*0.5**o
        ref[1,1,1]=(-0.751752416764)/(o+1.)+(0.165004967668)+(-0.0577698432779)*0.5**o
        ref[2,0,0]=(-0.0172781449053)/(o+1.)+(-0.197949307988)+(0.343874535203)*0.5**o
        ref[2,0,1]=(0.499625949394)/(o+1.)+(-0.0639621511173)+(-0.185428877371)*0.5**o
        ref[2,1,0]=(0.965002845499)/(o+1.)+(0.296405176523)+(0.87676037618)*0.5**o
        ref[2,1,1]=(0.571427043323)/(o+1.)+(0.355568778058)+(-0.788632335234)*0.5**o
        ref[3,0,0]=(0.643869395266)/(o+1.)+(-0.248603098629)+(-0.460359589438)*0.5**o
        ref[3,0,1]=(0.847552650465)/(o+1.)+(-0.471374980899)+(-0.528527941319)*0.5**o
        ref[3,1,0]=(0.792482290139)/(o+1.)+(-0.288835326975)+(-0.601765878397)*0.5**o
        ref[3,1,1]=(0.674315208514)/(o+1.)+(-0.301924186259)+(0.8566130123)*0.5**o
        ref[4,0,0]=(0.14482725218)/(o+1.)+(0.123667770392)+(0.497194991988)*0.5**o
        ref[4,0,1]=(0.207166197869)/(o+1.)+(-0.00140080206462)+(-0.63538216825)*0.5**o
        ref[4,1,0]=(0.134064796438)/(o+1.)+(-0.583980524751)+(0.461593526773)*0.5**o
        ref[4,1,1]=(0.329865203686)/(o+1.)+(-0.0277930553577)+(-0.670078275492)*0.5**o
        ref[5,0,0]=(-0.671043415594)/(o+1.)+(-0.0772895763368)+(0.477016277134)*0.5**o
        ref[5,0,1]=(-0.863660332362)/(o+1.)+(-0.862182891111)+(-0.888470672335)*0.5**o
        ref[5,1,0]=(0.734550839405)/(o+1.)+(-0.00320324272275)+(0.580399166789)*0.5**o
        ref[5,1,1]=(-0.186529971258)/(o+1.)+(0.409102503591)+(-0.486700649159)*0.5**o
      else:
        s[0,0,0]=(1.49896775273)*x[0]+(-1.78250968743)*x[1]+(1.41503337092)*x[2]
        s[0,0,1]=(-0.643818266916)*x[0]+(-0.375134170105)*x[1]+(0.650514899885)*x[2]
        s[0,1,0]=(-0.0528798419185)*x[0]+(0.665778795831)*x[1]+(1.21069663928)*x[2]
        s[0,1,1]=(-1.45464226351)*x[0]+(-0.19467401007)*x[1]+(0.442665314437)*x[2]
        s[1,0,0]=(-0.836797292596)*x[0]+(-0.0542158346848)*x[1]+(-0.54209479287)*x[2]
        s[1,0,1]=(-1.09189713442)*x[0]+(1.03696758011)*x[1]+(0.0722113737424)*x[2]
        s[1,1,0]=(-0.493099825558)*x[0]+(-0.914461955024)*x[1]+(1.50580577272)*x[2]
        s[1,1,1]=(-0.702256378373)*x[0]+(-1.55374444057)*x[1]+(0.187304514552)*x[2]
        s[2,0,0]=(-0.924126823838)*x[0]+(-1.20470503473)*x[1]+(-0.702166397011)*x[2]
        s[2,0,1]=(1.04661135244)*x[0]+(0.995774871402)*x[1]+(0.912661077993)*x[2]
        s[2,1,0]=(0.0694812274474)*x[0]+(-0.384400463672)*x[1]+(-1.27290008863)*x[2]
        s[2,1,1]=(-0.196348771836)*x[0]+(0.314419668134)*x[1]+(-0.147174148507)*x[2]
        s[3,0,0]=(0.555982166527)*x[0]+(1.26542592993)*x[1]+(1.94075238034)*x[2]
        s[3,0,1]=(-0.15925841434)*x[0]+(0.235963126853)*x[1]+(1.01719057653)*x[2]
        s[3,1,0]=(1.8766116496)*x[0]+(0.0416539079261)*x[1]+(0.692631820852)*x[2]
        s[3,1,1]=(-1.60594678123)*x[0]+(0.418185984317)*x[1]+(1.00380643717)*x[2]
        s[4,0,0]=(-0.496656665568)*x[0]+(-0.519100502539)*x[1]+(-0.402964765219)*x[2]
        s[4,0,1]=(0.810723701732)*x[0]+(-0.9121518445)*x[1]+(0.0693075114891)*x[2]
        s[4,1,0]=(-1.38980929277)*x[0]+(-0.135080762705)*x[1]+(-0.708040071312)*x[2]
        s[4,1,1]=(0.0366128814416)*x[0]+(-0.598343408328)*x[1]+(-1.05172726191)*x[2]
        s[5,0,0]=(0.626645586525)*x[0]+(0.368702379272)*x[1]+(1.06419625419)*x[2]
        s[5,0,1]=(0.823542930912)*x[0]+(-0.397291307792)*x[1]+(1.47677515709)*x[2]
        s[5,1,0]=(1.0044323164)*x[0]+(-0.400460024166)*x[1]+(0.0881220069594)*x[2]
        s[5,1,1]=(0.841418294962)*x[0]+(0.448350914459)*x[1]+(-0.535982888316)*x[2]
        ref[0,0,0]=(0.0352781393333)/(o+1.)+(0.227326591212)+(0.64156011446)*0.5**o
        ref[0,0,1]=(-0.122693252863)/(o+1.)+(-0.221154887636)+(0.196565490999)*0.5**o
        ref[0,1,0]=(0.784542061966)/(o+1.)+(0.731576295626)+(-0.424099060024)*0.5**o
        ref[0,1,1]=(0.118776006377)/(o+1.)+(-0.423066319495)+(-0.479294326528)*0.5**o
        ref[1,0,0]=(1.18086837157)/(o+1.)+(-1.14308640082)+(-0.327803490075)*0.5**o
        ref[1,0,1]=(1.11878511573)/(o+1.)+(-0.427948355911)+(-0.245606584475)*0.5**o
        ref[1,1,0]=(-0.256395599928)/(o+1.)+(0.22748609258)+(-0.100332593094)*0.5**o
        ref[1,1,1]=(-0.192853498607)/(o+1.)+(-0.925785926894)+(-0.0242709519946)*0.5**o
        ref[2,0,0]=(-0.415752649176)/(o+1.)+(-1.14572035003)+(-0.123804906338)*0.5**o
        ref[2,0,1]=(0.384199373535)/(o+1.)+(0.843162532691)+(0.884522862915)*0.5**o
        ref[2,1,0]=(-0.229595030341)/(o+1.)+(-1.14751601921)+(0.936807743902)*0.5**o
        ref[2,1,1]=(0.267255066685)/(o+1.)+(-0.491644195523)+(0.686930072154)*0.5**o
        ref[3,0,0]=(1.68424865452)/(o+1.)+(0.955116898624)+(0.167678025029)*0.5**o
        ref[3,0,1]=(0.0034339126924)/(o+1.)+(0.718824562093)+(-0.347187747836)*0.5**o
        ref[3,1,0]=(1.62164005566)/(o+1.)+(0.0562907078128)+(0.876675907094)*0.5**o
        ref[3,1,1]=(0.472424872079)/(o+1.)+(0.0308594354746)+(-0.718098102775)*0.5**o
        ref[4,0,0]=(-0.0285810697935)/(o+1.)+(-0.525693855415)+(-0.338753152702)*0.5**o
        ref[4,0,1]=(0.303057697082)/(o+1.)+(-0.350175453973)+(0.365172579586)*0.5**o
        ref[4,1,0]=(-1.04637813553)/(o+1.)+(-0.308024248014)+(-0.57050349523)*0.5**o
        ref[4,1,1]=(-0.500793745646)/(o+1.)+(-0.424344892355)+(-0.26397425844)*0.5**o
        ref[5,0,0]=(0.365122763119)/(o+1.)+(0.568706597744)+(0.557008261376)*0.5**o
        ref[5,0,1]=(0.510379226755)/(o+1.)+(0.436754108223)+(0.519139337008)*0.5**o
        ref[5,1,0]=(-0.200129774692)/(o+1.)+(0.274983568325)+(0.342256937234)*0.5**o
        ref[5,1,1]=(0.566120139584)/(o+1.)+(-0.312318326061)+(0.812302833644)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactOne_fromSymbol_ReducedContinuousFunction_rank4(self):
      """
      tests integral of rank 4 Symbol on the FunctionOnContactOne

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports integral on FunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5, 3, 2))
      s=Data(0,(4, 5, 3, 2),w)
      ref=numpy.zeros((4, 5, 3, 2),numpy.float_)
      if dim==2:
        s[0,0,0,0]=(-0.36635416566)*x[0]+(0.454278450476)*x[1]
        s[0,0,0,1]=(0.449553060759)*x[0]+(-0.226772438798)*x[1]
        s[0,0,1,0]=(0.648343714053)*x[0]+(0.233007941149)*x[1]
        s[0,0,1,1]=(-0.135283076943)*x[0]+(1.61027554283)*x[1]
        s[0,0,2,0]=(0.619161973877)*x[0]+(-0.112220917964)*x[1]
        s[0,0,2,1]=(0.857902398581)*x[0]+(1.51928597673)*x[1]
        s[0,1,0,0]=(0.331827670107)*x[0]+(1.40775973855)*x[1]
        s[0,1,0,1]=(0.421840987115)*x[0]+(0.118563659741)*x[1]
        s[0,1,1,0]=(0.447268593931)*x[0]+(0.484984994669)*x[1]
        s[0,1,1,1]=(-0.189018546367)*x[0]+(0.467542988869)*x[1]
        s[0,1,2,0]=(0.466123494496)*x[0]+(-0.713267013868)*x[1]
        s[0,1,2,1]=(1.14105709142)*x[0]+(-0.134715682223)*x[1]
        s[0,2,0,0]=(0.871597208182)*x[0]+(-1.49080786854)*x[1]
        s[0,2,0,1]=(-0.94888847468)*x[0]+(0.0275999018363)*x[1]
        s[0,2,1,0]=(0.77534514887)*x[0]+(1.67162256756)*x[1]
        s[0,2,1,1]=(-0.786790506628)*x[0]+(-0.0692869424346)*x[1]
        s[0,2,2,0]=(1.78371863881)*x[0]+(-0.0481919137551)*x[1]
        s[0,2,2,1]=(-0.992074926087)*x[0]+(-1.02372593432)*x[1]
        s[0,3,0,0]=(0.113042447336)*x[0]+(-0.0790352303856)*x[1]
        s[0,3,0,1]=(1.12030253557)*x[0]+(0.393484268041)*x[1]
        s[0,3,1,0]=(0.379936292181)*x[0]+(-1.53869802379)*x[1]
        s[0,3,1,1]=(-0.191443496207)*x[0]+(0.563534464341)*x[1]
        s[0,3,2,0]=(0.104758691419)*x[0]+(-0.99439187659)*x[1]
        s[0,3,2,1]=(-0.479774987108)*x[0]+(-0.142765773584)*x[1]
        s[0,4,0,0]=(-1.50474690932)*x[0]+(-0.507691997905)*x[1]
        s[0,4,0,1]=(-0.78875103976)*x[0]+(0.132625919742)*x[1]
        s[0,4,1,0]=(-0.692847502554)*x[0]+(0.437815890612)*x[1]
        s[0,4,1,1]=(-0.808194455413)*x[0]+(0.376369555557)*x[1]
        s[0,4,2,0]=(0.919158598696)*x[0]+(-1.18391426084)*x[1]
        s[0,4,2,1]=(0.42374001855)*x[0]+(0.812305143875)*x[1]
        s[1,0,0,0]=(0.218364606199)*x[0]+(-1.40515806247)*x[1]
        s[1,0,0,1]=(-0.351603222911)*x[0]+(1.45105178061)*x[1]
        s[1,0,1,0]=(1.39849984972)*x[0]+(-1.35867734934)*x[1]
        s[1,0,1,1]=(-0.00360307599185)*x[0]+(0.246277359726)*x[1]
        s[1,0,2,0]=(0.514944968042)*x[0]+(0.428253029894)*x[1]
        s[1,0,2,1]=(-0.786120897197)*x[0]+(0.920810959626)*x[1]
        s[1,1,0,0]=(-0.602179171574)*x[0]+(0.515213890072)*x[1]
        s[1,1,0,1]=(-0.0861102153588)*x[0]+(0.477359706262)*x[1]
        s[1,1,1,0]=(-0.688164903151)*x[0]+(-0.313549483397)*x[1]
        s[1,1,1,1]=(-0.429990373791)*x[0]+(-0.0543708786137)*x[1]
        s[1,1,2,0]=(-0.39223682603)*x[0]+(0.796504408378)*x[1]
        s[1,1,2,1]=(-1.69046328113)*x[0]+(-0.344952802268)*x[1]
        s[1,2,0,0]=(-0.528152430512)*x[0]+(-0.269147355)*x[1]
        s[1,2,0,1]=(-1.68741664082)*x[0]+(0.568028242365)*x[1]
        s[1,2,1,0]=(-0.076694122635)*x[0]+(0.512745790897)*x[1]
        s[1,2,1,1]=(-0.738719294653)*x[0]+(1.72566985283)*x[1]
        s[1,2,2,0]=(0.679026798185)*x[0]+(-0.766391547753)*x[1]
        s[1,2,2,1]=(-1.7766436376)*x[0]+(-0.626962152238)*x[1]
        s[1,3,0,0]=(-0.0629592766364)*x[0]+(-0.602623643371)*x[1]
        s[1,3,0,1]=(1.46050982285)*x[0]+(-1.29912311111)*x[1]
        s[1,3,1,0]=(0.565052310821)*x[0]+(0.451146130683)*x[1]
        s[1,3,1,1]=(-1.72352086869)*x[0]+(-0.729969658338)*x[1]
        s[1,3,2,0]=(-0.699562180077)*x[0]+(0.785838373001)*x[1]
        s[1,3,2,1]=(0.0504742963364)*x[0]+(-0.856596707973)*x[1]
        s[1,4,0,0]=(-0.220685313017)*x[0]+(1.49415344926)*x[1]
        s[1,4,0,1]=(0.965835473202)*x[0]+(-0.664523896748)*x[1]
        s[1,4,1,0]=(-0.643560593026)*x[0]+(-1.06826452446)*x[1]
        s[1,4,1,1]=(-0.347993256951)*x[0]+(0.240926576999)*x[1]
        s[1,4,2,0]=(-0.666305003293)*x[0]+(1.5567540195)*x[1]
        s[1,4,2,1]=(0.752122481773)*x[0]+(-0.940078899457)*x[1]
        s[2,0,0,0]=(-0.428935565836)*x[0]+(-0.471904780642)*x[1]
        s[2,0,0,1]=(0.157225979958)*x[0]+(-0.707126471203)*x[1]
        s[2,0,1,0]=(-0.127873936162)*x[0]+(-1.01452234653)*x[1]
        s[2,0,1,1]=(-1.43314905378)*x[0]+(1.09406218538)*x[1]
        s[2,0,2,0]=(0.125475529763)*x[0]+(-0.144317966895)*x[1]
        s[2,0,2,1]=(0.781409858349)*x[0]+(-0.27747495012)*x[1]
        s[2,1,0,0]=(-1.21917608112)*x[0]+(-0.785716389383)*x[1]
        s[2,1,0,1]=(-0.26082261117)*x[0]+(-0.376402142009)*x[1]
        s[2,1,1,0]=(-0.125232448715)*x[0]+(0.261783624866)*x[1]
        s[2,1,1,1]=(-1.25985888818)*x[0]+(-0.0711437757878)*x[1]
        s[2,1,2,0]=(-0.300629742217)*x[0]+(-0.46939686121)*x[1]
        s[2,1,2,1]=(-1.77973256304)*x[0]+(0.713178700308)*x[1]
        s[2,2,0,0]=(-0.900064650204)*x[0]+(0.507949604762)*x[1]
        s[2,2,0,1]=(-0.885252734166)*x[0]+(-0.154018162942)*x[1]
        s[2,2,1,0]=(-0.7944479569)*x[0]+(1.77390521085)*x[1]
        s[2,2,1,1]=(1.1725888609)*x[0]+(0.662244216625)*x[1]
        s[2,2,2,0]=(-0.236310280848)*x[0]+(-0.542661345078)*x[1]
        s[2,2,2,1]=(1.15526522118)*x[0]+(0.0247167490822)*x[1]
        s[2,3,0,0]=(0.175989767802)*x[0]+(1.43963679604)*x[1]
        s[2,3,0,1]=(-0.692773425778)*x[0]+(-0.297594800061)*x[1]
        s[2,3,1,0]=(-0.847486832282)*x[0]+(0.0810250645286)*x[1]
        s[2,3,1,1]=(-0.681973624741)*x[0]+(-1.24971452577)*x[1]
        s[2,3,2,0]=(-1.06756371509)*x[0]+(0.977349108627)*x[1]
        s[2,3,2,1]=(0.846764513647)*x[0]+(-0.547998096826)*x[1]
        s[2,4,0,0]=(0.634255041776)*x[0]+(0.339363455414)*x[1]
        s[2,4,0,1]=(-1.39093420159)*x[0]+(0.716421880025)*x[1]
        s[2,4,1,0]=(0.257472890363)*x[0]+(-0.197982551362)*x[1]
        s[2,4,1,1]=(-0.45529259808)*x[0]+(-0.761148165674)*x[1]
        s[2,4,2,0]=(0.955343907343)*x[0]+(-1.29615358423)*x[1]
        s[2,4,2,1]=(0.362935017626)*x[0]+(1.25503750319)*x[1]
        s[3,0,0,0]=(0.179219886497)*x[0]+(1.78329032848)*x[1]
        s[3,0,0,1]=(0.43735751276)*x[0]+(0.0403938192717)*x[1]
        s[3,0,1,0]=(1.11478662344)*x[0]+(-0.240063001264)*x[1]
        s[3,0,1,1]=(0.172583585916)*x[0]+(0.506428670417)*x[1]
        s[3,0,2,0]=(1.33494651584)*x[0]+(0.781224884171)*x[1]
        s[3,0,2,1]=(0.0432954454652)*x[0]+(-0.68034279444)*x[1]
        s[3,1,0,0]=(-0.856892482415)*x[0]+(0.836687116891)*x[1]
        s[3,1,0,1]=(0.487431626464)*x[0]+(-0.148084918499)*x[1]
        s[3,1,1,0]=(0.331122282784)*x[0]+(-1.38429248259)*x[1]
        s[3,1,1,1]=(-0.215232051361)*x[0]+(0.537089578691)*x[1]
        s[3,1,2,0]=(1.24485548633)*x[0]+(0.509639847837)*x[1]
        s[3,1,2,1]=(-0.321450199534)*x[0]+(0.211384054649)*x[1]
        s[3,2,0,0]=(0.355385603273)*x[0]+(0.928139739798)*x[1]
        s[3,2,0,1]=(0.927589198094)*x[0]+(1.38324753996)*x[1]
        s[3,2,1,0]=(-1.16347040084)*x[0]+(-0.351854201414)*x[1]
        s[3,2,1,1]=(0.717488068131)*x[0]+(-0.0977120861443)*x[1]
        s[3,2,2,0]=(-0.799786558433)*x[0]+(0.829503505995)*x[1]
        s[3,2,2,1]=(1.37899235577)*x[0]+(-1.77910476445)*x[1]
        s[3,3,0,0]=(0.763361907215)*x[0]+(-0.861336281243)*x[1]
        s[3,3,0,1]=(0.476903208654)*x[0]+(0.903897305941)*x[1]
        s[3,3,1,0]=(1.60374028575)*x[0]+(-0.315620315575)*x[1]
        s[3,3,1,1]=(0.972809255696)*x[0]+(0.363233068227)*x[1]
        s[3,3,2,0]=(0.602799894079)*x[0]+(1.20772970348)*x[1]
        s[3,3,2,1]=(-0.000443665606339)*x[0]+(-1.6279763946)*x[1]
        s[3,4,0,0]=(-1.26676161275)*x[0]+(0.148063159145)*x[1]
        s[3,4,0,1]=(0.322891919608)*x[0]+(-1.08096025287)*x[1]
        s[3,4,1,0]=(-0.0944277709182)*x[0]+(-0.824212826197)*x[1]
        s[3,4,1,1]=(1.13530096846)*x[0]+(0.217321895058)*x[1]
        s[3,4,2,0]=(1.25663262459)*x[0]+(-0.308861390599)*x[1]
        s[3,4,2,1]=(-0.379182493224)*x[0]+(0.331110509538)*x[1]
        ref[0,0,0,0]=(0.408958259232)/(o+1.)+(0.247137449101)+(-0.815308872619)*0.5**o
        ref[0,0,0,1]=(0.13228131757)/(o+1.)+(0.165819155398)+(-0.241139006405)*0.5**o
        ref[0,0,1,0]=(0.128399251556)/(o+1.)+(0.371321245227)+(0.0103099131928)*0.5**o
        ref[0,0,1,1]=(0.966281299065)/(o+1.)+(0.381611325047)+(-0.254511483272)*0.5**o
        ref[0,0,2,0]=(-0.200167961434)/(o+1.)+(-0.0928181112535)+(0.892745239853)*0.5**o
        ref[0,0,2,1]=(0.830787814329)/(o+1.)+(0.314595496988)+(0.917209567011)*0.5**o
        ref[0,1,0,0]=(0.612992604133)/(o+1.)+(0.66241770717)+(-0.19824060982)*0.5**o
        ref[0,1,0,1]=(-0.0163161609645)/(o+1.)+(0.2025662158)+(0.151588376221)*0.5**o
        ref[0,1,1,0]=(-0.126917561445)/(o+1.)+(0.630548380093)+(-0.201925610142)*0.5**o
        ref[0,1,1,1]=(0.819096391825)/(o+1.)+(-0.370043224043)+(0.199514498764)*0.5**o
        ref[0,1,2,0]=(-0.96777130286)/(o+1.)+(0.367817797326)+(-0.0150078111633)*0.5**o
        ref[0,1,2,1]=(0.045654631235)/(o+1.)+(0.396367346781)+(0.167952084398)*0.5**o
        ref[0,2,0,0]=(-0.784765277575)/(o+1.)+(-0.298358907675)+(0.762272432566)*0.5**o
        ref[0,2,0,1]=(0.386572441281)/(o+1.)+(-0.51566858578)+(-0.276523842565)*0.5**o
        ref[0,2,1,0]=(0.680861847658)/(o+1.)+(0.740328104187)+(0.285449660402)*0.5**o
        ref[0,2,1,1]=(0.325836705829)/(o+1.)+(-0.49316269795)+(-0.195588758992)*0.5**o
        ref[0,2,2,0]=(0.783589269167)/(o+1.)+(0.0810542535881)+(0.789828948707)*0.5**o
        ref[0,2,2,1]=(-0.370218192335)/(o+1.)+(-0.608350209702)+(-0.428882248665)*0.5**o
        ref[0,3,0,0]=(0.188865301872)/(o+1.)+(-0.465976959809)+(0.777095834696)*0.5**o
        ref[0,3,0,1]=(0.764156203059)/(o+1.)+(-0.0928154457191)+(0.935261491991)*0.5**o
        ref[0,3,1,0]=(-0.689450927985)/(o+1.)+(-0.664550854911)+(0.8597909062)*0.5**o
        ref[0,3,1,1]=(0.555793140817)/(o+1.)+(0.142885227944)+(-0.469472628571)*0.5**o
        ref[0,3,2,0]=(-0.625798760976)/(o+1.)+(-0.499725272457)+(0.735616120719)*0.5**o
        ref[0,3,2,1]=(-0.907491695474)/(o+1.)+(0.385181050937)+(-0.485411167091)*0.5**o
        ref[0,4,0,0]=(-0.879336661239)/(o+1.)+(-0.205953419637)+(-0.721195406712)*0.5**o
        ref[0,4,0,1]=(0.86693894952)/(o+1.)+(-0.819677383073)+(0.11629069661)*0.5**o
        ref[0,4,1,0]=(-0.310289123464)/(o+1.)+(0.401826760004)+(-0.748396008487)*0.5**o
        ref[0,4,1,1]=(0.931547956812)/(o+1.)+(-0.740019832824)+(0.116666808981)*0.5**o
        ref[0,4,2,0]=(-0.847165088314)/(o+1.)+(-0.124403947896)+(0.831217321961)*0.5**o
        ref[0,4,2,1]=(0.177734973756)/(o+1.)+(0.441537508054)+(0.175235172561)*0.5**o
        ref[1,0,0,0]=(-0.542365809895)/(o+1.)+(-0.0827972838196)+(-0.478833078736)*0.5**o
        ref[1,0,0,1]=(0.844666854947)/(o+1.)+(0.498180748967)+(-0.741579795179)*0.5**o
        ref[1,0,1,0]=(-0.491126713824)/(o+1.)+(0.0335621009267)+(0.463825012354)*0.5**o
        ref[1,0,1,1]=(-0.309670980539)/(o+1.)+(0.472409867826)+(-0.392474471379)*0.5**o
        ref[1,0,2,0]=(0.278634540091)/(o+1.)+(-0.121141274647)+(0.906846007138)*0.5**o
        ref[1,0,2,1]=(-0.00603476025805)/(o+1.)+(0.182935877942)+(-0.225146933197)*0.5**o
        ref[1,1,0,0]=(0.580918965968)/(o+1.)+(0.0504878873855)+(-0.768860022241)*0.5**o
        ref[1,1,0,1]=(0.631736797799)/(o+1.)+(-0.537539597912)+(0.834591888928)*0.5**o
        ref[1,1,1,0]=(0.291226625025)/(o+1.)+(-0.703641010727)+(0.11434100988)*0.5**o
        ref[1,1,1,1]=(-0.909671911939)/(o+1.)+(0.237461939347)+(-0.0496132191606)*0.5**o
        ref[1,1,2,0]=(0.21073170992)/(o+1.)+(0.325715980645)+(-0.457896088863)*0.5**o
        ref[1,1,2,1]=(-0.725096486574)/(o+1.)+(-0.296742654874)+(-0.716834287076)*0.5**o
        ref[1,2,0,0]=(-0.164851515918)/(o+1.)+(-0.100676749276)+(-0.431094771042)*0.5**o
        ref[1,2,0,1]=(0.294840872181)/(o+1.)+(-0.329953166746)+(-0.754322937148)*0.5**o
        ref[1,2,1,0]=(0.341767621978)/(o+1.)+(0.400623115147)+(-0.70696218401)*0.5**o
        ref[1,2,1,1]=(0.821671220024)/(o+1.)+(0.193132433528)+(-0.220985528909)*0.5**o
        ref[1,2,2,0]=(-0.135508676597)/(o+1.)+(-0.38093100269)+(0.81000593241)*0.5**o
        ref[1,2,2,1]=(0.101637058171)/(o+1.)+(-0.771937440463)+(-0.961367967078)*0.5**o
        ref[1,3,0,0]=(-0.354951988253)/(o+1.)+(-0.41833260094)+(0.526034270126)*0.5**o
        ref[1,3,0,1]=(-0.461724677474)/(o+1.)+(-0.082145170992)+(0.787401731204)*0.5**o
        ref[1,3,1,0]=(0.681652074623)/(o+1.)+(-0.127199759209)+(0.5889458853)*0.5**o
        ref[1,3,1,1]=(-0.816769655641)/(o+1.)+(-0.382750879464)+(-0.871219112459)*0.5**o
        ref[1,3,2,0]=(0.549856768471)/(o+1.)+(-0.136659604755)+(-0.190261366036)*0.5**o
        ref[1,3,2,1]=(-0.298435559802)/(o+1.)+(-0.364033867566)+(0.220380883298)*0.5**o
        ref[1,4,0,0]=(0.692982064919)/(o+1.)+(0.445763556713)+(-0.311041042098)*0.5**o
        ref[1,4,0,1]=(-0.0284802774195)/(o+1.)+(0.173564544909)+(-0.0173372359445)*0.5**o
        ref[1,4,1,0]=(-0.521190661286)/(o+1.)+(-0.19266811161)+(-0.805298232979)*0.5**o
        ref[1,4,1,1]=(-0.344391605752)/(o+1.)+(0.344434513106)+(-0.451544100412)*0.5**o
        ref[1,4,2,0]=(0.590871123139)/(o+1.)+(0.123309088073)+(0.0529597169219)*0.5**o
        ref[1,4,2,1]=(-0.00996351412971)/(o+1.)+(-0.328705279006)+(0.479417654458)*0.5**o
        ref[2,0,0,0]=(0.220674489827)/(o+1.)+(-0.618024232079)+(0.114533627854)*0.5**o
        ref[2,0,0,1]=(-0.0844798976424)/(o+1.)+(0.070811636225)+(-0.607043866052)*0.5**o
        ref[2,0,1,0]=(-0.445683449242)/(o+1.)+(-0.113440437004)+(-0.469831959437)*0.5**o
        ref[2,0,1,1]=(0.271633369975)/(o+1.)+(0.132535807211)+(-0.875791852797)*0.5**o
        ref[2,0,2,0]=(0.649542525852)/(o+1.)+(0.0590165361575)+(-0.786418035298)*0.5**o
        ref[2,0,2,1]=(-0.317621366014)/(o+1.)+(0.329998426635)+(0.161559420974)*0.5**o
        ref[2,1,0,0]=(-0.835806177468)/(o+1.)+(-0.214223776118)+(-0.7406387408)*0.5**o
        ref[2,1,0,1]=(0.429741516432)/(o+1.)+(-0.552565623152)+(0.0381649766927)*0.5**o
        ref[2,1,1,0]=(-0.0472916937886)/(o+1.)+(-0.00552013887145)+(0.194883147682)*0.5**o
        ref[2,1,1,1]=(-0.199584048679)/(o+1.)+(-0.27639819939)+(-0.578622216512)*0.5**o
        ref[2,1,2,0]=(-0.941086888016)/(o+1.)+(-0.0385375072603)+(0.24813529911)*0.5**o
        ref[2,1,2,1]=(-0.0868737022971)/(o+1.)+(-0.0970507102796)+(-0.785578739872)*0.5**o
        ref[2,2,0,0]=(-0.202279119459)/(o+1.)+(-0.0839554337907)+(-0.0219250584016)*0.5**o
        ref[2,2,0,1]=(-0.184000614671)/(o+1.)+(-0.477928353442)+(0.100586424448)*0.5**o
        ref[2,2,1,0]=(0.927675340201)/(o+1.)+(0.0186691089268)+(0.0144436958933)*0.5**o
        ref[2,2,1,1]=(0.304792611717)/(o+1.)+(0.459166286092)+(0.611707893628)*0.5**o
        ref[2,2,2,0]=(0.0815687138062)/(o+1.)+(-0.768726979684)+(0.676913619635)*0.5**o
        ref[2,2,2,1]=(0.849269282518)/(o+1.)+(-0.25478410885)+(0.840280905449)*0.5**o
        ref[2,3,0,0]=(0.820793959083)/(o+1.)+(0.575169789593)+(-0.355506974424)*0.5**o
        ref[2,3,0,1]=(0.283603638973)/(o+1.)+(-0.335480628352)+(-0.603010608107)*0.5**o
        ref[2,3,1,0]=(-0.381013144765)/(o+1.)+(0.190974767434)+(-0.767398157855)*0.5**o
        ref[2,3,1,1]=(-0.773551356194)/(o+1.)+(-0.620890671367)+(0.0836445484134)*0.5**o
        ref[2,3,2,0]=(0.570212356041)/(o+1.)+(-0.289714757955)+(-0.0809974465983)*0.5**o
        ref[2,3,2,1]=(0.21581143181)/(o+1.)+(-0.415400501134)+(0.913755987278)*0.5**o
        ref[2,4,0,0]=(0.323568653669)/(o+1.)+(0.0799583453359)+(0.490133152849)*0.5**o
        ref[2,4,0,1]=(0.532408915634)/(o+1.)+(-0.115548404995)+(-0.97582442721)*0.5**o
        ref[2,4,1,0]=(-0.82114168932)/(o+1.)+(0.702564400414)+(-0.524496772508)*0.5**o
        ref[2,4,1,1]=(-0.479224200985)/(o+1.)+(-0.49364246599)+(0.25006836921)*0.5**o
        ref[2,4,2,0]=(-0.389740913016)/(o+1.)+(-0.0884282111182)+(0.225787658364)*0.5**o
        ref[2,4,2,1]=(0.508843816312)/(o+1.)+(0.17685868642)+(0.755411331668)*0.5**o
        ref[3,0,0,0]=(0.954468971244)/(o+1.)+(0.671012220464)+(-0.333983197198)*0.5**o
        ref[3,0,0,1]=(-0.312850718368)/(o+1.)+(-0.0723748107028)+(0.935351671806)*0.5**o
        ref[3,0,1,0]=(0.0640122675963)/(o+1.)+(0.220683721613)+(0.36934391135)*0.5**o
        ref[3,0,1,1]=(0.855735115439)/(o+1.)+(-0.366272230844)+(0.555821602582)*0.5**o
        ref[3,0,2,0]=(0.0417837662591)/(o+1.)+(0.556307682679)+(0.961772268394)*0.5**o
        ref[3,0,2,1]=(-0.928375029285)/(o+1.)+(-0.133425494561)+(0.558178669432)*0.5**o
        ref[3,1,0,0]=(0.610808931498)/(o+1.)+(-0.00371515949037)+(-0.623583978041)*0.5**o
        ref[3,1,0,1]=(-0.274141402005)/(o+1.)+(0.180852020544)+(0.251784068882)*0.5**o
        ref[3,1,1,0]=(-0.481032880244)/(o+1.)+(-0.705203882431)+(0.838270445302)*0.5**o
        ref[3,1,1,1]=(0.389002730435)/(o+1.)+(0.371327956101)+(-0.809801115307)*0.5**o
        ref[3,1,2,0]=(-0.217052039031)/(o+1.)+(0.504898297656)+(0.961750777882)*0.5**o
        ref[3,1,2,1]=(0.983589747751)/(o+1.)+(-0.768560013671)+(0.443464134708)*0.5**o
        ref[3,2,0,0]=(0.0850720894356)/(o+1.)+(0.295360321829)+(0.607732609977)*0.5**o
        ref[3,2,0,1]=(0.812866987322)/(o+1.)+(0.672388816065)+(0.153192118599)*0.5**o
        ref[3,2,1,0]=(-0.947419039593)/(o+1.)+(-0.179746473508)+(-0.208412615647)*0.5**o
        ref[3,2,1,1]=(-0.694332599484)/(o+1.)+(0.574069301711)+(0.165969978048)*0.5**o
        ref[3,2,2,0]=(0.746797880687)/(o+1.)+(0.0731831278015)+(-0.863447188728)*0.5**o
        ref[3,2,2,1]=(-0.9955274799)/(o+1.)+(-0.100141136686)+(0.795697344588)*0.5**o
        ref[3,3,0,0]=(-0.874362496111)/(o+1.)+(0.185559038089)+(0.405270045904)*0.5**o
        ref[3,3,0,1]=(0.557583002677)/(o+1.)+(0.556451155279)+(-0.28968479864)*0.5**o
        ref[3,3,1,0]=(0.435966507789)/(o+1.)+(0.00561393027505)+(0.840925601832)*0.5**o
        ref[3,3,1,1]=(-0.277310702324)/(o+1.)+(0.808938926899)+(-0.00452482755147)*0.5**o
        ref[3,3,2,0]=(0.878069034303)/(o+1.)+(0.0250230902204)+(0.882414382818)*0.5**o
        ref[3,3,2,1]=(-0.953670613054)/(o+1.)+(-0.626136165517)+(0.577522883879)*0.5**o
        ref[3,4,0,0]=(-0.0572166464991)/(o+1.)+(-0.195764673671)+(-0.669952459767)*0.5**o
        ref[3,4,0,1]=(-0.611801368818)/(o+1.)+(0.23313248955)+(-0.612531943539)*0.5**o
        ref[3,4,1,0]=(0.0451584859358)/(o+1.)+(-0.693097871519)+(0.422396659987)*0.5**o
        ref[3,4,1,1]=(0.0616735716897)/(o+1.)+(0.360358324628)+(0.57023264257)*0.5**o
        ref[3,4,2,0]=(0.404353246317)/(o+1.)+(0.02838608211)+(0.486645823449)*0.5**o
        ref[3,4,2,1]=(0.0039199584997)/(o+1.)+(0.344800813187)+(-0.741593568559)*0.5**o
      else:
        s[0,0,0,0]=(-0.00215071746669)*x[0]+(-0.332788203774)*x[1]+(-1.01240104354)*x[2]
        s[0,0,0,1]=(0.534102846033)*x[0]+(0.432162550346)*x[1]+(0.382807104545)*x[2]
        s[0,0,1,0]=(-0.774071131493)*x[0]+(0.174280314187)*x[1]+(0.235599208299)*x[2]
        s[0,0,1,1]=(1.66560900439)*x[0]+(1.38276801197)*x[1]+(0.451912961751)*x[2]
        s[0,0,2,0]=(-0.308642285607)*x[0]+(0.925763055269)*x[1]+(1.30533127575)*x[2]
        s[0,0,2,1]=(-0.15617828399)*x[0]+(-0.405834652149)*x[1]+(-0.846888299486)*x[2]
        s[0,1,0,0]=(-0.0743207250799)*x[0]+(-0.0831917437929)*x[1]+(-1.78311394968)*x[2]
        s[0,1,0,1]=(1.24178762381)*x[0]+(-0.121417608169)*x[1]+(0.0528284855272)*x[2]
        s[0,1,1,0]=(-1.14437209159)*x[0]+(0.0618138540822)*x[1]+(0.19124990065)*x[2]
        s[0,1,1,1]=(0.854246128467)*x[0]+(1.56688838996)*x[1]+(1.30424035038)*x[2]
        s[0,1,2,0]=(1.3184765993)*x[0]+(-1.40883832059)*x[1]+(0.832393276179)*x[2]
        s[0,1,2,1]=(-1.1625604886)*x[0]+(-0.104956544515)*x[1]+(-1.52389901089)*x[2]
        s[0,2,0,0]=(-0.106003547387)*x[0]+(0.852941663718)*x[1]+(0.24003638122)*x[2]
        s[0,2,0,1]=(0.892312082297)*x[0]+(-0.156996856451)*x[1]+(-1.2674975556)*x[2]
        s[0,2,1,0]=(0.0300043771968)*x[0]+(-0.848942993281)*x[1]+(-0.322425526609)*x[2]
        s[0,2,1,1]=(-0.361971760465)*x[0]+(1.89870303113)*x[1]+(0.861574517617)*x[2]
        s[0,2,2,0]=(0.97819437229)*x[0]+(-0.132714911288)*x[1]+(-0.957702168325)*x[2]
        s[0,2,2,1]=(0.588644003365)*x[0]+(-0.568265700558)*x[1]+(0.150521572866)*x[2]
        s[0,3,0,0]=(0.700807304996)*x[0]+(-1.14422226798)*x[1]+(0.587223436541)*x[2]
        s[0,3,0,1]=(-0.862702931786)*x[0]+(0.423016472206)*x[1]+(-0.555427243683)*x[2]
        s[0,3,1,0]=(0.574966038173)*x[0]+(-0.487199028698)*x[1]+(-0.32389521042)*x[2]
        s[0,3,1,1]=(0.130862033386)*x[0]+(0.42056248135)*x[1]+(0.925321527155)*x[2]
        s[0,3,2,0]=(-0.735071923108)*x[0]+(-0.528993360596)*x[1]+(0.485622476133)*x[2]
        s[0,3,2,1]=(0.126763646072)*x[0]+(0.0323222279682)*x[1]+(1.42515358228)*x[2]
        s[0,4,0,0]=(-0.334189936412)*x[0]+(0.14123257798)*x[1]+(-0.487379116976)*x[2]
        s[0,4,0,1]=(-0.528967405148)*x[0]+(-0.0622502130673)*x[1]+(-1.22107834734)*x[2]
        s[0,4,1,0]=(-1.67869230303)*x[0]+(-0.427809293008)*x[1]+(0.35027727689)*x[2]
        s[0,4,1,1]=(-1.3023324224)*x[0]+(0.364963594938)*x[1]+(1.25798740064)*x[2]
        s[0,4,2,0]=(0.789575151987)*x[0]+(-1.08985193505)*x[1]+(0.286072474971)*x[2]
        s[0,4,2,1]=(0.0321971599533)*x[0]+(1.64269617809)*x[1]+(0.448773378519)*x[2]
        s[1,0,0,0]=(-0.206961758788)*x[0]+(0.188339936734)*x[1]+(-1.36499979129)*x[2]
        s[1,0,0,1]=(-0.439106473295)*x[0]+(-1.35694867727)*x[1]+(0.17565861577)*x[2]
        s[1,0,1,0]=(0.754962041339)*x[0]+(0.377298691237)*x[1]+(-0.225684064888)*x[2]
        s[1,0,1,1]=(-0.164606093877)*x[0]+(-1.12165840203)*x[1]+(0.475143697931)*x[2]
        s[1,0,2,0]=(-0.563561093763)*x[0]+(-0.605592098451)*x[1]+(-0.382814067375)*x[2]
        s[1,0,2,1]=(-0.529311859218)*x[0]+(-0.440753365779)*x[1]+(-0.213919010047)*x[2]
        s[1,1,0,0]=(-0.381937118975)*x[0]+(0.115015087)*x[1]+(-1.58208233129)*x[2]
        s[1,1,0,1]=(-0.898803250164)*x[0]+(1.21794054101)*x[1]+(0.000708470416828)*x[2]
        s[1,1,1,0]=(0.417980628363)*x[0]+(-0.89943521283)*x[1]+(1.18981658848)*x[2]
        s[1,1,1,1]=(0.400600150308)*x[0]+(1.59168041332)*x[1]+(0.526855874334)*x[2]
        s[1,1,2,0]=(-0.658036770065)*x[0]+(0.903572546571)*x[1]+(-0.0322095817377)*x[2]
        s[1,1,2,1]=(-0.487427224544)*x[0]+(0.25991247483)*x[1]+(1.71801479005)*x[2]
        s[1,2,0,0]=(-1.23648267242)*x[0]+(-1.14397314877)*x[1]+(1.51420721169)*x[2]
        s[1,2,0,1]=(1.76604962545)*x[0]+(0.183730664401)*x[1]+(0.119413404231)*x[2]
        s[1,2,1,0]=(-0.371372970825)*x[0]+(1.67376148418)*x[1]+(-1.4673890303)*x[2]
        s[1,2,1,1]=(-1.05767796861)*x[0]+(1.57674524903)*x[1]+(0.907317505281)*x[2]
        s[1,2,2,0]=(-0.560171709945)*x[0]+(0.932731419096)*x[1]+(-1.34845422611)*x[2]
        s[1,2,2,1]=(0.299743316165)*x[0]+(1.27560261018)*x[1]+(-0.835084977966)*x[2]
        s[1,3,0,0]=(-0.255337437533)*x[0]+(-0.597132326112)*x[1]+(-0.136201264537)*x[2]
        s[1,3,0,1]=(0.593835766145)*x[0]+(-0.799771329645)*x[1]+(-0.980158768361)*x[2]
        s[1,3,1,0]=(1.38678662193)*x[0]+(0.145756993166)*x[1]+(-0.687830261701)*x[2]
        s[1,3,1,1]=(-0.535937612709)*x[0]+(-0.0191554551358)*x[1]+(0.360231645604)*x[2]
        s[1,3,2,0]=(0.464446491173)*x[0]+(0.601115829679)*x[1]+(-1.23243462932)*x[2]
        s[1,3,2,1]=(-1.00252524049)*x[0]+(0.38542247592)*x[1]+(-0.786817371152)*x[2]
        s[1,4,0,0]=(0.127520587286)*x[0]+(0.0193180871908)*x[1]+(0.443880703635)*x[2]
        s[1,4,0,1]=(0.459087583966)*x[0]+(-0.0133413082011)*x[1]+(-0.469574441754)*x[2]
        s[1,4,1,0]=(-0.0408745863162)*x[0]+(-0.450729520372)*x[1]+(0.49494190957)*x[2]
        s[1,4,1,1]=(0.698205417361)*x[0]+(0.475698414334)*x[1]+(-0.738661347545)*x[2]
        s[1,4,2,0]=(1.20797915788)*x[0]+(-0.132247786402)*x[1]+(-0.282227294584)*x[2]
        s[1,4,2,1]=(-0.76998141325)*x[0]+(-0.0677377690684)*x[1]+(-0.0836477753317)*x[2]
        s[2,0,0,0]=(-1.10579544966)*x[0]+(1.40458332668)*x[1]+(0.666829139108)*x[2]
        s[2,0,0,1]=(0.316772911983)*x[0]+(-0.722315825552)*x[1]+(-0.420491453418)*x[2]
        s[2,0,1,0]=(1.28978684389)*x[0]+(-1.31024104856)*x[1]+(-0.909892235182)*x[2]
        s[2,0,1,1]=(0.516882102505)*x[0]+(0.137837613805)*x[1]+(-0.0746054680439)*x[2]
        s[2,0,2,0]=(-0.684536226359)*x[0]+(1.26314901343)*x[1]+(-0.80088731126)*x[2]
        s[2,0,2,1]=(1.04894156753)*x[0]+(-0.292514668329)*x[1]+(0.189452265734)*x[2]
        s[2,1,0,0]=(-1.89697799819)*x[0]+(0.842415025171)*x[1]+(1.50175587499)*x[2]
        s[2,1,0,1]=(-0.132472949872)*x[0]+(0.755809155321)*x[1]+(1.4210576902)*x[2]
        s[2,1,1,0]=(-1.20812267424)*x[0]+(0.776283189748)*x[1]+(0.466844337822)*x[2]
        s[2,1,1,1]=(-0.545404145415)*x[0]+(0.630064653938)*x[1]+(0.416024592778)*x[2]
        s[2,1,2,0]=(-1.25450160322)*x[0]+(0.712099345608)*x[1]+(-1.40448708108)*x[2]
        s[2,1,2,1]=(0.8675638655)*x[0]+(-0.694288742412)*x[1]+(0.423917775454)*x[2]
        s[2,2,0,0]=(0.611369389916)*x[0]+(-0.282854928933)*x[1]+(0.421814371036)*x[2]
        s[2,2,0,1]=(0.87143261059)*x[0]+(0.626319549038)*x[1]+(-0.247486033705)*x[2]
        s[2,2,1,0]=(-1.09640818879)*x[0]+(-0.625440705278)*x[1]+(1.06488909366)*x[2]
        s[2,2,1,1]=(-1.63825859868)*x[0]+(0.243702450075)*x[1]+(0.37689118934)*x[2]
        s[2,2,2,0]=(1.14554562168)*x[0]+(-0.656545182621)*x[1]+(0.122881381407)*x[2]
        s[2,2,2,1]=(1.04057428387)*x[0]+(0.605147014643)*x[1]+(-0.0624571196862)*x[2]
        s[2,3,0,0]=(-0.687211572887)*x[0]+(-1.33387647369)*x[1]+(1.56461600602)*x[2]
        s[2,3,0,1]=(-0.439360840188)*x[0]+(-0.175050010481)*x[1]+(-0.493088639437)*x[2]
        s[2,3,1,0]=(1.01647468205)*x[0]+(1.607854298)*x[1]+(0.154041122248)*x[2]
        s[2,3,1,1]=(-1.17279615871)*x[0]+(0.265464982509)*x[1]+(0.536445292976)*x[2]
        s[2,3,2,0]=(-0.370774714547)*x[0]+(-0.140305952222)*x[1]+(-0.27755251782)*x[2]
        s[2,3,2,1]=(-0.601589093641)*x[0]+(-0.0914485078099)*x[1]+(-0.246092622851)*x[2]
        s[2,4,0,0]=(1.16222773615)*x[0]+(-1.0556446667)*x[1]+(1.18749640554)*x[2]
        s[2,4,0,1]=(0.441185917779)*x[0]+(0.106062084171)*x[1]+(-0.150699537615)*x[2]
        s[2,4,1,0]=(1.24765736189)*x[0]+(-1.28446862056)*x[1]+(-1.15581931924)*x[2]
        s[2,4,1,1]=(0.751269018864)*x[0]+(1.03373528238)*x[1]+(0.953542233964)*x[2]
        s[2,4,2,0]=(-0.507055271742)*x[0]+(-0.139767333963)*x[1]+(0.369122007484)*x[2]
        s[2,4,2,1]=(1.75708904398)*x[0]+(0.229784892513)*x[1]+(0.138439119614)*x[2]
        s[3,0,0,0]=(0.488579497744)*x[0]+(0.227848351934)*x[1]+(-0.503567606843)*x[2]
        s[3,0,0,1]=(1.48513008903)*x[0]+(0.0119808614235)*x[1]+(0.50138290689)*x[2]
        s[3,0,1,0]=(1.89811058679)*x[0]+(0.936442809385)*x[1]+(0.618927531336)*x[2]
        s[3,0,1,1]=(0.520596564131)*x[0]+(-1.34097734027)*x[1]+(-1.24288925352)*x[2]
        s[3,0,2,0]=(-1.22679833631)*x[0]+(0.946723115385)*x[1]+(-0.444825501575)*x[2]
        s[3,0,2,1]=(-0.786039767784)*x[0]+(1.01276854439)*x[1]+(0.868428470305)*x[2]
        s[3,1,0,0]=(-1.14928294841)*x[0]+(1.83212625566)*x[1]+(-0.131295307828)*x[2]
        s[3,1,0,1]=(0.530939875091)*x[0]+(-0.00936464244878)*x[1]+(-0.54397285021)*x[2]
        s[3,1,1,0]=(0.776394250264)*x[0]+(-0.0432924713397)*x[1]+(-0.48992936123)*x[2]
        s[3,1,1,1]=(-0.295285841654)*x[0]+(1.60838322278)*x[1]+(-0.822857254214)*x[2]
        s[3,1,2,0]=(-0.445325691811)*x[0]+(0.143992033055)*x[1]+(-0.562336605856)*x[2]
        s[3,1,2,1]=(-0.424042508176)*x[0]+(-0.119602696473)*x[1]+(0.50936759192)*x[2]
        s[3,2,0,0]=(0.0376712305683)*x[0]+(-0.127442186223)*x[1]+(-0.98477458245)*x[2]
        s[3,2,0,1]=(-0.964194007245)*x[0]+(-0.951386585236)*x[1]+(-0.376434083193)*x[2]
        s[3,2,1,0]=(-0.645708363662)*x[0]+(0.729523237957)*x[1]+(-0.815980951689)*x[2]
        s[3,2,1,1]=(1.54280019708)*x[0]+(0.347142929119)*x[1]+(0.117050155931)*x[2]
        s[3,2,2,0]=(-0.70653056148)*x[0]+(0.658564301815)*x[1]+(1.5766896863)*x[2]
        s[3,2,2,1]=(1.28238648298)*x[0]+(0.415173079215)*x[1]+(1.25492135064)*x[2]
        s[3,3,0,0]=(-0.907422225702)*x[0]+(0.354252510498)*x[1]+(-0.617733757576)*x[2]
        s[3,3,0,1]=(-0.687653032786)*x[0]+(0.166147005719)*x[1]+(0.0668202207665)*x[2]
        s[3,3,1,0]=(-0.515227984328)*x[0]+(-1.20687483216)*x[1]+(0.325703771711)*x[2]
        s[3,3,1,1]=(-0.239992988401)*x[0]+(-0.625429153195)*x[1]+(-0.987302012149)*x[2]
        s[3,3,2,0]=(0.48686562504)*x[0]+(-0.687077509478)*x[1]+(0.630530432518)*x[2]
        s[3,3,2,1]=(0.575785040914)*x[0]+(0.978122193974)*x[1]+(0.0753481784122)*x[2]
        s[3,4,0,0]=(-0.201304266512)*x[0]+(-0.563373770577)*x[1]+(-0.0445045378284)*x[2]
        s[3,4,0,1]=(0.348899505512)*x[0]+(-0.983874875078)*x[1]+(0.561390754019)*x[2]
        s[3,4,1,0]=(0.687251502909)*x[0]+(0.806428209931)*x[1]+(-0.848431263257)*x[2]
        s[3,4,1,1]=(-0.0690481011866)*x[0]+(-0.637464706806)*x[1]+(1.32335195878)*x[2]
        s[3,4,2,0]=(0.640286948745)*x[0]+(-1.44112416361)*x[1]+(-0.724716414365)*x[2]
        s[3,4,2,1]=(-0.967100763601)*x[0]+(0.0347001879437)*x[1]+(-0.888183629447)*x[2]
        ref[0,0,0,0]=(-1.47634692775)/(o+1.)+(0.413823374944)+(-0.69863978692)*0.5**o
        ref[0,0,0,1]=(0.467438056347)/(o+1.)+(-0.049348895708)+(0.980332235993)*0.5**o
        ref[0,0,1,0]=(0.0324608468542)/(o+1.)+(0.227219571463)+(-0.851091598789)*0.5**o
        ref[0,0,1,1]=(1.24663216229)/(o+1.)+(0.680736038169)+(0.892185739473)*0.5**o
        ref[0,0,2,0]=(1.04128811669)/(o+1.)+(0.801526892617)+(-0.721889856517)*0.5**o
        ref[0,0,2,1]=(-1.57019640933)/(o+1.)+(-0.0149100811556)+(0.191115336015)*0.5**o
        ref[0,1,0,0]=(-1.06234362705)/(o+1.)+(-0.881380501065)+(0.88447821062)*0.5**o
        ref[0,1,0,1]=(0.267287676352)/(o+1.)+(0.0934259729578)+(0.719058878904)*0.5**o
        ref[0,1,1,0]=(-0.83257335756)/(o+1.)+(0.056558158197)+(-0.171851295688)*0.5**o
        ref[0,1,1,1]=(1.59747135641)/(o+1.)+(1.00837611714)+(0.111151278122)*0.5**o
        ref[0,1,2,0]=(0.181694306894)/(o+1.)+(-0.0366161862313)+(0.633569620457)*0.5**o
        ref[0,1,2,1]=(-0.116206059658)/(o+1.)+(-1.20967709253)+(-0.255855799289)*0.5**o
        ref[0,2,0,0]=(0.615063090083)/(o+1.)+(-0.0944203632267)+(0.560752133921)*0.5**o
        ref[0,2,0,1]=(-1.68718452323)/(o+1.)+(0.415922156486)+(0.323157880499)*0.5**o
        ref[0,2,1,0]=(-1.38436480215)/(o+1.)+(0.514201265835)+(-0.785401872214)*0.5**o
        ref[0,2,1,1]=(1.07260243754)/(o+1.)+(1.03208581185)+(-0.738468272962)*0.5**o
        ref[0,2,2,0]=(-0.619469154652)/(o+1.)+(0.156513881249)+(0.19421868483)*0.5**o
        ref[0,2,2,1]=(0.742851517381)/(o+1.)+(-0.577857666542)+(0.583763691375)*0.5**o
        ref[0,3,0,0]=(-0.0209287146839)/(o+1.)+(-0.0417684031135)+(0.248273994472)*0.5**o
        ref[0,3,0,1]=(-0.876757092127)/(o+1.)+(0.156654815475)+(-0.431666242085)*0.5**o
        ref[0,3,1,0]=(-1.40731912274)/(o+1.)+(0.295731262157)+(0.579728397479)*0.5**o
        ref[0,3,1,1]=(0.766713725705)/(o+1.)+(0.520952712789)+(-0.331873109391)*0.5**o
        ref[0,3,2,0]=(-1.37493161295)/(o+1.)+(0.626269025128)+(-0.656049244875)*0.5**o
        ref[0,3,2,1]=(0.738828256274)/(o+1.)+(0.299922102653)+(0.245566994744)*0.5**o
        ref[0,4,0,0]=(-0.220320428485)/(o+1.)+(-0.381104395776)+(0.30219274463)*0.5**o
        ref[0,4,0,1]=(-1.71138026305)/(o+1.)+(0.0858133974006)+(-0.272542497315)*0.5**o
        ref[0,4,1,0]=(-0.0512711854771)/(o+1.)+(-0.398429115874)+(-0.908094901926)*0.5**o
        ref[0,4,1,1]=(0.775088175251)/(o+1.)+(0.153496176444)+(-0.76146195497)*0.5**o
        ref[0,4,2,0]=(-0.188892896681)/(o+1.)+(-0.212757962174)+(0.600204512934)*0.5**o
        ref[0,4,2,1]=(0.786135983408)/(o+1.)+(0.709443217024)+(-0.0813557008918)*0.5**o
        ref[1,0,0,0]=(-0.682877215956)/(o+1.)+(0.0430800979484)+(-0.786904593287)*0.5**o
        ref[1,0,0,1]=(-0.209743447547)/(o+1.)+(-0.529335530226)+(-0.351982026793)*0.5**o
        ref[1,0,1,0]=(1.7467904951)/(o+1.)+(-0.522734985482)+(0.205256143552)*0.5**o
        ref[1,0,1,1]=(-0.681949654071)/(o+1.)+(0.430192257885)+(-0.989555659669)*0.5**o
        ref[1,0,2,0]=(0.00559076557551)/(o+1.)+(-0.355981322114)+(-0.845595380938)*0.5**o
        ref[1,0,2,1]=(-1.0795557316)/(o+1.)+(0.433501713489)+(-0.971431930418)*0.5**o
        ref[1,1,0,0]=(-1.41228679449)/(o+1.)+(0.0214610301511)+(-0.47963962908)*0.5**o
        ref[1,1,0,1]=(0.675271467105)/(o+1.)+(0.258295057037)+(-0.872015819915)*0.5**o
        ref[1,1,1,0]=(-0.0723449260888)/(o+1.)+(0.642648455626)+(-0.504589981145)*0.5**o
        ref[1,1,1,1]=(1.69082429043)/(o+1.)+(0.351194447364)+(0.125923252805)*0.5**o
        ref[1,1,2,0]=(1.04100385874)/(o+1.)+(-0.253396731491)+(-0.320884200985)*0.5**o
        ref[1,1,2,1]=(1.44000111068)/(o+1.)+(0.167426476075)+(-0.284354022497)*0.5**o
        ref[1,2,0,0]=(0.762931449029)/(o+1.)+(-0.397841053459)+(-0.833497951611)*0.5**o
        ref[1,2,0,1]=(-0.353193148804)/(o+1.)+(0.729759415974)+(0.962868010939)*0.5**o
        ref[1,2,1,0]=(-0.0142626154573)/(o+1.)+(0.205452893539)+(-0.561643688565)*0.5**o
        ref[1,2,1,1]=(1.31669689043)/(o+1.)+(0.322616901969)+(-0.535545908666)*0.5**o
        ref[1,2,2,0]=(-0.720144981522)/(o+1.)+(-0.253513298251)+(0.251277061063)*0.5**o
        ref[1,2,2,1]=(0.171627338105)/(o+1.)+(0.304508788985)+(-0.0403839677009)*0.5**o
        ref[1,3,0,0]=(-0.385245021569)/(o+1.)+(0.0533837214678)+(-0.710193449549)*0.5**o
        ref[1,3,0,1]=(-1.66770468862)/(o+1.)+(0.0386999300625)+(0.404210496633)*0.5**o
        ref[1,3,1,0]=(0.1536232451)/(o+1.)+(-0.114369222402)+(0.919828553095)*0.5**o
        ref[1,3,1,1]=(0.279969861456)/(o+1.)+(0.199789198624)+(-0.874409680945)*0.5**o
        ref[1,3,2,0]=(-0.29883793799)/(o+1.)+(-0.179133605171)+(0.490232839865)*0.5**o
        ref[1,3,2,1]=(-0.20045707503)/(o+1.)+(-0.419654937503)+(-0.364153185689)*0.5**o
        ref[1,4,0,0]=(0.328269609451)/(o+1.)+(0.0976736768568)+(0.0671024149466)*0.5**o
        ref[1,4,0,1]=(0.70998344827)/(o+1.)+(-0.831520044753)+(0.929228475247)*0.5**o
        ref[1,4,1,0]=(1.14467737453)/(o+1.)+(-0.309242254267)+(-0.522855063115)*0.5**o
        ref[1,4,1,1]=(0.386397027677)/(o+1.)+(-0.152706345419)+(0.354258147311)*0.5**o
        ref[1,4,2,0]=(0.565989417809)/(o+1.)+(-0.0912836510331)+(0.410081961152)*0.5**o
        ref[1,4,2,1]=(-0.108228050157)/(o+1.)+(-0.382022658574)+(-0.0490935903447)*0.5**o
        ref[2,0,0,0]=(1.30816654656)/(o+1.)+(0.218850490887)+(-0.780250512205)*0.5**o
        ref[2,0,0,1]=(-0.072210934601)/(o+1.)+(-0.376572060485)+(-0.000679311415873)*0.5**o
        ref[2,0,1,0]=(-1.03731005117)/(o+1.)+(-0.369008331776)+(0.844980274869)*0.5**o
        ref[2,0,1,1]=(-0.37673073433)/(o+1.)+(0.147779920342)+(0.661285141912)*0.5**o
        ref[2,0,2,0]=(-0.126456722513)/(o+1.)+(-0.173173731574)+(0.250529661474)*0.5**o
        ref[2,0,2,1]=(0.269921444109)/(o+1.)+(0.256112039487)+(0.163733641849)*0.5**o
        ref[2,1,0,0]=(1.02033624645)/(o+1.)+(0.162073420045)+(-0.897290184569)*0.5**o
        ref[2,1,0,1]=(0.673075377238)/(o+1.)+(0.377904898438)+(0.615508721534)*0.5**o
        ref[2,1,1,0]=(-0.536750510163)/(o+1.)+(0.687164638156)+(-0.802573912815)*0.5**o
        ref[2,1,1,1]=(1.39522327504)/(o+1.)+(-0.168020484927)+(-0.558497203882)*0.5**o
        ref[2,1,2,0]=(-1.21446181317)/(o+1.)+(-0.219360528348)+(-0.293706468828)*0.5**o
        ref[2,1,2,1]=(0.352877691113)/(o+1.)+(0.0397286675094)+(0.16485787241)*0.5**o
        ref[2,2,0,0]=(-0.927761522597)/(o+1.)+(1.03238921925)+(-0.386688083878)*0.5**o
        ref[2,2,0,1]=(0.875914163347)/(o+1.)+(-0.0805989643579)+(0.535549891292)*0.5**o
        ref[2,2,1,0]=(-0.268027052879)/(o+1.)+(0.24501270411)+(-0.878958155751)*0.5**o
        ref[2,2,1,1]=(0.579267469532)/(o+1.)+(-0.419360478349)+(-0.758211472099)*0.5**o
        ref[2,2,2,0]=(-0.407532480549)/(o+1.)+(0.0236731772421)+(0.972067946527)*0.5**o
        ref[2,2,2,1]=(-0.848446514529)/(o+1.)+(0.986605187547)+(0.458500318263)*0.5**o
        ref[2,3,0,0]=(0.0139531165204)/(o+1.)+(0.0612674782454)+(-0.592960113562)*0.5**o
        ref[2,3,0,1]=(-0.434410189326)/(o+1.)+(0.0699472934754)+(-0.81298388773)*0.5**o
        ref[2,3,1,0]=(1.51687712753)/(o+1.)+(0.29518962448)+(0.671113725811)*0.5**o
        ref[2,3,1,1]=(-0.825018275156)/(o+1.)+(0.348395434187)+(-0.242658476441)*0.5**o
        ref[2,3,2,0]=(-0.129038376537)/(o+1.)+(0.0407049218889)+(-0.74100465183)*0.5**o
        ref[2,3,2,1]=(-0.61436863216)/(o+1.)+(0.0569955443553)+(-0.438752680853)*0.5**o
        ref[2,4,0,0]=(-0.186811568504)/(o+1.)+(0.324468688913)+(0.831953665661)*0.5**o
        ref[2,4,0,1]=(1.01471089788)/(o+1.)+(-0.404804936557)+(0.191447439569)*0.5**o
        ref[2,4,1,0]=(-0.833336708992)/(o+1.)+(-0.465420260939)+(0.571546652968)*0.5**o
        ref[2,4,1,1]=(1.48092456289)/(o+1.)+(0.548316338846)+(0.16098929463)*0.5**o
        ref[2,4,2,0]=(-1.20254311473)/(o+1.)+(0.778023417549)+(-0.631204318592)*0.5**o
        ref[2,4,2,1]=(0.672521404448)/(o+1.)+(0.265167920765)+(0.922455810129)*0.5**o
        ref[3,0,0,0]=(-0.400146773167)/(o+1.)+(0.151226335458)+(0.310554345086)*0.5**o
        ref[3,0,0,1]=(0.580039994556)/(o+1.)+(0.287523081772)+(0.843407699243)*0.5**o
        ref[3,0,1,0]=(0.930972332962)/(o+1.)+(0.768302886889)+(0.985902820768)*0.5**o
        ref[3,0,1,1]=(-0.991140747279)/(o+1.)+(-0.914680203238)+(0.757231124095)*0.5**o
        ref[3,0,2,0]=(-0.458287257967)/(o+1.)+(0.237995705359)+(-0.742604875255)*0.5**o
        ref[3,0,2,1]=(1.14385326779)/(o+1.)+(0.243645854371)+(-0.535987729627)*0.5**o
        ref[3,1,0,0]=(0.322354058507)/(o+1.)+(0.341162149011)+(-0.453130357106)*0.5**o
        ref[3,1,0,1]=(-1.78201691527)/(o+1.)+(1.04199074308)+(-0.324362188462)*0.5**o
        ref[3,1,1,0]=(-0.0578982797037)/(o+1.)+(-0.110933167998)+(0.522937033394)*0.5**o
        ref[3,1,1,1]=(-0.276009095212)/(o+1.)+(0.517113652949)+(-0.267978083771)*0.5**o
        ref[3,1,2,0]=(-0.023345636799)/(o+1.)+(-0.66878791133)+(0.497251194848)*0.5**o
        ref[3,1,2,1]=(1.2604140706)/(o+1.)+(-0.878310666552)+(0.461929649774)*0.5**o
        ref[3,2,0,0]=(-0.00211014675007)/(o+1.)+(-0.814091029436)+(0.555746667517)*0.5**o
        ref[3,2,0,1]=(-1.07650025357)/(o+1.)+(-0.26199073554)+(-0.691532951025)*0.5**o
        ref[3,2,1,0]=(-0.0770650645465)/(o+1.)+(-0.307619169254)+(-0.0398626743392)*0.5**o
        ref[3,2,1,1]=(1.12398638253)/(o+1.)+(-0.00626411324864)+(0.895535126103)*0.5**o
        ref[3,2,2,0]=(0.822758496096)/(o+1.)+(0.448681597953)+(-0.191398265373)*0.5**o
        ref[3,2,2,1]=(1.38402514586)/(o+1.)+(0.489672391609)+(0.589110983761)*0.5**o
        ref[3,3,0,0]=(0.626837013941)/(o+1.)+(-0.698370737203)+(-0.400999012316)*0.5**o
        ref[3,3,0,1]=(-0.819551723109)/(o+1.)+(0.588359566557)+(-0.811853216306)*0.5**o
        ref[3,3,1,0]=(-0.472944922304)/(o+1.)+(0.024839136659)+(-0.973132395795)*0.5**o
        ref[3,3,1,1]=(-0.674351724226)/(o+1.)+(-0.596697469314)+(0.0150225091087)*0.5**o
        ref[3,3,2,0]=(-0.653245573198)/(o+1.)+(0.590003731898)+(-0.0964433425189)*0.5**o
        ref[3,3,2,1]=(1.15741134315)/(o+1.)+(-0.128540702232)+(0.728925474609)*0.5**o
        ref[3,4,0,0]=(-0.745079746498)/(o+1.)+(-0.313462120963)+(0.562821413507)*0.5**o
        ref[3,4,0,1]=(-0.585232841208)/(o+1.)+(0.464633596697)+(-0.417618967733)*0.5**o
        ref[3,4,1,0]=(0.792309614109)/(o+1.)+(-0.478065247211)+(0.809069329896)*0.5**o
        ref[3,4,1,1]=(0.0536562315177)/(o+1.)+(-0.128980351611)+(0.821143622493)*0.5**o
        ref[3,4,2,0]=(-1.20913408459)/(o+1.)+(-0.144016051074)+(-0.0283874424941)*0.5**o
        ref[3,4,2,1]=(-1.12139150964)/(o+1.)+(-0.135856270303)+(-0.427480154859)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactOne_fromSymbol_FunctionOnContactOne_rank0(self):
      """
      tests integral of rank 0 Symbol on the FunctionOnContactOne

      assumptions: FunctionOnContactOne(self.domain) exists
                   self.domain supports integral on FunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      w=FunctionOnContactOne(self.domain)
      x=w.getX()
      arg=Symbol(shape=())
      s=Data(0,(),w)
      if dim==2:
        s=(0.0457734760303)*x[0]**o+(0.469200083987)*x[0]+(0.871163132136)*x[1]**o+(-0.848809543077)*x[1]
        ref=(0.871163132136)/(o+1.)+(-0.189804729545)+(0.0457734760303)*0.5**o
      else:
        s=(-0.217546543734)*x[0]**o+(-0.296791310389)*x[0]+(0.295478793948)*x[1]**o+(0.355506997608)*x[1]+(-0.173745407638)*x[2]**o+(0.733848286432)*x[2]
        ref=(0.12173338631)/(o+1.)+(0.396281986825)+(-0.217546543734)*0.5**o
      res=integrate(arg)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactOne_fromSymbol_FunctionOnContactOne_rank1(self):
      """
      tests integral of rank 1 Symbol on the FunctionOnContactOne

      assumptions: FunctionOnContactOne(self.domain) exists
                   self.domain supports integral on FunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      w=FunctionOnContactOne(self.domain)
      x=w.getX()
      arg=Symbol(shape=(2,))
      s=Data(0,(2,),w)
      ref=numpy.zeros((2,),numpy.float_)
      if dim==2:
        s[0]=(0.375635683415)*x[0]**o+(-0.367500579656)*x[0]+(0.0137704362217)*x[1]**o+(-0.861006616375)*x[1]
        s[1]=(-0.787660325374)*x[0]**o+(0.114019988756)*x[0]+(0.120533570643)*x[1]**o+(0.619732700586)*x[1]
        ref[0]=(0.0137704362217)/(o+1.)+(-0.614253598016)+(0.375635683415)*0.5**o
        ref[1]=(0.120533570643)/(o+1.)+(0.366876344671)+(-0.787660325374)*0.5**o
      else:
        s[0]=(-0.496247804966)*x[0]**o+(-0.717417759132)*x[0]+(-0.89565067445)*x[1]**o+(0.382056160402)*x[1]+(0.994707732251)*x[2]**o+(0.988370997667)*x[2]
        s[1]=(0.274829833844)*x[0]**o+(0.699113046456)*x[0]+(-0.780626603094)*x[1]**o+(0.00719416836835)*x[1]+(0.0771370370478)*x[2]**o+(-0.883519240597)*x[2]
        ref[0]=(0.0990570578009)/(o+1.)+(0.326504699469)+(-0.496247804966)*0.5**o
        ref[1]=(-0.703489566047)/(o+1.)+(-0.0886060128866)+(0.274829833844)*0.5**o
      res=integrate(arg)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactOne_fromSymbol_FunctionOnContactOne_rank2(self):
      """
      tests integral of rank 2 Symbol on the FunctionOnContactOne

      assumptions: FunctionOnContactOne(self.domain) exists
                   self.domain supports integral on FunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      w=FunctionOnContactOne(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5))
      s=Data(0,(4, 5),w)
      ref=numpy.zeros((4, 5),numpy.float_)
      if dim==2:
        s[0,0]=(0.997138766521)*x[0]**o+(-0.553379306564)*x[0]+(-0.234324119126)*x[1]**o+(-0.355669375308)*x[1]
        s[0,1]=(-0.0722078180457)*x[0]**o+(0.29155764813)*x[0]+(0.30568465631)*x[1]**o+(-0.977462121264)*x[1]
        s[0,2]=(0.0627729453295)*x[0]**o+(0.522234421268)*x[0]+(-0.265521199465)*x[1]**o+(0.0773272883492)*x[1]
        s[0,3]=(0.120580856562)*x[0]**o+(-0.576605249922)*x[0]+(-0.515839258724)*x[1]**o+(0.364676391176)*x[1]
        s[0,4]=(-0.369132454849)*x[0]**o+(-0.304739979654)*x[0]+(0.731091855933)*x[1]**o+(0.312380588453)*x[1]
        s[1,0]=(-0.277744449846)*x[0]**o+(0.963258283702)*x[0]+(0.0646683091177)*x[1]**o+(0.508336130337)*x[1]
        s[1,1]=(0.879838558141)*x[0]**o+(-0.575726599149)*x[0]+(-0.770954127597)*x[1]**o+(-0.29742399168)*x[1]
        s[1,2]=(0.105461068455)*x[0]**o+(0.325901606009)*x[0]+(-0.787936986763)*x[1]**o+(0.275010392849)*x[1]
        s[1,3]=(0.634687837213)*x[0]**o+(0.449318030603)*x[0]+(0.113843972437)*x[1]**o+(0.135522890516)*x[1]
        s[1,4]=(-0.447823860246)*x[0]**o+(-0.980688332644)*x[0]+(0.791824549606)*x[1]**o+(-0.171760132019)*x[1]
        s[2,0]=(0.61803013069)*x[0]**o+(-0.539345725331)*x[0]+(0.552856742671)*x[1]**o+(0.760667455882)*x[1]
        s[2,1]=(0.707626098607)*x[0]**o+(0.682940904637)*x[0]+(-0.057070815024)*x[1]**o+(-0.889583848012)*x[1]
        s[2,2]=(0.0226275928114)*x[0]**o+(-0.58368440817)*x[0]+(0.503278007027)*x[1]**o+(0.607225060945)*x[1]
        s[2,3]=(-0.643463757169)*x[0]**o+(0.166312879954)*x[0]+(0.889781254515)*x[1]**o+(-0.820343616323)*x[1]
        s[2,4]=(-0.920378124042)*x[0]**o+(-0.879688658227)*x[0]+(0.578052078637)*x[1]**o+(0.895382276619)*x[1]
        s[3,0]=(-0.748327596968)*x[0]**o+(-0.613914500566)*x[0]+(-0.0478077540856)*x[1]**o+(-0.838510710621)*x[1]
        s[3,1]=(-0.831715563374)*x[0]**o+(0.177460645482)*x[0]+(-0.19690401481)*x[1]**o+(-0.180828782822)*x[1]
        s[3,2]=(0.0692984740547)*x[0]**o+(-0.798996415727)*x[0]+(-0.193350054474)*x[1]**o+(-0.160594229856)*x[1]
        s[3,3]=(0.989346428804)*x[0]**o+(0.285816940366)*x[0]+(-0.296547093918)*x[1]**o+(-0.0529210232469)*x[1]
        s[3,4]=(0.24851689771)*x[0]**o+(0.452794657799)*x[0]+(-0.555079559)*x[1]**o+(-0.336250468184)*x[1]
        ref[0,0]=(-0.234324119126)/(o+1.)+(-0.454524340936)+(0.997138766521)*0.5**o
        ref[0,1]=(0.30568465631)/(o+1.)+(-0.342952236567)+(-0.0722078180457)*0.5**o
        ref[0,2]=(-0.265521199465)/(o+1.)+(0.299780854808)+(0.0627729453295)*0.5**o
        ref[0,3]=(-0.515839258724)/(o+1.)+(-0.105964429373)+(0.120580856562)*0.5**o
        ref[0,4]=(0.731091855933)/(o+1.)+(0.00382030439927)+(-0.369132454849)*0.5**o
        ref[1,0]=(0.0646683091177)/(o+1.)+(0.73579720702)+(-0.277744449846)*0.5**o
        ref[1,1]=(-0.770954127597)/(o+1.)+(-0.436575295414)+(0.879838558141)*0.5**o
        ref[1,2]=(-0.787936986763)/(o+1.)+(0.300455999429)+(0.105461068455)*0.5**o
        ref[1,3]=(0.113843972437)/(o+1.)+(0.292420460559)+(0.634687837213)*0.5**o
        ref[1,4]=(0.791824549606)/(o+1.)+(-0.576224232331)+(-0.447823860246)*0.5**o
        ref[2,0]=(0.552856742671)/(o+1.)+(0.110660865275)+(0.61803013069)*0.5**o
        ref[2,1]=(-0.057070815024)/(o+1.)+(-0.103321471687)+(0.707626098607)*0.5**o
        ref[2,2]=(0.503278007027)/(o+1.)+(0.0117703263876)+(0.0226275928114)*0.5**o
        ref[2,3]=(0.889781254515)/(o+1.)+(-0.327015368185)+(-0.643463757169)*0.5**o
        ref[2,4]=(0.578052078637)/(o+1.)+(0.00784680919582)+(-0.920378124042)*0.5**o
        ref[3,0]=(-0.0478077540856)/(o+1.)+(-0.726212605594)+(-0.748327596968)*0.5**o
        ref[3,1]=(-0.19690401481)/(o+1.)+(-0.00168406867019)+(-0.831715563374)*0.5**o
        ref[3,2]=(-0.193350054474)/(o+1.)+(-0.479795322792)+(0.0692984740547)*0.5**o
        ref[3,3]=(-0.296547093918)/(o+1.)+(0.11644795856)+(0.989346428804)*0.5**o
        ref[3,4]=(-0.555079559)/(o+1.)+(0.0582720948078)+(0.24851689771)*0.5**o
      else:
        s[0,0]=(-0.885439150233)*x[0]**o+(-0.0707651237907)*x[0]+(-0.0221938474237)*x[1]**o+(-0.657402959688)*x[1]+(0.492040451029)*x[2]**o+(-0.500851664471)*x[2]
        s[0,1]=(-0.915950017556)*x[0]**o+(-0.223230342138)*x[0]+(0.251513308151)*x[1]**o+(0.340647147217)*x[1]+(0.785842086262)*x[2]**o+(-0.375289319286)*x[2]
        s[0,2]=(0.0171536999294)*x[0]**o+(0.187222850003)*x[0]+(0.923344343515)*x[1]**o+(0.919899906728)*x[1]+(0.330573465731)*x[2]**o+(0.12075255513)*x[2]
        s[0,3]=(-0.490667671112)*x[0]**o+(0.4056039259)*x[0]+(-0.955154570277)*x[1]**o+(0.986366754754)*x[1]+(-0.853519593222)*x[2]**o+(-0.825901616105)*x[2]
        s[0,4]=(-0.253213854519)*x[0]**o+(-0.401738411717)*x[0]+(0.872412916488)*x[1]**o+(-0.00224326832344)*x[1]+(0.480217592874)*x[2]**o+(0.144994806066)*x[2]
        s[1,0]=(0.131992177305)*x[0]**o+(0.788963820881)*x[0]+(-0.0198901140504)*x[1]**o+(0.731616952503)*x[1]+(0.59309950089)*x[2]**o+(0.694630924739)*x[2]
        s[1,1]=(0.88075735692)*x[0]**o+(0.678228523078)*x[0]+(-0.682632234228)*x[1]**o+(-0.888294066081)*x[1]+(-0.302130074515)*x[2]**o+(-0.539380276098)*x[2]
        s[1,2]=(0.371868372987)*x[0]**o+(0.287766535066)*x[0]+(0.0885117264942)*x[1]**o+(0.249213255154)*x[1]+(-0.0473853043874)*x[2]**o+(0.345934167493)*x[2]
        s[1,3]=(-0.580406999101)*x[0]**o+(0.762553897732)*x[0]+(0.498019309339)*x[1]**o+(-0.275962658061)*x[1]+(-0.232793721572)*x[2]**o+(-0.404455792741)*x[2]
        s[1,4]=(-0.193670730589)*x[0]**o+(0.107221166466)*x[0]+(-0.253421260284)*x[1]**o+(-0.630230774271)*x[1]+(0.569627074227)*x[2]**o+(-0.471574680318)*x[2]
        s[2,0]=(-0.734235834531)*x[0]**o+(-0.0769718054424)*x[0]+(0.337185508531)*x[1]**o+(-0.695399200032)*x[1]+(0.670045013654)*x[2]**o+(0.602223461093)*x[2]
        s[2,1]=(0.40853552724)*x[0]**o+(-0.109296862931)*x[0]+(-0.898346018309)*x[1]**o+(0.797317601123)*x[1]+(-0.243636906258)*x[2]**o+(-0.7404427898)*x[2]
        s[2,2]=(-0.300099773564)*x[0]**o+(0.391661796552)*x[0]+(0.081340069378)*x[1]**o+(-0.540467785776)*x[1]+(0.609927398687)*x[2]**o+(0.342702876243)*x[2]
        s[2,3]=(-0.795553246099)*x[0]**o+(0.12091791031)*x[0]+(-0.358760047024)*x[1]**o+(0.00992547080721)*x[1]+(-0.776142821278)*x[2]**o+(0.960839850387)*x[2]
        s[2,4]=(0.0312887697988)*x[0]**o+(0.192922412706)*x[0]+(0.930373533919)*x[1]**o+(0.565991224895)*x[1]+(-0.620790693282)*x[2]**o+(-0.0894617381169)*x[2]
        s[3,0]=(0.58118631126)*x[0]**o+(0.0951313787407)*x[0]+(-0.481475099458)*x[1]**o+(-0.844977136281)*x[1]+(-0.505637296865)*x[2]**o+(-0.117240523527)*x[2]
        s[3,1]=(0.0560022697918)*x[0]**o+(-0.667387267392)*x[0]+(0.18270168123)*x[1]**o+(0.820389934276)*x[1]+(0.541891208803)*x[2]**o+(-0.813083374651)*x[2]
        s[3,2]=(-0.217967373894)*x[0]**o+(0.0740362777592)*x[0]+(-0.532958237386)*x[1]**o+(0.254923920582)*x[1]+(0.0983414559274)*x[2]**o+(-0.330473156862)*x[2]
        s[3,3]=(-0.51343561245)*x[0]**o+(-0.314818156795)*x[0]+(0.349539891727)*x[1]**o+(0.182246887466)*x[1]+(0.378368043277)*x[2]**o+(0.394897385803)*x[2]
        s[3,4]=(-0.440074964176)*x[0]**o+(0.644345562557)*x[0]+(0.157396506088)*x[1]**o+(-0.446161901195)*x[1]+(0.0406752725721)*x[2]**o+(0.00497105229147)*x[2]
        ref[0,0]=(0.469846603605)/(o+1.)+(-0.614509873975)+(-0.885439150233)*0.5**o
        ref[0,1]=(1.03735539441)/(o+1.)+(-0.128936257103)+(-0.915950017556)*0.5**o
        ref[0,2]=(1.25391780925)/(o+1.)+(0.613937655931)+(0.0171536999294)*0.5**o
        ref[0,3]=(-1.8086741635)/(o+1.)+(0.283034532275)+(-0.490667671112)*0.5**o
        ref[0,4]=(1.35263050936)/(o+1.)+(-0.129493436987)+(-0.253213854519)*0.5**o
        ref[1,0]=(0.57320938684)/(o+1.)+(1.10760584906)+(0.131992177305)*0.5**o
        ref[1,1]=(-0.984762308743)/(o+1.)+(-0.37472290955)+(0.88075735692)*0.5**o
        ref[1,2]=(0.0411264221069)/(o+1.)+(0.441456978856)+(0.371868372987)*0.5**o
        ref[1,3]=(0.265225587768)/(o+1.)+(0.0410677234649)+(-0.580406999101)*0.5**o
        ref[1,4]=(0.316205813942)/(o+1.)+(-0.497292144061)+(-0.193670730589)*0.5**o
        ref[2,0]=(1.00723052218)/(o+1.)+(-0.0850737721911)+(-0.734235834531)*0.5**o
        ref[2,1]=(-1.14198292457)/(o+1.)+(-0.0262110258043)+(0.40853552724)*0.5**o
        ref[2,2]=(0.691267468065)/(o+1.)+(0.0969484435098)+(-0.300099773564)*0.5**o
        ref[2,3]=(-1.1349028683)/(o+1.)+(0.545841615752)+(-0.795553246099)*0.5**o
        ref[2,4]=(0.309582840637)/(o+1.)+(0.334725949742)+(0.0312887697988)*0.5**o
        ref[3,0]=(-0.987112396323)/(o+1.)+(-0.433543140534)+(0.58118631126)*0.5**o
        ref[3,1]=(0.724592890033)/(o+1.)+(-0.330040353884)+(0.0560022697918)*0.5**o
        ref[3,2]=(-0.434616781458)/(o+1.)+(-0.000756479260437)+(-0.217967373894)*0.5**o
        ref[3,3]=(0.727907935004)/(o+1.)+(0.131163058237)+(-0.51343561245)*0.5**o
        ref[3,4]=(0.19807177866)/(o+1.)+(0.101577356827)+(-0.440074964176)*0.5**o
      res=integrate(arg)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactOne_fromSymbol_FunctionOnContactOne_rank3(self):
      """
      tests integral of rank 3 Symbol on the FunctionOnContactOne

      assumptions: FunctionOnContactOne(self.domain) exists
                   self.domain supports integral on FunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      w=FunctionOnContactOne(self.domain)
      x=w.getX()
      arg=Symbol(shape=(6, 2, 2))
      s=Data(0,(6, 2, 2),w)
      ref=numpy.zeros((6, 2, 2),numpy.float_)
      if dim==2:
        s[0,0,0]=(-0.0899144489202)*x[0]**o+(-0.939343727238)*x[0]+(-0.838709152888)*x[1]**o+(-0.695938051041)*x[1]
        s[0,0,1]=(0.0915399473414)*x[0]**o+(-0.088933669169)*x[0]+(0.569689228579)*x[1]**o+(0.52563406191)*x[1]
        s[0,1,0]=(-0.0755502530652)*x[0]**o+(0.224201529435)*x[0]+(-0.0936878789524)*x[1]**o+(0.678476395607)*x[1]
        s[0,1,1]=(-0.722700224525)*x[0]**o+(-0.428992070285)*x[0]+(-0.967545686857)*x[1]**o+(0.339344032355)*x[1]
        s[1,0,0]=(-0.807565232378)*x[0]**o+(-0.389243328274)*x[0]+(-0.188155419411)*x[1]**o+(0.801573366554)*x[1]
        s[1,0,1]=(-0.217662099148)*x[0]**o+(0.0709316329807)*x[0]+(0.240070973919)*x[1]**o+(0.0450329893887)*x[1]
        s[1,1,0]=(-0.211774876096)*x[0]**o+(-0.585658640262)*x[0]+(0.772212206058)*x[1]**o+(0.471350730303)*x[1]
        s[1,1,1]=(-0.251097154538)*x[0]**o+(-0.936636380186)*x[0]+(-0.29953620509)*x[1]**o+(-0.7023587162)*x[1]
        s[2,0,0]=(-0.226008179803)*x[0]**o+(0.11175986541)*x[0]+(-0.815791532209)*x[1]**o+(0.661034404078)*x[1]
        s[2,0,1]=(-0.347375034321)*x[0]**o+(-0.38889482021)*x[0]+(-0.298497991695)*x[1]**o+(0.905510136504)*x[1]
        s[2,1,0]=(-0.414130770151)*x[0]**o+(0.00845860563915)*x[0]+(-0.622228227201)*x[1]**o+(0.239872393913)*x[1]
        s[2,1,1]=(0.985892855782)*x[0]**o+(-0.601178096316)*x[0]+(-0.671498979354)*x[1]**o+(0.600179658495)*x[1]
        s[3,0,0]=(-0.547441466002)*x[0]**o+(-0.390199968809)*x[0]+(0.356244378791)*x[1]**o+(-0.91693977081)*x[1]
        s[3,0,1]=(-0.317803998398)*x[0]**o+(-0.212192581141)*x[0]+(0.942307305401)*x[1]**o+(-0.528247885421)*x[1]
        s[3,1,0]=(-0.269300473712)*x[0]**o+(-0.806351998808)*x[0]+(-0.437943014771)*x[1]**o+(0.308252458482)*x[1]
        s[3,1,1]=(0.0826330889069)*x[0]**o+(0.357329175375)*x[0]+(-0.866644131299)*x[1]**o+(-0.993298142744)*x[1]
        s[4,0,0]=(0.478312250921)*x[0]**o+(0.949219183322)*x[0]+(-0.408977166217)*x[1]**o+(0.525189299992)*x[1]
        s[4,0,1]=(-0.221152804907)*x[0]**o+(0.683685107755)*x[0]+(-0.878859546858)*x[1]**o+(0.313936907822)*x[1]
        s[4,1,0]=(-0.361901807816)*x[0]**o+(0.473080571341)*x[0]+(0.0726134206382)*x[1]**o+(-0.552823213053)*x[1]
        s[4,1,1]=(-0.729188028594)*x[0]**o+(0.742375152366)*x[0]+(-0.11559899139)*x[1]**o+(-0.408146075502)*x[1]
        s[5,0,0]=(0.492678269676)*x[0]**o+(0.683105631523)*x[0]+(-0.0108127700464)*x[1]**o+(-0.0516040293712)*x[1]
        s[5,0,1]=(-0.0972486870808)*x[0]**o+(-0.354724382566)*x[0]+(0.383281344894)*x[1]**o+(0.547336907663)*x[1]
        s[5,1,0]=(0.442321568121)*x[0]**o+(-0.776464559839)*x[0]+(0.00851654852488)*x[1]**o+(-0.0830181237664)*x[1]
        s[5,1,1]=(-0.723222258936)*x[0]**o+(-0.294417512199)*x[0]+(-0.141223730539)*x[1]**o+(-0.757548646984)*x[1]
        ref[0,0,0]=(-0.838709152888)/(o+1.)+(-0.81764088914)+(-0.0899144489202)*0.5**o
        ref[0,0,1]=(0.569689228579)/(o+1.)+(0.21835019637)+(0.0915399473414)*0.5**o
        ref[0,1,0]=(-0.0936878789524)/(o+1.)+(0.451338962521)+(-0.0755502530652)*0.5**o
        ref[0,1,1]=(-0.967545686857)/(o+1.)+(-0.0448240189648)+(-0.722700224525)*0.5**o
        ref[1,0,0]=(-0.188155419411)/(o+1.)+(0.20616501914)+(-0.807565232378)*0.5**o
        ref[1,0,1]=(0.240070973919)/(o+1.)+(0.0579823111847)+(-0.217662099148)*0.5**o
        ref[1,1,0]=(0.772212206058)/(o+1.)+(-0.0571539549792)+(-0.211774876096)*0.5**o
        ref[1,1,1]=(-0.29953620509)/(o+1.)+(-0.819497548193)+(-0.251097154538)*0.5**o
        ref[2,0,0]=(-0.815791532209)/(o+1.)+(0.386397134744)+(-0.226008179803)*0.5**o
        ref[2,0,1]=(-0.298497991695)/(o+1.)+(0.258307658147)+(-0.347375034321)*0.5**o
        ref[2,1,0]=(-0.622228227201)/(o+1.)+(0.124165499776)+(-0.414130770151)*0.5**o
        ref[2,1,1]=(-0.671498979354)/(o+1.)+(-0.000499218910338)+(0.985892855782)*0.5**o
        ref[3,0,0]=(0.356244378791)/(o+1.)+(-0.653569869809)+(-0.547441466002)*0.5**o
        ref[3,0,1]=(0.942307305401)/(o+1.)+(-0.370220233281)+(-0.317803998398)*0.5**o
        ref[3,1,0]=(-0.437943014771)/(o+1.)+(-0.249049770163)+(-0.269300473712)*0.5**o
        ref[3,1,1]=(-0.866644131299)/(o+1.)+(-0.317984483685)+(0.0826330889069)*0.5**o
        ref[4,0,0]=(-0.408977166217)/(o+1.)+(0.737204241657)+(0.478312250921)*0.5**o
        ref[4,0,1]=(-0.878859546858)/(o+1.)+(0.498811007788)+(-0.221152804907)*0.5**o
        ref[4,1,0]=(0.0726134206382)/(o+1.)+(-0.0398713208557)+(-0.361901807816)*0.5**o
        ref[4,1,1]=(-0.11559899139)/(o+1.)+(0.167114538432)+(-0.729188028594)*0.5**o
        ref[5,0,0]=(-0.0108127700464)/(o+1.)+(0.315750801076)+(0.492678269676)*0.5**o
        ref[5,0,1]=(0.383281344894)/(o+1.)+(0.0963062625484)+(-0.0972486870808)*0.5**o
        ref[5,1,0]=(0.00851654852488)/(o+1.)+(-0.429741341803)+(0.442321568121)*0.5**o
        ref[5,1,1]=(-0.141223730539)/(o+1.)+(-0.525983079592)+(-0.723222258936)*0.5**o
      else:
        s[0,0,0]=(0.242677714297)*x[0]**o+(-0.400213939273)*x[0]+(0.406302331134)*x[1]**o+(0.0334900110197)*x[1]+(0.386616491205)*x[2]**o+(0.0312866694695)*x[2]
        s[0,0,1]=(-0.656430796333)*x[0]**o+(-0.563403496155)*x[0]+(0.364261714172)*x[1]**o+(0.967871502294)*x[1]+(-0.172721573545)*x[2]**o+(0.338588840536)*x[2]
        s[0,1,0]=(-0.204331459058)*x[0]**o+(-0.445139554107)*x[0]+(0.75436650149)*x[1]**o+(-0.932237793601)*x[1]+(0.767285047485)*x[2]**o+(-0.447011217796)*x[2]
        s[0,1,1]=(0.117575733479)*x[0]**o+(-0.474180475568)*x[0]+(0.259955578397)*x[1]**o+(-0.988666169557)*x[1]+(-0.773227945982)*x[2]**o+(-0.228748677163)*x[2]
        s[1,0,0]=(-0.885708131129)*x[0]**o+(0.255910330816)*x[0]+(-0.160503663494)*x[1]**o+(0.444458737043)*x[1]+(0.501800660641)*x[2]**o+(-0.403629782618)*x[2]
        s[1,0,1]=(-0.302400007694)*x[0]**o+(0.0508740505816)*x[0]+(0.149935410009)*x[1]**o+(-0.333426355106)*x[1]+(-0.83475153405)*x[2]**o+(-0.180357923768)*x[2]
        s[1,1,0]=(0.0405056122891)*x[0]**o+(-0.679078314383)*x[0]+(-0.630686479421)*x[1]**o+(0.882857130415)*x[1]+(-0.880049081361)*x[2]**o+(0.894022667917)*x[2]
        s[1,1,1]=(0.987734368607)*x[0]**o+(0.202787820601)*x[0]+(0.201427368636)*x[1]**o+(-0.888074805769)*x[1]+(-0.00577238788728)*x[2]**o+(-0.852216657415)*x[2]
        s[2,0,0]=(0.0784793303496)*x[0]**o+(0.525537564231)*x[0]+(-0.743068512178)*x[1]**o+(-0.96875188044)*x[1]+(0.598270641364)*x[2]**o+(-0.738260965829)*x[2]
        s[2,0,1]=(0.0371151946549)*x[0]**o+(0.834712924788)*x[0]+(0.831916271847)*x[1]**o+(0.986042089018)*x[1]+(-0.271258489529)*x[2]**o+(-0.0270137352729)*x[2]
        s[2,1,0]=(-0.610486701052)*x[0]**o+(0.747490812106)*x[0]+(-0.137581460043)*x[1]**o+(0.764360090052)*x[1]+(-0.992754974221)*x[2]**o+(-0.170181357609)*x[2]
        s[2,1,1]=(0.0760419308376)*x[0]**o+(-0.610739481473)*x[0]+(0.268671321666)*x[1]**o+(0.556159088599)*x[1]+(0.16105750649)*x[2]**o+(0.168476671552)*x[2]
        s[3,0,0]=(0.332144310632)*x[0]**o+(0.645502258606)*x[0]+(-0.303557252905)*x[1]**o+(-0.992657076111)*x[1]+(-0.367089118757)*x[2]**o+(-0.755868370964)*x[2]
        s[3,0,1]=(-0.538228335462)*x[0]**o+(0.853705065724)*x[0]+(-0.608592283692)*x[1]**o+(0.29533491918)*x[1]+(0.226379502601)*x[2]**o+(-0.20850693991)*x[2]
        s[3,1,0]=(-0.33389690791)*x[0]**o+(-0.804233206619)*x[0]+(0.787650315768)*x[1]**o+(-0.333017031742)*x[1]+(-0.803108583785)*x[2]**o+(-0.450538082625)*x[2]
        s[3,1,1]=(0.491492662254)*x[0]**o+(0.171534400605)*x[0]+(-0.777282924417)*x[1]**o+(-0.572907396269)*x[1]+(0.785138934858)*x[2]**o+(-0.436325021186)*x[2]
        s[4,0,0]=(0.247706032995)*x[0]**o+(0.905275953458)*x[0]+(0.0609169269224)*x[1]**o+(-0.256774805033)*x[1]+(0.998972177574)*x[2]**o+(-0.868940150916)*x[2]
        s[4,0,1]=(0.118732551906)*x[0]**o+(-0.622269538749)*x[0]+(0.118525354684)*x[1]**o+(-0.849217003233)*x[1]+(0.685082692712)*x[2]**o+(-0.230753706424)*x[2]
        s[4,1,0]=(-0.590456775201)*x[0]**o+(0.631958889331)*x[0]+(-0.0714558191005)*x[1]**o+(-0.336225807259)*x[1]+(0.896478419817)*x[2]**o+(0.376282690958)*x[2]
        s[4,1,1]=(0.437488625548)*x[0]**o+(0.883207845535)*x[0]+(0.594979146)*x[1]**o+(-0.627687609281)*x[1]+(0.9812267289)*x[2]**o+(0.571200283614)*x[2]
        s[5,0,0]=(-0.4090238662)*x[0]**o+(-0.712922930229)*x[0]+(-0.356126501112)*x[1]**o+(0.362147566863)*x[1]+(0.164164629436)*x[2]**o+(0.85169934858)*x[2]
        s[5,0,1]=(0.137560590284)*x[0]**o+(0.00568020697538)*x[0]+(-0.176059142701)*x[1]**o+(0.386227905303)*x[1]+(-0.823564872242)*x[2]**o+(-0.956332461174)*x[2]
        s[5,1,0]=(-0.719767553566)*x[0]**o+(-0.595774805843)*x[0]+(-0.783953676642)*x[1]**o+(0.686259147413)*x[1]+(-0.254758248118)*x[2]**o+(0.574267022665)*x[2]
        s[5,1,1]=(-0.321911973511)*x[0]**o+(-0.336478066213)*x[0]+(-0.777795569678)*x[1]**o+(0.896311892021)*x[1]+(-0.516413038874)*x[2]**o+(-0.786093273801)*x[2]
        ref[0,0,0]=(0.792918822339)/(o+1.)+(-0.167718629392)+(0.242677714297)*0.5**o
        ref[0,0,1]=(0.191540140628)/(o+1.)+(0.371528423337)+(-0.656430796333)*0.5**o
        ref[0,1,0]=(1.52165154897)/(o+1.)+(-0.912194282752)+(-0.204331459058)*0.5**o
        ref[0,1,1]=(-0.513272367585)/(o+1.)+(-0.845797661144)+(0.117575733479)*0.5**o
        ref[1,0,0]=(0.341296997147)/(o+1.)+(0.148369642621)+(-0.885708131129)*0.5**o
        ref[1,0,1]=(-0.684816124041)/(o+1.)+(-0.231455114146)+(-0.302400007694)*0.5**o
        ref[1,1,0]=(-1.51073556078)/(o+1.)+(0.548900741975)+(0.0405056122891)*0.5**o
        ref[1,1,1]=(0.195654980748)/(o+1.)+(-0.768751821292)+(0.987734368607)*0.5**o
        ref[2,0,0]=(-0.144797870814)/(o+1.)+(-0.590737641019)+(0.0784793303496)*0.5**o
        ref[2,0,1]=(0.560657782318)/(o+1.)+(0.896870639267)+(0.0371151946549)*0.5**o
        ref[2,1,0]=(-1.13033643426)/(o+1.)+(0.670834772275)+(-0.610486701052)*0.5**o
        ref[2,1,1]=(0.429728828155)/(o+1.)+(0.0569481393388)+(0.0760419308376)*0.5**o
        ref[3,0,0]=(-0.670646371662)/(o+1.)+(-0.551511594234)+(0.332144310632)*0.5**o
        ref[3,0,1]=(-0.382212781091)/(o+1.)+(0.470266522497)+(-0.538228335462)*0.5**o
        ref[3,1,0]=(-0.0154582680178)/(o+1.)+(-0.793894160493)+(-0.33389690791)*0.5**o
        ref[3,1,1]=(0.00785601044065)/(o+1.)+(-0.418849008425)+(0.491492662254)*0.5**o
        ref[4,0,0]=(1.0598891045)/(o+1.)+(-0.110219501245)+(0.247706032995)*0.5**o
        ref[4,0,1]=(0.803608047396)/(o+1.)+(-0.851120124203)+(0.118732551906)*0.5**o
        ref[4,1,0]=(0.825022600717)/(o+1.)+(0.336007886515)+(-0.590456775201)*0.5**o
        ref[4,1,1]=(1.5762058749)/(o+1.)+(0.413360259934)+(0.437488625548)*0.5**o
        ref[5,0,0]=(-0.191961871676)/(o+1.)+(0.250461992607)+(-0.4090238662)*0.5**o
        ref[5,0,1]=(-0.999624014943)/(o+1.)+(-0.282212174448)+(0.137560590284)*0.5**o
        ref[5,1,0]=(-1.03871192476)/(o+1.)+(0.332375682117)+(-0.719767553566)*0.5**o
        ref[5,1,1]=(-1.29420860855)/(o+1.)+(-0.113129723997)+(-0.321911973511)*0.5**o
      res=integrate(arg)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onFunctionOnContactOne_fromSymbol_FunctionOnContactOne_rank4(self):
      """
      tests integral of rank 4 Symbol on the FunctionOnContactOne

      assumptions: FunctionOnContactOne(self.domain) exists
                   self.domain supports integral on FunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      w=FunctionOnContactOne(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5, 3, 2))
      s=Data(0,(4, 5, 3, 2),w)
      ref=numpy.zeros((4, 5, 3, 2),numpy.float_)
      if dim==2:
        s[0,0,0,0]=(0.245364066984)*x[0]**o+(-0.0501894287161)*x[0]+(0.666238426788)*x[1]**o+(0.136096078492)*x[1]
        s[0,0,0,1]=(0.716203034408)*x[0]**o+(-0.816588231639)*x[0]+(0.800426762254)*x[1]**o+(-0.182765397296)*x[1]
        s[0,0,1,0]=(0.122659986993)*x[0]**o+(-0.00208237425405)*x[0]+(-0.347405577261)*x[1]**o+(0.0376630412624)*x[1]
        s[0,0,1,1]=(0.791965912204)*x[0]**o+(0.935296430194)*x[0]+(0.330494636783)*x[1]**o+(0.990030993503)*x[1]
        s[0,0,2,0]=(0.761523262625)*x[0]**o+(0.47673649822)*x[0]+(-0.814581462689)*x[1]**o+(0.172965644708)*x[1]
        s[0,0,2,1]=(0.384651054435)*x[0]**o+(-0.723316715392)*x[0]+(0.349412253363)*x[1]**o+(-0.687923947756)*x[1]
        s[0,1,0,0]=(0.114523224402)*x[0]**o+(0.0462112886306)*x[0]+(-0.770917205125)*x[1]**o+(-0.251081721462)*x[1]
        s[0,1,0,1]=(0.697263286229)*x[0]**o+(-0.0252158836846)*x[0]+(0.933694211325)*x[1]**o+(0.650498198726)*x[1]
        s[0,1,1,0]=(-0.215924750352)*x[0]**o+(0.97660438665)*x[0]+(0.358477857654)*x[1]**o+(-0.603580756297)*x[1]
        s[0,1,1,1]=(0.642382357196)*x[0]**o+(0.891810567307)*x[0]+(0.839137774823)*x[1]**o+(-0.492580774398)*x[1]
        s[0,1,2,0]=(-0.260657110217)*x[0]**o+(-0.71383748794)*x[0]+(0.846765712381)*x[1]**o+(-0.686699503594)*x[1]
        s[0,1,2,1]=(0.733925405267)*x[0]**o+(-0.318186624057)*x[0]+(-0.690232848808)*x[1]**o+(0.941086052943)*x[1]
        s[0,2,0,0]=(0.230747459963)*x[0]**o+(0.873973601431)*x[0]+(0.0114100757165)*x[1]**o+(-0.172280871128)*x[1]
        s[0,2,0,1]=(0.191593969982)*x[0]**o+(-0.275332585321)*x[0]+(-0.403823163425)*x[1]**o+(0.765493218693)*x[1]
        s[0,2,1,0]=(-0.954865298124)*x[0]**o+(-0.840274453058)*x[0]+(-0.575828297224)*x[1]**o+(0.543288135427)*x[1]
        s[0,2,1,1]=(-0.304205093844)*x[0]**o+(0.366868842463)*x[0]+(0.950680046864)*x[1]**o+(0.644901674439)*x[1]
        s[0,2,2,0]=(-0.46074924128)*x[0]**o+(-0.100058859295)*x[0]+(-0.172613322727)*x[1]**o+(-0.10169890762)*x[1]
        s[0,2,2,1]=(0.119499701836)*x[0]**o+(-0.301349539686)*x[0]+(0.0936397977601)*x[1]**o+(0.361269456081)*x[1]
        s[0,3,0,0]=(-0.525741080321)*x[0]**o+(-0.680808261787)*x[0]+(0.0553816248689)*x[1]**o+(0.478185322022)*x[1]
        s[0,3,0,1]=(-0.241502398998)*x[0]**o+(0.831483674337)*x[0]+(-0.862336855278)*x[1]**o+(0.847378846767)*x[1]
        s[0,3,1,0]=(0.925044786414)*x[0]**o+(-0.20650710731)*x[0]+(-0.459505697865)*x[1]**o+(0.322417326119)*x[1]
        s[0,3,1,1]=(0.571196115307)*x[0]**o+(0.0591802893515)*x[0]+(0.880717796832)*x[1]**o+(0.813021896995)*x[1]
        s[0,3,2,0]=(-0.878805794321)*x[0]**o+(-0.404133715278)*x[0]+(-0.538866339841)*x[1]**o+(-0.51964634727)*x[1]
        s[0,3,2,1]=(0.968894119383)*x[0]**o+(0.535010672206)*x[0]+(-0.543638502129)*x[1]**o+(-0.0775549893758)*x[1]
        s[0,4,0,0]=(-0.473571740079)*x[0]**o+(0.981704155322)*x[0]+(-0.732685132335)*x[1]**o+(0.151201399401)*x[1]
        s[0,4,0,1]=(0.107676577578)*x[0]**o+(-0.0898161469871)*x[0]+(-0.243173822645)*x[1]**o+(0.425562025311)*x[1]
        s[0,4,1,0]=(-0.912740962221)*x[0]**o+(-0.307527338956)*x[0]+(-0.482823172795)*x[1]**o+(-0.836767222685)*x[1]
        s[0,4,1,1]=(-0.106203879274)*x[0]**o+(0.0999297363975)*x[0]+(0.605357795695)*x[1]**o+(-0.864222319735)*x[1]
        s[0,4,2,0]=(0.587529237973)*x[0]**o+(0.140953847793)*x[0]+(0.64759005987)*x[1]**o+(-0.0275484404904)*x[1]
        s[0,4,2,1]=(0.482928938234)*x[0]**o+(0.775154595317)*x[0]+(0.898533268559)*x[1]**o+(-0.238327086159)*x[1]
        s[1,0,0,0]=(-0.574069031279)*x[0]**o+(-0.59623538758)*x[0]+(0.388920605463)*x[1]**o+(-0.994406853141)*x[1]
        s[1,0,0,1]=(0.939040505856)*x[0]**o+(-0.038754823699)*x[0]+(-0.279256974829)*x[1]**o+(-0.890808737563)*x[1]
        s[1,0,1,0]=(-0.944753068273)*x[0]**o+(-0.696943036951)*x[0]+(-0.220060979436)*x[1]**o+(0.223607800841)*x[1]
        s[1,0,1,1]=(-0.390329725129)*x[0]**o+(-0.262404608658)*x[0]+(-0.421100034358)*x[1]**o+(0.922126389343)*x[1]
        s[1,0,2,0]=(-0.7832351829)*x[0]**o+(-0.155019228142)*x[0]+(-0.316428480376)*x[1]**o+(0.604192200421)*x[1]
        s[1,0,2,1]=(0.414384140175)*x[0]**o+(-0.000244010932926)*x[0]+(-0.46577339251)*x[1]**o+(0.869470477908)*x[1]
        s[1,1,0,0]=(-0.571703274446)*x[0]**o+(-0.897490452042)*x[0]+(0.151601513312)*x[1]**o+(-0.820411349281)*x[1]
        s[1,1,0,1]=(0.883610930016)*x[0]**o+(0.430543960795)*x[0]+(0.174748731929)*x[1]**o+(-0.346504921967)*x[1]
        s[1,1,1,0]=(-0.965747586125)*x[0]**o+(0.652587346149)*x[0]+(-0.454627455637)*x[1]**o+(-0.126065048744)*x[1]
        s[1,1,1,1]=(-0.778723024919)*x[0]**o+(-0.352555666852)*x[0]+(0.378393901391)*x[1]**o+(-0.393622640877)*x[1]
        s[1,1,2,0]=(0.785822593379)*x[0]**o+(0.374891382708)*x[0]+(-0.993366900714)*x[1]**o+(-0.406778939882)*x[1]
        s[1,1,2,1]=(0.238887912688)*x[0]**o+(-0.668847103585)*x[0]+(0.422750071837)*x[1]**o+(0.290969707905)*x[1]
        s[1,2,0,0]=(0.306452161355)*x[0]**o+(0.0845930872771)*x[0]+(0.288188272131)*x[1]**o+(0.611272197726)*x[1]
        s[1,2,0,1]=(0.535104365658)*x[0]**o+(-0.868361395667)*x[0]+(-0.557799747963)*x[1]**o+(0.189710523839)*x[1]
        s[1,2,1,0]=(0.653062501973)*x[0]**o+(-0.980469515167)*x[0]+(-0.126034299636)*x[1]**o+(-0.702252714095)*x[1]
        s[1,2,1,1]=(0.193513872604)*x[0]**o+(0.716250857589)*x[0]+(0.532447219707)*x[1]**o+(0.0396611049339)*x[1]
        s[1,2,2,0]=(0.471722480056)*x[0]**o+(-0.386703912908)*x[0]+(0.508907147346)*x[1]**o+(-0.431622849878)*x[1]
        s[1,2,2,1]=(0.145178046285)*x[0]**o+(-0.0110222671767)*x[0]+(-0.793600779578)*x[1]**o+(0.482117912584)*x[1]
        s[1,3,0,0]=(0.883901734656)*x[0]**o+(0.252857222204)*x[0]+(-0.349615695314)*x[1]**o+(-0.831352541736)*x[1]
        s[1,3,0,1]=(-0.439872366735)*x[0]**o+(0.855050596466)*x[0]+(-0.892859579327)*x[1]**o+(-0.499483379044)*x[1]
        s[1,3,1,0]=(-0.310902411804)*x[0]**o+(0.909694833553)*x[0]+(0.432470372195)*x[1]**o+(-0.181254854143)*x[1]
        s[1,3,1,1]=(-0.457038984067)*x[0]**o+(-0.932819775606)*x[0]+(-0.0215549839659)*x[1]**o+(-0.724030992167)*x[1]
        s[1,3,2,0]=(-0.307363570855)*x[0]**o+(-0.53814730682)*x[0]+(-0.0248867951466)*x[1]**o+(0.230518328746)*x[1]
        s[1,3,2,1]=(0.303121965084)*x[0]**o+(-0.493904339796)*x[0]+(0.917168913893)*x[1]**o+(0.169108758838)*x[1]
        s[1,4,0,0]=(-0.713672877707)*x[0]**o+(0.850083130815)*x[0]+(0.859962057023)*x[1]**o+(0.154568816707)*x[1]
        s[1,4,0,1]=(-0.708070236639)*x[0]**o+(0.264936920001)*x[0]+(0.376600276672)*x[1]**o+(0.880588901929)*x[1]
        s[1,4,1,0]=(0.203382859429)*x[0]**o+(0.834221276526)*x[0]+(-0.271980968262)*x[1]**o+(0.840406001263)*x[1]
        s[1,4,1,1]=(-0.610374454703)*x[0]**o+(-0.299939554248)*x[0]+(-0.876972535039)*x[1]**o+(0.613391714781)*x[1]
        s[1,4,2,0]=(-0.131008373859)*x[0]**o+(0.564430441667)*x[0]+(0.512449401496)*x[1]**o+(-0.072742245846)*x[1]
        s[1,4,2,1]=(0.994369815068)*x[0]**o+(0.214403792021)*x[0]+(-0.731489669807)*x[1]**o+(-0.778494326982)*x[1]
        s[2,0,0,0]=(0.687813573864)*x[0]**o+(-0.342489594146)*x[0]+(-0.092041610109)*x[1]**o+(-0.388723275983)*x[1]
        s[2,0,0,1]=(-0.889705548572)*x[0]**o+(-0.962296902814)*x[0]+(0.284484113765)*x[1]**o+(-0.312781157642)*x[1]
        s[2,0,1,0]=(-0.0691392356773)*x[0]**o+(0.391307995277)*x[0]+(-0.343166268962)*x[1]**o+(-0.100528288088)*x[1]
        s[2,0,1,1]=(-0.767504390408)*x[0]**o+(0.44775317228)*x[0]+(0.00928458441193)*x[1]**o+(0.481385818795)*x[1]
        s[2,0,2,0]=(0.151265878662)*x[0]**o+(0.934060057849)*x[0]+(0.772262051249)*x[1]**o+(0.0303447230092)*x[1]
        s[2,0,2,1]=(0.638051378377)*x[0]**o+(0.795393839494)*x[0]+(0.812739684548)*x[1]**o+(0.683335661484)*x[1]
        s[2,1,0,0]=(0.699425983168)*x[0]**o+(-0.113127267833)*x[0]+(0.753464652023)*x[1]**o+(-0.860186623524)*x[1]
        s[2,1,0,1]=(-0.228981326624)*x[0]**o+(-0.144953596548)*x[0]+(-0.028962291177)*x[1]**o+(0.274475259334)*x[1]
        s[2,1,1,0]=(0.436232377097)*x[0]**o+(0.851458153457)*x[0]+(0.449876871146)*x[1]**o+(0.929900037818)*x[1]
        s[2,1,1,1]=(0.108152697095)*x[0]**o+(0.701445596547)*x[0]+(0.857688044529)*x[1]**o+(-0.32024176733)*x[1]
        s[2,1,2,0]=(0.776899779272)*x[0]**o+(-0.77488067334)*x[0]+(0.515979568236)*x[1]**o+(0.60407405593)*x[1]
        s[2,1,2,1]=(0.486236495643)*x[0]**o+(0.759871578074)*x[0]+(0.320690937285)*x[1]**o+(-0.178307017545)*x[1]
        s[2,2,0,0]=(-0.2834031779)*x[0]**o+(-0.99059970496)*x[0]+(-0.951630400835)*x[1]**o+(-0.822318070375)*x[1]
        s[2,2,0,1]=(-0.246680973215)*x[0]**o+(0.895886965624)*x[0]+(0.331676331677)*x[1]**o+(0.275149795318)*x[1]
        s[2,2,1,0]=(0.682460024363)*x[0]**o+(-0.191950621609)*x[0]+(0.998873537117)*x[1]**o+(-0.62286471982)*x[1]
        s[2,2,1,1]=(0.371007993444)*x[0]**o+(0.75354844415)*x[0]+(0.231689506568)*x[1]**o+(0.319850912994)*x[1]
        s[2,2,2,0]=(0.273036358499)*x[0]**o+(-0.365860549733)*x[0]+(-0.255255520802)*x[1]**o+(-0.856177999361)*x[1]
        s[2,2,2,1]=(-0.829888508727)*x[0]**o+(-0.869979484799)*x[0]+(0.0422078078521)*x[1]**o+(-0.225105932622)*x[1]
        s[2,3,0,0]=(-0.901686992463)*x[0]**o+(0.52317892576)*x[0]+(0.450664578307)*x[1]**o+(0.539264486669)*x[1]
        s[2,3,0,1]=(-0.146310214982)*x[0]**o+(-0.866238792091)*x[0]+(-0.411575583885)*x[1]**o+(-0.767910846528)*x[1]
        s[2,3,1,0]=(0.796379240086)*x[0]**o+(-0.941611444233)*x[0]+(0.457329256522)*x[1]**o+(-0.205094413155)*x[1]
        s[2,3,1,1]=(0.00162132693744)*x[0]**o+(0.809781891022)*x[0]+(-0.521975026805)*x[1]**o+(-0.832935611569)*x[1]
        s[2,3,2,0]=(0.85913616245)*x[0]**o+(-0.97226394808)*x[0]+(-0.958220202856)*x[1]**o+(-0.838570412967)*x[1]
        s[2,3,2,1]=(-0.394075398541)*x[0]**o+(0.363958258657)*x[0]+(-0.0245518988658)*x[1]**o+(-0.0641697369324)*x[1]
        s[2,4,0,0]=(0.53874901568)*x[0]**o+(0.381652052125)*x[0]+(-0.569006390059)*x[1]**o+(0.803939044804)*x[1]
        s[2,4,0,1]=(-0.161281051202)*x[0]**o+(-0.29794326221)*x[0]+(-0.975738807839)*x[1]**o+(-0.365943217167)*x[1]
        s[2,4,1,0]=(0.937384486165)*x[0]**o+(-0.111220331573)*x[0]+(-0.705253089868)*x[1]**o+(-0.0964593740972)*x[1]
        s[2,4,1,1]=(-0.801537343551)*x[0]**o+(-0.0721007760916)*x[0]+(-0.858896629841)*x[1]**o+(0.893882022578)*x[1]
        s[2,4,2,0]=(-0.386195011979)*x[0]**o+(-0.7171939977)*x[0]+(-0.291590089274)*x[1]**o+(0.653160575582)*x[1]
        s[2,4,2,1]=(0.15418789046)*x[0]**o+(-0.337070580033)*x[0]+(-0.826397574623)*x[1]**o+(-0.492621231443)*x[1]
        s[3,0,0,0]=(0.208187397837)*x[0]**o+(-0.941176621475)*x[0]+(-0.553746758603)*x[1]**o+(0.298235037771)*x[1]
        s[3,0,0,1]=(-0.608160638828)*x[0]**o+(0.969633806238)*x[0]+(0.594884783145)*x[1]**o+(0.256426888478)*x[1]
        s[3,0,1,0]=(0.156268978359)*x[0]**o+(-0.618114287912)*x[0]+(-0.736845954486)*x[1]**o+(0.578007765377)*x[1]
        s[3,0,1,1]=(0.0779253738219)*x[0]**o+(0.487197266572)*x[0]+(0.712697073421)*x[1]**o+(-0.333683256167)*x[1]
        s[3,0,2,0]=(-0.432372166788)*x[0]**o+(-0.670271358766)*x[0]+(-0.187482013069)*x[1]**o+(0.177483210094)*x[1]
        s[3,0,2,1]=(0.893891903718)*x[0]**o+(0.0668637308586)*x[0]+(-0.434296768839)*x[1]**o+(-0.297584773434)*x[1]
        s[3,1,0,0]=(-0.226039889634)*x[0]**o+(0.0244084275305)*x[0]+(0.0904503965701)*x[1]**o+(0.826411056795)*x[1]
        s[3,1,0,1]=(0.605633212937)*x[0]**o+(0.69508670693)*x[0]+(0.317716807233)*x[1]**o+(-0.71784966885)*x[1]
        s[3,1,1,0]=(0.697916267298)*x[0]**o+(-0.766126387311)*x[0]+(0.679356018736)*x[1]**o+(0.814149484578)*x[1]
        s[3,1,1,1]=(0.495275603887)*x[0]**o+(0.92183314391)*x[0]+(0.0319182837637)*x[1]**o+(0.550197566786)*x[1]
        s[3,1,2,0]=(-0.549402940272)*x[0]**o+(0.16459055678)*x[0]+(0.6282270637)*x[1]**o+(-0.19466468363)*x[1]
        s[3,1,2,1]=(0.722507137464)*x[0]**o+(0.0201257259776)*x[0]+(0.864760483197)*x[1]**o+(-0.70699130253)*x[1]
        s[3,2,0,0]=(-0.911053389534)*x[0]**o+(0.242037718776)*x[0]+(0.296048767645)*x[1]**o+(-0.786412484197)*x[1]
        s[3,2,0,1]=(-0.278556298185)*x[0]**o+(-0.330948724575)*x[0]+(-0.808707884082)*x[1]**o+(0.649152363362)*x[1]
        s[3,2,1,0]=(0.527331068218)*x[0]**o+(0.535793375918)*x[0]+(0.0173164046642)*x[1]**o+(0.673124828911)*x[1]
        s[3,2,1,1]=(-0.122860429792)*x[0]**o+(-0.33128580821)*x[0]+(-0.875323756482)*x[1]**o+(-0.404630192101)*x[1]
        s[3,2,2,0]=(0.396517792786)*x[0]**o+(-0.992046803715)*x[0]+(0.994743180916)*x[1]**o+(-0.355368679756)*x[1]
        s[3,2,2,1]=(0.988551628034)*x[0]**o+(-0.248207320263)*x[0]+(-0.536981937311)*x[1]**o+(-0.832557705835)*x[1]
        s[3,3,0,0]=(-0.687076681329)*x[0]**o+(0.301843762611)*x[0]+(0.0261628413994)*x[1]**o+(0.257680047964)*x[1]
        s[3,3,0,1]=(-0.0679744087296)*x[0]**o+(-0.388391714998)*x[0]+(-0.305536788937)*x[1]**o+(0.813610001342)*x[1]
        s[3,3,1,0]=(0.520286217752)*x[0]**o+(0.787102883932)*x[0]+(-0.443820422756)*x[1]**o+(0.348550770428)*x[1]
        s[3,3,1,1]=(-0.403687854392)*x[0]**o+(0.224198640317)*x[0]+(0.908268189821)*x[1]**o+(-0.0189418862158)*x[1]
        s[3,3,2,0]=(0.106397622508)*x[0]**o+(0.408482462665)*x[0]+(-0.189398394495)*x[1]**o+(0.892444309304)*x[1]
        s[3,3,2,1]=(-0.158875590567)*x[0]**o+(0.659863215048)*x[0]+(-0.116276105024)*x[1]**o+(-0.435483985612)*x[1]
        s[3,4,0,0]=(-0.00526118581715)*x[0]**o+(0.390573315765)*x[0]+(-0.865881629704)*x[1]**o+(0.188788198915)*x[1]
        s[3,4,0,1]=(0.457024238224)*x[0]**o+(-0.142788838274)*x[0]+(0.514831683476)*x[1]**o+(0.645719634512)*x[1]
        s[3,4,1,0]=(-0.221092754553)*x[0]**o+(0.42876217463)*x[0]+(-0.0146700209904)*x[1]**o+(-0.558199901549)*x[1]
        s[3,4,1,1]=(-0.0854320790552)*x[0]**o+(0.495872652018)*x[0]+(-0.307890759071)*x[1]**o+(-0.80450589936)*x[1]
        s[3,4,2,0]=(0.390350106853)*x[0]**o+(-0.469202978344)*x[0]+(0.21377496022)*x[1]**o+(-0.83671140107)*x[1]
        s[3,4,2,1]=(-0.248241865193)*x[0]**o+(-0.0543747795217)*x[0]+(-0.946132717359)*x[1]**o+(0.72654761542)*x[1]
        ref[0,0,0,0]=(0.666238426788)/(o+1.)+(0.0429533248881)+(0.245364066984)*0.5**o
        ref[0,0,0,1]=(0.800426762254)/(o+1.)+(-0.499676814468)+(0.716203034408)*0.5**o
        ref[0,0,1,0]=(-0.347405577261)/(o+1.)+(0.0177903335042)+(0.122659986993)*0.5**o
        ref[0,0,1,1]=(0.330494636783)/(o+1.)+(0.962663711849)+(0.791965912204)*0.5**o
        ref[0,0,2,0]=(-0.814581462689)/(o+1.)+(0.324851071464)+(0.761523262625)*0.5**o
        ref[0,0,2,1]=(0.349412253363)/(o+1.)+(-0.705620331574)+(0.384651054435)*0.5**o
        ref[0,1,0,0]=(-0.770917205125)/(o+1.)+(-0.102435216416)+(0.114523224402)*0.5**o
        ref[0,1,0,1]=(0.933694211325)/(o+1.)+(0.312641157521)+(0.697263286229)*0.5**o
        ref[0,1,1,0]=(0.358477857654)/(o+1.)+(0.186511815176)+(-0.215924750352)*0.5**o
        ref[0,1,1,1]=(0.839137774823)/(o+1.)+(0.199614896454)+(0.642382357196)*0.5**o
        ref[0,1,2,0]=(0.846765712381)/(o+1.)+(-0.700268495767)+(-0.260657110217)*0.5**o
        ref[0,1,2,1]=(-0.690232848808)/(o+1.)+(0.311449714443)+(0.733925405267)*0.5**o
        ref[0,2,0,0]=(0.0114100757165)/(o+1.)+(0.350846365151)+(0.230747459963)*0.5**o
        ref[0,2,0,1]=(-0.403823163425)/(o+1.)+(0.245080316686)+(0.191593969982)*0.5**o
        ref[0,2,1,0]=(-0.575828297224)/(o+1.)+(-0.148493158816)+(-0.954865298124)*0.5**o
        ref[0,2,1,1]=(0.950680046864)/(o+1.)+(0.505885258451)+(-0.304205093844)*0.5**o
        ref[0,2,2,0]=(-0.172613322727)/(o+1.)+(-0.100878883458)+(-0.46074924128)*0.5**o
        ref[0,2,2,1]=(0.0936397977601)/(o+1.)+(0.0299599581971)+(0.119499701836)*0.5**o
        ref[0,3,0,0]=(0.0553816248689)/(o+1.)+(-0.101311469882)+(-0.525741080321)*0.5**o
        ref[0,3,0,1]=(-0.862336855278)/(o+1.)+(0.839431260552)+(-0.241502398998)*0.5**o
        ref[0,3,1,0]=(-0.459505697865)/(o+1.)+(0.0579551094043)+(0.925044786414)*0.5**o
        ref[0,3,1,1]=(0.880717796832)/(o+1.)+(0.436101093173)+(0.571196115307)*0.5**o
        ref[0,3,2,0]=(-0.538866339841)/(o+1.)+(-0.461890031274)+(-0.878805794321)*0.5**o
        ref[0,3,2,1]=(-0.543638502129)/(o+1.)+(0.228727841415)+(0.968894119383)*0.5**o
        ref[0,4,0,0]=(-0.732685132335)/(o+1.)+(0.566452777361)+(-0.473571740079)*0.5**o
        ref[0,4,0,1]=(-0.243173822645)/(o+1.)+(0.167872939162)+(0.107676577578)*0.5**o
        ref[0,4,1,0]=(-0.482823172795)/(o+1.)+(-0.572147280821)+(-0.912740962221)*0.5**o
        ref[0,4,1,1]=(0.605357795695)/(o+1.)+(-0.382146291669)+(-0.106203879274)*0.5**o
        ref[0,4,2,0]=(0.64759005987)/(o+1.)+(0.0567027036512)+(0.587529237973)*0.5**o
        ref[0,4,2,1]=(0.898533268559)/(o+1.)+(0.268413754579)+(0.482928938234)*0.5**o
        ref[1,0,0,0]=(0.388920605463)/(o+1.)+(-0.795321120361)+(-0.574069031279)*0.5**o
        ref[1,0,0,1]=(-0.279256974829)/(o+1.)+(-0.464781780631)+(0.939040505856)*0.5**o
        ref[1,0,1,0]=(-0.220060979436)/(o+1.)+(-0.236667618055)+(-0.944753068273)*0.5**o
        ref[1,0,1,1]=(-0.421100034358)/(o+1.)+(0.329860890342)+(-0.390329725129)*0.5**o
        ref[1,0,2,0]=(-0.316428480376)/(o+1.)+(0.22458648614)+(-0.7832351829)*0.5**o
        ref[1,0,2,1]=(-0.46577339251)/(o+1.)+(0.434613233487)+(0.414384140175)*0.5**o
        ref[1,1,0,0]=(0.151601513312)/(o+1.)+(-0.858950900661)+(-0.571703274446)*0.5**o
        ref[1,1,0,1]=(0.174748731929)/(o+1.)+(0.0420195194137)+(0.883610930016)*0.5**o
        ref[1,1,1,0]=(-0.454627455637)/(o+1.)+(0.263261148702)+(-0.965747586125)*0.5**o
        ref[1,1,1,1]=(0.378393901391)/(o+1.)+(-0.373089153865)+(-0.778723024919)*0.5**o
        ref[1,1,2,0]=(-0.993366900714)/(o+1.)+(-0.0159437785867)+(0.785822593379)*0.5**o
        ref[1,1,2,1]=(0.422750071837)/(o+1.)+(-0.18893869784)+(0.238887912688)*0.5**o
        ref[1,2,0,0]=(0.288188272131)/(o+1.)+(0.347932642501)+(0.306452161355)*0.5**o
        ref[1,2,0,1]=(-0.557799747963)/(o+1.)+(-0.339325435914)+(0.535104365658)*0.5**o
        ref[1,2,1,0]=(-0.126034299636)/(o+1.)+(-0.841361114631)+(0.653062501973)*0.5**o
        ref[1,2,1,1]=(0.532447219707)/(o+1.)+(0.377955981261)+(0.193513872604)*0.5**o
        ref[1,2,2,0]=(0.508907147346)/(o+1.)+(-0.409163381393)+(0.471722480056)*0.5**o
        ref[1,2,2,1]=(-0.793600779578)/(o+1.)+(0.235547822704)+(0.145178046285)*0.5**o
        ref[1,3,0,0]=(-0.349615695314)/(o+1.)+(-0.289247659766)+(0.883901734656)*0.5**o
        ref[1,3,0,1]=(-0.892859579327)/(o+1.)+(0.177783608711)+(-0.439872366735)*0.5**o
        ref[1,3,1,0]=(0.432470372195)/(o+1.)+(0.364219989705)+(-0.310902411804)*0.5**o
        ref[1,3,1,1]=(-0.0215549839659)/(o+1.)+(-0.828425383886)+(-0.457038984067)*0.5**o
        ref[1,3,2,0]=(-0.0248867951466)/(o+1.)+(-0.153814489037)+(-0.307363570855)*0.5**o
        ref[1,3,2,1]=(0.917168913893)/(o+1.)+(-0.162397790479)+(0.303121965084)*0.5**o
        ref[1,4,0,0]=(0.859962057023)/(o+1.)+(0.502325973761)+(-0.713672877707)*0.5**o
        ref[1,4,0,1]=(0.376600276672)/(o+1.)+(0.572762910965)+(-0.708070236639)*0.5**o
        ref[1,4,1,0]=(-0.271980968262)/(o+1.)+(0.837313638895)+(0.203382859429)*0.5**o
        ref[1,4,1,1]=(-0.876972535039)/(o+1.)+(0.156726080267)+(-0.610374454703)*0.5**o
        ref[1,4,2,0]=(0.512449401496)/(o+1.)+(0.245844097911)+(-0.131008373859)*0.5**o
        ref[1,4,2,1]=(-0.731489669807)/(o+1.)+(-0.28204526748)+(0.994369815068)*0.5**o
        ref[2,0,0,0]=(-0.092041610109)/(o+1.)+(-0.365606435065)+(0.687813573864)*0.5**o
        ref[2,0,0,1]=(0.284484113765)/(o+1.)+(-0.637539030228)+(-0.889705548572)*0.5**o
        ref[2,0,1,0]=(-0.343166268962)/(o+1.)+(0.145389853595)+(-0.0691392356773)*0.5**o
        ref[2,0,1,1]=(0.00928458441193)/(o+1.)+(0.464569495537)+(-0.767504390408)*0.5**o
        ref[2,0,2,0]=(0.772262051249)/(o+1.)+(0.482202390429)+(0.151265878662)*0.5**o
        ref[2,0,2,1]=(0.812739684548)/(o+1.)+(0.739364750489)+(0.638051378377)*0.5**o
        ref[2,1,0,0]=(0.753464652023)/(o+1.)+(-0.486656945678)+(0.699425983168)*0.5**o
        ref[2,1,0,1]=(-0.028962291177)/(o+1.)+(0.0647608313931)+(-0.228981326624)*0.5**o
        ref[2,1,1,0]=(0.449876871146)/(o+1.)+(0.890679095638)+(0.436232377097)*0.5**o
        ref[2,1,1,1]=(0.857688044529)/(o+1.)+(0.190601914609)+(0.108152697095)*0.5**o
        ref[2,1,2,0]=(0.515979568236)/(o+1.)+(-0.0854033087054)+(0.776899779272)*0.5**o
        ref[2,1,2,1]=(0.320690937285)/(o+1.)+(0.290782280265)+(0.486236495643)*0.5**o
        ref[2,2,0,0]=(-0.951630400835)/(o+1.)+(-0.906458887668)+(-0.2834031779)*0.5**o
        ref[2,2,0,1]=(0.331676331677)/(o+1.)+(0.585518380471)+(-0.246680973215)*0.5**o
        ref[2,2,1,0]=(0.998873537117)/(o+1.)+(-0.407407670715)+(0.682460024363)*0.5**o
        ref[2,2,1,1]=(0.231689506568)/(o+1.)+(0.536699678572)+(0.371007993444)*0.5**o
        ref[2,2,2,0]=(-0.255255520802)/(o+1.)+(-0.611019274547)+(0.273036358499)*0.5**o
        ref[2,2,2,1]=(0.0422078078521)/(o+1.)+(-0.54754270871)+(-0.829888508727)*0.5**o
        ref[2,3,0,0]=(0.450664578307)/(o+1.)+(0.531221706214)+(-0.901686992463)*0.5**o
        ref[2,3,0,1]=(-0.411575583885)/(o+1.)+(-0.81707481931)+(-0.146310214982)*0.5**o
        ref[2,3,1,0]=(0.457329256522)/(o+1.)+(-0.573352928694)+(0.796379240086)*0.5**o
        ref[2,3,1,1]=(-0.521975026805)/(o+1.)+(-0.0115768602733)+(0.00162132693744)*0.5**o
        ref[2,3,2,0]=(-0.958220202856)/(o+1.)+(-0.905417180523)+(0.85913616245)*0.5**o
        ref[2,3,2,1]=(-0.0245518988658)/(o+1.)+(0.149894260862)+(-0.394075398541)*0.5**o
        ref[2,4,0,0]=(-0.569006390059)/(o+1.)+(0.592795548465)+(0.53874901568)*0.5**o
        ref[2,4,0,1]=(-0.975738807839)/(o+1.)+(-0.331943239689)+(-0.161281051202)*0.5**o
        ref[2,4,1,0]=(-0.705253089868)/(o+1.)+(-0.103839852835)+(0.937384486165)*0.5**o
        ref[2,4,1,1]=(-0.858896629841)/(o+1.)+(0.410890623243)+(-0.801537343551)*0.5**o
        ref[2,4,2,0]=(-0.291590089274)/(o+1.)+(-0.032016711059)+(-0.386195011979)*0.5**o
        ref[2,4,2,1]=(-0.826397574623)/(o+1.)+(-0.414845905738)+(0.15418789046)*0.5**o
        ref[3,0,0,0]=(-0.553746758603)/(o+1.)+(-0.321470791852)+(0.208187397837)*0.5**o
        ref[3,0,0,1]=(0.594884783145)/(o+1.)+(0.613030347358)+(-0.608160638828)*0.5**o
        ref[3,0,1,0]=(-0.736845954486)/(o+1.)+(-0.0200532612677)+(0.156268978359)*0.5**o
        ref[3,0,1,1]=(0.712697073421)/(o+1.)+(0.0767570052027)+(0.0779253738219)*0.5**o
        ref[3,0,2,0]=(-0.187482013069)/(o+1.)+(-0.246394074336)+(-0.432372166788)*0.5**o
        ref[3,0,2,1]=(-0.434296768839)/(o+1.)+(-0.115360521288)+(0.893891903718)*0.5**o
        ref[3,1,0,0]=(0.0904503965701)/(o+1.)+(0.425409742163)+(-0.226039889634)*0.5**o
        ref[3,1,0,1]=(0.317716807233)/(o+1.)+(-0.0113814809604)+(0.605633212937)*0.5**o
        ref[3,1,1,0]=(0.679356018736)/(o+1.)+(0.0240115486338)+(0.697916267298)*0.5**o
        ref[3,1,1,1]=(0.0319182837637)/(o+1.)+(0.736015355348)+(0.495275603887)*0.5**o
        ref[3,1,2,0]=(0.6282270637)/(o+1.)+(-0.015037063425)+(-0.549402940272)*0.5**o
        ref[3,1,2,1]=(0.864760483197)/(o+1.)+(-0.343432788276)+(0.722507137464)*0.5**o
        ref[3,2,0,0]=(0.296048767645)/(o+1.)+(-0.272187382711)+(-0.911053389534)*0.5**o
        ref[3,2,0,1]=(-0.808707884082)/(o+1.)+(0.159101819394)+(-0.278556298185)*0.5**o
        ref[3,2,1,0]=(0.0173164046642)/(o+1.)+(0.604459102415)+(0.527331068218)*0.5**o
        ref[3,2,1,1]=(-0.875323756482)/(o+1.)+(-0.367958000155)+(-0.122860429792)*0.5**o
        ref[3,2,2,0]=(0.994743180916)/(o+1.)+(-0.673707741736)+(0.396517792786)*0.5**o
        ref[3,2,2,1]=(-0.536981937311)/(o+1.)+(-0.540382513049)+(0.988551628034)*0.5**o
        ref[3,3,0,0]=(0.0261628413994)/(o+1.)+(0.279761905288)+(-0.687076681329)*0.5**o
        ref[3,3,0,1]=(-0.305536788937)/(o+1.)+(0.212609143172)+(-0.0679744087296)*0.5**o
        ref[3,3,1,0]=(-0.443820422756)/(o+1.)+(0.56782682718)+(0.520286217752)*0.5**o
        ref[3,3,1,1]=(0.908268189821)/(o+1.)+(0.10262837705)+(-0.403687854392)*0.5**o
        ref[3,3,2,0]=(-0.189398394495)/(o+1.)+(0.650463385984)+(0.106397622508)*0.5**o
        ref[3,3,2,1]=(-0.116276105024)/(o+1.)+(0.112189614718)+(-0.158875590567)*0.5**o
        ref[3,4,0,0]=(-0.865881629704)/(o+1.)+(0.28968075734)+(-0.00526118581715)*0.5**o
        ref[3,4,0,1]=(0.514831683476)/(o+1.)+(0.251465398119)+(0.457024238224)*0.5**o
        ref[3,4,1,0]=(-0.0146700209904)/(o+1.)+(-0.0647188634597)+(-0.221092754553)*0.5**o
        ref[3,4,1,1]=(-0.307890759071)/(o+1.)+(-0.154316623671)+(-0.0854320790552)*0.5**o
        ref[3,4,2,0]=(0.21377496022)/(o+1.)+(-0.652957189707)+(0.390350106853)*0.5**o
        ref[3,4,2,1]=(-0.946132717359)/(o+1.)+(0.336086417949)+(-0.248241865193)*0.5**o
      else:
        s[0,0,0,0]=(-0.210183765573)*x[0]**o+(0.259610104977)*x[0]+(0.840819899605)*x[1]**o+(-0.103339866116)*x[1]+(0.737406932578)*x[2]**o+(0.712156905811)*x[2]
        s[0,0,0,1]=(-0.323968020416)*x[0]**o+(-0.876969115567)*x[0]+(-0.906271643259)*x[1]**o+(-0.366737699368)*x[1]+(0.367538667762)*x[2]**o+(0.80273967237)*x[2]
        s[0,0,1,0]=(0.467518636672)*x[0]**o+(0.684316475842)*x[0]+(-0.948313733183)*x[1]**o+(-0.768576249031)*x[1]+(-0.169935408354)*x[2]**o+(-0.747881358075)*x[2]
        s[0,0,1,1]=(0.976876531867)*x[0]**o+(0.561100376553)*x[0]+(0.450819300656)*x[1]**o+(-0.0490235546318)*x[1]+(-0.698776567424)*x[2]**o+(-0.01350381399)*x[2]
        s[0,0,2,0]=(-0.552336862133)*x[0]**o+(-0.564574791577)*x[0]+(0.95546748665)*x[1]**o+(-0.211505026053)*x[1]+(-0.224770603639)*x[2]**o+(0.498420086189)*x[2]
        s[0,0,2,1]=(0.110734087997)*x[0]**o+(0.256257276464)*x[0]+(-0.682963863817)*x[1]**o+(-0.832585277356)*x[1]+(-0.0216621586129)*x[2]**o+(0.171233268331)*x[2]
        s[0,1,0,0]=(0.979769828626)*x[0]**o+(-0.918534054584)*x[0]+(-0.770334932534)*x[1]**o+(-0.266282551716)*x[1]+(-0.528515244986)*x[2]**o+(0.452232798522)*x[2]
        s[0,1,0,1]=(0.184115163182)*x[0]**o+(0.0372768333589)*x[0]+(0.0697681712771)*x[1]**o+(0.650340145402)*x[1]+(0.89670654092)*x[2]**o+(0.0666608228764)*x[2]
        s[0,1,1,0]=(0.0822751335539)*x[0]**o+(-0.145731529462)*x[0]+(-0.804584197287)*x[1]**o+(-0.367596869981)*x[1]+(-0.819068186934)*x[2]**o+(-0.761154318012)*x[2]
        s[0,1,1,1]=(0.672877076993)*x[0]**o+(-0.060901199793)*x[0]+(-0.334248974895)*x[1]**o+(-0.872695915274)*x[1]+(0.246713393819)*x[2]**o+(-0.819057815971)*x[2]
        s[0,1,2,0]=(0.968772211014)*x[0]**o+(0.71842819735)*x[0]+(-0.0772702390821)*x[1]**o+(-0.338266585527)*x[1]+(-0.506850093136)*x[2]**o+(0.707040138867)*x[2]
        s[0,1,2,1]=(-0.804147712129)*x[0]**o+(0.48962140515)*x[0]+(-0.247303878423)*x[1]**o+(0.889311125721)*x[1]+(0.457530011837)*x[2]**o+(-0.721486338348)*x[2]
        s[0,2,0,0]=(0.140474174736)*x[0]**o+(-0.0453977330632)*x[0]+(0.9083497372)*x[1]**o+(-0.719356277596)*x[1]+(0.72748037113)*x[2]**o+(-0.0518197409882)*x[2]
        s[0,2,0,1]=(-0.0758350230998)*x[0]**o+(-0.367910921003)*x[0]+(-0.118137700915)*x[1]**o+(0.620139843753)*x[1]+(0.648111986033)*x[2]**o+(-0.250478603635)*x[2]
        s[0,2,1,0]=(0.128776260898)*x[0]**o+(0.772839138201)*x[0]+(0.667219128346)*x[1]**o+(-0.291604907274)*x[1]+(0.976574302911)*x[2]**o+(-0.60715514616)*x[2]
        s[0,2,1,1]=(-0.85960053799)*x[0]**o+(0.421741754709)*x[0]+(0.137156504307)*x[1]**o+(0.31044942099)*x[1]+(-0.0607312333184)*x[2]**o+(-0.371068928897)*x[2]
        s[0,2,2,0]=(0.703498831008)*x[0]**o+(0.351119263938)*x[0]+(0.0238555242548)*x[1]**o+(0.709263994991)*x[1]+(-0.822927451312)*x[2]**o+(0.289556375179)*x[2]
        s[0,2,2,1]=(-0.285309117139)*x[0]**o+(0.13890400764)*x[0]+(-0.604129730872)*x[1]**o+(-0.0821311416855)*x[1]+(-0.731018609109)*x[2]**o+(0.529525274631)*x[2]
        s[0,3,0,0]=(0.641292392051)*x[0]**o+(0.41865118466)*x[0]+(0.963763232952)*x[1]**o+(0.290032616399)*x[1]+(-0.506676751351)*x[2]**o+(-0.895169772311)*x[2]
        s[0,3,0,1]=(-0.616101594244)*x[0]**o+(0.791759779662)*x[0]+(0.690427410929)*x[1]**o+(0.970117852706)*x[1]+(-0.813314826297)*x[2]**o+(0.637157627363)*x[2]
        s[0,3,1,0]=(0.973134693253)*x[0]**o+(-0.294894425912)*x[0]+(-0.342056908291)*x[1]**o+(-0.569749335695)*x[1]+(0.402640595819)*x[2]**o+(0.132057879818)*x[2]
        s[0,3,1,1]=(-0.785989986755)*x[0]**o+(-0.319383158643)*x[0]+(0.875111156622)*x[1]**o+(0.607783276248)*x[1]+(0.522924160022)*x[2]**o+(-0.963598048032)*x[2]
        s[0,3,2,0]=(0.367812735951)*x[0]**o+(-0.687977977398)*x[0]+(-0.787244484503)*x[1]**o+(0.240728636869)*x[1]+(-0.974034052698)*x[2]**o+(-0.810505857708)*x[2]
        s[0,3,2,1]=(-0.661827774609)*x[0]**o+(0.569720489249)*x[0]+(-0.0944794378735)*x[1]**o+(-0.793223846926)*x[1]+(0.54128448366)*x[2]**o+(-0.893409198606)*x[2]
        s[0,4,0,0]=(0.259245470871)*x[0]**o+(-0.226239257386)*x[0]+(-0.308196311787)*x[1]**o+(-0.23628967322)*x[1]+(0.547994188289)*x[2]**o+(-0.671911794405)*x[2]
        s[0,4,0,1]=(-0.00101716691769)*x[0]**o+(-0.509353224753)*x[0]+(0.592369934948)*x[1]**o+(-0.632019867229)*x[1]+(-0.890240436074)*x[2]**o+(0.895675476663)*x[2]
        s[0,4,1,0]=(0.960337962272)*x[0]**o+(0.555794125077)*x[0]+(0.993174576462)*x[1]**o+(-0.772336499274)*x[1]+(0.0103297090652)*x[2]**o+(0.118606584111)*x[2]
        s[0,4,1,1]=(0.0420511106146)*x[0]**o+(-0.758220545738)*x[0]+(-0.119140693961)*x[1]**o+(0.0775988947583)*x[1]+(0.828731522222)*x[2]**o+(-0.762715251326)*x[2]
        s[0,4,2,0]=(0.549182098853)*x[0]**o+(0.36156976435)*x[0]+(-0.152823474029)*x[1]**o+(-0.315519239615)*x[1]+(0.577553409367)*x[2]**o+(0.467722402358)*x[2]
        s[0,4,2,1]=(0.117551959782)*x[0]**o+(0.850326589959)*x[0]+(0.881226923201)*x[1]**o+(-0.659482296116)*x[1]+(0.618324879901)*x[2]**o+(0.720404333688)*x[2]
        s[1,0,0,0]=(-0.761147885924)*x[0]**o+(0.802435468296)*x[0]+(0.794658337218)*x[1]**o+(0.0458609096746)*x[1]+(0.603568994654)*x[2]**o+(-0.543591666298)*x[2]
        s[1,0,0,1]=(-0.568265516859)*x[0]**o+(-0.217464502958)*x[0]+(0.945091213325)*x[1]**o+(0.701546510287)*x[1]+(0.218015344978)*x[2]**o+(-0.296437711896)*x[2]
        s[1,0,1,0]=(0.131167263333)*x[0]**o+(0.443503731963)*x[0]+(0.936129884208)*x[1]**o+(0.731105012686)*x[1]+(0.886062834899)*x[2]**o+(0.949880973913)*x[2]
        s[1,0,1,1]=(-0.324713526984)*x[0]**o+(0.525830634904)*x[0]+(0.258554314238)*x[1]**o+(-0.206218826462)*x[1]+(0.945107982827)*x[2]**o+(0.301738246936)*x[2]
        s[1,0,2,0]=(-0.0983483498843)*x[0]**o+(-0.107666500098)*x[0]+(0.572563468846)*x[1]**o+(-0.877027769074)*x[1]+(0.0499191410048)*x[2]**o+(0.78320557846)*x[2]
        s[1,0,2,1]=(-0.0549251955703)*x[0]**o+(-0.684098254591)*x[0]+(0.974543898137)*x[1]**o+(0.572617020679)*x[1]+(-0.745704992018)*x[2]**o+(0.694741262801)*x[2]
        s[1,1,0,0]=(0.140734267352)*x[0]**o+(0.684555621818)*x[0]+(-0.988693128074)*x[1]**o+(0.663988019213)*x[1]+(0.207716488749)*x[2]**o+(-0.218440531714)*x[2]
        s[1,1,0,1]=(0.906718149076)*x[0]**o+(-0.823295142671)*x[0]+(0.599921015401)*x[1]**o+(0.712671366397)*x[1]+(0.560173956846)*x[2]**o+(-0.583700172601)*x[2]
        s[1,1,1,0]=(0.412161237427)*x[0]**o+(0.449013323323)*x[0]+(-0.422313884483)*x[1]**o+(-0.856352846316)*x[1]+(-0.131018950683)*x[2]**o+(-0.113127977867)*x[2]
        s[1,1,1,1]=(0.342974599199)*x[0]**o+(0.918991859784)*x[0]+(-0.380736598315)*x[1]**o+(-0.844001076048)*x[1]+(0.103584145435)*x[2]**o+(-0.0224438802577)*x[2]
        s[1,1,2,0]=(0.405298238411)*x[0]**o+(-0.0260196814125)*x[0]+(0.192953354082)*x[1]**o+(-0.768884750367)*x[1]+(-0.898989048123)*x[2]**o+(-0.625718514351)*x[2]
        s[1,1,2,1]=(-0.679709121137)*x[0]**o+(0.829310202601)*x[0]+(0.488556797655)*x[1]**o+(-0.546577821454)*x[1]+(0.215139403299)*x[2]**o+(-0.597837941999)*x[2]
        s[1,2,0,0]=(-0.763765181249)*x[0]**o+(0.471498461874)*x[0]+(-0.0657374404825)*x[1]**o+(-0.990465192866)*x[1]+(0.755407325384)*x[2]**o+(-0.725455335361)*x[2]
        s[1,2,0,1]=(-0.707293534802)*x[0]**o+(0.741579146085)*x[0]+(-0.827571233981)*x[1]**o+(-0.924507469851)*x[1]+(-0.11981466602)*x[2]**o+(-0.227349968773)*x[2]
        s[1,2,1,0]=(0.02761949765)*x[0]**o+(0.23242129234)*x[0]+(-0.744337789362)*x[1]**o+(-0.69986289257)*x[1]+(0.284991295889)*x[2]**o+(-0.82971704627)*x[2]
        s[1,2,1,1]=(-0.399349607364)*x[0]**o+(0.681805497713)*x[0]+(0.50892365528)*x[1]**o+(-0.24206992271)*x[1]+(-0.864035697056)*x[2]**o+(0.311678612996)*x[2]
        s[1,2,2,0]=(0.900552977578)*x[0]**o+(0.352782843752)*x[0]+(0.343335797615)*x[1]**o+(0.12459367011)*x[1]+(-0.78722966105)*x[2]**o+(0.0289887373712)*x[2]
        s[1,2,2,1]=(-0.231556056016)*x[0]**o+(-0.454277914592)*x[0]+(-0.723413876928)*x[1]**o+(-0.167712035756)*x[1]+(0.297646282013)*x[2]**o+(0.142123612721)*x[2]
        s[1,3,0,0]=(-0.753062984495)*x[0]**o+(0.834144701924)*x[0]+(0.198694009378)*x[1]**o+(-0.522412387399)*x[1]+(-0.823293930133)*x[2]**o+(-0.875699299314)*x[2]
        s[1,3,0,1]=(0.810523164741)*x[0]**o+(-0.363171422715)*x[0]+(-0.0125819806617)*x[1]**o+(-0.461550939044)*x[1]+(-0.0610628843793)*x[2]**o+(0.792146078844)*x[2]
        s[1,3,1,0]=(-0.435427701923)*x[0]**o+(0.0485485027059)*x[0]+(-0.533976468056)*x[1]**o+(0.907837942847)*x[1]+(-0.70993588953)*x[2]**o+(0.707193663015)*x[2]
        s[1,3,1,1]=(-0.647252848948)*x[0]**o+(0.889142781795)*x[0]+(-0.625573458304)*x[1]**o+(0.40306299061)*x[1]+(0.0918220173097)*x[2]**o+(0.563306928169)*x[2]
        s[1,3,2,0]=(-0.607331889118)*x[0]**o+(0.812730322666)*x[0]+(0.555953335767)*x[1]**o+(0.661784189166)*x[1]+(0.391287509324)*x[2]**o+(-0.916911048454)*x[2]
        s[1,3,2,1]=(-0.484392978469)*x[0]**o+(-0.541102106822)*x[0]+(-0.245428871935)*x[1]**o+(0.278664918051)*x[1]+(-0.955672198176)*x[2]**o+(-0.548226453411)*x[2]
        s[1,4,0,0]=(0.503702684537)*x[0]**o+(0.233988182801)*x[0]+(-0.093833060394)*x[1]**o+(-0.167779518637)*x[1]+(-0.224463708091)*x[2]**o+(-0.710844655028)*x[2]
        s[1,4,0,1]=(0.255645379803)*x[0]**o+(-0.319588660935)*x[0]+(-0.156476801364)*x[1]**o+(0.764377089178)*x[1]+(0.766935604368)*x[2]**o+(-0.157460600111)*x[2]
        s[1,4,1,0]=(0.15831529726)*x[0]**o+(-0.922418423102)*x[0]+(0.561598779551)*x[1]**o+(-0.745598205601)*x[1]+(-0.694388851252)*x[2]**o+(-0.247051354544)*x[2]
        s[1,4,1,1]=(-0.547080506797)*x[0]**o+(-0.814888293172)*x[0]+(0.122207354764)*x[1]**o+(-0.489157382765)*x[1]+(-0.165490463352)*x[2]**o+(-0.114446229543)*x[2]
        s[1,4,2,0]=(-0.369250635169)*x[0]**o+(0.423782722049)*x[0]+(0.968680330397)*x[1]**o+(0.28815964147)*x[1]+(-0.143666599619)*x[2]**o+(0.499469160546)*x[2]
        s[1,4,2,1]=(0.843010459592)*x[0]**o+(0.676625633805)*x[0]+(-0.635607829653)*x[1]**o+(-0.760424296046)*x[1]+(0.457005234774)*x[2]**o+(-0.102642224679)*x[2]
        s[2,0,0,0]=(-0.416783290665)*x[0]**o+(-0.0326686788419)*x[0]+(-0.144394865356)*x[1]**o+(-0.911119053945)*x[1]+(-0.55810658213)*x[2]**o+(0.40998976988)*x[2]
        s[2,0,0,1]=(0.513962651738)*x[0]**o+(-0.998965858933)*x[0]+(0.917310302297)*x[1]**o+(-0.92170256086)*x[1]+(-0.832148513501)*x[2]**o+(-0.551050270978)*x[2]
        s[2,0,1,0]=(0.431543077341)*x[0]**o+(0.042920807321)*x[0]+(-0.143174202064)*x[1]**o+(0.90359005289)*x[1]+(-0.732368795811)*x[2]**o+(-0.0689734110721)*x[2]
        s[2,0,1,1]=(-0.167337582492)*x[0]**o+(-0.685847720118)*x[0]+(-0.574599145898)*x[1]**o+(0.685408949158)*x[1]+(0.583748780479)*x[2]**o+(0.494510625069)*x[2]
        s[2,0,2,0]=(0.76364604242)*x[0]**o+(-0.432951084836)*x[0]+(0.425205211657)*x[1]**o+(-0.879037059825)*x[1]+(-0.013713147532)*x[2]**o+(0.569703973439)*x[2]
        s[2,0,2,1]=(-0.656200752037)*x[0]**o+(0.0808097604648)*x[0]+(0.503360202878)*x[1]**o+(0.487461964251)*x[1]+(-0.405721164154)*x[2]**o+(0.126364295441)*x[2]
        s[2,1,0,0]=(-0.168286367707)*x[0]**o+(0.497189264604)*x[0]+(0.309898214306)*x[1]**o+(-0.51880727488)*x[1]+(0.518790399303)*x[2]**o+(0.401150188648)*x[2]
        s[2,1,0,1]=(0.573088756891)*x[0]**o+(-0.528498236574)*x[0]+(-0.866435772203)*x[1]**o+(-0.0338712475717)*x[1]+(0.869338257704)*x[2]**o+(0.276166855245)*x[2]
        s[2,1,1,0]=(-0.200994668199)*x[0]**o+(-0.227638621809)*x[0]+(-0.243755130686)*x[1]**o+(0.983124311993)*x[1]+(0.834307124674)*x[2]**o+(0.191048093109)*x[2]
        s[2,1,1,1]=(0.423677882618)*x[0]**o+(0.451735240848)*x[0]+(0.417686763179)*x[1]**o+(0.766280670471)*x[1]+(-0.829023066103)*x[2]**o+(-0.780956171089)*x[2]
        s[2,1,2,0]=(0.640928997042)*x[0]**o+(0.671670205561)*x[0]+(0.14746453586)*x[1]**o+(-0.744942640395)*x[1]+(0.936412384681)*x[2]**o+(-0.797412536669)*x[2]
        s[2,1,2,1]=(-0.730418336064)*x[0]**o+(-0.912540120589)*x[0]+(0.31782754852)*x[1]**o+(-0.476487014675)*x[1]+(-0.166085203325)*x[2]**o+(-0.592216218131)*x[2]
        s[2,2,0,0]=(0.373430973034)*x[0]**o+(-0.569005417932)*x[0]+(0.33244882787)*x[1]**o+(-0.348297838584)*x[1]+(-0.903746783477)*x[2]**o+(0.976129823909)*x[2]
        s[2,2,0,1]=(-0.384047362737)*x[0]**o+(-0.268433421634)*x[0]+(-0.838287069736)*x[1]**o+(0.537590839064)*x[1]+(-0.871855225879)*x[2]**o+(0.772511342839)*x[2]
        s[2,2,1,0]=(-0.978119189005)*x[0]**o+(-0.121978949728)*x[0]+(-0.329862921919)*x[1]**o+(-0.709859714296)*x[1]+(0.330028081384)*x[2]**o+(-0.0970288297589)*x[2]
        s[2,2,1,1]=(-0.666661297789)*x[0]**o+(-0.480759645285)*x[0]+(-0.344863683771)*x[1]**o+(0.385174072898)*x[1]+(-0.40458447828)*x[2]**o+(0.81018598482)*x[2]
        s[2,2,2,0]=(-0.419553660247)*x[0]**o+(-0.96154803369)*x[0]+(-0.979914830867)*x[1]**o+(0.802474237871)*x[1]+(-0.273952110245)*x[2]**o+(-0.273921739584)*x[2]
        s[2,2,2,1]=(0.204828260085)*x[0]**o+(-0.207525741758)*x[0]+(-0.157511344538)*x[1]**o+(-0.242922426999)*x[1]+(0.750045639114)*x[2]**o+(0.625828940704)*x[2]
        s[2,3,0,0]=(-0.433426692328)*x[0]**o+(0.546111969738)*x[0]+(0.380856694436)*x[1]**o+(-0.33958919138)*x[1]+(-0.612713564248)*x[2]**o+(-0.468136706051)*x[2]
        s[2,3,0,1]=(-0.753353152037)*x[0]**o+(0.898355698117)*x[0]+(-0.748796477533)*x[1]**o+(-0.84014959853)*x[1]+(0.00222130390134)*x[2]**o+(0.892747426493)*x[2]
        s[2,3,1,0]=(-0.770467840127)*x[0]**o+(-0.365528437394)*x[0]+(0.823481948409)*x[1]**o+(0.688841559491)*x[1]+(0.399335743364)*x[2]**o+(-0.11069734561)*x[2]
        s[2,3,1,1]=(-0.490969508319)*x[0]**o+(0.516182950326)*x[0]+(-0.729563968179)*x[1]**o+(-0.738386155156)*x[1]+(0.646183209813)*x[2]**o+(0.319822004437)*x[2]
        s[2,3,2,0]=(-0.719374773896)*x[0]**o+(0.227265715278)*x[0]+(-0.378942759956)*x[1]**o+(-0.0227600257348)*x[1]+(-0.319915586734)*x[2]**o+(0.901101476715)*x[2]
        s[2,3,2,1]=(-0.514878672798)*x[0]**o+(-0.496279665164)*x[0]+(0.301404632426)*x[1]**o+(0.408945490654)*x[1]+(0.993826467678)*x[2]**o+(0.466263681916)*x[2]
        s[2,4,0,0]=(0.163562057356)*x[0]**o+(0.653997800691)*x[0]+(0.0851835561474)*x[1]**o+(0.940843692093)*x[1]+(0.047617990547)*x[2]**o+(-0.859043423533)*x[2]
        s[2,4,0,1]=(0.634549693325)*x[0]**o+(0.914675769662)*x[0]+(-0.172241120744)*x[1]**o+(0.534043363834)*x[1]+(-0.655408958703)*x[2]**o+(0.688624822376)*x[2]
        s[2,4,1,0]=(0.330425495069)*x[0]**o+(0.686177305834)*x[0]+(0.752515146248)*x[1]**o+(-0.552696418525)*x[1]+(-0.955945706288)*x[2]**o+(0.470133484592)*x[2]
        s[2,4,1,1]=(0.106341453568)*x[0]**o+(-0.531668585129)*x[0]+(-0.520032286959)*x[1]**o+(-0.738038080149)*x[1]+(0.525162627468)*x[2]**o+(-0.655104090288)*x[2]
        s[2,4,2,0]=(0.994443804198)*x[0]**o+(0.510472126015)*x[0]+(0.902454289513)*x[1]**o+(-0.191032483827)*x[1]+(-0.871408191409)*x[2]**o+(0.547124971054)*x[2]
        s[2,4,2,1]=(0.113561397179)*x[0]**o+(-0.505522632919)*x[0]+(0.30818367646)*x[1]**o+(-0.00465438758811)*x[1]+(0.594009196981)*x[2]**o+(-0.226939247299)*x[2]
        s[3,0,0,0]=(-0.463635004027)*x[0]**o+(-0.473760265395)*x[0]+(0.878709353532)*x[1]**o+(0.0947459582975)*x[1]+(0.0219625500089)*x[2]**o+(0.784199705647)*x[2]
        s[3,0,0,1]=(0.676123626258)*x[0]**o+(-0.718629965972)*x[0]+(0.955205674618)*x[1]**o+(0.706290464564)*x[1]+(-0.604943464128)*x[2]**o+(0.694623387198)*x[2]
        s[3,0,1,0]=(-0.304361833494)*x[0]**o+(-0.658618417297)*x[0]+(0.171679811142)*x[1]**o+(0.995668711479)*x[1]+(-0.0331315333318)*x[2]**o+(-0.425870682329)*x[2]
        s[3,0,1,1]=(0.307231747905)*x[0]**o+(0.0052926672705)*x[0]+(-0.0603771488434)*x[1]**o+(0.201207904112)*x[1]+(0.769253259019)*x[2]**o+(0.809891168107)*x[2]
        s[3,0,2,0]=(-0.130591836024)*x[0]**o+(-0.0929104424471)*x[0]+(0.702701120328)*x[1]**o+(0.79105646679)*x[1]+(0.279504582566)*x[2]**o+(-0.0685634410467)*x[2]
        s[3,0,2,1]=(-0.576303721224)*x[0]**o+(0.996201825902)*x[0]+(0.415602469244)*x[1]**o+(-0.51130556243)*x[1]+(0.924720232267)*x[2]**o+(-0.653996834451)*x[2]
        s[3,1,0,0]=(-0.798641869619)*x[0]**o+(-0.474090853446)*x[0]+(-0.597109245363)*x[1]**o+(0.441151246969)*x[1]+(-0.460726722134)*x[2]**o+(0.129538461429)*x[2]
        s[3,1,0,1]=(-0.00692013378255)*x[0]**o+(0.12877226476)*x[0]+(-0.357533404018)*x[1]**o+(-0.389134623739)*x[1]+(0.0387875396629)*x[2]**o+(-0.81001355285)*x[2]
        s[3,1,1,0]=(0.975364017255)*x[0]**o+(0.0184628364974)*x[0]+(0.738788422118)*x[1]**o+(-0.897723661125)*x[1]+(0.342855398212)*x[2]**o+(0.766278709935)*x[2]
        s[3,1,1,1]=(0.103423718636)*x[0]**o+(0.264183054502)*x[0]+(-0.413231981251)*x[1]**o+(0.50816929065)*x[1]+(0.717301020458)*x[2]**o+(0.863858366933)*x[2]
        s[3,1,2,0]=(0.0547068997542)*x[0]**o+(-0.0697197526742)*x[0]+(0.560106582056)*x[1]**o+(0.914213149957)*x[1]+(-0.357975004563)*x[2]**o+(0.138675813902)*x[2]
        s[3,1,2,1]=(0.236730250658)*x[0]**o+(0.835033750765)*x[0]+(-0.408412262014)*x[1]**o+(0.707762870875)*x[1]+(-0.381638364848)*x[2]**o+(-0.237912635268)*x[2]
        s[3,2,0,0]=(0.388806525405)*x[0]**o+(0.434371077714)*x[0]+(-0.637939265061)*x[1]**o+(0.869903625445)*x[1]+(0.751102627943)*x[2]**o+(-0.591787739898)*x[2]
        s[3,2,0,1]=(-0.328460151904)*x[0]**o+(-0.469089195258)*x[0]+(0.189752113153)*x[1]**o+(-0.223739735753)*x[1]+(0.90243464478)*x[2]**o+(0.393444417035)*x[2]
        s[3,2,1,0]=(-0.350954406466)*x[0]**o+(0.222756398721)*x[0]+(0.881035791552)*x[1]**o+(-0.601290867644)*x[1]+(-0.845027376811)*x[2]**o+(-0.658344721797)*x[2]
        s[3,2,1,1]=(-0.804545424944)*x[0]**o+(0.723106876602)*x[0]+(-0.710610485987)*x[1]**o+(0.616671752141)*x[1]+(0.255075132475)*x[2]**o+(-0.344194652837)*x[2]
        s[3,2,2,0]=(0.867664525177)*x[0]**o+(-0.584510407392)*x[0]+(-0.50466882786)*x[1]**o+(-0.976134817563)*x[1]+(-0.0741803228365)*x[2]**o+(-0.816773953701)*x[2]
        s[3,2,2,1]=(0.997671019163)*x[0]**o+(-0.0614667721506)*x[0]+(-0.978519383696)*x[1]**o+(0.0645671450044)*x[1]+(0.465596917785)*x[2]**o+(-0.279520773991)*x[2]
        s[3,3,0,0]=(-0.564960743631)*x[0]**o+(0.923212123601)*x[0]+(0.42281995555)*x[1]**o+(-0.701296001463)*x[1]+(0.791187577221)*x[2]**o+(-0.665209151969)*x[2]
        s[3,3,0,1]=(0.872459084593)*x[0]**o+(-0.14974585293)*x[0]+(0.257403669429)*x[1]**o+(-0.236794230318)*x[1]+(-0.458881609061)*x[2]**o+(0.614204776708)*x[2]
        s[3,3,1,0]=(-0.256375661857)*x[0]**o+(-0.0451909988296)*x[0]+(-0.38620807221)*x[1]**o+(0.905631567256)*x[1]+(-0.293636725178)*x[2]**o+(-0.522921339974)*x[2]
        s[3,3,1,1]=(-0.649399488997)*x[0]**o+(-0.92131361011)*x[0]+(0.228306527968)*x[1]**o+(0.914959459994)*x[1]+(-0.689881365519)*x[2]**o+(0.769874183342)*x[2]
        s[3,3,2,0]=(0.237419242359)*x[0]**o+(-0.578361971709)*x[0]+(-0.63121773371)*x[1]**o+(0.446149460264)*x[1]+(-0.975635743387)*x[2]**o+(-0.0926780614368)*x[2]
        s[3,3,2,1]=(0.898298006002)*x[0]**o+(0.738210632094)*x[0]+(0.954104559567)*x[1]**o+(-0.0586956189155)*x[1]+(-0.197544744403)*x[2]**o+(-0.458321606749)*x[2]
        s[3,4,0,0]=(0.152469131812)*x[0]**o+(0.636503984792)*x[0]+(0.634624949432)*x[1]**o+(-0.191145041451)*x[1]+(-0.586391727404)*x[2]**o+(0.470501756725)*x[2]
        s[3,4,0,1]=(-0.902280446763)*x[0]**o+(0.276584347986)*x[0]+(0.627059019624)*x[1]**o+(0.256966014366)*x[1]+(0.993138399883)*x[2]**o+(0.444820746069)*x[2]
        s[3,4,1,0]=(0.226692588742)*x[0]**o+(-0.462068747651)*x[0]+(0.243555291569)*x[1]**o+(-0.913735726033)*x[1]+(0.149578352207)*x[2]**o+(-0.330136710272)*x[2]
        s[3,4,1,1]=(0.99435141349)*x[0]**o+(0.0799718813232)*x[0]+(0.191619055132)*x[1]**o+(0.85076051922)*x[1]+(0.111360148299)*x[2]**o+(-0.683082083045)*x[2]
        s[3,4,2,0]=(-0.574679864968)*x[0]**o+(-0.836650014193)*x[0]+(-0.548132149017)*x[1]**o+(-0.255129120741)*x[1]+(0.639700961749)*x[2]**o+(0.932732223318)*x[2]
        s[3,4,2,1]=(0.670125652473)*x[0]**o+(-0.717633581197)*x[0]+(-0.908717470386)*x[1]**o+(0.420401361995)*x[1]+(-0.93231135796)*x[2]**o+(0.820828865038)*x[2]
        ref[0,0,0,0]=(1.57822683218)/(o+1.)+(0.434213572336)+(-0.210183765573)*0.5**o
        ref[0,0,0,1]=(-0.538732975497)/(o+1.)+(-0.220483571283)+(-0.323968020416)*0.5**o
        ref[0,0,1,0]=(-1.11824914154)/(o+1.)+(-0.416070565632)+(0.467518636672)*0.5**o
        ref[0,0,1,1]=(-0.247957266768)/(o+1.)+(0.249286503966)+(0.976876531867)*0.5**o
        ref[0,0,2,0]=(0.730696883011)/(o+1.)+(-0.13882986572)+(-0.552336862133)*0.5**o
        ref[0,0,2,1]=(-0.70462602243)/(o+1.)+(-0.20254736628)+(0.110734087997)*0.5**o
        ref[0,1,0,0]=(-1.29885017752)/(o+1.)+(-0.366291903889)+(0.979769828626)*0.5**o
        ref[0,1,0,1]=(0.966474712198)/(o+1.)+(0.377138900819)+(0.184115163182)*0.5**o
        ref[0,1,1,0]=(-1.62365238422)/(o+1.)+(-0.637241358728)+(0.0822751335539)*0.5**o
        ref[0,1,1,1]=(-0.0875355810752)/(o+1.)+(-0.876327465519)+(0.672877076993)*0.5**o
        ref[0,1,2,0]=(-0.584120332218)/(o+1.)+(0.543600875345)+(0.968772211014)*0.5**o
        ref[0,1,2,1]=(0.210226133414)/(o+1.)+(0.328723096261)+(-0.804147712129)*0.5**o
        ref[0,2,0,0]=(1.63583010833)/(o+1.)+(-0.408286875823)+(0.140474174736)*0.5**o
        ref[0,2,0,1]=(0.529974285118)/(o+1.)+(0.000875159557622)+(-0.0758350230998)*0.5**o
        ref[0,2,1,0]=(1.64379343126)/(o+1.)+(-0.0629604576161)+(0.128776260898)*0.5**o
        ref[0,2,1,1]=(0.0764252709887)/(o+1.)+(0.180561123401)+(-0.85960053799)*0.5**o
        ref[0,2,2,0]=(-0.799071927058)/(o+1.)+(0.674969817054)+(0.703498831008)*0.5**o
        ref[0,2,2,1]=(-1.33514833998)/(o+1.)+(0.293149070293)+(-0.285309117139)*0.5**o
        ref[0,3,0,0]=(0.457086481601)/(o+1.)+(-0.0932429856258)+(0.641292392051)*0.5**o
        ref[0,3,0,1]=(-0.122887415369)/(o+1.)+(1.19951762987)+(-0.616101594244)*0.5**o
        ref[0,3,1,0]=(0.0605836875282)/(o+1.)+(-0.366292940894)+(0.973134693253)*0.5**o
        ref[0,3,1,1]=(1.39803531664)/(o+1.)+(-0.337598965213)+(-0.785989986755)*0.5**o
        ref[0,3,2,0]=(-1.7612785372)/(o+1.)+(-0.628877599118)+(0.367812735951)*0.5**o
        ref[0,3,2,1]=(0.446805045787)/(o+1.)+(-0.558456278141)+(-0.661827774609)*0.5**o
        ref[0,4,0,0]=(0.239797876502)/(o+1.)+(-0.567220362506)+(0.259245470871)*0.5**o
        ref[0,4,0,1]=(-0.297870501125)/(o+1.)+(-0.12284880766)+(-0.00101716691769)*0.5**o
        ref[0,4,1,0]=(1.00350428553)/(o+1.)+(-0.0489678950431)+(0.960337962272)*0.5**o
        ref[0,4,1,1]=(0.709590828261)/(o+1.)+(-0.721668451153)+(0.0420511106146)*0.5**o
        ref[0,4,2,0]=(0.424729935338)/(o+1.)+(0.256886463547)+(0.549182098853)*0.5**o
        ref[0,4,2,1]=(1.4995518031)/(o+1.)+(0.455624313765)+(0.117551959782)*0.5**o
        ref[1,0,0,0]=(1.39822733187)/(o+1.)+(0.152352355836)+(-0.761147885924)*0.5**o
        ref[1,0,0,1]=(1.1631065583)/(o+1.)+(0.0938221477164)+(-0.568265516859)*0.5**o
        ref[1,0,1,0]=(1.82219271911)/(o+1.)+(1.06224485928)+(0.131167263333)*0.5**o
        ref[1,0,1,1]=(1.20366229707)/(o+1.)+(0.310675027689)+(-0.324713526984)*0.5**o
        ref[1,0,2,0]=(0.622482609851)/(o+1.)+(-0.100744345356)+(-0.0983483498843)*0.5**o
        ref[1,0,2,1]=(0.228838906118)/(o+1.)+(0.291630014445)+(-0.0549251955703)*0.5**o
        ref[1,1,0,0]=(-0.780976639325)/(o+1.)+(0.565051554659)+(0.140734267352)*0.5**o
        ref[1,1,0,1]=(1.16009497225)/(o+1.)+(-0.347161974437)+(0.906718149076)*0.5**o
        ref[1,1,1,0]=(-0.553332835166)/(o+1.)+(-0.26023375043)+(0.412161237427)*0.5**o
        ref[1,1,1,1]=(-0.277152452881)/(o+1.)+(0.026273451739)+(0.342974599199)*0.5**o
        ref[1,1,2,0]=(-0.706035694041)/(o+1.)+(-0.710311473065)+(0.405298238411)*0.5**o
        ref[1,1,2,1]=(0.703696200954)/(o+1.)+(-0.157552780426)+(-0.679709121137)*0.5**o
        ref[1,2,0,0]=(0.689669884901)/(o+1.)+(-0.622211033177)+(-0.763765181249)*0.5**o
        ref[1,2,0,1]=(-0.947385900002)/(o+1.)+(-0.20513914627)+(-0.707293534802)*0.5**o
        ref[1,2,1,0]=(-0.459346493473)/(o+1.)+(-0.64857932325)+(0.02761949765)*0.5**o
        ref[1,2,1,1]=(-0.355112041776)/(o+1.)+(0.375707094)+(-0.399349607364)*0.5**o
        ref[1,2,2,0]=(-0.443893863435)/(o+1.)+(0.253182625616)+(0.900552977578)*0.5**o
        ref[1,2,2,1]=(-0.425767594915)/(o+1.)+(-0.239933168814)+(-0.231556056016)*0.5**o
        ref[1,3,0,0]=(-0.624599920754)/(o+1.)+(-0.281983492394)+(-0.753062984495)*0.5**o
        ref[1,3,0,1]=(-0.073644865041)/(o+1.)+(-0.0162881414573)+(0.810523164741)*0.5**o
        ref[1,3,1,0]=(-1.24391235759)/(o+1.)+(0.831790054284)+(-0.435427701923)*0.5**o
        ref[1,3,1,1]=(-0.533751440995)/(o+1.)+(0.927756350287)+(-0.647252848948)*0.5**o
        ref[1,3,2,0]=(0.94724084509)/(o+1.)+(0.278801731688)+(-0.607331889118)*0.5**o
        ref[1,3,2,1]=(-1.20110107011)/(o+1.)+(-0.405331821091)+(-0.484392978469)*0.5**o
        ref[1,4,0,0]=(-0.318296768485)/(o+1.)+(-0.322317995432)+(0.503702684537)*0.5**o
        ref[1,4,0,1]=(0.610458803004)/(o+1.)+(0.143663914066)+(0.255645379803)*0.5**o
        ref[1,4,1,0]=(-0.1327900717)/(o+1.)+(-0.957533991624)+(0.15831529726)*0.5**o
        ref[1,4,1,1]=(-0.0432831085882)/(o+1.)+(-0.70924595274)+(-0.547080506797)*0.5**o
        ref[1,4,2,0]=(0.825013730779)/(o+1.)+(0.605705762033)+(-0.369250635169)*0.5**o
        ref[1,4,2,1]=(-0.17860259488)/(o+1.)+(-0.09322044346)+(0.843010459592)*0.5**o
        ref[2,0,0,0]=(-0.702501447486)/(o+1.)+(-0.266898981454)+(-0.416783290665)*0.5**o
        ref[2,0,0,1]=(0.0851617887959)/(o+1.)+(-1.23585934539)+(0.513962651738)*0.5**o
        ref[2,0,1,0]=(-0.875542997875)/(o+1.)+(0.43876872457)+(0.431543077341)*0.5**o
        ref[2,0,1,1]=(0.00914963458101)/(o+1.)+(0.247035927055)+(-0.167337582492)*0.5**o
        ref[2,0,2,0]=(0.411492064125)/(o+1.)+(-0.371142085611)+(0.76364604242)*0.5**o
        ref[2,0,2,1]=(0.0976390387239)/(o+1.)+(0.347318010078)+(-0.656200752037)*0.5**o
        ref[2,1,0,0]=(0.828688613609)/(o+1.)+(0.189766089186)+(-0.168286367707)*0.5**o
        ref[2,1,0,1]=(0.00290248550037)/(o+1.)+(-0.143101314451)+(0.573088756891)*0.5**o
        ref[2,1,1,0]=(0.590551993988)/(o+1.)+(0.473266891646)+(-0.200994668199)*0.5**o
        ref[2,1,1,1]=(-0.411336302924)/(o+1.)+(0.218529870115)+(0.423677882618)*0.5**o
        ref[2,1,2,0]=(1.08387692054)/(o+1.)+(-0.435342485751)+(0.640928997042)*0.5**o
        ref[2,1,2,1]=(0.151742345195)/(o+1.)+(-0.990621676697)+(-0.730418336064)*0.5**o
        ref[2,2,0,0]=(-0.571297955608)/(o+1.)+(0.0294132836968)+(0.373430973034)*0.5**o
        ref[2,2,0,1]=(-1.71014229561)/(o+1.)+(0.520834380134)+(-0.384047362737)*0.5**o
        ref[2,2,1,0]=(0.000165159465638)/(o+1.)+(-0.464433746891)+(-0.978119189005)*0.5**o
        ref[2,2,1,1]=(-0.749448162051)/(o+1.)+(0.357300206217)+(-0.666661297789)*0.5**o
        ref[2,2,2,0]=(-1.25386694111)/(o+1.)+(-0.216497767701)+(-0.419553660247)*0.5**o
        ref[2,2,2,1]=(0.592534294577)/(o+1.)+(0.0876903859733)+(0.204828260085)*0.5**o
        ref[2,3,0,0]=(-0.231856869811)/(o+1.)+(-0.130806963846)+(-0.433426692328)*0.5**o
        ref[2,3,0,1]=(-0.746575173632)/(o+1.)+(0.47547676304)+(-0.753353152037)*0.5**o
        ref[2,3,1,0]=(1.22281769177)/(o+1.)+(0.106307888243)+(-0.770467840127)*0.5**o
        ref[2,3,1,1]=(-0.0833807583658)/(o+1.)+(0.0488093998039)+(-0.490969508319)*0.5**o
        ref[2,3,2,0]=(-0.69885834669)/(o+1.)+(0.552803583129)+(-0.719374773896)*0.5**o
        ref[2,3,2,1]=(1.2952311001)/(o+1.)+(0.189464753703)+(-0.514878672798)*0.5**o
        ref[2,4,0,0]=(0.132801546694)/(o+1.)+(0.367899034626)+(0.163562057356)*0.5**o
        ref[2,4,0,1]=(-0.827650079446)/(o+1.)+(1.06867197794)+(0.634549693325)*0.5**o
        ref[2,4,1,0]=(-0.20343056004)/(o+1.)+(0.301807185951)+(0.330425495069)*0.5**o
        ref[2,4,1,1]=(0.00513034050895)/(o+1.)+(-0.962405377783)+(0.106341453568)*0.5**o
        ref[2,4,2,0]=(0.0310460981037)/(o+1.)+(0.433282306621)+(0.994443804198)*0.5**o
        ref[2,4,2,1]=(0.902192873442)/(o+1.)+(-0.368558133903)+(0.113561397179)*0.5**o
        ref[3,0,0,0]=(0.900671903541)/(o+1.)+(0.202592699275)+(-0.463635004027)*0.5**o
        ref[3,0,0,1]=(0.35026221049)/(o+1.)+(0.341141942895)+(0.676123626258)*0.5**o
        ref[3,0,1,0]=(0.138548277811)/(o+1.)+(-0.0444101940734)+(-0.304361833494)*0.5**o
        ref[3,0,1,1]=(0.708876110175)/(o+1.)+(0.508195869745)+(0.307231747905)*0.5**o
        ref[3,0,2,0]=(0.982205702893)/(o+1.)+(0.314791291648)+(-0.130591836024)*0.5**o
        ref[3,0,2,1]=(1.34032270151)/(o+1.)+(-0.0845502854893)+(-0.576303721224)*0.5**o
        ref[3,1,0,0]=(-1.0578359675)/(o+1.)+(0.0482994274758)+(-0.798641869619)*0.5**o
        ref[3,1,0,1]=(-0.318745864355)/(o+1.)+(-0.535187955914)+(-0.00692013378255)*0.5**o
        ref[3,1,1,0]=(1.08164382033)/(o+1.)+(-0.0564910573463)+(0.975364017255)*0.5**o
        ref[3,1,1,1]=(0.304069039206)/(o+1.)+(0.818105356043)+(0.103423718636)*0.5**o
        ref[3,1,2,0]=(0.202131577493)/(o+1.)+(0.491584605592)+(0.0547068997542)*0.5**o
        ref[3,1,2,1]=(-0.790050626862)/(o+1.)+(0.652441993186)+(0.236730250658)*0.5**o
        ref[3,2,0,0]=(0.113163362881)/(o+1.)+(0.356243481631)+(0.388806525405)*0.5**o
        ref[3,2,0,1]=(1.09218675793)/(o+1.)+(-0.149692256988)+(-0.328460151904)*0.5**o
        ref[3,2,1,0]=(0.0360084147411)/(o+1.)+(-0.51843959536)+(-0.350954406466)*0.5**o
        ref[3,2,1,1]=(-0.455535353512)/(o+1.)+(0.497791987953)+(-0.804545424944)*0.5**o
        ref[3,2,2,0]=(-0.578849150697)/(o+1.)+(-1.18870958933)+(0.867664525177)*0.5**o
        ref[3,2,2,1]=(-0.51292246591)/(o+1.)+(-0.138210200569)+(0.997671019163)*0.5**o
        ref[3,3,0,0]=(1.21400753277)/(o+1.)+(-0.221646514915)+(-0.564960743631)*0.5**o
        ref[3,3,0,1]=(-0.201477939631)/(o+1.)+(0.113832346729)+(0.872459084593)*0.5**o
        ref[3,3,1,0]=(-0.679844797388)/(o+1.)+(0.168759614226)+(-0.256375661857)*0.5**o
        ref[3,3,1,1]=(-0.461574837551)/(o+1.)+(0.381760016613)+(-0.649399488997)*0.5**o
        ref[3,3,2,0]=(-1.6068534771)/(o+1.)+(-0.112445286441)+(0.237419242359)*0.5**o
        ref[3,3,2,1]=(0.756559815164)/(o+1.)+(0.110596703215)+(0.898298006002)*0.5**o
        ref[3,4,0,0]=(0.0482332220277)/(o+1.)+(0.457930350033)+(0.152469131812)*0.5**o
        ref[3,4,0,1]=(1.62019741951)/(o+1.)+(0.48918555421)+(-0.902280446763)*0.5**o
        ref[3,4,1,0]=(0.393133643776)/(o+1.)+(-0.852970591978)+(0.226692588742)*0.5**o
        ref[3,4,1,1]=(0.30297920343)/(o+1.)+(0.123825158749)+(0.99435141349)*0.5**o
        ref[3,4,2,0]=(0.0915688127321)/(o+1.)+(-0.0795234558081)+(-0.574679864968)*0.5**o
        ref[3,4,2,1]=(-1.84102882835)/(o+1.)+(0.261798322918)+(0.670125652473)*0.5**o
      res=integrate(arg)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactOne_fromData_ContinuousFunction_rank0(self):
      """
      tests integral of rank 0 Data on the ReducedFunctionOnContactOne

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(),w)
      if dim==2:
        arg=(-0.0333010052777)*x[0]**o+(-0.242011298555)*x[0]+(0.686457578633)*x[1]**o+(0.438506741453)*x[1]
        ref=(0.686457578633)/(o+1.)+(0.0982477214491)+(-0.0333010052777)*0.5**o
      else:
        arg=(-0.365813216932)*x[0]**o+(0.969816151161)*x[0]+(0.349995764077)*x[1]**o+(0.867492181692)*x[1]+(-0.866177355796)*x[2]**o+(-0.203219031102)*x[2]
        ref=(-0.516181591719)/(o+1.)+(0.817044650875)+(-0.365813216932)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactOne_fromData_ContinuousFunction_rank1(self):
      """
      tests integral of rank 1 Data on the ReducedFunctionOnContactOne

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(2,),w)
      ref=numpy.zeros((2,),numpy.float_)
      if dim==2:
        arg[0]=(0.038446371199)*x[0]**o+(0.323075821067)*x[0]+(0.770528210463)*x[1]**o+(0.0434528043023)*x[1]
        arg[1]=(-0.461876402566)*x[0]**o+(0.746965376214)*x[0]+(0.861234840159)*x[1]**o+(-0.568165009958)*x[1]
        ref[0]=(0.770528210463)/(o+1.)+(0.183264312685)+(0.038446371199)*0.5**o
        ref[1]=(0.861234840159)/(o+1.)+(0.0894001831281)+(-0.461876402566)*0.5**o
      else:
        arg[0]=(-0.472858663642)*x[0]**o+(-0.680210745369)*x[0]+(0.175425838082)*x[1]**o+(0.0189427039667)*x[1]+(0.0365198456146)*x[2]**o+(0.7840527457)*x[2]
        arg[1]=(-0.0191775972693)*x[0]**o+(0.19303923194)*x[0]+(0.775528295333)*x[1]**o+(-0.765500708496)*x[1]+(0.351369471606)*x[2]**o+(-0.351587727906)*x[2]
        ref[0]=(0.211945683696)/(o+1.)+(0.0613923521485)+(-0.472858663642)*0.5**o
        ref[1]=(1.12689776694)/(o+1.)+(-0.462024602231)+(-0.0191775972693)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactOne_fromData_ContinuousFunction_rank2(self):
      """
      tests integral of rank 2 Data on the ReducedFunctionOnContactOne

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5),w)
      ref=numpy.zeros((4, 5),numpy.float_)
      if dim==2:
        arg[0,0]=(-0.994443255559)*x[0]**o+(-0.403427184912)*x[0]+(-0.889656446862)*x[1]**o+(-0.411934106034)*x[1]
        arg[0,1]=(-0.864086447061)*x[0]**o+(-0.277489357308)*x[0]+(0.977910864864)*x[1]**o+(-0.0505775488074)*x[1]
        arg[0,2]=(-0.397179114534)*x[0]**o+(-0.448046821439)*x[0]+(-0.340253609736)*x[1]**o+(0.624988837716)*x[1]
        arg[0,3]=(-0.9155191116)*x[0]**o+(0.133777450278)*x[0]+(-0.809567026065)*x[1]**o+(-0.815374059364)*x[1]
        arg[0,4]=(-0.416764684181)*x[0]**o+(0.0965164570638)*x[0]+(0.190191799833)*x[1]**o+(0.0970709424519)*x[1]
        arg[1,0]=(0.646527294535)*x[0]**o+(-0.337119506636)*x[0]+(0.42669563245)*x[1]**o+(-0.185228568706)*x[1]
        arg[1,1]=(-0.411532647934)*x[0]**o+(0.178830973832)*x[0]+(-0.904886020892)*x[1]**o+(-0.740623071275)*x[1]
        arg[1,2]=(0.523024622808)*x[0]**o+(-0.226298848077)*x[0]+(-0.707740568617)*x[1]**o+(0.998182796571)*x[1]
        arg[1,3]=(0.840649917357)*x[0]**o+(-0.736279151571)*x[0]+(0.996017554392)*x[1]**o+(-0.397833319931)*x[1]
        arg[1,4]=(0.76711764419)*x[0]**o+(-0.905452991298)*x[0]+(-0.925675237951)*x[1]**o+(-0.982712645237)*x[1]
        arg[2,0]=(0.198547103682)*x[0]**o+(0.674537682234)*x[0]+(-0.891195618193)*x[1]**o+(0.979109691481)*x[1]
        arg[2,1]=(0.171629921737)*x[0]**o+(0.0102727001751)*x[0]+(0.467578393374)*x[1]**o+(0.583960984645)*x[1]
        arg[2,2]=(-0.637526271497)*x[0]**o+(-0.900821073531)*x[0]+(0.61637733965)*x[1]**o+(0.43717505341)*x[1]
        arg[2,3]=(-0.205861963967)*x[0]**o+(-0.104621062003)*x[0]+(0.582584591946)*x[1]**o+(0.351739662419)*x[1]
        arg[2,4]=(-0.179355911316)*x[0]**o+(-0.188415282678)*x[0]+(0.598342895007)*x[1]**o+(0.364638908577)*x[1]
        arg[3,0]=(0.150471526119)*x[0]**o+(-0.940789873202)*x[0]+(-0.524275825864)*x[1]**o+(0.729004683023)*x[1]
        arg[3,1]=(0.123933417087)*x[0]**o+(0.192413160498)*x[0]+(0.203730760507)*x[1]**o+(0.852010172966)*x[1]
        arg[3,2]=(-0.297766950847)*x[0]**o+(0.269905131596)*x[0]+(-0.0830931256847)*x[1]**o+(0.26964712569)*x[1]
        arg[3,3]=(0.96961517265)*x[0]**o+(0.363312967302)*x[0]+(-0.811142762589)*x[1]**o+(-0.420442983537)*x[1]
        arg[3,4]=(-0.314185602206)*x[0]**o+(-0.0446515646101)*x[0]+(-0.884651038732)*x[1]**o+(-0.166386477611)*x[1]
        ref[0,0]=(-0.889656446862)/(o+1.)+(-0.407680645473)+(-0.994443255559)*0.5**o
        ref[0,1]=(0.977910864864)/(o+1.)+(-0.164033453058)+(-0.864086447061)*0.5**o
        ref[0,2]=(-0.340253609736)/(o+1.)+(0.0884710081389)+(-0.397179114534)*0.5**o
        ref[0,3]=(-0.809567026065)/(o+1.)+(-0.340798304543)+(-0.9155191116)*0.5**o
        ref[0,4]=(0.190191799833)/(o+1.)+(0.0967936997578)+(-0.416764684181)*0.5**o
        ref[1,0]=(0.42669563245)/(o+1.)+(-0.261174037671)+(0.646527294535)*0.5**o
        ref[1,1]=(-0.904886020892)/(o+1.)+(-0.280896048722)+(-0.411532647934)*0.5**o
        ref[1,2]=(-0.707740568617)/(o+1.)+(0.385941974247)+(0.523024622808)*0.5**o
        ref[1,3]=(0.996017554392)/(o+1.)+(-0.567056235751)+(0.840649917357)*0.5**o
        ref[1,4]=(-0.925675237951)/(o+1.)+(-0.944082818268)+(0.76711764419)*0.5**o
        ref[2,0]=(-0.891195618193)/(o+1.)+(0.826823686857)+(0.198547103682)*0.5**o
        ref[2,1]=(0.467578393374)/(o+1.)+(0.29711684241)+(0.171629921737)*0.5**o
        ref[2,2]=(0.61637733965)/(o+1.)+(-0.231823010061)+(-0.637526271497)*0.5**o
        ref[2,3]=(0.582584591946)/(o+1.)+(0.123559300208)+(-0.205861963967)*0.5**o
        ref[2,4]=(0.598342895007)/(o+1.)+(0.0881118129495)+(-0.179355911316)*0.5**o
        ref[3,0]=(-0.524275825864)/(o+1.)+(-0.105892595089)+(0.150471526119)*0.5**o
        ref[3,1]=(0.203730760507)/(o+1.)+(0.522211666732)+(0.123933417087)*0.5**o
        ref[3,2]=(-0.0830931256847)/(o+1.)+(0.269776128643)+(-0.297766950847)*0.5**o
        ref[3,3]=(-0.811142762589)/(o+1.)+(-0.0285650081177)+(0.96961517265)*0.5**o
        ref[3,4]=(-0.884651038732)/(o+1.)+(-0.105519021111)+(-0.314185602206)*0.5**o
      else:
        arg[0,0]=(0.264691789613)*x[0]**o+(0.816541340997)*x[0]+(-0.402990464667)*x[1]**o+(-0.57479619687)*x[1]+(0.336313666995)*x[2]**o+(-0.353125285536)*x[2]
        arg[0,1]=(-0.0526559162494)*x[0]**o+(-0.901336706583)*x[0]+(0.237482536727)*x[1]**o+(-0.458564275773)*x[1]+(-0.90715992057)*x[2]**o+(-0.352062856077)*x[2]
        arg[0,2]=(0.786349112412)*x[0]**o+(-0.395569856472)*x[0]+(-0.996593023157)*x[1]**o+(-0.277001350627)*x[1]+(0.435375913289)*x[2]**o+(0.00380742297736)*x[2]
        arg[0,3]=(0.3336751197)*x[0]**o+(0.592185657129)*x[0]+(0.148956266814)*x[1]**o+(-0.852264651722)*x[1]+(-0.680249245511)*x[2]**o+(0.238843544413)*x[2]
        arg[0,4]=(0.875206676786)*x[0]**o+(0.0420423017124)*x[0]+(-0.690488811412)*x[1]**o+(-0.191328696331)*x[1]+(-0.861423624177)*x[2]**o+(0.483925405411)*x[2]
        arg[1,0]=(-0.573216715158)*x[0]**o+(-0.0668114726198)*x[0]+(0.382097918101)*x[1]**o+(-0.126485626438)*x[1]+(0.37521336616)*x[2]**o+(0.42402066058)*x[2]
        arg[1,1]=(-0.0919326514467)*x[0]**o+(-0.791351880042)*x[0]+(0.772694542694)*x[1]**o+(-0.537902003787)*x[1]+(0.424413342503)*x[2]**o+(-0.285553230731)*x[2]
        arg[1,2]=(0.855490362389)*x[0]**o+(0.60428104628)*x[0]+(-0.846565232506)*x[1]**o+(-0.243727717758)*x[1]+(0.900077399935)*x[2]**o+(-0.974305489047)*x[2]
        arg[1,3]=(-0.90802343526)*x[0]**o+(0.00950587842259)*x[0]+(0.821234711321)*x[1]**o+(-0.0628912085092)*x[1]+(-0.773563396868)*x[2]**o+(-0.403955952419)*x[2]
        arg[1,4]=(-0.33139358787)*x[0]**o+(-0.901382943687)*x[0]+(-0.981921375707)*x[1]**o+(-0.0654519807133)*x[1]+(0.880069861158)*x[2]**o+(-0.0796172270076)*x[2]
        arg[2,0]=(-0.202818692518)*x[0]**o+(-0.155436611971)*x[0]+(0.739467305515)*x[1]**o+(-0.12096877475)*x[1]+(0.479644796804)*x[2]**o+(0.0580715322389)*x[2]
        arg[2,1]=(0.0461326006936)*x[0]**o+(0.122168224674)*x[0]+(0.986788073931)*x[1]**o+(-0.435933938764)*x[1]+(-0.312527130215)*x[2]**o+(0.726805187165)*x[2]
        arg[2,2]=(-0.539659153805)*x[0]**o+(-0.26082353681)*x[0]+(0.367739719819)*x[1]**o+(0.943584828354)*x[1]+(0.688225221556)*x[2]**o+(-0.121563525696)*x[2]
        arg[2,3]=(0.861717128203)*x[0]**o+(0.0666983176426)*x[0]+(0.984510848927)*x[1]**o+(-0.321865378029)*x[1]+(0.522584734)*x[2]**o+(0.589527116625)*x[2]
        arg[2,4]=(0.778575699703)*x[0]**o+(0.515802361664)*x[0]+(0.817655905624)*x[1]**o+(0.883824717911)*x[1]+(0.656833761714)*x[2]**o+(-0.991585145611)*x[2]
        arg[3,0]=(0.752875513075)*x[0]**o+(0.615136567082)*x[0]+(-0.205265870704)*x[1]**o+(0.483918142728)*x[1]+(-0.0495322147557)*x[2]**o+(-0.132474571954)*x[2]
        arg[3,1]=(0.0563875061408)*x[0]**o+(-0.603033440119)*x[0]+(-0.0944809376206)*x[1]**o+(0.725330686731)*x[1]+(0.00527671533343)*x[2]**o+(-0.434519675586)*x[2]
        arg[3,2]=(0.100078550382)*x[0]**o+(-0.340988639989)*x[0]+(-0.392591649097)*x[1]**o+(-0.924705734588)*x[1]+(0.420410192075)*x[2]**o+(-0.94741221593)*x[2]
        arg[3,3]=(0.536414145801)*x[0]**o+(0.142421640429)*x[0]+(0.213162255423)*x[1]**o+(-0.695193008147)*x[1]+(0.9625091599)*x[2]**o+(0.534409542153)*x[2]
        arg[3,4]=(0.305683606235)*x[0]**o+(0.340964210305)*x[0]+(0.757807973246)*x[1]**o+(0.11459551493)*x[1]+(-0.81395047506)*x[2]**o+(0.636392281491)*x[2]
        ref[0,0]=(-0.0666767976714)/(o+1.)+(-0.0556900707042)+(0.264691789613)*0.5**o
        ref[0,1]=(-0.669677383843)/(o+1.)+(-0.855981919217)+(-0.0526559162494)*0.5**o
        ref[0,2]=(-0.561217109869)/(o+1.)+(-0.334381892061)+(0.786349112412)*0.5**o
        ref[0,3]=(-0.531292978697)/(o+1.)+(-0.0106177250896)+(0.3336751197)*0.5**o
        ref[0,4]=(-1.55191243559)/(o+1.)+(0.167319505396)+(0.875206676786)*0.5**o
        ref[1,0]=(0.757311284261)/(o+1.)+(0.115361780761)+(-0.573216715158)*0.5**o
        ref[1,1]=(1.1971078852)/(o+1.)+(-0.80740355728)+(-0.0919326514467)*0.5**o
        ref[1,2]=(0.0535121674292)/(o+1.)+(-0.306876080263)+(0.855490362389)*0.5**o
        ref[1,3]=(0.0476713144526)/(o+1.)+(-0.228670641253)+(-0.90802343526)*0.5**o
        ref[1,4]=(-0.101851514548)/(o+1.)+(-0.523226075704)+(-0.33139358787)*0.5**o
        ref[2,0]=(1.21911210232)/(o+1.)+(-0.109166927241)+(-0.202818692518)*0.5**o
        ref[2,1]=(0.674260943716)/(o+1.)+(0.206519736538)+(0.0461326006936)*0.5**o
        ref[2,2]=(1.05596494137)/(o+1.)+(0.280598882924)+(-0.539659153805)*0.5**o
        ref[2,3]=(1.50709558293)/(o+1.)+(0.167180028119)+(0.861717128203)*0.5**o
        ref[2,4]=(1.47448966734)/(o+1.)+(0.204020966982)+(0.778575699703)*0.5**o
        ref[3,0]=(-0.25479808546)/(o+1.)+(0.483290068928)+(0.752875513075)*0.5**o
        ref[3,1]=(-0.0892042222872)/(o+1.)+(-0.156111214487)+(0.0563875061408)*0.5**o
        ref[3,2]=(0.0278185429781)/(o+1.)+(-1.10655329525)+(0.100078550382)*0.5**o
        ref[3,3]=(1.17567141532)/(o+1.)+(-0.00918091278228)+(0.536414145801)*0.5**o
        ref[3,4]=(-0.0561425018138)/(o+1.)+(0.545976003363)+(0.305683606235)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactOne_fromData_ContinuousFunction_rank3(self):
      """
      tests integral of rank 3 Data on the ReducedFunctionOnContactOne

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(6, 2, 2),w)
      ref=numpy.zeros((6, 2, 2),numpy.float_)
      if dim==2:
        arg[0,0,0]=(0.448877659671)*x[0]**o+(0.130599173692)*x[0]+(-0.264185555629)*x[1]**o+(-0.722726040492)*x[1]
        arg[0,0,1]=(-0.543806308372)*x[0]**o+(-0.685242399166)*x[0]+(0.325336801501)*x[1]**o+(0.572960380228)*x[1]
        arg[0,1,0]=(-0.893080563297)*x[0]**o+(-0.251553981066)*x[0]+(0.256078388483)*x[1]**o+(-0.926701685259)*x[1]
        arg[0,1,1]=(0.346088914447)*x[0]**o+(0.166180418696)*x[0]+(0.353024178635)*x[1]**o+(0.731811935721)*x[1]
        arg[1,0,0]=(0.553083243483)*x[0]**o+(0.976153803522)*x[0]+(-0.260777317634)*x[1]**o+(-0.641936915628)*x[1]
        arg[1,0,1]=(-0.561076901068)*x[0]**o+(0.837832618302)*x[0]+(-0.638970151828)*x[1]**o+(-0.495169195543)*x[1]
        arg[1,1,0]=(-0.292805925132)*x[0]**o+(0.670472035586)*x[0]+(-0.752026521307)*x[1]**o+(-0.473087472861)*x[1]
        arg[1,1,1]=(-0.432857805275)*x[0]**o+(0.495387797249)*x[0]+(0.0579881469257)*x[1]**o+(0.831644440274)*x[1]
        arg[2,0,0]=(0.413433231384)*x[0]**o+(0.919831571878)*x[0]+(-0.373581626416)*x[1]**o+(0.16423318333)*x[1]
        arg[2,0,1]=(0.857566619293)*x[0]**o+(0.340454525569)*x[0]+(0.139298007219)*x[1]**o+(0.220542978265)*x[1]
        arg[2,1,0]=(-0.914184896115)*x[0]**o+(-0.0279186159835)*x[0]+(0.893417478351)*x[1]**o+(-0.108178714604)*x[1]
        arg[2,1,1]=(-0.749629612712)*x[0]**o+(-0.649957092299)*x[0]+(0.181944194719)*x[1]**o+(-0.988782927329)*x[1]
        arg[3,0,0]=(0.292013067678)*x[0]**o+(0.491680364317)*x[0]+(0.951144487798)*x[1]**o+(0.980949630108)*x[1]
        arg[3,0,1]=(0.89779245885)*x[0]**o+(-0.147546711846)*x[0]+(0.0902450198362)*x[1]**o+(-0.669984062969)*x[1]
        arg[3,1,0]=(0.62590631314)*x[0]**o+(0.329384454978)*x[0]+(-0.676917185409)*x[1]**o+(-0.0903173226074)*x[1]
        arg[3,1,1]=(-0.668268661271)*x[0]**o+(-0.847167879584)*x[0]+(0.329431843319)*x[1]**o+(-0.989253343115)*x[1]
        arg[4,0,0]=(-0.675111374291)*x[0]**o+(0.738030423939)*x[0]+(-0.413299452027)*x[1]**o+(0.36859580301)*x[1]
        arg[4,0,1]=(-0.207625666422)*x[0]**o+(-0.455639256266)*x[0]+(0.554118743878)*x[1]**o+(0.14044849621)*x[1]
        arg[4,1,0]=(-0.313337039275)*x[0]**o+(-0.451797308521)*x[0]+(0.201296275104)*x[1]**o+(-0.779869953962)*x[1]
        arg[4,1,1]=(-0.0308697706768)*x[0]**o+(0.214664850788)*x[0]+(-0.824157101269)*x[1]**o+(0.156012836486)*x[1]
        arg[5,0,0]=(0.277885566309)*x[0]**o+(-0.387820938776)*x[0]+(0.820602896312)*x[1]**o+(0.162715513355)*x[1]
        arg[5,0,1]=(-0.216951421349)*x[0]**o+(0.906019329549)*x[0]+(-0.330132336808)*x[1]**o+(0.416109194357)*x[1]
        arg[5,1,0]=(0.923496627826)*x[0]**o+(-0.177644464371)*x[0]+(0.482278717995)*x[1]**o+(-0.362003106154)*x[1]
        arg[5,1,1]=(0.9109111591)*x[0]**o+(-0.0919782899769)*x[0]+(0.103950387446)*x[1]**o+(0.447452648868)*x[1]
        ref[0,0,0]=(-0.264185555629)/(o+1.)+(-0.2960634334)+(0.448877659671)*0.5**o
        ref[0,0,1]=(0.325336801501)/(o+1.)+(-0.0561410094689)+(-0.543806308372)*0.5**o
        ref[0,1,0]=(0.256078388483)/(o+1.)+(-0.589127833162)+(-0.893080563297)*0.5**o
        ref[0,1,1]=(0.353024178635)/(o+1.)+(0.448996177208)+(0.346088914447)*0.5**o
        ref[1,0,0]=(-0.260777317634)/(o+1.)+(0.167108443947)+(0.553083243483)*0.5**o
        ref[1,0,1]=(-0.638970151828)/(o+1.)+(0.17133171138)+(-0.561076901068)*0.5**o
        ref[1,1,0]=(-0.752026521307)/(o+1.)+(0.0986922813626)+(-0.292805925132)*0.5**o
        ref[1,1,1]=(0.0579881469257)/(o+1.)+(0.663516118762)+(-0.432857805275)*0.5**o
        ref[2,0,0]=(-0.373581626416)/(o+1.)+(0.542032377604)+(0.413433231384)*0.5**o
        ref[2,0,1]=(0.139298007219)/(o+1.)+(0.280498751917)+(0.857566619293)*0.5**o
        ref[2,1,0]=(0.893417478351)/(o+1.)+(-0.0680486652939)+(-0.914184896115)*0.5**o
        ref[2,1,1]=(0.181944194719)/(o+1.)+(-0.819370009814)+(-0.749629612712)*0.5**o
        ref[3,0,0]=(0.951144487798)/(o+1.)+(0.736314997212)+(0.292013067678)*0.5**o
        ref[3,0,1]=(0.0902450198362)/(o+1.)+(-0.408765387408)+(0.89779245885)*0.5**o
        ref[3,1,0]=(-0.676917185409)/(o+1.)+(0.119533566185)+(0.62590631314)*0.5**o
        ref[3,1,1]=(0.329431843319)/(o+1.)+(-0.91821061135)+(-0.668268661271)*0.5**o
        ref[4,0,0]=(-0.413299452027)/(o+1.)+(0.553313113475)+(-0.675111374291)*0.5**o
        ref[4,0,1]=(0.554118743878)/(o+1.)+(-0.157595380028)+(-0.207625666422)*0.5**o
        ref[4,1,0]=(0.201296275104)/(o+1.)+(-0.615833631241)+(-0.313337039275)*0.5**o
        ref[4,1,1]=(-0.824157101269)/(o+1.)+(0.185338843637)+(-0.0308697706768)*0.5**o
        ref[5,0,0]=(0.820602896312)/(o+1.)+(-0.11255271271)+(0.277885566309)*0.5**o
        ref[5,0,1]=(-0.330132336808)/(o+1.)+(0.661064261953)+(-0.216951421349)*0.5**o
        ref[5,1,0]=(0.482278717995)/(o+1.)+(-0.269823785262)+(0.923496627826)*0.5**o
        ref[5,1,1]=(0.103950387446)/(o+1.)+(0.177737179446)+(0.9109111591)*0.5**o
      else:
        arg[0,0,0]=(0.843169691966)*x[0]**o+(0.252560741792)*x[0]+(0.0428740803857)*x[1]**o+(-0.702885246223)*x[1]+(0.175406156382)*x[2]**o+(-0.819206174419)*x[2]
        arg[0,0,1]=(-0.780678250187)*x[0]**o+(-0.628781977796)*x[0]+(0.345241549754)*x[1]**o+(-0.686783366275)*x[1]+(0.298501249797)*x[2]**o+(-0.592222734778)*x[2]
        arg[0,1,0]=(0.680331984783)*x[0]**o+(0.20945731105)*x[0]+(0.582220641498)*x[1]**o+(0.159144772508)*x[1]+(0.197544685245)*x[2]**o+(-0.542847499692)*x[2]
        arg[0,1,1]=(0.561649107089)*x[0]**o+(0.409538404439)*x[0]+(0.687461001448)*x[1]**o+(0.327986995905)*x[1]+(0.701007758697)*x[2]**o+(0.95919995454)*x[2]
        arg[1,0,0]=(0.0620287124894)*x[0]**o+(-0.946536581396)*x[0]+(-0.813637608115)*x[1]**o+(0.0808967031579)*x[1]+(0.545408819059)*x[2]**o+(0.250558134093)*x[2]
        arg[1,0,1]=(-0.0778300410758)*x[0]**o+(-0.852201314682)*x[0]+(0.712528858815)*x[1]**o+(-0.69009644973)*x[1]+(-0.0836140529567)*x[2]**o+(0.687579338803)*x[2]
        arg[1,1,0]=(-0.37648103863)*x[0]**o+(-0.264461846666)*x[0]+(-0.45921598716)*x[1]**o+(-0.516551658367)*x[1]+(0.512446750786)*x[2]**o+(0.850266522322)*x[2]
        arg[1,1,1]=(-0.951741356481)*x[0]**o+(-0.989023540683)*x[0]+(0.338787640816)*x[1]**o+(0.764043925298)*x[1]+(0.524009019713)*x[2]**o+(-0.912507794534)*x[2]
        arg[2,0,0]=(0.613468181683)*x[0]**o+(-0.357325179573)*x[0]+(0.153221256731)*x[1]**o+(-0.748686771579)*x[1]+(0.364085012368)*x[2]**o+(-0.134535056195)*x[2]
        arg[2,0,1]=(-0.728948376511)*x[0]**o+(-0.0657162843876)*x[0]+(-0.336013775703)*x[1]**o+(0.299044747397)*x[1]+(-0.712877779871)*x[2]**o+(0.229974926734)*x[2]
        arg[2,1,0]=(-0.213428159952)*x[0]**o+(0.680250111549)*x[0]+(-0.986942572737)*x[1]**o+(0.549006049647)*x[1]+(0.572921637402)*x[2]**o+(-0.241851127933)*x[2]
        arg[2,1,1]=(0.0367155018841)*x[0]**o+(-0.768111161527)*x[0]+(-0.748085216726)*x[1]**o+(-0.752536272228)*x[1]+(0.555945063877)*x[2]**o+(0.012687074005)*x[2]
        arg[3,0,0]=(-0.945391902075)*x[0]**o+(-0.581330133808)*x[0]+(-0.253390460088)*x[1]**o+(-0.885589795648)*x[1]+(-0.555900062171)*x[2]**o+(-0.241006607054)*x[2]
        arg[3,0,1]=(0.924110231188)*x[0]**o+(0.428520753545)*x[0]+(0.187773835144)*x[1]**o+(-0.53787305121)*x[1]+(0.0970284978901)*x[2]**o+(0.0580212350951)*x[2]
        arg[3,1,0]=(0.426389283701)*x[0]**o+(-0.0976148877161)*x[0]+(-0.364828466078)*x[1]**o+(-0.536398529197)*x[1]+(0.648605043498)*x[2]**o+(-0.234629706559)*x[2]
        arg[3,1,1]=(-0.736098543054)*x[0]**o+(0.985014182905)*x[0]+(-0.563513361433)*x[1]**o+(0.589434310221)*x[1]+(-0.360296462891)*x[2]**o+(-0.586591636182)*x[2]
        arg[4,0,0]=(0.953916732391)*x[0]**o+(-0.835620914249)*x[0]+(0.562803727423)*x[1]**o+(-0.904263689551)*x[1]+(-0.349815820142)*x[2]**o+(-0.156410988081)*x[2]
        arg[4,0,1]=(-0.236710267891)*x[0]**o+(0.958992060128)*x[0]+(0.711565908363)*x[1]**o+(-0.639771085045)*x[1]+(-0.919042661652)*x[2]**o+(-0.198907436333)*x[2]
        arg[4,1,0]=(0.275058935551)*x[0]**o+(0.563461500878)*x[0]+(0.438833998471)*x[1]**o+(0.686375712335)*x[1]+(-0.930322605963)*x[2]**o+(0.710777122765)*x[2]
        arg[4,1,1]=(-0.507845102241)*x[0]**o+(-0.390574086909)*x[0]+(-0.170020669232)*x[1]**o+(-0.107455242464)*x[1]+(-0.97056652533)*x[2]**o+(-0.720180192255)*x[2]
        arg[5,0,0]=(-0.922597172214)*x[0]**o+(0.229197305733)*x[0]+(-0.943214499543)*x[1]**o+(0.269256599046)*x[1]+(-0.11540537404)*x[2]**o+(0.956940487934)*x[2]
        arg[5,0,1]=(-0.0350868975578)*x[0]**o+(-0.547620412319)*x[0]+(-0.818344866429)*x[1]**o+(-0.0632082353078)*x[1]+(-0.490536679501)*x[2]**o+(-0.79641747407)*x[2]
        arg[5,1,0]=(0.877279477171)*x[0]**o+(-0.408752853807)*x[0]+(0.143432191997)*x[1]**o+(0.230679627667)*x[1]+(0.0346230515425)*x[2]**o+(0.357377660033)*x[2]
        arg[5,1,1]=(0.294375780312)*x[0]**o+(-0.688447065878)*x[0]+(0.213334642918)*x[1]**o+(0.664021137697)*x[1]+(-0.532996065961)*x[2]**o+(0.794315136407)*x[2]
        ref[0,0,0]=(0.218280236768)/(o+1.)+(-0.634765339425)+(0.843169691966)*0.5**o
        ref[0,0,1]=(0.64374279955)/(o+1.)+(-0.953894039424)+(-0.780678250187)*0.5**o
        ref[0,1,0]=(0.779765326742)/(o+1.)+(-0.0871227080671)+(0.680331984783)*0.5**o
        ref[0,1,1]=(1.38846876015)/(o+1.)+(0.848362677442)+(0.561649107089)*0.5**o
        ref[1,0,0]=(-0.268228789055)/(o+1.)+(-0.307540872072)+(0.0620287124894)*0.5**o
        ref[1,0,1]=(0.628914805859)/(o+1.)+(-0.427359212804)+(-0.0778300410758)*0.5**o
        ref[1,1,0]=(0.0532307636262)/(o+1.)+(0.0346265086445)+(-0.37648103863)*0.5**o
        ref[1,1,1]=(0.862796660529)/(o+1.)+(-0.56874370496)+(-0.951741356481)*0.5**o
        ref[2,0,0]=(0.5173062691)/(o+1.)+(-0.620273503673)+(0.613468181683)*0.5**o
        ref[2,0,1]=(-1.04889155557)/(o+1.)+(0.231651694871)+(-0.728948376511)*0.5**o
        ref[2,1,0]=(-0.414020935335)/(o+1.)+(0.493702516631)+(-0.213428159952)*0.5**o
        ref[2,1,1]=(-0.192140152849)/(o+1.)+(-0.753980179875)+(0.0367155018841)*0.5**o
        ref[3,0,0]=(-0.809290522259)/(o+1.)+(-0.853963268255)+(-0.945391902075)*0.5**o
        ref[3,0,1]=(0.284802333034)/(o+1.)+(-0.0256655312851)+(0.924110231188)*0.5**o
        ref[3,1,0]=(0.28377657742)/(o+1.)+(-0.434321561736)+(0.426389283701)*0.5**o
        ref[3,1,1]=(-0.923809824324)/(o+1.)+(0.493928428472)+(-0.736098543054)*0.5**o
        ref[4,0,0]=(0.212987907281)/(o+1.)+(-0.948147795941)+(0.953916732391)*0.5**o
        ref[4,0,1]=(-0.207476753289)/(o+1.)+(0.0601567693744)+(-0.236710267891)*0.5**o
        ref[4,1,0]=(-0.491488607492)/(o+1.)+(0.980307167989)+(0.275058935551)*0.5**o
        ref[4,1,1]=(-1.14058719456)/(o+1.)+(-0.609104760814)+(-0.507845102241)*0.5**o
        ref[5,0,0]=(-1.05861987358)/(o+1.)+(0.727697196357)+(-0.922597172214)*0.5**o
        ref[5,0,1]=(-1.30888154593)/(o+1.)+(-0.703623060848)+(-0.0350868975578)*0.5**o
        ref[5,1,0]=(0.17805524354)/(o+1.)+(0.0896522169463)+(0.877279477171)*0.5**o
        ref[5,1,1]=(-0.319661423044)/(o+1.)+(0.384944604113)+(0.294375780312)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactOne_fromData_ContinuousFunction_rank4(self):
      """
      tests integral of rank 4 Data on the ReducedFunctionOnContactOne

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5, 3, 2),w)
      ref=numpy.zeros((4, 5, 3, 2),numpy.float_)
      if dim==2:
        arg[0,0,0,0]=(0.182956096072)*x[0]**o+(0.857065806542)*x[0]+(-0.0352052809673)*x[1]**o+(0.950661773214)*x[1]
        arg[0,0,0,1]=(-0.81342189211)*x[0]**o+(0.36137453013)*x[0]+(0.0610373691872)*x[1]**o+(-0.974859653337)*x[1]
        arg[0,0,1,0]=(-0.149103531408)*x[0]**o+(0.853517194849)*x[0]+(0.623943596759)*x[1]**o+(0.441477983782)*x[1]
        arg[0,0,1,1]=(0.74912087771)*x[0]**o+(0.935896347068)*x[0]+(0.136600247183)*x[1]**o+(0.915043476739)*x[1]
        arg[0,0,2,0]=(-0.289042694587)*x[0]**o+(-0.0324580134489)*x[0]+(0.966454801911)*x[1]**o+(0.158747002668)*x[1]
        arg[0,0,2,1]=(0.0809737838902)*x[0]**o+(0.483125829124)*x[0]+(-0.156074498516)*x[1]**o+(-0.387215102608)*x[1]
        arg[0,1,0,0]=(0.638822193145)*x[0]**o+(-0.211942445721)*x[0]+(-0.016865109839)*x[1]**o+(-0.596486509057)*x[1]
        arg[0,1,0,1]=(0.611604473356)*x[0]**o+(-0.330484367791)*x[0]+(-0.897448372147)*x[1]**o+(-0.0188556077191)*x[1]
        arg[0,1,1,0]=(-0.745407006471)*x[0]**o+(0.547985829362)*x[0]+(-0.229098133178)*x[1]**o+(0.339357994536)*x[1]
        arg[0,1,1,1]=(0.756528549281)*x[0]**o+(0.0968167016962)*x[0]+(0.481206893907)*x[1]**o+(-0.320299781291)*x[1]
        arg[0,1,2,0]=(0.335544368565)*x[0]**o+(0.143490409353)*x[0]+(0.367303506039)*x[1]**o+(-0.459001464112)*x[1]
        arg[0,1,2,1]=(0.669933676654)*x[0]**o+(0.0679909847194)*x[0]+(0.182483888026)*x[1]**o+(-0.382138842505)*x[1]
        arg[0,2,0,0]=(-0.929163949925)*x[0]**o+(0.670533855076)*x[0]+(0.587675819052)*x[1]**o+(0.221939557678)*x[1]
        arg[0,2,0,1]=(0.639896773528)*x[0]**o+(-0.166901547286)*x[0]+(-0.83934259298)*x[1]**o+(-0.522536841927)*x[1]
        arg[0,2,1,0]=(-0.399556357179)*x[0]**o+(0.860582177909)*x[0]+(0.675592435624)*x[1]**o+(0.77480796903)*x[1]
        arg[0,2,1,1]=(-0.281969675471)*x[0]**o+(-0.250831251067)*x[0]+(0.14396936899)*x[1]**o+(-0.65775670156)*x[1]
        arg[0,2,2,0]=(0.339204154215)*x[0]**o+(0.125319656279)*x[0]+(0.417557775604)*x[1]**o+(-0.985431777777)*x[1]
        arg[0,2,2,1]=(-0.655500794874)*x[0]**o+(-0.419889131296)*x[0]+(-0.0763188160723)*x[1]**o+(0.995600932757)*x[1]
        arg[0,3,0,0]=(-0.483746115201)*x[0]**o+(-0.256632816498)*x[0]+(-0.0533973517656)*x[1]**o+(-0.982541041474)*x[1]
        arg[0,3,0,1]=(0.446810992304)*x[0]**o+(-0.823523374302)*x[0]+(-0.607238533539)*x[1]**o+(-0.97562384609)*x[1]
        arg[0,3,1,0]=(-0.0235907627093)*x[0]**o+(-0.0720587626585)*x[0]+(0.930589323947)*x[1]**o+(0.327916365569)*x[1]
        arg[0,3,1,1]=(0.486342062258)*x[0]**o+(0.430279082913)*x[0]+(0.237445053125)*x[1]**o+(0.577650421809)*x[1]
        arg[0,3,2,0]=(-0.581025309937)*x[0]**o+(0.97026755344)*x[0]+(0.180505264557)*x[1]**o+(0.289585805796)*x[1]
        arg[0,3,2,1]=(-0.523356041281)*x[0]**o+(-0.502230606793)*x[0]+(0.915270822508)*x[1]**o+(0.622076448072)*x[1]
        arg[0,4,0,0]=(-0.584782530121)*x[0]**o+(-0.433761163729)*x[0]+(-0.449731522195)*x[1]**o+(0.392234342377)*x[1]
        arg[0,4,0,1]=(-0.0724106581167)*x[0]**o+(0.76896002907)*x[0]+(0.0374933582543)*x[1]**o+(-0.417790382045)*x[1]
        arg[0,4,1,0]=(0.0460360663465)*x[0]**o+(-0.0343116732267)*x[0]+(0.815864788848)*x[1]**o+(-0.922236546574)*x[1]
        arg[0,4,1,1]=(-0.978584425712)*x[0]**o+(0.249337867126)*x[0]+(-0.822961904166)*x[1]**o+(0.627473480274)*x[1]
        arg[0,4,2,0]=(-0.267227111945)*x[0]**o+(0.791320996051)*x[0]+(0.721009228711)*x[1]**o+(0.00199546728243)*x[1]
        arg[0,4,2,1]=(-0.141372777283)*x[0]**o+(-0.996912487736)*x[0]+(0.94281427253)*x[1]**o+(-0.780685413978)*x[1]
        arg[1,0,0,0]=(-0.721911758033)*x[0]**o+(0.0119756829508)*x[0]+(0.583083015744)*x[1]**o+(0.80826073507)*x[1]
        arg[1,0,0,1]=(0.929636966638)*x[0]**o+(-0.35107696413)*x[0]+(-0.656523378462)*x[1]**o+(0.610681274197)*x[1]
        arg[1,0,1,0]=(0.594847872723)*x[0]**o+(-0.15013091781)*x[0]+(-0.562311637909)*x[1]**o+(0.871283992407)*x[1]
        arg[1,0,1,1]=(0.529274819592)*x[0]**o+(0.998035152453)*x[0]+(-0.265309908189)*x[1]**o+(-0.703391843389)*x[1]
        arg[1,0,2,0]=(0.2321959905)*x[0]**o+(-0.781602927232)*x[0]+(-0.0509405210126)*x[1]**o+(0.839298348419)*x[1]
        arg[1,0,2,1]=(0.0811824999287)*x[0]**o+(0.916373714851)*x[0]+(0.817375877796)*x[1]**o+(-0.926274023089)*x[1]
        arg[1,1,0,0]=(0.418525096774)*x[0]**o+(0.064653235493)*x[0]+(0.15912570069)*x[1]**o+(0.47570489259)*x[1]
        arg[1,1,0,1]=(0.784138248952)*x[0]**o+(-0.715130508682)*x[0]+(-0.299421222048)*x[1]**o+(-0.816706522916)*x[1]
        arg[1,1,1,0]=(-0.911449050332)*x[0]**o+(-0.472182975049)*x[0]+(0.935180253062)*x[1]**o+(0.527208406385)*x[1]
        arg[1,1,1,1]=(-0.378301315644)*x[0]**o+(0.430065353544)*x[0]+(0.827581865806)*x[1]**o+(-0.355263817375)*x[1]
        arg[1,1,2,0]=(-0.882543004053)*x[0]**o+(0.503794963637)*x[0]+(-0.580086073626)*x[1]**o+(0.545676378283)*x[1]
        arg[1,1,2,1]=(-0.116197074531)*x[0]**o+(-0.0629871602139)*x[0]+(0.320343551712)*x[1]**o+(0.0552550885696)*x[1]
        arg[1,2,0,0]=(-0.870347384636)*x[0]**o+(-0.611439594673)*x[0]+(0.707766015783)*x[1]**o+(0.0653036268139)*x[1]
        arg[1,2,0,1]=(-0.219872666014)*x[0]**o+(-0.435867669656)*x[0]+(0.825489444837)*x[1]**o+(-0.737253859382)*x[1]
        arg[1,2,1,0]=(-0.963149287055)*x[0]**o+(0.964306345088)*x[0]+(0.502736178959)*x[1]**o+(0.998017404878)*x[1]
        arg[1,2,1,1]=(-0.586350249194)*x[0]**o+(-0.437455605038)*x[0]+(-0.182719660538)*x[1]**o+(-0.50797820305)*x[1]
        arg[1,2,2,0]=(-0.423818475166)*x[0]**o+(0.290902418459)*x[0]+(-0.304129806418)*x[1]**o+(-0.67571878331)*x[1]
        arg[1,2,2,1]=(0.901384945886)*x[0]**o+(0.0526906910209)*x[0]+(0.438755721196)*x[1]**o+(-0.774352941544)*x[1]
        arg[1,3,0,0]=(-0.797968602257)*x[0]**o+(0.856637297517)*x[0]+(0.427243495756)*x[1]**o+(0.212942988457)*x[1]
        arg[1,3,0,1]=(-0.696126104381)*x[0]**o+(0.616445534254)*x[0]+(-0.943037028688)*x[1]**o+(0.807265721375)*x[1]
        arg[1,3,1,0]=(-0.69028368426)*x[0]**o+(-0.652017674488)*x[0]+(-0.772965740874)*x[1]**o+(-0.726830598291)*x[1]
        arg[1,3,1,1]=(-0.534347181294)*x[0]**o+(-0.861020241321)*x[0]+(-0.250549859157)*x[1]**o+(-0.638289396201)*x[1]
        arg[1,3,2,0]=(-0.827981962115)*x[0]**o+(-0.427544131008)*x[0]+(-0.516514308221)*x[1]**o+(-0.101906062547)*x[1]
        arg[1,3,2,1]=(0.864701869194)*x[0]**o+(0.876327129524)*x[0]+(0.806201346091)*x[1]**o+(0.845259832272)*x[1]
        arg[1,4,0,0]=(-0.589044559231)*x[0]**o+(0.697104039773)*x[0]+(-0.288389617381)*x[1]**o+(0.422618969092)*x[1]
        arg[1,4,0,1]=(0.814971550967)*x[0]**o+(-0.665151459593)*x[0]+(-0.696145763418)*x[1]**o+(0.906949165457)*x[1]
        arg[1,4,1,0]=(0.760190450826)*x[0]**o+(-0.883964913626)*x[0]+(-0.507159320061)*x[1]**o+(-0.15707026975)*x[1]
        arg[1,4,1,1]=(-0.0416603697257)*x[0]**o+(0.560561779164)*x[0]+(0.0153574982122)*x[1]**o+(0.75228872946)*x[1]
        arg[1,4,2,0]=(0.105929120872)*x[0]**o+(0.130738968719)*x[0]+(-0.0991159921294)*x[1]**o+(0.972627650896)*x[1]
        arg[1,4,2,1]=(-0.503911541225)*x[0]**o+(-0.00357063650385)*x[0]+(0.377083955928)*x[1]**o+(0.343010153434)*x[1]
        arg[2,0,0,0]=(0.953733725947)*x[0]**o+(-0.988629842686)*x[0]+(0.196766730386)*x[1]**o+(0.0427993038279)*x[1]
        arg[2,0,0,1]=(0.491469820597)*x[0]**o+(0.708248604732)*x[0]+(-0.600347310418)*x[1]**o+(-0.0380188651191)*x[1]
        arg[2,0,1,0]=(0.615118477478)*x[0]**o+(0.832030863133)*x[0]+(0.783037278839)*x[1]**o+(-0.331824170309)*x[1]
        arg[2,0,1,1]=(-0.806125471265)*x[0]**o+(0.763233458333)*x[0]+(-0.381389710969)*x[1]**o+(0.652974431609)*x[1]
        arg[2,0,2,0]=(-0.831727679795)*x[0]**o+(0.239296785135)*x[0]+(-0.13737609858)*x[1]**o+(0.583210422723)*x[1]
        arg[2,0,2,1]=(-0.805843206194)*x[0]**o+(-0.523576669019)*x[0]+(0.159758297984)*x[1]**o+(-0.298308874886)*x[1]
        arg[2,1,0,0]=(0.589135530105)*x[0]**o+(0.272145879514)*x[0]+(0.325741074892)*x[1]**o+(-0.579782654647)*x[1]
        arg[2,1,0,1]=(0.106049062774)*x[0]**o+(0.790280056075)*x[0]+(0.0171600098883)*x[1]**o+(-0.419540445094)*x[1]
        arg[2,1,1,0]=(-0.181511670026)*x[0]**o+(-0.814531462508)*x[0]+(-0.576165158737)*x[1]**o+(0.679678310717)*x[1]
        arg[2,1,1,1]=(-0.709695453728)*x[0]**o+(-0.489487606257)*x[0]+(-0.424093531654)*x[1]**o+(0.535427169862)*x[1]
        arg[2,1,2,0]=(0.60069280023)*x[0]**o+(-0.909882731214)*x[0]+(0.640141428017)*x[1]**o+(-0.697396064954)*x[1]
        arg[2,1,2,1]=(0.896964594562)*x[0]**o+(0.976387328697)*x[0]+(0.144061490066)*x[1]**o+(0.378042869796)*x[1]
        arg[2,2,0,0]=(0.447633246258)*x[0]**o+(-0.531416367062)*x[0]+(0.199304002742)*x[1]**o+(0.85195706213)*x[1]
        arg[2,2,0,1]=(-0.683614706554)*x[0]**o+(0.933255565209)*x[0]+(-0.328047392082)*x[1]**o+(-0.0865769137182)*x[1]
        arg[2,2,1,0]=(0.22474720984)*x[0]**o+(0.0910500178874)*x[0]+(0.449220001922)*x[1]**o+(-0.0406214557829)*x[1]
        arg[2,2,1,1]=(-0.266419887395)*x[0]**o+(-0.48883285131)*x[0]+(-0.794933934805)*x[1]**o+(0.182439649969)*x[1]
        arg[2,2,2,0]=(0.949852186215)*x[0]**o+(-0.116121704259)*x[0]+(-0.977131144423)*x[1]**o+(0.698402625747)*x[1]
        arg[2,2,2,1]=(-0.0446180768654)*x[0]**o+(-0.742263614438)*x[0]+(-0.898675367333)*x[1]**o+(-0.260891146746)*x[1]
        arg[2,3,0,0]=(0.819774310005)*x[0]**o+(-0.0677371406664)*x[0]+(0.276747165681)*x[1]**o+(-0.42828783)*x[1]
        arg[2,3,0,1]=(-0.391001671385)*x[0]**o+(0.0558814994728)*x[0]+(-0.624786978129)*x[1]**o+(0.178742280566)*x[1]
        arg[2,3,1,0]=(0.904597103646)*x[0]**o+(-0.825472271707)*x[0]+(0.112221476422)*x[1]**o+(0.269349199527)*x[1]
        arg[2,3,1,1]=(-0.190955697638)*x[0]**o+(0.441698994606)*x[0]+(-0.26229840557)*x[1]**o+(0.148974540187)*x[1]
        arg[2,3,2,0]=(-0.873098622934)*x[0]**o+(-0.624915584376)*x[0]+(-0.151646578512)*x[1]**o+(-0.914878228283)*x[1]
        arg[2,3,2,1]=(-0.712569947477)*x[0]**o+(-0.72679361801)*x[0]+(-0.0946869166491)*x[1]**o+(-0.580512407933)*x[1]
        arg[2,4,0,0]=(-0.655752680768)*x[0]**o+(-0.409544975855)*x[0]+(0.865325438086)*x[1]**o+(-0.227397026937)*x[1]
        arg[2,4,0,1]=(0.670566711209)*x[0]**o+(-0.794343533883)*x[0]+(-0.127116396219)*x[1]**o+(-0.422419866647)*x[1]
        arg[2,4,1,0]=(0.46903441444)*x[0]**o+(0.0689129759715)*x[0]+(-0.549325647826)*x[1]**o+(0.893700020399)*x[1]
        arg[2,4,1,1]=(-0.0099246750652)*x[0]**o+(0.446742615509)*x[0]+(0.900251202513)*x[1]**o+(0.221599931055)*x[1]
        arg[2,4,2,0]=(0.218854380066)*x[0]**o+(0.877849336909)*x[0]+(-0.338211083089)*x[1]**o+(0.746950973063)*x[1]
        arg[2,4,2,1]=(-0.890762146523)*x[0]**o+(0.439653904008)*x[0]+(0.0802890292215)*x[1]**o+(-0.305233739845)*x[1]
        arg[3,0,0,0]=(0.284622209957)*x[0]**o+(0.983284338924)*x[0]+(-0.732588048234)*x[1]**o+(0.654066388109)*x[1]
        arg[3,0,0,1]=(0.379628076244)*x[0]**o+(-0.75362875047)*x[0]+(0.288422119968)*x[1]**o+(0.139197401407)*x[1]
        arg[3,0,1,0]=(0.0594472092467)*x[0]**o+(-0.391581973307)*x[0]+(-0.898996153568)*x[1]**o+(-2.75165483707e-05)*x[1]
        arg[3,0,1,1]=(-0.241252797172)*x[0]**o+(-0.87384982932)*x[0]+(0.811628957112)*x[1]**o+(0.0170539509618)*x[1]
        arg[3,0,2,0]=(0.255617459837)*x[0]**o+(-0.744071491889)*x[0]+(0.745552769828)*x[1]**o+(0.128612266243)*x[1]
        arg[3,0,2,1]=(0.0725082197254)*x[0]**o+(0.779602587374)*x[0]+(-0.751879287668)*x[1]**o+(0.838360641095)*x[1]
        arg[3,1,0,0]=(-0.444520674459)*x[0]**o+(-0.811442143542)*x[0]+(-0.485010478999)*x[1]**o+(0.727600464745)*x[1]
        arg[3,1,0,1]=(0.260061065042)*x[0]**o+(0.932320346965)*x[0]+(0.556377882844)*x[1]**o+(0.284348284894)*x[1]
        arg[3,1,1,0]=(0.561755002522)*x[0]**o+(-0.842983435746)*x[0]+(0.729209244377)*x[1]**o+(-0.254683846974)*x[1]
        arg[3,1,1,1]=(-0.949556399673)*x[0]**o+(-0.0804522142379)*x[0]+(0.066141483426)*x[1]**o+(0.78626145966)*x[1]
        arg[3,1,2,0]=(0.833198651881)*x[0]**o+(-0.297955946153)*x[0]+(-0.561247934789)*x[1]**o+(-0.645124432611)*x[1]
        arg[3,1,2,1]=(-0.641727656043)*x[0]**o+(0.32981201062)*x[0]+(-0.152588766563)*x[1]**o+(-0.289611460761)*x[1]
        arg[3,2,0,0]=(0.825643539598)*x[0]**o+(0.924286336436)*x[0]+(0.413089850944)*x[1]**o+(-0.640198886209)*x[1]
        arg[3,2,0,1]=(0.0855693847128)*x[0]**o+(-0.363881253576)*x[0]+(0.0397854550832)*x[1]**o+(-0.844391996875)*x[1]
        arg[3,2,1,0]=(-0.8248537795)*x[0]**o+(0.153070857979)*x[0]+(0.970009460355)*x[1]**o+(0.431000028243)*x[1]
        arg[3,2,1,1]=(-0.978024209918)*x[0]**o+(-0.527494257814)*x[0]+(-0.814691105217)*x[1]**o+(0.458334868681)*x[1]
        arg[3,2,2,0]=(0.944359157869)*x[0]**o+(-0.868016282191)*x[0]+(0.410849160429)*x[1]**o+(0.480177162607)*x[1]
        arg[3,2,2,1]=(0.109166287134)*x[0]**o+(0.940100154901)*x[0]+(0.630946061805)*x[1]**o+(0.347160020469)*x[1]
        arg[3,3,0,0]=(0.364585502975)*x[0]**o+(0.717282598047)*x[0]+(0.517805310585)*x[1]**o+(0.963452728824)*x[1]
        arg[3,3,0,1]=(0.0373453558524)*x[0]**o+(0.943887402198)*x[0]+(0.983624425071)*x[1]**o+(-0.396242794059)*x[1]
        arg[3,3,1,0]=(-0.563724031351)*x[0]**o+(0.959753988518)*x[0]+(0.118506885299)*x[1]**o+(-0.587748202342)*x[1]
        arg[3,3,1,1]=(0.460419807013)*x[0]**o+(0.237753399499)*x[0]+(0.585589833251)*x[1]**o+(0.935177937667)*x[1]
        arg[3,3,2,0]=(-0.750248086829)*x[0]**o+(-0.561629848994)*x[0]+(-0.919919077536)*x[1]**o+(0.239984179086)*x[1]
        arg[3,3,2,1]=(-0.0969897163659)*x[0]**o+(0.185433022889)*x[0]+(0.733111631837)*x[1]**o+(-0.508147081928)*x[1]
        arg[3,4,0,0]=(0.247054568876)*x[0]**o+(0.721751593201)*x[0]+(-0.684740924498)*x[1]**o+(0.28395202476)*x[1]
        arg[3,4,0,1]=(-0.0949248638736)*x[0]**o+(0.270285273316)*x[0]+(0.850836559464)*x[1]**o+(-0.771085947382)*x[1]
        arg[3,4,1,0]=(-0.965410502362)*x[0]**o+(-0.582448742001)*x[0]+(-0.90307507827)*x[1]**o+(0.914051445745)*x[1]
        arg[3,4,1,1]=(0.443340058617)*x[0]**o+(0.434462120769)*x[0]+(0.522015964301)*x[1]**o+(-0.520184389865)*x[1]
        arg[3,4,2,0]=(-0.961375562593)*x[0]**o+(-0.474630747481)*x[0]+(0.59574636341)*x[1]**o+(0.434190359663)*x[1]
        arg[3,4,2,1]=(0.174154210501)*x[0]**o+(-0.510858596799)*x[0]+(-0.0279700806065)*x[1]**o+(-0.0401612371912)*x[1]
        ref[0,0,0,0]=(-0.0352052809673)/(o+1.)+(0.903863789878)+(0.182956096072)*0.5**o
        ref[0,0,0,1]=(0.0610373691872)/(o+1.)+(-0.306742561603)+(-0.81342189211)*0.5**o
        ref[0,0,1,0]=(0.623943596759)/(o+1.)+(0.647497589315)+(-0.149103531408)*0.5**o
        ref[0,0,1,1]=(0.136600247183)/(o+1.)+(0.925469911903)+(0.74912087771)*0.5**o
        ref[0,0,2,0]=(0.966454801911)/(o+1.)+(0.0631444946098)+(-0.289042694587)*0.5**o
        ref[0,0,2,1]=(-0.156074498516)/(o+1.)+(0.047955363258)+(0.0809737838902)*0.5**o
        ref[0,1,0,0]=(-0.016865109839)/(o+1.)+(-0.404214477389)+(0.638822193145)*0.5**o
        ref[0,1,0,1]=(-0.897448372147)/(o+1.)+(-0.174669987755)+(0.611604473356)*0.5**o
        ref[0,1,1,0]=(-0.229098133178)/(o+1.)+(0.443671911949)+(-0.745407006471)*0.5**o
        ref[0,1,1,1]=(0.481206893907)/(o+1.)+(-0.111741539797)+(0.756528549281)*0.5**o
        ref[0,1,2,0]=(0.367303506039)/(o+1.)+(-0.157755527379)+(0.335544368565)*0.5**o
        ref[0,1,2,1]=(0.182483888026)/(o+1.)+(-0.157073928893)+(0.669933676654)*0.5**o
        ref[0,2,0,0]=(0.587675819052)/(o+1.)+(0.446236706377)+(-0.929163949925)*0.5**o
        ref[0,2,0,1]=(-0.83934259298)/(o+1.)+(-0.344719194607)+(0.639896773528)*0.5**o
        ref[0,2,1,0]=(0.675592435624)/(o+1.)+(0.81769507347)+(-0.399556357179)*0.5**o
        ref[0,2,1,1]=(0.14396936899)/(o+1.)+(-0.454293976313)+(-0.281969675471)*0.5**o
        ref[0,2,2,0]=(0.417557775604)/(o+1.)+(-0.430056060749)+(0.339204154215)*0.5**o
        ref[0,2,2,1]=(-0.0763188160723)/(o+1.)+(0.28785590073)+(-0.655500794874)*0.5**o
        ref[0,3,0,0]=(-0.0533973517656)/(o+1.)+(-0.619586928986)+(-0.483746115201)*0.5**o
        ref[0,3,0,1]=(-0.607238533539)/(o+1.)+(-0.899573610196)+(0.446810992304)*0.5**o
        ref[0,3,1,0]=(0.930589323947)/(o+1.)+(0.127928801455)+(-0.0235907627093)*0.5**o
        ref[0,3,1,1]=(0.237445053125)/(o+1.)+(0.503964752361)+(0.486342062258)*0.5**o
        ref[0,3,2,0]=(0.180505264557)/(o+1.)+(0.629926679618)+(-0.581025309937)*0.5**o
        ref[0,3,2,1]=(0.915270822508)/(o+1.)+(0.0599229206396)+(-0.523356041281)*0.5**o
        ref[0,4,0,0]=(-0.449731522195)/(o+1.)+(-0.0207634106763)+(-0.584782530121)*0.5**o
        ref[0,4,0,1]=(0.0374933582543)/(o+1.)+(0.175584823513)+(-0.0724106581167)*0.5**o
        ref[0,4,1,0]=(0.815864788848)/(o+1.)+(-0.4782741099)+(0.0460360663465)*0.5**o
        ref[0,4,1,1]=(-0.822961904166)/(o+1.)+(0.4384056737)+(-0.978584425712)*0.5**o
        ref[0,4,2,0]=(0.721009228711)/(o+1.)+(0.396658231667)+(-0.267227111945)*0.5**o
        ref[0,4,2,1]=(0.94281427253)/(o+1.)+(-0.888798950857)+(-0.141372777283)*0.5**o
        ref[1,0,0,0]=(0.583083015744)/(o+1.)+(0.41011820901)+(-0.721911758033)*0.5**o
        ref[1,0,0,1]=(-0.656523378462)/(o+1.)+(0.129802155034)+(0.929636966638)*0.5**o
        ref[1,0,1,0]=(-0.562311637909)/(o+1.)+(0.360576537299)+(0.594847872723)*0.5**o
        ref[1,0,1,1]=(-0.265309908189)/(o+1.)+(0.147321654532)+(0.529274819592)*0.5**o
        ref[1,0,2,0]=(-0.0509405210126)/(o+1.)+(0.0288477105933)+(0.2321959905)*0.5**o
        ref[1,0,2,1]=(0.817375877796)/(o+1.)+(-0.00495015411903)+(0.0811824999287)*0.5**o
        ref[1,1,0,0]=(0.15912570069)/(o+1.)+(0.270179064041)+(0.418525096774)*0.5**o
        ref[1,1,0,1]=(-0.299421222048)/(o+1.)+(-0.765918515799)+(0.784138248952)*0.5**o
        ref[1,1,1,0]=(0.935180253062)/(o+1.)+(0.0275127156677)+(-0.911449050332)*0.5**o
        ref[1,1,1,1]=(0.827581865806)/(o+1.)+(0.0374007680845)+(-0.378301315644)*0.5**o
        ref[1,1,2,0]=(-0.580086073626)/(o+1.)+(0.52473567096)+(-0.882543004053)*0.5**o
        ref[1,1,2,1]=(0.320343551712)/(o+1.)+(-0.00386603582212)+(-0.116197074531)*0.5**o
        ref[1,2,0,0]=(0.707766015783)/(o+1.)+(-0.273067983929)+(-0.870347384636)*0.5**o
        ref[1,2,0,1]=(0.825489444837)/(o+1.)+(-0.586560764519)+(-0.219872666014)*0.5**o
        ref[1,2,1,0]=(0.502736178959)/(o+1.)+(0.981161874983)+(-0.963149287055)*0.5**o
        ref[1,2,1,1]=(-0.182719660538)/(o+1.)+(-0.472716904044)+(-0.586350249194)*0.5**o
        ref[1,2,2,0]=(-0.304129806418)/(o+1.)+(-0.192408182425)+(-0.423818475166)*0.5**o
        ref[1,2,2,1]=(0.438755721196)/(o+1.)+(-0.360831125262)+(0.901384945886)*0.5**o
        ref[1,3,0,0]=(0.427243495756)/(o+1.)+(0.534790142987)+(-0.797968602257)*0.5**o
        ref[1,3,0,1]=(-0.943037028688)/(o+1.)+(0.711855627815)+(-0.696126104381)*0.5**o
        ref[1,3,1,0]=(-0.772965740874)/(o+1.)+(-0.689424136389)+(-0.69028368426)*0.5**o
        ref[1,3,1,1]=(-0.250549859157)/(o+1.)+(-0.749654818761)+(-0.534347181294)*0.5**o
        ref[1,3,2,0]=(-0.516514308221)/(o+1.)+(-0.264725096778)+(-0.827981962115)*0.5**o
        ref[1,3,2,1]=(0.806201346091)/(o+1.)+(0.860793480898)+(0.864701869194)*0.5**o
        ref[1,4,0,0]=(-0.288389617381)/(o+1.)+(0.559861504432)+(-0.589044559231)*0.5**o
        ref[1,4,0,1]=(-0.696145763418)/(o+1.)+(0.120898852932)+(0.814971550967)*0.5**o
        ref[1,4,1,0]=(-0.507159320061)/(o+1.)+(-0.520517591688)+(0.760190450826)*0.5**o
        ref[1,4,1,1]=(0.0153574982122)/(o+1.)+(0.656425254312)+(-0.0416603697257)*0.5**o
        ref[1,4,2,0]=(-0.0991159921294)/(o+1.)+(0.551683309807)+(0.105929120872)*0.5**o
        ref[1,4,2,1]=(0.377083955928)/(o+1.)+(0.169719758465)+(-0.503911541225)*0.5**o
        ref[2,0,0,0]=(0.196766730386)/(o+1.)+(-0.472915269429)+(0.953733725947)*0.5**o
        ref[2,0,0,1]=(-0.600347310418)/(o+1.)+(0.335114869806)+(0.491469820597)*0.5**o
        ref[2,0,1,0]=(0.783037278839)/(o+1.)+(0.250103346412)+(0.615118477478)*0.5**o
        ref[2,0,1,1]=(-0.381389710969)/(o+1.)+(0.708103944971)+(-0.806125471265)*0.5**o
        ref[2,0,2,0]=(-0.13737609858)/(o+1.)+(0.411253603929)+(-0.831727679795)*0.5**o
        ref[2,0,2,1]=(0.159758297984)/(o+1.)+(-0.410942771953)+(-0.805843206194)*0.5**o
        ref[2,1,0,0]=(0.325741074892)/(o+1.)+(-0.153818387566)+(0.589135530105)*0.5**o
        ref[2,1,0,1]=(0.0171600098883)/(o+1.)+(0.185369805491)+(0.106049062774)*0.5**o
        ref[2,1,1,0]=(-0.576165158737)/(o+1.)+(-0.0674265758955)+(-0.181511670026)*0.5**o
        ref[2,1,1,1]=(-0.424093531654)/(o+1.)+(0.0229697818029)+(-0.709695453728)*0.5**o
        ref[2,1,2,0]=(0.640141428017)/(o+1.)+(-0.803639398084)+(0.60069280023)*0.5**o
        ref[2,1,2,1]=(0.144061490066)/(o+1.)+(0.677215099246)+(0.896964594562)*0.5**o
        ref[2,2,0,0]=(0.199304002742)/(o+1.)+(0.160270347534)+(0.447633246258)*0.5**o
        ref[2,2,0,1]=(-0.328047392082)/(o+1.)+(0.423339325745)+(-0.683614706554)*0.5**o
        ref[2,2,1,0]=(0.449220001922)/(o+1.)+(0.0252142810523)+(0.22474720984)*0.5**o
        ref[2,2,1,1]=(-0.794933934805)/(o+1.)+(-0.153196600671)+(-0.266419887395)*0.5**o
        ref[2,2,2,0]=(-0.977131144423)/(o+1.)+(0.291140460744)+(0.949852186215)*0.5**o
        ref[2,2,2,1]=(-0.898675367333)/(o+1.)+(-0.501577380592)+(-0.0446180768654)*0.5**o
        ref[2,3,0,0]=(0.276747165681)/(o+1.)+(-0.248012485333)+(0.819774310005)*0.5**o
        ref[2,3,0,1]=(-0.624786978129)/(o+1.)+(0.11731189002)+(-0.391001671385)*0.5**o
        ref[2,3,1,0]=(0.112221476422)/(o+1.)+(-0.27806153609)+(0.904597103646)*0.5**o
        ref[2,3,1,1]=(-0.26229840557)/(o+1.)+(0.295336767397)+(-0.190955697638)*0.5**o
        ref[2,3,2,0]=(-0.151646578512)/(o+1.)+(-0.76989690633)+(-0.873098622934)*0.5**o
        ref[2,3,2,1]=(-0.0946869166491)/(o+1.)+(-0.653653012971)+(-0.712569947477)*0.5**o
        ref[2,4,0,0]=(0.865325438086)/(o+1.)+(-0.318471001396)+(-0.655752680768)*0.5**o
        ref[2,4,0,1]=(-0.127116396219)/(o+1.)+(-0.608381700265)+(0.670566711209)*0.5**o
        ref[2,4,1,0]=(-0.549325647826)/(o+1.)+(0.481306498185)+(0.46903441444)*0.5**o
        ref[2,4,1,1]=(0.900251202513)/(o+1.)+(0.334171273282)+(-0.0099246750652)*0.5**o
        ref[2,4,2,0]=(-0.338211083089)/(o+1.)+(0.812400154986)+(0.218854380066)*0.5**o
        ref[2,4,2,1]=(0.0802890292215)/(o+1.)+(0.0672100820814)+(-0.890762146523)*0.5**o
        ref[3,0,0,0]=(-0.732588048234)/(o+1.)+(0.818675363516)+(0.284622209957)*0.5**o
        ref[3,0,0,1]=(0.288422119968)/(o+1.)+(-0.307215674532)+(0.379628076244)*0.5**o
        ref[3,0,1,0]=(-0.898996153568)/(o+1.)+(-0.195804744928)+(0.0594472092467)*0.5**o
        ref[3,0,1,1]=(0.811628957112)/(o+1.)+(-0.428397939179)+(-0.241252797172)*0.5**o
        ref[3,0,2,0]=(0.745552769828)/(o+1.)+(-0.307729612823)+(0.255617459837)*0.5**o
        ref[3,0,2,1]=(-0.751879287668)/(o+1.)+(0.808981614234)+(0.0725082197254)*0.5**o
        ref[3,1,0,0]=(-0.485010478999)/(o+1.)+(-0.0419208393982)+(-0.444520674459)*0.5**o
        ref[3,1,0,1]=(0.556377882844)/(o+1.)+(0.60833431593)+(0.260061065042)*0.5**o
        ref[3,1,1,0]=(0.729209244377)/(o+1.)+(-0.54883364136)+(0.561755002522)*0.5**o
        ref[3,1,1,1]=(0.066141483426)/(o+1.)+(0.352904622711)+(-0.949556399673)*0.5**o
        ref[3,1,2,0]=(-0.561247934789)/(o+1.)+(-0.471540189382)+(0.833198651881)*0.5**o
        ref[3,1,2,1]=(-0.152588766563)/(o+1.)+(0.0201002749295)+(-0.641727656043)*0.5**o
        ref[3,2,0,0]=(0.413089850944)/(o+1.)+(0.142043725114)+(0.825643539598)*0.5**o
        ref[3,2,0,1]=(0.0397854550832)/(o+1.)+(-0.604136625225)+(0.0855693847128)*0.5**o
        ref[3,2,1,0]=(0.970009460355)/(o+1.)+(0.292035443111)+(-0.8248537795)*0.5**o
        ref[3,2,1,1]=(-0.814691105217)/(o+1.)+(-0.0345796945668)+(-0.978024209918)*0.5**o
        ref[3,2,2,0]=(0.410849160429)/(o+1.)+(-0.193919559792)+(0.944359157869)*0.5**o
        ref[3,2,2,1]=(0.630946061805)/(o+1.)+(0.643630087685)+(0.109166287134)*0.5**o
        ref[3,3,0,0]=(0.517805310585)/(o+1.)+(0.840367663435)+(0.364585502975)*0.5**o
        ref[3,3,0,1]=(0.983624425071)/(o+1.)+(0.273822304069)+(0.0373453558524)*0.5**o
        ref[3,3,1,0]=(0.118506885299)/(o+1.)+(0.186002893088)+(-0.563724031351)*0.5**o
        ref[3,3,1,1]=(0.585589833251)/(o+1.)+(0.586465668583)+(0.460419807013)*0.5**o
        ref[3,3,2,0]=(-0.919919077536)/(o+1.)+(-0.160822834954)+(-0.750248086829)*0.5**o
        ref[3,3,2,1]=(0.733111631837)/(o+1.)+(-0.161357029519)+(-0.0969897163659)*0.5**o
        ref[3,4,0,0]=(-0.684740924498)/(o+1.)+(0.50285180898)+(0.247054568876)*0.5**o
        ref[3,4,0,1]=(0.850836559464)/(o+1.)+(-0.250400337033)+(-0.0949248638736)*0.5**o
        ref[3,4,1,0]=(-0.90307507827)/(o+1.)+(0.165801351872)+(-0.965410502362)*0.5**o
        ref[3,4,1,1]=(0.522015964301)/(o+1.)+(-0.0428611345479)+(0.443340058617)*0.5**o
        ref[3,4,2,0]=(0.59574636341)/(o+1.)+(-0.0202201939092)+(-0.961375562593)*0.5**o
        ref[3,4,2,1]=(-0.0279700806065)/(o+1.)+(-0.275509916995)+(0.174154210501)*0.5**o
      else:
        arg[0,0,0,0]=(-0.793251694335)*x[0]**o+(0.616970271582)*x[0]+(-0.805609243189)*x[1]**o+(0.949515692474)*x[1]+(0.484069738431)*x[2]**o+(-0.0407533911672)*x[2]
        arg[0,0,0,1]=(-0.624206604764)*x[0]**o+(-0.992961148702)*x[0]+(-0.737741763879)*x[1]**o+(0.161544434398)*x[1]+(0.47292947772)*x[2]**o+(0.525538846945)*x[2]
        arg[0,0,1,0]=(0.115999561481)*x[0]**o+(-0.612694096044)*x[0]+(0.625539443386)*x[1]**o+(0.857353715882)*x[1]+(-0.0811256709155)*x[2]**o+(0.183474706394)*x[2]
        arg[0,0,1,1]=(-0.972466831671)*x[0]**o+(0.262980887655)*x[0]+(0.920924576059)*x[1]**o+(0.983517658667)*x[1]+(0.579443973505)*x[2]**o+(0.101139612823)*x[2]
        arg[0,0,2,0]=(0.708180260094)*x[0]**o+(-0.0950913766285)*x[0]+(0.361602191798)*x[1]**o+(-0.880757848201)*x[1]+(0.326272205058)*x[2]**o+(-0.997155694972)*x[2]
        arg[0,0,2,1]=(0.0282615573045)*x[0]**o+(-0.317851994728)*x[0]+(0.449330147388)*x[1]**o+(-0.156946318079)*x[1]+(0.344196894245)*x[2]**o+(-0.045082290904)*x[2]
        arg[0,1,0,0]=(0.231621820235)*x[0]**o+(0.0985804817846)*x[0]+(-0.343623333022)*x[1]**o+(0.115043475441)*x[1]+(-0.910742772023)*x[2]**o+(-0.663592068553)*x[2]
        arg[0,1,0,1]=(0.114056801155)*x[0]**o+(-0.463711315494)*x[0]+(0.380521147429)*x[1]**o+(0.294823684238)*x[1]+(0.510501672453)*x[2]**o+(-0.161234251875)*x[2]
        arg[0,1,1,0]=(-0.973115195017)*x[0]**o+(-0.0133290715609)*x[0]+(-0.642072931588)*x[1]**o+(-0.580655605735)*x[1]+(-0.191371539061)*x[2]**o+(-0.0513051439064)*x[2]
        arg[0,1,1,1]=(-0.304758579369)*x[0]**o+(-0.321904612871)*x[0]+(-0.389525877547)*x[1]**o+(-0.0602638359252)*x[1]+(0.199726168523)*x[2]**o+(-0.10313327614)*x[2]
        arg[0,1,2,0]=(0.0923844967157)*x[0]**o+(-0.369319307184)*x[0]+(-0.614761123101)*x[1]**o+(0.78817430449)*x[1]+(-0.783809995839)*x[2]**o+(-0.701522582246)*x[2]
        arg[0,1,2,1]=(0.793568921835)*x[0]**o+(0.352244892726)*x[0]+(0.185546719529)*x[1]**o+(0.145969792058)*x[1]+(0.85147546253)*x[2]**o+(-0.862069843156)*x[2]
        arg[0,2,0,0]=(0.570673248048)*x[0]**o+(0.607924942767)*x[0]+(-0.353952625913)*x[1]**o+(0.111796842318)*x[1]+(-0.765670542263)*x[2]**o+(0.589532724922)*x[2]
        arg[0,2,0,1]=(0.299061155215)*x[0]**o+(-0.542978954882)*x[0]+(0.0946151623892)*x[1]**o+(0.310266302534)*x[1]+(-0.00880229885823)*x[2]**o+(0.585508874849)*x[2]
        arg[0,2,1,0]=(0.694725244159)*x[0]**o+(0.530004132186)*x[0]+(-0.32588478629)*x[1]**o+(0.357278329148)*x[1]+(-0.93877612203)*x[2]**o+(0.34973013935)*x[2]
        arg[0,2,1,1]=(-0.83896527178)*x[0]**o+(0.487231460025)*x[0]+(0.493952010418)*x[1]**o+(0.247442509692)*x[1]+(-0.312456515075)*x[2]**o+(0.90248200924)*x[2]
        arg[0,2,2,0]=(0.2228052704)*x[0]**o+(0.901373696962)*x[0]+(0.149826948427)*x[1]**o+(0.234013159005)*x[1]+(0.717685535862)*x[2]**o+(-0.42865545543)*x[2]
        arg[0,2,2,1]=(0.63141216243)*x[0]**o+(0.448446314659)*x[0]+(0.379625538779)*x[1]**o+(-0.807448154272)*x[1]+(0.238724835223)*x[2]**o+(-0.478216911196)*x[2]
        arg[0,3,0,0]=(0.00698107166168)*x[0]**o+(-0.727207190747)*x[0]+(-0.78731647832)*x[1]**o+(0.885371753974)*x[1]+(-0.0146821479126)*x[2]**o+(-0.763310617205)*x[2]
        arg[0,3,0,1]=(0.243423742325)*x[0]**o+(0.0587322175837)*x[0]+(-0.453213790216)*x[1]**o+(0.861270481703)*x[1]+(0.867332526083)*x[2]**o+(-0.713187214416)*x[2]
        arg[0,3,1,0]=(-0.106629554393)*x[0]**o+(0.723693136654)*x[0]+(-0.0276604293602)*x[1]**o+(0.339456904686)*x[1]+(-0.102075459215)*x[2]**o+(-0.0348969356596)*x[2]
        arg[0,3,1,1]=(0.375143733111)*x[0]**o+(-0.490781234413)*x[0]+(0.168903339622)*x[1]**o+(0.594080859808)*x[1]+(-0.0802336915361)*x[2]**o+(0.472348979787)*x[2]
        arg[0,3,2,0]=(0.955685455438)*x[0]**o+(0.105142628533)*x[0]+(-0.446384800971)*x[1]**o+(-0.905998305229)*x[1]+(-0.30011637186)*x[2]**o+(0.0218195326037)*x[2]
        arg[0,3,2,1]=(0.353654445481)*x[0]**o+(-0.0477024594709)*x[0]+(-0.495075817497)*x[1]**o+(-0.425091398447)*x[1]+(-0.571207600446)*x[2]**o+(0.177935012163)*x[2]
        arg[0,4,0,0]=(0.974759684151)*x[0]**o+(-0.876975976179)*x[0]+(-0.472336406836)*x[1]**o+(0.991410874408)*x[1]+(0.297823444758)*x[2]**o+(-0.161436760659)*x[2]
        arg[0,4,0,1]=(-0.0799922241539)*x[0]**o+(0.975369691404)*x[0]+(0.302361880712)*x[1]**o+(0.171779641966)*x[1]+(0.878428630413)*x[2]**o+(0.814497247319)*x[2]
        arg[0,4,1,0]=(0.532633280629)*x[0]**o+(0.902436316468)*x[0]+(-0.676910177358)*x[1]**o+(-0.986029752926)*x[1]+(-0.519990783496)*x[2]**o+(0.430468215617)*x[2]
        arg[0,4,1,1]=(-0.353243658293)*x[0]**o+(-0.908938062877)*x[0]+(0.922933758125)*x[1]**o+(0.00500161721147)*x[1]+(0.189861558609)*x[2]**o+(-0.623307494462)*x[2]
        arg[0,4,2,0]=(-0.720840869801)*x[0]**o+(0.0349736577597)*x[0]+(-0.516608060915)*x[1]**o+(0.688756843577)*x[1]+(-0.302494303699)*x[2]**o+(-0.0703783111937)*x[2]
        arg[0,4,2,1]=(-0.179073400214)*x[0]**o+(0.128485402427)*x[0]+(-0.726789308786)*x[1]**o+(-0.691163482057)*x[1]+(0.329009412577)*x[2]**o+(-0.79293394396)*x[2]
        arg[1,0,0,0]=(-0.384011553333)*x[0]**o+(0.285271264922)*x[0]+(0.512105930181)*x[1]**o+(0.190321101872)*x[1]+(-0.509334982029)*x[2]**o+(0.128097742502)*x[2]
        arg[1,0,0,1]=(-0.808211570649)*x[0]**o+(-0.730450956462)*x[0]+(0.92309757237)*x[1]**o+(-0.703749565272)*x[1]+(0.301129716544)*x[2]**o+(0.244379489105)*x[2]
        arg[1,0,1,0]=(0.0404274840294)*x[0]**o+(-0.483903392253)*x[0]+(0.73680959705)*x[1]**o+(0.190850758522)*x[1]+(0.133136432605)*x[2]**o+(0.910055186498)*x[2]
        arg[1,0,1,1]=(0.302394139901)*x[0]**o+(0.802524687002)*x[0]+(0.970871728408)*x[1]**o+(0.207348092291)*x[1]+(0.666310912781)*x[2]**o+(0.510392805767)*x[2]
        arg[1,0,2,0]=(-0.981981997649)*x[0]**o+(-0.465269233707)*x[0]+(0.916146418639)*x[1]**o+(0.143602438774)*x[1]+(-0.947874263512)*x[2]**o+(0.0840394012229)*x[2]
        arg[1,0,2,1]=(-0.237479805943)*x[0]**o+(-0.89993757244)*x[0]+(-0.759650487895)*x[1]**o+(-0.807139164373)*x[1]+(-0.377055204014)*x[2]**o+(-0.57656201026)*x[2]
        arg[1,1,0,0]=(-0.543719871098)*x[0]**o+(-0.726308160246)*x[0]+(0.821244616742)*x[1]**o+(0.718112961487)*x[1]+(-0.655445553192)*x[2]**o+(-0.599104608292)*x[2]
        arg[1,1,0,1]=(-0.2624625471)*x[0]**o+(0.85250071151)*x[0]+(-0.665066081561)*x[1]**o+(-0.566502138196)*x[1]+(-0.572858653628)*x[2]**o+(-0.862890633323)*x[2]
        arg[1,1,1,0]=(0.88003513115)*x[0]**o+(-0.0150959972442)*x[0]+(-0.698880737368)*x[1]**o+(-0.236909810427)*x[1]+(-0.721315930706)*x[2]**o+(-0.219894530829)*x[2]
        arg[1,1,1,1]=(0.177746615088)*x[0]**o+(0.205790814183)*x[0]+(-0.216021190604)*x[1]**o+(0.384079010991)*x[1]+(0.61634008905)*x[2]**o+(0.940900776108)*x[2]
        arg[1,1,2,0]=(0.889070121473)*x[0]**o+(0.68131533527)*x[0]+(0.709207182081)*x[1]**o+(-0.765184991252)*x[1]+(0.506328018952)*x[2]**o+(-0.46041174473)*x[2]
        arg[1,1,2,1]=(0.951736215954)*x[0]**o+(-0.147499702642)*x[0]+(0.60348358384)*x[1]**o+(0.777404433567)*x[1]+(-0.482441452679)*x[2]**o+(0.714835256434)*x[2]
        arg[1,2,0,0]=(-0.954701036263)*x[0]**o+(-0.360901887475)*x[0]+(-0.644285028623)*x[1]**o+(0.169271524632)*x[1]+(-0.687047203668)*x[2]**o+(0.205115312108)*x[2]
        arg[1,2,0,1]=(0.256489282842)*x[0]**o+(0.17406448362)*x[0]+(0.97500243816)*x[1]**o+(0.614577997602)*x[1]+(0.0537536804583)*x[2]**o+(-0.211963274032)*x[2]
        arg[1,2,1,0]=(-0.94015914517)*x[0]**o+(0.169982490736)*x[0]+(0.482085495724)*x[1]**o+(-0.556994404153)*x[1]+(-0.809311296883)*x[2]**o+(0.719907503024)*x[2]
        arg[1,2,1,1]=(0.367156421931)*x[0]**o+(0.489648825153)*x[0]+(-0.0398225517816)*x[1]**o+(-0.495726296032)*x[1]+(0.359374061053)*x[2]**o+(0.231991952482)*x[2]
        arg[1,2,2,0]=(-0.812516694781)*x[0]**o+(-0.631181257391)*x[0]+(-0.723784471687)*x[1]**o+(0.178976877342)*x[1]+(-0.629293758013)*x[2]**o+(-0.313897724608)*x[2]
        arg[1,2,2,1]=(-0.497461944186)*x[0]**o+(0.268998525103)*x[0]+(-0.478017492026)*x[1]**o+(0.026395792841)*x[1]+(0.581256386137)*x[2]**o+(-0.476961644324)*x[2]
        arg[1,3,0,0]=(-0.227077550639)*x[0]**o+(0.400407424161)*x[0]+(-0.53529812303)*x[1]**o+(-0.418594299853)*x[1]+(-0.195190916346)*x[2]**o+(-0.624284483774)*x[2]
        arg[1,3,0,1]=(0.822091021645)*x[0]**o+(-0.31133140026)*x[0]+(0.0665130965049)*x[1]**o+(0.596697216047)*x[1]+(0.466195828646)*x[2]**o+(-0.477988391299)*x[2]
        arg[1,3,1,0]=(-0.0341577310045)*x[0]**o+(-0.77936644392)*x[0]+(0.954811382784)*x[1]**o+(0.837374830631)*x[1]+(0.738992234768)*x[2]**o+(-0.501187788985)*x[2]
        arg[1,3,1,1]=(-0.102867224088)*x[0]**o+(0.00829793194765)*x[0]+(0.081790556957)*x[1]**o+(-0.779351210472)*x[1]+(-0.735321115716)*x[2]**o+(0.126919542418)*x[2]
        arg[1,3,2,0]=(0.556301076488)*x[0]**o+(0.0923434629749)*x[0]+(-0.191048735713)*x[1]**o+(-0.218833028687)*x[1]+(-0.088867516146)*x[2]**o+(-0.247214200389)*x[2]
        arg[1,3,2,1]=(0.888436531887)*x[0]**o+(-0.79225495914)*x[0]+(0.0154788262219)*x[1]**o+(-0.210622890864)*x[1]+(0.782629134632)*x[2]**o+(0.350772185791)*x[2]
        arg[1,4,0,0]=(0.651898073685)*x[0]**o+(-0.84098616689)*x[0]+(0.674088959443)*x[1]**o+(-0.658824952898)*x[1]+(-0.423551541558)*x[2]**o+(0.985098530356)*x[2]
        arg[1,4,0,1]=(-0.149187282052)*x[0]**o+(0.0880370482988)*x[0]+(0.780809276595)*x[1]**o+(-0.610833366472)*x[1]+(0.794946384304)*x[2]**o+(-0.817031796429)*x[2]
        arg[1,4,1,0]=(-0.285938055108)*x[0]**o+(-0.106165154734)*x[0]+(-0.948376845189)*x[1]**o+(0.874607682056)*x[1]+(-0.100051825909)*x[2]**o+(-0.483640835026)*x[2]
        arg[1,4,1,1]=(-0.34015136472)*x[0]**o+(0.991331180352)*x[0]+(-0.925796518928)*x[1]**o+(-0.872775691287)*x[1]+(0.953898056819)*x[2]**o+(0.985025365955)*x[2]
        arg[1,4,2,0]=(-0.464687865356)*x[0]**o+(-0.439271850879)*x[0]+(0.302716649876)*x[1]**o+(0.235186289965)*x[1]+(0.672062761925)*x[2]**o+(-0.793689262392)*x[2]
        arg[1,4,2,1]=(-0.777967458564)*x[0]**o+(0.0565891925223)*x[0]+(-0.27539733623)*x[1]**o+(0.252329868046)*x[1]+(-0.269529792944)*x[2]**o+(-0.953051723483)*x[2]
        arg[2,0,0,0]=(0.665321754689)*x[0]**o+(0.00639681939987)*x[0]+(0.262193891905)*x[1]**o+(-0.312800735151)*x[1]+(0.8403244871)*x[2]**o+(-0.607084328204)*x[2]
        arg[2,0,0,1]=(0.876435018569)*x[0]**o+(-0.388250784543)*x[0]+(-0.799866349878)*x[1]**o+(0.114896594902)*x[1]+(-0.438064877186)*x[2]**o+(-0.458871344685)*x[2]
        arg[2,0,1,0]=(-0.344552466761)*x[0]**o+(0.967217640605)*x[0]+(-0.00183230845245)*x[1]**o+(-0.861206591247)*x[1]+(0.795588257851)*x[2]**o+(-0.333607767932)*x[2]
        arg[2,0,1,1]=(0.841069648591)*x[0]**o+(0.580162460873)*x[0]+(-0.0277035983747)*x[1]**o+(-0.404616536792)*x[1]+(-0.403195438454)*x[2]**o+(-0.202877947947)*x[2]
        arg[2,0,2,0]=(0.192691357169)*x[0]**o+(-0.519968251714)*x[0]+(-0.575397578091)*x[1]**o+(-0.029378989829)*x[1]+(-0.958861195684)*x[2]**o+(0.848705509543)*x[2]
        arg[2,0,2,1]=(0.556616365284)*x[0]**o+(0.679993071506)*x[0]+(-0.800652717274)*x[1]**o+(0.979182847258)*x[1]+(-0.501953582042)*x[2]**o+(0.127003985335)*x[2]
        arg[2,1,0,0]=(0.947750123445)*x[0]**o+(-0.0350691553359)*x[0]+(0.0650485999092)*x[1]**o+(-0.131216537555)*x[1]+(0.888782559831)*x[2]**o+(0.0338703818654)*x[2]
        arg[2,1,0,1]=(0.962918564559)*x[0]**o+(-0.119367421389)*x[0]+(0.43003549616)*x[1]**o+(-0.097883763913)*x[1]+(0.230422206305)*x[2]**o+(0.187273479711)*x[2]
        arg[2,1,1,0]=(-0.549909211973)*x[0]**o+(0.263443098451)*x[0]+(-0.476703606205)*x[1]**o+(0.216159301182)*x[1]+(-0.561829938323)*x[2]**o+(0.943016557203)*x[2]
        arg[2,1,1,1]=(0.382867749635)*x[0]**o+(0.404317006935)*x[0]+(0.786117847428)*x[1]**o+(-0.80719258612)*x[1]+(0.471144889777)*x[2]**o+(0.6478813349)*x[2]
        arg[2,1,2,0]=(0.355742024596)*x[0]**o+(-0.728812391059)*x[0]+(0.766039604734)*x[1]**o+(-0.532629598363)*x[1]+(-0.518284303077)*x[2]**o+(-0.972348067947)*x[2]
        arg[2,1,2,1]=(-0.882334243081)*x[0]**o+(-0.0940633993231)*x[0]+(-0.00166643832426)*x[1]**o+(-0.971243725495)*x[1]+(-0.0422614083707)*x[2]**o+(-0.547881520447)*x[2]
        arg[2,2,0,0]=(0.372164678051)*x[0]**o+(-0.0644361811234)*x[0]+(0.886078780084)*x[1]**o+(-0.459054628604)*x[1]+(-0.983880547097)*x[2]**o+(-0.669749873181)*x[2]
        arg[2,2,0,1]=(0.126650330827)*x[0]**o+(0.375111974063)*x[0]+(-0.759797654917)*x[1]**o+(-0.663216091322)*x[1]+(-0.683191373782)*x[2]**o+(-0.607048449469)*x[2]
        arg[2,2,1,0]=(-0.805920040628)*x[0]**o+(-0.0144685877057)*x[0]+(0.00875222056291)*x[1]**o+(-0.110000267929)*x[1]+(-0.478201664476)*x[2]**o+(-0.0537439708493)*x[2]
        arg[2,2,1,1]=(-0.550624517803)*x[0]**o+(0.570407090431)*x[0]+(0.661026826148)*x[1]**o+(0.905189813242)*x[1]+(0.00707531661352)*x[2]**o+(0.711072343064)*x[2]
        arg[2,2,2,0]=(-0.644276976064)*x[0]**o+(-0.151665972531)*x[0]+(-0.645369927259)*x[1]**o+(0.915967227715)*x[1]+(-0.211750817665)*x[2]**o+(-0.596646212118)*x[2]
        arg[2,2,2,1]=(0.564688999107)*x[0]**o+(0.635094410913)*x[0]+(0.281863287848)*x[1]**o+(-0.927529659408)*x[1]+(0.581071087484)*x[2]**o+(0.164399277305)*x[2]
        arg[2,3,0,0]=(0.64754765253)*x[0]**o+(0.176693707964)*x[0]+(0.172801737347)*x[1]**o+(-0.769408746261)*x[1]+(-0.176026486601)*x[2]**o+(0.187304474822)*x[2]
        arg[2,3,0,1]=(0.365315558093)*x[0]**o+(0.888853264239)*x[0]+(0.0541759434362)*x[1]**o+(0.241489226711)*x[1]+(0.441822201832)*x[2]**o+(0.075541840076)*x[2]
        arg[2,3,1,0]=(-0.984940830356)*x[0]**o+(0.521596714942)*x[0]+(-0.933671223761)*x[1]**o+(-0.132907654205)*x[1]+(-0.225804727932)*x[2]**o+(0.987334445626)*x[2]
        arg[2,3,1,1]=(-0.785516613303)*x[0]**o+(0.0324126413814)*x[0]+(-0.084233004699)*x[1]**o+(0.45131442059)*x[1]+(0.36498759129)*x[2]**o+(0.417036982561)*x[2]
        arg[2,3,2,0]=(-0.310575473358)*x[0]**o+(-0.769219454218)*x[0]+(-0.263431454577)*x[1]**o+(0.474143448264)*x[1]+(-0.490633240671)*x[2]**o+(0.435609988849)*x[2]
        arg[2,3,2,1]=(0.230960030802)*x[0]**o+(-0.480593443732)*x[0]+(0.547888534735)*x[1]**o+(-0.818842709599)*x[1]+(-0.857141410253)*x[2]**o+(-0.287898487847)*x[2]
        arg[2,4,0,0]=(-0.811008715357)*x[0]**o+(-0.608505768579)*x[0]+(-0.117642307757)*x[1]**o+(-0.658229045784)*x[1]+(0.246542958193)*x[2]**o+(-0.42783248265)*x[2]
        arg[2,4,0,1]=(0.817994599948)*x[0]**o+(0.394931907893)*x[0]+(-0.273993145403)*x[1]**o+(0.9572397013)*x[1]+(0.101025833331)*x[2]**o+(-0.925874548613)*x[2]
        arg[2,4,1,0]=(-0.626152000269)*x[0]**o+(0.821322294557)*x[0]+(0.543761841392)*x[1]**o+(0.112341361443)*x[1]+(-0.395657376418)*x[2]**o+(0.815481187683)*x[2]
        arg[2,4,1,1]=(0.97134483493)*x[0]**o+(0.303341541402)*x[0]+(-0.270102237344)*x[1]**o+(-0.982452271352)*x[1]+(0.435568243011)*x[2]**o+(-0.200051496445)*x[2]
        arg[2,4,2,0]=(-0.827528679064)*x[0]**o+(-0.559447627475)*x[0]+(-0.867397899368)*x[1]**o+(0.915539157819)*x[1]+(0.370118757042)*x[2]**o+(0.578366179408)*x[2]
        arg[2,4,2,1]=(0.685882920875)*x[0]**o+(0.830016468585)*x[0]+(-0.603607199824)*x[1]**o+(0.888619939961)*x[1]+(-0.264173896988)*x[2]**o+(-0.588065135532)*x[2]
        arg[3,0,0,0]=(0.480000662344)*x[0]**o+(-0.240072007989)*x[0]+(0.252669294205)*x[1]**o+(-0.657466732916)*x[1]+(0.945149322572)*x[2]**o+(0.340592627149)*x[2]
        arg[3,0,0,1]=(-0.819956304014)*x[0]**o+(0.910408628621)*x[0]+(-0.875714560314)*x[1]**o+(0.653323198674)*x[1]+(0.910127896386)*x[2]**o+(0.21382952806)*x[2]
        arg[3,0,1,0]=(0.736655857249)*x[0]**o+(-0.982636511328)*x[0]+(0.747965385081)*x[1]**o+(-0.952056699676)*x[1]+(0.143168283422)*x[2]**o+(0.891386576579)*x[2]
        arg[3,0,1,1]=(0.615723453617)*x[0]**o+(0.57376968582)*x[0]+(0.766802940979)*x[1]**o+(-0.294849095727)*x[1]+(-0.26718123792)*x[2]**o+(-0.979175148343)*x[2]
        arg[3,0,2,0]=(-0.299975789693)*x[0]**o+(-0.941879795797)*x[0]+(-0.332633604206)*x[1]**o+(0.887589360228)*x[1]+(0.570321438438)*x[2]**o+(0.107180499249)*x[2]
        arg[3,0,2,1]=(-0.77479605873)*x[0]**o+(0.928359807986)*x[0]+(-0.144170026638)*x[1]**o+(0.533835316696)*x[1]+(0.183938136052)*x[2]**o+(-0.87336340483)*x[2]
        arg[3,1,0,0]=(-0.121858163593)*x[0]**o+(-0.971122306623)*x[0]+(-0.351891554168)*x[1]**o+(-0.989754547278)*x[1]+(0.48106592456)*x[2]**o+(0.750009716044)*x[2]
        arg[3,1,0,1]=(-0.332320017719)*x[0]**o+(0.229814969323)*x[0]+(0.950038106614)*x[1]**o+(-0.364221636516)*x[1]+(-0.842684764717)*x[2]**o+(-0.430003876831)*x[2]
        arg[3,1,1,0]=(0.1036461168)*x[0]**o+(0.243047542195)*x[0]+(-0.189881740385)*x[1]**o+(-0.646948186795)*x[1]+(-0.246523431838)*x[2]**o+(0.0363222140239)*x[2]
        arg[3,1,1,1]=(0.629747493968)*x[0]**o+(0.257110968013)*x[0]+(-0.594332699815)*x[1]**o+(-0.74041867705)*x[1]+(0.656583120031)*x[2]**o+(0.392776534802)*x[2]
        arg[3,1,2,0]=(0.997368307137)*x[0]**o+(-0.709004475665)*x[0]+(-0.379447862403)*x[1]**o+(-0.368700793823)*x[1]+(0.293673648471)*x[2]**o+(-0.555545691082)*x[2]
        arg[3,1,2,1]=(-0.323027033928)*x[0]**o+(0.804640557762)*x[0]+(-0.199434780533)*x[1]**o+(0.604194682736)*x[1]+(0.945516011113)*x[2]**o+(-0.831838977373)*x[2]
        arg[3,2,0,0]=(0.255071922458)*x[0]**o+(0.261075290259)*x[0]+(0.198792431961)*x[1]**o+(0.987681568369)*x[1]+(-0.257626079249)*x[2]**o+(0.817801578613)*x[2]
        arg[3,2,0,1]=(-0.732615916833)*x[0]**o+(0.8114347023)*x[0]+(0.308896981216)*x[1]**o+(-0.164981244979)*x[1]+(0.724411853266)*x[2]**o+(-0.647969863039)*x[2]
        arg[3,2,1,0]=(0.126755528121)*x[0]**o+(0.0203968097627)*x[0]+(0.350888385037)*x[1]**o+(0.765584878761)*x[1]+(0.43521215637)*x[2]**o+(0.5821821143)*x[2]
        arg[3,2,1,1]=(0.795956472753)*x[0]**o+(0.363631111688)*x[0]+(-0.296417527594)*x[1]**o+(-0.0927953540926)*x[1]+(0.396041397741)*x[2]**o+(0.43932000066)*x[2]
        arg[3,2,2,0]=(-0.926754341344)*x[0]**o+(-0.688414966522)*x[0]+(-0.660984101932)*x[1]**o+(-0.627883344982)*x[1]+(0.403036784646)*x[2]**o+(0.818277068314)*x[2]
        arg[3,2,2,1]=(-0.358921307079)*x[0]**o+(0.636847962491)*x[0]+(-0.510050460541)*x[1]**o+(-0.758190944228)*x[1]+(-0.0319326931965)*x[2]**o+(-0.643422203077)*x[2]
        arg[3,3,0,0]=(-0.930730482483)*x[0]**o+(-0.883991363505)*x[0]+(0.156751809769)*x[1]**o+(0.669949596594)*x[1]+(0.697840698407)*x[2]**o+(0.721622061073)*x[2]
        arg[3,3,0,1]=(0.108814498453)*x[0]**o+(-0.164024398834)*x[0]+(-0.172084798233)*x[1]**o+(0.031530303081)*x[1]+(-0.985663868487)*x[2]**o+(-0.290224642955)*x[2]
        arg[3,3,1,0]=(0.690575802875)*x[0]**o+(-0.720428203968)*x[0]+(-0.837111899789)*x[1]**o+(0.140256279088)*x[1]+(0.532813462778)*x[2]**o+(-0.557885762558)*x[2]
        arg[3,3,1,1]=(-0.0785388159589)*x[0]**o+(-0.40595445804)*x[0]+(-0.691255332383)*x[1]**o+(0.54108326284)*x[1]+(0.737684932212)*x[2]**o+(0.423508847197)*x[2]
        arg[3,3,2,0]=(-0.589838770269)*x[0]**o+(-0.557133995504)*x[0]+(-0.297626023793)*x[1]**o+(-0.511809000987)*x[1]+(-0.933566600332)*x[2]**o+(-0.21434993446)*x[2]
        arg[3,3,2,1]=(0.19892263441)*x[0]**o+(0.633952342142)*x[0]+(0.0179400729177)*x[1]**o+(0.154020947355)*x[1]+(-0.620690264897)*x[2]**o+(0.524853027077)*x[2]
        arg[3,4,0,0]=(0.475231682303)*x[0]**o+(-0.48174419167)*x[0]+(-0.100854189221)*x[1]**o+(0.818166518054)*x[1]+(-0.922566309657)*x[2]**o+(-0.498745068755)*x[2]
        arg[3,4,0,1]=(0.613982633884)*x[0]**o+(-0.947323459945)*x[0]+(0.17395204013)*x[1]**o+(0.929669087853)*x[1]+(-0.617278089466)*x[2]**o+(0.900537470392)*x[2]
        arg[3,4,1,0]=(-0.403633510324)*x[0]**o+(0.954097943555)*x[0]+(-0.463422043607)*x[1]**o+(0.843916487223)*x[1]+(-0.5955912138)*x[2]**o+(-0.765444657727)*x[2]
        arg[3,4,1,1]=(-0.663412886802)*x[0]**o+(-0.547440885077)*x[0]+(-0.479993570508)*x[1]**o+(0.0620303034445)*x[1]+(-0.653759917895)*x[2]**o+(-0.681004441743)*x[2]
        arg[3,4,2,0]=(-0.994529375243)*x[0]**o+(0.546983479745)*x[0]+(0.17913912435)*x[1]**o+(0.660366383627)*x[1]+(0.0747943517387)*x[2]**o+(-0.321063660452)*x[2]
        arg[3,4,2,1]=(-0.538074654323)*x[0]**o+(-0.81330034965)*x[0]+(0.660614896398)*x[1]**o+(-0.918192909648)*x[1]+(0.553635166581)*x[2]**o+(-0.918131802189)*x[2]
        ref[0,0,0,0]=(-0.321539504757)/(o+1.)+(0.762866286444)+(-0.793251694335)*0.5**o
        ref[0,0,0,1]=(-0.264812286159)/(o+1.)+(-0.15293893368)+(-0.624206604764)*0.5**o
        ref[0,0,1,0]=(0.54441377247)/(o+1.)+(0.214067163116)+(0.115999561481)*0.5**o
        ref[0,0,1,1]=(1.50036854956)/(o+1.)+(0.673819079573)+(-0.972466831671)*0.5**o
        ref[0,0,2,0]=(0.687874396856)/(o+1.)+(-0.986502459901)+(0.708180260094)*0.5**o
        ref[0,0,2,1]=(0.793527041633)/(o+1.)+(-0.259940301856)+(0.0282615573045)*0.5**o
        ref[0,1,0,0]=(-1.25436610505)/(o+1.)+(-0.224984055664)+(0.231621820235)*0.5**o
        ref[0,1,0,1]=(0.891022819882)/(o+1.)+(-0.165060941565)+(0.114056801155)*0.5**o
        ref[0,1,1,0]=(-0.833444470649)/(o+1.)+(-0.322644910601)+(-0.973115195017)*0.5**o
        ref[0,1,1,1]=(-0.189799709023)/(o+1.)+(-0.242650862468)+(-0.304758579369)*0.5**o
        ref[0,1,2,0]=(-1.39857111894)/(o+1.)+(-0.14133379247)+(0.0923844967157)*0.5**o
        ref[0,1,2,1]=(1.03702218206)/(o+1.)+(-0.181927579186)+(0.793568921835)*0.5**o
        ref[0,2,0,0]=(-1.11962316818)/(o+1.)+(0.654627255004)+(0.570673248048)*0.5**o
        ref[0,2,0,1]=(0.085812863531)/(o+1.)+(0.17639811125)+(0.299061155215)*0.5**o
        ref[0,2,1,0]=(-1.26466090832)/(o+1.)+(0.618506300342)+(0.694725244159)*0.5**o
        ref[0,2,1,1]=(0.181495495342)/(o+1.)+(0.818577989479)+(-0.83896527178)*0.5**o
        ref[0,2,2,0]=(0.867512484289)/(o+1.)+(0.353365700269)+(0.2228052704)*0.5**o
        ref[0,2,2,1]=(0.618350374002)/(o+1.)+(-0.418609375405)+(0.63141216243)*0.5**o
        ref[0,3,0,0]=(-0.801998626232)/(o+1.)+(-0.302573026989)+(0.00698107166168)*0.5**o
        ref[0,3,0,1]=(0.414118735868)/(o+1.)+(0.103407742435)+(0.243423742325)*0.5**o
        ref[0,3,1,0]=(-0.129735888575)/(o+1.)+(0.51412655284)+(-0.106629554393)*0.5**o
        ref[0,3,1,1]=(0.0886696480864)/(o+1.)+(0.287824302591)+(0.375143733111)*0.5**o
        ref[0,3,2,0]=(-0.746501172831)/(o+1.)+(-0.389518072046)+(0.955685455438)*0.5**o
        ref[0,3,2,1]=(-1.06628341794)/(o+1.)+(-0.147429422878)+(0.353654445481)*0.5**o
        ref[0,4,0,0]=(-0.174512962078)/(o+1.)+(-0.023500931215)+(0.974759684151)*0.5**o
        ref[0,4,0,1]=(1.18079051113)/(o+1.)+(0.980823290345)+(-0.0799922241539)*0.5**o
        ref[0,4,1,0]=(-1.19690096085)/(o+1.)+(0.173437389579)+(0.532633280629)*0.5**o
        ref[0,4,1,1]=(1.11279531673)/(o+1.)+(-0.763621970064)+(-0.353243658293)*0.5**o
        ref[0,4,2,0]=(-0.819102364613)/(o+1.)+(0.326676095071)+(-0.720840869801)*0.5**o
        ref[0,4,2,1]=(-0.397779896209)/(o+1.)+(-0.677806011795)+(-0.179073400214)*0.5**o
        ref[1,0,0,0]=(0.00277094815158)/(o+1.)+(0.301845054648)+(-0.384011553333)*0.5**o
        ref[1,0,0,1]=(1.22422728891)/(o+1.)+(-0.594910516315)+(-0.808211570649)*0.5**o
        ref[1,0,1,0]=(0.869946029655)/(o+1.)+(0.308501276384)+(0.0404274840294)*0.5**o
        ref[1,0,1,1]=(1.63718264119)/(o+1.)+(0.76013279253)+(0.302394139901)*0.5**o
        ref[1,0,2,0]=(-0.0317278448724)/(o+1.)+(-0.118813696855)+(-0.981981997649)*0.5**o
        ref[1,0,2,1]=(-1.13670569191)/(o+1.)+(-1.14181937354)+(-0.237479805943)*0.5**o
        ref[1,1,0,0]=(0.16579906355)/(o+1.)+(-0.303649903525)+(-0.543719871098)*0.5**o
        ref[1,1,0,1]=(-1.23792473519)/(o+1.)+(-0.288446030005)+(-0.2624625471)*0.5**o
        ref[1,1,1,0]=(-1.42019666807)/(o+1.)+(-0.23595016925)+(0.88003513115)*0.5**o
        ref[1,1,1,1]=(0.400318898446)/(o+1.)+(0.765385300641)+(0.177746615088)*0.5**o
        ref[1,1,2,0]=(1.21553520103)/(o+1.)+(-0.272140700356)+(0.889070121473)*0.5**o
        ref[1,1,2,1]=(0.121042131161)/(o+1.)+(0.672369993679)+(0.951736215954)*0.5**o
        ref[1,2,0,0]=(-1.33133223229)/(o+1.)+(0.0067424746325)+(-0.954701036263)*0.5**o
        ref[1,2,0,1]=(1.02875611862)/(o+1.)+(0.288339603595)+(0.256489282842)*0.5**o
        ref[1,2,1,0]=(-0.32722580116)/(o+1.)+(0.166447794803)+(-0.94015914517)*0.5**o
        ref[1,2,1,1]=(0.319551509271)/(o+1.)+(0.112957240802)+(0.367156421931)*0.5**o
        ref[1,2,2,0]=(-1.3530782297)/(o+1.)+(-0.383051052329)+(-0.812516694781)*0.5**o
        ref[1,2,2,1]=(0.103238894111)/(o+1.)+(-0.09078366319)+(-0.497461944186)*0.5**o
        ref[1,3,0,0]=(-0.730489039376)/(o+1.)+(-0.321235679733)+(-0.227077550639)*0.5**o
        ref[1,3,0,1]=(0.532708925151)/(o+1.)+(-0.0963112877562)+(0.822091021645)*0.5**o
        ref[1,3,1,0]=(1.69380361755)/(o+1.)+(-0.221589701137)+(-0.0341577310045)*0.5**o
        ref[1,3,1,1]=(-0.653530558759)/(o+1.)+(-0.322066868053)+(-0.102867224088)*0.5**o
        ref[1,3,2,0]=(-0.279916251859)/(o+1.)+(-0.18685188305)+(0.556301076488)*0.5**o
        ref[1,3,2,1]=(0.798107960854)/(o+1.)+(-0.326052832107)+(0.888436531887)*0.5**o
        ref[1,4,0,0]=(0.250537417885)/(o+1.)+(-0.257356294716)+(0.651898073685)*0.5**o
        ref[1,4,0,1]=(1.5757556609)/(o+1.)+(-0.669914057301)+(-0.149187282052)*0.5**o
        ref[1,4,1,0]=(-1.0484286711)/(o+1.)+(0.142400846148)+(-0.285938055108)*0.5**o
        ref[1,4,1,1]=(0.028101537891)/(o+1.)+(0.55179042751)+(-0.34015136472)*0.5**o
        ref[1,4,2,0]=(0.974779411801)/(o+1.)+(-0.498887411653)+(-0.464687865356)*0.5**o
        ref[1,4,2,1]=(-0.544927129174)/(o+1.)+(-0.322066331457)+(-0.777967458564)*0.5**o
        ref[2,0,0,0]=(1.102518379)/(o+1.)+(-0.456744121978)+(0.665321754689)*0.5**o
        ref[2,0,0,1]=(-1.23793122706)/(o+1.)+(-0.366112767163)+(0.876435018569)*0.5**o
        ref[2,0,1,0]=(0.793755949398)/(o+1.)+(-0.113798359287)+(-0.344552466761)*0.5**o
        ref[2,0,1,1]=(-0.430899036829)/(o+1.)+(-0.0136660119329)+(0.841069648591)*0.5**o
        ref[2,0,2,0]=(-1.53425877377)/(o+1.)+(0.149679134)+(0.192691357169)*0.5**o
        ref[2,0,2,1]=(-1.30260629932)/(o+1.)+(0.89308995205)+(0.556616365284)*0.5**o
        ref[2,1,0,0]=(0.95383115974)/(o+1.)+(-0.0662076555126)+(0.947750123445)*0.5**o
        ref[2,1,0,1]=(0.660457702465)/(o+1.)+(-0.0149888527952)+(0.962918564559)*0.5**o
        ref[2,1,1,0]=(-1.03853354453)/(o+1.)+(0.711309478418)+(-0.549909211973)*0.5**o
        ref[2,1,1,1]=(1.25726273721)/(o+1.)+(0.122502877857)+(0.382867749635)*0.5**o
        ref[2,1,2,0]=(0.247755301657)/(o+1.)+(-1.11689502868)+(0.355742024596)*0.5**o
        ref[2,1,2,1]=(-0.043927846695)/(o+1.)+(-0.806594322632)+(-0.882334243081)*0.5**o
        ref[2,2,0,0]=(-0.0978017670124)/(o+1.)+(-0.596620341454)+(0.372164678051)*0.5**o
        ref[2,2,0,1]=(-1.4429890287)/(o+1.)+(-0.447576283364)+(0.126650330827)*0.5**o
        ref[2,2,1,0]=(-0.469449443913)/(o+1.)+(-0.0891064132419)+(-0.805920040628)*0.5**o
        ref[2,2,1,1]=(0.668102142762)/(o+1.)+(1.09333462337)+(-0.550624517803)*0.5**o
        ref[2,2,2,0]=(-0.857120744924)/(o+1.)+(0.0838275215328)+(-0.644276976064)*0.5**o
        ref[2,2,2,1]=(0.862934375332)/(o+1.)+(-0.0640179855954)+(0.564688999107)*0.5**o
        ref[2,3,0,0]=(-0.00322474925369)/(o+1.)+(-0.202705281737)+(0.64754765253)*0.5**o
        ref[2,3,0,1]=(0.495998145268)/(o+1.)+(0.602942165513)+(0.365315558093)*0.5**o
        ref[2,3,1,0]=(-1.15947595169)/(o+1.)+(0.688011753181)+(-0.984940830356)*0.5**o
        ref[2,3,1,1]=(0.280754586591)/(o+1.)+(0.450382022266)+(-0.785516613303)*0.5**o
        ref[2,3,2,0]=(-0.754064695248)/(o+1.)+(0.0702669914474)+(-0.310575473358)*0.5**o
        ref[2,3,2,1]=(-0.309252875518)/(o+1.)+(-0.793667320589)+(0.230960030802)*0.5**o
        ref[2,4,0,0]=(0.128900650436)/(o+1.)+(-0.847283648506)+(-0.811008715357)*0.5**o
        ref[2,4,0,1]=(-0.172967312072)/(o+1.)+(0.21314853029)+(0.817994599948)*0.5**o
        ref[2,4,1,0]=(0.148104464974)/(o+1.)+(0.874572421841)+(-0.626152000269)*0.5**o
        ref[2,4,1,1]=(0.165466005667)/(o+1.)+(-0.439581113198)+(0.97134483493)*0.5**o
        ref[2,4,2,0]=(-0.497279142326)/(o+1.)+(0.467228854876)+(-0.827528679064)*0.5**o
        ref[2,4,2,1]=(-0.867781096812)/(o+1.)+(0.565285636507)+(0.685882920875)*0.5**o
        ref[3,0,0,0]=(1.19781861678)/(o+1.)+(-0.278473056878)+(0.480000662344)*0.5**o
        ref[3,0,0,1]=(0.0344133360711)/(o+1.)+(0.888780677678)+(-0.819956304014)*0.5**o
        ref[3,0,1,0]=(0.891133668503)/(o+1.)+(-0.521653317213)+(0.736655857249)*0.5**o
        ref[3,0,1,1]=(0.499621703059)/(o+1.)+(-0.350127279125)+(0.615723453617)*0.5**o
        ref[3,0,2,0]=(0.237687834232)/(o+1.)+(0.0264450318399)+(-0.299975789693)*0.5**o
        ref[3,0,2,1]=(0.0397681094138)/(o+1.)+(0.294415859926)+(-0.77479605873)*0.5**o
        ref[3,1,0,0]=(0.129174370391)/(o+1.)+(-0.605433568929)+(-0.121858163593)*0.5**o
        ref[3,1,0,1]=(0.107353341897)/(o+1.)+(-0.282205272012)+(-0.332320017719)*0.5**o
        ref[3,1,1,0]=(-0.436405172223)/(o+1.)+(-0.183789215288)+(0.1036461168)*0.5**o
        ref[3,1,1,1]=(0.0622504202157)/(o+1.)+(-0.0452655871179)+(0.629747493968)*0.5**o
        ref[3,1,2,0]=(-0.0857742139322)/(o+1.)+(-0.816625480285)+(0.997368307137)*0.5**o
        ref[3,1,2,1]=(0.746081230581)/(o+1.)+(0.288498131562)+(-0.323027033928)*0.5**o
        ref[3,2,0,0]=(-0.0588336472879)/(o+1.)+(1.03327921862)+(0.255071922458)*0.5**o
        ref[3,2,0,1]=(1.03330883448)/(o+1.)+(-0.000758202858916)+(-0.732615916833)*0.5**o
        ref[3,2,1,0]=(0.786100541406)/(o+1.)+(0.684081901412)+(0.126755528121)*0.5**o
        ref[3,2,1,1]=(0.099623870147)/(o+1.)+(0.355077879128)+(0.795956472753)*0.5**o
        ref[3,2,2,0]=(-0.257947317287)/(o+1.)+(-0.249010621595)+(-0.926754341344)*0.5**o
        ref[3,2,2,1]=(-0.541983153738)/(o+1.)+(-0.382382592407)+(-0.358921307079)*0.5**o
        ref[3,3,0,0]=(0.854592508176)/(o+1.)+(0.253790147081)+(-0.930730482483)*0.5**o
        ref[3,3,0,1]=(-1.15774866672)/(o+1.)+(-0.211359369354)+(0.108814498453)*0.5**o
        ref[3,3,1,0]=(-0.304298437011)/(o+1.)+(-0.569028843719)+(0.690575802875)*0.5**o
        ref[3,3,1,1]=(0.0464295998283)/(o+1.)+(0.279318825998)+(-0.0785388159589)*0.5**o
        ref[3,3,2,0]=(-1.23119262412)/(o+1.)+(-0.641646465476)+(-0.589838770269)*0.5**o
        ref[3,3,2,1]=(-0.60275019198)/(o+1.)+(0.656413158287)+(0.19892263441)*0.5**o
        ref[3,4,0,0]=(-1.02342049888)/(o+1.)+(-0.0811613711857)+(0.475231682303)*0.5**o
        ref[3,4,0,1]=(-0.443326049337)/(o+1.)+(0.44144154915)+(0.613982633884)*0.5**o
        ref[3,4,1,0]=(-1.05901325741)/(o+1.)+(0.516284886526)+(-0.403633510324)*0.5**o
        ref[3,4,1,1]=(-1.1337534884)/(o+1.)+(-0.583207511688)+(-0.663412886802)*0.5**o
        ref[3,4,2,0]=(0.253933476088)/(o+1.)+(0.44314310146)+(-0.994529375243)*0.5**o
        ref[3,4,2,1]=(1.21425006298)/(o+1.)+(-1.32481253074)+(-0.538074654323)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactOne_fromData_Solution_rank0(self):
      """
      tests integral of rank 0 Data on the ReducedFunctionOnContactOne

      assumptions: Solution(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      w=Solution(self.domain)
      x=w.getX()
      arg=Data(0,(),w)
      if dim==2:
        arg=(-0.441098788712)*x[0]**o+(0.082953252724)*x[0]+(-0.97005044508)*x[1]**o+(0.928239302707)*x[1]
        ref=(-0.97005044508)/(o+1.)+(0.505596277715)+(-0.441098788712)*0.5**o
      else:
        arg=(0.384995424392)*x[0]**o+(0.999298118235)*x[0]+(0.144745460948)*x[1]**o+(0.0545198589076)*x[1]+(-0.773375797864)*x[2]**o+(0.145524911495)*x[2]
        ref=(-0.628630336917)/(o+1.)+(0.599671444318)+(0.384995424392)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactOne_fromData_Solution_rank1(self):
      """
      tests integral of rank 1 Data on the ReducedFunctionOnContactOne

      assumptions: Solution(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      w=Solution(self.domain)
      x=w.getX()
      arg=Data(0,(2,),w)
      ref=numpy.zeros((2,),numpy.float_)
      if dim==2:
        arg[0]=(0.0745889108767)*x[0]**o+(-0.694812761318)*x[0]+(-0.357791726239)*x[1]**o+(-0.785448350489)*x[1]
        arg[1]=(0.436006633056)*x[0]**o+(-0.797943632034)*x[0]+(-0.903321835716)*x[1]**o+(0.299527545727)*x[1]
        ref[0]=(-0.357791726239)/(o+1.)+(-0.740130555904)+(0.0745889108767)*0.5**o
        ref[1]=(-0.903321835716)/(o+1.)+(-0.249208043154)+(0.436006633056)*0.5**o
      else:
        arg[0]=(-0.292048194246)*x[0]**o+(-0.612402340241)*x[0]+(0.830253896674)*x[1]**o+(-0.0173245051245)*x[1]+(-0.105062444571)*x[2]**o+(0.968232377167)*x[2]
        arg[1]=(0.153118126553)*x[0]**o+(0.958628221601)*x[0]+(-0.326368437133)*x[1]**o+(0.878861879269)*x[1]+(-0.297398071379)*x[2]**o+(0.155202299471)*x[2]
        ref[0]=(0.725191452103)/(o+1.)+(0.169252765901)+(-0.292048194246)*0.5**o
        ref[1]=(-0.623766508512)/(o+1.)+(0.99634620017)+(0.153118126553)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactOne_fromData_Solution_rank2(self):
      """
      tests integral of rank 2 Data on the ReducedFunctionOnContactOne

      assumptions: Solution(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      w=Solution(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5),w)
      ref=numpy.zeros((4, 5),numpy.float_)
      if dim==2:
        arg[0,0]=(0.410346849722)*x[0]**o+(0.86102012198)*x[0]+(-0.0143604505833)*x[1]**o+(-0.333268586255)*x[1]
        arg[0,1]=(-0.104991947387)*x[0]**o+(0.51540973662)*x[0]+(0.449773354197)*x[1]**o+(0.0397236625379)*x[1]
        arg[0,2]=(0.0010671797269)*x[0]**o+(0.165202831822)*x[0]+(-0.536444395178)*x[1]**o+(0.142112903729)*x[1]
        arg[0,3]=(0.0655247081801)*x[0]**o+(-0.59093644572)*x[0]+(-0.0135713515051)*x[1]**o+(0.665578431202)*x[1]
        arg[0,4]=(0.902382714765)*x[0]**o+(-0.358716290301)*x[0]+(-0.504792876433)*x[1]**o+(-0.760982991404)*x[1]
        arg[1,0]=(0.663815334483)*x[0]**o+(-0.967978720711)*x[0]+(-0.308953852146)*x[1]**o+(-0.250315651366)*x[1]
        arg[1,1]=(0.602976841368)*x[0]**o+(0.407346455361)*x[0]+(0.820076160142)*x[1]**o+(0.720380314572)*x[1]
        arg[1,2]=(0.0899286747557)*x[0]**o+(-0.636748133402)*x[0]+(0.615853620374)*x[1]**o+(0.389136375831)*x[1]
        arg[1,3]=(-0.695859848989)*x[0]**o+(-0.160892907203)*x[0]+(0.972252211687)*x[1]**o+(0.971653054313)*x[1]
        arg[1,4]=(-0.229141368314)*x[0]**o+(0.434757135281)*x[0]+(0.508751791725)*x[1]**o+(-0.593022987916)*x[1]
        arg[2,0]=(-0.732297666821)*x[0]**o+(-0.885623150041)*x[0]+(0.355264312146)*x[1]**o+(-0.203582600498)*x[1]
        arg[2,1]=(-0.0683706126862)*x[0]**o+(0.390953181924)*x[0]+(-0.104115827873)*x[1]**o+(-0.0586349400277)*x[1]
        arg[2,2]=(0.213052201915)*x[0]**o+(0.813758731408)*x[0]+(-0.927085393446)*x[1]**o+(-0.70884993627)*x[1]
        arg[2,3]=(-0.11792311156)*x[0]**o+(0.705053056049)*x[0]+(-0.988858028527)*x[1]**o+(-0.532493233644)*x[1]
        arg[2,4]=(0.643258167584)*x[0]**o+(0.991059864292)*x[0]+(0.466734669253)*x[1]**o+(-0.763209421816)*x[1]
        arg[3,0]=(-0.686036752646)*x[0]**o+(-0.0152278415398)*x[0]+(0.572818298467)*x[1]**o+(0.0883817883272)*x[1]
        arg[3,1]=(-0.498088836117)*x[0]**o+(0.413958831603)*x[0]+(-0.788338642889)*x[1]**o+(-0.219412111863)*x[1]
        arg[3,2]=(0.895103303091)*x[0]**o+(0.967224972282)*x[0]+(0.778129303031)*x[1]**o+(-0.239067231978)*x[1]
        arg[3,3]=(-0.715385920659)*x[0]**o+(-0.928446013585)*x[0]+(-0.919578735328)*x[1]**o+(0.735438220506)*x[1]
        arg[3,4]=(0.273825880641)*x[0]**o+(0.602304312788)*x[0]+(-0.788415866411)*x[1]**o+(0.841776933557)*x[1]
        ref[0,0]=(-0.0143604505833)/(o+1.)+(0.263875767862)+(0.410346849722)*0.5**o
        ref[0,1]=(0.449773354197)/(o+1.)+(0.277566699579)+(-0.104991947387)*0.5**o
        ref[0,2]=(-0.536444395178)/(o+1.)+(0.153657867776)+(0.0010671797269)*0.5**o
        ref[0,3]=(-0.0135713515051)/(o+1.)+(0.0373209927411)+(0.0655247081801)*0.5**o
        ref[0,4]=(-0.504792876433)/(o+1.)+(-0.559849640852)+(0.902382714765)*0.5**o
        ref[1,0]=(-0.308953852146)/(o+1.)+(-0.609147186038)+(0.663815334483)*0.5**o
        ref[1,1]=(0.820076160142)/(o+1.)+(0.563863384966)+(0.602976841368)*0.5**o
        ref[1,2]=(0.615853620374)/(o+1.)+(-0.123805878786)+(0.0899286747557)*0.5**o
        ref[1,3]=(0.972252211687)/(o+1.)+(0.405380073555)+(-0.695859848989)*0.5**o
        ref[1,4]=(0.508751791725)/(o+1.)+(-0.0791329263176)+(-0.229141368314)*0.5**o
        ref[2,0]=(0.355264312146)/(o+1.)+(-0.544602875269)+(-0.732297666821)*0.5**o
        ref[2,1]=(-0.104115827873)/(o+1.)+(0.166159120948)+(-0.0683706126862)*0.5**o
        ref[2,2]=(-0.927085393446)/(o+1.)+(0.0524543975689)+(0.213052201915)*0.5**o
        ref[2,3]=(-0.988858028527)/(o+1.)+(0.0862799112023)+(-0.11792311156)*0.5**o
        ref[2,4]=(0.466734669253)/(o+1.)+(0.113925221238)+(0.643258167584)*0.5**o
        ref[3,0]=(0.572818298467)/(o+1.)+(0.0365769733937)+(-0.686036752646)*0.5**o
        ref[3,1]=(-0.788338642889)/(o+1.)+(0.0972733598697)+(-0.498088836117)*0.5**o
        ref[3,2]=(0.778129303031)/(o+1.)+(0.364078870152)+(0.895103303091)*0.5**o
        ref[3,3]=(-0.919578735328)/(o+1.)+(-0.0965038965398)+(-0.715385920659)*0.5**o
        ref[3,4]=(-0.788415866411)/(o+1.)+(0.722040623173)+(0.273825880641)*0.5**o
      else:
        arg[0,0]=(-0.903197979191)*x[0]**o+(-0.497252911882)*x[0]+(0.5689188153)*x[1]**o+(-0.569434759407)*x[1]+(-0.204533481399)*x[2]**o+(-0.0255207088467)*x[2]
        arg[0,1]=(-0.0296606374337)*x[0]**o+(0.917552577919)*x[0]+(0.746132694062)*x[1]**o+(0.285568252958)*x[1]+(0.175571603104)*x[2]**o+(0.5701931124)*x[2]
        arg[0,2]=(0.657981464039)*x[0]**o+(-0.105816675859)*x[0]+(-0.210726712386)*x[1]**o+(-0.457303699577)*x[1]+(-0.225517453595)*x[2]**o+(0.61593257413)*x[2]
        arg[0,3]=(-0.652766009035)*x[0]**o+(0.538908603782)*x[0]+(0.226594150058)*x[1]**o+(0.691428293374)*x[1]+(-0.210896072508)*x[2]**o+(0.538061905948)*x[2]
        arg[0,4]=(0.175024193446)*x[0]**o+(0.433285241007)*x[0]+(0.928411521986)*x[1]**o+(-0.241175862661)*x[1]+(-0.779192552542)*x[2]**o+(0.428100531631)*x[2]
        arg[1,0]=(-0.094230207815)*x[0]**o+(-0.113255272088)*x[0]+(-0.880816596679)*x[1]**o+(0.56261116141)*x[1]+(-0.418606336511)*x[2]**o+(0.858058623759)*x[2]
        arg[1,1]=(0.08556853498)*x[0]**o+(0.95491440456)*x[0]+(-0.435275492231)*x[1]**o+(0.339105115812)*x[1]+(-0.634101556975)*x[2]**o+(0.357557194572)*x[2]
        arg[1,2]=(-0.692883812524)*x[0]**o+(0.673540357384)*x[0]+(0.240219465837)*x[1]**o+(0.139957925862)*x[1]+(0.135892430596)*x[2]**o+(0.383465818668)*x[2]
        arg[1,3]=(-0.892921935206)*x[0]**o+(0.0694647682287)*x[0]+(0.0530888770821)*x[1]**o+(-0.0735398003251)*x[1]+(-0.40702392212)*x[2]**o+(-0.223494881098)*x[2]
        arg[1,4]=(0.743112585337)*x[0]**o+(-0.12393012942)*x[0]+(-0.660616664197)*x[1]**o+(-0.0334489536139)*x[1]+(-0.0566394013558)*x[2]**o+(-0.0281273872158)*x[2]
        arg[2,0]=(-0.500500871517)*x[0]**o+(-0.326237506809)*x[0]+(-0.960480146386)*x[1]**o+(0.0901654083154)*x[1]+(0.466124845857)*x[2]**o+(0.6908848619)*x[2]
        arg[2,1]=(0.120233846989)*x[0]**o+(0.520609850629)*x[0]+(-0.407423400749)*x[1]**o+(0.59145151128)*x[1]+(0.981171407459)*x[2]**o+(-0.703987165108)*x[2]
        arg[2,2]=(0.811775583161)*x[0]**o+(-0.209650983138)*x[0]+(-0.286223486261)*x[1]**o+(-0.864122954179)*x[1]+(0.609725467363)*x[2]**o+(0.0311464898903)*x[2]
        arg[2,3]=(-0.325647787579)*x[0]**o+(0.956001955119)*x[0]+(0.637272546129)*x[1]**o+(-0.951109798103)*x[1]+(0.406445558466)*x[2]**o+(-0.930854597943)*x[2]
        arg[2,4]=(0.0621315789778)*x[0]**o+(-0.917033227802)*x[0]+(-0.179584904974)*x[1]**o+(0.283357518809)*x[1]+(-0.778863909832)*x[2]**o+(-0.524497331838)*x[2]
        arg[3,0]=(0.140605486894)*x[0]**o+(0.172882849759)*x[0]+(-0.883586756712)*x[1]**o+(-0.160461884399)*x[1]+(-0.718343713337)*x[2]**o+(0.196937721256)*x[2]
        arg[3,1]=(-0.660232764173)*x[0]**o+(-0.92091042631)*x[0]+(-0.568852134661)*x[1]**o+(-0.331441330509)*x[1]+(-0.220286922375)*x[2]**o+(-0.256407439105)*x[2]
        arg[3,2]=(0.593940468197)*x[0]**o+(0.527756437117)*x[0]+(0.943980933807)*x[1]**o+(0.749415167795)*x[1]+(0.484043332105)*x[2]**o+(0.40135030381)*x[2]
        arg[3,3]=(0.870508483705)*x[0]**o+(-0.234209640456)*x[0]+(-0.62692310882)*x[1]**o+(0.352628981854)*x[1]+(-0.24937844453)*x[2]**o+(0.73438735864)*x[2]
        arg[3,4]=(-0.638437449353)*x[0]**o+(0.308007229542)*x[0]+(0.537590844528)*x[1]**o+(-0.755595388361)*x[1]+(-0.152781228745)*x[2]**o+(-0.450122095362)*x[2]
        ref[0,0]=(0.364385333901)/(o+1.)+(-0.546104190068)+(-0.903197979191)*0.5**o
        ref[0,1]=(0.921704297167)/(o+1.)+(0.886656971638)+(-0.0296606374337)*0.5**o
        ref[0,2]=(-0.436244165981)/(o+1.)+(0.0264060993469)+(0.657981464039)*0.5**o
        ref[0,3]=(0.0156980775503)/(o+1.)+(0.884199401552)+(-0.652766009035)*0.5**o
        ref[0,4]=(0.149218969444)/(o+1.)+(0.310104954989)+(0.175024193446)*0.5**o
        ref[1,0]=(-1.29942293319)/(o+1.)+(0.653707256541)+(-0.094230207815)*0.5**o
        ref[1,1]=(-1.06937704921)/(o+1.)+(0.825788357473)+(0.08556853498)*0.5**o
        ref[1,2]=(0.376111896433)/(o+1.)+(0.598482050957)+(-0.692883812524)*0.5**o
        ref[1,3]=(-0.353935045038)/(o+1.)+(-0.113784956597)+(-0.892921935206)*0.5**o
        ref[1,4]=(-0.717256065552)/(o+1.)+(-0.0927532351247)+(0.743112585337)*0.5**o
        ref[2,0]=(-0.494355300528)/(o+1.)+(0.227406381703)+(-0.500500871517)*0.5**o
        ref[2,1]=(0.57374800671)/(o+1.)+(0.2040370984)+(0.120233846989)*0.5**o
        ref[2,2]=(0.323501981101)/(o+1.)+(-0.521313723714)+(0.811775583161)*0.5**o
        ref[2,3]=(1.04371810459)/(o+1.)+(-0.462981220464)+(-0.325647787579)*0.5**o
        ref[2,4]=(-0.958448814807)/(o+1.)+(-0.579086520416)+(0.0621315789778)*0.5**o
        ref[3,0]=(-1.60193047005)/(o+1.)+(0.104679343308)+(0.140605486894)*0.5**o
        ref[3,1]=(-0.789139057035)/(o+1.)+(-0.754379597962)+(-0.660232764173)*0.5**o
        ref[3,2]=(1.42802426591)/(o+1.)+(0.839260954361)+(0.593940468197)*0.5**o
        ref[3,3]=(-0.87630155335)/(o+1.)+(0.426403350019)+(0.870508483705)*0.5**o
        ref[3,4]=(0.384809615783)/(o+1.)+(-0.448855127091)+(-0.638437449353)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactOne_fromData_Solution_rank3(self):
      """
      tests integral of rank 3 Data on the ReducedFunctionOnContactOne

      assumptions: Solution(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      w=Solution(self.domain)
      x=w.getX()
      arg=Data(0,(6, 2, 2),w)
      ref=numpy.zeros((6, 2, 2),numpy.float_)
      if dim==2:
        arg[0,0,0]=(-0.315756069525)*x[0]**o+(0.253240552004)*x[0]+(0.86540335823)*x[1]**o+(-0.616203780065)*x[1]
        arg[0,0,1]=(-0.948116617977)*x[0]**o+(0.892154336439)*x[0]+(0.136605864823)*x[1]**o+(-0.135599413854)*x[1]
        arg[0,1,0]=(0.249388669107)*x[0]**o+(-0.299043026617)*x[0]+(0.335963868826)*x[1]**o+(-0.147202749474)*x[1]
        arg[0,1,1]=(0.649003124445)*x[0]**o+(0.102344972673)*x[0]+(-0.795038258171)*x[1]**o+(0.149716983842)*x[1]
        arg[1,0,0]=(0.814606374941)*x[0]**o+(-0.949251236145)*x[0]+(0.723727126173)*x[1]**o+(-0.75419822708)*x[1]
        arg[1,0,1]=(0.043251358201)*x[0]**o+(-0.227748374318)*x[0]+(-0.231556019421)*x[1]**o+(-0.152524439909)*x[1]
        arg[1,1,0]=(0.565989211326)*x[0]**o+(-0.488248650798)*x[0]+(-0.353077780419)*x[1]**o+(-0.0139123365766)*x[1]
        arg[1,1,1]=(-0.106085432662)*x[0]**o+(0.627311916563)*x[0]+(-0.496301561324)*x[1]**o+(-0.648448849796)*x[1]
        arg[2,0,0]=(0.741868098354)*x[0]**o+(-0.486898566862)*x[0]+(0.0461464851414)*x[1]**o+(0.0133916949979)*x[1]
        arg[2,0,1]=(0.969108066213)*x[0]**o+(-0.688797899381)*x[0]+(0.897017408844)*x[1]**o+(0.807714585761)*x[1]
        arg[2,1,0]=(-0.138159486947)*x[0]**o+(-0.377759480876)*x[0]+(-0.764484355987)*x[1]**o+(-0.723912265361)*x[1]
        arg[2,1,1]=(-0.781167132076)*x[0]**o+(0.119512137054)*x[0]+(-0.535230596139)*x[1]**o+(0.441358968417)*x[1]
        arg[3,0,0]=(-0.663820836909)*x[0]**o+(0.907668749761)*x[0]+(-0.345114213135)*x[1]**o+(0.0580058218243)*x[1]
        arg[3,0,1]=(-0.359054701663)*x[0]**o+(0.0389188839076)*x[0]+(0.168050801441)*x[1]**o+(-0.695770024565)*x[1]
        arg[3,1,0]=(0.70118269628)*x[0]**o+(0.87976625188)*x[0]+(0.154503286758)*x[1]**o+(0.807878896951)*x[1]
        arg[3,1,1]=(-0.143516320927)*x[0]**o+(-0.369452129865)*x[0]+(-0.0392268449697)*x[1]**o+(-0.046104227955)*x[1]
        arg[4,0,0]=(0.197244254754)*x[0]**o+(-0.580444991466)*x[0]+(0.334036744845)*x[1]**o+(0.000791083620504)*x[1]
        arg[4,0,1]=(-0.892626226865)*x[0]**o+(0.140042741589)*x[0]+(0.158775259845)*x[1]**o+(-0.308266331028)*x[1]
        arg[4,1,0]=(0.650374542633)*x[0]**o+(0.586152174865)*x[0]+(0.0201324360301)*x[1]**o+(-0.813221777119)*x[1]
        arg[4,1,1]=(0.149656968314)*x[0]**o+(0.700641349434)*x[0]+(-0.529780404433)*x[1]**o+(0.512159137234)*x[1]
        arg[5,0,0]=(0.288664621864)*x[0]**o+(0.119652133593)*x[0]+(-0.599113573241)*x[1]**o+(0.879966724309)*x[1]
        arg[5,0,1]=(0.861443356689)*x[0]**o+(0.530607121787)*x[0]+(0.240306513701)*x[1]**o+(-0.575772515225)*x[1]
        arg[5,1,0]=(0.721635890008)*x[0]**o+(-0.109537583449)*x[0]+(0.162041613112)*x[1]**o+(0.229143033467)*x[1]
        arg[5,1,1]=(-0.876930820494)*x[0]**o+(0.749378811252)*x[0]+(0.886213554632)*x[1]**o+(0.433327797349)*x[1]
        ref[0,0,0]=(0.86540335823)/(o+1.)+(-0.18148161403)+(-0.315756069525)*0.5**o
        ref[0,0,1]=(0.136605864823)/(o+1.)+(0.378277461292)+(-0.948116617977)*0.5**o
        ref[0,1,0]=(0.335963868826)/(o+1.)+(-0.223122888045)+(0.249388669107)*0.5**o
        ref[0,1,1]=(-0.795038258171)/(o+1.)+(0.126030978258)+(0.649003124445)*0.5**o
        ref[1,0,0]=(0.723727126173)/(o+1.)+(-0.851724731613)+(0.814606374941)*0.5**o
        ref[1,0,1]=(-0.231556019421)/(o+1.)+(-0.190136407114)+(0.043251358201)*0.5**o
        ref[1,1,0]=(-0.353077780419)/(o+1.)+(-0.251080493687)+(0.565989211326)*0.5**o
        ref[1,1,1]=(-0.496301561324)/(o+1.)+(-0.0105684666165)+(-0.106085432662)*0.5**o
        ref[2,0,0]=(0.0461464851414)/(o+1.)+(-0.236753435932)+(0.741868098354)*0.5**o
        ref[2,0,1]=(0.897017408844)/(o+1.)+(0.05945834319)+(0.969108066213)*0.5**o
        ref[2,1,0]=(-0.764484355987)/(o+1.)+(-0.550835873118)+(-0.138159486947)*0.5**o
        ref[2,1,1]=(-0.535230596139)/(o+1.)+(0.280435552736)+(-0.781167132076)*0.5**o
        ref[3,0,0]=(-0.345114213135)/(o+1.)+(0.482837285793)+(-0.663820836909)*0.5**o
        ref[3,0,1]=(0.168050801441)/(o+1.)+(-0.328425570329)+(-0.359054701663)*0.5**o
        ref[3,1,0]=(0.154503286758)/(o+1.)+(0.843822574415)+(0.70118269628)*0.5**o
        ref[3,1,1]=(-0.0392268449697)/(o+1.)+(-0.20777817891)+(-0.143516320927)*0.5**o
        ref[4,0,0]=(0.334036744845)/(o+1.)+(-0.289826953923)+(0.197244254754)*0.5**o
        ref[4,0,1]=(0.158775259845)/(o+1.)+(-0.0841117947196)+(-0.892626226865)*0.5**o
        ref[4,1,0]=(0.0201324360301)/(o+1.)+(-0.113534801127)+(0.650374542633)*0.5**o
        ref[4,1,1]=(-0.529780404433)/(o+1.)+(0.606400243334)+(0.149656968314)*0.5**o
        ref[5,0,0]=(-0.599113573241)/(o+1.)+(0.499809428951)+(0.288664621864)*0.5**o
        ref[5,0,1]=(0.240306513701)/(o+1.)+(-0.022582696719)+(0.861443356689)*0.5**o
        ref[5,1,0]=(0.162041613112)/(o+1.)+(0.0598027250089)+(0.721635890008)*0.5**o
        ref[5,1,1]=(0.886213554632)/(o+1.)+(0.5913533043)+(-0.876930820494)*0.5**o
      else:
        arg[0,0,0]=(0.804252705823)*x[0]**o+(-0.215722921442)*x[0]+(-0.806316349529)*x[1]**o+(-0.984002993222)*x[1]+(-0.411946014731)*x[2]**o+(-0.526356191855)*x[2]
        arg[0,0,1]=(-0.845902068304)*x[0]**o+(0.0773568471048)*x[0]+(-0.746877301439)*x[1]**o+(-0.581914187059)*x[1]+(-0.0299873869697)*x[2]**o+(-0.133702230427)*x[2]
        arg[0,1,0]=(0.0113663840954)*x[0]**o+(-0.155702215771)*x[0]+(-0.134490626112)*x[1]**o+(0.973329917011)*x[1]+(-0.71585624135)*x[2]**o+(-0.840113198085)*x[2]
        arg[0,1,1]=(0.708589516593)*x[0]**o+(0.40282025998)*x[0]+(0.114498353117)*x[1]**o+(0.117244230433)*x[1]+(0.110135318853)*x[2]**o+(-0.0227879044898)*x[2]
        arg[1,0,0]=(0.836102144172)*x[0]**o+(0.212564417583)*x[0]+(0.744354739679)*x[1]**o+(-0.265633978841)*x[1]+(0.843108381264)*x[2]**o+(-0.288428427501)*x[2]
        arg[1,0,1]=(-0.0712209756059)*x[0]**o+(-0.186158140814)*x[0]+(0.184457099888)*x[1]**o+(0.381365494077)*x[1]+(-0.547874738232)*x[2]**o+(0.645124013254)*x[2]
        arg[1,1,0]=(-0.982992052158)*x[0]**o+(0.259627201342)*x[0]+(-0.885159598031)*x[1]**o+(-0.720754022982)*x[1]+(-0.762444117798)*x[2]**o+(0.737661938668)*x[2]
        arg[1,1,1]=(0.518426662725)*x[0]**o+(0.718331452733)*x[0]+(0.523870992899)*x[1]**o+(0.527831420124)*x[1]+(0.791634534316)*x[2]**o+(0.17320870773)*x[2]
        arg[2,0,0]=(-0.0368697940364)*x[0]**o+(-0.101718169515)*x[0]+(0.933712377315)*x[1]**o+(-0.465193151914)*x[1]+(-0.113637506449)*x[2]**o+(0.606619949802)*x[2]
        arg[2,0,1]=(-0.785633726304)*x[0]**o+(0.526518355588)*x[0]+(0.151859749675)*x[1]**o+(0.805094094642)*x[1]+(-0.881122119524)*x[2]**o+(0.85143600659)*x[2]
        arg[2,1,0]=(0.969244001518)*x[0]**o+(0.22600349917)*x[0]+(0.404826692149)*x[1]**o+(0.633039564539)*x[1]+(0.470567669552)*x[2]**o+(0.378348877016)*x[2]
        arg[2,1,1]=(-0.310035929524)*x[0]**o+(0.42632271287)*x[0]+(0.866930472029)*x[1]**o+(-0.124483457934)*x[1]+(0.291920406068)*x[2]**o+(0.573984377154)*x[2]
        arg[3,0,0]=(0.245553971829)*x[0]**o+(-0.567563766526)*x[0]+(0.326654705159)*x[1]**o+(0.642370867777)*x[1]+(-0.538450732704)*x[2]**o+(0.773349470647)*x[2]
        arg[3,0,1]=(0.427017913395)*x[0]**o+(0.885132021286)*x[0]+(0.0338747045378)*x[1]**o+(-0.978508839789)*x[1]+(-0.734559666249)*x[2]**o+(0.379991690191)*x[2]
        arg[3,1,0]=(-0.29533334088)*x[0]**o+(-0.894548834889)*x[0]+(0.537043269081)*x[1]**o+(-0.559173506199)*x[1]+(0.359216600837)*x[2]**o+(0.269563745611)*x[2]
        arg[3,1,1]=(-0.146152544907)*x[0]**o+(-0.95681567795)*x[0]+(-0.0323978120064)*x[1]**o+(-0.5351716012)*x[1]+(-0.548536561192)*x[2]**o+(0.489675460511)*x[2]
        arg[4,0,0]=(0.0552991386013)*x[0]**o+(0.772417012152)*x[0]+(-0.298212559744)*x[1]**o+(-0.189934984078)*x[1]+(0.74843296731)*x[2]**o+(-0.00412448772016)*x[2]
        arg[4,0,1]=(0.577046858044)*x[0]**o+(-0.804142676861)*x[0]+(-0.0703211154568)*x[1]**o+(-0.84092953544)*x[1]+(0.254119640557)*x[2]**o+(-0.765048199333)*x[2]
        arg[4,1,0]=(-0.358558179011)*x[0]**o+(-0.998072158283)*x[0]+(0.508152998513)*x[1]**o+(-0.0813296609683)*x[1]+(-0.534332816004)*x[2]**o+(-0.189623317277)*x[2]
        arg[4,1,1]=(0.840098089823)*x[0]**o+(0.966726062679)*x[0]+(0.920079609258)*x[1]**o+(-0.242532131681)*x[1]+(0.429713768025)*x[2]**o+(0.171223902751)*x[2]
        arg[5,0,0]=(0.695331559401)*x[0]**o+(0.819482211381)*x[0]+(0.550452400412)*x[1]**o+(0.518496496336)*x[1]+(-0.24727850431)*x[2]**o+(-0.796406493488)*x[2]
        arg[5,0,1]=(0.469488028534)*x[0]**o+(0.457530124732)*x[0]+(0.922206279456)*x[1]**o+(0.678758573006)*x[1]+(0.34102098746)*x[2]**o+(0.108616278675)*x[2]
        arg[5,1,0]=(0.444799933589)*x[0]**o+(-0.361568786934)*x[0]+(0.470174557338)*x[1]**o+(-0.344557214342)*x[1]+(0.894860467988)*x[2]**o+(-0.347027743579)*x[2]
        arg[5,1,1]=(0.0304781332872)*x[0]**o+(-0.69046475182)*x[0]+(0.873260975232)*x[1]**o+(-0.705100555993)*x[1]+(0.660057279013)*x[2]**o+(-0.910002146105)*x[2]
        ref[0,0,0]=(-1.21826236426)/(o+1.)+(-0.863041053259)+(0.804252705823)*0.5**o
        ref[0,0,1]=(-0.776864688409)/(o+1.)+(-0.31912978519)+(-0.845902068304)*0.5**o
        ref[0,1,0]=(-0.850346867462)/(o+1.)+(-0.0112427484226)+(0.0113663840954)*0.5**o
        ref[0,1,1]=(0.22463367197)/(o+1.)+(0.248638292962)+(0.708589516593)*0.5**o
        ref[1,0,0]=(1.58746312094)/(o+1.)+(-0.17074899438)+(0.836102144172)*0.5**o
        ref[1,0,1]=(-0.363417638344)/(o+1.)+(0.420165683258)+(-0.0712209756059)*0.5**o
        ref[1,1,0]=(-1.64760371583)/(o+1.)+(0.138267558514)+(-0.982992052158)*0.5**o
        ref[1,1,1]=(1.31550552722)/(o+1.)+(0.709685790293)+(0.518426662725)*0.5**o
        ref[2,0,0]=(0.820074870865)/(o+1.)+(0.0198543141864)+(-0.0368697940364)*0.5**o
        ref[2,0,1]=(-0.729262369848)/(o+1.)+(1.09152422841)+(-0.785633726304)*0.5**o
        ref[2,1,0]=(0.875394361701)/(o+1.)+(0.618695970363)+(0.969244001518)*0.5**o
        ref[2,1,1]=(1.1588508781)/(o+1.)+(0.437911816045)+(-0.310035929524)*0.5**o
        ref[3,0,0]=(-0.211796027546)/(o+1.)+(0.424078285949)+(0.245553971829)*0.5**o
        ref[3,0,1]=(-0.700684961711)/(o+1.)+(0.143307435844)+(0.427017913395)*0.5**o
        ref[3,1,0]=(0.896259869918)/(o+1.)+(-0.592079297738)+(-0.29533334088)*0.5**o
        ref[3,1,1]=(-0.580934373198)/(o+1.)+(-0.50115590932)+(-0.146152544907)*0.5**o
        ref[4,0,0]=(0.450220407566)/(o+1.)+(0.289178770177)+(0.0552991386013)*0.5**o
        ref[4,0,1]=(0.1837985251)/(o+1.)+(-1.20506020582)+(0.577046858044)*0.5**o
        ref[4,1,0]=(-0.0261798174917)/(o+1.)+(-0.634512568264)+(-0.358558179011)*0.5**o
        ref[4,1,1]=(1.34979337728)/(o+1.)+(0.447708916874)+(0.840098089823)*0.5**o
        ref[5,0,0]=(0.303173896103)/(o+1.)+(0.270786107114)+(0.695331559401)*0.5**o
        ref[5,0,1]=(1.26322726692)/(o+1.)+(0.622452488207)+(0.469488028534)*0.5**o
        ref[5,1,0]=(1.36503502533)/(o+1.)+(-0.526576872427)+(0.444799933589)*0.5**o
        ref[5,1,1]=(1.53331825424)/(o+1.)+(-1.15278372696)+(0.0304781332872)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactOne_fromData_Solution_rank4(self):
      """
      tests integral of rank 4 Data on the ReducedFunctionOnContactOne

      assumptions: Solution(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      w=Solution(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5, 3, 2),w)
      ref=numpy.zeros((4, 5, 3, 2),numpy.float_)
      if dim==2:
        arg[0,0,0,0]=(-0.348972036919)*x[0]**o+(0.123542094911)*x[0]+(-0.596353540657)*x[1]**o+(0.717656729405)*x[1]
        arg[0,0,0,1]=(0.187733014896)*x[0]**o+(0.76244697815)*x[0]+(0.261409664458)*x[1]**o+(0.0213714031642)*x[1]
        arg[0,0,1,0]=(0.951655197513)*x[0]**o+(-0.865274104487)*x[0]+(0.59127040848)*x[1]**o+(-0.261622300136)*x[1]
        arg[0,0,1,1]=(-0.188127934304)*x[0]**o+(-0.513768204254)*x[0]+(0.77162116582)*x[1]**o+(-0.12621176914)*x[1]
        arg[0,0,2,0]=(0.770717242337)*x[0]**o+(-0.991038362841)*x[0]+(0.908566805373)*x[1]**o+(-0.24890064672)*x[1]
        arg[0,0,2,1]=(-0.543253359807)*x[0]**o+(-0.0504511921259)*x[0]+(-0.724584621389)*x[1]**o+(0.999965312141)*x[1]
        arg[0,1,0,0]=(0.986939140609)*x[0]**o+(0.236894127675)*x[0]+(0.520162431169)*x[1]**o+(0.306413415759)*x[1]
        arg[0,1,0,1]=(-0.171773212929)*x[0]**o+(-0.248436968201)*x[0]+(0.43818378458)*x[1]**o+(-0.495734982435)*x[1]
        arg[0,1,1,0]=(-0.369350989684)*x[0]**o+(0.59047357584)*x[0]+(0.422340945702)*x[1]**o+(0.326270009775)*x[1]
        arg[0,1,1,1]=(0.559606389467)*x[0]**o+(-0.0507064379501)*x[0]+(-0.0464190048363)*x[1]**o+(0.18964879235)*x[1]
        arg[0,1,2,0]=(-0.989408371198)*x[0]**o+(0.108750148268)*x[0]+(-0.347984444559)*x[1]**o+(-0.819590699683)*x[1]
        arg[0,1,2,1]=(-0.289295900777)*x[0]**o+(-0.579698633174)*x[0]+(-0.124928198101)*x[1]**o+(0.640901698875)*x[1]
        arg[0,2,0,0]=(0.132599667548)*x[0]**o+(-0.102095433147)*x[0]+(0.446169236302)*x[1]**o+(-0.844062525718)*x[1]
        arg[0,2,0,1]=(-0.452662917924)*x[0]**o+(-0.0497770418153)*x[0]+(-0.258525519083)*x[1]**o+(-0.0365197858914)*x[1]
        arg[0,2,1,0]=(0.318740871592)*x[0]**o+(0.562764515639)*x[0]+(-0.209089266173)*x[1]**o+(0.922024900771)*x[1]
        arg[0,2,1,1]=(-0.825757440563)*x[0]**o+(0.634009249217)*x[0]+(0.954836398565)*x[1]**o+(-0.910652766097)*x[1]
        arg[0,2,2,0]=(-0.640337007803)*x[0]**o+(0.862625623816)*x[0]+(-0.0327529254483)*x[1]**o+(-0.327305638603)*x[1]
        arg[0,2,2,1]=(-0.956300293885)*x[0]**o+(0.456116376134)*x[0]+(0.312023159058)*x[1]**o+(0.768546792276)*x[1]
        arg[0,3,0,0]=(-0.927040285049)*x[0]**o+(-0.614994799603)*x[0]+(-0.431859510779)*x[1]**o+(-0.088774748513)*x[1]
        arg[0,3,0,1]=(-0.807009561768)*x[0]**o+(0.00323409463398)*x[0]+(0.274814268019)*x[1]**o+(-0.132755038246)*x[1]
        arg[0,3,1,0]=(0.931774139846)*x[0]**o+(0.665327108449)*x[0]+(0.130437344004)*x[1]**o+(-0.00859025546518)*x[1]
        arg[0,3,1,1]=(-0.314188008746)*x[0]**o+(0.405251729437)*x[0]+(-0.313484462262)*x[1]**o+(-0.670765905491)*x[1]
        arg[0,3,2,0]=(0.748691505108)*x[0]**o+(0.717110963353)*x[0]+(-0.451892315269)*x[1]**o+(-0.103930621637)*x[1]
        arg[0,3,2,1]=(-0.124290384657)*x[0]**o+(0.0761550925153)*x[0]+(0.864166710803)*x[1]**o+(0.050297196468)*x[1]
        arg[0,4,0,0]=(-0.726606985714)*x[0]**o+(-0.304862755503)*x[0]+(-0.364735884875)*x[1]**o+(0.769999145518)*x[1]
        arg[0,4,0,1]=(-0.716457484159)*x[0]**o+(0.850185561874)*x[0]+(0.924535166786)*x[1]**o+(0.414539526778)*x[1]
        arg[0,4,1,0]=(-0.67154700238)*x[0]**o+(-0.0990079744678)*x[0]+(-0.738258048173)*x[1]**o+(-0.105333364414)*x[1]
        arg[0,4,1,1]=(-0.568569022142)*x[0]**o+(-0.865340476096)*x[0]+(0.592223896873)*x[1]**o+(0.545216501911)*x[1]
        arg[0,4,2,0]=(0.959619955514)*x[0]**o+(-0.625097978453)*x[0]+(0.447944032533)*x[1]**o+(-0.792670959618)*x[1]
        arg[0,4,2,1]=(-0.446578795272)*x[0]**o+(0.516341492325)*x[0]+(0.383143400712)*x[1]**o+(-0.339255681885)*x[1]
        arg[1,0,0,0]=(0.130600745253)*x[0]**o+(-0.225084866063)*x[0]+(-0.871963625443)*x[1]**o+(-0.0741251944735)*x[1]
        arg[1,0,0,1]=(-0.684630134211)*x[0]**o+(-0.298097256903)*x[0]+(0.511206122216)*x[1]**o+(-0.784104136535)*x[1]
        arg[1,0,1,0]=(0.624178362847)*x[0]**o+(0.161787148072)*x[0]+(-0.0906465041298)*x[1]**o+(-0.0302149132362)*x[1]
        arg[1,0,1,1]=(-0.403988173215)*x[0]**o+(-0.423895624882)*x[0]+(-0.0594261878161)*x[1]**o+(0.71753539369)*x[1]
        arg[1,0,2,0]=(0.411689797339)*x[0]**o+(0.0555768292842)*x[0]+(0.786614145981)*x[1]**o+(-0.701418967065)*x[1]
        arg[1,0,2,1]=(0.971933572797)*x[0]**o+(0.203165120814)*x[0]+(0.50340688009)*x[1]**o+(-0.443869607992)*x[1]
        arg[1,1,0,0]=(0.517782412737)*x[0]**o+(-0.233071827503)*x[0]+(0.310115588175)*x[1]**o+(-0.970604318783)*x[1]
        arg[1,1,0,1]=(0.250379154253)*x[0]**o+(-0.0914513321258)*x[0]+(0.466403827523)*x[1]**o+(-0.454369386353)*x[1]
        arg[1,1,1,0]=(-0.235520035004)*x[0]**o+(0.0932661387959)*x[0]+(0.0964933173918)*x[1]**o+(0.585245824454)*x[1]
        arg[1,1,1,1]=(-0.466145341257)*x[0]**o+(0.532910678544)*x[0]+(0.757545184326)*x[1]**o+(-0.488241176428)*x[1]
        arg[1,1,2,0]=(0.802070127079)*x[0]**o+(0.30227826497)*x[0]+(0.76818718588)*x[1]**o+(-0.966380392422)*x[1]
        arg[1,1,2,1]=(0.764411747219)*x[0]**o+(0.62151105703)*x[0]+(0.310875960956)*x[1]**o+(0.218417169467)*x[1]
        arg[1,2,0,0]=(0.627865841486)*x[0]**o+(0.328844171432)*x[0]+(0.47405718918)*x[1]**o+(0.238471462812)*x[1]
        arg[1,2,0,1]=(-0.890402471541)*x[0]**o+(0.0654874441479)*x[0]+(0.411199516291)*x[1]**o+(0.0669990208336)*x[1]
        arg[1,2,1,0]=(-0.789401901979)*x[0]**o+(-0.379725885915)*x[0]+(-0.103817842125)*x[1]**o+(0.358055283478)*x[1]
        arg[1,2,1,1]=(-0.619688531158)*x[0]**o+(0.314022911152)*x[0]+(-0.515963904961)*x[1]**o+(0.807912272743)*x[1]
        arg[1,2,2,0]=(0.696584946679)*x[0]**o+(-0.156405028983)*x[0]+(0.99374085906)*x[1]**o+(0.692362331558)*x[1]
        arg[1,2,2,1]=(0.463393460998)*x[0]**o+(-0.0302433372264)*x[0]+(0.243006695756)*x[1]**o+(0.934578685307)*x[1]
        arg[1,3,0,0]=(-0.760149691585)*x[0]**o+(0.305312987392)*x[0]+(0.478665319824)*x[1]**o+(-0.395734016727)*x[1]
        arg[1,3,0,1]=(-0.495638457166)*x[0]**o+(-0.352868042523)*x[0]+(0.0577132713041)*x[1]**o+(0.856507718332)*x[1]
        arg[1,3,1,0]=(-0.588160041725)*x[0]**o+(-0.941242199161)*x[0]+(-0.883563055172)*x[1]**o+(-0.474349426459)*x[1]
        arg[1,3,1,1]=(-0.822752260982)*x[0]**o+(0.284881595545)*x[0]+(-0.914198363943)*x[1]**o+(0.421107500666)*x[1]
        arg[1,3,2,0]=(-0.125099230328)*x[0]**o+(-0.0167977306674)*x[0]+(-0.880147366581)*x[1]**o+(0.947650728684)*x[1]
        arg[1,3,2,1]=(-0.770895899992)*x[0]**o+(-0.121705997003)*x[0]+(-0.0486858273347)*x[1]**o+(0.666179109672)*x[1]
        arg[1,4,0,0]=(-0.767476534476)*x[0]**o+(0.720013119195)*x[0]+(-0.540220631624)*x[1]**o+(-0.363702385125)*x[1]
        arg[1,4,0,1]=(0.90195509915)*x[0]**o+(0.104569298687)*x[0]+(0.901816813435)*x[1]**o+(0.421060929159)*x[1]
        arg[1,4,1,0]=(0.0901631681262)*x[0]**o+(-0.330570875574)*x[0]+(0.892147778885)*x[1]**o+(-0.102976747188)*x[1]
        arg[1,4,1,1]=(-0.336016437275)*x[0]**o+(-0.00913798206221)*x[0]+(-0.20211674106)*x[1]**o+(-0.0470039081275)*x[1]
        arg[1,4,2,0]=(0.194383418491)*x[0]**o+(-0.913993330791)*x[0]+(-0.0207457431142)*x[1]**o+(-0.728130242724)*x[1]
        arg[1,4,2,1]=(-0.285983741049)*x[0]**o+(-0.359824200302)*x[0]+(-0.286913656927)*x[1]**o+(-0.801477464894)*x[1]
        arg[2,0,0,0]=(-0.266085322677)*x[0]**o+(-0.477036323188)*x[0]+(0.967304544659)*x[1]**o+(-0.843721203165)*x[1]
        arg[2,0,0,1]=(-0.431692878995)*x[0]**o+(-0.326793076488)*x[0]+(0.478220893643)*x[1]**o+(-0.953607426852)*x[1]
        arg[2,0,1,0]=(0.800398295672)*x[0]**o+(-0.726923243756)*x[0]+(0.641194491232)*x[1]**o+(0.933755071699)*x[1]
        arg[2,0,1,1]=(0.778158607269)*x[0]**o+(0.192895772287)*x[0]+(0.160540751687)*x[1]**o+(0.132780133987)*x[1]
        arg[2,0,2,0]=(0.0700487728725)*x[0]**o+(0.366571342769)*x[0]+(0.213123464402)*x[1]**o+(0.0224970803568)*x[1]
        arg[2,0,2,1]=(-0.608321696505)*x[0]**o+(0.416761589908)*x[0]+(0.0499280394381)*x[1]**o+(-0.678150075911)*x[1]
        arg[2,1,0,0]=(-0.580987690899)*x[0]**o+(-0.625951863984)*x[0]+(0.0064094419355)*x[1]**o+(0.646045286912)*x[1]
        arg[2,1,0,1]=(-0.881255703221)*x[0]**o+(-0.153796152409)*x[0]+(0.320566497467)*x[1]**o+(-0.217827916224)*x[1]
        arg[2,1,1,0]=(0.981593086009)*x[0]**o+(0.659643450249)*x[0]+(0.824530802391)*x[1]**o+(0.109125926037)*x[1]
        arg[2,1,1,1]=(-0.79695617371)*x[0]**o+(-0.340030091407)*x[0]+(0.476648191338)*x[1]**o+(-0.751961375342)*x[1]
        arg[2,1,2,0]=(0.00785807621726)*x[0]**o+(-0.177037931946)*x[0]+(-0.322685883101)*x[1]**o+(0.639485466195)*x[1]
        arg[2,1,2,1]=(0.386667953213)*x[0]**o+(0.805115373435)*x[0]+(0.489529148574)*x[1]**o+(0.380650996708)*x[1]
        arg[2,2,0,0]=(-0.205678281473)*x[0]**o+(0.725798252716)*x[0]+(-0.884620261631)*x[1]**o+(0.737678769923)*x[1]
        arg[2,2,0,1]=(-0.859841848792)*x[0]**o+(0.215497396508)*x[0]+(-0.9231710768)*x[1]**o+(-0.504914651252)*x[1]
        arg[2,2,1,0]=(-0.0161889255216)*x[0]**o+(0.218150095097)*x[0]+(0.508505799895)*x[1]**o+(0.441270271663)*x[1]
        arg[2,2,1,1]=(0.645494826531)*x[0]**o+(0.812462584734)*x[0]+(0.498639261012)*x[1]**o+(0.695415642432)*x[1]
        arg[2,2,2,0]=(-0.855342734108)*x[0]**o+(0.174928383231)*x[0]+(-0.550636347213)*x[1]**o+(0.814295216919)*x[1]
        arg[2,2,2,1]=(-0.146152468476)*x[0]**o+(-0.810876776267)*x[0]+(0.77444775812)*x[1]**o+(0.758171250375)*x[1]
        arg[2,3,0,0]=(0.914771099535)*x[0]**o+(-0.801103658802)*x[0]+(-0.339761119169)*x[1]**o+(-0.0697409372498)*x[1]
        arg[2,3,0,1]=(-0.278017621273)*x[0]**o+(-0.916896227964)*x[0]+(0.0151205004002)*x[1]**o+(-0.36424189479)*x[1]
        arg[2,3,1,0]=(0.635845368235)*x[0]**o+(-0.286420165946)*x[0]+(-0.547314386502)*x[1]**o+(-0.297882680065)*x[1]
        arg[2,3,1,1]=(0.236860892332)*x[0]**o+(0.670289851551)*x[0]+(-0.437288046493)*x[1]**o+(0.0610684872034)*x[1]
        arg[2,3,2,0]=(0.096479176127)*x[0]**o+(-0.992608241155)*x[0]+(0.622650933171)*x[1]**o+(-0.171594052732)*x[1]
        arg[2,3,2,1]=(0.259913661654)*x[0]**o+(0.521353257934)*x[0]+(0.141212812345)*x[1]**o+(0.0311971708742)*x[1]
        arg[2,4,0,0]=(0.208498140469)*x[0]**o+(0.914285328223)*x[0]+(-0.368860713612)*x[1]**o+(0.744057640483)*x[1]
        arg[2,4,0,1]=(-0.0760701135919)*x[0]**o+(-0.0666410245297)*x[0]+(-0.442858585142)*x[1]**o+(-0.869273907698)*x[1]
        arg[2,4,1,0]=(0.694291134331)*x[0]**o+(0.153487646161)*x[0]+(-0.454726570824)*x[1]**o+(0.12533511793)*x[1]
        arg[2,4,1,1]=(0.505643124198)*x[0]**o+(0.935208927517)*x[0]+(0.120271053976)*x[1]**o+(0.472499209985)*x[1]
        arg[2,4,2,0]=(0.458890639035)*x[0]**o+(-0.38125264637)*x[0]+(-0.742525095759)*x[1]**o+(0.0727341692272)*x[1]
        arg[2,4,2,1]=(-0.228856512525)*x[0]**o+(0.215540806099)*x[0]+(-0.492076254398)*x[1]**o+(0.325121281149)*x[1]
        arg[3,0,0,0]=(0.791320435019)*x[0]**o+(0.264208069532)*x[0]+(0.105218141263)*x[1]**o+(-0.947073121631)*x[1]
        arg[3,0,0,1]=(0.384081306848)*x[0]**o+(-0.735420806741)*x[0]+(0.695451719819)*x[1]**o+(-0.104071285853)*x[1]
        arg[3,0,1,0]=(0.176280431402)*x[0]**o+(0.766248516714)*x[0]+(0.620920827635)*x[1]**o+(0.571778731591)*x[1]
        arg[3,0,1,1]=(0.895148060637)*x[0]**o+(0.120401904357)*x[0]+(0.500131556768)*x[1]**o+(-0.432523916076)*x[1]
        arg[3,0,2,0]=(-0.171375065793)*x[0]**o+(0.731777853738)*x[0]+(0.914774635339)*x[1]**o+(-0.329504614878)*x[1]
        arg[3,0,2,1]=(-0.65523706909)*x[0]**o+(-0.309487164889)*x[0]+(-0.244763521442)*x[1]**o+(0.790744116827)*x[1]
        arg[3,1,0,0]=(-0.515742288)*x[0]**o+(0.688021032329)*x[0]+(0.744558730681)*x[1]**o+(0.663410706435)*x[1]
        arg[3,1,0,1]=(0.299920493853)*x[0]**o+(0.221554378219)*x[0]+(-0.262111371211)*x[1]**o+(0.597219370531)*x[1]
        arg[3,1,1,0]=(-0.491648532317)*x[0]**o+(-0.0763921400417)*x[0]+(0.517994656474)*x[1]**o+(0.690649210498)*x[1]
        arg[3,1,1,1]=(0.291881729319)*x[0]**o+(0.388296683726)*x[0]+(0.304223657331)*x[1]**o+(-0.17249216188)*x[1]
        arg[3,1,2,0]=(0.309288981861)*x[0]**o+(-0.638172982762)*x[0]+(-0.566768084047)*x[1]**o+(-0.694439437371)*x[1]
        arg[3,1,2,1]=(-0.680391436031)*x[0]**o+(0.876096306554)*x[0]+(0.676979754874)*x[1]**o+(0.376507786465)*x[1]
        arg[3,2,0,0]=(-0.688464200232)*x[0]**o+(0.404140014906)*x[0]+(-0.389359624725)*x[1]**o+(0.218257423404)*x[1]
        arg[3,2,0,1]=(-0.321661287857)*x[0]**o+(0.305707842777)*x[0]+(0.0488067596906)*x[1]**o+(0.475071042874)*x[1]
        arg[3,2,1,0]=(-0.172172832131)*x[0]**o+(-0.0589598637728)*x[0]+(0.997578239775)*x[1]**o+(0.0919614884141)*x[1]
        arg[3,2,1,1]=(0.500590025777)*x[0]**o+(0.258724052781)*x[0]+(0.233819575609)*x[1]**o+(0.787949345995)*x[1]
        arg[3,2,2,0]=(-0.464931198743)*x[0]**o+(0.689246395117)*x[0]+(0.0701366334357)*x[1]**o+(-0.206391031385)*x[1]
        arg[3,2,2,1]=(-0.352492545609)*x[0]**o+(-0.160385273073)*x[0]+(0.876791526094)*x[1]**o+(0.748257988817)*x[1]
        arg[3,3,0,0]=(0.70346530112)*x[0]**o+(-0.351192123239)*x[0]+(-0.985298342079)*x[1]**o+(-0.286817451723)*x[1]
        arg[3,3,0,1]=(0.841285056577)*x[0]**o+(0.741986440026)*x[0]+(-0.758645627646)*x[1]**o+(-0.914494381783)*x[1]
        arg[3,3,1,0]=(-0.614293213484)*x[0]**o+(-0.746555667296)*x[0]+(-0.301781252786)*x[1]**o+(-0.721633195357)*x[1]
        arg[3,3,1,1]=(-0.287510519978)*x[0]**o+(-0.764960517935)*x[0]+(-0.948032248565)*x[1]**o+(-0.548676002552)*x[1]
        arg[3,3,2,0]=(0.145506943729)*x[0]**o+(-0.353925293653)*x[0]+(0.786904887352)*x[1]**o+(-0.36548133551)*x[1]
        arg[3,3,2,1]=(0.60782215735)*x[0]**o+(0.872444227513)*x[0]+(-0.237571902274)*x[1]**o+(-0.0559745367747)*x[1]
        arg[3,4,0,0]=(0.691107698337)*x[0]**o+(0.0800532750053)*x[0]+(-0.847456183316)*x[1]**o+(0.911068576288)*x[1]
        arg[3,4,0,1]=(-0.00167606578449)*x[0]**o+(-0.697603805697)*x[0]+(-0.362440886816)*x[1]**o+(-0.0604379772766)*x[1]
        arg[3,4,1,0]=(-0.74026325403)*x[0]**o+(-0.539076677004)*x[0]+(-0.317176913637)*x[1]**o+(0.759776288772)*x[1]
        arg[3,4,1,1]=(-0.230005454291)*x[0]**o+(-0.929557352119)*x[0]+(-0.435125210116)*x[1]**o+(0.106892135893)*x[1]
        arg[3,4,2,0]=(-0.638149620463)*x[0]**o+(-0.0511424128986)*x[0]+(0.460436795546)*x[1]**o+(-0.919535518181)*x[1]
        arg[3,4,2,1]=(-0.793537245531)*x[0]**o+(0.409768940347)*x[0]+(0.632560678557)*x[1]**o+(0.700596370851)*x[1]
        ref[0,0,0,0]=(-0.596353540657)/(o+1.)+(0.420599412158)+(-0.348972036919)*0.5**o
        ref[0,0,0,1]=(0.261409664458)/(o+1.)+(0.391909190657)+(0.187733014896)*0.5**o
        ref[0,0,1,0]=(0.59127040848)/(o+1.)+(-0.563448202311)+(0.951655197513)*0.5**o
        ref[0,0,1,1]=(0.77162116582)/(o+1.)+(-0.319989986697)+(-0.188127934304)*0.5**o
        ref[0,0,2,0]=(0.908566805373)/(o+1.)+(-0.619969504781)+(0.770717242337)*0.5**o
        ref[0,0,2,1]=(-0.724584621389)/(o+1.)+(0.474757060008)+(-0.543253359807)*0.5**o
        ref[0,1,0,0]=(0.520162431169)/(o+1.)+(0.271653771717)+(0.986939140609)*0.5**o
        ref[0,1,0,1]=(0.43818378458)/(o+1.)+(-0.372085975318)+(-0.171773212929)*0.5**o
        ref[0,1,1,0]=(0.422340945702)/(o+1.)+(0.458371792807)+(-0.369350989684)*0.5**o
        ref[0,1,1,1]=(-0.0464190048363)/(o+1.)+(0.0694711771999)+(0.559606389467)*0.5**o
        ref[0,1,2,0]=(-0.347984444559)/(o+1.)+(-0.355420275708)+(-0.989408371198)*0.5**o
        ref[0,1,2,1]=(-0.124928198101)/(o+1.)+(0.0306015328506)+(-0.289295900777)*0.5**o
        ref[0,2,0,0]=(0.446169236302)/(o+1.)+(-0.473078979432)+(0.132599667548)*0.5**o
        ref[0,2,0,1]=(-0.258525519083)/(o+1.)+(-0.0431484138534)+(-0.452662917924)*0.5**o
        ref[0,2,1,0]=(-0.209089266173)/(o+1.)+(0.742394708205)+(0.318740871592)*0.5**o
        ref[0,2,1,1]=(0.954836398565)/(o+1.)+(-0.13832175844)+(-0.825757440563)*0.5**o
        ref[0,2,2,0]=(-0.0327529254483)/(o+1.)+(0.267659992607)+(-0.640337007803)*0.5**o
        ref[0,2,2,1]=(0.312023159058)/(o+1.)+(0.612331584205)+(-0.956300293885)*0.5**o
        ref[0,3,0,0]=(-0.431859510779)/(o+1.)+(-0.351884774058)+(-0.927040285049)*0.5**o
        ref[0,3,0,1]=(0.274814268019)/(o+1.)+(-0.064760471806)+(-0.807009561768)*0.5**o
        ref[0,3,1,0]=(0.130437344004)/(o+1.)+(0.328368426492)+(0.931774139846)*0.5**o
        ref[0,3,1,1]=(-0.313484462262)/(o+1.)+(-0.132757088027)+(-0.314188008746)*0.5**o
        ref[0,3,2,0]=(-0.451892315269)/(o+1.)+(0.306590170858)+(0.748691505108)*0.5**o
        ref[0,3,2,1]=(0.864166710803)/(o+1.)+(0.0632261444916)+(-0.124290384657)*0.5**o
        ref[0,4,0,0]=(-0.364735884875)/(o+1.)+(0.232568195007)+(-0.726606985714)*0.5**o
        ref[0,4,0,1]=(0.924535166786)/(o+1.)+(0.632362544326)+(-0.716457484159)*0.5**o
        ref[0,4,1,0]=(-0.738258048173)/(o+1.)+(-0.102170669441)+(-0.67154700238)*0.5**o
        ref[0,4,1,1]=(0.592223896873)/(o+1.)+(-0.160061987092)+(-0.568569022142)*0.5**o
        ref[0,4,2,0]=(0.447944032533)/(o+1.)+(-0.708884469036)+(0.959619955514)*0.5**o
        ref[0,4,2,1]=(0.383143400712)/(o+1.)+(0.0885429052198)+(-0.446578795272)*0.5**o
        ref[1,0,0,0]=(-0.871963625443)/(o+1.)+(-0.149605030268)+(0.130600745253)*0.5**o
        ref[1,0,0,1]=(0.511206122216)/(o+1.)+(-0.541100696719)+(-0.684630134211)*0.5**o
        ref[1,0,1,0]=(-0.0906465041298)/(o+1.)+(0.065786117418)+(0.624178362847)*0.5**o
        ref[1,0,1,1]=(-0.0594261878161)/(o+1.)+(0.146819884404)+(-0.403988173215)*0.5**o
        ref[1,0,2,0]=(0.786614145981)/(o+1.)+(-0.32292106889)+(0.411689797339)*0.5**o
        ref[1,0,2,1]=(0.50340688009)/(o+1.)+(-0.120352243589)+(0.971933572797)*0.5**o
        ref[1,1,0,0]=(0.310115588175)/(o+1.)+(-0.601838073143)+(0.517782412737)*0.5**o
        ref[1,1,0,1]=(0.466403827523)/(o+1.)+(-0.27291035924)+(0.250379154253)*0.5**o
        ref[1,1,1,0]=(0.0964933173918)/(o+1.)+(0.339255981625)+(-0.235520035004)*0.5**o
        ref[1,1,1,1]=(0.757545184326)/(o+1.)+(0.0223347510579)+(-0.466145341257)*0.5**o
        ref[1,1,2,0]=(0.76818718588)/(o+1.)+(-0.332051063726)+(0.802070127079)*0.5**o
        ref[1,1,2,1]=(0.310875960956)/(o+1.)+(0.419964113249)+(0.764411747219)*0.5**o
        ref[1,2,0,0]=(0.47405718918)/(o+1.)+(0.283657817122)+(0.627865841486)*0.5**o
        ref[1,2,0,1]=(0.411199516291)/(o+1.)+(0.0662432324908)+(-0.890402471541)*0.5**o
        ref[1,2,1,0]=(-0.103817842125)/(o+1.)+(-0.0108353012186)+(-0.789401901979)*0.5**o
        ref[1,2,1,1]=(-0.515963904961)/(o+1.)+(0.560967591948)+(-0.619688531158)*0.5**o
        ref[1,2,2,0]=(0.99374085906)/(o+1.)+(0.267978651288)+(0.696584946679)*0.5**o
        ref[1,2,2,1]=(0.243006695756)/(o+1.)+(0.45216767404)+(0.463393460998)*0.5**o
        ref[1,3,0,0]=(0.478665319824)/(o+1.)+(-0.0452105146678)+(-0.760149691585)*0.5**o
        ref[1,3,0,1]=(0.0577132713041)/(o+1.)+(0.251819837904)+(-0.495638457166)*0.5**o
        ref[1,3,1,0]=(-0.883563055172)/(o+1.)+(-0.70779581281)+(-0.588160041725)*0.5**o
        ref[1,3,1,1]=(-0.914198363943)/(o+1.)+(0.352994548105)+(-0.822752260982)*0.5**o
        ref[1,3,2,0]=(-0.880147366581)/(o+1.)+(0.465426499008)+(-0.125099230328)*0.5**o
        ref[1,3,2,1]=(-0.0486858273347)/(o+1.)+(0.272236556335)+(-0.770895899992)*0.5**o
        ref[1,4,0,0]=(-0.540220631624)/(o+1.)+(0.178155367035)+(-0.767476534476)*0.5**o
        ref[1,4,0,1]=(0.901816813435)/(o+1.)+(0.262815113923)+(0.90195509915)*0.5**o
        ref[1,4,1,0]=(0.892147778885)/(o+1.)+(-0.216773811381)+(0.0901631681262)*0.5**o
        ref[1,4,1,1]=(-0.20211674106)/(o+1.)+(-0.0280709450949)+(-0.336016437275)*0.5**o
        ref[1,4,2,0]=(-0.0207457431142)/(o+1.)+(-0.821061786757)+(0.194383418491)*0.5**o
        ref[1,4,2,1]=(-0.286913656927)/(o+1.)+(-0.580650832598)+(-0.285983741049)*0.5**o
        ref[2,0,0,0]=(0.967304544659)/(o+1.)+(-0.660378763176)+(-0.266085322677)*0.5**o
        ref[2,0,0,1]=(0.478220893643)/(o+1.)+(-0.64020025167)+(-0.431692878995)*0.5**o
        ref[2,0,1,0]=(0.641194491232)/(o+1.)+(0.103415913972)+(0.800398295672)*0.5**o
        ref[2,0,1,1]=(0.160540751687)/(o+1.)+(0.162837953137)+(0.778158607269)*0.5**o
        ref[2,0,2,0]=(0.213123464402)/(o+1.)+(0.194534211563)+(0.0700487728725)*0.5**o
        ref[2,0,2,1]=(0.0499280394381)/(o+1.)+(-0.130694243001)+(-0.608321696505)*0.5**o
        ref[2,1,0,0]=(0.0064094419355)/(o+1.)+(0.0100467114639)+(-0.580987690899)*0.5**o
        ref[2,1,0,1]=(0.320566497467)/(o+1.)+(-0.185812034316)+(-0.881255703221)*0.5**o
        ref[2,1,1,0]=(0.824530802391)/(o+1.)+(0.384384688143)+(0.981593086009)*0.5**o
        ref[2,1,1,1]=(0.476648191338)/(o+1.)+(-0.545995733375)+(-0.79695617371)*0.5**o
        ref[2,1,2,0]=(-0.322685883101)/(o+1.)+(0.231223767125)+(0.00785807621726)*0.5**o
        ref[2,1,2,1]=(0.489529148574)/(o+1.)+(0.592883185071)+(0.386667953213)*0.5**o
        ref[2,2,0,0]=(-0.884620261631)/(o+1.)+(0.73173851132)+(-0.205678281473)*0.5**o
        ref[2,2,0,1]=(-0.9231710768)/(o+1.)+(-0.144708627372)+(-0.859841848792)*0.5**o
        ref[2,2,1,0]=(0.508505799895)/(o+1.)+(0.32971018338)+(-0.0161889255216)*0.5**o
        ref[2,2,1,1]=(0.498639261012)/(o+1.)+(0.753939113583)+(0.645494826531)*0.5**o
        ref[2,2,2,0]=(-0.550636347213)/(o+1.)+(0.494611800075)+(-0.855342734108)*0.5**o
        ref[2,2,2,1]=(0.77444775812)/(o+1.)+(-0.0263527629459)+(-0.146152468476)*0.5**o
        ref[2,3,0,0]=(-0.339761119169)/(o+1.)+(-0.435422298026)+(0.914771099535)*0.5**o
        ref[2,3,0,1]=(0.0151205004002)/(o+1.)+(-0.640569061377)+(-0.278017621273)*0.5**o
        ref[2,3,1,0]=(-0.547314386502)/(o+1.)+(-0.292151423006)+(0.635845368235)*0.5**o
        ref[2,3,1,1]=(-0.437288046493)/(o+1.)+(0.365679169377)+(0.236860892332)*0.5**o
        ref[2,3,2,0]=(0.622650933171)/(o+1.)+(-0.582101146943)+(0.096479176127)*0.5**o
        ref[2,3,2,1]=(0.141212812345)/(o+1.)+(0.276275214404)+(0.259913661654)*0.5**o
        ref[2,4,0,0]=(-0.368860713612)/(o+1.)+(0.829171484353)+(0.208498140469)*0.5**o
        ref[2,4,0,1]=(-0.442858585142)/(o+1.)+(-0.467957466114)+(-0.0760701135919)*0.5**o
        ref[2,4,1,0]=(-0.454726570824)/(o+1.)+(0.139411382046)+(0.694291134331)*0.5**o
        ref[2,4,1,1]=(0.120271053976)/(o+1.)+(0.703854068751)+(0.505643124198)*0.5**o
        ref[2,4,2,0]=(-0.742525095759)/(o+1.)+(-0.154259238572)+(0.458890639035)*0.5**o
        ref[2,4,2,1]=(-0.492076254398)/(o+1.)+(0.270331043624)+(-0.228856512525)*0.5**o
        ref[3,0,0,0]=(0.105218141263)/(o+1.)+(-0.34143252605)+(0.791320435019)*0.5**o
        ref[3,0,0,1]=(0.695451719819)/(o+1.)+(-0.419746046297)+(0.384081306848)*0.5**o
        ref[3,0,1,0]=(0.620920827635)/(o+1.)+(0.669013624153)+(0.176280431402)*0.5**o
        ref[3,0,1,1]=(0.500131556768)/(o+1.)+(-0.15606100586)+(0.895148060637)*0.5**o
        ref[3,0,2,0]=(0.914774635339)/(o+1.)+(0.20113661943)+(-0.171375065793)*0.5**o
        ref[3,0,2,1]=(-0.244763521442)/(o+1.)+(0.240628475969)+(-0.65523706909)*0.5**o
        ref[3,1,0,0]=(0.744558730681)/(o+1.)+(0.675715869382)+(-0.515742288)*0.5**o
        ref[3,1,0,1]=(-0.262111371211)/(o+1.)+(0.409386874375)+(0.299920493853)*0.5**o
        ref[3,1,1,0]=(0.517994656474)/(o+1.)+(0.307128535228)+(-0.491648532317)*0.5**o
        ref[3,1,1,1]=(0.304223657331)/(o+1.)+(0.107902260923)+(0.291881729319)*0.5**o
        ref[3,1,2,0]=(-0.566768084047)/(o+1.)+(-0.666306210066)+(0.309288981861)*0.5**o
        ref[3,1,2,1]=(0.676979754874)/(o+1.)+(0.62630204651)+(-0.680391436031)*0.5**o
        ref[3,2,0,0]=(-0.389359624725)/(o+1.)+(0.311198719155)+(-0.688464200232)*0.5**o
        ref[3,2,0,1]=(0.0488067596906)/(o+1.)+(0.390389442826)+(-0.321661287857)*0.5**o
        ref[3,2,1,0]=(0.997578239775)/(o+1.)+(0.0165008123207)+(-0.172172832131)*0.5**o
        ref[3,2,1,1]=(0.233819575609)/(o+1.)+(0.523336699388)+(0.500590025777)*0.5**o
        ref[3,2,2,0]=(0.0701366334357)/(o+1.)+(0.241427681866)+(-0.464931198743)*0.5**o
        ref[3,2,2,1]=(0.876791526094)/(o+1.)+(0.293936357872)+(-0.352492545609)*0.5**o
        ref[3,3,0,0]=(-0.985298342079)/(o+1.)+(-0.319004787481)+(0.70346530112)*0.5**o
        ref[3,3,0,1]=(-0.758645627646)/(o+1.)+(-0.0862539708788)+(0.841285056577)*0.5**o
        ref[3,3,1,0]=(-0.301781252786)/(o+1.)+(-0.734094431327)+(-0.614293213484)*0.5**o
        ref[3,3,1,1]=(-0.948032248565)/(o+1.)+(-0.656818260243)+(-0.287510519978)*0.5**o
        ref[3,3,2,0]=(0.786904887352)/(o+1.)+(-0.359703314582)+(0.145506943729)*0.5**o
        ref[3,3,2,1]=(-0.237571902274)/(o+1.)+(0.408234845369)+(0.60782215735)*0.5**o
        ref[3,4,0,0]=(-0.847456183316)/(o+1.)+(0.495560925646)+(0.691107698337)*0.5**o
        ref[3,4,0,1]=(-0.362440886816)/(o+1.)+(-0.379020891487)+(-0.00167606578449)*0.5**o
        ref[3,4,1,0]=(-0.317176913637)/(o+1.)+(0.110349805884)+(-0.74026325403)*0.5**o
        ref[3,4,1,1]=(-0.435125210116)/(o+1.)+(-0.411332608113)+(-0.230005454291)*0.5**o
        ref[3,4,2,0]=(0.460436795546)/(o+1.)+(-0.48533896554)+(-0.638149620463)*0.5**o
        ref[3,4,2,1]=(0.632560678557)/(o+1.)+(0.555182655599)+(-0.793537245531)*0.5**o
      else:
        arg[0,0,0,0]=(0.816458625566)*x[0]**o+(-0.438180707327)*x[0]+(-0.39836196112)*x[1]**o+(0.641701267855)*x[1]+(0.919064096208)*x[2]**o+(0.0155653082454)*x[2]
        arg[0,0,0,1]=(-0.709065020122)*x[0]**o+(-0.787938005558)*x[0]+(-0.459829890247)*x[1]**o+(-0.306587811189)*x[1]+(0.648348776519)*x[2]**o+(-0.782857657922)*x[2]
        arg[0,0,1,0]=(-0.09165636265)*x[0]**o+(-0.215464577259)*x[0]+(-0.445060581145)*x[1]**o+(0.541797069021)*x[1]+(-0.312096860473)*x[2]**o+(0.307061070632)*x[2]
        arg[0,0,1,1]=(0.900670566988)*x[0]**o+(-0.157226841228)*x[0]+(-0.00514019601556)*x[1]**o+(0.167616993369)*x[1]+(0.331077209212)*x[2]**o+(0.0797784869412)*x[2]
        arg[0,0,2,0]=(-0.883617789072)*x[0]**o+(-0.233507045302)*x[0]+(0.540311388635)*x[1]**o+(-0.0698351272775)*x[1]+(0.598059668701)*x[2]**o+(-0.842961707358)*x[2]
        arg[0,0,2,1]=(-0.770041366552)*x[0]**o+(-0.683157969301)*x[0]+(0.235580399076)*x[1]**o+(-0.994177047985)*x[1]+(-0.90074656484)*x[2]**o+(-0.0133600820046)*x[2]
        arg[0,1,0,0]=(0.90396926296)*x[0]**o+(0.0492056402145)*x[0]+(0.32744155595)*x[1]**o+(0.0162505824727)*x[1]+(0.386607661085)*x[2]**o+(0.759639829913)*x[2]
        arg[0,1,0,1]=(0.47956310963)*x[0]**o+(0.638067028609)*x[0]+(-0.520596788355)*x[1]**o+(0.476897948019)*x[1]+(0.950320232053)*x[2]**o+(-0.603034001297)*x[2]
        arg[0,1,1,0]=(-0.75177304495)*x[0]**o+(0.369014786745)*x[0]+(-0.332220011735)*x[1]**o+(-0.835059332186)*x[1]+(0.0494323092557)*x[2]**o+(-0.796200422375)*x[2]
        arg[0,1,1,1]=(-0.109937058453)*x[0]**o+(0.607637390544)*x[0]+(0.464147925564)*x[1]**o+(0.650325883743)*x[1]+(0.698196425121)*x[2]**o+(0.933002044297)*x[2]
        arg[0,1,2,0]=(-0.13801307581)*x[0]**o+(0.75768744715)*x[0]+(-0.35990643116)*x[1]**o+(-0.787975707318)*x[1]+(-0.259281382264)*x[2]**o+(0.971003238549)*x[2]
        arg[0,1,2,1]=(-0.518540484981)*x[0]**o+(0.197912267942)*x[0]+(0.481058074022)*x[1]**o+(0.918189335061)*x[1]+(0.160239271801)*x[2]**o+(0.75950986792)*x[2]
        arg[0,2,0,0]=(0.286922773164)*x[0]**o+(-0.874592150595)*x[0]+(0.416014232771)*x[1]**o+(-0.492379239759)*x[1]+(0.950640982197)*x[2]**o+(-0.10625158241)*x[2]
        arg[0,2,0,1]=(-0.116447648923)*x[0]**o+(-0.499481854319)*x[0]+(-0.556602044084)*x[1]**o+(0.753256358343)*x[1]+(0.280780022694)*x[2]**o+(-0.879728755507)*x[2]
        arg[0,2,1,0]=(-0.322803067618)*x[0]**o+(0.0974878233964)*x[0]+(0.420547949274)*x[1]**o+(0.304839957099)*x[1]+(-0.12605355567)*x[2]**o+(-0.166970509739)*x[2]
        arg[0,2,1,1]=(0.997823909059)*x[0]**o+(0.76749010767)*x[0]+(0.376351544673)*x[1]**o+(-0.316937252621)*x[1]+(-0.627670639733)*x[2]**o+(0.584121037987)*x[2]
        arg[0,2,2,0]=(0.516866019209)*x[0]**o+(0.413564799736)*x[0]+(-0.0730736747833)*x[1]**o+(-0.500984764568)*x[1]+(-0.989323894645)*x[2]**o+(-0.261458643035)*x[2]
        arg[0,2,2,1]=(0.777538237337)*x[0]**o+(-0.0623276952959)*x[0]+(0.167068351133)*x[1]**o+(-0.345550070545)*x[1]+(-0.28166577069)*x[2]**o+(-0.568650037569)*x[2]
        arg[0,3,0,0]=(-0.622067362782)*x[0]**o+(0.971384323624)*x[0]+(-0.099597286042)*x[1]**o+(-0.265680407306)*x[1]+(-0.446275631545)*x[2]**o+(-0.0637591060995)*x[2]
        arg[0,3,0,1]=(-0.49698112241)*x[0]**o+(0.766181263321)*x[0]+(-0.977014162391)*x[1]**o+(0.806444170647)*x[1]+(-0.441023882645)*x[2]**o+(-0.919279365683)*x[2]
        arg[0,3,1,0]=(0.917091542097)*x[0]**o+(-0.67016353228)*x[0]+(0.0468346020734)*x[1]**o+(0.81092419675)*x[1]+(0.174037096652)*x[2]**o+(0.564190407348)*x[2]
        arg[0,3,1,1]=(-0.437188100982)*x[0]**o+(-0.0857174204982)*x[0]+(-0.971592105442)*x[1]**o+(-0.122352019844)*x[1]+(-0.853225566811)*x[2]**o+(-0.861128476903)*x[2]
        arg[0,3,2,0]=(-0.841265485658)*x[0]**o+(0.553186811235)*x[0]+(0.414489259007)*x[1]**o+(-0.800997281522)*x[1]+(0.630269543743)*x[2]**o+(0.263584335528)*x[2]
        arg[0,3,2,1]=(0.383282548921)*x[0]**o+(0.260041562865)*x[0]+(-0.95180999378)*x[1]**o+(0.566954668715)*x[1]+(0.688843907846)*x[2]**o+(-0.166837115553)*x[2]
        arg[0,4,0,0]=(0.0385891845805)*x[0]**o+(0.324932244772)*x[0]+(-0.946997425504)*x[1]**o+(-0.745439090316)*x[1]+(-0.721657519976)*x[2]**o+(-0.714366519961)*x[2]
        arg[0,4,0,1]=(-0.387739686761)*x[0]**o+(-0.860915144676)*x[0]+(0.270056169204)*x[1]**o+(-0.914861208322)*x[1]+(-0.453514265988)*x[2]**o+(-0.495768314604)*x[2]
        arg[0,4,1,0]=(-0.329464483421)*x[0]**o+(0.313424566493)*x[0]+(0.185677682455)*x[1]**o+(-0.720200750426)*x[1]+(0.425742092834)*x[2]**o+(0.505209526425)*x[2]
        arg[0,4,1,1]=(0.63272559561)*x[0]**o+(-0.026429919604)*x[0]+(-0.14023723263)*x[1]**o+(-0.580805084228)*x[1]+(0.141232832837)*x[2]**o+(0.241327430534)*x[2]
        arg[0,4,2,0]=(-0.847582850604)*x[0]**o+(0.207270297356)*x[0]+(0.851958035148)*x[1]**o+(0.336594484304)*x[1]+(-0.221434386211)*x[2]**o+(-0.607602400875)*x[2]
        arg[0,4,2,1]=(-0.722990535113)*x[0]**o+(-0.379964881881)*x[0]+(0.825779173038)*x[1]**o+(-0.285509543358)*x[1]+(-0.518809494335)*x[2]**o+(-0.788507606636)*x[2]
        arg[1,0,0,0]=(0.0725951752952)*x[0]**o+(-0.282800153416)*x[0]+(-0.0228303688498)*x[1]**o+(-0.863127549259)*x[1]+(-0.383553228278)*x[2]**o+(-0.0151737809185)*x[2]
        arg[1,0,0,1]=(-0.254920010173)*x[0]**o+(0.160195189848)*x[0]+(0.216407190109)*x[1]**o+(-0.814599655672)*x[1]+(0.771746025302)*x[2]**o+(0.0636221760766)*x[2]
        arg[1,0,1,0]=(0.486342223622)*x[0]**o+(0.736242651359)*x[0]+(0.720722697611)*x[1]**o+(0.974853194395)*x[1]+(-0.834707965096)*x[2]**o+(-0.947169299038)*x[2]
        arg[1,0,1,1]=(0.777015150638)*x[0]**o+(-0.228082772046)*x[0]+(-0.51095285588)*x[1]**o+(-0.835057692508)*x[1]+(-0.564012944223)*x[2]**o+(-0.898040787549)*x[2]
        arg[1,0,2,0]=(0.240813275278)*x[0]**o+(-0.509566588932)*x[0]+(0.247803237451)*x[1]**o+(0.156442781059)*x[1]+(0.618367580757)*x[2]**o+(0.943435734207)*x[2]
        arg[1,0,2,1]=(-0.670257601629)*x[0]**o+(-0.470716006203)*x[0]+(-0.559207336128)*x[1]**o+(-0.81577896974)*x[1]+(0.0571093198914)*x[2]**o+(0.477103420898)*x[2]
        arg[1,1,0,0]=(0.985895581872)*x[0]**o+(-0.00359465828149)*x[0]+(0.00739312124399)*x[1]**o+(-0.931299968425)*x[1]+(-0.10047347666)*x[2]**o+(0.410589913002)*x[2]
        arg[1,1,0,1]=(-0.954132399275)*x[0]**o+(0.159222338169)*x[0]+(-0.865370419503)*x[1]**o+(-0.744696059283)*x[1]+(-0.794993976252)*x[2]**o+(0.961664562839)*x[2]
        arg[1,1,1,0]=(0.462810965625)*x[0]**o+(0.927378551017)*x[0]+(-0.105005418179)*x[1]**o+(0.282340667975)*x[1]+(-0.443931241323)*x[2]**o+(0.740071617715)*x[2]
        arg[1,1,1,1]=(0.645629353541)*x[0]**o+(0.848804271896)*x[0]+(-0.882497356826)*x[1]**o+(-0.74661619875)*x[1]+(-0.927133238344)*x[2]**o+(-0.502758459229)*x[2]
        arg[1,1,2,0]=(-0.554993653726)*x[0]**o+(-0.473286752336)*x[0]+(0.759624773306)*x[1]**o+(0.809169103932)*x[1]+(0.657766085675)*x[2]**o+(0.763913478843)*x[2]
        arg[1,1,2,1]=(0.1476587556)*x[0]**o+(0.385711162597)*x[0]+(-0.0767252754622)*x[1]**o+(-0.544968629318)*x[1]+(0.16751959089)*x[2]**o+(-0.742697412247)*x[2]
        arg[1,2,0,0]=(-0.715887407694)*x[0]**o+(0.273818352078)*x[0]+(0.0426629688432)*x[1]**o+(-0.808531265186)*x[1]+(-0.699839650334)*x[2]**o+(-0.332093073009)*x[2]
        arg[1,2,0,1]=(0.48440337855)*x[0]**o+(0.765443731546)*x[0]+(-0.0761968055499)*x[1]**o+(0.0516389293584)*x[1]+(-0.486224925751)*x[2]**o+(-0.796105709176)*x[2]
        arg[1,2,1,0]=(0.621562227153)*x[0]**o+(0.907213416636)*x[0]+(0.591392192921)*x[1]**o+(-0.825911160831)*x[1]+(0.690773959314)*x[2]**o+(-0.744144678185)*x[2]
        arg[1,2,1,1]=(0.472846565983)*x[0]**o+(-0.837395663533)*x[0]+(0.526438435448)*x[1]**o+(0.310053110281)*x[1]+(0.634301291566)*x[2]**o+(0.608270679412)*x[2]
        arg[1,2,2,0]=(-0.212000572333)*x[0]**o+(0.644218219058)*x[0]+(0.827779140409)*x[1]**o+(0.563968121281)*x[1]+(-0.0556319977635)*x[2]**o+(0.98126338415)*x[2]
        arg[1,2,2,1]=(-0.632752637148)*x[0]**o+(-0.999490611532)*x[0]+(0.0717950178432)*x[1]**o+(-0.0611949850049)*x[1]+(0.740177288895)*x[2]**o+(0.241010363236)*x[2]
        arg[1,3,0,0]=(0.390081216336)*x[0]**o+(-0.389460065845)*x[0]+(0.0812808695521)*x[1]**o+(0.0843698990866)*x[1]+(0.672844227186)*x[2]**o+(0.306345015188)*x[2]
        arg[1,3,0,1]=(-0.402887151891)*x[0]**o+(-0.908615473265)*x[0]+(-0.296307155209)*x[1]**o+(0.228858413043)*x[1]+(0.509401868055)*x[2]**o+(-0.793787805038)*x[2]
        arg[1,3,1,0]=(0.253947492129)*x[0]**o+(0.401465417831)*x[0]+(-0.942329176703)*x[1]**o+(-0.518371272083)*x[1]+(0.90142806242)*x[2]**o+(0.694084281869)*x[2]
        arg[1,3,1,1]=(-0.435656267291)*x[0]**o+(-0.743058541536)*x[0]+(-0.22434079396)*x[1]**o+(-0.414125965702)*x[1]+(0.743546448275)*x[2]**o+(0.323188772386)*x[2]
        arg[1,3,2,0]=(-0.069319863146)*x[0]**o+(-0.0749969565014)*x[0]+(0.29700943253)*x[1]**o+(0.972639470201)*x[1]+(-0.270592469279)*x[2]**o+(-0.355098058901)*x[2]
        arg[1,3,2,1]=(0.741825647399)*x[0]**o+(0.542055833087)*x[0]+(-0.0684409109262)*x[1]**o+(0.842094761143)*x[1]+(-0.777320219447)*x[2]**o+(0.758570478904)*x[2]
        arg[1,4,0,0]=(-0.66003338591)*x[0]**o+(-0.393755284573)*x[0]+(0.617154869065)*x[1]**o+(-0.480999198967)*x[1]+(0.350625972066)*x[2]**o+(0.733472524967)*x[2]
        arg[1,4,0,1]=(-0.0001759210396)*x[0]**o+(-0.0900574950432)*x[0]+(-0.592936627579)*x[1]**o+(-0.647214933825)*x[1]+(-0.43249743908)*x[2]**o+(-0.901387692551)*x[2]
        arg[1,4,1,0]=(0.052435899043)*x[0]**o+(0.163326392489)*x[0]+(0.433169876656)*x[1]**o+(0.977133611649)*x[1]+(-0.311959404527)*x[2]**o+(0.297218570651)*x[2]
        arg[1,4,1,1]=(0.00806522196579)*x[0]**o+(-0.121807466189)*x[0]+(-0.176557734776)*x[1]**o+(0.865146379005)*x[1]+(-0.304648040177)*x[2]**o+(0.269150163824)*x[2]
        arg[1,4,2,0]=(0.440212388104)*x[0]**o+(-0.590292628428)*x[0]+(-0.595359323633)*x[1]**o+(0.606734888384)*x[1]+(-0.951693103773)*x[2]**o+(-0.539741612207)*x[2]
        arg[1,4,2,1]=(0.346121727918)*x[0]**o+(0.188961570963)*x[0]+(0.402818736255)*x[1]**o+(-0.474029523974)*x[1]+(-0.95972237672)*x[2]**o+(0.983543070357)*x[2]
        arg[2,0,0,0]=(-0.801941188348)*x[0]**o+(-0.318215868754)*x[0]+(0.322031073259)*x[1]**o+(-0.00368402006147)*x[1]+(0.473846239338)*x[2]**o+(0.512752561753)*x[2]
        arg[2,0,0,1]=(0.786459471841)*x[0]**o+(0.941049841845)*x[0]+(0.064717660139)*x[1]**o+(-0.891518891268)*x[1]+(-0.317681565962)*x[2]**o+(-0.672310725168)*x[2]
        arg[2,0,1,0]=(0.813571093948)*x[0]**o+(-0.146551267883)*x[0]+(0.942106359835)*x[1]**o+(0.165543949281)*x[1]+(-0.0301736488709)*x[2]**o+(0.936915469225)*x[2]
        arg[2,0,1,1]=(-0.0298664722355)*x[0]**o+(-0.596644819792)*x[0]+(-0.306728572634)*x[1]**o+(0.770671782936)*x[1]+(-0.334606536003)*x[2]**o+(0.888940190159)*x[2]
        arg[2,0,2,0]=(0.688553055025)*x[0]**o+(-0.241541868482)*x[0]+(-0.694609526571)*x[1]**o+(-0.509896457784)*x[1]+(-0.0610272843414)*x[2]**o+(-0.251290389152)*x[2]
        arg[2,0,2,1]=(-0.959007707329)*x[0]**o+(-0.193856482901)*x[0]+(-0.138697126551)*x[1]**o+(0.768166305153)*x[1]+(0.951006645306)*x[2]**o+(-0.533986863953)*x[2]
        arg[2,1,0,0]=(0.648021743651)*x[0]**o+(-0.839074631201)*x[0]+(-0.445337118707)*x[1]**o+(0.997475278653)*x[1]+(-0.154475323496)*x[2]**o+(-0.45292992537)*x[2]
        arg[2,1,0,1]=(-0.134785884326)*x[0]**o+(-0.906738442209)*x[0]+(0.00509900741927)*x[1]**o+(0.489865232888)*x[1]+(0.620823018353)*x[2]**o+(-0.480127263607)*x[2]
        arg[2,1,1,0]=(-0.210371458935)*x[0]**o+(0.308453655681)*x[0]+(-0.510473100104)*x[1]**o+(-0.0776488834445)*x[1]+(0.58103002434)*x[2]**o+(0.039267622777)*x[2]
        arg[2,1,1,1]=(-0.124204683462)*x[0]**o+(0.429525935007)*x[0]+(0.183174600008)*x[1]**o+(-0.665914708196)*x[1]+(-0.576713549033)*x[2]**o+(-0.3788567811)*x[2]
        arg[2,1,2,0]=(-0.864065825608)*x[0]**o+(0.644046059464)*x[0]+(-0.265959280151)*x[1]**o+(0.989465634246)*x[1]+(0.536466956842)*x[2]**o+(0.689999246325)*x[2]
        arg[2,1,2,1]=(0.37622234855)*x[0]**o+(-0.905554688955)*x[0]+(-0.0258037833143)*x[1]**o+(-0.23415538846)*x[1]+(-0.64461795706)*x[2]**o+(0.262259040646)*x[2]
        arg[2,2,0,0]=(0.0541619627989)*x[0]**o+(-0.978918782799)*x[0]+(0.478333900479)*x[1]**o+(0.958652455667)*x[1]+(-0.746701804693)*x[2]**o+(-0.557405653804)*x[2]
        arg[2,2,0,1]=(0.611472347101)*x[0]**o+(-0.409754132769)*x[0]+(0.291274496706)*x[1]**o+(0.901983238042)*x[1]+(-0.102917678824)*x[2]**o+(-0.668205224505)*x[2]
        arg[2,2,1,0]=(-0.926062469523)*x[0]**o+(0.101032437783)*x[0]+(-0.385485330853)*x[1]**o+(0.0521756546126)*x[1]+(0.132320977758)*x[2]**o+(0.471619572767)*x[2]
        arg[2,2,1,1]=(0.0689111652346)*x[0]**o+(-0.517070173331)*x[0]+(-0.599184868344)*x[1]**o+(-0.814239996638)*x[1]+(0.97403158037)*x[2]**o+(-0.546843890524)*x[2]
        arg[2,2,2,0]=(-0.133251706696)*x[0]**o+(-0.0954736405212)*x[0]+(0.780584398494)*x[1]**o+(-0.505305860913)*x[1]+(-0.11518705364)*x[2]**o+(0.0855698333433)*x[2]
        arg[2,2,2,1]=(-0.567309977436)*x[0]**o+(-0.284182307402)*x[0]+(0.220302202269)*x[1]**o+(-0.252628588826)*x[1]+(-0.368390646272)*x[2]**o+(-0.577499260947)*x[2]
        arg[2,3,0,0]=(0.477968626266)*x[0]**o+(0.522764690241)*x[0]+(-0.0339165683557)*x[1]**o+(0.623897338329)*x[1]+(-0.169558925831)*x[2]**o+(-0.426663511213)*x[2]
        arg[2,3,0,1]=(0.185482762361)*x[0]**o+(0.496017413966)*x[0]+(-0.217189284009)*x[1]**o+(0.488205372581)*x[1]+(0.445762454794)*x[2]**o+(-0.423587587222)*x[2]
        arg[2,3,1,0]=(-0.489549524298)*x[0]**o+(-0.482299084307)*x[0]+(0.567729727254)*x[1]**o+(-0.341933025591)*x[1]+(-0.325429025641)*x[2]**o+(0.476030984261)*x[2]
        arg[2,3,1,1]=(-0.390754226646)*x[0]**o+(-0.980255652462)*x[0]+(0.109328516217)*x[1]**o+(-0.0141509875662)*x[1]+(0.486458644215)*x[2]**o+(-0.668018419371)*x[2]
        arg[2,3,2,0]=(0.352912006363)*x[0]**o+(-0.542969561227)*x[0]+(0.512067200296)*x[1]**o+(0.661110389562)*x[1]+(0.289744832231)*x[2]**o+(-0.176943484459)*x[2]
        arg[2,3,2,1]=(0.535262244127)*x[0]**o+(0.528325019892)*x[0]+(-0.106312276613)*x[1]**o+(-0.484575045858)*x[1]+(-0.407717587021)*x[2]**o+(-0.786366604025)*x[2]
        arg[2,4,0,0]=(-0.511771630262)*x[0]**o+(-0.590485876141)*x[0]+(0.493685168985)*x[1]**o+(-0.726700517739)*x[1]+(-0.213137223607)*x[2]**o+(-0.566923814015)*x[2]
        arg[2,4,0,1]=(0.421559088561)*x[0]**o+(0.838784944841)*x[0]+(-0.693654869402)*x[1]**o+(-0.0755493272975)*x[1]+(-0.174245799979)*x[2]**o+(0.242245946157)*x[2]
        arg[2,4,1,0]=(0.681235817735)*x[0]**o+(-0.313564199551)*x[0]+(0.663598935418)*x[1]**o+(-0.604327253895)*x[1]+(0.698985821444)*x[2]**o+(0.298899431146)*x[2]
        arg[2,4,1,1]=(0.432866305507)*x[0]**o+(-0.891501121013)*x[0]+(-0.290106178906)*x[1]**o+(0.268152357012)*x[1]+(0.75279498948)*x[2]**o+(0.211312203371)*x[2]
        arg[2,4,2,0]=(-0.956901622446)*x[0]**o+(0.742548241435)*x[0]+(-0.956209816095)*x[1]**o+(-0.539017080821)*x[1]+(-0.08062797007)*x[2]**o+(0.383843198102)*x[2]
        arg[2,4,2,1]=(-0.16700370288)*x[0]**o+(0.388837619298)*x[0]+(-0.190586726252)*x[1]**o+(0.916004535916)*x[1]+(0.310254387059)*x[2]**o+(-0.380371301717)*x[2]
        arg[3,0,0,0]=(0.64046380712)*x[0]**o+(0.254058760972)*x[0]+(-0.829050222873)*x[1]**o+(0.289942284927)*x[1]+(0.196268101069)*x[2]**o+(0.748880970446)*x[2]
        arg[3,0,0,1]=(-0.226057776579)*x[0]**o+(0.982685008224)*x[0]+(0.819680514589)*x[1]**o+(0.48005222662)*x[1]+(-0.615248069776)*x[2]**o+(-0.0923489923012)*x[2]
        arg[3,0,1,0]=(-0.358031084019)*x[0]**o+(-0.83941518843)*x[0]+(-0.0687218422525)*x[1]**o+(-0.648976147029)*x[1]+(0.711009285701)*x[2]**o+(0.360758898243)*x[2]
        arg[3,0,1,1]=(-0.312175299678)*x[0]**o+(0.360877864123)*x[0]+(0.653283960509)*x[1]**o+(0.323527948404)*x[1]+(-0.278454805659)*x[2]**o+(0.0585194127844)*x[2]
        arg[3,0,2,0]=(0.367207247645)*x[0]**o+(-0.0743547693027)*x[0]+(-0.473358544608)*x[1]**o+(0.332985041128)*x[1]+(-0.899612971909)*x[2]**o+(0.69409768473)*x[2]
        arg[3,0,2,1]=(-0.806787228991)*x[0]**o+(-0.110344204889)*x[0]+(-0.631610779885)*x[1]**o+(0.0239853435419)*x[1]+(0.83036063388)*x[2]**o+(0.623825256849)*x[2]
        arg[3,1,0,0]=(-0.268344201624)*x[0]**o+(-0.59230799177)*x[0]+(0.493719194706)*x[1]**o+(-0.128351291806)*x[1]+(0.0584636422077)*x[2]**o+(0.0859725996454)*x[2]
        arg[3,1,0,1]=(-0.870410307785)*x[0]**o+(-0.917370176071)*x[0]+(0.395701966434)*x[1]**o+(0.694856510628)*x[1]+(-0.748440730513)*x[2]**o+(0.748465450614)*x[2]
        arg[3,1,1,0]=(-0.178465888488)*x[0]**o+(-0.956884701156)*x[0]+(0.0472790389286)*x[1]**o+(0.946702274722)*x[1]+(-0.200668750878)*x[2]**o+(-0.0104023894447)*x[2]
        arg[3,1,1,1]=(0.113559334778)*x[0]**o+(0.270505514561)*x[0]+(-0.238008799138)*x[1]**o+(0.533067625871)*x[1]+(-0.586814102301)*x[2]**o+(0.045657074859)*x[2]
        arg[3,1,2,0]=(-0.574716699633)*x[0]**o+(-0.659499274639)*x[0]+(0.892605757655)*x[1]**o+(-0.39697940457)*x[1]+(0.363011145793)*x[2]**o+(-0.364722526091)*x[2]
        arg[3,1,2,1]=(-0.0193167048813)*x[0]**o+(0.761104649925)*x[0]+(0.490164183635)*x[1]**o+(0.10693267057)*x[1]+(0.635328989408)*x[2]**o+(-0.353114187925)*x[2]
        arg[3,2,0,0]=(0.0777423524941)*x[0]**o+(-0.431011355257)*x[0]+(-0.356762042134)*x[1]**o+(-0.94361679144)*x[1]+(-0.716191700456)*x[2]**o+(0.49906825221)*x[2]
        arg[3,2,0,1]=(0.80859948321)*x[0]**o+(0.542680788872)*x[0]+(-0.626897369633)*x[1]**o+(-0.821877255135)*x[1]+(-0.455575534023)*x[2]**o+(0.317480554025)*x[2]
        arg[3,2,1,0]=(-0.0144328272818)*x[0]**o+(-0.395367848628)*x[0]+(-0.601042089925)*x[1]**o+(-0.0822921879892)*x[1]+(0.896199761801)*x[2]**o+(-0.215055215166)*x[2]
        arg[3,2,1,1]=(0.377945289524)*x[0]**o+(-0.580064012583)*x[0]+(0.993926713349)*x[1]**o+(-0.736307101943)*x[1]+(-0.939628422411)*x[2]**o+(-0.923219473393)*x[2]
        arg[3,2,2,0]=(-0.733150287394)*x[0]**o+(0.692501827859)*x[0]+(0.904655347343)*x[1]**o+(0.310386512564)*x[1]+(-0.93795078156)*x[2]**o+(0.772534565721)*x[2]
        arg[3,2,2,1]=(-0.155229801255)*x[0]**o+(0.580356480314)*x[0]+(0.0673352927499)*x[1]**o+(-0.274975805243)*x[1]+(0.553818209037)*x[2]**o+(-0.835142674988)*x[2]
        arg[3,3,0,0]=(0.60752130978)*x[0]**o+(0.847750395856)*x[0]+(0.8635279539)*x[1]**o+(0.16341072973)*x[1]+(0.470011948821)*x[2]**o+(0.41529971393)*x[2]
        arg[3,3,0,1]=(0.755049219363)*x[0]**o+(0.66080573214)*x[0]+(-0.0256982576552)*x[1]**o+(-0.0311937109251)*x[1]+(-0.028843413794)*x[2]**o+(-0.19345503082)*x[2]
        arg[3,3,1,0]=(-0.39765237082)*x[0]**o+(-0.746359385218)*x[0]+(0.275396324626)*x[1]**o+(0.064571659107)*x[1]+(0.771289010752)*x[2]**o+(-6.69891594269e-05)*x[2]
        arg[3,3,1,1]=(0.350497253938)*x[0]**o+(-0.236419516561)*x[0]+(0.673483791847)*x[1]**o+(-0.934644138115)*x[1]+(0.994408125332)*x[2]**o+(-0.398423596854)*x[2]
        arg[3,3,2,0]=(-0.952544750261)*x[0]**o+(0.24256752245)*x[0]+(0.376782843601)*x[1]**o+(-0.349840168156)*x[1]+(-0.260325141903)*x[2]**o+(-0.452866344447)*x[2]
        arg[3,3,2,1]=(-0.0232941915849)*x[0]**o+(0.191625715695)*x[0]+(-0.490187753329)*x[1]**o+(-0.823976612466)*x[1]+(0.476804203752)*x[2]**o+(0.61057521184)*x[2]
        arg[3,4,0,0]=(0.0279130167137)*x[0]**o+(-0.134953133648)*x[0]+(0.0821614836141)*x[1]**o+(-0.894046763697)*x[1]+(-0.497197221983)*x[2]**o+(0.586297244251)*x[2]
        arg[3,4,0,1]=(0.0423746259802)*x[0]**o+(0.458075512165)*x[0]+(0.855157549678)*x[1]**o+(-0.607654212682)*x[1]+(-0.672693159955)*x[2]**o+(-0.300116892313)*x[2]
        arg[3,4,1,0]=(-0.695685046398)*x[0]**o+(-0.709807910289)*x[0]+(-0.167879145281)*x[1]**o+(0.524084329764)*x[1]+(0.763706690019)*x[2]**o+(-0.219502671318)*x[2]
        arg[3,4,1,1]=(0.864599640696)*x[0]**o+(-0.341125612414)*x[0]+(-0.548888211648)*x[1]**o+(0.918497762429)*x[1]+(-0.445600024463)*x[2]**o+(0.221658300325)*x[2]
        arg[3,4,2,0]=(-0.893419673448)*x[0]**o+(0.653718653341)*x[0]+(0.323868832441)*x[1]**o+(0.905933530129)*x[1]+(0.62045953406)*x[2]**o+(0.145474797235)*x[2]
        arg[3,4,2,1]=(0.480643160777)*x[0]**o+(-0.15896921521)*x[0]+(-0.329564714008)*x[1]**o+(-0.613303857496)*x[1]+(-0.483827450625)*x[2]**o+(-0.91023336048)*x[2]
        ref[0,0,0,0]=(0.520702135089)/(o+1.)+(0.109542934387)+(0.816458625566)*0.5**o
        ref[0,0,0,1]=(0.188518886272)/(o+1.)+(-0.938691737334)+(-0.709065020122)*0.5**o
        ref[0,0,1,0]=(-0.757157441618)/(o+1.)+(0.316696781197)+(-0.09165636265)*0.5**o
        ref[0,0,1,1]=(0.325937013197)/(o+1.)+(0.0450843195409)+(0.900670566988)*0.5**o
        ref[0,0,2,0]=(1.13837105734)/(o+1.)+(-0.573151939969)+(-0.883617789072)*0.5**o
        ref[0,0,2,1]=(-0.665166165764)/(o+1.)+(-0.845347549645)+(-0.770041366552)*0.5**o
        ref[0,1,0,0]=(0.714049217035)/(o+1.)+(0.4125480263)+(0.90396926296)*0.5**o
        ref[0,1,0,1]=(0.429723443697)/(o+1.)+(0.255965487666)+(0.47956310963)*0.5**o
        ref[0,1,1,0]=(-0.282787702479)/(o+1.)+(-0.631122483908)+(-0.75177304495)*0.5**o
        ref[0,1,1,1]=(1.16234435068)/(o+1.)+(1.09548265929)+(-0.109937058453)*0.5**o
        ref[0,1,2,0]=(-0.619187813423)/(o+1.)+(0.47035748919)+(-0.13801307581)*0.5**o
        ref[0,1,2,1]=(0.641297345823)/(o+1.)+(0.937805735461)+(-0.518540484981)*0.5**o
        ref[0,2,0,0]=(1.36665521497)/(o+1.)+(-0.736611486382)+(0.286922773164)*0.5**o
        ref[0,2,0,1]=(-0.27582202139)/(o+1.)+(-0.312977125741)+(-0.116447648923)*0.5**o
        ref[0,2,1,0]=(0.294494393603)/(o+1.)+(0.117678635379)+(-0.322803067618)*0.5**o
        ref[0,2,1,1]=(-0.251319095061)/(o+1.)+(0.517336946518)+(0.997823909059)*0.5**o
        ref[0,2,2,0]=(-1.06239756943)/(o+1.)+(-0.174439303934)+(0.516866019209)*0.5**o
        ref[0,2,2,1]=(-0.114597419557)/(o+1.)+(-0.488263901705)+(0.777538237337)*0.5**o
        ref[0,3,0,0]=(-0.545872917587)/(o+1.)+(0.320972405109)+(-0.622067362782)*0.5**o
        ref[0,3,0,1]=(-1.41803804504)/(o+1.)+(0.326673034142)+(-0.49698112241)*0.5**o
        ref[0,3,1,0]=(0.220871698725)/(o+1.)+(0.352475535909)+(0.917091542097)*0.5**o
        ref[0,3,1,1]=(-1.82481767225)/(o+1.)+(-0.534598958622)+(-0.437188100982)*0.5**o
        ref[0,3,2,0]=(1.04475880275)/(o+1.)+(0.00788693262039)+(-0.841265485658)*0.5**o
        ref[0,3,2,1]=(-0.262966085934)/(o+1.)+(0.330079558014)+(0.383282548921)*0.5**o
        ref[0,4,0,0]=(-1.66865494548)/(o+1.)+(-0.567436682752)+(0.0385891845805)*0.5**o
        ref[0,4,0,1]=(-0.183458096784)/(o+1.)+(-1.1357723338)+(-0.387739686761)*0.5**o
        ref[0,4,1,0]=(0.611419775289)/(o+1.)+(0.0492166712458)+(-0.329464483421)*0.5**o
        ref[0,4,1,1]=(0.000995600207189)/(o+1.)+(-0.182953786649)+(0.63272559561)*0.5**o
        ref[0,4,2,0]=(0.630523648936)/(o+1.)+(-0.0318688096074)+(-0.847582850604)*0.5**o
        ref[0,4,2,1]=(0.306969678703)/(o+1.)+(-0.726991015938)+(-0.722990535113)*0.5**o
        ref[1,0,0,0]=(-0.406383597128)/(o+1.)+(-0.580550741796)+(0.0725951752952)*0.5**o
        ref[1,0,0,1]=(0.988153215411)/(o+1.)+(-0.295391144874)+(-0.254920010173)*0.5**o
        ref[1,0,1,0]=(-0.113985267485)/(o+1.)+(0.381963273358)+(0.486342223622)*0.5**o
        ref[1,0,1,1]=(-1.0749658001)/(o+1.)+(-0.980590626052)+(0.777015150638)*0.5**o
        ref[1,0,2,0]=(0.866170818208)/(o+1.)+(0.295155963167)+(0.240813275278)*0.5**o
        ref[1,0,2,1]=(-0.502098016236)/(o+1.)+(-0.404695777523)+(-0.670257601629)*0.5**o
        ref[1,1,0,0]=(-0.0930803554155)/(o+1.)+(-0.262152356852)+(0.985895581872)*0.5**o
        ref[1,1,0,1]=(-1.66036439576)/(o+1.)+(0.188095420862)+(-0.954132399275)*0.5**o
        ref[1,1,1,0]=(-0.548936659502)/(o+1.)+(0.974895418354)+(0.462810965625)*0.5**o
        ref[1,1,1,1]=(-1.80963059517)/(o+1.)+(-0.200285193041)+(0.645629353541)*0.5**o
        ref[1,1,2,0]=(1.41739085898)/(o+1.)+(0.549897915219)+(-0.554993653726)*0.5**o
        ref[1,1,2,1]=(0.0907943154278)/(o+1.)+(-0.450977439484)+(0.1476587556)*0.5**o
        ref[1,2,0,0]=(-0.657176681491)/(o+1.)+(-0.433402993059)+(-0.715887407694)*0.5**o
        ref[1,2,0,1]=(-0.562421731301)/(o+1.)+(0.0104884758643)+(0.48440337855)*0.5**o
        ref[1,2,1,0]=(1.28216615223)/(o+1.)+(-0.33142121119)+(0.621562227153)*0.5**o
        ref[1,2,1,1]=(1.16073972701)/(o+1.)+(0.0404640630801)+(0.472846565983)*0.5**o
        ref[1,2,2,0]=(0.772147142645)/(o+1.)+(1.09472486224)+(-0.212000572333)*0.5**o
        ref[1,2,2,1]=(0.811972306738)/(o+1.)+(-0.409837616651)+(-0.632752637148)*0.5**o
        ref[1,3,0,0]=(0.754125096738)/(o+1.)+(0.000627424214716)+(0.390081216336)*0.5**o
        ref[1,3,0,1]=(0.213094712846)/(o+1.)+(-0.73677243263)+(-0.402887151891)*0.5**o
        ref[1,3,1,0]=(-0.0409011142831)/(o+1.)+(0.288589213809)+(0.253947492129)*0.5**o
        ref[1,3,1,1]=(0.519205654314)/(o+1.)+(-0.416997867426)+(-0.435656267291)*0.5**o
        ref[1,3,2,0]=(0.0264169632513)/(o+1.)+(0.271272227399)+(-0.069319863146)*0.5**o
        ref[1,3,2,1]=(-0.845761130374)/(o+1.)+(1.07136053657)+(0.741825647399)*0.5**o
        ref[1,4,0,0]=(0.967780841131)/(o+1.)+(-0.0706409792865)+(-0.66003338591)*0.5**o
        ref[1,4,0,1]=(-1.02543406666)/(o+1.)+(-0.81933006071)+(-0.0001759210396)*0.5**o
        ref[1,4,1,0]=(0.121210472128)/(o+1.)+(0.718839287394)+(0.052435899043)*0.5**o
        ref[1,4,1,1]=(-0.481205774953)/(o+1.)+(0.50624453832)+(0.00806522196579)*0.5**o
        ref[1,4,2,0]=(-1.54705242741)/(o+1.)+(-0.261649676126)+(0.440212388104)*0.5**o
        ref[1,4,2,1]=(-0.556903640465)/(o+1.)+(0.349237558673)+(0.346121727918)*0.5**o
        ref[2,0,0,0]=(0.795877312597)/(o+1.)+(0.0954263364687)+(-0.801941188348)*0.5**o
        ref[2,0,0,1]=(-0.252963905823)/(o+1.)+(-0.311389887295)+(0.786459471841)*0.5**o
        ref[2,0,1,0]=(0.911932710964)/(o+1.)+(0.477954075311)+(0.813571093948)*0.5**o
        ref[2,0,1,1]=(-0.641335108636)/(o+1.)+(0.531483576652)+(-0.0298664722355)*0.5**o
        ref[2,0,2,0]=(-0.755636810913)/(o+1.)+(-0.501364357709)+(0.688553055025)*0.5**o
        ref[2,0,2,1]=(0.812309518755)/(o+1.)+(0.0201614791497)+(-0.959007707329)*0.5**o
        ref[2,1,0,0]=(-0.599812442203)/(o+1.)+(-0.147264638959)+(0.648021743651)*0.5**o
        ref[2,1,0,1]=(0.625922025772)/(o+1.)+(-0.448500236464)+(-0.134785884326)*0.5**o
        ref[2,1,1,0]=(0.0705569242367)/(o+1.)+(0.135036197507)+(-0.210371458935)*0.5**o
        ref[2,1,1,1]=(-0.393538949024)/(o+1.)+(-0.307622777145)+(-0.124204683462)*0.5**o
        ref[2,1,2,0]=(0.270507676691)/(o+1.)+(1.16175547002)+(-0.864065825608)*0.5**o
        ref[2,1,2,1]=(-0.670421740374)/(o+1.)+(-0.438725518385)+(0.37622234855)*0.5**o
        ref[2,2,0,0]=(-0.268367904213)/(o+1.)+(-0.288835990468)+(0.0541619627989)*0.5**o
        ref[2,2,0,1]=(0.188356817882)/(o+1.)+(-0.0879880596161)+(0.611472347101)*0.5**o
        ref[2,2,1,0]=(-0.253164353095)/(o+1.)+(0.312413832582)+(-0.926062469523)*0.5**o
        ref[2,2,1,1]=(0.374846712027)/(o+1.)+(-0.939077030246)+(0.0689111652346)*0.5**o
        ref[2,2,2,0]=(0.665397344854)/(o+1.)+(-0.257604834045)+(-0.133251706696)*0.5**o
        ref[2,2,2,1]=(-0.148088444003)/(o+1.)+(-0.557155078588)+(-0.567309977436)*0.5**o
        ref[2,3,0,0]=(-0.203475494187)/(o+1.)+(0.359999258678)+(0.477968626266)*0.5**o
        ref[2,3,0,1]=(0.228573170785)/(o+1.)+(0.280317599662)+(0.185482762361)*0.5**o
        ref[2,3,1,0]=(0.242300701614)/(o+1.)+(-0.174100562819)+(-0.489549524298)*0.5**o
        ref[2,3,1,1]=(0.595787160431)/(o+1.)+(-0.8312125297)+(-0.390754226646)*0.5**o
        ref[2,3,2,0]=(0.801812032527)/(o+1.)+(-0.0294013280621)+(0.352912006363)*0.5**o
        ref[2,3,2,1]=(-0.514029863634)/(o+1.)+(-0.371308314996)+(0.535262244127)*0.5**o
        ref[2,4,0,0]=(0.280547945378)/(o+1.)+(-0.942055103948)+(-0.511771630262)*0.5**o
        ref[2,4,0,1]=(-0.867900669381)/(o+1.)+(0.50274078185)+(0.421559088561)*0.5**o
        ref[2,4,1,0]=(1.36258475686)/(o+1.)+(-0.30949601115)+(0.681235817735)*0.5**o
        ref[2,4,1,1]=(0.462688810574)/(o+1.)+(-0.206018280315)+(0.432866305507)*0.5**o
        ref[2,4,2,0]=(-1.03683778617)/(o+1.)+(0.293687179358)+(-0.956901622446)*0.5**o
        ref[2,4,2,1]=(0.119667660807)/(o+1.)+(0.462235426749)+(-0.16700370288)*0.5**o
        ref[3,0,0,0]=(-0.632782121804)/(o+1.)+(0.646441008172)+(0.64046380712)*0.5**o
        ref[3,0,0,1]=(0.204432444813)/(o+1.)+(0.685194121272)+(-0.226057776579)*0.5**o
        ref[3,0,1,0]=(0.642287443449)/(o+1.)+(-0.563816218608)+(-0.358031084019)*0.5**o
        ref[3,0,1,1]=(0.37482915485)/(o+1.)+(0.371462612656)+(-0.312175299678)*0.5**o
        ref[3,0,2,0]=(-1.37297151652)/(o+1.)+(0.476363978278)+(0.367207247645)*0.5**o
        ref[3,0,2,1]=(0.198749853995)/(o+1.)+(0.268733197751)+(-0.806787228991)*0.5**o
        ref[3,1,0,0]=(0.552182836914)/(o+1.)+(-0.317343341965)+(-0.268344201624)*0.5**o
        ref[3,1,0,1]=(-0.352738764079)/(o+1.)+(0.262975892585)+(-0.870410307785)*0.5**o
        ref[3,1,1,0]=(-0.153389711949)/(o+1.)+(-0.0102924079393)+(-0.178465888488)*0.5**o
        ref[3,1,1,1]=(-0.824822901439)/(o+1.)+(0.424615107645)+(0.113559334778)*0.5**o
        ref[3,1,2,0]=(1.25561690345)/(o+1.)+(-0.71060060265)+(-0.574716699633)*0.5**o
        ref[3,1,2,1]=(1.12549317304)/(o+1.)+(0.257461566285)+(-0.0193167048813)*0.5**o
        ref[3,2,0,0]=(-1.07295374259)/(o+1.)+(-0.437779947243)+(0.0777423524941)*0.5**o
        ref[3,2,0,1]=(-1.08247290366)/(o+1.)+(0.019142043881)+(0.80859948321)*0.5**o
        ref[3,2,1,0]=(0.295157671875)/(o+1.)+(-0.346357625892)+(-0.0144328272818)*0.5**o
        ref[3,2,1,1]=(0.054298290938)/(o+1.)+(-1.11979529396)+(0.377945289524)*0.5**o
        ref[3,2,2,0]=(-0.0332954342171)/(o+1.)+(0.887711453071)+(-0.733150287394)*0.5**o
        ref[3,2,2,1]=(0.621153501787)/(o+1.)+(-0.264880999958)+(-0.155229801255)*0.5**o
        ref[3,3,0,0]=(1.33353990272)/(o+1.)+(0.713230419758)+(0.60752130978)*0.5**o
        ref[3,3,0,1]=(-0.0545416714491)/(o+1.)+(0.218078495198)+(0.755049219363)*0.5**o
        ref[3,3,1,0]=(1.04668533538)/(o+1.)+(-0.340927357635)+(-0.39765237082)*0.5**o
        ref[3,3,1,1]=(1.66789191718)/(o+1.)+(-0.784743625765)+(0.350497253938)*0.5**o
        ref[3,3,2,0]=(0.116457701698)/(o+1.)+(-0.280069495076)+(-0.952544750261)*0.5**o
        ref[3,3,2,1]=(-0.0133835495768)/(o+1.)+(-0.0108878424651)+(-0.0232941915849)*0.5**o
        ref[3,4,0,0]=(-0.415035738369)/(o+1.)+(-0.221351326547)+(0.0279130167137)*0.5**o
        ref[3,4,0,1]=(0.182464389723)/(o+1.)+(-0.224847796415)+(0.0423746259802)*0.5**o
        ref[3,4,1,0]=(0.595827544738)/(o+1.)+(-0.202613125922)+(-0.695685046398)*0.5**o
        ref[3,4,1,1]=(-0.99448823611)/(o+1.)+(0.39951522517)+(0.864599640696)*0.5**o
        ref[3,4,2,0]=(0.944328366501)/(o+1.)+(0.852563490353)+(-0.893419673448)*0.5**o
        ref[3,4,2,1]=(-0.813392164634)/(o+1.)+(-0.841253216593)+(0.480643160777)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactOne_fromData_ReducedSolution_rank0(self):
      """
      tests integral of rank 0 Data on the ReducedFunctionOnContactOne

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Data(0,(),w)
      if dim==2:
        arg=(0.241972460279)*x[0]+(0.539729925673)*x[1]
        ref=(-0.11832904353)/(o+1.)+(0.649911364492)+(-0.399791299503)*0.5**o
      else:
        arg=(0.53919797656)*x[0]+(0.379010664884)*x[1]+(0.274416064529)*x[2]
        ref=(0.307947301472)/(o+1.)+(0.120674157748)+(0.643329089005)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactOne_fromData_ReducedSolution_rank1(self):
      """
      tests integral of rank 1 Data on the ReducedFunctionOnContactOne

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Data(0,(2,),w)
      ref=numpy.zeros((2,),numpy.float_)
      if dim==2:
        arg[0]=(0.84849017056)*x[0]+(-0.161962845202)*x[1]
        arg[1]=(-0.794717919135)*x[0]+(1.16003773358)*x[1]
        ref[0]=(0.780514448202)/(o+1.)+(-0.34628958021)+(0.598592037576)*0.5**o
        ref[1]=(0.570158156278)/(o+1.)+(0.241192316704)+(-0.687222975237)*0.5**o
      else:
        arg[0]=(-0.637352163634)*x[0]+(1.49477462643)*x[1]+(-0.284018740238)*x[2]
        arg[1]=(0.125937459904)*x[0]+(-0.206499869891)*x[1]+(-0.342856240345)*x[2]
        ref[0]=(-0.274486314451)/(o+1.)+(0.379737553853)+(0.0884149293071)*0.5**o
        ref[1]=(-0.78452814021)/(o+1.)+(-0.122017410137)+(0.605144310151)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactOne_fromData_ReducedSolution_rank2(self):
      """
      tests integral of rank 2 Data on the ReducedFunctionOnContactOne

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5),w)
      ref=numpy.zeros((4, 5),numpy.float_)
      if dim==2:
        arg[0,0]=(-0.440868762581)*x[0]+(-0.216223666616)*x[1]
        arg[0,1]=(-1.78807355591)*x[0]+(-0.208278123838)*x[1]
        arg[0,2]=(-0.448066998103)*x[0]+(-0.819815507826)*x[1]
        arg[0,3]=(1.27247881689)*x[0]+(0.0653439567221)*x[1]
        arg[0,4]=(-1.00344990223)*x[0]+(-0.291263790583)*x[1]
        arg[1,0]=(0.440080423838)*x[0]+(0.0356836558113)*x[1]
        arg[1,1]=(-1.49545340091)*x[0]+(-0.133615590211)*x[1]
        arg[1,2]=(1.05947638385)*x[0]+(0.408685904238)*x[1]
        arg[1,3]=(-0.170309086605)*x[0]+(0.660523970285)*x[1]
        arg[1,4]=(0.764323678475)*x[0]+(-0.529415080303)*x[1]
        arg[2,0]=(0.55270081325)*x[0]+(-1.60630146231)*x[1]
        arg[2,1]=(1.57419330248)*x[0]+(-1.24644288706)*x[1]
        arg[2,2]=(0.0416757238363)*x[0]+(0.0247379738465)*x[1]
        arg[2,3]=(1.0200370393)*x[0]+(0.364209137693)*x[1]
        arg[2,4]=(-1.21858490389)*x[0]+(0.523056254546)*x[1]
        arg[3,0]=(0.0524205820842)*x[0]+(1.65953217854)*x[1]
        arg[3,1]=(1.21824883395)*x[0]+(0.905190834998)*x[1]
        arg[3,2]=(0.226344434525)*x[0]+(-0.735349531462)*x[1]
        arg[3,3]=(-1.00295245717)*x[0]+(-0.719605376499)*x[1]
        arg[3,4]=(-0.446228759693)*x[0]+(1.23441974277)*x[1]
        ref[0,0]=(0.347328582602)/(o+1.)+(-0.520484463519)+(0.0365479152384)*0.5**o
        ref[0,1]=(-0.274564442352)/(o+1.)+(-0.413557210678)+(-0.894672816039)*0.5**o
        ref[0,2]=(0.0720644688954)/(o+1.)+(-0.906636966188)+(0.473326957552)*0.5**o
        ref[0,3]=(-0.747749341777)/(o+1.)+(0.730249972378)+(0.625072170633)*0.5**o
        ref[0,4]=(-0.194031116169)/(o+1.)+(-0.432063325829)+(-0.236555924983)*0.5**o
        ref[1,0]=(0.807081538033)/(o+1.)+(-0.612266650007)+(0.893215841629)*0.5**o
        ref[1,1]=(-0.662516518957)/(o+1.)+(-0.176318925886)+(-0.61391462039)*0.5**o
        ref[1,2]=(-0.570345636156)/(o+1.)+(0.791877954963)+(0.454752014319)*0.5**o
        ref[1,3]=(-0.312400910795)/(o+1.)+(0.212935137466)+(0.376745519544)*0.5**o
        ref[1,4]=(-0.764680414264)/(o+1.)+(0.174215486579)+(0.651158039278)*0.5**o
        ref[2,0]=(-0.837942771642)/(o+1.)+(-0.187938486702)+(0.160219095985)*0.5**o
        ref[2,1]=(-0.672421039761)/(o+1.)+(0.17384002882)+(0.652491397549)*0.5**o
        ref[2,2]=(-0.656187949922)/(o+1.)+(0.559888736121)+(-0.397175824637)*0.5**o
        ref[2,3]=(0.259558368195)/(o+1.)+(0.55148644978)+(0.0217149092406)*0.5**o
        ref[2,4]=(-0.188397537901)/(o+1.)+(-0.0561706170512)+(-0.394789877345)*0.5**o
        ref[3,0]=(0.91055628617)/(o+1.)+(0.574211274098)+(-0.347026073739)*0.5**o
        ref[3,1]=(0.29225509237)/(o+1.)+(0.64447705482)+(0.542230466941)*0.5**o
        ref[3,2]=(0.121065624384)/(o+1.)+(0.0316495277836)+(-0.693369776888)*0.5**o
        ref[3,3]=(-0.943523334976)/(o+1.)+(-0.266302644031)+(-0.246429210629)*0.5**o
        ref[3,4]=(0.714915399455)/(o+1.)+(0.459963131994)+(-0.846650680369)*0.5**o
      else:
        arg[0,0]=(-0.493869713454)*x[0]+(-0.381969413515)*x[1]+(1.59862368848)*x[2]
        arg[0,1]=(-0.581245543547)*x[0]+(0.961117271889)*x[1]+(-0.158073515343)*x[2]
        arg[0,2]=(-0.635283143641)*x[0]+(0.695573236987)*x[1]+(-0.719180832278)*x[2]
        arg[0,3]=(-0.737780596733)*x[0]+(-1.11171283352)*x[1]+(0.459954346315)*x[2]
        arg[0,4]=(1.12522065074)*x[0]+(-1.08427904854)*x[1]+(-0.715679673556)*x[2]
        arg[1,0]=(0.972468186181)*x[0]+(-0.211014530435)*x[1]+(-0.734414099002)*x[2]
        arg[1,1]=(-1.67337704748)*x[0]+(0.871096213089)*x[1]+(1.51063313223)*x[2]
        arg[1,2]=(-0.222013007834)*x[0]+(1.46474508326)*x[1]+(-0.979243908862)*x[2]
        arg[1,3]=(1.21195281087)*x[0]+(0.274082389329)*x[1]+(1.56144436465)*x[2]
        arg[1,4]=(1.35812865884)*x[0]+(1.25819798853)*x[1]+(0.895334048395)*x[2]
        arg[2,0]=(-0.167404344437)*x[0]+(0.0710591934315)*x[1]+(-0.124469683665)*x[2]
        arg[2,1]=(-0.492240632996)*x[0]+(-0.317251117382)*x[1]+(0.0195461703529)*x[2]
        arg[2,2]=(-0.277590266225)*x[0]+(-1.65859476899)*x[1]+(-0.0100216405668)*x[2]
        arg[2,3]=(-0.0701281298455)*x[0]+(0.502137254703)*x[1]+(0.701220923443)*x[2]
        arg[2,4]=(-0.72186096407)*x[0]+(-0.900487568179)*x[1]+(-0.735573922826)*x[2]
        arg[3,0]=(-0.0677324474129)*x[0]+(-0.649963209718)*x[1]+(-0.770861099726)*x[2]
        arg[3,1]=(0.563588189481)*x[0]+(0.714614932251)*x[1]+(-0.0957764632886)*x[2]
        arg[3,2]=(1.13337395062)*x[0]+(0.507556316073)*x[1]+(-0.196759085949)*x[2]
        arg[3,3]=(-1.2636883168)*x[0]+(-0.811660344076)*x[1]+(0.645462052598)*x[2]
        arg[3,4]=(-0.104175186987)*x[0]+(-0.929176038155)*x[1]+(1.07530379856)*x[2]
        ref[0,0]=(0.84538674973)/(o+1.)+(0.0569713556632)+(-0.236544899547)*0.5**o
        ref[0,1]=(-0.307689589699)/(o+1.)+(0.245040337642)+(0.0394071274131)*0.5**o
        ref[0,2]=(0.955244736357)/(o+1.)+(-0.341771136504)+(-0.930593202281)*0.5**o
        ref[0,3]=(-0.353490461722)/(o+1.)+(-0.476228433901)+(-0.083591754415)*0.5**o
        ref[0,4]=(-1.31220485482)/(o+1.)+(0.0466426656676)+(0.544181452128)*0.5**o
        ref[1,0]=(0.22773475414)/(o+1.)+(-0.488454801654)+(0.776214405914)*0.5**o
        ref[1,1]=(1.50688678707)/(o+1.)+(0.0851415404695)+(-0.96881757017)*0.5**o
        ref[1,2]=(0.313471097102)/(o+1.)+(-0.0875594518156)+(0.125135973096)*0.5**o
        ref[1,3]=(0.269521220179)/(o+1.)+(1.1192007195)+(0.539556905661)*0.5**o
        ref[1,4]=(1.91499876064)/(o+1.)+(0.509063797893)+(0.578534339343)*0.5**o
        ref[2,0]=(0.941352651562)/(o+1.)+(-0.966199402186)+(0.770231318139)*0.5**o
        ref[2,1]=(-0.141490823562)/(o+1.)+(-0.20919987022)+(-0.230055016023)*0.5**o
        ref[2,2]=(-0.916148047971)/(o+1.)+(-0.371740600192)+(-0.286577427426)*0.5**o
        ref[2,3]=(0.229516219331)/(o+1.)+(0.274790532736)+(0.354132763498)*0.5**o
        ref[2,4]=(-1.19120612619)/(o+1.)+(-0.139859859385)+(-0.88699661011)*0.5**o
        ref[3,0]=(-1.48630121465)/(o+1.)+(0.0744166251063)+(-0.151088792417)*0.5**o
        ref[3,1]=(-0.335583560513)/(o+1.)+(0.277436410841)+(0.963137397276)*0.5**o
        ref[3,2]=(-0.673942688644)/(o+1.)+(0.70077509905)+(0.716563671283)*0.5**o
        ref[3,3]=(-0.148394092795)/(o+1.)+(-0.237190239072)+(-0.807112037336)*0.5**o
        ref[3,4]=(0.529983886211)/(o+1.)+(-0.371055083588)+(0.254078854382)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactOne_fromData_ReducedSolution_rank3(self):
      """
      tests integral of rank 3 Data on the ReducedFunctionOnContactOne

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Data(0,(6, 2, 2),w)
      ref=numpy.zeros((6, 2, 2),numpy.float_)
      if dim==2:
        arg[0,0,0]=(0.661725715165)*x[0]+(0.219720717581)*x[1]
        arg[0,0,1]=(-0.642240082304)*x[0]+(-0.279652756776)*x[1]
        arg[0,1,0]=(0.0204235813151)*x[0]+(-0.374873249933)*x[1]
        arg[0,1,1]=(1.4418481252)*x[0]+(0.205656834772)*x[1]
        arg[1,0,0]=(0.65992179951)*x[0]+(-1.04887875458)*x[1]
        arg[1,0,1]=(-0.195480216199)*x[0]+(-0.373905275039)*x[1]
        arg[1,1,0]=(0.781251543352)*x[0]+(0.0416248109536)*x[1]
        arg[1,1,1]=(-0.925751319189)*x[0]+(-0.113992910046)*x[1]
        arg[2,0,0]=(-0.0767414065227)*x[0]+(-0.156682840973)*x[1]
        arg[2,0,1]=(0.688457272529)*x[0]+(0.524916336046)*x[1]
        arg[2,1,0]=(0.753722830084)*x[0]+(-0.890325839466)*x[1]
        arg[2,1,1]=(0.730789290145)*x[0]+(-0.278986703716)*x[1]
        arg[3,0,0]=(-0.287376796439)*x[0]+(-0.153987233224)*x[1]
        arg[3,0,1]=(-1.15687435255)*x[0]+(0.250005890238)*x[1]
        arg[3,1,0]=(-0.845312830629)*x[0]+(0.256211962352)*x[1]
        arg[3,1,1]=(-0.574311820519)*x[0]+(-0.440297035998)*x[1]
        arg[4,0,0]=(-0.360607647308)*x[0]+(-1.29380437557)*x[1]
        arg[4,0,1]=(1.32019433485)*x[0]+(0.295255956618)*x[1]
        arg[4,1,0]=(-1.64162941972)*x[0]+(0.117482372162)*x[1]
        arg[4,1,1]=(-0.303103646619)*x[0]+(-0.88458982681)*x[1]
        arg[5,0,0]=(-0.666761879038)*x[0]+(-0.735786278877)*x[1]
        arg[5,0,1]=(0.0892896382617)*x[0]+(-0.45478872773)*x[1]
        arg[5,1,0]=(-0.50985624415)*x[0]+(-0.400022939567)*x[1]
        arg[5,1,1]=(1.39240706368)*x[0]+(0.0149740822664)*x[1]
        ref[0,0,0]=(-0.114834538488)/(o+1.)+(0.240621006484)+(0.515038958264)*0.5**o
        ref[0,0,1]=(0.588698337077)/(o+1.)+(-0.702256377233)+(-0.10607842169)*0.5**o
        ref[0,1,0]=(0.13849818224)/(o+1.)+(-0.277555277991)+(0.0621627051247)*0.5**o
        ref[0,1,1]=(-0.0878009521163)/(o+1.)+(0.630168348947)+(0.474969214192)*0.5**o
        ref[1,0,0]=(-0.696600538111)/(o+1.)+(-0.343318260912)+(0.994280104861)*0.5**o
        ref[1,0,1]=(-0.0887026688527)/(o+1.)+(-0.385753227979)+(0.290823633572)*0.5**o
        ref[1,1,0]=(0.831408608956)/(o+1.)+(-0.242985671853)+(0.477439089057)*0.5**o
        ref[1,1,1]=(-0.621059053191)/(o+1.)+(-0.0700203481241)+(-0.278644479797)*0.5**o
        ref[2,0,0]=(0.7890720399)/(o+1.)+(-0.539721196393)+(0.0569461053905)*0.5**o
        ref[2,0,1]=(-0.228628151401)/(o+1.)+(0.826946074171)+(-0.211890388366)*0.5**o
        ref[2,1,0]=(-0.837617325493)/(o+1.)+(0.289656653751)+(0.121701008608)*0.5**o
        ref[2,1,1]=(0.246577988625)/(o+1.)+(0.0512227634376)+(0.102779070928)*0.5**o
        ref[3,0,0]=(-0.0109584684212)/(o+1.)+(-0.334888666171)+(0.239371771102)*0.5**o
        ref[3,0,1]=(-0.125987676419)/(o+1.)+(-0.170179004032)+(-0.440522777827)*0.5**o
        ref[3,1,0]=(-0.354093141636)/(o+1.)+(-0.177294275919)+(0.119580825198)*0.5**o
        ref[3,1,1]=(-0.746761075412)/(o+1.)+(0.192068760731)+(-0.651985302566)*0.5**o
        ref[4,0,0]=(-0.42411673219)/(o+1.)+(-0.579162944937)+(-0.0719694008158)*0.5**o
        ref[4,0,1]=(0.169881169584)/(o+1.)+(0.483614029132)+(0.478341063616)*0.5**o
        ref[4,1,0]=(0.943567836264)/(o+1.)+(-0.898782941264)+(-0.670149001297)*0.5**o
        ref[4,1,1]=(-0.703610807936)/(o+1.)+(0.225342628657)+(-0.934767922806)*0.5**o
        ref[5,0,0]=(-0.788792225213)/(o+1.)+(-0.1768857093)+(-0.259984514102)*0.5**o
        ref[5,0,1]=(-0.788708879899)/(o+1.)+(-0.202288491141)+(0.827786772711)*0.5**o
        ref[5,1,0]=(-0.823081809707)/(o+1.)+(0.274720963093)+(-0.636239300197)*0.5**o
        ref[5,1,1]=(0.498922549807)/(o+1.)+(0.0129388257746)+(0.882580944588)*0.5**o
      else:
        arg[0,0,0]=(-0.426613654844)*x[0]+(-1.01433980801)*x[1]+(0.564621874916)*x[2]
        arg[0,0,1]=(-0.438521968882)*x[0]+(1.6033775267)*x[1]+(0.691349995832)*x[2]
        arg[0,1,0]=(-0.04337513731)*x[0]+(0.142927668234)*x[1]+(0.506581723199)*x[2]
        arg[0,1,1]=(-0.120598868162)*x[0]+(-0.213666787813)*x[1]+(-1.04221679332)*x[2]
        arg[1,0,0]=(0.153696490712)*x[0]+(-0.122516529815)*x[1]+(-1.08837272712)*x[2]
        arg[1,0,1]=(0.410706464237)*x[0]+(0.720079363315)*x[1]+(-0.767790357539)*x[2]
        arg[1,1,0]=(-1.03726428173)*x[0]+(-0.316809853003)*x[1]+(-0.780405868082)*x[2]
        arg[1,1,1]=(-0.427547967216)*x[0]+(-0.897766643609)*x[1]+(-1.56913552404)*x[2]
        arg[2,0,0]=(-0.282789248898)*x[0]+(-0.110846936031)*x[1]+(-1.4329391906)*x[2]
        arg[2,0,1]=(1.46866588617)*x[0]+(-1.23022501959)*x[1]+(-0.3123390018)*x[2]
        arg[2,1,0]=(-0.314115718621)*x[0]+(1.7333934438)*x[1]+(0.833572700726)*x[2]
        arg[2,1,1]=(0.464246462638)*x[0]+(0.267484068646)*x[1]+(0.342009204787)*x[2]
        arg[3,0,0]=(0.578853099163)*x[0]+(-0.0453855508465)*x[1]+(-0.0751578475516)*x[2]
        arg[3,0,1]=(-0.526734485752)*x[0]+(-1.03286310016)*x[1]+(-0.398986583917)*x[2]
        arg[3,1,0]=(0.0239088277798)*x[0]+(0.205105470192)*x[1]+(-1.04986349166)*x[2]
        arg[3,1,1]=(0.130893744001)*x[0]+(-0.273702140669)*x[1]+(-0.0804222415128)*x[2]
        arg[4,0,0]=(-0.822661618584)*x[0]+(-0.267189204519)*x[1]+(-0.290013766971)*x[2]
        arg[4,0,1]=(-1.46759823734)*x[0]+(0.317465945316)*x[1]+(0.108284317752)*x[2]
        arg[4,1,0]=(-1.3920990386)*x[0]+(-1.17823778694)*x[1]+(-0.18366434307)*x[2]
        arg[4,1,1]=(-0.0915496209516)*x[0]+(-0.367624609832)*x[1]+(1.30630257323)*x[2]
        arg[5,0,0]=(0.117840830455)*x[0]+(-0.667875433194)*x[1]+(-1.59716688455)*x[2]
        arg[5,0,1]=(-0.818693227605)*x[0]+(0.483842075248)*x[1]+(-0.408373205535)*x[2]
        arg[5,1,0]=(1.36906770301)*x[0]+(0.494926179999)*x[1]+(0.717041707262)*x[2]
        arg[5,1,1]=(0.819757116081)*x[0]+(0.0982458749015)*x[1]+(0.275080391599)*x[2]
        ref[0,0,0]=(0.236560699949)/(o+1.)+(-0.392784388084)+(-0.327323511722)*0.5**o
        ref[0,0,1]=(1.32491522045)/(o+1.)+(0.525146863711)+(-0.519003394222)*0.5**o
        ref[0,1,0]=(0.433749865563)/(o+1.)+(0.23007126008)+(-0.287758131599)*0.5**o
        ref[0,1,1]=(-0.0563021060686)/(o+1.)+(-0.403921160703)+(-0.512338021825)*0.5**o
        ref[1,0,0]=(-0.170496965963)/(o+1.)+(-0.914421777259)+(0.942147754258)*0.5**o
        ref[1,0,1]=(-0.372707483726)/(o+1.)+(0.636041158872)+(-0.536379364005)*0.5**o
        ref[1,1,0]=(-0.211529878364)/(o+1.)+(-0.912151985776)+(-0.0986461529033)*0.5**o
        ref[1,1,1]=(-1.32533998437)/(o+1.)+(-0.685030198386)+(-0.19904975372)*0.5**o
        ref[2,0,0]=(0.152048902469)/(o+1.)+(-0.88880590209)+(-0.201012473813)*0.5**o
        ref[2,0,1]=(-0.748900052246)/(o+1.)+(-0.0226120471998)+(0.720226011425)*0.5**o
        ref[2,1,0]=(0.77797707941)/(o+1.)+(1.01635055442)+(-0.557827762351)*0.5**o
        ref[2,1,1]=(1.08969334716)/(o+1.)+(-0.404644728844)+(0.793335846595)*0.5**o
        ref[3,0,0]=(1.56498448669)/(o+1.)+(-0.699967188685)+(0.293259591449)*0.5**o
        ref[3,0,1]=(-0.620133427128)/(o+1.)+(-0.550037413688)+(-0.238375915329)*0.5**o
        ref[3,1,0]=(-0.156380586288)/(o+1.)+(-0.0154369973565)+(-0.633594612687)*0.5**o
        ref[3,1,1]=(-0.352989559326)/(o+1.)+(-0.290547031627)+(0.710852984398)*0.5**o
        ref[4,0,0]=(-0.147579361355)/(o+1.)+(-0.615173424765)+(-0.00193837918766)*0.5**o
        ref[4,0,1]=(0.598152345277)/(o+1.)+(-0.540248606426)+(-0.559503106696)*0.5**o
        ref[4,1,0]=(-1.17182448825)/(o+1.)+(-0.590551681576)+(-0.401073317208)*0.5**o
        ref[4,1,1]=(0.837504141173)/(o+1.)+(0.0807414267186)+(-0.15185865216)*0.5**o
        ref[5,0,0]=(-0.646601115533)/(o+1.)+(-1.20895265273)+(0.9173049337)*0.5**o
        ref[5,0,1]=(1.26626110052)/(o+1.)+(-0.56471923973)+(-0.880046978955)*0.5**o
        ref[5,1,0]=(-0.069063169298)/(o+1.)+(0.883536796964)+(0.883025165644)*0.5**o
        ref[5,1,1]=(1.05999338993)/(o+1.)+(-0.392519102893)+(0.918128198439)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactOne_fromData_ReducedSolution_rank4(self):
      """
      tests integral of rank 4 Data on the ReducedFunctionOnContactOne

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5, 3, 2),w)
      ref=numpy.zeros((4, 5, 3, 2),numpy.float_)
      if dim==2:
        arg[0,0,0,0]=(0.257898226293)*x[0]+(-0.01230083314)*x[1]
        arg[0,0,0,1]=(0.0531183070647)*x[0]+(-0.170192287836)*x[1]
        arg[0,0,1,0]=(-0.364644940864)*x[0]+(1.04227651446)*x[1]
        arg[0,0,1,1]=(-0.894334108203)*x[0]+(0.151390331211)*x[1]
        arg[0,0,2,0]=(1.03825401708)*x[0]+(-0.882136097149)*x[1]
        arg[0,0,2,1]=(-1.05304910155)*x[0]+(1.06866311708)*x[1]
        arg[0,1,0,0]=(1.54742542159)*x[0]+(0.941778348482)*x[1]
        arg[0,1,0,1]=(1.1797380956)*x[0]+(-0.296417138234)*x[1]
        arg[0,1,1,0]=(-1.09196028027)*x[0]+(-0.783377624006)*x[1]
        arg[0,1,1,1]=(0.370952191443)*x[0]+(-0.0854141858991)*x[1]
        arg[0,1,2,0]=(-1.23552368139)*x[0]+(1.23848088087)*x[1]
        arg[0,1,2,1]=(0.216345350114)*x[0]+(-0.136524468427)*x[1]
        arg[0,2,0,0]=(-1.21206901263)*x[0]+(0.804219582144)*x[1]
        arg[0,2,0,1]=(0.192956526208)*x[0]+(0.588417678858)*x[1]
        arg[0,2,1,0]=(-0.940175603742)*x[0]+(0.329047260784)*x[1]
        arg[0,2,1,1]=(-0.138597387792)*x[0]+(0.398298326223)*x[1]
        arg[0,2,2,0]=(0.10089766824)*x[0]+(1.25459493298)*x[1]
        arg[0,2,2,1]=(-0.401163525883)*x[0]+(0.276542765282)*x[1]
        arg[0,3,0,0]=(0.252156795268)*x[0]+(-0.50223063539)*x[1]
        arg[0,3,0,1]=(0.466244610905)*x[0]+(-0.587025224301)*x[1]
        arg[0,3,1,0]=(-0.268082012713)*x[0]+(-1.28874675764)*x[1]
        arg[0,3,1,1]=(1.10867463519)*x[0]+(-0.165500912864)*x[1]
        arg[0,3,2,0]=(-1.94681908266)*x[0]+(0.166854108476)*x[1]
        arg[0,3,2,1]=(-0.537150417922)*x[0]+(0.0872851457131)*x[1]
        arg[0,4,0,0]=(-0.376548581807)*x[0]+(-0.245201133388)*x[1]
        arg[0,4,0,1]=(-0.135287690767)*x[0]+(0.750376238942)*x[1]
        arg[0,4,1,0]=(-0.0240491596907)*x[0]+(-1.21337088424)*x[1]
        arg[0,4,1,1]=(0.699215733397)*x[0]+(0.0596356032853)*x[1]
        arg[0,4,2,0]=(-0.521316010702)*x[0]+(-1.75309802615)*x[1]
        arg[0,4,2,1]=(0.403640172305)*x[0]+(1.41465827077)*x[1]
        arg[1,0,0,0]=(-1.01538089315)*x[0]+(-1.14785578909)*x[1]
        arg[1,0,0,1]=(1.10045201832)*x[0]+(0.630761728795)*x[1]
        arg[1,0,1,0]=(0.137673646147)*x[0]+(-0.387129090253)*x[1]
        arg[1,0,1,1]=(-1.04588287863)*x[0]+(-1.22433029859)*x[1]
        arg[1,0,2,0]=(-0.567830263535)*x[0]+(-0.405808616295)*x[1]
        arg[1,0,2,1]=(0.139326102495)*x[0]+(-1.93153384363)*x[1]
        arg[1,1,0,0]=(-0.638524762921)*x[0]+(0.506405751653)*x[1]
        arg[1,1,0,1]=(0.130775153144)*x[0]+(0.531317202208)*x[1]
        arg[1,1,1,0]=(0.557476206038)*x[0]+(0.447588676184)*x[1]
        arg[1,1,1,1]=(-0.538795916875)*x[0]+(-0.593869493531)*x[1]
        arg[1,1,2,0]=(-1.09827483942)*x[0]+(0.376431061876)*x[1]
        arg[1,1,2,1]=(-0.0074505620387)*x[0]+(-0.798645151486)*x[1]
        arg[1,2,0,0]=(0.251283911459)*x[0]+(0.194595071813)*x[1]
        arg[1,2,0,1]=(1.00482763799)*x[0]+(-1.00059863394)*x[1]
        arg[1,2,1,0]=(-0.453095434684)*x[0]+(-0.513922011418)*x[1]
        arg[1,2,1,1]=(0.558548837484)*x[0]+(0.0696328406825)*x[1]
        arg[1,2,2,0]=(-1.20625552545)*x[0]+(-0.151371710692)*x[1]
        arg[1,2,2,1]=(0.100262209656)*x[0]+(-0.314628804179)*x[1]
        arg[1,3,0,0]=(0.947063141882)*x[0]+(-0.297271200562)*x[1]
        arg[1,3,0,1]=(0.0760277251261)*x[0]+(-1.17982146387)*x[1]
        arg[1,3,1,0]=(-1.46946063969)*x[0]+(-0.615302663567)*x[1]
        arg[1,3,1,1]=(0.419747293729)*x[0]+(-0.629842531949)*x[1]
        arg[1,3,2,0]=(-1.44607954917)*x[0]+(0.520024093924)*x[1]
        arg[1,3,2,1]=(-1.01556392596)*x[0]+(-0.307173811354)*x[1]
        arg[1,4,0,0]=(-0.164339335244)*x[0]+(0.390736414982)*x[1]
        arg[1,4,0,1]=(-0.266932380288)*x[0]+(-0.731533848078)*x[1]
        arg[1,4,1,0]=(-0.903825489055)*x[0]+(-1.81564784421)*x[1]
        arg[1,4,1,1]=(-0.426008882665)*x[0]+(-1.09792948908)*x[1]
        arg[1,4,2,0]=(-1.32984927589)*x[0]+(1.58196302541)*x[1]
        arg[1,4,2,1]=(-0.0581032806145)*x[0]+(-0.323073355302)*x[1]
        arg[2,0,0,0]=(-0.744014251373)*x[0]+(-0.818253254872)*x[1]
        arg[2,0,0,1]=(0.368656965396)*x[0]+(-0.0864646380237)*x[1]
        arg[2,0,1,0]=(-0.112428845773)*x[0]+(0.109170104421)*x[1]
        arg[2,0,1,1]=(0.0682241548911)*x[0]+(-1.65999482283)*x[1]
        arg[2,0,2,0]=(0.212777058993)*x[0]+(0.127163480945)*x[1]
        arg[2,0,2,1]=(-0.256989443356)*x[0]+(0.183206687414)*x[1]
        arg[2,1,0,0]=(0.578997994373)*x[0]+(0.413087453689)*x[1]
        arg[2,1,0,1]=(0.886532090807)*x[0]+(0.330645525532)*x[1]
        arg[2,1,1,0]=(0.18224742814)*x[0]+(-0.713166591262)*x[1]
        arg[2,1,1,1]=(-0.25218296706)*x[0]+(0.736797897014)*x[1]
        arg[2,1,2,0]=(0.885270371408)*x[0]+(-1.35641397127)*x[1]
        arg[2,1,2,1]=(-0.963214290879)*x[0]+(0.348421598509)*x[1]
        arg[2,2,0,0]=(0.884120745351)*x[0]+(0.956857507884)*x[1]
        arg[2,2,0,1]=(-0.00795298517561)*x[0]+(0.51243740322)*x[1]
        arg[2,2,1,0]=(-0.563655252975)*x[0]+(-0.716305131721)*x[1]
        arg[2,2,1,1]=(0.648243491623)*x[0]+(-0.142239786485)*x[1]
        arg[2,2,2,0]=(1.50677848542)*x[0]+(-0.359722006818)*x[1]
        arg[2,2,2,1]=(-0.522592086302)*x[0]+(-0.18544927497)*x[1]
        arg[2,3,0,0]=(0.0822787762966)*x[0]+(0.196309662733)*x[1]
        arg[2,3,0,1]=(0.232573258627)*x[0]+(-0.0597976775432)*x[1]
        arg[2,3,1,0]=(1.50895737052)*x[0]+(0.546625934312)*x[1]
        arg[2,3,1,1]=(-1.64023373286)*x[0]+(0.220053261777)*x[1]
        arg[2,3,2,0]=(1.5897208842)*x[0]+(-0.0355885594992)*x[1]
        arg[2,3,2,1]=(-1.52822349982)*x[0]+(-0.0906934717656)*x[1]
        arg[2,4,0,0]=(0.496696582601)*x[0]+(-0.303888675059)*x[1]
        arg[2,4,0,1]=(-0.0819586112177)*x[0]+(-0.0449047294378)*x[1]
        arg[2,4,1,0]=(-0.678530758513)*x[0]+(-1.2173963447)*x[1]
        arg[2,4,1,1]=(-0.645450475268)*x[0]+(0.356771113426)*x[1]
        arg[2,4,2,0]=(-0.486643021867)*x[0]+(0.977763869689)*x[1]
        arg[2,4,2,1]=(-0.458991807397)*x[0]+(1.80038432923)*x[1]
        arg[3,0,0,0]=(0.270795948755)*x[0]+(-0.691512096338)*x[1]
        arg[3,0,0,1]=(-0.756717130099)*x[0]+(1.16189640534)*x[1]
        arg[3,0,1,0]=(0.967554346699)*x[0]+(-0.272119008963)*x[1]
        arg[3,0,1,1]=(1.14296491195)*x[0]+(-0.707175662379)*x[1]
        arg[3,0,2,0]=(1.15665030692)*x[0]+(0.414803389409)*x[1]
        arg[3,0,2,1]=(1.38997274955)*x[0]+(1.07065568192)*x[1]
        arg[3,1,0,0]=(-0.623998925511)*x[0]+(0.375545806421)*x[1]
        arg[3,1,0,1]=(-0.363431948398)*x[0]+(-0.183100417297)*x[1]
        arg[3,1,1,0]=(0.0435866301729)*x[0]+(-0.557137572068)*x[1]
        arg[3,1,1,1]=(-0.253225217468)*x[0]+(-0.0657932188329)*x[1]
        arg[3,1,2,0]=(0.147700921064)*x[0]+(0.681624563015)*x[1]
        arg[3,1,2,1]=(-0.075394297616)*x[0]+(0.271741245789)*x[1]
        arg[3,2,0,0]=(1.34803044812)*x[0]+(0.87991065609)*x[1]
        arg[3,2,0,1]=(0.0529487135353)*x[0]+(0.806480092352)*x[1]
        arg[3,2,1,0]=(-1.52038624166)*x[0]+(-0.992718138685)*x[1]
        arg[3,2,1,1]=(0.0859852188749)*x[0]+(1.63512596202)*x[1]
        arg[3,2,2,0]=(1.46925148575)*x[0]+(-1.60383564005)*x[1]
        arg[3,2,2,1]=(0.114463395473)*x[0]+(-1.0186377376)*x[1]
        arg[3,3,0,0]=(-1.02962502658)*x[0]+(-1.06788543146)*x[1]
        arg[3,3,0,1]=(-0.184391597652)*x[0]+(0.462603936483)*x[1]
        arg[3,3,1,0]=(0.611113335782)*x[0]+(0.282417397043)*x[1]
        arg[3,3,1,1]=(-0.193423873796)*x[0]+(-0.56108614964)*x[1]
        arg[3,3,2,0]=(-0.0290687598224)*x[0]+(-0.460221158417)*x[1]
        arg[3,3,2,1]=(0.784513522959)*x[0]+(-0.484813129206)*x[1]
        arg[3,4,0,0]=(0.423780748664)*x[0]+(-0.344020787894)*x[1]
        arg[3,4,0,1]=(1.10623024451)*x[0]+(-0.775008153301)*x[1]
        arg[3,4,1,0]=(0.554847534291)*x[0]+(-0.854749107999)*x[1]
        arg[3,4,1,1]=(-0.639595693597)*x[0]+(-0.966152813212)*x[1]
        arg[3,4,2,0]=(-0.0172621996023)*x[0]+(0.321815074269)*x[1]
        arg[3,4,2,1]=(0.39605156824)*x[0]+(-0.499138231749)*x[1]
        ref[0,0,0,0]=(0.169892838337)/(o+1.)+(-0.323261281472)+(0.722227117759)*0.5**o
        ref[0,0,0,1]=(-0.899797330597)/(o+1.)+(0.731242549464)+(-0.679761749104)*0.5**o
        ref[0,0,1,0]=(0.948969615551)/(o+1.)+(-0.173478478707)+(0.0756189154575)*0.5**o
        ref[0,0,1,1]=(-0.484419525942)/(o+1.)+(0.287761319651)+(-0.834046890353)*0.5**o
        ref[0,0,2,0]=(-0.275366085384)/(o+1.)+(-0.051636507926)+(0.534757021171)*0.5**o
        ref[0,0,2,1]=(0.338862631582)/(o+1.)+(0.076489529483)+(-0.47622767501)*0.5**o
        ref[0,1,0,0]=(0.960885993642)/(o+1.)+(0.36959166751)+(0.789134441414)*0.5**o
        ref[0,1,0,1]=(-0.101908900511)/(o+1.)+(0.343409131566)+(0.298411594744)*0.5**o
        ref[0,1,1,0]=(-0.50227382988)/(o+1.)+(-0.431549057432)+(-0.509965959533)*0.5**o
        ref[0,1,1,1]=(-0.294216689224)/(o+1.)+(0.293114907564)+(-0.00647512035965)*0.5**o
        ref[0,1,2,0]=(0.330313139948)/(o+1.)+(0.0601559902451)+(-0.447667920962)*0.5**o
        ref[0,1,2,1]=(-0.855325117375)/(o+1.)+(0.455706690618)+(0.0237326178252)*0.5**o
        ref[0,2,0,0]=(0.958948557218)/(o+1.)+(-0.307015973897)+(-0.752766039908)*0.5**o
        ref[0,2,0,1]=(0.622666086232)/(o+1.)+(0.417782845058)+(-0.676857571283)*0.5**o
        ref[0,2,1,0]=(-0.476269525636)/(o+1.)+(0.203518519787)+(-0.541895856898)*0.5**o
        ref[0,2,1,1]=(0.396862410812)/(o+1.)+(-0.318442063423)+(0.499722654464)*0.5**o
        ref[0,2,2,0]=(0.291546173382)/(o+1.)+(0.345688404218)+(0.372569619405)*0.5**o
        ref[0,2,2,1]=(-0.0863432016899)/(o+1.)+(0.296494638577)+(-0.631266836064)*0.5**o
        ref[0,3,0,0]=(0.437179077921)/(o+1.)+(-0.340515851088)+(-0.00622121586734)*0.5**o
        ref[0,3,0,1]=(-0.0670787919003)/(o+1.)+(-0.492735838895)+(0.931769856293)*0.5**o
        ref[0,3,1,0]=(-0.455620785864)/(o+1.)+(-0.866886545624)+(0.632565106759)*0.5**o
        ref[0,3,1,1]=(0.769395985412)/(o+1.)+(-0.303642587154)+(0.781062911223)*0.5**o
        ref[0,3,2,0]=(-0.338160377272)/(o+1.)+(-0.237545833658)+(-0.966712929597)*0.5**o
        ref[0,3,2,1]=(-0.796435534783)/(o+1.)+(0.544922318125)+(-0.743274373676)*0.5**o
        ref[0,4,0,0]=(0.452932120968)/(o+1.)+(-0.512874769603)+(-0.048932296957)*0.5**o
        ref[0,4,0,1]=(0.361579034112)/(o+1.)+(-0.0561290568189)+(0.365767627701)*0.5**o
        ref[0,4,1,0]=(-0.626053161593)/(o+1.)+(-0.761324946295)+(0.911283010254)*0.5**o
        ref[0,4,1,1]=(-0.0455977379835)/(o+1.)+(0.292122388548)+(0.22020429757)*0.5**o
        ref[0,4,2,0]=(-0.881052488675)/(o+1.)+(-0.210171400955)+(-0.973018746263)*0.5**o
        ref[0,4,2,1]=(0.955475719603)/(o+1.)+(0.367959093676)+(0.126904536122)*0.5**o
        ref[1,0,0,0]=(-0.314523133291)/(o+1.)+(-0.649561146707)+(-0.54959125554)*0.5**o
        ref[1,0,0,1]=(0.776726613012)/(o+1.)+(0.0747896120381)+(0.804907910029)*0.5**o
        ref[1,0,1,0]=(0.416309554773)/(o+1.)+(0.0650606132082)+(-0.795886225295)*0.5**o
        ref[1,0,1,1]=(-0.642083920221)/(o+1.)+(-0.518812997818)+(-0.590503261366)*0.5**o
        ref[1,0,2,0]=(0.209400082886)/(o+1.)+(-0.592256267485)+(0.00147357225359)*0.5**o
        ref[1,0,2,1]=(-0.969945221754)/(o+1.)+(-0.364559871358)+(-0.0931427766681)*0.5**o
        ref[1,1,0,0]=(0.941138659497)/(o+1.)+(-0.103485523138)+(-0.866286624489)*0.5**o
        ref[1,1,0,1]=(0.527395399401)/(o+1.)+(0.373358049173)+(-0.612019142395)*0.5**o
        ref[1,1,1,0]=(-0.276942103917)/(o+1.)+(0.624509599487)+(0.0329877871648)*0.5**o
        ref[1,1,1,1]=(-0.153477829221)/(o+1.)+(-0.0616268433725)+(-0.85593389444)*0.5**o
        ref[1,1,2,0]=(0.2685285297)/(o+1.)+(-0.301824551915)+(-0.386723203413)*0.5**o
        ref[1,1,2,1]=(-0.530960074652)/(o+1.)+(0.248621312801)+(-0.772378264474)*0.5**o
        ref[1,2,0,0]=(0.745071931955)/(o+1.)+(-0.149268301427)+(-0.000656345828701)*0.5**o
        ref[1,2,0,1]=(-0.581822428087)/(o+1.)+(0.2315691847)+(0.122913062738)*0.5**o
        ref[1,2,1,0]=(-0.325165099021)/(o+1.)+(-0.414758139139)+(0.187663931198)*0.5**o
        ref[1,2,1,1]=(-0.389683593964)/(o+1.)+(0.265985790556)+(0.485893691018)*0.5**o
        ref[1,2,2,0]=(0.661969003098)/(o+1.)+(-0.514584521701)+(-0.990427195841)*0.5**o
        ref[1,2,2,1]=(-0.0828384828188)/(o+1.)+(-0.482166570368)+(0.832805029032)*0.5**o
        ref[1,3,0,0]=(-0.756020671713)/(o+1.)+(0.627704058228)+(0.150404496577)*0.5**o
        ref[1,3,0,1]=(-0.93648010541)/(o+1.)+(0.11022703478)+(-0.387767702896)*0.5**o
        ref[1,3,1,0]=(-0.995823513997)/(o+1.)+(-0.242072805569)+(-0.604794178119)*0.5**o
        ref[1,3,1,1]=(-0.543071878541)/(o+1.)+(-0.160890620667)+(0.654757881656)*0.5**o
        ref[1,3,2,0]=(0.803184441579)/(o+1.)+(-0.612293875048)+(-0.504652146723)*0.5**o
        ref[1,3,2,1]=(-0.786714439322)/(o+1.)+(-0.175556007517)+(-0.18491128296)*0.5**o
        ref[1,4,0,0]=(-0.302928744298)/(o+1.)+(0.476360987754)+(-0.423396151472)*0.5**o
        ref[1,4,0,1]=(-0.410240602476)/(o+1.)+(-0.49551630458)+(0.40280698327)*0.5**o
        ref[1,4,1,0]=(-0.820677400346)/(o+1.)+(-0.573037933363)+(-0.752720066188)*0.5**o
        ref[1,4,1,1]=(-0.492748923658)/(o+1.)+(-0.600739806799)+(0.170290165507)*0.5**o
        ref[1,4,2,0]=(0.926654770824)/(o+1.)+(0.011282908181)+(-0.697106837662)*0.5**o
        ref[1,4,2,1]=(-0.958477382611)/(o+1.)+(0.618515858902)+(-0.65973097111)*0.5**o
        ref[2,0,0,0]=(-0.15636950624)/(o+1.)+(-0.520670547262)+(-0.36455690548)*0.5**o
        ref[2,0,0,1]=(-0.726424878286)/(o+1.)+(0.697757734621)+(-0.386898263585)*0.5**o
        ref[2,0,1,0]=(-0.055162393508)/(o+1.)+(0.247015646426)+(-0.442127640696)*0.5**o
        ref[2,0,1,1]=(-0.769332288875)/(o+1.)+(-0.0917175733456)+(-0.639003232373)*0.5**o
        ref[2,0,2,0]=(-0.196390038297)/(o+1.)+(0.0977907270548)+(0.340749124126)*0.5**o
        ref[2,0,2,1]=(0.841927481859)/(o+1.)+(-0.489185426146)+(0.0626606144906)*0.5**o
        ref[2,1,0,0]=(0.590018249148)/(o+1.)+(0.378476967182)+(-0.354886735451)*0.5**o
        ref[2,1,0,1]=(0.178495604742)/(o+1.)+(0.233073081069)+(0.572535849459)*0.5**o
        ref[2,1,1,0]=(-0.762671064919)/(o+1.)+(-0.108539686142)+(0.448831274081)*0.5**o
        ref[2,1,1,1]=(0.927591406535)/(o+1.)+(0.0441635703243)+(-0.53130361723)*0.5**o
        ref[2,1,2,0]=(-0.604360706259)/(o+1.)+(-0.134848102332)+(0.402913311063)*0.5**o
        ref[2,1,2,1]=(0.229727627441)/(o+1.)+(-0.323296604293)+(-0.197927111225)*0.5**o
        ref[2,2,0,0]=(0.287478356112)/(o+1.)+(0.762635506201)+(0.0282288847209)*0.5**o
        ref[2,2,0,1]=(0.243518182621)/(o+1.)+(-0.241502069199)+(0.743970373822)*0.5**o
        ref[2,2,1,0]=(-0.369049760378)/(o+1.)+(-0.21941778965)+(-0.472075045017)*0.5**o
        ref[2,2,1,1]=(-0.745343658666)/(o+1.)+(0.193375557775)+(0.864596248255)*0.5**o
        ref[2,2,2,0]=(-0.491769251494)/(o+1.)+(0.442742387618)+(0.753340954862)*0.5**o
        ref[2,2,2,1]=(0.551196377158)/(o+1.)+(-0.523535406613)+(-0.212166925203)*0.5**o
        ref[2,3,0,0]=(0.192271522168)/(o+1.)+(-0.0181953944687)+(0.122707705798)*0.5**o
        ref[2,3,0,1]=(-0.593768738594)/(o+1.)+(0.199088910545)+(0.368366498588)*0.5**o
        ref[2,3,1,0]=(0.0866211808983)/(o+1.)+(0.723512694127)+(0.521936735685)*0.5**o
        ref[2,3,1,1]=(-0.453288268254)/(o+1.)+(-0.15580970281)+(-0.655272797205)*0.5**o
        ref[2,3,2,0]=(0.614105802669)/(o+1.)+(0.0411427694898)+(0.85774098305)*0.5**o
        ref[2,3,2,1]=(0.263133533654)/(o+1.)+(-0.628776355867)+(-0.624497793507)*0.5**o
        ref[2,4,0,0]=(-0.809347914722)/(o+1.)+(0.367719412237)+(0.26671699779)*0.5**o
        ref[2,4,0,1]=(0.773124137566)/(o+1.)+(-0.442496395542)+(-0.0149946871367)*0.5**o
        ref[2,4,1,0]=(-0.927311525126)/(o+1.)+(-0.0125177466568)+(-0.943580084777)*0.5**o
        ref[2,4,1,1]=(0.306409665483)/(o+1.)+(0.128569553604)+(-0.852228134533)*0.5**o
        ref[2,4,2,0]=(0.926185522145)/(o+1.)+(0.0750538146738)+(-0.58517230367)*0.5**o
        ref[2,4,2,1]=(0.858245470922)/(o+1.)+(0.0125876600893)+(0.457971730734)*0.5**o
        ref[3,0,0,0]=(-0.74501947953)/(o+1.)+(0.343942243752)+(-0.363581155556)*0.5**o
        ref[3,0,0,1]=(0.850890737948)/(o+1.)+(0.157997002133)+(-0.761705466968)*0.5**o
        ref[3,0,1,0]=(-0.0468962248591)/(o+1.)+(0.0297980532763)+(0.682735456043)*0.5**o
        ref[3,0,1,1]=(0.0126326425456)/(o+1.)+(-0.076115296096)+(0.575387199213)*0.5**o
        ref[3,0,2,0]=(-0.410549752333)/(o+1.)+(0.85556811978)+(0.270867209104)*0.5**o
        ref[3,0,2,1]=(0.580487848545)/(o+1.)+(0.59069171729)+(0.698757148346)*0.5**o
        ref[3,1,0,0]=(0.139052775786)/(o+1.)+(-0.334747475889)+(0.281989056901)*0.5**o
        ref[3,1,0,1]=(0.591083610998)/(o+1.)+(-0.322680889668)+(-0.492254197357)*0.5**o
        ref[3,1,1,0]=(0.384713965427)/(o+1.)+(-0.522411606102)+(0.146558304882)*0.5**o
        ref[3,1,1,1]=(-0.827615582009)/(o+1.)+(-0.00295218575854)+(0.514501517225)*0.5**o
        ref[3,1,2,0]=(0.84762578765)/(o+1.)+(0.208361555685)+(-0.43502341494)*0.5**o
        ref[3,1,2,1]=(-0.554668615664)/(o+1.)+(0.825347835483)+(-0.899680107129)*0.5**o
        ref[3,2,0,0]=(0.652802712143)/(o+1.)+(0.397197582731)+(0.7807432266)*0.5**o
        ref[3,2,0,1]=(0.421495251937)/(o+1.)+(-0.119571710512)+(0.677076974975)*0.5**o
        ref[3,2,1,0]=(-0.230439630375)/(o+1.)+(-0.733165339017)+(-0.816334071941)*0.5**o
        ref[3,2,1,1]=(0.64153308098)/(o+1.)+(0.778374176432)+(-0.477170252948)*0.5**o
        ref[3,2,2,0]=(-0.987848717403)/(o+1.)+(-0.0606435888479)+(0.974551740798)*0.5**o
        ref[3,2,2,1]=(-0.318593711859)/(o+1.)+(-0.459904454409)+(0.334228278552)*0.5**o
        ref[3,3,0,0]=(-0.418115413351)/(o+1.)+(-0.348616063735)+(-0.982162917221)*0.5**o
        ref[3,3,0,1]=(-0.437684435015)/(o+1.)+(0.804145379606)+(-0.892393985366)*0.5**o
        ref[3,3,1,0]=(0.944100110135)/(o+1.)+(-0.059994562245)+(0.0694197471801)*0.5**o
        ref[3,3,1,1]=(-0.860656048613)/(o+1.)+(0.178136109535)+(-0.250126193893)*0.5**o
        ref[3,3,2,0]=(-0.363631991908)/(o+1.)+(-0.298384287282)+(0.471110648232)*0.5**o
        ref[3,3,2,1]=(-0.578021736479)/(o+1.)+(0.320913094365)+(0.235895941501)*0.5**o
        ref[3,4,0,0]=(-0.946995649946)/(o+1.)+(0.0451978887455)+(0.936359833225)*0.5**o
        ref[3,4,0,1]=(-0.636305818291)/(o+1.)+(0.258932088634)+(0.449663732234)*0.5**o
        ref[3,4,1,0]=(-0.823421937366)/(o+1.)+(0.112980674356)+(0.297559014947)*0.5**o
        ref[3,4,1,1]=(-0.441768022626)/(o+1.)+(-0.240840962617)+(-0.682298558949)*0.5**o
        ref[3,4,2,0]=(-0.258808936822)/(o+1.)+(0.510306577882)+(-0.457251344276)*0.5**o
        ref[3,4,2,1]=(-0.489365749016)/(o+1.)+(-0.296617385332)+(0.979513856171)*0.5**o
      else:
        arg[0,0,0,0]=(-0.474247989728)*x[0]+(-0.768689352235)*x[1]+(-1.08527900853)*x[2]
        arg[0,0,0,1]=(-0.237113797789)*x[0]+(0.0842735741018)*x[1]+(0.203858551029)*x[2]
        arg[0,0,1,0]=(0.24237127665)*x[0]+(0.421022955803)*x[1]+(-1.52818540927)*x[2]
        arg[0,0,1,1]=(1.34624865817)*x[0]+(-0.798943272818)*x[1]+(-0.823168386315)*x[2]
        arg[0,0,2,0]=(-0.906723771267)*x[0]+(0.328893123771)*x[1]+(-0.942770202463)*x[2]
        arg[0,0,2,1]=(1.54305197694)*x[0]+(0.813063941984)*x[1]+(0.205532727168)*x[2]
        arg[0,1,0,0]=(0.399159979872)*x[0]+(-1.52089837934)*x[1]+(-1.4064251406)*x[2]
        arg[0,1,0,1]=(-0.121139354731)*x[0]+(1.12768246422)*x[1]+(0.234299409411)*x[2]
        arg[0,1,1,0]=(-1.62513899388)*x[0]+(-0.173131700583)*x[1]+(1.24199996799)*x[2]
        arg[0,1,1,1]=(-0.371929434177)*x[0]+(-0.606278610409)*x[1]+(-0.00600790063066)*x[2]
        arg[0,1,2,0]=(-0.804531294423)*x[0]+(0.869392248002)*x[1]+(0.840417359943)*x[2]
        arg[0,1,2,1]=(0.329452946275)*x[0]+(1.1072495446)*x[1]+(-0.213517932323)*x[2]
        arg[0,2,0,0]=(1.02784887224)*x[0]+(0.435517042337)*x[1]+(1.58767206677)*x[2]
        arg[0,2,0,1]=(-0.0120997177)*x[0]+(0.430135803314)*x[1]+(-0.717473068956)*x[2]
        arg[0,2,1,0]=(0.955455517302)*x[0]+(0.712976918968)*x[1]+(-0.624804508641)*x[2]
        arg[0,2,1,1]=(-1.03293986594)*x[0]+(-0.539062834482)*x[1]+(-1.61172701428)*x[2]
        arg[0,2,2,0]=(-0.177662810138)*x[0]+(0.553352366127)*x[1]+(0.695417121111)*x[2]
        arg[0,2,2,1]=(0.318133474679)*x[0]+(-0.452528947926)*x[1]+(0.468602505091)*x[2]
        arg[0,3,0,0]=(-0.294467725916)*x[0]+(1.67314409894)*x[1]+(0.100736440007)*x[2]
        arg[0,3,0,1]=(-0.517162349887)*x[0]+(-1.57430191254)*x[1]+(1.24045877402)*x[2]
        arg[0,3,1,0]=(0.667393640368)*x[0]+(0.340993632212)*x[1]+(-1.09920535559)*x[2]
        arg[0,3,1,1]=(0.37451406727)*x[0]+(0.37350649326)*x[1]+(0.357440871322)*x[2]
        arg[0,3,2,0]=(-0.989366398194)*x[0]+(0.596179154759)*x[1]+(-0.573280308087)*x[2]
        arg[0,3,2,1]=(-0.25561190141)*x[0]+(1.42296905238)*x[1]+(-1.21032031399)*x[2]
        arg[0,4,0,0]=(-0.41394085285)*x[0]+(-0.751213929646)*x[1]+(-0.68470718293)*x[2]
        arg[0,4,0,1]=(0.702985204167)*x[0]+(-0.0375821582587)*x[1]+(0.290630803819)*x[2]
        arg[0,4,1,0]=(0.0558204653441)*x[0]+(1.61940057302)*x[1]+(-0.518863528354)*x[2]
        arg[0,4,1,1]=(1.25876303604)*x[0]+(-1.40923202908)*x[1]+(1.05202333762)*x[2]
        arg[0,4,2,0]=(0.0715295483294)*x[0]+(0.883869702529)*x[1]+(-0.463253474408)*x[2]
        arg[0,4,2,1]=(0.564563789615)*x[0]+(-0.135049742988)*x[1]+(-1.3118878567)*x[2]
        arg[1,0,0,0]=(0.707267335291)*x[0]+(-0.197244449872)*x[1]+(-0.114362773781)*x[2]
        arg[1,0,0,1]=(-0.144762815659)*x[0]+(0.434177706452)*x[1]+(0.0754859099691)*x[2]
        arg[1,0,1,0]=(-0.906934991359)*x[0]+(0.179011995627)*x[1]+(0.823936036469)*x[2]
        arg[1,0,1,1]=(1.11165724605)*x[0]+(0.48023223731)*x[1]+(0.981353808181)*x[2]
        arg[1,0,2,0]=(0.499460583481)*x[0]+(0.171586243561)*x[1]+(-0.252474775657)*x[2]
        arg[1,0,2,1]=(-0.604289370884)*x[0]+(1.11610914086)*x[1]+(0.747439062711)*x[2]
        arg[1,1,0,0]=(0.696191463882)*x[0]+(0.349547714404)*x[1]+(0.651361019042)*x[2]
        arg[1,1,0,1]=(0.152679923426)*x[0]+(-0.448442911425)*x[1]+(1.33964371501)*x[2]
        arg[1,1,1,0]=(-0.74612592015)*x[0]+(-1.27089239786)*x[1]+(-0.261725631736)*x[2]
        arg[1,1,1,1]=(1.50839493929)*x[0]+(1.93229724987)*x[1]+(-0.963996845831)*x[2]
        arg[1,1,2,0]=(0.867984447375)*x[0]+(-0.777493811854)*x[1]+(0.847847553133)*x[2]
        arg[1,1,2,1]=(1.34249389498)*x[0]+(0.776160804062)*x[1]+(-0.295393972199)*x[2]
        arg[1,2,0,0]=(-0.655771613243)*x[0]+(1.58220214455)*x[1]+(-0.406292477629)*x[2]
        arg[1,2,0,1]=(0.59294895986)*x[0]+(0.544961164723)*x[1]+(0.634279670948)*x[2]
        arg[1,2,1,0]=(1.15324179862)*x[0]+(0.645764835217)*x[1]+(0.670364725309)*x[2]
        arg[1,2,1,1]=(-0.819459106271)*x[0]+(0.024815625404)*x[1]+(0.769753650676)*x[2]
        arg[1,2,2,0]=(1.19042366161)*x[0]+(0.982020548972)*x[1]+(0.92730609738)*x[2]
        arg[1,2,2,1]=(-0.742441488349)*x[0]+(-1.3101429812)*x[1]+(-0.198104246324)*x[2]
        arg[1,3,0,0]=(0.349205030437)*x[0]+(-0.745240339596)*x[1]+(-0.744742092259)*x[2]
        arg[1,3,0,1]=(0.993101296711)*x[0]+(-1.48331355036)*x[1]+(-1.00616778677)*x[2]
        arg[1,3,1,0]=(-1.10586442824)*x[0]+(1.71922449779)*x[1]+(-0.943451153259)*x[2]
        arg[1,3,1,1]=(-1.72393763035)*x[0]+(-1.14113495956)*x[1]+(-0.211909615809)*x[2]
        arg[1,3,2,0]=(-0.378302963108)*x[0]+(-0.494705322002)*x[1]+(0.940164363332)*x[2]
        arg[1,3,2,1]=(0.850541148743)*x[0]+(-1.45461945468)*x[1]+(1.20025821328)*x[2]
        arg[1,4,0,0]=(1.33882463795)*x[0]+(0.475208133119)*x[1]+(0.722686059253)*x[2]
        arg[1,4,0,1]=(0.0750422155969)*x[0]+(1.29758288845)*x[1]+(-1.41431668738)*x[2]
        arg[1,4,1,0]=(1.36072344575)*x[0]+(-0.345743547563)*x[1]+(-0.63248018754)*x[2]
        arg[1,4,1,1]=(0.579049229913)*x[0]+(0.954245074078)*x[1]+(-0.170538926965)*x[2]
        arg[1,4,2,0]=(1.67615484634)*x[0]+(0.561067019023)*x[1]+(-1.26952326914)*x[2]
        arg[1,4,2,1]=(0.700910547934)*x[0]+(0.58717328764)*x[1]+(0.795138196965)*x[2]
        arg[2,0,0,0]=(0.763193592359)*x[0]+(-0.397549328381)*x[1]+(-0.230593559545)*x[2]
        arg[2,0,0,1]=(0.454026153468)*x[0]+(-1.12464443717)*x[1]+(-0.147462098246)*x[2]
        arg[2,0,1,0]=(-0.276457034935)*x[0]+(0.118173156654)*x[1]+(0.57296366275)*x[2]
        arg[2,0,1,1]=(1.31887812253)*x[0]+(-0.544644483254)*x[1]+(0.0013728350582)*x[2]
        arg[2,0,2,0]=(-0.257589384265)*x[0]+(-0.016005453203)*x[1]+(-0.508836430965)*x[2]
        arg[2,0,2,1]=(-0.259420301773)*x[0]+(0.122645293168)*x[1]+(0.381613885289)*x[2]
        arg[2,1,0,0]=(-1.64168512418)*x[0]+(0.025182006192)*x[1]+(0.0268780458975)*x[2]
        arg[2,1,0,1]=(-0.0885344200898)*x[0]+(0.408795219114)*x[1]+(0.635798801606)*x[2]
        arg[2,1,1,0]=(-0.959658503283)*x[0]+(1.034415556)*x[1]+(0.725681105748)*x[2]
        arg[2,1,1,1]=(1.59312707789)*x[0]+(-0.116514952101)*x[1]+(-0.86121633507)*x[2]
        arg[2,1,2,0]=(-1.44859572067)*x[0]+(-1.28344247275)*x[1]+(0.299296401869)*x[2]
        arg[2,1,2,1]=(-0.488424372501)*x[0]+(1.23481402288)*x[1]+(0.01702902463)*x[2]
        arg[2,2,0,0]=(-1.29986951244)*x[0]+(-1.20190912751)*x[1]+(-0.755848291051)*x[2]
        arg[2,2,0,1]=(-0.52421656312)*x[0]+(-0.335471509689)*x[1]+(-1.01264312499)*x[2]
        arg[2,2,1,0]=(-1.12066642609)*x[0]+(-1.02138558053)*x[1]+(-1.00777755552)*x[2]
        arg[2,2,1,1]=(-1.42127024744)*x[0]+(1.63480022479)*x[1]+(0.542244906643)*x[2]
        arg[2,2,2,0]=(-0.418852880297)*x[0]+(-0.983110436453)*x[1]+(-0.622603673805)*x[2]
        arg[2,2,2,1]=(0.727366446825)*x[0]+(-0.256324856361)*x[1]+(-0.20608740758)*x[2]
        arg[2,3,0,0]=(0.548268302883)*x[0]+(0.0102173127665)*x[1]+(0.869833576217)*x[2]
        arg[2,3,0,1]=(-1.70159091061)*x[0]+(1.15876875083)*x[1]+(-0.234757706773)*x[2]
        arg[2,3,1,0]=(-1.72817024817)*x[0]+(0.690055132218)*x[1]+(-0.149398998825)*x[2]
        arg[2,3,1,1]=(1.80829412387)*x[0]+(-0.217899654855)*x[1]+(-0.922672217822)*x[2]
        arg[2,3,2,0]=(1.23855200328)*x[0]+(0.412795411508)*x[1]+(0.960476191755)*x[2]
        arg[2,3,2,1]=(-1.43061904621)*x[0]+(-0.766039000745)*x[1]+(-1.25765397361)*x[2]
        arg[2,4,0,0]=(-0.660872016687)*x[0]+(-0.000405233718708)*x[1]+(-0.178507968219)*x[2]
        arg[2,4,0,1]=(-0.168526882871)*x[0]+(-1.06936802685)*x[1]+(1.54083857104)*x[2]
        arg[2,4,1,0]=(0.565807359529)*x[0]+(-1.41401297939)*x[1]+(-1.23254260537)*x[2]
        arg[2,4,1,1]=(-0.373920075802)*x[0]+(0.407330812903)*x[1]+(-0.204667001328)*x[2]
        arg[2,4,2,0]=(0.874100879885)*x[0]+(-0.191238094267)*x[1]+(1.54890969328)*x[2]
        arg[2,4,2,1]=(0.566428125313)*x[0]+(-1.60899595601)*x[1]+(-1.05112529768)*x[2]
        arg[3,0,0,0]=(-0.688037248111)*x[0]+(-1.56483484784)*x[1]+(0.26782504619)*x[2]
        arg[3,0,0,1]=(0.418788954459)*x[0]+(-0.670455226092)*x[1]+(-0.0562760591375)*x[2]
        arg[3,0,1,0]=(-0.2614362754)*x[0]+(0.271050519855)*x[1]+(0.227811982923)*x[2]
        arg[3,0,1,1]=(-0.873198928647)*x[0]+(-0.483763141395)*x[1]+(0.33532383979)*x[2]
        arg[3,0,2,0]=(-0.864161938176)*x[0]+(-1.4682754386)*x[1]+(-0.38126411162)*x[2]
        arg[3,0,2,1]=(0.142827521381)*x[0]+(0.464483940188)*x[1]+(0.214297576606)*x[2]
        arg[3,1,0,0]=(0.188747079113)*x[0]+(0.590821253713)*x[1]+(0.908850367108)*x[2]
        arg[3,1,0,1]=(0.0587541582624)*x[0]+(0.955494266151)*x[1]+(1.70822209849)*x[2]
        arg[3,1,1,0]=(-0.420356861523)*x[0]+(-0.551445918262)*x[1]+(-0.0187114754269)*x[2]
        arg[3,1,1,1]=(1.07071658401)*x[0]+(-0.589035602194)*x[1]+(0.133725074027)*x[2]
        arg[3,1,2,0]=(0.370477043767)*x[0]+(0.226262353618)*x[1]+(0.566087784489)*x[2]
        arg[3,1,2,1]=(1.77828660075)*x[0]+(0.607419742685)*x[1]+(0.0297054744582)*x[2]
        arg[3,2,0,0]=(-1.19326939474)*x[0]+(1.20875153887)*x[1]+(-0.124756222091)*x[2]
        arg[3,2,0,1]=(0.0645190914271)*x[0]+(0.357580475716)*x[1]+(-1.59461469718)*x[2]
        arg[3,2,1,0]=(1.4853619905)*x[0]+(0.751582301648)*x[1]+(0.859363177167)*x[2]
        arg[3,2,1,1]=(1.36658655075)*x[0]+(-0.776650996613)*x[1]+(0.654743994261)*x[2]
        arg[3,2,2,0]=(0.406696005677)*x[0]+(-0.528546876384)*x[1]+(-0.439775325396)*x[2]
        arg[3,2,2,1]=(-0.725198159461)*x[0]+(-0.0933136564416)*x[1]+(-0.720365141393)*x[2]
        arg[3,3,0,0]=(-0.428669112669)*x[0]+(0.626162511175)*x[1]+(-0.382776233923)*x[2]
        arg[3,3,0,1]=(0.340474939218)*x[0]+(-0.140550383925)*x[1]+(0.353836983746)*x[2]
        arg[3,3,1,0]=(0.851646816711)*x[0]+(0.0171684195949)*x[1]+(0.38173044322)*x[2]
        arg[3,3,1,1]=(0.359647722225)*x[0]+(0.425799941165)*x[1]+(-0.80174052212)*x[2]
        arg[3,3,2,0]=(0.13012568736)*x[0]+(-0.573735693651)*x[1]+(1.08843730088)*x[2]
        arg[3,3,2,1]=(-1.01372953294)*x[0]+(0.335282956856)*x[1]+(1.24570512628)*x[2]
        arg[3,4,0,0]=(-0.05674609039)*x[0]+(-0.814879575064)*x[1]+(-0.169789408264)*x[2]
        arg[3,4,0,1]=(1.5968230938)*x[0]+(0.0679819277023)*x[1]+(0.892284314512)*x[2]
        arg[3,4,1,0]=(1.22220298478)*x[0]+(-0.0258417926472)*x[1]+(0.440670138568)*x[2]
        arg[3,4,1,1]=(0.647072768716)*x[0]+(-0.177651087906)*x[1]+(-0.115553644646)*x[2]
        arg[3,4,2,0]=(1.29816363306)*x[0]+(-1.18656044167)*x[1]+(-0.465776533157)*x[2]
        arg[3,4,2,1]=(0.328907496091)*x[0]+(-0.179857937151)*x[1]+(-0.476306938306)*x[2]
        ref[0,0,0,0]=(-0.530378359482)/(o+1.)+(-0.935143327523)+(0.0724486640358)*0.5**o
        ref[0,0,0,1]=(1.22837847871)/(o+1.)+(-0.113350954728)+(-0.950658241915)*0.5**o
        ref[0,0,1,0]=(-0.000414683535449)/(o+1.)+(-0.183158731552)+(-0.49805903018)*0.5**o
        ref[0,0,1,1]=(-0.882730389132)/(o+1.)+(-0.112174958816)+(0.831217305802)*0.5**o
        ref[0,0,2,0]=(-1.1913557966)/(o+1.)+(-0.0615750891074)+(-0.206094875146)*0.5**o
        ref[0,0,2,1]=(0.634536492589)/(o+1.)+(0.649890059464)+(0.627332034578)*0.5**o
        ref[0,1,0,0]=(-1.38665625775)/(o+1.)+(-1.0210653524)+(0.900623422471)*0.5**o
        ref[0,1,0,1]=(1.20633678024)/(o+1.)+(0.0888193121856)+(-0.143132885707)*0.5**o
        ref[0,1,1,0]=(0.452225278035)/(o+1.)+(-0.148926943801)+(-0.710642116901)*0.5**o
        ref[0,1,1,1]=(-0.244223430204)/(o+1.)+(-0.619607191996)+(0.499221868978)*0.5**o
        ref[0,1,2,0]=(0.0383849036606)/(o+1.)+(0.759484971644)+(-0.652076533427)*0.5**o
        ref[0,1,2,1]=(0.207558055759)/(o+1.)+(0.502305382384)+(0.0110157380215)*0.5**o
        ref[0,2,0,0]=(0.761374280144)/(o+1.)+(0.777616611515)+(0.734430478174)*0.5**o
        ref[0,2,0,1]=(0.554502862028)/(o+1.)+(-0.409274602016)+(-0.0353906413371)*0.5**o
        ref[0,2,1,0]=(-0.252543841709)/(o+1.)+(0.386640377894)+(0.522891013551)*0.5**o
        ref[0,2,1,1]=(-1.52497235887)/(o+1.)+(-0.807603254401)+(-0.0435508470367)*0.5**o
        ref[0,2,2,0]=(0.172485606257)/(o+1.)+(0.350297526187)+(0.19802601847)*0.5**o
        ref[0,2,2,1]=(-0.141271359379)/(o+1.)+(0.0509198134622)+(0.373638764298)*0.5**o
        ref[0,3,0,0]=(1.31278836809)/(o+1.)+(-0.0918739323256)+(0.350372309595)*0.5**o
        ref[0,3,0,1]=(-0.396933928109)/(o+1.)+(-0.436137805999)+(0.418204051696)*0.5**o
        ref[0,3,1,0]=(-0.206619840099)/(o+1.)+(-0.118703289798)+(0.353208336689)*0.5**o
        ref[0,3,1,1]=(-0.595153371293)/(o+1.)+(0.615722581562)+(0.469169640021)*0.5**o
        ref[0,3,2,0]=(-0.680858997986)/(o+1.)+(-0.0393223937964)+(-0.206963765944)*0.5**o
        ref[0,3,2,1]=(0.012753264598)/(o+1.)+(0.246188741489)+(-0.548093910595)*0.5**o
        ref[0,4,0,0]=(-0.40478987122)/(o+1.)+(-0.890399681609)+(0.33572726901)*0.5**o
        ref[0,4,0,1]=(0.0280062557264)/(o+1.)+(0.0358049413228)+(0.856417711355)*0.5**o
        ref[0,4,1,0]=(0.181560004565)/(o+1.)+(0.329885369854)+(0.31502676574)*0.5**o
        ref[0,4,1,1]=(0.0552681276841)/(o+1.)+(0.00372919426319)+(0.838827828375)*0.5**o
        ref[0,4,2,0]=(0.656057162724)/(o+1.)+(0.179315527684)+(-0.522542441641)*0.5**o
        ref[0,4,2,1]=(0.126189389317)/(o+1.)+(-0.429452001643)+(-0.149659196108)*0.5**o
        ref[1,0,0,0]=(1.43443485527)/(o+1.)+(-0.55091754491)+(0.0630603461901)*0.5**o
        ref[1,0,0,1]=(-0.612191779306)/(o+1.)+(0.604025235381)+(-0.230957890695)*0.5**o
        ref[1,0,1,0]=(0.373085423681)/(o+1.)+(-0.0817391530697)+(-0.113594076805)*0.5**o
        ref[1,0,1,1]=(0.761862677657)/(o+1.)+(0.819524484428)+(0.172331645033)*0.5**o
        ref[1,0,2,0]=(0.929265450746)/(o+1.)+(-0.382585784938)+(0.254478170515)*0.5**o
        ref[1,0,2,1]=(1.58236203463)/(o+1.)+(-0.280641188811)+(0.238179175679)*0.5**o
        ref[1,1,0,0]=(-0.117055726312)/(o+1.)+(0.957914090571)+(-0.101672257501)*0.5**o
        ref[1,1,0,1]=(0.0849062553796)/(o+1.)+(0.0703564566846)+(0.818261558262)*0.5**o
        ref[1,1,1,0]=(-1.72219113622)/(o+1.)+(-0.357454724939)+(0.15835663635)*0.5**o
        ref[1,1,1,1]=(0.739078400468)/(o+1.)+(0.532292543169)+(0.673031856531)*0.5**o
        ref[1,1,2,0]=(0.255023909903)/(o+1.)+(0.229656917655)+(0.22400044344)*0.5**o
        ref[1,1,2,1]=(0.518856765199)/(o+1.)+(0.380552790125)+(0.543298381396)*0.5**o
        ref[1,2,0,0]=(0.0321563884831)/(o+1.)+(0.25981567762)+(-0.0316496900482)*0.5**o
        ref[1,2,0,1]=(1.09318176771)/(o+1.)+(0.0572442907603)+(0.564519446305)*0.5**o
        ref[1,2,1,0]=(0.756962791387)/(o+1.)+(0.69419429624)+(0.32401997528)*0.5**o
        ref[1,2,1,1]=(1.28480299241)/(o+1.)+(-0.201990593541)+(-0.905711635518)*0.5**o
        ref[1,2,2,0]=(1.04969566442)/(o+1.)+(0.72854097372)+(0.592972696101)*0.5**o
        ref[1,2,2,1]=(-1.77086264775)/(o+1.)+(-0.261914180361)+(0.0440022926066)*0.5**o
        ref[1,3,0,0]=(-0.477780531124)/(o+1.)+(-0.181354085163)+(-0.300288699967)*0.5**o
        ref[1,3,0,1]=(-0.797627013456)/(o+1.)+(-0.523163955197)+(0.347574883435)*0.5**o
        ref[1,3,1,0]=(0.428806106821)/(o+1.)+(-0.0514584302394)+(-0.655980330052)*0.5**o
        ref[1,3,1,1]=(-0.141740459457)/(o+1.)+(-1.05699960931)+(-0.821242527644)*0.5**o
        ref[1,3,2,0]=(0.0181810985873)/(o+1.)+(-0.180697139553)+(0.41036925874)*0.5**o
        ref[1,3,2,1]=(-0.061077367)/(o+1.)+(0.250077446763)+(0.157102380812)*0.5**o
        ref[1,4,0,0]=(1.14087212913)/(o+1.)+(0.368059029864)+(0.659728641463)*0.5**o
        ref[1,4,0,1]=(0.365658713316)/(o+1.)+(0.088533821044)+(-0.584417938734)*0.5**o
        ref[1,4,1,0]=(-1.11681308321)/(o+1.)+(0.438275560627)+(0.622761672604)*0.5**o
        ref[1,4,1,1]=(1.18893372018)/(o+1.)+(0.172934240923)+(-0.172046824999)*0.5**o
        ref[1,4,2,0]=(0.16240489493)/(o+1.)+(-0.0145923852256)+(0.834478471749)*0.5**o
        ref[1,4,2,1]=(0.571355829378)/(o+1.)+(0.526896417749)+(0.458073367663)*0.5**o
        ref[2,0,0,0]=(-0.489234572937)/(o+1.)+(-0.0135589093901)+(0.65140309615)*0.5**o
        ref[2,0,0,1]=(-1.55383491931)/(o+1.)+(0.0161644619972)+(0.703425613366)*0.5**o
        ref[2,0,1,0]=(-0.258604855723)/(o+1.)+(0.0827812678671)+(0.507722104457)*0.5**o
        ref[2,0,1,1]=(0.0504178529481)/(o+1.)+(0.182991850332)+(0.359204920726)*0.5**o
        ref[2,0,2,0]=(0.652043479841)/(o+1.)+(-0.443015450013)+(-0.548443848248)*0.5**o
        ref[2,0,2,1]=(0.340317839857)/(o+1.)+(-0.379565594545)+(0.663652225917)*0.5**o
        ref[2,1,0,0]=(-0.587863196667)/(o+1.)+(-0.0137797674284)+(-0.974202340562)*0.5**o
        ref[2,1,0,1]=(1.00030747039)/(o+1.)+(-0.399997754768)+(0.755747639771)*0.5**o
        ref[2,1,1,0]=(0.703413281036)/(o+1.)+(0.487016208736)+(-0.877007540039)*0.5**o
        ref[2,1,1,1]=(0.263166830572)/(o+1.)+(-0.157259452849)+(0.666747865841)*0.5**o
        ref[2,1,2,0]=(0.104532343004)/(o+1.)+(-1.02548001493)+(-0.486314104701)*0.5**o
        ref[2,1,2,1]=(0.933518655478)/(o+1.)+(0.0369928229139)+(-0.244085626298)*0.5**o
        ref[2,2,0,0]=(-1.50550384519)/(o+1.)+(-0.590996227594)+(-0.570130630622)*0.5**o
        ref[2,2,0,1]=(-0.60318683839)/(o+1.)+(-0.38308245528)+(-0.502979448853)*0.5**o
        ref[2,2,1,0]=(-1.11258910879)/(o+1.)+(-0.518777277969)+(-0.999685897413)*0.5**o
        ref[2,2,1,1]=(1.84366617107)/(o+1.)+(-0.232866738261)+(-0.622157810558)*0.5**o
        ref[2,2,2,0]=(-0.620181988585)/(o+1.)+(-0.836162444183)+(0.267939886396)*0.5**o
        ref[2,2,2,1]=(-0.826647389099)/(o+1.)+(0.0929536518109)+(0.905694268362)*0.5**o
        ref[2,3,0,0]=(0.184067674443)/(o+1.)+(0.259745858757)+(0.724759799909)*0.5**o
        ref[2,3,0,1]=(0.0175595303049)/(o+1.)+(0.0560117358912)+(-0.907162868639)*0.5**o
        ref[2,3,1,0]=(-0.609443154617)/(o+1.)+(0.186540819401)+(-0.951152598958)*0.5**o
        ref[2,3,1,1]=(-1.61858082094)/(o+1.)+(0.679439858578)+(0.927423354971)*0.5**o
        ref[2,3,2,0]=(0.44350652486)/(o+1.)+(0.81466177481)+(0.53899353207)*0.5**o
        ref[2,3,2,1]=(-0.877289862245)/(o+1.)+(-1.05742260849)+(-0.462176941342)*0.5**o
        ref[2,4,0,0]=(-0.736381416539)/(o+1.)+(-0.0139374259612)+(-0.0755289501641)*0.5**o
        ref[2,4,0,1]=(0.690429326999)/(o+1.)+(-0.0293644018251)+(-0.328756862036)*0.5**o
        ref[2,4,1,0]=(-1.57117034002)/(o+1.)+(-0.531463765387)+(0.553349645561)*0.5**o
        ref[2,4,1,1]=(1.24898175282)/(o+1.)+(-0.452273024959)+(-0.515691967125)*0.5**o
        ref[2,4,2,0]=(0.239126191643)/(o+1.)+(0.722504563932)+(0.547637159392)*0.5**o
        ref[2,4,2,1]=(-1.40763254209)/(o+1.)+(-0.809456441012)+(0.932852295739)*0.5**o
        ref[3,0,0,0]=(-0.416554297686)/(o+1.)+(-0.632193610911)+(-0.304105530258)*0.5**o
        ref[3,0,0,1]=(-1.02983448036)/(o+1.)+(0.579212231778)+(-0.436532313966)*0.5**o
        ref[3,0,1,0]=(0.42124133216)/(o+1.)+(0.368290418827)+(-0.920395942436)*0.5**o
        ref[3,0,1,1]=(0.158195474854)/(o+1.)+(-0.17459474514)+(-0.830644214826)*0.5**o
        ref[3,0,2,0]=(-1.18749647329)/(o+1.)+(-0.506173996178)+(-0.513857022748)*0.5**o
        ref[3,0,2,1]=(0.233437753018)/(o+1.)+(0.455705145371)+(-0.323239005586)*0.5**o
        ref[3,1,0,0]=(1.42991958566)/(o+1.)+(0.053769921022)+(0.150959272234)*0.5**o
        ref[3,1,0,1]=(1.48423731111)/(o+1.)+(0.964048315997)+(-0.689863420196)*0.5**o
        ref[3,1,1,0]=(0.868837676765)/(o+1.)+(-1.01261543254)+(0.165878933098)*0.5**o
        ref[3,1,1,1]=(0.820750307831)/(o+1.)+(-0.300930223821)+(0.396516195659)*0.5**o
        ref[3,1,2,0]=(0.477077782202)/(o+1.)+(-0.0416336989494)+(0.769016797571)*0.5**o
        ref[3,1,2,1]=(0.359895440587)/(o+1.)+(0.555322395811)+(0.944871585683)*0.5**o
        ref[3,2,0,0]=(0.657668315832)/(o+1.)+(-0.240321895174)+(-0.286298603451)*0.5**o
        ref[3,2,0,1]=(-0.941360568282)/(o+1.)+(-0.0246544136593)+(-0.181845734439)*0.5**o
        ref[3,2,1,0]=(1.60316834971)/(o+1.)+(0.252291602298)+(0.988555915005)*0.5**o
        ref[3,2,1,1]=(0.833811957678)/(o+1.)+(-0.111970621871)+(0.634808834458)*0.5**o
        ref[3,2,2,0]=(-1.38877073784)/(o+1.)+(0.538356636307)+(-0.249568730877)*0.5**o
        ref[3,2,2,1]=(-0.311839184097)/(o+1.)+(-0.698114103264)+(0.169190433329)*0.5**o
        ref[3,3,0,0]=(0.743170989696)/(o+1.)+(-0.499224746788)+(0.0699956684628)*0.5**o
        ref[3,3,0,1]=(-0.293538786051)/(o+1.)+(0.26667992866)+(0.313940467768)*0.5**o
        ref[3,3,1,0]=(0.951483461345)/(o+1.)+(0.148317904561)+(0.00242640905887)*0.5**o
        ref[3,3,1,1]=(-0.110753912776)/(o+1.)+(-0.44308689138)+(0.980634836805)*0.5**o
        ref[3,3,2,0]=(-0.337895223544)/(o+1.)+(0.358941646037)+(0.264839226057)*0.5**o
        ref[3,3,2,1]=(0.857823611315)/(o+1.)+(0.30404611512)+(-0.898657291359)*0.5**o
        ref[3,4,0,0]=(0.037944509376)/(o+1.)+(-0.364801700413)+(-0.349756182268)*0.5**o
        ref[3,4,0,1]=(1.01114399396)/(o+1.)+(0.457103279485)+(0.631738783076)*0.5**o
        ref[3,4,1,0]=(0.969931913888)/(o+1.)+(-0.165922085756)+(0.998943588321)*0.5**o
        ref[3,4,1,1]=(1.35441114248)/(o+1.)+(-0.502422727199)+(0.00430234808496)*0.5**o
        ref[3,4,2,0]=(-0.367201097566)/(o+1.)+(-0.337780780531)+(0.688589316866)*0.5**o
        ref[3,4,2,1]=(-0.0547905846061)/(o+1.)+(-0.293870420685)+(0.31527404661)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactOne_fromData_ReducedContinuousFunction_rank0(self):
      """
      tests integral of rank 0 Data on the ReducedFunctionOnContactOne

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(),w)
      if dim==2:
        arg=(0.241972460279)*x[0]+(0.539729925673)*x[1]
        ref=(-0.11832904353)/(o+1.)+(0.649911364492)+(-0.399791299503)*0.5**o
      else:
        arg=(0.53919797656)*x[0]+(0.379010664884)*x[1]+(0.274416064529)*x[2]
        ref=(0.307947301472)/(o+1.)+(0.120674157748)+(0.643329089005)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactOne_fromData_ReducedContinuousFunction_rank1(self):
      """
      tests integral of rank 1 Data on the ReducedFunctionOnContactOne

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(2,),w)
      ref=numpy.zeros((2,),numpy.float_)
      if dim==2:
        arg[0]=(0.84849017056)*x[0]+(-0.161962845202)*x[1]
        arg[1]=(-0.794717919135)*x[0]+(1.16003773358)*x[1]
        ref[0]=(0.780514448202)/(o+1.)+(-0.34628958021)+(0.598592037576)*0.5**o
        ref[1]=(0.570158156278)/(o+1.)+(0.241192316704)+(-0.687222975237)*0.5**o
      else:
        arg[0]=(-0.637352163634)*x[0]+(1.49477462643)*x[1]+(-0.284018740238)*x[2]
        arg[1]=(0.125937459904)*x[0]+(-0.206499869891)*x[1]+(-0.342856240345)*x[2]
        ref[0]=(-0.274486314451)/(o+1.)+(0.379737553853)+(0.0884149293071)*0.5**o
        ref[1]=(-0.78452814021)/(o+1.)+(-0.122017410137)+(0.605144310151)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactOne_fromData_ReducedContinuousFunction_rank2(self):
      """
      tests integral of rank 2 Data on the ReducedFunctionOnContactOne

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5),w)
      ref=numpy.zeros((4, 5),numpy.float_)
      if dim==2:
        arg[0,0]=(-0.440868762581)*x[0]+(-0.216223666616)*x[1]
        arg[0,1]=(-1.78807355591)*x[0]+(-0.208278123838)*x[1]
        arg[0,2]=(-0.448066998103)*x[0]+(-0.819815507826)*x[1]
        arg[0,3]=(1.27247881689)*x[0]+(0.0653439567221)*x[1]
        arg[0,4]=(-1.00344990223)*x[0]+(-0.291263790583)*x[1]
        arg[1,0]=(0.440080423838)*x[0]+(0.0356836558113)*x[1]
        arg[1,1]=(-1.49545340091)*x[0]+(-0.133615590211)*x[1]
        arg[1,2]=(1.05947638385)*x[0]+(0.408685904238)*x[1]
        arg[1,3]=(-0.170309086605)*x[0]+(0.660523970285)*x[1]
        arg[1,4]=(0.764323678475)*x[0]+(-0.529415080303)*x[1]
        arg[2,0]=(0.55270081325)*x[0]+(-1.60630146231)*x[1]
        arg[2,1]=(1.57419330248)*x[0]+(-1.24644288706)*x[1]
        arg[2,2]=(0.0416757238363)*x[0]+(0.0247379738465)*x[1]
        arg[2,3]=(1.0200370393)*x[0]+(0.364209137693)*x[1]
        arg[2,4]=(-1.21858490389)*x[0]+(0.523056254546)*x[1]
        arg[3,0]=(0.0524205820842)*x[0]+(1.65953217854)*x[1]
        arg[3,1]=(1.21824883395)*x[0]+(0.905190834998)*x[1]
        arg[3,2]=(0.226344434525)*x[0]+(-0.735349531462)*x[1]
        arg[3,3]=(-1.00295245717)*x[0]+(-0.719605376499)*x[1]
        arg[3,4]=(-0.446228759693)*x[0]+(1.23441974277)*x[1]
        ref[0,0]=(0.347328582602)/(o+1.)+(-0.520484463519)+(0.0365479152384)*0.5**o
        ref[0,1]=(-0.274564442352)/(o+1.)+(-0.413557210678)+(-0.894672816039)*0.5**o
        ref[0,2]=(0.0720644688954)/(o+1.)+(-0.906636966188)+(0.473326957552)*0.5**o
        ref[0,3]=(-0.747749341777)/(o+1.)+(0.730249972378)+(0.625072170633)*0.5**o
        ref[0,4]=(-0.194031116169)/(o+1.)+(-0.432063325829)+(-0.236555924983)*0.5**o
        ref[1,0]=(0.807081538033)/(o+1.)+(-0.612266650007)+(0.893215841629)*0.5**o
        ref[1,1]=(-0.662516518957)/(o+1.)+(-0.176318925886)+(-0.61391462039)*0.5**o
        ref[1,2]=(-0.570345636156)/(o+1.)+(0.791877954963)+(0.454752014319)*0.5**o
        ref[1,3]=(-0.312400910795)/(o+1.)+(0.212935137466)+(0.376745519544)*0.5**o
        ref[1,4]=(-0.764680414264)/(o+1.)+(0.174215486579)+(0.651158039278)*0.5**o
        ref[2,0]=(-0.837942771642)/(o+1.)+(-0.187938486702)+(0.160219095985)*0.5**o
        ref[2,1]=(-0.672421039761)/(o+1.)+(0.17384002882)+(0.652491397549)*0.5**o
        ref[2,2]=(-0.656187949922)/(o+1.)+(0.559888736121)+(-0.397175824637)*0.5**o
        ref[2,3]=(0.259558368195)/(o+1.)+(0.55148644978)+(0.0217149092406)*0.5**o
        ref[2,4]=(-0.188397537901)/(o+1.)+(-0.0561706170512)+(-0.394789877345)*0.5**o
        ref[3,0]=(0.91055628617)/(o+1.)+(0.574211274098)+(-0.347026073739)*0.5**o
        ref[3,1]=(0.29225509237)/(o+1.)+(0.64447705482)+(0.542230466941)*0.5**o
        ref[3,2]=(0.121065624384)/(o+1.)+(0.0316495277836)+(-0.693369776888)*0.5**o
        ref[3,3]=(-0.943523334976)/(o+1.)+(-0.266302644031)+(-0.246429210629)*0.5**o
        ref[3,4]=(0.714915399455)/(o+1.)+(0.459963131994)+(-0.846650680369)*0.5**o
      else:
        arg[0,0]=(-0.493869713454)*x[0]+(-0.381969413515)*x[1]+(1.59862368848)*x[2]
        arg[0,1]=(-0.581245543547)*x[0]+(0.961117271889)*x[1]+(-0.158073515343)*x[2]
        arg[0,2]=(-0.635283143641)*x[0]+(0.695573236987)*x[1]+(-0.719180832278)*x[2]
        arg[0,3]=(-0.737780596733)*x[0]+(-1.11171283352)*x[1]+(0.459954346315)*x[2]
        arg[0,4]=(1.12522065074)*x[0]+(-1.08427904854)*x[1]+(-0.715679673556)*x[2]
        arg[1,0]=(0.972468186181)*x[0]+(-0.211014530435)*x[1]+(-0.734414099002)*x[2]
        arg[1,1]=(-1.67337704748)*x[0]+(0.871096213089)*x[1]+(1.51063313223)*x[2]
        arg[1,2]=(-0.222013007834)*x[0]+(1.46474508326)*x[1]+(-0.979243908862)*x[2]
        arg[1,3]=(1.21195281087)*x[0]+(0.274082389329)*x[1]+(1.56144436465)*x[2]
        arg[1,4]=(1.35812865884)*x[0]+(1.25819798853)*x[1]+(0.895334048395)*x[2]
        arg[2,0]=(-0.167404344437)*x[0]+(0.0710591934315)*x[1]+(-0.124469683665)*x[2]
        arg[2,1]=(-0.492240632996)*x[0]+(-0.317251117382)*x[1]+(0.0195461703529)*x[2]
        arg[2,2]=(-0.277590266225)*x[0]+(-1.65859476899)*x[1]+(-0.0100216405668)*x[2]
        arg[2,3]=(-0.0701281298455)*x[0]+(0.502137254703)*x[1]+(0.701220923443)*x[2]
        arg[2,4]=(-0.72186096407)*x[0]+(-0.900487568179)*x[1]+(-0.735573922826)*x[2]
        arg[3,0]=(-0.0677324474129)*x[0]+(-0.649963209718)*x[1]+(-0.770861099726)*x[2]
        arg[3,1]=(0.563588189481)*x[0]+(0.714614932251)*x[1]+(-0.0957764632886)*x[2]
        arg[3,2]=(1.13337395062)*x[0]+(0.507556316073)*x[1]+(-0.196759085949)*x[2]
        arg[3,3]=(-1.2636883168)*x[0]+(-0.811660344076)*x[1]+(0.645462052598)*x[2]
        arg[3,4]=(-0.104175186987)*x[0]+(-0.929176038155)*x[1]+(1.07530379856)*x[2]
        ref[0,0]=(0.84538674973)/(o+1.)+(0.0569713556632)+(-0.236544899547)*0.5**o
        ref[0,1]=(-0.307689589699)/(o+1.)+(0.245040337642)+(0.0394071274131)*0.5**o
        ref[0,2]=(0.955244736357)/(o+1.)+(-0.341771136504)+(-0.930593202281)*0.5**o
        ref[0,3]=(-0.353490461722)/(o+1.)+(-0.476228433901)+(-0.083591754415)*0.5**o
        ref[0,4]=(-1.31220485482)/(o+1.)+(0.0466426656676)+(0.544181452128)*0.5**o
        ref[1,0]=(0.22773475414)/(o+1.)+(-0.488454801654)+(0.776214405914)*0.5**o
        ref[1,1]=(1.50688678707)/(o+1.)+(0.0851415404695)+(-0.96881757017)*0.5**o
        ref[1,2]=(0.313471097102)/(o+1.)+(-0.0875594518156)+(0.125135973096)*0.5**o
        ref[1,3]=(0.269521220179)/(o+1.)+(1.1192007195)+(0.539556905661)*0.5**o
        ref[1,4]=(1.91499876064)/(o+1.)+(0.509063797893)+(0.578534339343)*0.5**o
        ref[2,0]=(0.941352651562)/(o+1.)+(-0.966199402186)+(0.770231318139)*0.5**o
        ref[2,1]=(-0.141490823562)/(o+1.)+(-0.20919987022)+(-0.230055016023)*0.5**o
        ref[2,2]=(-0.916148047971)/(o+1.)+(-0.371740600192)+(-0.286577427426)*0.5**o
        ref[2,3]=(0.229516219331)/(o+1.)+(0.274790532736)+(0.354132763498)*0.5**o
        ref[2,4]=(-1.19120612619)/(o+1.)+(-0.139859859385)+(-0.88699661011)*0.5**o
        ref[3,0]=(-1.48630121465)/(o+1.)+(0.0744166251063)+(-0.151088792417)*0.5**o
        ref[3,1]=(-0.335583560513)/(o+1.)+(0.277436410841)+(0.963137397276)*0.5**o
        ref[3,2]=(-0.673942688644)/(o+1.)+(0.70077509905)+(0.716563671283)*0.5**o
        ref[3,3]=(-0.148394092795)/(o+1.)+(-0.237190239072)+(-0.807112037336)*0.5**o
        ref[3,4]=(0.529983886211)/(o+1.)+(-0.371055083588)+(0.254078854382)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactOne_fromData_ReducedContinuousFunction_rank3(self):
      """
      tests integral of rank 3 Data on the ReducedFunctionOnContactOne

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(6, 2, 2),w)
      ref=numpy.zeros((6, 2, 2),numpy.float_)
      if dim==2:
        arg[0,0,0]=(0.661725715165)*x[0]+(0.219720717581)*x[1]
        arg[0,0,1]=(-0.642240082304)*x[0]+(-0.279652756776)*x[1]
        arg[0,1,0]=(0.0204235813151)*x[0]+(-0.374873249933)*x[1]
        arg[0,1,1]=(1.4418481252)*x[0]+(0.205656834772)*x[1]
        arg[1,0,0]=(0.65992179951)*x[0]+(-1.04887875458)*x[1]
        arg[1,0,1]=(-0.195480216199)*x[0]+(-0.373905275039)*x[1]
        arg[1,1,0]=(0.781251543352)*x[0]+(0.0416248109536)*x[1]
        arg[1,1,1]=(-0.925751319189)*x[0]+(-0.113992910046)*x[1]
        arg[2,0,0]=(-0.0767414065227)*x[0]+(-0.156682840973)*x[1]
        arg[2,0,1]=(0.688457272529)*x[0]+(0.524916336046)*x[1]
        arg[2,1,0]=(0.753722830084)*x[0]+(-0.890325839466)*x[1]
        arg[2,1,1]=(0.730789290145)*x[0]+(-0.278986703716)*x[1]
        arg[3,0,0]=(-0.287376796439)*x[0]+(-0.153987233224)*x[1]
        arg[3,0,1]=(-1.15687435255)*x[0]+(0.250005890238)*x[1]
        arg[3,1,0]=(-0.845312830629)*x[0]+(0.256211962352)*x[1]
        arg[3,1,1]=(-0.574311820519)*x[0]+(-0.440297035998)*x[1]
        arg[4,0,0]=(-0.360607647308)*x[0]+(-1.29380437557)*x[1]
        arg[4,0,1]=(1.32019433485)*x[0]+(0.295255956618)*x[1]
        arg[4,1,0]=(-1.64162941972)*x[0]+(0.117482372162)*x[1]
        arg[4,1,1]=(-0.303103646619)*x[0]+(-0.88458982681)*x[1]
        arg[5,0,0]=(-0.666761879038)*x[0]+(-0.735786278877)*x[1]
        arg[5,0,1]=(0.0892896382617)*x[0]+(-0.45478872773)*x[1]
        arg[5,1,0]=(-0.50985624415)*x[0]+(-0.400022939567)*x[1]
        arg[5,1,1]=(1.39240706368)*x[0]+(0.0149740822664)*x[1]
        ref[0,0,0]=(-0.114834538488)/(o+1.)+(0.240621006484)+(0.515038958264)*0.5**o
        ref[0,0,1]=(0.588698337077)/(o+1.)+(-0.702256377233)+(-0.10607842169)*0.5**o
        ref[0,1,0]=(0.13849818224)/(o+1.)+(-0.277555277991)+(0.0621627051247)*0.5**o
        ref[0,1,1]=(-0.0878009521163)/(o+1.)+(0.630168348947)+(0.474969214192)*0.5**o
        ref[1,0,0]=(-0.696600538111)/(o+1.)+(-0.343318260912)+(0.994280104861)*0.5**o
        ref[1,0,1]=(-0.0887026688527)/(o+1.)+(-0.385753227979)+(0.290823633572)*0.5**o
        ref[1,1,0]=(0.831408608956)/(o+1.)+(-0.242985671853)+(0.477439089057)*0.5**o
        ref[1,1,1]=(-0.621059053191)/(o+1.)+(-0.0700203481241)+(-0.278644479797)*0.5**o
        ref[2,0,0]=(0.7890720399)/(o+1.)+(-0.539721196393)+(0.0569461053905)*0.5**o
        ref[2,0,1]=(-0.228628151401)/(o+1.)+(0.826946074171)+(-0.211890388366)*0.5**o
        ref[2,1,0]=(-0.837617325493)/(o+1.)+(0.289656653751)+(0.121701008608)*0.5**o
        ref[2,1,1]=(0.246577988625)/(o+1.)+(0.0512227634376)+(0.102779070928)*0.5**o
        ref[3,0,0]=(-0.0109584684212)/(o+1.)+(-0.334888666171)+(0.239371771102)*0.5**o
        ref[3,0,1]=(-0.125987676419)/(o+1.)+(-0.170179004032)+(-0.440522777827)*0.5**o
        ref[3,1,0]=(-0.354093141636)/(o+1.)+(-0.177294275919)+(0.119580825198)*0.5**o
        ref[3,1,1]=(-0.746761075412)/(o+1.)+(0.192068760731)+(-0.651985302566)*0.5**o
        ref[4,0,0]=(-0.42411673219)/(o+1.)+(-0.579162944937)+(-0.0719694008158)*0.5**o
        ref[4,0,1]=(0.169881169584)/(o+1.)+(0.483614029132)+(0.478341063616)*0.5**o
        ref[4,1,0]=(0.943567836264)/(o+1.)+(-0.898782941264)+(-0.670149001297)*0.5**o
        ref[4,1,1]=(-0.703610807936)/(o+1.)+(0.225342628657)+(-0.934767922806)*0.5**o
        ref[5,0,0]=(-0.788792225213)/(o+1.)+(-0.1768857093)+(-0.259984514102)*0.5**o
        ref[5,0,1]=(-0.788708879899)/(o+1.)+(-0.202288491141)+(0.827786772711)*0.5**o
        ref[5,1,0]=(-0.823081809707)/(o+1.)+(0.274720963093)+(-0.636239300197)*0.5**o
        ref[5,1,1]=(0.498922549807)/(o+1.)+(0.0129388257746)+(0.882580944588)*0.5**o
      else:
        arg[0,0,0]=(-0.426613654844)*x[0]+(-1.01433980801)*x[1]+(0.564621874916)*x[2]
        arg[0,0,1]=(-0.438521968882)*x[0]+(1.6033775267)*x[1]+(0.691349995832)*x[2]
        arg[0,1,0]=(-0.04337513731)*x[0]+(0.142927668234)*x[1]+(0.506581723199)*x[2]
        arg[0,1,1]=(-0.120598868162)*x[0]+(-0.213666787813)*x[1]+(-1.04221679332)*x[2]
        arg[1,0,0]=(0.153696490712)*x[0]+(-0.122516529815)*x[1]+(-1.08837272712)*x[2]
        arg[1,0,1]=(0.410706464237)*x[0]+(0.720079363315)*x[1]+(-0.767790357539)*x[2]
        arg[1,1,0]=(-1.03726428173)*x[0]+(-0.316809853003)*x[1]+(-0.780405868082)*x[2]
        arg[1,1,1]=(-0.427547967216)*x[0]+(-0.897766643609)*x[1]+(-1.56913552404)*x[2]
        arg[2,0,0]=(-0.282789248898)*x[0]+(-0.110846936031)*x[1]+(-1.4329391906)*x[2]
        arg[2,0,1]=(1.46866588617)*x[0]+(-1.23022501959)*x[1]+(-0.3123390018)*x[2]
        arg[2,1,0]=(-0.314115718621)*x[0]+(1.7333934438)*x[1]+(0.833572700726)*x[2]
        arg[2,1,1]=(0.464246462638)*x[0]+(0.267484068646)*x[1]+(0.342009204787)*x[2]
        arg[3,0,0]=(0.578853099163)*x[0]+(-0.0453855508465)*x[1]+(-0.0751578475516)*x[2]
        arg[3,0,1]=(-0.526734485752)*x[0]+(-1.03286310016)*x[1]+(-0.398986583917)*x[2]
        arg[3,1,0]=(0.0239088277798)*x[0]+(0.205105470192)*x[1]+(-1.04986349166)*x[2]
        arg[3,1,1]=(0.130893744001)*x[0]+(-0.273702140669)*x[1]+(-0.0804222415128)*x[2]
        arg[4,0,0]=(-0.822661618584)*x[0]+(-0.267189204519)*x[1]+(-0.290013766971)*x[2]
        arg[4,0,1]=(-1.46759823734)*x[0]+(0.317465945316)*x[1]+(0.108284317752)*x[2]
        arg[4,1,0]=(-1.3920990386)*x[0]+(-1.17823778694)*x[1]+(-0.18366434307)*x[2]
        arg[4,1,1]=(-0.0915496209516)*x[0]+(-0.367624609832)*x[1]+(1.30630257323)*x[2]
        arg[5,0,0]=(0.117840830455)*x[0]+(-0.667875433194)*x[1]+(-1.59716688455)*x[2]
        arg[5,0,1]=(-0.818693227605)*x[0]+(0.483842075248)*x[1]+(-0.408373205535)*x[2]
        arg[5,1,0]=(1.36906770301)*x[0]+(0.494926179999)*x[1]+(0.717041707262)*x[2]
        arg[5,1,1]=(0.819757116081)*x[0]+(0.0982458749015)*x[1]+(0.275080391599)*x[2]
        ref[0,0,0]=(0.236560699949)/(o+1.)+(-0.392784388084)+(-0.327323511722)*0.5**o
        ref[0,0,1]=(1.32491522045)/(o+1.)+(0.525146863711)+(-0.519003394222)*0.5**o
        ref[0,1,0]=(0.433749865563)/(o+1.)+(0.23007126008)+(-0.287758131599)*0.5**o
        ref[0,1,1]=(-0.0563021060686)/(o+1.)+(-0.403921160703)+(-0.512338021825)*0.5**o
        ref[1,0,0]=(-0.170496965963)/(o+1.)+(-0.914421777259)+(0.942147754258)*0.5**o
        ref[1,0,1]=(-0.372707483726)/(o+1.)+(0.636041158872)+(-0.536379364005)*0.5**o
        ref[1,1,0]=(-0.211529878364)/(o+1.)+(-0.912151985776)+(-0.0986461529033)*0.5**o
        ref[1,1,1]=(-1.32533998437)/(o+1.)+(-0.685030198386)+(-0.19904975372)*0.5**o
        ref[2,0,0]=(0.152048902469)/(o+1.)+(-0.88880590209)+(-0.201012473813)*0.5**o
        ref[2,0,1]=(-0.748900052246)/(o+1.)+(-0.0226120471998)+(0.720226011425)*0.5**o
        ref[2,1,0]=(0.77797707941)/(o+1.)+(1.01635055442)+(-0.557827762351)*0.5**o
        ref[2,1,1]=(1.08969334716)/(o+1.)+(-0.404644728844)+(0.793335846595)*0.5**o
        ref[3,0,0]=(1.56498448669)/(o+1.)+(-0.699967188685)+(0.293259591449)*0.5**o
        ref[3,0,1]=(-0.620133427128)/(o+1.)+(-0.550037413688)+(-0.238375915329)*0.5**o
        ref[3,1,0]=(-0.156380586288)/(o+1.)+(-0.0154369973565)+(-0.633594612687)*0.5**o
        ref[3,1,1]=(-0.352989559326)/(o+1.)+(-0.290547031627)+(0.710852984398)*0.5**o
        ref[4,0,0]=(-0.147579361355)/(o+1.)+(-0.615173424765)+(-0.00193837918766)*0.5**o
        ref[4,0,1]=(0.598152345277)/(o+1.)+(-0.540248606426)+(-0.559503106696)*0.5**o
        ref[4,1,0]=(-1.17182448825)/(o+1.)+(-0.590551681576)+(-0.401073317208)*0.5**o
        ref[4,1,1]=(0.837504141173)/(o+1.)+(0.0807414267186)+(-0.15185865216)*0.5**o
        ref[5,0,0]=(-0.646601115533)/(o+1.)+(-1.20895265273)+(0.9173049337)*0.5**o
        ref[5,0,1]=(1.26626110052)/(o+1.)+(-0.56471923973)+(-0.880046978955)*0.5**o
        ref[5,1,0]=(-0.069063169298)/(o+1.)+(0.883536796964)+(0.883025165644)*0.5**o
        ref[5,1,1]=(1.05999338993)/(o+1.)+(-0.392519102893)+(0.918128198439)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactOne_fromData_ReducedContinuousFunction_rank4(self):
      """
      tests integral of rank 4 Data on the ReducedFunctionOnContactOne

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5, 3, 2),w)
      ref=numpy.zeros((4, 5, 3, 2),numpy.float_)
      if dim==2:
        arg[0,0,0,0]=(0.257898226293)*x[0]+(-0.01230083314)*x[1]
        arg[0,0,0,1]=(0.0531183070647)*x[0]+(-0.170192287836)*x[1]
        arg[0,0,1,0]=(-0.364644940864)*x[0]+(1.04227651446)*x[1]
        arg[0,0,1,1]=(-0.894334108203)*x[0]+(0.151390331211)*x[1]
        arg[0,0,2,0]=(1.03825401708)*x[0]+(-0.882136097149)*x[1]
        arg[0,0,2,1]=(-1.05304910155)*x[0]+(1.06866311708)*x[1]
        arg[0,1,0,0]=(1.54742542159)*x[0]+(0.941778348482)*x[1]
        arg[0,1,0,1]=(1.1797380956)*x[0]+(-0.296417138234)*x[1]
        arg[0,1,1,0]=(-1.09196028027)*x[0]+(-0.783377624006)*x[1]
        arg[0,1,1,1]=(0.370952191443)*x[0]+(-0.0854141858991)*x[1]
        arg[0,1,2,0]=(-1.23552368139)*x[0]+(1.23848088087)*x[1]
        arg[0,1,2,1]=(0.216345350114)*x[0]+(-0.136524468427)*x[1]
        arg[0,2,0,0]=(-1.21206901263)*x[0]+(0.804219582144)*x[1]
        arg[0,2,0,1]=(0.192956526208)*x[0]+(0.588417678858)*x[1]
        arg[0,2,1,0]=(-0.940175603742)*x[0]+(0.329047260784)*x[1]
        arg[0,2,1,1]=(-0.138597387792)*x[0]+(0.398298326223)*x[1]
        arg[0,2,2,0]=(0.10089766824)*x[0]+(1.25459493298)*x[1]
        arg[0,2,2,1]=(-0.401163525883)*x[0]+(0.276542765282)*x[1]
        arg[0,3,0,0]=(0.252156795268)*x[0]+(-0.50223063539)*x[1]
        arg[0,3,0,1]=(0.466244610905)*x[0]+(-0.587025224301)*x[1]
        arg[0,3,1,0]=(-0.268082012713)*x[0]+(-1.28874675764)*x[1]
        arg[0,3,1,1]=(1.10867463519)*x[0]+(-0.165500912864)*x[1]
        arg[0,3,2,0]=(-1.94681908266)*x[0]+(0.166854108476)*x[1]
        arg[0,3,2,1]=(-0.537150417922)*x[0]+(0.0872851457131)*x[1]
        arg[0,4,0,0]=(-0.376548581807)*x[0]+(-0.245201133388)*x[1]
        arg[0,4,0,1]=(-0.135287690767)*x[0]+(0.750376238942)*x[1]
        arg[0,4,1,0]=(-0.0240491596907)*x[0]+(-1.21337088424)*x[1]
        arg[0,4,1,1]=(0.699215733397)*x[0]+(0.0596356032853)*x[1]
        arg[0,4,2,0]=(-0.521316010702)*x[0]+(-1.75309802615)*x[1]
        arg[0,4,2,1]=(0.403640172305)*x[0]+(1.41465827077)*x[1]
        arg[1,0,0,0]=(-1.01538089315)*x[0]+(-1.14785578909)*x[1]
        arg[1,0,0,1]=(1.10045201832)*x[0]+(0.630761728795)*x[1]
        arg[1,0,1,0]=(0.137673646147)*x[0]+(-0.387129090253)*x[1]
        arg[1,0,1,1]=(-1.04588287863)*x[0]+(-1.22433029859)*x[1]
        arg[1,0,2,0]=(-0.567830263535)*x[0]+(-0.405808616295)*x[1]
        arg[1,0,2,1]=(0.139326102495)*x[0]+(-1.93153384363)*x[1]
        arg[1,1,0,0]=(-0.638524762921)*x[0]+(0.506405751653)*x[1]
        arg[1,1,0,1]=(0.130775153144)*x[0]+(0.531317202208)*x[1]
        arg[1,1,1,0]=(0.557476206038)*x[0]+(0.447588676184)*x[1]
        arg[1,1,1,1]=(-0.538795916875)*x[0]+(-0.593869493531)*x[1]
        arg[1,1,2,0]=(-1.09827483942)*x[0]+(0.376431061876)*x[1]
        arg[1,1,2,1]=(-0.0074505620387)*x[0]+(-0.798645151486)*x[1]
        arg[1,2,0,0]=(0.251283911459)*x[0]+(0.194595071813)*x[1]
        arg[1,2,0,1]=(1.00482763799)*x[0]+(-1.00059863394)*x[1]
        arg[1,2,1,0]=(-0.453095434684)*x[0]+(-0.513922011418)*x[1]
        arg[1,2,1,1]=(0.558548837484)*x[0]+(0.0696328406825)*x[1]
        arg[1,2,2,0]=(-1.20625552545)*x[0]+(-0.151371710692)*x[1]
        arg[1,2,2,1]=(0.100262209656)*x[0]+(-0.314628804179)*x[1]
        arg[1,3,0,0]=(0.947063141882)*x[0]+(-0.297271200562)*x[1]
        arg[1,3,0,1]=(0.0760277251261)*x[0]+(-1.17982146387)*x[1]
        arg[1,3,1,0]=(-1.46946063969)*x[0]+(-0.615302663567)*x[1]
        arg[1,3,1,1]=(0.419747293729)*x[0]+(-0.629842531949)*x[1]
        arg[1,3,2,0]=(-1.44607954917)*x[0]+(0.520024093924)*x[1]
        arg[1,3,2,1]=(-1.01556392596)*x[0]+(-0.307173811354)*x[1]
        arg[1,4,0,0]=(-0.164339335244)*x[0]+(0.390736414982)*x[1]
        arg[1,4,0,1]=(-0.266932380288)*x[0]+(-0.731533848078)*x[1]
        arg[1,4,1,0]=(-0.903825489055)*x[0]+(-1.81564784421)*x[1]
        arg[1,4,1,1]=(-0.426008882665)*x[0]+(-1.09792948908)*x[1]
        arg[1,4,2,0]=(-1.32984927589)*x[0]+(1.58196302541)*x[1]
        arg[1,4,2,1]=(-0.0581032806145)*x[0]+(-0.323073355302)*x[1]
        arg[2,0,0,0]=(-0.744014251373)*x[0]+(-0.818253254872)*x[1]
        arg[2,0,0,1]=(0.368656965396)*x[0]+(-0.0864646380237)*x[1]
        arg[2,0,1,0]=(-0.112428845773)*x[0]+(0.109170104421)*x[1]
        arg[2,0,1,1]=(0.0682241548911)*x[0]+(-1.65999482283)*x[1]
        arg[2,0,2,0]=(0.212777058993)*x[0]+(0.127163480945)*x[1]
        arg[2,0,2,1]=(-0.256989443356)*x[0]+(0.183206687414)*x[1]
        arg[2,1,0,0]=(0.578997994373)*x[0]+(0.413087453689)*x[1]
        arg[2,1,0,1]=(0.886532090807)*x[0]+(0.330645525532)*x[1]
        arg[2,1,1,0]=(0.18224742814)*x[0]+(-0.713166591262)*x[1]
        arg[2,1,1,1]=(-0.25218296706)*x[0]+(0.736797897014)*x[1]
        arg[2,1,2,0]=(0.885270371408)*x[0]+(-1.35641397127)*x[1]
        arg[2,1,2,1]=(-0.963214290879)*x[0]+(0.348421598509)*x[1]
        arg[2,2,0,0]=(0.884120745351)*x[0]+(0.956857507884)*x[1]
        arg[2,2,0,1]=(-0.00795298517561)*x[0]+(0.51243740322)*x[1]
        arg[2,2,1,0]=(-0.563655252975)*x[0]+(-0.716305131721)*x[1]
        arg[2,2,1,1]=(0.648243491623)*x[0]+(-0.142239786485)*x[1]
        arg[2,2,2,0]=(1.50677848542)*x[0]+(-0.359722006818)*x[1]
        arg[2,2,2,1]=(-0.522592086302)*x[0]+(-0.18544927497)*x[1]
        arg[2,3,0,0]=(0.0822787762966)*x[0]+(0.196309662733)*x[1]
        arg[2,3,0,1]=(0.232573258627)*x[0]+(-0.0597976775432)*x[1]
        arg[2,3,1,0]=(1.50895737052)*x[0]+(0.546625934312)*x[1]
        arg[2,3,1,1]=(-1.64023373286)*x[0]+(0.220053261777)*x[1]
        arg[2,3,2,0]=(1.5897208842)*x[0]+(-0.0355885594992)*x[1]
        arg[2,3,2,1]=(-1.52822349982)*x[0]+(-0.0906934717656)*x[1]
        arg[2,4,0,0]=(0.496696582601)*x[0]+(-0.303888675059)*x[1]
        arg[2,4,0,1]=(-0.0819586112177)*x[0]+(-0.0449047294378)*x[1]
        arg[2,4,1,0]=(-0.678530758513)*x[0]+(-1.2173963447)*x[1]
        arg[2,4,1,1]=(-0.645450475268)*x[0]+(0.356771113426)*x[1]
        arg[2,4,2,0]=(-0.486643021867)*x[0]+(0.977763869689)*x[1]
        arg[2,4,2,1]=(-0.458991807397)*x[0]+(1.80038432923)*x[1]
        arg[3,0,0,0]=(0.270795948755)*x[0]+(-0.691512096338)*x[1]
        arg[3,0,0,1]=(-0.756717130099)*x[0]+(1.16189640534)*x[1]
        arg[3,0,1,0]=(0.967554346699)*x[0]+(-0.272119008963)*x[1]
        arg[3,0,1,1]=(1.14296491195)*x[0]+(-0.707175662379)*x[1]
        arg[3,0,2,0]=(1.15665030692)*x[0]+(0.414803389409)*x[1]
        arg[3,0,2,1]=(1.38997274955)*x[0]+(1.07065568192)*x[1]
        arg[3,1,0,0]=(-0.623998925511)*x[0]+(0.375545806421)*x[1]
        arg[3,1,0,1]=(-0.363431948398)*x[0]+(-0.183100417297)*x[1]
        arg[3,1,1,0]=(0.0435866301729)*x[0]+(-0.557137572068)*x[1]
        arg[3,1,1,1]=(-0.253225217468)*x[0]+(-0.0657932188329)*x[1]
        arg[3,1,2,0]=(0.147700921064)*x[0]+(0.681624563015)*x[1]
        arg[3,1,2,1]=(-0.075394297616)*x[0]+(0.271741245789)*x[1]
        arg[3,2,0,0]=(1.34803044812)*x[0]+(0.87991065609)*x[1]
        arg[3,2,0,1]=(0.0529487135353)*x[0]+(0.806480092352)*x[1]
        arg[3,2,1,0]=(-1.52038624166)*x[0]+(-0.992718138685)*x[1]
        arg[3,2,1,1]=(0.0859852188749)*x[0]+(1.63512596202)*x[1]
        arg[3,2,2,0]=(1.46925148575)*x[0]+(-1.60383564005)*x[1]
        arg[3,2,2,1]=(0.114463395473)*x[0]+(-1.0186377376)*x[1]
        arg[3,3,0,0]=(-1.02962502658)*x[0]+(-1.06788543146)*x[1]
        arg[3,3,0,1]=(-0.184391597652)*x[0]+(0.462603936483)*x[1]
        arg[3,3,1,0]=(0.611113335782)*x[0]+(0.282417397043)*x[1]
        arg[3,3,1,1]=(-0.193423873796)*x[0]+(-0.56108614964)*x[1]
        arg[3,3,2,0]=(-0.0290687598224)*x[0]+(-0.460221158417)*x[1]
        arg[3,3,2,1]=(0.784513522959)*x[0]+(-0.484813129206)*x[1]
        arg[3,4,0,0]=(0.423780748664)*x[0]+(-0.344020787894)*x[1]
        arg[3,4,0,1]=(1.10623024451)*x[0]+(-0.775008153301)*x[1]
        arg[3,4,1,0]=(0.554847534291)*x[0]+(-0.854749107999)*x[1]
        arg[3,4,1,1]=(-0.639595693597)*x[0]+(-0.966152813212)*x[1]
        arg[3,4,2,0]=(-0.0172621996023)*x[0]+(0.321815074269)*x[1]
        arg[3,4,2,1]=(0.39605156824)*x[0]+(-0.499138231749)*x[1]
        ref[0,0,0,0]=(0.169892838337)/(o+1.)+(-0.323261281472)+(0.722227117759)*0.5**o
        ref[0,0,0,1]=(-0.899797330597)/(o+1.)+(0.731242549464)+(-0.679761749104)*0.5**o
        ref[0,0,1,0]=(0.948969615551)/(o+1.)+(-0.173478478707)+(0.0756189154575)*0.5**o
        ref[0,0,1,1]=(-0.484419525942)/(o+1.)+(0.287761319651)+(-0.834046890353)*0.5**o
        ref[0,0,2,0]=(-0.275366085384)/(o+1.)+(-0.051636507926)+(0.534757021171)*0.5**o
        ref[0,0,2,1]=(0.338862631582)/(o+1.)+(0.076489529483)+(-0.47622767501)*0.5**o
        ref[0,1,0,0]=(0.960885993642)/(o+1.)+(0.36959166751)+(0.789134441414)*0.5**o
        ref[0,1,0,1]=(-0.101908900511)/(o+1.)+(0.343409131566)+(0.298411594744)*0.5**o
        ref[0,1,1,0]=(-0.50227382988)/(o+1.)+(-0.431549057432)+(-0.509965959533)*0.5**o
        ref[0,1,1,1]=(-0.294216689224)/(o+1.)+(0.293114907564)+(-0.00647512035965)*0.5**o
        ref[0,1,2,0]=(0.330313139948)/(o+1.)+(0.0601559902451)+(-0.447667920962)*0.5**o
        ref[0,1,2,1]=(-0.855325117375)/(o+1.)+(0.455706690618)+(0.0237326178252)*0.5**o
        ref[0,2,0,0]=(0.958948557218)/(o+1.)+(-0.307015973897)+(-0.752766039908)*0.5**o
        ref[0,2,0,1]=(0.622666086232)/(o+1.)+(0.417782845058)+(-0.676857571283)*0.5**o
        ref[0,2,1,0]=(-0.476269525636)/(o+1.)+(0.203518519787)+(-0.541895856898)*0.5**o
        ref[0,2,1,1]=(0.396862410812)/(o+1.)+(-0.318442063423)+(0.499722654464)*0.5**o
        ref[0,2,2,0]=(0.291546173382)/(o+1.)+(0.345688404218)+(0.372569619405)*0.5**o
        ref[0,2,2,1]=(-0.0863432016899)/(o+1.)+(0.296494638577)+(-0.631266836064)*0.5**o
        ref[0,3,0,0]=(0.437179077921)/(o+1.)+(-0.340515851088)+(-0.00622121586734)*0.5**o
        ref[0,3,0,1]=(-0.0670787919003)/(o+1.)+(-0.492735838895)+(0.931769856293)*0.5**o
        ref[0,3,1,0]=(-0.455620785864)/(o+1.)+(-0.866886545624)+(0.632565106759)*0.5**o
        ref[0,3,1,1]=(0.769395985412)/(o+1.)+(-0.303642587154)+(0.781062911223)*0.5**o
        ref[0,3,2,0]=(-0.338160377272)/(o+1.)+(-0.237545833658)+(-0.966712929597)*0.5**o
        ref[0,3,2,1]=(-0.796435534783)/(o+1.)+(0.544922318125)+(-0.743274373676)*0.5**o
        ref[0,4,0,0]=(0.452932120968)/(o+1.)+(-0.512874769603)+(-0.048932296957)*0.5**o
        ref[0,4,0,1]=(0.361579034112)/(o+1.)+(-0.0561290568189)+(0.365767627701)*0.5**o
        ref[0,4,1,0]=(-0.626053161593)/(o+1.)+(-0.761324946295)+(0.911283010254)*0.5**o
        ref[0,4,1,1]=(-0.0455977379835)/(o+1.)+(0.292122388548)+(0.22020429757)*0.5**o
        ref[0,4,2,0]=(-0.881052488675)/(o+1.)+(-0.210171400955)+(-0.973018746263)*0.5**o
        ref[0,4,2,1]=(0.955475719603)/(o+1.)+(0.367959093676)+(0.126904536122)*0.5**o
        ref[1,0,0,0]=(-0.314523133291)/(o+1.)+(-0.649561146707)+(-0.54959125554)*0.5**o
        ref[1,0,0,1]=(0.776726613012)/(o+1.)+(0.0747896120381)+(0.804907910029)*0.5**o
        ref[1,0,1,0]=(0.416309554773)/(o+1.)+(0.0650606132082)+(-0.795886225295)*0.5**o
        ref[1,0,1,1]=(-0.642083920221)/(o+1.)+(-0.518812997818)+(-0.590503261366)*0.5**o
        ref[1,0,2,0]=(0.209400082886)/(o+1.)+(-0.592256267485)+(0.00147357225359)*0.5**o
        ref[1,0,2,1]=(-0.969945221754)/(o+1.)+(-0.364559871358)+(-0.0931427766681)*0.5**o
        ref[1,1,0,0]=(0.941138659497)/(o+1.)+(-0.103485523138)+(-0.866286624489)*0.5**o
        ref[1,1,0,1]=(0.527395399401)/(o+1.)+(0.373358049173)+(-0.612019142395)*0.5**o
        ref[1,1,1,0]=(-0.276942103917)/(o+1.)+(0.624509599487)+(0.0329877871648)*0.5**o
        ref[1,1,1,1]=(-0.153477829221)/(o+1.)+(-0.0616268433725)+(-0.85593389444)*0.5**o
        ref[1,1,2,0]=(0.2685285297)/(o+1.)+(-0.301824551915)+(-0.386723203413)*0.5**o
        ref[1,1,2,1]=(-0.530960074652)/(o+1.)+(0.248621312801)+(-0.772378264474)*0.5**o
        ref[1,2,0,0]=(0.745071931955)/(o+1.)+(-0.149268301427)+(-0.000656345828701)*0.5**o
        ref[1,2,0,1]=(-0.581822428087)/(o+1.)+(0.2315691847)+(0.122913062738)*0.5**o
        ref[1,2,1,0]=(-0.325165099021)/(o+1.)+(-0.414758139139)+(0.187663931198)*0.5**o
        ref[1,2,1,1]=(-0.389683593964)/(o+1.)+(0.265985790556)+(0.485893691018)*0.5**o
        ref[1,2,2,0]=(0.661969003098)/(o+1.)+(-0.514584521701)+(-0.990427195841)*0.5**o
        ref[1,2,2,1]=(-0.0828384828188)/(o+1.)+(-0.482166570368)+(0.832805029032)*0.5**o
        ref[1,3,0,0]=(-0.756020671713)/(o+1.)+(0.627704058228)+(0.150404496577)*0.5**o
        ref[1,3,0,1]=(-0.93648010541)/(o+1.)+(0.11022703478)+(-0.387767702896)*0.5**o
        ref[1,3,1,0]=(-0.995823513997)/(o+1.)+(-0.242072805569)+(-0.604794178119)*0.5**o
        ref[1,3,1,1]=(-0.543071878541)/(o+1.)+(-0.160890620667)+(0.654757881656)*0.5**o
        ref[1,3,2,0]=(0.803184441579)/(o+1.)+(-0.612293875048)+(-0.504652146723)*0.5**o
        ref[1,3,2,1]=(-0.786714439322)/(o+1.)+(-0.175556007517)+(-0.18491128296)*0.5**o
        ref[1,4,0,0]=(-0.302928744298)/(o+1.)+(0.476360987754)+(-0.423396151472)*0.5**o
        ref[1,4,0,1]=(-0.410240602476)/(o+1.)+(-0.49551630458)+(0.40280698327)*0.5**o
        ref[1,4,1,0]=(-0.820677400346)/(o+1.)+(-0.573037933363)+(-0.752720066188)*0.5**o
        ref[1,4,1,1]=(-0.492748923658)/(o+1.)+(-0.600739806799)+(0.170290165507)*0.5**o
        ref[1,4,2,0]=(0.926654770824)/(o+1.)+(0.011282908181)+(-0.697106837662)*0.5**o
        ref[1,4,2,1]=(-0.958477382611)/(o+1.)+(0.618515858902)+(-0.65973097111)*0.5**o
        ref[2,0,0,0]=(-0.15636950624)/(o+1.)+(-0.520670547262)+(-0.36455690548)*0.5**o
        ref[2,0,0,1]=(-0.726424878286)/(o+1.)+(0.697757734621)+(-0.386898263585)*0.5**o
        ref[2,0,1,0]=(-0.055162393508)/(o+1.)+(0.247015646426)+(-0.442127640696)*0.5**o
        ref[2,0,1,1]=(-0.769332288875)/(o+1.)+(-0.0917175733456)+(-0.639003232373)*0.5**o
        ref[2,0,2,0]=(-0.196390038297)/(o+1.)+(0.0977907270548)+(0.340749124126)*0.5**o
        ref[2,0,2,1]=(0.841927481859)/(o+1.)+(-0.489185426146)+(0.0626606144906)*0.5**o
        ref[2,1,0,0]=(0.590018249148)/(o+1.)+(0.378476967182)+(-0.354886735451)*0.5**o
        ref[2,1,0,1]=(0.178495604742)/(o+1.)+(0.233073081069)+(0.572535849459)*0.5**o
        ref[2,1,1,0]=(-0.762671064919)/(o+1.)+(-0.108539686142)+(0.448831274081)*0.5**o
        ref[2,1,1,1]=(0.927591406535)/(o+1.)+(0.0441635703243)+(-0.53130361723)*0.5**o
        ref[2,1,2,0]=(-0.604360706259)/(o+1.)+(-0.134848102332)+(0.402913311063)*0.5**o
        ref[2,1,2,1]=(0.229727627441)/(o+1.)+(-0.323296604293)+(-0.197927111225)*0.5**o
        ref[2,2,0,0]=(0.287478356112)/(o+1.)+(0.762635506201)+(0.0282288847209)*0.5**o
        ref[2,2,0,1]=(0.243518182621)/(o+1.)+(-0.241502069199)+(0.743970373822)*0.5**o
        ref[2,2,1,0]=(-0.369049760378)/(o+1.)+(-0.21941778965)+(-0.472075045017)*0.5**o
        ref[2,2,1,1]=(-0.745343658666)/(o+1.)+(0.193375557775)+(0.864596248255)*0.5**o
        ref[2,2,2,0]=(-0.491769251494)/(o+1.)+(0.442742387618)+(0.753340954862)*0.5**o
        ref[2,2,2,1]=(0.551196377158)/(o+1.)+(-0.523535406613)+(-0.212166925203)*0.5**o
        ref[2,3,0,0]=(0.192271522168)/(o+1.)+(-0.0181953944687)+(0.122707705798)*0.5**o
        ref[2,3,0,1]=(-0.593768738594)/(o+1.)+(0.199088910545)+(0.368366498588)*0.5**o
        ref[2,3,1,0]=(0.0866211808983)/(o+1.)+(0.723512694127)+(0.521936735685)*0.5**o
        ref[2,3,1,1]=(-0.453288268254)/(o+1.)+(-0.15580970281)+(-0.655272797205)*0.5**o
        ref[2,3,2,0]=(0.614105802669)/(o+1.)+(0.0411427694898)+(0.85774098305)*0.5**o
        ref[2,3,2,1]=(0.263133533654)/(o+1.)+(-0.628776355867)+(-0.624497793507)*0.5**o
        ref[2,4,0,0]=(-0.809347914722)/(o+1.)+(0.367719412237)+(0.26671699779)*0.5**o
        ref[2,4,0,1]=(0.773124137566)/(o+1.)+(-0.442496395542)+(-0.0149946871367)*0.5**o
        ref[2,4,1,0]=(-0.927311525126)/(o+1.)+(-0.0125177466568)+(-0.943580084777)*0.5**o
        ref[2,4,1,1]=(0.306409665483)/(o+1.)+(0.128569553604)+(-0.852228134533)*0.5**o
        ref[2,4,2,0]=(0.926185522145)/(o+1.)+(0.0750538146738)+(-0.58517230367)*0.5**o
        ref[2,4,2,1]=(0.858245470922)/(o+1.)+(0.0125876600893)+(0.457971730734)*0.5**o
        ref[3,0,0,0]=(-0.74501947953)/(o+1.)+(0.343942243752)+(-0.363581155556)*0.5**o
        ref[3,0,0,1]=(0.850890737948)/(o+1.)+(0.157997002133)+(-0.761705466968)*0.5**o
        ref[3,0,1,0]=(-0.0468962248591)/(o+1.)+(0.0297980532763)+(0.682735456043)*0.5**o
        ref[3,0,1,1]=(0.0126326425456)/(o+1.)+(-0.076115296096)+(0.575387199213)*0.5**o
        ref[3,0,2,0]=(-0.410549752333)/(o+1.)+(0.85556811978)+(0.270867209104)*0.5**o
        ref[3,0,2,1]=(0.580487848545)/(o+1.)+(0.59069171729)+(0.698757148346)*0.5**o
        ref[3,1,0,0]=(0.139052775786)/(o+1.)+(-0.334747475889)+(0.281989056901)*0.5**o
        ref[3,1,0,1]=(0.591083610998)/(o+1.)+(-0.322680889668)+(-0.492254197357)*0.5**o
        ref[3,1,1,0]=(0.384713965427)/(o+1.)+(-0.522411606102)+(0.146558304882)*0.5**o
        ref[3,1,1,1]=(-0.827615582009)/(o+1.)+(-0.00295218575854)+(0.514501517225)*0.5**o
        ref[3,1,2,0]=(0.84762578765)/(o+1.)+(0.208361555685)+(-0.43502341494)*0.5**o
        ref[3,1,2,1]=(-0.554668615664)/(o+1.)+(0.825347835483)+(-0.899680107129)*0.5**o
        ref[3,2,0,0]=(0.652802712143)/(o+1.)+(0.397197582731)+(0.7807432266)*0.5**o
        ref[3,2,0,1]=(0.421495251937)/(o+1.)+(-0.119571710512)+(0.677076974975)*0.5**o
        ref[3,2,1,0]=(-0.230439630375)/(o+1.)+(-0.733165339017)+(-0.816334071941)*0.5**o
        ref[3,2,1,1]=(0.64153308098)/(o+1.)+(0.778374176432)+(-0.477170252948)*0.5**o
        ref[3,2,2,0]=(-0.987848717403)/(o+1.)+(-0.0606435888479)+(0.974551740798)*0.5**o
        ref[3,2,2,1]=(-0.318593711859)/(o+1.)+(-0.459904454409)+(0.334228278552)*0.5**o
        ref[3,3,0,0]=(-0.418115413351)/(o+1.)+(-0.348616063735)+(-0.982162917221)*0.5**o
        ref[3,3,0,1]=(-0.437684435015)/(o+1.)+(0.804145379606)+(-0.892393985366)*0.5**o
        ref[3,3,1,0]=(0.944100110135)/(o+1.)+(-0.059994562245)+(0.0694197471801)*0.5**o
        ref[3,3,1,1]=(-0.860656048613)/(o+1.)+(0.178136109535)+(-0.250126193893)*0.5**o
        ref[3,3,2,0]=(-0.363631991908)/(o+1.)+(-0.298384287282)+(0.471110648232)*0.5**o
        ref[3,3,2,1]=(-0.578021736479)/(o+1.)+(0.320913094365)+(0.235895941501)*0.5**o
        ref[3,4,0,0]=(-0.946995649946)/(o+1.)+(0.0451978887455)+(0.936359833225)*0.5**o
        ref[3,4,0,1]=(-0.636305818291)/(o+1.)+(0.258932088634)+(0.449663732234)*0.5**o
        ref[3,4,1,0]=(-0.823421937366)/(o+1.)+(0.112980674356)+(0.297559014947)*0.5**o
        ref[3,4,1,1]=(-0.441768022626)/(o+1.)+(-0.240840962617)+(-0.682298558949)*0.5**o
        ref[3,4,2,0]=(-0.258808936822)/(o+1.)+(0.510306577882)+(-0.457251344276)*0.5**o
        ref[3,4,2,1]=(-0.489365749016)/(o+1.)+(-0.296617385332)+(0.979513856171)*0.5**o
      else:
        arg[0,0,0,0]=(-0.474247989728)*x[0]+(-0.768689352235)*x[1]+(-1.08527900853)*x[2]
        arg[0,0,0,1]=(-0.237113797789)*x[0]+(0.0842735741018)*x[1]+(0.203858551029)*x[2]
        arg[0,0,1,0]=(0.24237127665)*x[0]+(0.421022955803)*x[1]+(-1.52818540927)*x[2]
        arg[0,0,1,1]=(1.34624865817)*x[0]+(-0.798943272818)*x[1]+(-0.823168386315)*x[2]
        arg[0,0,2,0]=(-0.906723771267)*x[0]+(0.328893123771)*x[1]+(-0.942770202463)*x[2]
        arg[0,0,2,1]=(1.54305197694)*x[0]+(0.813063941984)*x[1]+(0.205532727168)*x[2]
        arg[0,1,0,0]=(0.399159979872)*x[0]+(-1.52089837934)*x[1]+(-1.4064251406)*x[2]
        arg[0,1,0,1]=(-0.121139354731)*x[0]+(1.12768246422)*x[1]+(0.234299409411)*x[2]
        arg[0,1,1,0]=(-1.62513899388)*x[0]+(-0.173131700583)*x[1]+(1.24199996799)*x[2]
        arg[0,1,1,1]=(-0.371929434177)*x[0]+(-0.606278610409)*x[1]+(-0.00600790063066)*x[2]
        arg[0,1,2,0]=(-0.804531294423)*x[0]+(0.869392248002)*x[1]+(0.840417359943)*x[2]
        arg[0,1,2,1]=(0.329452946275)*x[0]+(1.1072495446)*x[1]+(-0.213517932323)*x[2]
        arg[0,2,0,0]=(1.02784887224)*x[0]+(0.435517042337)*x[1]+(1.58767206677)*x[2]
        arg[0,2,0,1]=(-0.0120997177)*x[0]+(0.430135803314)*x[1]+(-0.717473068956)*x[2]
        arg[0,2,1,0]=(0.955455517302)*x[0]+(0.712976918968)*x[1]+(-0.624804508641)*x[2]
        arg[0,2,1,1]=(-1.03293986594)*x[0]+(-0.539062834482)*x[1]+(-1.61172701428)*x[2]
        arg[0,2,2,0]=(-0.177662810138)*x[0]+(0.553352366127)*x[1]+(0.695417121111)*x[2]
        arg[0,2,2,1]=(0.318133474679)*x[0]+(-0.452528947926)*x[1]+(0.468602505091)*x[2]
        arg[0,3,0,0]=(-0.294467725916)*x[0]+(1.67314409894)*x[1]+(0.100736440007)*x[2]
        arg[0,3,0,1]=(-0.517162349887)*x[0]+(-1.57430191254)*x[1]+(1.24045877402)*x[2]
        arg[0,3,1,0]=(0.667393640368)*x[0]+(0.340993632212)*x[1]+(-1.09920535559)*x[2]
        arg[0,3,1,1]=(0.37451406727)*x[0]+(0.37350649326)*x[1]+(0.357440871322)*x[2]
        arg[0,3,2,0]=(-0.989366398194)*x[0]+(0.596179154759)*x[1]+(-0.573280308087)*x[2]
        arg[0,3,2,1]=(-0.25561190141)*x[0]+(1.42296905238)*x[1]+(-1.21032031399)*x[2]
        arg[0,4,0,0]=(-0.41394085285)*x[0]+(-0.751213929646)*x[1]+(-0.68470718293)*x[2]
        arg[0,4,0,1]=(0.702985204167)*x[0]+(-0.0375821582587)*x[1]+(0.290630803819)*x[2]
        arg[0,4,1,0]=(0.0558204653441)*x[0]+(1.61940057302)*x[1]+(-0.518863528354)*x[2]
        arg[0,4,1,1]=(1.25876303604)*x[0]+(-1.40923202908)*x[1]+(1.05202333762)*x[2]
        arg[0,4,2,0]=(0.0715295483294)*x[0]+(0.883869702529)*x[1]+(-0.463253474408)*x[2]
        arg[0,4,2,1]=(0.564563789615)*x[0]+(-0.135049742988)*x[1]+(-1.3118878567)*x[2]
        arg[1,0,0,0]=(0.707267335291)*x[0]+(-0.197244449872)*x[1]+(-0.114362773781)*x[2]
        arg[1,0,0,1]=(-0.144762815659)*x[0]+(0.434177706452)*x[1]+(0.0754859099691)*x[2]
        arg[1,0,1,0]=(-0.906934991359)*x[0]+(0.179011995627)*x[1]+(0.823936036469)*x[2]
        arg[1,0,1,1]=(1.11165724605)*x[0]+(0.48023223731)*x[1]+(0.981353808181)*x[2]
        arg[1,0,2,0]=(0.499460583481)*x[0]+(0.171586243561)*x[1]+(-0.252474775657)*x[2]
        arg[1,0,2,1]=(-0.604289370884)*x[0]+(1.11610914086)*x[1]+(0.747439062711)*x[2]
        arg[1,1,0,0]=(0.696191463882)*x[0]+(0.349547714404)*x[1]+(0.651361019042)*x[2]
        arg[1,1,0,1]=(0.152679923426)*x[0]+(-0.448442911425)*x[1]+(1.33964371501)*x[2]
        arg[1,1,1,0]=(-0.74612592015)*x[0]+(-1.27089239786)*x[1]+(-0.261725631736)*x[2]
        arg[1,1,1,1]=(1.50839493929)*x[0]+(1.93229724987)*x[1]+(-0.963996845831)*x[2]
        arg[1,1,2,0]=(0.867984447375)*x[0]+(-0.777493811854)*x[1]+(0.847847553133)*x[2]
        arg[1,1,2,1]=(1.34249389498)*x[0]+(0.776160804062)*x[1]+(-0.295393972199)*x[2]
        arg[1,2,0,0]=(-0.655771613243)*x[0]+(1.58220214455)*x[1]+(-0.406292477629)*x[2]
        arg[1,2,0,1]=(0.59294895986)*x[0]+(0.544961164723)*x[1]+(0.634279670948)*x[2]
        arg[1,2,1,0]=(1.15324179862)*x[0]+(0.645764835217)*x[1]+(0.670364725309)*x[2]
        arg[1,2,1,1]=(-0.819459106271)*x[0]+(0.024815625404)*x[1]+(0.769753650676)*x[2]
        arg[1,2,2,0]=(1.19042366161)*x[0]+(0.982020548972)*x[1]+(0.92730609738)*x[2]
        arg[1,2,2,1]=(-0.742441488349)*x[0]+(-1.3101429812)*x[1]+(-0.198104246324)*x[2]
        arg[1,3,0,0]=(0.349205030437)*x[0]+(-0.745240339596)*x[1]+(-0.744742092259)*x[2]
        arg[1,3,0,1]=(0.993101296711)*x[0]+(-1.48331355036)*x[1]+(-1.00616778677)*x[2]
        arg[1,3,1,0]=(-1.10586442824)*x[0]+(1.71922449779)*x[1]+(-0.943451153259)*x[2]
        arg[1,3,1,1]=(-1.72393763035)*x[0]+(-1.14113495956)*x[1]+(-0.211909615809)*x[2]
        arg[1,3,2,0]=(-0.378302963108)*x[0]+(-0.494705322002)*x[1]+(0.940164363332)*x[2]
        arg[1,3,2,1]=(0.850541148743)*x[0]+(-1.45461945468)*x[1]+(1.20025821328)*x[2]
        arg[1,4,0,0]=(1.33882463795)*x[0]+(0.475208133119)*x[1]+(0.722686059253)*x[2]
        arg[1,4,0,1]=(0.0750422155969)*x[0]+(1.29758288845)*x[1]+(-1.41431668738)*x[2]
        arg[1,4,1,0]=(1.36072344575)*x[0]+(-0.345743547563)*x[1]+(-0.63248018754)*x[2]
        arg[1,4,1,1]=(0.579049229913)*x[0]+(0.954245074078)*x[1]+(-0.170538926965)*x[2]
        arg[1,4,2,0]=(1.67615484634)*x[0]+(0.561067019023)*x[1]+(-1.26952326914)*x[2]
        arg[1,4,2,1]=(0.700910547934)*x[0]+(0.58717328764)*x[1]+(0.795138196965)*x[2]
        arg[2,0,0,0]=(0.763193592359)*x[0]+(-0.397549328381)*x[1]+(-0.230593559545)*x[2]
        arg[2,0,0,1]=(0.454026153468)*x[0]+(-1.12464443717)*x[1]+(-0.147462098246)*x[2]
        arg[2,0,1,0]=(-0.276457034935)*x[0]+(0.118173156654)*x[1]+(0.57296366275)*x[2]
        arg[2,0,1,1]=(1.31887812253)*x[0]+(-0.544644483254)*x[1]+(0.0013728350582)*x[2]
        arg[2,0,2,0]=(-0.257589384265)*x[0]+(-0.016005453203)*x[1]+(-0.508836430965)*x[2]
        arg[2,0,2,1]=(-0.259420301773)*x[0]+(0.122645293168)*x[1]+(0.381613885289)*x[2]
        arg[2,1,0,0]=(-1.64168512418)*x[0]+(0.025182006192)*x[1]+(0.0268780458975)*x[2]
        arg[2,1,0,1]=(-0.0885344200898)*x[0]+(0.408795219114)*x[1]+(0.635798801606)*x[2]
        arg[2,1,1,0]=(-0.959658503283)*x[0]+(1.034415556)*x[1]+(0.725681105748)*x[2]
        arg[2,1,1,1]=(1.59312707789)*x[0]+(-0.116514952101)*x[1]+(-0.86121633507)*x[2]
        arg[2,1,2,0]=(-1.44859572067)*x[0]+(-1.28344247275)*x[1]+(0.299296401869)*x[2]
        arg[2,1,2,1]=(-0.488424372501)*x[0]+(1.23481402288)*x[1]+(0.01702902463)*x[2]
        arg[2,2,0,0]=(-1.29986951244)*x[0]+(-1.20190912751)*x[1]+(-0.755848291051)*x[2]
        arg[2,2,0,1]=(-0.52421656312)*x[0]+(-0.335471509689)*x[1]+(-1.01264312499)*x[2]
        arg[2,2,1,0]=(-1.12066642609)*x[0]+(-1.02138558053)*x[1]+(-1.00777755552)*x[2]
        arg[2,2,1,1]=(-1.42127024744)*x[0]+(1.63480022479)*x[1]+(0.542244906643)*x[2]
        arg[2,2,2,0]=(-0.418852880297)*x[0]+(-0.983110436453)*x[1]+(-0.622603673805)*x[2]
        arg[2,2,2,1]=(0.727366446825)*x[0]+(-0.256324856361)*x[1]+(-0.20608740758)*x[2]
        arg[2,3,0,0]=(0.548268302883)*x[0]+(0.0102173127665)*x[1]+(0.869833576217)*x[2]
        arg[2,3,0,1]=(-1.70159091061)*x[0]+(1.15876875083)*x[1]+(-0.234757706773)*x[2]
        arg[2,3,1,0]=(-1.72817024817)*x[0]+(0.690055132218)*x[1]+(-0.149398998825)*x[2]
        arg[2,3,1,1]=(1.80829412387)*x[0]+(-0.217899654855)*x[1]+(-0.922672217822)*x[2]
        arg[2,3,2,0]=(1.23855200328)*x[0]+(0.412795411508)*x[1]+(0.960476191755)*x[2]
        arg[2,3,2,1]=(-1.43061904621)*x[0]+(-0.766039000745)*x[1]+(-1.25765397361)*x[2]
        arg[2,4,0,0]=(-0.660872016687)*x[0]+(-0.000405233718708)*x[1]+(-0.178507968219)*x[2]
        arg[2,4,0,1]=(-0.168526882871)*x[0]+(-1.06936802685)*x[1]+(1.54083857104)*x[2]
        arg[2,4,1,0]=(0.565807359529)*x[0]+(-1.41401297939)*x[1]+(-1.23254260537)*x[2]
        arg[2,4,1,1]=(-0.373920075802)*x[0]+(0.407330812903)*x[1]+(-0.204667001328)*x[2]
        arg[2,4,2,0]=(0.874100879885)*x[0]+(-0.191238094267)*x[1]+(1.54890969328)*x[2]
        arg[2,4,2,1]=(0.566428125313)*x[0]+(-1.60899595601)*x[1]+(-1.05112529768)*x[2]
        arg[3,0,0,0]=(-0.688037248111)*x[0]+(-1.56483484784)*x[1]+(0.26782504619)*x[2]
        arg[3,0,0,1]=(0.418788954459)*x[0]+(-0.670455226092)*x[1]+(-0.0562760591375)*x[2]
        arg[3,0,1,0]=(-0.2614362754)*x[0]+(0.271050519855)*x[1]+(0.227811982923)*x[2]
        arg[3,0,1,1]=(-0.873198928647)*x[0]+(-0.483763141395)*x[1]+(0.33532383979)*x[2]
        arg[3,0,2,0]=(-0.864161938176)*x[0]+(-1.4682754386)*x[1]+(-0.38126411162)*x[2]
        arg[3,0,2,1]=(0.142827521381)*x[0]+(0.464483940188)*x[1]+(0.214297576606)*x[2]
        arg[3,1,0,0]=(0.188747079113)*x[0]+(0.590821253713)*x[1]+(0.908850367108)*x[2]
        arg[3,1,0,1]=(0.0587541582624)*x[0]+(0.955494266151)*x[1]+(1.70822209849)*x[2]
        arg[3,1,1,0]=(-0.420356861523)*x[0]+(-0.551445918262)*x[1]+(-0.0187114754269)*x[2]
        arg[3,1,1,1]=(1.07071658401)*x[0]+(-0.589035602194)*x[1]+(0.133725074027)*x[2]
        arg[3,1,2,0]=(0.370477043767)*x[0]+(0.226262353618)*x[1]+(0.566087784489)*x[2]
        arg[3,1,2,1]=(1.77828660075)*x[0]+(0.607419742685)*x[1]+(0.0297054744582)*x[2]
        arg[3,2,0,0]=(-1.19326939474)*x[0]+(1.20875153887)*x[1]+(-0.124756222091)*x[2]
        arg[3,2,0,1]=(0.0645190914271)*x[0]+(0.357580475716)*x[1]+(-1.59461469718)*x[2]
        arg[3,2,1,0]=(1.4853619905)*x[0]+(0.751582301648)*x[1]+(0.859363177167)*x[2]
        arg[3,2,1,1]=(1.36658655075)*x[0]+(-0.776650996613)*x[1]+(0.654743994261)*x[2]
        arg[3,2,2,0]=(0.406696005677)*x[0]+(-0.528546876384)*x[1]+(-0.439775325396)*x[2]
        arg[3,2,2,1]=(-0.725198159461)*x[0]+(-0.0933136564416)*x[1]+(-0.720365141393)*x[2]
        arg[3,3,0,0]=(-0.428669112669)*x[0]+(0.626162511175)*x[1]+(-0.382776233923)*x[2]
        arg[3,3,0,1]=(0.340474939218)*x[0]+(-0.140550383925)*x[1]+(0.353836983746)*x[2]
        arg[3,3,1,0]=(0.851646816711)*x[0]+(0.0171684195949)*x[1]+(0.38173044322)*x[2]
        arg[3,3,1,1]=(0.359647722225)*x[0]+(0.425799941165)*x[1]+(-0.80174052212)*x[2]
        arg[3,3,2,0]=(0.13012568736)*x[0]+(-0.573735693651)*x[1]+(1.08843730088)*x[2]
        arg[3,3,2,1]=(-1.01372953294)*x[0]+(0.335282956856)*x[1]+(1.24570512628)*x[2]
        arg[3,4,0,0]=(-0.05674609039)*x[0]+(-0.814879575064)*x[1]+(-0.169789408264)*x[2]
        arg[3,4,0,1]=(1.5968230938)*x[0]+(0.0679819277023)*x[1]+(0.892284314512)*x[2]
        arg[3,4,1,0]=(1.22220298478)*x[0]+(-0.0258417926472)*x[1]+(0.440670138568)*x[2]
        arg[3,4,1,1]=(0.647072768716)*x[0]+(-0.177651087906)*x[1]+(-0.115553644646)*x[2]
        arg[3,4,2,0]=(1.29816363306)*x[0]+(-1.18656044167)*x[1]+(-0.465776533157)*x[2]
        arg[3,4,2,1]=(0.328907496091)*x[0]+(-0.179857937151)*x[1]+(-0.476306938306)*x[2]
        ref[0,0,0,0]=(-0.530378359482)/(o+1.)+(-0.935143327523)+(0.0724486640358)*0.5**o
        ref[0,0,0,1]=(1.22837847871)/(o+1.)+(-0.113350954728)+(-0.950658241915)*0.5**o
        ref[0,0,1,0]=(-0.000414683535449)/(o+1.)+(-0.183158731552)+(-0.49805903018)*0.5**o
        ref[0,0,1,1]=(-0.882730389132)/(o+1.)+(-0.112174958816)+(0.831217305802)*0.5**o
        ref[0,0,2,0]=(-1.1913557966)/(o+1.)+(-0.0615750891074)+(-0.206094875146)*0.5**o
        ref[0,0,2,1]=(0.634536492589)/(o+1.)+(0.649890059464)+(0.627332034578)*0.5**o
        ref[0,1,0,0]=(-1.38665625775)/(o+1.)+(-1.0210653524)+(0.900623422471)*0.5**o
        ref[0,1,0,1]=(1.20633678024)/(o+1.)+(0.0888193121856)+(-0.143132885707)*0.5**o
        ref[0,1,1,0]=(0.452225278035)/(o+1.)+(-0.148926943801)+(-0.710642116901)*0.5**o
        ref[0,1,1,1]=(-0.244223430204)/(o+1.)+(-0.619607191996)+(0.499221868978)*0.5**o
        ref[0,1,2,0]=(0.0383849036606)/(o+1.)+(0.759484971644)+(-0.652076533427)*0.5**o
        ref[0,1,2,1]=(0.207558055759)/(o+1.)+(0.502305382384)+(0.0110157380215)*0.5**o
        ref[0,2,0,0]=(0.761374280144)/(o+1.)+(0.777616611515)+(0.734430478174)*0.5**o
        ref[0,2,0,1]=(0.554502862028)/(o+1.)+(-0.409274602016)+(-0.0353906413371)*0.5**o
        ref[0,2,1,0]=(-0.252543841709)/(o+1.)+(0.386640377894)+(0.522891013551)*0.5**o
        ref[0,2,1,1]=(-1.52497235887)/(o+1.)+(-0.807603254401)+(-0.0435508470367)*0.5**o
        ref[0,2,2,0]=(0.172485606257)/(o+1.)+(0.350297526187)+(0.19802601847)*0.5**o
        ref[0,2,2,1]=(-0.141271359379)/(o+1.)+(0.0509198134622)+(0.373638764298)*0.5**o
        ref[0,3,0,0]=(1.31278836809)/(o+1.)+(-0.0918739323256)+(0.350372309595)*0.5**o
        ref[0,3,0,1]=(-0.396933928109)/(o+1.)+(-0.436137805999)+(0.418204051696)*0.5**o
        ref[0,3,1,0]=(-0.206619840099)/(o+1.)+(-0.118703289798)+(0.353208336689)*0.5**o
        ref[0,3,1,1]=(-0.595153371293)/(o+1.)+(0.615722581562)+(0.469169640021)*0.5**o
        ref[0,3,2,0]=(-0.680858997986)/(o+1.)+(-0.0393223937964)+(-0.206963765944)*0.5**o
        ref[0,3,2,1]=(0.012753264598)/(o+1.)+(0.246188741489)+(-0.548093910595)*0.5**o
        ref[0,4,0,0]=(-0.40478987122)/(o+1.)+(-0.890399681609)+(0.33572726901)*0.5**o
        ref[0,4,0,1]=(0.0280062557264)/(o+1.)+(0.0358049413228)+(0.856417711355)*0.5**o
        ref[0,4,1,0]=(0.181560004565)/(o+1.)+(0.329885369854)+(0.31502676574)*0.5**o
        ref[0,4,1,1]=(0.0552681276841)/(o+1.)+(0.00372919426319)+(0.838827828375)*0.5**o
        ref[0,4,2,0]=(0.656057162724)/(o+1.)+(0.179315527684)+(-0.522542441641)*0.5**o
        ref[0,4,2,1]=(0.126189389317)/(o+1.)+(-0.429452001643)+(-0.149659196108)*0.5**o
        ref[1,0,0,0]=(1.43443485527)/(o+1.)+(-0.55091754491)+(0.0630603461901)*0.5**o
        ref[1,0,0,1]=(-0.612191779306)/(o+1.)+(0.604025235381)+(-0.230957890695)*0.5**o
        ref[1,0,1,0]=(0.373085423681)/(o+1.)+(-0.0817391530697)+(-0.113594076805)*0.5**o
        ref[1,0,1,1]=(0.761862677657)/(o+1.)+(0.819524484428)+(0.172331645033)*0.5**o
        ref[1,0,2,0]=(0.929265450746)/(o+1.)+(-0.382585784938)+(0.254478170515)*0.5**o
        ref[1,0,2,1]=(1.58236203463)/(o+1.)+(-0.280641188811)+(0.238179175679)*0.5**o
        ref[1,1,0,0]=(-0.117055726312)/(o+1.)+(0.957914090571)+(-0.101672257501)*0.5**o
        ref[1,1,0,1]=(0.0849062553796)/(o+1.)+(0.0703564566846)+(0.818261558262)*0.5**o
        ref[1,1,1,0]=(-1.72219113622)/(o+1.)+(-0.357454724939)+(0.15835663635)*0.5**o
        ref[1,1,1,1]=(0.739078400468)/(o+1.)+(0.532292543169)+(0.673031856531)*0.5**o
        ref[1,1,2,0]=(0.255023909903)/(o+1.)+(0.229656917655)+(0.22400044344)*0.5**o
        ref[1,1,2,1]=(0.518856765199)/(o+1.)+(0.380552790125)+(0.543298381396)*0.5**o
        ref[1,2,0,0]=(0.0321563884831)/(o+1.)+(0.25981567762)+(-0.0316496900482)*0.5**o
        ref[1,2,0,1]=(1.09318176771)/(o+1.)+(0.0572442907603)+(0.564519446305)*0.5**o
        ref[1,2,1,0]=(0.756962791387)/(o+1.)+(0.69419429624)+(0.32401997528)*0.5**o
        ref[1,2,1,1]=(1.28480299241)/(o+1.)+(-0.201990593541)+(-0.905711635518)*0.5**o
        ref[1,2,2,0]=(1.04969566442)/(o+1.)+(0.72854097372)+(0.592972696101)*0.5**o
        ref[1,2,2,1]=(-1.77086264775)/(o+1.)+(-0.261914180361)+(0.0440022926066)*0.5**o
        ref[1,3,0,0]=(-0.477780531124)/(o+1.)+(-0.181354085163)+(-0.300288699967)*0.5**o
        ref[1,3,0,1]=(-0.797627013456)/(o+1.)+(-0.523163955197)+(0.347574883435)*0.5**o
        ref[1,3,1,0]=(0.428806106821)/(o+1.)+(-0.0514584302394)+(-0.655980330052)*0.5**o
        ref[1,3,1,1]=(-0.141740459457)/(o+1.)+(-1.05699960931)+(-0.821242527644)*0.5**o
        ref[1,3,2,0]=(0.0181810985873)/(o+1.)+(-0.180697139553)+(0.41036925874)*0.5**o
        ref[1,3,2,1]=(-0.061077367)/(o+1.)+(0.250077446763)+(0.157102380812)*0.5**o
        ref[1,4,0,0]=(1.14087212913)/(o+1.)+(0.368059029864)+(0.659728641463)*0.5**o
        ref[1,4,0,1]=(0.365658713316)/(o+1.)+(0.088533821044)+(-0.584417938734)*0.5**o
        ref[1,4,1,0]=(-1.11681308321)/(o+1.)+(0.438275560627)+(0.622761672604)*0.5**o
        ref[1,4,1,1]=(1.18893372018)/(o+1.)+(0.172934240923)+(-0.172046824999)*0.5**o
        ref[1,4,2,0]=(0.16240489493)/(o+1.)+(-0.0145923852256)+(0.834478471749)*0.5**o
        ref[1,4,2,1]=(0.571355829378)/(o+1.)+(0.526896417749)+(0.458073367663)*0.5**o
        ref[2,0,0,0]=(-0.489234572937)/(o+1.)+(-0.0135589093901)+(0.65140309615)*0.5**o
        ref[2,0,0,1]=(-1.55383491931)/(o+1.)+(0.0161644619972)+(0.703425613366)*0.5**o
        ref[2,0,1,0]=(-0.258604855723)/(o+1.)+(0.0827812678671)+(0.507722104457)*0.5**o
        ref[2,0,1,1]=(0.0504178529481)/(o+1.)+(0.182991850332)+(0.359204920726)*0.5**o
        ref[2,0,2,0]=(0.652043479841)/(o+1.)+(-0.443015450013)+(-0.548443848248)*0.5**o
        ref[2,0,2,1]=(0.340317839857)/(o+1.)+(-0.379565594545)+(0.663652225917)*0.5**o
        ref[2,1,0,0]=(-0.587863196667)/(o+1.)+(-0.0137797674284)+(-0.974202340562)*0.5**o
        ref[2,1,0,1]=(1.00030747039)/(o+1.)+(-0.399997754768)+(0.755747639771)*0.5**o
        ref[2,1,1,0]=(0.703413281036)/(o+1.)+(0.487016208736)+(-0.877007540039)*0.5**o
        ref[2,1,1,1]=(0.263166830572)/(o+1.)+(-0.157259452849)+(0.666747865841)*0.5**o
        ref[2,1,2,0]=(0.104532343004)/(o+1.)+(-1.02548001493)+(-0.486314104701)*0.5**o
        ref[2,1,2,1]=(0.933518655478)/(o+1.)+(0.0369928229139)+(-0.244085626298)*0.5**o
        ref[2,2,0,0]=(-1.50550384519)/(o+1.)+(-0.590996227594)+(-0.570130630622)*0.5**o
        ref[2,2,0,1]=(-0.60318683839)/(o+1.)+(-0.38308245528)+(-0.502979448853)*0.5**o
        ref[2,2,1,0]=(-1.11258910879)/(o+1.)+(-0.518777277969)+(-0.999685897413)*0.5**o
        ref[2,2,1,1]=(1.84366617107)/(o+1.)+(-0.232866738261)+(-0.622157810558)*0.5**o
        ref[2,2,2,0]=(-0.620181988585)/(o+1.)+(-0.836162444183)+(0.267939886396)*0.5**o
        ref[2,2,2,1]=(-0.826647389099)/(o+1.)+(0.0929536518109)+(0.905694268362)*0.5**o
        ref[2,3,0,0]=(0.184067674443)/(o+1.)+(0.259745858757)+(0.724759799909)*0.5**o
        ref[2,3,0,1]=(0.0175595303049)/(o+1.)+(0.0560117358912)+(-0.907162868639)*0.5**o
        ref[2,3,1,0]=(-0.609443154617)/(o+1.)+(0.186540819401)+(-0.951152598958)*0.5**o
        ref[2,3,1,1]=(-1.61858082094)/(o+1.)+(0.679439858578)+(0.927423354971)*0.5**o
        ref[2,3,2,0]=(0.44350652486)/(o+1.)+(0.81466177481)+(0.53899353207)*0.5**o
        ref[2,3,2,1]=(-0.877289862245)/(o+1.)+(-1.05742260849)+(-0.462176941342)*0.5**o
        ref[2,4,0,0]=(-0.736381416539)/(o+1.)+(-0.0139374259612)+(-0.0755289501641)*0.5**o
        ref[2,4,0,1]=(0.690429326999)/(o+1.)+(-0.0293644018251)+(-0.328756862036)*0.5**o
        ref[2,4,1,0]=(-1.57117034002)/(o+1.)+(-0.531463765387)+(0.553349645561)*0.5**o
        ref[2,4,1,1]=(1.24898175282)/(o+1.)+(-0.452273024959)+(-0.515691967125)*0.5**o
        ref[2,4,2,0]=(0.239126191643)/(o+1.)+(0.722504563932)+(0.547637159392)*0.5**o
        ref[2,4,2,1]=(-1.40763254209)/(o+1.)+(-0.809456441012)+(0.932852295739)*0.5**o
        ref[3,0,0,0]=(-0.416554297686)/(o+1.)+(-0.632193610911)+(-0.304105530258)*0.5**o
        ref[3,0,0,1]=(-1.02983448036)/(o+1.)+(0.579212231778)+(-0.436532313966)*0.5**o
        ref[3,0,1,0]=(0.42124133216)/(o+1.)+(0.368290418827)+(-0.920395942436)*0.5**o
        ref[3,0,1,1]=(0.158195474854)/(o+1.)+(-0.17459474514)+(-0.830644214826)*0.5**o
        ref[3,0,2,0]=(-1.18749647329)/(o+1.)+(-0.506173996178)+(-0.513857022748)*0.5**o
        ref[3,0,2,1]=(0.233437753018)/(o+1.)+(0.455705145371)+(-0.323239005586)*0.5**o
        ref[3,1,0,0]=(1.42991958566)/(o+1.)+(0.053769921022)+(0.150959272234)*0.5**o
        ref[3,1,0,1]=(1.48423731111)/(o+1.)+(0.964048315997)+(-0.689863420196)*0.5**o
        ref[3,1,1,0]=(0.868837676765)/(o+1.)+(-1.01261543254)+(0.165878933098)*0.5**o
        ref[3,1,1,1]=(0.820750307831)/(o+1.)+(-0.300930223821)+(0.396516195659)*0.5**o
        ref[3,1,2,0]=(0.477077782202)/(o+1.)+(-0.0416336989494)+(0.769016797571)*0.5**o
        ref[3,1,2,1]=(0.359895440587)/(o+1.)+(0.555322395811)+(0.944871585683)*0.5**o
        ref[3,2,0,0]=(0.657668315832)/(o+1.)+(-0.240321895174)+(-0.286298603451)*0.5**o
        ref[3,2,0,1]=(-0.941360568282)/(o+1.)+(-0.0246544136593)+(-0.181845734439)*0.5**o
        ref[3,2,1,0]=(1.60316834971)/(o+1.)+(0.252291602298)+(0.988555915005)*0.5**o
        ref[3,2,1,1]=(0.833811957678)/(o+1.)+(-0.111970621871)+(0.634808834458)*0.5**o
        ref[3,2,2,0]=(-1.38877073784)/(o+1.)+(0.538356636307)+(-0.249568730877)*0.5**o
        ref[3,2,2,1]=(-0.311839184097)/(o+1.)+(-0.698114103264)+(0.169190433329)*0.5**o
        ref[3,3,0,0]=(0.743170989696)/(o+1.)+(-0.499224746788)+(0.0699956684628)*0.5**o
        ref[3,3,0,1]=(-0.293538786051)/(o+1.)+(0.26667992866)+(0.313940467768)*0.5**o
        ref[3,3,1,0]=(0.951483461345)/(o+1.)+(0.148317904561)+(0.00242640905887)*0.5**o
        ref[3,3,1,1]=(-0.110753912776)/(o+1.)+(-0.44308689138)+(0.980634836805)*0.5**o
        ref[3,3,2,0]=(-0.337895223544)/(o+1.)+(0.358941646037)+(0.264839226057)*0.5**o
        ref[3,3,2,1]=(0.857823611315)/(o+1.)+(0.30404611512)+(-0.898657291359)*0.5**o
        ref[3,4,0,0]=(0.037944509376)/(o+1.)+(-0.364801700413)+(-0.349756182268)*0.5**o
        ref[3,4,0,1]=(1.01114399396)/(o+1.)+(0.457103279485)+(0.631738783076)*0.5**o
        ref[3,4,1,0]=(0.969931913888)/(o+1.)+(-0.165922085756)+(0.998943588321)*0.5**o
        ref[3,4,1,1]=(1.35441114248)/(o+1.)+(-0.502422727199)+(0.00430234808496)*0.5**o
        ref[3,4,2,0]=(-0.367201097566)/(o+1.)+(-0.337780780531)+(0.688589316866)*0.5**o
        ref[3,4,2,1]=(-0.0547905846061)/(o+1.)+(-0.293870420685)+(0.31527404661)*0.5**o
      res=integrate(arg,where=w_ref)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactOne_fromData_ReducedFunctionOnContactOne_rank0(self):
      """
      tests integral of rank 0 Data on the ReducedFunctionOnContactOne

      assumptions: ReducedFunctionOnContactOne(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      w=ReducedFunctionOnContactOne(self.domain)
      x=w.getX()
      arg=Data(0,(),w)
      if dim==2:
        arg=(0.311787641998)*x[0]**o+(0.161673282875)*x[0]+(0.32570909237)*x[1]**o+(-0.249404742764)*x[1]
        ref=(0.32570909237)/(o+1.)+(-0.0438657299447)+(0.311787641998)*0.5**o
      else:
        arg=(0.0675036998003)*x[0]**o+(-0.247751392136)*x[0]+(0.791676220029)*x[1]**o+(-0.269015236163)*x[1]+(0.265735156875)*x[2]**o+(0.820319192519)*x[2]
        ref=(1.0574113769)/(o+1.)+(0.15177628211)+(0.0675036998003)*0.5**o
      res=integrate(arg)
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactOne_fromData_ReducedFunctionOnContactOne_rank1(self):
      """
      tests integral of rank 1 Data on the ReducedFunctionOnContactOne

      assumptions: ReducedFunctionOnContactOne(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      w=ReducedFunctionOnContactOne(self.domain)
      x=w.getX()
      arg=Data(0,(2,),w)
      ref=numpy.zeros((2,),numpy.float_)
      if dim==2:
        arg[0]=(0.99235950851)*x[0]**o+(0.00867392831518)*x[0]+(-0.598024390869)*x[1]**o+(0.00945232777857)*x[1]
        arg[1]=(-0.140261338996)*x[0]**o+(-0.166524016889)*x[0]+(0.59850076881)*x[1]**o+(-0.805756215354)*x[1]
        ref[0]=(-0.598024390869)/(o+1.)+(0.00906312804687)+(0.99235950851)*0.5**o
        ref[1]=(0.59850076881)/(o+1.)+(-0.486140116122)+(-0.140261338996)*0.5**o
      else:
        arg[0]=(-0.158686556523)*x[0]**o+(0.249193164275)*x[0]+(-0.801383648208)*x[1]**o+(-0.439021533504)*x[1]+(-0.112035191366)*x[2]**o+(-0.323385862625)*x[2]
        arg[1]=(0.392997873484)*x[0]**o+(-0.867468854104)*x[0]+(0.804459243476)*x[1]**o+(0.129028860314)*x[1]+(-0.252715502862)*x[2]**o+(0.457355923469)*x[2]
        ref[0]=(-0.913418839574)/(o+1.)+(-0.256607115927)+(-0.158686556523)*0.5**o
        ref[1]=(0.551743740614)/(o+1.)+(-0.140542035161)+(0.392997873484)*0.5**o
      res=integrate(arg)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactOne_fromData_ReducedFunctionOnContactOne_rank2(self):
      """
      tests integral of rank 2 Data on the ReducedFunctionOnContactOne

      assumptions: ReducedFunctionOnContactOne(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      w=ReducedFunctionOnContactOne(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5),w)
      ref=numpy.zeros((4, 5),numpy.float_)
      if dim==2:
        arg[0,0]=(0.330520725992)*x[0]**o+(0.160118740246)*x[0]+(-0.178670895269)*x[1]**o+(-0.281660040648)*x[1]
        arg[0,1]=(-0.871801084852)*x[0]**o+(0.692549035429)*x[0]+(-0.265102051042)*x[1]**o+(-0.799752901882)*x[1]
        arg[0,2]=(-0.816985024574)*x[0]**o+(0.534242721856)*x[0]+(0.81492666918)*x[1]**o+(0.891189158633)*x[1]
        arg[0,3]=(-0.20728122917)*x[0]**o+(-0.656873565737)*x[0]+(-0.924660993906)*x[1]**o+(0.453699593426)*x[1]
        arg[0,4]=(-0.541377726375)*x[0]**o+(0.916287049432)*x[0]+(-0.15879052465)*x[1]**o+(-0.319875161126)*x[1]
        arg[1,0]=(0.296966289838)*x[0]**o+(0.170347587334)*x[0]+(0.755207250839)*x[1]**o+(0.677658805212)*x[1]
        arg[1,1]=(-0.507591010573)*x[0]**o+(0.299638021889)*x[0]+(-0.757702071557)*x[1]**o+(-0.194564860748)*x[1]
        arg[1,2]=(0.821972467197)*x[0]**o+(-0.75902704466)*x[0]+(0.19424332718)*x[1]**o+(-0.070182646035)*x[1]
        arg[1,3]=(-0.340805687409)*x[0]**o+(0.585961973082)*x[0]+(-0.602942732318)*x[1]**o+(-0.288941395094)*x[1]
        arg[1,4]=(0.0314374795024)*x[0]**o+(0.293710824013)*x[0]+(0.576509865762)*x[1]**o+(-0.481377812302)*x[1]
        arg[2,0]=(0.250117528833)*x[0]**o+(-0.318419344173)*x[0]+(-0.793830015706)*x[1]**o+(0.0445553063586)*x[1]
        arg[2,1]=(-0.347974394766)*x[0]**o+(-0.569347105354)*x[0]+(0.838687005759)*x[1]**o+(0.534306628908)*x[1]
        arg[2,2]=(0.53063272248)*x[0]**o+(0.426504428158)*x[0]+(0.215398162753)*x[1]**o+(0.324348187788)*x[1]
        arg[2,3]=(-0.336752121527)*x[0]**o+(0.666074768462)*x[0]+(-0.739123251932)*x[1]**o+(-0.538899653147)*x[1]
        arg[2,4]=(-0.832783810519)*x[0]**o+(0.558572116384)*x[0]+(0.119988536199)*x[1]**o+(-0.664170918761)*x[1]
        arg[3,0]=(0.4941356722)*x[0]**o+(-0.403591783006)*x[0]+(0.205050527245)*x[1]**o+(0.351642574939)*x[1]
        arg[3,1]=(0.12538190695)*x[0]**o+(-0.890722638937)*x[0]+(-0.897011486902)*x[1]**o+(-0.465522770696)*x[1]
        arg[3,2]=(-0.862829007552)*x[0]**o+(-0.950531038019)*x[0]+(-0.970016379643)*x[1]**o+(-0.500580755888)*x[1]
        arg[3,3]=(0.131451743264)*x[0]**o+(0.43845495141)*x[0]+(-0.929808474589)*x[1]**o+(-0.104714452616)*x[1]
        arg[3,4]=(-0.81935224348)*x[0]**o+(-0.53791483312)*x[0]+(-0.714512816761)*x[1]**o+(0.329859667225)*x[1]
        ref[0,0]=(-0.178670895269)/(o+1.)+(-0.0607706502011)+(0.330520725992)*0.5**o
        ref[0,1]=(-0.265102051042)/(o+1.)+(-0.0536019332266)+(-0.871801084852)*0.5**o
        ref[0,2]=(0.81492666918)/(o+1.)+(0.712715940244)+(-0.816985024574)*0.5**o
        ref[0,3]=(-0.924660993906)/(o+1.)+(-0.101586986156)+(-0.20728122917)*0.5**o
        ref[0,4]=(-0.15879052465)/(o+1.)+(0.298205944153)+(-0.541377726375)*0.5**o
        ref[1,0]=(0.755207250839)/(o+1.)+(0.424003196273)+(0.296966289838)*0.5**o
        ref[1,1]=(-0.757702071557)/(o+1.)+(0.0525365805706)+(-0.507591010573)*0.5**o
        ref[1,2]=(0.19424332718)/(o+1.)+(-0.414604845348)+(0.821972467197)*0.5**o
        ref[1,3]=(-0.602942732318)/(o+1.)+(0.148510288994)+(-0.340805687409)*0.5**o
        ref[1,4]=(0.576509865762)/(o+1.)+(-0.0938334941445)+(0.0314374795024)*0.5**o
        ref[2,0]=(-0.793830015706)/(o+1.)+(-0.136932018907)+(0.250117528833)*0.5**o
        ref[2,1]=(0.838687005759)/(o+1.)+(-0.0175202382229)+(-0.347974394766)*0.5**o
        ref[2,2]=(0.215398162753)/(o+1.)+(0.375426307973)+(0.53063272248)*0.5**o
        ref[2,3]=(-0.739123251932)/(o+1.)+(0.0635875576574)+(-0.336752121527)*0.5**o
        ref[2,4]=(0.119988536199)/(o+1.)+(-0.0527994011886)+(-0.832783810519)*0.5**o
        ref[3,0]=(0.205050527245)/(o+1.)+(-0.0259746040335)+(0.4941356722)*0.5**o
        ref[3,1]=(-0.897011486902)/(o+1.)+(-0.678122704817)+(0.12538190695)*0.5**o
        ref[3,2]=(-0.970016379643)/(o+1.)+(-0.725555896953)+(-0.862829007552)*0.5**o
        ref[3,3]=(-0.929808474589)/(o+1.)+(0.166870249397)+(0.131451743264)*0.5**o
        ref[3,4]=(-0.714512816761)/(o+1.)+(-0.104027582947)+(-0.81935224348)*0.5**o
      else:
        arg[0,0]=(-0.916251178344)*x[0]**o+(0.858306033367)*x[0]+(0.814515071272)*x[1]**o+(-0.31988121358)*x[1]+(-0.973228319626)*x[2]**o+(-0.448894628432)*x[2]
        arg[0,1]=(0.394307707909)*x[0]**o+(-0.270473398988)*x[0]+(0.904831123674)*x[1]**o+(-0.949869961069)*x[1]+(0.771889549168)*x[2]**o+(0.650652953462)*x[2]
        arg[0,2]=(-0.0387282383528)*x[0]**o+(0.0292919821249)*x[0]+(-0.165189629855)*x[1]**o+(-0.265182468614)*x[1]+(0.919042632361)*x[2]**o+(0.710368688769)*x[2]
        arg[0,3]=(0.680006301334)*x[0]**o+(0.96959940913)*x[0]+(0.773153544603)*x[1]**o+(-0.460435754764)*x[1]+(-0.202448514604)*x[2]**o+(-0.495739672831)*x[2]
        arg[0,4]=(-0.972790915366)*x[0]**o+(-0.14148426268)*x[0]+(0.511995390348)*x[1]**o+(-0.252881217719)*x[1]+(0.255711210131)*x[2]**o+(0.729991325048)*x[2]
        arg[1,0]=(-0.105824689194)*x[0]**o+(-0.201427239595)*x[0]+(-0.00550922402684)*x[1]**o+(-0.732794556814)*x[1]+(-0.0820953451459)*x[2]**o+(0.289904794134)*x[2]
        arg[1,1]=(-0.378707957559)*x[0]**o+(0.776904510463)*x[0]+(0.0406721415351)*x[1]**o+(0.159288221237)*x[1]+(0.961004037621)*x[2]**o+(-0.0866180279992)*x[2]
        arg[1,2]=(-0.0226058491089)*x[0]**o+(-0.569995151391)*x[0]+(-0.0675265986408)*x[1]**o+(-0.141280346236)*x[1]+(0.116098815262)*x[2]**o+(0.654630437206)*x[2]
        arg[1,3]=(-0.567270755897)*x[0]**o+(0.580139672878)*x[0]+(-0.411816626246)*x[1]**o+(-0.490525831459)*x[1]+(0.9322522728)*x[2]**o+(-0.948230068413)*x[2]
        arg[1,4]=(0.977251285351)*x[0]**o+(-0.457921996364)*x[0]+(-0.586207130671)*x[1]**o+(0.801370818191)*x[1]+(-0.667233775445)*x[2]**o+(0.277054304674)*x[2]
        arg[2,0]=(0.997778241906)*x[0]**o+(0.490242237175)*x[0]+(0.406384527224)*x[1]**o+(-0.00724475574186)*x[1]+(-0.59799957401)*x[2]**o+(0.520598391022)*x[2]
        arg[2,1]=(0.59766081214)*x[0]**o+(-0.144793824209)*x[0]+(-0.191121009265)*x[1]**o+(-0.178016091024)*x[1]+(0.168431462883)*x[2]**o+(-0.766200678925)*x[2]
        arg[2,2]=(0.633988266946)*x[0]**o+(-0.880495194817)*x[0]+(-0.217502420499)*x[1]**o+(-0.730159316723)*x[1]+(0.255599562732)*x[2]**o+(-0.510511087905)*x[2]
        arg[2,3]=(0.102332382261)*x[0]**o+(-0.370884332534)*x[0]+(-0.0926186085648)*x[1]**o+(0.86446607832)*x[1]+(0.0643112986026)*x[2]**o+(0.950118360785)*x[2]
        arg[2,4]=(0.928323090624)*x[0]**o+(0.942943574803)*x[0]+(0.162570981489)*x[1]**o+(-0.971092675635)*x[1]+(0.641489870001)*x[2]**o+(0.695989498774)*x[2]
        arg[3,0]=(-0.988813741403)*x[0]**o+(0.407960892494)*x[0]+(0.671572481461)*x[1]**o+(0.119756938854)*x[1]+(-0.923671788537)*x[2]**o+(-0.458931381612)*x[2]
        arg[3,1]=(-0.241202616681)*x[0]**o+(-0.369134099039)*x[0]+(-0.129164268709)*x[1]**o+(-0.469311903775)*x[1]+(0.633909205241)*x[2]**o+(0.564043934538)*x[2]
        arg[3,2]=(-0.397081811407)*x[0]**o+(0.909387986899)*x[0]+(0.260590665694)*x[1]**o+(-0.371472818152)*x[1]+(-0.0369834262513)*x[2]**o+(-0.887598789153)*x[2]
        arg[3,3]=(-0.304262529036)*x[0]**o+(0.741316478862)*x[0]+(-0.384317206475)*x[1]**o+(-0.854685760579)*x[1]+(0.502373892581)*x[2]**o+(-0.923561844368)*x[2]
        arg[3,4]=(-0.720555420423)*x[0]**o+(-0.115786593478)*x[0]+(0.832043243085)*x[1]**o+(-0.912631896773)*x[1]+(-0.583397798668)*x[2]**o+(-0.508174721594)*x[2]
        ref[0,0]=(-0.158713248354)/(o+1.)+(0.0447650956777)+(-0.916251178344)*0.5**o
        ref[0,1]=(1.67672067284)/(o+1.)+(-0.284845203298)+(0.394307707909)*0.5**o
        ref[0,2]=(0.753853002506)/(o+1.)+(0.23723910114)+(-0.0387282383528)*0.5**o
        ref[0,3]=(0.570705029999)/(o+1.)+(0.00671199076774)+(0.680006301334)*0.5**o
        ref[0,4]=(0.767706600479)/(o+1.)+(0.167812922325)+(-0.972790915366)*0.5**o
        ref[1,0]=(-0.0876045691728)/(o+1.)+(-0.322158501138)+(-0.105824689194)*0.5**o
        ref[1,1]=(1.00167617916)/(o+1.)+(0.42478735185)+(-0.378707957559)*0.5**o
        ref[1,2]=(0.0485722166212)/(o+1.)+(-0.0283225302105)+(-0.0226058491089)*0.5**o
        ref[1,3]=(0.520435646554)/(o+1.)+(-0.429308113497)+(-0.567270755897)*0.5**o
        ref[1,4]=(-1.25344090612)/(o+1.)+(0.31025156325)+(0.977251285351)*0.5**o
        ref[2,0]=(-0.191615046786)/(o+1.)+(0.501797936227)+(0.997778241906)*0.5**o
        ref[2,1]=(-0.0226895463818)/(o+1.)+(-0.544505297079)+(0.59766081214)*0.5**o
        ref[2,2]=(0.0380971422336)/(o+1.)+(-1.06058279972)+(0.633988266946)*0.5**o
        ref[2,3]=(-0.0283073099622)/(o+1.)+(0.721850053286)+(0.102332382261)*0.5**o
        ref[2,4]=(0.80406085149)/(o+1.)+(0.333920198971)+(0.928323090624)*0.5**o
        ref[3,0]=(-0.252099307076)/(o+1.)+(0.0343932248679)+(-0.988813741403)*0.5**o
        ref[3,1]=(0.504744936532)/(o+1.)+(-0.137201034138)+(-0.241202616681)*0.5**o
        ref[3,2]=(0.223607239442)/(o+1.)+(-0.174841810203)+(-0.397081811407)*0.5**o
        ref[3,3]=(0.118056686106)/(o+1.)+(-0.518465563043)+(-0.304262529036)*0.5**o
        ref[3,4]=(0.248645444416)/(o+1.)+(-0.768296605922)+(-0.720555420423)*0.5**o
      res=integrate(arg)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactOne_fromData_ReducedFunctionOnContactOne_rank3(self):
      """
      tests integral of rank 3 Data on the ReducedFunctionOnContactOne

      assumptions: ReducedFunctionOnContactOne(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      w=ReducedFunctionOnContactOne(self.domain)
      x=w.getX()
      arg=Data(0,(6, 2, 2),w)
      ref=numpy.zeros((6, 2, 2),numpy.float_)
      if dim==2:
        arg[0,0,0]=(-0.347273720148)*x[0]**o+(0.325185727721)*x[0]+(-0.855515194735)*x[1]**o+(0.825807975597)*x[1]
        arg[0,0,1]=(0.842054669437)*x[0]**o+(0.353768311241)*x[0]+(0.0148900864182)*x[1]**o+(-0.402154654143)*x[1]
        arg[0,1,0]=(-0.191525791617)*x[0]**o+(-0.299181220926)*x[0]+(-0.696260869529)*x[1]**o+(0.285645399154)*x[1]
        arg[0,1,1]=(0.190151633835)*x[0]**o+(0.184074338131)*x[0]+(-0.497689504982)*x[1]**o+(-0.813410371313)*x[1]
        arg[1,0,0]=(-0.695114302024)*x[0]**o+(0.427133485674)*x[0]+(-0.711899825683)*x[1]**o+(-0.296004734911)*x[1]
        arg[1,0,1]=(0.37620789583)*x[0]**o+(-0.944850444544)*x[0]+(0.666194812839)*x[1]**o+(-0.0715430628168)*x[1]
        arg[1,1,0]=(-0.0312291891427)*x[0]**o+(0.403009732443)*x[0]+(-0.378486005359)*x[1]**o+(-0.992729541609)*x[1]
        arg[1,1,1]=(0.0675984132698)*x[0]**o+(0.241001129867)*x[0]+(0.340546237588)*x[1]**o+(-0.024830810543)*x[1]
        arg[2,0,0]=(-0.146870758676)*x[0]**o+(-0.170434897147)*x[0]+(-0.579618469996)*x[1]**o+(-0.770817574749)*x[1]
        arg[2,0,1]=(-0.3029068201)*x[0]**o+(-0.110636302601)*x[0]+(-0.905841665338)*x[1]**o+(-0.280977886629)*x[1]
        arg[2,1,0]=(0.0547810600344)*x[0]**o+(0.00763321517272)*x[0]+(0.0156822194106)*x[1]**o+(-0.528337857094)*x[1]
        arg[2,1,1]=(0.12016647407)*x[0]**o+(0.4590743733)*x[0]+(-0.574499998403)*x[1]**o+(0.698871172503)*x[1]
        arg[3,0,0]=(0.481811757276)*x[0]**o+(0.748356476205)*x[0]+(-0.300272959749)*x[1]**o+(-0.707230263391)*x[1]
        arg[3,0,1]=(-0.347858576311)*x[0]**o+(-0.177810365974)*x[0]+(0.384658344075)*x[1]**o+(-0.837385668636)*x[1]
        arg[3,1,0]=(0.943559460135)*x[0]**o+(0.178778474835)*x[0]+(-0.831384000346)*x[1]**o+(-0.474975888568)*x[1]
        arg[3,1,1]=(-0.589903969401)*x[0]**o+(0.271114795948)*x[0]+(-0.946594221253)*x[1]**o+(-0.964029170264)*x[1]
        arg[4,0,0]=(0.221077049797)*x[0]**o+(0.986052914095)*x[0]+(-0.392982812685)*x[1]**o+(0.370828365061)*x[1]
        arg[4,0,1]=(0.463121232737)*x[0]**o+(-0.91920434872)*x[0]+(0.830655891827)*x[1]**o+(0.193876462558)*x[1]
        arg[4,1,0]=(0.925740559506)*x[0]**o+(0.534342942418)*x[0]+(0.446730463706)*x[1]**o+(0.181095371535)*x[1]
        arg[4,1,1]=(-0.777093674353)*x[0]**o+(-0.435130879793)*x[0]+(0.134406853212)*x[1]**o+(0.861099956528)*x[1]
        arg[5,0,0]=(-0.443943796359)*x[0]**o+(0.359190127208)*x[0]+(0.0570788998709)*x[1]**o+(0.548070594378)*x[1]
        arg[5,0,1]=(0.762962865085)*x[0]**o+(0.969248002849)*x[0]+(0.729222816543)*x[1]**o+(-0.865271983926)*x[1]
        arg[5,1,0]=(-0.915609563739)*x[0]**o+(-0.338560068696)*x[0]+(0.972145654544)*x[1]**o+(0.0941277349395)*x[1]
        arg[5,1,1]=(-0.486023356984)*x[0]**o+(-0.426382776921)*x[0]+(-0.114606039743)*x[1]**o+(-0.992152486495)*x[1]
        ref[0,0,0]=(-0.855515194735)/(o+1.)+(0.575496851659)+(-0.347273720148)*0.5**o
        ref[0,0,1]=(0.0148900864182)/(o+1.)+(-0.0241931714509)+(0.842054669437)*0.5**o
        ref[0,1,0]=(-0.696260869529)/(o+1.)+(-0.00676791088595)+(-0.191525791617)*0.5**o
        ref[0,1,1]=(-0.497689504982)/(o+1.)+(-0.314668016591)+(0.190151633835)*0.5**o
        ref[1,0,0]=(-0.711899825683)/(o+1.)+(0.0655643753813)+(-0.695114302024)*0.5**o
        ref[1,0,1]=(0.666194812839)/(o+1.)+(-0.50819675368)+(0.37620789583)*0.5**o
        ref[1,1,0]=(-0.378486005359)/(o+1.)+(-0.294859904583)+(-0.0312291891427)*0.5**o
        ref[1,1,1]=(0.340546237588)/(o+1.)+(0.108085159662)+(0.0675984132698)*0.5**o
        ref[2,0,0]=(-0.579618469996)/(o+1.)+(-0.470626235948)+(-0.146870758676)*0.5**o
        ref[2,0,1]=(-0.905841665338)/(o+1.)+(-0.195807094615)+(-0.3029068201)*0.5**o
        ref[2,1,0]=(0.0156822194106)/(o+1.)+(-0.260352320961)+(0.0547810600344)*0.5**o
        ref[2,1,1]=(-0.574499998403)/(o+1.)+(0.578972772902)+(0.12016647407)*0.5**o
        ref[3,0,0]=(-0.300272959749)/(o+1.)+(0.0205631064069)+(0.481811757276)*0.5**o
        ref[3,0,1]=(0.384658344075)/(o+1.)+(-0.507598017305)+(-0.347858576311)*0.5**o
        ref[3,1,0]=(-0.831384000346)/(o+1.)+(-0.148098706867)+(0.943559460135)*0.5**o
        ref[3,1,1]=(-0.946594221253)/(o+1.)+(-0.346457187158)+(-0.589903969401)*0.5**o
        ref[4,0,0]=(-0.392982812685)/(o+1.)+(0.678440639578)+(0.221077049797)*0.5**o
        ref[4,0,1]=(0.830655891827)/(o+1.)+(-0.362663943081)+(0.463121232737)*0.5**o
        ref[4,1,0]=(0.446730463706)/(o+1.)+(0.357719156977)+(0.925740559506)*0.5**o
        ref[4,1,1]=(0.134406853212)/(o+1.)+(0.212984538367)+(-0.777093674353)*0.5**o
        ref[5,0,0]=(0.0570788998709)/(o+1.)+(0.453630360793)+(-0.443943796359)*0.5**o
        ref[5,0,1]=(0.729222816543)/(o+1.)+(0.0519880094615)+(0.762962865085)*0.5**o
        ref[5,1,0]=(0.972145654544)/(o+1.)+(-0.122216166878)+(-0.915609563739)*0.5**o
        ref[5,1,1]=(-0.114606039743)/(o+1.)+(-0.709267631708)+(-0.486023356984)*0.5**o
      else:
        arg[0,0,0]=(0.455440122065)*x[0]**o+(0.911340571887)*x[0]+(0.158409397783)*x[1]**o+(0.353755257124)*x[1]+(-0.392556456322)*x[2]**o+(0.707232271095)*x[2]
        arg[0,0,1]=(-0.816405087957)*x[0]**o+(0.0691299991887)*x[0]+(0.178677657085)*x[1]**o+(0.526176369634)*x[1]+(-0.159459641444)*x[2]**o+(-0.197907778155)*x[2]
        arg[0,1,0]=(-0.463418217546)*x[0]**o+(-0.485835567862)*x[0]+(0.760458548482)*x[1]**o+(0.902873590314)*x[1]+(-0.760679635511)*x[2]**o+(-0.483163544794)*x[2]
        arg[0,1,1]=(0.743029549626)*x[0]**o+(-0.39131913217)*x[0]+(0.488809682734)*x[1]**o+(-0.256115251135)*x[1]+(0.0918611991036)*x[2]**o+(-0.281291060967)*x[2]
        arg[1,0,0]=(0.794367740289)*x[0]**o+(0.727663222288)*x[0]+(-0.631739003283)*x[1]**o+(0.733862609049)*x[1]+(-0.721230552103)*x[2]**o+(0.787774790111)*x[2]
        arg[1,0,1]=(0.505578181889)*x[0]**o+(0.616497234291)*x[0]+(-0.616903198005)*x[1]**o+(-0.355969913594)*x[1]+(0.0105854597103)*x[2]**o+(0.510273945857)*x[2]
        arg[1,1,0]=(0.35370601878)*x[0]**o+(-0.0414232536827)*x[0]+(0.0546619209334)*x[1]**o+(0.0459241421788)*x[1]+(0.796266758386)*x[2]**o+(0.303998757002)*x[2]
        arg[1,1,1]=(-0.245326163282)*x[0]**o+(0.0893045737315)*x[0]+(0.019401528413)*x[1]**o+(-0.838249399348)*x[1]+(0.510320088517)*x[2]**o+(0.544558918049)*x[2]
        arg[2,0,0]=(-0.660208039977)*x[0]**o+(-0.224874460802)*x[0]+(-0.0467836584153)*x[1]**o+(-0.463886703951)*x[1]+(-0.0951679104966)*x[2]**o+(-0.735787783779)*x[2]
        arg[2,0,1]=(-0.588259372806)*x[0]**o+(-0.18950606147)*x[0]+(-0.373439432467)*x[1]**o+(-0.116285718036)*x[1]+(-0.944687791765)*x[2]**o+(-0.243001707318)*x[2]
        arg[2,1,0]=(0.798498386971)*x[0]**o+(0.500125941024)*x[0]+(0.197339376576)*x[1]**o+(-0.068777545221)*x[1]+(0.36649695624)*x[2]**o+(0.193956440608)*x[2]
        arg[2,1,1]=(0.27501104189)*x[0]**o+(0.203066488761)*x[0]+(0.379278873515)*x[1]**o+(-0.789154057614)*x[1]+(0.893889499095)*x[2]**o+(0.867881975803)*x[2]
        arg[3,0,0]=(0.0579414247006)*x[0]**o+(0.661826954985)*x[0]+(0.313969775635)*x[1]**o+(-0.125420102972)*x[1]+(-0.808513922748)*x[2]**o+(0.0301073398588)*x[2]
        arg[3,0,1]=(-0.254544981247)*x[0]**o+(0.755983567644)*x[0]+(0.129524533092)*x[1]**o+(0.562828535682)*x[1]+(0.712881323083)*x[2]**o+(0.375450604678)*x[2]
        arg[3,1,0]=(-0.100221176995)*x[0]**o+(-0.470987646782)*x[0]+(-0.444245257427)*x[1]**o+(-0.0112626639353)*x[1]+(-0.516917833845)*x[2]**o+(-0.664158938756)*x[2]
        arg[3,1,1]=(0.259306174564)*x[0]**o+(0.367178744389)*x[0]+(0.177897569676)*x[1]**o+(-0.355724319562)*x[1]+(-0.809141944951)*x[2]**o+(0.962429694607)*x[2]
        arg[4,0,0]=(0.351990799011)*x[0]**o+(-0.41657457182)*x[0]+(-0.769748341564)*x[1]**o+(-0.856355494001)*x[1]+(-0.777902936204)*x[2]**o+(-0.388980567423)*x[2]
        arg[4,0,1]=(0.875307372684)*x[0]**o+(0.061375057221)*x[0]+(-0.262062525469)*x[1]**o+(0.0950324968281)*x[1]+(-0.145145751811)*x[2]**o+(-0.246872045818)*x[2]
        arg[4,1,0]=(0.115340652181)*x[0]**o+(0.910414483949)*x[0]+(0.347463744624)*x[1]**o+(0.399961393031)*x[1]+(0.0491863598064)*x[2]**o+(-0.992304094864)*x[2]
        arg[4,1,1]=(-0.125763267852)*x[0]**o+(0.485867057874)*x[0]+(-0.414172783931)*x[1]**o+(0.164145362806)*x[1]+(-0.488738184728)*x[2]**o+(-0.232758639076)*x[2]
        arg[5,0,0]=(-0.748184063441)*x[0]**o+(-0.252715593115)*x[0]+(0.688609235584)*x[1]**o+(0.0645665122312)*x[1]+(0.365465805991)*x[2]**o+(-0.217347848328)*x[2]
        arg[5,0,1]=(-0.298018437594)*x[0]**o+(-0.39683066279)*x[0]+(-0.0609074331294)*x[1]**o+(0.146930937933)*x[1]+(-0.00737018012329)*x[2]**o+(-0.700002160047)*x[2]
        arg[5,1,0]=(0.974781669859)*x[0]**o+(0.0634072121163)*x[0]+(0.0194044697894)*x[1]**o+(-0.919375545742)*x[1]+(-0.779806244021)*x[2]**o+(0.641761171945)*x[2]
        arg[5,1,1]=(0.485215976154)*x[0]**o+(0.879804212343)*x[0]+(-0.765170516406)*x[1]**o+(-0.205158612954)*x[1]+(-0.586708070331)*x[2]**o+(0.506670980992)*x[2]
        ref[0,0,0]=(-0.23414705854)/(o+1.)+(0.986164050053)+(0.455440122065)*0.5**o
        ref[0,0,1]=(0.0192180156412)/(o+1.)+(0.198699295334)+(-0.816405087957)*0.5**o
        ref[0,1,0]=(-0.000221087029094)/(o+1.)+(-0.0330627611712)+(-0.463418217546)*0.5**o
        ref[0,1,1]=(0.580670881837)/(o+1.)+(-0.464362722135)+(0.743029549626)*0.5**o
        ref[1,0,0]=(-1.35296955539)/(o+1.)+(1.12465031072)+(0.794367740289)*0.5**o
        ref[1,0,1]=(-0.606317738295)/(o+1.)+(0.385400633277)+(0.505578181889)*0.5**o
        ref[1,1,0]=(0.85092867932)/(o+1.)+(0.154249822749)+(0.35370601878)*0.5**o
        ref[1,1,1]=(0.52972161693)/(o+1.)+(-0.102192953784)+(-0.245326163282)*0.5**o
        ref[2,0,0]=(-0.141951568912)/(o+1.)+(-0.712274474266)+(-0.660208039977)*0.5**o
        ref[2,0,1]=(-1.31812722423)/(o+1.)+(-0.274396743412)+(-0.588259372806)*0.5**o
        ref[2,1,0]=(0.563836332815)/(o+1.)+(0.312652418205)+(0.798498386971)*0.5**o
        ref[2,1,1]=(1.27316837261)/(o+1.)+(0.140897203475)+(0.27501104189)*0.5**o
        ref[3,0,0]=(-0.494544147113)/(o+1.)+(0.283257095936)+(0.0579414247006)*0.5**o
        ref[3,0,1]=(0.842405856175)/(o+1.)+(0.847131354002)+(-0.254544981247)*0.5**o
        ref[3,1,0]=(-0.961163091272)/(o+1.)+(-0.573204624737)+(-0.100221176995)*0.5**o
        ref[3,1,1]=(-0.631244375275)/(o+1.)+(0.486942059717)+(0.259306174564)*0.5**o
        ref[4,0,0]=(-1.54765127777)/(o+1.)+(-0.830955316622)+(0.351990799011)*0.5**o
        ref[4,0,1]=(-0.407208277279)/(o+1.)+(-0.0452322458845)+(0.875307372684)*0.5**o
        ref[4,1,0]=(0.39665010443)/(o+1.)+(0.159035891058)+(0.115340652181)*0.5**o
        ref[4,1,1]=(-0.902910968659)/(o+1.)+(0.208626890802)+(-0.125763267852)*0.5**o
        ref[5,0,0]=(1.05407504158)/(o+1.)+(-0.202748464606)+(-0.748184063441)*0.5**o
        ref[5,0,1]=(-0.0682776132527)/(o+1.)+(-0.474950942452)+(-0.298018437594)*0.5**o
        ref[5,1,0]=(-0.760401774232)/(o+1.)+(-0.10710358084)+(0.974781669859)*0.5**o
        ref[5,1,1]=(-1.35187858674)/(o+1.)+(0.590658290191)+(0.485215976154)*0.5**o
      res=integrate(arg)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactOne_fromData_ReducedFunctionOnContactOne_rank4(self):
      """
      tests integral of rank 4 Data on the ReducedFunctionOnContactOne

      assumptions: ReducedFunctionOnContactOne(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      w=ReducedFunctionOnContactOne(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5, 3, 2),w)
      ref=numpy.zeros((4, 5, 3, 2),numpy.float_)
      if dim==2:
        arg[0,0,0,0]=(-0.177078428867)*x[0]**o+(0.338116995819)*x[0]+(-0.750823220445)*x[1]**o+(0.0630074420245)*x[1]
        arg[0,0,0,1]=(-0.682838671264)*x[0]**o+(-0.893797611147)*x[0]+(-0.0127659464437)*x[1]**o+(0.144120749888)*x[1]
        arg[0,0,1,0]=(0.352439319413)*x[0]**o+(0.380064866457)*x[0]+(-0.932452125435)*x[1]**o+(0.914519937443)*x[1]
        arg[0,0,1,1]=(-0.354033408023)*x[0]**o+(0.771069338542)*x[0]+(-0.702758889809)*x[1]**o+(-0.484485925087)*x[1]
        arg[0,0,2,0]=(0.706826713479)*x[0]**o+(0.0526230471775)*x[0]+(-0.535244256862)*x[1]**o+(-0.085929844746)*x[1]
        arg[0,0,2,1]=(-0.893497856528)*x[0]**o+(-0.892309840177)*x[0]+(0.742770093639)*x[1]**o+(0.987917060326)*x[1]
        arg[0,1,0,0]=(-0.536624329097)*x[0]**o+(-0.569536051343)*x[0]+(-0.7740511254)*x[1]**o+(0.873150177653)*x[1]
        arg[0,1,0,1]=(0.388396767346)*x[0]**o+(-0.902569510334)*x[0]+(0.196975646933)*x[1]**o+(0.184178333918)*x[1]
        arg[0,1,1,0]=(-0.125771297178)*x[0]**o+(-0.335093336446)*x[0]+(0.372752949038)*x[1]**o+(-0.883928748878)*x[1]
        arg[0,1,1,1]=(-0.313637256508)*x[0]**o+(-0.49607955109)*x[0]+(-0.726599229867)*x[1]**o+(0.360837115079)*x[1]
        arg[0,1,2,0]=(0.92425817047)*x[0]**o+(0.734728600171)*x[0]+(1.60583902129e-05)*x[1]**o+(-0.124600971339)*x[1]
        arg[0,1,2,1]=(-0.0981827607557)*x[0]**o+(-0.556005942436)*x[0]+(-0.888060617134)*x[1]**o+(0.857508807636)*x[1]
        arg[0,2,0,0]=(-0.871590768072)*x[0]**o+(-0.208546938841)*x[0]+(-0.228336105988)*x[1]**o+(0.444098195066)*x[1]
        arg[0,2,0,1]=(-0.628675127006)*x[0]**o+(0.834043549769)*x[0]+(-0.509662470381)*x[1]**o+(-0.801907226651)*x[1]
        arg[0,2,1,0]=(-0.767528433339)*x[0]**o+(0.305967685996)*x[0]+(-0.587902586916)*x[1]**o+(0.38652781193)*x[1]
        arg[0,2,1,1]=(0.388984380682)*x[0]**o+(0.490673256209)*x[0]+(-0.78470263647)*x[1]**o+(0.34909383158)*x[1]
        arg[0,2,2,0]=(-0.910514531543)*x[0]**o+(-0.418810366626)*x[0]+(-0.637159792471)*x[1]**o+(0.794653160897)*x[1]
        arg[0,2,2,1]=(0.570535552656)*x[0]**o+(-0.590385806831)*x[0]+(-0.298806908011)*x[1]**o+(-0.901531327347)*x[1]
        arg[0,3,0,0]=(0.916864925136)*x[0]**o+(-0.019733476459)*x[0]+(-0.91370863065)*x[1]**o+(0.291932128733)*x[1]
        arg[0,3,0,1]=(-0.851147484632)*x[0]**o+(-0.159304969602)*x[0]+(0.699832871063)*x[1]**o+(0.327932729929)*x[1]
        arg[0,3,1,0]=(0.461273862468)*x[0]**o+(0.40068516653)*x[0]+(0.0203170502559)*x[1]**o+(0.973716167207)*x[1]
        arg[0,3,1,1]=(0.63984877633)*x[0]**o+(0.580871491133)*x[0]+(-0.00587359808953)*x[1]**o+(-0.104805621161)*x[1]
        arg[0,3,2,0]=(-0.159416471436)*x[0]**o+(-0.585039922994)*x[0]+(0.689553978928)*x[1]**o+(0.569617375406)*x[1]
        arg[0,3,2,1]=(0.782397836312)*x[0]**o+(-0.581894025693)*x[0]+(0.0370277809126)*x[1]**o+(0.597309139432)*x[1]
        arg[0,4,0,0]=(-0.0063433205485)*x[0]**o+(-0.242291627085)*x[0]+(0.110218625254)*x[1]**o+(0.819656578408)*x[1]
        arg[0,4,0,1]=(-0.128215646666)*x[0]**o+(0.821312510893)*x[0]+(0.910723121611)*x[1]**o+(-0.0174623980585)*x[1]
        arg[0,4,1,0]=(-0.604528735513)*x[0]**o+(-0.358705990504)*x[0]+(-0.281954207008)*x[1]**o+(0.123469639952)*x[1]
        arg[0,4,1,1]=(0.65096807032)*x[0]**o+(0.882935341204)*x[0]+(-0.761226249914)*x[1]**o+(-0.830800218214)*x[1]
        arg[0,4,2,0]=(-0.710718948634)*x[0]**o+(0.452128874613)*x[0]+(-0.780671499903)*x[1]**o+(-0.942033435912)*x[1]
        arg[0,4,2,1]=(-0.0443187506467)*x[0]**o+(0.513943853321)*x[0]+(0.709902333411)*x[1]**o+(0.755430555318)*x[1]
        arg[1,0,0,0]=(0.731890221753)*x[0]**o+(-0.821835735249)*x[0]+(-0.713098084853)*x[1]**o+(0.875738077525)*x[1]
        arg[1,0,0,1]=(-0.130335125365)*x[0]**o+(0.580579210083)*x[0]+(-0.427105331069)*x[1]**o+(0.0435798068353)*x[1]
        arg[1,0,1,0]=(-0.203908509107)*x[0]**o+(-0.941974700448)*x[0]+(0.344459670502)*x[1]**o+(0.758336135081)*x[1]
        arg[1,0,1,1]=(0.186770929724)*x[0]**o+(0.348332058705)*x[0]+(-0.0683630778679)*x[1]**o+(0.480425212189)*x[1]
        arg[1,0,2,0]=(-0.891275312975)*x[0]**o+(0.260630881729)*x[0]+(0.127955472707)*x[1]**o+(-0.554992279729)*x[1]
        arg[1,0,2,1]=(-0.74620344519)*x[0]**o+(0.944025669845)*x[0]+(-0.052031562679)*x[1]**o+(0.966060927262)*x[1]
        arg[1,1,0,0]=(0.871423198418)*x[0]**o+(-0.0769723128617)*x[0]+(0.783714758356)*x[1]**o+(0.530209967997)*x[1]
        arg[1,1,0,1]=(0.943197824902)*x[0]**o+(-0.62624855429)*x[0]+(-0.00888785913795)*x[1]**o+(-0.0408857497082)*x[1]
        arg[1,1,1,0]=(-0.74971981056)*x[0]**o+(0.925175163834)*x[0]+(0.481063204532)*x[1]**o+(-0.476700447185)*x[1]
        arg[1,1,1,1]=(-0.431776603043)*x[0]**o+(-0.495358165507)*x[0]+(-0.0110588803484)*x[1]**o+(-0.575147325496)*x[1]
        arg[1,1,2,0]=(0.578105297721)*x[0]**o+(-0.88169100464)*x[0]+(-0.646224518073)*x[1]**o+(0.465619837803)*x[1]
        arg[1,1,2,1]=(0.442066689582)*x[0]**o+(0.0937839719519)*x[0]+(-0.101534585934)*x[1]**o+(-0.974003442605)*x[1]
        arg[1,2,0,0]=(-0.206702379228)*x[0]**o+(0.330161049866)*x[0]+(0.185493587693)*x[1]**o+(0.0391986992855)*x[1]
        arg[1,2,0,1]=(0.79092697362)*x[0]**o+(0.692054695304)*x[0]+(0.141622971369)*x[1]**o+(0.643869474153)*x[1]
        arg[1,2,1,0]=(0.914281902407)*x[0]**o+(0.936365302055)*x[0]+(0.249095964198)*x[1]**o+(-0.440892551355)*x[1]
        arg[1,2,1,1]=(-0.602926709108)*x[0]**o+(0.817198874641)*x[0]+(0.262528290838)*x[1]**o+(0.493607450371)*x[1]
        arg[1,2,2,0]=(-0.724435531493)*x[0]**o+(-0.455127093687)*x[0]+(0.144972402754)*x[1]**o+(-0.361255031527)*x[1]
        arg[1,2,2,1]=(-0.611538590545)*x[0]**o+(-0.780730652602)*x[0]+(-0.150439352532)*x[1]**o+(0.450875229163)*x[1]
        arg[1,3,0,0]=(-0.913553947133)*x[0]**o+(-0.930172924067)*x[0]+(-0.509426399026)*x[1]**o+(0.3953360754)*x[1]
        arg[1,3,0,1]=(-0.709834132519)*x[0]**o+(0.972632526254)*x[0]+(0.540358729462)*x[1]**o+(-0.0295254269136)*x[1]
        arg[1,3,1,0]=(-0.30788139278)*x[0]**o+(-0.362561853154)*x[0]+(0.0135015706177)*x[1]**o+(0.558034983013)*x[1]
        arg[1,3,1,1]=(-0.243327801257)*x[0]**o+(-0.156242539226)*x[0]+(0.076082439418)*x[1]**o+(0.818613998695)*x[1]
        arg[1,3,2,0]=(-0.0290765899224)*x[0]**o+(-0.108604693002)*x[0]+(-0.0741034599215)*x[1]**o+(0.44467098076)*x[1]
        arg[1,3,2,1]=(-0.751374584848)*x[0]**o+(0.568700665267)*x[0]+(-0.431665912498)*x[1]**o+(0.881337230634)*x[1]
        arg[1,4,0,0]=(-0.940864464113)*x[0]**o+(-0.204392585977)*x[0]+(-0.872729326815)*x[1]**o+(0.454069061239)*x[1]
        arg[1,4,0,1]=(0.620544898911)*x[0]**o+(-0.194050689442)*x[0]+(0.399921161647)*x[1]**o+(0.812456444893)*x[1]
        arg[1,4,1,0]=(-0.398096053023)*x[0]**o+(-0.521802671926)*x[0]+(-0.525522030503)*x[1]**o+(-0.795276120062)*x[1]
        arg[1,4,1,1]=(0.277239330261)*x[0]**o+(-0.376159942175)*x[0]+(0.785902674573)*x[1]**o+(-0.00686958652716)*x[1]
        arg[1,4,2,0]=(-0.469242672532)*x[0]**o+(0.289432797089)*x[0]+(0.728305062026)*x[1]**o+(0.821761742668)*x[1]
        arg[1,4,2,1]=(0.828113301834)*x[0]**o+(-0.149301261166)*x[0]+(0.279230829643)*x[1]**o+(-0.971944918618)*x[1]
        arg[2,0,0,0]=(-0.724500208669)*x[0]**o+(-0.761179166402)*x[0]+(0.302409732402)*x[1]**o+(0.509101646177)*x[1]
        arg[2,0,0,1]=(-0.133300605917)*x[0]**o+(-0.0299057843108)*x[0]+(0.190028790905)*x[1]**o+(-0.668097775221)*x[1]
        arg[2,0,1,0]=(0.828019154028)*x[0]**o+(-0.446721913762)*x[0]+(0.745204383075)*x[1]**o+(0.363968853359)*x[1]
        arg[2,0,1,1]=(-0.653962912288)*x[0]**o+(0.753529804971)*x[0]+(-0.104712073949)*x[1]**o+(-0.44152589042)*x[1]
        arg[2,0,2,0]=(0.21572653818)*x[0]**o+(-0.553685840495)*x[0]+(0.372296357092)*x[1]**o+(-0.23566755183)*x[1]
        arg[2,0,2,1]=(-0.0943992743407)*x[0]**o+(0.677836956763)*x[0]+(-0.33055202826)*x[1]**o+(0.444414877097)*x[1]
        arg[2,1,0,0]=(-0.812500024009)*x[0]**o+(0.4856670967)*x[0]+(-0.891481528328)*x[1]**o+(-0.244043957515)*x[1]
        arg[2,1,0,1]=(-0.958152880518)*x[0]**o+(0.494470945352)*x[0]+(-0.000696143847979)*x[1]**o+(-0.036680781152)*x[1]
        arg[2,1,1,0]=(-0.81109120479)*x[0]**o+(-0.929246624754)*x[0]+(0.128356074186)*x[1]**o+(0.0107006959497)*x[1]
        arg[2,1,1,1]=(-0.72470858095)*x[0]**o+(-0.202254265463)*x[0]+(-0.995176229907)*x[1]**o+(-0.752592973024)*x[1]
        arg[2,1,2,0]=(-0.191810147267)*x[0]**o+(-0.819397723879)*x[0]+(0.306860971474)*x[1]**o+(0.681274705052)*x[1]
        arg[2,1,2,1]=(-0.867447323913)*x[0]**o+(-0.279148200325)*x[0]+(-0.195907590382)*x[1]**o+(-0.305826952304)*x[1]
        arg[2,2,0,0]=(0.983782298896)*x[0]**o+(-0.284064487432)*x[0]+(-0.0928333646602)*x[1]**o+(-0.929708520581)*x[1]
        arg[2,2,0,1]=(0.121619221979)*x[0]**o+(-0.876309915691)*x[0]+(0.455030678927)*x[1]**o+(0.635763125356)*x[1]
        arg[2,2,1,0]=(0.378726440151)*x[0]**o+(-0.626329947078)*x[0]+(-0.0239503446136)*x[1]**o+(0.32696940295)*x[1]
        arg[2,2,1,1]=(0.442586981104)*x[0]**o+(-0.427714534417)*x[0]+(0.906235883406)*x[1]**o+(0.819595435756)*x[1]
        arg[2,2,2,0]=(-0.564560890159)*x[0]**o+(-0.115075498726)*x[0]+(-0.520596320159)*x[1]**o+(-0.368366112222)*x[1]
        arg[2,2,2,1]=(0.215858165597)*x[0]**o+(-0.575665537635)*x[0]+(0.792289256264)*x[1]**o+(0.215575294284)*x[1]
        arg[2,3,0,0]=(-0.596335978041)*x[0]**o+(-0.677983108509)*x[0]+(-0.551360906631)*x[1]**o+(-0.874453959198)*x[1]
        arg[2,3,0,1]=(-0.913806992124)*x[0]**o+(0.192343143898)*x[0]+(0.821070238869)*x[1]**o+(-0.188359117641)*x[1]
        arg[2,3,1,0]=(-0.0831972104302)*x[0]**o+(-0.927005811846)*x[0]+(-0.43240265235)*x[1]**o+(-0.144332400422)*x[1]
        arg[2,3,1,1]=(0.643878326697)*x[0]**o+(0.212328499471)*x[0]+(0.72072318435)*x[1]**o+(-0.146144387035)*x[1]
        arg[2,3,2,0]=(0.843652876743)*x[0]**o+(-0.0386075132797)*x[0]+(0.754511827443)*x[1]**o+(-0.26945353599)*x[1]
        arg[2,3,2,1]=(0.0413857115672)*x[0]**o+(0.682727309008)*x[0]+(0.261249117327)*x[1]**o+(-0.753970539308)*x[1]
        arg[2,4,0,0]=(-0.607477693828)*x[0]**o+(-0.5870344058)*x[0]+(-0.0678823396451)*x[1]**o+(0.201869223616)*x[1]
        arg[2,4,0,1]=(-0.618172945146)*x[0]**o+(0.685084114438)*x[0]+(0.0457149578306)*x[1]**o+(-0.877971038996)*x[1]
        arg[2,4,1,0]=(0.722668769621)*x[0]**o+(0.302142466253)*x[0]+(0.630735604709)*x[1]**o+(-0.427754766567)*x[1]
        arg[2,4,1,1]=(-0.847113860795)*x[0]**o+(-0.779747246887)*x[0]+(-0.693659242703)*x[1]**o+(0.897451884568)*x[1]
        arg[2,4,2,0]=(-0.437578500878)*x[0]**o+(0.399407036331)*x[0]+(0.705207362097)*x[1]**o+(-0.271060427211)*x[1]
        arg[2,4,2,1]=(0.329570005179)*x[0]**o+(-0.712089906899)*x[0]+(0.860707329294)*x[1]**o+(0.553218273352)*x[1]
        arg[3,0,0,0]=(0.932602317046)*x[0]**o+(-0.948445141026)*x[0]+(0.549409370786)*x[1]**o+(0.782964260926)*x[1]
        arg[3,0,0,1]=(0.975116319103)*x[0]**o+(-0.0131313829738)*x[0]+(0.222376331882)*x[1]**o+(0.545375534163)*x[1]
        arg[3,0,1,0]=(0.019317277942)*x[0]**o+(0.918003018679)*x[0]+(-0.481194397466)*x[1]**o+(0.762780571026)*x[1]
        arg[3,0,1,1]=(-0.393162543578)*x[0]**o+(0.984304464298)*x[0]+(0.737901448601)*x[1]**o+(-0.890589811376)*x[1]
        arg[3,0,2,0]=(0.458096711279)*x[0]**o+(0.0343270226524)*x[0]+(0.278242150139)*x[1]**o+(-0.158378755134)*x[1]
        arg[3,0,2,1]=(0.307111052767)*x[0]**o+(-0.294724312785)*x[0]+(-0.489134379327)*x[1]**o+(-0.211139185303)*x[1]
        arg[3,1,0,0]=(0.420707084879)*x[0]**o+(-0.998173294986)*x[0]+(0.508808683)*x[1]**o+(-0.189902487212)*x[1]
        arg[3,1,0,1]=(-0.745253594401)*x[0]**o+(-0.344809947215)*x[0]+(0.198212694401)*x[1]**o+(-0.125428245606)*x[1]
        arg[3,1,1,0]=(0.535215252931)*x[0]**o+(0.565625295858)*x[0]+(-0.0741524230667)*x[1]**o+(-0.898600294889)*x[1]
        arg[3,1,1,1]=(0.193417593556)*x[0]**o+(-0.80053921694)*x[0]+(-0.664372557476)*x[1]**o+(0.47786060257)*x[1]
        arg[3,1,2,0]=(0.553690237434)*x[0]**o+(0.970763166888)*x[0]+(0.408815221365)*x[1]**o+(0.143175119269)*x[1]
        arg[3,1,2,1]=(-0.211683249218)*x[0]**o+(0.0643055656631)*x[0]+(-0.111028404397)*x[1]**o+(0.506093872018)*x[1]
        arg[3,2,0,0]=(-0.866119051438)*x[0]**o+(-0.417310807983)*x[0]+(-0.228385234131)*x[1]**o+(0.554977378653)*x[1]
        arg[3,2,0,1]=(-0.146618266885)*x[0]**o+(0.817733306502)*x[0]+(-0.709059272627)*x[1]**o+(-0.271140815458)*x[1]
        arg[3,2,1,0]=(0.622973500199)*x[0]**o+(-0.568858605993)*x[0]+(-0.651609755171)*x[1]**o+(-0.866184190672)*x[1]
        arg[3,2,1,1]=(-0.538565077894)*x[0]**o+(-0.676237465393)*x[0]+(0.106659392436)*x[1]**o+(0.488735158765)*x[1]
        arg[3,2,2,0]=(-0.325897413686)*x[0]**o+(0.99502293321)*x[0]+(-0.339096900793)*x[1]**o+(0.238083096263)*x[1]
        arg[3,2,2,1]=(0.861430484293)*x[0]**o+(-0.761739126386)*x[0]+(-0.10651751912)*x[1]**o+(0.688993400068)*x[1]
        arg[3,3,0,0]=(0.0890838652267)*x[0]**o+(0.151733228604)*x[0]+(0.230063809846)*x[1]**o+(-0.135041740614)*x[1]
        arg[3,3,0,1]=(-0.969251937403)*x[0]**o+(0.345709865473)*x[0]+(-0.309971427898)*x[1]**o+(-0.262217856737)*x[1]
        arg[3,3,1,0]=(0.640075232231)*x[0]**o+(-0.00817560718155)*x[0]+(0.55773788655)*x[1]**o+(0.642205213407)*x[1]
        arg[3,3,1,1]=(-0.350002630494)*x[0]**o+(0.373270925939)*x[0]+(-0.655757122598)*x[1]**o+(-0.74598967873)*x[1]
        arg[3,3,2,0]=(-0.262399316954)*x[0]**o+(-0.37991094527)*x[0]+(0.310310045807)*x[1]**o+(0.838942372822)*x[1]
        arg[3,3,2,1]=(-0.977039760257)*x[0]**o+(-0.0804751651971)*x[0]+(-0.844329846245)*x[1]**o+(0.905827257908)*x[1]
        arg[3,4,0,0]=(0.749594099085)*x[0]**o+(0.382880202285)*x[0]+(0.39699250956)*x[1]**o+(-0.708672318437)*x[1]
        arg[3,4,0,1]=(0.913703789336)*x[0]**o+(-0.497792024208)*x[0]+(-0.34125011876)*x[1]**o+(-0.657159430634)*x[1]
        arg[3,4,1,0]=(-0.397439923687)*x[0]**o+(0.595274583577)*x[0]+(-0.292338728991)*x[1]**o+(-0.314101057271)*x[1]
        arg[3,4,1,1]=(-0.403584993513)*x[0]**o+(-0.758728905384)*x[0]+(-0.182501473325)*x[1]**o+(-0.0942108622967)*x[1]
        arg[3,4,2,0]=(-0.263608698751)*x[0]**o+(-0.591451983536)*x[0]+(0.620412677389)*x[1]**o+(0.600005456466)*x[1]
        arg[3,4,2,1]=(-0.633502278259)*x[0]**o+(0.733127573934)*x[0]+(-0.765468966987)*x[1]**o+(-0.24211874772)*x[1]
        ref[0,0,0,0]=(-0.750823220445)/(o+1.)+(0.200562218922)+(-0.177078428867)*0.5**o
        ref[0,0,0,1]=(-0.0127659464437)/(o+1.)+(-0.374838430629)+(-0.682838671264)*0.5**o
        ref[0,0,1,0]=(-0.932452125435)/(o+1.)+(0.64729240195)+(0.352439319413)*0.5**o
        ref[0,0,1,1]=(-0.702758889809)/(o+1.)+(0.143291706728)+(-0.354033408023)*0.5**o
        ref[0,0,2,0]=(-0.535244256862)/(o+1.)+(-0.0166533987843)+(0.706826713479)*0.5**o
        ref[0,0,2,1]=(0.742770093639)/(o+1.)+(0.0478036100749)+(-0.893497856528)*0.5**o
        ref[0,1,0,0]=(-0.7740511254)/(o+1.)+(0.151807063155)+(-0.536624329097)*0.5**o
        ref[0,1,0,1]=(0.196975646933)/(o+1.)+(-0.359195588208)+(0.388396767346)*0.5**o
        ref[0,1,1,0]=(0.372752949038)/(o+1.)+(-0.609511042662)+(-0.125771297178)*0.5**o
        ref[0,1,1,1]=(-0.726599229867)/(o+1.)+(-0.0676212180057)+(-0.313637256508)*0.5**o
        ref[0,1,2,0]=(1.60583902129e-05)/(o+1.)+(0.305063814416)+(0.92425817047)*0.5**o
        ref[0,1,2,1]=(-0.888060617134)/(o+1.)+(0.1507514326)+(-0.0981827607557)*0.5**o
        ref[0,2,0,0]=(-0.228336105988)/(o+1.)+(0.117775628113)+(-0.871590768072)*0.5**o
        ref[0,2,0,1]=(-0.509662470381)/(o+1.)+(0.0160681615592)+(-0.628675127006)*0.5**o
        ref[0,2,1,0]=(-0.587902586916)/(o+1.)+(0.346247748963)+(-0.767528433339)*0.5**o
        ref[0,2,1,1]=(-0.78470263647)/(o+1.)+(0.419883543895)+(0.388984380682)*0.5**o
        ref[0,2,2,0]=(-0.637159792471)/(o+1.)+(0.187921397135)+(-0.910514531543)*0.5**o
        ref[0,2,2,1]=(-0.298806908011)/(o+1.)+(-0.745958567089)+(0.570535552656)*0.5**o
        ref[0,3,0,0]=(-0.91370863065)/(o+1.)+(0.136099326137)+(0.916864925136)*0.5**o
        ref[0,3,0,1]=(0.699832871063)/(o+1.)+(0.0843138801635)+(-0.851147484632)*0.5**o
        ref[0,3,1,0]=(0.0203170502559)/(o+1.)+(0.687200666868)+(0.461273862468)*0.5**o
        ref[0,3,1,1]=(-0.00587359808953)/(o+1.)+(0.238032934986)+(0.63984877633)*0.5**o
        ref[0,3,2,0]=(0.689553978928)/(o+1.)+(-0.00771127379434)+(-0.159416471436)*0.5**o
        ref[0,3,2,1]=(0.0370277809126)/(o+1.)+(0.00770755686953)+(0.782397836312)*0.5**o
        ref[0,4,0,0]=(0.110218625254)/(o+1.)+(0.288682475662)+(-0.0063433205485)*0.5**o
        ref[0,4,0,1]=(0.910723121611)/(o+1.)+(0.401925056417)+(-0.128215646666)*0.5**o
        ref[0,4,1,0]=(-0.281954207008)/(o+1.)+(-0.117618175276)+(-0.604528735513)*0.5**o
        ref[0,4,1,1]=(-0.761226249914)/(o+1.)+(0.0260675614951)+(0.65096807032)*0.5**o
        ref[0,4,2,0]=(-0.780671499903)/(o+1.)+(-0.244952280649)+(-0.710718948634)*0.5**o
        ref[0,4,2,1]=(0.709902333411)/(o+1.)+(0.63468720432)+(-0.0443187506467)*0.5**o
        ref[1,0,0,0]=(-0.713098084853)/(o+1.)+(0.026951171138)+(0.731890221753)*0.5**o
        ref[1,0,0,1]=(-0.427105331069)/(o+1.)+(0.312079508459)+(-0.130335125365)*0.5**o
        ref[1,0,1,0]=(0.344459670502)/(o+1.)+(-0.0918192826838)+(-0.203908509107)*0.5**o
        ref[1,0,1,1]=(-0.0683630778679)/(o+1.)+(0.414378635447)+(0.186770929724)*0.5**o
        ref[1,0,2,0]=(0.127955472707)/(o+1.)+(-0.147180699)+(-0.891275312975)*0.5**o
        ref[1,0,2,1]=(-0.052031562679)/(o+1.)+(0.955043298554)+(-0.74620344519)*0.5**o
        ref[1,1,0,0]=(0.783714758356)/(o+1.)+(0.226618827568)+(0.871423198418)*0.5**o
        ref[1,1,0,1]=(-0.00888785913795)/(o+1.)+(-0.333567151999)+(0.943197824902)*0.5**o
        ref[1,1,1,0]=(0.481063204532)/(o+1.)+(0.224237358325)+(-0.74971981056)*0.5**o
        ref[1,1,1,1]=(-0.0110588803484)/(o+1.)+(-0.535252745501)+(-0.431776603043)*0.5**o
        ref[1,1,2,0]=(-0.646224518073)/(o+1.)+(-0.208035583419)+(0.578105297721)*0.5**o
        ref[1,1,2,1]=(-0.101534585934)/(o+1.)+(-0.440109735326)+(0.442066689582)*0.5**o
        ref[1,2,0,0]=(0.185493587693)/(o+1.)+(0.184679874576)+(-0.206702379228)*0.5**o
        ref[1,2,0,1]=(0.141622971369)/(o+1.)+(0.667962084728)+(0.79092697362)*0.5**o
        ref[1,2,1,0]=(0.249095964198)/(o+1.)+(0.24773637535)+(0.914281902407)*0.5**o
        ref[1,2,1,1]=(0.262528290838)/(o+1.)+(0.655403162506)+(-0.602926709108)*0.5**o
        ref[1,2,2,0]=(0.144972402754)/(o+1.)+(-0.408191062607)+(-0.724435531493)*0.5**o
        ref[1,2,2,1]=(-0.150439352532)/(o+1.)+(-0.16492771172)+(-0.611538590545)*0.5**o
        ref[1,3,0,0]=(-0.509426399026)/(o+1.)+(-0.267418424334)+(-0.913553947133)*0.5**o
        ref[1,3,0,1]=(0.540358729462)/(o+1.)+(0.47155354967)+(-0.709834132519)*0.5**o
        ref[1,3,1,0]=(0.0135015706177)/(o+1.)+(0.0977365649293)+(-0.30788139278)*0.5**o
        ref[1,3,1,1]=(0.076082439418)/(o+1.)+(0.331185729735)+(-0.243327801257)*0.5**o
        ref[1,3,2,0]=(-0.0741034599215)/(o+1.)+(0.168033143879)+(-0.0290765899224)*0.5**o
        ref[1,3,2,1]=(-0.431665912498)/(o+1.)+(0.72501894795)+(-0.751374584848)*0.5**o
        ref[1,4,0,0]=(-0.872729326815)/(o+1.)+(0.124838237631)+(-0.940864464113)*0.5**o
        ref[1,4,0,1]=(0.399921161647)/(o+1.)+(0.309202877726)+(0.620544898911)*0.5**o
        ref[1,4,1,0]=(-0.525522030503)/(o+1.)+(-0.658539395994)+(-0.398096053023)*0.5**o
        ref[1,4,1,1]=(0.785902674573)/(o+1.)+(-0.191514764351)+(0.277239330261)*0.5**o
        ref[1,4,2,0]=(0.728305062026)/(o+1.)+(0.555597269878)+(-0.469242672532)*0.5**o
        ref[1,4,2,1]=(0.279230829643)/(o+1.)+(-0.560623089892)+(0.828113301834)*0.5**o
        ref[2,0,0,0]=(0.302409732402)/(o+1.)+(-0.126038760113)+(-0.724500208669)*0.5**o
        ref[2,0,0,1]=(0.190028790905)/(o+1.)+(-0.349001779766)+(-0.133300605917)*0.5**o
        ref[2,0,1,0]=(0.745204383075)/(o+1.)+(-0.0413765302013)+(0.828019154028)*0.5**o
        ref[2,0,1,1]=(-0.104712073949)/(o+1.)+(0.156001957276)+(-0.653962912288)*0.5**o
        ref[2,0,2,0]=(0.372296357092)/(o+1.)+(-0.394676696163)+(0.21572653818)*0.5**o
        ref[2,0,2,1]=(-0.33055202826)/(o+1.)+(0.56112591693)+(-0.0943992743407)*0.5**o
        ref[2,1,0,0]=(-0.891481528328)/(o+1.)+(0.120811569592)+(-0.812500024009)*0.5**o
        ref[2,1,0,1]=(-0.000696143847979)/(o+1.)+(0.2288950821)+(-0.958152880518)*0.5**o
        ref[2,1,1,0]=(0.128356074186)/(o+1.)+(-0.459272964402)+(-0.81109120479)*0.5**o
        ref[2,1,1,1]=(-0.995176229907)/(o+1.)+(-0.477423619244)+(-0.72470858095)*0.5**o
        ref[2,1,2,0]=(0.306860971474)/(o+1.)+(-0.0690615094134)+(-0.191810147267)*0.5**o
        ref[2,1,2,1]=(-0.195907590382)/(o+1.)+(-0.292487576314)+(-0.867447323913)*0.5**o
        ref[2,2,0,0]=(-0.0928333646602)/(o+1.)+(-0.606886504006)+(0.983782298896)*0.5**o
        ref[2,2,0,1]=(0.455030678927)/(o+1.)+(-0.120273395167)+(0.121619221979)*0.5**o
        ref[2,2,1,0]=(-0.0239503446136)/(o+1.)+(-0.149680272064)+(0.378726440151)*0.5**o
        ref[2,2,1,1]=(0.906235883406)/(o+1.)+(0.19594045067)+(0.442586981104)*0.5**o
        ref[2,2,2,0]=(-0.520596320159)/(o+1.)+(-0.241720805474)+(-0.564560890159)*0.5**o
        ref[2,2,2,1]=(0.792289256264)/(o+1.)+(-0.180045121676)+(0.215858165597)*0.5**o
        ref[2,3,0,0]=(-0.551360906631)/(o+1.)+(-0.776218533854)+(-0.596335978041)*0.5**o
        ref[2,3,0,1]=(0.821070238869)/(o+1.)+(0.00199201312847)+(-0.913806992124)*0.5**o
        ref[2,3,1,0]=(-0.43240265235)/(o+1.)+(-0.535669106134)+(-0.0831972104302)*0.5**o
        ref[2,3,1,1]=(0.72072318435)/(o+1.)+(0.0330920562179)+(0.643878326697)*0.5**o
        ref[2,3,2,0]=(0.754511827443)/(o+1.)+(-0.154030524635)+(0.843652876743)*0.5**o
        ref[2,3,2,1]=(0.261249117327)/(o+1.)+(-0.03562161515)+(0.0413857115672)*0.5**o
        ref[2,4,0,0]=(-0.0678823396451)/(o+1.)+(-0.192582591092)+(-0.607477693828)*0.5**o
        ref[2,4,0,1]=(0.0457149578306)/(o+1.)+(-0.0964434622791)+(-0.618172945146)*0.5**o
        ref[2,4,1,0]=(0.630735604709)/(o+1.)+(-0.0628061501569)+(0.722668769621)*0.5**o
        ref[2,4,1,1]=(-0.693659242703)/(o+1.)+(0.0588523188405)+(-0.847113860795)*0.5**o
        ref[2,4,2,0]=(0.705207362097)/(o+1.)+(0.0641733045603)+(-0.437578500878)*0.5**o
        ref[2,4,2,1]=(0.860707329294)/(o+1.)+(-0.0794358167735)+(0.329570005179)*0.5**o
        ref[3,0,0,0]=(0.549409370786)/(o+1.)+(-0.0827404400499)+(0.932602317046)*0.5**o
        ref[3,0,0,1]=(0.222376331882)/(o+1.)+(0.266122075595)+(0.975116319103)*0.5**o
        ref[3,0,1,0]=(-0.481194397466)/(o+1.)+(0.840391794852)+(0.019317277942)*0.5**o
        ref[3,0,1,1]=(0.737901448601)/(o+1.)+(0.0468573264614)+(-0.393162543578)*0.5**o
        ref[3,0,2,0]=(0.278242150139)/(o+1.)+(-0.0620258662406)+(0.458096711279)*0.5**o
        ref[3,0,2,1]=(-0.489134379327)/(o+1.)+(-0.252931749044)+(0.307111052767)*0.5**o
        ref[3,1,0,0]=(0.508808683)/(o+1.)+(-0.594037891099)+(0.420707084879)*0.5**o
        ref[3,1,0,1]=(0.198212694401)/(o+1.)+(-0.23511909641)+(-0.745253594401)*0.5**o
        ref[3,1,1,0]=(-0.0741524230667)/(o+1.)+(-0.166487499516)+(0.535215252931)*0.5**o
        ref[3,1,1,1]=(-0.664372557476)/(o+1.)+(-0.161339307185)+(0.193417593556)*0.5**o
        ref[3,1,2,0]=(0.408815221365)/(o+1.)+(0.556969143078)+(0.553690237434)*0.5**o
        ref[3,1,2,1]=(-0.111028404397)/(o+1.)+(0.28519971884)+(-0.211683249218)*0.5**o
        ref[3,2,0,0]=(-0.228385234131)/(o+1.)+(0.068833285335)+(-0.866119051438)*0.5**o
        ref[3,2,0,1]=(-0.709059272627)/(o+1.)+(0.273296245522)+(-0.146618266885)*0.5**o
        ref[3,2,1,0]=(-0.651609755171)/(o+1.)+(-0.717521398332)+(0.622973500199)*0.5**o
        ref[3,2,1,1]=(0.106659392436)/(o+1.)+(-0.0937511533137)+(-0.538565077894)*0.5**o
        ref[3,2,2,0]=(-0.339096900793)/(o+1.)+(0.616553014737)+(-0.325897413686)*0.5**o
        ref[3,2,2,1]=(-0.10651751912)/(o+1.)+(-0.0363728631586)+(0.861430484293)*0.5**o
        ref[3,3,0,0]=(0.230063809846)/(o+1.)+(0.0083457439947)+(0.0890838652267)*0.5**o
        ref[3,3,0,1]=(-0.309971427898)/(o+1.)+(0.0417460043682)+(-0.969251937403)*0.5**o
        ref[3,3,1,0]=(0.55773788655)/(o+1.)+(0.317014803113)+(0.640075232231)*0.5**o
        ref[3,3,1,1]=(-0.655757122598)/(o+1.)+(-0.186359376395)+(-0.350002630494)*0.5**o
        ref[3,3,2,0]=(0.310310045807)/(o+1.)+(0.229515713776)+(-0.262399316954)*0.5**o
        ref[3,3,2,1]=(-0.844329846245)/(o+1.)+(0.412676046356)+(-0.977039760257)*0.5**o
        ref[3,4,0,0]=(0.39699250956)/(o+1.)+(-0.162896058076)+(0.749594099085)*0.5**o
        ref[3,4,0,1]=(-0.34125011876)/(o+1.)+(-0.577475727421)+(0.913703789336)*0.5**o
        ref[3,4,1,0]=(-0.292338728991)/(o+1.)+(0.140586763153)+(-0.397439923687)*0.5**o
        ref[3,4,1,1]=(-0.182501473325)/(o+1.)+(-0.42646988384)+(-0.403584993513)*0.5**o
        ref[3,4,2,0]=(0.620412677389)/(o+1.)+(0.00427673646471)+(-0.263608698751)*0.5**o
        ref[3,4,2,1]=(-0.765468966987)/(o+1.)+(0.245504413107)+(-0.633502278259)*0.5**o
      else:
        arg[0,0,0,0]=(0.480896043196)*x[0]**o+(0.113355160927)*x[0]+(0.498630339771)*x[1]**o+(0.0149514785179)*x[1]+(-0.944439525708)*x[2]**o+(-0.349299625744)*x[2]
        arg[0,0,0,1]=(-0.609632888023)*x[0]**o+(-0.576508904483)*x[0]+(-0.0610487008887)*x[1]**o+(-0.844087384492)*x[1]+(-0.715427733517)*x[2]**o+(-0.263424969452)*x[2]
        arg[0,0,1,0]=(0.205065167181)*x[0]**o+(-0.840225514826)*x[0]+(0.808161497555)*x[1]**o+(0.685902712814)*x[1]+(-0.625183952417)*x[2]**o+(-0.0947932775112)*x[2]
        arg[0,0,1,1]=(0.875716461075)*x[0]**o+(0.814425331995)*x[0]+(-0.805331622235)*x[1]**o+(-0.287737068993)*x[1]+(-0.365715978159)*x[2]**o+(0.689907502778)*x[2]
        arg[0,0,2,0]=(0.711372030289)*x[0]**o+(-0.393612062405)*x[0]+(0.642793863702)*x[1]**o+(-0.66636178423)*x[1]+(0.00488425355017)*x[2]**o+(-0.444523817626)*x[2]
        arg[0,0,2,1]=(-0.0536842545617)*x[0]**o+(-0.714279418499)*x[0]+(-0.487452457649)*x[1]**o+(0.917805820162)*x[1]+(0.445007135027)*x[2]**o+(0.101201310289)*x[2]
        arg[0,1,0,0]=(-0.710091748251)*x[0]**o+(0.470243562863)*x[0]+(-0.0178862343634)*x[1]**o+(0.393644042332)*x[1]+(-0.952839337577)*x[2]**o+(-0.152392013939)*x[2]
        arg[0,1,0,1]=(0.250989366649)*x[0]**o+(0.592158236555)*x[0]+(0.698185234877)*x[1]**o+(0.112154263635)*x[1]+(0.843915127813)*x[2]**o+(-0.825331842242)*x[2]
        arg[0,1,1,0]=(0.690360096543)*x[0]**o+(0.78906430602)*x[0]+(0.596189773531)*x[1]**o+(-0.293452572358)*x[1]+(0.737565980909)*x[2]**o+(-0.96988090057)*x[2]
        arg[0,1,1,1]=(-0.815220933301)*x[0]**o+(-0.875224889387)*x[0]+(-0.0562950344908)*x[1]**o+(0.56567431969)*x[1]+(0.133489969036)*x[2]**o+(0.299403989148)*x[2]
        arg[0,1,2,0]=(0.169442617085)*x[0]**o+(-0.413300800662)*x[0]+(0.580773652363)*x[1]**o+(-0.627706012397)*x[1]+(-0.730218498677)*x[2]**o+(0.587233195345)*x[2]
        arg[0,1,2,1]=(-0.110636656112)*x[0]**o+(-0.302213944624)*x[0]+(0.643229732019)*x[1]**o+(-0.651805494828)*x[1]+(-0.528276901455)*x[2]**o+(-0.272057635286)*x[2]
        arg[0,2,0,0]=(0.340661037085)*x[0]**o+(-0.672207039496)*x[0]+(0.676291394824)*x[1]**o+(0.596998737167)*x[1]+(-0.689800391841)*x[2]**o+(-0.134273731064)*x[2]
        arg[0,2,0,1]=(0.185282186176)*x[0]**o+(0.252489447393)*x[0]+(-0.0447305595755)*x[1]**o+(-0.558361084175)*x[1]+(0.922681220429)*x[2]**o+(-0.484588607971)*x[2]
        arg[0,2,1,0]=(0.238772787004)*x[0]**o+(0.987311136495)*x[0]+(0.215079265839)*x[1]**o+(-0.797762453244)*x[1]+(0.0329055379323)*x[2]**o+(-0.88999503049)*x[2]
        arg[0,2,1,1]=(-0.175884373587)*x[0]**o+(0.806690051494)*x[0]+(0.0086744009376)*x[1]**o+(0.286131063746)*x[1]+(0.536839534689)*x[2]**o+(-0.0946282917135)*x[2]
        arg[0,2,2,0]=(-0.615695203188)*x[0]**o+(0.600565627235)*x[0]+(-0.869759996675)*x[1]**o+(0.648192341374)*x[1]+(-0.952728999251)*x[2]**o+(0.564068973312)*x[2]
        arg[0,2,2,1]=(-0.0992379560876)*x[0]**o+(-0.897179656441)*x[0]+(-0.519600820875)*x[1]**o+(0.70507167679)*x[1]+(0.903640275297)*x[2]**o+(-0.645039734963)*x[2]
        arg[0,3,0,0]=(0.432415072474)*x[0]**o+(-0.748137008742)*x[0]+(0.875883324091)*x[1]**o+(0.706172632952)*x[1]+(-0.47486904255)*x[2]**o+(0.238106343647)*x[2]
        arg[0,3,0,1]=(-0.656397468061)*x[0]**o+(-0.530867058489)*x[0]+(-0.904148914114)*x[1]**o+(0.87213389632)*x[1]+(0.0612056123548)*x[2]**o+(0.288620109264)*x[2]
        arg[0,3,1,0]=(-0.472896985897)*x[0]**o+(0.83328574859)*x[0]+(0.744310819801)*x[1]**o+(0.0601035189502)*x[1]+(-0.198693260491)*x[2]**o+(-0.882304016543)*x[2]
        arg[0,3,1,1]=(-0.515568063685)*x[0]**o+(0.484765564236)*x[0]+(0.754304059424)*x[1]**o+(0.654790289063)*x[1]+(-0.102335538236)*x[2]**o+(-0.0705158755887)*x[2]
        arg[0,3,2,0]=(-0.709802317012)*x[0]**o+(-0.346551215683)*x[0]+(0.312943005623)*x[1]**o+(0.476828461567)*x[1]+(0.901171463566)*x[2]**o+(0.538981245022)*x[2]
        arg[0,3,2,1]=(-0.20057625974)*x[0]**o+(0.0458666109661)*x[0]+(0.909610730794)*x[1]**o+(0.145637065192)*x[1]+(0.666420073244)*x[2]**o+(0.0550124082656)*x[2]
        arg[0,4,0,0]=(-0.726921583373)*x[0]**o+(0.891306726707)*x[0]+(-0.189151461127)*x[1]**o+(0.479274047923)*x[1]+(-0.129077261233)*x[2]**o+(-0.873599130413)*x[2]
        arg[0,4,0,1]=(0.765272127757)*x[0]**o+(0.159307695759)*x[0]+(-0.569162341331)*x[1]**o+(-0.530158933095)*x[1]+(-0.216727788754)*x[2]**o+(-0.519551969741)*x[2]
        arg[0,4,1,0]=(-0.613269202518)*x[0]**o+(0.612846846793)*x[0]+(-0.187285796021)*x[1]**o+(-0.462603518143)*x[1]+(-0.739162067312)*x[2]**o+(0.0737393951189)*x[2]
        arg[0,4,1,1]=(-0.727485674691)*x[0]**o+(-0.461427895477)*x[0]+(0.917029475887)*x[1]**o+(0.265874989879)*x[1]+(0.602505162351)*x[2]**o+(-0.372206475027)*x[2]
        arg[0,4,2,0]=(-0.663071984573)*x[0]**o+(0.511217165998)*x[0]+(-0.472765672078)*x[1]**o+(0.814962161218)*x[1]+(-0.608890631297)*x[2]**o+(-0.284872941025)*x[2]
        arg[0,4,2,1]=(0.893396132015)*x[0]**o+(0.183468149957)*x[0]+(0.647812435076)*x[1]**o+(-0.398672312216)*x[1]+(0.6359553902)*x[2]**o+(0.485509522238)*x[2]
        arg[1,0,0,0]=(-0.0771632829864)*x[0]**o+(-0.723025089864)*x[0]+(0.486209418481)*x[1]**o+(0.0869704919133)*x[1]+(-0.365499197901)*x[2]**o+(-0.0796713538898)*x[2]
        arg[1,0,0,1]=(0.133617787097)*x[0]**o+(0.993872304226)*x[0]+(-0.898504795319)*x[1]**o+(0.656793416434)*x[1]+(-0.821841885906)*x[2]**o+(0.726972748423)*x[2]
        arg[1,0,1,0]=(0.174277385391)*x[0]**o+(-0.301857401564)*x[0]+(-0.846153302312)*x[1]**o+(-0.559076632516)*x[1]+(-0.89870762709)*x[2]**o+(0.640171641512)*x[2]
        arg[1,0,1,1]=(0.795443369588)*x[0]**o+(-0.346082013333)*x[0]+(-0.439713049567)*x[1]**o+(0.860680213906)*x[1]+(-0.466761099917)*x[2]**o+(-0.843656974958)*x[2]
        arg[1,0,2,0]=(0.482814141263)*x[0]**o+(0.142978643528)*x[0]+(-0.735291704767)*x[1]**o+(-0.925675892787)*x[1]+(0.747235439093)*x[2]**o+(-0.569454067374)*x[2]
        arg[1,0,2,1]=(0.619359374773)*x[0]**o+(-0.547172728307)*x[0]+(0.0829642963797)*x[1]**o+(-0.0180882163346)*x[1]+(0.328627445196)*x[2]**o+(-0.274608686384)*x[2]
        arg[1,1,0,0]=(-0.510375454058)*x[0]**o+(-0.779894165181)*x[0]+(0.656168894408)*x[1]**o+(-0.449484133421)*x[1]+(-0.147284453744)*x[2]**o+(-0.488384108271)*x[2]
        arg[1,1,0,1]=(-0.302705575836)*x[0]**o+(0.256670307547)*x[0]+(0.110465379068)*x[1]**o+(0.319291387545)*x[1]+(0.408832172454)*x[2]**o+(0.718793126931)*x[2]
        arg[1,1,1,0]=(0.520882033451)*x[0]**o+(0.412524861348)*x[0]+(0.761080123209)*x[1]**o+(0.401223078157)*x[1]+(-0.65945839721)*x[2]**o+(0.71349489395)*x[2]
        arg[1,1,1,1]=(0.624622427594)*x[0]**o+(0.847907716976)*x[0]+(0.855268296474)*x[1]**o+(-0.671685662164)*x[1]+(-0.407895715308)*x[2]**o+(-0.719054767733)*x[2]
        arg[1,1,2,0]=(-0.044254701409)*x[0]**o+(0.72911290175)*x[0]+(0.869467934211)*x[1]**o+(0.613006764744)*x[1]+(0.678602796496)*x[2]**o+(0.388923815748)*x[2]
        arg[1,1,2,1]=(-0.240571107795)*x[0]**o+(0.838382917252)*x[0]+(0.0342933298383)*x[1]**o+(0.595421789197)*x[1]+(-0.989088178853)*x[2]**o+(-0.20002339454)*x[2]
        arg[1,2,0,0]=(0.0596937021722)*x[0]**o+(0.196768537308)*x[0]+(0.0834459161806)*x[1]**o+(0.194049508373)*x[1]+(-0.097309631361)*x[2]**o+(-0.867555781902)*x[2]
        arg[1,2,0,1]=(-0.935541542647)*x[0]**o+(-0.00702430636955)*x[0]+(-0.384657002538)*x[1]**o+(0.186884376746)*x[1]+(0.97207387841)*x[2]**o+(0.452460932869)*x[2]
        arg[1,2,1,0]=(0.276741406646)*x[0]**o+(0.838348610585)*x[0]+(-0.403054578066)*x[1]**o+(0.655318135938)*x[1]+(0.37614761635)*x[2]**o+(0.82736662494)*x[2]
        arg[1,2,1,1]=(0.828731604537)*x[0]**o+(0.762067030749)*x[0]+(-0.881827056079)*x[1]**o+(0.250928989647)*x[1]+(-0.771040367979)*x[2]**o+(0.239950799138)*x[2]
        arg[1,2,2,0]=(-0.380797583569)*x[0]**o+(0.112805267222)*x[0]+(-0.454730562411)*x[1]**o+(0.926061735219)*x[1]+(0.348331256905)*x[2]**o+(-0.725641369775)*x[2]
        arg[1,2,2,1]=(0.257022926473)*x[0]**o+(0.0160557858816)*x[0]+(-0.291367922966)*x[1]**o+(-0.611341260765)*x[1]+(0.232472950268)*x[2]**o+(-0.738139185521)*x[2]
        arg[1,3,0,0]=(-0.645340088797)*x[0]**o+(0.480235460274)*x[0]+(0.135346246274)*x[1]**o+(-0.560730820351)*x[1]+(-0.635582025321)*x[2]**o+(-0.476912924392)*x[2]
        arg[1,3,0,1]=(0.124132693667)*x[0]**o+(-0.813993821983)*x[0]+(-0.108593659227)*x[1]**o+(-0.432186668568)*x[1]+(0.209382070426)*x[2]**o+(0.928912669667)*x[2]
        arg[1,3,1,0]=(-0.7660131868)*x[0]**o+(-0.30274186797)*x[0]+(0.140498262981)*x[1]**o+(0.993084314471)*x[1]+(0.684043026981)*x[2]**o+(-0.339724714081)*x[2]
        arg[1,3,1,1]=(-0.150578257351)*x[0]**o+(-0.674840941109)*x[0]+(0.469240334383)*x[1]**o+(0.729963338278)*x[1]+(-0.800829664334)*x[2]**o+(0.0467922871395)*x[2]
        arg[1,3,2,0]=(-0.336661555676)*x[0]**o+(0.705661383379)*x[0]+(-0.76993460355)*x[1]**o+(0.459144989218)*x[1]+(0.545901025663)*x[2]**o+(0.385988013614)*x[2]
        arg[1,3,2,1]=(0.165928745462)*x[0]**o+(0.540718600752)*x[0]+(0.172773708803)*x[1]**o+(0.253664803784)*x[1]+(-0.264896626947)*x[2]**o+(-0.478577475495)*x[2]
        arg[1,4,0,0]=(0.283586653542)*x[0]**o+(-0.104364884571)*x[0]+(-0.840555806526)*x[1]**o+(0.701868415915)*x[1]+(-0.290943689679)*x[2]**o+(-0.73024942052)*x[2]
        arg[1,4,0,1]=(-0.772849835334)*x[0]**o+(-0.432170589951)*x[0]+(-0.376099423267)*x[1]**o+(0.0215357491826)*x[1]+(-0.346787011464)*x[2]**o+(0.620411709813)*x[2]
        arg[1,4,1,0]=(-0.718524564964)*x[0]**o+(0.119936682579)*x[0]+(-0.994276256462)*x[1]**o+(-0.839783920473)*x[1]+(-0.454576336782)*x[2]**o+(-0.515489988915)*x[2]
        arg[1,4,1,1]=(-0.771664154234)*x[0]**o+(-0.584052299067)*x[0]+(-0.0233756210307)*x[1]**o+(-0.174442816636)*x[1]+(-0.702383477714)*x[2]**o+(-0.279952858144)*x[2]
        arg[1,4,2,0]=(-0.591902790978)*x[0]**o+(-0.151826578123)*x[0]+(0.923906788219)*x[1]**o+(0.301090558476)*x[1]+(-0.518314647811)*x[2]**o+(-0.784331717681)*x[2]
        arg[1,4,2,1]=(0.303567440092)*x[0]**o+(0.744948852961)*x[0]+(0.719939071212)*x[1]**o+(-0.161138638319)*x[1]+(0.924953261766)*x[2]**o+(-0.875565390131)*x[2]
        arg[2,0,0,0]=(0.197574550087)*x[0]**o+(-0.962018785397)*x[0]+(0.498378037582)*x[1]**o+(-0.334199169606)*x[1]+(0.852426750588)*x[2]**o+(-0.650173474618)*x[2]
        arg[2,0,0,1]=(0.0663499012532)*x[0]**o+(0.00812461763474)*x[0]+(0.237233309304)*x[1]**o+(-0.839377881658)*x[1]+(-0.67246872567)*x[2]**o+(0.677471261363)*x[2]
        arg[2,0,1,0]=(-0.789090971877)*x[0]**o+(-0.387724517273)*x[0]+(0.455954877641)*x[1]**o+(-0.356995254842)*x[1]+(-0.897019846935)*x[2]**o+(-0.0703543939474)*x[2]
        arg[2,0,1,1]=(0.222873274437)*x[0]**o+(0.405228982254)*x[0]+(-0.532697993554)*x[1]**o+(-0.836340143789)*x[1]+(-0.139525062661)*x[2]**o+(-0.916307770461)*x[2]
        arg[2,0,2,0]=(-0.300828039715)*x[0]**o+(-0.584457605313)*x[0]+(-0.443125109903)*x[1]**o+(0.292734004842)*x[1]+(-0.55875691535)*x[2]**o+(0.391237822545)*x[2]
        arg[2,0,2,1]=(-0.501507304569)*x[0]**o+(-0.203549089015)*x[0]+(-0.113050138121)*x[1]**o+(0.712978728546)*x[1]+(-0.642617274635)*x[2]**o+(-0.307119076865)*x[2]
        arg[2,1,0,0]=(-0.809237327867)*x[0]**o+(0.984684980548)*x[0]+(-0.643310231704)*x[1]**o+(0.759430082065)*x[1]+(-0.393902220205)*x[2]**o+(-0.900173371318)*x[2]
        arg[2,1,0,1]=(-0.722795722506)*x[0]**o+(0.433309366611)*x[0]+(-0.559871890844)*x[1]**o+(0.417301432603)*x[1]+(-0.364192390887)*x[2]**o+(0.64549736058)*x[2]
        arg[2,1,1,0]=(-0.551325611251)*x[0]**o+(-0.638520772682)*x[0]+(0.222655558982)*x[1]**o+(-0.230002147207)*x[1]+(-0.290821163957)*x[2]**o+(0.0647728628594)*x[2]
        arg[2,1,1,1]=(0.743453045588)*x[0]**o+(-0.901163108996)*x[0]+(-0.546031302921)*x[1]**o+(-0.512614476111)*x[1]+(-0.886430682715)*x[2]**o+(0.720650678026)*x[2]
        arg[2,1,2,0]=(-0.033787671958)*x[0]**o+(0.930927279443)*x[0]+(-0.530482747719)*x[1]**o+(0.714053784656)*x[1]+(-0.0978565618487)*x[2]**o+(-0.0423072209447)*x[2]
        arg[2,1,2,1]=(0.127121989623)*x[0]**o+(0.809654796091)*x[0]+(0.718533701795)*x[1]**o+(0.542738494557)*x[1]+(-0.342918607246)*x[2]**o+(-0.627991145352)*x[2]
        arg[2,2,0,0]=(-0.867530011926)*x[0]**o+(0.870295856967)*x[0]+(-0.13043222912)*x[1]**o+(0.2670058777)*x[1]+(0.395016123247)*x[2]**o+(-0.811207959629)*x[2]
        arg[2,2,0,1]=(-0.00858729113056)*x[0]**o+(-0.885551110851)*x[0]+(0.820441870015)*x[1]**o+(-0.348896120549)*x[1]+(0.295475751596)*x[2]**o+(-0.672688762902)*x[2]
        arg[2,2,1,0]=(-0.982123891355)*x[0]**o+(0.232770408062)*x[0]+(0.374641341895)*x[1]**o+(-0.21375197203)*x[1]+(0.436992590849)*x[2]**o+(-0.473399371298)*x[2]
        arg[2,2,1,1]=(-0.415533190881)*x[0]**o+(0.0945615271507)*x[0]+(-0.281959759072)*x[1]**o+(0.0807942449559)*x[1]+(0.225501997354)*x[2]**o+(0.305247648387)*x[2]
        arg[2,2,2,0]=(-0.559916632426)*x[0]**o+(-0.372679495574)*x[0]+(0.392857820158)*x[1]**o+(0.726824319268)*x[1]+(0.36923535014)*x[2]**o+(-0.208169256022)*x[2]
        arg[2,2,2,1]=(-0.869055045156)*x[0]**o+(0.359188490398)*x[0]+(0.702110073265)*x[1]**o+(0.668757858248)*x[1]+(-0.499300337779)*x[2]**o+(0.0471054271112)*x[2]
        arg[2,3,0,0]=(0.754089271004)*x[0]**o+(-0.0240479506036)*x[0]+(-0.0775830745475)*x[1]**o+(0.215564363064)*x[1]+(-0.791969192801)*x[2]**o+(-0.130308630692)*x[2]
        arg[2,3,0,1]=(0.448970570476)*x[0]**o+(0.510346880872)*x[0]+(-0.561996764085)*x[1]**o+(0.217897696493)*x[1]+(-0.0610826665756)*x[2]**o+(0.387896322636)*x[2]
        arg[2,3,1,0]=(0.737635683404)*x[0]**o+(-0.35395540781)*x[0]+(0.0419652573208)*x[1]**o+(-0.519735381528)*x[1]+(0.739523085214)*x[2]**o+(0.808529094094)*x[2]
        arg[2,3,1,1]=(0.551348499194)*x[0]**o+(-0.40145493368)*x[0]+(0.211723169473)*x[1]**o+(-0.762751144824)*x[1]+(-0.17710033067)*x[2]**o+(0.688961224051)*x[2]
        arg[2,3,2,0]=(-0.870721462645)*x[0]**o+(-0.0356869329313)*x[0]+(0.277142920424)*x[1]**o+(-0.154474999666)*x[1]+(0.837001124911)*x[2]**o+(-0.233028214567)*x[2]
        arg[2,3,2,1]=(-0.234260935528)*x[0]**o+(0.913718165996)*x[0]+(0.385678747942)*x[1]**o+(0.237966356277)*x[1]+(0.382072927695)*x[2]**o+(0.627666395757)*x[2]
        arg[2,4,0,0]=(-0.474674316363)*x[0]**o+(-0.575533683959)*x[0]+(0.751489557666)*x[1]**o+(-0.0623565445313)*x[1]+(-0.517274743843)*x[2]**o+(-0.712143034168)*x[2]
        arg[2,4,0,1]=(0.259723799797)*x[0]**o+(0.345952822815)*x[0]+(0.169940774832)*x[1]**o+(-0.185154259609)*x[1]+(0.418430344681)*x[2]**o+(0.861837327531)*x[2]
        arg[2,4,1,0]=(-0.960970748682)*x[0]**o+(0.842706511282)*x[0]+(-0.771779045108)*x[1]**o+(-0.63675621795)*x[1]+(-0.317416556239)*x[2]**o+(0.287317968341)*x[2]
        arg[2,4,1,1]=(-0.753944077832)*x[0]**o+(0.570245260805)*x[0]+(-0.0226797611003)*x[1]**o+(0.0776773843077)*x[1]+(0.629536717673)*x[2]**o+(0.271216337699)*x[2]
        arg[2,4,2,0]=(-0.233304828171)*x[0]**o+(-0.217436697538)*x[0]+(-0.373618121496)*x[1]**o+(-0.342584906659)*x[1]+(0.0114484228954)*x[2]**o+(-0.639753178531)*x[2]
        arg[2,4,2,1]=(0.341373328517)*x[0]**o+(-0.432795268788)*x[0]+(-0.871263113926)*x[1]**o+(0.843590946949)*x[1]+(-0.653058566267)*x[2]**o+(-0.894366240333)*x[2]
        arg[3,0,0,0]=(-0.472652289489)*x[0]**o+(0.697119499826)*x[0]+(-0.195781503945)*x[1]**o+(-0.716872496009)*x[1]+(-0.159070341485)*x[2]**o+(-0.700548071467)*x[2]
        arg[3,0,0,1]=(-0.398674023611)*x[0]**o+(-0.690683017472)*x[0]+(0.957112237189)*x[1]**o+(-0.782702815224)*x[1]+(-0.249336230999)*x[2]**o+(0.22734983387)*x[2]
        arg[3,0,1,0]=(-0.0216721345971)*x[0]**o+(0.283765576696)*x[0]+(-0.35531833417)*x[1]**o+(0.974402352836)*x[1]+(-0.568820213603)*x[2]**o+(0.957610279127)*x[2]
        arg[3,0,1,1]=(0.0606265274442)*x[0]**o+(-0.941169961629)*x[0]+(0.804902933622)*x[1]**o+(-0.857158317629)*x[1]+(-0.227167312547)*x[2]**o+(0.945175823328)*x[2]
        arg[3,0,2,0]=(0.630444608821)*x[0]**o+(-0.192343222551)*x[0]+(0.0342249337232)*x[1]**o+(0.970401334961)*x[1]+(-0.608568046213)*x[2]**o+(-0.539374299133)*x[2]
        arg[3,0,2,1]=(-0.348127940577)*x[0]**o+(-0.895424161569)*x[0]+(0.918173840926)*x[1]**o+(0.99658998821)*x[1]+(-0.28733372377)*x[2]**o+(-0.516441029544)*x[2]
        arg[3,1,0,0]=(0.735069593965)*x[0]**o+(-0.727899981109)*x[0]+(-0.325639355247)*x[1]**o+(0.751064830331)*x[1]+(-0.297126073245)*x[2]**o+(-0.673421779764)*x[2]
        arg[3,1,0,1]=(0.00281719106153)*x[0]**o+(0.246929980679)*x[0]+(-0.48091940116)*x[1]**o+(0.501927059357)*x[1]+(-0.520099802736)*x[2]**o+(-0.902706012118)*x[2]
        arg[3,1,1,0]=(-0.341732637339)*x[0]**o+(-0.169802234575)*x[0]+(0.156725375114)*x[1]**o+(0.739176104936)*x[1]+(-0.374572295196)*x[2]**o+(-0.436641001832)*x[2]
        arg[3,1,1,1]=(0.644746847185)*x[0]**o+(-0.992126325722)*x[0]+(0.551602368714)*x[1]**o+(-0.839306768893)*x[1]+(-0.298981617653)*x[2]**o+(0.089795271605)*x[2]
        arg[3,1,2,0]=(0.382425223801)*x[0]**o+(-0.949095041278)*x[0]+(-0.135997266858)*x[1]**o+(-0.871114033611)*x[1]+(-0.422984170709)*x[2]**o+(-0.698818784989)*x[2]
        arg[3,1,2,1]=(0.630558573276)*x[0]**o+(0.90644556976)*x[0]+(0.512953870249)*x[1]**o+(-0.58514271049)*x[1]+(-0.0587225891088)*x[2]**o+(-0.390796465925)*x[2]
        arg[3,2,0,0]=(-0.404884771697)*x[0]**o+(-0.178144326223)*x[0]+(-0.606503995405)*x[1]**o+(0.190025878248)*x[1]+(0.735831203504)*x[2]**o+(-0.567394607398)*x[2]
        arg[3,2,0,1]=(0.28819895431)*x[0]**o+(0.456573144203)*x[0]+(0.00856899200079)*x[1]**o+(0.547827005333)*x[1]+(0.087428132552)*x[2]**o+(0.759650060552)*x[2]
        arg[3,2,1,0]=(0.949790710961)*x[0]**o+(-0.685602682574)*x[0]+(0.75278809536)*x[1]**o+(-0.876689834021)*x[1]+(0.0116394079709)*x[2]**o+(0.319113513167)*x[2]
        arg[3,2,1,1]=(0.619659549755)*x[0]**o+(-0.397388124185)*x[0]+(0.460679661901)*x[1]**o+(-0.507122241204)*x[1]+(0.985741194076)*x[2]**o+(0.916231863626)*x[2]
        arg[3,2,2,0]=(-0.457269257294)*x[0]**o+(-0.47265466694)*x[0]+(0.900051837969)*x[1]**o+(-0.148802803439)*x[1]+(-0.195540487054)*x[2]**o+(0.0784524130754)*x[2]
        arg[3,2,2,1]=(0.933553130797)*x[0]**o+(0.617362831046)*x[0]+(0.263616954879)*x[1]**o+(-0.0658887625105)*x[1]+(0.48091174024)*x[2]**o+(0.452935484504)*x[2]
        arg[3,3,0,0]=(0.825594594128)*x[0]**o+(0.350203407373)*x[0]+(0.211125321741)*x[1]**o+(0.194914147746)*x[1]+(-0.639432220749)*x[2]**o+(-0.678718852935)*x[2]
        arg[3,3,0,1]=(0.514394313629)*x[0]**o+(-0.595261005308)*x[0]+(-0.0219208527235)*x[1]**o+(-0.77204975709)*x[1]+(-0.483587370025)*x[2]**o+(-0.559464911044)*x[2]
        arg[3,3,1,0]=(0.821240511475)*x[0]**o+(-0.403492177329)*x[0]+(0.486132478046)*x[1]**o+(-0.0187529457547)*x[1]+(0.73209003456)*x[2]**o+(0.226791335797)*x[2]
        arg[3,3,1,1]=(-0.644256768839)*x[0]**o+(0.0372588784686)*x[0]+(0.726561650796)*x[1]**o+(0.518003809017)*x[1]+(0.738146681601)*x[2]**o+(-0.256036433171)*x[2]
        arg[3,3,2,0]=(0.534862520097)*x[0]**o+(0.275559826342)*x[0]+(0.292383677513)*x[1]**o+(-0.448938027415)*x[1]+(0.281053776971)*x[2]**o+(0.0119964402657)*x[2]
        arg[3,3,2,1]=(0.722071427786)*x[0]**o+(0.150280744346)*x[0]+(-0.0354786178164)*x[1]**o+(-0.703984035201)*x[1]+(0.105365795033)*x[2]**o+(-0.556567650379)*x[2]
        arg[3,4,0,0]=(0.188625467698)*x[0]**o+(0.737108753179)*x[0]+(0.481952725951)*x[1]**o+(0.841958412185)*x[1]+(-0.329504392569)*x[2]**o+(0.228456367852)*x[2]
        arg[3,4,0,1]=(0.181430484096)*x[0]**o+(0.307078178179)*x[0]+(0.262079226853)*x[1]**o+(0.111656765153)*x[1]+(0.15340094176)*x[2]**o+(-0.859774252333)*x[2]
        arg[3,4,1,0]=(0.213491785855)*x[0]**o+(-0.479441582203)*x[0]+(0.0527576231613)*x[1]**o+(0.328465118263)*x[1]+(-0.645719217198)*x[2]**o+(-0.550569446193)*x[2]
        arg[3,4,1,1]=(-0.401744411188)*x[0]**o+(-0.626446165761)*x[0]+(-0.191670239416)*x[1]**o+(0.866888407009)*x[1]+(-0.341220740645)*x[2]**o+(-0.742685031762)*x[2]
        arg[3,4,2,0]=(-0.0813611687514)*x[0]**o+(0.318680684889)*x[0]+(-0.780434724843)*x[1]**o+(-0.0902735483921)*x[1]+(0.586108990767)*x[2]**o+(0.934041577436)*x[2]
        arg[3,4,2,1]=(-0.0991425040958)*x[0]**o+(0.99274076773)*x[0]+(0.924248544389)*x[1]**o+(0.290576034321)*x[1]+(0.452104871421)*x[2]**o+(0.774017129246)*x[2]
        ref[0,0,0,0]=(-0.445809185937)/(o+1.)+(-0.11049649315)+(0.480896043196)*0.5**o
        ref[0,0,0,1]=(-0.776476434405)/(o+1.)+(-0.842010629213)+(-0.609632888023)*0.5**o
        ref[0,0,1,0]=(0.182977545138)/(o+1.)+(-0.124558039762)+(0.205065167181)*0.5**o
        ref[0,0,1,1]=(-1.17104760039)/(o+1.)+(0.608297882891)+(0.875716461075)*0.5**o
        ref[0,0,2,0]=(0.647678117252)/(o+1.)+(-0.752248832131)+(0.711372030289)*0.5**o
        ref[0,0,2,1]=(-0.0424453226222)/(o+1.)+(0.152363855976)+(-0.0536842545617)*0.5**o
        ref[0,1,0,0]=(-0.97072557194)/(o+1.)+(0.355747795628)+(-0.710091748251)*0.5**o
        ref[0,1,0,1]=(1.54210036269)/(o+1.)+(-0.0605096710259)+(0.250989366649)*0.5**o
        ref[0,1,1,0]=(1.33375575444)/(o+1.)+(-0.237134583454)+(0.690360096543)*0.5**o
        ref[0,1,1,1]=(0.0771949345448)/(o+1.)+(-0.00507329027461)+(-0.815220933301)*0.5**o
        ref[0,1,2,0]=(-0.149444846314)/(o+1.)+(-0.226886808857)+(0.169442617085)*0.5**o
        ref[0,1,2,1]=(0.114952830564)/(o+1.)+(-0.613038537369)+(-0.110636656112)*0.5**o
        ref[0,2,0,0]=(-0.0135089970172)/(o+1.)+(-0.104741016697)+(0.340661037085)*0.5**o
        ref[0,2,0,1]=(0.877950660854)/(o+1.)+(-0.395230122376)+(0.185282186176)*0.5**o
        ref[0,2,1,0]=(0.247984803771)/(o+1.)+(-0.35022317362)+(0.238772787004)*0.5**o
        ref[0,2,1,1]=(0.545513935627)/(o+1.)+(0.499096411763)+(-0.175884373587)*0.5**o
        ref[0,2,2,0]=(-1.82248899593)/(o+1.)+(0.90641347096)+(-0.615695203188)*0.5**o
        ref[0,2,2,1]=(0.384039454422)/(o+1.)+(-0.418573857307)+(-0.0992379560876)*0.5**o
        ref[0,3,0,0]=(0.401014281541)/(o+1.)+(0.0980709839284)+(0.432415072474)*0.5**o
        ref[0,3,0,1]=(-0.842943301759)/(o+1.)+(0.314943473548)+(-0.656397468061)*0.5**o
        ref[0,3,1,0]=(0.54561755931)/(o+1.)+(0.00554262549843)+(-0.472896985897)*0.5**o
        ref[0,3,1,1]=(0.651968521189)/(o+1.)+(0.534519988855)+(-0.515568063685)*0.5**o
        ref[0,3,2,0]=(1.21411446919)/(o+1.)+(0.334629245453)+(-0.709802317012)*0.5**o
        ref[0,3,2,1]=(1.57603080404)/(o+1.)+(0.123258042212)+(-0.20057625974)*0.5**o
        ref[0,4,0,0]=(-0.318228722361)/(o+1.)+(0.248490822109)+(-0.726921583373)*0.5**o
        ref[0,4,0,1]=(-0.785890130085)/(o+1.)+(-0.445201603538)+(0.765272127757)*0.5**o
        ref[0,4,1,0]=(-0.926447863334)/(o+1.)+(0.111991361885)+(-0.613269202518)*0.5**o
        ref[0,4,1,1]=(1.51953463824)/(o+1.)+(-0.283879690312)+(-0.727485674691)*0.5**o
        ref[0,4,2,0]=(-1.08165630338)/(o+1.)+(0.520653193096)+(-0.663071984573)*0.5**o
        ref[0,4,2,1]=(1.28376782528)/(o+1.)+(0.135152679989)+(0.893396132015)*0.5**o
        ref[1,0,0,0]=(0.12071022058)/(o+1.)+(-0.35786297592)+(-0.0771632829864)*0.5**o
        ref[1,0,0,1]=(-1.72034668123)/(o+1.)+(1.18881923454)+(0.133617787097)*0.5**o
        ref[1,0,1,0]=(-1.7448609294)/(o+1.)+(-0.110381196284)+(0.174277385391)*0.5**o
        ref[1,0,1,1]=(-0.906474149485)/(o+1.)+(-0.164529387192)+(0.795443369588)*0.5**o
        ref[1,0,2,0]=(0.0119437343259)/(o+1.)+(-0.676075658316)+(0.482814141263)*0.5**o
        ref[1,0,2,1]=(0.411591741576)/(o+1.)+(-0.419934815512)+(0.619359374773)*0.5**o
        ref[1,1,0,0]=(0.508884440665)/(o+1.)+(-0.858881203437)+(-0.510375454058)*0.5**o
        ref[1,1,0,1]=(0.519297551522)/(o+1.)+(0.647377411012)+(-0.302705575836)*0.5**o
        ref[1,1,1,0]=(0.101621725999)/(o+1.)+(0.763621416728)+(0.520882033451)*0.5**o
        ref[1,1,1,1]=(0.447372581166)/(o+1.)+(-0.27141635646)+(0.624622427594)*0.5**o
        ref[1,1,2,0]=(1.54807073071)/(o+1.)+(0.865521741121)+(-0.044254701409)*0.5**o
        ref[1,1,2,1]=(-0.954794849015)/(o+1.)+(0.616890655954)+(-0.240571107795)*0.5**o
        ref[1,2,0,0]=(-0.0138637151804)/(o+1.)+(-0.23836886811)+(0.0596937021722)*0.5**o
        ref[1,2,0,1]=(0.587416875871)/(o+1.)+(0.316160501623)+(-0.935541542647)*0.5**o
        ref[1,2,1,0]=(-0.0269069617163)/(o+1.)+(1.16051668573)+(0.276741406646)*0.5**o
        ref[1,2,1,1]=(-1.65286742406)/(o+1.)+(0.626473409767)+(0.828731604537)*0.5**o
        ref[1,2,2,0]=(-0.106399305507)/(o+1.)+(0.156612816333)+(-0.380797583569)*0.5**o
        ref[1,2,2,1]=(-0.0588949726984)/(o+1.)+(-0.666712330202)+(0.257022926473)*0.5**o
        ref[1,3,0,0]=(-0.500235779047)/(o+1.)+(-0.278704142234)+(-0.645340088797)*0.5**o
        ref[1,3,0,1]=(0.100788411199)/(o+1.)+(-0.158633910442)+(0.124132693667)*0.5**o
        ref[1,3,1,0]=(0.824541289962)/(o+1.)+(0.17530886621)+(-0.7660131868)*0.5**o
        ref[1,3,1,1]=(-0.331589329951)/(o+1.)+(0.0509573421542)+(-0.150578257351)*0.5**o
        ref[1,3,2,0]=(-0.224033577887)/(o+1.)+(0.775397193106)+(-0.336661555676)*0.5**o
        ref[1,3,2,1]=(-0.0921229181433)/(o+1.)+(0.15790296452)+(0.165928745462)*0.5**o
        ref[1,4,0,0]=(-1.13149949621)/(o+1.)+(-0.0663729445879)+(0.283586653542)*0.5**o
        ref[1,4,0,1]=(-0.722886434731)/(o+1.)+(0.104888434522)+(-0.772849835334)*0.5**o
        ref[1,4,1,0]=(-1.44885259324)/(o+1.)+(-0.617668613405)+(-0.718524564964)*0.5**o
        ref[1,4,1,1]=(-0.725759098745)/(o+1.)+(-0.519223986923)+(-0.771664154234)*0.5**o
        ref[1,4,2,0]=(0.405592140408)/(o+1.)+(-0.317533868664)+(-0.591902790978)*0.5**o
        ref[1,4,2,1]=(1.64489233298)/(o+1.)+(-0.145877587744)+(0.303567440092)*0.5**o
        ref[2,0,0,0]=(1.35080478817)/(o+1.)+(-0.97319571481)+(0.197574550087)*0.5**o
        ref[2,0,0,1]=(-0.435235416366)/(o+1.)+(-0.0768910013301)+(0.0663499012532)*0.5**o
        ref[2,0,1,0]=(-0.441064969294)/(o+1.)+(-0.407537083031)+(-0.789090971877)*0.5**o
        ref[2,0,1,1]=(-0.672223056216)/(o+1.)+(-0.673709465998)+(0.222873274437)*0.5**o
        ref[2,0,2,0]=(-1.00188202525)/(o+1.)+(0.0497571110368)+(-0.300828039715)*0.5**o
        ref[2,0,2,1]=(-0.755667412756)/(o+1.)+(0.101155281333)+(-0.501507304569)*0.5**o
        ref[2,1,0,0]=(-1.03721245191)/(o+1.)+(0.421970845648)+(-0.809237327867)*0.5**o
        ref[2,1,0,1]=(-0.924064281731)/(o+1.)+(0.748054079897)+(-0.722795722506)*0.5**o
        ref[2,1,1,0]=(-0.0681656049743)/(o+1.)+(-0.401875028515)+(-0.551325611251)*0.5**o
        ref[2,1,1,1]=(-1.43246198564)/(o+1.)+(-0.346563453541)+(0.743453045588)*0.5**o
        ref[2,1,2,0]=(-0.628339309568)/(o+1.)+(0.801336921577)+(-0.033787671958)*0.5**o
        ref[2,1,2,1]=(0.375615094549)/(o+1.)+(0.362201072648)+(0.127121989623)*0.5**o
        ref[2,2,0,0]=(0.264583894127)/(o+1.)+(0.163046887519)+(-0.867530011926)*0.5**o
        ref[2,2,0,1]=(1.11591762161)/(o+1.)+(-0.953567997151)+(-0.00858729113056)*0.5**o
        ref[2,2,1,0]=(0.811633932744)/(o+1.)+(-0.227190467633)+(-0.982123891355)*0.5**o
        ref[2,2,1,1]=(-0.0564577617182)/(o+1.)+(0.240301710247)+(-0.415533190881)*0.5**o
        ref[2,2,2,0]=(0.762093170298)/(o+1.)+(0.072987783836)+(-0.559916632426)*0.5**o
        ref[2,2,2,1]=(0.202809735486)/(o+1.)+(0.537525887878)+(-0.869055045156)*0.5**o
        ref[2,3,0,0]=(-0.869552267348)/(o+1.)+(0.0306038908843)+(0.754089271004)*0.5**o
        ref[2,3,0,1]=(-0.62307943066)/(o+1.)+(0.55807045)+(0.448970570476)*0.5**o
        ref[2,3,1,0]=(0.781488342534)/(o+1.)+(-0.0325808476221)+(0.737635683404)*0.5**o
        ref[2,3,1,1]=(0.0346228388027)/(o+1.)+(-0.237622427226)+(0.551348499194)*0.5**o
        ref[2,3,2,0]=(1.11414404534)/(o+1.)+(-0.211595073582)+(-0.870721462645)*0.5**o
        ref[2,3,2,1]=(0.767751675637)/(o+1.)+(0.889675459015)+(-0.234260935528)*0.5**o
        ref[2,4,0,0]=(0.234214813823)/(o+1.)+(-0.675016631329)+(-0.474674316363)*0.5**o
        ref[2,4,0,1]=(0.588371119514)/(o+1.)+(0.511317945368)+(0.259723799797)*0.5**o
        ref[2,4,1,0]=(-1.08919560135)/(o+1.)+(0.246634130836)+(-0.960970748682)*0.5**o
        ref[2,4,1,1]=(0.606856956573)/(o+1.)+(0.459569491406)+(-0.753944077832)*0.5**o
        ref[2,4,2,0]=(-0.3621696986)/(o+1.)+(-0.599887391364)+(-0.233304828171)*0.5**o
        ref[2,4,2,1]=(-1.52432168019)/(o+1.)+(-0.241785281086)+(0.341373328517)*0.5**o
        ref[3,0,0,0]=(-0.35485184543)/(o+1.)+(-0.360150533825)+(-0.472652289489)*0.5**o
        ref[3,0,0,1]=(0.70777600619)/(o+1.)+(-0.623017999413)+(-0.398674023611)*0.5**o
        ref[3,0,1,0]=(-0.924138547773)/(o+1.)+(1.10788910433)+(-0.0216721345971)*0.5**o
        ref[3,0,1,1]=(0.577735621075)/(o+1.)+(-0.426576227965)+(0.0606265274442)*0.5**o
        ref[3,0,2,0]=(-0.57434311249)/(o+1.)+(0.119341906638)+(0.630444608821)*0.5**o
        ref[3,0,2,1]=(0.630840117156)/(o+1.)+(-0.207637601452)+(-0.348127940577)*0.5**o
        ref[3,1,0,0]=(-0.622765428492)/(o+1.)+(-0.32512846527)+(0.735069593965)*0.5**o
        ref[3,1,0,1]=(-1.0010192039)/(o+1.)+(-0.076924486041)+(0.00281719106153)*0.5**o
        ref[3,1,1,0]=(-0.217846920082)/(o+1.)+(0.0663664342643)+(-0.341732637339)*0.5**o
        ref[3,1,1,1]=(0.25262075106)/(o+1.)+(-0.870818911505)+(0.644746847185)*0.5**o
        ref[3,1,2,0]=(-0.558981437567)/(o+1.)+(-1.25951392994)+(0.382425223801)*0.5**o
        ref[3,1,2,1]=(0.454231281141)/(o+1.)+(-0.034746803327)+(0.630558573276)*0.5**o
        ref[3,2,0,0]=(0.129327208099)/(o+1.)+(-0.277756527686)+(-0.404884771697)*0.5**o
        ref[3,2,0,1]=(0.0959971245528)/(o+1.)+(0.882025105044)+(0.28819895431)*0.5**o
        ref[3,2,1,0]=(0.764427503331)/(o+1.)+(-0.621589501714)+(0.949790710961)*0.5**o
        ref[3,2,1,1]=(1.44642085598)/(o+1.)+(0.00586074911794)+(0.619659549755)*0.5**o
        ref[3,2,2,0]=(0.704511350915)/(o+1.)+(-0.271502528652)+(-0.457269257294)*0.5**o
        ref[3,2,2,1]=(0.744528695119)/(o+1.)+(0.50220477652)+(0.933553130797)*0.5**o
        ref[3,3,0,0]=(-0.428306899008)/(o+1.)+(-0.0668006489082)+(0.825594594128)*0.5**o
        ref[3,3,0,1]=(-0.505508222749)/(o+1.)+(-0.963387836721)+(0.514394313629)*0.5**o
        ref[3,3,1,0]=(1.21822251261)/(o+1.)+(-0.0977268936434)+(0.821240511475)*0.5**o
        ref[3,3,1,1]=(1.4647083324)/(o+1.)+(0.149613127157)+(-0.644256768839)*0.5**o
        ref[3,3,2,0]=(0.573437454483)/(o+1.)+(-0.0806908804035)+(0.534862520097)*0.5**o
        ref[3,3,2,1]=(0.069887177217)/(o+1.)+(-0.555135470617)+(0.722071427786)*0.5**o
        ref[3,4,0,0]=(0.152448333383)/(o+1.)+(0.903761766608)+(0.188625467698)*0.5**o
        ref[3,4,0,1]=(0.415480168613)/(o+1.)+(-0.220519654501)+(0.181430484096)*0.5**o
        ref[3,4,1,0]=(-0.592961594036)/(o+1.)+(-0.350772955066)+(0.213491785855)*0.5**o
        ref[3,4,1,1]=(-0.532890980061)/(o+1.)+(-0.251121395257)+(-0.401744411188)*0.5**o
        ref[3,4,2,0]=(-0.194325734076)/(o+1.)+(0.581224356966)+(-0.0813611687514)*0.5**o
        ref[3,4,2,1]=(1.37635341581)/(o+1.)+(1.02866696565)+(-0.0991425040958)*0.5**o
      res=integrate(arg)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactOne_fromSymbol_ContinuousFunction_rank0(self):
      """
      tests integral of rank 0 Symbol on the ReducedFunctionOnContactOne

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=())
      s=Data(0,(),w)
      if dim==2:
        s=(0.88016247767)*x[0]**o+(0.546409254589)*x[0]+(-0.782935386141)*x[1]**o+(-0.21996752206)*x[1]
        ref=(-0.782935386141)/(o+1.)+(0.163220866264)+(0.88016247767)*0.5**o
      else:
        s=(0.436925914346)*x[0]**o+(0.18630264415)*x[0]+(-0.374230617371)*x[1]**o+(0.331719266581)*x[1]+(-0.327334599603)*x[2]**o+(-0.376893151193)*x[2]
        ref=(-0.701565216974)/(o+1.)+(0.0705643797689)+(0.436925914346)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactOne_fromSymbol_ContinuousFunction_rank1(self):
      """
      tests integral of rank 1 Symbol on the ReducedFunctionOnContactOne

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(2,))
      s=Data(0,(2,),w)
      ref=numpy.zeros((2,),numpy.float_)
      if dim==2:
        s[0]=(-0.0725655727924)*x[0]**o+(-0.539202679596)*x[0]+(0.00383805967589)*x[1]**o+(-0.42164917219)*x[1]
        s[1]=(0.799189397651)*x[0]**o+(0.468822667976)*x[0]+(0.946068596129)*x[1]**o+(-0.75397954631)*x[1]
        ref[0]=(0.00383805967589)/(o+1.)+(-0.480425925893)+(-0.0725655727924)*0.5**o
        ref[1]=(0.946068596129)/(o+1.)+(-0.142578439167)+(0.799189397651)*0.5**o
      else:
        s[0]=(0.94482411471)*x[0]**o+(-0.308967528984)*x[0]+(-0.417912410083)*x[1]**o+(-0.363984940852)*x[1]+(0.991921856553)*x[2]**o+(-0.264144586743)*x[2]
        s[1]=(-0.190159227288)*x[0]**o+(0.774206651681)*x[0]+(-0.585389796317)*x[1]**o+(-0.8203250901)*x[1]+(-0.959196504133)*x[2]**o+(0.118981346607)*x[2]
        ref[0]=(0.57400944647)/(o+1.)+(-0.46854852829)+(0.94482411471)*0.5**o
        ref[1]=(-1.54458630045)/(o+1.)+(0.036431454094)+(-0.190159227288)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactOne_fromSymbol_ContinuousFunction_rank2(self):
      """
      tests integral of rank 2 Symbol on the ReducedFunctionOnContactOne

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5))
      s=Data(0,(4, 5),w)
      ref=numpy.zeros((4, 5),numpy.float_)
      if dim==2:
        s[0,0]=(0.821123722738)*x[0]**o+(-0.953533567845)*x[0]+(0.761401252898)*x[1]**o+(0.560845798718)*x[1]
        s[0,1]=(0.388458475385)*x[0]**o+(0.787499438329)*x[0]+(0.13496554321)*x[1]**o+(0.558370810363)*x[1]
        s[0,2]=(-0.583597257844)*x[0]**o+(-0.203553852077)*x[0]+(0.400123717156)*x[1]**o+(-0.748904505246)*x[1]
        s[0,3]=(0.215629523788)*x[0]**o+(0.380761852867)*x[0]+(-0.656661093703)*x[1]**o+(0.707314164752)*x[1]
        s[0,4]=(-0.945147360815)*x[0]**o+(0.761197937419)*x[0]+(-0.242270565927)*x[1]**o+(0.197444941556)*x[1]
        s[1,0]=(-0.260643768467)*x[0]**o+(-0.429658355574)*x[0]+(0.832795677562)*x[1]**o+(0.0465882509708)*x[1]
        s[1,1]=(0.80280828918)*x[0]**o+(0.824378440768)*x[0]+(-0.0704713499903)*x[1]**o+(0.882600027718)*x[1]
        s[1,2]=(-0.188924905219)*x[0]**o+(-0.532960871866)*x[0]+(0.130210761298)*x[1]**o+(-0.847312599467)*x[1]
        s[1,3]=(-0.637355923048)*x[0]**o+(0.102559784731)*x[0]+(-0.00202798859835)*x[1]**o+(-0.275087027189)*x[1]
        s[1,4]=(-0.677415150613)*x[0]**o+(0.212870653829)*x[0]+(0.60045622057)*x[1]**o+(0.554319358843)*x[1]
        s[2,0]=(-0.758104298178)*x[0]**o+(0.122102590939)*x[0]+(0.82242042837)*x[1]**o+(0.877931466117)*x[1]
        s[2,1]=(0.295560509979)*x[0]**o+(0.0550082286163)*x[0]+(0.28149317542)*x[1]**o+(0.933306020272)*x[1]
        s[2,2]=(0.164458091911)*x[0]**o+(0.895923571884)*x[0]+(-0.89721569291)*x[1]**o+(-0.797286890655)*x[1]
        s[2,3]=(-0.455008349519)*x[0]**o+(-0.950347778241)*x[0]+(-0.52866158871)*x[1]**o+(0.784423551832)*x[1]
        s[2,4]=(0.0404391288743)*x[0]**o+(0.919895028488)*x[0]+(0.14131342368)*x[1]**o+(-0.534436226168)*x[1]
        s[3,0]=(0.313572834288)*x[0]**o+(-0.981579650669)*x[0]+(-0.105890320276)*x[1]**o+(0.184272310362)*x[1]
        s[3,1]=(0.517529943685)*x[0]**o+(-0.225899698465)*x[0]+(0.314725287596)*x[1]**o+(-0.520678188485)*x[1]
        s[3,2]=(0.457697644746)*x[0]**o+(0.580651564264)*x[0]+(0.0819245033577)*x[1]**o+(-0.962585666695)*x[1]
        s[3,3]=(-0.540006520671)*x[0]**o+(0.00341289905337)*x[0]+(-0.75656675994)*x[1]**o+(0.355040790858)*x[1]
        s[3,4]=(0.211995984801)*x[0]**o+(-0.907257558871)*x[0]+(0.736102007034)*x[1]**o+(-0.841283087071)*x[1]
        ref[0,0]=(0.761401252898)/(o+1.)+(-0.196343884564)+(0.821123722738)*0.5**o
        ref[0,1]=(0.13496554321)/(o+1.)+(0.672935124346)+(0.388458475385)*0.5**o
        ref[0,2]=(0.400123717156)/(o+1.)+(-0.476229178662)+(-0.583597257844)*0.5**o
        ref[0,3]=(-0.656661093703)/(o+1.)+(0.544038008809)+(0.215629523788)*0.5**o
        ref[0,4]=(-0.242270565927)/(o+1.)+(0.479321439487)+(-0.945147360815)*0.5**o
        ref[1,0]=(0.832795677562)/(o+1.)+(-0.191535052302)+(-0.260643768467)*0.5**o
        ref[1,1]=(-0.0704713499903)/(o+1.)+(0.853489234243)+(0.80280828918)*0.5**o
        ref[1,2]=(0.130210761298)/(o+1.)+(-0.690136735666)+(-0.188924905219)*0.5**o
        ref[1,3]=(-0.00202798859835)/(o+1.)+(-0.0862636212292)+(-0.637355923048)*0.5**o
        ref[1,4]=(0.60045622057)/(o+1.)+(0.383595006336)+(-0.677415150613)*0.5**o
        ref[2,0]=(0.82242042837)/(o+1.)+(0.500017028528)+(-0.758104298178)*0.5**o
        ref[2,1]=(0.28149317542)/(o+1.)+(0.494157124444)+(0.295560509979)*0.5**o
        ref[2,2]=(-0.89721569291)/(o+1.)+(0.0493183406148)+(0.164458091911)*0.5**o
        ref[2,3]=(-0.52866158871)/(o+1.)+(-0.0829621132044)+(-0.455008349519)*0.5**o
        ref[2,4]=(0.14131342368)/(o+1.)+(0.19272940116)+(0.0404391288743)*0.5**o
        ref[3,0]=(-0.105890320276)/(o+1.)+(-0.398653670153)+(0.313572834288)*0.5**o
        ref[3,1]=(0.314725287596)/(o+1.)+(-0.373288943475)+(0.517529943685)*0.5**o
        ref[3,2]=(0.0819245033577)/(o+1.)+(-0.190967051216)+(0.457697644746)*0.5**o
        ref[3,3]=(-0.75656675994)/(o+1.)+(0.179226844956)+(-0.540006520671)*0.5**o
        ref[3,4]=(0.736102007034)/(o+1.)+(-0.874270322971)+(0.211995984801)*0.5**o
      else:
        s[0,0]=(-0.26796664441)*x[0]**o+(0.412787191224)*x[0]+(0.291586980058)*x[1]**o+(0.363446579326)*x[1]+(-0.925503103962)*x[2]**o+(-0.815680149808)*x[2]
        s[0,1]=(0.816233130972)*x[0]**o+(-0.412368706967)*x[0]+(-0.633601565886)*x[1]**o+(-0.371336006063)*x[1]+(0.296754676336)*x[2]**o+(0.670055343075)*x[2]
        s[0,2]=(0.0115846578221)*x[0]**o+(0.0092074168084)*x[0]+(0.852395533346)*x[1]**o+(-0.536392264791)*x[1]+(-0.937066324616)*x[2]**o+(0.164445148469)*x[2]
        s[0,3]=(0.334112468463)*x[0]**o+(-0.197414532294)*x[0]+(0.32282859646)*x[1]**o+(-0.829489106617)*x[1]+(0.637135896229)*x[2]**o+(-0.385346619948)*x[2]
        s[0,4]=(0.633179246285)*x[0]**o+(-0.775108312054)*x[0]+(-0.0275404480368)*x[1]**o+(-0.119328918403)*x[1]+(0.56571174294)*x[2]**o+(-0.306269532057)*x[2]
        s[1,0]=(-0.440298361388)*x[0]**o+(-0.644698995147)*x[0]+(-0.815801747709)*x[1]**o+(-0.347258269595)*x[1]+(-0.614702701308)*x[2]**o+(0.439358767152)*x[2]
        s[1,1]=(-0.397657154377)*x[0]**o+(0.529856649499)*x[0]+(-0.737187187383)*x[1]**o+(-0.242702727135)*x[1]+(-0.823515970874)*x[2]**o+(0.609907664585)*x[2]
        s[1,2]=(0.438570722796)*x[0]**o+(0.699038412356)*x[0]+(-0.731471431635)*x[1]**o+(0.334652466823)*x[1]+(0.728626361708)*x[2]**o+(0.140706080335)*x[2]
        s[1,3]=(0.445813070034)*x[0]**o+(-0.223359077861)*x[0]+(0.607122403242)*x[1]**o+(0.684390142218)*x[1]+(-0.0448436266022)*x[2]**o+(0.0177789962153)*x[2]
        s[1,4]=(-0.618164233561)*x[0]**o+(0.174313138771)*x[0]+(-0.891694340174)*x[1]**o+(-0.210641679269)*x[1]+(0.961104454395)*x[2]**o+(-0.335141946858)*x[2]
        s[2,0]=(0.824078577041)*x[0]**o+(-0.206045431624)*x[0]+(0.055588699321)*x[1]**o+(0.350116387216)*x[1]+(0.607866274938)*x[2]**o+(0.407638105768)*x[2]
        s[2,1]=(-0.210261210166)*x[0]**o+(-0.0133233759756)*x[0]+(0.801022275664)*x[1]**o+(-0.267209121471)*x[1]+(0.535577530444)*x[2]**o+(-0.541166709451)*x[2]
        s[2,2]=(0.346542013177)*x[0]**o+(0.305602431422)*x[0]+(0.45213719675)*x[1]**o+(0.32765019418)*x[1]+(0.367619644833)*x[2]**o+(0.933760631839)*x[2]
        s[2,3]=(-0.244955187752)*x[0]**o+(-0.645641982024)*x[0]+(0.0261013017433)*x[1]**o+(-0.230403895757)*x[1]+(0.201233200314)*x[2]**o+(-0.783467696033)*x[2]
        s[2,4]=(0.783620590334)*x[0]**o+(0.325268485711)*x[0]+(-0.912841443404)*x[1]**o+(-0.500647833534)*x[1]+(-0.980126267268)*x[2]**o+(-0.425755185755)*x[2]
        s[3,0]=(-0.4690769573)*x[0]**o+(-0.547691242018)*x[0]+(0.996570266539)*x[1]**o+(-0.472334765569)*x[1]+(0.81903298756)*x[2]**o+(-0.654817201284)*x[2]
        s[3,1]=(0.617564682102)*x[0]**o+(-0.991757850114)*x[0]+(-0.34214251344)*x[1]**o+(-0.321397198526)*x[1]+(0.503434489432)*x[2]**o+(0.683522308565)*x[2]
        s[3,2]=(0.999035085465)*x[0]**o+(0.431261164559)*x[0]+(0.660309836881)*x[1]**o+(0.596465902975)*x[1]+(-0.575064993246)*x[2]**o+(-0.290933980474)*x[2]
        s[3,3]=(0.355357213484)*x[0]**o+(-0.638400362695)*x[0]+(0.596482099946)*x[1]**o+(0.861245903515)*x[1]+(0.877992083876)*x[2]**o+(0.221520840799)*x[2]
        s[3,4]=(-0.0280956096914)*x[0]**o+(0.880536602898)*x[0]+(0.00969665494423)*x[1]**o+(-0.182537740984)*x[1]+(0.84927089239)*x[2]**o+(0.11423223273)*x[2]
        ref[0,0]=(-0.633916123904)/(o+1.)+(-0.0197231896285)+(-0.26796664441)*0.5**o
        ref[0,1]=(-0.33684688955)/(o+1.)+(-0.0568246849779)+(0.816233130972)*0.5**o
        ref[0,2]=(-0.0846707912707)/(o+1.)+(-0.181369849757)+(0.0115846578221)*0.5**o
        ref[0,3]=(0.959964492689)/(o+1.)+(-0.70612512943)+(0.334112468463)*0.5**o
        ref[0,4]=(0.538171294903)/(o+1.)+(-0.600353381257)+(0.633179246285)*0.5**o
        ref[1,0]=(-1.43050444902)/(o+1.)+(-0.276299248795)+(-0.440298361388)*0.5**o
        ref[1,1]=(-1.56070315826)/(o+1.)+(0.448530793475)+(-0.397657154377)*0.5**o
        ref[1,2]=(-0.00284506992628)/(o+1.)+(0.587198479757)+(0.438570722796)*0.5**o
        ref[1,3]=(0.56227877664)/(o+1.)+(0.239405030286)+(0.445813070034)*0.5**o
        ref[1,4]=(0.0694101142218)/(o+1.)+(-0.185735243678)+(-0.618164233561)*0.5**o
        ref[2,0]=(0.663454974259)/(o+1.)+(0.27585453068)+(0.824078577041)*0.5**o
        ref[2,1]=(1.33659980611)/(o+1.)+(-0.410849603449)+(-0.210261210166)*0.5**o
        ref[2,2]=(0.819756841583)/(o+1.)+(0.78350662872)+(0.346542013177)*0.5**o
        ref[2,3]=(0.227334502058)/(o+1.)+(-0.829756786907)+(-0.244955187752)*0.5**o
        ref[2,4]=(-1.89296771067)/(o+1.)+(-0.300567266789)+(0.783620590334)*0.5**o
        ref[3,0]=(1.8156032541)/(o+1.)+(-0.837421604436)+(-0.4690769573)*0.5**o
        ref[3,1]=(0.161291975992)/(o+1.)+(-0.314816370037)+(0.617564682102)*0.5**o
        ref[3,2]=(0.0852448436355)/(o+1.)+(0.36839654353)+(0.999035085465)*0.5**o
        ref[3,3]=(1.47447418382)/(o+1.)+(0.222183190809)+(0.355357213484)*0.5**o
        ref[3,4]=(0.858967547335)/(o+1.)+(0.406115547322)+(-0.0280956096914)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactOne_fromSymbol_ContinuousFunction_rank3(self):
      """
      tests integral of rank 3 Symbol on the ReducedFunctionOnContactOne

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(6, 2, 2))
      s=Data(0,(6, 2, 2),w)
      ref=numpy.zeros((6, 2, 2),numpy.float_)
      if dim==2:
        s[0,0,0]=(0.230469008444)*x[0]**o+(0.508027324248)*x[0]+(0.696459735501)*x[1]**o+(-0.195259592119)*x[1]
        s[0,0,1]=(0.135411134228)*x[0]**o+(-0.684964577317)*x[0]+(0.804062624988)*x[1]**o+(0.388383344617)*x[1]
        s[0,1,0]=(-0.965361972117)*x[0]**o+(-0.0740755184742)*x[0]+(0.00222395987317)*x[1]**o+(0.116566925701)*x[1]
        s[0,1,1]=(0.773749135888)*x[0]**o+(-0.236062924811)*x[0]+(0.824300232936)*x[1]**o+(0.58451198061)*x[1]
        s[1,0,0]=(0.213202511915)*x[0]**o+(-0.0164093785777)*x[0]+(-0.152629558111)*x[1]**o+(-0.663542824781)*x[1]
        s[1,0,1]=(-0.446430833827)*x[0]**o+(0.643410978113)*x[0]+(0.35649029144)*x[1]**o+(-0.914705065197)*x[1]
        s[1,1,0]=(0.616559079235)*x[0]**o+(0.347804225396)*x[0]+(-0.733945941231)*x[1]**o+(0.49771373099)*x[1]
        s[1,1,1]=(-0.398631849965)*x[0]**o+(-0.854719511493)*x[0]+(0.501641783941)*x[1]**o+(0.360567791644)*x[1]
        s[2,0,0]=(0.404312756516)*x[0]**o+(0.805967978591)*x[0]+(0.980468298218)*x[1]**o+(0.368530205154)*x[1]
        s[2,0,1]=(0.92908409018)*x[0]**o+(0.984893826849)*x[0]+(-0.706723295076)*x[1]**o+(-0.68485578941)*x[1]
        s[2,1,0]=(-0.395198069725)*x[0]**o+(-0.865731577936)*x[0]+(0.284248647254)*x[1]**o+(0.199510619169)*x[1]
        s[2,1,1]=(-0.383178260681)*x[0]**o+(-0.0753678998731)*x[0]+(0.68853605046)*x[1]**o+(0.693301872966)*x[1]
        s[3,0,0]=(-0.504525570613)*x[0]**o+(-0.295454443098)*x[0]+(-0.85334585793)*x[1]**o+(-0.450742110681)*x[1]
        s[3,0,1]=(0.0306385968872)*x[0]**o+(-0.0918977201143)*x[0]+(-0.106665376273)*x[1]**o+(0.158103530025)*x[1]
        s[3,1,0]=(-0.371403581207)*x[0]**o+(0.662079642408)*x[0]+(0.818746082332)*x[1]**o+(0.601651002406)*x[1]
        s[3,1,1]=(-0.93523720418)*x[0]**o+(0.871800352376)*x[0]+(-0.294513821338)*x[1]**o+(-0.947893176361)*x[1]
        s[4,0,0]=(-0.734908326587)*x[0]**o+(0.481209886891)*x[0]+(0.61811626923)*x[1]**o+(-0.21778354281)*x[1]
        s[4,0,1]=(-0.689233989565)*x[0]**o+(0.1273357199)*x[0]+(0.398820148577)*x[1]**o+(0.538945475692)*x[1]
        s[4,1,0]=(0.885178741071)*x[0]**o+(-0.351046930859)*x[0]+(0.492384326646)*x[1]**o+(0.584584261959)*x[1]
        s[4,1,1]=(-0.561791063188)*x[0]**o+(0.701763941021)*x[0]+(-0.242041214153)*x[1]**o+(0.297044144162)*x[1]
        s[5,0,0]=(-0.487786682467)*x[0]**o+(-0.69717117402)*x[0]+(-0.308038610282)*x[1]**o+(0.966752012177)*x[1]
        s[5,0,1]=(0.26556821402)*x[0]**o+(-0.983047207652)*x[0]+(0.276098865961)*x[1]**o+(0.840073469203)*x[1]
        s[5,1,0]=(-0.152606883933)*x[0]**o+(-0.481720281304)*x[0]+(0.417680077266)*x[1]**o+(-0.882323563007)*x[1]
        s[5,1,1]=(0.0674752363532)*x[0]**o+(0.727094741288)*x[0]+(-0.299947571476)*x[1]**o+(0.391301157504)*x[1]
        ref[0,0,0]=(0.696459735501)/(o+1.)+(0.156383866065)+(0.230469008444)*0.5**o
        ref[0,0,1]=(0.804062624988)/(o+1.)+(-0.14829061635)+(0.135411134228)*0.5**o
        ref[0,1,0]=(0.00222395987317)/(o+1.)+(0.0212457036136)+(-0.965361972117)*0.5**o
        ref[0,1,1]=(0.824300232936)/(o+1.)+(0.174224527899)+(0.773749135888)*0.5**o
        ref[1,0,0]=(-0.152629558111)/(o+1.)+(-0.339976101679)+(0.213202511915)*0.5**o
        ref[1,0,1]=(0.35649029144)/(o+1.)+(-0.135647043542)+(-0.446430833827)*0.5**o
        ref[1,1,0]=(-0.733945941231)/(o+1.)+(0.422758978193)+(0.616559079235)*0.5**o
        ref[1,1,1]=(0.501641783941)/(o+1.)+(-0.247075859924)+(-0.398631849965)*0.5**o
        ref[2,0,0]=(0.980468298218)/(o+1.)+(0.587249091872)+(0.404312756516)*0.5**o
        ref[2,0,1]=(-0.706723295076)/(o+1.)+(0.15001901872)+(0.92908409018)*0.5**o
        ref[2,1,0]=(0.284248647254)/(o+1.)+(-0.333110479383)+(-0.395198069725)*0.5**o
        ref[2,1,1]=(0.68853605046)/(o+1.)+(0.308966986547)+(-0.383178260681)*0.5**o
        ref[3,0,0]=(-0.85334585793)/(o+1.)+(-0.37309827689)+(-0.504525570613)*0.5**o
        ref[3,0,1]=(-0.106665376273)/(o+1.)+(0.0331029049554)+(0.0306385968872)*0.5**o
        ref[3,1,0]=(0.818746082332)/(o+1.)+(0.631865322407)+(-0.371403581207)*0.5**o
        ref[3,1,1]=(-0.294513821338)/(o+1.)+(-0.0380464119924)+(-0.93523720418)*0.5**o
        ref[4,0,0]=(0.61811626923)/(o+1.)+(0.131713172041)+(-0.734908326587)*0.5**o
        ref[4,0,1]=(0.398820148577)/(o+1.)+(0.333140597796)+(-0.689233989565)*0.5**o
        ref[4,1,0]=(0.492384326646)/(o+1.)+(0.11676866555)+(0.885178741071)*0.5**o
        ref[4,1,1]=(-0.242041214153)/(o+1.)+(0.499404042592)+(-0.561791063188)*0.5**o
        ref[5,0,0]=(-0.308038610282)/(o+1.)+(0.134790419079)+(-0.487786682467)*0.5**o
        ref[5,0,1]=(0.276098865961)/(o+1.)+(-0.0714868692245)+(0.26556821402)*0.5**o
        ref[5,1,0]=(0.417680077266)/(o+1.)+(-0.682021922156)+(-0.152606883933)*0.5**o
        ref[5,1,1]=(-0.299947571476)/(o+1.)+(0.559197949396)+(0.0674752363532)*0.5**o
      else:
        s[0,0,0]=(-0.226564859572)*x[0]**o+(0.759975882725)*x[0]+(0.659032148551)*x[1]**o+(0.0807090996868)*x[1]+(-0.586765501877)*x[2]**o+(0.0274224545344)*x[2]
        s[0,0,1]=(0.926463229691)*x[0]**o+(0.134519539954)*x[0]+(0.976918539353)*x[1]**o+(-0.352762024083)*x[1]+(0.132042653159)*x[2]**o+(-0.686161947886)*x[2]
        s[0,1,0]=(0.371916775894)*x[0]**o+(-0.586490002548)*x[0]+(-0.304695527165)*x[1]**o+(0.440748639515)*x[1]+(-0.277758803854)*x[2]**o+(0.933648409631)*x[2]
        s[0,1,1]=(-0.871254388888)*x[0]**o+(-0.373959360688)*x[0]+(0.613060994689)*x[1]**o+(0.261906429329)*x[1]+(-0.0255596243956)*x[2]**o+(-0.912700299076)*x[2]
        s[1,0,0]=(0.0075965229641)*x[0]**o+(-0.977596529897)*x[0]+(0.160602293774)*x[1]**o+(-0.407220178255)*x[1]+(0.722872196439)*x[2]**o+(0.96683722852)*x[2]
        s[1,0,1]=(0.197284854866)*x[0]**o+(0.689180749331)*x[0]+(0.476641393876)*x[1]**o+(-0.575644071726)*x[1]+(-0.988391730968)*x[2]**o+(0.6278575683)*x[2]
        s[1,1,0]=(-0.968464883049)*x[0]**o+(0.415629727234)*x[0]+(0.698016769139)*x[1]**o+(0.967345907027)*x[1]+(0.0278637129728)*x[2]**o+(-0.895746388886)*x[2]
        s[1,1,1]=(-0.240984840545)*x[0]**o+(-0.0680920485806)*x[0]+(-0.159143749001)*x[1]**o+(-0.0595387242977)*x[1]+(0.564402939251)*x[2]**o+(-0.701335469294)*x[2]
        s[2,0,0]=(0.239124234745)*x[0]**o+(-0.86570078201)*x[0]+(-0.331189286269)*x[1]**o+(0.141412115059)*x[1]+(-0.498257980574)*x[2]**o+(-0.560030264671)*x[2]
        s[2,0,1]=(-0.126152295994)*x[0]**o+(0.220733769906)*x[0]+(0.293598600333)*x[1]**o+(0.0232185863704)*x[1]+(-0.277149571127)*x[2]**o+(0.686042508229)*x[2]
        s[2,1,0]=(-0.415700364121)*x[0]**o+(-0.998036555412)*x[0]+(0.932589123491)*x[1]**o+(0.474376426544)*x[1]+(0.868363410527)*x[2]**o+(0.20412367755)*x[2]
        s[2,1,1]=(-0.699650926538)*x[0]**o+(-0.885390927226)*x[0]+(0.315114808594)*x[1]**o+(0.0964378966171)*x[1]+(0.208547711734)*x[2]**o+(-0.234035721141)*x[2]
        s[3,0,0]=(-0.876365887814)*x[0]**o+(-0.809219980908)*x[0]+(-0.956629553931)*x[1]**o+(0.879512507986)*x[1]+(0.263252429544)*x[2]**o+(-0.0839386837172)*x[2]
        s[3,0,1]=(-0.667178396657)*x[0]**o+(-0.16553182711)*x[0]+(0.118512822439)*x[1]**o+(0.855526624871)*x[1]+(0.367375405597)*x[2]**o+(0.809554378159)*x[2]
        s[3,1,0]=(-0.822091423901)*x[0]**o+(0.805358337724)*x[0]+(-0.0169105328354)*x[1]**o+(0.911509787417)*x[1]+(0.201376799793)*x[2]**o+(-0.421841209698)*x[2]
        s[3,1,1]=(-0.924137416522)*x[0]**o+(-0.513720649867)*x[0]+(0.997645049075)*x[1]**o+(0.0427298712226)*x[1]+(0.0973602746566)*x[2]**o+(-0.210094600037)*x[2]
        s[4,0,0]=(-0.900263073705)*x[0]**o+(-0.348978017479)*x[0]+(-0.670123763869)*x[1]**o+(0.842214206909)*x[1]+(0.487852589522)*x[2]**o+(-0.990296977407)*x[2]
        s[4,0,1]=(0.733939716764)*x[0]**o+(0.802871411295)*x[0]+(-0.72554972641)*x[1]**o+(-0.675299974439)*x[1]+(-0.851276375175)*x[2]**o+(0.185926545414)*x[2]
        s[4,1,0]=(-0.00700280887647)*x[0]**o+(-0.588965317678)*x[0]+(-0.709984221808)*x[1]**o+(-0.278713234487)*x[1]+(0.571081276655)*x[2]**o+(-0.226676185948)*x[2]
        s[4,1,1]=(0.227749111196)*x[0]**o+(-0.682960403376)*x[0]+(0.26793708046)*x[1]**o+(-0.366550488255)*x[1]+(-0.888857273365)*x[2]**o+(0.0688817602584)*x[2]
        s[5,0,0]=(-0.125200843744)*x[0]**o+(-0.629160785496)*x[0]+(0.899283261399)*x[1]**o+(0.550308867618)*x[1]+(0.768807415182)*x[2]**o+(0.443292284662)*x[2]
        s[5,0,1]=(-0.0304781411569)*x[0]**o+(-0.560459929762)*x[0]+(0.588807987213)*x[1]**o+(0.330516228598)*x[1]+(-0.620114930397)*x[2]**o+(-0.598508456872)*x[2]
        s[5,1,0]=(0.538580607824)*x[0]**o+(0.0303519877944)*x[0]+(-0.26997035534)*x[1]**o+(0.364935864108)*x[1]+(0.748685142241)*x[2]**o+(0.0962889009161)*x[2]
        s[5,1,1]=(-0.54994010911)*x[0]**o+(0.159394722103)*x[0]+(-0.21371135673)*x[1]**o+(-0.395979383594)*x[1]+(-0.314185117077)*x[2]**o+(-0.204275202783)*x[2]
        ref[0,0,0]=(0.0722666466735)/(o+1.)+(0.434053718473)+(-0.226564859572)*0.5**o
        ref[0,0,1]=(1.10896119251)/(o+1.)+(-0.452202216007)+(0.926463229691)*0.5**o
        ref[0,1,0]=(-0.58245433102)/(o+1.)+(0.393953523299)+(0.371916775894)*0.5**o
        ref[0,1,1]=(0.587501370293)/(o+1.)+(-0.512376615217)+(-0.871254388888)*0.5**o
        ref[1,0,0]=(0.883474490213)/(o+1.)+(-0.208989739816)+(0.0075965229641)*0.5**o
        ref[1,0,1]=(-0.511750337092)/(o+1.)+(0.370697122953)+(0.197284854866)*0.5**o
        ref[1,1,0]=(0.725880482112)/(o+1.)+(0.243614622688)+(-0.968464883049)*0.5**o
        ref[1,1,1]=(0.405259190249)/(o+1.)+(-0.414483121086)+(-0.240984840545)*0.5**o
        ref[2,0,0]=(-0.829447266843)/(o+1.)+(-0.642159465811)+(0.239124234745)*0.5**o
        ref[2,0,1]=(0.0164490292067)/(o+1.)+(0.464997432253)+(-0.126152295994)*0.5**o
        ref[2,1,0]=(1.80095253402)/(o+1.)+(-0.159768225659)+(-0.415700364121)*0.5**o
        ref[2,1,1]=(0.523662520328)/(o+1.)+(-0.511494375875)+(-0.699650926538)*0.5**o
        ref[3,0,0]=(-0.693377124388)/(o+1.)+(-0.00682307831984)+(-0.876365887814)*0.5**o
        ref[3,0,1]=(0.485888228036)/(o+1.)+(0.74977458796)+(-0.667178396657)*0.5**o
        ref[3,1,0]=(0.184466266958)/(o+1.)+(0.647513457721)+(-0.822091423901)*0.5**o
        ref[3,1,1]=(1.09500532373)/(o+1.)+(-0.340542689341)+(-0.924137416522)*0.5**o
        ref[4,0,0]=(-0.182271174347)/(o+1.)+(-0.248530393989)+(-0.900263073705)*0.5**o
        ref[4,0,1]=(-1.57682610159)/(o+1.)+(0.156748991135)+(0.733939716764)*0.5**o
        ref[4,1,0]=(-0.138902945153)/(o+1.)+(-0.547177369056)+(-0.00700280887647)*0.5**o
        ref[4,1,1]=(-0.620920192905)/(o+1.)+(-0.490314565686)+(0.227749111196)*0.5**o
        ref[5,0,0]=(1.66809067658)/(o+1.)+(0.182220183392)+(-0.125200843744)*0.5**o
        ref[5,0,1]=(-0.0313069431837)/(o+1.)+(-0.414226079018)+(-0.0304781411569)*0.5**o
        ref[5,1,0]=(0.478714786901)/(o+1.)+(0.245788376409)+(0.538580607824)*0.5**o
        ref[5,1,1]=(-0.527896473807)/(o+1.)+(-0.220429932137)+(-0.54994010911)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactOne_fromSymbol_ContinuousFunction_rank4(self):
      """
      tests integral of rank 4 Symbol on the ReducedFunctionOnContactOne

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5, 3, 2))
      s=Data(0,(4, 5, 3, 2),w)
      ref=numpy.zeros((4, 5, 3, 2),numpy.float_)
      if dim==2:
        s[0,0,0,0]=(0.826151835054)*x[0]**o+(0.778522285868)*x[0]+(-0.798735149387)*x[1]**o+(-0.875242059866)*x[1]
        s[0,0,0,1]=(0.281853758764)*x[0]**o+(-0.194633585298)*x[0]+(-0.559216870357)*x[1]**o+(0.699122025971)*x[1]
        s[0,0,1,0]=(0.734893466243)*x[0]**o+(0.629132659777)*x[0]+(0.147096504378)*x[1]**o+(0.11035713758)*x[1]
        s[0,0,1,1]=(0.928300107915)*x[0]**o+(-0.334284397134)*x[0]+(-0.978653421141)*x[1]**o+(-0.523491002413)*x[1]
        s[0,0,2,0]=(-0.84104720017)*x[0]**o+(-0.977429111913)*x[0]+(0.975786707279)*x[1]**o+(0.202832002651)*x[1]
        s[0,0,2,1]=(0.505216717255)*x[0]**o+(0.893953504598)*x[0]+(-0.478609919678)*x[1]**o+(-0.0960789116423)*x[1]
        s[0,1,0,0]=(-0.588826769132)*x[0]**o+(-0.793581403019)*x[0]+(0.0325820539534)*x[1]**o+(0.479489246372)*x[1]
        s[0,1,0,1]=(0.201705696091)*x[0]**o+(0.807828768455)*x[0]+(0.943593904137)*x[1]**o+(0.805216955013)*x[1]
        s[0,1,1,0]=(0.458858343943)*x[0]**o+(0.687368390273)*x[0]+(0.106488644042)*x[1]**o+(-0.53370113771)*x[1]
        s[0,1,1,1]=(-0.445360193781)*x[0]**o+(-0.0579880476857)*x[0]+(0.164158781184)*x[1]**o+(0.610932454554)*x[1]
        s[0,1,2,0]=(0.631597276525)*x[0]**o+(-0.958557448004)*x[0]+(-0.484354639056)*x[1]**o+(0.151153827438)*x[1]
        s[0,1,2,1]=(-0.742804913722)*x[0]**o+(-0.841835209231)*x[0]+(-0.209463960361)*x[1]**o+(-0.858110011148)*x[1]
        s[0,2,0,0]=(-0.757921024916)*x[0]**o+(0.63659891937)*x[0]+(0.807935626935)*x[1]**o+(0.902142249374)*x[1]
        s[0,2,0,1]=(-0.256981783889)*x[0]**o+(-0.610855048181)*x[0]+(-0.574033583989)*x[1]**o+(0.776282494782)*x[1]
        s[0,2,1,0]=(-0.943218252718)*x[0]**o+(0.516037703505)*x[0]+(0.341543474661)*x[1]**o+(-0.240364780932)*x[1]
        s[0,2,1,1]=(0.792692625218)*x[0]**o+(-0.323976519304)*x[0]+(-0.0146665511353)*x[1]**o+(-0.735190429984)*x[1]
        s[0,2,2,0]=(-0.30799523168)*x[0]**o+(-0.679564713739)*x[0]+(0.459127847422)*x[1]**o+(0.232465201074)*x[1]
        s[0,2,2,1]=(-0.610374384606)*x[0]**o+(-0.953734773788)*x[0]+(0.548484423475)*x[1]**o+(-0.615367713658)*x[1]
        s[0,3,0,0]=(0.657511913357)*x[0]**o+(-0.145435401393)*x[0]+(0.994651694429)*x[1]**o+(0.384878777184)*x[1]
        s[0,3,0,1]=(0.583248456434)*x[0]**o+(-0.187182307545)*x[0]+(0.944791134312)*x[1]**o+(-0.186032769285)*x[1]
        s[0,3,1,0]=(-0.781508935766)*x[0]**o+(0.860022230884)*x[0]+(-0.101044667455)*x[1]**o+(-0.222316103336)*x[1]
        s[0,3,1,1]=(-0.867519386038)*x[0]**o+(0.929281502241)*x[0]+(-0.946297144806)*x[1]**o+(0.817224805708)*x[1]
        s[0,3,2,0]=(-0.281309467001)*x[0]**o+(-0.037739241916)*x[0]+(0.535177813023)*x[1]**o+(-0.729632726113)*x[1]
        s[0,3,2,1]=(-0.860996437102)*x[0]**o+(-0.0283243105523)*x[0]+(0.291253331071)*x[1]**o+(-0.49621519253)*x[1]
        s[0,4,0,0]=(-0.499079353771)*x[0]**o+(0.0512126982011)*x[0]+(-0.264697628051)*x[1]**o+(-0.557092064587)*x[1]
        s[0,4,0,1]=(0.886952704303)*x[0]**o+(0.835576589274)*x[0]+(0.436241831047)*x[1]**o+(0.205000756969)*x[1]
        s[0,4,1,0]=(-0.794113947126)*x[0]**o+(0.272838850139)*x[0]+(0.596844427835)*x[1]**o+(-0.0770378590458)*x[1]
        s[0,4,1,1]=(-0.0741729381021)*x[0]**o+(-0.352184991227)*x[0]+(-0.47050747773)*x[1]**o+(-0.0426137892879)*x[1]
        s[0,4,2,0]=(-0.574730110675)*x[0]**o+(0.72928953398)*x[0]+(0.570813169088)*x[1]**o+(-0.140526724895)*x[1]
        s[0,4,2,1]=(-0.956657753944)*x[0]**o+(0.83653343098)*x[0]+(0.156596161146)*x[1]**o+(0.565547857113)*x[1]
        s[1,0,0,0]=(-0.951078063957)*x[0]**o+(-0.76755461909)*x[0]+(0.889791199852)*x[1]**o+(-0.751303752794)*x[1]
        s[1,0,0,1]=(-0.601689692228)*x[0]**o+(-0.872977808857)*x[0]+(-0.216836641085)*x[1]**o+(-0.141390507272)*x[1]
        s[1,0,1,0]=(0.963503254167)*x[0]**o+(0.881676487866)*x[0]+(0.98286908038)*x[1]**o+(-0.387200882531)*x[1]
        s[1,0,1,1]=(-0.220998216482)*x[0]**o+(-0.942896953922)*x[0]+(-0.784318364616)*x[1]**o+(-0.117636798188)*x[1]
        s[1,0,2,0]=(-0.0629547087136)*x[0]**o+(0.375822895811)*x[0]+(0.297365047085)*x[1]**o+(-0.108671078054)*x[1]
        s[1,0,2,1]=(-0.898579538704)*x[0]**o+(-0.611755404412)*x[0]+(0.8150001419)*x[1]**o+(-0.956535007847)*x[1]
        s[1,1,0,0]=(-0.0983531790455)*x[0]**o+(0.399432233914)*x[0]+(0.455058141159)*x[1]**o+(-0.272367862823)*x[1]
        s[1,1,0,1]=(0.198516567418)*x[0]**o+(-0.599045119583)*x[0]+(-0.660954533474)*x[1]**o+(-0.138080345883)*x[1]
        s[1,1,1,0]=(-0.345947424795)*x[0]**o+(-0.112667218524)*x[0]+(0.389246060633)*x[1]**o+(0.554453113274)*x[1]
        s[1,1,1,1]=(-0.195677951352)*x[0]**o+(-0.926026514503)*x[0]+(0.206735465226)*x[1]**o+(0.789299584752)*x[1]
        s[1,1,2,0]=(0.286674162919)*x[0]**o+(-0.836060882258)*x[0]+(0.728613552854)*x[1]**o+(0.684001924365)*x[1]
        s[1,1,2,1]=(0.462677800728)*x[0]**o+(0.128558178852)*x[0]+(-0.234360929327)*x[1]**o+(-0.0906684049145)*x[1]
        s[1,2,0,0]=(-0.618359459011)*x[0]**o+(0.31224194045)*x[0]+(0.826946704031)*x[1]**o+(-0.383586867066)*x[1]
        s[1,2,0,1]=(0.79180929497)*x[0]**o+(-0.00403790010433)*x[0]+(-0.610232300686)*x[1]**o+(0.655034409375)*x[1]
        s[1,2,1,0]=(-0.454630490901)*x[0]**o+(-0.294433649415)*x[0]+(0.222616002742)*x[1]**o+(-0.880830082688)*x[1]
        s[1,2,1,1]=(-0.358323719668)*x[0]**o+(-0.506159090503)*x[0]+(-0.298070302518)*x[1]**o+(0.670788233813)*x[1]
        s[1,2,2,0]=(-0.953302373475)*x[0]**o+(0.262673942284)*x[0]+(0.293175442678)*x[1]**o+(0.327941364282)*x[1]
        s[1,2,2,1]=(0.0847099318446)*x[0]**o+(-0.378899343492)*x[0]+(-0.656552393979)*x[1]**o+(0.578336235439)*x[1]
        s[1,3,0,0]=(0.0792265074365)*x[0]**o+(-0.703381588828)*x[0]+(0.0644008346021)*x[1]**o+(-0.414604593416)*x[1]
        s[1,3,0,1]=(-0.312474679341)*x[0]**o+(0.117859249242)*x[0]+(-0.712500149358)*x[1]**o+(-0.160567536664)*x[1]
        s[1,3,1,0]=(0.0930078501772)*x[0]**o+(0.235241217821)*x[0]+(-0.74265516534)*x[1]**o+(-0.737722766295)*x[1]
        s[1,3,1,1]=(0.839924906913)*x[0]**o+(0.937666844502)*x[0]+(0.0794334270857)*x[1]**o+(0.863046562296)*x[1]
        s[1,3,2,0]=(0.154200394821)*x[0]**o+(-0.66290218047)*x[0]+(-0.206887854301)*x[1]**o+(0.504705955597)*x[1]
        s[1,3,2,1]=(0.427091397128)*x[0]**o+(0.646434479942)*x[0]+(-0.499236661621)*x[1]**o+(-0.97608178001)*x[1]
        s[1,4,0,0]=(0.993473425442)*x[0]**o+(-0.385478858515)*x[0]+(0.991785227798)*x[1]**o+(-0.317386719533)*x[1]
        s[1,4,0,1]=(0.905969952921)*x[0]**o+(0.580348604986)*x[0]+(-0.549435231471)*x[1]**o+(-0.864673820376)*x[1]
        s[1,4,1,0]=(-0.414114927962)*x[0]**o+(0.192124891008)*x[0]+(0.270145165409)*x[1]**o+(0.984593711748)*x[1]
        s[1,4,1,1]=(-0.537634564837)*x[0]**o+(-0.832898745236)*x[0]+(0.987067570291)*x[1]**o+(-0.829413594749)*x[1]
        s[1,4,2,0]=(0.298204375484)*x[0]**o+(0.595594979515)*x[0]+(0.691510051463)*x[1]**o+(0.404779663287)*x[1]
        s[1,4,2,1]=(0.924136889038)*x[0]**o+(-0.834498479792)*x[0]+(-0.0920482079502)*x[1]**o+(0.653266085415)*x[1]
        s[2,0,0,0]=(-0.623512868124)*x[0]**o+(-0.674266041327)*x[0]+(-0.0496779342723)*x[1]**o+(-0.0576390340208)*x[1]
        s[2,0,0,1]=(-0.675571390646)*x[0]**o+(0.212900825131)*x[0]+(-0.698331157728)*x[1]**o+(0.323039640381)*x[1]
        s[2,0,1,0]=(-0.137713646762)*x[0]**o+(0.678939423651)*x[0]+(0.724413391261)*x[1]**o+(-0.758331483567)*x[1]
        s[2,0,1,1]=(-0.790333229868)*x[0]**o+(0.0388624829814)*x[0]+(-0.163758448345)*x[1]**o+(0.749265467726)*x[1]
        s[2,0,2,0]=(0.401526106126)*x[0]**o+(-0.654255384177)*x[0]+(0.654866937965)*x[1]**o+(0.395353725816)*x[1]
        s[2,0,2,1]=(-0.691391545016)*x[0]**o+(0.576097089046)*x[0]+(-0.351703606836)*x[1]**o+(0.272891465932)*x[1]
        s[2,1,0,0]=(-0.994782416678)*x[0]**o+(0.417827176298)*x[0]+(0.933443347736)*x[1]**o+(0.729641898342)*x[1]
        s[2,1,0,1]=(0.888393462781)*x[0]**o+(0.0669727248876)*x[0]+(0.0751230852899)*x[1]**o+(0.79394522379)*x[1]
        s[2,1,1,0]=(-0.67951464573)*x[0]**o+(-0.398740440509)*x[0]+(-0.200110899369)*x[1]**o+(-0.0313062595594)*x[1]
        s[2,1,1,1]=(0.769634138219)*x[0]**o+(-0.873544326827)*x[0]+(-0.231357242392)*x[1]**o+(0.728827227861)*x[1]
        s[2,1,2,0]=(-0.740242586287)*x[0]**o+(0.779752812047)*x[0]+(-0.935858357069)*x[1]**o+(0.423733076889)*x[1]
        s[2,1,2,1]=(0.0522567442585)*x[0]**o+(-0.196246914879)*x[0]+(0.594009331267)*x[1]**o+(0.390510523467)*x[1]
        s[2,2,0,0]=(-0.202135247802)*x[0]**o+(0.406919371556)*x[0]+(0.336054961442)*x[1]**o+(-0.00656791632921)*x[1]
        s[2,2,0,1]=(-0.18797538396)*x[0]**o+(-0.502138169684)*x[0]+(0.728985032074)*x[1]**o+(-0.851899173542)*x[1]
        s[2,2,1,0]=(-0.1657521328)*x[0]**o+(-0.235483396897)*x[0]+(0.923860896997)*x[1]**o+(-0.534524775419)*x[1]
        s[2,2,1,1]=(0.318366946697)*x[0]**o+(-0.714726079393)*x[0]+(0.625778742556)*x[1]**o+(-0.364822812091)*x[1]
        s[2,2,2,0]=(0.796633973705)*x[0]**o+(0.20378716389)*x[0]+(-0.428464082609)*x[1]**o+(0.414097140984)*x[1]
        s[2,2,2,1]=(0.430909811076)*x[0]**o+(-0.711249875145)*x[0]+(0.234909867943)*x[1]**o+(0.469828473347)*x[1]
        s[2,3,0,0]=(0.0637185370545)*x[0]**o+(-0.347041553087)*x[0]+(-0.415901909163)*x[1]**o+(0.302973266512)*x[1]
        s[2,3,0,1]=(-0.0514048718664)*x[0]**o+(0.577688892083)*x[0]+(-0.847060157535)*x[1]**o+(-0.295910657349)*x[1]
        s[2,3,1,0]=(0.272384814885)*x[0]**o+(-0.457091092283)*x[0]+(0.294803721831)*x[1]**o+(0.863908100714)*x[1]
        s[2,3,1,1]=(-0.171649346026)*x[0]**o+(0.283285517302)*x[0]+(-0.525102967397)*x[1]**o+(0.517875061987)*x[1]
        s[2,3,2,0]=(-0.393961839836)*x[0]**o+(0.594690475758)*x[0]+(0.165742336314)*x[1]**o+(0.655933809047)*x[1]
        s[2,3,2,1]=(-0.911200880858)*x[0]**o+(0.182403795244)*x[0]+(0.489125521474)*x[1]**o+(0.0496056049896)*x[1]
        s[2,4,0,0]=(-0.159105772726)*x[0]**o+(0.625018347934)*x[0]+(-0.0170170509106)*x[1]**o+(0.302248488132)*x[1]
        s[2,4,0,1]=(0.898869898377)*x[0]**o+(-0.172540097096)*x[0]+(0.11416927686)*x[1]**o+(0.465614034585)*x[1]
        s[2,4,1,0]=(-0.72081868999)*x[0]**o+(-0.605232985848)*x[0]+(-0.889107167889)*x[1]**o+(-0.970389096931)*x[1]
        s[2,4,1,1]=(0.566591889778)*x[0]**o+(0.844421879775)*x[0]+(-0.68950638965)*x[1]**o+(-0.555658441668)*x[1]
        s[2,4,2,0]=(-0.677923523128)*x[0]**o+(-0.764792213508)*x[0]+(-0.940720806672)*x[1]**o+(-0.349132395924)*x[1]
        s[2,4,2,1]=(0.511357852175)*x[0]**o+(-0.688298821028)*x[0]+(0.260299341082)*x[1]**o+(-0.172719107959)*x[1]
        s[3,0,0,0]=(0.96250974067)*x[0]**o+(-0.126604059489)*x[0]+(-0.117880197012)*x[1]**o+(-0.914802074674)*x[1]
        s[3,0,0,1]=(0.149007631585)*x[0]**o+(-0.673349488321)*x[0]+(-0.993719500465)*x[1]**o+(-0.598353371432)*x[1]
        s[3,0,1,0]=(-0.808696105441)*x[0]**o+(-0.288375046884)*x[0]+(-0.770109544749)*x[1]**o+(0.830326211908)*x[1]
        s[3,0,1,1]=(-0.156477225552)*x[0]**o+(0.739349262635)*x[0]+(0.364144184522)*x[1]**o+(0.628195257168)*x[1]
        s[3,0,2,0]=(-0.447857061312)*x[0]**o+(0.974956110157)*x[0]+(-0.470610412287)*x[1]**o+(-0.576396903386)*x[1]
        s[3,0,2,1]=(-0.953524211957)*x[0]**o+(0.0172359158991)*x[0]+(-0.587968819301)*x[1]**o+(-0.0404250064791)*x[1]
        s[3,1,0,0]=(-0.438441555817)*x[0]**o+(-0.0863380014049)*x[0]+(-0.651726866234)*x[1]**o+(-0.757302907976)*x[1]
        s[3,1,0,1]=(0.365066315119)*x[0]**o+(0.252165659991)*x[0]+(-0.1682171231)*x[1]**o+(-0.597926603548)*x[1]
        s[3,1,1,0]=(-0.0608091331878)*x[0]**o+(-0.118607423591)*x[0]+(0.572219801453)*x[1]**o+(-0.620635555103)*x[1]
        s[3,1,1,1]=(-0.258140275212)*x[0]**o+(0.106650435513)*x[0]+(-0.739971594229)*x[1]**o+(0.595195330003)*x[1]
        s[3,1,2,0]=(-0.229810808135)*x[0]**o+(-0.891173708591)*x[0]+(0.257932505524)*x[1]**o+(0.561295616991)*x[1]
        s[3,1,2,1]=(-0.579302432005)*x[0]**o+(0.607984452716)*x[0]+(-0.155861940983)*x[1]**o+(0.388572816738)*x[1]
        s[3,2,0,0]=(0.87799491053)*x[0]**o+(0.434743191656)*x[0]+(0.219633220676)*x[1]**o+(-0.515493145923)*x[1]
        s[3,2,0,1]=(-0.855493053888)*x[0]**o+(0.173518924283)*x[0]+(0.0675155754173)*x[1]**o+(-0.700260220492)*x[1]
        s[3,2,1,0]=(-0.412519942294)*x[0]**o+(-0.0114933134972)*x[0]+(0.232846893376)*x[1]**o+(0.272202827241)*x[1]
        s[3,2,1,1]=(0.325402950163)*x[0]**o+(-0.455151259934)*x[0]+(0.88929658634)*x[1]**o+(0.147699225253)*x[1]
        s[3,2,2,0]=(-0.246804489391)*x[0]**o+(0.626746060036)*x[0]+(0.70088034171)*x[1]**o+(-0.492021577395)*x[1]
        s[3,2,2,1]=(0.161987385265)*x[0]**o+(-0.038894607114)*x[0]+(-0.326936847239)*x[1]**o+(0.181111852946)*x[1]
        s[3,3,0,0]=(-0.287388091741)*x[0]**o+(0.516752550941)*x[0]+(-0.941603336187)*x[1]**o+(0.549867040053)*x[1]
        s[3,3,0,1]=(-0.711983817048)*x[0]**o+(-0.257572806886)*x[0]+(-0.441118042147)*x[1]**o+(-0.672544376584)*x[1]
        s[3,3,1,0]=(0.27179221385)*x[0]**o+(0.722679194783)*x[0]+(0.154628962433)*x[1]**o+(0.406406592446)*x[1]
        s[3,3,1,1]=(-0.484359850969)*x[0]**o+(-0.179916115792)*x[0]+(-0.461756438449)*x[1]**o+(-0.484596755972)*x[1]
        s[3,3,2,0]=(0.06709022871)*x[0]**o+(0.79525514856)*x[0]+(0.815544226952)*x[1]**o+(-0.498264422355)*x[1]
        s[3,3,2,1]=(0.843256724396)*x[0]**o+(0.0796294835103)*x[0]+(0.301414364597)*x[1]**o+(0.790478221822)*x[1]
        s[3,4,0,0]=(0.723595119935)*x[0]**o+(-0.757921136678)*x[0]+(-0.823917708581)*x[1]**o+(0.0624785383724)*x[1]
        s[3,4,0,1]=(0.733680579182)*x[0]**o+(-0.383756889549)*x[0]+(0.161828894605)*x[1]**o+(-0.192838756613)*x[1]
        s[3,4,1,0]=(0.147016857318)*x[0]**o+(0.0580828047654)*x[0]+(-0.628782245085)*x[1]**o+(-0.607487083314)*x[1]
        s[3,4,1,1]=(-0.0882094727061)*x[0]**o+(0.790852741583)*x[0]+(0.47412676393)*x[1]**o+(-0.873733941999)*x[1]
        s[3,4,2,0]=(0.0294423769171)*x[0]**o+(0.522861977201)*x[0]+(-0.360682765428)*x[1]**o+(-0.460704794865)*x[1]
        s[3,4,2,1]=(0.143274846749)*x[0]**o+(-0.164306681884)*x[0]+(0.632228849654)*x[1]**o+(-0.488837677105)*x[1]
        ref[0,0,0,0]=(-0.798735149387)/(o+1.)+(-0.0483598869992)+(0.826151835054)*0.5**o
        ref[0,0,0,1]=(-0.559216870357)/(o+1.)+(0.252244220336)+(0.281853758764)*0.5**o
        ref[0,0,1,0]=(0.147096504378)/(o+1.)+(0.369744898679)+(0.734893466243)*0.5**o
        ref[0,0,1,1]=(-0.978653421141)/(o+1.)+(-0.428887699773)+(0.928300107915)*0.5**o
        ref[0,0,2,0]=(0.975786707279)/(o+1.)+(-0.387298554631)+(-0.84104720017)*0.5**o
        ref[0,0,2,1]=(-0.478609919678)/(o+1.)+(0.398937296478)+(0.505216717255)*0.5**o
        ref[0,1,0,0]=(0.0325820539534)/(o+1.)+(-0.157046078324)+(-0.588826769132)*0.5**o
        ref[0,1,0,1]=(0.943593904137)/(o+1.)+(0.806522861734)+(0.201705696091)*0.5**o
        ref[0,1,1,0]=(0.106488644042)/(o+1.)+(0.0768336262816)+(0.458858343943)*0.5**o
        ref[0,1,1,1]=(0.164158781184)/(o+1.)+(0.276472203434)+(-0.445360193781)*0.5**o
        ref[0,1,2,0]=(-0.484354639056)/(o+1.)+(-0.403701810283)+(0.631597276525)*0.5**o
        ref[0,1,2,1]=(-0.209463960361)/(o+1.)+(-0.84997261019)+(-0.742804913722)*0.5**o
        ref[0,2,0,0]=(0.807935626935)/(o+1.)+(0.769370584372)+(-0.757921024916)*0.5**o
        ref[0,2,0,1]=(-0.574033583989)/(o+1.)+(0.0827137233007)+(-0.256981783889)*0.5**o
        ref[0,2,1,0]=(0.341543474661)/(o+1.)+(0.137836461287)+(-0.943218252718)*0.5**o
        ref[0,2,1,1]=(-0.0146665511353)/(o+1.)+(-0.529583474644)+(0.792692625218)*0.5**o
        ref[0,2,2,0]=(0.459127847422)/(o+1.)+(-0.223549756332)+(-0.30799523168)*0.5**o
        ref[0,2,2,1]=(0.548484423475)/(o+1.)+(-0.784551243723)+(-0.610374384606)*0.5**o
        ref[0,3,0,0]=(0.994651694429)/(o+1.)+(0.119721687896)+(0.657511913357)*0.5**o
        ref[0,3,0,1]=(0.944791134312)/(o+1.)+(-0.186607538415)+(0.583248456434)*0.5**o
        ref[0,3,1,0]=(-0.101044667455)/(o+1.)+(0.318853063774)+(-0.781508935766)*0.5**o
        ref[0,3,1,1]=(-0.946297144806)/(o+1.)+(0.873253153974)+(-0.867519386038)*0.5**o
        ref[0,3,2,0]=(0.535177813023)/(o+1.)+(-0.383685984015)+(-0.281309467001)*0.5**o
        ref[0,3,2,1]=(0.291253331071)/(o+1.)+(-0.262269751541)+(-0.860996437102)*0.5**o
        ref[0,4,0,0]=(-0.264697628051)/(o+1.)+(-0.252939683193)+(-0.499079353771)*0.5**o
        ref[0,4,0,1]=(0.436241831047)/(o+1.)+(0.520288673122)+(0.886952704303)*0.5**o
        ref[0,4,1,0]=(0.596844427835)/(o+1.)+(0.0979004955464)+(-0.794113947126)*0.5**o
        ref[0,4,1,1]=(-0.47050747773)/(o+1.)+(-0.197399390257)+(-0.0741729381021)*0.5**o
        ref[0,4,2,0]=(0.570813169088)/(o+1.)+(0.294381404543)+(-0.574730110675)*0.5**o
        ref[0,4,2,1]=(0.156596161146)/(o+1.)+(0.701040644046)+(-0.956657753944)*0.5**o
        ref[1,0,0,0]=(0.889791199852)/(o+1.)+(-0.759429185942)+(-0.951078063957)*0.5**o
        ref[1,0,0,1]=(-0.216836641085)/(o+1.)+(-0.507184158064)+(-0.601689692228)*0.5**o
        ref[1,0,1,0]=(0.98286908038)/(o+1.)+(0.247237802667)+(0.963503254167)*0.5**o
        ref[1,0,1,1]=(-0.784318364616)/(o+1.)+(-0.530266876055)+(-0.220998216482)*0.5**o
        ref[1,0,2,0]=(0.297365047085)/(o+1.)+(0.133575908879)+(-0.0629547087136)*0.5**o
        ref[1,0,2,1]=(0.8150001419)/(o+1.)+(-0.784145206129)+(-0.898579538704)*0.5**o
        ref[1,1,0,0]=(0.455058141159)/(o+1.)+(0.0635321855459)+(-0.0983531790455)*0.5**o
        ref[1,1,0,1]=(-0.660954533474)/(o+1.)+(-0.368562732733)+(0.198516567418)*0.5**o
        ref[1,1,1,0]=(0.389246060633)/(o+1.)+(0.220892947375)+(-0.345947424795)*0.5**o
        ref[1,1,1,1]=(0.206735465226)/(o+1.)+(-0.0683634648758)+(-0.195677951352)*0.5**o
        ref[1,1,2,0]=(0.728613552854)/(o+1.)+(-0.0760294789467)+(0.286674162919)*0.5**o
        ref[1,1,2,1]=(-0.234360929327)/(o+1.)+(0.0189448869689)+(0.462677800728)*0.5**o
        ref[1,2,0,0]=(0.826946704031)/(o+1.)+(-0.035672463308)+(-0.618359459011)*0.5**o
        ref[1,2,0,1]=(-0.610232300686)/(o+1.)+(0.325498254635)+(0.79180929497)*0.5**o
        ref[1,2,1,0]=(0.222616002742)/(o+1.)+(-0.587631866051)+(-0.454630490901)*0.5**o
        ref[1,2,1,1]=(-0.298070302518)/(o+1.)+(0.082314571655)+(-0.358323719668)*0.5**o
        ref[1,2,2,0]=(0.293175442678)/(o+1.)+(0.295307653283)+(-0.953302373475)*0.5**o
        ref[1,2,2,1]=(-0.656552393979)/(o+1.)+(0.0997184459739)+(0.0847099318446)*0.5**o
        ref[1,3,0,0]=(0.0644008346021)/(o+1.)+(-0.558993091122)+(0.0792265074365)*0.5**o
        ref[1,3,0,1]=(-0.712500149358)/(o+1.)+(-0.021354143711)+(-0.312474679341)*0.5**o
        ref[1,3,1,0]=(-0.74265516534)/(o+1.)+(-0.251240774237)+(0.0930078501772)*0.5**o
        ref[1,3,1,1]=(0.0794334270857)/(o+1.)+(0.900356703399)+(0.839924906913)*0.5**o
        ref[1,3,2,0]=(-0.206887854301)/(o+1.)+(-0.0790981124364)+(0.154200394821)*0.5**o
        ref[1,3,2,1]=(-0.499236661621)/(o+1.)+(-0.164823650034)+(0.427091397128)*0.5**o
        ref[1,4,0,0]=(0.991785227798)/(o+1.)+(-0.351432789024)+(0.993473425442)*0.5**o
        ref[1,4,0,1]=(-0.549435231471)/(o+1.)+(-0.142162607695)+(0.905969952921)*0.5**o
        ref[1,4,1,0]=(0.270145165409)/(o+1.)+(0.588359301378)+(-0.414114927962)*0.5**o
        ref[1,4,1,1]=(0.987067570291)/(o+1.)+(-0.831156169992)+(-0.537634564837)*0.5**o
        ref[1,4,2,0]=(0.691510051463)/(o+1.)+(0.500187321401)+(0.298204375484)*0.5**o
        ref[1,4,2,1]=(-0.0920482079502)/(o+1.)+(-0.0906161971883)+(0.924136889038)*0.5**o
        ref[2,0,0,0]=(-0.0496779342723)/(o+1.)+(-0.365952537674)+(-0.623512868124)*0.5**o
        ref[2,0,0,1]=(-0.698331157728)/(o+1.)+(0.267970232756)+(-0.675571390646)*0.5**o
        ref[2,0,1,0]=(0.724413391261)/(o+1.)+(-0.0396960299582)+(-0.137713646762)*0.5**o
        ref[2,0,1,1]=(-0.163758448345)/(o+1.)+(0.394063975354)+(-0.790333229868)*0.5**o
        ref[2,0,2,0]=(0.654866937965)/(o+1.)+(-0.12945082918)+(0.401526106126)*0.5**o
        ref[2,0,2,1]=(-0.351703606836)/(o+1.)+(0.424494277489)+(-0.691391545016)*0.5**o
        ref[2,1,0,0]=(0.933443347736)/(o+1.)+(0.57373453732)+(-0.994782416678)*0.5**o
        ref[2,1,0,1]=(0.0751230852899)/(o+1.)+(0.430458974339)+(0.888393462781)*0.5**o
        ref[2,1,1,0]=(-0.200110899369)/(o+1.)+(-0.215023350034)+(-0.67951464573)*0.5**o
        ref[2,1,1,1]=(-0.231357242392)/(o+1.)+(-0.0723585494832)+(0.769634138219)*0.5**o
        ref[2,1,2,0]=(-0.935858357069)/(o+1.)+(0.601742944468)+(-0.740242586287)*0.5**o
        ref[2,1,2,1]=(0.594009331267)/(o+1.)+(0.0971318042938)+(0.0522567442585)*0.5**o
        ref[2,2,0,0]=(0.336054961442)/(o+1.)+(0.200175727613)+(-0.202135247802)*0.5**o
        ref[2,2,0,1]=(0.728985032074)/(o+1.)+(-0.677018671613)+(-0.18797538396)*0.5**o
        ref[2,2,1,0]=(0.923860896997)/(o+1.)+(-0.385004086158)+(-0.1657521328)*0.5**o
        ref[2,2,1,1]=(0.625778742556)/(o+1.)+(-0.539774445742)+(0.318366946697)*0.5**o
        ref[2,2,2,0]=(-0.428464082609)/(o+1.)+(0.308942152437)+(0.796633973705)*0.5**o
        ref[2,2,2,1]=(0.234909867943)/(o+1.)+(-0.120710700899)+(0.430909811076)*0.5**o
        ref[2,3,0,0]=(-0.415901909163)/(o+1.)+(-0.0220341432872)+(0.0637185370545)*0.5**o
        ref[2,3,0,1]=(-0.847060157535)/(o+1.)+(0.140889117367)+(-0.0514048718664)*0.5**o
        ref[2,3,1,0]=(0.294803721831)/(o+1.)+(0.203408504215)+(0.272384814885)*0.5**o
        ref[2,3,1,1]=(-0.525102967397)/(o+1.)+(0.400580289645)+(-0.171649346026)*0.5**o
        ref[2,3,2,0]=(0.165742336314)/(o+1.)+(0.625312142402)+(-0.393961839836)*0.5**o
        ref[2,3,2,1]=(0.489125521474)/(o+1.)+(0.116004700117)+(-0.911200880858)*0.5**o
        ref[2,4,0,0]=(-0.0170170509106)/(o+1.)+(0.463633418033)+(-0.159105772726)*0.5**o
        ref[2,4,0,1]=(0.11416927686)/(o+1.)+(0.146536968744)+(0.898869898377)*0.5**o
        ref[2,4,1,0]=(-0.889107167889)/(o+1.)+(-0.78781104139)+(-0.72081868999)*0.5**o
        ref[2,4,1,1]=(-0.68950638965)/(o+1.)+(0.144381719054)+(0.566591889778)*0.5**o
        ref[2,4,2,0]=(-0.940720806672)/(o+1.)+(-0.556962304716)+(-0.677923523128)*0.5**o
        ref[2,4,2,1]=(0.260299341082)/(o+1.)+(-0.430508964494)+(0.511357852175)*0.5**o
        ref[3,0,0,0]=(-0.117880197012)/(o+1.)+(-0.520703067081)+(0.96250974067)*0.5**o
        ref[3,0,0,1]=(-0.993719500465)/(o+1.)+(-0.635851429876)+(0.149007631585)*0.5**o
        ref[3,0,1,0]=(-0.770109544749)/(o+1.)+(0.270975582512)+(-0.808696105441)*0.5**o
        ref[3,0,1,1]=(0.364144184522)/(o+1.)+(0.683772259902)+(-0.156477225552)*0.5**o
        ref[3,0,2,0]=(-0.470610412287)/(o+1.)+(0.199279603385)+(-0.447857061312)*0.5**o
        ref[3,0,2,1]=(-0.587968819301)/(o+1.)+(-0.01159454529)+(-0.953524211957)*0.5**o
        ref[3,1,0,0]=(-0.651726866234)/(o+1.)+(-0.42182045469)+(-0.438441555817)*0.5**o
        ref[3,1,0,1]=(-0.1682171231)/(o+1.)+(-0.172880471778)+(0.365066315119)*0.5**o
        ref[3,1,1,0]=(0.572219801453)/(o+1.)+(-0.369621489347)+(-0.0608091331878)*0.5**o
        ref[3,1,1,1]=(-0.739971594229)/(o+1.)+(0.350922882758)+(-0.258140275212)*0.5**o
        ref[3,1,2,0]=(0.257932505524)/(o+1.)+(-0.1649390458)+(-0.229810808135)*0.5**o
        ref[3,1,2,1]=(-0.155861940983)/(o+1.)+(0.498278634727)+(-0.579302432005)*0.5**o
        ref[3,2,0,0]=(0.219633220676)/(o+1.)+(-0.0403749771337)+(0.87799491053)*0.5**o
        ref[3,2,0,1]=(0.0675155754173)/(o+1.)+(-0.263370648105)+(-0.855493053888)*0.5**o
        ref[3,2,1,0]=(0.232846893376)/(o+1.)+(0.130354756872)+(-0.412519942294)*0.5**o
        ref[3,2,1,1]=(0.88929658634)/(o+1.)+(-0.15372601734)+(0.325402950163)*0.5**o
        ref[3,2,2,0]=(0.70088034171)/(o+1.)+(0.0673622413205)+(-0.246804489391)*0.5**o
        ref[3,2,2,1]=(-0.326936847239)/(o+1.)+(0.071108622916)+(0.161987385265)*0.5**o
        ref[3,3,0,0]=(-0.941603336187)/(o+1.)+(0.533309795497)+(-0.287388091741)*0.5**o
        ref[3,3,0,1]=(-0.441118042147)/(o+1.)+(-0.465058591735)+(-0.711983817048)*0.5**o
        ref[3,3,1,0]=(0.154628962433)/(o+1.)+(0.564542893615)+(0.27179221385)*0.5**o
        ref[3,3,1,1]=(-0.461756438449)/(o+1.)+(-0.332256435882)+(-0.484359850969)*0.5**o
        ref[3,3,2,0]=(0.815544226952)/(o+1.)+(0.148495363102)+(0.06709022871)*0.5**o
        ref[3,3,2,1]=(0.301414364597)/(o+1.)+(0.435053852666)+(0.843256724396)*0.5**o
        ref[3,4,0,0]=(-0.823917708581)/(o+1.)+(-0.347721299153)+(0.723595119935)*0.5**o
        ref[3,4,0,1]=(0.161828894605)/(o+1.)+(-0.288297823081)+(0.733680579182)*0.5**o
        ref[3,4,1,0]=(-0.628782245085)/(o+1.)+(-0.274702139274)+(0.147016857318)*0.5**o
        ref[3,4,1,1]=(0.47412676393)/(o+1.)+(-0.0414406002081)+(-0.0882094727061)*0.5**o
        ref[3,4,2,0]=(-0.360682765428)/(o+1.)+(0.0310785911681)+(0.0294423769171)*0.5**o
        ref[3,4,2,1]=(0.632228849654)/(o+1.)+(-0.326572179494)+(0.143274846749)*0.5**o
      else:
        s[0,0,0,0]=(0.356314179645)*x[0]**o+(-0.32357850671)*x[0]+(-0.824656670425)*x[1]**o+(-0.627681417632)*x[1]+(-0.269037662166)*x[2]**o+(-0.506111289647)*x[2]
        s[0,0,0,1]=(0.209589534917)*x[0]**o+(-0.245850334755)*x[0]+(-0.933340495998)*x[1]**o+(-0.091551513676)*x[1]+(-0.903932207206)*x[2]**o+(-0.462176705079)*x[2]
        s[0,0,1,0]=(-0.242847848445)*x[0]**o+(-0.303798875175)*x[0]+(-0.89496979627)*x[1]**o+(-0.442220551)*x[1]+(0.686085734196)*x[2]**o+(-0.422307267905)*x[2]
        s[0,0,1,1]=(0.255545417045)*x[0]**o+(0.0456454977479)*x[0]+(-0.378619350633)*x[1]**o+(0.720981703656)*x[1]+(0.251875164005)*x[2]**o+(-0.220351133411)*x[2]
        s[0,0,2,0]=(-0.962070081459)*x[0]**o+(0.697584448738)*x[0]+(0.855321986438)*x[1]**o+(0.818489831335)*x[1]+(-0.892304050049)*x[2]**o+(0.609699703411)*x[2]
        s[0,0,2,1]=(0.636687005195)*x[0]**o+(-0.4086000867)*x[0]+(-0.0426817477748)*x[1]**o+(-0.842690837453)*x[1]+(-0.704599957019)*x[2]**o+(0.518333776253)*x[2]
        s[0,1,0,0]=(0.521790025796)*x[0]**o+(-0.987699503457)*x[0]+(0.0903316739912)*x[1]**o+(0.307827334512)*x[1]+(-0.401791463687)*x[2]**o+(-0.702368788162)*x[2]
        s[0,1,0,1]=(-0.642544941536)*x[0]**o+(-0.881565606872)*x[0]+(-0.863360075641)*x[1]**o+(0.927565055153)*x[1]+(0.286728291693)*x[2]**o+(0.0652315285371)*x[2]
        s[0,1,1,0]=(0.987946287729)*x[0]**o+(-0.95542434477)*x[0]+(0.876907614088)*x[1]**o+(0.917353337436)*x[1]+(-0.40420250407)*x[2]**o+(-0.984390334421)*x[2]
        s[0,1,1,1]=(-0.848091494031)*x[0]**o+(0.205692942903)*x[0]+(0.804893714353)*x[1]**o+(0.34369751042)*x[1]+(-0.948317185277)*x[2]**o+(-0.937737932921)*x[2]
        s[0,1,2,0]=(-0.411836604088)*x[0]**o+(-0.520147883853)*x[0]+(0.10615721253)*x[1]**o+(0.674921463188)*x[1]+(0.245733286882)*x[2]**o+(-0.653168166924)*x[2]
        s[0,1,2,1]=(0.0397981165279)*x[0]**o+(0.667676059311)*x[0]+(0.720734430282)*x[1]**o+(-0.232075762355)*x[1]+(-0.558842944595)*x[2]**o+(0.39581457339)*x[2]
        s[0,2,0,0]=(-0.043373970003)*x[0]**o+(0.797078438464)*x[0]+(-0.488567358998)*x[1]**o+(0.0955394876958)*x[1]+(0.469780932691)*x[2]**o+(0.665728903694)*x[2]
        s[0,2,0,1]=(0.204098937116)*x[0]**o+(0.16435985408)*x[0]+(-0.245228652593)*x[1]**o+(-0.423628218893)*x[1]+(-0.0691452707964)*x[2]**o+(-0.709704899728)*x[2]
        s[0,2,1,0]=(0.506878702079)*x[0]**o+(0.740560436601)*x[0]+(0.193885598321)*x[1]**o+(0.521852898085)*x[1]+(0.0368080590022)*x[2]**o+(0.986970659964)*x[2]
        s[0,2,1,1]=(-0.261656641804)*x[0]**o+(-0.407521171557)*x[0]+(0.966151887826)*x[1]**o+(-0.918908288322)*x[1]+(-0.370153374393)*x[2]**o+(0.236661704457)*x[2]
        s[0,2,2,0]=(-0.491629682103)*x[0]**o+(-0.593057448837)*x[0]+(-0.022126429921)*x[1]**o+(-0.509356725061)*x[1]+(-0.680259680283)*x[2]**o+(-0.889690558667)*x[2]
        s[0,2,2,1]=(-0.564089634025)*x[0]**o+(0.0574486294037)*x[0]+(0.0578539449865)*x[1]**o+(0.258278489918)*x[1]+(0.174461168842)*x[2]**o+(0.0842548610258)*x[2]
        s[0,3,0,0]=(0.0606363217868)*x[0]**o+(0.160338081583)*x[0]+(-0.0636204375814)*x[1]**o+(0.287489206156)*x[1]+(-0.592019780262)*x[2]**o+(0.508884293667)*x[2]
        s[0,3,0,1]=(-0.702879071079)*x[0]**o+(0.874961299195)*x[0]+(-0.00733354592216)*x[1]**o+(0.0223647905227)*x[1]+(0.956232905909)*x[2]**o+(0.369832655586)*x[2]
        s[0,3,1,0]=(-0.824855043544)*x[0]**o+(-0.529925913352)*x[0]+(-0.0897704543542)*x[1]**o+(0.655124384268)*x[1]+(-0.586221756018)*x[2]**o+(0.104165560368)*x[2]
        s[0,3,1,1]=(-0.890319589507)*x[0]**o+(0.791154767703)*x[0]+(-0.315134766255)*x[1]**o+(-0.384720188852)*x[1]+(0.130920946919)*x[2]**o+(0.894095912396)*x[2]
        s[0,3,2,0]=(0.366123401919)*x[0]**o+(0.806533885372)*x[0]+(-0.579579932468)*x[1]**o+(0.490534769858)*x[1]+(0.250581383716)*x[2]**o+(-0.326970590496)*x[2]
        s[0,3,2,1]=(-0.438035010616)*x[0]**o+(-0.630762254554)*x[0]+(0.0848919367225)*x[1]**o+(-0.872733528687)*x[1]+(-0.465429840653)*x[2]**o+(0.567524409476)*x[2]
        s[0,4,0,0]=(0.319555220189)*x[0]**o+(0.397272010924)*x[0]+(-0.827915669722)*x[1]**o+(-0.0370119836485)*x[1]+(0.232621621392)*x[2]**o+(0.376285172425)*x[2]
        s[0,4,0,1]=(-0.874144853872)*x[0]**o+(0.740754812769)*x[0]+(0.878074282473)*x[1]**o+(-0.168290826137)*x[1]+(-0.0466881676326)*x[2]**o+(0.997360121528)*x[2]
        s[0,4,1,0]=(0.622984208352)*x[0]**o+(-0.60223710711)*x[0]+(-0.684599543294)*x[1]**o+(-0.654758989528)*x[1]+(-0.209806071733)*x[2]**o+(0.0331737697413)*x[2]
        s[0,4,1,1]=(-0.175155841122)*x[0]**o+(-0.73030489546)*x[0]+(0.226809166193)*x[1]**o+(-0.242344848157)*x[1]+(-0.331283539462)*x[2]**o+(-0.392781337615)*x[2]
        s[0,4,2,0]=(0.111634467654)*x[0]**o+(-0.0622591150598)*x[0]+(-0.675485413951)*x[1]**o+(-0.0169689015565)*x[1]+(-0.373014699305)*x[2]**o+(-0.827576200434)*x[2]
        s[0,4,2,1]=(0.769210305419)*x[0]**o+(-0.380741842544)*x[0]+(-0.0136492374016)*x[1]**o+(0.231367701468)*x[1]+(-0.156422804273)*x[2]**o+(0.69522180005)*x[2]
        s[1,0,0,0]=(-0.907821074278)*x[0]**o+(-0.466564046295)*x[0]+(-0.0115096148313)*x[1]**o+(0.682571066758)*x[1]+(-0.188270659989)*x[2]**o+(0.769027925506)*x[2]
        s[1,0,0,1]=(-0.935692950448)*x[0]**o+(0.801782734141)*x[0]+(-0.694482272453)*x[1]**o+(-0.357697785902)*x[1]+(0.604605447034)*x[2]**o+(-0.484488572975)*x[2]
        s[1,0,1,0]=(0.958399972856)*x[0]**o+(0.684229220211)*x[0]+(0.952743494375)*x[1]**o+(0.247944736517)*x[1]+(-0.251674820491)*x[2]**o+(-0.948785335663)*x[2]
        s[1,0,1,1]=(0.480962582731)*x[0]**o+(-0.233075914633)*x[0]+(-0.407820663337)*x[1]**o+(0.74897610034)*x[1]+(0.869138425397)*x[2]**o+(-0.115144267319)*x[2]
        s[1,0,2,0]=(-0.459646484527)*x[0]**o+(-0.42415176628)*x[0]+(0.361277035435)*x[1]**o+(-0.894956929165)*x[1]+(-0.710531250988)*x[2]**o+(-0.0704645482372)*x[2]
        s[1,0,2,1]=(-0.663523285059)*x[0]**o+(0.42693709011)*x[0]+(-0.225209635066)*x[1]**o+(0.505033755104)*x[1]+(-0.612237573846)*x[2]**o+(-0.824405122543)*x[2]
        s[1,1,0,0]=(-0.963081878353)*x[0]**o+(-0.318420605399)*x[0]+(-0.841731614457)*x[1]**o+(0.0524113793424)*x[1]+(0.824353304271)*x[2]**o+(-0.347662639363)*x[2]
        s[1,1,0,1]=(-0.426340341727)*x[0]**o+(-0.621345618708)*x[0]+(-0.599191472774)*x[1]**o+(0.720942014806)*x[1]+(0.699997023602)*x[2]**o+(-0.0401845986151)*x[2]
        s[1,1,1,0]=(0.195081954019)*x[0]**o+(0.404513751671)*x[0]+(-0.691733548084)*x[1]**o+(0.494301087961)*x[1]+(0.660194160561)*x[2]**o+(-0.719750815548)*x[2]
        s[1,1,1,1]=(0.174243105183)*x[0]**o+(-0.490648510315)*x[0]+(-0.921695831048)*x[1]**o+(0.03988622749)*x[1]+(0.600722960619)*x[2]**o+(-0.190758484114)*x[2]
        s[1,1,2,0]=(0.16246409321)*x[0]**o+(-0.861115959612)*x[0]+(-0.813061871575)*x[1]**o+(0.335664223473)*x[1]+(0.0462405018327)*x[2]**o+(-0.187262892401)*x[2]
        s[1,1,2,1]=(0.0961690353732)*x[0]**o+(0.388828620097)*x[0]+(0.108950572002)*x[1]**o+(-0.163688546076)*x[1]+(-0.278121101224)*x[2]**o+(0.612009658813)*x[2]
        s[1,2,0,0]=(-0.641539480622)*x[0]**o+(-0.459587156004)*x[0]+(0.672390673512)*x[1]**o+(-0.306916192961)*x[1]+(-0.350373910022)*x[2]**o+(0.846575285764)*x[2]
        s[1,2,0,1]=(0.305867854526)*x[0]**o+(0.494447320742)*x[0]+(0.325369680551)*x[1]**o+(-0.698268159382)*x[1]+(0.88031161562)*x[2]**o+(0.049240112332)*x[2]
        s[1,2,1,0]=(0.0262323517248)*x[0]**o+(0.165055680035)*x[0]+(0.978941268257)*x[1]**o+(-0.166940249199)*x[1]+(0.169948801883)*x[2]**o+(0.521926242685)*x[2]
        s[1,2,1,1]=(-0.73494826134)*x[0]**o+(-0.908473852515)*x[0]+(0.19661832779)*x[1]**o+(-0.0112732740593)*x[1]+(0.931924432628)*x[2]**o+(-0.433369394703)*x[2]
        s[1,2,2,0]=(-0.405925175898)*x[0]**o+(0.190346680156)*x[0]+(-0.486067244548)*x[1]**o+(-0.745250360184)*x[1]+(0.367855074214)*x[2]**o+(-0.489383863963)*x[2]
        s[1,2,2,1]=(-0.552448017406)*x[0]**o+(-0.10269646845)*x[0]+(0.216638861349)*x[1]**o+(0.285051823646)*x[1]+(-0.230326588392)*x[2]**o+(-0.963919984207)*x[2]
        s[1,3,0,0]=(0.498736551815)*x[0]**o+(0.714104256637)*x[0]+(-0.919467236631)*x[1]**o+(0.435774954621)*x[1]+(0.984691072998)*x[2]**o+(-0.385931573529)*x[2]
        s[1,3,0,1]=(-0.570322022545)*x[0]**o+(-0.431865640646)*x[0]+(-0.670344645547)*x[1]**o+(-0.0350120918436)*x[1]+(-0.574585723354)*x[2]**o+(-0.223611830882)*x[2]
        s[1,3,1,0]=(-0.978507788576)*x[0]**o+(0.692133143224)*x[0]+(-0.914092231492)*x[1]**o+(-0.878609687469)*x[1]+(0.123811943713)*x[2]**o+(0.566089817653)*x[2]
        s[1,3,1,1]=(0.413054411011)*x[0]**o+(0.134306111827)*x[0]+(0.0881588523756)*x[1]**o+(-0.260874928976)*x[1]+(0.59908530007)*x[2]**o+(-0.790312441625)*x[2]
        s[1,3,2,0]=(-0.56020772952)*x[0]**o+(-0.821899156728)*x[0]+(0.657107279954)*x[1]**o+(0.714198844049)*x[1]+(-0.797822774901)*x[2]**o+(-0.683075322516)*x[2]
        s[1,3,2,1]=(-0.784712199154)*x[0]**o+(0.257785308806)*x[0]+(0.668249127767)*x[1]**o+(-0.175010396143)*x[1]+(-0.450998789628)*x[2]**o+(-0.496319680914)*x[2]
        s[1,4,0,0]=(-0.361383889488)*x[0]**o+(-0.507929033088)*x[0]+(0.326087065339)*x[1]**o+(-0.672507245637)*x[1]+(-0.933796910616)*x[2]**o+(-0.964546616709)*x[2]
        s[1,4,0,1]=(-0.359553116054)*x[0]**o+(0.0378595816875)*x[0]+(0.866729184315)*x[1]**o+(-0.109325131607)*x[1]+(-0.436395616824)*x[2]**o+(-0.0250686006967)*x[2]
        s[1,4,1,0]=(-0.822256610634)*x[0]**o+(0.717956881162)*x[0]+(0.325671939606)*x[1]**o+(0.678862239184)*x[1]+(-0.721071141214)*x[2]**o+(0.371297227578)*x[2]
        s[1,4,1,1]=(-0.193864272459)*x[0]**o+(-0.165902167768)*x[0]+(-0.0799495476929)*x[1]**o+(-0.205949634256)*x[1]+(-0.830680911165)*x[2]**o+(-0.152405450656)*x[2]
        s[1,4,2,0]=(0.613697794358)*x[0]**o+(0.912521661263)*x[0]+(-0.00649065024755)*x[1]**o+(-0.215763322626)*x[1]+(-0.263693770435)*x[2]**o+(-0.596480965279)*x[2]
        s[1,4,2,1]=(0.234126882167)*x[0]**o+(0.912840520614)*x[0]+(0.933840609517)*x[1]**o+(0.713211976776)*x[1]+(0.936197279948)*x[2]**o+(-0.293174979732)*x[2]
        s[2,0,0,0]=(0.892309367374)*x[0]**o+(0.633921503439)*x[0]+(0.349595598938)*x[1]**o+(0.461978252492)*x[1]+(-0.711515661832)*x[2]**o+(0.0488550856917)*x[2]
        s[2,0,0,1]=(-0.180492557153)*x[0]**o+(0.56511183613)*x[0]+(-0.665614227144)*x[1]**o+(-0.27485018599)*x[1]+(0.882474739326)*x[2]**o+(0.55427047278)*x[2]
        s[2,0,1,0]=(-0.757743026711)*x[0]**o+(0.940849781783)*x[0]+(-0.269186132914)*x[1]**o+(0.955046584823)*x[1]+(0.378760429217)*x[2]**o+(-0.496548644452)*x[2]
        s[2,0,1,1]=(-0.793137192102)*x[0]**o+(-0.86866411939)*x[0]+(-0.132179169569)*x[1]**o+(0.809599174407)*x[1]+(-0.106002493623)*x[2]**o+(0.109415507767)*x[2]
        s[2,0,2,0]=(0.0272436873719)*x[0]**o+(0.882059697054)*x[0]+(0.195295016692)*x[1]**o+(-0.569734997529)*x[1]+(-0.211564162771)*x[2]**o+(-0.372926144524)*x[2]
        s[2,0,2,1]=(-0.578202088804)*x[0]**o+(-0.466997144939)*x[0]+(0.113612650512)*x[1]**o+(-0.364257850046)*x[1]+(0.260193611289)*x[2]**o+(0.084672789632)*x[2]
        s[2,1,0,0]=(-0.768791448871)*x[0]**o+(-0.0283052260408)*x[0]+(-0.424122141665)*x[1]**o+(-0.568176781664)*x[1]+(0.57776948712)*x[2]**o+(0.974311520873)*x[2]
        s[2,1,0,1]=(-0.868527274026)*x[0]**o+(0.898580042728)*x[0]+(0.293187514185)*x[1]**o+(-0.966501893286)*x[1]+(-0.984906818414)*x[2]**o+(0.385872938659)*x[2]
        s[2,1,1,0]=(0.912855062225)*x[0]**o+(-0.153613990867)*x[0]+(-0.498660070196)*x[1]**o+(0.324379040976)*x[1]+(0.595621093076)*x[2]**o+(0.952723749417)*x[2]
        s[2,1,1,1]=(-0.332918724678)*x[0]**o+(-0.134970034861)*x[0]+(0.0260338270417)*x[1]**o+(0.19534841947)*x[1]+(-0.192624928148)*x[2]**o+(0.351094519153)*x[2]
        s[2,1,2,0]=(-0.130412878953)*x[0]**o+(0.31027939409)*x[0]+(-0.475936742043)*x[1]**o+(-0.304153006832)*x[1]+(-0.293242151796)*x[2]**o+(0.305389900042)*x[2]
        s[2,1,2,1]=(0.701755042286)*x[0]**o+(-0.141745871059)*x[0]+(-0.8205828496)*x[1]**o+(-0.198914354743)*x[1]+(0.785036948065)*x[2]**o+(0.938605909098)*x[2]
        s[2,2,0,0]=(0.551929105975)*x[0]**o+(-0.307038421321)*x[0]+(-0.599015821782)*x[1]**o+(0.0392829428763)*x[1]+(0.000979795616653)*x[2]**o+(-0.674643683481)*x[2]
        s[2,2,0,1]=(0.620978846723)*x[0]**o+(0.440011791598)*x[0]+(0.810951443299)*x[1]**o+(0.99070790399)*x[1]+(0.287374917435)*x[2]**o+(-0.336581470165)*x[2]
        s[2,2,1,0]=(0.772920441549)*x[0]**o+(-0.425704048392)*x[0]+(0.186213730232)*x[1]**o+(-0.69328085038)*x[1]+(-0.496606146226)*x[2]**o+(-0.344844378531)*x[2]
        s[2,2,1,1]=(-0.278910103316)*x[0]**o+(-0.674237998285)*x[0]+(0.83739091668)*x[1]**o+(-0.675256965617)*x[1]+(-0.0220815626466)*x[2]**o+(-0.0600167098977)*x[2]
        s[2,2,2,0]=(0.161601546272)*x[0]**o+(-0.51703043722)*x[0]+(0.192844187688)*x[1]**o+(-0.588393830312)*x[1]+(-0.496441908334)*x[2]**o+(-0.655211307185)*x[2]
        s[2,2,2,1]=(-0.0849641142284)*x[0]**o+(-0.354892880977)*x[0]+(0.990421698189)*x[1]**o+(-0.560596614092)*x[1]+(0.341655795654)*x[2]**o+(0.494914875774)*x[2]
        s[2,3,0,0]=(0.0118611095917)*x[0]**o+(-0.379145547458)*x[0]+(0.443702691226)*x[1]**o+(-0.162411128493)*x[1]+(-0.235962251228)*x[2]**o+(0.647389691482)*x[2]
        s[2,3,0,1]=(0.876617418633)*x[0]**o+(-0.212420835108)*x[0]+(0.931783450145)*x[1]**o+(0.618767260975)*x[1]+(0.865648987862)*x[2]**o+(0.274171020917)*x[2]
        s[2,3,1,0]=(0.0490082163721)*x[0]**o+(-0.719949810351)*x[0]+(0.623252286197)*x[1]**o+(0.797654854976)*x[1]+(-0.688042858141)*x[2]**o+(-0.832375529422)*x[2]
        s[2,3,1,1]=(-0.219830798217)*x[0]**o+(0.404707913477)*x[0]+(0.586989479036)*x[1]**o+(-0.189149824721)*x[1]+(0.439899446736)*x[2]**o+(0.830665884353)*x[2]
        s[2,3,2,0]=(0.176667646979)*x[0]**o+(0.251389955823)*x[0]+(-0.589715137652)*x[1]**o+(0.454775590379)*x[1]+(-0.931971045729)*x[2]**o+(0.688896927587)*x[2]
        s[2,3,2,1]=(0.990217880611)*x[0]**o+(0.813900249046)*x[0]+(0.223918634272)*x[1]**o+(-0.52004580243)*x[1]+(-0.0715278690435)*x[2]**o+(-0.904847118013)*x[2]
        s[2,4,0,0]=(0.64966558184)*x[0]**o+(0.368126307546)*x[0]+(-0.464732694831)*x[1]**o+(0.411720375551)*x[1]+(0.228411933233)*x[2]**o+(0.241400675638)*x[2]
        s[2,4,0,1]=(-0.549913707284)*x[0]**o+(-0.805046099696)*x[0]+(0.0672499044192)*x[1]**o+(-0.443313070041)*x[1]+(0.724365127624)*x[2]**o+(0.796281599759)*x[2]
        s[2,4,1,0]=(-0.849777254443)*x[0]**o+(-0.856417954364)*x[0]+(-0.972865529342)*x[1]**o+(-0.928514101686)*x[1]+(-0.913751924661)*x[2]**o+(-0.988656521899)*x[2]
        s[2,4,1,1]=(-0.911477188787)*x[0]**o+(-0.779986760795)*x[0]+(-0.5389591554)*x[1]**o+(-0.510754176334)*x[1]+(0.897140016867)*x[2]**o+(-0.693629592672)*x[2]
        s[2,4,2,0]=(-0.714385212743)*x[0]**o+(0.335762026267)*x[0]+(-0.402732468944)*x[1]**o+(0.928476377932)*x[1]+(-0.0142181788391)*x[2]**o+(-0.878381714054)*x[2]
        s[2,4,2,1]=(-0.67410120519)*x[0]**o+(-0.958305251297)*x[0]+(-0.380909627066)*x[1]**o+(0.211866670591)*x[1]+(-0.366711818102)*x[2]**o+(0.0984085882283)*x[2]
        s[3,0,0,0]=(0.155086211904)*x[0]**o+(-0.754156385746)*x[0]+(0.0473073793112)*x[1]**o+(0.219578851828)*x[1]+(-0.687387954807)*x[2]**o+(-0.0446886834524)*x[2]
        s[3,0,0,1]=(-0.123628783768)*x[0]**o+(-0.731797394112)*x[0]+(0.603667790885)*x[1]**o+(0.668012714955)*x[1]+(0.992297575688)*x[2]**o+(0.710188099523)*x[2]
        s[3,0,1,0]=(0.142848824593)*x[0]**o+(-0.573105206297)*x[0]+(-0.458611102127)*x[1]**o+(-0.513741139974)*x[1]+(0.184040207705)*x[2]**o+(-0.381655193731)*x[2]
        s[3,0,1,1]=(0.441456662454)*x[0]**o+(0.743195285557)*x[0]+(-0.636813637657)*x[1]**o+(0.579742356597)*x[1]+(-0.100949836318)*x[2]**o+(-0.180042447693)*x[2]
        s[3,0,2,0]=(0.565324561738)*x[0]**o+(-0.734887633464)*x[0]+(-0.70603014663)*x[1]**o+(0.482330325634)*x[1]+(0.0468851802149)*x[2]**o+(0.15022059502)*x[2]
        s[3,0,2,1]=(0.209254635781)*x[0]**o+(0.249109731308)*x[0]+(0.0674776536717)*x[1]**o+(-0.514230929025)*x[1]+(0.209025695623)*x[2]**o+(0.828692252433)*x[2]
        s[3,1,0,0]=(-0.0724637124382)*x[0]**o+(-0.777393915285)*x[0]+(-0.388956221906)*x[1]**o+(0.237031963641)*x[1]+(0.695616762278)*x[2]**o+(-0.990904158717)*x[2]
        s[3,1,0,1]=(0.682394453758)*x[0]**o+(0.971411939071)*x[0]+(0.578523591228)*x[1]**o+(-0.999316997575)*x[1]+(0.967221232651)*x[2]**o+(0.967839631913)*x[2]
        s[3,1,1,0]=(0.790017081833)*x[0]**o+(0.00541365718201)*x[0]+(0.326051127661)*x[1]**o+(-0.0256095948145)*x[1]+(0.723206050948)*x[2]**o+(0.859401412085)*x[2]
        s[3,1,1,1]=(0.632718697794)*x[0]**o+(0.357404127028)*x[0]+(0.74859722663)*x[1]**o+(-0.467286199965)*x[1]+(-0.689766250946)*x[2]**o+(0.143607079021)*x[2]
        s[3,1,2,0]=(0.189165561366)*x[0]**o+(-0.569471633508)*x[0]+(0.827816159248)*x[1]**o+(-0.755414119587)*x[1]+(0.735612525192)*x[2]**o+(0.960576961762)*x[2]
        s[3,1,2,1]=(-0.591990093184)*x[0]**o+(-0.397066100484)*x[0]+(-0.983328257627)*x[1]**o+(0.38188756521)*x[1]+(0.342986157929)*x[2]**o+(-0.654635950498)*x[2]
        s[3,2,0,0]=(-0.539663780822)*x[0]**o+(-0.616950865853)*x[0]+(-0.173236358465)*x[1]**o+(-0.697589613553)*x[1]+(0.881000305923)*x[2]**o+(-0.313131191688)*x[2]
        s[3,2,0,1]=(0.694379614088)*x[0]**o+(0.0482437007544)*x[0]+(-0.752154726269)*x[1]**o+(0.70487516283)*x[1]+(-0.679034313009)*x[2]**o+(-0.759378421017)*x[2]
        s[3,2,1,0]=(0.110285075533)*x[0]**o+(-0.937673509278)*x[0]+(0.588965619655)*x[1]**o+(-0.296106521794)*x[1]+(-0.251861128346)*x[2]**o+(0.0499356188571)*x[2]
        s[3,2,1,1]=(-0.237031027497)*x[0]**o+(-0.555130292547)*x[0]+(0.0266496695782)*x[1]**o+(-0.457831015699)*x[1]+(-0.525100370138)*x[2]**o+(-0.110106658247)*x[2]
        s[3,2,2,0]=(-0.346203985282)*x[0]**o+(0.452969593191)*x[0]+(0.366653289186)*x[1]**o+(-0.431403207447)*x[1]+(0.00416181323958)*x[2]**o+(0.948257819905)*x[2]
        s[3,2,2,1]=(-0.161866645182)*x[0]**o+(-0.223171463142)*x[0]+(0.0335615555403)*x[1]**o+(-0.031701739926)*x[1]+(-0.682660238537)*x[2]**o+(-0.201265341974)*x[2]
        s[3,3,0,0]=(0.600997603354)*x[0]**o+(-0.0794128467746)*x[0]+(0.850816035448)*x[1]**o+(0.987737939576)*x[1]+(0.445092093996)*x[2]**o+(0.209682549773)*x[2]
        s[3,3,0,1]=(-0.281229513599)*x[0]**o+(-0.671047123259)*x[0]+(0.727339757353)*x[1]**o+(0.00116796626742)*x[1]+(0.750903575671)*x[2]**o+(0.688698278099)*x[2]
        s[3,3,1,0]=(0.482788258833)*x[0]**o+(-0.763378332747)*x[0]+(0.958666605693)*x[1]**o+(0.815260251405)*x[1]+(-0.185679132639)*x[2]**o+(-0.978181554387)*x[2]
        s[3,3,1,1]=(0.422342206704)*x[0]**o+(0.92124205703)*x[0]+(0.717119813424)*x[1]**o+(0.358838278821)*x[1]+(-0.905962064034)*x[2]**o+(-0.110063748765)*x[2]
        s[3,3,2,0]=(0.638413147984)*x[0]**o+(0.150065454883)*x[0]+(-0.185782870659)*x[1]**o+(-0.617305598909)*x[1]+(-0.744983914507)*x[2]**o+(-0.557435768794)*x[2]
        s[3,3,2,1]=(-0.373080897783)*x[0]**o+(-0.11883999233)*x[0]+(0.918434263505)*x[1]**o+(0.190498804258)*x[1]+(0.145703720162)*x[2]**o+(0.760937920293)*x[2]
        s[3,4,0,0]=(-0.655458683507)*x[0]**o+(0.181703803374)*x[0]+(0.399180648629)*x[1]**o+(-0.252039523462)*x[1]+(-0.304219626464)*x[2]**o+(0.146502412947)*x[2]
        s[3,4,0,1]=(-0.967498004243)*x[0]**o+(-0.720977530602)*x[0]+(-0.197853229934)*x[1]**o+(-0.206241841259)*x[1]+(-0.842800053665)*x[2]**o+(0.248622158299)*x[2]
        s[3,4,1,0]=(-0.20852177925)*x[0]**o+(0.451565234256)*x[0]+(-0.195228349745)*x[1]**o+(-0.176510706654)*x[1]+(-0.446406234444)*x[2]**o+(-0.391189367221)*x[2]
        s[3,4,1,1]=(-0.0245191385208)*x[0]**o+(-0.4318092516)*x[0]+(0.434565500298)*x[1]**o+(-0.226869925672)*x[1]+(-0.541039601648)*x[2]**o+(-0.661612910117)*x[2]
        s[3,4,2,0]=(0.66840921186)*x[0]**o+(-0.50096681529)*x[0]+(0.584113673029)*x[1]**o+(0.624256279715)*x[1]+(-0.97424822131)*x[2]**o+(-0.436152188081)*x[2]
        s[3,4,2,1]=(-0.524152888193)*x[0]**o+(-0.631030243012)*x[0]+(0.712790078556)*x[1]**o+(0.444287995295)*x[1]+(-0.32562141887)*x[2]**o+(0.999485979189)*x[2]
        ref[0,0,0,0]=(-1.09369433259)/(o+1.)+(-0.728685606995)+(0.356314179645)*0.5**o
        ref[0,0,0,1]=(-1.8372727032)/(o+1.)+(-0.399789276755)+(0.209589534917)*0.5**o
        ref[0,0,1,0]=(-0.208884062074)/(o+1.)+(-0.58416334704)+(-0.242847848445)*0.5**o
        ref[0,0,1,1]=(-0.126744186628)/(o+1.)+(0.273138033997)+(0.255545417045)*0.5**o
        ref[0,0,2,0]=(-0.0369820636107)/(o+1.)+(1.06288699174)+(-0.962070081459)*0.5**o
        ref[0,0,2,1]=(-0.747281704793)/(o+1.)+(-0.36647857395)+(0.636687005195)*0.5**o
        ref[0,1,0,0]=(-0.311459789696)/(o+1.)+(-0.691120478554)+(0.521790025796)*0.5**o
        ref[0,1,0,1]=(-0.576631783947)/(o+1.)+(0.0556154884091)+(-0.642544941536)*0.5**o
        ref[0,1,1,0]=(0.472705110018)/(o+1.)+(-0.511230670877)+(0.987946287729)*0.5**o
        ref[0,1,1,1]=(-0.143423470924)/(o+1.)+(-0.194173739799)+(-0.848091494031)*0.5**o
        ref[0,1,2,0]=(0.351890499412)/(o+1.)+(-0.249197293795)+(-0.411836604088)*0.5**o
        ref[0,1,2,1]=(0.161891485687)/(o+1.)+(0.415707435173)+(0.0397981165279)*0.5**o
        ref[0,2,0,0]=(-0.0187864263063)/(o+1.)+(0.779173414927)+(-0.043373970003)*0.5**o
        ref[0,2,0,1]=(-0.31437392339)/(o+1.)+(-0.48448663227)+(0.204098937116)*0.5**o
        ref[0,2,1,0]=(0.230693657323)/(o+1.)+(1.12469199732)+(0.506878702079)*0.5**o
        ref[0,2,1,1]=(0.595998513433)/(o+1.)+(-0.544883877711)+(-0.261656641804)*0.5**o
        ref[0,2,2,0]=(-0.702386110204)/(o+1.)+(-0.996052366283)+(-0.491629682103)*0.5**o
        ref[0,2,2,1]=(0.232315113828)/(o+1.)+(0.199990990174)+(-0.564089634025)*0.5**o
        ref[0,3,0,0]=(-0.655640217843)/(o+1.)+(0.478355790703)+(0.0606363217868)*0.5**o
        ref[0,3,0,1]=(0.948899359986)/(o+1.)+(0.633579372651)+(-0.702879071079)*0.5**o
        ref[0,3,1,0]=(-0.675992210372)/(o+1.)+(0.114682015642)+(-0.824855043544)*0.5**o
        ref[0,3,1,1]=(-0.184213819336)/(o+1.)+(0.650265245624)+(-0.890319589507)*0.5**o
        ref[0,3,2,0]=(-0.328998548752)/(o+1.)+(0.485049032367)+(0.366123401919)*0.5**o
        ref[0,3,2,1]=(-0.380537903931)/(o+1.)+(-0.467985686882)+(-0.438035010616)*0.5**o
        ref[0,4,0,0]=(-0.59529404833)/(o+1.)+(0.36827259985)+(0.319555220189)*0.5**o
        ref[0,4,0,1]=(0.83138611484)/(o+1.)+(0.78491205408)+(-0.874144853872)*0.5**o
        ref[0,4,1,0]=(-0.894405615027)/(o+1.)+(-0.611911163449)+(0.622984208352)*0.5**o
        ref[0,4,1,1]=(-0.104474373269)/(o+1.)+(-0.682715540615)+(-0.175155841122)*0.5**o
        ref[0,4,2,0]=(-1.04850011326)/(o+1.)+(-0.453402108525)+(0.111634467654)*0.5**o
        ref[0,4,2,1]=(-0.170072041674)/(o+1.)+(0.272923829487)+(0.769210305419)*0.5**o
        ref[1,0,0,0]=(-0.19978027482)/(o+1.)+(0.492517472984)+(-0.907821074278)*0.5**o
        ref[1,0,0,1]=(-0.0898768254184)/(o+1.)+(-0.0202018123679)+(-0.935692950448)*0.5**o
        ref[1,0,1,0]=(0.701068673884)/(o+1.)+(-0.00830568946752)+(0.958399972856)*0.5**o
        ref[1,0,1,1]=(0.46131776206)/(o+1.)+(0.200377959194)+(0.480962582731)*0.5**o
        ref[1,0,2,0]=(-0.349254215553)/(o+1.)+(-0.694786621841)+(-0.459646484527)*0.5**o
        ref[1,0,2,1]=(-0.837447208911)/(o+1.)+(0.0537828613354)+(-0.663523285059)*0.5**o
        ref[1,1,0,0]=(-0.0173783101856)/(o+1.)+(-0.30683593271)+(-0.963081878353)*0.5**o
        ref[1,1,0,1]=(0.100805550828)/(o+1.)+(0.0297058987412)+(-0.426340341727)*0.5**o
        ref[1,1,1,0]=(-0.0315393875231)/(o+1.)+(0.0895320120418)+(0.195081954019)*0.5**o
        ref[1,1,1,1]=(-0.320972870429)/(o+1.)+(-0.320760383469)+(0.174243105183)*0.5**o
        ref[1,1,2,0]=(-0.766821369743)/(o+1.)+(-0.35635731427)+(0.16246409321)*0.5**o
        ref[1,1,2,1]=(-0.169170529222)/(o+1.)+(0.418574866417)+(0.0961690353732)*0.5**o
        ref[1,2,0,0]=(0.32201676349)/(o+1.)+(0.0400359683992)+(-0.641539480622)*0.5**o
        ref[1,2,0,1]=(1.20568129617)/(o+1.)+(-0.0772903631539)+(0.305867854526)*0.5**o
        ref[1,2,1,0]=(1.14889007014)/(o+1.)+(0.26002083676)+(0.0262323517248)*0.5**o
        ref[1,2,1,1]=(1.12854276042)/(o+1.)+(-0.676558260639)+(-0.73494826134)*0.5**o
        ref[1,2,2,0]=(-0.118212170334)/(o+1.)+(-0.522143771996)+(-0.405925175898)*0.5**o
        ref[1,2,2,1]=(-0.013687727043)/(o+1.)+(-0.390782314506)+(-0.552448017406)*0.5**o
        ref[1,3,0,0]=(0.0652238363667)/(o+1.)+(0.381973818864)+(0.498736551815)*0.5**o
        ref[1,3,0,1]=(-1.2449303689)/(o+1.)+(-0.345244781686)+(-0.570322022545)*0.5**o
        ref[1,3,1,0]=(-0.790280287779)/(o+1.)+(0.189806636704)+(-0.978507788576)*0.5**o
        ref[1,3,1,1]=(0.687244152446)/(o+1.)+(-0.458440629387)+(0.413054411011)*0.5**o
        ref[1,3,2,0]=(-0.140715494947)/(o+1.)+(-0.395387817597)+(-0.56020772952)*0.5**o
        ref[1,3,2,1]=(0.217250338139)/(o+1.)+(-0.206772384125)+(-0.784712199154)*0.5**o
        ref[1,4,0,0]=(-0.607709845277)/(o+1.)+(-1.07249144772)+(-0.361383889488)*0.5**o
        ref[1,4,0,1]=(0.430333567491)/(o+1.)+(-0.0482670753081)+(-0.359553116054)*0.5**o
        ref[1,4,1,0]=(-0.395399201609)/(o+1.)+(0.884058173962)+(-0.822256610634)*0.5**o
        ref[1,4,1,1]=(-0.910630458857)/(o+1.)+(-0.26212862634)+(-0.193864272459)*0.5**o
        ref[1,4,2,0]=(-0.270184420683)/(o+1.)+(0.0501386866791)+(0.613697794358)*0.5**o
        ref[1,4,2,1]=(1.87003788947)/(o+1.)+(0.666438758829)+(0.234126882167)*0.5**o
        ref[2,0,0,0]=(-0.361920062895)/(o+1.)+(0.572377420811)+(0.892309367374)*0.5**o
        ref[2,0,0,1]=(0.216860512182)/(o+1.)+(0.42226606146)+(-0.180492557153)*0.5**o
        ref[2,0,1,0]=(0.109574296303)/(o+1.)+(0.699673861077)+(-0.757743026711)*0.5**o
        ref[2,0,1,1]=(-0.238181663192)/(o+1.)+(0.0251752813916)+(-0.793137192102)*0.5**o
        ref[2,0,2,0]=(-0.0162691460785)/(o+1.)+(-0.0303007224997)+(0.0272436873719)*0.5**o
        ref[2,0,2,1]=(0.373806261801)/(o+1.)+(-0.373291102676)+(-0.578202088804)*0.5**o
        ref[2,1,0,0]=(0.153647345455)/(o+1.)+(0.188914756584)+(-0.768791448871)*0.5**o
        ref[2,1,0,1]=(-0.691719304229)/(o+1.)+(0.158975544051)+(-0.868527274026)*0.5**o
        ref[2,1,1,0]=(0.0969610228798)/(o+1.)+(0.561744399763)+(0.912855062225)*0.5**o
        ref[2,1,1,1]=(-0.166591101107)/(o+1.)+(0.205736451881)+(-0.332918724678)*0.5**o
        ref[2,1,2,0]=(-0.769178893839)/(o+1.)+(0.15575814365)+(-0.130412878953)*0.5**o
        ref[2,1,2,1]=(-0.0355459015346)/(o+1.)+(0.298972841648)+(0.701755042286)*0.5**o
        ref[2,2,0,0]=(-0.598036026165)/(o+1.)+(-0.471199580962)+(0.551929105975)*0.5**o
        ref[2,2,0,1]=(1.09832636073)/(o+1.)+(0.547069112712)+(0.620978846723)*0.5**o
        ref[2,2,1,0]=(-0.310392415994)/(o+1.)+(-0.731914638652)+(0.772920441549)*0.5**o
        ref[2,2,1,1]=(0.815309354033)/(o+1.)+(-0.7047558369)+(-0.278910103316)*0.5**o
        ref[2,2,2,0]=(-0.303597720645)/(o+1.)+(-0.880317787358)+(0.161601546272)*0.5**o
        ref[2,2,2,1]=(1.33207749384)/(o+1.)+(-0.210287309648)+(-0.0849641142284)*0.5**o
        ref[2,3,0,0]=(0.207740439998)/(o+1.)+(0.0529165077655)+(0.0118611095917)*0.5**o
        ref[2,3,0,1]=(1.79743243801)/(o+1.)+(0.340258723392)+(0.876617418633)*0.5**o
        ref[2,3,1,0]=(-0.0647905719439)/(o+1.)+(-0.377335242398)+(0.0490082163721)*0.5**o
        ref[2,3,1,1]=(1.02688892577)/(o+1.)+(0.523111986555)+(-0.219830798217)*0.5**o
        ref[2,3,2,0]=(-1.52168618338)/(o+1.)+(0.697531236895)+(0.176667646979)*0.5**o
        ref[2,3,2,1]=(0.152390765229)/(o+1.)+(-0.305496335698)+(0.990217880611)*0.5**o
        ref[2,4,0,0]=(-0.236320761598)/(o+1.)+(0.510623679368)+(0.64966558184)*0.5**o
        ref[2,4,0,1]=(0.791615032044)/(o+1.)+(-0.226038784989)+(-0.549913707284)*0.5**o
        ref[2,4,1,0]=(-1.886617454)/(o+1.)+(-1.38679428897)+(-0.849777254443)*0.5**o
        ref[2,4,1,1]=(0.358180861467)/(o+1.)+(-0.9921852649)+(-0.911477188787)*0.5**o
        ref[2,4,2,0]=(-0.416950647783)/(o+1.)+(0.192928345072)+(-0.714385212743)*0.5**o
        ref[2,4,2,1]=(-0.747621445168)/(o+1.)+(-0.324014996239)+(-0.67410120519)*0.5**o
        ref[3,0,0,0]=(-0.640080575496)/(o+1.)+(-0.289633108685)+(0.155086211904)*0.5**o
        ref[3,0,0,1]=(1.59596536657)/(o+1.)+(0.323201710183)+(-0.123628783768)*0.5**o
        ref[3,0,1,0]=(-0.274570894421)/(o+1.)+(-0.734250770001)+(0.142848824593)*0.5**o
        ref[3,0,1,1]=(-0.737763473975)/(o+1.)+(0.57144759723)+(0.441456662454)*0.5**o
        ref[3,0,2,0]=(-0.659144966415)/(o+1.)+(-0.0511683564048)+(0.565324561738)*0.5**o
        ref[3,0,2,1]=(0.276503349295)/(o+1.)+(0.281785527358)+(0.209254635781)*0.5**o
        ref[3,1,0,0]=(0.306660540372)/(o+1.)+(-0.76563305518)+(-0.0724637124382)*0.5**o
        ref[3,1,0,1]=(1.54574482388)/(o+1.)+(0.469967286704)+(0.682394453758)*0.5**o
        ref[3,1,1,0]=(1.04925717861)/(o+1.)+(0.419602737226)+(0.790017081833)*0.5**o
        ref[3,1,1,1]=(0.0588309756839)/(o+1.)+(0.016862503042)+(0.632718697794)*0.5**o
        ref[3,1,2,0]=(1.56342868444)/(o+1.)+(-0.182154395667)+(0.189165561366)*0.5**o
        ref[3,1,2,1]=(-0.640342099698)/(o+1.)+(-0.334907242886)+(-0.591990093184)*0.5**o
        ref[3,2,0,0]=(0.707763947458)/(o+1.)+(-0.813835835547)+(-0.539663780822)*0.5**o
        ref[3,2,0,1]=(-1.43118903928)/(o+1.)+(-0.00312977871602)+(0.694379614088)*0.5**o
        ref[3,2,1,0]=(0.33710449131)/(o+1.)+(-0.591922206108)+(0.110285075533)*0.5**o
        ref[3,2,1,1]=(-0.49845070056)/(o+1.)+(-0.561533983246)+(-0.237031027497)*0.5**o
        ref[3,2,2,0]=(0.370815102426)/(o+1.)+(0.484912102825)+(-0.346203985282)*0.5**o
        ref[3,2,2,1]=(-0.649098682997)/(o+1.)+(-0.228069272521)+(-0.161866645182)*0.5**o
        ref[3,3,0,0]=(1.29590812944)/(o+1.)+(0.559003821287)+(0.600997603354)*0.5**o
        ref[3,3,0,1]=(1.47824333302)/(o+1.)+(0.00940956055351)+(-0.281229513599)*0.5**o
        ref[3,3,1,0]=(0.772987473054)/(o+1.)+(-0.463149817864)+(0.482788258833)*0.5**o
        ref[3,3,1,1]=(-0.18884225061)/(o+1.)+(0.585008293543)+(0.422342206704)*0.5**o
        ref[3,3,2,0]=(-0.930766785165)/(o+1.)+(-0.51233795641)+(0.638413147984)*0.5**o
        ref[3,3,2,1]=(1.06413798367)/(o+1.)+(0.416298366111)+(-0.373080897783)*0.5**o
        ref[3,4,0,0]=(0.0949610221643)/(o+1.)+(0.0380833464296)+(-0.655458683507)*0.5**o
        ref[3,4,0,1]=(-1.0406532836)/(o+1.)+(-0.339298606781)+(-0.967498004243)*0.5**o
        ref[3,4,1,0]=(-0.641634584189)/(o+1.)+(-0.0580674198095)+(-0.20852177925)*0.5**o
        ref[3,4,1,1]=(-0.10647410135)/(o+1.)+(-0.660146043694)+(-0.0245191385208)*0.5**o
        ref[3,4,2,0]=(-0.390134548281)/(o+1.)+(-0.156431361828)+(0.66840921186)*0.5**o
        ref[3,4,2,1]=(0.387168659686)/(o+1.)+(0.406371865736)+(-0.524152888193)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactOne_fromSymbol_Solution_rank0(self):
      """
      tests integral of rank 0 Symbol on the ReducedFunctionOnContactOne

      assumptions: Solution(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      w=Solution(self.domain)
      x=w.getX()
      arg=Symbol(shape=())
      s=Data(0,(),w)
      if dim==2:
        s=(0.353053645706)*x[0]**o+(0.664927675255)*x[0]+(-0.942226663664)*x[1]**o+(0.928445933833)*x[1]
        ref=(-0.942226663664)/(o+1.)+(0.796686804544)+(0.353053645706)*0.5**o
      else:
        s=(-0.488825085875)*x[0]**o+(0.0402522219678)*x[0]+(-0.741198644498)*x[1]**o+(0.683886232761)*x[1]+(0.988205530376)*x[2]**o+(0.448214857867)*x[2]
        ref=(0.247006885878)/(o+1.)+(0.586176656298)+(-0.488825085875)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactOne_fromSymbol_Solution_rank1(self):
      """
      tests integral of rank 1 Symbol on the ReducedFunctionOnContactOne

      assumptions: Solution(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      w=Solution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(2,))
      s=Data(0,(2,),w)
      ref=numpy.zeros((2,),numpy.float_)
      if dim==2:
        s[0]=(-0.615767854828)*x[0]**o+(-0.986142825206)*x[0]+(-0.129819716269)*x[1]**o+(0.250395618525)*x[1]
        s[1]=(-0.958601580261)*x[0]**o+(0.991062188074)*x[0]+(-0.644161609496)*x[1]**o+(0.50196388747)*x[1]
        ref[0]=(-0.129819716269)/(o+1.)+(-0.367873603341)+(-0.615767854828)*0.5**o
        ref[1]=(-0.644161609496)/(o+1.)+(0.746513037772)+(-0.958601580261)*0.5**o
      else:
        s[0]=(0.878288543713)*x[0]**o+(-0.0231318363536)*x[0]+(0.800114568351)*x[1]**o+(-0.631838147181)*x[1]+(0.747070597667)*x[2]**o+(-0.73748208859)*x[2]
        s[1]=(-0.393659677321)*x[0]**o+(0.219319690151)*x[0]+(0.579665458601)*x[1]**o+(0.158199277855)*x[1]+(0.347253385647)*x[2]**o+(-0.581467925155)*x[2]
        ref[0]=(1.54718516602)/(o+1.)+(-0.696226036062)+(0.878288543713)*0.5**o
        ref[1]=(0.926918844249)/(o+1.)+(-0.101974478575)+(-0.393659677321)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactOne_fromSymbol_Solution_rank2(self):
      """
      tests integral of rank 2 Symbol on the ReducedFunctionOnContactOne

      assumptions: Solution(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      w=Solution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5))
      s=Data(0,(4, 5),w)
      ref=numpy.zeros((4, 5),numpy.float_)
      if dim==2:
        s[0,0]=(0.387453655624)*x[0]**o+(-0.469658136898)*x[0]+(0.250257043285)*x[1]**o+(0.618837167268)*x[1]
        s[0,1]=(-0.698973184774)*x[0]**o+(0.439366278802)*x[0]+(0.365483171998)*x[1]**o+(-0.596967887336)*x[1]
        s[0,2]=(-0.789327020147)*x[0]**o+(-0.720992513079)*x[0]+(0.254736202401)*x[1]**o+(-0.793036130784)*x[1]
        s[0,3]=(0.401178787667)*x[0]**o+(0.272725219284)*x[0]+(0.406915869496)*x[1]**o+(0.650109741413)*x[1]
        s[0,4]=(0.872851752065)*x[0]**o+(0.247512120353)*x[0]+(-0.849447364813)*x[1]**o+(0.341218375884)*x[1]
        s[1,0]=(-0.686440933718)*x[0]**o+(-0.699753221902)*x[0]+(0.416449038712)*x[1]**o+(0.953196364155)*x[1]
        s[1,1]=(-0.445063641949)*x[0]**o+(0.96357651863)*x[0]+(0.928584143634)*x[1]**o+(0.0134608093458)*x[1]
        s[1,2]=(-0.818205311776)*x[0]**o+(0.270136549223)*x[0]+(-0.684209682916)*x[1]**o+(-0.932729815977)*x[1]
        s[1,3]=(-0.0620403511795)*x[0]**o+(0.059231247402)*x[0]+(-0.688733974701)*x[1]**o+(0.581759221954)*x[1]
        s[1,4]=(-0.592213327841)*x[0]**o+(-0.339248053705)*x[0]+(0.384679583558)*x[1]**o+(-0.244452373913)*x[1]
        s[2,0]=(0.602926131051)*x[0]**o+(0.756963449553)*x[0]+(0.764520163983)*x[1]**o+(0.79924075351)*x[1]
        s[2,1]=(0.134902654892)*x[0]**o+(-0.983297874332)*x[0]+(0.919649726121)*x[1]**o+(0.408073376003)*x[1]
        s[2,2]=(0.806897621747)*x[0]**o+(-0.976625920159)*x[0]+(0.478598025599)*x[1]**o+(0.516714148448)*x[1]
        s[2,3]=(0.507729761378)*x[0]**o+(0.701695604786)*x[0]+(0.888872787083)*x[1]**o+(-0.903571907502)*x[1]
        s[2,4]=(0.171628496764)*x[0]**o+(0.54301146383)*x[0]+(0.463317911232)*x[1]**o+(0.998895766351)*x[1]
        s[3,0]=(0.957039213706)*x[0]**o+(-0.740478587736)*x[0]+(-0.915271935195)*x[1]**o+(0.566376957715)*x[1]
        s[3,1]=(0.187513468884)*x[0]**o+(-0.929127411057)*x[0]+(-0.227725906511)*x[1]**o+(-0.807394073167)*x[1]
        s[3,2]=(-0.947087097369)*x[0]**o+(-0.612267326048)*x[0]+(-0.456362542424)*x[1]**o+(-0.353534087643)*x[1]
        s[3,3]=(0.310532071177)*x[0]**o+(-0.221968959306)*x[0]+(-0.349150016728)*x[1]**o+(-0.509141774122)*x[1]
        s[3,4]=(-0.210845769048)*x[0]**o+(0.193660848384)*x[0]+(0.776869628574)*x[1]**o+(-0.154145417271)*x[1]
        ref[0,0]=(0.250257043285)/(o+1.)+(0.074589515185)+(0.387453655624)*0.5**o
        ref[0,1]=(0.365483171998)/(o+1.)+(-0.0788008042673)+(-0.698973184774)*0.5**o
        ref[0,2]=(0.254736202401)/(o+1.)+(-0.757014321931)+(-0.789327020147)*0.5**o
        ref[0,3]=(0.406915869496)/(o+1.)+(0.461417480349)+(0.401178787667)*0.5**o
        ref[0,4]=(-0.849447364813)/(o+1.)+(0.294365248118)+(0.872851752065)*0.5**o
        ref[1,0]=(0.416449038712)/(o+1.)+(0.126721571126)+(-0.686440933718)*0.5**o
        ref[1,1]=(0.928584143634)/(o+1.)+(0.488518663988)+(-0.445063641949)*0.5**o
        ref[1,2]=(-0.684209682916)/(o+1.)+(-0.331296633377)+(-0.818205311776)*0.5**o
        ref[1,3]=(-0.688733974701)/(o+1.)+(0.320495234678)+(-0.0620403511795)*0.5**o
        ref[1,4]=(0.384679583558)/(o+1.)+(-0.291850213809)+(-0.592213327841)*0.5**o
        ref[2,0]=(0.764520163983)/(o+1.)+(0.778102101531)+(0.602926131051)*0.5**o
        ref[2,1]=(0.919649726121)/(o+1.)+(-0.287612249164)+(0.134902654892)*0.5**o
        ref[2,2]=(0.478598025599)/(o+1.)+(-0.229955885856)+(0.806897621747)*0.5**o
        ref[2,3]=(0.888872787083)/(o+1.)+(-0.100938151358)+(0.507729761378)*0.5**o
        ref[2,4]=(0.463317911232)/(o+1.)+(0.77095361509)+(0.171628496764)*0.5**o
        ref[3,0]=(-0.915271935195)/(o+1.)+(-0.0870508150107)+(0.957039213706)*0.5**o
        ref[3,1]=(-0.227725906511)/(o+1.)+(-0.868260742112)+(0.187513468884)*0.5**o
        ref[3,2]=(-0.456362542424)/(o+1.)+(-0.482900706845)+(-0.947087097369)*0.5**o
        ref[3,3]=(-0.349150016728)/(o+1.)+(-0.365555366714)+(0.310532071177)*0.5**o
        ref[3,4]=(0.776869628574)/(o+1.)+(0.0197577155566)+(-0.210845769048)*0.5**o
      else:
        s[0,0]=(-0.0355854650857)*x[0]**o+(0.508097047577)*x[0]+(-0.559809522597)*x[1]**o+(0.274518262337)*x[1]+(-0.346896675687)*x[2]**o+(-0.820202383679)*x[2]
        s[0,1]=(-0.611401860909)*x[0]**o+(-0.804858756979)*x[0]+(0.792251514945)*x[1]**o+(0.127533284836)*x[1]+(0.96691512797)*x[2]**o+(0.0831709389735)*x[2]
        s[0,2]=(-0.0563416061339)*x[0]**o+(0.0591734952104)*x[0]+(0.992243816319)*x[1]**o+(0.457585067795)*x[1]+(0.248903218205)*x[2]**o+(-0.702938956647)*x[2]
        s[0,3]=(0.66016331397)*x[0]**o+(-0.460404888792)*x[0]+(0.834756895572)*x[1]**o+(-0.723598073525)*x[1]+(0.0804592839349)*x[2]**o+(-0.612859372888)*x[2]
        s[0,4]=(0.948196944613)*x[0]**o+(-0.954326701869)*x[0]+(-0.974120082057)*x[1]**o+(0.57658599582)*x[1]+(0.18517252062)*x[2]**o+(0.404027673976)*x[2]
        s[1,0]=(0.096690428758)*x[0]**o+(-0.335528774766)*x[0]+(0.529021814282)*x[1]**o+(0.0944401555902)*x[1]+(-0.0106293914555)*x[2]**o+(-0.885415125356)*x[2]
        s[1,1]=(-0.187544326204)*x[0]**o+(-0.832847850387)*x[0]+(-0.273330813724)*x[1]**o+(-0.0875781530468)*x[1]+(-0.422135351424)*x[2]**o+(-0.44851049492)*x[2]
        s[1,2]=(-0.416290563688)*x[0]**o+(-0.60796418234)*x[0]+(-0.786874489548)*x[1]**o+(-0.83344429526)*x[1]+(-0.636021842684)*x[2]**o+(0.76569386339)*x[2]
        s[1,3]=(-0.888962254312)*x[0]**o+(0.705830842388)*x[0]+(-0.628888150094)*x[1]**o+(0.699724558139)*x[1]+(-0.407095551825)*x[2]**o+(0.796316782037)*x[2]
        s[1,4]=(0.29031515404)*x[0]**o+(0.896505881191)*x[0]+(-0.314937896718)*x[1]**o+(-0.749045267106)*x[1]+(-0.464412869164)*x[2]**o+(-0.357988323816)*x[2]
        s[2,0]=(0.881903169624)*x[0]**o+(-0.962653173961)*x[0]+(0.826332318664)*x[1]**o+(0.625644776113)*x[1]+(0.330804909101)*x[2]**o+(-0.17772544382)*x[2]
        s[2,1]=(-0.503232182675)*x[0]**o+(-0.957203265575)*x[0]+(0.145865955701)*x[1]**o+(0.0915745220394)*x[1]+(-0.470345130645)*x[2]**o+(0.5949845374)*x[2]
        s[2,2]=(0.72138938011)*x[0]**o+(-0.914048390135)*x[0]+(-0.798916451586)*x[1]**o+(-0.720041081497)*x[1]+(-0.292635838745)*x[2]**o+(0.335321951624)*x[2]
        s[2,3]=(-0.182060346819)*x[0]**o+(0.173777377621)*x[0]+(-0.690798771753)*x[1]**o+(-0.792633276338)*x[1]+(0.691527659288)*x[2]**o+(-0.187507542541)*x[2]
        s[2,4]=(-0.5702456536)*x[0]**o+(0.601033902695)*x[0]+(-0.761437618934)*x[1]**o+(-0.258358320383)*x[1]+(-0.718966730705)*x[2]**o+(0.300570256286)*x[2]
        s[3,0]=(0.445936563031)*x[0]**o+(0.859485429121)*x[0]+(-0.191921809371)*x[1]**o+(-0.552489927682)*x[1]+(0.236031566969)*x[2]**o+(0.224011130312)*x[2]
        s[3,1]=(0.383619361689)*x[0]**o+(0.850743712521)*x[0]+(-0.583234052125)*x[1]**o+(-0.957466724321)*x[1]+(0.117664170019)*x[2]**o+(-0.118273519288)*x[2]
        s[3,2]=(0.32727041247)*x[0]**o+(-0.273021170896)*x[0]+(-0.488888392431)*x[1]**o+(0.924810229071)*x[1]+(0.120181256008)*x[2]**o+(0.292893993789)*x[2]
        s[3,3]=(-0.684680337734)*x[0]**o+(-0.494182942675)*x[0]+(-0.0912106871209)*x[1]**o+(-0.483261802779)*x[1]+(0.743468674518)*x[2]**o+(-0.0453199629827)*x[2]
        s[3,4]=(-0.790516141209)*x[0]**o+(0.939271717684)*x[0]+(0.939581799691)*x[1]**o+(0.961123577915)*x[1]+(0.653210734978)*x[2]**o+(0.245098724735)*x[2]
        ref[0,0]=(-0.906706198284)/(o+1.)+(-0.0187935368825)+(-0.0355854650857)*0.5**o
        ref[0,1]=(1.75916664291)/(o+1.)+(-0.297077266585)+(-0.611401860909)*0.5**o
        ref[0,2]=(1.24114703452)/(o+1.)+(-0.0930901968209)+(-0.0563416061339)*0.5**o
        ref[0,3]=(0.915216179507)/(o+1.)+(-0.898431167602)+(0.66016331397)*0.5**o
        ref[0,4]=(-0.788947561438)/(o+1.)+(0.0131434839635)+(0.948196944613)*0.5**o
        ref[1,0]=(0.518392422826)/(o+1.)+(-0.563251872266)+(0.096690428758)*0.5**o
        ref[1,1]=(-0.695466165148)/(o+1.)+(-0.684468249177)+(-0.187544326204)*0.5**o
        ref[1,2]=(-1.42289633223)/(o+1.)+(-0.337857307105)+(-0.416290563688)*0.5**o
        ref[1,3]=(-1.03598370192)/(o+1.)+(1.10093609128)+(-0.888962254312)*0.5**o
        ref[1,4]=(-0.779350765882)/(o+1.)+(-0.105263854865)+(0.29031515404)*0.5**o
        ref[2,0]=(1.15713722776)/(o+1.)+(-0.257366920834)+(0.881903169624)*0.5**o
        ref[2,1]=(-0.324479174944)/(o+1.)+(-0.135322103068)+(-0.503232182675)*0.5**o
        ref[2,2]=(-1.09155229033)/(o+1.)+(-0.649383760004)+(0.72138938011)*0.5**o
        ref[2,3]=(0.000728887534846)/(o+1.)+(-0.403181720629)+(-0.182060346819)*0.5**o
        ref[2,4]=(-1.48040434964)/(o+1.)+(0.321622919299)+(-0.5702456536)*0.5**o
        ref[3,0]=(0.0441097575979)/(o+1.)+(0.265503315876)+(0.445936563031)*0.5**o
        ref[3,1]=(-0.465569882106)/(o+1.)+(-0.112498265544)+(0.383619361689)*0.5**o
        ref[3,2]=(-0.368707136422)/(o+1.)+(0.472341525982)+(0.32727041247)*0.5**o
        ref[3,3]=(0.652257987397)/(o+1.)+(-0.511382354218)+(-0.684680337734)*0.5**o
        ref[3,4]=(1.59279253467)/(o+1.)+(1.07274701017)+(-0.790516141209)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactOne_fromSymbol_Solution_rank3(self):
      """
      tests integral of rank 3 Symbol on the ReducedFunctionOnContactOne

      assumptions: Solution(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      w=Solution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(6, 2, 2))
      s=Data(0,(6, 2, 2),w)
      ref=numpy.zeros((6, 2, 2),numpy.float_)
      if dim==2:
        s[0,0,0]=(0.692838611059)*x[0]**o+(0.701451067887)*x[0]+(-0.958627133469)*x[1]**o+(0.309968446224)*x[1]
        s[0,0,1]=(-0.601729371833)*x[0]**o+(-0.00949649239345)*x[0]+(0.178428915942)*x[1]**o+(-0.378187092112)*x[1]
        s[0,1,0]=(-0.911621902906)*x[0]**o+(0.699730287366)*x[0]+(0.849669560241)*x[1]**o+(-0.24390831078)*x[1]
        s[0,1,1]=(-0.772410377566)*x[0]**o+(-0.775571271165)*x[0]+(0.429781604307)*x[1]**o+(-0.258252436014)*x[1]
        s[1,0,0]=(0.573401470974)*x[0]**o+(0.292621928093)*x[0]+(-0.584549055271)*x[1]**o+(-0.660681435117)*x[1]
        s[1,0,1]=(-0.0867987965936)*x[0]**o+(0.195581072977)*x[0]+(-0.259345214698)*x[1]**o+(-0.431225825128)*x[1]
        s[1,1,0]=(0.170068866316)*x[0]**o+(-0.0665083899809)*x[0]+(0.814824812224)*x[1]**o+(0.278218338402)*x[1]
        s[1,1,1]=(-0.424226747289)*x[0]**o+(-0.0605087462286)*x[0]+(0.72215867299)*x[1]**o+(0.636823345898)*x[1]
        s[2,0,0]=(0.948475744468)*x[0]**o+(-0.256081956766)*x[0]+(-0.265551833564)*x[1]**o+(0.219079882965)*x[1]
        s[2,0,1]=(-0.620771197949)*x[0]**o+(0.203458256679)*x[0]+(-0.950417242448)*x[1]**o+(0.0830268150339)*x[1]
        s[2,1,0]=(-0.392672423199)*x[0]**o+(0.0910137955111)*x[0]+(-0.8417031543)*x[1]**o+(-0.456474678014)*x[1]
        s[2,1,1]=(-0.0312010236388)*x[0]**o+(0.307221450065)*x[0]+(-0.479938290822)*x[1]**o+(-0.689609404462)*x[1]
        s[3,0,0]=(0.124108191486)*x[0]**o+(-0.786546027037)*x[0]+(0.0449572653773)*x[1]**o+(-0.887765762037)*x[1]
        s[3,0,1]=(0.327690655208)*x[0]**o+(-0.651767483905)*x[0]+(-0.14328969555)*x[1]**o+(-0.574417616475)*x[1]
        s[3,1,0]=(0.375233560681)*x[0]**o+(-0.4952402604)*x[0]+(0.419976842971)*x[1]**o+(0.573510110207)*x[1]
        s[3,1,1]=(-0.61513881981)*x[0]**o+(-0.42349537427)*x[0]+(-0.513786717031)*x[1]**o+(-0.656407725495)*x[1]
        s[4,0,0]=(-0.212361872891)*x[0]**o+(-0.274380100495)*x[0]+(-0.115417216962)*x[1]**o+(-0.407361424262)*x[1]
        s[4,0,1]=(-0.26215416052)*x[0]**o+(0.560215913083)*x[0]+(0.949340402876)*x[1]**o+(-0.311511101847)*x[1]
        s[4,1,0]=(0.35481412373)*x[0]**o+(0.55002753014)*x[0]+(-0.427957479742)*x[1]**o+(-0.571074070608)*x[1]
        s[4,1,1]=(0.228369065715)*x[0]**o+(0.34597130297)*x[0]+(-0.099176239601)*x[1]**o+(0.0149309525659)*x[1]
        s[5,0,0]=(-0.230638577613)*x[0]**o+(-0.450379175116)*x[0]+(0.150498069811)*x[1]**o+(-0.26480588837)*x[1]
        s[5,0,1]=(-0.563920707349)*x[0]**o+(-0.678734552317)*x[0]+(0.499409088964)*x[1]**o+(0.636349121568)*x[1]
        s[5,1,0]=(-0.654320489103)*x[0]**o+(-0.759308190818)*x[0]+(0.638645835733)*x[1]**o+(-0.932431876846)*x[1]
        s[5,1,1]=(-0.067089857507)*x[0]**o+(-0.828076668167)*x[0]+(0.329157131175)*x[1]**o+(0.191945408984)*x[1]
        ref[0,0,0]=(-0.958627133469)/(o+1.)+(0.505709757056)+(0.692838611059)*0.5**o
        ref[0,0,1]=(0.178428915942)/(o+1.)+(-0.193841792253)+(-0.601729371833)*0.5**o
        ref[0,1,0]=(0.849669560241)/(o+1.)+(0.227910988293)+(-0.911621902906)*0.5**o
        ref[0,1,1]=(0.429781604307)/(o+1.)+(-0.51691185359)+(-0.772410377566)*0.5**o
        ref[1,0,0]=(-0.584549055271)/(o+1.)+(-0.184029753512)+(0.573401470974)*0.5**o
        ref[1,0,1]=(-0.259345214698)/(o+1.)+(-0.117822376075)+(-0.0867987965936)*0.5**o
        ref[1,1,0]=(0.814824812224)/(o+1.)+(0.10585497421)+(0.170068866316)*0.5**o
        ref[1,1,1]=(0.72215867299)/(o+1.)+(0.288157299835)+(-0.424226747289)*0.5**o
        ref[2,0,0]=(-0.265551833564)/(o+1.)+(-0.0185010369005)+(0.948475744468)*0.5**o
        ref[2,0,1]=(-0.950417242448)/(o+1.)+(0.143242535857)+(-0.620771197949)*0.5**o
        ref[2,1,0]=(-0.8417031543)/(o+1.)+(-0.182730441252)+(-0.392672423199)*0.5**o
        ref[2,1,1]=(-0.479938290822)/(o+1.)+(-0.191193977199)+(-0.0312010236388)*0.5**o
        ref[3,0,0]=(0.0449572653773)/(o+1.)+(-0.837155894537)+(0.124108191486)*0.5**o
        ref[3,0,1]=(-0.14328969555)/(o+1.)+(-0.61309255019)+(0.327690655208)*0.5**o
        ref[3,1,0]=(0.419976842971)/(o+1.)+(0.0391349249035)+(0.375233560681)*0.5**o
        ref[3,1,1]=(-0.513786717031)/(o+1.)+(-0.539951549882)+(-0.61513881981)*0.5**o
        ref[4,0,0]=(-0.115417216962)/(o+1.)+(-0.340870762378)+(-0.212361872891)*0.5**o
        ref[4,0,1]=(0.949340402876)/(o+1.)+(0.124352405618)+(-0.26215416052)*0.5**o
        ref[4,1,0]=(-0.427957479742)/(o+1.)+(-0.0105232702342)+(0.35481412373)*0.5**o
        ref[4,1,1]=(-0.099176239601)/(o+1.)+(0.180451127768)+(0.228369065715)*0.5**o
        ref[5,0,0]=(0.150498069811)/(o+1.)+(-0.357592531743)+(-0.230638577613)*0.5**o
        ref[5,0,1]=(0.499409088964)/(o+1.)+(-0.0211927153744)+(-0.563920707349)*0.5**o
        ref[5,1,0]=(0.638645835733)/(o+1.)+(-0.845870033832)+(-0.654320489103)*0.5**o
        ref[5,1,1]=(0.329157131175)/(o+1.)+(-0.318065629591)+(-0.067089857507)*0.5**o
      else:
        s[0,0,0]=(-0.746527687404)*x[0]**o+(-0.596594966324)*x[0]+(0.2810066152)*x[1]**o+(-0.811343974865)*x[1]+(-0.021161977844)*x[2]**o+(-0.955799066641)*x[2]
        s[0,0,1]=(-0.431368611796)*x[0]**o+(0.209275860867)*x[0]+(-0.744753738662)*x[1]**o+(0.427090838531)*x[1]+(-0.125264890706)*x[2]**o+(0.212116081598)*x[2]
        s[0,1,0]=(-0.262032384111)*x[0]**o+(-0.690463175517)*x[0]+(-0.942716909227)*x[1]**o+(-0.512385399849)*x[1]+(0.0838382201449)*x[2]**o+(-0.645258119863)*x[2]
        s[0,1,1]=(0.528688164571)*x[0]**o+(0.436957291986)*x[0]+(-0.537210963816)*x[1]**o+(-0.857589396313)*x[1]+(-0.748363024129)*x[2]**o+(-0.562281821776)*x[2]
        s[1,0,0]=(0.612146020485)*x[0]**o+(0.732593742961)*x[0]+(-0.0856859718674)*x[1]**o+(0.620805675371)*x[1]+(-0.675276046976)*x[2]**o+(0.768362539829)*x[2]
        s[1,0,1]=(0.953317188148)*x[0]**o+(0.501280748029)*x[0]+(0.613804663039)*x[1]**o+(-0.346135069339)*x[1]+(-0.429549388245)*x[2]**o+(-0.460492494281)*x[2]
        s[1,1,0]=(-0.800228112753)*x[0]**o+(-0.849501214281)*x[0]+(-0.397413958567)*x[1]**o+(0.126586313232)*x[1]+(0.117902029118)*x[2]**o+(-0.163407646867)*x[2]
        s[1,1,1]=(0.814833462883)*x[0]**o+(0.888315382151)*x[0]+(-0.729112863291)*x[1]**o+(-0.990948475544)*x[1]+(-0.407293718484)*x[2]**o+(0.0356918466558)*x[2]
        s[2,0,0]=(0.82229169437)*x[0]**o+(0.346827253309)*x[0]+(-0.747273461294)*x[1]**o+(-0.398145482826)*x[1]+(0.889222392384)*x[2]**o+(0.618983213439)*x[2]
        s[2,0,1]=(-0.829164090486)*x[0]**o+(0.101408063492)*x[0]+(-0.323588772419)*x[1]**o+(-0.513436504297)*x[1]+(0.603168070349)*x[2]**o+(0.469577367849)*x[2]
        s[2,1,0]=(-0.174002172236)*x[0]**o+(0.0457253546329)*x[0]+(0.0680387720249)*x[1]**o+(0.108246528319)*x[1]+(-0.378115826261)*x[2]**o+(0.320064337589)*x[2]
        s[2,1,1]=(0.526364662089)*x[0]**o+(0.715789167446)*x[0]+(0.938035509553)*x[1]**o+(0.811137499085)*x[1]+(0.0343661926035)*x[2]**o+(-0.134159658752)*x[2]
        s[3,0,0]=(-0.391083668074)*x[0]**o+(-0.286558934534)*x[0]+(-0.900015970878)*x[1]**o+(-0.44319367626)*x[1]+(0.97392981229)*x[2]**o+(-0.470626411857)*x[2]
        s[3,0,1]=(-0.733432908462)*x[0]**o+(-0.678130107598)*x[0]+(0.150603930952)*x[1]**o+(-0.598239006862)*x[1]+(-0.768485734549)*x[2]**o+(-0.970399443344)*x[2]
        s[3,1,0]=(-0.247608296411)*x[0]**o+(0.153504414444)*x[0]+(-0.990050628042)*x[1]**o+(-0.274052014666)*x[1]+(-0.614045325729)*x[2]**o+(0.742660852789)*x[2]
        s[3,1,1]=(-0.848161397237)*x[0]**o+(0.0389689968506)*x[0]+(-0.746088757211)*x[1]**o+(0.964788844721)*x[1]+(0.0365840578283)*x[2]**o+(0.0937959359018)*x[2]
        s[4,0,0]=(-0.998118542373)*x[0]**o+(-0.622412978573)*x[0]+(-0.23375496789)*x[1]**o+(0.713065960655)*x[1]+(-0.134547861774)*x[2]**o+(-0.65617589708)*x[2]
        s[4,0,1]=(-0.633127771386)*x[0]**o+(-0.621567651537)*x[0]+(-0.54904676147)*x[1]**o+(-0.868056268787)*x[1]+(0.234405832043)*x[2]**o+(0.737621357594)*x[2]
        s[4,1,0]=(0.862411543309)*x[0]**o+(-0.737128008238)*x[0]+(0.307586357348)*x[1]**o+(0.760442280183)*x[1]+(-0.56282765669)*x[2]**o+(-0.0147875945863)*x[2]
        s[4,1,1]=(0.314415379367)*x[0]**o+(0.626003051386)*x[0]+(0.997284366098)*x[1]**o+(-0.199517448031)*x[1]+(0.375147056596)*x[2]**o+(0.732546433867)*x[2]
        s[5,0,0]=(0.819256041498)*x[0]**o+(0.733896748535)*x[0]+(0.0839088873455)*x[1]**o+(0.515382312439)*x[1]+(-0.0194919927864)*x[2]**o+(0.0318260275007)*x[2]
        s[5,0,1]=(0.940274091937)*x[0]**o+(-0.735201721223)*x[0]+(-0.953504926957)*x[1]**o+(-0.945986665743)*x[1]+(-0.0220685469559)*x[2]**o+(0.622957541524)*x[2]
        s[5,1,0]=(-0.93927853219)*x[0]**o+(-0.886451123487)*x[0]+(0.448010704185)*x[1]**o+(-0.9971779094)*x[1]+(0.782692635361)*x[2]**o+(0.690868313501)*x[2]
        s[5,1,1]=(0.101661872917)*x[0]**o+(0.636433790255)*x[0]+(0.526460976779)*x[1]**o+(-0.603459918432)*x[1]+(0.107035082256)*x[2]**o+(0.252479495202)*x[2]
        ref[0,0,0]=(0.259844637356)/(o+1.)+(-1.18186900391)+(-0.746527687404)*0.5**o
        ref[0,0,1]=(-0.870018629367)/(o+1.)+(0.424241390497)+(-0.431368611796)*0.5**o
        ref[0,1,0]=(-0.858878689082)/(o+1.)+(-0.924053347614)+(-0.262032384111)*0.5**o
        ref[0,1,1]=(-1.28557398794)/(o+1.)+(-0.491456963051)+(0.528688164571)*0.5**o
        ref[1,0,0]=(-0.760962018843)/(o+1.)+(1.06088097908)+(0.612146020485)*0.5**o
        ref[1,0,1]=(0.184255274794)/(o+1.)+(-0.152673407796)+(0.953317188148)*0.5**o
        ref[1,1,0]=(-0.279511929449)/(o+1.)+(-0.443161273958)+(-0.800228112753)*0.5**o
        ref[1,1,1]=(-1.13640658178)/(o+1.)+(-0.0334706233686)+(0.814833462883)*0.5**o
        ref[2,0,0]=(0.14194893109)/(o+1.)+(0.283832491961)+(0.82229169437)*0.5**o
        ref[2,0,1]=(0.27957929793)/(o+1.)+(0.0287744635219)+(-0.829164090486)*0.5**o
        ref[2,1,0]=(-0.310077054236)/(o+1.)+(0.23701811027)+(-0.174002172236)*0.5**o
        ref[2,1,1]=(0.972401702157)/(o+1.)+(0.696383503889)+(0.526364662089)*0.5**o
        ref[3,0,0]=(0.0739138414121)/(o+1.)+(-0.600189511325)+(-0.391083668074)*0.5**o
        ref[3,0,1]=(-0.617881803597)/(o+1.)+(-1.1233842789)+(-0.733432908462)*0.5**o
        ref[3,1,0]=(-1.60409595377)/(o+1.)+(0.311056626283)+(-0.247608296411)*0.5**o
        ref[3,1,1]=(-0.709504699383)/(o+1.)+(0.548776888736)+(-0.848161397237)*0.5**o
        ref[4,0,0]=(-0.368302829664)/(o+1.)+(-0.282761457499)+(-0.998118542373)*0.5**o
        ref[4,0,1]=(-0.314640929426)/(o+1.)+(-0.376001281365)+(-0.633127771386)*0.5**o
        ref[4,1,0]=(-0.255241299343)/(o+1.)+(0.00426333867911)+(0.862411543309)*0.5**o
        ref[4,1,1]=(1.37243142269)/(o+1.)+(0.579516018611)+(0.314415379367)*0.5**o
        ref[5,0,0]=(0.0644168945591)/(o+1.)+(0.640552544238)+(0.819256041498)*0.5**o
        ref[5,0,1]=(-0.975573473912)/(o+1.)+(-0.529115422721)+(0.940274091937)*0.5**o
        ref[5,1,0]=(1.23070333955)/(o+1.)+(-0.596380359693)+(-0.93927853219)*0.5**o
        ref[5,1,1]=(0.633496059035)/(o+1.)+(0.142726683513)+(0.101661872917)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactOne_fromSymbol_Solution_rank4(self):
      """
      tests integral of rank 4 Symbol on the ReducedFunctionOnContactOne

      assumptions: Solution(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      w=Solution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5, 3, 2))
      s=Data(0,(4, 5, 3, 2),w)
      ref=numpy.zeros((4, 5, 3, 2),numpy.float_)
      if dim==2:
        s[0,0,0,0]=(-0.686834873081)*x[0]**o+(0.894217065974)*x[0]+(0.464807422099)*x[1]**o+(0.117486893484)*x[1]
        s[0,0,0,1]=(0.423267211666)*x[0]**o+(-0.926368250674)*x[0]+(-0.00127389833363)*x[1]**o+(0.644912230378)*x[1]
        s[0,0,1,0]=(0.225761019463)*x[0]**o+(0.941520398103)*x[0]+(0.698674836081)*x[1]**o+(-0.545668525315)*x[1]
        s[0,0,1,1]=(-0.0578569008694)*x[0]**o+(0.886606073204)*x[0]+(-0.943532460794)*x[1]**o+(0.63316605865)*x[1]
        s[0,0,2,0]=(-0.955100804268)*x[0]**o+(-0.197482002011)*x[0]+(0.727953088594)*x[1]**o+(0.842355085503)*x[1]
        s[0,0,2,1]=(0.55641264625)*x[0]**o+(0.26186880746)*x[0]+(0.769897594142)*x[1]**o+(0.630360270402)*x[1]
        s[0,1,0,0]=(0.0417005921153)*x[0]**o+(-0.254303592363)*x[0]+(0.517125320203)*x[1]**o+(0.356198004099)*x[1]
        s[0,1,0,1]=(-0.975918028511)*x[0]**o+(-0.655984560339)*x[0]+(0.00235297753913)*x[1]**o+(-0.111765878548)*x[1]
        s[0,1,1,0]=(-0.16393042329)*x[0]**o+(0.420621371478)*x[0]+(0.367090269805)*x[1]**o+(0.610715442136)*x[1]
        s[0,1,1,1]=(-0.875851967836)*x[0]**o+(-0.816358619616)*x[0]+(0.80277045991)*x[1]**o+(0.717265655871)*x[1]
        s[0,1,2,0]=(0.871764357933)*x[0]**o+(-0.867731309803)*x[0]+(0.410055383976)*x[1]**o+(-0.722483045113)*x[1]
        s[0,1,2,1]=(-0.427119083496)*x[0]**o+(-0.833805789919)*x[0]+(-0.275401454293)*x[1]**o+(0.442220067398)*x[1]
        s[0,2,0,0]=(0.462551904284)*x[0]**o+(-0.0535451795861)*x[0]+(0.270504950569)*x[1]**o+(0.834360806219)*x[1]
        s[0,2,0,1]=(-0.0507590824972)*x[0]**o+(0.26574820566)*x[0]+(0.610544260364)*x[1]**o+(-0.890035786158)*x[1]
        s[0,2,1,0]=(-0.638445791035)*x[0]**o+(-0.305351157409)*x[0]+(0.965223780441)*x[1]**o+(0.982229276343)*x[1]
        s[0,2,1,1]=(-0.731025223911)*x[0]**o+(0.307228768356)*x[0]+(0.645857556893)*x[1]**o+(0.913645113546)*x[1]
        s[0,2,2,0]=(-0.196899050904)*x[0]**o+(0.150588444027)*x[0]+(0.603646143067)*x[1]**o+(0.811245615941)*x[1]
        s[0,2,2,1]=(0.546119342726)*x[0]**o+(0.204141566001)*x[0]+(0.114502748703)*x[1]**o+(-0.698961842363)*x[1]
        s[0,3,0,0]=(-0.0426650685874)*x[0]**o+(-0.335440677298)*x[0]+(-0.174118589494)*x[1]**o+(-0.851843219794)*x[1]
        s[0,3,0,1]=(-0.14536830456)*x[0]**o+(0.505179058635)*x[0]+(-0.942847104809)*x[1]**o+(0.518276559236)*x[1]
        s[0,3,1,0]=(-0.737548624771)*x[0]**o+(0.70571598977)*x[0]+(-0.746943114818)*x[1]**o+(-0.2127583186)*x[1]
        s[0,3,1,1]=(0.360491166663)*x[0]**o+(0.986095817206)*x[0]+(-0.918270622952)*x[1]**o+(0.617678170994)*x[1]
        s[0,3,2,0]=(-0.570338406694)*x[0]**o+(-0.513173501835)*x[0]+(0.572925790394)*x[1]**o+(0.130209822282)*x[1]
        s[0,3,2,1]=(0.112118675411)*x[0]**o+(0.561084973521)*x[0]+(0.598978537086)*x[1]**o+(-0.0313515131555)*x[1]
        s[0,4,0,0]=(-0.971043700474)*x[0]**o+(0.841925646149)*x[0]+(0.858700520274)*x[1]**o+(-0.304258365028)*x[1]
        s[0,4,0,1]=(-0.33210808558)*x[0]**o+(-0.453977822683)*x[0]+(-0.74202650883)*x[1]**o+(0.546471602539)*x[1]
        s[0,4,1,0]=(-0.622155482312)*x[0]**o+(-0.0779995859909)*x[0]+(0.221889403634)*x[1]**o+(0.342594640865)*x[1]
        s[0,4,1,1]=(-0.83327572159)*x[0]**o+(-0.736091414594)*x[0]+(-0.907324696118)*x[1]**o+(0.543711697923)*x[1]
        s[0,4,2,0]=(0.97730521508)*x[0]**o+(0.680289384194)*x[0]+(0.749574300046)*x[1]**o+(-0.279718517953)*x[1]
        s[0,4,2,1]=(-0.188385359119)*x[0]**o+(-0.517585952202)*x[0]+(-0.92321177387)*x[1]**o+(0.794132586536)*x[1]
        s[1,0,0,0]=(-0.691363379735)*x[0]**o+(-0.338291851496)*x[0]+(0.708824882363)*x[1]**o+(-0.154240182249)*x[1]
        s[1,0,0,1]=(0.391349048346)*x[0]**o+(0.10762051179)*x[0]+(0.971913362543)*x[1]**o+(-0.0532019503958)*x[1]
        s[1,0,1,0]=(-0.669458166029)*x[0]**o+(0.353354538692)*x[0]+(-0.441190419207)*x[1]**o+(-0.528430239493)*x[1]
        s[1,0,1,1]=(0.107848426266)*x[0]**o+(-0.076965091351)*x[0]+(0.745243283501)*x[1]**o+(-0.782833446988)*x[1]
        s[1,0,2,0]=(0.364065471931)*x[0]**o+(0.54771360956)*x[0]+(0.565373904163)*x[1]**o+(0.616558822792)*x[1]
        s[1,0,2,1]=(-0.379996842423)*x[0]**o+(0.392499507062)*x[0]+(-0.808821894825)*x[1]**o+(-0.615251700689)*x[1]
        s[1,1,0,0]=(-0.245969602554)*x[0]**o+(0.436762196801)*x[0]+(0.860717658702)*x[1]**o+(0.132856923997)*x[1]
        s[1,1,0,1]=(-0.461963226134)*x[0]**o+(0.620555910148)*x[0]+(-0.119798710079)*x[1]**o+(0.0221208440902)*x[1]
        s[1,1,1,0]=(-0.357540385682)*x[0]**o+(0.35759692542)*x[0]+(-0.542207154591)*x[1]**o+(-0.334151697958)*x[1]
        s[1,1,1,1]=(-0.507609494267)*x[0]**o+(0.298724176211)*x[0]+(-0.689016521358)*x[1]**o+(-0.574923042649)*x[1]
        s[1,1,2,0]=(-0.314205160643)*x[0]**o+(-0.855793612952)*x[0]+(0.245833632085)*x[1]**o+(0.65947369599)*x[1]
        s[1,1,2,1]=(0.92975337751)*x[0]**o+(0.981531336944)*x[0]+(0.889631731534)*x[1]**o+(0.96084291721)*x[1]
        s[1,2,0,0]=(-0.206928230723)*x[0]**o+(-0.0227833929279)*x[0]+(0.910908314307)*x[1]**o+(-0.293206125585)*x[1]
        s[1,2,0,1]=(0.655611121589)*x[0]**o+(-0.363559028874)*x[0]+(0.944323994431)*x[1]**o+(0.675785829867)*x[1]
        s[1,2,1,0]=(0.097163050774)*x[0]**o+(-0.306871139813)*x[0]+(-0.583236391219)*x[1]**o+(0.2082878001)*x[1]
        s[1,2,1,1]=(0.966536588817)*x[0]**o+(0.366298955916)*x[0]+(0.637843032912)*x[1]**o+(0.808769955133)*x[1]
        s[1,2,2,0]=(-0.688421104726)*x[0]**o+(-0.208613716503)*x[0]+(-0.264530414111)*x[1]**o+(-0.869467631827)*x[1]
        s[1,2,2,1]=(0.572370662453)*x[0]**o+(0.0176965600004)*x[0]+(-0.252518792378)*x[1]**o+(-0.696779486832)*x[1]
        s[1,3,0,0]=(-0.23128498)*x[0]**o+(0.0318637329932)*x[0]+(0.477807829207)*x[1]**o+(-0.512393411225)*x[1]
        s[1,3,0,1]=(-0.121822943652)*x[0]**o+(0.264705755391)*x[0]+(-0.477007018414)*x[1]**o+(0.991097887584)*x[1]
        s[1,3,1,0]=(-0.338674627679)*x[0]**o+(0.779182013477)*x[0]+(-0.0510727374236)*x[1]**o+(0.613462237402)*x[1]
        s[1,3,1,1]=(-0.842082566923)*x[0]**o+(0.659487619964)*x[0]+(-0.0616578458759)*x[1]**o+(0.691256407339)*x[1]
        s[1,3,2,0]=(0.462359358829)*x[0]**o+(-0.782666568422)*x[0]+(-0.00922728778114)*x[1]**o+(0.935329586498)*x[1]
        s[1,3,2,1]=(0.0965038514703)*x[0]**o+(0.937329942152)*x[0]+(-0.0289919380082)*x[1]**o+(0.453957829701)*x[1]
        s[1,4,0,0]=(-0.577911003192)*x[0]**o+(0.74727013652)*x[0]+(-0.970205360146)*x[1]**o+(-0.525871518563)*x[1]
        s[1,4,0,1]=(0.0798727685012)*x[0]**o+(0.181295188234)*x[0]+(0.321952537611)*x[1]**o+(-0.702604584966)*x[1]
        s[1,4,1,0]=(-0.132278278329)*x[0]**o+(0.975101058477)*x[0]+(0.236705098567)*x[1]**o+(-0.10937616935)*x[1]
        s[1,4,1,1]=(-0.361402070774)*x[0]**o+(-0.370364725782)*x[0]+(-0.610674940874)*x[1]**o+(-0.0962382083614)*x[1]
        s[1,4,2,0]=(0.813396048857)*x[0]**o+(0.292472923811)*x[0]+(-0.901067471391)*x[1]**o+(0.547614067474)*x[1]
        s[1,4,2,1]=(0.390285391184)*x[0]**o+(-0.385915061071)*x[0]+(-0.0402809003623)*x[1]**o+(-0.988197507151)*x[1]
        s[2,0,0,0]=(-0.249708353038)*x[0]**o+(-0.906354186065)*x[0]+(0.962323825393)*x[1]**o+(0.0966005473089)*x[1]
        s[2,0,0,1]=(0.815655802435)*x[0]**o+(0.59037691594)*x[0]+(-0.913553917647)*x[1]**o+(0.195455032055)*x[1]
        s[2,0,1,0]=(-0.041263589419)*x[0]**o+(-0.722404855046)*x[0]+(0.792396772777)*x[1]**o+(0.980090324531)*x[1]
        s[2,0,1,1]=(0.336059537191)*x[0]**o+(-0.953661694087)*x[0]+(-0.377727072925)*x[1]**o+(-0.00288817417099)*x[1]
        s[2,0,2,0]=(0.0637701377122)*x[0]**o+(0.185264659888)*x[0]+(-0.63229573614)*x[1]**o+(-0.743036413091)*x[1]
        s[2,0,2,1]=(-0.187859444818)*x[0]**o+(-0.513109580827)*x[0]+(-0.649825178883)*x[1]**o+(0.883502750572)*x[1]
        s[2,1,0,0]=(0.87708393158)*x[0]**o+(-0.868702717082)*x[0]+(-0.727725348725)*x[1]**o+(-0.977109996353)*x[1]
        s[2,1,0,1]=(-0.192407804254)*x[0]**o+(0.240131364352)*x[0]+(-0.907681129252)*x[1]**o+(0.558779298439)*x[1]
        s[2,1,1,0]=(0.153001662465)*x[0]**o+(0.795683040566)*x[0]+(0.0438148749584)*x[1]**o+(-0.0621663419181)*x[1]
        s[2,1,1,1]=(0.591374562403)*x[0]**o+(0.0792161473602)*x[0]+(0.692300008749)*x[1]**o+(0.497404274834)*x[1]
        s[2,1,2,0]=(-0.742751457969)*x[0]**o+(0.963865160953)*x[0]+(-0.217320783858)*x[1]**o+(-0.532084042794)*x[1]
        s[2,1,2,1]=(-0.834015860027)*x[0]**o+(0.154439983509)*x[0]+(0.497803852733)*x[1]**o+(0.205893944848)*x[1]
        s[2,2,0,0]=(-0.203653128753)*x[0]**o+(0.240132473537)*x[0]+(-0.655435581477)*x[1]**o+(-0.858878900716)*x[1]
        s[2,2,0,1]=(0.971807071325)*x[0]**o+(-0.373206373801)*x[0]+(0.140960526316)*x[1]**o+(-0.726655373182)*x[1]
        s[2,2,1,0]=(0.36337495614)*x[0]**o+(0.024734800399)*x[0]+(-0.87272068545)*x[1]**o+(-0.342758158473)*x[1]
        s[2,2,1,1]=(0.703205712066)*x[0]**o+(-0.701536561698)*x[0]+(0.849847759033)*x[1]**o+(0.436346580068)*x[1]
        s[2,2,2,0]=(0.814179476293)*x[0]**o+(-0.722566145563)*x[0]+(0.730674735708)*x[1]**o+(0.433386430731)*x[1]
        s[2,2,2,1]=(0.465577907694)*x[0]**o+(-0.0864038816161)*x[0]+(0.927807005045)*x[1]**o+(0.56469159985)*x[1]
        s[2,3,0,0]=(-0.0809501575869)*x[0]**o+(-0.618484411939)*x[0]+(0.737902566912)*x[1]**o+(0.599580560022)*x[1]
        s[2,3,0,1]=(-0.974260986562)*x[0]**o+(0.403417818779)*x[0]+(0.688538873306)*x[1]**o+(0.12462953818)*x[1]
        s[2,3,1,0]=(0.256567297589)*x[0]**o+(0.996662217474)*x[0]+(-0.892476705335)*x[1]**o+(-0.586859542133)*x[1]
        s[2,3,1,1]=(0.600281177478)*x[0]**o+(-0.884090855927)*x[0]+(0.724531635238)*x[1]**o+(-0.16428377776)*x[1]
        s[2,3,2,0]=(-0.316555549763)*x[0]**o+(0.654066898461)*x[0]+(-0.636613154487)*x[1]**o+(-0.0527299937889)*x[1]
        s[2,3,2,1]=(0.509096161125)*x[0]**o+(0.553056557071)*x[0]+(0.241657689068)*x[1]**o+(-0.988729739553)*x[1]
        s[2,4,0,0]=(-0.00190589099569)*x[0]**o+(-0.838680435068)*x[0]+(-0.531592666371)*x[1]**o+(0.26101901932)*x[1]
        s[2,4,0,1]=(0.727384767513)*x[0]**o+(-0.584151796751)*x[0]+(-0.32109862114)*x[1]**o+(-0.440158856065)*x[1]
        s[2,4,1,0]=(-0.190511998024)*x[0]**o+(0.695377622072)*x[0]+(0.815200064372)*x[1]**o+(-0.91046353217)*x[1]
        s[2,4,1,1]=(0.505564114308)*x[0]**o+(-0.139255460306)*x[0]+(0.323832915733)*x[1]**o+(-0.307972664581)*x[1]
        s[2,4,2,0]=(-0.199017289187)*x[0]**o+(-0.748501229072)*x[0]+(-0.378591546965)*x[1]**o+(0.862316455474)*x[1]
        s[2,4,2,1]=(-0.166203292304)*x[0]**o+(-0.0809782096656)*x[0]+(0.0866258209859)*x[1]**o+(0.443578824637)*x[1]
        s[3,0,0,0]=(0.264566793834)*x[0]**o+(0.811278578401)*x[0]+(0.42099538494)*x[1]**o+(0.199600034959)*x[1]
        s[3,0,0,1]=(0.235746402593)*x[0]**o+(-0.921869283354)*x[0]+(-0.566125315231)*x[1]**o+(0.0537623665249)*x[1]
        s[3,0,1,0]=(0.862345276865)*x[0]**o+(0.227476031253)*x[0]+(-0.448144780994)*x[1]**o+(-0.861358696525)*x[1]
        s[3,0,1,1]=(-0.896403179984)*x[0]**o+(0.431197708747)*x[0]+(-0.652497974262)*x[1]**o+(0.104540024804)*x[1]
        s[3,0,2,0]=(0.788731583728)*x[0]**o+(0.570545432128)*x[0]+(-0.455838556745)*x[1]**o+(0.629398526061)*x[1]
        s[3,0,2,1]=(-0.727929054337)*x[0]**o+(0.561826310816)*x[0]+(0.158791540215)*x[1]**o+(0.458507235686)*x[1]
        s[3,1,0,0]=(0.349297794591)*x[0]**o+(-0.179009101281)*x[0]+(-0.0535819231755)*x[1]**o+(-0.153059139984)*x[1]
        s[3,1,0,1]=(-0.759321371341)*x[0]**o+(-0.464845449228)*x[0]+(-0.800973035912)*x[1]**o+(0.905300621325)*x[1]
        s[3,1,1,0]=(-0.597928946862)*x[0]**o+(-0.621215873514)*x[0]+(0.521881881398)*x[1]**o+(0.570001614348)*x[1]
        s[3,1,1,1]=(-0.263124019379)*x[0]**o+(-0.970277462371)*x[0]+(0.445221632359)*x[1]**o+(-0.610952489733)*x[1]
        s[3,1,2,0]=(0.434074902924)*x[0]**o+(-0.34516630325)*x[0]+(0.812893197377)*x[1]**o+(-0.755651456445)*x[1]
        s[3,1,2,1]=(-0.724153308139)*x[0]**o+(-0.111288572288)*x[0]+(-0.40676151673)*x[1]**o+(0.547624797328)*x[1]
        s[3,2,0,0]=(0.915215698154)*x[0]**o+(-0.770595818702)*x[0]+(-0.88668061529)*x[1]**o+(0.37772776152)*x[1]
        s[3,2,0,1]=(-0.456607413762)*x[0]**o+(-0.972000077172)*x[0]+(0.453597269106)*x[1]**o+(-0.535557277591)*x[1]
        s[3,2,1,0]=(-0.428848816409)*x[0]**o+(-0.957711101461)*x[0]+(0.0771381998527)*x[1]**o+(0.578640765452)*x[1]
        s[3,2,1,1]=(-0.488115399257)*x[0]**o+(0.966283881858)*x[0]+(0.786396903927)*x[1]**o+(-0.427389628779)*x[1]
        s[3,2,2,0]=(-0.748108272191)*x[0]**o+(-0.411931818676)*x[0]+(-0.104450005208)*x[1]**o+(-0.548784991612)*x[1]
        s[3,2,2,1]=(-0.288896584199)*x[0]**o+(-0.599403262911)*x[0]+(-0.107194826319)*x[1]**o+(-0.714445846353)*x[1]
        s[3,3,0,0]=(-0.643859183999)*x[0]**o+(0.0290410872689)*x[0]+(0.184456668026)*x[1]**o+(0.62348637013)*x[1]
        s[3,3,0,1]=(-0.258292072759)*x[0]**o+(0.337782506924)*x[0]+(-0.653836797208)*x[1]**o+(-0.373955061219)*x[1]
        s[3,3,1,0]=(-0.659219139805)*x[0]**o+(0.39140958876)*x[0]+(0.684973413357)*x[1]**o+(-0.0642329755552)*x[1]
        s[3,3,1,1]=(-0.649682331365)*x[0]**o+(-0.00281746580006)*x[0]+(0.967988326693)*x[1]**o+(0.895373576808)*x[1]
        s[3,3,2,0]=(-0.0299199232233)*x[0]**o+(0.458695499665)*x[0]+(0.911862994231)*x[1]**o+(-0.748787338345)*x[1]
        s[3,3,2,1]=(0.197795840564)*x[0]**o+(-0.765596541276)*x[0]+(0.121310526207)*x[1]**o+(-0.300366665995)*x[1]
        s[3,4,0,0]=(0.278568700563)*x[0]**o+(-0.81691908238)*x[0]+(0.144475855023)*x[1]**o+(0.104006513674)*x[1]
        s[3,4,0,1]=(-0.608132003655)*x[0]**o+(0.847594429708)*x[0]+(-0.846906570194)*x[1]**o+(0.350085739363)*x[1]
        s[3,4,1,0]=(-0.490163043631)*x[0]**o+(0.929230880519)*x[0]+(-0.599983340725)*x[1]**o+(-0.252289176277)*x[1]
        s[3,4,1,1]=(-0.802228749858)*x[0]**o+(-0.835578269096)*x[0]+(-0.466727935821)*x[1]**o+(0.208416931397)*x[1]
        s[3,4,2,0]=(0.963914397515)*x[0]**o+(0.552034808675)*x[0]+(-0.0529537502248)*x[1]**o+(0.56193543593)*x[1]
        s[3,4,2,1]=(-0.201051432425)*x[0]**o+(0.720512883794)*x[0]+(0.942559524116)*x[1]**o+(-0.660858122132)*x[1]
        ref[0,0,0,0]=(0.464807422099)/(o+1.)+(0.505851979729)+(-0.686834873081)*0.5**o
        ref[0,0,0,1]=(-0.00127389833363)/(o+1.)+(-0.140728010148)+(0.423267211666)*0.5**o
        ref[0,0,1,0]=(0.698674836081)/(o+1.)+(0.197925936394)+(0.225761019463)*0.5**o
        ref[0,0,1,1]=(-0.943532460794)/(o+1.)+(0.759886065927)+(-0.0578569008694)*0.5**o
        ref[0,0,2,0]=(0.727953088594)/(o+1.)+(0.322436541746)+(-0.955100804268)*0.5**o
        ref[0,0,2,1]=(0.769897594142)/(o+1.)+(0.446114538931)+(0.55641264625)*0.5**o
        ref[0,1,0,0]=(0.517125320203)/(o+1.)+(0.0509472058681)+(0.0417005921153)*0.5**o
        ref[0,1,0,1]=(0.00235297753913)/(o+1.)+(-0.383875219444)+(-0.975918028511)*0.5**o
        ref[0,1,1,0]=(0.367090269805)/(o+1.)+(0.515668406807)+(-0.16393042329)*0.5**o
        ref[0,1,1,1]=(0.80277045991)/(o+1.)+(-0.0495464818722)+(-0.875851967836)*0.5**o
        ref[0,1,2,0]=(0.410055383976)/(o+1.)+(-0.795107177458)+(0.871764357933)*0.5**o
        ref[0,1,2,1]=(-0.275401454293)/(o+1.)+(-0.195792861261)+(-0.427119083496)*0.5**o
        ref[0,2,0,0]=(0.270504950569)/(o+1.)+(0.390407813316)+(0.462551904284)*0.5**o
        ref[0,2,0,1]=(0.610544260364)/(o+1.)+(-0.312143790249)+(-0.0507590824972)*0.5**o
        ref[0,2,1,0]=(0.965223780441)/(o+1.)+(0.338439059467)+(-0.638445791035)*0.5**o
        ref[0,2,1,1]=(0.645857556893)/(o+1.)+(0.610436940951)+(-0.731025223911)*0.5**o
        ref[0,2,2,0]=(0.603646143067)/(o+1.)+(0.480917029984)+(-0.196899050904)*0.5**o
        ref[0,2,2,1]=(0.114502748703)/(o+1.)+(-0.247410138181)+(0.546119342726)*0.5**o
        ref[0,3,0,0]=(-0.174118589494)/(o+1.)+(-0.593641948546)+(-0.0426650685874)*0.5**o
        ref[0,3,0,1]=(-0.942847104809)/(o+1.)+(0.511727808935)+(-0.14536830456)*0.5**o
        ref[0,3,1,0]=(-0.746943114818)/(o+1.)+(0.246478835585)+(-0.737548624771)*0.5**o
        ref[0,3,1,1]=(-0.918270622952)/(o+1.)+(0.8018869941)+(0.360491166663)*0.5**o
        ref[0,3,2,0]=(0.572925790394)/(o+1.)+(-0.191481839776)+(-0.570338406694)*0.5**o
        ref[0,3,2,1]=(0.598978537086)/(o+1.)+(0.264866730183)+(0.112118675411)*0.5**o
        ref[0,4,0,0]=(0.858700520274)/(o+1.)+(0.26883364056)+(-0.971043700474)*0.5**o
        ref[0,4,0,1]=(-0.74202650883)/(o+1.)+(0.0462468899283)+(-0.33210808558)*0.5**o
        ref[0,4,1,0]=(0.221889403634)/(o+1.)+(0.132297527437)+(-0.622155482312)*0.5**o
        ref[0,4,1,1]=(-0.907324696118)/(o+1.)+(-0.0961898583351)+(-0.83327572159)*0.5**o
        ref[0,4,2,0]=(0.749574300046)/(o+1.)+(0.200285433121)+(0.97730521508)*0.5**o
        ref[0,4,2,1]=(-0.92321177387)/(o+1.)+(0.138273317167)+(-0.188385359119)*0.5**o
        ref[1,0,0,0]=(0.708824882363)/(o+1.)+(-0.246266016873)+(-0.691363379735)*0.5**o
        ref[1,0,0,1]=(0.971913362543)/(o+1.)+(0.0272092806971)+(0.391349048346)*0.5**o
        ref[1,0,1,0]=(-0.441190419207)/(o+1.)+(-0.0875378504002)+(-0.669458166029)*0.5**o
        ref[1,0,1,1]=(0.745243283501)/(o+1.)+(-0.42989926917)+(0.107848426266)*0.5**o
        ref[1,0,2,0]=(0.565373904163)/(o+1.)+(0.582136216176)+(0.364065471931)*0.5**o
        ref[1,0,2,1]=(-0.808821894825)/(o+1.)+(-0.111376096814)+(-0.379996842423)*0.5**o
        ref[1,1,0,0]=(0.860717658702)/(o+1.)+(0.284809560399)+(-0.245969602554)*0.5**o
        ref[1,1,0,1]=(-0.119798710079)/(o+1.)+(0.321338377119)+(-0.461963226134)*0.5**o
        ref[1,1,1,0]=(-0.542207154591)/(o+1.)+(0.0117226137313)+(-0.357540385682)*0.5**o
        ref[1,1,1,1]=(-0.689016521358)/(o+1.)+(-0.138099433219)+(-0.507609494267)*0.5**o
        ref[1,1,2,0]=(0.245833632085)/(o+1.)+(-0.0981599584809)+(-0.314205160643)*0.5**o
        ref[1,1,2,1]=(0.889631731534)/(o+1.)+(0.971187127077)+(0.92975337751)*0.5**o
        ref[1,2,0,0]=(0.910908314307)/(o+1.)+(-0.157994759256)+(-0.206928230723)*0.5**o
        ref[1,2,0,1]=(0.944323994431)/(o+1.)+(0.156113400497)+(0.655611121589)*0.5**o
        ref[1,2,1,0]=(-0.583236391219)/(o+1.)+(-0.0492916698561)+(0.097163050774)*0.5**o
        ref[1,2,1,1]=(0.637843032912)/(o+1.)+(0.587534455525)+(0.966536588817)*0.5**o
        ref[1,2,2,0]=(-0.264530414111)/(o+1.)+(-0.539040674165)+(-0.688421104726)*0.5**o
        ref[1,2,2,1]=(-0.252518792378)/(o+1.)+(-0.339541463416)+(0.572370662453)*0.5**o
        ref[1,3,0,0]=(0.477807829207)/(o+1.)+(-0.240264839116)+(-0.23128498)*0.5**o
        ref[1,3,0,1]=(-0.477007018414)/(o+1.)+(0.627901821488)+(-0.121822943652)*0.5**o
        ref[1,3,1,0]=(-0.0510727374236)/(o+1.)+(0.69632212544)+(-0.338674627679)*0.5**o
        ref[1,3,1,1]=(-0.0616578458759)/(o+1.)+(0.675372013651)+(-0.842082566923)*0.5**o
        ref[1,3,2,0]=(-0.00922728778114)/(o+1.)+(0.0763315090377)+(0.462359358829)*0.5**o
        ref[1,3,2,1]=(-0.0289919380082)/(o+1.)+(0.695643885927)+(0.0965038514703)*0.5**o
        ref[1,4,0,0]=(-0.970205360146)/(o+1.)+(0.110699308979)+(-0.577911003192)*0.5**o
        ref[1,4,0,1]=(0.321952537611)/(o+1.)+(-0.260654698366)+(0.0798727685012)*0.5**o
        ref[1,4,1,0]=(0.236705098567)/(o+1.)+(0.432862444563)+(-0.132278278329)*0.5**o
        ref[1,4,1,1]=(-0.610674940874)/(o+1.)+(-0.233301467071)+(-0.361402070774)*0.5**o
        ref[1,4,2,0]=(-0.901067471391)/(o+1.)+(0.420043495642)+(0.813396048857)*0.5**o
        ref[1,4,2,1]=(-0.0402809003623)/(o+1.)+(-0.687056284111)+(0.390285391184)*0.5**o
        ref[2,0,0,0]=(0.962323825393)/(o+1.)+(-0.404876819378)+(-0.249708353038)*0.5**o
        ref[2,0,0,1]=(-0.913553917647)/(o+1.)+(0.392915973998)+(0.815655802435)*0.5**o
        ref[2,0,1,0]=(0.792396772777)/(o+1.)+(0.128842734742)+(-0.041263589419)*0.5**o
        ref[2,0,1,1]=(-0.377727072925)/(o+1.)+(-0.478274934129)+(0.336059537191)*0.5**o
        ref[2,0,2,0]=(-0.63229573614)/(o+1.)+(-0.278885876602)+(0.0637701377122)*0.5**o
        ref[2,0,2,1]=(-0.649825178883)/(o+1.)+(0.185196584873)+(-0.187859444818)*0.5**o
        ref[2,1,0,0]=(-0.727725348725)/(o+1.)+(-0.922906356717)+(0.87708393158)*0.5**o
        ref[2,1,0,1]=(-0.907681129252)/(o+1.)+(0.399455331395)+(-0.192407804254)*0.5**o
        ref[2,1,1,0]=(0.0438148749584)/(o+1.)+(0.366758349324)+(0.153001662465)*0.5**o
        ref[2,1,1,1]=(0.692300008749)/(o+1.)+(0.288310211097)+(0.591374562403)*0.5**o
        ref[2,1,2,0]=(-0.217320783858)/(o+1.)+(0.21589055908)+(-0.742751457969)*0.5**o
        ref[2,1,2,1]=(0.497803852733)/(o+1.)+(0.180166964178)+(-0.834015860027)*0.5**o
        ref[2,2,0,0]=(-0.655435581477)/(o+1.)+(-0.30937321359)+(-0.203653128753)*0.5**o
        ref[2,2,0,1]=(0.140960526316)/(o+1.)+(-0.549930873491)+(0.971807071325)*0.5**o
        ref[2,2,1,0]=(-0.87272068545)/(o+1.)+(-0.159011679037)+(0.36337495614)*0.5**o
        ref[2,2,1,1]=(0.849847759033)/(o+1.)+(-0.132594990815)+(0.703205712066)*0.5**o
        ref[2,2,2,0]=(0.730674735708)/(o+1.)+(-0.144589857416)+(0.814179476293)*0.5**o
        ref[2,2,2,1]=(0.927807005045)/(o+1.)+(0.239143859117)+(0.465577907694)*0.5**o
        ref[2,3,0,0]=(0.737902566912)/(o+1.)+(-0.00945192595853)+(-0.0809501575869)*0.5**o
        ref[2,3,0,1]=(0.688538873306)/(o+1.)+(0.26402367848)+(-0.974260986562)*0.5**o
        ref[2,3,1,0]=(-0.892476705335)/(o+1.)+(0.20490133767)+(0.256567297589)*0.5**o
        ref[2,3,1,1]=(0.724531635238)/(o+1.)+(-0.524187316843)+(0.600281177478)*0.5**o
        ref[2,3,2,0]=(-0.636613154487)/(o+1.)+(0.300668452336)+(-0.316555549763)*0.5**o
        ref[2,3,2,1]=(0.241657689068)/(o+1.)+(-0.217836591241)+(0.509096161125)*0.5**o
        ref[2,4,0,0]=(-0.531592666371)/(o+1.)+(-0.288830707874)+(-0.00190589099569)*0.5**o
        ref[2,4,0,1]=(-0.32109862114)/(o+1.)+(-0.512155326408)+(0.727384767513)*0.5**o
        ref[2,4,1,0]=(0.815200064372)/(o+1.)+(-0.107542955049)+(-0.190511998024)*0.5**o
        ref[2,4,1,1]=(0.323832915733)/(o+1.)+(-0.223614062444)+(0.505564114308)*0.5**o
        ref[2,4,2,0]=(-0.378591546965)/(o+1.)+(0.0569076132008)+(-0.199017289187)*0.5**o
        ref[2,4,2,1]=(0.0866258209859)/(o+1.)+(0.181300307486)+(-0.166203292304)*0.5**o
        ref[3,0,0,0]=(0.42099538494)/(o+1.)+(0.50543930668)+(0.264566793834)*0.5**o
        ref[3,0,0,1]=(-0.566125315231)/(o+1.)+(-0.434053458415)+(0.235746402593)*0.5**o
        ref[3,0,1,0]=(-0.448144780994)/(o+1.)+(-0.316941332636)+(0.862345276865)*0.5**o
        ref[3,0,1,1]=(-0.652497974262)/(o+1.)+(0.267868866775)+(-0.896403179984)*0.5**o
        ref[3,0,2,0]=(-0.455838556745)/(o+1.)+(0.599971979095)+(0.788731583728)*0.5**o
        ref[3,0,2,1]=(0.158791540215)/(o+1.)+(0.510166773251)+(-0.727929054337)*0.5**o
        ref[3,1,0,0]=(-0.0535819231755)/(o+1.)+(-0.166034120633)+(0.349297794591)*0.5**o
        ref[3,1,0,1]=(-0.800973035912)/(o+1.)+(0.220227586048)+(-0.759321371341)*0.5**o
        ref[3,1,1,0]=(0.521881881398)/(o+1.)+(-0.025607129583)+(-0.597928946862)*0.5**o
        ref[3,1,1,1]=(0.445221632359)/(o+1.)+(-0.790614976052)+(-0.263124019379)*0.5**o
        ref[3,1,2,0]=(0.812893197377)/(o+1.)+(-0.550408879848)+(0.434074902924)*0.5**o
        ref[3,1,2,1]=(-0.40676151673)/(o+1.)+(0.21816811252)+(-0.724153308139)*0.5**o
        ref[3,2,0,0]=(-0.88668061529)/(o+1.)+(-0.196434028591)+(0.915215698154)*0.5**o
        ref[3,2,0,1]=(0.453597269106)/(o+1.)+(-0.753778677381)+(-0.456607413762)*0.5**o
        ref[3,2,1,0]=(0.0771381998527)/(o+1.)+(-0.189535168005)+(-0.428848816409)*0.5**o
        ref[3,2,1,1]=(0.786396903927)/(o+1.)+(0.26944712654)+(-0.488115399257)*0.5**o
        ref[3,2,2,0]=(-0.104450005208)/(o+1.)+(-0.480358405144)+(-0.748108272191)*0.5**o
        ref[3,2,2,1]=(-0.107194826319)/(o+1.)+(-0.656924554632)+(-0.288896584199)*0.5**o
        ref[3,3,0,0]=(0.184456668026)/(o+1.)+(0.3262637287)+(-0.643859183999)*0.5**o
        ref[3,3,0,1]=(-0.653836797208)/(o+1.)+(-0.0180862771476)+(-0.258292072759)*0.5**o
        ref[3,3,1,0]=(0.684973413357)/(o+1.)+(0.163588306602)+(-0.659219139805)*0.5**o
        ref[3,3,1,1]=(0.967988326693)/(o+1.)+(0.446278055504)+(-0.649682331365)*0.5**o
        ref[3,3,2,0]=(0.911862994231)/(o+1.)+(-0.14504591934)+(-0.0299199232233)*0.5**o
        ref[3,3,2,1]=(0.121310526207)/(o+1.)+(-0.532981603636)+(0.197795840564)*0.5**o
        ref[3,4,0,0]=(0.144475855023)/(o+1.)+(-0.356456284353)+(0.278568700563)*0.5**o
        ref[3,4,0,1]=(-0.846906570194)/(o+1.)+(0.598840084536)+(-0.608132003655)*0.5**o
        ref[3,4,1,0]=(-0.599983340725)/(o+1.)+(0.338470852121)+(-0.490163043631)*0.5**o
        ref[3,4,1,1]=(-0.466727935821)/(o+1.)+(-0.31358066885)+(-0.802228749858)*0.5**o
        ref[3,4,2,0]=(-0.0529537502248)/(o+1.)+(0.556985122302)+(0.963914397515)*0.5**o
        ref[3,4,2,1]=(0.942559524116)/(o+1.)+(0.029827380831)+(-0.201051432425)*0.5**o
      else:
        s[0,0,0,0]=(-0.345724047005)*x[0]**o+(-0.625686769843)*x[0]+(0.783605206054)*x[1]**o+(0.333976064143)*x[1]+(-0.567769956785)*x[2]**o+(-0.275228142751)*x[2]
        s[0,0,0,1]=(0.867013285813)*x[0]**o+(-0.0918669275484)*x[0]+(-0.162282139173)*x[1]**o+(-0.263245701183)*x[1]+(-0.358346293782)*x[2]**o+(0.170233601974)*x[2]
        s[0,0,1,0]=(-0.450333878814)*x[0]**o+(0.122384873389)*x[0]+(0.590676593091)*x[1]**o+(-0.373630926322)*x[1]+(-0.566894605215)*x[2]**o+(0.172738889453)*x[2]
        s[0,0,1,1]=(0.056183164677)*x[0]**o+(0.30593250486)*x[0]+(0.460330185746)*x[1]**o+(0.0649603857353)*x[1]+(0.530774242761)*x[2]**o+(-0.854200502062)*x[2]
        s[0,0,2,0]=(0.49271509708)*x[0]**o+(-0.822365387684)*x[0]+(0.150952101807)*x[1]**o+(-0.833532543948)*x[1]+(0.223699495078)*x[2]**o+(0.407256610259)*x[2]
        s[0,0,2,1]=(0.827558639052)*x[0]**o+(-0.310957726073)*x[0]+(-0.675858834923)*x[1]**o+(0.902788148659)*x[1]+(-0.569665674677)*x[2]**o+(0.971281178693)*x[2]
        s[0,1,0,0]=(-0.272375589027)*x[0]**o+(-0.285740311002)*x[0]+(0.181997544124)*x[1]**o+(-0.658402113041)*x[1]+(-0.278568799963)*x[2]**o+(-0.41754973361)*x[2]
        s[0,1,0,1]=(0.156364531059)*x[0]**o+(0.813560862368)*x[0]+(0.705700363405)*x[1]**o+(-0.817696115885)*x[1]+(-0.205894570429)*x[2]**o+(0.0674942609577)*x[2]
        s[0,1,1,0]=(0.429893143403)*x[0]**o+(-0.965234368002)*x[0]+(-0.974790265682)*x[1]**o+(0.0680256806875)*x[1]+(-0.433563362185)*x[2]**o+(-0.0492745942903)*x[2]
        s[0,1,1,1]=(-0.298098272504)*x[0]**o+(0.869998704901)*x[0]+(-0.18901211086)*x[1]**o+(-0.51577699091)*x[1]+(-0.328314462204)*x[2]**o+(-0.890845857013)*x[2]
        s[0,1,2,0]=(-0.43467847673)*x[0]**o+(-0.147481947495)*x[0]+(-0.542466755031)*x[1]**o+(-0.495418577682)*x[1]+(-0.2950922494)*x[2]**o+(-0.937573117558)*x[2]
        s[0,1,2,1]=(0.58285783575)*x[0]**o+(-0.902747768997)*x[0]+(-0.639055427646)*x[1]**o+(0.964665499733)*x[1]+(0.797615111504)*x[2]**o+(0.751347091591)*x[2]
        s[0,2,0,0]=(-0.53571726132)*x[0]**o+(-0.26462757313)*x[0]+(-0.456233906167)*x[1]**o+(0.881775957626)*x[1]+(-0.00514091075029)*x[2]**o+(-0.933937418957)*x[2]
        s[0,2,0,1]=(0.651527417402)*x[0]**o+(0.236630784815)*x[0]+(0.932958220915)*x[1]**o+(-0.819866395436)*x[1]+(0.253538518366)*x[2]**o+(-0.0163262782643)*x[2]
        s[0,2,1,0]=(0.452240733592)*x[0]**o+(0.450162809474)*x[0]+(0.699968406858)*x[1]**o+(0.164739887321)*x[1]+(-0.834564901207)*x[2]**o+(0.0249125503308)*x[2]
        s[0,2,1,1]=(-0.227907378794)*x[0]**o+(0.887204637473)*x[0]+(-0.514717641924)*x[1]**o+(0.911423718912)*x[1]+(0.876658247977)*x[2]**o+(-0.371520030414)*x[2]
        s[0,2,2,0]=(0.619773382249)*x[0]**o+(0.541760616441)*x[0]+(0.788185482567)*x[1]**o+(0.0250030620125)*x[1]+(0.163917412766)*x[2]**o+(0.191995043846)*x[2]
        s[0,2,2,1]=(-0.690898841846)*x[0]**o+(-0.708275716521)*x[0]+(-0.164538994288)*x[1]**o+(-0.803752886405)*x[1]+(-0.814464457618)*x[2]**o+(-0.910750603852)*x[2]
        s[0,3,0,0]=(0.752428867688)*x[0]**o+(0.911742295221)*x[0]+(-0.223242212004)*x[1]**o+(-0.757553597062)*x[1]+(-0.473559676565)*x[2]**o+(-0.827199094093)*x[2]
        s[0,3,0,1]=(0.300816669101)*x[0]**o+(-0.199692557045)*x[0]+(0.0825396055332)*x[1]**o+(-0.0693834001394)*x[1]+(-0.382683371052)*x[2]**o+(-0.0699997449306)*x[2]
        s[0,3,1,0]=(-0.370042079378)*x[0]**o+(-0.884622307688)*x[0]+(0.264943777413)*x[1]**o+(0.404099727226)*x[1]+(0.374755570635)*x[2]**o+(0.588241231168)*x[2]
        s[0,3,1,1]=(0.289583633117)*x[0]**o+(0.449941678672)*x[0]+(-0.353365382802)*x[1]**o+(0.515120940717)*x[1]+(0.740445432416)*x[2]**o+(-0.879930463576)*x[2]
        s[0,3,2,0]=(-0.347713282112)*x[0]**o+(-0.162259810338)*x[0]+(-0.696042768288)*x[1]**o+(-0.162202814362)*x[1]+(0.716572044832)*x[2]**o+(0.50377518734)*x[2]
        s[0,3,2,1]=(0.292567121826)*x[0]**o+(-0.76570190516)*x[0]+(0.0867231138296)*x[1]**o+(0.306479775529)*x[1]+(-0.377616255939)*x[2]**o+(0.232957710694)*x[2]
        s[0,4,0,0]=(0.73220485775)*x[0]**o+(-0.159793133992)*x[0]+(-0.377832723802)*x[1]**o+(-0.756658738359)*x[1]+(0.96057353991)*x[2]**o+(0.411157040033)*x[2]
        s[0,4,0,1]=(-0.331390253762)*x[0]**o+(-0.560289084334)*x[0]+(-0.827417830024)*x[1]**o+(0.719380560747)*x[1]+(-0.822147291472)*x[2]**o+(0.860119669443)*x[2]
        s[0,4,1,0]=(-0.505838817009)*x[0]**o+(-0.879628389565)*x[0]+(0.488746892572)*x[1]**o+(0.613837247188)*x[1]+(-0.154303229874)*x[2]**o+(0.00570279899539)*x[2]
        s[0,4,1,1]=(-0.122955617948)*x[0]**o+(-0.953071253159)*x[0]+(0.844003480214)*x[1]**o+(-0.746851086453)*x[1]+(-0.108832545758)*x[2]**o+(0.434183611414)*x[2]
        s[0,4,2,0]=(0.789617751574)*x[0]**o+(0.760644604676)*x[0]+(0.909793043646)*x[1]**o+(-0.279682873326)*x[1]+(0.700690761413)*x[2]**o+(-0.760237637314)*x[2]
        s[0,4,2,1]=(-0.882408865196)*x[0]**o+(0.328073069067)*x[0]+(0.858835691096)*x[1]**o+(0.650470359577)*x[1]+(-0.8717493079)*x[2]**o+(-0.177563387753)*x[2]
        s[1,0,0,0]=(-0.0500679538311)*x[0]**o+(-0.137932236517)*x[0]+(-0.742651074891)*x[1]**o+(-0.427978749091)*x[1]+(0.0361372113862)*x[2]**o+(-0.558139606192)*x[2]
        s[1,0,0,1]=(-0.00386321275167)*x[0]**o+(-0.51227499671)*x[0]+(-0.295080251306)*x[1]**o+(-0.326209704618)*x[1]+(-0.705968952206)*x[2]**o+(-0.848585657427)*x[2]
        s[1,0,1,0]=(0.173864935883)*x[0]**o+(0.148289460355)*x[0]+(0.40615421494)*x[1]**o+(-0.715315862741)*x[1]+(0.455318023779)*x[2]**o+(-0.539497210174)*x[2]
        s[1,0,1,1]=(-0.0628270158133)*x[0]**o+(-0.659218788396)*x[0]+(-0.0956661811574)*x[1]**o+(0.980329463956)*x[1]+(0.0114144009766)*x[2]**o+(-0.827594275845)*x[2]
        s[1,0,2,0]=(0.696208347195)*x[0]**o+(0.0807576571591)*x[0]+(0.998089746929)*x[1]**o+(0.177352400068)*x[1]+(0.685074203515)*x[2]**o+(-0.0635353733634)*x[2]
        s[1,0,2,1]=(-0.0223157590993)*x[0]**o+(0.559991238653)*x[0]+(0.548345826656)*x[1]**o+(-0.741886839548)*x[1]+(0.114218185947)*x[2]**o+(0.825638317499)*x[2]
        s[1,1,0,0]=(0.163206474973)*x[0]**o+(0.254896515732)*x[0]+(0.447114620799)*x[1]**o+(-0.563952237815)*x[1]+(0.57961770388)*x[2]**o+(0.0819856064889)*x[2]
        s[1,1,0,1]=(-0.328504704157)*x[0]**o+(0.0868460228579)*x[0]+(0.816944360875)*x[1]**o+(-0.697748850943)*x[1]+(-0.809725796641)*x[2]**o+(0.932721918575)*x[2]
        s[1,1,1,0]=(-0.287904638148)*x[0]**o+(-0.013943513773)*x[0]+(-0.54196693436)*x[1]**o+(-0.388084049637)*x[1]+(-0.702953342435)*x[2]**o+(-0.830664911774)*x[2]
        s[1,1,1,1]=(0.684091346652)*x[0]**o+(0.806375097652)*x[0]+(0.476003712147)*x[1]**o+(-0.864998508113)*x[1]+(-0.497254896777)*x[2]**o+(0.781872111446)*x[2]
        s[1,1,2,0]=(-0.844264089035)*x[0]**o+(-0.679014107829)*x[0]+(-0.198479658612)*x[1]**o+(0.375191314697)*x[1]+(-0.575465869392)*x[2]**o+(0.530364062529)*x[2]
        s[1,1,2,1]=(0.676323957189)*x[0]**o+(-0.501144179827)*x[0]+(-0.962137734649)*x[1]**o+(-0.356432667657)*x[1]+(-0.940336010235)*x[2]**o+(-0.236428299429)*x[2]
        s[1,2,0,0]=(0.355428925779)*x[0]**o+(0.274208957908)*x[0]+(-0.932593631234)*x[1]**o+(-0.376131921686)*x[1]+(0.286983572479)*x[2]**o+(-0.330880385128)*x[2]
        s[1,2,0,1]=(0.266570372333)*x[0]**o+(0.921388865301)*x[0]+(-0.1720476903)*x[1]**o+(0.998284527237)*x[1]+(0.0238473892294)*x[2]**o+(0.854216047199)*x[2]
        s[1,2,1,0]=(0.441558473549)*x[0]**o+(0.373044064317)*x[0]+(0.649370981178)*x[1]**o+(0.510194580982)*x[1]+(0.371823975996)*x[2]**o+(0.665301207824)*x[2]
        s[1,2,1,1]=(0.754004050836)*x[0]**o+(0.50557389845)*x[0]+(-0.251967535648)*x[1]**o+(0.704262653029)*x[1]+(0.645159509055)*x[2]**o+(0.183884331443)*x[2]
        s[1,2,2,0]=(-0.0673953798558)*x[0]**o+(0.708470323283)*x[0]+(-0.817934961085)*x[1]**o+(0.270394037097)*x[1]+(0.761348716495)*x[2]**o+(0.265483378874)*x[2]
        s[1,2,2,1]=(0.569853744398)*x[0]**o+(0.612861422857)*x[0]+(0.655472955979)*x[1]**o+(-0.387384090661)*x[1]+(0.026712723359)*x[2]**o+(0.504626498333)*x[2]
        s[1,3,0,0]=(-0.706312246902)*x[0]**o+(-0.309993808911)*x[0]+(0.25462597295)*x[1]**o+(-0.709061516294)*x[1]+(0.886706821135)*x[2]**o+(0.946960487449)*x[2]
        s[1,3,0,1]=(0.315339476547)*x[0]**o+(0.706694261286)*x[0]+(0.536483540093)*x[1]**o+(0.420431748645)*x[1]+(-0.401936712588)*x[2]**o+(-0.0303951289428)*x[2]
        s[1,3,1,0]=(-0.830759344331)*x[0]**o+(-0.0138001332539)*x[0]+(0.684117354857)*x[1]**o+(-0.521483470147)*x[1]+(0.261449467808)*x[2]**o+(0.81578910141)*x[2]
        s[1,3,1,1]=(0.218558430087)*x[0]**o+(0.117511078171)*x[0]+(-0.367757102068)*x[1]**o+(-0.301875631785)*x[1]+(-0.933013656177)*x[2]**o+(0.517230809269)*x[2]
        s[1,3,2,0]=(-0.273720099467)*x[0]**o+(-0.00869721580633)*x[0]+(-0.796835310998)*x[1]**o+(0.587901955041)*x[1]+(-0.664296869655)*x[2]**o+(-0.896980337429)*x[2]
        s[1,3,2,1]=(-0.068913139717)*x[0]**o+(0.656400342524)*x[0]+(0.855587799259)*x[1]**o+(0.876438035564)*x[1]+(-0.0622439736181)*x[2]**o+(-0.0977398744709)*x[2]
        s[1,4,0,0]=(0.427481073197)*x[0]**o+(0.20467745629)*x[0]+(-0.125277072461)*x[1]**o+(0.274783610604)*x[1]+(-0.384842532213)*x[2]**o+(0.790482142388)*x[2]
        s[1,4,0,1]=(0.723515401941)*x[0]**o+(-0.0529425602105)*x[0]+(-0.818443886494)*x[1]**o+(0.786875708369)*x[1]+(-0.747927140569)*x[2]**o+(-0.731906020242)*x[2]
        s[1,4,1,0]=(0.9774352561)*x[0]**o+(0.0769002693408)*x[0]+(-0.576964776525)*x[1]**o+(-0.313768570439)*x[1]+(-0.896910128278)*x[2]**o+(-0.280920538425)*x[2]
        s[1,4,1,1]=(0.138312702528)*x[0]**o+(-0.458814456032)*x[0]+(0.163241743373)*x[1]**o+(-0.803420897174)*x[1]+(-0.140886706823)*x[2]**o+(-0.14390608176)*x[2]
        s[1,4,2,0]=(0.529048827233)*x[0]**o+(-0.0554100800212)*x[0]+(0.456015489421)*x[1]**o+(0.422262108301)*x[1]+(-0.313743004281)*x[2]**o+(-0.13975864585)*x[2]
        s[1,4,2,1]=(0.838760174947)*x[0]**o+(-0.223195641306)*x[0]+(0.191058937589)*x[1]**o+(-0.0383799908705)*x[1]+(-0.123273771781)*x[2]**o+(0.375902823133)*x[2]
        s[2,0,0,0]=(-0.376325501997)*x[0]**o+(-0.999695820134)*x[0]+(0.145774433039)*x[1]**o+(0.0848795062374)*x[1]+(-0.789276953104)*x[2]**o+(-0.936604606234)*x[2]
        s[2,0,0,1]=(-0.393633363986)*x[0]**o+(0.122599807309)*x[0]+(-0.308578856774)*x[1]**o+(-0.123177461702)*x[1]+(-0.135481435454)*x[2]**o+(0.583777567706)*x[2]
        s[2,0,1,0]=(-0.573207876072)*x[0]**o+(-0.194869976915)*x[0]+(0.611265046543)*x[1]**o+(0.218225055561)*x[1]+(-0.88953645988)*x[2]**o+(0.314028438499)*x[2]
        s[2,0,1,1]=(0.82764053208)*x[0]**o+(0.408293655023)*x[0]+(-0.158898571521)*x[1]**o+(0.557257506806)*x[1]+(0.22438907333)*x[2]**o+(-0.616052713022)*x[2]
        s[2,0,2,0]=(0.680002851599)*x[0]**o+(-0.833669903036)*x[0]+(0.0182560742948)*x[1]**o+(-0.401224011261)*x[1]+(-0.601050104604)*x[2]**o+(0.354512589668)*x[2]
        s[2,0,2,1]=(-0.99533923296)*x[0]**o+(-0.252792558615)*x[0]+(-0.417436606414)*x[1]**o+(-0.905526797068)*x[1]+(-0.323384139739)*x[2]**o+(0.217030918198)*x[2]
        s[2,1,0,0]=(-0.757427161409)*x[0]**o+(-0.0463730910958)*x[0]+(0.44512589496)*x[1]**o+(0.545180882409)*x[1]+(0.78683574265)*x[2]**o+(-0.684406722817)*x[2]
        s[2,1,0,1]=(-0.61819592214)*x[0]**o+(-0.797609085554)*x[0]+(0.983105716733)*x[1]**o+(0.322191127831)*x[1]+(0.453981669947)*x[2]**o+(-0.909192167062)*x[2]
        s[2,1,1,0]=(-0.898119938157)*x[0]**o+(0.311855250372)*x[0]+(0.505532351296)*x[1]**o+(0.139215174265)*x[1]+(0.772001036932)*x[2]**o+(-0.225670248604)*x[2]
        s[2,1,1,1]=(0.0316457581752)*x[0]**o+(-0.488943159802)*x[0]+(-0.773453487185)*x[1]**o+(0.815697082082)*x[1]+(0.206189975058)*x[2]**o+(-0.575428943874)*x[2]
        s[2,1,2,0]=(0.319688798055)*x[0]**o+(-0.0695712382178)*x[0]+(0.689876574974)*x[1]**o+(0.190028248325)*x[1]+(-0.766718072759)*x[2]**o+(-0.52180809158)*x[2]
        s[2,1,2,1]=(-0.865578912594)*x[0]**o+(0.460194992238)*x[0]+(-0.0025880450282)*x[1]**o+(-0.400524494504)*x[1]+(0.417535391073)*x[2]**o+(-0.0342670929868)*x[2]
        s[2,2,0,0]=(0.421599950602)*x[0]**o+(0.266747809147)*x[0]+(-0.169043748241)*x[1]**o+(-0.164542032135)*x[1]+(0.555070565368)*x[2]**o+(-0.0217786140345)*x[2]
        s[2,2,0,1]=(-0.625527951515)*x[0]**o+(-0.998518258266)*x[0]+(-0.0978678841265)*x[1]**o+(0.00726403245479)*x[1]+(-0.210296926513)*x[2]**o+(0.0169357807292)*x[2]
        s[2,2,1,0]=(-0.927097383803)*x[0]**o+(0.142344498674)*x[0]+(-0.565097374968)*x[1]**o+(-0.19883325806)*x[1]+(-0.409577428899)*x[2]**o+(-0.433045796687)*x[2]
        s[2,2,1,1]=(0.0900116992509)*x[0]**o+(0.907232745809)*x[0]+(0.164351943946)*x[1]**o+(0.831012029454)*x[1]+(0.659298356625)*x[2]**o+(0.658166177134)*x[2]
        s[2,2,2,0]=(-0.468765776872)*x[0]**o+(-0.56578531156)*x[0]+(-0.515309324035)*x[1]**o+(-0.891192406798)*x[1]+(-0.229751474081)*x[2]**o+(0.861043357121)*x[2]
        s[2,2,2,1]=(-0.102463609474)*x[0]**o+(0.795582169816)*x[0]+(0.473799689371)*x[1]**o+(-0.624789163959)*x[1]+(-0.120758299241)*x[2]**o+(-0.790110725507)*x[2]
        s[2,3,0,0]=(-0.61205272187)*x[0]**o+(-0.40060893961)*x[0]+(-0.789664972057)*x[1]**o+(0.515295029759)*x[1]+(-0.0475219131724)*x[2]**o+(-0.148006363455)*x[2]
        s[2,3,0,1]=(-0.866234572657)*x[0]**o+(-0.980788916374)*x[0]+(0.110830262867)*x[1]**o+(0.76938462126)*x[1]+(0.338028197458)*x[2]**o+(0.695510468586)*x[2]
        s[2,3,1,0]=(-0.61195493774)*x[0]**o+(0.418983087999)*x[0]+(0.698112113863)*x[1]**o+(-0.961155169263)*x[1]+(-0.237690733199)*x[2]**o+(0.160246107308)*x[2]
        s[2,3,1,1]=(0.186790790362)*x[0]**o+(0.726925157226)*x[0]+(-0.876208560721)*x[1]**o+(0.684333403771)*x[1]+(0.859789784415)*x[2]**o+(0.180765658997)*x[2]
        s[2,3,2,0]=(-0.107447528726)*x[0]**o+(0.23776834378)*x[0]+(0.203151310439)*x[1]**o+(0.939282834891)*x[1]+(-0.739341306594)*x[2]**o+(0.526784359658)*x[2]
        s[2,3,2,1]=(-0.659583948903)*x[0]**o+(-0.876524913083)*x[0]+(-0.140502393442)*x[1]**o+(0.220988067839)*x[1]+(-0.315594035706)*x[2]**o+(-0.337007362379)*x[2]
        s[2,4,0,0]=(0.824520364974)*x[0]**o+(-0.0700628701349)*x[0]+(-0.337899096069)*x[1]**o+(-0.969230762171)*x[1]+(0.451483693063)*x[2]**o+(0.560376952474)*x[2]
        s[2,4,0,1]=(0.19676925667)*x[0]**o+(-0.509702667102)*x[0]+(-0.560085912439)*x[1]**o+(0.850131177282)*x[1]+(0.0209473113715)*x[2]**o+(-0.524835181986)*x[2]
        s[2,4,1,0]=(0.696459080737)*x[0]**o+(-0.0300420621411)*x[0]+(0.872376074883)*x[1]**o+(0.721784659413)*x[1]+(0.0469660828352)*x[2]**o+(0.848375867902)*x[2]
        s[2,4,1,1]=(-0.0300055784957)*x[0]**o+(0.86110892248)*x[0]+(-0.0372850528438)*x[1]**o+(-0.626675690839)*x[1]+(-0.96746918975)*x[2]**o+(-0.118288468609)*x[2]
        s[2,4,2,0]=(-0.773844175596)*x[0]**o+(0.97042325748)*x[0]+(-0.709105396658)*x[1]**o+(-0.436388373122)*x[1]+(-0.841519653893)*x[2]**o+(-0.661541598363)*x[2]
        s[2,4,2,1]=(-0.289904092026)*x[0]**o+(0.838224779606)*x[0]+(0.272732151891)*x[1]**o+(-0.789836453998)*x[1]+(0.574134239472)*x[2]**o+(-0.196980303288)*x[2]
        s[3,0,0,0]=(0.921207531349)*x[0]**o+(-0.944820675428)*x[0]+(-0.632934426248)*x[1]**o+(-0.986209890868)*x[1]+(0.889736953978)*x[2]**o+(0.308722225546)*x[2]
        s[3,0,0,1]=(-0.349795723275)*x[0]**o+(-0.748469735322)*x[0]+(-0.581786511254)*x[1]**o+(0.696378145052)*x[1]+(0.619384687507)*x[2]**o+(-0.0953346757601)*x[2]
        s[3,0,1,0]=(0.539616405472)*x[0]**o+(-0.265315341201)*x[0]+(0.042248150432)*x[1]**o+(-0.115847796714)*x[1]+(-0.217838196299)*x[2]**o+(-0.0631408906436)*x[2]
        s[3,0,1,1]=(-0.823978915895)*x[0]**o+(-0.608867256385)*x[0]+(0.446515851959)*x[1]**o+(0.531432611215)*x[1]+(0.689019079647)*x[2]**o+(-0.0317787775133)*x[2]
        s[3,0,2,0]=(0.347425369617)*x[0]**o+(-0.739691856701)*x[0]+(-0.398130284368)*x[1]**o+(0.108805616885)*x[1]+(-0.0912940693547)*x[2]**o+(0.117214641864)*x[2]
        s[3,0,2,1]=(-0.383805789854)*x[0]**o+(-0.0456422576844)*x[0]+(0.533382056608)*x[1]**o+(-0.218226601069)*x[1]+(-0.676716489399)*x[2]**o+(-0.00199666729435)*x[2]
        s[3,1,0,0]=(-0.372478384483)*x[0]**o+(0.586299542288)*x[0]+(-0.454391251107)*x[1]**o+(0.0882830261313)*x[1]+(-0.623432789013)*x[2]**o+(0.218598634148)*x[2]
        s[3,1,0,1]=(0.578294223191)*x[0]**o+(-0.355620351739)*x[0]+(0.231898319431)*x[1]**o+(0.423862975521)*x[1]+(0.646375025433)*x[2]**o+(-0.213634599835)*x[2]
        s[3,1,1,0]=(0.0525838616186)*x[0]**o+(0.118311802828)*x[0]+(0.548544020336)*x[1]**o+(0.584362580997)*x[1]+(-0.234381266126)*x[2]**o+(-0.845553660597)*x[2]
        s[3,1,1,1]=(-0.677671170669)*x[0]**o+(-0.290465667864)*x[0]+(-0.0167835285288)*x[1]**o+(-0.100024088985)*x[1]+(-0.150243328686)*x[2]**o+(0.679119492133)*x[2]
        s[3,1,2,0]=(-0.121237844262)*x[0]**o+(-0.458196712924)*x[0]+(0.963853129183)*x[1]**o+(-0.211094701554)*x[1]+(0.868445433817)*x[2]**o+(0.902283926176)*x[2]
        s[3,1,2,1]=(0.504161603066)*x[0]**o+(-0.190472686354)*x[0]+(-0.0917827193891)*x[1]**o+(-0.352839230228)*x[1]+(-0.353813861163)*x[2]**o+(0.230960874753)*x[2]
        s[3,2,0,0]=(-0.34790984999)*x[0]**o+(-0.395507825603)*x[0]+(-0.6489230899)*x[1]**o+(0.712401749209)*x[1]+(-0.0821476018477)*x[2]**o+(0.690431614802)*x[2]
        s[3,2,0,1]=(-0.557444127335)*x[0]**o+(-0.834050224977)*x[0]+(-0.22666297073)*x[1]**o+(0.405731464147)*x[1]+(-0.671211465442)*x[2]**o+(-0.644382670803)*x[2]
        s[3,2,1,0]=(0.965420676359)*x[0]**o+(-0.228764004519)*x[0]+(-0.859916958223)*x[1]**o+(-0.452756903956)*x[1]+(-0.973849033231)*x[2]**o+(-0.24133735483)*x[2]
        s[3,2,1,1]=(-0.950755375408)*x[0]**o+(-0.851938171221)*x[0]+(0.0181318395336)*x[1]**o+(-0.499474780924)*x[1]+(0.910360613628)*x[2]**o+(0.199314643801)*x[2]
        s[3,2,2,0]=(0.339657614739)*x[0]**o+(0.145101826319)*x[0]+(0.0477774162796)*x[1]**o+(-0.883319372262)*x[1]+(0.297676781475)*x[2]**o+(0.375455002664)*x[2]
        s[3,2,2,1]=(0.427508981997)*x[0]**o+(0.735804891833)*x[0]+(-0.941349025764)*x[1]**o+(-0.848431716981)*x[1]+(0.878538549993)*x[2]**o+(0.0726603284636)*x[2]
        s[3,3,0,0]=(-0.973352015556)*x[0]**o+(0.836731359075)*x[0]+(0.665815761101)*x[1]**o+(0.255531920855)*x[1]+(0.782674767576)*x[2]**o+(-0.570574483199)*x[2]
        s[3,3,0,1]=(0.950365830796)*x[0]**o+(0.421946061345)*x[0]+(0.905170182227)*x[1]**o+(0.613401792858)*x[1]+(-0.442076185783)*x[2]**o+(-0.484925057701)*x[2]
        s[3,3,1,0]=(-0.00222827033475)*x[0]**o+(-0.531682094474)*x[0]+(0.458841364479)*x[1]**o+(-0.655184028959)*x[1]+(-0.22086141276)*x[2]**o+(-0.813839980108)*x[2]
        s[3,3,1,1]=(0.887884370757)*x[0]**o+(-0.0538573388546)*x[0]+(0.927878694132)*x[1]**o+(0.471741834858)*x[1]+(-0.313646320846)*x[2]**o+(0.311547715614)*x[2]
        s[3,3,2,0]=(-0.0214623472154)*x[0]**o+(-0.444460344328)*x[0]+(0.0788294611308)*x[1]**o+(0.41576519959)*x[1]+(-0.778008962903)*x[2]**o+(0.900765472759)*x[2]
        s[3,3,2,1]=(0.949491478772)*x[0]**o+(-0.471010680081)*x[0]+(-0.876305371565)*x[1]**o+(-0.120079570893)*x[1]+(0.0947323737758)*x[2]**o+(-0.361392549886)*x[2]
        s[3,4,0,0]=(0.354489762998)*x[0]**o+(-0.425422633745)*x[0]+(-0.566692257139)*x[1]**o+(0.504748998989)*x[1]+(0.226102951066)*x[2]**o+(-0.744917446958)*x[2]
        s[3,4,0,1]=(0.552196121696)*x[0]**o+(0.533127640573)*x[0]+(-0.147082768815)*x[1]**o+(-0.801997447891)*x[1]+(0.104711977816)*x[2]**o+(0.99454944575)*x[2]
        s[3,4,1,0]=(0.839982844034)*x[0]**o+(-0.779008776044)*x[0]+(-0.561078679707)*x[1]**o+(-0.0519865292547)*x[1]+(-0.566301067503)*x[2]**o+(-0.267957119508)*x[2]
        s[3,4,1,1]=(0.78962834926)*x[0]**o+(-0.93892858306)*x[0]+(-0.114227870549)*x[1]**o+(0.420485886976)*x[1]+(0.974178392694)*x[2]**o+(-0.421944523239)*x[2]
        s[3,4,2,0]=(-0.882936934355)*x[0]**o+(-0.441556030186)*x[0]+(0.0681795049997)*x[1]**o+(0.731317357905)*x[1]+(-0.965801964833)*x[2]**o+(-0.319270469244)*x[2]
        s[3,4,2,1]=(0.342579062975)*x[0]**o+(0.23386981431)*x[0]+(-0.571662448793)*x[1]**o+(-0.257739898874)*x[1]+(0.176906895006)*x[2]**o+(0.892856914788)*x[2]
        ref[0,0,0,0]=(0.215835249268)/(o+1.)+(-0.283469424226)+(-0.345724047005)*0.5**o
        ref[0,0,0,1]=(-0.520628432955)/(o+1.)+(-0.0924395133784)+(0.867013285813)*0.5**o
        ref[0,0,1,0]=(0.0237819878764)/(o+1.)+(-0.0392535817398)+(-0.450333878814)*0.5**o
        ref[0,0,1,1]=(0.991104428507)/(o+1.)+(-0.241653805734)+(0.056183164677)*0.5**o
        ref[0,0,2,0]=(0.374651596885)/(o+1.)+(-0.624320660687)+(0.49271509708)*0.5**o
        ref[0,0,2,1]=(-1.2455245096)/(o+1.)+(0.781555800639)+(0.827558639052)*0.5**o
        ref[0,1,0,0]=(-0.0965712558395)/(o+1.)+(-0.680846078827)+(-0.272375589027)*0.5**o
        ref[0,1,0,1]=(0.499805792977)/(o+1.)+(0.0316795037204)+(0.156364531059)*0.5**o
        ref[0,1,1,0]=(-1.40835362787)/(o+1.)+(-0.473241640802)+(0.429893143403)*0.5**o
        ref[0,1,1,1]=(-0.517326573063)/(o+1.)+(-0.268312071511)+(-0.298098272504)*0.5**o
        ref[0,1,2,0]=(-0.837559004431)/(o+1.)+(-0.790236821368)+(-0.43467847673)*0.5**o
        ref[0,1,2,1]=(0.158559683859)/(o+1.)+(0.406632411163)+(0.58285783575)*0.5**o
        ref[0,2,0,0]=(-0.461374816917)/(o+1.)+(-0.158394517231)+(-0.53571726132)*0.5**o
        ref[0,2,0,1]=(1.18649673928)/(o+1.)+(-0.299780944442)+(0.651527417402)*0.5**o
        ref[0,2,1,0]=(-0.13459649435)/(o+1.)+(0.319907623563)+(0.452240733592)*0.5**o
        ref[0,2,1,1]=(0.361940606053)/(o+1.)+(0.713554162985)+(-0.227907378794)*0.5**o
        ref[0,2,2,0]=(0.952102895333)/(o+1.)+(0.37937936115)+(0.619773382249)*0.5**o
        ref[0,2,2,1]=(-0.979003451906)/(o+1.)+(-1.21138960339)+(-0.690898841846)*0.5**o
        ref[0,3,0,0]=(-0.696801888569)/(o+1.)+(-0.336505197967)+(0.752428867688)*0.5**o
        ref[0,3,0,1]=(-0.300143765518)/(o+1.)+(-0.169537851057)+(0.300816669101)*0.5**o
        ref[0,3,1,0]=(0.639699348048)/(o+1.)+(0.0538593253528)+(-0.370042079378)*0.5**o
        ref[0,3,1,1]=(0.387080049614)/(o+1.)+(0.0425660779061)+(0.289583633117)*0.5**o
        ref[0,3,2,0]=(0.0205292765437)/(o+1.)+(0.0896562813201)+(-0.347713282112)*0.5**o
        ref[0,3,2,1]=(-0.29089314211)/(o+1.)+(-0.113132209469)+(0.292567121826)*0.5**o
        ref[0,4,0,0]=(0.582740816108)/(o+1.)+(-0.252647416159)+(0.73220485775)*0.5**o
        ref[0,4,0,1]=(-1.6495651215)/(o+1.)+(0.509605572928)+(-0.331390253762)*0.5**o
        ref[0,4,1,0]=(0.334443662697)/(o+1.)+(-0.130044171691)+(-0.505838817009)*0.5**o
        ref[0,4,1,1]=(0.735170934456)/(o+1.)+(-0.632869364099)+(-0.122955617948)*0.5**o
        ref[0,4,2,0]=(1.61048380506)/(o+1.)+(-0.139637952982)+(0.789617751574)*0.5**o
        ref[0,4,2,1]=(-0.012913616804)/(o+1.)+(0.400490020446)+(-0.882408865196)*0.5**o
        ref[1,0,0,0]=(-0.706513863505)/(o+1.)+(-0.5620252959)+(-0.0500679538311)*0.5**o
        ref[1,0,0,1]=(-1.00104920351)/(o+1.)+(-0.843535179378)+(-0.00386321275167)*0.5**o
        ref[1,0,1,0]=(0.861472238719)/(o+1.)+(-0.55326180628)+(0.173864935883)*0.5**o
        ref[1,0,1,1]=(-0.0842517801808)/(o+1.)+(-0.253241800142)+(-0.0628270158133)*0.5**o
        ref[1,0,2,0]=(1.68316395044)/(o+1.)+(0.097287341932)+(0.696208347195)*0.5**o
        ref[1,0,2,1]=(0.662564012602)/(o+1.)+(0.321871358302)+(-0.0223157590993)*0.5**o
        ref[1,1,0,0]=(1.02673232468)/(o+1.)+(-0.113535057797)+(0.163206474973)*0.5**o
        ref[1,1,0,1]=(0.00721856423403)/(o+1.)+(0.160909545245)+(-0.328504704157)*0.5**o
        ref[1,1,1,0]=(-1.24492027679)/(o+1.)+(-0.616346237592)+(-0.287904638148)*0.5**o
        ref[1,1,1,1]=(-0.0212511846297)/(o+1.)+(0.361624350493)+(0.684091346652)*0.5**o
        ref[1,1,2,0]=(-0.773945528004)/(o+1.)+(0.113270634698)+(-0.844264089035)*0.5**o
        ref[1,1,2,1]=(-1.90247374488)/(o+1.)+(-0.547002573456)+(0.676323957189)*0.5**o
        ref[1,2,0,0]=(-0.645610058755)/(o+1.)+(-0.216401674453)+(0.355428925779)*0.5**o
        ref[1,2,0,1]=(-0.148200301071)/(o+1.)+(1.38694471987)+(0.266570372333)*0.5**o
        ref[1,2,1,0]=(1.02119495717)/(o+1.)+(0.774269926562)+(0.441558473549)*0.5**o
        ref[1,2,1,1]=(0.393191973407)/(o+1.)+(0.696860441461)+(0.754004050836)*0.5**o
        ref[1,2,2,0]=(-0.0565862445904)/(o+1.)+(0.622173869627)+(-0.0673953798558)*0.5**o
        ref[1,2,2,1]=(0.682185679338)/(o+1.)+(0.365051915264)+(0.569853744398)*0.5**o
        ref[1,3,0,0]=(1.14133279409)/(o+1.)+(-0.0360474188784)+(-0.706312246902)*0.5**o
        ref[1,3,0,1]=(0.134546827505)/(o+1.)+(0.548365440494)+(0.315339476547)*0.5**o
        ref[1,3,1,0]=(0.945566822665)/(o+1.)+(0.140252749005)+(-0.830759344331)*0.5**o
        ref[1,3,1,1]=(-1.30077075824)/(o+1.)+(0.166433127827)+(0.218558430087)*0.5**o
        ref[1,3,2,0]=(-1.46113218065)/(o+1.)+(-0.158887799097)+(-0.273720099467)*0.5**o
        ref[1,3,2,1]=(0.793343825641)/(o+1.)+(0.717549251808)+(-0.068913139717)*0.5**o
        ref[1,4,0,0]=(-0.510119604674)/(o+1.)+(0.634971604641)+(0.427481073197)*0.5**o
        ref[1,4,0,1]=(-1.56637102706)/(o+1.)+(0.00101356395823)+(0.723515401941)*0.5**o
        ref[1,4,1,0]=(-1.4738749048)/(o+1.)+(-0.258894419762)+(0.9774352561)*0.5**o
        ref[1,4,1,1]=(0.0223550365494)/(o+1.)+(-0.703070717483)+(0.138312702528)*0.5**o
        ref[1,4,2,0]=(0.14227248514)/(o+1.)+(0.113546691215)+(0.529048827233)*0.5**o
        ref[1,4,2,1]=(0.0677851658086)/(o+1.)+(0.0571635954781)+(0.838760174947)*0.5**o
        ref[2,0,0,0]=(-0.643502520065)/(o+1.)+(-0.925710460066)+(-0.376325501997)*0.5**o
        ref[2,0,0,1]=(-0.444060292228)/(o+1.)+(0.291599956656)+(-0.393633363986)*0.5**o
        ref[2,0,1,0]=(-0.278271413338)/(o+1.)+(0.168691758573)+(-0.573207876072)*0.5**o
        ref[2,0,1,1]=(0.0654905018086)/(o+1.)+(0.174749224404)+(0.82764053208)*0.5**o
        ref[2,0,2,0]=(-0.582794030309)/(o+1.)+(-0.440190662315)+(0.680002851599)*0.5**o
        ref[2,0,2,1]=(-0.740820746153)/(o+1.)+(-0.470644218743)+(-0.99533923296)*0.5**o
        ref[2,1,0,0]=(1.23196163761)/(o+1.)+(-0.0927994657521)+(-0.757427161409)*0.5**o
        ref[2,1,0,1]=(1.43708738668)/(o+1.)+(-0.692305062393)+(-0.61819592214)*0.5**o
        ref[2,1,1,0]=(1.27753338823)/(o+1.)+(0.112700088017)+(-0.898119938157)*0.5**o
        ref[2,1,1,1]=(-0.567263512127)/(o+1.)+(-0.124337510797)+(0.0316457581752)*0.5**o
        ref[2,1,2,0]=(-0.0768414977849)/(o+1.)+(-0.200675540737)+(0.319688798055)*0.5**o
        ref[2,1,2,1]=(0.414947346045)/(o+1.)+(0.0127017023739)+(-0.865578912594)*0.5**o
        ref[2,2,0,0]=(0.386026817127)/(o+1.)+(0.0402135814888)+(0.421599950602)*0.5**o
        ref[2,2,0,1]=(-0.30816481064)/(o+1.)+(-0.487159222541)+(-0.625527951515)*0.5**o
        ref[2,2,1,0]=(-0.974674803867)/(o+1.)+(-0.244767278037)+(-0.927097383803)*0.5**o
        ref[2,2,1,1]=(0.82365030057)/(o+1.)+(1.1982054762)+(0.0900116992509)*0.5**o
        ref[2,2,2,0]=(-0.745060798116)/(o+1.)+(-0.297967180618)+(-0.468765776872)*0.5**o
        ref[2,2,2,1]=(0.353041390129)/(o+1.)+(-0.309658859825)+(-0.102463609474)*0.5**o
        ref[2,3,0,0]=(-0.83718688523)/(o+1.)+(-0.0166601366533)+(-0.61205272187)*0.5**o
        ref[2,3,0,1]=(0.448858460325)/(o+1.)+(0.242053086736)+(-0.866234572657)*0.5**o
        ref[2,3,1,0]=(0.460421380665)/(o+1.)+(-0.190962986978)+(-0.61195493774)*0.5**o
        ref[2,3,1,1]=(-0.0164187763059)/(o+1.)+(0.796012109997)+(0.186790790362)*0.5**o
        ref[2,3,2,0]=(-0.536189996155)/(o+1.)+(0.851917769165)+(-0.107447528726)*0.5**o
        ref[2,3,2,1]=(-0.456096429148)/(o+1.)+(-0.496272103811)+(-0.659583948903)*0.5**o
        ref[2,4,0,0]=(0.113584596994)/(o+1.)+(-0.239458339916)+(0.824520364974)*0.5**o
        ref[2,4,0,1]=(-0.539138601067)/(o+1.)+(-0.0922033359031)+(0.19676925667)*0.5**o
        ref[2,4,1,0]=(0.919342157718)/(o+1.)+(0.770059232587)+(0.696459080737)*0.5**o
        ref[2,4,1,1]=(-1.00475424259)/(o+1.)+(0.0580723815159)+(-0.0300055784957)*0.5**o
        ref[2,4,2,0]=(-1.55062505055)/(o+1.)+(-0.0637533570027)+(-0.773844175596)*0.5**o
        ref[2,4,2,1]=(0.846866391363)/(o+1.)+(-0.0742959888403)+(-0.289904092026)*0.5**o
        ref[3,0,0,0]=(0.25680252773)/(o+1.)+(-0.811154170375)+(0.921207531349)*0.5**o
        ref[3,0,0,1]=(0.037598176253)/(o+1.)+(-0.0737131330151)+(-0.349795723275)*0.5**o
        ref[3,0,1,0]=(-0.175590045867)/(o+1.)+(-0.222152014279)+(0.539616405472)*0.5**o
        ref[3,0,1,1]=(1.13553493161)/(o+1.)+(-0.0546067113414)+(-0.823978915895)*0.5**o
        ref[3,0,2,0]=(-0.489424353723)/(o+1.)+(-0.256835798975)+(0.347425369617)*0.5**o
        ref[3,0,2,1]=(-0.143334432791)/(o+1.)+(-0.132932763024)+(-0.383805789854)*0.5**o
        ref[3,1,0,0]=(-1.07782404012)/(o+1.)+(0.446590601283)+(-0.372478384483)*0.5**o
        ref[3,1,0,1]=(0.878273344865)/(o+1.)+(-0.0726959880264)+(0.578294223191)*0.5**o
        ref[3,1,1,0]=(0.314162754211)/(o+1.)+(-0.0714396383864)+(0.0525838616186)*0.5**o
        ref[3,1,1,1]=(-0.167026857215)/(o+1.)+(0.144314867642)+(-0.677671170669)*0.5**o
        ref[3,1,2,0]=(1.832298563)/(o+1.)+(0.116496255849)+(-0.121237844262)*0.5**o
        ref[3,1,2,1]=(-0.445596580552)/(o+1.)+(-0.156175520914)+(0.504161603066)*0.5**o
        ref[3,2,0,0]=(-0.731070691747)/(o+1.)+(0.503662769204)+(-0.34790984999)*0.5**o
        ref[3,2,0,1]=(-0.897874436171)/(o+1.)+(-0.536350715816)+(-0.557444127335)*0.5**o
        ref[3,2,1,0]=(-1.83376599145)/(o+1.)+(-0.461429131652)+(0.965420676359)*0.5**o
        ref[3,2,1,1]=(0.928492453162)/(o+1.)+(-0.576049154172)+(-0.950755375408)*0.5**o
        ref[3,2,2,0]=(0.345454197754)/(o+1.)+(-0.181381271639)+(0.339657614739)*0.5**o
        ref[3,2,2,1]=(-0.0628104757709)/(o+1.)+(-0.0199832483422)+(0.427508981997)*0.5**o
        ref[3,3,0,0]=(1.44849052868)/(o+1.)+(0.260844398365)+(-0.973352015556)*0.5**o
        ref[3,3,0,1]=(0.463093996444)/(o+1.)+(0.275211398251)+(0.950365830796)*0.5**o
        ref[3,3,1,0]=(0.237979951719)/(o+1.)+(-1.00035305177)+(-0.00222827033475)*0.5**o
        ref[3,3,1,1]=(0.614232373286)/(o+1.)+(0.364716105808)+(0.887884370757)*0.5**o
        ref[3,3,2,0]=(-0.699179501772)/(o+1.)+(0.436035164011)+(-0.0214623472154)*0.5**o
        ref[3,3,2,1]=(-0.781572997789)/(o+1.)+(-0.47624140043)+(0.949491478772)*0.5**o
        ref[3,4,0,0]=(-0.340589306073)/(o+1.)+(-0.332795540857)+(0.354489762998)*0.5**o
        ref[3,4,0,1]=(-0.0423707909991)/(o+1.)+(0.362839819216)+(0.552196121696)*0.5**o
        ref[3,4,1,0]=(-1.12737974721)/(o+1.)+(-0.549476212403)+(0.839982844034)*0.5**o
        ref[3,4,1,1]=(0.859950522145)/(o+1.)+(-0.470193609661)+(0.78962834926)*0.5**o
        ref[3,4,2,0]=(-0.897622459834)/(o+1.)+(-0.0147545707622)+(-0.882936934355)*0.5**o
        ref[3,4,2,1]=(-0.394755553787)/(o+1.)+(0.434493415112)+(0.342579062975)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactOne_fromSymbol_ReducedSolution_rank0(self):
      """
      tests integral of rank 0 Symbol on the ReducedFunctionOnContactOne

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Symbol(shape=())
      s=Data(0,(),w)
      if dim==2:
        s=(-0.0123111028353)*x[0]+(-1.54593060503)*x[1]
        ref=(-0.883444958139)/(o+1.)+(-0.0326056141412)+(-0.609585521441)*0.5**o
      else:
        s=(0.465974344755)*x[0]+(-0.406015068796)*x[1]+(0.217033091571)*x[2]
        ref=(-0.964783366814)/(o+1.)+(0.787210317182)+(-0.332644900019)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactOne_fromSymbol_ReducedSolution_rank1(self):
      """
      tests integral of rank 1 Symbol on the ReducedFunctionOnContactOne

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(2,))
      s=Data(0,(2,),w)
      ref=numpy.zeros((2,),numpy.float_)
      if dim==2:
        s[0]=(-1.01806458946)*x[0]+(-0.84209500794)*x[1]
        s[1]=(-0.372739669691)*x[0]+(0.123203057686)*x[1]
        ref[0]=(-0.910310319387)/(o+1.)+(-0.35457604)+(-0.240697198017)*0.5**o
        ref[1]=(0.817182634856)/(o+1.)+(-0.752992097502)+(0.439264948143)*0.5**o
      else:
        s[0]=(-0.262859740698)*x[0]+(0.398827301114)*x[1]+(-0.0121863377176)*x[2]
        s[1]=(0.533472917947)*x[0]+(-0.654130540619)*x[1]+(-0.155059686066)*x[2]
        ref[0]=(0.173661143447)/(o+1.)+(-0.0377591573443)+(0.0256383939398)*0.5**o
        ref[1]=(-1.0263552046)/(o+1.)+(0.265270569058)+(0.220096757743)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactOne_fromSymbol_ReducedSolution_rank2(self):
      """
      tests integral of rank 2 Symbol on the ReducedFunctionOnContactOne

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5))
      s=Data(0,(4, 5),w)
      ref=numpy.zeros((4, 5),numpy.float_)
      if dim==2:
        s[0,0]=(0.0355078592116)*x[0]+(-1.45322051958)*x[1]
        s[0,1]=(0.940649610535)*x[0]+(1.03783851181)*x[1]
        s[0,2]=(-0.285465603086)*x[0]+(-1.0279436539)*x[1]
        s[0,3]=(-1.05683604808)*x[0]+(-0.606790848933)*x[1]
        s[0,4]=(0.155725423448)*x[0]+(-0.0350991646899)*x[1]
        s[1,0]=(0.288671804193)*x[0]+(0.427933920845)*x[1]
        s[1,1]=(-0.0117065079514)*x[0]+(0.96117958225)*x[1]
        s[1,2]=(0.209419996754)*x[0]+(0.21574344987)*x[1]
        s[1,3]=(0.885781216051)*x[0]+(-0.68144471245)*x[1]
        s[1,4]=(-1.08228199466)*x[0]+(-0.155862749309)*x[1]
        s[2,0]=(-0.680644329614)*x[0]+(-1.38778484373)*x[1]
        s[2,1]=(0.0400929370385)*x[0]+(0.415428765055)*x[1]
        s[2,2]=(-1.20113508704)*x[0]+(-0.876683252298)*x[1]
        s[2,3]=(-0.312684596061)*x[0]+(-0.70367059229)*x[1]
        s[2,4]=(0.214059442951)*x[0]+(1.42513598809)*x[1]
        s[3,0]=(-0.24713062126)*x[0]+(1.02991464042)*x[1]
        s[3,1]=(-0.83089702409)*x[0]+(0.587925745944)*x[1]
        s[3,2]=(0.950881339072)*x[0]+(0.051413472076)*x[1]
        s[3,3]=(0.182953617513)*x[0]+(0.94052327152)*x[1]
        s[3,4]=(-0.387816734481)*x[0]+(0.366589363018)*x[1]
        ref[0,0]=(-0.563896045541)/(o+1.)+(-0.436916928584)+(0.0200172423412)*0.5**o
        ref[0,1]=(0.583712973931)/(o+1.)+(0.277276607579)+(0.84022193326)*0.5**o
        ref[0,2]=(-0.381794683559)/(o+1.)+(-0.29471407057)+(-0.342186432287)*0.5**o
        ref[0,3]=(0.0937197621552)/(o+1.)+(-0.417759182193)+(-0.921828294785)*0.5**o
        ref[0,4]=(-0.00732587396541)/(o+1.)+(0.282507495385)+(-0.437062858047)*0.5**o
        ref[1,0]=(0.656328782568)/(o+1.)+(0.105444180081)+(-0.150611417692)*0.5**o
        ref[1,1]=(0.0862490471738)/(o+1.)+(-0.0358008007742)+(0.934825628673)*0.5**o
        ref[1,2]=(-0.681994265679)/(o+1.)+(0.52707424273)+(0.053009226843)*0.5**o
        ref[1,3]=(0.162048053458)/(o+1.)+(-0.131185707156)+(0.304659864456)*0.5**o
        ref[1,4]=(-0.321375806412)/(o+1.)+(-0.12650756243)+(-0.663753812697)*0.5**o
        ref[2,0]=(-0.706054805711)/(o+1.)+(-0.805432593593)+(0.248490819557)*0.5**o
        ref[2,1]=(0.587809899116)/(o+1.)+(-0.0542554861756)+(-0.0237772246706)*0.5**o
        ref[2,2]=(0.044861811334)/(o+1.)+(-0.933527479587)+(-0.255625191498)*0.5**o
        ref[2,3]=(0.0752678267397)/(o+1.)+(-0.281993375709)+(-0.527636263672)*0.5**o
        ref[2,4]=(0.684054536011)/(o+1.)+(0.209350429588)+(0.536440035851)*0.5**o
        ref[3,0]=(0.827817530316)/(o+1.)+(-0.211443979463)+(0.37785444777)*0.5**o
        ref[3,1]=(-0.410871022592)/(o+1.)+(0.0530892951706)+(0.061721154105)*0.5**o
        ref[3,2]=(0.199285164042)/(o+1.)+(0.122582366509)+(0.557844914088)*0.5**o
        ref[3,3]=(0.195462635242)/(o+1.)+(0.027388051617)+(0.873238150557)*0.5**o
        ref[3,4]=(0.231128200422)/(o+1.)+(0.0245220097267)+(-0.301399591338)*0.5**o
      else:
        s[0,0]=(0.995116002112)*x[0]+(0.591812853606)*x[1]+(0.436070205257)*x[2]
        s[0,1]=(-0.118607336615)*x[0]+(0.594691767995)*x[1]+(-0.56002370897)*x[2]
        s[0,2]=(1.14520644838)*x[0]+(0.18460040375)*x[1]+(-0.960923519429)*x[2]
        s[0,3]=(0.392881958399)*x[0]+(-1.48738000894)*x[1]+(1.04349887295)*x[2]
        s[0,4]=(0.232444347548)*x[0]+(-0.83025700349)*x[1]+(0.612703936854)*x[2]
        s[1,0]=(0.190581558342)*x[0]+(-0.619707808792)*x[1]+(1.05804982356)*x[2]
        s[1,1]=(-0.42811715218)*x[0]+(-0.447035862369)*x[1]+(-1.24096539043)*x[2]
        s[1,2]=(0.200357664023)*x[0]+(-0.936410127129)*x[1]+(0.750394727948)*x[2]
        s[1,3]=(-0.612730981377)*x[0]+(-0.653699725295)*x[1]+(0.0015189792328)*x[2]
        s[1,4]=(0.288644454156)*x[0]+(0.93337277237)*x[1]+(-0.602600880867)*x[2]
        s[2,0]=(-1.17236877273)*x[0]+(1.45496830635)*x[1]+(-1.12383998094)*x[2]
        s[2,1]=(-0.232230799241)*x[0]+(0.415001155159)*x[1]+(-0.60248885989)*x[2]
        s[2,2]=(-0.83830117983)*x[0]+(0.192640431036)*x[1]+(1.43556749653)*x[2]
        s[2,3]=(0.973424823298)*x[0]+(0.369914981625)*x[1]+(-1.40643757872)*x[2]
        s[2,4]=(-0.458923994338)*x[0]+(-1.04785209961)*x[1]+(-0.592533584766)*x[2]
        s[3,0]=(0.545327477107)*x[0]+(1.9382915447)*x[1]+(1.47179032667)*x[2]
        s[3,1]=(-0.415696009498)*x[0]+(-0.594177620783)*x[1]+(-1.35108058064)*x[2]
        s[3,2]=(0.831178664645)*x[0]+(-0.0275877269769)*x[1]+(0.302241965885)*x[2]
        s[3,3]=(-0.360815522042)*x[0]+(-0.826462125533)*x[1]+(0.530840397679)*x[2]
        s[3,4]=(0.135928452703)*x[0]+(-1.51122126726)*x[1]+(1.6120424041)*x[2]
        ref[0,0]=(1.24742647646)/(o+1.)+(0.269060816234)+(0.237450952046)*0.5**o
        ref[0,1]=(1.12585957806)/(o+1.)+(-0.755828198463)+(0.301857541273)*0.5**o
        ref[0,2]=(-0.435989664675)/(o+1.)+(0.295471183049)+(0.213930631278)*0.5**o
        ref[0,3]=(-0.488921466935)/(o+1.)+(0.294480448259)+(-0.151038607178)*0.5**o
        ref[0,4]=(0.516230612817)/(o+1.)+(-0.0787984362944)+(-0.343742459316)*0.5**o
        ref[1,0]=(-0.492183705773)/(o+1.)+(0.501230271461)+(0.118646735962)*0.5**o
        ref[1,1]=(-1.43995574312)/(o+1.)+(-0.28437529362)+(-0.107412074623)*0.5**o
        ref[1,2]=(-0.101017880825)/(o+1.)+(0.28028462641)+(-0.445209107154)*0.5**o
        ref[1,3]=(0.128471434955)/(o+1.)+(-0.384371968608)+(-0.62463922518)*0.5**o
        ref[1,4]=(-0.0918241851534)/(o+1.)+(-0.0317649881343)+(0.774770507081)*0.5**o
        ref[2,0]=(0.221291051677)/(o+1.)+(-0.302519614649)+(-0.457492269704)*0.5**o
        ref[2,1]=(0.100276041699)/(o+1.)+(-0.387935114184)+(0.255875682695)*0.5**o
        ref[2,2]=(0.67334801807)/(o+1.)+(0.363637415491)+(-0.610716101317)*0.5**o
        ref[2,3]=(-0.101257531168)/(o+1.)+(-0.225887466089)+(0.48993468955)*0.5**o
        ref[2,4]=(-0.451085705642)/(o+1.)+(-0.415483032516)+(-0.817257908036)*0.5**o
        ref[3,0]=(1.87076639925)/(o+1.)+(1.20828670015)+(-0.331930451075)*0.5**o
        ref[3,1]=(-1.43689409978)/(o+1.)+(0.0171611774403)+(-0.958382466017)*0.5**o
        ref[3,2]=(0.146538702473)/(o+1.)+(0.391890493041)+(0.175513214998)*0.5**o
        ref[3,3]=(-0.122934034211)/(o+1.)+(0.140609173665)+(-0.814721563016)*0.5**o
        ref[3,4]=(0.223164895319)/(o+1.)+(-0.31244215916)+(0.638469012551)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactOne_fromSymbol_ReducedSolution_rank3(self):
      """
      tests integral of rank 3 Symbol on the ReducedFunctionOnContactOne

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(6, 2, 2))
      s=Data(0,(6, 2, 2),w)
      ref=numpy.zeros((6, 2, 2),numpy.float_)
      if dim==2:
        s[0,0,0]=(-0.35078097593)*x[0]+(0.306926522015)*x[1]
        s[0,0,1]=(-0.803534689121)*x[0]+(-0.0602880575367)*x[1]
        s[0,1,0]=(-0.903470951227)*x[0]+(0.374123966765)*x[1]
        s[0,1,1]=(-1.18641387512)*x[0]+(0.273430931834)*x[1]
        s[1,0,0]=(0.493295479958)*x[0]+(1.54668330013)*x[1]
        s[1,0,1]=(-1.27848534954)*x[0]+(0.0588492485708)*x[1]
        s[1,1,0]=(-1.42953381447)*x[0]+(0.717393314307)*x[1]
        s[1,1,1]=(-0.467730806598)*x[0]+(-0.0117815181086)*x[1]
        s[2,0,0]=(-0.602626661024)*x[0]+(0.533324435346)*x[1]
        s[2,0,1]=(-0.904855697799)*x[0]+(1.09112846759)*x[1]
        s[2,1,0]=(1.20360337188)*x[0]+(1.23097020284)*x[1]
        s[2,1,1]=(0.0885447539403)*x[0]+(0.405387510263)*x[1]
        s[3,0,0]=(-0.730436778413)*x[0]+(0.416740386984)*x[1]
        s[3,0,1]=(-1.04126166563)*x[0]+(0.41753641298)*x[1]
        s[3,1,0]=(-0.271195830488)*x[0]+(-0.11575841172)*x[1]
        s[3,1,1]=(1.16100889666)*x[0]+(-0.233929048364)*x[1]
        s[4,0,0]=(0.664549807695)*x[0]+(0.224807977257)*x[1]
        s[4,0,1]=(-0.796935211563)*x[0]+(0.365917637053)*x[1]
        s[4,1,0]=(0.0401977690429)*x[0]+(-0.612500495333)*x[1]
        s[4,1,1]=(-0.115172707709)*x[0]+(-0.280626474812)*x[1]
        s[5,0,0]=(0.844835935845)*x[0]+(-1.19344222698)*x[1]
        s[5,0,1]=(-1.81553680308)*x[0]+(-1.66095998384)*x[1]
        s[5,1,0]=(0.761230206372)*x[0]+(0.547313314376)*x[1]
        s[5,1,1]=(-0.246895651321)*x[0]+(0.391870038086)*x[1]
        ref[0,0,0]=(-0.600965801098)/(o+1.)+(0.338185808482)+(-0.119260269781)*0.5**o
        ref[0,0,1]=(-0.224867141426)/(o+1.)+(-0.334457471517)+(0.0299593378011)*0.5**o
        ref[0,1,0]=(0.0129821011627)/(o+1.)+(-0.0950564992708)+(-0.352216087084)*0.5**o
        ref[0,1,1]=(0.0332057375276)/(o+1.)+(-0.0551431610872)+(-0.835902358643)*0.5**o
        ref[1,0,0]=(0.553424856182)/(o+1.)+(0.848186123918)+(-0.209818323927)*0.5**o
        ref[1,0,1]=(-0.75127641037)/(o+1.)+(-0.0481529718983)+(-0.372053746804)*0.5**o
        ref[1,1,0]=(-0.218691700126)/(o+1.)+(0.0665842087159)+(-0.626617217469)*0.5**o
        ref[1,1,1]=(-0.751752416764)/(o+1.)+(0.165004967668)+(-0.0577698432779)*0.5**o
        ref[2,0,0]=(-0.0172781449053)/(o+1.)+(-0.197949307988)+(0.343874535203)*0.5**o
        ref[2,0,1]=(0.499625949394)/(o+1.)+(-0.0639621511173)+(-0.185428877371)*0.5**o
        ref[2,1,0]=(0.965002845499)/(o+1.)+(0.296405176523)+(0.87676037618)*0.5**o
        ref[2,1,1]=(0.571427043323)/(o+1.)+(0.355568778058)+(-0.788632335234)*0.5**o
        ref[3,0,0]=(0.643869395266)/(o+1.)+(-0.248603098629)+(-0.460359589438)*0.5**o
        ref[3,0,1]=(0.847552650465)/(o+1.)+(-0.471374980899)+(-0.528527941319)*0.5**o
        ref[3,1,0]=(0.792482290139)/(o+1.)+(-0.288835326975)+(-0.601765878397)*0.5**o
        ref[3,1,1]=(0.674315208514)/(o+1.)+(-0.301924186259)+(0.8566130123)*0.5**o
        ref[4,0,0]=(0.14482725218)/(o+1.)+(0.123667770392)+(0.497194991988)*0.5**o
        ref[4,0,1]=(0.207166197869)/(o+1.)+(-0.00140080206462)+(-0.63538216825)*0.5**o
        ref[4,1,0]=(0.134064796438)/(o+1.)+(-0.583980524751)+(0.461593526773)*0.5**o
        ref[4,1,1]=(0.329865203686)/(o+1.)+(-0.0277930553577)+(-0.670078275492)*0.5**o
        ref[5,0,0]=(-0.671043415594)/(o+1.)+(-0.0772895763368)+(0.477016277134)*0.5**o
        ref[5,0,1]=(-0.863660332362)/(o+1.)+(-0.862182891111)+(-0.888470672335)*0.5**o
        ref[5,1,0]=(0.734550839405)/(o+1.)+(-0.00320324272275)+(0.580399166789)*0.5**o
        ref[5,1,1]=(-0.186529971258)/(o+1.)+(0.409102503591)+(-0.486700649159)*0.5**o
      else:
        s[0,0,0]=(1.49896775273)*x[0]+(-1.78250968743)*x[1]+(1.41503337092)*x[2]
        s[0,0,1]=(-0.643818266916)*x[0]+(-0.375134170105)*x[1]+(0.650514899885)*x[2]
        s[0,1,0]=(-0.0528798419185)*x[0]+(0.665778795831)*x[1]+(1.21069663928)*x[2]
        s[0,1,1]=(-1.45464226351)*x[0]+(-0.19467401007)*x[1]+(0.442665314437)*x[2]
        s[1,0,0]=(-0.836797292596)*x[0]+(-0.0542158346848)*x[1]+(-0.54209479287)*x[2]
        s[1,0,1]=(-1.09189713442)*x[0]+(1.03696758011)*x[1]+(0.0722113737424)*x[2]
        s[1,1,0]=(-0.493099825558)*x[0]+(-0.914461955024)*x[1]+(1.50580577272)*x[2]
        s[1,1,1]=(-0.702256378373)*x[0]+(-1.55374444057)*x[1]+(0.187304514552)*x[2]
        s[2,0,0]=(-0.924126823838)*x[0]+(-1.20470503473)*x[1]+(-0.702166397011)*x[2]
        s[2,0,1]=(1.04661135244)*x[0]+(0.995774871402)*x[1]+(0.912661077993)*x[2]
        s[2,1,0]=(0.0694812274474)*x[0]+(-0.384400463672)*x[1]+(-1.27290008863)*x[2]
        s[2,1,1]=(-0.196348771836)*x[0]+(0.314419668134)*x[1]+(-0.147174148507)*x[2]
        s[3,0,0]=(0.555982166527)*x[0]+(1.26542592993)*x[1]+(1.94075238034)*x[2]
        s[3,0,1]=(-0.15925841434)*x[0]+(0.235963126853)*x[1]+(1.01719057653)*x[2]
        s[3,1,0]=(1.8766116496)*x[0]+(0.0416539079261)*x[1]+(0.692631820852)*x[2]
        s[3,1,1]=(-1.60594678123)*x[0]+(0.418185984317)*x[1]+(1.00380643717)*x[2]
        s[4,0,0]=(-0.496656665568)*x[0]+(-0.519100502539)*x[1]+(-0.402964765219)*x[2]
        s[4,0,1]=(0.810723701732)*x[0]+(-0.9121518445)*x[1]+(0.0693075114891)*x[2]
        s[4,1,0]=(-1.38980929277)*x[0]+(-0.135080762705)*x[1]+(-0.708040071312)*x[2]
        s[4,1,1]=(0.0366128814416)*x[0]+(-0.598343408328)*x[1]+(-1.05172726191)*x[2]
        s[5,0,0]=(0.626645586525)*x[0]+(0.368702379272)*x[1]+(1.06419625419)*x[2]
        s[5,0,1]=(0.823542930912)*x[0]+(-0.397291307792)*x[1]+(1.47677515709)*x[2]
        s[5,1,0]=(1.0044323164)*x[0]+(-0.400460024166)*x[1]+(0.0881220069594)*x[2]
        s[5,1,1]=(0.841418294962)*x[0]+(0.448350914459)*x[1]+(-0.535982888316)*x[2]
        ref[0,0,0]=(0.0352781393333)/(o+1.)+(0.227326591212)+(0.64156011446)*0.5**o
        ref[0,0,1]=(-0.122693252863)/(o+1.)+(-0.221154887636)+(0.196565490999)*0.5**o
        ref[0,1,0]=(0.784542061966)/(o+1.)+(0.731576295626)+(-0.424099060024)*0.5**o
        ref[0,1,1]=(0.118776006377)/(o+1.)+(-0.423066319495)+(-0.479294326528)*0.5**o
        ref[1,0,0]=(1.18086837157)/(o+1.)+(-1.14308640082)+(-0.327803490075)*0.5**o
        ref[1,0,1]=(1.11878511573)/(o+1.)+(-0.427948355911)+(-0.245606584475)*0.5**o
        ref[1,1,0]=(-0.256395599928)/(o+1.)+(0.22748609258)+(-0.100332593094)*0.5**o
        ref[1,1,1]=(-0.192853498607)/(o+1.)+(-0.925785926894)+(-0.0242709519946)*0.5**o
        ref[2,0,0]=(-0.415752649176)/(o+1.)+(-1.14572035003)+(-0.123804906338)*0.5**o
        ref[2,0,1]=(0.384199373535)/(o+1.)+(0.843162532691)+(0.884522862915)*0.5**o
        ref[2,1,0]=(-0.229595030341)/(o+1.)+(-1.14751601921)+(0.936807743902)*0.5**o
        ref[2,1,1]=(0.267255066685)/(o+1.)+(-0.491644195523)+(0.686930072154)*0.5**o
        ref[3,0,0]=(1.68424865452)/(o+1.)+(0.955116898624)+(0.167678025029)*0.5**o
        ref[3,0,1]=(0.0034339126924)/(o+1.)+(0.718824562093)+(-0.347187747836)*0.5**o
        ref[3,1,0]=(1.62164005566)/(o+1.)+(0.0562907078128)+(0.876675907094)*0.5**o
        ref[3,1,1]=(0.472424872079)/(o+1.)+(0.0308594354746)+(-0.718098102775)*0.5**o
        ref[4,0,0]=(-0.0285810697935)/(o+1.)+(-0.525693855415)+(-0.338753152702)*0.5**o
        ref[4,0,1]=(0.303057697082)/(o+1.)+(-0.350175453973)+(0.365172579586)*0.5**o
        ref[4,1,0]=(-1.04637813553)/(o+1.)+(-0.308024248014)+(-0.57050349523)*0.5**o
        ref[4,1,1]=(-0.500793745646)/(o+1.)+(-0.424344892355)+(-0.26397425844)*0.5**o
        ref[5,0,0]=(0.365122763119)/(o+1.)+(0.568706597744)+(0.557008261376)*0.5**o
        ref[5,0,1]=(0.510379226755)/(o+1.)+(0.436754108223)+(0.519139337008)*0.5**o
        ref[5,1,0]=(-0.200129774692)/(o+1.)+(0.274983568325)+(0.342256937234)*0.5**o
        ref[5,1,1]=(0.566120139584)/(o+1.)+(-0.312318326061)+(0.812302833644)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactOne_fromSymbol_ReducedSolution_rank4(self):
      """
      tests integral of rank 4 Symbol on the ReducedFunctionOnContactOne

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5, 3, 2))
      s=Data(0,(4, 5, 3, 2),w)
      ref=numpy.zeros((4, 5, 3, 2),numpy.float_)
      if dim==2:
        s[0,0,0,0]=(-0.36635416566)*x[0]+(0.454278450476)*x[1]
        s[0,0,0,1]=(0.449553060759)*x[0]+(-0.226772438798)*x[1]
        s[0,0,1,0]=(0.648343714053)*x[0]+(0.233007941149)*x[1]
        s[0,0,1,1]=(-0.135283076943)*x[0]+(1.61027554283)*x[1]
        s[0,0,2,0]=(0.619161973877)*x[0]+(-0.112220917964)*x[1]
        s[0,0,2,1]=(0.857902398581)*x[0]+(1.51928597673)*x[1]
        s[0,1,0,0]=(0.331827670107)*x[0]+(1.40775973855)*x[1]
        s[0,1,0,1]=(0.421840987115)*x[0]+(0.118563659741)*x[1]
        s[0,1,1,0]=(0.447268593931)*x[0]+(0.484984994669)*x[1]
        s[0,1,1,1]=(-0.189018546367)*x[0]+(0.467542988869)*x[1]
        s[0,1,2,0]=(0.466123494496)*x[0]+(-0.713267013868)*x[1]
        s[0,1,2,1]=(1.14105709142)*x[0]+(-0.134715682223)*x[1]
        s[0,2,0,0]=(0.871597208182)*x[0]+(-1.49080786854)*x[1]
        s[0,2,0,1]=(-0.94888847468)*x[0]+(0.0275999018363)*x[1]
        s[0,2,1,0]=(0.77534514887)*x[0]+(1.67162256756)*x[1]
        s[0,2,1,1]=(-0.786790506628)*x[0]+(-0.0692869424346)*x[1]
        s[0,2,2,0]=(1.78371863881)*x[0]+(-0.0481919137551)*x[1]
        s[0,2,2,1]=(-0.992074926087)*x[0]+(-1.02372593432)*x[1]
        s[0,3,0,0]=(0.113042447336)*x[0]+(-0.0790352303856)*x[1]
        s[0,3,0,1]=(1.12030253557)*x[0]+(0.393484268041)*x[1]
        s[0,3,1,0]=(0.379936292181)*x[0]+(-1.53869802379)*x[1]
        s[0,3,1,1]=(-0.191443496207)*x[0]+(0.563534464341)*x[1]
        s[0,3,2,0]=(0.104758691419)*x[0]+(-0.99439187659)*x[1]
        s[0,3,2,1]=(-0.479774987108)*x[0]+(-0.142765773584)*x[1]
        s[0,4,0,0]=(-1.50474690932)*x[0]+(-0.507691997905)*x[1]
        s[0,4,0,1]=(-0.78875103976)*x[0]+(0.132625919742)*x[1]
        s[0,4,1,0]=(-0.692847502554)*x[0]+(0.437815890612)*x[1]
        s[0,4,1,1]=(-0.808194455413)*x[0]+(0.376369555557)*x[1]
        s[0,4,2,0]=(0.919158598696)*x[0]+(-1.18391426084)*x[1]
        s[0,4,2,1]=(0.42374001855)*x[0]+(0.812305143875)*x[1]
        s[1,0,0,0]=(0.218364606199)*x[0]+(-1.40515806247)*x[1]
        s[1,0,0,1]=(-0.351603222911)*x[0]+(1.45105178061)*x[1]
        s[1,0,1,0]=(1.39849984972)*x[0]+(-1.35867734934)*x[1]
        s[1,0,1,1]=(-0.00360307599185)*x[0]+(0.246277359726)*x[1]
        s[1,0,2,0]=(0.514944968042)*x[0]+(0.428253029894)*x[1]
        s[1,0,2,1]=(-0.786120897197)*x[0]+(0.920810959626)*x[1]
        s[1,1,0,0]=(-0.602179171574)*x[0]+(0.515213890072)*x[1]
        s[1,1,0,1]=(-0.0861102153588)*x[0]+(0.477359706262)*x[1]
        s[1,1,1,0]=(-0.688164903151)*x[0]+(-0.313549483397)*x[1]
        s[1,1,1,1]=(-0.429990373791)*x[0]+(-0.0543708786137)*x[1]
        s[1,1,2,0]=(-0.39223682603)*x[0]+(0.796504408378)*x[1]
        s[1,1,2,1]=(-1.69046328113)*x[0]+(-0.344952802268)*x[1]
        s[1,2,0,0]=(-0.528152430512)*x[0]+(-0.269147355)*x[1]
        s[1,2,0,1]=(-1.68741664082)*x[0]+(0.568028242365)*x[1]
        s[1,2,1,0]=(-0.076694122635)*x[0]+(0.512745790897)*x[1]
        s[1,2,1,1]=(-0.738719294653)*x[0]+(1.72566985283)*x[1]
        s[1,2,2,0]=(0.679026798185)*x[0]+(-0.766391547753)*x[1]
        s[1,2,2,1]=(-1.7766436376)*x[0]+(-0.626962152238)*x[1]
        s[1,3,0,0]=(-0.0629592766364)*x[0]+(-0.602623643371)*x[1]
        s[1,3,0,1]=(1.46050982285)*x[0]+(-1.29912311111)*x[1]
        s[1,3,1,0]=(0.565052310821)*x[0]+(0.451146130683)*x[1]
        s[1,3,1,1]=(-1.72352086869)*x[0]+(-0.729969658338)*x[1]
        s[1,3,2,0]=(-0.699562180077)*x[0]+(0.785838373001)*x[1]
        s[1,3,2,1]=(0.0504742963364)*x[0]+(-0.856596707973)*x[1]
        s[1,4,0,0]=(-0.220685313017)*x[0]+(1.49415344926)*x[1]
        s[1,4,0,1]=(0.965835473202)*x[0]+(-0.664523896748)*x[1]
        s[1,4,1,0]=(-0.643560593026)*x[0]+(-1.06826452446)*x[1]
        s[1,4,1,1]=(-0.347993256951)*x[0]+(0.240926576999)*x[1]
        s[1,4,2,0]=(-0.666305003293)*x[0]+(1.5567540195)*x[1]
        s[1,4,2,1]=(0.752122481773)*x[0]+(-0.940078899457)*x[1]
        s[2,0,0,0]=(-0.428935565836)*x[0]+(-0.471904780642)*x[1]
        s[2,0,0,1]=(0.157225979958)*x[0]+(-0.707126471203)*x[1]
        s[2,0,1,0]=(-0.127873936162)*x[0]+(-1.01452234653)*x[1]
        s[2,0,1,1]=(-1.43314905378)*x[0]+(1.09406218538)*x[1]
        s[2,0,2,0]=(0.125475529763)*x[0]+(-0.144317966895)*x[1]
        s[2,0,2,1]=(0.781409858349)*x[0]+(-0.27747495012)*x[1]
        s[2,1,0,0]=(-1.21917608112)*x[0]+(-0.785716389383)*x[1]
        s[2,1,0,1]=(-0.26082261117)*x[0]+(-0.376402142009)*x[1]
        s[2,1,1,0]=(-0.125232448715)*x[0]+(0.261783624866)*x[1]
        s[2,1,1,1]=(-1.25985888818)*x[0]+(-0.0711437757878)*x[1]
        s[2,1,2,0]=(-0.300629742217)*x[0]+(-0.46939686121)*x[1]
        s[2,1,2,1]=(-1.77973256304)*x[0]+(0.713178700308)*x[1]
        s[2,2,0,0]=(-0.900064650204)*x[0]+(0.507949604762)*x[1]
        s[2,2,0,1]=(-0.885252734166)*x[0]+(-0.154018162942)*x[1]
        s[2,2,1,0]=(-0.7944479569)*x[0]+(1.77390521085)*x[1]
        s[2,2,1,1]=(1.1725888609)*x[0]+(0.662244216625)*x[1]
        s[2,2,2,0]=(-0.236310280848)*x[0]+(-0.542661345078)*x[1]
        s[2,2,2,1]=(1.15526522118)*x[0]+(0.0247167490822)*x[1]
        s[2,3,0,0]=(0.175989767802)*x[0]+(1.43963679604)*x[1]
        s[2,3,0,1]=(-0.692773425778)*x[0]+(-0.297594800061)*x[1]
        s[2,3,1,0]=(-0.847486832282)*x[0]+(0.0810250645286)*x[1]
        s[2,3,1,1]=(-0.681973624741)*x[0]+(-1.24971452577)*x[1]
        s[2,3,2,0]=(-1.06756371509)*x[0]+(0.977349108627)*x[1]
        s[2,3,2,1]=(0.846764513647)*x[0]+(-0.547998096826)*x[1]
        s[2,4,0,0]=(0.634255041776)*x[0]+(0.339363455414)*x[1]
        s[2,4,0,1]=(-1.39093420159)*x[0]+(0.716421880025)*x[1]
        s[2,4,1,0]=(0.257472890363)*x[0]+(-0.197982551362)*x[1]
        s[2,4,1,1]=(-0.45529259808)*x[0]+(-0.761148165674)*x[1]
        s[2,4,2,0]=(0.955343907343)*x[0]+(-1.29615358423)*x[1]
        s[2,4,2,1]=(0.362935017626)*x[0]+(1.25503750319)*x[1]
        s[3,0,0,0]=(0.179219886497)*x[0]+(1.78329032848)*x[1]
        s[3,0,0,1]=(0.43735751276)*x[0]+(0.0403938192717)*x[1]
        s[3,0,1,0]=(1.11478662344)*x[0]+(-0.240063001264)*x[1]
        s[3,0,1,1]=(0.172583585916)*x[0]+(0.506428670417)*x[1]
        s[3,0,2,0]=(1.33494651584)*x[0]+(0.781224884171)*x[1]
        s[3,0,2,1]=(0.0432954454652)*x[0]+(-0.68034279444)*x[1]
        s[3,1,0,0]=(-0.856892482415)*x[0]+(0.836687116891)*x[1]
        s[3,1,0,1]=(0.487431626464)*x[0]+(-0.148084918499)*x[1]
        s[3,1,1,0]=(0.331122282784)*x[0]+(-1.38429248259)*x[1]
        s[3,1,1,1]=(-0.215232051361)*x[0]+(0.537089578691)*x[1]
        s[3,1,2,0]=(1.24485548633)*x[0]+(0.509639847837)*x[1]
        s[3,1,2,1]=(-0.321450199534)*x[0]+(0.211384054649)*x[1]
        s[3,2,0,0]=(0.355385603273)*x[0]+(0.928139739798)*x[1]
        s[3,2,0,1]=(0.927589198094)*x[0]+(1.38324753996)*x[1]
        s[3,2,1,0]=(-1.16347040084)*x[0]+(-0.351854201414)*x[1]
        s[3,2,1,1]=(0.717488068131)*x[0]+(-0.0977120861443)*x[1]
        s[3,2,2,0]=(-0.799786558433)*x[0]+(0.829503505995)*x[1]
        s[3,2,2,1]=(1.37899235577)*x[0]+(-1.77910476445)*x[1]
        s[3,3,0,0]=(0.763361907215)*x[0]+(-0.861336281243)*x[1]
        s[3,3,0,1]=(0.476903208654)*x[0]+(0.903897305941)*x[1]
        s[3,3,1,0]=(1.60374028575)*x[0]+(-0.315620315575)*x[1]
        s[3,3,1,1]=(0.972809255696)*x[0]+(0.363233068227)*x[1]
        s[3,3,2,0]=(0.602799894079)*x[0]+(1.20772970348)*x[1]
        s[3,3,2,1]=(-0.000443665606339)*x[0]+(-1.6279763946)*x[1]
        s[3,4,0,0]=(-1.26676161275)*x[0]+(0.148063159145)*x[1]
        s[3,4,0,1]=(0.322891919608)*x[0]+(-1.08096025287)*x[1]
        s[3,4,1,0]=(-0.0944277709182)*x[0]+(-0.824212826197)*x[1]
        s[3,4,1,1]=(1.13530096846)*x[0]+(0.217321895058)*x[1]
        s[3,4,2,0]=(1.25663262459)*x[0]+(-0.308861390599)*x[1]
        s[3,4,2,1]=(-0.379182493224)*x[0]+(0.331110509538)*x[1]
        ref[0,0,0,0]=(0.408958259232)/(o+1.)+(0.247137449101)+(-0.815308872619)*0.5**o
        ref[0,0,0,1]=(0.13228131757)/(o+1.)+(0.165819155398)+(-0.241139006405)*0.5**o
        ref[0,0,1,0]=(0.128399251556)/(o+1.)+(0.371321245227)+(0.0103099131928)*0.5**o
        ref[0,0,1,1]=(0.966281299065)/(o+1.)+(0.381611325047)+(-0.254511483272)*0.5**o
        ref[0,0,2,0]=(-0.200167961434)/(o+1.)+(-0.0928181112535)+(0.892745239853)*0.5**o
        ref[0,0,2,1]=(0.830787814329)/(o+1.)+(0.314595496988)+(0.917209567011)*0.5**o
        ref[0,1,0,0]=(0.612992604133)/(o+1.)+(0.66241770717)+(-0.19824060982)*0.5**o
        ref[0,1,0,1]=(-0.0163161609645)/(o+1.)+(0.2025662158)+(0.151588376221)*0.5**o
        ref[0,1,1,0]=(-0.126917561445)/(o+1.)+(0.630548380093)+(-0.201925610142)*0.5**o
        ref[0,1,1,1]=(0.819096391825)/(o+1.)+(-0.370043224043)+(0.199514498764)*0.5**o
        ref[0,1,2,0]=(-0.96777130286)/(o+1.)+(0.367817797326)+(-0.0150078111633)*0.5**o
        ref[0,1,2,1]=(0.045654631235)/(o+1.)+(0.396367346781)+(0.167952084398)*0.5**o
        ref[0,2,0,0]=(-0.784765277575)/(o+1.)+(-0.298358907675)+(0.762272432566)*0.5**o
        ref[0,2,0,1]=(0.386572441281)/(o+1.)+(-0.51566858578)+(-0.276523842565)*0.5**o
        ref[0,2,1,0]=(0.680861847658)/(o+1.)+(0.740328104187)+(0.285449660402)*0.5**o
        ref[0,2,1,1]=(0.325836705829)/(o+1.)+(-0.49316269795)+(-0.195588758992)*0.5**o
        ref[0,2,2,0]=(0.783589269167)/(o+1.)+(0.0810542535881)+(0.789828948707)*0.5**o
        ref[0,2,2,1]=(-0.370218192335)/(o+1.)+(-0.608350209702)+(-0.428882248665)*0.5**o
        ref[0,3,0,0]=(0.188865301872)/(o+1.)+(-0.465976959809)+(0.777095834696)*0.5**o
        ref[0,3,0,1]=(0.764156203059)/(o+1.)+(-0.0928154457191)+(0.935261491991)*0.5**o
        ref[0,3,1,0]=(-0.689450927985)/(o+1.)+(-0.664550854911)+(0.8597909062)*0.5**o
        ref[0,3,1,1]=(0.555793140817)/(o+1.)+(0.142885227944)+(-0.469472628571)*0.5**o
        ref[0,3,2,0]=(-0.625798760976)/(o+1.)+(-0.499725272457)+(0.735616120719)*0.5**o
        ref[0,3,2,1]=(-0.907491695474)/(o+1.)+(0.385181050937)+(-0.485411167091)*0.5**o
        ref[0,4,0,0]=(-0.879336661239)/(o+1.)+(-0.205953419637)+(-0.721195406712)*0.5**o
        ref[0,4,0,1]=(0.86693894952)/(o+1.)+(-0.819677383073)+(0.11629069661)*0.5**o
        ref[0,4,1,0]=(-0.310289123464)/(o+1.)+(0.401826760004)+(-0.748396008487)*0.5**o
        ref[0,4,1,1]=(0.931547956812)/(o+1.)+(-0.740019832824)+(0.116666808981)*0.5**o
        ref[0,4,2,0]=(-0.847165088314)/(o+1.)+(-0.124403947896)+(0.831217321961)*0.5**o
        ref[0,4,2,1]=(0.177734973756)/(o+1.)+(0.441537508054)+(0.175235172561)*0.5**o
        ref[1,0,0,0]=(-0.542365809895)/(o+1.)+(-0.0827972838196)+(-0.478833078736)*0.5**o
        ref[1,0,0,1]=(0.844666854947)/(o+1.)+(0.498180748967)+(-0.741579795179)*0.5**o
        ref[1,0,1,0]=(-0.491126713824)/(o+1.)+(0.0335621009267)+(0.463825012354)*0.5**o
        ref[1,0,1,1]=(-0.309670980539)/(o+1.)+(0.472409867826)+(-0.392474471379)*0.5**o
        ref[1,0,2,0]=(0.278634540091)/(o+1.)+(-0.121141274647)+(0.906846007138)*0.5**o
        ref[1,0,2,1]=(-0.00603476025805)/(o+1.)+(0.182935877942)+(-0.225146933197)*0.5**o
        ref[1,1,0,0]=(0.580918965968)/(o+1.)+(0.0504878873855)+(-0.768860022241)*0.5**o
        ref[1,1,0,1]=(0.631736797799)/(o+1.)+(-0.537539597912)+(0.834591888928)*0.5**o
        ref[1,1,1,0]=(0.291226625025)/(o+1.)+(-0.703641010727)+(0.11434100988)*0.5**o
        ref[1,1,1,1]=(-0.909671911939)/(o+1.)+(0.237461939347)+(-0.0496132191606)*0.5**o
        ref[1,1,2,0]=(0.21073170992)/(o+1.)+(0.325715980645)+(-0.457896088863)*0.5**o
        ref[1,1,2,1]=(-0.725096486574)/(o+1.)+(-0.296742654874)+(-0.716834287076)*0.5**o
        ref[1,2,0,0]=(-0.164851515918)/(o+1.)+(-0.100676749276)+(-0.431094771042)*0.5**o
        ref[1,2,0,1]=(0.294840872181)/(o+1.)+(-0.329953166746)+(-0.754322937148)*0.5**o
        ref[1,2,1,0]=(0.341767621978)/(o+1.)+(0.400623115147)+(-0.70696218401)*0.5**o
        ref[1,2,1,1]=(0.821671220024)/(o+1.)+(0.193132433528)+(-0.220985528909)*0.5**o
        ref[1,2,2,0]=(-0.135508676597)/(o+1.)+(-0.38093100269)+(0.81000593241)*0.5**o
        ref[1,2,2,1]=(0.101637058171)/(o+1.)+(-0.771937440463)+(-0.961367967078)*0.5**o
        ref[1,3,0,0]=(-0.354951988253)/(o+1.)+(-0.41833260094)+(0.526034270126)*0.5**o
        ref[1,3,0,1]=(-0.461724677474)/(o+1.)+(-0.082145170992)+(0.787401731204)*0.5**o
        ref[1,3,1,0]=(0.681652074623)/(o+1.)+(-0.127199759209)+(0.5889458853)*0.5**o
        ref[1,3,1,1]=(-0.816769655641)/(o+1.)+(-0.382750879464)+(-0.871219112459)*0.5**o
        ref[1,3,2,0]=(0.549856768471)/(o+1.)+(-0.136659604755)+(-0.190261366036)*0.5**o
        ref[1,3,2,1]=(-0.298435559802)/(o+1.)+(-0.364033867566)+(0.220380883298)*0.5**o
        ref[1,4,0,0]=(0.692982064919)/(o+1.)+(0.445763556713)+(-0.311041042098)*0.5**o
        ref[1,4,0,1]=(-0.0284802774195)/(o+1.)+(0.173564544909)+(-0.0173372359445)*0.5**o
        ref[1,4,1,0]=(-0.521190661286)/(o+1.)+(-0.19266811161)+(-0.805298232979)*0.5**o
        ref[1,4,1,1]=(-0.344391605752)/(o+1.)+(0.344434513106)+(-0.451544100412)*0.5**o
        ref[1,4,2,0]=(0.590871123139)/(o+1.)+(0.123309088073)+(0.0529597169219)*0.5**o
        ref[1,4,2,1]=(-0.00996351412971)/(o+1.)+(-0.328705279006)+(0.479417654458)*0.5**o
        ref[2,0,0,0]=(0.220674489827)/(o+1.)+(-0.618024232079)+(0.114533627854)*0.5**o
        ref[2,0,0,1]=(-0.0844798976424)/(o+1.)+(0.070811636225)+(-0.607043866052)*0.5**o
        ref[2,0,1,0]=(-0.445683449242)/(o+1.)+(-0.113440437004)+(-0.469831959437)*0.5**o
        ref[2,0,1,1]=(0.271633369975)/(o+1.)+(0.132535807211)+(-0.875791852797)*0.5**o
        ref[2,0,2,0]=(0.649542525852)/(o+1.)+(0.0590165361575)+(-0.786418035298)*0.5**o
        ref[2,0,2,1]=(-0.317621366014)/(o+1.)+(0.329998426635)+(0.161559420974)*0.5**o
        ref[2,1,0,0]=(-0.835806177468)/(o+1.)+(-0.214223776118)+(-0.7406387408)*0.5**o
        ref[2,1,0,1]=(0.429741516432)/(o+1.)+(-0.552565623152)+(0.0381649766927)*0.5**o
        ref[2,1,1,0]=(-0.0472916937886)/(o+1.)+(-0.00552013887145)+(0.194883147682)*0.5**o
        ref[2,1,1,1]=(-0.199584048679)/(o+1.)+(-0.27639819939)+(-0.578622216512)*0.5**o
        ref[2,1,2,0]=(-0.941086888016)/(o+1.)+(-0.0385375072603)+(0.24813529911)*0.5**o
        ref[2,1,2,1]=(-0.0868737022971)/(o+1.)+(-0.0970507102796)+(-0.785578739872)*0.5**o
        ref[2,2,0,0]=(-0.202279119459)/(o+1.)+(-0.0839554337907)+(-0.0219250584016)*0.5**o
        ref[2,2,0,1]=(-0.184000614671)/(o+1.)+(-0.477928353442)+(0.100586424448)*0.5**o
        ref[2,2,1,0]=(0.927675340201)/(o+1.)+(0.0186691089268)+(0.0144436958933)*0.5**o
        ref[2,2,1,1]=(0.304792611717)/(o+1.)+(0.459166286092)+(0.611707893628)*0.5**o
        ref[2,2,2,0]=(0.0815687138062)/(o+1.)+(-0.768726979684)+(0.676913619635)*0.5**o
        ref[2,2,2,1]=(0.849269282518)/(o+1.)+(-0.25478410885)+(0.840280905449)*0.5**o
        ref[2,3,0,0]=(0.820793959083)/(o+1.)+(0.575169789593)+(-0.355506974424)*0.5**o
        ref[2,3,0,1]=(0.283603638973)/(o+1.)+(-0.335480628352)+(-0.603010608107)*0.5**o
        ref[2,3,1,0]=(-0.381013144765)/(o+1.)+(0.190974767434)+(-0.767398157855)*0.5**o
        ref[2,3,1,1]=(-0.773551356194)/(o+1.)+(-0.620890671367)+(0.0836445484134)*0.5**o
        ref[2,3,2,0]=(0.570212356041)/(o+1.)+(-0.289714757955)+(-0.0809974465983)*0.5**o
        ref[2,3,2,1]=(0.21581143181)/(o+1.)+(-0.415400501134)+(0.913755987278)*0.5**o
        ref[2,4,0,0]=(0.323568653669)/(o+1.)+(0.0799583453359)+(0.490133152849)*0.5**o
        ref[2,4,0,1]=(0.532408915634)/(o+1.)+(-0.115548404995)+(-0.97582442721)*0.5**o
        ref[2,4,1,0]=(-0.82114168932)/(o+1.)+(0.702564400414)+(-0.524496772508)*0.5**o
        ref[2,4,1,1]=(-0.479224200985)/(o+1.)+(-0.49364246599)+(0.25006836921)*0.5**o
        ref[2,4,2,0]=(-0.389740913016)/(o+1.)+(-0.0884282111182)+(0.225787658364)*0.5**o
        ref[2,4,2,1]=(0.508843816312)/(o+1.)+(0.17685868642)+(0.755411331668)*0.5**o
        ref[3,0,0,0]=(0.954468971244)/(o+1.)+(0.671012220464)+(-0.333983197198)*0.5**o
        ref[3,0,0,1]=(-0.312850718368)/(o+1.)+(-0.0723748107028)+(0.935351671806)*0.5**o
        ref[3,0,1,0]=(0.0640122675963)/(o+1.)+(0.220683721613)+(0.36934391135)*0.5**o
        ref[3,0,1,1]=(0.855735115439)/(o+1.)+(-0.366272230844)+(0.555821602582)*0.5**o
        ref[3,0,2,0]=(0.0417837662591)/(o+1.)+(0.556307682679)+(0.961772268394)*0.5**o
        ref[3,0,2,1]=(-0.928375029285)/(o+1.)+(-0.133425494561)+(0.558178669432)*0.5**o
        ref[3,1,0,0]=(0.610808931498)/(o+1.)+(-0.00371515949037)+(-0.623583978041)*0.5**o
        ref[3,1,0,1]=(-0.274141402005)/(o+1.)+(0.180852020544)+(0.251784068882)*0.5**o
        ref[3,1,1,0]=(-0.481032880244)/(o+1.)+(-0.705203882431)+(0.838270445302)*0.5**o
        ref[3,1,1,1]=(0.389002730435)/(o+1.)+(0.371327956101)+(-0.809801115307)*0.5**o
        ref[3,1,2,0]=(-0.217052039031)/(o+1.)+(0.504898297656)+(0.961750777882)*0.5**o
        ref[3,1,2,1]=(0.983589747751)/(o+1.)+(-0.768560013671)+(0.443464134708)*0.5**o
        ref[3,2,0,0]=(0.0850720894356)/(o+1.)+(0.295360321829)+(0.607732609977)*0.5**o
        ref[3,2,0,1]=(0.812866987322)/(o+1.)+(0.672388816065)+(0.153192118599)*0.5**o
        ref[3,2,1,0]=(-0.947419039593)/(o+1.)+(-0.179746473508)+(-0.208412615647)*0.5**o
        ref[3,2,1,1]=(-0.694332599484)/(o+1.)+(0.574069301711)+(0.165969978048)*0.5**o
        ref[3,2,2,0]=(0.746797880687)/(o+1.)+(0.0731831278015)+(-0.863447188728)*0.5**o
        ref[3,2,2,1]=(-0.9955274799)/(o+1.)+(-0.100141136686)+(0.795697344588)*0.5**o
        ref[3,3,0,0]=(-0.874362496111)/(o+1.)+(0.185559038089)+(0.405270045904)*0.5**o
        ref[3,3,0,1]=(0.557583002677)/(o+1.)+(0.556451155279)+(-0.28968479864)*0.5**o
        ref[3,3,1,0]=(0.435966507789)/(o+1.)+(0.00561393027505)+(0.840925601832)*0.5**o
        ref[3,3,1,1]=(-0.277310702324)/(o+1.)+(0.808938926899)+(-0.00452482755147)*0.5**o
        ref[3,3,2,0]=(0.878069034303)/(o+1.)+(0.0250230902204)+(0.882414382818)*0.5**o
        ref[3,3,2,1]=(-0.953670613054)/(o+1.)+(-0.626136165517)+(0.577522883879)*0.5**o
        ref[3,4,0,0]=(-0.0572166464991)/(o+1.)+(-0.195764673671)+(-0.669952459767)*0.5**o
        ref[3,4,0,1]=(-0.611801368818)/(o+1.)+(0.23313248955)+(-0.612531943539)*0.5**o
        ref[3,4,1,0]=(0.0451584859358)/(o+1.)+(-0.693097871519)+(0.422396659987)*0.5**o
        ref[3,4,1,1]=(0.0616735716897)/(o+1.)+(0.360358324628)+(0.57023264257)*0.5**o
        ref[3,4,2,0]=(0.404353246317)/(o+1.)+(0.02838608211)+(0.486645823449)*0.5**o
        ref[3,4,2,1]=(0.0039199584997)/(o+1.)+(0.344800813187)+(-0.741593568559)*0.5**o
      else:
        s[0,0,0,0]=(-0.00215071746669)*x[0]+(-0.332788203774)*x[1]+(-1.01240104354)*x[2]
        s[0,0,0,1]=(0.534102846033)*x[0]+(0.432162550346)*x[1]+(0.382807104545)*x[2]
        s[0,0,1,0]=(-0.774071131493)*x[0]+(0.174280314187)*x[1]+(0.235599208299)*x[2]
        s[0,0,1,1]=(1.66560900439)*x[0]+(1.38276801197)*x[1]+(0.451912961751)*x[2]
        s[0,0,2,0]=(-0.308642285607)*x[0]+(0.925763055269)*x[1]+(1.30533127575)*x[2]
        s[0,0,2,1]=(-0.15617828399)*x[0]+(-0.405834652149)*x[1]+(-0.846888299486)*x[2]
        s[0,1,0,0]=(-0.0743207250799)*x[0]+(-0.0831917437929)*x[1]+(-1.78311394968)*x[2]
        s[0,1,0,1]=(1.24178762381)*x[0]+(-0.121417608169)*x[1]+(0.0528284855272)*x[2]
        s[0,1,1,0]=(-1.14437209159)*x[0]+(0.0618138540822)*x[1]+(0.19124990065)*x[2]
        s[0,1,1,1]=(0.854246128467)*x[0]+(1.56688838996)*x[1]+(1.30424035038)*x[2]
        s[0,1,2,0]=(1.3184765993)*x[0]+(-1.40883832059)*x[1]+(0.832393276179)*x[2]
        s[0,1,2,1]=(-1.1625604886)*x[0]+(-0.104956544515)*x[1]+(-1.52389901089)*x[2]
        s[0,2,0,0]=(-0.106003547387)*x[0]+(0.852941663718)*x[1]+(0.24003638122)*x[2]
        s[0,2,0,1]=(0.892312082297)*x[0]+(-0.156996856451)*x[1]+(-1.2674975556)*x[2]
        s[0,2,1,0]=(0.0300043771968)*x[0]+(-0.848942993281)*x[1]+(-0.322425526609)*x[2]
        s[0,2,1,1]=(-0.361971760465)*x[0]+(1.89870303113)*x[1]+(0.861574517617)*x[2]
        s[0,2,2,0]=(0.97819437229)*x[0]+(-0.132714911288)*x[1]+(-0.957702168325)*x[2]
        s[0,2,2,1]=(0.588644003365)*x[0]+(-0.568265700558)*x[1]+(0.150521572866)*x[2]
        s[0,3,0,0]=(0.700807304996)*x[0]+(-1.14422226798)*x[1]+(0.587223436541)*x[2]
        s[0,3,0,1]=(-0.862702931786)*x[0]+(0.423016472206)*x[1]+(-0.555427243683)*x[2]
        s[0,3,1,0]=(0.574966038173)*x[0]+(-0.487199028698)*x[1]+(-0.32389521042)*x[2]
        s[0,3,1,1]=(0.130862033386)*x[0]+(0.42056248135)*x[1]+(0.925321527155)*x[2]
        s[0,3,2,0]=(-0.735071923108)*x[0]+(-0.528993360596)*x[1]+(0.485622476133)*x[2]
        s[0,3,2,1]=(0.126763646072)*x[0]+(0.0323222279682)*x[1]+(1.42515358228)*x[2]
        s[0,4,0,0]=(-0.334189936412)*x[0]+(0.14123257798)*x[1]+(-0.487379116976)*x[2]
        s[0,4,0,1]=(-0.528967405148)*x[0]+(-0.0622502130673)*x[1]+(-1.22107834734)*x[2]
        s[0,4,1,0]=(-1.67869230303)*x[0]+(-0.427809293008)*x[1]+(0.35027727689)*x[2]
        s[0,4,1,1]=(-1.3023324224)*x[0]+(0.364963594938)*x[1]+(1.25798740064)*x[2]
        s[0,4,2,0]=(0.789575151987)*x[0]+(-1.08985193505)*x[1]+(0.286072474971)*x[2]
        s[0,4,2,1]=(0.0321971599533)*x[0]+(1.64269617809)*x[1]+(0.448773378519)*x[2]
        s[1,0,0,0]=(-0.206961758788)*x[0]+(0.188339936734)*x[1]+(-1.36499979129)*x[2]
        s[1,0,0,1]=(-0.439106473295)*x[0]+(-1.35694867727)*x[1]+(0.17565861577)*x[2]
        s[1,0,1,0]=(0.754962041339)*x[0]+(0.377298691237)*x[1]+(-0.225684064888)*x[2]
        s[1,0,1,1]=(-0.164606093877)*x[0]+(-1.12165840203)*x[1]+(0.475143697931)*x[2]
        s[1,0,2,0]=(-0.563561093763)*x[0]+(-0.605592098451)*x[1]+(-0.382814067375)*x[2]
        s[1,0,2,1]=(-0.529311859218)*x[0]+(-0.440753365779)*x[1]+(-0.213919010047)*x[2]
        s[1,1,0,0]=(-0.381937118975)*x[0]+(0.115015087)*x[1]+(-1.58208233129)*x[2]
        s[1,1,0,1]=(-0.898803250164)*x[0]+(1.21794054101)*x[1]+(0.000708470416828)*x[2]
        s[1,1,1,0]=(0.417980628363)*x[0]+(-0.89943521283)*x[1]+(1.18981658848)*x[2]
        s[1,1,1,1]=(0.400600150308)*x[0]+(1.59168041332)*x[1]+(0.526855874334)*x[2]
        s[1,1,2,0]=(-0.658036770065)*x[0]+(0.903572546571)*x[1]+(-0.0322095817377)*x[2]
        s[1,1,2,1]=(-0.487427224544)*x[0]+(0.25991247483)*x[1]+(1.71801479005)*x[2]
        s[1,2,0,0]=(-1.23648267242)*x[0]+(-1.14397314877)*x[1]+(1.51420721169)*x[2]
        s[1,2,0,1]=(1.76604962545)*x[0]+(0.183730664401)*x[1]+(0.119413404231)*x[2]
        s[1,2,1,0]=(-0.371372970825)*x[0]+(1.67376148418)*x[1]+(-1.4673890303)*x[2]
        s[1,2,1,1]=(-1.05767796861)*x[0]+(1.57674524903)*x[1]+(0.907317505281)*x[2]
        s[1,2,2,0]=(-0.560171709945)*x[0]+(0.932731419096)*x[1]+(-1.34845422611)*x[2]
        s[1,2,2,1]=(0.299743316165)*x[0]+(1.27560261018)*x[1]+(-0.835084977966)*x[2]
        s[1,3,0,0]=(-0.255337437533)*x[0]+(-0.597132326112)*x[1]+(-0.136201264537)*x[2]
        s[1,3,0,1]=(0.593835766145)*x[0]+(-0.799771329645)*x[1]+(-0.980158768361)*x[2]
        s[1,3,1,0]=(1.38678662193)*x[0]+(0.145756993166)*x[1]+(-0.687830261701)*x[2]
        s[1,3,1,1]=(-0.535937612709)*x[0]+(-0.0191554551358)*x[1]+(0.360231645604)*x[2]
        s[1,3,2,0]=(0.464446491173)*x[0]+(0.601115829679)*x[1]+(-1.23243462932)*x[2]
        s[1,3,2,1]=(-1.00252524049)*x[0]+(0.38542247592)*x[1]+(-0.786817371152)*x[2]
        s[1,4,0,0]=(0.127520587286)*x[0]+(0.0193180871908)*x[1]+(0.443880703635)*x[2]
        s[1,4,0,1]=(0.459087583966)*x[0]+(-0.0133413082011)*x[1]+(-0.469574441754)*x[2]
        s[1,4,1,0]=(-0.0408745863162)*x[0]+(-0.450729520372)*x[1]+(0.49494190957)*x[2]
        s[1,4,1,1]=(0.698205417361)*x[0]+(0.475698414334)*x[1]+(-0.738661347545)*x[2]
        s[1,4,2,0]=(1.20797915788)*x[0]+(-0.132247786402)*x[1]+(-0.282227294584)*x[2]
        s[1,4,2,1]=(-0.76998141325)*x[0]+(-0.0677377690684)*x[1]+(-0.0836477753317)*x[2]
        s[2,0,0,0]=(-1.10579544966)*x[0]+(1.40458332668)*x[1]+(0.666829139108)*x[2]
        s[2,0,0,1]=(0.316772911983)*x[0]+(-0.722315825552)*x[1]+(-0.420491453418)*x[2]
        s[2,0,1,0]=(1.28978684389)*x[0]+(-1.31024104856)*x[1]+(-0.909892235182)*x[2]
        s[2,0,1,1]=(0.516882102505)*x[0]+(0.137837613805)*x[1]+(-0.0746054680439)*x[2]
        s[2,0,2,0]=(-0.684536226359)*x[0]+(1.26314901343)*x[1]+(-0.80088731126)*x[2]
        s[2,0,2,1]=(1.04894156753)*x[0]+(-0.292514668329)*x[1]+(0.189452265734)*x[2]
        s[2,1,0,0]=(-1.89697799819)*x[0]+(0.842415025171)*x[1]+(1.50175587499)*x[2]
        s[2,1,0,1]=(-0.132472949872)*x[0]+(0.755809155321)*x[1]+(1.4210576902)*x[2]
        s[2,1,1,0]=(-1.20812267424)*x[0]+(0.776283189748)*x[1]+(0.466844337822)*x[2]
        s[2,1,1,1]=(-0.545404145415)*x[0]+(0.630064653938)*x[1]+(0.416024592778)*x[2]
        s[2,1,2,0]=(-1.25450160322)*x[0]+(0.712099345608)*x[1]+(-1.40448708108)*x[2]
        s[2,1,2,1]=(0.8675638655)*x[0]+(-0.694288742412)*x[1]+(0.423917775454)*x[2]
        s[2,2,0,0]=(0.611369389916)*x[0]+(-0.282854928933)*x[1]+(0.421814371036)*x[2]
        s[2,2,0,1]=(0.87143261059)*x[0]+(0.626319549038)*x[1]+(-0.247486033705)*x[2]
        s[2,2,1,0]=(-1.09640818879)*x[0]+(-0.625440705278)*x[1]+(1.06488909366)*x[2]
        s[2,2,1,1]=(-1.63825859868)*x[0]+(0.243702450075)*x[1]+(0.37689118934)*x[2]
        s[2,2,2,0]=(1.14554562168)*x[0]+(-0.656545182621)*x[1]+(0.122881381407)*x[2]
        s[2,2,2,1]=(1.04057428387)*x[0]+(0.605147014643)*x[1]+(-0.0624571196862)*x[2]
        s[2,3,0,0]=(-0.687211572887)*x[0]+(-1.33387647369)*x[1]+(1.56461600602)*x[2]
        s[2,3,0,1]=(-0.439360840188)*x[0]+(-0.175050010481)*x[1]+(-0.493088639437)*x[2]
        s[2,3,1,0]=(1.01647468205)*x[0]+(1.607854298)*x[1]+(0.154041122248)*x[2]
        s[2,3,1,1]=(-1.17279615871)*x[0]+(0.265464982509)*x[1]+(0.536445292976)*x[2]
        s[2,3,2,0]=(-0.370774714547)*x[0]+(-0.140305952222)*x[1]+(-0.27755251782)*x[2]
        s[2,3,2,1]=(-0.601589093641)*x[0]+(-0.0914485078099)*x[1]+(-0.246092622851)*x[2]
        s[2,4,0,0]=(1.16222773615)*x[0]+(-1.0556446667)*x[1]+(1.18749640554)*x[2]
        s[2,4,0,1]=(0.441185917779)*x[0]+(0.106062084171)*x[1]+(-0.150699537615)*x[2]
        s[2,4,1,0]=(1.24765736189)*x[0]+(-1.28446862056)*x[1]+(-1.15581931924)*x[2]
        s[2,4,1,1]=(0.751269018864)*x[0]+(1.03373528238)*x[1]+(0.953542233964)*x[2]
        s[2,4,2,0]=(-0.507055271742)*x[0]+(-0.139767333963)*x[1]+(0.369122007484)*x[2]
        s[2,4,2,1]=(1.75708904398)*x[0]+(0.229784892513)*x[1]+(0.138439119614)*x[2]
        s[3,0,0,0]=(0.488579497744)*x[0]+(0.227848351934)*x[1]+(-0.503567606843)*x[2]
        s[3,0,0,1]=(1.48513008903)*x[0]+(0.0119808614235)*x[1]+(0.50138290689)*x[2]
        s[3,0,1,0]=(1.89811058679)*x[0]+(0.936442809385)*x[1]+(0.618927531336)*x[2]
        s[3,0,1,1]=(0.520596564131)*x[0]+(-1.34097734027)*x[1]+(-1.24288925352)*x[2]
        s[3,0,2,0]=(-1.22679833631)*x[0]+(0.946723115385)*x[1]+(-0.444825501575)*x[2]
        s[3,0,2,1]=(-0.786039767784)*x[0]+(1.01276854439)*x[1]+(0.868428470305)*x[2]
        s[3,1,0,0]=(-1.14928294841)*x[0]+(1.83212625566)*x[1]+(-0.131295307828)*x[2]
        s[3,1,0,1]=(0.530939875091)*x[0]+(-0.00936464244878)*x[1]+(-0.54397285021)*x[2]
        s[3,1,1,0]=(0.776394250264)*x[0]+(-0.0432924713397)*x[1]+(-0.48992936123)*x[2]
        s[3,1,1,1]=(-0.295285841654)*x[0]+(1.60838322278)*x[1]+(-0.822857254214)*x[2]
        s[3,1,2,0]=(-0.445325691811)*x[0]+(0.143992033055)*x[1]+(-0.562336605856)*x[2]
        s[3,1,2,1]=(-0.424042508176)*x[0]+(-0.119602696473)*x[1]+(0.50936759192)*x[2]
        s[3,2,0,0]=(0.0376712305683)*x[0]+(-0.127442186223)*x[1]+(-0.98477458245)*x[2]
        s[3,2,0,1]=(-0.964194007245)*x[0]+(-0.951386585236)*x[1]+(-0.376434083193)*x[2]
        s[3,2,1,0]=(-0.645708363662)*x[0]+(0.729523237957)*x[1]+(-0.815980951689)*x[2]
        s[3,2,1,1]=(1.54280019708)*x[0]+(0.347142929119)*x[1]+(0.117050155931)*x[2]
        s[3,2,2,0]=(-0.70653056148)*x[0]+(0.658564301815)*x[1]+(1.5766896863)*x[2]
        s[3,2,2,1]=(1.28238648298)*x[0]+(0.415173079215)*x[1]+(1.25492135064)*x[2]
        s[3,3,0,0]=(-0.907422225702)*x[0]+(0.354252510498)*x[1]+(-0.617733757576)*x[2]
        s[3,3,0,1]=(-0.687653032786)*x[0]+(0.166147005719)*x[1]+(0.0668202207665)*x[2]
        s[3,3,1,0]=(-0.515227984328)*x[0]+(-1.20687483216)*x[1]+(0.325703771711)*x[2]
        s[3,3,1,1]=(-0.239992988401)*x[0]+(-0.625429153195)*x[1]+(-0.987302012149)*x[2]
        s[3,3,2,0]=(0.48686562504)*x[0]+(-0.687077509478)*x[1]+(0.630530432518)*x[2]
        s[3,3,2,1]=(0.575785040914)*x[0]+(0.978122193974)*x[1]+(0.0753481784122)*x[2]
        s[3,4,0,0]=(-0.201304266512)*x[0]+(-0.563373770577)*x[1]+(-0.0445045378284)*x[2]
        s[3,4,0,1]=(0.348899505512)*x[0]+(-0.983874875078)*x[1]+(0.561390754019)*x[2]
        s[3,4,1,0]=(0.687251502909)*x[0]+(0.806428209931)*x[1]+(-0.848431263257)*x[2]
        s[3,4,1,1]=(-0.0690481011866)*x[0]+(-0.637464706806)*x[1]+(1.32335195878)*x[2]
        s[3,4,2,0]=(0.640286948745)*x[0]+(-1.44112416361)*x[1]+(-0.724716414365)*x[2]
        s[3,4,2,1]=(-0.967100763601)*x[0]+(0.0347001879437)*x[1]+(-0.888183629447)*x[2]
        ref[0,0,0,0]=(-1.47634692775)/(o+1.)+(0.413823374944)+(-0.69863978692)*0.5**o
        ref[0,0,0,1]=(0.467438056347)/(o+1.)+(-0.049348895708)+(0.980332235993)*0.5**o
        ref[0,0,1,0]=(0.0324608468542)/(o+1.)+(0.227219571463)+(-0.851091598789)*0.5**o
        ref[0,0,1,1]=(1.24663216229)/(o+1.)+(0.680736038169)+(0.892185739473)*0.5**o
        ref[0,0,2,0]=(1.04128811669)/(o+1.)+(0.801526892617)+(-0.721889856517)*0.5**o
        ref[0,0,2,1]=(-1.57019640933)/(o+1.)+(-0.0149100811556)+(0.191115336015)*0.5**o
        ref[0,1,0,0]=(-1.06234362705)/(o+1.)+(-0.881380501065)+(0.88447821062)*0.5**o
        ref[0,1,0,1]=(0.267287676352)/(o+1.)+(0.0934259729578)+(0.719058878904)*0.5**o
        ref[0,1,1,0]=(-0.83257335756)/(o+1.)+(0.056558158197)+(-0.171851295688)*0.5**o
        ref[0,1,1,1]=(1.59747135641)/(o+1.)+(1.00837611714)+(0.111151278122)*0.5**o
        ref[0,1,2,0]=(0.181694306894)/(o+1.)+(-0.0366161862313)+(0.633569620457)*0.5**o
        ref[0,1,2,1]=(-0.116206059658)/(o+1.)+(-1.20967709253)+(-0.255855799289)*0.5**o
        ref[0,2,0,0]=(0.615063090083)/(o+1.)+(-0.0944203632267)+(0.560752133921)*0.5**o
        ref[0,2,0,1]=(-1.68718452323)/(o+1.)+(0.415922156486)+(0.323157880499)*0.5**o
        ref[0,2,1,0]=(-1.38436480215)/(o+1.)+(0.514201265835)+(-0.785401872214)*0.5**o
        ref[0,2,1,1]=(1.07260243754)/(o+1.)+(1.03208581185)+(-0.738468272962)*0.5**o
        ref[0,2,2,0]=(-0.619469154652)/(o+1.)+(0.156513881249)+(0.19421868483)*0.5**o
        ref[0,2,2,1]=(0.742851517381)/(o+1.)+(-0.577857666542)+(0.583763691375)*0.5**o
        ref[0,3,0,0]=(-0.0209287146839)/(o+1.)+(-0.0417684031135)+(0.248273994472)*0.5**o
        ref[0,3,0,1]=(-0.876757092127)/(o+1.)+(0.156654815475)+(-0.431666242085)*0.5**o
        ref[0,3,1,0]=(-1.40731912274)/(o+1.)+(0.295731262157)+(0.579728397479)*0.5**o
        ref[0,3,1,1]=(0.766713725705)/(o+1.)+(0.520952712789)+(-0.331873109391)*0.5**o
        ref[0,3,2,0]=(-1.37493161295)/(o+1.)+(0.626269025128)+(-0.656049244875)*0.5**o
        ref[0,3,2,1]=(0.738828256274)/(o+1.)+(0.299922102653)+(0.245566994744)*0.5**o
        ref[0,4,0,0]=(-0.220320428485)/(o+1.)+(-0.381104395776)+(0.30219274463)*0.5**o
        ref[0,4,0,1]=(-1.71138026305)/(o+1.)+(0.0858133974006)+(-0.272542497315)*0.5**o
        ref[0,4,1,0]=(-0.0512711854771)/(o+1.)+(-0.398429115874)+(-0.908094901926)*0.5**o
        ref[0,4,1,1]=(0.775088175251)/(o+1.)+(0.153496176444)+(-0.76146195497)*0.5**o
        ref[0,4,2,0]=(-0.188892896681)/(o+1.)+(-0.212757962174)+(0.600204512934)*0.5**o
        ref[0,4,2,1]=(0.786135983408)/(o+1.)+(0.709443217024)+(-0.0813557008918)*0.5**o
        ref[1,0,0,0]=(-0.682877215956)/(o+1.)+(0.0430800979484)+(-0.786904593287)*0.5**o
        ref[1,0,0,1]=(-0.209743447547)/(o+1.)+(-0.529335530226)+(-0.351982026793)*0.5**o
        ref[1,0,1,0]=(1.7467904951)/(o+1.)+(-0.522734985482)+(0.205256143552)*0.5**o
        ref[1,0,1,1]=(-0.681949654071)/(o+1.)+(0.430192257885)+(-0.989555659669)*0.5**o
        ref[1,0,2,0]=(0.00559076557551)/(o+1.)+(-0.355981322114)+(-0.845595380938)*0.5**o
        ref[1,0,2,1]=(-1.0795557316)/(o+1.)+(0.433501713489)+(-0.971431930418)*0.5**o
        ref[1,1,0,0]=(-1.41228679449)/(o+1.)+(0.0214610301511)+(-0.47963962908)*0.5**o
        ref[1,1,0,1]=(0.675271467105)/(o+1.)+(0.258295057037)+(-0.872015819915)*0.5**o
        ref[1,1,1,0]=(-0.0723449260888)/(o+1.)+(0.642648455626)+(-0.504589981145)*0.5**o
        ref[1,1,1,1]=(1.69082429043)/(o+1.)+(0.351194447364)+(0.125923252805)*0.5**o
        ref[1,1,2,0]=(1.04100385874)/(o+1.)+(-0.253396731491)+(-0.320884200985)*0.5**o
        ref[1,1,2,1]=(1.44000111068)/(o+1.)+(0.167426476075)+(-0.284354022497)*0.5**o
        ref[1,2,0,0]=(0.762931449029)/(o+1.)+(-0.397841053459)+(-0.833497951611)*0.5**o
        ref[1,2,0,1]=(-0.353193148804)/(o+1.)+(0.729759415974)+(0.962868010939)*0.5**o
        ref[1,2,1,0]=(-0.0142626154573)/(o+1.)+(0.205452893539)+(-0.561643688565)*0.5**o
        ref[1,2,1,1]=(1.31669689043)/(o+1.)+(0.322616901969)+(-0.535545908666)*0.5**o
        ref[1,2,2,0]=(-0.720144981522)/(o+1.)+(-0.253513298251)+(0.251277061063)*0.5**o
        ref[1,2,2,1]=(0.171627338105)/(o+1.)+(0.304508788985)+(-0.0403839677009)*0.5**o
        ref[1,3,0,0]=(-0.385245021569)/(o+1.)+(0.0533837214678)+(-0.710193449549)*0.5**o
        ref[1,3,0,1]=(-1.66770468862)/(o+1.)+(0.0386999300625)+(0.404210496633)*0.5**o
        ref[1,3,1,0]=(0.1536232451)/(o+1.)+(-0.114369222402)+(0.919828553095)*0.5**o
        ref[1,3,1,1]=(0.279969861456)/(o+1.)+(0.199789198624)+(-0.874409680945)*0.5**o
        ref[1,3,2,0]=(-0.29883793799)/(o+1.)+(-0.179133605171)+(0.490232839865)*0.5**o
        ref[1,3,2,1]=(-0.20045707503)/(o+1.)+(-0.419654937503)+(-0.364153185689)*0.5**o
        ref[1,4,0,0]=(0.328269609451)/(o+1.)+(0.0976736768568)+(0.0671024149466)*0.5**o
        ref[1,4,0,1]=(0.70998344827)/(o+1.)+(-0.831520044753)+(0.929228475247)*0.5**o
        ref[1,4,1,0]=(1.14467737453)/(o+1.)+(-0.309242254267)+(-0.522855063115)*0.5**o
        ref[1,4,1,1]=(0.386397027677)/(o+1.)+(-0.152706345419)+(0.354258147311)*0.5**o
        ref[1,4,2,0]=(0.565989417809)/(o+1.)+(-0.0912836510331)+(0.410081961152)*0.5**o
        ref[1,4,2,1]=(-0.108228050157)/(o+1.)+(-0.382022658574)+(-0.0490935903447)*0.5**o
        ref[2,0,0,0]=(1.30816654656)/(o+1.)+(0.218850490887)+(-0.780250512205)*0.5**o
        ref[2,0,0,1]=(-0.072210934601)/(o+1.)+(-0.376572060485)+(-0.000679311415873)*0.5**o
        ref[2,0,1,0]=(-1.03731005117)/(o+1.)+(-0.369008331776)+(0.844980274869)*0.5**o
        ref[2,0,1,1]=(-0.37673073433)/(o+1.)+(0.147779920342)+(0.661285141912)*0.5**o
        ref[2,0,2,0]=(-0.126456722513)/(o+1.)+(-0.173173731574)+(0.250529661474)*0.5**o
        ref[2,0,2,1]=(0.269921444109)/(o+1.)+(0.256112039487)+(0.163733641849)*0.5**o
        ref[2,1,0,0]=(1.02033624645)/(o+1.)+(0.162073420045)+(-0.897290184569)*0.5**o
        ref[2,1,0,1]=(0.673075377238)/(o+1.)+(0.377904898438)+(0.615508721534)*0.5**o
        ref[2,1,1,0]=(-0.536750510163)/(o+1.)+(0.687164638156)+(-0.802573912815)*0.5**o
        ref[2,1,1,1]=(1.39522327504)/(o+1.)+(-0.168020484927)+(-0.558497203882)*0.5**o
        ref[2,1,2,0]=(-1.21446181317)/(o+1.)+(-0.219360528348)+(-0.293706468828)*0.5**o
        ref[2,1,2,1]=(0.352877691113)/(o+1.)+(0.0397286675094)+(0.16485787241)*0.5**o
        ref[2,2,0,0]=(-0.927761522597)/(o+1.)+(1.03238921925)+(-0.386688083878)*0.5**o
        ref[2,2,0,1]=(0.875914163347)/(o+1.)+(-0.0805989643579)+(0.535549891292)*0.5**o
        ref[2,2,1,0]=(-0.268027052879)/(o+1.)+(0.24501270411)+(-0.878958155751)*0.5**o
        ref[2,2,1,1]=(0.579267469532)/(o+1.)+(-0.419360478349)+(-0.758211472099)*0.5**o
        ref[2,2,2,0]=(-0.407532480549)/(o+1.)+(0.0236731772421)+(0.972067946527)*0.5**o
        ref[2,2,2,1]=(-0.848446514529)/(o+1.)+(0.986605187547)+(0.458500318263)*0.5**o
        ref[2,3,0,0]=(0.0139531165204)/(o+1.)+(0.0612674782454)+(-0.592960113562)*0.5**o
        ref[2,3,0,1]=(-0.434410189326)/(o+1.)+(0.0699472934754)+(-0.81298388773)*0.5**o
        ref[2,3,1,0]=(1.51687712753)/(o+1.)+(0.29518962448)+(0.671113725811)*0.5**o
        ref[2,3,1,1]=(-0.825018275156)/(o+1.)+(0.348395434187)+(-0.242658476441)*0.5**o
        ref[2,3,2,0]=(-0.129038376537)/(o+1.)+(0.0407049218889)+(-0.74100465183)*0.5**o
        ref[2,3,2,1]=(-0.61436863216)/(o+1.)+(0.0569955443553)+(-0.438752680853)*0.5**o
        ref[2,4,0,0]=(-0.186811568504)/(o+1.)+(0.324468688913)+(0.831953665661)*0.5**o
        ref[2,4,0,1]=(1.01471089788)/(o+1.)+(-0.404804936557)+(0.191447439569)*0.5**o
        ref[2,4,1,0]=(-0.833336708992)/(o+1.)+(-0.465420260939)+(0.571546652968)*0.5**o
        ref[2,4,1,1]=(1.48092456289)/(o+1.)+(0.548316338846)+(0.16098929463)*0.5**o
        ref[2,4,2,0]=(-1.20254311473)/(o+1.)+(0.778023417549)+(-0.631204318592)*0.5**o
        ref[2,4,2,1]=(0.672521404448)/(o+1.)+(0.265167920765)+(0.922455810129)*0.5**o
        ref[3,0,0,0]=(-0.400146773167)/(o+1.)+(0.151226335458)+(0.310554345086)*0.5**o
        ref[3,0,0,1]=(0.580039994556)/(o+1.)+(0.287523081772)+(0.843407699243)*0.5**o
        ref[3,0,1,0]=(0.930972332962)/(o+1.)+(0.768302886889)+(0.985902820768)*0.5**o
        ref[3,0,1,1]=(-0.991140747279)/(o+1.)+(-0.914680203238)+(0.757231124095)*0.5**o
        ref[3,0,2,0]=(-0.458287257967)/(o+1.)+(0.237995705359)+(-0.742604875255)*0.5**o
        ref[3,0,2,1]=(1.14385326779)/(o+1.)+(0.243645854371)+(-0.535987729627)*0.5**o
        ref[3,1,0,0]=(0.322354058507)/(o+1.)+(0.341162149011)+(-0.453130357106)*0.5**o
        ref[3,1,0,1]=(-1.78201691527)/(o+1.)+(1.04199074308)+(-0.324362188462)*0.5**o
        ref[3,1,1,0]=(-0.0578982797037)/(o+1.)+(-0.110933167998)+(0.522937033394)*0.5**o
        ref[3,1,1,1]=(-0.276009095212)/(o+1.)+(0.517113652949)+(-0.267978083771)*0.5**o
        ref[3,1,2,0]=(-0.023345636799)/(o+1.)+(-0.66878791133)+(0.497251194848)*0.5**o
        ref[3,1,2,1]=(1.2604140706)/(o+1.)+(-0.878310666552)+(0.461929649774)*0.5**o
        ref[3,2,0,0]=(-0.00211014675007)/(o+1.)+(-0.814091029436)+(0.555746667517)*0.5**o
        ref[3,2,0,1]=(-1.07650025357)/(o+1.)+(-0.26199073554)+(-0.691532951025)*0.5**o
        ref[3,2,1,0]=(-0.0770650645465)/(o+1.)+(-0.307619169254)+(-0.0398626743392)*0.5**o
        ref[3,2,1,1]=(1.12398638253)/(o+1.)+(-0.00626411324864)+(0.895535126103)*0.5**o
        ref[3,2,2,0]=(0.822758496096)/(o+1.)+(0.448681597953)+(-0.191398265373)*0.5**o
        ref[3,2,2,1]=(1.38402514586)/(o+1.)+(0.489672391609)+(0.589110983761)*0.5**o
        ref[3,3,0,0]=(0.626837013941)/(o+1.)+(-0.698370737203)+(-0.400999012316)*0.5**o
        ref[3,3,0,1]=(-0.819551723109)/(o+1.)+(0.588359566557)+(-0.811853216306)*0.5**o
        ref[3,3,1,0]=(-0.472944922304)/(o+1.)+(0.024839136659)+(-0.973132395795)*0.5**o
        ref[3,3,1,1]=(-0.674351724226)/(o+1.)+(-0.596697469314)+(0.0150225091087)*0.5**o
        ref[3,3,2,0]=(-0.653245573198)/(o+1.)+(0.590003731898)+(-0.0964433425189)*0.5**o
        ref[3,3,2,1]=(1.15741134315)/(o+1.)+(-0.128540702232)+(0.728925474609)*0.5**o
        ref[3,4,0,0]=(-0.745079746498)/(o+1.)+(-0.313462120963)+(0.562821413507)*0.5**o
        ref[3,4,0,1]=(-0.585232841208)/(o+1.)+(0.464633596697)+(-0.417618967733)*0.5**o
        ref[3,4,1,0]=(0.792309614109)/(o+1.)+(-0.478065247211)+(0.809069329896)*0.5**o
        ref[3,4,1,1]=(0.0536562315177)/(o+1.)+(-0.128980351611)+(0.821143622493)*0.5**o
        ref[3,4,2,0]=(-1.20913408459)/(o+1.)+(-0.144016051074)+(-0.0283874424941)*0.5**o
        ref[3,4,2,1]=(-1.12139150964)/(o+1.)+(-0.135856270303)+(-0.427480154859)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactOne_fromSymbol_ReducedContinuousFunction_rank0(self):
      """
      tests integral of rank 0 Symbol on the ReducedFunctionOnContactOne

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=())
      s=Data(0,(),w)
      if dim==2:
        s=(-0.0123111028353)*x[0]+(-1.54593060503)*x[1]
        ref=(-0.883444958139)/(o+1.)+(-0.0326056141412)+(-0.609585521441)*0.5**o
      else:
        s=(0.465974344755)*x[0]+(-0.406015068796)*x[1]+(0.217033091571)*x[2]
        ref=(-0.964783366814)/(o+1.)+(0.787210317182)+(-0.332644900019)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactOne_fromSymbol_ReducedContinuousFunction_rank1(self):
      """
      tests integral of rank 1 Symbol on the ReducedFunctionOnContactOne

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(2,))
      s=Data(0,(2,),w)
      ref=numpy.zeros((2,),numpy.float_)
      if dim==2:
        s[0]=(-1.01806458946)*x[0]+(-0.84209500794)*x[1]
        s[1]=(-0.372739669691)*x[0]+(0.123203057686)*x[1]
        ref[0]=(-0.910310319387)/(o+1.)+(-0.35457604)+(-0.240697198017)*0.5**o
        ref[1]=(0.817182634856)/(o+1.)+(-0.752992097502)+(0.439264948143)*0.5**o
      else:
        s[0]=(-0.262859740698)*x[0]+(0.398827301114)*x[1]+(-0.0121863377176)*x[2]
        s[1]=(0.533472917947)*x[0]+(-0.654130540619)*x[1]+(-0.155059686066)*x[2]
        ref[0]=(0.173661143447)/(o+1.)+(-0.0377591573443)+(0.0256383939398)*0.5**o
        ref[1]=(-1.0263552046)/(o+1.)+(0.265270569058)+(0.220096757743)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactOne_fromSymbol_ReducedContinuousFunction_rank2(self):
      """
      tests integral of rank 2 Symbol on the ReducedFunctionOnContactOne

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5))
      s=Data(0,(4, 5),w)
      ref=numpy.zeros((4, 5),numpy.float_)
      if dim==2:
        s[0,0]=(0.0355078592116)*x[0]+(-1.45322051958)*x[1]
        s[0,1]=(0.940649610535)*x[0]+(1.03783851181)*x[1]
        s[0,2]=(-0.285465603086)*x[0]+(-1.0279436539)*x[1]
        s[0,3]=(-1.05683604808)*x[0]+(-0.606790848933)*x[1]
        s[0,4]=(0.155725423448)*x[0]+(-0.0350991646899)*x[1]
        s[1,0]=(0.288671804193)*x[0]+(0.427933920845)*x[1]
        s[1,1]=(-0.0117065079514)*x[0]+(0.96117958225)*x[1]
        s[1,2]=(0.209419996754)*x[0]+(0.21574344987)*x[1]
        s[1,3]=(0.885781216051)*x[0]+(-0.68144471245)*x[1]
        s[1,4]=(-1.08228199466)*x[0]+(-0.155862749309)*x[1]
        s[2,0]=(-0.680644329614)*x[0]+(-1.38778484373)*x[1]
        s[2,1]=(0.0400929370385)*x[0]+(0.415428765055)*x[1]
        s[2,2]=(-1.20113508704)*x[0]+(-0.876683252298)*x[1]
        s[2,3]=(-0.312684596061)*x[0]+(-0.70367059229)*x[1]
        s[2,4]=(0.214059442951)*x[0]+(1.42513598809)*x[1]
        s[3,0]=(-0.24713062126)*x[0]+(1.02991464042)*x[1]
        s[3,1]=(-0.83089702409)*x[0]+(0.587925745944)*x[1]
        s[3,2]=(0.950881339072)*x[0]+(0.051413472076)*x[1]
        s[3,3]=(0.182953617513)*x[0]+(0.94052327152)*x[1]
        s[3,4]=(-0.387816734481)*x[0]+(0.366589363018)*x[1]
        ref[0,0]=(-0.563896045541)/(o+1.)+(-0.436916928584)+(0.0200172423412)*0.5**o
        ref[0,1]=(0.583712973931)/(o+1.)+(0.277276607579)+(0.84022193326)*0.5**o
        ref[0,2]=(-0.381794683559)/(o+1.)+(-0.29471407057)+(-0.342186432287)*0.5**o
        ref[0,3]=(0.0937197621552)/(o+1.)+(-0.417759182193)+(-0.921828294785)*0.5**o
        ref[0,4]=(-0.00732587396541)/(o+1.)+(0.282507495385)+(-0.437062858047)*0.5**o
        ref[1,0]=(0.656328782568)/(o+1.)+(0.105444180081)+(-0.150611417692)*0.5**o
        ref[1,1]=(0.0862490471738)/(o+1.)+(-0.0358008007742)+(0.934825628673)*0.5**o
        ref[1,2]=(-0.681994265679)/(o+1.)+(0.52707424273)+(0.053009226843)*0.5**o
        ref[1,3]=(0.162048053458)/(o+1.)+(-0.131185707156)+(0.304659864456)*0.5**o
        ref[1,4]=(-0.321375806412)/(o+1.)+(-0.12650756243)+(-0.663753812697)*0.5**o
        ref[2,0]=(-0.706054805711)/(o+1.)+(-0.805432593593)+(0.248490819557)*0.5**o
        ref[2,1]=(0.587809899116)/(o+1.)+(-0.0542554861756)+(-0.0237772246706)*0.5**o
        ref[2,2]=(0.044861811334)/(o+1.)+(-0.933527479587)+(-0.255625191498)*0.5**o
        ref[2,3]=(0.0752678267397)/(o+1.)+(-0.281993375709)+(-0.527636263672)*0.5**o
        ref[2,4]=(0.684054536011)/(o+1.)+(0.209350429588)+(0.536440035851)*0.5**o
        ref[3,0]=(0.827817530316)/(o+1.)+(-0.211443979463)+(0.37785444777)*0.5**o
        ref[3,1]=(-0.410871022592)/(o+1.)+(0.0530892951706)+(0.061721154105)*0.5**o
        ref[3,2]=(0.199285164042)/(o+1.)+(0.122582366509)+(0.557844914088)*0.5**o
        ref[3,3]=(0.195462635242)/(o+1.)+(0.027388051617)+(0.873238150557)*0.5**o
        ref[3,4]=(0.231128200422)/(o+1.)+(0.0245220097267)+(-0.301399591338)*0.5**o
      else:
        s[0,0]=(0.995116002112)*x[0]+(0.591812853606)*x[1]+(0.436070205257)*x[2]
        s[0,1]=(-0.118607336615)*x[0]+(0.594691767995)*x[1]+(-0.56002370897)*x[2]
        s[0,2]=(1.14520644838)*x[0]+(0.18460040375)*x[1]+(-0.960923519429)*x[2]
        s[0,3]=(0.392881958399)*x[0]+(-1.48738000894)*x[1]+(1.04349887295)*x[2]
        s[0,4]=(0.232444347548)*x[0]+(-0.83025700349)*x[1]+(0.612703936854)*x[2]
        s[1,0]=(0.190581558342)*x[0]+(-0.619707808792)*x[1]+(1.05804982356)*x[2]
        s[1,1]=(-0.42811715218)*x[0]+(-0.447035862369)*x[1]+(-1.24096539043)*x[2]
        s[1,2]=(0.200357664023)*x[0]+(-0.936410127129)*x[1]+(0.750394727948)*x[2]
        s[1,3]=(-0.612730981377)*x[0]+(-0.653699725295)*x[1]+(0.0015189792328)*x[2]
        s[1,4]=(0.288644454156)*x[0]+(0.93337277237)*x[1]+(-0.602600880867)*x[2]
        s[2,0]=(-1.17236877273)*x[0]+(1.45496830635)*x[1]+(-1.12383998094)*x[2]
        s[2,1]=(-0.232230799241)*x[0]+(0.415001155159)*x[1]+(-0.60248885989)*x[2]
        s[2,2]=(-0.83830117983)*x[0]+(0.192640431036)*x[1]+(1.43556749653)*x[2]
        s[2,3]=(0.973424823298)*x[0]+(0.369914981625)*x[1]+(-1.40643757872)*x[2]
        s[2,4]=(-0.458923994338)*x[0]+(-1.04785209961)*x[1]+(-0.592533584766)*x[2]
        s[3,0]=(0.545327477107)*x[0]+(1.9382915447)*x[1]+(1.47179032667)*x[2]
        s[3,1]=(-0.415696009498)*x[0]+(-0.594177620783)*x[1]+(-1.35108058064)*x[2]
        s[3,2]=(0.831178664645)*x[0]+(-0.0275877269769)*x[1]+(0.302241965885)*x[2]
        s[3,3]=(-0.360815522042)*x[0]+(-0.826462125533)*x[1]+(0.530840397679)*x[2]
        s[3,4]=(0.135928452703)*x[0]+(-1.51122126726)*x[1]+(1.6120424041)*x[2]
        ref[0,0]=(1.24742647646)/(o+1.)+(0.269060816234)+(0.237450952046)*0.5**o
        ref[0,1]=(1.12585957806)/(o+1.)+(-0.755828198463)+(0.301857541273)*0.5**o
        ref[0,2]=(-0.435989664675)/(o+1.)+(0.295471183049)+(0.213930631278)*0.5**o
        ref[0,3]=(-0.488921466935)/(o+1.)+(0.294480448259)+(-0.151038607178)*0.5**o
        ref[0,4]=(0.516230612817)/(o+1.)+(-0.0787984362944)+(-0.343742459316)*0.5**o
        ref[1,0]=(-0.492183705773)/(o+1.)+(0.501230271461)+(0.118646735962)*0.5**o
        ref[1,1]=(-1.43995574312)/(o+1.)+(-0.28437529362)+(-0.107412074623)*0.5**o
        ref[1,2]=(-0.101017880825)/(o+1.)+(0.28028462641)+(-0.445209107154)*0.5**o
        ref[1,3]=(0.128471434955)/(o+1.)+(-0.384371968608)+(-0.62463922518)*0.5**o
        ref[1,4]=(-0.0918241851534)/(o+1.)+(-0.0317649881343)+(0.774770507081)*0.5**o
        ref[2,0]=(0.221291051677)/(o+1.)+(-0.302519614649)+(-0.457492269704)*0.5**o
        ref[2,1]=(0.100276041699)/(o+1.)+(-0.387935114184)+(0.255875682695)*0.5**o
        ref[2,2]=(0.67334801807)/(o+1.)+(0.363637415491)+(-0.610716101317)*0.5**o
        ref[2,3]=(-0.101257531168)/(o+1.)+(-0.225887466089)+(0.48993468955)*0.5**o
        ref[2,4]=(-0.451085705642)/(o+1.)+(-0.415483032516)+(-0.817257908036)*0.5**o
        ref[3,0]=(1.87076639925)/(o+1.)+(1.20828670015)+(-0.331930451075)*0.5**o
        ref[3,1]=(-1.43689409978)/(o+1.)+(0.0171611774403)+(-0.958382466017)*0.5**o
        ref[3,2]=(0.146538702473)/(o+1.)+(0.391890493041)+(0.175513214998)*0.5**o
        ref[3,3]=(-0.122934034211)/(o+1.)+(0.140609173665)+(-0.814721563016)*0.5**o
        ref[3,4]=(0.223164895319)/(o+1.)+(-0.31244215916)+(0.638469012551)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactOne_fromSymbol_ReducedContinuousFunction_rank3(self):
      """
      tests integral of rank 3 Symbol on the ReducedFunctionOnContactOne

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(6, 2, 2))
      s=Data(0,(6, 2, 2),w)
      ref=numpy.zeros((6, 2, 2),numpy.float_)
      if dim==2:
        s[0,0,0]=(-0.35078097593)*x[0]+(0.306926522015)*x[1]
        s[0,0,1]=(-0.803534689121)*x[0]+(-0.0602880575367)*x[1]
        s[0,1,0]=(-0.903470951227)*x[0]+(0.374123966765)*x[1]
        s[0,1,1]=(-1.18641387512)*x[0]+(0.273430931834)*x[1]
        s[1,0,0]=(0.493295479958)*x[0]+(1.54668330013)*x[1]
        s[1,0,1]=(-1.27848534954)*x[0]+(0.0588492485708)*x[1]
        s[1,1,0]=(-1.42953381447)*x[0]+(0.717393314307)*x[1]
        s[1,1,1]=(-0.467730806598)*x[0]+(-0.0117815181086)*x[1]
        s[2,0,0]=(-0.602626661024)*x[0]+(0.533324435346)*x[1]
        s[2,0,1]=(-0.904855697799)*x[0]+(1.09112846759)*x[1]
        s[2,1,0]=(1.20360337188)*x[0]+(1.23097020284)*x[1]
        s[2,1,1]=(0.0885447539403)*x[0]+(0.405387510263)*x[1]
        s[3,0,0]=(-0.730436778413)*x[0]+(0.416740386984)*x[1]
        s[3,0,1]=(-1.04126166563)*x[0]+(0.41753641298)*x[1]
        s[3,1,0]=(-0.271195830488)*x[0]+(-0.11575841172)*x[1]
        s[3,1,1]=(1.16100889666)*x[0]+(-0.233929048364)*x[1]
        s[4,0,0]=(0.664549807695)*x[0]+(0.224807977257)*x[1]
        s[4,0,1]=(-0.796935211563)*x[0]+(0.365917637053)*x[1]
        s[4,1,0]=(0.0401977690429)*x[0]+(-0.612500495333)*x[1]
        s[4,1,1]=(-0.115172707709)*x[0]+(-0.280626474812)*x[1]
        s[5,0,0]=(0.844835935845)*x[0]+(-1.19344222698)*x[1]
        s[5,0,1]=(-1.81553680308)*x[0]+(-1.66095998384)*x[1]
        s[5,1,0]=(0.761230206372)*x[0]+(0.547313314376)*x[1]
        s[5,1,1]=(-0.246895651321)*x[0]+(0.391870038086)*x[1]
        ref[0,0,0]=(-0.600965801098)/(o+1.)+(0.338185808482)+(-0.119260269781)*0.5**o
        ref[0,0,1]=(-0.224867141426)/(o+1.)+(-0.334457471517)+(0.0299593378011)*0.5**o
        ref[0,1,0]=(0.0129821011627)/(o+1.)+(-0.0950564992708)+(-0.352216087084)*0.5**o
        ref[0,1,1]=(0.0332057375276)/(o+1.)+(-0.0551431610872)+(-0.835902358643)*0.5**o
        ref[1,0,0]=(0.553424856182)/(o+1.)+(0.848186123918)+(-0.209818323927)*0.5**o
        ref[1,0,1]=(-0.75127641037)/(o+1.)+(-0.0481529718983)+(-0.372053746804)*0.5**o
        ref[1,1,0]=(-0.218691700126)/(o+1.)+(0.0665842087159)+(-0.626617217469)*0.5**o
        ref[1,1,1]=(-0.751752416764)/(o+1.)+(0.165004967668)+(-0.0577698432779)*0.5**o
        ref[2,0,0]=(-0.0172781449053)/(o+1.)+(-0.197949307988)+(0.343874535203)*0.5**o
        ref[2,0,1]=(0.499625949394)/(o+1.)+(-0.0639621511173)+(-0.185428877371)*0.5**o
        ref[2,1,0]=(0.965002845499)/(o+1.)+(0.296405176523)+(0.87676037618)*0.5**o
        ref[2,1,1]=(0.571427043323)/(o+1.)+(0.355568778058)+(-0.788632335234)*0.5**o
        ref[3,0,0]=(0.643869395266)/(o+1.)+(-0.248603098629)+(-0.460359589438)*0.5**o
        ref[3,0,1]=(0.847552650465)/(o+1.)+(-0.471374980899)+(-0.528527941319)*0.5**o
        ref[3,1,0]=(0.792482290139)/(o+1.)+(-0.288835326975)+(-0.601765878397)*0.5**o
        ref[3,1,1]=(0.674315208514)/(o+1.)+(-0.301924186259)+(0.8566130123)*0.5**o
        ref[4,0,0]=(0.14482725218)/(o+1.)+(0.123667770392)+(0.497194991988)*0.5**o
        ref[4,0,1]=(0.207166197869)/(o+1.)+(-0.00140080206462)+(-0.63538216825)*0.5**o
        ref[4,1,0]=(0.134064796438)/(o+1.)+(-0.583980524751)+(0.461593526773)*0.5**o
        ref[4,1,1]=(0.329865203686)/(o+1.)+(-0.0277930553577)+(-0.670078275492)*0.5**o
        ref[5,0,0]=(-0.671043415594)/(o+1.)+(-0.0772895763368)+(0.477016277134)*0.5**o
        ref[5,0,1]=(-0.863660332362)/(o+1.)+(-0.862182891111)+(-0.888470672335)*0.5**o
        ref[5,1,0]=(0.734550839405)/(o+1.)+(-0.00320324272275)+(0.580399166789)*0.5**o
        ref[5,1,1]=(-0.186529971258)/(o+1.)+(0.409102503591)+(-0.486700649159)*0.5**o
      else:
        s[0,0,0]=(1.49896775273)*x[0]+(-1.78250968743)*x[1]+(1.41503337092)*x[2]
        s[0,0,1]=(-0.643818266916)*x[0]+(-0.375134170105)*x[1]+(0.650514899885)*x[2]
        s[0,1,0]=(-0.0528798419185)*x[0]+(0.665778795831)*x[1]+(1.21069663928)*x[2]
        s[0,1,1]=(-1.45464226351)*x[0]+(-0.19467401007)*x[1]+(0.442665314437)*x[2]
        s[1,0,0]=(-0.836797292596)*x[0]+(-0.0542158346848)*x[1]+(-0.54209479287)*x[2]
        s[1,0,1]=(-1.09189713442)*x[0]+(1.03696758011)*x[1]+(0.0722113737424)*x[2]
        s[1,1,0]=(-0.493099825558)*x[0]+(-0.914461955024)*x[1]+(1.50580577272)*x[2]
        s[1,1,1]=(-0.702256378373)*x[0]+(-1.55374444057)*x[1]+(0.187304514552)*x[2]
        s[2,0,0]=(-0.924126823838)*x[0]+(-1.20470503473)*x[1]+(-0.702166397011)*x[2]
        s[2,0,1]=(1.04661135244)*x[0]+(0.995774871402)*x[1]+(0.912661077993)*x[2]
        s[2,1,0]=(0.0694812274474)*x[0]+(-0.384400463672)*x[1]+(-1.27290008863)*x[2]
        s[2,1,1]=(-0.196348771836)*x[0]+(0.314419668134)*x[1]+(-0.147174148507)*x[2]
        s[3,0,0]=(0.555982166527)*x[0]+(1.26542592993)*x[1]+(1.94075238034)*x[2]
        s[3,0,1]=(-0.15925841434)*x[0]+(0.235963126853)*x[1]+(1.01719057653)*x[2]
        s[3,1,0]=(1.8766116496)*x[0]+(0.0416539079261)*x[1]+(0.692631820852)*x[2]
        s[3,1,1]=(-1.60594678123)*x[0]+(0.418185984317)*x[1]+(1.00380643717)*x[2]
        s[4,0,0]=(-0.496656665568)*x[0]+(-0.519100502539)*x[1]+(-0.402964765219)*x[2]
        s[4,0,1]=(0.810723701732)*x[0]+(-0.9121518445)*x[1]+(0.0693075114891)*x[2]
        s[4,1,0]=(-1.38980929277)*x[0]+(-0.135080762705)*x[1]+(-0.708040071312)*x[2]
        s[4,1,1]=(0.0366128814416)*x[0]+(-0.598343408328)*x[1]+(-1.05172726191)*x[2]
        s[5,0,0]=(0.626645586525)*x[0]+(0.368702379272)*x[1]+(1.06419625419)*x[2]
        s[5,0,1]=(0.823542930912)*x[0]+(-0.397291307792)*x[1]+(1.47677515709)*x[2]
        s[5,1,0]=(1.0044323164)*x[0]+(-0.400460024166)*x[1]+(0.0881220069594)*x[2]
        s[5,1,1]=(0.841418294962)*x[0]+(0.448350914459)*x[1]+(-0.535982888316)*x[2]
        ref[0,0,0]=(0.0352781393333)/(o+1.)+(0.227326591212)+(0.64156011446)*0.5**o
        ref[0,0,1]=(-0.122693252863)/(o+1.)+(-0.221154887636)+(0.196565490999)*0.5**o
        ref[0,1,0]=(0.784542061966)/(o+1.)+(0.731576295626)+(-0.424099060024)*0.5**o
        ref[0,1,1]=(0.118776006377)/(o+1.)+(-0.423066319495)+(-0.479294326528)*0.5**o
        ref[1,0,0]=(1.18086837157)/(o+1.)+(-1.14308640082)+(-0.327803490075)*0.5**o
        ref[1,0,1]=(1.11878511573)/(o+1.)+(-0.427948355911)+(-0.245606584475)*0.5**o
        ref[1,1,0]=(-0.256395599928)/(o+1.)+(0.22748609258)+(-0.100332593094)*0.5**o
        ref[1,1,1]=(-0.192853498607)/(o+1.)+(-0.925785926894)+(-0.0242709519946)*0.5**o
        ref[2,0,0]=(-0.415752649176)/(o+1.)+(-1.14572035003)+(-0.123804906338)*0.5**o
        ref[2,0,1]=(0.384199373535)/(o+1.)+(0.843162532691)+(0.884522862915)*0.5**o
        ref[2,1,0]=(-0.229595030341)/(o+1.)+(-1.14751601921)+(0.936807743902)*0.5**o
        ref[2,1,1]=(0.267255066685)/(o+1.)+(-0.491644195523)+(0.686930072154)*0.5**o
        ref[3,0,0]=(1.68424865452)/(o+1.)+(0.955116898624)+(0.167678025029)*0.5**o
        ref[3,0,1]=(0.0034339126924)/(o+1.)+(0.718824562093)+(-0.347187747836)*0.5**o
        ref[3,1,0]=(1.62164005566)/(o+1.)+(0.0562907078128)+(0.876675907094)*0.5**o
        ref[3,1,1]=(0.472424872079)/(o+1.)+(0.0308594354746)+(-0.718098102775)*0.5**o
        ref[4,0,0]=(-0.0285810697935)/(o+1.)+(-0.525693855415)+(-0.338753152702)*0.5**o
        ref[4,0,1]=(0.303057697082)/(o+1.)+(-0.350175453973)+(0.365172579586)*0.5**o
        ref[4,1,0]=(-1.04637813553)/(o+1.)+(-0.308024248014)+(-0.57050349523)*0.5**o
        ref[4,1,1]=(-0.500793745646)/(o+1.)+(-0.424344892355)+(-0.26397425844)*0.5**o
        ref[5,0,0]=(0.365122763119)/(o+1.)+(0.568706597744)+(0.557008261376)*0.5**o
        ref[5,0,1]=(0.510379226755)/(o+1.)+(0.436754108223)+(0.519139337008)*0.5**o
        ref[5,1,0]=(-0.200129774692)/(o+1.)+(0.274983568325)+(0.342256937234)*0.5**o
        ref[5,1,1]=(0.566120139584)/(o+1.)+(-0.312318326061)+(0.812302833644)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactOne_fromSymbol_ReducedContinuousFunction_rank4(self):
      """
      tests integral of rank 4 Symbol on the ReducedFunctionOnContactOne

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5, 3, 2))
      s=Data(0,(4, 5, 3, 2),w)
      ref=numpy.zeros((4, 5, 3, 2),numpy.float_)
      if dim==2:
        s[0,0,0,0]=(-0.36635416566)*x[0]+(0.454278450476)*x[1]
        s[0,0,0,1]=(0.449553060759)*x[0]+(-0.226772438798)*x[1]
        s[0,0,1,0]=(0.648343714053)*x[0]+(0.233007941149)*x[1]
        s[0,0,1,1]=(-0.135283076943)*x[0]+(1.61027554283)*x[1]
        s[0,0,2,0]=(0.619161973877)*x[0]+(-0.112220917964)*x[1]
        s[0,0,2,1]=(0.857902398581)*x[0]+(1.51928597673)*x[1]
        s[0,1,0,0]=(0.331827670107)*x[0]+(1.40775973855)*x[1]
        s[0,1,0,1]=(0.421840987115)*x[0]+(0.118563659741)*x[1]
        s[0,1,1,0]=(0.447268593931)*x[0]+(0.484984994669)*x[1]
        s[0,1,1,1]=(-0.189018546367)*x[0]+(0.467542988869)*x[1]
        s[0,1,2,0]=(0.466123494496)*x[0]+(-0.713267013868)*x[1]
        s[0,1,2,1]=(1.14105709142)*x[0]+(-0.134715682223)*x[1]
        s[0,2,0,0]=(0.871597208182)*x[0]+(-1.49080786854)*x[1]
        s[0,2,0,1]=(-0.94888847468)*x[0]+(0.0275999018363)*x[1]
        s[0,2,1,0]=(0.77534514887)*x[0]+(1.67162256756)*x[1]
        s[0,2,1,1]=(-0.786790506628)*x[0]+(-0.0692869424346)*x[1]
        s[0,2,2,0]=(1.78371863881)*x[0]+(-0.0481919137551)*x[1]
        s[0,2,2,1]=(-0.992074926087)*x[0]+(-1.02372593432)*x[1]
        s[0,3,0,0]=(0.113042447336)*x[0]+(-0.0790352303856)*x[1]
        s[0,3,0,1]=(1.12030253557)*x[0]+(0.393484268041)*x[1]
        s[0,3,1,0]=(0.379936292181)*x[0]+(-1.53869802379)*x[1]
        s[0,3,1,1]=(-0.191443496207)*x[0]+(0.563534464341)*x[1]
        s[0,3,2,0]=(0.104758691419)*x[0]+(-0.99439187659)*x[1]
        s[0,3,2,1]=(-0.479774987108)*x[0]+(-0.142765773584)*x[1]
        s[0,4,0,0]=(-1.50474690932)*x[0]+(-0.507691997905)*x[1]
        s[0,4,0,1]=(-0.78875103976)*x[0]+(0.132625919742)*x[1]
        s[0,4,1,0]=(-0.692847502554)*x[0]+(0.437815890612)*x[1]
        s[0,4,1,1]=(-0.808194455413)*x[0]+(0.376369555557)*x[1]
        s[0,4,2,0]=(0.919158598696)*x[0]+(-1.18391426084)*x[1]
        s[0,4,2,1]=(0.42374001855)*x[0]+(0.812305143875)*x[1]
        s[1,0,0,0]=(0.218364606199)*x[0]+(-1.40515806247)*x[1]
        s[1,0,0,1]=(-0.351603222911)*x[0]+(1.45105178061)*x[1]
        s[1,0,1,0]=(1.39849984972)*x[0]+(-1.35867734934)*x[1]
        s[1,0,1,1]=(-0.00360307599185)*x[0]+(0.246277359726)*x[1]
        s[1,0,2,0]=(0.514944968042)*x[0]+(0.428253029894)*x[1]
        s[1,0,2,1]=(-0.786120897197)*x[0]+(0.920810959626)*x[1]
        s[1,1,0,0]=(-0.602179171574)*x[0]+(0.515213890072)*x[1]
        s[1,1,0,1]=(-0.0861102153588)*x[0]+(0.477359706262)*x[1]
        s[1,1,1,0]=(-0.688164903151)*x[0]+(-0.313549483397)*x[1]
        s[1,1,1,1]=(-0.429990373791)*x[0]+(-0.0543708786137)*x[1]
        s[1,1,2,0]=(-0.39223682603)*x[0]+(0.796504408378)*x[1]
        s[1,1,2,1]=(-1.69046328113)*x[0]+(-0.344952802268)*x[1]
        s[1,2,0,0]=(-0.528152430512)*x[0]+(-0.269147355)*x[1]
        s[1,2,0,1]=(-1.68741664082)*x[0]+(0.568028242365)*x[1]
        s[1,2,1,0]=(-0.076694122635)*x[0]+(0.512745790897)*x[1]
        s[1,2,1,1]=(-0.738719294653)*x[0]+(1.72566985283)*x[1]
        s[1,2,2,0]=(0.679026798185)*x[0]+(-0.766391547753)*x[1]
        s[1,2,2,1]=(-1.7766436376)*x[0]+(-0.626962152238)*x[1]
        s[1,3,0,0]=(-0.0629592766364)*x[0]+(-0.602623643371)*x[1]
        s[1,3,0,1]=(1.46050982285)*x[0]+(-1.29912311111)*x[1]
        s[1,3,1,0]=(0.565052310821)*x[0]+(0.451146130683)*x[1]
        s[1,3,1,1]=(-1.72352086869)*x[0]+(-0.729969658338)*x[1]
        s[1,3,2,0]=(-0.699562180077)*x[0]+(0.785838373001)*x[1]
        s[1,3,2,1]=(0.0504742963364)*x[0]+(-0.856596707973)*x[1]
        s[1,4,0,0]=(-0.220685313017)*x[0]+(1.49415344926)*x[1]
        s[1,4,0,1]=(0.965835473202)*x[0]+(-0.664523896748)*x[1]
        s[1,4,1,0]=(-0.643560593026)*x[0]+(-1.06826452446)*x[1]
        s[1,4,1,1]=(-0.347993256951)*x[0]+(0.240926576999)*x[1]
        s[1,4,2,0]=(-0.666305003293)*x[0]+(1.5567540195)*x[1]
        s[1,4,2,1]=(0.752122481773)*x[0]+(-0.940078899457)*x[1]
        s[2,0,0,0]=(-0.428935565836)*x[0]+(-0.471904780642)*x[1]
        s[2,0,0,1]=(0.157225979958)*x[0]+(-0.707126471203)*x[1]
        s[2,0,1,0]=(-0.127873936162)*x[0]+(-1.01452234653)*x[1]
        s[2,0,1,1]=(-1.43314905378)*x[0]+(1.09406218538)*x[1]
        s[2,0,2,0]=(0.125475529763)*x[0]+(-0.144317966895)*x[1]
        s[2,0,2,1]=(0.781409858349)*x[0]+(-0.27747495012)*x[1]
        s[2,1,0,0]=(-1.21917608112)*x[0]+(-0.785716389383)*x[1]
        s[2,1,0,1]=(-0.26082261117)*x[0]+(-0.376402142009)*x[1]
        s[2,1,1,0]=(-0.125232448715)*x[0]+(0.261783624866)*x[1]
        s[2,1,1,1]=(-1.25985888818)*x[0]+(-0.0711437757878)*x[1]
        s[2,1,2,0]=(-0.300629742217)*x[0]+(-0.46939686121)*x[1]
        s[2,1,2,1]=(-1.77973256304)*x[0]+(0.713178700308)*x[1]
        s[2,2,0,0]=(-0.900064650204)*x[0]+(0.507949604762)*x[1]
        s[2,2,0,1]=(-0.885252734166)*x[0]+(-0.154018162942)*x[1]
        s[2,2,1,0]=(-0.7944479569)*x[0]+(1.77390521085)*x[1]
        s[2,2,1,1]=(1.1725888609)*x[0]+(0.662244216625)*x[1]
        s[2,2,2,0]=(-0.236310280848)*x[0]+(-0.542661345078)*x[1]
        s[2,2,2,1]=(1.15526522118)*x[0]+(0.0247167490822)*x[1]
        s[2,3,0,0]=(0.175989767802)*x[0]+(1.43963679604)*x[1]
        s[2,3,0,1]=(-0.692773425778)*x[0]+(-0.297594800061)*x[1]
        s[2,3,1,0]=(-0.847486832282)*x[0]+(0.0810250645286)*x[1]
        s[2,3,1,1]=(-0.681973624741)*x[0]+(-1.24971452577)*x[1]
        s[2,3,2,0]=(-1.06756371509)*x[0]+(0.977349108627)*x[1]
        s[2,3,2,1]=(0.846764513647)*x[0]+(-0.547998096826)*x[1]
        s[2,4,0,0]=(0.634255041776)*x[0]+(0.339363455414)*x[1]
        s[2,4,0,1]=(-1.39093420159)*x[0]+(0.716421880025)*x[1]
        s[2,4,1,0]=(0.257472890363)*x[0]+(-0.197982551362)*x[1]
        s[2,4,1,1]=(-0.45529259808)*x[0]+(-0.761148165674)*x[1]
        s[2,4,2,0]=(0.955343907343)*x[0]+(-1.29615358423)*x[1]
        s[2,4,2,1]=(0.362935017626)*x[0]+(1.25503750319)*x[1]
        s[3,0,0,0]=(0.179219886497)*x[0]+(1.78329032848)*x[1]
        s[3,0,0,1]=(0.43735751276)*x[0]+(0.0403938192717)*x[1]
        s[3,0,1,0]=(1.11478662344)*x[0]+(-0.240063001264)*x[1]
        s[3,0,1,1]=(0.172583585916)*x[0]+(0.506428670417)*x[1]
        s[3,0,2,0]=(1.33494651584)*x[0]+(0.781224884171)*x[1]
        s[3,0,2,1]=(0.0432954454652)*x[0]+(-0.68034279444)*x[1]
        s[3,1,0,0]=(-0.856892482415)*x[0]+(0.836687116891)*x[1]
        s[3,1,0,1]=(0.487431626464)*x[0]+(-0.148084918499)*x[1]
        s[3,1,1,0]=(0.331122282784)*x[0]+(-1.38429248259)*x[1]
        s[3,1,1,1]=(-0.215232051361)*x[0]+(0.537089578691)*x[1]
        s[3,1,2,0]=(1.24485548633)*x[0]+(0.509639847837)*x[1]
        s[3,1,2,1]=(-0.321450199534)*x[0]+(0.211384054649)*x[1]
        s[3,2,0,0]=(0.355385603273)*x[0]+(0.928139739798)*x[1]
        s[3,2,0,1]=(0.927589198094)*x[0]+(1.38324753996)*x[1]
        s[3,2,1,0]=(-1.16347040084)*x[0]+(-0.351854201414)*x[1]
        s[3,2,1,1]=(0.717488068131)*x[0]+(-0.0977120861443)*x[1]
        s[3,2,2,0]=(-0.799786558433)*x[0]+(0.829503505995)*x[1]
        s[3,2,2,1]=(1.37899235577)*x[0]+(-1.77910476445)*x[1]
        s[3,3,0,0]=(0.763361907215)*x[0]+(-0.861336281243)*x[1]
        s[3,3,0,1]=(0.476903208654)*x[0]+(0.903897305941)*x[1]
        s[3,3,1,0]=(1.60374028575)*x[0]+(-0.315620315575)*x[1]
        s[3,3,1,1]=(0.972809255696)*x[0]+(0.363233068227)*x[1]
        s[3,3,2,0]=(0.602799894079)*x[0]+(1.20772970348)*x[1]
        s[3,3,2,1]=(-0.000443665606339)*x[0]+(-1.6279763946)*x[1]
        s[3,4,0,0]=(-1.26676161275)*x[0]+(0.148063159145)*x[1]
        s[3,4,0,1]=(0.322891919608)*x[0]+(-1.08096025287)*x[1]
        s[3,4,1,0]=(-0.0944277709182)*x[0]+(-0.824212826197)*x[1]
        s[3,4,1,1]=(1.13530096846)*x[0]+(0.217321895058)*x[1]
        s[3,4,2,0]=(1.25663262459)*x[0]+(-0.308861390599)*x[1]
        s[3,4,2,1]=(-0.379182493224)*x[0]+(0.331110509538)*x[1]
        ref[0,0,0,0]=(0.408958259232)/(o+1.)+(0.247137449101)+(-0.815308872619)*0.5**o
        ref[0,0,0,1]=(0.13228131757)/(o+1.)+(0.165819155398)+(-0.241139006405)*0.5**o
        ref[0,0,1,0]=(0.128399251556)/(o+1.)+(0.371321245227)+(0.0103099131928)*0.5**o
        ref[0,0,1,1]=(0.966281299065)/(o+1.)+(0.381611325047)+(-0.254511483272)*0.5**o
        ref[0,0,2,0]=(-0.200167961434)/(o+1.)+(-0.0928181112535)+(0.892745239853)*0.5**o
        ref[0,0,2,1]=(0.830787814329)/(o+1.)+(0.314595496988)+(0.917209567011)*0.5**o
        ref[0,1,0,0]=(0.612992604133)/(o+1.)+(0.66241770717)+(-0.19824060982)*0.5**o
        ref[0,1,0,1]=(-0.0163161609645)/(o+1.)+(0.2025662158)+(0.151588376221)*0.5**o
        ref[0,1,1,0]=(-0.126917561445)/(o+1.)+(0.630548380093)+(-0.201925610142)*0.5**o
        ref[0,1,1,1]=(0.819096391825)/(o+1.)+(-0.370043224043)+(0.199514498764)*0.5**o
        ref[0,1,2,0]=(-0.96777130286)/(o+1.)+(0.367817797326)+(-0.0150078111633)*0.5**o
        ref[0,1,2,1]=(0.045654631235)/(o+1.)+(0.396367346781)+(0.167952084398)*0.5**o
        ref[0,2,0,0]=(-0.784765277575)/(o+1.)+(-0.298358907675)+(0.762272432566)*0.5**o
        ref[0,2,0,1]=(0.386572441281)/(o+1.)+(-0.51566858578)+(-0.276523842565)*0.5**o
        ref[0,2,1,0]=(0.680861847658)/(o+1.)+(0.740328104187)+(0.285449660402)*0.5**o
        ref[0,2,1,1]=(0.325836705829)/(o+1.)+(-0.49316269795)+(-0.195588758992)*0.5**o
        ref[0,2,2,0]=(0.783589269167)/(o+1.)+(0.0810542535881)+(0.789828948707)*0.5**o
        ref[0,2,2,1]=(-0.370218192335)/(o+1.)+(-0.608350209702)+(-0.428882248665)*0.5**o
        ref[0,3,0,0]=(0.188865301872)/(o+1.)+(-0.465976959809)+(0.777095834696)*0.5**o
        ref[0,3,0,1]=(0.764156203059)/(o+1.)+(-0.0928154457191)+(0.935261491991)*0.5**o
        ref[0,3,1,0]=(-0.689450927985)/(o+1.)+(-0.664550854911)+(0.8597909062)*0.5**o
        ref[0,3,1,1]=(0.555793140817)/(o+1.)+(0.142885227944)+(-0.469472628571)*0.5**o
        ref[0,3,2,0]=(-0.625798760976)/(o+1.)+(-0.499725272457)+(0.735616120719)*0.5**o
        ref[0,3,2,1]=(-0.907491695474)/(o+1.)+(0.385181050937)+(-0.485411167091)*0.5**o
        ref[0,4,0,0]=(-0.879336661239)/(o+1.)+(-0.205953419637)+(-0.721195406712)*0.5**o
        ref[0,4,0,1]=(0.86693894952)/(o+1.)+(-0.819677383073)+(0.11629069661)*0.5**o
        ref[0,4,1,0]=(-0.310289123464)/(o+1.)+(0.401826760004)+(-0.748396008487)*0.5**o
        ref[0,4,1,1]=(0.931547956812)/(o+1.)+(-0.740019832824)+(0.116666808981)*0.5**o
        ref[0,4,2,0]=(-0.847165088314)/(o+1.)+(-0.124403947896)+(0.831217321961)*0.5**o
        ref[0,4,2,1]=(0.177734973756)/(o+1.)+(0.441537508054)+(0.175235172561)*0.5**o
        ref[1,0,0,0]=(-0.542365809895)/(o+1.)+(-0.0827972838196)+(-0.478833078736)*0.5**o
        ref[1,0,0,1]=(0.844666854947)/(o+1.)+(0.498180748967)+(-0.741579795179)*0.5**o
        ref[1,0,1,0]=(-0.491126713824)/(o+1.)+(0.0335621009267)+(0.463825012354)*0.5**o
        ref[1,0,1,1]=(-0.309670980539)/(o+1.)+(0.472409867826)+(-0.392474471379)*0.5**o
        ref[1,0,2,0]=(0.278634540091)/(o+1.)+(-0.121141274647)+(0.906846007138)*0.5**o
        ref[1,0,2,1]=(-0.00603476025805)/(o+1.)+(0.182935877942)+(-0.225146933197)*0.5**o
        ref[1,1,0,0]=(0.580918965968)/(o+1.)+(0.0504878873855)+(-0.768860022241)*0.5**o
        ref[1,1,0,1]=(0.631736797799)/(o+1.)+(-0.537539597912)+(0.834591888928)*0.5**o
        ref[1,1,1,0]=(0.291226625025)/(o+1.)+(-0.703641010727)+(0.11434100988)*0.5**o
        ref[1,1,1,1]=(-0.909671911939)/(o+1.)+(0.237461939347)+(-0.0496132191606)*0.5**o
        ref[1,1,2,0]=(0.21073170992)/(o+1.)+(0.325715980645)+(-0.457896088863)*0.5**o
        ref[1,1,2,1]=(-0.725096486574)/(o+1.)+(-0.296742654874)+(-0.716834287076)*0.5**o
        ref[1,2,0,0]=(-0.164851515918)/(o+1.)+(-0.100676749276)+(-0.431094771042)*0.5**o
        ref[1,2,0,1]=(0.294840872181)/(o+1.)+(-0.329953166746)+(-0.754322937148)*0.5**o
        ref[1,2,1,0]=(0.341767621978)/(o+1.)+(0.400623115147)+(-0.70696218401)*0.5**o
        ref[1,2,1,1]=(0.821671220024)/(o+1.)+(0.193132433528)+(-0.220985528909)*0.5**o
        ref[1,2,2,0]=(-0.135508676597)/(o+1.)+(-0.38093100269)+(0.81000593241)*0.5**o
        ref[1,2,2,1]=(0.101637058171)/(o+1.)+(-0.771937440463)+(-0.961367967078)*0.5**o
        ref[1,3,0,0]=(-0.354951988253)/(o+1.)+(-0.41833260094)+(0.526034270126)*0.5**o
        ref[1,3,0,1]=(-0.461724677474)/(o+1.)+(-0.082145170992)+(0.787401731204)*0.5**o
        ref[1,3,1,0]=(0.681652074623)/(o+1.)+(-0.127199759209)+(0.5889458853)*0.5**o
        ref[1,3,1,1]=(-0.816769655641)/(o+1.)+(-0.382750879464)+(-0.871219112459)*0.5**o
        ref[1,3,2,0]=(0.549856768471)/(o+1.)+(-0.136659604755)+(-0.190261366036)*0.5**o
        ref[1,3,2,1]=(-0.298435559802)/(o+1.)+(-0.364033867566)+(0.220380883298)*0.5**o
        ref[1,4,0,0]=(0.692982064919)/(o+1.)+(0.445763556713)+(-0.311041042098)*0.5**o
        ref[1,4,0,1]=(-0.0284802774195)/(o+1.)+(0.173564544909)+(-0.0173372359445)*0.5**o
        ref[1,4,1,0]=(-0.521190661286)/(o+1.)+(-0.19266811161)+(-0.805298232979)*0.5**o
        ref[1,4,1,1]=(-0.344391605752)/(o+1.)+(0.344434513106)+(-0.451544100412)*0.5**o
        ref[1,4,2,0]=(0.590871123139)/(o+1.)+(0.123309088073)+(0.0529597169219)*0.5**o
        ref[1,4,2,1]=(-0.00996351412971)/(o+1.)+(-0.328705279006)+(0.479417654458)*0.5**o
        ref[2,0,0,0]=(0.220674489827)/(o+1.)+(-0.618024232079)+(0.114533627854)*0.5**o
        ref[2,0,0,1]=(-0.0844798976424)/(o+1.)+(0.070811636225)+(-0.607043866052)*0.5**o
        ref[2,0,1,0]=(-0.445683449242)/(o+1.)+(-0.113440437004)+(-0.469831959437)*0.5**o
        ref[2,0,1,1]=(0.271633369975)/(o+1.)+(0.132535807211)+(-0.875791852797)*0.5**o
        ref[2,0,2,0]=(0.649542525852)/(o+1.)+(0.0590165361575)+(-0.786418035298)*0.5**o
        ref[2,0,2,1]=(-0.317621366014)/(o+1.)+(0.329998426635)+(0.161559420974)*0.5**o
        ref[2,1,0,0]=(-0.835806177468)/(o+1.)+(-0.214223776118)+(-0.7406387408)*0.5**o
        ref[2,1,0,1]=(0.429741516432)/(o+1.)+(-0.552565623152)+(0.0381649766927)*0.5**o
        ref[2,1,1,0]=(-0.0472916937886)/(o+1.)+(-0.00552013887145)+(0.194883147682)*0.5**o
        ref[2,1,1,1]=(-0.199584048679)/(o+1.)+(-0.27639819939)+(-0.578622216512)*0.5**o
        ref[2,1,2,0]=(-0.941086888016)/(o+1.)+(-0.0385375072603)+(0.24813529911)*0.5**o
        ref[2,1,2,1]=(-0.0868737022971)/(o+1.)+(-0.0970507102796)+(-0.785578739872)*0.5**o
        ref[2,2,0,0]=(-0.202279119459)/(o+1.)+(-0.0839554337907)+(-0.0219250584016)*0.5**o
        ref[2,2,0,1]=(-0.184000614671)/(o+1.)+(-0.477928353442)+(0.100586424448)*0.5**o
        ref[2,2,1,0]=(0.927675340201)/(o+1.)+(0.0186691089268)+(0.0144436958933)*0.5**o
        ref[2,2,1,1]=(0.304792611717)/(o+1.)+(0.459166286092)+(0.611707893628)*0.5**o
        ref[2,2,2,0]=(0.0815687138062)/(o+1.)+(-0.768726979684)+(0.676913619635)*0.5**o
        ref[2,2,2,1]=(0.849269282518)/(o+1.)+(-0.25478410885)+(0.840280905449)*0.5**o
        ref[2,3,0,0]=(0.820793959083)/(o+1.)+(0.575169789593)+(-0.355506974424)*0.5**o
        ref[2,3,0,1]=(0.283603638973)/(o+1.)+(-0.335480628352)+(-0.603010608107)*0.5**o
        ref[2,3,1,0]=(-0.381013144765)/(o+1.)+(0.190974767434)+(-0.767398157855)*0.5**o
        ref[2,3,1,1]=(-0.773551356194)/(o+1.)+(-0.620890671367)+(0.0836445484134)*0.5**o
        ref[2,3,2,0]=(0.570212356041)/(o+1.)+(-0.289714757955)+(-0.0809974465983)*0.5**o
        ref[2,3,2,1]=(0.21581143181)/(o+1.)+(-0.415400501134)+(0.913755987278)*0.5**o
        ref[2,4,0,0]=(0.323568653669)/(o+1.)+(0.0799583453359)+(0.490133152849)*0.5**o
        ref[2,4,0,1]=(0.532408915634)/(o+1.)+(-0.115548404995)+(-0.97582442721)*0.5**o
        ref[2,4,1,0]=(-0.82114168932)/(o+1.)+(0.702564400414)+(-0.524496772508)*0.5**o
        ref[2,4,1,1]=(-0.479224200985)/(o+1.)+(-0.49364246599)+(0.25006836921)*0.5**o
        ref[2,4,2,0]=(-0.389740913016)/(o+1.)+(-0.0884282111182)+(0.225787658364)*0.5**o
        ref[2,4,2,1]=(0.508843816312)/(o+1.)+(0.17685868642)+(0.755411331668)*0.5**o
        ref[3,0,0,0]=(0.954468971244)/(o+1.)+(0.671012220464)+(-0.333983197198)*0.5**o
        ref[3,0,0,1]=(-0.312850718368)/(o+1.)+(-0.0723748107028)+(0.935351671806)*0.5**o
        ref[3,0,1,0]=(0.0640122675963)/(o+1.)+(0.220683721613)+(0.36934391135)*0.5**o
        ref[3,0,1,1]=(0.855735115439)/(o+1.)+(-0.366272230844)+(0.555821602582)*0.5**o
        ref[3,0,2,0]=(0.0417837662591)/(o+1.)+(0.556307682679)+(0.961772268394)*0.5**o
        ref[3,0,2,1]=(-0.928375029285)/(o+1.)+(-0.133425494561)+(0.558178669432)*0.5**o
        ref[3,1,0,0]=(0.610808931498)/(o+1.)+(-0.00371515949037)+(-0.623583978041)*0.5**o
        ref[3,1,0,1]=(-0.274141402005)/(o+1.)+(0.180852020544)+(0.251784068882)*0.5**o
        ref[3,1,1,0]=(-0.481032880244)/(o+1.)+(-0.705203882431)+(0.838270445302)*0.5**o
        ref[3,1,1,1]=(0.389002730435)/(o+1.)+(0.371327956101)+(-0.809801115307)*0.5**o
        ref[3,1,2,0]=(-0.217052039031)/(o+1.)+(0.504898297656)+(0.961750777882)*0.5**o
        ref[3,1,2,1]=(0.983589747751)/(o+1.)+(-0.768560013671)+(0.443464134708)*0.5**o
        ref[3,2,0,0]=(0.0850720894356)/(o+1.)+(0.295360321829)+(0.607732609977)*0.5**o
        ref[3,2,0,1]=(0.812866987322)/(o+1.)+(0.672388816065)+(0.153192118599)*0.5**o
        ref[3,2,1,0]=(-0.947419039593)/(o+1.)+(-0.179746473508)+(-0.208412615647)*0.5**o
        ref[3,2,1,1]=(-0.694332599484)/(o+1.)+(0.574069301711)+(0.165969978048)*0.5**o
        ref[3,2,2,0]=(0.746797880687)/(o+1.)+(0.0731831278015)+(-0.863447188728)*0.5**o
        ref[3,2,2,1]=(-0.9955274799)/(o+1.)+(-0.100141136686)+(0.795697344588)*0.5**o
        ref[3,3,0,0]=(-0.874362496111)/(o+1.)+(0.185559038089)+(0.405270045904)*0.5**o
        ref[3,3,0,1]=(0.557583002677)/(o+1.)+(0.556451155279)+(-0.28968479864)*0.5**o
        ref[3,3,1,0]=(0.435966507789)/(o+1.)+(0.00561393027505)+(0.840925601832)*0.5**o
        ref[3,3,1,1]=(-0.277310702324)/(o+1.)+(0.808938926899)+(-0.00452482755147)*0.5**o
        ref[3,3,2,0]=(0.878069034303)/(o+1.)+(0.0250230902204)+(0.882414382818)*0.5**o
        ref[3,3,2,1]=(-0.953670613054)/(o+1.)+(-0.626136165517)+(0.577522883879)*0.5**o
        ref[3,4,0,0]=(-0.0572166464991)/(o+1.)+(-0.195764673671)+(-0.669952459767)*0.5**o
        ref[3,4,0,1]=(-0.611801368818)/(o+1.)+(0.23313248955)+(-0.612531943539)*0.5**o
        ref[3,4,1,0]=(0.0451584859358)/(o+1.)+(-0.693097871519)+(0.422396659987)*0.5**o
        ref[3,4,1,1]=(0.0616735716897)/(o+1.)+(0.360358324628)+(0.57023264257)*0.5**o
        ref[3,4,2,0]=(0.404353246317)/(o+1.)+(0.02838608211)+(0.486645823449)*0.5**o
        ref[3,4,2,1]=(0.0039199584997)/(o+1.)+(0.344800813187)+(-0.741593568559)*0.5**o
      else:
        s[0,0,0,0]=(-0.00215071746669)*x[0]+(-0.332788203774)*x[1]+(-1.01240104354)*x[2]
        s[0,0,0,1]=(0.534102846033)*x[0]+(0.432162550346)*x[1]+(0.382807104545)*x[2]
        s[0,0,1,0]=(-0.774071131493)*x[0]+(0.174280314187)*x[1]+(0.235599208299)*x[2]
        s[0,0,1,1]=(1.66560900439)*x[0]+(1.38276801197)*x[1]+(0.451912961751)*x[2]
        s[0,0,2,0]=(-0.308642285607)*x[0]+(0.925763055269)*x[1]+(1.30533127575)*x[2]
        s[0,0,2,1]=(-0.15617828399)*x[0]+(-0.405834652149)*x[1]+(-0.846888299486)*x[2]
        s[0,1,0,0]=(-0.0743207250799)*x[0]+(-0.0831917437929)*x[1]+(-1.78311394968)*x[2]
        s[0,1,0,1]=(1.24178762381)*x[0]+(-0.121417608169)*x[1]+(0.0528284855272)*x[2]
        s[0,1,1,0]=(-1.14437209159)*x[0]+(0.0618138540822)*x[1]+(0.19124990065)*x[2]
        s[0,1,1,1]=(0.854246128467)*x[0]+(1.56688838996)*x[1]+(1.30424035038)*x[2]
        s[0,1,2,0]=(1.3184765993)*x[0]+(-1.40883832059)*x[1]+(0.832393276179)*x[2]
        s[0,1,2,1]=(-1.1625604886)*x[0]+(-0.104956544515)*x[1]+(-1.52389901089)*x[2]
        s[0,2,0,0]=(-0.106003547387)*x[0]+(0.852941663718)*x[1]+(0.24003638122)*x[2]
        s[0,2,0,1]=(0.892312082297)*x[0]+(-0.156996856451)*x[1]+(-1.2674975556)*x[2]
        s[0,2,1,0]=(0.0300043771968)*x[0]+(-0.848942993281)*x[1]+(-0.322425526609)*x[2]
        s[0,2,1,1]=(-0.361971760465)*x[0]+(1.89870303113)*x[1]+(0.861574517617)*x[2]
        s[0,2,2,0]=(0.97819437229)*x[0]+(-0.132714911288)*x[1]+(-0.957702168325)*x[2]
        s[0,2,2,1]=(0.588644003365)*x[0]+(-0.568265700558)*x[1]+(0.150521572866)*x[2]
        s[0,3,0,0]=(0.700807304996)*x[0]+(-1.14422226798)*x[1]+(0.587223436541)*x[2]
        s[0,3,0,1]=(-0.862702931786)*x[0]+(0.423016472206)*x[1]+(-0.555427243683)*x[2]
        s[0,3,1,0]=(0.574966038173)*x[0]+(-0.487199028698)*x[1]+(-0.32389521042)*x[2]
        s[0,3,1,1]=(0.130862033386)*x[0]+(0.42056248135)*x[1]+(0.925321527155)*x[2]
        s[0,3,2,0]=(-0.735071923108)*x[0]+(-0.528993360596)*x[1]+(0.485622476133)*x[2]
        s[0,3,2,1]=(0.126763646072)*x[0]+(0.0323222279682)*x[1]+(1.42515358228)*x[2]
        s[0,4,0,0]=(-0.334189936412)*x[0]+(0.14123257798)*x[1]+(-0.487379116976)*x[2]
        s[0,4,0,1]=(-0.528967405148)*x[0]+(-0.0622502130673)*x[1]+(-1.22107834734)*x[2]
        s[0,4,1,0]=(-1.67869230303)*x[0]+(-0.427809293008)*x[1]+(0.35027727689)*x[2]
        s[0,4,1,1]=(-1.3023324224)*x[0]+(0.364963594938)*x[1]+(1.25798740064)*x[2]
        s[0,4,2,0]=(0.789575151987)*x[0]+(-1.08985193505)*x[1]+(0.286072474971)*x[2]
        s[0,4,2,1]=(0.0321971599533)*x[0]+(1.64269617809)*x[1]+(0.448773378519)*x[2]
        s[1,0,0,0]=(-0.206961758788)*x[0]+(0.188339936734)*x[1]+(-1.36499979129)*x[2]
        s[1,0,0,1]=(-0.439106473295)*x[0]+(-1.35694867727)*x[1]+(0.17565861577)*x[2]
        s[1,0,1,0]=(0.754962041339)*x[0]+(0.377298691237)*x[1]+(-0.225684064888)*x[2]
        s[1,0,1,1]=(-0.164606093877)*x[0]+(-1.12165840203)*x[1]+(0.475143697931)*x[2]
        s[1,0,2,0]=(-0.563561093763)*x[0]+(-0.605592098451)*x[1]+(-0.382814067375)*x[2]
        s[1,0,2,1]=(-0.529311859218)*x[0]+(-0.440753365779)*x[1]+(-0.213919010047)*x[2]
        s[1,1,0,0]=(-0.381937118975)*x[0]+(0.115015087)*x[1]+(-1.58208233129)*x[2]
        s[1,1,0,1]=(-0.898803250164)*x[0]+(1.21794054101)*x[1]+(0.000708470416828)*x[2]
        s[1,1,1,0]=(0.417980628363)*x[0]+(-0.89943521283)*x[1]+(1.18981658848)*x[2]
        s[1,1,1,1]=(0.400600150308)*x[0]+(1.59168041332)*x[1]+(0.526855874334)*x[2]
        s[1,1,2,0]=(-0.658036770065)*x[0]+(0.903572546571)*x[1]+(-0.0322095817377)*x[2]
        s[1,1,2,1]=(-0.487427224544)*x[0]+(0.25991247483)*x[1]+(1.71801479005)*x[2]
        s[1,2,0,0]=(-1.23648267242)*x[0]+(-1.14397314877)*x[1]+(1.51420721169)*x[2]
        s[1,2,0,1]=(1.76604962545)*x[0]+(0.183730664401)*x[1]+(0.119413404231)*x[2]
        s[1,2,1,0]=(-0.371372970825)*x[0]+(1.67376148418)*x[1]+(-1.4673890303)*x[2]
        s[1,2,1,1]=(-1.05767796861)*x[0]+(1.57674524903)*x[1]+(0.907317505281)*x[2]
        s[1,2,2,0]=(-0.560171709945)*x[0]+(0.932731419096)*x[1]+(-1.34845422611)*x[2]
        s[1,2,2,1]=(0.299743316165)*x[0]+(1.27560261018)*x[1]+(-0.835084977966)*x[2]
        s[1,3,0,0]=(-0.255337437533)*x[0]+(-0.597132326112)*x[1]+(-0.136201264537)*x[2]
        s[1,3,0,1]=(0.593835766145)*x[0]+(-0.799771329645)*x[1]+(-0.980158768361)*x[2]
        s[1,3,1,0]=(1.38678662193)*x[0]+(0.145756993166)*x[1]+(-0.687830261701)*x[2]
        s[1,3,1,1]=(-0.535937612709)*x[0]+(-0.0191554551358)*x[1]+(0.360231645604)*x[2]
        s[1,3,2,0]=(0.464446491173)*x[0]+(0.601115829679)*x[1]+(-1.23243462932)*x[2]
        s[1,3,2,1]=(-1.00252524049)*x[0]+(0.38542247592)*x[1]+(-0.786817371152)*x[2]
        s[1,4,0,0]=(0.127520587286)*x[0]+(0.0193180871908)*x[1]+(0.443880703635)*x[2]
        s[1,4,0,1]=(0.459087583966)*x[0]+(-0.0133413082011)*x[1]+(-0.469574441754)*x[2]
        s[1,4,1,0]=(-0.0408745863162)*x[0]+(-0.450729520372)*x[1]+(0.49494190957)*x[2]
        s[1,4,1,1]=(0.698205417361)*x[0]+(0.475698414334)*x[1]+(-0.738661347545)*x[2]
        s[1,4,2,0]=(1.20797915788)*x[0]+(-0.132247786402)*x[1]+(-0.282227294584)*x[2]
        s[1,4,2,1]=(-0.76998141325)*x[0]+(-0.0677377690684)*x[1]+(-0.0836477753317)*x[2]
        s[2,0,0,0]=(-1.10579544966)*x[0]+(1.40458332668)*x[1]+(0.666829139108)*x[2]
        s[2,0,0,1]=(0.316772911983)*x[0]+(-0.722315825552)*x[1]+(-0.420491453418)*x[2]
        s[2,0,1,0]=(1.28978684389)*x[0]+(-1.31024104856)*x[1]+(-0.909892235182)*x[2]
        s[2,0,1,1]=(0.516882102505)*x[0]+(0.137837613805)*x[1]+(-0.0746054680439)*x[2]
        s[2,0,2,0]=(-0.684536226359)*x[0]+(1.26314901343)*x[1]+(-0.80088731126)*x[2]
        s[2,0,2,1]=(1.04894156753)*x[0]+(-0.292514668329)*x[1]+(0.189452265734)*x[2]
        s[2,1,0,0]=(-1.89697799819)*x[0]+(0.842415025171)*x[1]+(1.50175587499)*x[2]
        s[2,1,0,1]=(-0.132472949872)*x[0]+(0.755809155321)*x[1]+(1.4210576902)*x[2]
        s[2,1,1,0]=(-1.20812267424)*x[0]+(0.776283189748)*x[1]+(0.466844337822)*x[2]
        s[2,1,1,1]=(-0.545404145415)*x[0]+(0.630064653938)*x[1]+(0.416024592778)*x[2]
        s[2,1,2,0]=(-1.25450160322)*x[0]+(0.712099345608)*x[1]+(-1.40448708108)*x[2]
        s[2,1,2,1]=(0.8675638655)*x[0]+(-0.694288742412)*x[1]+(0.423917775454)*x[2]
        s[2,2,0,0]=(0.611369389916)*x[0]+(-0.282854928933)*x[1]+(0.421814371036)*x[2]
        s[2,2,0,1]=(0.87143261059)*x[0]+(0.626319549038)*x[1]+(-0.247486033705)*x[2]
        s[2,2,1,0]=(-1.09640818879)*x[0]+(-0.625440705278)*x[1]+(1.06488909366)*x[2]
        s[2,2,1,1]=(-1.63825859868)*x[0]+(0.243702450075)*x[1]+(0.37689118934)*x[2]
        s[2,2,2,0]=(1.14554562168)*x[0]+(-0.656545182621)*x[1]+(0.122881381407)*x[2]
        s[2,2,2,1]=(1.04057428387)*x[0]+(0.605147014643)*x[1]+(-0.0624571196862)*x[2]
        s[2,3,0,0]=(-0.687211572887)*x[0]+(-1.33387647369)*x[1]+(1.56461600602)*x[2]
        s[2,3,0,1]=(-0.439360840188)*x[0]+(-0.175050010481)*x[1]+(-0.493088639437)*x[2]
        s[2,3,1,0]=(1.01647468205)*x[0]+(1.607854298)*x[1]+(0.154041122248)*x[2]
        s[2,3,1,1]=(-1.17279615871)*x[0]+(0.265464982509)*x[1]+(0.536445292976)*x[2]
        s[2,3,2,0]=(-0.370774714547)*x[0]+(-0.140305952222)*x[1]+(-0.27755251782)*x[2]
        s[2,3,2,1]=(-0.601589093641)*x[0]+(-0.0914485078099)*x[1]+(-0.246092622851)*x[2]
        s[2,4,0,0]=(1.16222773615)*x[0]+(-1.0556446667)*x[1]+(1.18749640554)*x[2]
        s[2,4,0,1]=(0.441185917779)*x[0]+(0.106062084171)*x[1]+(-0.150699537615)*x[2]
        s[2,4,1,0]=(1.24765736189)*x[0]+(-1.28446862056)*x[1]+(-1.15581931924)*x[2]
        s[2,4,1,1]=(0.751269018864)*x[0]+(1.03373528238)*x[1]+(0.953542233964)*x[2]
        s[2,4,2,0]=(-0.507055271742)*x[0]+(-0.139767333963)*x[1]+(0.369122007484)*x[2]
        s[2,4,2,1]=(1.75708904398)*x[0]+(0.229784892513)*x[1]+(0.138439119614)*x[2]
        s[3,0,0,0]=(0.488579497744)*x[0]+(0.227848351934)*x[1]+(-0.503567606843)*x[2]
        s[3,0,0,1]=(1.48513008903)*x[0]+(0.0119808614235)*x[1]+(0.50138290689)*x[2]
        s[3,0,1,0]=(1.89811058679)*x[0]+(0.936442809385)*x[1]+(0.618927531336)*x[2]
        s[3,0,1,1]=(0.520596564131)*x[0]+(-1.34097734027)*x[1]+(-1.24288925352)*x[2]
        s[3,0,2,0]=(-1.22679833631)*x[0]+(0.946723115385)*x[1]+(-0.444825501575)*x[2]
        s[3,0,2,1]=(-0.786039767784)*x[0]+(1.01276854439)*x[1]+(0.868428470305)*x[2]
        s[3,1,0,0]=(-1.14928294841)*x[0]+(1.83212625566)*x[1]+(-0.131295307828)*x[2]
        s[3,1,0,1]=(0.530939875091)*x[0]+(-0.00936464244878)*x[1]+(-0.54397285021)*x[2]
        s[3,1,1,0]=(0.776394250264)*x[0]+(-0.0432924713397)*x[1]+(-0.48992936123)*x[2]
        s[3,1,1,1]=(-0.295285841654)*x[0]+(1.60838322278)*x[1]+(-0.822857254214)*x[2]
        s[3,1,2,0]=(-0.445325691811)*x[0]+(0.143992033055)*x[1]+(-0.562336605856)*x[2]
        s[3,1,2,1]=(-0.424042508176)*x[0]+(-0.119602696473)*x[1]+(0.50936759192)*x[2]
        s[3,2,0,0]=(0.0376712305683)*x[0]+(-0.127442186223)*x[1]+(-0.98477458245)*x[2]
        s[3,2,0,1]=(-0.964194007245)*x[0]+(-0.951386585236)*x[1]+(-0.376434083193)*x[2]
        s[3,2,1,0]=(-0.645708363662)*x[0]+(0.729523237957)*x[1]+(-0.815980951689)*x[2]
        s[3,2,1,1]=(1.54280019708)*x[0]+(0.347142929119)*x[1]+(0.117050155931)*x[2]
        s[3,2,2,0]=(-0.70653056148)*x[0]+(0.658564301815)*x[1]+(1.5766896863)*x[2]
        s[3,2,2,1]=(1.28238648298)*x[0]+(0.415173079215)*x[1]+(1.25492135064)*x[2]
        s[3,3,0,0]=(-0.907422225702)*x[0]+(0.354252510498)*x[1]+(-0.617733757576)*x[2]
        s[3,3,0,1]=(-0.687653032786)*x[0]+(0.166147005719)*x[1]+(0.0668202207665)*x[2]
        s[3,3,1,0]=(-0.515227984328)*x[0]+(-1.20687483216)*x[1]+(0.325703771711)*x[2]
        s[3,3,1,1]=(-0.239992988401)*x[0]+(-0.625429153195)*x[1]+(-0.987302012149)*x[2]
        s[3,3,2,0]=(0.48686562504)*x[0]+(-0.687077509478)*x[1]+(0.630530432518)*x[2]
        s[3,3,2,1]=(0.575785040914)*x[0]+(0.978122193974)*x[1]+(0.0753481784122)*x[2]
        s[3,4,0,0]=(-0.201304266512)*x[0]+(-0.563373770577)*x[1]+(-0.0445045378284)*x[2]
        s[3,4,0,1]=(0.348899505512)*x[0]+(-0.983874875078)*x[1]+(0.561390754019)*x[2]
        s[3,4,1,0]=(0.687251502909)*x[0]+(0.806428209931)*x[1]+(-0.848431263257)*x[2]
        s[3,4,1,1]=(-0.0690481011866)*x[0]+(-0.637464706806)*x[1]+(1.32335195878)*x[2]
        s[3,4,2,0]=(0.640286948745)*x[0]+(-1.44112416361)*x[1]+(-0.724716414365)*x[2]
        s[3,4,2,1]=(-0.967100763601)*x[0]+(0.0347001879437)*x[1]+(-0.888183629447)*x[2]
        ref[0,0,0,0]=(-1.47634692775)/(o+1.)+(0.413823374944)+(-0.69863978692)*0.5**o
        ref[0,0,0,1]=(0.467438056347)/(o+1.)+(-0.049348895708)+(0.980332235993)*0.5**o
        ref[0,0,1,0]=(0.0324608468542)/(o+1.)+(0.227219571463)+(-0.851091598789)*0.5**o
        ref[0,0,1,1]=(1.24663216229)/(o+1.)+(0.680736038169)+(0.892185739473)*0.5**o
        ref[0,0,2,0]=(1.04128811669)/(o+1.)+(0.801526892617)+(-0.721889856517)*0.5**o
        ref[0,0,2,1]=(-1.57019640933)/(o+1.)+(-0.0149100811556)+(0.191115336015)*0.5**o
        ref[0,1,0,0]=(-1.06234362705)/(o+1.)+(-0.881380501065)+(0.88447821062)*0.5**o
        ref[0,1,0,1]=(0.267287676352)/(o+1.)+(0.0934259729578)+(0.719058878904)*0.5**o
        ref[0,1,1,0]=(-0.83257335756)/(o+1.)+(0.056558158197)+(-0.171851295688)*0.5**o
        ref[0,1,1,1]=(1.59747135641)/(o+1.)+(1.00837611714)+(0.111151278122)*0.5**o
        ref[0,1,2,0]=(0.181694306894)/(o+1.)+(-0.0366161862313)+(0.633569620457)*0.5**o
        ref[0,1,2,1]=(-0.116206059658)/(o+1.)+(-1.20967709253)+(-0.255855799289)*0.5**o
        ref[0,2,0,0]=(0.615063090083)/(o+1.)+(-0.0944203632267)+(0.560752133921)*0.5**o
        ref[0,2,0,1]=(-1.68718452323)/(o+1.)+(0.415922156486)+(0.323157880499)*0.5**o
        ref[0,2,1,0]=(-1.38436480215)/(o+1.)+(0.514201265835)+(-0.785401872214)*0.5**o
        ref[0,2,1,1]=(1.07260243754)/(o+1.)+(1.03208581185)+(-0.738468272962)*0.5**o
        ref[0,2,2,0]=(-0.619469154652)/(o+1.)+(0.156513881249)+(0.19421868483)*0.5**o
        ref[0,2,2,1]=(0.742851517381)/(o+1.)+(-0.577857666542)+(0.583763691375)*0.5**o
        ref[0,3,0,0]=(-0.0209287146839)/(o+1.)+(-0.0417684031135)+(0.248273994472)*0.5**o
        ref[0,3,0,1]=(-0.876757092127)/(o+1.)+(0.156654815475)+(-0.431666242085)*0.5**o
        ref[0,3,1,0]=(-1.40731912274)/(o+1.)+(0.295731262157)+(0.579728397479)*0.5**o
        ref[0,3,1,1]=(0.766713725705)/(o+1.)+(0.520952712789)+(-0.331873109391)*0.5**o
        ref[0,3,2,0]=(-1.37493161295)/(o+1.)+(0.626269025128)+(-0.656049244875)*0.5**o
        ref[0,3,2,1]=(0.738828256274)/(o+1.)+(0.299922102653)+(0.245566994744)*0.5**o
        ref[0,4,0,0]=(-0.220320428485)/(o+1.)+(-0.381104395776)+(0.30219274463)*0.5**o
        ref[0,4,0,1]=(-1.71138026305)/(o+1.)+(0.0858133974006)+(-0.272542497315)*0.5**o
        ref[0,4,1,0]=(-0.0512711854771)/(o+1.)+(-0.398429115874)+(-0.908094901926)*0.5**o
        ref[0,4,1,1]=(0.775088175251)/(o+1.)+(0.153496176444)+(-0.76146195497)*0.5**o
        ref[0,4,2,0]=(-0.188892896681)/(o+1.)+(-0.212757962174)+(0.600204512934)*0.5**o
        ref[0,4,2,1]=(0.786135983408)/(o+1.)+(0.709443217024)+(-0.0813557008918)*0.5**o
        ref[1,0,0,0]=(-0.682877215956)/(o+1.)+(0.0430800979484)+(-0.786904593287)*0.5**o
        ref[1,0,0,1]=(-0.209743447547)/(o+1.)+(-0.529335530226)+(-0.351982026793)*0.5**o
        ref[1,0,1,0]=(1.7467904951)/(o+1.)+(-0.522734985482)+(0.205256143552)*0.5**o
        ref[1,0,1,1]=(-0.681949654071)/(o+1.)+(0.430192257885)+(-0.989555659669)*0.5**o
        ref[1,0,2,0]=(0.00559076557551)/(o+1.)+(-0.355981322114)+(-0.845595380938)*0.5**o
        ref[1,0,2,1]=(-1.0795557316)/(o+1.)+(0.433501713489)+(-0.971431930418)*0.5**o
        ref[1,1,0,0]=(-1.41228679449)/(o+1.)+(0.0214610301511)+(-0.47963962908)*0.5**o
        ref[1,1,0,1]=(0.675271467105)/(o+1.)+(0.258295057037)+(-0.872015819915)*0.5**o
        ref[1,1,1,0]=(-0.0723449260888)/(o+1.)+(0.642648455626)+(-0.504589981145)*0.5**o
        ref[1,1,1,1]=(1.69082429043)/(o+1.)+(0.351194447364)+(0.125923252805)*0.5**o
        ref[1,1,2,0]=(1.04100385874)/(o+1.)+(-0.253396731491)+(-0.320884200985)*0.5**o
        ref[1,1,2,1]=(1.44000111068)/(o+1.)+(0.167426476075)+(-0.284354022497)*0.5**o
        ref[1,2,0,0]=(0.762931449029)/(o+1.)+(-0.397841053459)+(-0.833497951611)*0.5**o
        ref[1,2,0,1]=(-0.353193148804)/(o+1.)+(0.729759415974)+(0.962868010939)*0.5**o
        ref[1,2,1,0]=(-0.0142626154573)/(o+1.)+(0.205452893539)+(-0.561643688565)*0.5**o
        ref[1,2,1,1]=(1.31669689043)/(o+1.)+(0.322616901969)+(-0.535545908666)*0.5**o
        ref[1,2,2,0]=(-0.720144981522)/(o+1.)+(-0.253513298251)+(0.251277061063)*0.5**o
        ref[1,2,2,1]=(0.171627338105)/(o+1.)+(0.304508788985)+(-0.0403839677009)*0.5**o
        ref[1,3,0,0]=(-0.385245021569)/(o+1.)+(0.0533837214678)+(-0.710193449549)*0.5**o
        ref[1,3,0,1]=(-1.66770468862)/(o+1.)+(0.0386999300625)+(0.404210496633)*0.5**o
        ref[1,3,1,0]=(0.1536232451)/(o+1.)+(-0.114369222402)+(0.919828553095)*0.5**o
        ref[1,3,1,1]=(0.279969861456)/(o+1.)+(0.199789198624)+(-0.874409680945)*0.5**o
        ref[1,3,2,0]=(-0.29883793799)/(o+1.)+(-0.179133605171)+(0.490232839865)*0.5**o
        ref[1,3,2,1]=(-0.20045707503)/(o+1.)+(-0.419654937503)+(-0.364153185689)*0.5**o
        ref[1,4,0,0]=(0.328269609451)/(o+1.)+(0.0976736768568)+(0.0671024149466)*0.5**o
        ref[1,4,0,1]=(0.70998344827)/(o+1.)+(-0.831520044753)+(0.929228475247)*0.5**o
        ref[1,4,1,0]=(1.14467737453)/(o+1.)+(-0.309242254267)+(-0.522855063115)*0.5**o
        ref[1,4,1,1]=(0.386397027677)/(o+1.)+(-0.152706345419)+(0.354258147311)*0.5**o
        ref[1,4,2,0]=(0.565989417809)/(o+1.)+(-0.0912836510331)+(0.410081961152)*0.5**o
        ref[1,4,2,1]=(-0.108228050157)/(o+1.)+(-0.382022658574)+(-0.0490935903447)*0.5**o
        ref[2,0,0,0]=(1.30816654656)/(o+1.)+(0.218850490887)+(-0.780250512205)*0.5**o
        ref[2,0,0,1]=(-0.072210934601)/(o+1.)+(-0.376572060485)+(-0.000679311415873)*0.5**o
        ref[2,0,1,0]=(-1.03731005117)/(o+1.)+(-0.369008331776)+(0.844980274869)*0.5**o
        ref[2,0,1,1]=(-0.37673073433)/(o+1.)+(0.147779920342)+(0.661285141912)*0.5**o
        ref[2,0,2,0]=(-0.126456722513)/(o+1.)+(-0.173173731574)+(0.250529661474)*0.5**o
        ref[2,0,2,1]=(0.269921444109)/(o+1.)+(0.256112039487)+(0.163733641849)*0.5**o
        ref[2,1,0,0]=(1.02033624645)/(o+1.)+(0.162073420045)+(-0.897290184569)*0.5**o
        ref[2,1,0,1]=(0.673075377238)/(o+1.)+(0.377904898438)+(0.615508721534)*0.5**o
        ref[2,1,1,0]=(-0.536750510163)/(o+1.)+(0.687164638156)+(-0.802573912815)*0.5**o
        ref[2,1,1,1]=(1.39522327504)/(o+1.)+(-0.168020484927)+(-0.558497203882)*0.5**o
        ref[2,1,2,0]=(-1.21446181317)/(o+1.)+(-0.219360528348)+(-0.293706468828)*0.5**o
        ref[2,1,2,1]=(0.352877691113)/(o+1.)+(0.0397286675094)+(0.16485787241)*0.5**o
        ref[2,2,0,0]=(-0.927761522597)/(o+1.)+(1.03238921925)+(-0.386688083878)*0.5**o
        ref[2,2,0,1]=(0.875914163347)/(o+1.)+(-0.0805989643579)+(0.535549891292)*0.5**o
        ref[2,2,1,0]=(-0.268027052879)/(o+1.)+(0.24501270411)+(-0.878958155751)*0.5**o
        ref[2,2,1,1]=(0.579267469532)/(o+1.)+(-0.419360478349)+(-0.758211472099)*0.5**o
        ref[2,2,2,0]=(-0.407532480549)/(o+1.)+(0.0236731772421)+(0.972067946527)*0.5**o
        ref[2,2,2,1]=(-0.848446514529)/(o+1.)+(0.986605187547)+(0.458500318263)*0.5**o
        ref[2,3,0,0]=(0.0139531165204)/(o+1.)+(0.0612674782454)+(-0.592960113562)*0.5**o
        ref[2,3,0,1]=(-0.434410189326)/(o+1.)+(0.0699472934754)+(-0.81298388773)*0.5**o
        ref[2,3,1,0]=(1.51687712753)/(o+1.)+(0.29518962448)+(0.671113725811)*0.5**o
        ref[2,3,1,1]=(-0.825018275156)/(o+1.)+(0.348395434187)+(-0.242658476441)*0.5**o
        ref[2,3,2,0]=(-0.129038376537)/(o+1.)+(0.0407049218889)+(-0.74100465183)*0.5**o
        ref[2,3,2,1]=(-0.61436863216)/(o+1.)+(0.0569955443553)+(-0.438752680853)*0.5**o
        ref[2,4,0,0]=(-0.186811568504)/(o+1.)+(0.324468688913)+(0.831953665661)*0.5**o
        ref[2,4,0,1]=(1.01471089788)/(o+1.)+(-0.404804936557)+(0.191447439569)*0.5**o
        ref[2,4,1,0]=(-0.833336708992)/(o+1.)+(-0.465420260939)+(0.571546652968)*0.5**o
        ref[2,4,1,1]=(1.48092456289)/(o+1.)+(0.548316338846)+(0.16098929463)*0.5**o
        ref[2,4,2,0]=(-1.20254311473)/(o+1.)+(0.778023417549)+(-0.631204318592)*0.5**o
        ref[2,4,2,1]=(0.672521404448)/(o+1.)+(0.265167920765)+(0.922455810129)*0.5**o
        ref[3,0,0,0]=(-0.400146773167)/(o+1.)+(0.151226335458)+(0.310554345086)*0.5**o
        ref[3,0,0,1]=(0.580039994556)/(o+1.)+(0.287523081772)+(0.843407699243)*0.5**o
        ref[3,0,1,0]=(0.930972332962)/(o+1.)+(0.768302886889)+(0.985902820768)*0.5**o
        ref[3,0,1,1]=(-0.991140747279)/(o+1.)+(-0.914680203238)+(0.757231124095)*0.5**o
        ref[3,0,2,0]=(-0.458287257967)/(o+1.)+(0.237995705359)+(-0.742604875255)*0.5**o
        ref[3,0,2,1]=(1.14385326779)/(o+1.)+(0.243645854371)+(-0.535987729627)*0.5**o
        ref[3,1,0,0]=(0.322354058507)/(o+1.)+(0.341162149011)+(-0.453130357106)*0.5**o
        ref[3,1,0,1]=(-1.78201691527)/(o+1.)+(1.04199074308)+(-0.324362188462)*0.5**o
        ref[3,1,1,0]=(-0.0578982797037)/(o+1.)+(-0.110933167998)+(0.522937033394)*0.5**o
        ref[3,1,1,1]=(-0.276009095212)/(o+1.)+(0.517113652949)+(-0.267978083771)*0.5**o
        ref[3,1,2,0]=(-0.023345636799)/(o+1.)+(-0.66878791133)+(0.497251194848)*0.5**o
        ref[3,1,2,1]=(1.2604140706)/(o+1.)+(-0.878310666552)+(0.461929649774)*0.5**o
        ref[3,2,0,0]=(-0.00211014675007)/(o+1.)+(-0.814091029436)+(0.555746667517)*0.5**o
        ref[3,2,0,1]=(-1.07650025357)/(o+1.)+(-0.26199073554)+(-0.691532951025)*0.5**o
        ref[3,2,1,0]=(-0.0770650645465)/(o+1.)+(-0.307619169254)+(-0.0398626743392)*0.5**o
        ref[3,2,1,1]=(1.12398638253)/(o+1.)+(-0.00626411324864)+(0.895535126103)*0.5**o
        ref[3,2,2,0]=(0.822758496096)/(o+1.)+(0.448681597953)+(-0.191398265373)*0.5**o
        ref[3,2,2,1]=(1.38402514586)/(o+1.)+(0.489672391609)+(0.589110983761)*0.5**o
        ref[3,3,0,0]=(0.626837013941)/(o+1.)+(-0.698370737203)+(-0.400999012316)*0.5**o
        ref[3,3,0,1]=(-0.819551723109)/(o+1.)+(0.588359566557)+(-0.811853216306)*0.5**o
        ref[3,3,1,0]=(-0.472944922304)/(o+1.)+(0.024839136659)+(-0.973132395795)*0.5**o
        ref[3,3,1,1]=(-0.674351724226)/(o+1.)+(-0.596697469314)+(0.0150225091087)*0.5**o
        ref[3,3,2,0]=(-0.653245573198)/(o+1.)+(0.590003731898)+(-0.0964433425189)*0.5**o
        ref[3,3,2,1]=(1.15741134315)/(o+1.)+(-0.128540702232)+(0.728925474609)*0.5**o
        ref[3,4,0,0]=(-0.745079746498)/(o+1.)+(-0.313462120963)+(0.562821413507)*0.5**o
        ref[3,4,0,1]=(-0.585232841208)/(o+1.)+(0.464633596697)+(-0.417618967733)*0.5**o
        ref[3,4,1,0]=(0.792309614109)/(o+1.)+(-0.478065247211)+(0.809069329896)*0.5**o
        ref[3,4,1,1]=(0.0536562315177)/(o+1.)+(-0.128980351611)+(0.821143622493)*0.5**o
        ref[3,4,2,0]=(-1.20913408459)/(o+1.)+(-0.144016051074)+(-0.0283874424941)*0.5**o
        ref[3,4,2,1]=(-1.12139150964)/(o+1.)+(-0.135856270303)+(-0.427480154859)*0.5**o
      res=integrate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactOne_fromSymbol_ReducedFunctionOnContactOne_rank0(self):
      """
      tests integral of rank 0 Symbol on the ReducedFunctionOnContactOne

      assumptions: ReducedFunctionOnContactOne(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      w=ReducedFunctionOnContactOne(self.domain)
      x=w.getX()
      arg=Symbol(shape=())
      s=Data(0,(),w)
      if dim==2:
        s=(0.0457734760303)*x[0]**o+(0.469200083987)*x[0]+(0.871163132136)*x[1]**o+(-0.848809543077)*x[1]
        ref=(0.871163132136)/(o+1.)+(-0.189804729545)+(0.0457734760303)*0.5**o
      else:
        s=(-0.217546543734)*x[0]**o+(-0.296791310389)*x[0]+(0.295478793948)*x[1]**o+(0.355506997608)*x[1]+(-0.173745407638)*x[2]**o+(0.733848286432)*x[2]
        ref=(0.12173338631)/(o+1.)+(0.396281986825)+(-0.217546543734)*0.5**o
      res=integrate(arg)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactOne_fromSymbol_ReducedFunctionOnContactOne_rank1(self):
      """
      tests integral of rank 1 Symbol on the ReducedFunctionOnContactOne

      assumptions: ReducedFunctionOnContactOne(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      w=ReducedFunctionOnContactOne(self.domain)
      x=w.getX()
      arg=Symbol(shape=(2,))
      s=Data(0,(2,),w)
      ref=numpy.zeros((2,),numpy.float_)
      if dim==2:
        s[0]=(0.375635683415)*x[0]**o+(-0.367500579656)*x[0]+(0.0137704362217)*x[1]**o+(-0.861006616375)*x[1]
        s[1]=(-0.787660325374)*x[0]**o+(0.114019988756)*x[0]+(0.120533570643)*x[1]**o+(0.619732700586)*x[1]
        ref[0]=(0.0137704362217)/(o+1.)+(-0.614253598016)+(0.375635683415)*0.5**o
        ref[1]=(0.120533570643)/(o+1.)+(0.366876344671)+(-0.787660325374)*0.5**o
      else:
        s[0]=(-0.496247804966)*x[0]**o+(-0.717417759132)*x[0]+(-0.89565067445)*x[1]**o+(0.382056160402)*x[1]+(0.994707732251)*x[2]**o+(0.988370997667)*x[2]
        s[1]=(0.274829833844)*x[0]**o+(0.699113046456)*x[0]+(-0.780626603094)*x[1]**o+(0.00719416836835)*x[1]+(0.0771370370478)*x[2]**o+(-0.883519240597)*x[2]
        ref[0]=(0.0990570578009)/(o+1.)+(0.326504699469)+(-0.496247804966)*0.5**o
        ref[1]=(-0.703489566047)/(o+1.)+(-0.0886060128866)+(0.274829833844)*0.5**o
      res=integrate(arg)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactOne_fromSymbol_ReducedFunctionOnContactOne_rank2(self):
      """
      tests integral of rank 2 Symbol on the ReducedFunctionOnContactOne

      assumptions: ReducedFunctionOnContactOne(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      w=ReducedFunctionOnContactOne(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5))
      s=Data(0,(4, 5),w)
      ref=numpy.zeros((4, 5),numpy.float_)
      if dim==2:
        s[0,0]=(0.997138766521)*x[0]**o+(-0.553379306564)*x[0]+(-0.234324119126)*x[1]**o+(-0.355669375308)*x[1]
        s[0,1]=(-0.0722078180457)*x[0]**o+(0.29155764813)*x[0]+(0.30568465631)*x[1]**o+(-0.977462121264)*x[1]
        s[0,2]=(0.0627729453295)*x[0]**o+(0.522234421268)*x[0]+(-0.265521199465)*x[1]**o+(0.0773272883492)*x[1]
        s[0,3]=(0.120580856562)*x[0]**o+(-0.576605249922)*x[0]+(-0.515839258724)*x[1]**o+(0.364676391176)*x[1]
        s[0,4]=(-0.369132454849)*x[0]**o+(-0.304739979654)*x[0]+(0.731091855933)*x[1]**o+(0.312380588453)*x[1]
        s[1,0]=(-0.277744449846)*x[0]**o+(0.963258283702)*x[0]+(0.0646683091177)*x[1]**o+(0.508336130337)*x[1]
        s[1,1]=(0.879838558141)*x[0]**o+(-0.575726599149)*x[0]+(-0.770954127597)*x[1]**o+(-0.29742399168)*x[1]
        s[1,2]=(0.105461068455)*x[0]**o+(0.325901606009)*x[0]+(-0.787936986763)*x[1]**o+(0.275010392849)*x[1]
        s[1,3]=(0.634687837213)*x[0]**o+(0.449318030603)*x[0]+(0.113843972437)*x[1]**o+(0.135522890516)*x[1]
        s[1,4]=(-0.447823860246)*x[0]**o+(-0.980688332644)*x[0]+(0.791824549606)*x[1]**o+(-0.171760132019)*x[1]
        s[2,0]=(0.61803013069)*x[0]**o+(-0.539345725331)*x[0]+(0.552856742671)*x[1]**o+(0.760667455882)*x[1]
        s[2,1]=(0.707626098607)*x[0]**o+(0.682940904637)*x[0]+(-0.057070815024)*x[1]**o+(-0.889583848012)*x[1]
        s[2,2]=(0.0226275928114)*x[0]**o+(-0.58368440817)*x[0]+(0.503278007027)*x[1]**o+(0.607225060945)*x[1]
        s[2,3]=(-0.643463757169)*x[0]**o+(0.166312879954)*x[0]+(0.889781254515)*x[1]**o+(-0.820343616323)*x[1]
        s[2,4]=(-0.920378124042)*x[0]**o+(-0.879688658227)*x[0]+(0.578052078637)*x[1]**o+(0.895382276619)*x[1]
        s[3,0]=(-0.748327596968)*x[0]**o+(-0.613914500566)*x[0]+(-0.0478077540856)*x[1]**o+(-0.838510710621)*x[1]
        s[3,1]=(-0.831715563374)*x[0]**o+(0.177460645482)*x[0]+(-0.19690401481)*x[1]**o+(-0.180828782822)*x[1]
        s[3,2]=(0.0692984740547)*x[0]**o+(-0.798996415727)*x[0]+(-0.193350054474)*x[1]**o+(-0.160594229856)*x[1]
        s[3,3]=(0.989346428804)*x[0]**o+(0.285816940366)*x[0]+(-0.296547093918)*x[1]**o+(-0.0529210232469)*x[1]
        s[3,4]=(0.24851689771)*x[0]**o+(0.452794657799)*x[0]+(-0.555079559)*x[1]**o+(-0.336250468184)*x[1]
        ref[0,0]=(-0.234324119126)/(o+1.)+(-0.454524340936)+(0.997138766521)*0.5**o
        ref[0,1]=(0.30568465631)/(o+1.)+(-0.342952236567)+(-0.0722078180457)*0.5**o
        ref[0,2]=(-0.265521199465)/(o+1.)+(0.299780854808)+(0.0627729453295)*0.5**o
        ref[0,3]=(-0.515839258724)/(o+1.)+(-0.105964429373)+(0.120580856562)*0.5**o
        ref[0,4]=(0.731091855933)/(o+1.)+(0.00382030439927)+(-0.369132454849)*0.5**o
        ref[1,0]=(0.0646683091177)/(o+1.)+(0.73579720702)+(-0.277744449846)*0.5**o
        ref[1,1]=(-0.770954127597)/(o+1.)+(-0.436575295414)+(0.879838558141)*0.5**o
        ref[1,2]=(-0.787936986763)/(o+1.)+(0.300455999429)+(0.105461068455)*0.5**o
        ref[1,3]=(0.113843972437)/(o+1.)+(0.292420460559)+(0.634687837213)*0.5**o
        ref[1,4]=(0.791824549606)/(o+1.)+(-0.576224232331)+(-0.447823860246)*0.5**o
        ref[2,0]=(0.552856742671)/(o+1.)+(0.110660865275)+(0.61803013069)*0.5**o
        ref[2,1]=(-0.057070815024)/(o+1.)+(-0.103321471687)+(0.707626098607)*0.5**o
        ref[2,2]=(0.503278007027)/(o+1.)+(0.0117703263876)+(0.0226275928114)*0.5**o
        ref[2,3]=(0.889781254515)/(o+1.)+(-0.327015368185)+(-0.643463757169)*0.5**o
        ref[2,4]=(0.578052078637)/(o+1.)+(0.00784680919582)+(-0.920378124042)*0.5**o
        ref[3,0]=(-0.0478077540856)/(o+1.)+(-0.726212605594)+(-0.748327596968)*0.5**o
        ref[3,1]=(-0.19690401481)/(o+1.)+(-0.00168406867019)+(-0.831715563374)*0.5**o
        ref[3,2]=(-0.193350054474)/(o+1.)+(-0.479795322792)+(0.0692984740547)*0.5**o
        ref[3,3]=(-0.296547093918)/(o+1.)+(0.11644795856)+(0.989346428804)*0.5**o
        ref[3,4]=(-0.555079559)/(o+1.)+(0.0582720948078)+(0.24851689771)*0.5**o
      else:
        s[0,0]=(-0.885439150233)*x[0]**o+(-0.0707651237907)*x[0]+(-0.0221938474237)*x[1]**o+(-0.657402959688)*x[1]+(0.492040451029)*x[2]**o+(-0.500851664471)*x[2]
        s[0,1]=(-0.915950017556)*x[0]**o+(-0.223230342138)*x[0]+(0.251513308151)*x[1]**o+(0.340647147217)*x[1]+(0.785842086262)*x[2]**o+(-0.375289319286)*x[2]
        s[0,2]=(0.0171536999294)*x[0]**o+(0.187222850003)*x[0]+(0.923344343515)*x[1]**o+(0.919899906728)*x[1]+(0.330573465731)*x[2]**o+(0.12075255513)*x[2]
        s[0,3]=(-0.490667671112)*x[0]**o+(0.4056039259)*x[0]+(-0.955154570277)*x[1]**o+(0.986366754754)*x[1]+(-0.853519593222)*x[2]**o+(-0.825901616105)*x[2]
        s[0,4]=(-0.253213854519)*x[0]**o+(-0.401738411717)*x[0]+(0.872412916488)*x[1]**o+(-0.00224326832344)*x[1]+(0.480217592874)*x[2]**o+(0.144994806066)*x[2]
        s[1,0]=(0.131992177305)*x[0]**o+(0.788963820881)*x[0]+(-0.0198901140504)*x[1]**o+(0.731616952503)*x[1]+(0.59309950089)*x[2]**o+(0.694630924739)*x[2]
        s[1,1]=(0.88075735692)*x[0]**o+(0.678228523078)*x[0]+(-0.682632234228)*x[1]**o+(-0.888294066081)*x[1]+(-0.302130074515)*x[2]**o+(-0.539380276098)*x[2]
        s[1,2]=(0.371868372987)*x[0]**o+(0.287766535066)*x[0]+(0.0885117264942)*x[1]**o+(0.249213255154)*x[1]+(-0.0473853043874)*x[2]**o+(0.345934167493)*x[2]
        s[1,3]=(-0.580406999101)*x[0]**o+(0.762553897732)*x[0]+(0.498019309339)*x[1]**o+(-0.275962658061)*x[1]+(-0.232793721572)*x[2]**o+(-0.404455792741)*x[2]
        s[1,4]=(-0.193670730589)*x[0]**o+(0.107221166466)*x[0]+(-0.253421260284)*x[1]**o+(-0.630230774271)*x[1]+(0.569627074227)*x[2]**o+(-0.471574680318)*x[2]
        s[2,0]=(-0.734235834531)*x[0]**o+(-0.0769718054424)*x[0]+(0.337185508531)*x[1]**o+(-0.695399200032)*x[1]+(0.670045013654)*x[2]**o+(0.602223461093)*x[2]
        s[2,1]=(0.40853552724)*x[0]**o+(-0.109296862931)*x[0]+(-0.898346018309)*x[1]**o+(0.797317601123)*x[1]+(-0.243636906258)*x[2]**o+(-0.7404427898)*x[2]
        s[2,2]=(-0.300099773564)*x[0]**o+(0.391661796552)*x[0]+(0.081340069378)*x[1]**o+(-0.540467785776)*x[1]+(0.609927398687)*x[2]**o+(0.342702876243)*x[2]
        s[2,3]=(-0.795553246099)*x[0]**o+(0.12091791031)*x[0]+(-0.358760047024)*x[1]**o+(0.00992547080721)*x[1]+(-0.776142821278)*x[2]**o+(0.960839850387)*x[2]
        s[2,4]=(0.0312887697988)*x[0]**o+(0.192922412706)*x[0]+(0.930373533919)*x[1]**o+(0.565991224895)*x[1]+(-0.620790693282)*x[2]**o+(-0.0894617381169)*x[2]
        s[3,0]=(0.58118631126)*x[0]**o+(0.0951313787407)*x[0]+(-0.481475099458)*x[1]**o+(-0.844977136281)*x[1]+(-0.505637296865)*x[2]**o+(-0.117240523527)*x[2]
        s[3,1]=(0.0560022697918)*x[0]**o+(-0.667387267392)*x[0]+(0.18270168123)*x[1]**o+(0.820389934276)*x[1]+(0.541891208803)*x[2]**o+(-0.813083374651)*x[2]
        s[3,2]=(-0.217967373894)*x[0]**o+(0.0740362777592)*x[0]+(-0.532958237386)*x[1]**o+(0.254923920582)*x[1]+(0.0983414559274)*x[2]**o+(-0.330473156862)*x[2]
        s[3,3]=(-0.51343561245)*x[0]**o+(-0.314818156795)*x[0]+(0.349539891727)*x[1]**o+(0.182246887466)*x[1]+(0.378368043277)*x[2]**o+(0.394897385803)*x[2]
        s[3,4]=(-0.440074964176)*x[0]**o+(0.644345562557)*x[0]+(0.157396506088)*x[1]**o+(-0.446161901195)*x[1]+(0.0406752725721)*x[2]**o+(0.00497105229147)*x[2]
        ref[0,0]=(0.469846603605)/(o+1.)+(-0.614509873975)+(-0.885439150233)*0.5**o
        ref[0,1]=(1.03735539441)/(o+1.)+(-0.128936257103)+(-0.915950017556)*0.5**o
        ref[0,2]=(1.25391780925)/(o+1.)+(0.613937655931)+(0.0171536999294)*0.5**o
        ref[0,3]=(-1.8086741635)/(o+1.)+(0.283034532275)+(-0.490667671112)*0.5**o
        ref[0,4]=(1.35263050936)/(o+1.)+(-0.129493436987)+(-0.253213854519)*0.5**o
        ref[1,0]=(0.57320938684)/(o+1.)+(1.10760584906)+(0.131992177305)*0.5**o
        ref[1,1]=(-0.984762308743)/(o+1.)+(-0.37472290955)+(0.88075735692)*0.5**o
        ref[1,2]=(0.0411264221069)/(o+1.)+(0.441456978856)+(0.371868372987)*0.5**o
        ref[1,3]=(0.265225587768)/(o+1.)+(0.0410677234649)+(-0.580406999101)*0.5**o
        ref[1,4]=(0.316205813942)/(o+1.)+(-0.497292144061)+(-0.193670730589)*0.5**o
        ref[2,0]=(1.00723052218)/(o+1.)+(-0.0850737721911)+(-0.734235834531)*0.5**o
        ref[2,1]=(-1.14198292457)/(o+1.)+(-0.0262110258043)+(0.40853552724)*0.5**o
        ref[2,2]=(0.691267468065)/(o+1.)+(0.0969484435098)+(-0.300099773564)*0.5**o
        ref[2,3]=(-1.1349028683)/(o+1.)+(0.545841615752)+(-0.795553246099)*0.5**o
        ref[2,4]=(0.309582840637)/(o+1.)+(0.334725949742)+(0.0312887697988)*0.5**o
        ref[3,0]=(-0.987112396323)/(o+1.)+(-0.433543140534)+(0.58118631126)*0.5**o
        ref[3,1]=(0.724592890033)/(o+1.)+(-0.330040353884)+(0.0560022697918)*0.5**o
        ref[3,2]=(-0.434616781458)/(o+1.)+(-0.000756479260437)+(-0.217967373894)*0.5**o
        ref[3,3]=(0.727907935004)/(o+1.)+(0.131163058237)+(-0.51343561245)*0.5**o
        ref[3,4]=(0.19807177866)/(o+1.)+(0.101577356827)+(-0.440074964176)*0.5**o
      res=integrate(arg)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactOne_fromSymbol_ReducedFunctionOnContactOne_rank3(self):
      """
      tests integral of rank 3 Symbol on the ReducedFunctionOnContactOne

      assumptions: ReducedFunctionOnContactOne(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      w=ReducedFunctionOnContactOne(self.domain)
      x=w.getX()
      arg=Symbol(shape=(6, 2, 2))
      s=Data(0,(6, 2, 2),w)
      ref=numpy.zeros((6, 2, 2),numpy.float_)
      if dim==2:
        s[0,0,0]=(-0.0899144489202)*x[0]**o+(-0.939343727238)*x[0]+(-0.838709152888)*x[1]**o+(-0.695938051041)*x[1]
        s[0,0,1]=(0.0915399473414)*x[0]**o+(-0.088933669169)*x[0]+(0.569689228579)*x[1]**o+(0.52563406191)*x[1]
        s[0,1,0]=(-0.0755502530652)*x[0]**o+(0.224201529435)*x[0]+(-0.0936878789524)*x[1]**o+(0.678476395607)*x[1]
        s[0,1,1]=(-0.722700224525)*x[0]**o+(-0.428992070285)*x[0]+(-0.967545686857)*x[1]**o+(0.339344032355)*x[1]
        s[1,0,0]=(-0.807565232378)*x[0]**o+(-0.389243328274)*x[0]+(-0.188155419411)*x[1]**o+(0.801573366554)*x[1]
        s[1,0,1]=(-0.217662099148)*x[0]**o+(0.0709316329807)*x[0]+(0.240070973919)*x[1]**o+(0.0450329893887)*x[1]
        s[1,1,0]=(-0.211774876096)*x[0]**o+(-0.585658640262)*x[0]+(0.772212206058)*x[1]**o+(0.471350730303)*x[1]
        s[1,1,1]=(-0.251097154538)*x[0]**o+(-0.936636380186)*x[0]+(-0.29953620509)*x[1]**o+(-0.7023587162)*x[1]
        s[2,0,0]=(-0.226008179803)*x[0]**o+(0.11175986541)*x[0]+(-0.815791532209)*x[1]**o+(0.661034404078)*x[1]
        s[2,0,1]=(-0.347375034321)*x[0]**o+(-0.38889482021)*x[0]+(-0.298497991695)*x[1]**o+(0.905510136504)*x[1]
        s[2,1,0]=(-0.414130770151)*x[0]**o+(0.00845860563915)*x[0]+(-0.622228227201)*x[1]**o+(0.239872393913)*x[1]
        s[2,1,1]=(0.985892855782)*x[0]**o+(-0.601178096316)*x[0]+(-0.671498979354)*x[1]**o+(0.600179658495)*x[1]
        s[3,0,0]=(-0.547441466002)*x[0]**o+(-0.390199968809)*x[0]+(0.356244378791)*x[1]**o+(-0.91693977081)*x[1]
        s[3,0,1]=(-0.317803998398)*x[0]**o+(-0.212192581141)*x[0]+(0.942307305401)*x[1]**o+(-0.528247885421)*x[1]
        s[3,1,0]=(-0.269300473712)*x[0]**o+(-0.806351998808)*x[0]+(-0.437943014771)*x[1]**o+(0.308252458482)*x[1]
        s[3,1,1]=(0.0826330889069)*x[0]**o+(0.357329175375)*x[0]+(-0.866644131299)*x[1]**o+(-0.993298142744)*x[1]
        s[4,0,0]=(0.478312250921)*x[0]**o+(0.949219183322)*x[0]+(-0.408977166217)*x[1]**o+(0.525189299992)*x[1]
        s[4,0,1]=(-0.221152804907)*x[0]**o+(0.683685107755)*x[0]+(-0.878859546858)*x[1]**o+(0.313936907822)*x[1]
        s[4,1,0]=(-0.361901807816)*x[0]**o+(0.473080571341)*x[0]+(0.0726134206382)*x[1]**o+(-0.552823213053)*x[1]
        s[4,1,1]=(-0.729188028594)*x[0]**o+(0.742375152366)*x[0]+(-0.11559899139)*x[1]**o+(-0.408146075502)*x[1]
        s[5,0,0]=(0.492678269676)*x[0]**o+(0.683105631523)*x[0]+(-0.0108127700464)*x[1]**o+(-0.0516040293712)*x[1]
        s[5,0,1]=(-0.0972486870808)*x[0]**o+(-0.354724382566)*x[0]+(0.383281344894)*x[1]**o+(0.547336907663)*x[1]
        s[5,1,0]=(0.442321568121)*x[0]**o+(-0.776464559839)*x[0]+(0.00851654852488)*x[1]**o+(-0.0830181237664)*x[1]
        s[5,1,1]=(-0.723222258936)*x[0]**o+(-0.294417512199)*x[0]+(-0.141223730539)*x[1]**o+(-0.757548646984)*x[1]
        ref[0,0,0]=(-0.838709152888)/(o+1.)+(-0.81764088914)+(-0.0899144489202)*0.5**o
        ref[0,0,1]=(0.569689228579)/(o+1.)+(0.21835019637)+(0.0915399473414)*0.5**o
        ref[0,1,0]=(-0.0936878789524)/(o+1.)+(0.451338962521)+(-0.0755502530652)*0.5**o
        ref[0,1,1]=(-0.967545686857)/(o+1.)+(-0.0448240189648)+(-0.722700224525)*0.5**o
        ref[1,0,0]=(-0.188155419411)/(o+1.)+(0.20616501914)+(-0.807565232378)*0.5**o
        ref[1,0,1]=(0.240070973919)/(o+1.)+(0.0579823111847)+(-0.217662099148)*0.5**o
        ref[1,1,0]=(0.772212206058)/(o+1.)+(-0.0571539549792)+(-0.211774876096)*0.5**o
        ref[1,1,1]=(-0.29953620509)/(o+1.)+(-0.819497548193)+(-0.251097154538)*0.5**o
        ref[2,0,0]=(-0.815791532209)/(o+1.)+(0.386397134744)+(-0.226008179803)*0.5**o
        ref[2,0,1]=(-0.298497991695)/(o+1.)+(0.258307658147)+(-0.347375034321)*0.5**o
        ref[2,1,0]=(-0.622228227201)/(o+1.)+(0.124165499776)+(-0.414130770151)*0.5**o
        ref[2,1,1]=(-0.671498979354)/(o+1.)+(-0.000499218910338)+(0.985892855782)*0.5**o
        ref[3,0,0]=(0.356244378791)/(o+1.)+(-0.653569869809)+(-0.547441466002)*0.5**o
        ref[3,0,1]=(0.942307305401)/(o+1.)+(-0.370220233281)+(-0.317803998398)*0.5**o
        ref[3,1,0]=(-0.437943014771)/(o+1.)+(-0.249049770163)+(-0.269300473712)*0.5**o
        ref[3,1,1]=(-0.866644131299)/(o+1.)+(-0.317984483685)+(0.0826330889069)*0.5**o
        ref[4,0,0]=(-0.408977166217)/(o+1.)+(0.737204241657)+(0.478312250921)*0.5**o
        ref[4,0,1]=(-0.878859546858)/(o+1.)+(0.498811007788)+(-0.221152804907)*0.5**o
        ref[4,1,0]=(0.0726134206382)/(o+1.)+(-0.0398713208557)+(-0.361901807816)*0.5**o
        ref[4,1,1]=(-0.11559899139)/(o+1.)+(0.167114538432)+(-0.729188028594)*0.5**o
        ref[5,0,0]=(-0.0108127700464)/(o+1.)+(0.315750801076)+(0.492678269676)*0.5**o
        ref[5,0,1]=(0.383281344894)/(o+1.)+(0.0963062625484)+(-0.0972486870808)*0.5**o
        ref[5,1,0]=(0.00851654852488)/(o+1.)+(-0.429741341803)+(0.442321568121)*0.5**o
        ref[5,1,1]=(-0.141223730539)/(o+1.)+(-0.525983079592)+(-0.723222258936)*0.5**o
      else:
        s[0,0,0]=(0.242677714297)*x[0]**o+(-0.400213939273)*x[0]+(0.406302331134)*x[1]**o+(0.0334900110197)*x[1]+(0.386616491205)*x[2]**o+(0.0312866694695)*x[2]
        s[0,0,1]=(-0.656430796333)*x[0]**o+(-0.563403496155)*x[0]+(0.364261714172)*x[1]**o+(0.967871502294)*x[1]+(-0.172721573545)*x[2]**o+(0.338588840536)*x[2]
        s[0,1,0]=(-0.204331459058)*x[0]**o+(-0.445139554107)*x[0]+(0.75436650149)*x[1]**o+(-0.932237793601)*x[1]+(0.767285047485)*x[2]**o+(-0.447011217796)*x[2]
        s[0,1,1]=(0.117575733479)*x[0]**o+(-0.474180475568)*x[0]+(0.259955578397)*x[1]**o+(-0.988666169557)*x[1]+(-0.773227945982)*x[2]**o+(-0.228748677163)*x[2]
        s[1,0,0]=(-0.885708131129)*x[0]**o+(0.255910330816)*x[0]+(-0.160503663494)*x[1]**o+(0.444458737043)*x[1]+(0.501800660641)*x[2]**o+(-0.403629782618)*x[2]
        s[1,0,1]=(-0.302400007694)*x[0]**o+(0.0508740505816)*x[0]+(0.149935410009)*x[1]**o+(-0.333426355106)*x[1]+(-0.83475153405)*x[2]**o+(-0.180357923768)*x[2]
        s[1,1,0]=(0.0405056122891)*x[0]**o+(-0.679078314383)*x[0]+(-0.630686479421)*x[1]**o+(0.882857130415)*x[1]+(-0.880049081361)*x[2]**o+(0.894022667917)*x[2]
        s[1,1,1]=(0.987734368607)*x[0]**o+(0.202787820601)*x[0]+(0.201427368636)*x[1]**o+(-0.888074805769)*x[1]+(-0.00577238788728)*x[2]**o+(-0.852216657415)*x[2]
        s[2,0,0]=(0.0784793303496)*x[0]**o+(0.525537564231)*x[0]+(-0.743068512178)*x[1]**o+(-0.96875188044)*x[1]+(0.598270641364)*x[2]**o+(-0.738260965829)*x[2]
        s[2,0,1]=(0.0371151946549)*x[0]**o+(0.834712924788)*x[0]+(0.831916271847)*x[1]**o+(0.986042089018)*x[1]+(-0.271258489529)*x[2]**o+(-0.0270137352729)*x[2]
        s[2,1,0]=(-0.610486701052)*x[0]**o+(0.747490812106)*x[0]+(-0.137581460043)*x[1]**o+(0.764360090052)*x[1]+(-0.992754974221)*x[2]**o+(-0.170181357609)*x[2]
        s[2,1,1]=(0.0760419308376)*x[0]**o+(-0.610739481473)*x[0]+(0.268671321666)*x[1]**o+(0.556159088599)*x[1]+(0.16105750649)*x[2]**o+(0.168476671552)*x[2]
        s[3,0,0]=(0.332144310632)*x[0]**o+(0.645502258606)*x[0]+(-0.303557252905)*x[1]**o+(-0.992657076111)*x[1]+(-0.367089118757)*x[2]**o+(-0.755868370964)*x[2]
        s[3,0,1]=(-0.538228335462)*x[0]**o+(0.853705065724)*x[0]+(-0.608592283692)*x[1]**o+(0.29533491918)*x[1]+(0.226379502601)*x[2]**o+(-0.20850693991)*x[2]
        s[3,1,0]=(-0.33389690791)*x[0]**o+(-0.804233206619)*x[0]+(0.787650315768)*x[1]**o+(-0.333017031742)*x[1]+(-0.803108583785)*x[2]**o+(-0.450538082625)*x[2]
        s[3,1,1]=(0.491492662254)*x[0]**o+(0.171534400605)*x[0]+(-0.777282924417)*x[1]**o+(-0.572907396269)*x[1]+(0.785138934858)*x[2]**o+(-0.436325021186)*x[2]
        s[4,0,0]=(0.247706032995)*x[0]**o+(0.905275953458)*x[0]+(0.0609169269224)*x[1]**o+(-0.256774805033)*x[1]+(0.998972177574)*x[2]**o+(-0.868940150916)*x[2]
        s[4,0,1]=(0.118732551906)*x[0]**o+(-0.622269538749)*x[0]+(0.118525354684)*x[1]**o+(-0.849217003233)*x[1]+(0.685082692712)*x[2]**o+(-0.230753706424)*x[2]
        s[4,1,0]=(-0.590456775201)*x[0]**o+(0.631958889331)*x[0]+(-0.0714558191005)*x[1]**o+(-0.336225807259)*x[1]+(0.896478419817)*x[2]**o+(0.376282690958)*x[2]
        s[4,1,1]=(0.437488625548)*x[0]**o+(0.883207845535)*x[0]+(0.594979146)*x[1]**o+(-0.627687609281)*x[1]+(0.9812267289)*x[2]**o+(0.571200283614)*x[2]
        s[5,0,0]=(-0.4090238662)*x[0]**o+(-0.712922930229)*x[0]+(-0.356126501112)*x[1]**o+(0.362147566863)*x[1]+(0.164164629436)*x[2]**o+(0.85169934858)*x[2]
        s[5,0,1]=(0.137560590284)*x[0]**o+(0.00568020697538)*x[0]+(-0.176059142701)*x[1]**o+(0.386227905303)*x[1]+(-0.823564872242)*x[2]**o+(-0.956332461174)*x[2]
        s[5,1,0]=(-0.719767553566)*x[0]**o+(-0.595774805843)*x[0]+(-0.783953676642)*x[1]**o+(0.686259147413)*x[1]+(-0.254758248118)*x[2]**o+(0.574267022665)*x[2]
        s[5,1,1]=(-0.321911973511)*x[0]**o+(-0.336478066213)*x[0]+(-0.777795569678)*x[1]**o+(0.896311892021)*x[1]+(-0.516413038874)*x[2]**o+(-0.786093273801)*x[2]
        ref[0,0,0]=(0.792918822339)/(o+1.)+(-0.167718629392)+(0.242677714297)*0.5**o
        ref[0,0,1]=(0.191540140628)/(o+1.)+(0.371528423337)+(-0.656430796333)*0.5**o
        ref[0,1,0]=(1.52165154897)/(o+1.)+(-0.912194282752)+(-0.204331459058)*0.5**o
        ref[0,1,1]=(-0.513272367585)/(o+1.)+(-0.845797661144)+(0.117575733479)*0.5**o
        ref[1,0,0]=(0.341296997147)/(o+1.)+(0.148369642621)+(-0.885708131129)*0.5**o
        ref[1,0,1]=(-0.684816124041)/(o+1.)+(-0.231455114146)+(-0.302400007694)*0.5**o
        ref[1,1,0]=(-1.51073556078)/(o+1.)+(0.548900741975)+(0.0405056122891)*0.5**o
        ref[1,1,1]=(0.195654980748)/(o+1.)+(-0.768751821292)+(0.987734368607)*0.5**o
        ref[2,0,0]=(-0.144797870814)/(o+1.)+(-0.590737641019)+(0.0784793303496)*0.5**o
        ref[2,0,1]=(0.560657782318)/(o+1.)+(0.896870639267)+(0.0371151946549)*0.5**o
        ref[2,1,0]=(-1.13033643426)/(o+1.)+(0.670834772275)+(-0.610486701052)*0.5**o
        ref[2,1,1]=(0.429728828155)/(o+1.)+(0.0569481393388)+(0.0760419308376)*0.5**o
        ref[3,0,0]=(-0.670646371662)/(o+1.)+(-0.551511594234)+(0.332144310632)*0.5**o
        ref[3,0,1]=(-0.382212781091)/(o+1.)+(0.470266522497)+(-0.538228335462)*0.5**o
        ref[3,1,0]=(-0.0154582680178)/(o+1.)+(-0.793894160493)+(-0.33389690791)*0.5**o
        ref[3,1,1]=(0.00785601044065)/(o+1.)+(-0.418849008425)+(0.491492662254)*0.5**o
        ref[4,0,0]=(1.0598891045)/(o+1.)+(-0.110219501245)+(0.247706032995)*0.5**o
        ref[4,0,1]=(0.803608047396)/(o+1.)+(-0.851120124203)+(0.118732551906)*0.5**o
        ref[4,1,0]=(0.825022600717)/(o+1.)+(0.336007886515)+(-0.590456775201)*0.5**o
        ref[4,1,1]=(1.5762058749)/(o+1.)+(0.413360259934)+(0.437488625548)*0.5**o
        ref[5,0,0]=(-0.191961871676)/(o+1.)+(0.250461992607)+(-0.4090238662)*0.5**o
        ref[5,0,1]=(-0.999624014943)/(o+1.)+(-0.282212174448)+(0.137560590284)*0.5**o
        ref[5,1,0]=(-1.03871192476)/(o+1.)+(0.332375682117)+(-0.719767553566)*0.5**o
        ref[5,1,1]=(-1.29420860855)/(o+1.)+(-0.113129723997)+(-0.321911973511)*0.5**o
      res=integrate(arg)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_integrate_onReducedFunctionOnContactOne_fromSymbol_ReducedFunctionOnContactOne_rank4(self):
      """
      tests integral of rank 4 Symbol on the ReducedFunctionOnContactOne

      assumptions: ReducedFunctionOnContactOne(self.domain) exists
                   self.domain supports integral on ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      w=ReducedFunctionOnContactOne(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5, 3, 2))
      s=Data(0,(4, 5, 3, 2),w)
      ref=numpy.zeros((4, 5, 3, 2),numpy.float_)
      if dim==2:
        s[0,0,0,0]=(0.245364066984)*x[0]**o+(-0.0501894287161)*x[0]+(0.666238426788)*x[1]**o+(0.136096078492)*x[1]
        s[0,0,0,1]=(0.716203034408)*x[0]**o+(-0.816588231639)*x[0]+(0.800426762254)*x[1]**o+(-0.182765397296)*x[1]
        s[0,0,1,0]=(0.122659986993)*x[0]**o+(-0.00208237425405)*x[0]+(-0.347405577261)*x[1]**o+(0.0376630412624)*x[1]
        s[0,0,1,1]=(0.791965912204)*x[0]**o+(0.935296430194)*x[0]+(0.330494636783)*x[1]**o+(0.990030993503)*x[1]
        s[0,0,2,0]=(0.761523262625)*x[0]**o+(0.47673649822)*x[0]+(-0.814581462689)*x[1]**o+(0.172965644708)*x[1]
        s[0,0,2,1]=(0.384651054435)*x[0]**o+(-0.723316715392)*x[0]+(0.349412253363)*x[1]**o+(-0.687923947756)*x[1]
        s[0,1,0,0]=(0.114523224402)*x[0]**o+(0.0462112886306)*x[0]+(-0.770917205125)*x[1]**o+(-0.251081721462)*x[1]
        s[0,1,0,1]=(0.697263286229)*x[0]**o+(-0.0252158836846)*x[0]+(0.933694211325)*x[1]**o+(0.650498198726)*x[1]
        s[0,1,1,0]=(-0.215924750352)*x[0]**o+(0.97660438665)*x[0]+(0.358477857654)*x[1]**o+(-0.603580756297)*x[1]
        s[0,1,1,1]=(0.642382357196)*x[0]**o+(0.891810567307)*x[0]+(0.839137774823)*x[1]**o+(-0.492580774398)*x[1]
        s[0,1,2,0]=(-0.260657110217)*x[0]**o+(-0.71383748794)*x[0]+(0.846765712381)*x[1]**o+(-0.686699503594)*x[1]
        s[0,1,2,1]=(0.733925405267)*x[0]**o+(-0.318186624057)*x[0]+(-0.690232848808)*x[1]**o+(0.941086052943)*x[1]
        s[0,2,0,0]=(0.230747459963)*x[0]**o+(0.873973601431)*x[0]+(0.0114100757165)*x[1]**o+(-0.172280871128)*x[1]
        s[0,2,0,1]=(0.191593969982)*x[0]**o+(-0.275332585321)*x[0]+(-0.403823163425)*x[1]**o+(0.765493218693)*x[1]
        s[0,2,1,0]=(-0.954865298124)*x[0]**o+(-0.840274453058)*x[0]+(-0.575828297224)*x[1]**o+(0.543288135427)*x[1]
        s[0,2,1,1]=(-0.304205093844)*x[0]**o+(0.366868842463)*x[0]+(0.950680046864)*x[1]**o+(0.644901674439)*x[1]
        s[0,2,2,0]=(-0.46074924128)*x[0]**o+(-0.100058859295)*x[0]+(-0.172613322727)*x[1]**o+(-0.10169890762)*x[1]
        s[0,2,2,1]=(0.119499701836)*x[0]**o+(-0.301349539686)*x[0]+(0.0936397977601)*x[1]**o+(0.361269456081)*x[1]
        s[0,3,0,0]=(-0.525741080321)*x[0]**o+(-0.680808261787)*x[0]+(0.0553816248689)*x[1]**o+(0.478185322022)*x[1]
        s[0,3,0,1]=(-0.241502398998)*x[0]**o+(0.831483674337)*x[0]+(-0.862336855278)*x[1]**o+(0.847378846767)*x[1]
        s[0,3,1,0]=(0.925044786414)*x[0]**o+(-0.20650710731)*x[0]+(-0.459505697865)*x[1]**o+(0.322417326119)*x[1]
        s[0,3,1,1]=(0.571196115307)*x[0]**o+(0.0591802893515)*x[0]+(0.880717796832)*x[1]**o+(0.813021896995)*x[1]
        s[0,3,2,0]=(-0.878805794321)*x[0]**o+(-0.404133715278)*x[0]+(-0.538866339841)*x[1]**o+(-0.51964634727)*x[1]
        s[0,3,2,1]=(0.968894119383)*x[0]**o+(0.535010672206)*x[0]+(-0.543638502129)*x[1]**o+(-0.0775549893758)*x[1]
        s[0,4,0,0]=(-0.473571740079)*x[0]**o+(0.981704155322)*x[0]+(-0.732685132335)*x[1]**o+(0.151201399401)*x[1]
        s[0,4,0,1]=(0.107676577578)*x[0]**o+(-0.0898161469871)*x[0]+(-0.243173822645)*x[1]**o+(0.425562025311)*x[1]
        s[0,4,1,0]=(-0.912740962221)*x[0]**o+(-0.307527338956)*x[0]+(-0.482823172795)*x[1]**o+(-0.836767222685)*x[1]
        s[0,4,1,1]=(-0.106203879274)*x[0]**o+(0.0999297363975)*x[0]+(0.605357795695)*x[1]**o+(-0.864222319735)*x[1]
        s[0,4,2,0]=(0.587529237973)*x[0]**o+(0.140953847793)*x[0]+(0.64759005987)*x[1]**o+(-0.0275484404904)*x[1]
        s[0,4,2,1]=(0.482928938234)*x[0]**o+(0.775154595317)*x[0]+(0.898533268559)*x[1]**o+(-0.238327086159)*x[1]
        s[1,0,0,0]=(-0.574069031279)*x[0]**o+(-0.59623538758)*x[0]+(0.388920605463)*x[1]**o+(-0.994406853141)*x[1]
        s[1,0,0,1]=(0.939040505856)*x[0]**o+(-0.038754823699)*x[0]+(-0.279256974829)*x[1]**o+(-0.890808737563)*x[1]
        s[1,0,1,0]=(-0.944753068273)*x[0]**o+(-0.696943036951)*x[0]+(-0.220060979436)*x[1]**o+(0.223607800841)*x[1]
        s[1,0,1,1]=(-0.390329725129)*x[0]**o+(-0.262404608658)*x[0]+(-0.421100034358)*x[1]**o+(0.922126389343)*x[1]
        s[1,0,2,0]=(-0.7832351829)*x[0]**o+(-0.155019228142)*x[0]+(-0.316428480376)*x[1]**o+(0.604192200421)*x[1]
        s[1,0,2,1]=(0.414384140175)*x[0]**o+(-0.000244010932926)*x[0]+(-0.46577339251)*x[1]**o+(0.869470477908)*x[1]
        s[1,1,0,0]=(-0.571703274446)*x[0]**o+(-0.897490452042)*x[0]+(0.151601513312)*x[1]**o+(-0.820411349281)*x[1]
        s[1,1,0,1]=(0.883610930016)*x[0]**o+(0.430543960795)*x[0]+(0.174748731929)*x[1]**o+(-0.346504921967)*x[1]
        s[1,1,1,0]=(-0.965747586125)*x[0]**o+(0.652587346149)*x[0]+(-0.454627455637)*x[1]**o+(-0.126065048744)*x[1]
        s[1,1,1,1]=(-0.778723024919)*x[0]**o+(-0.352555666852)*x[0]+(0.378393901391)*x[1]**o+(-0.393622640877)*x[1]
        s[1,1,2,0]=(0.785822593379)*x[0]**o+(0.374891382708)*x[0]+(-0.993366900714)*x[1]**o+(-0.406778939882)*x[1]
        s[1,1,2,1]=(0.238887912688)*x[0]**o+(-0.668847103585)*x[0]+(0.422750071837)*x[1]**o+(0.290969707905)*x[1]
        s[1,2,0,0]=(0.306452161355)*x[0]**o+(0.0845930872771)*x[0]+(0.288188272131)*x[1]**o+(0.611272197726)*x[1]
        s[1,2,0,1]=(0.535104365658)*x[0]**o+(-0.868361395667)*x[0]+(-0.557799747963)*x[1]**o+(0.189710523839)*x[1]
        s[1,2,1,0]=(0.653062501973)*x[0]**o+(-0.980469515167)*x[0]+(-0.126034299636)*x[1]**o+(-0.702252714095)*x[1]
        s[1,2,1,1]=(0.193513872604)*x[0]**o+(0.716250857589)*x[0]+(0.532447219707)*x[1]**o+(0.0396611049339)*x[1]
        s[1,2,2,0]=(0.471722480056)*x[0]**o+(-0.386703912908)*x[0]+(0.508907147346)*x[1]**o+(-0.431622849878)*x[1]
        s[1,2,2,1]=(0.145178046285)*x[0]**o+(-0.0110222671767)*x[0]+(-0.793600779578)*x[1]**o+(0.482117912584)*x[1]
        s[1,3,0,0]=(0.883901734656)*x[0]**o+(0.252857222204)*x[0]+(-0.349615695314)*x[1]**o+(-0.831352541736)*x[1]
        s[1,3,0,1]=(-0.439872366735)*x[0]**o+(0.855050596466)*x[0]+(-0.892859579327)*x[1]**o+(-0.499483379044)*x[1]
        s[1,3,1,0]=(-0.310902411804)*x[0]**o+(0.909694833553)*x[0]+(0.432470372195)*x[1]**o+(-0.181254854143)*x[1]
        s[1,3,1,1]=(-0.457038984067)*x[0]**o+(-0.932819775606)*x[0]+(-0.0215549839659)*x[1]**o+(-0.724030992167)*x[1]
        s[1,3,2,0]=(-0.307363570855)*x[0]**o+(-0.53814730682)*x[0]+(-0.0248867951466)*x[1]**o+(0.230518328746)*x[1]
        s[1,3,2,1]=(0.303121965084)*x[0]**o+(-0.493904339796)*x[0]+(0.917168913893)*x[1]**o+(0.169108758838)*x[1]
        s[1,4,0,0]=(-0.713672877707)*x[0]**o+(0.850083130815)*x[0]+(0.859962057023)*x[1]**o+(0.154568816707)*x[1]
        s[1,4,0,1]=(-0.708070236639)*x[0]**o+(0.264936920001)*x[0]+(0.376600276672)*x[1]**o+(0.880588901929)*x[1]
        s[1,4,1,0]=(0.203382859429)*x[0]**o+(0.834221276526)*x[0]+(-0.271980968262)*x[1]**o+(0.840406001263)*x[1]
        s[1,4,1,1]=(-0.610374454703)*x[0]**o+(-0.299939554248)*x[0]+(-0.876972535039)*x[1]**o+(0.613391714781)*x[1]
        s[1,4,2,0]=(-0.131008373859)*x[0]**o+(0.564430441667)*x[0]+(0.512449401496)*x[1]**o+(-0.072742245846)*x[1]
        s[1,4,2,1]=(0.994369815068)*x[0]**o+(0.214403792021)*x[0]+(-0.731489669807)*x[1]**o+(-0.778494326982)*x[1]
        s[2,0,0,0]=(0.687813573864)*x[0]**o+(-0.342489594146)*x[0]+(-0.092041610109)*x[1]**o+(-0.388723275983)*x[1]
        s[2,0,0,1]=(-0.889705548572)*x[0]**o+(-0.962296902814)*x[0]+(0.284484113765)*x[1]**o+(-0.312781157642)*x[1]
        s[2,0,1,0]=(-0.0691392356773)*x[0]**o+(0.391307995277)*x[0]+(-0.343166268962)*x[1]**o+(-0.100528288088)*x[1]
        s[2,0,1,1]=(-0.767504390408)*x[0]**o+(0.44775317228)*x[0]+(0.00928458441193)*x[1]**o+(0.481385818795)*x[1]
        s[2,0,2,0]=(0.151265878662)*x[0]**o+(0.934060057849)*x[0]+(0.772262051249)*x[1]**o+(0.0303447230092)*x[1]
        s[2,0,2,1]=(0.638051378377)*x[0]**o+(0.795393839494)*x[0]+(0.812739684548)*x[1]**o+(0.683335661484)*x[1]
        s[2,1,0,0]=(0.699425983168)*x[0]**o+(-0.113127267833)*x[0]+(0.753464652023)*x[1]**o+(-0.860186623524)*x[1]
        s[2,1,0,1]=(-0.228981326624)*x[0]**o+(-0.144953596548)*x[0]+(-0.028962291177)*x[1]**o+(0.274475259334)*x[1]
        s[2,1,1,0]=(0.436232377097)*x[0]**o+(0.851458153457)*x[0]+(0.449876871146)*x[1]**o+(0.929900037818)*x[1]
        s[2,1,1,1]=(0.108152697095)*x[0]**o+(0.701445596547)*x[0]+(0.857688044529)*x[1]**o+(-0.32024176733)*x[1]
        s[2,1,2,0]=(0.776899779272)*x[0]**o+(-0.77488067334)*x[0]+(0.515979568236)*x[1]**o+(0.60407405593)*x[1]
        s[2,1,2,1]=(0.486236495643)*x[0]**o+(0.759871578074)*x[0]+(0.320690937285)*x[1]**o+(-0.178307017545)*x[1]
        s[2,2,0,0]=(-0.2834031779)*x[0]**o+(-0.99059970496)*x[0]+(-0.951630400835)*x[1]**o+(-0.822318070375)*x[1]
        s[2,2,0,1]=(-0.246680973215)*x[0]**o+(0.895886965624)*x[0]+(0.331676331677)*x[1]**o+(0.275149795318)*x[1]
        s[2,2,1,0]=(0.682460024363)*x[0]**o+(-0.191950621609)*x[0]+(0.998873537117)*x[1]**o+(-0.62286471982)*x[1]
        s[2,2,1,1]=(0.371007993444)*x[0]**o+(0.75354844415)*x[0]+(0.231689506568)*x[1]**o+(0.319850912994)*x[1]
        s[2,2,2,0]=(0.273036358499)*x[0]**o+(-0.365860549733)*x[0]+(-0.255255520802)*x[1]**o+(-0.856177999361)*x[1]
        s[2,2,2,1]=(-0.829888508727)*x[0]**o+(-0.869979484799)*x[0]+(0.0422078078521)*x[1]**o+(-0.225105932622)*x[1]
        s[2,3,0,0]=(-0.901686992463)*x[0]**o+(0.52317892576)*x[0]+(0.450664578307)*x[1]**o+(0.539264486669)*x[1]
        s[2,3,0,1]=(-0.146310214982)*x[0]**o+(-0.866238792091)*x[0]+(-0.411575583885)*x[1]**o+(-0.767910846528)*x[1]
        s[2,3,1,0]=(0.796379240086)*x[0]**o+(-0.941611444233)*x[0]+(0.457329256522)*x[1]**o+(-0.205094413155)*x[1]
        s[2,3,1,1]=(0.00162132693744)*x[0]**o+(0.809781891022)*x[0]+(-0.521975026805)*x[1]**o+(-0.832935611569)*x[1]
        s[2,3,2,0]=(0.85913616245)*x[0]**o+(-0.97226394808)*x[0]+(-0.958220202856)*x[1]**o+(-0.838570412967)*x[1]
        s[2,3,2,1]=(-0.394075398541)*x[0]**o+(0.363958258657)*x[0]+(-0.0245518988658)*x[1]**o+(-0.0641697369324)*x[1]
        s[2,4,0,0]=(0.53874901568)*x[0]**o+(0.381652052125)*x[0]+(-0.569006390059)*x[1]**o+(0.803939044804)*x[1]
        s[2,4,0,1]=(-0.161281051202)*x[0]**o+(-0.29794326221)*x[0]+(-0.975738807839)*x[1]**o+(-0.365943217167)*x[1]
        s[2,4,1,0]=(0.937384486165)*x[0]**o+(-0.111220331573)*x[0]+(-0.705253089868)*x[1]**o+(-0.0964593740972)*x[1]
        s[2,4,1,1]=(-0.801537343551)*x[0]**o+(-0.0721007760916)*x[0]+(-0.858896629841)*x[1]**o+(0.893882022578)*x[1]
        s[2,4,2,0]=(-0.386195011979)*x[0]**o+(-0.7171939977)*x[0]+(-0.291590089274)*x[1]**o+(0.653160575582)*x[1]
        s[2,4,2,1]=(0.15418789046)*x[0]**o+(-0.337070580033)*x[0]+(-0.826397574623)*x[1]**o+(-0.492621231443)*x[1]
        s[3,0,0,0]=(0.208187397837)*x[0]**o+(-0.941176621475)*x[0]+(-0.553746758603)*x[1]**o+(0.298235037771)*x[1]
        s[3,0,0,1]=(-0.608160638828)*x[0]**o+(0.969633806238)*x[0]+(0.594884783145)*x[1]**o+(0.256426888478)*x[1]
        s[3,0,1,0]=(0.156268978359)*x[0]**o+(-0.618114287912)*x[0]+(-0.736845954486)*x[1]**o+(0.578007765377)*x[1]
        s[3,0,1,1]=(0.0779253738219)*x[0]**o+(0.487197266572)*x[0]+(0.712697073421)*x[1]**o+(-0.333683256167)*x[1]
        s[3,0,2,0]=(-0.432372166788)*x[0]**o+(-0.670271358766)*x[0]+(-0.187482013069)*x[1]**o+(0.177483210094)*x[1]
        s[3,0,2,1]=(0.893891903718)*x[0]**o+(0.0668637308586)*x[0]+(-0.434296768839)*x[1]**o+(-0.297584773434)*x[1]
        s[3,1,0,0]=(-0.226039889634)*x[0]**o+(0.0244084275305)*x[0]+(0.0904503965701)*x[1]**o+(0.826411056795)*x[1]
        s[3,1,0,1]=(0.605633212937)*x[0]**o+(0.69508670693)*x[0]+(0.317716807233)*x[1]**o+(-0.71784966885)*x[1]
        s[3,1,1,0]=(0.697916267298)*x[0]**o+(-0.766126387311)*x[0]+(0.679356018736)*x[1]**o+(0.814149484578)*x[1]
        s[3,1,1,1]=(0.495275603887)*x[0]**o+(0.92183314391)*x[0]+(0.0319182837637)*x[1]**o+(0.550197566786)*x[1]
        s[3,1,2,0]=(-0.549402940272)*x[0]**o+(0.16459055678)*x[0]+(0.6282270637)*x[1]**o+(-0.19466468363)*x[1]
        s[3,1,2,1]=(0.722507137464)*x[0]**o+(0.0201257259776)*x[0]+(0.864760483197)*x[1]**o+(-0.70699130253)*x[1]
        s[3,2,0,0]=(-0.911053389534)*x[0]**o+(0.242037718776)*x[0]+(0.296048767645)*x[1]**o+(-0.786412484197)*x[1]
        s[3,2,0,1]=(-0.278556298185)*x[0]**o+(-0.330948724575)*x[0]+(-0.808707884082)*x[1]**o+(0.649152363362)*x[1]
        s[3,2,1,0]=(0.527331068218)*x[0]**o+(0.535793375918)*x[0]+(0.0173164046642)*x[1]**o+(0.673124828911)*x[1]
        s[3,2,1,1]=(-0.122860429792)*x[0]**o+(-0.33128580821)*x[0]+(-0.875323756482)*x[1]**o+(-0.404630192101)*x[1]
        s[3,2,2,0]=(0.396517792786)*x[0]**o+(-0.992046803715)*x[0]+(0.994743180916)*x[1]**o+(-0.355368679756)*x[1]
        s[3,2,2,1]=(0.988551628034)*x[0]**o+(-0.248207320263)*x[0]+(-0.536981937311)*x[1]**o+(-0.832557705835)*x[1]
        s[3,3,0,0]=(-0.687076681329)*x[0]**o+(0.301843762611)*x[0]+(0.0261628413994)*x[1]**o+(0.257680047964)*x[1]
        s[3,3,0,1]=(-0.0679744087296)*x[0]**o+(-0.388391714998)*x[0]+(-0.305536788937)*x[1]**o+(0.813610001342)*x[1]
        s[3,3,1,0]=(0.520286217752)*x[0]**o+(0.787102883932)*x[0]+(-0.443820422756)*x[1]**o+(0.348550770428)*x[1]
        s[3,3,1,1]=(-0.403687854392)*x[0]**o+(0.224198640317)*x[0]+(0.908268189821)*x[1]**o+(-0.0189418862158)*x[1]
        s[3,3,2,0]=(0.106397622508)*x[0]**o+(0.408482462665)*x[0]+(-0.189398394495)*x[1]**o+(0.892444309304)*x[1]
        s[3,3,2,1]=(-0.158875590567)*x[0]**o+(0.659863215048)*x[0]+(-0.116276105024)*x[1]**o+(-0.435483985612)*x[1]
        s[3,4,0,0]=(-0.00526118581715)*x[0]**o+(0.390573315765)*x[0]+(-0.865881629704)*x[1]**o+(0.188788198915)*x[1]
        s[3,4,0,1]=(0.457024238224)*x[0]**o+(-0.142788838274)*x[0]+(0.514831683476)*x[1]**o+(0.645719634512)*x[1]
        s[3,4,1,0]=(-0.221092754553)*x[0]**o+(0.42876217463)*x[0]+(-0.0146700209904)*x[1]**o+(-0.558199901549)*x[1]
        s[3,4,1,1]=(-0.0854320790552)*x[0]**o+(0.495872652018)*x[0]+(-0.307890759071)*x[1]**o+(-0.80450589936)*x[1]
        s[3,4,2,0]=(0.390350106853)*x[0]**o+(-0.469202978344)*x[0]+(0.21377496022)*x[1]**o+(-0.83671140107)*x[1]
        s[3,4,2,1]=(-0.248241865193)*x[0]**o+(-0.0543747795217)*x[0]+(-0.946132717359)*x[1]**o+(0.72654761542)*x[1]
        ref[0,0,0,0]=(0.666238426788)/(o+1.)+(0.0429533248881)+(0.245364066984)*0.5**o
        ref[0,0,0,1]=(0.800426762254)/(o+1.)+(-0.499676814468)+(0.716203034408)*0.5**o
        ref[0,0,1,0]=(-0.347405577261)/(o+1.)+(0.0177903335042)+(0.122659986993)*0.5**o
        ref[0,0,1,1]=(0.330494636783)/(o+1.)+(0.962663711849)+(0.791965912204)*0.5**o
        ref[0,0,2,0]=(-0.814581462689)/(o+1.)+(0.324851071464)+(0.761523262625)*0.5**o
        ref[0,0,2,1]=(0.349412253363)/(o+1.)+(-0.705620331574)+(0.384651054435)*0.5**o
        ref[0,1,0,0]=(-0.770917205125)/(o+1.)+(-0.102435216416)+(0.114523224402)*0.5**o
        ref[0,1,0,1]=(0.933694211325)/(o+1.)+(0.312641157521)+(0.697263286229)*0.5**o
        ref[0,1,1,0]=(0.358477857654)/(o+1.)+(0.186511815176)+(-0.215924750352)*0.5**o
        ref[0,1,1,1]=(0.839137774823)/(o+1.)+(0.199614896454)+(0.642382357196)*0.5**o
        ref[0,1,2,0]=(0.846765712381)/(o+1.)+(-0.700268495767)+(-0.260657110217)*0.5**o
        ref[0,1,2,1]=(-0.690232848808)/(o+1.)+(0.311449714443)+(0.733925405267)*0.5**o
        ref[0,2,0,0]=(0.0114100757165)/(o+1.)+(0.350846365151)+(0.230747459963)*0.5**o
        ref[0,2,0,1]=(-0.403823163425)/(o+1.)+(0.245080316686)+(0.191593969982)*0.5**o
        ref[0,2,1,0]=(-0.575828297224)/(o+1.)+(-0.148493158816)+(-0.954865298124)*0.5**o
        ref[0,2,1,1]=(0.950680046864)/(o+1.)+(0.505885258451)+(-0.304205093844)*0.5**o
        ref[0,2,2,0]=(-0.172613322727)/(o+1.)+(-0.100878883458)+(-0.46074924128)*0.5**o
        ref[0,2,2,1]=(0.0936397977601)/(o+1.)+(0.0299599581971)+(0.119499701836)*0.5**o
        ref[0,3,0,0]=(0.0553816248689)/(o+1.)+(-0.101311469882)+(-0.525741080321)*0.5**o
        ref[0,3,0,1]=(-0.862336855278)/(o+1.)+(0.839431260552)+(-0.241502398998)*0.5**o
        ref[0,3,1,0]=(-0.459505697865)/(o+1.)+(0.0579551094043)+(0.925044786414)*0.5**o
        ref[0,3,1,1]=(0.880717796832)/(o+1.)+(0.436101093173)+(0.571196115307)*0.5**o
        ref[0,3,2,0]=(-0.538866339841)/(o+1.)+(-0.461890031274)+(-0.878805794321)*0.5**o
        ref[0,3,2,1]=(-0.543638502129)/(o+1.)+(0.228727841415)+(0.968894119383)*0.5**o
        ref[0,4,0,0]=(-0.732685132335)/(o+1.)+(0.566452777361)+(-0.473571740079)*0.5**o
        ref[0,4,0,1]=(-0.243173822645)/(o+1.)+(0.167872939162)+(0.107676577578)*0.5**o
        ref[0,4,1,0]=(-0.482823172795)/(o+1.)+(-0.572147280821)+(-0.912740962221)*0.5**o
        ref[0,4,1,1]=(0.605357795695)/(o+1.)+(-0.382146291669)+(-0.106203879274)*0.5**o
        ref[0,4,2,0]=(0.64759005987)/(o+1.)+(0.0567027036512)+(0.587529237973)*0.5**o
        ref[0,4,2,1]=(0.898533268559)/(o+1.)+(0.268413754579)+(0.482928938234)*0.5**o
        ref[1,0,0,0]=(0.388920605463)/(o+1.)+(-0.795321120361)+(-0.574069031279)*0.5**o
        ref[1,0,0,1]=(-0.279256974829)/(o+1.)+(-0.464781780631)+(0.939040505856)*0.5**o
        ref[1,0,1,0]=(-0.220060979436)/(o+1.)+(-0.236667618055)+(-0.944753068273)*0.5**o
        ref[1,0,1,1]=(-0.421100034358)/(o+1.)+(0.329860890342)+(-0.390329725129)*0.5**o
        ref[1,0,2,0]=(-0.316428480376)/(o+1.)+(0.22458648614)+(-0.7832351829)*0.5**o
        ref[1,0,2,1]=(-0.46577339251)/(o+1.)+(0.434613233487)+(0.414384140175)*0.5**o
        ref[1,1,0,0]=(0.151601513312)/(o+1.)+(-0.858950900661)+(-0.571703274446)*0.5**o
        ref[1,1,0,1]=(0.174748731929)/(o+1.)+(0.0420195194137)+(0.883610930016)*0.5**o
        ref[1,1,1,0]=(-0.454627455637)/(o+1.)+(0.263261148702)+(-0.965747586125)*0.5**o
        ref[1,1,1,1]=(0.378393901391)/(o+1.)+(-0.373089153865)+(-0.778723024919)*0.5**o
        ref[1,1,2,0]=(-0.993366900714)/(o+1.)+(-0.0159437785867)+(0.785822593379)*0.5**o
        ref[1,1,2,1]=(0.422750071837)/(o+1.)+(-0.18893869784)+(0.238887912688)*0.5**o
        ref[1,2,0,0]=(0.288188272131)/(o+1.)+(0.347932642501)+(0.306452161355)*0.5**o
        ref[1,2,0,1]=(-0.557799747963)/(o+1.)+(-0.339325435914)+(0.535104365658)*0.5**o
        ref[1,2,1,0]=(-0.126034299636)/(o+1.)+(-0.841361114631)+(0.653062501973)*0.5**o
        ref[1,2,1,1]=(0.532447219707)/(o+1.)+(0.377955981261)+(0.193513872604)*0.5**o
        ref[1,2,2,0]=(0.508907147346)/(o+1.)+(-0.409163381393)+(0.471722480056)*0.5**o
        ref[1,2,2,1]=(-0.793600779578)/(o+1.)+(0.235547822704)+(0.145178046285)*0.5**o
        ref[1,3,0,0]=(-0.349615695314)/(o+1.)+(-0.289247659766)+(0.883901734656)*0.5**o
        ref[1,3,0,1]=(-0.892859579327)/(o+1.)+(0.177783608711)+(-0.439872366735)*0.5**o
        ref[1,3,1,0]=(0.432470372195)/(o+1.)+(0.364219989705)+(-0.310902411804)*0.5**o
        ref[1,3,1,1]=(-0.0215549839659)/(o+1.)+(-0.828425383886)+(-0.457038984067)*0.5**o
        ref[1,3,2,0]=(-0.0248867951466)/(o+1.)+(-0.153814489037)+(-0.307363570855)*0.5**o
        ref[1,3,2,1]=(0.917168913893)/(o+1.)+(-0.162397790479)+(0.303121965084)*0.5**o
        ref[1,4,0,0]=(0.859962057023)/(o+1.)+(0.502325973761)+(-0.713672877707)*0.5**o
        ref[1,4,0,1]=(0.376600276672)/(o+1.)+(0.572762910965)+(-0.708070236639)*0.5**o
        ref[1,4,1,0]=(-0.271980968262)/(o+1.)+(0.837313638895)+(0.203382859429)*0.5**o
        ref[1,4,1,1]=(-0.876972535039)/(o+1.)+(0.156726080267)+(-0.610374454703)*0.5**o
        ref[1,4,2,0]=(0.512449401496)/(o+1.)+(0.245844097911)+(-0.131008373859)*0.5**o
        ref[1,4,2,1]=(-0.731489669807)/(o+1.)+(-0.28204526748)+(0.994369815068)*0.5**o
        ref[2,0,0,0]=(-0.092041610109)/(o+1.)+(-0.365606435065)+(0.687813573864)*0.5**o
        ref[2,0,0,1]=(0.284484113765)/(o+1.)+(-0.637539030228)+(-0.889705548572)*0.5**o
        ref[2,0,1,0]=(-0.343166268962)/(o+1.)+(0.145389853595)+(-0.0691392356773)*0.5**o
        ref[2,0,1,1]=(0.00928458441193)/(o+1.)+(0.464569495537)+(-0.767504390408)*0.5**o
        ref[2,0,2,0]=(0.772262051249)/(o+1.)+(0.482202390429)+(0.151265878662)*0.5**o
        ref[2,0,2,1]=(0.812739684548)/(o+1.)+(0.739364750489)+(0.638051378377)*0.5**o
        ref[2,1,0,0]=(0.753464652023)/(o+1.)+(-0.486656945678)+(0.699425983168)*0.5**o
        ref[2,1,0,1]=(-0.028962291177)/(o+1.)+(0.0647608313931)+(-0.228981326624)*0.5**o
        ref[2,1,1,0]=(0.449876871146)/(o+1.)+(0.890679095638)+(0.436232377097)*0.5**o
        ref[2,1,1,1]=(0.857688044529)/(o+1.)+(0.190601914609)+(0.108152697095)*0.5**o
        ref[2,1,2,0]=(0.515979568236)/(o+1.)+(-0.0854033087054)+(0.776899779272)*0.5**o
        ref[2,1,2,1]=(0.320690937285)/(o+1.)+(0.290782280265)+(0.486236495643)*0.5**o
        ref[2,2,0,0]=(-0.951630400835)/(o+1.)+(-0.906458887668)+(-0.2834031779)*0.5**o
        ref[2,2,0,1]=(0.331676331677)/(o+1.)+(0.585518380471)+(-0.246680973215)*0.5**o
        ref[2,2,1,0]=(0.998873537117)/(o+1.)+(-0.407407670715)+(0.682460024363)*0.5**o
        ref[2,2,1,1]=(0.231689506568)/(o+1.)+(0.536699678572)+(0.371007993444)*0.5**o
        ref[2,2,2,0]=(-0.255255520802)/(o+1.)+(-0.611019274547)+(0.273036358499)*0.5**o
        ref[2,2,2,1]=(0.0422078078521)/(o+1.)+(-0.54754270871)+(-0.829888508727)*0.5**o
        ref[2,3,0,0]=(0.450664578307)/(o+1.)+(0.531221706214)+(-0.901686992463)*0.5**o
        ref[2,3,0,1]=(-0.411575583885)/(o+1.)+(-0.81707481931)+(-0.146310214982)*0.5**o
        ref[2,3,1,0]=(0.457329256522)/(o+1.)+(-0.573352928694)+(0.796379240086)*0.5**o
        ref[2,3,1,1]=(-0.521975026805)/(o+1.)+(-0.0115768602733)+(0.00162132693744)*0.5**o
        ref[2,3,2,0]=(-0.958220202856)/(o+1.)+(-0.905417180523)+(0.85913616245)*0.5**o
        ref[2,3,2,1]=(-0.0245518988658)/(o+1.)+(0.149894260862)+(-0.394075398541)*0.5**o
        ref[2,4,0,0]=(-0.569006390059)/(o+1.)+(0.592795548465)+(0.53874901568)*0.5**o
        ref[2,4,0,1]=(-0.975738807839)/(o+1.)+(-0.331943239689)+(-0.161281051202)*0.5**o
        ref[2,4,1,0]=(-0.705253089868)/(o+1.)+(-0.103839852835)+(0.937384486165)*0.5**o
        ref[2,4,1,1]=(-0.858896629841)/(o+1.)+(0.410890623243)+(-0.801537343551)*0.5**o
        ref[2,4,2,0]=(-0.291590089274)/(o+1.)+(-0.032016711059)+(-0.386195011979)*0.5**o
        ref[2,4,2,1]=(-0.826397574623)/(o+1.)+(-0.414845905738)+(0.15418789046)*0.5**o
        ref[3,0,0,0]=(-0.553746758603)/(o+1.)+(-0.321470791852)+(0.208187397837)*0.5**o
        ref[3,0,0,1]=(0.594884783145)/(o+1.)+(0.613030347358)+(-0.608160638828)*0.5**o
        ref[3,0,1,0]=(-0.736845954486)/(o+1.)+(-0.0200532612677)+(0.156268978359)*0.5**o
        ref[3,0,1,1]=(0.712697073421)/(o+1.)+(0.0767570052027)+(0.0779253738219)*0.5**o
        ref[3,0,2,0]=(-0.187482013069)/(o+1.)+(-0.246394074336)+(-0.432372166788)*0.5**o
        ref[3,0,2,1]=(-0.434296768839)/(o+1.)+(-0.115360521288)+(0.893891903718)*0.5**o
        ref[3,1,0,0]=(0.0904503965701)/(o+1.)+(0.425409742163)+(-0.226039889634)*0.5**o
        ref[3,1,0,1]=(0.317716807233)/(o+1.)+(-0.0113814809604)+(0.605633212937)*0.5**o
        ref[3,1,1,0]=(0.679356018736)/(o+1.)+(0.0240115486338)+(0.697916267298)*0.5**o
        ref[3,1,1,1]=(0.0319182837637)/(o+1.)+(0.736015355348)+(0.495275603887)*0.5**o
        ref[3,1,2,0]=(0.6282270637)/(o+1.)+(-0.015037063425)+(-0.549402940272)*0.5**o
        ref[3,1,2,1]=(0.864760483197)/(o+1.)+(-0.343432788276)+(0.722507137464)*0.5**o
        ref[3,2,0,0]=(0.296048767645)/(o+1.)+(-0.272187382711)+(-0.911053389534)*0.5**o
        ref[3,2,0,1]=(-0.808707884082)/(o+1.)+(0.159101819394)+(-0.278556298185)*0.5**o
        ref[3,2,1,0]=(0.0173164046642)/(o+1.)+(0.604459102415)+(0.527331068218)*0.5**o
        ref[3,2,1,1]=(-0.875323756482)/(o+1.)+(-0.367958000155)+(-0.122860429792)*0.5**o
        ref[3,2,2,0]=(0.994743180916)/(o+1.)+(-0.673707741736)+(0.396517792786)*0.5**o
        ref[3,2,2,1]=(-0.536981937311)/(o+1.)+(-0.540382513049)+(0.988551628034)*0.5**o
        ref[3,3,0,0]=(0.0261628413994)/(o+1.)+(0.279761905288)+(-0.687076681329)*0.5**o
        ref[3,3,0,1]=(-0.305536788937)/(o+1.)+(0.212609143172)+(-0.0679744087296)*0.5**o
        ref[3,3,1,0]=(-0.443820422756)/(o+1.)+(0.56782682718)+(0.520286217752)*0.5**o
        ref[3,3,1,1]=(0.908268189821)/(o+1.)+(0.10262837705)+(-0.403687854392)*0.5**o
        ref[3,3,2,0]=(-0.189398394495)/(o+1.)+(0.650463385984)+(0.106397622508)*0.5**o
        ref[3,3,2,1]=(-0.116276105024)/(o+1.)+(0.112189614718)+(-0.158875590567)*0.5**o
        ref[3,4,0,0]=(-0.865881629704)/(o+1.)+(0.28968075734)+(-0.00526118581715)*0.5**o
        ref[3,4,0,1]=(0.514831683476)/(o+1.)+(0.251465398119)+(0.457024238224)*0.5**o
        ref[3,4,1,0]=(-0.0146700209904)/(o+1.)+(-0.0647188634597)+(-0.221092754553)*0.5**o
        ref[3,4,1,1]=(-0.307890759071)/(o+1.)+(-0.154316623671)+(-0.0854320790552)*0.5**o
        ref[3,4,2,0]=(0.21377496022)/(o+1.)+(-0.652957189707)+(0.390350106853)*0.5**o
        ref[3,4,2,1]=(-0.946132717359)/(o+1.)+(0.336086417949)+(-0.248241865193)*0.5**o
      else:
        s[0,0,0,0]=(-0.210183765573)*x[0]**o+(0.259610104977)*x[0]+(0.840819899605)*x[1]**o+(-0.103339866116)*x[1]+(0.737406932578)*x[2]**o+(0.712156905811)*x[2]
        s[0,0,0,1]=(-0.323968020416)*x[0]**o+(-0.876969115567)*x[0]+(-0.906271643259)*x[1]**o+(-0.366737699368)*x[1]+(0.367538667762)*x[2]**o+(0.80273967237)*x[2]
        s[0,0,1,0]=(0.467518636672)*x[0]**o+(0.684316475842)*x[0]+(-0.948313733183)*x[1]**o+(-0.768576249031)*x[1]+(-0.169935408354)*x[2]**o+(-0.747881358075)*x[2]
        s[0,0,1,1]=(0.976876531867)*x[0]**o+(0.561100376553)*x[0]+(0.450819300656)*x[1]**o+(-0.0490235546318)*x[1]+(-0.698776567424)*x[2]**o+(-0.01350381399)*x[2]
        s[0,0,2,0]=(-0.552336862133)*x[0]**o+(-0.564574791577)*x[0]+(0.95546748665)*x[1]**o+(-0.211505026053)*x[1]+(-0.224770603639)*x[2]**o+(0.498420086189)*x[2]
        s[0,0,2,1]=(0.110734087997)*x[0]**o+(0.256257276464)*x[0]+(-0.682963863817)*x[1]**o+(-0.832585277356)*x[1]+(-0.0216621586129)*x[2]**o+(0.171233268331)*x[2]
        s[0,1,0,0]=(0.979769828626)*x[0]**o+(-0.918534054584)*x[0]+(-0.770334932534)*x[1]**o+(-0.266282551716)*x[1]+(-0.528515244986)*x[2]**o+(0.452232798522)*x[2]
        s[0,1,0,1]=(0.184115163182)*x[0]**o+(0.0372768333589)*x[0]+(0.0697681712771)*x[1]**o+(0.650340145402)*x[1]+(0.89670654092)*x[2]**o+(0.0666608228764)*x[2]
        s[0,1,1,0]=(0.0822751335539)*x[0]**o+(-0.145731529462)*x[0]+(-0.804584197287)*x[1]**o+(-0.367596869981)*x[1]+(-0.819068186934)*x[2]**o+(-0.761154318012)*x[2]
        s[0,1,1,1]=(0.672877076993)*x[0]**o+(-0.060901199793)*x[0]+(-0.334248974895)*x[1]**o+(-0.872695915274)*x[1]+(0.246713393819)*x[2]**o+(-0.819057815971)*x[2]
        s[0,1,2,0]=(0.968772211014)*x[0]**o+(0.71842819735)*x[0]+(-0.0772702390821)*x[1]**o+(-0.338266585527)*x[1]+(-0.506850093136)*x[2]**o+(0.707040138867)*x[2]
        s[0,1,2,1]=(-0.804147712129)*x[0]**o+(0.48962140515)*x[0]+(-0.247303878423)*x[1]**o+(0.889311125721)*x[1]+(0.457530011837)*x[2]**o+(-0.721486338348)*x[2]
        s[0,2,0,0]=(0.140474174736)*x[0]**o+(-0.0453977330632)*x[0]+(0.9083497372)*x[1]**o+(-0.719356277596)*x[1]+(0.72748037113)*x[2]**o+(-0.0518197409882)*x[2]
        s[0,2,0,1]=(-0.0758350230998)*x[0]**o+(-0.367910921003)*x[0]+(-0.118137700915)*x[1]**o+(0.620139843753)*x[1]+(0.648111986033)*x[2]**o+(-0.250478603635)*x[2]
        s[0,2,1,0]=(0.128776260898)*x[0]**o+(0.772839138201)*x[0]+(0.667219128346)*x[1]**o+(-0.291604907274)*x[1]+(0.976574302911)*x[2]**o+(-0.60715514616)*x[2]
        s[0,2,1,1]=(-0.85960053799)*x[0]**o+(0.421741754709)*x[0]+(0.137156504307)*x[1]**o+(0.31044942099)*x[1]+(-0.0607312333184)*x[2]**o+(-0.371068928897)*x[2]
        s[0,2,2,0]=(0.703498831008)*x[0]**o+(0.351119263938)*x[0]+(0.0238555242548)*x[1]**o+(0.709263994991)*x[1]+(-0.822927451312)*x[2]**o+(0.289556375179)*x[2]
        s[0,2,2,1]=(-0.285309117139)*x[0]**o+(0.13890400764)*x[0]+(-0.604129730872)*x[1]**o+(-0.0821311416855)*x[1]+(-0.731018609109)*x[2]**o+(0.529525274631)*x[2]
        s[0,3,0,0]=(0.641292392051)*x[0]**o+(0.41865118466)*x[0]+(0.963763232952)*x[1]**o+(0.290032616399)*x[1]+(-0.506676751351)*x[2]**o+(-0.895169772311)*x[2]
        s[0,3,0,1]=(-0.616101594244)*x[0]**o+(0.791759779662)*x[0]+(0.690427410929)*x[1]**o+(0.970117852706)*x[1]+(-0.813314826297)*x[2]**o+(0.637157627363)*x[2]
        s[0,3,1,0]=(0.973134693253)*x[0]**o+(-0.294894425912)*x[0]+(-0.342056908291)*x[1]**o+(-0.569749335695)*x[1]+(0.402640595819)*x[2]**o+(0.132057879818)*x[2]
        s[0,3,1,1]=(-0.785989986755)*x[0]**o+(-0.319383158643)*x[0]+(0.875111156622)*x[1]**o+(0.607783276248)*x[1]+(0.522924160022)*x[2]**o+(-0.963598048032)*x[2]
        s[0,3,2,0]=(0.367812735951)*x[0]**o+(-0.687977977398)*x[0]+(-0.787244484503)*x[1]**o+(0.240728636869)*x[1]+(-0.974034052698)*x[2]**o+(-0.810505857708)*x[2]
        s[0,3,2,1]=(-0.661827774609)*x[0]**o+(0.569720489249)*x[0]+(-0.0944794378735)*x[1]**o+(-0.793223846926)*x[1]+(0.54128448366)*x[2]**o+(-0.893409198606)*x[2]
        s[0,4,0,0]=(0.259245470871)*x[0]**o+(-0.226239257386)*x[0]+(-0.308196311787)*x[1]**o+(-0.23628967322)*x[1]+(0.547994188289)*x[2]**o+(-0.671911794405)*x[2]
        s[0,4,0,1]=(-0.00101716691769)*x[0]**o+(-0.509353224753)*x[0]+(0.592369934948)*x[1]**o+(-0.632019867229)*x[1]+(-0.890240436074)*x[2]**o+(0.895675476663)*x[2]
        s[0,4,1,0]=(0.960337962272)*x[0]**o+(0.555794125077)*x[0]+(0.993174576462)*x[1]**o+(-0.772336499274)*x[1]+(0.0103297090652)*x[2]**o+(0.118606584111)*x[2]
        s[0,4,1,1]=(0.0420511106146)*x[0]**o+(-0.758220545738)*x[0]+(-0.119140693961)*x[1]**o+(0.0775988947583)*x[1]+(0.828731522222)*x[2]**o+(-0.762715251326)*x[2]
        s[0,4,2,0]=(0.549182098853)*x[0]**o+(0.36156976435)*x[0]+(-0.152823474029)*x[1]**o+(-0.315519239615)*x[1]+(0.577553409367)*x[2]**o+(0.467722402358)*x[2]
        s[0,4,2,1]=(0.117551959782)*x[0]**o+(0.850326589959)*x[0]+(0.881226923201)*x[1]**o+(-0.659482296116)*x[1]+(0.618324879901)*x[2]**o+(0.720404333688)*x[2]
        s[1,0,0,0]=(-0.761147885924)*x[0]**o+(0.802435468296)*x[0]+(0.794658337218)*x[1]**o+(0.0458609096746)*x[1]+(0.603568994654)*x[2]**o+(-0.543591666298)*x[2]
        s[1,0,0,1]=(-0.568265516859)*x[0]**o+(-0.217464502958)*x[0]+(0.945091213325)*x[1]**o+(0.701546510287)*x[1]+(0.218015344978)*x[2]**o+(-0.296437711896)*x[2]
        s[1,0,1,0]=(0.131167263333)*x[0]**o+(0.443503731963)*x[0]+(0.936129884208)*x[1]**o+(0.731105012686)*x[1]+(0.886062834899)*x[2]**o+(0.949880973913)*x[2]
        s[1,0,1,1]=(-0.324713526984)*x[0]**o+(0.525830634904)*x[0]+(0.258554314238)*x[1]**o+(-0.206218826462)*x[1]+(0.945107982827)*x[2]**o+(0.301738246936)*x[2]
        s[1,0,2,0]=(-0.0983483498843)*x[0]**o+(-0.107666500098)*x[0]+(0.572563468846)*x[1]**o+(-0.877027769074)*x[1]+(0.0499191410048)*x[2]**o+(0.78320557846)*x[2]
        s[1,0,2,1]=(-0.0549251955703)*x[0]**o+(-0.684098254591)*x[0]+(0.974543898137)*x[1]**o+(0.572617020679)*x[1]+(-0.745704992018)*x[2]**o+(0.694741262801)*x[2]
        s[1,1,0,0]=(0.140734267352)*x[0]**o+(0.684555621818)*x[0]+(-0.988693128074)*x[1]**o+(0.663988019213)*x[1]+(0.207716488749)*x[2]**o+(-0.218440531714)*x[2]
        s[1,1,0,1]=(0.906718149076)*x[0]**o+(-0.823295142671)*x[0]+(0.599921015401)*x[1]**o+(0.712671366397)*x[1]+(0.560173956846)*x[2]**o+(-0.583700172601)*x[2]
        s[1,1,1,0]=(0.412161237427)*x[0]**o+(0.449013323323)*x[0]+(-0.422313884483)*x[1]**o+(-0.856352846316)*x[1]+(-0.131018950683)*x[2]**o+(-0.113127977867)*x[2]
        s[1,1,1,1]=(0.342974599199)*x[0]**o+(0.918991859784)*x[0]+(-0.380736598315)*x[1]**o+(-0.844001076048)*x[1]+(0.103584145435)*x[2]**o+(-0.0224438802577)*x[2]
        s[1,1,2,0]=(0.405298238411)*x[0]**o+(-0.0260196814125)*x[0]+(0.192953354082)*x[1]**o+(-0.768884750367)*x[1]+(-0.898989048123)*x[2]**o+(-0.625718514351)*x[2]
        s[1,1,2,1]=(-0.679709121137)*x[0]**o+(0.829310202601)*x[0]+(0.488556797655)*x[1]**o+(-0.546577821454)*x[1]+(0.215139403299)*x[2]**o+(-0.597837941999)*x[2]
        s[1,2,0,0]=(-0.763765181249)*x[0]**o+(0.471498461874)*x[0]+(-0.0657374404825)*x[1]**o+(-0.990465192866)*x[1]+(0.755407325384)*x[2]**o+(-0.725455335361)*x[2]
        s[1,2,0,1]=(-0.707293534802)*x[0]**o+(0.741579146085)*x[0]+(-0.827571233981)*x[1]**o+(-0.924507469851)*x[1]+(-0.11981466602)*x[2]**o+(-0.227349968773)*x[2]
        s[1,2,1,0]=(0.02761949765)*x[0]**o+(0.23242129234)*x[0]+(-0.744337789362)*x[1]**o+(-0.69986289257)*x[1]+(0.284991295889)*x[2]**o+(-0.82971704627)*x[2]
        s[1,2,1,1]=(-0.399349607364)*x[0]**o+(0.681805497713)*x[0]+(0.50892365528)*x[1]**o+(-0.24206992271)*x[1]+(-0.864035697056)*x[2]**o+(0.311678612996)*x[2]
        s[1,2,2,0]=(0.900552977578)*x[0]**o+(0.352782843752)*x[0]+(0.343335797615)*x[1]**o+(0.12459367011)*x[1]+(-0.78722966105)*x[2]**o+(0.0289887373712)*x[2]
        s[1,2,2,1]=(-0.231556056016)*x[0]**o+(-0.454277914592)*x[0]+(-0.723413876928)*x[1]**o+(-0.167712035756)*x[1]+(0.297646282013)*x[2]**o+(0.142123612721)*x[2]
        s[1,3,0,0]=(-0.753062984495)*x[0]**o+(0.834144701924)*x[0]+(0.198694009378)*x[1]**o+(-0.522412387399)*x[1]+(-0.823293930133)*x[2]**o+(-0.875699299314)*x[2]
        s[1,3,0,1]=(0.810523164741)*x[0]**o+(-0.363171422715)*x[0]+(-0.0125819806617)*x[1]**o+(-0.461550939044)*x[1]+(-0.0610628843793)*x[2]**o+(0.792146078844)*x[2]
        s[1,3,1,0]=(-0.435427701923)*x[0]**o+(0.0485485027059)*x[0]+(-0.533976468056)*x[1]**o+(0.907837942847)*x[1]+(-0.70993588953)*x[2]**o+(0.707193663015)*x[2]
        s[1,3,1,1]=(-0.647252848948)*x[0]**o+(0.889142781795)*x[0]+(-0.625573458304)*x[1]**o+(0.40306299061)*x[1]+(0.0918220173097)*x[2]**o+(0.563306928169)*x[2]
        s[1,3,2,0]=(-0.607331889118)*x[0]**o+(0.812730322666)*x[0]+(0.555953335767)*x[1]**o+(0.661784189166)*x[1]+(0.391287509324)*x[2]**o+(-0.916911048454)*x[2]
        s[1,3,2,1]=(-0.484392978469)*x[0]**o+(-0.541102106822)*x[0]+(-0.245428871935)*x[1]**o+(0.278664918051)*x[1]+(-0.955672198176)*x[2]**o+(-0.548226453411)*x[2]
        s[1,4,0,0]=(0.503702684537)*x[0]**o+(0.233988182801)*x[0]+(-0.093833060394)*x[1]**o+(-0.167779518637)*x[1]+(-0.224463708091)*x[2]**o+(-0.710844655028)*x[2]
        s[1,4,0,1]=(0.255645379803)*x[0]**o+(-0.319588660935)*x[0]+(-0.156476801364)*x[1]**o+(0.764377089178)*x[1]+(0.766935604368)*x[2]**o+(-0.157460600111)*x[2]
        s[1,4,1,0]=(0.15831529726)*x[0]**o+(-0.922418423102)*x[0]+(0.561598779551)*x[1]**o+(-0.745598205601)*x[1]+(-0.694388851252)*x[2]**o+(-0.247051354544)*x[2]
        s[1,4,1,1]=(-0.547080506797)*x[0]**o+(-0.814888293172)*x[0]+(0.122207354764)*x[1]**o+(-0.489157382765)*x[1]+(-0.165490463352)*x[2]**o+(-0.114446229543)*x[2]
        s[1,4,2,0]=(-0.369250635169)*x[0]**o+(0.423782722049)*x[0]+(0.968680330397)*x[1]**o+(0.28815964147)*x[1]+(-0.143666599619)*x[2]**o+(0.499469160546)*x[2]
        s[1,4,2,1]=(0.843010459592)*x[0]**o+(0.676625633805)*x[0]+(-0.635607829653)*x[1]**o+(-0.760424296046)*x[1]+(0.457005234774)*x[2]**o+(-0.102642224679)*x[2]
        s[2,0,0,0]=(-0.416783290665)*x[0]**o+(-0.0326686788419)*x[0]+(-0.144394865356)*x[1]**o+(-0.911119053945)*x[1]+(-0.55810658213)*x[2]**o+(0.40998976988)*x[2]
        s[2,0,0,1]=(0.513962651738)*x[0]**o+(-0.998965858933)*x[0]+(0.917310302297)*x[1]**o+(-0.92170256086)*x[1]+(-0.832148513501)*x[2]**o+(-0.551050270978)*x[2]
        s[2,0,1,0]=(0.431543077341)*x[0]**o+(0.042920807321)*x[0]+(-0.143174202064)*x[1]**o+(0.90359005289)*x[1]+(-0.732368795811)*x[2]**o+(-0.0689734110721)*x[2]
        s[2,0,1,1]=(-0.167337582492)*x[0]**o+(-0.685847720118)*x[0]+(-0.574599145898)*x[1]**o+(0.685408949158)*x[1]+(0.583748780479)*x[2]**o+(0.494510625069)*x[2]
        s[2,0,2,0]=(0.76364604242)*x[0]**o+(-0.432951084836)*x[0]+(0.425205211657)*x[1]**o+(-0.879037059825)*x[1]+(-0.013713147532)*x[2]**o+(0.569703973439)*x[2]
        s[2,0,2,1]=(-0.656200752037)*x[0]**o+(0.0808097604648)*x[0]+(0.503360202878)*x[1]**o+(0.487461964251)*x[1]+(-0.405721164154)*x[2]**o+(0.126364295441)*x[2]
        s[2,1,0,0]=(-0.168286367707)*x[0]**o+(0.497189264604)*x[0]+(0.309898214306)*x[1]**o+(-0.51880727488)*x[1]+(0.518790399303)*x[2]**o+(0.401150188648)*x[2]
        s[2,1,0,1]=(0.573088756891)*x[0]**o+(-0.528498236574)*x[0]+(-0.866435772203)*x[1]**o+(-0.0338712475717)*x[1]+(0.869338257704)*x[2]**o+(0.276166855245)*x[2]
        s[2,1,1,0]=(-0.200994668199)*x[0]**o+(-0.227638621809)*x[0]+(-0.243755130686)*x[1]**o+(0.983124311993)*x[1]+(0.834307124674)*x[2]**o+(0.191048093109)*x[2]
        s[2,1,1,1]=(0.423677882618)*x[0]**o+(0.451735240848)*x[0]+(0.417686763179)*x[1]**o+(0.766280670471)*x[1]+(-0.829023066103)*x[2]**o+(-0.780956171089)*x[2]
        s[2,1,2,0]=(0.640928997042)*x[0]**o+(0.671670205561)*x[0]+(0.14746453586)*x[1]**o+(-0.744942640395)*x[1]+(0.936412384681)*x[2]**o+(-0.797412536669)*x[2]
        s[2,1,2,1]=(-0.730418336064)*x[0]**o+(-0.912540120589)*x[0]+(0.31782754852)*x[1]**o+(-0.476487014675)*x[1]+(-0.166085203325)*x[2]**o+(-0.592216218131)*x[2]
        s[2,2,0,0]=(0.373430973034)*x[0]**o+(-0.569005417932)*x[0]+(0.33244882787)*x[1]**o+(-0.348297838584)*x[1]+(-0.903746783477)*x[2]**o+(0.976129823909)*x[2]
        s[2,2,0,1]=(-0.384047362737)*x[0]**o+(-0.268433421634)*x[0]+(-0.838287069736)*x[1]**o+(0.537590839064)*x[1]+(-0.871855225879)*x[2]**o+(0.772511342839)*x[2]
        s[2,2,1,0]=(-0.978119189005)*x[0]**o+(-0.121978949728)*x[0]+(-0.329862921919)*x[1]**o+(-0.709859714296)*x[1]+(0.330028081384)*x[2]**o+(-0.0970288297589)*x[2]
        s[2,2,1,1]=(-0.666661297789)*x[0]**o+(-0.480759645285)*x[0]+(-0.344863683771)*x[1]**o+(0.385174072898)*x[1]+(-0.40458447828)*x[2]**o+(0.81018598482)*x[2]
        s[2,2,2,0]=(-0.419553660247)*x[0]**o+(-0.96154803369)*x[0]+(-0.979914830867)*x[1]**o+(0.802474237871)*x[1]+(-0.273952110245)*x[2]**o+(-0.273921739584)*x[2]
        s[2,2,2,1]=(0.204828260085)*x[0]**o+(-0.207525741758)*x[0]+(-0.157511344538)*x[1]**o+(-0.242922426999)*x[1]+(0.750045639114)*x[2]**o+(0.625828940704)*x[2]
        s[2,3,0,0]=(-0.433426692328)*x[0]**o+(0.546111969738)*x[0]+(0.380856694436)*x[1]**o+(-0.33958919138)*x[1]+(-0.612713564248)*x[2]**o+(-0.468136706051)*x[2]
        s[2,3,0,1]=(-0.753353152037)*x[0]**o+(0.898355698117)*x[0]+(-0.748796477533)*x[1]**o+(-0.84014959853)*x[1]+(0.00222130390134)*x[2]**o+(0.892747426493)*x[2]
        s[2,3,1,0]=(-0.770467840127)*x[0]**o+(-0.365528437394)*x[0]+(0.823481948409)*x[1]**o+(0.688841559491)*x[1]+(0.399335743364)*x[2]**o+(-0.11069734561)*x[2]
        s[2,3,1,1]=(-0.490969508319)*x[0]**o+(0.516182950326)*x[0]+(-0.729563968179)*x[1]**o+(-0.738386155156)*x[1]+(0.646183209813)*x[2]**o+(0.319822004437)*x[2]
        s[2,3,2,0]=(-0.719374773896)*x[0]**o+(0.227265715278)*x[0]+(-0.378942759956)*x[1]**o+(-0.0227600257348)*x[1]+(-0.319915586734)*x[2]**o+(0.901101476715)*x[2]
        s[2,3,2,1]=(-0.514878672798)*x[0]**o+(-0.496279665164)*x[0]+(0.301404632426)*x[1]**o+(0.408945490654)*x[1]+(0.993826467678)*x[2]**o+(0.466263681916)*x[2]
        s[2,4,0,0]=(0.163562057356)*x[0]**o+(0.653997800691)*x[0]+(0.0851835561474)*x[1]**o+(0.940843692093)*x[1]+(0.047617990547)*x[2]**o+(-0.859043423533)*x[2]
        s[2,4,0,1]=(0.634549693325)*x[0]**o+(0.914675769662)*x[0]+(-0.172241120744)*x[1]**o+(0.534043363834)*x[1]+(-0.655408958703)*x[2]**o+(0.688624822376)*x[2]
        s[2,4,1,0]=(0.330425495069)*x[0]**o+(0.686177305834)*x[0]+(0.752515146248)*x[1]**o+(-0.552696418525)*x[1]+(-0.955945706288)*x[2]**o+(0.470133484592)*x[2]
        s[2,4,1,1]=(0.106341453568)*x[0]**o+(-0.531668585129)*x[0]+(-0.520032286959)*x[1]**o+(-0.738038080149)*x[1]+(0.525162627468)*x[2]**o+(-0.655104090288)*x[2]
        s[2,4,2,0]=(0.994443804198)*x[0]**o+(0.510472126015)*x[0]+(0.902454289513)*x[1]**o+(-0.191032483827)*x[1]+(-0.871408191409)*x[2]**o+(0.547124971054)*x[2]
        s[2,4,2,1]=(0.113561397179)*x[0]**o+(-0.505522632919)*x[0]+(0.30818367646)*x[1]**o+(-0.00465438758811)*x[1]+(0.594009196981)*x[2]**o+(-0.226939247299)*x[2]
        s[3,0,0,0]=(-0.463635004027)*x[0]**o+(-0.473760265395)*x[0]+(0.878709353532)*x[1]**o+(0.0947459582975)*x[1]+(0.0219625500089)*x[2]**o+(0.784199705647)*x[2]
        s[3,0,0,1]=(0.676123626258)*x[0]**o+(-0.718629965972)*x[0]+(0.955205674618)*x[1]**o+(0.706290464564)*x[1]+(-0.604943464128)*x[2]**o+(0.694623387198)*x[2]
        s[3,0,1,0]=(-0.304361833494)*x[0]**o+(-0.658618417297)*x[0]+(0.171679811142)*x[1]**o+(0.995668711479)*x[1]+(-0.0331315333318)*x[2]**o+(-0.425870682329)*x[2]
        s[3,0,1,1]=(0.307231747905)*x[0]**o+(0.0052926672705)*x[0]+(-0.0603771488434)*x[1]**o+(0.201207904112)*x[1]+(0.769253259019)*x[2]**o+(0.809891168107)*x[2]
        s[3,0,2,0]=(-0.130591836024)*x[0]**o+(-0.0929104424471)*x[0]+(0.702701120328)*x[1]**o+(0.79105646679)*x[1]+(0.279504582566)*x[2]**o+(-0.0685634410467)*x[2]
        s[3,0,2,1]=(-0.576303721224)*x[0]**o+(0.996201825902)*x[0]+(0.415602469244)*x[1]**o+(-0.51130556243)*x[1]+(0.924720232267)*x[2]**o+(-0.653996834451)*x[2]
        s[3,1,0,0]=(-0.798641869619)*x[0]**o+(-0.474090853446)*x[0]+(-0.597109245363)*x[1]**o+(0.441151246969)*x[1]+(-0.460726722134)*x[2]**o+(0.129538461429)*x[2]
        s[3,1,0,1]=(-0.00692013378255)*x[0]**o+(0.12877226476)*x[0]+(-0.357533404018)*x[1]**o+(-0.389134623739)*x[1]+(0.0387875396629)*x[2]**o+(-0.81001355285)*x[2]
        s[3,1,1,0]=(0.975364017255)*x[0]**o+(0.0184628364974)*x[0]+(0.738788422118)*x[1]**o+(-0.897723661125)*x[1]+(0.342855398212)*x[2]**o+(0.766278709935)*x[2]
        s[3,1,1,1]=(0.103423718636)*x[0]**o+(0.264183054502)*x[0]+(-0.413231981251)*x[1]**o+(0.50816929065)*x[1]+(0.717301020458)*x[2]**o+(0.863858366933)*x[2]
        s[3,1,2,0]=(0.0547068997542)*x[0]**o+(-0.0697197526742)*x[0]+(0.560106582056)*x[1]**o+(0.914213149957)*x[1]+(-0.357975004563)*x[2]**o+(0.138675813902)*x[2]
        s[3,1,2,1]=(0.236730250658)*x[0]**o+(0.835033750765)*x[0]+(-0.408412262014)*x[1]**o+(0.707762870875)*x[1]+(-0.381638364848)*x[2]**o+(-0.237912635268)*x[2]
        s[3,2,0,0]=(0.388806525405)*x[0]**o+(0.434371077714)*x[0]+(-0.637939265061)*x[1]**o+(0.869903625445)*x[1]+(0.751102627943)*x[2]**o+(-0.591787739898)*x[2]
        s[3,2,0,1]=(-0.328460151904)*x[0]**o+(-0.469089195258)*x[0]+(0.189752113153)*x[1]**o+(-0.223739735753)*x[1]+(0.90243464478)*x[2]**o+(0.393444417035)*x[2]
        s[3,2,1,0]=(-0.350954406466)*x[0]**o+(0.222756398721)*x[0]+(0.881035791552)*x[1]**o+(-0.601290867644)*x[1]+(-0.845027376811)*x[2]**o+(-0.658344721797)*x[2]
        s[3,2,1,1]=(-0.804545424944)*x[0]**o+(0.723106876602)*x[0]+(-0.710610485987)*x[1]**o+(0.616671752141)*x[1]+(0.255075132475)*x[2]**o+(-0.344194652837)*x[2]
        s[3,2,2,0]=(0.867664525177)*x[0]**o+(-0.584510407392)*x[0]+(-0.50466882786)*x[1]**o+(-0.976134817563)*x[1]+(-0.0741803228365)*x[2]**o+(-0.816773953701)*x[2]
        s[3,2,2,1]=(0.997671019163)*x[0]**o+(-0.0614667721506)*x[0]+(-0.978519383696)*x[1]**o+(0.0645671450044)*x[1]+(0.465596917785)*x[2]**o+(-0.279520773991)*x[2]
        s[3,3,0,0]=(-0.564960743631)*x[0]**o+(0.923212123601)*x[0]+(0.42281995555)*x[1]**o+(-0.701296001463)*x[1]+(0.791187577221)*x[2]**o+(-0.665209151969)*x[2]
        s[3,3,0,1]=(0.872459084593)*x[0]**o+(-0.14974585293)*x[0]+(0.257403669429)*x[1]**o+(-0.236794230318)*x[1]+(-0.458881609061)*x[2]**o+(0.614204776708)*x[2]
        s[3,3,1,0]=(-0.256375661857)*x[0]**o+(-0.0451909988296)*x[0]+(-0.38620807221)*x[1]**o+(0.905631567256)*x[1]+(-0.293636725178)*x[2]**o+(-0.522921339974)*x[2]
        s[3,3,1,1]=(-0.649399488997)*x[0]**o+(-0.92131361011)*x[0]+(0.228306527968)*x[1]**o+(0.914959459994)*x[1]+(-0.689881365519)*x[2]**o+(0.769874183342)*x[2]
        s[3,3,2,0]=(0.237419242359)*x[0]**o+(-0.578361971709)*x[0]+(-0.63121773371)*x[1]**o+(0.446149460264)*x[1]+(-0.975635743387)*x[2]**o+(-0.0926780614368)*x[2]
        s[3,3,2,1]=(0.898298006002)*x[0]**o+(0.738210632094)*x[0]+(0.954104559567)*x[1]**o+(-0.0586956189155)*x[1]+(-0.197544744403)*x[2]**o+(-0.458321606749)*x[2]
        s[3,4,0,0]=(0.152469131812)*x[0]**o+(0.636503984792)*x[0]+(0.634624949432)*x[1]**o+(-0.191145041451)*x[1]+(-0.586391727404)*x[2]**o+(0.470501756725)*x[2]
        s[3,4,0,1]=(-0.902280446763)*x[0]**o+(0.276584347986)*x[0]+(0.627059019624)*x[1]**o+(0.256966014366)*x[1]+(0.993138399883)*x[2]**o+(0.444820746069)*x[2]
        s[3,4,1,0]=(0.226692588742)*x[0]**o+(-0.462068747651)*x[0]+(0.243555291569)*x[1]**o+(-0.913735726033)*x[1]+(0.149578352207)*x[2]**o+(-0.330136710272)*x[2]
        s[3,4,1,1]=(0.99435141349)*x[0]**o+(0.0799718813232)*x[0]+(0.191619055132)*x[1]**o+(0.85076051922)*x[1]+(0.111360148299)*x[2]**o+(-0.683082083045)*x[2]
        s[3,4,2,0]=(-0.574679864968)*x[0]**o+(-0.836650014193)*x[0]+(-0.548132149017)*x[1]**o+(-0.255129120741)*x[1]+(0.639700961749)*x[2]**o+(0.932732223318)*x[2]
        s[3,4,2,1]=(0.670125652473)*x[0]**o+(-0.717633581197)*x[0]+(-0.908717470386)*x[1]**o+(0.420401361995)*x[1]+(-0.93231135796)*x[2]**o+(0.820828865038)*x[2]
        ref[0,0,0,0]=(1.57822683218)/(o+1.)+(0.434213572336)+(-0.210183765573)*0.5**o
        ref[0,0,0,1]=(-0.538732975497)/(o+1.)+(-0.220483571283)+(-0.323968020416)*0.5**o
        ref[0,0,1,0]=(-1.11824914154)/(o+1.)+(-0.416070565632)+(0.467518636672)*0.5**o
        ref[0,0,1,1]=(-0.247957266768)/(o+1.)+(0.249286503966)+(0.976876531867)*0.5**o
        ref[0,0,2,0]=(0.730696883011)/(o+1.)+(-0.13882986572)+(-0.552336862133)*0.5**o
        ref[0,0,2,1]=(-0.70462602243)/(o+1.)+(-0.20254736628)+(0.110734087997)*0.5**o
        ref[0,1,0,0]=(-1.29885017752)/(o+1.)+(-0.366291903889)+(0.979769828626)*0.5**o
        ref[0,1,0,1]=(0.966474712198)/(o+1.)+(0.377138900819)+(0.184115163182)*0.5**o
        ref[0,1,1,0]=(-1.62365238422)/(o+1.)+(-0.637241358728)+(0.0822751335539)*0.5**o
        ref[0,1,1,1]=(-0.0875355810752)/(o+1.)+(-0.876327465519)+(0.672877076993)*0.5**o
        ref[0,1,2,0]=(-0.584120332218)/(o+1.)+(0.543600875345)+(0.968772211014)*0.5**o
        ref[0,1,2,1]=(0.210226133414)/(o+1.)+(0.328723096261)+(-0.804147712129)*0.5**o
        ref[0,2,0,0]=(1.63583010833)/(o+1.)+(-0.408286875823)+(0.140474174736)*0.5**o
        ref[0,2,0,1]=(0.529974285118)/(o+1.)+(0.000875159557622)+(-0.0758350230998)*0.5**o
        ref[0,2,1,0]=(1.64379343126)/(o+1.)+(-0.0629604576161)+(0.128776260898)*0.5**o
        ref[0,2,1,1]=(0.0764252709887)/(o+1.)+(0.180561123401)+(-0.85960053799)*0.5**o
        ref[0,2,2,0]=(-0.799071927058)/(o+1.)+(0.674969817054)+(0.703498831008)*0.5**o
        ref[0,2,2,1]=(-1.33514833998)/(o+1.)+(0.293149070293)+(-0.285309117139)*0.5**o
        ref[0,3,0,0]=(0.457086481601)/(o+1.)+(-0.0932429856258)+(0.641292392051)*0.5**o
        ref[0,3,0,1]=(-0.122887415369)/(o+1.)+(1.19951762987)+(-0.616101594244)*0.5**o
        ref[0,3,1,0]=(0.0605836875282)/(o+1.)+(-0.366292940894)+(0.973134693253)*0.5**o
        ref[0,3,1,1]=(1.39803531664)/(o+1.)+(-0.337598965213)+(-0.785989986755)*0.5**o
        ref[0,3,2,0]=(-1.7612785372)/(o+1.)+(-0.628877599118)+(0.367812735951)*0.5**o
        ref[0,3,2,1]=(0.446805045787)/(o+1.)+(-0.558456278141)+(-0.661827774609)*0.5**o
        ref[0,4,0,0]=(0.239797876502)/(o+1.)+(-0.567220362506)+(0.259245470871)*0.5**o
        ref[0,4,0,1]=(-0.297870501125)/(o+1.)+(-0.12284880766)+(-0.00101716691769)*0.5**o
        ref[0,4,1,0]=(1.00350428553)/(o+1.)+(-0.0489678950431)+(0.960337962272)*0.5**o
        ref[0,4,1,1]=(0.709590828261)/(o+1.)+(-0.721668451153)+(0.0420511106146)*0.5**o
        ref[0,4,2,0]=(0.424729935338)/(o+1.)+(0.256886463547)+(0.549182098853)*0.5**o
        ref[0,4,2,1]=(1.4995518031)/(o+1.)+(0.455624313765)+(0.117551959782)*0.5**o
        ref[1,0,0,0]=(1.39822733187)/(o+1.)+(0.152352355836)+(-0.761147885924)*0.5**o
        ref[1,0,0,1]=(1.1631065583)/(o+1.)+(0.0938221477164)+(-0.568265516859)*0.5**o
        ref[1,0,1,0]=(1.82219271911)/(o+1.)+(1.06224485928)+(0.131167263333)*0.5**o
        ref[1,0,1,1]=(1.20366229707)/(o+1.)+(0.310675027689)+(-0.324713526984)*0.5**o
        ref[1,0,2,0]=(0.622482609851)/(o+1.)+(-0.100744345356)+(-0.0983483498843)*0.5**o
        ref[1,0,2,1]=(0.228838906118)/(o+1.)+(0.291630014445)+(-0.0549251955703)*0.5**o
        ref[1,1,0,0]=(-0.780976639325)/(o+1.)+(0.565051554659)+(0.140734267352)*0.5**o
        ref[1,1,0,1]=(1.16009497225)/(o+1.)+(-0.347161974437)+(0.906718149076)*0.5**o
        ref[1,1,1,0]=(-0.553332835166)/(o+1.)+(-0.26023375043)+(0.412161237427)*0.5**o
        ref[1,1,1,1]=(-0.277152452881)/(o+1.)+(0.026273451739)+(0.342974599199)*0.5**o
        ref[1,1,2,0]=(-0.706035694041)/(o+1.)+(-0.710311473065)+(0.405298238411)*0.5**o
        ref[1,1,2,1]=(0.703696200954)/(o+1.)+(-0.157552780426)+(-0.679709121137)*0.5**o
        ref[1,2,0,0]=(0.689669884901)/(o+1.)+(-0.622211033177)+(-0.763765181249)*0.5**o
        ref[1,2,0,1]=(-0.947385900002)/(o+1.)+(-0.20513914627)+(-0.707293534802)*0.5**o
        ref[1,2,1,0]=(-0.459346493473)/(o+1.)+(-0.64857932325)+(0.02761949765)*0.5**o
        ref[1,2,1,1]=(-0.355112041776)/(o+1.)+(0.375707094)+(-0.399349607364)*0.5**o
        ref[1,2,2,0]=(-0.443893863435)/(o+1.)+(0.253182625616)+(0.900552977578)*0.5**o
        ref[1,2,2,1]=(-0.425767594915)/(o+1.)+(-0.239933168814)+(-0.231556056016)*0.5**o
        ref[1,3,0,0]=(-0.624599920754)/(o+1.)+(-0.281983492394)+(-0.753062984495)*0.5**o
        ref[1,3,0,1]=(-0.073644865041)/(o+1.)+(-0.0162881414573)+(0.810523164741)*0.5**o
        ref[1,3,1,0]=(-1.24391235759)/(o+1.)+(0.831790054284)+(-0.435427701923)*0.5**o
        ref[1,3,1,1]=(-0.533751440995)/(o+1.)+(0.927756350287)+(-0.647252848948)*0.5**o
        ref[1,3,2,0]=(0.94724084509)/(o+1.)+(0.278801731688)+(-0.607331889118)*0.5**o
        ref[1,3,2,1]=(-1.20110107011)/(o+1.)+(-0.405331821091)+(-0.484392978469)*0.5**o
        ref[1,4,0,0]=(-0.318296768485)/(o+1.)+(-0.322317995432)+(0.503702684537)*0.5**o
        ref[1,4,0,1]=(0.610458803004)/(o+1.)+(0.143663914066)+(0.255645379803)*0.5**o
        ref[1,4,1,0]=(-0.1327900717)/(o+1.)+(-0.957533991624)+(0.15831529726)*0.5**o
        ref[1,4,1,1]=(-0.0432831085882)/(o+1.)+(-0.70924595274)+(-0.547080506797)*0.5**o
        ref[1,4,2,0]=(0.825013730779)/(o+1.)+(0.605705762033)+(-0.369250635169)*0.5**o
        ref[1,4,2,1]=(-0.17860259488)/(o+1.)+(-0.09322044346)+(0.843010459592)*0.5**o
        ref[2,0,0,0]=(-0.702501447486)/(o+1.)+(-0.266898981454)+(-0.416783290665)*0.5**o
        ref[2,0,0,1]=(0.0851617887959)/(o+1.)+(-1.23585934539)+(0.513962651738)*0.5**o
        ref[2,0,1,0]=(-0.875542997875)/(o+1.)+(0.43876872457)+(0.431543077341)*0.5**o
        ref[2,0,1,1]=(0.00914963458101)/(o+1.)+(0.247035927055)+(-0.167337582492)*0.5**o
        ref[2,0,2,0]=(0.411492064125)/(o+1.)+(-0.371142085611)+(0.76364604242)*0.5**o
        ref[2,0,2,1]=(0.0976390387239)/(o+1.)+(0.347318010078)+(-0.656200752037)*0.5**o
        ref[2,1,0,0]=(0.828688613609)/(o+1.)+(0.189766089186)+(-0.168286367707)*0.5**o
        ref[2,1,0,1]=(0.00290248550037)/(o+1.)+(-0.143101314451)+(0.573088756891)*0.5**o
        ref[2,1,1,0]=(0.590551993988)/(o+1.)+(0.473266891646)+(-0.200994668199)*0.5**o
        ref[2,1,1,1]=(-0.411336302924)/(o+1.)+(0.218529870115)+(0.423677882618)*0.5**o
        ref[2,1,2,0]=(1.08387692054)/(o+1.)+(-0.435342485751)+(0.640928997042)*0.5**o
        ref[2,1,2,1]=(0.151742345195)/(o+1.)+(-0.990621676697)+(-0.730418336064)*0.5**o
        ref[2,2,0,0]=(-0.571297955608)/(o+1.)+(0.0294132836968)+(0.373430973034)*0.5**o
        ref[2,2,0,1]=(-1.71014229561)/(o+1.)+(0.520834380134)+(-0.384047362737)*0.5**o
        ref[2,2,1,0]=(0.000165159465638)/(o+1.)+(-0.464433746891)+(-0.978119189005)*0.5**o
        ref[2,2,1,1]=(-0.749448162051)/(o+1.)+(0.357300206217)+(-0.666661297789)*0.5**o
        ref[2,2,2,0]=(-1.25386694111)/(o+1.)+(-0.216497767701)+(-0.419553660247)*0.5**o
        ref[2,2,2,1]=(0.592534294577)/(o+1.)+(0.0876903859733)+(0.204828260085)*0.5**o
        ref[2,3,0,0]=(-0.231856869811)/(o+1.)+(-0.130806963846)+(-0.433426692328)*0.5**o
        ref[2,3,0,1]=(-0.746575173632)/(o+1.)+(0.47547676304)+(-0.753353152037)*0.5**o
        ref[2,3,1,0]=(1.22281769177)/(o+1.)+(0.106307888243)+(-0.770467840127)*0.5**o
        ref[2,3,1,1]=(-0.0833807583658)/(o+1.)+(0.0488093998039)+(-0.490969508319)*0.5**o
        ref[2,3,2,0]=(-0.69885834669)/(o+1.)+(0.552803583129)+(-0.719374773896)*0.5**o
        ref[2,3,2,1]=(1.2952311001)/(o+1.)+(0.189464753703)+(-0.514878672798)*0.5**o
        ref[2,4,0,0]=(0.132801546694)/(o+1.)+(0.367899034626)+(0.163562057356)*0.5**o
        ref[2,4,0,1]=(-0.827650079446)/(o+1.)+(1.06867197794)+(0.634549693325)*0.5**o
        ref[2,4,1,0]=(-0.20343056004)/(o+1.)+(0.301807185951)+(0.330425495069)*0.5**o
        ref[2,4,1,1]=(0.00513034050895)/(o+1.)+(-0.962405377783)+(0.106341453568)*0.5**o
        ref[2,4,2,0]=(0.0310460981037)/(o+1.)+(0.433282306621)+(0.994443804198)*0.5**o
        ref[2,4,2,1]=(0.902192873442)/(o+1.)+(-0.368558133903)+(0.113561397179)*0.5**o
        ref[3,0,0,0]=(0.900671903541)/(o+1.)+(0.202592699275)+(-0.463635004027)*0.5**o
        ref[3,0,0,1]=(0.35026221049)/(o+1.)+(0.341141942895)+(0.676123626258)*0.5**o
        ref[3,0,1,0]=(0.138548277811)/(o+1.)+(-0.0444101940734)+(-0.304361833494)*0.5**o
        ref[3,0,1,1]=(0.708876110175)/(o+1.)+(0.508195869745)+(0.307231747905)*0.5**o
        ref[3,0,2,0]=(0.982205702893)/(o+1.)+(0.314791291648)+(-0.130591836024)*0.5**o
        ref[3,0,2,1]=(1.34032270151)/(o+1.)+(-0.0845502854893)+(-0.576303721224)*0.5**o
        ref[3,1,0,0]=(-1.0578359675)/(o+1.)+(0.0482994274758)+(-0.798641869619)*0.5**o
        ref[3,1,0,1]=(-0.318745864355)/(o+1.)+(-0.535187955914)+(-0.00692013378255)*0.5**o
        ref[3,1,1,0]=(1.08164382033)/(o+1.)+(-0.0564910573463)+(0.975364017255)*0.5**o
        ref[3,1,1,1]=(0.304069039206)/(o+1.)+(0.818105356043)+(0.103423718636)*0.5**o
        ref[3,1,2,0]=(0.202131577493)/(o+1.)+(0.491584605592)+(0.0547068997542)*0.5**o
        ref[3,1,2,1]=(-0.790050626862)/(o+1.)+(0.652441993186)+(0.236730250658)*0.5**o
        ref[3,2,0,0]=(0.113163362881)/(o+1.)+(0.356243481631)+(0.388806525405)*0.5**o
        ref[3,2,0,1]=(1.09218675793)/(o+1.)+(-0.149692256988)+(-0.328460151904)*0.5**o
        ref[3,2,1,0]=(0.0360084147411)/(o+1.)+(-0.51843959536)+(-0.350954406466)*0.5**o
        ref[3,2,1,1]=(-0.455535353512)/(o+1.)+(0.497791987953)+(-0.804545424944)*0.5**o
        ref[3,2,2,0]=(-0.578849150697)/(o+1.)+(-1.18870958933)+(0.867664525177)*0.5**o
        ref[3,2,2,1]=(-0.51292246591)/(o+1.)+(-0.138210200569)+(0.997671019163)*0.5**o
        ref[3,3,0,0]=(1.21400753277)/(o+1.)+(-0.221646514915)+(-0.564960743631)*0.5**o
        ref[3,3,0,1]=(-0.201477939631)/(o+1.)+(0.113832346729)+(0.872459084593)*0.5**o
        ref[3,3,1,0]=(-0.679844797388)/(o+1.)+(0.168759614226)+(-0.256375661857)*0.5**o
        ref[3,3,1,1]=(-0.461574837551)/(o+1.)+(0.381760016613)+(-0.649399488997)*0.5**o
        ref[3,3,2,0]=(-1.6068534771)/(o+1.)+(-0.112445286441)+(0.237419242359)*0.5**o
        ref[3,3,2,1]=(0.756559815164)/(o+1.)+(0.110596703215)+(0.898298006002)*0.5**o
        ref[3,4,0,0]=(0.0482332220277)/(o+1.)+(0.457930350033)+(0.152469131812)*0.5**o
        ref[3,4,0,1]=(1.62019741951)/(o+1.)+(0.48918555421)+(-0.902280446763)*0.5**o
        ref[3,4,1,0]=(0.393133643776)/(o+1.)+(-0.852970591978)+(0.226692588742)*0.5**o
        ref[3,4,1,1]=(0.30297920343)/(o+1.)+(0.123825158749)+(0.99435141349)*0.5**o
        ref[3,4,2,0]=(0.0915688127321)/(o+1.)+(-0.0795234558081)+(-0.574679864968)*0.5**o
        ref[3,4,2,1]=(-1.84102882835)/(o+1.)+(0.261798322918)+(0.670125652473)*0.5**o
      res=integrate(arg)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_ContinuousFunction_rank0(self):
      """
      tests interpolate for rank 0 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ContinuousFunction onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(),w)
      ref=Data(0,(),w_ref)
      if dim==2:
        arg=(0.492060381705)*x[0]**o+(0.578791868713)*x[0]+(-0.439849757114)*x[1]**o+(-0.52629328967)*x[1]
        ref=(0.492060381705)*x_ref[0]**o+(0.578791868713)*x_ref[0]+(-0.439849757114)*x_ref[1]**o+(-0.52629328967)*x_ref[1]
      else:
        arg=(0.954737963273)*x[0]**o+(0.452578946878)*x[0]+(0.0471085060698)*x[1]**o+(-0.640169119832)*x[1]+(0.710401490627)*x[2]**o+(-0.382589884739)*x[2]
        ref=(0.954737963273)*x_ref[0]**o+(0.452578946878)*x_ref[0]+(0.0471085060698)*x_ref[1]**o+(-0.640169119832)*x_ref[1]+(0.710401490627)*x_ref[2]**o+(-0.382589884739)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_ContinuousFunction_rank1(self):
      """
      tests interpolate for rank 1 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ContinuousFunction onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(2,),w)
      ref=Data(0,(2,),w_ref)
      if dim==2:
        arg[0]=(-0.59604585527)*x[0]**o+(0.826981169401)*x[0]+(0.227617349484)*x[1]**o+(0.184907819029)*x[1]
        arg[1]=(-0.424684036731)*x[0]**o+(0.346537997905)*x[0]+(0.891921272597)*x[1]**o+(-0.145924182872)*x[1]
        ref[0]=(-0.59604585527)*x_ref[0]**o+(0.826981169401)*x_ref[0]+(0.227617349484)*x_ref[1]**o+(0.184907819029)*x_ref[1]
        ref[1]=(-0.424684036731)*x_ref[0]**o+(0.346537997905)*x_ref[0]+(0.891921272597)*x_ref[1]**o+(-0.145924182872)*x_ref[1]
      else:
        arg[0]=(0.777228173978)*x[0]**o+(-0.812215827155)*x[0]+(-0.618922680968)*x[1]**o+(0.0660312103206)*x[1]+(-0.917545504466)*x[2]**o+(0.988524619414)*x[2]
        arg[1]=(0.110219469565)*x[0]**o+(-0.441866866062)*x[0]+(-0.654447764335)*x[1]**o+(0.230716133289)*x[1]+(0.347487317165)*x[2]**o+(-0.948877903043)*x[2]
        ref[0]=(0.777228173978)*x_ref[0]**o+(-0.812215827155)*x_ref[0]+(-0.618922680968)*x_ref[1]**o+(0.0660312103206)*x_ref[1]+(-0.917545504466)*x_ref[2]**o+(0.988524619414)*x_ref[2]
        ref[1]=(0.110219469565)*x_ref[0]**o+(-0.441866866062)*x_ref[0]+(-0.654447764335)*x_ref[1]**o+(0.230716133289)*x_ref[1]+(0.347487317165)*x_ref[2]**o+(-0.948877903043)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_ContinuousFunction_rank2(self):
      """
      tests interpolate for rank 2 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ContinuousFunction onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5),w)
      ref=Data(0,(4, 5),w_ref)
      if dim==2:
        arg[0,0]=(-0.958809754913)*x[0]**o+(-0.844717049041)*x[0]+(0.462564514427)*x[1]**o+(-0.247678335594)*x[1]
        arg[0,1]=(0.700700054189)*x[0]**o+(-0.653724611457)*x[0]+(0.52659177803)*x[1]**o+(-0.387559446418)*x[1]
        arg[0,2]=(-0.326124300134)*x[0]**o+(0.115197897152)*x[0]+(-0.876340641846)*x[1]**o+(0.433060472581)*x[1]
        arg[0,3]=(-0.101215736407)*x[0]**o+(-0.483684510661)*x[0]+(0.0418770907036)*x[1]**o+(-0.845564414421)*x[1]
        arg[0,4]=(-0.293094855718)*x[0]**o+(0.710954930773)*x[0]+(0.238671713869)*x[1]**o+(0.576638343412)*x[1]
        arg[1,0]=(0.529861280122)*x[0]**o+(-0.259437904048)*x[0]+(-0.620725288158)*x[1]**o+(0.326787999382)*x[1]
        arg[1,1]=(0.421245543012)*x[0]**o+(-0.954139636698)*x[0]+(0.0498199734926)*x[1]**o+(0.24010238499)*x[1]
        arg[1,2]=(0.333362362373)*x[0]**o+(-0.999953175952)*x[0]+(0.866193849246)*x[1]**o+(0.764287410177)*x[1]
        arg[1,3]=(0.470415957539)*x[0]**o+(-0.133477966871)*x[0]+(-0.0416970809832)*x[1]**o+(-0.416352740444)*x[1]
        arg[1,4]=(-0.178613018492)*x[0]**o+(-0.967264985978)*x[0]+(-0.297102423687)*x[1]**o+(0.884184973413)*x[1]
        arg[2,0]=(-0.29363453294)*x[0]**o+(-0.0237360594862)*x[0]+(-0.300776450747)*x[1]**o+(-0.493932721521)*x[1]
        arg[2,1]=(0.795045745699)*x[0]**o+(0.984956851154)*x[0]+(-0.724208405235)*x[1]**o+(0.59029067239)*x[1]
        arg[2,2]=(-0.803012112531)*x[0]**o+(-0.831727785428)*x[0]+(0.330295112332)*x[1]**o+(0.0966317908147)*x[1]
        arg[2,3]=(-0.652332429144)*x[0]**o+(-0.38689457016)*x[0]+(-0.391780825295)*x[1]**o+(-0.444284790848)*x[1]
        arg[2,4]=(0.416775161398)*x[0]**o+(-0.16281006536)*x[0]+(-0.742938932867)*x[1]**o+(0.456486183714)*x[1]
        arg[3,0]=(0.506731632086)*x[0]**o+(-0.222468320035)*x[0]+(-0.906398223556)*x[1]**o+(-0.0250630448611)*x[1]
        arg[3,1]=(-0.39207398783)*x[0]**o+(-0.574739374252)*x[0]+(0.262101584569)*x[1]**o+(0.249587535087)*x[1]
        arg[3,2]=(-0.252593913503)*x[0]**o+(-0.864294052989)*x[0]+(0.0110434296336)*x[1]**o+(-0.224585040999)*x[1]
        arg[3,3]=(-0.631704469212)*x[0]**o+(-0.495153785048)*x[0]+(-0.304420746672)*x[1]**o+(-0.254005222021)*x[1]
        arg[3,4]=(0.775631055421)*x[0]**o+(0.370095409961)*x[0]+(0.894627612994)*x[1]**o+(-0.329716708629)*x[1]
        ref[0,0]=(-0.958809754913)*x_ref[0]**o+(-0.844717049041)*x_ref[0]+(0.462564514427)*x_ref[1]**o+(-0.247678335594)*x_ref[1]
        ref[0,1]=(0.700700054189)*x_ref[0]**o+(-0.653724611457)*x_ref[0]+(0.52659177803)*x_ref[1]**o+(-0.387559446418)*x_ref[1]
        ref[0,2]=(-0.326124300134)*x_ref[0]**o+(0.115197897152)*x_ref[0]+(-0.876340641846)*x_ref[1]**o+(0.433060472581)*x_ref[1]
        ref[0,3]=(-0.101215736407)*x_ref[0]**o+(-0.483684510661)*x_ref[0]+(0.0418770907036)*x_ref[1]**o+(-0.845564414421)*x_ref[1]
        ref[0,4]=(-0.293094855718)*x_ref[0]**o+(0.710954930773)*x_ref[0]+(0.238671713869)*x_ref[1]**o+(0.576638343412)*x_ref[1]
        ref[1,0]=(0.529861280122)*x_ref[0]**o+(-0.259437904048)*x_ref[0]+(-0.620725288158)*x_ref[1]**o+(0.326787999382)*x_ref[1]
        ref[1,1]=(0.421245543012)*x_ref[0]**o+(-0.954139636698)*x_ref[0]+(0.0498199734926)*x_ref[1]**o+(0.24010238499)*x_ref[1]
        ref[1,2]=(0.333362362373)*x_ref[0]**o+(-0.999953175952)*x_ref[0]+(0.866193849246)*x_ref[1]**o+(0.764287410177)*x_ref[1]
        ref[1,3]=(0.470415957539)*x_ref[0]**o+(-0.133477966871)*x_ref[0]+(-0.0416970809832)*x_ref[1]**o+(-0.416352740444)*x_ref[1]
        ref[1,4]=(-0.178613018492)*x_ref[0]**o+(-0.967264985978)*x_ref[0]+(-0.297102423687)*x_ref[1]**o+(0.884184973413)*x_ref[1]
        ref[2,0]=(-0.29363453294)*x_ref[0]**o+(-0.0237360594862)*x_ref[0]+(-0.300776450747)*x_ref[1]**o+(-0.493932721521)*x_ref[1]
        ref[2,1]=(0.795045745699)*x_ref[0]**o+(0.984956851154)*x_ref[0]+(-0.724208405235)*x_ref[1]**o+(0.59029067239)*x_ref[1]
        ref[2,2]=(-0.803012112531)*x_ref[0]**o+(-0.831727785428)*x_ref[0]+(0.330295112332)*x_ref[1]**o+(0.0966317908147)*x_ref[1]
        ref[2,3]=(-0.652332429144)*x_ref[0]**o+(-0.38689457016)*x_ref[0]+(-0.391780825295)*x_ref[1]**o+(-0.444284790848)*x_ref[1]
        ref[2,4]=(0.416775161398)*x_ref[0]**o+(-0.16281006536)*x_ref[0]+(-0.742938932867)*x_ref[1]**o+(0.456486183714)*x_ref[1]
        ref[3,0]=(0.506731632086)*x_ref[0]**o+(-0.222468320035)*x_ref[0]+(-0.906398223556)*x_ref[1]**o+(-0.0250630448611)*x_ref[1]
        ref[3,1]=(-0.39207398783)*x_ref[0]**o+(-0.574739374252)*x_ref[0]+(0.262101584569)*x_ref[1]**o+(0.249587535087)*x_ref[1]
        ref[3,2]=(-0.252593913503)*x_ref[0]**o+(-0.864294052989)*x_ref[0]+(0.0110434296336)*x_ref[1]**o+(-0.224585040999)*x_ref[1]
        ref[3,3]=(-0.631704469212)*x_ref[0]**o+(-0.495153785048)*x_ref[0]+(-0.304420746672)*x_ref[1]**o+(-0.254005222021)*x_ref[1]
        ref[3,4]=(0.775631055421)*x_ref[0]**o+(0.370095409961)*x_ref[0]+(0.894627612994)*x_ref[1]**o+(-0.329716708629)*x_ref[1]
      else:
        arg[0,0]=(0.392450639899)*x[0]**o+(-0.685488230002)*x[0]+(-0.41545928428)*x[1]**o+(0.244144392379)*x[1]+(0.602642156528)*x[2]**o+(-0.348717378461)*x[2]
        arg[0,1]=(-0.341243165541)*x[0]**o+(-0.616870164204)*x[0]+(-0.510613066362)*x[1]**o+(-0.81387010212)*x[1]+(0.856239691119)*x[2]**o+(0.0744651854239)*x[2]
        arg[0,2]=(-0.765983413604)*x[0]**o+(0.474551394174)*x[0]+(-0.939524929765)*x[1]**o+(0.418297898065)*x[1]+(0.00589909539531)*x[2]**o+(0.0889012994897)*x[2]
        arg[0,3]=(-0.521217563352)*x[0]**o+(-0.256316459647)*x[0]+(-0.0265965098379)*x[1]**o+(-0.597562278966)*x[1]+(0.0389151661339)*x[2]**o+(-0.572109451352)*x[2]
        arg[0,4]=(-0.853801268491)*x[0]**o+(-0.691274812057)*x[0]+(-0.682435618581)*x[1]**o+(-0.974334375359)*x[1]+(-0.642333316794)*x[2]**o+(0.353816751252)*x[2]
        arg[1,0]=(-0.657545394322)*x[0]**o+(-0.334172338096)*x[0]+(0.016748279459)*x[1]**o+(-0.294858621125)*x[1]+(-0.49403880046)*x[2]**o+(-0.152416774822)*x[2]
        arg[1,1]=(-0.602731976042)*x[0]**o+(0.685046816545)*x[0]+(0.750001644716)*x[1]**o+(-0.202076972289)*x[1]+(-0.0232888078434)*x[2]**o+(-0.0774477649878)*x[2]
        arg[1,2]=(-0.707321785527)*x[0]**o+(0.618526464205)*x[0]+(-0.578548442468)*x[1]**o+(-0.411374744442)*x[1]+(-0.944277708787)*x[2]**o+(0.647856975707)*x[2]
        arg[1,3]=(-0.222503271022)*x[0]**o+(-0.929428092728)*x[0]+(-0.976553212424)*x[1]**o+(0.0376439617084)*x[1]+(0.0428078568512)*x[2]**o+(-0.441850944272)*x[2]
        arg[1,4]=(-0.862164898455)*x[0]**o+(-0.989900221991)*x[0]+(0.110610844567)*x[1]**o+(0.419384612236)*x[1]+(-0.00271948841262)*x[2]**o+(0.301635101629)*x[2]
        arg[2,0]=(-0.0950421881942)*x[0]**o+(-0.741293053909)*x[0]+(-0.461538879874)*x[1]**o+(-0.8199842553)*x[1]+(0.0590344916829)*x[2]**o+(0.352541518434)*x[2]
        arg[2,1]=(-0.734407995647)*x[0]**o+(-0.726167721992)*x[0]+(0.700521229916)*x[1]**o+(0.753555863428)*x[1]+(-0.891638139696)*x[2]**o+(-0.996550711213)*x[2]
        arg[2,2]=(0.777110352629)*x[0]**o+(0.309719201157)*x[0]+(0.026816547522)*x[1]**o+(0.33161010287)*x[1]+(0.257929504352)*x[2]**o+(0.71871314654)*x[2]
        arg[2,3]=(-0.0982522143825)*x[0]**o+(0.0532144836415)*x[0]+(0.155895442381)*x[1]**o+(-0.872222127033)*x[1]+(0.810538165596)*x[2]**o+(-0.917071363009)*x[2]
        arg[2,4]=(0.882674783877)*x[0]**o+(-0.246439463648)*x[0]+(-0.478821457401)*x[1]**o+(-0.629183697296)*x[1]+(0.833283185393)*x[2]**o+(-0.895796651184)*x[2]
        arg[3,0]=(0.949019461395)*x[0]**o+(0.437511668107)*x[0]+(0.46679355983)*x[1]**o+(0.375391913775)*x[1]+(0.992336073925)*x[2]**o+(0.784847856608)*x[2]
        arg[3,1]=(0.10730061928)*x[0]**o+(0.624678931808)*x[0]+(-0.105084917663)*x[1]**o+(-0.0650034223387)*x[1]+(-0.271922353868)*x[2]**o+(-0.182967296038)*x[2]
        arg[3,2]=(0.115856234055)*x[0]**o+(0.711912354922)*x[0]+(-0.317962972122)*x[1]**o+(0.577856092951)*x[1]+(-0.246833922867)*x[2]**o+(0.354116462218)*x[2]
        arg[3,3]=(0.40937236699)*x[0]**o+(0.639252679403)*x[0]+(0.540206943219)*x[1]**o+(0.287006139758)*x[1]+(0.123311433598)*x[2]**o+(-0.0300672780213)*x[2]
        arg[3,4]=(-0.190299257713)*x[0]**o+(-0.022055675261)*x[0]+(-0.640751994447)*x[1]**o+(-0.411801947317)*x[1]+(0.0675008125057)*x[2]**o+(-0.659424255203)*x[2]
        ref[0,0]=(0.392450639899)*x_ref[0]**o+(-0.685488230002)*x_ref[0]+(-0.41545928428)*x_ref[1]**o+(0.244144392379)*x_ref[1]+(0.602642156528)*x_ref[2]**o+(-0.348717378461)*x_ref[2]
        ref[0,1]=(-0.341243165541)*x_ref[0]**o+(-0.616870164204)*x_ref[0]+(-0.510613066362)*x_ref[1]**o+(-0.81387010212)*x_ref[1]+(0.856239691119)*x_ref[2]**o+(0.0744651854239)*x_ref[2]
        ref[0,2]=(-0.765983413604)*x_ref[0]**o+(0.474551394174)*x_ref[0]+(-0.939524929765)*x_ref[1]**o+(0.418297898065)*x_ref[1]+(0.00589909539531)*x_ref[2]**o+(0.0889012994897)*x_ref[2]
        ref[0,3]=(-0.521217563352)*x_ref[0]**o+(-0.256316459647)*x_ref[0]+(-0.0265965098379)*x_ref[1]**o+(-0.597562278966)*x_ref[1]+(0.0389151661339)*x_ref[2]**o+(-0.572109451352)*x_ref[2]
        ref[0,4]=(-0.853801268491)*x_ref[0]**o+(-0.691274812057)*x_ref[0]+(-0.682435618581)*x_ref[1]**o+(-0.974334375359)*x_ref[1]+(-0.642333316794)*x_ref[2]**o+(0.353816751252)*x_ref[2]
        ref[1,0]=(-0.657545394322)*x_ref[0]**o+(-0.334172338096)*x_ref[0]+(0.016748279459)*x_ref[1]**o+(-0.294858621125)*x_ref[1]+(-0.49403880046)*x_ref[2]**o+(-0.152416774822)*x_ref[2]
        ref[1,1]=(-0.602731976042)*x_ref[0]**o+(0.685046816545)*x_ref[0]+(0.750001644716)*x_ref[1]**o+(-0.202076972289)*x_ref[1]+(-0.0232888078434)*x_ref[2]**o+(-0.0774477649878)*x_ref[2]
        ref[1,2]=(-0.707321785527)*x_ref[0]**o+(0.618526464205)*x_ref[0]+(-0.578548442468)*x_ref[1]**o+(-0.411374744442)*x_ref[1]+(-0.944277708787)*x_ref[2]**o+(0.647856975707)*x_ref[2]
        ref[1,3]=(-0.222503271022)*x_ref[0]**o+(-0.929428092728)*x_ref[0]+(-0.976553212424)*x_ref[1]**o+(0.0376439617084)*x_ref[1]+(0.0428078568512)*x_ref[2]**o+(-0.441850944272)*x_ref[2]
        ref[1,4]=(-0.862164898455)*x_ref[0]**o+(-0.989900221991)*x_ref[0]+(0.110610844567)*x_ref[1]**o+(0.419384612236)*x_ref[1]+(-0.00271948841262)*x_ref[2]**o+(0.301635101629)*x_ref[2]
        ref[2,0]=(-0.0950421881942)*x_ref[0]**o+(-0.741293053909)*x_ref[0]+(-0.461538879874)*x_ref[1]**o+(-0.8199842553)*x_ref[1]+(0.0590344916829)*x_ref[2]**o+(0.352541518434)*x_ref[2]
        ref[2,1]=(-0.734407995647)*x_ref[0]**o+(-0.726167721992)*x_ref[0]+(0.700521229916)*x_ref[1]**o+(0.753555863428)*x_ref[1]+(-0.891638139696)*x_ref[2]**o+(-0.996550711213)*x_ref[2]
        ref[2,2]=(0.777110352629)*x_ref[0]**o+(0.309719201157)*x_ref[0]+(0.026816547522)*x_ref[1]**o+(0.33161010287)*x_ref[1]+(0.257929504352)*x_ref[2]**o+(0.71871314654)*x_ref[2]
        ref[2,3]=(-0.0982522143825)*x_ref[0]**o+(0.0532144836415)*x_ref[0]+(0.155895442381)*x_ref[1]**o+(-0.872222127033)*x_ref[1]+(0.810538165596)*x_ref[2]**o+(-0.917071363009)*x_ref[2]
        ref[2,4]=(0.882674783877)*x_ref[0]**o+(-0.246439463648)*x_ref[0]+(-0.478821457401)*x_ref[1]**o+(-0.629183697296)*x_ref[1]+(0.833283185393)*x_ref[2]**o+(-0.895796651184)*x_ref[2]
        ref[3,0]=(0.949019461395)*x_ref[0]**o+(0.437511668107)*x_ref[0]+(0.46679355983)*x_ref[1]**o+(0.375391913775)*x_ref[1]+(0.992336073925)*x_ref[2]**o+(0.784847856608)*x_ref[2]
        ref[3,1]=(0.10730061928)*x_ref[0]**o+(0.624678931808)*x_ref[0]+(-0.105084917663)*x_ref[1]**o+(-0.0650034223387)*x_ref[1]+(-0.271922353868)*x_ref[2]**o+(-0.182967296038)*x_ref[2]
        ref[3,2]=(0.115856234055)*x_ref[0]**o+(0.711912354922)*x_ref[0]+(-0.317962972122)*x_ref[1]**o+(0.577856092951)*x_ref[1]+(-0.246833922867)*x_ref[2]**o+(0.354116462218)*x_ref[2]
        ref[3,3]=(0.40937236699)*x_ref[0]**o+(0.639252679403)*x_ref[0]+(0.540206943219)*x_ref[1]**o+(0.287006139758)*x_ref[1]+(0.123311433598)*x_ref[2]**o+(-0.0300672780213)*x_ref[2]
        ref[3,4]=(-0.190299257713)*x_ref[0]**o+(-0.022055675261)*x_ref[0]+(-0.640751994447)*x_ref[1]**o+(-0.411801947317)*x_ref[1]+(0.0675008125057)*x_ref[2]**o+(-0.659424255203)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_ContinuousFunction_rank3(self):
      """
      tests interpolate for rank 3 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ContinuousFunction onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2),w_ref)
      if dim==2:
        arg[0,0,0]=(-0.697323110242)*x[0]**o+(-0.806494185218)*x[0]+(0.387914602484)*x[1]**o+(0.841835487965)*x[1]
        arg[0,0,1]=(-0.77670653908)*x[0]**o+(0.311710212706)*x[0]+(-0.0951864020181)*x[1]**o+(-0.14909139946)*x[1]
        arg[0,1,0]=(0.80933791543)*x[0]**o+(0.72441968276)*x[0]+(-0.446054850085)*x[1]**o+(0.419303534988)*x[1]
        arg[0,1,1]=(0.174464190017)*x[0]**o+(0.330609661878)*x[0]+(0.798365390058)*x[1]**o+(0.354899223817)*x[1]
        arg[1,0,0]=(0.36073672839)*x[0]**o+(0.831900846214)*x[0]+(-0.656354645044)*x[1]**o+(0.200645044646)*x[1]
        arg[1,0,1]=(0.817889801065)*x[0]**o+(0.34052889294)*x[0]+(-0.2416944924)*x[1]**o+(0.0922342148934)*x[1]
        arg[1,1,0]=(-0.962116509558)*x[0]**o+(-0.0785232077542)*x[0]+(-0.981399563356)*x[1]**o+(0.785509095614)*x[1]
        arg[1,1,1]=(-0.688444030508)*x[0]**o+(0.651617638972)*x[0]+(0.160584046401)*x[1]**o+(0.253258991923)*x[1]
        arg[2,0,0]=(0.0728970412574)*x[0]**o+(-0.647328240217)*x[0]+(0.517665494535)*x[1]**o+(-0.416383753573)*x[1]
        arg[2,0,1]=(0.148104739733)*x[0]**o+(-0.378371639837)*x[0]+(0.848608631336)*x[1]**o+(-0.881516167852)*x[1]
        arg[2,1,0]=(-0.506376676256)*x[0]**o+(0.234090340446)*x[0]+(-0.737828959566)*x[1]**o+(-0.74093763135)*x[1]
        arg[2,1,1]=(0.259605354135)*x[0]**o+(0.537424356513)*x[0]+(-0.405718119693)*x[1]**o+(0.120663712044)*x[1]
        arg[3,0,0]=(-0.609659975269)*x[0]**o+(-0.455923293874)*x[0]+(-0.00765676965971)*x[1]**o+(-0.710899349259)*x[1]
        arg[3,0,1]=(-0.731011612904)*x[0]**o+(-0.355060765358)*x[0]+(-0.127369372538)*x[1]**o+(0.694265371908)*x[1]
        arg[3,1,0]=(0.188766337132)*x[0]**o+(-0.194878304215)*x[0]+(-0.963356218109)*x[1]**o+(0.327053372293)*x[1]
        arg[3,1,1]=(-0.566712779324)*x[0]**o+(-0.251473305071)*x[0]+(0.0181297146138)*x[1]**o+(0.257664697754)*x[1]
        arg[4,0,0]=(0.574100764569)*x[0]**o+(-0.717776999574)*x[0]+(0.880998124155)*x[1]**o+(-0.810615928848)*x[1]
        arg[4,0,1]=(0.42552502518)*x[0]**o+(0.456751341147)*x[0]+(-0.484998619005)*x[1]**o+(0.932151187696)*x[1]
        arg[4,1,0]=(0.56865980485)*x[0]**o+(0.593928872734)*x[0]+(-0.475639894836)*x[1]**o+(-0.990718402693)*x[1]
        arg[4,1,1]=(0.129586342261)*x[0]**o+(-0.408951830619)*x[0]+(0.132518475289)*x[1]**o+(0.708612820692)*x[1]
        arg[5,0,0]=(0.0708038739082)*x[0]**o+(0.524270558649)*x[0]+(0.368504533482)*x[1]**o+(0.935828810393)*x[1]
        arg[5,0,1]=(-0.383650306829)*x[0]**o+(-0.0439362943961)*x[0]+(0.80939275198)*x[1]**o+(-0.61335508023)*x[1]
        arg[5,1,0]=(-0.621571339247)*x[0]**o+(-0.185069293199)*x[0]+(0.368472011431)*x[1]**o+(-0.388752495932)*x[1]
        arg[5,1,1]=(-0.263310763174)*x[0]**o+(-0.694616630015)*x[0]+(0.271238274203)*x[1]**o+(-0.477861402347)*x[1]
        ref[0,0,0]=(-0.697323110242)*x_ref[0]**o+(-0.806494185218)*x_ref[0]+(0.387914602484)*x_ref[1]**o+(0.841835487965)*x_ref[1]
        ref[0,0,1]=(-0.77670653908)*x_ref[0]**o+(0.311710212706)*x_ref[0]+(-0.0951864020181)*x_ref[1]**o+(-0.14909139946)*x_ref[1]
        ref[0,1,0]=(0.80933791543)*x_ref[0]**o+(0.72441968276)*x_ref[0]+(-0.446054850085)*x_ref[1]**o+(0.419303534988)*x_ref[1]
        ref[0,1,1]=(0.174464190017)*x_ref[0]**o+(0.330609661878)*x_ref[0]+(0.798365390058)*x_ref[1]**o+(0.354899223817)*x_ref[1]
        ref[1,0,0]=(0.36073672839)*x_ref[0]**o+(0.831900846214)*x_ref[0]+(-0.656354645044)*x_ref[1]**o+(0.200645044646)*x_ref[1]
        ref[1,0,1]=(0.817889801065)*x_ref[0]**o+(0.34052889294)*x_ref[0]+(-0.2416944924)*x_ref[1]**o+(0.0922342148934)*x_ref[1]
        ref[1,1,0]=(-0.962116509558)*x_ref[0]**o+(-0.0785232077542)*x_ref[0]+(-0.981399563356)*x_ref[1]**o+(0.785509095614)*x_ref[1]
        ref[1,1,1]=(-0.688444030508)*x_ref[0]**o+(0.651617638972)*x_ref[0]+(0.160584046401)*x_ref[1]**o+(0.253258991923)*x_ref[1]
        ref[2,0,0]=(0.0728970412574)*x_ref[0]**o+(-0.647328240217)*x_ref[0]+(0.517665494535)*x_ref[1]**o+(-0.416383753573)*x_ref[1]
        ref[2,0,1]=(0.148104739733)*x_ref[0]**o+(-0.378371639837)*x_ref[0]+(0.848608631336)*x_ref[1]**o+(-0.881516167852)*x_ref[1]
        ref[2,1,0]=(-0.506376676256)*x_ref[0]**o+(0.234090340446)*x_ref[0]+(-0.737828959566)*x_ref[1]**o+(-0.74093763135)*x_ref[1]
        ref[2,1,1]=(0.259605354135)*x_ref[0]**o+(0.537424356513)*x_ref[0]+(-0.405718119693)*x_ref[1]**o+(0.120663712044)*x_ref[1]
        ref[3,0,0]=(-0.609659975269)*x_ref[0]**o+(-0.455923293874)*x_ref[0]+(-0.00765676965971)*x_ref[1]**o+(-0.710899349259)*x_ref[1]
        ref[3,0,1]=(-0.731011612904)*x_ref[0]**o+(-0.355060765358)*x_ref[0]+(-0.127369372538)*x_ref[1]**o+(0.694265371908)*x_ref[1]
        ref[3,1,0]=(0.188766337132)*x_ref[0]**o+(-0.194878304215)*x_ref[0]+(-0.963356218109)*x_ref[1]**o+(0.327053372293)*x_ref[1]
        ref[3,1,1]=(-0.566712779324)*x_ref[0]**o+(-0.251473305071)*x_ref[0]+(0.0181297146138)*x_ref[1]**o+(0.257664697754)*x_ref[1]
        ref[4,0,0]=(0.574100764569)*x_ref[0]**o+(-0.717776999574)*x_ref[0]+(0.880998124155)*x_ref[1]**o+(-0.810615928848)*x_ref[1]
        ref[4,0,1]=(0.42552502518)*x_ref[0]**o+(0.456751341147)*x_ref[0]+(-0.484998619005)*x_ref[1]**o+(0.932151187696)*x_ref[1]
        ref[4,1,0]=(0.56865980485)*x_ref[0]**o+(0.593928872734)*x_ref[0]+(-0.475639894836)*x_ref[1]**o+(-0.990718402693)*x_ref[1]
        ref[4,1,1]=(0.129586342261)*x_ref[0]**o+(-0.408951830619)*x_ref[0]+(0.132518475289)*x_ref[1]**o+(0.708612820692)*x_ref[1]
        ref[5,0,0]=(0.0708038739082)*x_ref[0]**o+(0.524270558649)*x_ref[0]+(0.368504533482)*x_ref[1]**o+(0.935828810393)*x_ref[1]
        ref[5,0,1]=(-0.383650306829)*x_ref[0]**o+(-0.0439362943961)*x_ref[0]+(0.80939275198)*x_ref[1]**o+(-0.61335508023)*x_ref[1]
        ref[5,1,0]=(-0.621571339247)*x_ref[0]**o+(-0.185069293199)*x_ref[0]+(0.368472011431)*x_ref[1]**o+(-0.388752495932)*x_ref[1]
        ref[5,1,1]=(-0.263310763174)*x_ref[0]**o+(-0.694616630015)*x_ref[0]+(0.271238274203)*x_ref[1]**o+(-0.477861402347)*x_ref[1]
      else:
        arg[0,0,0]=(0.287534645778)*x[0]**o+(0.94865229379)*x[0]+(0.655299232864)*x[1]**o+(0.350034924637)*x[1]+(0.556341391183)*x[2]**o+(0.840643265765)*x[2]
        arg[0,0,1]=(-0.207213781549)*x[0]**o+(-0.0394497047797)*x[0]+(-0.0236016308575)*x[1]**o+(0.344122299199)*x[1]+(0.389588325996)*x[2]**o+(0.941980468657)*x[2]
        arg[0,1,0]=(0.00980412235862)*x[0]**o+(0.74862549447)*x[0]+(0.630165284383)*x[1]**o+(0.234711268989)*x[1]+(-0.22509981272)*x[2]**o+(-0.0153888168649)*x[2]
        arg[0,1,1]=(-0.87309186262)*x[0]**o+(0.833526923917)*x[0]+(0.78785479415)*x[1]**o+(0.823967259171)*x[1]+(0.320244316109)*x[2]**o+(-0.735225854417)*x[2]
        arg[1,0,0]=(-0.0892232551051)*x[0]**o+(-0.346728833563)*x[0]+(0.422774634671)*x[1]**o+(0.793222381957)*x[1]+(-0.670476677006)*x[2]**o+(0.896884540116)*x[2]
        arg[1,0,1]=(-0.0023382136345)*x[0]**o+(0.73058281545)*x[0]+(0.11828880374)*x[1]**o+(-0.691839091619)*x[1]+(-0.39086463392)*x[2]**o+(0.66200209782)*x[2]
        arg[1,1,0]=(-0.924238055558)*x[0]**o+(-0.334208271705)*x[0]+(-0.57106045555)*x[1]**o+(-0.765650999193)*x[1]+(0.0404533468054)*x[2]**o+(-0.225975260777)*x[2]
        arg[1,1,1]=(-0.351110999108)*x[0]**o+(0.626166346251)*x[0]+(0.385737198467)*x[1]**o+(0.577561466908)*x[1]+(0.708325217242)*x[2]**o+(0.193727923904)*x[2]
        arg[2,0,0]=(0.278942080997)*x[0]**o+(-0.569761653572)*x[0]+(-0.624879270918)*x[1]**o+(-0.0151754112896)*x[1]+(0.411399083825)*x[2]**o+(-0.846669568641)*x[2]
        arg[2,0,1]=(-0.543586963879)*x[0]**o+(-0.383959956625)*x[0]+(0.184375944938)*x[1]**o+(-0.104178233562)*x[1]+(-0.223101894662)*x[2]**o+(-0.17221835595)*x[2]
        arg[2,1,0]=(0.317103921207)*x[0]**o+(-0.919542461354)*x[0]+(0.56666554454)*x[1]**o+(0.895772298847)*x[1]+(0.714550707822)*x[2]**o+(0.634702089556)*x[2]
        arg[2,1,1]=(0.986576254164)*x[0]**o+(0.544493787633)*x[0]+(0.964620027203)*x[1]**o+(0.259991384678)*x[1]+(-0.577361835739)*x[2]**o+(0.651593456822)*x[2]
        arg[3,0,0]=(0.362841472614)*x[0]**o+(-0.521771228041)*x[0]+(-0.594530755517)*x[1]**o+(0.700647681241)*x[1]+(-0.131683781311)*x[2]**o+(0.636310189834)*x[2]
        arg[3,0,1]=(0.885891862321)*x[0]**o+(0.397438320764)*x[0]+(0.711380452937)*x[1]**o+(0.962571325144)*x[1]+(-0.300114113339)*x[2]**o+(0.34700388868)*x[2]
        arg[3,1,0]=(0.52140726408)*x[0]**o+(-0.0921295295522)*x[0]+(-0.823076091446)*x[1]**o+(0.477401876125)*x[1]+(-0.973057905067)*x[2]**o+(-0.975526794128)*x[2]
        arg[3,1,1]=(-0.241503887113)*x[0]**o+(0.133988088285)*x[0]+(0.243026393121)*x[1]**o+(0.0848716779473)*x[1]+(0.140349614791)*x[2]**o+(-0.217182025695)*x[2]
        arg[4,0,0]=(0.680080194053)*x[0]**o+(0.512009633844)*x[0]+(-0.640214066363)*x[1]**o+(0.571800633214)*x[1]+(-0.749395617545)*x[2]**o+(0.448487668786)*x[2]
        arg[4,0,1]=(-0.792959916349)*x[0]**o+(-0.433770516776)*x[0]+(-0.380980758215)*x[1]**o+(-0.831154769101)*x[1]+(0.277551460651)*x[2]**o+(0.626476984233)*x[2]
        arg[4,1,0]=(-0.285102566546)*x[0]**o+(0.745636275253)*x[0]+(0.4999427836)*x[1]**o+(0.561343302884)*x[1]+(0.796220302361)*x[2]**o+(-0.830020520143)*x[2]
        arg[4,1,1]=(0.237420507314)*x[0]**o+(0.327181858092)*x[0]+(-0.895743235593)*x[1]**o+(-0.496964477443)*x[1]+(0.57336428204)*x[2]**o+(-0.154267071287)*x[2]
        arg[5,0,0]=(0.0401925997607)*x[0]**o+(-0.0155957424748)*x[0]+(-0.658363451239)*x[1]**o+(-0.175223211879)*x[1]+(-0.743729768327)*x[2]**o+(0.391531327128)*x[2]
        arg[5,0,1]=(0.836239801824)*x[0]**o+(-0.72288085675)*x[0]+(0.177981042887)*x[1]**o+(0.354505260098)*x[1]+(-0.180520504267)*x[2]**o+(-0.380306239974)*x[2]
        arg[5,1,0]=(-0.65536853024)*x[0]**o+(-0.00573854242476)*x[0]+(-0.261460514314)*x[1]**o+(-0.289771405693)*x[1]+(0.750775397523)*x[2]**o+(-0.333215608562)*x[2]
        arg[5,1,1]=(-0.579532884024)*x[0]**o+(-0.208677373724)*x[0]+(-0.379612024024)*x[1]**o+(-0.0565813995114)*x[1]+(-0.867913562592)*x[2]**o+(-0.254654298293)*x[2]
        ref[0,0,0]=(0.287534645778)*x_ref[0]**o+(0.94865229379)*x_ref[0]+(0.655299232864)*x_ref[1]**o+(0.350034924637)*x_ref[1]+(0.556341391183)*x_ref[2]**o+(0.840643265765)*x_ref[2]
        ref[0,0,1]=(-0.207213781549)*x_ref[0]**o+(-0.0394497047797)*x_ref[0]+(-0.0236016308575)*x_ref[1]**o+(0.344122299199)*x_ref[1]+(0.389588325996)*x_ref[2]**o+(0.941980468657)*x_ref[2]
        ref[0,1,0]=(0.00980412235862)*x_ref[0]**o+(0.74862549447)*x_ref[0]+(0.630165284383)*x_ref[1]**o+(0.234711268989)*x_ref[1]+(-0.22509981272)*x_ref[2]**o+(-0.0153888168649)*x_ref[2]
        ref[0,1,1]=(-0.87309186262)*x_ref[0]**o+(0.833526923917)*x_ref[0]+(0.78785479415)*x_ref[1]**o+(0.823967259171)*x_ref[1]+(0.320244316109)*x_ref[2]**o+(-0.735225854417)*x_ref[2]
        ref[1,0,0]=(-0.0892232551051)*x_ref[0]**o+(-0.346728833563)*x_ref[0]+(0.422774634671)*x_ref[1]**o+(0.793222381957)*x_ref[1]+(-0.670476677006)*x_ref[2]**o+(0.896884540116)*x_ref[2]
        ref[1,0,1]=(-0.0023382136345)*x_ref[0]**o+(0.73058281545)*x_ref[0]+(0.11828880374)*x_ref[1]**o+(-0.691839091619)*x_ref[1]+(-0.39086463392)*x_ref[2]**o+(0.66200209782)*x_ref[2]
        ref[1,1,0]=(-0.924238055558)*x_ref[0]**o+(-0.334208271705)*x_ref[0]+(-0.57106045555)*x_ref[1]**o+(-0.765650999193)*x_ref[1]+(0.0404533468054)*x_ref[2]**o+(-0.225975260777)*x_ref[2]
        ref[1,1,1]=(-0.351110999108)*x_ref[0]**o+(0.626166346251)*x_ref[0]+(0.385737198467)*x_ref[1]**o+(0.577561466908)*x_ref[1]+(0.708325217242)*x_ref[2]**o+(0.193727923904)*x_ref[2]
        ref[2,0,0]=(0.278942080997)*x_ref[0]**o+(-0.569761653572)*x_ref[0]+(-0.624879270918)*x_ref[1]**o+(-0.0151754112896)*x_ref[1]+(0.411399083825)*x_ref[2]**o+(-0.846669568641)*x_ref[2]
        ref[2,0,1]=(-0.543586963879)*x_ref[0]**o+(-0.383959956625)*x_ref[0]+(0.184375944938)*x_ref[1]**o+(-0.104178233562)*x_ref[1]+(-0.223101894662)*x_ref[2]**o+(-0.17221835595)*x_ref[2]
        ref[2,1,0]=(0.317103921207)*x_ref[0]**o+(-0.919542461354)*x_ref[0]+(0.56666554454)*x_ref[1]**o+(0.895772298847)*x_ref[1]+(0.714550707822)*x_ref[2]**o+(0.634702089556)*x_ref[2]
        ref[2,1,1]=(0.986576254164)*x_ref[0]**o+(0.544493787633)*x_ref[0]+(0.964620027203)*x_ref[1]**o+(0.259991384678)*x_ref[1]+(-0.577361835739)*x_ref[2]**o+(0.651593456822)*x_ref[2]
        ref[3,0,0]=(0.362841472614)*x_ref[0]**o+(-0.521771228041)*x_ref[0]+(-0.594530755517)*x_ref[1]**o+(0.700647681241)*x_ref[1]+(-0.131683781311)*x_ref[2]**o+(0.636310189834)*x_ref[2]
        ref[3,0,1]=(0.885891862321)*x_ref[0]**o+(0.397438320764)*x_ref[0]+(0.711380452937)*x_ref[1]**o+(0.962571325144)*x_ref[1]+(-0.300114113339)*x_ref[2]**o+(0.34700388868)*x_ref[2]
        ref[3,1,0]=(0.52140726408)*x_ref[0]**o+(-0.0921295295522)*x_ref[0]+(-0.823076091446)*x_ref[1]**o+(0.477401876125)*x_ref[1]+(-0.973057905067)*x_ref[2]**o+(-0.975526794128)*x_ref[2]
        ref[3,1,1]=(-0.241503887113)*x_ref[0]**o+(0.133988088285)*x_ref[0]+(0.243026393121)*x_ref[1]**o+(0.0848716779473)*x_ref[1]+(0.140349614791)*x_ref[2]**o+(-0.217182025695)*x_ref[2]
        ref[4,0,0]=(0.680080194053)*x_ref[0]**o+(0.512009633844)*x_ref[0]+(-0.640214066363)*x_ref[1]**o+(0.571800633214)*x_ref[1]+(-0.749395617545)*x_ref[2]**o+(0.448487668786)*x_ref[2]
        ref[4,0,1]=(-0.792959916349)*x_ref[0]**o+(-0.433770516776)*x_ref[0]+(-0.380980758215)*x_ref[1]**o+(-0.831154769101)*x_ref[1]+(0.277551460651)*x_ref[2]**o+(0.626476984233)*x_ref[2]
        ref[4,1,0]=(-0.285102566546)*x_ref[0]**o+(0.745636275253)*x_ref[0]+(0.4999427836)*x_ref[1]**o+(0.561343302884)*x_ref[1]+(0.796220302361)*x_ref[2]**o+(-0.830020520143)*x_ref[2]
        ref[4,1,1]=(0.237420507314)*x_ref[0]**o+(0.327181858092)*x_ref[0]+(-0.895743235593)*x_ref[1]**o+(-0.496964477443)*x_ref[1]+(0.57336428204)*x_ref[2]**o+(-0.154267071287)*x_ref[2]
        ref[5,0,0]=(0.0401925997607)*x_ref[0]**o+(-0.0155957424748)*x_ref[0]+(-0.658363451239)*x_ref[1]**o+(-0.175223211879)*x_ref[1]+(-0.743729768327)*x_ref[2]**o+(0.391531327128)*x_ref[2]
        ref[5,0,1]=(0.836239801824)*x_ref[0]**o+(-0.72288085675)*x_ref[0]+(0.177981042887)*x_ref[1]**o+(0.354505260098)*x_ref[1]+(-0.180520504267)*x_ref[2]**o+(-0.380306239974)*x_ref[2]
        ref[5,1,0]=(-0.65536853024)*x_ref[0]**o+(-0.00573854242476)*x_ref[0]+(-0.261460514314)*x_ref[1]**o+(-0.289771405693)*x_ref[1]+(0.750775397523)*x_ref[2]**o+(-0.333215608562)*x_ref[2]
        ref[5,1,1]=(-0.579532884024)*x_ref[0]**o+(-0.208677373724)*x_ref[0]+(-0.379612024024)*x_ref[1]**o+(-0.0565813995114)*x_ref[1]+(-0.867913562592)*x_ref[2]**o+(-0.254654298293)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_ContinuousFunction_rank4(self):
      """
      tests interpolate for rank 4 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ContinuousFunction onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5, 3, 2),w)
      ref=Data(0,(4, 5, 3, 2),w_ref)
      if dim==2:
        arg[0,0,0,0]=(-0.274472293956)*x[0]**o+(-0.293347130929)*x[0]+(0.392586448254)*x[1]**o+(-0.403066854054)*x[1]
        arg[0,0,0,1]=(0.385933192907)*x[0]**o+(0.484240065867)*x[0]+(0.144565631551)*x[1]**o+(0.882508492739)*x[1]
        arg[0,0,1,0]=(0.993368905137)*x[0]**o+(0.340349919343)*x[0]+(-0.956806746524)*x[1]**o+(0.990607958001)*x[1]
        arg[0,0,1,1]=(-0.88610250102)*x[0]**o+(-0.543705587259)*x[0]+(0.170798164924)*x[1]**o+(0.685331195754)*x[1]
        arg[0,0,2,0]=(-0.922211611143)*x[0]**o+(-0.103640037893)*x[0]+(0.401354703434)*x[1]**o+(-0.117826261635)*x[1]
        arg[0,0,2,1]=(-0.391601387132)*x[0]**o+(-0.472753344804)*x[0]+(-0.473433838993)*x[1]**o+(-0.173501635989)*x[1]
        arg[0,1,0,0]=(0.747353274916)*x[0]**o+(-0.960856612086)*x[0]+(-0.54809012978)*x[1]**o+(0.862023893332)*x[1]
        arg[0,1,0,1]=(-0.256359001149)*x[0]**o+(-0.868975044191)*x[0]+(0.494314269744)*x[1]**o+(0.949957481172)*x[1]
        arg[0,1,1,0]=(-0.0256658750601)*x[0]**o+(0.145499104203)*x[0]+(-0.68669791629)*x[1]**o+(0.447326207295)*x[1]
        arg[0,1,1,1]=(0.150617284467)*x[0]**o+(-0.350046088196)*x[0]+(-0.367671302053)*x[1]**o+(0.633220122068)*x[1]
        arg[0,1,2,0]=(-0.714145362271)*x[0]**o+(-0.957661490758)*x[0]+(0.386253770747)*x[1]**o+(0.982634262634)*x[1]
        arg[0,1,2,1]=(0.707565096824)*x[0]**o+(0.281758058573)*x[0]+(-0.407791315587)*x[1]**o+(0.86069036983)*x[1]
        arg[0,2,0,0]=(-0.453860220598)*x[0]**o+(0.944457388061)*x[0]+(-0.903818960162)*x[1]**o+(-0.799423205297)*x[1]
        arg[0,2,0,1]=(-0.621257500941)*x[0]**o+(0.748963737937)*x[0]+(-0.12660934458)*x[1]**o+(-0.362830639592)*x[1]
        arg[0,2,1,0]=(-0.702485820037)*x[0]**o+(0.355584898593)*x[0]+(0.657127357751)*x[1]**o+(0.535533390762)*x[1]
        arg[0,2,1,1]=(0.795733125008)*x[0]**o+(0.983607327249)*x[0]+(-0.463818780644)*x[1]**o+(-0.681831588244)*x[1]
        arg[0,2,2,0]=(0.335783204138)*x[0]**o+(-0.712225782679)*x[0]+(0.853426644096)*x[1]**o+(-0.143845263686)*x[1]
        arg[0,2,2,1]=(0.389724375144)*x[0]**o+(0.715176227705)*x[0]+(-0.660690893888)*x[1]**o+(0.428289724709)*x[1]
        arg[0,3,0,0]=(-0.363836203724)*x[0]**o+(-0.257775427457)*x[0]+(0.407904889098)*x[1]**o+(-0.731273814284)*x[1]
        arg[0,3,0,1]=(0.402038262494)*x[0]**o+(0.33289103638)*x[0]+(0.444957562329)*x[1]**o+(0.0657971935664)*x[1]
        arg[0,3,1,0]=(0.979673814602)*x[0]**o+(-0.133405332848)*x[0]+(0.23884689304)*x[1]**o+(0.728464997681)*x[1]
        arg[0,3,1,1]=(0.724603429246)*x[0]**o+(-0.391040898144)*x[0]+(-0.966816499003)*x[1]**o+(0.0984302578905)*x[1]
        arg[0,3,2,0]=(-0.986139645754)*x[0]**o+(0.60807351165)*x[0]+(0.799807413231)*x[1]**o+(0.12091479711)*x[1]
        arg[0,3,2,1]=(0.387453999377)*x[0]**o+(0.212725006628)*x[0]+(-0.120374039139)*x[1]**o+(-0.262304757951)*x[1]
        arg[0,4,0,0]=(-0.260545986276)*x[0]**o+(0.251085312932)*x[0]+(0.738082708518)*x[1]**o+(-0.0804883797938)*x[1]
        arg[0,4,0,1]=(-0.190701356138)*x[0]**o+(-0.0205295935522)*x[0]+(0.904261681819)*x[1]**o+(-0.859647709957)*x[1]
        arg[0,4,1,0]=(-0.945357492849)*x[0]**o+(-0.85298463389)*x[0]+(0.688964336673)*x[1]**o+(-0.672881696932)*x[1]
        arg[0,4,1,1]=(0.877720258045)*x[0]**o+(-0.361228026805)*x[0]+(-0.326626527006)*x[1]**o+(-0.274100421232)*x[1]
        arg[0,4,2,0]=(0.202358729084)*x[0]**o+(-0.997571481837)*x[0]+(0.431193711862)*x[1]**o+(0.513561123915)*x[1]
        arg[0,4,2,1]=(-0.136602954592)*x[0]**o+(-0.40864934437)*x[0]+(0.675945574351)*x[1]**o+(0.728757793536)*x[1]
        arg[1,0,0,0]=(0.835464261851)*x[0]**o+(-0.131447020601)*x[0]+(0.0810352521446)*x[1]**o+(0.664744750899)*x[1]
        arg[1,0,0,1]=(0.239716928689)*x[0]**o+(-0.192258999531)*x[0]+(0.615877054036)*x[1]**o+(0.835274774437)*x[1]
        arg[1,0,1,0]=(0.509342803767)*x[0]**o+(-0.633944481882)*x[0]+(0.262098006086)*x[1]**o+(0.159902070799)*x[1]
        arg[1,0,1,1]=(0.894432440588)*x[0]**o+(-0.167778431319)*x[0]+(-0.984183148477)*x[1]**o+(-0.554981658632)*x[1]
        arg[1,0,2,0]=(-0.762472607231)*x[0]**o+(0.405550139256)*x[0]+(0.4051290641)*x[1]**o+(-0.27180186672)*x[1]
        arg[1,0,2,1]=(0.663869273816)*x[0]**o+(0.867453974628)*x[0]+(-0.279659969132)*x[1]**o+(-0.370539101204)*x[1]
        arg[1,1,0,0]=(-0.969326778164)*x[0]**o+(0.965471349497)*x[0]+(-0.529983232946)*x[1]**o+(0.714091610977)*x[1]
        arg[1,1,0,1]=(-0.840360667093)*x[0]**o+(-0.399319808256)*x[0]+(0.568769915582)*x[1]**o+(0.872427494442)*x[1]
        arg[1,1,1,0]=(0.46099116941)*x[0]**o+(-0.986008874142)*x[0]+(-0.53999023556)*x[1]**o+(-0.549695567855)*x[1]
        arg[1,1,1,1]=(0.485944918559)*x[0]**o+(0.189773828651)*x[0]+(-0.611654980493)*x[1]**o+(-0.337071911657)*x[1]
        arg[1,1,2,0]=(0.191663603248)*x[0]**o+(0.0906374965004)*x[0]+(-0.271051598251)*x[1]**o+(0.880507788825)*x[1]
        arg[1,1,2,1]=(-0.439514997001)*x[0]**o+(0.701230426224)*x[0]+(-0.193231641029)*x[1]**o+(0.950584054332)*x[1]
        arg[1,2,0,0]=(0.439456565259)*x[0]**o+(-0.25664171484)*x[0]+(-0.653059819329)*x[1]**o+(-0.680544747777)*x[1]
        arg[1,2,0,1]=(0.34890454443)*x[0]**o+(-0.69851200963)*x[0]+(-0.00159381688549)*x[1]**o+(-0.612359675499)*x[1]
        arg[1,2,1,0]=(0.132759671375)*x[0]**o+(-0.235126716401)*x[0]+(0.652068885826)*x[1]**o+(0.749698645371)*x[1]
        arg[1,2,1,1]=(0.934980403627)*x[0]**o+(0.321409273437)*x[0]+(-0.605909929553)*x[1]**o+(0.135554927418)*x[1]
        arg[1,2,2,0]=(-0.68327709329)*x[0]**o+(-0.0381292756005)*x[0]+(0.710953191114)*x[1]**o+(-0.344522256689)*x[1]
        arg[1,2,2,1]=(-0.813976235652)*x[0]**o+(0.613879708123)*x[0]+(0.451599277293)*x[1]**o+(-0.6134026375)*x[1]
        arg[1,3,0,0]=(0.595310128627)*x[0]**o+(-0.247621573131)*x[0]+(0.165607783104)*x[1]**o+(0.578132180805)*x[1]
        arg[1,3,0,1]=(0.848827238882)*x[0]**o+(0.173154067999)*x[0]+(0.340662419437)*x[1]**o+(-0.389656447251)*x[1]
        arg[1,3,1,0]=(0.250888415228)*x[0]**o+(-0.278203136198)*x[0]+(-0.693211186734)*x[1]**o+(0.613812194735)*x[1]
        arg[1,3,1,1]=(0.634533003162)*x[0]**o+(-0.0924665762471)*x[0]+(0.749575894384)*x[1]**o+(-0.737205017415)*x[1]
        arg[1,3,2,0]=(0.607750966973)*x[0]**o+(0.905016928404)*x[0]+(-0.171049542163)*x[1]**o+(-0.20526244601)*x[1]
        arg[1,3,2,1]=(0.723946025405)*x[0]**o+(0.0157576743987)*x[0]+(0.190875428312)*x[1]**o+(0.407925899509)*x[1]
        arg[1,4,0,0]=(-0.561582695803)*x[0]**o+(-0.350262461371)*x[0]+(0.57610409872)*x[1]**o+(0.881780963877)*x[1]
        arg[1,4,0,1]=(0.077066498695)*x[0]**o+(0.515597802516)*x[0]+(0.332011413016)*x[1]**o+(0.215491334787)*x[1]
        arg[1,4,1,0]=(0.33858727737)*x[0]**o+(-0.573231800768)*x[0]+(-0.559915809132)*x[1]**o+(0.0240869142176)*x[1]
        arg[1,4,1,1]=(-0.684581727952)*x[0]**o+(0.365541904201)*x[0]+(0.204252397376)*x[1]**o+(0.149300766054)*x[1]
        arg[1,4,2,0]=(-0.574225088383)*x[0]**o+(0.0286339776067)*x[0]+(-0.358732122607)*x[1]**o+(-0.416932738957)*x[1]
        arg[1,4,2,1]=(-0.308460184932)*x[0]**o+(0.389214987836)*x[0]+(-0.358303257031)*x[1]**o+(0.348109483688)*x[1]
        arg[2,0,0,0]=(-0.811318668804)*x[0]**o+(-0.298028116982)*x[0]+(0.679871523483)*x[1]**o+(0.958042925287)*x[1]
        arg[2,0,0,1]=(-0.616521818338)*x[0]**o+(-0.0817124854496)*x[0]+(0.403257366893)*x[1]**o+(0.740689710435)*x[1]
        arg[2,0,1,0]=(0.271046179127)*x[0]**o+(0.814928093947)*x[0]+(0.423722781579)*x[1]**o+(0.925333209811)*x[1]
        arg[2,0,1,1]=(0.0596651806816)*x[0]**o+(0.493854605359)*x[0]+(0.539998537439)*x[1]**o+(-0.910865639375)*x[1]
        arg[2,0,2,0]=(0.503267602327)*x[0]**o+(-0.270966471743)*x[0]+(-0.342063366676)*x[1]**o+(-0.975700224705)*x[1]
        arg[2,0,2,1]=(-0.0362358939195)*x[0]**o+(-0.766033869339)*x[0]+(0.891365100088)*x[1]**o+(0.815025898929)*x[1]
        arg[2,1,0,0]=(-0.667240459984)*x[0]**o+(0.8659076701)*x[0]+(0.215089397562)*x[1]**o+(0.464769182307)*x[1]
        arg[2,1,0,1]=(-0.298625221931)*x[0]**o+(-0.504329743472)*x[0]+(0.412472510208)*x[1]**o+(0.241693152889)*x[1]
        arg[2,1,1,0]=(-0.670730179133)*x[0]**o+(0.0201070113832)*x[0]+(-0.572725898919)*x[1]**o+(0.665609259014)*x[1]
        arg[2,1,1,1]=(-0.319080984089)*x[0]**o+(0.0299423045224)*x[0]+(0.0152311810613)*x[1]**o+(-0.784646189026)*x[1]
        arg[2,1,2,0]=(0.153308789716)*x[0]**o+(0.725808150613)*x[0]+(0.62876493338)*x[1]**o+(0.80275284252)*x[1]
        arg[2,1,2,1]=(0.804698870013)*x[0]**o+(-0.631078773061)*x[0]+(-0.015221975698)*x[1]**o+(-0.954169686746)*x[1]
        arg[2,2,0,0]=(0.169022797498)*x[0]**o+(0.733025791099)*x[0]+(0.321554996633)*x[1]**o+(0.292173365364)*x[1]
        arg[2,2,0,1]=(0.327345955109)*x[0]**o+(0.21757823896)*x[0]+(0.873027160446)*x[1]**o+(-0.250192861579)*x[1]
        arg[2,2,1,0]=(0.394387190832)*x[0]**o+(-0.613661883482)*x[0]+(0.806965291386)*x[1]**o+(0.244513244016)*x[1]
        arg[2,2,1,1]=(0.043913907536)*x[0]**o+(-0.303567806424)*x[0]+(0.547037093323)*x[1]**o+(-0.598687385328)*x[1]
        arg[2,2,2,0]=(0.875663154787)*x[0]**o+(-0.34984832772)*x[0]+(0.879143174396)*x[1]**o+(-0.193154051109)*x[1]
        arg[2,2,2,1]=(0.940311132918)*x[0]**o+(-0.358708743382)*x[0]+(0.117370945146)*x[1]**o+(-0.664208034757)*x[1]
        arg[2,3,0,0]=(0.0334525660588)*x[0]**o+(0.592805915255)*x[0]+(-0.278216157971)*x[1]**o+(0.894369967671)*x[1]
        arg[2,3,0,1]=(-0.966591106326)*x[0]**o+(-0.882160390768)*x[0]+(0.843982702732)*x[1]**o+(0.363868921656)*x[1]
        arg[2,3,1,0]=(-0.246392342771)*x[0]**o+(-0.934459096242)*x[0]+(-0.472771368537)*x[1]**o+(0.148854304706)*x[1]
        arg[2,3,1,1]=(0.248218424051)*x[0]**o+(0.0980396576111)*x[0]+(0.413322166489)*x[1]**o+(0.429747261346)*x[1]
        arg[2,3,2,0]=(0.742987565733)*x[0]**o+(-0.213736450136)*x[0]+(0.996975741314)*x[1]**o+(0.960709958665)*x[1]
        arg[2,3,2,1]=(0.436149490977)*x[0]**o+(0.87632181197)*x[0]+(-0.881934669912)*x[1]**o+(-0.457520504175)*x[1]
        arg[2,4,0,0]=(-0.519014330693)*x[0]**o+(0.938619608407)*x[0]+(-0.847954563609)*x[1]**o+(0.791774379621)*x[1]
        arg[2,4,0,1]=(-0.599487299453)*x[0]**o+(0.977867555132)*x[0]+(0.693105630579)*x[1]**o+(-0.688827812942)*x[1]
        arg[2,4,1,0]=(0.955907964163)*x[0]**o+(0.0724056986172)*x[0]+(0.254773619772)*x[1]**o+(0.0428861146964)*x[1]
        arg[2,4,1,1]=(-0.422202101998)*x[0]**o+(0.803263223544)*x[0]+(0.456048140785)*x[1]**o+(0.0419133126183)*x[1]
        arg[2,4,2,0]=(-0.182308704781)*x[0]**o+(0.862282443488)*x[0]+(-0.844027007955)*x[1]**o+(0.586220526376)*x[1]
        arg[2,4,2,1]=(-0.801542666229)*x[0]**o+(0.903685299455)*x[0]+(0.428926544377)*x[1]**o+(-0.891220471549)*x[1]
        arg[3,0,0,0]=(0.602917787713)*x[0]**o+(-0.592239549743)*x[0]+(0.761830006718)*x[1]**o+(-0.784264167777)*x[1]
        arg[3,0,0,1]=(0.819962957732)*x[0]**o+(-0.455126790372)*x[0]+(-0.144898728525)*x[1]**o+(0.121566345339)*x[1]
        arg[3,0,1,0]=(0.570694941559)*x[0]**o+(-0.505678656386)*x[0]+(0.295536587355)*x[1]**o+(-0.852779737814)*x[1]
        arg[3,0,1,1]=(-0.226246858753)*x[0]**o+(-0.513900306228)*x[0]+(-0.389483591407)*x[1]**o+(-0.735105335539)*x[1]
        arg[3,0,2,0]=(0.941877610938)*x[0]**o+(0.93808330298)*x[0]+(0.189785162182)*x[1]**o+(0.546525366304)*x[1]
        arg[3,0,2,1]=(0.10639100485)*x[0]**o+(-0.773361007972)*x[0]+(-0.470531427579)*x[1]**o+(0.467359342808)*x[1]
        arg[3,1,0,0]=(0.857725599589)*x[0]**o+(0.0881746413132)*x[0]+(0.040713151152)*x[1]**o+(-0.52070729883)*x[1]
        arg[3,1,0,1]=(0.266807652146)*x[0]**o+(0.337496673892)*x[0]+(-0.562266297445)*x[1]**o+(0.326015195113)*x[1]
        arg[3,1,1,0]=(0.177275509465)*x[0]**o+(-0.647855761301)*x[0]+(-0.0963373631995)*x[1]**o+(-0.535256252994)*x[1]
        arg[3,1,1,1]=(0.430309295606)*x[0]**o+(-0.0894749557428)*x[0]+(0.503470724483)*x[1]**o+(-0.202361122248)*x[1]
        arg[3,1,2,0]=(0.283995014194)*x[0]**o+(-0.544419907194)*x[0]+(0.78478571712)*x[1]**o+(0.341540664147)*x[1]
        arg[3,1,2,1]=(-0.281042869936)*x[0]**o+(0.728771766492)*x[0]+(-0.298974388557)*x[1]**o+(-0.836789906985)*x[1]
        arg[3,2,0,0]=(-0.415323371561)*x[0]**o+(0.583971137371)*x[0]+(-0.265658679489)*x[1]**o+(-0.432447373809)*x[1]
        arg[3,2,0,1]=(0.690486298855)*x[0]**o+(0.552521632226)*x[0]+(0.237622739042)*x[1]**o+(-0.254513693924)*x[1]
        arg[3,2,1,0]=(0.453985687207)*x[0]**o+(-0.785385580962)*x[0]+(0.0190987841634)*x[1]**o+(-0.7923288861)*x[1]
        arg[3,2,1,1]=(-0.964082656379)*x[0]**o+(-0.806315937371)*x[0]+(0.271145420366)*x[1]**o+(-0.839014535945)*x[1]
        arg[3,2,2,0]=(-0.372062773997)*x[0]**o+(-0.069749560567)*x[0]+(-0.126310395383)*x[1]**o+(0.972025865285)*x[1]
        arg[3,2,2,1]=(-0.580353020086)*x[0]**o+(-0.152746821477)*x[0]+(0.297553660535)*x[1]**o+(0.444791568145)*x[1]
        arg[3,3,0,0]=(-0.361520029439)*x[0]**o+(-0.810391070499)*x[0]+(0.462123882642)*x[1]**o+(-0.974534227742)*x[1]
        arg[3,3,0,1]=(0.113370617837)*x[0]**o+(0.428257355435)*x[0]+(-0.18656623758)*x[1]**o+(0.206503327394)*x[1]
        arg[3,3,1,0]=(0.341328445689)*x[0]**o+(0.34002947054)*x[0]+(0.997677286362)*x[1]**o+(0.43300073225)*x[1]
        arg[3,3,1,1]=(-0.513912982823)*x[0]**o+(0.826573827064)*x[0]+(-0.701417864085)*x[1]**o+(0.567078883219)*x[1]
        arg[3,3,2,0]=(-0.325536866965)*x[0]**o+(0.924405767451)*x[0]+(-0.0481381954346)*x[1]**o+(0.912210617073)*x[1]
        arg[3,3,2,1]=(0.491532434302)*x[0]**o+(-0.584980848036)*x[0]+(-0.788317530858)*x[1]**o+(-0.900681893172)*x[1]
        arg[3,4,0,0]=(0.432249855148)*x[0]**o+(-0.673369312552)*x[0]+(0.244704845849)*x[1]**o+(-0.300995595367)*x[1]
        arg[3,4,0,1]=(0.383037211984)*x[0]**o+(0.299711400488)*x[0]+(0.870584255454)*x[1]**o+(-0.33571823329)*x[1]
        arg[3,4,1,0]=(0.737134180177)*x[0]**o+(-0.101307090628)*x[0]+(-0.970632061143)*x[1]**o+(-0.449042684137)*x[1]
        arg[3,4,1,1]=(0.00684250643632)*x[0]**o+(0.676964654022)*x[0]+(0.789069834858)*x[1]**o+(-0.829739508197)*x[1]
        arg[3,4,2,0]=(0.7279044944)*x[0]**o+(0.215711122038)*x[0]+(0.385631252632)*x[1]**o+(0.629390433529)*x[1]
        arg[3,4,2,1]=(-0.380750716629)*x[0]**o+(-0.413040692437)*x[0]+(0.26795244639)*x[1]**o+(0.492475504659)*x[1]
        ref[0,0,0,0]=(-0.274472293956)*x_ref[0]**o+(-0.293347130929)*x_ref[0]+(0.392586448254)*x_ref[1]**o+(-0.403066854054)*x_ref[1]
        ref[0,0,0,1]=(0.385933192907)*x_ref[0]**o+(0.484240065867)*x_ref[0]+(0.144565631551)*x_ref[1]**o+(0.882508492739)*x_ref[1]
        ref[0,0,1,0]=(0.993368905137)*x_ref[0]**o+(0.340349919343)*x_ref[0]+(-0.956806746524)*x_ref[1]**o+(0.990607958001)*x_ref[1]
        ref[0,0,1,1]=(-0.88610250102)*x_ref[0]**o+(-0.543705587259)*x_ref[0]+(0.170798164924)*x_ref[1]**o+(0.685331195754)*x_ref[1]
        ref[0,0,2,0]=(-0.922211611143)*x_ref[0]**o+(-0.103640037893)*x_ref[0]+(0.401354703434)*x_ref[1]**o+(-0.117826261635)*x_ref[1]
        ref[0,0,2,1]=(-0.391601387132)*x_ref[0]**o+(-0.472753344804)*x_ref[0]+(-0.473433838993)*x_ref[1]**o+(-0.173501635989)*x_ref[1]
        ref[0,1,0,0]=(0.747353274916)*x_ref[0]**o+(-0.960856612086)*x_ref[0]+(-0.54809012978)*x_ref[1]**o+(0.862023893332)*x_ref[1]
        ref[0,1,0,1]=(-0.256359001149)*x_ref[0]**o+(-0.868975044191)*x_ref[0]+(0.494314269744)*x_ref[1]**o+(0.949957481172)*x_ref[1]
        ref[0,1,1,0]=(-0.0256658750601)*x_ref[0]**o+(0.145499104203)*x_ref[0]+(-0.68669791629)*x_ref[1]**o+(0.447326207295)*x_ref[1]
        ref[0,1,1,1]=(0.150617284467)*x_ref[0]**o+(-0.350046088196)*x_ref[0]+(-0.367671302053)*x_ref[1]**o+(0.633220122068)*x_ref[1]
        ref[0,1,2,0]=(-0.714145362271)*x_ref[0]**o+(-0.957661490758)*x_ref[0]+(0.386253770747)*x_ref[1]**o+(0.982634262634)*x_ref[1]
        ref[0,1,2,1]=(0.707565096824)*x_ref[0]**o+(0.281758058573)*x_ref[0]+(-0.407791315587)*x_ref[1]**o+(0.86069036983)*x_ref[1]
        ref[0,2,0,0]=(-0.453860220598)*x_ref[0]**o+(0.944457388061)*x_ref[0]+(-0.903818960162)*x_ref[1]**o+(-0.799423205297)*x_ref[1]
        ref[0,2,0,1]=(-0.621257500941)*x_ref[0]**o+(0.748963737937)*x_ref[0]+(-0.12660934458)*x_ref[1]**o+(-0.362830639592)*x_ref[1]
        ref[0,2,1,0]=(-0.702485820037)*x_ref[0]**o+(0.355584898593)*x_ref[0]+(0.657127357751)*x_ref[1]**o+(0.535533390762)*x_ref[1]
        ref[0,2,1,1]=(0.795733125008)*x_ref[0]**o+(0.983607327249)*x_ref[0]+(-0.463818780644)*x_ref[1]**o+(-0.681831588244)*x_ref[1]
        ref[0,2,2,0]=(0.335783204138)*x_ref[0]**o+(-0.712225782679)*x_ref[0]+(0.853426644096)*x_ref[1]**o+(-0.143845263686)*x_ref[1]
        ref[0,2,2,1]=(0.389724375144)*x_ref[0]**o+(0.715176227705)*x_ref[0]+(-0.660690893888)*x_ref[1]**o+(0.428289724709)*x_ref[1]
        ref[0,3,0,0]=(-0.363836203724)*x_ref[0]**o+(-0.257775427457)*x_ref[0]+(0.407904889098)*x_ref[1]**o+(-0.731273814284)*x_ref[1]
        ref[0,3,0,1]=(0.402038262494)*x_ref[0]**o+(0.33289103638)*x_ref[0]+(0.444957562329)*x_ref[1]**o+(0.0657971935664)*x_ref[1]
        ref[0,3,1,0]=(0.979673814602)*x_ref[0]**o+(-0.133405332848)*x_ref[0]+(0.23884689304)*x_ref[1]**o+(0.728464997681)*x_ref[1]
        ref[0,3,1,1]=(0.724603429246)*x_ref[0]**o+(-0.391040898144)*x_ref[0]+(-0.966816499003)*x_ref[1]**o+(0.0984302578905)*x_ref[1]
        ref[0,3,2,0]=(-0.986139645754)*x_ref[0]**o+(0.60807351165)*x_ref[0]+(0.799807413231)*x_ref[1]**o+(0.12091479711)*x_ref[1]
        ref[0,3,2,1]=(0.387453999377)*x_ref[0]**o+(0.212725006628)*x_ref[0]+(-0.120374039139)*x_ref[1]**o+(-0.262304757951)*x_ref[1]
        ref[0,4,0,0]=(-0.260545986276)*x_ref[0]**o+(0.251085312932)*x_ref[0]+(0.738082708518)*x_ref[1]**o+(-0.0804883797938)*x_ref[1]
        ref[0,4,0,1]=(-0.190701356138)*x_ref[0]**o+(-0.0205295935522)*x_ref[0]+(0.904261681819)*x_ref[1]**o+(-0.859647709957)*x_ref[1]
        ref[0,4,1,0]=(-0.945357492849)*x_ref[0]**o+(-0.85298463389)*x_ref[0]+(0.688964336673)*x_ref[1]**o+(-0.672881696932)*x_ref[1]
        ref[0,4,1,1]=(0.877720258045)*x_ref[0]**o+(-0.361228026805)*x_ref[0]+(-0.326626527006)*x_ref[1]**o+(-0.274100421232)*x_ref[1]
        ref[0,4,2,0]=(0.202358729084)*x_ref[0]**o+(-0.997571481837)*x_ref[0]+(0.431193711862)*x_ref[1]**o+(0.513561123915)*x_ref[1]
        ref[0,4,2,1]=(-0.136602954592)*x_ref[0]**o+(-0.40864934437)*x_ref[0]+(0.675945574351)*x_ref[1]**o+(0.728757793536)*x_ref[1]
        ref[1,0,0,0]=(0.835464261851)*x_ref[0]**o+(-0.131447020601)*x_ref[0]+(0.0810352521446)*x_ref[1]**o+(0.664744750899)*x_ref[1]
        ref[1,0,0,1]=(0.239716928689)*x_ref[0]**o+(-0.192258999531)*x_ref[0]+(0.615877054036)*x_ref[1]**o+(0.835274774437)*x_ref[1]
        ref[1,0,1,0]=(0.509342803767)*x_ref[0]**o+(-0.633944481882)*x_ref[0]+(0.262098006086)*x_ref[1]**o+(0.159902070799)*x_ref[1]
        ref[1,0,1,1]=(0.894432440588)*x_ref[0]**o+(-0.167778431319)*x_ref[0]+(-0.984183148477)*x_ref[1]**o+(-0.554981658632)*x_ref[1]
        ref[1,0,2,0]=(-0.762472607231)*x_ref[0]**o+(0.405550139256)*x_ref[0]+(0.4051290641)*x_ref[1]**o+(-0.27180186672)*x_ref[1]
        ref[1,0,2,1]=(0.663869273816)*x_ref[0]**o+(0.867453974628)*x_ref[0]+(-0.279659969132)*x_ref[1]**o+(-0.370539101204)*x_ref[1]
        ref[1,1,0,0]=(-0.969326778164)*x_ref[0]**o+(0.965471349497)*x_ref[0]+(-0.529983232946)*x_ref[1]**o+(0.714091610977)*x_ref[1]
        ref[1,1,0,1]=(-0.840360667093)*x_ref[0]**o+(-0.399319808256)*x_ref[0]+(0.568769915582)*x_ref[1]**o+(0.872427494442)*x_ref[1]
        ref[1,1,1,0]=(0.46099116941)*x_ref[0]**o+(-0.986008874142)*x_ref[0]+(-0.53999023556)*x_ref[1]**o+(-0.549695567855)*x_ref[1]
        ref[1,1,1,1]=(0.485944918559)*x_ref[0]**o+(0.189773828651)*x_ref[0]+(-0.611654980493)*x_ref[1]**o+(-0.337071911657)*x_ref[1]
        ref[1,1,2,0]=(0.191663603248)*x_ref[0]**o+(0.0906374965004)*x_ref[0]+(-0.271051598251)*x_ref[1]**o+(0.880507788825)*x_ref[1]
        ref[1,1,2,1]=(-0.439514997001)*x_ref[0]**o+(0.701230426224)*x_ref[0]+(-0.193231641029)*x_ref[1]**o+(0.950584054332)*x_ref[1]
        ref[1,2,0,0]=(0.439456565259)*x_ref[0]**o+(-0.25664171484)*x_ref[0]+(-0.653059819329)*x_ref[1]**o+(-0.680544747777)*x_ref[1]
        ref[1,2,0,1]=(0.34890454443)*x_ref[0]**o+(-0.69851200963)*x_ref[0]+(-0.00159381688549)*x_ref[1]**o+(-0.612359675499)*x_ref[1]
        ref[1,2,1,0]=(0.132759671375)*x_ref[0]**o+(-0.235126716401)*x_ref[0]+(0.652068885826)*x_ref[1]**o+(0.749698645371)*x_ref[1]
        ref[1,2,1,1]=(0.934980403627)*x_ref[0]**o+(0.321409273437)*x_ref[0]+(-0.605909929553)*x_ref[1]**o+(0.135554927418)*x_ref[1]
        ref[1,2,2,0]=(-0.68327709329)*x_ref[0]**o+(-0.0381292756005)*x_ref[0]+(0.710953191114)*x_ref[1]**o+(-0.344522256689)*x_ref[1]
        ref[1,2,2,1]=(-0.813976235652)*x_ref[0]**o+(0.613879708123)*x_ref[0]+(0.451599277293)*x_ref[1]**o+(-0.6134026375)*x_ref[1]
        ref[1,3,0,0]=(0.595310128627)*x_ref[0]**o+(-0.247621573131)*x_ref[0]+(0.165607783104)*x_ref[1]**o+(0.578132180805)*x_ref[1]
        ref[1,3,0,1]=(0.848827238882)*x_ref[0]**o+(0.173154067999)*x_ref[0]+(0.340662419437)*x_ref[1]**o+(-0.389656447251)*x_ref[1]
        ref[1,3,1,0]=(0.250888415228)*x_ref[0]**o+(-0.278203136198)*x_ref[0]+(-0.693211186734)*x_ref[1]**o+(0.613812194735)*x_ref[1]
        ref[1,3,1,1]=(0.634533003162)*x_ref[0]**o+(-0.0924665762471)*x_ref[0]+(0.749575894384)*x_ref[1]**o+(-0.737205017415)*x_ref[1]
        ref[1,3,2,0]=(0.607750966973)*x_ref[0]**o+(0.905016928404)*x_ref[0]+(-0.171049542163)*x_ref[1]**o+(-0.20526244601)*x_ref[1]
        ref[1,3,2,1]=(0.723946025405)*x_ref[0]**o+(0.0157576743987)*x_ref[0]+(0.190875428312)*x_ref[1]**o+(0.407925899509)*x_ref[1]
        ref[1,4,0,0]=(-0.561582695803)*x_ref[0]**o+(-0.350262461371)*x_ref[0]+(0.57610409872)*x_ref[1]**o+(0.881780963877)*x_ref[1]
        ref[1,4,0,1]=(0.077066498695)*x_ref[0]**o+(0.515597802516)*x_ref[0]+(0.332011413016)*x_ref[1]**o+(0.215491334787)*x_ref[1]
        ref[1,4,1,0]=(0.33858727737)*x_ref[0]**o+(-0.573231800768)*x_ref[0]+(-0.559915809132)*x_ref[1]**o+(0.0240869142176)*x_ref[1]
        ref[1,4,1,1]=(-0.684581727952)*x_ref[0]**o+(0.365541904201)*x_ref[0]+(0.204252397376)*x_ref[1]**o+(0.149300766054)*x_ref[1]
        ref[1,4,2,0]=(-0.574225088383)*x_ref[0]**o+(0.0286339776067)*x_ref[0]+(-0.358732122607)*x_ref[1]**o+(-0.416932738957)*x_ref[1]
        ref[1,4,2,1]=(-0.308460184932)*x_ref[0]**o+(0.389214987836)*x_ref[0]+(-0.358303257031)*x_ref[1]**o+(0.348109483688)*x_ref[1]
        ref[2,0,0,0]=(-0.811318668804)*x_ref[0]**o+(-0.298028116982)*x_ref[0]+(0.679871523483)*x_ref[1]**o+(0.958042925287)*x_ref[1]
        ref[2,0,0,1]=(-0.616521818338)*x_ref[0]**o+(-0.0817124854496)*x_ref[0]+(0.403257366893)*x_ref[1]**o+(0.740689710435)*x_ref[1]
        ref[2,0,1,0]=(0.271046179127)*x_ref[0]**o+(0.814928093947)*x_ref[0]+(0.423722781579)*x_ref[1]**o+(0.925333209811)*x_ref[1]
        ref[2,0,1,1]=(0.0596651806816)*x_ref[0]**o+(0.493854605359)*x_ref[0]+(0.539998537439)*x_ref[1]**o+(-0.910865639375)*x_ref[1]
        ref[2,0,2,0]=(0.503267602327)*x_ref[0]**o+(-0.270966471743)*x_ref[0]+(-0.342063366676)*x_ref[1]**o+(-0.975700224705)*x_ref[1]
        ref[2,0,2,1]=(-0.0362358939195)*x_ref[0]**o+(-0.766033869339)*x_ref[0]+(0.891365100088)*x_ref[1]**o+(0.815025898929)*x_ref[1]
        ref[2,1,0,0]=(-0.667240459984)*x_ref[0]**o+(0.8659076701)*x_ref[0]+(0.215089397562)*x_ref[1]**o+(0.464769182307)*x_ref[1]
        ref[2,1,0,1]=(-0.298625221931)*x_ref[0]**o+(-0.504329743472)*x_ref[0]+(0.412472510208)*x_ref[1]**o+(0.241693152889)*x_ref[1]
        ref[2,1,1,0]=(-0.670730179133)*x_ref[0]**o+(0.0201070113832)*x_ref[0]+(-0.572725898919)*x_ref[1]**o+(0.665609259014)*x_ref[1]
        ref[2,1,1,1]=(-0.319080984089)*x_ref[0]**o+(0.0299423045224)*x_ref[0]+(0.0152311810613)*x_ref[1]**o+(-0.784646189026)*x_ref[1]
        ref[2,1,2,0]=(0.153308789716)*x_ref[0]**o+(0.725808150613)*x_ref[0]+(0.62876493338)*x_ref[1]**o+(0.80275284252)*x_ref[1]
        ref[2,1,2,1]=(0.804698870013)*x_ref[0]**o+(-0.631078773061)*x_ref[0]+(-0.015221975698)*x_ref[1]**o+(-0.954169686746)*x_ref[1]
        ref[2,2,0,0]=(0.169022797498)*x_ref[0]**o+(0.733025791099)*x_ref[0]+(0.321554996633)*x_ref[1]**o+(0.292173365364)*x_ref[1]
        ref[2,2,0,1]=(0.327345955109)*x_ref[0]**o+(0.21757823896)*x_ref[0]+(0.873027160446)*x_ref[1]**o+(-0.250192861579)*x_ref[1]
        ref[2,2,1,0]=(0.394387190832)*x_ref[0]**o+(-0.613661883482)*x_ref[0]+(0.806965291386)*x_ref[1]**o+(0.244513244016)*x_ref[1]
        ref[2,2,1,1]=(0.043913907536)*x_ref[0]**o+(-0.303567806424)*x_ref[0]+(0.547037093323)*x_ref[1]**o+(-0.598687385328)*x_ref[1]
        ref[2,2,2,0]=(0.875663154787)*x_ref[0]**o+(-0.34984832772)*x_ref[0]+(0.879143174396)*x_ref[1]**o+(-0.193154051109)*x_ref[1]
        ref[2,2,2,1]=(0.940311132918)*x_ref[0]**o+(-0.358708743382)*x_ref[0]+(0.117370945146)*x_ref[1]**o+(-0.664208034757)*x_ref[1]
        ref[2,3,0,0]=(0.0334525660588)*x_ref[0]**o+(0.592805915255)*x_ref[0]+(-0.278216157971)*x_ref[1]**o+(0.894369967671)*x_ref[1]
        ref[2,3,0,1]=(-0.966591106326)*x_ref[0]**o+(-0.882160390768)*x_ref[0]+(0.843982702732)*x_ref[1]**o+(0.363868921656)*x_ref[1]
        ref[2,3,1,0]=(-0.246392342771)*x_ref[0]**o+(-0.934459096242)*x_ref[0]+(-0.472771368537)*x_ref[1]**o+(0.148854304706)*x_ref[1]
        ref[2,3,1,1]=(0.248218424051)*x_ref[0]**o+(0.0980396576111)*x_ref[0]+(0.413322166489)*x_ref[1]**o+(0.429747261346)*x_ref[1]
        ref[2,3,2,0]=(0.742987565733)*x_ref[0]**o+(-0.213736450136)*x_ref[0]+(0.996975741314)*x_ref[1]**o+(0.960709958665)*x_ref[1]
        ref[2,3,2,1]=(0.436149490977)*x_ref[0]**o+(0.87632181197)*x_ref[0]+(-0.881934669912)*x_ref[1]**o+(-0.457520504175)*x_ref[1]
        ref[2,4,0,0]=(-0.519014330693)*x_ref[0]**o+(0.938619608407)*x_ref[0]+(-0.847954563609)*x_ref[1]**o+(0.791774379621)*x_ref[1]
        ref[2,4,0,1]=(-0.599487299453)*x_ref[0]**o+(0.977867555132)*x_ref[0]+(0.693105630579)*x_ref[1]**o+(-0.688827812942)*x_ref[1]
        ref[2,4,1,0]=(0.955907964163)*x_ref[0]**o+(0.0724056986172)*x_ref[0]+(0.254773619772)*x_ref[1]**o+(0.0428861146964)*x_ref[1]
        ref[2,4,1,1]=(-0.422202101998)*x_ref[0]**o+(0.803263223544)*x_ref[0]+(0.456048140785)*x_ref[1]**o+(0.0419133126183)*x_ref[1]
        ref[2,4,2,0]=(-0.182308704781)*x_ref[0]**o+(0.862282443488)*x_ref[0]+(-0.844027007955)*x_ref[1]**o+(0.586220526376)*x_ref[1]
        ref[2,4,2,1]=(-0.801542666229)*x_ref[0]**o+(0.903685299455)*x_ref[0]+(0.428926544377)*x_ref[1]**o+(-0.891220471549)*x_ref[1]
        ref[3,0,0,0]=(0.602917787713)*x_ref[0]**o+(-0.592239549743)*x_ref[0]+(0.761830006718)*x_ref[1]**o+(-0.784264167777)*x_ref[1]
        ref[3,0,0,1]=(0.819962957732)*x_ref[0]**o+(-0.455126790372)*x_ref[0]+(-0.144898728525)*x_ref[1]**o+(0.121566345339)*x_ref[1]
        ref[3,0,1,0]=(0.570694941559)*x_ref[0]**o+(-0.505678656386)*x_ref[0]+(0.295536587355)*x_ref[1]**o+(-0.852779737814)*x_ref[1]
        ref[3,0,1,1]=(-0.226246858753)*x_ref[0]**o+(-0.513900306228)*x_ref[0]+(-0.389483591407)*x_ref[1]**o+(-0.735105335539)*x_ref[1]
        ref[3,0,2,0]=(0.941877610938)*x_ref[0]**o+(0.93808330298)*x_ref[0]+(0.189785162182)*x_ref[1]**o+(0.546525366304)*x_ref[1]
        ref[3,0,2,1]=(0.10639100485)*x_ref[0]**o+(-0.773361007972)*x_ref[0]+(-0.470531427579)*x_ref[1]**o+(0.467359342808)*x_ref[1]
        ref[3,1,0,0]=(0.857725599589)*x_ref[0]**o+(0.0881746413132)*x_ref[0]+(0.040713151152)*x_ref[1]**o+(-0.52070729883)*x_ref[1]
        ref[3,1,0,1]=(0.266807652146)*x_ref[0]**o+(0.337496673892)*x_ref[0]+(-0.562266297445)*x_ref[1]**o+(0.326015195113)*x_ref[1]
        ref[3,1,1,0]=(0.177275509465)*x_ref[0]**o+(-0.647855761301)*x_ref[0]+(-0.0963373631995)*x_ref[1]**o+(-0.535256252994)*x_ref[1]
        ref[3,1,1,1]=(0.430309295606)*x_ref[0]**o+(-0.0894749557428)*x_ref[0]+(0.503470724483)*x_ref[1]**o+(-0.202361122248)*x_ref[1]
        ref[3,1,2,0]=(0.283995014194)*x_ref[0]**o+(-0.544419907194)*x_ref[0]+(0.78478571712)*x_ref[1]**o+(0.341540664147)*x_ref[1]
        ref[3,1,2,1]=(-0.281042869936)*x_ref[0]**o+(0.728771766492)*x_ref[0]+(-0.298974388557)*x_ref[1]**o+(-0.836789906985)*x_ref[1]
        ref[3,2,0,0]=(-0.415323371561)*x_ref[0]**o+(0.583971137371)*x_ref[0]+(-0.265658679489)*x_ref[1]**o+(-0.432447373809)*x_ref[1]
        ref[3,2,0,1]=(0.690486298855)*x_ref[0]**o+(0.552521632226)*x_ref[0]+(0.237622739042)*x_ref[1]**o+(-0.254513693924)*x_ref[1]
        ref[3,2,1,0]=(0.453985687207)*x_ref[0]**o+(-0.785385580962)*x_ref[0]+(0.0190987841634)*x_ref[1]**o+(-0.7923288861)*x_ref[1]
        ref[3,2,1,1]=(-0.964082656379)*x_ref[0]**o+(-0.806315937371)*x_ref[0]+(0.271145420366)*x_ref[1]**o+(-0.839014535945)*x_ref[1]
        ref[3,2,2,0]=(-0.372062773997)*x_ref[0]**o+(-0.069749560567)*x_ref[0]+(-0.126310395383)*x_ref[1]**o+(0.972025865285)*x_ref[1]
        ref[3,2,2,1]=(-0.580353020086)*x_ref[0]**o+(-0.152746821477)*x_ref[0]+(0.297553660535)*x_ref[1]**o+(0.444791568145)*x_ref[1]
        ref[3,3,0,0]=(-0.361520029439)*x_ref[0]**o+(-0.810391070499)*x_ref[0]+(0.462123882642)*x_ref[1]**o+(-0.974534227742)*x_ref[1]
        ref[3,3,0,1]=(0.113370617837)*x_ref[0]**o+(0.428257355435)*x_ref[0]+(-0.18656623758)*x_ref[1]**o+(0.206503327394)*x_ref[1]
        ref[3,3,1,0]=(0.341328445689)*x_ref[0]**o+(0.34002947054)*x_ref[0]+(0.997677286362)*x_ref[1]**o+(0.43300073225)*x_ref[1]
        ref[3,3,1,1]=(-0.513912982823)*x_ref[0]**o+(0.826573827064)*x_ref[0]+(-0.701417864085)*x_ref[1]**o+(0.567078883219)*x_ref[1]
        ref[3,3,2,0]=(-0.325536866965)*x_ref[0]**o+(0.924405767451)*x_ref[0]+(-0.0481381954346)*x_ref[1]**o+(0.912210617073)*x_ref[1]
        ref[3,3,2,1]=(0.491532434302)*x_ref[0]**o+(-0.584980848036)*x_ref[0]+(-0.788317530858)*x_ref[1]**o+(-0.900681893172)*x_ref[1]
        ref[3,4,0,0]=(0.432249855148)*x_ref[0]**o+(-0.673369312552)*x_ref[0]+(0.244704845849)*x_ref[1]**o+(-0.300995595367)*x_ref[1]
        ref[3,4,0,1]=(0.383037211984)*x_ref[0]**o+(0.299711400488)*x_ref[0]+(0.870584255454)*x_ref[1]**o+(-0.33571823329)*x_ref[1]
        ref[3,4,1,0]=(0.737134180177)*x_ref[0]**o+(-0.101307090628)*x_ref[0]+(-0.970632061143)*x_ref[1]**o+(-0.449042684137)*x_ref[1]
        ref[3,4,1,1]=(0.00684250643632)*x_ref[0]**o+(0.676964654022)*x_ref[0]+(0.789069834858)*x_ref[1]**o+(-0.829739508197)*x_ref[1]
        ref[3,4,2,0]=(0.7279044944)*x_ref[0]**o+(0.215711122038)*x_ref[0]+(0.385631252632)*x_ref[1]**o+(0.629390433529)*x_ref[1]
        ref[3,4,2,1]=(-0.380750716629)*x_ref[0]**o+(-0.413040692437)*x_ref[0]+(0.26795244639)*x_ref[1]**o+(0.492475504659)*x_ref[1]
      else:
        arg[0,0,0,0]=(0.287160004495)*x[0]**o+(0.35112250158)*x[0]+(0.061693427026)*x[1]**o+(-0.752505628029)*x[1]+(-0.57137756028)*x[2]**o+(-0.0397598892504)*x[2]
        arg[0,0,0,1]=(0.272799584607)*x[0]**o+(-0.569947956933)*x[0]+(0.322189261657)*x[1]**o+(-0.440552272224)*x[1]+(-0.10460241481)*x[2]**o+(-0.884848984752)*x[2]
        arg[0,0,1,0]=(0.147766301604)*x[0]**o+(0.405651654428)*x[0]+(-0.440827209529)*x[1]**o+(-0.399952864271)*x[1]+(-0.770191009459)*x[2]**o+(-0.789853968049)*x[2]
        arg[0,0,1,1]=(0.688053165375)*x[0]**o+(-0.154890421924)*x[0]+(0.583684164319)*x[1]**o+(-0.398082068217)*x[1]+(-0.573142923099)*x[2]**o+(-0.205271877337)*x[2]
        arg[0,0,2,0]=(0.807043275877)*x[0]**o+(0.843046383231)*x[0]+(-0.588146471402)*x[1]**o+(0.157787304557)*x[1]+(-0.771009748568)*x[2]**o+(0.162090791609)*x[2]
        arg[0,0,2,1]=(-0.0706800489392)*x[0]**o+(0.499961344665)*x[0]+(0.208926178797)*x[1]**o+(-0.7462472769)*x[1]+(-0.105475405648)*x[2]**o+(-0.18670229948)*x[2]
        arg[0,1,0,0]=(-0.696657281943)*x[0]**o+(-0.0251785345759)*x[0]+(-0.579575530451)*x[1]**o+(-0.0628378665725)*x[1]+(0.668395441302)*x[2]**o+(0.813478105092)*x[2]
        arg[0,1,0,1]=(-0.574784314876)*x[0]**o+(-0.321268045204)*x[0]+(0.160704464213)*x[1]**o+(-0.676016748103)*x[1]+(0.918971102865)*x[2]**o+(0.252082410081)*x[2]
        arg[0,1,1,0]=(-0.905062846817)*x[0]**o+(0.788906658477)*x[0]+(-0.408807611936)*x[1]**o+(0.114686186437)*x[1]+(-0.654685568493)*x[2]**o+(0.0879202139254)*x[2]
        arg[0,1,1,1]=(-0.68020231443)*x[0]**o+(-0.82049202902)*x[0]+(-0.958352740187)*x[1]**o+(-0.33201745802)*x[1]+(0.539810363682)*x[2]**o+(0.744988573087)*x[2]
        arg[0,1,2,0]=(-0.960884744997)*x[0]**o+(-0.704157132503)*x[0]+(0.282236905414)*x[1]**o+(0.756363456433)*x[1]+(-0.365400101792)*x[2]**o+(0.975735846554)*x[2]
        arg[0,1,2,1]=(-0.691780076951)*x[0]**o+(-0.952575440059)*x[0]+(-0.921247808098)*x[1]**o+(-0.88872834095)*x[1]+(0.534103013833)*x[2]**o+(-0.602295186104)*x[2]
        arg[0,2,0,0]=(0.460698415704)*x[0]**o+(0.770243814897)*x[0]+(-0.380381727574)*x[1]**o+(0.476360845668)*x[1]+(-0.681040425102)*x[2]**o+(-0.244369435116)*x[2]
        arg[0,2,0,1]=(0.251591652281)*x[0]**o+(0.748234919686)*x[0]+(-0.698189037307)*x[1]**o+(-0.292400412508)*x[1]+(0.613850696577)*x[2]**o+(-0.942243226834)*x[2]
        arg[0,2,1,0]=(-0.718090066328)*x[0]**o+(-0.322499511794)*x[0]+(-0.75105623397)*x[1]**o+(0.526680271832)*x[1]+(-0.186671531793)*x[2]**o+(-0.449273875991)*x[2]
        arg[0,2,1,1]=(-0.52165552452)*x[0]**o+(0.902446006085)*x[0]+(-0.915575172702)*x[1]**o+(-0.150816194662)*x[1]+(0.596782785689)*x[2]**o+(0.546880865642)*x[2]
        arg[0,2,2,0]=(-0.104486704816)*x[0]**o+(0.723863699457)*x[0]+(0.758622117279)*x[1]**o+(0.372271580145)*x[1]+(-0.0820570383792)*x[2]**o+(-0.63539781104)*x[2]
        arg[0,2,2,1]=(0.0355930823603)*x[0]**o+(0.0931942600432)*x[0]+(-0.118096985978)*x[1]**o+(0.296983782212)*x[1]+(-0.864491763712)*x[2]**o+(0.942230663978)*x[2]
        arg[0,3,0,0]=(0.371153709815)*x[0]**o+(0.0427309083893)*x[0]+(-0.886563919084)*x[1]**o+(-0.14946513404)*x[1]+(0.80903804519)*x[2]**o+(0.69728533073)*x[2]
        arg[0,3,0,1]=(0.0593930396142)*x[0]**o+(-0.370327974089)*x[0]+(-0.388354464683)*x[1]**o+(-0.367648771812)*x[1]+(-0.573754821458)*x[2]**o+(-0.947499155479)*x[2]
        arg[0,3,1,0]=(-0.869817317)*x[0]**o+(0.874317772678)*x[0]+(-0.870060178375)*x[1]**o+(0.66571074524)*x[1]+(-0.669052377378)*x[2]**o+(0.940212680575)*x[2]
        arg[0,3,1,1]=(-0.417445606772)*x[0]**o+(0.942606057619)*x[0]+(-0.133936034409)*x[1]**o+(-0.363274280866)*x[1]+(0.83487439935)*x[2]**o+(-0.108398981461)*x[2]
        arg[0,3,2,0]=(0.363623383293)*x[0]**o+(0.624267309217)*x[0]+(0.801719984899)*x[1]**o+(0.189212370122)*x[1]+(0.464633630928)*x[2]**o+(-0.979808318438)*x[2]
        arg[0,3,2,1]=(0.85384408805)*x[0]**o+(-0.889214754116)*x[0]+(0.164915391224)*x[1]**o+(0.0168991405201)*x[1]+(0.097171994236)*x[2]**o+(-0.146094197339)*x[2]
        arg[0,4,0,0]=(0.778356158972)*x[0]**o+(0.910241265149)*x[0]+(0.123690863272)*x[1]**o+(0.87802121779)*x[1]+(-0.714615668595)*x[2]**o+(-0.437472831583)*x[2]
        arg[0,4,0,1]=(-0.157185481783)*x[0]**o+(0.352802163968)*x[0]+(0.390184123612)*x[1]**o+(-0.680415093098)*x[1]+(-0.121195042454)*x[2]**o+(-0.713693712117)*x[2]
        arg[0,4,1,0]=(0.778115668646)*x[0]**o+(-0.111729042892)*x[0]+(0.451518002735)*x[1]**o+(0.232731183374)*x[1]+(-0.764246158086)*x[2]**o+(-0.795525155328)*x[2]
        arg[0,4,1,1]=(-0.880652367027)*x[0]**o+(-0.314966079126)*x[0]+(-0.640720373065)*x[1]**o+(0.336230541901)*x[1]+(0.566956999526)*x[2]**o+(0.74098258811)*x[2]
        arg[0,4,2,0]=(0.908657274301)*x[0]**o+(0.636427823614)*x[0]+(0.650137414386)*x[1]**o+(0.695558379005)*x[1]+(-0.489852401087)*x[2]**o+(-0.479227108044)*x[2]
        arg[0,4,2,1]=(0.015734241525)*x[0]**o+(0.0719884727088)*x[0]+(0.811864895261)*x[1]**o+(0.959127986784)*x[1]+(-0.238146663368)*x[2]**o+(-0.454939322571)*x[2]
        arg[1,0,0,0]=(-0.918218246559)*x[0]**o+(-0.662787169269)*x[0]+(-0.954035144111)*x[1]**o+(-0.647264685228)*x[1]+(-0.834085960213)*x[2]**o+(-0.167541951024)*x[2]
        arg[1,0,0,1]=(-0.670950869382)*x[0]**o+(0.826472159989)*x[0]+(0.180410872786)*x[1]**o+(-0.715162871594)*x[1]+(0.653287853169)*x[2]**o+(0.892060529625)*x[2]
        arg[1,0,1,0]=(0.398789786407)*x[0]**o+(0.689408033554)*x[0]+(0.124338245128)*x[1]**o+(0.748280808587)*x[1]+(0.418357122274)*x[2]**o+(0.230707440075)*x[2]
        arg[1,0,1,1]=(-0.912778964055)*x[0]**o+(0.904288708694)*x[0]+(-0.874744856901)*x[1]**o+(0.550162195719)*x[1]+(-0.7526423373)*x[2]**o+(0.820993231654)*x[2]
        arg[1,0,2,0]=(-0.843459684648)*x[0]**o+(-0.579708764091)*x[0]+(-0.876072131849)*x[1]**o+(0.933694599696)*x[1]+(0.507781209021)*x[2]**o+(-0.719729482133)*x[2]
        arg[1,0,2,1]=(0.594566925796)*x[0]**o+(-0.994301292358)*x[0]+(0.808114389564)*x[1]**o+(-0.970803712864)*x[1]+(0.433544910389)*x[2]**o+(0.260659440299)*x[2]
        arg[1,1,0,0]=(0.997377553172)*x[0]**o+(0.458428616383)*x[0]+(0.357280778493)*x[1]**o+(0.349837110741)*x[1]+(-0.709592214445)*x[2]**o+(-0.630000739033)*x[2]
        arg[1,1,0,1]=(0.121595650501)*x[0]**o+(0.40870120924)*x[0]+(0.576917123807)*x[1]**o+(-0.932268308325)*x[1]+(0.710043472116)*x[2]**o+(-0.716528412077)*x[2]
        arg[1,1,1,0]=(-0.151416035033)*x[0]**o+(0.0255347649787)*x[0]+(-0.752987701889)*x[1]**o+(-0.480969875813)*x[1]+(-0.610514504179)*x[2]**o+(0.959274757402)*x[2]
        arg[1,1,1,1]=(0.76695706106)*x[0]**o+(-0.0750965975512)*x[0]+(0.846833900735)*x[1]**o+(-0.105838465439)*x[1]+(0.541027486457)*x[2]**o+(0.152543017589)*x[2]
        arg[1,1,2,0]=(0.0856977181865)*x[0]**o+(-0.188139369706)*x[0]+(0.936071565289)*x[1]**o+(-0.3958713036)*x[1]+(-0.350945878893)*x[2]**o+(0.382302674883)*x[2]
        arg[1,1,2,1]=(0.873054234789)*x[0]**o+(-0.490869716146)*x[0]+(0.46720526212)*x[1]**o+(-0.334203412334)*x[1]+(-0.81140895636)*x[2]**o+(-0.973858971749)*x[2]
        arg[1,2,0,0]=(-0.716676352443)*x[0]**o+(0.659821396277)*x[0]+(0.335964292954)*x[1]**o+(0.11609664773)*x[1]+(-0.889186749466)*x[2]**o+(0.454231973946)*x[2]
        arg[1,2,0,1]=(0.367048872648)*x[0]**o+(0.276083713126)*x[0]+(0.686639128043)*x[1]**o+(-0.991414415293)*x[1]+(0.25360515964)*x[2]**o+(-0.377623315093)*x[2]
        arg[1,2,1,0]=(-0.723089358777)*x[0]**o+(-0.516690112406)*x[0]+(0.262334273979)*x[1]**o+(0.140773502537)*x[1]+(0.447333972396)*x[2]**o+(-0.551395692966)*x[2]
        arg[1,2,1,1]=(0.738791182491)*x[0]**o+(-0.978120626668)*x[0]+(-0.217970433713)*x[1]**o+(0.0181011067492)*x[1]+(0.756322867523)*x[2]**o+(0.495269195978)*x[2]
        arg[1,2,2,0]=(-0.630506449854)*x[0]**o+(-0.0865482364231)*x[0]+(-0.415896403157)*x[1]**o+(-0.712833149895)*x[1]+(-0.180482789777)*x[2]**o+(0.00788290661235)*x[2]
        arg[1,2,2,1]=(0.0946913865377)*x[0]**o+(0.752854827716)*x[0]+(0.257846220843)*x[1]**o+(-0.953926983578)*x[1]+(-0.388505920682)*x[2]**o+(-0.10751401251)*x[2]
        arg[1,3,0,0]=(0.694014030611)*x[0]**o+(0.0433103547797)*x[0]+(-0.0805025225569)*x[1]**o+(0.389052254754)*x[1]+(-0.813210939614)*x[2]**o+(-0.593788787896)*x[2]
        arg[1,3,0,1]=(0.0101606353406)*x[0]**o+(-0.849874258855)*x[0]+(-0.0578973959381)*x[1]**o+(0.942113140424)*x[1]+(0.613716459216)*x[2]**o+(0.72631176833)*x[2]
        arg[1,3,1,0]=(0.264703608811)*x[0]**o+(0.457962548326)*x[0]+(-0.433795242673)*x[1]**o+(0.650995712848)*x[1]+(0.372471807967)*x[2]**o+(0.00416636841047)*x[2]
        arg[1,3,1,1]=(-0.73686451669)*x[0]**o+(-0.0425614910776)*x[0]+(-0.587820431311)*x[1]**o+(-0.377628617833)*x[1]+(-0.463964014406)*x[2]**o+(0.283273874989)*x[2]
        arg[1,3,2,0]=(-0.689341439143)*x[0]**o+(-0.675861918889)*x[0]+(-0.918622662352)*x[1]**o+(0.740990261888)*x[1]+(0.0454007677041)*x[2]**o+(0.513623361074)*x[2]
        arg[1,3,2,1]=(0.13229101848)*x[0]**o+(0.171873799817)*x[0]+(-0.154491174434)*x[1]**o+(0.897976884089)*x[1]+(-0.598886311821)*x[2]**o+(0.738696710918)*x[2]
        arg[1,4,0,0]=(0.59403636966)*x[0]**o+(0.93856848969)*x[0]+(0.80164849167)*x[1]**o+(-0.753322979568)*x[1]+(-0.107098073916)*x[2]**o+(-0.623047487973)*x[2]
        arg[1,4,0,1]=(0.291046099082)*x[0]**o+(-0.437982471049)*x[0]+(0.017395762947)*x[1]**o+(0.852440593923)*x[1]+(0.630825424359)*x[2]**o+(0.419347491727)*x[2]
        arg[1,4,1,0]=(-0.747962012701)*x[0]**o+(-0.11622032094)*x[0]+(0.324950549781)*x[1]**o+(0.950911398327)*x[1]+(-0.18098990205)*x[2]**o+(-0.676060633522)*x[2]
        arg[1,4,1,1]=(0.545224363128)*x[0]**o+(0.753376367121)*x[0]+(0.271426695753)*x[1]**o+(-0.139309844675)*x[1]+(-0.923434716375)*x[2]**o+(-0.985988324389)*x[2]
        arg[1,4,2,0]=(-0.532260822047)*x[0]**o+(0.901407135724)*x[0]+(0.937451330141)*x[1]**o+(0.285217666559)*x[1]+(-0.133028984824)*x[2]**o+(0.993965133696)*x[2]
        arg[1,4,2,1]=(0.2231815607)*x[0]**o+(0.994845537582)*x[0]+(0.0368727378195)*x[1]**o+(0.511772933777)*x[1]+(-0.871580215976)*x[2]**o+(-0.587010057329)*x[2]
        arg[2,0,0,0]=(-0.55149411913)*x[0]**o+(-0.905541278559)*x[0]+(0.461217373096)*x[1]**o+(-0.53359853241)*x[1]+(-0.666643290883)*x[2]**o+(0.151495826926)*x[2]
        arg[2,0,0,1]=(-0.980485062148)*x[0]**o+(0.150165616162)*x[0]+(0.59165725487)*x[1]**o+(-0.243853536017)*x[1]+(0.66816261686)*x[2]**o+(0.2425696131)*x[2]
        arg[2,0,1,0]=(-0.509337213758)*x[0]**o+(0.652936706687)*x[0]+(-0.849214338091)*x[1]**o+(-0.523354123549)*x[1]+(0.371515689887)*x[2]**o+(0.799989815414)*x[2]
        arg[2,0,1,1]=(0.461584544541)*x[0]**o+(0.662931355954)*x[0]+(0.744415634786)*x[1]**o+(-0.016743801256)*x[1]+(0.172945846325)*x[2]**o+(-0.0889827089341)*x[2]
        arg[2,0,2,0]=(0.506514791562)*x[0]**o+(0.974477160694)*x[0]+(-0.968485343167)*x[1]**o+(0.427480013599)*x[1]+(0.0460755879358)*x[2]**o+(0.186500994948)*x[2]
        arg[2,0,2,1]=(0.259831607993)*x[0]**o+(-0.0141290444125)*x[0]+(-0.968452177682)*x[1]**o+(0.635619125009)*x[1]+(-0.958848431827)*x[2]**o+(-0.0977241523308)*x[2]
        arg[2,1,0,0]=(0.204233222933)*x[0]**o+(0.834739381877)*x[0]+(0.19961185048)*x[1]**o+(-0.238470380578)*x[1]+(-0.749979713773)*x[2]**o+(-0.268168886251)*x[2]
        arg[2,1,0,1]=(0.998123643088)*x[0]**o+(0.635823733041)*x[0]+(-0.265447124419)*x[1]**o+(-0.225582751946)*x[1]+(0.91705937439)*x[2]**o+(-0.739461510792)*x[2]
        arg[2,1,1,0]=(0.0800715288646)*x[0]**o+(-0.37677196659)*x[0]+(-0.0737286350219)*x[1]**o+(0.032076062933)*x[1]+(0.245421912709)*x[2]**o+(-0.8378023935)*x[2]
        arg[2,1,1,1]=(-0.572376055603)*x[0]**o+(0.8675468469)*x[0]+(-0.533012889867)*x[1]**o+(-0.247215658297)*x[1]+(0.673144564693)*x[2]**o+(-0.749136108985)*x[2]
        arg[2,1,2,0]=(-0.661188192488)*x[0]**o+(-0.924435593008)*x[0]+(-0.791845808916)*x[1]**o+(0.00133765351092)*x[1]+(0.233676862873)*x[2]**o+(0.981812737962)*x[2]
        arg[2,1,2,1]=(0.332072445986)*x[0]**o+(-0.909373798588)*x[0]+(0.927025647819)*x[1]**o+(-0.936348942617)*x[1]+(0.251508592152)*x[2]**o+(-0.468147973987)*x[2]
        arg[2,2,0,0]=(0.892377893841)*x[0]**o+(-0.147933293091)*x[0]+(0.950491102076)*x[1]**o+(0.595250355926)*x[1]+(0.0169764649992)*x[2]**o+(-0.683137752586)*x[2]
        arg[2,2,0,1]=(0.00579397005645)*x[0]**o+(0.627197373917)*x[0]+(0.663574756196)*x[1]**o+(0.934106767513)*x[1]+(0.105844878022)*x[2]**o+(0.513322917688)*x[2]
        arg[2,2,1,0]=(-0.168776016133)*x[0]**o+(-0.940071612944)*x[0]+(0.0366053554566)*x[1]**o+(-0.628010909771)*x[1]+(0.250023045906)*x[2]**o+(-0.00327322332166)*x[2]
        arg[2,2,1,1]=(0.904155764376)*x[0]**o+(-0.258018307306)*x[0]+(0.65862820521)*x[1]**o+(0.434480798515)*x[1]+(0.4089419173)*x[2]**o+(-0.518032695598)*x[2]
        arg[2,2,2,0]=(-0.41470070958)*x[0]**o+(-0.783186751952)*x[0]+(0.548309561942)*x[1]**o+(0.837130592803)*x[1]+(-0.346808491848)*x[2]**o+(0.165416040985)*x[2]
        arg[2,2,2,1]=(0.997845909195)*x[0]**o+(0.448428042158)*x[0]+(-0.657050004311)*x[1]**o+(0.493104851747)*x[1]+(0.558315071197)*x[2]**o+(0.206659432923)*x[2]
        arg[2,3,0,0]=(0.433874360232)*x[0]**o+(-0.0635646197131)*x[0]+(-0.521690521523)*x[1]**o+(-0.298130684888)*x[1]+(-0.131905313644)*x[2]**o+(0.64389451112)*x[2]
        arg[2,3,0,1]=(0.457729315216)*x[0]**o+(-0.419323242135)*x[0]+(0.614544051135)*x[1]**o+(-0.701671999275)*x[1]+(0.905494300727)*x[2]**o+(-0.565716985337)*x[2]
        arg[2,3,1,0]=(0.256570728097)*x[0]**o+(0.866349285103)*x[0]+(-0.115768354896)*x[1]**o+(-0.613044456549)*x[1]+(-0.20074185156)*x[2]**o+(-0.188403571776)*x[2]
        arg[2,3,1,1]=(0.767479943414)*x[0]**o+(0.884066215007)*x[0]+(0.363855420056)*x[1]**o+(0.919698981104)*x[1]+(0.225747762317)*x[2]**o+(-0.220362271104)*x[2]
        arg[2,3,2,0]=(-0.746469501382)*x[0]**o+(0.680520398001)*x[0]+(0.926950996325)*x[1]**o+(-0.122831303044)*x[1]+(-0.27226561675)*x[2]**o+(0.933304087951)*x[2]
        arg[2,3,2,1]=(-0.0961434747216)*x[0]**o+(0.987660516767)*x[0]+(0.288175205045)*x[1]**o+(0.683720820403)*x[1]+(0.354403143642)*x[2]**o+(-0.745694089132)*x[2]
        arg[2,4,0,0]=(0.921750835986)*x[0]**o+(-0.0358861598394)*x[0]+(0.190129645259)*x[1]**o+(0.377067803331)*x[1]+(-0.0188615236582)*x[2]**o+(0.804533888824)*x[2]
        arg[2,4,0,1]=(-0.256132234484)*x[0]**o+(-0.102661411017)*x[0]+(-0.903465157009)*x[1]**o+(0.50133310168)*x[1]+(-0.0889102598619)*x[2]**o+(-0.0310174238365)*x[2]
        arg[2,4,1,0]=(0.58483798855)*x[0]**o+(0.967943344305)*x[0]+(-0.0701080005438)*x[1]**o+(0.489476104406)*x[1]+(0.615308427038)*x[2]**o+(-0.38065505709)*x[2]
        arg[2,4,1,1]=(0.647734130724)*x[0]**o+(-0.930934392719)*x[0]+(0.267370540491)*x[1]**o+(-0.0175019424556)*x[1]+(0.38593657129)*x[2]**o+(-0.985922292359)*x[2]
        arg[2,4,2,0]=(-0.289971479933)*x[0]**o+(0.882014313157)*x[0]+(0.147321067579)*x[1]**o+(-0.176771838489)*x[1]+(0.710916531544)*x[2]**o+(-0.0856290170401)*x[2]
        arg[2,4,2,1]=(-0.465930997733)*x[0]**o+(-0.305563942094)*x[0]+(-0.824535262061)*x[1]**o+(-0.917892226972)*x[1]+(0.624462871955)*x[2]**o+(0.544736500065)*x[2]
        arg[3,0,0,0]=(-0.673230815097)*x[0]**o+(-0.726292712917)*x[0]+(0.0531037021688)*x[1]**o+(-0.467219030159)*x[1]+(-0.0729861369718)*x[2]**o+(-0.440394335015)*x[2]
        arg[3,0,0,1]=(-0.358235295558)*x[0]**o+(-0.301184713671)*x[0]+(-0.496030649086)*x[1]**o+(-0.125922184777)*x[1]+(0.946715036973)*x[2]**o+(0.846370337893)*x[2]
        arg[3,0,1,0]=(-0.394175109612)*x[0]**o+(-0.85137582307)*x[0]+(-0.667804785044)*x[1]**o+(0.156762109891)*x[1]+(-0.616444700525)*x[2]**o+(0.0738729399356)*x[2]
        arg[3,0,1,1]=(-0.652875462534)*x[0]**o+(-0.0605650192541)*x[0]+(0.840539933823)*x[1]**o+(-0.422819423301)*x[1]+(0.113440226819)*x[2]**o+(-0.915154532306)*x[2]
        arg[3,0,2,0]=(-0.162863352328)*x[0]**o+(-0.364461332292)*x[0]+(-0.222181741398)*x[1]**o+(-0.563668008365)*x[1]+(0.118491770105)*x[2]**o+(0.0809155053022)*x[2]
        arg[3,0,2,1]=(-0.985278929437)*x[0]**o+(-0.919440760157)*x[0]+(-0.776910686)*x[1]**o+(-0.884270488943)*x[1]+(0.984279010363)*x[2]**o+(-0.489503278582)*x[2]
        arg[3,1,0,0]=(-0.806565449823)*x[0]**o+(-0.0851385375281)*x[0]+(-0.88851381773)*x[1]**o+(-0.666539867528)*x[1]+(0.154825531411)*x[2]**o+(0.764069849144)*x[2]
        arg[3,1,0,1]=(-0.222125783918)*x[0]**o+(0.568035675467)*x[0]+(0.470171778932)*x[1]**o+(-0.307608710881)*x[1]+(-0.836083742471)*x[2]**o+(0.553597553381)*x[2]
        arg[3,1,1,0]=(0.431237389847)*x[0]**o+(-0.141391012471)*x[0]+(0.726895025519)*x[1]**o+(-0.527868751679)*x[1]+(0.711761083137)*x[2]**o+(0.996992864103)*x[2]
        arg[3,1,1,1]=(-0.123885959924)*x[0]**o+(0.877034475529)*x[0]+(-0.0432492682928)*x[1]**o+(-0.370315494618)*x[1]+(0.243139719071)*x[2]**o+(-0.684318634902)*x[2]
        arg[3,1,2,0]=(-0.477214489355)*x[0]**o+(0.340808576619)*x[0]+(-0.0991802306558)*x[1]**o+(0.462599821908)*x[1]+(0.091768281564)*x[2]**o+(0.0315552838839)*x[2]
        arg[3,1,2,1]=(0.676189142894)*x[0]**o+(0.510136386913)*x[0]+(-0.558579041952)*x[1]**o+(0.961615047877)*x[1]+(0.510717386124)*x[2]**o+(0.774430183432)*x[2]
        arg[3,2,0,0]=(0.158370232444)*x[0]**o+(0.209424855992)*x[0]+(0.691438753013)*x[1]**o+(-0.781594298738)*x[1]+(-0.0188536627517)*x[2]**o+(0.136400798995)*x[2]
        arg[3,2,0,1]=(-0.440674414328)*x[0]**o+(0.118228612687)*x[0]+(-0.700026492613)*x[1]**o+(-0.981341215714)*x[1]+(-0.172847519578)*x[2]**o+(-0.423095202172)*x[2]
        arg[3,2,1,0]=(-0.0694960391695)*x[0]**o+(-0.66448610135)*x[0]+(-0.177572703748)*x[1]**o+(-0.060488161727)*x[1]+(0.306021853885)*x[2]**o+(0.0730338716187)*x[2]
        arg[3,2,1,1]=(-0.360662154547)*x[0]**o+(0.577679359648)*x[0]+(-0.515313534966)*x[1]**o+(0.217352687358)*x[1]+(0.728367577882)*x[2]**o+(0.38505849834)*x[2]
        arg[3,2,2,0]=(0.568671861232)*x[0]**o+(-0.0792490917882)*x[0]+(0.0188002885229)*x[1]**o+(-0.956862575759)*x[1]+(-0.844332442983)*x[2]**o+(0.670792928042)*x[2]
        arg[3,2,2,1]=(0.833099204713)*x[0]**o+(0.368482169933)*x[0]+(-0.926002498892)*x[1]**o+(-0.130646446568)*x[1]+(0.81164928914)*x[2]**o+(-0.641079919927)*x[2]
        arg[3,3,0,0]=(-0.277769350989)*x[0]**o+(0.753452018649)*x[0]+(-0.766322344505)*x[1]**o+(-0.679108224612)*x[1]+(0.0799499795424)*x[2]**o+(0.255751308975)*x[2]
        arg[3,3,0,1]=(0.429176753733)*x[0]**o+(0.123106050259)*x[0]+(-0.169037728992)*x[1]**o+(0.749245933687)*x[1]+(0.560140859355)*x[2]**o+(-0.937091840021)*x[2]
        arg[3,3,1,0]=(0.648792557593)*x[0]**o+(-0.057555187513)*x[0]+(0.883290331726)*x[1]**o+(0.539276905995)*x[1]+(0.473614408583)*x[2]**o+(0.723258340333)*x[2]
        arg[3,3,1,1]=(0.527298833572)*x[0]**o+(0.0753824058279)*x[0]+(0.719196099257)*x[1]**o+(-0.0124603350035)*x[1]+(-0.964484650253)*x[2]**o+(0.404444269694)*x[2]
        arg[3,3,2,0]=(-0.403640677462)*x[0]**o+(0.906195964766)*x[0]+(-0.561419078404)*x[1]**o+(0.645408717838)*x[1]+(0.671624799609)*x[2]**o+(-0.573901868621)*x[2]
        arg[3,3,2,1]=(-0.288117182165)*x[0]**o+(-0.391003584945)*x[0]+(-0.555037182038)*x[1]**o+(0.299362209017)*x[1]+(-0.552429724326)*x[2]**o+(-0.416923306774)*x[2]
        arg[3,4,0,0]=(-0.19594719192)*x[0]**o+(0.150898934692)*x[0]+(0.210935232044)*x[1]**o+(-0.319764293347)*x[1]+(-0.642450751195)*x[2]**o+(0.22231085168)*x[2]
        arg[3,4,0,1]=(-0.0514589460225)*x[0]**o+(0.0366025646754)*x[0]+(0.416280576691)*x[1]**o+(0.609178063951)*x[1]+(0.835611677062)*x[2]**o+(-0.131630571478)*x[2]
        arg[3,4,1,0]=(0.171509152214)*x[0]**o+(-0.761188271093)*x[0]+(-0.915951802896)*x[1]**o+(-0.451796709915)*x[1]+(-0.979847828162)*x[2]**o+(0.0566453393315)*x[2]
        arg[3,4,1,1]=(0.690427998102)*x[0]**o+(-0.358318625695)*x[0]+(0.330270871613)*x[1]**o+(-0.724392522137)*x[1]+(-0.535474623489)*x[2]**o+(-0.12341743885)*x[2]
        arg[3,4,2,0]=(0.488090753426)*x[0]**o+(-0.352417078372)*x[0]+(0.971988099319)*x[1]**o+(-0.288929516009)*x[1]+(0.612576734094)*x[2]**o+(-0.43799514603)*x[2]
        arg[3,4,2,1]=(-0.098768101356)*x[0]**o+(0.123300661895)*x[0]+(-0.0489724280789)*x[1]**o+(-0.422508669137)*x[1]+(-0.861483104139)*x[2]**o+(0.950198316001)*x[2]
        ref[0,0,0,0]=(0.287160004495)*x_ref[0]**o+(0.35112250158)*x_ref[0]+(0.061693427026)*x_ref[1]**o+(-0.752505628029)*x_ref[1]+(-0.57137756028)*x_ref[2]**o+(-0.0397598892504)*x_ref[2]
        ref[0,0,0,1]=(0.272799584607)*x_ref[0]**o+(-0.569947956933)*x_ref[0]+(0.322189261657)*x_ref[1]**o+(-0.440552272224)*x_ref[1]+(-0.10460241481)*x_ref[2]**o+(-0.884848984752)*x_ref[2]
        ref[0,0,1,0]=(0.147766301604)*x_ref[0]**o+(0.405651654428)*x_ref[0]+(-0.440827209529)*x_ref[1]**o+(-0.399952864271)*x_ref[1]+(-0.770191009459)*x_ref[2]**o+(-0.789853968049)*x_ref[2]
        ref[0,0,1,1]=(0.688053165375)*x_ref[0]**o+(-0.154890421924)*x_ref[0]+(0.583684164319)*x_ref[1]**o+(-0.398082068217)*x_ref[1]+(-0.573142923099)*x_ref[2]**o+(-0.205271877337)*x_ref[2]
        ref[0,0,2,0]=(0.807043275877)*x_ref[0]**o+(0.843046383231)*x_ref[0]+(-0.588146471402)*x_ref[1]**o+(0.157787304557)*x_ref[1]+(-0.771009748568)*x_ref[2]**o+(0.162090791609)*x_ref[2]
        ref[0,0,2,1]=(-0.0706800489392)*x_ref[0]**o+(0.499961344665)*x_ref[0]+(0.208926178797)*x_ref[1]**o+(-0.7462472769)*x_ref[1]+(-0.105475405648)*x_ref[2]**o+(-0.18670229948)*x_ref[2]
        ref[0,1,0,0]=(-0.696657281943)*x_ref[0]**o+(-0.0251785345759)*x_ref[0]+(-0.579575530451)*x_ref[1]**o+(-0.0628378665725)*x_ref[1]+(0.668395441302)*x_ref[2]**o+(0.813478105092)*x_ref[2]
        ref[0,1,0,1]=(-0.574784314876)*x_ref[0]**o+(-0.321268045204)*x_ref[0]+(0.160704464213)*x_ref[1]**o+(-0.676016748103)*x_ref[1]+(0.918971102865)*x_ref[2]**o+(0.252082410081)*x_ref[2]
        ref[0,1,1,0]=(-0.905062846817)*x_ref[0]**o+(0.788906658477)*x_ref[0]+(-0.408807611936)*x_ref[1]**o+(0.114686186437)*x_ref[1]+(-0.654685568493)*x_ref[2]**o+(0.0879202139254)*x_ref[2]
        ref[0,1,1,1]=(-0.68020231443)*x_ref[0]**o+(-0.82049202902)*x_ref[0]+(-0.958352740187)*x_ref[1]**o+(-0.33201745802)*x_ref[1]+(0.539810363682)*x_ref[2]**o+(0.744988573087)*x_ref[2]
        ref[0,1,2,0]=(-0.960884744997)*x_ref[0]**o+(-0.704157132503)*x_ref[0]+(0.282236905414)*x_ref[1]**o+(0.756363456433)*x_ref[1]+(-0.365400101792)*x_ref[2]**o+(0.975735846554)*x_ref[2]
        ref[0,1,2,1]=(-0.691780076951)*x_ref[0]**o+(-0.952575440059)*x_ref[0]+(-0.921247808098)*x_ref[1]**o+(-0.88872834095)*x_ref[1]+(0.534103013833)*x_ref[2]**o+(-0.602295186104)*x_ref[2]
        ref[0,2,0,0]=(0.460698415704)*x_ref[0]**o+(0.770243814897)*x_ref[0]+(-0.380381727574)*x_ref[1]**o+(0.476360845668)*x_ref[1]+(-0.681040425102)*x_ref[2]**o+(-0.244369435116)*x_ref[2]
        ref[0,2,0,1]=(0.251591652281)*x_ref[0]**o+(0.748234919686)*x_ref[0]+(-0.698189037307)*x_ref[1]**o+(-0.292400412508)*x_ref[1]+(0.613850696577)*x_ref[2]**o+(-0.942243226834)*x_ref[2]
        ref[0,2,1,0]=(-0.718090066328)*x_ref[0]**o+(-0.322499511794)*x_ref[0]+(-0.75105623397)*x_ref[1]**o+(0.526680271832)*x_ref[1]+(-0.186671531793)*x_ref[2]**o+(-0.449273875991)*x_ref[2]
        ref[0,2,1,1]=(-0.52165552452)*x_ref[0]**o+(0.902446006085)*x_ref[0]+(-0.915575172702)*x_ref[1]**o+(-0.150816194662)*x_ref[1]+(0.596782785689)*x_ref[2]**o+(0.546880865642)*x_ref[2]
        ref[0,2,2,0]=(-0.104486704816)*x_ref[0]**o+(0.723863699457)*x_ref[0]+(0.758622117279)*x_ref[1]**o+(0.372271580145)*x_ref[1]+(-0.0820570383792)*x_ref[2]**o+(-0.63539781104)*x_ref[2]
        ref[0,2,2,1]=(0.0355930823603)*x_ref[0]**o+(0.0931942600432)*x_ref[0]+(-0.118096985978)*x_ref[1]**o+(0.296983782212)*x_ref[1]+(-0.864491763712)*x_ref[2]**o+(0.942230663978)*x_ref[2]
        ref[0,3,0,0]=(0.371153709815)*x_ref[0]**o+(0.0427309083893)*x_ref[0]+(-0.886563919084)*x_ref[1]**o+(-0.14946513404)*x_ref[1]+(0.80903804519)*x_ref[2]**o+(0.69728533073)*x_ref[2]
        ref[0,3,0,1]=(0.0593930396142)*x_ref[0]**o+(-0.370327974089)*x_ref[0]+(-0.388354464683)*x_ref[1]**o+(-0.367648771812)*x_ref[1]+(-0.573754821458)*x_ref[2]**o+(-0.947499155479)*x_ref[2]
        ref[0,3,1,0]=(-0.869817317)*x_ref[0]**o+(0.874317772678)*x_ref[0]+(-0.870060178375)*x_ref[1]**o+(0.66571074524)*x_ref[1]+(-0.669052377378)*x_ref[2]**o+(0.940212680575)*x_ref[2]
        ref[0,3,1,1]=(-0.417445606772)*x_ref[0]**o+(0.942606057619)*x_ref[0]+(-0.133936034409)*x_ref[1]**o+(-0.363274280866)*x_ref[1]+(0.83487439935)*x_ref[2]**o+(-0.108398981461)*x_ref[2]
        ref[0,3,2,0]=(0.363623383293)*x_ref[0]**o+(0.624267309217)*x_ref[0]+(0.801719984899)*x_ref[1]**o+(0.189212370122)*x_ref[1]+(0.464633630928)*x_ref[2]**o+(-0.979808318438)*x_ref[2]
        ref[0,3,2,1]=(0.85384408805)*x_ref[0]**o+(-0.889214754116)*x_ref[0]+(0.164915391224)*x_ref[1]**o+(0.0168991405201)*x_ref[1]+(0.097171994236)*x_ref[2]**o+(-0.146094197339)*x_ref[2]
        ref[0,4,0,0]=(0.778356158972)*x_ref[0]**o+(0.910241265149)*x_ref[0]+(0.123690863272)*x_ref[1]**o+(0.87802121779)*x_ref[1]+(-0.714615668595)*x_ref[2]**o+(-0.437472831583)*x_ref[2]
        ref[0,4,0,1]=(-0.157185481783)*x_ref[0]**o+(0.352802163968)*x_ref[0]+(0.390184123612)*x_ref[1]**o+(-0.680415093098)*x_ref[1]+(-0.121195042454)*x_ref[2]**o+(-0.713693712117)*x_ref[2]
        ref[0,4,1,0]=(0.778115668646)*x_ref[0]**o+(-0.111729042892)*x_ref[0]+(0.451518002735)*x_ref[1]**o+(0.232731183374)*x_ref[1]+(-0.764246158086)*x_ref[2]**o+(-0.795525155328)*x_ref[2]
        ref[0,4,1,1]=(-0.880652367027)*x_ref[0]**o+(-0.314966079126)*x_ref[0]+(-0.640720373065)*x_ref[1]**o+(0.336230541901)*x_ref[1]+(0.566956999526)*x_ref[2]**o+(0.74098258811)*x_ref[2]
        ref[0,4,2,0]=(0.908657274301)*x_ref[0]**o+(0.636427823614)*x_ref[0]+(0.650137414386)*x_ref[1]**o+(0.695558379005)*x_ref[1]+(-0.489852401087)*x_ref[2]**o+(-0.479227108044)*x_ref[2]
        ref[0,4,2,1]=(0.015734241525)*x_ref[0]**o+(0.0719884727088)*x_ref[0]+(0.811864895261)*x_ref[1]**o+(0.959127986784)*x_ref[1]+(-0.238146663368)*x_ref[2]**o+(-0.454939322571)*x_ref[2]
        ref[1,0,0,0]=(-0.918218246559)*x_ref[0]**o+(-0.662787169269)*x_ref[0]+(-0.954035144111)*x_ref[1]**o+(-0.647264685228)*x_ref[1]+(-0.834085960213)*x_ref[2]**o+(-0.167541951024)*x_ref[2]
        ref[1,0,0,1]=(-0.670950869382)*x_ref[0]**o+(0.826472159989)*x_ref[0]+(0.180410872786)*x_ref[1]**o+(-0.715162871594)*x_ref[1]+(0.653287853169)*x_ref[2]**o+(0.892060529625)*x_ref[2]
        ref[1,0,1,0]=(0.398789786407)*x_ref[0]**o+(0.689408033554)*x_ref[0]+(0.124338245128)*x_ref[1]**o+(0.748280808587)*x_ref[1]+(0.418357122274)*x_ref[2]**o+(0.230707440075)*x_ref[2]
        ref[1,0,1,1]=(-0.912778964055)*x_ref[0]**o+(0.904288708694)*x_ref[0]+(-0.874744856901)*x_ref[1]**o+(0.550162195719)*x_ref[1]+(-0.7526423373)*x_ref[2]**o+(0.820993231654)*x_ref[2]
        ref[1,0,2,0]=(-0.843459684648)*x_ref[0]**o+(-0.579708764091)*x_ref[0]+(-0.876072131849)*x_ref[1]**o+(0.933694599696)*x_ref[1]+(0.507781209021)*x_ref[2]**o+(-0.719729482133)*x_ref[2]
        ref[1,0,2,1]=(0.594566925796)*x_ref[0]**o+(-0.994301292358)*x_ref[0]+(0.808114389564)*x_ref[1]**o+(-0.970803712864)*x_ref[1]+(0.433544910389)*x_ref[2]**o+(0.260659440299)*x_ref[2]
        ref[1,1,0,0]=(0.997377553172)*x_ref[0]**o+(0.458428616383)*x_ref[0]+(0.357280778493)*x_ref[1]**o+(0.349837110741)*x_ref[1]+(-0.709592214445)*x_ref[2]**o+(-0.630000739033)*x_ref[2]
        ref[1,1,0,1]=(0.121595650501)*x_ref[0]**o+(0.40870120924)*x_ref[0]+(0.576917123807)*x_ref[1]**o+(-0.932268308325)*x_ref[1]+(0.710043472116)*x_ref[2]**o+(-0.716528412077)*x_ref[2]
        ref[1,1,1,0]=(-0.151416035033)*x_ref[0]**o+(0.0255347649787)*x_ref[0]+(-0.752987701889)*x_ref[1]**o+(-0.480969875813)*x_ref[1]+(-0.610514504179)*x_ref[2]**o+(0.959274757402)*x_ref[2]
        ref[1,1,1,1]=(0.76695706106)*x_ref[0]**o+(-0.0750965975512)*x_ref[0]+(0.846833900735)*x_ref[1]**o+(-0.105838465439)*x_ref[1]+(0.541027486457)*x_ref[2]**o+(0.152543017589)*x_ref[2]
        ref[1,1,2,0]=(0.0856977181865)*x_ref[0]**o+(-0.188139369706)*x_ref[0]+(0.936071565289)*x_ref[1]**o+(-0.3958713036)*x_ref[1]+(-0.350945878893)*x_ref[2]**o+(0.382302674883)*x_ref[2]
        ref[1,1,2,1]=(0.873054234789)*x_ref[0]**o+(-0.490869716146)*x_ref[0]+(0.46720526212)*x_ref[1]**o+(-0.334203412334)*x_ref[1]+(-0.81140895636)*x_ref[2]**o+(-0.973858971749)*x_ref[2]
        ref[1,2,0,0]=(-0.716676352443)*x_ref[0]**o+(0.659821396277)*x_ref[0]+(0.335964292954)*x_ref[1]**o+(0.11609664773)*x_ref[1]+(-0.889186749466)*x_ref[2]**o+(0.454231973946)*x_ref[2]
        ref[1,2,0,1]=(0.367048872648)*x_ref[0]**o+(0.276083713126)*x_ref[0]+(0.686639128043)*x_ref[1]**o+(-0.991414415293)*x_ref[1]+(0.25360515964)*x_ref[2]**o+(-0.377623315093)*x_ref[2]
        ref[1,2,1,0]=(-0.723089358777)*x_ref[0]**o+(-0.516690112406)*x_ref[0]+(0.262334273979)*x_ref[1]**o+(0.140773502537)*x_ref[1]+(0.447333972396)*x_ref[2]**o+(-0.551395692966)*x_ref[2]
        ref[1,2,1,1]=(0.738791182491)*x_ref[0]**o+(-0.978120626668)*x_ref[0]+(-0.217970433713)*x_ref[1]**o+(0.0181011067492)*x_ref[1]+(0.756322867523)*x_ref[2]**o+(0.495269195978)*x_ref[2]
        ref[1,2,2,0]=(-0.630506449854)*x_ref[0]**o+(-0.0865482364231)*x_ref[0]+(-0.415896403157)*x_ref[1]**o+(-0.712833149895)*x_ref[1]+(-0.180482789777)*x_ref[2]**o+(0.00788290661235)*x_ref[2]
        ref[1,2,2,1]=(0.0946913865377)*x_ref[0]**o+(0.752854827716)*x_ref[0]+(0.257846220843)*x_ref[1]**o+(-0.953926983578)*x_ref[1]+(-0.388505920682)*x_ref[2]**o+(-0.10751401251)*x_ref[2]
        ref[1,3,0,0]=(0.694014030611)*x_ref[0]**o+(0.0433103547797)*x_ref[0]+(-0.0805025225569)*x_ref[1]**o+(0.389052254754)*x_ref[1]+(-0.813210939614)*x_ref[2]**o+(-0.593788787896)*x_ref[2]
        ref[1,3,0,1]=(0.0101606353406)*x_ref[0]**o+(-0.849874258855)*x_ref[0]+(-0.0578973959381)*x_ref[1]**o+(0.942113140424)*x_ref[1]+(0.613716459216)*x_ref[2]**o+(0.72631176833)*x_ref[2]
        ref[1,3,1,0]=(0.264703608811)*x_ref[0]**o+(0.457962548326)*x_ref[0]+(-0.433795242673)*x_ref[1]**o+(0.650995712848)*x_ref[1]+(0.372471807967)*x_ref[2]**o+(0.00416636841047)*x_ref[2]
        ref[1,3,1,1]=(-0.73686451669)*x_ref[0]**o+(-0.0425614910776)*x_ref[0]+(-0.587820431311)*x_ref[1]**o+(-0.377628617833)*x_ref[1]+(-0.463964014406)*x_ref[2]**o+(0.283273874989)*x_ref[2]
        ref[1,3,2,0]=(-0.689341439143)*x_ref[0]**o+(-0.675861918889)*x_ref[0]+(-0.918622662352)*x_ref[1]**o+(0.740990261888)*x_ref[1]+(0.0454007677041)*x_ref[2]**o+(0.513623361074)*x_ref[2]
        ref[1,3,2,1]=(0.13229101848)*x_ref[0]**o+(0.171873799817)*x_ref[0]+(-0.154491174434)*x_ref[1]**o+(0.897976884089)*x_ref[1]+(-0.598886311821)*x_ref[2]**o+(0.738696710918)*x_ref[2]
        ref[1,4,0,0]=(0.59403636966)*x_ref[0]**o+(0.93856848969)*x_ref[0]+(0.80164849167)*x_ref[1]**o+(-0.753322979568)*x_ref[1]+(-0.107098073916)*x_ref[2]**o+(-0.623047487973)*x_ref[2]
        ref[1,4,0,1]=(0.291046099082)*x_ref[0]**o+(-0.437982471049)*x_ref[0]+(0.017395762947)*x_ref[1]**o+(0.852440593923)*x_ref[1]+(0.630825424359)*x_ref[2]**o+(0.419347491727)*x_ref[2]
        ref[1,4,1,0]=(-0.747962012701)*x_ref[0]**o+(-0.11622032094)*x_ref[0]+(0.324950549781)*x_ref[1]**o+(0.950911398327)*x_ref[1]+(-0.18098990205)*x_ref[2]**o+(-0.676060633522)*x_ref[2]
        ref[1,4,1,1]=(0.545224363128)*x_ref[0]**o+(0.753376367121)*x_ref[0]+(0.271426695753)*x_ref[1]**o+(-0.139309844675)*x_ref[1]+(-0.923434716375)*x_ref[2]**o+(-0.985988324389)*x_ref[2]
        ref[1,4,2,0]=(-0.532260822047)*x_ref[0]**o+(0.901407135724)*x_ref[0]+(0.937451330141)*x_ref[1]**o+(0.285217666559)*x_ref[1]+(-0.133028984824)*x_ref[2]**o+(0.993965133696)*x_ref[2]
        ref[1,4,2,1]=(0.2231815607)*x_ref[0]**o+(0.994845537582)*x_ref[0]+(0.0368727378195)*x_ref[1]**o+(0.511772933777)*x_ref[1]+(-0.871580215976)*x_ref[2]**o+(-0.587010057329)*x_ref[2]
        ref[2,0,0,0]=(-0.55149411913)*x_ref[0]**o+(-0.905541278559)*x_ref[0]+(0.461217373096)*x_ref[1]**o+(-0.53359853241)*x_ref[1]+(-0.666643290883)*x_ref[2]**o+(0.151495826926)*x_ref[2]
        ref[2,0,0,1]=(-0.980485062148)*x_ref[0]**o+(0.150165616162)*x_ref[0]+(0.59165725487)*x_ref[1]**o+(-0.243853536017)*x_ref[1]+(0.66816261686)*x_ref[2]**o+(0.2425696131)*x_ref[2]
        ref[2,0,1,0]=(-0.509337213758)*x_ref[0]**o+(0.652936706687)*x_ref[0]+(-0.849214338091)*x_ref[1]**o+(-0.523354123549)*x_ref[1]+(0.371515689887)*x_ref[2]**o+(0.799989815414)*x_ref[2]
        ref[2,0,1,1]=(0.461584544541)*x_ref[0]**o+(0.662931355954)*x_ref[0]+(0.744415634786)*x_ref[1]**o+(-0.016743801256)*x_ref[1]+(0.172945846325)*x_ref[2]**o+(-0.0889827089341)*x_ref[2]
        ref[2,0,2,0]=(0.506514791562)*x_ref[0]**o+(0.974477160694)*x_ref[0]+(-0.968485343167)*x_ref[1]**o+(0.427480013599)*x_ref[1]+(0.0460755879358)*x_ref[2]**o+(0.186500994948)*x_ref[2]
        ref[2,0,2,1]=(0.259831607993)*x_ref[0]**o+(-0.0141290444125)*x_ref[0]+(-0.968452177682)*x_ref[1]**o+(0.635619125009)*x_ref[1]+(-0.958848431827)*x_ref[2]**o+(-0.0977241523308)*x_ref[2]
        ref[2,1,0,0]=(0.204233222933)*x_ref[0]**o+(0.834739381877)*x_ref[0]+(0.19961185048)*x_ref[1]**o+(-0.238470380578)*x_ref[1]+(-0.749979713773)*x_ref[2]**o+(-0.268168886251)*x_ref[2]
        ref[2,1,0,1]=(0.998123643088)*x_ref[0]**o+(0.635823733041)*x_ref[0]+(-0.265447124419)*x_ref[1]**o+(-0.225582751946)*x_ref[1]+(0.91705937439)*x_ref[2]**o+(-0.739461510792)*x_ref[2]
        ref[2,1,1,0]=(0.0800715288646)*x_ref[0]**o+(-0.37677196659)*x_ref[0]+(-0.0737286350219)*x_ref[1]**o+(0.032076062933)*x_ref[1]+(0.245421912709)*x_ref[2]**o+(-0.8378023935)*x_ref[2]
        ref[2,1,1,1]=(-0.572376055603)*x_ref[0]**o+(0.8675468469)*x_ref[0]+(-0.533012889867)*x_ref[1]**o+(-0.247215658297)*x_ref[1]+(0.673144564693)*x_ref[2]**o+(-0.749136108985)*x_ref[2]
        ref[2,1,2,0]=(-0.661188192488)*x_ref[0]**o+(-0.924435593008)*x_ref[0]+(-0.791845808916)*x_ref[1]**o+(0.00133765351092)*x_ref[1]+(0.233676862873)*x_ref[2]**o+(0.981812737962)*x_ref[2]
        ref[2,1,2,1]=(0.332072445986)*x_ref[0]**o+(-0.909373798588)*x_ref[0]+(0.927025647819)*x_ref[1]**o+(-0.936348942617)*x_ref[1]+(0.251508592152)*x_ref[2]**o+(-0.468147973987)*x_ref[2]
        ref[2,2,0,0]=(0.892377893841)*x_ref[0]**o+(-0.147933293091)*x_ref[0]+(0.950491102076)*x_ref[1]**o+(0.595250355926)*x_ref[1]+(0.0169764649992)*x_ref[2]**o+(-0.683137752586)*x_ref[2]
        ref[2,2,0,1]=(0.00579397005645)*x_ref[0]**o+(0.627197373917)*x_ref[0]+(0.663574756196)*x_ref[1]**o+(0.934106767513)*x_ref[1]+(0.105844878022)*x_ref[2]**o+(0.513322917688)*x_ref[2]
        ref[2,2,1,0]=(-0.168776016133)*x_ref[0]**o+(-0.940071612944)*x_ref[0]+(0.0366053554566)*x_ref[1]**o+(-0.628010909771)*x_ref[1]+(0.250023045906)*x_ref[2]**o+(-0.00327322332166)*x_ref[2]
        ref[2,2,1,1]=(0.904155764376)*x_ref[0]**o+(-0.258018307306)*x_ref[0]+(0.65862820521)*x_ref[1]**o+(0.434480798515)*x_ref[1]+(0.4089419173)*x_ref[2]**o+(-0.518032695598)*x_ref[2]
        ref[2,2,2,0]=(-0.41470070958)*x_ref[0]**o+(-0.783186751952)*x_ref[0]+(0.548309561942)*x_ref[1]**o+(0.837130592803)*x_ref[1]+(-0.346808491848)*x_ref[2]**o+(0.165416040985)*x_ref[2]
        ref[2,2,2,1]=(0.997845909195)*x_ref[0]**o+(0.448428042158)*x_ref[0]+(-0.657050004311)*x_ref[1]**o+(0.493104851747)*x_ref[1]+(0.558315071197)*x_ref[2]**o+(0.206659432923)*x_ref[2]
        ref[2,3,0,0]=(0.433874360232)*x_ref[0]**o+(-0.0635646197131)*x_ref[0]+(-0.521690521523)*x_ref[1]**o+(-0.298130684888)*x_ref[1]+(-0.131905313644)*x_ref[2]**o+(0.64389451112)*x_ref[2]
        ref[2,3,0,1]=(0.457729315216)*x_ref[0]**o+(-0.419323242135)*x_ref[0]+(0.614544051135)*x_ref[1]**o+(-0.701671999275)*x_ref[1]+(0.905494300727)*x_ref[2]**o+(-0.565716985337)*x_ref[2]
        ref[2,3,1,0]=(0.256570728097)*x_ref[0]**o+(0.866349285103)*x_ref[0]+(-0.115768354896)*x_ref[1]**o+(-0.613044456549)*x_ref[1]+(-0.20074185156)*x_ref[2]**o+(-0.188403571776)*x_ref[2]
        ref[2,3,1,1]=(0.767479943414)*x_ref[0]**o+(0.884066215007)*x_ref[0]+(0.363855420056)*x_ref[1]**o+(0.919698981104)*x_ref[1]+(0.225747762317)*x_ref[2]**o+(-0.220362271104)*x_ref[2]
        ref[2,3,2,0]=(-0.746469501382)*x_ref[0]**o+(0.680520398001)*x_ref[0]+(0.926950996325)*x_ref[1]**o+(-0.122831303044)*x_ref[1]+(-0.27226561675)*x_ref[2]**o+(0.933304087951)*x_ref[2]
        ref[2,3,2,1]=(-0.0961434747216)*x_ref[0]**o+(0.987660516767)*x_ref[0]+(0.288175205045)*x_ref[1]**o+(0.683720820403)*x_ref[1]+(0.354403143642)*x_ref[2]**o+(-0.745694089132)*x_ref[2]
        ref[2,4,0,0]=(0.921750835986)*x_ref[0]**o+(-0.0358861598394)*x_ref[0]+(0.190129645259)*x_ref[1]**o+(0.377067803331)*x_ref[1]+(-0.0188615236582)*x_ref[2]**o+(0.804533888824)*x_ref[2]
        ref[2,4,0,1]=(-0.256132234484)*x_ref[0]**o+(-0.102661411017)*x_ref[0]+(-0.903465157009)*x_ref[1]**o+(0.50133310168)*x_ref[1]+(-0.0889102598619)*x_ref[2]**o+(-0.0310174238365)*x_ref[2]
        ref[2,4,1,0]=(0.58483798855)*x_ref[0]**o+(0.967943344305)*x_ref[0]+(-0.0701080005438)*x_ref[1]**o+(0.489476104406)*x_ref[1]+(0.615308427038)*x_ref[2]**o+(-0.38065505709)*x_ref[2]
        ref[2,4,1,1]=(0.647734130724)*x_ref[0]**o+(-0.930934392719)*x_ref[0]+(0.267370540491)*x_ref[1]**o+(-0.0175019424556)*x_ref[1]+(0.38593657129)*x_ref[2]**o+(-0.985922292359)*x_ref[2]
        ref[2,4,2,0]=(-0.289971479933)*x_ref[0]**o+(0.882014313157)*x_ref[0]+(0.147321067579)*x_ref[1]**o+(-0.176771838489)*x_ref[1]+(0.710916531544)*x_ref[2]**o+(-0.0856290170401)*x_ref[2]
        ref[2,4,2,1]=(-0.465930997733)*x_ref[0]**o+(-0.305563942094)*x_ref[0]+(-0.824535262061)*x_ref[1]**o+(-0.917892226972)*x_ref[1]+(0.624462871955)*x_ref[2]**o+(0.544736500065)*x_ref[2]
        ref[3,0,0,0]=(-0.673230815097)*x_ref[0]**o+(-0.726292712917)*x_ref[0]+(0.0531037021688)*x_ref[1]**o+(-0.467219030159)*x_ref[1]+(-0.0729861369718)*x_ref[2]**o+(-0.440394335015)*x_ref[2]
        ref[3,0,0,1]=(-0.358235295558)*x_ref[0]**o+(-0.301184713671)*x_ref[0]+(-0.496030649086)*x_ref[1]**o+(-0.125922184777)*x_ref[1]+(0.946715036973)*x_ref[2]**o+(0.846370337893)*x_ref[2]
        ref[3,0,1,0]=(-0.394175109612)*x_ref[0]**o+(-0.85137582307)*x_ref[0]+(-0.667804785044)*x_ref[1]**o+(0.156762109891)*x_ref[1]+(-0.616444700525)*x_ref[2]**o+(0.0738729399356)*x_ref[2]
        ref[3,0,1,1]=(-0.652875462534)*x_ref[0]**o+(-0.0605650192541)*x_ref[0]+(0.840539933823)*x_ref[1]**o+(-0.422819423301)*x_ref[1]+(0.113440226819)*x_ref[2]**o+(-0.915154532306)*x_ref[2]
        ref[3,0,2,0]=(-0.162863352328)*x_ref[0]**o+(-0.364461332292)*x_ref[0]+(-0.222181741398)*x_ref[1]**o+(-0.563668008365)*x_ref[1]+(0.118491770105)*x_ref[2]**o+(0.0809155053022)*x_ref[2]
        ref[3,0,2,1]=(-0.985278929437)*x_ref[0]**o+(-0.919440760157)*x_ref[0]+(-0.776910686)*x_ref[1]**o+(-0.884270488943)*x_ref[1]+(0.984279010363)*x_ref[2]**o+(-0.489503278582)*x_ref[2]
        ref[3,1,0,0]=(-0.806565449823)*x_ref[0]**o+(-0.0851385375281)*x_ref[0]+(-0.88851381773)*x_ref[1]**o+(-0.666539867528)*x_ref[1]+(0.154825531411)*x_ref[2]**o+(0.764069849144)*x_ref[2]
        ref[3,1,0,1]=(-0.222125783918)*x_ref[0]**o+(0.568035675467)*x_ref[0]+(0.470171778932)*x_ref[1]**o+(-0.307608710881)*x_ref[1]+(-0.836083742471)*x_ref[2]**o+(0.553597553381)*x_ref[2]
        ref[3,1,1,0]=(0.431237389847)*x_ref[0]**o+(-0.141391012471)*x_ref[0]+(0.726895025519)*x_ref[1]**o+(-0.527868751679)*x_ref[1]+(0.711761083137)*x_ref[2]**o+(0.996992864103)*x_ref[2]
        ref[3,1,1,1]=(-0.123885959924)*x_ref[0]**o+(0.877034475529)*x_ref[0]+(-0.0432492682928)*x_ref[1]**o+(-0.370315494618)*x_ref[1]+(0.243139719071)*x_ref[2]**o+(-0.684318634902)*x_ref[2]
        ref[3,1,2,0]=(-0.477214489355)*x_ref[0]**o+(0.340808576619)*x_ref[0]+(-0.0991802306558)*x_ref[1]**o+(0.462599821908)*x_ref[1]+(0.091768281564)*x_ref[2]**o+(0.0315552838839)*x_ref[2]
        ref[3,1,2,1]=(0.676189142894)*x_ref[0]**o+(0.510136386913)*x_ref[0]+(-0.558579041952)*x_ref[1]**o+(0.961615047877)*x_ref[1]+(0.510717386124)*x_ref[2]**o+(0.774430183432)*x_ref[2]
        ref[3,2,0,0]=(0.158370232444)*x_ref[0]**o+(0.209424855992)*x_ref[0]+(0.691438753013)*x_ref[1]**o+(-0.781594298738)*x_ref[1]+(-0.0188536627517)*x_ref[2]**o+(0.136400798995)*x_ref[2]
        ref[3,2,0,1]=(-0.440674414328)*x_ref[0]**o+(0.118228612687)*x_ref[0]+(-0.700026492613)*x_ref[1]**o+(-0.981341215714)*x_ref[1]+(-0.172847519578)*x_ref[2]**o+(-0.423095202172)*x_ref[2]
        ref[3,2,1,0]=(-0.0694960391695)*x_ref[0]**o+(-0.66448610135)*x_ref[0]+(-0.177572703748)*x_ref[1]**o+(-0.060488161727)*x_ref[1]+(0.306021853885)*x_ref[2]**o+(0.0730338716187)*x_ref[2]
        ref[3,2,1,1]=(-0.360662154547)*x_ref[0]**o+(0.577679359648)*x_ref[0]+(-0.515313534966)*x_ref[1]**o+(0.217352687358)*x_ref[1]+(0.728367577882)*x_ref[2]**o+(0.38505849834)*x_ref[2]
        ref[3,2,2,0]=(0.568671861232)*x_ref[0]**o+(-0.0792490917882)*x_ref[0]+(0.0188002885229)*x_ref[1]**o+(-0.956862575759)*x_ref[1]+(-0.844332442983)*x_ref[2]**o+(0.670792928042)*x_ref[2]
        ref[3,2,2,1]=(0.833099204713)*x_ref[0]**o+(0.368482169933)*x_ref[0]+(-0.926002498892)*x_ref[1]**o+(-0.130646446568)*x_ref[1]+(0.81164928914)*x_ref[2]**o+(-0.641079919927)*x_ref[2]
        ref[3,3,0,0]=(-0.277769350989)*x_ref[0]**o+(0.753452018649)*x_ref[0]+(-0.766322344505)*x_ref[1]**o+(-0.679108224612)*x_ref[1]+(0.0799499795424)*x_ref[2]**o+(0.255751308975)*x_ref[2]
        ref[3,3,0,1]=(0.429176753733)*x_ref[0]**o+(0.123106050259)*x_ref[0]+(-0.169037728992)*x_ref[1]**o+(0.749245933687)*x_ref[1]+(0.560140859355)*x_ref[2]**o+(-0.937091840021)*x_ref[2]
        ref[3,3,1,0]=(0.648792557593)*x_ref[0]**o+(-0.057555187513)*x_ref[0]+(0.883290331726)*x_ref[1]**o+(0.539276905995)*x_ref[1]+(0.473614408583)*x_ref[2]**o+(0.723258340333)*x_ref[2]
        ref[3,3,1,1]=(0.527298833572)*x_ref[0]**o+(0.0753824058279)*x_ref[0]+(0.719196099257)*x_ref[1]**o+(-0.0124603350035)*x_ref[1]+(-0.964484650253)*x_ref[2]**o+(0.404444269694)*x_ref[2]
        ref[3,3,2,0]=(-0.403640677462)*x_ref[0]**o+(0.906195964766)*x_ref[0]+(-0.561419078404)*x_ref[1]**o+(0.645408717838)*x_ref[1]+(0.671624799609)*x_ref[2]**o+(-0.573901868621)*x_ref[2]
        ref[3,3,2,1]=(-0.288117182165)*x_ref[0]**o+(-0.391003584945)*x_ref[0]+(-0.555037182038)*x_ref[1]**o+(0.299362209017)*x_ref[1]+(-0.552429724326)*x_ref[2]**o+(-0.416923306774)*x_ref[2]
        ref[3,4,0,0]=(-0.19594719192)*x_ref[0]**o+(0.150898934692)*x_ref[0]+(0.210935232044)*x_ref[1]**o+(-0.319764293347)*x_ref[1]+(-0.642450751195)*x_ref[2]**o+(0.22231085168)*x_ref[2]
        ref[3,4,0,1]=(-0.0514589460225)*x_ref[0]**o+(0.0366025646754)*x_ref[0]+(0.416280576691)*x_ref[1]**o+(0.609178063951)*x_ref[1]+(0.835611677062)*x_ref[2]**o+(-0.131630571478)*x_ref[2]
        ref[3,4,1,0]=(0.171509152214)*x_ref[0]**o+(-0.761188271093)*x_ref[0]+(-0.915951802896)*x_ref[1]**o+(-0.451796709915)*x_ref[1]+(-0.979847828162)*x_ref[2]**o+(0.0566453393315)*x_ref[2]
        ref[3,4,1,1]=(0.690427998102)*x_ref[0]**o+(-0.358318625695)*x_ref[0]+(0.330270871613)*x_ref[1]**o+(-0.724392522137)*x_ref[1]+(-0.535474623489)*x_ref[2]**o+(-0.12341743885)*x_ref[2]
        ref[3,4,2,0]=(0.488090753426)*x_ref[0]**o+(-0.352417078372)*x_ref[0]+(0.971988099319)*x_ref[1]**o+(-0.288929516009)*x_ref[1]+(0.612576734094)*x_ref[2]**o+(-0.43799514603)*x_ref[2]
        ref[3,4,2,1]=(-0.098768101356)*x_ref[0]**o+(0.123300661895)*x_ref[0]+(-0.0489724280789)*x_ref[1]**o+(-0.422508669137)*x_ref[1]+(-0.861483104139)*x_ref[2]**o+(0.950198316001)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_Solution_rank0(self):
      """
      tests interpolate for rank 0 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from Solution onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Data(0,(),w)
      ref=Data(0,(),w_ref)
      if dim==2:
        arg=(0.724373964817)*x[0]**o+(-0.0560850774901)*x[0]+(0.201815078281)*x[1]**o+(-0.721683697048)*x[1]
        ref=(0.724373964817)*x_ref[0]**o+(-0.0560850774901)*x_ref[0]+(0.201815078281)*x_ref[1]**o+(-0.721683697048)*x_ref[1]
      else:
        arg=(0.390296010052)*x[0]**o+(-0.279464234123)*x[0]+(-0.791334947488)*x[1]**o+(-0.0210629346704)*x[1]+(0.0816556526012)*x[2]**o+(0.77326269137)*x[2]
        ref=(0.390296010052)*x_ref[0]**o+(-0.279464234123)*x_ref[0]+(-0.791334947488)*x_ref[1]**o+(-0.0210629346704)*x_ref[1]+(0.0816556526012)*x_ref[2]**o+(0.77326269137)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_Solution_rank1(self):
      """
      tests interpolate for rank 1 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from Solution onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Data(0,(2,),w)
      ref=Data(0,(2,),w_ref)
      if dim==2:
        arg[0]=(0.645585823917)*x[0]**o+(-0.203443728436)*x[0]+(0.278126891692)*x[1]**o+(0.922093564911)*x[1]
        arg[1]=(0.23370137029)*x[0]**o+(0.84371173541)*x[0]+(0.99297574548)*x[1]**o+(0.72481785931)*x[1]
        ref[0]=(0.645585823917)*x_ref[0]**o+(-0.203443728436)*x_ref[0]+(0.278126891692)*x_ref[1]**o+(0.922093564911)*x_ref[1]
        ref[1]=(0.23370137029)*x_ref[0]**o+(0.84371173541)*x_ref[0]+(0.99297574548)*x_ref[1]**o+(0.72481785931)*x_ref[1]
      else:
        arg[0]=(0.159998049951)*x[0]**o+(-0.863367327023)*x[0]+(-0.0628498246638)*x[1]**o+(0.685139358273)*x[1]+(0.783213360641)*x[2]**o+(0.397526421338)*x[2]
        arg[1]=(0.165519279053)*x[0]**o+(-0.0936018967661)*x[0]+(0.863047696619)*x[1]**o+(0.653118186434)*x[1]+(-0.408224131944)*x[2]**o+(0.82400132854)*x[2]
        ref[0]=(0.159998049951)*x_ref[0]**o+(-0.863367327023)*x_ref[0]+(-0.0628498246638)*x_ref[1]**o+(0.685139358273)*x_ref[1]+(0.783213360641)*x_ref[2]**o+(0.397526421338)*x_ref[2]
        ref[1]=(0.165519279053)*x_ref[0]**o+(-0.0936018967661)*x_ref[0]+(0.863047696619)*x_ref[1]**o+(0.653118186434)*x_ref[1]+(-0.408224131944)*x_ref[2]**o+(0.82400132854)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_Solution_rank2(self):
      """
      tests interpolate for rank 2 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from Solution onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5),w)
      ref=Data(0,(4, 5),w_ref)
      if dim==2:
        arg[0,0]=(0.490709482882)*x[0]**o+(0.518382584887)*x[0]+(-0.940707425885)*x[1]**o+(0.224578726673)*x[1]
        arg[0,1]=(-0.747621141516)*x[0]**o+(-0.217452489097)*x[0]+(0.929120128008)*x[1]**o+(0.55263700209)*x[1]
        arg[0,2]=(-0.183545119672)*x[0]**o+(0.148192386895)*x[0]+(-0.22046309708)*x[1]**o+(0.433620446668)*x[1]
        arg[0,3]=(-0.0724840266959)*x[0]**o+(0.835144620602)*x[0]+(-0.962971534144)*x[1]**o+(-0.149681811534)*x[1]
        arg[0,4]=(-0.380150192228)*x[0]**o+(-0.309494603814)*x[0]+(-0.217425635182)*x[1]**o+(0.93567286482)*x[1]
        arg[1,0]=(0.0367921563947)*x[0]**o+(0.220217625576)*x[0]+(-0.120595819238)*x[1]**o+(0.308108793279)*x[1]
        arg[1,1]=(-0.142860237109)*x[0]**o+(-0.789758888253)*x[0]+(0.748085258229)*x[1]**o+(0.784767485883)*x[1]
        arg[1,2]=(-0.154690818845)*x[0]**o+(-0.678458370784)*x[0]+(0.750742996193)*x[1]**o+(-0.350199095531)*x[1]
        arg[1,3]=(-0.893295215013)*x[0]**o+(0.0295805814985)*x[0]+(0.265893730887)*x[1]**o+(0.435487860418)*x[1]
        arg[1,4]=(-0.302701704727)*x[0]**o+(0.618274652266)*x[0]+(-0.688386407308)*x[1]**o+(-0.832515064466)*x[1]
        arg[2,0]=(-0.926658745469)*x[0]**o+(0.548174350496)*x[0]+(-0.350700366641)*x[1]**o+(0.530188602011)*x[1]
        arg[2,1]=(0.900965385167)*x[0]**o+(0.0234051845124)*x[0]+(-0.00783834941699)*x[1]**o+(-0.766352156598)*x[1]
        arg[2,2]=(0.694733282444)*x[0]**o+(-0.0644519234093)*x[0]+(0.0493475528391)*x[1]**o+(-0.589101546403)*x[1]
        arg[2,3]=(-0.100333102737)*x[0]**o+(-0.633740106784)*x[0]+(0.408697826034)*x[1]**o+(-0.429059439472)*x[1]
        arg[2,4]=(0.635563324404)*x[0]**o+(0.955881346028)*x[0]+(0.0755013269452)*x[1]**o+(-0.743681282337)*x[1]
        arg[3,0]=(0.604891134834)*x[0]**o+(0.942047409702)*x[0]+(-0.00544373299112)*x[1]**o+(0.447523533166)*x[1]
        arg[3,1]=(-0.980967446837)*x[0]**o+(0.019824933239)*x[0]+(-0.826876930392)*x[1]**o+(0.113548968693)*x[1]
        arg[3,2]=(-0.731417395373)*x[0]**o+(-0.275072472108)*x[0]+(0.933867711492)*x[1]**o+(-0.444847972819)*x[1]
        arg[3,3]=(0.00121172413626)*x[0]**o+(-0.171073908218)*x[0]+(0.913182873344)*x[1]**o+(0.960256570926)*x[1]
        arg[3,4]=(-0.395134372727)*x[0]**o+(0.646385082409)*x[0]+(0.17387019219)*x[1]**o+(-0.287550462397)*x[1]
        ref[0,0]=(0.490709482882)*x_ref[0]**o+(0.518382584887)*x_ref[0]+(-0.940707425885)*x_ref[1]**o+(0.224578726673)*x_ref[1]
        ref[0,1]=(-0.747621141516)*x_ref[0]**o+(-0.217452489097)*x_ref[0]+(0.929120128008)*x_ref[1]**o+(0.55263700209)*x_ref[1]
        ref[0,2]=(-0.183545119672)*x_ref[0]**o+(0.148192386895)*x_ref[0]+(-0.22046309708)*x_ref[1]**o+(0.433620446668)*x_ref[1]
        ref[0,3]=(-0.0724840266959)*x_ref[0]**o+(0.835144620602)*x_ref[0]+(-0.962971534144)*x_ref[1]**o+(-0.149681811534)*x_ref[1]
        ref[0,4]=(-0.380150192228)*x_ref[0]**o+(-0.309494603814)*x_ref[0]+(-0.217425635182)*x_ref[1]**o+(0.93567286482)*x_ref[1]
        ref[1,0]=(0.0367921563947)*x_ref[0]**o+(0.220217625576)*x_ref[0]+(-0.120595819238)*x_ref[1]**o+(0.308108793279)*x_ref[1]
        ref[1,1]=(-0.142860237109)*x_ref[0]**o+(-0.789758888253)*x_ref[0]+(0.748085258229)*x_ref[1]**o+(0.784767485883)*x_ref[1]
        ref[1,2]=(-0.154690818845)*x_ref[0]**o+(-0.678458370784)*x_ref[0]+(0.750742996193)*x_ref[1]**o+(-0.350199095531)*x_ref[1]
        ref[1,3]=(-0.893295215013)*x_ref[0]**o+(0.0295805814985)*x_ref[0]+(0.265893730887)*x_ref[1]**o+(0.435487860418)*x_ref[1]
        ref[1,4]=(-0.302701704727)*x_ref[0]**o+(0.618274652266)*x_ref[0]+(-0.688386407308)*x_ref[1]**o+(-0.832515064466)*x_ref[1]
        ref[2,0]=(-0.926658745469)*x_ref[0]**o+(0.548174350496)*x_ref[0]+(-0.350700366641)*x_ref[1]**o+(0.530188602011)*x_ref[1]
        ref[2,1]=(0.900965385167)*x_ref[0]**o+(0.0234051845124)*x_ref[0]+(-0.00783834941699)*x_ref[1]**o+(-0.766352156598)*x_ref[1]
        ref[2,2]=(0.694733282444)*x_ref[0]**o+(-0.0644519234093)*x_ref[0]+(0.0493475528391)*x_ref[1]**o+(-0.589101546403)*x_ref[1]
        ref[2,3]=(-0.100333102737)*x_ref[0]**o+(-0.633740106784)*x_ref[0]+(0.408697826034)*x_ref[1]**o+(-0.429059439472)*x_ref[1]
        ref[2,4]=(0.635563324404)*x_ref[0]**o+(0.955881346028)*x_ref[0]+(0.0755013269452)*x_ref[1]**o+(-0.743681282337)*x_ref[1]
        ref[3,0]=(0.604891134834)*x_ref[0]**o+(0.942047409702)*x_ref[0]+(-0.00544373299112)*x_ref[1]**o+(0.447523533166)*x_ref[1]
        ref[3,1]=(-0.980967446837)*x_ref[0]**o+(0.019824933239)*x_ref[0]+(-0.826876930392)*x_ref[1]**o+(0.113548968693)*x_ref[1]
        ref[3,2]=(-0.731417395373)*x_ref[0]**o+(-0.275072472108)*x_ref[0]+(0.933867711492)*x_ref[1]**o+(-0.444847972819)*x_ref[1]
        ref[3,3]=(0.00121172413626)*x_ref[0]**o+(-0.171073908218)*x_ref[0]+(0.913182873344)*x_ref[1]**o+(0.960256570926)*x_ref[1]
        ref[3,4]=(-0.395134372727)*x_ref[0]**o+(0.646385082409)*x_ref[0]+(0.17387019219)*x_ref[1]**o+(-0.287550462397)*x_ref[1]
      else:
        arg[0,0]=(0.386545340192)*x[0]**o+(-0.397636296285)*x[0]+(-0.849437816516)*x[1]**o+(-0.83083406493)*x[1]+(0.479216612031)*x[2]**o+(0.0903787541308)*x[2]
        arg[0,1]=(-0.331651855726)*x[0]**o+(0.849421626918)*x[0]+(0.528368477362)*x[1]**o+(-0.015122214824)*x[1]+(0.494310578452)*x[2]**o+(0.190239928161)*x[2]
        arg[0,2]=(-0.662389759646)*x[0]**o+(0.204378343962)*x[0]+(0.486802744897)*x[1]**o+(0.822174784922)*x[1]+(-0.309203812647)*x[2]**o+(-0.101665040799)*x[2]
        arg[0,3]=(0.609106341828)*x[0]**o+(0.658404088333)*x[0]+(-0.0092991120732)*x[1]**o+(-0.73794934348)*x[1]+(0.270245135914)*x[2]**o+(-0.594067838407)*x[2]
        arg[0,4]=(-0.291752213357)*x[0]**o+(-0.596751343066)*x[0]+(-0.109196903078)*x[1]**o+(-0.868610307708)*x[1]+(0.622541308544)*x[2]**o+(0.482202762224)*x[2]
        arg[1,0]=(-0.113988933227)*x[0]**o+(-0.846328804695)*x[0]+(0.691769553626)*x[1]**o+(-0.377441645148)*x[1]+(-0.769857970682)*x[2]**o+(-0.583083729037)*x[2]
        arg[1,1]=(-0.635333757145)*x[0]**o+(0.194761117849)*x[0]+(-0.268287296731)*x[1]**o+(0.463906930647)*x[1]+(0.768293821029)*x[2]**o+(-0.532824906687)*x[2]
        arg[1,2]=(0.355380090138)*x[0]**o+(-0.181191575848)*x[0]+(-0.558667766467)*x[1]**o+(0.561707671872)*x[1]+(-0.427004232725)*x[2]**o+(-0.996564299989)*x[2]
        arg[1,3]=(-0.891638892893)*x[0]**o+(0.0867350850512)*x[0]+(0.880258390584)*x[1]**o+(-0.798123407525)*x[1]+(-0.571505202201)*x[2]**o+(0.35374245063)*x[2]
        arg[1,4]=(0.543632496941)*x[0]**o+(-0.630363803582)*x[0]+(0.38107312058)*x[1]**o+(0.222401831883)*x[1]+(-0.932212080175)*x[2]**o+(-0.0482899826174)*x[2]
        arg[2,0]=(0.46514308325)*x[0]**o+(-0.907464652384)*x[0]+(-0.706105595148)*x[1]**o+(0.486989958675)*x[1]+(0.422702683485)*x[2]**o+(0.640916925932)*x[2]
        arg[2,1]=(0.912808317479)*x[0]**o+(-0.860779312763)*x[0]+(-0.915364956133)*x[1]**o+(-0.314186117224)*x[1]+(-0.860814410035)*x[2]**o+(0.837570442722)*x[2]
        arg[2,2]=(0.00932917919464)*x[0]**o+(0.393773283049)*x[0]+(0.699998572668)*x[1]**o+(0.153407885795)*x[1]+(-0.166245189281)*x[2]**o+(-0.430541214236)*x[2]
        arg[2,3]=(-0.00127511229816)*x[0]**o+(-0.327443174924)*x[0]+(0.61881322537)*x[1]**o+(-0.725854102962)*x[1]+(0.650649062427)*x[2]**o+(0.246373184041)*x[2]
        arg[2,4]=(-0.530580898049)*x[0]**o+(-0.208506132901)*x[0]+(-0.102049332015)*x[1]**o+(-0.537037644189)*x[1]+(0.740113470729)*x[2]**o+(0.12094405264)*x[2]
        arg[3,0]=(-0.103423602042)*x[0]**o+(-0.535742323666)*x[0]+(0.678663019405)*x[1]**o+(0.579375950057)*x[1]+(0.323627490052)*x[2]**o+(-0.681323301356)*x[2]
        arg[3,1]=(0.670239087519)*x[0]**o+(0.209657968128)*x[0]+(-0.357490300727)*x[1]**o+(-0.441776782321)*x[1]+(0.00927389037562)*x[2]**o+(0.633527599723)*x[2]
        arg[3,2]=(0.415569606138)*x[0]**o+(-0.422211616068)*x[0]+(-0.695086235645)*x[1]**o+(0.709158289848)*x[1]+(-0.523450845183)*x[2]**o+(0.555778994398)*x[2]
        arg[3,3]=(0.0188993892075)*x[0]**o+(0.611088737828)*x[0]+(0.0313795167921)*x[1]**o+(0.0842651755016)*x[1]+(0.288414199415)*x[2]**o+(0.695910780235)*x[2]
        arg[3,4]=(-0.500338800879)*x[0]**o+(0.0984192369328)*x[0]+(0.71163172992)*x[1]**o+(-0.102123116139)*x[1]+(0.954613955678)*x[2]**o+(0.384872409422)*x[2]
        ref[0,0]=(0.386545340192)*x_ref[0]**o+(-0.397636296285)*x_ref[0]+(-0.849437816516)*x_ref[1]**o+(-0.83083406493)*x_ref[1]+(0.479216612031)*x_ref[2]**o+(0.0903787541308)*x_ref[2]
        ref[0,1]=(-0.331651855726)*x_ref[0]**o+(0.849421626918)*x_ref[0]+(0.528368477362)*x_ref[1]**o+(-0.015122214824)*x_ref[1]+(0.494310578452)*x_ref[2]**o+(0.190239928161)*x_ref[2]
        ref[0,2]=(-0.662389759646)*x_ref[0]**o+(0.204378343962)*x_ref[0]+(0.486802744897)*x_ref[1]**o+(0.822174784922)*x_ref[1]+(-0.309203812647)*x_ref[2]**o+(-0.101665040799)*x_ref[2]
        ref[0,3]=(0.609106341828)*x_ref[0]**o+(0.658404088333)*x_ref[0]+(-0.0092991120732)*x_ref[1]**o+(-0.73794934348)*x_ref[1]+(0.270245135914)*x_ref[2]**o+(-0.594067838407)*x_ref[2]
        ref[0,4]=(-0.291752213357)*x_ref[0]**o+(-0.596751343066)*x_ref[0]+(-0.109196903078)*x_ref[1]**o+(-0.868610307708)*x_ref[1]+(0.622541308544)*x_ref[2]**o+(0.482202762224)*x_ref[2]
        ref[1,0]=(-0.113988933227)*x_ref[0]**o+(-0.846328804695)*x_ref[0]+(0.691769553626)*x_ref[1]**o+(-0.377441645148)*x_ref[1]+(-0.769857970682)*x_ref[2]**o+(-0.583083729037)*x_ref[2]
        ref[1,1]=(-0.635333757145)*x_ref[0]**o+(0.194761117849)*x_ref[0]+(-0.268287296731)*x_ref[1]**o+(0.463906930647)*x_ref[1]+(0.768293821029)*x_ref[2]**o+(-0.532824906687)*x_ref[2]
        ref[1,2]=(0.355380090138)*x_ref[0]**o+(-0.181191575848)*x_ref[0]+(-0.558667766467)*x_ref[1]**o+(0.561707671872)*x_ref[1]+(-0.427004232725)*x_ref[2]**o+(-0.996564299989)*x_ref[2]
        ref[1,3]=(-0.891638892893)*x_ref[0]**o+(0.0867350850512)*x_ref[0]+(0.880258390584)*x_ref[1]**o+(-0.798123407525)*x_ref[1]+(-0.571505202201)*x_ref[2]**o+(0.35374245063)*x_ref[2]
        ref[1,4]=(0.543632496941)*x_ref[0]**o+(-0.630363803582)*x_ref[0]+(0.38107312058)*x_ref[1]**o+(0.222401831883)*x_ref[1]+(-0.932212080175)*x_ref[2]**o+(-0.0482899826174)*x_ref[2]
        ref[2,0]=(0.46514308325)*x_ref[0]**o+(-0.907464652384)*x_ref[0]+(-0.706105595148)*x_ref[1]**o+(0.486989958675)*x_ref[1]+(0.422702683485)*x_ref[2]**o+(0.640916925932)*x_ref[2]
        ref[2,1]=(0.912808317479)*x_ref[0]**o+(-0.860779312763)*x_ref[0]+(-0.915364956133)*x_ref[1]**o+(-0.314186117224)*x_ref[1]+(-0.860814410035)*x_ref[2]**o+(0.837570442722)*x_ref[2]
        ref[2,2]=(0.00932917919464)*x_ref[0]**o+(0.393773283049)*x_ref[0]+(0.699998572668)*x_ref[1]**o+(0.153407885795)*x_ref[1]+(-0.166245189281)*x_ref[2]**o+(-0.430541214236)*x_ref[2]
        ref[2,3]=(-0.00127511229816)*x_ref[0]**o+(-0.327443174924)*x_ref[0]+(0.61881322537)*x_ref[1]**o+(-0.725854102962)*x_ref[1]+(0.650649062427)*x_ref[2]**o+(0.246373184041)*x_ref[2]
        ref[2,4]=(-0.530580898049)*x_ref[0]**o+(-0.208506132901)*x_ref[0]+(-0.102049332015)*x_ref[1]**o+(-0.537037644189)*x_ref[1]+(0.740113470729)*x_ref[2]**o+(0.12094405264)*x_ref[2]
        ref[3,0]=(-0.103423602042)*x_ref[0]**o+(-0.535742323666)*x_ref[0]+(0.678663019405)*x_ref[1]**o+(0.579375950057)*x_ref[1]+(0.323627490052)*x_ref[2]**o+(-0.681323301356)*x_ref[2]
        ref[3,1]=(0.670239087519)*x_ref[0]**o+(0.209657968128)*x_ref[0]+(-0.357490300727)*x_ref[1]**o+(-0.441776782321)*x_ref[1]+(0.00927389037562)*x_ref[2]**o+(0.633527599723)*x_ref[2]
        ref[3,2]=(0.415569606138)*x_ref[0]**o+(-0.422211616068)*x_ref[0]+(-0.695086235645)*x_ref[1]**o+(0.709158289848)*x_ref[1]+(-0.523450845183)*x_ref[2]**o+(0.555778994398)*x_ref[2]
        ref[3,3]=(0.0188993892075)*x_ref[0]**o+(0.611088737828)*x_ref[0]+(0.0313795167921)*x_ref[1]**o+(0.0842651755016)*x_ref[1]+(0.288414199415)*x_ref[2]**o+(0.695910780235)*x_ref[2]
        ref[3,4]=(-0.500338800879)*x_ref[0]**o+(0.0984192369328)*x_ref[0]+(0.71163172992)*x_ref[1]**o+(-0.102123116139)*x_ref[1]+(0.954613955678)*x_ref[2]**o+(0.384872409422)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_Solution_rank3(self):
      """
      tests interpolate for rank 3 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from Solution onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2),w_ref)
      if dim==2:
        arg[0,0,0]=(0.317274364174)*x[0]**o+(-0.565307843766)*x[0]+(-0.367874638208)*x[1]**o+(0.0029581033546)*x[1]
        arg[0,0,1]=(0.832728755858)*x[0]**o+(0.245271193641)*x[0]+(-0.727130029377)*x[1]**o+(0.137912975829)*x[1]
        arg[0,1,0]=(-0.634508968932)*x[0]**o+(-0.864200861637)*x[0]+(-0.566985971867)*x[1]**o+(0.458187145519)*x[1]
        arg[0,1,1]=(0.133772683885)*x[0]**o+(-0.630800341215)*x[0]+(0.958934971335)*x[1]**o+(-0.15839182257)*x[1]
        arg[1,0,0]=(-0.0948724752087)*x[0]**o+(-0.25589337469)*x[0]+(-0.315713312867)*x[1]**o+(0.223487524432)*x[1]
        arg[1,0,1]=(0.794634271698)*x[0]**o+(-0.505797079712)*x[0]+(0.54367349338)*x[1]**o+(0.610149581933)*x[1]
        arg[1,1,0]=(-0.447039667341)*x[0]**o+(-0.917009838715)*x[0]+(0.525331113522)*x[1]**o+(-0.626176620973)*x[1]
        arg[1,1,1]=(0.0749028334839)*x[0]**o+(0.106950341725)*x[0]+(-0.375321288987)*x[1]**o+(0.510260762626)*x[1]
        arg[2,0,0]=(-0.286661970669)*x[0]**o+(0.584169107201)*x[0]+(0.189852415568)*x[1]**o+(-0.685695910848)*x[1]
        arg[2,0,1]=(-0.689509796724)*x[0]**o+(-0.316067007053)*x[0]+(0.93758916753)*x[1]**o+(0.987368705364)*x[1]
        arg[2,1,0]=(-0.411695937554)*x[0]**o+(0.858528628791)*x[0]+(-0.579361651418)*x[1]**o+(-0.179617869494)*x[1]
        arg[2,1,1]=(0.871441175049)*x[0]**o+(0.445176827889)*x[0]+(0.282613647019)*x[1]**o+(-0.0414230375843)*x[1]
        arg[3,0,0]=(-0.242136375691)*x[0]**o+(-0.91917484201)*x[0]+(-0.88026838607)*x[1]**o+(0.363309786006)*x[1]
        arg[3,0,1]=(-0.991221762386)*x[0]**o+(-0.0149280738883)*x[0]+(0.420322977563)*x[1]**o+(-0.0533076687981)*x[1]
        arg[3,1,0]=(-0.889003524303)*x[0]**o+(-0.023519502168)*x[0]+(0.826015855889)*x[1]**o+(0.0568467842617)*x[1]
        arg[3,1,1]=(0.608567002824)*x[0]**o+(-0.531288593441)*x[0]+(0.0886678034719)*x[1]**o+(-0.732353220319)*x[1]
        arg[4,0,0]=(-0.0537755786538)*x[0]**o+(-0.475886396481)*x[0]+(-0.153787571145)*x[1]**o+(0.689850737569)*x[1]
        arg[4,0,1]=(0.413017674549)*x[0]**o+(-0.993353261494)*x[0]+(-0.944597571268)*x[1]**o+(0.566423551758)*x[1]
        arg[4,1,0]=(-0.716864417268)*x[0]**o+(-0.871574272865)*x[0]+(0.394071572628)*x[1]**o+(0.452903048597)*x[1]
        arg[4,1,1]=(0.861790785883)*x[0]**o+(-0.511789401943)*x[0]+(-0.272452527007)*x[1]**o+(-0.379021521619)*x[1]
        arg[5,0,0]=(0.509668147054)*x[0]**o+(0.919355584411)*x[0]+(-0.673215166917)*x[1]**o+(0.766459631931)*x[1]
        arg[5,0,1]=(0.177006227916)*x[0]**o+(-0.275118477)*x[0]+(0.0302367843701)*x[1]**o+(-0.761226360328)*x[1]
        arg[5,1,0]=(0.013432534638)*x[0]**o+(-0.911626567027)*x[0]+(-0.710520442529)*x[1]**o+(-0.882317448423)*x[1]
        arg[5,1,1]=(0.595583093615)*x[0]**o+(0.163527905156)*x[0]+(-0.122165051333)*x[1]**o+(-0.526359288181)*x[1]
        ref[0,0,0]=(0.317274364174)*x_ref[0]**o+(-0.565307843766)*x_ref[0]+(-0.367874638208)*x_ref[1]**o+(0.0029581033546)*x_ref[1]
        ref[0,0,1]=(0.832728755858)*x_ref[0]**o+(0.245271193641)*x_ref[0]+(-0.727130029377)*x_ref[1]**o+(0.137912975829)*x_ref[1]
        ref[0,1,0]=(-0.634508968932)*x_ref[0]**o+(-0.864200861637)*x_ref[0]+(-0.566985971867)*x_ref[1]**o+(0.458187145519)*x_ref[1]
        ref[0,1,1]=(0.133772683885)*x_ref[0]**o+(-0.630800341215)*x_ref[0]+(0.958934971335)*x_ref[1]**o+(-0.15839182257)*x_ref[1]
        ref[1,0,0]=(-0.0948724752087)*x_ref[0]**o+(-0.25589337469)*x_ref[0]+(-0.315713312867)*x_ref[1]**o+(0.223487524432)*x_ref[1]
        ref[1,0,1]=(0.794634271698)*x_ref[0]**o+(-0.505797079712)*x_ref[0]+(0.54367349338)*x_ref[1]**o+(0.610149581933)*x_ref[1]
        ref[1,1,0]=(-0.447039667341)*x_ref[0]**o+(-0.917009838715)*x_ref[0]+(0.525331113522)*x_ref[1]**o+(-0.626176620973)*x_ref[1]
        ref[1,1,1]=(0.0749028334839)*x_ref[0]**o+(0.106950341725)*x_ref[0]+(-0.375321288987)*x_ref[1]**o+(0.510260762626)*x_ref[1]
        ref[2,0,0]=(-0.286661970669)*x_ref[0]**o+(0.584169107201)*x_ref[0]+(0.189852415568)*x_ref[1]**o+(-0.685695910848)*x_ref[1]
        ref[2,0,1]=(-0.689509796724)*x_ref[0]**o+(-0.316067007053)*x_ref[0]+(0.93758916753)*x_ref[1]**o+(0.987368705364)*x_ref[1]
        ref[2,1,0]=(-0.411695937554)*x_ref[0]**o+(0.858528628791)*x_ref[0]+(-0.579361651418)*x_ref[1]**o+(-0.179617869494)*x_ref[1]
        ref[2,1,1]=(0.871441175049)*x_ref[0]**o+(0.445176827889)*x_ref[0]+(0.282613647019)*x_ref[1]**o+(-0.0414230375843)*x_ref[1]
        ref[3,0,0]=(-0.242136375691)*x_ref[0]**o+(-0.91917484201)*x_ref[0]+(-0.88026838607)*x_ref[1]**o+(0.363309786006)*x_ref[1]
        ref[3,0,1]=(-0.991221762386)*x_ref[0]**o+(-0.0149280738883)*x_ref[0]+(0.420322977563)*x_ref[1]**o+(-0.0533076687981)*x_ref[1]
        ref[3,1,0]=(-0.889003524303)*x_ref[0]**o+(-0.023519502168)*x_ref[0]+(0.826015855889)*x_ref[1]**o+(0.0568467842617)*x_ref[1]
        ref[3,1,1]=(0.608567002824)*x_ref[0]**o+(-0.531288593441)*x_ref[0]+(0.0886678034719)*x_ref[1]**o+(-0.732353220319)*x_ref[1]
        ref[4,0,0]=(-0.0537755786538)*x_ref[0]**o+(-0.475886396481)*x_ref[0]+(-0.153787571145)*x_ref[1]**o+(0.689850737569)*x_ref[1]
        ref[4,0,1]=(0.413017674549)*x_ref[0]**o+(-0.993353261494)*x_ref[0]+(-0.944597571268)*x_ref[1]**o+(0.566423551758)*x_ref[1]
        ref[4,1,0]=(-0.716864417268)*x_ref[0]**o+(-0.871574272865)*x_ref[0]+(0.394071572628)*x_ref[1]**o+(0.452903048597)*x_ref[1]
        ref[4,1,1]=(0.861790785883)*x_ref[0]**o+(-0.511789401943)*x_ref[0]+(-0.272452527007)*x_ref[1]**o+(-0.379021521619)*x_ref[1]
        ref[5,0,0]=(0.509668147054)*x_ref[0]**o+(0.919355584411)*x_ref[0]+(-0.673215166917)*x_ref[1]**o+(0.766459631931)*x_ref[1]
        ref[5,0,1]=(0.177006227916)*x_ref[0]**o+(-0.275118477)*x_ref[0]+(0.0302367843701)*x_ref[1]**o+(-0.761226360328)*x_ref[1]
        ref[5,1,0]=(0.013432534638)*x_ref[0]**o+(-0.911626567027)*x_ref[0]+(-0.710520442529)*x_ref[1]**o+(-0.882317448423)*x_ref[1]
        ref[5,1,1]=(0.595583093615)*x_ref[0]**o+(0.163527905156)*x_ref[0]+(-0.122165051333)*x_ref[1]**o+(-0.526359288181)*x_ref[1]
      else:
        arg[0,0,0]=(0.230695674262)*x[0]**o+(-0.987212801716)*x[0]+(-0.0599001434987)*x[1]**o+(0.719591259555)*x[1]+(-0.665186369759)*x[2]**o+(-0.563552602515)*x[2]
        arg[0,0,1]=(-0.775441966131)*x[0]**o+(-0.152166849702)*x[0]+(0.626466789363)*x[1]**o+(0.179356346581)*x[1]+(-0.687406927434)*x[2]**o+(-0.563492313114)*x[2]
        arg[0,1,0]=(0.749522748133)*x[0]**o+(-0.582456308337)*x[0]+(0.676882643058)*x[1]**o+(0.365363182598)*x[1]+(-0.550867708812)*x[2]**o+(0.777938151183)*x[2]
        arg[0,1,1]=(0.389788552775)*x[0]**o+(-0.298877489997)*x[0]+(0.560326209269)*x[1]**o+(-0.113249832155)*x[1]+(0.588087510254)*x[2]**o+(-0.593503776461)*x[2]
        arg[1,0,0]=(-0.354661103909)*x[0]**o+(-0.195375369549)*x[0]+(-0.798795069895)*x[1]**o+(0.0968631048579)*x[1]+(-0.016118190391)*x[2]**o+(-0.903745407705)*x[2]
        arg[1,0,1]=(-0.383688834784)*x[0]**o+(0.861256231933)*x[0]+(-0.767204834667)*x[1]**o+(0.338875482372)*x[1]+(0.117146310891)*x[2]**o+(-0.302651846083)*x[2]
        arg[1,1,0]=(0.646263670969)*x[0]**o+(-0.447984008871)*x[0]+(-0.169322492785)*x[1]**o+(-0.570614724481)*x[1]+(-0.848518457618)*x[2]**o+(-0.0750005539216)*x[2]
        arg[1,1,1]=(-0.232496745304)*x[0]**o+(0.182581173413)*x[0]+(0.8032436357)*x[1]**o+(-0.391266449476)*x[1]+(0.491677629766)*x[2]**o+(-0.188361708708)*x[2]
        arg[2,0,0]=(-0.571145099091)*x[0]**o+(-0.486855242188)*x[0]+(0.52833582073)*x[1]**o+(0.326716348242)*x[1]+(0.528475192869)*x[2]**o+(0.163280575137)*x[2]
        arg[2,0,1]=(-0.368889674601)*x[0]**o+(0.601775816775)*x[0]+(0.0281742455403)*x[1]**o+(-0.198869716243)*x[1]+(0.193343065506)*x[2]**o+(-0.0614451571108)*x[2]
        arg[2,1,0]=(0.724595792492)*x[0]**o+(-0.206983407651)*x[0]+(-0.33966393719)*x[1]**o+(0.929465830796)*x[1]+(-0.938517195863)*x[2]**o+(0.70874077222)*x[2]
        arg[2,1,1]=(-0.536303362606)*x[0]**o+(-0.430663025812)*x[0]+(-0.0845923469062)*x[1]**o+(0.866655631144)*x[1]+(-0.2378102403)*x[2]**o+(0.969070797268)*x[2]
        arg[3,0,0]=(0.345137352559)*x[0]**o+(-0.696652518382)*x[0]+(0.914998974008)*x[1]**o+(-0.874935163156)*x[1]+(0.947812661283)*x[2]**o+(0.868611662789)*x[2]
        arg[3,0,1]=(-0.421640746358)*x[0]**o+(-0.37925991208)*x[0]+(0.468281241452)*x[1]**o+(0.439086689805)*x[1]+(-0.607986317001)*x[2]**o+(0.269488168937)*x[2]
        arg[3,1,0]=(-0.550485456473)*x[0]**o+(-0.520018714495)*x[0]+(0.439679309654)*x[1]**o+(-0.0652434084668)*x[1]+(-0.584838668708)*x[2]**o+(-0.46047281034)*x[2]
        arg[3,1,1]=(-0.421746456038)*x[0]**o+(0.370489549278)*x[0]+(-0.683682295105)*x[1]**o+(0.324051718035)*x[1]+(-0.579046430636)*x[2]**o+(0.587623604941)*x[2]
        arg[4,0,0]=(-0.167582584882)*x[0]**o+(0.524904122243)*x[0]+(0.665109348362)*x[1]**o+(-0.093655366509)*x[1]+(0.704404656254)*x[2]**o+(-0.402602441741)*x[2]
        arg[4,0,1]=(0.633854964824)*x[0]**o+(-0.836385840194)*x[0]+(-0.0995437996628)*x[1]**o+(-0.200285338869)*x[1]+(0.535023783881)*x[2]**o+(0.884702543593)*x[2]
        arg[4,1,0]=(-0.953619378208)*x[0]**o+(-0.390518815669)*x[0]+(-0.0590889178236)*x[1]**o+(-0.997225825019)*x[1]+(0.0520418736402)*x[2]**o+(0.484749465262)*x[2]
        arg[4,1,1]=(-0.440389854756)*x[0]**o+(0.15520431778)*x[0]+(0.346839174513)*x[1]**o+(0.0172676873159)*x[1]+(0.86275919224)*x[2]**o+(0.568069148992)*x[2]
        arg[5,0,0]=(-0.303687941217)*x[0]**o+(0.516115915687)*x[0]+(-0.750276597519)*x[1]**o+(0.734039058778)*x[1]+(-0.624499210252)*x[2]**o+(-0.926338135421)*x[2]
        arg[5,0,1]=(-0.472230777895)*x[0]**o+(-0.557827728373)*x[0]+(-0.403783023157)*x[1]**o+(0.70433336955)*x[1]+(-0.0211737454531)*x[2]**o+(0.0774406125836)*x[2]
        arg[5,1,0]=(0.718149875204)*x[0]**o+(-0.183000739787)*x[0]+(-0.655724252373)*x[1]**o+(-0.634941710776)*x[1]+(-0.26300163323)*x[2]**o+(-0.25105424715)*x[2]
        arg[5,1,1]=(0.109601904441)*x[0]**o+(-0.00640764631554)*x[0]+(-0.488035610009)*x[1]**o+(0.909668323769)*x[1]+(0.229692168501)*x[2]**o+(-0.739241309497)*x[2]
        ref[0,0,0]=(0.230695674262)*x_ref[0]**o+(-0.987212801716)*x_ref[0]+(-0.0599001434987)*x_ref[1]**o+(0.719591259555)*x_ref[1]+(-0.665186369759)*x_ref[2]**o+(-0.563552602515)*x_ref[2]
        ref[0,0,1]=(-0.775441966131)*x_ref[0]**o+(-0.152166849702)*x_ref[0]+(0.626466789363)*x_ref[1]**o+(0.179356346581)*x_ref[1]+(-0.687406927434)*x_ref[2]**o+(-0.563492313114)*x_ref[2]
        ref[0,1,0]=(0.749522748133)*x_ref[0]**o+(-0.582456308337)*x_ref[0]+(0.676882643058)*x_ref[1]**o+(0.365363182598)*x_ref[1]+(-0.550867708812)*x_ref[2]**o+(0.777938151183)*x_ref[2]
        ref[0,1,1]=(0.389788552775)*x_ref[0]**o+(-0.298877489997)*x_ref[0]+(0.560326209269)*x_ref[1]**o+(-0.113249832155)*x_ref[1]+(0.588087510254)*x_ref[2]**o+(-0.593503776461)*x_ref[2]
        ref[1,0,0]=(-0.354661103909)*x_ref[0]**o+(-0.195375369549)*x_ref[0]+(-0.798795069895)*x_ref[1]**o+(0.0968631048579)*x_ref[1]+(-0.016118190391)*x_ref[2]**o+(-0.903745407705)*x_ref[2]
        ref[1,0,1]=(-0.383688834784)*x_ref[0]**o+(0.861256231933)*x_ref[0]+(-0.767204834667)*x_ref[1]**o+(0.338875482372)*x_ref[1]+(0.117146310891)*x_ref[2]**o+(-0.302651846083)*x_ref[2]
        ref[1,1,0]=(0.646263670969)*x_ref[0]**o+(-0.447984008871)*x_ref[0]+(-0.169322492785)*x_ref[1]**o+(-0.570614724481)*x_ref[1]+(-0.848518457618)*x_ref[2]**o+(-0.0750005539216)*x_ref[2]
        ref[1,1,1]=(-0.232496745304)*x_ref[0]**o+(0.182581173413)*x_ref[0]+(0.8032436357)*x_ref[1]**o+(-0.391266449476)*x_ref[1]+(0.491677629766)*x_ref[2]**o+(-0.188361708708)*x_ref[2]
        ref[2,0,0]=(-0.571145099091)*x_ref[0]**o+(-0.486855242188)*x_ref[0]+(0.52833582073)*x_ref[1]**o+(0.326716348242)*x_ref[1]+(0.528475192869)*x_ref[2]**o+(0.163280575137)*x_ref[2]
        ref[2,0,1]=(-0.368889674601)*x_ref[0]**o+(0.601775816775)*x_ref[0]+(0.0281742455403)*x_ref[1]**o+(-0.198869716243)*x_ref[1]+(0.193343065506)*x_ref[2]**o+(-0.0614451571108)*x_ref[2]
        ref[2,1,0]=(0.724595792492)*x_ref[0]**o+(-0.206983407651)*x_ref[0]+(-0.33966393719)*x_ref[1]**o+(0.929465830796)*x_ref[1]+(-0.938517195863)*x_ref[2]**o+(0.70874077222)*x_ref[2]
        ref[2,1,1]=(-0.536303362606)*x_ref[0]**o+(-0.430663025812)*x_ref[0]+(-0.0845923469062)*x_ref[1]**o+(0.866655631144)*x_ref[1]+(-0.2378102403)*x_ref[2]**o+(0.969070797268)*x_ref[2]
        ref[3,0,0]=(0.345137352559)*x_ref[0]**o+(-0.696652518382)*x_ref[0]+(0.914998974008)*x_ref[1]**o+(-0.874935163156)*x_ref[1]+(0.947812661283)*x_ref[2]**o+(0.868611662789)*x_ref[2]
        ref[3,0,1]=(-0.421640746358)*x_ref[0]**o+(-0.37925991208)*x_ref[0]+(0.468281241452)*x_ref[1]**o+(0.439086689805)*x_ref[1]+(-0.607986317001)*x_ref[2]**o+(0.269488168937)*x_ref[2]
        ref[3,1,0]=(-0.550485456473)*x_ref[0]**o+(-0.520018714495)*x_ref[0]+(0.439679309654)*x_ref[1]**o+(-0.0652434084668)*x_ref[1]+(-0.584838668708)*x_ref[2]**o+(-0.46047281034)*x_ref[2]
        ref[3,1,1]=(-0.421746456038)*x_ref[0]**o+(0.370489549278)*x_ref[0]+(-0.683682295105)*x_ref[1]**o+(0.324051718035)*x_ref[1]+(-0.579046430636)*x_ref[2]**o+(0.587623604941)*x_ref[2]
        ref[4,0,0]=(-0.167582584882)*x_ref[0]**o+(0.524904122243)*x_ref[0]+(0.665109348362)*x_ref[1]**o+(-0.093655366509)*x_ref[1]+(0.704404656254)*x_ref[2]**o+(-0.402602441741)*x_ref[2]
        ref[4,0,1]=(0.633854964824)*x_ref[0]**o+(-0.836385840194)*x_ref[0]+(-0.0995437996628)*x_ref[1]**o+(-0.200285338869)*x_ref[1]+(0.535023783881)*x_ref[2]**o+(0.884702543593)*x_ref[2]
        ref[4,1,0]=(-0.953619378208)*x_ref[0]**o+(-0.390518815669)*x_ref[0]+(-0.0590889178236)*x_ref[1]**o+(-0.997225825019)*x_ref[1]+(0.0520418736402)*x_ref[2]**o+(0.484749465262)*x_ref[2]
        ref[4,1,1]=(-0.440389854756)*x_ref[0]**o+(0.15520431778)*x_ref[0]+(0.346839174513)*x_ref[1]**o+(0.0172676873159)*x_ref[1]+(0.86275919224)*x_ref[2]**o+(0.568069148992)*x_ref[2]
        ref[5,0,0]=(-0.303687941217)*x_ref[0]**o+(0.516115915687)*x_ref[0]+(-0.750276597519)*x_ref[1]**o+(0.734039058778)*x_ref[1]+(-0.624499210252)*x_ref[2]**o+(-0.926338135421)*x_ref[2]
        ref[5,0,1]=(-0.472230777895)*x_ref[0]**o+(-0.557827728373)*x_ref[0]+(-0.403783023157)*x_ref[1]**o+(0.70433336955)*x_ref[1]+(-0.0211737454531)*x_ref[2]**o+(0.0774406125836)*x_ref[2]
        ref[5,1,0]=(0.718149875204)*x_ref[0]**o+(-0.183000739787)*x_ref[0]+(-0.655724252373)*x_ref[1]**o+(-0.634941710776)*x_ref[1]+(-0.26300163323)*x_ref[2]**o+(-0.25105424715)*x_ref[2]
        ref[5,1,1]=(0.109601904441)*x_ref[0]**o+(-0.00640764631554)*x_ref[0]+(-0.488035610009)*x_ref[1]**o+(0.909668323769)*x_ref[1]+(0.229692168501)*x_ref[2]**o+(-0.739241309497)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_Solution_rank4(self):
      """
      tests interpolate for rank 4 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from Solution onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5, 3, 2),w)
      ref=Data(0,(4, 5, 3, 2),w_ref)
      if dim==2:
        arg[0,0,0,0]=(0.912518456494)*x[0]**o+(-0.828176151903)*x[0]+(-0.313940298121)*x[1]**o+(-0.850120336531)*x[1]
        arg[0,0,0,1]=(0.14415852783)*x[0]**o+(-0.294155246668)*x[0]+(0.562517248232)*x[1]**o+(-0.204505938286)*x[1]
        arg[0,0,1,0]=(0.555455404368)*x[0]**o+(0.916425343576)*x[0]+(0.547252315693)*x[1]**o+(-0.0992842367479)*x[1]
        arg[0,0,1,1]=(0.603632473928)*x[0]**o+(0.505571549062)*x[0]+(-0.490402921542)*x[1]**o+(0.908624427316)*x[1]
        arg[0,0,2,0]=(0.729542308985)*x[0]**o+(-0.654310868638)*x[0]+(0.0845981276245)*x[1]**o+(-0.95231604686)*x[1]
        arg[0,0,2,1]=(0.0511615606745)*x[0]**o+(-0.939469887975)*x[0]+(-0.206229418155)*x[1]**o+(0.254879844701)*x[1]
        arg[0,1,0,0]=(0.257917344507)*x[0]**o+(0.860729621288)*x[0]+(0.489966193348)*x[1]**o+(0.174897233858)*x[1]
        arg[0,1,0,1]=(0.336588362051)*x[0]**o+(-0.690598635114)*x[0]+(-0.0913735300483)*x[1]**o+(-0.932914439004)*x[1]
        arg[0,1,1,0]=(0.244583093108)*x[0]**o+(-0.827768167758)*x[0]+(0.573065359412)*x[1]**o+(0.176685515596)*x[1]
        arg[0,1,1,1]=(-0.520550726127)*x[0]**o+(-0.605243177286)*x[0]+(0.130549556226)*x[1]**o+(0.750019884067)*x[1]
        arg[0,1,2,0]=(-0.680201627614)*x[0]**o+(0.679925324882)*x[0]+(-0.060408604263)*x[1]**o+(0.568125689359)*x[1]
        arg[0,1,2,1]=(-0.111908859432)*x[0]**o+(0.256521037018)*x[0]+(0.332050616611)*x[1]**o+(-0.0500016397683)*x[1]
        arg[0,2,0,0]=(-0.484953933663)*x[0]**o+(0.628032211831)*x[0]+(-0.63297141454)*x[1]**o+(0.141016662755)*x[1]
        arg[0,2,0,1]=(-0.463920680465)*x[0]**o+(-0.868825520511)*x[0]+(-0.36594425006)*x[1]**o+(-0.823052216578)*x[1]
        arg[0,2,1,0]=(0.972337802357)*x[0]**o+(0.943605157343)*x[0]+(-0.113950717844)*x[1]**o+(-0.839596452897)*x[1]
        arg[0,2,1,1]=(-0.295562401302)*x[0]**o+(0.735770621017)*x[0]+(-0.183838912348)*x[1]**o+(-0.929365380252)*x[1]
        arg[0,2,2,0]=(0.912886298388)*x[0]**o+(0.847507982065)*x[0]+(-0.341306036323)*x[1]**o+(0.393627025978)*x[1]
        arg[0,2,2,1]=(-0.769650638386)*x[0]**o+(-0.619672356488)*x[0]+(-0.126894288906)*x[1]**o+(0.231547632139)*x[1]
        arg[0,3,0,0]=(-0.884442976834)*x[0]**o+(-0.0194505760676)*x[0]+(-0.338764675072)*x[1]**o+(0.723481600171)*x[1]
        arg[0,3,0,1]=(0.655973555091)*x[0]**o+(-0.556230649412)*x[0]+(-0.850356832441)*x[1]**o+(0.115267314853)*x[1]
        arg[0,3,1,0]=(0.659628443559)*x[0]**o+(-0.586677822491)*x[0]+(0.135930588518)*x[1]**o+(0.206866895948)*x[1]
        arg[0,3,1,1]=(-0.700258957557)*x[0]**o+(-0.699774198764)*x[0]+(-0.294009479186)*x[1]**o+(0.848159666503)*x[1]
        arg[0,3,2,0]=(-0.927501754488)*x[0]**o+(-0.981178071331)*x[0]+(0.614074194451)*x[1]**o+(0.711549573086)*x[1]
        arg[0,3,2,1]=(0.312434473198)*x[0]**o+(-0.241506125991)*x[0]+(-0.0840979088912)*x[1]**o+(0.902948463117)*x[1]
        arg[0,4,0,0]=(-0.892269860242)*x[0]**o+(0.823434435855)*x[0]+(0.50499383365)*x[1]**o+(0.763736541034)*x[1]
        arg[0,4,0,1]=(-0.606959691191)*x[0]**o+(0.945562536791)*x[0]+(-0.46271929481)*x[1]**o+(0.929514432096)*x[1]
        arg[0,4,1,0]=(0.953382685939)*x[0]**o+(0.553766359805)*x[0]+(-0.191004010293)*x[1]**o+(0.556369491698)*x[1]
        arg[0,4,1,1]=(-0.269402649165)*x[0]**o+(-0.158926232076)*x[0]+(-0.390038749681)*x[1]**o+(0.553655192666)*x[1]
        arg[0,4,2,0]=(0.0785586372006)*x[0]**o+(-0.778452221862)*x[0]+(-0.927627273292)*x[1]**o+(-0.187742405246)*x[1]
        arg[0,4,2,1]=(-0.965329945447)*x[0]**o+(0.304539006089)*x[0]+(0.213031689568)*x[1]**o+(-0.2807790149)*x[1]
        arg[1,0,0,0]=(0.981534698569)*x[0]**o+(0.466451685806)*x[0]+(0.845322931856)*x[1]**o+(0.582422032334)*x[1]
        arg[1,0,0,1]=(-0.0645798740394)*x[0]**o+(0.0475214206619)*x[0]+(-0.687759433076)*x[1]**o+(0.403440938285)*x[1]
        arg[1,0,1,0]=(-0.317887773392)*x[0]**o+(0.59101163416)*x[0]+(0.658626263291)*x[1]**o+(-0.618167594137)*x[1]
        arg[1,0,1,1]=(-0.314093092813)*x[0]**o+(0.699933899942)*x[0]+(0.824195260822)*x[1]**o+(-0.42875327345)*x[1]
        arg[1,0,2,0]=(-0.254254956887)*x[0]**o+(-0.912137080163)*x[0]+(0.360457121452)*x[1]**o+(-0.902596912588)*x[1]
        arg[1,0,2,1]=(-0.577214593683)*x[0]**o+(-0.419495663008)*x[0]+(-0.803459783331)*x[1]**o+(0.490885513568)*x[1]
        arg[1,1,0,0]=(0.477947766892)*x[0]**o+(-0.853046971581)*x[0]+(0.576044041601)*x[1]**o+(-0.447007701252)*x[1]
        arg[1,1,0,1]=(0.5886933409)*x[0]**o+(0.253650778351)*x[0]+(-0.198070684431)*x[1]**o+(0.189479364528)*x[1]
        arg[1,1,1,0]=(-0.528744478872)*x[0]**o+(-0.504072059557)*x[0]+(-0.0161832474015)*x[1]**o+(0.639592055966)*x[1]
        arg[1,1,1,1]=(-0.581844827169)*x[0]**o+(0.997883224538)*x[0]+(0.9354364489)*x[1]**o+(0.884060785136)*x[1]
        arg[1,1,2,0]=(0.858226955817)*x[0]**o+(0.707261698312)*x[0]+(-0.738309111076)*x[1]**o+(-0.644655976448)*x[1]
        arg[1,1,2,1]=(-0.254976445964)*x[0]**o+(-0.624982185158)*x[0]+(0.304854486996)*x[1]**o+(0.597089811682)*x[1]
        arg[1,2,0,0]=(-0.625190759215)*x[0]**o+(-0.267820197905)*x[0]+(-0.721452024377)*x[1]**o+(-0.830984245819)*x[1]
        arg[1,2,0,1]=(-0.972257608766)*x[0]**o+(0.981368890544)*x[0]+(-0.925886510064)*x[1]**o+(-0.337107356862)*x[1]
        arg[1,2,1,0]=(-0.178932954207)*x[0]**o+(-0.941789507374)*x[0]+(0.360726744794)*x[1]**o+(0.347621863123)*x[1]
        arg[1,2,1,1]=(-0.59790695611)*x[0]**o+(0.552509013166)*x[0]+(0.592412443998)*x[1]**o+(0.105369637971)*x[1]
        arg[1,2,2,0]=(0.300713195225)*x[0]**o+(0.814257503574)*x[0]+(-0.281407483492)*x[1]**o+(-0.803431681095)*x[1]
        arg[1,2,2,1]=(0.80113022942)*x[0]**o+(0.749336171665)*x[0]+(-0.0842928982283)*x[1]**o+(-0.974041792058)*x[1]
        arg[1,3,0,0]=(0.533118876666)*x[0]**o+(-0.00888178987567)*x[0]+(0.372869213762)*x[1]**o+(-0.652513683144)*x[1]
        arg[1,3,0,1]=(-0.295090222537)*x[0]**o+(0.772749124303)*x[0]+(-0.00857029468729)*x[1]**o+(0.927794431125)*x[1]
        arg[1,3,1,0]=(-0.152537465441)*x[0]**o+(0.154900534979)*x[0]+(-0.488743959543)*x[1]**o+(-0.0318497199232)*x[1]
        arg[1,3,1,1]=(-0.754333787921)*x[0]**o+(-0.0583007358701)*x[0]+(-0.050921106891)*x[1]**o+(0.161426452156)*x[1]
        arg[1,3,2,0]=(0.387357469451)*x[0]**o+(-0.213640305885)*x[0]+(-0.844609663023)*x[1]**o+(0.0761914594192)*x[1]
        arg[1,3,2,1]=(0.36964862459)*x[0]**o+(0.887888401952)*x[0]+(-0.933930511093)*x[1]**o+(0.769046614932)*x[1]
        arg[1,4,0,0]=(0.680878887266)*x[0]**o+(0.62930033679)*x[0]+(0.894793542585)*x[1]**o+(-0.636881609067)*x[1]
        arg[1,4,0,1]=(-0.346053756598)*x[0]**o+(0.225254433923)*x[0]+(-0.861052985621)*x[1]**o+(0.415123108136)*x[1]
        arg[1,4,1,0]=(-0.997758408183)*x[0]**o+(-0.265055614341)*x[0]+(-0.7568881617)*x[1]**o+(0.95144584777)*x[1]
        arg[1,4,1,1]=(0.227777912008)*x[0]**o+(-0.83647660817)*x[0]+(-0.275733410165)*x[1]**o+(-0.971560248343)*x[1]
        arg[1,4,2,0]=(0.718803859114)*x[0]**o+(-0.978966510682)*x[0]+(0.463102438649)*x[1]**o+(0.987805610534)*x[1]
        arg[1,4,2,1]=(-0.107543838917)*x[0]**o+(-0.362684018694)*x[0]+(-0.310926939373)*x[1]**o+(-0.571571044712)*x[1]
        arg[2,0,0,0]=(-0.475977416182)*x[0]**o+(-0.627173002609)*x[0]+(-0.369962970984)*x[1]**o+(-0.245077302537)*x[1]
        arg[2,0,0,1]=(0.913949611751)*x[0]**o+(0.645611424617)*x[0]+(0.639304734076)*x[1]**o+(-0.12171441998)*x[1]
        arg[2,0,1,0]=(0.251434098263)*x[0]**o+(-0.119898861407)*x[0]+(-0.365614523709)*x[1]**o+(-0.802928738296)*x[1]
        arg[2,0,1,1]=(0.756390927006)*x[0]**o+(0.246578936199)*x[0]+(0.714968085461)*x[1]**o+(-0.929428808096)*x[1]
        arg[2,0,2,0]=(-0.516888361447)*x[0]**o+(0.0344339250886)*x[0]+(-0.383709788774)*x[1]**o+(-0.437344417623)*x[1]
        arg[2,0,2,1]=(-0.219307162317)*x[0]**o+(0.704909718842)*x[0]+(-0.340454491855)*x[1]**o+(-0.933866231773)*x[1]
        arg[2,1,0,0]=(-0.0618967891627)*x[0]**o+(-0.0590712030789)*x[0]+(0.116776433077)*x[1]**o+(0.0241239764104)*x[1]
        arg[2,1,0,1]=(-0.802207645638)*x[0]**o+(0.590732740531)*x[0]+(-0.25990272933)*x[1]**o+(-0.257633805416)*x[1]
        arg[2,1,1,0]=(0.827623274379)*x[0]**o+(-0.450310573599)*x[0]+(0.213809775826)*x[1]**o+(-0.905410777787)*x[1]
        arg[2,1,1,1]=(-0.624510830356)*x[0]**o+(-0.705682607787)*x[0]+(0.430105716993)*x[1]**o+(0.975510832521)*x[1]
        arg[2,1,2,0]=(-0.297562223309)*x[0]**o+(-0.99090348994)*x[0]+(0.548589416291)*x[1]**o+(0.591718478583)*x[1]
        arg[2,1,2,1]=(-0.531899919376)*x[0]**o+(-0.227453719581)*x[0]+(-0.131664806128)*x[1]**o+(0.0521895817509)*x[1]
        arg[2,2,0,0]=(-0.592291745786)*x[0]**o+(-0.586879961972)*x[0]+(0.563196552327)*x[1]**o+(0.876989321187)*x[1]
        arg[2,2,0,1]=(-0.273360445369)*x[0]**o+(-0.714740527775)*x[0]+(-0.647699497837)*x[1]**o+(-0.0405089612762)*x[1]
        arg[2,2,1,0]=(-0.536629732224)*x[0]**o+(0.72656373966)*x[0]+(0.986630594165)*x[1]**o+(0.0741630711667)*x[1]
        arg[2,2,1,1]=(0.770746088838)*x[0]**o+(-0.219279833426)*x[0]+(0.875445174779)*x[1]**o+(-0.947889926444)*x[1]
        arg[2,2,2,0]=(-0.41428815119)*x[0]**o+(-0.696166268903)*x[0]+(-0.96838062553)*x[1]**o+(0.781394205899)*x[1]
        arg[2,2,2,1]=(-0.810267960752)*x[0]**o+(-0.873632687702)*x[0]+(0.0131971321611)*x[1]**o+(-0.552767192193)*x[1]
        arg[2,3,0,0]=(-0.216091366291)*x[0]**o+(-0.17824047472)*x[0]+(-0.4644515725)*x[1]**o+(0.135819458957)*x[1]
        arg[2,3,0,1]=(-0.0823616523105)*x[0]**o+(0.33121362151)*x[0]+(0.926719919839)*x[1]**o+(0.663475641636)*x[1]
        arg[2,3,1,0]=(-0.278303066047)*x[0]**o+(-0.457286378314)*x[0]+(0.186792859092)*x[1]**o+(-0.734245612464)*x[1]
        arg[2,3,1,1]=(0.810871459779)*x[0]**o+(-0.706641995324)*x[0]+(-0.287827176091)*x[1]**o+(0.453216090282)*x[1]
        arg[2,3,2,0]=(0.83826221049)*x[0]**o+(0.556887792111)*x[0]+(-0.000231102245991)*x[1]**o+(-0.440730958954)*x[1]
        arg[2,3,2,1]=(-0.455087877907)*x[0]**o+(-0.132259634036)*x[0]+(-0.492728335525)*x[1]**o+(-0.880311317777)*x[1]
        arg[2,4,0,0]=(-0.303141352388)*x[0]**o+(0.426137046804)*x[0]+(0.788934776738)*x[1]**o+(0.157477785179)*x[1]
        arg[2,4,0,1]=(0.603411288614)*x[0]**o+(-0.533561451987)*x[0]+(-0.901259210942)*x[1]**o+(0.547372624746)*x[1]
        arg[2,4,1,0]=(0.69517961739)*x[0]**o+(-0.106282888246)*x[0]+(0.257484565681)*x[1]**o+(0.655749633294)*x[1]
        arg[2,4,1,1]=(-0.072748573118)*x[0]**o+(0.905966560113)*x[0]+(0.433145718123)*x[1]**o+(0.187620401615)*x[1]
        arg[2,4,2,0]=(-0.0280176325954)*x[0]**o+(0.489610131347)*x[0]+(-0.740554922134)*x[1]**o+(-0.85045302096)*x[1]
        arg[2,4,2,1]=(-0.158503180709)*x[0]**o+(0.525929643075)*x[0]+(-0.352022652515)*x[1]**o+(-0.706553104932)*x[1]
        arg[3,0,0,0]=(-0.837316508127)*x[0]**o+(-0.89490181779)*x[0]+(0.510084075291)*x[1]**o+(0.321206310024)*x[1]
        arg[3,0,0,1]=(-0.325038591079)*x[0]**o+(0.140342922753)*x[0]+(-0.937466403004)*x[1]**o+(0.580557383497)*x[1]
        arg[3,0,1,0]=(-0.939713654062)*x[0]**o+(-0.394660244166)*x[0]+(0.762376614557)*x[1]**o+(-0.135565547532)*x[1]
        arg[3,0,1,1]=(0.0275332524464)*x[0]**o+(0.368737701882)*x[0]+(0.983641372218)*x[1]**o+(0.573815516366)*x[1]
        arg[3,0,2,0]=(0.816957570314)*x[0]**o+(-0.357626892562)*x[0]+(-0.368933149642)*x[1]**o+(-0.692502486914)*x[1]
        arg[3,0,2,1]=(-0.372408810926)*x[0]**o+(-0.113036390416)*x[0]+(-0.355880297003)*x[1]**o+(0.755478191481)*x[1]
        arg[3,1,0,0]=(0.0399012900832)*x[0]**o+(-0.879576900086)*x[0]+(-0.660858776134)*x[1]**o+(-0.59837040429)*x[1]
        arg[3,1,0,1]=(0.569397128496)*x[0]**o+(-0.138333417746)*x[0]+(0.257769147727)*x[1]**o+(0.174737141027)*x[1]
        arg[3,1,1,0]=(-0.67938408823)*x[0]**o+(0.862731161981)*x[0]+(-0.792538485264)*x[1]**o+(-0.0805197796157)*x[1]
        arg[3,1,1,1]=(0.0113523229279)*x[0]**o+(0.475393597561)*x[0]+(-0.705748021886)*x[1]**o+(0.00836460135285)*x[1]
        arg[3,1,2,0]=(0.463839509799)*x[0]**o+(-0.12663544643)*x[0]+(0.353158399738)*x[1]**o+(0.674929537597)*x[1]
        arg[3,1,2,1]=(-0.6019896019)*x[0]**o+(-0.638280685565)*x[0]+(0.280047072988)*x[1]**o+(0.569239958138)*x[1]
        arg[3,2,0,0]=(-0.113906500346)*x[0]**o+(-0.493494498501)*x[0]+(-0.686419229162)*x[1]**o+(-0.0727046763516)*x[1]
        arg[3,2,0,1]=(-0.302092112795)*x[0]**o+(-0.74819167)*x[0]+(-0.478908975282)*x[1]**o+(0.430024789627)*x[1]
        arg[3,2,1,0]=(0.353535608984)*x[0]**o+(-0.743408549264)*x[0]+(-0.069122881538)*x[1]**o+(-0.0767595089529)*x[1]
        arg[3,2,1,1]=(-0.629946386234)*x[0]**o+(0.851779590894)*x[0]+(0.728577108695)*x[1]**o+(-0.366983040147)*x[1]
        arg[3,2,2,0]=(0.1685167508)*x[0]**o+(0.149999747637)*x[0]+(0.709320430374)*x[1]**o+(-0.280950677222)*x[1]
        arg[3,2,2,1]=(-0.956164974578)*x[0]**o+(0.240460380901)*x[0]+(-0.2820176577)*x[1]**o+(-0.595667156955)*x[1]
        arg[3,3,0,0]=(-0.0711295055938)*x[0]**o+(-0.314513251791)*x[0]+(-0.41924591293)*x[1]**o+(0.0813197780959)*x[1]
        arg[3,3,0,1]=(-0.230592584877)*x[0]**o+(-0.488005144715)*x[0]+(-0.450611759976)*x[1]**o+(-0.0571152769768)*x[1]
        arg[3,3,1,0]=(-0.608255699014)*x[0]**o+(-0.697324108575)*x[0]+(-0.885935900746)*x[1]**o+(-0.884497550016)*x[1]
        arg[3,3,1,1]=(0.857286225292)*x[0]**o+(0.509954550626)*x[0]+(0.182030821433)*x[1]**o+(-0.268076060316)*x[1]
        arg[3,3,2,0]=(-0.77322653479)*x[0]**o+(-0.583013210018)*x[0]+(-0.683867300277)*x[1]**o+(0.754530541251)*x[1]
        arg[3,3,2,1]=(-0.744667985181)*x[0]**o+(0.164165128978)*x[0]+(0.412340155958)*x[1]**o+(-0.38451107118)*x[1]
        arg[3,4,0,0]=(0.216720755049)*x[0]**o+(-0.347803433722)*x[0]+(0.417376268797)*x[1]**o+(0.475787304652)*x[1]
        arg[3,4,0,1]=(-0.737038295974)*x[0]**o+(0.579172519178)*x[0]+(-0.900272993672)*x[1]**o+(0.407018603625)*x[1]
        arg[3,4,1,0]=(-0.898959236474)*x[0]**o+(-0.886636008121)*x[0]+(-0.901972610044)*x[1]**o+(-0.518393782318)*x[1]
        arg[3,4,1,1]=(0.976342884921)*x[0]**o+(-0.273568319493)*x[0]+(-0.332424980846)*x[1]**o+(-0.0146661731139)*x[1]
        arg[3,4,2,0]=(0.794792214027)*x[0]**o+(0.846927984413)*x[0]+(0.535818809709)*x[1]**o+(-0.49324400033)*x[1]
        arg[3,4,2,1]=(0.695766308812)*x[0]**o+(-0.37923153689)*x[0]+(0.0633513026782)*x[1]**o+(0.12334892105)*x[1]
        ref[0,0,0,0]=(0.912518456494)*x_ref[0]**o+(-0.828176151903)*x_ref[0]+(-0.313940298121)*x_ref[1]**o+(-0.850120336531)*x_ref[1]
        ref[0,0,0,1]=(0.14415852783)*x_ref[0]**o+(-0.294155246668)*x_ref[0]+(0.562517248232)*x_ref[1]**o+(-0.204505938286)*x_ref[1]
        ref[0,0,1,0]=(0.555455404368)*x_ref[0]**o+(0.916425343576)*x_ref[0]+(0.547252315693)*x_ref[1]**o+(-0.0992842367479)*x_ref[1]
        ref[0,0,1,1]=(0.603632473928)*x_ref[0]**o+(0.505571549062)*x_ref[0]+(-0.490402921542)*x_ref[1]**o+(0.908624427316)*x_ref[1]
        ref[0,0,2,0]=(0.729542308985)*x_ref[0]**o+(-0.654310868638)*x_ref[0]+(0.0845981276245)*x_ref[1]**o+(-0.95231604686)*x_ref[1]
        ref[0,0,2,1]=(0.0511615606745)*x_ref[0]**o+(-0.939469887975)*x_ref[0]+(-0.206229418155)*x_ref[1]**o+(0.254879844701)*x_ref[1]
        ref[0,1,0,0]=(0.257917344507)*x_ref[0]**o+(0.860729621288)*x_ref[0]+(0.489966193348)*x_ref[1]**o+(0.174897233858)*x_ref[1]
        ref[0,1,0,1]=(0.336588362051)*x_ref[0]**o+(-0.690598635114)*x_ref[0]+(-0.0913735300483)*x_ref[1]**o+(-0.932914439004)*x_ref[1]
        ref[0,1,1,0]=(0.244583093108)*x_ref[0]**o+(-0.827768167758)*x_ref[0]+(0.573065359412)*x_ref[1]**o+(0.176685515596)*x_ref[1]
        ref[0,1,1,1]=(-0.520550726127)*x_ref[0]**o+(-0.605243177286)*x_ref[0]+(0.130549556226)*x_ref[1]**o+(0.750019884067)*x_ref[1]
        ref[0,1,2,0]=(-0.680201627614)*x_ref[0]**o+(0.679925324882)*x_ref[0]+(-0.060408604263)*x_ref[1]**o+(0.568125689359)*x_ref[1]
        ref[0,1,2,1]=(-0.111908859432)*x_ref[0]**o+(0.256521037018)*x_ref[0]+(0.332050616611)*x_ref[1]**o+(-0.0500016397683)*x_ref[1]
        ref[0,2,0,0]=(-0.484953933663)*x_ref[0]**o+(0.628032211831)*x_ref[0]+(-0.63297141454)*x_ref[1]**o+(0.141016662755)*x_ref[1]
        ref[0,2,0,1]=(-0.463920680465)*x_ref[0]**o+(-0.868825520511)*x_ref[0]+(-0.36594425006)*x_ref[1]**o+(-0.823052216578)*x_ref[1]
        ref[0,2,1,0]=(0.972337802357)*x_ref[0]**o+(0.943605157343)*x_ref[0]+(-0.113950717844)*x_ref[1]**o+(-0.839596452897)*x_ref[1]
        ref[0,2,1,1]=(-0.295562401302)*x_ref[0]**o+(0.735770621017)*x_ref[0]+(-0.183838912348)*x_ref[1]**o+(-0.929365380252)*x_ref[1]
        ref[0,2,2,0]=(0.912886298388)*x_ref[0]**o+(0.847507982065)*x_ref[0]+(-0.341306036323)*x_ref[1]**o+(0.393627025978)*x_ref[1]
        ref[0,2,2,1]=(-0.769650638386)*x_ref[0]**o+(-0.619672356488)*x_ref[0]+(-0.126894288906)*x_ref[1]**o+(0.231547632139)*x_ref[1]
        ref[0,3,0,0]=(-0.884442976834)*x_ref[0]**o+(-0.0194505760676)*x_ref[0]+(-0.338764675072)*x_ref[1]**o+(0.723481600171)*x_ref[1]
        ref[0,3,0,1]=(0.655973555091)*x_ref[0]**o+(-0.556230649412)*x_ref[0]+(-0.850356832441)*x_ref[1]**o+(0.115267314853)*x_ref[1]
        ref[0,3,1,0]=(0.659628443559)*x_ref[0]**o+(-0.586677822491)*x_ref[0]+(0.135930588518)*x_ref[1]**o+(0.206866895948)*x_ref[1]
        ref[0,3,1,1]=(-0.700258957557)*x_ref[0]**o+(-0.699774198764)*x_ref[0]+(-0.294009479186)*x_ref[1]**o+(0.848159666503)*x_ref[1]
        ref[0,3,2,0]=(-0.927501754488)*x_ref[0]**o+(-0.981178071331)*x_ref[0]+(0.614074194451)*x_ref[1]**o+(0.711549573086)*x_ref[1]
        ref[0,3,2,1]=(0.312434473198)*x_ref[0]**o+(-0.241506125991)*x_ref[0]+(-0.0840979088912)*x_ref[1]**o+(0.902948463117)*x_ref[1]
        ref[0,4,0,0]=(-0.892269860242)*x_ref[0]**o+(0.823434435855)*x_ref[0]+(0.50499383365)*x_ref[1]**o+(0.763736541034)*x_ref[1]
        ref[0,4,0,1]=(-0.606959691191)*x_ref[0]**o+(0.945562536791)*x_ref[0]+(-0.46271929481)*x_ref[1]**o+(0.929514432096)*x_ref[1]
        ref[0,4,1,0]=(0.953382685939)*x_ref[0]**o+(0.553766359805)*x_ref[0]+(-0.191004010293)*x_ref[1]**o+(0.556369491698)*x_ref[1]
        ref[0,4,1,1]=(-0.269402649165)*x_ref[0]**o+(-0.158926232076)*x_ref[0]+(-0.390038749681)*x_ref[1]**o+(0.553655192666)*x_ref[1]
        ref[0,4,2,0]=(0.0785586372006)*x_ref[0]**o+(-0.778452221862)*x_ref[0]+(-0.927627273292)*x_ref[1]**o+(-0.187742405246)*x_ref[1]
        ref[0,4,2,1]=(-0.965329945447)*x_ref[0]**o+(0.304539006089)*x_ref[0]+(0.213031689568)*x_ref[1]**o+(-0.2807790149)*x_ref[1]
        ref[1,0,0,0]=(0.981534698569)*x_ref[0]**o+(0.466451685806)*x_ref[0]+(0.845322931856)*x_ref[1]**o+(0.582422032334)*x_ref[1]
        ref[1,0,0,1]=(-0.0645798740394)*x_ref[0]**o+(0.0475214206619)*x_ref[0]+(-0.687759433076)*x_ref[1]**o+(0.403440938285)*x_ref[1]
        ref[1,0,1,0]=(-0.317887773392)*x_ref[0]**o+(0.59101163416)*x_ref[0]+(0.658626263291)*x_ref[1]**o+(-0.618167594137)*x_ref[1]
        ref[1,0,1,1]=(-0.314093092813)*x_ref[0]**o+(0.699933899942)*x_ref[0]+(0.824195260822)*x_ref[1]**o+(-0.42875327345)*x_ref[1]
        ref[1,0,2,0]=(-0.254254956887)*x_ref[0]**o+(-0.912137080163)*x_ref[0]+(0.360457121452)*x_ref[1]**o+(-0.902596912588)*x_ref[1]
        ref[1,0,2,1]=(-0.577214593683)*x_ref[0]**o+(-0.419495663008)*x_ref[0]+(-0.803459783331)*x_ref[1]**o+(0.490885513568)*x_ref[1]
        ref[1,1,0,0]=(0.477947766892)*x_ref[0]**o+(-0.853046971581)*x_ref[0]+(0.576044041601)*x_ref[1]**o+(-0.447007701252)*x_ref[1]
        ref[1,1,0,1]=(0.5886933409)*x_ref[0]**o+(0.253650778351)*x_ref[0]+(-0.198070684431)*x_ref[1]**o+(0.189479364528)*x_ref[1]
        ref[1,1,1,0]=(-0.528744478872)*x_ref[0]**o+(-0.504072059557)*x_ref[0]+(-0.0161832474015)*x_ref[1]**o+(0.639592055966)*x_ref[1]
        ref[1,1,1,1]=(-0.581844827169)*x_ref[0]**o+(0.997883224538)*x_ref[0]+(0.9354364489)*x_ref[1]**o+(0.884060785136)*x_ref[1]
        ref[1,1,2,0]=(0.858226955817)*x_ref[0]**o+(0.707261698312)*x_ref[0]+(-0.738309111076)*x_ref[1]**o+(-0.644655976448)*x_ref[1]
        ref[1,1,2,1]=(-0.254976445964)*x_ref[0]**o+(-0.624982185158)*x_ref[0]+(0.304854486996)*x_ref[1]**o+(0.597089811682)*x_ref[1]
        ref[1,2,0,0]=(-0.625190759215)*x_ref[0]**o+(-0.267820197905)*x_ref[0]+(-0.721452024377)*x_ref[1]**o+(-0.830984245819)*x_ref[1]
        ref[1,2,0,1]=(-0.972257608766)*x_ref[0]**o+(0.981368890544)*x_ref[0]+(-0.925886510064)*x_ref[1]**o+(-0.337107356862)*x_ref[1]
        ref[1,2,1,0]=(-0.178932954207)*x_ref[0]**o+(-0.941789507374)*x_ref[0]+(0.360726744794)*x_ref[1]**o+(0.347621863123)*x_ref[1]
        ref[1,2,1,1]=(-0.59790695611)*x_ref[0]**o+(0.552509013166)*x_ref[0]+(0.592412443998)*x_ref[1]**o+(0.105369637971)*x_ref[1]
        ref[1,2,2,0]=(0.300713195225)*x_ref[0]**o+(0.814257503574)*x_ref[0]+(-0.281407483492)*x_ref[1]**o+(-0.803431681095)*x_ref[1]
        ref[1,2,2,1]=(0.80113022942)*x_ref[0]**o+(0.749336171665)*x_ref[0]+(-0.0842928982283)*x_ref[1]**o+(-0.974041792058)*x_ref[1]
        ref[1,3,0,0]=(0.533118876666)*x_ref[0]**o+(-0.00888178987567)*x_ref[0]+(0.372869213762)*x_ref[1]**o+(-0.652513683144)*x_ref[1]
        ref[1,3,0,1]=(-0.295090222537)*x_ref[0]**o+(0.772749124303)*x_ref[0]+(-0.00857029468729)*x_ref[1]**o+(0.927794431125)*x_ref[1]
        ref[1,3,1,0]=(-0.152537465441)*x_ref[0]**o+(0.154900534979)*x_ref[0]+(-0.488743959543)*x_ref[1]**o+(-0.0318497199232)*x_ref[1]
        ref[1,3,1,1]=(-0.754333787921)*x_ref[0]**o+(-0.0583007358701)*x_ref[0]+(-0.050921106891)*x_ref[1]**o+(0.161426452156)*x_ref[1]
        ref[1,3,2,0]=(0.387357469451)*x_ref[0]**o+(-0.213640305885)*x_ref[0]+(-0.844609663023)*x_ref[1]**o+(0.0761914594192)*x_ref[1]
        ref[1,3,2,1]=(0.36964862459)*x_ref[0]**o+(0.887888401952)*x_ref[0]+(-0.933930511093)*x_ref[1]**o+(0.769046614932)*x_ref[1]
        ref[1,4,0,0]=(0.680878887266)*x_ref[0]**o+(0.62930033679)*x_ref[0]+(0.894793542585)*x_ref[1]**o+(-0.636881609067)*x_ref[1]
        ref[1,4,0,1]=(-0.346053756598)*x_ref[0]**o+(0.225254433923)*x_ref[0]+(-0.861052985621)*x_ref[1]**o+(0.415123108136)*x_ref[1]
        ref[1,4,1,0]=(-0.997758408183)*x_ref[0]**o+(-0.265055614341)*x_ref[0]+(-0.7568881617)*x_ref[1]**o+(0.95144584777)*x_ref[1]
        ref[1,4,1,1]=(0.227777912008)*x_ref[0]**o+(-0.83647660817)*x_ref[0]+(-0.275733410165)*x_ref[1]**o+(-0.971560248343)*x_ref[1]
        ref[1,4,2,0]=(0.718803859114)*x_ref[0]**o+(-0.978966510682)*x_ref[0]+(0.463102438649)*x_ref[1]**o+(0.987805610534)*x_ref[1]
        ref[1,4,2,1]=(-0.107543838917)*x_ref[0]**o+(-0.362684018694)*x_ref[0]+(-0.310926939373)*x_ref[1]**o+(-0.571571044712)*x_ref[1]
        ref[2,0,0,0]=(-0.475977416182)*x_ref[0]**o+(-0.627173002609)*x_ref[0]+(-0.369962970984)*x_ref[1]**o+(-0.245077302537)*x_ref[1]
        ref[2,0,0,1]=(0.913949611751)*x_ref[0]**o+(0.645611424617)*x_ref[0]+(0.639304734076)*x_ref[1]**o+(-0.12171441998)*x_ref[1]
        ref[2,0,1,0]=(0.251434098263)*x_ref[0]**o+(-0.119898861407)*x_ref[0]+(-0.365614523709)*x_ref[1]**o+(-0.802928738296)*x_ref[1]
        ref[2,0,1,1]=(0.756390927006)*x_ref[0]**o+(0.246578936199)*x_ref[0]+(0.714968085461)*x_ref[1]**o+(-0.929428808096)*x_ref[1]
        ref[2,0,2,0]=(-0.516888361447)*x_ref[0]**o+(0.0344339250886)*x_ref[0]+(-0.383709788774)*x_ref[1]**o+(-0.437344417623)*x_ref[1]
        ref[2,0,2,1]=(-0.219307162317)*x_ref[0]**o+(0.704909718842)*x_ref[0]+(-0.340454491855)*x_ref[1]**o+(-0.933866231773)*x_ref[1]
        ref[2,1,0,0]=(-0.0618967891627)*x_ref[0]**o+(-0.0590712030789)*x_ref[0]+(0.116776433077)*x_ref[1]**o+(0.0241239764104)*x_ref[1]
        ref[2,1,0,1]=(-0.802207645638)*x_ref[0]**o+(0.590732740531)*x_ref[0]+(-0.25990272933)*x_ref[1]**o+(-0.257633805416)*x_ref[1]
        ref[2,1,1,0]=(0.827623274379)*x_ref[0]**o+(-0.450310573599)*x_ref[0]+(0.213809775826)*x_ref[1]**o+(-0.905410777787)*x_ref[1]
        ref[2,1,1,1]=(-0.624510830356)*x_ref[0]**o+(-0.705682607787)*x_ref[0]+(0.430105716993)*x_ref[1]**o+(0.975510832521)*x_ref[1]
        ref[2,1,2,0]=(-0.297562223309)*x_ref[0]**o+(-0.99090348994)*x_ref[0]+(0.548589416291)*x_ref[1]**o+(0.591718478583)*x_ref[1]
        ref[2,1,2,1]=(-0.531899919376)*x_ref[0]**o+(-0.227453719581)*x_ref[0]+(-0.131664806128)*x_ref[1]**o+(0.0521895817509)*x_ref[1]
        ref[2,2,0,0]=(-0.592291745786)*x_ref[0]**o+(-0.586879961972)*x_ref[0]+(0.563196552327)*x_ref[1]**o+(0.876989321187)*x_ref[1]
        ref[2,2,0,1]=(-0.273360445369)*x_ref[0]**o+(-0.714740527775)*x_ref[0]+(-0.647699497837)*x_ref[1]**o+(-0.0405089612762)*x_ref[1]
        ref[2,2,1,0]=(-0.536629732224)*x_ref[0]**o+(0.72656373966)*x_ref[0]+(0.986630594165)*x_ref[1]**o+(0.0741630711667)*x_ref[1]
        ref[2,2,1,1]=(0.770746088838)*x_ref[0]**o+(-0.219279833426)*x_ref[0]+(0.875445174779)*x_ref[1]**o+(-0.947889926444)*x_ref[1]
        ref[2,2,2,0]=(-0.41428815119)*x_ref[0]**o+(-0.696166268903)*x_ref[0]+(-0.96838062553)*x_ref[1]**o+(0.781394205899)*x_ref[1]
        ref[2,2,2,1]=(-0.810267960752)*x_ref[0]**o+(-0.873632687702)*x_ref[0]+(0.0131971321611)*x_ref[1]**o+(-0.552767192193)*x_ref[1]
        ref[2,3,0,0]=(-0.216091366291)*x_ref[0]**o+(-0.17824047472)*x_ref[0]+(-0.4644515725)*x_ref[1]**o+(0.135819458957)*x_ref[1]
        ref[2,3,0,1]=(-0.0823616523105)*x_ref[0]**o+(0.33121362151)*x_ref[0]+(0.926719919839)*x_ref[1]**o+(0.663475641636)*x_ref[1]
        ref[2,3,1,0]=(-0.278303066047)*x_ref[0]**o+(-0.457286378314)*x_ref[0]+(0.186792859092)*x_ref[1]**o+(-0.734245612464)*x_ref[1]
        ref[2,3,1,1]=(0.810871459779)*x_ref[0]**o+(-0.706641995324)*x_ref[0]+(-0.287827176091)*x_ref[1]**o+(0.453216090282)*x_ref[1]
        ref[2,3,2,0]=(0.83826221049)*x_ref[0]**o+(0.556887792111)*x_ref[0]+(-0.000231102245991)*x_ref[1]**o+(-0.440730958954)*x_ref[1]
        ref[2,3,2,1]=(-0.455087877907)*x_ref[0]**o+(-0.132259634036)*x_ref[0]+(-0.492728335525)*x_ref[1]**o+(-0.880311317777)*x_ref[1]
        ref[2,4,0,0]=(-0.303141352388)*x_ref[0]**o+(0.426137046804)*x_ref[0]+(0.788934776738)*x_ref[1]**o+(0.157477785179)*x_ref[1]
        ref[2,4,0,1]=(0.603411288614)*x_ref[0]**o+(-0.533561451987)*x_ref[0]+(-0.901259210942)*x_ref[1]**o+(0.547372624746)*x_ref[1]
        ref[2,4,1,0]=(0.69517961739)*x_ref[0]**o+(-0.106282888246)*x_ref[0]+(0.257484565681)*x_ref[1]**o+(0.655749633294)*x_ref[1]
        ref[2,4,1,1]=(-0.072748573118)*x_ref[0]**o+(0.905966560113)*x_ref[0]+(0.433145718123)*x_ref[1]**o+(0.187620401615)*x_ref[1]
        ref[2,4,2,0]=(-0.0280176325954)*x_ref[0]**o+(0.489610131347)*x_ref[0]+(-0.740554922134)*x_ref[1]**o+(-0.85045302096)*x_ref[1]
        ref[2,4,2,1]=(-0.158503180709)*x_ref[0]**o+(0.525929643075)*x_ref[0]+(-0.352022652515)*x_ref[1]**o+(-0.706553104932)*x_ref[1]
        ref[3,0,0,0]=(-0.837316508127)*x_ref[0]**o+(-0.89490181779)*x_ref[0]+(0.510084075291)*x_ref[1]**o+(0.321206310024)*x_ref[1]
        ref[3,0,0,1]=(-0.325038591079)*x_ref[0]**o+(0.140342922753)*x_ref[0]+(-0.937466403004)*x_ref[1]**o+(0.580557383497)*x_ref[1]
        ref[3,0,1,0]=(-0.939713654062)*x_ref[0]**o+(-0.394660244166)*x_ref[0]+(0.762376614557)*x_ref[1]**o+(-0.135565547532)*x_ref[1]
        ref[3,0,1,1]=(0.0275332524464)*x_ref[0]**o+(0.368737701882)*x_ref[0]+(0.983641372218)*x_ref[1]**o+(0.573815516366)*x_ref[1]
        ref[3,0,2,0]=(0.816957570314)*x_ref[0]**o+(-0.357626892562)*x_ref[0]+(-0.368933149642)*x_ref[1]**o+(-0.692502486914)*x_ref[1]
        ref[3,0,2,1]=(-0.372408810926)*x_ref[0]**o+(-0.113036390416)*x_ref[0]+(-0.355880297003)*x_ref[1]**o+(0.755478191481)*x_ref[1]
        ref[3,1,0,0]=(0.0399012900832)*x_ref[0]**o+(-0.879576900086)*x_ref[0]+(-0.660858776134)*x_ref[1]**o+(-0.59837040429)*x_ref[1]
        ref[3,1,0,1]=(0.569397128496)*x_ref[0]**o+(-0.138333417746)*x_ref[0]+(0.257769147727)*x_ref[1]**o+(0.174737141027)*x_ref[1]
        ref[3,1,1,0]=(-0.67938408823)*x_ref[0]**o+(0.862731161981)*x_ref[0]+(-0.792538485264)*x_ref[1]**o+(-0.0805197796157)*x_ref[1]
        ref[3,1,1,1]=(0.0113523229279)*x_ref[0]**o+(0.475393597561)*x_ref[0]+(-0.705748021886)*x_ref[1]**o+(0.00836460135285)*x_ref[1]
        ref[3,1,2,0]=(0.463839509799)*x_ref[0]**o+(-0.12663544643)*x_ref[0]+(0.353158399738)*x_ref[1]**o+(0.674929537597)*x_ref[1]
        ref[3,1,2,1]=(-0.6019896019)*x_ref[0]**o+(-0.638280685565)*x_ref[0]+(0.280047072988)*x_ref[1]**o+(0.569239958138)*x_ref[1]
        ref[3,2,0,0]=(-0.113906500346)*x_ref[0]**o+(-0.493494498501)*x_ref[0]+(-0.686419229162)*x_ref[1]**o+(-0.0727046763516)*x_ref[1]
        ref[3,2,0,1]=(-0.302092112795)*x_ref[0]**o+(-0.74819167)*x_ref[0]+(-0.478908975282)*x_ref[1]**o+(0.430024789627)*x_ref[1]
        ref[3,2,1,0]=(0.353535608984)*x_ref[0]**o+(-0.743408549264)*x_ref[0]+(-0.069122881538)*x_ref[1]**o+(-0.0767595089529)*x_ref[1]
        ref[3,2,1,1]=(-0.629946386234)*x_ref[0]**o+(0.851779590894)*x_ref[0]+(0.728577108695)*x_ref[1]**o+(-0.366983040147)*x_ref[1]
        ref[3,2,2,0]=(0.1685167508)*x_ref[0]**o+(0.149999747637)*x_ref[0]+(0.709320430374)*x_ref[1]**o+(-0.280950677222)*x_ref[1]
        ref[3,2,2,1]=(-0.956164974578)*x_ref[0]**o+(0.240460380901)*x_ref[0]+(-0.2820176577)*x_ref[1]**o+(-0.595667156955)*x_ref[1]
        ref[3,3,0,0]=(-0.0711295055938)*x_ref[0]**o+(-0.314513251791)*x_ref[0]+(-0.41924591293)*x_ref[1]**o+(0.0813197780959)*x_ref[1]
        ref[3,3,0,1]=(-0.230592584877)*x_ref[0]**o+(-0.488005144715)*x_ref[0]+(-0.450611759976)*x_ref[1]**o+(-0.0571152769768)*x_ref[1]
        ref[3,3,1,0]=(-0.608255699014)*x_ref[0]**o+(-0.697324108575)*x_ref[0]+(-0.885935900746)*x_ref[1]**o+(-0.884497550016)*x_ref[1]
        ref[3,3,1,1]=(0.857286225292)*x_ref[0]**o+(0.509954550626)*x_ref[0]+(0.182030821433)*x_ref[1]**o+(-0.268076060316)*x_ref[1]
        ref[3,3,2,0]=(-0.77322653479)*x_ref[0]**o+(-0.583013210018)*x_ref[0]+(-0.683867300277)*x_ref[1]**o+(0.754530541251)*x_ref[1]
        ref[3,3,2,1]=(-0.744667985181)*x_ref[0]**o+(0.164165128978)*x_ref[0]+(0.412340155958)*x_ref[1]**o+(-0.38451107118)*x_ref[1]
        ref[3,4,0,0]=(0.216720755049)*x_ref[0]**o+(-0.347803433722)*x_ref[0]+(0.417376268797)*x_ref[1]**o+(0.475787304652)*x_ref[1]
        ref[3,4,0,1]=(-0.737038295974)*x_ref[0]**o+(0.579172519178)*x_ref[0]+(-0.900272993672)*x_ref[1]**o+(0.407018603625)*x_ref[1]
        ref[3,4,1,0]=(-0.898959236474)*x_ref[0]**o+(-0.886636008121)*x_ref[0]+(-0.901972610044)*x_ref[1]**o+(-0.518393782318)*x_ref[1]
        ref[3,4,1,1]=(0.976342884921)*x_ref[0]**o+(-0.273568319493)*x_ref[0]+(-0.332424980846)*x_ref[1]**o+(-0.0146661731139)*x_ref[1]
        ref[3,4,2,0]=(0.794792214027)*x_ref[0]**o+(0.846927984413)*x_ref[0]+(0.535818809709)*x_ref[1]**o+(-0.49324400033)*x_ref[1]
        ref[3,4,2,1]=(0.695766308812)*x_ref[0]**o+(-0.37923153689)*x_ref[0]+(0.0633513026782)*x_ref[1]**o+(0.12334892105)*x_ref[1]
      else:
        arg[0,0,0,0]=(-0.401901998082)*x[0]**o+(-0.805091116808)*x[0]+(0.997659882792)*x[1]**o+(-0.953957389492)*x[1]+(-0.450185408289)*x[2]**o+(0.636472207773)*x[2]
        arg[0,0,0,1]=(-0.786386817128)*x[0]**o+(-0.882559775393)*x[0]+(0.117871400819)*x[1]**o+(-0.625922274774)*x[1]+(0.410431886931)*x[2]**o+(-0.979356501067)*x[2]
        arg[0,0,1,0]=(0.686025409275)*x[0]**o+(-0.19806907659)*x[0]+(-0.710420224449)*x[1]**o+(-0.839236953983)*x[1]+(-0.79971896093)*x[2]**o+(0.378944111319)*x[2]
        arg[0,0,1,1]=(0.771205112484)*x[0]**o+(-0.007083192859)*x[0]+(0.672918588001)*x[1]**o+(-0.351652495793)*x[1]+(-0.841767161187)*x[2]**o+(-0.345642373014)*x[2]
        arg[0,0,2,0]=(0.266930729015)*x[0]**o+(-0.873808152307)*x[0]+(0.206466759135)*x[1]**o+(0.539888649948)*x[1]+(0.17985862191)*x[2]**o+(0.240651460153)*x[2]
        arg[0,0,2,1]=(-0.232569768041)*x[0]**o+(-0.579841454072)*x[0]+(-0.706956104891)*x[1]**o+(0.822863815698)*x[1]+(0.345774991358)*x[2]**o+(-0.0848567668311)*x[2]
        arg[0,1,0,0]=(0.51593851507)*x[0]**o+(-0.233342434349)*x[0]+(-0.92498298026)*x[1]**o+(0.164540234829)*x[1]+(0.356559744499)*x[2]**o+(0.483902511717)*x[2]
        arg[0,1,0,1]=(-0.811509654443)*x[0]**o+(0.73929453189)*x[0]+(0.256205706418)*x[1]**o+(-0.667924913793)*x[1]+(0.266065641257)*x[2]**o+(0.856490258029)*x[2]
        arg[0,1,1,0]=(0.502051590284)*x[0]**o+(0.927283797128)*x[0]+(0.500854521411)*x[1]**o+(-0.486326996369)*x[1]+(-0.327578523685)*x[2]**o+(-0.28106070397)*x[2]
        arg[0,1,1,1]=(0.201740014513)*x[0]**o+(-0.166988241759)*x[0]+(-0.513072955566)*x[1]**o+(0.67179532417)*x[1]+(-0.72135980627)*x[2]**o+(0.763548886467)*x[2]
        arg[0,1,2,0]=(0.3995590338)*x[0]**o+(0.433286929621)*x[0]+(-0.287309453792)*x[1]**o+(-0.0671679443074)*x[1]+(-0.80815399993)*x[2]**o+(-0.398740362114)*x[2]
        arg[0,1,2,1]=(-0.16314640684)*x[0]**o+(-0.698681697367)*x[0]+(-0.11609738418)*x[1]**o+(-0.798465154739)*x[1]+(-0.169052628971)*x[2]**o+(-0.880316962539)*x[2]
        arg[0,2,0,0]=(-0.219731279442)*x[0]**o+(0.731464233112)*x[0]+(0.637811533993)*x[1]**o+(0.381218512692)*x[1]+(-0.181357790048)*x[2]**o+(-0.296793024723)*x[2]
        arg[0,2,0,1]=(0.566451624572)*x[0]**o+(0.302904935559)*x[0]+(-0.771526665232)*x[1]**o+(0.460577683302)*x[1]+(-0.921312455507)*x[2]**o+(0.604299765394)*x[2]
        arg[0,2,1,0]=(-0.0887516895197)*x[0]**o+(0.872208558285)*x[0]+(-0.306564750999)*x[1]**o+(0.841982500263)*x[1]+(-0.550398737046)*x[2]**o+(-0.869697162427)*x[2]
        arg[0,2,1,1]=(-0.652471383912)*x[0]**o+(0.634592304393)*x[0]+(0.0709212473167)*x[1]**o+(-0.671330082579)*x[1]+(-0.232895004478)*x[2]**o+(0.479008351165)*x[2]
        arg[0,2,2,0]=(-0.0764115046136)*x[0]**o+(0.649230346025)*x[0]+(0.544503480999)*x[1]**o+(0.788838981797)*x[1]+(0.416236148267)*x[2]**o+(0.722979541918)*x[2]
        arg[0,2,2,1]=(-0.0776368628837)*x[0]**o+(0.801984674386)*x[0]+(0.443670313486)*x[1]**o+(0.424676792255)*x[1]+(-0.895989937982)*x[2]**o+(0.0774648143589)*x[2]
        arg[0,3,0,0]=(-0.39041318438)*x[0]**o+(-0.174335409262)*x[0]+(0.658666482337)*x[1]**o+(0.23060605921)*x[1]+(-0.336603364419)*x[2]**o+(0.871530192198)*x[2]
        arg[0,3,0,1]=(-0.493104661363)*x[0]**o+(0.590779697499)*x[0]+(-0.903897457443)*x[1]**o+(-0.377640520704)*x[1]+(0.227174544833)*x[2]**o+(-0.860150483539)*x[2]
        arg[0,3,1,0]=(-0.093367253954)*x[0]**o+(0.159230499811)*x[0]+(-0.731396509936)*x[1]**o+(0.600088148838)*x[1]+(0.829175660147)*x[2]**o+(-0.435878280919)*x[2]
        arg[0,3,1,1]=(0.665125540711)*x[0]**o+(-0.92128737118)*x[0]+(-0.33260297543)*x[1]**o+(0.682569476816)*x[1]+(0.999761405058)*x[2]**o+(-0.748303702618)*x[2]
        arg[0,3,2,0]=(-0.430442277525)*x[0]**o+(-0.841437664378)*x[0]+(-0.374267371055)*x[1]**o+(0.667860327373)*x[1]+(0.694185699238)*x[2]**o+(-0.673629629622)*x[2]
        arg[0,3,2,1]=(0.304808617243)*x[0]**o+(-0.894407776037)*x[0]+(-0.758585281699)*x[1]**o+(0.20519963241)*x[1]+(0.538222015181)*x[2]**o+(-0.792962435573)*x[2]
        arg[0,4,0,0]=(0.568300300269)*x[0]**o+(0.407455936625)*x[0]+(0.980898344535)*x[1]**o+(0.988220474461)*x[1]+(0.0598892620089)*x[2]**o+(-0.279555207723)*x[2]
        arg[0,4,0,1]=(-0.208391854989)*x[0]**o+(0.9428019078)*x[0]+(-0.540916877275)*x[1]**o+(-0.427185538555)*x[1]+(-0.879268085857)*x[2]**o+(0.025102321099)*x[2]
        arg[0,4,1,0]=(0.412690112527)*x[0]**o+(-0.957376863734)*x[0]+(-0.913623139719)*x[1]**o+(-0.462418849979)*x[1]+(0.463336747019)*x[2]**o+(0.330747531293)*x[2]
        arg[0,4,1,1]=(-0.632140552118)*x[0]**o+(0.830769577671)*x[0]+(-0.888012251479)*x[1]**o+(-0.0560181569699)*x[1]+(-0.70068099248)*x[2]**o+(0.834483253039)*x[2]
        arg[0,4,2,0]=(0.0429717567385)*x[0]**o+(-0.691784042595)*x[0]+(-0.264988764713)*x[1]**o+(0.359873170931)*x[1]+(-0.116275128437)*x[2]**o+(0.0348873917345)*x[2]
        arg[0,4,2,1]=(-0.203754358311)*x[0]**o+(0.177896174646)*x[0]+(-0.0785920580727)*x[1]**o+(-0.124905949184)*x[1]+(0.968871355367)*x[2]**o+(-0.0369722247589)*x[2]
        arg[1,0,0,0]=(0.594851756786)*x[0]**o+(-0.594400822936)*x[0]+(0.868401350949)*x[1]**o+(0.186739616297)*x[1]+(-0.937219946754)*x[2]**o+(-0.248916422131)*x[2]
        arg[1,0,0,1]=(-0.639865978203)*x[0]**o+(-0.00990375501714)*x[0]+(-0.038266357411)*x[1]**o+(-0.741522380136)*x[1]+(0.460572922772)*x[2]**o+(0.4314205896)*x[2]
        arg[1,0,1,0]=(-0.460110413486)*x[0]**o+(-0.0964934735182)*x[0]+(0.474209024352)*x[1]**o+(-0.928476110805)*x[1]+(0.598621411487)*x[2]**o+(-0.00858812754345)*x[2]
        arg[1,0,1,1]=(0.936466172807)*x[0]**o+(-0.363937334197)*x[0]+(-0.540224651799)*x[1]**o+(0.484530722263)*x[1]+(0.675416993296)*x[2]**o+(0.89657344343)*x[2]
        arg[1,0,2,0]=(-0.713050692555)*x[0]**o+(-0.00851892179229)*x[0]+(0.689494113809)*x[1]**o+(0.633341028044)*x[1]+(0.272726911113)*x[2]**o+(-0.122017566819)*x[2]
        arg[1,0,2,1]=(0.793152080981)*x[0]**o+(0.287379739639)*x[0]+(0.304047372771)*x[1]**o+(-0.509079361053)*x[1]+(0.888302006985)*x[2]**o+(-0.297023045775)*x[2]
        arg[1,1,0,0]=(-0.69695118901)*x[0]**o+(0.505182668512)*x[0]+(0.827555590952)*x[1]**o+(-0.777232447056)*x[1]+(-0.460918138036)*x[2]**o+(0.782090206922)*x[2]
        arg[1,1,0,1]=(-0.435398535566)*x[0]**o+(-0.0997593681984)*x[0]+(0.140250252691)*x[1]**o+(-0.0204469256488)*x[1]+(-0.445836771028)*x[2]**o+(-0.775366037861)*x[2]
        arg[1,1,1,0]=(0.918015347957)*x[0]**o+(-0.491940111489)*x[0]+(0.793071617663)*x[1]**o+(-0.607035294859)*x[1]+(0.610076618566)*x[2]**o+(0.148116733513)*x[2]
        arg[1,1,1,1]=(0.224062518811)*x[0]**o+(-0.145897156735)*x[0]+(0.165380866901)*x[1]**o+(0.910562981814)*x[1]+(-0.406620561548)*x[2]**o+(0.528862812468)*x[2]
        arg[1,1,2,0]=(0.85148444093)*x[0]**o+(0.13210280653)*x[0]+(0.364136256712)*x[1]**o+(0.520388575386)*x[1]+(0.0145641287346)*x[2]**o+(-0.7849036351)*x[2]
        arg[1,1,2,1]=(0.493837193832)*x[0]**o+(-0.193326176569)*x[0]+(-0.665920972154)*x[1]**o+(-0.321645070095)*x[1]+(0.0209354853869)*x[2]**o+(-0.328315981752)*x[2]
        arg[1,2,0,0]=(-0.349281164766)*x[0]**o+(-0.318415198113)*x[0]+(-0.687972781234)*x[1]**o+(0.91825998605)*x[1]+(0.396868773798)*x[2]**o+(0.998204458027)*x[2]
        arg[1,2,0,1]=(-0.967493222935)*x[0]**o+(-0.721128465653)*x[0]+(-0.672178802974)*x[1]**o+(0.0924145224431)*x[1]+(0.114040587594)*x[2]**o+(0.707623895537)*x[2]
        arg[1,2,1,0]=(-0.253327088773)*x[0]**o+(0.206063827625)*x[0]+(0.355152654895)*x[1]**o+(-0.27339285621)*x[1]+(-0.0135567122291)*x[2]**o+(-0.983990446362)*x[2]
        arg[1,2,1,1]=(0.98471687132)*x[0]**o+(0.262902745655)*x[0]+(-0.589359138227)*x[1]**o+(-0.0982548958556)*x[1]+(-0.526161200133)*x[2]**o+(-0.850939484752)*x[2]
        arg[1,2,2,0]=(0.37583596705)*x[0]**o+(0.274845981222)*x[0]+(0.601331110943)*x[1]**o+(0.788294448218)*x[1]+(0.366445166459)*x[2]**o+(-0.554935072394)*x[2]
        arg[1,2,2,1]=(-0.986387850237)*x[0]**o+(0.68772989781)*x[0]+(0.345166604933)*x[1]**o+(0.930376726384)*x[1]+(0.760974421701)*x[2]**o+(0.261797396808)*x[2]
        arg[1,3,0,0]=(-0.171005013523)*x[0]**o+(-0.339229596812)*x[0]+(-0.150317419556)*x[1]**o+(-0.0146533427463)*x[1]+(0.957020663875)*x[2]**o+(0.616513940347)*x[2]
        arg[1,3,0,1]=(-0.0727328269499)*x[0]**o+(-0.635591748073)*x[0]+(-0.502426989774)*x[1]**o+(0.772595119619)*x[1]+(-0.76006511271)*x[2]**o+(-0.665920288084)*x[2]
        arg[1,3,1,0]=(-0.394284205622)*x[0]**o+(-0.105959165811)*x[0]+(0.72649045701)*x[1]**o+(-0.970276379473)*x[1]+(-0.236520349415)*x[2]**o+(0.994973267783)*x[2]
        arg[1,3,1,1]=(-0.444149867906)*x[0]**o+(0.748449201475)*x[0]+(0.419604170756)*x[1]**o+(-0.0291702705178)*x[1]+(-0.109580142628)*x[2]**o+(-0.0791870699838)*x[2]
        arg[1,3,2,0]=(-0.629657820523)*x[0]**o+(-0.995576552624)*x[0]+(0.530135462206)*x[1]**o+(0.197336335845)*x[1]+(0.0224815558807)*x[2]**o+(0.240498463912)*x[2]
        arg[1,3,2,1]=(0.52933948346)*x[0]**o+(0.98353913011)*x[0]+(0.0217614943138)*x[1]**o+(0.0157778453682)*x[1]+(0.177956932986)*x[2]**o+(0.234629778706)*x[2]
        arg[1,4,0,0]=(0.593028367137)*x[0]**o+(0.0340892538994)*x[0]+(-0.0295133170051)*x[1]**o+(0.0865921041963)*x[1]+(0.641131718656)*x[2]**o+(0.788858018609)*x[2]
        arg[1,4,0,1]=(-0.283333820634)*x[0]**o+(0.864482725099)*x[0]+(-0.560218794286)*x[1]**o+(-0.0083115362546)*x[1]+(0.726423285129)*x[2]**o+(0.031531339166)*x[2]
        arg[1,4,1,0]=(0.801452411554)*x[0]**o+(-0.59916393038)*x[0]+(-0.106568863591)*x[1]**o+(0.357442125133)*x[1]+(-0.203380280132)*x[2]**o+(-0.129031606441)*x[2]
        arg[1,4,1,1]=(-0.589220928637)*x[0]**o+(-0.308949389428)*x[0]+(0.785549858338)*x[1]**o+(-0.131153853015)*x[1]+(-0.30571855555)*x[2]**o+(-0.329634802588)*x[2]
        arg[1,4,2,0]=(-0.479686944)*x[0]**o+(-0.707702026873)*x[0]+(0.635894172352)*x[1]**o+(0.370673635069)*x[1]+(0.942862875991)*x[2]**o+(-0.846943456738)*x[2]
        arg[1,4,2,1]=(-0.403737324968)*x[0]**o+(0.101664426734)*x[0]+(-0.677093482354)*x[1]**o+(-0.36526066745)*x[1]+(-0.0889619136182)*x[2]**o+(-0.772491940844)*x[2]
        arg[2,0,0,0]=(0.753148325987)*x[0]**o+(-0.342843083128)*x[0]+(0.812661029553)*x[1]**o+(-0.112683035459)*x[1]+(0.900976470275)*x[2]**o+(0.00934086955102)*x[2]
        arg[2,0,0,1]=(-0.682315007143)*x[0]**o+(-0.694069900861)*x[0]+(-0.469082247747)*x[1]**o+(-0.522165903301)*x[1]+(-0.0997886221924)*x[2]**o+(-0.954161552419)*x[2]
        arg[2,0,1,0]=(0.780125539705)*x[0]**o+(-0.445331450225)*x[0]+(-0.282765510424)*x[1]**o+(-0.0131898361592)*x[1]+(-0.200186433923)*x[2]**o+(0.715139435696)*x[2]
        arg[2,0,1,1]=(-0.132310660786)*x[0]**o+(-0.0981344234369)*x[0]+(0.959297529475)*x[1]**o+(-0.85944894515)*x[1]+(-0.396258659619)*x[2]**o+(-0.09672066185)*x[2]
        arg[2,0,2,0]=(-0.344134014191)*x[0]**o+(0.534345864287)*x[0]+(0.797209509484)*x[1]**o+(0.0479050101499)*x[1]+(0.13620476987)*x[2]**o+(0.82708115383)*x[2]
        arg[2,0,2,1]=(-0.342509328238)*x[0]**o+(-0.833164493006)*x[0]+(0.472047721532)*x[1]**o+(0.301613158637)*x[1]+(0.858691679638)*x[2]**o+(-0.691744164006)*x[2]
        arg[2,1,0,0]=(0.498979390299)*x[0]**o+(0.371513881824)*x[0]+(-0.243506167971)*x[1]**o+(-0.245423890439)*x[1]+(-0.0494707840274)*x[2]**o+(0.0965573709928)*x[2]
        arg[2,1,0,1]=(0.665982455862)*x[0]**o+(-0.409710866336)*x[0]+(0.669257568046)*x[1]**o+(0.711947704788)*x[1]+(-0.0362568635785)*x[2]**o+(-0.30394278188)*x[2]
        arg[2,1,1,0]=(0.296245249928)*x[0]**o+(0.132631019281)*x[0]+(-0.376880152158)*x[1]**o+(0.303184156241)*x[1]+(0.836046560882)*x[2]**o+(0.195205645106)*x[2]
        arg[2,1,1,1]=(-0.971045820953)*x[0]**o+(0.154708896467)*x[0]+(-0.663844656602)*x[1]**o+(-0.0873797588245)*x[1]+(0.985123372542)*x[2]**o+(0.349554922131)*x[2]
        arg[2,1,2,0]=(-0.0117443431348)*x[0]**o+(-0.71980503985)*x[0]+(0.175348078994)*x[1]**o+(-0.883987892161)*x[1]+(0.0587275450458)*x[2]**o+(0.464811715548)*x[2]
        arg[2,1,2,1]=(-0.309202575765)*x[0]**o+(-0.528513248453)*x[0]+(-0.509298303972)*x[1]**o+(-0.228278863496)*x[1]+(0.933917053676)*x[2]**o+(-0.823678942264)*x[2]
        arg[2,2,0,0]=(0.374387386929)*x[0]**o+(0.64969993847)*x[0]+(-0.939822635641)*x[1]**o+(0.578249430422)*x[1]+(-0.973412660635)*x[2]**o+(0.484052718386)*x[2]
        arg[2,2,0,1]=(-0.972635162618)*x[0]**o+(-0.255007433802)*x[0]+(0.134165187097)*x[1]**o+(0.313213615976)*x[1]+(0.143298148686)*x[2]**o+(0.901819440229)*x[2]
        arg[2,2,1,0]=(0.868173256315)*x[0]**o+(-0.532325343453)*x[0]+(-0.712500880845)*x[1]**o+(0.216943462308)*x[1]+(0.733920091212)*x[2]**o+(0.0992400603383)*x[2]
        arg[2,2,1,1]=(0.642839350527)*x[0]**o+(0.127721543559)*x[0]+(-0.152686523032)*x[1]**o+(-0.399032361234)*x[1]+(0.317014518842)*x[2]**o+(0.905538790115)*x[2]
        arg[2,2,2,0]=(0.685267932413)*x[0]**o+(-0.167571547782)*x[0]+(-0.213446144405)*x[1]**o+(0.420270456377)*x[1]+(0.827725350621)*x[2]**o+(-0.920242274399)*x[2]
        arg[2,2,2,1]=(-0.507424593437)*x[0]**o+(-0.694222840019)*x[0]+(-0.904108970188)*x[1]**o+(-0.750899614966)*x[1]+(0.783997553302)*x[2]**o+(0.196327513447)*x[2]
        arg[2,3,0,0]=(-0.473900600479)*x[0]**o+(-0.464347097789)*x[0]+(-0.472423358671)*x[1]**o+(0.373786456364)*x[1]+(-0.989371300414)*x[2]**o+(-0.959508065239)*x[2]
        arg[2,3,0,1]=(0.174375189399)*x[0]**o+(0.925503059886)*x[0]+(-0.647274384723)*x[1]**o+(-0.577149479679)*x[1]+(0.171298877768)*x[2]**o+(-0.936612100882)*x[2]
        arg[2,3,1,0]=(0.482959343396)*x[0]**o+(0.842243554011)*x[0]+(0.814423633314)*x[1]**o+(-0.989445050315)*x[1]+(0.817933433065)*x[2]**o+(0.552396881404)*x[2]
        arg[2,3,1,1]=(0.55641736229)*x[0]**o+(0.498094640058)*x[0]+(0.0890113606037)*x[1]**o+(0.36592723553)*x[1]+(0.458607593386)*x[2]**o+(0.603047420063)*x[2]
        arg[2,3,2,0]=(0.884515080124)*x[0]**o+(-0.561898381022)*x[0]+(-0.292475503834)*x[1]**o+(0.425634423478)*x[1]+(0.65813323252)*x[2]**o+(-0.548850399842)*x[2]
        arg[2,3,2,1]=(-0.72895566982)*x[0]**o+(-0.569405588005)*x[0]+(0.117770875594)*x[1]**o+(-0.33515735969)*x[1]+(0.102062736382)*x[2]**o+(0.611532417683)*x[2]
        arg[2,4,0,0]=(-0.669208630424)*x[0]**o+(0.944393014177)*x[0]+(-0.00345511889683)*x[1]**o+(0.0695279595783)*x[1]+(0.496948250833)*x[2]**o+(-0.899750463971)*x[2]
        arg[2,4,0,1]=(-0.387828717292)*x[0]**o+(0.621172267004)*x[0]+(-0.280632674111)*x[1]**o+(0.401702567575)*x[1]+(-0.65554489378)*x[2]**o+(0.403018047007)*x[2]
        arg[2,4,1,0]=(-0.0770246925803)*x[0]**o+(0.52913302745)*x[0]+(0.922468724357)*x[1]**o+(-0.304442750439)*x[1]+(-0.556575079059)*x[2]**o+(-0.725790672251)*x[2]
        arg[2,4,1,1]=(-0.828378342911)*x[0]**o+(-0.0696370821776)*x[0]+(0.394018581667)*x[1]**o+(-0.251099848269)*x[1]+(-0.29878985656)*x[2]**o+(0.925753371056)*x[2]
        arg[2,4,2,0]=(-0.818277786029)*x[0]**o+(0.225862354706)*x[0]+(-0.629050212595)*x[1]**o+(0.139186107062)*x[1]+(-0.0134835519238)*x[2]**o+(0.301899528641)*x[2]
        arg[2,4,2,1]=(-0.783053684767)*x[0]**o+(-0.0926938783599)*x[0]+(0.519034210352)*x[1]**o+(-0.494564657126)*x[1]+(0.536117310622)*x[2]**o+(0.6278756634)*x[2]
        arg[3,0,0,0]=(-0.694353957685)*x[0]**o+(-0.361037363965)*x[0]+(-0.327004091885)*x[1]**o+(-0.765735142884)*x[1]+(-0.810915729713)*x[2]**o+(0.0640539311609)*x[2]
        arg[3,0,0,1]=(-0.131601548312)*x[0]**o+(0.271587756708)*x[0]+(0.322486168005)*x[1]**o+(0.238086581801)*x[1]+(-0.488081720555)*x[2]**o+(0.607278995635)*x[2]
        arg[3,0,1,0]=(0.903075418102)*x[0]**o+(-0.738609834848)*x[0]+(-0.165097989126)*x[1]**o+(-0.25967140914)*x[1]+(0.271649887428)*x[2]**o+(-0.786131648727)*x[2]
        arg[3,0,1,1]=(-0.963442989157)*x[0]**o+(-0.240205602334)*x[0]+(0.102153389731)*x[1]**o+(-0.106402434995)*x[1]+(-0.383097724211)*x[2]**o+(0.680290963195)*x[2]
        arg[3,0,2,0]=(-0.373162835796)*x[0]**o+(-0.186323761814)*x[0]+(0.661494315705)*x[1]**o+(0.320072189494)*x[1]+(0.0964443431067)*x[2]**o+(0.829015838241)*x[2]
        arg[3,0,2,1]=(0.527719514862)*x[0]**o+(0.0819948261665)*x[0]+(0.826598712178)*x[1]**o+(-0.588102975125)*x[1]+(-0.632584402292)*x[2]**o+(0.0826992233702)*x[2]
        arg[3,1,0,0]=(-0.102243686287)*x[0]**o+(-0.497336112929)*x[0]+(0.941827592628)*x[1]**o+(0.68678991944)*x[1]+(-0.508684841667)*x[2]**o+(-0.752939965161)*x[2]
        arg[3,1,0,1]=(-0.896417523183)*x[0]**o+(0.759224781685)*x[0]+(0.138537695823)*x[1]**o+(0.466727480471)*x[1]+(-0.838380824412)*x[2]**o+(0.285761157398)*x[2]
        arg[3,1,1,0]=(-0.547853182919)*x[0]**o+(-0.41595413414)*x[0]+(0.76894221584)*x[1]**o+(-0.142394275192)*x[1]+(-0.96349294581)*x[2]**o+(-0.873639964693)*x[2]
        arg[3,1,1,1]=(-0.151299403888)*x[0]**o+(0.789406063878)*x[0]+(0.163280670253)*x[1]**o+(-0.287315494745)*x[1]+(-0.627764670218)*x[2]**o+(0.842131632598)*x[2]
        arg[3,1,2,0]=(-0.957096489189)*x[0]**o+(0.0616037805677)*x[0]+(0.255858200166)*x[1]**o+(-0.430555450798)*x[1]+(0.730436930737)*x[2]**o+(0.405094271653)*x[2]
        arg[3,1,2,1]=(-0.924756832665)*x[0]**o+(0.525754162391)*x[0]+(0.281303228038)*x[1]**o+(-0.479500161878)*x[1]+(0.658416209181)*x[2]**o+(-0.0413881595557)*x[2]
        arg[3,2,0,0]=(0.516429381997)*x[0]**o+(-0.8387318467)*x[0]+(0.329773978227)*x[1]**o+(0.599641753085)*x[1]+(0.939937743271)*x[2]**o+(-0.247331169626)*x[2]
        arg[3,2,0,1]=(-0.644982044558)*x[0]**o+(-0.242033352774)*x[0]+(0.531835185779)*x[1]**o+(-0.851565064183)*x[1]+(-0.79213624915)*x[2]**o+(0.0607365690732)*x[2]
        arg[3,2,1,0]=(0.359781383537)*x[0]**o+(-0.0529212842074)*x[0]+(0.845915169452)*x[1]**o+(-0.323979570458)*x[1]+(0.828909997631)*x[2]**o+(-0.839886339138)*x[2]
        arg[3,2,1,1]=(0.559476630421)*x[0]**o+(-0.189725214615)*x[0]+(-0.00927446700413)*x[1]**o+(-0.234703720782)*x[1]+(-0.608362806755)*x[2]**o+(0.968571765534)*x[2]
        arg[3,2,2,0]=(0.924837507913)*x[0]**o+(0.846397611121)*x[0]+(-0.837765472403)*x[1]**o+(0.0225105985057)*x[1]+(-0.834187513844)*x[2]**o+(0.194311845578)*x[2]
        arg[3,2,2,1]=(-0.183561089961)*x[0]**o+(0.0745071729471)*x[0]+(-0.660415615202)*x[1]**o+(-0.528184997345)*x[1]+(0.205554094557)*x[2]**o+(-0.0309246318944)*x[2]
        arg[3,3,0,0]=(-0.67124095974)*x[0]**o+(-0.48501497083)*x[0]+(-0.0394345869094)*x[1]**o+(0.586692270429)*x[1]+(-0.593760749687)*x[2]**o+(-0.124979751327)*x[2]
        arg[3,3,0,1]=(0.778544173855)*x[0]**o+(-0.931466478494)*x[0]+(-0.746894777998)*x[1]**o+(0.806417936297)*x[1]+(0.989460929848)*x[2]**o+(0.666534366369)*x[2]
        arg[3,3,1,0]=(-0.798870838602)*x[0]**o+(-0.294618868937)*x[0]+(0.275029092968)*x[1]**o+(0.45390580364)*x[1]+(-0.311797818508)*x[2]**o+(0.751606624393)*x[2]
        arg[3,3,1,1]=(-0.924260987477)*x[0]**o+(0.771970726902)*x[0]+(-0.909099224213)*x[1]**o+(-0.45204206766)*x[1]+(0.630042614017)*x[2]**o+(-0.990915130464)*x[2]
        arg[3,3,2,0]=(0.734962154852)*x[0]**o+(-0.00621898011475)*x[0]+(-0.786362527189)*x[1]**o+(0.623507947406)*x[1]+(0.867175567104)*x[2]**o+(-0.383283255448)*x[2]
        arg[3,3,2,1]=(-0.955613156717)*x[0]**o+(0.688013754841)*x[0]+(-0.600559375208)*x[1]**o+(0.453342626645)*x[1]+(-0.864890477961)*x[2]**o+(-0.458029873063)*x[2]
        arg[3,4,0,0]=(-0.897010884834)*x[0]**o+(-0.251223576458)*x[0]+(-0.0338052047964)*x[1]**o+(-0.903421466132)*x[1]+(0.836864217874)*x[2]**o+(0.660805905175)*x[2]
        arg[3,4,0,1]=(-0.286025690215)*x[0]**o+(0.746732975604)*x[0]+(0.375810853218)*x[1]**o+(-0.985192974869)*x[1]+(-0.731463214455)*x[2]**o+(-0.465450352444)*x[2]
        arg[3,4,1,0]=(-0.500910000652)*x[0]**o+(0.387502419727)*x[0]+(0.962369164802)*x[1]**o+(-0.10933824251)*x[1]+(-0.33642703989)*x[2]**o+(0.546851133256)*x[2]
        arg[3,4,1,1]=(-0.117959459645)*x[0]**o+(0.867699323392)*x[0]+(0.950506403861)*x[1]**o+(0.613806207745)*x[1]+(0.425723173807)*x[2]**o+(-0.906937548619)*x[2]
        arg[3,4,2,0]=(0.331114892194)*x[0]**o+(-0.168829643424)*x[0]+(-0.855962311982)*x[1]**o+(-0.144202406673)*x[1]+(0.0769985713088)*x[2]**o+(0.856003746423)*x[2]
        arg[3,4,2,1]=(-0.439397439364)*x[0]**o+(0.686116343481)*x[0]+(0.519883854523)*x[1]**o+(0.716231469289)*x[1]+(0.270098231149)*x[2]**o+(-0.448525436427)*x[2]
        ref[0,0,0,0]=(-0.401901998082)*x_ref[0]**o+(-0.805091116808)*x_ref[0]+(0.997659882792)*x_ref[1]**o+(-0.953957389492)*x_ref[1]+(-0.450185408289)*x_ref[2]**o+(0.636472207773)*x_ref[2]
        ref[0,0,0,1]=(-0.786386817128)*x_ref[0]**o+(-0.882559775393)*x_ref[0]+(0.117871400819)*x_ref[1]**o+(-0.625922274774)*x_ref[1]+(0.410431886931)*x_ref[2]**o+(-0.979356501067)*x_ref[2]
        ref[0,0,1,0]=(0.686025409275)*x_ref[0]**o+(-0.19806907659)*x_ref[0]+(-0.710420224449)*x_ref[1]**o+(-0.839236953983)*x_ref[1]+(-0.79971896093)*x_ref[2]**o+(0.378944111319)*x_ref[2]
        ref[0,0,1,1]=(0.771205112484)*x_ref[0]**o+(-0.007083192859)*x_ref[0]+(0.672918588001)*x_ref[1]**o+(-0.351652495793)*x_ref[1]+(-0.841767161187)*x_ref[2]**o+(-0.345642373014)*x_ref[2]
        ref[0,0,2,0]=(0.266930729015)*x_ref[0]**o+(-0.873808152307)*x_ref[0]+(0.206466759135)*x_ref[1]**o+(0.539888649948)*x_ref[1]+(0.17985862191)*x_ref[2]**o+(0.240651460153)*x_ref[2]
        ref[0,0,2,1]=(-0.232569768041)*x_ref[0]**o+(-0.579841454072)*x_ref[0]+(-0.706956104891)*x_ref[1]**o+(0.822863815698)*x_ref[1]+(0.345774991358)*x_ref[2]**o+(-0.0848567668311)*x_ref[2]
        ref[0,1,0,0]=(0.51593851507)*x_ref[0]**o+(-0.233342434349)*x_ref[0]+(-0.92498298026)*x_ref[1]**o+(0.164540234829)*x_ref[1]+(0.356559744499)*x_ref[2]**o+(0.483902511717)*x_ref[2]
        ref[0,1,0,1]=(-0.811509654443)*x_ref[0]**o+(0.73929453189)*x_ref[0]+(0.256205706418)*x_ref[1]**o+(-0.667924913793)*x_ref[1]+(0.266065641257)*x_ref[2]**o+(0.856490258029)*x_ref[2]
        ref[0,1,1,0]=(0.502051590284)*x_ref[0]**o+(0.927283797128)*x_ref[0]+(0.500854521411)*x_ref[1]**o+(-0.486326996369)*x_ref[1]+(-0.327578523685)*x_ref[2]**o+(-0.28106070397)*x_ref[2]
        ref[0,1,1,1]=(0.201740014513)*x_ref[0]**o+(-0.166988241759)*x_ref[0]+(-0.513072955566)*x_ref[1]**o+(0.67179532417)*x_ref[1]+(-0.72135980627)*x_ref[2]**o+(0.763548886467)*x_ref[2]
        ref[0,1,2,0]=(0.3995590338)*x_ref[0]**o+(0.433286929621)*x_ref[0]+(-0.287309453792)*x_ref[1]**o+(-0.0671679443074)*x_ref[1]+(-0.80815399993)*x_ref[2]**o+(-0.398740362114)*x_ref[2]
        ref[0,1,2,1]=(-0.16314640684)*x_ref[0]**o+(-0.698681697367)*x_ref[0]+(-0.11609738418)*x_ref[1]**o+(-0.798465154739)*x_ref[1]+(-0.169052628971)*x_ref[2]**o+(-0.880316962539)*x_ref[2]
        ref[0,2,0,0]=(-0.219731279442)*x_ref[0]**o+(0.731464233112)*x_ref[0]+(0.637811533993)*x_ref[1]**o+(0.381218512692)*x_ref[1]+(-0.181357790048)*x_ref[2]**o+(-0.296793024723)*x_ref[2]
        ref[0,2,0,1]=(0.566451624572)*x_ref[0]**o+(0.302904935559)*x_ref[0]+(-0.771526665232)*x_ref[1]**o+(0.460577683302)*x_ref[1]+(-0.921312455507)*x_ref[2]**o+(0.604299765394)*x_ref[2]
        ref[0,2,1,0]=(-0.0887516895197)*x_ref[0]**o+(0.872208558285)*x_ref[0]+(-0.306564750999)*x_ref[1]**o+(0.841982500263)*x_ref[1]+(-0.550398737046)*x_ref[2]**o+(-0.869697162427)*x_ref[2]
        ref[0,2,1,1]=(-0.652471383912)*x_ref[0]**o+(0.634592304393)*x_ref[0]+(0.0709212473167)*x_ref[1]**o+(-0.671330082579)*x_ref[1]+(-0.232895004478)*x_ref[2]**o+(0.479008351165)*x_ref[2]
        ref[0,2,2,0]=(-0.0764115046136)*x_ref[0]**o+(0.649230346025)*x_ref[0]+(0.544503480999)*x_ref[1]**o+(0.788838981797)*x_ref[1]+(0.416236148267)*x_ref[2]**o+(0.722979541918)*x_ref[2]
        ref[0,2,2,1]=(-0.0776368628837)*x_ref[0]**o+(0.801984674386)*x_ref[0]+(0.443670313486)*x_ref[1]**o+(0.424676792255)*x_ref[1]+(-0.895989937982)*x_ref[2]**o+(0.0774648143589)*x_ref[2]
        ref[0,3,0,0]=(-0.39041318438)*x_ref[0]**o+(-0.174335409262)*x_ref[0]+(0.658666482337)*x_ref[1]**o+(0.23060605921)*x_ref[1]+(-0.336603364419)*x_ref[2]**o+(0.871530192198)*x_ref[2]
        ref[0,3,0,1]=(-0.493104661363)*x_ref[0]**o+(0.590779697499)*x_ref[0]+(-0.903897457443)*x_ref[1]**o+(-0.377640520704)*x_ref[1]+(0.227174544833)*x_ref[2]**o+(-0.860150483539)*x_ref[2]
        ref[0,3,1,0]=(-0.093367253954)*x_ref[0]**o+(0.159230499811)*x_ref[0]+(-0.731396509936)*x_ref[1]**o+(0.600088148838)*x_ref[1]+(0.829175660147)*x_ref[2]**o+(-0.435878280919)*x_ref[2]
        ref[0,3,1,1]=(0.665125540711)*x_ref[0]**o+(-0.92128737118)*x_ref[0]+(-0.33260297543)*x_ref[1]**o+(0.682569476816)*x_ref[1]+(0.999761405058)*x_ref[2]**o+(-0.748303702618)*x_ref[2]
        ref[0,3,2,0]=(-0.430442277525)*x_ref[0]**o+(-0.841437664378)*x_ref[0]+(-0.374267371055)*x_ref[1]**o+(0.667860327373)*x_ref[1]+(0.694185699238)*x_ref[2]**o+(-0.673629629622)*x_ref[2]
        ref[0,3,2,1]=(0.304808617243)*x_ref[0]**o+(-0.894407776037)*x_ref[0]+(-0.758585281699)*x_ref[1]**o+(0.20519963241)*x_ref[1]+(0.538222015181)*x_ref[2]**o+(-0.792962435573)*x_ref[2]
        ref[0,4,0,0]=(0.568300300269)*x_ref[0]**o+(0.407455936625)*x_ref[0]+(0.980898344535)*x_ref[1]**o+(0.988220474461)*x_ref[1]+(0.0598892620089)*x_ref[2]**o+(-0.279555207723)*x_ref[2]
        ref[0,4,0,1]=(-0.208391854989)*x_ref[0]**o+(0.9428019078)*x_ref[0]+(-0.540916877275)*x_ref[1]**o+(-0.427185538555)*x_ref[1]+(-0.879268085857)*x_ref[2]**o+(0.025102321099)*x_ref[2]
        ref[0,4,1,0]=(0.412690112527)*x_ref[0]**o+(-0.957376863734)*x_ref[0]+(-0.913623139719)*x_ref[1]**o+(-0.462418849979)*x_ref[1]+(0.463336747019)*x_ref[2]**o+(0.330747531293)*x_ref[2]
        ref[0,4,1,1]=(-0.632140552118)*x_ref[0]**o+(0.830769577671)*x_ref[0]+(-0.888012251479)*x_ref[1]**o+(-0.0560181569699)*x_ref[1]+(-0.70068099248)*x_ref[2]**o+(0.834483253039)*x_ref[2]
        ref[0,4,2,0]=(0.0429717567385)*x_ref[0]**o+(-0.691784042595)*x_ref[0]+(-0.264988764713)*x_ref[1]**o+(0.359873170931)*x_ref[1]+(-0.116275128437)*x_ref[2]**o+(0.0348873917345)*x_ref[2]
        ref[0,4,2,1]=(-0.203754358311)*x_ref[0]**o+(0.177896174646)*x_ref[0]+(-0.0785920580727)*x_ref[1]**o+(-0.124905949184)*x_ref[1]+(0.968871355367)*x_ref[2]**o+(-0.0369722247589)*x_ref[2]
        ref[1,0,0,0]=(0.594851756786)*x_ref[0]**o+(-0.594400822936)*x_ref[0]+(0.868401350949)*x_ref[1]**o+(0.186739616297)*x_ref[1]+(-0.937219946754)*x_ref[2]**o+(-0.248916422131)*x_ref[2]
        ref[1,0,0,1]=(-0.639865978203)*x_ref[0]**o+(-0.00990375501714)*x_ref[0]+(-0.038266357411)*x_ref[1]**o+(-0.741522380136)*x_ref[1]+(0.460572922772)*x_ref[2]**o+(0.4314205896)*x_ref[2]
        ref[1,0,1,0]=(-0.460110413486)*x_ref[0]**o+(-0.0964934735182)*x_ref[0]+(0.474209024352)*x_ref[1]**o+(-0.928476110805)*x_ref[1]+(0.598621411487)*x_ref[2]**o+(-0.00858812754345)*x_ref[2]
        ref[1,0,1,1]=(0.936466172807)*x_ref[0]**o+(-0.363937334197)*x_ref[0]+(-0.540224651799)*x_ref[1]**o+(0.484530722263)*x_ref[1]+(0.675416993296)*x_ref[2]**o+(0.89657344343)*x_ref[2]
        ref[1,0,2,0]=(-0.713050692555)*x_ref[0]**o+(-0.00851892179229)*x_ref[0]+(0.689494113809)*x_ref[1]**o+(0.633341028044)*x_ref[1]+(0.272726911113)*x_ref[2]**o+(-0.122017566819)*x_ref[2]
        ref[1,0,2,1]=(0.793152080981)*x_ref[0]**o+(0.287379739639)*x_ref[0]+(0.304047372771)*x_ref[1]**o+(-0.509079361053)*x_ref[1]+(0.888302006985)*x_ref[2]**o+(-0.297023045775)*x_ref[2]
        ref[1,1,0,0]=(-0.69695118901)*x_ref[0]**o+(0.505182668512)*x_ref[0]+(0.827555590952)*x_ref[1]**o+(-0.777232447056)*x_ref[1]+(-0.460918138036)*x_ref[2]**o+(0.782090206922)*x_ref[2]
        ref[1,1,0,1]=(-0.435398535566)*x_ref[0]**o+(-0.0997593681984)*x_ref[0]+(0.140250252691)*x_ref[1]**o+(-0.0204469256488)*x_ref[1]+(-0.445836771028)*x_ref[2]**o+(-0.775366037861)*x_ref[2]
        ref[1,1,1,0]=(0.918015347957)*x_ref[0]**o+(-0.491940111489)*x_ref[0]+(0.793071617663)*x_ref[1]**o+(-0.607035294859)*x_ref[1]+(0.610076618566)*x_ref[2]**o+(0.148116733513)*x_ref[2]
        ref[1,1,1,1]=(0.224062518811)*x_ref[0]**o+(-0.145897156735)*x_ref[0]+(0.165380866901)*x_ref[1]**o+(0.910562981814)*x_ref[1]+(-0.406620561548)*x_ref[2]**o+(0.528862812468)*x_ref[2]
        ref[1,1,2,0]=(0.85148444093)*x_ref[0]**o+(0.13210280653)*x_ref[0]+(0.364136256712)*x_ref[1]**o+(0.520388575386)*x_ref[1]+(0.0145641287346)*x_ref[2]**o+(-0.7849036351)*x_ref[2]
        ref[1,1,2,1]=(0.493837193832)*x_ref[0]**o+(-0.193326176569)*x_ref[0]+(-0.665920972154)*x_ref[1]**o+(-0.321645070095)*x_ref[1]+(0.0209354853869)*x_ref[2]**o+(-0.328315981752)*x_ref[2]
        ref[1,2,0,0]=(-0.349281164766)*x_ref[0]**o+(-0.318415198113)*x_ref[0]+(-0.687972781234)*x_ref[1]**o+(0.91825998605)*x_ref[1]+(0.396868773798)*x_ref[2]**o+(0.998204458027)*x_ref[2]
        ref[1,2,0,1]=(-0.967493222935)*x_ref[0]**o+(-0.721128465653)*x_ref[0]+(-0.672178802974)*x_ref[1]**o+(0.0924145224431)*x_ref[1]+(0.114040587594)*x_ref[2]**o+(0.707623895537)*x_ref[2]
        ref[1,2,1,0]=(-0.253327088773)*x_ref[0]**o+(0.206063827625)*x_ref[0]+(0.355152654895)*x_ref[1]**o+(-0.27339285621)*x_ref[1]+(-0.0135567122291)*x_ref[2]**o+(-0.983990446362)*x_ref[2]
        ref[1,2,1,1]=(0.98471687132)*x_ref[0]**o+(0.262902745655)*x_ref[0]+(-0.589359138227)*x_ref[1]**o+(-0.0982548958556)*x_ref[1]+(-0.526161200133)*x_ref[2]**o+(-0.850939484752)*x_ref[2]
        ref[1,2,2,0]=(0.37583596705)*x_ref[0]**o+(0.274845981222)*x_ref[0]+(0.601331110943)*x_ref[1]**o+(0.788294448218)*x_ref[1]+(0.366445166459)*x_ref[2]**o+(-0.554935072394)*x_ref[2]
        ref[1,2,2,1]=(-0.986387850237)*x_ref[0]**o+(0.68772989781)*x_ref[0]+(0.345166604933)*x_ref[1]**o+(0.930376726384)*x_ref[1]+(0.760974421701)*x_ref[2]**o+(0.261797396808)*x_ref[2]
        ref[1,3,0,0]=(-0.171005013523)*x_ref[0]**o+(-0.339229596812)*x_ref[0]+(-0.150317419556)*x_ref[1]**o+(-0.0146533427463)*x_ref[1]+(0.957020663875)*x_ref[2]**o+(0.616513940347)*x_ref[2]
        ref[1,3,0,1]=(-0.0727328269499)*x_ref[0]**o+(-0.635591748073)*x_ref[0]+(-0.502426989774)*x_ref[1]**o+(0.772595119619)*x_ref[1]+(-0.76006511271)*x_ref[2]**o+(-0.665920288084)*x_ref[2]
        ref[1,3,1,0]=(-0.394284205622)*x_ref[0]**o+(-0.105959165811)*x_ref[0]+(0.72649045701)*x_ref[1]**o+(-0.970276379473)*x_ref[1]+(-0.236520349415)*x_ref[2]**o+(0.994973267783)*x_ref[2]
        ref[1,3,1,1]=(-0.444149867906)*x_ref[0]**o+(0.748449201475)*x_ref[0]+(0.419604170756)*x_ref[1]**o+(-0.0291702705178)*x_ref[1]+(-0.109580142628)*x_ref[2]**o+(-0.0791870699838)*x_ref[2]
        ref[1,3,2,0]=(-0.629657820523)*x_ref[0]**o+(-0.995576552624)*x_ref[0]+(0.530135462206)*x_ref[1]**o+(0.197336335845)*x_ref[1]+(0.0224815558807)*x_ref[2]**o+(0.240498463912)*x_ref[2]
        ref[1,3,2,1]=(0.52933948346)*x_ref[0]**o+(0.98353913011)*x_ref[0]+(0.0217614943138)*x_ref[1]**o+(0.0157778453682)*x_ref[1]+(0.177956932986)*x_ref[2]**o+(0.234629778706)*x_ref[2]
        ref[1,4,0,0]=(0.593028367137)*x_ref[0]**o+(0.0340892538994)*x_ref[0]+(-0.0295133170051)*x_ref[1]**o+(0.0865921041963)*x_ref[1]+(0.641131718656)*x_ref[2]**o+(0.788858018609)*x_ref[2]
        ref[1,4,0,1]=(-0.283333820634)*x_ref[0]**o+(0.864482725099)*x_ref[0]+(-0.560218794286)*x_ref[1]**o+(-0.0083115362546)*x_ref[1]+(0.726423285129)*x_ref[2]**o+(0.031531339166)*x_ref[2]
        ref[1,4,1,0]=(0.801452411554)*x_ref[0]**o+(-0.59916393038)*x_ref[0]+(-0.106568863591)*x_ref[1]**o+(0.357442125133)*x_ref[1]+(-0.203380280132)*x_ref[2]**o+(-0.129031606441)*x_ref[2]
        ref[1,4,1,1]=(-0.589220928637)*x_ref[0]**o+(-0.308949389428)*x_ref[0]+(0.785549858338)*x_ref[1]**o+(-0.131153853015)*x_ref[1]+(-0.30571855555)*x_ref[2]**o+(-0.329634802588)*x_ref[2]
        ref[1,4,2,0]=(-0.479686944)*x_ref[0]**o+(-0.707702026873)*x_ref[0]+(0.635894172352)*x_ref[1]**o+(0.370673635069)*x_ref[1]+(0.942862875991)*x_ref[2]**o+(-0.846943456738)*x_ref[2]
        ref[1,4,2,1]=(-0.403737324968)*x_ref[0]**o+(0.101664426734)*x_ref[0]+(-0.677093482354)*x_ref[1]**o+(-0.36526066745)*x_ref[1]+(-0.0889619136182)*x_ref[2]**o+(-0.772491940844)*x_ref[2]
        ref[2,0,0,0]=(0.753148325987)*x_ref[0]**o+(-0.342843083128)*x_ref[0]+(0.812661029553)*x_ref[1]**o+(-0.112683035459)*x_ref[1]+(0.900976470275)*x_ref[2]**o+(0.00934086955102)*x_ref[2]
        ref[2,0,0,1]=(-0.682315007143)*x_ref[0]**o+(-0.694069900861)*x_ref[0]+(-0.469082247747)*x_ref[1]**o+(-0.522165903301)*x_ref[1]+(-0.0997886221924)*x_ref[2]**o+(-0.954161552419)*x_ref[2]
        ref[2,0,1,0]=(0.780125539705)*x_ref[0]**o+(-0.445331450225)*x_ref[0]+(-0.282765510424)*x_ref[1]**o+(-0.0131898361592)*x_ref[1]+(-0.200186433923)*x_ref[2]**o+(0.715139435696)*x_ref[2]
        ref[2,0,1,1]=(-0.132310660786)*x_ref[0]**o+(-0.0981344234369)*x_ref[0]+(0.959297529475)*x_ref[1]**o+(-0.85944894515)*x_ref[1]+(-0.396258659619)*x_ref[2]**o+(-0.09672066185)*x_ref[2]
        ref[2,0,2,0]=(-0.344134014191)*x_ref[0]**o+(0.534345864287)*x_ref[0]+(0.797209509484)*x_ref[1]**o+(0.0479050101499)*x_ref[1]+(0.13620476987)*x_ref[2]**o+(0.82708115383)*x_ref[2]
        ref[2,0,2,1]=(-0.342509328238)*x_ref[0]**o+(-0.833164493006)*x_ref[0]+(0.472047721532)*x_ref[1]**o+(0.301613158637)*x_ref[1]+(0.858691679638)*x_ref[2]**o+(-0.691744164006)*x_ref[2]
        ref[2,1,0,0]=(0.498979390299)*x_ref[0]**o+(0.371513881824)*x_ref[0]+(-0.243506167971)*x_ref[1]**o+(-0.245423890439)*x_ref[1]+(-0.0494707840274)*x_ref[2]**o+(0.0965573709928)*x_ref[2]
        ref[2,1,0,1]=(0.665982455862)*x_ref[0]**o+(-0.409710866336)*x_ref[0]+(0.669257568046)*x_ref[1]**o+(0.711947704788)*x_ref[1]+(-0.0362568635785)*x_ref[2]**o+(-0.30394278188)*x_ref[2]
        ref[2,1,1,0]=(0.296245249928)*x_ref[0]**o+(0.132631019281)*x_ref[0]+(-0.376880152158)*x_ref[1]**o+(0.303184156241)*x_ref[1]+(0.836046560882)*x_ref[2]**o+(0.195205645106)*x_ref[2]
        ref[2,1,1,1]=(-0.971045820953)*x_ref[0]**o+(0.154708896467)*x_ref[0]+(-0.663844656602)*x_ref[1]**o+(-0.0873797588245)*x_ref[1]+(0.985123372542)*x_ref[2]**o+(0.349554922131)*x_ref[2]
        ref[2,1,2,0]=(-0.0117443431348)*x_ref[0]**o+(-0.71980503985)*x_ref[0]+(0.175348078994)*x_ref[1]**o+(-0.883987892161)*x_ref[1]+(0.0587275450458)*x_ref[2]**o+(0.464811715548)*x_ref[2]
        ref[2,1,2,1]=(-0.309202575765)*x_ref[0]**o+(-0.528513248453)*x_ref[0]+(-0.509298303972)*x_ref[1]**o+(-0.228278863496)*x_ref[1]+(0.933917053676)*x_ref[2]**o+(-0.823678942264)*x_ref[2]
        ref[2,2,0,0]=(0.374387386929)*x_ref[0]**o+(0.64969993847)*x_ref[0]+(-0.939822635641)*x_ref[1]**o+(0.578249430422)*x_ref[1]+(-0.973412660635)*x_ref[2]**o+(0.484052718386)*x_ref[2]
        ref[2,2,0,1]=(-0.972635162618)*x_ref[0]**o+(-0.255007433802)*x_ref[0]+(0.134165187097)*x_ref[1]**o+(0.313213615976)*x_ref[1]+(0.143298148686)*x_ref[2]**o+(0.901819440229)*x_ref[2]
        ref[2,2,1,0]=(0.868173256315)*x_ref[0]**o+(-0.532325343453)*x_ref[0]+(-0.712500880845)*x_ref[1]**o+(0.216943462308)*x_ref[1]+(0.733920091212)*x_ref[2]**o+(0.0992400603383)*x_ref[2]
        ref[2,2,1,1]=(0.642839350527)*x_ref[0]**o+(0.127721543559)*x_ref[0]+(-0.152686523032)*x_ref[1]**o+(-0.399032361234)*x_ref[1]+(0.317014518842)*x_ref[2]**o+(0.905538790115)*x_ref[2]
        ref[2,2,2,0]=(0.685267932413)*x_ref[0]**o+(-0.167571547782)*x_ref[0]+(-0.213446144405)*x_ref[1]**o+(0.420270456377)*x_ref[1]+(0.827725350621)*x_ref[2]**o+(-0.920242274399)*x_ref[2]
        ref[2,2,2,1]=(-0.507424593437)*x_ref[0]**o+(-0.694222840019)*x_ref[0]+(-0.904108970188)*x_ref[1]**o+(-0.750899614966)*x_ref[1]+(0.783997553302)*x_ref[2]**o+(0.196327513447)*x_ref[2]
        ref[2,3,0,0]=(-0.473900600479)*x_ref[0]**o+(-0.464347097789)*x_ref[0]+(-0.472423358671)*x_ref[1]**o+(0.373786456364)*x_ref[1]+(-0.989371300414)*x_ref[2]**o+(-0.959508065239)*x_ref[2]
        ref[2,3,0,1]=(0.174375189399)*x_ref[0]**o+(0.925503059886)*x_ref[0]+(-0.647274384723)*x_ref[1]**o+(-0.577149479679)*x_ref[1]+(0.171298877768)*x_ref[2]**o+(-0.936612100882)*x_ref[2]
        ref[2,3,1,0]=(0.482959343396)*x_ref[0]**o+(0.842243554011)*x_ref[0]+(0.814423633314)*x_ref[1]**o+(-0.989445050315)*x_ref[1]+(0.817933433065)*x_ref[2]**o+(0.552396881404)*x_ref[2]
        ref[2,3,1,1]=(0.55641736229)*x_ref[0]**o+(0.498094640058)*x_ref[0]+(0.0890113606037)*x_ref[1]**o+(0.36592723553)*x_ref[1]+(0.458607593386)*x_ref[2]**o+(0.603047420063)*x_ref[2]
        ref[2,3,2,0]=(0.884515080124)*x_ref[0]**o+(-0.561898381022)*x_ref[0]+(-0.292475503834)*x_ref[1]**o+(0.425634423478)*x_ref[1]+(0.65813323252)*x_ref[2]**o+(-0.548850399842)*x_ref[2]
        ref[2,3,2,1]=(-0.72895566982)*x_ref[0]**o+(-0.569405588005)*x_ref[0]+(0.117770875594)*x_ref[1]**o+(-0.33515735969)*x_ref[1]+(0.102062736382)*x_ref[2]**o+(0.611532417683)*x_ref[2]
        ref[2,4,0,0]=(-0.669208630424)*x_ref[0]**o+(0.944393014177)*x_ref[0]+(-0.00345511889683)*x_ref[1]**o+(0.0695279595783)*x_ref[1]+(0.496948250833)*x_ref[2]**o+(-0.899750463971)*x_ref[2]
        ref[2,4,0,1]=(-0.387828717292)*x_ref[0]**o+(0.621172267004)*x_ref[0]+(-0.280632674111)*x_ref[1]**o+(0.401702567575)*x_ref[1]+(-0.65554489378)*x_ref[2]**o+(0.403018047007)*x_ref[2]
        ref[2,4,1,0]=(-0.0770246925803)*x_ref[0]**o+(0.52913302745)*x_ref[0]+(0.922468724357)*x_ref[1]**o+(-0.304442750439)*x_ref[1]+(-0.556575079059)*x_ref[2]**o+(-0.725790672251)*x_ref[2]
        ref[2,4,1,1]=(-0.828378342911)*x_ref[0]**o+(-0.0696370821776)*x_ref[0]+(0.394018581667)*x_ref[1]**o+(-0.251099848269)*x_ref[1]+(-0.29878985656)*x_ref[2]**o+(0.925753371056)*x_ref[2]
        ref[2,4,2,0]=(-0.818277786029)*x_ref[0]**o+(0.225862354706)*x_ref[0]+(-0.629050212595)*x_ref[1]**o+(0.139186107062)*x_ref[1]+(-0.0134835519238)*x_ref[2]**o+(0.301899528641)*x_ref[2]
        ref[2,4,2,1]=(-0.783053684767)*x_ref[0]**o+(-0.0926938783599)*x_ref[0]+(0.519034210352)*x_ref[1]**o+(-0.494564657126)*x_ref[1]+(0.536117310622)*x_ref[2]**o+(0.6278756634)*x_ref[2]
        ref[3,0,0,0]=(-0.694353957685)*x_ref[0]**o+(-0.361037363965)*x_ref[0]+(-0.327004091885)*x_ref[1]**o+(-0.765735142884)*x_ref[1]+(-0.810915729713)*x_ref[2]**o+(0.0640539311609)*x_ref[2]
        ref[3,0,0,1]=(-0.131601548312)*x_ref[0]**o+(0.271587756708)*x_ref[0]+(0.322486168005)*x_ref[1]**o+(0.238086581801)*x_ref[1]+(-0.488081720555)*x_ref[2]**o+(0.607278995635)*x_ref[2]
        ref[3,0,1,0]=(0.903075418102)*x_ref[0]**o+(-0.738609834848)*x_ref[0]+(-0.165097989126)*x_ref[1]**o+(-0.25967140914)*x_ref[1]+(0.271649887428)*x_ref[2]**o+(-0.786131648727)*x_ref[2]
        ref[3,0,1,1]=(-0.963442989157)*x_ref[0]**o+(-0.240205602334)*x_ref[0]+(0.102153389731)*x_ref[1]**o+(-0.106402434995)*x_ref[1]+(-0.383097724211)*x_ref[2]**o+(0.680290963195)*x_ref[2]
        ref[3,0,2,0]=(-0.373162835796)*x_ref[0]**o+(-0.186323761814)*x_ref[0]+(0.661494315705)*x_ref[1]**o+(0.320072189494)*x_ref[1]+(0.0964443431067)*x_ref[2]**o+(0.829015838241)*x_ref[2]
        ref[3,0,2,1]=(0.527719514862)*x_ref[0]**o+(0.0819948261665)*x_ref[0]+(0.826598712178)*x_ref[1]**o+(-0.588102975125)*x_ref[1]+(-0.632584402292)*x_ref[2]**o+(0.0826992233702)*x_ref[2]
        ref[3,1,0,0]=(-0.102243686287)*x_ref[0]**o+(-0.497336112929)*x_ref[0]+(0.941827592628)*x_ref[1]**o+(0.68678991944)*x_ref[1]+(-0.508684841667)*x_ref[2]**o+(-0.752939965161)*x_ref[2]
        ref[3,1,0,1]=(-0.896417523183)*x_ref[0]**o+(0.759224781685)*x_ref[0]+(0.138537695823)*x_ref[1]**o+(0.466727480471)*x_ref[1]+(-0.838380824412)*x_ref[2]**o+(0.285761157398)*x_ref[2]
        ref[3,1,1,0]=(-0.547853182919)*x_ref[0]**o+(-0.41595413414)*x_ref[0]+(0.76894221584)*x_ref[1]**o+(-0.142394275192)*x_ref[1]+(-0.96349294581)*x_ref[2]**o+(-0.873639964693)*x_ref[2]
        ref[3,1,1,1]=(-0.151299403888)*x_ref[0]**o+(0.789406063878)*x_ref[0]+(0.163280670253)*x_ref[1]**o+(-0.287315494745)*x_ref[1]+(-0.627764670218)*x_ref[2]**o+(0.842131632598)*x_ref[2]
        ref[3,1,2,0]=(-0.957096489189)*x_ref[0]**o+(0.0616037805677)*x_ref[0]+(0.255858200166)*x_ref[1]**o+(-0.430555450798)*x_ref[1]+(0.730436930737)*x_ref[2]**o+(0.405094271653)*x_ref[2]
        ref[3,1,2,1]=(-0.924756832665)*x_ref[0]**o+(0.525754162391)*x_ref[0]+(0.281303228038)*x_ref[1]**o+(-0.479500161878)*x_ref[1]+(0.658416209181)*x_ref[2]**o+(-0.0413881595557)*x_ref[2]
        ref[3,2,0,0]=(0.516429381997)*x_ref[0]**o+(-0.8387318467)*x_ref[0]+(0.329773978227)*x_ref[1]**o+(0.599641753085)*x_ref[1]+(0.939937743271)*x_ref[2]**o+(-0.247331169626)*x_ref[2]
        ref[3,2,0,1]=(-0.644982044558)*x_ref[0]**o+(-0.242033352774)*x_ref[0]+(0.531835185779)*x_ref[1]**o+(-0.851565064183)*x_ref[1]+(-0.79213624915)*x_ref[2]**o+(0.0607365690732)*x_ref[2]
        ref[3,2,1,0]=(0.359781383537)*x_ref[0]**o+(-0.0529212842074)*x_ref[0]+(0.845915169452)*x_ref[1]**o+(-0.323979570458)*x_ref[1]+(0.828909997631)*x_ref[2]**o+(-0.839886339138)*x_ref[2]
        ref[3,2,1,1]=(0.559476630421)*x_ref[0]**o+(-0.189725214615)*x_ref[0]+(-0.00927446700413)*x_ref[1]**o+(-0.234703720782)*x_ref[1]+(-0.608362806755)*x_ref[2]**o+(0.968571765534)*x_ref[2]
        ref[3,2,2,0]=(0.924837507913)*x_ref[0]**o+(0.846397611121)*x_ref[0]+(-0.837765472403)*x_ref[1]**o+(0.0225105985057)*x_ref[1]+(-0.834187513844)*x_ref[2]**o+(0.194311845578)*x_ref[2]
        ref[3,2,2,1]=(-0.183561089961)*x_ref[0]**o+(0.0745071729471)*x_ref[0]+(-0.660415615202)*x_ref[1]**o+(-0.528184997345)*x_ref[1]+(0.205554094557)*x_ref[2]**o+(-0.0309246318944)*x_ref[2]
        ref[3,3,0,0]=(-0.67124095974)*x_ref[0]**o+(-0.48501497083)*x_ref[0]+(-0.0394345869094)*x_ref[1]**o+(0.586692270429)*x_ref[1]+(-0.593760749687)*x_ref[2]**o+(-0.124979751327)*x_ref[2]
        ref[3,3,0,1]=(0.778544173855)*x_ref[0]**o+(-0.931466478494)*x_ref[0]+(-0.746894777998)*x_ref[1]**o+(0.806417936297)*x_ref[1]+(0.989460929848)*x_ref[2]**o+(0.666534366369)*x_ref[2]
        ref[3,3,1,0]=(-0.798870838602)*x_ref[0]**o+(-0.294618868937)*x_ref[0]+(0.275029092968)*x_ref[1]**o+(0.45390580364)*x_ref[1]+(-0.311797818508)*x_ref[2]**o+(0.751606624393)*x_ref[2]
        ref[3,3,1,1]=(-0.924260987477)*x_ref[0]**o+(0.771970726902)*x_ref[0]+(-0.909099224213)*x_ref[1]**o+(-0.45204206766)*x_ref[1]+(0.630042614017)*x_ref[2]**o+(-0.990915130464)*x_ref[2]
        ref[3,3,2,0]=(0.734962154852)*x_ref[0]**o+(-0.00621898011475)*x_ref[0]+(-0.786362527189)*x_ref[1]**o+(0.623507947406)*x_ref[1]+(0.867175567104)*x_ref[2]**o+(-0.383283255448)*x_ref[2]
        ref[3,3,2,1]=(-0.955613156717)*x_ref[0]**o+(0.688013754841)*x_ref[0]+(-0.600559375208)*x_ref[1]**o+(0.453342626645)*x_ref[1]+(-0.864890477961)*x_ref[2]**o+(-0.458029873063)*x_ref[2]
        ref[3,4,0,0]=(-0.897010884834)*x_ref[0]**o+(-0.251223576458)*x_ref[0]+(-0.0338052047964)*x_ref[1]**o+(-0.903421466132)*x_ref[1]+(0.836864217874)*x_ref[2]**o+(0.660805905175)*x_ref[2]
        ref[3,4,0,1]=(-0.286025690215)*x_ref[0]**o+(0.746732975604)*x_ref[0]+(0.375810853218)*x_ref[1]**o+(-0.985192974869)*x_ref[1]+(-0.731463214455)*x_ref[2]**o+(-0.465450352444)*x_ref[2]
        ref[3,4,1,0]=(-0.500910000652)*x_ref[0]**o+(0.387502419727)*x_ref[0]+(0.962369164802)*x_ref[1]**o+(-0.10933824251)*x_ref[1]+(-0.33642703989)*x_ref[2]**o+(0.546851133256)*x_ref[2]
        ref[3,4,1,1]=(-0.117959459645)*x_ref[0]**o+(0.867699323392)*x_ref[0]+(0.950506403861)*x_ref[1]**o+(0.613806207745)*x_ref[1]+(0.425723173807)*x_ref[2]**o+(-0.906937548619)*x_ref[2]
        ref[3,4,2,0]=(0.331114892194)*x_ref[0]**o+(-0.168829643424)*x_ref[0]+(-0.855962311982)*x_ref[1]**o+(-0.144202406673)*x_ref[1]+(0.0769985713088)*x_ref[2]**o+(0.856003746423)*x_ref[2]
        ref[3,4,2,1]=(-0.439397439364)*x_ref[0]**o+(0.686116343481)*x_ref[0]+(0.519883854523)*x_ref[1]**o+(0.716231469289)*x_ref[1]+(0.270098231149)*x_ref[2]**o+(-0.448525436427)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_ReducedSolution_rank0(self):
      """
      tests interpolate for rank 0 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedSolution onto ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Data(0,(),w)
      ref=Data(0,(),w_ref)
      if dim==2:
        arg=(-0.787646538574)*x[0]+(0.912422904218)*x[1]
        ref=(-0.787646538574)*x_ref[0]+(0.912422904218)*x_ref[1]
      else:
        arg=(0.934044939024)*x[0]+(0.628477728603)*x[1]+(-0.786339405837)*x[2]
        ref=(0.934044939024)*x_ref[0]+(0.628477728603)*x_ref[1]+(-0.786339405837)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_ReducedSolution_rank1(self):
      """
      tests interpolate for rank 1 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedSolution onto ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Data(0,(2,),w)
      ref=Data(0,(2,),w_ref)
      if dim==2:
        arg[0]=(-0.588326094839)*x[0]+(1.56296941343)*x[1]
        arg[1]=(-0.802191222185)*x[0]+(-1.13471016062)*x[1]
        ref[0]=(-0.588326094839)*x[0]+(1.56296941343)*x[1]
        ref[1]=(-0.802191222185)*x[0]+(-1.13471016062)*x[1]
      else:
        arg[0]=(0.160702771321)*x[0]+(-0.0784425374814)*x[1]+(1.65226639722)*x[2]
        arg[1]=(0.745050820919)*x[0]+(-0.183859360073)*x[1]+(0.399813139488)*x[2]
        ref[0]=(0.160702771321)*x[0]+(-0.0784425374814)*x[1]+(1.65226639722)*x[2]
        ref[1]=(0.745050820919)*x[0]+(-0.183859360073)*x[1]+(0.399813139488)*x[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_ReducedSolution_rank2(self):
      """
      tests interpolate for rank 2 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedSolution onto ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5),w)
      ref=Data(0,(4, 5),w_ref)
      if dim==2:
        arg[0,0]=(-0.797066914479)*x[0]+(-0.0269853164939)*x[1]
        arg[0,1]=(-1.07443661798)*x[0]+(0.818510216043)*x[1]
        arg[0,2]=(0.398735262179)*x[0]+(-0.580387872083)*x[1]
        arg[0,3]=(-0.003185613959)*x[0]+(-0.670506330615)*x[1]
        arg[0,4]=(0.385009570186)*x[0]+(0.858570512936)*x[1]
        arg[1,0]=(1.60917176753)*x[0]+(-0.0866362449992)*x[1]
        arg[1,1]=(-0.883782122452)*x[0]+(-0.402187705578)*x[1]
        arg[1,2]=(0.153893095122)*x[0]+(-1.19636362644)*x[1]
        arg[1,3]=(1.92041961031)*x[0]+(0.325857877636)*x[1]
        arg[1,4]=(-0.0947698843729)*x[0]+(-0.0663785758627)*x[1]
        arg[2,0]=(0.0232949467051)*x[0]+(1.60134210326)*x[1]
        arg[2,1]=(-0.521182882924)*x[0]+(-0.507401571343)*x[1]
        arg[2,2]=(0.833797117616)*x[0]+(-0.268886477194)*x[1]
        arg[2,3]=(-0.867762611227)*x[0]+(1.03727088566)*x[1]
        arg[2,4]=(0.380510651918)*x[0]+(0.350246888456)*x[1]
        arg[3,0]=(-1.10155500839)*x[0]+(0.414683749118)*x[1]
        arg[3,1]=(-0.0306733233552)*x[0]+(-0.351251014237)*x[1]
        arg[3,2]=(-0.0708773001498)*x[0]+(-0.326636587948)*x[1]
        arg[3,3]=(-1.32553119895)*x[0]+(-1.5852157477)*x[1]
        arg[3,4]=(1.40177154802)*x[0]+(-1.24804787994)*x[1]
        ref[0,0]=(-0.797066914479)*x_ref[0]+(-0.0269853164939)*x_ref[1]
        ref[0,1]=(-1.07443661798)*x_ref[0]+(0.818510216043)*x_ref[1]
        ref[0,2]=(0.398735262179)*x_ref[0]+(-0.580387872083)*x_ref[1]
        ref[0,3]=(-0.003185613959)*x_ref[0]+(-0.670506330615)*x_ref[1]
        ref[0,4]=(0.385009570186)*x_ref[0]+(0.858570512936)*x_ref[1]
        ref[1,0]=(1.60917176753)*x_ref[0]+(-0.0866362449992)*x_ref[1]
        ref[1,1]=(-0.883782122452)*x_ref[0]+(-0.402187705578)*x_ref[1]
        ref[1,2]=(0.153893095122)*x_ref[0]+(-1.19636362644)*x_ref[1]
        ref[1,3]=(1.92041961031)*x_ref[0]+(0.325857877636)*x_ref[1]
        ref[1,4]=(-0.0947698843729)*x_ref[0]+(-0.0663785758627)*x_ref[1]
        ref[2,0]=(0.0232949467051)*x_ref[0]+(1.60134210326)*x_ref[1]
        ref[2,1]=(-0.521182882924)*x_ref[0]+(-0.507401571343)*x_ref[1]
        ref[2,2]=(0.833797117616)*x_ref[0]+(-0.268886477194)*x_ref[1]
        ref[2,3]=(-0.867762611227)*x_ref[0]+(1.03727088566)*x_ref[1]
        ref[2,4]=(0.380510651918)*x_ref[0]+(0.350246888456)*x_ref[1]
        ref[3,0]=(-1.10155500839)*x_ref[0]+(0.414683749118)*x_ref[1]
        ref[3,1]=(-0.0306733233552)*x_ref[0]+(-0.351251014237)*x_ref[1]
        ref[3,2]=(-0.0708773001498)*x_ref[0]+(-0.326636587948)*x_ref[1]
        ref[3,3]=(-1.32553119895)*x_ref[0]+(-1.5852157477)*x_ref[1]
        ref[3,4]=(1.40177154802)*x_ref[0]+(-1.24804787994)*x_ref[1]
      else:
        arg[0,0]=(0.000362194678523)*x[0]+(-1.63949220671)*x[1]+(-0.176997156562)*x[2]
        arg[0,1]=(-0.357589049408)*x[0]+(0.828191515325)*x[1]+(-0.269706780136)*x[2]
        arg[0,2]=(0.886395070544)*x[0]+(0.815688451839)*x[1]+(-0.658412270558)*x[2]
        arg[0,3]=(-0.222573318026)*x[0]+(0.137763803092)*x[1]+(0.171902150187)*x[2]
        arg[0,4]=(1.05159741251)*x[0]+(0.923836841038)*x[1]+(-0.426590801731)*x[2]
        arg[1,0]=(-0.00718419057788)*x[0]+(1.55809964998)*x[1]+(-0.912766657496)*x[2]
        arg[1,1]=(1.30769045785)*x[0]+(-0.0356609338389)*x[1]+(0.214543084134)*x[2]
        arg[1,2]=(-1.56763014241)*x[0]+(-0.582970995722)*x[1]+(-0.513972924752)*x[2]
        arg[1,3]=(0.118027736928)*x[0]+(-0.28795014876)*x[1]+(0.620718623034)*x[2]
        arg[1,4]=(-0.784496170193)*x[0]+(0.233037866924)*x[1]+(-0.0500718134572)*x[2]
        arg[2,0]=(0.800851281806)*x[0]+(0.219346484846)*x[1]+(1.19125787701)*x[2]
        arg[2,1]=(-1.45025879429)*x[0]+(-1.32011538154)*x[1]+(0.47946669308)*x[2]
        arg[2,2]=(-0.231377110469)*x[0]+(-0.921551695999)*x[1]+(-0.298964635841)*x[2]
        arg[2,3]=(0.965766335546)*x[0]+(0.424791610089)*x[1]+(1.01860652092)*x[2]
        arg[2,4]=(0.676247093636)*x[0]+(-1.50802741511)*x[1]+(0.0385960918391)*x[2]
        arg[3,0]=(0.322378151756)*x[0]+(-0.483633005509)*x[1]+(-0.852371707865)*x[2]
        arg[3,1]=(-0.241998307153)*x[0]+(-1.08134340082)*x[1]+(-0.849574532537)*x[2]
        arg[3,2]=(-0.0185751405467)*x[0]+(0.179558416001)*x[1]+(-0.488727308548)*x[2]
        arg[3,3]=(0.0984555057845)*x[0]+(1.03085967788)*x[1]+(-0.357009094371)*x[2]
        arg[3,4]=(0.430593980121)*x[0]+(0.689565614489)*x[1]+(-0.0916541808491)*x[2]
        ref[0,0]=(0.000362194678523)*x_ref[0]+(-1.63949220671)*x_ref[1]+(-0.176997156562)*x_ref[2]
        ref[0,1]=(-0.357589049408)*x_ref[0]+(0.828191515325)*x_ref[1]+(-0.269706780136)*x_ref[2]
        ref[0,2]=(0.886395070544)*x_ref[0]+(0.815688451839)*x_ref[1]+(-0.658412270558)*x_ref[2]
        ref[0,3]=(-0.222573318026)*x_ref[0]+(0.137763803092)*x_ref[1]+(0.171902150187)*x_ref[2]
        ref[0,4]=(1.05159741251)*x_ref[0]+(0.923836841038)*x_ref[1]+(-0.426590801731)*x_ref[2]
        ref[1,0]=(-0.00718419057788)*x_ref[0]+(1.55809964998)*x_ref[1]+(-0.912766657496)*x_ref[2]
        ref[1,1]=(1.30769045785)*x_ref[0]+(-0.0356609338389)*x_ref[1]+(0.214543084134)*x_ref[2]
        ref[1,2]=(-1.56763014241)*x_ref[0]+(-0.582970995722)*x_ref[1]+(-0.513972924752)*x_ref[2]
        ref[1,3]=(0.118027736928)*x_ref[0]+(-0.28795014876)*x_ref[1]+(0.620718623034)*x_ref[2]
        ref[1,4]=(-0.784496170193)*x_ref[0]+(0.233037866924)*x_ref[1]+(-0.0500718134572)*x_ref[2]
        ref[2,0]=(0.800851281806)*x_ref[0]+(0.219346484846)*x_ref[1]+(1.19125787701)*x_ref[2]
        ref[2,1]=(-1.45025879429)*x_ref[0]+(-1.32011538154)*x_ref[1]+(0.47946669308)*x_ref[2]
        ref[2,2]=(-0.231377110469)*x_ref[0]+(-0.921551695999)*x_ref[1]+(-0.298964635841)*x_ref[2]
        ref[2,3]=(0.965766335546)*x_ref[0]+(0.424791610089)*x_ref[1]+(1.01860652092)*x_ref[2]
        ref[2,4]=(0.676247093636)*x_ref[0]+(-1.50802741511)*x_ref[1]+(0.0385960918391)*x_ref[2]
        ref[3,0]=(0.322378151756)*x_ref[0]+(-0.483633005509)*x_ref[1]+(-0.852371707865)*x_ref[2]
        ref[3,1]=(-0.241998307153)*x_ref[0]+(-1.08134340082)*x_ref[1]+(-0.849574532537)*x_ref[2]
        ref[3,2]=(-0.0185751405467)*x_ref[0]+(0.179558416001)*x_ref[1]+(-0.488727308548)*x_ref[2]
        ref[3,3]=(0.0984555057845)*x_ref[0]+(1.03085967788)*x_ref[1]+(-0.357009094371)*x_ref[2]
        ref[3,4]=(0.430593980121)*x_ref[0]+(0.689565614489)*x_ref[1]+(-0.0916541808491)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_ReducedSolution_rank3(self):
      """
      tests interpolate for rank 3 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedSolution onto ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2),w_ref)
      if dim==2:
        arg[0,0,0]=(-0.0641544023301)*x[0]+(0.364286357763)*x[1]
        arg[0,0,1]=(-0.858288240411)*x[0]+(-0.847764393559)*x[1]
        arg[0,1,0]=(-0.656948775029)*x[0]+(1.69748608992)*x[1]
        arg[0,1,1]=(0.579399191549)*x[0]+(1.48407368783)*x[1]
        arg[1,0,0]=(-0.35948657403)*x[0]+(0.795436090765)*x[1]
        arg[1,0,1]=(-1.32807156246)*x[0]+(0.921960610294)*x[1]
        arg[1,1,0]=(1.0904085219)*x[0]+(-1.41870997296)*x[1]
        arg[1,1,1]=(-0.677691596793)*x[0]+(-0.395373706333)*x[1]
        arg[2,0,0]=(0.295591917216)*x[0]+(-1.75550830457)*x[1]
        arg[2,0,1]=(0.506665326626)*x[0]+(0.273265713965)*x[1]
        arg[2,1,0]=(0.543899865352)*x[0]+(-0.0678457091657)*x[1]
        arg[2,1,1]=(-0.887824769535)*x[0]+(0.527475538055)*x[1]
        arg[3,0,0]=(-0.788511557365)*x[0]+(-0.635972100675)*x[1]
        arg[3,0,1]=(-1.02799066459)*x[0]+(-0.416241247725)*x[1]
        arg[3,1,0]=(-0.310072483091)*x[0]+(1.2445160445)*x[1]
        arg[3,1,1]=(0.912462196567)*x[0]+(0.631855269783)*x[1]
        arg[4,0,0]=(0.700426861807)*x[0]+(-0.491365913713)*x[1]
        arg[4,0,1]=(0.628791538338)*x[0]+(0.640767645586)*x[1]
        arg[4,1,0]=(0.370886720329)*x[0]+(0.823663448834)*x[1]
        arg[4,1,1]=(1.75857310774)*x[0]+(0.445439424641)*x[1]
        arg[5,0,0]=(1.39637385066)*x[0]+(-0.812211860169)*x[1]
        arg[5,0,1]=(0.865050350195)*x[0]+(-1.25983380375)*x[1]
        arg[5,1,0]=(-0.0429057645445)*x[0]+(-1.08490578934)*x[1]
        arg[5,1,1]=(1.42851264594)*x[0]+(1.52192971749)*x[1]
        ref[0,0,0]=(-0.0641544023301)*x_ref[0]+(0.364286357763)*x_ref[1]
        ref[0,0,1]=(-0.858288240411)*x_ref[0]+(-0.847764393559)*x_ref[1]
        ref[0,1,0]=(-0.656948775029)*x_ref[0]+(1.69748608992)*x_ref[1]
        ref[0,1,1]=(0.579399191549)*x_ref[0]+(1.48407368783)*x_ref[1]
        ref[1,0,0]=(-0.35948657403)*x_ref[0]+(0.795436090765)*x_ref[1]
        ref[1,0,1]=(-1.32807156246)*x_ref[0]+(0.921960610294)*x_ref[1]
        ref[1,1,0]=(1.0904085219)*x_ref[0]+(-1.41870997296)*x_ref[1]
        ref[1,1,1]=(-0.677691596793)*x_ref[0]+(-0.395373706333)*x_ref[1]
        ref[2,0,0]=(0.295591917216)*x_ref[0]+(-1.75550830457)*x_ref[1]
        ref[2,0,1]=(0.506665326626)*x_ref[0]+(0.273265713965)*x_ref[1]
        ref[2,1,0]=(0.543899865352)*x_ref[0]+(-0.0678457091657)*x_ref[1]
        ref[2,1,1]=(-0.887824769535)*x_ref[0]+(0.527475538055)*x_ref[1]
        ref[3,0,0]=(-0.788511557365)*x_ref[0]+(-0.635972100675)*x_ref[1]
        ref[3,0,1]=(-1.02799066459)*x_ref[0]+(-0.416241247725)*x_ref[1]
        ref[3,1,0]=(-0.310072483091)*x_ref[0]+(1.2445160445)*x_ref[1]
        ref[3,1,1]=(0.912462196567)*x_ref[0]+(0.631855269783)*x_ref[1]
        ref[4,0,0]=(0.700426861807)*x_ref[0]+(-0.491365913713)*x_ref[1]
        ref[4,0,1]=(0.628791538338)*x_ref[0]+(0.640767645586)*x_ref[1]
        ref[4,1,0]=(0.370886720329)*x_ref[0]+(0.823663448834)*x_ref[1]
        ref[4,1,1]=(1.75857310774)*x_ref[0]+(0.445439424641)*x_ref[1]
        ref[5,0,0]=(1.39637385066)*x_ref[0]+(-0.812211860169)*x_ref[1]
        ref[5,0,1]=(0.865050350195)*x_ref[0]+(-1.25983380375)*x_ref[1]
        ref[5,1,0]=(-0.0429057645445)*x_ref[0]+(-1.08490578934)*x_ref[1]
        ref[5,1,1]=(1.42851264594)*x_ref[0]+(1.52192971749)*x_ref[1]
      else:
        arg[0,0,0]=(0.731194732468)*x[0]+(-0.0822115660022)*x[1]+(-0.403175091289)*x[2]
        arg[0,0,1]=(0.39292230307)*x[0]+(1.48598681615)*x[1]+(1.05545374107)*x[2]
        arg[0,1,0]=(0.306844260706)*x[0]+(0.836496026763)*x[1]+(-0.121120983155)*x[2]
        arg[0,1,1]=(-0.155245802062)*x[0]+(0.800232934864)*x[1]+(0.373397160636)*x[2]
        arg[1,0,0]=(0.155941790359)*x[0]+(0.826821969281)*x[1]+(0.849596807425)*x[2]
        arg[1,0,1]=(-0.0988854656187)*x[0]+(-0.855032499842)*x[1]+(0.141988288566)*x[2]
        arg[1,1,0]=(-0.761032539296)*x[0]+(-0.0892979073234)*x[1]+(1.93034385132)*x[2]
        arg[1,1,1]=(-1.08303603558)*x[0]+(-0.487250117659)*x[1]+(0.0358501036173)*x[2]
        arg[2,0,0]=(0.148391465413)*x[0]+(-0.731963025625)*x[1]+(0.484236965689)*x[2]
        arg[2,0,1]=(1.06691217722)*x[0]+(-0.638095266244)*x[1]+(-0.971838410779)*x[2]
        arg[2,1,0]=(-0.377750829061)*x[0]+(-0.101467967263)*x[1]+(-1.54241387349)*x[2]
        arg[2,1,1]=(0.62210350231)*x[0]+(0.0435334873911)*x[1]+(-0.299303646623)*x[2]
        arg[3,0,0]=(0.523811147108)*x[0]+(-1.09709839108)*x[1]+(1.83592280935)*x[2]
        arg[3,0,1]=(0.957738143475)*x[0]+(0.282158440867)*x[1]+(0.621694180522)*x[2]
        arg[3,1,0]=(0.366557891822)*x[0]+(0.812194047631)*x[1]+(-0.245067876349)*x[2]
        arg[3,1,1]=(0.604859464982)*x[0]+(1.08415514854)*x[1]+(-1.470128848)*x[2]
        arg[4,0,0]=(-0.13031502499)*x[0]+(-1.20707238733)*x[1]+(0.108382497552)*x[2]
        arg[4,0,1]=(1.30453246317)*x[0]+(1.59696612588)*x[1]+(-0.234259043697)*x[2]
        arg[4,1,0]=(-0.233494640836)*x[0]+(1.10701061816)*x[1]+(-0.409927013674)*x[2]
        arg[4,1,1]=(0.102708240266)*x[0]+(-0.381598345002)*x[1]+(-1.39077748656)*x[2]
        arg[5,0,0]=(1.06217404493)*x[0]+(1.1600791444)*x[1]+(-1.45242735389)*x[2]
        arg[5,0,1]=(-0.423024189506)*x[0]+(-0.0868948651641)*x[1]+(0.0894539043522)*x[2]
        arg[5,1,0]=(1.17650613067)*x[0]+(0.993832919965)*x[1]+(1.16546768569)*x[2]
        arg[5,1,1]=(-0.61549132897)*x[0]+(-0.499619662734)*x[1]+(-0.643422281669)*x[2]
        ref[0,0,0]=(0.731194732468)*x_ref[0]+(-0.0822115660022)*x_ref[1]+(-0.403175091289)*x_ref[2]
        ref[0,0,1]=(0.39292230307)*x_ref[0]+(1.48598681615)*x_ref[1]+(1.05545374107)*x_ref[2]
        ref[0,1,0]=(0.306844260706)*x_ref[0]+(0.836496026763)*x_ref[1]+(-0.121120983155)*x_ref[2]
        ref[0,1,1]=(-0.155245802062)*x_ref[0]+(0.800232934864)*x_ref[1]+(0.373397160636)*x_ref[2]
        ref[1,0,0]=(0.155941790359)*x_ref[0]+(0.826821969281)*x_ref[1]+(0.849596807425)*x_ref[2]
        ref[1,0,1]=(-0.0988854656187)*x_ref[0]+(-0.855032499842)*x_ref[1]+(0.141988288566)*x_ref[2]
        ref[1,1,0]=(-0.761032539296)*x_ref[0]+(-0.0892979073234)*x_ref[1]+(1.93034385132)*x_ref[2]
        ref[1,1,1]=(-1.08303603558)*x_ref[0]+(-0.487250117659)*x_ref[1]+(0.0358501036173)*x_ref[2]
        ref[2,0,0]=(0.148391465413)*x_ref[0]+(-0.731963025625)*x_ref[1]+(0.484236965689)*x_ref[2]
        ref[2,0,1]=(1.06691217722)*x_ref[0]+(-0.638095266244)*x_ref[1]+(-0.971838410779)*x_ref[2]
        ref[2,1,0]=(-0.377750829061)*x_ref[0]+(-0.101467967263)*x_ref[1]+(-1.54241387349)*x_ref[2]
        ref[2,1,1]=(0.62210350231)*x_ref[0]+(0.0435334873911)*x_ref[1]+(-0.299303646623)*x_ref[2]
        ref[3,0,0]=(0.523811147108)*x_ref[0]+(-1.09709839108)*x_ref[1]+(1.83592280935)*x_ref[2]
        ref[3,0,1]=(0.957738143475)*x_ref[0]+(0.282158440867)*x_ref[1]+(0.621694180522)*x_ref[2]
        ref[3,1,0]=(0.366557891822)*x_ref[0]+(0.812194047631)*x_ref[1]+(-0.245067876349)*x_ref[2]
        ref[3,1,1]=(0.604859464982)*x_ref[0]+(1.08415514854)*x_ref[1]+(-1.470128848)*x_ref[2]
        ref[4,0,0]=(-0.13031502499)*x_ref[0]+(-1.20707238733)*x_ref[1]+(0.108382497552)*x_ref[2]
        ref[4,0,1]=(1.30453246317)*x_ref[0]+(1.59696612588)*x_ref[1]+(-0.234259043697)*x_ref[2]
        ref[4,1,0]=(-0.233494640836)*x_ref[0]+(1.10701061816)*x_ref[1]+(-0.409927013674)*x_ref[2]
        ref[4,1,1]=(0.102708240266)*x_ref[0]+(-0.381598345002)*x_ref[1]+(-1.39077748656)*x_ref[2]
        ref[5,0,0]=(1.06217404493)*x_ref[0]+(1.1600791444)*x_ref[1]+(-1.45242735389)*x_ref[2]
        ref[5,0,1]=(-0.423024189506)*x_ref[0]+(-0.0868948651641)*x_ref[1]+(0.0894539043522)*x_ref[2]
        ref[5,1,0]=(1.17650613067)*x_ref[0]+(0.993832919965)*x_ref[1]+(1.16546768569)*x_ref[2]
        ref[5,1,1]=(-0.61549132897)*x_ref[0]+(-0.499619662734)*x_ref[1]+(-0.643422281669)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_ReducedSolution_rank4(self):
      """
      tests interpolate for rank 4 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedSolution onto ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5, 3, 2),w)
      ref=Data(0,(4, 5, 3, 2),w_ref)
      if dim==2:
        arg[0,0,0,0]=(0.227969627252)*x[0]+(0.778177350248)*x[1]
        arg[0,0,0,1]=(-0.684883711494)*x[0]+(-0.574285957132)*x[1]
        arg[0,0,1,0]=(0.483090608974)*x[0]+(0.0779929087959)*x[1]
        arg[0,0,1,1]=(-0.293150814932)*x[0]+(1.35368931993)*x[1]
        arg[0,0,2,0]=(-0.190664734662)*x[0]+(0.910883386754)*x[1]
        arg[0,0,2,1]=(-1.51255575457)*x[0]+(0.421560181501)*x[1]
        arg[0,1,0,0]=(-0.833402828032)*x[0]+(-0.400466806781)*x[1]
        arg[0,1,0,1]=(-0.212099323605)*x[0]+(-0.112478697221)*x[1]
        arg[0,1,1,0]=(0.57687662926)*x[0]+(0.869758954551)*x[1]
        arg[0,1,1,1]=(1.37261769917)*x[0]+(1.37855789907)*x[1]
        arg[0,1,2,0]=(-1.02838589722)*x[0]+(-0.280409246204)*x[1]
        arg[0,1,2,1]=(1.2333373468)*x[0]+(0.337631183798)*x[1]
        arg[0,2,0,0]=(-0.249065448772)*x[0]+(0.834483646317)*x[1]
        arg[0,2,0,1]=(-1.30860297683)*x[0]+(-0.653929967017)*x[1]
        arg[0,2,1,0]=(-0.0155994484453)*x[0]+(-0.274023166937)*x[1]
        arg[0,2,1,1]=(0.31458360335)*x[0]+(1.59507700673)*x[1]
        arg[0,2,2,0]=(-0.11442619553)*x[0]+(1.6577028576)*x[1]
        arg[0,2,2,1]=(0.274261132032)*x[0]+(1.15236962479)*x[1]
        arg[0,3,0,0]=(1.91433860239)*x[0]+(-0.316349747811)*x[1]
        arg[0,3,0,1]=(-1.42457591054)*x[0]+(-0.362812690805)*x[1]
        arg[0,3,1,0]=(0.572734883096)*x[0]+(0.424724787283)*x[1]
        arg[0,3,1,1]=(-0.118538376813)*x[0]+(0.25190312527)*x[1]
        arg[0,3,2,0]=(-0.533409510081)*x[0]+(-0.804349107915)*x[1]
        arg[0,3,2,1]=(-0.290555363666)*x[0]+(-0.912641477145)*x[1]
        arg[0,4,0,0]=(-0.469612775918)*x[0]+(-0.0447411310793)*x[1]
        arg[0,4,0,1]=(0.108538258831)*x[0]+(0.0453962016662)*x[1]
        arg[0,4,1,0]=(-0.130785944028)*x[0]+(0.0887280914032)*x[1]
        arg[0,4,1,1]=(0.166412309585)*x[0]+(-0.75130797706)*x[1]
        arg[0,4,2,0]=(1.22538232455)*x[0]+(0.112925588343)*x[1]
        arg[0,4,2,1]=(-1.02455171854)*x[0]+(-0.725754942749)*x[1]
        arg[1,0,0,0]=(-0.869171086952)*x[0]+(-0.948876783682)*x[1]
        arg[1,0,0,1]=(0.17730673689)*x[0]+(-0.00166180855579)*x[1]
        arg[1,0,1,0]=(0.482072704433)*x[0]+(0.895847096783)*x[1]
        arg[1,0,1,1]=(0.695736834617)*x[0]+(0.358082704558)*x[1]
        arg[1,0,2,0]=(-0.501799636565)*x[0]+(0.363612558025)*x[1]
        arg[1,0,2,1]=(-1.47204447226)*x[0]+(1.91281426121)*x[1]
        arg[1,1,0,0]=(0.241893182978)*x[0]+(0.95199758669)*x[1]
        arg[1,1,0,1]=(-0.68112361827)*x[0]+(-0.241935141301)*x[1]
        arg[1,1,1,0]=(-0.553030972306)*x[0]+(0.0244503690439)*x[1]
        arg[1,1,1,1]=(0.278177176896)*x[0]+(1.58693846427)*x[1]
        arg[1,1,2,0]=(-0.87980015517)*x[0]+(0.315618837123)*x[1]
        arg[1,1,2,1]=(-0.335783961379)*x[0]+(0.368650136138)*x[1]
        arg[1,2,0,0]=(-0.903535146802)*x[0]+(1.68182923194)*x[1]
        arg[1,2,0,1]=(1.06511511382)*x[0]+(-1.08524762302)*x[1]
        arg[1,2,1,0]=(-0.0455274784681)*x[0]+(-0.519576710661)*x[1]
        arg[1,2,1,1]=(0.735100217062)*x[0]+(-0.986268682671)*x[1]
        arg[1,2,2,0]=(-0.544610797462)*x[0]+(0.447237114882)*x[1]
        arg[1,2,2,1]=(0.385242031153)*x[0]+(-0.960538844247)*x[1]
        arg[1,3,0,0]=(-0.354927072897)*x[0]+(0.842574324062)*x[1]
        arg[1,3,0,1]=(0.319036427919)*x[0]+(-0.156282262536)*x[1]
        arg[1,3,1,0]=(0.587123807717)*x[0]+(1.65277990726)*x[1]
        arg[1,3,1,1]=(0.791697370298)*x[0]+(-1.5681680344)*x[1]
        arg[1,3,2,0]=(-1.41931242045)*x[0]+(0.351285136557)*x[1]
        arg[1,3,2,1]=(0.53175704402)*x[0]+(0.987523241324)*x[1]
        arg[1,4,0,0]=(0.00533504818465)*x[0]+(-0.158886930817)*x[1]
        arg[1,4,0,1]=(1.23725297303)*x[0]+(-0.302896452172)*x[1]
        arg[1,4,1,0]=(0.433953146329)*x[0]+(-0.0721602234887)*x[1]
        arg[1,4,1,1]=(-0.383499726865)*x[0]+(-0.253834671235)*x[1]
        arg[1,4,2,0]=(-0.480776695247)*x[0]+(-0.302145524759)*x[1]
        arg[1,4,2,1]=(1.14113180903)*x[0]+(-1.24304726003)*x[1]
        arg[2,0,0,0]=(0.513776013893)*x[0]+(-0.16660259885)*x[1]
        arg[2,0,0,1]=(1.55739209698)*x[0]+(0.0524084654531)*x[1]
        arg[2,0,1,0]=(0.222567881446)*x[0]+(-0.357158470388)*x[1]
        arg[2,0,1,1]=(0.0179816596229)*x[0]+(0.231534460784)*x[1]
        arg[2,0,2,0]=(1.58460345297)*x[0]+(-0.383588433888)*x[1]
        arg[2,0,2,1]=(-1.73386465536)*x[0]+(-0.743326657544)*x[1]
        arg[2,1,0,0]=(-0.737976950246)*x[0]+(-1.01053396224)*x[1]
        arg[2,1,0,1]=(0.926911190393)*x[0]+(0.620251783925)*x[1]
        arg[2,1,1,0]=(-1.34289630129)*x[0]+(-0.429933681523)*x[1]
        arg[2,1,1,1]=(1.79530806415)*x[0]+(-0.539783957021)*x[1]
        arg[2,1,2,0]=(0.237950974938)*x[0]+(1.93783427703)*x[1]
        arg[2,1,2,1]=(-0.439640741973)*x[0]+(-0.61543984673)*x[1]
        arg[2,2,0,0]=(-1.66403612438)*x[0]+(1.00839566441)*x[1]
        arg[2,2,0,1]=(0.249635983351)*x[0]+(-0.722261363675)*x[1]
        arg[2,2,1,0]=(1.34179674459)*x[0]+(0.502542144188)*x[1]
        arg[2,2,1,1]=(-0.815862586074)*x[0]+(-0.110688273707)*x[1]
        arg[2,2,2,0]=(-0.686810719526)*x[0]+(0.500406563421)*x[1]
        arg[2,2,2,1]=(0.681231089598)*x[0]+(0.605219505613)*x[1]
        arg[2,3,0,0]=(-0.953299029956)*x[0]+(-0.223725858372)*x[1]
        arg[2,3,0,1]=(0.831380756684)*x[0]+(-0.0379777076335)*x[1]
        arg[2,3,1,0]=(-1.63804393157)*x[0]+(0.148235731301)*x[1]
        arg[2,3,1,1]=(-0.17786458643)*x[0]+(-1.77342152699)*x[1]
        arg[2,3,2,0]=(0.0210446244192)*x[0]+(1.59778781524)*x[1]
        arg[2,3,2,1]=(0.877162721214)*x[0]+(0.262990448614)*x[1]
        arg[2,4,0,0]=(-0.650265970331)*x[0]+(-1.07003604369)*x[1]
        arg[2,4,0,1]=(1.12940101029)*x[0]+(0.860589290891)*x[1]
        arg[2,4,1,0]=(0.520056210056)*x[0]+(-1.47691193979)*x[1]
        arg[2,4,1,1]=(1.26823535235)*x[0]+(0.496811720647)*x[1]
        arg[2,4,2,0]=(-1.388913443)*x[0]+(0.585343197943)*x[1]
        arg[2,4,2,1]=(-0.762904547594)*x[0]+(0.503438815917)*x[1]
        arg[3,0,0,0]=(0.382862686993)*x[0]+(-0.923525513303)*x[1]
        arg[3,0,0,1]=(-0.54305794485)*x[0]+(0.467513531444)*x[1]
        arg[3,0,1,0]=(1.24228759776)*x[0]+(-1.65740117421)*x[1]
        arg[3,0,1,1]=(-0.463045362742)*x[0]+(0.186438587087)*x[1]
        arg[3,0,2,0]=(0.296881980828)*x[0]+(-0.385878814949)*x[1]
        arg[3,0,2,1]=(-0.299705899564)*x[0]+(0.490736038538)*x[1]
        arg[3,1,0,0]=(-0.786851428658)*x[0]+(-0.760593794308)*x[1]
        arg[3,1,0,1]=(0.506851336483)*x[0]+(0.995163951197)*x[1]
        arg[3,1,1,0]=(-0.678883937341)*x[0]+(0.56996778248)*x[1]
        arg[3,1,1,1]=(-0.401483642528)*x[0]+(0.0476797332537)*x[1]
        arg[3,1,2,0]=(-0.585357955375)*x[0]+(0.330103647539)*x[1]
        arg[3,1,2,1]=(-0.355539976825)*x[0]+(0.416287924349)*x[1]
        arg[3,2,0,0]=(-0.662837130286)*x[0]+(-0.26640899674)*x[1]
        arg[3,2,0,1]=(0.397553900588)*x[0]+(-1.21708620294)*x[1]
        arg[3,2,1,0]=(0.464963326429)*x[0]+(-0.774515114142)*x[1]
        arg[3,2,1,1]=(0.834351469791)*x[0]+(0.487629540715)*x[1]
        arg[3,2,2,0]=(0.27622693716)*x[0]+(0.811338337719)*x[1]
        arg[3,2,2,1]=(0.0384997772653)*x[0]+(1.03724996578)*x[1]
        arg[3,3,0,0]=(-0.805491987874)*x[0]+(-0.476384547853)*x[1]
        arg[3,3,0,1]=(0.615551780195)*x[0]+(-0.312805967691)*x[1]
        arg[3,3,1,0]=(0.348666798955)*x[0]+(-1.31271156524)*x[1]
        arg[3,3,1,1]=(-0.423569975374)*x[0]+(0.930713125389)*x[1]
        arg[3,3,2,0]=(0.325632686234)*x[0]+(-0.166401033259)*x[1]
        arg[3,3,2,1]=(0.126924861797)*x[0]+(-0.425380897969)*x[1]
        arg[3,4,0,0]=(0.169532204538)*x[0]+(0.614632525765)*x[1]
        arg[3,4,0,1]=(-0.938098696851)*x[0]+(0.648854147587)*x[1]
        arg[3,4,1,0]=(-0.545143970434)*x[0]+(-1.59787515966)*x[1]
        arg[3,4,1,1]=(-0.711190914978)*x[0]+(0.897331522911)*x[1]
        arg[3,4,2,0]=(-0.386498124525)*x[0]+(0.299740420711)*x[1]
        arg[3,4,2,1]=(0.617683032062)*x[0]+(-0.449587703061)*x[1]
        ref[0,0,0,0]=(0.227969627252)*x_ref[0]+(0.778177350248)*x_ref[1]
        ref[0,0,0,1]=(-0.684883711494)*x_ref[0]+(-0.574285957132)*x_ref[1]
        ref[0,0,1,0]=(0.483090608974)*x_ref[0]+(0.0779929087959)*x_ref[1]
        ref[0,0,1,1]=(-0.293150814932)*x_ref[0]+(1.35368931993)*x_ref[1]
        ref[0,0,2,0]=(-0.190664734662)*x_ref[0]+(0.910883386754)*x_ref[1]
        ref[0,0,2,1]=(-1.51255575457)*x_ref[0]+(0.421560181501)*x_ref[1]
        ref[0,1,0,0]=(-0.833402828032)*x_ref[0]+(-0.400466806781)*x_ref[1]
        ref[0,1,0,1]=(-0.212099323605)*x_ref[0]+(-0.112478697221)*x_ref[1]
        ref[0,1,1,0]=(0.57687662926)*x_ref[0]+(0.869758954551)*x_ref[1]
        ref[0,1,1,1]=(1.37261769917)*x_ref[0]+(1.37855789907)*x_ref[1]
        ref[0,1,2,0]=(-1.02838589722)*x_ref[0]+(-0.280409246204)*x_ref[1]
        ref[0,1,2,1]=(1.2333373468)*x_ref[0]+(0.337631183798)*x_ref[1]
        ref[0,2,0,0]=(-0.249065448772)*x_ref[0]+(0.834483646317)*x_ref[1]
        ref[0,2,0,1]=(-1.30860297683)*x_ref[0]+(-0.653929967017)*x_ref[1]
        ref[0,2,1,0]=(-0.0155994484453)*x_ref[0]+(-0.274023166937)*x_ref[1]
        ref[0,2,1,1]=(0.31458360335)*x_ref[0]+(1.59507700673)*x_ref[1]
        ref[0,2,2,0]=(-0.11442619553)*x_ref[0]+(1.6577028576)*x_ref[1]
        ref[0,2,2,1]=(0.274261132032)*x_ref[0]+(1.15236962479)*x_ref[1]
        ref[0,3,0,0]=(1.91433860239)*x_ref[0]+(-0.316349747811)*x_ref[1]
        ref[0,3,0,1]=(-1.42457591054)*x_ref[0]+(-0.362812690805)*x_ref[1]
        ref[0,3,1,0]=(0.572734883096)*x_ref[0]+(0.424724787283)*x_ref[1]
        ref[0,3,1,1]=(-0.118538376813)*x_ref[0]+(0.25190312527)*x_ref[1]
        ref[0,3,2,0]=(-0.533409510081)*x_ref[0]+(-0.804349107915)*x_ref[1]
        ref[0,3,2,1]=(-0.290555363666)*x_ref[0]+(-0.912641477145)*x_ref[1]
        ref[0,4,0,0]=(-0.469612775918)*x_ref[0]+(-0.0447411310793)*x_ref[1]
        ref[0,4,0,1]=(0.108538258831)*x_ref[0]+(0.0453962016662)*x_ref[1]
        ref[0,4,1,0]=(-0.130785944028)*x_ref[0]+(0.0887280914032)*x_ref[1]
        ref[0,4,1,1]=(0.166412309585)*x_ref[0]+(-0.75130797706)*x_ref[1]
        ref[0,4,2,0]=(1.22538232455)*x_ref[0]+(0.112925588343)*x_ref[1]
        ref[0,4,2,1]=(-1.02455171854)*x_ref[0]+(-0.725754942749)*x_ref[1]
        ref[1,0,0,0]=(-0.869171086952)*x_ref[0]+(-0.948876783682)*x_ref[1]
        ref[1,0,0,1]=(0.17730673689)*x_ref[0]+(-0.00166180855579)*x_ref[1]
        ref[1,0,1,0]=(0.482072704433)*x_ref[0]+(0.895847096783)*x_ref[1]
        ref[1,0,1,1]=(0.695736834617)*x_ref[0]+(0.358082704558)*x_ref[1]
        ref[1,0,2,0]=(-0.501799636565)*x_ref[0]+(0.363612558025)*x_ref[1]
        ref[1,0,2,1]=(-1.47204447226)*x_ref[0]+(1.91281426121)*x_ref[1]
        ref[1,1,0,0]=(0.241893182978)*x_ref[0]+(0.95199758669)*x_ref[1]
        ref[1,1,0,1]=(-0.68112361827)*x_ref[0]+(-0.241935141301)*x_ref[1]
        ref[1,1,1,0]=(-0.553030972306)*x_ref[0]+(0.0244503690439)*x_ref[1]
        ref[1,1,1,1]=(0.278177176896)*x_ref[0]+(1.58693846427)*x_ref[1]
        ref[1,1,2,0]=(-0.87980015517)*x_ref[0]+(0.315618837123)*x_ref[1]
        ref[1,1,2,1]=(-0.335783961379)*x_ref[0]+(0.368650136138)*x_ref[1]
        ref[1,2,0,0]=(-0.903535146802)*x_ref[0]+(1.68182923194)*x_ref[1]
        ref[1,2,0,1]=(1.06511511382)*x_ref[0]+(-1.08524762302)*x_ref[1]
        ref[1,2,1,0]=(-0.0455274784681)*x_ref[0]+(-0.519576710661)*x_ref[1]
        ref[1,2,1,1]=(0.735100217062)*x_ref[0]+(-0.986268682671)*x_ref[1]
        ref[1,2,2,0]=(-0.544610797462)*x_ref[0]+(0.447237114882)*x_ref[1]
        ref[1,2,2,1]=(0.385242031153)*x_ref[0]+(-0.960538844247)*x_ref[1]
        ref[1,3,0,0]=(-0.354927072897)*x_ref[0]+(0.842574324062)*x_ref[1]
        ref[1,3,0,1]=(0.319036427919)*x_ref[0]+(-0.156282262536)*x_ref[1]
        ref[1,3,1,0]=(0.587123807717)*x_ref[0]+(1.65277990726)*x_ref[1]
        ref[1,3,1,1]=(0.791697370298)*x_ref[0]+(-1.5681680344)*x_ref[1]
        ref[1,3,2,0]=(-1.41931242045)*x_ref[0]+(0.351285136557)*x_ref[1]
        ref[1,3,2,1]=(0.53175704402)*x_ref[0]+(0.987523241324)*x_ref[1]
        ref[1,4,0,0]=(0.00533504818465)*x_ref[0]+(-0.158886930817)*x_ref[1]
        ref[1,4,0,1]=(1.23725297303)*x_ref[0]+(-0.302896452172)*x_ref[1]
        ref[1,4,1,0]=(0.433953146329)*x_ref[0]+(-0.0721602234887)*x_ref[1]
        ref[1,4,1,1]=(-0.383499726865)*x_ref[0]+(-0.253834671235)*x_ref[1]
        ref[1,4,2,0]=(-0.480776695247)*x_ref[0]+(-0.302145524759)*x_ref[1]
        ref[1,4,2,1]=(1.14113180903)*x_ref[0]+(-1.24304726003)*x_ref[1]
        ref[2,0,0,0]=(0.513776013893)*x_ref[0]+(-0.16660259885)*x_ref[1]
        ref[2,0,0,1]=(1.55739209698)*x_ref[0]+(0.0524084654531)*x_ref[1]
        ref[2,0,1,0]=(0.222567881446)*x_ref[0]+(-0.357158470388)*x_ref[1]
        ref[2,0,1,1]=(0.0179816596229)*x_ref[0]+(0.231534460784)*x_ref[1]
        ref[2,0,2,0]=(1.58460345297)*x_ref[0]+(-0.383588433888)*x_ref[1]
        ref[2,0,2,1]=(-1.73386465536)*x_ref[0]+(-0.743326657544)*x_ref[1]
        ref[2,1,0,0]=(-0.737976950246)*x_ref[0]+(-1.01053396224)*x_ref[1]
        ref[2,1,0,1]=(0.926911190393)*x_ref[0]+(0.620251783925)*x_ref[1]
        ref[2,1,1,0]=(-1.34289630129)*x_ref[0]+(-0.429933681523)*x_ref[1]
        ref[2,1,1,1]=(1.79530806415)*x_ref[0]+(-0.539783957021)*x_ref[1]
        ref[2,1,2,0]=(0.237950974938)*x_ref[0]+(1.93783427703)*x_ref[1]
        ref[2,1,2,1]=(-0.439640741973)*x_ref[0]+(-0.61543984673)*x_ref[1]
        ref[2,2,0,0]=(-1.66403612438)*x_ref[0]+(1.00839566441)*x_ref[1]
        ref[2,2,0,1]=(0.249635983351)*x_ref[0]+(-0.722261363675)*x_ref[1]
        ref[2,2,1,0]=(1.34179674459)*x_ref[0]+(0.502542144188)*x_ref[1]
        ref[2,2,1,1]=(-0.815862586074)*x_ref[0]+(-0.110688273707)*x_ref[1]
        ref[2,2,2,0]=(-0.686810719526)*x_ref[0]+(0.500406563421)*x_ref[1]
        ref[2,2,2,1]=(0.681231089598)*x_ref[0]+(0.605219505613)*x_ref[1]
        ref[2,3,0,0]=(-0.953299029956)*x_ref[0]+(-0.223725858372)*x_ref[1]
        ref[2,3,0,1]=(0.831380756684)*x_ref[0]+(-0.0379777076335)*x_ref[1]
        ref[2,3,1,0]=(-1.63804393157)*x_ref[0]+(0.148235731301)*x_ref[1]
        ref[2,3,1,1]=(-0.17786458643)*x_ref[0]+(-1.77342152699)*x_ref[1]
        ref[2,3,2,0]=(0.0210446244192)*x_ref[0]+(1.59778781524)*x_ref[1]
        ref[2,3,2,1]=(0.877162721214)*x_ref[0]+(0.262990448614)*x_ref[1]
        ref[2,4,0,0]=(-0.650265970331)*x_ref[0]+(-1.07003604369)*x_ref[1]
        ref[2,4,0,1]=(1.12940101029)*x_ref[0]+(0.860589290891)*x_ref[1]
        ref[2,4,1,0]=(0.520056210056)*x_ref[0]+(-1.47691193979)*x_ref[1]
        ref[2,4,1,1]=(1.26823535235)*x_ref[0]+(0.496811720647)*x_ref[1]
        ref[2,4,2,0]=(-1.388913443)*x_ref[0]+(0.585343197943)*x_ref[1]
        ref[2,4,2,1]=(-0.762904547594)*x_ref[0]+(0.503438815917)*x_ref[1]
        ref[3,0,0,0]=(0.382862686993)*x_ref[0]+(-0.923525513303)*x_ref[1]
        ref[3,0,0,1]=(-0.54305794485)*x_ref[0]+(0.467513531444)*x_ref[1]
        ref[3,0,1,0]=(1.24228759776)*x_ref[0]+(-1.65740117421)*x_ref[1]
        ref[3,0,1,1]=(-0.463045362742)*x_ref[0]+(0.186438587087)*x_ref[1]
        ref[3,0,2,0]=(0.296881980828)*x_ref[0]+(-0.385878814949)*x_ref[1]
        ref[3,0,2,1]=(-0.299705899564)*x_ref[0]+(0.490736038538)*x_ref[1]
        ref[3,1,0,0]=(-0.786851428658)*x_ref[0]+(-0.760593794308)*x_ref[1]
        ref[3,1,0,1]=(0.506851336483)*x_ref[0]+(0.995163951197)*x_ref[1]
        ref[3,1,1,0]=(-0.678883937341)*x_ref[0]+(0.56996778248)*x_ref[1]
        ref[3,1,1,1]=(-0.401483642528)*x_ref[0]+(0.0476797332537)*x_ref[1]
        ref[3,1,2,0]=(-0.585357955375)*x_ref[0]+(0.330103647539)*x_ref[1]
        ref[3,1,2,1]=(-0.355539976825)*x_ref[0]+(0.416287924349)*x_ref[1]
        ref[3,2,0,0]=(-0.662837130286)*x_ref[0]+(-0.26640899674)*x_ref[1]
        ref[3,2,0,1]=(0.397553900588)*x_ref[0]+(-1.21708620294)*x_ref[1]
        ref[3,2,1,0]=(0.464963326429)*x_ref[0]+(-0.774515114142)*x_ref[1]
        ref[3,2,1,1]=(0.834351469791)*x_ref[0]+(0.487629540715)*x_ref[1]
        ref[3,2,2,0]=(0.27622693716)*x_ref[0]+(0.811338337719)*x_ref[1]
        ref[3,2,2,1]=(0.0384997772653)*x_ref[0]+(1.03724996578)*x_ref[1]
        ref[3,3,0,0]=(-0.805491987874)*x_ref[0]+(-0.476384547853)*x_ref[1]
        ref[3,3,0,1]=(0.615551780195)*x_ref[0]+(-0.312805967691)*x_ref[1]
        ref[3,3,1,0]=(0.348666798955)*x_ref[0]+(-1.31271156524)*x_ref[1]
        ref[3,3,1,1]=(-0.423569975374)*x_ref[0]+(0.930713125389)*x_ref[1]
        ref[3,3,2,0]=(0.325632686234)*x_ref[0]+(-0.166401033259)*x_ref[1]
        ref[3,3,2,1]=(0.126924861797)*x_ref[0]+(-0.425380897969)*x_ref[1]
        ref[3,4,0,0]=(0.169532204538)*x_ref[0]+(0.614632525765)*x_ref[1]
        ref[3,4,0,1]=(-0.938098696851)*x_ref[0]+(0.648854147587)*x_ref[1]
        ref[3,4,1,0]=(-0.545143970434)*x_ref[0]+(-1.59787515966)*x_ref[1]
        ref[3,4,1,1]=(-0.711190914978)*x_ref[0]+(0.897331522911)*x_ref[1]
        ref[3,4,2,0]=(-0.386498124525)*x_ref[0]+(0.299740420711)*x_ref[1]
        ref[3,4,2,1]=(0.617683032062)*x_ref[0]+(-0.449587703061)*x_ref[1]
      else:
        arg[0,0,0,0]=(-0.791620170192)*x[0]+(1.03871460301)*x[1]+(-0.225035657687)*x[2]
        arg[0,0,0,1]=(-0.464422319203)*x[0]+(0.769454931053)*x[1]+(-0.783260637771)*x[2]
        arg[0,0,1,0]=(0.72655538215)*x[0]+(-1.65736312015)*x[1]+(1.08402505927)*x[2]
        arg[0,0,1,1]=(-0.207576709573)*x[0]+(-0.55250594761)*x[1]+(-0.923702938288)*x[2]
        arg[0,0,2,0]=(-0.701991085516)*x[0]+(-1.76863643107)*x[1]+(0.831382933104)*x[2]
        arg[0,0,2,1]=(0.0640035527319)*x[0]+(0.425446342976)*x[1]+(1.64626766696)*x[2]
        arg[0,1,0,0]=(1.01022963801)*x[0]+(-0.0215015059716)*x[1]+(0.849862425592)*x[2]
        arg[0,1,0,1]=(-0.45776932943)*x[0]+(0.7621342151)*x[1]+(-1.40123018391)*x[2]
        arg[0,1,1,0]=(0.737485054127)*x[0]+(-0.147513905858)*x[1]+(-0.616928595213)*x[2]
        arg[0,1,1,1]=(-0.0608005391894)*x[0]+(-0.154978210573)*x[1]+(-1.40322011577)*x[2]
        arg[0,1,2,0]=(1.59908177714)*x[0]+(-0.378738573932)*x[1]+(1.45285315247)*x[2]
        arg[0,1,2,1]=(-0.622552157046)*x[0]+(1.19996417516)*x[1]+(0.728842900843)*x[2]
        arg[0,2,0,0]=(-0.46942034691)*x[0]+(0.333309721372)*x[1]+(-0.280846553832)*x[2]
        arg[0,2,0,1]=(1.22208788109)*x[0]+(0.369339594285)*x[1]+(1.31078425589)*x[2]
        arg[0,2,1,0]=(0.0794165004114)*x[0]+(0.44665451846)*x[1]+(-0.814625708916)*x[2]
        arg[0,2,1,1]=(0.594200807165)*x[0]+(-0.00616720333193)*x[1]+(0.382706175172)*x[2]
        arg[0,2,2,0]=(0.0372230170459)*x[0]+(0.440339923291)*x[1]+(0.137641070028)*x[2]
        arg[0,2,2,1]=(-1.29863078274)*x[0]+(0.283142455171)*x[1]+(0.0844540074659)*x[2]
        arg[0,3,0,0]=(1.72811808036)*x[0]+(0.536456781522)*x[1]+(0.0266737704706)*x[2]
        arg[0,3,0,1]=(0.148219555101)*x[0]+(-0.6887500275)*x[1]+(-0.405613136632)*x[2]
        arg[0,3,1,0]=(0.708262537279)*x[0]+(1.16319635773)*x[1]+(-0.030808721674)*x[2]
        arg[0,3,1,1]=(0.80834615863)*x[0]+(-0.176789000925)*x[1]+(-1.72576120963)*x[2]
        arg[0,3,2,0]=(0.21128315299)*x[0]+(0.770037046126)*x[1]+(0.988028934422)*x[2]
        arg[0,3,2,1]=(1.10522136317)*x[0]+(1.6319351743)*x[1]+(-0.784693713387)*x[2]
        arg[0,4,0,0]=(0.121088121064)*x[0]+(-1.79718067791)*x[1]+(-0.907857173318)*x[2]
        arg[0,4,0,1]=(-0.695887294217)*x[0]+(-1.24166787807)*x[1]+(-1.24968023856)*x[2]
        arg[0,4,1,0]=(-0.202510797291)*x[0]+(0.628310156105)*x[1]+(0.605146447822)*x[2]
        arg[0,4,1,1]=(0.149227458424)*x[0]+(-1.5650392775)*x[1]+(-0.763944892663)*x[2]
        arg[0,4,2,0]=(-0.393218625872)*x[0]+(0.479145914948)*x[1]+(-0.313960447492)*x[2]
        arg[0,4,2,1]=(-0.0191807443799)*x[0]+(0.592992659119)*x[1]+(-0.475633115484)*x[2]
        arg[1,0,0,0]=(-0.964829354261)*x[0]+(0.453014802906)*x[1]+(-0.779502566938)*x[2]
        arg[1,0,0,1]=(0.0151205739508)*x[0]+(-0.153706429594)*x[1]+(0.00866460475724)*x[2]
        arg[1,0,1,0]=(0.92592276366)*x[0]+(0.530787400525)*x[1]+(-0.0577912606876)*x[2]
        arg[1,0,1,1]=(0.948404467077)*x[0]+(-0.265905470751)*x[1]+(0.472349192005)*x[2]
        arg[1,0,2,0]=(0.0176615383702)*x[0]+(0.679605821761)*x[1]+(-0.0333590911941)*x[2]
        arg[1,0,2,1]=(-1.15594952629)*x[0]+(0.264868376508)*x[1]+(-0.883690535499)*x[2]
        arg[1,1,0,0]=(-0.738712703994)*x[0]+(0.545839232808)*x[1]+(0.592474501134)*x[2]
        arg[1,1,0,1]=(-1.3468960453)*x[0]+(-0.00848202443614)*x[1]+(-0.0124138286126)*x[2]
        arg[1,1,1,0]=(0.900228409707)*x[0]+(-1.86848082357)*x[1]+(0.421539431142)*x[2]
        arg[1,1,1,1]=(1.42754013963)*x[0]+(-0.861376440449)*x[1]+(0.0965520362291)*x[2]
        arg[1,1,2,0]=(-0.35873674515)*x[0]+(0.0862592617941)*x[1]+(-0.536106207025)*x[2]
        arg[1,1,2,1]=(0.90712910097)*x[0]+(0.974545253055)*x[1]+(0.570708510858)*x[2]
        arg[1,2,0,0]=(1.19310775669)*x[0]+(0.309414331679)*x[1]+(-0.271298061586)*x[2]
        arg[1,2,0,1]=(0.551778129545)*x[0]+(0.539028430134)*x[1]+(0.239406239262)*x[2]
        arg[1,2,1,0]=(-0.810625054344)*x[0]+(-0.241072190233)*x[1]+(-1.58590369097)*x[2]
        arg[1,2,1,1]=(0.529669075734)*x[0]+(-0.890365024163)*x[1]+(-1.69798509084)*x[2]
        arg[1,2,2,0]=(0.0437087707448)*x[0]+(0.88372190198)*x[1]+(1.86547747806)*x[2]
        arg[1,2,2,1]=(1.30739438898)*x[0]+(0.315850112101)*x[1]+(-0.262001133683)*x[2]
        arg[1,3,0,0]=(-0.143426236837)*x[0]+(-1.1703384826)*x[1]+(0.50608348646)*x[2]
        arg[1,3,0,1]=(-0.0444408776922)*x[0]+(1.26952782036)*x[1]+(1.67192602934)*x[2]
        arg[1,3,1,0]=(0.672742824368)*x[0]+(-0.785246676537)*x[1]+(0.716130664056)*x[2]
        arg[1,3,1,1]=(-0.295182508335)*x[0]+(-1.53658860897)*x[1]+(-1.87133995422)*x[2]
        arg[1,3,2,0]=(-0.440859530081)*x[0]+(0.0730120015821)*x[1]+(-1.02465537353)*x[2]
        arg[1,3,2,1]=(0.838320612003)*x[0]+(1.81233688454)*x[1]+(-1.0633876685)*x[2]
        arg[1,4,0,0]=(0.52440453051)*x[0]+(-0.0182588999022)*x[1]+(1.05556310351)*x[2]
        arg[1,4,0,1]=(0.766612727495)*x[0]+(1.58987590746)*x[1]+(-0.703801484569)*x[2]
        arg[1,4,1,0]=(0.373439757535)*x[0]+(-0.0676963704305)*x[1]+(0.0948034094293)*x[2]
        arg[1,4,1,1]=(-0.542283543375)*x[0]+(-0.454408605761)*x[1]+(-0.398904635316)*x[2]
        arg[1,4,2,0]=(1.3374871143)*x[0]+(0.0067849394587)*x[1]+(-0.319452094882)*x[2]
        arg[1,4,2,1]=(0.38530755393)*x[0]+(1.05356462429)*x[1]+(-0.302179447719)*x[2]
        arg[2,0,0,0]=(-0.123109761583)*x[0]+(0.69223848143)*x[1]+(-1.36903487092)*x[2]
        arg[2,0,0,1]=(0.121957328459)*x[0]+(0.102229685387)*x[1]+(0.962499974737)*x[2]
        arg[2,0,1,0]=(0.478614188653)*x[0]+(-1.23058817023)*x[1]+(1.30950997478)*x[2]
        arg[2,0,1,1]=(-0.529033562461)*x[0]+(0.563030509591)*x[1]+(-0.564939226654)*x[2]
        arg[2,0,2,0]=(-0.377740282406)*x[0]+(1.20532110343)*x[1]+(1.55615174947)*x[2]
        arg[2,0,2,1]=(0.539119710703)*x[0]+(1.91771865857)*x[1]+(0.0809882488048)*x[2]
        arg[2,1,0,0]=(0.00548664737796)*x[0]+(-0.157716099068)*x[1]+(-1.40009044235)*x[2]
        arg[2,1,0,1]=(0.0667557890666)*x[0]+(0.742484448131)*x[1]+(-0.325246111346)*x[2]
        arg[2,1,1,0]=(-0.214965474071)*x[0]+(-0.215055151964)*x[1]+(1.15673960537)*x[2]
        arg[2,1,1,1]=(0.605764831789)*x[0]+(-1.0432964832)*x[1]+(0.645482523161)*x[2]
        arg[2,1,2,0]=(-1.13098364088)*x[0]+(-0.932044089041)*x[1]+(-0.308053828241)*x[2]
        arg[2,1,2,1]=(0.481961305459)*x[0]+(-0.101277472029)*x[1]+(-0.17520207974)*x[2]
        arg[2,2,0,0]=(0.579831852303)*x[0]+(0.538731990254)*x[1]+(-0.0897460374205)*x[2]
        arg[2,2,0,1]=(-1.8763181434)*x[0]+(-0.132710928569)*x[1]+(1.03169751651)*x[2]
        arg[2,2,1,0]=(0.56871511431)*x[0]+(0.075333481656)*x[1]+(1.10238818953)*x[2]
        arg[2,2,1,1]=(1.49347755246)*x[0]+(-1.00500941047)*x[1]+(-0.181733196359)*x[2]
        arg[2,2,2,0]=(0.532881509971)*x[0]+(-0.364333964659)*x[1]+(1.09633720883)*x[2]
        arg[2,2,2,1]=(-0.518419048615)*x[0]+(-0.48091100593)*x[1]+(0.724628840153)*x[2]
        arg[2,3,0,0]=(0.353475859331)*x[0]+(-0.0239879269825)*x[1]+(1.32068893718)*x[2]
        arg[2,3,0,1]=(0.553999122102)*x[0]+(-0.128530869239)*x[1]+(0.134663336287)*x[2]
        arg[2,3,1,0]=(-1.59775882569)*x[0]+(-1.07335424785)*x[1]+(1.39223587988)*x[2]
        arg[2,3,1,1]=(-0.921252080095)*x[0]+(1.46226299067)*x[1]+(-1.13412499932)*x[2]
        arg[2,3,2,0]=(0.417572910178)*x[0]+(0.0814889599489)*x[1]+(0.292159249719)*x[2]
        arg[2,3,2,1]=(1.44620228921)*x[0]+(0.767854577367)*x[1]+(-0.0113709806383)*x[2]
        arg[2,4,0,0]=(-0.156987001681)*x[0]+(-0.922524257833)*x[1]+(0.521128360838)*x[2]
        arg[2,4,0,1]=(0.154198133354)*x[0]+(0.462020370175)*x[1]+(0.966768653665)*x[2]
        arg[2,4,1,0]=(0.507836849242)*x[0]+(1.16701911244)*x[1]+(0.264543038211)*x[2]
        arg[2,4,1,1]=(0.452059436879)*x[0]+(0.409662432778)*x[1]+(-0.403800872595)*x[2]
        arg[2,4,2,0]=(0.521175521008)*x[0]+(-0.362551578816)*x[1]+(0.0922727331623)*x[2]
        arg[2,4,2,1]=(-1.0990361823)*x[0]+(1.73655146851)*x[1]+(-0.867917402922)*x[2]
        arg[3,0,0,0]=(-0.0527010882236)*x[0]+(-1.11528215488)*x[1]+(1.3141906393)*x[2]
        arg[3,0,0,1]=(0.151284271917)*x[0]+(-0.0159077450668)*x[1]+(1.1798338931)*x[2]
        arg[3,0,1,0]=(-1.7648963435)*x[0]+(0.229511437402)*x[1]+(0.924415900312)*x[2]
        arg[3,0,1,1]=(-0.498271821094)*x[0]+(1.58363063019)*x[1]+(1.64871750388)*x[2]
        arg[3,0,2,0]=(-1.11245170725)*x[0]+(1.27662612569)*x[1]+(0.145950372369)*x[2]
        arg[3,0,2,1]=(-0.691643924238)*x[0]+(-0.178211874279)*x[1]+(-0.341071050366)*x[2]
        arg[3,1,0,0]=(-0.929804889461)*x[0]+(0.199087067309)*x[1]+(1.00150168883)*x[2]
        arg[3,1,0,1]=(-1.25152265093)*x[0]+(0.517076294911)*x[1]+(0.103798312362)*x[2]
        arg[3,1,1,0]=(0.797316290623)*x[0]+(0.0239285379293)*x[1]+(-0.836519299418)*x[2]
        arg[3,1,1,1]=(-0.470084592176)*x[0]+(0.844364701728)*x[1]+(-0.027373275093)*x[2]
        arg[3,1,2,0]=(1.0340279373)*x[0]+(-0.0133951276315)*x[1]+(1.17710546194)*x[2]
        arg[3,1,2,1]=(0.0325929541496)*x[0]+(1.13784867184)*x[1]+(-0.0529194266769)*x[2]
        arg[3,2,0,0]=(1.92837561772)*x[0]+(-1.63633468831)*x[1]+(1.4212324281)*x[2]
        arg[3,2,0,1]=(1.09556054408)*x[0]+(1.43286059545)*x[1]+(1.20005604593)*x[2]
        arg[3,2,1,0]=(0.308887169169)*x[0]+(-0.149214224208)*x[1]+(-1.13559614877)*x[2]
        arg[3,2,1,1]=(-0.0648352909004)*x[0]+(-0.5645470596)*x[1]+(-0.525131730514)*x[2]
        arg[3,2,2,0]=(0.21826829974)*x[0]+(1.5950239996)*x[1]+(0.333592425438)*x[2]
        arg[3,2,2,1]=(-1.25613956172)*x[0]+(-0.242338679844)*x[1]+(-0.958875577074)*x[2]
        arg[3,3,0,0]=(1.65005362159)*x[0]+(-1.16713867165)*x[1]+(-0.282495293685)*x[2]
        arg[3,3,0,1]=(1.04854406427)*x[0]+(0.876270949531)*x[1]+(0.291036304743)*x[2]
        arg[3,3,1,0]=(-1.66619451175)*x[0]+(0.656275192484)*x[1]+(-1.27104499942)*x[2]
        arg[3,3,1,1]=(1.55407146907)*x[0]+(-0.322950943159)*x[1]+(-0.287565403896)*x[2]
        arg[3,3,2,0]=(-1.54760268583)*x[0]+(-1.45847761887)*x[1]+(1.13939567009)*x[2]
        arg[3,3,2,1]=(1.24632351699)*x[0]+(0.0265500081594)*x[1]+(1.61925095159)*x[2]
        arg[3,4,0,0]=(-0.0430889433549)*x[0]+(-1.46995559463)*x[1]+(-1.14915159823)*x[2]
        arg[3,4,0,1]=(0.0392641403409)*x[0]+(0.0603053049639)*x[1]+(0.647321169934)*x[2]
        arg[3,4,1,0]=(-0.156300985227)*x[0]+(0.79369911928)*x[1]+(-0.413124762507)*x[2]
        arg[3,4,1,1]=(0.258402059015)*x[0]+(0.00227307988585)*x[1]+(0.70067141295)*x[2]
        arg[3,4,2,0]=(-1.15131298872)*x[0]+(-0.627449043157)*x[1]+(-0.402268345861)*x[2]
        arg[3,4,2,1]=(0.455934238179)*x[0]+(-0.951120759108)*x[1]+(-0.280459367131)*x[2]
        ref[0,0,0,0]=(-0.791620170192)*x_ref[0]+(1.03871460301)*x_ref[1]+(-0.225035657687)*x_ref[2]
        ref[0,0,0,1]=(-0.464422319203)*x_ref[0]+(0.769454931053)*x_ref[1]+(-0.783260637771)*x_ref[2]
        ref[0,0,1,0]=(0.72655538215)*x_ref[0]+(-1.65736312015)*x_ref[1]+(1.08402505927)*x_ref[2]
        ref[0,0,1,1]=(-0.207576709573)*x_ref[0]+(-0.55250594761)*x_ref[1]+(-0.923702938288)*x_ref[2]
        ref[0,0,2,0]=(-0.701991085516)*x_ref[0]+(-1.76863643107)*x_ref[1]+(0.831382933104)*x_ref[2]
        ref[0,0,2,1]=(0.0640035527319)*x_ref[0]+(0.425446342976)*x_ref[1]+(1.64626766696)*x_ref[2]
        ref[0,1,0,0]=(1.01022963801)*x_ref[0]+(-0.0215015059716)*x_ref[1]+(0.849862425592)*x_ref[2]
        ref[0,1,0,1]=(-0.45776932943)*x_ref[0]+(0.7621342151)*x_ref[1]+(-1.40123018391)*x_ref[2]
        ref[0,1,1,0]=(0.737485054127)*x_ref[0]+(-0.147513905858)*x_ref[1]+(-0.616928595213)*x_ref[2]
        ref[0,1,1,1]=(-0.0608005391894)*x_ref[0]+(-0.154978210573)*x_ref[1]+(-1.40322011577)*x_ref[2]
        ref[0,1,2,0]=(1.59908177714)*x_ref[0]+(-0.378738573932)*x_ref[1]+(1.45285315247)*x_ref[2]
        ref[0,1,2,1]=(-0.622552157046)*x_ref[0]+(1.19996417516)*x_ref[1]+(0.728842900843)*x_ref[2]
        ref[0,2,0,0]=(-0.46942034691)*x_ref[0]+(0.333309721372)*x_ref[1]+(-0.280846553832)*x_ref[2]
        ref[0,2,0,1]=(1.22208788109)*x_ref[0]+(0.369339594285)*x_ref[1]+(1.31078425589)*x_ref[2]
        ref[0,2,1,0]=(0.0794165004114)*x_ref[0]+(0.44665451846)*x_ref[1]+(-0.814625708916)*x_ref[2]
        ref[0,2,1,1]=(0.594200807165)*x_ref[0]+(-0.00616720333193)*x_ref[1]+(0.382706175172)*x_ref[2]
        ref[0,2,2,0]=(0.0372230170459)*x_ref[0]+(0.440339923291)*x_ref[1]+(0.137641070028)*x_ref[2]
        ref[0,2,2,1]=(-1.29863078274)*x_ref[0]+(0.283142455171)*x_ref[1]+(0.0844540074659)*x_ref[2]
        ref[0,3,0,0]=(1.72811808036)*x_ref[0]+(0.536456781522)*x_ref[1]+(0.0266737704706)*x_ref[2]
        ref[0,3,0,1]=(0.148219555101)*x_ref[0]+(-0.6887500275)*x_ref[1]+(-0.405613136632)*x_ref[2]
        ref[0,3,1,0]=(0.708262537279)*x_ref[0]+(1.16319635773)*x_ref[1]+(-0.030808721674)*x_ref[2]
        ref[0,3,1,1]=(0.80834615863)*x_ref[0]+(-0.176789000925)*x_ref[1]+(-1.72576120963)*x_ref[2]
        ref[0,3,2,0]=(0.21128315299)*x_ref[0]+(0.770037046126)*x_ref[1]+(0.988028934422)*x_ref[2]
        ref[0,3,2,1]=(1.10522136317)*x_ref[0]+(1.6319351743)*x_ref[1]+(-0.784693713387)*x_ref[2]
        ref[0,4,0,0]=(0.121088121064)*x_ref[0]+(-1.79718067791)*x_ref[1]+(-0.907857173318)*x_ref[2]
        ref[0,4,0,1]=(-0.695887294217)*x_ref[0]+(-1.24166787807)*x_ref[1]+(-1.24968023856)*x_ref[2]
        ref[0,4,1,0]=(-0.202510797291)*x_ref[0]+(0.628310156105)*x_ref[1]+(0.605146447822)*x_ref[2]
        ref[0,4,1,1]=(0.149227458424)*x_ref[0]+(-1.5650392775)*x_ref[1]+(-0.763944892663)*x_ref[2]
        ref[0,4,2,0]=(-0.393218625872)*x_ref[0]+(0.479145914948)*x_ref[1]+(-0.313960447492)*x_ref[2]
        ref[0,4,2,1]=(-0.0191807443799)*x_ref[0]+(0.592992659119)*x_ref[1]+(-0.475633115484)*x_ref[2]
        ref[1,0,0,0]=(-0.964829354261)*x_ref[0]+(0.453014802906)*x_ref[1]+(-0.779502566938)*x_ref[2]
        ref[1,0,0,1]=(0.0151205739508)*x_ref[0]+(-0.153706429594)*x_ref[1]+(0.00866460475724)*x_ref[2]
        ref[1,0,1,0]=(0.92592276366)*x_ref[0]+(0.530787400525)*x_ref[1]+(-0.0577912606876)*x_ref[2]
        ref[1,0,1,1]=(0.948404467077)*x_ref[0]+(-0.265905470751)*x_ref[1]+(0.472349192005)*x_ref[2]
        ref[1,0,2,0]=(0.0176615383702)*x_ref[0]+(0.679605821761)*x_ref[1]+(-0.0333590911941)*x_ref[2]
        ref[1,0,2,1]=(-1.15594952629)*x_ref[0]+(0.264868376508)*x_ref[1]+(-0.883690535499)*x_ref[2]
        ref[1,1,0,0]=(-0.738712703994)*x_ref[0]+(0.545839232808)*x_ref[1]+(0.592474501134)*x_ref[2]
        ref[1,1,0,1]=(-1.3468960453)*x_ref[0]+(-0.00848202443614)*x_ref[1]+(-0.0124138286126)*x_ref[2]
        ref[1,1,1,0]=(0.900228409707)*x_ref[0]+(-1.86848082357)*x_ref[1]+(0.421539431142)*x_ref[2]
        ref[1,1,1,1]=(1.42754013963)*x_ref[0]+(-0.861376440449)*x_ref[1]+(0.0965520362291)*x_ref[2]
        ref[1,1,2,0]=(-0.35873674515)*x_ref[0]+(0.0862592617941)*x_ref[1]+(-0.536106207025)*x_ref[2]
        ref[1,1,2,1]=(0.90712910097)*x_ref[0]+(0.974545253055)*x_ref[1]+(0.570708510858)*x_ref[2]
        ref[1,2,0,0]=(1.19310775669)*x_ref[0]+(0.309414331679)*x_ref[1]+(-0.271298061586)*x_ref[2]
        ref[1,2,0,1]=(0.551778129545)*x_ref[0]+(0.539028430134)*x_ref[1]+(0.239406239262)*x_ref[2]
        ref[1,2,1,0]=(-0.810625054344)*x_ref[0]+(-0.241072190233)*x_ref[1]+(-1.58590369097)*x_ref[2]
        ref[1,2,1,1]=(0.529669075734)*x_ref[0]+(-0.890365024163)*x_ref[1]+(-1.69798509084)*x_ref[2]
        ref[1,2,2,0]=(0.0437087707448)*x_ref[0]+(0.88372190198)*x_ref[1]+(1.86547747806)*x_ref[2]
        ref[1,2,2,1]=(1.30739438898)*x_ref[0]+(0.315850112101)*x_ref[1]+(-0.262001133683)*x_ref[2]
        ref[1,3,0,0]=(-0.143426236837)*x_ref[0]+(-1.1703384826)*x_ref[1]+(0.50608348646)*x_ref[2]
        ref[1,3,0,1]=(-0.0444408776922)*x_ref[0]+(1.26952782036)*x_ref[1]+(1.67192602934)*x_ref[2]
        ref[1,3,1,0]=(0.672742824368)*x_ref[0]+(-0.785246676537)*x_ref[1]+(0.716130664056)*x_ref[2]
        ref[1,3,1,1]=(-0.295182508335)*x_ref[0]+(-1.53658860897)*x_ref[1]+(-1.87133995422)*x_ref[2]
        ref[1,3,2,0]=(-0.440859530081)*x_ref[0]+(0.0730120015821)*x_ref[1]+(-1.02465537353)*x_ref[2]
        ref[1,3,2,1]=(0.838320612003)*x_ref[0]+(1.81233688454)*x_ref[1]+(-1.0633876685)*x_ref[2]
        ref[1,4,0,0]=(0.52440453051)*x_ref[0]+(-0.0182588999022)*x_ref[1]+(1.05556310351)*x_ref[2]
        ref[1,4,0,1]=(0.766612727495)*x_ref[0]+(1.58987590746)*x_ref[1]+(-0.703801484569)*x_ref[2]
        ref[1,4,1,0]=(0.373439757535)*x_ref[0]+(-0.0676963704305)*x_ref[1]+(0.0948034094293)*x_ref[2]
        ref[1,4,1,1]=(-0.542283543375)*x_ref[0]+(-0.454408605761)*x_ref[1]+(-0.398904635316)*x_ref[2]
        ref[1,4,2,0]=(1.3374871143)*x_ref[0]+(0.0067849394587)*x_ref[1]+(-0.319452094882)*x_ref[2]
        ref[1,4,2,1]=(0.38530755393)*x_ref[0]+(1.05356462429)*x_ref[1]+(-0.302179447719)*x_ref[2]
        ref[2,0,0,0]=(-0.123109761583)*x_ref[0]+(0.69223848143)*x_ref[1]+(-1.36903487092)*x_ref[2]
        ref[2,0,0,1]=(0.121957328459)*x_ref[0]+(0.102229685387)*x_ref[1]+(0.962499974737)*x_ref[2]
        ref[2,0,1,0]=(0.478614188653)*x_ref[0]+(-1.23058817023)*x_ref[1]+(1.30950997478)*x_ref[2]
        ref[2,0,1,1]=(-0.529033562461)*x_ref[0]+(0.563030509591)*x_ref[1]+(-0.564939226654)*x_ref[2]
        ref[2,0,2,0]=(-0.377740282406)*x_ref[0]+(1.20532110343)*x_ref[1]+(1.55615174947)*x_ref[2]
        ref[2,0,2,1]=(0.539119710703)*x_ref[0]+(1.91771865857)*x_ref[1]+(0.0809882488048)*x_ref[2]
        ref[2,1,0,0]=(0.00548664737796)*x_ref[0]+(-0.157716099068)*x_ref[1]+(-1.40009044235)*x_ref[2]
        ref[2,1,0,1]=(0.0667557890666)*x_ref[0]+(0.742484448131)*x_ref[1]+(-0.325246111346)*x_ref[2]
        ref[2,1,1,0]=(-0.214965474071)*x_ref[0]+(-0.215055151964)*x_ref[1]+(1.15673960537)*x_ref[2]
        ref[2,1,1,1]=(0.605764831789)*x_ref[0]+(-1.0432964832)*x_ref[1]+(0.645482523161)*x_ref[2]
        ref[2,1,2,0]=(-1.13098364088)*x_ref[0]+(-0.932044089041)*x_ref[1]+(-0.308053828241)*x_ref[2]
        ref[2,1,2,1]=(0.481961305459)*x_ref[0]+(-0.101277472029)*x_ref[1]+(-0.17520207974)*x_ref[2]
        ref[2,2,0,0]=(0.579831852303)*x_ref[0]+(0.538731990254)*x_ref[1]+(-0.0897460374205)*x_ref[2]
        ref[2,2,0,1]=(-1.8763181434)*x_ref[0]+(-0.132710928569)*x_ref[1]+(1.03169751651)*x_ref[2]
        ref[2,2,1,0]=(0.56871511431)*x_ref[0]+(0.075333481656)*x_ref[1]+(1.10238818953)*x_ref[2]
        ref[2,2,1,1]=(1.49347755246)*x_ref[0]+(-1.00500941047)*x_ref[1]+(-0.181733196359)*x_ref[2]
        ref[2,2,2,0]=(0.532881509971)*x_ref[0]+(-0.364333964659)*x_ref[1]+(1.09633720883)*x_ref[2]
        ref[2,2,2,1]=(-0.518419048615)*x_ref[0]+(-0.48091100593)*x_ref[1]+(0.724628840153)*x_ref[2]
        ref[2,3,0,0]=(0.353475859331)*x_ref[0]+(-0.0239879269825)*x_ref[1]+(1.32068893718)*x_ref[2]
        ref[2,3,0,1]=(0.553999122102)*x_ref[0]+(-0.128530869239)*x_ref[1]+(0.134663336287)*x_ref[2]
        ref[2,3,1,0]=(-1.59775882569)*x_ref[0]+(-1.07335424785)*x_ref[1]+(1.39223587988)*x_ref[2]
        ref[2,3,1,1]=(-0.921252080095)*x_ref[0]+(1.46226299067)*x_ref[1]+(-1.13412499932)*x_ref[2]
        ref[2,3,2,0]=(0.417572910178)*x_ref[0]+(0.0814889599489)*x_ref[1]+(0.292159249719)*x_ref[2]
        ref[2,3,2,1]=(1.44620228921)*x_ref[0]+(0.767854577367)*x_ref[1]+(-0.0113709806383)*x_ref[2]
        ref[2,4,0,0]=(-0.156987001681)*x_ref[0]+(-0.922524257833)*x_ref[1]+(0.521128360838)*x_ref[2]
        ref[2,4,0,1]=(0.154198133354)*x_ref[0]+(0.462020370175)*x_ref[1]+(0.966768653665)*x_ref[2]
        ref[2,4,1,0]=(0.507836849242)*x_ref[0]+(1.16701911244)*x_ref[1]+(0.264543038211)*x_ref[2]
        ref[2,4,1,1]=(0.452059436879)*x_ref[0]+(0.409662432778)*x_ref[1]+(-0.403800872595)*x_ref[2]
        ref[2,4,2,0]=(0.521175521008)*x_ref[0]+(-0.362551578816)*x_ref[1]+(0.0922727331623)*x_ref[2]
        ref[2,4,2,1]=(-1.0990361823)*x_ref[0]+(1.73655146851)*x_ref[1]+(-0.867917402922)*x_ref[2]
        ref[3,0,0,0]=(-0.0527010882236)*x_ref[0]+(-1.11528215488)*x_ref[1]+(1.3141906393)*x_ref[2]
        ref[3,0,0,1]=(0.151284271917)*x_ref[0]+(-0.0159077450668)*x_ref[1]+(1.1798338931)*x_ref[2]
        ref[3,0,1,0]=(-1.7648963435)*x_ref[0]+(0.229511437402)*x_ref[1]+(0.924415900312)*x_ref[2]
        ref[3,0,1,1]=(-0.498271821094)*x_ref[0]+(1.58363063019)*x_ref[1]+(1.64871750388)*x_ref[2]
        ref[3,0,2,0]=(-1.11245170725)*x_ref[0]+(1.27662612569)*x_ref[1]+(0.145950372369)*x_ref[2]
        ref[3,0,2,1]=(-0.691643924238)*x_ref[0]+(-0.178211874279)*x_ref[1]+(-0.341071050366)*x_ref[2]
        ref[3,1,0,0]=(-0.929804889461)*x_ref[0]+(0.199087067309)*x_ref[1]+(1.00150168883)*x_ref[2]
        ref[3,1,0,1]=(-1.25152265093)*x_ref[0]+(0.517076294911)*x_ref[1]+(0.103798312362)*x_ref[2]
        ref[3,1,1,0]=(0.797316290623)*x_ref[0]+(0.0239285379293)*x_ref[1]+(-0.836519299418)*x_ref[2]
        ref[3,1,1,1]=(-0.470084592176)*x_ref[0]+(0.844364701728)*x_ref[1]+(-0.027373275093)*x_ref[2]
        ref[3,1,2,0]=(1.0340279373)*x_ref[0]+(-0.0133951276315)*x_ref[1]+(1.17710546194)*x_ref[2]
        ref[3,1,2,1]=(0.0325929541496)*x_ref[0]+(1.13784867184)*x_ref[1]+(-0.0529194266769)*x_ref[2]
        ref[3,2,0,0]=(1.92837561772)*x_ref[0]+(-1.63633468831)*x_ref[1]+(1.4212324281)*x_ref[2]
        ref[3,2,0,1]=(1.09556054408)*x_ref[0]+(1.43286059545)*x_ref[1]+(1.20005604593)*x_ref[2]
        ref[3,2,1,0]=(0.308887169169)*x_ref[0]+(-0.149214224208)*x_ref[1]+(-1.13559614877)*x_ref[2]
        ref[3,2,1,1]=(-0.0648352909004)*x_ref[0]+(-0.5645470596)*x_ref[1]+(-0.525131730514)*x_ref[2]
        ref[3,2,2,0]=(0.21826829974)*x_ref[0]+(1.5950239996)*x_ref[1]+(0.333592425438)*x_ref[2]
        ref[3,2,2,1]=(-1.25613956172)*x_ref[0]+(-0.242338679844)*x_ref[1]+(-0.958875577074)*x_ref[2]
        ref[3,3,0,0]=(1.65005362159)*x_ref[0]+(-1.16713867165)*x_ref[1]+(-0.282495293685)*x_ref[2]
        ref[3,3,0,1]=(1.04854406427)*x_ref[0]+(0.876270949531)*x_ref[1]+(0.291036304743)*x_ref[2]
        ref[3,3,1,0]=(-1.66619451175)*x_ref[0]+(0.656275192484)*x_ref[1]+(-1.27104499942)*x_ref[2]
        ref[3,3,1,1]=(1.55407146907)*x_ref[0]+(-0.322950943159)*x_ref[1]+(-0.287565403896)*x_ref[2]
        ref[3,3,2,0]=(-1.54760268583)*x_ref[0]+(-1.45847761887)*x_ref[1]+(1.13939567009)*x_ref[2]
        ref[3,3,2,1]=(1.24632351699)*x_ref[0]+(0.0265500081594)*x_ref[1]+(1.61925095159)*x_ref[2]
        ref[3,4,0,0]=(-0.0430889433549)*x_ref[0]+(-1.46995559463)*x_ref[1]+(-1.14915159823)*x_ref[2]
        ref[3,4,0,1]=(0.0392641403409)*x_ref[0]+(0.0603053049639)*x_ref[1]+(0.647321169934)*x_ref[2]
        ref[3,4,1,0]=(-0.156300985227)*x_ref[0]+(0.79369911928)*x_ref[1]+(-0.413124762507)*x_ref[2]
        ref[3,4,1,1]=(0.258402059015)*x_ref[0]+(0.00227307988585)*x_ref[1]+(0.70067141295)*x_ref[2]
        ref[3,4,2,0]=(-1.15131298872)*x_ref[0]+(-0.627449043157)*x_ref[1]+(-0.402268345861)*x_ref[2]
        ref[3,4,2,1]=(0.455934238179)*x_ref[0]+(-0.951120759108)*x_ref[1]+(-0.280459367131)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_ReducedContinuousFunction_rank0(self):
      """
      tests interpolate for rank 0 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedContinuousFunction onto ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(),w)
      ref=Data(0,(),w_ref)
      if dim==2:
        arg=(-0.787646538574)*x[0]+(0.912422904218)*x[1]
        ref=(-0.787646538574)*x_ref[0]+(0.912422904218)*x_ref[1]
      else:
        arg=(0.934044939024)*x[0]+(0.628477728603)*x[1]+(-0.786339405837)*x[2]
        ref=(0.934044939024)*x_ref[0]+(0.628477728603)*x_ref[1]+(-0.786339405837)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_ReducedContinuousFunction_rank1(self):
      """
      tests interpolate for rank 1 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedContinuousFunction onto ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(2,),w)
      ref=Data(0,(2,),w_ref)
      if dim==2:
        arg[0]=(-0.588326094839)*x[0]+(1.56296941343)*x[1]
        arg[1]=(-0.802191222185)*x[0]+(-1.13471016062)*x[1]
        ref[0]=(-0.588326094839)*x[0]+(1.56296941343)*x[1]
        ref[1]=(-0.802191222185)*x[0]+(-1.13471016062)*x[1]
      else:
        arg[0]=(0.160702771321)*x[0]+(-0.0784425374814)*x[1]+(1.65226639722)*x[2]
        arg[1]=(0.745050820919)*x[0]+(-0.183859360073)*x[1]+(0.399813139488)*x[2]
        ref[0]=(0.160702771321)*x[0]+(-0.0784425374814)*x[1]+(1.65226639722)*x[2]
        ref[1]=(0.745050820919)*x[0]+(-0.183859360073)*x[1]+(0.399813139488)*x[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_ReducedContinuousFunction_rank2(self):
      """
      tests interpolate for rank 2 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedContinuousFunction onto ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5),w)
      ref=Data(0,(4, 5),w_ref)
      if dim==2:
        arg[0,0]=(-0.797066914479)*x[0]+(-0.0269853164939)*x[1]
        arg[0,1]=(-1.07443661798)*x[0]+(0.818510216043)*x[1]
        arg[0,2]=(0.398735262179)*x[0]+(-0.580387872083)*x[1]
        arg[0,3]=(-0.003185613959)*x[0]+(-0.670506330615)*x[1]
        arg[0,4]=(0.385009570186)*x[0]+(0.858570512936)*x[1]
        arg[1,0]=(1.60917176753)*x[0]+(-0.0866362449992)*x[1]
        arg[1,1]=(-0.883782122452)*x[0]+(-0.402187705578)*x[1]
        arg[1,2]=(0.153893095122)*x[0]+(-1.19636362644)*x[1]
        arg[1,3]=(1.92041961031)*x[0]+(0.325857877636)*x[1]
        arg[1,4]=(-0.0947698843729)*x[0]+(-0.0663785758627)*x[1]
        arg[2,0]=(0.0232949467051)*x[0]+(1.60134210326)*x[1]
        arg[2,1]=(-0.521182882924)*x[0]+(-0.507401571343)*x[1]
        arg[2,2]=(0.833797117616)*x[0]+(-0.268886477194)*x[1]
        arg[2,3]=(-0.867762611227)*x[0]+(1.03727088566)*x[1]
        arg[2,4]=(0.380510651918)*x[0]+(0.350246888456)*x[1]
        arg[3,0]=(-1.10155500839)*x[0]+(0.414683749118)*x[1]
        arg[3,1]=(-0.0306733233552)*x[0]+(-0.351251014237)*x[1]
        arg[3,2]=(-0.0708773001498)*x[0]+(-0.326636587948)*x[1]
        arg[3,3]=(-1.32553119895)*x[0]+(-1.5852157477)*x[1]
        arg[3,4]=(1.40177154802)*x[0]+(-1.24804787994)*x[1]
        ref[0,0]=(-0.797066914479)*x_ref[0]+(-0.0269853164939)*x_ref[1]
        ref[0,1]=(-1.07443661798)*x_ref[0]+(0.818510216043)*x_ref[1]
        ref[0,2]=(0.398735262179)*x_ref[0]+(-0.580387872083)*x_ref[1]
        ref[0,3]=(-0.003185613959)*x_ref[0]+(-0.670506330615)*x_ref[1]
        ref[0,4]=(0.385009570186)*x_ref[0]+(0.858570512936)*x_ref[1]
        ref[1,0]=(1.60917176753)*x_ref[0]+(-0.0866362449992)*x_ref[1]
        ref[1,1]=(-0.883782122452)*x_ref[0]+(-0.402187705578)*x_ref[1]
        ref[1,2]=(0.153893095122)*x_ref[0]+(-1.19636362644)*x_ref[1]
        ref[1,3]=(1.92041961031)*x_ref[0]+(0.325857877636)*x_ref[1]
        ref[1,4]=(-0.0947698843729)*x_ref[0]+(-0.0663785758627)*x_ref[1]
        ref[2,0]=(0.0232949467051)*x_ref[0]+(1.60134210326)*x_ref[1]
        ref[2,1]=(-0.521182882924)*x_ref[0]+(-0.507401571343)*x_ref[1]
        ref[2,2]=(0.833797117616)*x_ref[0]+(-0.268886477194)*x_ref[1]
        ref[2,3]=(-0.867762611227)*x_ref[0]+(1.03727088566)*x_ref[1]
        ref[2,4]=(0.380510651918)*x_ref[0]+(0.350246888456)*x_ref[1]
        ref[3,0]=(-1.10155500839)*x_ref[0]+(0.414683749118)*x_ref[1]
        ref[3,1]=(-0.0306733233552)*x_ref[0]+(-0.351251014237)*x_ref[1]
        ref[3,2]=(-0.0708773001498)*x_ref[0]+(-0.326636587948)*x_ref[1]
        ref[3,3]=(-1.32553119895)*x_ref[0]+(-1.5852157477)*x_ref[1]
        ref[3,4]=(1.40177154802)*x_ref[0]+(-1.24804787994)*x_ref[1]
      else:
        arg[0,0]=(0.000362194678523)*x[0]+(-1.63949220671)*x[1]+(-0.176997156562)*x[2]
        arg[0,1]=(-0.357589049408)*x[0]+(0.828191515325)*x[1]+(-0.269706780136)*x[2]
        arg[0,2]=(0.886395070544)*x[0]+(0.815688451839)*x[1]+(-0.658412270558)*x[2]
        arg[0,3]=(-0.222573318026)*x[0]+(0.137763803092)*x[1]+(0.171902150187)*x[2]
        arg[0,4]=(1.05159741251)*x[0]+(0.923836841038)*x[1]+(-0.426590801731)*x[2]
        arg[1,0]=(-0.00718419057788)*x[0]+(1.55809964998)*x[1]+(-0.912766657496)*x[2]
        arg[1,1]=(1.30769045785)*x[0]+(-0.0356609338389)*x[1]+(0.214543084134)*x[2]
        arg[1,2]=(-1.56763014241)*x[0]+(-0.582970995722)*x[1]+(-0.513972924752)*x[2]
        arg[1,3]=(0.118027736928)*x[0]+(-0.28795014876)*x[1]+(0.620718623034)*x[2]
        arg[1,4]=(-0.784496170193)*x[0]+(0.233037866924)*x[1]+(-0.0500718134572)*x[2]
        arg[2,0]=(0.800851281806)*x[0]+(0.219346484846)*x[1]+(1.19125787701)*x[2]
        arg[2,1]=(-1.45025879429)*x[0]+(-1.32011538154)*x[1]+(0.47946669308)*x[2]
        arg[2,2]=(-0.231377110469)*x[0]+(-0.921551695999)*x[1]+(-0.298964635841)*x[2]
        arg[2,3]=(0.965766335546)*x[0]+(0.424791610089)*x[1]+(1.01860652092)*x[2]
        arg[2,4]=(0.676247093636)*x[0]+(-1.50802741511)*x[1]+(0.0385960918391)*x[2]
        arg[3,0]=(0.322378151756)*x[0]+(-0.483633005509)*x[1]+(-0.852371707865)*x[2]
        arg[3,1]=(-0.241998307153)*x[0]+(-1.08134340082)*x[1]+(-0.849574532537)*x[2]
        arg[3,2]=(-0.0185751405467)*x[0]+(0.179558416001)*x[1]+(-0.488727308548)*x[2]
        arg[3,3]=(0.0984555057845)*x[0]+(1.03085967788)*x[1]+(-0.357009094371)*x[2]
        arg[3,4]=(0.430593980121)*x[0]+(0.689565614489)*x[1]+(-0.0916541808491)*x[2]
        ref[0,0]=(0.000362194678523)*x_ref[0]+(-1.63949220671)*x_ref[1]+(-0.176997156562)*x_ref[2]
        ref[0,1]=(-0.357589049408)*x_ref[0]+(0.828191515325)*x_ref[1]+(-0.269706780136)*x_ref[2]
        ref[0,2]=(0.886395070544)*x_ref[0]+(0.815688451839)*x_ref[1]+(-0.658412270558)*x_ref[2]
        ref[0,3]=(-0.222573318026)*x_ref[0]+(0.137763803092)*x_ref[1]+(0.171902150187)*x_ref[2]
        ref[0,4]=(1.05159741251)*x_ref[0]+(0.923836841038)*x_ref[1]+(-0.426590801731)*x_ref[2]
        ref[1,0]=(-0.00718419057788)*x_ref[0]+(1.55809964998)*x_ref[1]+(-0.912766657496)*x_ref[2]
        ref[1,1]=(1.30769045785)*x_ref[0]+(-0.0356609338389)*x_ref[1]+(0.214543084134)*x_ref[2]
        ref[1,2]=(-1.56763014241)*x_ref[0]+(-0.582970995722)*x_ref[1]+(-0.513972924752)*x_ref[2]
        ref[1,3]=(0.118027736928)*x_ref[0]+(-0.28795014876)*x_ref[1]+(0.620718623034)*x_ref[2]
        ref[1,4]=(-0.784496170193)*x_ref[0]+(0.233037866924)*x_ref[1]+(-0.0500718134572)*x_ref[2]
        ref[2,0]=(0.800851281806)*x_ref[0]+(0.219346484846)*x_ref[1]+(1.19125787701)*x_ref[2]
        ref[2,1]=(-1.45025879429)*x_ref[0]+(-1.32011538154)*x_ref[1]+(0.47946669308)*x_ref[2]
        ref[2,2]=(-0.231377110469)*x_ref[0]+(-0.921551695999)*x_ref[1]+(-0.298964635841)*x_ref[2]
        ref[2,3]=(0.965766335546)*x_ref[0]+(0.424791610089)*x_ref[1]+(1.01860652092)*x_ref[2]
        ref[2,4]=(0.676247093636)*x_ref[0]+(-1.50802741511)*x_ref[1]+(0.0385960918391)*x_ref[2]
        ref[3,0]=(0.322378151756)*x_ref[0]+(-0.483633005509)*x_ref[1]+(-0.852371707865)*x_ref[2]
        ref[3,1]=(-0.241998307153)*x_ref[0]+(-1.08134340082)*x_ref[1]+(-0.849574532537)*x_ref[2]
        ref[3,2]=(-0.0185751405467)*x_ref[0]+(0.179558416001)*x_ref[1]+(-0.488727308548)*x_ref[2]
        ref[3,3]=(0.0984555057845)*x_ref[0]+(1.03085967788)*x_ref[1]+(-0.357009094371)*x_ref[2]
        ref[3,4]=(0.430593980121)*x_ref[0]+(0.689565614489)*x_ref[1]+(-0.0916541808491)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_ReducedContinuousFunction_rank3(self):
      """
      tests interpolate for rank 3 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedContinuousFunction onto ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2),w_ref)
      if dim==2:
        arg[0,0,0]=(-0.0641544023301)*x[0]+(0.364286357763)*x[1]
        arg[0,0,1]=(-0.858288240411)*x[0]+(-0.847764393559)*x[1]
        arg[0,1,0]=(-0.656948775029)*x[0]+(1.69748608992)*x[1]
        arg[0,1,1]=(0.579399191549)*x[0]+(1.48407368783)*x[1]
        arg[1,0,0]=(-0.35948657403)*x[0]+(0.795436090765)*x[1]
        arg[1,0,1]=(-1.32807156246)*x[0]+(0.921960610294)*x[1]
        arg[1,1,0]=(1.0904085219)*x[0]+(-1.41870997296)*x[1]
        arg[1,1,1]=(-0.677691596793)*x[0]+(-0.395373706333)*x[1]
        arg[2,0,0]=(0.295591917216)*x[0]+(-1.75550830457)*x[1]
        arg[2,0,1]=(0.506665326626)*x[0]+(0.273265713965)*x[1]
        arg[2,1,0]=(0.543899865352)*x[0]+(-0.0678457091657)*x[1]
        arg[2,1,1]=(-0.887824769535)*x[0]+(0.527475538055)*x[1]
        arg[3,0,0]=(-0.788511557365)*x[0]+(-0.635972100675)*x[1]
        arg[3,0,1]=(-1.02799066459)*x[0]+(-0.416241247725)*x[1]
        arg[3,1,0]=(-0.310072483091)*x[0]+(1.2445160445)*x[1]
        arg[3,1,1]=(0.912462196567)*x[0]+(0.631855269783)*x[1]
        arg[4,0,0]=(0.700426861807)*x[0]+(-0.491365913713)*x[1]
        arg[4,0,1]=(0.628791538338)*x[0]+(0.640767645586)*x[1]
        arg[4,1,0]=(0.370886720329)*x[0]+(0.823663448834)*x[1]
        arg[4,1,1]=(1.75857310774)*x[0]+(0.445439424641)*x[1]
        arg[5,0,0]=(1.39637385066)*x[0]+(-0.812211860169)*x[1]
        arg[5,0,1]=(0.865050350195)*x[0]+(-1.25983380375)*x[1]
        arg[5,1,0]=(-0.0429057645445)*x[0]+(-1.08490578934)*x[1]
        arg[5,1,1]=(1.42851264594)*x[0]+(1.52192971749)*x[1]
        ref[0,0,0]=(-0.0641544023301)*x_ref[0]+(0.364286357763)*x_ref[1]
        ref[0,0,1]=(-0.858288240411)*x_ref[0]+(-0.847764393559)*x_ref[1]
        ref[0,1,0]=(-0.656948775029)*x_ref[0]+(1.69748608992)*x_ref[1]
        ref[0,1,1]=(0.579399191549)*x_ref[0]+(1.48407368783)*x_ref[1]
        ref[1,0,0]=(-0.35948657403)*x_ref[0]+(0.795436090765)*x_ref[1]
        ref[1,0,1]=(-1.32807156246)*x_ref[0]+(0.921960610294)*x_ref[1]
        ref[1,1,0]=(1.0904085219)*x_ref[0]+(-1.41870997296)*x_ref[1]
        ref[1,1,1]=(-0.677691596793)*x_ref[0]+(-0.395373706333)*x_ref[1]
        ref[2,0,0]=(0.295591917216)*x_ref[0]+(-1.75550830457)*x_ref[1]
        ref[2,0,1]=(0.506665326626)*x_ref[0]+(0.273265713965)*x_ref[1]
        ref[2,1,0]=(0.543899865352)*x_ref[0]+(-0.0678457091657)*x_ref[1]
        ref[2,1,1]=(-0.887824769535)*x_ref[0]+(0.527475538055)*x_ref[1]
        ref[3,0,0]=(-0.788511557365)*x_ref[0]+(-0.635972100675)*x_ref[1]
        ref[3,0,1]=(-1.02799066459)*x_ref[0]+(-0.416241247725)*x_ref[1]
        ref[3,1,0]=(-0.310072483091)*x_ref[0]+(1.2445160445)*x_ref[1]
        ref[3,1,1]=(0.912462196567)*x_ref[0]+(0.631855269783)*x_ref[1]
        ref[4,0,0]=(0.700426861807)*x_ref[0]+(-0.491365913713)*x_ref[1]
        ref[4,0,1]=(0.628791538338)*x_ref[0]+(0.640767645586)*x_ref[1]
        ref[4,1,0]=(0.370886720329)*x_ref[0]+(0.823663448834)*x_ref[1]
        ref[4,1,1]=(1.75857310774)*x_ref[0]+(0.445439424641)*x_ref[1]
        ref[5,0,0]=(1.39637385066)*x_ref[0]+(-0.812211860169)*x_ref[1]
        ref[5,0,1]=(0.865050350195)*x_ref[0]+(-1.25983380375)*x_ref[1]
        ref[5,1,0]=(-0.0429057645445)*x_ref[0]+(-1.08490578934)*x_ref[1]
        ref[5,1,1]=(1.42851264594)*x_ref[0]+(1.52192971749)*x_ref[1]
      else:
        arg[0,0,0]=(0.731194732468)*x[0]+(-0.0822115660022)*x[1]+(-0.403175091289)*x[2]
        arg[0,0,1]=(0.39292230307)*x[0]+(1.48598681615)*x[1]+(1.05545374107)*x[2]
        arg[0,1,0]=(0.306844260706)*x[0]+(0.836496026763)*x[1]+(-0.121120983155)*x[2]
        arg[0,1,1]=(-0.155245802062)*x[0]+(0.800232934864)*x[1]+(0.373397160636)*x[2]
        arg[1,0,0]=(0.155941790359)*x[0]+(0.826821969281)*x[1]+(0.849596807425)*x[2]
        arg[1,0,1]=(-0.0988854656187)*x[0]+(-0.855032499842)*x[1]+(0.141988288566)*x[2]
        arg[1,1,0]=(-0.761032539296)*x[0]+(-0.0892979073234)*x[1]+(1.93034385132)*x[2]
        arg[1,1,1]=(-1.08303603558)*x[0]+(-0.487250117659)*x[1]+(0.0358501036173)*x[2]
        arg[2,0,0]=(0.148391465413)*x[0]+(-0.731963025625)*x[1]+(0.484236965689)*x[2]
        arg[2,0,1]=(1.06691217722)*x[0]+(-0.638095266244)*x[1]+(-0.971838410779)*x[2]
        arg[2,1,0]=(-0.377750829061)*x[0]+(-0.101467967263)*x[1]+(-1.54241387349)*x[2]
        arg[2,1,1]=(0.62210350231)*x[0]+(0.0435334873911)*x[1]+(-0.299303646623)*x[2]
        arg[3,0,0]=(0.523811147108)*x[0]+(-1.09709839108)*x[1]+(1.83592280935)*x[2]
        arg[3,0,1]=(0.957738143475)*x[0]+(0.282158440867)*x[1]+(0.621694180522)*x[2]
        arg[3,1,0]=(0.366557891822)*x[0]+(0.812194047631)*x[1]+(-0.245067876349)*x[2]
        arg[3,1,1]=(0.604859464982)*x[0]+(1.08415514854)*x[1]+(-1.470128848)*x[2]
        arg[4,0,0]=(-0.13031502499)*x[0]+(-1.20707238733)*x[1]+(0.108382497552)*x[2]
        arg[4,0,1]=(1.30453246317)*x[0]+(1.59696612588)*x[1]+(-0.234259043697)*x[2]
        arg[4,1,0]=(-0.233494640836)*x[0]+(1.10701061816)*x[1]+(-0.409927013674)*x[2]
        arg[4,1,1]=(0.102708240266)*x[0]+(-0.381598345002)*x[1]+(-1.39077748656)*x[2]
        arg[5,0,0]=(1.06217404493)*x[0]+(1.1600791444)*x[1]+(-1.45242735389)*x[2]
        arg[5,0,1]=(-0.423024189506)*x[0]+(-0.0868948651641)*x[1]+(0.0894539043522)*x[2]
        arg[5,1,0]=(1.17650613067)*x[0]+(0.993832919965)*x[1]+(1.16546768569)*x[2]
        arg[5,1,1]=(-0.61549132897)*x[0]+(-0.499619662734)*x[1]+(-0.643422281669)*x[2]
        ref[0,0,0]=(0.731194732468)*x_ref[0]+(-0.0822115660022)*x_ref[1]+(-0.403175091289)*x_ref[2]
        ref[0,0,1]=(0.39292230307)*x_ref[0]+(1.48598681615)*x_ref[1]+(1.05545374107)*x_ref[2]
        ref[0,1,0]=(0.306844260706)*x_ref[0]+(0.836496026763)*x_ref[1]+(-0.121120983155)*x_ref[2]
        ref[0,1,1]=(-0.155245802062)*x_ref[0]+(0.800232934864)*x_ref[1]+(0.373397160636)*x_ref[2]
        ref[1,0,0]=(0.155941790359)*x_ref[0]+(0.826821969281)*x_ref[1]+(0.849596807425)*x_ref[2]
        ref[1,0,1]=(-0.0988854656187)*x_ref[0]+(-0.855032499842)*x_ref[1]+(0.141988288566)*x_ref[2]
        ref[1,1,0]=(-0.761032539296)*x_ref[0]+(-0.0892979073234)*x_ref[1]+(1.93034385132)*x_ref[2]
        ref[1,1,1]=(-1.08303603558)*x_ref[0]+(-0.487250117659)*x_ref[1]+(0.0358501036173)*x_ref[2]
        ref[2,0,0]=(0.148391465413)*x_ref[0]+(-0.731963025625)*x_ref[1]+(0.484236965689)*x_ref[2]
        ref[2,0,1]=(1.06691217722)*x_ref[0]+(-0.638095266244)*x_ref[1]+(-0.971838410779)*x_ref[2]
        ref[2,1,0]=(-0.377750829061)*x_ref[0]+(-0.101467967263)*x_ref[1]+(-1.54241387349)*x_ref[2]
        ref[2,1,1]=(0.62210350231)*x_ref[0]+(0.0435334873911)*x_ref[1]+(-0.299303646623)*x_ref[2]
        ref[3,0,0]=(0.523811147108)*x_ref[0]+(-1.09709839108)*x_ref[1]+(1.83592280935)*x_ref[2]
        ref[3,0,1]=(0.957738143475)*x_ref[0]+(0.282158440867)*x_ref[1]+(0.621694180522)*x_ref[2]
        ref[3,1,0]=(0.366557891822)*x_ref[0]+(0.812194047631)*x_ref[1]+(-0.245067876349)*x_ref[2]
        ref[3,1,1]=(0.604859464982)*x_ref[0]+(1.08415514854)*x_ref[1]+(-1.470128848)*x_ref[2]
        ref[4,0,0]=(-0.13031502499)*x_ref[0]+(-1.20707238733)*x_ref[1]+(0.108382497552)*x_ref[2]
        ref[4,0,1]=(1.30453246317)*x_ref[0]+(1.59696612588)*x_ref[1]+(-0.234259043697)*x_ref[2]
        ref[4,1,0]=(-0.233494640836)*x_ref[0]+(1.10701061816)*x_ref[1]+(-0.409927013674)*x_ref[2]
        ref[4,1,1]=(0.102708240266)*x_ref[0]+(-0.381598345002)*x_ref[1]+(-1.39077748656)*x_ref[2]
        ref[5,0,0]=(1.06217404493)*x_ref[0]+(1.1600791444)*x_ref[1]+(-1.45242735389)*x_ref[2]
        ref[5,0,1]=(-0.423024189506)*x_ref[0]+(-0.0868948651641)*x_ref[1]+(0.0894539043522)*x_ref[2]
        ref[5,1,0]=(1.17650613067)*x_ref[0]+(0.993832919965)*x_ref[1]+(1.16546768569)*x_ref[2]
        ref[5,1,1]=(-0.61549132897)*x_ref[0]+(-0.499619662734)*x_ref[1]+(-0.643422281669)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_ReducedContinuousFunction_rank4(self):
      """
      tests interpolate for rank 4 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedContinuousFunction onto ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5, 3, 2),w)
      ref=Data(0,(4, 5, 3, 2),w_ref)
      if dim==2:
        arg[0,0,0,0]=(0.227969627252)*x[0]+(0.778177350248)*x[1]
        arg[0,0,0,1]=(-0.684883711494)*x[0]+(-0.574285957132)*x[1]
        arg[0,0,1,0]=(0.483090608974)*x[0]+(0.0779929087959)*x[1]
        arg[0,0,1,1]=(-0.293150814932)*x[0]+(1.35368931993)*x[1]
        arg[0,0,2,0]=(-0.190664734662)*x[0]+(0.910883386754)*x[1]
        arg[0,0,2,1]=(-1.51255575457)*x[0]+(0.421560181501)*x[1]
        arg[0,1,0,0]=(-0.833402828032)*x[0]+(-0.400466806781)*x[1]
        arg[0,1,0,1]=(-0.212099323605)*x[0]+(-0.112478697221)*x[1]
        arg[0,1,1,0]=(0.57687662926)*x[0]+(0.869758954551)*x[1]
        arg[0,1,1,1]=(1.37261769917)*x[0]+(1.37855789907)*x[1]
        arg[0,1,2,0]=(-1.02838589722)*x[0]+(-0.280409246204)*x[1]
        arg[0,1,2,1]=(1.2333373468)*x[0]+(0.337631183798)*x[1]
        arg[0,2,0,0]=(-0.249065448772)*x[0]+(0.834483646317)*x[1]
        arg[0,2,0,1]=(-1.30860297683)*x[0]+(-0.653929967017)*x[1]
        arg[0,2,1,0]=(-0.0155994484453)*x[0]+(-0.274023166937)*x[1]
        arg[0,2,1,1]=(0.31458360335)*x[0]+(1.59507700673)*x[1]
        arg[0,2,2,0]=(-0.11442619553)*x[0]+(1.6577028576)*x[1]
        arg[0,2,2,1]=(0.274261132032)*x[0]+(1.15236962479)*x[1]
        arg[0,3,0,0]=(1.91433860239)*x[0]+(-0.316349747811)*x[1]
        arg[0,3,0,1]=(-1.42457591054)*x[0]+(-0.362812690805)*x[1]
        arg[0,3,1,0]=(0.572734883096)*x[0]+(0.424724787283)*x[1]
        arg[0,3,1,1]=(-0.118538376813)*x[0]+(0.25190312527)*x[1]
        arg[0,3,2,0]=(-0.533409510081)*x[0]+(-0.804349107915)*x[1]
        arg[0,3,2,1]=(-0.290555363666)*x[0]+(-0.912641477145)*x[1]
        arg[0,4,0,0]=(-0.469612775918)*x[0]+(-0.0447411310793)*x[1]
        arg[0,4,0,1]=(0.108538258831)*x[0]+(0.0453962016662)*x[1]
        arg[0,4,1,0]=(-0.130785944028)*x[0]+(0.0887280914032)*x[1]
        arg[0,4,1,1]=(0.166412309585)*x[0]+(-0.75130797706)*x[1]
        arg[0,4,2,0]=(1.22538232455)*x[0]+(0.112925588343)*x[1]
        arg[0,4,2,1]=(-1.02455171854)*x[0]+(-0.725754942749)*x[1]
        arg[1,0,0,0]=(-0.869171086952)*x[0]+(-0.948876783682)*x[1]
        arg[1,0,0,1]=(0.17730673689)*x[0]+(-0.00166180855579)*x[1]
        arg[1,0,1,0]=(0.482072704433)*x[0]+(0.895847096783)*x[1]
        arg[1,0,1,1]=(0.695736834617)*x[0]+(0.358082704558)*x[1]
        arg[1,0,2,0]=(-0.501799636565)*x[0]+(0.363612558025)*x[1]
        arg[1,0,2,1]=(-1.47204447226)*x[0]+(1.91281426121)*x[1]
        arg[1,1,0,0]=(0.241893182978)*x[0]+(0.95199758669)*x[1]
        arg[1,1,0,1]=(-0.68112361827)*x[0]+(-0.241935141301)*x[1]
        arg[1,1,1,0]=(-0.553030972306)*x[0]+(0.0244503690439)*x[1]
        arg[1,1,1,1]=(0.278177176896)*x[0]+(1.58693846427)*x[1]
        arg[1,1,2,0]=(-0.87980015517)*x[0]+(0.315618837123)*x[1]
        arg[1,1,2,1]=(-0.335783961379)*x[0]+(0.368650136138)*x[1]
        arg[1,2,0,0]=(-0.903535146802)*x[0]+(1.68182923194)*x[1]
        arg[1,2,0,1]=(1.06511511382)*x[0]+(-1.08524762302)*x[1]
        arg[1,2,1,0]=(-0.0455274784681)*x[0]+(-0.519576710661)*x[1]
        arg[1,2,1,1]=(0.735100217062)*x[0]+(-0.986268682671)*x[1]
        arg[1,2,2,0]=(-0.544610797462)*x[0]+(0.447237114882)*x[1]
        arg[1,2,2,1]=(0.385242031153)*x[0]+(-0.960538844247)*x[1]
        arg[1,3,0,0]=(-0.354927072897)*x[0]+(0.842574324062)*x[1]
        arg[1,3,0,1]=(0.319036427919)*x[0]+(-0.156282262536)*x[1]
        arg[1,3,1,0]=(0.587123807717)*x[0]+(1.65277990726)*x[1]
        arg[1,3,1,1]=(0.791697370298)*x[0]+(-1.5681680344)*x[1]
        arg[1,3,2,0]=(-1.41931242045)*x[0]+(0.351285136557)*x[1]
        arg[1,3,2,1]=(0.53175704402)*x[0]+(0.987523241324)*x[1]
        arg[1,4,0,0]=(0.00533504818465)*x[0]+(-0.158886930817)*x[1]
        arg[1,4,0,1]=(1.23725297303)*x[0]+(-0.302896452172)*x[1]
        arg[1,4,1,0]=(0.433953146329)*x[0]+(-0.0721602234887)*x[1]
        arg[1,4,1,1]=(-0.383499726865)*x[0]+(-0.253834671235)*x[1]
        arg[1,4,2,0]=(-0.480776695247)*x[0]+(-0.302145524759)*x[1]
        arg[1,4,2,1]=(1.14113180903)*x[0]+(-1.24304726003)*x[1]
        arg[2,0,0,0]=(0.513776013893)*x[0]+(-0.16660259885)*x[1]
        arg[2,0,0,1]=(1.55739209698)*x[0]+(0.0524084654531)*x[1]
        arg[2,0,1,0]=(0.222567881446)*x[0]+(-0.357158470388)*x[1]
        arg[2,0,1,1]=(0.0179816596229)*x[0]+(0.231534460784)*x[1]
        arg[2,0,2,0]=(1.58460345297)*x[0]+(-0.383588433888)*x[1]
        arg[2,0,2,1]=(-1.73386465536)*x[0]+(-0.743326657544)*x[1]
        arg[2,1,0,0]=(-0.737976950246)*x[0]+(-1.01053396224)*x[1]
        arg[2,1,0,1]=(0.926911190393)*x[0]+(0.620251783925)*x[1]
        arg[2,1,1,0]=(-1.34289630129)*x[0]+(-0.429933681523)*x[1]
        arg[2,1,1,1]=(1.79530806415)*x[0]+(-0.539783957021)*x[1]
        arg[2,1,2,0]=(0.237950974938)*x[0]+(1.93783427703)*x[1]
        arg[2,1,2,1]=(-0.439640741973)*x[0]+(-0.61543984673)*x[1]
        arg[2,2,0,0]=(-1.66403612438)*x[0]+(1.00839566441)*x[1]
        arg[2,2,0,1]=(0.249635983351)*x[0]+(-0.722261363675)*x[1]
        arg[2,2,1,0]=(1.34179674459)*x[0]+(0.502542144188)*x[1]
        arg[2,2,1,1]=(-0.815862586074)*x[0]+(-0.110688273707)*x[1]
        arg[2,2,2,0]=(-0.686810719526)*x[0]+(0.500406563421)*x[1]
        arg[2,2,2,1]=(0.681231089598)*x[0]+(0.605219505613)*x[1]
        arg[2,3,0,0]=(-0.953299029956)*x[0]+(-0.223725858372)*x[1]
        arg[2,3,0,1]=(0.831380756684)*x[0]+(-0.0379777076335)*x[1]
        arg[2,3,1,0]=(-1.63804393157)*x[0]+(0.148235731301)*x[1]
        arg[2,3,1,1]=(-0.17786458643)*x[0]+(-1.77342152699)*x[1]
        arg[2,3,2,0]=(0.0210446244192)*x[0]+(1.59778781524)*x[1]
        arg[2,3,2,1]=(0.877162721214)*x[0]+(0.262990448614)*x[1]
        arg[2,4,0,0]=(-0.650265970331)*x[0]+(-1.07003604369)*x[1]
        arg[2,4,0,1]=(1.12940101029)*x[0]+(0.860589290891)*x[1]
        arg[2,4,1,0]=(0.520056210056)*x[0]+(-1.47691193979)*x[1]
        arg[2,4,1,1]=(1.26823535235)*x[0]+(0.496811720647)*x[1]
        arg[2,4,2,0]=(-1.388913443)*x[0]+(0.585343197943)*x[1]
        arg[2,4,2,1]=(-0.762904547594)*x[0]+(0.503438815917)*x[1]
        arg[3,0,0,0]=(0.382862686993)*x[0]+(-0.923525513303)*x[1]
        arg[3,0,0,1]=(-0.54305794485)*x[0]+(0.467513531444)*x[1]
        arg[3,0,1,0]=(1.24228759776)*x[0]+(-1.65740117421)*x[1]
        arg[3,0,1,1]=(-0.463045362742)*x[0]+(0.186438587087)*x[1]
        arg[3,0,2,0]=(0.296881980828)*x[0]+(-0.385878814949)*x[1]
        arg[3,0,2,1]=(-0.299705899564)*x[0]+(0.490736038538)*x[1]
        arg[3,1,0,0]=(-0.786851428658)*x[0]+(-0.760593794308)*x[1]
        arg[3,1,0,1]=(0.506851336483)*x[0]+(0.995163951197)*x[1]
        arg[3,1,1,0]=(-0.678883937341)*x[0]+(0.56996778248)*x[1]
        arg[3,1,1,1]=(-0.401483642528)*x[0]+(0.0476797332537)*x[1]
        arg[3,1,2,0]=(-0.585357955375)*x[0]+(0.330103647539)*x[1]
        arg[3,1,2,1]=(-0.355539976825)*x[0]+(0.416287924349)*x[1]
        arg[3,2,0,0]=(-0.662837130286)*x[0]+(-0.26640899674)*x[1]
        arg[3,2,0,1]=(0.397553900588)*x[0]+(-1.21708620294)*x[1]
        arg[3,2,1,0]=(0.464963326429)*x[0]+(-0.774515114142)*x[1]
        arg[3,2,1,1]=(0.834351469791)*x[0]+(0.487629540715)*x[1]
        arg[3,2,2,0]=(0.27622693716)*x[0]+(0.811338337719)*x[1]
        arg[3,2,2,1]=(0.0384997772653)*x[0]+(1.03724996578)*x[1]
        arg[3,3,0,0]=(-0.805491987874)*x[0]+(-0.476384547853)*x[1]
        arg[3,3,0,1]=(0.615551780195)*x[0]+(-0.312805967691)*x[1]
        arg[3,3,1,0]=(0.348666798955)*x[0]+(-1.31271156524)*x[1]
        arg[3,3,1,1]=(-0.423569975374)*x[0]+(0.930713125389)*x[1]
        arg[3,3,2,0]=(0.325632686234)*x[0]+(-0.166401033259)*x[1]
        arg[3,3,2,1]=(0.126924861797)*x[0]+(-0.425380897969)*x[1]
        arg[3,4,0,0]=(0.169532204538)*x[0]+(0.614632525765)*x[1]
        arg[3,4,0,1]=(-0.938098696851)*x[0]+(0.648854147587)*x[1]
        arg[3,4,1,0]=(-0.545143970434)*x[0]+(-1.59787515966)*x[1]
        arg[3,4,1,1]=(-0.711190914978)*x[0]+(0.897331522911)*x[1]
        arg[3,4,2,0]=(-0.386498124525)*x[0]+(0.299740420711)*x[1]
        arg[3,4,2,1]=(0.617683032062)*x[0]+(-0.449587703061)*x[1]
        ref[0,0,0,0]=(0.227969627252)*x_ref[0]+(0.778177350248)*x_ref[1]
        ref[0,0,0,1]=(-0.684883711494)*x_ref[0]+(-0.574285957132)*x_ref[1]
        ref[0,0,1,0]=(0.483090608974)*x_ref[0]+(0.0779929087959)*x_ref[1]
        ref[0,0,1,1]=(-0.293150814932)*x_ref[0]+(1.35368931993)*x_ref[1]
        ref[0,0,2,0]=(-0.190664734662)*x_ref[0]+(0.910883386754)*x_ref[1]
        ref[0,0,2,1]=(-1.51255575457)*x_ref[0]+(0.421560181501)*x_ref[1]
        ref[0,1,0,0]=(-0.833402828032)*x_ref[0]+(-0.400466806781)*x_ref[1]
        ref[0,1,0,1]=(-0.212099323605)*x_ref[0]+(-0.112478697221)*x_ref[1]
        ref[0,1,1,0]=(0.57687662926)*x_ref[0]+(0.869758954551)*x_ref[1]
        ref[0,1,1,1]=(1.37261769917)*x_ref[0]+(1.37855789907)*x_ref[1]
        ref[0,1,2,0]=(-1.02838589722)*x_ref[0]+(-0.280409246204)*x_ref[1]
        ref[0,1,2,1]=(1.2333373468)*x_ref[0]+(0.337631183798)*x_ref[1]
        ref[0,2,0,0]=(-0.249065448772)*x_ref[0]+(0.834483646317)*x_ref[1]
        ref[0,2,0,1]=(-1.30860297683)*x_ref[0]+(-0.653929967017)*x_ref[1]
        ref[0,2,1,0]=(-0.0155994484453)*x_ref[0]+(-0.274023166937)*x_ref[1]
        ref[0,2,1,1]=(0.31458360335)*x_ref[0]+(1.59507700673)*x_ref[1]
        ref[0,2,2,0]=(-0.11442619553)*x_ref[0]+(1.6577028576)*x_ref[1]
        ref[0,2,2,1]=(0.274261132032)*x_ref[0]+(1.15236962479)*x_ref[1]
        ref[0,3,0,0]=(1.91433860239)*x_ref[0]+(-0.316349747811)*x_ref[1]
        ref[0,3,0,1]=(-1.42457591054)*x_ref[0]+(-0.362812690805)*x_ref[1]
        ref[0,3,1,0]=(0.572734883096)*x_ref[0]+(0.424724787283)*x_ref[1]
        ref[0,3,1,1]=(-0.118538376813)*x_ref[0]+(0.25190312527)*x_ref[1]
        ref[0,3,2,0]=(-0.533409510081)*x_ref[0]+(-0.804349107915)*x_ref[1]
        ref[0,3,2,1]=(-0.290555363666)*x_ref[0]+(-0.912641477145)*x_ref[1]
        ref[0,4,0,0]=(-0.469612775918)*x_ref[0]+(-0.0447411310793)*x_ref[1]
        ref[0,4,0,1]=(0.108538258831)*x_ref[0]+(0.0453962016662)*x_ref[1]
        ref[0,4,1,0]=(-0.130785944028)*x_ref[0]+(0.0887280914032)*x_ref[1]
        ref[0,4,1,1]=(0.166412309585)*x_ref[0]+(-0.75130797706)*x_ref[1]
        ref[0,4,2,0]=(1.22538232455)*x_ref[0]+(0.112925588343)*x_ref[1]
        ref[0,4,2,1]=(-1.02455171854)*x_ref[0]+(-0.725754942749)*x_ref[1]
        ref[1,0,0,0]=(-0.869171086952)*x_ref[0]+(-0.948876783682)*x_ref[1]
        ref[1,0,0,1]=(0.17730673689)*x_ref[0]+(-0.00166180855579)*x_ref[1]
        ref[1,0,1,0]=(0.482072704433)*x_ref[0]+(0.895847096783)*x_ref[1]
        ref[1,0,1,1]=(0.695736834617)*x_ref[0]+(0.358082704558)*x_ref[1]
        ref[1,0,2,0]=(-0.501799636565)*x_ref[0]+(0.363612558025)*x_ref[1]
        ref[1,0,2,1]=(-1.47204447226)*x_ref[0]+(1.91281426121)*x_ref[1]
        ref[1,1,0,0]=(0.241893182978)*x_ref[0]+(0.95199758669)*x_ref[1]
        ref[1,1,0,1]=(-0.68112361827)*x_ref[0]+(-0.241935141301)*x_ref[1]
        ref[1,1,1,0]=(-0.553030972306)*x_ref[0]+(0.0244503690439)*x_ref[1]
        ref[1,1,1,1]=(0.278177176896)*x_ref[0]+(1.58693846427)*x_ref[1]
        ref[1,1,2,0]=(-0.87980015517)*x_ref[0]+(0.315618837123)*x_ref[1]
        ref[1,1,2,1]=(-0.335783961379)*x_ref[0]+(0.368650136138)*x_ref[1]
        ref[1,2,0,0]=(-0.903535146802)*x_ref[0]+(1.68182923194)*x_ref[1]
        ref[1,2,0,1]=(1.06511511382)*x_ref[0]+(-1.08524762302)*x_ref[1]
        ref[1,2,1,0]=(-0.0455274784681)*x_ref[0]+(-0.519576710661)*x_ref[1]
        ref[1,2,1,1]=(0.735100217062)*x_ref[0]+(-0.986268682671)*x_ref[1]
        ref[1,2,2,0]=(-0.544610797462)*x_ref[0]+(0.447237114882)*x_ref[1]
        ref[1,2,2,1]=(0.385242031153)*x_ref[0]+(-0.960538844247)*x_ref[1]
        ref[1,3,0,0]=(-0.354927072897)*x_ref[0]+(0.842574324062)*x_ref[1]
        ref[1,3,0,1]=(0.319036427919)*x_ref[0]+(-0.156282262536)*x_ref[1]
        ref[1,3,1,0]=(0.587123807717)*x_ref[0]+(1.65277990726)*x_ref[1]
        ref[1,3,1,1]=(0.791697370298)*x_ref[0]+(-1.5681680344)*x_ref[1]
        ref[1,3,2,0]=(-1.41931242045)*x_ref[0]+(0.351285136557)*x_ref[1]
        ref[1,3,2,1]=(0.53175704402)*x_ref[0]+(0.987523241324)*x_ref[1]
        ref[1,4,0,0]=(0.00533504818465)*x_ref[0]+(-0.158886930817)*x_ref[1]
        ref[1,4,0,1]=(1.23725297303)*x_ref[0]+(-0.302896452172)*x_ref[1]
        ref[1,4,1,0]=(0.433953146329)*x_ref[0]+(-0.0721602234887)*x_ref[1]
        ref[1,4,1,1]=(-0.383499726865)*x_ref[0]+(-0.253834671235)*x_ref[1]
        ref[1,4,2,0]=(-0.480776695247)*x_ref[0]+(-0.302145524759)*x_ref[1]
        ref[1,4,2,1]=(1.14113180903)*x_ref[0]+(-1.24304726003)*x_ref[1]
        ref[2,0,0,0]=(0.513776013893)*x_ref[0]+(-0.16660259885)*x_ref[1]
        ref[2,0,0,1]=(1.55739209698)*x_ref[0]+(0.0524084654531)*x_ref[1]
        ref[2,0,1,0]=(0.222567881446)*x_ref[0]+(-0.357158470388)*x_ref[1]
        ref[2,0,1,1]=(0.0179816596229)*x_ref[0]+(0.231534460784)*x_ref[1]
        ref[2,0,2,0]=(1.58460345297)*x_ref[0]+(-0.383588433888)*x_ref[1]
        ref[2,0,2,1]=(-1.73386465536)*x_ref[0]+(-0.743326657544)*x_ref[1]
        ref[2,1,0,0]=(-0.737976950246)*x_ref[0]+(-1.01053396224)*x_ref[1]
        ref[2,1,0,1]=(0.926911190393)*x_ref[0]+(0.620251783925)*x_ref[1]
        ref[2,1,1,0]=(-1.34289630129)*x_ref[0]+(-0.429933681523)*x_ref[1]
        ref[2,1,1,1]=(1.79530806415)*x_ref[0]+(-0.539783957021)*x_ref[1]
        ref[2,1,2,0]=(0.237950974938)*x_ref[0]+(1.93783427703)*x_ref[1]
        ref[2,1,2,1]=(-0.439640741973)*x_ref[0]+(-0.61543984673)*x_ref[1]
        ref[2,2,0,0]=(-1.66403612438)*x_ref[0]+(1.00839566441)*x_ref[1]
        ref[2,2,0,1]=(0.249635983351)*x_ref[0]+(-0.722261363675)*x_ref[1]
        ref[2,2,1,0]=(1.34179674459)*x_ref[0]+(0.502542144188)*x_ref[1]
        ref[2,2,1,1]=(-0.815862586074)*x_ref[0]+(-0.110688273707)*x_ref[1]
        ref[2,2,2,0]=(-0.686810719526)*x_ref[0]+(0.500406563421)*x_ref[1]
        ref[2,2,2,1]=(0.681231089598)*x_ref[0]+(0.605219505613)*x_ref[1]
        ref[2,3,0,0]=(-0.953299029956)*x_ref[0]+(-0.223725858372)*x_ref[1]
        ref[2,3,0,1]=(0.831380756684)*x_ref[0]+(-0.0379777076335)*x_ref[1]
        ref[2,3,1,0]=(-1.63804393157)*x_ref[0]+(0.148235731301)*x_ref[1]
        ref[2,3,1,1]=(-0.17786458643)*x_ref[0]+(-1.77342152699)*x_ref[1]
        ref[2,3,2,0]=(0.0210446244192)*x_ref[0]+(1.59778781524)*x_ref[1]
        ref[2,3,2,1]=(0.877162721214)*x_ref[0]+(0.262990448614)*x_ref[1]
        ref[2,4,0,0]=(-0.650265970331)*x_ref[0]+(-1.07003604369)*x_ref[1]
        ref[2,4,0,1]=(1.12940101029)*x_ref[0]+(0.860589290891)*x_ref[1]
        ref[2,4,1,0]=(0.520056210056)*x_ref[0]+(-1.47691193979)*x_ref[1]
        ref[2,4,1,1]=(1.26823535235)*x_ref[0]+(0.496811720647)*x_ref[1]
        ref[2,4,2,0]=(-1.388913443)*x_ref[0]+(0.585343197943)*x_ref[1]
        ref[2,4,2,1]=(-0.762904547594)*x_ref[0]+(0.503438815917)*x_ref[1]
        ref[3,0,0,0]=(0.382862686993)*x_ref[0]+(-0.923525513303)*x_ref[1]
        ref[3,0,0,1]=(-0.54305794485)*x_ref[0]+(0.467513531444)*x_ref[1]
        ref[3,0,1,0]=(1.24228759776)*x_ref[0]+(-1.65740117421)*x_ref[1]
        ref[3,0,1,1]=(-0.463045362742)*x_ref[0]+(0.186438587087)*x_ref[1]
        ref[3,0,2,0]=(0.296881980828)*x_ref[0]+(-0.385878814949)*x_ref[1]
        ref[3,0,2,1]=(-0.299705899564)*x_ref[0]+(0.490736038538)*x_ref[1]
        ref[3,1,0,0]=(-0.786851428658)*x_ref[0]+(-0.760593794308)*x_ref[1]
        ref[3,1,0,1]=(0.506851336483)*x_ref[0]+(0.995163951197)*x_ref[1]
        ref[3,1,1,0]=(-0.678883937341)*x_ref[0]+(0.56996778248)*x_ref[1]
        ref[3,1,1,1]=(-0.401483642528)*x_ref[0]+(0.0476797332537)*x_ref[1]
        ref[3,1,2,0]=(-0.585357955375)*x_ref[0]+(0.330103647539)*x_ref[1]
        ref[3,1,2,1]=(-0.355539976825)*x_ref[0]+(0.416287924349)*x_ref[1]
        ref[3,2,0,0]=(-0.662837130286)*x_ref[0]+(-0.26640899674)*x_ref[1]
        ref[3,2,0,1]=(0.397553900588)*x_ref[0]+(-1.21708620294)*x_ref[1]
        ref[3,2,1,0]=(0.464963326429)*x_ref[0]+(-0.774515114142)*x_ref[1]
        ref[3,2,1,1]=(0.834351469791)*x_ref[0]+(0.487629540715)*x_ref[1]
        ref[3,2,2,0]=(0.27622693716)*x_ref[0]+(0.811338337719)*x_ref[1]
        ref[3,2,2,1]=(0.0384997772653)*x_ref[0]+(1.03724996578)*x_ref[1]
        ref[3,3,0,0]=(-0.805491987874)*x_ref[0]+(-0.476384547853)*x_ref[1]
        ref[3,3,0,1]=(0.615551780195)*x_ref[0]+(-0.312805967691)*x_ref[1]
        ref[3,3,1,0]=(0.348666798955)*x_ref[0]+(-1.31271156524)*x_ref[1]
        ref[3,3,1,1]=(-0.423569975374)*x_ref[0]+(0.930713125389)*x_ref[1]
        ref[3,3,2,0]=(0.325632686234)*x_ref[0]+(-0.166401033259)*x_ref[1]
        ref[3,3,2,1]=(0.126924861797)*x_ref[0]+(-0.425380897969)*x_ref[1]
        ref[3,4,0,0]=(0.169532204538)*x_ref[0]+(0.614632525765)*x_ref[1]
        ref[3,4,0,1]=(-0.938098696851)*x_ref[0]+(0.648854147587)*x_ref[1]
        ref[3,4,1,0]=(-0.545143970434)*x_ref[0]+(-1.59787515966)*x_ref[1]
        ref[3,4,1,1]=(-0.711190914978)*x_ref[0]+(0.897331522911)*x_ref[1]
        ref[3,4,2,0]=(-0.386498124525)*x_ref[0]+(0.299740420711)*x_ref[1]
        ref[3,4,2,1]=(0.617683032062)*x_ref[0]+(-0.449587703061)*x_ref[1]
      else:
        arg[0,0,0,0]=(-0.791620170192)*x[0]+(1.03871460301)*x[1]+(-0.225035657687)*x[2]
        arg[0,0,0,1]=(-0.464422319203)*x[0]+(0.769454931053)*x[1]+(-0.783260637771)*x[2]
        arg[0,0,1,0]=(0.72655538215)*x[0]+(-1.65736312015)*x[1]+(1.08402505927)*x[2]
        arg[0,0,1,1]=(-0.207576709573)*x[0]+(-0.55250594761)*x[1]+(-0.923702938288)*x[2]
        arg[0,0,2,0]=(-0.701991085516)*x[0]+(-1.76863643107)*x[1]+(0.831382933104)*x[2]
        arg[0,0,2,1]=(0.0640035527319)*x[0]+(0.425446342976)*x[1]+(1.64626766696)*x[2]
        arg[0,1,0,0]=(1.01022963801)*x[0]+(-0.0215015059716)*x[1]+(0.849862425592)*x[2]
        arg[0,1,0,1]=(-0.45776932943)*x[0]+(0.7621342151)*x[1]+(-1.40123018391)*x[2]
        arg[0,1,1,0]=(0.737485054127)*x[0]+(-0.147513905858)*x[1]+(-0.616928595213)*x[2]
        arg[0,1,1,1]=(-0.0608005391894)*x[0]+(-0.154978210573)*x[1]+(-1.40322011577)*x[2]
        arg[0,1,2,0]=(1.59908177714)*x[0]+(-0.378738573932)*x[1]+(1.45285315247)*x[2]
        arg[0,1,2,1]=(-0.622552157046)*x[0]+(1.19996417516)*x[1]+(0.728842900843)*x[2]
        arg[0,2,0,0]=(-0.46942034691)*x[0]+(0.333309721372)*x[1]+(-0.280846553832)*x[2]
        arg[0,2,0,1]=(1.22208788109)*x[0]+(0.369339594285)*x[1]+(1.31078425589)*x[2]
        arg[0,2,1,0]=(0.0794165004114)*x[0]+(0.44665451846)*x[1]+(-0.814625708916)*x[2]
        arg[0,2,1,1]=(0.594200807165)*x[0]+(-0.00616720333193)*x[1]+(0.382706175172)*x[2]
        arg[0,2,2,0]=(0.0372230170459)*x[0]+(0.440339923291)*x[1]+(0.137641070028)*x[2]
        arg[0,2,2,1]=(-1.29863078274)*x[0]+(0.283142455171)*x[1]+(0.0844540074659)*x[2]
        arg[0,3,0,0]=(1.72811808036)*x[0]+(0.536456781522)*x[1]+(0.0266737704706)*x[2]
        arg[0,3,0,1]=(0.148219555101)*x[0]+(-0.6887500275)*x[1]+(-0.405613136632)*x[2]
        arg[0,3,1,0]=(0.708262537279)*x[0]+(1.16319635773)*x[1]+(-0.030808721674)*x[2]
        arg[0,3,1,1]=(0.80834615863)*x[0]+(-0.176789000925)*x[1]+(-1.72576120963)*x[2]
        arg[0,3,2,0]=(0.21128315299)*x[0]+(0.770037046126)*x[1]+(0.988028934422)*x[2]
        arg[0,3,2,1]=(1.10522136317)*x[0]+(1.6319351743)*x[1]+(-0.784693713387)*x[2]
        arg[0,4,0,0]=(0.121088121064)*x[0]+(-1.79718067791)*x[1]+(-0.907857173318)*x[2]
        arg[0,4,0,1]=(-0.695887294217)*x[0]+(-1.24166787807)*x[1]+(-1.24968023856)*x[2]
        arg[0,4,1,0]=(-0.202510797291)*x[0]+(0.628310156105)*x[1]+(0.605146447822)*x[2]
        arg[0,4,1,1]=(0.149227458424)*x[0]+(-1.5650392775)*x[1]+(-0.763944892663)*x[2]
        arg[0,4,2,0]=(-0.393218625872)*x[0]+(0.479145914948)*x[1]+(-0.313960447492)*x[2]
        arg[0,4,2,1]=(-0.0191807443799)*x[0]+(0.592992659119)*x[1]+(-0.475633115484)*x[2]
        arg[1,0,0,0]=(-0.964829354261)*x[0]+(0.453014802906)*x[1]+(-0.779502566938)*x[2]
        arg[1,0,0,1]=(0.0151205739508)*x[0]+(-0.153706429594)*x[1]+(0.00866460475724)*x[2]
        arg[1,0,1,0]=(0.92592276366)*x[0]+(0.530787400525)*x[1]+(-0.0577912606876)*x[2]
        arg[1,0,1,1]=(0.948404467077)*x[0]+(-0.265905470751)*x[1]+(0.472349192005)*x[2]
        arg[1,0,2,0]=(0.0176615383702)*x[0]+(0.679605821761)*x[1]+(-0.0333590911941)*x[2]
        arg[1,0,2,1]=(-1.15594952629)*x[0]+(0.264868376508)*x[1]+(-0.883690535499)*x[2]
        arg[1,1,0,0]=(-0.738712703994)*x[0]+(0.545839232808)*x[1]+(0.592474501134)*x[2]
        arg[1,1,0,1]=(-1.3468960453)*x[0]+(-0.00848202443614)*x[1]+(-0.0124138286126)*x[2]
        arg[1,1,1,0]=(0.900228409707)*x[0]+(-1.86848082357)*x[1]+(0.421539431142)*x[2]
        arg[1,1,1,1]=(1.42754013963)*x[0]+(-0.861376440449)*x[1]+(0.0965520362291)*x[2]
        arg[1,1,2,0]=(-0.35873674515)*x[0]+(0.0862592617941)*x[1]+(-0.536106207025)*x[2]
        arg[1,1,2,1]=(0.90712910097)*x[0]+(0.974545253055)*x[1]+(0.570708510858)*x[2]
        arg[1,2,0,0]=(1.19310775669)*x[0]+(0.309414331679)*x[1]+(-0.271298061586)*x[2]
        arg[1,2,0,1]=(0.551778129545)*x[0]+(0.539028430134)*x[1]+(0.239406239262)*x[2]
        arg[1,2,1,0]=(-0.810625054344)*x[0]+(-0.241072190233)*x[1]+(-1.58590369097)*x[2]
        arg[1,2,1,1]=(0.529669075734)*x[0]+(-0.890365024163)*x[1]+(-1.69798509084)*x[2]
        arg[1,2,2,0]=(0.0437087707448)*x[0]+(0.88372190198)*x[1]+(1.86547747806)*x[2]
        arg[1,2,2,1]=(1.30739438898)*x[0]+(0.315850112101)*x[1]+(-0.262001133683)*x[2]
        arg[1,3,0,0]=(-0.143426236837)*x[0]+(-1.1703384826)*x[1]+(0.50608348646)*x[2]
        arg[1,3,0,1]=(-0.0444408776922)*x[0]+(1.26952782036)*x[1]+(1.67192602934)*x[2]
        arg[1,3,1,0]=(0.672742824368)*x[0]+(-0.785246676537)*x[1]+(0.716130664056)*x[2]
        arg[1,3,1,1]=(-0.295182508335)*x[0]+(-1.53658860897)*x[1]+(-1.87133995422)*x[2]
        arg[1,3,2,0]=(-0.440859530081)*x[0]+(0.0730120015821)*x[1]+(-1.02465537353)*x[2]
        arg[1,3,2,1]=(0.838320612003)*x[0]+(1.81233688454)*x[1]+(-1.0633876685)*x[2]
        arg[1,4,0,0]=(0.52440453051)*x[0]+(-0.0182588999022)*x[1]+(1.05556310351)*x[2]
        arg[1,4,0,1]=(0.766612727495)*x[0]+(1.58987590746)*x[1]+(-0.703801484569)*x[2]
        arg[1,4,1,0]=(0.373439757535)*x[0]+(-0.0676963704305)*x[1]+(0.0948034094293)*x[2]
        arg[1,4,1,1]=(-0.542283543375)*x[0]+(-0.454408605761)*x[1]+(-0.398904635316)*x[2]
        arg[1,4,2,0]=(1.3374871143)*x[0]+(0.0067849394587)*x[1]+(-0.319452094882)*x[2]
        arg[1,4,2,1]=(0.38530755393)*x[0]+(1.05356462429)*x[1]+(-0.302179447719)*x[2]
        arg[2,0,0,0]=(-0.123109761583)*x[0]+(0.69223848143)*x[1]+(-1.36903487092)*x[2]
        arg[2,0,0,1]=(0.121957328459)*x[0]+(0.102229685387)*x[1]+(0.962499974737)*x[2]
        arg[2,0,1,0]=(0.478614188653)*x[0]+(-1.23058817023)*x[1]+(1.30950997478)*x[2]
        arg[2,0,1,1]=(-0.529033562461)*x[0]+(0.563030509591)*x[1]+(-0.564939226654)*x[2]
        arg[2,0,2,0]=(-0.377740282406)*x[0]+(1.20532110343)*x[1]+(1.55615174947)*x[2]
        arg[2,0,2,1]=(0.539119710703)*x[0]+(1.91771865857)*x[1]+(0.0809882488048)*x[2]
        arg[2,1,0,0]=(0.00548664737796)*x[0]+(-0.157716099068)*x[1]+(-1.40009044235)*x[2]
        arg[2,1,0,1]=(0.0667557890666)*x[0]+(0.742484448131)*x[1]+(-0.325246111346)*x[2]
        arg[2,1,1,0]=(-0.214965474071)*x[0]+(-0.215055151964)*x[1]+(1.15673960537)*x[2]
        arg[2,1,1,1]=(0.605764831789)*x[0]+(-1.0432964832)*x[1]+(0.645482523161)*x[2]
        arg[2,1,2,0]=(-1.13098364088)*x[0]+(-0.932044089041)*x[1]+(-0.308053828241)*x[2]
        arg[2,1,2,1]=(0.481961305459)*x[0]+(-0.101277472029)*x[1]+(-0.17520207974)*x[2]
        arg[2,2,0,0]=(0.579831852303)*x[0]+(0.538731990254)*x[1]+(-0.0897460374205)*x[2]
        arg[2,2,0,1]=(-1.8763181434)*x[0]+(-0.132710928569)*x[1]+(1.03169751651)*x[2]
        arg[2,2,1,0]=(0.56871511431)*x[0]+(0.075333481656)*x[1]+(1.10238818953)*x[2]
        arg[2,2,1,1]=(1.49347755246)*x[0]+(-1.00500941047)*x[1]+(-0.181733196359)*x[2]
        arg[2,2,2,0]=(0.532881509971)*x[0]+(-0.364333964659)*x[1]+(1.09633720883)*x[2]
        arg[2,2,2,1]=(-0.518419048615)*x[0]+(-0.48091100593)*x[1]+(0.724628840153)*x[2]
        arg[2,3,0,0]=(0.353475859331)*x[0]+(-0.0239879269825)*x[1]+(1.32068893718)*x[2]
        arg[2,3,0,1]=(0.553999122102)*x[0]+(-0.128530869239)*x[1]+(0.134663336287)*x[2]
        arg[2,3,1,0]=(-1.59775882569)*x[0]+(-1.07335424785)*x[1]+(1.39223587988)*x[2]
        arg[2,3,1,1]=(-0.921252080095)*x[0]+(1.46226299067)*x[1]+(-1.13412499932)*x[2]
        arg[2,3,2,0]=(0.417572910178)*x[0]+(0.0814889599489)*x[1]+(0.292159249719)*x[2]
        arg[2,3,2,1]=(1.44620228921)*x[0]+(0.767854577367)*x[1]+(-0.0113709806383)*x[2]
        arg[2,4,0,0]=(-0.156987001681)*x[0]+(-0.922524257833)*x[1]+(0.521128360838)*x[2]
        arg[2,4,0,1]=(0.154198133354)*x[0]+(0.462020370175)*x[1]+(0.966768653665)*x[2]
        arg[2,4,1,0]=(0.507836849242)*x[0]+(1.16701911244)*x[1]+(0.264543038211)*x[2]
        arg[2,4,1,1]=(0.452059436879)*x[0]+(0.409662432778)*x[1]+(-0.403800872595)*x[2]
        arg[2,4,2,0]=(0.521175521008)*x[0]+(-0.362551578816)*x[1]+(0.0922727331623)*x[2]
        arg[2,4,2,1]=(-1.0990361823)*x[0]+(1.73655146851)*x[1]+(-0.867917402922)*x[2]
        arg[3,0,0,0]=(-0.0527010882236)*x[0]+(-1.11528215488)*x[1]+(1.3141906393)*x[2]
        arg[3,0,0,1]=(0.151284271917)*x[0]+(-0.0159077450668)*x[1]+(1.1798338931)*x[2]
        arg[3,0,1,0]=(-1.7648963435)*x[0]+(0.229511437402)*x[1]+(0.924415900312)*x[2]
        arg[3,0,1,1]=(-0.498271821094)*x[0]+(1.58363063019)*x[1]+(1.64871750388)*x[2]
        arg[3,0,2,0]=(-1.11245170725)*x[0]+(1.27662612569)*x[1]+(0.145950372369)*x[2]
        arg[3,0,2,1]=(-0.691643924238)*x[0]+(-0.178211874279)*x[1]+(-0.341071050366)*x[2]
        arg[3,1,0,0]=(-0.929804889461)*x[0]+(0.199087067309)*x[1]+(1.00150168883)*x[2]
        arg[3,1,0,1]=(-1.25152265093)*x[0]+(0.517076294911)*x[1]+(0.103798312362)*x[2]
        arg[3,1,1,0]=(0.797316290623)*x[0]+(0.0239285379293)*x[1]+(-0.836519299418)*x[2]
        arg[3,1,1,1]=(-0.470084592176)*x[0]+(0.844364701728)*x[1]+(-0.027373275093)*x[2]
        arg[3,1,2,0]=(1.0340279373)*x[0]+(-0.0133951276315)*x[1]+(1.17710546194)*x[2]
        arg[3,1,2,1]=(0.0325929541496)*x[0]+(1.13784867184)*x[1]+(-0.0529194266769)*x[2]
        arg[3,2,0,0]=(1.92837561772)*x[0]+(-1.63633468831)*x[1]+(1.4212324281)*x[2]
        arg[3,2,0,1]=(1.09556054408)*x[0]+(1.43286059545)*x[1]+(1.20005604593)*x[2]
        arg[3,2,1,0]=(0.308887169169)*x[0]+(-0.149214224208)*x[1]+(-1.13559614877)*x[2]
        arg[3,2,1,1]=(-0.0648352909004)*x[0]+(-0.5645470596)*x[1]+(-0.525131730514)*x[2]
        arg[3,2,2,0]=(0.21826829974)*x[0]+(1.5950239996)*x[1]+(0.333592425438)*x[2]
        arg[3,2,2,1]=(-1.25613956172)*x[0]+(-0.242338679844)*x[1]+(-0.958875577074)*x[2]
        arg[3,3,0,0]=(1.65005362159)*x[0]+(-1.16713867165)*x[1]+(-0.282495293685)*x[2]
        arg[3,3,0,1]=(1.04854406427)*x[0]+(0.876270949531)*x[1]+(0.291036304743)*x[2]
        arg[3,3,1,0]=(-1.66619451175)*x[0]+(0.656275192484)*x[1]+(-1.27104499942)*x[2]
        arg[3,3,1,1]=(1.55407146907)*x[0]+(-0.322950943159)*x[1]+(-0.287565403896)*x[2]
        arg[3,3,2,0]=(-1.54760268583)*x[0]+(-1.45847761887)*x[1]+(1.13939567009)*x[2]
        arg[3,3,2,1]=(1.24632351699)*x[0]+(0.0265500081594)*x[1]+(1.61925095159)*x[2]
        arg[3,4,0,0]=(-0.0430889433549)*x[0]+(-1.46995559463)*x[1]+(-1.14915159823)*x[2]
        arg[3,4,0,1]=(0.0392641403409)*x[0]+(0.0603053049639)*x[1]+(0.647321169934)*x[2]
        arg[3,4,1,0]=(-0.156300985227)*x[0]+(0.79369911928)*x[1]+(-0.413124762507)*x[2]
        arg[3,4,1,1]=(0.258402059015)*x[0]+(0.00227307988585)*x[1]+(0.70067141295)*x[2]
        arg[3,4,2,0]=(-1.15131298872)*x[0]+(-0.627449043157)*x[1]+(-0.402268345861)*x[2]
        arg[3,4,2,1]=(0.455934238179)*x[0]+(-0.951120759108)*x[1]+(-0.280459367131)*x[2]
        ref[0,0,0,0]=(-0.791620170192)*x_ref[0]+(1.03871460301)*x_ref[1]+(-0.225035657687)*x_ref[2]
        ref[0,0,0,1]=(-0.464422319203)*x_ref[0]+(0.769454931053)*x_ref[1]+(-0.783260637771)*x_ref[2]
        ref[0,0,1,0]=(0.72655538215)*x_ref[0]+(-1.65736312015)*x_ref[1]+(1.08402505927)*x_ref[2]
        ref[0,0,1,1]=(-0.207576709573)*x_ref[0]+(-0.55250594761)*x_ref[1]+(-0.923702938288)*x_ref[2]
        ref[0,0,2,0]=(-0.701991085516)*x_ref[0]+(-1.76863643107)*x_ref[1]+(0.831382933104)*x_ref[2]
        ref[0,0,2,1]=(0.0640035527319)*x_ref[0]+(0.425446342976)*x_ref[1]+(1.64626766696)*x_ref[2]
        ref[0,1,0,0]=(1.01022963801)*x_ref[0]+(-0.0215015059716)*x_ref[1]+(0.849862425592)*x_ref[2]
        ref[0,1,0,1]=(-0.45776932943)*x_ref[0]+(0.7621342151)*x_ref[1]+(-1.40123018391)*x_ref[2]
        ref[0,1,1,0]=(0.737485054127)*x_ref[0]+(-0.147513905858)*x_ref[1]+(-0.616928595213)*x_ref[2]
        ref[0,1,1,1]=(-0.0608005391894)*x_ref[0]+(-0.154978210573)*x_ref[1]+(-1.40322011577)*x_ref[2]
        ref[0,1,2,0]=(1.59908177714)*x_ref[0]+(-0.378738573932)*x_ref[1]+(1.45285315247)*x_ref[2]
        ref[0,1,2,1]=(-0.622552157046)*x_ref[0]+(1.19996417516)*x_ref[1]+(0.728842900843)*x_ref[2]
        ref[0,2,0,0]=(-0.46942034691)*x_ref[0]+(0.333309721372)*x_ref[1]+(-0.280846553832)*x_ref[2]
        ref[0,2,0,1]=(1.22208788109)*x_ref[0]+(0.369339594285)*x_ref[1]+(1.31078425589)*x_ref[2]
        ref[0,2,1,0]=(0.0794165004114)*x_ref[0]+(0.44665451846)*x_ref[1]+(-0.814625708916)*x_ref[2]
        ref[0,2,1,1]=(0.594200807165)*x_ref[0]+(-0.00616720333193)*x_ref[1]+(0.382706175172)*x_ref[2]
        ref[0,2,2,0]=(0.0372230170459)*x_ref[0]+(0.440339923291)*x_ref[1]+(0.137641070028)*x_ref[2]
        ref[0,2,2,1]=(-1.29863078274)*x_ref[0]+(0.283142455171)*x_ref[1]+(0.0844540074659)*x_ref[2]
        ref[0,3,0,0]=(1.72811808036)*x_ref[0]+(0.536456781522)*x_ref[1]+(0.0266737704706)*x_ref[2]
        ref[0,3,0,1]=(0.148219555101)*x_ref[0]+(-0.6887500275)*x_ref[1]+(-0.405613136632)*x_ref[2]
        ref[0,3,1,0]=(0.708262537279)*x_ref[0]+(1.16319635773)*x_ref[1]+(-0.030808721674)*x_ref[2]
        ref[0,3,1,1]=(0.80834615863)*x_ref[0]+(-0.176789000925)*x_ref[1]+(-1.72576120963)*x_ref[2]
        ref[0,3,2,0]=(0.21128315299)*x_ref[0]+(0.770037046126)*x_ref[1]+(0.988028934422)*x_ref[2]
        ref[0,3,2,1]=(1.10522136317)*x_ref[0]+(1.6319351743)*x_ref[1]+(-0.784693713387)*x_ref[2]
        ref[0,4,0,0]=(0.121088121064)*x_ref[0]+(-1.79718067791)*x_ref[1]+(-0.907857173318)*x_ref[2]
        ref[0,4,0,1]=(-0.695887294217)*x_ref[0]+(-1.24166787807)*x_ref[1]+(-1.24968023856)*x_ref[2]
        ref[0,4,1,0]=(-0.202510797291)*x_ref[0]+(0.628310156105)*x_ref[1]+(0.605146447822)*x_ref[2]
        ref[0,4,1,1]=(0.149227458424)*x_ref[0]+(-1.5650392775)*x_ref[1]+(-0.763944892663)*x_ref[2]
        ref[0,4,2,0]=(-0.393218625872)*x_ref[0]+(0.479145914948)*x_ref[1]+(-0.313960447492)*x_ref[2]
        ref[0,4,2,1]=(-0.0191807443799)*x_ref[0]+(0.592992659119)*x_ref[1]+(-0.475633115484)*x_ref[2]
        ref[1,0,0,0]=(-0.964829354261)*x_ref[0]+(0.453014802906)*x_ref[1]+(-0.779502566938)*x_ref[2]
        ref[1,0,0,1]=(0.0151205739508)*x_ref[0]+(-0.153706429594)*x_ref[1]+(0.00866460475724)*x_ref[2]
        ref[1,0,1,0]=(0.92592276366)*x_ref[0]+(0.530787400525)*x_ref[1]+(-0.0577912606876)*x_ref[2]
        ref[1,0,1,1]=(0.948404467077)*x_ref[0]+(-0.265905470751)*x_ref[1]+(0.472349192005)*x_ref[2]
        ref[1,0,2,0]=(0.0176615383702)*x_ref[0]+(0.679605821761)*x_ref[1]+(-0.0333590911941)*x_ref[2]
        ref[1,0,2,1]=(-1.15594952629)*x_ref[0]+(0.264868376508)*x_ref[1]+(-0.883690535499)*x_ref[2]
        ref[1,1,0,0]=(-0.738712703994)*x_ref[0]+(0.545839232808)*x_ref[1]+(0.592474501134)*x_ref[2]
        ref[1,1,0,1]=(-1.3468960453)*x_ref[0]+(-0.00848202443614)*x_ref[1]+(-0.0124138286126)*x_ref[2]
        ref[1,1,1,0]=(0.900228409707)*x_ref[0]+(-1.86848082357)*x_ref[1]+(0.421539431142)*x_ref[2]
        ref[1,1,1,1]=(1.42754013963)*x_ref[0]+(-0.861376440449)*x_ref[1]+(0.0965520362291)*x_ref[2]
        ref[1,1,2,0]=(-0.35873674515)*x_ref[0]+(0.0862592617941)*x_ref[1]+(-0.536106207025)*x_ref[2]
        ref[1,1,2,1]=(0.90712910097)*x_ref[0]+(0.974545253055)*x_ref[1]+(0.570708510858)*x_ref[2]
        ref[1,2,0,0]=(1.19310775669)*x_ref[0]+(0.309414331679)*x_ref[1]+(-0.271298061586)*x_ref[2]
        ref[1,2,0,1]=(0.551778129545)*x_ref[0]+(0.539028430134)*x_ref[1]+(0.239406239262)*x_ref[2]
        ref[1,2,1,0]=(-0.810625054344)*x_ref[0]+(-0.241072190233)*x_ref[1]+(-1.58590369097)*x_ref[2]
        ref[1,2,1,1]=(0.529669075734)*x_ref[0]+(-0.890365024163)*x_ref[1]+(-1.69798509084)*x_ref[2]
        ref[1,2,2,0]=(0.0437087707448)*x_ref[0]+(0.88372190198)*x_ref[1]+(1.86547747806)*x_ref[2]
        ref[1,2,2,1]=(1.30739438898)*x_ref[0]+(0.315850112101)*x_ref[1]+(-0.262001133683)*x_ref[2]
        ref[1,3,0,0]=(-0.143426236837)*x_ref[0]+(-1.1703384826)*x_ref[1]+(0.50608348646)*x_ref[2]
        ref[1,3,0,1]=(-0.0444408776922)*x_ref[0]+(1.26952782036)*x_ref[1]+(1.67192602934)*x_ref[2]
        ref[1,3,1,0]=(0.672742824368)*x_ref[0]+(-0.785246676537)*x_ref[1]+(0.716130664056)*x_ref[2]
        ref[1,3,1,1]=(-0.295182508335)*x_ref[0]+(-1.53658860897)*x_ref[1]+(-1.87133995422)*x_ref[2]
        ref[1,3,2,0]=(-0.440859530081)*x_ref[0]+(0.0730120015821)*x_ref[1]+(-1.02465537353)*x_ref[2]
        ref[1,3,2,1]=(0.838320612003)*x_ref[0]+(1.81233688454)*x_ref[1]+(-1.0633876685)*x_ref[2]
        ref[1,4,0,0]=(0.52440453051)*x_ref[0]+(-0.0182588999022)*x_ref[1]+(1.05556310351)*x_ref[2]
        ref[1,4,0,1]=(0.766612727495)*x_ref[0]+(1.58987590746)*x_ref[1]+(-0.703801484569)*x_ref[2]
        ref[1,4,1,0]=(0.373439757535)*x_ref[0]+(-0.0676963704305)*x_ref[1]+(0.0948034094293)*x_ref[2]
        ref[1,4,1,1]=(-0.542283543375)*x_ref[0]+(-0.454408605761)*x_ref[1]+(-0.398904635316)*x_ref[2]
        ref[1,4,2,0]=(1.3374871143)*x_ref[0]+(0.0067849394587)*x_ref[1]+(-0.319452094882)*x_ref[2]
        ref[1,4,2,1]=(0.38530755393)*x_ref[0]+(1.05356462429)*x_ref[1]+(-0.302179447719)*x_ref[2]
        ref[2,0,0,0]=(-0.123109761583)*x_ref[0]+(0.69223848143)*x_ref[1]+(-1.36903487092)*x_ref[2]
        ref[2,0,0,1]=(0.121957328459)*x_ref[0]+(0.102229685387)*x_ref[1]+(0.962499974737)*x_ref[2]
        ref[2,0,1,0]=(0.478614188653)*x_ref[0]+(-1.23058817023)*x_ref[1]+(1.30950997478)*x_ref[2]
        ref[2,0,1,1]=(-0.529033562461)*x_ref[0]+(0.563030509591)*x_ref[1]+(-0.564939226654)*x_ref[2]
        ref[2,0,2,0]=(-0.377740282406)*x_ref[0]+(1.20532110343)*x_ref[1]+(1.55615174947)*x_ref[2]
        ref[2,0,2,1]=(0.539119710703)*x_ref[0]+(1.91771865857)*x_ref[1]+(0.0809882488048)*x_ref[2]
        ref[2,1,0,0]=(0.00548664737796)*x_ref[0]+(-0.157716099068)*x_ref[1]+(-1.40009044235)*x_ref[2]
        ref[2,1,0,1]=(0.0667557890666)*x_ref[0]+(0.742484448131)*x_ref[1]+(-0.325246111346)*x_ref[2]
        ref[2,1,1,0]=(-0.214965474071)*x_ref[0]+(-0.215055151964)*x_ref[1]+(1.15673960537)*x_ref[2]
        ref[2,1,1,1]=(0.605764831789)*x_ref[0]+(-1.0432964832)*x_ref[1]+(0.645482523161)*x_ref[2]
        ref[2,1,2,0]=(-1.13098364088)*x_ref[0]+(-0.932044089041)*x_ref[1]+(-0.308053828241)*x_ref[2]
        ref[2,1,2,1]=(0.481961305459)*x_ref[0]+(-0.101277472029)*x_ref[1]+(-0.17520207974)*x_ref[2]
        ref[2,2,0,0]=(0.579831852303)*x_ref[0]+(0.538731990254)*x_ref[1]+(-0.0897460374205)*x_ref[2]
        ref[2,2,0,1]=(-1.8763181434)*x_ref[0]+(-0.132710928569)*x_ref[1]+(1.03169751651)*x_ref[2]
        ref[2,2,1,0]=(0.56871511431)*x_ref[0]+(0.075333481656)*x_ref[1]+(1.10238818953)*x_ref[2]
        ref[2,2,1,1]=(1.49347755246)*x_ref[0]+(-1.00500941047)*x_ref[1]+(-0.181733196359)*x_ref[2]
        ref[2,2,2,0]=(0.532881509971)*x_ref[0]+(-0.364333964659)*x_ref[1]+(1.09633720883)*x_ref[2]
        ref[2,2,2,1]=(-0.518419048615)*x_ref[0]+(-0.48091100593)*x_ref[1]+(0.724628840153)*x_ref[2]
        ref[2,3,0,0]=(0.353475859331)*x_ref[0]+(-0.0239879269825)*x_ref[1]+(1.32068893718)*x_ref[2]
        ref[2,3,0,1]=(0.553999122102)*x_ref[0]+(-0.128530869239)*x_ref[1]+(0.134663336287)*x_ref[2]
        ref[2,3,1,0]=(-1.59775882569)*x_ref[0]+(-1.07335424785)*x_ref[1]+(1.39223587988)*x_ref[2]
        ref[2,3,1,1]=(-0.921252080095)*x_ref[0]+(1.46226299067)*x_ref[1]+(-1.13412499932)*x_ref[2]
        ref[2,3,2,0]=(0.417572910178)*x_ref[0]+(0.0814889599489)*x_ref[1]+(0.292159249719)*x_ref[2]
        ref[2,3,2,1]=(1.44620228921)*x_ref[0]+(0.767854577367)*x_ref[1]+(-0.0113709806383)*x_ref[2]
        ref[2,4,0,0]=(-0.156987001681)*x_ref[0]+(-0.922524257833)*x_ref[1]+(0.521128360838)*x_ref[2]
        ref[2,4,0,1]=(0.154198133354)*x_ref[0]+(0.462020370175)*x_ref[1]+(0.966768653665)*x_ref[2]
        ref[2,4,1,0]=(0.507836849242)*x_ref[0]+(1.16701911244)*x_ref[1]+(0.264543038211)*x_ref[2]
        ref[2,4,1,1]=(0.452059436879)*x_ref[0]+(0.409662432778)*x_ref[1]+(-0.403800872595)*x_ref[2]
        ref[2,4,2,0]=(0.521175521008)*x_ref[0]+(-0.362551578816)*x_ref[1]+(0.0922727331623)*x_ref[2]
        ref[2,4,2,1]=(-1.0990361823)*x_ref[0]+(1.73655146851)*x_ref[1]+(-0.867917402922)*x_ref[2]
        ref[3,0,0,0]=(-0.0527010882236)*x_ref[0]+(-1.11528215488)*x_ref[1]+(1.3141906393)*x_ref[2]
        ref[3,0,0,1]=(0.151284271917)*x_ref[0]+(-0.0159077450668)*x_ref[1]+(1.1798338931)*x_ref[2]
        ref[3,0,1,0]=(-1.7648963435)*x_ref[0]+(0.229511437402)*x_ref[1]+(0.924415900312)*x_ref[2]
        ref[3,0,1,1]=(-0.498271821094)*x_ref[0]+(1.58363063019)*x_ref[1]+(1.64871750388)*x_ref[2]
        ref[3,0,2,0]=(-1.11245170725)*x_ref[0]+(1.27662612569)*x_ref[1]+(0.145950372369)*x_ref[2]
        ref[3,0,2,1]=(-0.691643924238)*x_ref[0]+(-0.178211874279)*x_ref[1]+(-0.341071050366)*x_ref[2]
        ref[3,1,0,0]=(-0.929804889461)*x_ref[0]+(0.199087067309)*x_ref[1]+(1.00150168883)*x_ref[2]
        ref[3,1,0,1]=(-1.25152265093)*x_ref[0]+(0.517076294911)*x_ref[1]+(0.103798312362)*x_ref[2]
        ref[3,1,1,0]=(0.797316290623)*x_ref[0]+(0.0239285379293)*x_ref[1]+(-0.836519299418)*x_ref[2]
        ref[3,1,1,1]=(-0.470084592176)*x_ref[0]+(0.844364701728)*x_ref[1]+(-0.027373275093)*x_ref[2]
        ref[3,1,2,0]=(1.0340279373)*x_ref[0]+(-0.0133951276315)*x_ref[1]+(1.17710546194)*x_ref[2]
        ref[3,1,2,1]=(0.0325929541496)*x_ref[0]+(1.13784867184)*x_ref[1]+(-0.0529194266769)*x_ref[2]
        ref[3,2,0,0]=(1.92837561772)*x_ref[0]+(-1.63633468831)*x_ref[1]+(1.4212324281)*x_ref[2]
        ref[3,2,0,1]=(1.09556054408)*x_ref[0]+(1.43286059545)*x_ref[1]+(1.20005604593)*x_ref[2]
        ref[3,2,1,0]=(0.308887169169)*x_ref[0]+(-0.149214224208)*x_ref[1]+(-1.13559614877)*x_ref[2]
        ref[3,2,1,1]=(-0.0648352909004)*x_ref[0]+(-0.5645470596)*x_ref[1]+(-0.525131730514)*x_ref[2]
        ref[3,2,2,0]=(0.21826829974)*x_ref[0]+(1.5950239996)*x_ref[1]+(0.333592425438)*x_ref[2]
        ref[3,2,2,1]=(-1.25613956172)*x_ref[0]+(-0.242338679844)*x_ref[1]+(-0.958875577074)*x_ref[2]
        ref[3,3,0,0]=(1.65005362159)*x_ref[0]+(-1.16713867165)*x_ref[1]+(-0.282495293685)*x_ref[2]
        ref[3,3,0,1]=(1.04854406427)*x_ref[0]+(0.876270949531)*x_ref[1]+(0.291036304743)*x_ref[2]
        ref[3,3,1,0]=(-1.66619451175)*x_ref[0]+(0.656275192484)*x_ref[1]+(-1.27104499942)*x_ref[2]
        ref[3,3,1,1]=(1.55407146907)*x_ref[0]+(-0.322950943159)*x_ref[1]+(-0.287565403896)*x_ref[2]
        ref[3,3,2,0]=(-1.54760268583)*x_ref[0]+(-1.45847761887)*x_ref[1]+(1.13939567009)*x_ref[2]
        ref[3,3,2,1]=(1.24632351699)*x_ref[0]+(0.0265500081594)*x_ref[1]+(1.61925095159)*x_ref[2]
        ref[3,4,0,0]=(-0.0430889433549)*x_ref[0]+(-1.46995559463)*x_ref[1]+(-1.14915159823)*x_ref[2]
        ref[3,4,0,1]=(0.0392641403409)*x_ref[0]+(0.0603053049639)*x_ref[1]+(0.647321169934)*x_ref[2]
        ref[3,4,1,0]=(-0.156300985227)*x_ref[0]+(0.79369911928)*x_ref[1]+(-0.413124762507)*x_ref[2]
        ref[3,4,1,1]=(0.258402059015)*x_ref[0]+(0.00227307988585)*x_ref[1]+(0.70067141295)*x_ref[2]
        ref[3,4,2,0]=(-1.15131298872)*x_ref[0]+(-0.627449043157)*x_ref[1]+(-0.402268345861)*x_ref[2]
        ref[3,4,2,1]=(0.455934238179)*x_ref[0]+(-0.951120759108)*x_ref[1]+(-0.280459367131)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_ReducedFunctionOnContactZero_rank0(self):
      """
      tests interpolate for rank 0 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactZero onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactZero(self.domain)
      x=w.getX()
      arg=Data(0,(),w)
      ref=Data(0,(),w_ref)
      if dim==2:
        arg=(0.203238386406)*x[0]**o+(0.734542238717)*x[0]+(-0.627248060056)*x[1]**o+(0.347224669854)*x[1]
        ref=(0.203238386406)*x_ref[0]**o+(0.734542238717)*x_ref[0]+(-0.627248060056)*x_ref[1]**o+(0.347224669854)*x_ref[1]
      else:
        arg=(-0.387501489724)*x[0]**o+(0.752554332333)*x[0]+(-0.0831588271355)*x[1]**o+(0.615191870812)*x[1]+(-0.264702923622)*x[2]**o+(0.404594629993)*x[2]
        ref=(-0.387501489724)*x_ref[0]**o+(0.752554332333)*x_ref[0]+(-0.0831588271355)*x_ref[1]**o+(0.615191870812)*x_ref[1]+(-0.264702923622)*x_ref[2]**o+(0.404594629993)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_ReducedFunctionOnContactZero_rank1(self):
      """
      tests interpolate for rank 1 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactZero onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactZero(self.domain)
      x=w.getX()
      arg=Data(0,(2,),w)
      ref=Data(0,(2,),w_ref)
      if dim==2:
        arg[0]=(0.938444792136)*x[0]**o+(-0.809407040724)*x[0]+(-0.963148615762)*x[1]**o+(-0.257000244129)*x[1]
        arg[1]=(-0.791280640039)*x[0]**o+(0.293858623095)*x[0]+(0.964942589784)*x[1]**o+(0.522071353081)*x[1]
        ref[0]=(0.938444792136)*x_ref[0]**o+(-0.809407040724)*x_ref[0]+(-0.963148615762)*x_ref[1]**o+(-0.257000244129)*x_ref[1]
        ref[1]=(-0.791280640039)*x_ref[0]**o+(0.293858623095)*x_ref[0]+(0.964942589784)*x_ref[1]**o+(0.522071353081)*x_ref[1]
      else:
        arg[0]=(0.566110580666)*x[0]**o+(-0.89723913213)*x[0]+(-0.54781629706)*x[1]**o+(-0.0405507445628)*x[1]+(0.435477036501)*x[2]**o+(-0.296404435183)*x[2]
        arg[1]=(-0.436126359475)*x[0]**o+(0.863706244241)*x[0]+(-0.149818409243)*x[1]**o+(-0.654801920098)*x[1]+(-0.281005092809)*x[2]**o+(-0.0257179420057)*x[2]
        ref[0]=(0.566110580666)*x_ref[0]**o+(-0.89723913213)*x_ref[0]+(-0.54781629706)*x_ref[1]**o+(-0.0405507445628)*x_ref[1]+(0.435477036501)*x_ref[2]**o+(-0.296404435183)*x_ref[2]
        ref[1]=(-0.436126359475)*x_ref[0]**o+(0.863706244241)*x_ref[0]+(-0.149818409243)*x_ref[1]**o+(-0.654801920098)*x_ref[1]+(-0.281005092809)*x_ref[2]**o+(-0.0257179420057)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_ReducedFunctionOnContactZero_rank2(self):
      """
      tests interpolate for rank 2 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactZero onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactZero(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5),w)
      ref=Data(0,(4, 5),w_ref)
      if dim==2:
        arg[0,0]=(0.0615713819144)*x[0]**o+(0.325133297443)*x[0]+(-0.529769594257)*x[1]**o+(-0.206126939263)*x[1]
        arg[0,1]=(0.090956391947)*x[0]**o+(0.105841035331)*x[0]+(-0.537033610998)*x[1]**o+(0.429033781495)*x[1]
        arg[0,2]=(-0.568216263139)*x[0]**o+(0.698057796806)*x[0]+(-0.0981614063428)*x[1]**o+(-0.952212332523)*x[1]
        arg[0,3]=(0.128505813477)*x[0]**o+(0.206828305852)*x[0]+(0.938851873532)*x[1]**o+(-0.339234589281)*x[1]
        arg[0,4]=(0.581805477855)*x[0]**o+(-0.621122347153)*x[0]+(-0.601029625565)*x[1]**o+(0.243084366725)*x[1]
        arg[1,0]=(-0.342411991491)*x[0]**o+(0.642434320731)*x[0]+(-0.452657094515)*x[1]**o+(-0.899779109444)*x[1]
        arg[1,1]=(-0.423230603077)*x[0]**o+(0.739600521332)*x[0]+(0.916226480419)*x[1]**o+(0.755264173162)*x[1]
        arg[1,2]=(0.381864221314)*x[0]**o+(0.806402187902)*x[0]+(0.410055116458)*x[1]**o+(-0.264405213472)*x[1]
        arg[1,3]=(0.293401427429)*x[0]**o+(-0.439209675154)*x[0]+(0.299588226145)*x[1]**o+(-0.289652201976)*x[1]
        arg[1,4]=(0.157626502865)*x[0]**o+(0.32264788209)*x[0]+(0.284522039757)*x[1]**o+(-0.460413671191)*x[1]
        arg[2,0]=(0.486148207)*x[0]**o+(0.0366950072658)*x[0]+(0.315299830901)*x[1]**o+(-0.320405042262)*x[1]
        arg[2,1]=(0.207440897542)*x[0]**o+(-0.604659890692)*x[0]+(-0.64841431916)*x[1]**o+(0.0881921209641)*x[1]
        arg[2,2]=(-0.61376840347)*x[0]**o+(0.421837679185)*x[0]+(-0.0445850531916)*x[1]**o+(0.771892018903)*x[1]
        arg[2,3]=(0.632443182263)*x[0]**o+(-0.965297013012)*x[0]+(-0.029677959681)*x[1]**o+(0.598431910551)*x[1]
        arg[2,4]=(-0.457656411094)*x[0]**o+(0.122435925199)*x[0]+(0.772690251937)*x[1]**o+(0.954829922926)*x[1]
        arg[3,0]=(0.929112925461)*x[0]**o+(-0.35689901655)*x[0]+(-0.139192866782)*x[1]**o+(-0.677883436952)*x[1]
        arg[3,1]=(-0.410318721103)*x[0]**o+(0.370830312699)*x[0]+(0.429011619434)*x[1]**o+(0.81780420854)*x[1]
        arg[3,2]=(0.312456682985)*x[0]**o+(0.526447664999)*x[0]+(0.763610570932)*x[1]**o+(-0.631887901017)*x[1]
        arg[3,3]=(0.749121567955)*x[0]**o+(0.168525340464)*x[0]+(-0.838409619895)*x[1]**o+(0.912745137098)*x[1]
        arg[3,4]=(-0.665336604643)*x[0]**o+(0.638293226479)*x[0]+(0.515957084712)*x[1]**o+(-0.261426720977)*x[1]
        ref[0,0]=(0.0615713819144)*x_ref[0]**o+(0.325133297443)*x_ref[0]+(-0.529769594257)*x_ref[1]**o+(-0.206126939263)*x_ref[1]
        ref[0,1]=(0.090956391947)*x_ref[0]**o+(0.105841035331)*x_ref[0]+(-0.537033610998)*x_ref[1]**o+(0.429033781495)*x_ref[1]
        ref[0,2]=(-0.568216263139)*x_ref[0]**o+(0.698057796806)*x_ref[0]+(-0.0981614063428)*x_ref[1]**o+(-0.952212332523)*x_ref[1]
        ref[0,3]=(0.128505813477)*x_ref[0]**o+(0.206828305852)*x_ref[0]+(0.938851873532)*x_ref[1]**o+(-0.339234589281)*x_ref[1]
        ref[0,4]=(0.581805477855)*x_ref[0]**o+(-0.621122347153)*x_ref[0]+(-0.601029625565)*x_ref[1]**o+(0.243084366725)*x_ref[1]
        ref[1,0]=(-0.342411991491)*x_ref[0]**o+(0.642434320731)*x_ref[0]+(-0.452657094515)*x_ref[1]**o+(-0.899779109444)*x_ref[1]
        ref[1,1]=(-0.423230603077)*x_ref[0]**o+(0.739600521332)*x_ref[0]+(0.916226480419)*x_ref[1]**o+(0.755264173162)*x_ref[1]
        ref[1,2]=(0.381864221314)*x_ref[0]**o+(0.806402187902)*x_ref[0]+(0.410055116458)*x_ref[1]**o+(-0.264405213472)*x_ref[1]
        ref[1,3]=(0.293401427429)*x_ref[0]**o+(-0.439209675154)*x_ref[0]+(0.299588226145)*x_ref[1]**o+(-0.289652201976)*x_ref[1]
        ref[1,4]=(0.157626502865)*x_ref[0]**o+(0.32264788209)*x_ref[0]+(0.284522039757)*x_ref[1]**o+(-0.460413671191)*x_ref[1]
        ref[2,0]=(0.486148207)*x_ref[0]**o+(0.0366950072658)*x_ref[0]+(0.315299830901)*x_ref[1]**o+(-0.320405042262)*x_ref[1]
        ref[2,1]=(0.207440897542)*x_ref[0]**o+(-0.604659890692)*x_ref[0]+(-0.64841431916)*x_ref[1]**o+(0.0881921209641)*x_ref[1]
        ref[2,2]=(-0.61376840347)*x_ref[0]**o+(0.421837679185)*x_ref[0]+(-0.0445850531916)*x_ref[1]**o+(0.771892018903)*x_ref[1]
        ref[2,3]=(0.632443182263)*x_ref[0]**o+(-0.965297013012)*x_ref[0]+(-0.029677959681)*x_ref[1]**o+(0.598431910551)*x_ref[1]
        ref[2,4]=(-0.457656411094)*x_ref[0]**o+(0.122435925199)*x_ref[0]+(0.772690251937)*x_ref[1]**o+(0.954829922926)*x_ref[1]
        ref[3,0]=(0.929112925461)*x_ref[0]**o+(-0.35689901655)*x_ref[0]+(-0.139192866782)*x_ref[1]**o+(-0.677883436952)*x_ref[1]
        ref[3,1]=(-0.410318721103)*x_ref[0]**o+(0.370830312699)*x_ref[0]+(0.429011619434)*x_ref[1]**o+(0.81780420854)*x_ref[1]
        ref[3,2]=(0.312456682985)*x_ref[0]**o+(0.526447664999)*x_ref[0]+(0.763610570932)*x_ref[1]**o+(-0.631887901017)*x_ref[1]
        ref[3,3]=(0.749121567955)*x_ref[0]**o+(0.168525340464)*x_ref[0]+(-0.838409619895)*x_ref[1]**o+(0.912745137098)*x_ref[1]
        ref[3,4]=(-0.665336604643)*x_ref[0]**o+(0.638293226479)*x_ref[0]+(0.515957084712)*x_ref[1]**o+(-0.261426720977)*x_ref[1]
      else:
        arg[0,0]=(-0.330165086386)*x[0]**o+(0.60381119065)*x[0]+(0.981018763011)*x[1]**o+(0.0487965999608)*x[1]+(-0.389835388349)*x[2]**o+(0.111199857376)*x[2]
        arg[0,1]=(0.518239033178)*x[0]**o+(0.60358784684)*x[0]+(-0.476990159655)*x[1]**o+(-0.93871565843)*x[1]+(-0.869845295741)*x[2]**o+(0.391409745802)*x[2]
        arg[0,2]=(0.0927396115951)*x[0]**o+(0.546122822928)*x[0]+(-0.557424056733)*x[1]**o+(0.0623374966436)*x[1]+(-0.414595153536)*x[2]**o+(-0.196069073366)*x[2]
        arg[0,3]=(-0.678432580882)*x[0]**o+(0.281850265141)*x[0]+(-0.323978960602)*x[1]**o+(0.655558769921)*x[1]+(-0.295747032089)*x[2]**o+(-0.152719838982)*x[2]
        arg[0,4]=(-0.904177669203)*x[0]**o+(0.750395234205)*x[0]+(0.267431783461)*x[1]**o+(-0.781406235306)*x[1]+(0.856906655757)*x[2]**o+(-0.875554940224)*x[2]
        arg[1,0]=(-0.243144846569)*x[0]**o+(0.970584922467)*x[0]+(-0.977250652473)*x[1]**o+(0.0239563255869)*x[1]+(-0.769423602851)*x[2]**o+(-0.433896361294)*x[2]
        arg[1,1]=(0.31104031046)*x[0]**o+(0.899859709831)*x[0]+(0.692351561919)*x[1]**o+(-0.0838870550002)*x[1]+(-0.276105616798)*x[2]**o+(0.115501348248)*x[2]
        arg[1,2]=(0.56702114595)*x[0]**o+(0.00240284646131)*x[0]+(-0.676314888884)*x[1]**o+(-0.678204346328)*x[1]+(0.472697088984)*x[2]**o+(-0.493949191712)*x[2]
        arg[1,3]=(-0.242548554067)*x[0]**o+(0.980927507053)*x[0]+(-0.955706390436)*x[1]**o+(0.347031550126)*x[1]+(0.234352268238)*x[2]**o+(0.321126345333)*x[2]
        arg[1,4]=(-0.390890566228)*x[0]**o+(-0.958497753144)*x[0]+(0.316397716043)*x[1]**o+(0.952289821769)*x[1]+(-0.027182029514)*x[2]**o+(-0.454422377872)*x[2]
        arg[2,0]=(0.295865166318)*x[0]**o+(-0.301129718468)*x[0]+(-0.748135755269)*x[1]**o+(-0.762413814085)*x[1]+(0.572270067591)*x[2]**o+(-0.393847032684)*x[2]
        arg[2,1]=(0.20229063622)*x[0]**o+(-0.768444028163)*x[0]+(0.378669775381)*x[1]**o+(-0.703384410944)*x[1]+(0.355653859396)*x[2]**o+(-0.228168949639)*x[2]
        arg[2,2]=(-0.31725134742)*x[0]**o+(0.525802867508)*x[0]+(0.13375126758)*x[1]**o+(-0.673551491186)*x[1]+(0.498905883728)*x[2]**o+(-0.102794461221)*x[2]
        arg[2,3]=(-0.396813838168)*x[0]**o+(0.0892605807843)*x[0]+(0.244133298111)*x[1]**o+(0.310900636168)*x[1]+(-0.256807063422)*x[2]**o+(0.565370697688)*x[2]
        arg[2,4]=(0.42011835881)*x[0]**o+(0.439847594375)*x[0]+(0.0731897272006)*x[1]**o+(-0.373911054753)*x[1]+(-0.50313315963)*x[2]**o+(0.356362677601)*x[2]
        arg[3,0]=(-0.444672845607)*x[0]**o+(-0.978424347535)*x[0]+(0.421699748442)*x[1]**o+(-0.0111798084161)*x[1]+(0.206042685881)*x[2]**o+(0.990523831215)*x[2]
        arg[3,1]=(0.942429627068)*x[0]**o+(-0.769989484282)*x[0]+(-0.879916738075)*x[1]**o+(-0.746448112871)*x[1]+(0.280824650861)*x[2]**o+(0.0632084823022)*x[2]
        arg[3,2]=(0.281070481177)*x[0]**o+(0.628020001876)*x[0]+(0.347767303962)*x[1]**o+(-0.276761252142)*x[1]+(0.950345260734)*x[2]**o+(-0.00148186335195)*x[2]
        arg[3,3]=(-0.37590210465)*x[0]**o+(0.034147350068)*x[0]+(0.245173418336)*x[1]**o+(0.0564051241759)*x[1]+(-0.157928106127)*x[2]**o+(-0.611677441847)*x[2]
        arg[3,4]=(-0.279066532276)*x[0]**o+(0.316033119063)*x[0]+(0.655948999472)*x[1]**o+(-0.520504180943)*x[1]+(0.531738158819)*x[2]**o+(0.899490827443)*x[2]
        ref[0,0]=(-0.330165086386)*x_ref[0]**o+(0.60381119065)*x_ref[0]+(0.981018763011)*x_ref[1]**o+(0.0487965999608)*x_ref[1]+(-0.389835388349)*x_ref[2]**o+(0.111199857376)*x_ref[2]
        ref[0,1]=(0.518239033178)*x_ref[0]**o+(0.60358784684)*x_ref[0]+(-0.476990159655)*x_ref[1]**o+(-0.93871565843)*x_ref[1]+(-0.869845295741)*x_ref[2]**o+(0.391409745802)*x_ref[2]
        ref[0,2]=(0.0927396115951)*x_ref[0]**o+(0.546122822928)*x_ref[0]+(-0.557424056733)*x_ref[1]**o+(0.0623374966436)*x_ref[1]+(-0.414595153536)*x_ref[2]**o+(-0.196069073366)*x_ref[2]
        ref[0,3]=(-0.678432580882)*x_ref[0]**o+(0.281850265141)*x_ref[0]+(-0.323978960602)*x_ref[1]**o+(0.655558769921)*x_ref[1]+(-0.295747032089)*x_ref[2]**o+(-0.152719838982)*x_ref[2]
        ref[0,4]=(-0.904177669203)*x_ref[0]**o+(0.750395234205)*x_ref[0]+(0.267431783461)*x_ref[1]**o+(-0.781406235306)*x_ref[1]+(0.856906655757)*x_ref[2]**o+(-0.875554940224)*x_ref[2]
        ref[1,0]=(-0.243144846569)*x_ref[0]**o+(0.970584922467)*x_ref[0]+(-0.977250652473)*x_ref[1]**o+(0.0239563255869)*x_ref[1]+(-0.769423602851)*x_ref[2]**o+(-0.433896361294)*x_ref[2]
        ref[1,1]=(0.31104031046)*x_ref[0]**o+(0.899859709831)*x_ref[0]+(0.692351561919)*x_ref[1]**o+(-0.0838870550002)*x_ref[1]+(-0.276105616798)*x_ref[2]**o+(0.115501348248)*x_ref[2]
        ref[1,2]=(0.56702114595)*x_ref[0]**o+(0.00240284646131)*x_ref[0]+(-0.676314888884)*x_ref[1]**o+(-0.678204346328)*x_ref[1]+(0.472697088984)*x_ref[2]**o+(-0.493949191712)*x_ref[2]
        ref[1,3]=(-0.242548554067)*x_ref[0]**o+(0.980927507053)*x_ref[0]+(-0.955706390436)*x_ref[1]**o+(0.347031550126)*x_ref[1]+(0.234352268238)*x_ref[2]**o+(0.321126345333)*x_ref[2]
        ref[1,4]=(-0.390890566228)*x_ref[0]**o+(-0.958497753144)*x_ref[0]+(0.316397716043)*x_ref[1]**o+(0.952289821769)*x_ref[1]+(-0.027182029514)*x_ref[2]**o+(-0.454422377872)*x_ref[2]
        ref[2,0]=(0.295865166318)*x_ref[0]**o+(-0.301129718468)*x_ref[0]+(-0.748135755269)*x_ref[1]**o+(-0.762413814085)*x_ref[1]+(0.572270067591)*x_ref[2]**o+(-0.393847032684)*x_ref[2]
        ref[2,1]=(0.20229063622)*x_ref[0]**o+(-0.768444028163)*x_ref[0]+(0.378669775381)*x_ref[1]**o+(-0.703384410944)*x_ref[1]+(0.355653859396)*x_ref[2]**o+(-0.228168949639)*x_ref[2]
        ref[2,2]=(-0.31725134742)*x_ref[0]**o+(0.525802867508)*x_ref[0]+(0.13375126758)*x_ref[1]**o+(-0.673551491186)*x_ref[1]+(0.498905883728)*x_ref[2]**o+(-0.102794461221)*x_ref[2]
        ref[2,3]=(-0.396813838168)*x_ref[0]**o+(0.0892605807843)*x_ref[0]+(0.244133298111)*x_ref[1]**o+(0.310900636168)*x_ref[1]+(-0.256807063422)*x_ref[2]**o+(0.565370697688)*x_ref[2]
        ref[2,4]=(0.42011835881)*x_ref[0]**o+(0.439847594375)*x_ref[0]+(0.0731897272006)*x_ref[1]**o+(-0.373911054753)*x_ref[1]+(-0.50313315963)*x_ref[2]**o+(0.356362677601)*x_ref[2]
        ref[3,0]=(-0.444672845607)*x_ref[0]**o+(-0.978424347535)*x_ref[0]+(0.421699748442)*x_ref[1]**o+(-0.0111798084161)*x_ref[1]+(0.206042685881)*x_ref[2]**o+(0.990523831215)*x_ref[2]
        ref[3,1]=(0.942429627068)*x_ref[0]**o+(-0.769989484282)*x_ref[0]+(-0.879916738075)*x_ref[1]**o+(-0.746448112871)*x_ref[1]+(0.280824650861)*x_ref[2]**o+(0.0632084823022)*x_ref[2]
        ref[3,2]=(0.281070481177)*x_ref[0]**o+(0.628020001876)*x_ref[0]+(0.347767303962)*x_ref[1]**o+(-0.276761252142)*x_ref[1]+(0.950345260734)*x_ref[2]**o+(-0.00148186335195)*x_ref[2]
        ref[3,3]=(-0.37590210465)*x_ref[0]**o+(0.034147350068)*x_ref[0]+(0.245173418336)*x_ref[1]**o+(0.0564051241759)*x_ref[1]+(-0.157928106127)*x_ref[2]**o+(-0.611677441847)*x_ref[2]
        ref[3,4]=(-0.279066532276)*x_ref[0]**o+(0.316033119063)*x_ref[0]+(0.655948999472)*x_ref[1]**o+(-0.520504180943)*x_ref[1]+(0.531738158819)*x_ref[2]**o+(0.899490827443)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_ReducedFunctionOnContactZero_rank3(self):
      """
      tests interpolate for rank 3 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactZero onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactZero(self.domain)
      x=w.getX()
      arg=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2),w_ref)
      if dim==2:
        arg[0,0,0]=(0.415438516191)*x[0]**o+(0.374659323173)*x[0]+(0.688470746611)*x[1]**o+(0.498183692176)*x[1]
        arg[0,0,1]=(-0.846725329522)*x[0]**o+(0.414376816326)*x[0]+(0.651609661615)*x[1]**o+(0.357606396667)*x[1]
        arg[0,1,0]=(0.728029172023)*x[0]**o+(0.975728459112)*x[0]+(-0.658211371639)*x[1]**o+(0.411894140351)*x[1]
        arg[0,1,1]=(0.807350281817)*x[0]**o+(-0.369755733522)*x[0]+(-0.519483139314)*x[1]**o+(-0.709353537648)*x[1]
        arg[1,0,0]=(0.0588259329479)*x[0]**o+(0.218362870441)*x[0]+(-0.0279239318934)*x[1]**o+(-0.75783635329)*x[1]
        arg[1,0,1]=(-0.429573558505)*x[0]**o+(-0.339046134618)*x[0]+(0.412777825743)*x[1]**o+(-0.306607333437)*x[1]
        arg[1,1,0]=(0.0222661407543)*x[0]**o+(0.927923674781)*x[0]+(-0.767468570033)*x[1]**o+(-0.0659150590367)*x[1]
        arg[1,1,1]=(0.372811733787)*x[0]**o+(-0.286500260061)*x[0]+(-0.807504375569)*x[1]**o+(-0.992850148234)*x[1]
        arg[2,0,0]=(0.368229828596)*x[0]**o+(0.672540290379)*x[0]+(-0.760462595095)*x[1]**o+(-0.761242492425)*x[1]
        arg[2,0,1]=(0.306249523133)*x[0]**o+(0.769928924717)*x[0]+(0.610293800742)*x[1]**o+(-0.221996271153)*x[1]
        arg[2,1,0]=(-0.909148832991)*x[0]**o+(-0.0293038712703)*x[0]+(-0.161707473269)*x[1]**o+(-0.534345319122)*x[1]
        arg[2,1,1]=(-0.146432487737)*x[0]**o+(0.93729266958)*x[0]+(-0.561919238789)*x[1]**o+(-0.449440501171)*x[1]
        arg[3,0,0]=(0.411711661035)*x[0]**o+(-0.0699878047274)*x[0]+(-0.0227688826645)*x[1]**o+(-0.939787633547)*x[1]
        arg[3,0,1]=(-0.82108820245)*x[0]**o+(0.225501168847)*x[0]+(0.622921674682)*x[1]**o+(0.61092413094)*x[1]
        arg[3,1,0]=(0.758116155912)*x[0]**o+(-0.117316615447)*x[0]+(0.0702245119124)*x[1]**o+(-0.275114508561)*x[1]
        arg[3,1,1]=(-0.32568550821)*x[0]**o+(-0.280144846326)*x[0]+(0.194971187876)*x[1]**o+(-0.728447244746)*x[1]
        arg[4,0,0]=(0.880042622981)*x[0]**o+(-0.395684607563)*x[0]+(-0.563304061265)*x[1]**o+(0.968663606772)*x[1]
        arg[4,0,1]=(-0.515392712458)*x[0]**o+(0.820857647342)*x[0]+(-0.476289639495)*x[1]**o+(-0.478062589609)*x[1]
        arg[4,1,0]=(0.534322425269)*x[0]**o+(0.975618695265)*x[0]+(-0.195764005045)*x[1]**o+(0.561996200643)*x[1]
        arg[4,1,1]=(-0.906243138672)*x[0]**o+(-0.580985047294)*x[0]+(-0.094609405051)*x[1]**o+(0.734326711649)*x[1]
        arg[5,0,0]=(-0.0655516121632)*x[0]**o+(0.785493123174)*x[0]+(-0.760068333675)*x[1]**o+(0.693869247554)*x[1]
        arg[5,0,1]=(-0.931019838892)*x[0]**o+(0.80958291621)*x[0]+(-0.590591206111)*x[1]**o+(-0.800900346601)*x[1]
        arg[5,1,0]=(0.448779048952)*x[0]**o+(-0.793450503268)*x[0]+(0.161067403301)*x[1]**o+(0.432054043544)*x[1]
        arg[5,1,1]=(0.0423607361972)*x[0]**o+(-0.664958045702)*x[0]+(-0.174728816918)*x[1]**o+(-0.394390068699)*x[1]
        ref[0,0,0]=(0.415438516191)*x_ref[0]**o+(0.374659323173)*x_ref[0]+(0.688470746611)*x_ref[1]**o+(0.498183692176)*x_ref[1]
        ref[0,0,1]=(-0.846725329522)*x_ref[0]**o+(0.414376816326)*x_ref[0]+(0.651609661615)*x_ref[1]**o+(0.357606396667)*x_ref[1]
        ref[0,1,0]=(0.728029172023)*x_ref[0]**o+(0.975728459112)*x_ref[0]+(-0.658211371639)*x_ref[1]**o+(0.411894140351)*x_ref[1]
        ref[0,1,1]=(0.807350281817)*x_ref[0]**o+(-0.369755733522)*x_ref[0]+(-0.519483139314)*x_ref[1]**o+(-0.709353537648)*x_ref[1]
        ref[1,0,0]=(0.0588259329479)*x_ref[0]**o+(0.218362870441)*x_ref[0]+(-0.0279239318934)*x_ref[1]**o+(-0.75783635329)*x_ref[1]
        ref[1,0,1]=(-0.429573558505)*x_ref[0]**o+(-0.339046134618)*x_ref[0]+(0.412777825743)*x_ref[1]**o+(-0.306607333437)*x_ref[1]
        ref[1,1,0]=(0.0222661407543)*x_ref[0]**o+(0.927923674781)*x_ref[0]+(-0.767468570033)*x_ref[1]**o+(-0.0659150590367)*x_ref[1]
        ref[1,1,1]=(0.372811733787)*x_ref[0]**o+(-0.286500260061)*x_ref[0]+(-0.807504375569)*x_ref[1]**o+(-0.992850148234)*x_ref[1]
        ref[2,0,0]=(0.368229828596)*x_ref[0]**o+(0.672540290379)*x_ref[0]+(-0.760462595095)*x_ref[1]**o+(-0.761242492425)*x_ref[1]
        ref[2,0,1]=(0.306249523133)*x_ref[0]**o+(0.769928924717)*x_ref[0]+(0.610293800742)*x_ref[1]**o+(-0.221996271153)*x_ref[1]
        ref[2,1,0]=(-0.909148832991)*x_ref[0]**o+(-0.0293038712703)*x_ref[0]+(-0.161707473269)*x_ref[1]**o+(-0.534345319122)*x_ref[1]
        ref[2,1,1]=(-0.146432487737)*x_ref[0]**o+(0.93729266958)*x_ref[0]+(-0.561919238789)*x_ref[1]**o+(-0.449440501171)*x_ref[1]
        ref[3,0,0]=(0.411711661035)*x_ref[0]**o+(-0.0699878047274)*x_ref[0]+(-0.0227688826645)*x_ref[1]**o+(-0.939787633547)*x_ref[1]
        ref[3,0,1]=(-0.82108820245)*x_ref[0]**o+(0.225501168847)*x_ref[0]+(0.622921674682)*x_ref[1]**o+(0.61092413094)*x_ref[1]
        ref[3,1,0]=(0.758116155912)*x_ref[0]**o+(-0.117316615447)*x_ref[0]+(0.0702245119124)*x_ref[1]**o+(-0.275114508561)*x_ref[1]
        ref[3,1,1]=(-0.32568550821)*x_ref[0]**o+(-0.280144846326)*x_ref[0]+(0.194971187876)*x_ref[1]**o+(-0.728447244746)*x_ref[1]
        ref[4,0,0]=(0.880042622981)*x_ref[0]**o+(-0.395684607563)*x_ref[0]+(-0.563304061265)*x_ref[1]**o+(0.968663606772)*x_ref[1]
        ref[4,0,1]=(-0.515392712458)*x_ref[0]**o+(0.820857647342)*x_ref[0]+(-0.476289639495)*x_ref[1]**o+(-0.478062589609)*x_ref[1]
        ref[4,1,0]=(0.534322425269)*x_ref[0]**o+(0.975618695265)*x_ref[0]+(-0.195764005045)*x_ref[1]**o+(0.561996200643)*x_ref[1]
        ref[4,1,1]=(-0.906243138672)*x_ref[0]**o+(-0.580985047294)*x_ref[0]+(-0.094609405051)*x_ref[1]**o+(0.734326711649)*x_ref[1]
        ref[5,0,0]=(-0.0655516121632)*x_ref[0]**o+(0.785493123174)*x_ref[0]+(-0.760068333675)*x_ref[1]**o+(0.693869247554)*x_ref[1]
        ref[5,0,1]=(-0.931019838892)*x_ref[0]**o+(0.80958291621)*x_ref[0]+(-0.590591206111)*x_ref[1]**o+(-0.800900346601)*x_ref[1]
        ref[5,1,0]=(0.448779048952)*x_ref[0]**o+(-0.793450503268)*x_ref[0]+(0.161067403301)*x_ref[1]**o+(0.432054043544)*x_ref[1]
        ref[5,1,1]=(0.0423607361972)*x_ref[0]**o+(-0.664958045702)*x_ref[0]+(-0.174728816918)*x_ref[1]**o+(-0.394390068699)*x_ref[1]
      else:
        arg[0,0,0]=(0.34355250678)*x[0]**o+(0.554590991175)*x[0]+(0.00673862595967)*x[1]**o+(-0.405764224977)*x[1]+(0.0865564883661)*x[2]**o+(0.102294137607)*x[2]
        arg[0,0,1]=(0.855234460708)*x[0]**o+(-0.710901486015)*x[0]+(0.546393138087)*x[1]**o+(0.847113479239)*x[1]+(-0.650701604407)*x[2]**o+(-0.914755510262)*x[2]
        arg[0,1,0]=(0.252544568503)*x[0]**o+(-0.473471976093)*x[0]+(-0.587315274764)*x[1]**o+(-0.269940894375)*x[1]+(0.360340735507)*x[2]**o+(-0.745440004827)*x[2]
        arg[0,1,1]=(0.11967506555)*x[0]**o+(0.696836453942)*x[0]+(-0.871162187396)*x[1]**o+(-0.315170111067)*x[1]+(0.232066109266)*x[2]**o+(-0.464730524838)*x[2]
        arg[1,0,0]=(-0.790535662997)*x[0]**o+(-0.964779159884)*x[0]+(-0.689937881374)*x[1]**o+(0.422799231262)*x[1]+(-0.97315874804)*x[2]**o+(-0.532103115205)*x[2]
        arg[1,0,1]=(-0.0487589554945)*x[0]**o+(-0.622735713658)*x[0]+(-0.520842613265)*x[1]**o+(0.803711422599)*x[1]+(0.440648875556)*x[2]**o+(0.98786672574)*x[2]
        arg[1,1,0]=(0.811189977538)*x[0]**o+(0.0175150486788)*x[0]+(-0.124328899976)*x[1]**o+(-0.942398680503)*x[1]+(-0.149270209895)*x[2]**o+(-0.853084407696)*x[2]
        arg[1,1,1]=(0.688378159744)*x[0]**o+(0.408766927878)*x[0]+(0.834128919609)*x[1]**o+(0.253605923886)*x[1]+(0.424992121054)*x[2]**o+(0.21016117963)*x[2]
        arg[2,0,0]=(0.277469067087)*x[0]**o+(0.539821557611)*x[0]+(0.348679402521)*x[1]**o+(-0.469772506537)*x[1]+(0.454583229137)*x[2]**o+(0.653681481133)*x[2]
        arg[2,0,1]=(-0.934067178483)*x[0]**o+(-0.172735098497)*x[0]+(-0.0347256850948)*x[1]**o+(-0.459971424944)*x[1]+(0.932661853019)*x[2]**o+(-0.186822590955)*x[2]
        arg[2,1,0]=(0.71698574321)*x[0]**o+(-0.197244831261)*x[0]+(0.183419576182)*x[1]**o+(-0.871451480853)*x[1]+(-0.349396269955)*x[2]**o+(0.095909246936)*x[2]
        arg[2,1,1]=(-0.00972985962106)*x[0]**o+(-0.919204152811)*x[0]+(-0.751744404365)*x[1]**o+(0.377096436852)*x[1]+(-0.924228885362)*x[2]**o+(0.451053813959)*x[2]
        arg[3,0,0]=(0.951119968373)*x[0]**o+(-0.0680027504528)*x[0]+(-0.113660990287)*x[1]**o+(-0.428999453003)*x[1]+(0.171804544103)*x[2]**o+(0.677635036297)*x[2]
        arg[3,0,1]=(0.62534864581)*x[0]**o+(0.389496246771)*x[0]+(0.621295327948)*x[1]**o+(-0.684165739023)*x[1]+(0.258637161492)*x[2]**o+(0.0441460764076)*x[2]
        arg[3,1,0]=(0.0348922721145)*x[0]**o+(0.091201286767)*x[0]+(0.421079169132)*x[1]**o+(-0.616504862591)*x[1]+(0.563927673176)*x[2]**o+(-0.183993394822)*x[2]
        arg[3,1,1]=(0.657959811783)*x[0]**o+(0.229729671382)*x[0]+(0.549853155207)*x[1]**o+(-0.847412930565)*x[1]+(-0.0780682500337)*x[2]**o+(0.574435563406)*x[2]
        arg[4,0,0]=(-0.679988515344)*x[0]**o+(0.0517331630343)*x[0]+(-0.160281064879)*x[1]**o+(-0.0672405944261)*x[1]+(-0.118968444607)*x[2]**o+(0.00980065834148)*x[2]
        arg[4,0,1]=(-0.150457972951)*x[0]**o+(0.671586511406)*x[0]+(-0.844112675259)*x[1]**o+(0.42766255101)*x[1]+(-0.22449721018)*x[2]**o+(-0.150416151059)*x[2]
        arg[4,1,0]=(0.0768992597753)*x[0]**o+(0.0139605476692)*x[0]+(-0.831269554863)*x[1]**o+(-0.279339344511)*x[1]+(0.387318821185)*x[2]**o+(0.967687121678)*x[2]
        arg[4,1,1]=(-0.670585948598)*x[0]**o+(0.422715818654)*x[0]+(0.623702390878)*x[1]**o+(0.922646081646)*x[1]+(-0.714191287039)*x[2]**o+(-0.99018084869)*x[2]
        arg[5,0,0]=(0.100273819257)*x[0]**o+(-0.769422291863)*x[0]+(-0.152611272068)*x[1]**o+(-0.396798631305)*x[1]+(0.502251047462)*x[2]**o+(0.993791216449)*x[2]
        arg[5,0,1]=(-0.0222540900657)*x[0]**o+(-0.339810353987)*x[0]+(0.241665078696)*x[1]**o+(0.111020568564)*x[1]+(0.298147316931)*x[2]**o+(0.391278660594)*x[2]
        arg[5,1,0]=(0.828297599555)*x[0]**o+(0.237678875371)*x[0]+(0.924551126274)*x[1]**o+(-0.663695049485)*x[1]+(-0.547123493132)*x[2]**o+(-0.788635745118)*x[2]
        arg[5,1,1]=(0.0726334370982)*x[0]**o+(-0.875688492957)*x[0]+(0.473726235108)*x[1]**o+(0.023025596151)*x[1]+(0.827449396402)*x[2]**o+(-0.154698941604)*x[2]
        ref[0,0,0]=(0.34355250678)*x_ref[0]**o+(0.554590991175)*x_ref[0]+(0.00673862595967)*x_ref[1]**o+(-0.405764224977)*x_ref[1]+(0.0865564883661)*x_ref[2]**o+(0.102294137607)*x_ref[2]
        ref[0,0,1]=(0.855234460708)*x_ref[0]**o+(-0.710901486015)*x_ref[0]+(0.546393138087)*x_ref[1]**o+(0.847113479239)*x_ref[1]+(-0.650701604407)*x_ref[2]**o+(-0.914755510262)*x_ref[2]
        ref[0,1,0]=(0.252544568503)*x_ref[0]**o+(-0.473471976093)*x_ref[0]+(-0.587315274764)*x_ref[1]**o+(-0.269940894375)*x_ref[1]+(0.360340735507)*x_ref[2]**o+(-0.745440004827)*x_ref[2]
        ref[0,1,1]=(0.11967506555)*x_ref[0]**o+(0.696836453942)*x_ref[0]+(-0.871162187396)*x_ref[1]**o+(-0.315170111067)*x_ref[1]+(0.232066109266)*x_ref[2]**o+(-0.464730524838)*x_ref[2]
        ref[1,0,0]=(-0.790535662997)*x_ref[0]**o+(-0.964779159884)*x_ref[0]+(-0.689937881374)*x_ref[1]**o+(0.422799231262)*x_ref[1]+(-0.97315874804)*x_ref[2]**o+(-0.532103115205)*x_ref[2]
        ref[1,0,1]=(-0.0487589554945)*x_ref[0]**o+(-0.622735713658)*x_ref[0]+(-0.520842613265)*x_ref[1]**o+(0.803711422599)*x_ref[1]+(0.440648875556)*x_ref[2]**o+(0.98786672574)*x_ref[2]
        ref[1,1,0]=(0.811189977538)*x_ref[0]**o+(0.0175150486788)*x_ref[0]+(-0.124328899976)*x_ref[1]**o+(-0.942398680503)*x_ref[1]+(-0.149270209895)*x_ref[2]**o+(-0.853084407696)*x_ref[2]
        ref[1,1,1]=(0.688378159744)*x_ref[0]**o+(0.408766927878)*x_ref[0]+(0.834128919609)*x_ref[1]**o+(0.253605923886)*x_ref[1]+(0.424992121054)*x_ref[2]**o+(0.21016117963)*x_ref[2]
        ref[2,0,0]=(0.277469067087)*x_ref[0]**o+(0.539821557611)*x_ref[0]+(0.348679402521)*x_ref[1]**o+(-0.469772506537)*x_ref[1]+(0.454583229137)*x_ref[2]**o+(0.653681481133)*x_ref[2]
        ref[2,0,1]=(-0.934067178483)*x_ref[0]**o+(-0.172735098497)*x_ref[0]+(-0.0347256850948)*x_ref[1]**o+(-0.459971424944)*x_ref[1]+(0.932661853019)*x_ref[2]**o+(-0.186822590955)*x_ref[2]
        ref[2,1,0]=(0.71698574321)*x_ref[0]**o+(-0.197244831261)*x_ref[0]+(0.183419576182)*x_ref[1]**o+(-0.871451480853)*x_ref[1]+(-0.349396269955)*x_ref[2]**o+(0.095909246936)*x_ref[2]
        ref[2,1,1]=(-0.00972985962106)*x_ref[0]**o+(-0.919204152811)*x_ref[0]+(-0.751744404365)*x_ref[1]**o+(0.377096436852)*x_ref[1]+(-0.924228885362)*x_ref[2]**o+(0.451053813959)*x_ref[2]
        ref[3,0,0]=(0.951119968373)*x_ref[0]**o+(-0.0680027504528)*x_ref[0]+(-0.113660990287)*x_ref[1]**o+(-0.428999453003)*x_ref[1]+(0.171804544103)*x_ref[2]**o+(0.677635036297)*x_ref[2]
        ref[3,0,1]=(0.62534864581)*x_ref[0]**o+(0.389496246771)*x_ref[0]+(0.621295327948)*x_ref[1]**o+(-0.684165739023)*x_ref[1]+(0.258637161492)*x_ref[2]**o+(0.0441460764076)*x_ref[2]
        ref[3,1,0]=(0.0348922721145)*x_ref[0]**o+(0.091201286767)*x_ref[0]+(0.421079169132)*x_ref[1]**o+(-0.616504862591)*x_ref[1]+(0.563927673176)*x_ref[2]**o+(-0.183993394822)*x_ref[2]
        ref[3,1,1]=(0.657959811783)*x_ref[0]**o+(0.229729671382)*x_ref[0]+(0.549853155207)*x_ref[1]**o+(-0.847412930565)*x_ref[1]+(-0.0780682500337)*x_ref[2]**o+(0.574435563406)*x_ref[2]
        ref[4,0,0]=(-0.679988515344)*x_ref[0]**o+(0.0517331630343)*x_ref[0]+(-0.160281064879)*x_ref[1]**o+(-0.0672405944261)*x_ref[1]+(-0.118968444607)*x_ref[2]**o+(0.00980065834148)*x_ref[2]
        ref[4,0,1]=(-0.150457972951)*x_ref[0]**o+(0.671586511406)*x_ref[0]+(-0.844112675259)*x_ref[1]**o+(0.42766255101)*x_ref[1]+(-0.22449721018)*x_ref[2]**o+(-0.150416151059)*x_ref[2]
        ref[4,1,0]=(0.0768992597753)*x_ref[0]**o+(0.0139605476692)*x_ref[0]+(-0.831269554863)*x_ref[1]**o+(-0.279339344511)*x_ref[1]+(0.387318821185)*x_ref[2]**o+(0.967687121678)*x_ref[2]
        ref[4,1,1]=(-0.670585948598)*x_ref[0]**o+(0.422715818654)*x_ref[0]+(0.623702390878)*x_ref[1]**o+(0.922646081646)*x_ref[1]+(-0.714191287039)*x_ref[2]**o+(-0.99018084869)*x_ref[2]
        ref[5,0,0]=(0.100273819257)*x_ref[0]**o+(-0.769422291863)*x_ref[0]+(-0.152611272068)*x_ref[1]**o+(-0.396798631305)*x_ref[1]+(0.502251047462)*x_ref[2]**o+(0.993791216449)*x_ref[2]
        ref[5,0,1]=(-0.0222540900657)*x_ref[0]**o+(-0.339810353987)*x_ref[0]+(0.241665078696)*x_ref[1]**o+(0.111020568564)*x_ref[1]+(0.298147316931)*x_ref[2]**o+(0.391278660594)*x_ref[2]
        ref[5,1,0]=(0.828297599555)*x_ref[0]**o+(0.237678875371)*x_ref[0]+(0.924551126274)*x_ref[1]**o+(-0.663695049485)*x_ref[1]+(-0.547123493132)*x_ref[2]**o+(-0.788635745118)*x_ref[2]
        ref[5,1,1]=(0.0726334370982)*x_ref[0]**o+(-0.875688492957)*x_ref[0]+(0.473726235108)*x_ref[1]**o+(0.023025596151)*x_ref[1]+(0.827449396402)*x_ref[2]**o+(-0.154698941604)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_ReducedFunctionOnContactZero_rank4(self):
      """
      tests interpolate for rank 4 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactZero onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactZero(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5, 3, 2),w)
      ref=Data(0,(4, 5, 3, 2),w_ref)
      if dim==2:
        arg[0,0,0,0]=(-0.0338299192888)*x[0]**o+(-0.114906274359)*x[0]+(0.15010339398)*x[1]**o+(0.625295991328)*x[1]
        arg[0,0,0,1]=(-0.1311901039)*x[0]**o+(-0.551712861217)*x[0]+(-0.287411404506)*x[1]**o+(0.0242820629572)*x[1]
        arg[0,0,1,0]=(0.121962094111)*x[0]**o+(0.203169622345)*x[0]+(0.241504218053)*x[1]**o+(-0.0696356239498)*x[1]
        arg[0,0,1,1]=(0.0461537352329)*x[0]**o+(-0.275908938567)*x[0]+(-0.271184121027)*x[1]**o+(-0.364024729567)*x[1]
        arg[0,0,2,0]=(0.199272234315)*x[0]**o+(-0.906364258855)*x[0]+(-0.92102930037)*x[1]**o+(0.0718981903454)*x[1]
        arg[0,0,2,1]=(-0.742950958383)*x[0]**o+(0.0301464847722)*x[0]+(0.466993477687)*x[1]**o+(-0.402131750698)*x[1]
        arg[0,1,0,0]=(-0.0653776546375)*x[0]**o+(-0.679360612372)*x[0]+(-0.32783837768)*x[1]**o+(-0.113753016428)*x[1]
        arg[0,1,0,1]=(0.58610119414)*x[0]**o+(0.142613166231)*x[0]+(0.0409105619986)*x[1]**o+(-0.660641663558)*x[1]
        arg[0,1,1,0]=(0.212172308585)*x[0]**o+(-0.452767768344)*x[0]+(0.963053969889)*x[1]**o+(0.752691773953)*x[1]
        arg[0,1,1,1]=(-0.514998377887)*x[0]**o+(0.0872464329801)*x[0]+(0.265590295505)*x[1]**o+(0.557238826549)*x[1]
        arg[0,1,2,0]=(0.360523954724)*x[0]**o+(0.550238402994)*x[0]+(-0.206349415539)*x[1]**o+(0.357304730511)*x[1]
        arg[0,1,2,1]=(-0.872755217435)*x[0]**o+(0.190943683664)*x[0]+(-0.245454292354)*x[1]**o+(0.507229578506)*x[1]
        arg[0,2,0,0]=(0.994619505257)*x[0]**o+(0.33482820568)*x[0]+(-0.12721791236)*x[1]**o+(-0.505873171195)*x[1]
        arg[0,2,0,1]=(0.224805131522)*x[0]**o+(0.332022434603)*x[0]+(0.0583494507268)*x[1]**o+(0.857185410994)*x[1]
        arg[0,2,1,0]=(-0.425367077516)*x[0]**o+(-0.25207101061)*x[0]+(-0.11164491753)*x[1]**o+(0.44752084285)*x[1]
        arg[0,2,1,1]=(-0.158725210751)*x[0]**o+(-0.0692363138932)*x[0]+(0.214803128256)*x[1]**o+(0.886699106748)*x[1]
        arg[0,2,2,0]=(-0.332146331257)*x[0]**o+(0.114845050128)*x[0]+(-0.0781411611124)*x[1]**o+(0.474972057387)*x[1]
        arg[0,2,2,1]=(-0.481949630694)*x[0]**o+(0.644984972151)*x[0]+(0.171146519527)*x[1]**o+(0.57534803966)*x[1]
        arg[0,3,0,0]=(0.7580633024)*x[0]**o+(0.0280511776577)*x[0]+(0.375583652078)*x[1]**o+(-0.809112787284)*x[1]
        arg[0,3,0,1]=(-0.466737159934)*x[0]**o+(0.410032775686)*x[0]+(0.176141189584)*x[1]**o+(-0.228288309456)*x[1]
        arg[0,3,1,0]=(0.427129582196)*x[0]**o+(-0.718385859483)*x[0]+(-0.172261212989)*x[1]**o+(0.913157476188)*x[1]
        arg[0,3,1,1]=(0.952506706487)*x[0]**o+(-0.497478163899)*x[0]+(-0.930273038716)*x[1]**o+(0.673573149025)*x[1]
        arg[0,3,2,0]=(-0.0074302930747)*x[0]**o+(0.226424842035)*x[0]+(0.871190620671)*x[1]**o+(0.277695333679)*x[1]
        arg[0,3,2,1]=(-0.540820239946)*x[0]**o+(0.33044763211)*x[0]+(-0.772648820924)*x[1]**o+(-0.362410088021)*x[1]
        arg[0,4,0,0]=(-0.748396039111)*x[0]**o+(0.215131518212)*x[0]+(0.514087824403)*x[1]**o+(0.335563854205)*x[1]
        arg[0,4,0,1]=(-0.443263590391)*x[0]**o+(-0.523167282542)*x[0]+(0.122612452799)*x[1]**o+(0.216527857615)*x[1]
        arg[0,4,1,0]=(0.448176296777)*x[0]**o+(0.478292146131)*x[0]+(-0.558921083502)*x[1]**o+(-0.17018258654)*x[1]
        arg[0,4,1,1]=(-0.912765673953)*x[0]**o+(0.751746623223)*x[0]+(-0.104811579057)*x[1]**o+(0.456436870323)*x[1]
        arg[0,4,2,0]=(0.157946450932)*x[0]**o+(0.147858587857)*x[0]+(0.775747362874)*x[1]**o+(0.478478862463)*x[1]
        arg[0,4,2,1]=(-0.819625058185)*x[0]**o+(-0.0231980393919)*x[0]+(-0.186647585522)*x[1]**o+(0.513350371371)*x[1]
        arg[1,0,0,0]=(-0.942373946995)*x[0]**o+(-0.761326160931)*x[0]+(0.192511867167)*x[1]**o+(-0.50895969579)*x[1]
        arg[1,0,0,1]=(0.0670813349951)*x[0]**o+(0.55602953512)*x[0]+(0.757621178635)*x[1]**o+(0.741550201759)*x[1]
        arg[1,0,1,0]=(-0.114642609493)*x[0]**o+(-0.639500048447)*x[0]+(-0.861813039243)*x[1]**o+(0.36413406744)*x[1]
        arg[1,0,1,1]=(0.126555451594)*x[0]**o+(-0.885172557082)*x[0]+(-0.376073252921)*x[1]**o+(0.166102850653)*x[1]
        arg[1,0,2,0]=(-0.381517068219)*x[0]**o+(0.94870328267)*x[0]+(-0.129934777755)*x[1]**o+(0.78090715758)*x[1]
        arg[1,0,2,1]=(-0.535852302247)*x[0]**o+(0.63751039318)*x[0]+(-0.537556159925)*x[1]**o+(0.147766714221)*x[1]
        arg[1,1,0,0]=(-0.605543047533)*x[0]**o+(-0.366612785372)*x[0]+(0.544900149212)*x[1]**o+(0.722108900447)*x[1]
        arg[1,1,0,1]=(0.133378561142)*x[0]**o+(-0.227808772494)*x[0]+(-0.729399624175)*x[1]**o+(0.307571743455)*x[1]
        arg[1,1,1,0]=(0.0458830538139)*x[0]**o+(0.0585044493026)*x[0]+(-0.136630547605)*x[1]**o+(-0.0823288389309)*x[1]
        arg[1,1,1,1]=(0.339263808784)*x[0]**o+(0.00988917109076)*x[0]+(-0.786591838083)*x[1]**o+(-0.361686799179)*x[1]
        arg[1,1,2,0]=(0.789942037592)*x[0]**o+(-0.266689758676)*x[0]+(0.464856327495)*x[1]**o+(0.138022774293)*x[1]
        arg[1,1,2,1]=(-0.723544867217)*x[0]**o+(-0.523023674205)*x[0]+(0.629726026331)*x[1]**o+(-0.867808120997)*x[1]
        arg[1,2,0,0]=(-0.495589518787)*x[0]**o+(-0.561664880262)*x[0]+(0.0319320417078)*x[1]**o+(-0.153720661498)*x[1]
        arg[1,2,0,1]=(0.293168114024)*x[0]**o+(-0.344107163981)*x[0]+(-0.275629670068)*x[1]**o+(0.06103653487)*x[1]
        arg[1,2,1,0]=(0.495640927339)*x[0]**o+(0.596540095043)*x[0]+(0.208792567756)*x[1]**o+(0.886040724145)*x[1]
        arg[1,2,1,1]=(0.331476398093)*x[0]**o+(-0.472588572455)*x[0]+(-0.455670457285)*x[1]**o+(-0.731223615642)*x[1]
        arg[1,2,2,0]=(-0.127107829674)*x[0]**o+(-0.21113128456)*x[0]+(-0.733928194168)*x[1]**o+(-0.174127696853)*x[1]
        arg[1,2,2,1]=(0.521879316151)*x[0]**o+(0.983424557865)*x[0]+(-0.288659660182)*x[1]**o+(-0.431045203113)*x[1]
        arg[1,3,0,0]=(0.292348765746)*x[0]**o+(0.0730741240915)*x[0]+(0.079624022683)*x[1]**o+(0.237483071633)*x[1]
        arg[1,3,0,1]=(0.291035771848)*x[0]**o+(-0.482909477855)*x[0]+(-0.999142129615)*x[1]**o+(-0.559723367197)*x[1]
        arg[1,3,1,0]=(0.747096275653)*x[0]**o+(0.762757649624)*x[0]+(-0.821650889507)*x[1]**o+(0.301128670189)*x[1]
        arg[1,3,1,1]=(0.82703681414)*x[0]**o+(-0.336570070436)*x[0]+(0.798131952979)*x[1]**o+(0.603562737956)*x[1]
        arg[1,3,2,0]=(-0.164875914126)*x[0]**o+(0.219409873902)*x[0]+(0.784253160678)*x[1]**o+(0.0219597460707)*x[1]
        arg[1,3,2,1]=(0.124778899794)*x[0]**o+(0.785367842145)*x[0]+(0.948111125908)*x[1]**o+(-0.887427847089)*x[1]
        arg[1,4,0,0]=(0.0911816002883)*x[0]**o+(0.645410022441)*x[0]+(0.720551531965)*x[1]**o+(-0.388407683413)*x[1]
        arg[1,4,0,1]=(-0.0416762613062)*x[0]**o+(-0.548874829834)*x[0]+(-0.377952555557)*x[1]**o+(0.742795234361)*x[1]
        arg[1,4,1,0]=(-0.896929017971)*x[0]**o+(0.678343686516)*x[0]+(0.711572763082)*x[1]**o+(-0.43397331651)*x[1]
        arg[1,4,1,1]=(-0.190804421566)*x[0]**o+(-0.0848405124789)*x[0]+(0.36246957905)*x[1]**o+(0.963599086227)*x[1]
        arg[1,4,2,0]=(-0.0210248567982)*x[0]**o+(0.545977814581)*x[0]+(0.858962864401)*x[1]**o+(0.113792851997)*x[1]
        arg[1,4,2,1]=(0.229641892622)*x[0]**o+(-0.739561617198)*x[0]+(-0.164372740617)*x[1]**o+(0.537097868843)*x[1]
        arg[2,0,0,0]=(-0.13321389343)*x[0]**o+(-0.668403350503)*x[0]+(-0.924879105983)*x[1]**o+(-0.5161790717)*x[1]
        arg[2,0,0,1]=(0.26582844685)*x[0]**o+(0.89048660684)*x[0]+(-0.377225981719)*x[1]**o+(-0.303790710194)*x[1]
        arg[2,0,1,0]=(0.0997470860283)*x[0]**o+(0.0385427264217)*x[0]+(-0.71387347273)*x[1]**o+(-0.621376499192)*x[1]
        arg[2,0,1,1]=(0.595544616326)*x[0]**o+(-0.850529503975)*x[0]+(0.602203957938)*x[1]**o+(-0.329484843907)*x[1]
        arg[2,0,2,0]=(-0.106031967029)*x[0]**o+(-0.165122821622)*x[0]+(0.647473815919)*x[1]**o+(0.28942226062)*x[1]
        arg[2,0,2,1]=(0.585955875472)*x[0]**o+(0.922667710403)*x[0]+(-0.0710905957841)*x[1]**o+(-0.669138163697)*x[1]
        arg[2,1,0,0]=(0.461941211081)*x[0]**o+(0.723022466263)*x[0]+(-0.118551893472)*x[1]**o+(0.00445187353637)*x[1]
        arg[2,1,0,1]=(-0.253168248108)*x[0]**o+(0.476938842605)*x[0]+(0.476641658034)*x[1]**o+(-0.916886387595)*x[1]
        arg[2,1,1,0]=(0.722029309503)*x[0]**o+(0.0588848593425)*x[0]+(0.84737370043)*x[1]**o+(-0.674604415213)*x[1]
        arg[2,1,1,1]=(-0.644736759298)*x[0]**o+(-0.212847303893)*x[0]+(0.25394587552)*x[1]**o+(-0.366955175081)*x[1]
        arg[2,1,2,0]=(-0.657395296801)*x[0]**o+(-0.967570027599)*x[0]+(0.180926628541)*x[1]**o+(-0.940599897942)*x[1]
        arg[2,1,2,1]=(0.849897328801)*x[0]**o+(-0.299199474051)*x[0]+(0.427221207827)*x[1]**o+(0.212027985441)*x[1]
        arg[2,2,0,0]=(-0.561573507381)*x[0]**o+(-0.446925224908)*x[0]+(0.132091224948)*x[1]**o+(-0.673375661849)*x[1]
        arg[2,2,0,1]=(-0.386657291574)*x[0]**o+(0.356784637112)*x[0]+(0.644292006233)*x[1]**o+(0.922937167842)*x[1]
        arg[2,2,1,0]=(0.484446757285)*x[0]**o+(-0.822469870455)*x[0]+(-0.89605890123)*x[1]**o+(-0.895891500031)*x[1]
        arg[2,2,1,1]=(-0.0930740520192)*x[0]**o+(0.953356473678)*x[0]+(-0.714418458806)*x[1]**o+(-0.100584950946)*x[1]
        arg[2,2,2,0]=(0.945572242667)*x[0]**o+(-0.259280849605)*x[0]+(-0.925459846341)*x[1]**o+(0.906002758873)*x[1]
        arg[2,2,2,1]=(-0.638993332975)*x[0]**o+(-0.668594641703)*x[0]+(0.012666231369)*x[1]**o+(0.89732685856)*x[1]
        arg[2,3,0,0]=(-0.885061116144)*x[0]**o+(-0.576097536483)*x[0]+(-0.43166075788)*x[1]**o+(0.538821620917)*x[1]
        arg[2,3,0,1]=(-0.0743638381797)*x[0]**o+(0.251037500035)*x[0]+(-0.905989928076)*x[1]**o+(0.243771000867)*x[1]
        arg[2,3,1,0]=(-0.016045622351)*x[0]**o+(-0.632944072174)*x[0]+(-0.296175918793)*x[1]**o+(-0.00120123641365)*x[1]
        arg[2,3,1,1]=(-0.896354921812)*x[0]**o+(-0.331348989027)*x[0]+(-0.450865285201)*x[1]**o+(-0.243268582009)*x[1]
        arg[2,3,2,0]=(0.609332927431)*x[0]**o+(-0.0815856802115)*x[0]+(0.201924792273)*x[1]**o+(-0.890424451348)*x[1]
        arg[2,3,2,1]=(0.395090137264)*x[0]**o+(0.294230442573)*x[0]+(-0.345371510364)*x[1]**o+(0.181229663241)*x[1]
        arg[2,4,0,0]=(-0.823684509034)*x[0]**o+(0.431019301942)*x[0]+(0.554707015054)*x[1]**o+(-0.846322903003)*x[1]
        arg[2,4,0,1]=(0.862984476658)*x[0]**o+(0.20512500102)*x[0]+(0.0247756159287)*x[1]**o+(-0.374795100946)*x[1]
        arg[2,4,1,0]=(-0.0849914269517)*x[0]**o+(0.754908200339)*x[0]+(0.593363754807)*x[1]**o+(0.124923961126)*x[1]
        arg[2,4,1,1]=(-0.691440264234)*x[0]**o+(0.825739298015)*x[0]+(0.426452231478)*x[1]**o+(0.0995610721341)*x[1]
        arg[2,4,2,0]=(-0.175322750567)*x[0]**o+(0.419750865569)*x[0]+(0.208448987712)*x[1]**o+(0.150584798952)*x[1]
        arg[2,4,2,1]=(0.931680152344)*x[0]**o+(0.390947206307)*x[0]+(0.388437348378)*x[1]**o+(-0.808413079032)*x[1]
        arg[3,0,0,0]=(0.58855431402)*x[0]**o+(0.383865913364)*x[0]+(-0.691445323219)*x[1]**o+(-0.443370658523)*x[1]
        arg[3,0,0,1]=(-0.657371481478)*x[0]**o+(0.746891082673)*x[0]+(-0.829229947773)*x[1]**o+(0.761915670404)*x[1]
        arg[3,0,1,0]=(-0.431502932714)*x[0]**o+(-0.961472285716)*x[0]+(0.0423384388031)*x[1]**o+(0.339373458118)*x[1]
        arg[3,0,1,1]=(-0.272831163259)*x[0]**o+(-0.533438904494)*x[0]+(-0.595625384875)*x[1]**o+(0.0366857022323)*x[1]
        arg[3,0,2,0]=(-0.319684662306)*x[0]**o+(0.672185828912)*x[0]+(0.483126548994)*x[1]**o+(0.745872629604)*x[1]
        arg[3,0,2,1]=(0.437895277404)*x[0]**o+(-0.337209416963)*x[0]+(0.101080522788)*x[1]**o+(0.892022488644)*x[1]
        arg[3,1,0,0]=(-0.582325078455)*x[0]**o+(-0.854689347157)*x[0]+(0.0308638312259)*x[1]**o+(0.633382365704)*x[1]
        arg[3,1,0,1]=(0.319735178376)*x[0]**o+(-0.782764593952)*x[0]+(0.082230132244)*x[1]**o+(-0.946010937852)*x[1]
        arg[3,1,1,0]=(-0.698490410666)*x[0]**o+(-0.801358802689)*x[0]+(0.20695648235)*x[1]**o+(0.75257523471)*x[1]
        arg[3,1,1,1]=(0.66321696527)*x[0]**o+(0.704572918868)*x[0]+(0.528208879002)*x[1]**o+(-0.408378325513)*x[1]
        arg[3,1,2,0]=(-0.925177198417)*x[0]**o+(-0.152427811549)*x[0]+(0.341781993583)*x[1]**o+(-0.589976456377)*x[1]
        arg[3,1,2,1]=(0.206706628353)*x[0]**o+(0.673777173329)*x[0]+(-0.994138314254)*x[1]**o+(-0.843696565863)*x[1]
        arg[3,2,0,0]=(-0.588881067333)*x[0]**o+(-0.537572388341)*x[0]+(-0.206570371675)*x[1]**o+(-0.127468554294)*x[1]
        arg[3,2,0,1]=(0.498638284096)*x[0]**o+(-0.488278831086)*x[0]+(-0.28320874594)*x[1]**o+(-0.638241576615)*x[1]
        arg[3,2,1,0]=(-0.445780790822)*x[0]**o+(-0.347493953089)*x[0]+(-0.558010086232)*x[1]**o+(-0.0287339307689)*x[1]
        arg[3,2,1,1]=(-0.361727811272)*x[0]**o+(-0.559565378007)*x[0]+(-0.211685558972)*x[1]**o+(0.330088396811)*x[1]
        arg[3,2,2,0]=(-0.42600117831)*x[0]**o+(-0.488997111988)*x[0]+(0.127421278051)*x[1]**o+(-0.48233167806)*x[1]
        arg[3,2,2,1]=(0.120468829686)*x[0]**o+(0.0675603906118)*x[0]+(-0.60674361864)*x[1]**o+(0.639115708258)*x[1]
        arg[3,3,0,0]=(0.369343033015)*x[0]**o+(0.707735743593)*x[0]+(-0.426275591836)*x[1]**o+(-0.733377765625)*x[1]
        arg[3,3,0,1]=(-0.931245338156)*x[0]**o+(-0.465861794185)*x[0]+(-0.650664500687)*x[1]**o+(0.103077662402)*x[1]
        arg[3,3,1,0]=(0.731834744512)*x[0]**o+(0.267840571071)*x[0]+(0.952106457495)*x[1]**o+(-0.943845364366)*x[1]
        arg[3,3,1,1]=(-0.21692720916)*x[0]**o+(0.580841374997)*x[0]+(-0.412129341223)*x[1]**o+(0.40414445304)*x[1]
        arg[3,3,2,0]=(-0.447758598726)*x[0]**o+(0.0616926352912)*x[0]+(-0.824624226122)*x[1]**o+(0.19099833104)*x[1]
        arg[3,3,2,1]=(-0.615519138165)*x[0]**o+(0.233926822164)*x[0]+(-0.969014060236)*x[1]**o+(0.835103627893)*x[1]
        arg[3,4,0,0]=(0.550778983301)*x[0]**o+(0.0198253530882)*x[0]+(-0.39383849187)*x[1]**o+(0.367721826304)*x[1]
        arg[3,4,0,1]=(0.444877923149)*x[0]**o+(-0.899159835223)*x[0]+(0.923392540452)*x[1]**o+(0.512905141718)*x[1]
        arg[3,4,1,0]=(-0.563807675145)*x[0]**o+(0.994405383899)*x[0]+(-0.777352948218)*x[1]**o+(0.865994906227)*x[1]
        arg[3,4,1,1]=(-0.593800452139)*x[0]**o+(0.613778803237)*x[0]+(0.582200979718)*x[1]**o+(0.448026243271)*x[1]
        arg[3,4,2,0]=(-0.900331125416)*x[0]**o+(0.234613965424)*x[0]+(0.510400273805)*x[1]**o+(0.0836773641661)*x[1]
        arg[3,4,2,1]=(0.710999097307)*x[0]**o+(0.886603988265)*x[0]+(-0.405644887679)*x[1]**o+(0.362257258809)*x[1]
        ref[0,0,0,0]=(-0.0338299192888)*x_ref[0]**o+(-0.114906274359)*x_ref[0]+(0.15010339398)*x_ref[1]**o+(0.625295991328)*x_ref[1]
        ref[0,0,0,1]=(-0.1311901039)*x_ref[0]**o+(-0.551712861217)*x_ref[0]+(-0.287411404506)*x_ref[1]**o+(0.0242820629572)*x_ref[1]
        ref[0,0,1,0]=(0.121962094111)*x_ref[0]**o+(0.203169622345)*x_ref[0]+(0.241504218053)*x_ref[1]**o+(-0.0696356239498)*x_ref[1]
        ref[0,0,1,1]=(0.0461537352329)*x_ref[0]**o+(-0.275908938567)*x_ref[0]+(-0.271184121027)*x_ref[1]**o+(-0.364024729567)*x_ref[1]
        ref[0,0,2,0]=(0.199272234315)*x_ref[0]**o+(-0.906364258855)*x_ref[0]+(-0.92102930037)*x_ref[1]**o+(0.0718981903454)*x_ref[1]
        ref[0,0,2,1]=(-0.742950958383)*x_ref[0]**o+(0.0301464847722)*x_ref[0]+(0.466993477687)*x_ref[1]**o+(-0.402131750698)*x_ref[1]
        ref[0,1,0,0]=(-0.0653776546375)*x_ref[0]**o+(-0.679360612372)*x_ref[0]+(-0.32783837768)*x_ref[1]**o+(-0.113753016428)*x_ref[1]
        ref[0,1,0,1]=(0.58610119414)*x_ref[0]**o+(0.142613166231)*x_ref[0]+(0.0409105619986)*x_ref[1]**o+(-0.660641663558)*x_ref[1]
        ref[0,1,1,0]=(0.212172308585)*x_ref[0]**o+(-0.452767768344)*x_ref[0]+(0.963053969889)*x_ref[1]**o+(0.752691773953)*x_ref[1]
        ref[0,1,1,1]=(-0.514998377887)*x_ref[0]**o+(0.0872464329801)*x_ref[0]+(0.265590295505)*x_ref[1]**o+(0.557238826549)*x_ref[1]
        ref[0,1,2,0]=(0.360523954724)*x_ref[0]**o+(0.550238402994)*x_ref[0]+(-0.206349415539)*x_ref[1]**o+(0.357304730511)*x_ref[1]
        ref[0,1,2,1]=(-0.872755217435)*x_ref[0]**o+(0.190943683664)*x_ref[0]+(-0.245454292354)*x_ref[1]**o+(0.507229578506)*x_ref[1]
        ref[0,2,0,0]=(0.994619505257)*x_ref[0]**o+(0.33482820568)*x_ref[0]+(-0.12721791236)*x_ref[1]**o+(-0.505873171195)*x_ref[1]
        ref[0,2,0,1]=(0.224805131522)*x_ref[0]**o+(0.332022434603)*x_ref[0]+(0.0583494507268)*x_ref[1]**o+(0.857185410994)*x_ref[1]
        ref[0,2,1,0]=(-0.425367077516)*x_ref[0]**o+(-0.25207101061)*x_ref[0]+(-0.11164491753)*x_ref[1]**o+(0.44752084285)*x_ref[1]
        ref[0,2,1,1]=(-0.158725210751)*x_ref[0]**o+(-0.0692363138932)*x_ref[0]+(0.214803128256)*x_ref[1]**o+(0.886699106748)*x_ref[1]
        ref[0,2,2,0]=(-0.332146331257)*x_ref[0]**o+(0.114845050128)*x_ref[0]+(-0.0781411611124)*x_ref[1]**o+(0.474972057387)*x_ref[1]
        ref[0,2,2,1]=(-0.481949630694)*x_ref[0]**o+(0.644984972151)*x_ref[0]+(0.171146519527)*x_ref[1]**o+(0.57534803966)*x_ref[1]
        ref[0,3,0,0]=(0.7580633024)*x_ref[0]**o+(0.0280511776577)*x_ref[0]+(0.375583652078)*x_ref[1]**o+(-0.809112787284)*x_ref[1]
        ref[0,3,0,1]=(-0.466737159934)*x_ref[0]**o+(0.410032775686)*x_ref[0]+(0.176141189584)*x_ref[1]**o+(-0.228288309456)*x_ref[1]
        ref[0,3,1,0]=(0.427129582196)*x_ref[0]**o+(-0.718385859483)*x_ref[0]+(-0.172261212989)*x_ref[1]**o+(0.913157476188)*x_ref[1]
        ref[0,3,1,1]=(0.952506706487)*x_ref[0]**o+(-0.497478163899)*x_ref[0]+(-0.930273038716)*x_ref[1]**o+(0.673573149025)*x_ref[1]
        ref[0,3,2,0]=(-0.0074302930747)*x_ref[0]**o+(0.226424842035)*x_ref[0]+(0.871190620671)*x_ref[1]**o+(0.277695333679)*x_ref[1]
        ref[0,3,2,1]=(-0.540820239946)*x_ref[0]**o+(0.33044763211)*x_ref[0]+(-0.772648820924)*x_ref[1]**o+(-0.362410088021)*x_ref[1]
        ref[0,4,0,0]=(-0.748396039111)*x_ref[0]**o+(0.215131518212)*x_ref[0]+(0.514087824403)*x_ref[1]**o+(0.335563854205)*x_ref[1]
        ref[0,4,0,1]=(-0.443263590391)*x_ref[0]**o+(-0.523167282542)*x_ref[0]+(0.122612452799)*x_ref[1]**o+(0.216527857615)*x_ref[1]
        ref[0,4,1,0]=(0.448176296777)*x_ref[0]**o+(0.478292146131)*x_ref[0]+(-0.558921083502)*x_ref[1]**o+(-0.17018258654)*x_ref[1]
        ref[0,4,1,1]=(-0.912765673953)*x_ref[0]**o+(0.751746623223)*x_ref[0]+(-0.104811579057)*x_ref[1]**o+(0.456436870323)*x_ref[1]
        ref[0,4,2,0]=(0.157946450932)*x_ref[0]**o+(0.147858587857)*x_ref[0]+(0.775747362874)*x_ref[1]**o+(0.478478862463)*x_ref[1]
        ref[0,4,2,1]=(-0.819625058185)*x_ref[0]**o+(-0.0231980393919)*x_ref[0]+(-0.186647585522)*x_ref[1]**o+(0.513350371371)*x_ref[1]
        ref[1,0,0,0]=(-0.942373946995)*x_ref[0]**o+(-0.761326160931)*x_ref[0]+(0.192511867167)*x_ref[1]**o+(-0.50895969579)*x_ref[1]
        ref[1,0,0,1]=(0.0670813349951)*x_ref[0]**o+(0.55602953512)*x_ref[0]+(0.757621178635)*x_ref[1]**o+(0.741550201759)*x_ref[1]
        ref[1,0,1,0]=(-0.114642609493)*x_ref[0]**o+(-0.639500048447)*x_ref[0]+(-0.861813039243)*x_ref[1]**o+(0.36413406744)*x_ref[1]
        ref[1,0,1,1]=(0.126555451594)*x_ref[0]**o+(-0.885172557082)*x_ref[0]+(-0.376073252921)*x_ref[1]**o+(0.166102850653)*x_ref[1]
        ref[1,0,2,0]=(-0.381517068219)*x_ref[0]**o+(0.94870328267)*x_ref[0]+(-0.129934777755)*x_ref[1]**o+(0.78090715758)*x_ref[1]
        ref[1,0,2,1]=(-0.535852302247)*x_ref[0]**o+(0.63751039318)*x_ref[0]+(-0.537556159925)*x_ref[1]**o+(0.147766714221)*x_ref[1]
        ref[1,1,0,0]=(-0.605543047533)*x_ref[0]**o+(-0.366612785372)*x_ref[0]+(0.544900149212)*x_ref[1]**o+(0.722108900447)*x_ref[1]
        ref[1,1,0,1]=(0.133378561142)*x_ref[0]**o+(-0.227808772494)*x_ref[0]+(-0.729399624175)*x_ref[1]**o+(0.307571743455)*x_ref[1]
        ref[1,1,1,0]=(0.0458830538139)*x_ref[0]**o+(0.0585044493026)*x_ref[0]+(-0.136630547605)*x_ref[1]**o+(-0.0823288389309)*x_ref[1]
        ref[1,1,1,1]=(0.339263808784)*x_ref[0]**o+(0.00988917109076)*x_ref[0]+(-0.786591838083)*x_ref[1]**o+(-0.361686799179)*x_ref[1]
        ref[1,1,2,0]=(0.789942037592)*x_ref[0]**o+(-0.266689758676)*x_ref[0]+(0.464856327495)*x_ref[1]**o+(0.138022774293)*x_ref[1]
        ref[1,1,2,1]=(-0.723544867217)*x_ref[0]**o+(-0.523023674205)*x_ref[0]+(0.629726026331)*x_ref[1]**o+(-0.867808120997)*x_ref[1]
        ref[1,2,0,0]=(-0.495589518787)*x_ref[0]**o+(-0.561664880262)*x_ref[0]+(0.0319320417078)*x_ref[1]**o+(-0.153720661498)*x_ref[1]
        ref[1,2,0,1]=(0.293168114024)*x_ref[0]**o+(-0.344107163981)*x_ref[0]+(-0.275629670068)*x_ref[1]**o+(0.06103653487)*x_ref[1]
        ref[1,2,1,0]=(0.495640927339)*x_ref[0]**o+(0.596540095043)*x_ref[0]+(0.208792567756)*x_ref[1]**o+(0.886040724145)*x_ref[1]
        ref[1,2,1,1]=(0.331476398093)*x_ref[0]**o+(-0.472588572455)*x_ref[0]+(-0.455670457285)*x_ref[1]**o+(-0.731223615642)*x_ref[1]
        ref[1,2,2,0]=(-0.127107829674)*x_ref[0]**o+(-0.21113128456)*x_ref[0]+(-0.733928194168)*x_ref[1]**o+(-0.174127696853)*x_ref[1]
        ref[1,2,2,1]=(0.521879316151)*x_ref[0]**o+(0.983424557865)*x_ref[0]+(-0.288659660182)*x_ref[1]**o+(-0.431045203113)*x_ref[1]
        ref[1,3,0,0]=(0.292348765746)*x_ref[0]**o+(0.0730741240915)*x_ref[0]+(0.079624022683)*x_ref[1]**o+(0.237483071633)*x_ref[1]
        ref[1,3,0,1]=(0.291035771848)*x_ref[0]**o+(-0.482909477855)*x_ref[0]+(-0.999142129615)*x_ref[1]**o+(-0.559723367197)*x_ref[1]
        ref[1,3,1,0]=(0.747096275653)*x_ref[0]**o+(0.762757649624)*x_ref[0]+(-0.821650889507)*x_ref[1]**o+(0.301128670189)*x_ref[1]
        ref[1,3,1,1]=(0.82703681414)*x_ref[0]**o+(-0.336570070436)*x_ref[0]+(0.798131952979)*x_ref[1]**o+(0.603562737956)*x_ref[1]
        ref[1,3,2,0]=(-0.164875914126)*x_ref[0]**o+(0.219409873902)*x_ref[0]+(0.784253160678)*x_ref[1]**o+(0.0219597460707)*x_ref[1]
        ref[1,3,2,1]=(0.124778899794)*x_ref[0]**o+(0.785367842145)*x_ref[0]+(0.948111125908)*x_ref[1]**o+(-0.887427847089)*x_ref[1]
        ref[1,4,0,0]=(0.0911816002883)*x_ref[0]**o+(0.645410022441)*x_ref[0]+(0.720551531965)*x_ref[1]**o+(-0.388407683413)*x_ref[1]
        ref[1,4,0,1]=(-0.0416762613062)*x_ref[0]**o+(-0.548874829834)*x_ref[0]+(-0.377952555557)*x_ref[1]**o+(0.742795234361)*x_ref[1]
        ref[1,4,1,0]=(-0.896929017971)*x_ref[0]**o+(0.678343686516)*x_ref[0]+(0.711572763082)*x_ref[1]**o+(-0.43397331651)*x_ref[1]
        ref[1,4,1,1]=(-0.190804421566)*x_ref[0]**o+(-0.0848405124789)*x_ref[0]+(0.36246957905)*x_ref[1]**o+(0.963599086227)*x_ref[1]
        ref[1,4,2,0]=(-0.0210248567982)*x_ref[0]**o+(0.545977814581)*x_ref[0]+(0.858962864401)*x_ref[1]**o+(0.113792851997)*x_ref[1]
        ref[1,4,2,1]=(0.229641892622)*x_ref[0]**o+(-0.739561617198)*x_ref[0]+(-0.164372740617)*x_ref[1]**o+(0.537097868843)*x_ref[1]
        ref[2,0,0,0]=(-0.13321389343)*x_ref[0]**o+(-0.668403350503)*x_ref[0]+(-0.924879105983)*x_ref[1]**o+(-0.5161790717)*x_ref[1]
        ref[2,0,0,1]=(0.26582844685)*x_ref[0]**o+(0.89048660684)*x_ref[0]+(-0.377225981719)*x_ref[1]**o+(-0.303790710194)*x_ref[1]
        ref[2,0,1,0]=(0.0997470860283)*x_ref[0]**o+(0.0385427264217)*x_ref[0]+(-0.71387347273)*x_ref[1]**o+(-0.621376499192)*x_ref[1]
        ref[2,0,1,1]=(0.595544616326)*x_ref[0]**o+(-0.850529503975)*x_ref[0]+(0.602203957938)*x_ref[1]**o+(-0.329484843907)*x_ref[1]
        ref[2,0,2,0]=(-0.106031967029)*x_ref[0]**o+(-0.165122821622)*x_ref[0]+(0.647473815919)*x_ref[1]**o+(0.28942226062)*x_ref[1]
        ref[2,0,2,1]=(0.585955875472)*x_ref[0]**o+(0.922667710403)*x_ref[0]+(-0.0710905957841)*x_ref[1]**o+(-0.669138163697)*x_ref[1]
        ref[2,1,0,0]=(0.461941211081)*x_ref[0]**o+(0.723022466263)*x_ref[0]+(-0.118551893472)*x_ref[1]**o+(0.00445187353637)*x_ref[1]
        ref[2,1,0,1]=(-0.253168248108)*x_ref[0]**o+(0.476938842605)*x_ref[0]+(0.476641658034)*x_ref[1]**o+(-0.916886387595)*x_ref[1]
        ref[2,1,1,0]=(0.722029309503)*x_ref[0]**o+(0.0588848593425)*x_ref[0]+(0.84737370043)*x_ref[1]**o+(-0.674604415213)*x_ref[1]
        ref[2,1,1,1]=(-0.644736759298)*x_ref[0]**o+(-0.212847303893)*x_ref[0]+(0.25394587552)*x_ref[1]**o+(-0.366955175081)*x_ref[1]
        ref[2,1,2,0]=(-0.657395296801)*x_ref[0]**o+(-0.967570027599)*x_ref[0]+(0.180926628541)*x_ref[1]**o+(-0.940599897942)*x_ref[1]
        ref[2,1,2,1]=(0.849897328801)*x_ref[0]**o+(-0.299199474051)*x_ref[0]+(0.427221207827)*x_ref[1]**o+(0.212027985441)*x_ref[1]
        ref[2,2,0,0]=(-0.561573507381)*x_ref[0]**o+(-0.446925224908)*x_ref[0]+(0.132091224948)*x_ref[1]**o+(-0.673375661849)*x_ref[1]
        ref[2,2,0,1]=(-0.386657291574)*x_ref[0]**o+(0.356784637112)*x_ref[0]+(0.644292006233)*x_ref[1]**o+(0.922937167842)*x_ref[1]
        ref[2,2,1,0]=(0.484446757285)*x_ref[0]**o+(-0.822469870455)*x_ref[0]+(-0.89605890123)*x_ref[1]**o+(-0.895891500031)*x_ref[1]
        ref[2,2,1,1]=(-0.0930740520192)*x_ref[0]**o+(0.953356473678)*x_ref[0]+(-0.714418458806)*x_ref[1]**o+(-0.100584950946)*x_ref[1]
        ref[2,2,2,0]=(0.945572242667)*x_ref[0]**o+(-0.259280849605)*x_ref[0]+(-0.925459846341)*x_ref[1]**o+(0.906002758873)*x_ref[1]
        ref[2,2,2,1]=(-0.638993332975)*x_ref[0]**o+(-0.668594641703)*x_ref[0]+(0.012666231369)*x_ref[1]**o+(0.89732685856)*x_ref[1]
        ref[2,3,0,0]=(-0.885061116144)*x_ref[0]**o+(-0.576097536483)*x_ref[0]+(-0.43166075788)*x_ref[1]**o+(0.538821620917)*x_ref[1]
        ref[2,3,0,1]=(-0.0743638381797)*x_ref[0]**o+(0.251037500035)*x_ref[0]+(-0.905989928076)*x_ref[1]**o+(0.243771000867)*x_ref[1]
        ref[2,3,1,0]=(-0.016045622351)*x_ref[0]**o+(-0.632944072174)*x_ref[0]+(-0.296175918793)*x_ref[1]**o+(-0.00120123641365)*x_ref[1]
        ref[2,3,1,1]=(-0.896354921812)*x_ref[0]**o+(-0.331348989027)*x_ref[0]+(-0.450865285201)*x_ref[1]**o+(-0.243268582009)*x_ref[1]
        ref[2,3,2,0]=(0.609332927431)*x_ref[0]**o+(-0.0815856802115)*x_ref[0]+(0.201924792273)*x_ref[1]**o+(-0.890424451348)*x_ref[1]
        ref[2,3,2,1]=(0.395090137264)*x_ref[0]**o+(0.294230442573)*x_ref[0]+(-0.345371510364)*x_ref[1]**o+(0.181229663241)*x_ref[1]
        ref[2,4,0,0]=(-0.823684509034)*x_ref[0]**o+(0.431019301942)*x_ref[0]+(0.554707015054)*x_ref[1]**o+(-0.846322903003)*x_ref[1]
        ref[2,4,0,1]=(0.862984476658)*x_ref[0]**o+(0.20512500102)*x_ref[0]+(0.0247756159287)*x_ref[1]**o+(-0.374795100946)*x_ref[1]
        ref[2,4,1,0]=(-0.0849914269517)*x_ref[0]**o+(0.754908200339)*x_ref[0]+(0.593363754807)*x_ref[1]**o+(0.124923961126)*x_ref[1]
        ref[2,4,1,1]=(-0.691440264234)*x_ref[0]**o+(0.825739298015)*x_ref[0]+(0.426452231478)*x_ref[1]**o+(0.0995610721341)*x_ref[1]
        ref[2,4,2,0]=(-0.175322750567)*x_ref[0]**o+(0.419750865569)*x_ref[0]+(0.208448987712)*x_ref[1]**o+(0.150584798952)*x_ref[1]
        ref[2,4,2,1]=(0.931680152344)*x_ref[0]**o+(0.390947206307)*x_ref[0]+(0.388437348378)*x_ref[1]**o+(-0.808413079032)*x_ref[1]
        ref[3,0,0,0]=(0.58855431402)*x_ref[0]**o+(0.383865913364)*x_ref[0]+(-0.691445323219)*x_ref[1]**o+(-0.443370658523)*x_ref[1]
        ref[3,0,0,1]=(-0.657371481478)*x_ref[0]**o+(0.746891082673)*x_ref[0]+(-0.829229947773)*x_ref[1]**o+(0.761915670404)*x_ref[1]
        ref[3,0,1,0]=(-0.431502932714)*x_ref[0]**o+(-0.961472285716)*x_ref[0]+(0.0423384388031)*x_ref[1]**o+(0.339373458118)*x_ref[1]
        ref[3,0,1,1]=(-0.272831163259)*x_ref[0]**o+(-0.533438904494)*x_ref[0]+(-0.595625384875)*x_ref[1]**o+(0.0366857022323)*x_ref[1]
        ref[3,0,2,0]=(-0.319684662306)*x_ref[0]**o+(0.672185828912)*x_ref[0]+(0.483126548994)*x_ref[1]**o+(0.745872629604)*x_ref[1]
        ref[3,0,2,1]=(0.437895277404)*x_ref[0]**o+(-0.337209416963)*x_ref[0]+(0.101080522788)*x_ref[1]**o+(0.892022488644)*x_ref[1]
        ref[3,1,0,0]=(-0.582325078455)*x_ref[0]**o+(-0.854689347157)*x_ref[0]+(0.0308638312259)*x_ref[1]**o+(0.633382365704)*x_ref[1]
        ref[3,1,0,1]=(0.319735178376)*x_ref[0]**o+(-0.782764593952)*x_ref[0]+(0.082230132244)*x_ref[1]**o+(-0.946010937852)*x_ref[1]
        ref[3,1,1,0]=(-0.698490410666)*x_ref[0]**o+(-0.801358802689)*x_ref[0]+(0.20695648235)*x_ref[1]**o+(0.75257523471)*x_ref[1]
        ref[3,1,1,1]=(0.66321696527)*x_ref[0]**o+(0.704572918868)*x_ref[0]+(0.528208879002)*x_ref[1]**o+(-0.408378325513)*x_ref[1]
        ref[3,1,2,0]=(-0.925177198417)*x_ref[0]**o+(-0.152427811549)*x_ref[0]+(0.341781993583)*x_ref[1]**o+(-0.589976456377)*x_ref[1]
        ref[3,1,2,1]=(0.206706628353)*x_ref[0]**o+(0.673777173329)*x_ref[0]+(-0.994138314254)*x_ref[1]**o+(-0.843696565863)*x_ref[1]
        ref[3,2,0,0]=(-0.588881067333)*x_ref[0]**o+(-0.537572388341)*x_ref[0]+(-0.206570371675)*x_ref[1]**o+(-0.127468554294)*x_ref[1]
        ref[3,2,0,1]=(0.498638284096)*x_ref[0]**o+(-0.488278831086)*x_ref[0]+(-0.28320874594)*x_ref[1]**o+(-0.638241576615)*x_ref[1]
        ref[3,2,1,0]=(-0.445780790822)*x_ref[0]**o+(-0.347493953089)*x_ref[0]+(-0.558010086232)*x_ref[1]**o+(-0.0287339307689)*x_ref[1]
        ref[3,2,1,1]=(-0.361727811272)*x_ref[0]**o+(-0.559565378007)*x_ref[0]+(-0.211685558972)*x_ref[1]**o+(0.330088396811)*x_ref[1]
        ref[3,2,2,0]=(-0.42600117831)*x_ref[0]**o+(-0.488997111988)*x_ref[0]+(0.127421278051)*x_ref[1]**o+(-0.48233167806)*x_ref[1]
        ref[3,2,2,1]=(0.120468829686)*x_ref[0]**o+(0.0675603906118)*x_ref[0]+(-0.60674361864)*x_ref[1]**o+(0.639115708258)*x_ref[1]
        ref[3,3,0,0]=(0.369343033015)*x_ref[0]**o+(0.707735743593)*x_ref[0]+(-0.426275591836)*x_ref[1]**o+(-0.733377765625)*x_ref[1]
        ref[3,3,0,1]=(-0.931245338156)*x_ref[0]**o+(-0.465861794185)*x_ref[0]+(-0.650664500687)*x_ref[1]**o+(0.103077662402)*x_ref[1]
        ref[3,3,1,0]=(0.731834744512)*x_ref[0]**o+(0.267840571071)*x_ref[0]+(0.952106457495)*x_ref[1]**o+(-0.943845364366)*x_ref[1]
        ref[3,3,1,1]=(-0.21692720916)*x_ref[0]**o+(0.580841374997)*x_ref[0]+(-0.412129341223)*x_ref[1]**o+(0.40414445304)*x_ref[1]
        ref[3,3,2,0]=(-0.447758598726)*x_ref[0]**o+(0.0616926352912)*x_ref[0]+(-0.824624226122)*x_ref[1]**o+(0.19099833104)*x_ref[1]
        ref[3,3,2,1]=(-0.615519138165)*x_ref[0]**o+(0.233926822164)*x_ref[0]+(-0.969014060236)*x_ref[1]**o+(0.835103627893)*x_ref[1]
        ref[3,4,0,0]=(0.550778983301)*x_ref[0]**o+(0.0198253530882)*x_ref[0]+(-0.39383849187)*x_ref[1]**o+(0.367721826304)*x_ref[1]
        ref[3,4,0,1]=(0.444877923149)*x_ref[0]**o+(-0.899159835223)*x_ref[0]+(0.923392540452)*x_ref[1]**o+(0.512905141718)*x_ref[1]
        ref[3,4,1,0]=(-0.563807675145)*x_ref[0]**o+(0.994405383899)*x_ref[0]+(-0.777352948218)*x_ref[1]**o+(0.865994906227)*x_ref[1]
        ref[3,4,1,1]=(-0.593800452139)*x_ref[0]**o+(0.613778803237)*x_ref[0]+(0.582200979718)*x_ref[1]**o+(0.448026243271)*x_ref[1]
        ref[3,4,2,0]=(-0.900331125416)*x_ref[0]**o+(0.234613965424)*x_ref[0]+(0.510400273805)*x_ref[1]**o+(0.0836773641661)*x_ref[1]
        ref[3,4,2,1]=(0.710999097307)*x_ref[0]**o+(0.886603988265)*x_ref[0]+(-0.405644887679)*x_ref[1]**o+(0.362257258809)*x_ref[1]
      else:
        arg[0,0,0,0]=(0.310272924207)*x[0]**o+(-0.664497806351)*x[0]+(0.741942132609)*x[1]**o+(0.636051298779)*x[1]+(0.76627159146)*x[2]**o+(0.492960732588)*x[2]
        arg[0,0,0,1]=(-0.513740801773)*x[0]**o+(-0.633330750909)*x[0]+(0.177934618983)*x[1]**o+(-0.323745723166)*x[1]+(0.694167180904)*x[2]**o+(0.441239251435)*x[2]
        arg[0,0,1,0]=(0.417477695848)*x[0]**o+(-0.923415976002)*x[0]+(0.0765555276662)*x[1]**o+(-0.277290890834)*x[1]+(0.0177191326614)*x[2]**o+(-0.0842520100387)*x[2]
        arg[0,0,1,1]=(-0.761916249894)*x[0]**o+(0.169673139552)*x[0]+(0.0306290828795)*x[1]**o+(0.842234592613)*x[1]+(-0.350342692669)*x[2]**o+(0.321292270658)*x[2]
        arg[0,0,2,0]=(-0.707494137624)*x[0]**o+(0.346873490395)*x[0]+(-0.68281925138)*x[1]**o+(-0.830538083274)*x[1]+(0.329372141613)*x[2]**o+(-0.121366157163)*x[2]
        arg[0,0,2,1]=(-0.708018186507)*x[0]**o+(-0.778051239314)*x[0]+(-0.43106291563)*x[1]**o+(-0.0115914251143)*x[1]+(0.487334215892)*x[2]**o+(-0.289232284658)*x[2]
        arg[0,1,0,0]=(0.530614704199)*x[0]**o+(0.518622028317)*x[0]+(-0.0143129589551)*x[1]**o+(0.808469352257)*x[1]+(-0.156787373225)*x[2]**o+(0.668887813143)*x[2]
        arg[0,1,0,1]=(0.953223233706)*x[0]**o+(-0.164323838353)*x[0]+(0.336717320858)*x[1]**o+(-0.520230488172)*x[1]+(0.689960738002)*x[2]**o+(0.487630947578)*x[2]
        arg[0,1,1,0]=(-0.418095373429)*x[0]**o+(-0.941817844939)*x[0]+(0.88105115607)*x[1]**o+(-0.820502501439)*x[1]+(-0.693858267667)*x[2]**o+(0.530567713127)*x[2]
        arg[0,1,1,1]=(-0.716861320218)*x[0]**o+(-0.736619859467)*x[0]+(-0.885478605601)*x[1]**o+(-0.129375578967)*x[1]+(-0.0181433907318)*x[2]**o+(0.373255537959)*x[2]
        arg[0,1,2,0]=(-0.174742583167)*x[0]**o+(-0.0887967480112)*x[0]+(-0.291168698662)*x[1]**o+(-0.188608506769)*x[1]+(0.22479613454)*x[2]**o+(0.34173928582)*x[2]
        arg[0,1,2,1]=(0.117208929504)*x[0]**o+(-0.420969282079)*x[0]+(0.585298109836)*x[1]**o+(0.771355162745)*x[1]+(-0.51374493747)*x[2]**o+(0.785016548855)*x[2]
        arg[0,2,0,0]=(0.737394887886)*x[0]**o+(0.966080271342)*x[0]+(-0.373178384492)*x[1]**o+(-0.845505386142)*x[1]+(-0.274654519371)*x[2]**o+(0.709356557887)*x[2]
        arg[0,2,0,1]=(0.0365864090636)*x[0]**o+(-0.393382479787)*x[0]+(0.018273360977)*x[1]**o+(-0.509201909328)*x[1]+(-0.777852392534)*x[2]**o+(0.0417280247507)*x[2]
        arg[0,2,1,0]=(0.637829302751)*x[0]**o+(-0.767824790186)*x[0]+(0.843382009511)*x[1]**o+(-0.599633234403)*x[1]+(0.14283138941)*x[2]**o+(-0.354404633991)*x[2]
        arg[0,2,1,1]=(-0.465258613599)*x[0]**o+(0.09341154383)*x[0]+(-0.629114782988)*x[1]**o+(0.91608655383)*x[1]+(-0.692901135848)*x[2]**o+(-0.707225310667)*x[2]
        arg[0,2,2,0]=(0.9935958759)*x[0]**o+(-0.385008742466)*x[0]+(0.153885356854)*x[1]**o+(0.146270680656)*x[1]+(0.305838410343)*x[2]**o+(0.141193552717)*x[2]
        arg[0,2,2,1]=(-0.487538023894)*x[0]**o+(-0.814283442647)*x[0]+(0.166008520199)*x[1]**o+(0.313655774416)*x[1]+(0.282308782973)*x[2]**o+(0.309176062484)*x[2]
        arg[0,3,0,0]=(0.00203507798913)*x[0]**o+(-0.19902542604)*x[0]+(0.88110918624)*x[1]**o+(0.223549219288)*x[1]+(0.135758952369)*x[2]**o+(0.59116783639)*x[2]
        arg[0,3,0,1]=(-0.185063700767)*x[0]**o+(0.833862155173)*x[0]+(-0.0939117509645)*x[1]**o+(0.288003692231)*x[1]+(0.580196157839)*x[2]**o+(0.299320177754)*x[2]
        arg[0,3,1,0]=(-0.039100718826)*x[0]**o+(0.423179869395)*x[0]+(-0.013669578975)*x[1]**o+(-0.355053343085)*x[1]+(0.89285158975)*x[2]**o+(0.392984367877)*x[2]
        arg[0,3,1,1]=(0.0988989416429)*x[0]**o+(-0.446243235174)*x[0]+(-0.691253313617)*x[1]**o+(0.338823693086)*x[1]+(0.707054126334)*x[2]**o+(0.601121388278)*x[2]
        arg[0,3,2,0]=(-0.191366770484)*x[0]**o+(0.961791275903)*x[0]+(-0.103547475035)*x[1]**o+(-0.745236510079)*x[1]+(-0.157506981712)*x[2]**o+(0.986229506787)*x[2]
        arg[0,3,2,1]=(-0.886091883647)*x[0]**o+(0.6979804248)*x[0]+(-0.366575419428)*x[1]**o+(-0.958601153259)*x[1]+(-0.0119724511494)*x[2]**o+(-0.76101132076)*x[2]
        arg[0,4,0,0]=(0.923290554434)*x[0]**o+(0.411359492198)*x[0]+(-0.977999562875)*x[1]**o+(0.352085161527)*x[1]+(0.955745235349)*x[2]**o+(-0.978885609257)*x[2]
        arg[0,4,0,1]=(0.226973479105)*x[0]**o+(-0.423881897251)*x[0]+(0.889101969481)*x[1]**o+(-0.994236389028)*x[1]+(0.0159169033361)*x[2]**o+(-0.559935154203)*x[2]
        arg[0,4,1,0]=(-0.225427291797)*x[0]**o+(-0.557334621848)*x[0]+(-0.486252586142)*x[1]**o+(0.0497120905216)*x[1]+(0.84165736691)*x[2]**o+(-0.185010156797)*x[2]
        arg[0,4,1,1]=(0.762694602609)*x[0]**o+(-0.447367170727)*x[0]+(-0.61103181818)*x[1]**o+(0.689761057656)*x[1]+(-0.631855120122)*x[2]**o+(0.33274655602)*x[2]
        arg[0,4,2,0]=(-0.198884868914)*x[0]**o+(0.716371959278)*x[0]+(0.260871768919)*x[1]**o+(0.622241740082)*x[1]+(0.774103524035)*x[2]**o+(0.186132703676)*x[2]
        arg[0,4,2,1]=(-0.309450645317)*x[0]**o+(0.204754571336)*x[0]+(0.483402810171)*x[1]**o+(0.404620744188)*x[1]+(-0.699137246959)*x[2]**o+(0.467428706196)*x[2]
        arg[1,0,0,0]=(0.642289506448)*x[0]**o+(0.383990304394)*x[0]+(0.556940973318)*x[1]**o+(0.135219729041)*x[1]+(0.577195836358)*x[2]**o+(-0.338924419674)*x[2]
        arg[1,0,0,1]=(-0.920873911295)*x[0]**o+(-0.330610192304)*x[0]+(-0.117226803856)*x[1]**o+(0.172482867684)*x[1]+(0.909140902709)*x[2]**o+(-0.590494167913)*x[2]
        arg[1,0,1,0]=(0.210161538861)*x[0]**o+(-0.83860241761)*x[0]+(0.0257180382661)*x[1]**o+(0.877633854348)*x[1]+(0.26045311754)*x[2]**o+(-0.971349902714)*x[2]
        arg[1,0,1,1]=(0.70736843125)*x[0]**o+(-0.765121985431)*x[0]+(-0.196525225943)*x[1]**o+(-0.175928729346)*x[1]+(0.945523028748)*x[2]**o+(0.877830322823)*x[2]
        arg[1,0,2,0]=(-0.963513016065)*x[0]**o+(0.0820537856351)*x[0]+(0.0122070196118)*x[1]**o+(-0.154147449592)*x[1]+(0.323844633268)*x[2]**o+(-0.124897698214)*x[2]
        arg[1,0,2,1]=(0.656267566405)*x[0]**o+(0.0174786620768)*x[0]+(0.763902435618)*x[1]**o+(-0.986021506043)*x[1]+(0.632625555994)*x[2]**o+(-0.110657743523)*x[2]
        arg[1,1,0,0]=(0.00354144688732)*x[0]**o+(-0.813515808171)*x[0]+(-0.100717717446)*x[1]**o+(0.0543708163629)*x[1]+(0.785927880762)*x[2]**o+(0.0109166652227)*x[2]
        arg[1,1,0,1]=(-0.239705688631)*x[0]**o+(0.402915642509)*x[0]+(0.463371068163)*x[1]**o+(-0.507302462928)*x[1]+(-0.0789844128295)*x[2]**o+(0.669144891499)*x[2]
        arg[1,1,1,0]=(-0.491891367001)*x[0]**o+(-0.524377636928)*x[0]+(0.903091863917)*x[1]**o+(-0.554368807183)*x[1]+(0.332899195339)*x[2]**o+(-0.645216147049)*x[2]
        arg[1,1,1,1]=(-0.915843941243)*x[0]**o+(-0.233175654748)*x[0]+(0.467730425037)*x[1]**o+(-0.765957936683)*x[1]+(0.726678877276)*x[2]**o+(0.770533947942)*x[2]
        arg[1,1,2,0]=(0.678071852032)*x[0]**o+(0.266537844237)*x[0]+(0.324335300151)*x[1]**o+(-0.286713971373)*x[1]+(-0.702384414014)*x[2]**o+(0.887198276519)*x[2]
        arg[1,1,2,1]=(0.0466444959648)*x[0]**o+(0.689162912677)*x[0]+(-0.35033923676)*x[1]**o+(-0.0536380766876)*x[1]+(-0.511833442517)*x[2]**o+(0.125566299829)*x[2]
        arg[1,2,0,0]=(-0.803170431622)*x[0]**o+(0.476211169752)*x[0]+(-0.170510267953)*x[1]**o+(-0.765945735317)*x[1]+(-0.169966329291)*x[2]**o+(-0.988525918724)*x[2]
        arg[1,2,0,1]=(0.289618751212)*x[0]**o+(0.294379040976)*x[0]+(-0.922478416838)*x[1]**o+(0.210546495537)*x[1]+(-0.87266381496)*x[2]**o+(-0.865571669577)*x[2]
        arg[1,2,1,0]=(-0.292811054005)*x[0]**o+(0.651280833746)*x[0]+(-0.625004203736)*x[1]**o+(0.372078169172)*x[1]+(-0.0539099055169)*x[2]**o+(0.0911869270008)*x[2]
        arg[1,2,1,1]=(0.496217698294)*x[0]**o+(-0.986021787133)*x[0]+(0.246417059532)*x[1]**o+(0.610226802401)*x[1]+(0.991534665298)*x[2]**o+(-0.507441185982)*x[2]
        arg[1,2,2,0]=(-0.839902565947)*x[0]**o+(-0.279607147278)*x[0]+(0.278212307963)*x[1]**o+(0.609793640036)*x[1]+(0.940471913774)*x[2]**o+(0.21860828705)*x[2]
        arg[1,2,2,1]=(0.789578112278)*x[0]**o+(-0.0226697818608)*x[0]+(0.216321340291)*x[1]**o+(-0.701818463454)*x[1]+(-0.166689030646)*x[2]**o+(0.966131542292)*x[2]
        arg[1,3,0,0]=(0.149337042551)*x[0]**o+(-0.785050579674)*x[0]+(0.530219924721)*x[1]**o+(-0.971927239398)*x[1]+(0.950917908603)*x[2]**o+(0.132668077447)*x[2]
        arg[1,3,0,1]=(0.983081559724)*x[0]**o+(-0.358694498455)*x[0]+(0.181080099281)*x[1]**o+(-0.699756291643)*x[1]+(-0.744335953304)*x[2]**o+(0.00262043221382)*x[2]
        arg[1,3,1,0]=(0.31776032045)*x[0]**o+(-0.0726184220029)*x[0]+(-0.759883735971)*x[1]**o+(-0.6389710873)*x[1]+(-0.607858499502)*x[2]**o+(-0.397169895784)*x[2]
        arg[1,3,1,1]=(-0.950468389728)*x[0]**o+(0.831121915521)*x[0]+(0.552857011083)*x[1]**o+(0.848599672594)*x[1]+(0.432183564231)*x[2]**o+(0.57537721362)*x[2]
        arg[1,3,2,0]=(0.736080143601)*x[0]**o+(-0.804140307747)*x[0]+(-0.163864867884)*x[1]**o+(0.950874042315)*x[1]+(-0.99676139412)*x[2]**o+(0.332320294246)*x[2]
        arg[1,3,2,1]=(0.119082582041)*x[0]**o+(-0.969740778852)*x[0]+(0.828432583532)*x[1]**o+(0.190324050395)*x[1]+(-0.42639142674)*x[2]**o+(0.315519578192)*x[2]
        arg[1,4,0,0]=(-0.798465141775)*x[0]**o+(-0.717282977908)*x[0]+(0.935655148713)*x[1]**o+(-0.655752964484)*x[1]+(0.159990149099)*x[2]**o+(-0.537115062687)*x[2]
        arg[1,4,0,1]=(0.445151775007)*x[0]**o+(-0.867956302051)*x[0]+(-0.809856879825)*x[1]**o+(-0.326897750981)*x[1]+(0.756352311987)*x[2]**o+(0.577585513459)*x[2]
        arg[1,4,1,0]=(0.294243347591)*x[0]**o+(0.774741656931)*x[0]+(-0.0799985737984)*x[1]**o+(0.179789107412)*x[1]+(-0.612198025189)*x[2]**o+(-0.07086009611)*x[2]
        arg[1,4,1,1]=(0.0692586287045)*x[0]**o+(-0.671956300122)*x[0]+(-0.948667104683)*x[1]**o+(0.68656246801)*x[1]+(-0.352975297696)*x[2]**o+(0.277952113716)*x[2]
        arg[1,4,2,0]=(-0.491316095517)*x[0]**o+(0.725899087239)*x[0]+(0.634113963431)*x[1]**o+(0.235694393453)*x[1]+(-0.15600720713)*x[2]**o+(0.954814976539)*x[2]
        arg[1,4,2,1]=(0.120206996339)*x[0]**o+(-0.86151411823)*x[0]+(-0.137093650097)*x[1]**o+(0.633402978246)*x[1]+(-0.42515364728)*x[2]**o+(-0.987634210617)*x[2]
        arg[2,0,0,0]=(0.539463971222)*x[0]**o+(-0.03435650579)*x[0]+(-0.884395295544)*x[1]**o+(0.215454879022)*x[1]+(0.842377328246)*x[2]**o+(0.370920136871)*x[2]
        arg[2,0,0,1]=(-0.411505463039)*x[0]**o+(0.446494039439)*x[0]+(0.0313771969556)*x[1]**o+(0.602893223061)*x[1]+(-0.30383490021)*x[2]**o+(-0.788028756758)*x[2]
        arg[2,0,1,0]=(0.489009615999)*x[0]**o+(-0.95508248016)*x[0]+(-0.21489475022)*x[1]**o+(-0.857263478431)*x[1]+(-0.123703409045)*x[2]**o+(0.663389562431)*x[2]
        arg[2,0,1,1]=(-0.0224763310929)*x[0]**o+(-0.315886249237)*x[0]+(-0.318201849695)*x[1]**o+(0.423732682462)*x[1]+(0.396065636381)*x[2]**o+(0.0816717878865)*x[2]
        arg[2,0,2,0]=(-0.191432618224)*x[0]**o+(0.688389390795)*x[0]+(-0.382948112159)*x[1]**o+(0.562905173818)*x[1]+(-0.118396714029)*x[2]**o+(0.234478642577)*x[2]
        arg[2,0,2,1]=(-0.348008173171)*x[0]**o+(0.191994166636)*x[0]+(0.450556057147)*x[1]**o+(-0.710821693343)*x[1]+(-0.245119333857)*x[2]**o+(0.262869525239)*x[2]
        arg[2,1,0,0]=(-0.160444506203)*x[0]**o+(-0.816142837432)*x[0]+(-0.619722480492)*x[1]**o+(-0.989077039675)*x[1]+(-0.0282349103977)*x[2]**o+(-0.629849819691)*x[2]
        arg[2,1,0,1]=(0.379513765297)*x[0]**o+(0.934652105586)*x[0]+(-0.632636427868)*x[1]**o+(-0.0492550496553)*x[1]+(0.137733572307)*x[2]**o+(0.779782637942)*x[2]
        arg[2,1,1,0]=(0.469499320417)*x[0]**o+(0.570880924125)*x[0]+(-0.0329833251419)*x[1]**o+(-0.0388751034604)*x[1]+(-0.878520094389)*x[2]**o+(0.992656649048)*x[2]
        arg[2,1,1,1]=(0.592005181995)*x[0]**o+(-0.553980493526)*x[0]+(0.359837665019)*x[1]**o+(0.811107941179)*x[1]+(-0.441929650912)*x[2]**o+(-0.768618378636)*x[2]
        arg[2,1,2,0]=(0.741376258816)*x[0]**o+(-0.238909756717)*x[0]+(0.204631082449)*x[1]**o+(0.970501662949)*x[1]+(-0.615638092239)*x[2]**o+(-0.575948750844)*x[2]
        arg[2,1,2,1]=(0.774050933112)*x[0]**o+(-0.872234076721)*x[0]+(-0.0936189240061)*x[1]**o+(-0.773608756961)*x[1]+(-0.119171907808)*x[2]**o+(-0.256124796845)*x[2]
        arg[2,2,0,0]=(0.459481754425)*x[0]**o+(0.478364929493)*x[0]+(0.460199733287)*x[1]**o+(0.66880294659)*x[1]+(-0.859525058389)*x[2]**o+(0.77960284414)*x[2]
        arg[2,2,0,1]=(0.690557835207)*x[0]**o+(-0.812305038592)*x[0]+(0.837924124791)*x[1]**o+(-0.785979206364)*x[1]+(-0.295553949819)*x[2]**o+(0.575096295897)*x[2]
        arg[2,2,1,0]=(0.443193683205)*x[0]**o+(0.0355033222253)*x[0]+(0.497641002766)*x[1]**o+(-0.636955142451)*x[1]+(-0.265324357454)*x[2]**o+(0.734148569078)*x[2]
        arg[2,2,1,1]=(-0.088707242599)*x[0]**o+(0.513403518669)*x[0]+(0.337741737651)*x[1]**o+(-0.174843924478)*x[1]+(0.687291255986)*x[2]**o+(0.473850502258)*x[2]
        arg[2,2,2,0]=(0.5305624642)*x[0]**o+(-0.972997835953)*x[0]+(0.00197107287888)*x[1]**o+(-0.75490872634)*x[1]+(-0.0569624569435)*x[2]**o+(-0.0780532868631)*x[2]
        arg[2,2,2,1]=(-0.744584471866)*x[0]**o+(0.915494909272)*x[0]+(0.239235050038)*x[1]**o+(-0.192466753187)*x[1]+(0.933556550824)*x[2]**o+(-0.786420251487)*x[2]
        arg[2,3,0,0]=(0.472114530627)*x[0]**o+(0.0693211807557)*x[0]+(-0.274273425713)*x[1]**o+(0.136242554894)*x[1]+(-0.329240675785)*x[2]**o+(0.0212539247343)*x[2]
        arg[2,3,0,1]=(-0.780426108099)*x[0]**o+(-0.632382551025)*x[0]+(0.554926949239)*x[1]**o+(0.264204009059)*x[1]+(-0.868007525824)*x[2]**o+(-0.937631120201)*x[2]
        arg[2,3,1,0]=(-0.826692080947)*x[0]**o+(0.915241858538)*x[0]+(-0.49986160859)*x[1]**o+(0.921686464658)*x[1]+(0.71996666304)*x[2]**o+(0.297609501096)*x[2]
        arg[2,3,1,1]=(0.991896994614)*x[0]**o+(-0.543310008103)*x[0]+(0.0791899631111)*x[1]**o+(-0.914088200145)*x[1]+(-0.496337624951)*x[2]**o+(0.708950898614)*x[2]
        arg[2,3,2,0]=(-0.235116966432)*x[0]**o+(-0.492848730207)*x[0]+(0.428728646789)*x[1]**o+(-0.410747634691)*x[1]+(0.705059815458)*x[2]**o+(-0.185604633831)*x[2]
        arg[2,3,2,1]=(0.838829037117)*x[0]**o+(0.670561147563)*x[0]+(0.621407965075)*x[1]**o+(-0.10624946305)*x[1]+(0.893703402687)*x[2]**o+(0.985504995821)*x[2]
        arg[2,4,0,0]=(0.635072504682)*x[0]**o+(-0.681962427555)*x[0]+(-0.00774518011081)*x[1]**o+(-0.264996228668)*x[1]+(0.690668403198)*x[2]**o+(0.883573761062)*x[2]
        arg[2,4,0,1]=(0.545632268651)*x[0]**o+(0.439668519394)*x[0]+(-0.482479808491)*x[1]**o+(0.752028962391)*x[1]+(0.0596773623939)*x[2]**o+(-0.369987181127)*x[2]
        arg[2,4,1,0]=(0.659005322796)*x[0]**o+(0.503940833428)*x[0]+(0.643633321586)*x[1]**o+(0.927082323252)*x[1]+(0.354072146184)*x[2]**o+(-0.638737560608)*x[2]
        arg[2,4,1,1]=(-0.443548860913)*x[0]**o+(0.707607048458)*x[0]+(-0.955225746402)*x[1]**o+(0.515995563585)*x[1]+(-0.288406637165)*x[2]**o+(-0.760864937318)*x[2]
        arg[2,4,2,0]=(0.898146580355)*x[0]**o+(-0.590650787106)*x[0]+(0.036095789753)*x[1]**o+(0.326040260581)*x[1]+(0.680069361306)*x[2]**o+(0.333839099341)*x[2]
        arg[2,4,2,1]=(-0.151364260271)*x[0]**o+(-0.675198183832)*x[0]+(-0.183626298978)*x[1]**o+(-0.182565217146)*x[1]+(0.0345632129814)*x[2]**o+(-0.581534100763)*x[2]
        arg[3,0,0,0]=(-0.942351703246)*x[0]**o+(-0.699567649453)*x[0]+(-0.832068665702)*x[1]**o+(0.163439416504)*x[1]+(0.742500339519)*x[2]**o+(-0.492882887489)*x[2]
        arg[3,0,0,1]=(0.990078296484)*x[0]**o+(-0.999837336587)*x[0]+(0.197589456459)*x[1]**o+(0.431403059019)*x[1]+(0.859864015871)*x[2]**o+(-0.267474829282)*x[2]
        arg[3,0,1,0]=(-0.776773660723)*x[0]**o+(-0.189048370486)*x[0]+(0.926953856802)*x[1]**o+(-0.72314744652)*x[1]+(0.330064363809)*x[2]**o+(0.188533334359)*x[2]
        arg[3,0,1,1]=(-0.854830058762)*x[0]**o+(0.832073883276)*x[0]+(-0.425691165124)*x[1]**o+(0.162351500316)*x[1]+(0.370604864512)*x[2]**o+(-0.581907071484)*x[2]
        arg[3,0,2,0]=(-0.316716909548)*x[0]**o+(0.335417284383)*x[0]+(-0.783061160352)*x[1]**o+(-0.253972519239)*x[1]+(-0.826432933721)*x[2]**o+(0.267368378363)*x[2]
        arg[3,0,2,1]=(-0.765899705462)*x[0]**o+(-0.0993875861476)*x[0]+(-0.211785004352)*x[1]**o+(-0.635330151497)*x[1]+(0.414575412599)*x[2]**o+(0.586882274623)*x[2]
        arg[3,1,0,0]=(0.987776377237)*x[0]**o+(0.796660669191)*x[0]+(0.580724891862)*x[1]**o+(-0.24149253995)*x[1]+(-0.633421016909)*x[2]**o+(-0.220110913711)*x[2]
        arg[3,1,0,1]=(-0.182240918081)*x[0]**o+(-0.290526050572)*x[0]+(0.00283230959976)*x[1]**o+(-0.933282166805)*x[1]+(-0.857893235883)*x[2]**o+(0.551414074127)*x[2]
        arg[3,1,1,0]=(-0.854435997816)*x[0]**o+(0.706455144263)*x[0]+(0.406237965254)*x[1]**o+(0.192071142415)*x[1]+(0.357966796834)*x[2]**o+(0.887011087987)*x[2]
        arg[3,1,1,1]=(0.832013396996)*x[0]**o+(0.696133072321)*x[0]+(0.682726968712)*x[1]**o+(0.613316240514)*x[1]+(0.12600339887)*x[2]**o+(-0.620088231382)*x[2]
        arg[3,1,2,0]=(-0.178681615221)*x[0]**o+(-0.952835986168)*x[0]+(0.299692693746)*x[1]**o+(0.892451908416)*x[1]+(0.202403273634)*x[2]**o+(0.677212897943)*x[2]
        arg[3,1,2,1]=(0.0592594053342)*x[0]**o+(0.56391153959)*x[0]+(-0.874866627667)*x[1]**o+(0.432102900769)*x[1]+(-0.49073126285)*x[2]**o+(-0.010495860028)*x[2]
        arg[3,2,0,0]=(0.548006445814)*x[0]**o+(0.667313392202)*x[0]+(0.397805355499)*x[1]**o+(-0.788100267808)*x[1]+(0.869234055258)*x[2]**o+(-0.830241520401)*x[2]
        arg[3,2,0,1]=(0.72586139216)*x[0]**o+(-0.931878957878)*x[0]+(-0.94119555858)*x[1]**o+(-0.133544806169)*x[1]+(0.754263323382)*x[2]**o+(-0.446604740157)*x[2]
        arg[3,2,1,0]=(-0.28135830707)*x[0]**o+(-0.478744058518)*x[0]+(0.88219033731)*x[1]**o+(0.460587286945)*x[1]+(-0.849692146096)*x[2]**o+(0.0927701345107)*x[2]
        arg[3,2,1,1]=(-0.681859641611)*x[0]**o+(-0.670164293088)*x[0]+(0.796366374435)*x[1]**o+(0.837230317054)*x[1]+(-0.413283915904)*x[2]**o+(0.779624227187)*x[2]
        arg[3,2,2,0]=(-0.803242771255)*x[0]**o+(-0.0687616271776)*x[0]+(-0.948856027999)*x[1]**o+(0.762568790931)*x[1]+(0.937588292604)*x[2]**o+(-0.0224232611283)*x[2]
        arg[3,2,2,1]=(-0.215752944937)*x[0]**o+(-0.807114296075)*x[0]+(0.970625375964)*x[1]**o+(0.68578124692)*x[1]+(-0.165007047419)*x[2]**o+(0.942718439981)*x[2]
        arg[3,3,0,0]=(0.152001104716)*x[0]**o+(-0.657242342357)*x[0]+(0.559608661705)*x[1]**o+(0.0764403273251)*x[1]+(0.633414754695)*x[2]**o+(0.430777868344)*x[2]
        arg[3,3,0,1]=(-0.898109497759)*x[0]**o+(0.736574249406)*x[0]+(-0.569584184574)*x[1]**o+(-0.145501395506)*x[1]+(-0.412424116475)*x[2]**o+(-0.416350923096)*x[2]
        arg[3,3,1,0]=(0.602022795614)*x[0]**o+(-0.848525569825)*x[0]+(-0.713426287505)*x[1]**o+(-0.829278452687)*x[1]+(-0.491257659316)*x[2]**o+(0.90219908999)*x[2]
        arg[3,3,1,1]=(-0.769110216066)*x[0]**o+(-0.789464037338)*x[0]+(-0.363686172584)*x[1]**o+(-0.86190787068)*x[1]+(0.275438119232)*x[2]**o+(0.285209834248)*x[2]
        arg[3,3,2,0]=(0.351662405853)*x[0]**o+(0.8403302307)*x[0]+(-0.449869332519)*x[1]**o+(0.936043614838)*x[1]+(-0.656908777335)*x[2]**o+(-0.231923872531)*x[2]
        arg[3,3,2,1]=(0.873848945653)*x[0]**o+(0.551429883959)*x[0]+(-0.577942548701)*x[1]**o+(-0.254317163049)*x[1]+(0.64878089878)*x[2]**o+(0.7135339515)*x[2]
        arg[3,4,0,0]=(0.891435610494)*x[0]**o+(0.598893704498)*x[0]+(0.24682094756)*x[1]**o+(0.0895612710338)*x[1]+(-0.513686244953)*x[2]**o+(0.307205505502)*x[2]
        arg[3,4,0,1]=(-0.577752944196)*x[0]**o+(-0.458383751623)*x[0]+(-0.913956988143)*x[1]**o+(0.470857602235)*x[1]+(0.631726850665)*x[2]**o+(-0.112962289904)*x[2]
        arg[3,4,1,0]=(-0.65648177489)*x[0]**o+(-0.843530972747)*x[0]+(-0.0332153723809)*x[1]**o+(0.230484263355)*x[1]+(-0.865687508592)*x[2]**o+(0.122751735082)*x[2]
        arg[3,4,1,1]=(-0.353502640734)*x[0]**o+(-0.816020900282)*x[0]+(0.943688796868)*x[1]**o+(-0.526707881958)*x[1]+(-0.790258298117)*x[2]**o+(-0.748706809042)*x[2]
        arg[3,4,2,0]=(-0.596345912175)*x[0]**o+(0.872771121837)*x[0]+(0.100107395575)*x[1]**o+(0.950482249217)*x[1]+(0.0487634582783)*x[2]**o+(-0.0285218097887)*x[2]
        arg[3,4,2,1]=(-0.827443553933)*x[0]**o+(-0.922564800149)*x[0]+(-0.810354385287)*x[1]**o+(0.630079217293)*x[1]+(-0.316280559723)*x[2]**o+(0.977830435871)*x[2]
        ref[0,0,0,0]=(0.310272924207)*x_ref[0]**o+(-0.664497806351)*x_ref[0]+(0.741942132609)*x_ref[1]**o+(0.636051298779)*x_ref[1]+(0.76627159146)*x_ref[2]**o+(0.492960732588)*x_ref[2]
        ref[0,0,0,1]=(-0.513740801773)*x_ref[0]**o+(-0.633330750909)*x_ref[0]+(0.177934618983)*x_ref[1]**o+(-0.323745723166)*x_ref[1]+(0.694167180904)*x_ref[2]**o+(0.441239251435)*x_ref[2]
        ref[0,0,1,0]=(0.417477695848)*x_ref[0]**o+(-0.923415976002)*x_ref[0]+(0.0765555276662)*x_ref[1]**o+(-0.277290890834)*x_ref[1]+(0.0177191326614)*x_ref[2]**o+(-0.0842520100387)*x_ref[2]
        ref[0,0,1,1]=(-0.761916249894)*x_ref[0]**o+(0.169673139552)*x_ref[0]+(0.0306290828795)*x_ref[1]**o+(0.842234592613)*x_ref[1]+(-0.350342692669)*x_ref[2]**o+(0.321292270658)*x_ref[2]
        ref[0,0,2,0]=(-0.707494137624)*x_ref[0]**o+(0.346873490395)*x_ref[0]+(-0.68281925138)*x_ref[1]**o+(-0.830538083274)*x_ref[1]+(0.329372141613)*x_ref[2]**o+(-0.121366157163)*x_ref[2]
        ref[0,0,2,1]=(-0.708018186507)*x_ref[0]**o+(-0.778051239314)*x_ref[0]+(-0.43106291563)*x_ref[1]**o+(-0.0115914251143)*x_ref[1]+(0.487334215892)*x_ref[2]**o+(-0.289232284658)*x_ref[2]
        ref[0,1,0,0]=(0.530614704199)*x_ref[0]**o+(0.518622028317)*x_ref[0]+(-0.0143129589551)*x_ref[1]**o+(0.808469352257)*x_ref[1]+(-0.156787373225)*x_ref[2]**o+(0.668887813143)*x_ref[2]
        ref[0,1,0,1]=(0.953223233706)*x_ref[0]**o+(-0.164323838353)*x_ref[0]+(0.336717320858)*x_ref[1]**o+(-0.520230488172)*x_ref[1]+(0.689960738002)*x_ref[2]**o+(0.487630947578)*x_ref[2]
        ref[0,1,1,0]=(-0.418095373429)*x_ref[0]**o+(-0.941817844939)*x_ref[0]+(0.88105115607)*x_ref[1]**o+(-0.820502501439)*x_ref[1]+(-0.693858267667)*x_ref[2]**o+(0.530567713127)*x_ref[2]
        ref[0,1,1,1]=(-0.716861320218)*x_ref[0]**o+(-0.736619859467)*x_ref[0]+(-0.885478605601)*x_ref[1]**o+(-0.129375578967)*x_ref[1]+(-0.0181433907318)*x_ref[2]**o+(0.373255537959)*x_ref[2]
        ref[0,1,2,0]=(-0.174742583167)*x_ref[0]**o+(-0.0887967480112)*x_ref[0]+(-0.291168698662)*x_ref[1]**o+(-0.188608506769)*x_ref[1]+(0.22479613454)*x_ref[2]**o+(0.34173928582)*x_ref[2]
        ref[0,1,2,1]=(0.117208929504)*x_ref[0]**o+(-0.420969282079)*x_ref[0]+(0.585298109836)*x_ref[1]**o+(0.771355162745)*x_ref[1]+(-0.51374493747)*x_ref[2]**o+(0.785016548855)*x_ref[2]
        ref[0,2,0,0]=(0.737394887886)*x_ref[0]**o+(0.966080271342)*x_ref[0]+(-0.373178384492)*x_ref[1]**o+(-0.845505386142)*x_ref[1]+(-0.274654519371)*x_ref[2]**o+(0.709356557887)*x_ref[2]
        ref[0,2,0,1]=(0.0365864090636)*x_ref[0]**o+(-0.393382479787)*x_ref[0]+(0.018273360977)*x_ref[1]**o+(-0.509201909328)*x_ref[1]+(-0.777852392534)*x_ref[2]**o+(0.0417280247507)*x_ref[2]
        ref[0,2,1,0]=(0.637829302751)*x_ref[0]**o+(-0.767824790186)*x_ref[0]+(0.843382009511)*x_ref[1]**o+(-0.599633234403)*x_ref[1]+(0.14283138941)*x_ref[2]**o+(-0.354404633991)*x_ref[2]
        ref[0,2,1,1]=(-0.465258613599)*x_ref[0]**o+(0.09341154383)*x_ref[0]+(-0.629114782988)*x_ref[1]**o+(0.91608655383)*x_ref[1]+(-0.692901135848)*x_ref[2]**o+(-0.707225310667)*x_ref[2]
        ref[0,2,2,0]=(0.9935958759)*x_ref[0]**o+(-0.385008742466)*x_ref[0]+(0.153885356854)*x_ref[1]**o+(0.146270680656)*x_ref[1]+(0.305838410343)*x_ref[2]**o+(0.141193552717)*x_ref[2]
        ref[0,2,2,1]=(-0.487538023894)*x_ref[0]**o+(-0.814283442647)*x_ref[0]+(0.166008520199)*x_ref[1]**o+(0.313655774416)*x_ref[1]+(0.282308782973)*x_ref[2]**o+(0.309176062484)*x_ref[2]
        ref[0,3,0,0]=(0.00203507798913)*x_ref[0]**o+(-0.19902542604)*x_ref[0]+(0.88110918624)*x_ref[1]**o+(0.223549219288)*x_ref[1]+(0.135758952369)*x_ref[2]**o+(0.59116783639)*x_ref[2]
        ref[0,3,0,1]=(-0.185063700767)*x_ref[0]**o+(0.833862155173)*x_ref[0]+(-0.0939117509645)*x_ref[1]**o+(0.288003692231)*x_ref[1]+(0.580196157839)*x_ref[2]**o+(0.299320177754)*x_ref[2]
        ref[0,3,1,0]=(-0.039100718826)*x_ref[0]**o+(0.423179869395)*x_ref[0]+(-0.013669578975)*x_ref[1]**o+(-0.355053343085)*x_ref[1]+(0.89285158975)*x_ref[2]**o+(0.392984367877)*x_ref[2]
        ref[0,3,1,1]=(0.0988989416429)*x_ref[0]**o+(-0.446243235174)*x_ref[0]+(-0.691253313617)*x_ref[1]**o+(0.338823693086)*x_ref[1]+(0.707054126334)*x_ref[2]**o+(0.601121388278)*x_ref[2]
        ref[0,3,2,0]=(-0.191366770484)*x_ref[0]**o+(0.961791275903)*x_ref[0]+(-0.103547475035)*x_ref[1]**o+(-0.745236510079)*x_ref[1]+(-0.157506981712)*x_ref[2]**o+(0.986229506787)*x_ref[2]
        ref[0,3,2,1]=(-0.886091883647)*x_ref[0]**o+(0.6979804248)*x_ref[0]+(-0.366575419428)*x_ref[1]**o+(-0.958601153259)*x_ref[1]+(-0.0119724511494)*x_ref[2]**o+(-0.76101132076)*x_ref[2]
        ref[0,4,0,0]=(0.923290554434)*x_ref[0]**o+(0.411359492198)*x_ref[0]+(-0.977999562875)*x_ref[1]**o+(0.352085161527)*x_ref[1]+(0.955745235349)*x_ref[2]**o+(-0.978885609257)*x_ref[2]
        ref[0,4,0,1]=(0.226973479105)*x_ref[0]**o+(-0.423881897251)*x_ref[0]+(0.889101969481)*x_ref[1]**o+(-0.994236389028)*x_ref[1]+(0.0159169033361)*x_ref[2]**o+(-0.559935154203)*x_ref[2]
        ref[0,4,1,0]=(-0.225427291797)*x_ref[0]**o+(-0.557334621848)*x_ref[0]+(-0.486252586142)*x_ref[1]**o+(0.0497120905216)*x_ref[1]+(0.84165736691)*x_ref[2]**o+(-0.185010156797)*x_ref[2]
        ref[0,4,1,1]=(0.762694602609)*x_ref[0]**o+(-0.447367170727)*x_ref[0]+(-0.61103181818)*x_ref[1]**o+(0.689761057656)*x_ref[1]+(-0.631855120122)*x_ref[2]**o+(0.33274655602)*x_ref[2]
        ref[0,4,2,0]=(-0.198884868914)*x_ref[0]**o+(0.716371959278)*x_ref[0]+(0.260871768919)*x_ref[1]**o+(0.622241740082)*x_ref[1]+(0.774103524035)*x_ref[2]**o+(0.186132703676)*x_ref[2]
        ref[0,4,2,1]=(-0.309450645317)*x_ref[0]**o+(0.204754571336)*x_ref[0]+(0.483402810171)*x_ref[1]**o+(0.404620744188)*x_ref[1]+(-0.699137246959)*x_ref[2]**o+(0.467428706196)*x_ref[2]
        ref[1,0,0,0]=(0.642289506448)*x_ref[0]**o+(0.383990304394)*x_ref[0]+(0.556940973318)*x_ref[1]**o+(0.135219729041)*x_ref[1]+(0.577195836358)*x_ref[2]**o+(-0.338924419674)*x_ref[2]
        ref[1,0,0,1]=(-0.920873911295)*x_ref[0]**o+(-0.330610192304)*x_ref[0]+(-0.117226803856)*x_ref[1]**o+(0.172482867684)*x_ref[1]+(0.909140902709)*x_ref[2]**o+(-0.590494167913)*x_ref[2]
        ref[1,0,1,0]=(0.210161538861)*x_ref[0]**o+(-0.83860241761)*x_ref[0]+(0.0257180382661)*x_ref[1]**o+(0.877633854348)*x_ref[1]+(0.26045311754)*x_ref[2]**o+(-0.971349902714)*x_ref[2]
        ref[1,0,1,1]=(0.70736843125)*x_ref[0]**o+(-0.765121985431)*x_ref[0]+(-0.196525225943)*x_ref[1]**o+(-0.175928729346)*x_ref[1]+(0.945523028748)*x_ref[2]**o+(0.877830322823)*x_ref[2]
        ref[1,0,2,0]=(-0.963513016065)*x_ref[0]**o+(0.0820537856351)*x_ref[0]+(0.0122070196118)*x_ref[1]**o+(-0.154147449592)*x_ref[1]+(0.323844633268)*x_ref[2]**o+(-0.124897698214)*x_ref[2]
        ref[1,0,2,1]=(0.656267566405)*x_ref[0]**o+(0.0174786620768)*x_ref[0]+(0.763902435618)*x_ref[1]**o+(-0.986021506043)*x_ref[1]+(0.632625555994)*x_ref[2]**o+(-0.110657743523)*x_ref[2]
        ref[1,1,0,0]=(0.00354144688732)*x_ref[0]**o+(-0.813515808171)*x_ref[0]+(-0.100717717446)*x_ref[1]**o+(0.0543708163629)*x_ref[1]+(0.785927880762)*x_ref[2]**o+(0.0109166652227)*x_ref[2]
        ref[1,1,0,1]=(-0.239705688631)*x_ref[0]**o+(0.402915642509)*x_ref[0]+(0.463371068163)*x_ref[1]**o+(-0.507302462928)*x_ref[1]+(-0.0789844128295)*x_ref[2]**o+(0.669144891499)*x_ref[2]
        ref[1,1,1,0]=(-0.491891367001)*x_ref[0]**o+(-0.524377636928)*x_ref[0]+(0.903091863917)*x_ref[1]**o+(-0.554368807183)*x_ref[1]+(0.332899195339)*x_ref[2]**o+(-0.645216147049)*x_ref[2]
        ref[1,1,1,1]=(-0.915843941243)*x_ref[0]**o+(-0.233175654748)*x_ref[0]+(0.467730425037)*x_ref[1]**o+(-0.765957936683)*x_ref[1]+(0.726678877276)*x_ref[2]**o+(0.770533947942)*x_ref[2]
        ref[1,1,2,0]=(0.678071852032)*x_ref[0]**o+(0.266537844237)*x_ref[0]+(0.324335300151)*x_ref[1]**o+(-0.286713971373)*x_ref[1]+(-0.702384414014)*x_ref[2]**o+(0.887198276519)*x_ref[2]
        ref[1,1,2,1]=(0.0466444959648)*x_ref[0]**o+(0.689162912677)*x_ref[0]+(-0.35033923676)*x_ref[1]**o+(-0.0536380766876)*x_ref[1]+(-0.511833442517)*x_ref[2]**o+(0.125566299829)*x_ref[2]
        ref[1,2,0,0]=(-0.803170431622)*x_ref[0]**o+(0.476211169752)*x_ref[0]+(-0.170510267953)*x_ref[1]**o+(-0.765945735317)*x_ref[1]+(-0.169966329291)*x_ref[2]**o+(-0.988525918724)*x_ref[2]
        ref[1,2,0,1]=(0.289618751212)*x_ref[0]**o+(0.294379040976)*x_ref[0]+(-0.922478416838)*x_ref[1]**o+(0.210546495537)*x_ref[1]+(-0.87266381496)*x_ref[2]**o+(-0.865571669577)*x_ref[2]
        ref[1,2,1,0]=(-0.292811054005)*x_ref[0]**o+(0.651280833746)*x_ref[0]+(-0.625004203736)*x_ref[1]**o+(0.372078169172)*x_ref[1]+(-0.0539099055169)*x_ref[2]**o+(0.0911869270008)*x_ref[2]
        ref[1,2,1,1]=(0.496217698294)*x_ref[0]**o+(-0.986021787133)*x_ref[0]+(0.246417059532)*x_ref[1]**o+(0.610226802401)*x_ref[1]+(0.991534665298)*x_ref[2]**o+(-0.507441185982)*x_ref[2]
        ref[1,2,2,0]=(-0.839902565947)*x_ref[0]**o+(-0.279607147278)*x_ref[0]+(0.278212307963)*x_ref[1]**o+(0.609793640036)*x_ref[1]+(0.940471913774)*x_ref[2]**o+(0.21860828705)*x_ref[2]
        ref[1,2,2,1]=(0.789578112278)*x_ref[0]**o+(-0.0226697818608)*x_ref[0]+(0.216321340291)*x_ref[1]**o+(-0.701818463454)*x_ref[1]+(-0.166689030646)*x_ref[2]**o+(0.966131542292)*x_ref[2]
        ref[1,3,0,0]=(0.149337042551)*x_ref[0]**o+(-0.785050579674)*x_ref[0]+(0.530219924721)*x_ref[1]**o+(-0.971927239398)*x_ref[1]+(0.950917908603)*x_ref[2]**o+(0.132668077447)*x_ref[2]
        ref[1,3,0,1]=(0.983081559724)*x_ref[0]**o+(-0.358694498455)*x_ref[0]+(0.181080099281)*x_ref[1]**o+(-0.699756291643)*x_ref[1]+(-0.744335953304)*x_ref[2]**o+(0.00262043221382)*x_ref[2]
        ref[1,3,1,0]=(0.31776032045)*x_ref[0]**o+(-0.0726184220029)*x_ref[0]+(-0.759883735971)*x_ref[1]**o+(-0.6389710873)*x_ref[1]+(-0.607858499502)*x_ref[2]**o+(-0.397169895784)*x_ref[2]
        ref[1,3,1,1]=(-0.950468389728)*x_ref[0]**o+(0.831121915521)*x_ref[0]+(0.552857011083)*x_ref[1]**o+(0.848599672594)*x_ref[1]+(0.432183564231)*x_ref[2]**o+(0.57537721362)*x_ref[2]
        ref[1,3,2,0]=(0.736080143601)*x_ref[0]**o+(-0.804140307747)*x_ref[0]+(-0.163864867884)*x_ref[1]**o+(0.950874042315)*x_ref[1]+(-0.99676139412)*x_ref[2]**o+(0.332320294246)*x_ref[2]
        ref[1,3,2,1]=(0.119082582041)*x_ref[0]**o+(-0.969740778852)*x_ref[0]+(0.828432583532)*x_ref[1]**o+(0.190324050395)*x_ref[1]+(-0.42639142674)*x_ref[2]**o+(0.315519578192)*x_ref[2]
        ref[1,4,0,0]=(-0.798465141775)*x_ref[0]**o+(-0.717282977908)*x_ref[0]+(0.935655148713)*x_ref[1]**o+(-0.655752964484)*x_ref[1]+(0.159990149099)*x_ref[2]**o+(-0.537115062687)*x_ref[2]
        ref[1,4,0,1]=(0.445151775007)*x_ref[0]**o+(-0.867956302051)*x_ref[0]+(-0.809856879825)*x_ref[1]**o+(-0.326897750981)*x_ref[1]+(0.756352311987)*x_ref[2]**o+(0.577585513459)*x_ref[2]
        ref[1,4,1,0]=(0.294243347591)*x_ref[0]**o+(0.774741656931)*x_ref[0]+(-0.0799985737984)*x_ref[1]**o+(0.179789107412)*x_ref[1]+(-0.612198025189)*x_ref[2]**o+(-0.07086009611)*x_ref[2]
        ref[1,4,1,1]=(0.0692586287045)*x_ref[0]**o+(-0.671956300122)*x_ref[0]+(-0.948667104683)*x_ref[1]**o+(0.68656246801)*x_ref[1]+(-0.352975297696)*x_ref[2]**o+(0.277952113716)*x_ref[2]
        ref[1,4,2,0]=(-0.491316095517)*x_ref[0]**o+(0.725899087239)*x_ref[0]+(0.634113963431)*x_ref[1]**o+(0.235694393453)*x_ref[1]+(-0.15600720713)*x_ref[2]**o+(0.954814976539)*x_ref[2]
        ref[1,4,2,1]=(0.120206996339)*x_ref[0]**o+(-0.86151411823)*x_ref[0]+(-0.137093650097)*x_ref[1]**o+(0.633402978246)*x_ref[1]+(-0.42515364728)*x_ref[2]**o+(-0.987634210617)*x_ref[2]
        ref[2,0,0,0]=(0.539463971222)*x_ref[0]**o+(-0.03435650579)*x_ref[0]+(-0.884395295544)*x_ref[1]**o+(0.215454879022)*x_ref[1]+(0.842377328246)*x_ref[2]**o+(0.370920136871)*x_ref[2]
        ref[2,0,0,1]=(-0.411505463039)*x_ref[0]**o+(0.446494039439)*x_ref[0]+(0.0313771969556)*x_ref[1]**o+(0.602893223061)*x_ref[1]+(-0.30383490021)*x_ref[2]**o+(-0.788028756758)*x_ref[2]
        ref[2,0,1,0]=(0.489009615999)*x_ref[0]**o+(-0.95508248016)*x_ref[0]+(-0.21489475022)*x_ref[1]**o+(-0.857263478431)*x_ref[1]+(-0.123703409045)*x_ref[2]**o+(0.663389562431)*x_ref[2]
        ref[2,0,1,1]=(-0.0224763310929)*x_ref[0]**o+(-0.315886249237)*x_ref[0]+(-0.318201849695)*x_ref[1]**o+(0.423732682462)*x_ref[1]+(0.396065636381)*x_ref[2]**o+(0.0816717878865)*x_ref[2]
        ref[2,0,2,0]=(-0.191432618224)*x_ref[0]**o+(0.688389390795)*x_ref[0]+(-0.382948112159)*x_ref[1]**o+(0.562905173818)*x_ref[1]+(-0.118396714029)*x_ref[2]**o+(0.234478642577)*x_ref[2]
        ref[2,0,2,1]=(-0.348008173171)*x_ref[0]**o+(0.191994166636)*x_ref[0]+(0.450556057147)*x_ref[1]**o+(-0.710821693343)*x_ref[1]+(-0.245119333857)*x_ref[2]**o+(0.262869525239)*x_ref[2]
        ref[2,1,0,0]=(-0.160444506203)*x_ref[0]**o+(-0.816142837432)*x_ref[0]+(-0.619722480492)*x_ref[1]**o+(-0.989077039675)*x_ref[1]+(-0.0282349103977)*x_ref[2]**o+(-0.629849819691)*x_ref[2]
        ref[2,1,0,1]=(0.379513765297)*x_ref[0]**o+(0.934652105586)*x_ref[0]+(-0.632636427868)*x_ref[1]**o+(-0.0492550496553)*x_ref[1]+(0.137733572307)*x_ref[2]**o+(0.779782637942)*x_ref[2]
        ref[2,1,1,0]=(0.469499320417)*x_ref[0]**o+(0.570880924125)*x_ref[0]+(-0.0329833251419)*x_ref[1]**o+(-0.0388751034604)*x_ref[1]+(-0.878520094389)*x_ref[2]**o+(0.992656649048)*x_ref[2]
        ref[2,1,1,1]=(0.592005181995)*x_ref[0]**o+(-0.553980493526)*x_ref[0]+(0.359837665019)*x_ref[1]**o+(0.811107941179)*x_ref[1]+(-0.441929650912)*x_ref[2]**o+(-0.768618378636)*x_ref[2]
        ref[2,1,2,0]=(0.741376258816)*x_ref[0]**o+(-0.238909756717)*x_ref[0]+(0.204631082449)*x_ref[1]**o+(0.970501662949)*x_ref[1]+(-0.615638092239)*x_ref[2]**o+(-0.575948750844)*x_ref[2]
        ref[2,1,2,1]=(0.774050933112)*x_ref[0]**o+(-0.872234076721)*x_ref[0]+(-0.0936189240061)*x_ref[1]**o+(-0.773608756961)*x_ref[1]+(-0.119171907808)*x_ref[2]**o+(-0.256124796845)*x_ref[2]
        ref[2,2,0,0]=(0.459481754425)*x_ref[0]**o+(0.478364929493)*x_ref[0]+(0.460199733287)*x_ref[1]**o+(0.66880294659)*x_ref[1]+(-0.859525058389)*x_ref[2]**o+(0.77960284414)*x_ref[2]
        ref[2,2,0,1]=(0.690557835207)*x_ref[0]**o+(-0.812305038592)*x_ref[0]+(0.837924124791)*x_ref[1]**o+(-0.785979206364)*x_ref[1]+(-0.295553949819)*x_ref[2]**o+(0.575096295897)*x_ref[2]
        ref[2,2,1,0]=(0.443193683205)*x_ref[0]**o+(0.0355033222253)*x_ref[0]+(0.497641002766)*x_ref[1]**o+(-0.636955142451)*x_ref[1]+(-0.265324357454)*x_ref[2]**o+(0.734148569078)*x_ref[2]
        ref[2,2,1,1]=(-0.088707242599)*x_ref[0]**o+(0.513403518669)*x_ref[0]+(0.337741737651)*x_ref[1]**o+(-0.174843924478)*x_ref[1]+(0.687291255986)*x_ref[2]**o+(0.473850502258)*x_ref[2]
        ref[2,2,2,0]=(0.5305624642)*x_ref[0]**o+(-0.972997835953)*x_ref[0]+(0.00197107287888)*x_ref[1]**o+(-0.75490872634)*x_ref[1]+(-0.0569624569435)*x_ref[2]**o+(-0.0780532868631)*x_ref[2]
        ref[2,2,2,1]=(-0.744584471866)*x_ref[0]**o+(0.915494909272)*x_ref[0]+(0.239235050038)*x_ref[1]**o+(-0.192466753187)*x_ref[1]+(0.933556550824)*x_ref[2]**o+(-0.786420251487)*x_ref[2]
        ref[2,3,0,0]=(0.472114530627)*x_ref[0]**o+(0.0693211807557)*x_ref[0]+(-0.274273425713)*x_ref[1]**o+(0.136242554894)*x_ref[1]+(-0.329240675785)*x_ref[2]**o+(0.0212539247343)*x_ref[2]
        ref[2,3,0,1]=(-0.780426108099)*x_ref[0]**o+(-0.632382551025)*x_ref[0]+(0.554926949239)*x_ref[1]**o+(0.264204009059)*x_ref[1]+(-0.868007525824)*x_ref[2]**o+(-0.937631120201)*x_ref[2]
        ref[2,3,1,0]=(-0.826692080947)*x_ref[0]**o+(0.915241858538)*x_ref[0]+(-0.49986160859)*x_ref[1]**o+(0.921686464658)*x_ref[1]+(0.71996666304)*x_ref[2]**o+(0.297609501096)*x_ref[2]
        ref[2,3,1,1]=(0.991896994614)*x_ref[0]**o+(-0.543310008103)*x_ref[0]+(0.0791899631111)*x_ref[1]**o+(-0.914088200145)*x_ref[1]+(-0.496337624951)*x_ref[2]**o+(0.708950898614)*x_ref[2]
        ref[2,3,2,0]=(-0.235116966432)*x_ref[0]**o+(-0.492848730207)*x_ref[0]+(0.428728646789)*x_ref[1]**o+(-0.410747634691)*x_ref[1]+(0.705059815458)*x_ref[2]**o+(-0.185604633831)*x_ref[2]
        ref[2,3,2,1]=(0.838829037117)*x_ref[0]**o+(0.670561147563)*x_ref[0]+(0.621407965075)*x_ref[1]**o+(-0.10624946305)*x_ref[1]+(0.893703402687)*x_ref[2]**o+(0.985504995821)*x_ref[2]
        ref[2,4,0,0]=(0.635072504682)*x_ref[0]**o+(-0.681962427555)*x_ref[0]+(-0.00774518011081)*x_ref[1]**o+(-0.264996228668)*x_ref[1]+(0.690668403198)*x_ref[2]**o+(0.883573761062)*x_ref[2]
        ref[2,4,0,1]=(0.545632268651)*x_ref[0]**o+(0.439668519394)*x_ref[0]+(-0.482479808491)*x_ref[1]**o+(0.752028962391)*x_ref[1]+(0.0596773623939)*x_ref[2]**o+(-0.369987181127)*x_ref[2]
        ref[2,4,1,0]=(0.659005322796)*x_ref[0]**o+(0.503940833428)*x_ref[0]+(0.643633321586)*x_ref[1]**o+(0.927082323252)*x_ref[1]+(0.354072146184)*x_ref[2]**o+(-0.638737560608)*x_ref[2]
        ref[2,4,1,1]=(-0.443548860913)*x_ref[0]**o+(0.707607048458)*x_ref[0]+(-0.955225746402)*x_ref[1]**o+(0.515995563585)*x_ref[1]+(-0.288406637165)*x_ref[2]**o+(-0.760864937318)*x_ref[2]
        ref[2,4,2,0]=(0.898146580355)*x_ref[0]**o+(-0.590650787106)*x_ref[0]+(0.036095789753)*x_ref[1]**o+(0.326040260581)*x_ref[1]+(0.680069361306)*x_ref[2]**o+(0.333839099341)*x_ref[2]
        ref[2,4,2,1]=(-0.151364260271)*x_ref[0]**o+(-0.675198183832)*x_ref[0]+(-0.183626298978)*x_ref[1]**o+(-0.182565217146)*x_ref[1]+(0.0345632129814)*x_ref[2]**o+(-0.581534100763)*x_ref[2]
        ref[3,0,0,0]=(-0.942351703246)*x_ref[0]**o+(-0.699567649453)*x_ref[0]+(-0.832068665702)*x_ref[1]**o+(0.163439416504)*x_ref[1]+(0.742500339519)*x_ref[2]**o+(-0.492882887489)*x_ref[2]
        ref[3,0,0,1]=(0.990078296484)*x_ref[0]**o+(-0.999837336587)*x_ref[0]+(0.197589456459)*x_ref[1]**o+(0.431403059019)*x_ref[1]+(0.859864015871)*x_ref[2]**o+(-0.267474829282)*x_ref[2]
        ref[3,0,1,0]=(-0.776773660723)*x_ref[0]**o+(-0.189048370486)*x_ref[0]+(0.926953856802)*x_ref[1]**o+(-0.72314744652)*x_ref[1]+(0.330064363809)*x_ref[2]**o+(0.188533334359)*x_ref[2]
        ref[3,0,1,1]=(-0.854830058762)*x_ref[0]**o+(0.832073883276)*x_ref[0]+(-0.425691165124)*x_ref[1]**o+(0.162351500316)*x_ref[1]+(0.370604864512)*x_ref[2]**o+(-0.581907071484)*x_ref[2]
        ref[3,0,2,0]=(-0.316716909548)*x_ref[0]**o+(0.335417284383)*x_ref[0]+(-0.783061160352)*x_ref[1]**o+(-0.253972519239)*x_ref[1]+(-0.826432933721)*x_ref[2]**o+(0.267368378363)*x_ref[2]
        ref[3,0,2,1]=(-0.765899705462)*x_ref[0]**o+(-0.0993875861476)*x_ref[0]+(-0.211785004352)*x_ref[1]**o+(-0.635330151497)*x_ref[1]+(0.414575412599)*x_ref[2]**o+(0.586882274623)*x_ref[2]
        ref[3,1,0,0]=(0.987776377237)*x_ref[0]**o+(0.796660669191)*x_ref[0]+(0.580724891862)*x_ref[1]**o+(-0.24149253995)*x_ref[1]+(-0.633421016909)*x_ref[2]**o+(-0.220110913711)*x_ref[2]
        ref[3,1,0,1]=(-0.182240918081)*x_ref[0]**o+(-0.290526050572)*x_ref[0]+(0.00283230959976)*x_ref[1]**o+(-0.933282166805)*x_ref[1]+(-0.857893235883)*x_ref[2]**o+(0.551414074127)*x_ref[2]
        ref[3,1,1,0]=(-0.854435997816)*x_ref[0]**o+(0.706455144263)*x_ref[0]+(0.406237965254)*x_ref[1]**o+(0.192071142415)*x_ref[1]+(0.357966796834)*x_ref[2]**o+(0.887011087987)*x_ref[2]
        ref[3,1,1,1]=(0.832013396996)*x_ref[0]**o+(0.696133072321)*x_ref[0]+(0.682726968712)*x_ref[1]**o+(0.613316240514)*x_ref[1]+(0.12600339887)*x_ref[2]**o+(-0.620088231382)*x_ref[2]
        ref[3,1,2,0]=(-0.178681615221)*x_ref[0]**o+(-0.952835986168)*x_ref[0]+(0.299692693746)*x_ref[1]**o+(0.892451908416)*x_ref[1]+(0.202403273634)*x_ref[2]**o+(0.677212897943)*x_ref[2]
        ref[3,1,2,1]=(0.0592594053342)*x_ref[0]**o+(0.56391153959)*x_ref[0]+(-0.874866627667)*x_ref[1]**o+(0.432102900769)*x_ref[1]+(-0.49073126285)*x_ref[2]**o+(-0.010495860028)*x_ref[2]
        ref[3,2,0,0]=(0.548006445814)*x_ref[0]**o+(0.667313392202)*x_ref[0]+(0.397805355499)*x_ref[1]**o+(-0.788100267808)*x_ref[1]+(0.869234055258)*x_ref[2]**o+(-0.830241520401)*x_ref[2]
        ref[3,2,0,1]=(0.72586139216)*x_ref[0]**o+(-0.931878957878)*x_ref[0]+(-0.94119555858)*x_ref[1]**o+(-0.133544806169)*x_ref[1]+(0.754263323382)*x_ref[2]**o+(-0.446604740157)*x_ref[2]
        ref[3,2,1,0]=(-0.28135830707)*x_ref[0]**o+(-0.478744058518)*x_ref[0]+(0.88219033731)*x_ref[1]**o+(0.460587286945)*x_ref[1]+(-0.849692146096)*x_ref[2]**o+(0.0927701345107)*x_ref[2]
        ref[3,2,1,1]=(-0.681859641611)*x_ref[0]**o+(-0.670164293088)*x_ref[0]+(0.796366374435)*x_ref[1]**o+(0.837230317054)*x_ref[1]+(-0.413283915904)*x_ref[2]**o+(0.779624227187)*x_ref[2]
        ref[3,2,2,0]=(-0.803242771255)*x_ref[0]**o+(-0.0687616271776)*x_ref[0]+(-0.948856027999)*x_ref[1]**o+(0.762568790931)*x_ref[1]+(0.937588292604)*x_ref[2]**o+(-0.0224232611283)*x_ref[2]
        ref[3,2,2,1]=(-0.215752944937)*x_ref[0]**o+(-0.807114296075)*x_ref[0]+(0.970625375964)*x_ref[1]**o+(0.68578124692)*x_ref[1]+(-0.165007047419)*x_ref[2]**o+(0.942718439981)*x_ref[2]
        ref[3,3,0,0]=(0.152001104716)*x_ref[0]**o+(-0.657242342357)*x_ref[0]+(0.559608661705)*x_ref[1]**o+(0.0764403273251)*x_ref[1]+(0.633414754695)*x_ref[2]**o+(0.430777868344)*x_ref[2]
        ref[3,3,0,1]=(-0.898109497759)*x_ref[0]**o+(0.736574249406)*x_ref[0]+(-0.569584184574)*x_ref[1]**o+(-0.145501395506)*x_ref[1]+(-0.412424116475)*x_ref[2]**o+(-0.416350923096)*x_ref[2]
        ref[3,3,1,0]=(0.602022795614)*x_ref[0]**o+(-0.848525569825)*x_ref[0]+(-0.713426287505)*x_ref[1]**o+(-0.829278452687)*x_ref[1]+(-0.491257659316)*x_ref[2]**o+(0.90219908999)*x_ref[2]
        ref[3,3,1,1]=(-0.769110216066)*x_ref[0]**o+(-0.789464037338)*x_ref[0]+(-0.363686172584)*x_ref[1]**o+(-0.86190787068)*x_ref[1]+(0.275438119232)*x_ref[2]**o+(0.285209834248)*x_ref[2]
        ref[3,3,2,0]=(0.351662405853)*x_ref[0]**o+(0.8403302307)*x_ref[0]+(-0.449869332519)*x_ref[1]**o+(0.936043614838)*x_ref[1]+(-0.656908777335)*x_ref[2]**o+(-0.231923872531)*x_ref[2]
        ref[3,3,2,1]=(0.873848945653)*x_ref[0]**o+(0.551429883959)*x_ref[0]+(-0.577942548701)*x_ref[1]**o+(-0.254317163049)*x_ref[1]+(0.64878089878)*x_ref[2]**o+(0.7135339515)*x_ref[2]
        ref[3,4,0,0]=(0.891435610494)*x_ref[0]**o+(0.598893704498)*x_ref[0]+(0.24682094756)*x_ref[1]**o+(0.0895612710338)*x_ref[1]+(-0.513686244953)*x_ref[2]**o+(0.307205505502)*x_ref[2]
        ref[3,4,0,1]=(-0.577752944196)*x_ref[0]**o+(-0.458383751623)*x_ref[0]+(-0.913956988143)*x_ref[1]**o+(0.470857602235)*x_ref[1]+(0.631726850665)*x_ref[2]**o+(-0.112962289904)*x_ref[2]
        ref[3,4,1,0]=(-0.65648177489)*x_ref[0]**o+(-0.843530972747)*x_ref[0]+(-0.0332153723809)*x_ref[1]**o+(0.230484263355)*x_ref[1]+(-0.865687508592)*x_ref[2]**o+(0.122751735082)*x_ref[2]
        ref[3,4,1,1]=(-0.353502640734)*x_ref[0]**o+(-0.816020900282)*x_ref[0]+(0.943688796868)*x_ref[1]**o+(-0.526707881958)*x_ref[1]+(-0.790258298117)*x_ref[2]**o+(-0.748706809042)*x_ref[2]
        ref[3,4,2,0]=(-0.596345912175)*x_ref[0]**o+(0.872771121837)*x_ref[0]+(0.100107395575)*x_ref[1]**o+(0.950482249217)*x_ref[1]+(0.0487634582783)*x_ref[2]**o+(-0.0285218097887)*x_ref[2]
        ref[3,4,2,1]=(-0.827443553933)*x_ref[0]**o+(-0.922564800149)*x_ref[0]+(-0.810354385287)*x_ref[1]**o+(0.630079217293)*x_ref[1]+(-0.316280559723)*x_ref[2]**o+(0.977830435871)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_ReducedFunctionOnContactOne_rank0(self):
      """
      tests interpolate for rank 0 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactOne onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactOne(self.domain)
      x=w.getX()
      arg=Data(0,(),w)
      ref=Data(0,(),w_ref)
      if dim==2:
        arg=(0.142445811229)*x[0]**o+(-0.170111870439)*x[0]+(-0.571150729608)*x[1]**o+(-0.799596368319)*x[1]
        ref=(0.142445811229)*x_ref[0]**o+(-0.170111870439)*x_ref[0]+(-0.571150729608)*x_ref[1]**o+(-0.799596368319)*x_ref[1]
      else:
        arg=(0.131269935442)*x[0]**o+(0.67764849927)*x[0]+(0.199946665)*x[1]**o+(0.775678673404)*x[1]+(-0.818505707274)*x[2]**o+(-0.141160005914)*x[2]
        ref=(0.131269935442)*x_ref[0]**o+(0.67764849927)*x_ref[0]+(0.199946665)*x_ref[1]**o+(0.775678673404)*x_ref[1]+(-0.818505707274)*x_ref[2]**o+(-0.141160005914)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_ReducedFunctionOnContactOne_rank1(self):
      """
      tests interpolate for rank 1 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactOne onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactOne(self.domain)
      x=w.getX()
      arg=Data(0,(2,),w)
      ref=Data(0,(2,),w_ref)
      if dim==2:
        arg[0]=(-0.271030346059)*x[0]**o+(-0.604054986035)*x[0]+(-0.856492962015)*x[1]**o+(0.229582187423)*x[1]
        arg[1]=(-0.152539485929)*x[0]**o+(0.936634973224)*x[0]+(0.805016132937)*x[1]**o+(-0.6832854412)*x[1]
        ref[0]=(-0.271030346059)*x_ref[0]**o+(-0.604054986035)*x_ref[0]+(-0.856492962015)*x_ref[1]**o+(0.229582187423)*x_ref[1]
        ref[1]=(-0.152539485929)*x_ref[0]**o+(0.936634973224)*x_ref[0]+(0.805016132937)*x_ref[1]**o+(-0.6832854412)*x_ref[1]
      else:
        arg[0]=(-0.484580515247)*x[0]**o+(-0.134539997002)*x[0]+(0.589742989514)*x[1]**o+(-0.950803827747)*x[1]+(-0.852976687869)*x[2]**o+(0.0999526415345)*x[2]
        arg[1]=(-0.863620642149)*x[0]**o+(-0.16255544441)*x[0]+(-0.4963232161)*x[1]**o+(0.180988811217)*x[1]+(-0.787186716126)*x[2]**o+(-0.359825851725)*x[2]
        ref[0]=(-0.484580515247)*x_ref[0]**o+(-0.134539997002)*x_ref[0]+(0.589742989514)*x_ref[1]**o+(-0.950803827747)*x_ref[1]+(-0.852976687869)*x_ref[2]**o+(0.0999526415345)*x_ref[2]
        ref[1]=(-0.863620642149)*x_ref[0]**o+(-0.16255544441)*x_ref[0]+(-0.4963232161)*x_ref[1]**o+(0.180988811217)*x_ref[1]+(-0.787186716126)*x_ref[2]**o+(-0.359825851725)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_ReducedFunctionOnContactOne_rank2(self):
      """
      tests interpolate for rank 2 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactOne onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactOne(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5),w)
      ref=Data(0,(4, 5),w_ref)
      if dim==2:
        arg[0,0]=(-0.88064835903)*x[0]**o+(-0.140271048585)*x[0]+(-0.363467917345)*x[1]**o+(0.382530733181)*x[1]
        arg[0,1]=(0.835096280471)*x[0]**o+(0.132077921375)*x[0]+(-0.814392115734)*x[1]**o+(-0.379227781873)*x[1]
        arg[0,2]=(0.257391174596)*x[0]**o+(-0.135017751364)*x[0]+(-0.462760793484)*x[1]**o+(-0.332510860616)*x[1]
        arg[0,3]=(0.303541666957)*x[0]**o+(0.729054207976)*x[0]+(-0.0474210800089)*x[1]**o+(0.213859363809)*x[1]
        arg[0,4]=(0.564466740909)*x[0]**o+(0.684761789227)*x[0]+(0.432934636771)*x[1]**o+(0.261269614075)*x[1]
        arg[1,0]=(0.83080596334)*x[0]**o+(0.968795153198)*x[0]+(0.835115581928)*x[1]**o+(-0.134688885728)*x[1]
        arg[1,1]=(-0.876235093042)*x[0]**o+(-0.43923440557)*x[0]+(-0.641186633503)*x[1]**o+(-0.987129244994)*x[1]
        arg[1,2]=(-0.994007922096)*x[0]**o+(0.534914510762)*x[0]+(0.754420969331)*x[1]**o+(0.241533891038)*x[1]
        arg[1,3]=(-0.196027247194)*x[0]**o+(0.962231147494)*x[0]+(-0.819146467532)*x[1]**o+(0.966024528731)*x[1]
        arg[1,4]=(-0.577014723723)*x[0]**o+(0.350310620855)*x[0]+(0.16266475329)*x[1]**o+(-0.926739533525)*x[1]
        arg[2,0]=(-0.262964780787)*x[0]**o+(-0.824445469277)*x[0]+(-0.982497605841)*x[1]**o+(0.900080022932)*x[1]
        arg[2,1]=(0.54409272334)*x[0]**o+(0.789421020125)*x[0]+(0.672239490644)*x[1]**o+(-0.0306885937363)*x[1]
        arg[2,2]=(-0.130272075526)*x[0]**o+(0.51213564573)*x[0]+(-0.819817699321)*x[1]**o+(-0.259068167835)*x[1]
        arg[2,3]=(-0.61526267092)*x[0]**o+(-0.504362726269)*x[0]+(0.362429515671)*x[1]**o+(-0.0921852465615)*x[1]
        arg[2,4]=(-0.344772989987)*x[0]**o+(0.260747743828)*x[0]+(0.474457474131)*x[1]**o+(0.436008642452)*x[1]
        arg[3,0]=(-0.72740499107)*x[0]**o+(-0.297524430077)*x[0]+(-0.809254720524)*x[1]**o+(-0.230695822532)*x[1]
        arg[3,1]=(-0.0817042383885)*x[0]**o+(-0.324890823023)*x[0]+(0.923635342281)*x[1]**o+(-0.336825317481)*x[1]
        arg[3,2]=(-0.872954045409)*x[0]**o+(0.242165239629)*x[0]+(0.919264824591)*x[1]**o+(0.424247774945)*x[1]
        arg[3,3]=(0.146344350418)*x[0]**o+(-0.705285807962)*x[0]+(-0.0735095471635)*x[1]**o+(-0.464180824267)*x[1]
        arg[3,4]=(0.624085113245)*x[0]**o+(0.719452515121)*x[0]+(0.28331108038)*x[1]**o+(-0.897435702442)*x[1]
        ref[0,0]=(-0.88064835903)*x_ref[0]**o+(-0.140271048585)*x_ref[0]+(-0.363467917345)*x_ref[1]**o+(0.382530733181)*x_ref[1]
        ref[0,1]=(0.835096280471)*x_ref[0]**o+(0.132077921375)*x_ref[0]+(-0.814392115734)*x_ref[1]**o+(-0.379227781873)*x_ref[1]
        ref[0,2]=(0.257391174596)*x_ref[0]**o+(-0.135017751364)*x_ref[0]+(-0.462760793484)*x_ref[1]**o+(-0.332510860616)*x_ref[1]
        ref[0,3]=(0.303541666957)*x_ref[0]**o+(0.729054207976)*x_ref[0]+(-0.0474210800089)*x_ref[1]**o+(0.213859363809)*x_ref[1]
        ref[0,4]=(0.564466740909)*x_ref[0]**o+(0.684761789227)*x_ref[0]+(0.432934636771)*x_ref[1]**o+(0.261269614075)*x_ref[1]
        ref[1,0]=(0.83080596334)*x_ref[0]**o+(0.968795153198)*x_ref[0]+(0.835115581928)*x_ref[1]**o+(-0.134688885728)*x_ref[1]
        ref[1,1]=(-0.876235093042)*x_ref[0]**o+(-0.43923440557)*x_ref[0]+(-0.641186633503)*x_ref[1]**o+(-0.987129244994)*x_ref[1]
        ref[1,2]=(-0.994007922096)*x_ref[0]**o+(0.534914510762)*x_ref[0]+(0.754420969331)*x_ref[1]**o+(0.241533891038)*x_ref[1]
        ref[1,3]=(-0.196027247194)*x_ref[0]**o+(0.962231147494)*x_ref[0]+(-0.819146467532)*x_ref[1]**o+(0.966024528731)*x_ref[1]
        ref[1,4]=(-0.577014723723)*x_ref[0]**o+(0.350310620855)*x_ref[0]+(0.16266475329)*x_ref[1]**o+(-0.926739533525)*x_ref[1]
        ref[2,0]=(-0.262964780787)*x_ref[0]**o+(-0.824445469277)*x_ref[0]+(-0.982497605841)*x_ref[1]**o+(0.900080022932)*x_ref[1]
        ref[2,1]=(0.54409272334)*x_ref[0]**o+(0.789421020125)*x_ref[0]+(0.672239490644)*x_ref[1]**o+(-0.0306885937363)*x_ref[1]
        ref[2,2]=(-0.130272075526)*x_ref[0]**o+(0.51213564573)*x_ref[0]+(-0.819817699321)*x_ref[1]**o+(-0.259068167835)*x_ref[1]
        ref[2,3]=(-0.61526267092)*x_ref[0]**o+(-0.504362726269)*x_ref[0]+(0.362429515671)*x_ref[1]**o+(-0.0921852465615)*x_ref[1]
        ref[2,4]=(-0.344772989987)*x_ref[0]**o+(0.260747743828)*x_ref[0]+(0.474457474131)*x_ref[1]**o+(0.436008642452)*x_ref[1]
        ref[3,0]=(-0.72740499107)*x_ref[0]**o+(-0.297524430077)*x_ref[0]+(-0.809254720524)*x_ref[1]**o+(-0.230695822532)*x_ref[1]
        ref[3,1]=(-0.0817042383885)*x_ref[0]**o+(-0.324890823023)*x_ref[0]+(0.923635342281)*x_ref[1]**o+(-0.336825317481)*x_ref[1]
        ref[3,2]=(-0.872954045409)*x_ref[0]**o+(0.242165239629)*x_ref[0]+(0.919264824591)*x_ref[1]**o+(0.424247774945)*x_ref[1]
        ref[3,3]=(0.146344350418)*x_ref[0]**o+(-0.705285807962)*x_ref[0]+(-0.0735095471635)*x_ref[1]**o+(-0.464180824267)*x_ref[1]
        ref[3,4]=(0.624085113245)*x_ref[0]**o+(0.719452515121)*x_ref[0]+(0.28331108038)*x_ref[1]**o+(-0.897435702442)*x_ref[1]
      else:
        arg[0,0]=(0.47332206873)*x[0]**o+(0.20564453167)*x[0]+(-0.175485469473)*x[1]**o+(-0.25166656853)*x[1]+(-0.242693318919)*x[2]**o+(-0.773417228526)*x[2]
        arg[0,1]=(0.788563488587)*x[0]**o+(0.598162739778)*x[0]+(-0.736463777722)*x[1]**o+(-0.161555683484)*x[1]+(0.480782882175)*x[2]**o+(-0.500552084345)*x[2]
        arg[0,2]=(-0.207893050318)*x[0]**o+(0.206221241577)*x[0]+(0.306329036151)*x[1]**o+(0.585237890094)*x[1]+(0.0276469766188)*x[2]**o+(0.0502503670529)*x[2]
        arg[0,3]=(0.914950534537)*x[0]**o+(0.401315333176)*x[0]+(0.245224334898)*x[1]**o+(0.526049350134)*x[1]+(-0.735617023846)*x[2]**o+(-0.674692140531)*x[2]
        arg[0,4]=(-0.359566330106)*x[0]**o+(-0.279627730979)*x[0]+(-0.08398829656)*x[1]**o+(-0.789678621983)*x[1]+(-0.357374658443)*x[2]**o+(0.795724504883)*x[2]
        arg[1,0]=(-0.0715278193161)*x[0]**o+(-0.562120168502)*x[0]+(0.428614380101)*x[1]**o+(0.115408373591)*x[1]+(-0.154674397872)*x[2]**o+(0.233468972682)*x[2]
        arg[1,1]=(0.707445640039)*x[0]**o+(-0.719649334705)*x[0]+(-0.349743923514)*x[1]**o+(0.369262203159)*x[1]+(0.712786695739)*x[2]**o+(-0.962450154391)*x[2]
        arg[1,2]=(-0.306983094408)*x[0]**o+(-0.933322822964)*x[0]+(-0.525515899443)*x[1]**o+(0.642320930433)*x[1]+(-0.950421802297)*x[2]**o+(-0.392142385683)*x[2]
        arg[1,3]=(-0.765158324235)*x[0]**o+(-0.362010493356)*x[0]+(-0.918673930909)*x[1]**o+(0.72772784184)*x[1]+(-0.392848438408)*x[2]**o+(0.566119231816)*x[2]
        arg[1,4]=(0.429984662315)*x[0]**o+(-0.165476415697)*x[0]+(0.778539911928)*x[1]**o+(0.411518911472)*x[1]+(0.418323432603)*x[2]**o+(-0.846495913126)*x[2]
        arg[2,0]=(-0.836556750858)*x[0]**o+(0.976027316021)*x[0]+(-0.556499096672)*x[1]**o+(0.320038395915)*x[1]+(0.377663884516)*x[2]**o+(0.491202727923)*x[2]
        arg[2,1]=(-0.329597826951)*x[0]**o+(-0.555819984044)*x[0]+(0.110029358927)*x[1]**o+(-0.737070187836)*x[1]+(-0.587142473737)*x[2]**o+(-0.0388834177112)*x[2]
        arg[2,2]=(0.442251030646)*x[0]**o+(0.285010574287)*x[0]+(0.603267537331)*x[1]**o+(-0.0776884302384)*x[1]+(-0.123541484708)*x[2]**o+(0.44717125529)*x[2]
        arg[2,3]=(0.474458930313)*x[0]**o+(0.415947401939)*x[0]+(0.381539850696)*x[1]**o+(-0.307590909422)*x[1]+(0.488348451586)*x[2]**o+(0.336883810306)*x[2]
        arg[2,4]=(0.804466542611)*x[0]**o+(0.891337158669)*x[0]+(-0.599119487307)*x[1]**o+(-0.92047732727)*x[1]+(-0.251489480657)*x[2]**o+(-0.266483991526)*x[2]
        arg[3,0]=(-0.733633520827)*x[0]**o+(0.494805537624)*x[0]+(-0.117702558274)*x[1]**o+(-0.000461693303442)*x[1]+(-0.102033981202)*x[2]**o+(-0.772865985272)*x[2]
        arg[3,1]=(-0.636901276465)*x[0]**o+(-0.436330072041)*x[0]+(0.403048850222)*x[1]**o+(-0.94737100009)*x[1]+(0.781059874101)*x[2]**o+(-0.952781516037)*x[2]
        arg[3,2]=(-0.489372336164)*x[0]**o+(0.221211043152)*x[0]+(0.793975373906)*x[1]**o+(0.424349965288)*x[1]+(0.453332885662)*x[2]**o+(-0.513541753005)*x[2]
        arg[3,3]=(-0.700899677836)*x[0]**o+(-0.984271896306)*x[0]+(-0.663909486898)*x[1]**o+(0.461302174316)*x[1]+(-0.906457707724)*x[2]**o+(-0.0440003164038)*x[2]
        arg[3,4]=(0.838326295366)*x[0]**o+(-0.483407593962)*x[0]+(-0.122458752397)*x[1]**o+(0.607337951095)*x[1]+(0.750481384434)*x[2]**o+(-0.677155633204)*x[2]
        ref[0,0]=(0.47332206873)*x_ref[0]**o+(0.20564453167)*x_ref[0]+(-0.175485469473)*x_ref[1]**o+(-0.25166656853)*x_ref[1]+(-0.242693318919)*x_ref[2]**o+(-0.773417228526)*x_ref[2]
        ref[0,1]=(0.788563488587)*x_ref[0]**o+(0.598162739778)*x_ref[0]+(-0.736463777722)*x_ref[1]**o+(-0.161555683484)*x_ref[1]+(0.480782882175)*x_ref[2]**o+(-0.500552084345)*x_ref[2]
        ref[0,2]=(-0.207893050318)*x_ref[0]**o+(0.206221241577)*x_ref[0]+(0.306329036151)*x_ref[1]**o+(0.585237890094)*x_ref[1]+(0.0276469766188)*x_ref[2]**o+(0.0502503670529)*x_ref[2]
        ref[0,3]=(0.914950534537)*x_ref[0]**o+(0.401315333176)*x_ref[0]+(0.245224334898)*x_ref[1]**o+(0.526049350134)*x_ref[1]+(-0.735617023846)*x_ref[2]**o+(-0.674692140531)*x_ref[2]
        ref[0,4]=(-0.359566330106)*x_ref[0]**o+(-0.279627730979)*x_ref[0]+(-0.08398829656)*x_ref[1]**o+(-0.789678621983)*x_ref[1]+(-0.357374658443)*x_ref[2]**o+(0.795724504883)*x_ref[2]
        ref[1,0]=(-0.0715278193161)*x_ref[0]**o+(-0.562120168502)*x_ref[0]+(0.428614380101)*x_ref[1]**o+(0.115408373591)*x_ref[1]+(-0.154674397872)*x_ref[2]**o+(0.233468972682)*x_ref[2]
        ref[1,1]=(0.707445640039)*x_ref[0]**o+(-0.719649334705)*x_ref[0]+(-0.349743923514)*x_ref[1]**o+(0.369262203159)*x_ref[1]+(0.712786695739)*x_ref[2]**o+(-0.962450154391)*x_ref[2]
        ref[1,2]=(-0.306983094408)*x_ref[0]**o+(-0.933322822964)*x_ref[0]+(-0.525515899443)*x_ref[1]**o+(0.642320930433)*x_ref[1]+(-0.950421802297)*x_ref[2]**o+(-0.392142385683)*x_ref[2]
        ref[1,3]=(-0.765158324235)*x_ref[0]**o+(-0.362010493356)*x_ref[0]+(-0.918673930909)*x_ref[1]**o+(0.72772784184)*x_ref[1]+(-0.392848438408)*x_ref[2]**o+(0.566119231816)*x_ref[2]
        ref[1,4]=(0.429984662315)*x_ref[0]**o+(-0.165476415697)*x_ref[0]+(0.778539911928)*x_ref[1]**o+(0.411518911472)*x_ref[1]+(0.418323432603)*x_ref[2]**o+(-0.846495913126)*x_ref[2]
        ref[2,0]=(-0.836556750858)*x_ref[0]**o+(0.976027316021)*x_ref[0]+(-0.556499096672)*x_ref[1]**o+(0.320038395915)*x_ref[1]+(0.377663884516)*x_ref[2]**o+(0.491202727923)*x_ref[2]
        ref[2,1]=(-0.329597826951)*x_ref[0]**o+(-0.555819984044)*x_ref[0]+(0.110029358927)*x_ref[1]**o+(-0.737070187836)*x_ref[1]+(-0.587142473737)*x_ref[2]**o+(-0.0388834177112)*x_ref[2]
        ref[2,2]=(0.442251030646)*x_ref[0]**o+(0.285010574287)*x_ref[0]+(0.603267537331)*x_ref[1]**o+(-0.0776884302384)*x_ref[1]+(-0.123541484708)*x_ref[2]**o+(0.44717125529)*x_ref[2]
        ref[2,3]=(0.474458930313)*x_ref[0]**o+(0.415947401939)*x_ref[0]+(0.381539850696)*x_ref[1]**o+(-0.307590909422)*x_ref[1]+(0.488348451586)*x_ref[2]**o+(0.336883810306)*x_ref[2]
        ref[2,4]=(0.804466542611)*x_ref[0]**o+(0.891337158669)*x_ref[0]+(-0.599119487307)*x_ref[1]**o+(-0.92047732727)*x_ref[1]+(-0.251489480657)*x_ref[2]**o+(-0.266483991526)*x_ref[2]
        ref[3,0]=(-0.733633520827)*x_ref[0]**o+(0.494805537624)*x_ref[0]+(-0.117702558274)*x_ref[1]**o+(-0.000461693303442)*x_ref[1]+(-0.102033981202)*x_ref[2]**o+(-0.772865985272)*x_ref[2]
        ref[3,1]=(-0.636901276465)*x_ref[0]**o+(-0.436330072041)*x_ref[0]+(0.403048850222)*x_ref[1]**o+(-0.94737100009)*x_ref[1]+(0.781059874101)*x_ref[2]**o+(-0.952781516037)*x_ref[2]
        ref[3,2]=(-0.489372336164)*x_ref[0]**o+(0.221211043152)*x_ref[0]+(0.793975373906)*x_ref[1]**o+(0.424349965288)*x_ref[1]+(0.453332885662)*x_ref[2]**o+(-0.513541753005)*x_ref[2]
        ref[3,3]=(-0.700899677836)*x_ref[0]**o+(-0.984271896306)*x_ref[0]+(-0.663909486898)*x_ref[1]**o+(0.461302174316)*x_ref[1]+(-0.906457707724)*x_ref[2]**o+(-0.0440003164038)*x_ref[2]
        ref[3,4]=(0.838326295366)*x_ref[0]**o+(-0.483407593962)*x_ref[0]+(-0.122458752397)*x_ref[1]**o+(0.607337951095)*x_ref[1]+(0.750481384434)*x_ref[2]**o+(-0.677155633204)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_ReducedFunctionOnContactOne_rank3(self):
      """
      tests interpolate for rank 3 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactOne onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactOne(self.domain)
      x=w.getX()
      arg=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2),w_ref)
      if dim==2:
        arg[0,0,0]=(0.111473275114)*x[0]**o+(-0.552264448532)*x[0]+(0.67142534913)*x[1]**o+(-0.642305751076)*x[1]
        arg[0,0,1]=(-0.431075800974)*x[0]**o+(0.911382078454)*x[0]+(0.546541644612)*x[1]**o+(-0.67679652536)*x[1]
        arg[0,1,0]=(0.103056292292)*x[0]**o+(-0.640858804416)*x[0]+(0.159596964992)*x[1]**o+(-0.237782007316)*x[1]
        arg[0,1,1]=(0.975898170389)*x[0]**o+(0.255916326383)*x[0]+(-0.692770462322)*x[1]**o+(0.786355114846)*x[1]
        arg[1,0,0]=(-0.669378370759)*x[0]**o+(-0.0153788709146)*x[0]+(0.924383188557)*x[1]**o+(0.885037556829)*x[1]
        arg[1,0,1]=(-0.509891819881)*x[0]**o+(-0.377779514697)*x[0]+(-0.343176473036)*x[1]**o+(-0.39978416725)*x[1]
        arg[1,1,0]=(0.457959467665)*x[0]**o+(0.786511967341)*x[0]+(0.256561665146)*x[1]**o+(-0.261300099643)*x[1]
        arg[1,1,1]=(0.219754515189)*x[0]**o+(-0.131747762867)*x[0]+(0.736017818893)*x[1]**o+(-0.450963700533)*x[1]
        arg[2,0,0]=(-0.182946832789)*x[0]**o+(-0.103064731935)*x[0]+(-0.223584334222)*x[1]**o+(-0.578088201628)*x[1]
        arg[2,0,1]=(-0.830744157425)*x[0]**o+(0.590627167207)*x[0]+(-0.538048378049)*x[1]**o+(-0.0529793352741)*x[1]
        arg[2,1,0]=(-0.307908670388)*x[0]**o+(-0.929987734045)*x[0]+(-0.167190563092)*x[1]**o+(0.126466628227)*x[1]
        arg[2,1,1]=(-0.812045378916)*x[0]**o+(0.676973938072)*x[0]+(0.528243909204)*x[1]**o+(-0.594241387691)*x[1]
        arg[3,0,0]=(-0.0198606164114)*x[0]**o+(-0.00574708685554)*x[0]+(0.908946312745)*x[1]**o+(0.775054099788)*x[1]
        arg[3,0,1]=(-0.810078120518)*x[0]**o+(-0.960585888485)*x[0]+(-0.0747272427198)*x[1]**o+(0.164107641817)*x[1]
        arg[3,1,0]=(-0.535791917139)*x[0]**o+(0.0424226440074)*x[0]+(0.701479263902)*x[1]**o+(0.787745423088)*x[1]
        arg[3,1,1]=(-0.896532152793)*x[0]**o+(-0.0224566920251)*x[0]+(-0.554655626689)*x[1]**o+(0.41012867952)*x[1]
        arg[4,0,0]=(0.972433817496)*x[0]**o+(-0.440662536033)*x[0]+(-0.910456530943)*x[1]**o+(-0.333429535028)*x[1]
        arg[4,0,1]=(-0.415824313942)*x[0]**o+(0.00255557712922)*x[0]+(0.304897639195)*x[1]**o+(-0.866600848052)*x[1]
        arg[4,1,0]=(-0.866277739076)*x[0]**o+(0.282976169776)*x[0]+(0.477528278501)*x[1]**o+(-0.233506497695)*x[1]
        arg[4,1,1]=(-0.12729626381)*x[0]**o+(-0.723204796785)*x[0]+(0.0856495713442)*x[1]**o+(-0.214819437731)*x[1]
        arg[5,0,0]=(0.208133503644)*x[0]**o+(-0.955529202895)*x[0]+(0.0291569127831)*x[1]**o+(0.489913918305)*x[1]
        arg[5,0,1]=(0.96923667477)*x[0]**o+(0.543776525315)*x[0]+(0.38666450029)*x[1]**o+(-0.193493550272)*x[1]
        arg[5,1,0]=(-0.0912940016888)*x[0]**o+(-0.477561072681)*x[0]+(0.474816021613)*x[1]**o+(-0.252664229253)*x[1]
        arg[5,1,1]=(0.748337265159)*x[0]**o+(-0.706194454618)*x[0]+(-0.504466281507)*x[1]**o+(0.582500817679)*x[1]
        ref[0,0,0]=(0.111473275114)*x_ref[0]**o+(-0.552264448532)*x_ref[0]+(0.67142534913)*x_ref[1]**o+(-0.642305751076)*x_ref[1]
        ref[0,0,1]=(-0.431075800974)*x_ref[0]**o+(0.911382078454)*x_ref[0]+(0.546541644612)*x_ref[1]**o+(-0.67679652536)*x_ref[1]
        ref[0,1,0]=(0.103056292292)*x_ref[0]**o+(-0.640858804416)*x_ref[0]+(0.159596964992)*x_ref[1]**o+(-0.237782007316)*x_ref[1]
        ref[0,1,1]=(0.975898170389)*x_ref[0]**o+(0.255916326383)*x_ref[0]+(-0.692770462322)*x_ref[1]**o+(0.786355114846)*x_ref[1]
        ref[1,0,0]=(-0.669378370759)*x_ref[0]**o+(-0.0153788709146)*x_ref[0]+(0.924383188557)*x_ref[1]**o+(0.885037556829)*x_ref[1]
        ref[1,0,1]=(-0.509891819881)*x_ref[0]**o+(-0.377779514697)*x_ref[0]+(-0.343176473036)*x_ref[1]**o+(-0.39978416725)*x_ref[1]
        ref[1,1,0]=(0.457959467665)*x_ref[0]**o+(0.786511967341)*x_ref[0]+(0.256561665146)*x_ref[1]**o+(-0.261300099643)*x_ref[1]
        ref[1,1,1]=(0.219754515189)*x_ref[0]**o+(-0.131747762867)*x_ref[0]+(0.736017818893)*x_ref[1]**o+(-0.450963700533)*x_ref[1]
        ref[2,0,0]=(-0.182946832789)*x_ref[0]**o+(-0.103064731935)*x_ref[0]+(-0.223584334222)*x_ref[1]**o+(-0.578088201628)*x_ref[1]
        ref[2,0,1]=(-0.830744157425)*x_ref[0]**o+(0.590627167207)*x_ref[0]+(-0.538048378049)*x_ref[1]**o+(-0.0529793352741)*x_ref[1]
        ref[2,1,0]=(-0.307908670388)*x_ref[0]**o+(-0.929987734045)*x_ref[0]+(-0.167190563092)*x_ref[1]**o+(0.126466628227)*x_ref[1]
        ref[2,1,1]=(-0.812045378916)*x_ref[0]**o+(0.676973938072)*x_ref[0]+(0.528243909204)*x_ref[1]**o+(-0.594241387691)*x_ref[1]
        ref[3,0,0]=(-0.0198606164114)*x_ref[0]**o+(-0.00574708685554)*x_ref[0]+(0.908946312745)*x_ref[1]**o+(0.775054099788)*x_ref[1]
        ref[3,0,1]=(-0.810078120518)*x_ref[0]**o+(-0.960585888485)*x_ref[0]+(-0.0747272427198)*x_ref[1]**o+(0.164107641817)*x_ref[1]
        ref[3,1,0]=(-0.535791917139)*x_ref[0]**o+(0.0424226440074)*x_ref[0]+(0.701479263902)*x_ref[1]**o+(0.787745423088)*x_ref[1]
        ref[3,1,1]=(-0.896532152793)*x_ref[0]**o+(-0.0224566920251)*x_ref[0]+(-0.554655626689)*x_ref[1]**o+(0.41012867952)*x_ref[1]
        ref[4,0,0]=(0.972433817496)*x_ref[0]**o+(-0.440662536033)*x_ref[0]+(-0.910456530943)*x_ref[1]**o+(-0.333429535028)*x_ref[1]
        ref[4,0,1]=(-0.415824313942)*x_ref[0]**o+(0.00255557712922)*x_ref[0]+(0.304897639195)*x_ref[1]**o+(-0.866600848052)*x_ref[1]
        ref[4,1,0]=(-0.866277739076)*x_ref[0]**o+(0.282976169776)*x_ref[0]+(0.477528278501)*x_ref[1]**o+(-0.233506497695)*x_ref[1]
        ref[4,1,1]=(-0.12729626381)*x_ref[0]**o+(-0.723204796785)*x_ref[0]+(0.0856495713442)*x_ref[1]**o+(-0.214819437731)*x_ref[1]
        ref[5,0,0]=(0.208133503644)*x_ref[0]**o+(-0.955529202895)*x_ref[0]+(0.0291569127831)*x_ref[1]**o+(0.489913918305)*x_ref[1]
        ref[5,0,1]=(0.96923667477)*x_ref[0]**o+(0.543776525315)*x_ref[0]+(0.38666450029)*x_ref[1]**o+(-0.193493550272)*x_ref[1]
        ref[5,1,0]=(-0.0912940016888)*x_ref[0]**o+(-0.477561072681)*x_ref[0]+(0.474816021613)*x_ref[1]**o+(-0.252664229253)*x_ref[1]
        ref[5,1,1]=(0.748337265159)*x_ref[0]**o+(-0.706194454618)*x_ref[0]+(-0.504466281507)*x_ref[1]**o+(0.582500817679)*x_ref[1]
      else:
        arg[0,0,0]=(-0.179972720095)*x[0]**o+(-0.546715268523)*x[0]+(0.835867431319)*x[1]**o+(0.626798499259)*x[1]+(0.397356013278)*x[2]**o+(0.0771586784158)*x[2]
        arg[0,0,1]=(0.625676031039)*x[0]**o+(0.65843849475)*x[0]+(0.456469199307)*x[1]**o+(-0.970709759479)*x[1]+(-0.699606143992)*x[2]**o+(0.76429400724)*x[2]
        arg[0,1,0]=(0.613916424165)*x[0]**o+(0.556778588303)*x[0]+(-0.509658743529)*x[1]**o+(-0.978718968218)*x[1]+(-0.603928369428)*x[2]**o+(-0.550813953692)*x[2]
        arg[0,1,1]=(-0.935214755436)*x[0]**o+(0.44745901813)*x[0]+(-0.582828846453)*x[1]**o+(-0.181547183233)*x[1]+(-0.183912053485)*x[2]**o+(0.446152494296)*x[2]
        arg[1,0,0]=(-0.486345219779)*x[0]**o+(0.753529436833)*x[0]+(0.138157856552)*x[1]**o+(-0.874849020539)*x[1]+(0.374396543792)*x[2]**o+(0.26548456046)*x[2]
        arg[1,0,1]=(0.492908753036)*x[0]**o+(-0.418118685105)*x[0]+(0.880817569823)*x[1]**o+(-0.827691145524)*x[1]+(0.238633156133)*x[2]**o+(-0.463687731341)*x[2]
        arg[1,1,0]=(-0.397373578442)*x[0]**o+(-0.316321105607)*x[0]+(0.647511225459)*x[1]**o+(-0.82553752195)*x[1]+(-0.715549303171)*x[2]**o+(0.804623957459)*x[2]
        arg[1,1,1]=(0.414743130234)*x[0]**o+(0.0201794413683)*x[0]+(-0.636337599584)*x[1]**o+(-0.230053616026)*x[1]+(-0.4302153311)*x[2]**o+(-0.387468029626)*x[2]
        arg[2,0,0]=(0.715100517336)*x[0]**o+(-0.4286120178)*x[0]+(0.0987858542117)*x[1]**o+(0.815093470301)*x[1]+(0.293455353257)*x[2]**o+(0.200576090379)*x[2]
        arg[2,0,1]=(-0.158330015822)*x[0]**o+(0.363339023457)*x[0]+(-0.796055594971)*x[1]**o+(-0.213299205306)*x[1]+(-0.136620988812)*x[2]**o+(0.10733886947)*x[2]
        arg[2,1,0]=(-0.258729405118)*x[0]**o+(0.0669553480038)*x[0]+(0.997663166613)*x[1]**o+(0.00398861063931)*x[1]+(-0.989459856803)*x[2]**o+(0.932687384325)*x[2]
        arg[2,1,1]=(-0.974431322319)*x[0]**o+(-0.486447431804)*x[0]+(0.871784873347)*x[1]**o+(-0.978904038765)*x[1]+(0.647341201506)*x[2]**o+(-0.269683556528)*x[2]
        arg[3,0,0]=(-0.369507398073)*x[0]**o+(-0.772085607074)*x[0]+(0.417550881937)*x[1]**o+(0.271743719652)*x[1]+(0.881807154737)*x[2]**o+(0.54725428039)*x[2]
        arg[3,0,1]=(0.440089335941)*x[0]**o+(0.0844171421294)*x[0]+(0.926389704362)*x[1]**o+(-0.0709182998297)*x[1]+(0.903978660317)*x[2]**o+(-0.160556206716)*x[2]
        arg[3,1,0]=(-0.49936391282)*x[0]**o+(-0.632753931881)*x[0]+(-0.604686084554)*x[1]**o+(-0.954205199531)*x[1]+(-0.535102184845)*x[2]**o+(0.621683814123)*x[2]
        arg[3,1,1]=(-0.117790973132)*x[0]**o+(-0.649091727896)*x[0]+(0.925682916785)*x[1]**o+(0.852053515469)*x[1]+(-0.479218429897)*x[2]**o+(0.426823478575)*x[2]
        arg[4,0,0]=(0.454174564319)*x[0]**o+(-0.855975647406)*x[0]+(0.775972969288)*x[1]**o+(0.267900999893)*x[1]+(0.221387501995)*x[2]**o+(-0.26413791201)*x[2]
        arg[4,0,1]=(0.875191302431)*x[0]**o+(-0.362782502152)*x[0]+(0.24600070339)*x[1]**o+(-0.566709533298)*x[1]+(-0.82187455531)*x[2]**o+(-0.943929640301)*x[2]
        arg[4,1,0]=(-0.409313259973)*x[0]**o+(-0.784833509001)*x[0]+(-0.939690505056)*x[1]**o+(-0.0736280226974)*x[1]+(0.939072745486)*x[2]**o+(0.611697159959)*x[2]
        arg[4,1,1]=(0.893374246808)*x[0]**o+(0.36113513561)*x[0]+(0.227899271271)*x[1]**o+(-0.847221657012)*x[1]+(-0.0106368947417)*x[2]**o+(0.0358896817595)*x[2]
        arg[5,0,0]=(0.0329230524924)*x[0]**o+(0.0565192084014)*x[0]+(-0.68398448658)*x[1]**o+(0.909035618076)*x[1]+(-0.900060440872)*x[2]**o+(-0.481014787833)*x[2]
        arg[5,0,1]=(0.404966692391)*x[0]**o+(0.647468764373)*x[0]+(-0.723847499346)*x[1]**o+(-0.0740896957606)*x[1]+(-0.00628515079923)*x[2]**o+(0.906667863539)*x[2]
        arg[5,1,0]=(0.983181781531)*x[0]**o+(-0.297325990236)*x[0]+(0.00489064543352)*x[1]**o+(0.667560767038)*x[1]+(-0.665768141301)*x[2]**o+(0.772761393608)*x[2]
        arg[5,1,1]=(-0.496581691354)*x[0]**o+(0.0121337979505)*x[0]+(0.492281382726)*x[1]**o+(0.545232714025)*x[1]+(0.418321481532)*x[2]**o+(0.489635969764)*x[2]
        ref[0,0,0]=(-0.179972720095)*x_ref[0]**o+(-0.546715268523)*x_ref[0]+(0.835867431319)*x_ref[1]**o+(0.626798499259)*x_ref[1]+(0.397356013278)*x_ref[2]**o+(0.0771586784158)*x_ref[2]
        ref[0,0,1]=(0.625676031039)*x_ref[0]**o+(0.65843849475)*x_ref[0]+(0.456469199307)*x_ref[1]**o+(-0.970709759479)*x_ref[1]+(-0.699606143992)*x_ref[2]**o+(0.76429400724)*x_ref[2]
        ref[0,1,0]=(0.613916424165)*x_ref[0]**o+(0.556778588303)*x_ref[0]+(-0.509658743529)*x_ref[1]**o+(-0.978718968218)*x_ref[1]+(-0.603928369428)*x_ref[2]**o+(-0.550813953692)*x_ref[2]
        ref[0,1,1]=(-0.935214755436)*x_ref[0]**o+(0.44745901813)*x_ref[0]+(-0.582828846453)*x_ref[1]**o+(-0.181547183233)*x_ref[1]+(-0.183912053485)*x_ref[2]**o+(0.446152494296)*x_ref[2]
        ref[1,0,0]=(-0.486345219779)*x_ref[0]**o+(0.753529436833)*x_ref[0]+(0.138157856552)*x_ref[1]**o+(-0.874849020539)*x_ref[1]+(0.374396543792)*x_ref[2]**o+(0.26548456046)*x_ref[2]
        ref[1,0,1]=(0.492908753036)*x_ref[0]**o+(-0.418118685105)*x_ref[0]+(0.880817569823)*x_ref[1]**o+(-0.827691145524)*x_ref[1]+(0.238633156133)*x_ref[2]**o+(-0.463687731341)*x_ref[2]
        ref[1,1,0]=(-0.397373578442)*x_ref[0]**o+(-0.316321105607)*x_ref[0]+(0.647511225459)*x_ref[1]**o+(-0.82553752195)*x_ref[1]+(-0.715549303171)*x_ref[2]**o+(0.804623957459)*x_ref[2]
        ref[1,1,1]=(0.414743130234)*x_ref[0]**o+(0.0201794413683)*x_ref[0]+(-0.636337599584)*x_ref[1]**o+(-0.230053616026)*x_ref[1]+(-0.4302153311)*x_ref[2]**o+(-0.387468029626)*x_ref[2]
        ref[2,0,0]=(0.715100517336)*x_ref[0]**o+(-0.4286120178)*x_ref[0]+(0.0987858542117)*x_ref[1]**o+(0.815093470301)*x_ref[1]+(0.293455353257)*x_ref[2]**o+(0.200576090379)*x_ref[2]
        ref[2,0,1]=(-0.158330015822)*x_ref[0]**o+(0.363339023457)*x_ref[0]+(-0.796055594971)*x_ref[1]**o+(-0.213299205306)*x_ref[1]+(-0.136620988812)*x_ref[2]**o+(0.10733886947)*x_ref[2]
        ref[2,1,0]=(-0.258729405118)*x_ref[0]**o+(0.0669553480038)*x_ref[0]+(0.997663166613)*x_ref[1]**o+(0.00398861063931)*x_ref[1]+(-0.989459856803)*x_ref[2]**o+(0.932687384325)*x_ref[2]
        ref[2,1,1]=(-0.974431322319)*x_ref[0]**o+(-0.486447431804)*x_ref[0]+(0.871784873347)*x_ref[1]**o+(-0.978904038765)*x_ref[1]+(0.647341201506)*x_ref[2]**o+(-0.269683556528)*x_ref[2]
        ref[3,0,0]=(-0.369507398073)*x_ref[0]**o+(-0.772085607074)*x_ref[0]+(0.417550881937)*x_ref[1]**o+(0.271743719652)*x_ref[1]+(0.881807154737)*x_ref[2]**o+(0.54725428039)*x_ref[2]
        ref[3,0,1]=(0.440089335941)*x_ref[0]**o+(0.0844171421294)*x_ref[0]+(0.926389704362)*x_ref[1]**o+(-0.0709182998297)*x_ref[1]+(0.903978660317)*x_ref[2]**o+(-0.160556206716)*x_ref[2]
        ref[3,1,0]=(-0.49936391282)*x_ref[0]**o+(-0.632753931881)*x_ref[0]+(-0.604686084554)*x_ref[1]**o+(-0.954205199531)*x_ref[1]+(-0.535102184845)*x_ref[2]**o+(0.621683814123)*x_ref[2]
        ref[3,1,1]=(-0.117790973132)*x_ref[0]**o+(-0.649091727896)*x_ref[0]+(0.925682916785)*x_ref[1]**o+(0.852053515469)*x_ref[1]+(-0.479218429897)*x_ref[2]**o+(0.426823478575)*x_ref[2]
        ref[4,0,0]=(0.454174564319)*x_ref[0]**o+(-0.855975647406)*x_ref[0]+(0.775972969288)*x_ref[1]**o+(0.267900999893)*x_ref[1]+(0.221387501995)*x_ref[2]**o+(-0.26413791201)*x_ref[2]
        ref[4,0,1]=(0.875191302431)*x_ref[0]**o+(-0.362782502152)*x_ref[0]+(0.24600070339)*x_ref[1]**o+(-0.566709533298)*x_ref[1]+(-0.82187455531)*x_ref[2]**o+(-0.943929640301)*x_ref[2]
        ref[4,1,0]=(-0.409313259973)*x_ref[0]**o+(-0.784833509001)*x_ref[0]+(-0.939690505056)*x_ref[1]**o+(-0.0736280226974)*x_ref[1]+(0.939072745486)*x_ref[2]**o+(0.611697159959)*x_ref[2]
        ref[4,1,1]=(0.893374246808)*x_ref[0]**o+(0.36113513561)*x_ref[0]+(0.227899271271)*x_ref[1]**o+(-0.847221657012)*x_ref[1]+(-0.0106368947417)*x_ref[2]**o+(0.0358896817595)*x_ref[2]
        ref[5,0,0]=(0.0329230524924)*x_ref[0]**o+(0.0565192084014)*x_ref[0]+(-0.68398448658)*x_ref[1]**o+(0.909035618076)*x_ref[1]+(-0.900060440872)*x_ref[2]**o+(-0.481014787833)*x_ref[2]
        ref[5,0,1]=(0.404966692391)*x_ref[0]**o+(0.647468764373)*x_ref[0]+(-0.723847499346)*x_ref[1]**o+(-0.0740896957606)*x_ref[1]+(-0.00628515079923)*x_ref[2]**o+(0.906667863539)*x_ref[2]
        ref[5,1,0]=(0.983181781531)*x_ref[0]**o+(-0.297325990236)*x_ref[0]+(0.00489064543352)*x_ref[1]**o+(0.667560767038)*x_ref[1]+(-0.665768141301)*x_ref[2]**o+(0.772761393608)*x_ref[2]
        ref[5,1,1]=(-0.496581691354)*x_ref[0]**o+(0.0121337979505)*x_ref[0]+(0.492281382726)*x_ref[1]**o+(0.545232714025)*x_ref[1]+(0.418321481532)*x_ref[2]**o+(0.489635969764)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_ReducedFunctionOnContactOne_rank4(self):
      """
      tests interpolate for rank 4 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactOne onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactOne(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5, 3, 2),w)
      ref=Data(0,(4, 5, 3, 2),w_ref)
      if dim==2:
        arg[0,0,0,0]=(-0.282394950702)*x[0]**o+(0.836580157584)*x[0]+(0.337652262648)*x[1]**o+(-0.442694263325)*x[1]
        arg[0,0,0,1]=(0.85150774028)*x[0]**o+(0.998297296593)*x[0]+(-0.710029954342)*x[1]**o+(0.0816168287837)*x[1]
        arg[0,0,1,0]=(-0.636435102838)*x[0]**o+(-0.613479863553)*x[0]+(-0.861506513168)*x[1]**o+(-0.242998961478)*x[1]
        arg[0,0,1,1]=(-0.00671863978521)*x[0]**o+(0.909404084068)*x[0]+(-0.734956067026)*x[1]**o+(0.256530538264)*x[1]
        arg[0,0,2,0]=(-0.646631608999)*x[0]**o+(-0.521130455843)*x[0]+(-0.44328404557)*x[1]**o+(-0.156860325817)*x[1]
        arg[0,0,2,1]=(-0.817550008949)*x[0]**o+(0.702414497795)*x[0]+(0.946449680763)*x[1]**o+(-0.61576361925)*x[1]
        arg[0,1,0,0]=(-0.527206691595)*x[0]**o+(-0.43469216984)*x[0]+(-0.108455901154)*x[1]**o+(0.435646962156)*x[1]
        arg[0,1,0,1]=(-0.148522219535)*x[0]**o+(0.146912249755)*x[0]+(-0.414934945665)*x[1]**o+(-0.422754988714)*x[1]
        arg[0,1,1,0]=(0.177760972947)*x[0]**o+(-0.938522530862)*x[0]+(0.566229652264)*x[1]**o+(0.255269743628)*x[1]
        arg[0,1,1,1]=(-0.317807933792)*x[0]**o+(0.432181451246)*x[0]+(-0.358821513362)*x[1]**o+(-0.470517232585)*x[1]
        arg[0,1,2,0]=(0.486536430227)*x[0]**o+(-0.533653809349)*x[0]+(-0.126612514045)*x[1]**o+(0.103088115487)*x[1]
        arg[0,1,2,1]=(-0.995976559589)*x[0]**o+(0.965537167421)*x[0]+(-0.351307699531)*x[1]**o+(-0.684367434666)*x[1]
        arg[0,2,0,0]=(0.976008613768)*x[0]**o+(-0.890573007002)*x[0]+(-0.440709423944)*x[1]**o+(0.315724459367)*x[1]
        arg[0,2,0,1]=(-0.595275811962)*x[0]**o+(-0.599634144509)*x[0]+(0.646466002346)*x[1]**o+(-0.94529568651)*x[1]
        arg[0,2,1,0]=(0.977691521821)*x[0]**o+(-0.0730997643682)*x[0]+(-0.338897498737)*x[1]**o+(0.246308678824)*x[1]
        arg[0,2,1,1]=(-0.111317410732)*x[0]**o+(-0.574381793398)*x[0]+(-0.0314431899843)*x[1]**o+(0.393105047096)*x[1]
        arg[0,2,2,0]=(0.886320020757)*x[0]**o+(-0.785042003544)*x[0]+(0.82609269979)*x[1]**o+(-0.677169484337)*x[1]
        arg[0,2,2,1]=(0.811027138079)*x[0]**o+(-0.375108470948)*x[0]+(0.781472464253)*x[1]**o+(0.217640794143)*x[1]
        arg[0,3,0,0]=(0.264884117091)*x[0]**o+(0.116591319557)*x[0]+(0.282667679763)*x[1]**o+(0.111123454337)*x[1]
        arg[0,3,0,1]=(0.0451626985922)*x[0]**o+(-0.93771279728)*x[0]+(0.129171764072)*x[1]**o+(0.598367954162)*x[1]
        arg[0,3,1,0]=(0.905243511086)*x[0]**o+(0.0837062573586)*x[0]+(0.0657223248433)*x[1]**o+(-0.746644690072)*x[1]
        arg[0,3,1,1]=(-0.0547856755485)*x[0]**o+(-0.22157806755)*x[0]+(0.124623625108)*x[1]**o+(0.0436590157081)*x[1]
        arg[0,3,2,0]=(-0.558323744577)*x[0]**o+(-0.646043519007)*x[0]+(0.991885458004)*x[1]**o+(-0.634140808163)*x[1]
        arg[0,3,2,1]=(-0.27247983028)*x[0]**o+(0.803906404485)*x[0]+(-0.816826609924)*x[1]**o+(-0.969532783336)*x[1]
        arg[0,4,0,0]=(0.638733842891)*x[0]**o+(-0.972793270208)*x[0]+(0.714611008099)*x[1]**o+(0.586988595847)*x[1]
        arg[0,4,0,1]=(0.63584760734)*x[0]**o+(-0.653740236747)*x[0]+(-0.596304731446)*x[1]**o+(0.42519786429)*x[1]
        arg[0,4,1,0]=(0.0665562973529)*x[0]**o+(0.362152092093)*x[0]+(0.732530335739)*x[1]**o+(0.618536326941)*x[1]
        arg[0,4,1,1]=(0.486060667608)*x[0]**o+(0.59489013279)*x[0]+(-0.547237017498)*x[1]**o+(0.0371894705857)*x[1]
        arg[0,4,2,0]=(0.395505844397)*x[0]**o+(-0.869090434535)*x[0]+(-0.606640827437)*x[1]**o+(0.269995459083)*x[1]
        arg[0,4,2,1]=(0.304671547176)*x[0]**o+(-0.515772523849)*x[0]+(0.546114300876)*x[1]**o+(0.530247404341)*x[1]
        arg[1,0,0,0]=(0.408332677433)*x[0]**o+(-0.0778807359918)*x[0]+(-0.803337489297)*x[1]**o+(-0.123835718422)*x[1]
        arg[1,0,0,1]=(-0.00441620254944)*x[0]**o+(0.0321259598831)*x[0]+(-0.998381508472)*x[1]**o+(0.800767747511)*x[1]
        arg[1,0,1,0]=(-0.923683057418)*x[0]**o+(0.579405114902)*x[0]+(-0.939639823585)*x[1]**o+(-0.138423358134)*x[1]
        arg[1,0,1,1]=(-0.558420234848)*x[0]**o+(0.133316569813)*x[0]+(-0.294134717329)*x[1]**o+(-0.750897749044)*x[1]
        arg[1,0,2,0]=(0.478951298545)*x[0]**o+(-0.807173978799)*x[0]+(0.852438146763)*x[1]**o+(-0.103814229563)*x[1]
        arg[1,0,2,1]=(-0.875250726603)*x[0]**o+(-0.934803428491)*x[0]+(0.143352873672)*x[1]**o+(0.600934881206)*x[1]
        arg[1,1,0,0]=(-0.0539476321018)*x[0]**o+(0.194177663417)*x[0]+(-0.862877991009)*x[1]**o+(-0.438337993664)*x[1]
        arg[1,1,0,1]=(-0.623379355474)*x[0]**o+(-0.0746994739748)*x[0]+(0.365005438956)*x[1]**o+(-0.431737688205)*x[1]
        arg[1,1,1,0]=(0.0936054461735)*x[0]**o+(-0.974594269446)*x[0]+(-0.121917598428)*x[1]**o+(-0.84429083727)*x[1]
        arg[1,1,1,1]=(0.967763348672)*x[0]**o+(0.331698826475)*x[0]+(0.66448515877)*x[1]**o+(0.00214985930661)*x[1]
        arg[1,1,2,0]=(-0.312146580762)*x[0]**o+(0.998152287706)*x[0]+(-0.474242942777)*x[1]**o+(0.637827347846)*x[1]
        arg[1,1,2,1]=(-0.556765443436)*x[0]**o+(0.227327929774)*x[0]+(0.739457845569)*x[1]**o+(-0.615070017663)*x[1]
        arg[1,2,0,0]=(-0.23030660552)*x[0]**o+(0.750479465953)*x[0]+(0.278215408392)*x[1]**o+(-0.640618698224)*x[1]
        arg[1,2,0,1]=(-0.0713217101129)*x[0]**o+(0.500737477887)*x[0]+(-0.972834770005)*x[1]**o+(-0.420386056033)*x[1]
        arg[1,2,1,0]=(-0.497322989847)*x[0]**o+(-0.528685124841)*x[0]+(-0.934386348496)*x[1]**o+(0.191930553839)*x[1]
        arg[1,2,1,1]=(0.124362419781)*x[0]**o+(-0.0457849787456)*x[0]+(-0.360596154712)*x[1]**o+(0.799736253715)*x[1]
        arg[1,2,2,0]=(-0.346294878893)*x[0]**o+(0.427772309361)*x[0]+(-0.0642666791012)*x[1]**o+(-0.214442238311)*x[1]
        arg[1,2,2,1]=(0.24434837411)*x[0]**o+(-0.913373159786)*x[0]+(0.158750862391)*x[1]**o+(-0.544328807576)*x[1]
        arg[1,3,0,0]=(0.870847002687)*x[0]**o+(-0.466884462301)*x[0]+(-0.343358829445)*x[1]**o+(-0.180956419766)*x[1]
        arg[1,3,0,1]=(-0.813518850863)*x[0]**o+(0.960810467501)*x[0]+(0.35091416904)*x[1]**o+(0.84534544948)*x[1]
        arg[1,3,1,0]=(-0.284687751218)*x[0]**o+(-0.274583519355)*x[0]+(0.338892764102)*x[1]**o+(-0.145142443753)*x[1]
        arg[1,3,1,1]=(0.921064262137)*x[0]**o+(0.152552421678)*x[0]+(0.827247693375)*x[1]**o+(-0.569529848619)*x[1]
        arg[1,3,2,0]=(0.133567630356)*x[0]**o+(-0.0429397358364)*x[0]+(-0.175383170663)*x[1]**o+(0.584407631334)*x[1]
        arg[1,3,2,1]=(0.203880167371)*x[0]**o+(-0.422978073825)*x[0]+(-0.210109267588)*x[1]**o+(-0.902327373219)*x[1]
        arg[1,4,0,0]=(-0.566386864876)*x[0]**o+(-0.139795085483)*x[0]+(-0.00852913464238)*x[1]**o+(0.569151381884)*x[1]
        arg[1,4,0,1]=(-0.937256625212)*x[0]**o+(0.563438253128)*x[0]+(-0.586182238715)*x[1]**o+(-0.566169678723)*x[1]
        arg[1,4,1,0]=(-0.73296890712)*x[0]**o+(0.359249594406)*x[0]+(0.66050790616)*x[1]**o+(-0.367919797426)*x[1]
        arg[1,4,1,1]=(-0.81165734248)*x[0]**o+(0.460592817838)*x[0]+(0.953573909492)*x[1]**o+(0.329659021274)*x[1]
        arg[1,4,2,0]=(0.345994483595)*x[0]**o+(-0.18547324343)*x[0]+(0.669452926238)*x[1]**o+(0.42513249272)*x[1]
        arg[1,4,2,1]=(0.250677642616)*x[0]**o+(0.466118496803)*x[0]+(-0.776608543595)*x[1]**o+(0.449144243682)*x[1]
        arg[2,0,0,0]=(0.244172909021)*x[0]**o+(0.0311068322898)*x[0]+(-0.909348039508)*x[1]**o+(0.376500719414)*x[1]
        arg[2,0,0,1]=(-0.033516640667)*x[0]**o+(0.698424718328)*x[0]+(-0.282814785542)*x[1]**o+(0.61118022947)*x[1]
        arg[2,0,1,0]=(-0.946672649538)*x[0]**o+(0.814325244591)*x[0]+(-0.353215562825)*x[1]**o+(-0.138150257429)*x[1]
        arg[2,0,1,1]=(-0.886509051692)*x[0]**o+(-0.0468334385819)*x[0]+(0.71984719189)*x[1]**o+(-0.324744971493)*x[1]
        arg[2,0,2,0]=(-0.780921647053)*x[0]**o+(-0.401418496932)*x[0]+(0.647847274332)*x[1]**o+(-0.831834322278)*x[1]
        arg[2,0,2,1]=(0.835155004862)*x[0]**o+(0.684459986902)*x[0]+(-0.301244240677)*x[1]**o+(-0.161292780187)*x[1]
        arg[2,1,0,0]=(-0.652199398394)*x[0]**o+(0.62480722178)*x[0]+(-0.474535712914)*x[1]**o+(0.926061797766)*x[1]
        arg[2,1,0,1]=(-0.813365478568)*x[0]**o+(-0.38795792417)*x[0]+(0.378161353697)*x[1]**o+(0.70581806211)*x[1]
        arg[2,1,1,0]=(0.729265594934)*x[0]**o+(-0.469397842359)*x[0]+(0.642472199926)*x[1]**o+(-0.779527283011)*x[1]
        arg[2,1,1,1]=(0.0517342917574)*x[0]**o+(-0.414111007917)*x[0]+(-0.89437375241)*x[1]**o+(-0.830119585842)*x[1]
        arg[2,1,2,0]=(0.769898486257)*x[0]**o+(0.791725624832)*x[0]+(0.297519329396)*x[1]**o+(-0.959462006611)*x[1]
        arg[2,1,2,1]=(0.817802144251)*x[0]**o+(0.69851258911)*x[0]+(-0.174401735228)*x[1]**o+(0.13449696498)*x[1]
        arg[2,2,0,0]=(0.547972662256)*x[0]**o+(-0.66491929979)*x[0]+(0.330783041672)*x[1]**o+(0.816196730606)*x[1]
        arg[2,2,0,1]=(0.245971097763)*x[0]**o+(0.213087481481)*x[0]+(-0.840766291392)*x[1]**o+(0.388414557376)*x[1]
        arg[2,2,1,0]=(0.523151676444)*x[0]**o+(0.854817486027)*x[0]+(-0.53570188528)*x[1]**o+(-0.68149362242)*x[1]
        arg[2,2,1,1]=(-0.984200247787)*x[0]**o+(0.83324365092)*x[0]+(-0.896184408816)*x[1]**o+(0.102276243487)*x[1]
        arg[2,2,2,0]=(-0.216357164308)*x[0]**o+(-0.597984882107)*x[0]+(-0.205435921511)*x[1]**o+(0.484947652759)*x[1]
        arg[2,2,2,1]=(-0.650046253868)*x[0]**o+(-0.624878186105)*x[0]+(0.786519914406)*x[1]**o+(0.28027273095)*x[1]
        arg[2,3,0,0]=(-0.0821718346579)*x[0]**o+(-0.687049917467)*x[0]+(0.380792333736)*x[1]**o+(0.151185837715)*x[1]
        arg[2,3,0,1]=(0.61796113854)*x[0]**o+(0.844624413065)*x[0]+(-0.980619401098)*x[1]**o+(-0.854924862641)*x[1]
        arg[2,3,1,0]=(0.812293906883)*x[0]**o+(-0.225593208768)*x[0]+(0.964175374163)*x[1]**o+(0.848850064926)*x[1]
        arg[2,3,1,1]=(0.376453572937)*x[0]**o+(0.994818812059)*x[0]+(0.58872018303)*x[1]**o+(0.68332473361)*x[1]
        arg[2,3,2,0]=(-0.00948421972234)*x[0]**o+(-0.680685462738)*x[0]+(-0.767076278167)*x[1]**o+(0.210165437284)*x[1]
        arg[2,3,2,1]=(-0.455199698263)*x[0]**o+(-0.763430401419)*x[0]+(0.183773350212)*x[1]**o+(0.43105940923)*x[1]
        arg[2,4,0,0]=(-0.673962025988)*x[0]**o+(-0.679073069725)*x[0]+(0.11018431715)*x[1]**o+(0.349020527482)*x[1]
        arg[2,4,0,1]=(-0.876691282076)*x[0]**o+(0.920902226631)*x[0]+(-0.477417470246)*x[1]**o+(0.836417982216)*x[1]
        arg[2,4,1,0]=(0.567966851024)*x[0]**o+(0.583241431716)*x[0]+(-0.271851271008)*x[1]**o+(0.678727902863)*x[1]
        arg[2,4,1,1]=(0.707400094)*x[0]**o+(0.802820856444)*x[0]+(0.84518937616)*x[1]**o+(-0.0267193533403)*x[1]
        arg[2,4,2,0]=(0.213268920036)*x[0]**o+(0.780721415328)*x[0]+(0.839292948154)*x[1]**o+(-0.0376334744443)*x[1]
        arg[2,4,2,1]=(0.136255687031)*x[0]**o+(-0.582766278693)*x[0]+(0.257662521321)*x[1]**o+(0.808337521979)*x[1]
        arg[3,0,0,0]=(-0.524068770822)*x[0]**o+(0.942236878842)*x[0]+(0.889496255793)*x[1]**o+(-0.904810742393)*x[1]
        arg[3,0,0,1]=(0.780157465475)*x[0]**o+(-0.763811668832)*x[0]+(-0.517482070639)*x[1]**o+(0.662770780676)*x[1]
        arg[3,0,1,0]=(-0.693227564812)*x[0]**o+(-0.763506211841)*x[0]+(-0.427709881126)*x[1]**o+(0.159109827204)*x[1]
        arg[3,0,1,1]=(0.644975629084)*x[0]**o+(-0.928112761899)*x[0]+(-0.530772740083)*x[1]**o+(-0.723174790087)*x[1]
        arg[3,0,2,0]=(-0.923467274097)*x[0]**o+(0.806128117288)*x[0]+(0.177779363207)*x[1]**o+(0.862243885542)*x[1]
        arg[3,0,2,1]=(-0.0823252901358)*x[0]**o+(0.289022916284)*x[0]+(-0.518508771021)*x[1]**o+(0.174564769845)*x[1]
        arg[3,1,0,0]=(0.608837578444)*x[0]**o+(0.898216758361)*x[0]+(-0.844730109761)*x[1]**o+(0.745974173207)*x[1]
        arg[3,1,0,1]=(-0.11776411078)*x[0]**o+(0.186256502356)*x[0]+(0.565289400333)*x[1]**o+(-0.776173716206)*x[1]
        arg[3,1,1,0]=(-0.170736580152)*x[0]**o+(0.829581661683)*x[0]+(0.310135833859)*x[1]**o+(-0.509629545687)*x[1]
        arg[3,1,1,1]=(-0.225246122751)*x[0]**o+(0.586754377294)*x[0]+(-0.726064725635)*x[1]**o+(0.136952605704)*x[1]
        arg[3,1,2,0]=(-0.287932115478)*x[0]**o+(0.858031396153)*x[0]+(-0.493441745353)*x[1]**o+(0.601827664994)*x[1]
        arg[3,1,2,1]=(0.665503842862)*x[0]**o+(-0.515998970452)*x[0]+(-0.890055558624)*x[1]**o+(-0.0446371451457)*x[1]
        arg[3,2,0,0]=(-0.33741557585)*x[0]**o+(-0.416183752333)*x[0]+(-0.92430073538)*x[1]**o+(-0.486265090904)*x[1]
        arg[3,2,0,1]=(0.910860925994)*x[0]**o+(-0.896869755389)*x[0]+(0.0634658185324)*x[1]**o+(-0.383136455739)*x[1]
        arg[3,2,1,0]=(0.386839297976)*x[0]**o+(-0.190659927071)*x[0]+(-0.304846978185)*x[1]**o+(-0.92705511899)*x[1]
        arg[3,2,1,1]=(-0.0197812018683)*x[0]**o+(-0.479083095775)*x[0]+(0.502489976089)*x[1]**o+(-0.859440665475)*x[1]
        arg[3,2,2,0]=(-0.507672045299)*x[0]**o+(0.498689676072)*x[0]+(-0.719735804645)*x[1]**o+(-0.522053306674)*x[1]
        arg[3,2,2,1]=(0.46207629211)*x[0]**o+(-0.496697066039)*x[0]+(0.994069339904)*x[1]**o+(0.873222754449)*x[1]
        arg[3,3,0,0]=(0.126410029692)*x[0]**o+(0.51067050196)*x[0]+(-0.978384824088)*x[1]**o+(-0.68606930503)*x[1]
        arg[3,3,0,1]=(0.376888157008)*x[0]**o+(-0.14015764025)*x[0]+(0.523667700758)*x[1]**o+(0.0277726855258)*x[1]
        arg[3,3,1,0]=(-0.105865233294)*x[0]**o+(-0.0515230741203)*x[0]+(0.203766525036)*x[1]**o+(-0.0431893014387)*x[1]
        arg[3,3,1,1]=(-0.404351448486)*x[0]**o+(-0.639161429157)*x[0]+(-0.957341091708)*x[1]**o+(-0.103346466984)*x[1]
        arg[3,3,2,0]=(-0.0523508780623)*x[0]**o+(-0.563589575544)*x[0]+(0.158487727325)*x[1]**o+(-0.519455492178)*x[1]
        arg[3,3,2,1]=(-0.823286342218)*x[0]**o+(-0.493663499535)*x[0]+(-0.409825574064)*x[1]**o+(-0.0406101279806)*x[1]
        arg[3,4,0,0]=(0.926212910387)*x[0]**o+(-0.0944507963)*x[0]+(0.640740499391)*x[1]**o+(-0.106969076047)*x[1]
        arg[3,4,0,1]=(-0.912044993941)*x[0]**o+(-0.495323777918)*x[0]+(-0.595831845108)*x[1]**o+(-0.313723426318)*x[1]
        arg[3,4,1,0]=(-0.0730898742548)*x[0]**o+(0.0524008484492)*x[0]+(-0.948232935497)*x[1]**o+(0.952670824572)*x[1]
        arg[3,4,1,1]=(-0.81869438138)*x[0]**o+(-0.894654266952)*x[0]+(0.414078621872)*x[1]**o+(-0.876690817847)*x[1]
        arg[3,4,2,0]=(0.00688155078441)*x[0]**o+(0.845964586551)*x[0]+(0.0183149371892)*x[1]**o+(-0.882931394537)*x[1]
        arg[3,4,2,1]=(-0.734830097149)*x[0]**o+(-0.37824588859)*x[0]+(-0.815402620311)*x[1]**o+(-0.357905685517)*x[1]
        ref[0,0,0,0]=(-0.282394950702)*x_ref[0]**o+(0.836580157584)*x_ref[0]+(0.337652262648)*x_ref[1]**o+(-0.442694263325)*x_ref[1]
        ref[0,0,0,1]=(0.85150774028)*x_ref[0]**o+(0.998297296593)*x_ref[0]+(-0.710029954342)*x_ref[1]**o+(0.0816168287837)*x_ref[1]
        ref[0,0,1,0]=(-0.636435102838)*x_ref[0]**o+(-0.613479863553)*x_ref[0]+(-0.861506513168)*x_ref[1]**o+(-0.242998961478)*x_ref[1]
        ref[0,0,1,1]=(-0.00671863978521)*x_ref[0]**o+(0.909404084068)*x_ref[0]+(-0.734956067026)*x_ref[1]**o+(0.256530538264)*x_ref[1]
        ref[0,0,2,0]=(-0.646631608999)*x_ref[0]**o+(-0.521130455843)*x_ref[0]+(-0.44328404557)*x_ref[1]**o+(-0.156860325817)*x_ref[1]
        ref[0,0,2,1]=(-0.817550008949)*x_ref[0]**o+(0.702414497795)*x_ref[0]+(0.946449680763)*x_ref[1]**o+(-0.61576361925)*x_ref[1]
        ref[0,1,0,0]=(-0.527206691595)*x_ref[0]**o+(-0.43469216984)*x_ref[0]+(-0.108455901154)*x_ref[1]**o+(0.435646962156)*x_ref[1]
        ref[0,1,0,1]=(-0.148522219535)*x_ref[0]**o+(0.146912249755)*x_ref[0]+(-0.414934945665)*x_ref[1]**o+(-0.422754988714)*x_ref[1]
        ref[0,1,1,0]=(0.177760972947)*x_ref[0]**o+(-0.938522530862)*x_ref[0]+(0.566229652264)*x_ref[1]**o+(0.255269743628)*x_ref[1]
        ref[0,1,1,1]=(-0.317807933792)*x_ref[0]**o+(0.432181451246)*x_ref[0]+(-0.358821513362)*x_ref[1]**o+(-0.470517232585)*x_ref[1]
        ref[0,1,2,0]=(0.486536430227)*x_ref[0]**o+(-0.533653809349)*x_ref[0]+(-0.126612514045)*x_ref[1]**o+(0.103088115487)*x_ref[1]
        ref[0,1,2,1]=(-0.995976559589)*x_ref[0]**o+(0.965537167421)*x_ref[0]+(-0.351307699531)*x_ref[1]**o+(-0.684367434666)*x_ref[1]
        ref[0,2,0,0]=(0.976008613768)*x_ref[0]**o+(-0.890573007002)*x_ref[0]+(-0.440709423944)*x_ref[1]**o+(0.315724459367)*x_ref[1]
        ref[0,2,0,1]=(-0.595275811962)*x_ref[0]**o+(-0.599634144509)*x_ref[0]+(0.646466002346)*x_ref[1]**o+(-0.94529568651)*x_ref[1]
        ref[0,2,1,0]=(0.977691521821)*x_ref[0]**o+(-0.0730997643682)*x_ref[0]+(-0.338897498737)*x_ref[1]**o+(0.246308678824)*x_ref[1]
        ref[0,2,1,1]=(-0.111317410732)*x_ref[0]**o+(-0.574381793398)*x_ref[0]+(-0.0314431899843)*x_ref[1]**o+(0.393105047096)*x_ref[1]
        ref[0,2,2,0]=(0.886320020757)*x_ref[0]**o+(-0.785042003544)*x_ref[0]+(0.82609269979)*x_ref[1]**o+(-0.677169484337)*x_ref[1]
        ref[0,2,2,1]=(0.811027138079)*x_ref[0]**o+(-0.375108470948)*x_ref[0]+(0.781472464253)*x_ref[1]**o+(0.217640794143)*x_ref[1]
        ref[0,3,0,0]=(0.264884117091)*x_ref[0]**o+(0.116591319557)*x_ref[0]+(0.282667679763)*x_ref[1]**o+(0.111123454337)*x_ref[1]
        ref[0,3,0,1]=(0.0451626985922)*x_ref[0]**o+(-0.93771279728)*x_ref[0]+(0.129171764072)*x_ref[1]**o+(0.598367954162)*x_ref[1]
        ref[0,3,1,0]=(0.905243511086)*x_ref[0]**o+(0.0837062573586)*x_ref[0]+(0.0657223248433)*x_ref[1]**o+(-0.746644690072)*x_ref[1]
        ref[0,3,1,1]=(-0.0547856755485)*x_ref[0]**o+(-0.22157806755)*x_ref[0]+(0.124623625108)*x_ref[1]**o+(0.0436590157081)*x_ref[1]
        ref[0,3,2,0]=(-0.558323744577)*x_ref[0]**o+(-0.646043519007)*x_ref[0]+(0.991885458004)*x_ref[1]**o+(-0.634140808163)*x_ref[1]
        ref[0,3,2,1]=(-0.27247983028)*x_ref[0]**o+(0.803906404485)*x_ref[0]+(-0.816826609924)*x_ref[1]**o+(-0.969532783336)*x_ref[1]
        ref[0,4,0,0]=(0.638733842891)*x_ref[0]**o+(-0.972793270208)*x_ref[0]+(0.714611008099)*x_ref[1]**o+(0.586988595847)*x_ref[1]
        ref[0,4,0,1]=(0.63584760734)*x_ref[0]**o+(-0.653740236747)*x_ref[0]+(-0.596304731446)*x_ref[1]**o+(0.42519786429)*x_ref[1]
        ref[0,4,1,0]=(0.0665562973529)*x_ref[0]**o+(0.362152092093)*x_ref[0]+(0.732530335739)*x_ref[1]**o+(0.618536326941)*x_ref[1]
        ref[0,4,1,1]=(0.486060667608)*x_ref[0]**o+(0.59489013279)*x_ref[0]+(-0.547237017498)*x_ref[1]**o+(0.0371894705857)*x_ref[1]
        ref[0,4,2,0]=(0.395505844397)*x_ref[0]**o+(-0.869090434535)*x_ref[0]+(-0.606640827437)*x_ref[1]**o+(0.269995459083)*x_ref[1]
        ref[0,4,2,1]=(0.304671547176)*x_ref[0]**o+(-0.515772523849)*x_ref[0]+(0.546114300876)*x_ref[1]**o+(0.530247404341)*x_ref[1]
        ref[1,0,0,0]=(0.408332677433)*x_ref[0]**o+(-0.0778807359918)*x_ref[0]+(-0.803337489297)*x_ref[1]**o+(-0.123835718422)*x_ref[1]
        ref[1,0,0,1]=(-0.00441620254944)*x_ref[0]**o+(0.0321259598831)*x_ref[0]+(-0.998381508472)*x_ref[1]**o+(0.800767747511)*x_ref[1]
        ref[1,0,1,0]=(-0.923683057418)*x_ref[0]**o+(0.579405114902)*x_ref[0]+(-0.939639823585)*x_ref[1]**o+(-0.138423358134)*x_ref[1]
        ref[1,0,1,1]=(-0.558420234848)*x_ref[0]**o+(0.133316569813)*x_ref[0]+(-0.294134717329)*x_ref[1]**o+(-0.750897749044)*x_ref[1]
        ref[1,0,2,0]=(0.478951298545)*x_ref[0]**o+(-0.807173978799)*x_ref[0]+(0.852438146763)*x_ref[1]**o+(-0.103814229563)*x_ref[1]
        ref[1,0,2,1]=(-0.875250726603)*x_ref[0]**o+(-0.934803428491)*x_ref[0]+(0.143352873672)*x_ref[1]**o+(0.600934881206)*x_ref[1]
        ref[1,1,0,0]=(-0.0539476321018)*x_ref[0]**o+(0.194177663417)*x_ref[0]+(-0.862877991009)*x_ref[1]**o+(-0.438337993664)*x_ref[1]
        ref[1,1,0,1]=(-0.623379355474)*x_ref[0]**o+(-0.0746994739748)*x_ref[0]+(0.365005438956)*x_ref[1]**o+(-0.431737688205)*x_ref[1]
        ref[1,1,1,0]=(0.0936054461735)*x_ref[0]**o+(-0.974594269446)*x_ref[0]+(-0.121917598428)*x_ref[1]**o+(-0.84429083727)*x_ref[1]
        ref[1,1,1,1]=(0.967763348672)*x_ref[0]**o+(0.331698826475)*x_ref[0]+(0.66448515877)*x_ref[1]**o+(0.00214985930661)*x_ref[1]
        ref[1,1,2,0]=(-0.312146580762)*x_ref[0]**o+(0.998152287706)*x_ref[0]+(-0.474242942777)*x_ref[1]**o+(0.637827347846)*x_ref[1]
        ref[1,1,2,1]=(-0.556765443436)*x_ref[0]**o+(0.227327929774)*x_ref[0]+(0.739457845569)*x_ref[1]**o+(-0.615070017663)*x_ref[1]
        ref[1,2,0,0]=(-0.23030660552)*x_ref[0]**o+(0.750479465953)*x_ref[0]+(0.278215408392)*x_ref[1]**o+(-0.640618698224)*x_ref[1]
        ref[1,2,0,1]=(-0.0713217101129)*x_ref[0]**o+(0.500737477887)*x_ref[0]+(-0.972834770005)*x_ref[1]**o+(-0.420386056033)*x_ref[1]
        ref[1,2,1,0]=(-0.497322989847)*x_ref[0]**o+(-0.528685124841)*x_ref[0]+(-0.934386348496)*x_ref[1]**o+(0.191930553839)*x_ref[1]
        ref[1,2,1,1]=(0.124362419781)*x_ref[0]**o+(-0.0457849787456)*x_ref[0]+(-0.360596154712)*x_ref[1]**o+(0.799736253715)*x_ref[1]
        ref[1,2,2,0]=(-0.346294878893)*x_ref[0]**o+(0.427772309361)*x_ref[0]+(-0.0642666791012)*x_ref[1]**o+(-0.214442238311)*x_ref[1]
        ref[1,2,2,1]=(0.24434837411)*x_ref[0]**o+(-0.913373159786)*x_ref[0]+(0.158750862391)*x_ref[1]**o+(-0.544328807576)*x_ref[1]
        ref[1,3,0,0]=(0.870847002687)*x_ref[0]**o+(-0.466884462301)*x_ref[0]+(-0.343358829445)*x_ref[1]**o+(-0.180956419766)*x_ref[1]
        ref[1,3,0,1]=(-0.813518850863)*x_ref[0]**o+(0.960810467501)*x_ref[0]+(0.35091416904)*x_ref[1]**o+(0.84534544948)*x_ref[1]
        ref[1,3,1,0]=(-0.284687751218)*x_ref[0]**o+(-0.274583519355)*x_ref[0]+(0.338892764102)*x_ref[1]**o+(-0.145142443753)*x_ref[1]
        ref[1,3,1,1]=(0.921064262137)*x_ref[0]**o+(0.152552421678)*x_ref[0]+(0.827247693375)*x_ref[1]**o+(-0.569529848619)*x_ref[1]
        ref[1,3,2,0]=(0.133567630356)*x_ref[0]**o+(-0.0429397358364)*x_ref[0]+(-0.175383170663)*x_ref[1]**o+(0.584407631334)*x_ref[1]
        ref[1,3,2,1]=(0.203880167371)*x_ref[0]**o+(-0.422978073825)*x_ref[0]+(-0.210109267588)*x_ref[1]**o+(-0.902327373219)*x_ref[1]
        ref[1,4,0,0]=(-0.566386864876)*x_ref[0]**o+(-0.139795085483)*x_ref[0]+(-0.00852913464238)*x_ref[1]**o+(0.569151381884)*x_ref[1]
        ref[1,4,0,1]=(-0.937256625212)*x_ref[0]**o+(0.563438253128)*x_ref[0]+(-0.586182238715)*x_ref[1]**o+(-0.566169678723)*x_ref[1]
        ref[1,4,1,0]=(-0.73296890712)*x_ref[0]**o+(0.359249594406)*x_ref[0]+(0.66050790616)*x_ref[1]**o+(-0.367919797426)*x_ref[1]
        ref[1,4,1,1]=(-0.81165734248)*x_ref[0]**o+(0.460592817838)*x_ref[0]+(0.953573909492)*x_ref[1]**o+(0.329659021274)*x_ref[1]
        ref[1,4,2,0]=(0.345994483595)*x_ref[0]**o+(-0.18547324343)*x_ref[0]+(0.669452926238)*x_ref[1]**o+(0.42513249272)*x_ref[1]
        ref[1,4,2,1]=(0.250677642616)*x_ref[0]**o+(0.466118496803)*x_ref[0]+(-0.776608543595)*x_ref[1]**o+(0.449144243682)*x_ref[1]
        ref[2,0,0,0]=(0.244172909021)*x_ref[0]**o+(0.0311068322898)*x_ref[0]+(-0.909348039508)*x_ref[1]**o+(0.376500719414)*x_ref[1]
        ref[2,0,0,1]=(-0.033516640667)*x_ref[0]**o+(0.698424718328)*x_ref[0]+(-0.282814785542)*x_ref[1]**o+(0.61118022947)*x_ref[1]
        ref[2,0,1,0]=(-0.946672649538)*x_ref[0]**o+(0.814325244591)*x_ref[0]+(-0.353215562825)*x_ref[1]**o+(-0.138150257429)*x_ref[1]
        ref[2,0,1,1]=(-0.886509051692)*x_ref[0]**o+(-0.0468334385819)*x_ref[0]+(0.71984719189)*x_ref[1]**o+(-0.324744971493)*x_ref[1]
        ref[2,0,2,0]=(-0.780921647053)*x_ref[0]**o+(-0.401418496932)*x_ref[0]+(0.647847274332)*x_ref[1]**o+(-0.831834322278)*x_ref[1]
        ref[2,0,2,1]=(0.835155004862)*x_ref[0]**o+(0.684459986902)*x_ref[0]+(-0.301244240677)*x_ref[1]**o+(-0.161292780187)*x_ref[1]
        ref[2,1,0,0]=(-0.652199398394)*x_ref[0]**o+(0.62480722178)*x_ref[0]+(-0.474535712914)*x_ref[1]**o+(0.926061797766)*x_ref[1]
        ref[2,1,0,1]=(-0.813365478568)*x_ref[0]**o+(-0.38795792417)*x_ref[0]+(0.378161353697)*x_ref[1]**o+(0.70581806211)*x_ref[1]
        ref[2,1,1,0]=(0.729265594934)*x_ref[0]**o+(-0.469397842359)*x_ref[0]+(0.642472199926)*x_ref[1]**o+(-0.779527283011)*x_ref[1]
        ref[2,1,1,1]=(0.0517342917574)*x_ref[0]**o+(-0.414111007917)*x_ref[0]+(-0.89437375241)*x_ref[1]**o+(-0.830119585842)*x_ref[1]
        ref[2,1,2,0]=(0.769898486257)*x_ref[0]**o+(0.791725624832)*x_ref[0]+(0.297519329396)*x_ref[1]**o+(-0.959462006611)*x_ref[1]
        ref[2,1,2,1]=(0.817802144251)*x_ref[0]**o+(0.69851258911)*x_ref[0]+(-0.174401735228)*x_ref[1]**o+(0.13449696498)*x_ref[1]
        ref[2,2,0,0]=(0.547972662256)*x_ref[0]**o+(-0.66491929979)*x_ref[0]+(0.330783041672)*x_ref[1]**o+(0.816196730606)*x_ref[1]
        ref[2,2,0,1]=(0.245971097763)*x_ref[0]**o+(0.213087481481)*x_ref[0]+(-0.840766291392)*x_ref[1]**o+(0.388414557376)*x_ref[1]
        ref[2,2,1,0]=(0.523151676444)*x_ref[0]**o+(0.854817486027)*x_ref[0]+(-0.53570188528)*x_ref[1]**o+(-0.68149362242)*x_ref[1]
        ref[2,2,1,1]=(-0.984200247787)*x_ref[0]**o+(0.83324365092)*x_ref[0]+(-0.896184408816)*x_ref[1]**o+(0.102276243487)*x_ref[1]
        ref[2,2,2,0]=(-0.216357164308)*x_ref[0]**o+(-0.597984882107)*x_ref[0]+(-0.205435921511)*x_ref[1]**o+(0.484947652759)*x_ref[1]
        ref[2,2,2,1]=(-0.650046253868)*x_ref[0]**o+(-0.624878186105)*x_ref[0]+(0.786519914406)*x_ref[1]**o+(0.28027273095)*x_ref[1]
        ref[2,3,0,0]=(-0.0821718346579)*x_ref[0]**o+(-0.687049917467)*x_ref[0]+(0.380792333736)*x_ref[1]**o+(0.151185837715)*x_ref[1]
        ref[2,3,0,1]=(0.61796113854)*x_ref[0]**o+(0.844624413065)*x_ref[0]+(-0.980619401098)*x_ref[1]**o+(-0.854924862641)*x_ref[1]
        ref[2,3,1,0]=(0.812293906883)*x_ref[0]**o+(-0.225593208768)*x_ref[0]+(0.964175374163)*x_ref[1]**o+(0.848850064926)*x_ref[1]
        ref[2,3,1,1]=(0.376453572937)*x_ref[0]**o+(0.994818812059)*x_ref[0]+(0.58872018303)*x_ref[1]**o+(0.68332473361)*x_ref[1]
        ref[2,3,2,0]=(-0.00948421972234)*x_ref[0]**o+(-0.680685462738)*x_ref[0]+(-0.767076278167)*x_ref[1]**o+(0.210165437284)*x_ref[1]
        ref[2,3,2,1]=(-0.455199698263)*x_ref[0]**o+(-0.763430401419)*x_ref[0]+(0.183773350212)*x_ref[1]**o+(0.43105940923)*x_ref[1]
        ref[2,4,0,0]=(-0.673962025988)*x_ref[0]**o+(-0.679073069725)*x_ref[0]+(0.11018431715)*x_ref[1]**o+(0.349020527482)*x_ref[1]
        ref[2,4,0,1]=(-0.876691282076)*x_ref[0]**o+(0.920902226631)*x_ref[0]+(-0.477417470246)*x_ref[1]**o+(0.836417982216)*x_ref[1]
        ref[2,4,1,0]=(0.567966851024)*x_ref[0]**o+(0.583241431716)*x_ref[0]+(-0.271851271008)*x_ref[1]**o+(0.678727902863)*x_ref[1]
        ref[2,4,1,1]=(0.707400094)*x_ref[0]**o+(0.802820856444)*x_ref[0]+(0.84518937616)*x_ref[1]**o+(-0.0267193533403)*x_ref[1]
        ref[2,4,2,0]=(0.213268920036)*x_ref[0]**o+(0.780721415328)*x_ref[0]+(0.839292948154)*x_ref[1]**o+(-0.0376334744443)*x_ref[1]
        ref[2,4,2,1]=(0.136255687031)*x_ref[0]**o+(-0.582766278693)*x_ref[0]+(0.257662521321)*x_ref[1]**o+(0.808337521979)*x_ref[1]
        ref[3,0,0,0]=(-0.524068770822)*x_ref[0]**o+(0.942236878842)*x_ref[0]+(0.889496255793)*x_ref[1]**o+(-0.904810742393)*x_ref[1]
        ref[3,0,0,1]=(0.780157465475)*x_ref[0]**o+(-0.763811668832)*x_ref[0]+(-0.517482070639)*x_ref[1]**o+(0.662770780676)*x_ref[1]
        ref[3,0,1,0]=(-0.693227564812)*x_ref[0]**o+(-0.763506211841)*x_ref[0]+(-0.427709881126)*x_ref[1]**o+(0.159109827204)*x_ref[1]
        ref[3,0,1,1]=(0.644975629084)*x_ref[0]**o+(-0.928112761899)*x_ref[0]+(-0.530772740083)*x_ref[1]**o+(-0.723174790087)*x_ref[1]
        ref[3,0,2,0]=(-0.923467274097)*x_ref[0]**o+(0.806128117288)*x_ref[0]+(0.177779363207)*x_ref[1]**o+(0.862243885542)*x_ref[1]
        ref[3,0,2,1]=(-0.0823252901358)*x_ref[0]**o+(0.289022916284)*x_ref[0]+(-0.518508771021)*x_ref[1]**o+(0.174564769845)*x_ref[1]
        ref[3,1,0,0]=(0.608837578444)*x_ref[0]**o+(0.898216758361)*x_ref[0]+(-0.844730109761)*x_ref[1]**o+(0.745974173207)*x_ref[1]
        ref[3,1,0,1]=(-0.11776411078)*x_ref[0]**o+(0.186256502356)*x_ref[0]+(0.565289400333)*x_ref[1]**o+(-0.776173716206)*x_ref[1]
        ref[3,1,1,0]=(-0.170736580152)*x_ref[0]**o+(0.829581661683)*x_ref[0]+(0.310135833859)*x_ref[1]**o+(-0.509629545687)*x_ref[1]
        ref[3,1,1,1]=(-0.225246122751)*x_ref[0]**o+(0.586754377294)*x_ref[0]+(-0.726064725635)*x_ref[1]**o+(0.136952605704)*x_ref[1]
        ref[3,1,2,0]=(-0.287932115478)*x_ref[0]**o+(0.858031396153)*x_ref[0]+(-0.493441745353)*x_ref[1]**o+(0.601827664994)*x_ref[1]
        ref[3,1,2,1]=(0.665503842862)*x_ref[0]**o+(-0.515998970452)*x_ref[0]+(-0.890055558624)*x_ref[1]**o+(-0.0446371451457)*x_ref[1]
        ref[3,2,0,0]=(-0.33741557585)*x_ref[0]**o+(-0.416183752333)*x_ref[0]+(-0.92430073538)*x_ref[1]**o+(-0.486265090904)*x_ref[1]
        ref[3,2,0,1]=(0.910860925994)*x_ref[0]**o+(-0.896869755389)*x_ref[0]+(0.0634658185324)*x_ref[1]**o+(-0.383136455739)*x_ref[1]
        ref[3,2,1,0]=(0.386839297976)*x_ref[0]**o+(-0.190659927071)*x_ref[0]+(-0.304846978185)*x_ref[1]**o+(-0.92705511899)*x_ref[1]
        ref[3,2,1,1]=(-0.0197812018683)*x_ref[0]**o+(-0.479083095775)*x_ref[0]+(0.502489976089)*x_ref[1]**o+(-0.859440665475)*x_ref[1]
        ref[3,2,2,0]=(-0.507672045299)*x_ref[0]**o+(0.498689676072)*x_ref[0]+(-0.719735804645)*x_ref[1]**o+(-0.522053306674)*x_ref[1]
        ref[3,2,2,1]=(0.46207629211)*x_ref[0]**o+(-0.496697066039)*x_ref[0]+(0.994069339904)*x_ref[1]**o+(0.873222754449)*x_ref[1]
        ref[3,3,0,0]=(0.126410029692)*x_ref[0]**o+(0.51067050196)*x_ref[0]+(-0.978384824088)*x_ref[1]**o+(-0.68606930503)*x_ref[1]
        ref[3,3,0,1]=(0.376888157008)*x_ref[0]**o+(-0.14015764025)*x_ref[0]+(0.523667700758)*x_ref[1]**o+(0.0277726855258)*x_ref[1]
        ref[3,3,1,0]=(-0.105865233294)*x_ref[0]**o+(-0.0515230741203)*x_ref[0]+(0.203766525036)*x_ref[1]**o+(-0.0431893014387)*x_ref[1]
        ref[3,3,1,1]=(-0.404351448486)*x_ref[0]**o+(-0.639161429157)*x_ref[0]+(-0.957341091708)*x_ref[1]**o+(-0.103346466984)*x_ref[1]
        ref[3,3,2,0]=(-0.0523508780623)*x_ref[0]**o+(-0.563589575544)*x_ref[0]+(0.158487727325)*x_ref[1]**o+(-0.519455492178)*x_ref[1]
        ref[3,3,2,1]=(-0.823286342218)*x_ref[0]**o+(-0.493663499535)*x_ref[0]+(-0.409825574064)*x_ref[1]**o+(-0.0406101279806)*x_ref[1]
        ref[3,4,0,0]=(0.926212910387)*x_ref[0]**o+(-0.0944507963)*x_ref[0]+(0.640740499391)*x_ref[1]**o+(-0.106969076047)*x_ref[1]
        ref[3,4,0,1]=(-0.912044993941)*x_ref[0]**o+(-0.495323777918)*x_ref[0]+(-0.595831845108)*x_ref[1]**o+(-0.313723426318)*x_ref[1]
        ref[3,4,1,0]=(-0.0730898742548)*x_ref[0]**o+(0.0524008484492)*x_ref[0]+(-0.948232935497)*x_ref[1]**o+(0.952670824572)*x_ref[1]
        ref[3,4,1,1]=(-0.81869438138)*x_ref[0]**o+(-0.894654266952)*x_ref[0]+(0.414078621872)*x_ref[1]**o+(-0.876690817847)*x_ref[1]
        ref[3,4,2,0]=(0.00688155078441)*x_ref[0]**o+(0.845964586551)*x_ref[0]+(0.0183149371892)*x_ref[1]**o+(-0.882931394537)*x_ref[1]
        ref[3,4,2,1]=(-0.734830097149)*x_ref[0]**o+(-0.37824588859)*x_ref[0]+(-0.815402620311)*x_ref[1]**o+(-0.357905685517)*x_ref[1]
      else:
        arg[0,0,0,0]=(-0.110519315354)*x[0]**o+(0.904227374292)*x[0]+(0.459005710707)*x[1]**o+(-0.980231422256)*x[1]+(-0.819713355475)*x[2]**o+(-0.93459517989)*x[2]
        arg[0,0,0,1]=(-0.326902946413)*x[0]**o+(0.489327766219)*x[0]+(-0.965308741061)*x[1]**o+(0.0351399111471)*x[1]+(0.257760096595)*x[2]**o+(0.125123023244)*x[2]
        arg[0,0,1,0]=(-0.584360710004)*x[0]**o+(0.681892732544)*x[0]+(-0.118257741614)*x[1]**o+(0.839875738892)*x[1]+(0.107934154241)*x[2]**o+(0.279757054665)*x[2]
        arg[0,0,1,1]=(0.90973214794)*x[0]**o+(-0.776804618826)*x[0]+(-0.910633853381)*x[1]**o+(-0.25678788529)*x[1]+(0.783709363646)*x[2]**o+(-0.922660855915)*x[2]
        arg[0,0,2,0]=(-0.802273263394)*x[0]**o+(-0.917749665952)*x[0]+(0.796206817226)*x[1]**o+(0.917284637885)*x[1]+(0.219841508893)*x[2]**o+(0.0655848200083)*x[2]
        arg[0,0,2,1]=(0.550596792343)*x[0]**o+(-0.760207057617)*x[0]+(0.0368469683098)*x[1]**o+(0.509347717331)*x[1]+(0.813780778487)*x[2]**o+(-0.432656298451)*x[2]
        arg[0,1,0,0]=(-0.238502042941)*x[0]**o+(-0.966632221032)*x[0]+(-0.373745652933)*x[1]**o+(-0.436176336389)*x[1]+(0.124280197699)*x[2]**o+(0.552700562803)*x[2]
        arg[0,1,0,1]=(-0.19848053276)*x[0]**o+(0.110167634957)*x[0]+(-0.748553857473)*x[1]**o+(0.355932508959)*x[1]+(-0.226987914753)*x[2]**o+(-0.68870238197)*x[2]
        arg[0,1,1,0]=(0.365822518882)*x[0]**o+(0.327285102897)*x[0]+(0.549212504635)*x[1]**o+(0.918179946504)*x[1]+(0.20644004679)*x[2]**o+(0.821299409756)*x[2]
        arg[0,1,1,1]=(-0.366167489101)*x[0]**o+(0.529933186572)*x[0]+(-0.604965310967)*x[1]**o+(-0.833093092352)*x[1]+(-0.73593913912)*x[2]**o+(0.872325614453)*x[2]
        arg[0,1,2,0]=(0.289123646902)*x[0]**o+(0.705862153789)*x[0]+(-0.319756886822)*x[1]**o+(-0.453220232524)*x[1]+(-0.192336374167)*x[2]**o+(0.363575495006)*x[2]
        arg[0,1,2,1]=(-0.213014833353)*x[0]**o+(-0.0706784230627)*x[0]+(-0.941272982229)*x[1]**o+(-0.96641361036)*x[1]+(0.289919014964)*x[2]**o+(0.57658772066)*x[2]
        arg[0,2,0,0]=(-0.406683991704)*x[0]**o+(-0.0110042213592)*x[0]+(-0.273148083258)*x[1]**o+(0.054257030447)*x[1]+(0.387598147681)*x[2]**o+(0.480398109069)*x[2]
        arg[0,2,0,1]=(0.318713057562)*x[0]**o+(-0.449467749585)*x[0]+(0.669991267829)*x[1]**o+(0.237986860249)*x[1]+(0.894370523471)*x[2]**o+(-0.397832436233)*x[2]
        arg[0,2,1,0]=(-0.858603253642)*x[0]**o+(-0.713122015919)*x[0]+(0.870251906188)*x[1]**o+(-0.781601254578)*x[1]+(-0.504531525822)*x[2]**o+(-0.699819349556)*x[2]
        arg[0,2,1,1]=(-0.825591116361)*x[0]**o+(-0.552452876394)*x[0]+(-0.262627261618)*x[1]**o+(-0.147127256184)*x[1]+(-0.171485665103)*x[2]**o+(0.823279741501)*x[2]
        arg[0,2,2,0]=(-0.0416279689068)*x[0]**o+(-0.0903338000001)*x[0]+(0.911673780635)*x[1]**o+(0.86298673049)*x[1]+(-0.60251179847)*x[2]**o+(0.378303877746)*x[2]
        arg[0,2,2,1]=(0.261384106387)*x[0]**o+(0.0326416199811)*x[0]+(-0.426740102269)*x[1]**o+(-0.445401626921)*x[1]+(-0.388668254526)*x[2]**o+(-0.977942913284)*x[2]
        arg[0,3,0,0]=(-0.0195053813483)*x[0]**o+(-0.294061998664)*x[0]+(0.790461098426)*x[1]**o+(-0.506334268838)*x[1]+(0.394015541605)*x[2]**o+(-0.562248105432)*x[2]
        arg[0,3,0,1]=(0.390726516735)*x[0]**o+(-0.757597437859)*x[0]+(0.861882149861)*x[1]**o+(0.621870451314)*x[1]+(0.900777666162)*x[2]**o+(0.271507131938)*x[2]
        arg[0,3,1,0]=(-0.725635966475)*x[0]**o+(-0.695837548737)*x[0]+(-0.163078176328)*x[1]**o+(-0.0435808418729)*x[1]+(0.0890756544611)*x[2]**o+(-0.270422627949)*x[2]
        arg[0,3,1,1]=(-0.175498720647)*x[0]**o+(-0.213605441632)*x[0]+(-0.358632620362)*x[1]**o+(-0.193273004152)*x[1]+(0.937704823805)*x[2]**o+(0.861707832291)*x[2]
        arg[0,3,2,0]=(0.0976010265611)*x[0]**o+(0.995135925341)*x[0]+(0.755339287387)*x[1]**o+(-0.924722068193)*x[1]+(-0.480212181904)*x[2]**o+(-0.40237760439)*x[2]
        arg[0,3,2,1]=(-0.679215392952)*x[0]**o+(-0.00796096253297)*x[0]+(-0.964972599421)*x[1]**o+(-0.410415435656)*x[1]+(-0.292638067785)*x[2]**o+(-0.950383976561)*x[2]
        arg[0,4,0,0]=(0.245586594928)*x[0]**o+(-0.593420401896)*x[0]+(0.553450486534)*x[1]**o+(0.173814965689)*x[1]+(0.0719392610563)*x[2]**o+(0.676231437486)*x[2]
        arg[0,4,0,1]=(0.154382756425)*x[0]**o+(-0.0655740293432)*x[0]+(0.924689745142)*x[1]**o+(-0.246876531175)*x[1]+(-0.241460713543)*x[2]**o+(-0.489840108568)*x[2]
        arg[0,4,1,0]=(-0.482705840359)*x[0]**o+(-0.392960380114)*x[0]+(-0.503463823972)*x[1]**o+(0.317163204738)*x[1]+(0.775048877356)*x[2]**o+(0.599889732326)*x[2]
        arg[0,4,1,1]=(-0.122555861716)*x[0]**o+(0.930175770449)*x[0]+(0.483503764941)*x[1]**o+(0.340001033217)*x[1]+(0.672924280979)*x[2]**o+(0.746489359683)*x[2]
        arg[0,4,2,0]=(-0.339421141033)*x[0]**o+(-0.604966575169)*x[0]+(0.862413017648)*x[1]**o+(0.936752078454)*x[1]+(-0.0549628353285)*x[2]**o+(-0.0858885835314)*x[2]
        arg[0,4,2,1]=(-0.581767357421)*x[0]**o+(-0.644286027548)*x[0]+(0.76948396865)*x[1]**o+(0.692265904744)*x[1]+(0.955871091531)*x[2]**o+(-0.378888503457)*x[2]
        arg[1,0,0,0]=(-0.0407846595046)*x[0]**o+(-0.293709943251)*x[0]+(-0.310552931582)*x[1]**o+(-0.696244178466)*x[1]+(0.962985284252)*x[2]**o+(-0.0266457621404)*x[2]
        arg[1,0,0,1]=(-0.480942811584)*x[0]**o+(-0.400158759679)*x[0]+(0.311171869807)*x[1]**o+(-0.79518809427)*x[1]+(-0.497004450478)*x[2]**o+(-0.639822005752)*x[2]
        arg[1,0,1,0]=(0.158738648483)*x[0]**o+(0.274859816468)*x[0]+(-0.208677612654)*x[1]**o+(0.911107598366)*x[1]+(-0.400163007778)*x[2]**o+(0.276296890267)*x[2]
        arg[1,0,1,1]=(0.613312276864)*x[0]**o+(-0.644237218776)*x[0]+(0.81927272895)*x[1]**o+(-0.12971860556)*x[1]+(0.214379406799)*x[2]**o+(0.856335675043)*x[2]
        arg[1,0,2,0]=(-0.699925173766)*x[0]**o+(-0.560613547952)*x[0]+(-0.355325107637)*x[1]**o+(-0.598747822041)*x[1]+(0.243087037553)*x[2]**o+(-0.484176993015)*x[2]
        arg[1,0,2,1]=(-0.218912300848)*x[0]**o+(-0.259483312969)*x[0]+(-0.863788186295)*x[1]**o+(0.924423774757)*x[1]+(-0.0823752192204)*x[2]**o+(-0.928377900602)*x[2]
        arg[1,1,0,0]=(-0.368901144425)*x[0]**o+(0.544104655592)*x[0]+(0.321295151176)*x[1]**o+(0.395692754134)*x[1]+(-0.721319111754)*x[2]**o+(-0.698518442137)*x[2]
        arg[1,1,0,1]=(-0.592867336047)*x[0]**o+(0.145120231038)*x[0]+(0.110390771366)*x[1]**o+(0.168333368086)*x[1]+(-0.577866419921)*x[2]**o+(0.705027708469)*x[2]
        arg[1,1,1,0]=(-0.210603428235)*x[0]**o+(-0.250156447116)*x[0]+(-0.397803248976)*x[1]**o+(0.332163380867)*x[1]+(-0.250648216541)*x[2]**o+(-0.0693729055697)*x[2]
        arg[1,1,1,1]=(0.324927558096)*x[0]**o+(-0.262726116223)*x[0]+(-0.555439758488)*x[1]**o+(-0.865019248609)*x[1]+(-0.0971907425857)*x[2]**o+(-0.742359236711)*x[2]
        arg[1,1,2,0]=(0.161058869414)*x[0]**o+(0.349191500791)*x[0]+(0.468336473315)*x[1]**o+(-0.856364775995)*x[1]+(0.491736696689)*x[2]**o+(0.352984104173)*x[2]
        arg[1,1,2,1]=(-0.243275976508)*x[0]**o+(-0.785605388878)*x[0]+(-0.282478919296)*x[1]**o+(-0.728851522341)*x[1]+(0.375134848237)*x[2]**o+(-0.486200561256)*x[2]
        arg[1,2,0,0]=(-0.172241451405)*x[0]**o+(0.422361498196)*x[0]+(0.464862755164)*x[1]**o+(0.466661717105)*x[1]+(-0.886209677512)*x[2]**o+(-0.43529076789)*x[2]
        arg[1,2,0,1]=(0.946582222405)*x[0]**o+(0.513640973894)*x[0]+(-0.923788000633)*x[1]**o+(-0.358163567195)*x[1]+(-0.709999132129)*x[2]**o+(-0.761163299694)*x[2]
        arg[1,2,1,0]=(0.236339981889)*x[0]**o+(0.259874866134)*x[0]+(-0.32460762691)*x[1]**o+(-0.629786950313)*x[1]+(-0.679261685402)*x[2]**o+(0.593069192918)*x[2]
        arg[1,2,1,1]=(0.11006800009)*x[0]**o+(0.970532991787)*x[0]+(-0.467608188361)*x[1]**o+(-0.959281984552)*x[1]+(0.610396622519)*x[2]**o+(0.282468600058)*x[2]
        arg[1,2,2,0]=(-0.656419157742)*x[0]**o+(0.234027308171)*x[0]+(-0.67027206978)*x[1]**o+(-0.746034436817)*x[1]+(-0.771894073652)*x[2]**o+(0.949545830287)*x[2]
        arg[1,2,2,1]=(0.615884236811)*x[0]**o+(-0.670156612555)*x[0]+(0.543165493345)*x[1]**o+(-0.618294277594)*x[1]+(-0.974114255061)*x[2]**o+(-0.762320542298)*x[2]
        arg[1,3,0,0]=(0.096201957648)*x[0]**o+(0.440870816208)*x[0]+(0.124448859607)*x[1]**o+(0.507919407561)*x[1]+(-0.401666263687)*x[2]**o+(-0.437653713723)*x[2]
        arg[1,3,0,1]=(0.165129229294)*x[0]**o+(-0.353217428955)*x[0]+(0.473666065504)*x[1]**o+(0.907667166988)*x[1]+(-0.864298533288)*x[2]**o+(0.641505117281)*x[2]
        arg[1,3,1,0]=(0.381900624425)*x[0]**o+(0.427520178597)*x[0]+(-0.560279540293)*x[1]**o+(0.190814532271)*x[1]+(-0.187947688629)*x[2]**o+(-0.127694148593)*x[2]
        arg[1,3,1,1]=(-0.903921363895)*x[0]**o+(-0.659942040828)*x[0]+(-0.549460213492)*x[1]**o+(0.869909651895)*x[1]+(-0.962273439484)*x[2]**o+(-0.046365857751)*x[2]
        arg[1,3,2,0]=(-0.021688482187)*x[0]**o+(-0.209531000717)*x[0]+(0.668564395133)*x[1]**o+(-0.0347978700975)*x[1]+(0.947677147488)*x[2]**o+(-0.0152385052475)*x[2]
        arg[1,3,2,1]=(0.880839485817)*x[0]**o+(-0.319275929312)*x[0]+(-0.940056657485)*x[1]**o+(0.570855397726)*x[1]+(0.279710812074)*x[2]**o+(0.496622215051)*x[2]
        arg[1,4,0,0]=(-0.139033645615)*x[0]**o+(-0.271453722366)*x[0]+(-0.268516720244)*x[1]**o+(-0.0467555017438)*x[1]+(-0.41618262265)*x[2]**o+(0.315320363813)*x[2]
        arg[1,4,0,1]=(0.296108607204)*x[0]**o+(-0.184582165661)*x[0]+(0.121416543744)*x[1]**o+(-0.922284926751)*x[1]+(-0.437600089668)*x[2]**o+(-0.608800626513)*x[2]
        arg[1,4,1,0]=(0.8942028423)*x[0]**o+(0.673402831188)*x[0]+(-0.622529195562)*x[1]**o+(-0.266052338798)*x[1]+(-0.273167851702)*x[2]**o+(-0.312211982747)*x[2]
        arg[1,4,1,1]=(-0.102315711178)*x[0]**o+(-0.885515719371)*x[0]+(-0.532910101204)*x[1]**o+(0.798525944348)*x[1]+(0.640586802303)*x[2]**o+(0.36308806536)*x[2]
        arg[1,4,2,0]=(0.376793506541)*x[0]**o+(-0.667063564875)*x[0]+(0.609591985816)*x[1]**o+(0.28347745142)*x[1]+(-0.751832936302)*x[2]**o+(-0.683738856965)*x[2]
        arg[1,4,2,1]=(-0.51412252819)*x[0]**o+(0.559560297206)*x[0]+(0.751134603429)*x[1]**o+(-0.0031812310797)*x[1]+(-0.525776122178)*x[2]**o+(0.338471942848)*x[2]
        arg[2,0,0,0]=(0.937572244905)*x[0]**o+(-0.293020870379)*x[0]+(-0.918591520817)*x[1]**o+(0.622194477261)*x[1]+(0.31641482851)*x[2]**o+(0.276912266864)*x[2]
        arg[2,0,0,1]=(0.853041599998)*x[0]**o+(-0.0220623724171)*x[0]+(0.89909044296)*x[1]**o+(-0.838782642732)*x[1]+(0.0133197592042)*x[2]**o+(-0.868977618993)*x[2]
        arg[2,0,1,0]=(-0.325799042976)*x[0]**o+(0.746269971994)*x[0]+(0.882671061757)*x[1]**o+(-0.284133373252)*x[1]+(0.120086549521)*x[2]**o+(-0.397352610969)*x[2]
        arg[2,0,1,1]=(0.282336934951)*x[0]**o+(-0.339893978547)*x[0]+(-0.0861660125386)*x[1]**o+(0.283063579798)*x[1]+(0.714762478569)*x[2]**o+(-0.497021718549)*x[2]
        arg[2,0,2,0]=(0.891782779491)*x[0]**o+(-0.817647577644)*x[0]+(0.364088142085)*x[1]**o+(-0.950577637048)*x[1]+(0.4070663557)*x[2]**o+(0.270753179246)*x[2]
        arg[2,0,2,1]=(0.316594913808)*x[0]**o+(0.310731486851)*x[0]+(0.250421828765)*x[1]**o+(0.772786141458)*x[1]+(0.64843040679)*x[2]**o+(-0.691903332511)*x[2]
        arg[2,1,0,0]=(0.87180715028)*x[0]**o+(-0.953173230375)*x[0]+(-0.823516344548)*x[1]**o+(0.934653756548)*x[1]+(-0.537124962489)*x[2]**o+(-0.233365939659)*x[2]
        arg[2,1,0,1]=(-0.407189744631)*x[0]**o+(-0.754425720288)*x[0]+(0.737526767897)*x[1]**o+(0.745393943502)*x[1]+(0.811744293685)*x[2]**o+(-0.459556659177)*x[2]
        arg[2,1,1,0]=(-0.693529259877)*x[0]**o+(0.580197938048)*x[0]+(0.506055440405)*x[1]**o+(0.122531014759)*x[1]+(0.427736137775)*x[2]**o+(-0.83260140401)*x[2]
        arg[2,1,1,1]=(0.542621975026)*x[0]**o+(-0.568425713106)*x[0]+(0.303002043446)*x[1]**o+(0.363286261651)*x[1]+(0.2729476289)*x[2]**o+(-0.0113629744659)*x[2]
        arg[2,1,2,0]=(-0.266750235694)*x[0]**o+(0.10116400306)*x[0]+(0.315977962897)*x[1]**o+(-0.406236201454)*x[1]+(-0.586845072374)*x[2]**o+(0.513937047282)*x[2]
        arg[2,1,2,1]=(-0.518780811975)*x[0]**o+(0.275053640829)*x[0]+(0.4234970646)*x[1]**o+(0.389585163376)*x[1]+(-0.609771068778)*x[2]**o+(-0.121467061755)*x[2]
        arg[2,2,0,0]=(0.138823207466)*x[0]**o+(-0.779837455014)*x[0]+(0.70167049555)*x[1]**o+(0.766428813369)*x[1]+(0.114023530366)*x[2]**o+(0.650098536187)*x[2]
        arg[2,2,0,1]=(0.974998667633)*x[0]**o+(0.958471050654)*x[0]+(0.193515162503)*x[1]**o+(-0.252148167253)*x[1]+(0.495475460125)*x[2]**o+(0.562693347983)*x[2]
        arg[2,2,1,0]=(-0.552379548988)*x[0]**o+(-0.831399971618)*x[0]+(0.437581216647)*x[1]**o+(-0.775331267696)*x[1]+(0.729041628844)*x[2]**o+(0.633113359893)*x[2]
        arg[2,2,1,1]=(0.379858437049)*x[0]**o+(0.923586504203)*x[0]+(0.707244783969)*x[1]**o+(-0.293157375059)*x[1]+(-0.368717941876)*x[2]**o+(-0.202610288616)*x[2]
        arg[2,2,2,0]=(-0.804826108231)*x[0]**o+(-0.0208712096312)*x[0]+(0.0482547072498)*x[1]**o+(0.852366543609)*x[1]+(0.328962500136)*x[2]**o+(-0.132777835512)*x[2]
        arg[2,2,2,1]=(0.91373752217)*x[0]**o+(-0.792112120729)*x[0]+(-0.0457564087072)*x[1]**o+(0.309694852949)*x[1]+(0.089219285337)*x[2]**o+(-0.516000514614)*x[2]
        arg[2,3,0,0]=(0.343215129115)*x[0]**o+(0.40804099837)*x[0]+(-0.865942442914)*x[1]**o+(0.842241683043)*x[1]+(0.0727803297601)*x[2]**o+(-0.395225682949)*x[2]
        arg[2,3,0,1]=(-0.965340522771)*x[0]**o+(-0.897056478677)*x[0]+(-0.305205944062)*x[1]**o+(0.393941789685)*x[1]+(-0.253328379131)*x[2]**o+(0.0951510469271)*x[2]
        arg[2,3,1,0]=(0.536251514752)*x[0]**o+(0.0629776386779)*x[0]+(-0.706311222379)*x[1]**o+(-0.84362242029)*x[1]+(0.00110678760539)*x[2]**o+(-0.30286059941)*x[2]
        arg[2,3,1,1]=(0.114584737295)*x[0]**o+(0.0708763738067)*x[0]+(-0.454760062397)*x[1]**o+(0.228742539922)*x[1]+(0.514960810352)*x[2]**o+(-0.815749211085)*x[2]
        arg[2,3,2,0]=(0.246335688008)*x[0]**o+(-0.239199299885)*x[0]+(-0.447390151659)*x[1]**o+(0.918949356391)*x[1]+(-0.307864840175)*x[2]**o+(0.560139080481)*x[2]
        arg[2,3,2,1]=(0.118766035723)*x[0]**o+(0.610711164899)*x[0]+(0.336628771776)*x[1]**o+(0.833699268751)*x[1]+(-0.424694647429)*x[2]**o+(-0.639929688104)*x[2]
        arg[2,4,0,0]=(0.814543506987)*x[0]**o+(-0.797684321368)*x[0]+(0.337242635783)*x[1]**o+(0.278139837072)*x[1]+(0.633511526441)*x[2]**o+(0.372175615028)*x[2]
        arg[2,4,0,1]=(-0.0590668694094)*x[0]**o+(0.256732067267)*x[0]+(0.519737187577)*x[1]**o+(-0.376391855948)*x[1]+(-0.810934658651)*x[2]**o+(0.826899476928)*x[2]
        arg[2,4,1,0]=(-0.428976843661)*x[0]**o+(0.672667833313)*x[0]+(0.263194890946)*x[1]**o+(-0.894790773661)*x[1]+(-0.550298557295)*x[2]**o+(-0.00560384035502)*x[2]
        arg[2,4,1,1]=(-0.818372430144)*x[0]**o+(-0.829920833073)*x[0]+(-0.359560636119)*x[1]**o+(-0.113484563205)*x[1]+(0.586796110365)*x[2]**o+(0.296820898426)*x[2]
        arg[2,4,2,0]=(-0.691597969566)*x[0]**o+(0.715813153538)*x[0]+(0.954093612503)*x[1]**o+(0.88803420754)*x[1]+(0.697447667765)*x[2]**o+(0.793730496625)*x[2]
        arg[2,4,2,1]=(0.351099833336)*x[0]**o+(-0.716419272807)*x[0]+(0.318726674998)*x[1]**o+(0.246151914414)*x[1]+(-0.945520673884)*x[2]**o+(-0.489026233954)*x[2]
        arg[3,0,0,0]=(0.511598022804)*x[0]**o+(0.567617269582)*x[0]+(-0.526293319137)*x[1]**o+(0.333750098146)*x[1]+(-0.0869228849144)*x[2]**o+(-0.204072834867)*x[2]
        arg[3,0,0,1]=(0.905384007172)*x[0]**o+(-0.661067390314)*x[0]+(-0.470790632295)*x[1]**o+(-0.149608017616)*x[1]+(0.427879278267)*x[2]**o+(0.987040179184)*x[2]
        arg[3,0,1,0]=(-0.946562006809)*x[0]**o+(0.64141909633)*x[0]+(-0.762076826276)*x[1]**o+(-0.642912850794)*x[1]+(0.712567898559)*x[2]**o+(-0.292128944205)*x[2]
        arg[3,0,1,1]=(-0.955087949361)*x[0]**o+(0.426588041133)*x[0]+(0.339313104127)*x[1]**o+(-0.375147344045)*x[1]+(0.967259014679)*x[2]**o+(0.504521081858)*x[2]
        arg[3,0,2,0]=(0.141876598344)*x[0]**o+(-0.335579252825)*x[0]+(0.514552993602)*x[1]**o+(-0.531116948499)*x[1]+(-0.369319413141)*x[2]**o+(0.0374413262362)*x[2]
        arg[3,0,2,1]=(0.678360463658)*x[0]**o+(-0.357900119646)*x[0]+(0.300750717059)*x[1]**o+(0.516196189448)*x[1]+(0.0535466018203)*x[2]**o+(-0.149961940786)*x[2]
        arg[3,1,0,0]=(0.0975994351079)*x[0]**o+(0.634544068499)*x[0]+(-0.963217046352)*x[1]**o+(-0.804658091456)*x[1]+(-0.620132720823)*x[2]**o+(0.862250619369)*x[2]
        arg[3,1,0,1]=(0.210306328879)*x[0]**o+(-0.457341187966)*x[0]+(0.957556084801)*x[1]**o+(0.704073170928)*x[1]+(0.423134492133)*x[2]**o+(-0.167268375412)*x[2]
        arg[3,1,1,0]=(-0.909228817744)*x[0]**o+(0.110665018921)*x[0]+(0.36275630147)*x[1]**o+(0.346866824763)*x[1]+(-0.535600547489)*x[2]**o+(0.627638814375)*x[2]
        arg[3,1,1,1]=(0.328780933823)*x[0]**o+(-0.249563994161)*x[0]+(-0.555789062892)*x[1]**o+(-0.917287525036)*x[1]+(0.386191175017)*x[2]**o+(0.123904319061)*x[2]
        arg[3,1,2,0]=(-0.838409901623)*x[0]**o+(0.673479705393)*x[0]+(0.482921104917)*x[1]**o+(0.136411388588)*x[1]+(0.583674116225)*x[2]**o+(0.305100836877)*x[2]
        arg[3,1,2,1]=(0.952961008312)*x[0]**o+(-0.0501981079532)*x[0]+(0.661434464642)*x[1]**o+(0.65030469399)*x[1]+(-0.157562001295)*x[2]**o+(-0.249551892917)*x[2]
        arg[3,2,0,0]=(-0.0232804500286)*x[0]**o+(-0.559229392203)*x[0]+(0.127967930273)*x[1]**o+(-0.635827181645)*x[1]+(-0.0426703224383)*x[2]**o+(0.47720230258)*x[2]
        arg[3,2,0,1]=(-0.421658996106)*x[0]**o+(-0.468260038077)*x[0]+(0.479083572554)*x[1]**o+(-0.214417881841)*x[1]+(0.372430534261)*x[2]**o+(0.133063789546)*x[2]
        arg[3,2,1,0]=(0.905811586567)*x[0]**o+(0.245447340384)*x[0]+(-0.337650999146)*x[1]**o+(0.908188235976)*x[1]+(-0.804029751277)*x[2]**o+(0.0512676456643)*x[2]
        arg[3,2,1,1]=(-0.891486839409)*x[0]**o+(0.20753794329)*x[0]+(0.879183464252)*x[1]**o+(0.902028906992)*x[1]+(0.416776813598)*x[2]**o+(0.491964374373)*x[2]
        arg[3,2,2,0]=(0.653067027119)*x[0]**o+(0.765293890745)*x[0]+(-0.348902169508)*x[1]**o+(0.324429035849)*x[1]+(-0.363026365065)*x[2]**o+(0.0816186405511)*x[2]
        arg[3,2,2,1]=(-0.0415375052325)*x[0]**o+(-0.480779075784)*x[0]+(-0.137223552938)*x[1]**o+(-0.915733625512)*x[1]+(0.750199445159)*x[2]**o+(-0.789818945171)*x[2]
        arg[3,3,0,0]=(0.0744921239083)*x[0]**o+(0.0700589144159)*x[0]+(-0.327105425297)*x[1]**o+(0.469843257765)*x[1]+(-0.170191600844)*x[2]**o+(0.732071534155)*x[2]
        arg[3,3,0,1]=(0.172643927441)*x[0]**o+(0.848924114383)*x[0]+(-0.79125762228)*x[1]**o+(-0.694725775976)*x[1]+(-0.309710503985)*x[2]**o+(-0.405310234927)*x[2]
        arg[3,3,1,0]=(-0.465911787595)*x[0]**o+(-0.71080474709)*x[0]+(0.959925934203)*x[1]**o+(-0.111110644825)*x[1]+(-0.145349925218)*x[2]**o+(0.122955165811)*x[2]
        arg[3,3,1,1]=(-0.769111863754)*x[0]**o+(0.348036938532)*x[0]+(0.485890856474)*x[1]**o+(0.382595218985)*x[1]+(-0.897488692981)*x[2]**o+(0.830822333819)*x[2]
        arg[3,3,2,0]=(0.341890793681)*x[0]**o+(-0.667145936571)*x[0]+(-0.145094108268)*x[1]**o+(0.771433911212)*x[1]+(-0.299752164325)*x[2]**o+(-0.357856549677)*x[2]
        arg[3,3,2,1]=(0.26381838796)*x[0]**o+(-0.44474724998)*x[0]+(0.310283988195)*x[1]**o+(0.698675181314)*x[1]+(0.437624133238)*x[2]**o+(0.394916054445)*x[2]
        arg[3,4,0,0]=(-0.576499406674)*x[0]**o+(0.770936634959)*x[0]+(0.707441735783)*x[1]**o+(-0.794953257043)*x[1]+(-0.535342943209)*x[2]**o+(-0.509446811787)*x[2]
        arg[3,4,0,1]=(0.161115012113)*x[0]**o+(0.11079092252)*x[0]+(0.0633804181164)*x[1]**o+(-0.406049293019)*x[1]+(-0.824283994336)*x[2]**o+(-0.481107560499)*x[2]
        arg[3,4,1,0]=(-0.411133810295)*x[0]**o+(0.988486625099)*x[0]+(-0.36050191367)*x[1]**o+(0.469479821965)*x[1]+(-0.431175357325)*x[2]**o+(-0.966295015263)*x[2]
        arg[3,4,1,1]=(0.233731410367)*x[0]**o+(-0.821900740766)*x[0]+(-0.823123461385)*x[1]**o+(-0.380537914421)*x[1]+(-0.896146300355)*x[2]**o+(-0.775172990257)*x[2]
        arg[3,4,2,0]=(-0.96323363586)*x[0]**o+(0.0935412405245)*x[0]+(0.929333026259)*x[1]**o+(-0.322236627348)*x[1]+(-0.390898135964)*x[2]**o+(-0.766178290162)*x[2]
        arg[3,4,2,1]=(-0.773670533268)*x[0]**o+(-0.156975157098)*x[0]+(-0.91512996795)*x[1]**o+(0.0247803682126)*x[1]+(0.995159733782)*x[2]**o+(-0.669224144225)*x[2]
        ref[0,0,0,0]=(-0.110519315354)*x_ref[0]**o+(0.904227374292)*x_ref[0]+(0.459005710707)*x_ref[1]**o+(-0.980231422256)*x_ref[1]+(-0.819713355475)*x_ref[2]**o+(-0.93459517989)*x_ref[2]
        ref[0,0,0,1]=(-0.326902946413)*x_ref[0]**o+(0.489327766219)*x_ref[0]+(-0.965308741061)*x_ref[1]**o+(0.0351399111471)*x_ref[1]+(0.257760096595)*x_ref[2]**o+(0.125123023244)*x_ref[2]
        ref[0,0,1,0]=(-0.584360710004)*x_ref[0]**o+(0.681892732544)*x_ref[0]+(-0.118257741614)*x_ref[1]**o+(0.839875738892)*x_ref[1]+(0.107934154241)*x_ref[2]**o+(0.279757054665)*x_ref[2]
        ref[0,0,1,1]=(0.90973214794)*x_ref[0]**o+(-0.776804618826)*x_ref[0]+(-0.910633853381)*x_ref[1]**o+(-0.25678788529)*x_ref[1]+(0.783709363646)*x_ref[2]**o+(-0.922660855915)*x_ref[2]
        ref[0,0,2,0]=(-0.802273263394)*x_ref[0]**o+(-0.917749665952)*x_ref[0]+(0.796206817226)*x_ref[1]**o+(0.917284637885)*x_ref[1]+(0.219841508893)*x_ref[2]**o+(0.0655848200083)*x_ref[2]
        ref[0,0,2,1]=(0.550596792343)*x_ref[0]**o+(-0.760207057617)*x_ref[0]+(0.0368469683098)*x_ref[1]**o+(0.509347717331)*x_ref[1]+(0.813780778487)*x_ref[2]**o+(-0.432656298451)*x_ref[2]
        ref[0,1,0,0]=(-0.238502042941)*x_ref[0]**o+(-0.966632221032)*x_ref[0]+(-0.373745652933)*x_ref[1]**o+(-0.436176336389)*x_ref[1]+(0.124280197699)*x_ref[2]**o+(0.552700562803)*x_ref[2]
        ref[0,1,0,1]=(-0.19848053276)*x_ref[0]**o+(0.110167634957)*x_ref[0]+(-0.748553857473)*x_ref[1]**o+(0.355932508959)*x_ref[1]+(-0.226987914753)*x_ref[2]**o+(-0.68870238197)*x_ref[2]
        ref[0,1,1,0]=(0.365822518882)*x_ref[0]**o+(0.327285102897)*x_ref[0]+(0.549212504635)*x_ref[1]**o+(0.918179946504)*x_ref[1]+(0.20644004679)*x_ref[2]**o+(0.821299409756)*x_ref[2]
        ref[0,1,1,1]=(-0.366167489101)*x_ref[0]**o+(0.529933186572)*x_ref[0]+(-0.604965310967)*x_ref[1]**o+(-0.833093092352)*x_ref[1]+(-0.73593913912)*x_ref[2]**o+(0.872325614453)*x_ref[2]
        ref[0,1,2,0]=(0.289123646902)*x_ref[0]**o+(0.705862153789)*x_ref[0]+(-0.319756886822)*x_ref[1]**o+(-0.453220232524)*x_ref[1]+(-0.192336374167)*x_ref[2]**o+(0.363575495006)*x_ref[2]
        ref[0,1,2,1]=(-0.213014833353)*x_ref[0]**o+(-0.0706784230627)*x_ref[0]+(-0.941272982229)*x_ref[1]**o+(-0.96641361036)*x_ref[1]+(0.289919014964)*x_ref[2]**o+(0.57658772066)*x_ref[2]
        ref[0,2,0,0]=(-0.406683991704)*x_ref[0]**o+(-0.0110042213592)*x_ref[0]+(-0.273148083258)*x_ref[1]**o+(0.054257030447)*x_ref[1]+(0.387598147681)*x_ref[2]**o+(0.480398109069)*x_ref[2]
        ref[0,2,0,1]=(0.318713057562)*x_ref[0]**o+(-0.449467749585)*x_ref[0]+(0.669991267829)*x_ref[1]**o+(0.237986860249)*x_ref[1]+(0.894370523471)*x_ref[2]**o+(-0.397832436233)*x_ref[2]
        ref[0,2,1,0]=(-0.858603253642)*x_ref[0]**o+(-0.713122015919)*x_ref[0]+(0.870251906188)*x_ref[1]**o+(-0.781601254578)*x_ref[1]+(-0.504531525822)*x_ref[2]**o+(-0.699819349556)*x_ref[2]
        ref[0,2,1,1]=(-0.825591116361)*x_ref[0]**o+(-0.552452876394)*x_ref[0]+(-0.262627261618)*x_ref[1]**o+(-0.147127256184)*x_ref[1]+(-0.171485665103)*x_ref[2]**o+(0.823279741501)*x_ref[2]
        ref[0,2,2,0]=(-0.0416279689068)*x_ref[0]**o+(-0.0903338000001)*x_ref[0]+(0.911673780635)*x_ref[1]**o+(0.86298673049)*x_ref[1]+(-0.60251179847)*x_ref[2]**o+(0.378303877746)*x_ref[2]
        ref[0,2,2,1]=(0.261384106387)*x_ref[0]**o+(0.0326416199811)*x_ref[0]+(-0.426740102269)*x_ref[1]**o+(-0.445401626921)*x_ref[1]+(-0.388668254526)*x_ref[2]**o+(-0.977942913284)*x_ref[2]
        ref[0,3,0,0]=(-0.0195053813483)*x_ref[0]**o+(-0.294061998664)*x_ref[0]+(0.790461098426)*x_ref[1]**o+(-0.506334268838)*x_ref[1]+(0.394015541605)*x_ref[2]**o+(-0.562248105432)*x_ref[2]
        ref[0,3,0,1]=(0.390726516735)*x_ref[0]**o+(-0.757597437859)*x_ref[0]+(0.861882149861)*x_ref[1]**o+(0.621870451314)*x_ref[1]+(0.900777666162)*x_ref[2]**o+(0.271507131938)*x_ref[2]
        ref[0,3,1,0]=(-0.725635966475)*x_ref[0]**o+(-0.695837548737)*x_ref[0]+(-0.163078176328)*x_ref[1]**o+(-0.0435808418729)*x_ref[1]+(0.0890756544611)*x_ref[2]**o+(-0.270422627949)*x_ref[2]
        ref[0,3,1,1]=(-0.175498720647)*x_ref[0]**o+(-0.213605441632)*x_ref[0]+(-0.358632620362)*x_ref[1]**o+(-0.193273004152)*x_ref[1]+(0.937704823805)*x_ref[2]**o+(0.861707832291)*x_ref[2]
        ref[0,3,2,0]=(0.0976010265611)*x_ref[0]**o+(0.995135925341)*x_ref[0]+(0.755339287387)*x_ref[1]**o+(-0.924722068193)*x_ref[1]+(-0.480212181904)*x_ref[2]**o+(-0.40237760439)*x_ref[2]
        ref[0,3,2,1]=(-0.679215392952)*x_ref[0]**o+(-0.00796096253297)*x_ref[0]+(-0.964972599421)*x_ref[1]**o+(-0.410415435656)*x_ref[1]+(-0.292638067785)*x_ref[2]**o+(-0.950383976561)*x_ref[2]
        ref[0,4,0,0]=(0.245586594928)*x_ref[0]**o+(-0.593420401896)*x_ref[0]+(0.553450486534)*x_ref[1]**o+(0.173814965689)*x_ref[1]+(0.0719392610563)*x_ref[2]**o+(0.676231437486)*x_ref[2]
        ref[0,4,0,1]=(0.154382756425)*x_ref[0]**o+(-0.0655740293432)*x_ref[0]+(0.924689745142)*x_ref[1]**o+(-0.246876531175)*x_ref[1]+(-0.241460713543)*x_ref[2]**o+(-0.489840108568)*x_ref[2]
        ref[0,4,1,0]=(-0.482705840359)*x_ref[0]**o+(-0.392960380114)*x_ref[0]+(-0.503463823972)*x_ref[1]**o+(0.317163204738)*x_ref[1]+(0.775048877356)*x_ref[2]**o+(0.599889732326)*x_ref[2]
        ref[0,4,1,1]=(-0.122555861716)*x_ref[0]**o+(0.930175770449)*x_ref[0]+(0.483503764941)*x_ref[1]**o+(0.340001033217)*x_ref[1]+(0.672924280979)*x_ref[2]**o+(0.746489359683)*x_ref[2]
        ref[0,4,2,0]=(-0.339421141033)*x_ref[0]**o+(-0.604966575169)*x_ref[0]+(0.862413017648)*x_ref[1]**o+(0.936752078454)*x_ref[1]+(-0.0549628353285)*x_ref[2]**o+(-0.0858885835314)*x_ref[2]
        ref[0,4,2,1]=(-0.581767357421)*x_ref[0]**o+(-0.644286027548)*x_ref[0]+(0.76948396865)*x_ref[1]**o+(0.692265904744)*x_ref[1]+(0.955871091531)*x_ref[2]**o+(-0.378888503457)*x_ref[2]
        ref[1,0,0,0]=(-0.0407846595046)*x_ref[0]**o+(-0.293709943251)*x_ref[0]+(-0.310552931582)*x_ref[1]**o+(-0.696244178466)*x_ref[1]+(0.962985284252)*x_ref[2]**o+(-0.0266457621404)*x_ref[2]
        ref[1,0,0,1]=(-0.480942811584)*x_ref[0]**o+(-0.400158759679)*x_ref[0]+(0.311171869807)*x_ref[1]**o+(-0.79518809427)*x_ref[1]+(-0.497004450478)*x_ref[2]**o+(-0.639822005752)*x_ref[2]
        ref[1,0,1,0]=(0.158738648483)*x_ref[0]**o+(0.274859816468)*x_ref[0]+(-0.208677612654)*x_ref[1]**o+(0.911107598366)*x_ref[1]+(-0.400163007778)*x_ref[2]**o+(0.276296890267)*x_ref[2]
        ref[1,0,1,1]=(0.613312276864)*x_ref[0]**o+(-0.644237218776)*x_ref[0]+(0.81927272895)*x_ref[1]**o+(-0.12971860556)*x_ref[1]+(0.214379406799)*x_ref[2]**o+(0.856335675043)*x_ref[2]
        ref[1,0,2,0]=(-0.699925173766)*x_ref[0]**o+(-0.560613547952)*x_ref[0]+(-0.355325107637)*x_ref[1]**o+(-0.598747822041)*x_ref[1]+(0.243087037553)*x_ref[2]**o+(-0.484176993015)*x_ref[2]
        ref[1,0,2,1]=(-0.218912300848)*x_ref[0]**o+(-0.259483312969)*x_ref[0]+(-0.863788186295)*x_ref[1]**o+(0.924423774757)*x_ref[1]+(-0.0823752192204)*x_ref[2]**o+(-0.928377900602)*x_ref[2]
        ref[1,1,0,0]=(-0.368901144425)*x_ref[0]**o+(0.544104655592)*x_ref[0]+(0.321295151176)*x_ref[1]**o+(0.395692754134)*x_ref[1]+(-0.721319111754)*x_ref[2]**o+(-0.698518442137)*x_ref[2]
        ref[1,1,0,1]=(-0.592867336047)*x_ref[0]**o+(0.145120231038)*x_ref[0]+(0.110390771366)*x_ref[1]**o+(0.168333368086)*x_ref[1]+(-0.577866419921)*x_ref[2]**o+(0.705027708469)*x_ref[2]
        ref[1,1,1,0]=(-0.210603428235)*x_ref[0]**o+(-0.250156447116)*x_ref[0]+(-0.397803248976)*x_ref[1]**o+(0.332163380867)*x_ref[1]+(-0.250648216541)*x_ref[2]**o+(-0.0693729055697)*x_ref[2]
        ref[1,1,1,1]=(0.324927558096)*x_ref[0]**o+(-0.262726116223)*x_ref[0]+(-0.555439758488)*x_ref[1]**o+(-0.865019248609)*x_ref[1]+(-0.0971907425857)*x_ref[2]**o+(-0.742359236711)*x_ref[2]
        ref[1,1,2,0]=(0.161058869414)*x_ref[0]**o+(0.349191500791)*x_ref[0]+(0.468336473315)*x_ref[1]**o+(-0.856364775995)*x_ref[1]+(0.491736696689)*x_ref[2]**o+(0.352984104173)*x_ref[2]
        ref[1,1,2,1]=(-0.243275976508)*x_ref[0]**o+(-0.785605388878)*x_ref[0]+(-0.282478919296)*x_ref[1]**o+(-0.728851522341)*x_ref[1]+(0.375134848237)*x_ref[2]**o+(-0.486200561256)*x_ref[2]
        ref[1,2,0,0]=(-0.172241451405)*x_ref[0]**o+(0.422361498196)*x_ref[0]+(0.464862755164)*x_ref[1]**o+(0.466661717105)*x_ref[1]+(-0.886209677512)*x_ref[2]**o+(-0.43529076789)*x_ref[2]
        ref[1,2,0,1]=(0.946582222405)*x_ref[0]**o+(0.513640973894)*x_ref[0]+(-0.923788000633)*x_ref[1]**o+(-0.358163567195)*x_ref[1]+(-0.709999132129)*x_ref[2]**o+(-0.761163299694)*x_ref[2]
        ref[1,2,1,0]=(0.236339981889)*x_ref[0]**o+(0.259874866134)*x_ref[0]+(-0.32460762691)*x_ref[1]**o+(-0.629786950313)*x_ref[1]+(-0.679261685402)*x_ref[2]**o+(0.593069192918)*x_ref[2]
        ref[1,2,1,1]=(0.11006800009)*x_ref[0]**o+(0.970532991787)*x_ref[0]+(-0.467608188361)*x_ref[1]**o+(-0.959281984552)*x_ref[1]+(0.610396622519)*x_ref[2]**o+(0.282468600058)*x_ref[2]
        ref[1,2,2,0]=(-0.656419157742)*x_ref[0]**o+(0.234027308171)*x_ref[0]+(-0.67027206978)*x_ref[1]**o+(-0.746034436817)*x_ref[1]+(-0.771894073652)*x_ref[2]**o+(0.949545830287)*x_ref[2]
        ref[1,2,2,1]=(0.615884236811)*x_ref[0]**o+(-0.670156612555)*x_ref[0]+(0.543165493345)*x_ref[1]**o+(-0.618294277594)*x_ref[1]+(-0.974114255061)*x_ref[2]**o+(-0.762320542298)*x_ref[2]
        ref[1,3,0,0]=(0.096201957648)*x_ref[0]**o+(0.440870816208)*x_ref[0]+(0.124448859607)*x_ref[1]**o+(0.507919407561)*x_ref[1]+(-0.401666263687)*x_ref[2]**o+(-0.437653713723)*x_ref[2]
        ref[1,3,0,1]=(0.165129229294)*x_ref[0]**o+(-0.353217428955)*x_ref[0]+(0.473666065504)*x_ref[1]**o+(0.907667166988)*x_ref[1]+(-0.864298533288)*x_ref[2]**o+(0.641505117281)*x_ref[2]
        ref[1,3,1,0]=(0.381900624425)*x_ref[0]**o+(0.427520178597)*x_ref[0]+(-0.560279540293)*x_ref[1]**o+(0.190814532271)*x_ref[1]+(-0.187947688629)*x_ref[2]**o+(-0.127694148593)*x_ref[2]
        ref[1,3,1,1]=(-0.903921363895)*x_ref[0]**o+(-0.659942040828)*x_ref[0]+(-0.549460213492)*x_ref[1]**o+(0.869909651895)*x_ref[1]+(-0.962273439484)*x_ref[2]**o+(-0.046365857751)*x_ref[2]
        ref[1,3,2,0]=(-0.021688482187)*x_ref[0]**o+(-0.209531000717)*x_ref[0]+(0.668564395133)*x_ref[1]**o+(-0.0347978700975)*x_ref[1]+(0.947677147488)*x_ref[2]**o+(-0.0152385052475)*x_ref[2]
        ref[1,3,2,1]=(0.880839485817)*x_ref[0]**o+(-0.319275929312)*x_ref[0]+(-0.940056657485)*x_ref[1]**o+(0.570855397726)*x_ref[1]+(0.279710812074)*x_ref[2]**o+(0.496622215051)*x_ref[2]
        ref[1,4,0,0]=(-0.139033645615)*x_ref[0]**o+(-0.271453722366)*x_ref[0]+(-0.268516720244)*x_ref[1]**o+(-0.0467555017438)*x_ref[1]+(-0.41618262265)*x_ref[2]**o+(0.315320363813)*x_ref[2]
        ref[1,4,0,1]=(0.296108607204)*x_ref[0]**o+(-0.184582165661)*x_ref[0]+(0.121416543744)*x_ref[1]**o+(-0.922284926751)*x_ref[1]+(-0.437600089668)*x_ref[2]**o+(-0.608800626513)*x_ref[2]
        ref[1,4,1,0]=(0.8942028423)*x_ref[0]**o+(0.673402831188)*x_ref[0]+(-0.622529195562)*x_ref[1]**o+(-0.266052338798)*x_ref[1]+(-0.273167851702)*x_ref[2]**o+(-0.312211982747)*x_ref[2]
        ref[1,4,1,1]=(-0.102315711178)*x_ref[0]**o+(-0.885515719371)*x_ref[0]+(-0.532910101204)*x_ref[1]**o+(0.798525944348)*x_ref[1]+(0.640586802303)*x_ref[2]**o+(0.36308806536)*x_ref[2]
        ref[1,4,2,0]=(0.376793506541)*x_ref[0]**o+(-0.667063564875)*x_ref[0]+(0.609591985816)*x_ref[1]**o+(0.28347745142)*x_ref[1]+(-0.751832936302)*x_ref[2]**o+(-0.683738856965)*x_ref[2]
        ref[1,4,2,1]=(-0.51412252819)*x_ref[0]**o+(0.559560297206)*x_ref[0]+(0.751134603429)*x_ref[1]**o+(-0.0031812310797)*x_ref[1]+(-0.525776122178)*x_ref[2]**o+(0.338471942848)*x_ref[2]
        ref[2,0,0,0]=(0.937572244905)*x_ref[0]**o+(-0.293020870379)*x_ref[0]+(-0.918591520817)*x_ref[1]**o+(0.622194477261)*x_ref[1]+(0.31641482851)*x_ref[2]**o+(0.276912266864)*x_ref[2]
        ref[2,0,0,1]=(0.853041599998)*x_ref[0]**o+(-0.0220623724171)*x_ref[0]+(0.89909044296)*x_ref[1]**o+(-0.838782642732)*x_ref[1]+(0.0133197592042)*x_ref[2]**o+(-0.868977618993)*x_ref[2]
        ref[2,0,1,0]=(-0.325799042976)*x_ref[0]**o+(0.746269971994)*x_ref[0]+(0.882671061757)*x_ref[1]**o+(-0.284133373252)*x_ref[1]+(0.120086549521)*x_ref[2]**o+(-0.397352610969)*x_ref[2]
        ref[2,0,1,1]=(0.282336934951)*x_ref[0]**o+(-0.339893978547)*x_ref[0]+(-0.0861660125386)*x_ref[1]**o+(0.283063579798)*x_ref[1]+(0.714762478569)*x_ref[2]**o+(-0.497021718549)*x_ref[2]
        ref[2,0,2,0]=(0.891782779491)*x_ref[0]**o+(-0.817647577644)*x_ref[0]+(0.364088142085)*x_ref[1]**o+(-0.950577637048)*x_ref[1]+(0.4070663557)*x_ref[2]**o+(0.270753179246)*x_ref[2]
        ref[2,0,2,1]=(0.316594913808)*x_ref[0]**o+(0.310731486851)*x_ref[0]+(0.250421828765)*x_ref[1]**o+(0.772786141458)*x_ref[1]+(0.64843040679)*x_ref[2]**o+(-0.691903332511)*x_ref[2]
        ref[2,1,0,0]=(0.87180715028)*x_ref[0]**o+(-0.953173230375)*x_ref[0]+(-0.823516344548)*x_ref[1]**o+(0.934653756548)*x_ref[1]+(-0.537124962489)*x_ref[2]**o+(-0.233365939659)*x_ref[2]
        ref[2,1,0,1]=(-0.407189744631)*x_ref[0]**o+(-0.754425720288)*x_ref[0]+(0.737526767897)*x_ref[1]**o+(0.745393943502)*x_ref[1]+(0.811744293685)*x_ref[2]**o+(-0.459556659177)*x_ref[2]
        ref[2,1,1,0]=(-0.693529259877)*x_ref[0]**o+(0.580197938048)*x_ref[0]+(0.506055440405)*x_ref[1]**o+(0.122531014759)*x_ref[1]+(0.427736137775)*x_ref[2]**o+(-0.83260140401)*x_ref[2]
        ref[2,1,1,1]=(0.542621975026)*x_ref[0]**o+(-0.568425713106)*x_ref[0]+(0.303002043446)*x_ref[1]**o+(0.363286261651)*x_ref[1]+(0.2729476289)*x_ref[2]**o+(-0.0113629744659)*x_ref[2]
        ref[2,1,2,0]=(-0.266750235694)*x_ref[0]**o+(0.10116400306)*x_ref[0]+(0.315977962897)*x_ref[1]**o+(-0.406236201454)*x_ref[1]+(-0.586845072374)*x_ref[2]**o+(0.513937047282)*x_ref[2]
        ref[2,1,2,1]=(-0.518780811975)*x_ref[0]**o+(0.275053640829)*x_ref[0]+(0.4234970646)*x_ref[1]**o+(0.389585163376)*x_ref[1]+(-0.609771068778)*x_ref[2]**o+(-0.121467061755)*x_ref[2]
        ref[2,2,0,0]=(0.138823207466)*x_ref[0]**o+(-0.779837455014)*x_ref[0]+(0.70167049555)*x_ref[1]**o+(0.766428813369)*x_ref[1]+(0.114023530366)*x_ref[2]**o+(0.650098536187)*x_ref[2]
        ref[2,2,0,1]=(0.974998667633)*x_ref[0]**o+(0.958471050654)*x_ref[0]+(0.193515162503)*x_ref[1]**o+(-0.252148167253)*x_ref[1]+(0.495475460125)*x_ref[2]**o+(0.562693347983)*x_ref[2]
        ref[2,2,1,0]=(-0.552379548988)*x_ref[0]**o+(-0.831399971618)*x_ref[0]+(0.437581216647)*x_ref[1]**o+(-0.775331267696)*x_ref[1]+(0.729041628844)*x_ref[2]**o+(0.633113359893)*x_ref[2]
        ref[2,2,1,1]=(0.379858437049)*x_ref[0]**o+(0.923586504203)*x_ref[0]+(0.707244783969)*x_ref[1]**o+(-0.293157375059)*x_ref[1]+(-0.368717941876)*x_ref[2]**o+(-0.202610288616)*x_ref[2]
        ref[2,2,2,0]=(-0.804826108231)*x_ref[0]**o+(-0.0208712096312)*x_ref[0]+(0.0482547072498)*x_ref[1]**o+(0.852366543609)*x_ref[1]+(0.328962500136)*x_ref[2]**o+(-0.132777835512)*x_ref[2]
        ref[2,2,2,1]=(0.91373752217)*x_ref[0]**o+(-0.792112120729)*x_ref[0]+(-0.0457564087072)*x_ref[1]**o+(0.309694852949)*x_ref[1]+(0.089219285337)*x_ref[2]**o+(-0.516000514614)*x_ref[2]
        ref[2,3,0,0]=(0.343215129115)*x_ref[0]**o+(0.40804099837)*x_ref[0]+(-0.865942442914)*x_ref[1]**o+(0.842241683043)*x_ref[1]+(0.0727803297601)*x_ref[2]**o+(-0.395225682949)*x_ref[2]
        ref[2,3,0,1]=(-0.965340522771)*x_ref[0]**o+(-0.897056478677)*x_ref[0]+(-0.305205944062)*x_ref[1]**o+(0.393941789685)*x_ref[1]+(-0.253328379131)*x_ref[2]**o+(0.0951510469271)*x_ref[2]
        ref[2,3,1,0]=(0.536251514752)*x_ref[0]**o+(0.0629776386779)*x_ref[0]+(-0.706311222379)*x_ref[1]**o+(-0.84362242029)*x_ref[1]+(0.00110678760539)*x_ref[2]**o+(-0.30286059941)*x_ref[2]
        ref[2,3,1,1]=(0.114584737295)*x_ref[0]**o+(0.0708763738067)*x_ref[0]+(-0.454760062397)*x_ref[1]**o+(0.228742539922)*x_ref[1]+(0.514960810352)*x_ref[2]**o+(-0.815749211085)*x_ref[2]
        ref[2,3,2,0]=(0.246335688008)*x_ref[0]**o+(-0.239199299885)*x_ref[0]+(-0.447390151659)*x_ref[1]**o+(0.918949356391)*x_ref[1]+(-0.307864840175)*x_ref[2]**o+(0.560139080481)*x_ref[2]
        ref[2,3,2,1]=(0.118766035723)*x_ref[0]**o+(0.610711164899)*x_ref[0]+(0.336628771776)*x_ref[1]**o+(0.833699268751)*x_ref[1]+(-0.424694647429)*x_ref[2]**o+(-0.639929688104)*x_ref[2]
        ref[2,4,0,0]=(0.814543506987)*x_ref[0]**o+(-0.797684321368)*x_ref[0]+(0.337242635783)*x_ref[1]**o+(0.278139837072)*x_ref[1]+(0.633511526441)*x_ref[2]**o+(0.372175615028)*x_ref[2]
        ref[2,4,0,1]=(-0.0590668694094)*x_ref[0]**o+(0.256732067267)*x_ref[0]+(0.519737187577)*x_ref[1]**o+(-0.376391855948)*x_ref[1]+(-0.810934658651)*x_ref[2]**o+(0.826899476928)*x_ref[2]
        ref[2,4,1,0]=(-0.428976843661)*x_ref[0]**o+(0.672667833313)*x_ref[0]+(0.263194890946)*x_ref[1]**o+(-0.894790773661)*x_ref[1]+(-0.550298557295)*x_ref[2]**o+(-0.00560384035502)*x_ref[2]
        ref[2,4,1,1]=(-0.818372430144)*x_ref[0]**o+(-0.829920833073)*x_ref[0]+(-0.359560636119)*x_ref[1]**o+(-0.113484563205)*x_ref[1]+(0.586796110365)*x_ref[2]**o+(0.296820898426)*x_ref[2]
        ref[2,4,2,0]=(-0.691597969566)*x_ref[0]**o+(0.715813153538)*x_ref[0]+(0.954093612503)*x_ref[1]**o+(0.88803420754)*x_ref[1]+(0.697447667765)*x_ref[2]**o+(0.793730496625)*x_ref[2]
        ref[2,4,2,1]=(0.351099833336)*x_ref[0]**o+(-0.716419272807)*x_ref[0]+(0.318726674998)*x_ref[1]**o+(0.246151914414)*x_ref[1]+(-0.945520673884)*x_ref[2]**o+(-0.489026233954)*x_ref[2]
        ref[3,0,0,0]=(0.511598022804)*x_ref[0]**o+(0.567617269582)*x_ref[0]+(-0.526293319137)*x_ref[1]**o+(0.333750098146)*x_ref[1]+(-0.0869228849144)*x_ref[2]**o+(-0.204072834867)*x_ref[2]
        ref[3,0,0,1]=(0.905384007172)*x_ref[0]**o+(-0.661067390314)*x_ref[0]+(-0.470790632295)*x_ref[1]**o+(-0.149608017616)*x_ref[1]+(0.427879278267)*x_ref[2]**o+(0.987040179184)*x_ref[2]
        ref[3,0,1,0]=(-0.946562006809)*x_ref[0]**o+(0.64141909633)*x_ref[0]+(-0.762076826276)*x_ref[1]**o+(-0.642912850794)*x_ref[1]+(0.712567898559)*x_ref[2]**o+(-0.292128944205)*x_ref[2]
        ref[3,0,1,1]=(-0.955087949361)*x_ref[0]**o+(0.426588041133)*x_ref[0]+(0.339313104127)*x_ref[1]**o+(-0.375147344045)*x_ref[1]+(0.967259014679)*x_ref[2]**o+(0.504521081858)*x_ref[2]
        ref[3,0,2,0]=(0.141876598344)*x_ref[0]**o+(-0.335579252825)*x_ref[0]+(0.514552993602)*x_ref[1]**o+(-0.531116948499)*x_ref[1]+(-0.369319413141)*x_ref[2]**o+(0.0374413262362)*x_ref[2]
        ref[3,0,2,1]=(0.678360463658)*x_ref[0]**o+(-0.357900119646)*x_ref[0]+(0.300750717059)*x_ref[1]**o+(0.516196189448)*x_ref[1]+(0.0535466018203)*x_ref[2]**o+(-0.149961940786)*x_ref[2]
        ref[3,1,0,0]=(0.0975994351079)*x_ref[0]**o+(0.634544068499)*x_ref[0]+(-0.963217046352)*x_ref[1]**o+(-0.804658091456)*x_ref[1]+(-0.620132720823)*x_ref[2]**o+(0.862250619369)*x_ref[2]
        ref[3,1,0,1]=(0.210306328879)*x_ref[0]**o+(-0.457341187966)*x_ref[0]+(0.957556084801)*x_ref[1]**o+(0.704073170928)*x_ref[1]+(0.423134492133)*x_ref[2]**o+(-0.167268375412)*x_ref[2]
        ref[3,1,1,0]=(-0.909228817744)*x_ref[0]**o+(0.110665018921)*x_ref[0]+(0.36275630147)*x_ref[1]**o+(0.346866824763)*x_ref[1]+(-0.535600547489)*x_ref[2]**o+(0.627638814375)*x_ref[2]
        ref[3,1,1,1]=(0.328780933823)*x_ref[0]**o+(-0.249563994161)*x_ref[0]+(-0.555789062892)*x_ref[1]**o+(-0.917287525036)*x_ref[1]+(0.386191175017)*x_ref[2]**o+(0.123904319061)*x_ref[2]
        ref[3,1,2,0]=(-0.838409901623)*x_ref[0]**o+(0.673479705393)*x_ref[0]+(0.482921104917)*x_ref[1]**o+(0.136411388588)*x_ref[1]+(0.583674116225)*x_ref[2]**o+(0.305100836877)*x_ref[2]
        ref[3,1,2,1]=(0.952961008312)*x_ref[0]**o+(-0.0501981079532)*x_ref[0]+(0.661434464642)*x_ref[1]**o+(0.65030469399)*x_ref[1]+(-0.157562001295)*x_ref[2]**o+(-0.249551892917)*x_ref[2]
        ref[3,2,0,0]=(-0.0232804500286)*x_ref[0]**o+(-0.559229392203)*x_ref[0]+(0.127967930273)*x_ref[1]**o+(-0.635827181645)*x_ref[1]+(-0.0426703224383)*x_ref[2]**o+(0.47720230258)*x_ref[2]
        ref[3,2,0,1]=(-0.421658996106)*x_ref[0]**o+(-0.468260038077)*x_ref[0]+(0.479083572554)*x_ref[1]**o+(-0.214417881841)*x_ref[1]+(0.372430534261)*x_ref[2]**o+(0.133063789546)*x_ref[2]
        ref[3,2,1,0]=(0.905811586567)*x_ref[0]**o+(0.245447340384)*x_ref[0]+(-0.337650999146)*x_ref[1]**o+(0.908188235976)*x_ref[1]+(-0.804029751277)*x_ref[2]**o+(0.0512676456643)*x_ref[2]
        ref[3,2,1,1]=(-0.891486839409)*x_ref[0]**o+(0.20753794329)*x_ref[0]+(0.879183464252)*x_ref[1]**o+(0.902028906992)*x_ref[1]+(0.416776813598)*x_ref[2]**o+(0.491964374373)*x_ref[2]
        ref[3,2,2,0]=(0.653067027119)*x_ref[0]**o+(0.765293890745)*x_ref[0]+(-0.348902169508)*x_ref[1]**o+(0.324429035849)*x_ref[1]+(-0.363026365065)*x_ref[2]**o+(0.0816186405511)*x_ref[2]
        ref[3,2,2,1]=(-0.0415375052325)*x_ref[0]**o+(-0.480779075784)*x_ref[0]+(-0.137223552938)*x_ref[1]**o+(-0.915733625512)*x_ref[1]+(0.750199445159)*x_ref[2]**o+(-0.789818945171)*x_ref[2]
        ref[3,3,0,0]=(0.0744921239083)*x_ref[0]**o+(0.0700589144159)*x_ref[0]+(-0.327105425297)*x_ref[1]**o+(0.469843257765)*x_ref[1]+(-0.170191600844)*x_ref[2]**o+(0.732071534155)*x_ref[2]
        ref[3,3,0,1]=(0.172643927441)*x_ref[0]**o+(0.848924114383)*x_ref[0]+(-0.79125762228)*x_ref[1]**o+(-0.694725775976)*x_ref[1]+(-0.309710503985)*x_ref[2]**o+(-0.405310234927)*x_ref[2]
        ref[3,3,1,0]=(-0.465911787595)*x_ref[0]**o+(-0.71080474709)*x_ref[0]+(0.959925934203)*x_ref[1]**o+(-0.111110644825)*x_ref[1]+(-0.145349925218)*x_ref[2]**o+(0.122955165811)*x_ref[2]
        ref[3,3,1,1]=(-0.769111863754)*x_ref[0]**o+(0.348036938532)*x_ref[0]+(0.485890856474)*x_ref[1]**o+(0.382595218985)*x_ref[1]+(-0.897488692981)*x_ref[2]**o+(0.830822333819)*x_ref[2]
        ref[3,3,2,0]=(0.341890793681)*x_ref[0]**o+(-0.667145936571)*x_ref[0]+(-0.145094108268)*x_ref[1]**o+(0.771433911212)*x_ref[1]+(-0.299752164325)*x_ref[2]**o+(-0.357856549677)*x_ref[2]
        ref[3,3,2,1]=(0.26381838796)*x_ref[0]**o+(-0.44474724998)*x_ref[0]+(0.310283988195)*x_ref[1]**o+(0.698675181314)*x_ref[1]+(0.437624133238)*x_ref[2]**o+(0.394916054445)*x_ref[2]
        ref[3,4,0,0]=(-0.576499406674)*x_ref[0]**o+(0.770936634959)*x_ref[0]+(0.707441735783)*x_ref[1]**o+(-0.794953257043)*x_ref[1]+(-0.535342943209)*x_ref[2]**o+(-0.509446811787)*x_ref[2]
        ref[3,4,0,1]=(0.161115012113)*x_ref[0]**o+(0.11079092252)*x_ref[0]+(0.0633804181164)*x_ref[1]**o+(-0.406049293019)*x_ref[1]+(-0.824283994336)*x_ref[2]**o+(-0.481107560499)*x_ref[2]
        ref[3,4,1,0]=(-0.411133810295)*x_ref[0]**o+(0.988486625099)*x_ref[0]+(-0.36050191367)*x_ref[1]**o+(0.469479821965)*x_ref[1]+(-0.431175357325)*x_ref[2]**o+(-0.966295015263)*x_ref[2]
        ref[3,4,1,1]=(0.233731410367)*x_ref[0]**o+(-0.821900740766)*x_ref[0]+(-0.823123461385)*x_ref[1]**o+(-0.380537914421)*x_ref[1]+(-0.896146300355)*x_ref[2]**o+(-0.775172990257)*x_ref[2]
        ref[3,4,2,0]=(-0.96323363586)*x_ref[0]**o+(0.0935412405245)*x_ref[0]+(0.929333026259)*x_ref[1]**o+(-0.322236627348)*x_ref[1]+(-0.390898135964)*x_ref[2]**o+(-0.766178290162)*x_ref[2]
        ref[3,4,2,1]=(-0.773670533268)*x_ref[0]**o+(-0.156975157098)*x_ref[0]+(-0.91512996795)*x_ref[1]**o+(0.0247803682126)*x_ref[1]+(0.995159733782)*x_ref[2]**o+(-0.669224144225)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_ContinuousFunction_rank0(self):
      """
      tests interpolate for rank 0 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ContinuousFunction onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(),dim=dim)
      s=Data(0,(),w)
      ref=Data(0,(),w_ref)
      if dim==2:
        s=(-0.0411847356752)*x[0]**o+(-0.978252807465)*x[0]+(-0.765516202331)*x[1]**o+(-0.60628403943)*x[1]
        ref=(-0.0411847356752)*x_ref[0]**o+(-0.978252807465)*x_ref[0]+(-0.765516202331)*x_ref[1]**o+(-0.60628403943)*x_ref[1]
      else:
        s=(-0.520571545539)*x[0]**o+(0.516384538651)*x[0]+(-0.865529136972)*x[1]**o+(0.80164386579)*x[1]+(-0.330695181072)*x[2]**o+(-0.758708438741)*x[2]
        ref=(-0.520571545539)*x_ref[0]**o+(0.516384538651)*x_ref[0]+(-0.865529136972)*x_ref[1]**o+(0.80164386579)*x_ref[1]+(-0.330695181072)*x_ref[2]**o+(-0.758708438741)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_ContinuousFunction_rank1(self):
      """
      tests interpolate for rank 1 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ContinuousFunction onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(2,),dim=dim)
      s=Data(0,(2,),w)
      ref=Data(0,(2,),w_ref)
      if dim==2:
        s[0]=(0.505196018365)*x[0]**o+(0.731510032507)*x[0]+(-0.0776298756069)*x[1]**o+(0.223327557462)*x[1]
        s[1]=(-0.239558790149)*x[0]**o+(0.608302150075)*x[0]+(-0.840461486286)*x[1]**o+(-0.0797379857546)*x[1]
        ref[0]=(0.505196018365)*x_ref[0]**o+(0.731510032507)*x_ref[0]+(-0.0776298756069)*x_ref[1]**o+(0.223327557462)*x_ref[1]
        ref[1]=(-0.239558790149)*x_ref[0]**o+(0.608302150075)*x_ref[0]+(-0.840461486286)*x_ref[1]**o+(-0.0797379857546)*x_ref[1]
      else:
        s[0]=(-0.853964974211)*x[0]**o+(-0.436921639846)*x[0]+(-0.356712309058)*x[1]**o+(0.324151975277)*x[1]+(0.338538264809)*x[2]**o+(0.254654297562)*x[2]
        s[1]=(0.46616823299)*x[0]**o+(0.64792464198)*x[0]+(-0.655892247116)*x[1]**o+(-0.9837201382)*x[1]+(0.36604513951)*x[2]**o+(0.343454378544)*x[2]
        ref[0]=(-0.853964974211)*x_ref[0]**o+(-0.436921639846)*x_ref[0]+(-0.356712309058)*x_ref[1]**o+(0.324151975277)*x_ref[1]+(0.338538264809)*x_ref[2]**o+(0.254654297562)*x_ref[2]
        ref[1]=(0.46616823299)*x_ref[0]**o+(0.64792464198)*x_ref[0]+(-0.655892247116)*x_ref[1]**o+(-0.9837201382)*x_ref[1]+(0.36604513951)*x_ref[2]**o+(0.343454378544)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_ContinuousFunction_rank2(self):
      """
      tests interpolate for rank 2 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ContinuousFunction onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5),dim=dim)
      s=Data(0,(4, 5),w)
      ref=Data(0,(4, 5),w_ref)
      if dim==2:
        s[0,0]=(0.403739435229)*x[0]**o+(-0.976543623588)*x[0]+(0.408638664295)*x[1]**o+(0.290441090188)*x[1]
        s[0,1]=(-0.186112372212)*x[0]**o+(-0.885114653425)*x[0]+(0.79020447405)*x[1]**o+(-0.988138900337)*x[1]
        s[0,2]=(-0.749718519557)*x[0]**o+(-0.556662006553)*x[0]+(-0.388628750048)*x[1]**o+(0.488811308506)*x[1]
        s[0,3]=(0.55958060511)*x[0]**o+(-0.692678975478)*x[0]+(-0.507024092372)*x[1]**o+(0.547618022755)*x[1]
        s[0,4]=(0.393523537188)*x[0]**o+(-0.278215081058)*x[0]+(0.524241376975)*x[1]**o+(-0.18209236848)*x[1]
        s[1,0]=(0.277364605261)*x[0]**o+(0.867787382941)*x[0]+(-0.468812178843)*x[1]**o+(-0.915273555727)*x[1]
        s[1,1]=(-0.883469445503)*x[0]**o+(0.949713147084)*x[0]+(-0.195972546782)*x[1]**o+(-0.550840574214)*x[1]
        s[1,2]=(-0.0612993770053)*x[0]**o+(0.349066961111)*x[0]+(0.117833853269)*x[1]**o+(0.915145781122)*x[1]
        s[1,3]=(-0.221469359751)*x[0]**o+(0.860157157768)*x[0]+(0.334660915861)*x[1]**o+(0.0202241427453)*x[1]
        s[1,4]=(-0.859671577801)*x[0]**o+(-0.751878816346)*x[0]+(0.970178380651)*x[1]**o+(-0.852998297383)*x[1]
        s[2,0]=(-0.273019645199)*x[0]**o+(0.77546843291)*x[0]+(0.40198451057)*x[1]**o+(0.719427949335)*x[1]
        s[2,1]=(-0.615814919857)*x[0]**o+(-0.804830902046)*x[0]+(-0.55666668286)*x[1]**o+(-0.436274362037)*x[1]
        s[2,2]=(-0.257042626769)*x[0]**o+(-0.0674177291518)*x[0]+(-0.487890885639)*x[1]**o+(-0.276346354771)*x[1]
        s[2,3]=(-0.0863705553448)*x[0]**o+(-0.456256145846)*x[0]+(0.279365337349)*x[1]**o+(-0.788366887305)*x[1]
        s[2,4]=(0.00696265740333)*x[0]**o+(-0.903180556586)*x[0]+(0.299924222843)*x[1]**o+(0.432554144375)*x[1]
        s[3,0]=(0.928471341856)*x[0]**o+(-0.58681191854)*x[0]+(0.592280256168)*x[1]**o+(-0.486182643564)*x[1]
        s[3,1]=(-0.949679045727)*x[0]**o+(0.28219904154)*x[0]+(0.742879063098)*x[1]**o+(-0.0189753136959)*x[1]
        s[3,2]=(-0.386608884833)*x[0]**o+(0.771169732795)*x[0]+(-0.864253528029)*x[1]**o+(-0.0196378783985)*x[1]
        s[3,3]=(0.744817366451)*x[0]**o+(0.973334266028)*x[0]+(0.879085180711)*x[1]**o+(-0.682866003591)*x[1]
        s[3,4]=(-0.453687552098)*x[0]**o+(-0.421237560962)*x[0]+(0.55363269981)*x[1]**o+(-0.446292920116)*x[1]
        ref[0,0]=(0.403739435229)*x_ref[0]**o+(-0.976543623588)*x_ref[0]+(0.408638664295)*x_ref[1]**o+(0.290441090188)*x_ref[1]
        ref[0,1]=(-0.186112372212)*x_ref[0]**o+(-0.885114653425)*x_ref[0]+(0.79020447405)*x_ref[1]**o+(-0.988138900337)*x_ref[1]
        ref[0,2]=(-0.749718519557)*x_ref[0]**o+(-0.556662006553)*x_ref[0]+(-0.388628750048)*x_ref[1]**o+(0.488811308506)*x_ref[1]
        ref[0,3]=(0.55958060511)*x_ref[0]**o+(-0.692678975478)*x_ref[0]+(-0.507024092372)*x_ref[1]**o+(0.547618022755)*x_ref[1]
        ref[0,4]=(0.393523537188)*x_ref[0]**o+(-0.278215081058)*x_ref[0]+(0.524241376975)*x_ref[1]**o+(-0.18209236848)*x_ref[1]
        ref[1,0]=(0.277364605261)*x_ref[0]**o+(0.867787382941)*x_ref[0]+(-0.468812178843)*x_ref[1]**o+(-0.915273555727)*x_ref[1]
        ref[1,1]=(-0.883469445503)*x_ref[0]**o+(0.949713147084)*x_ref[0]+(-0.195972546782)*x_ref[1]**o+(-0.550840574214)*x_ref[1]
        ref[1,2]=(-0.0612993770053)*x_ref[0]**o+(0.349066961111)*x_ref[0]+(0.117833853269)*x_ref[1]**o+(0.915145781122)*x_ref[1]
        ref[1,3]=(-0.221469359751)*x_ref[0]**o+(0.860157157768)*x_ref[0]+(0.334660915861)*x_ref[1]**o+(0.0202241427453)*x_ref[1]
        ref[1,4]=(-0.859671577801)*x_ref[0]**o+(-0.751878816346)*x_ref[0]+(0.970178380651)*x_ref[1]**o+(-0.852998297383)*x_ref[1]
        ref[2,0]=(-0.273019645199)*x_ref[0]**o+(0.77546843291)*x_ref[0]+(0.40198451057)*x_ref[1]**o+(0.719427949335)*x_ref[1]
        ref[2,1]=(-0.615814919857)*x_ref[0]**o+(-0.804830902046)*x_ref[0]+(-0.55666668286)*x_ref[1]**o+(-0.436274362037)*x_ref[1]
        ref[2,2]=(-0.257042626769)*x_ref[0]**o+(-0.0674177291518)*x_ref[0]+(-0.487890885639)*x_ref[1]**o+(-0.276346354771)*x_ref[1]
        ref[2,3]=(-0.0863705553448)*x_ref[0]**o+(-0.456256145846)*x_ref[0]+(0.279365337349)*x_ref[1]**o+(-0.788366887305)*x_ref[1]
        ref[2,4]=(0.00696265740333)*x_ref[0]**o+(-0.903180556586)*x_ref[0]+(0.299924222843)*x_ref[1]**o+(0.432554144375)*x_ref[1]
        ref[3,0]=(0.928471341856)*x_ref[0]**o+(-0.58681191854)*x_ref[0]+(0.592280256168)*x_ref[1]**o+(-0.486182643564)*x_ref[1]
        ref[3,1]=(-0.949679045727)*x_ref[0]**o+(0.28219904154)*x_ref[0]+(0.742879063098)*x_ref[1]**o+(-0.0189753136959)*x_ref[1]
        ref[3,2]=(-0.386608884833)*x_ref[0]**o+(0.771169732795)*x_ref[0]+(-0.864253528029)*x_ref[1]**o+(-0.0196378783985)*x_ref[1]
        ref[3,3]=(0.744817366451)*x_ref[0]**o+(0.973334266028)*x_ref[0]+(0.879085180711)*x_ref[1]**o+(-0.682866003591)*x_ref[1]
        ref[3,4]=(-0.453687552098)*x_ref[0]**o+(-0.421237560962)*x_ref[0]+(0.55363269981)*x_ref[1]**o+(-0.446292920116)*x_ref[1]
      else:
        s[0,0]=(-0.200149935444)*x[0]**o+(0.507524859849)*x[0]+(0.619745083854)*x[1]**o+(0.627105339142)*x[1]+(0.563386979803)*x[2]**o+(-0.961618026889)*x[2]
        s[0,1]=(0.363899424005)*x[0]**o+(0.924425901309)*x[0]+(-0.658883398191)*x[1]**o+(0.481606823837)*x[1]+(-0.159370395541)*x[2]**o+(-0.311388746226)*x[2]
        s[0,2]=(-0.0293410199192)*x[0]**o+(0.256908390845)*x[0]+(-0.909877958762)*x[1]**o+(0.105112556474)*x[1]+(0.252859198357)*x[2]**o+(0.92186026337)*x[2]
        s[0,3]=(-0.00170582078327)*x[0]**o+(0.730720627097)*x[0]+(-0.801575927568)*x[1]**o+(0.455500499782)*x[1]+(0.380759901094)*x[2]**o+(-0.823963841034)*x[2]
        s[0,4]=(0.259808988031)*x[0]**o+(0.863791713598)*x[0]+(0.553188235768)*x[1]**o+(-0.464509312659)*x[1]+(0.42570171)*x[2]**o+(0.192687787615)*x[2]
        s[1,0]=(0.994753432059)*x[0]**o+(-0.68864217861)*x[0]+(-0.0204526591618)*x[1]**o+(-0.306443480795)*x[1]+(0.148406858899)*x[2]**o+(0.854009274563)*x[2]
        s[1,1]=(0.173721730295)*x[0]**o+(-0.987657366279)*x[0]+(0.253868939396)*x[1]**o+(-0.771588800243)*x[1]+(0.53766861285)*x[2]**o+(0.719877605356)*x[2]
        s[1,2]=(0.430924077377)*x[0]**o+(-0.746769688215)*x[0]+(0.828996359633)*x[1]**o+(-0.192590851955)*x[1]+(0.669402098684)*x[2]**o+(-0.962105583491)*x[2]
        s[1,3]=(-0.410727896031)*x[0]**o+(-0.833008011026)*x[0]+(-0.927201509946)*x[1]**o+(-0.876016513532)*x[1]+(0.0914553334551)*x[2]**o+(-0.348128799709)*x[2]
        s[1,4]=(-0.73394473562)*x[0]**o+(0.00706331438319)*x[0]+(-0.205588695357)*x[1]**o+(0.714894523918)*x[1]+(-0.713385984004)*x[2]**o+(-0.385550060437)*x[2]
        s[2,0]=(-0.0853783479307)*x[0]**o+(0.596547791334)*x[0]+(-0.0925317449093)*x[1]**o+(0.751414290883)*x[1]+(0.838736367666)*x[2]**o+(0.735536663934)*x[2]
        s[2,1]=(-0.159026214794)*x[0]**o+(0.949032823761)*x[0]+(-0.422829286588)*x[1]**o+(0.985896164837)*x[1]+(-0.758176201469)*x[2]**o+(0.283868871122)*x[2]
        s[2,2]=(-0.14639325109)*x[0]**o+(0.346468853582)*x[0]+(0.563105800299)*x[1]**o+(-0.531036765601)*x[1]+(-0.0990976207418)*x[2]**o+(0.750636702702)*x[2]
        s[2,3]=(0.849599418085)*x[0]**o+(-0.779136748258)*x[0]+(0.730951130588)*x[1]**o+(0.53628857622)*x[1]+(-0.947230186497)*x[2]**o+(-0.72450102135)*x[2]
        s[2,4]=(-0.700617249374)*x[0]**o+(-0.10080480368)*x[0]+(-0.507852228465)*x[1]**o+(-0.857417192918)*x[1]+(0.640292095556)*x[2]**o+(-0.929256110195)*x[2]
        s[3,0]=(0.653772333481)*x[0]**o+(0.000343679690948)*x[0]+(0.668997254669)*x[1]**o+(0.362901119538)*x[1]+(-0.830987535544)*x[2]**o+(-0.423720904628)*x[2]
        s[3,1]=(-0.0420273808908)*x[0]**o+(0.323011823157)*x[0]+(-0.24882314712)*x[1]**o+(-0.423985714012)*x[1]+(0.752345053106)*x[2]**o+(-0.160413317211)*x[2]
        s[3,2]=(0.890620617817)*x[0]**o+(0.783464544239)*x[0]+(0.790371129121)*x[1]**o+(0.237259212081)*x[1]+(0.610840739105)*x[2]**o+(0.973783326469)*x[2]
        s[3,3]=(-0.379379310577)*x[0]**o+(0.690137408246)*x[0]+(-0.779634423108)*x[1]**o+(-0.974077688357)*x[1]+(0.788216664671)*x[2]**o+(0.101022285352)*x[2]
        s[3,4]=(-0.505116742845)*x[0]**o+(0.923132757962)*x[0]+(-0.0146493711199)*x[1]**o+(0.994421781292)*x[1]+(-0.20080218328)*x[2]**o+(0.555271515554)*x[2]
        ref[0,0]=(-0.200149935444)*x_ref[0]**o+(0.507524859849)*x_ref[0]+(0.619745083854)*x_ref[1]**o+(0.627105339142)*x_ref[1]+(0.563386979803)*x_ref[2]**o+(-0.961618026889)*x_ref[2]
        ref[0,1]=(0.363899424005)*x_ref[0]**o+(0.924425901309)*x_ref[0]+(-0.658883398191)*x_ref[1]**o+(0.481606823837)*x_ref[1]+(-0.159370395541)*x_ref[2]**o+(-0.311388746226)*x_ref[2]
        ref[0,2]=(-0.0293410199192)*x_ref[0]**o+(0.256908390845)*x_ref[0]+(-0.909877958762)*x_ref[1]**o+(0.105112556474)*x_ref[1]+(0.252859198357)*x_ref[2]**o+(0.92186026337)*x_ref[2]
        ref[0,3]=(-0.00170582078327)*x_ref[0]**o+(0.730720627097)*x_ref[0]+(-0.801575927568)*x_ref[1]**o+(0.455500499782)*x_ref[1]+(0.380759901094)*x_ref[2]**o+(-0.823963841034)*x_ref[2]
        ref[0,4]=(0.259808988031)*x_ref[0]**o+(0.863791713598)*x_ref[0]+(0.553188235768)*x_ref[1]**o+(-0.464509312659)*x_ref[1]+(0.42570171)*x_ref[2]**o+(0.192687787615)*x_ref[2]
        ref[1,0]=(0.994753432059)*x_ref[0]**o+(-0.68864217861)*x_ref[0]+(-0.0204526591618)*x_ref[1]**o+(-0.306443480795)*x_ref[1]+(0.148406858899)*x_ref[2]**o+(0.854009274563)*x_ref[2]
        ref[1,1]=(0.173721730295)*x_ref[0]**o+(-0.987657366279)*x_ref[0]+(0.253868939396)*x_ref[1]**o+(-0.771588800243)*x_ref[1]+(0.53766861285)*x_ref[2]**o+(0.719877605356)*x_ref[2]
        ref[1,2]=(0.430924077377)*x_ref[0]**o+(-0.746769688215)*x_ref[0]+(0.828996359633)*x_ref[1]**o+(-0.192590851955)*x_ref[1]+(0.669402098684)*x_ref[2]**o+(-0.962105583491)*x_ref[2]
        ref[1,3]=(-0.410727896031)*x_ref[0]**o+(-0.833008011026)*x_ref[0]+(-0.927201509946)*x_ref[1]**o+(-0.876016513532)*x_ref[1]+(0.0914553334551)*x_ref[2]**o+(-0.348128799709)*x_ref[2]
        ref[1,4]=(-0.73394473562)*x_ref[0]**o+(0.00706331438319)*x_ref[0]+(-0.205588695357)*x_ref[1]**o+(0.714894523918)*x_ref[1]+(-0.713385984004)*x_ref[2]**o+(-0.385550060437)*x_ref[2]
        ref[2,0]=(-0.0853783479307)*x_ref[0]**o+(0.596547791334)*x_ref[0]+(-0.0925317449093)*x_ref[1]**o+(0.751414290883)*x_ref[1]+(0.838736367666)*x_ref[2]**o+(0.735536663934)*x_ref[2]
        ref[2,1]=(-0.159026214794)*x_ref[0]**o+(0.949032823761)*x_ref[0]+(-0.422829286588)*x_ref[1]**o+(0.985896164837)*x_ref[1]+(-0.758176201469)*x_ref[2]**o+(0.283868871122)*x_ref[2]
        ref[2,2]=(-0.14639325109)*x_ref[0]**o+(0.346468853582)*x_ref[0]+(0.563105800299)*x_ref[1]**o+(-0.531036765601)*x_ref[1]+(-0.0990976207418)*x_ref[2]**o+(0.750636702702)*x_ref[2]
        ref[2,3]=(0.849599418085)*x_ref[0]**o+(-0.779136748258)*x_ref[0]+(0.730951130588)*x_ref[1]**o+(0.53628857622)*x_ref[1]+(-0.947230186497)*x_ref[2]**o+(-0.72450102135)*x_ref[2]
        ref[2,4]=(-0.700617249374)*x_ref[0]**o+(-0.10080480368)*x_ref[0]+(-0.507852228465)*x_ref[1]**o+(-0.857417192918)*x_ref[1]+(0.640292095556)*x_ref[2]**o+(-0.929256110195)*x_ref[2]
        ref[3,0]=(0.653772333481)*x_ref[0]**o+(0.000343679690948)*x_ref[0]+(0.668997254669)*x_ref[1]**o+(0.362901119538)*x_ref[1]+(-0.830987535544)*x_ref[2]**o+(-0.423720904628)*x_ref[2]
        ref[3,1]=(-0.0420273808908)*x_ref[0]**o+(0.323011823157)*x_ref[0]+(-0.24882314712)*x_ref[1]**o+(-0.423985714012)*x_ref[1]+(0.752345053106)*x_ref[2]**o+(-0.160413317211)*x_ref[2]
        ref[3,2]=(0.890620617817)*x_ref[0]**o+(0.783464544239)*x_ref[0]+(0.790371129121)*x_ref[1]**o+(0.237259212081)*x_ref[1]+(0.610840739105)*x_ref[2]**o+(0.973783326469)*x_ref[2]
        ref[3,3]=(-0.379379310577)*x_ref[0]**o+(0.690137408246)*x_ref[0]+(-0.779634423108)*x_ref[1]**o+(-0.974077688357)*x_ref[1]+(0.788216664671)*x_ref[2]**o+(0.101022285352)*x_ref[2]
        ref[3,4]=(-0.505116742845)*x_ref[0]**o+(0.923132757962)*x_ref[0]+(-0.0146493711199)*x_ref[1]**o+(0.994421781292)*x_ref[1]+(-0.20080218328)*x_ref[2]**o+(0.555271515554)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_ContinuousFunction_rank3(self):
      """
      tests interpolate for rank 3 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ContinuousFunction onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(6, 2, 2),dim=dim)
      s=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2),w_ref)
      if dim==2:
        s[0,0,0]=(-0.564155350537)*x[0]**o+(-0.800380058082)*x[0]+(0.689149742912)*x[1]**o+(-0.730209879145)*x[1]
        s[0,0,1]=(-0.875752239419)*x[0]**o+(0.620770279709)*x[0]+(-0.405669056556)*x[1]**o+(-0.650461825052)*x[1]
        s[0,1,0]=(-0.986148210247)*x[0]**o+(-0.915312621763)*x[0]+(0.96862065918)*x[1]**o+(-0.789867091704)*x[1]
        s[0,1,1]=(0.937339541643)*x[0]**o+(0.707719614445)*x[0]+(-0.0185139390834)*x[1]**o+(0.370474130738)*x[1]
        s[1,0,0]=(0.351845644061)*x[0]**o+(-0.790772449987)*x[0]+(0.446801930114)*x[1]**o+(-0.79526180504)*x[1]
        s[1,0,1]=(0.807251261875)*x[0]**o+(-0.955059117804)*x[0]+(-0.797368998645)*x[1]**o+(-0.334719758479)*x[1]
        s[1,1,0]=(0.896257416662)*x[0]**o+(-0.630142892036)*x[0]+(0.697410593951)*x[1]**o+(-0.719230278941)*x[1]
        s[1,1,1]=(-0.628604182238)*x[0]**o+(-0.98443009117)*x[0]+(0.213668228562)*x[1]**o+(0.67702641015)*x[1]
        s[2,0,0]=(0.14317864629)*x[0]**o+(-0.668340038203)*x[0]+(-0.624330217214)*x[1]**o+(-0.18225231407)*x[1]
        s[2,0,1]=(-0.120451964275)*x[0]**o+(0.476453331533)*x[0]+(0.428548558072)*x[1]**o+(0.487245871651)*x[1]
        s[2,1,0]=(0.957821516578)*x[0]**o+(0.245067538044)*x[0]+(0.59057574623)*x[1]**o+(0.799481912975)*x[1]
        s[2,1,1]=(0.157267505975)*x[0]**o+(0.486829437266)*x[0]+(0.72573406263)*x[1]**o+(-0.854913668061)*x[1]
        s[3,0,0]=(0.981080241547)*x[0]**o+(-0.475318323052)*x[0]+(-0.309438112923)*x[1]**o+(0.432416200997)*x[1]
        s[3,0,1]=(0.598520022432)*x[0]**o+(-0.384489046077)*x[0]+(-0.563263372262)*x[1]**o+(-0.787292999177)*x[1]
        s[3,1,0]=(-0.581355275119)*x[0]**o+(-0.404673888552)*x[0]+(-0.481631833004)*x[1]**o+(-0.873167338173)*x[1]
        s[3,1,1]=(-0.809669221808)*x[0]**o+(0.271025961643)*x[0]+(0.532318218814)*x[1]**o+(0.892267754107)*x[1]
        s[4,0,0]=(-0.654686292022)*x[0]**o+(-0.21708498353)*x[0]+(-0.277205071363)*x[1]**o+(-0.427753053795)*x[1]
        s[4,0,1]=(-0.540515032413)*x[0]**o+(-0.902343725594)*x[0]+(-0.880588922705)*x[1]**o+(0.449347508903)*x[1]
        s[4,1,0]=(-0.429101982872)*x[0]**o+(0.537796527897)*x[0]+(0.417004167989)*x[1]**o+(-0.0986410894762)*x[1]
        s[4,1,1]=(0.760414524096)*x[0]**o+(0.587360702148)*x[0]+(-0.977847482262)*x[1]**o+(0.233948889011)*x[1]
        s[5,0,0]=(0.489553684109)*x[0]**o+(-0.576529833035)*x[0]+(0.534315888534)*x[1]**o+(-0.27340402533)*x[1]
        s[5,0,1]=(-0.399620876418)*x[0]**o+(-0.313714298147)*x[0]+(0.180876722218)*x[1]**o+(-0.783972277649)*x[1]
        s[5,1,0]=(-0.35422230106)*x[0]**o+(0.015751912455)*x[0]+(0.10875420172)*x[1]**o+(-0.264805104189)*x[1]
        s[5,1,1]=(0.275494542521)*x[0]**o+(-0.758423085522)*x[0]+(0.223846690671)*x[1]**o+(0.717226976294)*x[1]
        ref[0,0,0]=(-0.564155350537)*x_ref[0]**o+(-0.800380058082)*x_ref[0]+(0.689149742912)*x_ref[1]**o+(-0.730209879145)*x_ref[1]
        ref[0,0,1]=(-0.875752239419)*x_ref[0]**o+(0.620770279709)*x_ref[0]+(-0.405669056556)*x_ref[1]**o+(-0.650461825052)*x_ref[1]
        ref[0,1,0]=(-0.986148210247)*x_ref[0]**o+(-0.915312621763)*x_ref[0]+(0.96862065918)*x_ref[1]**o+(-0.789867091704)*x_ref[1]
        ref[0,1,1]=(0.937339541643)*x_ref[0]**o+(0.707719614445)*x_ref[0]+(-0.0185139390834)*x_ref[1]**o+(0.370474130738)*x_ref[1]
        ref[1,0,0]=(0.351845644061)*x_ref[0]**o+(-0.790772449987)*x_ref[0]+(0.446801930114)*x_ref[1]**o+(-0.79526180504)*x_ref[1]
        ref[1,0,1]=(0.807251261875)*x_ref[0]**o+(-0.955059117804)*x_ref[0]+(-0.797368998645)*x_ref[1]**o+(-0.334719758479)*x_ref[1]
        ref[1,1,0]=(0.896257416662)*x_ref[0]**o+(-0.630142892036)*x_ref[0]+(0.697410593951)*x_ref[1]**o+(-0.719230278941)*x_ref[1]
        ref[1,1,1]=(-0.628604182238)*x_ref[0]**o+(-0.98443009117)*x_ref[0]+(0.213668228562)*x_ref[1]**o+(0.67702641015)*x_ref[1]
        ref[2,0,0]=(0.14317864629)*x_ref[0]**o+(-0.668340038203)*x_ref[0]+(-0.624330217214)*x_ref[1]**o+(-0.18225231407)*x_ref[1]
        ref[2,0,1]=(-0.120451964275)*x_ref[0]**o+(0.476453331533)*x_ref[0]+(0.428548558072)*x_ref[1]**o+(0.487245871651)*x_ref[1]
        ref[2,1,0]=(0.957821516578)*x_ref[0]**o+(0.245067538044)*x_ref[0]+(0.59057574623)*x_ref[1]**o+(0.799481912975)*x_ref[1]
        ref[2,1,1]=(0.157267505975)*x_ref[0]**o+(0.486829437266)*x_ref[0]+(0.72573406263)*x_ref[1]**o+(-0.854913668061)*x_ref[1]
        ref[3,0,0]=(0.981080241547)*x_ref[0]**o+(-0.475318323052)*x_ref[0]+(-0.309438112923)*x_ref[1]**o+(0.432416200997)*x_ref[1]
        ref[3,0,1]=(0.598520022432)*x_ref[0]**o+(-0.384489046077)*x_ref[0]+(-0.563263372262)*x_ref[1]**o+(-0.787292999177)*x_ref[1]
        ref[3,1,0]=(-0.581355275119)*x_ref[0]**o+(-0.404673888552)*x_ref[0]+(-0.481631833004)*x_ref[1]**o+(-0.873167338173)*x_ref[1]
        ref[3,1,1]=(-0.809669221808)*x_ref[0]**o+(0.271025961643)*x_ref[0]+(0.532318218814)*x_ref[1]**o+(0.892267754107)*x_ref[1]
        ref[4,0,0]=(-0.654686292022)*x_ref[0]**o+(-0.21708498353)*x_ref[0]+(-0.277205071363)*x_ref[1]**o+(-0.427753053795)*x_ref[1]
        ref[4,0,1]=(-0.540515032413)*x_ref[0]**o+(-0.902343725594)*x_ref[0]+(-0.880588922705)*x_ref[1]**o+(0.449347508903)*x_ref[1]
        ref[4,1,0]=(-0.429101982872)*x_ref[0]**o+(0.537796527897)*x_ref[0]+(0.417004167989)*x_ref[1]**o+(-0.0986410894762)*x_ref[1]
        ref[4,1,1]=(0.760414524096)*x_ref[0]**o+(0.587360702148)*x_ref[0]+(-0.977847482262)*x_ref[1]**o+(0.233948889011)*x_ref[1]
        ref[5,0,0]=(0.489553684109)*x_ref[0]**o+(-0.576529833035)*x_ref[0]+(0.534315888534)*x_ref[1]**o+(-0.27340402533)*x_ref[1]
        ref[5,0,1]=(-0.399620876418)*x_ref[0]**o+(-0.313714298147)*x_ref[0]+(0.180876722218)*x_ref[1]**o+(-0.783972277649)*x_ref[1]
        ref[5,1,0]=(-0.35422230106)*x_ref[0]**o+(0.015751912455)*x_ref[0]+(0.10875420172)*x_ref[1]**o+(-0.264805104189)*x_ref[1]
        ref[5,1,1]=(0.275494542521)*x_ref[0]**o+(-0.758423085522)*x_ref[0]+(0.223846690671)*x_ref[1]**o+(0.717226976294)*x_ref[1]
      else:
        s[0,0,0]=(-0.774622440694)*x[0]**o+(-0.871495007805)*x[0]+(0.462893985101)*x[1]**o+(0.97809884411)*x[1]+(-0.294782632822)*x[2]**o+(0.758086059139)*x[2]
        s[0,0,1]=(0.593542531653)*x[0]**o+(-0.623508645258)*x[0]+(-0.869924719996)*x[1]**o+(0.683987028818)*x[1]+(0.214983363659)*x[2]**o+(0.937703017041)*x[2]
        s[0,1,0]=(0.683415745305)*x[0]**o+(0.514284128555)*x[0]+(-0.495949103899)*x[1]**o+(0.0846543741985)*x[1]+(0.713593706483)*x[2]**o+(0.68727976891)*x[2]
        s[0,1,1]=(0.890309435361)*x[0]**o+(-0.971477359218)*x[0]+(0.832140831851)*x[1]**o+(-0.208499413601)*x[1]+(0.632915577047)*x[2]**o+(0.255341853742)*x[2]
        s[1,0,0]=(0.180392491602)*x[0]**o+(-0.535445127886)*x[0]+(-0.785876938514)*x[1]**o+(0.0816315522405)*x[1]+(0.648551332906)*x[2]**o+(0.0429567512952)*x[2]
        s[1,0,1]=(-0.662744890855)*x[0]**o+(0.961507967025)*x[0]+(-0.535104990223)*x[1]**o+(-0.100555677207)*x[1]+(0.254286176467)*x[2]**o+(0.23822457275)*x[2]
        s[1,1,0]=(0.0618685350699)*x[0]**o+(0.801588371288)*x[0]+(0.775832395529)*x[1]**o+(0.76315781257)*x[1]+(0.201501889875)*x[2]**o+(-0.594080987915)*x[2]
        s[1,1,1]=(-0.472648941872)*x[0]**o+(-0.268595087901)*x[0]+(-0.634032640513)*x[1]**o+(0.895658995237)*x[1]+(-0.311243350516)*x[2]**o+(-0.941899709416)*x[2]
        s[2,0,0]=(-0.937843815317)*x[0]**o+(-0.732052451935)*x[0]+(-0.195729427352)*x[1]**o+(-0.686479979743)*x[1]+(0.233916070094)*x[2]**o+(-0.0840356688737)*x[2]
        s[2,0,1]=(0.010530087739)*x[0]**o+(-0.790921976555)*x[0]+(0.0748807338793)*x[1]**o+(-0.903727762342)*x[1]+(0.627134883108)*x[2]**o+(-0.0387413340061)*x[2]
        s[2,1,0]=(0.546735016177)*x[0]**o+(0.315849018818)*x[0]+(-0.95747551189)*x[1]**o+(0.0565452708853)*x[1]+(0.305498701829)*x[2]**o+(0.122314734589)*x[2]
        s[2,1,1]=(-0.838026718661)*x[0]**o+(0.128953039794)*x[0]+(0.624067160575)*x[1]**o+(0.444434080889)*x[1]+(-0.835923583772)*x[2]**o+(-0.286578073481)*x[2]
        s[3,0,0]=(0.00360603498589)*x[0]**o+(0.842043331421)*x[0]+(0.362497876527)*x[1]**o+(0.0681458369022)*x[1]+(0.75852693193)*x[2]**o+(-0.53716235)*x[2]
        s[3,0,1]=(-0.159678757928)*x[0]**o+(0.883308054535)*x[0]+(0.21591905568)*x[1]**o+(-0.898265348642)*x[1]+(-0.951423349936)*x[2]**o+(-0.240034084504)*x[2]
        s[3,1,0]=(0.692230687386)*x[0]**o+(0.0335587781762)*x[0]+(-0.766571197965)*x[1]**o+(0.128927596751)*x[1]+(0.986505471196)*x[2]**o+(0.149177674029)*x[2]
        s[3,1,1]=(0.135591222661)*x[0]**o+(0.931657866742)*x[0]+(-0.4633142936)*x[1]**o+(0.868618051626)*x[1]+(-0.563215256065)*x[2]**o+(0.937159665351)*x[2]
        s[4,0,0]=(-0.00496897513325)*x[0]**o+(0.64627613088)*x[0]+(-0.63362415209)*x[1]**o+(0.875299573404)*x[1]+(0.00915422996533)*x[2]**o+(-0.898719885994)*x[2]
        s[4,0,1]=(0.531986621993)*x[0]**o+(-0.592503622446)*x[0]+(0.168725128639)*x[1]**o+(-0.415718155368)*x[1]+(0.153807804669)*x[2]**o+(0.0517662389301)*x[2]
        s[4,1,0]=(-0.40167686609)*x[0]**o+(0.765205907898)*x[0]+(-0.330400749167)*x[1]**o+(-0.227287643007)*x[1]+(0.86248041007)*x[2]**o+(0.475472570688)*x[2]
        s[4,1,1]=(-0.379429448909)*x[0]**o+(-0.746283001238)*x[0]+(0.451902471866)*x[1]**o+(0.975850402176)*x[1]+(-0.752666040136)*x[2]**o+(0.310437725938)*x[2]
        s[5,0,0]=(-0.790243943683)*x[0]**o+(-0.483698083361)*x[0]+(0.298354468428)*x[1]**o+(-0.430878231617)*x[1]+(-0.970790186654)*x[2]**o+(0.295573821695)*x[2]
        s[5,0,1]=(0.342850991179)*x[0]**o+(0.716485409508)*x[0]+(0.0929941902588)*x[1]**o+(-0.804594470152)*x[1]+(-0.6262904765)*x[2]**o+(0.1281559082)*x[2]
        s[5,1,0]=(0.6989819417)*x[0]**o+(0.513436682661)*x[0]+(-0.398488542709)*x[1]**o+(0.429421472572)*x[1]+(0.0491769595898)*x[2]**o+(0.402704297546)*x[2]
        s[5,1,1]=(-0.638592369916)*x[0]**o+(0.164631869734)*x[0]+(0.306302338543)*x[1]**o+(0.293774443655)*x[1]+(-0.536801505818)*x[2]**o+(0.758326129903)*x[2]
        ref[0,0,0]=(-0.774622440694)*x_ref[0]**o+(-0.871495007805)*x_ref[0]+(0.462893985101)*x_ref[1]**o+(0.97809884411)*x_ref[1]+(-0.294782632822)*x_ref[2]**o+(0.758086059139)*x_ref[2]
        ref[0,0,1]=(0.593542531653)*x_ref[0]**o+(-0.623508645258)*x_ref[0]+(-0.869924719996)*x_ref[1]**o+(0.683987028818)*x_ref[1]+(0.214983363659)*x_ref[2]**o+(0.937703017041)*x_ref[2]
        ref[0,1,0]=(0.683415745305)*x_ref[0]**o+(0.514284128555)*x_ref[0]+(-0.495949103899)*x_ref[1]**o+(0.0846543741985)*x_ref[1]+(0.713593706483)*x_ref[2]**o+(0.68727976891)*x_ref[2]
        ref[0,1,1]=(0.890309435361)*x_ref[0]**o+(-0.971477359218)*x_ref[0]+(0.832140831851)*x_ref[1]**o+(-0.208499413601)*x_ref[1]+(0.632915577047)*x_ref[2]**o+(0.255341853742)*x_ref[2]
        ref[1,0,0]=(0.180392491602)*x_ref[0]**o+(-0.535445127886)*x_ref[0]+(-0.785876938514)*x_ref[1]**o+(0.0816315522405)*x_ref[1]+(0.648551332906)*x_ref[2]**o+(0.0429567512952)*x_ref[2]
        ref[1,0,1]=(-0.662744890855)*x_ref[0]**o+(0.961507967025)*x_ref[0]+(-0.535104990223)*x_ref[1]**o+(-0.100555677207)*x_ref[1]+(0.254286176467)*x_ref[2]**o+(0.23822457275)*x_ref[2]
        ref[1,1,0]=(0.0618685350699)*x_ref[0]**o+(0.801588371288)*x_ref[0]+(0.775832395529)*x_ref[1]**o+(0.76315781257)*x_ref[1]+(0.201501889875)*x_ref[2]**o+(-0.594080987915)*x_ref[2]
        ref[1,1,1]=(-0.472648941872)*x_ref[0]**o+(-0.268595087901)*x_ref[0]+(-0.634032640513)*x_ref[1]**o+(0.895658995237)*x_ref[1]+(-0.311243350516)*x_ref[2]**o+(-0.941899709416)*x_ref[2]
        ref[2,0,0]=(-0.937843815317)*x_ref[0]**o+(-0.732052451935)*x_ref[0]+(-0.195729427352)*x_ref[1]**o+(-0.686479979743)*x_ref[1]+(0.233916070094)*x_ref[2]**o+(-0.0840356688737)*x_ref[2]
        ref[2,0,1]=(0.010530087739)*x_ref[0]**o+(-0.790921976555)*x_ref[0]+(0.0748807338793)*x_ref[1]**o+(-0.903727762342)*x_ref[1]+(0.627134883108)*x_ref[2]**o+(-0.0387413340061)*x_ref[2]
        ref[2,1,0]=(0.546735016177)*x_ref[0]**o+(0.315849018818)*x_ref[0]+(-0.95747551189)*x_ref[1]**o+(0.0565452708853)*x_ref[1]+(0.305498701829)*x_ref[2]**o+(0.122314734589)*x_ref[2]
        ref[2,1,1]=(-0.838026718661)*x_ref[0]**o+(0.128953039794)*x_ref[0]+(0.624067160575)*x_ref[1]**o+(0.444434080889)*x_ref[1]+(-0.835923583772)*x_ref[2]**o+(-0.286578073481)*x_ref[2]
        ref[3,0,0]=(0.00360603498589)*x_ref[0]**o+(0.842043331421)*x_ref[0]+(0.362497876527)*x_ref[1]**o+(0.0681458369022)*x_ref[1]+(0.75852693193)*x_ref[2]**o+(-0.53716235)*x_ref[2]
        ref[3,0,1]=(-0.159678757928)*x_ref[0]**o+(0.883308054535)*x_ref[0]+(0.21591905568)*x_ref[1]**o+(-0.898265348642)*x_ref[1]+(-0.951423349936)*x_ref[2]**o+(-0.240034084504)*x_ref[2]
        ref[3,1,0]=(0.692230687386)*x_ref[0]**o+(0.0335587781762)*x_ref[0]+(-0.766571197965)*x_ref[1]**o+(0.128927596751)*x_ref[1]+(0.986505471196)*x_ref[2]**o+(0.149177674029)*x_ref[2]
        ref[3,1,1]=(0.135591222661)*x_ref[0]**o+(0.931657866742)*x_ref[0]+(-0.4633142936)*x_ref[1]**o+(0.868618051626)*x_ref[1]+(-0.563215256065)*x_ref[2]**o+(0.937159665351)*x_ref[2]
        ref[4,0,0]=(-0.00496897513325)*x_ref[0]**o+(0.64627613088)*x_ref[0]+(-0.63362415209)*x_ref[1]**o+(0.875299573404)*x_ref[1]+(0.00915422996533)*x_ref[2]**o+(-0.898719885994)*x_ref[2]
        ref[4,0,1]=(0.531986621993)*x_ref[0]**o+(-0.592503622446)*x_ref[0]+(0.168725128639)*x_ref[1]**o+(-0.415718155368)*x_ref[1]+(0.153807804669)*x_ref[2]**o+(0.0517662389301)*x_ref[2]
        ref[4,1,0]=(-0.40167686609)*x_ref[0]**o+(0.765205907898)*x_ref[0]+(-0.330400749167)*x_ref[1]**o+(-0.227287643007)*x_ref[1]+(0.86248041007)*x_ref[2]**o+(0.475472570688)*x_ref[2]
        ref[4,1,1]=(-0.379429448909)*x_ref[0]**o+(-0.746283001238)*x_ref[0]+(0.451902471866)*x_ref[1]**o+(0.975850402176)*x_ref[1]+(-0.752666040136)*x_ref[2]**o+(0.310437725938)*x_ref[2]
        ref[5,0,0]=(-0.790243943683)*x_ref[0]**o+(-0.483698083361)*x_ref[0]+(0.298354468428)*x_ref[1]**o+(-0.430878231617)*x_ref[1]+(-0.970790186654)*x_ref[2]**o+(0.295573821695)*x_ref[2]
        ref[5,0,1]=(0.342850991179)*x_ref[0]**o+(0.716485409508)*x_ref[0]+(0.0929941902588)*x_ref[1]**o+(-0.804594470152)*x_ref[1]+(-0.6262904765)*x_ref[2]**o+(0.1281559082)*x_ref[2]
        ref[5,1,0]=(0.6989819417)*x_ref[0]**o+(0.513436682661)*x_ref[0]+(-0.398488542709)*x_ref[1]**o+(0.429421472572)*x_ref[1]+(0.0491769595898)*x_ref[2]**o+(0.402704297546)*x_ref[2]
        ref[5,1,1]=(-0.638592369916)*x_ref[0]**o+(0.164631869734)*x_ref[0]+(0.306302338543)*x_ref[1]**o+(0.293774443655)*x_ref[1]+(-0.536801505818)*x_ref[2]**o+(0.758326129903)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_ContinuousFunction_rank4(self):
      """
      tests interpolate for rank 4 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ContinuousFunction onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5, 3, 2),dim=dim)
      s=Data(0,(4, 5, 3, 2),w)
      ref=Data(0,(4, 5, 3, 2),w_ref)
      if dim==2:
        s[0,0,0,0]=(-0.474386706114)*x[0]**o+(-0.0739442523179)*x[0]+(-0.934839682165)*x[1]**o+(-0.874748715361)*x[1]
        s[0,0,0,1]=(0.970835666471)*x[0]**o+(-0.210304396563)*x[0]+(-0.245577454075)*x[1]**o+(0.706874293911)*x[1]
        s[0,0,1,0]=(-0.886508211386)*x[0]**o+(0.40749471331)*x[0]+(-0.575539262859)*x[1]**o+(-0.714242182567)*x[1]
        s[0,0,1,1]=(0.0925249369772)*x[0]**o+(-0.326000469119)*x[0]+(0.427385578633)*x[1]**o+(0.723913449012)*x[1]
        s[0,0,2,0]=(0.178301174425)*x[0]**o+(-0.935408769107)*x[0]+(0.185756314077)*x[1]**o+(0.779639226265)*x[1]
        s[0,0,2,1]=(0.517627638569)*x[0]**o+(0.833228507097)*x[0]+(0.509792014643)*x[1]**o+(0.512759085233)*x[1]
        s[0,1,0,0]=(0.943220804246)*x[0]**o+(0.358695377859)*x[0]+(0.483282522344)*x[1]**o+(-0.780850766978)*x[1]
        s[0,1,0,1]=(0.228281969805)*x[0]**o+(0.378112450475)*x[0]+(-0.265569820367)*x[1]**o+(0.793063241358)*x[1]
        s[0,1,1,0]=(-0.991239767457)*x[0]**o+(0.584083573937)*x[0]+(0.00907947673395)*x[1]**o+(-0.0949125419342)*x[1]
        s[0,1,1,1]=(0.312976776903)*x[0]**o+(-0.572058365439)*x[0]+(0.337493203667)*x[1]**o+(-0.559749276661)*x[1]
        s[0,1,2,0]=(-0.154473081077)*x[0]**o+(0.946407875413)*x[0]+(0.721108882799)*x[1]**o+(0.409146005045)*x[1]
        s[0,1,2,1]=(0.42027715889)*x[0]**o+(-0.892090996743)*x[0]+(-0.718807145454)*x[1]**o+(0.229975539898)*x[1]
        s[0,2,0,0]=(0.378117082397)*x[0]**o+(-0.613916810385)*x[0]+(-0.564348246694)*x[1]**o+(0.101261637507)*x[1]
        s[0,2,0,1]=(0.839696159524)*x[0]**o+(0.24184619932)*x[0]+(-0.095338864129)*x[1]**o+(0.903106275587)*x[1]
        s[0,2,1,0]=(0.620479230354)*x[0]**o+(-0.34112105179)*x[0]+(0.624902166524)*x[1]**o+(-0.693289910121)*x[1]
        s[0,2,1,1]=(-0.39760464469)*x[0]**o+(-0.921383269336)*x[0]+(0.753657132321)*x[1]**o+(0.324768021371)*x[1]
        s[0,2,2,0]=(0.0243534816004)*x[0]**o+(0.290663121221)*x[0]+(0.0305666628617)*x[1]**o+(0.0446693318891)*x[1]
        s[0,2,2,1]=(-0.14154275886)*x[0]**o+(0.881207887999)*x[0]+(0.399029780289)*x[1]**o+(0.168497381665)*x[1]
        s[0,3,0,0]=(-0.233073561791)*x[0]**o+(-0.513212321666)*x[0]+(-0.651480028725)*x[1]**o+(0.147498573261)*x[1]
        s[0,3,0,1]=(-0.918136406734)*x[0]**o+(-0.279161242114)*x[0]+(0.983432543886)*x[1]**o+(-0.723257302563)*x[1]
        s[0,3,1,0]=(0.290070399137)*x[0]**o+(0.485832779323)*x[0]+(0.955571801349)*x[1]**o+(0.552993034163)*x[1]
        s[0,3,1,1]=(0.688570077213)*x[0]**o+(0.578716466405)*x[0]+(-0.40829418423)*x[1]**o+(-0.918196317732)*x[1]
        s[0,3,2,0]=(0.403069347997)*x[0]**o+(0.119390454784)*x[0]+(-0.322018782913)*x[1]**o+(0.224814772968)*x[1]
        s[0,3,2,1]=(-0.23084264112)*x[0]**o+(-0.398235674194)*x[0]+(-0.730556055687)*x[1]**o+(0.293981158644)*x[1]
        s[0,4,0,0]=(-0.0984201873925)*x[0]**o+(0.018359257988)*x[0]+(-0.0781876875229)*x[1]**o+(0.253344618889)*x[1]
        s[0,4,0,1]=(-0.20899724864)*x[0]**o+(-0.722019204308)*x[0]+(0.555162582715)*x[1]**o+(-0.958068826429)*x[1]
        s[0,4,1,0]=(0.477725976915)*x[0]**o+(0.295872229679)*x[0]+(-0.828208555398)*x[1]**o+(-0.931582818519)*x[1]
        s[0,4,1,1]=(0.721285453077)*x[0]**o+(-0.042927949943)*x[0]+(0.680231405034)*x[1]**o+(-0.16043805248)*x[1]
        s[0,4,2,0]=(0.985988751136)*x[0]**o+(0.885785903078)*x[0]+(-0.0410955856477)*x[1]**o+(-0.552089008201)*x[1]
        s[0,4,2,1]=(0.847466818225)*x[0]**o+(-0.51057282448)*x[0]+(0.163775791256)*x[1]**o+(0.0152553094575)*x[1]
        s[1,0,0,0]=(0.0343855486006)*x[0]**o+(-0.214334464898)*x[0]+(-0.573344070879)*x[1]**o+(0.24905956136)*x[1]
        s[1,0,0,1]=(0.783338978316)*x[0]**o+(0.781324842059)*x[0]+(-0.847509450777)*x[1]**o+(-0.858994488444)*x[1]
        s[1,0,1,0]=(0.59719044885)*x[0]**o+(0.347038111035)*x[0]+(-0.385761256521)*x[1]**o+(-0.571166989318)*x[1]
        s[1,0,1,1]=(0.636911008507)*x[0]**o+(-0.825376916398)*x[0]+(0.32394534216)*x[1]**o+(0.653119502004)*x[1]
        s[1,0,2,0]=(-0.669732532783)*x[0]**o+(-0.145427031253)*x[0]+(-0.303693857831)*x[1]**o+(-0.727878524833)*x[1]
        s[1,0,2,1]=(0.621061585764)*x[0]**o+(0.243432217146)*x[0]+(-0.888610099037)*x[1]**o+(0.298051589144)*x[1]
        s[1,1,0,0]=(0.357341721813)*x[0]**o+(0.346506498237)*x[0]+(-0.555625437961)*x[1]**o+(-0.0662579456147)*x[1]
        s[1,1,0,1]=(0.96955420495)*x[0]**o+(-0.190486134786)*x[0]+(-0.900124677941)*x[1]**o+(0.659993963277)*x[1]
        s[1,1,1,0]=(-0.517837108406)*x[0]**o+(0.00779665389526)*x[0]+(-0.0544022169324)*x[1]**o+(0.888217855388)*x[1]
        s[1,1,1,1]=(0.252515646862)*x[0]**o+(0.328948655235)*x[0]+(0.265916563145)*x[1]**o+(-0.491865215095)*x[1]
        s[1,1,2,0]=(0.922520219928)*x[0]**o+(0.463811222486)*x[0]+(0.651244201962)*x[1]**o+(0.982565677231)*x[1]
        s[1,1,2,1]=(0.257711278265)*x[0]**o+(0.23971381197)*x[0]+(-0.168559977915)*x[1]**o+(0.453909616568)*x[1]
        s[1,2,0,0]=(0.0426933948917)*x[0]**o+(-0.477381944712)*x[0]+(0.503797983762)*x[1]**o+(-0.943092701839)*x[1]
        s[1,2,0,1]=(0.913591544613)*x[0]**o+(-0.895063818412)*x[0]+(0.252264576564)*x[1]**o+(0.424846745491)*x[1]
        s[1,2,1,0]=(0.4199934247)*x[0]**o+(-0.128668379749)*x[0]+(-0.157997092493)*x[1]**o+(-0.174476338072)*x[1]
        s[1,2,1,1]=(0.394310091468)*x[0]**o+(0.599539798636)*x[0]+(0.133087615453)*x[1]**o+(-0.22271509754)*x[1]
        s[1,2,2,0]=(0.559521330007)*x[0]**o+(-0.411300693025)*x[0]+(-0.543921885179)*x[1]**o+(-0.0506834937614)*x[1]
        s[1,2,2,1]=(-0.423996886775)*x[0]**o+(0.14409349326)*x[0]+(-0.970632608145)*x[1]**o+(0.582937355294)*x[1]
        s[1,3,0,0]=(-0.884972026022)*x[0]**o+(-0.872260393893)*x[0]+(0.224463013859)*x[1]**o+(-0.245013144519)*x[1]
        s[1,3,0,1]=(-0.834873836296)*x[0]**o+(-0.778611513205)*x[0]+(0.989113281842)*x[1]**o+(0.847047912646)*x[1]
        s[1,3,1,0]=(-0.0900703847239)*x[0]**o+(-0.0295919916877)*x[0]+(0.510380741505)*x[1]**o+(-0.00354494905936)*x[1]
        s[1,3,1,1]=(0.0422103052661)*x[0]**o+(0.31376324435)*x[0]+(0.831890412519)*x[1]**o+(0.764406560706)*x[1]
        s[1,3,2,0]=(0.496699862489)*x[0]**o+(0.477514051712)*x[0]+(0.156435695163)*x[1]**o+(0.339443828388)*x[1]
        s[1,3,2,1]=(0.396689832399)*x[0]**o+(0.243605505552)*x[0]+(-0.0357728699149)*x[1]**o+(0.930734881576)*x[1]
        s[1,4,0,0]=(-0.90788627878)*x[0]**o+(0.82597715373)*x[0]+(0.287988132693)*x[1]**o+(-0.834307302134)*x[1]
        s[1,4,0,1]=(-0.942882799669)*x[0]**o+(-0.149609905045)*x[0]+(-0.340025012164)*x[1]**o+(0.539546162654)*x[1]
        s[1,4,1,0]=(-0.244377326727)*x[0]**o+(-0.586439384775)*x[0]+(0.89506494604)*x[1]**o+(0.233226803834)*x[1]
        s[1,4,1,1]=(0.864582224514)*x[0]**o+(0.0687754700765)*x[0]+(-0.641556536115)*x[1]**o+(0.22036806322)*x[1]
        s[1,4,2,0]=(-0.552589117461)*x[0]**o+(-0.705244093999)*x[0]+(0.378837921567)*x[1]**o+(-0.173968977986)*x[1]
        s[1,4,2,1]=(0.846900000181)*x[0]**o+(-0.52589472084)*x[0]+(0.872400573242)*x[1]**o+(0.182683546911)*x[1]
        s[2,0,0,0]=(0.305006901177)*x[0]**o+(-0.732858762152)*x[0]+(-0.882239786812)*x[1]**o+(0.481712295538)*x[1]
        s[2,0,0,1]=(0.836332373747)*x[0]**o+(0.63905574965)*x[0]+(0.039226709213)*x[1]**o+(0.704758586936)*x[1]
        s[2,0,1,0]=(0.51805564252)*x[0]**o+(0.641883965761)*x[0]+(0.901663292208)*x[1]**o+(0.375272715842)*x[1]
        s[2,0,1,1]=(-0.980058913052)*x[0]**o+(0.385619663167)*x[0]+(0.0722192483217)*x[1]**o+(0.00650731529315)*x[1]
        s[2,0,2,0]=(-0.25791454536)*x[0]**o+(0.460015385804)*x[0]+(0.234970544559)*x[1]**o+(-0.11962323176)*x[1]
        s[2,0,2,1]=(0.023390322577)*x[0]**o+(-0.104449150638)*x[0]+(0.149724956284)*x[1]**o+(-0.309848596985)*x[1]
        s[2,1,0,0]=(0.603565629307)*x[0]**o+(-0.143206239367)*x[0]+(-0.0430666954645)*x[1]**o+(-0.154546675756)*x[1]
        s[2,1,0,1]=(-0.980112660961)*x[0]**o+(0.419145835585)*x[0]+(-0.76202669209)*x[1]**o+(-0.648615698697)*x[1]
        s[2,1,1,0]=(0.805964945679)*x[0]**o+(0.650144650504)*x[0]+(0.308110033125)*x[1]**o+(0.0666476104838)*x[1]
        s[2,1,1,1]=(0.42561398203)*x[0]**o+(0.259276259654)*x[0]+(-0.215194398301)*x[1]**o+(0.880282506885)*x[1]
        s[2,1,2,0]=(-0.413779899229)*x[0]**o+(-0.796859182258)*x[0]+(0.606865226329)*x[1]**o+(-0.783313517188)*x[1]
        s[2,1,2,1]=(0.402767439742)*x[0]**o+(-0.692468937875)*x[0]+(0.27866251803)*x[1]**o+(0.95092559773)*x[1]
        s[2,2,0,0]=(0.749107532378)*x[0]**o+(-0.364633672479)*x[0]+(0.970710337513)*x[1]**o+(0.1986749977)*x[1]
        s[2,2,0,1]=(-0.722832876805)*x[0]**o+(-0.238825576333)*x[0]+(-0.101752751414)*x[1]**o+(-0.960067435044)*x[1]
        s[2,2,1,0]=(-0.704825996403)*x[0]**o+(0.74244530617)*x[0]+(-0.586571188279)*x[1]**o+(0.841297962355)*x[1]
        s[2,2,1,1]=(0.586127464995)*x[0]**o+(0.454050196541)*x[0]+(0.225294694159)*x[1]**o+(0.705925824031)*x[1]
        s[2,2,2,0]=(0.535446366333)*x[0]**o+(-0.787565531495)*x[0]+(-0.566053725536)*x[1]**o+(0.468968266813)*x[1]
        s[2,2,2,1]=(0.449236149776)*x[0]**o+(0.589799099557)*x[0]+(0.189540632578)*x[1]**o+(-0.775036139482)*x[1]
        s[2,3,0,0]=(0.979572074948)*x[0]**o+(-0.806420343846)*x[0]+(0.661682487968)*x[1]**o+(-0.615846811222)*x[1]
        s[2,3,0,1]=(0.021072244205)*x[0]**o+(0.878344628134)*x[0]+(-0.381067816502)*x[1]**o+(0.229027446937)*x[1]
        s[2,3,1,0]=(0.0182291979593)*x[0]**o+(0.625595584968)*x[0]+(-0.824161018488)*x[1]**o+(-0.316191235785)*x[1]
        s[2,3,1,1]=(-0.0168799900253)*x[0]**o+(-0.137230078645)*x[0]+(0.0240280311645)*x[1]**o+(-0.364099905801)*x[1]
        s[2,3,2,0]=(0.374193672582)*x[0]**o+(0.429814892173)*x[0]+(0.0194417431204)*x[1]**o+(0.830453174612)*x[1]
        s[2,3,2,1]=(-0.683758441576)*x[0]**o+(-0.759635036652)*x[0]+(-0.50102074202)*x[1]**o+(0.123764523628)*x[1]
        s[2,4,0,0]=(-0.0817296101008)*x[0]**o+(0.730474427452)*x[0]+(-0.0733544873209)*x[1]**o+(0.416303527556)*x[1]
        s[2,4,0,1]=(0.258598296267)*x[0]**o+(0.686740404407)*x[0]+(0.543414259362)*x[1]**o+(0.518071987904)*x[1]
        s[2,4,1,0]=(-0.32475099592)*x[0]**o+(-0.0460901003729)*x[0]+(0.275611466137)*x[1]**o+(0.8415303665)*x[1]
        s[2,4,1,1]=(0.5015310862)*x[0]**o+(-0.71451126551)*x[0]+(0.897703909295)*x[1]**o+(-0.0476780709524)*x[1]
        s[2,4,2,0]=(0.612722779576)*x[0]**o+(0.424485459817)*x[0]+(-0.136026922168)*x[1]**o+(0.16727850944)*x[1]
        s[2,4,2,1]=(-0.841159017194)*x[0]**o+(0.856949657441)*x[0]+(-0.975786438976)*x[1]**o+(-0.407229683889)*x[1]
        s[3,0,0,0]=(0.833459943775)*x[0]**o+(-0.230178834842)*x[0]+(-0.660801597291)*x[1]**o+(-0.738869823127)*x[1]
        s[3,0,0,1]=(0.30987970183)*x[0]**o+(-0.610707485471)*x[0]+(0.628798231358)*x[1]**o+(-0.0491116840572)*x[1]
        s[3,0,1,0]=(0.649739974341)*x[0]**o+(-0.450666468651)*x[0]+(0.901638992166)*x[1]**o+(-0.787549067401)*x[1]
        s[3,0,1,1]=(0.934583955853)*x[0]**o+(-0.859086605977)*x[0]+(0.0975143570756)*x[1]**o+(0.0350097808797)*x[1]
        s[3,0,2,0]=(0.837723849965)*x[0]**o+(-0.665696123522)*x[0]+(-0.70629678851)*x[1]**o+(-0.468945853667)*x[1]
        s[3,0,2,1]=(0.526302645633)*x[0]**o+(-0.0979568077223)*x[0]+(-0.767623484018)*x[1]**o+(-0.21912588217)*x[1]
        s[3,1,0,0]=(0.18832505785)*x[0]**o+(0.45004009293)*x[0]+(-0.823702397711)*x[1]**o+(-0.912642057295)*x[1]
        s[3,1,0,1]=(0.554483721388)*x[0]**o+(-0.607925140409)*x[0]+(-0.998308584629)*x[1]**o+(-0.538184347898)*x[1]
        s[3,1,1,0]=(-0.682363035722)*x[0]**o+(0.55318030401)*x[0]+(0.562408350319)*x[1]**o+(0.220145757421)*x[1]
        s[3,1,1,1]=(0.924816884714)*x[0]**o+(-0.295767203325)*x[0]+(0.230343937118)*x[1]**o+(0.957747143338)*x[1]
        s[3,1,2,0]=(0.235422171194)*x[0]**o+(0.264750603603)*x[0]+(0.13053017764)*x[1]**o+(0.628805980195)*x[1]
        s[3,1,2,1]=(-0.603111233811)*x[0]**o+(-0.325257245906)*x[0]+(-0.879402225173)*x[1]**o+(0.977736365104)*x[1]
        s[3,2,0,0]=(0.463242222753)*x[0]**o+(-0.802230749276)*x[0]+(0.085387156258)*x[1]**o+(0.18309223952)*x[1]
        s[3,2,0,1]=(0.778060805693)*x[0]**o+(0.32136550684)*x[0]+(0.775284717696)*x[1]**o+(0.151449430741)*x[1]
        s[3,2,1,0]=(0.288473735099)*x[0]**o+(0.675142038399)*x[0]+(-0.313098988069)*x[1]**o+(-0.771883975974)*x[1]
        s[3,2,1,1]=(0.715304130316)*x[0]**o+(0.994375393616)*x[0]+(-0.874667784061)*x[1]**o+(-0.29510377129)*x[1]
        s[3,2,2,0]=(-0.211496384269)*x[0]**o+(0.146746732763)*x[0]+(0.234317983556)*x[1]**o+(0.844710015939)*x[1]
        s[3,2,2,1]=(0.557426563891)*x[0]**o+(0.181380049852)*x[0]+(-0.068830884747)*x[1]**o+(0.424778582562)*x[1]
        s[3,3,0,0]=(-0.784389821334)*x[0]**o+(0.533073578323)*x[0]+(-0.172622226211)*x[1]**o+(-0.0882056556263)*x[1]
        s[3,3,0,1]=(-0.369034971239)*x[0]**o+(-0.323409858793)*x[0]+(0.408766889438)*x[1]**o+(-0.653470515807)*x[1]
        s[3,3,1,0]=(0.239411265848)*x[0]**o+(-0.672719205684)*x[0]+(-0.751715345243)*x[1]**o+(-0.4586462848)*x[1]
        s[3,3,1,1]=(-0.937724235502)*x[0]**o+(-0.9284117207)*x[0]+(-0.584507468424)*x[1]**o+(0.779562243368)*x[1]
        s[3,3,2,0]=(-0.563406066207)*x[0]**o+(0.115504525413)*x[0]+(0.0230187346201)*x[1]**o+(0.960113641875)*x[1]
        s[3,3,2,1]=(0.792686350755)*x[0]**o+(0.560920910518)*x[0]+(0.0766597516323)*x[1]**o+(-0.728345800422)*x[1]
        s[3,4,0,0]=(-0.182383283107)*x[0]**o+(0.604818623768)*x[0]+(0.756868334363)*x[1]**o+(0.0259669270239)*x[1]
        s[3,4,0,1]=(0.547592534695)*x[0]**o+(0.666130932562)*x[0]+(-0.922839241546)*x[1]**o+(-0.143547846716)*x[1]
        s[3,4,1,0]=(-0.43295353947)*x[0]**o+(0.342981294189)*x[0]+(0.785780092257)*x[1]**o+(0.480176709076)*x[1]
        s[3,4,1,1]=(-0.859691385907)*x[0]**o+(0.124338939229)*x[0]+(-0.883988405006)*x[1]**o+(0.0683789905855)*x[1]
        s[3,4,2,0]=(-0.106494101633)*x[0]**o+(0.276437588849)*x[0]+(0.408900924262)*x[1]**o+(-0.479798281555)*x[1]
        s[3,4,2,1]=(-0.226633020661)*x[0]**o+(0.487277344856)*x[0]+(-0.0241726779927)*x[1]**o+(-0.778823212273)*x[1]
        ref[0,0,0,0]=(-0.474386706114)*x_ref[0]**o+(-0.0739442523179)*x_ref[0]+(-0.934839682165)*x_ref[1]**o+(-0.874748715361)*x_ref[1]
        ref[0,0,0,1]=(0.970835666471)*x_ref[0]**o+(-0.210304396563)*x_ref[0]+(-0.245577454075)*x_ref[1]**o+(0.706874293911)*x_ref[1]
        ref[0,0,1,0]=(-0.886508211386)*x_ref[0]**o+(0.40749471331)*x_ref[0]+(-0.575539262859)*x_ref[1]**o+(-0.714242182567)*x_ref[1]
        ref[0,0,1,1]=(0.0925249369772)*x_ref[0]**o+(-0.326000469119)*x_ref[0]+(0.427385578633)*x_ref[1]**o+(0.723913449012)*x_ref[1]
        ref[0,0,2,0]=(0.178301174425)*x_ref[0]**o+(-0.935408769107)*x_ref[0]+(0.185756314077)*x_ref[1]**o+(0.779639226265)*x_ref[1]
        ref[0,0,2,1]=(0.517627638569)*x_ref[0]**o+(0.833228507097)*x_ref[0]+(0.509792014643)*x_ref[1]**o+(0.512759085233)*x_ref[1]
        ref[0,1,0,0]=(0.943220804246)*x_ref[0]**o+(0.358695377859)*x_ref[0]+(0.483282522344)*x_ref[1]**o+(-0.780850766978)*x_ref[1]
        ref[0,1,0,1]=(0.228281969805)*x_ref[0]**o+(0.378112450475)*x_ref[0]+(-0.265569820367)*x_ref[1]**o+(0.793063241358)*x_ref[1]
        ref[0,1,1,0]=(-0.991239767457)*x_ref[0]**o+(0.584083573937)*x_ref[0]+(0.00907947673395)*x_ref[1]**o+(-0.0949125419342)*x_ref[1]
        ref[0,1,1,1]=(0.312976776903)*x_ref[0]**o+(-0.572058365439)*x_ref[0]+(0.337493203667)*x_ref[1]**o+(-0.559749276661)*x_ref[1]
        ref[0,1,2,0]=(-0.154473081077)*x_ref[0]**o+(0.946407875413)*x_ref[0]+(0.721108882799)*x_ref[1]**o+(0.409146005045)*x_ref[1]
        ref[0,1,2,1]=(0.42027715889)*x_ref[0]**o+(-0.892090996743)*x_ref[0]+(-0.718807145454)*x_ref[1]**o+(0.229975539898)*x_ref[1]
        ref[0,2,0,0]=(0.378117082397)*x_ref[0]**o+(-0.613916810385)*x_ref[0]+(-0.564348246694)*x_ref[1]**o+(0.101261637507)*x_ref[1]
        ref[0,2,0,1]=(0.839696159524)*x_ref[0]**o+(0.24184619932)*x_ref[0]+(-0.095338864129)*x_ref[1]**o+(0.903106275587)*x_ref[1]
        ref[0,2,1,0]=(0.620479230354)*x_ref[0]**o+(-0.34112105179)*x_ref[0]+(0.624902166524)*x_ref[1]**o+(-0.693289910121)*x_ref[1]
        ref[0,2,1,1]=(-0.39760464469)*x_ref[0]**o+(-0.921383269336)*x_ref[0]+(0.753657132321)*x_ref[1]**o+(0.324768021371)*x_ref[1]
        ref[0,2,2,0]=(0.0243534816004)*x_ref[0]**o+(0.290663121221)*x_ref[0]+(0.0305666628617)*x_ref[1]**o+(0.0446693318891)*x_ref[1]
        ref[0,2,2,1]=(-0.14154275886)*x_ref[0]**o+(0.881207887999)*x_ref[0]+(0.399029780289)*x_ref[1]**o+(0.168497381665)*x_ref[1]
        ref[0,3,0,0]=(-0.233073561791)*x_ref[0]**o+(-0.513212321666)*x_ref[0]+(-0.651480028725)*x_ref[1]**o+(0.147498573261)*x_ref[1]
        ref[0,3,0,1]=(-0.918136406734)*x_ref[0]**o+(-0.279161242114)*x_ref[0]+(0.983432543886)*x_ref[1]**o+(-0.723257302563)*x_ref[1]
        ref[0,3,1,0]=(0.290070399137)*x_ref[0]**o+(0.485832779323)*x_ref[0]+(0.955571801349)*x_ref[1]**o+(0.552993034163)*x_ref[1]
        ref[0,3,1,1]=(0.688570077213)*x_ref[0]**o+(0.578716466405)*x_ref[0]+(-0.40829418423)*x_ref[1]**o+(-0.918196317732)*x_ref[1]
        ref[0,3,2,0]=(0.403069347997)*x_ref[0]**o+(0.119390454784)*x_ref[0]+(-0.322018782913)*x_ref[1]**o+(0.224814772968)*x_ref[1]
        ref[0,3,2,1]=(-0.23084264112)*x_ref[0]**o+(-0.398235674194)*x_ref[0]+(-0.730556055687)*x_ref[1]**o+(0.293981158644)*x_ref[1]
        ref[0,4,0,0]=(-0.0984201873925)*x_ref[0]**o+(0.018359257988)*x_ref[0]+(-0.0781876875229)*x_ref[1]**o+(0.253344618889)*x_ref[1]
        ref[0,4,0,1]=(-0.20899724864)*x_ref[0]**o+(-0.722019204308)*x_ref[0]+(0.555162582715)*x_ref[1]**o+(-0.958068826429)*x_ref[1]
        ref[0,4,1,0]=(0.477725976915)*x_ref[0]**o+(0.295872229679)*x_ref[0]+(-0.828208555398)*x_ref[1]**o+(-0.931582818519)*x_ref[1]
        ref[0,4,1,1]=(0.721285453077)*x_ref[0]**o+(-0.042927949943)*x_ref[0]+(0.680231405034)*x_ref[1]**o+(-0.16043805248)*x_ref[1]
        ref[0,4,2,0]=(0.985988751136)*x_ref[0]**o+(0.885785903078)*x_ref[0]+(-0.0410955856477)*x_ref[1]**o+(-0.552089008201)*x_ref[1]
        ref[0,4,2,1]=(0.847466818225)*x_ref[0]**o+(-0.51057282448)*x_ref[0]+(0.163775791256)*x_ref[1]**o+(0.0152553094575)*x_ref[1]
        ref[1,0,0,0]=(0.0343855486006)*x_ref[0]**o+(-0.214334464898)*x_ref[0]+(-0.573344070879)*x_ref[1]**o+(0.24905956136)*x_ref[1]
        ref[1,0,0,1]=(0.783338978316)*x_ref[0]**o+(0.781324842059)*x_ref[0]+(-0.847509450777)*x_ref[1]**o+(-0.858994488444)*x_ref[1]
        ref[1,0,1,0]=(0.59719044885)*x_ref[0]**o+(0.347038111035)*x_ref[0]+(-0.385761256521)*x_ref[1]**o+(-0.571166989318)*x_ref[1]
        ref[1,0,1,1]=(0.636911008507)*x_ref[0]**o+(-0.825376916398)*x_ref[0]+(0.32394534216)*x_ref[1]**o+(0.653119502004)*x_ref[1]
        ref[1,0,2,0]=(-0.669732532783)*x_ref[0]**o+(-0.145427031253)*x_ref[0]+(-0.303693857831)*x_ref[1]**o+(-0.727878524833)*x_ref[1]
        ref[1,0,2,1]=(0.621061585764)*x_ref[0]**o+(0.243432217146)*x_ref[0]+(-0.888610099037)*x_ref[1]**o+(0.298051589144)*x_ref[1]
        ref[1,1,0,0]=(0.357341721813)*x_ref[0]**o+(0.346506498237)*x_ref[0]+(-0.555625437961)*x_ref[1]**o+(-0.0662579456147)*x_ref[1]
        ref[1,1,0,1]=(0.96955420495)*x_ref[0]**o+(-0.190486134786)*x_ref[0]+(-0.900124677941)*x_ref[1]**o+(0.659993963277)*x_ref[1]
        ref[1,1,1,0]=(-0.517837108406)*x_ref[0]**o+(0.00779665389526)*x_ref[0]+(-0.0544022169324)*x_ref[1]**o+(0.888217855388)*x_ref[1]
        ref[1,1,1,1]=(0.252515646862)*x_ref[0]**o+(0.328948655235)*x_ref[0]+(0.265916563145)*x_ref[1]**o+(-0.491865215095)*x_ref[1]
        ref[1,1,2,0]=(0.922520219928)*x_ref[0]**o+(0.463811222486)*x_ref[0]+(0.651244201962)*x_ref[1]**o+(0.982565677231)*x_ref[1]
        ref[1,1,2,1]=(0.257711278265)*x_ref[0]**o+(0.23971381197)*x_ref[0]+(-0.168559977915)*x_ref[1]**o+(0.453909616568)*x_ref[1]
        ref[1,2,0,0]=(0.0426933948917)*x_ref[0]**o+(-0.477381944712)*x_ref[0]+(0.503797983762)*x_ref[1]**o+(-0.943092701839)*x_ref[1]
        ref[1,2,0,1]=(0.913591544613)*x_ref[0]**o+(-0.895063818412)*x_ref[0]+(0.252264576564)*x_ref[1]**o+(0.424846745491)*x_ref[1]
        ref[1,2,1,0]=(0.4199934247)*x_ref[0]**o+(-0.128668379749)*x_ref[0]+(-0.157997092493)*x_ref[1]**o+(-0.174476338072)*x_ref[1]
        ref[1,2,1,1]=(0.394310091468)*x_ref[0]**o+(0.599539798636)*x_ref[0]+(0.133087615453)*x_ref[1]**o+(-0.22271509754)*x_ref[1]
        ref[1,2,2,0]=(0.559521330007)*x_ref[0]**o+(-0.411300693025)*x_ref[0]+(-0.543921885179)*x_ref[1]**o+(-0.0506834937614)*x_ref[1]
        ref[1,2,2,1]=(-0.423996886775)*x_ref[0]**o+(0.14409349326)*x_ref[0]+(-0.970632608145)*x_ref[1]**o+(0.582937355294)*x_ref[1]
        ref[1,3,0,0]=(-0.884972026022)*x_ref[0]**o+(-0.872260393893)*x_ref[0]+(0.224463013859)*x_ref[1]**o+(-0.245013144519)*x_ref[1]
        ref[1,3,0,1]=(-0.834873836296)*x_ref[0]**o+(-0.778611513205)*x_ref[0]+(0.989113281842)*x_ref[1]**o+(0.847047912646)*x_ref[1]
        ref[1,3,1,0]=(-0.0900703847239)*x_ref[0]**o+(-0.0295919916877)*x_ref[0]+(0.510380741505)*x_ref[1]**o+(-0.00354494905936)*x_ref[1]
        ref[1,3,1,1]=(0.0422103052661)*x_ref[0]**o+(0.31376324435)*x_ref[0]+(0.831890412519)*x_ref[1]**o+(0.764406560706)*x_ref[1]
        ref[1,3,2,0]=(0.496699862489)*x_ref[0]**o+(0.477514051712)*x_ref[0]+(0.156435695163)*x_ref[1]**o+(0.339443828388)*x_ref[1]
        ref[1,3,2,1]=(0.396689832399)*x_ref[0]**o+(0.243605505552)*x_ref[0]+(-0.0357728699149)*x_ref[1]**o+(0.930734881576)*x_ref[1]
        ref[1,4,0,0]=(-0.90788627878)*x_ref[0]**o+(0.82597715373)*x_ref[0]+(0.287988132693)*x_ref[1]**o+(-0.834307302134)*x_ref[1]
        ref[1,4,0,1]=(-0.942882799669)*x_ref[0]**o+(-0.149609905045)*x_ref[0]+(-0.340025012164)*x_ref[1]**o+(0.539546162654)*x_ref[1]
        ref[1,4,1,0]=(-0.244377326727)*x_ref[0]**o+(-0.586439384775)*x_ref[0]+(0.89506494604)*x_ref[1]**o+(0.233226803834)*x_ref[1]
        ref[1,4,1,1]=(0.864582224514)*x_ref[0]**o+(0.0687754700765)*x_ref[0]+(-0.641556536115)*x_ref[1]**o+(0.22036806322)*x_ref[1]
        ref[1,4,2,0]=(-0.552589117461)*x_ref[0]**o+(-0.705244093999)*x_ref[0]+(0.378837921567)*x_ref[1]**o+(-0.173968977986)*x_ref[1]
        ref[1,4,2,1]=(0.846900000181)*x_ref[0]**o+(-0.52589472084)*x_ref[0]+(0.872400573242)*x_ref[1]**o+(0.182683546911)*x_ref[1]
        ref[2,0,0,0]=(0.305006901177)*x_ref[0]**o+(-0.732858762152)*x_ref[0]+(-0.882239786812)*x_ref[1]**o+(0.481712295538)*x_ref[1]
        ref[2,0,0,1]=(0.836332373747)*x_ref[0]**o+(0.63905574965)*x_ref[0]+(0.039226709213)*x_ref[1]**o+(0.704758586936)*x_ref[1]
        ref[2,0,1,0]=(0.51805564252)*x_ref[0]**o+(0.641883965761)*x_ref[0]+(0.901663292208)*x_ref[1]**o+(0.375272715842)*x_ref[1]
        ref[2,0,1,1]=(-0.980058913052)*x_ref[0]**o+(0.385619663167)*x_ref[0]+(0.0722192483217)*x_ref[1]**o+(0.00650731529315)*x_ref[1]
        ref[2,0,2,0]=(-0.25791454536)*x_ref[0]**o+(0.460015385804)*x_ref[0]+(0.234970544559)*x_ref[1]**o+(-0.11962323176)*x_ref[1]
        ref[2,0,2,1]=(0.023390322577)*x_ref[0]**o+(-0.104449150638)*x_ref[0]+(0.149724956284)*x_ref[1]**o+(-0.309848596985)*x_ref[1]
        ref[2,1,0,0]=(0.603565629307)*x_ref[0]**o+(-0.143206239367)*x_ref[0]+(-0.0430666954645)*x_ref[1]**o+(-0.154546675756)*x_ref[1]
        ref[2,1,0,1]=(-0.980112660961)*x_ref[0]**o+(0.419145835585)*x_ref[0]+(-0.76202669209)*x_ref[1]**o+(-0.648615698697)*x_ref[1]
        ref[2,1,1,0]=(0.805964945679)*x_ref[0]**o+(0.650144650504)*x_ref[0]+(0.308110033125)*x_ref[1]**o+(0.0666476104838)*x_ref[1]
        ref[2,1,1,1]=(0.42561398203)*x_ref[0]**o+(0.259276259654)*x_ref[0]+(-0.215194398301)*x_ref[1]**o+(0.880282506885)*x_ref[1]
        ref[2,1,2,0]=(-0.413779899229)*x_ref[0]**o+(-0.796859182258)*x_ref[0]+(0.606865226329)*x_ref[1]**o+(-0.783313517188)*x_ref[1]
        ref[2,1,2,1]=(0.402767439742)*x_ref[0]**o+(-0.692468937875)*x_ref[0]+(0.27866251803)*x_ref[1]**o+(0.95092559773)*x_ref[1]
        ref[2,2,0,0]=(0.749107532378)*x_ref[0]**o+(-0.364633672479)*x_ref[0]+(0.970710337513)*x_ref[1]**o+(0.1986749977)*x_ref[1]
        ref[2,2,0,1]=(-0.722832876805)*x_ref[0]**o+(-0.238825576333)*x_ref[0]+(-0.101752751414)*x_ref[1]**o+(-0.960067435044)*x_ref[1]
        ref[2,2,1,0]=(-0.704825996403)*x_ref[0]**o+(0.74244530617)*x_ref[0]+(-0.586571188279)*x_ref[1]**o+(0.841297962355)*x_ref[1]
        ref[2,2,1,1]=(0.586127464995)*x_ref[0]**o+(0.454050196541)*x_ref[0]+(0.225294694159)*x_ref[1]**o+(0.705925824031)*x_ref[1]
        ref[2,2,2,0]=(0.535446366333)*x_ref[0]**o+(-0.787565531495)*x_ref[0]+(-0.566053725536)*x_ref[1]**o+(0.468968266813)*x_ref[1]
        ref[2,2,2,1]=(0.449236149776)*x_ref[0]**o+(0.589799099557)*x_ref[0]+(0.189540632578)*x_ref[1]**o+(-0.775036139482)*x_ref[1]
        ref[2,3,0,0]=(0.979572074948)*x_ref[0]**o+(-0.806420343846)*x_ref[0]+(0.661682487968)*x_ref[1]**o+(-0.615846811222)*x_ref[1]
        ref[2,3,0,1]=(0.021072244205)*x_ref[0]**o+(0.878344628134)*x_ref[0]+(-0.381067816502)*x_ref[1]**o+(0.229027446937)*x_ref[1]
        ref[2,3,1,0]=(0.0182291979593)*x_ref[0]**o+(0.625595584968)*x_ref[0]+(-0.824161018488)*x_ref[1]**o+(-0.316191235785)*x_ref[1]
        ref[2,3,1,1]=(-0.0168799900253)*x_ref[0]**o+(-0.137230078645)*x_ref[0]+(0.0240280311645)*x_ref[1]**o+(-0.364099905801)*x_ref[1]
        ref[2,3,2,0]=(0.374193672582)*x_ref[0]**o+(0.429814892173)*x_ref[0]+(0.0194417431204)*x_ref[1]**o+(0.830453174612)*x_ref[1]
        ref[2,3,2,1]=(-0.683758441576)*x_ref[0]**o+(-0.759635036652)*x_ref[0]+(-0.50102074202)*x_ref[1]**o+(0.123764523628)*x_ref[1]
        ref[2,4,0,0]=(-0.0817296101008)*x_ref[0]**o+(0.730474427452)*x_ref[0]+(-0.0733544873209)*x_ref[1]**o+(0.416303527556)*x_ref[1]
        ref[2,4,0,1]=(0.258598296267)*x_ref[0]**o+(0.686740404407)*x_ref[0]+(0.543414259362)*x_ref[1]**o+(0.518071987904)*x_ref[1]
        ref[2,4,1,0]=(-0.32475099592)*x_ref[0]**o+(-0.0460901003729)*x_ref[0]+(0.275611466137)*x_ref[1]**o+(0.8415303665)*x_ref[1]
        ref[2,4,1,1]=(0.5015310862)*x_ref[0]**o+(-0.71451126551)*x_ref[0]+(0.897703909295)*x_ref[1]**o+(-0.0476780709524)*x_ref[1]
        ref[2,4,2,0]=(0.612722779576)*x_ref[0]**o+(0.424485459817)*x_ref[0]+(-0.136026922168)*x_ref[1]**o+(0.16727850944)*x_ref[1]
        ref[2,4,2,1]=(-0.841159017194)*x_ref[0]**o+(0.856949657441)*x_ref[0]+(-0.975786438976)*x_ref[1]**o+(-0.407229683889)*x_ref[1]
        ref[3,0,0,0]=(0.833459943775)*x_ref[0]**o+(-0.230178834842)*x_ref[0]+(-0.660801597291)*x_ref[1]**o+(-0.738869823127)*x_ref[1]
        ref[3,0,0,1]=(0.30987970183)*x_ref[0]**o+(-0.610707485471)*x_ref[0]+(0.628798231358)*x_ref[1]**o+(-0.0491116840572)*x_ref[1]
        ref[3,0,1,0]=(0.649739974341)*x_ref[0]**o+(-0.450666468651)*x_ref[0]+(0.901638992166)*x_ref[1]**o+(-0.787549067401)*x_ref[1]
        ref[3,0,1,1]=(0.934583955853)*x_ref[0]**o+(-0.859086605977)*x_ref[0]+(0.0975143570756)*x_ref[1]**o+(0.0350097808797)*x_ref[1]
        ref[3,0,2,0]=(0.837723849965)*x_ref[0]**o+(-0.665696123522)*x_ref[0]+(-0.70629678851)*x_ref[1]**o+(-0.468945853667)*x_ref[1]
        ref[3,0,2,1]=(0.526302645633)*x_ref[0]**o+(-0.0979568077223)*x_ref[0]+(-0.767623484018)*x_ref[1]**o+(-0.21912588217)*x_ref[1]
        ref[3,1,0,0]=(0.18832505785)*x_ref[0]**o+(0.45004009293)*x_ref[0]+(-0.823702397711)*x_ref[1]**o+(-0.912642057295)*x_ref[1]
        ref[3,1,0,1]=(0.554483721388)*x_ref[0]**o+(-0.607925140409)*x_ref[0]+(-0.998308584629)*x_ref[1]**o+(-0.538184347898)*x_ref[1]
        ref[3,1,1,0]=(-0.682363035722)*x_ref[0]**o+(0.55318030401)*x_ref[0]+(0.562408350319)*x_ref[1]**o+(0.220145757421)*x_ref[1]
        ref[3,1,1,1]=(0.924816884714)*x_ref[0]**o+(-0.295767203325)*x_ref[0]+(0.230343937118)*x_ref[1]**o+(0.957747143338)*x_ref[1]
        ref[3,1,2,0]=(0.235422171194)*x_ref[0]**o+(0.264750603603)*x_ref[0]+(0.13053017764)*x_ref[1]**o+(0.628805980195)*x_ref[1]
        ref[3,1,2,1]=(-0.603111233811)*x_ref[0]**o+(-0.325257245906)*x_ref[0]+(-0.879402225173)*x_ref[1]**o+(0.977736365104)*x_ref[1]
        ref[3,2,0,0]=(0.463242222753)*x_ref[0]**o+(-0.802230749276)*x_ref[0]+(0.085387156258)*x_ref[1]**o+(0.18309223952)*x_ref[1]
        ref[3,2,0,1]=(0.778060805693)*x_ref[0]**o+(0.32136550684)*x_ref[0]+(0.775284717696)*x_ref[1]**o+(0.151449430741)*x_ref[1]
        ref[3,2,1,0]=(0.288473735099)*x_ref[0]**o+(0.675142038399)*x_ref[0]+(-0.313098988069)*x_ref[1]**o+(-0.771883975974)*x_ref[1]
        ref[3,2,1,1]=(0.715304130316)*x_ref[0]**o+(0.994375393616)*x_ref[0]+(-0.874667784061)*x_ref[1]**o+(-0.29510377129)*x_ref[1]
        ref[3,2,2,0]=(-0.211496384269)*x_ref[0]**o+(0.146746732763)*x_ref[0]+(0.234317983556)*x_ref[1]**o+(0.844710015939)*x_ref[1]
        ref[3,2,2,1]=(0.557426563891)*x_ref[0]**o+(0.181380049852)*x_ref[0]+(-0.068830884747)*x_ref[1]**o+(0.424778582562)*x_ref[1]
        ref[3,3,0,0]=(-0.784389821334)*x_ref[0]**o+(0.533073578323)*x_ref[0]+(-0.172622226211)*x_ref[1]**o+(-0.0882056556263)*x_ref[1]
        ref[3,3,0,1]=(-0.369034971239)*x_ref[0]**o+(-0.323409858793)*x_ref[0]+(0.408766889438)*x_ref[1]**o+(-0.653470515807)*x_ref[1]
        ref[3,3,1,0]=(0.239411265848)*x_ref[0]**o+(-0.672719205684)*x_ref[0]+(-0.751715345243)*x_ref[1]**o+(-0.4586462848)*x_ref[1]
        ref[3,3,1,1]=(-0.937724235502)*x_ref[0]**o+(-0.9284117207)*x_ref[0]+(-0.584507468424)*x_ref[1]**o+(0.779562243368)*x_ref[1]
        ref[3,3,2,0]=(-0.563406066207)*x_ref[0]**o+(0.115504525413)*x_ref[0]+(0.0230187346201)*x_ref[1]**o+(0.960113641875)*x_ref[1]
        ref[3,3,2,1]=(0.792686350755)*x_ref[0]**o+(0.560920910518)*x_ref[0]+(0.0766597516323)*x_ref[1]**o+(-0.728345800422)*x_ref[1]
        ref[3,4,0,0]=(-0.182383283107)*x_ref[0]**o+(0.604818623768)*x_ref[0]+(0.756868334363)*x_ref[1]**o+(0.0259669270239)*x_ref[1]
        ref[3,4,0,1]=(0.547592534695)*x_ref[0]**o+(0.666130932562)*x_ref[0]+(-0.922839241546)*x_ref[1]**o+(-0.143547846716)*x_ref[1]
        ref[3,4,1,0]=(-0.43295353947)*x_ref[0]**o+(0.342981294189)*x_ref[0]+(0.785780092257)*x_ref[1]**o+(0.480176709076)*x_ref[1]
        ref[3,4,1,1]=(-0.859691385907)*x_ref[0]**o+(0.124338939229)*x_ref[0]+(-0.883988405006)*x_ref[1]**o+(0.0683789905855)*x_ref[1]
        ref[3,4,2,0]=(-0.106494101633)*x_ref[0]**o+(0.276437588849)*x_ref[0]+(0.408900924262)*x_ref[1]**o+(-0.479798281555)*x_ref[1]
        ref[3,4,2,1]=(-0.226633020661)*x_ref[0]**o+(0.487277344856)*x_ref[0]+(-0.0241726779927)*x_ref[1]**o+(-0.778823212273)*x_ref[1]
      else:
        s[0,0,0,0]=(-0.400665270553)*x[0]**o+(-0.465656794893)*x[0]+(-0.103120094839)*x[1]**o+(0.0464026120794)*x[1]+(0.030596268742)*x[2]**o+(0.00334628754684)*x[2]
        s[0,0,0,1]=(0.937480902128)*x[0]**o+(0.0466658012652)*x[0]+(0.817801316276)*x[1]**o+(-0.142948501497)*x[1]+(-0.946943305156)*x[2]**o+(-0.633505380306)*x[2]
        s[0,0,1,0]=(-0.441314521722)*x[0]**o+(-0.42022483938)*x[0]+(-0.833228022905)*x[1]**o+(-0.225426650714)*x[1]+(0.332718715913)*x[2]**o+(0.123261640846)*x[2]
        s[0,0,1,1]=(0.335875498268)*x[0]**o+(0.983700594491)*x[0]+(-0.214325088125)*x[1]**o+(-0.893543563881)*x[1]+(0.0631361789468)*x[2]**o+(0.4467209081)*x[2]
        s[0,0,2,0]=(0.658215366848)*x[0]**o+(-0.672295829784)*x[0]+(-0.0137993447332)*x[1]**o+(0.698632148806)*x[1]+(-0.221572355182)*x[2]**o+(0.956096660679)*x[2]
        s[0,0,2,1]=(-0.578795938537)*x[0]**o+(0.750684927067)*x[0]+(0.333363761953)*x[1]**o+(-0.617804591638)*x[1]+(-0.0469494394831)*x[2]**o+(0.16187318118)*x[2]
        s[0,1,0,0]=(-0.0638097634115)*x[0]**o+(-0.38507312448)*x[0]+(-0.663736729005)*x[1]**o+(-0.989965410511)*x[1]+(0.230748674134)*x[2]**o+(-0.830580398633)*x[2]
        s[0,1,0,1]=(-0.658956286059)*x[0]**o+(0.345180478933)*x[0]+(-0.544808816473)*x[1]**o+(0.799798879144)*x[1]+(-0.862695252289)*x[2]**o+(0.757134401664)*x[2]
        s[0,1,1,0]=(0.456144819828)*x[0]**o+(0.969408065204)*x[0]+(0.488759322896)*x[1]**o+(0.191259701658)*x[1]+(-0.211530905801)*x[2]**o+(0.0976530925065)*x[2]
        s[0,1,1,1]=(-0.160993701908)*x[0]**o+(-0.768084723399)*x[0]+(0.200776153747)*x[1]**o+(0.380923444552)*x[1]+(0.152388183214)*x[2]**o+(0.0345495856869)*x[2]
        s[0,1,2,0]=(-0.249731094314)*x[0]**o+(0.420921348364)*x[0]+(-0.745813017956)*x[1]**o+(-0.128319964472)*x[1]+(-0.134590180255)*x[2]**o+(-0.598364990007)*x[2]
        s[0,1,2,1]=(0.608497167751)*x[0]**o+(0.134066300157)*x[0]+(-0.219556933024)*x[1]**o+(0.845095947325)*x[1]+(-0.0910112208065)*x[2]**o+(-0.919387374125)*x[2]
        s[0,2,0,0]=(-0.603722566149)*x[0]**o+(-0.141049572403)*x[0]+(-0.433072627843)*x[1]**o+(-0.266640930301)*x[1]+(-0.0520863163599)*x[2]**o+(0.113826044084)*x[2]
        s[0,2,0,1]=(0.735600439472)*x[0]**o+(-0.683666203432)*x[0]+(0.439527311107)*x[1]**o+(0.167685906293)*x[1]+(-0.526256697628)*x[2]**o+(-0.152028404784)*x[2]
        s[0,2,1,0]=(-0.484268475407)*x[0]**o+(-0.1302369788)*x[0]+(0.0996963541291)*x[1]**o+(0.525432686771)*x[1]+(0.915056396412)*x[2]**o+(-0.93142015197)*x[2]
        s[0,2,1,1]=(0.103016217069)*x[0]**o+(0.00517836018812)*x[0]+(-0.28331822161)*x[1]**o+(0.413800832478)*x[1]+(0.94613674069)*x[2]**o+(-0.287932854599)*x[2]
        s[0,2,2,0]=(0.512958587386)*x[0]**o+(-0.851145503923)*x[0]+(-0.535377708818)*x[1]**o+(-0.379448444669)*x[1]+(-0.238808818844)*x[2]**o+(0.567146071008)*x[2]
        s[0,2,2,1]=(-0.141490825422)*x[0]**o+(0.133948738844)*x[0]+(0.336733111602)*x[1]**o+(-0.223425201179)*x[1]+(-0.646454138519)*x[2]**o+(0.823936105317)*x[2]
        s[0,3,0,0]=(-0.938725695354)*x[0]**o+(0.988131864396)*x[0]+(-0.82501934096)*x[1]**o+(-0.257156351526)*x[1]+(-0.00349993655072)*x[2]**o+(0.580041226419)*x[2]
        s[0,3,0,1]=(0.814169577155)*x[0]**o+(-0.686109074376)*x[0]+(-0.965459022554)*x[1]**o+(0.10941355421)*x[1]+(0.609050101353)*x[2]**o+(-0.965588393623)*x[2]
        s[0,3,1,0]=(-0.820571213003)*x[0]**o+(0.746749194774)*x[0]+(0.710345174116)*x[1]**o+(-0.892921665205)*x[1]+(0.4004855541)*x[2]**o+(-0.695117880583)*x[2]
        s[0,3,1,1]=(-0.122273772977)*x[0]**o+(0.570499591543)*x[0]+(0.795058034707)*x[1]**o+(0.925412519293)*x[1]+(-0.0932686873001)*x[2]**o+(-0.686679792186)*x[2]
        s[0,3,2,0]=(-0.0042793185541)*x[0]**o+(-0.821480764508)*x[0]+(0.875015113726)*x[1]**o+(0.649383518184)*x[1]+(-0.288257569005)*x[2]**o+(-0.743941827161)*x[2]
        s[0,3,2,1]=(0.974182567966)*x[0]**o+(0.733657486071)*x[0]+(0.780035494556)*x[1]**o+(0.749708949611)*x[1]+(-0.521673379431)*x[2]**o+(0.673844867694)*x[2]
        s[0,4,0,0]=(-0.150057729107)*x[0]**o+(0.102611114471)*x[0]+(-0.0633864241155)*x[1]**o+(-0.329289187178)*x[1]+(0.315706711936)*x[2]**o+(-0.554443679394)*x[2]
        s[0,4,0,1]=(0.564883543338)*x[0]**o+(0.535008108501)*x[0]+(0.815233609718)*x[1]**o+(-0.766536697052)*x[1]+(0.997595942191)*x[2]**o+(0.991881374395)*x[2]
        s[0,4,1,0]=(-0.725748156838)*x[0]**o+(-0.817693342658)*x[0]+(0.501105656184)*x[1]**o+(-0.331771590595)*x[1]+(-0.757619653898)*x[2]**o+(-0.885657271086)*x[2]
        s[0,4,1,1]=(-0.559814749411)*x[0]**o+(0.665634435712)*x[0]+(0.787002371365)*x[1]**o+(-0.762034037705)*x[1]+(0.508112385178)*x[2]**o+(-0.985681857439)*x[2]
        s[0,4,2,0]=(0.504217223699)*x[0]**o+(-0.361750429828)*x[0]+(0.19299099109)*x[1]**o+(0.338521927796)*x[1]+(0.549199682759)*x[2]**o+(0.891662590379)*x[2]
        s[0,4,2,1]=(-0.251870804657)*x[0]**o+(0.776378068034)*x[0]+(-0.769493074808)*x[1]**o+(0.929429944731)*x[1]+(0.679845951817)*x[2]**o+(0.261179050197)*x[2]
        s[1,0,0,0]=(0.339823121204)*x[0]**o+(0.212075564821)*x[0]+(0.103901436655)*x[1]**o+(0.887698805047)*x[1]+(-0.467697404092)*x[2]**o+(0.71153584324)*x[2]
        s[1,0,0,1]=(0.610147759489)*x[0]**o+(-0.820866072636)*x[0]+(0.505294661936)*x[1]**o+(-0.206382410595)*x[1]+(0.343889298437)*x[2]**o+(0.165680393811)*x[2]
        s[1,0,1,0]=(-0.155342751495)*x[0]**o+(-0.638727698114)*x[0]+(-0.245328510129)*x[1]**o+(0.816689513534)*x[1]+(0.138847830114)*x[2]**o+(0.286577642911)*x[2]
        s[1,0,1,1]=(0.804058022285)*x[0]**o+(-0.922679564915)*x[0]+(-0.755679292664)*x[1]**o+(-0.409251039141)*x[1]+(-0.697869565514)*x[2]**o+(-0.0749517480557)*x[2]
        s[1,0,2,0]=(-0.345641262813)*x[0]**o+(0.31066537094)*x[0]+(0.165733093512)*x[1]**o+(-0.733996040753)*x[1]+(0.240017268146)*x[2]**o+(0.981970338884)*x[2]
        s[1,0,2,1]=(-0.0394605078977)*x[0]**o+(-0.671255389736)*x[0]+(-0.58457671038)*x[1]**o+(-0.308411795131)*x[1]+(0.944299000242)*x[2]**o+(0.0274884386004)*x[2]
        s[1,1,0,0]=(0.285078336337)*x[0]**o+(-0.29609479737)*x[0]+(0.851762966843)*x[1]**o+(-0.56978612625)*x[1]+(-0.416902609766)*x[2]**o+(-0.209352333275)*x[2]
        s[1,1,0,1]=(0.165305243827)*x[0]**o+(0.569657682766)*x[0]+(-0.158936758165)*x[1]**o+(-0.826897725956)*x[1]+(-0.906153311902)*x[2]**o+(-0.508208690461)*x[2]
        s[1,1,1,0]=(-0.168856536186)*x[0]**o+(0.600931452022)*x[0]+(0.515269032144)*x[1]**o+(-0.615629628046)*x[1]+(-0.248016004788)*x[2]**o+(-0.236340748995)*x[2]
        s[1,1,1,1]=(-0.265342410313)*x[0]**o+(0.546304098539)*x[0]+(0.44451795278)*x[1]**o+(0.53863749512)*x[1]+(-0.400353954694)*x[2]**o+(0.762091735577)*x[2]
        s[1,1,2,0]=(-0.828507588085)*x[0]**o+(0.310151266525)*x[0]+(0.767699919156)*x[1]**o+(-0.802526284756)*x[1]+(-0.12986443685)*x[2]**o+(0.555810425336)*x[2]
        s[1,1,2,1]=(-0.0843531370201)*x[0]**o+(0.138067935996)*x[0]+(-0.819663672158)*x[1]**o+(0.227177408663)*x[1]+(-0.481488807321)*x[2]**o+(0.277780608851)*x[2]
        s[1,2,0,0]=(0.843912054965)*x[0]**o+(-0.532942686099)*x[0]+(-0.12263392247)*x[1]**o+(-0.918649820864)*x[1]+(-0.898969788765)*x[2]**o+(0.401718907796)*x[2]
        s[1,2,0,1]=(0.500181344385)*x[0]**o+(-0.695081419586)*x[0]+(0.49127001192)*x[1]**o+(0.424707942699)*x[1]+(-0.170525943009)*x[2]**o+(-0.908857558417)*x[2]
        s[1,2,1,0]=(0.374108805296)*x[0]**o+(-0.778311131014)*x[0]+(-0.619173488016)*x[1]**o+(-0.0906082150795)*x[1]+(0.312783760308)*x[2]**o+(0.50073883273)*x[2]
        s[1,2,1,1]=(0.474622980263)*x[0]**o+(0.00968769992516)*x[0]+(-0.246566756628)*x[1]**o+(0.868983907653)*x[1]+(-0.318924531295)*x[2]**o+(-0.340314263979)*x[2]
        s[1,2,2,0]=(0.621111695462)*x[0]**o+(0.655354521723)*x[0]+(-0.992404590253)*x[1]**o+(-0.497860913536)*x[1]+(-0.963211421906)*x[2]**o+(0.493347857543)*x[2]
        s[1,2,2,1]=(-0.32035342079)*x[0]**o+(0.351692950156)*x[0]+(-0.715457409869)*x[1]**o+(0.317941180953)*x[1]+(0.831453341583)*x[2]**o+(0.330720124015)*x[2]
        s[1,3,0,0]=(-0.775605593731)*x[0]**o+(-0.985544306967)*x[0]+(0.838313790256)*x[1]**o+(-0.0515692297878)*x[1]+(0.6757488996)*x[2]**o+(-0.398401163936)*x[2]
        s[1,3,0,1]=(0.129462614525)*x[0]**o+(-0.225290874002)*x[0]+(0.1627137973)*x[1]**o+(0.96490154212)*x[1]+(-0.0348196505137)*x[2]**o+(0.00501682452837)*x[2]
        s[1,3,1,0]=(-0.774906442617)*x[0]**o+(0.554579514554)*x[0]+(0.68683898073)*x[1]**o+(0.687422147497)*x[1]+(0.812740760401)*x[2]**o+(0.650574541761)*x[2]
        s[1,3,1,1]=(0.371414654481)*x[0]**o+(-0.460895148602)*x[0]+(-0.60035497386)*x[1]**o+(0.382490052311)*x[1]+(0.348365164241)*x[2]**o+(0.700564991456)*x[2]
        s[1,3,2,0]=(0.0627821130997)*x[0]**o+(-0.108341459129)*x[0]+(-0.459931858236)*x[1]**o+(-0.84999694423)*x[1]+(-0.369143500965)*x[2]**o+(-0.204962250898)*x[2]
        s[1,3,2,1]=(0.940231653623)*x[0]**o+(0.432366518466)*x[0]+(0.965573462127)*x[1]**o+(-0.400536850676)*x[1]+(0.627651239821)*x[2]**o+(0.541970384011)*x[2]
        s[1,4,0,0]=(-0.905529267496)*x[0]**o+(-0.728637228667)*x[0]+(0.528565304374)*x[1]**o+(-0.0791953805449)*x[1]+(0.278340538308)*x[2]**o+(0.0829173042009)*x[2]
        s[1,4,0,1]=(0.196476386515)*x[0]**o+(0.960964133173)*x[0]+(-0.975854821639)*x[1]**o+(-0.231600097119)*x[1]+(0.462289007605)*x[2]**o+(-0.41548631733)*x[2]
        s[1,4,1,0]=(0.133593658437)*x[0]**o+(0.27713134658)*x[0]+(-0.626190133661)*x[1]**o+(0.22128472818)*x[1]+(-0.0310045745464)*x[2]**o+(-0.308952151364)*x[2]
        s[1,4,1,1]=(-0.540782388402)*x[0]**o+(0.782610613508)*x[0]+(-0.121818568845)*x[1]**o+(-0.481514588156)*x[1]+(0.92789834048)*x[2]**o+(-0.053669614999)*x[2]
        s[1,4,2,0]=(0.228180064246)*x[0]**o+(-0.387791183012)*x[0]+(-0.809517349857)*x[1]**o+(0.104233744095)*x[1]+(-0.010561196572)*x[2]**o+(0.812403786777)*x[2]
        s[1,4,2,1]=(-0.213527688719)*x[0]**o+(0.444377334059)*x[0]+(0.250474600963)*x[1]**o+(-0.628454953053)*x[1]+(-0.149400612708)*x[2]**o+(-0.362500181895)*x[2]
        s[2,0,0,0]=(0.424970215816)*x[0]**o+(-0.409634055175)*x[0]+(-0.209831967682)*x[1]**o+(0.777759135142)*x[1]+(0.904286801356)*x[2]**o+(-0.533118859622)*x[2]
        s[2,0,0,1]=(-0.0634924164631)*x[0]**o+(0.61665557894)*x[0]+(0.886000765027)*x[1]**o+(-0.283672331088)*x[1]+(-0.104923941729)*x[2]**o+(0.647966459575)*x[2]
        s[2,0,1,0]=(-0.601180947882)*x[0]**o+(-0.853176409106)*x[0]+(0.589049472252)*x[1]**o+(0.947188905512)*x[1]+(0.937016925257)*x[2]**o+(-0.528933622785)*x[2]
        s[2,0,1,1]=(-0.493108989214)*x[0]**o+(0.123452031172)*x[0]+(0.579015108934)*x[1]**o+(-0.224656610786)*x[1]+(-0.351966996224)*x[2]**o+(-0.58667858467)*x[2]
        s[2,0,2,0]=(0.342923581511)*x[0]**o+(-0.787209375031)*x[0]+(0.0689438410554)*x[1]**o+(0.624797184447)*x[1]+(0.508789531503)*x[2]**o+(0.17151319396)*x[2]
        s[2,0,2,1]=(-0.875568056219)*x[0]**o+(-0.113031987088)*x[0]+(-0.0609746766081)*x[1]**o+(0.196111602161)*x[1]+(-0.00186601407674)*x[2]**o+(-0.695244625545)*x[2]
        s[2,1,0,0]=(0.0760124938281)*x[0]**o+(0.878269236128)*x[0]+(0.424404663038)*x[1]**o+(0.382191251895)*x[1]+(-0.799519816473)*x[2]**o+(0.756150518781)*x[2]
        s[2,1,0,1]=(-0.620026715848)*x[0]**o+(0.120584354877)*x[0]+(0.794571749541)*x[1]**o+(0.775886892396)*x[1]+(-0.455364446354)*x[2]**o+(-0.25890753368)*x[2]
        s[2,1,1,0]=(-0.434917779388)*x[0]**o+(0.893805701213)*x[0]+(-0.359966318659)*x[1]**o+(-0.92549769048)*x[1]+(0.632194530492)*x[2]**o+(0.783055504124)*x[2]
        s[2,1,1,1]=(-0.30435434794)*x[0]**o+(-0.132800060169)*x[0]+(0.200039404417)*x[1]**o+(-0.690955124262)*x[1]+(-0.984472889901)*x[2]**o+(0.0242811209734)*x[2]
        s[2,1,2,0]=(-0.223815583384)*x[0]**o+(0.592762849244)*x[0]+(0.281436806248)*x[1]**o+(0.822974431302)*x[1]+(-0.0648638855118)*x[2]**o+(-0.798035298439)*x[2]
        s[2,1,2,1]=(-0.847339863019)*x[0]**o+(0.603483610026)*x[0]+(0.865918681346)*x[1]**o+(-0.0625995244022)*x[1]+(0.767004105382)*x[2]**o+(0.536886296262)*x[2]
        s[2,2,0,0]=(0.0907868991281)*x[0]**o+(0.0361485757336)*x[0]+(0.725787806874)*x[1]**o+(-0.955232659581)*x[1]+(0.015387691978)*x[2]**o+(-0.969939112182)*x[2]
        s[2,2,0,1]=(-0.702771278477)*x[0]**o+(-0.376611311235)*x[0]+(0.321676098666)*x[1]**o+(0.2741544323)*x[1]+(-0.490015464554)*x[2]**o+(-0.287278266636)*x[2]
        s[2,2,1,0]=(-0.239838875645)*x[0]**o+(-0.46816615613)*x[0]+(0.208010055752)*x[1]**o+(0.343192409907)*x[1]+(-0.752447682834)*x[2]**o+(-0.450307042264)*x[2]
        s[2,2,1,1]=(-0.377645472017)*x[0]**o+(0.979388261808)*x[0]+(0.899484011996)*x[1]**o+(0.278589875985)*x[1]+(-0.0566572805419)*x[2]**o+(-0.329814353957)*x[2]
        s[2,2,2,0]=(0.329972082761)*x[0]**o+(-0.40893298634)*x[0]+(0.14025500949)*x[1]**o+(-0.915605553253)*x[1]+(-0.743877103988)*x[2]**o+(0.0625993015596)*x[2]
        s[2,2,2,1]=(-0.426922382741)*x[0]**o+(0.638893012106)*x[0]+(0.775217826454)*x[1]**o+(0.64924098439)*x[1]+(-0.157863119278)*x[2]**o+(-0.785891322569)*x[2]
        s[2,3,0,0]=(-0.573635249481)*x[0]**o+(0.466378353545)*x[0]+(-0.968280361551)*x[1]**o+(-0.521040950167)*x[1]+(-0.979861738542)*x[2]**o+(0.293316009588)*x[2]
        s[2,3,0,1]=(-0.987445004264)*x[0]**o+(-0.855558575123)*x[0]+(0.703881586225)*x[1]**o+(0.393607926606)*x[1]+(0.615218365579)*x[2]**o+(0.865070564586)*x[2]
        s[2,3,1,0]=(0.243032241627)*x[0]**o+(0.447193793692)*x[0]+(0.121835082448)*x[1]**o+(0.253731889763)*x[1]+(0.482861064484)*x[2]**o+(0.0857661253915)*x[2]
        s[2,3,1,1]=(0.630205092814)*x[0]**o+(-0.230885016198)*x[0]+(-0.00423203911773)*x[1]**o+(0.560066624606)*x[1]+(-0.850065237172)*x[2]**o+(-0.0608202441365)*x[2]
        s[2,3,2,0]=(0.956674282321)*x[0]**o+(0.478516276613)*x[0]+(-0.142801297861)*x[1]**o+(-0.470299296114)*x[1]+(0.947735333619)*x[2]**o+(0.343691324696)*x[2]
        s[2,3,2,1]=(0.366910635243)*x[0]**o+(-0.0831991185812)*x[0]+(-0.828901851379)*x[1]**o+(-0.537917965468)*x[1]+(-0.581342127867)*x[2]**o+(-0.307660153071)*x[2]
        s[2,4,0,0]=(0.340464857467)*x[0]**o+(0.881022180176)*x[0]+(-0.223178224315)*x[1]**o+(0.0108952156846)*x[1]+(-0.130766988014)*x[2]**o+(-0.655755053638)*x[2]
        s[2,4,0,1]=(-0.142352024958)*x[0]**o+(-0.279134821704)*x[0]+(0.696577046583)*x[1]**o+(-0.913765778235)*x[1]+(0.674571789555)*x[2]**o+(-0.236086993066)*x[2]
        s[2,4,1,0]=(0.993240530889)*x[0]**o+(0.152584699472)*x[0]+(0.364843511953)*x[1]**o+(0.900205378533)*x[1]+(0.397560553857)*x[2]**o+(0.334533062647)*x[2]
        s[2,4,1,1]=(-0.261396791645)*x[0]**o+(0.96949114295)*x[0]+(0.604187009028)*x[1]**o+(-0.667369171782)*x[1]+(-0.313439370957)*x[2]**o+(0.448859840994)*x[2]
        s[2,4,2,0]=(-0.508273068959)*x[0]**o+(0.473127945457)*x[0]+(0.512686326084)*x[1]**o+(-0.872890815913)*x[1]+(-0.515333454444)*x[2]**o+(-0.578402077636)*x[2]
        s[2,4,2,1]=(-0.78684369593)*x[0]**o+(0.719254214286)*x[0]+(-0.857414812359)*x[1]**o+(-0.260538452293)*x[1]+(0.940982479653)*x[2]**o+(-0.94620100965)*x[2]
        s[3,0,0,0]=(0.741419615809)*x[0]**o+(0.535310119259)*x[0]+(-0.396521376661)*x[1]**o+(0.238209214692)*x[1]+(0.550440543811)*x[2]**o+(0.592067475252)*x[2]
        s[3,0,0,1]=(0.423030333878)*x[0]**o+(0.434870473967)*x[0]+(-0.599556117755)*x[1]**o+(0.299434424134)*x[1]+(0.120835992748)*x[2]**o+(-0.314011916578)*x[2]
        s[3,0,1,0]=(-0.326589197488)*x[0]**o+(-0.5670140013)*x[0]+(0.0548382156857)*x[1]**o+(0.689609176876)*x[1]+(0.257405458159)*x[2]**o+(-0.896175631279)*x[2]
        s[3,0,1,1]=(0.221120334733)*x[0]**o+(-0.273395088002)*x[0]+(0.678993292365)*x[1]**o+(-0.261592406628)*x[1]+(0.0129202430825)*x[2]**o+(0.148164299785)*x[2]
        s[3,0,2,0]=(0.894503166055)*x[0]**o+(-0.468004653303)*x[0]+(-0.103574702833)*x[1]**o+(-0.390725885766)*x[1]+(0.081712057468)*x[2]**o+(-0.507015846204)*x[2]
        s[3,0,2,1]=(0.712909263678)*x[0]**o+(-0.412420758959)*x[0]+(-0.620377453769)*x[1]**o+(-0.166241669671)*x[1]+(-0.738733737609)*x[2]**o+(-0.757426176741)*x[2]
        s[3,1,0,0]=(-0.166116194562)*x[0]**o+(0.988666848357)*x[0]+(-0.483399298528)*x[1]**o+(0.777960698485)*x[1]+(-0.374839973139)*x[2]**o+(0.914310220752)*x[2]
        s[3,1,0,1]=(-0.578374538082)*x[0]**o+(0.228605767869)*x[0]+(0.120418953818)*x[1]**o+(-0.693317296354)*x[1]+(-0.691997757818)*x[2]**o+(0.551252364781)*x[2]
        s[3,1,1,0]=(-0.384863493904)*x[0]**o+(0.300215711827)*x[0]+(0.948187183083)*x[1]**o+(-0.184464558346)*x[1]+(-0.7671733816)*x[2]**o+(0.307217058571)*x[2]
        s[3,1,1,1]=(0.578689722929)*x[0]**o+(0.885714287205)*x[0]+(0.722247905148)*x[1]**o+(0.933475647861)*x[1]+(0.211727836913)*x[2]**o+(0.711589357201)*x[2]
        s[3,1,2,0]=(-0.116046568217)*x[0]**o+(-0.735782802049)*x[0]+(-0.968041163959)*x[1]**o+(0.314957910843)*x[1]+(0.639581098814)*x[2]**o+(-0.566524915874)*x[2]
        s[3,1,2,1]=(0.0690920573575)*x[0]**o+(-0.0344562175408)*x[0]+(-0.729181462751)*x[1]**o+(-0.352992891115)*x[1]+(-0.230186276771)*x[2]**o+(-0.392393764843)*x[2]
        s[3,2,0,0]=(-0.897572361738)*x[0]**o+(0.42292848631)*x[0]+(0.0402102445405)*x[1]**o+(-0.695803229507)*x[1]+(0.657341764505)*x[2]**o+(0.534681573441)*x[2]
        s[3,2,0,1]=(-0.883898106985)*x[0]**o+(0.563240147064)*x[0]+(0.596180998828)*x[1]**o+(0.278590998454)*x[1]+(-0.686188881246)*x[2]**o+(0.827407211045)*x[2]
        s[3,2,1,0]=(-0.896992911933)*x[0]**o+(0.349871189445)*x[0]+(0.959261344864)*x[1]**o+(0.567834800455)*x[1]+(0.624336079204)*x[2]**o+(-0.853743864993)*x[2]
        s[3,2,1,1]=(0.947681143718)*x[0]**o+(0.563362524754)*x[0]+(0.398321082655)*x[1]**o+(0.217952803297)*x[1]+(0.243654787599)*x[2]**o+(-0.927517802601)*x[2]
        s[3,2,2,0]=(0.330842075788)*x[0]**o+(0.630950656819)*x[0]+(-0.469619054343)*x[1]**o+(-0.354739266808)*x[1]+(0.253515569858)*x[2]**o+(0.635440000647)*x[2]
        s[3,2,2,1]=(0.729280956878)*x[0]**o+(0.734399392463)*x[0]+(0.800616353182)*x[1]**o+(-0.764367237486)*x[1]+(0.90307570667)*x[2]**o+(-0.957618675411)*x[2]
        s[3,3,0,0]=(-0.783664290925)*x[0]**o+(-0.609524665456)*x[0]+(0.0126829605834)*x[1]**o+(0.396479443362)*x[1]+(0.262779944509)*x[2]**o+(-0.666191777094)*x[2]
        s[3,3,0,1]=(0.412930217814)*x[0]**o+(0.993234310311)*x[0]+(-0.123107776537)*x[1]**o+(-0.156132010898)*x[1]+(-0.72881870001)*x[2]**o+(0.757371069942)*x[2]
        s[3,3,1,0]=(-0.829528778744)*x[0]**o+(-0.676499470344)*x[0]+(-0.65512195457)*x[1]**o+(0.758109011229)*x[1]+(-0.5027027576)*x[2]**o+(-0.139493760854)*x[2]
        s[3,3,1,1]=(-0.489794277367)*x[0]**o+(-0.58772339266)*x[0]+(-0.920341394628)*x[1]**o+(-0.0302202079576)*x[1]+(-0.874433835334)*x[2]**o+(-0.382644983451)*x[2]
        s[3,3,2,0]=(-0.729507420376)*x[0]**o+(0.785351449428)*x[0]+(0.134356586244)*x[1]**o+(-0.786187017995)*x[1]+(-0.460746337928)*x[2]**o+(0.24750031597)*x[2]
        s[3,3,2,1]=(0.422343530499)*x[0]**o+(0.951298731532)*x[0]+(0.746519769375)*x[1]**o+(-0.787687730811)*x[1]+(-0.189033858381)*x[2]**o+(0.875049681221)*x[2]
        s[3,4,0,0]=(-0.784862616375)*x[0]**o+(0.605595736907)*x[0]+(-0.758269958875)*x[1]**o+(0.983503797662)*x[1]+(-0.897215719033)*x[2]**o+(0.434216381093)*x[2]
        s[3,4,0,1]=(0.959540582812)*x[0]**o+(0.74585782554)*x[0]+(0.00254441839877)*x[1]**o+(0.026781582564)*x[1]+(-0.0121835909437)*x[2]**o+(-0.166652448502)*x[2]
        s[3,4,1,0]=(-0.875170954413)*x[0]**o+(0.963228995313)*x[0]+(-0.198757196024)*x[1]**o+(-0.16115897441)*x[1]+(0.668387545829)*x[2]**o+(0.784441651668)*x[2]
        s[3,4,1,1]=(-0.0966644403845)*x[0]**o+(-0.0128070214998)*x[0]+(0.471059883662)*x[1]**o+(0.792978130351)*x[1]+(-0.709010757124)*x[2]**o+(-0.535329042416)*x[2]
        s[3,4,2,0]=(0.390795392416)*x[0]**o+(0.0465531497232)*x[0]+(-0.665065120082)*x[1]**o+(-0.637116189764)*x[1]+(0.671385406225)*x[2]**o+(-0.663428041906)*x[2]
        s[3,4,2,1]=(-0.962153478958)*x[0]**o+(0.255601758555)*x[0]+(-0.843511999734)*x[1]**o+(0.677818749903)*x[1]+(-0.273341883139)*x[2]**o+(0.846596068215)*x[2]
        ref[0,0,0,0]=(-0.400665270553)*x_ref[0]**o+(-0.465656794893)*x_ref[0]+(-0.103120094839)*x_ref[1]**o+(0.0464026120794)*x_ref[1]+(0.030596268742)*x_ref[2]**o+(0.00334628754684)*x_ref[2]
        ref[0,0,0,1]=(0.937480902128)*x_ref[0]**o+(0.0466658012652)*x_ref[0]+(0.817801316276)*x_ref[1]**o+(-0.142948501497)*x_ref[1]+(-0.946943305156)*x_ref[2]**o+(-0.633505380306)*x_ref[2]
        ref[0,0,1,0]=(-0.441314521722)*x_ref[0]**o+(-0.42022483938)*x_ref[0]+(-0.833228022905)*x_ref[1]**o+(-0.225426650714)*x_ref[1]+(0.332718715913)*x_ref[2]**o+(0.123261640846)*x_ref[2]
        ref[0,0,1,1]=(0.335875498268)*x_ref[0]**o+(0.983700594491)*x_ref[0]+(-0.214325088125)*x_ref[1]**o+(-0.893543563881)*x_ref[1]+(0.0631361789468)*x_ref[2]**o+(0.4467209081)*x_ref[2]
        ref[0,0,2,0]=(0.658215366848)*x_ref[0]**o+(-0.672295829784)*x_ref[0]+(-0.0137993447332)*x_ref[1]**o+(0.698632148806)*x_ref[1]+(-0.221572355182)*x_ref[2]**o+(0.956096660679)*x_ref[2]
        ref[0,0,2,1]=(-0.578795938537)*x_ref[0]**o+(0.750684927067)*x_ref[0]+(0.333363761953)*x_ref[1]**o+(-0.617804591638)*x_ref[1]+(-0.0469494394831)*x_ref[2]**o+(0.16187318118)*x_ref[2]
        ref[0,1,0,0]=(-0.0638097634115)*x_ref[0]**o+(-0.38507312448)*x_ref[0]+(-0.663736729005)*x_ref[1]**o+(-0.989965410511)*x_ref[1]+(0.230748674134)*x_ref[2]**o+(-0.830580398633)*x_ref[2]
        ref[0,1,0,1]=(-0.658956286059)*x_ref[0]**o+(0.345180478933)*x_ref[0]+(-0.544808816473)*x_ref[1]**o+(0.799798879144)*x_ref[1]+(-0.862695252289)*x_ref[2]**o+(0.757134401664)*x_ref[2]
        ref[0,1,1,0]=(0.456144819828)*x_ref[0]**o+(0.969408065204)*x_ref[0]+(0.488759322896)*x_ref[1]**o+(0.191259701658)*x_ref[1]+(-0.211530905801)*x_ref[2]**o+(0.0976530925065)*x_ref[2]
        ref[0,1,1,1]=(-0.160993701908)*x_ref[0]**o+(-0.768084723399)*x_ref[0]+(0.200776153747)*x_ref[1]**o+(0.380923444552)*x_ref[1]+(0.152388183214)*x_ref[2]**o+(0.0345495856869)*x_ref[2]
        ref[0,1,2,0]=(-0.249731094314)*x_ref[0]**o+(0.420921348364)*x_ref[0]+(-0.745813017956)*x_ref[1]**o+(-0.128319964472)*x_ref[1]+(-0.134590180255)*x_ref[2]**o+(-0.598364990007)*x_ref[2]
        ref[0,1,2,1]=(0.608497167751)*x_ref[0]**o+(0.134066300157)*x_ref[0]+(-0.219556933024)*x_ref[1]**o+(0.845095947325)*x_ref[1]+(-0.0910112208065)*x_ref[2]**o+(-0.919387374125)*x_ref[2]
        ref[0,2,0,0]=(-0.603722566149)*x_ref[0]**o+(-0.141049572403)*x_ref[0]+(-0.433072627843)*x_ref[1]**o+(-0.266640930301)*x_ref[1]+(-0.0520863163599)*x_ref[2]**o+(0.113826044084)*x_ref[2]
        ref[0,2,0,1]=(0.735600439472)*x_ref[0]**o+(-0.683666203432)*x_ref[0]+(0.439527311107)*x_ref[1]**o+(0.167685906293)*x_ref[1]+(-0.526256697628)*x_ref[2]**o+(-0.152028404784)*x_ref[2]
        ref[0,2,1,0]=(-0.484268475407)*x_ref[0]**o+(-0.1302369788)*x_ref[0]+(0.0996963541291)*x_ref[1]**o+(0.525432686771)*x_ref[1]+(0.915056396412)*x_ref[2]**o+(-0.93142015197)*x_ref[2]
        ref[0,2,1,1]=(0.103016217069)*x_ref[0]**o+(0.00517836018812)*x_ref[0]+(-0.28331822161)*x_ref[1]**o+(0.413800832478)*x_ref[1]+(0.94613674069)*x_ref[2]**o+(-0.287932854599)*x_ref[2]
        ref[0,2,2,0]=(0.512958587386)*x_ref[0]**o+(-0.851145503923)*x_ref[0]+(-0.535377708818)*x_ref[1]**o+(-0.379448444669)*x_ref[1]+(-0.238808818844)*x_ref[2]**o+(0.567146071008)*x_ref[2]
        ref[0,2,2,1]=(-0.141490825422)*x_ref[0]**o+(0.133948738844)*x_ref[0]+(0.336733111602)*x_ref[1]**o+(-0.223425201179)*x_ref[1]+(-0.646454138519)*x_ref[2]**o+(0.823936105317)*x_ref[2]
        ref[0,3,0,0]=(-0.938725695354)*x_ref[0]**o+(0.988131864396)*x_ref[0]+(-0.82501934096)*x_ref[1]**o+(-0.257156351526)*x_ref[1]+(-0.00349993655072)*x_ref[2]**o+(0.580041226419)*x_ref[2]
        ref[0,3,0,1]=(0.814169577155)*x_ref[0]**o+(-0.686109074376)*x_ref[0]+(-0.965459022554)*x_ref[1]**o+(0.10941355421)*x_ref[1]+(0.609050101353)*x_ref[2]**o+(-0.965588393623)*x_ref[2]
        ref[0,3,1,0]=(-0.820571213003)*x_ref[0]**o+(0.746749194774)*x_ref[0]+(0.710345174116)*x_ref[1]**o+(-0.892921665205)*x_ref[1]+(0.4004855541)*x_ref[2]**o+(-0.695117880583)*x_ref[2]
        ref[0,3,1,1]=(-0.122273772977)*x_ref[0]**o+(0.570499591543)*x_ref[0]+(0.795058034707)*x_ref[1]**o+(0.925412519293)*x_ref[1]+(-0.0932686873001)*x_ref[2]**o+(-0.686679792186)*x_ref[2]
        ref[0,3,2,0]=(-0.0042793185541)*x_ref[0]**o+(-0.821480764508)*x_ref[0]+(0.875015113726)*x_ref[1]**o+(0.649383518184)*x_ref[1]+(-0.288257569005)*x_ref[2]**o+(-0.743941827161)*x_ref[2]
        ref[0,3,2,1]=(0.974182567966)*x_ref[0]**o+(0.733657486071)*x_ref[0]+(0.780035494556)*x_ref[1]**o+(0.749708949611)*x_ref[1]+(-0.521673379431)*x_ref[2]**o+(0.673844867694)*x_ref[2]
        ref[0,4,0,0]=(-0.150057729107)*x_ref[0]**o+(0.102611114471)*x_ref[0]+(-0.0633864241155)*x_ref[1]**o+(-0.329289187178)*x_ref[1]+(0.315706711936)*x_ref[2]**o+(-0.554443679394)*x_ref[2]
        ref[0,4,0,1]=(0.564883543338)*x_ref[0]**o+(0.535008108501)*x_ref[0]+(0.815233609718)*x_ref[1]**o+(-0.766536697052)*x_ref[1]+(0.997595942191)*x_ref[2]**o+(0.991881374395)*x_ref[2]
        ref[0,4,1,0]=(-0.725748156838)*x_ref[0]**o+(-0.817693342658)*x_ref[0]+(0.501105656184)*x_ref[1]**o+(-0.331771590595)*x_ref[1]+(-0.757619653898)*x_ref[2]**o+(-0.885657271086)*x_ref[2]
        ref[0,4,1,1]=(-0.559814749411)*x_ref[0]**o+(0.665634435712)*x_ref[0]+(0.787002371365)*x_ref[1]**o+(-0.762034037705)*x_ref[1]+(0.508112385178)*x_ref[2]**o+(-0.985681857439)*x_ref[2]
        ref[0,4,2,0]=(0.504217223699)*x_ref[0]**o+(-0.361750429828)*x_ref[0]+(0.19299099109)*x_ref[1]**o+(0.338521927796)*x_ref[1]+(0.549199682759)*x_ref[2]**o+(0.891662590379)*x_ref[2]
        ref[0,4,2,1]=(-0.251870804657)*x_ref[0]**o+(0.776378068034)*x_ref[0]+(-0.769493074808)*x_ref[1]**o+(0.929429944731)*x_ref[1]+(0.679845951817)*x_ref[2]**o+(0.261179050197)*x_ref[2]
        ref[1,0,0,0]=(0.339823121204)*x_ref[0]**o+(0.212075564821)*x_ref[0]+(0.103901436655)*x_ref[1]**o+(0.887698805047)*x_ref[1]+(-0.467697404092)*x_ref[2]**o+(0.71153584324)*x_ref[2]
        ref[1,0,0,1]=(0.610147759489)*x_ref[0]**o+(-0.820866072636)*x_ref[0]+(0.505294661936)*x_ref[1]**o+(-0.206382410595)*x_ref[1]+(0.343889298437)*x_ref[2]**o+(0.165680393811)*x_ref[2]
        ref[1,0,1,0]=(-0.155342751495)*x_ref[0]**o+(-0.638727698114)*x_ref[0]+(-0.245328510129)*x_ref[1]**o+(0.816689513534)*x_ref[1]+(0.138847830114)*x_ref[2]**o+(0.286577642911)*x_ref[2]
        ref[1,0,1,1]=(0.804058022285)*x_ref[0]**o+(-0.922679564915)*x_ref[0]+(-0.755679292664)*x_ref[1]**o+(-0.409251039141)*x_ref[1]+(-0.697869565514)*x_ref[2]**o+(-0.0749517480557)*x_ref[2]
        ref[1,0,2,0]=(-0.345641262813)*x_ref[0]**o+(0.31066537094)*x_ref[0]+(0.165733093512)*x_ref[1]**o+(-0.733996040753)*x_ref[1]+(0.240017268146)*x_ref[2]**o+(0.981970338884)*x_ref[2]
        ref[1,0,2,1]=(-0.0394605078977)*x_ref[0]**o+(-0.671255389736)*x_ref[0]+(-0.58457671038)*x_ref[1]**o+(-0.308411795131)*x_ref[1]+(0.944299000242)*x_ref[2]**o+(0.0274884386004)*x_ref[2]
        ref[1,1,0,0]=(0.285078336337)*x_ref[0]**o+(-0.29609479737)*x_ref[0]+(0.851762966843)*x_ref[1]**o+(-0.56978612625)*x_ref[1]+(-0.416902609766)*x_ref[2]**o+(-0.209352333275)*x_ref[2]
        ref[1,1,0,1]=(0.165305243827)*x_ref[0]**o+(0.569657682766)*x_ref[0]+(-0.158936758165)*x_ref[1]**o+(-0.826897725956)*x_ref[1]+(-0.906153311902)*x_ref[2]**o+(-0.508208690461)*x_ref[2]
        ref[1,1,1,0]=(-0.168856536186)*x_ref[0]**o+(0.600931452022)*x_ref[0]+(0.515269032144)*x_ref[1]**o+(-0.615629628046)*x_ref[1]+(-0.248016004788)*x_ref[2]**o+(-0.236340748995)*x_ref[2]
        ref[1,1,1,1]=(-0.265342410313)*x_ref[0]**o+(0.546304098539)*x_ref[0]+(0.44451795278)*x_ref[1]**o+(0.53863749512)*x_ref[1]+(-0.400353954694)*x_ref[2]**o+(0.762091735577)*x_ref[2]
        ref[1,1,2,0]=(-0.828507588085)*x_ref[0]**o+(0.310151266525)*x_ref[0]+(0.767699919156)*x_ref[1]**o+(-0.802526284756)*x_ref[1]+(-0.12986443685)*x_ref[2]**o+(0.555810425336)*x_ref[2]
        ref[1,1,2,1]=(-0.0843531370201)*x_ref[0]**o+(0.138067935996)*x_ref[0]+(-0.819663672158)*x_ref[1]**o+(0.227177408663)*x_ref[1]+(-0.481488807321)*x_ref[2]**o+(0.277780608851)*x_ref[2]
        ref[1,2,0,0]=(0.843912054965)*x_ref[0]**o+(-0.532942686099)*x_ref[0]+(-0.12263392247)*x_ref[1]**o+(-0.918649820864)*x_ref[1]+(-0.898969788765)*x_ref[2]**o+(0.401718907796)*x_ref[2]
        ref[1,2,0,1]=(0.500181344385)*x_ref[0]**o+(-0.695081419586)*x_ref[0]+(0.49127001192)*x_ref[1]**o+(0.424707942699)*x_ref[1]+(-0.170525943009)*x_ref[2]**o+(-0.908857558417)*x_ref[2]
        ref[1,2,1,0]=(0.374108805296)*x_ref[0]**o+(-0.778311131014)*x_ref[0]+(-0.619173488016)*x_ref[1]**o+(-0.0906082150795)*x_ref[1]+(0.312783760308)*x_ref[2]**o+(0.50073883273)*x_ref[2]
        ref[1,2,1,1]=(0.474622980263)*x_ref[0]**o+(0.00968769992516)*x_ref[0]+(-0.246566756628)*x_ref[1]**o+(0.868983907653)*x_ref[1]+(-0.318924531295)*x_ref[2]**o+(-0.340314263979)*x_ref[2]
        ref[1,2,2,0]=(0.621111695462)*x_ref[0]**o+(0.655354521723)*x_ref[0]+(-0.992404590253)*x_ref[1]**o+(-0.497860913536)*x_ref[1]+(-0.963211421906)*x_ref[2]**o+(0.493347857543)*x_ref[2]
        ref[1,2,2,1]=(-0.32035342079)*x_ref[0]**o+(0.351692950156)*x_ref[0]+(-0.715457409869)*x_ref[1]**o+(0.317941180953)*x_ref[1]+(0.831453341583)*x_ref[2]**o+(0.330720124015)*x_ref[2]
        ref[1,3,0,0]=(-0.775605593731)*x_ref[0]**o+(-0.985544306967)*x_ref[0]+(0.838313790256)*x_ref[1]**o+(-0.0515692297878)*x_ref[1]+(0.6757488996)*x_ref[2]**o+(-0.398401163936)*x_ref[2]
        ref[1,3,0,1]=(0.129462614525)*x_ref[0]**o+(-0.225290874002)*x_ref[0]+(0.1627137973)*x_ref[1]**o+(0.96490154212)*x_ref[1]+(-0.0348196505137)*x_ref[2]**o+(0.00501682452837)*x_ref[2]
        ref[1,3,1,0]=(-0.774906442617)*x_ref[0]**o+(0.554579514554)*x_ref[0]+(0.68683898073)*x_ref[1]**o+(0.687422147497)*x_ref[1]+(0.812740760401)*x_ref[2]**o+(0.650574541761)*x_ref[2]
        ref[1,3,1,1]=(0.371414654481)*x_ref[0]**o+(-0.460895148602)*x_ref[0]+(-0.60035497386)*x_ref[1]**o+(0.382490052311)*x_ref[1]+(0.348365164241)*x_ref[2]**o+(0.700564991456)*x_ref[2]
        ref[1,3,2,0]=(0.0627821130997)*x_ref[0]**o+(-0.108341459129)*x_ref[0]+(-0.459931858236)*x_ref[1]**o+(-0.84999694423)*x_ref[1]+(-0.369143500965)*x_ref[2]**o+(-0.204962250898)*x_ref[2]
        ref[1,3,2,1]=(0.940231653623)*x_ref[0]**o+(0.432366518466)*x_ref[0]+(0.965573462127)*x_ref[1]**o+(-0.400536850676)*x_ref[1]+(0.627651239821)*x_ref[2]**o+(0.541970384011)*x_ref[2]
        ref[1,4,0,0]=(-0.905529267496)*x_ref[0]**o+(-0.728637228667)*x_ref[0]+(0.528565304374)*x_ref[1]**o+(-0.0791953805449)*x_ref[1]+(0.278340538308)*x_ref[2]**o+(0.0829173042009)*x_ref[2]
        ref[1,4,0,1]=(0.196476386515)*x_ref[0]**o+(0.960964133173)*x_ref[0]+(-0.975854821639)*x_ref[1]**o+(-0.231600097119)*x_ref[1]+(0.462289007605)*x_ref[2]**o+(-0.41548631733)*x_ref[2]
        ref[1,4,1,0]=(0.133593658437)*x_ref[0]**o+(0.27713134658)*x_ref[0]+(-0.626190133661)*x_ref[1]**o+(0.22128472818)*x_ref[1]+(-0.0310045745464)*x_ref[2]**o+(-0.308952151364)*x_ref[2]
        ref[1,4,1,1]=(-0.540782388402)*x_ref[0]**o+(0.782610613508)*x_ref[0]+(-0.121818568845)*x_ref[1]**o+(-0.481514588156)*x_ref[1]+(0.92789834048)*x_ref[2]**o+(-0.053669614999)*x_ref[2]
        ref[1,4,2,0]=(0.228180064246)*x_ref[0]**o+(-0.387791183012)*x_ref[0]+(-0.809517349857)*x_ref[1]**o+(0.104233744095)*x_ref[1]+(-0.010561196572)*x_ref[2]**o+(0.812403786777)*x_ref[2]
        ref[1,4,2,1]=(-0.213527688719)*x_ref[0]**o+(0.444377334059)*x_ref[0]+(0.250474600963)*x_ref[1]**o+(-0.628454953053)*x_ref[1]+(-0.149400612708)*x_ref[2]**o+(-0.362500181895)*x_ref[2]
        ref[2,0,0,0]=(0.424970215816)*x_ref[0]**o+(-0.409634055175)*x_ref[0]+(-0.209831967682)*x_ref[1]**o+(0.777759135142)*x_ref[1]+(0.904286801356)*x_ref[2]**o+(-0.533118859622)*x_ref[2]
        ref[2,0,0,1]=(-0.0634924164631)*x_ref[0]**o+(0.61665557894)*x_ref[0]+(0.886000765027)*x_ref[1]**o+(-0.283672331088)*x_ref[1]+(-0.104923941729)*x_ref[2]**o+(0.647966459575)*x_ref[2]
        ref[2,0,1,0]=(-0.601180947882)*x_ref[0]**o+(-0.853176409106)*x_ref[0]+(0.589049472252)*x_ref[1]**o+(0.947188905512)*x_ref[1]+(0.937016925257)*x_ref[2]**o+(-0.528933622785)*x_ref[2]
        ref[2,0,1,1]=(-0.493108989214)*x_ref[0]**o+(0.123452031172)*x_ref[0]+(0.579015108934)*x_ref[1]**o+(-0.224656610786)*x_ref[1]+(-0.351966996224)*x_ref[2]**o+(-0.58667858467)*x_ref[2]
        ref[2,0,2,0]=(0.342923581511)*x_ref[0]**o+(-0.787209375031)*x_ref[0]+(0.0689438410554)*x_ref[1]**o+(0.624797184447)*x_ref[1]+(0.508789531503)*x_ref[2]**o+(0.17151319396)*x_ref[2]
        ref[2,0,2,1]=(-0.875568056219)*x_ref[0]**o+(-0.113031987088)*x_ref[0]+(-0.0609746766081)*x_ref[1]**o+(0.196111602161)*x_ref[1]+(-0.00186601407674)*x_ref[2]**o+(-0.695244625545)*x_ref[2]
        ref[2,1,0,0]=(0.0760124938281)*x_ref[0]**o+(0.878269236128)*x_ref[0]+(0.424404663038)*x_ref[1]**o+(0.382191251895)*x_ref[1]+(-0.799519816473)*x_ref[2]**o+(0.756150518781)*x_ref[2]
        ref[2,1,0,1]=(-0.620026715848)*x_ref[0]**o+(0.120584354877)*x_ref[0]+(0.794571749541)*x_ref[1]**o+(0.775886892396)*x_ref[1]+(-0.455364446354)*x_ref[2]**o+(-0.25890753368)*x_ref[2]
        ref[2,1,1,0]=(-0.434917779388)*x_ref[0]**o+(0.893805701213)*x_ref[0]+(-0.359966318659)*x_ref[1]**o+(-0.92549769048)*x_ref[1]+(0.632194530492)*x_ref[2]**o+(0.783055504124)*x_ref[2]
        ref[2,1,1,1]=(-0.30435434794)*x_ref[0]**o+(-0.132800060169)*x_ref[0]+(0.200039404417)*x_ref[1]**o+(-0.690955124262)*x_ref[1]+(-0.984472889901)*x_ref[2]**o+(0.0242811209734)*x_ref[2]
        ref[2,1,2,0]=(-0.223815583384)*x_ref[0]**o+(0.592762849244)*x_ref[0]+(0.281436806248)*x_ref[1]**o+(0.822974431302)*x_ref[1]+(-0.0648638855118)*x_ref[2]**o+(-0.798035298439)*x_ref[2]
        ref[2,1,2,1]=(-0.847339863019)*x_ref[0]**o+(0.603483610026)*x_ref[0]+(0.865918681346)*x_ref[1]**o+(-0.0625995244022)*x_ref[1]+(0.767004105382)*x_ref[2]**o+(0.536886296262)*x_ref[2]
        ref[2,2,0,0]=(0.0907868991281)*x_ref[0]**o+(0.0361485757336)*x_ref[0]+(0.725787806874)*x_ref[1]**o+(-0.955232659581)*x_ref[1]+(0.015387691978)*x_ref[2]**o+(-0.969939112182)*x_ref[2]
        ref[2,2,0,1]=(-0.702771278477)*x_ref[0]**o+(-0.376611311235)*x_ref[0]+(0.321676098666)*x_ref[1]**o+(0.2741544323)*x_ref[1]+(-0.490015464554)*x_ref[2]**o+(-0.287278266636)*x_ref[2]
        ref[2,2,1,0]=(-0.239838875645)*x_ref[0]**o+(-0.46816615613)*x_ref[0]+(0.208010055752)*x_ref[1]**o+(0.343192409907)*x_ref[1]+(-0.752447682834)*x_ref[2]**o+(-0.450307042264)*x_ref[2]
        ref[2,2,1,1]=(-0.377645472017)*x_ref[0]**o+(0.979388261808)*x_ref[0]+(0.899484011996)*x_ref[1]**o+(0.278589875985)*x_ref[1]+(-0.0566572805419)*x_ref[2]**o+(-0.329814353957)*x_ref[2]
        ref[2,2,2,0]=(0.329972082761)*x_ref[0]**o+(-0.40893298634)*x_ref[0]+(0.14025500949)*x_ref[1]**o+(-0.915605553253)*x_ref[1]+(-0.743877103988)*x_ref[2]**o+(0.0625993015596)*x_ref[2]
        ref[2,2,2,1]=(-0.426922382741)*x_ref[0]**o+(0.638893012106)*x_ref[0]+(0.775217826454)*x_ref[1]**o+(0.64924098439)*x_ref[1]+(-0.157863119278)*x_ref[2]**o+(-0.785891322569)*x_ref[2]
        ref[2,3,0,0]=(-0.573635249481)*x_ref[0]**o+(0.466378353545)*x_ref[0]+(-0.968280361551)*x_ref[1]**o+(-0.521040950167)*x_ref[1]+(-0.979861738542)*x_ref[2]**o+(0.293316009588)*x_ref[2]
        ref[2,3,0,1]=(-0.987445004264)*x_ref[0]**o+(-0.855558575123)*x_ref[0]+(0.703881586225)*x_ref[1]**o+(0.393607926606)*x_ref[1]+(0.615218365579)*x_ref[2]**o+(0.865070564586)*x_ref[2]
        ref[2,3,1,0]=(0.243032241627)*x_ref[0]**o+(0.447193793692)*x_ref[0]+(0.121835082448)*x_ref[1]**o+(0.253731889763)*x_ref[1]+(0.482861064484)*x_ref[2]**o+(0.0857661253915)*x_ref[2]
        ref[2,3,1,1]=(0.630205092814)*x_ref[0]**o+(-0.230885016198)*x_ref[0]+(-0.00423203911773)*x_ref[1]**o+(0.560066624606)*x_ref[1]+(-0.850065237172)*x_ref[2]**o+(-0.0608202441365)*x_ref[2]
        ref[2,3,2,0]=(0.956674282321)*x_ref[0]**o+(0.478516276613)*x_ref[0]+(-0.142801297861)*x_ref[1]**o+(-0.470299296114)*x_ref[1]+(0.947735333619)*x_ref[2]**o+(0.343691324696)*x_ref[2]
        ref[2,3,2,1]=(0.366910635243)*x_ref[0]**o+(-0.0831991185812)*x_ref[0]+(-0.828901851379)*x_ref[1]**o+(-0.537917965468)*x_ref[1]+(-0.581342127867)*x_ref[2]**o+(-0.307660153071)*x_ref[2]
        ref[2,4,0,0]=(0.340464857467)*x_ref[0]**o+(0.881022180176)*x_ref[0]+(-0.223178224315)*x_ref[1]**o+(0.0108952156846)*x_ref[1]+(-0.130766988014)*x_ref[2]**o+(-0.655755053638)*x_ref[2]
        ref[2,4,0,1]=(-0.142352024958)*x_ref[0]**o+(-0.279134821704)*x_ref[0]+(0.696577046583)*x_ref[1]**o+(-0.913765778235)*x_ref[1]+(0.674571789555)*x_ref[2]**o+(-0.236086993066)*x_ref[2]
        ref[2,4,1,0]=(0.993240530889)*x_ref[0]**o+(0.152584699472)*x_ref[0]+(0.364843511953)*x_ref[1]**o+(0.900205378533)*x_ref[1]+(0.397560553857)*x_ref[2]**o+(0.334533062647)*x_ref[2]
        ref[2,4,1,1]=(-0.261396791645)*x_ref[0]**o+(0.96949114295)*x_ref[0]+(0.604187009028)*x_ref[1]**o+(-0.667369171782)*x_ref[1]+(-0.313439370957)*x_ref[2]**o+(0.448859840994)*x_ref[2]
        ref[2,4,2,0]=(-0.508273068959)*x_ref[0]**o+(0.473127945457)*x_ref[0]+(0.512686326084)*x_ref[1]**o+(-0.872890815913)*x_ref[1]+(-0.515333454444)*x_ref[2]**o+(-0.578402077636)*x_ref[2]
        ref[2,4,2,1]=(-0.78684369593)*x_ref[0]**o+(0.719254214286)*x_ref[0]+(-0.857414812359)*x_ref[1]**o+(-0.260538452293)*x_ref[1]+(0.940982479653)*x_ref[2]**o+(-0.94620100965)*x_ref[2]
        ref[3,0,0,0]=(0.741419615809)*x_ref[0]**o+(0.535310119259)*x_ref[0]+(-0.396521376661)*x_ref[1]**o+(0.238209214692)*x_ref[1]+(0.550440543811)*x_ref[2]**o+(0.592067475252)*x_ref[2]
        ref[3,0,0,1]=(0.423030333878)*x_ref[0]**o+(0.434870473967)*x_ref[0]+(-0.599556117755)*x_ref[1]**o+(0.299434424134)*x_ref[1]+(0.120835992748)*x_ref[2]**o+(-0.314011916578)*x_ref[2]
        ref[3,0,1,0]=(-0.326589197488)*x_ref[0]**o+(-0.5670140013)*x_ref[0]+(0.0548382156857)*x_ref[1]**o+(0.689609176876)*x_ref[1]+(0.257405458159)*x_ref[2]**o+(-0.896175631279)*x_ref[2]
        ref[3,0,1,1]=(0.221120334733)*x_ref[0]**o+(-0.273395088002)*x_ref[0]+(0.678993292365)*x_ref[1]**o+(-0.261592406628)*x_ref[1]+(0.0129202430825)*x_ref[2]**o+(0.148164299785)*x_ref[2]
        ref[3,0,2,0]=(0.894503166055)*x_ref[0]**o+(-0.468004653303)*x_ref[0]+(-0.103574702833)*x_ref[1]**o+(-0.390725885766)*x_ref[1]+(0.081712057468)*x_ref[2]**o+(-0.507015846204)*x_ref[2]
        ref[3,0,2,1]=(0.712909263678)*x_ref[0]**o+(-0.412420758959)*x_ref[0]+(-0.620377453769)*x_ref[1]**o+(-0.166241669671)*x_ref[1]+(-0.738733737609)*x_ref[2]**o+(-0.757426176741)*x_ref[2]
        ref[3,1,0,0]=(-0.166116194562)*x_ref[0]**o+(0.988666848357)*x_ref[0]+(-0.483399298528)*x_ref[1]**o+(0.777960698485)*x_ref[1]+(-0.374839973139)*x_ref[2]**o+(0.914310220752)*x_ref[2]
        ref[3,1,0,1]=(-0.578374538082)*x_ref[0]**o+(0.228605767869)*x_ref[0]+(0.120418953818)*x_ref[1]**o+(-0.693317296354)*x_ref[1]+(-0.691997757818)*x_ref[2]**o+(0.551252364781)*x_ref[2]
        ref[3,1,1,0]=(-0.384863493904)*x_ref[0]**o+(0.300215711827)*x_ref[0]+(0.948187183083)*x_ref[1]**o+(-0.184464558346)*x_ref[1]+(-0.7671733816)*x_ref[2]**o+(0.307217058571)*x_ref[2]
        ref[3,1,1,1]=(0.578689722929)*x_ref[0]**o+(0.885714287205)*x_ref[0]+(0.722247905148)*x_ref[1]**o+(0.933475647861)*x_ref[1]+(0.211727836913)*x_ref[2]**o+(0.711589357201)*x_ref[2]
        ref[3,1,2,0]=(-0.116046568217)*x_ref[0]**o+(-0.735782802049)*x_ref[0]+(-0.968041163959)*x_ref[1]**o+(0.314957910843)*x_ref[1]+(0.639581098814)*x_ref[2]**o+(-0.566524915874)*x_ref[2]
        ref[3,1,2,1]=(0.0690920573575)*x_ref[0]**o+(-0.0344562175408)*x_ref[0]+(-0.729181462751)*x_ref[1]**o+(-0.352992891115)*x_ref[1]+(-0.230186276771)*x_ref[2]**o+(-0.392393764843)*x_ref[2]
        ref[3,2,0,0]=(-0.897572361738)*x_ref[0]**o+(0.42292848631)*x_ref[0]+(0.0402102445405)*x_ref[1]**o+(-0.695803229507)*x_ref[1]+(0.657341764505)*x_ref[2]**o+(0.534681573441)*x_ref[2]
        ref[3,2,0,1]=(-0.883898106985)*x_ref[0]**o+(0.563240147064)*x_ref[0]+(0.596180998828)*x_ref[1]**o+(0.278590998454)*x_ref[1]+(-0.686188881246)*x_ref[2]**o+(0.827407211045)*x_ref[2]
        ref[3,2,1,0]=(-0.896992911933)*x_ref[0]**o+(0.349871189445)*x_ref[0]+(0.959261344864)*x_ref[1]**o+(0.567834800455)*x_ref[1]+(0.624336079204)*x_ref[2]**o+(-0.853743864993)*x_ref[2]
        ref[3,2,1,1]=(0.947681143718)*x_ref[0]**o+(0.563362524754)*x_ref[0]+(0.398321082655)*x_ref[1]**o+(0.217952803297)*x_ref[1]+(0.243654787599)*x_ref[2]**o+(-0.927517802601)*x_ref[2]
        ref[3,2,2,0]=(0.330842075788)*x_ref[0]**o+(0.630950656819)*x_ref[0]+(-0.469619054343)*x_ref[1]**o+(-0.354739266808)*x_ref[1]+(0.253515569858)*x_ref[2]**o+(0.635440000647)*x_ref[2]
        ref[3,2,2,1]=(0.729280956878)*x_ref[0]**o+(0.734399392463)*x_ref[0]+(0.800616353182)*x_ref[1]**o+(-0.764367237486)*x_ref[1]+(0.90307570667)*x_ref[2]**o+(-0.957618675411)*x_ref[2]
        ref[3,3,0,0]=(-0.783664290925)*x_ref[0]**o+(-0.609524665456)*x_ref[0]+(0.0126829605834)*x_ref[1]**o+(0.396479443362)*x_ref[1]+(0.262779944509)*x_ref[2]**o+(-0.666191777094)*x_ref[2]
        ref[3,3,0,1]=(0.412930217814)*x_ref[0]**o+(0.993234310311)*x_ref[0]+(-0.123107776537)*x_ref[1]**o+(-0.156132010898)*x_ref[1]+(-0.72881870001)*x_ref[2]**o+(0.757371069942)*x_ref[2]
        ref[3,3,1,0]=(-0.829528778744)*x_ref[0]**o+(-0.676499470344)*x_ref[0]+(-0.65512195457)*x_ref[1]**o+(0.758109011229)*x_ref[1]+(-0.5027027576)*x_ref[2]**o+(-0.139493760854)*x_ref[2]
        ref[3,3,1,1]=(-0.489794277367)*x_ref[0]**o+(-0.58772339266)*x_ref[0]+(-0.920341394628)*x_ref[1]**o+(-0.0302202079576)*x_ref[1]+(-0.874433835334)*x_ref[2]**o+(-0.382644983451)*x_ref[2]
        ref[3,3,2,0]=(-0.729507420376)*x_ref[0]**o+(0.785351449428)*x_ref[0]+(0.134356586244)*x_ref[1]**o+(-0.786187017995)*x_ref[1]+(-0.460746337928)*x_ref[2]**o+(0.24750031597)*x_ref[2]
        ref[3,3,2,1]=(0.422343530499)*x_ref[0]**o+(0.951298731532)*x_ref[0]+(0.746519769375)*x_ref[1]**o+(-0.787687730811)*x_ref[1]+(-0.189033858381)*x_ref[2]**o+(0.875049681221)*x_ref[2]
        ref[3,4,0,0]=(-0.784862616375)*x_ref[0]**o+(0.605595736907)*x_ref[0]+(-0.758269958875)*x_ref[1]**o+(0.983503797662)*x_ref[1]+(-0.897215719033)*x_ref[2]**o+(0.434216381093)*x_ref[2]
        ref[3,4,0,1]=(0.959540582812)*x_ref[0]**o+(0.74585782554)*x_ref[0]+(0.00254441839877)*x_ref[1]**o+(0.026781582564)*x_ref[1]+(-0.0121835909437)*x_ref[2]**o+(-0.166652448502)*x_ref[2]
        ref[3,4,1,0]=(-0.875170954413)*x_ref[0]**o+(0.963228995313)*x_ref[0]+(-0.198757196024)*x_ref[1]**o+(-0.16115897441)*x_ref[1]+(0.668387545829)*x_ref[2]**o+(0.784441651668)*x_ref[2]
        ref[3,4,1,1]=(-0.0966644403845)*x_ref[0]**o+(-0.0128070214998)*x_ref[0]+(0.471059883662)*x_ref[1]**o+(0.792978130351)*x_ref[1]+(-0.709010757124)*x_ref[2]**o+(-0.535329042416)*x_ref[2]
        ref[3,4,2,0]=(0.390795392416)*x_ref[0]**o+(0.0465531497232)*x_ref[0]+(-0.665065120082)*x_ref[1]**o+(-0.637116189764)*x_ref[1]+(0.671385406225)*x_ref[2]**o+(-0.663428041906)*x_ref[2]
        ref[3,4,2,1]=(-0.962153478958)*x_ref[0]**o+(0.255601758555)*x_ref[0]+(-0.843511999734)*x_ref[1]**o+(0.677818749903)*x_ref[1]+(-0.273341883139)*x_ref[2]**o+(0.846596068215)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_Solution_rank0(self):
      """
      tests interpolate for rank 0 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from Solution onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(),dim=dim)
      s=Data(0,(),w)
      ref=Data(0,(),w_ref)
      if dim==2:
        s=(0.887413974092)*x[0]**o+(-0.7148023493)*x[0]+(0.39224588792)*x[1]**o+(0.712669303726)*x[1]
        ref=(0.887413974092)*x_ref[0]**o+(-0.7148023493)*x_ref[0]+(0.39224588792)*x_ref[1]**o+(0.712669303726)*x_ref[1]
      else:
        s=(0.435927213379)*x[0]**o+(-0.406836076828)*x[0]+(0.240829091966)*x[1]**o+(-0.782431907383)*x[1]+(0.968499178848)*x[2]**o+(0.11248632948)*x[2]
        ref=(0.435927213379)*x_ref[0]**o+(-0.406836076828)*x_ref[0]+(0.240829091966)*x_ref[1]**o+(-0.782431907383)*x_ref[1]+(0.968499178848)*x_ref[2]**o+(0.11248632948)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_Solution_rank1(self):
      """
      tests interpolate for rank 1 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from Solution onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(2,),dim=dim)
      s=Data(0,(2,),w)
      ref=Data(0,(2,),w_ref)
      if dim==2:
        s[0]=(0.497549313651)*x[0]**o+(0.342704471642)*x[0]+(0.576164780397)*x[1]**o+(0.331889751836)*x[1]
        s[1]=(0.245818716931)*x[0]**o+(0.275654781295)*x[0]+(0.444526351413)*x[1]**o+(-0.783404039317)*x[1]
        ref[0]=(0.497549313651)*x_ref[0]**o+(0.342704471642)*x_ref[0]+(0.576164780397)*x_ref[1]**o+(0.331889751836)*x_ref[1]
        ref[1]=(0.245818716931)*x_ref[0]**o+(0.275654781295)*x_ref[0]+(0.444526351413)*x_ref[1]**o+(-0.783404039317)*x_ref[1]
      else:
        s[0]=(0.234557077894)*x[0]**o+(-0.201831350239)*x[0]+(-0.869318965642)*x[1]**o+(-0.715212786062)*x[1]+(0.45304187029)*x[2]**o+(0.744681443133)*x[2]
        s[1]=(0.00903347119997)*x[0]**o+(0.746691772493)*x[0]+(-0.538042235565)*x[1]**o+(0.932594018318)*x[1]+(0.58848513662)*x[2]**o+(-0.977139295615)*x[2]
        ref[0]=(0.234557077894)*x_ref[0]**o+(-0.201831350239)*x_ref[0]+(-0.869318965642)*x_ref[1]**o+(-0.715212786062)*x_ref[1]+(0.45304187029)*x_ref[2]**o+(0.744681443133)*x_ref[2]
        ref[1]=(0.00903347119997)*x_ref[0]**o+(0.746691772493)*x_ref[0]+(-0.538042235565)*x_ref[1]**o+(0.932594018318)*x_ref[1]+(0.58848513662)*x_ref[2]**o+(-0.977139295615)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_Solution_rank2(self):
      """
      tests interpolate for rank 2 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from Solution onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5),dim=dim)
      s=Data(0,(4, 5),w)
      ref=Data(0,(4, 5),w_ref)
      if dim==2:
        s[0,0]=(-0.662397963931)*x[0]**o+(-0.398404018087)*x[0]+(-0.725256273358)*x[1]**o+(0.423211349101)*x[1]
        s[0,1]=(-0.167952506192)*x[0]**o+(-0.718540804013)*x[0]+(-0.781553467855)*x[1]**o+(0.0228480720358)*x[1]
        s[0,2]=(-0.13358806923)*x[0]**o+(0.398616852411)*x[0]+(-0.311696313693)*x[1]**o+(-0.631789739932)*x[1]
        s[0,3]=(0.855686693303)*x[0]**o+(0.630881886632)*x[0]+(-0.814763888676)*x[1]**o+(-0.614872151046)*x[1]
        s[0,4]=(0.937592800976)*x[0]**o+(-0.348259700685)*x[0]+(0.506034550337)*x[1]**o+(0.0144403536426)*x[1]
        s[1,0]=(-0.994330568705)*x[0]**o+(0.0966914505526)*x[0]+(0.353136362033)*x[1]**o+(0.241811294194)*x[1]
        s[1,1]=(-0.88991407014)*x[0]**o+(0.0439813622179)*x[0]+(0.922533826895)*x[1]**o+(0.0792543536698)*x[1]
        s[1,2]=(-0.57254546238)*x[0]**o+(0.132011764714)*x[0]+(0.919045869867)*x[1]**o+(-0.781631515229)*x[1]
        s[1,3]=(0.125157999689)*x[0]**o+(-0.239227948738)*x[0]+(-0.904449723359)*x[1]**o+(-0.726384322833)*x[1]
        s[1,4]=(0.157119099313)*x[0]**o+(0.149352102671)*x[0]+(-0.715944585517)*x[1]**o+(0.559674967195)*x[1]
        s[2,0]=(0.655983809801)*x[0]**o+(-0.0230135667792)*x[0]+(-0.963391677483)*x[1]**o+(-0.705626017783)*x[1]
        s[2,1]=(-0.405526943481)*x[0]**o+(0.528349592338)*x[0]+(0.881082721559)*x[1]**o+(0.683763434255)*x[1]
        s[2,2]=(0.747664855298)*x[0]**o+(-0.336598258345)*x[0]+(-0.15756896687)*x[1]**o+(0.657585259369)*x[1]
        s[2,3]=(-0.206021660879)*x[0]**o+(0.496139719147)*x[0]+(-0.0609353622346)*x[1]**o+(0.532090076594)*x[1]
        s[2,4]=(-0.870347013263)*x[0]**o+(0.540121560488)*x[0]+(0.373986056701)*x[1]**o+(0.586491974916)*x[1]
        s[3,0]=(-0.981237374017)*x[0]**o+(0.22435680807)*x[0]+(0.988571281429)*x[1]**o+(-0.66346591076)*x[1]
        s[3,1]=(-0.939588996339)*x[0]**o+(-0.350214307841)*x[0]+(-0.269044584808)*x[1]**o+(0.115043285059)*x[1]
        s[3,2]=(-0.401479061703)*x[0]**o+(-0.922855198541)*x[0]+(0.148477007737)*x[1]**o+(-0.64051890925)*x[1]
        s[3,3]=(0.324132581573)*x[0]**o+(0.328081603393)*x[0]+(0.938536399063)*x[1]**o+(0.742951802582)*x[1]
        s[3,4]=(-0.476640426723)*x[0]**o+(0.485808406776)*x[0]+(-0.649177668213)*x[1]**o+(0.925209215964)*x[1]
        ref[0,0]=(-0.662397963931)*x_ref[0]**o+(-0.398404018087)*x_ref[0]+(-0.725256273358)*x_ref[1]**o+(0.423211349101)*x_ref[1]
        ref[0,1]=(-0.167952506192)*x_ref[0]**o+(-0.718540804013)*x_ref[0]+(-0.781553467855)*x_ref[1]**o+(0.0228480720358)*x_ref[1]
        ref[0,2]=(-0.13358806923)*x_ref[0]**o+(0.398616852411)*x_ref[0]+(-0.311696313693)*x_ref[1]**o+(-0.631789739932)*x_ref[1]
        ref[0,3]=(0.855686693303)*x_ref[0]**o+(0.630881886632)*x_ref[0]+(-0.814763888676)*x_ref[1]**o+(-0.614872151046)*x_ref[1]
        ref[0,4]=(0.937592800976)*x_ref[0]**o+(-0.348259700685)*x_ref[0]+(0.506034550337)*x_ref[1]**o+(0.0144403536426)*x_ref[1]
        ref[1,0]=(-0.994330568705)*x_ref[0]**o+(0.0966914505526)*x_ref[0]+(0.353136362033)*x_ref[1]**o+(0.241811294194)*x_ref[1]
        ref[1,1]=(-0.88991407014)*x_ref[0]**o+(0.0439813622179)*x_ref[0]+(0.922533826895)*x_ref[1]**o+(0.0792543536698)*x_ref[1]
        ref[1,2]=(-0.57254546238)*x_ref[0]**o+(0.132011764714)*x_ref[0]+(0.919045869867)*x_ref[1]**o+(-0.781631515229)*x_ref[1]
        ref[1,3]=(0.125157999689)*x_ref[0]**o+(-0.239227948738)*x_ref[0]+(-0.904449723359)*x_ref[1]**o+(-0.726384322833)*x_ref[1]
        ref[1,4]=(0.157119099313)*x_ref[0]**o+(0.149352102671)*x_ref[0]+(-0.715944585517)*x_ref[1]**o+(0.559674967195)*x_ref[1]
        ref[2,0]=(0.655983809801)*x_ref[0]**o+(-0.0230135667792)*x_ref[0]+(-0.963391677483)*x_ref[1]**o+(-0.705626017783)*x_ref[1]
        ref[2,1]=(-0.405526943481)*x_ref[0]**o+(0.528349592338)*x_ref[0]+(0.881082721559)*x_ref[1]**o+(0.683763434255)*x_ref[1]
        ref[2,2]=(0.747664855298)*x_ref[0]**o+(-0.336598258345)*x_ref[0]+(-0.15756896687)*x_ref[1]**o+(0.657585259369)*x_ref[1]
        ref[2,3]=(-0.206021660879)*x_ref[0]**o+(0.496139719147)*x_ref[0]+(-0.0609353622346)*x_ref[1]**o+(0.532090076594)*x_ref[1]
        ref[2,4]=(-0.870347013263)*x_ref[0]**o+(0.540121560488)*x_ref[0]+(0.373986056701)*x_ref[1]**o+(0.586491974916)*x_ref[1]
        ref[3,0]=(-0.981237374017)*x_ref[0]**o+(0.22435680807)*x_ref[0]+(0.988571281429)*x_ref[1]**o+(-0.66346591076)*x_ref[1]
        ref[3,1]=(-0.939588996339)*x_ref[0]**o+(-0.350214307841)*x_ref[0]+(-0.269044584808)*x_ref[1]**o+(0.115043285059)*x_ref[1]
        ref[3,2]=(-0.401479061703)*x_ref[0]**o+(-0.922855198541)*x_ref[0]+(0.148477007737)*x_ref[1]**o+(-0.64051890925)*x_ref[1]
        ref[3,3]=(0.324132581573)*x_ref[0]**o+(0.328081603393)*x_ref[0]+(0.938536399063)*x_ref[1]**o+(0.742951802582)*x_ref[1]
        ref[3,4]=(-0.476640426723)*x_ref[0]**o+(0.485808406776)*x_ref[0]+(-0.649177668213)*x_ref[1]**o+(0.925209215964)*x_ref[1]
      else:
        s[0,0]=(-0.564914581723)*x[0]**o+(0.258795214127)*x[0]+(-0.220020909772)*x[1]**o+(0.810531687745)*x[1]+(0.454950462482)*x[2]**o+(0.601383225786)*x[2]
        s[0,1]=(-0.11600536761)*x[0]**o+(-0.690336577243)*x[0]+(-0.835351354712)*x[1]**o+(-0.158331093203)*x[1]+(0.645069697527)*x[2]**o+(0.126534640518)*x[2]
        s[0,2]=(-0.350053630252)*x[0]**o+(-0.890479764661)*x[0]+(-0.280608934641)*x[1]**o+(-0.855574890474)*x[1]+(-0.988708278729)*x[2]**o+(0.0791783723801)*x[2]
        s[0,3]=(0.987927769717)*x[0]**o+(-0.347061742182)*x[0]+(0.501082056996)*x[1]**o+(-0.708442600496)*x[1]+(0.948114708446)*x[2]**o+(0.211831323577)*x[2]
        s[0,4]=(-0.930773949241)*x[0]**o+(-0.545284390651)*x[0]+(0.198879874511)*x[1]**o+(-0.221794025714)*x[1]+(-0.447838984339)*x[2]**o+(-0.610640502396)*x[2]
        s[1,0]=(-0.156742047015)*x[0]**o+(0.843077944384)*x[0]+(-0.28777246252)*x[1]**o+(0.638858613604)*x[1]+(-0.503228005834)*x[2]**o+(-0.960025936087)*x[2]
        s[1,1]=(-0.270691254249)*x[0]**o+(0.0299500422959)*x[0]+(-0.894145166738)*x[1]**o+(0.85820171409)*x[1]+(0.61650626344)*x[2]**o+(-0.480895585635)*x[2]
        s[1,2]=(0.896861224301)*x[0]**o+(0.00816715231085)*x[0]+(-0.203903225872)*x[1]**o+(-0.126928738689)*x[1]+(-0.805120272595)*x[2]**o+(0.98588145789)*x[2]
        s[1,3]=(-0.21275125769)*x[0]**o+(0.458722054632)*x[0]+(0.153520900666)*x[1]**o+(0.512505902794)*x[1]+(0.576294451799)*x[2]**o+(0.604008619969)*x[2]
        s[1,4]=(-0.882533391878)*x[0]**o+(0.616362261559)*x[0]+(0.155750337317)*x[1]**o+(-0.0162797277104)*x[1]+(0.226933640416)*x[2]**o+(-0.118618269284)*x[2]
        s[2,0]=(-0.149844951974)*x[0]**o+(0.601452947295)*x[0]+(-0.447624109479)*x[1]**o+(-0.854580861366)*x[1]+(0.392948004882)*x[2]**o+(0.0820844858757)*x[2]
        s[2,1]=(-0.256158991676)*x[0]**o+(0.161885127371)*x[0]+(0.702594613311)*x[1]**o+(0.598617952297)*x[1]+(-0.338391564856)*x[2]**o+(0.669697240278)*x[2]
        s[2,2]=(0.441512025449)*x[0]**o+(-0.761281221569)*x[0]+(-0.445161879502)*x[1]**o+(0.268033115176)*x[1]+(-0.968003905235)*x[2]**o+(0.597022138075)*x[2]
        s[2,3]=(-0.214740575251)*x[0]**o+(0.318615769596)*x[0]+(0.554289329072)*x[1]**o+(-0.785944656086)*x[1]+(0.599828055452)*x[2]**o+(-0.221979522444)*x[2]
        s[2,4]=(-0.195666204125)*x[0]**o+(-0.0924232910586)*x[0]+(-0.442263606773)*x[1]**o+(0.114036215441)*x[1]+(-0.92582505345)*x[2]**o+(-0.138685852496)*x[2]
        s[3,0]=(0.376151237664)*x[0]**o+(0.352792436289)*x[0]+(0.755168859218)*x[1]**o+(-0.888367652073)*x[1]+(-0.0969244305571)*x[2]**o+(-0.210556734576)*x[2]
        s[3,1]=(-0.802814538265)*x[0]**o+(-0.512660557023)*x[0]+(-0.574936227852)*x[1]**o+(0.587382818046)*x[1]+(-0.500271058925)*x[2]**o+(-0.385863400163)*x[2]
        s[3,2]=(-0.487420035559)*x[0]**o+(-0.18302859198)*x[0]+(-0.477510753173)*x[1]**o+(-0.438221120428)*x[1]+(0.551062942324)*x[2]**o+(-0.0447436082813)*x[2]
        s[3,3]=(-0.934219151253)*x[0]**o+(0.398356052198)*x[0]+(0.317803529004)*x[1]**o+(0.996396715585)*x[1]+(-0.0662920794517)*x[2]**o+(-0.514553218432)*x[2]
        s[3,4]=(-0.775751006525)*x[0]**o+(-0.846022290911)*x[0]+(0.394673920051)*x[1]**o+(0.332106638437)*x[1]+(-0.925799206735)*x[2]**o+(0.908724437176)*x[2]
        ref[0,0]=(-0.564914581723)*x_ref[0]**o+(0.258795214127)*x_ref[0]+(-0.220020909772)*x_ref[1]**o+(0.810531687745)*x_ref[1]+(0.454950462482)*x_ref[2]**o+(0.601383225786)*x_ref[2]
        ref[0,1]=(-0.11600536761)*x_ref[0]**o+(-0.690336577243)*x_ref[0]+(-0.835351354712)*x_ref[1]**o+(-0.158331093203)*x_ref[1]+(0.645069697527)*x_ref[2]**o+(0.126534640518)*x_ref[2]
        ref[0,2]=(-0.350053630252)*x_ref[0]**o+(-0.890479764661)*x_ref[0]+(-0.280608934641)*x_ref[1]**o+(-0.855574890474)*x_ref[1]+(-0.988708278729)*x_ref[2]**o+(0.0791783723801)*x_ref[2]
        ref[0,3]=(0.987927769717)*x_ref[0]**o+(-0.347061742182)*x_ref[0]+(0.501082056996)*x_ref[1]**o+(-0.708442600496)*x_ref[1]+(0.948114708446)*x_ref[2]**o+(0.211831323577)*x_ref[2]
        ref[0,4]=(-0.930773949241)*x_ref[0]**o+(-0.545284390651)*x_ref[0]+(0.198879874511)*x_ref[1]**o+(-0.221794025714)*x_ref[1]+(-0.447838984339)*x_ref[2]**o+(-0.610640502396)*x_ref[2]
        ref[1,0]=(-0.156742047015)*x_ref[0]**o+(0.843077944384)*x_ref[0]+(-0.28777246252)*x_ref[1]**o+(0.638858613604)*x_ref[1]+(-0.503228005834)*x_ref[2]**o+(-0.960025936087)*x_ref[2]
        ref[1,1]=(-0.270691254249)*x_ref[0]**o+(0.0299500422959)*x_ref[0]+(-0.894145166738)*x_ref[1]**o+(0.85820171409)*x_ref[1]+(0.61650626344)*x_ref[2]**o+(-0.480895585635)*x_ref[2]
        ref[1,2]=(0.896861224301)*x_ref[0]**o+(0.00816715231085)*x_ref[0]+(-0.203903225872)*x_ref[1]**o+(-0.126928738689)*x_ref[1]+(-0.805120272595)*x_ref[2]**o+(0.98588145789)*x_ref[2]
        ref[1,3]=(-0.21275125769)*x_ref[0]**o+(0.458722054632)*x_ref[0]+(0.153520900666)*x_ref[1]**o+(0.512505902794)*x_ref[1]+(0.576294451799)*x_ref[2]**o+(0.604008619969)*x_ref[2]
        ref[1,4]=(-0.882533391878)*x_ref[0]**o+(0.616362261559)*x_ref[0]+(0.155750337317)*x_ref[1]**o+(-0.0162797277104)*x_ref[1]+(0.226933640416)*x_ref[2]**o+(-0.118618269284)*x_ref[2]
        ref[2,0]=(-0.149844951974)*x_ref[0]**o+(0.601452947295)*x_ref[0]+(-0.447624109479)*x_ref[1]**o+(-0.854580861366)*x_ref[1]+(0.392948004882)*x_ref[2]**o+(0.0820844858757)*x_ref[2]
        ref[2,1]=(-0.256158991676)*x_ref[0]**o+(0.161885127371)*x_ref[0]+(0.702594613311)*x_ref[1]**o+(0.598617952297)*x_ref[1]+(-0.338391564856)*x_ref[2]**o+(0.669697240278)*x_ref[2]
        ref[2,2]=(0.441512025449)*x_ref[0]**o+(-0.761281221569)*x_ref[0]+(-0.445161879502)*x_ref[1]**o+(0.268033115176)*x_ref[1]+(-0.968003905235)*x_ref[2]**o+(0.597022138075)*x_ref[2]
        ref[2,3]=(-0.214740575251)*x_ref[0]**o+(0.318615769596)*x_ref[0]+(0.554289329072)*x_ref[1]**o+(-0.785944656086)*x_ref[1]+(0.599828055452)*x_ref[2]**o+(-0.221979522444)*x_ref[2]
        ref[2,4]=(-0.195666204125)*x_ref[0]**o+(-0.0924232910586)*x_ref[0]+(-0.442263606773)*x_ref[1]**o+(0.114036215441)*x_ref[1]+(-0.92582505345)*x_ref[2]**o+(-0.138685852496)*x_ref[2]
        ref[3,0]=(0.376151237664)*x_ref[0]**o+(0.352792436289)*x_ref[0]+(0.755168859218)*x_ref[1]**o+(-0.888367652073)*x_ref[1]+(-0.0969244305571)*x_ref[2]**o+(-0.210556734576)*x_ref[2]
        ref[3,1]=(-0.802814538265)*x_ref[0]**o+(-0.512660557023)*x_ref[0]+(-0.574936227852)*x_ref[1]**o+(0.587382818046)*x_ref[1]+(-0.500271058925)*x_ref[2]**o+(-0.385863400163)*x_ref[2]
        ref[3,2]=(-0.487420035559)*x_ref[0]**o+(-0.18302859198)*x_ref[0]+(-0.477510753173)*x_ref[1]**o+(-0.438221120428)*x_ref[1]+(0.551062942324)*x_ref[2]**o+(-0.0447436082813)*x_ref[2]
        ref[3,3]=(-0.934219151253)*x_ref[0]**o+(0.398356052198)*x_ref[0]+(0.317803529004)*x_ref[1]**o+(0.996396715585)*x_ref[1]+(-0.0662920794517)*x_ref[2]**o+(-0.514553218432)*x_ref[2]
        ref[3,4]=(-0.775751006525)*x_ref[0]**o+(-0.846022290911)*x_ref[0]+(0.394673920051)*x_ref[1]**o+(0.332106638437)*x_ref[1]+(-0.925799206735)*x_ref[2]**o+(0.908724437176)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_Solution_rank3(self):
      """
      tests interpolate for rank 3 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from Solution onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(6, 2, 2),dim=dim)
      s=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2),w_ref)
      if dim==2:
        s[0,0,0]=(0.166904084074)*x[0]**o+(0.528859695436)*x[0]+(0.767492755038)*x[1]**o+(0.219756088661)*x[1]
        s[0,0,1]=(0.116354249122)*x[0]**o+(-0.489054596883)*x[0]+(0.843432052987)*x[1]**o+(0.792328551842)*x[1]
        s[0,1,0]=(0.316210380258)*x[0]**o+(0.567201270085)*x[0]+(-0.998412810858)*x[1]**o+(-0.523766343794)*x[1]
        s[0,1,1]=(-0.131361133794)*x[0]**o+(-0.415781217922)*x[0]+(-0.743444932389)*x[1]**o+(0.0440356388227)*x[1]
        s[1,0,0]=(-0.13886008398)*x[0]**o+(0.94188356362)*x[0]+(-0.310067426785)*x[1]**o+(-0.0123016016596)*x[1]
        s[1,0,1]=(-0.451586142333)*x[0]**o+(-0.00942276847807)*x[0]+(0.141967698659)*x[1]**o+(0.0763178193995)*x[1]
        s[1,1,0]=(-0.959190563677)*x[0]**o+(0.954437590409)*x[0]+(0.862816899262)*x[1]**o+(-0.849746515492)*x[1]
        s[1,1,1]=(0.277945454335)*x[0]**o+(-0.269801922972)*x[0]+(0.991405174196)*x[1]**o+(-0.481494765175)*x[1]
        s[2,0,0]=(-0.449455660138)*x[0]**o+(0.903463595246)*x[0]+(-0.796159239153)*x[1]**o+(0.178892715811)*x[1]
        s[2,0,1]=(-0.361748699382)*x[0]**o+(-0.380839684794)*x[0]+(0.167962669517)*x[1]**o+(-0.657300482681)*x[1]
        s[2,1,0]=(-0.00835791778047)*x[0]**o+(-0.964628782216)*x[0]+(-0.242890776847)*x[1]**o+(0.580549752444)*x[1]
        s[2,1,1]=(0.31424829894)*x[0]**o+(-0.902559989702)*x[0]+(0.11028478987)*x[1]**o+(-0.408366814277)*x[1]
        s[3,0,0]=(0.0269640344016)*x[0]**o+(0.742724075544)*x[0]+(-0.275698627658)*x[1]**o+(-0.666351552089)*x[1]
        s[3,0,1]=(0.634533702145)*x[0]**o+(-0.962956780064)*x[0]+(-0.117282164464)*x[1]**o+(0.116533409993)*x[1]
        s[3,1,0]=(0.0889039574007)*x[0]**o+(-0.302373214302)*x[0]+(0.0840411077509)*x[1]**o+(0.938857669151)*x[1]
        s[3,1,1]=(-0.814069429474)*x[0]**o+(-0.157632753675)*x[0]+(0.0908314139661)*x[1]**o+(0.918730301934)*x[1]
        s[4,0,0]=(0.535565487565)*x[0]**o+(-0.136652909062)*x[0]+(0.805198026614)*x[1]**o+(0.643003183224)*x[1]
        s[4,0,1]=(-0.80530904582)*x[0]**o+(0.966761926917)*x[0]+(0.427549028006)*x[1]**o+(0.262827083069)*x[1]
        s[4,1,0]=(-0.71456682036)*x[0]**o+(-0.0739127178784)*x[0]+(-0.438432370229)*x[1]**o+(-0.550981212497)*x[1]
        s[4,1,1]=(-0.536067456171)*x[0]**o+(0.343971454511)*x[0]+(-0.738912708731)*x[1]**o+(-0.721452063307)*x[1]
        s[5,0,0]=(-0.474192484949)*x[0]**o+(0.202037328764)*x[0]+(0.899258010081)*x[1]**o+(0.409255983049)*x[1]
        s[5,0,1]=(-0.0106650866193)*x[0]**o+(0.16386057287)*x[0]+(-0.027569820513)*x[1]**o+(0.49082096942)*x[1]
        s[5,1,0]=(0.888476355004)*x[0]**o+(-0.605284684678)*x[0]+(-0.767736685512)*x[1]**o+(0.443614918182)*x[1]
        s[5,1,1]=(0.538075122922)*x[0]**o+(0.0902918352337)*x[0]+(-0.848299366243)*x[1]**o+(-0.222800083729)*x[1]
        ref[0,0,0]=(0.166904084074)*x_ref[0]**o+(0.528859695436)*x_ref[0]+(0.767492755038)*x_ref[1]**o+(0.219756088661)*x_ref[1]
        ref[0,0,1]=(0.116354249122)*x_ref[0]**o+(-0.489054596883)*x_ref[0]+(0.843432052987)*x_ref[1]**o+(0.792328551842)*x_ref[1]
        ref[0,1,0]=(0.316210380258)*x_ref[0]**o+(0.567201270085)*x_ref[0]+(-0.998412810858)*x_ref[1]**o+(-0.523766343794)*x_ref[1]
        ref[0,1,1]=(-0.131361133794)*x_ref[0]**o+(-0.415781217922)*x_ref[0]+(-0.743444932389)*x_ref[1]**o+(0.0440356388227)*x_ref[1]
        ref[1,0,0]=(-0.13886008398)*x_ref[0]**o+(0.94188356362)*x_ref[0]+(-0.310067426785)*x_ref[1]**o+(-0.0123016016596)*x_ref[1]
        ref[1,0,1]=(-0.451586142333)*x_ref[0]**o+(-0.00942276847807)*x_ref[0]+(0.141967698659)*x_ref[1]**o+(0.0763178193995)*x_ref[1]
        ref[1,1,0]=(-0.959190563677)*x_ref[0]**o+(0.954437590409)*x_ref[0]+(0.862816899262)*x_ref[1]**o+(-0.849746515492)*x_ref[1]
        ref[1,1,1]=(0.277945454335)*x_ref[0]**o+(-0.269801922972)*x_ref[0]+(0.991405174196)*x_ref[1]**o+(-0.481494765175)*x_ref[1]
        ref[2,0,0]=(-0.449455660138)*x_ref[0]**o+(0.903463595246)*x_ref[0]+(-0.796159239153)*x_ref[1]**o+(0.178892715811)*x_ref[1]
        ref[2,0,1]=(-0.361748699382)*x_ref[0]**o+(-0.380839684794)*x_ref[0]+(0.167962669517)*x_ref[1]**o+(-0.657300482681)*x_ref[1]
        ref[2,1,0]=(-0.00835791778047)*x_ref[0]**o+(-0.964628782216)*x_ref[0]+(-0.242890776847)*x_ref[1]**o+(0.580549752444)*x_ref[1]
        ref[2,1,1]=(0.31424829894)*x_ref[0]**o+(-0.902559989702)*x_ref[0]+(0.11028478987)*x_ref[1]**o+(-0.408366814277)*x_ref[1]
        ref[3,0,0]=(0.0269640344016)*x_ref[0]**o+(0.742724075544)*x_ref[0]+(-0.275698627658)*x_ref[1]**o+(-0.666351552089)*x_ref[1]
        ref[3,0,1]=(0.634533702145)*x_ref[0]**o+(-0.962956780064)*x_ref[0]+(-0.117282164464)*x_ref[1]**o+(0.116533409993)*x_ref[1]
        ref[3,1,0]=(0.0889039574007)*x_ref[0]**o+(-0.302373214302)*x_ref[0]+(0.0840411077509)*x_ref[1]**o+(0.938857669151)*x_ref[1]
        ref[3,1,1]=(-0.814069429474)*x_ref[0]**o+(-0.157632753675)*x_ref[0]+(0.0908314139661)*x_ref[1]**o+(0.918730301934)*x_ref[1]
        ref[4,0,0]=(0.535565487565)*x_ref[0]**o+(-0.136652909062)*x_ref[0]+(0.805198026614)*x_ref[1]**o+(0.643003183224)*x_ref[1]
        ref[4,0,1]=(-0.80530904582)*x_ref[0]**o+(0.966761926917)*x_ref[0]+(0.427549028006)*x_ref[1]**o+(0.262827083069)*x_ref[1]
        ref[4,1,0]=(-0.71456682036)*x_ref[0]**o+(-0.0739127178784)*x_ref[0]+(-0.438432370229)*x_ref[1]**o+(-0.550981212497)*x_ref[1]
        ref[4,1,1]=(-0.536067456171)*x_ref[0]**o+(0.343971454511)*x_ref[0]+(-0.738912708731)*x_ref[1]**o+(-0.721452063307)*x_ref[1]
        ref[5,0,0]=(-0.474192484949)*x_ref[0]**o+(0.202037328764)*x_ref[0]+(0.899258010081)*x_ref[1]**o+(0.409255983049)*x_ref[1]
        ref[5,0,1]=(-0.0106650866193)*x_ref[0]**o+(0.16386057287)*x_ref[0]+(-0.027569820513)*x_ref[1]**o+(0.49082096942)*x_ref[1]
        ref[5,1,0]=(0.888476355004)*x_ref[0]**o+(-0.605284684678)*x_ref[0]+(-0.767736685512)*x_ref[1]**o+(0.443614918182)*x_ref[1]
        ref[5,1,1]=(0.538075122922)*x_ref[0]**o+(0.0902918352337)*x_ref[0]+(-0.848299366243)*x_ref[1]**o+(-0.222800083729)*x_ref[1]
      else:
        s[0,0,0]=(0.100353793751)*x[0]**o+(-0.00847651366844)*x[0]+(-0.916659095312)*x[1]**o+(0.43123854291)*x[1]+(-0.325548712866)*x[2]**o+(0.436984953031)*x[2]
        s[0,0,1]=(-0.450705520207)*x[0]**o+(-0.0420094946238)*x[0]+(0.0471679698967)*x[1]**o+(-0.666536961424)*x[1]+(0.629675854589)*x[2]**o+(0.789343254309)*x[2]
        s[0,1,0]=(-0.753618765099)*x[0]**o+(0.327794192265)*x[0]+(-0.230331681868)*x[1]**o+(0.414256616776)*x[1]+(0.861119329276)*x[2]**o+(-0.663194921545)*x[2]
        s[0,1,1]=(-0.897978378096)*x[0]**o+(-0.758509805972)*x[0]+(0.527577710213)*x[1]**o+(-0.668678399218)*x[1]+(0.274556839547)*x[2]**o+(0.838582981011)*x[2]
        s[1,0,0]=(-0.983732811569)*x[0]**o+(0.681831497881)*x[0]+(-0.834619144249)*x[1]**o+(-0.585649970853)*x[1]+(-0.443723853248)*x[2]**o+(-0.24230119834)*x[2]
        s[1,0,1]=(0.223700452689)*x[0]**o+(-0.976082665433)*x[0]+(0.682274036244)*x[1]**o+(0.513516465519)*x[1]+(0.604542635213)*x[2]**o+(0.461982295722)*x[2]
        s[1,1,0]=(-0.124528740755)*x[0]**o+(0.982942403581)*x[0]+(0.168657059059)*x[1]**o+(0.468525342776)*x[1]+(-0.628181131076)*x[2]**o+(-0.351466961112)*x[2]
        s[1,1,1]=(-0.147571648722)*x[0]**o+(0.823854564267)*x[0]+(0.808529030428)*x[1]**o+(0.809937338033)*x[1]+(0.86059984675)*x[2]**o+(-0.245256089602)*x[2]
        s[2,0,0]=(-0.421589109694)*x[0]**o+(0.739843964299)*x[0]+(-0.807211114691)*x[1]**o+(0.926253044589)*x[1]+(0.0247961654116)*x[2]**o+(0.372268514736)*x[2]
        s[2,0,1]=(-0.271986963767)*x[0]**o+(0.653599242745)*x[0]+(-0.950340669539)*x[1]**o+(0.486306280058)*x[1]+(-0.289166423358)*x[2]**o+(-0.05193840419)*x[2]
        s[2,1,0]=(-0.439847329001)*x[0]**o+(-0.0414982844889)*x[0]+(0.724606145337)*x[1]**o+(0.800164759919)*x[1]+(-0.948978159606)*x[2]**o+(0.330443199192)*x[2]
        s[2,1,1]=(0.145027947835)*x[0]**o+(-0.227814589191)*x[0]+(0.0969921713223)*x[1]**o+(0.70541078023)*x[1]+(0.536631305638)*x[2]**o+(0.761601818309)*x[2]
        s[3,0,0]=(-0.382247778154)*x[0]**o+(0.0243544380509)*x[0]+(0.515439251148)*x[1]**o+(0.531106239532)*x[1]+(-0.303961766943)*x[2]**o+(0.463673819842)*x[2]
        s[3,0,1]=(0.375551089648)*x[0]**o+(-0.0674026512267)*x[0]+(0.0797417786103)*x[1]**o+(0.90997116985)*x[1]+(-0.613366372871)*x[2]**o+(-0.0981062035417)*x[2]
        s[3,1,0]=(-0.467358089029)*x[0]**o+(-0.656202312608)*x[0]+(-0.00790864600562)*x[1]**o+(0.501618787933)*x[1]+(0.118835067496)*x[2]**o+(0.253600511775)*x[2]
        s[3,1,1]=(-0.0613387848811)*x[0]**o+(0.0555375789746)*x[0]+(0.467925348711)*x[1]**o+(-0.521361813902)*x[1]+(-0.890969702071)*x[2]**o+(-0.0588424357829)*x[2]
        s[4,0,0]=(-0.585761522751)*x[0]**o+(0.362766428544)*x[0]+(0.405833998664)*x[1]**o+(0.927768276759)*x[1]+(-0.515236556184)*x[2]**o+(0.903733254638)*x[2]
        s[4,0,1]=(-0.443373751826)*x[0]**o+(0.589695272429)*x[0]+(-0.547332598122)*x[1]**o+(0.547140256879)*x[1]+(0.0276104869591)*x[2]**o+(-0.00901656145103)*x[2]
        s[4,1,0]=(0.327714447478)*x[0]**o+(0.333181860896)*x[0]+(0.00402719775545)*x[1]**o+(0.119030826449)*x[1]+(0.121265643664)*x[2]**o+(0.3209309716)*x[2]
        s[4,1,1]=(-0.428606778799)*x[0]**o+(-0.0240488914911)*x[0]+(-0.617222089407)*x[1]**o+(0.326500165958)*x[1]+(0.695161972341)*x[2]**o+(-0.674385407299)*x[2]
        s[5,0,0]=(0.96840017886)*x[0]**o+(0.977460834915)*x[0]+(-0.0847928163657)*x[1]**o+(-0.289814405309)*x[1]+(-0.556056635259)*x[2]**o+(-0.127899784078)*x[2]
        s[5,0,1]=(-0.535056372252)*x[0]**o+(-0.557336923293)*x[0]+(-0.982050329504)*x[1]**o+(-0.186847262795)*x[1]+(0.859894693504)*x[2]**o+(-0.993989085549)*x[2]
        s[5,1,0]=(0.846310277081)*x[0]**o+(-0.996136776377)*x[0]+(0.221846701921)*x[1]**o+(0.947752560707)*x[1]+(-0.674009316486)*x[2]**o+(0.850085324498)*x[2]
        s[5,1,1]=(-0.0244426977213)*x[0]**o+(-0.768717160785)*x[0]+(0.42356601994)*x[1]**o+(-0.359166983456)*x[1]+(-0.671384564312)*x[2]**o+(0.850466416962)*x[2]
        ref[0,0,0]=(0.100353793751)*x_ref[0]**o+(-0.00847651366844)*x_ref[0]+(-0.916659095312)*x_ref[1]**o+(0.43123854291)*x_ref[1]+(-0.325548712866)*x_ref[2]**o+(0.436984953031)*x_ref[2]
        ref[0,0,1]=(-0.450705520207)*x_ref[0]**o+(-0.0420094946238)*x_ref[0]+(0.0471679698967)*x_ref[1]**o+(-0.666536961424)*x_ref[1]+(0.629675854589)*x_ref[2]**o+(0.789343254309)*x_ref[2]
        ref[0,1,0]=(-0.753618765099)*x_ref[0]**o+(0.327794192265)*x_ref[0]+(-0.230331681868)*x_ref[1]**o+(0.414256616776)*x_ref[1]+(0.861119329276)*x_ref[2]**o+(-0.663194921545)*x_ref[2]
        ref[0,1,1]=(-0.897978378096)*x_ref[0]**o+(-0.758509805972)*x_ref[0]+(0.527577710213)*x_ref[1]**o+(-0.668678399218)*x_ref[1]+(0.274556839547)*x_ref[2]**o+(0.838582981011)*x_ref[2]
        ref[1,0,0]=(-0.983732811569)*x_ref[0]**o+(0.681831497881)*x_ref[0]+(-0.834619144249)*x_ref[1]**o+(-0.585649970853)*x_ref[1]+(-0.443723853248)*x_ref[2]**o+(-0.24230119834)*x_ref[2]
        ref[1,0,1]=(0.223700452689)*x_ref[0]**o+(-0.976082665433)*x_ref[0]+(0.682274036244)*x_ref[1]**o+(0.513516465519)*x_ref[1]+(0.604542635213)*x_ref[2]**o+(0.461982295722)*x_ref[2]
        ref[1,1,0]=(-0.124528740755)*x_ref[0]**o+(0.982942403581)*x_ref[0]+(0.168657059059)*x_ref[1]**o+(0.468525342776)*x_ref[1]+(-0.628181131076)*x_ref[2]**o+(-0.351466961112)*x_ref[2]
        ref[1,1,1]=(-0.147571648722)*x_ref[0]**o+(0.823854564267)*x_ref[0]+(0.808529030428)*x_ref[1]**o+(0.809937338033)*x_ref[1]+(0.86059984675)*x_ref[2]**o+(-0.245256089602)*x_ref[2]
        ref[2,0,0]=(-0.421589109694)*x_ref[0]**o+(0.739843964299)*x_ref[0]+(-0.807211114691)*x_ref[1]**o+(0.926253044589)*x_ref[1]+(0.0247961654116)*x_ref[2]**o+(0.372268514736)*x_ref[2]
        ref[2,0,1]=(-0.271986963767)*x_ref[0]**o+(0.653599242745)*x_ref[0]+(-0.950340669539)*x_ref[1]**o+(0.486306280058)*x_ref[1]+(-0.289166423358)*x_ref[2]**o+(-0.05193840419)*x_ref[2]
        ref[2,1,0]=(-0.439847329001)*x_ref[0]**o+(-0.0414982844889)*x_ref[0]+(0.724606145337)*x_ref[1]**o+(0.800164759919)*x_ref[1]+(-0.948978159606)*x_ref[2]**o+(0.330443199192)*x_ref[2]
        ref[2,1,1]=(0.145027947835)*x_ref[0]**o+(-0.227814589191)*x_ref[0]+(0.0969921713223)*x_ref[1]**o+(0.70541078023)*x_ref[1]+(0.536631305638)*x_ref[2]**o+(0.761601818309)*x_ref[2]
        ref[3,0,0]=(-0.382247778154)*x_ref[0]**o+(0.0243544380509)*x_ref[0]+(0.515439251148)*x_ref[1]**o+(0.531106239532)*x_ref[1]+(-0.303961766943)*x_ref[2]**o+(0.463673819842)*x_ref[2]
        ref[3,0,1]=(0.375551089648)*x_ref[0]**o+(-0.0674026512267)*x_ref[0]+(0.0797417786103)*x_ref[1]**o+(0.90997116985)*x_ref[1]+(-0.613366372871)*x_ref[2]**o+(-0.0981062035417)*x_ref[2]
        ref[3,1,0]=(-0.467358089029)*x_ref[0]**o+(-0.656202312608)*x_ref[0]+(-0.00790864600562)*x_ref[1]**o+(0.501618787933)*x_ref[1]+(0.118835067496)*x_ref[2]**o+(0.253600511775)*x_ref[2]
        ref[3,1,1]=(-0.0613387848811)*x_ref[0]**o+(0.0555375789746)*x_ref[0]+(0.467925348711)*x_ref[1]**o+(-0.521361813902)*x_ref[1]+(-0.890969702071)*x_ref[2]**o+(-0.0588424357829)*x_ref[2]
        ref[4,0,0]=(-0.585761522751)*x_ref[0]**o+(0.362766428544)*x_ref[0]+(0.405833998664)*x_ref[1]**o+(0.927768276759)*x_ref[1]+(-0.515236556184)*x_ref[2]**o+(0.903733254638)*x_ref[2]
        ref[4,0,1]=(-0.443373751826)*x_ref[0]**o+(0.589695272429)*x_ref[0]+(-0.547332598122)*x_ref[1]**o+(0.547140256879)*x_ref[1]+(0.0276104869591)*x_ref[2]**o+(-0.00901656145103)*x_ref[2]
        ref[4,1,0]=(0.327714447478)*x_ref[0]**o+(0.333181860896)*x_ref[0]+(0.00402719775545)*x_ref[1]**o+(0.119030826449)*x_ref[1]+(0.121265643664)*x_ref[2]**o+(0.3209309716)*x_ref[2]
        ref[4,1,1]=(-0.428606778799)*x_ref[0]**o+(-0.0240488914911)*x_ref[0]+(-0.617222089407)*x_ref[1]**o+(0.326500165958)*x_ref[1]+(0.695161972341)*x_ref[2]**o+(-0.674385407299)*x_ref[2]
        ref[5,0,0]=(0.96840017886)*x_ref[0]**o+(0.977460834915)*x_ref[0]+(-0.0847928163657)*x_ref[1]**o+(-0.289814405309)*x_ref[1]+(-0.556056635259)*x_ref[2]**o+(-0.127899784078)*x_ref[2]
        ref[5,0,1]=(-0.535056372252)*x_ref[0]**o+(-0.557336923293)*x_ref[0]+(-0.982050329504)*x_ref[1]**o+(-0.186847262795)*x_ref[1]+(0.859894693504)*x_ref[2]**o+(-0.993989085549)*x_ref[2]
        ref[5,1,0]=(0.846310277081)*x_ref[0]**o+(-0.996136776377)*x_ref[0]+(0.221846701921)*x_ref[1]**o+(0.947752560707)*x_ref[1]+(-0.674009316486)*x_ref[2]**o+(0.850085324498)*x_ref[2]
        ref[5,1,1]=(-0.0244426977213)*x_ref[0]**o+(-0.768717160785)*x_ref[0]+(0.42356601994)*x_ref[1]**o+(-0.359166983456)*x_ref[1]+(-0.671384564312)*x_ref[2]**o+(0.850466416962)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_Solution_rank4(self):
      """
      tests interpolate for rank 4 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from Solution onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5, 3, 2),dim=dim)
      s=Data(0,(4, 5, 3, 2),w)
      ref=Data(0,(4, 5, 3, 2),w_ref)
      if dim==2:
        s[0,0,0,0]=(-0.099815403496)*x[0]**o+(-0.918641975262)*x[0]+(-0.643800270537)*x[1]**o+(-0.188947131246)*x[1]
        s[0,0,0,1]=(-0.654158499426)*x[0]**o+(-0.158917070952)*x[0]+(0.482580361439)*x[1]**o+(0.654085120986)*x[1]
        s[0,0,1,0]=(-0.82639526996)*x[0]**o+(-0.852788225935)*x[0]+(-0.883559061247)*x[1]**o+(-0.762093398747)*x[1]
        s[0,0,1,1]=(-0.410064006917)*x[0]**o+(-0.358595400023)*x[0]+(-0.744803691415)*x[1]**o+(-0.418642435874)*x[1]
        s[0,0,2,0]=(0.627425019462)*x[0]**o+(-0.759635228096)*x[0]+(-0.225478401461)*x[1]**o+(-0.394678152175)*x[1]
        s[0,0,2,1]=(0.282477965976)*x[0]**o+(0.591668341745)*x[0]+(0.997081550686)*x[1]**o+(-0.16137686071)*x[1]
        s[0,1,0,0]=(0.503328443933)*x[0]**o+(-0.349809904995)*x[0]+(0.675840382965)*x[1]**o+(0.122847940652)*x[1]
        s[0,1,0,1]=(-0.693842907429)*x[0]**o+(0.242153763819)*x[0]+(0.669516208483)*x[1]**o+(-0.966639099473)*x[1]
        s[0,1,1,0]=(-0.143996595411)*x[0]**o+(-0.0789367221661)*x[0]+(0.44978165828)*x[1]**o+(0.982272605823)*x[1]
        s[0,1,1,1]=(0.217504837315)*x[0]**o+(0.62345834313)*x[0]+(0.421663416031)*x[1]**o+(0.298214724947)*x[1]
        s[0,1,2,0]=(-0.192788493597)*x[0]**o+(0.59360099119)*x[0]+(0.798640786481)*x[1]**o+(-0.0618029984487)*x[1]
        s[0,1,2,1]=(-0.448474976435)*x[0]**o+(-0.136832009015)*x[0]+(-0.597189414931)*x[1]**o+(0.502732362863)*x[1]
        s[0,2,0,0]=(0.252384046343)*x[0]**o+(-0.400123690831)*x[0]+(-0.545481308678)*x[1]**o+(-0.388790715614)*x[1]
        s[0,2,0,1]=(-0.66240802679)*x[0]**o+(0.77064500558)*x[0]+(0.5239431782)*x[1]**o+(0.467410945489)*x[1]
        s[0,2,1,0]=(-0.85322788336)*x[0]**o+(-0.690030962118)*x[0]+(0.699888315472)*x[1]**o+(-0.89805156241)*x[1]
        s[0,2,1,1]=(-0.255889046261)*x[0]**o+(0.0541259188422)*x[0]+(-0.749587408291)*x[1]**o+(0.791965754891)*x[1]
        s[0,2,2,0]=(0.496462410445)*x[0]**o+(0.772074605719)*x[0]+(0.180850727108)*x[1]**o+(-0.487496085214)*x[1]
        s[0,2,2,1]=(0.706055709638)*x[0]**o+(-0.598906910405)*x[0]+(-0.152966385845)*x[1]**o+(0.871383558924)*x[1]
        s[0,3,0,0]=(-0.951858396669)*x[0]**o+(-0.794420515046)*x[0]+(0.834659431502)*x[1]**o+(0.481994252391)*x[1]
        s[0,3,0,1]=(-0.578325718357)*x[0]**o+(-0.843177109188)*x[0]+(0.38052630466)*x[1]**o+(0.0999564588392)*x[1]
        s[0,3,1,0]=(0.251016293115)*x[0]**o+(-0.846784412909)*x[0]+(0.615130638894)*x[1]**o+(-0.545472677499)*x[1]
        s[0,3,1,1]=(0.0242041003245)*x[0]**o+(-0.558470673629)*x[0]+(0.0355389532695)*x[1]**o+(-0.82512767128)*x[1]
        s[0,3,2,0]=(0.104901847054)*x[0]**o+(0.176002645436)*x[0]+(-0.0529040105457)*x[1]**o+(-0.497309382471)*x[1]
        s[0,3,2,1]=(-0.19187265394)*x[0]**o+(-0.459102146282)*x[0]+(0.263116634482)*x[1]**o+(0.573723666261)*x[1]
        s[0,4,0,0]=(-0.605664936504)*x[0]**o+(0.236623083899)*x[0]+(-0.573965234185)*x[1]**o+(-0.191541238111)*x[1]
        s[0,4,0,1]=(0.735297405377)*x[0]**o+(-0.863752806793)*x[0]+(0.130990772115)*x[1]**o+(0.153051776075)*x[1]
        s[0,4,1,0]=(0.83806722488)*x[0]**o+(0.952102086801)*x[0]+(0.921278259866)*x[1]**o+(0.100889262792)*x[1]
        s[0,4,1,1]=(0.422551160597)*x[0]**o+(-0.68105266886)*x[0]+(-0.0230668740384)*x[1]**o+(0.608853763751)*x[1]
        s[0,4,2,0]=(-0.509106957279)*x[0]**o+(0.535714529222)*x[0]+(0.407059274864)*x[1]**o+(0.0304780664648)*x[1]
        s[0,4,2,1]=(-0.899880077882)*x[0]**o+(-0.150489820435)*x[0]+(-0.329228777137)*x[1]**o+(-0.325670285955)*x[1]
        s[1,0,0,0]=(0.723148414781)*x[0]**o+(0.276861527427)*x[0]+(-0.457164017461)*x[1]**o+(0.587623869716)*x[1]
        s[1,0,0,1]=(-0.656042247607)*x[0]**o+(0.599876456972)*x[0]+(-0.487723393567)*x[1]**o+(-0.293330841893)*x[1]
        s[1,0,1,0]=(0.498385487237)*x[0]**o+(0.823826806076)*x[0]+(-0.260001108163)*x[1]**o+(-0.520083127954)*x[1]
        s[1,0,1,1]=(0.0511021547178)*x[0]**o+(0.0752387451382)*x[0]+(-0.295018309613)*x[1]**o+(0.410655676791)*x[1]
        s[1,0,2,0]=(-0.926038025877)*x[0]**o+(-0.101726746627)*x[0]+(-0.714060837737)*x[1]**o+(0.291653139474)*x[1]
        s[1,0,2,1]=(0.416231421476)*x[0]**o+(0.968331479531)*x[0]+(0.13575381957)*x[1]**o+(0.0581635556746)*x[1]
        s[1,1,0,0]=(0.483218423871)*x[0]**o+(0.837199391237)*x[0]+(-0.332232511767)*x[1]**o+(0.861714372392)*x[1]
        s[1,1,0,1]=(-0.861985810928)*x[0]**o+(0.704494207363)*x[0]+(0.0120560011404)*x[1]**o+(-0.922350499146)*x[1]
        s[1,1,1,0]=(0.710471809896)*x[0]**o+(0.10838308437)*x[0]+(-0.936553657991)*x[1]**o+(-0.157956217106)*x[1]
        s[1,1,1,1]=(0.61604545132)*x[0]**o+(0.161202252636)*x[0]+(-0.208993778286)*x[1]**o+(0.150797102116)*x[1]
        s[1,1,2,0]=(0.237396232488)*x[0]**o+(-0.260415056277)*x[0]+(-0.760219527005)*x[1]**o+(0.203694444805)*x[1]
        s[1,1,2,1]=(-0.656645187501)*x[0]**o+(-0.84629755936)*x[0]+(-0.465896055143)*x[1]**o+(0.335524534826)*x[1]
        s[1,2,0,0]=(-0.546393675559)*x[0]**o+(0.0344401682615)*x[0]+(-0.267054613816)*x[1]**o+(-0.345211291755)*x[1]
        s[1,2,0,1]=(0.807935620456)*x[0]**o+(0.89820026562)*x[0]+(-0.471804773843)*x[1]**o+(0.0183931405417)*x[1]
        s[1,2,1,0]=(0.347457777403)*x[0]**o+(0.894578611023)*x[0]+(0.506086327041)*x[1]**o+(-0.0552731455772)*x[1]
        s[1,2,1,1]=(0.898402940142)*x[0]**o+(-0.850905749447)*x[0]+(0.421790306199)*x[1]**o+(0.560435305462)*x[1]
        s[1,2,2,0]=(0.417854170807)*x[0]**o+(0.0808123920631)*x[0]+(0.685114818766)*x[1]**o+(0.627248749954)*x[1]
        s[1,2,2,1]=(0.608689720682)*x[0]**o+(-0.215111751974)*x[0]+(-0.157391477249)*x[1]**o+(-0.157398505688)*x[1]
        s[1,3,0,0]=(0.453375349378)*x[0]**o+(-0.232490378969)*x[0]+(0.370581316376)*x[1]**o+(0.2321645519)*x[1]
        s[1,3,0,1]=(-0.921453677654)*x[0]**o+(-0.272608399386)*x[0]+(0.980108178061)*x[1]**o+(0.0304202383868)*x[1]
        s[1,3,1,0]=(-0.703951869845)*x[0]**o+(0.232888375306)*x[0]+(0.737108419956)*x[1]**o+(-0.815317463111)*x[1]
        s[1,3,1,1]=(-0.849808179245)*x[0]**o+(-0.848051887981)*x[0]+(0.452157961656)*x[1]**o+(0.942533763098)*x[1]
        s[1,3,2,0]=(0.276310583942)*x[0]**o+(-0.426217969193)*x[0]+(0.383056703597)*x[1]**o+(-0.100608872707)*x[1]
        s[1,3,2,1]=(0.792743379061)*x[0]**o+(-0.638898354)*x[0]+(0.0900755365083)*x[1]**o+(0.346700761651)*x[1]
        s[1,4,0,0]=(0.227940749448)*x[0]**o+(-0.723141488706)*x[0]+(-0.597539261807)*x[1]**o+(-0.0547634570298)*x[1]
        s[1,4,0,1]=(-0.00150193441347)*x[0]**o+(-0.875771867697)*x[0]+(0.554295315299)*x[1]**o+(0.179291021879)*x[1]
        s[1,4,1,0]=(0.665263753178)*x[0]**o+(-0.261999229902)*x[0]+(0.177708351596)*x[1]**o+(-0.71011990376)*x[1]
        s[1,4,1,1]=(0.416094331837)*x[0]**o+(0.809191029905)*x[0]+(-0.631251155279)*x[1]**o+(0.758593347696)*x[1]
        s[1,4,2,0]=(0.50941363283)*x[0]**o+(-0.652536088555)*x[0]+(-0.0994076235125)*x[1]**o+(0.186614952474)*x[1]
        s[1,4,2,1]=(0.129263234225)*x[0]**o+(0.115870119224)*x[0]+(-0.25581908442)*x[1]**o+(0.295990548791)*x[1]
        s[2,0,0,0]=(-0.432850417862)*x[0]**o+(-0.491964804361)*x[0]+(0.0106143845453)*x[1]**o+(0.787362504175)*x[1]
        s[2,0,0,1]=(0.31617556259)*x[0]**o+(0.255445781486)*x[0]+(-0.302678692069)*x[1]**o+(0.840840019713)*x[1]
        s[2,0,1,0]=(-0.855359978332)*x[0]**o+(0.410554550223)*x[0]+(-0.703253338388)*x[1]**o+(0.59010970561)*x[1]
        s[2,0,1,1]=(-0.573270259685)*x[0]**o+(0.928396842624)*x[0]+(0.94155331127)*x[1]**o+(0.160140257961)*x[1]
        s[2,0,2,0]=(0.0524505638003)*x[0]**o+(-0.736508989499)*x[0]+(-0.565060891384)*x[1]**o+(-0.613736274563)*x[1]
        s[2,0,2,1]=(-0.776619329318)*x[0]**o+(0.485104655735)*x[0]+(-0.135863414015)*x[1]**o+(-0.14173677679)*x[1]
        s[2,1,0,0]=(0.662345907479)*x[0]**o+(-0.900993470049)*x[0]+(-0.882752065723)*x[1]**o+(0.642775104186)*x[1]
        s[2,1,0,1]=(-0.189295597666)*x[0]**o+(0.0165605763413)*x[0]+(0.989119296804)*x[1]**o+(0.262678778013)*x[1]
        s[2,1,1,0]=(0.927673133577)*x[0]**o+(-0.0248539427539)*x[0]+(0.504610255651)*x[1]**o+(-0.593911741472)*x[1]
        s[2,1,1,1]=(0.899525029448)*x[0]**o+(-0.97195963201)*x[0]+(-0.461931155897)*x[1]**o+(0.91354314585)*x[1]
        s[2,1,2,0]=(0.398654077456)*x[0]**o+(-0.429950726187)*x[0]+(0.378910374088)*x[1]**o+(-0.600308536915)*x[1]
        s[2,1,2,1]=(0.684159561112)*x[0]**o+(-0.408717922991)*x[0]+(0.0986838286044)*x[1]**o+(0.446743772828)*x[1]
        s[2,2,0,0]=(-0.657789003001)*x[0]**o+(-0.696243850831)*x[0]+(0.143074391914)*x[1]**o+(-0.784498727037)*x[1]
        s[2,2,0,1]=(-0.035450676833)*x[0]**o+(-0.453531837262)*x[0]+(0.59955708994)*x[1]**o+(0.35879284257)*x[1]
        s[2,2,1,0]=(0.302411531858)*x[0]**o+(-0.120739744552)*x[0]+(0.23248426438)*x[1]**o+(-0.39024395986)*x[1]
        s[2,2,1,1]=(0.957811592451)*x[0]**o+(-0.444133483391)*x[0]+(-0.917299241507)*x[1]**o+(0.66053932766)*x[1]
        s[2,2,2,0]=(0.361654572167)*x[0]**o+(-0.860498450204)*x[0]+(0.839403953375)*x[1]**o+(-0.386466411338)*x[1]
        s[2,2,2,1]=(-0.0143243836126)*x[0]**o+(0.444553204161)*x[0]+(0.0214936163191)*x[1]**o+(-0.257450046824)*x[1]
        s[2,3,0,0]=(0.841171215625)*x[0]**o+(-0.342091657341)*x[0]+(0.429215544167)*x[1]**o+(0.291784964305)*x[1]
        s[2,3,0,1]=(0.650468319582)*x[0]**o+(-0.121782834777)*x[0]+(0.887022664242)*x[1]**o+(0.957311577092)*x[1]
        s[2,3,1,0]=(0.156495854578)*x[0]**o+(0.855814547392)*x[0]+(0.320209813846)*x[1]**o+(0.458769463001)*x[1]
        s[2,3,1,1]=(0.34344265441)*x[0]**o+(-0.992393068373)*x[0]+(-0.0554086726336)*x[1]**o+(-0.985825317842)*x[1]
        s[2,3,2,0]=(-0.866276931509)*x[0]**o+(-0.343662754778)*x[0]+(0.0302242797749)*x[1]**o+(-0.652051073003)*x[1]
        s[2,3,2,1]=(-0.68451689603)*x[0]**o+(0.564121866438)*x[0]+(-0.136564076086)*x[1]**o+(0.648701900333)*x[1]
        s[2,4,0,0]=(-0.430429138775)*x[0]**o+(0.721222678082)*x[0]+(-0.0375465605006)*x[1]**o+(-0.462643638758)*x[1]
        s[2,4,0,1]=(-0.321870157728)*x[0]**o+(0.884979965704)*x[0]+(0.210984337677)*x[1]**o+(-0.796486469869)*x[1]
        s[2,4,1,0]=(0.365646287696)*x[0]**o+(-0.600693071289)*x[0]+(-0.998407218964)*x[1]**o+(-0.119949305545)*x[1]
        s[2,4,1,1]=(-0.763923462892)*x[0]**o+(-0.831011577325)*x[0]+(0.547670679354)*x[1]**o+(-0.856456618287)*x[1]
        s[2,4,2,0]=(0.107211541473)*x[0]**o+(-0.787737471224)*x[0]+(0.00381729482611)*x[1]**o+(-0.737759009165)*x[1]
        s[2,4,2,1]=(0.233196464862)*x[0]**o+(-0.464864449316)*x[0]+(0.229081432725)*x[1]**o+(-0.316321259146)*x[1]
        s[3,0,0,0]=(0.84633152323)*x[0]**o+(0.107225324454)*x[0]+(-0.768659465295)*x[1]**o+(0.441758939429)*x[1]
        s[3,0,0,1]=(-0.317081751011)*x[0]**o+(-0.557807443325)*x[0]+(-0.513518142998)*x[1]**o+(0.969406024047)*x[1]
        s[3,0,1,0]=(0.470133545453)*x[0]**o+(0.689603661125)*x[0]+(-0.358563691038)*x[1]**o+(0.0930824872861)*x[1]
        s[3,0,1,1]=(0.31919974075)*x[0]**o+(-0.555789020025)*x[0]+(0.0108880065973)*x[1]**o+(0.519548687393)*x[1]
        s[3,0,2,0]=(0.663881497435)*x[0]**o+(-0.668884360184)*x[0]+(-0.249669522726)*x[1]**o+(0.978543111491)*x[1]
        s[3,0,2,1]=(0.119167252087)*x[0]**o+(0.80927599582)*x[0]+(0.526489556971)*x[1]**o+(-0.440104597696)*x[1]
        s[3,1,0,0]=(-0.367446660377)*x[0]**o+(-0.326052348072)*x[0]+(0.841697795111)*x[1]**o+(0.403206833675)*x[1]
        s[3,1,0,1]=(-0.647927382559)*x[0]**o+(0.096852636085)*x[0]+(-0.126798636274)*x[1]**o+(0.117295901868)*x[1]
        s[3,1,1,0]=(0.827676755777)*x[0]**o+(-0.922576416262)*x[0]+(-0.3927169369)*x[1]**o+(0.828141309451)*x[1]
        s[3,1,1,1]=(-0.0442514478276)*x[0]**o+(-0.17904563214)*x[0]+(0.0870253733863)*x[1]**o+(-0.521249847127)*x[1]
        s[3,1,2,0]=(0.295531599451)*x[0]**o+(-0.125313746461)*x[0]+(0.982602753269)*x[1]**o+(-0.469029795983)*x[1]
        s[3,1,2,1]=(0.950273049304)*x[0]**o+(0.859212132249)*x[0]+(0.728565507333)*x[1]**o+(-0.274531603828)*x[1]
        s[3,2,0,0]=(-0.885845551336)*x[0]**o+(-0.704449229328)*x[0]+(-0.571836258871)*x[1]**o+(-0.0476396264682)*x[1]
        s[3,2,0,1]=(0.0713925637733)*x[0]**o+(0.294650464749)*x[0]+(-0.917378997351)*x[1]**o+(-0.715770468859)*x[1]
        s[3,2,1,0]=(-0.715304376467)*x[0]**o+(0.252625168227)*x[0]+(0.942098162684)*x[1]**o+(-0.0579654780836)*x[1]
        s[3,2,1,1]=(-0.937718965835)*x[0]**o+(0.649603348472)*x[0]+(-0.293284658463)*x[1]**o+(-0.347005762941)*x[1]
        s[3,2,2,0]=(-0.042358031525)*x[0]**o+(0.036891446424)*x[0]+(-0.229597055696)*x[1]**o+(0.0283282394156)*x[1]
        s[3,2,2,1]=(-0.0186366313091)*x[0]**o+(-0.0127649188124)*x[0]+(0.803780828202)*x[1]**o+(0.961045979755)*x[1]
        s[3,3,0,0]=(-0.27020299765)*x[0]**o+(-0.558034547573)*x[0]+(-0.524049515519)*x[1]**o+(-0.276595466979)*x[1]
        s[3,3,0,1]=(-0.510463688955)*x[0]**o+(-0.267341166705)*x[0]+(0.547679624943)*x[1]**o+(0.521119242662)*x[1]
        s[3,3,1,0]=(-0.155255975174)*x[0]**o+(0.365872110897)*x[0]+(-0.725704463467)*x[1]**o+(-0.790632654371)*x[1]
        s[3,3,1,1]=(-0.5111339808)*x[0]**o+(-0.0339146931841)*x[0]+(-0.696520447897)*x[1]**o+(-0.229679747746)*x[1]
        s[3,3,2,0]=(-0.552425399042)*x[0]**o+(-0.971205157087)*x[0]+(0.394260185879)*x[1]**o+(-0.164097031641)*x[1]
        s[3,3,2,1]=(-0.386876641825)*x[0]**o+(0.215459781811)*x[0]+(-0.332800684344)*x[1]**o+(0.419984531651)*x[1]
        s[3,4,0,0]=(-0.430256150077)*x[0]**o+(-0.110854674619)*x[0]+(0.687593230685)*x[1]**o+(-0.686895166454)*x[1]
        s[3,4,0,1]=(0.240753477602)*x[0]**o+(-0.815224522564)*x[0]+(0.499109852429)*x[1]**o+(-0.106870206363)*x[1]
        s[3,4,1,0]=(0.168547621411)*x[0]**o+(-0.509465054282)*x[0]+(-0.806347768707)*x[1]**o+(-0.799422054705)*x[1]
        s[3,4,1,1]=(-0.634806382514)*x[0]**o+(-0.125122793997)*x[0]+(-0.918019563645)*x[1]**o+(-0.341625694299)*x[1]
        s[3,4,2,0]=(-0.136108140149)*x[0]**o+(-0.780175946944)*x[0]+(0.0441301051696)*x[1]**o+(-0.326789608501)*x[1]
        s[3,4,2,1]=(0.181450357705)*x[0]**o+(0.467047306052)*x[0]+(0.182531831827)*x[1]**o+(-0.598701482526)*x[1]
        ref[0,0,0,0]=(-0.099815403496)*x_ref[0]**o+(-0.918641975262)*x_ref[0]+(-0.643800270537)*x_ref[1]**o+(-0.188947131246)*x_ref[1]
        ref[0,0,0,1]=(-0.654158499426)*x_ref[0]**o+(-0.158917070952)*x_ref[0]+(0.482580361439)*x_ref[1]**o+(0.654085120986)*x_ref[1]
        ref[0,0,1,0]=(-0.82639526996)*x_ref[0]**o+(-0.852788225935)*x_ref[0]+(-0.883559061247)*x_ref[1]**o+(-0.762093398747)*x_ref[1]
        ref[0,0,1,1]=(-0.410064006917)*x_ref[0]**o+(-0.358595400023)*x_ref[0]+(-0.744803691415)*x_ref[1]**o+(-0.418642435874)*x_ref[1]
        ref[0,0,2,0]=(0.627425019462)*x_ref[0]**o+(-0.759635228096)*x_ref[0]+(-0.225478401461)*x_ref[1]**o+(-0.394678152175)*x_ref[1]
        ref[0,0,2,1]=(0.282477965976)*x_ref[0]**o+(0.591668341745)*x_ref[0]+(0.997081550686)*x_ref[1]**o+(-0.16137686071)*x_ref[1]
        ref[0,1,0,0]=(0.503328443933)*x_ref[0]**o+(-0.349809904995)*x_ref[0]+(0.675840382965)*x_ref[1]**o+(0.122847940652)*x_ref[1]
        ref[0,1,0,1]=(-0.693842907429)*x_ref[0]**o+(0.242153763819)*x_ref[0]+(0.669516208483)*x_ref[1]**o+(-0.966639099473)*x_ref[1]
        ref[0,1,1,0]=(-0.143996595411)*x_ref[0]**o+(-0.0789367221661)*x_ref[0]+(0.44978165828)*x_ref[1]**o+(0.982272605823)*x_ref[1]
        ref[0,1,1,1]=(0.217504837315)*x_ref[0]**o+(0.62345834313)*x_ref[0]+(0.421663416031)*x_ref[1]**o+(0.298214724947)*x_ref[1]
        ref[0,1,2,0]=(-0.192788493597)*x_ref[0]**o+(0.59360099119)*x_ref[0]+(0.798640786481)*x_ref[1]**o+(-0.0618029984487)*x_ref[1]
        ref[0,1,2,1]=(-0.448474976435)*x_ref[0]**o+(-0.136832009015)*x_ref[0]+(-0.597189414931)*x_ref[1]**o+(0.502732362863)*x_ref[1]
        ref[0,2,0,0]=(0.252384046343)*x_ref[0]**o+(-0.400123690831)*x_ref[0]+(-0.545481308678)*x_ref[1]**o+(-0.388790715614)*x_ref[1]
        ref[0,2,0,1]=(-0.66240802679)*x_ref[0]**o+(0.77064500558)*x_ref[0]+(0.5239431782)*x_ref[1]**o+(0.467410945489)*x_ref[1]
        ref[0,2,1,0]=(-0.85322788336)*x_ref[0]**o+(-0.690030962118)*x_ref[0]+(0.699888315472)*x_ref[1]**o+(-0.89805156241)*x_ref[1]
        ref[0,2,1,1]=(-0.255889046261)*x_ref[0]**o+(0.0541259188422)*x_ref[0]+(-0.749587408291)*x_ref[1]**o+(0.791965754891)*x_ref[1]
        ref[0,2,2,0]=(0.496462410445)*x_ref[0]**o+(0.772074605719)*x_ref[0]+(0.180850727108)*x_ref[1]**o+(-0.487496085214)*x_ref[1]
        ref[0,2,2,1]=(0.706055709638)*x_ref[0]**o+(-0.598906910405)*x_ref[0]+(-0.152966385845)*x_ref[1]**o+(0.871383558924)*x_ref[1]
        ref[0,3,0,0]=(-0.951858396669)*x_ref[0]**o+(-0.794420515046)*x_ref[0]+(0.834659431502)*x_ref[1]**o+(0.481994252391)*x_ref[1]
        ref[0,3,0,1]=(-0.578325718357)*x_ref[0]**o+(-0.843177109188)*x_ref[0]+(0.38052630466)*x_ref[1]**o+(0.0999564588392)*x_ref[1]
        ref[0,3,1,0]=(0.251016293115)*x_ref[0]**o+(-0.846784412909)*x_ref[0]+(0.615130638894)*x_ref[1]**o+(-0.545472677499)*x_ref[1]
        ref[0,3,1,1]=(0.0242041003245)*x_ref[0]**o+(-0.558470673629)*x_ref[0]+(0.0355389532695)*x_ref[1]**o+(-0.82512767128)*x_ref[1]
        ref[0,3,2,0]=(0.104901847054)*x_ref[0]**o+(0.176002645436)*x_ref[0]+(-0.0529040105457)*x_ref[1]**o+(-0.497309382471)*x_ref[1]
        ref[0,3,2,1]=(-0.19187265394)*x_ref[0]**o+(-0.459102146282)*x_ref[0]+(0.263116634482)*x_ref[1]**o+(0.573723666261)*x_ref[1]
        ref[0,4,0,0]=(-0.605664936504)*x_ref[0]**o+(0.236623083899)*x_ref[0]+(-0.573965234185)*x_ref[1]**o+(-0.191541238111)*x_ref[1]
        ref[0,4,0,1]=(0.735297405377)*x_ref[0]**o+(-0.863752806793)*x_ref[0]+(0.130990772115)*x_ref[1]**o+(0.153051776075)*x_ref[1]
        ref[0,4,1,0]=(0.83806722488)*x_ref[0]**o+(0.952102086801)*x_ref[0]+(0.921278259866)*x_ref[1]**o+(0.100889262792)*x_ref[1]
        ref[0,4,1,1]=(0.422551160597)*x_ref[0]**o+(-0.68105266886)*x_ref[0]+(-0.0230668740384)*x_ref[1]**o+(0.608853763751)*x_ref[1]
        ref[0,4,2,0]=(-0.509106957279)*x_ref[0]**o+(0.535714529222)*x_ref[0]+(0.407059274864)*x_ref[1]**o+(0.0304780664648)*x_ref[1]
        ref[0,4,2,1]=(-0.899880077882)*x_ref[0]**o+(-0.150489820435)*x_ref[0]+(-0.329228777137)*x_ref[1]**o+(-0.325670285955)*x_ref[1]
        ref[1,0,0,0]=(0.723148414781)*x_ref[0]**o+(0.276861527427)*x_ref[0]+(-0.457164017461)*x_ref[1]**o+(0.587623869716)*x_ref[1]
        ref[1,0,0,1]=(-0.656042247607)*x_ref[0]**o+(0.599876456972)*x_ref[0]+(-0.487723393567)*x_ref[1]**o+(-0.293330841893)*x_ref[1]
        ref[1,0,1,0]=(0.498385487237)*x_ref[0]**o+(0.823826806076)*x_ref[0]+(-0.260001108163)*x_ref[1]**o+(-0.520083127954)*x_ref[1]
        ref[1,0,1,1]=(0.0511021547178)*x_ref[0]**o+(0.0752387451382)*x_ref[0]+(-0.295018309613)*x_ref[1]**o+(0.410655676791)*x_ref[1]
        ref[1,0,2,0]=(-0.926038025877)*x_ref[0]**o+(-0.101726746627)*x_ref[0]+(-0.714060837737)*x_ref[1]**o+(0.291653139474)*x_ref[1]
        ref[1,0,2,1]=(0.416231421476)*x_ref[0]**o+(0.968331479531)*x_ref[0]+(0.13575381957)*x_ref[1]**o+(0.0581635556746)*x_ref[1]
        ref[1,1,0,0]=(0.483218423871)*x_ref[0]**o+(0.837199391237)*x_ref[0]+(-0.332232511767)*x_ref[1]**o+(0.861714372392)*x_ref[1]
        ref[1,1,0,1]=(-0.861985810928)*x_ref[0]**o+(0.704494207363)*x_ref[0]+(0.0120560011404)*x_ref[1]**o+(-0.922350499146)*x_ref[1]
        ref[1,1,1,0]=(0.710471809896)*x_ref[0]**o+(0.10838308437)*x_ref[0]+(-0.936553657991)*x_ref[1]**o+(-0.157956217106)*x_ref[1]
        ref[1,1,1,1]=(0.61604545132)*x_ref[0]**o+(0.161202252636)*x_ref[0]+(-0.208993778286)*x_ref[1]**o+(0.150797102116)*x_ref[1]
        ref[1,1,2,0]=(0.237396232488)*x_ref[0]**o+(-0.260415056277)*x_ref[0]+(-0.760219527005)*x_ref[1]**o+(0.203694444805)*x_ref[1]
        ref[1,1,2,1]=(-0.656645187501)*x_ref[0]**o+(-0.84629755936)*x_ref[0]+(-0.465896055143)*x_ref[1]**o+(0.335524534826)*x_ref[1]
        ref[1,2,0,0]=(-0.546393675559)*x_ref[0]**o+(0.0344401682615)*x_ref[0]+(-0.267054613816)*x_ref[1]**o+(-0.345211291755)*x_ref[1]
        ref[1,2,0,1]=(0.807935620456)*x_ref[0]**o+(0.89820026562)*x_ref[0]+(-0.471804773843)*x_ref[1]**o+(0.0183931405417)*x_ref[1]
        ref[1,2,1,0]=(0.347457777403)*x_ref[0]**o+(0.894578611023)*x_ref[0]+(0.506086327041)*x_ref[1]**o+(-0.0552731455772)*x_ref[1]
        ref[1,2,1,1]=(0.898402940142)*x_ref[0]**o+(-0.850905749447)*x_ref[0]+(0.421790306199)*x_ref[1]**o+(0.560435305462)*x_ref[1]
        ref[1,2,2,0]=(0.417854170807)*x_ref[0]**o+(0.0808123920631)*x_ref[0]+(0.685114818766)*x_ref[1]**o+(0.627248749954)*x_ref[1]
        ref[1,2,2,1]=(0.608689720682)*x_ref[0]**o+(-0.215111751974)*x_ref[0]+(-0.157391477249)*x_ref[1]**o+(-0.157398505688)*x_ref[1]
        ref[1,3,0,0]=(0.453375349378)*x_ref[0]**o+(-0.232490378969)*x_ref[0]+(0.370581316376)*x_ref[1]**o+(0.2321645519)*x_ref[1]
        ref[1,3,0,1]=(-0.921453677654)*x_ref[0]**o+(-0.272608399386)*x_ref[0]+(0.980108178061)*x_ref[1]**o+(0.0304202383868)*x_ref[1]
        ref[1,3,1,0]=(-0.703951869845)*x_ref[0]**o+(0.232888375306)*x_ref[0]+(0.737108419956)*x_ref[1]**o+(-0.815317463111)*x_ref[1]
        ref[1,3,1,1]=(-0.849808179245)*x_ref[0]**o+(-0.848051887981)*x_ref[0]+(0.452157961656)*x_ref[1]**o+(0.942533763098)*x_ref[1]
        ref[1,3,2,0]=(0.276310583942)*x_ref[0]**o+(-0.426217969193)*x_ref[0]+(0.383056703597)*x_ref[1]**o+(-0.100608872707)*x_ref[1]
        ref[1,3,2,1]=(0.792743379061)*x_ref[0]**o+(-0.638898354)*x_ref[0]+(0.0900755365083)*x_ref[1]**o+(0.346700761651)*x_ref[1]
        ref[1,4,0,0]=(0.227940749448)*x_ref[0]**o+(-0.723141488706)*x_ref[0]+(-0.597539261807)*x_ref[1]**o+(-0.0547634570298)*x_ref[1]
        ref[1,4,0,1]=(-0.00150193441347)*x_ref[0]**o+(-0.875771867697)*x_ref[0]+(0.554295315299)*x_ref[1]**o+(0.179291021879)*x_ref[1]
        ref[1,4,1,0]=(0.665263753178)*x_ref[0]**o+(-0.261999229902)*x_ref[0]+(0.177708351596)*x_ref[1]**o+(-0.71011990376)*x_ref[1]
        ref[1,4,1,1]=(0.416094331837)*x_ref[0]**o+(0.809191029905)*x_ref[0]+(-0.631251155279)*x_ref[1]**o+(0.758593347696)*x_ref[1]
        ref[1,4,2,0]=(0.50941363283)*x_ref[0]**o+(-0.652536088555)*x_ref[0]+(-0.0994076235125)*x_ref[1]**o+(0.186614952474)*x_ref[1]
        ref[1,4,2,1]=(0.129263234225)*x_ref[0]**o+(0.115870119224)*x_ref[0]+(-0.25581908442)*x_ref[1]**o+(0.295990548791)*x_ref[1]
        ref[2,0,0,0]=(-0.432850417862)*x_ref[0]**o+(-0.491964804361)*x_ref[0]+(0.0106143845453)*x_ref[1]**o+(0.787362504175)*x_ref[1]
        ref[2,0,0,1]=(0.31617556259)*x_ref[0]**o+(0.255445781486)*x_ref[0]+(-0.302678692069)*x_ref[1]**o+(0.840840019713)*x_ref[1]
        ref[2,0,1,0]=(-0.855359978332)*x_ref[0]**o+(0.410554550223)*x_ref[0]+(-0.703253338388)*x_ref[1]**o+(0.59010970561)*x_ref[1]
        ref[2,0,1,1]=(-0.573270259685)*x_ref[0]**o+(0.928396842624)*x_ref[0]+(0.94155331127)*x_ref[1]**o+(0.160140257961)*x_ref[1]
        ref[2,0,2,0]=(0.0524505638003)*x_ref[0]**o+(-0.736508989499)*x_ref[0]+(-0.565060891384)*x_ref[1]**o+(-0.613736274563)*x_ref[1]
        ref[2,0,2,1]=(-0.776619329318)*x_ref[0]**o+(0.485104655735)*x_ref[0]+(-0.135863414015)*x_ref[1]**o+(-0.14173677679)*x_ref[1]
        ref[2,1,0,0]=(0.662345907479)*x_ref[0]**o+(-0.900993470049)*x_ref[0]+(-0.882752065723)*x_ref[1]**o+(0.642775104186)*x_ref[1]
        ref[2,1,0,1]=(-0.189295597666)*x_ref[0]**o+(0.0165605763413)*x_ref[0]+(0.989119296804)*x_ref[1]**o+(0.262678778013)*x_ref[1]
        ref[2,1,1,0]=(0.927673133577)*x_ref[0]**o+(-0.0248539427539)*x_ref[0]+(0.504610255651)*x_ref[1]**o+(-0.593911741472)*x_ref[1]
        ref[2,1,1,1]=(0.899525029448)*x_ref[0]**o+(-0.97195963201)*x_ref[0]+(-0.461931155897)*x_ref[1]**o+(0.91354314585)*x_ref[1]
        ref[2,1,2,0]=(0.398654077456)*x_ref[0]**o+(-0.429950726187)*x_ref[0]+(0.378910374088)*x_ref[1]**o+(-0.600308536915)*x_ref[1]
        ref[2,1,2,1]=(0.684159561112)*x_ref[0]**o+(-0.408717922991)*x_ref[0]+(0.0986838286044)*x_ref[1]**o+(0.446743772828)*x_ref[1]
        ref[2,2,0,0]=(-0.657789003001)*x_ref[0]**o+(-0.696243850831)*x_ref[0]+(0.143074391914)*x_ref[1]**o+(-0.784498727037)*x_ref[1]
        ref[2,2,0,1]=(-0.035450676833)*x_ref[0]**o+(-0.453531837262)*x_ref[0]+(0.59955708994)*x_ref[1]**o+(0.35879284257)*x_ref[1]
        ref[2,2,1,0]=(0.302411531858)*x_ref[0]**o+(-0.120739744552)*x_ref[0]+(0.23248426438)*x_ref[1]**o+(-0.39024395986)*x_ref[1]
        ref[2,2,1,1]=(0.957811592451)*x_ref[0]**o+(-0.444133483391)*x_ref[0]+(-0.917299241507)*x_ref[1]**o+(0.66053932766)*x_ref[1]
        ref[2,2,2,0]=(0.361654572167)*x_ref[0]**o+(-0.860498450204)*x_ref[0]+(0.839403953375)*x_ref[1]**o+(-0.386466411338)*x_ref[1]
        ref[2,2,2,1]=(-0.0143243836126)*x_ref[0]**o+(0.444553204161)*x_ref[0]+(0.0214936163191)*x_ref[1]**o+(-0.257450046824)*x_ref[1]
        ref[2,3,0,0]=(0.841171215625)*x_ref[0]**o+(-0.342091657341)*x_ref[0]+(0.429215544167)*x_ref[1]**o+(0.291784964305)*x_ref[1]
        ref[2,3,0,1]=(0.650468319582)*x_ref[0]**o+(-0.121782834777)*x_ref[0]+(0.887022664242)*x_ref[1]**o+(0.957311577092)*x_ref[1]
        ref[2,3,1,0]=(0.156495854578)*x_ref[0]**o+(0.855814547392)*x_ref[0]+(0.320209813846)*x_ref[1]**o+(0.458769463001)*x_ref[1]
        ref[2,3,1,1]=(0.34344265441)*x_ref[0]**o+(-0.992393068373)*x_ref[0]+(-0.0554086726336)*x_ref[1]**o+(-0.985825317842)*x_ref[1]
        ref[2,3,2,0]=(-0.866276931509)*x_ref[0]**o+(-0.343662754778)*x_ref[0]+(0.0302242797749)*x_ref[1]**o+(-0.652051073003)*x_ref[1]
        ref[2,3,2,1]=(-0.68451689603)*x_ref[0]**o+(0.564121866438)*x_ref[0]+(-0.136564076086)*x_ref[1]**o+(0.648701900333)*x_ref[1]
        ref[2,4,0,0]=(-0.430429138775)*x_ref[0]**o+(0.721222678082)*x_ref[0]+(-0.0375465605006)*x_ref[1]**o+(-0.462643638758)*x_ref[1]
        ref[2,4,0,1]=(-0.321870157728)*x_ref[0]**o+(0.884979965704)*x_ref[0]+(0.210984337677)*x_ref[1]**o+(-0.796486469869)*x_ref[1]
        ref[2,4,1,0]=(0.365646287696)*x_ref[0]**o+(-0.600693071289)*x_ref[0]+(-0.998407218964)*x_ref[1]**o+(-0.119949305545)*x_ref[1]
        ref[2,4,1,1]=(-0.763923462892)*x_ref[0]**o+(-0.831011577325)*x_ref[0]+(0.547670679354)*x_ref[1]**o+(-0.856456618287)*x_ref[1]
        ref[2,4,2,0]=(0.107211541473)*x_ref[0]**o+(-0.787737471224)*x_ref[0]+(0.00381729482611)*x_ref[1]**o+(-0.737759009165)*x_ref[1]
        ref[2,4,2,1]=(0.233196464862)*x_ref[0]**o+(-0.464864449316)*x_ref[0]+(0.229081432725)*x_ref[1]**o+(-0.316321259146)*x_ref[1]
        ref[3,0,0,0]=(0.84633152323)*x_ref[0]**o+(0.107225324454)*x_ref[0]+(-0.768659465295)*x_ref[1]**o+(0.441758939429)*x_ref[1]
        ref[3,0,0,1]=(-0.317081751011)*x_ref[0]**o+(-0.557807443325)*x_ref[0]+(-0.513518142998)*x_ref[1]**o+(0.969406024047)*x_ref[1]
        ref[3,0,1,0]=(0.470133545453)*x_ref[0]**o+(0.689603661125)*x_ref[0]+(-0.358563691038)*x_ref[1]**o+(0.0930824872861)*x_ref[1]
        ref[3,0,1,1]=(0.31919974075)*x_ref[0]**o+(-0.555789020025)*x_ref[0]+(0.0108880065973)*x_ref[1]**o+(0.519548687393)*x_ref[1]
        ref[3,0,2,0]=(0.663881497435)*x_ref[0]**o+(-0.668884360184)*x_ref[0]+(-0.249669522726)*x_ref[1]**o+(0.978543111491)*x_ref[1]
        ref[3,0,2,1]=(0.119167252087)*x_ref[0]**o+(0.80927599582)*x_ref[0]+(0.526489556971)*x_ref[1]**o+(-0.440104597696)*x_ref[1]
        ref[3,1,0,0]=(-0.367446660377)*x_ref[0]**o+(-0.326052348072)*x_ref[0]+(0.841697795111)*x_ref[1]**o+(0.403206833675)*x_ref[1]
        ref[3,1,0,1]=(-0.647927382559)*x_ref[0]**o+(0.096852636085)*x_ref[0]+(-0.126798636274)*x_ref[1]**o+(0.117295901868)*x_ref[1]
        ref[3,1,1,0]=(0.827676755777)*x_ref[0]**o+(-0.922576416262)*x_ref[0]+(-0.3927169369)*x_ref[1]**o+(0.828141309451)*x_ref[1]
        ref[3,1,1,1]=(-0.0442514478276)*x_ref[0]**o+(-0.17904563214)*x_ref[0]+(0.0870253733863)*x_ref[1]**o+(-0.521249847127)*x_ref[1]
        ref[3,1,2,0]=(0.295531599451)*x_ref[0]**o+(-0.125313746461)*x_ref[0]+(0.982602753269)*x_ref[1]**o+(-0.469029795983)*x_ref[1]
        ref[3,1,2,1]=(0.950273049304)*x_ref[0]**o+(0.859212132249)*x_ref[0]+(0.728565507333)*x_ref[1]**o+(-0.274531603828)*x_ref[1]
        ref[3,2,0,0]=(-0.885845551336)*x_ref[0]**o+(-0.704449229328)*x_ref[0]+(-0.571836258871)*x_ref[1]**o+(-0.0476396264682)*x_ref[1]
        ref[3,2,0,1]=(0.0713925637733)*x_ref[0]**o+(0.294650464749)*x_ref[0]+(-0.917378997351)*x_ref[1]**o+(-0.715770468859)*x_ref[1]
        ref[3,2,1,0]=(-0.715304376467)*x_ref[0]**o+(0.252625168227)*x_ref[0]+(0.942098162684)*x_ref[1]**o+(-0.0579654780836)*x_ref[1]
        ref[3,2,1,1]=(-0.937718965835)*x_ref[0]**o+(0.649603348472)*x_ref[0]+(-0.293284658463)*x_ref[1]**o+(-0.347005762941)*x_ref[1]
        ref[3,2,2,0]=(-0.042358031525)*x_ref[0]**o+(0.036891446424)*x_ref[0]+(-0.229597055696)*x_ref[1]**o+(0.0283282394156)*x_ref[1]
        ref[3,2,2,1]=(-0.0186366313091)*x_ref[0]**o+(-0.0127649188124)*x_ref[0]+(0.803780828202)*x_ref[1]**o+(0.961045979755)*x_ref[1]
        ref[3,3,0,0]=(-0.27020299765)*x_ref[0]**o+(-0.558034547573)*x_ref[0]+(-0.524049515519)*x_ref[1]**o+(-0.276595466979)*x_ref[1]
        ref[3,3,0,1]=(-0.510463688955)*x_ref[0]**o+(-0.267341166705)*x_ref[0]+(0.547679624943)*x_ref[1]**o+(0.521119242662)*x_ref[1]
        ref[3,3,1,0]=(-0.155255975174)*x_ref[0]**o+(0.365872110897)*x_ref[0]+(-0.725704463467)*x_ref[1]**o+(-0.790632654371)*x_ref[1]
        ref[3,3,1,1]=(-0.5111339808)*x_ref[0]**o+(-0.0339146931841)*x_ref[0]+(-0.696520447897)*x_ref[1]**o+(-0.229679747746)*x_ref[1]
        ref[3,3,2,0]=(-0.552425399042)*x_ref[0]**o+(-0.971205157087)*x_ref[0]+(0.394260185879)*x_ref[1]**o+(-0.164097031641)*x_ref[1]
        ref[3,3,2,1]=(-0.386876641825)*x_ref[0]**o+(0.215459781811)*x_ref[0]+(-0.332800684344)*x_ref[1]**o+(0.419984531651)*x_ref[1]
        ref[3,4,0,0]=(-0.430256150077)*x_ref[0]**o+(-0.110854674619)*x_ref[0]+(0.687593230685)*x_ref[1]**o+(-0.686895166454)*x_ref[1]
        ref[3,4,0,1]=(0.240753477602)*x_ref[0]**o+(-0.815224522564)*x_ref[0]+(0.499109852429)*x_ref[1]**o+(-0.106870206363)*x_ref[1]
        ref[3,4,1,0]=(0.168547621411)*x_ref[0]**o+(-0.509465054282)*x_ref[0]+(-0.806347768707)*x_ref[1]**o+(-0.799422054705)*x_ref[1]
        ref[3,4,1,1]=(-0.634806382514)*x_ref[0]**o+(-0.125122793997)*x_ref[0]+(-0.918019563645)*x_ref[1]**o+(-0.341625694299)*x_ref[1]
        ref[3,4,2,0]=(-0.136108140149)*x_ref[0]**o+(-0.780175946944)*x_ref[0]+(0.0441301051696)*x_ref[1]**o+(-0.326789608501)*x_ref[1]
        ref[3,4,2,1]=(0.181450357705)*x_ref[0]**o+(0.467047306052)*x_ref[0]+(0.182531831827)*x_ref[1]**o+(-0.598701482526)*x_ref[1]
      else:
        s[0,0,0,0]=(-0.0362500650077)*x[0]**o+(0.692407811012)*x[0]+(-0.855107417704)*x[1]**o+(-0.0397854200762)*x[1]+(0.488177999316)*x[2]**o+(0.946478266036)*x[2]
        s[0,0,0,1]=(-0.533284434038)*x[0]**o+(-0.599290705017)*x[0]+(0.198922961617)*x[1]**o+(0.652839426276)*x[1]+(0.560109621832)*x[2]**o+(0.167310997616)*x[2]
        s[0,0,1,0]=(-0.21016675446)*x[0]**o+(0.812498932794)*x[0]+(-0.611566184558)*x[1]**o+(0.626231122402)*x[1]+(0.536374628942)*x[2]**o+(0.8520547245)*x[2]
        s[0,0,1,1]=(0.26308571414)*x[0]**o+(-0.359237396089)*x[0]+(0.146879292748)*x[1]**o+(-0.241831631389)*x[1]+(0.812820522163)*x[2]**o+(0.824191692943)*x[2]
        s[0,0,2,0]=(-0.585682391332)*x[0]**o+(0.82273803111)*x[0]+(-0.715688315155)*x[1]**o+(-0.997893564593)*x[1]+(0.39588322481)*x[2]**o+(0.410447772982)*x[2]
        s[0,0,2,1]=(0.491817354047)*x[0]**o+(0.105289616977)*x[0]+(0.0581932965006)*x[1]**o+(-0.584153248946)*x[1]+(-0.837305609444)*x[2]**o+(-0.140188544108)*x[2]
        s[0,1,0,0]=(-0.158174387659)*x[0]**o+(0.774512978962)*x[0]+(0.953772629099)*x[1]**o+(-0.0474582135671)*x[1]+(-0.691917863221)*x[2]**o+(0.978550525536)*x[2]
        s[0,1,0,1]=(-0.696002534433)*x[0]**o+(-0.904833590237)*x[0]+(-0.991879336542)*x[1]**o+(-0.00301423741703)*x[1]+(-0.495919957691)*x[2]**o+(0.923872653662)*x[2]
        s[0,1,1,0]=(-0.617187346815)*x[0]**o+(-0.0283413268672)*x[0]+(0.439716253425)*x[1]**o+(0.679148169861)*x[1]+(-0.5494839537)*x[2]**o+(0.780531698733)*x[2]
        s[0,1,1,1]=(0.0763086894431)*x[0]**o+(0.149024988807)*x[0]+(0.660061724201)*x[1]**o+(0.227260856773)*x[1]+(0.604941557505)*x[2]**o+(-0.32106743372)*x[2]
        s[0,1,2,0]=(-0.710651974651)*x[0]**o+(0.729233023268)*x[0]+(0.830588938931)*x[1]**o+(0.930100931807)*x[1]+(0.576259601615)*x[2]**o+(0.995388406293)*x[2]
        s[0,1,2,1]=(0.635415205147)*x[0]**o+(-0.0475242382825)*x[0]+(0.779184290843)*x[1]**o+(0.161316071106)*x[1]+(-0.321426515539)*x[2]**o+(-0.188018565147)*x[2]
        s[0,2,0,0]=(-0.234573769896)*x[0]**o+(-0.35461213258)*x[0]+(0.322664245412)*x[1]**o+(-0.36584459555)*x[1]+(0.656560085187)*x[2]**o+(-0.380544830082)*x[2]
        s[0,2,0,1]=(0.314171153213)*x[0]**o+(-0.871055875712)*x[0]+(0.600399452124)*x[1]**o+(-0.710939791885)*x[1]+(-0.189029597187)*x[2]**o+(-0.29466779727)*x[2]
        s[0,2,1,0]=(0.911500533914)*x[0]**o+(-0.217077039937)*x[0]+(0.488325722425)*x[1]**o+(0.781071969291)*x[1]+(-0.883567213205)*x[2]**o+(0.938457876454)*x[2]
        s[0,2,1,1]=(0.947554813988)*x[0]**o+(0.108161094029)*x[0]+(0.0814722282926)*x[1]**o+(0.394738648377)*x[1]+(-0.585604351258)*x[2]**o+(0.424262882559)*x[2]
        s[0,2,2,0]=(0.130793077586)*x[0]**o+(-0.0759869806951)*x[0]+(0.524572237994)*x[1]**o+(-0.0494098932334)*x[1]+(-0.162573831498)*x[2]**o+(0.950665975392)*x[2]
        s[0,2,2,1]=(0.0636110845427)*x[0]**o+(-0.152586932248)*x[0]+(-0.428472289058)*x[1]**o+(0.943093556156)*x[1]+(-0.925181631256)*x[2]**o+(-0.850747416138)*x[2]
        s[0,3,0,0]=(0.803654693645)*x[0]**o+(-0.852793510169)*x[0]+(0.372325502382)*x[1]**o+(0.437191610865)*x[1]+(-0.626819779481)*x[2]**o+(-0.969085192013)*x[2]
        s[0,3,0,1]=(-0.498449247588)*x[0]**o+(-0.174254842954)*x[0]+(-0.751570511608)*x[1]**o+(0.0278714978689)*x[1]+(0.732061097781)*x[2]**o+(0.434922200273)*x[2]
        s[0,3,1,0]=(0.356966448806)*x[0]**o+(-0.754346917896)*x[0]+(-0.200096880735)*x[1]**o+(0.726202986575)*x[1]+(-0.273975127289)*x[2]**o+(0.922054852117)*x[2]
        s[0,3,1,1]=(0.294033872459)*x[0]**o+(0.435258372604)*x[0]+(-0.606222342027)*x[1]**o+(-0.418222134998)*x[1]+(-0.617263336049)*x[2]**o+(-0.655912764464)*x[2]
        s[0,3,2,0]=(-0.552805093123)*x[0]**o+(0.814985531695)*x[0]+(0.201701287999)*x[1]**o+(0.449378802496)*x[1]+(0.106877031271)*x[2]**o+(0.972523159602)*x[2]
        s[0,3,2,1]=(-0.123112956117)*x[0]**o+(-0.9669799128)*x[0]+(-0.365437347923)*x[1]**o+(0.95077682916)*x[1]+(-0.68795960948)*x[2]**o+(0.282323391952)*x[2]
        s[0,4,0,0]=(0.762949486092)*x[0]**o+(0.275310784251)*x[0]+(0.90824072581)*x[1]**o+(0.267277710972)*x[1]+(-0.387183781105)*x[2]**o+(-0.919714964098)*x[2]
        s[0,4,0,1]=(-0.219860124563)*x[0]**o+(0.458617314398)*x[0]+(-0.253950962469)*x[1]**o+(-0.213326656667)*x[1]+(-0.914189191308)*x[2]**o+(-0.88808493928)*x[2]
        s[0,4,1,0]=(0.0111879656405)*x[0]**o+(0.857083093627)*x[0]+(0.397087247889)*x[1]**o+(-0.411624885427)*x[1]+(-0.533535312833)*x[2]**o+(-0.473765290954)*x[2]
        s[0,4,1,1]=(0.408477302242)*x[0]**o+(-0.336255491206)*x[0]+(0.535004316057)*x[1]**o+(0.646300732798)*x[1]+(-0.683570698066)*x[2]**o+(-0.691723219637)*x[2]
        s[0,4,2,0]=(0.229517234528)*x[0]**o+(-0.327638251614)*x[0]+(0.752536339629)*x[1]**o+(0.440600000033)*x[1]+(-0.166203457183)*x[2]**o+(-0.632615828058)*x[2]
        s[0,4,2,1]=(0.128453646267)*x[0]**o+(-0.152501701527)*x[0]+(0.131616948331)*x[1]**o+(-0.560356219781)*x[1]+(0.369923630213)*x[2]**o+(-0.405937225725)*x[2]
        s[1,0,0,0]=(-0.498802287823)*x[0]**o+(0.974250100105)*x[0]+(-0.752335049339)*x[1]**o+(0.460106742976)*x[1]+(0.878914200699)*x[2]**o+(-0.85963539732)*x[2]
        s[1,0,0,1]=(0.937345235384)*x[0]**o+(0.0616042350221)*x[0]+(-0.63597655214)*x[1]**o+(0.200363730821)*x[1]+(0.677520498976)*x[2]**o+(0.310367850218)*x[2]
        s[1,0,1,0]=(-0.328289664191)*x[0]**o+(-0.439002316683)*x[0]+(-0.753405240499)*x[1]**o+(-0.749020227582)*x[1]+(0.200333899797)*x[2]**o+(0.63068525957)*x[2]
        s[1,0,1,1]=(-0.522381373851)*x[0]**o+(-0.877707833729)*x[0]+(-0.0334841793702)*x[1]**o+(0.00769816693416)*x[1]+(-0.81697893166)*x[2]**o+(-0.47349699259)*x[2]
        s[1,0,2,0]=(0.0786863277722)*x[0]**o+(-0.47462306827)*x[0]+(-0.850065158625)*x[1]**o+(0.0481804762762)*x[1]+(0.239465148926)*x[2]**o+(0.484166919183)*x[2]
        s[1,0,2,1]=(-0.0411433282485)*x[0]**o+(-0.851343034379)*x[0]+(-0.518404507699)*x[1]**o+(-0.357342937948)*x[1]+(-0.57398214485)*x[2]**o+(-0.0989103206187)*x[2]
        s[1,1,0,0]=(-0.060737698278)*x[0]**o+(-0.103483640965)*x[0]+(-0.206004957088)*x[1]**o+(0.556901021479)*x[1]+(0.951054830915)*x[2]**o+(0.668118573076)*x[2]
        s[1,1,0,1]=(0.0668860744649)*x[0]**o+(0.0490910382588)*x[0]+(0.817265796062)*x[1]**o+(-0.939419683478)*x[1]+(-0.800905110922)*x[2]**o+(-0.546647838589)*x[2]
        s[1,1,1,0]=(-0.437101637549)*x[0]**o+(0.269260162519)*x[0]+(-0.026802186298)*x[1]**o+(-0.932589891991)*x[1]+(-0.313814820129)*x[2]**o+(-0.436056943196)*x[2]
        s[1,1,1,1]=(-0.893118860321)*x[0]**o+(-0.837193118481)*x[0]+(-0.58313090725)*x[1]**o+(-0.657017739851)*x[1]+(-0.0158333033033)*x[2]**o+(0.286927848108)*x[2]
        s[1,1,2,0]=(-0.414914106956)*x[0]**o+(-0.295021056636)*x[0]+(-0.100513024651)*x[1]**o+(0.119321919323)*x[1]+(-0.911626057698)*x[2]**o+(0.0715733685275)*x[2]
        s[1,1,2,1]=(0.415073119201)*x[0]**o+(-0.79060072691)*x[0]+(-0.280560285152)*x[1]**o+(-0.651271923585)*x[1]+(-0.783523725291)*x[2]**o+(-0.165699899479)*x[2]
        s[1,2,0,0]=(0.204696440606)*x[0]**o+(0.58070882041)*x[0]+(0.498396642996)*x[1]**o+(0.270570176185)*x[1]+(-0.261604910878)*x[2]**o+(0.788402570665)*x[2]
        s[1,2,0,1]=(-0.163227090863)*x[0]**o+(0.17193387282)*x[0]+(0.559640507601)*x[1]**o+(0.39180422655)*x[1]+(0.182426715866)*x[2]**o+(0.52544860689)*x[2]
        s[1,2,1,0]=(-0.0826612289833)*x[0]**o+(-0.244737952715)*x[0]+(0.887738370973)*x[1]**o+(0.593438512127)*x[1]+(0.73128277327)*x[2]**o+(0.229067764431)*x[2]
        s[1,2,1,1]=(-0.117634040351)*x[0]**o+(-0.490365253501)*x[0]+(0.654067544802)*x[1]**o+(-0.862866217687)*x[1]+(-0.563450922747)*x[2]**o+(-0.785940469253)*x[2]
        s[1,2,2,0]=(0.486381246904)*x[0]**o+(-0.119137386359)*x[0]+(0.251388315083)*x[1]**o+(-0.142435376157)*x[1]+(-0.882418773242)*x[2]**o+(0.509436056026)*x[2]
        s[1,2,2,1]=(-0.69532427028)*x[0]**o+(-0.587331382396)*x[0]+(0.750092712907)*x[1]**o+(0.864188183197)*x[1]+(-0.0336735158745)*x[2]**o+(-0.118692941484)*x[2]
        s[1,3,0,0]=(0.287194598069)*x[0]**o+(0.775480786321)*x[0]+(0.24820537283)*x[1]**o+(0.579330345046)*x[1]+(0.641058615756)*x[2]**o+(-0.358716444584)*x[2]
        s[1,3,0,1]=(-0.218876040408)*x[0]**o+(-0.581134176647)*x[0]+(0.47937196949)*x[1]**o+(0.0529063680602)*x[1]+(-0.42659429504)*x[2]**o+(-0.445845798304)*x[2]
        s[1,3,1,0]=(0.500331134532)*x[0]**o+(0.764920932873)*x[0]+(0.256972231189)*x[1]**o+(0.279967260143)*x[1]+(0.90143027778)*x[2]**o+(-0.753075337423)*x[2]
        s[1,3,1,1]=(0.0478484319999)*x[0]**o+(0.239476880568)*x[0]+(-0.286765457909)*x[1]**o+(-0.952926972714)*x[1]+(-0.117698931152)*x[2]**o+(-0.950120391675)*x[2]
        s[1,3,2,0]=(-0.404765291554)*x[0]**o+(0.0796360913293)*x[0]+(0.743564239837)*x[1]**o+(-0.0431456147708)*x[1]+(-0.82536795966)*x[2]**o+(-0.770711063789)*x[2]
        s[1,3,2,1]=(-0.6113113129)*x[0]**o+(0.330568908804)*x[0]+(-0.391402722722)*x[1]**o+(0.415093002807)*x[1]+(-0.893617348746)*x[2]**o+(0.520968946942)*x[2]
        s[1,4,0,0]=(-0.209574438366)*x[0]**o+(0.644246054163)*x[0]+(-0.0664046919847)*x[1]**o+(0.214820911697)*x[1]+(-0.10545535704)*x[2]**o+(0.655063184877)*x[2]
        s[1,4,0,1]=(-0.38446029244)*x[0]**o+(-0.802691253671)*x[0]+(-0.306211901233)*x[1]**o+(-0.755184562233)*x[1]+(-0.704583961281)*x[2]**o+(0.0410695954026)*x[2]
        s[1,4,1,0]=(-0.512877649374)*x[0]**o+(-0.0190184293267)*x[0]+(0.588904418509)*x[1]**o+(-0.343320898588)*x[1]+(-0.16543066117)*x[2]**o+(-0.118363009949)*x[2]
        s[1,4,1,1]=(-0.410758135711)*x[0]**o+(0.302846037727)*x[0]+(0.674701250578)*x[1]**o+(-0.96295102282)*x[1]+(0.787271275616)*x[2]**o+(0.925818905258)*x[2]
        s[1,4,2,0]=(0.629642717166)*x[0]**o+(-0.696019558128)*x[0]+(0.465609587281)*x[1]**o+(0.191961667012)*x[1]+(-0.0182853530912)*x[2]**o+(0.504352699288)*x[2]
        s[1,4,2,1]=(-0.561295424723)*x[0]**o+(0.830136434947)*x[0]+(0.981595445935)*x[1]**o+(-0.0881426921172)*x[1]+(-0.581790941676)*x[2]**o+(-0.489101727188)*x[2]
        s[2,0,0,0]=(0.173249525275)*x[0]**o+(0.376871332154)*x[0]+(-0.379974081425)*x[1]**o+(-0.118151578515)*x[1]+(0.371715379047)*x[2]**o+(0.115273894116)*x[2]
        s[2,0,0,1]=(0.656339759814)*x[0]**o+(-0.0783529441613)*x[0]+(0.988608900721)*x[1]**o+(-0.498370704737)*x[1]+(0.886731679317)*x[2]**o+(0.251026122418)*x[2]
        s[2,0,1,0]=(-0.350592561402)*x[0]**o+(-0.592294993708)*x[0]+(0.860912193414)*x[1]**o+(0.678591565883)*x[1]+(-0.823185915223)*x[2]**o+(-0.882500202366)*x[2]
        s[2,0,1,1]=(0.84096537128)*x[0]**o+(-0.78582850022)*x[0]+(-0.872411932976)*x[1]**o+(0.208867262377)*x[1]+(0.795227125238)*x[2]**o+(0.434019201479)*x[2]
        s[2,0,2,0]=(0.670038316075)*x[0]**o+(0.221065948244)*x[0]+(-0.0801945207329)*x[1]**o+(0.35963157451)*x[1]+(-0.523905944746)*x[2]**o+(0.113327871154)*x[2]
        s[2,0,2,1]=(-0.790165918677)*x[0]**o+(0.826330759592)*x[0]+(0.816532564314)*x[1]**o+(-0.947088135238)*x[1]+(0.498519891556)*x[2]**o+(-0.221796346503)*x[2]
        s[2,1,0,0]=(0.679814393957)*x[0]**o+(-0.287910028702)*x[0]+(-0.179717372152)*x[1]**o+(-0.895091171351)*x[1]+(0.453459163192)*x[2]**o+(-0.732993783456)*x[2]
        s[2,1,0,1]=(-0.917014070487)*x[0]**o+(-0.28427666054)*x[0]+(0.520172057178)*x[1]**o+(0.973381422092)*x[1]+(-0.785604082848)*x[2]**o+(-0.280246362235)*x[2]
        s[2,1,1,0]=(-0.975543038311)*x[0]**o+(0.0471394295003)*x[0]+(-0.744014701)*x[1]**o+(-0.309281686644)*x[1]+(0.970209409582)*x[2]**o+(0.182638690597)*x[2]
        s[2,1,1,1]=(0.754879118472)*x[0]**o+(-0.345631769168)*x[0]+(0.864272677704)*x[1]**o+(-0.634026748154)*x[1]+(-0.25656646391)*x[2]**o+(-0.976807129917)*x[2]
        s[2,1,2,0]=(0.0378913818133)*x[0]**o+(-0.589418434757)*x[0]+(0.500291612315)*x[1]**o+(0.831173369231)*x[1]+(0.0773865261454)*x[2]**o+(-0.170449265388)*x[2]
        s[2,1,2,1]=(0.363668102897)*x[0]**o+(0.0152347755192)*x[0]+(0.295985339318)*x[1]**o+(-0.695348667793)*x[1]+(0.654035675777)*x[2]**o+(-0.998480773881)*x[2]
        s[2,2,0,0]=(-0.248968924843)*x[0]**o+(-0.454040417829)*x[0]+(0.618989286259)*x[1]**o+(-0.284627664137)*x[1]+(0.597922789444)*x[2]**o+(-0.0544707719136)*x[2]
        s[2,2,0,1]=(-0.564540493714)*x[0]**o+(0.948462945081)*x[0]+(-0.554305925084)*x[1]**o+(-0.683216336448)*x[1]+(-0.393720863511)*x[2]**o+(-0.454706611434)*x[2]
        s[2,2,1,0]=(0.00267926298461)*x[0]**o+(0.69057345924)*x[0]+(0.972990086053)*x[1]**o+(0.334585200812)*x[1]+(0.458913228071)*x[2]**o+(-0.927443691914)*x[2]
        s[2,2,1,1]=(-0.869817596024)*x[0]**o+(0.183295661514)*x[0]+(-0.215314084509)*x[1]**o+(0.274879624616)*x[1]+(-0.872459015851)*x[2]**o+(-0.43371203901)*x[2]
        s[2,2,2,0]=(-0.59916469262)*x[0]**o+(-0.635232312962)*x[0]+(-0.730547369574)*x[1]**o+(-0.199910106304)*x[1]+(-0.81824698793)*x[2]**o+(-0.351742356136)*x[2]
        s[2,2,2,1]=(0.52115654452)*x[0]**o+(-0.648726400371)*x[0]+(0.0705029035387)*x[1]**o+(0.82069581623)*x[1]+(0.262897099948)*x[2]**o+(0.302543200334)*x[2]
        s[2,3,0,0]=(-0.887996543825)*x[0]**o+(-0.141917392835)*x[0]+(0.458379292046)*x[1]**o+(0.279749722412)*x[1]+(-0.185582606045)*x[2]**o+(-0.408626717355)*x[2]
        s[2,3,0,1]=(-0.0708646458785)*x[0]**o+(-0.723087450876)*x[0]+(0.801053779885)*x[1]**o+(-0.316692437663)*x[1]+(0.313620016387)*x[2]**o+(-0.802237492322)*x[2]
        s[2,3,1,0]=(-0.413969268245)*x[0]**o+(-0.419189503807)*x[0]+(-0.150655589102)*x[1]**o+(-0.451064114786)*x[1]+(-0.11752330372)*x[2]**o+(-0.564979364641)*x[2]
        s[2,3,1,1]=(-0.127258808057)*x[0]**o+(-0.166240866424)*x[0]+(0.482435262134)*x[1]**o+(0.210083811762)*x[1]+(0.496523704735)*x[2]**o+(-0.528305427062)*x[2]
        s[2,3,2,0]=(-0.619368230074)*x[0]**o+(0.196929842772)*x[0]+(0.45226305027)*x[1]**o+(-0.949509654146)*x[1]+(-0.200199092979)*x[2]**o+(0.879368741494)*x[2]
        s[2,3,2,1]=(0.727974979035)*x[0]**o+(-0.486118068882)*x[0]+(0.330990540492)*x[1]**o+(0.627547710515)*x[1]+(-0.00312667743686)*x[2]**o+(0.34180677866)*x[2]
        s[2,4,0,0]=(-0.849340738634)*x[0]**o+(0.105575131464)*x[0]+(0.531365100651)*x[1]**o+(-0.885829562648)*x[1]+(-0.959105112582)*x[2]**o+(0.6210710685)*x[2]
        s[2,4,0,1]=(-0.525287919972)*x[0]**o+(0.685382709266)*x[0]+(0.706077262452)*x[1]**o+(-0.577859356102)*x[1]+(0.494281341217)*x[2]**o+(0.936337201867)*x[2]
        s[2,4,1,0]=(-0.0263664228096)*x[0]**o+(0.609335710155)*x[0]+(-0.417542795725)*x[1]**o+(0.127890280292)*x[1]+(0.0530446261715)*x[2]**o+(-0.203436845419)*x[2]
        s[2,4,1,1]=(0.58910765961)*x[0]**o+(-0.233711911277)*x[0]+(-0.68506108197)*x[1]**o+(-0.796608219305)*x[1]+(-0.55051026522)*x[2]**o+(-0.663284299983)*x[2]
        s[2,4,2,0]=(0.407867058666)*x[0]**o+(-0.155472572874)*x[0]+(0.0574795179339)*x[1]**o+(0.534562635965)*x[1]+(-0.35303239373)*x[2]**o+(0.690463242713)*x[2]
        s[2,4,2,1]=(0.458514172477)*x[0]**o+(0.493538792362)*x[0]+(-0.534087073406)*x[1]**o+(0.140190942794)*x[1]+(0.53904059561)*x[2]**o+(0.511924245166)*x[2]
        s[3,0,0,0]=(0.80688664483)*x[0]**o+(0.641990457435)*x[0]+(0.258439350653)*x[1]**o+(0.360949512497)*x[1]+(0.71055262439)*x[2]**o+(-0.14812833383)*x[2]
        s[3,0,0,1]=(0.349076196526)*x[0]**o+(-0.250426445203)*x[0]+(0.363350736371)*x[1]**o+(0.62331683755)*x[1]+(-0.799481730613)*x[2]**o+(0.86946545402)*x[2]
        s[3,0,1,0]=(-0.461900551752)*x[0]**o+(-0.530275233976)*x[0]+(0.778112655963)*x[1]**o+(-0.548160947996)*x[1]+(0.27643541045)*x[2]**o+(0.558519627971)*x[2]
        s[3,0,1,1]=(-0.870845434837)*x[0]**o+(0.90473098031)*x[0]+(0.356324338565)*x[1]**o+(-0.173579775601)*x[1]+(0.866960109024)*x[2]**o+(0.22810880895)*x[2]
        s[3,0,2,0]=(0.449562471445)*x[0]**o+(-0.133977676845)*x[0]+(-0.945220890263)*x[1]**o+(0.999013496171)*x[1]+(-0.612837266133)*x[2]**o+(-0.695797874487)*x[2]
        s[3,0,2,1]=(-0.588105630499)*x[0]**o+(0.176592342414)*x[0]+(-0.820991814796)*x[1]**o+(0.0010811086852)*x[1]+(0.498174763907)*x[2]**o+(-0.0223969132485)*x[2]
        s[3,1,0,0]=(0.653483773188)*x[0]**o+(0.460215104331)*x[0]+(-0.287173677147)*x[1]**o+(-0.837222104854)*x[1]+(0.457524748826)*x[2]**o+(-0.638268772518)*x[2]
        s[3,1,0,1]=(-0.0999951386915)*x[0]**o+(0.294288175142)*x[0]+(0.159178572562)*x[1]**o+(0.13437803751)*x[1]+(-0.411939992038)*x[2]**o+(0.283003770703)*x[2]
        s[3,1,1,0]=(-0.862929516639)*x[0]**o+(-0.455798937374)*x[0]+(0.972383754049)*x[1]**o+(0.251968652813)*x[1]+(-0.631965845999)*x[2]**o+(-0.418936218512)*x[2]
        s[3,1,1,1]=(-0.0265533722079)*x[0]**o+(-0.720054910456)*x[0]+(-0.134379948062)*x[1]**o+(-0.535223772178)*x[1]+(-0.711365379454)*x[2]**o+(0.39963284442)*x[2]
        s[3,1,2,0]=(-0.9691936764)*x[0]**o+(-0.220678275504)*x[0]+(0.423830480119)*x[1]**o+(-0.286816976029)*x[1]+(-0.637835943762)*x[2]**o+(0.389581647726)*x[2]
        s[3,1,2,1]=(-0.375225433223)*x[0]**o+(0.339950113552)*x[0]+(0.272089140767)*x[1]**o+(-0.671811329081)*x[1]+(-0.341834415939)*x[2]**o+(-0.974852723909)*x[2]
        s[3,2,0,0]=(-0.13212102165)*x[0]**o+(-0.529435682355)*x[0]+(0.57321409695)*x[1]**o+(0.123050260287)*x[1]+(-0.50608599056)*x[2]**o+(-0.258999018643)*x[2]
        s[3,2,0,1]=(-0.594320198162)*x[0]**o+(0.59617231387)*x[0]+(-0.132055038377)*x[1]**o+(0.0267055057967)*x[1]+(-0.0909023410879)*x[2]**o+(-0.327689716933)*x[2]
        s[3,2,1,0]=(0.352248348539)*x[0]**o+(-0.937172168074)*x[0]+(0.492884278868)*x[1]**o+(0.599164015797)*x[1]+(-0.306391881602)*x[2]**o+(-0.124914738757)*x[2]
        s[3,2,1,1]=(-0.124766143355)*x[0]**o+(-0.201629423485)*x[0]+(0.69199028272)*x[1]**o+(0.0909253140313)*x[1]+(-0.0807463990698)*x[2]**o+(0.274206212499)*x[2]
        s[3,2,2,0]=(-0.939257892238)*x[0]**o+(-0.0905023289297)*x[0]+(-0.101189471915)*x[1]**o+(-0.850452366024)*x[1]+(0.706374760942)*x[2]**o+(-0.0741989595692)*x[2]
        s[3,2,2,1]=(-0.956776001745)*x[0]**o+(0.313587985523)*x[0]+(0.501857466469)*x[1]**o+(0.866521730616)*x[1]+(-0.113777691526)*x[2]**o+(-0.43724474427)*x[2]
        s[3,3,0,0]=(-0.794240122389)*x[0]**o+(0.117246192404)*x[0]+(0.730901073854)*x[1]**o+(0.406753741761)*x[1]+(-0.987815418361)*x[2]**o+(0.543031203217)*x[2]
        s[3,3,0,1]=(-0.539899807208)*x[0]**o+(0.425096558373)*x[0]+(-0.261756005984)*x[1]**o+(0.219196945781)*x[1]+(-0.458238548056)*x[2]**o+(-0.396891207522)*x[2]
        s[3,3,1,0]=(-0.512322191935)*x[0]**o+(0.158356684082)*x[0]+(-0.702591537714)*x[1]**o+(0.202771352983)*x[1]+(-0.346772902785)*x[2]**o+(0.260366433089)*x[2]
        s[3,3,1,1]=(0.839858482584)*x[0]**o+(0.132933571004)*x[0]+(0.840682050408)*x[1]**o+(0.490904029834)*x[1]+(-0.298290729461)*x[2]**o+(-0.467229890802)*x[2]
        s[3,3,2,0]=(-0.691489850326)*x[0]**o+(0.852014173253)*x[0]+(0.484568640775)*x[1]**o+(0.0788611533234)*x[1]+(0.335927774531)*x[2]**o+(-0.507648906088)*x[2]
        s[3,3,2,1]=(0.140576758316)*x[0]**o+(-0.783440190349)*x[0]+(-0.408503163319)*x[1]**o+(0.0871935010894)*x[1]+(0.478151373206)*x[2]**o+(0.837020241411)*x[2]
        s[3,4,0,0]=(-0.956822615698)*x[0]**o+(0.594397877716)*x[0]+(0.844357571956)*x[1]**o+(0.173315874424)*x[1]+(0.96181728365)*x[2]**o+(-0.900445532048)*x[2]
        s[3,4,0,1]=(0.887824966747)*x[0]**o+(-0.579258071661)*x[0]+(0.983311456722)*x[1]**o+(0.996052998934)*x[1]+(0.215867343112)*x[2]**o+(0.614855622355)*x[2]
        s[3,4,1,0]=(0.482666172028)*x[0]**o+(0.0258626246719)*x[0]+(-0.394140399663)*x[1]**o+(0.192974809281)*x[1]+(-0.265879383852)*x[2]**o+(-0.0409441022179)*x[2]
        s[3,4,1,1]=(0.0600752930856)*x[0]**o+(-0.885644457634)*x[0]+(-0.54869761625)*x[1]**o+(0.4745411417)*x[1]+(-0.554764519536)*x[2]**o+(-0.159514159559)*x[2]
        s[3,4,2,0]=(-0.804675220841)*x[0]**o+(0.586839632342)*x[0]+(-0.329997060623)*x[1]**o+(0.242316008423)*x[1]+(0.272064229179)*x[2]**o+(0.24348627845)*x[2]
        s[3,4,2,1]=(0.847664079563)*x[0]**o+(0.927723091901)*x[0]+(-0.861048553467)*x[1]**o+(-0.413216499555)*x[1]+(0.546938488463)*x[2]**o+(0.0558689143145)*x[2]
        ref[0,0,0,0]=(-0.0362500650077)*x_ref[0]**o+(0.692407811012)*x_ref[0]+(-0.855107417704)*x_ref[1]**o+(-0.0397854200762)*x_ref[1]+(0.488177999316)*x_ref[2]**o+(0.946478266036)*x_ref[2]
        ref[0,0,0,1]=(-0.533284434038)*x_ref[0]**o+(-0.599290705017)*x_ref[0]+(0.198922961617)*x_ref[1]**o+(0.652839426276)*x_ref[1]+(0.560109621832)*x_ref[2]**o+(0.167310997616)*x_ref[2]
        ref[0,0,1,0]=(-0.21016675446)*x_ref[0]**o+(0.812498932794)*x_ref[0]+(-0.611566184558)*x_ref[1]**o+(0.626231122402)*x_ref[1]+(0.536374628942)*x_ref[2]**o+(0.8520547245)*x_ref[2]
        ref[0,0,1,1]=(0.26308571414)*x_ref[0]**o+(-0.359237396089)*x_ref[0]+(0.146879292748)*x_ref[1]**o+(-0.241831631389)*x_ref[1]+(0.812820522163)*x_ref[2]**o+(0.824191692943)*x_ref[2]
        ref[0,0,2,0]=(-0.585682391332)*x_ref[0]**o+(0.82273803111)*x_ref[0]+(-0.715688315155)*x_ref[1]**o+(-0.997893564593)*x_ref[1]+(0.39588322481)*x_ref[2]**o+(0.410447772982)*x_ref[2]
        ref[0,0,2,1]=(0.491817354047)*x_ref[0]**o+(0.105289616977)*x_ref[0]+(0.0581932965006)*x_ref[1]**o+(-0.584153248946)*x_ref[1]+(-0.837305609444)*x_ref[2]**o+(-0.140188544108)*x_ref[2]
        ref[0,1,0,0]=(-0.158174387659)*x_ref[0]**o+(0.774512978962)*x_ref[0]+(0.953772629099)*x_ref[1]**o+(-0.0474582135671)*x_ref[1]+(-0.691917863221)*x_ref[2]**o+(0.978550525536)*x_ref[2]
        ref[0,1,0,1]=(-0.696002534433)*x_ref[0]**o+(-0.904833590237)*x_ref[0]+(-0.991879336542)*x_ref[1]**o+(-0.00301423741703)*x_ref[1]+(-0.495919957691)*x_ref[2]**o+(0.923872653662)*x_ref[2]
        ref[0,1,1,0]=(-0.617187346815)*x_ref[0]**o+(-0.0283413268672)*x_ref[0]+(0.439716253425)*x_ref[1]**o+(0.679148169861)*x_ref[1]+(-0.5494839537)*x_ref[2]**o+(0.780531698733)*x_ref[2]
        ref[0,1,1,1]=(0.0763086894431)*x_ref[0]**o+(0.149024988807)*x_ref[0]+(0.660061724201)*x_ref[1]**o+(0.227260856773)*x_ref[1]+(0.604941557505)*x_ref[2]**o+(-0.32106743372)*x_ref[2]
        ref[0,1,2,0]=(-0.710651974651)*x_ref[0]**o+(0.729233023268)*x_ref[0]+(0.830588938931)*x_ref[1]**o+(0.930100931807)*x_ref[1]+(0.576259601615)*x_ref[2]**o+(0.995388406293)*x_ref[2]
        ref[0,1,2,1]=(0.635415205147)*x_ref[0]**o+(-0.0475242382825)*x_ref[0]+(0.779184290843)*x_ref[1]**o+(0.161316071106)*x_ref[1]+(-0.321426515539)*x_ref[2]**o+(-0.188018565147)*x_ref[2]
        ref[0,2,0,0]=(-0.234573769896)*x_ref[0]**o+(-0.35461213258)*x_ref[0]+(0.322664245412)*x_ref[1]**o+(-0.36584459555)*x_ref[1]+(0.656560085187)*x_ref[2]**o+(-0.380544830082)*x_ref[2]
        ref[0,2,0,1]=(0.314171153213)*x_ref[0]**o+(-0.871055875712)*x_ref[0]+(0.600399452124)*x_ref[1]**o+(-0.710939791885)*x_ref[1]+(-0.189029597187)*x_ref[2]**o+(-0.29466779727)*x_ref[2]
        ref[0,2,1,0]=(0.911500533914)*x_ref[0]**o+(-0.217077039937)*x_ref[0]+(0.488325722425)*x_ref[1]**o+(0.781071969291)*x_ref[1]+(-0.883567213205)*x_ref[2]**o+(0.938457876454)*x_ref[2]
        ref[0,2,1,1]=(0.947554813988)*x_ref[0]**o+(0.108161094029)*x_ref[0]+(0.0814722282926)*x_ref[1]**o+(0.394738648377)*x_ref[1]+(-0.585604351258)*x_ref[2]**o+(0.424262882559)*x_ref[2]
        ref[0,2,2,0]=(0.130793077586)*x_ref[0]**o+(-0.0759869806951)*x_ref[0]+(0.524572237994)*x_ref[1]**o+(-0.0494098932334)*x_ref[1]+(-0.162573831498)*x_ref[2]**o+(0.950665975392)*x_ref[2]
        ref[0,2,2,1]=(0.0636110845427)*x_ref[0]**o+(-0.152586932248)*x_ref[0]+(-0.428472289058)*x_ref[1]**o+(0.943093556156)*x_ref[1]+(-0.925181631256)*x_ref[2]**o+(-0.850747416138)*x_ref[2]
        ref[0,3,0,0]=(0.803654693645)*x_ref[0]**o+(-0.852793510169)*x_ref[0]+(0.372325502382)*x_ref[1]**o+(0.437191610865)*x_ref[1]+(-0.626819779481)*x_ref[2]**o+(-0.969085192013)*x_ref[2]
        ref[0,3,0,1]=(-0.498449247588)*x_ref[0]**o+(-0.174254842954)*x_ref[0]+(-0.751570511608)*x_ref[1]**o+(0.0278714978689)*x_ref[1]+(0.732061097781)*x_ref[2]**o+(0.434922200273)*x_ref[2]
        ref[0,3,1,0]=(0.356966448806)*x_ref[0]**o+(-0.754346917896)*x_ref[0]+(-0.200096880735)*x_ref[1]**o+(0.726202986575)*x_ref[1]+(-0.273975127289)*x_ref[2]**o+(0.922054852117)*x_ref[2]
        ref[0,3,1,1]=(0.294033872459)*x_ref[0]**o+(0.435258372604)*x_ref[0]+(-0.606222342027)*x_ref[1]**o+(-0.418222134998)*x_ref[1]+(-0.617263336049)*x_ref[2]**o+(-0.655912764464)*x_ref[2]
        ref[0,3,2,0]=(-0.552805093123)*x_ref[0]**o+(0.814985531695)*x_ref[0]+(0.201701287999)*x_ref[1]**o+(0.449378802496)*x_ref[1]+(0.106877031271)*x_ref[2]**o+(0.972523159602)*x_ref[2]
        ref[0,3,2,1]=(-0.123112956117)*x_ref[0]**o+(-0.9669799128)*x_ref[0]+(-0.365437347923)*x_ref[1]**o+(0.95077682916)*x_ref[1]+(-0.68795960948)*x_ref[2]**o+(0.282323391952)*x_ref[2]
        ref[0,4,0,0]=(0.762949486092)*x_ref[0]**o+(0.275310784251)*x_ref[0]+(0.90824072581)*x_ref[1]**o+(0.267277710972)*x_ref[1]+(-0.387183781105)*x_ref[2]**o+(-0.919714964098)*x_ref[2]
        ref[0,4,0,1]=(-0.219860124563)*x_ref[0]**o+(0.458617314398)*x_ref[0]+(-0.253950962469)*x_ref[1]**o+(-0.213326656667)*x_ref[1]+(-0.914189191308)*x_ref[2]**o+(-0.88808493928)*x_ref[2]
        ref[0,4,1,0]=(0.0111879656405)*x_ref[0]**o+(0.857083093627)*x_ref[0]+(0.397087247889)*x_ref[1]**o+(-0.411624885427)*x_ref[1]+(-0.533535312833)*x_ref[2]**o+(-0.473765290954)*x_ref[2]
        ref[0,4,1,1]=(0.408477302242)*x_ref[0]**o+(-0.336255491206)*x_ref[0]+(0.535004316057)*x_ref[1]**o+(0.646300732798)*x_ref[1]+(-0.683570698066)*x_ref[2]**o+(-0.691723219637)*x_ref[2]
        ref[0,4,2,0]=(0.229517234528)*x_ref[0]**o+(-0.327638251614)*x_ref[0]+(0.752536339629)*x_ref[1]**o+(0.440600000033)*x_ref[1]+(-0.166203457183)*x_ref[2]**o+(-0.632615828058)*x_ref[2]
        ref[0,4,2,1]=(0.128453646267)*x_ref[0]**o+(-0.152501701527)*x_ref[0]+(0.131616948331)*x_ref[1]**o+(-0.560356219781)*x_ref[1]+(0.369923630213)*x_ref[2]**o+(-0.405937225725)*x_ref[2]
        ref[1,0,0,0]=(-0.498802287823)*x_ref[0]**o+(0.974250100105)*x_ref[0]+(-0.752335049339)*x_ref[1]**o+(0.460106742976)*x_ref[1]+(0.878914200699)*x_ref[2]**o+(-0.85963539732)*x_ref[2]
        ref[1,0,0,1]=(0.937345235384)*x_ref[0]**o+(0.0616042350221)*x_ref[0]+(-0.63597655214)*x_ref[1]**o+(0.200363730821)*x_ref[1]+(0.677520498976)*x_ref[2]**o+(0.310367850218)*x_ref[2]
        ref[1,0,1,0]=(-0.328289664191)*x_ref[0]**o+(-0.439002316683)*x_ref[0]+(-0.753405240499)*x_ref[1]**o+(-0.749020227582)*x_ref[1]+(0.200333899797)*x_ref[2]**o+(0.63068525957)*x_ref[2]
        ref[1,0,1,1]=(-0.522381373851)*x_ref[0]**o+(-0.877707833729)*x_ref[0]+(-0.0334841793702)*x_ref[1]**o+(0.00769816693416)*x_ref[1]+(-0.81697893166)*x_ref[2]**o+(-0.47349699259)*x_ref[2]
        ref[1,0,2,0]=(0.0786863277722)*x_ref[0]**o+(-0.47462306827)*x_ref[0]+(-0.850065158625)*x_ref[1]**o+(0.0481804762762)*x_ref[1]+(0.239465148926)*x_ref[2]**o+(0.484166919183)*x_ref[2]
        ref[1,0,2,1]=(-0.0411433282485)*x_ref[0]**o+(-0.851343034379)*x_ref[0]+(-0.518404507699)*x_ref[1]**o+(-0.357342937948)*x_ref[1]+(-0.57398214485)*x_ref[2]**o+(-0.0989103206187)*x_ref[2]
        ref[1,1,0,0]=(-0.060737698278)*x_ref[0]**o+(-0.103483640965)*x_ref[0]+(-0.206004957088)*x_ref[1]**o+(0.556901021479)*x_ref[1]+(0.951054830915)*x_ref[2]**o+(0.668118573076)*x_ref[2]
        ref[1,1,0,1]=(0.0668860744649)*x_ref[0]**o+(0.0490910382588)*x_ref[0]+(0.817265796062)*x_ref[1]**o+(-0.939419683478)*x_ref[1]+(-0.800905110922)*x_ref[2]**o+(-0.546647838589)*x_ref[2]
        ref[1,1,1,0]=(-0.437101637549)*x_ref[0]**o+(0.269260162519)*x_ref[0]+(-0.026802186298)*x_ref[1]**o+(-0.932589891991)*x_ref[1]+(-0.313814820129)*x_ref[2]**o+(-0.436056943196)*x_ref[2]
        ref[1,1,1,1]=(-0.893118860321)*x_ref[0]**o+(-0.837193118481)*x_ref[0]+(-0.58313090725)*x_ref[1]**o+(-0.657017739851)*x_ref[1]+(-0.0158333033033)*x_ref[2]**o+(0.286927848108)*x_ref[2]
        ref[1,1,2,0]=(-0.414914106956)*x_ref[0]**o+(-0.295021056636)*x_ref[0]+(-0.100513024651)*x_ref[1]**o+(0.119321919323)*x_ref[1]+(-0.911626057698)*x_ref[2]**o+(0.0715733685275)*x_ref[2]
        ref[1,1,2,1]=(0.415073119201)*x_ref[0]**o+(-0.79060072691)*x_ref[0]+(-0.280560285152)*x_ref[1]**o+(-0.651271923585)*x_ref[1]+(-0.783523725291)*x_ref[2]**o+(-0.165699899479)*x_ref[2]
        ref[1,2,0,0]=(0.204696440606)*x_ref[0]**o+(0.58070882041)*x_ref[0]+(0.498396642996)*x_ref[1]**o+(0.270570176185)*x_ref[1]+(-0.261604910878)*x_ref[2]**o+(0.788402570665)*x_ref[2]
        ref[1,2,0,1]=(-0.163227090863)*x_ref[0]**o+(0.17193387282)*x_ref[0]+(0.559640507601)*x_ref[1]**o+(0.39180422655)*x_ref[1]+(0.182426715866)*x_ref[2]**o+(0.52544860689)*x_ref[2]
        ref[1,2,1,0]=(-0.0826612289833)*x_ref[0]**o+(-0.244737952715)*x_ref[0]+(0.887738370973)*x_ref[1]**o+(0.593438512127)*x_ref[1]+(0.73128277327)*x_ref[2]**o+(0.229067764431)*x_ref[2]
        ref[1,2,1,1]=(-0.117634040351)*x_ref[0]**o+(-0.490365253501)*x_ref[0]+(0.654067544802)*x_ref[1]**o+(-0.862866217687)*x_ref[1]+(-0.563450922747)*x_ref[2]**o+(-0.785940469253)*x_ref[2]
        ref[1,2,2,0]=(0.486381246904)*x_ref[0]**o+(-0.119137386359)*x_ref[0]+(0.251388315083)*x_ref[1]**o+(-0.142435376157)*x_ref[1]+(-0.882418773242)*x_ref[2]**o+(0.509436056026)*x_ref[2]
        ref[1,2,2,1]=(-0.69532427028)*x_ref[0]**o+(-0.587331382396)*x_ref[0]+(0.750092712907)*x_ref[1]**o+(0.864188183197)*x_ref[1]+(-0.0336735158745)*x_ref[2]**o+(-0.118692941484)*x_ref[2]
        ref[1,3,0,0]=(0.287194598069)*x_ref[0]**o+(0.775480786321)*x_ref[0]+(0.24820537283)*x_ref[1]**o+(0.579330345046)*x_ref[1]+(0.641058615756)*x_ref[2]**o+(-0.358716444584)*x_ref[2]
        ref[1,3,0,1]=(-0.218876040408)*x_ref[0]**o+(-0.581134176647)*x_ref[0]+(0.47937196949)*x_ref[1]**o+(0.0529063680602)*x_ref[1]+(-0.42659429504)*x_ref[2]**o+(-0.445845798304)*x_ref[2]
        ref[1,3,1,0]=(0.500331134532)*x_ref[0]**o+(0.764920932873)*x_ref[0]+(0.256972231189)*x_ref[1]**o+(0.279967260143)*x_ref[1]+(0.90143027778)*x_ref[2]**o+(-0.753075337423)*x_ref[2]
        ref[1,3,1,1]=(0.0478484319999)*x_ref[0]**o+(0.239476880568)*x_ref[0]+(-0.286765457909)*x_ref[1]**o+(-0.952926972714)*x_ref[1]+(-0.117698931152)*x_ref[2]**o+(-0.950120391675)*x_ref[2]
        ref[1,3,2,0]=(-0.404765291554)*x_ref[0]**o+(0.0796360913293)*x_ref[0]+(0.743564239837)*x_ref[1]**o+(-0.0431456147708)*x_ref[1]+(-0.82536795966)*x_ref[2]**o+(-0.770711063789)*x_ref[2]
        ref[1,3,2,1]=(-0.6113113129)*x_ref[0]**o+(0.330568908804)*x_ref[0]+(-0.391402722722)*x_ref[1]**o+(0.415093002807)*x_ref[1]+(-0.893617348746)*x_ref[2]**o+(0.520968946942)*x_ref[2]
        ref[1,4,0,0]=(-0.209574438366)*x_ref[0]**o+(0.644246054163)*x_ref[0]+(-0.0664046919847)*x_ref[1]**o+(0.214820911697)*x_ref[1]+(-0.10545535704)*x_ref[2]**o+(0.655063184877)*x_ref[2]
        ref[1,4,0,1]=(-0.38446029244)*x_ref[0]**o+(-0.802691253671)*x_ref[0]+(-0.306211901233)*x_ref[1]**o+(-0.755184562233)*x_ref[1]+(-0.704583961281)*x_ref[2]**o+(0.0410695954026)*x_ref[2]
        ref[1,4,1,0]=(-0.512877649374)*x_ref[0]**o+(-0.0190184293267)*x_ref[0]+(0.588904418509)*x_ref[1]**o+(-0.343320898588)*x_ref[1]+(-0.16543066117)*x_ref[2]**o+(-0.118363009949)*x_ref[2]
        ref[1,4,1,1]=(-0.410758135711)*x_ref[0]**o+(0.302846037727)*x_ref[0]+(0.674701250578)*x_ref[1]**o+(-0.96295102282)*x_ref[1]+(0.787271275616)*x_ref[2]**o+(0.925818905258)*x_ref[2]
        ref[1,4,2,0]=(0.629642717166)*x_ref[0]**o+(-0.696019558128)*x_ref[0]+(0.465609587281)*x_ref[1]**o+(0.191961667012)*x_ref[1]+(-0.0182853530912)*x_ref[2]**o+(0.504352699288)*x_ref[2]
        ref[1,4,2,1]=(-0.561295424723)*x_ref[0]**o+(0.830136434947)*x_ref[0]+(0.981595445935)*x_ref[1]**o+(-0.0881426921172)*x_ref[1]+(-0.581790941676)*x_ref[2]**o+(-0.489101727188)*x_ref[2]
        ref[2,0,0,0]=(0.173249525275)*x_ref[0]**o+(0.376871332154)*x_ref[0]+(-0.379974081425)*x_ref[1]**o+(-0.118151578515)*x_ref[1]+(0.371715379047)*x_ref[2]**o+(0.115273894116)*x_ref[2]
        ref[2,0,0,1]=(0.656339759814)*x_ref[0]**o+(-0.0783529441613)*x_ref[0]+(0.988608900721)*x_ref[1]**o+(-0.498370704737)*x_ref[1]+(0.886731679317)*x_ref[2]**o+(0.251026122418)*x_ref[2]
        ref[2,0,1,0]=(-0.350592561402)*x_ref[0]**o+(-0.592294993708)*x_ref[0]+(0.860912193414)*x_ref[1]**o+(0.678591565883)*x_ref[1]+(-0.823185915223)*x_ref[2]**o+(-0.882500202366)*x_ref[2]
        ref[2,0,1,1]=(0.84096537128)*x_ref[0]**o+(-0.78582850022)*x_ref[0]+(-0.872411932976)*x_ref[1]**o+(0.208867262377)*x_ref[1]+(0.795227125238)*x_ref[2]**o+(0.434019201479)*x_ref[2]
        ref[2,0,2,0]=(0.670038316075)*x_ref[0]**o+(0.221065948244)*x_ref[0]+(-0.0801945207329)*x_ref[1]**o+(0.35963157451)*x_ref[1]+(-0.523905944746)*x_ref[2]**o+(0.113327871154)*x_ref[2]
        ref[2,0,2,1]=(-0.790165918677)*x_ref[0]**o+(0.826330759592)*x_ref[0]+(0.816532564314)*x_ref[1]**o+(-0.947088135238)*x_ref[1]+(0.498519891556)*x_ref[2]**o+(-0.221796346503)*x_ref[2]
        ref[2,1,0,0]=(0.679814393957)*x_ref[0]**o+(-0.287910028702)*x_ref[0]+(-0.179717372152)*x_ref[1]**o+(-0.895091171351)*x_ref[1]+(0.453459163192)*x_ref[2]**o+(-0.732993783456)*x_ref[2]
        ref[2,1,0,1]=(-0.917014070487)*x_ref[0]**o+(-0.28427666054)*x_ref[0]+(0.520172057178)*x_ref[1]**o+(0.973381422092)*x_ref[1]+(-0.785604082848)*x_ref[2]**o+(-0.280246362235)*x_ref[2]
        ref[2,1,1,0]=(-0.975543038311)*x_ref[0]**o+(0.0471394295003)*x_ref[0]+(-0.744014701)*x_ref[1]**o+(-0.309281686644)*x_ref[1]+(0.970209409582)*x_ref[2]**o+(0.182638690597)*x_ref[2]
        ref[2,1,1,1]=(0.754879118472)*x_ref[0]**o+(-0.345631769168)*x_ref[0]+(0.864272677704)*x_ref[1]**o+(-0.634026748154)*x_ref[1]+(-0.25656646391)*x_ref[2]**o+(-0.976807129917)*x_ref[2]
        ref[2,1,2,0]=(0.0378913818133)*x_ref[0]**o+(-0.589418434757)*x_ref[0]+(0.500291612315)*x_ref[1]**o+(0.831173369231)*x_ref[1]+(0.0773865261454)*x_ref[2]**o+(-0.170449265388)*x_ref[2]
        ref[2,1,2,1]=(0.363668102897)*x_ref[0]**o+(0.0152347755192)*x_ref[0]+(0.295985339318)*x_ref[1]**o+(-0.695348667793)*x_ref[1]+(0.654035675777)*x_ref[2]**o+(-0.998480773881)*x_ref[2]
        ref[2,2,0,0]=(-0.248968924843)*x_ref[0]**o+(-0.454040417829)*x_ref[0]+(0.618989286259)*x_ref[1]**o+(-0.284627664137)*x_ref[1]+(0.597922789444)*x_ref[2]**o+(-0.0544707719136)*x_ref[2]
        ref[2,2,0,1]=(-0.564540493714)*x_ref[0]**o+(0.948462945081)*x_ref[0]+(-0.554305925084)*x_ref[1]**o+(-0.683216336448)*x_ref[1]+(-0.393720863511)*x_ref[2]**o+(-0.454706611434)*x_ref[2]
        ref[2,2,1,0]=(0.00267926298461)*x_ref[0]**o+(0.69057345924)*x_ref[0]+(0.972990086053)*x_ref[1]**o+(0.334585200812)*x_ref[1]+(0.458913228071)*x_ref[2]**o+(-0.927443691914)*x_ref[2]
        ref[2,2,1,1]=(-0.869817596024)*x_ref[0]**o+(0.183295661514)*x_ref[0]+(-0.215314084509)*x_ref[1]**o+(0.274879624616)*x_ref[1]+(-0.872459015851)*x_ref[2]**o+(-0.43371203901)*x_ref[2]
        ref[2,2,2,0]=(-0.59916469262)*x_ref[0]**o+(-0.635232312962)*x_ref[0]+(-0.730547369574)*x_ref[1]**o+(-0.199910106304)*x_ref[1]+(-0.81824698793)*x_ref[2]**o+(-0.351742356136)*x_ref[2]
        ref[2,2,2,1]=(0.52115654452)*x_ref[0]**o+(-0.648726400371)*x_ref[0]+(0.0705029035387)*x_ref[1]**o+(0.82069581623)*x_ref[1]+(0.262897099948)*x_ref[2]**o+(0.302543200334)*x_ref[2]
        ref[2,3,0,0]=(-0.887996543825)*x_ref[0]**o+(-0.141917392835)*x_ref[0]+(0.458379292046)*x_ref[1]**o+(0.279749722412)*x_ref[1]+(-0.185582606045)*x_ref[2]**o+(-0.408626717355)*x_ref[2]
        ref[2,3,0,1]=(-0.0708646458785)*x_ref[0]**o+(-0.723087450876)*x_ref[0]+(0.801053779885)*x_ref[1]**o+(-0.316692437663)*x_ref[1]+(0.313620016387)*x_ref[2]**o+(-0.802237492322)*x_ref[2]
        ref[2,3,1,0]=(-0.413969268245)*x_ref[0]**o+(-0.419189503807)*x_ref[0]+(-0.150655589102)*x_ref[1]**o+(-0.451064114786)*x_ref[1]+(-0.11752330372)*x_ref[2]**o+(-0.564979364641)*x_ref[2]
        ref[2,3,1,1]=(-0.127258808057)*x_ref[0]**o+(-0.166240866424)*x_ref[0]+(0.482435262134)*x_ref[1]**o+(0.210083811762)*x_ref[1]+(0.496523704735)*x_ref[2]**o+(-0.528305427062)*x_ref[2]
        ref[2,3,2,0]=(-0.619368230074)*x_ref[0]**o+(0.196929842772)*x_ref[0]+(0.45226305027)*x_ref[1]**o+(-0.949509654146)*x_ref[1]+(-0.200199092979)*x_ref[2]**o+(0.879368741494)*x_ref[2]
        ref[2,3,2,1]=(0.727974979035)*x_ref[0]**o+(-0.486118068882)*x_ref[0]+(0.330990540492)*x_ref[1]**o+(0.627547710515)*x_ref[1]+(-0.00312667743686)*x_ref[2]**o+(0.34180677866)*x_ref[2]
        ref[2,4,0,0]=(-0.849340738634)*x_ref[0]**o+(0.105575131464)*x_ref[0]+(0.531365100651)*x_ref[1]**o+(-0.885829562648)*x_ref[1]+(-0.959105112582)*x_ref[2]**o+(0.6210710685)*x_ref[2]
        ref[2,4,0,1]=(-0.525287919972)*x_ref[0]**o+(0.685382709266)*x_ref[0]+(0.706077262452)*x_ref[1]**o+(-0.577859356102)*x_ref[1]+(0.494281341217)*x_ref[2]**o+(0.936337201867)*x_ref[2]
        ref[2,4,1,0]=(-0.0263664228096)*x_ref[0]**o+(0.609335710155)*x_ref[0]+(-0.417542795725)*x_ref[1]**o+(0.127890280292)*x_ref[1]+(0.0530446261715)*x_ref[2]**o+(-0.203436845419)*x_ref[2]
        ref[2,4,1,1]=(0.58910765961)*x_ref[0]**o+(-0.233711911277)*x_ref[0]+(-0.68506108197)*x_ref[1]**o+(-0.796608219305)*x_ref[1]+(-0.55051026522)*x_ref[2]**o+(-0.663284299983)*x_ref[2]
        ref[2,4,2,0]=(0.407867058666)*x_ref[0]**o+(-0.155472572874)*x_ref[0]+(0.0574795179339)*x_ref[1]**o+(0.534562635965)*x_ref[1]+(-0.35303239373)*x_ref[2]**o+(0.690463242713)*x_ref[2]
        ref[2,4,2,1]=(0.458514172477)*x_ref[0]**o+(0.493538792362)*x_ref[0]+(-0.534087073406)*x_ref[1]**o+(0.140190942794)*x_ref[1]+(0.53904059561)*x_ref[2]**o+(0.511924245166)*x_ref[2]
        ref[3,0,0,0]=(0.80688664483)*x_ref[0]**o+(0.641990457435)*x_ref[0]+(0.258439350653)*x_ref[1]**o+(0.360949512497)*x_ref[1]+(0.71055262439)*x_ref[2]**o+(-0.14812833383)*x_ref[2]
        ref[3,0,0,1]=(0.349076196526)*x_ref[0]**o+(-0.250426445203)*x_ref[0]+(0.363350736371)*x_ref[1]**o+(0.62331683755)*x_ref[1]+(-0.799481730613)*x_ref[2]**o+(0.86946545402)*x_ref[2]
        ref[3,0,1,0]=(-0.461900551752)*x_ref[0]**o+(-0.530275233976)*x_ref[0]+(0.778112655963)*x_ref[1]**o+(-0.548160947996)*x_ref[1]+(0.27643541045)*x_ref[2]**o+(0.558519627971)*x_ref[2]
        ref[3,0,1,1]=(-0.870845434837)*x_ref[0]**o+(0.90473098031)*x_ref[0]+(0.356324338565)*x_ref[1]**o+(-0.173579775601)*x_ref[1]+(0.866960109024)*x_ref[2]**o+(0.22810880895)*x_ref[2]
        ref[3,0,2,0]=(0.449562471445)*x_ref[0]**o+(-0.133977676845)*x_ref[0]+(-0.945220890263)*x_ref[1]**o+(0.999013496171)*x_ref[1]+(-0.612837266133)*x_ref[2]**o+(-0.695797874487)*x_ref[2]
        ref[3,0,2,1]=(-0.588105630499)*x_ref[0]**o+(0.176592342414)*x_ref[0]+(-0.820991814796)*x_ref[1]**o+(0.0010811086852)*x_ref[1]+(0.498174763907)*x_ref[2]**o+(-0.0223969132485)*x_ref[2]
        ref[3,1,0,0]=(0.653483773188)*x_ref[0]**o+(0.460215104331)*x_ref[0]+(-0.287173677147)*x_ref[1]**o+(-0.837222104854)*x_ref[1]+(0.457524748826)*x_ref[2]**o+(-0.638268772518)*x_ref[2]
        ref[3,1,0,1]=(-0.0999951386915)*x_ref[0]**o+(0.294288175142)*x_ref[0]+(0.159178572562)*x_ref[1]**o+(0.13437803751)*x_ref[1]+(-0.411939992038)*x_ref[2]**o+(0.283003770703)*x_ref[2]
        ref[3,1,1,0]=(-0.862929516639)*x_ref[0]**o+(-0.455798937374)*x_ref[0]+(0.972383754049)*x_ref[1]**o+(0.251968652813)*x_ref[1]+(-0.631965845999)*x_ref[2]**o+(-0.418936218512)*x_ref[2]
        ref[3,1,1,1]=(-0.0265533722079)*x_ref[0]**o+(-0.720054910456)*x_ref[0]+(-0.134379948062)*x_ref[1]**o+(-0.535223772178)*x_ref[1]+(-0.711365379454)*x_ref[2]**o+(0.39963284442)*x_ref[2]
        ref[3,1,2,0]=(-0.9691936764)*x_ref[0]**o+(-0.220678275504)*x_ref[0]+(0.423830480119)*x_ref[1]**o+(-0.286816976029)*x_ref[1]+(-0.637835943762)*x_ref[2]**o+(0.389581647726)*x_ref[2]
        ref[3,1,2,1]=(-0.375225433223)*x_ref[0]**o+(0.339950113552)*x_ref[0]+(0.272089140767)*x_ref[1]**o+(-0.671811329081)*x_ref[1]+(-0.341834415939)*x_ref[2]**o+(-0.974852723909)*x_ref[2]
        ref[3,2,0,0]=(-0.13212102165)*x_ref[0]**o+(-0.529435682355)*x_ref[0]+(0.57321409695)*x_ref[1]**o+(0.123050260287)*x_ref[1]+(-0.50608599056)*x_ref[2]**o+(-0.258999018643)*x_ref[2]
        ref[3,2,0,1]=(-0.594320198162)*x_ref[0]**o+(0.59617231387)*x_ref[0]+(-0.132055038377)*x_ref[1]**o+(0.0267055057967)*x_ref[1]+(-0.0909023410879)*x_ref[2]**o+(-0.327689716933)*x_ref[2]
        ref[3,2,1,0]=(0.352248348539)*x_ref[0]**o+(-0.937172168074)*x_ref[0]+(0.492884278868)*x_ref[1]**o+(0.599164015797)*x_ref[1]+(-0.306391881602)*x_ref[2]**o+(-0.124914738757)*x_ref[2]
        ref[3,2,1,1]=(-0.124766143355)*x_ref[0]**o+(-0.201629423485)*x_ref[0]+(0.69199028272)*x_ref[1]**o+(0.0909253140313)*x_ref[1]+(-0.0807463990698)*x_ref[2]**o+(0.274206212499)*x_ref[2]
        ref[3,2,2,0]=(-0.939257892238)*x_ref[0]**o+(-0.0905023289297)*x_ref[0]+(-0.101189471915)*x_ref[1]**o+(-0.850452366024)*x_ref[1]+(0.706374760942)*x_ref[2]**o+(-0.0741989595692)*x_ref[2]
        ref[3,2,2,1]=(-0.956776001745)*x_ref[0]**o+(0.313587985523)*x_ref[0]+(0.501857466469)*x_ref[1]**o+(0.866521730616)*x_ref[1]+(-0.113777691526)*x_ref[2]**o+(-0.43724474427)*x_ref[2]
        ref[3,3,0,0]=(-0.794240122389)*x_ref[0]**o+(0.117246192404)*x_ref[0]+(0.730901073854)*x_ref[1]**o+(0.406753741761)*x_ref[1]+(-0.987815418361)*x_ref[2]**o+(0.543031203217)*x_ref[2]
        ref[3,3,0,1]=(-0.539899807208)*x_ref[0]**o+(0.425096558373)*x_ref[0]+(-0.261756005984)*x_ref[1]**o+(0.219196945781)*x_ref[1]+(-0.458238548056)*x_ref[2]**o+(-0.396891207522)*x_ref[2]
        ref[3,3,1,0]=(-0.512322191935)*x_ref[0]**o+(0.158356684082)*x_ref[0]+(-0.702591537714)*x_ref[1]**o+(0.202771352983)*x_ref[1]+(-0.346772902785)*x_ref[2]**o+(0.260366433089)*x_ref[2]
        ref[3,3,1,1]=(0.839858482584)*x_ref[0]**o+(0.132933571004)*x_ref[0]+(0.840682050408)*x_ref[1]**o+(0.490904029834)*x_ref[1]+(-0.298290729461)*x_ref[2]**o+(-0.467229890802)*x_ref[2]
        ref[3,3,2,0]=(-0.691489850326)*x_ref[0]**o+(0.852014173253)*x_ref[0]+(0.484568640775)*x_ref[1]**o+(0.0788611533234)*x_ref[1]+(0.335927774531)*x_ref[2]**o+(-0.507648906088)*x_ref[2]
        ref[3,3,2,1]=(0.140576758316)*x_ref[0]**o+(-0.783440190349)*x_ref[0]+(-0.408503163319)*x_ref[1]**o+(0.0871935010894)*x_ref[1]+(0.478151373206)*x_ref[2]**o+(0.837020241411)*x_ref[2]
        ref[3,4,0,0]=(-0.956822615698)*x_ref[0]**o+(0.594397877716)*x_ref[0]+(0.844357571956)*x_ref[1]**o+(0.173315874424)*x_ref[1]+(0.96181728365)*x_ref[2]**o+(-0.900445532048)*x_ref[2]
        ref[3,4,0,1]=(0.887824966747)*x_ref[0]**o+(-0.579258071661)*x_ref[0]+(0.983311456722)*x_ref[1]**o+(0.996052998934)*x_ref[1]+(0.215867343112)*x_ref[2]**o+(0.614855622355)*x_ref[2]
        ref[3,4,1,0]=(0.482666172028)*x_ref[0]**o+(0.0258626246719)*x_ref[0]+(-0.394140399663)*x_ref[1]**o+(0.192974809281)*x_ref[1]+(-0.265879383852)*x_ref[2]**o+(-0.0409441022179)*x_ref[2]
        ref[3,4,1,1]=(0.0600752930856)*x_ref[0]**o+(-0.885644457634)*x_ref[0]+(-0.54869761625)*x_ref[1]**o+(0.4745411417)*x_ref[1]+(-0.554764519536)*x_ref[2]**o+(-0.159514159559)*x_ref[2]
        ref[3,4,2,0]=(-0.804675220841)*x_ref[0]**o+(0.586839632342)*x_ref[0]+(-0.329997060623)*x_ref[1]**o+(0.242316008423)*x_ref[1]+(0.272064229179)*x_ref[2]**o+(0.24348627845)*x_ref[2]
        ref[3,4,2,1]=(0.847664079563)*x_ref[0]**o+(0.927723091901)*x_ref[0]+(-0.861048553467)*x_ref[1]**o+(-0.413216499555)*x_ref[1]+(0.546938488463)*x_ref[2]**o+(0.0558689143145)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_ReducedSolution_rank0(self):
      """
      tests interpolate for rank 0 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedSolution onto ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(),dim=dim)
      s=Data(0,(),w)
      ref=Data(0,(),w_ref)
      if dim==2:
        s=(-1.64974193328)*x[0]+(1.68675911612)*x[1]
        ref=(-1.64974193328)*x_ref[0]+(1.68675911612)*x_ref[1]
      else:
        s=(-0.517718083008)*x[0]+(0.479679773182)*x[1]+(0.72459612383)*x[2]
        ref=(-0.517718083008)*x_ref[0]+(0.479679773182)*x_ref[1]+(0.72459612383)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_ReducedSolution_rank1(self):
      """
      tests interpolate for rank 1 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedSolution onto ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(2,),dim=dim)
      s=Data(0,(2,),w)
      ref=Data(0,(2,),w_ref)
      if dim==2:
        s[0]=(-0.415290633997)*x[0]+(0.118775036022)*x[1]
        s[1]=(-0.833749289762)*x[0]+(0.121297221921)*x[1]
        ref[0]=(-0.415290633997)*x[0]+(0.118775036022)*x[1]
        ref[1]=(-0.833749289762)*x[0]+(0.121297221921)*x[1]
      else:
        s[0]=(-0.336174941008)*x[0]+(-0.844918885743)*x[1]+(-0.410649979186)*x[2]
        s[1]=(0.112527237498)*x[0]+(-0.867632388528)*x[1]+(1.42359104479)*x[2]
        ref[0]=(-0.336174941008)*x[0]+(-0.844918885743)*x[1]+(-0.410649979186)*x[2]
        ref[1]=(0.112527237498)*x[0]+(-0.867632388528)*x[1]+(1.42359104479)*x[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_ReducedSolution_rank2(self):
      """
      tests interpolate for rank 2 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedSolution onto ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5),dim=dim)
      s=Data(0,(4, 5),w)
      ref=Data(0,(4, 5),w_ref)
      if dim==2:
        s[0,0]=(-0.833920166667)*x[0]+(-0.105734397941)*x[1]
        s[0,1]=(-0.0535194301789)*x[0]+(1.2869199411)*x[1]
        s[0,2]=(-1.81783608134)*x[0]+(-0.0858549172679)*x[1]
        s[0,3]=(0.133570573696)*x[0]+(0.398281712973)*x[1]
        s[0,4]=(-1.2514060292)*x[0]+(-0.24743946144)*x[1]
        s[1,0]=(0.917760776668)*x[0]+(0.179758313228)*x[1]
        s[1,1]=(0.677772149724)*x[0]+(0.762705763999)*x[1]
        s[1,2]=(-0.505172381789)*x[0]+(0.412365083581)*x[1]
        s[1,3]=(0.0481873177823)*x[0]+(-0.449871653505)*x[1]
        s[1,4]=(-1.43052283016)*x[0]+(0.553426132082)*x[1]
        s[2,0]=(0.337172157552)*x[0]+(1.56154138911)*x[1]
        s[2,1]=(-0.338189138128)*x[0]+(-0.62614063612)*x[1]
        s[2,2]=(-1.51606676783)*x[0]+(-0.13283454533)*x[1]
        s[2,3]=(0.727068106091)*x[0]+(1.00949960235)*x[1]
        s[2,4]=(0.297941246402)*x[0]+(0.952119168452)*x[1]
        s[3,0]=(0.724157315025)*x[0]+(0.962405533793)*x[1]
        s[3,1]=(-0.239168209123)*x[0]+(1.28280794616)*x[1]
        s[3,2]=(-0.0336024791682)*x[0]+(-0.775860593407)*x[1]
        s[3,3]=(-0.118809020482)*x[0]+(0.51890714417)*x[1]
        s[3,4]=(0.374315302545)*x[0]+(-0.258942994)*x[1]
        ref[0,0]=(-0.833920166667)*x_ref[0]+(-0.105734397941)*x_ref[1]
        ref[0,1]=(-0.0535194301789)*x_ref[0]+(1.2869199411)*x_ref[1]
        ref[0,2]=(-1.81783608134)*x_ref[0]+(-0.0858549172679)*x_ref[1]
        ref[0,3]=(0.133570573696)*x_ref[0]+(0.398281712973)*x_ref[1]
        ref[0,4]=(-1.2514060292)*x_ref[0]+(-0.24743946144)*x_ref[1]
        ref[1,0]=(0.917760776668)*x_ref[0]+(0.179758313228)*x_ref[1]
        ref[1,1]=(0.677772149724)*x_ref[0]+(0.762705763999)*x_ref[1]
        ref[1,2]=(-0.505172381789)*x_ref[0]+(0.412365083581)*x_ref[1]
        ref[1,3]=(0.0481873177823)*x_ref[0]+(-0.449871653505)*x_ref[1]
        ref[1,4]=(-1.43052283016)*x_ref[0]+(0.553426132082)*x_ref[1]
        ref[2,0]=(0.337172157552)*x_ref[0]+(1.56154138911)*x_ref[1]
        ref[2,1]=(-0.338189138128)*x_ref[0]+(-0.62614063612)*x_ref[1]
        ref[2,2]=(-1.51606676783)*x_ref[0]+(-0.13283454533)*x_ref[1]
        ref[2,3]=(0.727068106091)*x_ref[0]+(1.00949960235)*x_ref[1]
        ref[2,4]=(0.297941246402)*x_ref[0]+(0.952119168452)*x_ref[1]
        ref[3,0]=(0.724157315025)*x_ref[0]+(0.962405533793)*x_ref[1]
        ref[3,1]=(-0.239168209123)*x_ref[0]+(1.28280794616)*x_ref[1]
        ref[3,2]=(-0.0336024791682)*x_ref[0]+(-0.775860593407)*x_ref[1]
        ref[3,3]=(-0.118809020482)*x_ref[0]+(0.51890714417)*x_ref[1]
        ref[3,4]=(0.374315302545)*x_ref[0]+(-0.258942994)*x_ref[1]
      else:
        s[0,0]=(1.18033303246)*x[0]+(0.59372740835)*x[1]+(-0.684959540977)*x[2]
        s[0,1]=(0.65301033503)*x[0]+(-1.17147722359)*x[1]+(0.455404166733)*x[2]
        s[0,2]=(-0.412048337979)*x[0]+(-1.24933561194)*x[1]+(-0.636287202752)*x[2]
        s[0,3]=(0.122156598551)*x[0]+(0.103952485283)*x[1]+(-0.564471099255)*x[2]
        s[0,4]=(0.737873603859)*x[0]+(0.139615778712)*x[1]+(0.523544473477)*x[2]
        s[1,0]=(0.0198124265514)*x[0]+(-0.335073882197)*x[1]+(-0.805965691618)*x[2]
        s[1,1]=(-1.28485879121)*x[0]+(-0.618847596536)*x[1]+(1.32879899911)*x[2]
        s[1,2]=(1.09319515176)*x[0]+(0.543230471078)*x[1]+(1.56430808151)*x[2]
        s[1,3]=(0.541425658238)*x[0]+(0.0140217060596)*x[1]+(0.603558416867)*x[2]
        s[1,4]=(0.303110106189)*x[0]+(-0.103009794147)*x[1]+(-1.68324105571)*x[2]
        s[2,0]=(-1.65609235169)*x[0]+(-0.290040263496)*x[1]+(0.477073850071)*x[2]
        s[2,1]=(-0.897378807931)*x[0]+(0.191583115578)*x[1]+(0.213226828436)*x[2]
        s[2,2]=(0.523724313062)*x[0]+(0.214413673529)*x[1]+(0.533345615094)*x[2]
        s[2,3]=(-0.994043729754)*x[0]+(-0.307957055273)*x[1]+(0.0422995421427)*x[2]
        s[2,4]=(1.22590973976)*x[0]+(-1.65014223339)*x[1]+(-1.59726996804)*x[2]
        s[3,0]=(-0.0870943824713)*x[0]+(-0.45766718732)*x[1]+(-0.878874753774)*x[2]
        s[3,1]=(0.244470697307)*x[0]+(-0.0429568231364)*x[1]+(-1.57098356615)*x[2]
        s[3,2]=(1.35091699523)*x[0]+(-0.127297590922)*x[1]+(-0.556702799475)*x[2]
        s[3,3]=(0.189117166393)*x[0]+(0.317562328009)*x[1]+(-0.59788285723)*x[2]
        s[3,4]=(-0.155355776878)*x[0]+(-0.616350184089)*x[1]+(0.296083681054)*x[2]
        ref[0,0]=(1.18033303246)*x_ref[0]+(0.59372740835)*x_ref[1]+(-0.684959540977)*x_ref[2]
        ref[0,1]=(0.65301033503)*x_ref[0]+(-1.17147722359)*x_ref[1]+(0.455404166733)*x_ref[2]
        ref[0,2]=(-0.412048337979)*x_ref[0]+(-1.24933561194)*x_ref[1]+(-0.636287202752)*x_ref[2]
        ref[0,3]=(0.122156598551)*x_ref[0]+(0.103952485283)*x_ref[1]+(-0.564471099255)*x_ref[2]
        ref[0,4]=(0.737873603859)*x_ref[0]+(0.139615778712)*x_ref[1]+(0.523544473477)*x_ref[2]
        ref[1,0]=(0.0198124265514)*x_ref[0]+(-0.335073882197)*x_ref[1]+(-0.805965691618)*x_ref[2]
        ref[1,1]=(-1.28485879121)*x_ref[0]+(-0.618847596536)*x_ref[1]+(1.32879899911)*x_ref[2]
        ref[1,2]=(1.09319515176)*x_ref[0]+(0.543230471078)*x_ref[1]+(1.56430808151)*x_ref[2]
        ref[1,3]=(0.541425658238)*x_ref[0]+(0.0140217060596)*x_ref[1]+(0.603558416867)*x_ref[2]
        ref[1,4]=(0.303110106189)*x_ref[0]+(-0.103009794147)*x_ref[1]+(-1.68324105571)*x_ref[2]
        ref[2,0]=(-1.65609235169)*x_ref[0]+(-0.290040263496)*x_ref[1]+(0.477073850071)*x_ref[2]
        ref[2,1]=(-0.897378807931)*x_ref[0]+(0.191583115578)*x_ref[1]+(0.213226828436)*x_ref[2]
        ref[2,2]=(0.523724313062)*x_ref[0]+(0.214413673529)*x_ref[1]+(0.533345615094)*x_ref[2]
        ref[2,3]=(-0.994043729754)*x_ref[0]+(-0.307957055273)*x_ref[1]+(0.0422995421427)*x_ref[2]
        ref[2,4]=(1.22590973976)*x_ref[0]+(-1.65014223339)*x_ref[1]+(-1.59726996804)*x_ref[2]
        ref[3,0]=(-0.0870943824713)*x_ref[0]+(-0.45766718732)*x_ref[1]+(-0.878874753774)*x_ref[2]
        ref[3,1]=(0.244470697307)*x_ref[0]+(-0.0429568231364)*x_ref[1]+(-1.57098356615)*x_ref[2]
        ref[3,2]=(1.35091699523)*x_ref[0]+(-0.127297590922)*x_ref[1]+(-0.556702799475)*x_ref[2]
        ref[3,3]=(0.189117166393)*x_ref[0]+(0.317562328009)*x_ref[1]+(-0.59788285723)*x_ref[2]
        ref[3,4]=(-0.155355776878)*x_ref[0]+(-0.616350184089)*x_ref[1]+(0.296083681054)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_ReducedSolution_rank3(self):
      """
      tests interpolate for rank 3 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedSolution onto ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(6, 2, 2),dim=dim)
      s=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2),w_ref)
      if dim==2:
        s[0,0,0]=(0.0730549738553)*x[0]+(0.585649475666)*x[1]
        s[0,0,1]=(-0.449075310223)*x[0]+(1.39116964942)*x[1]
        s[0,1,0]=(0.0210032238157)*x[0]+(1.33904896697)*x[1]
        s[0,1,1]=(0.556125399236)*x[0]+(-0.00311484949343)*x[1]
        s[1,0,0]=(0.938296971476)*x[0]+(1.13223213262)*x[1]
        s[1,0,1]=(0.744133292867)*x[0]+(0.0271311625871)*x[1]
        s[1,1,0]=(-0.0478922372672)*x[0]+(-1.08882574207)*x[1]
        s[1,1,1]=(0.738328994373)*x[0]+(-0.730449948444)*x[1]
        s[2,0,0]=(0.66593099915)*x[0]+(-0.185918605751)*x[1]
        s[2,0,1]=(0.450972410432)*x[0]+(1.3393675839)*x[1]
        s[2,1,0]=(0.897370528228)*x[0]+(0.765127571621)*x[1]
        s[2,1,1]=(-0.349658725406)*x[0]+(-0.536332219959)*x[1]
        s[3,0,0]=(1.11112898963)*x[0]+(1.50253301112)*x[1]
        s[3,0,1]=(0.238957188525)*x[0]+(0.775368032877)*x[1]
        s[3,1,0]=(0.612991596596)*x[0]+(0.0718037077356)*x[1]
        s[3,1,1]=(-0.40039392925)*x[0]+(-1.58712410632)*x[1]
        s[4,0,0]=(0.23241254342)*x[0]+(-0.547054930927)*x[1]
        s[4,0,1]=(-0.976995206698)*x[0]+(1.251791534)*x[1]
        s[4,1,0]=(-1.13243662763)*x[0]+(0.56739947752)*x[1]
        s[4,1,1]=(-0.543613180121)*x[0]+(0.649805921416)*x[1]
        s[5,0,0]=(-0.0690689383083)*x[0]+(0.818563064557)*x[1]
        s[5,0,1]=(0.362899720121)*x[0]+(-0.112604112729)*x[1]
        s[5,1,0]=(-0.291756903179)*x[0]+(-0.538221294595)*x[1]
        s[5,1,1]=(0.60406331286)*x[0]+(0.228744896658)*x[1]
        ref[0,0,0]=(0.0730549738553)*x_ref[0]+(0.585649475666)*x_ref[1]
        ref[0,0,1]=(-0.449075310223)*x_ref[0]+(1.39116964942)*x_ref[1]
        ref[0,1,0]=(0.0210032238157)*x_ref[0]+(1.33904896697)*x_ref[1]
        ref[0,1,1]=(0.556125399236)*x_ref[0]+(-0.00311484949343)*x_ref[1]
        ref[1,0,0]=(0.938296971476)*x_ref[0]+(1.13223213262)*x_ref[1]
        ref[1,0,1]=(0.744133292867)*x_ref[0]+(0.0271311625871)*x_ref[1]
        ref[1,1,0]=(-0.0478922372672)*x_ref[0]+(-1.08882574207)*x_ref[1]
        ref[1,1,1]=(0.738328994373)*x_ref[0]+(-0.730449948444)*x_ref[1]
        ref[2,0,0]=(0.66593099915)*x_ref[0]+(-0.185918605751)*x_ref[1]
        ref[2,0,1]=(0.450972410432)*x_ref[0]+(1.3393675839)*x_ref[1]
        ref[2,1,0]=(0.897370528228)*x_ref[0]+(0.765127571621)*x_ref[1]
        ref[2,1,1]=(-0.349658725406)*x_ref[0]+(-0.536332219959)*x_ref[1]
        ref[3,0,0]=(1.11112898963)*x_ref[0]+(1.50253301112)*x_ref[1]
        ref[3,0,1]=(0.238957188525)*x_ref[0]+(0.775368032877)*x_ref[1]
        ref[3,1,0]=(0.612991596596)*x_ref[0]+(0.0718037077356)*x_ref[1]
        ref[3,1,1]=(-0.40039392925)*x_ref[0]+(-1.58712410632)*x_ref[1]
        ref[4,0,0]=(0.23241254342)*x_ref[0]+(-0.547054930927)*x_ref[1]
        ref[4,0,1]=(-0.976995206698)*x_ref[0]+(1.251791534)*x_ref[1]
        ref[4,1,0]=(-1.13243662763)*x_ref[0]+(0.56739947752)*x_ref[1]
        ref[4,1,1]=(-0.543613180121)*x_ref[0]+(0.649805921416)*x_ref[1]
        ref[5,0,0]=(-0.0690689383083)*x_ref[0]+(0.818563064557)*x_ref[1]
        ref[5,0,1]=(0.362899720121)*x_ref[0]+(-0.112604112729)*x_ref[1]
        ref[5,1,0]=(-0.291756903179)*x_ref[0]+(-0.538221294595)*x_ref[1]
        ref[5,1,1]=(0.60406331286)*x_ref[0]+(0.228744896658)*x_ref[1]
      else:
        s[0,0,0]=(0.811538724853)*x[0]+(-0.347250364239)*x[1]+(-0.116011094391)*x[2]
        s[0,0,1]=(1.69112205586)*x[0]+(-1.04197300041)*x[1]+(0.849118123462)*x[2]
        s[0,1,0]=(1.30816943778)*x[0]+(1.01636941747)*x[1]+(-0.849746116393)*x[2]
        s[0,1,1]=(0.888683598925)*x[0]+(-1.60930356378)*x[1]+(0.522151570265)*x[2]
        s[1,0,0]=(1.03821316703)*x[0]+(-1.19647941504)*x[1]+(-1.08543246224)*x[2]
        s[1,0,1]=(-0.314182278588)*x[0]+(0.144825414517)*x[1]+(0.162613879231)*x[2]
        s[1,1,0]=(0.0416209427382)*x[0]+(0.0390092175082)*x[1]+(1.55616366687)*x[2]
        s[1,1,1]=(-0.577251716148)*x[0]+(0.717529140134)*x[1]+(-0.567199280535)*x[2]
        s[2,0,0]=(1.26716303575)*x[0]+(0.119576983216)*x[1]+(-0.602181033831)*x[2]
        s[2,0,1]=(0.533935330879)*x[0]+(0.292003869006)*x[1]+(-0.0917835680855)*x[2]
        s[2,1,0]=(-1.50218600857)*x[0]+(1.23418550746)*x[1]+(-0.600240530145)*x[2]
        s[2,1,1]=(0.689151741161)*x[0]+(0.909077564156)*x[1]+(0.924638961344)*x[2]
        s[3,0,0]=(-0.763411711557)*x[0]+(-0.862413521307)*x[1]+(1.49095932554)*x[2]
        s[3,0,1]=(-0.215589144977)*x[0]+(0.305301450314)*x[1]+(-0.468853406603)*x[2]
        s[3,1,0]=(-0.433045929433)*x[0]+(-0.907669929171)*x[1]+(0.917462300039)*x[2]
        s[3,1,1]=(0.229593212783)*x[0]+(-1.79371444035)*x[1]+(-0.848409349534)*x[2]
        s[4,0,0]=(1.40249316312)*x[0]+(-0.766609553683)*x[1]+(0.337911083882)*x[2]
        s[4,0,1]=(0.728650674867)*x[0]+(0.631222570369)*x[1]+(1.2491315131)*x[2]
        s[4,1,0]=(1.03998601102)*x[0]+(-1.81204917532)*x[1]+(0.781499164268)*x[2]
        s[4,1,1]=(-0.151999545303)*x[0]+(-0.767585861167)*x[1]+(1.17261654125)*x[2]
        s[5,0,0]=(0.0632249575346)*x[0]+(0.333565970302)*x[1]+(-1.66679001354)*x[2]
        s[5,0,1]=(0.945677231929)*x[0]+(-1.15936834775)*x[1]+(-0.644891644457)*x[2]
        s[5,1,0]=(1.08939087839)*x[0]+(0.687932839014)*x[1]+(-0.473426758917)*x[2]
        s[5,1,1]=(0.0349915672648)*x[0]+(-1.48593425129)*x[1]+(0.145421756236)*x[2]
        ref[0,0,0]=(0.811538724853)*x_ref[0]+(-0.347250364239)*x_ref[1]+(-0.116011094391)*x_ref[2]
        ref[0,0,1]=(1.69112205586)*x_ref[0]+(-1.04197300041)*x_ref[1]+(0.849118123462)*x_ref[2]
        ref[0,1,0]=(1.30816943778)*x_ref[0]+(1.01636941747)*x_ref[1]+(-0.849746116393)*x_ref[2]
        ref[0,1,1]=(0.888683598925)*x_ref[0]+(-1.60930356378)*x_ref[1]+(0.522151570265)*x_ref[2]
        ref[1,0,0]=(1.03821316703)*x_ref[0]+(-1.19647941504)*x_ref[1]+(-1.08543246224)*x_ref[2]
        ref[1,0,1]=(-0.314182278588)*x_ref[0]+(0.144825414517)*x_ref[1]+(0.162613879231)*x_ref[2]
        ref[1,1,0]=(0.0416209427382)*x_ref[0]+(0.0390092175082)*x_ref[1]+(1.55616366687)*x_ref[2]
        ref[1,1,1]=(-0.577251716148)*x_ref[0]+(0.717529140134)*x_ref[1]+(-0.567199280535)*x_ref[2]
        ref[2,0,0]=(1.26716303575)*x_ref[0]+(0.119576983216)*x_ref[1]+(-0.602181033831)*x_ref[2]
        ref[2,0,1]=(0.533935330879)*x_ref[0]+(0.292003869006)*x_ref[1]+(-0.0917835680855)*x_ref[2]
        ref[2,1,0]=(-1.50218600857)*x_ref[0]+(1.23418550746)*x_ref[1]+(-0.600240530145)*x_ref[2]
        ref[2,1,1]=(0.689151741161)*x_ref[0]+(0.909077564156)*x_ref[1]+(0.924638961344)*x_ref[2]
        ref[3,0,0]=(-0.763411711557)*x_ref[0]+(-0.862413521307)*x_ref[1]+(1.49095932554)*x_ref[2]
        ref[3,0,1]=(-0.215589144977)*x_ref[0]+(0.305301450314)*x_ref[1]+(-0.468853406603)*x_ref[2]
        ref[3,1,0]=(-0.433045929433)*x_ref[0]+(-0.907669929171)*x_ref[1]+(0.917462300039)*x_ref[2]
        ref[3,1,1]=(0.229593212783)*x_ref[0]+(-1.79371444035)*x_ref[1]+(-0.848409349534)*x_ref[2]
        ref[4,0,0]=(1.40249316312)*x_ref[0]+(-0.766609553683)*x_ref[1]+(0.337911083882)*x_ref[2]
        ref[4,0,1]=(0.728650674867)*x_ref[0]+(0.631222570369)*x_ref[1]+(1.2491315131)*x_ref[2]
        ref[4,1,0]=(1.03998601102)*x_ref[0]+(-1.81204917532)*x_ref[1]+(0.781499164268)*x_ref[2]
        ref[4,1,1]=(-0.151999545303)*x_ref[0]+(-0.767585861167)*x_ref[1]+(1.17261654125)*x_ref[2]
        ref[5,0,0]=(0.0632249575346)*x_ref[0]+(0.333565970302)*x_ref[1]+(-1.66679001354)*x_ref[2]
        ref[5,0,1]=(0.945677231929)*x_ref[0]+(-1.15936834775)*x_ref[1]+(-0.644891644457)*x_ref[2]
        ref[5,1,0]=(1.08939087839)*x_ref[0]+(0.687932839014)*x_ref[1]+(-0.473426758917)*x_ref[2]
        ref[5,1,1]=(0.0349915672648)*x_ref[0]+(-1.48593425129)*x_ref[1]+(0.145421756236)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_ReducedSolution_rank4(self):
      """
      tests interpolate for rank 4 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedSolution onto ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5, 3, 2),dim=dim)
      s=Data(0,(4, 5, 3, 2),w)
      ref=Data(0,(4, 5, 3, 2),w_ref)
      if dim==2:
        s[0,0,0,0]=(0.609727033659)*x[0]+(-0.901858940633)*x[1]
        s[0,0,0,1]=(-0.221259461243)*x[0]+(0.351136340908)*x[1]
        s[0,0,1,0]=(0.497173734282)*x[0]+(0.899142906899)*x[1]
        s[0,0,1,1]=(-1.55206649551)*x[0]+(-0.680257384044)*x[1]
        s[0,0,2,0]=(-0.932033638033)*x[0]+(-0.46503758126)*x[1]
        s[0,0,2,1]=(-0.0372650123525)*x[0]+(0.0173714029098)*x[1]
        s[0,1,0,0]=(0.855963098465)*x[0]+(0.760176320898)*x[1]
        s[0,1,0,1]=(1.37546120327)*x[0]+(1.72916158331)*x[1]
        s[0,1,1,0]=(0.25469861613)*x[0]+(0.71445716033)*x[1]
        s[0,1,1,1]=(0.198455659765)*x[0]+(0.0410276829495)*x[1]
        s[0,1,2,0]=(-0.628080427667)*x[0]+(-0.0197206025984)*x[1]
        s[0,1,2,1]=(1.26759029895)*x[0]+(1.15914953253)*x[1]
        s[0,2,0,0]=(-1.43203053441)*x[0]+(0.366301170069)*x[1]
        s[0,2,0,1]=(0.420527802229)*x[0]+(-0.709209747415)*x[1]
        s[0,2,1,0]=(0.399042264829)*x[0]+(0.0453132754731)*x[1]
        s[0,2,1,1]=(1.2074346062)*x[0]+(-1.14163410992)*x[1]
        s[0,2,2,0]=(-0.00270984158569)*x[0]+(-1.63216200182)*x[1]
        s[0,2,2,1]=(0.174276088847)*x[0]+(-1.04789472231)*x[1]
        s[0,3,0,0]=(-0.47704729036)*x[0]+(-1.2550637204)*x[1]
        s[0,3,0,1]=(0.348356873191)*x[0]+(-0.796668578265)*x[1]
        s[0,3,1,0]=(-0.52724725097)*x[0]+(1.26163123592)*x[1]
        s[0,3,1,1]=(-1.52681799261)*x[0]+(0.936041931735)*x[1]
        s[0,3,2,0]=(1.33689673219)*x[0]+(-1.34901214018)*x[1]
        s[0,3,2,1]=(1.57651286047)*x[0]+(0.245413762864)*x[1]
        s[0,4,0,0]=(0.194500181498)*x[0]+(-0.0231951844441)*x[1]
        s[0,4,0,1]=(-1.49372076697)*x[0]+(-0.111369678181)*x[1]
        s[0,4,1,0]=(-0.74121646836)*x[0]+(-0.817740156307)*x[1]
        s[0,4,1,1]=(-0.233643397123)*x[0]+(0.672715068796)*x[1]
        s[0,4,2,0]=(1.05850947579)*x[0]+(0.386484386191)*x[1]
        s[0,4,2,1]=(1.45105922963)*x[0]+(-0.367458625996)*x[1]
        s[1,0,0,0]=(-0.391803788669)*x[0]+(0.797592300662)*x[1]
        s[1,0,0,1]=(-0.205299072883)*x[0]+(0.677427935558)*x[1]
        s[1,0,1,0]=(1.56225329031)*x[0]+(1.17468092515)*x[1]
        s[1,0,1,1]=(1.4586005658)*x[0]+(1.36481062943)*x[1]
        s[1,0,2,0]=(-1.53281881413)*x[0]+(0.207028833974)*x[1]
        s[1,0,2,1]=(0.0469239691648)*x[0]+(1.03547812705)*x[1]
        s[1,1,0,0]=(1.39614335897)*x[0]+(0.82046489695)*x[1]
        s[1,1,0,1]=(-0.775462741637)*x[0]+(-0.275081761646)*x[1]
        s[1,1,1,0]=(0.173353028608)*x[0]+(-1.05620431816)*x[1]
        s[1,1,1,1]=(-0.464192870247)*x[0]+(0.212204784206)*x[1]
        s[1,1,2,0]=(-0.383813531866)*x[0]+(-0.0773759374234)*x[1]
        s[1,1,2,1]=(-1.07811385759)*x[0]+(-0.457232330819)*x[1]
        s[1,2,0,0]=(-0.295869147576)*x[0]+(0.166843915631)*x[1]
        s[1,2,0,1]=(0.571421034325)*x[0]+(1.46722583245)*x[1]
        s[1,2,1,0]=(-0.34494944257)*x[0]+(-0.286867052491)*x[1]
        s[1,2,1,1]=(1.43128885761)*x[0]+(0.21277887035)*x[1]
        s[1,2,2,0]=(-1.19397187873)*x[0]+(0.126397665227)*x[1]
        s[1,2,2,1]=(0.171368751954)*x[0]+(-1.12100254625)*x[1]
        s[1,3,0,0]=(0.0975333077604)*x[0]+(0.583524660188)*x[1]
        s[1,3,0,1]=(0.0271450955458)*x[0]+(0.368978332867)*x[1]
        s[1,3,1,0]=(1.22806968486)*x[0]+(-0.569932748468)*x[1]
        s[1,3,1,1]=(-0.691989839521)*x[0]+(-0.829822182922)*x[1]
        s[1,3,2,0]=(0.743776534098)*x[0]+(1.3668937127)*x[1]
        s[1,3,2,1]=(0.192206575204)*x[0]+(0.96576702824)*x[1]
        s[1,4,0,0]=(0.149381460079)*x[0]+(-0.987875538036)*x[1]
        s[1,4,0,1]=(-0.706855836683)*x[0]+(0.0410884211631)*x[1]
        s[1,4,1,0]=(-0.222650274)*x[0]+(0.59589242048)*x[1]
        s[1,4,1,1]=(0.541489186743)*x[0]+(-1.22543996074)*x[1]
        s[1,4,2,0]=(0.630234824652)*x[0]+(0.508436575179)*x[1]
        s[1,4,2,1]=(0.990062359294)*x[0]+(-0.639884693407)*x[1]
        s[2,0,0,0]=(0.378887461733)*x[0]+(1.71999809344)*x[1]
        s[2,0,0,1]=(0.182412683544)*x[0]+(-0.401183243877)*x[1]
        s[2,0,1,0]=(0.325775290396)*x[0]+(-0.656088786126)*x[1]
        s[2,0,1,1]=(0.316355229742)*x[0]+(1.54087545195)*x[1]
        s[2,0,2,0]=(1.14203881183)*x[0]+(0.703207191364)*x[1]
        s[2,0,2,1]=(-1.05211790226)*x[0]+(1.68596483036)*x[1]
        s[2,1,0,0]=(0.704109387934)*x[0]+(-0.589003921574)*x[1]
        s[2,1,0,1]=(-0.382489922028)*x[0]+(0.387647642942)*x[1]
        s[2,1,1,0]=(-0.0343434958891)*x[0]+(-0.119176404279)*x[1]
        s[2,1,1,1]=(0.567853527929)*x[0]+(1.32523261969)*x[1]
        s[2,1,2,0]=(1.03496000653)*x[0]+(0.110203591459)*x[1]
        s[2,1,2,1]=(0.560149997806)*x[0]+(-1.62893742873)*x[1]
        s[2,2,0,0]=(1.1801792312)*x[0]+(1.1912385935)*x[1]
        s[2,2,0,1]=(0.936932817634)*x[0]+(-1.49082021974)*x[1]
        s[2,2,1,0]=(-0.305582892069)*x[0]+(-0.267235698563)*x[1]
        s[2,2,1,1]=(0.107449730184)*x[0]+(-0.325267993615)*x[1]
        s[2,2,2,0]=(-1.08118986347)*x[0]+(0.144190727334)*x[1]
        s[2,2,2,1]=(1.44755060146)*x[0]+(-0.127612338817)*x[1]
        s[2,3,0,0]=(-0.666943112999)*x[0]+(-0.780402371989)*x[1]
        s[2,3,0,1]=(0.724206717771)*x[0]+(-0.523593908278)*x[1]
        s[2,3,1,0]=(1.86101197199)*x[0]+(-0.710894911302)*x[1]
        s[2,3,1,1]=(-1.74409232023)*x[0]+(0.407821816271)*x[1]
        s[2,3,2,0]=(0.646304172575)*x[0]+(0.0132628778543)*x[1]
        s[2,3,2,1]=(-0.746041438673)*x[0]+(0.516304872741)*x[1]
        s[2,4,0,0]=(0.0326900201283)*x[0]+(-0.533021156401)*x[1]
        s[2,4,0,1]=(0.088598956981)*x[0]+(0.764307970269)*x[1]
        s[2,4,1,0]=(-0.749553477756)*x[0]+(0.379646792939)*x[1]
        s[2,4,1,1]=(-0.243635321378)*x[0]+(-0.470022600261)*x[1]
        s[2,4,2,0]=(0.586398757247)*x[0]+(-0.116982897683)*x[1]
        s[2,4,2,1]=(-0.989012121879)*x[0]+(-1.25006535257)*x[1]
        s[3,0,0,0]=(1.25228306081)*x[0]+(-0.0932717035896)*x[1]
        s[3,0,0,1]=(1.5975516771)*x[0]+(-0.177733286111)*x[1]
        s[3,0,1,0]=(-0.659910070911)*x[0]+(-0.0999498944812)*x[1]
        s[3,0,1,1]=(-0.299870273662)*x[0]+(-1.22977656744)*x[1]
        s[3,0,2,0]=(-0.158211581143)*x[0]+(-0.141421380503)*x[1]
        s[3,0,2,1]=(-1.48226452788)*x[0]+(-0.731264943309)*x[1]
        s[3,1,0,0]=(-1.64589827853)*x[0]+(1.19692573998)*x[1]
        s[3,1,0,1]=(0.361329655357)*x[0]+(0.234310482171)*x[1]
        s[3,1,1,0]=(-1.14077649516)*x[0]+(0.132657084065)*x[1]
        s[3,1,1,1]=(1.41067975663)*x[0]+(0.818243117884)*x[1]
        s[3,1,2,0]=(-1.53137884342)*x[0]+(-0.0127805333093)*x[1]
        s[3,1,2,1]=(0.240455755486)*x[0]+(-1.02361753893)*x[1]
        s[3,2,0,0]=(-0.334803473072)*x[0]+(-0.807452510423)*x[1]
        s[3,2,0,1]=(0.617324536974)*x[0]+(-0.733163471774)*x[1]
        s[3,2,1,0]=(-0.634580050897)*x[0]+(0.362106278168)*x[1]
        s[3,2,1,1]=(-0.21525531818)*x[0]+(0.982902540898)*x[1]
        s[3,2,2,0]=(-0.590682803195)*x[0]+(-1.27537880632)*x[1]
        s[3,2,2,1]=(0.142578313438)*x[0]+(0.212341687291)*x[1]
        s[3,3,0,0]=(-0.758345317216)*x[0]+(-0.971914631359)*x[1]
        s[3,3,0,1]=(0.569463909234)*x[0]+(0.498848917338)*x[1]
        s[3,3,1,0]=(-0.771456575632)*x[0]+(-0.130456072944)*x[1]
        s[3,3,1,1]=(0.811799269245)*x[0]+(-0.829647203396)*x[1]
        s[3,3,2,0]=(-0.814979427693)*x[0]+(-0.0416092796813)*x[1]
        s[3,3,2,1]=(0.933434189127)*x[0]+(0.60794704936)*x[1]
        s[3,4,0,0]=(0.160280566478)*x[0]+(-0.0141616040181)*x[1]
        s[3,4,0,1]=(-1.34835766268)*x[0]+(0.178951225861)*x[1]
        s[3,4,1,0]=(1.67632814658)*x[0]+(1.31316096483)*x[1]
        s[3,4,1,1]=(-0.603403275376)*x[0]+(0.0607753526431)*x[1]
        s[3,4,2,0]=(-0.178440295473)*x[0]+(0.318102694371)*x[1]
        s[3,4,2,1]=(0.781285160749)*x[0]+(0.625651915981)*x[1]
        ref[0,0,0,0]=(0.609727033659)*x_ref[0]+(-0.901858940633)*x_ref[1]
        ref[0,0,0,1]=(-0.221259461243)*x_ref[0]+(0.351136340908)*x_ref[1]
        ref[0,0,1,0]=(0.497173734282)*x_ref[0]+(0.899142906899)*x_ref[1]
        ref[0,0,1,1]=(-1.55206649551)*x_ref[0]+(-0.680257384044)*x_ref[1]
        ref[0,0,2,0]=(-0.932033638033)*x_ref[0]+(-0.46503758126)*x_ref[1]
        ref[0,0,2,1]=(-0.0372650123525)*x_ref[0]+(0.0173714029098)*x_ref[1]
        ref[0,1,0,0]=(0.855963098465)*x_ref[0]+(0.760176320898)*x_ref[1]
        ref[0,1,0,1]=(1.37546120327)*x_ref[0]+(1.72916158331)*x_ref[1]
        ref[0,1,1,0]=(0.25469861613)*x_ref[0]+(0.71445716033)*x_ref[1]
        ref[0,1,1,1]=(0.198455659765)*x_ref[0]+(0.0410276829495)*x_ref[1]
        ref[0,1,2,0]=(-0.628080427667)*x_ref[0]+(-0.0197206025984)*x_ref[1]
        ref[0,1,2,1]=(1.26759029895)*x_ref[0]+(1.15914953253)*x_ref[1]
        ref[0,2,0,0]=(-1.43203053441)*x_ref[0]+(0.366301170069)*x_ref[1]
        ref[0,2,0,1]=(0.420527802229)*x_ref[0]+(-0.709209747415)*x_ref[1]
        ref[0,2,1,0]=(0.399042264829)*x_ref[0]+(0.0453132754731)*x_ref[1]
        ref[0,2,1,1]=(1.2074346062)*x_ref[0]+(-1.14163410992)*x_ref[1]
        ref[0,2,2,0]=(-0.00270984158569)*x_ref[0]+(-1.63216200182)*x_ref[1]
        ref[0,2,2,1]=(0.174276088847)*x_ref[0]+(-1.04789472231)*x_ref[1]
        ref[0,3,0,0]=(-0.47704729036)*x_ref[0]+(-1.2550637204)*x_ref[1]
        ref[0,3,0,1]=(0.348356873191)*x_ref[0]+(-0.796668578265)*x_ref[1]
        ref[0,3,1,0]=(-0.52724725097)*x_ref[0]+(1.26163123592)*x_ref[1]
        ref[0,3,1,1]=(-1.52681799261)*x_ref[0]+(0.936041931735)*x_ref[1]
        ref[0,3,2,0]=(1.33689673219)*x_ref[0]+(-1.34901214018)*x_ref[1]
        ref[0,3,2,1]=(1.57651286047)*x_ref[0]+(0.245413762864)*x_ref[1]
        ref[0,4,0,0]=(0.194500181498)*x_ref[0]+(-0.0231951844441)*x_ref[1]
        ref[0,4,0,1]=(-1.49372076697)*x_ref[0]+(-0.111369678181)*x_ref[1]
        ref[0,4,1,0]=(-0.74121646836)*x_ref[0]+(-0.817740156307)*x_ref[1]
        ref[0,4,1,1]=(-0.233643397123)*x_ref[0]+(0.672715068796)*x_ref[1]
        ref[0,4,2,0]=(1.05850947579)*x_ref[0]+(0.386484386191)*x_ref[1]
        ref[0,4,2,1]=(1.45105922963)*x_ref[0]+(-0.367458625996)*x_ref[1]
        ref[1,0,0,0]=(-0.391803788669)*x_ref[0]+(0.797592300662)*x_ref[1]
        ref[1,0,0,1]=(-0.205299072883)*x_ref[0]+(0.677427935558)*x_ref[1]
        ref[1,0,1,0]=(1.56225329031)*x_ref[0]+(1.17468092515)*x_ref[1]
        ref[1,0,1,1]=(1.4586005658)*x_ref[0]+(1.36481062943)*x_ref[1]
        ref[1,0,2,0]=(-1.53281881413)*x_ref[0]+(0.207028833974)*x_ref[1]
        ref[1,0,2,1]=(0.0469239691648)*x_ref[0]+(1.03547812705)*x_ref[1]
        ref[1,1,0,0]=(1.39614335897)*x_ref[0]+(0.82046489695)*x_ref[1]
        ref[1,1,0,1]=(-0.775462741637)*x_ref[0]+(-0.275081761646)*x_ref[1]
        ref[1,1,1,0]=(0.173353028608)*x_ref[0]+(-1.05620431816)*x_ref[1]
        ref[1,1,1,1]=(-0.464192870247)*x_ref[0]+(0.212204784206)*x_ref[1]
        ref[1,1,2,0]=(-0.383813531866)*x_ref[0]+(-0.0773759374234)*x_ref[1]
        ref[1,1,2,1]=(-1.07811385759)*x_ref[0]+(-0.457232330819)*x_ref[1]
        ref[1,2,0,0]=(-0.295869147576)*x_ref[0]+(0.166843915631)*x_ref[1]
        ref[1,2,0,1]=(0.571421034325)*x_ref[0]+(1.46722583245)*x_ref[1]
        ref[1,2,1,0]=(-0.34494944257)*x_ref[0]+(-0.286867052491)*x_ref[1]
        ref[1,2,1,1]=(1.43128885761)*x_ref[0]+(0.21277887035)*x_ref[1]
        ref[1,2,2,0]=(-1.19397187873)*x_ref[0]+(0.126397665227)*x_ref[1]
        ref[1,2,2,1]=(0.171368751954)*x_ref[0]+(-1.12100254625)*x_ref[1]
        ref[1,3,0,0]=(0.0975333077604)*x_ref[0]+(0.583524660188)*x_ref[1]
        ref[1,3,0,1]=(0.0271450955458)*x_ref[0]+(0.368978332867)*x_ref[1]
        ref[1,3,1,0]=(1.22806968486)*x_ref[0]+(-0.569932748468)*x_ref[1]
        ref[1,3,1,1]=(-0.691989839521)*x_ref[0]+(-0.829822182922)*x_ref[1]
        ref[1,3,2,0]=(0.743776534098)*x_ref[0]+(1.3668937127)*x_ref[1]
        ref[1,3,2,1]=(0.192206575204)*x_ref[0]+(0.96576702824)*x_ref[1]
        ref[1,4,0,0]=(0.149381460079)*x_ref[0]+(-0.987875538036)*x_ref[1]
        ref[1,4,0,1]=(-0.706855836683)*x_ref[0]+(0.0410884211631)*x_ref[1]
        ref[1,4,1,0]=(-0.222650274)*x_ref[0]+(0.59589242048)*x_ref[1]
        ref[1,4,1,1]=(0.541489186743)*x_ref[0]+(-1.22543996074)*x_ref[1]
        ref[1,4,2,0]=(0.630234824652)*x_ref[0]+(0.508436575179)*x_ref[1]
        ref[1,4,2,1]=(0.990062359294)*x_ref[0]+(-0.639884693407)*x_ref[1]
        ref[2,0,0,0]=(0.378887461733)*x_ref[0]+(1.71999809344)*x_ref[1]
        ref[2,0,0,1]=(0.182412683544)*x_ref[0]+(-0.401183243877)*x_ref[1]
        ref[2,0,1,0]=(0.325775290396)*x_ref[0]+(-0.656088786126)*x_ref[1]
        ref[2,0,1,1]=(0.316355229742)*x_ref[0]+(1.54087545195)*x_ref[1]
        ref[2,0,2,0]=(1.14203881183)*x_ref[0]+(0.703207191364)*x_ref[1]
        ref[2,0,2,1]=(-1.05211790226)*x_ref[0]+(1.68596483036)*x_ref[1]
        ref[2,1,0,0]=(0.704109387934)*x_ref[0]+(-0.589003921574)*x_ref[1]
        ref[2,1,0,1]=(-0.382489922028)*x_ref[0]+(0.387647642942)*x_ref[1]
        ref[2,1,1,0]=(-0.0343434958891)*x_ref[0]+(-0.119176404279)*x_ref[1]
        ref[2,1,1,1]=(0.567853527929)*x_ref[0]+(1.32523261969)*x_ref[1]
        ref[2,1,2,0]=(1.03496000653)*x_ref[0]+(0.110203591459)*x_ref[1]
        ref[2,1,2,1]=(0.560149997806)*x_ref[0]+(-1.62893742873)*x_ref[1]
        ref[2,2,0,0]=(1.1801792312)*x_ref[0]+(1.1912385935)*x_ref[1]
        ref[2,2,0,1]=(0.936932817634)*x_ref[0]+(-1.49082021974)*x_ref[1]
        ref[2,2,1,0]=(-0.305582892069)*x_ref[0]+(-0.267235698563)*x_ref[1]
        ref[2,2,1,1]=(0.107449730184)*x_ref[0]+(-0.325267993615)*x_ref[1]
        ref[2,2,2,0]=(-1.08118986347)*x_ref[0]+(0.144190727334)*x_ref[1]
        ref[2,2,2,1]=(1.44755060146)*x_ref[0]+(-0.127612338817)*x_ref[1]
        ref[2,3,0,0]=(-0.666943112999)*x_ref[0]+(-0.780402371989)*x_ref[1]
        ref[2,3,0,1]=(0.724206717771)*x_ref[0]+(-0.523593908278)*x_ref[1]
        ref[2,3,1,0]=(1.86101197199)*x_ref[0]+(-0.710894911302)*x_ref[1]
        ref[2,3,1,1]=(-1.74409232023)*x_ref[0]+(0.407821816271)*x_ref[1]
        ref[2,3,2,0]=(0.646304172575)*x_ref[0]+(0.0132628778543)*x_ref[1]
        ref[2,3,2,1]=(-0.746041438673)*x_ref[0]+(0.516304872741)*x_ref[1]
        ref[2,4,0,0]=(0.0326900201283)*x_ref[0]+(-0.533021156401)*x_ref[1]
        ref[2,4,0,1]=(0.088598956981)*x_ref[0]+(0.764307970269)*x_ref[1]
        ref[2,4,1,0]=(-0.749553477756)*x_ref[0]+(0.379646792939)*x_ref[1]
        ref[2,4,1,1]=(-0.243635321378)*x_ref[0]+(-0.470022600261)*x_ref[1]
        ref[2,4,2,0]=(0.586398757247)*x_ref[0]+(-0.116982897683)*x_ref[1]
        ref[2,4,2,1]=(-0.989012121879)*x_ref[0]+(-1.25006535257)*x_ref[1]
        ref[3,0,0,0]=(1.25228306081)*x_ref[0]+(-0.0932717035896)*x_ref[1]
        ref[3,0,0,1]=(1.5975516771)*x_ref[0]+(-0.177733286111)*x_ref[1]
        ref[3,0,1,0]=(-0.659910070911)*x_ref[0]+(-0.0999498944812)*x_ref[1]
        ref[3,0,1,1]=(-0.299870273662)*x_ref[0]+(-1.22977656744)*x_ref[1]
        ref[3,0,2,0]=(-0.158211581143)*x_ref[0]+(-0.141421380503)*x_ref[1]
        ref[3,0,2,1]=(-1.48226452788)*x_ref[0]+(-0.731264943309)*x_ref[1]
        ref[3,1,0,0]=(-1.64589827853)*x_ref[0]+(1.19692573998)*x_ref[1]
        ref[3,1,0,1]=(0.361329655357)*x_ref[0]+(0.234310482171)*x_ref[1]
        ref[3,1,1,0]=(-1.14077649516)*x_ref[0]+(0.132657084065)*x_ref[1]
        ref[3,1,1,1]=(1.41067975663)*x_ref[0]+(0.818243117884)*x_ref[1]
        ref[3,1,2,0]=(-1.53137884342)*x_ref[0]+(-0.0127805333093)*x_ref[1]
        ref[3,1,2,1]=(0.240455755486)*x_ref[0]+(-1.02361753893)*x_ref[1]
        ref[3,2,0,0]=(-0.334803473072)*x_ref[0]+(-0.807452510423)*x_ref[1]
        ref[3,2,0,1]=(0.617324536974)*x_ref[0]+(-0.733163471774)*x_ref[1]
        ref[3,2,1,0]=(-0.634580050897)*x_ref[0]+(0.362106278168)*x_ref[1]
        ref[3,2,1,1]=(-0.21525531818)*x_ref[0]+(0.982902540898)*x_ref[1]
        ref[3,2,2,0]=(-0.590682803195)*x_ref[0]+(-1.27537880632)*x_ref[1]
        ref[3,2,2,1]=(0.142578313438)*x_ref[0]+(0.212341687291)*x_ref[1]
        ref[3,3,0,0]=(-0.758345317216)*x_ref[0]+(-0.971914631359)*x_ref[1]
        ref[3,3,0,1]=(0.569463909234)*x_ref[0]+(0.498848917338)*x_ref[1]
        ref[3,3,1,0]=(-0.771456575632)*x_ref[0]+(-0.130456072944)*x_ref[1]
        ref[3,3,1,1]=(0.811799269245)*x_ref[0]+(-0.829647203396)*x_ref[1]
        ref[3,3,2,0]=(-0.814979427693)*x_ref[0]+(-0.0416092796813)*x_ref[1]
        ref[3,3,2,1]=(0.933434189127)*x_ref[0]+(0.60794704936)*x_ref[1]
        ref[3,4,0,0]=(0.160280566478)*x_ref[0]+(-0.0141616040181)*x_ref[1]
        ref[3,4,0,1]=(-1.34835766268)*x_ref[0]+(0.178951225861)*x_ref[1]
        ref[3,4,1,0]=(1.67632814658)*x_ref[0]+(1.31316096483)*x_ref[1]
        ref[3,4,1,1]=(-0.603403275376)*x_ref[0]+(0.0607753526431)*x_ref[1]
        ref[3,4,2,0]=(-0.178440295473)*x_ref[0]+(0.318102694371)*x_ref[1]
        ref[3,4,2,1]=(0.781285160749)*x_ref[0]+(0.625651915981)*x_ref[1]
      else:
        s[0,0,0,0]=(0.847845815599)*x[0]+(-0.610234211476)*x[1]+(-0.627625510082)*x[2]
        s[0,0,0,1]=(0.0126184651041)*x[0]+(-0.0651294063564)*x[1]+(-0.274615935201)*x[2]
        s[0,0,1,0]=(-0.545925417969)*x[0]+(0.421929302201)*x[1]+(-0.910418625383)*x[2]
        s[0,0,1,1]=(1.58754252937)*x[0]+(0.110111591047)*x[1]+(-1.64780563736)*x[2]
        s[0,0,2,0]=(-0.510331643992)*x[0]+(0.827685326542)*x[1]+(1.31447962917)*x[2]
        s[0,0,2,1]=(-1.29575211142)*x[0]+(-0.318096803401)*x[1]+(-0.274660613777)*x[2]
        s[0,1,0,0]=(0.77444944835)*x[0]+(-1.0893411122)*x[1]+(0.331702744143)*x[2]
        s[0,1,0,1]=(0.624693916432)*x[0]+(-0.306552086758)*x[1]+(-0.845524417807)*x[2]
        s[0,1,1,0]=(0.0219492668609)*x[0]+(-1.13712865204)*x[1]+(-0.193051406533)*x[2]
        s[0,1,1,1]=(-1.32696889246)*x[0]+(1.63857177904)*x[1]+(0.194095476936)*x[2]
        s[0,1,2,0]=(1.19169978642)*x[0]+(-1.88274910505)*x[1]+(0.321438561021)*x[2]
        s[0,1,2,1]=(-0.765748081809)*x[0]+(-0.163422665563)*x[1]+(0.256721885141)*x[2]
        s[0,2,0,0]=(-0.863281319991)*x[0]+(-0.021828308413)*x[1]+(0.868312889989)*x[2]
        s[0,2,0,1]=(-0.57479341478)*x[0]+(0.404861121887)*x[1]+(-0.274037007699)*x[2]
        s[0,2,1,0]=(-0.529171478131)*x[0]+(0.836001872018)*x[1]+(-1.12226189724)*x[2]
        s[0,2,1,1]=(-0.821929160411)*x[0]+(0.0466762602824)*x[1]+(-0.0514331500765)*x[2]
        s[0,2,2,0]=(-0.605940618544)*x[0]+(-1.54379919132)*x[1]+(1.63243021269)*x[2]
        s[0,2,2,1]=(-1.49695860655)*x[0]+(-0.601345463475)*x[1]+(-0.753443592162)*x[2]
        s[0,3,0,0]=(1.33514081849)*x[0]+(0.955849232345)*x[1]+(-0.872221354599)*x[2]
        s[0,3,0,1]=(0.939731890901)*x[0]+(-1.16116012822)*x[1]+(-0.601602002743)*x[2]
        s[0,3,1,0]=(-0.675545889094)*x[0]+(0.736755090077)*x[1]+(-0.437370345974)*x[2]
        s[0,3,1,1]=(-1.17899966213)*x[0]+(0.422125349488)*x[1]+(-0.815649229407)*x[2]
        s[0,3,2,0]=(0.0773790438387)*x[0]+(0.148378160687)*x[1]+(-1.0156040466)*x[2]
        s[0,3,2,1]=(0.519422925349)*x[0]+(0.189993892084)*x[1]+(1.59654399673)*x[2]
        s[0,4,0,0]=(-0.0593352727817)*x[0]+(0.0865384232804)*x[1]+(-0.316122229956)*x[2]
        s[0,4,0,1]=(0.935220504817)*x[0]+(0.868143343952)*x[1]+(-0.470286174039)*x[2]
        s[0,4,1,0]=(0.763506086212)*x[0]+(-0.0756795330833)*x[1]+(1.06812672934)*x[2]
        s[0,4,1,1]=(-0.446580256351)*x[0]+(0.653986880233)*x[1]+(0.634364696873)*x[2]
        s[0,4,2,0]=(-0.72871248403)*x[0]+(0.0839150289011)*x[1]+(0.178899292496)*x[2]
        s[0,4,2,1]=(1.30808288445)*x[0]+(0.548931211195)*x[1]+(-0.971585668771)*x[2]
        s[1,0,0,0]=(-0.737171903787)*x[0]+(-0.137110160891)*x[1]+(1.24817883553)*x[2]
        s[1,0,0,1]=(0.589824248199)*x[0]+(-0.191688810075)*x[1]+(1.33820051734)*x[2]
        s[1,0,1,0]=(-0.598063103136)*x[0]+(0.347705502684)*x[1]+(0.490747627936)*x[2]
        s[1,0,1,1]=(0.287829427303)*x[0]+(-0.853875346674)*x[1]+(-0.227722773909)*x[2]
        s[1,0,2,0]=(-1.22183236343)*x[0]+(-0.162406713796)*x[1]+(-0.436433884201)*x[2]
        s[1,0,2,1]=(0.929372688401)*x[0]+(0.469931776805)*x[1]+(0.739110972137)*x[2]
        s[1,1,0,0]=(0.391824586)*x[0]+(-0.551498198462)*x[1]+(-0.588935046796)*x[2]
        s[1,1,0,1]=(0.849805745827)*x[0]+(0.557121315313)*x[1]+(1.80284061615)*x[2]
        s[1,1,1,0]=(-1.62270432051)*x[0]+(-1.55943636923)*x[1]+(0.629204169009)*x[2]
        s[1,1,1,1]=(-0.191271118514)*x[0]+(-0.454718665581)*x[1]+(0.405403171722)*x[2]
        s[1,1,2,0]=(0.461750123507)*x[0]+(0.537594040674)*x[1]+(-0.500046587877)*x[2]
        s[1,1,2,1]=(1.07800193455)*x[0]+(0.0757746249706)*x[1]+(0.358223454759)*x[2]
        s[1,2,0,0]=(-1.31360669142)*x[0]+(0.908413426737)*x[1]+(0.986904165818)*x[2]
        s[1,2,0,1]=(-0.250344247716)*x[0]+(0.596746427887)*x[1]+(0.846846599186)*x[2]
        s[1,2,1,0]=(1.65761753214)*x[0]+(0.0969407749043)*x[1]+(0.197913502281)*x[2]
        s[1,2,1,1]=(0.808454446977)*x[0]+(-0.107176416129)*x[1]+(-0.115562549181)*x[2]
        s[1,2,2,0]=(-0.410108442793)*x[0]+(0.250496842214)*x[1]+(0.403806076052)*x[2]
        s[1,2,2,1]=(1.58110488679)*x[0]+(-0.171583112102)*x[1]+(0.441813895222)*x[2]
        s[1,3,0,0]=(-0.519973841295)*x[0]+(0.14862500493)*x[1]+(-0.0257159960311)*x[2]
        s[1,3,0,1]=(-0.710423889723)*x[0]+(0.0941990549739)*x[1]+(-0.147877062943)*x[2]
        s[1,3,1,0]=(-0.19917762038)*x[0]+(-1.06873791711)*x[1]+(1.37688275432)*x[2]
        s[1,3,1,1]=(-0.570477493665)*x[0]+(0.0705814270652)*x[1]+(-1.10186231057)*x[2]
        s[1,3,2,0]=(-1.04891017002)*x[0]+(-0.136784107636)*x[1]+(-1.65953016693)*x[2]
        s[1,3,2,1]=(-0.674805716398)*x[0]+(-0.211803508128)*x[1]+(-0.116463503974)*x[2]
        s[1,4,0,0]=(0.269217220297)*x[0]+(0.810722194772)*x[1]+(1.60977315112)*x[2]
        s[1,4,0,1]=(-0.0213286766746)*x[0]+(1.0974274553)*x[1]+(0.275633207025)*x[2]
        s[1,4,1,0]=(-0.601230245409)*x[0]+(-0.756418955933)*x[1]+(-0.46194303915)*x[2]
        s[1,4,1,1]=(-0.0907667209143)*x[0]+(-1.86623564673)*x[1]+(0.552729474701)*x[2]
        s[1,4,2,0]=(-1.09694541489)*x[0]+(-0.496193881555)*x[1]+(-0.0126133249168)*x[2]
        s[1,4,2,1]=(-0.648959246633)*x[0]+(-0.60655325639)*x[1]+(0.438258051102)*x[2]
        s[2,0,0,0]=(-0.100512547019)*x[0]+(-1.37291269079)*x[1]+(1.63084319876)*x[2]
        s[2,0,0,1]=(-0.50510326916)*x[0]+(0.308799737351)*x[1]+(1.63506761477)*x[2]
        s[2,0,1,0]=(0.708408519723)*x[0]+(-0.0759833841695)*x[1]+(-0.980974095374)*x[2]
        s[2,0,1,1]=(-1.59727324143)*x[0]+(0.995250195962)*x[1]+(0.169527316083)*x[2]
        s[2,0,2,0]=(0.766935817054)*x[0]+(0.455564399031)*x[1]+(0.605772951782)*x[2]
        s[2,0,2,1]=(-0.820755214894)*x[0]+(-0.43359371038)*x[1]+(-0.0942398466964)*x[2]
        s[2,1,0,0]=(0.390476682801)*x[0]+(1.11627447827)*x[1]+(0.670843405466)*x[2]
        s[2,1,0,1]=(-0.433594870483)*x[0]+(-0.940710004402)*x[1]+(1.10756041487)*x[2]
        s[2,1,1,0]=(-1.21880018868)*x[0]+(0.380422307344)*x[1]+(-0.864017731789)*x[2]
        s[2,1,1,1]=(-0.194168346974)*x[0]+(0.324183983028)*x[1]+(0.741456579501)*x[2]
        s[2,1,2,0]=(-0.134211025912)*x[0]+(0.497311041207)*x[1]+(0.33014196376)*x[2]
        s[2,1,2,1]=(-1.3501981937)*x[0]+(1.67601514859)*x[1]+(-1.60084725544)*x[2]
        s[2,2,0,0]=(-0.8367457236)*x[0]+(-1.15628524562)*x[1]+(0.342983125417)*x[2]
        s[2,2,0,1]=(0.956733778283)*x[0]+(-0.404880336577)*x[1]+(-0.350927185105)*x[2]
        s[2,2,1,0]=(0.854842449281)*x[0]+(-0.185001304765)*x[1]+(1.15245157168)*x[2]
        s[2,2,1,1]=(0.00584831242623)*x[0]+(-0.562950548984)*x[1]+(0.785766513431)*x[2]
        s[2,2,2,0]=(0.272759945164)*x[0]+(-0.568207376307)*x[1]+(-0.155997758854)*x[2]
        s[2,2,2,1]=(-0.105324133175)*x[0]+(1.11807221711)*x[1]+(0.122751975298)*x[2]
        s[2,3,0,0]=(0.685932400265)*x[0]+(-0.666847052302)*x[1]+(0.596321369477)*x[2]
        s[2,3,0,1]=(0.335067153518)*x[0]+(0.268453607431)*x[1]+(0.247881738828)*x[2]
        s[2,3,1,0]=(-1.89084909179)*x[0]+(-0.529751828744)*x[1]+(-0.959358202828)*x[2]
        s[2,3,1,1]=(1.06725689989)*x[0]+(-0.709878517637)*x[1]+(0.968886366535)*x[2]
        s[2,3,2,0]=(-1.63466018652)*x[0]+(0.517481248484)*x[1]+(0.109232845638)*x[2]
        s[2,3,2,1]=(0.0716425266603)*x[0]+(-0.58040972115)*x[1]+(0.346948738894)*x[2]
        s[2,4,0,0]=(0.193599854213)*x[0]+(-1.52744079327)*x[1]+(-0.0377038338183)*x[2]
        s[2,4,0,1]=(-0.0296855161658)*x[0]+(0.266976994363)*x[1]+(-0.832771979863)*x[2]
        s[2,4,1,0]=(-0.885217283694)*x[0]+(0.657425725373)*x[1]+(0.385584782531)*x[2]
        s[2,4,1,1]=(-0.249912733602)*x[0]+(0.156704487203)*x[1]+(0.000393408500501)*x[2]
        s[2,4,2,0]=(-0.083303562971)*x[0]+(0.748009334975)*x[1]+(-0.411323946112)*x[2]
        s[2,4,2,1]=(0.143782087293)*x[0]+(-1.86497054401)*x[1]+(-0.398161751889)*x[2]
        s[3,0,0,0]=(-0.531041035506)*x[0]+(0.998737023878)*x[1]+(-0.719326852701)*x[2]
        s[3,0,0,1]=(-1.10320774278)*x[0]+(-1.27795876669)*x[1]+(0.57451462957)*x[2]
        s[3,0,1,0]=(-0.319275578502)*x[0]+(-1.51721910203)*x[1]+(-0.677470578443)*x[2]
        s[3,0,1,1]=(0.7584807917)*x[0]+(0.0523998746885)*x[1]+(0.502182688228)*x[2]
        s[3,0,2,0]=(-0.905809892125)*x[0]+(0.479751975382)*x[1]+(-0.0778539387977)*x[2]
        s[3,0,2,1]=(-0.724891388357)*x[0]+(0.941493554841)*x[1]+(-1.51514565838)*x[2]
        s[3,1,0,0]=(0.815118793475)*x[0]+(0.0503368907581)*x[1]+(-0.502091253213)*x[2]
        s[3,1,0,1]=(-1.50859117948)*x[0]+(0.31279517213)*x[1]+(-0.339295035594)*x[2]
        s[3,1,1,0]=(0.246825549192)*x[0]+(0.687251908372)*x[1]+(0.133971781776)*x[2]
        s[3,1,1,1]=(0.470105585328)*x[0]+(0.382971101979)*x[1]+(-0.469397699886)*x[2]
        s[3,1,2,0]=(-1.45842815542)*x[0]+(-1.00979255513)*x[1]+(1.45386034482)*x[2]
        s[3,1,2,1]=(0.172130164577)*x[0]+(1.25034739991)*x[1]+(-1.15953423359)*x[2]
        s[3,2,0,0]=(-0.58261393055)*x[0]+(0.730381032663)*x[1]+(1.29377076249)*x[2]
        s[3,2,0,1]=(0.696715362351)*x[0]+(-1.37598351546)*x[1]+(-1.26545085967)*x[2]
        s[3,2,1,0]=(-0.0511282610422)*x[0]+(0.58041679118)*x[1]+(-0.0810357412151)*x[2]
        s[3,2,1,1]=(-0.750057228803)*x[0]+(-0.750703125074)*x[1]+(-1.21748956536)*x[2]
        s[3,2,2,0]=(-0.455746666635)*x[0]+(-0.720020627516)*x[1]+(-0.0911281889971)*x[2]
        s[3,2,2,1]=(-0.0681894163236)*x[0]+(0.101592029278)*x[1]+(1.10954864406)*x[2]
        s[3,3,0,0]=(0.433898443791)*x[0]+(0.901180030496)*x[1]+(0.795641263874)*x[2]
        s[3,3,0,1]=(-0.213306810738)*x[0]+(-0.852539239286)*x[1]+(-1.25905416259)*x[2]
        s[3,3,1,0]=(-0.668307384473)*x[0]+(-1.37753879862)*x[1]+(-0.593731800903)*x[2]
        s[3,3,1,1]=(0.815464294514)*x[0]+(0.835389640785)*x[1]+(0.933029468258)*x[2]
        s[3,3,2,0]=(0.565194274444)*x[0]+(0.584119707001)*x[1]+(-0.53812189952)*x[2]
        s[3,3,2,1]=(-0.903604209314)*x[0]+(0.370887007483)*x[1]+(-0.548135915608)*x[2]
        s[3,4,0,0]=(-0.819290139492)*x[0]+(-0.256800498012)*x[1]+(0.350363336935)*x[2]
        s[3,4,0,1]=(0.863170182723)*x[0]+(0.0101766489337)*x[1]+(1.34009635664)*x[2]
        s[3,4,1,0]=(1.03350734136)*x[0]+(1.37672500233)*x[1]+(1.35134434663)*x[2]
        s[3,4,1,1]=(1.4883144839)*x[0]+(0.410688244039)*x[1]+(0.759267998436)*x[2]
        s[3,4,2,0]=(0.373190738096)*x[0]+(1.22136515279)*x[1]+(-0.0365030690962)*x[2]
        s[3,4,2,1]=(-0.607437402737)*x[0]+(0.750130654364)*x[1]+(0.084778884789)*x[2]
        ref[0,0,0,0]=(0.847845815599)*x_ref[0]+(-0.610234211476)*x_ref[1]+(-0.627625510082)*x_ref[2]
        ref[0,0,0,1]=(0.0126184651041)*x_ref[0]+(-0.0651294063564)*x_ref[1]+(-0.274615935201)*x_ref[2]
        ref[0,0,1,0]=(-0.545925417969)*x_ref[0]+(0.421929302201)*x_ref[1]+(-0.910418625383)*x_ref[2]
        ref[0,0,1,1]=(1.58754252937)*x_ref[0]+(0.110111591047)*x_ref[1]+(-1.64780563736)*x_ref[2]
        ref[0,0,2,0]=(-0.510331643992)*x_ref[0]+(0.827685326542)*x_ref[1]+(1.31447962917)*x_ref[2]
        ref[0,0,2,1]=(-1.29575211142)*x_ref[0]+(-0.318096803401)*x_ref[1]+(-0.274660613777)*x_ref[2]
        ref[0,1,0,0]=(0.77444944835)*x_ref[0]+(-1.0893411122)*x_ref[1]+(0.331702744143)*x_ref[2]
        ref[0,1,0,1]=(0.624693916432)*x_ref[0]+(-0.306552086758)*x_ref[1]+(-0.845524417807)*x_ref[2]
        ref[0,1,1,0]=(0.0219492668609)*x_ref[0]+(-1.13712865204)*x_ref[1]+(-0.193051406533)*x_ref[2]
        ref[0,1,1,1]=(-1.32696889246)*x_ref[0]+(1.63857177904)*x_ref[1]+(0.194095476936)*x_ref[2]
        ref[0,1,2,0]=(1.19169978642)*x_ref[0]+(-1.88274910505)*x_ref[1]+(0.321438561021)*x_ref[2]
        ref[0,1,2,1]=(-0.765748081809)*x_ref[0]+(-0.163422665563)*x_ref[1]+(0.256721885141)*x_ref[2]
        ref[0,2,0,0]=(-0.863281319991)*x_ref[0]+(-0.021828308413)*x_ref[1]+(0.868312889989)*x_ref[2]
        ref[0,2,0,1]=(-0.57479341478)*x_ref[0]+(0.404861121887)*x_ref[1]+(-0.274037007699)*x_ref[2]
        ref[0,2,1,0]=(-0.529171478131)*x_ref[0]+(0.836001872018)*x_ref[1]+(-1.12226189724)*x_ref[2]
        ref[0,2,1,1]=(-0.821929160411)*x_ref[0]+(0.0466762602824)*x_ref[1]+(-0.0514331500765)*x_ref[2]
        ref[0,2,2,0]=(-0.605940618544)*x_ref[0]+(-1.54379919132)*x_ref[1]+(1.63243021269)*x_ref[2]
        ref[0,2,2,1]=(-1.49695860655)*x_ref[0]+(-0.601345463475)*x_ref[1]+(-0.753443592162)*x_ref[2]
        ref[0,3,0,0]=(1.33514081849)*x_ref[0]+(0.955849232345)*x_ref[1]+(-0.872221354599)*x_ref[2]
        ref[0,3,0,1]=(0.939731890901)*x_ref[0]+(-1.16116012822)*x_ref[1]+(-0.601602002743)*x_ref[2]
        ref[0,3,1,0]=(-0.675545889094)*x_ref[0]+(0.736755090077)*x_ref[1]+(-0.437370345974)*x_ref[2]
        ref[0,3,1,1]=(-1.17899966213)*x_ref[0]+(0.422125349488)*x_ref[1]+(-0.815649229407)*x_ref[2]
        ref[0,3,2,0]=(0.0773790438387)*x_ref[0]+(0.148378160687)*x_ref[1]+(-1.0156040466)*x_ref[2]
        ref[0,3,2,1]=(0.519422925349)*x_ref[0]+(0.189993892084)*x_ref[1]+(1.59654399673)*x_ref[2]
        ref[0,4,0,0]=(-0.0593352727817)*x_ref[0]+(0.0865384232804)*x_ref[1]+(-0.316122229956)*x_ref[2]
        ref[0,4,0,1]=(0.935220504817)*x_ref[0]+(0.868143343952)*x_ref[1]+(-0.470286174039)*x_ref[2]
        ref[0,4,1,0]=(0.763506086212)*x_ref[0]+(-0.0756795330833)*x_ref[1]+(1.06812672934)*x_ref[2]
        ref[0,4,1,1]=(-0.446580256351)*x_ref[0]+(0.653986880233)*x_ref[1]+(0.634364696873)*x_ref[2]
        ref[0,4,2,0]=(-0.72871248403)*x_ref[0]+(0.0839150289011)*x_ref[1]+(0.178899292496)*x_ref[2]
        ref[0,4,2,1]=(1.30808288445)*x_ref[0]+(0.548931211195)*x_ref[1]+(-0.971585668771)*x_ref[2]
        ref[1,0,0,0]=(-0.737171903787)*x_ref[0]+(-0.137110160891)*x_ref[1]+(1.24817883553)*x_ref[2]
        ref[1,0,0,1]=(0.589824248199)*x_ref[0]+(-0.191688810075)*x_ref[1]+(1.33820051734)*x_ref[2]
        ref[1,0,1,0]=(-0.598063103136)*x_ref[0]+(0.347705502684)*x_ref[1]+(0.490747627936)*x_ref[2]
        ref[1,0,1,1]=(0.287829427303)*x_ref[0]+(-0.853875346674)*x_ref[1]+(-0.227722773909)*x_ref[2]
        ref[1,0,2,0]=(-1.22183236343)*x_ref[0]+(-0.162406713796)*x_ref[1]+(-0.436433884201)*x_ref[2]
        ref[1,0,2,1]=(0.929372688401)*x_ref[0]+(0.469931776805)*x_ref[1]+(0.739110972137)*x_ref[2]
        ref[1,1,0,0]=(0.391824586)*x_ref[0]+(-0.551498198462)*x_ref[1]+(-0.588935046796)*x_ref[2]
        ref[1,1,0,1]=(0.849805745827)*x_ref[0]+(0.557121315313)*x_ref[1]+(1.80284061615)*x_ref[2]
        ref[1,1,1,0]=(-1.62270432051)*x_ref[0]+(-1.55943636923)*x_ref[1]+(0.629204169009)*x_ref[2]
        ref[1,1,1,1]=(-0.191271118514)*x_ref[0]+(-0.454718665581)*x_ref[1]+(0.405403171722)*x_ref[2]
        ref[1,1,2,0]=(0.461750123507)*x_ref[0]+(0.537594040674)*x_ref[1]+(-0.500046587877)*x_ref[2]
        ref[1,1,2,1]=(1.07800193455)*x_ref[0]+(0.0757746249706)*x_ref[1]+(0.358223454759)*x_ref[2]
        ref[1,2,0,0]=(-1.31360669142)*x_ref[0]+(0.908413426737)*x_ref[1]+(0.986904165818)*x_ref[2]
        ref[1,2,0,1]=(-0.250344247716)*x_ref[0]+(0.596746427887)*x_ref[1]+(0.846846599186)*x_ref[2]
        ref[1,2,1,0]=(1.65761753214)*x_ref[0]+(0.0969407749043)*x_ref[1]+(0.197913502281)*x_ref[2]
        ref[1,2,1,1]=(0.808454446977)*x_ref[0]+(-0.107176416129)*x_ref[1]+(-0.115562549181)*x_ref[2]
        ref[1,2,2,0]=(-0.410108442793)*x_ref[0]+(0.250496842214)*x_ref[1]+(0.403806076052)*x_ref[2]
        ref[1,2,2,1]=(1.58110488679)*x_ref[0]+(-0.171583112102)*x_ref[1]+(0.441813895222)*x_ref[2]
        ref[1,3,0,0]=(-0.519973841295)*x_ref[0]+(0.14862500493)*x_ref[1]+(-0.0257159960311)*x_ref[2]
        ref[1,3,0,1]=(-0.710423889723)*x_ref[0]+(0.0941990549739)*x_ref[1]+(-0.147877062943)*x_ref[2]
        ref[1,3,1,0]=(-0.19917762038)*x_ref[0]+(-1.06873791711)*x_ref[1]+(1.37688275432)*x_ref[2]
        ref[1,3,1,1]=(-0.570477493665)*x_ref[0]+(0.0705814270652)*x_ref[1]+(-1.10186231057)*x_ref[2]
        ref[1,3,2,0]=(-1.04891017002)*x_ref[0]+(-0.136784107636)*x_ref[1]+(-1.65953016693)*x_ref[2]
        ref[1,3,2,1]=(-0.674805716398)*x_ref[0]+(-0.211803508128)*x_ref[1]+(-0.116463503974)*x_ref[2]
        ref[1,4,0,0]=(0.269217220297)*x_ref[0]+(0.810722194772)*x_ref[1]+(1.60977315112)*x_ref[2]
        ref[1,4,0,1]=(-0.0213286766746)*x_ref[0]+(1.0974274553)*x_ref[1]+(0.275633207025)*x_ref[2]
        ref[1,4,1,0]=(-0.601230245409)*x_ref[0]+(-0.756418955933)*x_ref[1]+(-0.46194303915)*x_ref[2]
        ref[1,4,1,1]=(-0.0907667209143)*x_ref[0]+(-1.86623564673)*x_ref[1]+(0.552729474701)*x_ref[2]
        ref[1,4,2,0]=(-1.09694541489)*x_ref[0]+(-0.496193881555)*x_ref[1]+(-0.0126133249168)*x_ref[2]
        ref[1,4,2,1]=(-0.648959246633)*x_ref[0]+(-0.60655325639)*x_ref[1]+(0.438258051102)*x_ref[2]
        ref[2,0,0,0]=(-0.100512547019)*x_ref[0]+(-1.37291269079)*x_ref[1]+(1.63084319876)*x_ref[2]
        ref[2,0,0,1]=(-0.50510326916)*x_ref[0]+(0.308799737351)*x_ref[1]+(1.63506761477)*x_ref[2]
        ref[2,0,1,0]=(0.708408519723)*x_ref[0]+(-0.0759833841695)*x_ref[1]+(-0.980974095374)*x_ref[2]
        ref[2,0,1,1]=(-1.59727324143)*x_ref[0]+(0.995250195962)*x_ref[1]+(0.169527316083)*x_ref[2]
        ref[2,0,2,0]=(0.766935817054)*x_ref[0]+(0.455564399031)*x_ref[1]+(0.605772951782)*x_ref[2]
        ref[2,0,2,1]=(-0.820755214894)*x_ref[0]+(-0.43359371038)*x_ref[1]+(-0.0942398466964)*x_ref[2]
        ref[2,1,0,0]=(0.390476682801)*x_ref[0]+(1.11627447827)*x_ref[1]+(0.670843405466)*x_ref[2]
        ref[2,1,0,1]=(-0.433594870483)*x_ref[0]+(-0.940710004402)*x_ref[1]+(1.10756041487)*x_ref[2]
        ref[2,1,1,0]=(-1.21880018868)*x_ref[0]+(0.380422307344)*x_ref[1]+(-0.864017731789)*x_ref[2]
        ref[2,1,1,1]=(-0.194168346974)*x_ref[0]+(0.324183983028)*x_ref[1]+(0.741456579501)*x_ref[2]
        ref[2,1,2,0]=(-0.134211025912)*x_ref[0]+(0.497311041207)*x_ref[1]+(0.33014196376)*x_ref[2]
        ref[2,1,2,1]=(-1.3501981937)*x_ref[0]+(1.67601514859)*x_ref[1]+(-1.60084725544)*x_ref[2]
        ref[2,2,0,0]=(-0.8367457236)*x_ref[0]+(-1.15628524562)*x_ref[1]+(0.342983125417)*x_ref[2]
        ref[2,2,0,1]=(0.956733778283)*x_ref[0]+(-0.404880336577)*x_ref[1]+(-0.350927185105)*x_ref[2]
        ref[2,2,1,0]=(0.854842449281)*x_ref[0]+(-0.185001304765)*x_ref[1]+(1.15245157168)*x_ref[2]
        ref[2,2,1,1]=(0.00584831242623)*x_ref[0]+(-0.562950548984)*x_ref[1]+(0.785766513431)*x_ref[2]
        ref[2,2,2,0]=(0.272759945164)*x_ref[0]+(-0.568207376307)*x_ref[1]+(-0.155997758854)*x_ref[2]
        ref[2,2,2,1]=(-0.105324133175)*x_ref[0]+(1.11807221711)*x_ref[1]+(0.122751975298)*x_ref[2]
        ref[2,3,0,0]=(0.685932400265)*x_ref[0]+(-0.666847052302)*x_ref[1]+(0.596321369477)*x_ref[2]
        ref[2,3,0,1]=(0.335067153518)*x_ref[0]+(0.268453607431)*x_ref[1]+(0.247881738828)*x_ref[2]
        ref[2,3,1,0]=(-1.89084909179)*x_ref[0]+(-0.529751828744)*x_ref[1]+(-0.959358202828)*x_ref[2]
        ref[2,3,1,1]=(1.06725689989)*x_ref[0]+(-0.709878517637)*x_ref[1]+(0.968886366535)*x_ref[2]
        ref[2,3,2,0]=(-1.63466018652)*x_ref[0]+(0.517481248484)*x_ref[1]+(0.109232845638)*x_ref[2]
        ref[2,3,2,1]=(0.0716425266603)*x_ref[0]+(-0.58040972115)*x_ref[1]+(0.346948738894)*x_ref[2]
        ref[2,4,0,0]=(0.193599854213)*x_ref[0]+(-1.52744079327)*x_ref[1]+(-0.0377038338183)*x_ref[2]
        ref[2,4,0,1]=(-0.0296855161658)*x_ref[0]+(0.266976994363)*x_ref[1]+(-0.832771979863)*x_ref[2]
        ref[2,4,1,0]=(-0.885217283694)*x_ref[0]+(0.657425725373)*x_ref[1]+(0.385584782531)*x_ref[2]
        ref[2,4,1,1]=(-0.249912733602)*x_ref[0]+(0.156704487203)*x_ref[1]+(0.000393408500501)*x_ref[2]
        ref[2,4,2,0]=(-0.083303562971)*x_ref[0]+(0.748009334975)*x_ref[1]+(-0.411323946112)*x_ref[2]
        ref[2,4,2,1]=(0.143782087293)*x_ref[0]+(-1.86497054401)*x_ref[1]+(-0.398161751889)*x_ref[2]
        ref[3,0,0,0]=(-0.531041035506)*x_ref[0]+(0.998737023878)*x_ref[1]+(-0.719326852701)*x_ref[2]
        ref[3,0,0,1]=(-1.10320774278)*x_ref[0]+(-1.27795876669)*x_ref[1]+(0.57451462957)*x_ref[2]
        ref[3,0,1,0]=(-0.319275578502)*x_ref[0]+(-1.51721910203)*x_ref[1]+(-0.677470578443)*x_ref[2]
        ref[3,0,1,1]=(0.7584807917)*x_ref[0]+(0.0523998746885)*x_ref[1]+(0.502182688228)*x_ref[2]
        ref[3,0,2,0]=(-0.905809892125)*x_ref[0]+(0.479751975382)*x_ref[1]+(-0.0778539387977)*x_ref[2]
        ref[3,0,2,1]=(-0.724891388357)*x_ref[0]+(0.941493554841)*x_ref[1]+(-1.51514565838)*x_ref[2]
        ref[3,1,0,0]=(0.815118793475)*x_ref[0]+(0.0503368907581)*x_ref[1]+(-0.502091253213)*x_ref[2]
        ref[3,1,0,1]=(-1.50859117948)*x_ref[0]+(0.31279517213)*x_ref[1]+(-0.339295035594)*x_ref[2]
        ref[3,1,1,0]=(0.246825549192)*x_ref[0]+(0.687251908372)*x_ref[1]+(0.133971781776)*x_ref[2]
        ref[3,1,1,1]=(0.470105585328)*x_ref[0]+(0.382971101979)*x_ref[1]+(-0.469397699886)*x_ref[2]
        ref[3,1,2,0]=(-1.45842815542)*x_ref[0]+(-1.00979255513)*x_ref[1]+(1.45386034482)*x_ref[2]
        ref[3,1,2,1]=(0.172130164577)*x_ref[0]+(1.25034739991)*x_ref[1]+(-1.15953423359)*x_ref[2]
        ref[3,2,0,0]=(-0.58261393055)*x_ref[0]+(0.730381032663)*x_ref[1]+(1.29377076249)*x_ref[2]
        ref[3,2,0,1]=(0.696715362351)*x_ref[0]+(-1.37598351546)*x_ref[1]+(-1.26545085967)*x_ref[2]
        ref[3,2,1,0]=(-0.0511282610422)*x_ref[0]+(0.58041679118)*x_ref[1]+(-0.0810357412151)*x_ref[2]
        ref[3,2,1,1]=(-0.750057228803)*x_ref[0]+(-0.750703125074)*x_ref[1]+(-1.21748956536)*x_ref[2]
        ref[3,2,2,0]=(-0.455746666635)*x_ref[0]+(-0.720020627516)*x_ref[1]+(-0.0911281889971)*x_ref[2]
        ref[3,2,2,1]=(-0.0681894163236)*x_ref[0]+(0.101592029278)*x_ref[1]+(1.10954864406)*x_ref[2]
        ref[3,3,0,0]=(0.433898443791)*x_ref[0]+(0.901180030496)*x_ref[1]+(0.795641263874)*x_ref[2]
        ref[3,3,0,1]=(-0.213306810738)*x_ref[0]+(-0.852539239286)*x_ref[1]+(-1.25905416259)*x_ref[2]
        ref[3,3,1,0]=(-0.668307384473)*x_ref[0]+(-1.37753879862)*x_ref[1]+(-0.593731800903)*x_ref[2]
        ref[3,3,1,1]=(0.815464294514)*x_ref[0]+(0.835389640785)*x_ref[1]+(0.933029468258)*x_ref[2]
        ref[3,3,2,0]=(0.565194274444)*x_ref[0]+(0.584119707001)*x_ref[1]+(-0.53812189952)*x_ref[2]
        ref[3,3,2,1]=(-0.903604209314)*x_ref[0]+(0.370887007483)*x_ref[1]+(-0.548135915608)*x_ref[2]
        ref[3,4,0,0]=(-0.819290139492)*x_ref[0]+(-0.256800498012)*x_ref[1]+(0.350363336935)*x_ref[2]
        ref[3,4,0,1]=(0.863170182723)*x_ref[0]+(0.0101766489337)*x_ref[1]+(1.34009635664)*x_ref[2]
        ref[3,4,1,0]=(1.03350734136)*x_ref[0]+(1.37672500233)*x_ref[1]+(1.35134434663)*x_ref[2]
        ref[3,4,1,1]=(1.4883144839)*x_ref[0]+(0.410688244039)*x_ref[1]+(0.759267998436)*x_ref[2]
        ref[3,4,2,0]=(0.373190738096)*x_ref[0]+(1.22136515279)*x_ref[1]+(-0.0365030690962)*x_ref[2]
        ref[3,4,2,1]=(-0.607437402737)*x_ref[0]+(0.750130654364)*x_ref[1]+(0.084778884789)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_ReducedContinuousFunction_rank0(self):
      """
      tests interpolate for rank 0 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedContinuousFunction onto ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(),dim=dim)
      s=Data(0,(),w)
      ref=Data(0,(),w_ref)
      if dim==2:
        s=(-1.64974193328)*x[0]+(1.68675911612)*x[1]
        ref=(-1.64974193328)*x_ref[0]+(1.68675911612)*x_ref[1]
      else:
        s=(-0.517718083008)*x[0]+(0.479679773182)*x[1]+(0.72459612383)*x[2]
        ref=(-0.517718083008)*x_ref[0]+(0.479679773182)*x_ref[1]+(0.72459612383)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_ReducedContinuousFunction_rank1(self):
      """
      tests interpolate for rank 1 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedContinuousFunction onto ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(2,),dim=dim)
      s=Data(0,(2,),w)
      ref=Data(0,(2,),w_ref)
      if dim==2:
        s[0]=(-0.415290633997)*x[0]+(0.118775036022)*x[1]
        s[1]=(-0.833749289762)*x[0]+(0.121297221921)*x[1]
        ref[0]=(-0.415290633997)*x[0]+(0.118775036022)*x[1]
        ref[1]=(-0.833749289762)*x[0]+(0.121297221921)*x[1]
      else:
        s[0]=(-0.336174941008)*x[0]+(-0.844918885743)*x[1]+(-0.410649979186)*x[2]
        s[1]=(0.112527237498)*x[0]+(-0.867632388528)*x[1]+(1.42359104479)*x[2]
        ref[0]=(-0.336174941008)*x[0]+(-0.844918885743)*x[1]+(-0.410649979186)*x[2]
        ref[1]=(0.112527237498)*x[0]+(-0.867632388528)*x[1]+(1.42359104479)*x[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_ReducedContinuousFunction_rank2(self):
      """
      tests interpolate for rank 2 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedContinuousFunction onto ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5),dim=dim)
      s=Data(0,(4, 5),w)
      ref=Data(0,(4, 5),w_ref)
      if dim==2:
        s[0,0]=(-0.833920166667)*x[0]+(-0.105734397941)*x[1]
        s[0,1]=(-0.0535194301789)*x[0]+(1.2869199411)*x[1]
        s[0,2]=(-1.81783608134)*x[0]+(-0.0858549172679)*x[1]
        s[0,3]=(0.133570573696)*x[0]+(0.398281712973)*x[1]
        s[0,4]=(-1.2514060292)*x[0]+(-0.24743946144)*x[1]
        s[1,0]=(0.917760776668)*x[0]+(0.179758313228)*x[1]
        s[1,1]=(0.677772149724)*x[0]+(0.762705763999)*x[1]
        s[1,2]=(-0.505172381789)*x[0]+(0.412365083581)*x[1]
        s[1,3]=(0.0481873177823)*x[0]+(-0.449871653505)*x[1]
        s[1,4]=(-1.43052283016)*x[0]+(0.553426132082)*x[1]
        s[2,0]=(0.337172157552)*x[0]+(1.56154138911)*x[1]
        s[2,1]=(-0.338189138128)*x[0]+(-0.62614063612)*x[1]
        s[2,2]=(-1.51606676783)*x[0]+(-0.13283454533)*x[1]
        s[2,3]=(0.727068106091)*x[0]+(1.00949960235)*x[1]
        s[2,4]=(0.297941246402)*x[0]+(0.952119168452)*x[1]
        s[3,0]=(0.724157315025)*x[0]+(0.962405533793)*x[1]
        s[3,1]=(-0.239168209123)*x[0]+(1.28280794616)*x[1]
        s[3,2]=(-0.0336024791682)*x[0]+(-0.775860593407)*x[1]
        s[3,3]=(-0.118809020482)*x[0]+(0.51890714417)*x[1]
        s[3,4]=(0.374315302545)*x[0]+(-0.258942994)*x[1]
        ref[0,0]=(-0.833920166667)*x_ref[0]+(-0.105734397941)*x_ref[1]
        ref[0,1]=(-0.0535194301789)*x_ref[0]+(1.2869199411)*x_ref[1]
        ref[0,2]=(-1.81783608134)*x_ref[0]+(-0.0858549172679)*x_ref[1]
        ref[0,3]=(0.133570573696)*x_ref[0]+(0.398281712973)*x_ref[1]
        ref[0,4]=(-1.2514060292)*x_ref[0]+(-0.24743946144)*x_ref[1]
        ref[1,0]=(0.917760776668)*x_ref[0]+(0.179758313228)*x_ref[1]
        ref[1,1]=(0.677772149724)*x_ref[0]+(0.762705763999)*x_ref[1]
        ref[1,2]=(-0.505172381789)*x_ref[0]+(0.412365083581)*x_ref[1]
        ref[1,3]=(0.0481873177823)*x_ref[0]+(-0.449871653505)*x_ref[1]
        ref[1,4]=(-1.43052283016)*x_ref[0]+(0.553426132082)*x_ref[1]
        ref[2,0]=(0.337172157552)*x_ref[0]+(1.56154138911)*x_ref[1]
        ref[2,1]=(-0.338189138128)*x_ref[0]+(-0.62614063612)*x_ref[1]
        ref[2,2]=(-1.51606676783)*x_ref[0]+(-0.13283454533)*x_ref[1]
        ref[2,3]=(0.727068106091)*x_ref[0]+(1.00949960235)*x_ref[1]
        ref[2,4]=(0.297941246402)*x_ref[0]+(0.952119168452)*x_ref[1]
        ref[3,0]=(0.724157315025)*x_ref[0]+(0.962405533793)*x_ref[1]
        ref[3,1]=(-0.239168209123)*x_ref[0]+(1.28280794616)*x_ref[1]
        ref[3,2]=(-0.0336024791682)*x_ref[0]+(-0.775860593407)*x_ref[1]
        ref[3,3]=(-0.118809020482)*x_ref[0]+(0.51890714417)*x_ref[1]
        ref[3,4]=(0.374315302545)*x_ref[0]+(-0.258942994)*x_ref[1]
      else:
        s[0,0]=(1.18033303246)*x[0]+(0.59372740835)*x[1]+(-0.684959540977)*x[2]
        s[0,1]=(0.65301033503)*x[0]+(-1.17147722359)*x[1]+(0.455404166733)*x[2]
        s[0,2]=(-0.412048337979)*x[0]+(-1.24933561194)*x[1]+(-0.636287202752)*x[2]
        s[0,3]=(0.122156598551)*x[0]+(0.103952485283)*x[1]+(-0.564471099255)*x[2]
        s[0,4]=(0.737873603859)*x[0]+(0.139615778712)*x[1]+(0.523544473477)*x[2]
        s[1,0]=(0.0198124265514)*x[0]+(-0.335073882197)*x[1]+(-0.805965691618)*x[2]
        s[1,1]=(-1.28485879121)*x[0]+(-0.618847596536)*x[1]+(1.32879899911)*x[2]
        s[1,2]=(1.09319515176)*x[0]+(0.543230471078)*x[1]+(1.56430808151)*x[2]
        s[1,3]=(0.541425658238)*x[0]+(0.0140217060596)*x[1]+(0.603558416867)*x[2]
        s[1,4]=(0.303110106189)*x[0]+(-0.103009794147)*x[1]+(-1.68324105571)*x[2]
        s[2,0]=(-1.65609235169)*x[0]+(-0.290040263496)*x[1]+(0.477073850071)*x[2]
        s[2,1]=(-0.897378807931)*x[0]+(0.191583115578)*x[1]+(0.213226828436)*x[2]
        s[2,2]=(0.523724313062)*x[0]+(0.214413673529)*x[1]+(0.533345615094)*x[2]
        s[2,3]=(-0.994043729754)*x[0]+(-0.307957055273)*x[1]+(0.0422995421427)*x[2]
        s[2,4]=(1.22590973976)*x[0]+(-1.65014223339)*x[1]+(-1.59726996804)*x[2]
        s[3,0]=(-0.0870943824713)*x[0]+(-0.45766718732)*x[1]+(-0.878874753774)*x[2]
        s[3,1]=(0.244470697307)*x[0]+(-0.0429568231364)*x[1]+(-1.57098356615)*x[2]
        s[3,2]=(1.35091699523)*x[0]+(-0.127297590922)*x[1]+(-0.556702799475)*x[2]
        s[3,3]=(0.189117166393)*x[0]+(0.317562328009)*x[1]+(-0.59788285723)*x[2]
        s[3,4]=(-0.155355776878)*x[0]+(-0.616350184089)*x[1]+(0.296083681054)*x[2]
        ref[0,0]=(1.18033303246)*x_ref[0]+(0.59372740835)*x_ref[1]+(-0.684959540977)*x_ref[2]
        ref[0,1]=(0.65301033503)*x_ref[0]+(-1.17147722359)*x_ref[1]+(0.455404166733)*x_ref[2]
        ref[0,2]=(-0.412048337979)*x_ref[0]+(-1.24933561194)*x_ref[1]+(-0.636287202752)*x_ref[2]
        ref[0,3]=(0.122156598551)*x_ref[0]+(0.103952485283)*x_ref[1]+(-0.564471099255)*x_ref[2]
        ref[0,4]=(0.737873603859)*x_ref[0]+(0.139615778712)*x_ref[1]+(0.523544473477)*x_ref[2]
        ref[1,0]=(0.0198124265514)*x_ref[0]+(-0.335073882197)*x_ref[1]+(-0.805965691618)*x_ref[2]
        ref[1,1]=(-1.28485879121)*x_ref[0]+(-0.618847596536)*x_ref[1]+(1.32879899911)*x_ref[2]
        ref[1,2]=(1.09319515176)*x_ref[0]+(0.543230471078)*x_ref[1]+(1.56430808151)*x_ref[2]
        ref[1,3]=(0.541425658238)*x_ref[0]+(0.0140217060596)*x_ref[1]+(0.603558416867)*x_ref[2]
        ref[1,4]=(0.303110106189)*x_ref[0]+(-0.103009794147)*x_ref[1]+(-1.68324105571)*x_ref[2]
        ref[2,0]=(-1.65609235169)*x_ref[0]+(-0.290040263496)*x_ref[1]+(0.477073850071)*x_ref[2]
        ref[2,1]=(-0.897378807931)*x_ref[0]+(0.191583115578)*x_ref[1]+(0.213226828436)*x_ref[2]
        ref[2,2]=(0.523724313062)*x_ref[0]+(0.214413673529)*x_ref[1]+(0.533345615094)*x_ref[2]
        ref[2,3]=(-0.994043729754)*x_ref[0]+(-0.307957055273)*x_ref[1]+(0.0422995421427)*x_ref[2]
        ref[2,4]=(1.22590973976)*x_ref[0]+(-1.65014223339)*x_ref[1]+(-1.59726996804)*x_ref[2]
        ref[3,0]=(-0.0870943824713)*x_ref[0]+(-0.45766718732)*x_ref[1]+(-0.878874753774)*x_ref[2]
        ref[3,1]=(0.244470697307)*x_ref[0]+(-0.0429568231364)*x_ref[1]+(-1.57098356615)*x_ref[2]
        ref[3,2]=(1.35091699523)*x_ref[0]+(-0.127297590922)*x_ref[1]+(-0.556702799475)*x_ref[2]
        ref[3,3]=(0.189117166393)*x_ref[0]+(0.317562328009)*x_ref[1]+(-0.59788285723)*x_ref[2]
        ref[3,4]=(-0.155355776878)*x_ref[0]+(-0.616350184089)*x_ref[1]+(0.296083681054)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_ReducedContinuousFunction_rank3(self):
      """
      tests interpolate for rank 3 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedContinuousFunction onto ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(6, 2, 2),dim=dim)
      s=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2),w_ref)
      if dim==2:
        s[0,0,0]=(0.0730549738553)*x[0]+(0.585649475666)*x[1]
        s[0,0,1]=(-0.449075310223)*x[0]+(1.39116964942)*x[1]
        s[0,1,0]=(0.0210032238157)*x[0]+(1.33904896697)*x[1]
        s[0,1,1]=(0.556125399236)*x[0]+(-0.00311484949343)*x[1]
        s[1,0,0]=(0.938296971476)*x[0]+(1.13223213262)*x[1]
        s[1,0,1]=(0.744133292867)*x[0]+(0.0271311625871)*x[1]
        s[1,1,0]=(-0.0478922372672)*x[0]+(-1.08882574207)*x[1]
        s[1,1,1]=(0.738328994373)*x[0]+(-0.730449948444)*x[1]
        s[2,0,0]=(0.66593099915)*x[0]+(-0.185918605751)*x[1]
        s[2,0,1]=(0.450972410432)*x[0]+(1.3393675839)*x[1]
        s[2,1,0]=(0.897370528228)*x[0]+(0.765127571621)*x[1]
        s[2,1,1]=(-0.349658725406)*x[0]+(-0.536332219959)*x[1]
        s[3,0,0]=(1.11112898963)*x[0]+(1.50253301112)*x[1]
        s[3,0,1]=(0.238957188525)*x[0]+(0.775368032877)*x[1]
        s[3,1,0]=(0.612991596596)*x[0]+(0.0718037077356)*x[1]
        s[3,1,1]=(-0.40039392925)*x[0]+(-1.58712410632)*x[1]
        s[4,0,0]=(0.23241254342)*x[0]+(-0.547054930927)*x[1]
        s[4,0,1]=(-0.976995206698)*x[0]+(1.251791534)*x[1]
        s[4,1,0]=(-1.13243662763)*x[0]+(0.56739947752)*x[1]
        s[4,1,1]=(-0.543613180121)*x[0]+(0.649805921416)*x[1]
        s[5,0,0]=(-0.0690689383083)*x[0]+(0.818563064557)*x[1]
        s[5,0,1]=(0.362899720121)*x[0]+(-0.112604112729)*x[1]
        s[5,1,0]=(-0.291756903179)*x[0]+(-0.538221294595)*x[1]
        s[5,1,1]=(0.60406331286)*x[0]+(0.228744896658)*x[1]
        ref[0,0,0]=(0.0730549738553)*x_ref[0]+(0.585649475666)*x_ref[1]
        ref[0,0,1]=(-0.449075310223)*x_ref[0]+(1.39116964942)*x_ref[1]
        ref[0,1,0]=(0.0210032238157)*x_ref[0]+(1.33904896697)*x_ref[1]
        ref[0,1,1]=(0.556125399236)*x_ref[0]+(-0.00311484949343)*x_ref[1]
        ref[1,0,0]=(0.938296971476)*x_ref[0]+(1.13223213262)*x_ref[1]
        ref[1,0,1]=(0.744133292867)*x_ref[0]+(0.0271311625871)*x_ref[1]
        ref[1,1,0]=(-0.0478922372672)*x_ref[0]+(-1.08882574207)*x_ref[1]
        ref[1,1,1]=(0.738328994373)*x_ref[0]+(-0.730449948444)*x_ref[1]
        ref[2,0,0]=(0.66593099915)*x_ref[0]+(-0.185918605751)*x_ref[1]
        ref[2,0,1]=(0.450972410432)*x_ref[0]+(1.3393675839)*x_ref[1]
        ref[2,1,0]=(0.897370528228)*x_ref[0]+(0.765127571621)*x_ref[1]
        ref[2,1,1]=(-0.349658725406)*x_ref[0]+(-0.536332219959)*x_ref[1]
        ref[3,0,0]=(1.11112898963)*x_ref[0]+(1.50253301112)*x_ref[1]
        ref[3,0,1]=(0.238957188525)*x_ref[0]+(0.775368032877)*x_ref[1]
        ref[3,1,0]=(0.612991596596)*x_ref[0]+(0.0718037077356)*x_ref[1]
        ref[3,1,1]=(-0.40039392925)*x_ref[0]+(-1.58712410632)*x_ref[1]
        ref[4,0,0]=(0.23241254342)*x_ref[0]+(-0.547054930927)*x_ref[1]
        ref[4,0,1]=(-0.976995206698)*x_ref[0]+(1.251791534)*x_ref[1]
        ref[4,1,0]=(-1.13243662763)*x_ref[0]+(0.56739947752)*x_ref[1]
        ref[4,1,1]=(-0.543613180121)*x_ref[0]+(0.649805921416)*x_ref[1]
        ref[5,0,0]=(-0.0690689383083)*x_ref[0]+(0.818563064557)*x_ref[1]
        ref[5,0,1]=(0.362899720121)*x_ref[0]+(-0.112604112729)*x_ref[1]
        ref[5,1,0]=(-0.291756903179)*x_ref[0]+(-0.538221294595)*x_ref[1]
        ref[5,1,1]=(0.60406331286)*x_ref[0]+(0.228744896658)*x_ref[1]
      else:
        s[0,0,0]=(0.811538724853)*x[0]+(-0.347250364239)*x[1]+(-0.116011094391)*x[2]
        s[0,0,1]=(1.69112205586)*x[0]+(-1.04197300041)*x[1]+(0.849118123462)*x[2]
        s[0,1,0]=(1.30816943778)*x[0]+(1.01636941747)*x[1]+(-0.849746116393)*x[2]
        s[0,1,1]=(0.888683598925)*x[0]+(-1.60930356378)*x[1]+(0.522151570265)*x[2]
        s[1,0,0]=(1.03821316703)*x[0]+(-1.19647941504)*x[1]+(-1.08543246224)*x[2]
        s[1,0,1]=(-0.314182278588)*x[0]+(0.144825414517)*x[1]+(0.162613879231)*x[2]
        s[1,1,0]=(0.0416209427382)*x[0]+(0.0390092175082)*x[1]+(1.55616366687)*x[2]
        s[1,1,1]=(-0.577251716148)*x[0]+(0.717529140134)*x[1]+(-0.567199280535)*x[2]
        s[2,0,0]=(1.26716303575)*x[0]+(0.119576983216)*x[1]+(-0.602181033831)*x[2]
        s[2,0,1]=(0.533935330879)*x[0]+(0.292003869006)*x[1]+(-0.0917835680855)*x[2]
        s[2,1,0]=(-1.50218600857)*x[0]+(1.23418550746)*x[1]+(-0.600240530145)*x[2]
        s[2,1,1]=(0.689151741161)*x[0]+(0.909077564156)*x[1]+(0.924638961344)*x[2]
        s[3,0,0]=(-0.763411711557)*x[0]+(-0.862413521307)*x[1]+(1.49095932554)*x[2]
        s[3,0,1]=(-0.215589144977)*x[0]+(0.305301450314)*x[1]+(-0.468853406603)*x[2]
        s[3,1,0]=(-0.433045929433)*x[0]+(-0.907669929171)*x[1]+(0.917462300039)*x[2]
        s[3,1,1]=(0.229593212783)*x[0]+(-1.79371444035)*x[1]+(-0.848409349534)*x[2]
        s[4,0,0]=(1.40249316312)*x[0]+(-0.766609553683)*x[1]+(0.337911083882)*x[2]
        s[4,0,1]=(0.728650674867)*x[0]+(0.631222570369)*x[1]+(1.2491315131)*x[2]
        s[4,1,0]=(1.03998601102)*x[0]+(-1.81204917532)*x[1]+(0.781499164268)*x[2]
        s[4,1,1]=(-0.151999545303)*x[0]+(-0.767585861167)*x[1]+(1.17261654125)*x[2]
        s[5,0,0]=(0.0632249575346)*x[0]+(0.333565970302)*x[1]+(-1.66679001354)*x[2]
        s[5,0,1]=(0.945677231929)*x[0]+(-1.15936834775)*x[1]+(-0.644891644457)*x[2]
        s[5,1,0]=(1.08939087839)*x[0]+(0.687932839014)*x[1]+(-0.473426758917)*x[2]
        s[5,1,1]=(0.0349915672648)*x[0]+(-1.48593425129)*x[1]+(0.145421756236)*x[2]
        ref[0,0,0]=(0.811538724853)*x_ref[0]+(-0.347250364239)*x_ref[1]+(-0.116011094391)*x_ref[2]
        ref[0,0,1]=(1.69112205586)*x_ref[0]+(-1.04197300041)*x_ref[1]+(0.849118123462)*x_ref[2]
        ref[0,1,0]=(1.30816943778)*x_ref[0]+(1.01636941747)*x_ref[1]+(-0.849746116393)*x_ref[2]
        ref[0,1,1]=(0.888683598925)*x_ref[0]+(-1.60930356378)*x_ref[1]+(0.522151570265)*x_ref[2]
        ref[1,0,0]=(1.03821316703)*x_ref[0]+(-1.19647941504)*x_ref[1]+(-1.08543246224)*x_ref[2]
        ref[1,0,1]=(-0.314182278588)*x_ref[0]+(0.144825414517)*x_ref[1]+(0.162613879231)*x_ref[2]
        ref[1,1,0]=(0.0416209427382)*x_ref[0]+(0.0390092175082)*x_ref[1]+(1.55616366687)*x_ref[2]
        ref[1,1,1]=(-0.577251716148)*x_ref[0]+(0.717529140134)*x_ref[1]+(-0.567199280535)*x_ref[2]
        ref[2,0,0]=(1.26716303575)*x_ref[0]+(0.119576983216)*x_ref[1]+(-0.602181033831)*x_ref[2]
        ref[2,0,1]=(0.533935330879)*x_ref[0]+(0.292003869006)*x_ref[1]+(-0.0917835680855)*x_ref[2]
        ref[2,1,0]=(-1.50218600857)*x_ref[0]+(1.23418550746)*x_ref[1]+(-0.600240530145)*x_ref[2]
        ref[2,1,1]=(0.689151741161)*x_ref[0]+(0.909077564156)*x_ref[1]+(0.924638961344)*x_ref[2]
        ref[3,0,0]=(-0.763411711557)*x_ref[0]+(-0.862413521307)*x_ref[1]+(1.49095932554)*x_ref[2]
        ref[3,0,1]=(-0.215589144977)*x_ref[0]+(0.305301450314)*x_ref[1]+(-0.468853406603)*x_ref[2]
        ref[3,1,0]=(-0.433045929433)*x_ref[0]+(-0.907669929171)*x_ref[1]+(0.917462300039)*x_ref[2]
        ref[3,1,1]=(0.229593212783)*x_ref[0]+(-1.79371444035)*x_ref[1]+(-0.848409349534)*x_ref[2]
        ref[4,0,0]=(1.40249316312)*x_ref[0]+(-0.766609553683)*x_ref[1]+(0.337911083882)*x_ref[2]
        ref[4,0,1]=(0.728650674867)*x_ref[0]+(0.631222570369)*x_ref[1]+(1.2491315131)*x_ref[2]
        ref[4,1,0]=(1.03998601102)*x_ref[0]+(-1.81204917532)*x_ref[1]+(0.781499164268)*x_ref[2]
        ref[4,1,1]=(-0.151999545303)*x_ref[0]+(-0.767585861167)*x_ref[1]+(1.17261654125)*x_ref[2]
        ref[5,0,0]=(0.0632249575346)*x_ref[0]+(0.333565970302)*x_ref[1]+(-1.66679001354)*x_ref[2]
        ref[5,0,1]=(0.945677231929)*x_ref[0]+(-1.15936834775)*x_ref[1]+(-0.644891644457)*x_ref[2]
        ref[5,1,0]=(1.08939087839)*x_ref[0]+(0.687932839014)*x_ref[1]+(-0.473426758917)*x_ref[2]
        ref[5,1,1]=(0.0349915672648)*x_ref[0]+(-1.48593425129)*x_ref[1]+(0.145421756236)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_ReducedContinuousFunction_rank4(self):
      """
      tests interpolate for rank 4 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedContinuousFunction onto ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5, 3, 2),dim=dim)
      s=Data(0,(4, 5, 3, 2),w)
      ref=Data(0,(4, 5, 3, 2),w_ref)
      if dim==2:
        s[0,0,0,0]=(0.609727033659)*x[0]+(-0.901858940633)*x[1]
        s[0,0,0,1]=(-0.221259461243)*x[0]+(0.351136340908)*x[1]
        s[0,0,1,0]=(0.497173734282)*x[0]+(0.899142906899)*x[1]
        s[0,0,1,1]=(-1.55206649551)*x[0]+(-0.680257384044)*x[1]
        s[0,0,2,0]=(-0.932033638033)*x[0]+(-0.46503758126)*x[1]
        s[0,0,2,1]=(-0.0372650123525)*x[0]+(0.0173714029098)*x[1]
        s[0,1,0,0]=(0.855963098465)*x[0]+(0.760176320898)*x[1]
        s[0,1,0,1]=(1.37546120327)*x[0]+(1.72916158331)*x[1]
        s[0,1,1,0]=(0.25469861613)*x[0]+(0.71445716033)*x[1]
        s[0,1,1,1]=(0.198455659765)*x[0]+(0.0410276829495)*x[1]
        s[0,1,2,0]=(-0.628080427667)*x[0]+(-0.0197206025984)*x[1]
        s[0,1,2,1]=(1.26759029895)*x[0]+(1.15914953253)*x[1]
        s[0,2,0,0]=(-1.43203053441)*x[0]+(0.366301170069)*x[1]
        s[0,2,0,1]=(0.420527802229)*x[0]+(-0.709209747415)*x[1]
        s[0,2,1,0]=(0.399042264829)*x[0]+(0.0453132754731)*x[1]
        s[0,2,1,1]=(1.2074346062)*x[0]+(-1.14163410992)*x[1]
        s[0,2,2,0]=(-0.00270984158569)*x[0]+(-1.63216200182)*x[1]
        s[0,2,2,1]=(0.174276088847)*x[0]+(-1.04789472231)*x[1]
        s[0,3,0,0]=(-0.47704729036)*x[0]+(-1.2550637204)*x[1]
        s[0,3,0,1]=(0.348356873191)*x[0]+(-0.796668578265)*x[1]
        s[0,3,1,0]=(-0.52724725097)*x[0]+(1.26163123592)*x[1]
        s[0,3,1,1]=(-1.52681799261)*x[0]+(0.936041931735)*x[1]
        s[0,3,2,0]=(1.33689673219)*x[0]+(-1.34901214018)*x[1]
        s[0,3,2,1]=(1.57651286047)*x[0]+(0.245413762864)*x[1]
        s[0,4,0,0]=(0.194500181498)*x[0]+(-0.0231951844441)*x[1]
        s[0,4,0,1]=(-1.49372076697)*x[0]+(-0.111369678181)*x[1]
        s[0,4,1,0]=(-0.74121646836)*x[0]+(-0.817740156307)*x[1]
        s[0,4,1,1]=(-0.233643397123)*x[0]+(0.672715068796)*x[1]
        s[0,4,2,0]=(1.05850947579)*x[0]+(0.386484386191)*x[1]
        s[0,4,2,1]=(1.45105922963)*x[0]+(-0.367458625996)*x[1]
        s[1,0,0,0]=(-0.391803788669)*x[0]+(0.797592300662)*x[1]
        s[1,0,0,1]=(-0.205299072883)*x[0]+(0.677427935558)*x[1]
        s[1,0,1,0]=(1.56225329031)*x[0]+(1.17468092515)*x[1]
        s[1,0,1,1]=(1.4586005658)*x[0]+(1.36481062943)*x[1]
        s[1,0,2,0]=(-1.53281881413)*x[0]+(0.207028833974)*x[1]
        s[1,0,2,1]=(0.0469239691648)*x[0]+(1.03547812705)*x[1]
        s[1,1,0,0]=(1.39614335897)*x[0]+(0.82046489695)*x[1]
        s[1,1,0,1]=(-0.775462741637)*x[0]+(-0.275081761646)*x[1]
        s[1,1,1,0]=(0.173353028608)*x[0]+(-1.05620431816)*x[1]
        s[1,1,1,1]=(-0.464192870247)*x[0]+(0.212204784206)*x[1]
        s[1,1,2,0]=(-0.383813531866)*x[0]+(-0.0773759374234)*x[1]
        s[1,1,2,1]=(-1.07811385759)*x[0]+(-0.457232330819)*x[1]
        s[1,2,0,0]=(-0.295869147576)*x[0]+(0.166843915631)*x[1]
        s[1,2,0,1]=(0.571421034325)*x[0]+(1.46722583245)*x[1]
        s[1,2,1,0]=(-0.34494944257)*x[0]+(-0.286867052491)*x[1]
        s[1,2,1,1]=(1.43128885761)*x[0]+(0.21277887035)*x[1]
        s[1,2,2,0]=(-1.19397187873)*x[0]+(0.126397665227)*x[1]
        s[1,2,2,1]=(0.171368751954)*x[0]+(-1.12100254625)*x[1]
        s[1,3,0,0]=(0.0975333077604)*x[0]+(0.583524660188)*x[1]
        s[1,3,0,1]=(0.0271450955458)*x[0]+(0.368978332867)*x[1]
        s[1,3,1,0]=(1.22806968486)*x[0]+(-0.569932748468)*x[1]
        s[1,3,1,1]=(-0.691989839521)*x[0]+(-0.829822182922)*x[1]
        s[1,3,2,0]=(0.743776534098)*x[0]+(1.3668937127)*x[1]
        s[1,3,2,1]=(0.192206575204)*x[0]+(0.96576702824)*x[1]
        s[1,4,0,0]=(0.149381460079)*x[0]+(-0.987875538036)*x[1]
        s[1,4,0,1]=(-0.706855836683)*x[0]+(0.0410884211631)*x[1]
        s[1,4,1,0]=(-0.222650274)*x[0]+(0.59589242048)*x[1]
        s[1,4,1,1]=(0.541489186743)*x[0]+(-1.22543996074)*x[1]
        s[1,4,2,0]=(0.630234824652)*x[0]+(0.508436575179)*x[1]
        s[1,4,2,1]=(0.990062359294)*x[0]+(-0.639884693407)*x[1]
        s[2,0,0,0]=(0.378887461733)*x[0]+(1.71999809344)*x[1]
        s[2,0,0,1]=(0.182412683544)*x[0]+(-0.401183243877)*x[1]
        s[2,0,1,0]=(0.325775290396)*x[0]+(-0.656088786126)*x[1]
        s[2,0,1,1]=(0.316355229742)*x[0]+(1.54087545195)*x[1]
        s[2,0,2,0]=(1.14203881183)*x[0]+(0.703207191364)*x[1]
        s[2,0,2,1]=(-1.05211790226)*x[0]+(1.68596483036)*x[1]
        s[2,1,0,0]=(0.704109387934)*x[0]+(-0.589003921574)*x[1]
        s[2,1,0,1]=(-0.382489922028)*x[0]+(0.387647642942)*x[1]
        s[2,1,1,0]=(-0.0343434958891)*x[0]+(-0.119176404279)*x[1]
        s[2,1,1,1]=(0.567853527929)*x[0]+(1.32523261969)*x[1]
        s[2,1,2,0]=(1.03496000653)*x[0]+(0.110203591459)*x[1]
        s[2,1,2,1]=(0.560149997806)*x[0]+(-1.62893742873)*x[1]
        s[2,2,0,0]=(1.1801792312)*x[0]+(1.1912385935)*x[1]
        s[2,2,0,1]=(0.936932817634)*x[0]+(-1.49082021974)*x[1]
        s[2,2,1,0]=(-0.305582892069)*x[0]+(-0.267235698563)*x[1]
        s[2,2,1,1]=(0.107449730184)*x[0]+(-0.325267993615)*x[1]
        s[2,2,2,0]=(-1.08118986347)*x[0]+(0.144190727334)*x[1]
        s[2,2,2,1]=(1.44755060146)*x[0]+(-0.127612338817)*x[1]
        s[2,3,0,0]=(-0.666943112999)*x[0]+(-0.780402371989)*x[1]
        s[2,3,0,1]=(0.724206717771)*x[0]+(-0.523593908278)*x[1]
        s[2,3,1,0]=(1.86101197199)*x[0]+(-0.710894911302)*x[1]
        s[2,3,1,1]=(-1.74409232023)*x[0]+(0.407821816271)*x[1]
        s[2,3,2,0]=(0.646304172575)*x[0]+(0.0132628778543)*x[1]
        s[2,3,2,1]=(-0.746041438673)*x[0]+(0.516304872741)*x[1]
        s[2,4,0,0]=(0.0326900201283)*x[0]+(-0.533021156401)*x[1]
        s[2,4,0,1]=(0.088598956981)*x[0]+(0.764307970269)*x[1]
        s[2,4,1,0]=(-0.749553477756)*x[0]+(0.379646792939)*x[1]
        s[2,4,1,1]=(-0.243635321378)*x[0]+(-0.470022600261)*x[1]
        s[2,4,2,0]=(0.586398757247)*x[0]+(-0.116982897683)*x[1]
        s[2,4,2,1]=(-0.989012121879)*x[0]+(-1.25006535257)*x[1]
        s[3,0,0,0]=(1.25228306081)*x[0]+(-0.0932717035896)*x[1]
        s[3,0,0,1]=(1.5975516771)*x[0]+(-0.177733286111)*x[1]
        s[3,0,1,0]=(-0.659910070911)*x[0]+(-0.0999498944812)*x[1]
        s[3,0,1,1]=(-0.299870273662)*x[0]+(-1.22977656744)*x[1]
        s[3,0,2,0]=(-0.158211581143)*x[0]+(-0.141421380503)*x[1]
        s[3,0,2,1]=(-1.48226452788)*x[0]+(-0.731264943309)*x[1]
        s[3,1,0,0]=(-1.64589827853)*x[0]+(1.19692573998)*x[1]
        s[3,1,0,1]=(0.361329655357)*x[0]+(0.234310482171)*x[1]
        s[3,1,1,0]=(-1.14077649516)*x[0]+(0.132657084065)*x[1]
        s[3,1,1,1]=(1.41067975663)*x[0]+(0.818243117884)*x[1]
        s[3,1,2,0]=(-1.53137884342)*x[0]+(-0.0127805333093)*x[1]
        s[3,1,2,1]=(0.240455755486)*x[0]+(-1.02361753893)*x[1]
        s[3,2,0,0]=(-0.334803473072)*x[0]+(-0.807452510423)*x[1]
        s[3,2,0,1]=(0.617324536974)*x[0]+(-0.733163471774)*x[1]
        s[3,2,1,0]=(-0.634580050897)*x[0]+(0.362106278168)*x[1]
        s[3,2,1,1]=(-0.21525531818)*x[0]+(0.982902540898)*x[1]
        s[3,2,2,0]=(-0.590682803195)*x[0]+(-1.27537880632)*x[1]
        s[3,2,2,1]=(0.142578313438)*x[0]+(0.212341687291)*x[1]
        s[3,3,0,0]=(-0.758345317216)*x[0]+(-0.971914631359)*x[1]
        s[3,3,0,1]=(0.569463909234)*x[0]+(0.498848917338)*x[1]
        s[3,3,1,0]=(-0.771456575632)*x[0]+(-0.130456072944)*x[1]
        s[3,3,1,1]=(0.811799269245)*x[0]+(-0.829647203396)*x[1]
        s[3,3,2,0]=(-0.814979427693)*x[0]+(-0.0416092796813)*x[1]
        s[3,3,2,1]=(0.933434189127)*x[0]+(0.60794704936)*x[1]
        s[3,4,0,0]=(0.160280566478)*x[0]+(-0.0141616040181)*x[1]
        s[3,4,0,1]=(-1.34835766268)*x[0]+(0.178951225861)*x[1]
        s[3,4,1,0]=(1.67632814658)*x[0]+(1.31316096483)*x[1]
        s[3,4,1,1]=(-0.603403275376)*x[0]+(0.0607753526431)*x[1]
        s[3,4,2,0]=(-0.178440295473)*x[0]+(0.318102694371)*x[1]
        s[3,4,2,1]=(0.781285160749)*x[0]+(0.625651915981)*x[1]
        ref[0,0,0,0]=(0.609727033659)*x_ref[0]+(-0.901858940633)*x_ref[1]
        ref[0,0,0,1]=(-0.221259461243)*x_ref[0]+(0.351136340908)*x_ref[1]
        ref[0,0,1,0]=(0.497173734282)*x_ref[0]+(0.899142906899)*x_ref[1]
        ref[0,0,1,1]=(-1.55206649551)*x_ref[0]+(-0.680257384044)*x_ref[1]
        ref[0,0,2,0]=(-0.932033638033)*x_ref[0]+(-0.46503758126)*x_ref[1]
        ref[0,0,2,1]=(-0.0372650123525)*x_ref[0]+(0.0173714029098)*x_ref[1]
        ref[0,1,0,0]=(0.855963098465)*x_ref[0]+(0.760176320898)*x_ref[1]
        ref[0,1,0,1]=(1.37546120327)*x_ref[0]+(1.72916158331)*x_ref[1]
        ref[0,1,1,0]=(0.25469861613)*x_ref[0]+(0.71445716033)*x_ref[1]
        ref[0,1,1,1]=(0.198455659765)*x_ref[0]+(0.0410276829495)*x_ref[1]
        ref[0,1,2,0]=(-0.628080427667)*x_ref[0]+(-0.0197206025984)*x_ref[1]
        ref[0,1,2,1]=(1.26759029895)*x_ref[0]+(1.15914953253)*x_ref[1]
        ref[0,2,0,0]=(-1.43203053441)*x_ref[0]+(0.366301170069)*x_ref[1]
        ref[0,2,0,1]=(0.420527802229)*x_ref[0]+(-0.709209747415)*x_ref[1]
        ref[0,2,1,0]=(0.399042264829)*x_ref[0]+(0.0453132754731)*x_ref[1]
        ref[0,2,1,1]=(1.2074346062)*x_ref[0]+(-1.14163410992)*x_ref[1]
        ref[0,2,2,0]=(-0.00270984158569)*x_ref[0]+(-1.63216200182)*x_ref[1]
        ref[0,2,2,1]=(0.174276088847)*x_ref[0]+(-1.04789472231)*x_ref[1]
        ref[0,3,0,0]=(-0.47704729036)*x_ref[0]+(-1.2550637204)*x_ref[1]
        ref[0,3,0,1]=(0.348356873191)*x_ref[0]+(-0.796668578265)*x_ref[1]
        ref[0,3,1,0]=(-0.52724725097)*x_ref[0]+(1.26163123592)*x_ref[1]
        ref[0,3,1,1]=(-1.52681799261)*x_ref[0]+(0.936041931735)*x_ref[1]
        ref[0,3,2,0]=(1.33689673219)*x_ref[0]+(-1.34901214018)*x_ref[1]
        ref[0,3,2,1]=(1.57651286047)*x_ref[0]+(0.245413762864)*x_ref[1]
        ref[0,4,0,0]=(0.194500181498)*x_ref[0]+(-0.0231951844441)*x_ref[1]
        ref[0,4,0,1]=(-1.49372076697)*x_ref[0]+(-0.111369678181)*x_ref[1]
        ref[0,4,1,0]=(-0.74121646836)*x_ref[0]+(-0.817740156307)*x_ref[1]
        ref[0,4,1,1]=(-0.233643397123)*x_ref[0]+(0.672715068796)*x_ref[1]
        ref[0,4,2,0]=(1.05850947579)*x_ref[0]+(0.386484386191)*x_ref[1]
        ref[0,4,2,1]=(1.45105922963)*x_ref[0]+(-0.367458625996)*x_ref[1]
        ref[1,0,0,0]=(-0.391803788669)*x_ref[0]+(0.797592300662)*x_ref[1]
        ref[1,0,0,1]=(-0.205299072883)*x_ref[0]+(0.677427935558)*x_ref[1]
        ref[1,0,1,0]=(1.56225329031)*x_ref[0]+(1.17468092515)*x_ref[1]
        ref[1,0,1,1]=(1.4586005658)*x_ref[0]+(1.36481062943)*x_ref[1]
        ref[1,0,2,0]=(-1.53281881413)*x_ref[0]+(0.207028833974)*x_ref[1]
        ref[1,0,2,1]=(0.0469239691648)*x_ref[0]+(1.03547812705)*x_ref[1]
        ref[1,1,0,0]=(1.39614335897)*x_ref[0]+(0.82046489695)*x_ref[1]
        ref[1,1,0,1]=(-0.775462741637)*x_ref[0]+(-0.275081761646)*x_ref[1]
        ref[1,1,1,0]=(0.173353028608)*x_ref[0]+(-1.05620431816)*x_ref[1]
        ref[1,1,1,1]=(-0.464192870247)*x_ref[0]+(0.212204784206)*x_ref[1]
        ref[1,1,2,0]=(-0.383813531866)*x_ref[0]+(-0.0773759374234)*x_ref[1]
        ref[1,1,2,1]=(-1.07811385759)*x_ref[0]+(-0.457232330819)*x_ref[1]
        ref[1,2,0,0]=(-0.295869147576)*x_ref[0]+(0.166843915631)*x_ref[1]
        ref[1,2,0,1]=(0.571421034325)*x_ref[0]+(1.46722583245)*x_ref[1]
        ref[1,2,1,0]=(-0.34494944257)*x_ref[0]+(-0.286867052491)*x_ref[1]
        ref[1,2,1,1]=(1.43128885761)*x_ref[0]+(0.21277887035)*x_ref[1]
        ref[1,2,2,0]=(-1.19397187873)*x_ref[0]+(0.126397665227)*x_ref[1]
        ref[1,2,2,1]=(0.171368751954)*x_ref[0]+(-1.12100254625)*x_ref[1]
        ref[1,3,0,0]=(0.0975333077604)*x_ref[0]+(0.583524660188)*x_ref[1]
        ref[1,3,0,1]=(0.0271450955458)*x_ref[0]+(0.368978332867)*x_ref[1]
        ref[1,3,1,0]=(1.22806968486)*x_ref[0]+(-0.569932748468)*x_ref[1]
        ref[1,3,1,1]=(-0.691989839521)*x_ref[0]+(-0.829822182922)*x_ref[1]
        ref[1,3,2,0]=(0.743776534098)*x_ref[0]+(1.3668937127)*x_ref[1]
        ref[1,3,2,1]=(0.192206575204)*x_ref[0]+(0.96576702824)*x_ref[1]
        ref[1,4,0,0]=(0.149381460079)*x_ref[0]+(-0.987875538036)*x_ref[1]
        ref[1,4,0,1]=(-0.706855836683)*x_ref[0]+(0.0410884211631)*x_ref[1]
        ref[1,4,1,0]=(-0.222650274)*x_ref[0]+(0.59589242048)*x_ref[1]
        ref[1,4,1,1]=(0.541489186743)*x_ref[0]+(-1.22543996074)*x_ref[1]
        ref[1,4,2,0]=(0.630234824652)*x_ref[0]+(0.508436575179)*x_ref[1]
        ref[1,4,2,1]=(0.990062359294)*x_ref[0]+(-0.639884693407)*x_ref[1]
        ref[2,0,0,0]=(0.378887461733)*x_ref[0]+(1.71999809344)*x_ref[1]
        ref[2,0,0,1]=(0.182412683544)*x_ref[0]+(-0.401183243877)*x_ref[1]
        ref[2,0,1,0]=(0.325775290396)*x_ref[0]+(-0.656088786126)*x_ref[1]
        ref[2,0,1,1]=(0.316355229742)*x_ref[0]+(1.54087545195)*x_ref[1]
        ref[2,0,2,0]=(1.14203881183)*x_ref[0]+(0.703207191364)*x_ref[1]
        ref[2,0,2,1]=(-1.05211790226)*x_ref[0]+(1.68596483036)*x_ref[1]
        ref[2,1,0,0]=(0.704109387934)*x_ref[0]+(-0.589003921574)*x_ref[1]
        ref[2,1,0,1]=(-0.382489922028)*x_ref[0]+(0.387647642942)*x_ref[1]
        ref[2,1,1,0]=(-0.0343434958891)*x_ref[0]+(-0.119176404279)*x_ref[1]
        ref[2,1,1,1]=(0.567853527929)*x_ref[0]+(1.32523261969)*x_ref[1]
        ref[2,1,2,0]=(1.03496000653)*x_ref[0]+(0.110203591459)*x_ref[1]
        ref[2,1,2,1]=(0.560149997806)*x_ref[0]+(-1.62893742873)*x_ref[1]
        ref[2,2,0,0]=(1.1801792312)*x_ref[0]+(1.1912385935)*x_ref[1]
        ref[2,2,0,1]=(0.936932817634)*x_ref[0]+(-1.49082021974)*x_ref[1]
        ref[2,2,1,0]=(-0.305582892069)*x_ref[0]+(-0.267235698563)*x_ref[1]
        ref[2,2,1,1]=(0.107449730184)*x_ref[0]+(-0.325267993615)*x_ref[1]
        ref[2,2,2,0]=(-1.08118986347)*x_ref[0]+(0.144190727334)*x_ref[1]
        ref[2,2,2,1]=(1.44755060146)*x_ref[0]+(-0.127612338817)*x_ref[1]
        ref[2,3,0,0]=(-0.666943112999)*x_ref[0]+(-0.780402371989)*x_ref[1]
        ref[2,3,0,1]=(0.724206717771)*x_ref[0]+(-0.523593908278)*x_ref[1]
        ref[2,3,1,0]=(1.86101197199)*x_ref[0]+(-0.710894911302)*x_ref[1]
        ref[2,3,1,1]=(-1.74409232023)*x_ref[0]+(0.407821816271)*x_ref[1]
        ref[2,3,2,0]=(0.646304172575)*x_ref[0]+(0.0132628778543)*x_ref[1]
        ref[2,3,2,1]=(-0.746041438673)*x_ref[0]+(0.516304872741)*x_ref[1]
        ref[2,4,0,0]=(0.0326900201283)*x_ref[0]+(-0.533021156401)*x_ref[1]
        ref[2,4,0,1]=(0.088598956981)*x_ref[0]+(0.764307970269)*x_ref[1]
        ref[2,4,1,0]=(-0.749553477756)*x_ref[0]+(0.379646792939)*x_ref[1]
        ref[2,4,1,1]=(-0.243635321378)*x_ref[0]+(-0.470022600261)*x_ref[1]
        ref[2,4,2,0]=(0.586398757247)*x_ref[0]+(-0.116982897683)*x_ref[1]
        ref[2,4,2,1]=(-0.989012121879)*x_ref[0]+(-1.25006535257)*x_ref[1]
        ref[3,0,0,0]=(1.25228306081)*x_ref[0]+(-0.0932717035896)*x_ref[1]
        ref[3,0,0,1]=(1.5975516771)*x_ref[0]+(-0.177733286111)*x_ref[1]
        ref[3,0,1,0]=(-0.659910070911)*x_ref[0]+(-0.0999498944812)*x_ref[1]
        ref[3,0,1,1]=(-0.299870273662)*x_ref[0]+(-1.22977656744)*x_ref[1]
        ref[3,0,2,0]=(-0.158211581143)*x_ref[0]+(-0.141421380503)*x_ref[1]
        ref[3,0,2,1]=(-1.48226452788)*x_ref[0]+(-0.731264943309)*x_ref[1]
        ref[3,1,0,0]=(-1.64589827853)*x_ref[0]+(1.19692573998)*x_ref[1]
        ref[3,1,0,1]=(0.361329655357)*x_ref[0]+(0.234310482171)*x_ref[1]
        ref[3,1,1,0]=(-1.14077649516)*x_ref[0]+(0.132657084065)*x_ref[1]
        ref[3,1,1,1]=(1.41067975663)*x_ref[0]+(0.818243117884)*x_ref[1]
        ref[3,1,2,0]=(-1.53137884342)*x_ref[0]+(-0.0127805333093)*x_ref[1]
        ref[3,1,2,1]=(0.240455755486)*x_ref[0]+(-1.02361753893)*x_ref[1]
        ref[3,2,0,0]=(-0.334803473072)*x_ref[0]+(-0.807452510423)*x_ref[1]
        ref[3,2,0,1]=(0.617324536974)*x_ref[0]+(-0.733163471774)*x_ref[1]
        ref[3,2,1,0]=(-0.634580050897)*x_ref[0]+(0.362106278168)*x_ref[1]
        ref[3,2,1,1]=(-0.21525531818)*x_ref[0]+(0.982902540898)*x_ref[1]
        ref[3,2,2,0]=(-0.590682803195)*x_ref[0]+(-1.27537880632)*x_ref[1]
        ref[3,2,2,1]=(0.142578313438)*x_ref[0]+(0.212341687291)*x_ref[1]
        ref[3,3,0,0]=(-0.758345317216)*x_ref[0]+(-0.971914631359)*x_ref[1]
        ref[3,3,0,1]=(0.569463909234)*x_ref[0]+(0.498848917338)*x_ref[1]
        ref[3,3,1,0]=(-0.771456575632)*x_ref[0]+(-0.130456072944)*x_ref[1]
        ref[3,3,1,1]=(0.811799269245)*x_ref[0]+(-0.829647203396)*x_ref[1]
        ref[3,3,2,0]=(-0.814979427693)*x_ref[0]+(-0.0416092796813)*x_ref[1]
        ref[3,3,2,1]=(0.933434189127)*x_ref[0]+(0.60794704936)*x_ref[1]
        ref[3,4,0,0]=(0.160280566478)*x_ref[0]+(-0.0141616040181)*x_ref[1]
        ref[3,4,0,1]=(-1.34835766268)*x_ref[0]+(0.178951225861)*x_ref[1]
        ref[3,4,1,0]=(1.67632814658)*x_ref[0]+(1.31316096483)*x_ref[1]
        ref[3,4,1,1]=(-0.603403275376)*x_ref[0]+(0.0607753526431)*x_ref[1]
        ref[3,4,2,0]=(-0.178440295473)*x_ref[0]+(0.318102694371)*x_ref[1]
        ref[3,4,2,1]=(0.781285160749)*x_ref[0]+(0.625651915981)*x_ref[1]
      else:
        s[0,0,0,0]=(0.847845815599)*x[0]+(-0.610234211476)*x[1]+(-0.627625510082)*x[2]
        s[0,0,0,1]=(0.0126184651041)*x[0]+(-0.0651294063564)*x[1]+(-0.274615935201)*x[2]
        s[0,0,1,0]=(-0.545925417969)*x[0]+(0.421929302201)*x[1]+(-0.910418625383)*x[2]
        s[0,0,1,1]=(1.58754252937)*x[0]+(0.110111591047)*x[1]+(-1.64780563736)*x[2]
        s[0,0,2,0]=(-0.510331643992)*x[0]+(0.827685326542)*x[1]+(1.31447962917)*x[2]
        s[0,0,2,1]=(-1.29575211142)*x[0]+(-0.318096803401)*x[1]+(-0.274660613777)*x[2]
        s[0,1,0,0]=(0.77444944835)*x[0]+(-1.0893411122)*x[1]+(0.331702744143)*x[2]
        s[0,1,0,1]=(0.624693916432)*x[0]+(-0.306552086758)*x[1]+(-0.845524417807)*x[2]
        s[0,1,1,0]=(0.0219492668609)*x[0]+(-1.13712865204)*x[1]+(-0.193051406533)*x[2]
        s[0,1,1,1]=(-1.32696889246)*x[0]+(1.63857177904)*x[1]+(0.194095476936)*x[2]
        s[0,1,2,0]=(1.19169978642)*x[0]+(-1.88274910505)*x[1]+(0.321438561021)*x[2]
        s[0,1,2,1]=(-0.765748081809)*x[0]+(-0.163422665563)*x[1]+(0.256721885141)*x[2]
        s[0,2,0,0]=(-0.863281319991)*x[0]+(-0.021828308413)*x[1]+(0.868312889989)*x[2]
        s[0,2,0,1]=(-0.57479341478)*x[0]+(0.404861121887)*x[1]+(-0.274037007699)*x[2]
        s[0,2,1,0]=(-0.529171478131)*x[0]+(0.836001872018)*x[1]+(-1.12226189724)*x[2]
        s[0,2,1,1]=(-0.821929160411)*x[0]+(0.0466762602824)*x[1]+(-0.0514331500765)*x[2]
        s[0,2,2,0]=(-0.605940618544)*x[0]+(-1.54379919132)*x[1]+(1.63243021269)*x[2]
        s[0,2,2,1]=(-1.49695860655)*x[0]+(-0.601345463475)*x[1]+(-0.753443592162)*x[2]
        s[0,3,0,0]=(1.33514081849)*x[0]+(0.955849232345)*x[1]+(-0.872221354599)*x[2]
        s[0,3,0,1]=(0.939731890901)*x[0]+(-1.16116012822)*x[1]+(-0.601602002743)*x[2]
        s[0,3,1,0]=(-0.675545889094)*x[0]+(0.736755090077)*x[1]+(-0.437370345974)*x[2]
        s[0,3,1,1]=(-1.17899966213)*x[0]+(0.422125349488)*x[1]+(-0.815649229407)*x[2]
        s[0,3,2,0]=(0.0773790438387)*x[0]+(0.148378160687)*x[1]+(-1.0156040466)*x[2]
        s[0,3,2,1]=(0.519422925349)*x[0]+(0.189993892084)*x[1]+(1.59654399673)*x[2]
        s[0,4,0,0]=(-0.0593352727817)*x[0]+(0.0865384232804)*x[1]+(-0.316122229956)*x[2]
        s[0,4,0,1]=(0.935220504817)*x[0]+(0.868143343952)*x[1]+(-0.470286174039)*x[2]
        s[0,4,1,0]=(0.763506086212)*x[0]+(-0.0756795330833)*x[1]+(1.06812672934)*x[2]
        s[0,4,1,1]=(-0.446580256351)*x[0]+(0.653986880233)*x[1]+(0.634364696873)*x[2]
        s[0,4,2,0]=(-0.72871248403)*x[0]+(0.0839150289011)*x[1]+(0.178899292496)*x[2]
        s[0,4,2,1]=(1.30808288445)*x[0]+(0.548931211195)*x[1]+(-0.971585668771)*x[2]
        s[1,0,0,0]=(-0.737171903787)*x[0]+(-0.137110160891)*x[1]+(1.24817883553)*x[2]
        s[1,0,0,1]=(0.589824248199)*x[0]+(-0.191688810075)*x[1]+(1.33820051734)*x[2]
        s[1,0,1,0]=(-0.598063103136)*x[0]+(0.347705502684)*x[1]+(0.490747627936)*x[2]
        s[1,0,1,1]=(0.287829427303)*x[0]+(-0.853875346674)*x[1]+(-0.227722773909)*x[2]
        s[1,0,2,0]=(-1.22183236343)*x[0]+(-0.162406713796)*x[1]+(-0.436433884201)*x[2]
        s[1,0,2,1]=(0.929372688401)*x[0]+(0.469931776805)*x[1]+(0.739110972137)*x[2]
        s[1,1,0,0]=(0.391824586)*x[0]+(-0.551498198462)*x[1]+(-0.588935046796)*x[2]
        s[1,1,0,1]=(0.849805745827)*x[0]+(0.557121315313)*x[1]+(1.80284061615)*x[2]
        s[1,1,1,0]=(-1.62270432051)*x[0]+(-1.55943636923)*x[1]+(0.629204169009)*x[2]
        s[1,1,1,1]=(-0.191271118514)*x[0]+(-0.454718665581)*x[1]+(0.405403171722)*x[2]
        s[1,1,2,0]=(0.461750123507)*x[0]+(0.537594040674)*x[1]+(-0.500046587877)*x[2]
        s[1,1,2,1]=(1.07800193455)*x[0]+(0.0757746249706)*x[1]+(0.358223454759)*x[2]
        s[1,2,0,0]=(-1.31360669142)*x[0]+(0.908413426737)*x[1]+(0.986904165818)*x[2]
        s[1,2,0,1]=(-0.250344247716)*x[0]+(0.596746427887)*x[1]+(0.846846599186)*x[2]
        s[1,2,1,0]=(1.65761753214)*x[0]+(0.0969407749043)*x[1]+(0.197913502281)*x[2]
        s[1,2,1,1]=(0.808454446977)*x[0]+(-0.107176416129)*x[1]+(-0.115562549181)*x[2]
        s[1,2,2,0]=(-0.410108442793)*x[0]+(0.250496842214)*x[1]+(0.403806076052)*x[2]
        s[1,2,2,1]=(1.58110488679)*x[0]+(-0.171583112102)*x[1]+(0.441813895222)*x[2]
        s[1,3,0,0]=(-0.519973841295)*x[0]+(0.14862500493)*x[1]+(-0.0257159960311)*x[2]
        s[1,3,0,1]=(-0.710423889723)*x[0]+(0.0941990549739)*x[1]+(-0.147877062943)*x[2]
        s[1,3,1,0]=(-0.19917762038)*x[0]+(-1.06873791711)*x[1]+(1.37688275432)*x[2]
        s[1,3,1,1]=(-0.570477493665)*x[0]+(0.0705814270652)*x[1]+(-1.10186231057)*x[2]
        s[1,3,2,0]=(-1.04891017002)*x[0]+(-0.136784107636)*x[1]+(-1.65953016693)*x[2]
        s[1,3,2,1]=(-0.674805716398)*x[0]+(-0.211803508128)*x[1]+(-0.116463503974)*x[2]
        s[1,4,0,0]=(0.269217220297)*x[0]+(0.810722194772)*x[1]+(1.60977315112)*x[2]
        s[1,4,0,1]=(-0.0213286766746)*x[0]+(1.0974274553)*x[1]+(0.275633207025)*x[2]
        s[1,4,1,0]=(-0.601230245409)*x[0]+(-0.756418955933)*x[1]+(-0.46194303915)*x[2]
        s[1,4,1,1]=(-0.0907667209143)*x[0]+(-1.86623564673)*x[1]+(0.552729474701)*x[2]
        s[1,4,2,0]=(-1.09694541489)*x[0]+(-0.496193881555)*x[1]+(-0.0126133249168)*x[2]
        s[1,4,2,1]=(-0.648959246633)*x[0]+(-0.60655325639)*x[1]+(0.438258051102)*x[2]
        s[2,0,0,0]=(-0.100512547019)*x[0]+(-1.37291269079)*x[1]+(1.63084319876)*x[2]
        s[2,0,0,1]=(-0.50510326916)*x[0]+(0.308799737351)*x[1]+(1.63506761477)*x[2]
        s[2,0,1,0]=(0.708408519723)*x[0]+(-0.0759833841695)*x[1]+(-0.980974095374)*x[2]
        s[2,0,1,1]=(-1.59727324143)*x[0]+(0.995250195962)*x[1]+(0.169527316083)*x[2]
        s[2,0,2,0]=(0.766935817054)*x[0]+(0.455564399031)*x[1]+(0.605772951782)*x[2]
        s[2,0,2,1]=(-0.820755214894)*x[0]+(-0.43359371038)*x[1]+(-0.0942398466964)*x[2]
        s[2,1,0,0]=(0.390476682801)*x[0]+(1.11627447827)*x[1]+(0.670843405466)*x[2]
        s[2,1,0,1]=(-0.433594870483)*x[0]+(-0.940710004402)*x[1]+(1.10756041487)*x[2]
        s[2,1,1,0]=(-1.21880018868)*x[0]+(0.380422307344)*x[1]+(-0.864017731789)*x[2]
        s[2,1,1,1]=(-0.194168346974)*x[0]+(0.324183983028)*x[1]+(0.741456579501)*x[2]
        s[2,1,2,0]=(-0.134211025912)*x[0]+(0.497311041207)*x[1]+(0.33014196376)*x[2]
        s[2,1,2,1]=(-1.3501981937)*x[0]+(1.67601514859)*x[1]+(-1.60084725544)*x[2]
        s[2,2,0,0]=(-0.8367457236)*x[0]+(-1.15628524562)*x[1]+(0.342983125417)*x[2]
        s[2,2,0,1]=(0.956733778283)*x[0]+(-0.404880336577)*x[1]+(-0.350927185105)*x[2]
        s[2,2,1,0]=(0.854842449281)*x[0]+(-0.185001304765)*x[1]+(1.15245157168)*x[2]
        s[2,2,1,1]=(0.00584831242623)*x[0]+(-0.562950548984)*x[1]+(0.785766513431)*x[2]
        s[2,2,2,0]=(0.272759945164)*x[0]+(-0.568207376307)*x[1]+(-0.155997758854)*x[2]
        s[2,2,2,1]=(-0.105324133175)*x[0]+(1.11807221711)*x[1]+(0.122751975298)*x[2]
        s[2,3,0,0]=(0.685932400265)*x[0]+(-0.666847052302)*x[1]+(0.596321369477)*x[2]
        s[2,3,0,1]=(0.335067153518)*x[0]+(0.268453607431)*x[1]+(0.247881738828)*x[2]
        s[2,3,1,0]=(-1.89084909179)*x[0]+(-0.529751828744)*x[1]+(-0.959358202828)*x[2]
        s[2,3,1,1]=(1.06725689989)*x[0]+(-0.709878517637)*x[1]+(0.968886366535)*x[2]
        s[2,3,2,0]=(-1.63466018652)*x[0]+(0.517481248484)*x[1]+(0.109232845638)*x[2]
        s[2,3,2,1]=(0.0716425266603)*x[0]+(-0.58040972115)*x[1]+(0.346948738894)*x[2]
        s[2,4,0,0]=(0.193599854213)*x[0]+(-1.52744079327)*x[1]+(-0.0377038338183)*x[2]
        s[2,4,0,1]=(-0.0296855161658)*x[0]+(0.266976994363)*x[1]+(-0.832771979863)*x[2]
        s[2,4,1,0]=(-0.885217283694)*x[0]+(0.657425725373)*x[1]+(0.385584782531)*x[2]
        s[2,4,1,1]=(-0.249912733602)*x[0]+(0.156704487203)*x[1]+(0.000393408500501)*x[2]
        s[2,4,2,0]=(-0.083303562971)*x[0]+(0.748009334975)*x[1]+(-0.411323946112)*x[2]
        s[2,4,2,1]=(0.143782087293)*x[0]+(-1.86497054401)*x[1]+(-0.398161751889)*x[2]
        s[3,0,0,0]=(-0.531041035506)*x[0]+(0.998737023878)*x[1]+(-0.719326852701)*x[2]
        s[3,0,0,1]=(-1.10320774278)*x[0]+(-1.27795876669)*x[1]+(0.57451462957)*x[2]
        s[3,0,1,0]=(-0.319275578502)*x[0]+(-1.51721910203)*x[1]+(-0.677470578443)*x[2]
        s[3,0,1,1]=(0.7584807917)*x[0]+(0.0523998746885)*x[1]+(0.502182688228)*x[2]
        s[3,0,2,0]=(-0.905809892125)*x[0]+(0.479751975382)*x[1]+(-0.0778539387977)*x[2]
        s[3,0,2,1]=(-0.724891388357)*x[0]+(0.941493554841)*x[1]+(-1.51514565838)*x[2]
        s[3,1,0,0]=(0.815118793475)*x[0]+(0.0503368907581)*x[1]+(-0.502091253213)*x[2]
        s[3,1,0,1]=(-1.50859117948)*x[0]+(0.31279517213)*x[1]+(-0.339295035594)*x[2]
        s[3,1,1,0]=(0.246825549192)*x[0]+(0.687251908372)*x[1]+(0.133971781776)*x[2]
        s[3,1,1,1]=(0.470105585328)*x[0]+(0.382971101979)*x[1]+(-0.469397699886)*x[2]
        s[3,1,2,0]=(-1.45842815542)*x[0]+(-1.00979255513)*x[1]+(1.45386034482)*x[2]
        s[3,1,2,1]=(0.172130164577)*x[0]+(1.25034739991)*x[1]+(-1.15953423359)*x[2]
        s[3,2,0,0]=(-0.58261393055)*x[0]+(0.730381032663)*x[1]+(1.29377076249)*x[2]
        s[3,2,0,1]=(0.696715362351)*x[0]+(-1.37598351546)*x[1]+(-1.26545085967)*x[2]
        s[3,2,1,0]=(-0.0511282610422)*x[0]+(0.58041679118)*x[1]+(-0.0810357412151)*x[2]
        s[3,2,1,1]=(-0.750057228803)*x[0]+(-0.750703125074)*x[1]+(-1.21748956536)*x[2]
        s[3,2,2,0]=(-0.455746666635)*x[0]+(-0.720020627516)*x[1]+(-0.0911281889971)*x[2]
        s[3,2,2,1]=(-0.0681894163236)*x[0]+(0.101592029278)*x[1]+(1.10954864406)*x[2]
        s[3,3,0,0]=(0.433898443791)*x[0]+(0.901180030496)*x[1]+(0.795641263874)*x[2]
        s[3,3,0,1]=(-0.213306810738)*x[0]+(-0.852539239286)*x[1]+(-1.25905416259)*x[2]
        s[3,3,1,0]=(-0.668307384473)*x[0]+(-1.37753879862)*x[1]+(-0.593731800903)*x[2]
        s[3,3,1,1]=(0.815464294514)*x[0]+(0.835389640785)*x[1]+(0.933029468258)*x[2]
        s[3,3,2,0]=(0.565194274444)*x[0]+(0.584119707001)*x[1]+(-0.53812189952)*x[2]
        s[3,3,2,1]=(-0.903604209314)*x[0]+(0.370887007483)*x[1]+(-0.548135915608)*x[2]
        s[3,4,0,0]=(-0.819290139492)*x[0]+(-0.256800498012)*x[1]+(0.350363336935)*x[2]
        s[3,4,0,1]=(0.863170182723)*x[0]+(0.0101766489337)*x[1]+(1.34009635664)*x[2]
        s[3,4,1,0]=(1.03350734136)*x[0]+(1.37672500233)*x[1]+(1.35134434663)*x[2]
        s[3,4,1,1]=(1.4883144839)*x[0]+(0.410688244039)*x[1]+(0.759267998436)*x[2]
        s[3,4,2,0]=(0.373190738096)*x[0]+(1.22136515279)*x[1]+(-0.0365030690962)*x[2]
        s[3,4,2,1]=(-0.607437402737)*x[0]+(0.750130654364)*x[1]+(0.084778884789)*x[2]
        ref[0,0,0,0]=(0.847845815599)*x_ref[0]+(-0.610234211476)*x_ref[1]+(-0.627625510082)*x_ref[2]
        ref[0,0,0,1]=(0.0126184651041)*x_ref[0]+(-0.0651294063564)*x_ref[1]+(-0.274615935201)*x_ref[2]
        ref[0,0,1,0]=(-0.545925417969)*x_ref[0]+(0.421929302201)*x_ref[1]+(-0.910418625383)*x_ref[2]
        ref[0,0,1,1]=(1.58754252937)*x_ref[0]+(0.110111591047)*x_ref[1]+(-1.64780563736)*x_ref[2]
        ref[0,0,2,0]=(-0.510331643992)*x_ref[0]+(0.827685326542)*x_ref[1]+(1.31447962917)*x_ref[2]
        ref[0,0,2,1]=(-1.29575211142)*x_ref[0]+(-0.318096803401)*x_ref[1]+(-0.274660613777)*x_ref[2]
        ref[0,1,0,0]=(0.77444944835)*x_ref[0]+(-1.0893411122)*x_ref[1]+(0.331702744143)*x_ref[2]
        ref[0,1,0,1]=(0.624693916432)*x_ref[0]+(-0.306552086758)*x_ref[1]+(-0.845524417807)*x_ref[2]
        ref[0,1,1,0]=(0.0219492668609)*x_ref[0]+(-1.13712865204)*x_ref[1]+(-0.193051406533)*x_ref[2]
        ref[0,1,1,1]=(-1.32696889246)*x_ref[0]+(1.63857177904)*x_ref[1]+(0.194095476936)*x_ref[2]
        ref[0,1,2,0]=(1.19169978642)*x_ref[0]+(-1.88274910505)*x_ref[1]+(0.321438561021)*x_ref[2]
        ref[0,1,2,1]=(-0.765748081809)*x_ref[0]+(-0.163422665563)*x_ref[1]+(0.256721885141)*x_ref[2]
        ref[0,2,0,0]=(-0.863281319991)*x_ref[0]+(-0.021828308413)*x_ref[1]+(0.868312889989)*x_ref[2]
        ref[0,2,0,1]=(-0.57479341478)*x_ref[0]+(0.404861121887)*x_ref[1]+(-0.274037007699)*x_ref[2]
        ref[0,2,1,0]=(-0.529171478131)*x_ref[0]+(0.836001872018)*x_ref[1]+(-1.12226189724)*x_ref[2]
        ref[0,2,1,1]=(-0.821929160411)*x_ref[0]+(0.0466762602824)*x_ref[1]+(-0.0514331500765)*x_ref[2]
        ref[0,2,2,0]=(-0.605940618544)*x_ref[0]+(-1.54379919132)*x_ref[1]+(1.63243021269)*x_ref[2]
        ref[0,2,2,1]=(-1.49695860655)*x_ref[0]+(-0.601345463475)*x_ref[1]+(-0.753443592162)*x_ref[2]
        ref[0,3,0,0]=(1.33514081849)*x_ref[0]+(0.955849232345)*x_ref[1]+(-0.872221354599)*x_ref[2]
        ref[0,3,0,1]=(0.939731890901)*x_ref[0]+(-1.16116012822)*x_ref[1]+(-0.601602002743)*x_ref[2]
        ref[0,3,1,0]=(-0.675545889094)*x_ref[0]+(0.736755090077)*x_ref[1]+(-0.437370345974)*x_ref[2]
        ref[0,3,1,1]=(-1.17899966213)*x_ref[0]+(0.422125349488)*x_ref[1]+(-0.815649229407)*x_ref[2]
        ref[0,3,2,0]=(0.0773790438387)*x_ref[0]+(0.148378160687)*x_ref[1]+(-1.0156040466)*x_ref[2]
        ref[0,3,2,1]=(0.519422925349)*x_ref[0]+(0.189993892084)*x_ref[1]+(1.59654399673)*x_ref[2]
        ref[0,4,0,0]=(-0.0593352727817)*x_ref[0]+(0.0865384232804)*x_ref[1]+(-0.316122229956)*x_ref[2]
        ref[0,4,0,1]=(0.935220504817)*x_ref[0]+(0.868143343952)*x_ref[1]+(-0.470286174039)*x_ref[2]
        ref[0,4,1,0]=(0.763506086212)*x_ref[0]+(-0.0756795330833)*x_ref[1]+(1.06812672934)*x_ref[2]
        ref[0,4,1,1]=(-0.446580256351)*x_ref[0]+(0.653986880233)*x_ref[1]+(0.634364696873)*x_ref[2]
        ref[0,4,2,0]=(-0.72871248403)*x_ref[0]+(0.0839150289011)*x_ref[1]+(0.178899292496)*x_ref[2]
        ref[0,4,2,1]=(1.30808288445)*x_ref[0]+(0.548931211195)*x_ref[1]+(-0.971585668771)*x_ref[2]
        ref[1,0,0,0]=(-0.737171903787)*x_ref[0]+(-0.137110160891)*x_ref[1]+(1.24817883553)*x_ref[2]
        ref[1,0,0,1]=(0.589824248199)*x_ref[0]+(-0.191688810075)*x_ref[1]+(1.33820051734)*x_ref[2]
        ref[1,0,1,0]=(-0.598063103136)*x_ref[0]+(0.347705502684)*x_ref[1]+(0.490747627936)*x_ref[2]
        ref[1,0,1,1]=(0.287829427303)*x_ref[0]+(-0.853875346674)*x_ref[1]+(-0.227722773909)*x_ref[2]
        ref[1,0,2,0]=(-1.22183236343)*x_ref[0]+(-0.162406713796)*x_ref[1]+(-0.436433884201)*x_ref[2]
        ref[1,0,2,1]=(0.929372688401)*x_ref[0]+(0.469931776805)*x_ref[1]+(0.739110972137)*x_ref[2]
        ref[1,1,0,0]=(0.391824586)*x_ref[0]+(-0.551498198462)*x_ref[1]+(-0.588935046796)*x_ref[2]
        ref[1,1,0,1]=(0.849805745827)*x_ref[0]+(0.557121315313)*x_ref[1]+(1.80284061615)*x_ref[2]
        ref[1,1,1,0]=(-1.62270432051)*x_ref[0]+(-1.55943636923)*x_ref[1]+(0.629204169009)*x_ref[2]
        ref[1,1,1,1]=(-0.191271118514)*x_ref[0]+(-0.454718665581)*x_ref[1]+(0.405403171722)*x_ref[2]
        ref[1,1,2,0]=(0.461750123507)*x_ref[0]+(0.537594040674)*x_ref[1]+(-0.500046587877)*x_ref[2]
        ref[1,1,2,1]=(1.07800193455)*x_ref[0]+(0.0757746249706)*x_ref[1]+(0.358223454759)*x_ref[2]
        ref[1,2,0,0]=(-1.31360669142)*x_ref[0]+(0.908413426737)*x_ref[1]+(0.986904165818)*x_ref[2]
        ref[1,2,0,1]=(-0.250344247716)*x_ref[0]+(0.596746427887)*x_ref[1]+(0.846846599186)*x_ref[2]
        ref[1,2,1,0]=(1.65761753214)*x_ref[0]+(0.0969407749043)*x_ref[1]+(0.197913502281)*x_ref[2]
        ref[1,2,1,1]=(0.808454446977)*x_ref[0]+(-0.107176416129)*x_ref[1]+(-0.115562549181)*x_ref[2]
        ref[1,2,2,0]=(-0.410108442793)*x_ref[0]+(0.250496842214)*x_ref[1]+(0.403806076052)*x_ref[2]
        ref[1,2,2,1]=(1.58110488679)*x_ref[0]+(-0.171583112102)*x_ref[1]+(0.441813895222)*x_ref[2]
        ref[1,3,0,0]=(-0.519973841295)*x_ref[0]+(0.14862500493)*x_ref[1]+(-0.0257159960311)*x_ref[2]
        ref[1,3,0,1]=(-0.710423889723)*x_ref[0]+(0.0941990549739)*x_ref[1]+(-0.147877062943)*x_ref[2]
        ref[1,3,1,0]=(-0.19917762038)*x_ref[0]+(-1.06873791711)*x_ref[1]+(1.37688275432)*x_ref[2]
        ref[1,3,1,1]=(-0.570477493665)*x_ref[0]+(0.0705814270652)*x_ref[1]+(-1.10186231057)*x_ref[2]
        ref[1,3,2,0]=(-1.04891017002)*x_ref[0]+(-0.136784107636)*x_ref[1]+(-1.65953016693)*x_ref[2]
        ref[1,3,2,1]=(-0.674805716398)*x_ref[0]+(-0.211803508128)*x_ref[1]+(-0.116463503974)*x_ref[2]
        ref[1,4,0,0]=(0.269217220297)*x_ref[0]+(0.810722194772)*x_ref[1]+(1.60977315112)*x_ref[2]
        ref[1,4,0,1]=(-0.0213286766746)*x_ref[0]+(1.0974274553)*x_ref[1]+(0.275633207025)*x_ref[2]
        ref[1,4,1,0]=(-0.601230245409)*x_ref[0]+(-0.756418955933)*x_ref[1]+(-0.46194303915)*x_ref[2]
        ref[1,4,1,1]=(-0.0907667209143)*x_ref[0]+(-1.86623564673)*x_ref[1]+(0.552729474701)*x_ref[2]
        ref[1,4,2,0]=(-1.09694541489)*x_ref[0]+(-0.496193881555)*x_ref[1]+(-0.0126133249168)*x_ref[2]
        ref[1,4,2,1]=(-0.648959246633)*x_ref[0]+(-0.60655325639)*x_ref[1]+(0.438258051102)*x_ref[2]
        ref[2,0,0,0]=(-0.100512547019)*x_ref[0]+(-1.37291269079)*x_ref[1]+(1.63084319876)*x_ref[2]
        ref[2,0,0,1]=(-0.50510326916)*x_ref[0]+(0.308799737351)*x_ref[1]+(1.63506761477)*x_ref[2]
        ref[2,0,1,0]=(0.708408519723)*x_ref[0]+(-0.0759833841695)*x_ref[1]+(-0.980974095374)*x_ref[2]
        ref[2,0,1,1]=(-1.59727324143)*x_ref[0]+(0.995250195962)*x_ref[1]+(0.169527316083)*x_ref[2]
        ref[2,0,2,0]=(0.766935817054)*x_ref[0]+(0.455564399031)*x_ref[1]+(0.605772951782)*x_ref[2]
        ref[2,0,2,1]=(-0.820755214894)*x_ref[0]+(-0.43359371038)*x_ref[1]+(-0.0942398466964)*x_ref[2]
        ref[2,1,0,0]=(0.390476682801)*x_ref[0]+(1.11627447827)*x_ref[1]+(0.670843405466)*x_ref[2]
        ref[2,1,0,1]=(-0.433594870483)*x_ref[0]+(-0.940710004402)*x_ref[1]+(1.10756041487)*x_ref[2]
        ref[2,1,1,0]=(-1.21880018868)*x_ref[0]+(0.380422307344)*x_ref[1]+(-0.864017731789)*x_ref[2]
        ref[2,1,1,1]=(-0.194168346974)*x_ref[0]+(0.324183983028)*x_ref[1]+(0.741456579501)*x_ref[2]
        ref[2,1,2,0]=(-0.134211025912)*x_ref[0]+(0.497311041207)*x_ref[1]+(0.33014196376)*x_ref[2]
        ref[2,1,2,1]=(-1.3501981937)*x_ref[0]+(1.67601514859)*x_ref[1]+(-1.60084725544)*x_ref[2]
        ref[2,2,0,0]=(-0.8367457236)*x_ref[0]+(-1.15628524562)*x_ref[1]+(0.342983125417)*x_ref[2]
        ref[2,2,0,1]=(0.956733778283)*x_ref[0]+(-0.404880336577)*x_ref[1]+(-0.350927185105)*x_ref[2]
        ref[2,2,1,0]=(0.854842449281)*x_ref[0]+(-0.185001304765)*x_ref[1]+(1.15245157168)*x_ref[2]
        ref[2,2,1,1]=(0.00584831242623)*x_ref[0]+(-0.562950548984)*x_ref[1]+(0.785766513431)*x_ref[2]
        ref[2,2,2,0]=(0.272759945164)*x_ref[0]+(-0.568207376307)*x_ref[1]+(-0.155997758854)*x_ref[2]
        ref[2,2,2,1]=(-0.105324133175)*x_ref[0]+(1.11807221711)*x_ref[1]+(0.122751975298)*x_ref[2]
        ref[2,3,0,0]=(0.685932400265)*x_ref[0]+(-0.666847052302)*x_ref[1]+(0.596321369477)*x_ref[2]
        ref[2,3,0,1]=(0.335067153518)*x_ref[0]+(0.268453607431)*x_ref[1]+(0.247881738828)*x_ref[2]
        ref[2,3,1,0]=(-1.89084909179)*x_ref[0]+(-0.529751828744)*x_ref[1]+(-0.959358202828)*x_ref[2]
        ref[2,3,1,1]=(1.06725689989)*x_ref[0]+(-0.709878517637)*x_ref[1]+(0.968886366535)*x_ref[2]
        ref[2,3,2,0]=(-1.63466018652)*x_ref[0]+(0.517481248484)*x_ref[1]+(0.109232845638)*x_ref[2]
        ref[2,3,2,1]=(0.0716425266603)*x_ref[0]+(-0.58040972115)*x_ref[1]+(0.346948738894)*x_ref[2]
        ref[2,4,0,0]=(0.193599854213)*x_ref[0]+(-1.52744079327)*x_ref[1]+(-0.0377038338183)*x_ref[2]
        ref[2,4,0,1]=(-0.0296855161658)*x_ref[0]+(0.266976994363)*x_ref[1]+(-0.832771979863)*x_ref[2]
        ref[2,4,1,0]=(-0.885217283694)*x_ref[0]+(0.657425725373)*x_ref[1]+(0.385584782531)*x_ref[2]
        ref[2,4,1,1]=(-0.249912733602)*x_ref[0]+(0.156704487203)*x_ref[1]+(0.000393408500501)*x_ref[2]
        ref[2,4,2,0]=(-0.083303562971)*x_ref[0]+(0.748009334975)*x_ref[1]+(-0.411323946112)*x_ref[2]
        ref[2,4,2,1]=(0.143782087293)*x_ref[0]+(-1.86497054401)*x_ref[1]+(-0.398161751889)*x_ref[2]
        ref[3,0,0,0]=(-0.531041035506)*x_ref[0]+(0.998737023878)*x_ref[1]+(-0.719326852701)*x_ref[2]
        ref[3,0,0,1]=(-1.10320774278)*x_ref[0]+(-1.27795876669)*x_ref[1]+(0.57451462957)*x_ref[2]
        ref[3,0,1,0]=(-0.319275578502)*x_ref[0]+(-1.51721910203)*x_ref[1]+(-0.677470578443)*x_ref[2]
        ref[3,0,1,1]=(0.7584807917)*x_ref[0]+(0.0523998746885)*x_ref[1]+(0.502182688228)*x_ref[2]
        ref[3,0,2,0]=(-0.905809892125)*x_ref[0]+(0.479751975382)*x_ref[1]+(-0.0778539387977)*x_ref[2]
        ref[3,0,2,1]=(-0.724891388357)*x_ref[0]+(0.941493554841)*x_ref[1]+(-1.51514565838)*x_ref[2]
        ref[3,1,0,0]=(0.815118793475)*x_ref[0]+(0.0503368907581)*x_ref[1]+(-0.502091253213)*x_ref[2]
        ref[3,1,0,1]=(-1.50859117948)*x_ref[0]+(0.31279517213)*x_ref[1]+(-0.339295035594)*x_ref[2]
        ref[3,1,1,0]=(0.246825549192)*x_ref[0]+(0.687251908372)*x_ref[1]+(0.133971781776)*x_ref[2]
        ref[3,1,1,1]=(0.470105585328)*x_ref[0]+(0.382971101979)*x_ref[1]+(-0.469397699886)*x_ref[2]
        ref[3,1,2,0]=(-1.45842815542)*x_ref[0]+(-1.00979255513)*x_ref[1]+(1.45386034482)*x_ref[2]
        ref[3,1,2,1]=(0.172130164577)*x_ref[0]+(1.25034739991)*x_ref[1]+(-1.15953423359)*x_ref[2]
        ref[3,2,0,0]=(-0.58261393055)*x_ref[0]+(0.730381032663)*x_ref[1]+(1.29377076249)*x_ref[2]
        ref[3,2,0,1]=(0.696715362351)*x_ref[0]+(-1.37598351546)*x_ref[1]+(-1.26545085967)*x_ref[2]
        ref[3,2,1,0]=(-0.0511282610422)*x_ref[0]+(0.58041679118)*x_ref[1]+(-0.0810357412151)*x_ref[2]
        ref[3,2,1,1]=(-0.750057228803)*x_ref[0]+(-0.750703125074)*x_ref[1]+(-1.21748956536)*x_ref[2]
        ref[3,2,2,0]=(-0.455746666635)*x_ref[0]+(-0.720020627516)*x_ref[1]+(-0.0911281889971)*x_ref[2]
        ref[3,2,2,1]=(-0.0681894163236)*x_ref[0]+(0.101592029278)*x_ref[1]+(1.10954864406)*x_ref[2]
        ref[3,3,0,0]=(0.433898443791)*x_ref[0]+(0.901180030496)*x_ref[1]+(0.795641263874)*x_ref[2]
        ref[3,3,0,1]=(-0.213306810738)*x_ref[0]+(-0.852539239286)*x_ref[1]+(-1.25905416259)*x_ref[2]
        ref[3,3,1,0]=(-0.668307384473)*x_ref[0]+(-1.37753879862)*x_ref[1]+(-0.593731800903)*x_ref[2]
        ref[3,3,1,1]=(0.815464294514)*x_ref[0]+(0.835389640785)*x_ref[1]+(0.933029468258)*x_ref[2]
        ref[3,3,2,0]=(0.565194274444)*x_ref[0]+(0.584119707001)*x_ref[1]+(-0.53812189952)*x_ref[2]
        ref[3,3,2,1]=(-0.903604209314)*x_ref[0]+(0.370887007483)*x_ref[1]+(-0.548135915608)*x_ref[2]
        ref[3,4,0,0]=(-0.819290139492)*x_ref[0]+(-0.256800498012)*x_ref[1]+(0.350363336935)*x_ref[2]
        ref[3,4,0,1]=(0.863170182723)*x_ref[0]+(0.0101766489337)*x_ref[1]+(1.34009635664)*x_ref[2]
        ref[3,4,1,0]=(1.03350734136)*x_ref[0]+(1.37672500233)*x_ref[1]+(1.35134434663)*x_ref[2]
        ref[3,4,1,1]=(1.4883144839)*x_ref[0]+(0.410688244039)*x_ref[1]+(0.759267998436)*x_ref[2]
        ref[3,4,2,0]=(0.373190738096)*x_ref[0]+(1.22136515279)*x_ref[1]+(-0.0365030690962)*x_ref[2]
        ref[3,4,2,1]=(-0.607437402737)*x_ref[0]+(0.750130654364)*x_ref[1]+(0.084778884789)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_ReducedFunctionOnContactZero_rank0(self):
      """
      tests interpolate for rank 0 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactZero onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactZero(self.domain)
      x=w.getX()
      arg=Symbol(shape=(),dim=dim)
      s=Data(0,(),w)
      ref=Data(0,(),w_ref)
      if dim==2:
        s=(-0.478526643653)*x[0]**o+(-0.798079781236)*x[0]+(-0.499553919498)*x[1]**o+(0.336723069414)*x[1]
        ref=(-0.478526643653)*x_ref[0]**o+(-0.798079781236)*x_ref[0]+(-0.499553919498)*x_ref[1]**o+(0.336723069414)*x_ref[1]
      else:
        s=(-0.35871218678)*x[0]**o+(0.954679093985)*x[0]+(-0.127722494434)*x[1]**o+(0.0119904093364)*x[1]+(-0.647695475743)*x[2]**o+(-0.522381915873)*x[2]
        ref=(-0.35871218678)*x_ref[0]**o+(0.954679093985)*x_ref[0]+(-0.127722494434)*x_ref[1]**o+(0.0119904093364)*x_ref[1]+(-0.647695475743)*x_ref[2]**o+(-0.522381915873)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_ReducedFunctionOnContactZero_rank1(self):
      """
      tests interpolate for rank 1 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactZero onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactZero(self.domain)
      x=w.getX()
      arg=Symbol(shape=(2,),dim=dim)
      s=Data(0,(2,),w)
      ref=Data(0,(2,),w_ref)
      if dim==2:
        s[0]=(0.0333528524993)*x[0]**o+(-0.534169045864)*x[0]+(0.234758321366)*x[1]**o+(0.395702014106)*x[1]
        s[1]=(-0.559512507686)*x[0]**o+(0.473831121701)*x[0]+(0.315500482589)*x[1]**o+(0.659175367015)*x[1]
        ref[0]=(0.0333528524993)*x_ref[0]**o+(-0.534169045864)*x_ref[0]+(0.234758321366)*x_ref[1]**o+(0.395702014106)*x_ref[1]
        ref[1]=(-0.559512507686)*x_ref[0]**o+(0.473831121701)*x_ref[0]+(0.315500482589)*x_ref[1]**o+(0.659175367015)*x_ref[1]
      else:
        s[0]=(0.920431380976)*x[0]**o+(0.203182015633)*x[0]+(-0.30119291643)*x[1]**o+(0.743123673038)*x[1]+(0.158345217723)*x[2]**o+(-0.106151059285)*x[2]
        s[1]=(-0.246067927964)*x[0]**o+(0.249705802315)*x[0]+(0.87511193184)*x[1]**o+(0.174791560577)*x[1]+(-0.717945042302)*x[2]**o+(0.558392243221)*x[2]
        ref[0]=(0.920431380976)*x_ref[0]**o+(0.203182015633)*x_ref[0]+(-0.30119291643)*x_ref[1]**o+(0.743123673038)*x_ref[1]+(0.158345217723)*x_ref[2]**o+(-0.106151059285)*x_ref[2]
        ref[1]=(-0.246067927964)*x_ref[0]**o+(0.249705802315)*x_ref[0]+(0.87511193184)*x_ref[1]**o+(0.174791560577)*x_ref[1]+(-0.717945042302)*x_ref[2]**o+(0.558392243221)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_ReducedFunctionOnContactZero_rank2(self):
      """
      tests interpolate for rank 2 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactZero onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactZero(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5),dim=dim)
      s=Data(0,(4, 5),w)
      ref=Data(0,(4, 5),w_ref)
      if dim==2:
        s[0,0]=(-0.238674589068)*x[0]**o+(0.845146310144)*x[0]+(0.51393009023)*x[1]**o+(0.987659698068)*x[1]
        s[0,1]=(0.525738867968)*x[0]**o+(-0.541924459594)*x[0]+(-0.345083945286)*x[1]**o+(0.892536321695)*x[1]
        s[0,2]=(0.467434483897)*x[0]**o+(0.673356246793)*x[0]+(0.373066330502)*x[1]**o+(0.455273174229)*x[1]
        s[0,3]=(-0.39771044233)*x[0]**o+(0.890234781078)*x[0]+(-0.11624722564)*x[1]**o+(0.995857794085)*x[1]
        s[0,4]=(0.795987536438)*x[0]**o+(-0.683413643902)*x[0]+(-0.0517246803869)*x[1]**o+(0.149282877675)*x[1]
        s[1,0]=(0.215953459935)*x[0]**o+(0.0124143229743)*x[0]+(0.719573519094)*x[1]**o+(0.120838088026)*x[1]
        s[1,1]=(0.195894110038)*x[0]**o+(-0.481212957341)*x[0]+(-0.579369726661)*x[1]**o+(-0.825868817932)*x[1]
        s[1,2]=(0.242091565308)*x[0]**o+(-0.342866600818)*x[0]+(0.871948019343)*x[1]**o+(0.925599757496)*x[1]
        s[1,3]=(-0.228955457627)*x[0]**o+(-0.618007692175)*x[0]+(0.641736066096)*x[1]**o+(-0.648940539231)*x[1]
        s[1,4]=(0.949258368604)*x[0]**o+(0.46037118633)*x[0]+(0.264262232255)*x[1]**o+(-0.204687228041)*x[1]
        s[2,0]=(0.201719829764)*x[0]**o+(-0.758921537456)*x[0]+(0.809565809407)*x[1]**o+(0.18932187303)*x[1]
        s[2,1]=(0.0400837555441)*x[0]**o+(0.703393740665)*x[0]+(0.989844901722)*x[1]**o+(-0.920311703188)*x[1]
        s[2,2]=(-0.1127643095)*x[0]**o+(-0.181508597843)*x[0]+(0.845854218374)*x[1]**o+(-0.401564965436)*x[1]
        s[2,3]=(0.725120155616)*x[0]**o+(0.0408162472823)*x[0]+(0.849016789586)*x[1]**o+(-0.0894725097851)*x[1]
        s[2,4]=(0.00712438942747)*x[0]**o+(0.430728482242)*x[0]+(-0.630571782961)*x[1]**o+(-0.292289123038)*x[1]
        s[3,0]=(0.604980765238)*x[0]**o+(-0.545924520431)*x[0]+(0.365986250184)*x[1]**o+(-0.717572291319)*x[1]
        s[3,1]=(-0.105418444539)*x[0]**o+(-0.301186945237)*x[0]+(0.298798693172)*x[1]**o+(0.305159917178)*x[1]
        s[3,2]=(-0.116290679999)*x[0]**o+(-0.814717358777)*x[0]+(0.463687518523)*x[1]**o+(-0.142856072677)*x[1]
        s[3,3]=(-0.0933248161262)*x[0]**o+(-0.745036857332)*x[0]+(-0.143025048876)*x[1]**o+(-0.317322119337)*x[1]
        s[3,4]=(0.340645005623)*x[0]**o+(-0.369235909154)*x[0]+(-0.739698569483)*x[1]**o+(0.296590339275)*x[1]
        ref[0,0]=(-0.238674589068)*x_ref[0]**o+(0.845146310144)*x_ref[0]+(0.51393009023)*x_ref[1]**o+(0.987659698068)*x_ref[1]
        ref[0,1]=(0.525738867968)*x_ref[0]**o+(-0.541924459594)*x_ref[0]+(-0.345083945286)*x_ref[1]**o+(0.892536321695)*x_ref[1]
        ref[0,2]=(0.467434483897)*x_ref[0]**o+(0.673356246793)*x_ref[0]+(0.373066330502)*x_ref[1]**o+(0.455273174229)*x_ref[1]
        ref[0,3]=(-0.39771044233)*x_ref[0]**o+(0.890234781078)*x_ref[0]+(-0.11624722564)*x_ref[1]**o+(0.995857794085)*x_ref[1]
        ref[0,4]=(0.795987536438)*x_ref[0]**o+(-0.683413643902)*x_ref[0]+(-0.0517246803869)*x_ref[1]**o+(0.149282877675)*x_ref[1]
        ref[1,0]=(0.215953459935)*x_ref[0]**o+(0.0124143229743)*x_ref[0]+(0.719573519094)*x_ref[1]**o+(0.120838088026)*x_ref[1]
        ref[1,1]=(0.195894110038)*x_ref[0]**o+(-0.481212957341)*x_ref[0]+(-0.579369726661)*x_ref[1]**o+(-0.825868817932)*x_ref[1]
        ref[1,2]=(0.242091565308)*x_ref[0]**o+(-0.342866600818)*x_ref[0]+(0.871948019343)*x_ref[1]**o+(0.925599757496)*x_ref[1]
        ref[1,3]=(-0.228955457627)*x_ref[0]**o+(-0.618007692175)*x_ref[0]+(0.641736066096)*x_ref[1]**o+(-0.648940539231)*x_ref[1]
        ref[1,4]=(0.949258368604)*x_ref[0]**o+(0.46037118633)*x_ref[0]+(0.264262232255)*x_ref[1]**o+(-0.204687228041)*x_ref[1]
        ref[2,0]=(0.201719829764)*x_ref[0]**o+(-0.758921537456)*x_ref[0]+(0.809565809407)*x_ref[1]**o+(0.18932187303)*x_ref[1]
        ref[2,1]=(0.0400837555441)*x_ref[0]**o+(0.703393740665)*x_ref[0]+(0.989844901722)*x_ref[1]**o+(-0.920311703188)*x_ref[1]
        ref[2,2]=(-0.1127643095)*x_ref[0]**o+(-0.181508597843)*x_ref[0]+(0.845854218374)*x_ref[1]**o+(-0.401564965436)*x_ref[1]
        ref[2,3]=(0.725120155616)*x_ref[0]**o+(0.0408162472823)*x_ref[0]+(0.849016789586)*x_ref[1]**o+(-0.0894725097851)*x_ref[1]
        ref[2,4]=(0.00712438942747)*x_ref[0]**o+(0.430728482242)*x_ref[0]+(-0.630571782961)*x_ref[1]**o+(-0.292289123038)*x_ref[1]
        ref[3,0]=(0.604980765238)*x_ref[0]**o+(-0.545924520431)*x_ref[0]+(0.365986250184)*x_ref[1]**o+(-0.717572291319)*x_ref[1]
        ref[3,1]=(-0.105418444539)*x_ref[0]**o+(-0.301186945237)*x_ref[0]+(0.298798693172)*x_ref[1]**o+(0.305159917178)*x_ref[1]
        ref[3,2]=(-0.116290679999)*x_ref[0]**o+(-0.814717358777)*x_ref[0]+(0.463687518523)*x_ref[1]**o+(-0.142856072677)*x_ref[1]
        ref[3,3]=(-0.0933248161262)*x_ref[0]**o+(-0.745036857332)*x_ref[0]+(-0.143025048876)*x_ref[1]**o+(-0.317322119337)*x_ref[1]
        ref[3,4]=(0.340645005623)*x_ref[0]**o+(-0.369235909154)*x_ref[0]+(-0.739698569483)*x_ref[1]**o+(0.296590339275)*x_ref[1]
      else:
        s[0,0]=(-0.343988908393)*x[0]**o+(-0.249046543972)*x[0]+(-0.94669255683)*x[1]**o+(-0.125519622379)*x[1]+(-0.238439423996)*x[2]**o+(-0.270484426474)*x[2]
        s[0,1]=(-0.81960628266)*x[0]**o+(-0.752592323006)*x[0]+(-0.608566310221)*x[1]**o+(0.136425226211)*x[1]+(-0.417282843542)*x[2]**o+(0.510888775001)*x[2]
        s[0,2]=(-0.469706099858)*x[0]**o+(0.948757046189)*x[0]+(-0.752290984326)*x[1]**o+(0.423515758928)*x[1]+(0.031409477618)*x[2]**o+(0.234409805802)*x[2]
        s[0,3]=(-0.900755988795)*x[0]**o+(-0.767761655222)*x[0]+(0.569180403754)*x[1]**o+(-0.233362758546)*x[1]+(0.569347237378)*x[2]**o+(0.612139565636)*x[2]
        s[0,4]=(0.885682511121)*x[0]**o+(0.332415602786)*x[0]+(0.350335006384)*x[1]**o+(0.613490643585)*x[1]+(0.444593860725)*x[2]**o+(-0.000225980787608)*x[2]
        s[1,0]=(-0.584959823379)*x[0]**o+(-0.964957485122)*x[0]+(0.289141634104)*x[1]**o+(0.263120358818)*x[1]+(-0.028745699583)*x[2]**o+(0.982902290525)*x[2]
        s[1,1]=(-0.191593204495)*x[0]**o+(-0.604908583526)*x[0]+(0.804988763905)*x[1]**o+(0.741296673588)*x[1]+(0.808775189028)*x[2]**o+(0.320306927549)*x[2]
        s[1,2]=(0.658017563103)*x[0]**o+(0.150282440579)*x[0]+(0.988184604156)*x[1]**o+(-0.168752859833)*x[1]+(0.0248149744777)*x[2]**o+(0.158431888663)*x[2]
        s[1,3]=(0.482286527364)*x[0]**o+(0.794842099146)*x[0]+(-0.103524158001)*x[1]**o+(0.834955012893)*x[1]+(-0.724721525621)*x[2]**o+(-0.581451689942)*x[2]
        s[1,4]=(0.987701027083)*x[0]**o+(0.358477587611)*x[0]+(0.0447031116415)*x[1]**o+(-0.63585414781)*x[1]+(-0.493573694258)*x[2]**o+(-0.670666093134)*x[2]
        s[2,0]=(0.156348690681)*x[0]**o+(0.181907398102)*x[0]+(0.810783963046)*x[1]**o+(-0.0420809525478)*x[1]+(0.455236439578)*x[2]**o+(0.68359015036)*x[2]
        s[2,1]=(-0.277203778658)*x[0]**o+(0.906301662318)*x[0]+(0.507233384623)*x[1]**o+(-0.905644178764)*x[1]+(-0.116730257787)*x[2]**o+(-0.205065773315)*x[2]
        s[2,2]=(-0.951427408426)*x[0]**o+(0.418017573994)*x[0]+(-0.707741913019)*x[1]**o+(-0.517554096797)*x[1]+(-0.612052430927)*x[2]**o+(-0.831045221937)*x[2]
        s[2,3]=(0.0142781625544)*x[0]**o+(-0.0207359350934)*x[0]+(0.69086283301)*x[1]**o+(-0.487396868956)*x[1]+(0.868531537118)*x[2]**o+(-0.269368930177)*x[2]
        s[2,4]=(-0.657537261147)*x[0]**o+(0.411006377043)*x[0]+(0.641800881279)*x[1]**o+(-0.179106096535)*x[1]+(0.413169503059)*x[2]**o+(0.0796148767789)*x[2]
        s[3,0]=(-0.0465615058741)*x[0]**o+(0.497229745315)*x[0]+(0.048895565684)*x[1]**o+(0.881074858803)*x[1]+(-0.0904556886937)*x[2]**o+(-0.0486834982676)*x[2]
        s[3,1]=(0.472154117103)*x[0]**o+(-0.841976825688)*x[0]+(-0.840760456295)*x[1]**o+(-0.461023286336)*x[1]+(0.680506149471)*x[2]**o+(0.503845111856)*x[2]
        s[3,2]=(0.166611223306)*x[0]**o+(0.308064312352)*x[0]+(0.850239399334)*x[1]**o+(0.127827758007)*x[1]+(0.306675924384)*x[2]**o+(-0.35563117529)*x[2]
        s[3,3]=(0.0854397842075)*x[0]**o+(0.696191334187)*x[0]+(0.719296439336)*x[1]**o+(0.703852484675)*x[1]+(-0.841440898463)*x[2]**o+(-0.688038437262)*x[2]
        s[3,4]=(0.775596800509)*x[0]**o+(-0.0380327958147)*x[0]+(0.0582036420384)*x[1]**o+(0.0396609528486)*x[1]+(-0.449051573608)*x[2]**o+(0.139175147549)*x[2]
        ref[0,0]=(-0.343988908393)*x_ref[0]**o+(-0.249046543972)*x_ref[0]+(-0.94669255683)*x_ref[1]**o+(-0.125519622379)*x_ref[1]+(-0.238439423996)*x_ref[2]**o+(-0.270484426474)*x_ref[2]
        ref[0,1]=(-0.81960628266)*x_ref[0]**o+(-0.752592323006)*x_ref[0]+(-0.608566310221)*x_ref[1]**o+(0.136425226211)*x_ref[1]+(-0.417282843542)*x_ref[2]**o+(0.510888775001)*x_ref[2]
        ref[0,2]=(-0.469706099858)*x_ref[0]**o+(0.948757046189)*x_ref[0]+(-0.752290984326)*x_ref[1]**o+(0.423515758928)*x_ref[1]+(0.031409477618)*x_ref[2]**o+(0.234409805802)*x_ref[2]
        ref[0,3]=(-0.900755988795)*x_ref[0]**o+(-0.767761655222)*x_ref[0]+(0.569180403754)*x_ref[1]**o+(-0.233362758546)*x_ref[1]+(0.569347237378)*x_ref[2]**o+(0.612139565636)*x_ref[2]
        ref[0,4]=(0.885682511121)*x_ref[0]**o+(0.332415602786)*x_ref[0]+(0.350335006384)*x_ref[1]**o+(0.613490643585)*x_ref[1]+(0.444593860725)*x_ref[2]**o+(-0.000225980787608)*x_ref[2]
        ref[1,0]=(-0.584959823379)*x_ref[0]**o+(-0.964957485122)*x_ref[0]+(0.289141634104)*x_ref[1]**o+(0.263120358818)*x_ref[1]+(-0.028745699583)*x_ref[2]**o+(0.982902290525)*x_ref[2]
        ref[1,1]=(-0.191593204495)*x_ref[0]**o+(-0.604908583526)*x_ref[0]+(0.804988763905)*x_ref[1]**o+(0.741296673588)*x_ref[1]+(0.808775189028)*x_ref[2]**o+(0.320306927549)*x_ref[2]
        ref[1,2]=(0.658017563103)*x_ref[0]**o+(0.150282440579)*x_ref[0]+(0.988184604156)*x_ref[1]**o+(-0.168752859833)*x_ref[1]+(0.0248149744777)*x_ref[2]**o+(0.158431888663)*x_ref[2]
        ref[1,3]=(0.482286527364)*x_ref[0]**o+(0.794842099146)*x_ref[0]+(-0.103524158001)*x_ref[1]**o+(0.834955012893)*x_ref[1]+(-0.724721525621)*x_ref[2]**o+(-0.581451689942)*x_ref[2]
        ref[1,4]=(0.987701027083)*x_ref[0]**o+(0.358477587611)*x_ref[0]+(0.0447031116415)*x_ref[1]**o+(-0.63585414781)*x_ref[1]+(-0.493573694258)*x_ref[2]**o+(-0.670666093134)*x_ref[2]
        ref[2,0]=(0.156348690681)*x_ref[0]**o+(0.181907398102)*x_ref[0]+(0.810783963046)*x_ref[1]**o+(-0.0420809525478)*x_ref[1]+(0.455236439578)*x_ref[2]**o+(0.68359015036)*x_ref[2]
        ref[2,1]=(-0.277203778658)*x_ref[0]**o+(0.906301662318)*x_ref[0]+(0.507233384623)*x_ref[1]**o+(-0.905644178764)*x_ref[1]+(-0.116730257787)*x_ref[2]**o+(-0.205065773315)*x_ref[2]
        ref[2,2]=(-0.951427408426)*x_ref[0]**o+(0.418017573994)*x_ref[0]+(-0.707741913019)*x_ref[1]**o+(-0.517554096797)*x_ref[1]+(-0.612052430927)*x_ref[2]**o+(-0.831045221937)*x_ref[2]
        ref[2,3]=(0.0142781625544)*x_ref[0]**o+(-0.0207359350934)*x_ref[0]+(0.69086283301)*x_ref[1]**o+(-0.487396868956)*x_ref[1]+(0.868531537118)*x_ref[2]**o+(-0.269368930177)*x_ref[2]
        ref[2,4]=(-0.657537261147)*x_ref[0]**o+(0.411006377043)*x_ref[0]+(0.641800881279)*x_ref[1]**o+(-0.179106096535)*x_ref[1]+(0.413169503059)*x_ref[2]**o+(0.0796148767789)*x_ref[2]
        ref[3,0]=(-0.0465615058741)*x_ref[0]**o+(0.497229745315)*x_ref[0]+(0.048895565684)*x_ref[1]**o+(0.881074858803)*x_ref[1]+(-0.0904556886937)*x_ref[2]**o+(-0.0486834982676)*x_ref[2]
        ref[3,1]=(0.472154117103)*x_ref[0]**o+(-0.841976825688)*x_ref[0]+(-0.840760456295)*x_ref[1]**o+(-0.461023286336)*x_ref[1]+(0.680506149471)*x_ref[2]**o+(0.503845111856)*x_ref[2]
        ref[3,2]=(0.166611223306)*x_ref[0]**o+(0.308064312352)*x_ref[0]+(0.850239399334)*x_ref[1]**o+(0.127827758007)*x_ref[1]+(0.306675924384)*x_ref[2]**o+(-0.35563117529)*x_ref[2]
        ref[3,3]=(0.0854397842075)*x_ref[0]**o+(0.696191334187)*x_ref[0]+(0.719296439336)*x_ref[1]**o+(0.703852484675)*x_ref[1]+(-0.841440898463)*x_ref[2]**o+(-0.688038437262)*x_ref[2]
        ref[3,4]=(0.775596800509)*x_ref[0]**o+(-0.0380327958147)*x_ref[0]+(0.0582036420384)*x_ref[1]**o+(0.0396609528486)*x_ref[1]+(-0.449051573608)*x_ref[2]**o+(0.139175147549)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_ReducedFunctionOnContactZero_rank3(self):
      """
      tests interpolate for rank 3 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactZero onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactZero(self.domain)
      x=w.getX()
      arg=Symbol(shape=(6, 2, 2),dim=dim)
      s=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2),w_ref)
      if dim==2:
        s[0,0,0]=(0.748658213139)*x[0]**o+(0.588706109747)*x[0]+(-0.462395788697)*x[1]**o+(0.558348215492)*x[1]
        s[0,0,1]=(0.629957777572)*x[0]**o+(0.714493090646)*x[0]+(-0.491054832228)*x[1]**o+(0.710860562012)*x[1]
        s[0,1,0]=(-0.775354422191)*x[0]**o+(0.152096744898)*x[0]+(0.717568022676)*x[1]**o+(0.749449532926)*x[1]
        s[0,1,1]=(0.883871444687)*x[0]**o+(0.149367558371)*x[0]+(0.986279594706)*x[1]**o+(0.537229955955)*x[1]
        s[1,0,0]=(0.564688441996)*x[0]**o+(-0.206862774902)*x[0]+(0.522967069439)*x[1]**o+(-0.0390132413525)*x[1]
        s[1,0,1]=(0.706775860383)*x[0]**o+(0.973359689319)*x[0]+(0.567062178556)*x[1]**o+(-0.00264874871685)*x[1]
        s[1,1,0]=(-0.385131437493)*x[0]**o+(0.93223208004)*x[0]+(0.915604601744)*x[1]**o+(-0.604136431942)*x[1]
        s[1,1,1]=(-0.58509656422)*x[0]**o+(-0.46833032365)*x[0]+(0.143097299013)*x[1]**o+(-0.685071549088)*x[1]
        s[2,0,0]=(-0.987452159168)*x[0]**o+(0.609343916634)*x[0]+(0.425443014801)*x[1]**o+(0.58099336693)*x[1]
        s[2,0,1]=(0.957582337028)*x[0]**o+(0.62772269136)*x[0]+(-0.783891315097)*x[1]**o+(-0.77280642716)*x[1]
        s[2,1,0]=(0.327068296677)*x[0]**o+(0.0579535708335)*x[0]+(-0.798372147412)*x[1]**o+(-0.517396772711)*x[1]
        s[2,1,1]=(-0.863293845828)*x[0]**o+(0.688257458796)*x[0]+(0.777044937444)*x[1]**o+(0.210894669552)*x[1]
        s[3,0,0]=(0.641389559724)*x[0]**o+(-0.115831890978)*x[0]+(0.0852026159583)*x[1]**o+(0.101596282721)*x[1]
        s[3,0,1]=(0.0869751746538)*x[0]**o+(0.8310099269)*x[0]+(0.915409474965)*x[1]**o+(-0.140400418634)*x[1]
        s[3,1,0]=(-0.782218646979)*x[0]**o+(0.259242630384)*x[0]+(0.474259881734)*x[1]**o+(-0.339247837819)*x[1]
        s[3,1,1]=(0.110237392658)*x[0]**o+(-0.631167781024)*x[0]+(-0.901066615275)*x[1]**o+(0.380918387854)*x[1]
        s[4,0,0]=(-0.377519984827)*x[0]**o+(-0.77772320212)*x[0]+(0.441380184654)*x[1]**o+(0.601046552584)*x[1]
        s[4,0,1]=(-0.563858988542)*x[0]**o+(0.00445139485154)*x[0]+(-0.294713455734)*x[1]**o+(-0.863742009505)*x[1]
        s[4,1,0]=(-0.0472447099394)*x[0]**o+(-0.192880217627)*x[0]+(-0.299507318615)*x[1]**o+(-0.372320397063)*x[1]
        s[4,1,1]=(-0.81522394336)*x[0]**o+(0.822036737098)*x[0]+(0.944437317648)*x[1]**o+(-0.614529825138)*x[1]
        s[5,0,0]=(0.817613090618)*x[0]**o+(-0.611860825889)*x[0]+(-0.425821334232)*x[1]**o+(0.657829198468)*x[1]
        s[5,0,1]=(-0.562766866656)*x[0]**o+(0.374639711066)*x[0]+(0.5381903866)*x[1]**o+(0.642409148669)*x[1]
        s[5,1,0]=(0.98365300543)*x[0]**o+(0.95143935497)*x[0]+(-0.186391221276)*x[1]**o+(0.99144952063)*x[1]
        s[5,1,1]=(0.422131909022)*x[0]**o+(-0.503167260902)*x[0]+(-0.708590674746)*x[1]**o+(0.164597708191)*x[1]
        ref[0,0,0]=(0.748658213139)*x_ref[0]**o+(0.588706109747)*x_ref[0]+(-0.462395788697)*x_ref[1]**o+(0.558348215492)*x_ref[1]
        ref[0,0,1]=(0.629957777572)*x_ref[0]**o+(0.714493090646)*x_ref[0]+(-0.491054832228)*x_ref[1]**o+(0.710860562012)*x_ref[1]
        ref[0,1,0]=(-0.775354422191)*x_ref[0]**o+(0.152096744898)*x_ref[0]+(0.717568022676)*x_ref[1]**o+(0.749449532926)*x_ref[1]
        ref[0,1,1]=(0.883871444687)*x_ref[0]**o+(0.149367558371)*x_ref[0]+(0.986279594706)*x_ref[1]**o+(0.537229955955)*x_ref[1]
        ref[1,0,0]=(0.564688441996)*x_ref[0]**o+(-0.206862774902)*x_ref[0]+(0.522967069439)*x_ref[1]**o+(-0.0390132413525)*x_ref[1]
        ref[1,0,1]=(0.706775860383)*x_ref[0]**o+(0.973359689319)*x_ref[0]+(0.567062178556)*x_ref[1]**o+(-0.00264874871685)*x_ref[1]
        ref[1,1,0]=(-0.385131437493)*x_ref[0]**o+(0.93223208004)*x_ref[0]+(0.915604601744)*x_ref[1]**o+(-0.604136431942)*x_ref[1]
        ref[1,1,1]=(-0.58509656422)*x_ref[0]**o+(-0.46833032365)*x_ref[0]+(0.143097299013)*x_ref[1]**o+(-0.685071549088)*x_ref[1]
        ref[2,0,0]=(-0.987452159168)*x_ref[0]**o+(0.609343916634)*x_ref[0]+(0.425443014801)*x_ref[1]**o+(0.58099336693)*x_ref[1]
        ref[2,0,1]=(0.957582337028)*x_ref[0]**o+(0.62772269136)*x_ref[0]+(-0.783891315097)*x_ref[1]**o+(-0.77280642716)*x_ref[1]
        ref[2,1,0]=(0.327068296677)*x_ref[0]**o+(0.0579535708335)*x_ref[0]+(-0.798372147412)*x_ref[1]**o+(-0.517396772711)*x_ref[1]
        ref[2,1,1]=(-0.863293845828)*x_ref[0]**o+(0.688257458796)*x_ref[0]+(0.777044937444)*x_ref[1]**o+(0.210894669552)*x_ref[1]
        ref[3,0,0]=(0.641389559724)*x_ref[0]**o+(-0.115831890978)*x_ref[0]+(0.0852026159583)*x_ref[1]**o+(0.101596282721)*x_ref[1]
        ref[3,0,1]=(0.0869751746538)*x_ref[0]**o+(0.8310099269)*x_ref[0]+(0.915409474965)*x_ref[1]**o+(-0.140400418634)*x_ref[1]
        ref[3,1,0]=(-0.782218646979)*x_ref[0]**o+(0.259242630384)*x_ref[0]+(0.474259881734)*x_ref[1]**o+(-0.339247837819)*x_ref[1]
        ref[3,1,1]=(0.110237392658)*x_ref[0]**o+(-0.631167781024)*x_ref[0]+(-0.901066615275)*x_ref[1]**o+(0.380918387854)*x_ref[1]
        ref[4,0,0]=(-0.377519984827)*x_ref[0]**o+(-0.77772320212)*x_ref[0]+(0.441380184654)*x_ref[1]**o+(0.601046552584)*x_ref[1]
        ref[4,0,1]=(-0.563858988542)*x_ref[0]**o+(0.00445139485154)*x_ref[0]+(-0.294713455734)*x_ref[1]**o+(-0.863742009505)*x_ref[1]
        ref[4,1,0]=(-0.0472447099394)*x_ref[0]**o+(-0.192880217627)*x_ref[0]+(-0.299507318615)*x_ref[1]**o+(-0.372320397063)*x_ref[1]
        ref[4,1,1]=(-0.81522394336)*x_ref[0]**o+(0.822036737098)*x_ref[0]+(0.944437317648)*x_ref[1]**o+(-0.614529825138)*x_ref[1]
        ref[5,0,0]=(0.817613090618)*x_ref[0]**o+(-0.611860825889)*x_ref[0]+(-0.425821334232)*x_ref[1]**o+(0.657829198468)*x_ref[1]
        ref[5,0,1]=(-0.562766866656)*x_ref[0]**o+(0.374639711066)*x_ref[0]+(0.5381903866)*x_ref[1]**o+(0.642409148669)*x_ref[1]
        ref[5,1,0]=(0.98365300543)*x_ref[0]**o+(0.95143935497)*x_ref[0]+(-0.186391221276)*x_ref[1]**o+(0.99144952063)*x_ref[1]
        ref[5,1,1]=(0.422131909022)*x_ref[0]**o+(-0.503167260902)*x_ref[0]+(-0.708590674746)*x_ref[1]**o+(0.164597708191)*x_ref[1]
      else:
        s[0,0,0]=(0.286638887843)*x[0]**o+(0.522689277676)*x[0]+(0.467954875136)*x[1]**o+(0.445711366876)*x[1]+(-0.669973962002)*x[2]**o+(0.181395269699)*x[2]
        s[0,0,1]=(0.353016816809)*x[0]**o+(0.991829804601)*x[0]+(-0.258315747172)*x[1]**o+(0.543990055925)*x[1]+(-0.769057570955)*x[2]**o+(0.233571124807)*x[2]
        s[0,1,0]=(0.918139015102)*x[0]**o+(-0.947988526027)*x[0]+(0.331622282377)*x[1]**o+(-0.777541428786)*x[1]+(-0.664961721637)*x[2]**o+(-0.484643482585)*x[2]
        s[0,1,1]=(0.140004476479)*x[0]**o+(-0.610763924636)*x[0]+(0.789189906539)*x[1]**o+(-0.590757211504)*x[1]+(-0.0725209360897)*x[2]**o+(-0.766615663399)*x[2]
        s[1,0,0]=(0.725731333346)*x[0]**o+(-0.131718022099)*x[0]+(0.627777941816)*x[1]**o+(-0.482136262357)*x[1]+(0.413942360645)*x[2]**o+(-0.987161567454)*x[2]
        s[1,0,1]=(0.975906575301)*x[0]**o+(0.642044782609)*x[0]+(0.670119948857)*x[1]**o+(-0.93125168252)*x[1]+(-0.553725567396)*x[2]**o+(0.609585560905)*x[2]
        s[1,1,0]=(-0.073682338522)*x[0]**o+(-0.391983504903)*x[0]+(0.252884640925)*x[1]**o+(0.474040995664)*x[1]+(-0.763628395477)*x[2]**o+(-0.753583810078)*x[2]
        s[1,1,1]=(-0.680881408584)*x[0]**o+(-0.546411376016)*x[0]+(0.85433594417)*x[1]**o+(-0.990738365487)*x[1]+(-0.202162243501)*x[2]**o+(0.256411615615)*x[2]
        s[2,0,0]=(0.79289192889)*x[0]**o+(0.243831874798)*x[0]+(0.681415436664)*x[1]**o+(0.223205387356)*x[1]+(0.517155768633)*x[2]**o+(-0.283058167273)*x[2]
        s[2,0,1]=(-0.608869102346)*x[0]**o+(0.496990925284)*x[0]+(0.925407693967)*x[1]**o+(-0.358933716971)*x[1]+(-0.328654182706)*x[2]**o+(0.835965636453)*x[2]
        s[2,1,0]=(-0.830133878676)*x[0]**o+(-0.123104434072)*x[0]+(-0.662614589611)*x[1]**o+(-0.288827930561)*x[1]+(-0.411343484191)*x[2]**o+(0.632500606896)*x[2]
        s[2,1,1]=(0.272639418094)*x[0]**o+(-0.897989787007)*x[0]+(-0.838744589589)*x[1]**o+(0.492441488123)*x[1]+(-0.634424482735)*x[2]**o+(-0.509179635903)*x[2]
        s[3,0,0]=(-0.810786146104)*x[0]**o+(-0.582309522585)*x[0]+(-0.0537085055431)*x[1]**o+(0.230345910939)*x[1]+(0.482750960353)*x[2]**o+(0.248433602737)*x[2]
        s[3,0,1]=(0.148959585193)*x[0]**o+(0.222768449067)*x[0]+(0.819219237595)*x[1]**o+(0.386479159121)*x[1]+(-0.0808464606991)*x[2]**o+(-0.0578651077709)*x[2]
        s[3,1,0]=(-0.665745862321)*x[0]**o+(-0.864332853905)*x[0]+(-0.622626855999)*x[1]**o+(-0.961729702089)*x[1]+(-0.0528887263986)*x[2]**o+(0.203930539621)*x[2]
        s[3,1,1]=(0.216834395994)*x[0]**o+(0.14686771427)*x[0]+(-0.634838825712)*x[1]**o+(-0.0741875710555)*x[1]+(0.49863557383)*x[2]**o+(0.128134148051)*x[2]
        s[4,0,0]=(-0.0429185773768)*x[0]**o+(-0.884386434806)*x[0]+(0.590953595756)*x[1]**o+(0.229045075384)*x[1]+(0.289339613114)*x[2]**o+(0.799614281102)*x[2]
        s[4,0,1]=(0.401237094734)*x[0]**o+(0.374532696487)*x[0]+(0.382784705093)*x[1]**o+(-0.00752495997413)*x[1]+(-0.693554885405)*x[2]**o+(-0.0030745706685)*x[2]
        s[4,1,0]=(0.746124554182)*x[0]**o+(-0.863459707092)*x[0]+(0.586647115827)*x[1]**o+(-0.94164686768)*x[1]+(0.700937522698)*x[2]**o+(-0.0464264819588)*x[2]
        s[4,1,1]=(0.33270904188)*x[0]**o+(0.90735243167)*x[0]+(-0.479962340673)*x[1]**o+(-0.860540809179)*x[1]+(0.396295831574)*x[2]**o+(-0.47164342577)*x[2]
        s[5,0,0]=(-0.518396400617)*x[0]**o+(0.0893385548496)*x[0]+(0.695842336307)*x[1]**o+(0.552511723893)*x[1]+(0.0636197708084)*x[2]**o+(0.0221143500177)*x[2]
        s[5,0,1]=(0.612931629659)*x[0]**o+(-0.898561600235)*x[0]+(0.231387395725)*x[1]**o+(0.728648837106)*x[1]+(-0.819722749156)*x[2]**o+(-0.987410960592)*x[2]
        s[5,1,0]=(0.228541632142)*x[0]**o+(0.0352301546347)*x[0]+(-0.457859957282)*x[1]**o+(0.126487825769)*x[1]+(-0.685267391542)*x[2]**o+(-0.695662688997)*x[2]
        s[5,1,1]=(0.892154759933)*x[0]**o+(0.0404556267055)*x[0]+(-0.509790912326)*x[1]**o+(0.544457102938)*x[1]+(-0.110692172028)*x[2]**o+(-0.111481287291)*x[2]
        ref[0,0,0]=(0.286638887843)*x_ref[0]**o+(0.522689277676)*x_ref[0]+(0.467954875136)*x_ref[1]**o+(0.445711366876)*x_ref[1]+(-0.669973962002)*x_ref[2]**o+(0.181395269699)*x_ref[2]
        ref[0,0,1]=(0.353016816809)*x_ref[0]**o+(0.991829804601)*x_ref[0]+(-0.258315747172)*x_ref[1]**o+(0.543990055925)*x_ref[1]+(-0.769057570955)*x_ref[2]**o+(0.233571124807)*x_ref[2]
        ref[0,1,0]=(0.918139015102)*x_ref[0]**o+(-0.947988526027)*x_ref[0]+(0.331622282377)*x_ref[1]**o+(-0.777541428786)*x_ref[1]+(-0.664961721637)*x_ref[2]**o+(-0.484643482585)*x_ref[2]
        ref[0,1,1]=(0.140004476479)*x_ref[0]**o+(-0.610763924636)*x_ref[0]+(0.789189906539)*x_ref[1]**o+(-0.590757211504)*x_ref[1]+(-0.0725209360897)*x_ref[2]**o+(-0.766615663399)*x_ref[2]
        ref[1,0,0]=(0.725731333346)*x_ref[0]**o+(-0.131718022099)*x_ref[0]+(0.627777941816)*x_ref[1]**o+(-0.482136262357)*x_ref[1]+(0.413942360645)*x_ref[2]**o+(-0.987161567454)*x_ref[2]
        ref[1,0,1]=(0.975906575301)*x_ref[0]**o+(0.642044782609)*x_ref[0]+(0.670119948857)*x_ref[1]**o+(-0.93125168252)*x_ref[1]+(-0.553725567396)*x_ref[2]**o+(0.609585560905)*x_ref[2]
        ref[1,1,0]=(-0.073682338522)*x_ref[0]**o+(-0.391983504903)*x_ref[0]+(0.252884640925)*x_ref[1]**o+(0.474040995664)*x_ref[1]+(-0.763628395477)*x_ref[2]**o+(-0.753583810078)*x_ref[2]
        ref[1,1,1]=(-0.680881408584)*x_ref[0]**o+(-0.546411376016)*x_ref[0]+(0.85433594417)*x_ref[1]**o+(-0.990738365487)*x_ref[1]+(-0.202162243501)*x_ref[2]**o+(0.256411615615)*x_ref[2]
        ref[2,0,0]=(0.79289192889)*x_ref[0]**o+(0.243831874798)*x_ref[0]+(0.681415436664)*x_ref[1]**o+(0.223205387356)*x_ref[1]+(0.517155768633)*x_ref[2]**o+(-0.283058167273)*x_ref[2]
        ref[2,0,1]=(-0.608869102346)*x_ref[0]**o+(0.496990925284)*x_ref[0]+(0.925407693967)*x_ref[1]**o+(-0.358933716971)*x_ref[1]+(-0.328654182706)*x_ref[2]**o+(0.835965636453)*x_ref[2]
        ref[2,1,0]=(-0.830133878676)*x_ref[0]**o+(-0.123104434072)*x_ref[0]+(-0.662614589611)*x_ref[1]**o+(-0.288827930561)*x_ref[1]+(-0.411343484191)*x_ref[2]**o+(0.632500606896)*x_ref[2]
        ref[2,1,1]=(0.272639418094)*x_ref[0]**o+(-0.897989787007)*x_ref[0]+(-0.838744589589)*x_ref[1]**o+(0.492441488123)*x_ref[1]+(-0.634424482735)*x_ref[2]**o+(-0.509179635903)*x_ref[2]
        ref[3,0,0]=(-0.810786146104)*x_ref[0]**o+(-0.582309522585)*x_ref[0]+(-0.0537085055431)*x_ref[1]**o+(0.230345910939)*x_ref[1]+(0.482750960353)*x_ref[2]**o+(0.248433602737)*x_ref[2]
        ref[3,0,1]=(0.148959585193)*x_ref[0]**o+(0.222768449067)*x_ref[0]+(0.819219237595)*x_ref[1]**o+(0.386479159121)*x_ref[1]+(-0.0808464606991)*x_ref[2]**o+(-0.0578651077709)*x_ref[2]
        ref[3,1,0]=(-0.665745862321)*x_ref[0]**o+(-0.864332853905)*x_ref[0]+(-0.622626855999)*x_ref[1]**o+(-0.961729702089)*x_ref[1]+(-0.0528887263986)*x_ref[2]**o+(0.203930539621)*x_ref[2]
        ref[3,1,1]=(0.216834395994)*x_ref[0]**o+(0.14686771427)*x_ref[0]+(-0.634838825712)*x_ref[1]**o+(-0.0741875710555)*x_ref[1]+(0.49863557383)*x_ref[2]**o+(0.128134148051)*x_ref[2]
        ref[4,0,0]=(-0.0429185773768)*x_ref[0]**o+(-0.884386434806)*x_ref[0]+(0.590953595756)*x_ref[1]**o+(0.229045075384)*x_ref[1]+(0.289339613114)*x_ref[2]**o+(0.799614281102)*x_ref[2]
        ref[4,0,1]=(0.401237094734)*x_ref[0]**o+(0.374532696487)*x_ref[0]+(0.382784705093)*x_ref[1]**o+(-0.00752495997413)*x_ref[1]+(-0.693554885405)*x_ref[2]**o+(-0.0030745706685)*x_ref[2]
        ref[4,1,0]=(0.746124554182)*x_ref[0]**o+(-0.863459707092)*x_ref[0]+(0.586647115827)*x_ref[1]**o+(-0.94164686768)*x_ref[1]+(0.700937522698)*x_ref[2]**o+(-0.0464264819588)*x_ref[2]
        ref[4,1,1]=(0.33270904188)*x_ref[0]**o+(0.90735243167)*x_ref[0]+(-0.479962340673)*x_ref[1]**o+(-0.860540809179)*x_ref[1]+(0.396295831574)*x_ref[2]**o+(-0.47164342577)*x_ref[2]
        ref[5,0,0]=(-0.518396400617)*x_ref[0]**o+(0.0893385548496)*x_ref[0]+(0.695842336307)*x_ref[1]**o+(0.552511723893)*x_ref[1]+(0.0636197708084)*x_ref[2]**o+(0.0221143500177)*x_ref[2]
        ref[5,0,1]=(0.612931629659)*x_ref[0]**o+(-0.898561600235)*x_ref[0]+(0.231387395725)*x_ref[1]**o+(0.728648837106)*x_ref[1]+(-0.819722749156)*x_ref[2]**o+(-0.987410960592)*x_ref[2]
        ref[5,1,0]=(0.228541632142)*x_ref[0]**o+(0.0352301546347)*x_ref[0]+(-0.457859957282)*x_ref[1]**o+(0.126487825769)*x_ref[1]+(-0.685267391542)*x_ref[2]**o+(-0.695662688997)*x_ref[2]
        ref[5,1,1]=(0.892154759933)*x_ref[0]**o+(0.0404556267055)*x_ref[0]+(-0.509790912326)*x_ref[1]**o+(0.544457102938)*x_ref[1]+(-0.110692172028)*x_ref[2]**o+(-0.111481287291)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_ReducedFunctionOnContactZero_rank4(self):
      """
      tests interpolate for rank 4 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactZero onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactZero(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5, 3, 2),dim=dim)
      s=Data(0,(4, 5, 3, 2),w)
      ref=Data(0,(4, 5, 3, 2),w_ref)
      if dim==2:
        s[0,0,0,0]=(-0.0573009878147)*x[0]**o+(-0.350301437886)*x[0]+(0.253659046023)*x[1]**o+(0.658249302609)*x[1]
        s[0,0,0,1]=(0.830676080978)*x[0]**o+(-0.809626423382)*x[0]+(0.370613645219)*x[1]**o+(-0.172834681384)*x[1]
        s[0,0,1,0]=(-0.735478785962)*x[0]**o+(0.512724546806)*x[0]+(-0.717033770258)*x[1]**o+(-0.0946841286843)*x[1]
        s[0,0,1,1]=(-0.715337226505)*x[0]**o+(0.1253433121)*x[0]+(0.230909585243)*x[1]**o+(0.470778883904)*x[1]
        s[0,0,2,0]=(0.949793126568)*x[0]**o+(-0.12899598886)*x[0]+(-0.749733392851)*x[1]**o+(-0.340807421479)*x[1]
        s[0,0,2,1]=(0.261609659356)*x[0]**o+(0.336166427172)*x[0]+(0.657111532027)*x[1]**o+(-0.614814002143)*x[1]
        s[0,1,0,0]=(-0.0275060361172)*x[0]**o+(0.966092296743)*x[0]+(-0.532890777604)*x[1]**o+(0.517696025589)*x[1]
        s[0,1,0,1]=(0.00166569160565)*x[0]**o+(0.236625486597)*x[0]+(0.893101928693)*x[1]**o+(-0.574929038375)*x[1]
        s[0,1,1,0]=(0.872269789656)*x[0]**o+(-0.905740937959)*x[0]+(0.719538254619)*x[1]**o+(0.711635238494)*x[1]
        s[0,1,1,1]=(0.801801659725)*x[0]**o+(0.139019380599)*x[0]+(0.708524489966)*x[1]**o+(-0.77122453375)*x[1]
        s[0,1,2,0]=(-0.0481701009478)*x[0]**o+(0.167249653575)*x[0]+(0.0551704085528)*x[1]**o+(0.505944176073)*x[1]
        s[0,1,2,1]=(0.185898813821)*x[0]**o+(-0.750605793976)*x[0]+(-0.172006188497)*x[1]**o+(-0.0840963686282)*x[1]
        s[0,2,0,0]=(0.499076840583)*x[0]**o+(-0.230482058386)*x[0]+(0.522046309086)*x[1]**o+(-0.445372865956)*x[1]
        s[0,2,0,1]=(0.643144964505)*x[0]**o+(0.754412438054)*x[0]+(-0.527488849658)*x[1]**o+(-0.781951420648)*x[1]
        s[0,2,1,0]=(-0.428090027626)*x[0]**o+(0.941724629359)*x[0]+(-0.888361304631)*x[1]**o+(-0.193082322033)*x[1]
        s[0,2,1,1]=(0.737854520423)*x[0]**o+(-0.600462005647)*x[0]+(0.523944735668)*x[1]**o+(-0.88314904941)*x[1]
        s[0,2,2,0]=(0.401651360188)*x[0]**o+(0.605266809429)*x[0]+(0.114190031414)*x[1]**o+(-0.0554731448399)*x[1]
        s[0,2,2,1]=(0.49575503694)*x[0]**o+(0.842572557458)*x[0]+(-0.65557628937)*x[1]**o+(-0.110654902831)*x[1]
        s[0,3,0,0]=(-0.914580503709)*x[0]**o+(0.568110262697)*x[0]+(0.465998898456)*x[1]**o+(0.668680449802)*x[1]
        s[0,3,0,1]=(-0.406517611176)*x[0]**o+(0.751773194945)*x[0]+(-0.546709674237)*x[1]**o+(0.830905534576)*x[1]
        s[0,3,1,0]=(0.256170881332)*x[0]**o+(0.785200644094)*x[0]+(-0.0949323527814)*x[1]**o+(0.437799246842)*x[1]
        s[0,3,1,1]=(0.388926885643)*x[0]**o+(0.0649842722847)*x[0]+(-0.570138074228)*x[1]**o+(0.0125008958832)*x[1]
        s[0,3,2,0]=(-0.693124111229)*x[0]**o+(-0.135531989861)*x[0]+(-0.484364583498)*x[1]**o+(0.142028440685)*x[1]
        s[0,3,2,1]=(0.162852074905)*x[0]**o+(-0.211709648647)*x[0]+(-0.424964818278)*x[1]**o+(-0.0311289175964)*x[1]
        s[0,4,0,0]=(-0.893934248272)*x[0]**o+(-0.883334046545)*x[0]+(0.141127275532)*x[1]**o+(0.790251212849)*x[1]
        s[0,4,0,1]=(0.751030754129)*x[0]**o+(-0.0463112594115)*x[0]+(0.894754601054)*x[1]**o+(-0.986137408191)*x[1]
        s[0,4,1,0]=(-0.994216054583)*x[0]**o+(0.989450915377)*x[0]+(0.202383847504)*x[1]**o+(-0.829413728527)*x[1]
        s[0,4,1,1]=(-0.70581665488)*x[0]**o+(0.387180461053)*x[0]+(0.925669527701)*x[1]**o+(0.422052343171)*x[1]
        s[0,4,2,0]=(-0.498553799214)*x[0]**o+(0.70559513684)*x[0]+(-0.595097550839)*x[1]**o+(-0.177703117092)*x[1]
        s[0,4,2,1]=(-0.703437824074)*x[0]**o+(-0.574613675229)*x[0]+(-0.699959931134)*x[1]**o+(-0.503745753701)*x[1]
        s[1,0,0,0]=(-0.0673123845899)*x[0]**o+(-0.0603073393325)*x[0]+(0.724127339391)*x[1]**o+(-0.0875500989225)*x[1]
        s[1,0,0,1]=(0.0766117819496)*x[0]**o+(-0.489029658255)*x[0]+(0.602970700568)*x[1]**o+(0.624180485873)*x[1]
        s[1,0,1,0]=(0.537236166297)*x[0]**o+(0.805657082915)*x[0]+(-0.0263572051431)*x[1]**o+(0.683790156906)*x[1]
        s[1,0,1,1]=(0.55427198724)*x[0]**o+(0.631937641401)*x[0]+(-0.634744046324)*x[1]**o+(0.824382447181)*x[1]
        s[1,0,2,0]=(0.694551886505)*x[0]**o+(-0.123561028797)*x[0]+(-0.0237233372904)*x[1]**o+(-0.822938415295)*x[1]
        s[1,0,2,1]=(0.293944558832)*x[0]**o+(-0.34741820803)*x[0]+(-0.471269455638)*x[1]**o+(0.716959839747)*x[1]
        s[1,1,0,0]=(0.409876005614)*x[0]**o+(-0.322385353356)*x[0]+(0.362023900558)*x[1]**o+(-0.974915188765)*x[1]
        s[1,1,0,1]=(-0.823587457611)*x[0]**o+(-0.315566397595)*x[0]+(-0.681951254575)*x[1]**o+(-0.945803349426)*x[1]
        s[1,1,1,0]=(0.782950007967)*x[0]**o+(0.767220278425)*x[0]+(0.109277917483)*x[1]**o+(-0.336228598202)*x[1]
        s[1,1,1,1]=(-0.969811850973)*x[0]**o+(0.453588680715)*x[0]+(-0.296932776592)*x[1]**o+(-0.460668458023)*x[1]
        s[1,1,2,0]=(0.124616545015)*x[0]**o+(0.180238150583)*x[0]+(0.283723867146)*x[1]**o+(0.873774600692)*x[1]
        s[1,1,2,1]=(0.00334202091213)*x[0]**o+(-0.711821714868)*x[0]+(0.638247490899)*x[1]**o+(-0.409696943449)*x[1]
        s[1,2,0,0]=(-0.206885052591)*x[0]**o+(-0.677245253776)*x[0]+(-0.508637512178)*x[1]**o+(0.0862259084563)*x[1]
        s[1,2,0,1]=(0.402289447161)*x[0]**o+(0.242246771389)*x[0]+(0.775293224518)*x[1]**o+(-0.668881714475)*x[1]
        s[1,2,1,0]=(-0.094983969904)*x[0]**o+(0.615306379484)*x[0]+(-0.737839325648)*x[1]**o+(0.457810359747)*x[1]
        s[1,2,1,1]=(0.488668557736)*x[0]**o+(0.188170233286)*x[0]+(0.0947759637418)*x[1]**o+(-0.510206729955)*x[1]
        s[1,2,2,0]=(-0.102204043127)*x[0]**o+(0.584307889877)*x[0]+(-0.779746892074)*x[1]**o+(-0.767757863288)*x[1]
        s[1,2,2,1]=(-0.00230617324952)*x[0]**o+(-0.184632788675)*x[0]+(-0.779494086898)*x[1]**o+(-0.105311421755)*x[1]
        s[1,3,0,0]=(0.373541105531)*x[0]**o+(-0.895909586816)*x[0]+(0.853523284159)*x[1]**o+(0.915229883403)*x[1]
        s[1,3,0,1]=(-0.888315748473)*x[0]**o+(0.877859466924)*x[0]+(-0.830380928425)*x[1]**o+(-0.25922026388)*x[1]
        s[1,3,1,0]=(-0.944775455365)*x[0]**o+(0.694978915121)*x[0]+(0.00346060944841)*x[1]**o+(0.610096191934)*x[1]
        s[1,3,1,1]=(-0.175304757445)*x[0]**o+(0.883688655763)*x[0]+(-0.0260273718271)*x[1]**o+(0.440569935385)*x[1]
        s[1,3,2,0]=(0.973418372218)*x[0]**o+(-0.0725489149125)*x[0]+(-0.790861964708)*x[1]**o+(-0.764398676127)*x[1]
        s[1,3,2,1]=(0.188665821479)*x[0]**o+(-0.694573366343)*x[0]+(-0.860244164818)*x[1]**o+(0.642736876841)*x[1]
        s[1,4,0,0]=(-0.619221357105)*x[0]**o+(-0.0366612230467)*x[0]+(0.920044684933)*x[1]**o+(-0.901158565327)*x[1]
        s[1,4,0,1]=(-0.774227719954)*x[0]**o+(0.471966691541)*x[0]+(0.0706379157745)*x[1]**o+(-0.238713884878)*x[1]
        s[1,4,1,0]=(0.863551422475)*x[0]**o+(0.131078367733)*x[0]+(0.458326651999)*x[1]**o+(-0.750431841864)*x[1]
        s[1,4,1,1]=(0.105763306948)*x[0]**o+(-0.254538567936)*x[0]+(-0.559297954287)*x[1]**o+(0.21981759162)*x[1]
        s[1,4,2,0]=(0.922873574086)*x[0]**o+(-0.853230802788)*x[0]+(-0.982798222021)*x[1]**o+(-0.0866961947026)*x[1]
        s[1,4,2,1]=(-0.21664472336)*x[0]**o+(-0.915876043584)*x[0]+(0.6182964805)*x[1]**o+(0.665515072302)*x[1]
        s[2,0,0,0]=(-0.491230924471)*x[0]**o+(0.509670721146)*x[0]+(-0.44580759248)*x[1]**o+(-0.943168309946)*x[1]
        s[2,0,0,1]=(-0.0895802535424)*x[0]**o+(0.0780292649662)*x[0]+(-0.42293922141)*x[1]**o+(-0.76928974668)*x[1]
        s[2,0,1,0]=(-0.404348433578)*x[0]**o+(-0.986117091427)*x[0]+(0.562569938626)*x[1]**o+(-0.41119675761)*x[1]
        s[2,0,1,1]=(0.517587873308)*x[0]**o+(0.0731347448674)*x[0]+(0.830876774432)*x[1]**o+(-0.690590335528)*x[1]
        s[2,0,2,0]=(0.600931766711)*x[0]**o+(-0.786774940006)*x[0]+(-0.285975800855)*x[1]**o+(0.348122549519)*x[1]
        s[2,0,2,1]=(0.825033481951)*x[0]**o+(-0.744551393479)*x[0]+(0.452879660703)*x[1]**o+(-0.257025658518)*x[1]
        s[2,1,0,0]=(-0.0822184021881)*x[0]**o+(0.955001628077)*x[0]+(-0.878009442237)*x[1]**o+(-0.816970793701)*x[1]
        s[2,1,0,1]=(-0.151473282662)*x[0]**o+(0.937636279558)*x[0]+(-0.969793512643)*x[1]**o+(0.704581030148)*x[1]
        s[2,1,1,0]=(0.76728625844)*x[0]**o+(-0.90294390262)*x[0]+(-0.375443980814)*x[1]**o+(0.276219230561)*x[1]
        s[2,1,1,1]=(-0.994260771722)*x[0]**o+(0.64595057257)*x[0]+(0.486576661947)*x[1]**o+(0.731738848734)*x[1]
        s[2,1,2,0]=(-0.52048273513)*x[0]**o+(-0.51915057929)*x[0]+(0.184855975415)*x[1]**o+(0.417869171454)*x[1]
        s[2,1,2,1]=(-0.371019640816)*x[0]**o+(-0.617014410896)*x[0]+(0.725259821636)*x[1]**o+(-0.0393667759542)*x[1]
        s[2,2,0,0]=(0.070044943001)*x[0]**o+(-0.663066522147)*x[0]+(0.761006491068)*x[1]**o+(-0.137480662114)*x[1]
        s[2,2,0,1]=(-0.394035586896)*x[0]**o+(0.802334732427)*x[0]+(-0.191154645494)*x[1]**o+(-0.278490557185)*x[1]
        s[2,2,1,0]=(0.0664932052956)*x[0]**o+(0.938378455125)*x[0]+(0.997119543949)*x[1]**o+(-0.696820719616)*x[1]
        s[2,2,1,1]=(0.79758586389)*x[0]**o+(0.0393291272874)*x[0]+(0.644980805887)*x[1]**o+(0.61925875842)*x[1]
        s[2,2,2,0]=(0.0353810358089)*x[0]**o+(-0.0134652375886)*x[0]+(-0.942381983803)*x[1]**o+(0.288697669014)*x[1]
        s[2,2,2,1]=(-0.327577257504)*x[0]**o+(0.816374902745)*x[0]+(0.676853003934)*x[1]**o+(0.581394632094)*x[1]
        s[2,3,0,0]=(0.63036222221)*x[0]**o+(-0.899724810809)*x[0]+(0.276882104091)*x[1]**o+(-0.00448820301083)*x[1]
        s[2,3,0,1]=(-0.219899069159)*x[0]**o+(-0.439244974227)*x[0]+(-0.369366024932)*x[1]**o+(-0.161805301574)*x[1]
        s[2,3,1,0]=(0.165202872055)*x[0]**o+(-0.21521932039)*x[0]+(0.635188695229)*x[1]**o+(-0.254785217427)*x[1]
        s[2,3,1,1]=(0.743546803617)*x[0]**o+(0.876160636955)*x[0]+(-0.094730937905)*x[1]**o+(-0.898315912992)*x[1]
        s[2,3,2,0]=(0.946538368626)*x[0]**o+(0.715488078243)*x[0]+(0.830112020529)*x[1]**o+(-0.320605753644)*x[1]
        s[2,3,2,1]=(0.52106888624)*x[0]**o+(-0.938072665733)*x[0]+(-0.682830472037)*x[1]**o+(-0.787442211636)*x[1]
        s[2,4,0,0]=(0.802634158133)*x[0]**o+(0.350923640202)*x[0]+(-0.142397660222)*x[1]**o+(0.768827179039)*x[1]
        s[2,4,0,1]=(0.912207226957)*x[0]**o+(0.302955742414)*x[0]+(-0.125141454725)*x[1]**o+(0.115375955845)*x[1]
        s[2,4,1,0]=(0.88629189066)*x[0]**o+(-0.431145868333)*x[0]+(0.521817592458)*x[1]**o+(0.757372657079)*x[1]
        s[2,4,1,1]=(-0.34619470039)*x[0]**o+(0.0971941022378)*x[0]+(-0.235321219876)*x[1]**o+(0.919340221916)*x[1]
        s[2,4,2,0]=(-0.453744205858)*x[0]**o+(0.386107152737)*x[0]+(-0.53829794769)*x[1]**o+(-0.955985738202)*x[1]
        s[2,4,2,1]=(0.327564798989)*x[0]**o+(0.234176645121)*x[0]+(-0.647242055069)*x[1]**o+(-0.914279617484)*x[1]
        s[3,0,0,0]=(-0.765850994681)*x[0]**o+(0.0969021602734)*x[0]+(-0.003697715708)*x[1]**o+(0.642293903037)*x[1]
        s[3,0,0,1]=(-0.736912760299)*x[0]**o+(-0.496756935434)*x[0]+(0.00160221880808)*x[1]**o+(-0.0275314787811)*x[1]
        s[3,0,1,0]=(0.266302600859)*x[0]**o+(0.962232443249)*x[0]+(0.588094142167)*x[1]**o+(-0.740113379551)*x[1]
        s[3,0,1,1]=(-0.701204282707)*x[0]**o+(0.16377449778)*x[0]+(-0.636725536715)*x[1]**o+(0.925340403623)*x[1]
        s[3,0,2,0]=(-0.473492482064)*x[0]**o+(0.356976796364)*x[0]+(0.28490257392)*x[1]**o+(0.558859192591)*x[1]
        s[3,0,2,1]=(-0.398155056481)*x[0]**o+(-0.693010191754)*x[0]+(-0.457570700744)*x[1]**o+(0.899032980684)*x[1]
        s[3,1,0,0]=(0.676626616862)*x[0]**o+(-0.142127205425)*x[0]+(-0.0703163466342)*x[1]**o+(-0.603767804268)*x[1]
        s[3,1,0,1]=(0.901299559593)*x[0]**o+(-0.319119050502)*x[0]+(-0.230855076893)*x[1]**o+(0.923789250137)*x[1]
        s[3,1,1,0]=(0.0125691057912)*x[0]**o+(0.827401151454)*x[0]+(0.0881382970418)*x[1]**o+(0.12796232574)*x[1]
        s[3,1,1,1]=(-0.544917801861)*x[0]**o+(-0.163923780568)*x[0]+(0.107833995954)*x[1]**o+(-0.347470857919)*x[1]
        s[3,1,2,0]=(-0.127281534891)*x[0]**o+(0.410172113868)*x[0]+(0.516300378909)*x[1]**o+(-0.454031679856)*x[1]
        s[3,1,2,1]=(-0.797619552686)*x[0]**o+(0.0212633210895)*x[0]+(-0.0890328154993)*x[1]**o+(0.755392852372)*x[1]
        s[3,2,0,0]=(0.00976392699076)*x[0]**o+(-0.608813040196)*x[0]+(0.786997931081)*x[1]**o+(0.637450406747)*x[1]
        s[3,2,0,1]=(-0.594915968211)*x[0]**o+(0.909106492284)*x[0]+(0.209774113888)*x[1]**o+(0.926156671099)*x[1]
        s[3,2,1,0]=(0.226007901841)*x[0]**o+(-0.43873163217)*x[0]+(-0.414778768373)*x[1]**o+(0.681053132578)*x[1]
        s[3,2,1,1]=(0.733375971867)*x[0]**o+(0.861188515019)*x[0]+(0.036782797193)*x[1]**o+(0.145181529782)*x[1]
        s[3,2,2,0]=(0.635727397799)*x[0]**o+(0.874128725841)*x[0]+(-0.810169847222)*x[1]**o+(0.234369619159)*x[1]
        s[3,2,2,1]=(-0.137252742544)*x[0]**o+(0.817564886208)*x[0]+(0.524706682726)*x[1]**o+(0.381735970369)*x[1]
        s[3,3,0,0]=(0.537368746859)*x[0]**o+(-0.298760793455)*x[0]+(-0.890807690497)*x[1]**o+(-0.662222369966)*x[1]
        s[3,3,0,1]=(-0.67672238667)*x[0]**o+(-0.305431277061)*x[0]+(-0.435551021687)*x[1]**o+(0.0777923105041)*x[1]
        s[3,3,1,0]=(-0.163284186346)*x[0]**o+(-0.764058727495)*x[0]+(-0.255689425142)*x[1]**o+(0.285528254098)*x[1]
        s[3,3,1,1]=(-0.842634439814)*x[0]**o+(-0.194811378723)*x[0]+(0.814424721344)*x[1]**o+(-0.236826785563)*x[1]
        s[3,3,2,0]=(0.532936591284)*x[0]**o+(-0.920822945577)*x[0]+(-0.677841412043)*x[1]**o+(0.579981868038)*x[1]
        s[3,3,2,1]=(-0.470357673624)*x[0]**o+(-0.972798479564)*x[0]+(0.489692897505)*x[1]**o+(0.213937860142)*x[1]
        s[3,4,0,0]=(-0.169170544642)*x[0]**o+(0.543113611166)*x[0]+(0.536912197355)*x[1]**o+(0.304230092305)*x[1]
        s[3,4,0,1]=(-0.170701528868)*x[0]**o+(0.340606153319)*x[0]+(-0.470528666825)*x[1]**o+(-0.522028583228)*x[1]
        s[3,4,1,0]=(0.0194532320642)*x[0]**o+(0.335898702694)*x[0]+(-0.917025993458)*x[1]**o+(-0.539748486826)*x[1]
        s[3,4,1,1]=(-0.576465573138)*x[0]**o+(-0.906917910142)*x[0]+(-0.313000868178)*x[1]**o+(-0.20607256753)*x[1]
        s[3,4,2,0]=(-0.75394368612)*x[0]**o+(0.658866075222)*x[0]+(0.402200040518)*x[1]**o+(-0.544252067119)*x[1]
        s[3,4,2,1]=(0.0490609014064)*x[0]**o+(-0.873887848405)*x[0]+(0.91186722933)*x[1]**o+(0.375269571672)*x[1]
        ref[0,0,0,0]=(-0.0573009878147)*x_ref[0]**o+(-0.350301437886)*x_ref[0]+(0.253659046023)*x_ref[1]**o+(0.658249302609)*x_ref[1]
        ref[0,0,0,1]=(0.830676080978)*x_ref[0]**o+(-0.809626423382)*x_ref[0]+(0.370613645219)*x_ref[1]**o+(-0.172834681384)*x_ref[1]
        ref[0,0,1,0]=(-0.735478785962)*x_ref[0]**o+(0.512724546806)*x_ref[0]+(-0.717033770258)*x_ref[1]**o+(-0.0946841286843)*x_ref[1]
        ref[0,0,1,1]=(-0.715337226505)*x_ref[0]**o+(0.1253433121)*x_ref[0]+(0.230909585243)*x_ref[1]**o+(0.470778883904)*x_ref[1]
        ref[0,0,2,0]=(0.949793126568)*x_ref[0]**o+(-0.12899598886)*x_ref[0]+(-0.749733392851)*x_ref[1]**o+(-0.340807421479)*x_ref[1]
        ref[0,0,2,1]=(0.261609659356)*x_ref[0]**o+(0.336166427172)*x_ref[0]+(0.657111532027)*x_ref[1]**o+(-0.614814002143)*x_ref[1]
        ref[0,1,0,0]=(-0.0275060361172)*x_ref[0]**o+(0.966092296743)*x_ref[0]+(-0.532890777604)*x_ref[1]**o+(0.517696025589)*x_ref[1]
        ref[0,1,0,1]=(0.00166569160565)*x_ref[0]**o+(0.236625486597)*x_ref[0]+(0.893101928693)*x_ref[1]**o+(-0.574929038375)*x_ref[1]
        ref[0,1,1,0]=(0.872269789656)*x_ref[0]**o+(-0.905740937959)*x_ref[0]+(0.719538254619)*x_ref[1]**o+(0.711635238494)*x_ref[1]
        ref[0,1,1,1]=(0.801801659725)*x_ref[0]**o+(0.139019380599)*x_ref[0]+(0.708524489966)*x_ref[1]**o+(-0.77122453375)*x_ref[1]
        ref[0,1,2,0]=(-0.0481701009478)*x_ref[0]**o+(0.167249653575)*x_ref[0]+(0.0551704085528)*x_ref[1]**o+(0.505944176073)*x_ref[1]
        ref[0,1,2,1]=(0.185898813821)*x_ref[0]**o+(-0.750605793976)*x_ref[0]+(-0.172006188497)*x_ref[1]**o+(-0.0840963686282)*x_ref[1]
        ref[0,2,0,0]=(0.499076840583)*x_ref[0]**o+(-0.230482058386)*x_ref[0]+(0.522046309086)*x_ref[1]**o+(-0.445372865956)*x_ref[1]
        ref[0,2,0,1]=(0.643144964505)*x_ref[0]**o+(0.754412438054)*x_ref[0]+(-0.527488849658)*x_ref[1]**o+(-0.781951420648)*x_ref[1]
        ref[0,2,1,0]=(-0.428090027626)*x_ref[0]**o+(0.941724629359)*x_ref[0]+(-0.888361304631)*x_ref[1]**o+(-0.193082322033)*x_ref[1]
        ref[0,2,1,1]=(0.737854520423)*x_ref[0]**o+(-0.600462005647)*x_ref[0]+(0.523944735668)*x_ref[1]**o+(-0.88314904941)*x_ref[1]
        ref[0,2,2,0]=(0.401651360188)*x_ref[0]**o+(0.605266809429)*x_ref[0]+(0.114190031414)*x_ref[1]**o+(-0.0554731448399)*x_ref[1]
        ref[0,2,2,1]=(0.49575503694)*x_ref[0]**o+(0.842572557458)*x_ref[0]+(-0.65557628937)*x_ref[1]**o+(-0.110654902831)*x_ref[1]
        ref[0,3,0,0]=(-0.914580503709)*x_ref[0]**o+(0.568110262697)*x_ref[0]+(0.465998898456)*x_ref[1]**o+(0.668680449802)*x_ref[1]
        ref[0,3,0,1]=(-0.406517611176)*x_ref[0]**o+(0.751773194945)*x_ref[0]+(-0.546709674237)*x_ref[1]**o+(0.830905534576)*x_ref[1]
        ref[0,3,1,0]=(0.256170881332)*x_ref[0]**o+(0.785200644094)*x_ref[0]+(-0.0949323527814)*x_ref[1]**o+(0.437799246842)*x_ref[1]
        ref[0,3,1,1]=(0.388926885643)*x_ref[0]**o+(0.0649842722847)*x_ref[0]+(-0.570138074228)*x_ref[1]**o+(0.0125008958832)*x_ref[1]
        ref[0,3,2,0]=(-0.693124111229)*x_ref[0]**o+(-0.135531989861)*x_ref[0]+(-0.484364583498)*x_ref[1]**o+(0.142028440685)*x_ref[1]
        ref[0,3,2,1]=(0.162852074905)*x_ref[0]**o+(-0.211709648647)*x_ref[0]+(-0.424964818278)*x_ref[1]**o+(-0.0311289175964)*x_ref[1]
        ref[0,4,0,0]=(-0.893934248272)*x_ref[0]**o+(-0.883334046545)*x_ref[0]+(0.141127275532)*x_ref[1]**o+(0.790251212849)*x_ref[1]
        ref[0,4,0,1]=(0.751030754129)*x_ref[0]**o+(-0.0463112594115)*x_ref[0]+(0.894754601054)*x_ref[1]**o+(-0.986137408191)*x_ref[1]
        ref[0,4,1,0]=(-0.994216054583)*x_ref[0]**o+(0.989450915377)*x_ref[0]+(0.202383847504)*x_ref[1]**o+(-0.829413728527)*x_ref[1]
        ref[0,4,1,1]=(-0.70581665488)*x_ref[0]**o+(0.387180461053)*x_ref[0]+(0.925669527701)*x_ref[1]**o+(0.422052343171)*x_ref[1]
        ref[0,4,2,0]=(-0.498553799214)*x_ref[0]**o+(0.70559513684)*x_ref[0]+(-0.595097550839)*x_ref[1]**o+(-0.177703117092)*x_ref[1]
        ref[0,4,2,1]=(-0.703437824074)*x_ref[0]**o+(-0.574613675229)*x_ref[0]+(-0.699959931134)*x_ref[1]**o+(-0.503745753701)*x_ref[1]
        ref[1,0,0,0]=(-0.0673123845899)*x_ref[0]**o+(-0.0603073393325)*x_ref[0]+(0.724127339391)*x_ref[1]**o+(-0.0875500989225)*x_ref[1]
        ref[1,0,0,1]=(0.0766117819496)*x_ref[0]**o+(-0.489029658255)*x_ref[0]+(0.602970700568)*x_ref[1]**o+(0.624180485873)*x_ref[1]
        ref[1,0,1,0]=(0.537236166297)*x_ref[0]**o+(0.805657082915)*x_ref[0]+(-0.0263572051431)*x_ref[1]**o+(0.683790156906)*x_ref[1]
        ref[1,0,1,1]=(0.55427198724)*x_ref[0]**o+(0.631937641401)*x_ref[0]+(-0.634744046324)*x_ref[1]**o+(0.824382447181)*x_ref[1]
        ref[1,0,2,0]=(0.694551886505)*x_ref[0]**o+(-0.123561028797)*x_ref[0]+(-0.0237233372904)*x_ref[1]**o+(-0.822938415295)*x_ref[1]
        ref[1,0,2,1]=(0.293944558832)*x_ref[0]**o+(-0.34741820803)*x_ref[0]+(-0.471269455638)*x_ref[1]**o+(0.716959839747)*x_ref[1]
        ref[1,1,0,0]=(0.409876005614)*x_ref[0]**o+(-0.322385353356)*x_ref[0]+(0.362023900558)*x_ref[1]**o+(-0.974915188765)*x_ref[1]
        ref[1,1,0,1]=(-0.823587457611)*x_ref[0]**o+(-0.315566397595)*x_ref[0]+(-0.681951254575)*x_ref[1]**o+(-0.945803349426)*x_ref[1]
        ref[1,1,1,0]=(0.782950007967)*x_ref[0]**o+(0.767220278425)*x_ref[0]+(0.109277917483)*x_ref[1]**o+(-0.336228598202)*x_ref[1]
        ref[1,1,1,1]=(-0.969811850973)*x_ref[0]**o+(0.453588680715)*x_ref[0]+(-0.296932776592)*x_ref[1]**o+(-0.460668458023)*x_ref[1]
        ref[1,1,2,0]=(0.124616545015)*x_ref[0]**o+(0.180238150583)*x_ref[0]+(0.283723867146)*x_ref[1]**o+(0.873774600692)*x_ref[1]
        ref[1,1,2,1]=(0.00334202091213)*x_ref[0]**o+(-0.711821714868)*x_ref[0]+(0.638247490899)*x_ref[1]**o+(-0.409696943449)*x_ref[1]
        ref[1,2,0,0]=(-0.206885052591)*x_ref[0]**o+(-0.677245253776)*x_ref[0]+(-0.508637512178)*x_ref[1]**o+(0.0862259084563)*x_ref[1]
        ref[1,2,0,1]=(0.402289447161)*x_ref[0]**o+(0.242246771389)*x_ref[0]+(0.775293224518)*x_ref[1]**o+(-0.668881714475)*x_ref[1]
        ref[1,2,1,0]=(-0.094983969904)*x_ref[0]**o+(0.615306379484)*x_ref[0]+(-0.737839325648)*x_ref[1]**o+(0.457810359747)*x_ref[1]
        ref[1,2,1,1]=(0.488668557736)*x_ref[0]**o+(0.188170233286)*x_ref[0]+(0.0947759637418)*x_ref[1]**o+(-0.510206729955)*x_ref[1]
        ref[1,2,2,0]=(-0.102204043127)*x_ref[0]**o+(0.584307889877)*x_ref[0]+(-0.779746892074)*x_ref[1]**o+(-0.767757863288)*x_ref[1]
        ref[1,2,2,1]=(-0.00230617324952)*x_ref[0]**o+(-0.184632788675)*x_ref[0]+(-0.779494086898)*x_ref[1]**o+(-0.105311421755)*x_ref[1]
        ref[1,3,0,0]=(0.373541105531)*x_ref[0]**o+(-0.895909586816)*x_ref[0]+(0.853523284159)*x_ref[1]**o+(0.915229883403)*x_ref[1]
        ref[1,3,0,1]=(-0.888315748473)*x_ref[0]**o+(0.877859466924)*x_ref[0]+(-0.830380928425)*x_ref[1]**o+(-0.25922026388)*x_ref[1]
        ref[1,3,1,0]=(-0.944775455365)*x_ref[0]**o+(0.694978915121)*x_ref[0]+(0.00346060944841)*x_ref[1]**o+(0.610096191934)*x_ref[1]
        ref[1,3,1,1]=(-0.175304757445)*x_ref[0]**o+(0.883688655763)*x_ref[0]+(-0.0260273718271)*x_ref[1]**o+(0.440569935385)*x_ref[1]
        ref[1,3,2,0]=(0.973418372218)*x_ref[0]**o+(-0.0725489149125)*x_ref[0]+(-0.790861964708)*x_ref[1]**o+(-0.764398676127)*x_ref[1]
        ref[1,3,2,1]=(0.188665821479)*x_ref[0]**o+(-0.694573366343)*x_ref[0]+(-0.860244164818)*x_ref[1]**o+(0.642736876841)*x_ref[1]
        ref[1,4,0,0]=(-0.619221357105)*x_ref[0]**o+(-0.0366612230467)*x_ref[0]+(0.920044684933)*x_ref[1]**o+(-0.901158565327)*x_ref[1]
        ref[1,4,0,1]=(-0.774227719954)*x_ref[0]**o+(0.471966691541)*x_ref[0]+(0.0706379157745)*x_ref[1]**o+(-0.238713884878)*x_ref[1]
        ref[1,4,1,0]=(0.863551422475)*x_ref[0]**o+(0.131078367733)*x_ref[0]+(0.458326651999)*x_ref[1]**o+(-0.750431841864)*x_ref[1]
        ref[1,4,1,1]=(0.105763306948)*x_ref[0]**o+(-0.254538567936)*x_ref[0]+(-0.559297954287)*x_ref[1]**o+(0.21981759162)*x_ref[1]
        ref[1,4,2,0]=(0.922873574086)*x_ref[0]**o+(-0.853230802788)*x_ref[0]+(-0.982798222021)*x_ref[1]**o+(-0.0866961947026)*x_ref[1]
        ref[1,4,2,1]=(-0.21664472336)*x_ref[0]**o+(-0.915876043584)*x_ref[0]+(0.6182964805)*x_ref[1]**o+(0.665515072302)*x_ref[1]
        ref[2,0,0,0]=(-0.491230924471)*x_ref[0]**o+(0.509670721146)*x_ref[0]+(-0.44580759248)*x_ref[1]**o+(-0.943168309946)*x_ref[1]
        ref[2,0,0,1]=(-0.0895802535424)*x_ref[0]**o+(0.0780292649662)*x_ref[0]+(-0.42293922141)*x_ref[1]**o+(-0.76928974668)*x_ref[1]
        ref[2,0,1,0]=(-0.404348433578)*x_ref[0]**o+(-0.986117091427)*x_ref[0]+(0.562569938626)*x_ref[1]**o+(-0.41119675761)*x_ref[1]
        ref[2,0,1,1]=(0.517587873308)*x_ref[0]**o+(0.0731347448674)*x_ref[0]+(0.830876774432)*x_ref[1]**o+(-0.690590335528)*x_ref[1]
        ref[2,0,2,0]=(0.600931766711)*x_ref[0]**o+(-0.786774940006)*x_ref[0]+(-0.285975800855)*x_ref[1]**o+(0.348122549519)*x_ref[1]
        ref[2,0,2,1]=(0.825033481951)*x_ref[0]**o+(-0.744551393479)*x_ref[0]+(0.452879660703)*x_ref[1]**o+(-0.257025658518)*x_ref[1]
        ref[2,1,0,0]=(-0.0822184021881)*x_ref[0]**o+(0.955001628077)*x_ref[0]+(-0.878009442237)*x_ref[1]**o+(-0.816970793701)*x_ref[1]
        ref[2,1,0,1]=(-0.151473282662)*x_ref[0]**o+(0.937636279558)*x_ref[0]+(-0.969793512643)*x_ref[1]**o+(0.704581030148)*x_ref[1]
        ref[2,1,1,0]=(0.76728625844)*x_ref[0]**o+(-0.90294390262)*x_ref[0]+(-0.375443980814)*x_ref[1]**o+(0.276219230561)*x_ref[1]
        ref[2,1,1,1]=(-0.994260771722)*x_ref[0]**o+(0.64595057257)*x_ref[0]+(0.486576661947)*x_ref[1]**o+(0.731738848734)*x_ref[1]
        ref[2,1,2,0]=(-0.52048273513)*x_ref[0]**o+(-0.51915057929)*x_ref[0]+(0.184855975415)*x_ref[1]**o+(0.417869171454)*x_ref[1]
        ref[2,1,2,1]=(-0.371019640816)*x_ref[0]**o+(-0.617014410896)*x_ref[0]+(0.725259821636)*x_ref[1]**o+(-0.0393667759542)*x_ref[1]
        ref[2,2,0,0]=(0.070044943001)*x_ref[0]**o+(-0.663066522147)*x_ref[0]+(0.761006491068)*x_ref[1]**o+(-0.137480662114)*x_ref[1]
        ref[2,2,0,1]=(-0.394035586896)*x_ref[0]**o+(0.802334732427)*x_ref[0]+(-0.191154645494)*x_ref[1]**o+(-0.278490557185)*x_ref[1]
        ref[2,2,1,0]=(0.0664932052956)*x_ref[0]**o+(0.938378455125)*x_ref[0]+(0.997119543949)*x_ref[1]**o+(-0.696820719616)*x_ref[1]
        ref[2,2,1,1]=(0.79758586389)*x_ref[0]**o+(0.0393291272874)*x_ref[0]+(0.644980805887)*x_ref[1]**o+(0.61925875842)*x_ref[1]
        ref[2,2,2,0]=(0.0353810358089)*x_ref[0]**o+(-0.0134652375886)*x_ref[0]+(-0.942381983803)*x_ref[1]**o+(0.288697669014)*x_ref[1]
        ref[2,2,2,1]=(-0.327577257504)*x_ref[0]**o+(0.816374902745)*x_ref[0]+(0.676853003934)*x_ref[1]**o+(0.581394632094)*x_ref[1]
        ref[2,3,0,0]=(0.63036222221)*x_ref[0]**o+(-0.899724810809)*x_ref[0]+(0.276882104091)*x_ref[1]**o+(-0.00448820301083)*x_ref[1]
        ref[2,3,0,1]=(-0.219899069159)*x_ref[0]**o+(-0.439244974227)*x_ref[0]+(-0.369366024932)*x_ref[1]**o+(-0.161805301574)*x_ref[1]
        ref[2,3,1,0]=(0.165202872055)*x_ref[0]**o+(-0.21521932039)*x_ref[0]+(0.635188695229)*x_ref[1]**o+(-0.254785217427)*x_ref[1]
        ref[2,3,1,1]=(0.743546803617)*x_ref[0]**o+(0.876160636955)*x_ref[0]+(-0.094730937905)*x_ref[1]**o+(-0.898315912992)*x_ref[1]
        ref[2,3,2,0]=(0.946538368626)*x_ref[0]**o+(0.715488078243)*x_ref[0]+(0.830112020529)*x_ref[1]**o+(-0.320605753644)*x_ref[1]
        ref[2,3,2,1]=(0.52106888624)*x_ref[0]**o+(-0.938072665733)*x_ref[0]+(-0.682830472037)*x_ref[1]**o+(-0.787442211636)*x_ref[1]
        ref[2,4,0,0]=(0.802634158133)*x_ref[0]**o+(0.350923640202)*x_ref[0]+(-0.142397660222)*x_ref[1]**o+(0.768827179039)*x_ref[1]
        ref[2,4,0,1]=(0.912207226957)*x_ref[0]**o+(0.302955742414)*x_ref[0]+(-0.125141454725)*x_ref[1]**o+(0.115375955845)*x_ref[1]
        ref[2,4,1,0]=(0.88629189066)*x_ref[0]**o+(-0.431145868333)*x_ref[0]+(0.521817592458)*x_ref[1]**o+(0.757372657079)*x_ref[1]
        ref[2,4,1,1]=(-0.34619470039)*x_ref[0]**o+(0.0971941022378)*x_ref[0]+(-0.235321219876)*x_ref[1]**o+(0.919340221916)*x_ref[1]
        ref[2,4,2,0]=(-0.453744205858)*x_ref[0]**o+(0.386107152737)*x_ref[0]+(-0.53829794769)*x_ref[1]**o+(-0.955985738202)*x_ref[1]
        ref[2,4,2,1]=(0.327564798989)*x_ref[0]**o+(0.234176645121)*x_ref[0]+(-0.647242055069)*x_ref[1]**o+(-0.914279617484)*x_ref[1]
        ref[3,0,0,0]=(-0.765850994681)*x_ref[0]**o+(0.0969021602734)*x_ref[0]+(-0.003697715708)*x_ref[1]**o+(0.642293903037)*x_ref[1]
        ref[3,0,0,1]=(-0.736912760299)*x_ref[0]**o+(-0.496756935434)*x_ref[0]+(0.00160221880808)*x_ref[1]**o+(-0.0275314787811)*x_ref[1]
        ref[3,0,1,0]=(0.266302600859)*x_ref[0]**o+(0.962232443249)*x_ref[0]+(0.588094142167)*x_ref[1]**o+(-0.740113379551)*x_ref[1]
        ref[3,0,1,1]=(-0.701204282707)*x_ref[0]**o+(0.16377449778)*x_ref[0]+(-0.636725536715)*x_ref[1]**o+(0.925340403623)*x_ref[1]
        ref[3,0,2,0]=(-0.473492482064)*x_ref[0]**o+(0.356976796364)*x_ref[0]+(0.28490257392)*x_ref[1]**o+(0.558859192591)*x_ref[1]
        ref[3,0,2,1]=(-0.398155056481)*x_ref[0]**o+(-0.693010191754)*x_ref[0]+(-0.457570700744)*x_ref[1]**o+(0.899032980684)*x_ref[1]
        ref[3,1,0,0]=(0.676626616862)*x_ref[0]**o+(-0.142127205425)*x_ref[0]+(-0.0703163466342)*x_ref[1]**o+(-0.603767804268)*x_ref[1]
        ref[3,1,0,1]=(0.901299559593)*x_ref[0]**o+(-0.319119050502)*x_ref[0]+(-0.230855076893)*x_ref[1]**o+(0.923789250137)*x_ref[1]
        ref[3,1,1,0]=(0.0125691057912)*x_ref[0]**o+(0.827401151454)*x_ref[0]+(0.0881382970418)*x_ref[1]**o+(0.12796232574)*x_ref[1]
        ref[3,1,1,1]=(-0.544917801861)*x_ref[0]**o+(-0.163923780568)*x_ref[0]+(0.107833995954)*x_ref[1]**o+(-0.347470857919)*x_ref[1]
        ref[3,1,2,0]=(-0.127281534891)*x_ref[0]**o+(0.410172113868)*x_ref[0]+(0.516300378909)*x_ref[1]**o+(-0.454031679856)*x_ref[1]
        ref[3,1,2,1]=(-0.797619552686)*x_ref[0]**o+(0.0212633210895)*x_ref[0]+(-0.0890328154993)*x_ref[1]**o+(0.755392852372)*x_ref[1]
        ref[3,2,0,0]=(0.00976392699076)*x_ref[0]**o+(-0.608813040196)*x_ref[0]+(0.786997931081)*x_ref[1]**o+(0.637450406747)*x_ref[1]
        ref[3,2,0,1]=(-0.594915968211)*x_ref[0]**o+(0.909106492284)*x_ref[0]+(0.209774113888)*x_ref[1]**o+(0.926156671099)*x_ref[1]
        ref[3,2,1,0]=(0.226007901841)*x_ref[0]**o+(-0.43873163217)*x_ref[0]+(-0.414778768373)*x_ref[1]**o+(0.681053132578)*x_ref[1]
        ref[3,2,1,1]=(0.733375971867)*x_ref[0]**o+(0.861188515019)*x_ref[0]+(0.036782797193)*x_ref[1]**o+(0.145181529782)*x_ref[1]
        ref[3,2,2,0]=(0.635727397799)*x_ref[0]**o+(0.874128725841)*x_ref[0]+(-0.810169847222)*x_ref[1]**o+(0.234369619159)*x_ref[1]
        ref[3,2,2,1]=(-0.137252742544)*x_ref[0]**o+(0.817564886208)*x_ref[0]+(0.524706682726)*x_ref[1]**o+(0.381735970369)*x_ref[1]
        ref[3,3,0,0]=(0.537368746859)*x_ref[0]**o+(-0.298760793455)*x_ref[0]+(-0.890807690497)*x_ref[1]**o+(-0.662222369966)*x_ref[1]
        ref[3,3,0,1]=(-0.67672238667)*x_ref[0]**o+(-0.305431277061)*x_ref[0]+(-0.435551021687)*x_ref[1]**o+(0.0777923105041)*x_ref[1]
        ref[3,3,1,0]=(-0.163284186346)*x_ref[0]**o+(-0.764058727495)*x_ref[0]+(-0.255689425142)*x_ref[1]**o+(0.285528254098)*x_ref[1]
        ref[3,3,1,1]=(-0.842634439814)*x_ref[0]**o+(-0.194811378723)*x_ref[0]+(0.814424721344)*x_ref[1]**o+(-0.236826785563)*x_ref[1]
        ref[3,3,2,0]=(0.532936591284)*x_ref[0]**o+(-0.920822945577)*x_ref[0]+(-0.677841412043)*x_ref[1]**o+(0.579981868038)*x_ref[1]
        ref[3,3,2,1]=(-0.470357673624)*x_ref[0]**o+(-0.972798479564)*x_ref[0]+(0.489692897505)*x_ref[1]**o+(0.213937860142)*x_ref[1]
        ref[3,4,0,0]=(-0.169170544642)*x_ref[0]**o+(0.543113611166)*x_ref[0]+(0.536912197355)*x_ref[1]**o+(0.304230092305)*x_ref[1]
        ref[3,4,0,1]=(-0.170701528868)*x_ref[0]**o+(0.340606153319)*x_ref[0]+(-0.470528666825)*x_ref[1]**o+(-0.522028583228)*x_ref[1]
        ref[3,4,1,0]=(0.0194532320642)*x_ref[0]**o+(0.335898702694)*x_ref[0]+(-0.917025993458)*x_ref[1]**o+(-0.539748486826)*x_ref[1]
        ref[3,4,1,1]=(-0.576465573138)*x_ref[0]**o+(-0.906917910142)*x_ref[0]+(-0.313000868178)*x_ref[1]**o+(-0.20607256753)*x_ref[1]
        ref[3,4,2,0]=(-0.75394368612)*x_ref[0]**o+(0.658866075222)*x_ref[0]+(0.402200040518)*x_ref[1]**o+(-0.544252067119)*x_ref[1]
        ref[3,4,2,1]=(0.0490609014064)*x_ref[0]**o+(-0.873887848405)*x_ref[0]+(0.91186722933)*x_ref[1]**o+(0.375269571672)*x_ref[1]
      else:
        s[0,0,0,0]=(-0.853604830323)*x[0]**o+(0.378465791865)*x[0]+(-0.83364075876)*x[1]**o+(0.212018195408)*x[1]+(0.48856925163)*x[2]**o+(-0.179831169531)*x[2]
        s[0,0,0,1]=(0.868673673397)*x[0]**o+(-0.238560476072)*x[0]+(0.828785612257)*x[1]**o+(0.176759782363)*x[1]+(0.525888840201)*x[2]**o+(0.436158744893)*x[2]
        s[0,0,1,0]=(0.887888476899)*x[0]**o+(0.458452579078)*x[0]+(0.572312971484)*x[1]**o+(0.694648297654)*x[1]+(0.244770581638)*x[2]**o+(-0.326433505221)*x[2]
        s[0,0,1,1]=(0.958595740398)*x[0]**o+(-0.448016561436)*x[0]+(-0.898608139656)*x[1]**o+(0.0189044802348)*x[1]+(0.664802638624)*x[2]**o+(0.675631565244)*x[2]
        s[0,0,2,0]=(-0.0968613941718)*x[0]**o+(-0.580198871721)*x[0]+(-0.695139695389)*x[1]**o+(0.670627088913)*x[1]+(-0.466392817977)*x[2]**o+(-0.414367914263)*x[2]
        s[0,0,2,1]=(0.0563724518255)*x[0]**o+(0.141716006991)*x[0]+(-0.889611317888)*x[1]**o+(0.274193692615)*x[1]+(0.81766066569)*x[2]**o+(0.260242356947)*x[2]
        s[0,1,0,0]=(0.729916171245)*x[0]**o+(-0.0512883787314)*x[0]+(-0.395879068642)*x[1]**o+(0.65153799151)*x[1]+(-0.90747241683)*x[2]**o+(0.556056661876)*x[2]
        s[0,1,0,1]=(-0.336962711878)*x[0]**o+(0.494126376731)*x[0]+(-0.0180540415242)*x[1]**o+(-0.281208191895)*x[1]+(-0.891281821502)*x[2]**o+(0.434438114521)*x[2]
        s[0,1,1,0]=(-0.0203012608797)*x[0]**o+(-0.592593200931)*x[0]+(-0.908091999823)*x[1]**o+(0.917265173937)*x[1]+(0.0113338093354)*x[2]**o+(0.832522528967)*x[2]
        s[0,1,1,1]=(-0.287082151578)*x[0]**o+(-0.882563505538)*x[0]+(-0.805577758903)*x[1]**o+(-0.0877632778656)*x[1]+(-0.4141073805)*x[2]**o+(-0.0809945294401)*x[2]
        s[0,1,2,0]=(-0.745887649438)*x[0]**o+(0.105423480913)*x[0]+(0.642374896965)*x[1]**o+(0.479730302425)*x[1]+(0.334298930575)*x[2]**o+(0.214588275036)*x[2]
        s[0,1,2,1]=(-0.987939537144)*x[0]**o+(-0.889107799492)*x[0]+(-0.133927268357)*x[1]**o+(-0.991450009918)*x[1]+(-0.201645805359)*x[2]**o+(0.481104989077)*x[2]
        s[0,2,0,0]=(0.535918222555)*x[0]**o+(0.243041294608)*x[0]+(-0.205082148634)*x[1]**o+(0.233470577382)*x[1]+(0.398471417689)*x[2]**o+(-0.56437758006)*x[2]
        s[0,2,0,1]=(0.374626206936)*x[0]**o+(-0.800840698958)*x[0]+(0.336427483065)*x[1]**o+(0.143368857985)*x[1]+(0.822396957468)*x[2]**o+(-0.288884862626)*x[2]
        s[0,2,1,0]=(-0.152217915896)*x[0]**o+(0.868324413095)*x[0]+(0.93760108412)*x[1]**o+(-0.584628102458)*x[1]+(-0.678653990131)*x[2]**o+(-0.130768977951)*x[2]
        s[0,2,1,1]=(-0.962809494754)*x[0]**o+(0.057963920559)*x[0]+(-0.68685683212)*x[1]**o+(0.869986883492)*x[1]+(-0.193987500788)*x[2]**o+(0.0357315899177)*x[2]
        s[0,2,2,0]=(-0.296222468619)*x[0]**o+(0.853293489381)*x[0]+(0.886622498642)*x[1]**o+(-0.22952370037)*x[1]+(0.444962301355)*x[2]**o+(0.498735104677)*x[2]
        s[0,2,2,1]=(-0.556796221789)*x[0]**o+(-0.98717630759)*x[0]+(-0.56578841851)*x[1]**o+(-0.769428379953)*x[1]+(0.41417947942)*x[2]**o+(-0.99075609451)*x[2]
        s[0,3,0,0]=(-0.312173122533)*x[0]**o+(-0.0183061811911)*x[0]+(0.632028466217)*x[1]**o+(-0.516798544207)*x[1]+(0.467776726567)*x[2]**o+(-0.0750898474112)*x[2]
        s[0,3,0,1]=(0.151818577102)*x[0]**o+(0.275659634397)*x[0]+(-0.695866418429)*x[1]**o+(-0.158484337329)*x[1]+(-0.717635621103)*x[2]**o+(0.96218471765)*x[2]
        s[0,3,1,0]=(-0.959269035858)*x[0]**o+(-0.694952577107)*x[0]+(0.133713794285)*x[1]**o+(0.945801566586)*x[1]+(0.480976843012)*x[2]**o+(0.514096820167)*x[2]
        s[0,3,1,1]=(-0.696429878723)*x[0]**o+(0.844116956105)*x[0]+(0.986596486924)*x[1]**o+(0.213400513251)*x[1]+(0.168567391983)*x[2]**o+(-0.96577037709)*x[2]
        s[0,3,2,0]=(0.272554167042)*x[0]**o+(-0.134445947881)*x[0]+(-0.620927727232)*x[1]**o+(0.774446825459)*x[1]+(-0.140360641548)*x[2]**o+(-0.393011913595)*x[2]
        s[0,3,2,1]=(0.973937360802)*x[0]**o+(-0.780203033305)*x[0]+(0.62508420402)*x[1]**o+(-0.957359234026)*x[1]+(0.301032273116)*x[2]**o+(0.238130119107)*x[2]
        s[0,4,0,0]=(0.550167138562)*x[0]**o+(-0.975271434354)*x[0]+(-0.90226839553)*x[1]**o+(0.765360109563)*x[1]+(-0.0539842939871)*x[2]**o+(-0.992811018253)*x[2]
        s[0,4,0,1]=(0.115680644235)*x[0]**o+(0.6504959069)*x[0]+(0.0711117194347)*x[1]**o+(0.765247742051)*x[1]+(0.651478097229)*x[2]**o+(0.0605973413193)*x[2]
        s[0,4,1,0]=(-0.339495686607)*x[0]**o+(0.98910452173)*x[0]+(-0.97593906201)*x[1]**o+(-0.961098427995)*x[1]+(-0.970981798475)*x[2]**o+(-0.655846307864)*x[2]
        s[0,4,1,1]=(-0.676422431554)*x[0]**o+(0.17779634387)*x[0]+(-0.101241446408)*x[1]**o+(0.592319052036)*x[1]+(-0.328471629455)*x[2]**o+(0.842981957639)*x[2]
        s[0,4,2,0]=(0.475154370559)*x[0]**o+(-0.861452809594)*x[0]+(0.021220095967)*x[1]**o+(-0.167324332634)*x[1]+(-0.420334858996)*x[2]**o+(-0.243503287995)*x[2]
        s[0,4,2,1]=(0.507818170756)*x[0]**o+(0.0305556062843)*x[0]+(0.657465103712)*x[1]**o+(-0.133306218785)*x[1]+(0.596182931411)*x[2]**o+(0.26354220091)*x[2]
        s[1,0,0,0]=(0.141665798979)*x[0]**o+(0.257839463098)*x[0]+(0.20842719092)*x[1]**o+(0.0606481804876)*x[1]+(0.564188524366)*x[2]**o+(0.276431563089)*x[2]
        s[1,0,0,1]=(-0.475871120292)*x[0]**o+(-0.945285882278)*x[0]+(-0.852943036141)*x[1]**o+(-0.393053044641)*x[1]+(0.894360138187)*x[2]**o+(0.945580362088)*x[2]
        s[1,0,1,0]=(-0.269113046186)*x[0]**o+(0.979226351783)*x[0]+(0.59588973168)*x[1]**o+(0.709182601617)*x[1]+(0.203809169118)*x[2]**o+(0.843792678336)*x[2]
        s[1,0,1,1]=(0.625152197275)*x[0]**o+(0.700601491781)*x[0]+(-0.556266788732)*x[1]**o+(0.621296929469)*x[1]+(0.370627217553)*x[2]**o+(-0.86981081047)*x[2]
        s[1,0,2,0]=(0.337314024742)*x[0]**o+(0.4936522561)*x[0]+(-0.121004948433)*x[1]**o+(0.682816106721)*x[1]+(0.135016952475)*x[2]**o+(0.447788511803)*x[2]
        s[1,0,2,1]=(0.627048609871)*x[0]**o+(0.464430671751)*x[0]+(0.248071099972)*x[1]**o+(0.49700045944)*x[1]+(0.764133255754)*x[2]**o+(-0.407441090409)*x[2]
        s[1,1,0,0]=(-0.891648124769)*x[0]**o+(-0.852500694418)*x[0]+(-0.501966394474)*x[1]**o+(-0.951197952585)*x[1]+(-0.293731401112)*x[2]**o+(0.340542703849)*x[2]
        s[1,1,0,1]=(0.047745963453)*x[0]**o+(0.605499497949)*x[0]+(0.603025931833)*x[1]**o+(0.412711498009)*x[1]+(-0.706970380941)*x[2]**o+(-0.865479677059)*x[2]
        s[1,1,1,0]=(-0.809326436683)*x[0]**o+(0.50103064436)*x[0]+(-0.726029943072)*x[1]**o+(-0.0136545501065)*x[1]+(0.869298450797)*x[2]**o+(-0.0539795856427)*x[2]
        s[1,1,1,1]=(0.822265437858)*x[0]**o+(-0.13686562101)*x[0]+(-0.921877204478)*x[1]**o+(0.436216049238)*x[1]+(0.167449871886)*x[2]**o+(-0.376043518724)*x[2]
        s[1,1,2,0]=(-0.240904110506)*x[0]**o+(-0.709486977651)*x[0]+(0.589748137401)*x[1]**o+(0.997759434219)*x[1]+(0.211495210823)*x[2]**o+(0.857954370359)*x[2]
        s[1,1,2,1]=(-0.872650055395)*x[0]**o+(0.488392921139)*x[0]+(0.456778341472)*x[1]**o+(0.533360514682)*x[1]+(0.660463672454)*x[2]**o+(-0.287728736994)*x[2]
        s[1,2,0,0]=(-0.56865288172)*x[0]**o+(-0.257707278342)*x[0]+(0.492925973387)*x[1]**o+(-0.433521245223)*x[1]+(-0.214711127244)*x[2]**o+(-0.578235847487)*x[2]
        s[1,2,0,1]=(0.502020345167)*x[0]**o+(-0.898730931974)*x[0]+(0.152379872963)*x[1]**o+(0.963247300695)*x[1]+(-0.935219139549)*x[2]**o+(-0.322095550311)*x[2]
        s[1,2,1,0]=(0.755425499434)*x[0]**o+(0.922722006611)*x[0]+(-0.854913831787)*x[1]**o+(0.28200116496)*x[1]+(0.654754619366)*x[2]**o+(-0.87246359532)*x[2]
        s[1,2,1,1]=(0.476533428226)*x[0]**o+(-0.21427962663)*x[0]+(-0.42807986959)*x[1]**o+(-0.453270655163)*x[1]+(-0.0618352146195)*x[2]**o+(-0.0578564702081)*x[2]
        s[1,2,2,0]=(0.836242039415)*x[0]**o+(0.0582519660745)*x[0]+(-0.736245953793)*x[1]**o+(-0.0216606630383)*x[1]+(-0.241691127038)*x[2]**o+(-0.273769285415)*x[2]
        s[1,2,2,1]=(0.956981101891)*x[0]**o+(-0.619600309731)*x[0]+(0.255999571352)*x[1]**o+(0.931443107924)*x[1]+(0.803455662831)*x[2]**o+(0.587378690917)*x[2]
        s[1,3,0,0]=(0.84725602389)*x[0]**o+(0.099929556134)*x[0]+(0.918513623927)*x[1]**o+(-0.540239895987)*x[1]+(0.543336053141)*x[2]**o+(-0.35339148993)*x[2]
        s[1,3,0,1]=(0.0790066953583)*x[0]**o+(-0.622984122413)*x[0]+(0.844477077556)*x[1]**o+(-0.843967484949)*x[1]+(0.382810149795)*x[2]**o+(0.693483174672)*x[2]
        s[1,3,1,0]=(-0.606850566406)*x[0]**o+(-0.613588487235)*x[0]+(-0.982801043706)*x[1]**o+(-0.200157044491)*x[1]+(0.865010142689)*x[2]**o+(-0.926404146796)*x[2]
        s[1,3,1,1]=(-0.926332516404)*x[0]**o+(-0.644403030771)*x[0]+(-0.391941644842)*x[1]**o+(-0.628555138555)*x[1]+(0.584646630734)*x[2]**o+(0.627561535294)*x[2]
        s[1,3,2,0]=(0.777960693446)*x[0]**o+(-0.580698013493)*x[0]+(-0.061588850878)*x[1]**o+(0.909167097211)*x[1]+(-0.346386067681)*x[2]**o+(-0.698593831635)*x[2]
        s[1,3,2,1]=(-0.25740731816)*x[0]**o+(0.0609560824945)*x[0]+(-0.866636970162)*x[1]**o+(0.352496912986)*x[1]+(0.212276243322)*x[2]**o+(-0.29499231054)*x[2]
        s[1,4,0,0]=(0.176828476382)*x[0]**o+(0.592116728712)*x[0]+(-0.969737040463)*x[1]**o+(-0.0140144291993)*x[1]+(-0.203791892462)*x[2]**o+(0.687043221294)*x[2]
        s[1,4,0,1]=(-0.0673562914955)*x[0]**o+(0.104813124019)*x[0]+(0.766386480162)*x[1]**o+(-0.499215759851)*x[1]+(0.163490109505)*x[2]**o+(0.0477449791182)*x[2]
        s[1,4,1,0]=(-0.186436883357)*x[0]**o+(0.771658464504)*x[0]+(0.115845567912)*x[1]**o+(-0.573016245745)*x[1]+(0.759680038826)*x[2]**o+(0.0660145528929)*x[2]
        s[1,4,1,1]=(-0.983746295741)*x[0]**o+(-0.370146292969)*x[0]+(-0.276398856773)*x[1]**o+(-0.765483196375)*x[1]+(-0.0802845394538)*x[2]**o+(0.762085198906)*x[2]
        s[1,4,2,0]=(-0.437323501168)*x[0]**o+(-0.076976193125)*x[0]+(0.780179691457)*x[1]**o+(0.941748527645)*x[1]+(0.921220023779)*x[2]**o+(-0.611350278287)*x[2]
        s[1,4,2,1]=(-0.811077839585)*x[0]**o+(-0.0781699097656)*x[0]+(-0.955979549084)*x[1]**o+(0.742581422414)*x[1]+(-0.905829236752)*x[2]**o+(-0.671257774729)*x[2]
        s[2,0,0,0]=(0.3602498958)*x[0]**o+(0.567952303965)*x[0]+(-0.72126509422)*x[1]**o+(0.652912038045)*x[1]+(-0.448081819441)*x[2]**o+(-0.628140847819)*x[2]
        s[2,0,0,1]=(0.823191679062)*x[0]**o+(-0.12280206123)*x[0]+(0.202149797631)*x[1]**o+(-0.751457177895)*x[1]+(0.367057556408)*x[2]**o+(0.71681510921)*x[2]
        s[2,0,1,0]=(-0.553277801352)*x[0]**o+(-0.695114786088)*x[0]+(-0.506695185331)*x[1]**o+(0.947900606544)*x[1]+(0.393574900465)*x[2]**o+(-0.131907977332)*x[2]
        s[2,0,1,1]=(-0.683260439471)*x[0]**o+(0.108921790681)*x[0]+(0.694361429808)*x[1]**o+(-0.879827908202)*x[1]+(-0.768400141399)*x[2]**o+(-0.869535466479)*x[2]
        s[2,0,2,0]=(-0.377844410367)*x[0]**o+(-0.486876364642)*x[0]+(-0.139990170975)*x[1]**o+(0.521294626989)*x[1]+(0.384721944946)*x[2]**o+(0.912206678795)*x[2]
        s[2,0,2,1]=(0.624159317849)*x[0]**o+(0.910540130065)*x[0]+(-0.879166404339)*x[1]**o+(0.710229335366)*x[1]+(-0.756806360688)*x[2]**o+(-0.169809484076)*x[2]
        s[2,1,0,0]=(-0.887833054092)*x[0]**o+(-0.541562581783)*x[0]+(-0.439734700726)*x[1]**o+(-0.294103567325)*x[1]+(0.621145429347)*x[2]**o+(0.65618994558)*x[2]
        s[2,1,0,1]=(-0.199707709698)*x[0]**o+(0.952895238716)*x[0]+(-0.113807788254)*x[1]**o+(-0.209497731554)*x[1]+(-0.201935349119)*x[2]**o+(0.896172767252)*x[2]
        s[2,1,1,0]=(0.961864846935)*x[0]**o+(0.34037006989)*x[0]+(0.97866208125)*x[1]**o+(0.949937962054)*x[1]+(-0.943237740253)*x[2]**o+(0.492751804379)*x[2]
        s[2,1,1,1]=(0.0364913119154)*x[0]**o+(-0.706276345936)*x[0]+(-0.0562330252965)*x[1]**o+(-0.919546510941)*x[1]+(0.376680654917)*x[2]**o+(-0.184153093526)*x[2]
        s[2,1,2,0]=(-0.135265768495)*x[0]**o+(0.650164259074)*x[0]+(0.224627083284)*x[1]**o+(0.918537471241)*x[1]+(0.662719594601)*x[2]**o+(-0.4451558313)*x[2]
        s[2,1,2,1]=(0.968484347906)*x[0]**o+(0.261215594011)*x[0]+(-0.478839261077)*x[1]**o+(-0.806554334141)*x[1]+(0.576441463783)*x[2]**o+(-0.407338671799)*x[2]
        s[2,2,0,0]=(-0.691105566226)*x[0]**o+(0.860347396649)*x[0]+(0.354066286829)*x[1]**o+(-0.0402175279153)*x[1]+(0.415237302648)*x[2]**o+(0.905209054794)*x[2]
        s[2,2,0,1]=(-0.548678761348)*x[0]**o+(-0.294803736974)*x[0]+(-0.960170319863)*x[1]**o+(-0.27333394559)*x[1]+(0.866348038561)*x[2]**o+(-0.977985070269)*x[2]
        s[2,2,1,0]=(-0.942843839038)*x[0]**o+(-0.868858708934)*x[0]+(-0.208988345792)*x[1]**o+(-0.628702405302)*x[1]+(0.856538967932)*x[2]**o+(-0.940751252093)*x[2]
        s[2,2,1,1]=(0.493835163998)*x[0]**o+(0.236452180515)*x[0]+(0.92903313506)*x[1]**o+(0.817122194244)*x[1]+(-0.498919897775)*x[2]**o+(0.177040227977)*x[2]
        s[2,2,2,0]=(-0.750931164998)*x[0]**o+(0.859659424304)*x[0]+(0.415426727903)*x[1]**o+(-0.468797574814)*x[1]+(-0.294847888148)*x[2]**o+(-0.992000733445)*x[2]
        s[2,2,2,1]=(0.0182583094659)*x[0]**o+(-0.830672513808)*x[0]+(0.37453609414)*x[1]**o+(-0.57449821779)*x[1]+(0.134490719543)*x[2]**o+(-0.609352246473)*x[2]
        s[2,3,0,0]=(-0.872131310391)*x[0]**o+(0.0825093683843)*x[0]+(-0.982804116245)*x[1]**o+(0.22341420851)*x[1]+(-0.231362455053)*x[2]**o+(-0.904631856625)*x[2]
        s[2,3,0,1]=(-0.0582824887741)*x[0]**o+(-0.399156766464)*x[0]+(0.0772313065438)*x[1]**o+(0.00426296268846)*x[1]+(-0.803184293158)*x[2]**o+(-0.17731453976)*x[2]
        s[2,3,1,0]=(0.252580558693)*x[0]**o+(-0.670939935472)*x[0]+(-0.396636904351)*x[1]**o+(-0.774693006552)*x[1]+(0.451637181152)*x[2]**o+(-0.463784221351)*x[2]
        s[2,3,1,1]=(0.126767645109)*x[0]**o+(-0.395030113635)*x[0]+(-0.955454976686)*x[1]**o+(-0.710220994617)*x[1]+(-0.397874215052)*x[2]**o+(-0.0217615150269)*x[2]
        s[2,3,2,0]=(0.990529883771)*x[0]**o+(0.278013247448)*x[0]+(-0.430442262972)*x[1]**o+(-0.720399995623)*x[1]+(0.610560781511)*x[2]**o+(-0.264692262808)*x[2]
        s[2,3,2,1]=(-0.776369460759)*x[0]**o+(0.397198841934)*x[0]+(-0.604527190952)*x[1]**o+(0.94368439705)*x[1]+(-0.88032288089)*x[2]**o+(0.989155014613)*x[2]
        s[2,4,0,0]=(0.457573740068)*x[0]**o+(-0.352003681775)*x[0]+(0.996428644418)*x[1]**o+(-0.348011873806)*x[1]+(-0.58023500034)*x[2]**o+(0.727228005186)*x[2]
        s[2,4,0,1]=(0.50989726873)*x[0]**o+(0.83338864798)*x[0]+(-0.111380969484)*x[1]**o+(-0.278057155647)*x[1]+(0.0022286233284)*x[2]**o+(0.841662666668)*x[2]
        s[2,4,1,0]=(-0.901603581146)*x[0]**o+(0.475205009348)*x[0]+(-0.430007910717)*x[1]**o+(-0.114746630672)*x[1]+(0.847548245931)*x[2]**o+(-0.539570230562)*x[2]
        s[2,4,1,1]=(0.539830165452)*x[0]**o+(-0.415758645187)*x[0]+(0.712970160554)*x[1]**o+(0.833164796207)*x[1]+(-0.65521876468)*x[2]**o+(-0.478072278167)*x[2]
        s[2,4,2,0]=(0.651879536878)*x[0]**o+(0.782082841616)*x[0]+(0.0416994850732)*x[1]**o+(0.0989265074417)*x[1]+(0.417171388449)*x[2]**o+(0.204544481473)*x[2]
        s[2,4,2,1]=(-0.957695309447)*x[0]**o+(0.41057815979)*x[0]+(0.847095166862)*x[1]**o+(0.937811945321)*x[1]+(-0.794606224039)*x[2]**o+(0.021545807724)*x[2]
        s[3,0,0,0]=(-0.521599558272)*x[0]**o+(-0.640095375476)*x[0]+(-0.51854438692)*x[1]**o+(-0.87210774891)*x[1]+(-0.693848689324)*x[2]**o+(-0.261635833944)*x[2]
        s[3,0,0,1]=(0.620197170573)*x[0]**o+(-0.841665901801)*x[0]+(-0.546759437483)*x[1]**o+(-0.168579145925)*x[1]+(-0.576567932552)*x[2]**o+(-0.645039556295)*x[2]
        s[3,0,1,0]=(0.581444084796)*x[0]**o+(-0.0462338578443)*x[0]+(0.117244760003)*x[1]**o+(0.732980884473)*x[1]+(-0.765006403884)*x[2]**o+(-0.543045061256)*x[2]
        s[3,0,1,1]=(0.695840005801)*x[0]**o+(-0.650014497536)*x[0]+(-0.801799125938)*x[1]**o+(-0.482643409576)*x[1]+(-0.416787187079)*x[2]**o+(0.584087905418)*x[2]
        s[3,0,2,0]=(-0.474618797242)*x[0]**o+(0.0189438474293)*x[0]+(-0.982174873996)*x[1]**o+(0.866178328676)*x[1]+(0.208673392242)*x[2]**o+(0.260241441013)*x[2]
        s[3,0,2,1]=(0.764105959702)*x[0]**o+(-0.578329182229)*x[0]+(0.556936430537)*x[1]**o+(0.0927824162943)*x[1]+(0.541941913665)*x[2]**o+(-0.20051953035)*x[2]
        s[3,1,0,0]=(0.467656474356)*x[0]**o+(0.20979187037)*x[0]+(0.830045363621)*x[1]**o+(0.305144956736)*x[1]+(-0.75827792525)*x[2]**o+(-0.336399882938)*x[2]
        s[3,1,0,1]=(-0.608709029175)*x[0]**o+(0.856296131715)*x[0]+(0.740185602478)*x[1]**o+(-0.899456455583)*x[1]+(-0.876657902509)*x[2]**o+(0.364216124717)*x[2]
        s[3,1,1,0]=(-0.19192395812)*x[0]**o+(0.106118249552)*x[0]+(-0.894241926881)*x[1]**o+(-0.985455697871)*x[1]+(-0.782102817165)*x[2]**o+(0.600080360854)*x[2]
        s[3,1,1,1]=(0.721311552863)*x[0]**o+(0.882962028856)*x[0]+(0.150876040487)*x[1]**o+(0.726055027812)*x[1]+(0.379204844464)*x[2]**o+(0.451150402164)*x[2]
        s[3,1,2,0]=(-0.694311490249)*x[0]**o+(-0.0875834002548)*x[0]+(-0.496200108766)*x[1]**o+(-0.151348317151)*x[1]+(0.854949924292)*x[2]**o+(0.915644549228)*x[2]
        s[3,1,2,1]=(0.470664286879)*x[0]**o+(0.706777924428)*x[0]+(-0.402719482127)*x[1]**o+(0.354894757591)*x[1]+(-0.472386640978)*x[2]**o+(0.947399400151)*x[2]
        s[3,2,0,0]=(-0.348423303161)*x[0]**o+(0.555948931484)*x[0]+(-0.43100597992)*x[1]**o+(-0.616049833968)*x[1]+(-0.150797584473)*x[2]**o+(0.520748537644)*x[2]
        s[3,2,0,1]=(0.138677687017)*x[0]**o+(0.18650212002)*x[0]+(0.260025448474)*x[1]**o+(0.0500586777217)*x[1]+(0.588917447067)*x[2]**o+(0.631694471884)*x[2]
        s[3,2,1,0]=(0.732017093416)*x[0]**o+(0.454265152556)*x[0]+(0.656152173314)*x[1]**o+(-0.214651040504)*x[1]+(0.606594898698)*x[2]**o+(-0.944529888173)*x[2]
        s[3,2,1,1]=(0.81276769768)*x[0]**o+(0.955839641144)*x[0]+(-0.597801282701)*x[1]**o+(0.203084441108)*x[1]+(0.371816930827)*x[2]**o+(-0.271843831057)*x[2]
        s[3,2,2,0]=(-0.0175874259272)*x[0]**o+(0.714416611293)*x[0]+(-0.446272670147)*x[1]**o+(-0.749224258055)*x[1]+(0.0434115533854)*x[2]**o+(-0.988008357199)*x[2]
        s[3,2,2,1]=(-0.219583104946)*x[0]**o+(0.207359013184)*x[0]+(-0.891027011221)*x[1]**o+(0.587776096493)*x[1]+(-0.545546707003)*x[2]**o+(-0.964582011253)*x[2]
        s[3,3,0,0]=(-0.540327832705)*x[0]**o+(0.489344676308)*x[0]+(0.663864603046)*x[1]**o+(-0.382248762805)*x[1]+(-0.729410073862)*x[2]**o+(-0.0325319636233)*x[2]
        s[3,3,0,1]=(0.259904022099)*x[0]**o+(-0.954974552252)*x[0]+(0.807969897038)*x[1]**o+(-0.535211587892)*x[1]+(0.401555747708)*x[2]**o+(0.80508329455)*x[2]
        s[3,3,1,0]=(0.172742587833)*x[0]**o+(-0.548601545874)*x[0]+(-0.991112338996)*x[1]**o+(-0.43054209437)*x[1]+(-0.32739577851)*x[2]**o+(-0.774125262769)*x[2]
        s[3,3,1,1]=(0.845744554328)*x[0]**o+(-0.258867050163)*x[0]+(-0.204629368407)*x[1]**o+(-0.821603302256)*x[1]+(-0.828098528219)*x[2]**o+(0.777673858589)*x[2]
        s[3,3,2,0]=(0.852991896102)*x[0]**o+(-0.745196256354)*x[0]+(0.220492132429)*x[1]**o+(-0.953335053523)*x[1]+(-0.324748822468)*x[2]**o+(-0.271392188657)*x[2]
        s[3,3,2,1]=(-0.256699792962)*x[0]**o+(0.670574693398)*x[0]+(0.538127693066)*x[1]**o+(-0.8291145795)*x[1]+(-0.152733016837)*x[2]**o+(0.401659214289)*x[2]
        s[3,4,0,0]=(-0.185294031294)*x[0]**o+(0.0683648982912)*x[0]+(-0.375470928752)*x[1]**o+(-0.67810003631)*x[1]+(-0.189259339657)*x[2]**o+(-0.174734934234)*x[2]
        s[3,4,0,1]=(-0.690376152222)*x[0]**o+(-0.021959548712)*x[0]+(0.816545400525)*x[1]**o+(0.743074566253)*x[1]+(0.907319374762)*x[2]**o+(-0.399104880981)*x[2]
        s[3,4,1,0]=(0.00764653168626)*x[0]**o+(0.418055838525)*x[0]+(-0.711231894711)*x[1]**o+(-0.319907118923)*x[1]+(0.230294344053)*x[2]**o+(0.852841918128)*x[2]
        s[3,4,1,1]=(0.773576800189)*x[0]**o+(-0.258243482679)*x[0]+(-0.217450374857)*x[1]**o+(0.0765524835501)*x[1]+(0.602823749066)*x[2]**o+(0.960046729433)*x[2]
        s[3,4,2,0]=(0.049879518732)*x[0]**o+(0.780027362911)*x[0]+(0.352479979369)*x[1]**o+(-0.960180931719)*x[1]+(0.220657155288)*x[2]**o+(-0.534538076664)*x[2]
        s[3,4,2,1]=(-0.260286357506)*x[0]**o+(0.319391110493)*x[0]+(0.0657794068301)*x[1]**o+(0.254797964539)*x[1]+(-0.495401153451)*x[2]**o+(-0.339881488587)*x[2]
        ref[0,0,0,0]=(-0.853604830323)*x_ref[0]**o+(0.378465791865)*x_ref[0]+(-0.83364075876)*x_ref[1]**o+(0.212018195408)*x_ref[1]+(0.48856925163)*x_ref[2]**o+(-0.179831169531)*x_ref[2]
        ref[0,0,0,1]=(0.868673673397)*x_ref[0]**o+(-0.238560476072)*x_ref[0]+(0.828785612257)*x_ref[1]**o+(0.176759782363)*x_ref[1]+(0.525888840201)*x_ref[2]**o+(0.436158744893)*x_ref[2]
        ref[0,0,1,0]=(0.887888476899)*x_ref[0]**o+(0.458452579078)*x_ref[0]+(0.572312971484)*x_ref[1]**o+(0.694648297654)*x_ref[1]+(0.244770581638)*x_ref[2]**o+(-0.326433505221)*x_ref[2]
        ref[0,0,1,1]=(0.958595740398)*x_ref[0]**o+(-0.448016561436)*x_ref[0]+(-0.898608139656)*x_ref[1]**o+(0.0189044802348)*x_ref[1]+(0.664802638624)*x_ref[2]**o+(0.675631565244)*x_ref[2]
        ref[0,0,2,0]=(-0.0968613941718)*x_ref[0]**o+(-0.580198871721)*x_ref[0]+(-0.695139695389)*x_ref[1]**o+(0.670627088913)*x_ref[1]+(-0.466392817977)*x_ref[2]**o+(-0.414367914263)*x_ref[2]
        ref[0,0,2,1]=(0.0563724518255)*x_ref[0]**o+(0.141716006991)*x_ref[0]+(-0.889611317888)*x_ref[1]**o+(0.274193692615)*x_ref[1]+(0.81766066569)*x_ref[2]**o+(0.260242356947)*x_ref[2]
        ref[0,1,0,0]=(0.729916171245)*x_ref[0]**o+(-0.0512883787314)*x_ref[0]+(-0.395879068642)*x_ref[1]**o+(0.65153799151)*x_ref[1]+(-0.90747241683)*x_ref[2]**o+(0.556056661876)*x_ref[2]
        ref[0,1,0,1]=(-0.336962711878)*x_ref[0]**o+(0.494126376731)*x_ref[0]+(-0.0180540415242)*x_ref[1]**o+(-0.281208191895)*x_ref[1]+(-0.891281821502)*x_ref[2]**o+(0.434438114521)*x_ref[2]
        ref[0,1,1,0]=(-0.0203012608797)*x_ref[0]**o+(-0.592593200931)*x_ref[0]+(-0.908091999823)*x_ref[1]**o+(0.917265173937)*x_ref[1]+(0.0113338093354)*x_ref[2]**o+(0.832522528967)*x_ref[2]
        ref[0,1,1,1]=(-0.287082151578)*x_ref[0]**o+(-0.882563505538)*x_ref[0]+(-0.805577758903)*x_ref[1]**o+(-0.0877632778656)*x_ref[1]+(-0.4141073805)*x_ref[2]**o+(-0.0809945294401)*x_ref[2]
        ref[0,1,2,0]=(-0.745887649438)*x_ref[0]**o+(0.105423480913)*x_ref[0]+(0.642374896965)*x_ref[1]**o+(0.479730302425)*x_ref[1]+(0.334298930575)*x_ref[2]**o+(0.214588275036)*x_ref[2]
        ref[0,1,2,1]=(-0.987939537144)*x_ref[0]**o+(-0.889107799492)*x_ref[0]+(-0.133927268357)*x_ref[1]**o+(-0.991450009918)*x_ref[1]+(-0.201645805359)*x_ref[2]**o+(0.481104989077)*x_ref[2]
        ref[0,2,0,0]=(0.535918222555)*x_ref[0]**o+(0.243041294608)*x_ref[0]+(-0.205082148634)*x_ref[1]**o+(0.233470577382)*x_ref[1]+(0.398471417689)*x_ref[2]**o+(-0.56437758006)*x_ref[2]
        ref[0,2,0,1]=(0.374626206936)*x_ref[0]**o+(-0.800840698958)*x_ref[0]+(0.336427483065)*x_ref[1]**o+(0.143368857985)*x_ref[1]+(0.822396957468)*x_ref[2]**o+(-0.288884862626)*x_ref[2]
        ref[0,2,1,0]=(-0.152217915896)*x_ref[0]**o+(0.868324413095)*x_ref[0]+(0.93760108412)*x_ref[1]**o+(-0.584628102458)*x_ref[1]+(-0.678653990131)*x_ref[2]**o+(-0.130768977951)*x_ref[2]
        ref[0,2,1,1]=(-0.962809494754)*x_ref[0]**o+(0.057963920559)*x_ref[0]+(-0.68685683212)*x_ref[1]**o+(0.869986883492)*x_ref[1]+(-0.193987500788)*x_ref[2]**o+(0.0357315899177)*x_ref[2]
        ref[0,2,2,0]=(-0.296222468619)*x_ref[0]**o+(0.853293489381)*x_ref[0]+(0.886622498642)*x_ref[1]**o+(-0.22952370037)*x_ref[1]+(0.444962301355)*x_ref[2]**o+(0.498735104677)*x_ref[2]
        ref[0,2,2,1]=(-0.556796221789)*x_ref[0]**o+(-0.98717630759)*x_ref[0]+(-0.56578841851)*x_ref[1]**o+(-0.769428379953)*x_ref[1]+(0.41417947942)*x_ref[2]**o+(-0.99075609451)*x_ref[2]
        ref[0,3,0,0]=(-0.312173122533)*x_ref[0]**o+(-0.0183061811911)*x_ref[0]+(0.632028466217)*x_ref[1]**o+(-0.516798544207)*x_ref[1]+(0.467776726567)*x_ref[2]**o+(-0.0750898474112)*x_ref[2]
        ref[0,3,0,1]=(0.151818577102)*x_ref[0]**o+(0.275659634397)*x_ref[0]+(-0.695866418429)*x_ref[1]**o+(-0.158484337329)*x_ref[1]+(-0.717635621103)*x_ref[2]**o+(0.96218471765)*x_ref[2]
        ref[0,3,1,0]=(-0.959269035858)*x_ref[0]**o+(-0.694952577107)*x_ref[0]+(0.133713794285)*x_ref[1]**o+(0.945801566586)*x_ref[1]+(0.480976843012)*x_ref[2]**o+(0.514096820167)*x_ref[2]
        ref[0,3,1,1]=(-0.696429878723)*x_ref[0]**o+(0.844116956105)*x_ref[0]+(0.986596486924)*x_ref[1]**o+(0.213400513251)*x_ref[1]+(0.168567391983)*x_ref[2]**o+(-0.96577037709)*x_ref[2]
        ref[0,3,2,0]=(0.272554167042)*x_ref[0]**o+(-0.134445947881)*x_ref[0]+(-0.620927727232)*x_ref[1]**o+(0.774446825459)*x_ref[1]+(-0.140360641548)*x_ref[2]**o+(-0.393011913595)*x_ref[2]
        ref[0,3,2,1]=(0.973937360802)*x_ref[0]**o+(-0.780203033305)*x_ref[0]+(0.62508420402)*x_ref[1]**o+(-0.957359234026)*x_ref[1]+(0.301032273116)*x_ref[2]**o+(0.238130119107)*x_ref[2]
        ref[0,4,0,0]=(0.550167138562)*x_ref[0]**o+(-0.975271434354)*x_ref[0]+(-0.90226839553)*x_ref[1]**o+(0.765360109563)*x_ref[1]+(-0.0539842939871)*x_ref[2]**o+(-0.992811018253)*x_ref[2]
        ref[0,4,0,1]=(0.115680644235)*x_ref[0]**o+(0.6504959069)*x_ref[0]+(0.0711117194347)*x_ref[1]**o+(0.765247742051)*x_ref[1]+(0.651478097229)*x_ref[2]**o+(0.0605973413193)*x_ref[2]
        ref[0,4,1,0]=(-0.339495686607)*x_ref[0]**o+(0.98910452173)*x_ref[0]+(-0.97593906201)*x_ref[1]**o+(-0.961098427995)*x_ref[1]+(-0.970981798475)*x_ref[2]**o+(-0.655846307864)*x_ref[2]
        ref[0,4,1,1]=(-0.676422431554)*x_ref[0]**o+(0.17779634387)*x_ref[0]+(-0.101241446408)*x_ref[1]**o+(0.592319052036)*x_ref[1]+(-0.328471629455)*x_ref[2]**o+(0.842981957639)*x_ref[2]
        ref[0,4,2,0]=(0.475154370559)*x_ref[0]**o+(-0.861452809594)*x_ref[0]+(0.021220095967)*x_ref[1]**o+(-0.167324332634)*x_ref[1]+(-0.420334858996)*x_ref[2]**o+(-0.243503287995)*x_ref[2]
        ref[0,4,2,1]=(0.507818170756)*x_ref[0]**o+(0.0305556062843)*x_ref[0]+(0.657465103712)*x_ref[1]**o+(-0.133306218785)*x_ref[1]+(0.596182931411)*x_ref[2]**o+(0.26354220091)*x_ref[2]
        ref[1,0,0,0]=(0.141665798979)*x_ref[0]**o+(0.257839463098)*x_ref[0]+(0.20842719092)*x_ref[1]**o+(0.0606481804876)*x_ref[1]+(0.564188524366)*x_ref[2]**o+(0.276431563089)*x_ref[2]
        ref[1,0,0,1]=(-0.475871120292)*x_ref[0]**o+(-0.945285882278)*x_ref[0]+(-0.852943036141)*x_ref[1]**o+(-0.393053044641)*x_ref[1]+(0.894360138187)*x_ref[2]**o+(0.945580362088)*x_ref[2]
        ref[1,0,1,0]=(-0.269113046186)*x_ref[0]**o+(0.979226351783)*x_ref[0]+(0.59588973168)*x_ref[1]**o+(0.709182601617)*x_ref[1]+(0.203809169118)*x_ref[2]**o+(0.843792678336)*x_ref[2]
        ref[1,0,1,1]=(0.625152197275)*x_ref[0]**o+(0.700601491781)*x_ref[0]+(-0.556266788732)*x_ref[1]**o+(0.621296929469)*x_ref[1]+(0.370627217553)*x_ref[2]**o+(-0.86981081047)*x_ref[2]
        ref[1,0,2,0]=(0.337314024742)*x_ref[0]**o+(0.4936522561)*x_ref[0]+(-0.121004948433)*x_ref[1]**o+(0.682816106721)*x_ref[1]+(0.135016952475)*x_ref[2]**o+(0.447788511803)*x_ref[2]
        ref[1,0,2,1]=(0.627048609871)*x_ref[0]**o+(0.464430671751)*x_ref[0]+(0.248071099972)*x_ref[1]**o+(0.49700045944)*x_ref[1]+(0.764133255754)*x_ref[2]**o+(-0.407441090409)*x_ref[2]
        ref[1,1,0,0]=(-0.891648124769)*x_ref[0]**o+(-0.852500694418)*x_ref[0]+(-0.501966394474)*x_ref[1]**o+(-0.951197952585)*x_ref[1]+(-0.293731401112)*x_ref[2]**o+(0.340542703849)*x_ref[2]
        ref[1,1,0,1]=(0.047745963453)*x_ref[0]**o+(0.605499497949)*x_ref[0]+(0.603025931833)*x_ref[1]**o+(0.412711498009)*x_ref[1]+(-0.706970380941)*x_ref[2]**o+(-0.865479677059)*x_ref[2]
        ref[1,1,1,0]=(-0.809326436683)*x_ref[0]**o+(0.50103064436)*x_ref[0]+(-0.726029943072)*x_ref[1]**o+(-0.0136545501065)*x_ref[1]+(0.869298450797)*x_ref[2]**o+(-0.0539795856427)*x_ref[2]
        ref[1,1,1,1]=(0.822265437858)*x_ref[0]**o+(-0.13686562101)*x_ref[0]+(-0.921877204478)*x_ref[1]**o+(0.436216049238)*x_ref[1]+(0.167449871886)*x_ref[2]**o+(-0.376043518724)*x_ref[2]
        ref[1,1,2,0]=(-0.240904110506)*x_ref[0]**o+(-0.709486977651)*x_ref[0]+(0.589748137401)*x_ref[1]**o+(0.997759434219)*x_ref[1]+(0.211495210823)*x_ref[2]**o+(0.857954370359)*x_ref[2]
        ref[1,1,2,1]=(-0.872650055395)*x_ref[0]**o+(0.488392921139)*x_ref[0]+(0.456778341472)*x_ref[1]**o+(0.533360514682)*x_ref[1]+(0.660463672454)*x_ref[2]**o+(-0.287728736994)*x_ref[2]
        ref[1,2,0,0]=(-0.56865288172)*x_ref[0]**o+(-0.257707278342)*x_ref[0]+(0.492925973387)*x_ref[1]**o+(-0.433521245223)*x_ref[1]+(-0.214711127244)*x_ref[2]**o+(-0.578235847487)*x_ref[2]
        ref[1,2,0,1]=(0.502020345167)*x_ref[0]**o+(-0.898730931974)*x_ref[0]+(0.152379872963)*x_ref[1]**o+(0.963247300695)*x_ref[1]+(-0.935219139549)*x_ref[2]**o+(-0.322095550311)*x_ref[2]
        ref[1,2,1,0]=(0.755425499434)*x_ref[0]**o+(0.922722006611)*x_ref[0]+(-0.854913831787)*x_ref[1]**o+(0.28200116496)*x_ref[1]+(0.654754619366)*x_ref[2]**o+(-0.87246359532)*x_ref[2]
        ref[1,2,1,1]=(0.476533428226)*x_ref[0]**o+(-0.21427962663)*x_ref[0]+(-0.42807986959)*x_ref[1]**o+(-0.453270655163)*x_ref[1]+(-0.0618352146195)*x_ref[2]**o+(-0.0578564702081)*x_ref[2]
        ref[1,2,2,0]=(0.836242039415)*x_ref[0]**o+(0.0582519660745)*x_ref[0]+(-0.736245953793)*x_ref[1]**o+(-0.0216606630383)*x_ref[1]+(-0.241691127038)*x_ref[2]**o+(-0.273769285415)*x_ref[2]
        ref[1,2,2,1]=(0.956981101891)*x_ref[0]**o+(-0.619600309731)*x_ref[0]+(0.255999571352)*x_ref[1]**o+(0.931443107924)*x_ref[1]+(0.803455662831)*x_ref[2]**o+(0.587378690917)*x_ref[2]
        ref[1,3,0,0]=(0.84725602389)*x_ref[0]**o+(0.099929556134)*x_ref[0]+(0.918513623927)*x_ref[1]**o+(-0.540239895987)*x_ref[1]+(0.543336053141)*x_ref[2]**o+(-0.35339148993)*x_ref[2]
        ref[1,3,0,1]=(0.0790066953583)*x_ref[0]**o+(-0.622984122413)*x_ref[0]+(0.844477077556)*x_ref[1]**o+(-0.843967484949)*x_ref[1]+(0.382810149795)*x_ref[2]**o+(0.693483174672)*x_ref[2]
        ref[1,3,1,0]=(-0.606850566406)*x_ref[0]**o+(-0.613588487235)*x_ref[0]+(-0.982801043706)*x_ref[1]**o+(-0.200157044491)*x_ref[1]+(0.865010142689)*x_ref[2]**o+(-0.926404146796)*x_ref[2]
        ref[1,3,1,1]=(-0.926332516404)*x_ref[0]**o+(-0.644403030771)*x_ref[0]+(-0.391941644842)*x_ref[1]**o+(-0.628555138555)*x_ref[1]+(0.584646630734)*x_ref[2]**o+(0.627561535294)*x_ref[2]
        ref[1,3,2,0]=(0.777960693446)*x_ref[0]**o+(-0.580698013493)*x_ref[0]+(-0.061588850878)*x_ref[1]**o+(0.909167097211)*x_ref[1]+(-0.346386067681)*x_ref[2]**o+(-0.698593831635)*x_ref[2]
        ref[1,3,2,1]=(-0.25740731816)*x_ref[0]**o+(0.0609560824945)*x_ref[0]+(-0.866636970162)*x_ref[1]**o+(0.352496912986)*x_ref[1]+(0.212276243322)*x_ref[2]**o+(-0.29499231054)*x_ref[2]
        ref[1,4,0,0]=(0.176828476382)*x_ref[0]**o+(0.592116728712)*x_ref[0]+(-0.969737040463)*x_ref[1]**o+(-0.0140144291993)*x_ref[1]+(-0.203791892462)*x_ref[2]**o+(0.687043221294)*x_ref[2]
        ref[1,4,0,1]=(-0.0673562914955)*x_ref[0]**o+(0.104813124019)*x_ref[0]+(0.766386480162)*x_ref[1]**o+(-0.499215759851)*x_ref[1]+(0.163490109505)*x_ref[2]**o+(0.0477449791182)*x_ref[2]
        ref[1,4,1,0]=(-0.186436883357)*x_ref[0]**o+(0.771658464504)*x_ref[0]+(0.115845567912)*x_ref[1]**o+(-0.573016245745)*x_ref[1]+(0.759680038826)*x_ref[2]**o+(0.0660145528929)*x_ref[2]
        ref[1,4,1,1]=(-0.983746295741)*x_ref[0]**o+(-0.370146292969)*x_ref[0]+(-0.276398856773)*x_ref[1]**o+(-0.765483196375)*x_ref[1]+(-0.0802845394538)*x_ref[2]**o+(0.762085198906)*x_ref[2]
        ref[1,4,2,0]=(-0.437323501168)*x_ref[0]**o+(-0.076976193125)*x_ref[0]+(0.780179691457)*x_ref[1]**o+(0.941748527645)*x_ref[1]+(0.921220023779)*x_ref[2]**o+(-0.611350278287)*x_ref[2]
        ref[1,4,2,1]=(-0.811077839585)*x_ref[0]**o+(-0.0781699097656)*x_ref[0]+(-0.955979549084)*x_ref[1]**o+(0.742581422414)*x_ref[1]+(-0.905829236752)*x_ref[2]**o+(-0.671257774729)*x_ref[2]
        ref[2,0,0,0]=(0.3602498958)*x_ref[0]**o+(0.567952303965)*x_ref[0]+(-0.72126509422)*x_ref[1]**o+(0.652912038045)*x_ref[1]+(-0.448081819441)*x_ref[2]**o+(-0.628140847819)*x_ref[2]
        ref[2,0,0,1]=(0.823191679062)*x_ref[0]**o+(-0.12280206123)*x_ref[0]+(0.202149797631)*x_ref[1]**o+(-0.751457177895)*x_ref[1]+(0.367057556408)*x_ref[2]**o+(0.71681510921)*x_ref[2]
        ref[2,0,1,0]=(-0.553277801352)*x_ref[0]**o+(-0.695114786088)*x_ref[0]+(-0.506695185331)*x_ref[1]**o+(0.947900606544)*x_ref[1]+(0.393574900465)*x_ref[2]**o+(-0.131907977332)*x_ref[2]
        ref[2,0,1,1]=(-0.683260439471)*x_ref[0]**o+(0.108921790681)*x_ref[0]+(0.694361429808)*x_ref[1]**o+(-0.879827908202)*x_ref[1]+(-0.768400141399)*x_ref[2]**o+(-0.869535466479)*x_ref[2]
        ref[2,0,2,0]=(-0.377844410367)*x_ref[0]**o+(-0.486876364642)*x_ref[0]+(-0.139990170975)*x_ref[1]**o+(0.521294626989)*x_ref[1]+(0.384721944946)*x_ref[2]**o+(0.912206678795)*x_ref[2]
        ref[2,0,2,1]=(0.624159317849)*x_ref[0]**o+(0.910540130065)*x_ref[0]+(-0.879166404339)*x_ref[1]**o+(0.710229335366)*x_ref[1]+(-0.756806360688)*x_ref[2]**o+(-0.169809484076)*x_ref[2]
        ref[2,1,0,0]=(-0.887833054092)*x_ref[0]**o+(-0.541562581783)*x_ref[0]+(-0.439734700726)*x_ref[1]**o+(-0.294103567325)*x_ref[1]+(0.621145429347)*x_ref[2]**o+(0.65618994558)*x_ref[2]
        ref[2,1,0,1]=(-0.199707709698)*x_ref[0]**o+(0.952895238716)*x_ref[0]+(-0.113807788254)*x_ref[1]**o+(-0.209497731554)*x_ref[1]+(-0.201935349119)*x_ref[2]**o+(0.896172767252)*x_ref[2]
        ref[2,1,1,0]=(0.961864846935)*x_ref[0]**o+(0.34037006989)*x_ref[0]+(0.97866208125)*x_ref[1]**o+(0.949937962054)*x_ref[1]+(-0.943237740253)*x_ref[2]**o+(0.492751804379)*x_ref[2]
        ref[2,1,1,1]=(0.0364913119154)*x_ref[0]**o+(-0.706276345936)*x_ref[0]+(-0.0562330252965)*x_ref[1]**o+(-0.919546510941)*x_ref[1]+(0.376680654917)*x_ref[2]**o+(-0.184153093526)*x_ref[2]
        ref[2,1,2,0]=(-0.135265768495)*x_ref[0]**o+(0.650164259074)*x_ref[0]+(0.224627083284)*x_ref[1]**o+(0.918537471241)*x_ref[1]+(0.662719594601)*x_ref[2]**o+(-0.4451558313)*x_ref[2]
        ref[2,1,2,1]=(0.968484347906)*x_ref[0]**o+(0.261215594011)*x_ref[0]+(-0.478839261077)*x_ref[1]**o+(-0.806554334141)*x_ref[1]+(0.576441463783)*x_ref[2]**o+(-0.407338671799)*x_ref[2]
        ref[2,2,0,0]=(-0.691105566226)*x_ref[0]**o+(0.860347396649)*x_ref[0]+(0.354066286829)*x_ref[1]**o+(-0.0402175279153)*x_ref[1]+(0.415237302648)*x_ref[2]**o+(0.905209054794)*x_ref[2]
        ref[2,2,0,1]=(-0.548678761348)*x_ref[0]**o+(-0.294803736974)*x_ref[0]+(-0.960170319863)*x_ref[1]**o+(-0.27333394559)*x_ref[1]+(0.866348038561)*x_ref[2]**o+(-0.977985070269)*x_ref[2]
        ref[2,2,1,0]=(-0.942843839038)*x_ref[0]**o+(-0.868858708934)*x_ref[0]+(-0.208988345792)*x_ref[1]**o+(-0.628702405302)*x_ref[1]+(0.856538967932)*x_ref[2]**o+(-0.940751252093)*x_ref[2]
        ref[2,2,1,1]=(0.493835163998)*x_ref[0]**o+(0.236452180515)*x_ref[0]+(0.92903313506)*x_ref[1]**o+(0.817122194244)*x_ref[1]+(-0.498919897775)*x_ref[2]**o+(0.177040227977)*x_ref[2]
        ref[2,2,2,0]=(-0.750931164998)*x_ref[0]**o+(0.859659424304)*x_ref[0]+(0.415426727903)*x_ref[1]**o+(-0.468797574814)*x_ref[1]+(-0.294847888148)*x_ref[2]**o+(-0.992000733445)*x_ref[2]
        ref[2,2,2,1]=(0.0182583094659)*x_ref[0]**o+(-0.830672513808)*x_ref[0]+(0.37453609414)*x_ref[1]**o+(-0.57449821779)*x_ref[1]+(0.134490719543)*x_ref[2]**o+(-0.609352246473)*x_ref[2]
        ref[2,3,0,0]=(-0.872131310391)*x_ref[0]**o+(0.0825093683843)*x_ref[0]+(-0.982804116245)*x_ref[1]**o+(0.22341420851)*x_ref[1]+(-0.231362455053)*x_ref[2]**o+(-0.904631856625)*x_ref[2]
        ref[2,3,0,1]=(-0.0582824887741)*x_ref[0]**o+(-0.399156766464)*x_ref[0]+(0.0772313065438)*x_ref[1]**o+(0.00426296268846)*x_ref[1]+(-0.803184293158)*x_ref[2]**o+(-0.17731453976)*x_ref[2]
        ref[2,3,1,0]=(0.252580558693)*x_ref[0]**o+(-0.670939935472)*x_ref[0]+(-0.396636904351)*x_ref[1]**o+(-0.774693006552)*x_ref[1]+(0.451637181152)*x_ref[2]**o+(-0.463784221351)*x_ref[2]
        ref[2,3,1,1]=(0.126767645109)*x_ref[0]**o+(-0.395030113635)*x_ref[0]+(-0.955454976686)*x_ref[1]**o+(-0.710220994617)*x_ref[1]+(-0.397874215052)*x_ref[2]**o+(-0.0217615150269)*x_ref[2]
        ref[2,3,2,0]=(0.990529883771)*x_ref[0]**o+(0.278013247448)*x_ref[0]+(-0.430442262972)*x_ref[1]**o+(-0.720399995623)*x_ref[1]+(0.610560781511)*x_ref[2]**o+(-0.264692262808)*x_ref[2]
        ref[2,3,2,1]=(-0.776369460759)*x_ref[0]**o+(0.397198841934)*x_ref[0]+(-0.604527190952)*x_ref[1]**o+(0.94368439705)*x_ref[1]+(-0.88032288089)*x_ref[2]**o+(0.989155014613)*x_ref[2]
        ref[2,4,0,0]=(0.457573740068)*x_ref[0]**o+(-0.352003681775)*x_ref[0]+(0.996428644418)*x_ref[1]**o+(-0.348011873806)*x_ref[1]+(-0.58023500034)*x_ref[2]**o+(0.727228005186)*x_ref[2]
        ref[2,4,0,1]=(0.50989726873)*x_ref[0]**o+(0.83338864798)*x_ref[0]+(-0.111380969484)*x_ref[1]**o+(-0.278057155647)*x_ref[1]+(0.0022286233284)*x_ref[2]**o+(0.841662666668)*x_ref[2]
        ref[2,4,1,0]=(-0.901603581146)*x_ref[0]**o+(0.475205009348)*x_ref[0]+(-0.430007910717)*x_ref[1]**o+(-0.114746630672)*x_ref[1]+(0.847548245931)*x_ref[2]**o+(-0.539570230562)*x_ref[2]
        ref[2,4,1,1]=(0.539830165452)*x_ref[0]**o+(-0.415758645187)*x_ref[0]+(0.712970160554)*x_ref[1]**o+(0.833164796207)*x_ref[1]+(-0.65521876468)*x_ref[2]**o+(-0.478072278167)*x_ref[2]
        ref[2,4,2,0]=(0.651879536878)*x_ref[0]**o+(0.782082841616)*x_ref[0]+(0.0416994850732)*x_ref[1]**o+(0.0989265074417)*x_ref[1]+(0.417171388449)*x_ref[2]**o+(0.204544481473)*x_ref[2]
        ref[2,4,2,1]=(-0.957695309447)*x_ref[0]**o+(0.41057815979)*x_ref[0]+(0.847095166862)*x_ref[1]**o+(0.937811945321)*x_ref[1]+(-0.794606224039)*x_ref[2]**o+(0.021545807724)*x_ref[2]
        ref[3,0,0,0]=(-0.521599558272)*x_ref[0]**o+(-0.640095375476)*x_ref[0]+(-0.51854438692)*x_ref[1]**o+(-0.87210774891)*x_ref[1]+(-0.693848689324)*x_ref[2]**o+(-0.261635833944)*x_ref[2]
        ref[3,0,0,1]=(0.620197170573)*x_ref[0]**o+(-0.841665901801)*x_ref[0]+(-0.546759437483)*x_ref[1]**o+(-0.168579145925)*x_ref[1]+(-0.576567932552)*x_ref[2]**o+(-0.645039556295)*x_ref[2]
        ref[3,0,1,0]=(0.581444084796)*x_ref[0]**o+(-0.0462338578443)*x_ref[0]+(0.117244760003)*x_ref[1]**o+(0.732980884473)*x_ref[1]+(-0.765006403884)*x_ref[2]**o+(-0.543045061256)*x_ref[2]
        ref[3,0,1,1]=(0.695840005801)*x_ref[0]**o+(-0.650014497536)*x_ref[0]+(-0.801799125938)*x_ref[1]**o+(-0.482643409576)*x_ref[1]+(-0.416787187079)*x_ref[2]**o+(0.584087905418)*x_ref[2]
        ref[3,0,2,0]=(-0.474618797242)*x_ref[0]**o+(0.0189438474293)*x_ref[0]+(-0.982174873996)*x_ref[1]**o+(0.866178328676)*x_ref[1]+(0.208673392242)*x_ref[2]**o+(0.260241441013)*x_ref[2]
        ref[3,0,2,1]=(0.764105959702)*x_ref[0]**o+(-0.578329182229)*x_ref[0]+(0.556936430537)*x_ref[1]**o+(0.0927824162943)*x_ref[1]+(0.541941913665)*x_ref[2]**o+(-0.20051953035)*x_ref[2]
        ref[3,1,0,0]=(0.467656474356)*x_ref[0]**o+(0.20979187037)*x_ref[0]+(0.830045363621)*x_ref[1]**o+(0.305144956736)*x_ref[1]+(-0.75827792525)*x_ref[2]**o+(-0.336399882938)*x_ref[2]
        ref[3,1,0,1]=(-0.608709029175)*x_ref[0]**o+(0.856296131715)*x_ref[0]+(0.740185602478)*x_ref[1]**o+(-0.899456455583)*x_ref[1]+(-0.876657902509)*x_ref[2]**o+(0.364216124717)*x_ref[2]
        ref[3,1,1,0]=(-0.19192395812)*x_ref[0]**o+(0.106118249552)*x_ref[0]+(-0.894241926881)*x_ref[1]**o+(-0.985455697871)*x_ref[1]+(-0.782102817165)*x_ref[2]**o+(0.600080360854)*x_ref[2]
        ref[3,1,1,1]=(0.721311552863)*x_ref[0]**o+(0.882962028856)*x_ref[0]+(0.150876040487)*x_ref[1]**o+(0.726055027812)*x_ref[1]+(0.379204844464)*x_ref[2]**o+(0.451150402164)*x_ref[2]
        ref[3,1,2,0]=(-0.694311490249)*x_ref[0]**o+(-0.0875834002548)*x_ref[0]+(-0.496200108766)*x_ref[1]**o+(-0.151348317151)*x_ref[1]+(0.854949924292)*x_ref[2]**o+(0.915644549228)*x_ref[2]
        ref[3,1,2,1]=(0.470664286879)*x_ref[0]**o+(0.706777924428)*x_ref[0]+(-0.402719482127)*x_ref[1]**o+(0.354894757591)*x_ref[1]+(-0.472386640978)*x_ref[2]**o+(0.947399400151)*x_ref[2]
        ref[3,2,0,0]=(-0.348423303161)*x_ref[0]**o+(0.555948931484)*x_ref[0]+(-0.43100597992)*x_ref[1]**o+(-0.616049833968)*x_ref[1]+(-0.150797584473)*x_ref[2]**o+(0.520748537644)*x_ref[2]
        ref[3,2,0,1]=(0.138677687017)*x_ref[0]**o+(0.18650212002)*x_ref[0]+(0.260025448474)*x_ref[1]**o+(0.0500586777217)*x_ref[1]+(0.588917447067)*x_ref[2]**o+(0.631694471884)*x_ref[2]
        ref[3,2,1,0]=(0.732017093416)*x_ref[0]**o+(0.454265152556)*x_ref[0]+(0.656152173314)*x_ref[1]**o+(-0.214651040504)*x_ref[1]+(0.606594898698)*x_ref[2]**o+(-0.944529888173)*x_ref[2]
        ref[3,2,1,1]=(0.81276769768)*x_ref[0]**o+(0.955839641144)*x_ref[0]+(-0.597801282701)*x_ref[1]**o+(0.203084441108)*x_ref[1]+(0.371816930827)*x_ref[2]**o+(-0.271843831057)*x_ref[2]
        ref[3,2,2,0]=(-0.0175874259272)*x_ref[0]**o+(0.714416611293)*x_ref[0]+(-0.446272670147)*x_ref[1]**o+(-0.749224258055)*x_ref[1]+(0.0434115533854)*x_ref[2]**o+(-0.988008357199)*x_ref[2]
        ref[3,2,2,1]=(-0.219583104946)*x_ref[0]**o+(0.207359013184)*x_ref[0]+(-0.891027011221)*x_ref[1]**o+(0.587776096493)*x_ref[1]+(-0.545546707003)*x_ref[2]**o+(-0.964582011253)*x_ref[2]
        ref[3,3,0,0]=(-0.540327832705)*x_ref[0]**o+(0.489344676308)*x_ref[0]+(0.663864603046)*x_ref[1]**o+(-0.382248762805)*x_ref[1]+(-0.729410073862)*x_ref[2]**o+(-0.0325319636233)*x_ref[2]
        ref[3,3,0,1]=(0.259904022099)*x_ref[0]**o+(-0.954974552252)*x_ref[0]+(0.807969897038)*x_ref[1]**o+(-0.535211587892)*x_ref[1]+(0.401555747708)*x_ref[2]**o+(0.80508329455)*x_ref[2]
        ref[3,3,1,0]=(0.172742587833)*x_ref[0]**o+(-0.548601545874)*x_ref[0]+(-0.991112338996)*x_ref[1]**o+(-0.43054209437)*x_ref[1]+(-0.32739577851)*x_ref[2]**o+(-0.774125262769)*x_ref[2]
        ref[3,3,1,1]=(0.845744554328)*x_ref[0]**o+(-0.258867050163)*x_ref[0]+(-0.204629368407)*x_ref[1]**o+(-0.821603302256)*x_ref[1]+(-0.828098528219)*x_ref[2]**o+(0.777673858589)*x_ref[2]
        ref[3,3,2,0]=(0.852991896102)*x_ref[0]**o+(-0.745196256354)*x_ref[0]+(0.220492132429)*x_ref[1]**o+(-0.953335053523)*x_ref[1]+(-0.324748822468)*x_ref[2]**o+(-0.271392188657)*x_ref[2]
        ref[3,3,2,1]=(-0.256699792962)*x_ref[0]**o+(0.670574693398)*x_ref[0]+(0.538127693066)*x_ref[1]**o+(-0.8291145795)*x_ref[1]+(-0.152733016837)*x_ref[2]**o+(0.401659214289)*x_ref[2]
        ref[3,4,0,0]=(-0.185294031294)*x_ref[0]**o+(0.0683648982912)*x_ref[0]+(-0.375470928752)*x_ref[1]**o+(-0.67810003631)*x_ref[1]+(-0.189259339657)*x_ref[2]**o+(-0.174734934234)*x_ref[2]
        ref[3,4,0,1]=(-0.690376152222)*x_ref[0]**o+(-0.021959548712)*x_ref[0]+(0.816545400525)*x_ref[1]**o+(0.743074566253)*x_ref[1]+(0.907319374762)*x_ref[2]**o+(-0.399104880981)*x_ref[2]
        ref[3,4,1,0]=(0.00764653168626)*x_ref[0]**o+(0.418055838525)*x_ref[0]+(-0.711231894711)*x_ref[1]**o+(-0.319907118923)*x_ref[1]+(0.230294344053)*x_ref[2]**o+(0.852841918128)*x_ref[2]
        ref[3,4,1,1]=(0.773576800189)*x_ref[0]**o+(-0.258243482679)*x_ref[0]+(-0.217450374857)*x_ref[1]**o+(0.0765524835501)*x_ref[1]+(0.602823749066)*x_ref[2]**o+(0.960046729433)*x_ref[2]
        ref[3,4,2,0]=(0.049879518732)*x_ref[0]**o+(0.780027362911)*x_ref[0]+(0.352479979369)*x_ref[1]**o+(-0.960180931719)*x_ref[1]+(0.220657155288)*x_ref[2]**o+(-0.534538076664)*x_ref[2]
        ref[3,4,2,1]=(-0.260286357506)*x_ref[0]**o+(0.319391110493)*x_ref[0]+(0.0657794068301)*x_ref[1]**o+(0.254797964539)*x_ref[1]+(-0.495401153451)*x_ref[2]**o+(-0.339881488587)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_ReducedFunctionOnContactOne_rank0(self):
      """
      tests interpolate for rank 0 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactOne onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactOne(self.domain)
      x=w.getX()
      arg=Symbol(shape=(),dim=dim)
      s=Data(0,(),w)
      ref=Data(0,(),w_ref)
      if dim==2:
        s=(0.992324887623)*x[0]**o+(-0.399021408175)*x[0]+(-0.82044668182)*x[1]**o+(-0.115930766833)*x[1]
        ref=(0.992324887623)*x_ref[0]**o+(-0.399021408175)*x_ref[0]+(-0.82044668182)*x_ref[1]**o+(-0.115930766833)*x_ref[1]
      else:
        s=(-0.917591389532)*x[0]**o+(-0.636147522121)*x[0]+(-0.624726334305)*x[1]**o+(0.253824967861)*x[1]+(0.391376692454)*x[2]**o+(-0.635535244093)*x[2]
        ref=(-0.917591389532)*x_ref[0]**o+(-0.636147522121)*x_ref[0]+(-0.624726334305)*x_ref[1]**o+(0.253824967861)*x_ref[1]+(0.391376692454)*x_ref[2]**o+(-0.635535244093)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_ReducedFunctionOnContactOne_rank1(self):
      """
      tests interpolate for rank 1 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactOne onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactOne(self.domain)
      x=w.getX()
      arg=Symbol(shape=(2,),dim=dim)
      s=Data(0,(2,),w)
      ref=Data(0,(2,),w_ref)
      if dim==2:
        s[0]=(-0.420392705289)*x[0]**o+(-0.988154722022)*x[0]+(-0.865378713801)*x[1]**o+(0.992312897132)*x[1]
        s[1]=(-0.48354228784)*x[0]**o+(0.07109068872)*x[0]+(0.757923528294)*x[1]**o+(0.903265965794)*x[1]
        ref[0]=(-0.420392705289)*x_ref[0]**o+(-0.988154722022)*x_ref[0]+(-0.865378713801)*x_ref[1]**o+(0.992312897132)*x_ref[1]
        ref[1]=(-0.48354228784)*x_ref[0]**o+(0.07109068872)*x_ref[0]+(0.757923528294)*x_ref[1]**o+(0.903265965794)*x_ref[1]
      else:
        s[0]=(-0.119618139601)*x[0]**o+(0.674458706959)*x[0]+(0.283506026525)*x[1]**o+(0.867334648745)*x[1]+(0.793263586089)*x[2]**o+(-0.225225247588)*x[2]
        s[1]=(-0.458483133737)*x[0]**o+(0.636334230541)*x[0]+(0.433152852396)*x[1]**o+(0.946877061205)*x[1]+(-0.633441461314)*x[2]**o+(-0.00536782448735)*x[2]
        ref[0]=(-0.119618139601)*x_ref[0]**o+(0.674458706959)*x_ref[0]+(0.283506026525)*x_ref[1]**o+(0.867334648745)*x_ref[1]+(0.793263586089)*x_ref[2]**o+(-0.225225247588)*x_ref[2]
        ref[1]=(-0.458483133737)*x_ref[0]**o+(0.636334230541)*x_ref[0]+(0.433152852396)*x_ref[1]**o+(0.946877061205)*x_ref[1]+(-0.633441461314)*x_ref[2]**o+(-0.00536782448735)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_ReducedFunctionOnContactOne_rank2(self):
      """
      tests interpolate for rank 2 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactOne onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactOne(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5),dim=dim)
      s=Data(0,(4, 5),w)
      ref=Data(0,(4, 5),w_ref)
      if dim==2:
        s[0,0]=(-0.230877307866)*x[0]**o+(0.215021920364)*x[0]+(0.72160607128)*x[1]**o+(0.288782620524)*x[1]
        s[0,1]=(-0.397089347754)*x[0]**o+(-0.859308303321)*x[0]+(-0.655908162636)*x[1]**o+(-0.75086945631)*x[1]
        s[0,2]=(0.17849428647)*x[0]**o+(0.190815421543)*x[0]+(0.972002884101)*x[1]**o+(0.4108901607)*x[1]
        s[0,3]=(-0.288371936253)*x[0]**o+(-0.394846982668)*x[0]+(0.00326108093218)*x[1]**o+(0.218771269587)*x[1]
        s[0,4]=(0.629096144369)*x[0]**o+(0.820908159894)*x[0]+(0.691458623728)*x[1]**o+(0.214095362021)*x[1]
        s[1,0]=(0.709456114123)*x[0]**o+(-0.837635170883)*x[0]+(0.469741659937)*x[1]**o+(-0.225676816505)*x[1]
        s[1,1]=(-0.429024930015)*x[0]**o+(0.332522034373)*x[0]+(-0.14085265632)*x[1]**o+(0.739644399991)*x[1]
        s[1,2]=(-0.550855064281)*x[0]**o+(0.950491676704)*x[0]+(0.468631317567)*x[1]**o+(-0.78396330626)*x[1]
        s[1,3]=(0.0713337235427)*x[0]**o+(-0.371870909904)*x[0]+(-0.329491599348)*x[1]**o+(0.0765857177272)*x[1]
        s[1,4]=(-0.619922856297)*x[0]**o+(-0.230137321592)*x[0]+(0.883918096123)*x[1]**o+(-0.164797122993)*x[1]
        s[2,0]=(0.577972603997)*x[0]**o+(-0.535427767528)*x[0]+(0.105653927239)*x[1]**o+(-0.803115354047)*x[1]
        s[2,1]=(-0.941298066772)*x[0]**o+(0.633879774328)*x[0]+(-0.769467220886)*x[1]**o+(-0.830635312894)*x[1]
        s[2,2]=(-0.143658254898)*x[0]**o+(0.612045970085)*x[0]+(-0.234784912386)*x[1]**o+(-0.111427283432)*x[1]
        s[2,3]=(-0.0806971603544)*x[0]**o+(-0.478260278718)*x[0]+(0.405214636774)*x[1]**o+(-0.0171544079949)*x[1]
        s[2,4]=(-0.619407716965)*x[0]**o+(0.292456447643)*x[0]+(-0.404895807117)*x[1]**o+(0.955367226581)*x[1]
        s[3,0]=(-0.762723098154)*x[0]**o+(-0.547004242664)*x[0]+(-0.266917906271)*x[1]**o+(0.600697811809)*x[1]
        s[3,1]=(-0.567148999707)*x[0]**o+(0.374593601186)*x[0]+(-0.440814971395)*x[1]**o+(-0.737145147183)*x[1]
        s[3,2]=(0.630637925234)*x[0]**o+(-0.0884747935175)*x[0]+(-0.0319938816017)*x[1]**o+(0.865297163842)*x[1]
        s[3,3]=(-0.811769976054)*x[0]**o+(0.533258797244)*x[0]+(0.735904279517)*x[1]**o+(0.362604687369)*x[1]
        s[3,4]=(-0.711314210671)*x[0]**o+(-0.270182873191)*x[0]+(-0.965300380176)*x[1]**o+(0.50703939773)*x[1]
        ref[0,0]=(-0.230877307866)*x_ref[0]**o+(0.215021920364)*x_ref[0]+(0.72160607128)*x_ref[1]**o+(0.288782620524)*x_ref[1]
        ref[0,1]=(-0.397089347754)*x_ref[0]**o+(-0.859308303321)*x_ref[0]+(-0.655908162636)*x_ref[1]**o+(-0.75086945631)*x_ref[1]
        ref[0,2]=(0.17849428647)*x_ref[0]**o+(0.190815421543)*x_ref[0]+(0.972002884101)*x_ref[1]**o+(0.4108901607)*x_ref[1]
        ref[0,3]=(-0.288371936253)*x_ref[0]**o+(-0.394846982668)*x_ref[0]+(0.00326108093218)*x_ref[1]**o+(0.218771269587)*x_ref[1]
        ref[0,4]=(0.629096144369)*x_ref[0]**o+(0.820908159894)*x_ref[0]+(0.691458623728)*x_ref[1]**o+(0.214095362021)*x_ref[1]
        ref[1,0]=(0.709456114123)*x_ref[0]**o+(-0.837635170883)*x_ref[0]+(0.469741659937)*x_ref[1]**o+(-0.225676816505)*x_ref[1]
        ref[1,1]=(-0.429024930015)*x_ref[0]**o+(0.332522034373)*x_ref[0]+(-0.14085265632)*x_ref[1]**o+(0.739644399991)*x_ref[1]
        ref[1,2]=(-0.550855064281)*x_ref[0]**o+(0.950491676704)*x_ref[0]+(0.468631317567)*x_ref[1]**o+(-0.78396330626)*x_ref[1]
        ref[1,3]=(0.0713337235427)*x_ref[0]**o+(-0.371870909904)*x_ref[0]+(-0.329491599348)*x_ref[1]**o+(0.0765857177272)*x_ref[1]
        ref[1,4]=(-0.619922856297)*x_ref[0]**o+(-0.230137321592)*x_ref[0]+(0.883918096123)*x_ref[1]**o+(-0.164797122993)*x_ref[1]
        ref[2,0]=(0.577972603997)*x_ref[0]**o+(-0.535427767528)*x_ref[0]+(0.105653927239)*x_ref[1]**o+(-0.803115354047)*x_ref[1]
        ref[2,1]=(-0.941298066772)*x_ref[0]**o+(0.633879774328)*x_ref[0]+(-0.769467220886)*x_ref[1]**o+(-0.830635312894)*x_ref[1]
        ref[2,2]=(-0.143658254898)*x_ref[0]**o+(0.612045970085)*x_ref[0]+(-0.234784912386)*x_ref[1]**o+(-0.111427283432)*x_ref[1]
        ref[2,3]=(-0.0806971603544)*x_ref[0]**o+(-0.478260278718)*x_ref[0]+(0.405214636774)*x_ref[1]**o+(-0.0171544079949)*x_ref[1]
        ref[2,4]=(-0.619407716965)*x_ref[0]**o+(0.292456447643)*x_ref[0]+(-0.404895807117)*x_ref[1]**o+(0.955367226581)*x_ref[1]
        ref[3,0]=(-0.762723098154)*x_ref[0]**o+(-0.547004242664)*x_ref[0]+(-0.266917906271)*x_ref[1]**o+(0.600697811809)*x_ref[1]
        ref[3,1]=(-0.567148999707)*x_ref[0]**o+(0.374593601186)*x_ref[0]+(-0.440814971395)*x_ref[1]**o+(-0.737145147183)*x_ref[1]
        ref[3,2]=(0.630637925234)*x_ref[0]**o+(-0.0884747935175)*x_ref[0]+(-0.0319938816017)*x_ref[1]**o+(0.865297163842)*x_ref[1]
        ref[3,3]=(-0.811769976054)*x_ref[0]**o+(0.533258797244)*x_ref[0]+(0.735904279517)*x_ref[1]**o+(0.362604687369)*x_ref[1]
        ref[3,4]=(-0.711314210671)*x_ref[0]**o+(-0.270182873191)*x_ref[0]+(-0.965300380176)*x_ref[1]**o+(0.50703939773)*x_ref[1]
      else:
        s[0,0]=(0.817784424934)*x[0]**o+(0.140829586694)*x[0]+(0.395723095233)*x[1]**o+(-0.0916449245097)*x[1]+(-0.686149802471)*x[2]**o+(0.231879617817)*x[2]
        s[0,1]=(-0.753947196887)*x[0]**o+(0.0184422308138)*x[0]+(0.38127599678)*x[1]**o+(-0.958287760938)*x[1]+(-0.230273819981)*x[2]**o+(-0.922603290238)*x[2]
        s[0,2]=(0.44304320962)*x[0]**o+(0.35396422013)*x[0]+(-0.479605019935)*x[1]**o+(0.289724501875)*x[1]+(0.657449032305)*x[2]**o+(-0.706689994866)*x[2]
        s[0,3]=(0.0214930482601)*x[0]**o+(-0.915710233423)*x[0]+(-0.571112565179)*x[1]**o+(0.837855622741)*x[1]+(-0.402178902773)*x[2]**o+(-0.113536055723)*x[2]
        s[0,4]=(0.0332368088464)*x[0]**o+(0.488860583014)*x[0]+(0.687230732879)*x[1]**o+(0.417177289784)*x[1]+(-0.138583225765)*x[2]**o+(-0.288206976222)*x[2]
        s[1,0]=(-0.571959772672)*x[0]**o+(-0.806968233432)*x[0]+(-0.951330341336)*x[1]**o+(-0.725871940445)*x[1]+(0.473279498914)*x[2]**o+(0.693882912652)*x[2]
        s[1,1]=(-0.925147598095)*x[0]**o+(0.612875097386)*x[0]+(0.625107884558)*x[1]**o+(0.665316395744)*x[1]+(-0.012158847541)*x[2]**o+(0.114880783467)*x[2]
        s[1,2]=(-0.361803346001)*x[0]**o+(0.630266012707)*x[0]+(-0.698127168765)*x[1]**o+(-0.534017552333)*x[1]+(0.968004682982)*x[2]**o+(-0.607970906679)*x[2]
        s[1,3]=(-0.122803946559)*x[0]**o+(-0.506542460419)*x[0]+(-0.993759502512)*x[1]**o+(-0.335687805433)*x[1]+(0.143963376404)*x[2]**o+(-0.980235207299)*x[2]
        s[1,4]=(-0.998831548478)*x[0]**o+(0.985193839232)*x[0]+(-0.310482397827)*x[1]**o+(0.487407650791)*x[1]+(-0.974332702676)*x[2]**o+(-0.0921055039133)*x[2]
        s[2,0]=(-0.160666729317)*x[0]**o+(0.250220171076)*x[0]+(-0.868648917809)*x[1]**o+(-0.711318604851)*x[1]+(0.115301101729)*x[2]**o+(0.231696383247)*x[2]
        s[2,1]=(-0.775210051641)*x[0]**o+(0.586897738775)*x[0]+(0.449891876198)*x[1]**o+(-0.326417063196)*x[1]+(-0.693635538015)*x[2]**o+(-0.180803736532)*x[2]
        s[2,2]=(-0.208290352953)*x[0]**o+(-0.447870020368)*x[0]+(-0.518392174915)*x[1]**o+(-0.183558767418)*x[1]+(-0.496457835849)*x[2]**o+(-0.423284073396)*x[2]
        s[2,3]=(-0.994564109782)*x[0]**o+(-0.229677912537)*x[0]+(0.563307213686)*x[1]**o+(0.922013593154)*x[1]+(-0.371937088648)*x[2]**o+(-0.243995790101)*x[2]
        s[2,4]=(0.108680732756)*x[0]**o+(0.8978812368)*x[0]+(0.672348022342)*x[1]**o+(0.316624749349)*x[1]+(0.304319187223)*x[2]**o+(-0.604575730332)*x[2]
        s[3,0]=(0.0734716079145)*x[0]**o+(-0.627088377782)*x[0]+(0.620453085026)*x[1]**o+(0.331742115021)*x[1]+(-0.166118547603)*x[2]**o+(0.808314484323)*x[2]
        s[3,1]=(-0.0808492395011)*x[0]**o+(-0.0938687820054)*x[0]+(-0.888326387847)*x[1]**o+(-0.608268617177)*x[1]+(-0.316327102759)*x[2]**o+(-0.592763199764)*x[2]
        s[3,2]=(-0.255058846163)*x[0]**o+(0.103273543614)*x[0]+(-0.976707756405)*x[1]**o+(-0.723115640951)*x[1]+(-0.871816562933)*x[2]**o+(0.0248677291123)*x[2]
        s[3,3]=(-0.128009946516)*x[0]**o+(0.285430888656)*x[0]+(0.660763158647)*x[1]**o+(-0.96689913729)*x[1]+(-0.512425011889)*x[2]**o+(0.12926532516)*x[2]
        s[3,4]=(0.374659980563)*x[0]**o+(-0.13005773825)*x[0]+(-0.491643647934)*x[1]**o+(0.536781460391)*x[1]+(0.484891534824)*x[2]**o+(0.764854714727)*x[2]
        ref[0,0]=(0.817784424934)*x_ref[0]**o+(0.140829586694)*x_ref[0]+(0.395723095233)*x_ref[1]**o+(-0.0916449245097)*x_ref[1]+(-0.686149802471)*x_ref[2]**o+(0.231879617817)*x_ref[2]
        ref[0,1]=(-0.753947196887)*x_ref[0]**o+(0.0184422308138)*x_ref[0]+(0.38127599678)*x_ref[1]**o+(-0.958287760938)*x_ref[1]+(-0.230273819981)*x_ref[2]**o+(-0.922603290238)*x_ref[2]
        ref[0,2]=(0.44304320962)*x_ref[0]**o+(0.35396422013)*x_ref[0]+(-0.479605019935)*x_ref[1]**o+(0.289724501875)*x_ref[1]+(0.657449032305)*x_ref[2]**o+(-0.706689994866)*x_ref[2]
        ref[0,3]=(0.0214930482601)*x_ref[0]**o+(-0.915710233423)*x_ref[0]+(-0.571112565179)*x_ref[1]**o+(0.837855622741)*x_ref[1]+(-0.402178902773)*x_ref[2]**o+(-0.113536055723)*x_ref[2]
        ref[0,4]=(0.0332368088464)*x_ref[0]**o+(0.488860583014)*x_ref[0]+(0.687230732879)*x_ref[1]**o+(0.417177289784)*x_ref[1]+(-0.138583225765)*x_ref[2]**o+(-0.288206976222)*x_ref[2]
        ref[1,0]=(-0.571959772672)*x_ref[0]**o+(-0.806968233432)*x_ref[0]+(-0.951330341336)*x_ref[1]**o+(-0.725871940445)*x_ref[1]+(0.473279498914)*x_ref[2]**o+(0.693882912652)*x_ref[2]
        ref[1,1]=(-0.925147598095)*x_ref[0]**o+(0.612875097386)*x_ref[0]+(0.625107884558)*x_ref[1]**o+(0.665316395744)*x_ref[1]+(-0.012158847541)*x_ref[2]**o+(0.114880783467)*x_ref[2]
        ref[1,2]=(-0.361803346001)*x_ref[0]**o+(0.630266012707)*x_ref[0]+(-0.698127168765)*x_ref[1]**o+(-0.534017552333)*x_ref[1]+(0.968004682982)*x_ref[2]**o+(-0.607970906679)*x_ref[2]
        ref[1,3]=(-0.122803946559)*x_ref[0]**o+(-0.506542460419)*x_ref[0]+(-0.993759502512)*x_ref[1]**o+(-0.335687805433)*x_ref[1]+(0.143963376404)*x_ref[2]**o+(-0.980235207299)*x_ref[2]
        ref[1,4]=(-0.998831548478)*x_ref[0]**o+(0.985193839232)*x_ref[0]+(-0.310482397827)*x_ref[1]**o+(0.487407650791)*x_ref[1]+(-0.974332702676)*x_ref[2]**o+(-0.0921055039133)*x_ref[2]
        ref[2,0]=(-0.160666729317)*x_ref[0]**o+(0.250220171076)*x_ref[0]+(-0.868648917809)*x_ref[1]**o+(-0.711318604851)*x_ref[1]+(0.115301101729)*x_ref[2]**o+(0.231696383247)*x_ref[2]
        ref[2,1]=(-0.775210051641)*x_ref[0]**o+(0.586897738775)*x_ref[0]+(0.449891876198)*x_ref[1]**o+(-0.326417063196)*x_ref[1]+(-0.693635538015)*x_ref[2]**o+(-0.180803736532)*x_ref[2]
        ref[2,2]=(-0.208290352953)*x_ref[0]**o+(-0.447870020368)*x_ref[0]+(-0.518392174915)*x_ref[1]**o+(-0.183558767418)*x_ref[1]+(-0.496457835849)*x_ref[2]**o+(-0.423284073396)*x_ref[2]
        ref[2,3]=(-0.994564109782)*x_ref[0]**o+(-0.229677912537)*x_ref[0]+(0.563307213686)*x_ref[1]**o+(0.922013593154)*x_ref[1]+(-0.371937088648)*x_ref[2]**o+(-0.243995790101)*x_ref[2]
        ref[2,4]=(0.108680732756)*x_ref[0]**o+(0.8978812368)*x_ref[0]+(0.672348022342)*x_ref[1]**o+(0.316624749349)*x_ref[1]+(0.304319187223)*x_ref[2]**o+(-0.604575730332)*x_ref[2]
        ref[3,0]=(0.0734716079145)*x_ref[0]**o+(-0.627088377782)*x_ref[0]+(0.620453085026)*x_ref[1]**o+(0.331742115021)*x_ref[1]+(-0.166118547603)*x_ref[2]**o+(0.808314484323)*x_ref[2]
        ref[3,1]=(-0.0808492395011)*x_ref[0]**o+(-0.0938687820054)*x_ref[0]+(-0.888326387847)*x_ref[1]**o+(-0.608268617177)*x_ref[1]+(-0.316327102759)*x_ref[2]**o+(-0.592763199764)*x_ref[2]
        ref[3,2]=(-0.255058846163)*x_ref[0]**o+(0.103273543614)*x_ref[0]+(-0.976707756405)*x_ref[1]**o+(-0.723115640951)*x_ref[1]+(-0.871816562933)*x_ref[2]**o+(0.0248677291123)*x_ref[2]
        ref[3,3]=(-0.128009946516)*x_ref[0]**o+(0.285430888656)*x_ref[0]+(0.660763158647)*x_ref[1]**o+(-0.96689913729)*x_ref[1]+(-0.512425011889)*x_ref[2]**o+(0.12926532516)*x_ref[2]
        ref[3,4]=(0.374659980563)*x_ref[0]**o+(-0.13005773825)*x_ref[0]+(-0.491643647934)*x_ref[1]**o+(0.536781460391)*x_ref[1]+(0.484891534824)*x_ref[2]**o+(0.764854714727)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_ReducedFunctionOnContactOne_rank3(self):
      """
      tests interpolate for rank 3 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactOne onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactOne(self.domain)
      x=w.getX()
      arg=Symbol(shape=(6, 2, 2),dim=dim)
      s=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2),w_ref)
      if dim==2:
        s[0,0,0]=(-0.395079492423)*x[0]**o+(-0.627751628747)*x[0]+(-0.754320526654)*x[1]**o+(-0.498408852477)*x[1]
        s[0,0,1]=(0.498446911285)*x[0]**o+(0.0596046404343)*x[0]+(0.667944256533)*x[1]**o+(0.80046807163)*x[1]
        s[0,1,0]=(-0.0130333830081)*x[0]**o+(0.649913235604)*x[0]+(-0.374473401736)*x[1]**o+(-0.0186132358452)*x[1]
        s[0,1,1]=(-0.390035543793)*x[0]**o+(0.673194387531)*x[0]+(-0.867455391881)*x[1]**o+(-0.510668203571)*x[1]
        s[1,0,0]=(0.978997680408)*x[0]**o+(-0.343461179415)*x[0]+(0.582713911591)*x[1]**o+(-0.165697613343)*x[1]
        s[1,0,1]=(0.0923732379474)*x[0]**o+(-0.585219326869)*x[0]+(0.540073721296)*x[1]**o+(0.027063080312)*x[1]
        s[1,1,0]=(0.590126162972)*x[0]**o+(-0.136675336981)*x[0]+(-0.376003854007)*x[1]**o+(0.966535168787)*x[1]
        s[1,1,1]=(-0.493032436853)*x[0]**o+(0.239853630037)*x[0]+(-0.602522863181)*x[1]**o+(0.33785945991)*x[1]
        s[2,0,0]=(-0.529642904818)*x[0]**o+(-0.43628377125)*x[0]+(-0.996498099393)*x[1]**o+(-0.0591954433263)*x[1]
        s[2,0,1]=(-0.235182234411)*x[0]**o+(0.811387084032)*x[0]+(-0.443296230671)*x[1]**o+(-0.3314686181)*x[1]
        s[2,1,0]=(-0.110109607717)*x[0]**o+(-0.1409604355)*x[0]+(0.284844298465)*x[1]**o+(0.714735331234)*x[1]
        s[2,1,1]=(-0.936889720851)*x[0]**o+(-0.00858197667802)*x[0]+(0.680058166799)*x[1]**o+(0.436911645255)*x[1]
        s[3,0,0]=(-0.478297335601)*x[0]**o+(0.686114547145)*x[0]+(0.234700452186)*x[1]**o+(-0.330970013839)*x[1]
        s[3,0,1]=(0.975338382165)*x[0]**o+(0.607523981612)*x[0]+(-0.616521819955)*x[1]**o+(-0.292932591377)*x[1]
        s[3,1,0]=(0.987717426508)*x[0]**o+(0.6507515036)*x[0]+(-0.195762979545)*x[1]**o+(-0.667650116614)*x[1]
        s[3,1,1]=(0.53886053148)*x[0]**o+(-0.491668085193)*x[0]+(0.486615664224)*x[1]**o+(0.741560383409)*x[1]
        s[4,0,0]=(-0.936467387625)*x[0]**o+(-0.142421597802)*x[0]+(0.733595719048)*x[1]**o+(0.402826551348)*x[1]
        s[4,0,1]=(0.64501254965)*x[0]**o+(-0.660424630011)*x[0]+(-0.221052224259)*x[1]**o+(-0.199679968927)*x[1]
        s[4,1,0]=(0.809693682851)*x[0]**o+(-0.614404151383)*x[0]+(-0.754545121479)*x[1]**o+(-0.603231220423)*x[1]
        s[4,1,1]=(0.709936703222)*x[0]**o+(-0.389296803297)*x[0]+(-0.271161950977)*x[1]**o+(-0.929259781943)*x[1]
        s[5,0,0]=(-0.267950850602)*x[0]**o+(-0.660075303261)*x[0]+(0.0304564867312)*x[1]**o+(-0.42168319765)*x[1]
        s[5,0,1]=(0.137764595135)*x[0]**o+(0.265554426237)*x[0]+(-0.435216862466)*x[1]**o+(-0.338253702328)*x[1]
        s[5,1,0]=(0.858568660192)*x[0]**o+(-0.18371104351)*x[0]+(0.665662082165)*x[1]**o+(0.0684606399813)*x[1]
        s[5,1,1]=(0.897687912811)*x[0]**o+(0.675844284819)*x[0]+(0.0430574862378)*x[1]**o+(0.371728813053)*x[1]
        ref[0,0,0]=(-0.395079492423)*x_ref[0]**o+(-0.627751628747)*x_ref[0]+(-0.754320526654)*x_ref[1]**o+(-0.498408852477)*x_ref[1]
        ref[0,0,1]=(0.498446911285)*x_ref[0]**o+(0.0596046404343)*x_ref[0]+(0.667944256533)*x_ref[1]**o+(0.80046807163)*x_ref[1]
        ref[0,1,0]=(-0.0130333830081)*x_ref[0]**o+(0.649913235604)*x_ref[0]+(-0.374473401736)*x_ref[1]**o+(-0.0186132358452)*x_ref[1]
        ref[0,1,1]=(-0.390035543793)*x_ref[0]**o+(0.673194387531)*x_ref[0]+(-0.867455391881)*x_ref[1]**o+(-0.510668203571)*x_ref[1]
        ref[1,0,0]=(0.978997680408)*x_ref[0]**o+(-0.343461179415)*x_ref[0]+(0.582713911591)*x_ref[1]**o+(-0.165697613343)*x_ref[1]
        ref[1,0,1]=(0.0923732379474)*x_ref[0]**o+(-0.585219326869)*x_ref[0]+(0.540073721296)*x_ref[1]**o+(0.027063080312)*x_ref[1]
        ref[1,1,0]=(0.590126162972)*x_ref[0]**o+(-0.136675336981)*x_ref[0]+(-0.376003854007)*x_ref[1]**o+(0.966535168787)*x_ref[1]
        ref[1,1,1]=(-0.493032436853)*x_ref[0]**o+(0.239853630037)*x_ref[0]+(-0.602522863181)*x_ref[1]**o+(0.33785945991)*x_ref[1]
        ref[2,0,0]=(-0.529642904818)*x_ref[0]**o+(-0.43628377125)*x_ref[0]+(-0.996498099393)*x_ref[1]**o+(-0.0591954433263)*x_ref[1]
        ref[2,0,1]=(-0.235182234411)*x_ref[0]**o+(0.811387084032)*x_ref[0]+(-0.443296230671)*x_ref[1]**o+(-0.3314686181)*x_ref[1]
        ref[2,1,0]=(-0.110109607717)*x_ref[0]**o+(-0.1409604355)*x_ref[0]+(0.284844298465)*x_ref[1]**o+(0.714735331234)*x_ref[1]
        ref[2,1,1]=(-0.936889720851)*x_ref[0]**o+(-0.00858197667802)*x_ref[0]+(0.680058166799)*x_ref[1]**o+(0.436911645255)*x_ref[1]
        ref[3,0,0]=(-0.478297335601)*x_ref[0]**o+(0.686114547145)*x_ref[0]+(0.234700452186)*x_ref[1]**o+(-0.330970013839)*x_ref[1]
        ref[3,0,1]=(0.975338382165)*x_ref[0]**o+(0.607523981612)*x_ref[0]+(-0.616521819955)*x_ref[1]**o+(-0.292932591377)*x_ref[1]
        ref[3,1,0]=(0.987717426508)*x_ref[0]**o+(0.6507515036)*x_ref[0]+(-0.195762979545)*x_ref[1]**o+(-0.667650116614)*x_ref[1]
        ref[3,1,1]=(0.53886053148)*x_ref[0]**o+(-0.491668085193)*x_ref[0]+(0.486615664224)*x_ref[1]**o+(0.741560383409)*x_ref[1]
        ref[4,0,0]=(-0.936467387625)*x_ref[0]**o+(-0.142421597802)*x_ref[0]+(0.733595719048)*x_ref[1]**o+(0.402826551348)*x_ref[1]
        ref[4,0,1]=(0.64501254965)*x_ref[0]**o+(-0.660424630011)*x_ref[0]+(-0.221052224259)*x_ref[1]**o+(-0.199679968927)*x_ref[1]
        ref[4,1,0]=(0.809693682851)*x_ref[0]**o+(-0.614404151383)*x_ref[0]+(-0.754545121479)*x_ref[1]**o+(-0.603231220423)*x_ref[1]
        ref[4,1,1]=(0.709936703222)*x_ref[0]**o+(-0.389296803297)*x_ref[0]+(-0.271161950977)*x_ref[1]**o+(-0.929259781943)*x_ref[1]
        ref[5,0,0]=(-0.267950850602)*x_ref[0]**o+(-0.660075303261)*x_ref[0]+(0.0304564867312)*x_ref[1]**o+(-0.42168319765)*x_ref[1]
        ref[5,0,1]=(0.137764595135)*x_ref[0]**o+(0.265554426237)*x_ref[0]+(-0.435216862466)*x_ref[1]**o+(-0.338253702328)*x_ref[1]
        ref[5,1,0]=(0.858568660192)*x_ref[0]**o+(-0.18371104351)*x_ref[0]+(0.665662082165)*x_ref[1]**o+(0.0684606399813)*x_ref[1]
        ref[5,1,1]=(0.897687912811)*x_ref[0]**o+(0.675844284819)*x_ref[0]+(0.0430574862378)*x_ref[1]**o+(0.371728813053)*x_ref[1]
      else:
        s[0,0,0]=(-0.589511962709)*x[0]**o+(0.159792477428)*x[0]+(0.766030256721)*x[1]**o+(-0.312062526449)*x[1]+(0.64700443985)*x[2]**o+(0.0518757184544)*x[2]
        s[0,0,1]=(-0.894062305961)*x[0]**o+(0.864404945522)*x[0]+(0.560280553158)*x[1]**o+(0.918283462394)*x[1]+(-0.490840733192)*x[2]**o+(0.697691943154)*x[2]
        s[0,1,0]=(-0.0882043414483)*x[0]**o+(-0.096537953899)*x[0]+(0.927057867899)*x[1]**o+(0.77613572949)*x[1]+(-0.987702345599)*x[2]**o+(-0.574645805199)*x[2]
        s[0,1,1]=(-0.66012117034)*x[0]**o+(-0.189179227113)*x[0]+(0.292721630825)*x[1]**o+(0.0170456979372)*x[1]+(0.0183109554333)*x[2]**o+(-0.917454875678)*x[2]
        s[1,0,0]=(0.654605700477)*x[0]**o+(-0.170526279436)*x[0]+(0.343326885913)*x[1]**o+(0.210563849271)*x[1]+(0.534291586724)*x[2]**o+(-0.755048841391)*x[2]
        s[1,0,1]=(-0.364686989491)*x[0]**o+(0.552984183344)*x[0]+(-0.470357667845)*x[1]**o+(-0.464300631313)*x[1]+(-0.438895038261)*x[2]**o+(0.518824978532)*x[2]
        s[1,1,0]=(-0.0456454803782)*x[0]**o+(-0.696772845921)*x[0]+(-0.587520555693)*x[1]**o+(0.676523672069)*x[1]+(-0.905996237596)*x[2]**o+(-0.474417867255)*x[2]
        s[1,1,1]=(0.360939068763)*x[0]**o+(-0.743105635563)*x[0]+(-0.615952199646)*x[1]**o+(-0.538063201399)*x[1]+(-0.898283738708)*x[2]**o+(-0.97634117952)*x[2]
        s[2,0,0]=(-0.185178359078)*x[0]**o+(0.224443281068)*x[0]+(0.754102418405)*x[1]**o+(-0.411697272338)*x[1]+(-0.528900607926)*x[2]**o+(-0.807089685705)*x[2]
        s[2,0,1]=(-0.784480844715)*x[0]**o+(-0.823404986088)*x[0]+(0.749752559055)*x[1]**o+(-0.993803702434)*x[1]+(-0.03194828585)*x[2]**o+(-0.104230991195)*x[2]
        s[2,1,0]=(-0.3975495117)*x[0]**o+(-0.921611510193)*x[0]+(0.364776109304)*x[1]**o+(0.568483551949)*x[1]+(0.92620487977)*x[2]**o+(-0.203075275658)*x[2]
        s[2,1,1]=(0.483830827389)*x[0]**o+(-0.16426527248)*x[0]+(0.829148557161)*x[1]**o+(-0.476785306719)*x[1]+(0.498338738131)*x[2]**o+(-0.335483518227)*x[2]
        s[3,0,0]=(-0.673980780716)*x[0]**o+(0.565441119757)*x[0]+(-0.440135530887)*x[1]**o+(-0.505046260809)*x[1]+(-0.37380888836)*x[2]**o+(0.405503742087)*x[2]
        s[3,0,1]=(-0.330871586339)*x[0]**o+(-0.590160971509)*x[0]+(-0.737041178139)*x[1]**o+(0.998222421798)*x[1]+(0.910277176468)*x[2]**o+(0.429491322427)*x[2]
        s[3,1,0]=(-0.166111866877)*x[0]**o+(-0.758927317121)*x[0]+(0.287263609892)*x[1]**o+(-0.322563818793)*x[1]+(-0.430564368296)*x[2]**o+(0.174322344232)*x[2]
        s[3,1,1]=(-0.661309380996)*x[0]**o+(0.139785030015)*x[0]+(-0.682140312514)*x[1]**o+(-0.654285952086)*x[1]+(0.622815328424)*x[2]**o+(0.485021479372)*x[2]
        s[4,0,0]=(-0.594196524006)*x[0]**o+(-0.539707458079)*x[0]+(-0.793206045899)*x[1]**o+(-0.65733164652)*x[1]+(0.853898902594)*x[2]**o+(-0.68578755422)*x[2]
        s[4,0,1]=(-0.368655085475)*x[0]**o+(0.965754821996)*x[0]+(0.796746162388)*x[1]**o+(-0.0792750556783)*x[1]+(0.865822523139)*x[2]**o+(0.824557599127)*x[2]
        s[4,1,0]=(0.0309342911768)*x[0]**o+(-0.57039195842)*x[0]+(0.314105965219)*x[1]**o+(0.991995648934)*x[1]+(-0.344915004359)*x[2]**o+(0.461898675894)*x[2]
        s[4,1,1]=(0.287549691013)*x[0]**o+(0.342272217628)*x[0]+(-0.469131330709)*x[1]**o+(0.505146423329)*x[1]+(0.347751545765)*x[2]**o+(0.547278978185)*x[2]
        s[5,0,0]=(-0.884852695364)*x[0]**o+(-0.642157963952)*x[0]+(-0.787121736521)*x[1]**o+(0.187833667108)*x[1]+(-0.339842450451)*x[2]**o+(0.174012710957)*x[2]
        s[5,0,1]=(0.28878191697)*x[0]**o+(-0.360576326583)*x[0]+(-0.0945722807072)*x[1]**o+(-0.587351566921)*x[1]+(-0.933568527838)*x[2]**o+(-0.60716101577)*x[2]
        s[5,1,0]=(-0.581405265173)*x[0]**o+(-0.015810740193)*x[0]+(0.919122906998)*x[1]**o+(-0.676080492473)*x[1]+(0.0255111996852)*x[2]**o+(-0.693486219028)*x[2]
        s[5,1,1]=(-0.941020084511)*x[0]**o+(-0.330527617459)*x[0]+(0.98542398649)*x[1]**o+(-0.154193758572)*x[1]+(0.876454677452)*x[2]**o+(-0.278575401803)*x[2]
        ref[0,0,0]=(-0.589511962709)*x_ref[0]**o+(0.159792477428)*x_ref[0]+(0.766030256721)*x_ref[1]**o+(-0.312062526449)*x_ref[1]+(0.64700443985)*x_ref[2]**o+(0.0518757184544)*x_ref[2]
        ref[0,0,1]=(-0.894062305961)*x_ref[0]**o+(0.864404945522)*x_ref[0]+(0.560280553158)*x_ref[1]**o+(0.918283462394)*x_ref[1]+(-0.490840733192)*x_ref[2]**o+(0.697691943154)*x_ref[2]
        ref[0,1,0]=(-0.0882043414483)*x_ref[0]**o+(-0.096537953899)*x_ref[0]+(0.927057867899)*x_ref[1]**o+(0.77613572949)*x_ref[1]+(-0.987702345599)*x_ref[2]**o+(-0.574645805199)*x_ref[2]
        ref[0,1,1]=(-0.66012117034)*x_ref[0]**o+(-0.189179227113)*x_ref[0]+(0.292721630825)*x_ref[1]**o+(0.0170456979372)*x_ref[1]+(0.0183109554333)*x_ref[2]**o+(-0.917454875678)*x_ref[2]
        ref[1,0,0]=(0.654605700477)*x_ref[0]**o+(-0.170526279436)*x_ref[0]+(0.343326885913)*x_ref[1]**o+(0.210563849271)*x_ref[1]+(0.534291586724)*x_ref[2]**o+(-0.755048841391)*x_ref[2]
        ref[1,0,1]=(-0.364686989491)*x_ref[0]**o+(0.552984183344)*x_ref[0]+(-0.470357667845)*x_ref[1]**o+(-0.464300631313)*x_ref[1]+(-0.438895038261)*x_ref[2]**o+(0.518824978532)*x_ref[2]
        ref[1,1,0]=(-0.0456454803782)*x_ref[0]**o+(-0.696772845921)*x_ref[0]+(-0.587520555693)*x_ref[1]**o+(0.676523672069)*x_ref[1]+(-0.905996237596)*x_ref[2]**o+(-0.474417867255)*x_ref[2]
        ref[1,1,1]=(0.360939068763)*x_ref[0]**o+(-0.743105635563)*x_ref[0]+(-0.615952199646)*x_ref[1]**o+(-0.538063201399)*x_ref[1]+(-0.898283738708)*x_ref[2]**o+(-0.97634117952)*x_ref[2]
        ref[2,0,0]=(-0.185178359078)*x_ref[0]**o+(0.224443281068)*x_ref[0]+(0.754102418405)*x_ref[1]**o+(-0.411697272338)*x_ref[1]+(-0.528900607926)*x_ref[2]**o+(-0.807089685705)*x_ref[2]
        ref[2,0,1]=(-0.784480844715)*x_ref[0]**o+(-0.823404986088)*x_ref[0]+(0.749752559055)*x_ref[1]**o+(-0.993803702434)*x_ref[1]+(-0.03194828585)*x_ref[2]**o+(-0.104230991195)*x_ref[2]
        ref[2,1,0]=(-0.3975495117)*x_ref[0]**o+(-0.921611510193)*x_ref[0]+(0.364776109304)*x_ref[1]**o+(0.568483551949)*x_ref[1]+(0.92620487977)*x_ref[2]**o+(-0.203075275658)*x_ref[2]
        ref[2,1,1]=(0.483830827389)*x_ref[0]**o+(-0.16426527248)*x_ref[0]+(0.829148557161)*x_ref[1]**o+(-0.476785306719)*x_ref[1]+(0.498338738131)*x_ref[2]**o+(-0.335483518227)*x_ref[2]
        ref[3,0,0]=(-0.673980780716)*x_ref[0]**o+(0.565441119757)*x_ref[0]+(-0.440135530887)*x_ref[1]**o+(-0.505046260809)*x_ref[1]+(-0.37380888836)*x_ref[2]**o+(0.405503742087)*x_ref[2]
        ref[3,0,1]=(-0.330871586339)*x_ref[0]**o+(-0.590160971509)*x_ref[0]+(-0.737041178139)*x_ref[1]**o+(0.998222421798)*x_ref[1]+(0.910277176468)*x_ref[2]**o+(0.429491322427)*x_ref[2]
        ref[3,1,0]=(-0.166111866877)*x_ref[0]**o+(-0.758927317121)*x_ref[0]+(0.287263609892)*x_ref[1]**o+(-0.322563818793)*x_ref[1]+(-0.430564368296)*x_ref[2]**o+(0.174322344232)*x_ref[2]
        ref[3,1,1]=(-0.661309380996)*x_ref[0]**o+(0.139785030015)*x_ref[0]+(-0.682140312514)*x_ref[1]**o+(-0.654285952086)*x_ref[1]+(0.622815328424)*x_ref[2]**o+(0.485021479372)*x_ref[2]
        ref[4,0,0]=(-0.594196524006)*x_ref[0]**o+(-0.539707458079)*x_ref[0]+(-0.793206045899)*x_ref[1]**o+(-0.65733164652)*x_ref[1]+(0.853898902594)*x_ref[2]**o+(-0.68578755422)*x_ref[2]
        ref[4,0,1]=(-0.368655085475)*x_ref[0]**o+(0.965754821996)*x_ref[0]+(0.796746162388)*x_ref[1]**o+(-0.0792750556783)*x_ref[1]+(0.865822523139)*x_ref[2]**o+(0.824557599127)*x_ref[2]
        ref[4,1,0]=(0.0309342911768)*x_ref[0]**o+(-0.57039195842)*x_ref[0]+(0.314105965219)*x_ref[1]**o+(0.991995648934)*x_ref[1]+(-0.344915004359)*x_ref[2]**o+(0.461898675894)*x_ref[2]
        ref[4,1,1]=(0.287549691013)*x_ref[0]**o+(0.342272217628)*x_ref[0]+(-0.469131330709)*x_ref[1]**o+(0.505146423329)*x_ref[1]+(0.347751545765)*x_ref[2]**o+(0.547278978185)*x_ref[2]
        ref[5,0,0]=(-0.884852695364)*x_ref[0]**o+(-0.642157963952)*x_ref[0]+(-0.787121736521)*x_ref[1]**o+(0.187833667108)*x_ref[1]+(-0.339842450451)*x_ref[2]**o+(0.174012710957)*x_ref[2]
        ref[5,0,1]=(0.28878191697)*x_ref[0]**o+(-0.360576326583)*x_ref[0]+(-0.0945722807072)*x_ref[1]**o+(-0.587351566921)*x_ref[1]+(-0.933568527838)*x_ref[2]**o+(-0.60716101577)*x_ref[2]
        ref[5,1,0]=(-0.581405265173)*x_ref[0]**o+(-0.015810740193)*x_ref[0]+(0.919122906998)*x_ref[1]**o+(-0.676080492473)*x_ref[1]+(0.0255111996852)*x_ref[2]**o+(-0.693486219028)*x_ref[2]
        ref[5,1,1]=(-0.941020084511)*x_ref[0]**o+(-0.330527617459)*x_ref[0]+(0.98542398649)*x_ref[1]**o+(-0.154193758572)*x_ref[1]+(0.876454677452)*x_ref[2]**o+(-0.278575401803)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_ReducedFunctionOnContactOne_rank4(self):
      """
      tests interpolate for rank 4 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactOne onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactOne(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5, 3, 2),dim=dim)
      s=Data(0,(4, 5, 3, 2),w)
      ref=Data(0,(4, 5, 3, 2),w_ref)
      if dim==2:
        s[0,0,0,0]=(-0.939524201884)*x[0]**o+(-0.294114596857)*x[0]+(-0.662571514204)*x[1]**o+(-0.295055230761)*x[1]
        s[0,0,0,1]=(0.174447802399)*x[0]**o+(0.313121933505)*x[0]+(0.780423776572)*x[1]**o+(0.260329316622)*x[1]
        s[0,0,1,0]=(-0.307907192496)*x[0]**o+(0.924386500426)*x[0]+(-0.34849235247)*x[1]**o+(0.42450311854)*x[1]
        s[0,0,1,1]=(-0.221818525078)*x[0]**o+(0.334658153768)*x[0]+(0.730236845709)*x[1]**o+(0.923645750095)*x[1]
        s[0,0,2,0]=(-0.0573462957426)*x[0]**o+(0.10400906253)*x[0]+(-0.606757296289)*x[1]**o+(-0.171453259849)*x[1]
        s[0,0,2,1]=(-0.653190260251)*x[0]**o+(0.392209523496)*x[0]+(0.984792984469)*x[1]**o+(-0.310847119118)*x[1]
        s[0,1,0,0]=(-0.0980551705202)*x[0]**o+(0.475337517365)*x[0]+(-0.93300381011)*x[1]**o+(0.0593095200419)*x[1]
        s[0,1,0,1]=(0.833302217025)*x[0]**o+(-0.866194985925)*x[0]+(0.337967734176)*x[1]**o+(0.0581478569311)*x[1]
        s[0,1,1,0]=(0.936751875211)*x[0]**o+(-0.217226318173)*x[0]+(0.477011461247)*x[1]**o+(0.221436266198)*x[1]
        s[0,1,1,1]=(-0.537625551791)*x[0]**o+(0.718351842961)*x[0]+(0.0286304587001)*x[1]**o+(-0.721442785761)*x[1]
        s[0,1,2,0]=(-0.738645216931)*x[0]**o+(0.0195980002286)*x[0]+(0.827363980625)*x[1]**o+(-0.743955943985)*x[1]
        s[0,1,2,1]=(-0.145033137282)*x[0]**o+(0.98069402199)*x[0]+(-0.167153611864)*x[1]**o+(-0.577697516598)*x[1]
        s[0,2,0,0]=(0.594926068713)*x[0]**o+(-0.124581510742)*x[0]+(-0.392512448989)*x[1]**o+(-0.989702605289)*x[1]
        s[0,2,0,1]=(-0.208680726694)*x[0]**o+(0.0226857988067)*x[0]+(0.0127574486126)*x[1]**o+(-0.661169282937)*x[1]
        s[0,2,1,0]=(0.43109296375)*x[0]**o+(-0.966243011304)*x[0]+(-0.448863362387)*x[1]**o+(-0.531981406137)*x[1]
        s[0,2,1,1]=(0.427441889618)*x[0]**o+(-0.730863645031)*x[0]+(0.0948615738498)*x[1]**o+(0.349409965532)*x[1]
        s[0,2,2,0]=(0.0857935500808)*x[0]**o+(0.916612852535)*x[0]+(-0.5434740164)*x[1]**o+(0.651670987189)*x[1]
        s[0,2,2,1]=(0.269775748844)*x[0]**o+(-0.351727930666)*x[0]+(0.648234694172)*x[1]**o+(0.714331370173)*x[1]
        s[0,3,0,0]=(0.862594479045)*x[0]**o+(-0.787345711339)*x[0]+(0.1934849213)*x[1]**o+(-0.977224028721)*x[1]
        s[0,3,0,1]=(0.166215962959)*x[0]**o+(0.734752140512)*x[0]+(0.373558020776)*x[1]**o+(-0.589819927874)*x[1]
        s[0,3,1,0]=(0.0289724459782)*x[0]**o+(0.517327391578)*x[0]+(0.109956624336)*x[1]**o+(-0.290855234976)*x[1]
        s[0,3,1,1]=(0.369852037916)*x[0]**o+(0.184556562048)*x[0]+(-0.694630650679)*x[1]**o+(-0.526020870229)*x[1]
        s[0,3,2,0]=(0.397498255324)*x[0]**o+(-0.375327133358)*x[0]+(0.564571187906)*x[1]**o+(-0.295820211045)*x[1]
        s[0,3,2,1]=(0.550532413074)*x[0]**o+(-0.751307255234)*x[0]+(0.323446174542)*x[1]**o+(0.968475158376)*x[1]
        s[0,4,0,0]=(-0.112506956442)*x[0]**o+(-0.432032518771)*x[0]+(0.0205217950343)*x[1]**o+(-0.507714722498)*x[1]
        s[0,4,0,1]=(-0.501932335824)*x[0]**o+(-0.147371573026)*x[0]+(-0.421130372732)*x[1]**o+(-0.958029815205)*x[1]
        s[0,4,1,0]=(0.172226819831)*x[0]**o+(-0.712313983987)*x[0]+(0.859887204781)*x[1]**o+(0.0661613029313)*x[1]
        s[0,4,1,1]=(-0.863367396226)*x[0]**o+(0.150655310131)*x[0]+(-0.446461969683)*x[1]**o+(0.247377395525)*x[1]
        s[0,4,2,0]=(-0.0431980822723)*x[0]**o+(0.676680822195)*x[0]+(-0.613277443463)*x[1]**o+(0.787356918727)*x[1]
        s[0,4,2,1]=(-0.0569719722636)*x[0]**o+(0.641187476483)*x[0]+(-0.303617110099)*x[1]**o+(-0.888820557309)*x[1]
        s[1,0,0,0]=(-0.0552307803637)*x[0]**o+(0.306010657529)*x[0]+(0.488223930134)*x[1]**o+(-0.682392555784)*x[1]
        s[1,0,0,1]=(-0.253901726526)*x[0]**o+(0.73343629071)*x[0]+(-0.390491326134)*x[1]**o+(0.774605582697)*x[1]
        s[1,0,1,0]=(-0.42967954588)*x[0]**o+(-0.751284399784)*x[0]+(0.813738133871)*x[1]**o+(-0.0995163058125)*x[1]
        s[1,0,1,1]=(-0.0880959500234)*x[0]**o+(-0.781527901178)*x[0]+(-0.496899067642)*x[1]**o+(0.55524107606)*x[1]
        s[1,0,2,0]=(0.256546278479)*x[0]**o+(-0.595310894415)*x[0]+(-0.281823345122)*x[1]**o+(-0.953457305769)*x[1]
        s[1,0,2,1]=(0.365266273739)*x[0]**o+(-0.935192596652)*x[0]+(0.740921701447)*x[1]**o+(0.843620235195)*x[1]
        s[1,1,0,0]=(-0.992209271955)*x[0]**o+(0.267499041637)*x[0]+(-0.446633252289)*x[1]**o+(0.357054618942)*x[1]
        s[1,1,0,1]=(0.863355358433)*x[0]**o+(-0.104881968217)*x[0]+(0.0776056200756)*x[1]**o+(0.485205982195)*x[1]
        s[1,1,1,0]=(0.29822878169)*x[0]**o+(0.766559198241)*x[0]+(0.489839749671)*x[1]**o+(-0.694316951865)*x[1]
        s[1,1,1,1]=(-0.947135250126)*x[0]**o+(-0.952537408063)*x[0]+(-0.840569482634)*x[1]**o+(-0.0881389491948)*x[1]
        s[1,1,2,0]=(0.269982185854)*x[0]**o+(0.290850786316)*x[0]+(0.707550793741)*x[1]**o+(-0.583787562553)*x[1]
        s[1,1,2,1]=(-0.998478389198)*x[0]**o+(-0.902713626135)*x[0]+(-0.0809583498397)*x[1]**o+(0.242726761269)*x[1]
        s[1,2,0,0]=(-0.0752124155425)*x[0]**o+(0.287338426631)*x[0]+(-0.873281064449)*x[1]**o+(0.86906507097)*x[1]
        s[1,2,0,1]=(-0.451012489095)*x[0]**o+(0.692877058818)*x[0]+(-0.771278511209)*x[1]**o+(0.325983000966)*x[1]
        s[1,2,1,0]=(-0.633239034154)*x[0]**o+(0.751673044983)*x[0]+(-0.799706248119)*x[1]**o+(0.0783814978548)*x[1]
        s[1,2,1,1]=(-0.496449080562)*x[0]**o+(-0.382869354097)*x[0]+(-0.658792080006)*x[1]**o+(0.578499479656)*x[1]
        s[1,2,2,0]=(0.41041079346)*x[0]**o+(0.552454636943)*x[0]+(0.236050520221)*x[1]**o+(0.477587390293)*x[1]
        s[1,2,2,1]=(0.649795041558)*x[0]**o+(-0.947868231928)*x[0]+(-0.529895928082)*x[1]**o+(0.715695540026)*x[1]
        s[1,3,0,0]=(0.948112901731)*x[0]**o+(-0.374624145588)*x[0]+(0.806640283077)*x[1]**o+(0.729074938843)*x[1]
        s[1,3,0,1]=(0.266956301954)*x[0]**o+(-0.878004354786)*x[0]+(-0.422298760963)*x[1]**o+(-0.251922913127)*x[1]
        s[1,3,1,0]=(0.751159198422)*x[0]**o+(-0.576983418224)*x[0]+(0.427653287761)*x[1]**o+(0.351116956206)*x[1]
        s[1,3,1,1]=(-0.804702781998)*x[0]**o+(-0.218650676384)*x[0]+(-0.832220670207)*x[1]**o+(0.997469993794)*x[1]
        s[1,3,2,0]=(0.111110638087)*x[0]**o+(-0.435831664904)*x[0]+(-0.144481003079)*x[1]**o+(-0.604349850119)*x[1]
        s[1,3,2,1]=(0.470311907248)*x[0]**o+(-0.591709930358)*x[0]+(-0.562662398681)*x[1]**o+(0.0163307873851)*x[1]
        s[1,4,0,0]=(0.684349728229)*x[0]**o+(0.404890565831)*x[0]+(0.922438828147)*x[1]**o+(0.939539648586)*x[1]
        s[1,4,0,1]=(-0.179161968364)*x[0]**o+(0.983188684135)*x[0]+(-0.722578271046)*x[1]**o+(-0.450472366251)*x[1]
        s[1,4,1,0]=(0.243718494318)*x[0]**o+(-0.0251174594804)*x[0]+(0.952890549291)*x[1]**o+(-0.398408844924)*x[1]
        s[1,4,1,1]=(-0.883090916963)*x[0]**o+(-0.236073018867)*x[0]+(-0.245092152479)*x[1]**o+(0.846958498274)*x[1]
        s[1,4,2,0]=(-0.839928016329)*x[0]**o+(-0.677734468188)*x[0]+(-0.123639765438)*x[1]**o+(-0.680672570927)*x[1]
        s[1,4,2,1]=(-0.234629335025)*x[0]**o+(-0.567381745605)*x[0]+(0.623480113756)*x[1]**o+(0.2049427452)*x[1]
        s[2,0,0,0]=(-0.297265663727)*x[0]**o+(0.00899195467866)*x[0]+(0.992475896282)*x[1]**o+(0.863451618621)*x[1]
        s[2,0,0,1]=(-0.24914517208)*x[0]**o+(0.104091943078)*x[0]+(0.762436787588)*x[1]**o+(0.89363657955)*x[1]
        s[2,0,1,0]=(0.264734056412)*x[0]**o+(-0.716868164934)*x[0]+(-0.905285068877)*x[1]**o+(0.0227117696273)*x[1]
        s[2,0,1,1]=(-0.932617617299)*x[0]**o+(0.271634450254)*x[0]+(0.51812940988)*x[1]**o+(-0.484525267705)*x[1]
        s[2,0,2,0]=(0.364599957461)*x[0]**o+(0.818588026862)*x[0]+(0.70366241863)*x[1]**o+(0.43910943526)*x[1]
        s[2,0,2,1]=(0.557891341305)*x[0]**o+(-0.891810635437)*x[0]+(0.649138154483)*x[1]**o+(-0.213951745119)*x[1]
        s[2,1,0,0]=(0.994667066111)*x[0]**o+(0.433110731778)*x[0]+(0.787983438535)*x[1]**o+(0.979717998129)*x[1]
        s[2,1,0,1]=(0.619284628164)*x[0]**o+(0.662998143683)*x[0]+(0.0328849267442)*x[1]**o+(0.765881868994)*x[1]
        s[2,1,1,0]=(0.335939940328)*x[0]**o+(0.556758825457)*x[0]+(-0.524870937642)*x[1]**o+(0.0316796705422)*x[1]
        s[2,1,1,1]=(-0.702249603437)*x[0]**o+(0.544642480464)*x[0]+(0.330520285989)*x[1]**o+(0.313003559197)*x[1]
        s[2,1,2,0]=(0.0294247925115)*x[0]**o+(0.10067678181)*x[0]+(0.608148681239)*x[1]**o+(-0.95697997936)*x[1]
        s[2,1,2,1]=(0.101913506022)*x[0]**o+(-0.423061068258)*x[0]+(0.303518988539)*x[1]**o+(0.0108182161414)*x[1]
        s[2,2,0,0]=(0.390642040424)*x[0]**o+(0.0578342521895)*x[0]+(0.228499563768)*x[1]**o+(-0.944433241107)*x[1]
        s[2,2,0,1]=(0.522633870922)*x[0]**o+(0.724655140578)*x[0]+(-0.664760217849)*x[1]**o+(0.812535008015)*x[1]
        s[2,2,1,0]=(0.27608720491)*x[0]**o+(0.42350834695)*x[0]+(-0.732116287856)*x[1]**o+(0.930674651671)*x[1]
        s[2,2,1,1]=(0.214308063462)*x[0]**o+(0.815707217077)*x[0]+(0.231785784747)*x[1]**o+(-0.954429150944)*x[1]
        s[2,2,2,0]=(0.402814098347)*x[0]**o+(-0.650491293555)*x[0]+(-0.359159927669)*x[1]**o+(0.503842188774)*x[1]
        s[2,2,2,1]=(-0.191195415522)*x[0]**o+(0.980791691347)*x[0]+(0.228597878156)*x[1]**o+(0.535975172993)*x[1]
        s[2,3,0,0]=(0.495749327231)*x[0]**o+(0.311034011895)*x[0]+(-0.532686805928)*x[1]**o+(0.92438969176)*x[1]
        s[2,3,0,1]=(-0.38513599341)*x[0]**o+(-0.463099237819)*x[0]+(-0.992606328662)*x[1]**o+(-0.927920439344)*x[1]
        s[2,3,1,0]=(-0.868022271483)*x[0]**o+(-0.540096192748)*x[0]+(-0.0879858139685)*x[1]**o+(0.534604810517)*x[1]
        s[2,3,1,1]=(-0.686837754683)*x[0]**o+(-0.505585075528)*x[0]+(-0.218155668559)*x[1]**o+(-0.241972887878)*x[1]
        s[2,3,2,0]=(0.86187325655)*x[0]**o+(-0.733958307606)*x[0]+(-0.107393817742)*x[1]**o+(-0.830306745614)*x[1]
        s[2,3,2,1]=(0.404834271017)*x[0]**o+(0.44646270613)*x[0]+(-0.657038236282)*x[1]**o+(-0.540570795414)*x[1]
        s[2,4,0,0]=(-0.216828677229)*x[0]**o+(0.129702603598)*x[0]+(0.556920803376)*x[1]**o+(0.960021941421)*x[1]
        s[2,4,0,1]=(-0.21625137654)*x[0]**o+(-0.19604294366)*x[0]+(-0.889082170851)*x[1]**o+(-0.834967241538)*x[1]
        s[2,4,1,0]=(0.123104286845)*x[0]**o+(-0.43197703752)*x[0]+(0.737850827826)*x[1]**o+(0.344689639656)*x[1]
        s[2,4,1,1]=(0.119471348292)*x[0]**o+(0.948353928609)*x[0]+(-0.726863566036)*x[1]**o+(-0.957158639686)*x[1]
        s[2,4,2,0]=(0.606624368242)*x[0]**o+(-0.129636738653)*x[0]+(0.117469089776)*x[1]**o+(0.766024628396)*x[1]
        s[2,4,2,1]=(-0.83631907859)*x[0]**o+(-0.168433036335)*x[0]+(-0.500663355802)*x[1]**o+(0.0114866397792)*x[1]
        s[3,0,0,0]=(-0.629399168531)*x[0]**o+(-0.464524331386)*x[0]+(0.0406391749081)*x[1]**o+(-0.380881295086)*x[1]
        s[3,0,0,1]=(-0.614191170121)*x[0]**o+(0.90297284642)*x[0]+(0.949653922227)*x[1]**o+(0.947014467705)*x[1]
        s[3,0,1,0]=(0.128075669906)*x[0]**o+(-0.215027529464)*x[0]+(-0.230820603567)*x[1]**o+(0.656672788071)*x[1]
        s[3,0,1,1]=(0.11006414768)*x[0]**o+(0.575548749986)*x[0]+(0.803886215346)*x[1]**o+(0.627503866105)*x[1]
        s[3,0,2,0]=(-0.824524708364)*x[0]**o+(-0.399533304607)*x[0]+(-0.278996789484)*x[1]**o+(-0.139967848513)*x[1]
        s[3,0,2,1]=(-0.657392776251)*x[0]**o+(0.959283975752)*x[0]+(0.990715521701)*x[1]**o+(0.0999777335871)*x[1]
        s[3,1,0,0]=(-0.691053621801)*x[0]**o+(0.189005136467)*x[0]+(0.789186325959)*x[1]**o+(-0.981079749179)*x[1]
        s[3,1,0,1]=(0.228390867858)*x[0]**o+(-0.463941202938)*x[0]+(-0.782094886569)*x[1]**o+(0.464512245943)*x[1]
        s[3,1,1,0]=(-0.819255906497)*x[0]**o+(-0.204510874222)*x[0]+(-0.193473362908)*x[1]**o+(-0.0250165714593)*x[1]
        s[3,1,1,1]=(0.331636523937)*x[0]**o+(-0.0429730777685)*x[0]+(-0.547743299325)*x[1]**o+(-0.187953226002)*x[1]
        s[3,1,2,0]=(0.159370360993)*x[0]**o+(0.883315520778)*x[0]+(0.0933439209551)*x[1]**o+(-0.458319201953)*x[1]
        s[3,1,2,1]=(-0.273991258)*x[0]**o+(0.209739105337)*x[0]+(0.335210395431)*x[1]**o+(-0.91838746555)*x[1]
        s[3,2,0,0]=(-0.0618039563518)*x[0]**o+(-0.59799143228)*x[0]+(0.93203030688)*x[1]**o+(0.581545424351)*x[1]
        s[3,2,0,1]=(-0.321101903544)*x[0]**o+(-0.781260985212)*x[0]+(-0.106128211217)*x[1]**o+(0.913142993455)*x[1]
        s[3,2,1,0]=(0.584486365293)*x[0]**o+(-0.654479654421)*x[0]+(0.579992761397)*x[1]**o+(-0.984753896904)*x[1]
        s[3,2,1,1]=(0.186056629749)*x[0]**o+(-0.889254599383)*x[0]+(0.134406174831)*x[1]**o+(0.843880331117)*x[1]
        s[3,2,2,0]=(0.578447040133)*x[0]**o+(-0.354828599745)*x[0]+(0.943853840713)*x[1]**o+(0.668565941699)*x[1]
        s[3,2,2,1]=(-0.657678353823)*x[0]**o+(0.657623646562)*x[0]+(0.670144253782)*x[1]**o+(0.489533260867)*x[1]
        s[3,3,0,0]=(0.518241767069)*x[0]**o+(0.717113307079)*x[0]+(-0.986081844812)*x[1]**o+(-0.922915734137)*x[1]
        s[3,3,0,1]=(0.934651660463)*x[0]**o+(0.524253175591)*x[0]+(0.519169286504)*x[1]**o+(0.995798377114)*x[1]
        s[3,3,1,0]=(0.396606116937)*x[0]**o+(-0.479342680739)*x[0]+(0.677502711338)*x[1]**o+(-0.591869615963)*x[1]
        s[3,3,1,1]=(0.339631149039)*x[0]**o+(0.44124607198)*x[0]+(0.786875844013)*x[1]**o+(-0.925255814639)*x[1]
        s[3,3,2,0]=(-0.779510842649)*x[0]**o+(0.562505114965)*x[0]+(0.582094163828)*x[1]**o+(0.910521768351)*x[1]
        s[3,3,2,1]=(-0.661315883103)*x[0]**o+(0.226495727911)*x[0]+(-0.464093148756)*x[1]**o+(0.696758174053)*x[1]
        s[3,4,0,0]=(-0.0366658565966)*x[0]**o+(0.78669216474)*x[0]+(-0.784779466718)*x[1]**o+(0.137721843502)*x[1]
        s[3,4,0,1]=(-0.00884301625831)*x[0]**o+(0.25582694645)*x[0]+(-0.73173489905)*x[1]**o+(-0.956569853331)*x[1]
        s[3,4,1,0]=(-0.0884743360746)*x[0]**o+(-0.484605933801)*x[0]+(0.773028853802)*x[1]**o+(-0.13255190562)*x[1]
        s[3,4,1,1]=(0.597859646547)*x[0]**o+(-0.254684950139)*x[0]+(0.292389327091)*x[1]**o+(0.942836804031)*x[1]
        s[3,4,2,0]=(-0.82916540636)*x[0]**o+(0.533192478501)*x[0]+(0.577760569513)*x[1]**o+(-0.812132995663)*x[1]
        s[3,4,2,1]=(-0.0918171874792)*x[0]**o+(-0.232778513191)*x[0]+(-0.57345566951)*x[1]**o+(0.870122102)*x[1]
        ref[0,0,0,0]=(-0.939524201884)*x_ref[0]**o+(-0.294114596857)*x_ref[0]+(-0.662571514204)*x_ref[1]**o+(-0.295055230761)*x_ref[1]
        ref[0,0,0,1]=(0.174447802399)*x_ref[0]**o+(0.313121933505)*x_ref[0]+(0.780423776572)*x_ref[1]**o+(0.260329316622)*x_ref[1]
        ref[0,0,1,0]=(-0.307907192496)*x_ref[0]**o+(0.924386500426)*x_ref[0]+(-0.34849235247)*x_ref[1]**o+(0.42450311854)*x_ref[1]
        ref[0,0,1,1]=(-0.221818525078)*x_ref[0]**o+(0.334658153768)*x_ref[0]+(0.730236845709)*x_ref[1]**o+(0.923645750095)*x_ref[1]
        ref[0,0,2,0]=(-0.0573462957426)*x_ref[0]**o+(0.10400906253)*x_ref[0]+(-0.606757296289)*x_ref[1]**o+(-0.171453259849)*x_ref[1]
        ref[0,0,2,1]=(-0.653190260251)*x_ref[0]**o+(0.392209523496)*x_ref[0]+(0.984792984469)*x_ref[1]**o+(-0.310847119118)*x_ref[1]
        ref[0,1,0,0]=(-0.0980551705202)*x_ref[0]**o+(0.475337517365)*x_ref[0]+(-0.93300381011)*x_ref[1]**o+(0.0593095200419)*x_ref[1]
        ref[0,1,0,1]=(0.833302217025)*x_ref[0]**o+(-0.866194985925)*x_ref[0]+(0.337967734176)*x_ref[1]**o+(0.0581478569311)*x_ref[1]
        ref[0,1,1,0]=(0.936751875211)*x_ref[0]**o+(-0.217226318173)*x_ref[0]+(0.477011461247)*x_ref[1]**o+(0.221436266198)*x_ref[1]
        ref[0,1,1,1]=(-0.537625551791)*x_ref[0]**o+(0.718351842961)*x_ref[0]+(0.0286304587001)*x_ref[1]**o+(-0.721442785761)*x_ref[1]
        ref[0,1,2,0]=(-0.738645216931)*x_ref[0]**o+(0.0195980002286)*x_ref[0]+(0.827363980625)*x_ref[1]**o+(-0.743955943985)*x_ref[1]
        ref[0,1,2,1]=(-0.145033137282)*x_ref[0]**o+(0.98069402199)*x_ref[0]+(-0.167153611864)*x_ref[1]**o+(-0.577697516598)*x_ref[1]
        ref[0,2,0,0]=(0.594926068713)*x_ref[0]**o+(-0.124581510742)*x_ref[0]+(-0.392512448989)*x_ref[1]**o+(-0.989702605289)*x_ref[1]
        ref[0,2,0,1]=(-0.208680726694)*x_ref[0]**o+(0.0226857988067)*x_ref[0]+(0.0127574486126)*x_ref[1]**o+(-0.661169282937)*x_ref[1]
        ref[0,2,1,0]=(0.43109296375)*x_ref[0]**o+(-0.966243011304)*x_ref[0]+(-0.448863362387)*x_ref[1]**o+(-0.531981406137)*x_ref[1]
        ref[0,2,1,1]=(0.427441889618)*x_ref[0]**o+(-0.730863645031)*x_ref[0]+(0.0948615738498)*x_ref[1]**o+(0.349409965532)*x_ref[1]
        ref[0,2,2,0]=(0.0857935500808)*x_ref[0]**o+(0.916612852535)*x_ref[0]+(-0.5434740164)*x_ref[1]**o+(0.651670987189)*x_ref[1]
        ref[0,2,2,1]=(0.269775748844)*x_ref[0]**o+(-0.351727930666)*x_ref[0]+(0.648234694172)*x_ref[1]**o+(0.714331370173)*x_ref[1]
        ref[0,3,0,0]=(0.862594479045)*x_ref[0]**o+(-0.787345711339)*x_ref[0]+(0.1934849213)*x_ref[1]**o+(-0.977224028721)*x_ref[1]
        ref[0,3,0,1]=(0.166215962959)*x_ref[0]**o+(0.734752140512)*x_ref[0]+(0.373558020776)*x_ref[1]**o+(-0.589819927874)*x_ref[1]
        ref[0,3,1,0]=(0.0289724459782)*x_ref[0]**o+(0.517327391578)*x_ref[0]+(0.109956624336)*x_ref[1]**o+(-0.290855234976)*x_ref[1]
        ref[0,3,1,1]=(0.369852037916)*x_ref[0]**o+(0.184556562048)*x_ref[0]+(-0.694630650679)*x_ref[1]**o+(-0.526020870229)*x_ref[1]
        ref[0,3,2,0]=(0.397498255324)*x_ref[0]**o+(-0.375327133358)*x_ref[0]+(0.564571187906)*x_ref[1]**o+(-0.295820211045)*x_ref[1]
        ref[0,3,2,1]=(0.550532413074)*x_ref[0]**o+(-0.751307255234)*x_ref[0]+(0.323446174542)*x_ref[1]**o+(0.968475158376)*x_ref[1]
        ref[0,4,0,0]=(-0.112506956442)*x_ref[0]**o+(-0.432032518771)*x_ref[0]+(0.0205217950343)*x_ref[1]**o+(-0.507714722498)*x_ref[1]
        ref[0,4,0,1]=(-0.501932335824)*x_ref[0]**o+(-0.147371573026)*x_ref[0]+(-0.421130372732)*x_ref[1]**o+(-0.958029815205)*x_ref[1]
        ref[0,4,1,0]=(0.172226819831)*x_ref[0]**o+(-0.712313983987)*x_ref[0]+(0.859887204781)*x_ref[1]**o+(0.0661613029313)*x_ref[1]
        ref[0,4,1,1]=(-0.863367396226)*x_ref[0]**o+(0.150655310131)*x_ref[0]+(-0.446461969683)*x_ref[1]**o+(0.247377395525)*x_ref[1]
        ref[0,4,2,0]=(-0.0431980822723)*x_ref[0]**o+(0.676680822195)*x_ref[0]+(-0.613277443463)*x_ref[1]**o+(0.787356918727)*x_ref[1]
        ref[0,4,2,1]=(-0.0569719722636)*x_ref[0]**o+(0.641187476483)*x_ref[0]+(-0.303617110099)*x_ref[1]**o+(-0.888820557309)*x_ref[1]
        ref[1,0,0,0]=(-0.0552307803637)*x_ref[0]**o+(0.306010657529)*x_ref[0]+(0.488223930134)*x_ref[1]**o+(-0.682392555784)*x_ref[1]
        ref[1,0,0,1]=(-0.253901726526)*x_ref[0]**o+(0.73343629071)*x_ref[0]+(-0.390491326134)*x_ref[1]**o+(0.774605582697)*x_ref[1]
        ref[1,0,1,0]=(-0.42967954588)*x_ref[0]**o+(-0.751284399784)*x_ref[0]+(0.813738133871)*x_ref[1]**o+(-0.0995163058125)*x_ref[1]
        ref[1,0,1,1]=(-0.0880959500234)*x_ref[0]**o+(-0.781527901178)*x_ref[0]+(-0.496899067642)*x_ref[1]**o+(0.55524107606)*x_ref[1]
        ref[1,0,2,0]=(0.256546278479)*x_ref[0]**o+(-0.595310894415)*x_ref[0]+(-0.281823345122)*x_ref[1]**o+(-0.953457305769)*x_ref[1]
        ref[1,0,2,1]=(0.365266273739)*x_ref[0]**o+(-0.935192596652)*x_ref[0]+(0.740921701447)*x_ref[1]**o+(0.843620235195)*x_ref[1]
        ref[1,1,0,0]=(-0.992209271955)*x_ref[0]**o+(0.267499041637)*x_ref[0]+(-0.446633252289)*x_ref[1]**o+(0.357054618942)*x_ref[1]
        ref[1,1,0,1]=(0.863355358433)*x_ref[0]**o+(-0.104881968217)*x_ref[0]+(0.0776056200756)*x_ref[1]**o+(0.485205982195)*x_ref[1]
        ref[1,1,1,0]=(0.29822878169)*x_ref[0]**o+(0.766559198241)*x_ref[0]+(0.489839749671)*x_ref[1]**o+(-0.694316951865)*x_ref[1]
        ref[1,1,1,1]=(-0.947135250126)*x_ref[0]**o+(-0.952537408063)*x_ref[0]+(-0.840569482634)*x_ref[1]**o+(-0.0881389491948)*x_ref[1]
        ref[1,1,2,0]=(0.269982185854)*x_ref[0]**o+(0.290850786316)*x_ref[0]+(0.707550793741)*x_ref[1]**o+(-0.583787562553)*x_ref[1]
        ref[1,1,2,1]=(-0.998478389198)*x_ref[0]**o+(-0.902713626135)*x_ref[0]+(-0.0809583498397)*x_ref[1]**o+(0.242726761269)*x_ref[1]
        ref[1,2,0,0]=(-0.0752124155425)*x_ref[0]**o+(0.287338426631)*x_ref[0]+(-0.873281064449)*x_ref[1]**o+(0.86906507097)*x_ref[1]
        ref[1,2,0,1]=(-0.451012489095)*x_ref[0]**o+(0.692877058818)*x_ref[0]+(-0.771278511209)*x_ref[1]**o+(0.325983000966)*x_ref[1]
        ref[1,2,1,0]=(-0.633239034154)*x_ref[0]**o+(0.751673044983)*x_ref[0]+(-0.799706248119)*x_ref[1]**o+(0.0783814978548)*x_ref[1]
        ref[1,2,1,1]=(-0.496449080562)*x_ref[0]**o+(-0.382869354097)*x_ref[0]+(-0.658792080006)*x_ref[1]**o+(0.578499479656)*x_ref[1]
        ref[1,2,2,0]=(0.41041079346)*x_ref[0]**o+(0.552454636943)*x_ref[0]+(0.236050520221)*x_ref[1]**o+(0.477587390293)*x_ref[1]
        ref[1,2,2,1]=(0.649795041558)*x_ref[0]**o+(-0.947868231928)*x_ref[0]+(-0.529895928082)*x_ref[1]**o+(0.715695540026)*x_ref[1]
        ref[1,3,0,0]=(0.948112901731)*x_ref[0]**o+(-0.374624145588)*x_ref[0]+(0.806640283077)*x_ref[1]**o+(0.729074938843)*x_ref[1]
        ref[1,3,0,1]=(0.266956301954)*x_ref[0]**o+(-0.878004354786)*x_ref[0]+(-0.422298760963)*x_ref[1]**o+(-0.251922913127)*x_ref[1]
        ref[1,3,1,0]=(0.751159198422)*x_ref[0]**o+(-0.576983418224)*x_ref[0]+(0.427653287761)*x_ref[1]**o+(0.351116956206)*x_ref[1]
        ref[1,3,1,1]=(-0.804702781998)*x_ref[0]**o+(-0.218650676384)*x_ref[0]+(-0.832220670207)*x_ref[1]**o+(0.997469993794)*x_ref[1]
        ref[1,3,2,0]=(0.111110638087)*x_ref[0]**o+(-0.435831664904)*x_ref[0]+(-0.144481003079)*x_ref[1]**o+(-0.604349850119)*x_ref[1]
        ref[1,3,2,1]=(0.470311907248)*x_ref[0]**o+(-0.591709930358)*x_ref[0]+(-0.562662398681)*x_ref[1]**o+(0.0163307873851)*x_ref[1]
        ref[1,4,0,0]=(0.684349728229)*x_ref[0]**o+(0.404890565831)*x_ref[0]+(0.922438828147)*x_ref[1]**o+(0.939539648586)*x_ref[1]
        ref[1,4,0,1]=(-0.179161968364)*x_ref[0]**o+(0.983188684135)*x_ref[0]+(-0.722578271046)*x_ref[1]**o+(-0.450472366251)*x_ref[1]
        ref[1,4,1,0]=(0.243718494318)*x_ref[0]**o+(-0.0251174594804)*x_ref[0]+(0.952890549291)*x_ref[1]**o+(-0.398408844924)*x_ref[1]
        ref[1,4,1,1]=(-0.883090916963)*x_ref[0]**o+(-0.236073018867)*x_ref[0]+(-0.245092152479)*x_ref[1]**o+(0.846958498274)*x_ref[1]
        ref[1,4,2,0]=(-0.839928016329)*x_ref[0]**o+(-0.677734468188)*x_ref[0]+(-0.123639765438)*x_ref[1]**o+(-0.680672570927)*x_ref[1]
        ref[1,4,2,1]=(-0.234629335025)*x_ref[0]**o+(-0.567381745605)*x_ref[0]+(0.623480113756)*x_ref[1]**o+(0.2049427452)*x_ref[1]
        ref[2,0,0,0]=(-0.297265663727)*x_ref[0]**o+(0.00899195467866)*x_ref[0]+(0.992475896282)*x_ref[1]**o+(0.863451618621)*x_ref[1]
        ref[2,0,0,1]=(-0.24914517208)*x_ref[0]**o+(0.104091943078)*x_ref[0]+(0.762436787588)*x_ref[1]**o+(0.89363657955)*x_ref[1]
        ref[2,0,1,0]=(0.264734056412)*x_ref[0]**o+(-0.716868164934)*x_ref[0]+(-0.905285068877)*x_ref[1]**o+(0.0227117696273)*x_ref[1]
        ref[2,0,1,1]=(-0.932617617299)*x_ref[0]**o+(0.271634450254)*x_ref[0]+(0.51812940988)*x_ref[1]**o+(-0.484525267705)*x_ref[1]
        ref[2,0,2,0]=(0.364599957461)*x_ref[0]**o+(0.818588026862)*x_ref[0]+(0.70366241863)*x_ref[1]**o+(0.43910943526)*x_ref[1]
        ref[2,0,2,1]=(0.557891341305)*x_ref[0]**o+(-0.891810635437)*x_ref[0]+(0.649138154483)*x_ref[1]**o+(-0.213951745119)*x_ref[1]
        ref[2,1,0,0]=(0.994667066111)*x_ref[0]**o+(0.433110731778)*x_ref[0]+(0.787983438535)*x_ref[1]**o+(0.979717998129)*x_ref[1]
        ref[2,1,0,1]=(0.619284628164)*x_ref[0]**o+(0.662998143683)*x_ref[0]+(0.0328849267442)*x_ref[1]**o+(0.765881868994)*x_ref[1]
        ref[2,1,1,0]=(0.335939940328)*x_ref[0]**o+(0.556758825457)*x_ref[0]+(-0.524870937642)*x_ref[1]**o+(0.0316796705422)*x_ref[1]
        ref[2,1,1,1]=(-0.702249603437)*x_ref[0]**o+(0.544642480464)*x_ref[0]+(0.330520285989)*x_ref[1]**o+(0.313003559197)*x_ref[1]
        ref[2,1,2,0]=(0.0294247925115)*x_ref[0]**o+(0.10067678181)*x_ref[0]+(0.608148681239)*x_ref[1]**o+(-0.95697997936)*x_ref[1]
        ref[2,1,2,1]=(0.101913506022)*x_ref[0]**o+(-0.423061068258)*x_ref[0]+(0.303518988539)*x_ref[1]**o+(0.0108182161414)*x_ref[1]
        ref[2,2,0,0]=(0.390642040424)*x_ref[0]**o+(0.0578342521895)*x_ref[0]+(0.228499563768)*x_ref[1]**o+(-0.944433241107)*x_ref[1]
        ref[2,2,0,1]=(0.522633870922)*x_ref[0]**o+(0.724655140578)*x_ref[0]+(-0.664760217849)*x_ref[1]**o+(0.812535008015)*x_ref[1]
        ref[2,2,1,0]=(0.27608720491)*x_ref[0]**o+(0.42350834695)*x_ref[0]+(-0.732116287856)*x_ref[1]**o+(0.930674651671)*x_ref[1]
        ref[2,2,1,1]=(0.214308063462)*x_ref[0]**o+(0.815707217077)*x_ref[0]+(0.231785784747)*x_ref[1]**o+(-0.954429150944)*x_ref[1]
        ref[2,2,2,0]=(0.402814098347)*x_ref[0]**o+(-0.650491293555)*x_ref[0]+(-0.359159927669)*x_ref[1]**o+(0.503842188774)*x_ref[1]
        ref[2,2,2,1]=(-0.191195415522)*x_ref[0]**o+(0.980791691347)*x_ref[0]+(0.228597878156)*x_ref[1]**o+(0.535975172993)*x_ref[1]
        ref[2,3,0,0]=(0.495749327231)*x_ref[0]**o+(0.311034011895)*x_ref[0]+(-0.532686805928)*x_ref[1]**o+(0.92438969176)*x_ref[1]
        ref[2,3,0,1]=(-0.38513599341)*x_ref[0]**o+(-0.463099237819)*x_ref[0]+(-0.992606328662)*x_ref[1]**o+(-0.927920439344)*x_ref[1]
        ref[2,3,1,0]=(-0.868022271483)*x_ref[0]**o+(-0.540096192748)*x_ref[0]+(-0.0879858139685)*x_ref[1]**o+(0.534604810517)*x_ref[1]
        ref[2,3,1,1]=(-0.686837754683)*x_ref[0]**o+(-0.505585075528)*x_ref[0]+(-0.218155668559)*x_ref[1]**o+(-0.241972887878)*x_ref[1]
        ref[2,3,2,0]=(0.86187325655)*x_ref[0]**o+(-0.733958307606)*x_ref[0]+(-0.107393817742)*x_ref[1]**o+(-0.830306745614)*x_ref[1]
        ref[2,3,2,1]=(0.404834271017)*x_ref[0]**o+(0.44646270613)*x_ref[0]+(-0.657038236282)*x_ref[1]**o+(-0.540570795414)*x_ref[1]
        ref[2,4,0,0]=(-0.216828677229)*x_ref[0]**o+(0.129702603598)*x_ref[0]+(0.556920803376)*x_ref[1]**o+(0.960021941421)*x_ref[1]
        ref[2,4,0,1]=(-0.21625137654)*x_ref[0]**o+(-0.19604294366)*x_ref[0]+(-0.889082170851)*x_ref[1]**o+(-0.834967241538)*x_ref[1]
        ref[2,4,1,0]=(0.123104286845)*x_ref[0]**o+(-0.43197703752)*x_ref[0]+(0.737850827826)*x_ref[1]**o+(0.344689639656)*x_ref[1]
        ref[2,4,1,1]=(0.119471348292)*x_ref[0]**o+(0.948353928609)*x_ref[0]+(-0.726863566036)*x_ref[1]**o+(-0.957158639686)*x_ref[1]
        ref[2,4,2,0]=(0.606624368242)*x_ref[0]**o+(-0.129636738653)*x_ref[0]+(0.117469089776)*x_ref[1]**o+(0.766024628396)*x_ref[1]
        ref[2,4,2,1]=(-0.83631907859)*x_ref[0]**o+(-0.168433036335)*x_ref[0]+(-0.500663355802)*x_ref[1]**o+(0.0114866397792)*x_ref[1]
        ref[3,0,0,0]=(-0.629399168531)*x_ref[0]**o+(-0.464524331386)*x_ref[0]+(0.0406391749081)*x_ref[1]**o+(-0.380881295086)*x_ref[1]
        ref[3,0,0,1]=(-0.614191170121)*x_ref[0]**o+(0.90297284642)*x_ref[0]+(0.949653922227)*x_ref[1]**o+(0.947014467705)*x_ref[1]
        ref[3,0,1,0]=(0.128075669906)*x_ref[0]**o+(-0.215027529464)*x_ref[0]+(-0.230820603567)*x_ref[1]**o+(0.656672788071)*x_ref[1]
        ref[3,0,1,1]=(0.11006414768)*x_ref[0]**o+(0.575548749986)*x_ref[0]+(0.803886215346)*x_ref[1]**o+(0.627503866105)*x_ref[1]
        ref[3,0,2,0]=(-0.824524708364)*x_ref[0]**o+(-0.399533304607)*x_ref[0]+(-0.278996789484)*x_ref[1]**o+(-0.139967848513)*x_ref[1]
        ref[3,0,2,1]=(-0.657392776251)*x_ref[0]**o+(0.959283975752)*x_ref[0]+(0.990715521701)*x_ref[1]**o+(0.0999777335871)*x_ref[1]
        ref[3,1,0,0]=(-0.691053621801)*x_ref[0]**o+(0.189005136467)*x_ref[0]+(0.789186325959)*x_ref[1]**o+(-0.981079749179)*x_ref[1]
        ref[3,1,0,1]=(0.228390867858)*x_ref[0]**o+(-0.463941202938)*x_ref[0]+(-0.782094886569)*x_ref[1]**o+(0.464512245943)*x_ref[1]
        ref[3,1,1,0]=(-0.819255906497)*x_ref[0]**o+(-0.204510874222)*x_ref[0]+(-0.193473362908)*x_ref[1]**o+(-0.0250165714593)*x_ref[1]
        ref[3,1,1,1]=(0.331636523937)*x_ref[0]**o+(-0.0429730777685)*x_ref[0]+(-0.547743299325)*x_ref[1]**o+(-0.187953226002)*x_ref[1]
        ref[3,1,2,0]=(0.159370360993)*x_ref[0]**o+(0.883315520778)*x_ref[0]+(0.0933439209551)*x_ref[1]**o+(-0.458319201953)*x_ref[1]
        ref[3,1,2,1]=(-0.273991258)*x_ref[0]**o+(0.209739105337)*x_ref[0]+(0.335210395431)*x_ref[1]**o+(-0.91838746555)*x_ref[1]
        ref[3,2,0,0]=(-0.0618039563518)*x_ref[0]**o+(-0.59799143228)*x_ref[0]+(0.93203030688)*x_ref[1]**o+(0.581545424351)*x_ref[1]
        ref[3,2,0,1]=(-0.321101903544)*x_ref[0]**o+(-0.781260985212)*x_ref[0]+(-0.106128211217)*x_ref[1]**o+(0.913142993455)*x_ref[1]
        ref[3,2,1,0]=(0.584486365293)*x_ref[0]**o+(-0.654479654421)*x_ref[0]+(0.579992761397)*x_ref[1]**o+(-0.984753896904)*x_ref[1]
        ref[3,2,1,1]=(0.186056629749)*x_ref[0]**o+(-0.889254599383)*x_ref[0]+(0.134406174831)*x_ref[1]**o+(0.843880331117)*x_ref[1]
        ref[3,2,2,0]=(0.578447040133)*x_ref[0]**o+(-0.354828599745)*x_ref[0]+(0.943853840713)*x_ref[1]**o+(0.668565941699)*x_ref[1]
        ref[3,2,2,1]=(-0.657678353823)*x_ref[0]**o+(0.657623646562)*x_ref[0]+(0.670144253782)*x_ref[1]**o+(0.489533260867)*x_ref[1]
        ref[3,3,0,0]=(0.518241767069)*x_ref[0]**o+(0.717113307079)*x_ref[0]+(-0.986081844812)*x_ref[1]**o+(-0.922915734137)*x_ref[1]
        ref[3,3,0,1]=(0.934651660463)*x_ref[0]**o+(0.524253175591)*x_ref[0]+(0.519169286504)*x_ref[1]**o+(0.995798377114)*x_ref[1]
        ref[3,3,1,0]=(0.396606116937)*x_ref[0]**o+(-0.479342680739)*x_ref[0]+(0.677502711338)*x_ref[1]**o+(-0.591869615963)*x_ref[1]
        ref[3,3,1,1]=(0.339631149039)*x_ref[0]**o+(0.44124607198)*x_ref[0]+(0.786875844013)*x_ref[1]**o+(-0.925255814639)*x_ref[1]
        ref[3,3,2,0]=(-0.779510842649)*x_ref[0]**o+(0.562505114965)*x_ref[0]+(0.582094163828)*x_ref[1]**o+(0.910521768351)*x_ref[1]
        ref[3,3,2,1]=(-0.661315883103)*x_ref[0]**o+(0.226495727911)*x_ref[0]+(-0.464093148756)*x_ref[1]**o+(0.696758174053)*x_ref[1]
        ref[3,4,0,0]=(-0.0366658565966)*x_ref[0]**o+(0.78669216474)*x_ref[0]+(-0.784779466718)*x_ref[1]**o+(0.137721843502)*x_ref[1]
        ref[3,4,0,1]=(-0.00884301625831)*x_ref[0]**o+(0.25582694645)*x_ref[0]+(-0.73173489905)*x_ref[1]**o+(-0.956569853331)*x_ref[1]
        ref[3,4,1,0]=(-0.0884743360746)*x_ref[0]**o+(-0.484605933801)*x_ref[0]+(0.773028853802)*x_ref[1]**o+(-0.13255190562)*x_ref[1]
        ref[3,4,1,1]=(0.597859646547)*x_ref[0]**o+(-0.254684950139)*x_ref[0]+(0.292389327091)*x_ref[1]**o+(0.942836804031)*x_ref[1]
        ref[3,4,2,0]=(-0.82916540636)*x_ref[0]**o+(0.533192478501)*x_ref[0]+(0.577760569513)*x_ref[1]**o+(-0.812132995663)*x_ref[1]
        ref[3,4,2,1]=(-0.0918171874792)*x_ref[0]**o+(-0.232778513191)*x_ref[0]+(-0.57345566951)*x_ref[1]**o+(0.870122102)*x_ref[1]
      else:
        s[0,0,0,0]=(-0.419262547354)*x[0]**o+(-0.607792358178)*x[0]+(0.539129711536)*x[1]**o+(-0.954002022975)*x[1]+(0.52016184412)*x[2]**o+(-0.905599414589)*x[2]
        s[0,0,0,1]=(0.552249219683)*x[0]**o+(0.0747535682119)*x[0]+(-0.0550858389381)*x[1]**o+(-0.700756838491)*x[1]+(-0.62569620016)*x[2]**o+(-0.557287205831)*x[2]
        s[0,0,1,0]=(-0.84674266979)*x[0]**o+(-0.266089976589)*x[0]+(-0.994821395425)*x[1]**o+(-0.567614298529)*x[1]+(-0.814756268883)*x[2]**o+(-0.524622381582)*x[2]
        s[0,0,1,1]=(0.160195104873)*x[0]**o+(0.161596910068)*x[0]+(-0.0404024431157)*x[1]**o+(0.299140544701)*x[1]+(0.479120924271)*x[2]**o+(0.375793387197)*x[2]
        s[0,0,2,0]=(0.619011883068)*x[0]**o+(0.453196321806)*x[0]+(0.596546848845)*x[1]**o+(-0.148655559308)*x[1]+(-0.276643963879)*x[2]**o+(-0.556233199969)*x[2]
        s[0,0,2,1]=(-0.44936519278)*x[0]**o+(0.279187353573)*x[0]+(-0.626699833624)*x[1]**o+(0.172543451997)*x[1]+(0.324942859052)*x[2]**o+(0.227082355579)*x[2]
        s[0,1,0,0]=(0.348983572344)*x[0]**o+(-0.692310510135)*x[0]+(0.127542726356)*x[1]**o+(-0.719693998448)*x[1]+(0.190533765375)*x[2]**o+(-0.44411272509)*x[2]
        s[0,1,0,1]=(0.755193842627)*x[0]**o+(-0.0380801278139)*x[0]+(-0.416973165389)*x[1]**o+(-0.407050428042)*x[1]+(-0.803402606403)*x[2]**o+(0.860761604165)*x[2]
        s[0,1,1,0]=(0.903125811803)*x[0]**o+(-0.370434864257)*x[0]+(-0.810880082994)*x[1]**o+(0.214166691855)*x[1]+(-0.62621693665)*x[2]**o+(-0.043360368073)*x[2]
        s[0,1,1,1]=(-0.442516790862)*x[0]**o+(0.912139701807)*x[0]+(0.871611995743)*x[1]**o+(0.0291580448949)*x[1]+(-0.489427959855)*x[2]**o+(-0.950170294127)*x[2]
        s[0,1,2,0]=(-0.0867645917569)*x[0]**o+(0.556462347924)*x[0]+(-0.816554443299)*x[1]**o+(0.818773437796)*x[1]+(0.073905470924)*x[2]**o+(-0.714497094507)*x[2]
        s[0,1,2,1]=(0.699491100428)*x[0]**o+(-0.200275804269)*x[0]+(0.994316913102)*x[1]**o+(0.832890328003)*x[1]+(0.385381865669)*x[2]**o+(0.153056606745)*x[2]
        s[0,2,0,0]=(0.612319496073)*x[0]**o+(-0.301991428044)*x[0]+(-0.536866971844)*x[1]**o+(-0.375311119699)*x[1]+(0.733545441876)*x[2]**o+(0.0177187199528)*x[2]
        s[0,2,0,1]=(0.358991602948)*x[0]**o+(-0.862573091669)*x[0]+(0.464891723814)*x[1]**o+(0.602366461601)*x[1]+(-0.668916509347)*x[2]**o+(0.76048108017)*x[2]
        s[0,2,1,0]=(0.188195665573)*x[0]**o+(-0.927407688437)*x[0]+(-0.473014301986)*x[1]**o+(-0.775114885201)*x[1]+(-0.453403342886)*x[2]**o+(0.006776981492)*x[2]
        s[0,2,1,1]=(-0.700181448543)*x[0]**o+(-0.289180138599)*x[0]+(-0.867022883968)*x[1]**o+(-0.359339607516)*x[1]+(-0.809558496152)*x[2]**o+(-0.576358976651)*x[2]
        s[0,2,2,0]=(-0.557333080207)*x[0]**o+(0.724962853865)*x[0]+(0.995041226791)*x[1]**o+(-0.610460710065)*x[1]+(0.620016428769)*x[2]**o+(-0.449182254452)*x[2]
        s[0,2,2,1]=(-0.937207840292)*x[0]**o+(-0.66582207891)*x[0]+(0.0951881913165)*x[1]**o+(-0.0770694628972)*x[1]+(0.034224640398)*x[2]**o+(0.652057466224)*x[2]
        s[0,3,0,0]=(-0.0236750483037)*x[0]**o+(0.956130793675)*x[0]+(-0.589425964387)*x[1]**o+(-0.204770941511)*x[1]+(0.695255730164)*x[2]**o+(-0.796222694557)*x[2]
        s[0,3,0,1]=(0.138589818728)*x[0]**o+(0.493237559501)*x[0]+(0.139281181588)*x[1]**o+(-0.974201990591)*x[1]+(-0.15401887402)*x[2]**o+(0.23794136041)*x[2]
        s[0,3,1,0]=(-0.332886758751)*x[0]**o+(-0.231549718295)*x[0]+(0.581536450397)*x[1]**o+(0.790063092494)*x[1]+(0.358814129932)*x[2]**o+(-0.110547525643)*x[2]
        s[0,3,1,1]=(0.122775146487)*x[0]**o+(-0.00818530079912)*x[0]+(0.0510575666357)*x[1]**o+(-0.96911992662)*x[1]+(-0.820269176326)*x[2]**o+(0.875945222846)*x[2]
        s[0,3,2,0]=(0.761874596666)*x[0]**o+(0.621369218231)*x[0]+(0.960271412263)*x[1]**o+(-0.808832785376)*x[1]+(0.981428454481)*x[2]**o+(0.393331122254)*x[2]
        s[0,3,2,1]=(-0.646477042209)*x[0]**o+(-0.26169646294)*x[0]+(0.992265690785)*x[1]**o+(-0.678558332435)*x[1]+(-0.285531525349)*x[2]**o+(-0.571647919822)*x[2]
        s[0,4,0,0]=(0.492930260015)*x[0]**o+(-0.647489772768)*x[0]+(-0.381225997009)*x[1]**o+(0.560462598779)*x[1]+(-0.63571677055)*x[2]**o+(0.103213895387)*x[2]
        s[0,4,0,1]=(-0.0362476689653)*x[0]**o+(-0.478992488756)*x[0]+(0.544907192925)*x[1]**o+(-0.678350370776)*x[1]+(-0.0776363475051)*x[2]**o+(-0.803140341684)*x[2]
        s[0,4,1,0]=(0.0787683344872)*x[0]**o+(0.688990188782)*x[0]+(0.257089245341)*x[1]**o+(-0.648663765213)*x[1]+(-0.140632312728)*x[2]**o+(0.676209090063)*x[2]
        s[0,4,1,1]=(0.804994807205)*x[0]**o+(0.74750498671)*x[0]+(-0.00532761506025)*x[1]**o+(-0.295700694165)*x[1]+(-0.209175099115)*x[2]**o+(-0.0126407798374)*x[2]
        s[0,4,2,0]=(-0.789160064467)*x[0]**o+(0.332412023778)*x[0]+(0.242860020473)*x[1]**o+(0.0284658318544)*x[1]+(-0.357559965972)*x[2]**o+(0.57056582293)*x[2]
        s[0,4,2,1]=(-0.861027741096)*x[0]**o+(0.545166238342)*x[0]+(0.950975873381)*x[1]**o+(-0.436709572856)*x[1]+(0.93636977927)*x[2]**o+(-0.405205657349)*x[2]
        s[1,0,0,0]=(0.47880791873)*x[0]**o+(0.037486909403)*x[0]+(0.732218957477)*x[1]**o+(0.274811688271)*x[1]+(0.0686323288694)*x[2]**o+(-0.492347744678)*x[2]
        s[1,0,0,1]=(-0.618259414329)*x[0]**o+(0.630883795613)*x[0]+(0.269141377847)*x[1]**o+(-0.490916952452)*x[1]+(-0.333070678904)*x[2]**o+(0.0484209401349)*x[2]
        s[1,0,1,0]=(-0.906059152197)*x[0]**o+(0.0322820078282)*x[0]+(-0.47807076381)*x[1]**o+(0.0296380181906)*x[1]+(0.553686905633)*x[2]**o+(-0.408333784569)*x[2]
        s[1,0,1,1]=(0.282962689091)*x[0]**o+(-0.11495842442)*x[0]+(0.818142653998)*x[1]**o+(0.563460324128)*x[1]+(-0.993782402791)*x[2]**o+(-0.0193690715193)*x[2]
        s[1,0,2,0]=(0.287962506263)*x[0]**o+(-0.401293374286)*x[0]+(0.173559459587)*x[1]**o+(0.822343918838)*x[1]+(0.888309195417)*x[2]**o+(0.687878298228)*x[2]
        s[1,0,2,1]=(-0.926775125361)*x[0]**o+(0.396396669159)*x[0]+(0.632962652289)*x[1]**o+(-0.915667885738)*x[1]+(-0.31075960693)*x[2]**o+(0.920135418244)*x[2]
        s[1,1,0,0]=(-0.0915500113579)*x[0]**o+(0.489036843145)*x[0]+(0.477982330324)*x[1]**o+(0.703371804742)*x[1]+(-0.471773636091)*x[2]**o+(0.0665811350713)*x[2]
        s[1,1,0,1]=(0.525411480834)*x[0]**o+(-0.369843393433)*x[0]+(-0.7770705966)*x[1]**o+(0.534153802448)*x[1]+(0.309828458492)*x[2]**o+(0.627690678711)*x[2]
        s[1,1,1,0]=(0.149039257889)*x[0]**o+(-0.0585283560518)*x[0]+(-0.0505893024858)*x[1]**o+(0.677281229531)*x[1]+(-0.451950028252)*x[2]**o+(0.356600216864)*x[2]
        s[1,1,1,1]=(-0.528237952467)*x[0]**o+(0.533554438129)*x[0]+(0.741981923136)*x[1]**o+(-0.596922185095)*x[1]+(0.50521173977)*x[2]**o+(-0.247100318829)*x[2]
        s[1,1,2,0]=(0.667977944954)*x[0]**o+(0.306325804541)*x[0]+(-0.654643365837)*x[1]**o+(0.0176380527447)*x[1]+(-0.218012920296)*x[2]**o+(-0.181740681902)*x[2]
        s[1,1,2,1]=(0.440430981908)*x[0]**o+(0.994608754477)*x[0]+(0.114426402744)*x[1]**o+(-0.896222828258)*x[1]+(0.997911423594)*x[2]**o+(0.648328801403)*x[2]
        s[1,2,0,0]=(0.397328767103)*x[0]**o+(0.780110953188)*x[0]+(0.453955649367)*x[1]**o+(0.953839185994)*x[1]+(0.995536671917)*x[2]**o+(0.177278490615)*x[2]
        s[1,2,0,1]=(-0.154393442621)*x[0]**o+(-0.100267841021)*x[0]+(-0.781347482738)*x[1]**o+(0.391441261298)*x[1]+(0.469832428629)*x[2]**o+(0.608531800159)*x[2]
        s[1,2,1,0]=(-0.534167310466)*x[0]**o+(0.414564904301)*x[0]+(-0.0674568638795)*x[1]**o+(0.577721681963)*x[1]+(-0.00750850180411)*x[2]**o+(0.0817658166565)*x[2]
        s[1,2,1,1]=(-0.529547188418)*x[0]**o+(0.267917121456)*x[0]+(0.372449729423)*x[1]**o+(0.887072823755)*x[1]+(0.887821539758)*x[2]**o+(0.249771090594)*x[2]
        s[1,2,2,0]=(0.949056994737)*x[0]**o+(-0.174523839411)*x[0]+(-0.646085737348)*x[1]**o+(0.449358235114)*x[1]+(-0.970748494721)*x[2]**o+(-0.410800591341)*x[2]
        s[1,2,2,1]=(0.879006069022)*x[0]**o+(-0.510106021113)*x[0]+(0.0203703221469)*x[1]**o+(-0.844338135123)*x[1]+(0.752168319634)*x[2]**o+(0.158760999046)*x[2]
        s[1,3,0,0]=(0.845321813876)*x[0]**o+(0.405224606418)*x[0]+(-0.89159606951)*x[1]**o+(-0.949499370695)*x[1]+(0.501499536159)*x[2]**o+(-0.869583954204)*x[2]
        s[1,3,0,1]=(0.0887248259912)*x[0]**o+(0.232925681778)*x[0]+(0.0315176777236)*x[1]**o+(0.00568107770424)*x[1]+(-0.821963787641)*x[2]**o+(-0.634962715535)*x[2]
        s[1,3,1,0]=(0.546210053059)*x[0]**o+(-0.688171108228)*x[0]+(-0.699621490343)*x[1]**o+(-0.850445270177)*x[1]+(-0.725131419114)*x[2]**o+(0.484336843024)*x[2]
        s[1,3,1,1]=(0.278594806693)*x[0]**o+(-0.266710093152)*x[0]+(-0.151515190688)*x[1]**o+(-0.705005965054)*x[1]+(0.811008835405)*x[2]**o+(-0.342637057553)*x[2]
        s[1,3,2,0]=(0.4748294578)*x[0]**o+(-0.0741346092747)*x[0]+(0.148026337115)*x[1]**o+(0.0252686427245)*x[1]+(-0.479120476874)*x[2]**o+(0.0855008343792)*x[2]
        s[1,3,2,1]=(-0.337007818128)*x[0]**o+(-0.690464368815)*x[0]+(0.481925609432)*x[1]**o+(-0.0981705702921)*x[1]+(0.19218353071)*x[2]**o+(0.988337158745)*x[2]
        s[1,4,0,0]=(-0.757351506535)*x[0]**o+(-0.824203825425)*x[0]+(-0.998647750787)*x[1]**o+(-0.111654698732)*x[1]+(-0.713287873387)*x[2]**o+(0.779671510732)*x[2]
        s[1,4,0,1]=(-0.756370998169)*x[0]**o+(0.826863767951)*x[0]+(0.358310449165)*x[1]**o+(0.882430292099)*x[1]+(0.880156909975)*x[2]**o+(0.360411043274)*x[2]
        s[1,4,1,0]=(0.226080011536)*x[0]**o+(-0.740536947501)*x[0]+(-0.650527457572)*x[1]**o+(-0.890015156497)*x[1]+(0.688973071404)*x[2]**o+(0.29237820526)*x[2]
        s[1,4,1,1]=(0.574015626188)*x[0]**o+(-0.675641095131)*x[0]+(0.814289365677)*x[1]**o+(-0.625021588943)*x[1]+(-0.862079110641)*x[2]**o+(0.0363949889967)*x[2]
        s[1,4,2,0]=(0.126659193582)*x[0]**o+(0.16976937315)*x[0]+(0.18822384431)*x[1]**o+(0.546082142359)*x[1]+(0.551365996908)*x[2]**o+(0.395213783677)*x[2]
        s[1,4,2,1]=(-0.292436267133)*x[0]**o+(-0.442787521929)*x[0]+(-0.310488595304)*x[1]**o+(0.0918858003303)*x[1]+(-0.986695921261)*x[2]**o+(-0.52979957068)*x[2]
        s[2,0,0,0]=(0.396669972067)*x[0]**o+(-0.711415400722)*x[0]+(-0.717006050735)*x[1]**o+(-0.0935757817421)*x[1]+(0.743477556176)*x[2]**o+(0.823967976857)*x[2]
        s[2,0,0,1]=(0.488395228667)*x[0]**o+(0.230144035899)*x[0]+(0.125851877066)*x[1]**o+(0.0536284998009)*x[1]+(-0.0114515128334)*x[2]**o+(-0.494039400348)*x[2]
        s[2,0,1,0]=(0.900615108208)*x[0]**o+(0.432765237128)*x[0]+(0.981213121806)*x[1]**o+(-0.116934702769)*x[1]+(-0.612499086627)*x[2]**o+(-0.365572411348)*x[2]
        s[2,0,1,1]=(0.0342670705338)*x[0]**o+(0.54954404771)*x[0]+(0.428653302401)*x[1]**o+(0.537787906346)*x[1]+(-0.64287424457)*x[2]**o+(-0.173877644259)*x[2]
        s[2,0,2,0]=(-0.0510871045727)*x[0]**o+(-0.908007271573)*x[0]+(0.00194326164542)*x[1]**o+(0.466243299714)*x[1]+(-0.433962263339)*x[2]**o+(0.874724229969)*x[2]
        s[2,0,2,1]=(0.666308090297)*x[0]**o+(0.933188439742)*x[0]+(-0.0479445062129)*x[1]**o+(0.0747806162498)*x[1]+(0.911844507192)*x[2]**o+(-0.934538480024)*x[2]
        s[2,1,0,0]=(-0.516290157366)*x[0]**o+(0.437519025532)*x[0]+(-0.170999588789)*x[1]**o+(-0.934096554106)*x[1]+(0.88575800375)*x[2]**o+(0.636966058116)*x[2]
        s[2,1,0,1]=(0.0172669412005)*x[0]**o+(-0.867857912246)*x[0]+(-0.972747832278)*x[1]**o+(-0.780969292213)*x[1]+(-0.418154913379)*x[2]**o+(0.805902597544)*x[2]
        s[2,1,1,0]=(-0.964006922451)*x[0]**o+(0.236797578253)*x[0]+(-0.78802971076)*x[1]**o+(0.704165381448)*x[1]+(0.00112348607329)*x[2]**o+(-0.689065853536)*x[2]
        s[2,1,1,1]=(-0.722392335335)*x[0]**o+(-0.120018753634)*x[0]+(0.216541173904)*x[1]**o+(-0.374710223309)*x[1]+(0.532158723682)*x[2]**o+(0.464637663705)*x[2]
        s[2,1,2,0]=(-0.183637646888)*x[0]**o+(-0.0680674034088)*x[0]+(-0.114129008735)*x[1]**o+(-0.78666547603)*x[1]+(0.0956689943304)*x[2]**o+(0.743296994432)*x[2]
        s[2,1,2,1]=(0.611382565923)*x[0]**o+(-0.768682259494)*x[0]+(-0.540943249593)*x[1]**o+(0.753523431999)*x[1]+(0.610689642267)*x[2]**o+(0.0122811249496)*x[2]
        s[2,2,0,0]=(-0.789273374112)*x[0]**o+(0.0977540682694)*x[0]+(-0.829719260296)*x[1]**o+(0.824773509135)*x[1]+(0.216669854484)*x[2]**o+(-0.764947843182)*x[2]
        s[2,2,0,1]=(-0.340784170043)*x[0]**o+(-0.267437208067)*x[0]+(-0.430973997706)*x[1]**o+(-0.0846177320866)*x[1]+(0.688599546971)*x[2]**o+(0.265513958383)*x[2]
        s[2,2,1,0]=(-0.237677411545)*x[0]**o+(-0.792144835449)*x[0]+(-0.435763499999)*x[1]**o+(0.682920743804)*x[1]+(0.116459199169)*x[2]**o+(0.254527225097)*x[2]
        s[2,2,1,1]=(0.342906428734)*x[0]**o+(0.206470310722)*x[0]+(-0.352429884648)*x[1]**o+(-0.168905312607)*x[1]+(-0.105529665095)*x[2]**o+(0.199873788005)*x[2]
        s[2,2,2,0]=(0.677855511296)*x[0]**o+(0.983953993003)*x[0]+(-0.64997591911)*x[1]**o+(-0.629221981454)*x[1]+(0.990621105566)*x[2]**o+(0.785766133934)*x[2]
        s[2,2,2,1]=(-0.570482617922)*x[0]**o+(-0.151066051792)*x[0]+(-0.511453364326)*x[1]**o+(0.128824691419)*x[1]+(-0.520127324328)*x[2]**o+(-0.157450052117)*x[2]
        s[2,3,0,0]=(-0.423827660795)*x[0]**o+(-0.815943363397)*x[0]+(-0.636070023986)*x[1]**o+(0.251327239067)*x[1]+(-0.401641739847)*x[2]**o+(0.130280903755)*x[2]
        s[2,3,0,1]=(0.0411439121625)*x[0]**o+(0.621609981629)*x[0]+(-0.589059932668)*x[1]**o+(0.543095349334)*x[1]+(-0.0122421869863)*x[2]**o+(-0.776924587359)*x[2]
        s[2,3,1,0]=(0.63252327363)*x[0]**o+(0.275597518383)*x[0]+(0.832198526441)*x[1]**o+(-0.92855918951)*x[1]+(0.490343890624)*x[2]**o+(-0.412580022625)*x[2]
        s[2,3,1,1]=(-0.560907436774)*x[0]**o+(0.413956414518)*x[0]+(-0.849736975879)*x[1]**o+(0.167861053833)*x[1]+(-0.316526371498)*x[2]**o+(-0.431245755915)*x[2]
        s[2,3,2,0]=(0.883757549647)*x[0]**o+(-0.614133317417)*x[0]+(0.923442235162)*x[1]**o+(0.483998522495)*x[1]+(0.649003328717)*x[2]**o+(-0.320181801832)*x[2]
        s[2,3,2,1]=(0.181982086686)*x[0]**o+(-0.88761330228)*x[0]+(-0.752108545831)*x[1]**o+(0.105681418232)*x[1]+(0.211053107745)*x[2]**o+(-0.840650090772)*x[2]
        s[2,4,0,0]=(0.774235220873)*x[0]**o+(-0.394430371883)*x[0]+(-0.82551531411)*x[1]**o+(-0.397374538567)*x[1]+(0.51757884795)*x[2]**o+(-0.878919341817)*x[2]
        s[2,4,0,1]=(-0.282445913776)*x[0]**o+(-0.252614209563)*x[0]+(-0.66810962709)*x[1]**o+(0.912660025157)*x[1]+(0.786200789061)*x[2]**o+(-0.271329907892)*x[2]
        s[2,4,1,0]=(-0.415061697718)*x[0]**o+(-0.577077815899)*x[0]+(-0.479115469184)*x[1]**o+(0.285424287206)*x[1]+(0.580737671113)*x[2]**o+(0.345716282261)*x[2]
        s[2,4,1,1]=(0.103116885191)*x[0]**o+(0.472934825259)*x[0]+(0.192747945705)*x[1]**o+(0.106949097334)*x[1]+(-0.109457290729)*x[2]**o+(0.843345964596)*x[2]
        s[2,4,2,0]=(0.600281530422)*x[0]**o+(-0.427230650838)*x[0]+(-0.868150408946)*x[1]**o+(-0.203647711262)*x[1]+(-0.550477804035)*x[2]**o+(0.415736214172)*x[2]
        s[2,4,2,1]=(0.739232217202)*x[0]**o+(-0.476864848171)*x[0]+(0.367627911492)*x[1]**o+(0.105176582927)*x[1]+(0.482113806667)*x[2]**o+(0.782020005885)*x[2]
        s[3,0,0,0]=(0.938839141593)*x[0]**o+(-0.655793939395)*x[0]+(0.0224780413611)*x[1]**o+(0.252535317383)*x[1]+(-0.887022478757)*x[2]**o+(-0.451912746232)*x[2]
        s[3,0,0,1]=(-0.196994797427)*x[0]**o+(-0.222816920678)*x[0]+(0.37131460044)*x[1]**o+(0.124872956754)*x[1]+(-0.872680499884)*x[2]**o+(-0.490521736465)*x[2]
        s[3,0,1,0]=(-0.977712672434)*x[0]**o+(-0.665877989129)*x[0]+(-0.541103000349)*x[1]**o+(-0.527523471522)*x[1]+(0.469063433285)*x[2]**o+(-0.410680406273)*x[2]
        s[3,0,1,1]=(0.371612180735)*x[0]**o+(-0.882397978651)*x[0]+(-0.333119569273)*x[1]**o+(0.0439599202688)*x[1]+(-0.378698960758)*x[2]**o+(0.373830212832)*x[2]
        s[3,0,2,0]=(0.13552136805)*x[0]**o+(-0.347572305079)*x[0]+(0.194173745453)*x[1]**o+(-0.536374783086)*x[1]+(-0.0928402883882)*x[2]**o+(0.972386889251)*x[2]
        s[3,0,2,1]=(-0.673716579315)*x[0]**o+(0.497683603572)*x[0]+(0.0894501300871)*x[1]**o+(-0.691905501083)*x[1]+(-0.819373609736)*x[2]**o+(0.876955815163)*x[2]
        s[3,1,0,0]=(-0.612778977625)*x[0]**o+(0.898593519513)*x[0]+(-0.579505051936)*x[1]**o+(0.294087297529)*x[1]+(0.713022674221)*x[2]**o+(-0.0994065928128)*x[2]
        s[3,1,0,1]=(-0.141600251938)*x[0]**o+(-0.677815682935)*x[0]+(-0.408920459122)*x[1]**o+(-0.723029003437)*x[1]+(0.481057659884)*x[2]**o+(0.427909771498)*x[2]
        s[3,1,1,0]=(-0.516486224771)*x[0]**o+(0.526484181999)*x[0]+(0.911793059151)*x[1]**o+(0.409808964766)*x[1]+(-0.413591542253)*x[2]**o+(-0.759664326823)*x[2]
        s[3,1,1,1]=(0.905324377656)*x[0]**o+(-0.208896172985)*x[0]+(-0.422109371498)*x[1]**o+(0.491883832748)*x[1]+(-0.203413002012)*x[2]**o+(0.406737219074)*x[2]
        s[3,1,2,0]=(-0.573692277755)*x[0]**o+(0.895762539503)*x[0]+(0.176538736347)*x[1]**o+(-0.847640766515)*x[1]+(0.5650742337)*x[2]**o+(0.503165369442)*x[2]
        s[3,1,2,1]=(0.308521434074)*x[0]**o+(0.616776036606)*x[0]+(-0.405321909102)*x[1]**o+(0.497367404288)*x[1]+(-0.798636270652)*x[2]**o+(-0.337423592684)*x[2]
        s[3,2,0,0]=(-0.394455434548)*x[0]**o+(-0.292212771133)*x[0]+(-0.941613233015)*x[1]**o+(-0.404522244202)*x[1]+(-0.0112893909573)*x[2]**o+(-0.9706166943)*x[2]
        s[3,2,0,1]=(0.627774279675)*x[0]**o+(-0.645334179444)*x[0]+(0.691389142649)*x[1]**o+(0.763379715146)*x[1]+(0.610766030943)*x[2]**o+(-0.786459853785)*x[2]
        s[3,2,1,0]=(-0.0101413740932)*x[0]**o+(-0.0970748510013)*x[0]+(-0.52181886114)*x[1]**o+(-0.612494396935)*x[1]+(0.941988727244)*x[2]**o+(0.0361447091017)*x[2]
        s[3,2,1,1]=(-0.0943447427863)*x[0]**o+(0.708735490064)*x[0]+(-0.352853510872)*x[1]**o+(-0.545593672805)*x[1]+(-0.0106851492926)*x[2]**o+(0.712976402827)*x[2]
        s[3,2,2,0]=(-0.522341854545)*x[0]**o+(0.320365455548)*x[0]+(-0.111682800076)*x[1]**o+(-0.337351991987)*x[1]+(-0.33503967624)*x[2]**o+(-0.215395174878)*x[2]
        s[3,2,2,1]=(-0.191634883402)*x[0]**o+(-0.79050407204)*x[0]+(-0.881959919487)*x[1]**o+(0.962597211649)*x[1]+(0.602181506788)*x[2]**o+(-0.430152854642)*x[2]
        s[3,3,0,0]=(-0.458551188388)*x[0]**o+(0.62195840207)*x[0]+(0.646839412108)*x[1]**o+(-0.597989589884)*x[1]+(-0.160578706146)*x[2]**o+(0.885101198188)*x[2]
        s[3,3,0,1]=(-0.806015868617)*x[0]**o+(0.0617182982024)*x[0]+(0.296661747576)*x[1]**o+(0.381687206096)*x[1]+(0.954310501938)*x[2]**o+(0.112880684912)*x[2]
        s[3,3,1,0]=(0.485202154902)*x[0]**o+(0.528514725346)*x[0]+(0.550444453321)*x[1]**o+(-0.816313566191)*x[1]+(0.945151247472)*x[2]**o+(-0.201955849739)*x[2]
        s[3,3,1,1]=(0.245157893191)*x[0]**o+(-0.281606458201)*x[0]+(-0.312808561115)*x[1]**o+(0.185786713188)*x[1]+(0.996302657312)*x[2]**o+(-0.227884060488)*x[2]
        s[3,3,2,0]=(0.297297617074)*x[0]**o+(0.156994991329)*x[0]+(-0.403988935033)*x[1]**o+(-0.0894931373916)*x[1]+(-0.979214022227)*x[2]**o+(0.287684161706)*x[2]
        s[3,3,2,1]=(-0.287794860211)*x[0]**o+(0.169671378363)*x[0]+(0.0172804553692)*x[1]**o+(-0.973449618671)*x[1]+(0.0652371831894)*x[2]**o+(0.775978144983)*x[2]
        s[3,4,0,0]=(0.54203905332)*x[0]**o+(0.66731094672)*x[0]+(-0.865207826406)*x[1]**o+(0.466596502355)*x[1]+(-0.45336027189)*x[2]**o+(-0.266404323774)*x[2]
        s[3,4,0,1]=(-0.0264326659602)*x[0]**o+(0.102421850054)*x[0]+(-0.672364919167)*x[1]**o+(-0.461816126462)*x[1]+(0.11436848926)*x[2]**o+(-0.78486118431)*x[2]
        s[3,4,1,0]=(-0.941616359303)*x[0]**o+(-0.0519153474588)*x[0]+(0.8597678151)*x[1]**o+(0.164278414756)*x[1]+(0.496859863828)*x[2]**o+(-0.413785115519)*x[2]
        s[3,4,1,1]=(-0.726802133704)*x[0]**o+(0.685989768699)*x[0]+(0.499728992214)*x[1]**o+(0.675799006108)*x[1]+(0.205370350943)*x[2]**o+(0.983979039059)*x[2]
        s[3,4,2,0]=(-0.121723702011)*x[0]**o+(0.350456030225)*x[0]+(0.211968006265)*x[1]**o+(-0.960265509868)*x[1]+(-0.593090834374)*x[2]**o+(-0.631506211212)*x[2]
        s[3,4,2,1]=(0.279791655852)*x[0]**o+(0.236927673275)*x[0]+(0.84188823482)*x[1]**o+(0.208410505206)*x[1]+(0.392354982825)*x[2]**o+(0.671826571516)*x[2]
        ref[0,0,0,0]=(-0.419262547354)*x_ref[0]**o+(-0.607792358178)*x_ref[0]+(0.539129711536)*x_ref[1]**o+(-0.954002022975)*x_ref[1]+(0.52016184412)*x_ref[2]**o+(-0.905599414589)*x_ref[2]
        ref[0,0,0,1]=(0.552249219683)*x_ref[0]**o+(0.0747535682119)*x_ref[0]+(-0.0550858389381)*x_ref[1]**o+(-0.700756838491)*x_ref[1]+(-0.62569620016)*x_ref[2]**o+(-0.557287205831)*x_ref[2]
        ref[0,0,1,0]=(-0.84674266979)*x_ref[0]**o+(-0.266089976589)*x_ref[0]+(-0.994821395425)*x_ref[1]**o+(-0.567614298529)*x_ref[1]+(-0.814756268883)*x_ref[2]**o+(-0.524622381582)*x_ref[2]
        ref[0,0,1,1]=(0.160195104873)*x_ref[0]**o+(0.161596910068)*x_ref[0]+(-0.0404024431157)*x_ref[1]**o+(0.299140544701)*x_ref[1]+(0.479120924271)*x_ref[2]**o+(0.375793387197)*x_ref[2]
        ref[0,0,2,0]=(0.619011883068)*x_ref[0]**o+(0.453196321806)*x_ref[0]+(0.596546848845)*x_ref[1]**o+(-0.148655559308)*x_ref[1]+(-0.276643963879)*x_ref[2]**o+(-0.556233199969)*x_ref[2]
        ref[0,0,2,1]=(-0.44936519278)*x_ref[0]**o+(0.279187353573)*x_ref[0]+(-0.626699833624)*x_ref[1]**o+(0.172543451997)*x_ref[1]+(0.324942859052)*x_ref[2]**o+(0.227082355579)*x_ref[2]
        ref[0,1,0,0]=(0.348983572344)*x_ref[0]**o+(-0.692310510135)*x_ref[0]+(0.127542726356)*x_ref[1]**o+(-0.719693998448)*x_ref[1]+(0.190533765375)*x_ref[2]**o+(-0.44411272509)*x_ref[2]
        ref[0,1,0,1]=(0.755193842627)*x_ref[0]**o+(-0.0380801278139)*x_ref[0]+(-0.416973165389)*x_ref[1]**o+(-0.407050428042)*x_ref[1]+(-0.803402606403)*x_ref[2]**o+(0.860761604165)*x_ref[2]
        ref[0,1,1,0]=(0.903125811803)*x_ref[0]**o+(-0.370434864257)*x_ref[0]+(-0.810880082994)*x_ref[1]**o+(0.214166691855)*x_ref[1]+(-0.62621693665)*x_ref[2]**o+(-0.043360368073)*x_ref[2]
        ref[0,1,1,1]=(-0.442516790862)*x_ref[0]**o+(0.912139701807)*x_ref[0]+(0.871611995743)*x_ref[1]**o+(0.0291580448949)*x_ref[1]+(-0.489427959855)*x_ref[2]**o+(-0.950170294127)*x_ref[2]
        ref[0,1,2,0]=(-0.0867645917569)*x_ref[0]**o+(0.556462347924)*x_ref[0]+(-0.816554443299)*x_ref[1]**o+(0.818773437796)*x_ref[1]+(0.073905470924)*x_ref[2]**o+(-0.714497094507)*x_ref[2]
        ref[0,1,2,1]=(0.699491100428)*x_ref[0]**o+(-0.200275804269)*x_ref[0]+(0.994316913102)*x_ref[1]**o+(0.832890328003)*x_ref[1]+(0.385381865669)*x_ref[2]**o+(0.153056606745)*x_ref[2]
        ref[0,2,0,0]=(0.612319496073)*x_ref[0]**o+(-0.301991428044)*x_ref[0]+(-0.536866971844)*x_ref[1]**o+(-0.375311119699)*x_ref[1]+(0.733545441876)*x_ref[2]**o+(0.0177187199528)*x_ref[2]
        ref[0,2,0,1]=(0.358991602948)*x_ref[0]**o+(-0.862573091669)*x_ref[0]+(0.464891723814)*x_ref[1]**o+(0.602366461601)*x_ref[1]+(-0.668916509347)*x_ref[2]**o+(0.76048108017)*x_ref[2]
        ref[0,2,1,0]=(0.188195665573)*x_ref[0]**o+(-0.927407688437)*x_ref[0]+(-0.473014301986)*x_ref[1]**o+(-0.775114885201)*x_ref[1]+(-0.453403342886)*x_ref[2]**o+(0.006776981492)*x_ref[2]
        ref[0,2,1,1]=(-0.700181448543)*x_ref[0]**o+(-0.289180138599)*x_ref[0]+(-0.867022883968)*x_ref[1]**o+(-0.359339607516)*x_ref[1]+(-0.809558496152)*x_ref[2]**o+(-0.576358976651)*x_ref[2]
        ref[0,2,2,0]=(-0.557333080207)*x_ref[0]**o+(0.724962853865)*x_ref[0]+(0.995041226791)*x_ref[1]**o+(-0.610460710065)*x_ref[1]+(0.620016428769)*x_ref[2]**o+(-0.449182254452)*x_ref[2]
        ref[0,2,2,1]=(-0.937207840292)*x_ref[0]**o+(-0.66582207891)*x_ref[0]+(0.0951881913165)*x_ref[1]**o+(-0.0770694628972)*x_ref[1]+(0.034224640398)*x_ref[2]**o+(0.652057466224)*x_ref[2]
        ref[0,3,0,0]=(-0.0236750483037)*x_ref[0]**o+(0.956130793675)*x_ref[0]+(-0.589425964387)*x_ref[1]**o+(-0.204770941511)*x_ref[1]+(0.695255730164)*x_ref[2]**o+(-0.796222694557)*x_ref[2]
        ref[0,3,0,1]=(0.138589818728)*x_ref[0]**o+(0.493237559501)*x_ref[0]+(0.139281181588)*x_ref[1]**o+(-0.974201990591)*x_ref[1]+(-0.15401887402)*x_ref[2]**o+(0.23794136041)*x_ref[2]
        ref[0,3,1,0]=(-0.332886758751)*x_ref[0]**o+(-0.231549718295)*x_ref[0]+(0.581536450397)*x_ref[1]**o+(0.790063092494)*x_ref[1]+(0.358814129932)*x_ref[2]**o+(-0.110547525643)*x_ref[2]
        ref[0,3,1,1]=(0.122775146487)*x_ref[0]**o+(-0.00818530079912)*x_ref[0]+(0.0510575666357)*x_ref[1]**o+(-0.96911992662)*x_ref[1]+(-0.820269176326)*x_ref[2]**o+(0.875945222846)*x_ref[2]
        ref[0,3,2,0]=(0.761874596666)*x_ref[0]**o+(0.621369218231)*x_ref[0]+(0.960271412263)*x_ref[1]**o+(-0.808832785376)*x_ref[1]+(0.981428454481)*x_ref[2]**o+(0.393331122254)*x_ref[2]
        ref[0,3,2,1]=(-0.646477042209)*x_ref[0]**o+(-0.26169646294)*x_ref[0]+(0.992265690785)*x_ref[1]**o+(-0.678558332435)*x_ref[1]+(-0.285531525349)*x_ref[2]**o+(-0.571647919822)*x_ref[2]
        ref[0,4,0,0]=(0.492930260015)*x_ref[0]**o+(-0.647489772768)*x_ref[0]+(-0.381225997009)*x_ref[1]**o+(0.560462598779)*x_ref[1]+(-0.63571677055)*x_ref[2]**o+(0.103213895387)*x_ref[2]
        ref[0,4,0,1]=(-0.0362476689653)*x_ref[0]**o+(-0.478992488756)*x_ref[0]+(0.544907192925)*x_ref[1]**o+(-0.678350370776)*x_ref[1]+(-0.0776363475051)*x_ref[2]**o+(-0.803140341684)*x_ref[2]
        ref[0,4,1,0]=(0.0787683344872)*x_ref[0]**o+(0.688990188782)*x_ref[0]+(0.257089245341)*x_ref[1]**o+(-0.648663765213)*x_ref[1]+(-0.140632312728)*x_ref[2]**o+(0.676209090063)*x_ref[2]
        ref[0,4,1,1]=(0.804994807205)*x_ref[0]**o+(0.74750498671)*x_ref[0]+(-0.00532761506025)*x_ref[1]**o+(-0.295700694165)*x_ref[1]+(-0.209175099115)*x_ref[2]**o+(-0.0126407798374)*x_ref[2]
        ref[0,4,2,0]=(-0.789160064467)*x_ref[0]**o+(0.332412023778)*x_ref[0]+(0.242860020473)*x_ref[1]**o+(0.0284658318544)*x_ref[1]+(-0.357559965972)*x_ref[2]**o+(0.57056582293)*x_ref[2]
        ref[0,4,2,1]=(-0.861027741096)*x_ref[0]**o+(0.545166238342)*x_ref[0]+(0.950975873381)*x_ref[1]**o+(-0.436709572856)*x_ref[1]+(0.93636977927)*x_ref[2]**o+(-0.405205657349)*x_ref[2]
        ref[1,0,0,0]=(0.47880791873)*x_ref[0]**o+(0.037486909403)*x_ref[0]+(0.732218957477)*x_ref[1]**o+(0.274811688271)*x_ref[1]+(0.0686323288694)*x_ref[2]**o+(-0.492347744678)*x_ref[2]
        ref[1,0,0,1]=(-0.618259414329)*x_ref[0]**o+(0.630883795613)*x_ref[0]+(0.269141377847)*x_ref[1]**o+(-0.490916952452)*x_ref[1]+(-0.333070678904)*x_ref[2]**o+(0.0484209401349)*x_ref[2]
        ref[1,0,1,0]=(-0.906059152197)*x_ref[0]**o+(0.0322820078282)*x_ref[0]+(-0.47807076381)*x_ref[1]**o+(0.0296380181906)*x_ref[1]+(0.553686905633)*x_ref[2]**o+(-0.408333784569)*x_ref[2]
        ref[1,0,1,1]=(0.282962689091)*x_ref[0]**o+(-0.11495842442)*x_ref[0]+(0.818142653998)*x_ref[1]**o+(0.563460324128)*x_ref[1]+(-0.993782402791)*x_ref[2]**o+(-0.0193690715193)*x_ref[2]
        ref[1,0,2,0]=(0.287962506263)*x_ref[0]**o+(-0.401293374286)*x_ref[0]+(0.173559459587)*x_ref[1]**o+(0.822343918838)*x_ref[1]+(0.888309195417)*x_ref[2]**o+(0.687878298228)*x_ref[2]
        ref[1,0,2,1]=(-0.926775125361)*x_ref[0]**o+(0.396396669159)*x_ref[0]+(0.632962652289)*x_ref[1]**o+(-0.915667885738)*x_ref[1]+(-0.31075960693)*x_ref[2]**o+(0.920135418244)*x_ref[2]
        ref[1,1,0,0]=(-0.0915500113579)*x_ref[0]**o+(0.489036843145)*x_ref[0]+(0.477982330324)*x_ref[1]**o+(0.703371804742)*x_ref[1]+(-0.471773636091)*x_ref[2]**o+(0.0665811350713)*x_ref[2]
        ref[1,1,0,1]=(0.525411480834)*x_ref[0]**o+(-0.369843393433)*x_ref[0]+(-0.7770705966)*x_ref[1]**o+(0.534153802448)*x_ref[1]+(0.309828458492)*x_ref[2]**o+(0.627690678711)*x_ref[2]
        ref[1,1,1,0]=(0.149039257889)*x_ref[0]**o+(-0.0585283560518)*x_ref[0]+(-0.0505893024858)*x_ref[1]**o+(0.677281229531)*x_ref[1]+(-0.451950028252)*x_ref[2]**o+(0.356600216864)*x_ref[2]
        ref[1,1,1,1]=(-0.528237952467)*x_ref[0]**o+(0.533554438129)*x_ref[0]+(0.741981923136)*x_ref[1]**o+(-0.596922185095)*x_ref[1]+(0.50521173977)*x_ref[2]**o+(-0.247100318829)*x_ref[2]
        ref[1,1,2,0]=(0.667977944954)*x_ref[0]**o+(0.306325804541)*x_ref[0]+(-0.654643365837)*x_ref[1]**o+(0.0176380527447)*x_ref[1]+(-0.218012920296)*x_ref[2]**o+(-0.181740681902)*x_ref[2]
        ref[1,1,2,1]=(0.440430981908)*x_ref[0]**o+(0.994608754477)*x_ref[0]+(0.114426402744)*x_ref[1]**o+(-0.896222828258)*x_ref[1]+(0.997911423594)*x_ref[2]**o+(0.648328801403)*x_ref[2]
        ref[1,2,0,0]=(0.397328767103)*x_ref[0]**o+(0.780110953188)*x_ref[0]+(0.453955649367)*x_ref[1]**o+(0.953839185994)*x_ref[1]+(0.995536671917)*x_ref[2]**o+(0.177278490615)*x_ref[2]
        ref[1,2,0,1]=(-0.154393442621)*x_ref[0]**o+(-0.100267841021)*x_ref[0]+(-0.781347482738)*x_ref[1]**o+(0.391441261298)*x_ref[1]+(0.469832428629)*x_ref[2]**o+(0.608531800159)*x_ref[2]
        ref[1,2,1,0]=(-0.534167310466)*x_ref[0]**o+(0.414564904301)*x_ref[0]+(-0.0674568638795)*x_ref[1]**o+(0.577721681963)*x_ref[1]+(-0.00750850180411)*x_ref[2]**o+(0.0817658166565)*x_ref[2]
        ref[1,2,1,1]=(-0.529547188418)*x_ref[0]**o+(0.267917121456)*x_ref[0]+(0.372449729423)*x_ref[1]**o+(0.887072823755)*x_ref[1]+(0.887821539758)*x_ref[2]**o+(0.249771090594)*x_ref[2]
        ref[1,2,2,0]=(0.949056994737)*x_ref[0]**o+(-0.174523839411)*x_ref[0]+(-0.646085737348)*x_ref[1]**o+(0.449358235114)*x_ref[1]+(-0.970748494721)*x_ref[2]**o+(-0.410800591341)*x_ref[2]
        ref[1,2,2,1]=(0.879006069022)*x_ref[0]**o+(-0.510106021113)*x_ref[0]+(0.0203703221469)*x_ref[1]**o+(-0.844338135123)*x_ref[1]+(0.752168319634)*x_ref[2]**o+(0.158760999046)*x_ref[2]
        ref[1,3,0,0]=(0.845321813876)*x_ref[0]**o+(0.405224606418)*x_ref[0]+(-0.89159606951)*x_ref[1]**o+(-0.949499370695)*x_ref[1]+(0.501499536159)*x_ref[2]**o+(-0.869583954204)*x_ref[2]
        ref[1,3,0,1]=(0.0887248259912)*x_ref[0]**o+(0.232925681778)*x_ref[0]+(0.0315176777236)*x_ref[1]**o+(0.00568107770424)*x_ref[1]+(-0.821963787641)*x_ref[2]**o+(-0.634962715535)*x_ref[2]
        ref[1,3,1,0]=(0.546210053059)*x_ref[0]**o+(-0.688171108228)*x_ref[0]+(-0.699621490343)*x_ref[1]**o+(-0.850445270177)*x_ref[1]+(-0.725131419114)*x_ref[2]**o+(0.484336843024)*x_ref[2]
        ref[1,3,1,1]=(0.278594806693)*x_ref[0]**o+(-0.266710093152)*x_ref[0]+(-0.151515190688)*x_ref[1]**o+(-0.705005965054)*x_ref[1]+(0.811008835405)*x_ref[2]**o+(-0.342637057553)*x_ref[2]
        ref[1,3,2,0]=(0.4748294578)*x_ref[0]**o+(-0.0741346092747)*x_ref[0]+(0.148026337115)*x_ref[1]**o+(0.0252686427245)*x_ref[1]+(-0.479120476874)*x_ref[2]**o+(0.0855008343792)*x_ref[2]
        ref[1,3,2,1]=(-0.337007818128)*x_ref[0]**o+(-0.690464368815)*x_ref[0]+(0.481925609432)*x_ref[1]**o+(-0.0981705702921)*x_ref[1]+(0.19218353071)*x_ref[2]**o+(0.988337158745)*x_ref[2]
        ref[1,4,0,0]=(-0.757351506535)*x_ref[0]**o+(-0.824203825425)*x_ref[0]+(-0.998647750787)*x_ref[1]**o+(-0.111654698732)*x_ref[1]+(-0.713287873387)*x_ref[2]**o+(0.779671510732)*x_ref[2]
        ref[1,4,0,1]=(-0.756370998169)*x_ref[0]**o+(0.826863767951)*x_ref[0]+(0.358310449165)*x_ref[1]**o+(0.882430292099)*x_ref[1]+(0.880156909975)*x_ref[2]**o+(0.360411043274)*x_ref[2]
        ref[1,4,1,0]=(0.226080011536)*x_ref[0]**o+(-0.740536947501)*x_ref[0]+(-0.650527457572)*x_ref[1]**o+(-0.890015156497)*x_ref[1]+(0.688973071404)*x_ref[2]**o+(0.29237820526)*x_ref[2]
        ref[1,4,1,1]=(0.574015626188)*x_ref[0]**o+(-0.675641095131)*x_ref[0]+(0.814289365677)*x_ref[1]**o+(-0.625021588943)*x_ref[1]+(-0.862079110641)*x_ref[2]**o+(0.0363949889967)*x_ref[2]
        ref[1,4,2,0]=(0.126659193582)*x_ref[0]**o+(0.16976937315)*x_ref[0]+(0.18822384431)*x_ref[1]**o+(0.546082142359)*x_ref[1]+(0.551365996908)*x_ref[2]**o+(0.395213783677)*x_ref[2]
        ref[1,4,2,1]=(-0.292436267133)*x_ref[0]**o+(-0.442787521929)*x_ref[0]+(-0.310488595304)*x_ref[1]**o+(0.0918858003303)*x_ref[1]+(-0.986695921261)*x_ref[2]**o+(-0.52979957068)*x_ref[2]
        ref[2,0,0,0]=(0.396669972067)*x_ref[0]**o+(-0.711415400722)*x_ref[0]+(-0.717006050735)*x_ref[1]**o+(-0.0935757817421)*x_ref[1]+(0.743477556176)*x_ref[2]**o+(0.823967976857)*x_ref[2]
        ref[2,0,0,1]=(0.488395228667)*x_ref[0]**o+(0.230144035899)*x_ref[0]+(0.125851877066)*x_ref[1]**o+(0.0536284998009)*x_ref[1]+(-0.0114515128334)*x_ref[2]**o+(-0.494039400348)*x_ref[2]
        ref[2,0,1,0]=(0.900615108208)*x_ref[0]**o+(0.432765237128)*x_ref[0]+(0.981213121806)*x_ref[1]**o+(-0.116934702769)*x_ref[1]+(-0.612499086627)*x_ref[2]**o+(-0.365572411348)*x_ref[2]
        ref[2,0,1,1]=(0.0342670705338)*x_ref[0]**o+(0.54954404771)*x_ref[0]+(0.428653302401)*x_ref[1]**o+(0.537787906346)*x_ref[1]+(-0.64287424457)*x_ref[2]**o+(-0.173877644259)*x_ref[2]
        ref[2,0,2,0]=(-0.0510871045727)*x_ref[0]**o+(-0.908007271573)*x_ref[0]+(0.00194326164542)*x_ref[1]**o+(0.466243299714)*x_ref[1]+(-0.433962263339)*x_ref[2]**o+(0.874724229969)*x_ref[2]
        ref[2,0,2,1]=(0.666308090297)*x_ref[0]**o+(0.933188439742)*x_ref[0]+(-0.0479445062129)*x_ref[1]**o+(0.0747806162498)*x_ref[1]+(0.911844507192)*x_ref[2]**o+(-0.934538480024)*x_ref[2]
        ref[2,1,0,0]=(-0.516290157366)*x_ref[0]**o+(0.437519025532)*x_ref[0]+(-0.170999588789)*x_ref[1]**o+(-0.934096554106)*x_ref[1]+(0.88575800375)*x_ref[2]**o+(0.636966058116)*x_ref[2]
        ref[2,1,0,1]=(0.0172669412005)*x_ref[0]**o+(-0.867857912246)*x_ref[0]+(-0.972747832278)*x_ref[1]**o+(-0.780969292213)*x_ref[1]+(-0.418154913379)*x_ref[2]**o+(0.805902597544)*x_ref[2]
        ref[2,1,1,0]=(-0.964006922451)*x_ref[0]**o+(0.236797578253)*x_ref[0]+(-0.78802971076)*x_ref[1]**o+(0.704165381448)*x_ref[1]+(0.00112348607329)*x_ref[2]**o+(-0.689065853536)*x_ref[2]
        ref[2,1,1,1]=(-0.722392335335)*x_ref[0]**o+(-0.120018753634)*x_ref[0]+(0.216541173904)*x_ref[1]**o+(-0.374710223309)*x_ref[1]+(0.532158723682)*x_ref[2]**o+(0.464637663705)*x_ref[2]
        ref[2,1,2,0]=(-0.183637646888)*x_ref[0]**o+(-0.0680674034088)*x_ref[0]+(-0.114129008735)*x_ref[1]**o+(-0.78666547603)*x_ref[1]+(0.0956689943304)*x_ref[2]**o+(0.743296994432)*x_ref[2]
        ref[2,1,2,1]=(0.611382565923)*x_ref[0]**o+(-0.768682259494)*x_ref[0]+(-0.540943249593)*x_ref[1]**o+(0.753523431999)*x_ref[1]+(0.610689642267)*x_ref[2]**o+(0.0122811249496)*x_ref[2]
        ref[2,2,0,0]=(-0.789273374112)*x_ref[0]**o+(0.0977540682694)*x_ref[0]+(-0.829719260296)*x_ref[1]**o+(0.824773509135)*x_ref[1]+(0.216669854484)*x_ref[2]**o+(-0.764947843182)*x_ref[2]
        ref[2,2,0,1]=(-0.340784170043)*x_ref[0]**o+(-0.267437208067)*x_ref[0]+(-0.430973997706)*x_ref[1]**o+(-0.0846177320866)*x_ref[1]+(0.688599546971)*x_ref[2]**o+(0.265513958383)*x_ref[2]
        ref[2,2,1,0]=(-0.237677411545)*x_ref[0]**o+(-0.792144835449)*x_ref[0]+(-0.435763499999)*x_ref[1]**o+(0.682920743804)*x_ref[1]+(0.116459199169)*x_ref[2]**o+(0.254527225097)*x_ref[2]
        ref[2,2,1,1]=(0.342906428734)*x_ref[0]**o+(0.206470310722)*x_ref[0]+(-0.352429884648)*x_ref[1]**o+(-0.168905312607)*x_ref[1]+(-0.105529665095)*x_ref[2]**o+(0.199873788005)*x_ref[2]
        ref[2,2,2,0]=(0.677855511296)*x_ref[0]**o+(0.983953993003)*x_ref[0]+(-0.64997591911)*x_ref[1]**o+(-0.629221981454)*x_ref[1]+(0.990621105566)*x_ref[2]**o+(0.785766133934)*x_ref[2]
        ref[2,2,2,1]=(-0.570482617922)*x_ref[0]**o+(-0.151066051792)*x_ref[0]+(-0.511453364326)*x_ref[1]**o+(0.128824691419)*x_ref[1]+(-0.520127324328)*x_ref[2]**o+(-0.157450052117)*x_ref[2]
        ref[2,3,0,0]=(-0.423827660795)*x_ref[0]**o+(-0.815943363397)*x_ref[0]+(-0.636070023986)*x_ref[1]**o+(0.251327239067)*x_ref[1]+(-0.401641739847)*x_ref[2]**o+(0.130280903755)*x_ref[2]
        ref[2,3,0,1]=(0.0411439121625)*x_ref[0]**o+(0.621609981629)*x_ref[0]+(-0.589059932668)*x_ref[1]**o+(0.543095349334)*x_ref[1]+(-0.0122421869863)*x_ref[2]**o+(-0.776924587359)*x_ref[2]
        ref[2,3,1,0]=(0.63252327363)*x_ref[0]**o+(0.275597518383)*x_ref[0]+(0.832198526441)*x_ref[1]**o+(-0.92855918951)*x_ref[1]+(0.490343890624)*x_ref[2]**o+(-0.412580022625)*x_ref[2]
        ref[2,3,1,1]=(-0.560907436774)*x_ref[0]**o+(0.413956414518)*x_ref[0]+(-0.849736975879)*x_ref[1]**o+(0.167861053833)*x_ref[1]+(-0.316526371498)*x_ref[2]**o+(-0.431245755915)*x_ref[2]
        ref[2,3,2,0]=(0.883757549647)*x_ref[0]**o+(-0.614133317417)*x_ref[0]+(0.923442235162)*x_ref[1]**o+(0.483998522495)*x_ref[1]+(0.649003328717)*x_ref[2]**o+(-0.320181801832)*x_ref[2]
        ref[2,3,2,1]=(0.181982086686)*x_ref[0]**o+(-0.88761330228)*x_ref[0]+(-0.752108545831)*x_ref[1]**o+(0.105681418232)*x_ref[1]+(0.211053107745)*x_ref[2]**o+(-0.840650090772)*x_ref[2]
        ref[2,4,0,0]=(0.774235220873)*x_ref[0]**o+(-0.394430371883)*x_ref[0]+(-0.82551531411)*x_ref[1]**o+(-0.397374538567)*x_ref[1]+(0.51757884795)*x_ref[2]**o+(-0.878919341817)*x_ref[2]
        ref[2,4,0,1]=(-0.282445913776)*x_ref[0]**o+(-0.252614209563)*x_ref[0]+(-0.66810962709)*x_ref[1]**o+(0.912660025157)*x_ref[1]+(0.786200789061)*x_ref[2]**o+(-0.271329907892)*x_ref[2]
        ref[2,4,1,0]=(-0.415061697718)*x_ref[0]**o+(-0.577077815899)*x_ref[0]+(-0.479115469184)*x_ref[1]**o+(0.285424287206)*x_ref[1]+(0.580737671113)*x_ref[2]**o+(0.345716282261)*x_ref[2]
        ref[2,4,1,1]=(0.103116885191)*x_ref[0]**o+(0.472934825259)*x_ref[0]+(0.192747945705)*x_ref[1]**o+(0.106949097334)*x_ref[1]+(-0.109457290729)*x_ref[2]**o+(0.843345964596)*x_ref[2]
        ref[2,4,2,0]=(0.600281530422)*x_ref[0]**o+(-0.427230650838)*x_ref[0]+(-0.868150408946)*x_ref[1]**o+(-0.203647711262)*x_ref[1]+(-0.550477804035)*x_ref[2]**o+(0.415736214172)*x_ref[2]
        ref[2,4,2,1]=(0.739232217202)*x_ref[0]**o+(-0.476864848171)*x_ref[0]+(0.367627911492)*x_ref[1]**o+(0.105176582927)*x_ref[1]+(0.482113806667)*x_ref[2]**o+(0.782020005885)*x_ref[2]
        ref[3,0,0,0]=(0.938839141593)*x_ref[0]**o+(-0.655793939395)*x_ref[0]+(0.0224780413611)*x_ref[1]**o+(0.252535317383)*x_ref[1]+(-0.887022478757)*x_ref[2]**o+(-0.451912746232)*x_ref[2]
        ref[3,0,0,1]=(-0.196994797427)*x_ref[0]**o+(-0.222816920678)*x_ref[0]+(0.37131460044)*x_ref[1]**o+(0.124872956754)*x_ref[1]+(-0.872680499884)*x_ref[2]**o+(-0.490521736465)*x_ref[2]
        ref[3,0,1,0]=(-0.977712672434)*x_ref[0]**o+(-0.665877989129)*x_ref[0]+(-0.541103000349)*x_ref[1]**o+(-0.527523471522)*x_ref[1]+(0.469063433285)*x_ref[2]**o+(-0.410680406273)*x_ref[2]
        ref[3,0,1,1]=(0.371612180735)*x_ref[0]**o+(-0.882397978651)*x_ref[0]+(-0.333119569273)*x_ref[1]**o+(0.0439599202688)*x_ref[1]+(-0.378698960758)*x_ref[2]**o+(0.373830212832)*x_ref[2]
        ref[3,0,2,0]=(0.13552136805)*x_ref[0]**o+(-0.347572305079)*x_ref[0]+(0.194173745453)*x_ref[1]**o+(-0.536374783086)*x_ref[1]+(-0.0928402883882)*x_ref[2]**o+(0.972386889251)*x_ref[2]
        ref[3,0,2,1]=(-0.673716579315)*x_ref[0]**o+(0.497683603572)*x_ref[0]+(0.0894501300871)*x_ref[1]**o+(-0.691905501083)*x_ref[1]+(-0.819373609736)*x_ref[2]**o+(0.876955815163)*x_ref[2]
        ref[3,1,0,0]=(-0.612778977625)*x_ref[0]**o+(0.898593519513)*x_ref[0]+(-0.579505051936)*x_ref[1]**o+(0.294087297529)*x_ref[1]+(0.713022674221)*x_ref[2]**o+(-0.0994065928128)*x_ref[2]
        ref[3,1,0,1]=(-0.141600251938)*x_ref[0]**o+(-0.677815682935)*x_ref[0]+(-0.408920459122)*x_ref[1]**o+(-0.723029003437)*x_ref[1]+(0.481057659884)*x_ref[2]**o+(0.427909771498)*x_ref[2]
        ref[3,1,1,0]=(-0.516486224771)*x_ref[0]**o+(0.526484181999)*x_ref[0]+(0.911793059151)*x_ref[1]**o+(0.409808964766)*x_ref[1]+(-0.413591542253)*x_ref[2]**o+(-0.759664326823)*x_ref[2]
        ref[3,1,1,1]=(0.905324377656)*x_ref[0]**o+(-0.208896172985)*x_ref[0]+(-0.422109371498)*x_ref[1]**o+(0.491883832748)*x_ref[1]+(-0.203413002012)*x_ref[2]**o+(0.406737219074)*x_ref[2]
        ref[3,1,2,0]=(-0.573692277755)*x_ref[0]**o+(0.895762539503)*x_ref[0]+(0.176538736347)*x_ref[1]**o+(-0.847640766515)*x_ref[1]+(0.5650742337)*x_ref[2]**o+(0.503165369442)*x_ref[2]
        ref[3,1,2,1]=(0.308521434074)*x_ref[0]**o+(0.616776036606)*x_ref[0]+(-0.405321909102)*x_ref[1]**o+(0.497367404288)*x_ref[1]+(-0.798636270652)*x_ref[2]**o+(-0.337423592684)*x_ref[2]
        ref[3,2,0,0]=(-0.394455434548)*x_ref[0]**o+(-0.292212771133)*x_ref[0]+(-0.941613233015)*x_ref[1]**o+(-0.404522244202)*x_ref[1]+(-0.0112893909573)*x_ref[2]**o+(-0.9706166943)*x_ref[2]
        ref[3,2,0,1]=(0.627774279675)*x_ref[0]**o+(-0.645334179444)*x_ref[0]+(0.691389142649)*x_ref[1]**o+(0.763379715146)*x_ref[1]+(0.610766030943)*x_ref[2]**o+(-0.786459853785)*x_ref[2]
        ref[3,2,1,0]=(-0.0101413740932)*x_ref[0]**o+(-0.0970748510013)*x_ref[0]+(-0.52181886114)*x_ref[1]**o+(-0.612494396935)*x_ref[1]+(0.941988727244)*x_ref[2]**o+(0.0361447091017)*x_ref[2]
        ref[3,2,1,1]=(-0.0943447427863)*x_ref[0]**o+(0.708735490064)*x_ref[0]+(-0.352853510872)*x_ref[1]**o+(-0.545593672805)*x_ref[1]+(-0.0106851492926)*x_ref[2]**o+(0.712976402827)*x_ref[2]
        ref[3,2,2,0]=(-0.522341854545)*x_ref[0]**o+(0.320365455548)*x_ref[0]+(-0.111682800076)*x_ref[1]**o+(-0.337351991987)*x_ref[1]+(-0.33503967624)*x_ref[2]**o+(-0.215395174878)*x_ref[2]
        ref[3,2,2,1]=(-0.191634883402)*x_ref[0]**o+(-0.79050407204)*x_ref[0]+(-0.881959919487)*x_ref[1]**o+(0.962597211649)*x_ref[1]+(0.602181506788)*x_ref[2]**o+(-0.430152854642)*x_ref[2]
        ref[3,3,0,0]=(-0.458551188388)*x_ref[0]**o+(0.62195840207)*x_ref[0]+(0.646839412108)*x_ref[1]**o+(-0.597989589884)*x_ref[1]+(-0.160578706146)*x_ref[2]**o+(0.885101198188)*x_ref[2]
        ref[3,3,0,1]=(-0.806015868617)*x_ref[0]**o+(0.0617182982024)*x_ref[0]+(0.296661747576)*x_ref[1]**o+(0.381687206096)*x_ref[1]+(0.954310501938)*x_ref[2]**o+(0.112880684912)*x_ref[2]
        ref[3,3,1,0]=(0.485202154902)*x_ref[0]**o+(0.528514725346)*x_ref[0]+(0.550444453321)*x_ref[1]**o+(-0.816313566191)*x_ref[1]+(0.945151247472)*x_ref[2]**o+(-0.201955849739)*x_ref[2]
        ref[3,3,1,1]=(0.245157893191)*x_ref[0]**o+(-0.281606458201)*x_ref[0]+(-0.312808561115)*x_ref[1]**o+(0.185786713188)*x_ref[1]+(0.996302657312)*x_ref[2]**o+(-0.227884060488)*x_ref[2]
        ref[3,3,2,0]=(0.297297617074)*x_ref[0]**o+(0.156994991329)*x_ref[0]+(-0.403988935033)*x_ref[1]**o+(-0.0894931373916)*x_ref[1]+(-0.979214022227)*x_ref[2]**o+(0.287684161706)*x_ref[2]
        ref[3,3,2,1]=(-0.287794860211)*x_ref[0]**o+(0.169671378363)*x_ref[0]+(0.0172804553692)*x_ref[1]**o+(-0.973449618671)*x_ref[1]+(0.0652371831894)*x_ref[2]**o+(0.775978144983)*x_ref[2]
        ref[3,4,0,0]=(0.54203905332)*x_ref[0]**o+(0.66731094672)*x_ref[0]+(-0.865207826406)*x_ref[1]**o+(0.466596502355)*x_ref[1]+(-0.45336027189)*x_ref[2]**o+(-0.266404323774)*x_ref[2]
        ref[3,4,0,1]=(-0.0264326659602)*x_ref[0]**o+(0.102421850054)*x_ref[0]+(-0.672364919167)*x_ref[1]**o+(-0.461816126462)*x_ref[1]+(0.11436848926)*x_ref[2]**o+(-0.78486118431)*x_ref[2]
        ref[3,4,1,0]=(-0.941616359303)*x_ref[0]**o+(-0.0519153474588)*x_ref[0]+(0.8597678151)*x_ref[1]**o+(0.164278414756)*x_ref[1]+(0.496859863828)*x_ref[2]**o+(-0.413785115519)*x_ref[2]
        ref[3,4,1,1]=(-0.726802133704)*x_ref[0]**o+(0.685989768699)*x_ref[0]+(0.499728992214)*x_ref[1]**o+(0.675799006108)*x_ref[1]+(0.205370350943)*x_ref[2]**o+(0.983979039059)*x_ref[2]
        ref[3,4,2,0]=(-0.121723702011)*x_ref[0]**o+(0.350456030225)*x_ref[0]+(0.211968006265)*x_ref[1]**o+(-0.960265509868)*x_ref[1]+(-0.593090834374)*x_ref[2]**o+(-0.631506211212)*x_ref[2]
        ref[3,4,2,1]=(0.279791655852)*x_ref[0]**o+(0.236927673275)*x_ref[0]+(0.84188823482)*x_ref[1]**o+(0.208410505206)*x_ref[1]+(0.392354982825)*x_ref[2]**o+(0.671826571516)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_ContinuousFunction_rank0(self):
      """
      tests interpolate for rank 0 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ContinuousFunction onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(),w)
      ref=Data(0,(),w_ref)
      if dim==2:
        arg=(0.492060381705)*x[0]**o+(0.578791868713)*x[0]+(-0.439849757114)*x[1]**o+(-0.52629328967)*x[1]
        ref=(0.492060381705)*x_ref[0]**o+(0.578791868713)*x_ref[0]+(-0.439849757114)*x_ref[1]**o+(-0.52629328967)*x_ref[1]
      else:
        arg=(0.954737963273)*x[0]**o+(0.452578946878)*x[0]+(0.0471085060698)*x[1]**o+(-0.640169119832)*x[1]+(0.710401490627)*x[2]**o+(-0.382589884739)*x[2]
        ref=(0.954737963273)*x_ref[0]**o+(0.452578946878)*x_ref[0]+(0.0471085060698)*x_ref[1]**o+(-0.640169119832)*x_ref[1]+(0.710401490627)*x_ref[2]**o+(-0.382589884739)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_ContinuousFunction_rank1(self):
      """
      tests interpolate for rank 1 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ContinuousFunction onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(2,),w)
      ref=Data(0,(2,),w_ref)
      if dim==2:
        arg[0]=(-0.59604585527)*x[0]**o+(0.826981169401)*x[0]+(0.227617349484)*x[1]**o+(0.184907819029)*x[1]
        arg[1]=(-0.424684036731)*x[0]**o+(0.346537997905)*x[0]+(0.891921272597)*x[1]**o+(-0.145924182872)*x[1]
        ref[0]=(-0.59604585527)*x_ref[0]**o+(0.826981169401)*x_ref[0]+(0.227617349484)*x_ref[1]**o+(0.184907819029)*x_ref[1]
        ref[1]=(-0.424684036731)*x_ref[0]**o+(0.346537997905)*x_ref[0]+(0.891921272597)*x_ref[1]**o+(-0.145924182872)*x_ref[1]
      else:
        arg[0]=(0.777228173978)*x[0]**o+(-0.812215827155)*x[0]+(-0.618922680968)*x[1]**o+(0.0660312103206)*x[1]+(-0.917545504466)*x[2]**o+(0.988524619414)*x[2]
        arg[1]=(0.110219469565)*x[0]**o+(-0.441866866062)*x[0]+(-0.654447764335)*x[1]**o+(0.230716133289)*x[1]+(0.347487317165)*x[2]**o+(-0.948877903043)*x[2]
        ref[0]=(0.777228173978)*x_ref[0]**o+(-0.812215827155)*x_ref[0]+(-0.618922680968)*x_ref[1]**o+(0.0660312103206)*x_ref[1]+(-0.917545504466)*x_ref[2]**o+(0.988524619414)*x_ref[2]
        ref[1]=(0.110219469565)*x_ref[0]**o+(-0.441866866062)*x_ref[0]+(-0.654447764335)*x_ref[1]**o+(0.230716133289)*x_ref[1]+(0.347487317165)*x_ref[2]**o+(-0.948877903043)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_ContinuousFunction_rank2(self):
      """
      tests interpolate for rank 2 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ContinuousFunction onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5),w)
      ref=Data(0,(4, 5),w_ref)
      if dim==2:
        arg[0,0]=(-0.958809754913)*x[0]**o+(-0.844717049041)*x[0]+(0.462564514427)*x[1]**o+(-0.247678335594)*x[1]
        arg[0,1]=(0.700700054189)*x[0]**o+(-0.653724611457)*x[0]+(0.52659177803)*x[1]**o+(-0.387559446418)*x[1]
        arg[0,2]=(-0.326124300134)*x[0]**o+(0.115197897152)*x[0]+(-0.876340641846)*x[1]**o+(0.433060472581)*x[1]
        arg[0,3]=(-0.101215736407)*x[0]**o+(-0.483684510661)*x[0]+(0.0418770907036)*x[1]**o+(-0.845564414421)*x[1]
        arg[0,4]=(-0.293094855718)*x[0]**o+(0.710954930773)*x[0]+(0.238671713869)*x[1]**o+(0.576638343412)*x[1]
        arg[1,0]=(0.529861280122)*x[0]**o+(-0.259437904048)*x[0]+(-0.620725288158)*x[1]**o+(0.326787999382)*x[1]
        arg[1,1]=(0.421245543012)*x[0]**o+(-0.954139636698)*x[0]+(0.0498199734926)*x[1]**o+(0.24010238499)*x[1]
        arg[1,2]=(0.333362362373)*x[0]**o+(-0.999953175952)*x[0]+(0.866193849246)*x[1]**o+(0.764287410177)*x[1]
        arg[1,3]=(0.470415957539)*x[0]**o+(-0.133477966871)*x[0]+(-0.0416970809832)*x[1]**o+(-0.416352740444)*x[1]
        arg[1,4]=(-0.178613018492)*x[0]**o+(-0.967264985978)*x[0]+(-0.297102423687)*x[1]**o+(0.884184973413)*x[1]
        arg[2,0]=(-0.29363453294)*x[0]**o+(-0.0237360594862)*x[0]+(-0.300776450747)*x[1]**o+(-0.493932721521)*x[1]
        arg[2,1]=(0.795045745699)*x[0]**o+(0.984956851154)*x[0]+(-0.724208405235)*x[1]**o+(0.59029067239)*x[1]
        arg[2,2]=(-0.803012112531)*x[0]**o+(-0.831727785428)*x[0]+(0.330295112332)*x[1]**o+(0.0966317908147)*x[1]
        arg[2,3]=(-0.652332429144)*x[0]**o+(-0.38689457016)*x[0]+(-0.391780825295)*x[1]**o+(-0.444284790848)*x[1]
        arg[2,4]=(0.416775161398)*x[0]**o+(-0.16281006536)*x[0]+(-0.742938932867)*x[1]**o+(0.456486183714)*x[1]
        arg[3,0]=(0.506731632086)*x[0]**o+(-0.222468320035)*x[0]+(-0.906398223556)*x[1]**o+(-0.0250630448611)*x[1]
        arg[3,1]=(-0.39207398783)*x[0]**o+(-0.574739374252)*x[0]+(0.262101584569)*x[1]**o+(0.249587535087)*x[1]
        arg[3,2]=(-0.252593913503)*x[0]**o+(-0.864294052989)*x[0]+(0.0110434296336)*x[1]**o+(-0.224585040999)*x[1]
        arg[3,3]=(-0.631704469212)*x[0]**o+(-0.495153785048)*x[0]+(-0.304420746672)*x[1]**o+(-0.254005222021)*x[1]
        arg[3,4]=(0.775631055421)*x[0]**o+(0.370095409961)*x[0]+(0.894627612994)*x[1]**o+(-0.329716708629)*x[1]
        ref[0,0]=(-0.958809754913)*x_ref[0]**o+(-0.844717049041)*x_ref[0]+(0.462564514427)*x_ref[1]**o+(-0.247678335594)*x_ref[1]
        ref[0,1]=(0.700700054189)*x_ref[0]**o+(-0.653724611457)*x_ref[0]+(0.52659177803)*x_ref[1]**o+(-0.387559446418)*x_ref[1]
        ref[0,2]=(-0.326124300134)*x_ref[0]**o+(0.115197897152)*x_ref[0]+(-0.876340641846)*x_ref[1]**o+(0.433060472581)*x_ref[1]
        ref[0,3]=(-0.101215736407)*x_ref[0]**o+(-0.483684510661)*x_ref[0]+(0.0418770907036)*x_ref[1]**o+(-0.845564414421)*x_ref[1]
        ref[0,4]=(-0.293094855718)*x_ref[0]**o+(0.710954930773)*x_ref[0]+(0.238671713869)*x_ref[1]**o+(0.576638343412)*x_ref[1]
        ref[1,0]=(0.529861280122)*x_ref[0]**o+(-0.259437904048)*x_ref[0]+(-0.620725288158)*x_ref[1]**o+(0.326787999382)*x_ref[1]
        ref[1,1]=(0.421245543012)*x_ref[0]**o+(-0.954139636698)*x_ref[0]+(0.0498199734926)*x_ref[1]**o+(0.24010238499)*x_ref[1]
        ref[1,2]=(0.333362362373)*x_ref[0]**o+(-0.999953175952)*x_ref[0]+(0.866193849246)*x_ref[1]**o+(0.764287410177)*x_ref[1]
        ref[1,3]=(0.470415957539)*x_ref[0]**o+(-0.133477966871)*x_ref[0]+(-0.0416970809832)*x_ref[1]**o+(-0.416352740444)*x_ref[1]
        ref[1,4]=(-0.178613018492)*x_ref[0]**o+(-0.967264985978)*x_ref[0]+(-0.297102423687)*x_ref[1]**o+(0.884184973413)*x_ref[1]
        ref[2,0]=(-0.29363453294)*x_ref[0]**o+(-0.0237360594862)*x_ref[0]+(-0.300776450747)*x_ref[1]**o+(-0.493932721521)*x_ref[1]
        ref[2,1]=(0.795045745699)*x_ref[0]**o+(0.984956851154)*x_ref[0]+(-0.724208405235)*x_ref[1]**o+(0.59029067239)*x_ref[1]
        ref[2,2]=(-0.803012112531)*x_ref[0]**o+(-0.831727785428)*x_ref[0]+(0.330295112332)*x_ref[1]**o+(0.0966317908147)*x_ref[1]
        ref[2,3]=(-0.652332429144)*x_ref[0]**o+(-0.38689457016)*x_ref[0]+(-0.391780825295)*x_ref[1]**o+(-0.444284790848)*x_ref[1]
        ref[2,4]=(0.416775161398)*x_ref[0]**o+(-0.16281006536)*x_ref[0]+(-0.742938932867)*x_ref[1]**o+(0.456486183714)*x_ref[1]
        ref[3,0]=(0.506731632086)*x_ref[0]**o+(-0.222468320035)*x_ref[0]+(-0.906398223556)*x_ref[1]**o+(-0.0250630448611)*x_ref[1]
        ref[3,1]=(-0.39207398783)*x_ref[0]**o+(-0.574739374252)*x_ref[0]+(0.262101584569)*x_ref[1]**o+(0.249587535087)*x_ref[1]
        ref[3,2]=(-0.252593913503)*x_ref[0]**o+(-0.864294052989)*x_ref[0]+(0.0110434296336)*x_ref[1]**o+(-0.224585040999)*x_ref[1]
        ref[3,3]=(-0.631704469212)*x_ref[0]**o+(-0.495153785048)*x_ref[0]+(-0.304420746672)*x_ref[1]**o+(-0.254005222021)*x_ref[1]
        ref[3,4]=(0.775631055421)*x_ref[0]**o+(0.370095409961)*x_ref[0]+(0.894627612994)*x_ref[1]**o+(-0.329716708629)*x_ref[1]
      else:
        arg[0,0]=(0.392450639899)*x[0]**o+(-0.685488230002)*x[0]+(-0.41545928428)*x[1]**o+(0.244144392379)*x[1]+(0.602642156528)*x[2]**o+(-0.348717378461)*x[2]
        arg[0,1]=(-0.341243165541)*x[0]**o+(-0.616870164204)*x[0]+(-0.510613066362)*x[1]**o+(-0.81387010212)*x[1]+(0.856239691119)*x[2]**o+(0.0744651854239)*x[2]
        arg[0,2]=(-0.765983413604)*x[0]**o+(0.474551394174)*x[0]+(-0.939524929765)*x[1]**o+(0.418297898065)*x[1]+(0.00589909539531)*x[2]**o+(0.0889012994897)*x[2]
        arg[0,3]=(-0.521217563352)*x[0]**o+(-0.256316459647)*x[0]+(-0.0265965098379)*x[1]**o+(-0.597562278966)*x[1]+(0.0389151661339)*x[2]**o+(-0.572109451352)*x[2]
        arg[0,4]=(-0.853801268491)*x[0]**o+(-0.691274812057)*x[0]+(-0.682435618581)*x[1]**o+(-0.974334375359)*x[1]+(-0.642333316794)*x[2]**o+(0.353816751252)*x[2]
        arg[1,0]=(-0.657545394322)*x[0]**o+(-0.334172338096)*x[0]+(0.016748279459)*x[1]**o+(-0.294858621125)*x[1]+(-0.49403880046)*x[2]**o+(-0.152416774822)*x[2]
        arg[1,1]=(-0.602731976042)*x[0]**o+(0.685046816545)*x[0]+(0.750001644716)*x[1]**o+(-0.202076972289)*x[1]+(-0.0232888078434)*x[2]**o+(-0.0774477649878)*x[2]
        arg[1,2]=(-0.707321785527)*x[0]**o+(0.618526464205)*x[0]+(-0.578548442468)*x[1]**o+(-0.411374744442)*x[1]+(-0.944277708787)*x[2]**o+(0.647856975707)*x[2]
        arg[1,3]=(-0.222503271022)*x[0]**o+(-0.929428092728)*x[0]+(-0.976553212424)*x[1]**o+(0.0376439617084)*x[1]+(0.0428078568512)*x[2]**o+(-0.441850944272)*x[2]
        arg[1,4]=(-0.862164898455)*x[0]**o+(-0.989900221991)*x[0]+(0.110610844567)*x[1]**o+(0.419384612236)*x[1]+(-0.00271948841262)*x[2]**o+(0.301635101629)*x[2]
        arg[2,0]=(-0.0950421881942)*x[0]**o+(-0.741293053909)*x[0]+(-0.461538879874)*x[1]**o+(-0.8199842553)*x[1]+(0.0590344916829)*x[2]**o+(0.352541518434)*x[2]
        arg[2,1]=(-0.734407995647)*x[0]**o+(-0.726167721992)*x[0]+(0.700521229916)*x[1]**o+(0.753555863428)*x[1]+(-0.891638139696)*x[2]**o+(-0.996550711213)*x[2]
        arg[2,2]=(0.777110352629)*x[0]**o+(0.309719201157)*x[0]+(0.026816547522)*x[1]**o+(0.33161010287)*x[1]+(0.257929504352)*x[2]**o+(0.71871314654)*x[2]
        arg[2,3]=(-0.0982522143825)*x[0]**o+(0.0532144836415)*x[0]+(0.155895442381)*x[1]**o+(-0.872222127033)*x[1]+(0.810538165596)*x[2]**o+(-0.917071363009)*x[2]
        arg[2,4]=(0.882674783877)*x[0]**o+(-0.246439463648)*x[0]+(-0.478821457401)*x[1]**o+(-0.629183697296)*x[1]+(0.833283185393)*x[2]**o+(-0.895796651184)*x[2]
        arg[3,0]=(0.949019461395)*x[0]**o+(0.437511668107)*x[0]+(0.46679355983)*x[1]**o+(0.375391913775)*x[1]+(0.992336073925)*x[2]**o+(0.784847856608)*x[2]
        arg[3,1]=(0.10730061928)*x[0]**o+(0.624678931808)*x[0]+(-0.105084917663)*x[1]**o+(-0.0650034223387)*x[1]+(-0.271922353868)*x[2]**o+(-0.182967296038)*x[2]
        arg[3,2]=(0.115856234055)*x[0]**o+(0.711912354922)*x[0]+(-0.317962972122)*x[1]**o+(0.577856092951)*x[1]+(-0.246833922867)*x[2]**o+(0.354116462218)*x[2]
        arg[3,3]=(0.40937236699)*x[0]**o+(0.639252679403)*x[0]+(0.540206943219)*x[1]**o+(0.287006139758)*x[1]+(0.123311433598)*x[2]**o+(-0.0300672780213)*x[2]
        arg[3,4]=(-0.190299257713)*x[0]**o+(-0.022055675261)*x[0]+(-0.640751994447)*x[1]**o+(-0.411801947317)*x[1]+(0.0675008125057)*x[2]**o+(-0.659424255203)*x[2]
        ref[0,0]=(0.392450639899)*x_ref[0]**o+(-0.685488230002)*x_ref[0]+(-0.41545928428)*x_ref[1]**o+(0.244144392379)*x_ref[1]+(0.602642156528)*x_ref[2]**o+(-0.348717378461)*x_ref[2]
        ref[0,1]=(-0.341243165541)*x_ref[0]**o+(-0.616870164204)*x_ref[0]+(-0.510613066362)*x_ref[1]**o+(-0.81387010212)*x_ref[1]+(0.856239691119)*x_ref[2]**o+(0.0744651854239)*x_ref[2]
        ref[0,2]=(-0.765983413604)*x_ref[0]**o+(0.474551394174)*x_ref[0]+(-0.939524929765)*x_ref[1]**o+(0.418297898065)*x_ref[1]+(0.00589909539531)*x_ref[2]**o+(0.0889012994897)*x_ref[2]
        ref[0,3]=(-0.521217563352)*x_ref[0]**o+(-0.256316459647)*x_ref[0]+(-0.0265965098379)*x_ref[1]**o+(-0.597562278966)*x_ref[1]+(0.0389151661339)*x_ref[2]**o+(-0.572109451352)*x_ref[2]
        ref[0,4]=(-0.853801268491)*x_ref[0]**o+(-0.691274812057)*x_ref[0]+(-0.682435618581)*x_ref[1]**o+(-0.974334375359)*x_ref[1]+(-0.642333316794)*x_ref[2]**o+(0.353816751252)*x_ref[2]
        ref[1,0]=(-0.657545394322)*x_ref[0]**o+(-0.334172338096)*x_ref[0]+(0.016748279459)*x_ref[1]**o+(-0.294858621125)*x_ref[1]+(-0.49403880046)*x_ref[2]**o+(-0.152416774822)*x_ref[2]
        ref[1,1]=(-0.602731976042)*x_ref[0]**o+(0.685046816545)*x_ref[0]+(0.750001644716)*x_ref[1]**o+(-0.202076972289)*x_ref[1]+(-0.0232888078434)*x_ref[2]**o+(-0.0774477649878)*x_ref[2]
        ref[1,2]=(-0.707321785527)*x_ref[0]**o+(0.618526464205)*x_ref[0]+(-0.578548442468)*x_ref[1]**o+(-0.411374744442)*x_ref[1]+(-0.944277708787)*x_ref[2]**o+(0.647856975707)*x_ref[2]
        ref[1,3]=(-0.222503271022)*x_ref[0]**o+(-0.929428092728)*x_ref[0]+(-0.976553212424)*x_ref[1]**o+(0.0376439617084)*x_ref[1]+(0.0428078568512)*x_ref[2]**o+(-0.441850944272)*x_ref[2]
        ref[1,4]=(-0.862164898455)*x_ref[0]**o+(-0.989900221991)*x_ref[0]+(0.110610844567)*x_ref[1]**o+(0.419384612236)*x_ref[1]+(-0.00271948841262)*x_ref[2]**o+(0.301635101629)*x_ref[2]
        ref[2,0]=(-0.0950421881942)*x_ref[0]**o+(-0.741293053909)*x_ref[0]+(-0.461538879874)*x_ref[1]**o+(-0.8199842553)*x_ref[1]+(0.0590344916829)*x_ref[2]**o+(0.352541518434)*x_ref[2]
        ref[2,1]=(-0.734407995647)*x_ref[0]**o+(-0.726167721992)*x_ref[0]+(0.700521229916)*x_ref[1]**o+(0.753555863428)*x_ref[1]+(-0.891638139696)*x_ref[2]**o+(-0.996550711213)*x_ref[2]
        ref[2,2]=(0.777110352629)*x_ref[0]**o+(0.309719201157)*x_ref[0]+(0.026816547522)*x_ref[1]**o+(0.33161010287)*x_ref[1]+(0.257929504352)*x_ref[2]**o+(0.71871314654)*x_ref[2]
        ref[2,3]=(-0.0982522143825)*x_ref[0]**o+(0.0532144836415)*x_ref[0]+(0.155895442381)*x_ref[1]**o+(-0.872222127033)*x_ref[1]+(0.810538165596)*x_ref[2]**o+(-0.917071363009)*x_ref[2]
        ref[2,4]=(0.882674783877)*x_ref[0]**o+(-0.246439463648)*x_ref[0]+(-0.478821457401)*x_ref[1]**o+(-0.629183697296)*x_ref[1]+(0.833283185393)*x_ref[2]**o+(-0.895796651184)*x_ref[2]
        ref[3,0]=(0.949019461395)*x_ref[0]**o+(0.437511668107)*x_ref[0]+(0.46679355983)*x_ref[1]**o+(0.375391913775)*x_ref[1]+(0.992336073925)*x_ref[2]**o+(0.784847856608)*x_ref[2]
        ref[3,1]=(0.10730061928)*x_ref[0]**o+(0.624678931808)*x_ref[0]+(-0.105084917663)*x_ref[1]**o+(-0.0650034223387)*x_ref[1]+(-0.271922353868)*x_ref[2]**o+(-0.182967296038)*x_ref[2]
        ref[3,2]=(0.115856234055)*x_ref[0]**o+(0.711912354922)*x_ref[0]+(-0.317962972122)*x_ref[1]**o+(0.577856092951)*x_ref[1]+(-0.246833922867)*x_ref[2]**o+(0.354116462218)*x_ref[2]
        ref[3,3]=(0.40937236699)*x_ref[0]**o+(0.639252679403)*x_ref[0]+(0.540206943219)*x_ref[1]**o+(0.287006139758)*x_ref[1]+(0.123311433598)*x_ref[2]**o+(-0.0300672780213)*x_ref[2]
        ref[3,4]=(-0.190299257713)*x_ref[0]**o+(-0.022055675261)*x_ref[0]+(-0.640751994447)*x_ref[1]**o+(-0.411801947317)*x_ref[1]+(0.0675008125057)*x_ref[2]**o+(-0.659424255203)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_ContinuousFunction_rank3(self):
      """
      tests interpolate for rank 3 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ContinuousFunction onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2),w_ref)
      if dim==2:
        arg[0,0,0]=(-0.697323110242)*x[0]**o+(-0.806494185218)*x[0]+(0.387914602484)*x[1]**o+(0.841835487965)*x[1]
        arg[0,0,1]=(-0.77670653908)*x[0]**o+(0.311710212706)*x[0]+(-0.0951864020181)*x[1]**o+(-0.14909139946)*x[1]
        arg[0,1,0]=(0.80933791543)*x[0]**o+(0.72441968276)*x[0]+(-0.446054850085)*x[1]**o+(0.419303534988)*x[1]
        arg[0,1,1]=(0.174464190017)*x[0]**o+(0.330609661878)*x[0]+(0.798365390058)*x[1]**o+(0.354899223817)*x[1]
        arg[1,0,0]=(0.36073672839)*x[0]**o+(0.831900846214)*x[0]+(-0.656354645044)*x[1]**o+(0.200645044646)*x[1]
        arg[1,0,1]=(0.817889801065)*x[0]**o+(0.34052889294)*x[0]+(-0.2416944924)*x[1]**o+(0.0922342148934)*x[1]
        arg[1,1,0]=(-0.962116509558)*x[0]**o+(-0.0785232077542)*x[0]+(-0.981399563356)*x[1]**o+(0.785509095614)*x[1]
        arg[1,1,1]=(-0.688444030508)*x[0]**o+(0.651617638972)*x[0]+(0.160584046401)*x[1]**o+(0.253258991923)*x[1]
        arg[2,0,0]=(0.0728970412574)*x[0]**o+(-0.647328240217)*x[0]+(0.517665494535)*x[1]**o+(-0.416383753573)*x[1]
        arg[2,0,1]=(0.148104739733)*x[0]**o+(-0.378371639837)*x[0]+(0.848608631336)*x[1]**o+(-0.881516167852)*x[1]
        arg[2,1,0]=(-0.506376676256)*x[0]**o+(0.234090340446)*x[0]+(-0.737828959566)*x[1]**o+(-0.74093763135)*x[1]
        arg[2,1,1]=(0.259605354135)*x[0]**o+(0.537424356513)*x[0]+(-0.405718119693)*x[1]**o+(0.120663712044)*x[1]
        arg[3,0,0]=(-0.609659975269)*x[0]**o+(-0.455923293874)*x[0]+(-0.00765676965971)*x[1]**o+(-0.710899349259)*x[1]
        arg[3,0,1]=(-0.731011612904)*x[0]**o+(-0.355060765358)*x[0]+(-0.127369372538)*x[1]**o+(0.694265371908)*x[1]
        arg[3,1,0]=(0.188766337132)*x[0]**o+(-0.194878304215)*x[0]+(-0.963356218109)*x[1]**o+(0.327053372293)*x[1]
        arg[3,1,1]=(-0.566712779324)*x[0]**o+(-0.251473305071)*x[0]+(0.0181297146138)*x[1]**o+(0.257664697754)*x[1]
        arg[4,0,0]=(0.574100764569)*x[0]**o+(-0.717776999574)*x[0]+(0.880998124155)*x[1]**o+(-0.810615928848)*x[1]
        arg[4,0,1]=(0.42552502518)*x[0]**o+(0.456751341147)*x[0]+(-0.484998619005)*x[1]**o+(0.932151187696)*x[1]
        arg[4,1,0]=(0.56865980485)*x[0]**o+(0.593928872734)*x[0]+(-0.475639894836)*x[1]**o+(-0.990718402693)*x[1]
        arg[4,1,1]=(0.129586342261)*x[0]**o+(-0.408951830619)*x[0]+(0.132518475289)*x[1]**o+(0.708612820692)*x[1]
        arg[5,0,0]=(0.0708038739082)*x[0]**o+(0.524270558649)*x[0]+(0.368504533482)*x[1]**o+(0.935828810393)*x[1]
        arg[5,0,1]=(-0.383650306829)*x[0]**o+(-0.0439362943961)*x[0]+(0.80939275198)*x[1]**o+(-0.61335508023)*x[1]
        arg[5,1,0]=(-0.621571339247)*x[0]**o+(-0.185069293199)*x[0]+(0.368472011431)*x[1]**o+(-0.388752495932)*x[1]
        arg[5,1,1]=(-0.263310763174)*x[0]**o+(-0.694616630015)*x[0]+(0.271238274203)*x[1]**o+(-0.477861402347)*x[1]
        ref[0,0,0]=(-0.697323110242)*x_ref[0]**o+(-0.806494185218)*x_ref[0]+(0.387914602484)*x_ref[1]**o+(0.841835487965)*x_ref[1]
        ref[0,0,1]=(-0.77670653908)*x_ref[0]**o+(0.311710212706)*x_ref[0]+(-0.0951864020181)*x_ref[1]**o+(-0.14909139946)*x_ref[1]
        ref[0,1,0]=(0.80933791543)*x_ref[0]**o+(0.72441968276)*x_ref[0]+(-0.446054850085)*x_ref[1]**o+(0.419303534988)*x_ref[1]
        ref[0,1,1]=(0.174464190017)*x_ref[0]**o+(0.330609661878)*x_ref[0]+(0.798365390058)*x_ref[1]**o+(0.354899223817)*x_ref[1]
        ref[1,0,0]=(0.36073672839)*x_ref[0]**o+(0.831900846214)*x_ref[0]+(-0.656354645044)*x_ref[1]**o+(0.200645044646)*x_ref[1]
        ref[1,0,1]=(0.817889801065)*x_ref[0]**o+(0.34052889294)*x_ref[0]+(-0.2416944924)*x_ref[1]**o+(0.0922342148934)*x_ref[1]
        ref[1,1,0]=(-0.962116509558)*x_ref[0]**o+(-0.0785232077542)*x_ref[0]+(-0.981399563356)*x_ref[1]**o+(0.785509095614)*x_ref[1]
        ref[1,1,1]=(-0.688444030508)*x_ref[0]**o+(0.651617638972)*x_ref[0]+(0.160584046401)*x_ref[1]**o+(0.253258991923)*x_ref[1]
        ref[2,0,0]=(0.0728970412574)*x_ref[0]**o+(-0.647328240217)*x_ref[0]+(0.517665494535)*x_ref[1]**o+(-0.416383753573)*x_ref[1]
        ref[2,0,1]=(0.148104739733)*x_ref[0]**o+(-0.378371639837)*x_ref[0]+(0.848608631336)*x_ref[1]**o+(-0.881516167852)*x_ref[1]
        ref[2,1,0]=(-0.506376676256)*x_ref[0]**o+(0.234090340446)*x_ref[0]+(-0.737828959566)*x_ref[1]**o+(-0.74093763135)*x_ref[1]
        ref[2,1,1]=(0.259605354135)*x_ref[0]**o+(0.537424356513)*x_ref[0]+(-0.405718119693)*x_ref[1]**o+(0.120663712044)*x_ref[1]
        ref[3,0,0]=(-0.609659975269)*x_ref[0]**o+(-0.455923293874)*x_ref[0]+(-0.00765676965971)*x_ref[1]**o+(-0.710899349259)*x_ref[1]
        ref[3,0,1]=(-0.731011612904)*x_ref[0]**o+(-0.355060765358)*x_ref[0]+(-0.127369372538)*x_ref[1]**o+(0.694265371908)*x_ref[1]
        ref[3,1,0]=(0.188766337132)*x_ref[0]**o+(-0.194878304215)*x_ref[0]+(-0.963356218109)*x_ref[1]**o+(0.327053372293)*x_ref[1]
        ref[3,1,1]=(-0.566712779324)*x_ref[0]**o+(-0.251473305071)*x_ref[0]+(0.0181297146138)*x_ref[1]**o+(0.257664697754)*x_ref[1]
        ref[4,0,0]=(0.574100764569)*x_ref[0]**o+(-0.717776999574)*x_ref[0]+(0.880998124155)*x_ref[1]**o+(-0.810615928848)*x_ref[1]
        ref[4,0,1]=(0.42552502518)*x_ref[0]**o+(0.456751341147)*x_ref[0]+(-0.484998619005)*x_ref[1]**o+(0.932151187696)*x_ref[1]
        ref[4,1,0]=(0.56865980485)*x_ref[0]**o+(0.593928872734)*x_ref[0]+(-0.475639894836)*x_ref[1]**o+(-0.990718402693)*x_ref[1]
        ref[4,1,1]=(0.129586342261)*x_ref[0]**o+(-0.408951830619)*x_ref[0]+(0.132518475289)*x_ref[1]**o+(0.708612820692)*x_ref[1]
        ref[5,0,0]=(0.0708038739082)*x_ref[0]**o+(0.524270558649)*x_ref[0]+(0.368504533482)*x_ref[1]**o+(0.935828810393)*x_ref[1]
        ref[5,0,1]=(-0.383650306829)*x_ref[0]**o+(-0.0439362943961)*x_ref[0]+(0.80939275198)*x_ref[1]**o+(-0.61335508023)*x_ref[1]
        ref[5,1,0]=(-0.621571339247)*x_ref[0]**o+(-0.185069293199)*x_ref[0]+(0.368472011431)*x_ref[1]**o+(-0.388752495932)*x_ref[1]
        ref[5,1,1]=(-0.263310763174)*x_ref[0]**o+(-0.694616630015)*x_ref[0]+(0.271238274203)*x_ref[1]**o+(-0.477861402347)*x_ref[1]
      else:
        arg[0,0,0]=(0.287534645778)*x[0]**o+(0.94865229379)*x[0]+(0.655299232864)*x[1]**o+(0.350034924637)*x[1]+(0.556341391183)*x[2]**o+(0.840643265765)*x[2]
        arg[0,0,1]=(-0.207213781549)*x[0]**o+(-0.0394497047797)*x[0]+(-0.0236016308575)*x[1]**o+(0.344122299199)*x[1]+(0.389588325996)*x[2]**o+(0.941980468657)*x[2]
        arg[0,1,0]=(0.00980412235862)*x[0]**o+(0.74862549447)*x[0]+(0.630165284383)*x[1]**o+(0.234711268989)*x[1]+(-0.22509981272)*x[2]**o+(-0.0153888168649)*x[2]
        arg[0,1,1]=(-0.87309186262)*x[0]**o+(0.833526923917)*x[0]+(0.78785479415)*x[1]**o+(0.823967259171)*x[1]+(0.320244316109)*x[2]**o+(-0.735225854417)*x[2]
        arg[1,0,0]=(-0.0892232551051)*x[0]**o+(-0.346728833563)*x[0]+(0.422774634671)*x[1]**o+(0.793222381957)*x[1]+(-0.670476677006)*x[2]**o+(0.896884540116)*x[2]
        arg[1,0,1]=(-0.0023382136345)*x[0]**o+(0.73058281545)*x[0]+(0.11828880374)*x[1]**o+(-0.691839091619)*x[1]+(-0.39086463392)*x[2]**o+(0.66200209782)*x[2]
        arg[1,1,0]=(-0.924238055558)*x[0]**o+(-0.334208271705)*x[0]+(-0.57106045555)*x[1]**o+(-0.765650999193)*x[1]+(0.0404533468054)*x[2]**o+(-0.225975260777)*x[2]
        arg[1,1,1]=(-0.351110999108)*x[0]**o+(0.626166346251)*x[0]+(0.385737198467)*x[1]**o+(0.577561466908)*x[1]+(0.708325217242)*x[2]**o+(0.193727923904)*x[2]
        arg[2,0,0]=(0.278942080997)*x[0]**o+(-0.569761653572)*x[0]+(-0.624879270918)*x[1]**o+(-0.0151754112896)*x[1]+(0.411399083825)*x[2]**o+(-0.846669568641)*x[2]
        arg[2,0,1]=(-0.543586963879)*x[0]**o+(-0.383959956625)*x[0]+(0.184375944938)*x[1]**o+(-0.104178233562)*x[1]+(-0.223101894662)*x[2]**o+(-0.17221835595)*x[2]
        arg[2,1,0]=(0.317103921207)*x[0]**o+(-0.919542461354)*x[0]+(0.56666554454)*x[1]**o+(0.895772298847)*x[1]+(0.714550707822)*x[2]**o+(0.634702089556)*x[2]
        arg[2,1,1]=(0.986576254164)*x[0]**o+(0.544493787633)*x[0]+(0.964620027203)*x[1]**o+(0.259991384678)*x[1]+(-0.577361835739)*x[2]**o+(0.651593456822)*x[2]
        arg[3,0,0]=(0.362841472614)*x[0]**o+(-0.521771228041)*x[0]+(-0.594530755517)*x[1]**o+(0.700647681241)*x[1]+(-0.131683781311)*x[2]**o+(0.636310189834)*x[2]
        arg[3,0,1]=(0.885891862321)*x[0]**o+(0.397438320764)*x[0]+(0.711380452937)*x[1]**o+(0.962571325144)*x[1]+(-0.300114113339)*x[2]**o+(0.34700388868)*x[2]
        arg[3,1,0]=(0.52140726408)*x[0]**o+(-0.0921295295522)*x[0]+(-0.823076091446)*x[1]**o+(0.477401876125)*x[1]+(-0.973057905067)*x[2]**o+(-0.975526794128)*x[2]
        arg[3,1,1]=(-0.241503887113)*x[0]**o+(0.133988088285)*x[0]+(0.243026393121)*x[1]**o+(0.0848716779473)*x[1]+(0.140349614791)*x[2]**o+(-0.217182025695)*x[2]
        arg[4,0,0]=(0.680080194053)*x[0]**o+(0.512009633844)*x[0]+(-0.640214066363)*x[1]**o+(0.571800633214)*x[1]+(-0.749395617545)*x[2]**o+(0.448487668786)*x[2]
        arg[4,0,1]=(-0.792959916349)*x[0]**o+(-0.433770516776)*x[0]+(-0.380980758215)*x[1]**o+(-0.831154769101)*x[1]+(0.277551460651)*x[2]**o+(0.626476984233)*x[2]
        arg[4,1,0]=(-0.285102566546)*x[0]**o+(0.745636275253)*x[0]+(0.4999427836)*x[1]**o+(0.561343302884)*x[1]+(0.796220302361)*x[2]**o+(-0.830020520143)*x[2]
        arg[4,1,1]=(0.237420507314)*x[0]**o+(0.327181858092)*x[0]+(-0.895743235593)*x[1]**o+(-0.496964477443)*x[1]+(0.57336428204)*x[2]**o+(-0.154267071287)*x[2]
        arg[5,0,0]=(0.0401925997607)*x[0]**o+(-0.0155957424748)*x[0]+(-0.658363451239)*x[1]**o+(-0.175223211879)*x[1]+(-0.743729768327)*x[2]**o+(0.391531327128)*x[2]
        arg[5,0,1]=(0.836239801824)*x[0]**o+(-0.72288085675)*x[0]+(0.177981042887)*x[1]**o+(0.354505260098)*x[1]+(-0.180520504267)*x[2]**o+(-0.380306239974)*x[2]
        arg[5,1,0]=(-0.65536853024)*x[0]**o+(-0.00573854242476)*x[0]+(-0.261460514314)*x[1]**o+(-0.289771405693)*x[1]+(0.750775397523)*x[2]**o+(-0.333215608562)*x[2]
        arg[5,1,1]=(-0.579532884024)*x[0]**o+(-0.208677373724)*x[0]+(-0.379612024024)*x[1]**o+(-0.0565813995114)*x[1]+(-0.867913562592)*x[2]**o+(-0.254654298293)*x[2]
        ref[0,0,0]=(0.287534645778)*x_ref[0]**o+(0.94865229379)*x_ref[0]+(0.655299232864)*x_ref[1]**o+(0.350034924637)*x_ref[1]+(0.556341391183)*x_ref[2]**o+(0.840643265765)*x_ref[2]
        ref[0,0,1]=(-0.207213781549)*x_ref[0]**o+(-0.0394497047797)*x_ref[0]+(-0.0236016308575)*x_ref[1]**o+(0.344122299199)*x_ref[1]+(0.389588325996)*x_ref[2]**o+(0.941980468657)*x_ref[2]
        ref[0,1,0]=(0.00980412235862)*x_ref[0]**o+(0.74862549447)*x_ref[0]+(0.630165284383)*x_ref[1]**o+(0.234711268989)*x_ref[1]+(-0.22509981272)*x_ref[2]**o+(-0.0153888168649)*x_ref[2]
        ref[0,1,1]=(-0.87309186262)*x_ref[0]**o+(0.833526923917)*x_ref[0]+(0.78785479415)*x_ref[1]**o+(0.823967259171)*x_ref[1]+(0.320244316109)*x_ref[2]**o+(-0.735225854417)*x_ref[2]
        ref[1,0,0]=(-0.0892232551051)*x_ref[0]**o+(-0.346728833563)*x_ref[0]+(0.422774634671)*x_ref[1]**o+(0.793222381957)*x_ref[1]+(-0.670476677006)*x_ref[2]**o+(0.896884540116)*x_ref[2]
        ref[1,0,1]=(-0.0023382136345)*x_ref[0]**o+(0.73058281545)*x_ref[0]+(0.11828880374)*x_ref[1]**o+(-0.691839091619)*x_ref[1]+(-0.39086463392)*x_ref[2]**o+(0.66200209782)*x_ref[2]
        ref[1,1,0]=(-0.924238055558)*x_ref[0]**o+(-0.334208271705)*x_ref[0]+(-0.57106045555)*x_ref[1]**o+(-0.765650999193)*x_ref[1]+(0.0404533468054)*x_ref[2]**o+(-0.225975260777)*x_ref[2]
        ref[1,1,1]=(-0.351110999108)*x_ref[0]**o+(0.626166346251)*x_ref[0]+(0.385737198467)*x_ref[1]**o+(0.577561466908)*x_ref[1]+(0.708325217242)*x_ref[2]**o+(0.193727923904)*x_ref[2]
        ref[2,0,0]=(0.278942080997)*x_ref[0]**o+(-0.569761653572)*x_ref[0]+(-0.624879270918)*x_ref[1]**o+(-0.0151754112896)*x_ref[1]+(0.411399083825)*x_ref[2]**o+(-0.846669568641)*x_ref[2]
        ref[2,0,1]=(-0.543586963879)*x_ref[0]**o+(-0.383959956625)*x_ref[0]+(0.184375944938)*x_ref[1]**o+(-0.104178233562)*x_ref[1]+(-0.223101894662)*x_ref[2]**o+(-0.17221835595)*x_ref[2]
        ref[2,1,0]=(0.317103921207)*x_ref[0]**o+(-0.919542461354)*x_ref[0]+(0.56666554454)*x_ref[1]**o+(0.895772298847)*x_ref[1]+(0.714550707822)*x_ref[2]**o+(0.634702089556)*x_ref[2]
        ref[2,1,1]=(0.986576254164)*x_ref[0]**o+(0.544493787633)*x_ref[0]+(0.964620027203)*x_ref[1]**o+(0.259991384678)*x_ref[1]+(-0.577361835739)*x_ref[2]**o+(0.651593456822)*x_ref[2]
        ref[3,0,0]=(0.362841472614)*x_ref[0]**o+(-0.521771228041)*x_ref[0]+(-0.594530755517)*x_ref[1]**o+(0.700647681241)*x_ref[1]+(-0.131683781311)*x_ref[2]**o+(0.636310189834)*x_ref[2]
        ref[3,0,1]=(0.885891862321)*x_ref[0]**o+(0.397438320764)*x_ref[0]+(0.711380452937)*x_ref[1]**o+(0.962571325144)*x_ref[1]+(-0.300114113339)*x_ref[2]**o+(0.34700388868)*x_ref[2]
        ref[3,1,0]=(0.52140726408)*x_ref[0]**o+(-0.0921295295522)*x_ref[0]+(-0.823076091446)*x_ref[1]**o+(0.477401876125)*x_ref[1]+(-0.973057905067)*x_ref[2]**o+(-0.975526794128)*x_ref[2]
        ref[3,1,1]=(-0.241503887113)*x_ref[0]**o+(0.133988088285)*x_ref[0]+(0.243026393121)*x_ref[1]**o+(0.0848716779473)*x_ref[1]+(0.140349614791)*x_ref[2]**o+(-0.217182025695)*x_ref[2]
        ref[4,0,0]=(0.680080194053)*x_ref[0]**o+(0.512009633844)*x_ref[0]+(-0.640214066363)*x_ref[1]**o+(0.571800633214)*x_ref[1]+(-0.749395617545)*x_ref[2]**o+(0.448487668786)*x_ref[2]
        ref[4,0,1]=(-0.792959916349)*x_ref[0]**o+(-0.433770516776)*x_ref[0]+(-0.380980758215)*x_ref[1]**o+(-0.831154769101)*x_ref[1]+(0.277551460651)*x_ref[2]**o+(0.626476984233)*x_ref[2]
        ref[4,1,0]=(-0.285102566546)*x_ref[0]**o+(0.745636275253)*x_ref[0]+(0.4999427836)*x_ref[1]**o+(0.561343302884)*x_ref[1]+(0.796220302361)*x_ref[2]**o+(-0.830020520143)*x_ref[2]
        ref[4,1,1]=(0.237420507314)*x_ref[0]**o+(0.327181858092)*x_ref[0]+(-0.895743235593)*x_ref[1]**o+(-0.496964477443)*x_ref[1]+(0.57336428204)*x_ref[2]**o+(-0.154267071287)*x_ref[2]
        ref[5,0,0]=(0.0401925997607)*x_ref[0]**o+(-0.0155957424748)*x_ref[0]+(-0.658363451239)*x_ref[1]**o+(-0.175223211879)*x_ref[1]+(-0.743729768327)*x_ref[2]**o+(0.391531327128)*x_ref[2]
        ref[5,0,1]=(0.836239801824)*x_ref[0]**o+(-0.72288085675)*x_ref[0]+(0.177981042887)*x_ref[1]**o+(0.354505260098)*x_ref[1]+(-0.180520504267)*x_ref[2]**o+(-0.380306239974)*x_ref[2]
        ref[5,1,0]=(-0.65536853024)*x_ref[0]**o+(-0.00573854242476)*x_ref[0]+(-0.261460514314)*x_ref[1]**o+(-0.289771405693)*x_ref[1]+(0.750775397523)*x_ref[2]**o+(-0.333215608562)*x_ref[2]
        ref[5,1,1]=(-0.579532884024)*x_ref[0]**o+(-0.208677373724)*x_ref[0]+(-0.379612024024)*x_ref[1]**o+(-0.0565813995114)*x_ref[1]+(-0.867913562592)*x_ref[2]**o+(-0.254654298293)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_ContinuousFunction_rank4(self):
      """
      tests interpolate for rank 4 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ContinuousFunction onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5, 3, 2),w)
      ref=Data(0,(4, 5, 3, 2),w_ref)
      if dim==2:
        arg[0,0,0,0]=(-0.274472293956)*x[0]**o+(-0.293347130929)*x[0]+(0.392586448254)*x[1]**o+(-0.403066854054)*x[1]
        arg[0,0,0,1]=(0.385933192907)*x[0]**o+(0.484240065867)*x[0]+(0.144565631551)*x[1]**o+(0.882508492739)*x[1]
        arg[0,0,1,0]=(0.993368905137)*x[0]**o+(0.340349919343)*x[0]+(-0.956806746524)*x[1]**o+(0.990607958001)*x[1]
        arg[0,0,1,1]=(-0.88610250102)*x[0]**o+(-0.543705587259)*x[0]+(0.170798164924)*x[1]**o+(0.685331195754)*x[1]
        arg[0,0,2,0]=(-0.922211611143)*x[0]**o+(-0.103640037893)*x[0]+(0.401354703434)*x[1]**o+(-0.117826261635)*x[1]
        arg[0,0,2,1]=(-0.391601387132)*x[0]**o+(-0.472753344804)*x[0]+(-0.473433838993)*x[1]**o+(-0.173501635989)*x[1]
        arg[0,1,0,0]=(0.747353274916)*x[0]**o+(-0.960856612086)*x[0]+(-0.54809012978)*x[1]**o+(0.862023893332)*x[1]
        arg[0,1,0,1]=(-0.256359001149)*x[0]**o+(-0.868975044191)*x[0]+(0.494314269744)*x[1]**o+(0.949957481172)*x[1]
        arg[0,1,1,0]=(-0.0256658750601)*x[0]**o+(0.145499104203)*x[0]+(-0.68669791629)*x[1]**o+(0.447326207295)*x[1]
        arg[0,1,1,1]=(0.150617284467)*x[0]**o+(-0.350046088196)*x[0]+(-0.367671302053)*x[1]**o+(0.633220122068)*x[1]
        arg[0,1,2,0]=(-0.714145362271)*x[0]**o+(-0.957661490758)*x[0]+(0.386253770747)*x[1]**o+(0.982634262634)*x[1]
        arg[0,1,2,1]=(0.707565096824)*x[0]**o+(0.281758058573)*x[0]+(-0.407791315587)*x[1]**o+(0.86069036983)*x[1]
        arg[0,2,0,0]=(-0.453860220598)*x[0]**o+(0.944457388061)*x[0]+(-0.903818960162)*x[1]**o+(-0.799423205297)*x[1]
        arg[0,2,0,1]=(-0.621257500941)*x[0]**o+(0.748963737937)*x[0]+(-0.12660934458)*x[1]**o+(-0.362830639592)*x[1]
        arg[0,2,1,0]=(-0.702485820037)*x[0]**o+(0.355584898593)*x[0]+(0.657127357751)*x[1]**o+(0.535533390762)*x[1]
        arg[0,2,1,1]=(0.795733125008)*x[0]**o+(0.983607327249)*x[0]+(-0.463818780644)*x[1]**o+(-0.681831588244)*x[1]
        arg[0,2,2,0]=(0.335783204138)*x[0]**o+(-0.712225782679)*x[0]+(0.853426644096)*x[1]**o+(-0.143845263686)*x[1]
        arg[0,2,2,1]=(0.389724375144)*x[0]**o+(0.715176227705)*x[0]+(-0.660690893888)*x[1]**o+(0.428289724709)*x[1]
        arg[0,3,0,0]=(-0.363836203724)*x[0]**o+(-0.257775427457)*x[0]+(0.407904889098)*x[1]**o+(-0.731273814284)*x[1]
        arg[0,3,0,1]=(0.402038262494)*x[0]**o+(0.33289103638)*x[0]+(0.444957562329)*x[1]**o+(0.0657971935664)*x[1]
        arg[0,3,1,0]=(0.979673814602)*x[0]**o+(-0.133405332848)*x[0]+(0.23884689304)*x[1]**o+(0.728464997681)*x[1]
        arg[0,3,1,1]=(0.724603429246)*x[0]**o+(-0.391040898144)*x[0]+(-0.966816499003)*x[1]**o+(0.0984302578905)*x[1]
        arg[0,3,2,0]=(-0.986139645754)*x[0]**o+(0.60807351165)*x[0]+(0.799807413231)*x[1]**o+(0.12091479711)*x[1]
        arg[0,3,2,1]=(0.387453999377)*x[0]**o+(0.212725006628)*x[0]+(-0.120374039139)*x[1]**o+(-0.262304757951)*x[1]
        arg[0,4,0,0]=(-0.260545986276)*x[0]**o+(0.251085312932)*x[0]+(0.738082708518)*x[1]**o+(-0.0804883797938)*x[1]
        arg[0,4,0,1]=(-0.190701356138)*x[0]**o+(-0.0205295935522)*x[0]+(0.904261681819)*x[1]**o+(-0.859647709957)*x[1]
        arg[0,4,1,0]=(-0.945357492849)*x[0]**o+(-0.85298463389)*x[0]+(0.688964336673)*x[1]**o+(-0.672881696932)*x[1]
        arg[0,4,1,1]=(0.877720258045)*x[0]**o+(-0.361228026805)*x[0]+(-0.326626527006)*x[1]**o+(-0.274100421232)*x[1]
        arg[0,4,2,0]=(0.202358729084)*x[0]**o+(-0.997571481837)*x[0]+(0.431193711862)*x[1]**o+(0.513561123915)*x[1]
        arg[0,4,2,1]=(-0.136602954592)*x[0]**o+(-0.40864934437)*x[0]+(0.675945574351)*x[1]**o+(0.728757793536)*x[1]
        arg[1,0,0,0]=(0.835464261851)*x[0]**o+(-0.131447020601)*x[0]+(0.0810352521446)*x[1]**o+(0.664744750899)*x[1]
        arg[1,0,0,1]=(0.239716928689)*x[0]**o+(-0.192258999531)*x[0]+(0.615877054036)*x[1]**o+(0.835274774437)*x[1]
        arg[1,0,1,0]=(0.509342803767)*x[0]**o+(-0.633944481882)*x[0]+(0.262098006086)*x[1]**o+(0.159902070799)*x[1]
        arg[1,0,1,1]=(0.894432440588)*x[0]**o+(-0.167778431319)*x[0]+(-0.984183148477)*x[1]**o+(-0.554981658632)*x[1]
        arg[1,0,2,0]=(-0.762472607231)*x[0]**o+(0.405550139256)*x[0]+(0.4051290641)*x[1]**o+(-0.27180186672)*x[1]
        arg[1,0,2,1]=(0.663869273816)*x[0]**o+(0.867453974628)*x[0]+(-0.279659969132)*x[1]**o+(-0.370539101204)*x[1]
        arg[1,1,0,0]=(-0.969326778164)*x[0]**o+(0.965471349497)*x[0]+(-0.529983232946)*x[1]**o+(0.714091610977)*x[1]
        arg[1,1,0,1]=(-0.840360667093)*x[0]**o+(-0.399319808256)*x[0]+(0.568769915582)*x[1]**o+(0.872427494442)*x[1]
        arg[1,1,1,0]=(0.46099116941)*x[0]**o+(-0.986008874142)*x[0]+(-0.53999023556)*x[1]**o+(-0.549695567855)*x[1]
        arg[1,1,1,1]=(0.485944918559)*x[0]**o+(0.189773828651)*x[0]+(-0.611654980493)*x[1]**o+(-0.337071911657)*x[1]
        arg[1,1,2,0]=(0.191663603248)*x[0]**o+(0.0906374965004)*x[0]+(-0.271051598251)*x[1]**o+(0.880507788825)*x[1]
        arg[1,1,2,1]=(-0.439514997001)*x[0]**o+(0.701230426224)*x[0]+(-0.193231641029)*x[1]**o+(0.950584054332)*x[1]
        arg[1,2,0,0]=(0.439456565259)*x[0]**o+(-0.25664171484)*x[0]+(-0.653059819329)*x[1]**o+(-0.680544747777)*x[1]
        arg[1,2,0,1]=(0.34890454443)*x[0]**o+(-0.69851200963)*x[0]+(-0.00159381688549)*x[1]**o+(-0.612359675499)*x[1]
        arg[1,2,1,0]=(0.132759671375)*x[0]**o+(-0.235126716401)*x[0]+(0.652068885826)*x[1]**o+(0.749698645371)*x[1]
        arg[1,2,1,1]=(0.934980403627)*x[0]**o+(0.321409273437)*x[0]+(-0.605909929553)*x[1]**o+(0.135554927418)*x[1]
        arg[1,2,2,0]=(-0.68327709329)*x[0]**o+(-0.0381292756005)*x[0]+(0.710953191114)*x[1]**o+(-0.344522256689)*x[1]
        arg[1,2,2,1]=(-0.813976235652)*x[0]**o+(0.613879708123)*x[0]+(0.451599277293)*x[1]**o+(-0.6134026375)*x[1]
        arg[1,3,0,0]=(0.595310128627)*x[0]**o+(-0.247621573131)*x[0]+(0.165607783104)*x[1]**o+(0.578132180805)*x[1]
        arg[1,3,0,1]=(0.848827238882)*x[0]**o+(0.173154067999)*x[0]+(0.340662419437)*x[1]**o+(-0.389656447251)*x[1]
        arg[1,3,1,0]=(0.250888415228)*x[0]**o+(-0.278203136198)*x[0]+(-0.693211186734)*x[1]**o+(0.613812194735)*x[1]
        arg[1,3,1,1]=(0.634533003162)*x[0]**o+(-0.0924665762471)*x[0]+(0.749575894384)*x[1]**o+(-0.737205017415)*x[1]
        arg[1,3,2,0]=(0.607750966973)*x[0]**o+(0.905016928404)*x[0]+(-0.171049542163)*x[1]**o+(-0.20526244601)*x[1]
        arg[1,3,2,1]=(0.723946025405)*x[0]**o+(0.0157576743987)*x[0]+(0.190875428312)*x[1]**o+(0.407925899509)*x[1]
        arg[1,4,0,0]=(-0.561582695803)*x[0]**o+(-0.350262461371)*x[0]+(0.57610409872)*x[1]**o+(0.881780963877)*x[1]
        arg[1,4,0,1]=(0.077066498695)*x[0]**o+(0.515597802516)*x[0]+(0.332011413016)*x[1]**o+(0.215491334787)*x[1]
        arg[1,4,1,0]=(0.33858727737)*x[0]**o+(-0.573231800768)*x[0]+(-0.559915809132)*x[1]**o+(0.0240869142176)*x[1]
        arg[1,4,1,1]=(-0.684581727952)*x[0]**o+(0.365541904201)*x[0]+(0.204252397376)*x[1]**o+(0.149300766054)*x[1]
        arg[1,4,2,0]=(-0.574225088383)*x[0]**o+(0.0286339776067)*x[0]+(-0.358732122607)*x[1]**o+(-0.416932738957)*x[1]
        arg[1,4,2,1]=(-0.308460184932)*x[0]**o+(0.389214987836)*x[0]+(-0.358303257031)*x[1]**o+(0.348109483688)*x[1]
        arg[2,0,0,0]=(-0.811318668804)*x[0]**o+(-0.298028116982)*x[0]+(0.679871523483)*x[1]**o+(0.958042925287)*x[1]
        arg[2,0,0,1]=(-0.616521818338)*x[0]**o+(-0.0817124854496)*x[0]+(0.403257366893)*x[1]**o+(0.740689710435)*x[1]
        arg[2,0,1,0]=(0.271046179127)*x[0]**o+(0.814928093947)*x[0]+(0.423722781579)*x[1]**o+(0.925333209811)*x[1]
        arg[2,0,1,1]=(0.0596651806816)*x[0]**o+(0.493854605359)*x[0]+(0.539998537439)*x[1]**o+(-0.910865639375)*x[1]
        arg[2,0,2,0]=(0.503267602327)*x[0]**o+(-0.270966471743)*x[0]+(-0.342063366676)*x[1]**o+(-0.975700224705)*x[1]
        arg[2,0,2,1]=(-0.0362358939195)*x[0]**o+(-0.766033869339)*x[0]+(0.891365100088)*x[1]**o+(0.815025898929)*x[1]
        arg[2,1,0,0]=(-0.667240459984)*x[0]**o+(0.8659076701)*x[0]+(0.215089397562)*x[1]**o+(0.464769182307)*x[1]
        arg[2,1,0,1]=(-0.298625221931)*x[0]**o+(-0.504329743472)*x[0]+(0.412472510208)*x[1]**o+(0.241693152889)*x[1]
        arg[2,1,1,0]=(-0.670730179133)*x[0]**o+(0.0201070113832)*x[0]+(-0.572725898919)*x[1]**o+(0.665609259014)*x[1]
        arg[2,1,1,1]=(-0.319080984089)*x[0]**o+(0.0299423045224)*x[0]+(0.0152311810613)*x[1]**o+(-0.784646189026)*x[1]
        arg[2,1,2,0]=(0.153308789716)*x[0]**o+(0.725808150613)*x[0]+(0.62876493338)*x[1]**o+(0.80275284252)*x[1]
        arg[2,1,2,1]=(0.804698870013)*x[0]**o+(-0.631078773061)*x[0]+(-0.015221975698)*x[1]**o+(-0.954169686746)*x[1]
        arg[2,2,0,0]=(0.169022797498)*x[0]**o+(0.733025791099)*x[0]+(0.321554996633)*x[1]**o+(0.292173365364)*x[1]
        arg[2,2,0,1]=(0.327345955109)*x[0]**o+(0.21757823896)*x[0]+(0.873027160446)*x[1]**o+(-0.250192861579)*x[1]
        arg[2,2,1,0]=(0.394387190832)*x[0]**o+(-0.613661883482)*x[0]+(0.806965291386)*x[1]**o+(0.244513244016)*x[1]
        arg[2,2,1,1]=(0.043913907536)*x[0]**o+(-0.303567806424)*x[0]+(0.547037093323)*x[1]**o+(-0.598687385328)*x[1]
        arg[2,2,2,0]=(0.875663154787)*x[0]**o+(-0.34984832772)*x[0]+(0.879143174396)*x[1]**o+(-0.193154051109)*x[1]
        arg[2,2,2,1]=(0.940311132918)*x[0]**o+(-0.358708743382)*x[0]+(0.117370945146)*x[1]**o+(-0.664208034757)*x[1]
        arg[2,3,0,0]=(0.0334525660588)*x[0]**o+(0.592805915255)*x[0]+(-0.278216157971)*x[1]**o+(0.894369967671)*x[1]
        arg[2,3,0,1]=(-0.966591106326)*x[0]**o+(-0.882160390768)*x[0]+(0.843982702732)*x[1]**o+(0.363868921656)*x[1]
        arg[2,3,1,0]=(-0.246392342771)*x[0]**o+(-0.934459096242)*x[0]+(-0.472771368537)*x[1]**o+(0.148854304706)*x[1]
        arg[2,3,1,1]=(0.248218424051)*x[0]**o+(0.0980396576111)*x[0]+(0.413322166489)*x[1]**o+(0.429747261346)*x[1]
        arg[2,3,2,0]=(0.742987565733)*x[0]**o+(-0.213736450136)*x[0]+(0.996975741314)*x[1]**o+(0.960709958665)*x[1]
        arg[2,3,2,1]=(0.436149490977)*x[0]**o+(0.87632181197)*x[0]+(-0.881934669912)*x[1]**o+(-0.457520504175)*x[1]
        arg[2,4,0,0]=(-0.519014330693)*x[0]**o+(0.938619608407)*x[0]+(-0.847954563609)*x[1]**o+(0.791774379621)*x[1]
        arg[2,4,0,1]=(-0.599487299453)*x[0]**o+(0.977867555132)*x[0]+(0.693105630579)*x[1]**o+(-0.688827812942)*x[1]
        arg[2,4,1,0]=(0.955907964163)*x[0]**o+(0.0724056986172)*x[0]+(0.254773619772)*x[1]**o+(0.0428861146964)*x[1]
        arg[2,4,1,1]=(-0.422202101998)*x[0]**o+(0.803263223544)*x[0]+(0.456048140785)*x[1]**o+(0.0419133126183)*x[1]
        arg[2,4,2,0]=(-0.182308704781)*x[0]**o+(0.862282443488)*x[0]+(-0.844027007955)*x[1]**o+(0.586220526376)*x[1]
        arg[2,4,2,1]=(-0.801542666229)*x[0]**o+(0.903685299455)*x[0]+(0.428926544377)*x[1]**o+(-0.891220471549)*x[1]
        arg[3,0,0,0]=(0.602917787713)*x[0]**o+(-0.592239549743)*x[0]+(0.761830006718)*x[1]**o+(-0.784264167777)*x[1]
        arg[3,0,0,1]=(0.819962957732)*x[0]**o+(-0.455126790372)*x[0]+(-0.144898728525)*x[1]**o+(0.121566345339)*x[1]
        arg[3,0,1,0]=(0.570694941559)*x[0]**o+(-0.505678656386)*x[0]+(0.295536587355)*x[1]**o+(-0.852779737814)*x[1]
        arg[3,0,1,1]=(-0.226246858753)*x[0]**o+(-0.513900306228)*x[0]+(-0.389483591407)*x[1]**o+(-0.735105335539)*x[1]
        arg[3,0,2,0]=(0.941877610938)*x[0]**o+(0.93808330298)*x[0]+(0.189785162182)*x[1]**o+(0.546525366304)*x[1]
        arg[3,0,2,1]=(0.10639100485)*x[0]**o+(-0.773361007972)*x[0]+(-0.470531427579)*x[1]**o+(0.467359342808)*x[1]
        arg[3,1,0,0]=(0.857725599589)*x[0]**o+(0.0881746413132)*x[0]+(0.040713151152)*x[1]**o+(-0.52070729883)*x[1]
        arg[3,1,0,1]=(0.266807652146)*x[0]**o+(0.337496673892)*x[0]+(-0.562266297445)*x[1]**o+(0.326015195113)*x[1]
        arg[3,1,1,0]=(0.177275509465)*x[0]**o+(-0.647855761301)*x[0]+(-0.0963373631995)*x[1]**o+(-0.535256252994)*x[1]
        arg[3,1,1,1]=(0.430309295606)*x[0]**o+(-0.0894749557428)*x[0]+(0.503470724483)*x[1]**o+(-0.202361122248)*x[1]
        arg[3,1,2,0]=(0.283995014194)*x[0]**o+(-0.544419907194)*x[0]+(0.78478571712)*x[1]**o+(0.341540664147)*x[1]
        arg[3,1,2,1]=(-0.281042869936)*x[0]**o+(0.728771766492)*x[0]+(-0.298974388557)*x[1]**o+(-0.836789906985)*x[1]
        arg[3,2,0,0]=(-0.415323371561)*x[0]**o+(0.583971137371)*x[0]+(-0.265658679489)*x[1]**o+(-0.432447373809)*x[1]
        arg[3,2,0,1]=(0.690486298855)*x[0]**o+(0.552521632226)*x[0]+(0.237622739042)*x[1]**o+(-0.254513693924)*x[1]
        arg[3,2,1,0]=(0.453985687207)*x[0]**o+(-0.785385580962)*x[0]+(0.0190987841634)*x[1]**o+(-0.7923288861)*x[1]
        arg[3,2,1,1]=(-0.964082656379)*x[0]**o+(-0.806315937371)*x[0]+(0.271145420366)*x[1]**o+(-0.839014535945)*x[1]
        arg[3,2,2,0]=(-0.372062773997)*x[0]**o+(-0.069749560567)*x[0]+(-0.126310395383)*x[1]**o+(0.972025865285)*x[1]
        arg[3,2,2,1]=(-0.580353020086)*x[0]**o+(-0.152746821477)*x[0]+(0.297553660535)*x[1]**o+(0.444791568145)*x[1]
        arg[3,3,0,0]=(-0.361520029439)*x[0]**o+(-0.810391070499)*x[0]+(0.462123882642)*x[1]**o+(-0.974534227742)*x[1]
        arg[3,3,0,1]=(0.113370617837)*x[0]**o+(0.428257355435)*x[0]+(-0.18656623758)*x[1]**o+(0.206503327394)*x[1]
        arg[3,3,1,0]=(0.341328445689)*x[0]**o+(0.34002947054)*x[0]+(0.997677286362)*x[1]**o+(0.43300073225)*x[1]
        arg[3,3,1,1]=(-0.513912982823)*x[0]**o+(0.826573827064)*x[0]+(-0.701417864085)*x[1]**o+(0.567078883219)*x[1]
        arg[3,3,2,0]=(-0.325536866965)*x[0]**o+(0.924405767451)*x[0]+(-0.0481381954346)*x[1]**o+(0.912210617073)*x[1]
        arg[3,3,2,1]=(0.491532434302)*x[0]**o+(-0.584980848036)*x[0]+(-0.788317530858)*x[1]**o+(-0.900681893172)*x[1]
        arg[3,4,0,0]=(0.432249855148)*x[0]**o+(-0.673369312552)*x[0]+(0.244704845849)*x[1]**o+(-0.300995595367)*x[1]
        arg[3,4,0,1]=(0.383037211984)*x[0]**o+(0.299711400488)*x[0]+(0.870584255454)*x[1]**o+(-0.33571823329)*x[1]
        arg[3,4,1,0]=(0.737134180177)*x[0]**o+(-0.101307090628)*x[0]+(-0.970632061143)*x[1]**o+(-0.449042684137)*x[1]
        arg[3,4,1,1]=(0.00684250643632)*x[0]**o+(0.676964654022)*x[0]+(0.789069834858)*x[1]**o+(-0.829739508197)*x[1]
        arg[3,4,2,0]=(0.7279044944)*x[0]**o+(0.215711122038)*x[0]+(0.385631252632)*x[1]**o+(0.629390433529)*x[1]
        arg[3,4,2,1]=(-0.380750716629)*x[0]**o+(-0.413040692437)*x[0]+(0.26795244639)*x[1]**o+(0.492475504659)*x[1]
        ref[0,0,0,0]=(-0.274472293956)*x_ref[0]**o+(-0.293347130929)*x_ref[0]+(0.392586448254)*x_ref[1]**o+(-0.403066854054)*x_ref[1]
        ref[0,0,0,1]=(0.385933192907)*x_ref[0]**o+(0.484240065867)*x_ref[0]+(0.144565631551)*x_ref[1]**o+(0.882508492739)*x_ref[1]
        ref[0,0,1,0]=(0.993368905137)*x_ref[0]**o+(0.340349919343)*x_ref[0]+(-0.956806746524)*x_ref[1]**o+(0.990607958001)*x_ref[1]
        ref[0,0,1,1]=(-0.88610250102)*x_ref[0]**o+(-0.543705587259)*x_ref[0]+(0.170798164924)*x_ref[1]**o+(0.685331195754)*x_ref[1]
        ref[0,0,2,0]=(-0.922211611143)*x_ref[0]**o+(-0.103640037893)*x_ref[0]+(0.401354703434)*x_ref[1]**o+(-0.117826261635)*x_ref[1]
        ref[0,0,2,1]=(-0.391601387132)*x_ref[0]**o+(-0.472753344804)*x_ref[0]+(-0.473433838993)*x_ref[1]**o+(-0.173501635989)*x_ref[1]
        ref[0,1,0,0]=(0.747353274916)*x_ref[0]**o+(-0.960856612086)*x_ref[0]+(-0.54809012978)*x_ref[1]**o+(0.862023893332)*x_ref[1]
        ref[0,1,0,1]=(-0.256359001149)*x_ref[0]**o+(-0.868975044191)*x_ref[0]+(0.494314269744)*x_ref[1]**o+(0.949957481172)*x_ref[1]
        ref[0,1,1,0]=(-0.0256658750601)*x_ref[0]**o+(0.145499104203)*x_ref[0]+(-0.68669791629)*x_ref[1]**o+(0.447326207295)*x_ref[1]
        ref[0,1,1,1]=(0.150617284467)*x_ref[0]**o+(-0.350046088196)*x_ref[0]+(-0.367671302053)*x_ref[1]**o+(0.633220122068)*x_ref[1]
        ref[0,1,2,0]=(-0.714145362271)*x_ref[0]**o+(-0.957661490758)*x_ref[0]+(0.386253770747)*x_ref[1]**o+(0.982634262634)*x_ref[1]
        ref[0,1,2,1]=(0.707565096824)*x_ref[0]**o+(0.281758058573)*x_ref[0]+(-0.407791315587)*x_ref[1]**o+(0.86069036983)*x_ref[1]
        ref[0,2,0,0]=(-0.453860220598)*x_ref[0]**o+(0.944457388061)*x_ref[0]+(-0.903818960162)*x_ref[1]**o+(-0.799423205297)*x_ref[1]
        ref[0,2,0,1]=(-0.621257500941)*x_ref[0]**o+(0.748963737937)*x_ref[0]+(-0.12660934458)*x_ref[1]**o+(-0.362830639592)*x_ref[1]
        ref[0,2,1,0]=(-0.702485820037)*x_ref[0]**o+(0.355584898593)*x_ref[0]+(0.657127357751)*x_ref[1]**o+(0.535533390762)*x_ref[1]
        ref[0,2,1,1]=(0.795733125008)*x_ref[0]**o+(0.983607327249)*x_ref[0]+(-0.463818780644)*x_ref[1]**o+(-0.681831588244)*x_ref[1]
        ref[0,2,2,0]=(0.335783204138)*x_ref[0]**o+(-0.712225782679)*x_ref[0]+(0.853426644096)*x_ref[1]**o+(-0.143845263686)*x_ref[1]
        ref[0,2,2,1]=(0.389724375144)*x_ref[0]**o+(0.715176227705)*x_ref[0]+(-0.660690893888)*x_ref[1]**o+(0.428289724709)*x_ref[1]
        ref[0,3,0,0]=(-0.363836203724)*x_ref[0]**o+(-0.257775427457)*x_ref[0]+(0.407904889098)*x_ref[1]**o+(-0.731273814284)*x_ref[1]
        ref[0,3,0,1]=(0.402038262494)*x_ref[0]**o+(0.33289103638)*x_ref[0]+(0.444957562329)*x_ref[1]**o+(0.0657971935664)*x_ref[1]
        ref[0,3,1,0]=(0.979673814602)*x_ref[0]**o+(-0.133405332848)*x_ref[0]+(0.23884689304)*x_ref[1]**o+(0.728464997681)*x_ref[1]
        ref[0,3,1,1]=(0.724603429246)*x_ref[0]**o+(-0.391040898144)*x_ref[0]+(-0.966816499003)*x_ref[1]**o+(0.0984302578905)*x_ref[1]
        ref[0,3,2,0]=(-0.986139645754)*x_ref[0]**o+(0.60807351165)*x_ref[0]+(0.799807413231)*x_ref[1]**o+(0.12091479711)*x_ref[1]
        ref[0,3,2,1]=(0.387453999377)*x_ref[0]**o+(0.212725006628)*x_ref[0]+(-0.120374039139)*x_ref[1]**o+(-0.262304757951)*x_ref[1]
        ref[0,4,0,0]=(-0.260545986276)*x_ref[0]**o+(0.251085312932)*x_ref[0]+(0.738082708518)*x_ref[1]**o+(-0.0804883797938)*x_ref[1]
        ref[0,4,0,1]=(-0.190701356138)*x_ref[0]**o+(-0.0205295935522)*x_ref[0]+(0.904261681819)*x_ref[1]**o+(-0.859647709957)*x_ref[1]
        ref[0,4,1,0]=(-0.945357492849)*x_ref[0]**o+(-0.85298463389)*x_ref[0]+(0.688964336673)*x_ref[1]**o+(-0.672881696932)*x_ref[1]
        ref[0,4,1,1]=(0.877720258045)*x_ref[0]**o+(-0.361228026805)*x_ref[0]+(-0.326626527006)*x_ref[1]**o+(-0.274100421232)*x_ref[1]
        ref[0,4,2,0]=(0.202358729084)*x_ref[0]**o+(-0.997571481837)*x_ref[0]+(0.431193711862)*x_ref[1]**o+(0.513561123915)*x_ref[1]
        ref[0,4,2,1]=(-0.136602954592)*x_ref[0]**o+(-0.40864934437)*x_ref[0]+(0.675945574351)*x_ref[1]**o+(0.728757793536)*x_ref[1]
        ref[1,0,0,0]=(0.835464261851)*x_ref[0]**o+(-0.131447020601)*x_ref[0]+(0.0810352521446)*x_ref[1]**o+(0.664744750899)*x_ref[1]
        ref[1,0,0,1]=(0.239716928689)*x_ref[0]**o+(-0.192258999531)*x_ref[0]+(0.615877054036)*x_ref[1]**o+(0.835274774437)*x_ref[1]
        ref[1,0,1,0]=(0.509342803767)*x_ref[0]**o+(-0.633944481882)*x_ref[0]+(0.262098006086)*x_ref[1]**o+(0.159902070799)*x_ref[1]
        ref[1,0,1,1]=(0.894432440588)*x_ref[0]**o+(-0.167778431319)*x_ref[0]+(-0.984183148477)*x_ref[1]**o+(-0.554981658632)*x_ref[1]
        ref[1,0,2,0]=(-0.762472607231)*x_ref[0]**o+(0.405550139256)*x_ref[0]+(0.4051290641)*x_ref[1]**o+(-0.27180186672)*x_ref[1]
        ref[1,0,2,1]=(0.663869273816)*x_ref[0]**o+(0.867453974628)*x_ref[0]+(-0.279659969132)*x_ref[1]**o+(-0.370539101204)*x_ref[1]
        ref[1,1,0,0]=(-0.969326778164)*x_ref[0]**o+(0.965471349497)*x_ref[0]+(-0.529983232946)*x_ref[1]**o+(0.714091610977)*x_ref[1]
        ref[1,1,0,1]=(-0.840360667093)*x_ref[0]**o+(-0.399319808256)*x_ref[0]+(0.568769915582)*x_ref[1]**o+(0.872427494442)*x_ref[1]
        ref[1,1,1,0]=(0.46099116941)*x_ref[0]**o+(-0.986008874142)*x_ref[0]+(-0.53999023556)*x_ref[1]**o+(-0.549695567855)*x_ref[1]
        ref[1,1,1,1]=(0.485944918559)*x_ref[0]**o+(0.189773828651)*x_ref[0]+(-0.611654980493)*x_ref[1]**o+(-0.337071911657)*x_ref[1]
        ref[1,1,2,0]=(0.191663603248)*x_ref[0]**o+(0.0906374965004)*x_ref[0]+(-0.271051598251)*x_ref[1]**o+(0.880507788825)*x_ref[1]
        ref[1,1,2,1]=(-0.439514997001)*x_ref[0]**o+(0.701230426224)*x_ref[0]+(-0.193231641029)*x_ref[1]**o+(0.950584054332)*x_ref[1]
        ref[1,2,0,0]=(0.439456565259)*x_ref[0]**o+(-0.25664171484)*x_ref[0]+(-0.653059819329)*x_ref[1]**o+(-0.680544747777)*x_ref[1]
        ref[1,2,0,1]=(0.34890454443)*x_ref[0]**o+(-0.69851200963)*x_ref[0]+(-0.00159381688549)*x_ref[1]**o+(-0.612359675499)*x_ref[1]
        ref[1,2,1,0]=(0.132759671375)*x_ref[0]**o+(-0.235126716401)*x_ref[0]+(0.652068885826)*x_ref[1]**o+(0.749698645371)*x_ref[1]
        ref[1,2,1,1]=(0.934980403627)*x_ref[0]**o+(0.321409273437)*x_ref[0]+(-0.605909929553)*x_ref[1]**o+(0.135554927418)*x_ref[1]
        ref[1,2,2,0]=(-0.68327709329)*x_ref[0]**o+(-0.0381292756005)*x_ref[0]+(0.710953191114)*x_ref[1]**o+(-0.344522256689)*x_ref[1]
        ref[1,2,2,1]=(-0.813976235652)*x_ref[0]**o+(0.613879708123)*x_ref[0]+(0.451599277293)*x_ref[1]**o+(-0.6134026375)*x_ref[1]
        ref[1,3,0,0]=(0.595310128627)*x_ref[0]**o+(-0.247621573131)*x_ref[0]+(0.165607783104)*x_ref[1]**o+(0.578132180805)*x_ref[1]
        ref[1,3,0,1]=(0.848827238882)*x_ref[0]**o+(0.173154067999)*x_ref[0]+(0.340662419437)*x_ref[1]**o+(-0.389656447251)*x_ref[1]
        ref[1,3,1,0]=(0.250888415228)*x_ref[0]**o+(-0.278203136198)*x_ref[0]+(-0.693211186734)*x_ref[1]**o+(0.613812194735)*x_ref[1]
        ref[1,3,1,1]=(0.634533003162)*x_ref[0]**o+(-0.0924665762471)*x_ref[0]+(0.749575894384)*x_ref[1]**o+(-0.737205017415)*x_ref[1]
        ref[1,3,2,0]=(0.607750966973)*x_ref[0]**o+(0.905016928404)*x_ref[0]+(-0.171049542163)*x_ref[1]**o+(-0.20526244601)*x_ref[1]
        ref[1,3,2,1]=(0.723946025405)*x_ref[0]**o+(0.0157576743987)*x_ref[0]+(0.190875428312)*x_ref[1]**o+(0.407925899509)*x_ref[1]
        ref[1,4,0,0]=(-0.561582695803)*x_ref[0]**o+(-0.350262461371)*x_ref[0]+(0.57610409872)*x_ref[1]**o+(0.881780963877)*x_ref[1]
        ref[1,4,0,1]=(0.077066498695)*x_ref[0]**o+(0.515597802516)*x_ref[0]+(0.332011413016)*x_ref[1]**o+(0.215491334787)*x_ref[1]
        ref[1,4,1,0]=(0.33858727737)*x_ref[0]**o+(-0.573231800768)*x_ref[0]+(-0.559915809132)*x_ref[1]**o+(0.0240869142176)*x_ref[1]
        ref[1,4,1,1]=(-0.684581727952)*x_ref[0]**o+(0.365541904201)*x_ref[0]+(0.204252397376)*x_ref[1]**o+(0.149300766054)*x_ref[1]
        ref[1,4,2,0]=(-0.574225088383)*x_ref[0]**o+(0.0286339776067)*x_ref[0]+(-0.358732122607)*x_ref[1]**o+(-0.416932738957)*x_ref[1]
        ref[1,4,2,1]=(-0.308460184932)*x_ref[0]**o+(0.389214987836)*x_ref[0]+(-0.358303257031)*x_ref[1]**o+(0.348109483688)*x_ref[1]
        ref[2,0,0,0]=(-0.811318668804)*x_ref[0]**o+(-0.298028116982)*x_ref[0]+(0.679871523483)*x_ref[1]**o+(0.958042925287)*x_ref[1]
        ref[2,0,0,1]=(-0.616521818338)*x_ref[0]**o+(-0.0817124854496)*x_ref[0]+(0.403257366893)*x_ref[1]**o+(0.740689710435)*x_ref[1]
        ref[2,0,1,0]=(0.271046179127)*x_ref[0]**o+(0.814928093947)*x_ref[0]+(0.423722781579)*x_ref[1]**o+(0.925333209811)*x_ref[1]
        ref[2,0,1,1]=(0.0596651806816)*x_ref[0]**o+(0.493854605359)*x_ref[0]+(0.539998537439)*x_ref[1]**o+(-0.910865639375)*x_ref[1]
        ref[2,0,2,0]=(0.503267602327)*x_ref[0]**o+(-0.270966471743)*x_ref[0]+(-0.342063366676)*x_ref[1]**o+(-0.975700224705)*x_ref[1]
        ref[2,0,2,1]=(-0.0362358939195)*x_ref[0]**o+(-0.766033869339)*x_ref[0]+(0.891365100088)*x_ref[1]**o+(0.815025898929)*x_ref[1]
        ref[2,1,0,0]=(-0.667240459984)*x_ref[0]**o+(0.8659076701)*x_ref[0]+(0.215089397562)*x_ref[1]**o+(0.464769182307)*x_ref[1]
        ref[2,1,0,1]=(-0.298625221931)*x_ref[0]**o+(-0.504329743472)*x_ref[0]+(0.412472510208)*x_ref[1]**o+(0.241693152889)*x_ref[1]
        ref[2,1,1,0]=(-0.670730179133)*x_ref[0]**o+(0.0201070113832)*x_ref[0]+(-0.572725898919)*x_ref[1]**o+(0.665609259014)*x_ref[1]
        ref[2,1,1,1]=(-0.319080984089)*x_ref[0]**o+(0.0299423045224)*x_ref[0]+(0.0152311810613)*x_ref[1]**o+(-0.784646189026)*x_ref[1]
        ref[2,1,2,0]=(0.153308789716)*x_ref[0]**o+(0.725808150613)*x_ref[0]+(0.62876493338)*x_ref[1]**o+(0.80275284252)*x_ref[1]
        ref[2,1,2,1]=(0.804698870013)*x_ref[0]**o+(-0.631078773061)*x_ref[0]+(-0.015221975698)*x_ref[1]**o+(-0.954169686746)*x_ref[1]
        ref[2,2,0,0]=(0.169022797498)*x_ref[0]**o+(0.733025791099)*x_ref[0]+(0.321554996633)*x_ref[1]**o+(0.292173365364)*x_ref[1]
        ref[2,2,0,1]=(0.327345955109)*x_ref[0]**o+(0.21757823896)*x_ref[0]+(0.873027160446)*x_ref[1]**o+(-0.250192861579)*x_ref[1]
        ref[2,2,1,0]=(0.394387190832)*x_ref[0]**o+(-0.613661883482)*x_ref[0]+(0.806965291386)*x_ref[1]**o+(0.244513244016)*x_ref[1]
        ref[2,2,1,1]=(0.043913907536)*x_ref[0]**o+(-0.303567806424)*x_ref[0]+(0.547037093323)*x_ref[1]**o+(-0.598687385328)*x_ref[1]
        ref[2,2,2,0]=(0.875663154787)*x_ref[0]**o+(-0.34984832772)*x_ref[0]+(0.879143174396)*x_ref[1]**o+(-0.193154051109)*x_ref[1]
        ref[2,2,2,1]=(0.940311132918)*x_ref[0]**o+(-0.358708743382)*x_ref[0]+(0.117370945146)*x_ref[1]**o+(-0.664208034757)*x_ref[1]
        ref[2,3,0,0]=(0.0334525660588)*x_ref[0]**o+(0.592805915255)*x_ref[0]+(-0.278216157971)*x_ref[1]**o+(0.894369967671)*x_ref[1]
        ref[2,3,0,1]=(-0.966591106326)*x_ref[0]**o+(-0.882160390768)*x_ref[0]+(0.843982702732)*x_ref[1]**o+(0.363868921656)*x_ref[1]
        ref[2,3,1,0]=(-0.246392342771)*x_ref[0]**o+(-0.934459096242)*x_ref[0]+(-0.472771368537)*x_ref[1]**o+(0.148854304706)*x_ref[1]
        ref[2,3,1,1]=(0.248218424051)*x_ref[0]**o+(0.0980396576111)*x_ref[0]+(0.413322166489)*x_ref[1]**o+(0.429747261346)*x_ref[1]
        ref[2,3,2,0]=(0.742987565733)*x_ref[0]**o+(-0.213736450136)*x_ref[0]+(0.996975741314)*x_ref[1]**o+(0.960709958665)*x_ref[1]
        ref[2,3,2,1]=(0.436149490977)*x_ref[0]**o+(0.87632181197)*x_ref[0]+(-0.881934669912)*x_ref[1]**o+(-0.457520504175)*x_ref[1]
        ref[2,4,0,0]=(-0.519014330693)*x_ref[0]**o+(0.938619608407)*x_ref[0]+(-0.847954563609)*x_ref[1]**o+(0.791774379621)*x_ref[1]
        ref[2,4,0,1]=(-0.599487299453)*x_ref[0]**o+(0.977867555132)*x_ref[0]+(0.693105630579)*x_ref[1]**o+(-0.688827812942)*x_ref[1]
        ref[2,4,1,0]=(0.955907964163)*x_ref[0]**o+(0.0724056986172)*x_ref[0]+(0.254773619772)*x_ref[1]**o+(0.0428861146964)*x_ref[1]
        ref[2,4,1,1]=(-0.422202101998)*x_ref[0]**o+(0.803263223544)*x_ref[0]+(0.456048140785)*x_ref[1]**o+(0.0419133126183)*x_ref[1]
        ref[2,4,2,0]=(-0.182308704781)*x_ref[0]**o+(0.862282443488)*x_ref[0]+(-0.844027007955)*x_ref[1]**o+(0.586220526376)*x_ref[1]
        ref[2,4,2,1]=(-0.801542666229)*x_ref[0]**o+(0.903685299455)*x_ref[0]+(0.428926544377)*x_ref[1]**o+(-0.891220471549)*x_ref[1]
        ref[3,0,0,0]=(0.602917787713)*x_ref[0]**o+(-0.592239549743)*x_ref[0]+(0.761830006718)*x_ref[1]**o+(-0.784264167777)*x_ref[1]
        ref[3,0,0,1]=(0.819962957732)*x_ref[0]**o+(-0.455126790372)*x_ref[0]+(-0.144898728525)*x_ref[1]**o+(0.121566345339)*x_ref[1]
        ref[3,0,1,0]=(0.570694941559)*x_ref[0]**o+(-0.505678656386)*x_ref[0]+(0.295536587355)*x_ref[1]**o+(-0.852779737814)*x_ref[1]
        ref[3,0,1,1]=(-0.226246858753)*x_ref[0]**o+(-0.513900306228)*x_ref[0]+(-0.389483591407)*x_ref[1]**o+(-0.735105335539)*x_ref[1]
        ref[3,0,2,0]=(0.941877610938)*x_ref[0]**o+(0.93808330298)*x_ref[0]+(0.189785162182)*x_ref[1]**o+(0.546525366304)*x_ref[1]
        ref[3,0,2,1]=(0.10639100485)*x_ref[0]**o+(-0.773361007972)*x_ref[0]+(-0.470531427579)*x_ref[1]**o+(0.467359342808)*x_ref[1]
        ref[3,1,0,0]=(0.857725599589)*x_ref[0]**o+(0.0881746413132)*x_ref[0]+(0.040713151152)*x_ref[1]**o+(-0.52070729883)*x_ref[1]
        ref[3,1,0,1]=(0.266807652146)*x_ref[0]**o+(0.337496673892)*x_ref[0]+(-0.562266297445)*x_ref[1]**o+(0.326015195113)*x_ref[1]
        ref[3,1,1,0]=(0.177275509465)*x_ref[0]**o+(-0.647855761301)*x_ref[0]+(-0.0963373631995)*x_ref[1]**o+(-0.535256252994)*x_ref[1]
        ref[3,1,1,1]=(0.430309295606)*x_ref[0]**o+(-0.0894749557428)*x_ref[0]+(0.503470724483)*x_ref[1]**o+(-0.202361122248)*x_ref[1]
        ref[3,1,2,0]=(0.283995014194)*x_ref[0]**o+(-0.544419907194)*x_ref[0]+(0.78478571712)*x_ref[1]**o+(0.341540664147)*x_ref[1]
        ref[3,1,2,1]=(-0.281042869936)*x_ref[0]**o+(0.728771766492)*x_ref[0]+(-0.298974388557)*x_ref[1]**o+(-0.836789906985)*x_ref[1]
        ref[3,2,0,0]=(-0.415323371561)*x_ref[0]**o+(0.583971137371)*x_ref[0]+(-0.265658679489)*x_ref[1]**o+(-0.432447373809)*x_ref[1]
        ref[3,2,0,1]=(0.690486298855)*x_ref[0]**o+(0.552521632226)*x_ref[0]+(0.237622739042)*x_ref[1]**o+(-0.254513693924)*x_ref[1]
        ref[3,2,1,0]=(0.453985687207)*x_ref[0]**o+(-0.785385580962)*x_ref[0]+(0.0190987841634)*x_ref[1]**o+(-0.7923288861)*x_ref[1]
        ref[3,2,1,1]=(-0.964082656379)*x_ref[0]**o+(-0.806315937371)*x_ref[0]+(0.271145420366)*x_ref[1]**o+(-0.839014535945)*x_ref[1]
        ref[3,2,2,0]=(-0.372062773997)*x_ref[0]**o+(-0.069749560567)*x_ref[0]+(-0.126310395383)*x_ref[1]**o+(0.972025865285)*x_ref[1]
        ref[3,2,2,1]=(-0.580353020086)*x_ref[0]**o+(-0.152746821477)*x_ref[0]+(0.297553660535)*x_ref[1]**o+(0.444791568145)*x_ref[1]
        ref[3,3,0,0]=(-0.361520029439)*x_ref[0]**o+(-0.810391070499)*x_ref[0]+(0.462123882642)*x_ref[1]**o+(-0.974534227742)*x_ref[1]
        ref[3,3,0,1]=(0.113370617837)*x_ref[0]**o+(0.428257355435)*x_ref[0]+(-0.18656623758)*x_ref[1]**o+(0.206503327394)*x_ref[1]
        ref[3,3,1,0]=(0.341328445689)*x_ref[0]**o+(0.34002947054)*x_ref[0]+(0.997677286362)*x_ref[1]**o+(0.43300073225)*x_ref[1]
        ref[3,3,1,1]=(-0.513912982823)*x_ref[0]**o+(0.826573827064)*x_ref[0]+(-0.701417864085)*x_ref[1]**o+(0.567078883219)*x_ref[1]
        ref[3,3,2,0]=(-0.325536866965)*x_ref[0]**o+(0.924405767451)*x_ref[0]+(-0.0481381954346)*x_ref[1]**o+(0.912210617073)*x_ref[1]
        ref[3,3,2,1]=(0.491532434302)*x_ref[0]**o+(-0.584980848036)*x_ref[0]+(-0.788317530858)*x_ref[1]**o+(-0.900681893172)*x_ref[1]
        ref[3,4,0,0]=(0.432249855148)*x_ref[0]**o+(-0.673369312552)*x_ref[0]+(0.244704845849)*x_ref[1]**o+(-0.300995595367)*x_ref[1]
        ref[3,4,0,1]=(0.383037211984)*x_ref[0]**o+(0.299711400488)*x_ref[0]+(0.870584255454)*x_ref[1]**o+(-0.33571823329)*x_ref[1]
        ref[3,4,1,0]=(0.737134180177)*x_ref[0]**o+(-0.101307090628)*x_ref[0]+(-0.970632061143)*x_ref[1]**o+(-0.449042684137)*x_ref[1]
        ref[3,4,1,1]=(0.00684250643632)*x_ref[0]**o+(0.676964654022)*x_ref[0]+(0.789069834858)*x_ref[1]**o+(-0.829739508197)*x_ref[1]
        ref[3,4,2,0]=(0.7279044944)*x_ref[0]**o+(0.215711122038)*x_ref[0]+(0.385631252632)*x_ref[1]**o+(0.629390433529)*x_ref[1]
        ref[3,4,2,1]=(-0.380750716629)*x_ref[0]**o+(-0.413040692437)*x_ref[0]+(0.26795244639)*x_ref[1]**o+(0.492475504659)*x_ref[1]
      else:
        arg[0,0,0,0]=(0.287160004495)*x[0]**o+(0.35112250158)*x[0]+(0.061693427026)*x[1]**o+(-0.752505628029)*x[1]+(-0.57137756028)*x[2]**o+(-0.0397598892504)*x[2]
        arg[0,0,0,1]=(0.272799584607)*x[0]**o+(-0.569947956933)*x[0]+(0.322189261657)*x[1]**o+(-0.440552272224)*x[1]+(-0.10460241481)*x[2]**o+(-0.884848984752)*x[2]
        arg[0,0,1,0]=(0.147766301604)*x[0]**o+(0.405651654428)*x[0]+(-0.440827209529)*x[1]**o+(-0.399952864271)*x[1]+(-0.770191009459)*x[2]**o+(-0.789853968049)*x[2]
        arg[0,0,1,1]=(0.688053165375)*x[0]**o+(-0.154890421924)*x[0]+(0.583684164319)*x[1]**o+(-0.398082068217)*x[1]+(-0.573142923099)*x[2]**o+(-0.205271877337)*x[2]
        arg[0,0,2,0]=(0.807043275877)*x[0]**o+(0.843046383231)*x[0]+(-0.588146471402)*x[1]**o+(0.157787304557)*x[1]+(-0.771009748568)*x[2]**o+(0.162090791609)*x[2]
        arg[0,0,2,1]=(-0.0706800489392)*x[0]**o+(0.499961344665)*x[0]+(0.208926178797)*x[1]**o+(-0.7462472769)*x[1]+(-0.105475405648)*x[2]**o+(-0.18670229948)*x[2]
        arg[0,1,0,0]=(-0.696657281943)*x[0]**o+(-0.0251785345759)*x[0]+(-0.579575530451)*x[1]**o+(-0.0628378665725)*x[1]+(0.668395441302)*x[2]**o+(0.813478105092)*x[2]
        arg[0,1,0,1]=(-0.574784314876)*x[0]**o+(-0.321268045204)*x[0]+(0.160704464213)*x[1]**o+(-0.676016748103)*x[1]+(0.918971102865)*x[2]**o+(0.252082410081)*x[2]
        arg[0,1,1,0]=(-0.905062846817)*x[0]**o+(0.788906658477)*x[0]+(-0.408807611936)*x[1]**o+(0.114686186437)*x[1]+(-0.654685568493)*x[2]**o+(0.0879202139254)*x[2]
        arg[0,1,1,1]=(-0.68020231443)*x[0]**o+(-0.82049202902)*x[0]+(-0.958352740187)*x[1]**o+(-0.33201745802)*x[1]+(0.539810363682)*x[2]**o+(0.744988573087)*x[2]
        arg[0,1,2,0]=(-0.960884744997)*x[0]**o+(-0.704157132503)*x[0]+(0.282236905414)*x[1]**o+(0.756363456433)*x[1]+(-0.365400101792)*x[2]**o+(0.975735846554)*x[2]
        arg[0,1,2,1]=(-0.691780076951)*x[0]**o+(-0.952575440059)*x[0]+(-0.921247808098)*x[1]**o+(-0.88872834095)*x[1]+(0.534103013833)*x[2]**o+(-0.602295186104)*x[2]
        arg[0,2,0,0]=(0.460698415704)*x[0]**o+(0.770243814897)*x[0]+(-0.380381727574)*x[1]**o+(0.476360845668)*x[1]+(-0.681040425102)*x[2]**o+(-0.244369435116)*x[2]
        arg[0,2,0,1]=(0.251591652281)*x[0]**o+(0.748234919686)*x[0]+(-0.698189037307)*x[1]**o+(-0.292400412508)*x[1]+(0.613850696577)*x[2]**o+(-0.942243226834)*x[2]
        arg[0,2,1,0]=(-0.718090066328)*x[0]**o+(-0.322499511794)*x[0]+(-0.75105623397)*x[1]**o+(0.526680271832)*x[1]+(-0.186671531793)*x[2]**o+(-0.449273875991)*x[2]
        arg[0,2,1,1]=(-0.52165552452)*x[0]**o+(0.902446006085)*x[0]+(-0.915575172702)*x[1]**o+(-0.150816194662)*x[1]+(0.596782785689)*x[2]**o+(0.546880865642)*x[2]
        arg[0,2,2,0]=(-0.104486704816)*x[0]**o+(0.723863699457)*x[0]+(0.758622117279)*x[1]**o+(0.372271580145)*x[1]+(-0.0820570383792)*x[2]**o+(-0.63539781104)*x[2]
        arg[0,2,2,1]=(0.0355930823603)*x[0]**o+(0.0931942600432)*x[0]+(-0.118096985978)*x[1]**o+(0.296983782212)*x[1]+(-0.864491763712)*x[2]**o+(0.942230663978)*x[2]
        arg[0,3,0,0]=(0.371153709815)*x[0]**o+(0.0427309083893)*x[0]+(-0.886563919084)*x[1]**o+(-0.14946513404)*x[1]+(0.80903804519)*x[2]**o+(0.69728533073)*x[2]
        arg[0,3,0,1]=(0.0593930396142)*x[0]**o+(-0.370327974089)*x[0]+(-0.388354464683)*x[1]**o+(-0.367648771812)*x[1]+(-0.573754821458)*x[2]**o+(-0.947499155479)*x[2]
        arg[0,3,1,0]=(-0.869817317)*x[0]**o+(0.874317772678)*x[0]+(-0.870060178375)*x[1]**o+(0.66571074524)*x[1]+(-0.669052377378)*x[2]**o+(0.940212680575)*x[2]
        arg[0,3,1,1]=(-0.417445606772)*x[0]**o+(0.942606057619)*x[0]+(-0.133936034409)*x[1]**o+(-0.363274280866)*x[1]+(0.83487439935)*x[2]**o+(-0.108398981461)*x[2]
        arg[0,3,2,0]=(0.363623383293)*x[0]**o+(0.624267309217)*x[0]+(0.801719984899)*x[1]**o+(0.189212370122)*x[1]+(0.464633630928)*x[2]**o+(-0.979808318438)*x[2]
        arg[0,3,2,1]=(0.85384408805)*x[0]**o+(-0.889214754116)*x[0]+(0.164915391224)*x[1]**o+(0.0168991405201)*x[1]+(0.097171994236)*x[2]**o+(-0.146094197339)*x[2]
        arg[0,4,0,0]=(0.778356158972)*x[0]**o+(0.910241265149)*x[0]+(0.123690863272)*x[1]**o+(0.87802121779)*x[1]+(-0.714615668595)*x[2]**o+(-0.437472831583)*x[2]
        arg[0,4,0,1]=(-0.157185481783)*x[0]**o+(0.352802163968)*x[0]+(0.390184123612)*x[1]**o+(-0.680415093098)*x[1]+(-0.121195042454)*x[2]**o+(-0.713693712117)*x[2]
        arg[0,4,1,0]=(0.778115668646)*x[0]**o+(-0.111729042892)*x[0]+(0.451518002735)*x[1]**o+(0.232731183374)*x[1]+(-0.764246158086)*x[2]**o+(-0.795525155328)*x[2]
        arg[0,4,1,1]=(-0.880652367027)*x[0]**o+(-0.314966079126)*x[0]+(-0.640720373065)*x[1]**o+(0.336230541901)*x[1]+(0.566956999526)*x[2]**o+(0.74098258811)*x[2]
        arg[0,4,2,0]=(0.908657274301)*x[0]**o+(0.636427823614)*x[0]+(0.650137414386)*x[1]**o+(0.695558379005)*x[1]+(-0.489852401087)*x[2]**o+(-0.479227108044)*x[2]
        arg[0,4,2,1]=(0.015734241525)*x[0]**o+(0.0719884727088)*x[0]+(0.811864895261)*x[1]**o+(0.959127986784)*x[1]+(-0.238146663368)*x[2]**o+(-0.454939322571)*x[2]
        arg[1,0,0,0]=(-0.918218246559)*x[0]**o+(-0.662787169269)*x[0]+(-0.954035144111)*x[1]**o+(-0.647264685228)*x[1]+(-0.834085960213)*x[2]**o+(-0.167541951024)*x[2]
        arg[1,0,0,1]=(-0.670950869382)*x[0]**o+(0.826472159989)*x[0]+(0.180410872786)*x[1]**o+(-0.715162871594)*x[1]+(0.653287853169)*x[2]**o+(0.892060529625)*x[2]
        arg[1,0,1,0]=(0.398789786407)*x[0]**o+(0.689408033554)*x[0]+(0.124338245128)*x[1]**o+(0.748280808587)*x[1]+(0.418357122274)*x[2]**o+(0.230707440075)*x[2]
        arg[1,0,1,1]=(-0.912778964055)*x[0]**o+(0.904288708694)*x[0]+(-0.874744856901)*x[1]**o+(0.550162195719)*x[1]+(-0.7526423373)*x[2]**o+(0.820993231654)*x[2]
        arg[1,0,2,0]=(-0.843459684648)*x[0]**o+(-0.579708764091)*x[0]+(-0.876072131849)*x[1]**o+(0.933694599696)*x[1]+(0.507781209021)*x[2]**o+(-0.719729482133)*x[2]
        arg[1,0,2,1]=(0.594566925796)*x[0]**o+(-0.994301292358)*x[0]+(0.808114389564)*x[1]**o+(-0.970803712864)*x[1]+(0.433544910389)*x[2]**o+(0.260659440299)*x[2]
        arg[1,1,0,0]=(0.997377553172)*x[0]**o+(0.458428616383)*x[0]+(0.357280778493)*x[1]**o+(0.349837110741)*x[1]+(-0.709592214445)*x[2]**o+(-0.630000739033)*x[2]
        arg[1,1,0,1]=(0.121595650501)*x[0]**o+(0.40870120924)*x[0]+(0.576917123807)*x[1]**o+(-0.932268308325)*x[1]+(0.710043472116)*x[2]**o+(-0.716528412077)*x[2]
        arg[1,1,1,0]=(-0.151416035033)*x[0]**o+(0.0255347649787)*x[0]+(-0.752987701889)*x[1]**o+(-0.480969875813)*x[1]+(-0.610514504179)*x[2]**o+(0.959274757402)*x[2]
        arg[1,1,1,1]=(0.76695706106)*x[0]**o+(-0.0750965975512)*x[0]+(0.846833900735)*x[1]**o+(-0.105838465439)*x[1]+(0.541027486457)*x[2]**o+(0.152543017589)*x[2]
        arg[1,1,2,0]=(0.0856977181865)*x[0]**o+(-0.188139369706)*x[0]+(0.936071565289)*x[1]**o+(-0.3958713036)*x[1]+(-0.350945878893)*x[2]**o+(0.382302674883)*x[2]
        arg[1,1,2,1]=(0.873054234789)*x[0]**o+(-0.490869716146)*x[0]+(0.46720526212)*x[1]**o+(-0.334203412334)*x[1]+(-0.81140895636)*x[2]**o+(-0.973858971749)*x[2]
        arg[1,2,0,0]=(-0.716676352443)*x[0]**o+(0.659821396277)*x[0]+(0.335964292954)*x[1]**o+(0.11609664773)*x[1]+(-0.889186749466)*x[2]**o+(0.454231973946)*x[2]
        arg[1,2,0,1]=(0.367048872648)*x[0]**o+(0.276083713126)*x[0]+(0.686639128043)*x[1]**o+(-0.991414415293)*x[1]+(0.25360515964)*x[2]**o+(-0.377623315093)*x[2]
        arg[1,2,1,0]=(-0.723089358777)*x[0]**o+(-0.516690112406)*x[0]+(0.262334273979)*x[1]**o+(0.140773502537)*x[1]+(0.447333972396)*x[2]**o+(-0.551395692966)*x[2]
        arg[1,2,1,1]=(0.738791182491)*x[0]**o+(-0.978120626668)*x[0]+(-0.217970433713)*x[1]**o+(0.0181011067492)*x[1]+(0.756322867523)*x[2]**o+(0.495269195978)*x[2]
        arg[1,2,2,0]=(-0.630506449854)*x[0]**o+(-0.0865482364231)*x[0]+(-0.415896403157)*x[1]**o+(-0.712833149895)*x[1]+(-0.180482789777)*x[2]**o+(0.00788290661235)*x[2]
        arg[1,2,2,1]=(0.0946913865377)*x[0]**o+(0.752854827716)*x[0]+(0.257846220843)*x[1]**o+(-0.953926983578)*x[1]+(-0.388505920682)*x[2]**o+(-0.10751401251)*x[2]
        arg[1,3,0,0]=(0.694014030611)*x[0]**o+(0.0433103547797)*x[0]+(-0.0805025225569)*x[1]**o+(0.389052254754)*x[1]+(-0.813210939614)*x[2]**o+(-0.593788787896)*x[2]
        arg[1,3,0,1]=(0.0101606353406)*x[0]**o+(-0.849874258855)*x[0]+(-0.0578973959381)*x[1]**o+(0.942113140424)*x[1]+(0.613716459216)*x[2]**o+(0.72631176833)*x[2]
        arg[1,3,1,0]=(0.264703608811)*x[0]**o+(0.457962548326)*x[0]+(-0.433795242673)*x[1]**o+(0.650995712848)*x[1]+(0.372471807967)*x[2]**o+(0.00416636841047)*x[2]
        arg[1,3,1,1]=(-0.73686451669)*x[0]**o+(-0.0425614910776)*x[0]+(-0.587820431311)*x[1]**o+(-0.377628617833)*x[1]+(-0.463964014406)*x[2]**o+(0.283273874989)*x[2]
        arg[1,3,2,0]=(-0.689341439143)*x[0]**o+(-0.675861918889)*x[0]+(-0.918622662352)*x[1]**o+(0.740990261888)*x[1]+(0.0454007677041)*x[2]**o+(0.513623361074)*x[2]
        arg[1,3,2,1]=(0.13229101848)*x[0]**o+(0.171873799817)*x[0]+(-0.154491174434)*x[1]**o+(0.897976884089)*x[1]+(-0.598886311821)*x[2]**o+(0.738696710918)*x[2]
        arg[1,4,0,0]=(0.59403636966)*x[0]**o+(0.93856848969)*x[0]+(0.80164849167)*x[1]**o+(-0.753322979568)*x[1]+(-0.107098073916)*x[2]**o+(-0.623047487973)*x[2]
        arg[1,4,0,1]=(0.291046099082)*x[0]**o+(-0.437982471049)*x[0]+(0.017395762947)*x[1]**o+(0.852440593923)*x[1]+(0.630825424359)*x[2]**o+(0.419347491727)*x[2]
        arg[1,4,1,0]=(-0.747962012701)*x[0]**o+(-0.11622032094)*x[0]+(0.324950549781)*x[1]**o+(0.950911398327)*x[1]+(-0.18098990205)*x[2]**o+(-0.676060633522)*x[2]
        arg[1,4,1,1]=(0.545224363128)*x[0]**o+(0.753376367121)*x[0]+(0.271426695753)*x[1]**o+(-0.139309844675)*x[1]+(-0.923434716375)*x[2]**o+(-0.985988324389)*x[2]
        arg[1,4,2,0]=(-0.532260822047)*x[0]**o+(0.901407135724)*x[0]+(0.937451330141)*x[1]**o+(0.285217666559)*x[1]+(-0.133028984824)*x[2]**o+(0.993965133696)*x[2]
        arg[1,4,2,1]=(0.2231815607)*x[0]**o+(0.994845537582)*x[0]+(0.0368727378195)*x[1]**o+(0.511772933777)*x[1]+(-0.871580215976)*x[2]**o+(-0.587010057329)*x[2]
        arg[2,0,0,0]=(-0.55149411913)*x[0]**o+(-0.905541278559)*x[0]+(0.461217373096)*x[1]**o+(-0.53359853241)*x[1]+(-0.666643290883)*x[2]**o+(0.151495826926)*x[2]
        arg[2,0,0,1]=(-0.980485062148)*x[0]**o+(0.150165616162)*x[0]+(0.59165725487)*x[1]**o+(-0.243853536017)*x[1]+(0.66816261686)*x[2]**o+(0.2425696131)*x[2]
        arg[2,0,1,0]=(-0.509337213758)*x[0]**o+(0.652936706687)*x[0]+(-0.849214338091)*x[1]**o+(-0.523354123549)*x[1]+(0.371515689887)*x[2]**o+(0.799989815414)*x[2]
        arg[2,0,1,1]=(0.461584544541)*x[0]**o+(0.662931355954)*x[0]+(0.744415634786)*x[1]**o+(-0.016743801256)*x[1]+(0.172945846325)*x[2]**o+(-0.0889827089341)*x[2]
        arg[2,0,2,0]=(0.506514791562)*x[0]**o+(0.974477160694)*x[0]+(-0.968485343167)*x[1]**o+(0.427480013599)*x[1]+(0.0460755879358)*x[2]**o+(0.186500994948)*x[2]
        arg[2,0,2,1]=(0.259831607993)*x[0]**o+(-0.0141290444125)*x[0]+(-0.968452177682)*x[1]**o+(0.635619125009)*x[1]+(-0.958848431827)*x[2]**o+(-0.0977241523308)*x[2]
        arg[2,1,0,0]=(0.204233222933)*x[0]**o+(0.834739381877)*x[0]+(0.19961185048)*x[1]**o+(-0.238470380578)*x[1]+(-0.749979713773)*x[2]**o+(-0.268168886251)*x[2]
        arg[2,1,0,1]=(0.998123643088)*x[0]**o+(0.635823733041)*x[0]+(-0.265447124419)*x[1]**o+(-0.225582751946)*x[1]+(0.91705937439)*x[2]**o+(-0.739461510792)*x[2]
        arg[2,1,1,0]=(0.0800715288646)*x[0]**o+(-0.37677196659)*x[0]+(-0.0737286350219)*x[1]**o+(0.032076062933)*x[1]+(0.245421912709)*x[2]**o+(-0.8378023935)*x[2]
        arg[2,1,1,1]=(-0.572376055603)*x[0]**o+(0.8675468469)*x[0]+(-0.533012889867)*x[1]**o+(-0.247215658297)*x[1]+(0.673144564693)*x[2]**o+(-0.749136108985)*x[2]
        arg[2,1,2,0]=(-0.661188192488)*x[0]**o+(-0.924435593008)*x[0]+(-0.791845808916)*x[1]**o+(0.00133765351092)*x[1]+(0.233676862873)*x[2]**o+(0.981812737962)*x[2]
        arg[2,1,2,1]=(0.332072445986)*x[0]**o+(-0.909373798588)*x[0]+(0.927025647819)*x[1]**o+(-0.936348942617)*x[1]+(0.251508592152)*x[2]**o+(-0.468147973987)*x[2]
        arg[2,2,0,0]=(0.892377893841)*x[0]**o+(-0.147933293091)*x[0]+(0.950491102076)*x[1]**o+(0.595250355926)*x[1]+(0.0169764649992)*x[2]**o+(-0.683137752586)*x[2]
        arg[2,2,0,1]=(0.00579397005645)*x[0]**o+(0.627197373917)*x[0]+(0.663574756196)*x[1]**o+(0.934106767513)*x[1]+(0.105844878022)*x[2]**o+(0.513322917688)*x[2]
        arg[2,2,1,0]=(-0.168776016133)*x[0]**o+(-0.940071612944)*x[0]+(0.0366053554566)*x[1]**o+(-0.628010909771)*x[1]+(0.250023045906)*x[2]**o+(-0.00327322332166)*x[2]
        arg[2,2,1,1]=(0.904155764376)*x[0]**o+(-0.258018307306)*x[0]+(0.65862820521)*x[1]**o+(0.434480798515)*x[1]+(0.4089419173)*x[2]**o+(-0.518032695598)*x[2]
        arg[2,2,2,0]=(-0.41470070958)*x[0]**o+(-0.783186751952)*x[0]+(0.548309561942)*x[1]**o+(0.837130592803)*x[1]+(-0.346808491848)*x[2]**o+(0.165416040985)*x[2]
        arg[2,2,2,1]=(0.997845909195)*x[0]**o+(0.448428042158)*x[0]+(-0.657050004311)*x[1]**o+(0.493104851747)*x[1]+(0.558315071197)*x[2]**o+(0.206659432923)*x[2]
        arg[2,3,0,0]=(0.433874360232)*x[0]**o+(-0.0635646197131)*x[0]+(-0.521690521523)*x[1]**o+(-0.298130684888)*x[1]+(-0.131905313644)*x[2]**o+(0.64389451112)*x[2]
        arg[2,3,0,1]=(0.457729315216)*x[0]**o+(-0.419323242135)*x[0]+(0.614544051135)*x[1]**o+(-0.701671999275)*x[1]+(0.905494300727)*x[2]**o+(-0.565716985337)*x[2]
        arg[2,3,1,0]=(0.256570728097)*x[0]**o+(0.866349285103)*x[0]+(-0.115768354896)*x[1]**o+(-0.613044456549)*x[1]+(-0.20074185156)*x[2]**o+(-0.188403571776)*x[2]
        arg[2,3,1,1]=(0.767479943414)*x[0]**o+(0.884066215007)*x[0]+(0.363855420056)*x[1]**o+(0.919698981104)*x[1]+(0.225747762317)*x[2]**o+(-0.220362271104)*x[2]
        arg[2,3,2,0]=(-0.746469501382)*x[0]**o+(0.680520398001)*x[0]+(0.926950996325)*x[1]**o+(-0.122831303044)*x[1]+(-0.27226561675)*x[2]**o+(0.933304087951)*x[2]
        arg[2,3,2,1]=(-0.0961434747216)*x[0]**o+(0.987660516767)*x[0]+(0.288175205045)*x[1]**o+(0.683720820403)*x[1]+(0.354403143642)*x[2]**o+(-0.745694089132)*x[2]
        arg[2,4,0,0]=(0.921750835986)*x[0]**o+(-0.0358861598394)*x[0]+(0.190129645259)*x[1]**o+(0.377067803331)*x[1]+(-0.0188615236582)*x[2]**o+(0.804533888824)*x[2]
        arg[2,4,0,1]=(-0.256132234484)*x[0]**o+(-0.102661411017)*x[0]+(-0.903465157009)*x[1]**o+(0.50133310168)*x[1]+(-0.0889102598619)*x[2]**o+(-0.0310174238365)*x[2]
        arg[2,4,1,0]=(0.58483798855)*x[0]**o+(0.967943344305)*x[0]+(-0.0701080005438)*x[1]**o+(0.489476104406)*x[1]+(0.615308427038)*x[2]**o+(-0.38065505709)*x[2]
        arg[2,4,1,1]=(0.647734130724)*x[0]**o+(-0.930934392719)*x[0]+(0.267370540491)*x[1]**o+(-0.0175019424556)*x[1]+(0.38593657129)*x[2]**o+(-0.985922292359)*x[2]
        arg[2,4,2,0]=(-0.289971479933)*x[0]**o+(0.882014313157)*x[0]+(0.147321067579)*x[1]**o+(-0.176771838489)*x[1]+(0.710916531544)*x[2]**o+(-0.0856290170401)*x[2]
        arg[2,4,2,1]=(-0.465930997733)*x[0]**o+(-0.305563942094)*x[0]+(-0.824535262061)*x[1]**o+(-0.917892226972)*x[1]+(0.624462871955)*x[2]**o+(0.544736500065)*x[2]
        arg[3,0,0,0]=(-0.673230815097)*x[0]**o+(-0.726292712917)*x[0]+(0.0531037021688)*x[1]**o+(-0.467219030159)*x[1]+(-0.0729861369718)*x[2]**o+(-0.440394335015)*x[2]
        arg[3,0,0,1]=(-0.358235295558)*x[0]**o+(-0.301184713671)*x[0]+(-0.496030649086)*x[1]**o+(-0.125922184777)*x[1]+(0.946715036973)*x[2]**o+(0.846370337893)*x[2]
        arg[3,0,1,0]=(-0.394175109612)*x[0]**o+(-0.85137582307)*x[0]+(-0.667804785044)*x[1]**o+(0.156762109891)*x[1]+(-0.616444700525)*x[2]**o+(0.0738729399356)*x[2]
        arg[3,0,1,1]=(-0.652875462534)*x[0]**o+(-0.0605650192541)*x[0]+(0.840539933823)*x[1]**o+(-0.422819423301)*x[1]+(0.113440226819)*x[2]**o+(-0.915154532306)*x[2]
        arg[3,0,2,0]=(-0.162863352328)*x[0]**o+(-0.364461332292)*x[0]+(-0.222181741398)*x[1]**o+(-0.563668008365)*x[1]+(0.118491770105)*x[2]**o+(0.0809155053022)*x[2]
        arg[3,0,2,1]=(-0.985278929437)*x[0]**o+(-0.919440760157)*x[0]+(-0.776910686)*x[1]**o+(-0.884270488943)*x[1]+(0.984279010363)*x[2]**o+(-0.489503278582)*x[2]
        arg[3,1,0,0]=(-0.806565449823)*x[0]**o+(-0.0851385375281)*x[0]+(-0.88851381773)*x[1]**o+(-0.666539867528)*x[1]+(0.154825531411)*x[2]**o+(0.764069849144)*x[2]
        arg[3,1,0,1]=(-0.222125783918)*x[0]**o+(0.568035675467)*x[0]+(0.470171778932)*x[1]**o+(-0.307608710881)*x[1]+(-0.836083742471)*x[2]**o+(0.553597553381)*x[2]
        arg[3,1,1,0]=(0.431237389847)*x[0]**o+(-0.141391012471)*x[0]+(0.726895025519)*x[1]**o+(-0.527868751679)*x[1]+(0.711761083137)*x[2]**o+(0.996992864103)*x[2]
        arg[3,1,1,1]=(-0.123885959924)*x[0]**o+(0.877034475529)*x[0]+(-0.0432492682928)*x[1]**o+(-0.370315494618)*x[1]+(0.243139719071)*x[2]**o+(-0.684318634902)*x[2]
        arg[3,1,2,0]=(-0.477214489355)*x[0]**o+(0.340808576619)*x[0]+(-0.0991802306558)*x[1]**o+(0.462599821908)*x[1]+(0.091768281564)*x[2]**o+(0.0315552838839)*x[2]
        arg[3,1,2,1]=(0.676189142894)*x[0]**o+(0.510136386913)*x[0]+(-0.558579041952)*x[1]**o+(0.961615047877)*x[1]+(0.510717386124)*x[2]**o+(0.774430183432)*x[2]
        arg[3,2,0,0]=(0.158370232444)*x[0]**o+(0.209424855992)*x[0]+(0.691438753013)*x[1]**o+(-0.781594298738)*x[1]+(-0.0188536627517)*x[2]**o+(0.136400798995)*x[2]
        arg[3,2,0,1]=(-0.440674414328)*x[0]**o+(0.118228612687)*x[0]+(-0.700026492613)*x[1]**o+(-0.981341215714)*x[1]+(-0.172847519578)*x[2]**o+(-0.423095202172)*x[2]
        arg[3,2,1,0]=(-0.0694960391695)*x[0]**o+(-0.66448610135)*x[0]+(-0.177572703748)*x[1]**o+(-0.060488161727)*x[1]+(0.306021853885)*x[2]**o+(0.0730338716187)*x[2]
        arg[3,2,1,1]=(-0.360662154547)*x[0]**o+(0.577679359648)*x[0]+(-0.515313534966)*x[1]**o+(0.217352687358)*x[1]+(0.728367577882)*x[2]**o+(0.38505849834)*x[2]
        arg[3,2,2,0]=(0.568671861232)*x[0]**o+(-0.0792490917882)*x[0]+(0.0188002885229)*x[1]**o+(-0.956862575759)*x[1]+(-0.844332442983)*x[2]**o+(0.670792928042)*x[2]
        arg[3,2,2,1]=(0.833099204713)*x[0]**o+(0.368482169933)*x[0]+(-0.926002498892)*x[1]**o+(-0.130646446568)*x[1]+(0.81164928914)*x[2]**o+(-0.641079919927)*x[2]
        arg[3,3,0,0]=(-0.277769350989)*x[0]**o+(0.753452018649)*x[0]+(-0.766322344505)*x[1]**o+(-0.679108224612)*x[1]+(0.0799499795424)*x[2]**o+(0.255751308975)*x[2]
        arg[3,3,0,1]=(0.429176753733)*x[0]**o+(0.123106050259)*x[0]+(-0.169037728992)*x[1]**o+(0.749245933687)*x[1]+(0.560140859355)*x[2]**o+(-0.937091840021)*x[2]
        arg[3,3,1,0]=(0.648792557593)*x[0]**o+(-0.057555187513)*x[0]+(0.883290331726)*x[1]**o+(0.539276905995)*x[1]+(0.473614408583)*x[2]**o+(0.723258340333)*x[2]
        arg[3,3,1,1]=(0.527298833572)*x[0]**o+(0.0753824058279)*x[0]+(0.719196099257)*x[1]**o+(-0.0124603350035)*x[1]+(-0.964484650253)*x[2]**o+(0.404444269694)*x[2]
        arg[3,3,2,0]=(-0.403640677462)*x[0]**o+(0.906195964766)*x[0]+(-0.561419078404)*x[1]**o+(0.645408717838)*x[1]+(0.671624799609)*x[2]**o+(-0.573901868621)*x[2]
        arg[3,3,2,1]=(-0.288117182165)*x[0]**o+(-0.391003584945)*x[0]+(-0.555037182038)*x[1]**o+(0.299362209017)*x[1]+(-0.552429724326)*x[2]**o+(-0.416923306774)*x[2]
        arg[3,4,0,0]=(-0.19594719192)*x[0]**o+(0.150898934692)*x[0]+(0.210935232044)*x[1]**o+(-0.319764293347)*x[1]+(-0.642450751195)*x[2]**o+(0.22231085168)*x[2]
        arg[3,4,0,1]=(-0.0514589460225)*x[0]**o+(0.0366025646754)*x[0]+(0.416280576691)*x[1]**o+(0.609178063951)*x[1]+(0.835611677062)*x[2]**o+(-0.131630571478)*x[2]
        arg[3,4,1,0]=(0.171509152214)*x[0]**o+(-0.761188271093)*x[0]+(-0.915951802896)*x[1]**o+(-0.451796709915)*x[1]+(-0.979847828162)*x[2]**o+(0.0566453393315)*x[2]
        arg[3,4,1,1]=(0.690427998102)*x[0]**o+(-0.358318625695)*x[0]+(0.330270871613)*x[1]**o+(-0.724392522137)*x[1]+(-0.535474623489)*x[2]**o+(-0.12341743885)*x[2]
        arg[3,4,2,0]=(0.488090753426)*x[0]**o+(-0.352417078372)*x[0]+(0.971988099319)*x[1]**o+(-0.288929516009)*x[1]+(0.612576734094)*x[2]**o+(-0.43799514603)*x[2]
        arg[3,4,2,1]=(-0.098768101356)*x[0]**o+(0.123300661895)*x[0]+(-0.0489724280789)*x[1]**o+(-0.422508669137)*x[1]+(-0.861483104139)*x[2]**o+(0.950198316001)*x[2]
        ref[0,0,0,0]=(0.287160004495)*x_ref[0]**o+(0.35112250158)*x_ref[0]+(0.061693427026)*x_ref[1]**o+(-0.752505628029)*x_ref[1]+(-0.57137756028)*x_ref[2]**o+(-0.0397598892504)*x_ref[2]
        ref[0,0,0,1]=(0.272799584607)*x_ref[0]**o+(-0.569947956933)*x_ref[0]+(0.322189261657)*x_ref[1]**o+(-0.440552272224)*x_ref[1]+(-0.10460241481)*x_ref[2]**o+(-0.884848984752)*x_ref[2]
        ref[0,0,1,0]=(0.147766301604)*x_ref[0]**o+(0.405651654428)*x_ref[0]+(-0.440827209529)*x_ref[1]**o+(-0.399952864271)*x_ref[1]+(-0.770191009459)*x_ref[2]**o+(-0.789853968049)*x_ref[2]
        ref[0,0,1,1]=(0.688053165375)*x_ref[0]**o+(-0.154890421924)*x_ref[0]+(0.583684164319)*x_ref[1]**o+(-0.398082068217)*x_ref[1]+(-0.573142923099)*x_ref[2]**o+(-0.205271877337)*x_ref[2]
        ref[0,0,2,0]=(0.807043275877)*x_ref[0]**o+(0.843046383231)*x_ref[0]+(-0.588146471402)*x_ref[1]**o+(0.157787304557)*x_ref[1]+(-0.771009748568)*x_ref[2]**o+(0.162090791609)*x_ref[2]
        ref[0,0,2,1]=(-0.0706800489392)*x_ref[0]**o+(0.499961344665)*x_ref[0]+(0.208926178797)*x_ref[1]**o+(-0.7462472769)*x_ref[1]+(-0.105475405648)*x_ref[2]**o+(-0.18670229948)*x_ref[2]
        ref[0,1,0,0]=(-0.696657281943)*x_ref[0]**o+(-0.0251785345759)*x_ref[0]+(-0.579575530451)*x_ref[1]**o+(-0.0628378665725)*x_ref[1]+(0.668395441302)*x_ref[2]**o+(0.813478105092)*x_ref[2]
        ref[0,1,0,1]=(-0.574784314876)*x_ref[0]**o+(-0.321268045204)*x_ref[0]+(0.160704464213)*x_ref[1]**o+(-0.676016748103)*x_ref[1]+(0.918971102865)*x_ref[2]**o+(0.252082410081)*x_ref[2]
        ref[0,1,1,0]=(-0.905062846817)*x_ref[0]**o+(0.788906658477)*x_ref[0]+(-0.408807611936)*x_ref[1]**o+(0.114686186437)*x_ref[1]+(-0.654685568493)*x_ref[2]**o+(0.0879202139254)*x_ref[2]
        ref[0,1,1,1]=(-0.68020231443)*x_ref[0]**o+(-0.82049202902)*x_ref[0]+(-0.958352740187)*x_ref[1]**o+(-0.33201745802)*x_ref[1]+(0.539810363682)*x_ref[2]**o+(0.744988573087)*x_ref[2]
        ref[0,1,2,0]=(-0.960884744997)*x_ref[0]**o+(-0.704157132503)*x_ref[0]+(0.282236905414)*x_ref[1]**o+(0.756363456433)*x_ref[1]+(-0.365400101792)*x_ref[2]**o+(0.975735846554)*x_ref[2]
        ref[0,1,2,1]=(-0.691780076951)*x_ref[0]**o+(-0.952575440059)*x_ref[0]+(-0.921247808098)*x_ref[1]**o+(-0.88872834095)*x_ref[1]+(0.534103013833)*x_ref[2]**o+(-0.602295186104)*x_ref[2]
        ref[0,2,0,0]=(0.460698415704)*x_ref[0]**o+(0.770243814897)*x_ref[0]+(-0.380381727574)*x_ref[1]**o+(0.476360845668)*x_ref[1]+(-0.681040425102)*x_ref[2]**o+(-0.244369435116)*x_ref[2]
        ref[0,2,0,1]=(0.251591652281)*x_ref[0]**o+(0.748234919686)*x_ref[0]+(-0.698189037307)*x_ref[1]**o+(-0.292400412508)*x_ref[1]+(0.613850696577)*x_ref[2]**o+(-0.942243226834)*x_ref[2]
        ref[0,2,1,0]=(-0.718090066328)*x_ref[0]**o+(-0.322499511794)*x_ref[0]+(-0.75105623397)*x_ref[1]**o+(0.526680271832)*x_ref[1]+(-0.186671531793)*x_ref[2]**o+(-0.449273875991)*x_ref[2]
        ref[0,2,1,1]=(-0.52165552452)*x_ref[0]**o+(0.902446006085)*x_ref[0]+(-0.915575172702)*x_ref[1]**o+(-0.150816194662)*x_ref[1]+(0.596782785689)*x_ref[2]**o+(0.546880865642)*x_ref[2]
        ref[0,2,2,0]=(-0.104486704816)*x_ref[0]**o+(0.723863699457)*x_ref[0]+(0.758622117279)*x_ref[1]**o+(0.372271580145)*x_ref[1]+(-0.0820570383792)*x_ref[2]**o+(-0.63539781104)*x_ref[2]
        ref[0,2,2,1]=(0.0355930823603)*x_ref[0]**o+(0.0931942600432)*x_ref[0]+(-0.118096985978)*x_ref[1]**o+(0.296983782212)*x_ref[1]+(-0.864491763712)*x_ref[2]**o+(0.942230663978)*x_ref[2]
        ref[0,3,0,0]=(0.371153709815)*x_ref[0]**o+(0.0427309083893)*x_ref[0]+(-0.886563919084)*x_ref[1]**o+(-0.14946513404)*x_ref[1]+(0.80903804519)*x_ref[2]**o+(0.69728533073)*x_ref[2]
        ref[0,3,0,1]=(0.0593930396142)*x_ref[0]**o+(-0.370327974089)*x_ref[0]+(-0.388354464683)*x_ref[1]**o+(-0.367648771812)*x_ref[1]+(-0.573754821458)*x_ref[2]**o+(-0.947499155479)*x_ref[2]
        ref[0,3,1,0]=(-0.869817317)*x_ref[0]**o+(0.874317772678)*x_ref[0]+(-0.870060178375)*x_ref[1]**o+(0.66571074524)*x_ref[1]+(-0.669052377378)*x_ref[2]**o+(0.940212680575)*x_ref[2]
        ref[0,3,1,1]=(-0.417445606772)*x_ref[0]**o+(0.942606057619)*x_ref[0]+(-0.133936034409)*x_ref[1]**o+(-0.363274280866)*x_ref[1]+(0.83487439935)*x_ref[2]**o+(-0.108398981461)*x_ref[2]
        ref[0,3,2,0]=(0.363623383293)*x_ref[0]**o+(0.624267309217)*x_ref[0]+(0.801719984899)*x_ref[1]**o+(0.189212370122)*x_ref[1]+(0.464633630928)*x_ref[2]**o+(-0.979808318438)*x_ref[2]
        ref[0,3,2,1]=(0.85384408805)*x_ref[0]**o+(-0.889214754116)*x_ref[0]+(0.164915391224)*x_ref[1]**o+(0.0168991405201)*x_ref[1]+(0.097171994236)*x_ref[2]**o+(-0.146094197339)*x_ref[2]
        ref[0,4,0,0]=(0.778356158972)*x_ref[0]**o+(0.910241265149)*x_ref[0]+(0.123690863272)*x_ref[1]**o+(0.87802121779)*x_ref[1]+(-0.714615668595)*x_ref[2]**o+(-0.437472831583)*x_ref[2]
        ref[0,4,0,1]=(-0.157185481783)*x_ref[0]**o+(0.352802163968)*x_ref[0]+(0.390184123612)*x_ref[1]**o+(-0.680415093098)*x_ref[1]+(-0.121195042454)*x_ref[2]**o+(-0.713693712117)*x_ref[2]
        ref[0,4,1,0]=(0.778115668646)*x_ref[0]**o+(-0.111729042892)*x_ref[0]+(0.451518002735)*x_ref[1]**o+(0.232731183374)*x_ref[1]+(-0.764246158086)*x_ref[2]**o+(-0.795525155328)*x_ref[2]
        ref[0,4,1,1]=(-0.880652367027)*x_ref[0]**o+(-0.314966079126)*x_ref[0]+(-0.640720373065)*x_ref[1]**o+(0.336230541901)*x_ref[1]+(0.566956999526)*x_ref[2]**o+(0.74098258811)*x_ref[2]
        ref[0,4,2,0]=(0.908657274301)*x_ref[0]**o+(0.636427823614)*x_ref[0]+(0.650137414386)*x_ref[1]**o+(0.695558379005)*x_ref[1]+(-0.489852401087)*x_ref[2]**o+(-0.479227108044)*x_ref[2]
        ref[0,4,2,1]=(0.015734241525)*x_ref[0]**o+(0.0719884727088)*x_ref[0]+(0.811864895261)*x_ref[1]**o+(0.959127986784)*x_ref[1]+(-0.238146663368)*x_ref[2]**o+(-0.454939322571)*x_ref[2]
        ref[1,0,0,0]=(-0.918218246559)*x_ref[0]**o+(-0.662787169269)*x_ref[0]+(-0.954035144111)*x_ref[1]**o+(-0.647264685228)*x_ref[1]+(-0.834085960213)*x_ref[2]**o+(-0.167541951024)*x_ref[2]
        ref[1,0,0,1]=(-0.670950869382)*x_ref[0]**o+(0.826472159989)*x_ref[0]+(0.180410872786)*x_ref[1]**o+(-0.715162871594)*x_ref[1]+(0.653287853169)*x_ref[2]**o+(0.892060529625)*x_ref[2]
        ref[1,0,1,0]=(0.398789786407)*x_ref[0]**o+(0.689408033554)*x_ref[0]+(0.124338245128)*x_ref[1]**o+(0.748280808587)*x_ref[1]+(0.418357122274)*x_ref[2]**o+(0.230707440075)*x_ref[2]
        ref[1,0,1,1]=(-0.912778964055)*x_ref[0]**o+(0.904288708694)*x_ref[0]+(-0.874744856901)*x_ref[1]**o+(0.550162195719)*x_ref[1]+(-0.7526423373)*x_ref[2]**o+(0.820993231654)*x_ref[2]
        ref[1,0,2,0]=(-0.843459684648)*x_ref[0]**o+(-0.579708764091)*x_ref[0]+(-0.876072131849)*x_ref[1]**o+(0.933694599696)*x_ref[1]+(0.507781209021)*x_ref[2]**o+(-0.719729482133)*x_ref[2]
        ref[1,0,2,1]=(0.594566925796)*x_ref[0]**o+(-0.994301292358)*x_ref[0]+(0.808114389564)*x_ref[1]**o+(-0.970803712864)*x_ref[1]+(0.433544910389)*x_ref[2]**o+(0.260659440299)*x_ref[2]
        ref[1,1,0,0]=(0.997377553172)*x_ref[0]**o+(0.458428616383)*x_ref[0]+(0.357280778493)*x_ref[1]**o+(0.349837110741)*x_ref[1]+(-0.709592214445)*x_ref[2]**o+(-0.630000739033)*x_ref[2]
        ref[1,1,0,1]=(0.121595650501)*x_ref[0]**o+(0.40870120924)*x_ref[0]+(0.576917123807)*x_ref[1]**o+(-0.932268308325)*x_ref[1]+(0.710043472116)*x_ref[2]**o+(-0.716528412077)*x_ref[2]
        ref[1,1,1,0]=(-0.151416035033)*x_ref[0]**o+(0.0255347649787)*x_ref[0]+(-0.752987701889)*x_ref[1]**o+(-0.480969875813)*x_ref[1]+(-0.610514504179)*x_ref[2]**o+(0.959274757402)*x_ref[2]
        ref[1,1,1,1]=(0.76695706106)*x_ref[0]**o+(-0.0750965975512)*x_ref[0]+(0.846833900735)*x_ref[1]**o+(-0.105838465439)*x_ref[1]+(0.541027486457)*x_ref[2]**o+(0.152543017589)*x_ref[2]
        ref[1,1,2,0]=(0.0856977181865)*x_ref[0]**o+(-0.188139369706)*x_ref[0]+(0.936071565289)*x_ref[1]**o+(-0.3958713036)*x_ref[1]+(-0.350945878893)*x_ref[2]**o+(0.382302674883)*x_ref[2]
        ref[1,1,2,1]=(0.873054234789)*x_ref[0]**o+(-0.490869716146)*x_ref[0]+(0.46720526212)*x_ref[1]**o+(-0.334203412334)*x_ref[1]+(-0.81140895636)*x_ref[2]**o+(-0.973858971749)*x_ref[2]
        ref[1,2,0,0]=(-0.716676352443)*x_ref[0]**o+(0.659821396277)*x_ref[0]+(0.335964292954)*x_ref[1]**o+(0.11609664773)*x_ref[1]+(-0.889186749466)*x_ref[2]**o+(0.454231973946)*x_ref[2]
        ref[1,2,0,1]=(0.367048872648)*x_ref[0]**o+(0.276083713126)*x_ref[0]+(0.686639128043)*x_ref[1]**o+(-0.991414415293)*x_ref[1]+(0.25360515964)*x_ref[2]**o+(-0.377623315093)*x_ref[2]
        ref[1,2,1,0]=(-0.723089358777)*x_ref[0]**o+(-0.516690112406)*x_ref[0]+(0.262334273979)*x_ref[1]**o+(0.140773502537)*x_ref[1]+(0.447333972396)*x_ref[2]**o+(-0.551395692966)*x_ref[2]
        ref[1,2,1,1]=(0.738791182491)*x_ref[0]**o+(-0.978120626668)*x_ref[0]+(-0.217970433713)*x_ref[1]**o+(0.0181011067492)*x_ref[1]+(0.756322867523)*x_ref[2]**o+(0.495269195978)*x_ref[2]
        ref[1,2,2,0]=(-0.630506449854)*x_ref[0]**o+(-0.0865482364231)*x_ref[0]+(-0.415896403157)*x_ref[1]**o+(-0.712833149895)*x_ref[1]+(-0.180482789777)*x_ref[2]**o+(0.00788290661235)*x_ref[2]
        ref[1,2,2,1]=(0.0946913865377)*x_ref[0]**o+(0.752854827716)*x_ref[0]+(0.257846220843)*x_ref[1]**o+(-0.953926983578)*x_ref[1]+(-0.388505920682)*x_ref[2]**o+(-0.10751401251)*x_ref[2]
        ref[1,3,0,0]=(0.694014030611)*x_ref[0]**o+(0.0433103547797)*x_ref[0]+(-0.0805025225569)*x_ref[1]**o+(0.389052254754)*x_ref[1]+(-0.813210939614)*x_ref[2]**o+(-0.593788787896)*x_ref[2]
        ref[1,3,0,1]=(0.0101606353406)*x_ref[0]**o+(-0.849874258855)*x_ref[0]+(-0.0578973959381)*x_ref[1]**o+(0.942113140424)*x_ref[1]+(0.613716459216)*x_ref[2]**o+(0.72631176833)*x_ref[2]
        ref[1,3,1,0]=(0.264703608811)*x_ref[0]**o+(0.457962548326)*x_ref[0]+(-0.433795242673)*x_ref[1]**o+(0.650995712848)*x_ref[1]+(0.372471807967)*x_ref[2]**o+(0.00416636841047)*x_ref[2]
        ref[1,3,1,1]=(-0.73686451669)*x_ref[0]**o+(-0.0425614910776)*x_ref[0]+(-0.587820431311)*x_ref[1]**o+(-0.377628617833)*x_ref[1]+(-0.463964014406)*x_ref[2]**o+(0.283273874989)*x_ref[2]
        ref[1,3,2,0]=(-0.689341439143)*x_ref[0]**o+(-0.675861918889)*x_ref[0]+(-0.918622662352)*x_ref[1]**o+(0.740990261888)*x_ref[1]+(0.0454007677041)*x_ref[2]**o+(0.513623361074)*x_ref[2]
        ref[1,3,2,1]=(0.13229101848)*x_ref[0]**o+(0.171873799817)*x_ref[0]+(-0.154491174434)*x_ref[1]**o+(0.897976884089)*x_ref[1]+(-0.598886311821)*x_ref[2]**o+(0.738696710918)*x_ref[2]
        ref[1,4,0,0]=(0.59403636966)*x_ref[0]**o+(0.93856848969)*x_ref[0]+(0.80164849167)*x_ref[1]**o+(-0.753322979568)*x_ref[1]+(-0.107098073916)*x_ref[2]**o+(-0.623047487973)*x_ref[2]
        ref[1,4,0,1]=(0.291046099082)*x_ref[0]**o+(-0.437982471049)*x_ref[0]+(0.017395762947)*x_ref[1]**o+(0.852440593923)*x_ref[1]+(0.630825424359)*x_ref[2]**o+(0.419347491727)*x_ref[2]
        ref[1,4,1,0]=(-0.747962012701)*x_ref[0]**o+(-0.11622032094)*x_ref[0]+(0.324950549781)*x_ref[1]**o+(0.950911398327)*x_ref[1]+(-0.18098990205)*x_ref[2]**o+(-0.676060633522)*x_ref[2]
        ref[1,4,1,1]=(0.545224363128)*x_ref[0]**o+(0.753376367121)*x_ref[0]+(0.271426695753)*x_ref[1]**o+(-0.139309844675)*x_ref[1]+(-0.923434716375)*x_ref[2]**o+(-0.985988324389)*x_ref[2]
        ref[1,4,2,0]=(-0.532260822047)*x_ref[0]**o+(0.901407135724)*x_ref[0]+(0.937451330141)*x_ref[1]**o+(0.285217666559)*x_ref[1]+(-0.133028984824)*x_ref[2]**o+(0.993965133696)*x_ref[2]
        ref[1,4,2,1]=(0.2231815607)*x_ref[0]**o+(0.994845537582)*x_ref[0]+(0.0368727378195)*x_ref[1]**o+(0.511772933777)*x_ref[1]+(-0.871580215976)*x_ref[2]**o+(-0.587010057329)*x_ref[2]
        ref[2,0,0,0]=(-0.55149411913)*x_ref[0]**o+(-0.905541278559)*x_ref[0]+(0.461217373096)*x_ref[1]**o+(-0.53359853241)*x_ref[1]+(-0.666643290883)*x_ref[2]**o+(0.151495826926)*x_ref[2]
        ref[2,0,0,1]=(-0.980485062148)*x_ref[0]**o+(0.150165616162)*x_ref[0]+(0.59165725487)*x_ref[1]**o+(-0.243853536017)*x_ref[1]+(0.66816261686)*x_ref[2]**o+(0.2425696131)*x_ref[2]
        ref[2,0,1,0]=(-0.509337213758)*x_ref[0]**o+(0.652936706687)*x_ref[0]+(-0.849214338091)*x_ref[1]**o+(-0.523354123549)*x_ref[1]+(0.371515689887)*x_ref[2]**o+(0.799989815414)*x_ref[2]
        ref[2,0,1,1]=(0.461584544541)*x_ref[0]**o+(0.662931355954)*x_ref[0]+(0.744415634786)*x_ref[1]**o+(-0.016743801256)*x_ref[1]+(0.172945846325)*x_ref[2]**o+(-0.0889827089341)*x_ref[2]
        ref[2,0,2,0]=(0.506514791562)*x_ref[0]**o+(0.974477160694)*x_ref[0]+(-0.968485343167)*x_ref[1]**o+(0.427480013599)*x_ref[1]+(0.0460755879358)*x_ref[2]**o+(0.186500994948)*x_ref[2]
        ref[2,0,2,1]=(0.259831607993)*x_ref[0]**o+(-0.0141290444125)*x_ref[0]+(-0.968452177682)*x_ref[1]**o+(0.635619125009)*x_ref[1]+(-0.958848431827)*x_ref[2]**o+(-0.0977241523308)*x_ref[2]
        ref[2,1,0,0]=(0.204233222933)*x_ref[0]**o+(0.834739381877)*x_ref[0]+(0.19961185048)*x_ref[1]**o+(-0.238470380578)*x_ref[1]+(-0.749979713773)*x_ref[2]**o+(-0.268168886251)*x_ref[2]
        ref[2,1,0,1]=(0.998123643088)*x_ref[0]**o+(0.635823733041)*x_ref[0]+(-0.265447124419)*x_ref[1]**o+(-0.225582751946)*x_ref[1]+(0.91705937439)*x_ref[2]**o+(-0.739461510792)*x_ref[2]
        ref[2,1,1,0]=(0.0800715288646)*x_ref[0]**o+(-0.37677196659)*x_ref[0]+(-0.0737286350219)*x_ref[1]**o+(0.032076062933)*x_ref[1]+(0.245421912709)*x_ref[2]**o+(-0.8378023935)*x_ref[2]
        ref[2,1,1,1]=(-0.572376055603)*x_ref[0]**o+(0.8675468469)*x_ref[0]+(-0.533012889867)*x_ref[1]**o+(-0.247215658297)*x_ref[1]+(0.673144564693)*x_ref[2]**o+(-0.749136108985)*x_ref[2]
        ref[2,1,2,0]=(-0.661188192488)*x_ref[0]**o+(-0.924435593008)*x_ref[0]+(-0.791845808916)*x_ref[1]**o+(0.00133765351092)*x_ref[1]+(0.233676862873)*x_ref[2]**o+(0.981812737962)*x_ref[2]
        ref[2,1,2,1]=(0.332072445986)*x_ref[0]**o+(-0.909373798588)*x_ref[0]+(0.927025647819)*x_ref[1]**o+(-0.936348942617)*x_ref[1]+(0.251508592152)*x_ref[2]**o+(-0.468147973987)*x_ref[2]
        ref[2,2,0,0]=(0.892377893841)*x_ref[0]**o+(-0.147933293091)*x_ref[0]+(0.950491102076)*x_ref[1]**o+(0.595250355926)*x_ref[1]+(0.0169764649992)*x_ref[2]**o+(-0.683137752586)*x_ref[2]
        ref[2,2,0,1]=(0.00579397005645)*x_ref[0]**o+(0.627197373917)*x_ref[0]+(0.663574756196)*x_ref[1]**o+(0.934106767513)*x_ref[1]+(0.105844878022)*x_ref[2]**o+(0.513322917688)*x_ref[2]
        ref[2,2,1,0]=(-0.168776016133)*x_ref[0]**o+(-0.940071612944)*x_ref[0]+(0.0366053554566)*x_ref[1]**o+(-0.628010909771)*x_ref[1]+(0.250023045906)*x_ref[2]**o+(-0.00327322332166)*x_ref[2]
        ref[2,2,1,1]=(0.904155764376)*x_ref[0]**o+(-0.258018307306)*x_ref[0]+(0.65862820521)*x_ref[1]**o+(0.434480798515)*x_ref[1]+(0.4089419173)*x_ref[2]**o+(-0.518032695598)*x_ref[2]
        ref[2,2,2,0]=(-0.41470070958)*x_ref[0]**o+(-0.783186751952)*x_ref[0]+(0.548309561942)*x_ref[1]**o+(0.837130592803)*x_ref[1]+(-0.346808491848)*x_ref[2]**o+(0.165416040985)*x_ref[2]
        ref[2,2,2,1]=(0.997845909195)*x_ref[0]**o+(0.448428042158)*x_ref[0]+(-0.657050004311)*x_ref[1]**o+(0.493104851747)*x_ref[1]+(0.558315071197)*x_ref[2]**o+(0.206659432923)*x_ref[2]
        ref[2,3,0,0]=(0.433874360232)*x_ref[0]**o+(-0.0635646197131)*x_ref[0]+(-0.521690521523)*x_ref[1]**o+(-0.298130684888)*x_ref[1]+(-0.131905313644)*x_ref[2]**o+(0.64389451112)*x_ref[2]
        ref[2,3,0,1]=(0.457729315216)*x_ref[0]**o+(-0.419323242135)*x_ref[0]+(0.614544051135)*x_ref[1]**o+(-0.701671999275)*x_ref[1]+(0.905494300727)*x_ref[2]**o+(-0.565716985337)*x_ref[2]
        ref[2,3,1,0]=(0.256570728097)*x_ref[0]**o+(0.866349285103)*x_ref[0]+(-0.115768354896)*x_ref[1]**o+(-0.613044456549)*x_ref[1]+(-0.20074185156)*x_ref[2]**o+(-0.188403571776)*x_ref[2]
        ref[2,3,1,1]=(0.767479943414)*x_ref[0]**o+(0.884066215007)*x_ref[0]+(0.363855420056)*x_ref[1]**o+(0.919698981104)*x_ref[1]+(0.225747762317)*x_ref[2]**o+(-0.220362271104)*x_ref[2]
        ref[2,3,2,0]=(-0.746469501382)*x_ref[0]**o+(0.680520398001)*x_ref[0]+(0.926950996325)*x_ref[1]**o+(-0.122831303044)*x_ref[1]+(-0.27226561675)*x_ref[2]**o+(0.933304087951)*x_ref[2]
        ref[2,3,2,1]=(-0.0961434747216)*x_ref[0]**o+(0.987660516767)*x_ref[0]+(0.288175205045)*x_ref[1]**o+(0.683720820403)*x_ref[1]+(0.354403143642)*x_ref[2]**o+(-0.745694089132)*x_ref[2]
        ref[2,4,0,0]=(0.921750835986)*x_ref[0]**o+(-0.0358861598394)*x_ref[0]+(0.190129645259)*x_ref[1]**o+(0.377067803331)*x_ref[1]+(-0.0188615236582)*x_ref[2]**o+(0.804533888824)*x_ref[2]
        ref[2,4,0,1]=(-0.256132234484)*x_ref[0]**o+(-0.102661411017)*x_ref[0]+(-0.903465157009)*x_ref[1]**o+(0.50133310168)*x_ref[1]+(-0.0889102598619)*x_ref[2]**o+(-0.0310174238365)*x_ref[2]
        ref[2,4,1,0]=(0.58483798855)*x_ref[0]**o+(0.967943344305)*x_ref[0]+(-0.0701080005438)*x_ref[1]**o+(0.489476104406)*x_ref[1]+(0.615308427038)*x_ref[2]**o+(-0.38065505709)*x_ref[2]
        ref[2,4,1,1]=(0.647734130724)*x_ref[0]**o+(-0.930934392719)*x_ref[0]+(0.267370540491)*x_ref[1]**o+(-0.0175019424556)*x_ref[1]+(0.38593657129)*x_ref[2]**o+(-0.985922292359)*x_ref[2]
        ref[2,4,2,0]=(-0.289971479933)*x_ref[0]**o+(0.882014313157)*x_ref[0]+(0.147321067579)*x_ref[1]**o+(-0.176771838489)*x_ref[1]+(0.710916531544)*x_ref[2]**o+(-0.0856290170401)*x_ref[2]
        ref[2,4,2,1]=(-0.465930997733)*x_ref[0]**o+(-0.305563942094)*x_ref[0]+(-0.824535262061)*x_ref[1]**o+(-0.917892226972)*x_ref[1]+(0.624462871955)*x_ref[2]**o+(0.544736500065)*x_ref[2]
        ref[3,0,0,0]=(-0.673230815097)*x_ref[0]**o+(-0.726292712917)*x_ref[0]+(0.0531037021688)*x_ref[1]**o+(-0.467219030159)*x_ref[1]+(-0.0729861369718)*x_ref[2]**o+(-0.440394335015)*x_ref[2]
        ref[3,0,0,1]=(-0.358235295558)*x_ref[0]**o+(-0.301184713671)*x_ref[0]+(-0.496030649086)*x_ref[1]**o+(-0.125922184777)*x_ref[1]+(0.946715036973)*x_ref[2]**o+(0.846370337893)*x_ref[2]
        ref[3,0,1,0]=(-0.394175109612)*x_ref[0]**o+(-0.85137582307)*x_ref[0]+(-0.667804785044)*x_ref[1]**o+(0.156762109891)*x_ref[1]+(-0.616444700525)*x_ref[2]**o+(0.0738729399356)*x_ref[2]
        ref[3,0,1,1]=(-0.652875462534)*x_ref[0]**o+(-0.0605650192541)*x_ref[0]+(0.840539933823)*x_ref[1]**o+(-0.422819423301)*x_ref[1]+(0.113440226819)*x_ref[2]**o+(-0.915154532306)*x_ref[2]
        ref[3,0,2,0]=(-0.162863352328)*x_ref[0]**o+(-0.364461332292)*x_ref[0]+(-0.222181741398)*x_ref[1]**o+(-0.563668008365)*x_ref[1]+(0.118491770105)*x_ref[2]**o+(0.0809155053022)*x_ref[2]
        ref[3,0,2,1]=(-0.985278929437)*x_ref[0]**o+(-0.919440760157)*x_ref[0]+(-0.776910686)*x_ref[1]**o+(-0.884270488943)*x_ref[1]+(0.984279010363)*x_ref[2]**o+(-0.489503278582)*x_ref[2]
        ref[3,1,0,0]=(-0.806565449823)*x_ref[0]**o+(-0.0851385375281)*x_ref[0]+(-0.88851381773)*x_ref[1]**o+(-0.666539867528)*x_ref[1]+(0.154825531411)*x_ref[2]**o+(0.764069849144)*x_ref[2]
        ref[3,1,0,1]=(-0.222125783918)*x_ref[0]**o+(0.568035675467)*x_ref[0]+(0.470171778932)*x_ref[1]**o+(-0.307608710881)*x_ref[1]+(-0.836083742471)*x_ref[2]**o+(0.553597553381)*x_ref[2]
        ref[3,1,1,0]=(0.431237389847)*x_ref[0]**o+(-0.141391012471)*x_ref[0]+(0.726895025519)*x_ref[1]**o+(-0.527868751679)*x_ref[1]+(0.711761083137)*x_ref[2]**o+(0.996992864103)*x_ref[2]
        ref[3,1,1,1]=(-0.123885959924)*x_ref[0]**o+(0.877034475529)*x_ref[0]+(-0.0432492682928)*x_ref[1]**o+(-0.370315494618)*x_ref[1]+(0.243139719071)*x_ref[2]**o+(-0.684318634902)*x_ref[2]
        ref[3,1,2,0]=(-0.477214489355)*x_ref[0]**o+(0.340808576619)*x_ref[0]+(-0.0991802306558)*x_ref[1]**o+(0.462599821908)*x_ref[1]+(0.091768281564)*x_ref[2]**o+(0.0315552838839)*x_ref[2]
        ref[3,1,2,1]=(0.676189142894)*x_ref[0]**o+(0.510136386913)*x_ref[0]+(-0.558579041952)*x_ref[1]**o+(0.961615047877)*x_ref[1]+(0.510717386124)*x_ref[2]**o+(0.774430183432)*x_ref[2]
        ref[3,2,0,0]=(0.158370232444)*x_ref[0]**o+(0.209424855992)*x_ref[0]+(0.691438753013)*x_ref[1]**o+(-0.781594298738)*x_ref[1]+(-0.0188536627517)*x_ref[2]**o+(0.136400798995)*x_ref[2]
        ref[3,2,0,1]=(-0.440674414328)*x_ref[0]**o+(0.118228612687)*x_ref[0]+(-0.700026492613)*x_ref[1]**o+(-0.981341215714)*x_ref[1]+(-0.172847519578)*x_ref[2]**o+(-0.423095202172)*x_ref[2]
        ref[3,2,1,0]=(-0.0694960391695)*x_ref[0]**o+(-0.66448610135)*x_ref[0]+(-0.177572703748)*x_ref[1]**o+(-0.060488161727)*x_ref[1]+(0.306021853885)*x_ref[2]**o+(0.0730338716187)*x_ref[2]
        ref[3,2,1,1]=(-0.360662154547)*x_ref[0]**o+(0.577679359648)*x_ref[0]+(-0.515313534966)*x_ref[1]**o+(0.217352687358)*x_ref[1]+(0.728367577882)*x_ref[2]**o+(0.38505849834)*x_ref[2]
        ref[3,2,2,0]=(0.568671861232)*x_ref[0]**o+(-0.0792490917882)*x_ref[0]+(0.0188002885229)*x_ref[1]**o+(-0.956862575759)*x_ref[1]+(-0.844332442983)*x_ref[2]**o+(0.670792928042)*x_ref[2]
        ref[3,2,2,1]=(0.833099204713)*x_ref[0]**o+(0.368482169933)*x_ref[0]+(-0.926002498892)*x_ref[1]**o+(-0.130646446568)*x_ref[1]+(0.81164928914)*x_ref[2]**o+(-0.641079919927)*x_ref[2]
        ref[3,3,0,0]=(-0.277769350989)*x_ref[0]**o+(0.753452018649)*x_ref[0]+(-0.766322344505)*x_ref[1]**o+(-0.679108224612)*x_ref[1]+(0.0799499795424)*x_ref[2]**o+(0.255751308975)*x_ref[2]
        ref[3,3,0,1]=(0.429176753733)*x_ref[0]**o+(0.123106050259)*x_ref[0]+(-0.169037728992)*x_ref[1]**o+(0.749245933687)*x_ref[1]+(0.560140859355)*x_ref[2]**o+(-0.937091840021)*x_ref[2]
        ref[3,3,1,0]=(0.648792557593)*x_ref[0]**o+(-0.057555187513)*x_ref[0]+(0.883290331726)*x_ref[1]**o+(0.539276905995)*x_ref[1]+(0.473614408583)*x_ref[2]**o+(0.723258340333)*x_ref[2]
        ref[3,3,1,1]=(0.527298833572)*x_ref[0]**o+(0.0753824058279)*x_ref[0]+(0.719196099257)*x_ref[1]**o+(-0.0124603350035)*x_ref[1]+(-0.964484650253)*x_ref[2]**o+(0.404444269694)*x_ref[2]
        ref[3,3,2,0]=(-0.403640677462)*x_ref[0]**o+(0.906195964766)*x_ref[0]+(-0.561419078404)*x_ref[1]**o+(0.645408717838)*x_ref[1]+(0.671624799609)*x_ref[2]**o+(-0.573901868621)*x_ref[2]
        ref[3,3,2,1]=(-0.288117182165)*x_ref[0]**o+(-0.391003584945)*x_ref[0]+(-0.555037182038)*x_ref[1]**o+(0.299362209017)*x_ref[1]+(-0.552429724326)*x_ref[2]**o+(-0.416923306774)*x_ref[2]
        ref[3,4,0,0]=(-0.19594719192)*x_ref[0]**o+(0.150898934692)*x_ref[0]+(0.210935232044)*x_ref[1]**o+(-0.319764293347)*x_ref[1]+(-0.642450751195)*x_ref[2]**o+(0.22231085168)*x_ref[2]
        ref[3,4,0,1]=(-0.0514589460225)*x_ref[0]**o+(0.0366025646754)*x_ref[0]+(0.416280576691)*x_ref[1]**o+(0.609178063951)*x_ref[1]+(0.835611677062)*x_ref[2]**o+(-0.131630571478)*x_ref[2]
        ref[3,4,1,0]=(0.171509152214)*x_ref[0]**o+(-0.761188271093)*x_ref[0]+(-0.915951802896)*x_ref[1]**o+(-0.451796709915)*x_ref[1]+(-0.979847828162)*x_ref[2]**o+(0.0566453393315)*x_ref[2]
        ref[3,4,1,1]=(0.690427998102)*x_ref[0]**o+(-0.358318625695)*x_ref[0]+(0.330270871613)*x_ref[1]**o+(-0.724392522137)*x_ref[1]+(-0.535474623489)*x_ref[2]**o+(-0.12341743885)*x_ref[2]
        ref[3,4,2,0]=(0.488090753426)*x_ref[0]**o+(-0.352417078372)*x_ref[0]+(0.971988099319)*x_ref[1]**o+(-0.288929516009)*x_ref[1]+(0.612576734094)*x_ref[2]**o+(-0.43799514603)*x_ref[2]
        ref[3,4,2,1]=(-0.098768101356)*x_ref[0]**o+(0.123300661895)*x_ref[0]+(-0.0489724280789)*x_ref[1]**o+(-0.422508669137)*x_ref[1]+(-0.861483104139)*x_ref[2]**o+(0.950198316001)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_Solution_rank0(self):
      """
      tests interpolate for rank 0 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from Solution onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Data(0,(),w)
      ref=Data(0,(),w_ref)
      if dim==2:
        arg=(0.724373964817)*x[0]**o+(-0.0560850774901)*x[0]+(0.201815078281)*x[1]**o+(-0.721683697048)*x[1]
        ref=(0.724373964817)*x_ref[0]**o+(-0.0560850774901)*x_ref[0]+(0.201815078281)*x_ref[1]**o+(-0.721683697048)*x_ref[1]
      else:
        arg=(0.390296010052)*x[0]**o+(-0.279464234123)*x[0]+(-0.791334947488)*x[1]**o+(-0.0210629346704)*x[1]+(0.0816556526012)*x[2]**o+(0.77326269137)*x[2]
        ref=(0.390296010052)*x_ref[0]**o+(-0.279464234123)*x_ref[0]+(-0.791334947488)*x_ref[1]**o+(-0.0210629346704)*x_ref[1]+(0.0816556526012)*x_ref[2]**o+(0.77326269137)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_Solution_rank1(self):
      """
      tests interpolate for rank 1 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from Solution onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Data(0,(2,),w)
      ref=Data(0,(2,),w_ref)
      if dim==2:
        arg[0]=(0.645585823917)*x[0]**o+(-0.203443728436)*x[0]+(0.278126891692)*x[1]**o+(0.922093564911)*x[1]
        arg[1]=(0.23370137029)*x[0]**o+(0.84371173541)*x[0]+(0.99297574548)*x[1]**o+(0.72481785931)*x[1]
        ref[0]=(0.645585823917)*x_ref[0]**o+(-0.203443728436)*x_ref[0]+(0.278126891692)*x_ref[1]**o+(0.922093564911)*x_ref[1]
        ref[1]=(0.23370137029)*x_ref[0]**o+(0.84371173541)*x_ref[0]+(0.99297574548)*x_ref[1]**o+(0.72481785931)*x_ref[1]
      else:
        arg[0]=(0.159998049951)*x[0]**o+(-0.863367327023)*x[0]+(-0.0628498246638)*x[1]**o+(0.685139358273)*x[1]+(0.783213360641)*x[2]**o+(0.397526421338)*x[2]
        arg[1]=(0.165519279053)*x[0]**o+(-0.0936018967661)*x[0]+(0.863047696619)*x[1]**o+(0.653118186434)*x[1]+(-0.408224131944)*x[2]**o+(0.82400132854)*x[2]
        ref[0]=(0.159998049951)*x_ref[0]**o+(-0.863367327023)*x_ref[0]+(-0.0628498246638)*x_ref[1]**o+(0.685139358273)*x_ref[1]+(0.783213360641)*x_ref[2]**o+(0.397526421338)*x_ref[2]
        ref[1]=(0.165519279053)*x_ref[0]**o+(-0.0936018967661)*x_ref[0]+(0.863047696619)*x_ref[1]**o+(0.653118186434)*x_ref[1]+(-0.408224131944)*x_ref[2]**o+(0.82400132854)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_Solution_rank2(self):
      """
      tests interpolate for rank 2 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from Solution onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5),w)
      ref=Data(0,(4, 5),w_ref)
      if dim==2:
        arg[0,0]=(0.490709482882)*x[0]**o+(0.518382584887)*x[0]+(-0.940707425885)*x[1]**o+(0.224578726673)*x[1]
        arg[0,1]=(-0.747621141516)*x[0]**o+(-0.217452489097)*x[0]+(0.929120128008)*x[1]**o+(0.55263700209)*x[1]
        arg[0,2]=(-0.183545119672)*x[0]**o+(0.148192386895)*x[0]+(-0.22046309708)*x[1]**o+(0.433620446668)*x[1]
        arg[0,3]=(-0.0724840266959)*x[0]**o+(0.835144620602)*x[0]+(-0.962971534144)*x[1]**o+(-0.149681811534)*x[1]
        arg[0,4]=(-0.380150192228)*x[0]**o+(-0.309494603814)*x[0]+(-0.217425635182)*x[1]**o+(0.93567286482)*x[1]
        arg[1,0]=(0.0367921563947)*x[0]**o+(0.220217625576)*x[0]+(-0.120595819238)*x[1]**o+(0.308108793279)*x[1]
        arg[1,1]=(-0.142860237109)*x[0]**o+(-0.789758888253)*x[0]+(0.748085258229)*x[1]**o+(0.784767485883)*x[1]
        arg[1,2]=(-0.154690818845)*x[0]**o+(-0.678458370784)*x[0]+(0.750742996193)*x[1]**o+(-0.350199095531)*x[1]
        arg[1,3]=(-0.893295215013)*x[0]**o+(0.0295805814985)*x[0]+(0.265893730887)*x[1]**o+(0.435487860418)*x[1]
        arg[1,4]=(-0.302701704727)*x[0]**o+(0.618274652266)*x[0]+(-0.688386407308)*x[1]**o+(-0.832515064466)*x[1]
        arg[2,0]=(-0.926658745469)*x[0]**o+(0.548174350496)*x[0]+(-0.350700366641)*x[1]**o+(0.530188602011)*x[1]
        arg[2,1]=(0.900965385167)*x[0]**o+(0.0234051845124)*x[0]+(-0.00783834941699)*x[1]**o+(-0.766352156598)*x[1]
        arg[2,2]=(0.694733282444)*x[0]**o+(-0.0644519234093)*x[0]+(0.0493475528391)*x[1]**o+(-0.589101546403)*x[1]
        arg[2,3]=(-0.100333102737)*x[0]**o+(-0.633740106784)*x[0]+(0.408697826034)*x[1]**o+(-0.429059439472)*x[1]
        arg[2,4]=(0.635563324404)*x[0]**o+(0.955881346028)*x[0]+(0.0755013269452)*x[1]**o+(-0.743681282337)*x[1]
        arg[3,0]=(0.604891134834)*x[0]**o+(0.942047409702)*x[0]+(-0.00544373299112)*x[1]**o+(0.447523533166)*x[1]
        arg[3,1]=(-0.980967446837)*x[0]**o+(0.019824933239)*x[0]+(-0.826876930392)*x[1]**o+(0.113548968693)*x[1]
        arg[3,2]=(-0.731417395373)*x[0]**o+(-0.275072472108)*x[0]+(0.933867711492)*x[1]**o+(-0.444847972819)*x[1]
        arg[3,3]=(0.00121172413626)*x[0]**o+(-0.171073908218)*x[0]+(0.913182873344)*x[1]**o+(0.960256570926)*x[1]
        arg[3,4]=(-0.395134372727)*x[0]**o+(0.646385082409)*x[0]+(0.17387019219)*x[1]**o+(-0.287550462397)*x[1]
        ref[0,0]=(0.490709482882)*x_ref[0]**o+(0.518382584887)*x_ref[0]+(-0.940707425885)*x_ref[1]**o+(0.224578726673)*x_ref[1]
        ref[0,1]=(-0.747621141516)*x_ref[0]**o+(-0.217452489097)*x_ref[0]+(0.929120128008)*x_ref[1]**o+(0.55263700209)*x_ref[1]
        ref[0,2]=(-0.183545119672)*x_ref[0]**o+(0.148192386895)*x_ref[0]+(-0.22046309708)*x_ref[1]**o+(0.433620446668)*x_ref[1]
        ref[0,3]=(-0.0724840266959)*x_ref[0]**o+(0.835144620602)*x_ref[0]+(-0.962971534144)*x_ref[1]**o+(-0.149681811534)*x_ref[1]
        ref[0,4]=(-0.380150192228)*x_ref[0]**o+(-0.309494603814)*x_ref[0]+(-0.217425635182)*x_ref[1]**o+(0.93567286482)*x_ref[1]
        ref[1,0]=(0.0367921563947)*x_ref[0]**o+(0.220217625576)*x_ref[0]+(-0.120595819238)*x_ref[1]**o+(0.308108793279)*x_ref[1]
        ref[1,1]=(-0.142860237109)*x_ref[0]**o+(-0.789758888253)*x_ref[0]+(0.748085258229)*x_ref[1]**o+(0.784767485883)*x_ref[1]
        ref[1,2]=(-0.154690818845)*x_ref[0]**o+(-0.678458370784)*x_ref[0]+(0.750742996193)*x_ref[1]**o+(-0.350199095531)*x_ref[1]
        ref[1,3]=(-0.893295215013)*x_ref[0]**o+(0.0295805814985)*x_ref[0]+(0.265893730887)*x_ref[1]**o+(0.435487860418)*x_ref[1]
        ref[1,4]=(-0.302701704727)*x_ref[0]**o+(0.618274652266)*x_ref[0]+(-0.688386407308)*x_ref[1]**o+(-0.832515064466)*x_ref[1]
        ref[2,0]=(-0.926658745469)*x_ref[0]**o+(0.548174350496)*x_ref[0]+(-0.350700366641)*x_ref[1]**o+(0.530188602011)*x_ref[1]
        ref[2,1]=(0.900965385167)*x_ref[0]**o+(0.0234051845124)*x_ref[0]+(-0.00783834941699)*x_ref[1]**o+(-0.766352156598)*x_ref[1]
        ref[2,2]=(0.694733282444)*x_ref[0]**o+(-0.0644519234093)*x_ref[0]+(0.0493475528391)*x_ref[1]**o+(-0.589101546403)*x_ref[1]
        ref[2,3]=(-0.100333102737)*x_ref[0]**o+(-0.633740106784)*x_ref[0]+(0.408697826034)*x_ref[1]**o+(-0.429059439472)*x_ref[1]
        ref[2,4]=(0.635563324404)*x_ref[0]**o+(0.955881346028)*x_ref[0]+(0.0755013269452)*x_ref[1]**o+(-0.743681282337)*x_ref[1]
        ref[3,0]=(0.604891134834)*x_ref[0]**o+(0.942047409702)*x_ref[0]+(-0.00544373299112)*x_ref[1]**o+(0.447523533166)*x_ref[1]
        ref[3,1]=(-0.980967446837)*x_ref[0]**o+(0.019824933239)*x_ref[0]+(-0.826876930392)*x_ref[1]**o+(0.113548968693)*x_ref[1]
        ref[3,2]=(-0.731417395373)*x_ref[0]**o+(-0.275072472108)*x_ref[0]+(0.933867711492)*x_ref[1]**o+(-0.444847972819)*x_ref[1]
        ref[3,3]=(0.00121172413626)*x_ref[0]**o+(-0.171073908218)*x_ref[0]+(0.913182873344)*x_ref[1]**o+(0.960256570926)*x_ref[1]
        ref[3,4]=(-0.395134372727)*x_ref[0]**o+(0.646385082409)*x_ref[0]+(0.17387019219)*x_ref[1]**o+(-0.287550462397)*x_ref[1]
      else:
        arg[0,0]=(0.386545340192)*x[0]**o+(-0.397636296285)*x[0]+(-0.849437816516)*x[1]**o+(-0.83083406493)*x[1]+(0.479216612031)*x[2]**o+(0.0903787541308)*x[2]
        arg[0,1]=(-0.331651855726)*x[0]**o+(0.849421626918)*x[0]+(0.528368477362)*x[1]**o+(-0.015122214824)*x[1]+(0.494310578452)*x[2]**o+(0.190239928161)*x[2]
        arg[0,2]=(-0.662389759646)*x[0]**o+(0.204378343962)*x[0]+(0.486802744897)*x[1]**o+(0.822174784922)*x[1]+(-0.309203812647)*x[2]**o+(-0.101665040799)*x[2]
        arg[0,3]=(0.609106341828)*x[0]**o+(0.658404088333)*x[0]+(-0.0092991120732)*x[1]**o+(-0.73794934348)*x[1]+(0.270245135914)*x[2]**o+(-0.594067838407)*x[2]
        arg[0,4]=(-0.291752213357)*x[0]**o+(-0.596751343066)*x[0]+(-0.109196903078)*x[1]**o+(-0.868610307708)*x[1]+(0.622541308544)*x[2]**o+(0.482202762224)*x[2]
        arg[1,0]=(-0.113988933227)*x[0]**o+(-0.846328804695)*x[0]+(0.691769553626)*x[1]**o+(-0.377441645148)*x[1]+(-0.769857970682)*x[2]**o+(-0.583083729037)*x[2]
        arg[1,1]=(-0.635333757145)*x[0]**o+(0.194761117849)*x[0]+(-0.268287296731)*x[1]**o+(0.463906930647)*x[1]+(0.768293821029)*x[2]**o+(-0.532824906687)*x[2]
        arg[1,2]=(0.355380090138)*x[0]**o+(-0.181191575848)*x[0]+(-0.558667766467)*x[1]**o+(0.561707671872)*x[1]+(-0.427004232725)*x[2]**o+(-0.996564299989)*x[2]
        arg[1,3]=(-0.891638892893)*x[0]**o+(0.0867350850512)*x[0]+(0.880258390584)*x[1]**o+(-0.798123407525)*x[1]+(-0.571505202201)*x[2]**o+(0.35374245063)*x[2]
        arg[1,4]=(0.543632496941)*x[0]**o+(-0.630363803582)*x[0]+(0.38107312058)*x[1]**o+(0.222401831883)*x[1]+(-0.932212080175)*x[2]**o+(-0.0482899826174)*x[2]
        arg[2,0]=(0.46514308325)*x[0]**o+(-0.907464652384)*x[0]+(-0.706105595148)*x[1]**o+(0.486989958675)*x[1]+(0.422702683485)*x[2]**o+(0.640916925932)*x[2]
        arg[2,1]=(0.912808317479)*x[0]**o+(-0.860779312763)*x[0]+(-0.915364956133)*x[1]**o+(-0.314186117224)*x[1]+(-0.860814410035)*x[2]**o+(0.837570442722)*x[2]
        arg[2,2]=(0.00932917919464)*x[0]**o+(0.393773283049)*x[0]+(0.699998572668)*x[1]**o+(0.153407885795)*x[1]+(-0.166245189281)*x[2]**o+(-0.430541214236)*x[2]
        arg[2,3]=(-0.00127511229816)*x[0]**o+(-0.327443174924)*x[0]+(0.61881322537)*x[1]**o+(-0.725854102962)*x[1]+(0.650649062427)*x[2]**o+(0.246373184041)*x[2]
        arg[2,4]=(-0.530580898049)*x[0]**o+(-0.208506132901)*x[0]+(-0.102049332015)*x[1]**o+(-0.537037644189)*x[1]+(0.740113470729)*x[2]**o+(0.12094405264)*x[2]
        arg[3,0]=(-0.103423602042)*x[0]**o+(-0.535742323666)*x[0]+(0.678663019405)*x[1]**o+(0.579375950057)*x[1]+(0.323627490052)*x[2]**o+(-0.681323301356)*x[2]
        arg[3,1]=(0.670239087519)*x[0]**o+(0.209657968128)*x[0]+(-0.357490300727)*x[1]**o+(-0.441776782321)*x[1]+(0.00927389037562)*x[2]**o+(0.633527599723)*x[2]
        arg[3,2]=(0.415569606138)*x[0]**o+(-0.422211616068)*x[0]+(-0.695086235645)*x[1]**o+(0.709158289848)*x[1]+(-0.523450845183)*x[2]**o+(0.555778994398)*x[2]
        arg[3,3]=(0.0188993892075)*x[0]**o+(0.611088737828)*x[0]+(0.0313795167921)*x[1]**o+(0.0842651755016)*x[1]+(0.288414199415)*x[2]**o+(0.695910780235)*x[2]
        arg[3,4]=(-0.500338800879)*x[0]**o+(0.0984192369328)*x[0]+(0.71163172992)*x[1]**o+(-0.102123116139)*x[1]+(0.954613955678)*x[2]**o+(0.384872409422)*x[2]
        ref[0,0]=(0.386545340192)*x_ref[0]**o+(-0.397636296285)*x_ref[0]+(-0.849437816516)*x_ref[1]**o+(-0.83083406493)*x_ref[1]+(0.479216612031)*x_ref[2]**o+(0.0903787541308)*x_ref[2]
        ref[0,1]=(-0.331651855726)*x_ref[0]**o+(0.849421626918)*x_ref[0]+(0.528368477362)*x_ref[1]**o+(-0.015122214824)*x_ref[1]+(0.494310578452)*x_ref[2]**o+(0.190239928161)*x_ref[2]
        ref[0,2]=(-0.662389759646)*x_ref[0]**o+(0.204378343962)*x_ref[0]+(0.486802744897)*x_ref[1]**o+(0.822174784922)*x_ref[1]+(-0.309203812647)*x_ref[2]**o+(-0.101665040799)*x_ref[2]
        ref[0,3]=(0.609106341828)*x_ref[0]**o+(0.658404088333)*x_ref[0]+(-0.0092991120732)*x_ref[1]**o+(-0.73794934348)*x_ref[1]+(0.270245135914)*x_ref[2]**o+(-0.594067838407)*x_ref[2]
        ref[0,4]=(-0.291752213357)*x_ref[0]**o+(-0.596751343066)*x_ref[0]+(-0.109196903078)*x_ref[1]**o+(-0.868610307708)*x_ref[1]+(0.622541308544)*x_ref[2]**o+(0.482202762224)*x_ref[2]
        ref[1,0]=(-0.113988933227)*x_ref[0]**o+(-0.846328804695)*x_ref[0]+(0.691769553626)*x_ref[1]**o+(-0.377441645148)*x_ref[1]+(-0.769857970682)*x_ref[2]**o+(-0.583083729037)*x_ref[2]
        ref[1,1]=(-0.635333757145)*x_ref[0]**o+(0.194761117849)*x_ref[0]+(-0.268287296731)*x_ref[1]**o+(0.463906930647)*x_ref[1]+(0.768293821029)*x_ref[2]**o+(-0.532824906687)*x_ref[2]
        ref[1,2]=(0.355380090138)*x_ref[0]**o+(-0.181191575848)*x_ref[0]+(-0.558667766467)*x_ref[1]**o+(0.561707671872)*x_ref[1]+(-0.427004232725)*x_ref[2]**o+(-0.996564299989)*x_ref[2]
        ref[1,3]=(-0.891638892893)*x_ref[0]**o+(0.0867350850512)*x_ref[0]+(0.880258390584)*x_ref[1]**o+(-0.798123407525)*x_ref[1]+(-0.571505202201)*x_ref[2]**o+(0.35374245063)*x_ref[2]
        ref[1,4]=(0.543632496941)*x_ref[0]**o+(-0.630363803582)*x_ref[0]+(0.38107312058)*x_ref[1]**o+(0.222401831883)*x_ref[1]+(-0.932212080175)*x_ref[2]**o+(-0.0482899826174)*x_ref[2]
        ref[2,0]=(0.46514308325)*x_ref[0]**o+(-0.907464652384)*x_ref[0]+(-0.706105595148)*x_ref[1]**o+(0.486989958675)*x_ref[1]+(0.422702683485)*x_ref[2]**o+(0.640916925932)*x_ref[2]
        ref[2,1]=(0.912808317479)*x_ref[0]**o+(-0.860779312763)*x_ref[0]+(-0.915364956133)*x_ref[1]**o+(-0.314186117224)*x_ref[1]+(-0.860814410035)*x_ref[2]**o+(0.837570442722)*x_ref[2]
        ref[2,2]=(0.00932917919464)*x_ref[0]**o+(0.393773283049)*x_ref[0]+(0.699998572668)*x_ref[1]**o+(0.153407885795)*x_ref[1]+(-0.166245189281)*x_ref[2]**o+(-0.430541214236)*x_ref[2]
        ref[2,3]=(-0.00127511229816)*x_ref[0]**o+(-0.327443174924)*x_ref[0]+(0.61881322537)*x_ref[1]**o+(-0.725854102962)*x_ref[1]+(0.650649062427)*x_ref[2]**o+(0.246373184041)*x_ref[2]
        ref[2,4]=(-0.530580898049)*x_ref[0]**o+(-0.208506132901)*x_ref[0]+(-0.102049332015)*x_ref[1]**o+(-0.537037644189)*x_ref[1]+(0.740113470729)*x_ref[2]**o+(0.12094405264)*x_ref[2]
        ref[3,0]=(-0.103423602042)*x_ref[0]**o+(-0.535742323666)*x_ref[0]+(0.678663019405)*x_ref[1]**o+(0.579375950057)*x_ref[1]+(0.323627490052)*x_ref[2]**o+(-0.681323301356)*x_ref[2]
        ref[3,1]=(0.670239087519)*x_ref[0]**o+(0.209657968128)*x_ref[0]+(-0.357490300727)*x_ref[1]**o+(-0.441776782321)*x_ref[1]+(0.00927389037562)*x_ref[2]**o+(0.633527599723)*x_ref[2]
        ref[3,2]=(0.415569606138)*x_ref[0]**o+(-0.422211616068)*x_ref[0]+(-0.695086235645)*x_ref[1]**o+(0.709158289848)*x_ref[1]+(-0.523450845183)*x_ref[2]**o+(0.555778994398)*x_ref[2]
        ref[3,3]=(0.0188993892075)*x_ref[0]**o+(0.611088737828)*x_ref[0]+(0.0313795167921)*x_ref[1]**o+(0.0842651755016)*x_ref[1]+(0.288414199415)*x_ref[2]**o+(0.695910780235)*x_ref[2]
        ref[3,4]=(-0.500338800879)*x_ref[0]**o+(0.0984192369328)*x_ref[0]+(0.71163172992)*x_ref[1]**o+(-0.102123116139)*x_ref[1]+(0.954613955678)*x_ref[2]**o+(0.384872409422)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_Solution_rank3(self):
      """
      tests interpolate for rank 3 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from Solution onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2),w_ref)
      if dim==2:
        arg[0,0,0]=(0.317274364174)*x[0]**o+(-0.565307843766)*x[0]+(-0.367874638208)*x[1]**o+(0.0029581033546)*x[1]
        arg[0,0,1]=(0.832728755858)*x[0]**o+(0.245271193641)*x[0]+(-0.727130029377)*x[1]**o+(0.137912975829)*x[1]
        arg[0,1,0]=(-0.634508968932)*x[0]**o+(-0.864200861637)*x[0]+(-0.566985971867)*x[1]**o+(0.458187145519)*x[1]
        arg[0,1,1]=(0.133772683885)*x[0]**o+(-0.630800341215)*x[0]+(0.958934971335)*x[1]**o+(-0.15839182257)*x[1]
        arg[1,0,0]=(-0.0948724752087)*x[0]**o+(-0.25589337469)*x[0]+(-0.315713312867)*x[1]**o+(0.223487524432)*x[1]
        arg[1,0,1]=(0.794634271698)*x[0]**o+(-0.505797079712)*x[0]+(0.54367349338)*x[1]**o+(0.610149581933)*x[1]
        arg[1,1,0]=(-0.447039667341)*x[0]**o+(-0.917009838715)*x[0]+(0.525331113522)*x[1]**o+(-0.626176620973)*x[1]
        arg[1,1,1]=(0.0749028334839)*x[0]**o+(0.106950341725)*x[0]+(-0.375321288987)*x[1]**o+(0.510260762626)*x[1]
        arg[2,0,0]=(-0.286661970669)*x[0]**o+(0.584169107201)*x[0]+(0.189852415568)*x[1]**o+(-0.685695910848)*x[1]
        arg[2,0,1]=(-0.689509796724)*x[0]**o+(-0.316067007053)*x[0]+(0.93758916753)*x[1]**o+(0.987368705364)*x[1]
        arg[2,1,0]=(-0.411695937554)*x[0]**o+(0.858528628791)*x[0]+(-0.579361651418)*x[1]**o+(-0.179617869494)*x[1]
        arg[2,1,1]=(0.871441175049)*x[0]**o+(0.445176827889)*x[0]+(0.282613647019)*x[1]**o+(-0.0414230375843)*x[1]
        arg[3,0,0]=(-0.242136375691)*x[0]**o+(-0.91917484201)*x[0]+(-0.88026838607)*x[1]**o+(0.363309786006)*x[1]
        arg[3,0,1]=(-0.991221762386)*x[0]**o+(-0.0149280738883)*x[0]+(0.420322977563)*x[1]**o+(-0.0533076687981)*x[1]
        arg[3,1,0]=(-0.889003524303)*x[0]**o+(-0.023519502168)*x[0]+(0.826015855889)*x[1]**o+(0.0568467842617)*x[1]
        arg[3,1,1]=(0.608567002824)*x[0]**o+(-0.531288593441)*x[0]+(0.0886678034719)*x[1]**o+(-0.732353220319)*x[1]
        arg[4,0,0]=(-0.0537755786538)*x[0]**o+(-0.475886396481)*x[0]+(-0.153787571145)*x[1]**o+(0.689850737569)*x[1]
        arg[4,0,1]=(0.413017674549)*x[0]**o+(-0.993353261494)*x[0]+(-0.944597571268)*x[1]**o+(0.566423551758)*x[1]
        arg[4,1,0]=(-0.716864417268)*x[0]**o+(-0.871574272865)*x[0]+(0.394071572628)*x[1]**o+(0.452903048597)*x[1]
        arg[4,1,1]=(0.861790785883)*x[0]**o+(-0.511789401943)*x[0]+(-0.272452527007)*x[1]**o+(-0.379021521619)*x[1]
        arg[5,0,0]=(0.509668147054)*x[0]**o+(0.919355584411)*x[0]+(-0.673215166917)*x[1]**o+(0.766459631931)*x[1]
        arg[5,0,1]=(0.177006227916)*x[0]**o+(-0.275118477)*x[0]+(0.0302367843701)*x[1]**o+(-0.761226360328)*x[1]
        arg[5,1,0]=(0.013432534638)*x[0]**o+(-0.911626567027)*x[0]+(-0.710520442529)*x[1]**o+(-0.882317448423)*x[1]
        arg[5,1,1]=(0.595583093615)*x[0]**o+(0.163527905156)*x[0]+(-0.122165051333)*x[1]**o+(-0.526359288181)*x[1]
        ref[0,0,0]=(0.317274364174)*x_ref[0]**o+(-0.565307843766)*x_ref[0]+(-0.367874638208)*x_ref[1]**o+(0.0029581033546)*x_ref[1]
        ref[0,0,1]=(0.832728755858)*x_ref[0]**o+(0.245271193641)*x_ref[0]+(-0.727130029377)*x_ref[1]**o+(0.137912975829)*x_ref[1]
        ref[0,1,0]=(-0.634508968932)*x_ref[0]**o+(-0.864200861637)*x_ref[0]+(-0.566985971867)*x_ref[1]**o+(0.458187145519)*x_ref[1]
        ref[0,1,1]=(0.133772683885)*x_ref[0]**o+(-0.630800341215)*x_ref[0]+(0.958934971335)*x_ref[1]**o+(-0.15839182257)*x_ref[1]
        ref[1,0,0]=(-0.0948724752087)*x_ref[0]**o+(-0.25589337469)*x_ref[0]+(-0.315713312867)*x_ref[1]**o+(0.223487524432)*x_ref[1]
        ref[1,0,1]=(0.794634271698)*x_ref[0]**o+(-0.505797079712)*x_ref[0]+(0.54367349338)*x_ref[1]**o+(0.610149581933)*x_ref[1]
        ref[1,1,0]=(-0.447039667341)*x_ref[0]**o+(-0.917009838715)*x_ref[0]+(0.525331113522)*x_ref[1]**o+(-0.626176620973)*x_ref[1]
        ref[1,1,1]=(0.0749028334839)*x_ref[0]**o+(0.106950341725)*x_ref[0]+(-0.375321288987)*x_ref[1]**o+(0.510260762626)*x_ref[1]
        ref[2,0,0]=(-0.286661970669)*x_ref[0]**o+(0.584169107201)*x_ref[0]+(0.189852415568)*x_ref[1]**o+(-0.685695910848)*x_ref[1]
        ref[2,0,1]=(-0.689509796724)*x_ref[0]**o+(-0.316067007053)*x_ref[0]+(0.93758916753)*x_ref[1]**o+(0.987368705364)*x_ref[1]
        ref[2,1,0]=(-0.411695937554)*x_ref[0]**o+(0.858528628791)*x_ref[0]+(-0.579361651418)*x_ref[1]**o+(-0.179617869494)*x_ref[1]
        ref[2,1,1]=(0.871441175049)*x_ref[0]**o+(0.445176827889)*x_ref[0]+(0.282613647019)*x_ref[1]**o+(-0.0414230375843)*x_ref[1]
        ref[3,0,0]=(-0.242136375691)*x_ref[0]**o+(-0.91917484201)*x_ref[0]+(-0.88026838607)*x_ref[1]**o+(0.363309786006)*x_ref[1]
        ref[3,0,1]=(-0.991221762386)*x_ref[0]**o+(-0.0149280738883)*x_ref[0]+(0.420322977563)*x_ref[1]**o+(-0.0533076687981)*x_ref[1]
        ref[3,1,0]=(-0.889003524303)*x_ref[0]**o+(-0.023519502168)*x_ref[0]+(0.826015855889)*x_ref[1]**o+(0.0568467842617)*x_ref[1]
        ref[3,1,1]=(0.608567002824)*x_ref[0]**o+(-0.531288593441)*x_ref[0]+(0.0886678034719)*x_ref[1]**o+(-0.732353220319)*x_ref[1]
        ref[4,0,0]=(-0.0537755786538)*x_ref[0]**o+(-0.475886396481)*x_ref[0]+(-0.153787571145)*x_ref[1]**o+(0.689850737569)*x_ref[1]
        ref[4,0,1]=(0.413017674549)*x_ref[0]**o+(-0.993353261494)*x_ref[0]+(-0.944597571268)*x_ref[1]**o+(0.566423551758)*x_ref[1]
        ref[4,1,0]=(-0.716864417268)*x_ref[0]**o+(-0.871574272865)*x_ref[0]+(0.394071572628)*x_ref[1]**o+(0.452903048597)*x_ref[1]
        ref[4,1,1]=(0.861790785883)*x_ref[0]**o+(-0.511789401943)*x_ref[0]+(-0.272452527007)*x_ref[1]**o+(-0.379021521619)*x_ref[1]
        ref[5,0,0]=(0.509668147054)*x_ref[0]**o+(0.919355584411)*x_ref[0]+(-0.673215166917)*x_ref[1]**o+(0.766459631931)*x_ref[1]
        ref[5,0,1]=(0.177006227916)*x_ref[0]**o+(-0.275118477)*x_ref[0]+(0.0302367843701)*x_ref[1]**o+(-0.761226360328)*x_ref[1]
        ref[5,1,0]=(0.013432534638)*x_ref[0]**o+(-0.911626567027)*x_ref[0]+(-0.710520442529)*x_ref[1]**o+(-0.882317448423)*x_ref[1]
        ref[5,1,1]=(0.595583093615)*x_ref[0]**o+(0.163527905156)*x_ref[0]+(-0.122165051333)*x_ref[1]**o+(-0.526359288181)*x_ref[1]
      else:
        arg[0,0,0]=(0.230695674262)*x[0]**o+(-0.987212801716)*x[0]+(-0.0599001434987)*x[1]**o+(0.719591259555)*x[1]+(-0.665186369759)*x[2]**o+(-0.563552602515)*x[2]
        arg[0,0,1]=(-0.775441966131)*x[0]**o+(-0.152166849702)*x[0]+(0.626466789363)*x[1]**o+(0.179356346581)*x[1]+(-0.687406927434)*x[2]**o+(-0.563492313114)*x[2]
        arg[0,1,0]=(0.749522748133)*x[0]**o+(-0.582456308337)*x[0]+(0.676882643058)*x[1]**o+(0.365363182598)*x[1]+(-0.550867708812)*x[2]**o+(0.777938151183)*x[2]
        arg[0,1,1]=(0.389788552775)*x[0]**o+(-0.298877489997)*x[0]+(0.560326209269)*x[1]**o+(-0.113249832155)*x[1]+(0.588087510254)*x[2]**o+(-0.593503776461)*x[2]
        arg[1,0,0]=(-0.354661103909)*x[0]**o+(-0.195375369549)*x[0]+(-0.798795069895)*x[1]**o+(0.0968631048579)*x[1]+(-0.016118190391)*x[2]**o+(-0.903745407705)*x[2]
        arg[1,0,1]=(-0.383688834784)*x[0]**o+(0.861256231933)*x[0]+(-0.767204834667)*x[1]**o+(0.338875482372)*x[1]+(0.117146310891)*x[2]**o+(-0.302651846083)*x[2]
        arg[1,1,0]=(0.646263670969)*x[0]**o+(-0.447984008871)*x[0]+(-0.169322492785)*x[1]**o+(-0.570614724481)*x[1]+(-0.848518457618)*x[2]**o+(-0.0750005539216)*x[2]
        arg[1,1,1]=(-0.232496745304)*x[0]**o+(0.182581173413)*x[0]+(0.8032436357)*x[1]**o+(-0.391266449476)*x[1]+(0.491677629766)*x[2]**o+(-0.188361708708)*x[2]
        arg[2,0,0]=(-0.571145099091)*x[0]**o+(-0.486855242188)*x[0]+(0.52833582073)*x[1]**o+(0.326716348242)*x[1]+(0.528475192869)*x[2]**o+(0.163280575137)*x[2]
        arg[2,0,1]=(-0.368889674601)*x[0]**o+(0.601775816775)*x[0]+(0.0281742455403)*x[1]**o+(-0.198869716243)*x[1]+(0.193343065506)*x[2]**o+(-0.0614451571108)*x[2]
        arg[2,1,0]=(0.724595792492)*x[0]**o+(-0.206983407651)*x[0]+(-0.33966393719)*x[1]**o+(0.929465830796)*x[1]+(-0.938517195863)*x[2]**o+(0.70874077222)*x[2]
        arg[2,1,1]=(-0.536303362606)*x[0]**o+(-0.430663025812)*x[0]+(-0.0845923469062)*x[1]**o+(0.866655631144)*x[1]+(-0.2378102403)*x[2]**o+(0.969070797268)*x[2]
        arg[3,0,0]=(0.345137352559)*x[0]**o+(-0.696652518382)*x[0]+(0.914998974008)*x[1]**o+(-0.874935163156)*x[1]+(0.947812661283)*x[2]**o+(0.868611662789)*x[2]
        arg[3,0,1]=(-0.421640746358)*x[0]**o+(-0.37925991208)*x[0]+(0.468281241452)*x[1]**o+(0.439086689805)*x[1]+(-0.607986317001)*x[2]**o+(0.269488168937)*x[2]
        arg[3,1,0]=(-0.550485456473)*x[0]**o+(-0.520018714495)*x[0]+(0.439679309654)*x[1]**o+(-0.0652434084668)*x[1]+(-0.584838668708)*x[2]**o+(-0.46047281034)*x[2]
        arg[3,1,1]=(-0.421746456038)*x[0]**o+(0.370489549278)*x[0]+(-0.683682295105)*x[1]**o+(0.324051718035)*x[1]+(-0.579046430636)*x[2]**o+(0.587623604941)*x[2]
        arg[4,0,0]=(-0.167582584882)*x[0]**o+(0.524904122243)*x[0]+(0.665109348362)*x[1]**o+(-0.093655366509)*x[1]+(0.704404656254)*x[2]**o+(-0.402602441741)*x[2]
        arg[4,0,1]=(0.633854964824)*x[0]**o+(-0.836385840194)*x[0]+(-0.0995437996628)*x[1]**o+(-0.200285338869)*x[1]+(0.535023783881)*x[2]**o+(0.884702543593)*x[2]
        arg[4,1,0]=(-0.953619378208)*x[0]**o+(-0.390518815669)*x[0]+(-0.0590889178236)*x[1]**o+(-0.997225825019)*x[1]+(0.0520418736402)*x[2]**o+(0.484749465262)*x[2]
        arg[4,1,1]=(-0.440389854756)*x[0]**o+(0.15520431778)*x[0]+(0.346839174513)*x[1]**o+(0.0172676873159)*x[1]+(0.86275919224)*x[2]**o+(0.568069148992)*x[2]
        arg[5,0,0]=(-0.303687941217)*x[0]**o+(0.516115915687)*x[0]+(-0.750276597519)*x[1]**o+(0.734039058778)*x[1]+(-0.624499210252)*x[2]**o+(-0.926338135421)*x[2]
        arg[5,0,1]=(-0.472230777895)*x[0]**o+(-0.557827728373)*x[0]+(-0.403783023157)*x[1]**o+(0.70433336955)*x[1]+(-0.0211737454531)*x[2]**o+(0.0774406125836)*x[2]
        arg[5,1,0]=(0.718149875204)*x[0]**o+(-0.183000739787)*x[0]+(-0.655724252373)*x[1]**o+(-0.634941710776)*x[1]+(-0.26300163323)*x[2]**o+(-0.25105424715)*x[2]
        arg[5,1,1]=(0.109601904441)*x[0]**o+(-0.00640764631554)*x[0]+(-0.488035610009)*x[1]**o+(0.909668323769)*x[1]+(0.229692168501)*x[2]**o+(-0.739241309497)*x[2]
        ref[0,0,0]=(0.230695674262)*x_ref[0]**o+(-0.987212801716)*x_ref[0]+(-0.0599001434987)*x_ref[1]**o+(0.719591259555)*x_ref[1]+(-0.665186369759)*x_ref[2]**o+(-0.563552602515)*x_ref[2]
        ref[0,0,1]=(-0.775441966131)*x_ref[0]**o+(-0.152166849702)*x_ref[0]+(0.626466789363)*x_ref[1]**o+(0.179356346581)*x_ref[1]+(-0.687406927434)*x_ref[2]**o+(-0.563492313114)*x_ref[2]
        ref[0,1,0]=(0.749522748133)*x_ref[0]**o+(-0.582456308337)*x_ref[0]+(0.676882643058)*x_ref[1]**o+(0.365363182598)*x_ref[1]+(-0.550867708812)*x_ref[2]**o+(0.777938151183)*x_ref[2]
        ref[0,1,1]=(0.389788552775)*x_ref[0]**o+(-0.298877489997)*x_ref[0]+(0.560326209269)*x_ref[1]**o+(-0.113249832155)*x_ref[1]+(0.588087510254)*x_ref[2]**o+(-0.593503776461)*x_ref[2]
        ref[1,0,0]=(-0.354661103909)*x_ref[0]**o+(-0.195375369549)*x_ref[0]+(-0.798795069895)*x_ref[1]**o+(0.0968631048579)*x_ref[1]+(-0.016118190391)*x_ref[2]**o+(-0.903745407705)*x_ref[2]
        ref[1,0,1]=(-0.383688834784)*x_ref[0]**o+(0.861256231933)*x_ref[0]+(-0.767204834667)*x_ref[1]**o+(0.338875482372)*x_ref[1]+(0.117146310891)*x_ref[2]**o+(-0.302651846083)*x_ref[2]
        ref[1,1,0]=(0.646263670969)*x_ref[0]**o+(-0.447984008871)*x_ref[0]+(-0.169322492785)*x_ref[1]**o+(-0.570614724481)*x_ref[1]+(-0.848518457618)*x_ref[2]**o+(-0.0750005539216)*x_ref[2]
        ref[1,1,1]=(-0.232496745304)*x_ref[0]**o+(0.182581173413)*x_ref[0]+(0.8032436357)*x_ref[1]**o+(-0.391266449476)*x_ref[1]+(0.491677629766)*x_ref[2]**o+(-0.188361708708)*x_ref[2]
        ref[2,0,0]=(-0.571145099091)*x_ref[0]**o+(-0.486855242188)*x_ref[0]+(0.52833582073)*x_ref[1]**o+(0.326716348242)*x_ref[1]+(0.528475192869)*x_ref[2]**o+(0.163280575137)*x_ref[2]
        ref[2,0,1]=(-0.368889674601)*x_ref[0]**o+(0.601775816775)*x_ref[0]+(0.0281742455403)*x_ref[1]**o+(-0.198869716243)*x_ref[1]+(0.193343065506)*x_ref[2]**o+(-0.0614451571108)*x_ref[2]
        ref[2,1,0]=(0.724595792492)*x_ref[0]**o+(-0.206983407651)*x_ref[0]+(-0.33966393719)*x_ref[1]**o+(0.929465830796)*x_ref[1]+(-0.938517195863)*x_ref[2]**o+(0.70874077222)*x_ref[2]
        ref[2,1,1]=(-0.536303362606)*x_ref[0]**o+(-0.430663025812)*x_ref[0]+(-0.0845923469062)*x_ref[1]**o+(0.866655631144)*x_ref[1]+(-0.2378102403)*x_ref[2]**o+(0.969070797268)*x_ref[2]
        ref[3,0,0]=(0.345137352559)*x_ref[0]**o+(-0.696652518382)*x_ref[0]+(0.914998974008)*x_ref[1]**o+(-0.874935163156)*x_ref[1]+(0.947812661283)*x_ref[2]**o+(0.868611662789)*x_ref[2]
        ref[3,0,1]=(-0.421640746358)*x_ref[0]**o+(-0.37925991208)*x_ref[0]+(0.468281241452)*x_ref[1]**o+(0.439086689805)*x_ref[1]+(-0.607986317001)*x_ref[2]**o+(0.269488168937)*x_ref[2]
        ref[3,1,0]=(-0.550485456473)*x_ref[0]**o+(-0.520018714495)*x_ref[0]+(0.439679309654)*x_ref[1]**o+(-0.0652434084668)*x_ref[1]+(-0.584838668708)*x_ref[2]**o+(-0.46047281034)*x_ref[2]
        ref[3,1,1]=(-0.421746456038)*x_ref[0]**o+(0.370489549278)*x_ref[0]+(-0.683682295105)*x_ref[1]**o+(0.324051718035)*x_ref[1]+(-0.579046430636)*x_ref[2]**o+(0.587623604941)*x_ref[2]
        ref[4,0,0]=(-0.167582584882)*x_ref[0]**o+(0.524904122243)*x_ref[0]+(0.665109348362)*x_ref[1]**o+(-0.093655366509)*x_ref[1]+(0.704404656254)*x_ref[2]**o+(-0.402602441741)*x_ref[2]
        ref[4,0,1]=(0.633854964824)*x_ref[0]**o+(-0.836385840194)*x_ref[0]+(-0.0995437996628)*x_ref[1]**o+(-0.200285338869)*x_ref[1]+(0.535023783881)*x_ref[2]**o+(0.884702543593)*x_ref[2]
        ref[4,1,0]=(-0.953619378208)*x_ref[0]**o+(-0.390518815669)*x_ref[0]+(-0.0590889178236)*x_ref[1]**o+(-0.997225825019)*x_ref[1]+(0.0520418736402)*x_ref[2]**o+(0.484749465262)*x_ref[2]
        ref[4,1,1]=(-0.440389854756)*x_ref[0]**o+(0.15520431778)*x_ref[0]+(0.346839174513)*x_ref[1]**o+(0.0172676873159)*x_ref[1]+(0.86275919224)*x_ref[2]**o+(0.568069148992)*x_ref[2]
        ref[5,0,0]=(-0.303687941217)*x_ref[0]**o+(0.516115915687)*x_ref[0]+(-0.750276597519)*x_ref[1]**o+(0.734039058778)*x_ref[1]+(-0.624499210252)*x_ref[2]**o+(-0.926338135421)*x_ref[2]
        ref[5,0,1]=(-0.472230777895)*x_ref[0]**o+(-0.557827728373)*x_ref[0]+(-0.403783023157)*x_ref[1]**o+(0.70433336955)*x_ref[1]+(-0.0211737454531)*x_ref[2]**o+(0.0774406125836)*x_ref[2]
        ref[5,1,0]=(0.718149875204)*x_ref[0]**o+(-0.183000739787)*x_ref[0]+(-0.655724252373)*x_ref[1]**o+(-0.634941710776)*x_ref[1]+(-0.26300163323)*x_ref[2]**o+(-0.25105424715)*x_ref[2]
        ref[5,1,1]=(0.109601904441)*x_ref[0]**o+(-0.00640764631554)*x_ref[0]+(-0.488035610009)*x_ref[1]**o+(0.909668323769)*x_ref[1]+(0.229692168501)*x_ref[2]**o+(-0.739241309497)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_Solution_rank4(self):
      """
      tests interpolate for rank 4 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from Solution onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5, 3, 2),w)
      ref=Data(0,(4, 5, 3, 2),w_ref)
      if dim==2:
        arg[0,0,0,0]=(0.912518456494)*x[0]**o+(-0.828176151903)*x[0]+(-0.313940298121)*x[1]**o+(-0.850120336531)*x[1]
        arg[0,0,0,1]=(0.14415852783)*x[0]**o+(-0.294155246668)*x[0]+(0.562517248232)*x[1]**o+(-0.204505938286)*x[1]
        arg[0,0,1,0]=(0.555455404368)*x[0]**o+(0.916425343576)*x[0]+(0.547252315693)*x[1]**o+(-0.0992842367479)*x[1]
        arg[0,0,1,1]=(0.603632473928)*x[0]**o+(0.505571549062)*x[0]+(-0.490402921542)*x[1]**o+(0.908624427316)*x[1]
        arg[0,0,2,0]=(0.729542308985)*x[0]**o+(-0.654310868638)*x[0]+(0.0845981276245)*x[1]**o+(-0.95231604686)*x[1]
        arg[0,0,2,1]=(0.0511615606745)*x[0]**o+(-0.939469887975)*x[0]+(-0.206229418155)*x[1]**o+(0.254879844701)*x[1]
        arg[0,1,0,0]=(0.257917344507)*x[0]**o+(0.860729621288)*x[0]+(0.489966193348)*x[1]**o+(0.174897233858)*x[1]
        arg[0,1,0,1]=(0.336588362051)*x[0]**o+(-0.690598635114)*x[0]+(-0.0913735300483)*x[1]**o+(-0.932914439004)*x[1]
        arg[0,1,1,0]=(0.244583093108)*x[0]**o+(-0.827768167758)*x[0]+(0.573065359412)*x[1]**o+(0.176685515596)*x[1]
        arg[0,1,1,1]=(-0.520550726127)*x[0]**o+(-0.605243177286)*x[0]+(0.130549556226)*x[1]**o+(0.750019884067)*x[1]
        arg[0,1,2,0]=(-0.680201627614)*x[0]**o+(0.679925324882)*x[0]+(-0.060408604263)*x[1]**o+(0.568125689359)*x[1]
        arg[0,1,2,1]=(-0.111908859432)*x[0]**o+(0.256521037018)*x[0]+(0.332050616611)*x[1]**o+(-0.0500016397683)*x[1]
        arg[0,2,0,0]=(-0.484953933663)*x[0]**o+(0.628032211831)*x[0]+(-0.63297141454)*x[1]**o+(0.141016662755)*x[1]
        arg[0,2,0,1]=(-0.463920680465)*x[0]**o+(-0.868825520511)*x[0]+(-0.36594425006)*x[1]**o+(-0.823052216578)*x[1]
        arg[0,2,1,0]=(0.972337802357)*x[0]**o+(0.943605157343)*x[0]+(-0.113950717844)*x[1]**o+(-0.839596452897)*x[1]
        arg[0,2,1,1]=(-0.295562401302)*x[0]**o+(0.735770621017)*x[0]+(-0.183838912348)*x[1]**o+(-0.929365380252)*x[1]
        arg[0,2,2,0]=(0.912886298388)*x[0]**o+(0.847507982065)*x[0]+(-0.341306036323)*x[1]**o+(0.393627025978)*x[1]
        arg[0,2,2,1]=(-0.769650638386)*x[0]**o+(-0.619672356488)*x[0]+(-0.126894288906)*x[1]**o+(0.231547632139)*x[1]
        arg[0,3,0,0]=(-0.884442976834)*x[0]**o+(-0.0194505760676)*x[0]+(-0.338764675072)*x[1]**o+(0.723481600171)*x[1]
        arg[0,3,0,1]=(0.655973555091)*x[0]**o+(-0.556230649412)*x[0]+(-0.850356832441)*x[1]**o+(0.115267314853)*x[1]
        arg[0,3,1,0]=(0.659628443559)*x[0]**o+(-0.586677822491)*x[0]+(0.135930588518)*x[1]**o+(0.206866895948)*x[1]
        arg[0,3,1,1]=(-0.700258957557)*x[0]**o+(-0.699774198764)*x[0]+(-0.294009479186)*x[1]**o+(0.848159666503)*x[1]
        arg[0,3,2,0]=(-0.927501754488)*x[0]**o+(-0.981178071331)*x[0]+(0.614074194451)*x[1]**o+(0.711549573086)*x[1]
        arg[0,3,2,1]=(0.312434473198)*x[0]**o+(-0.241506125991)*x[0]+(-0.0840979088912)*x[1]**o+(0.902948463117)*x[1]
        arg[0,4,0,0]=(-0.892269860242)*x[0]**o+(0.823434435855)*x[0]+(0.50499383365)*x[1]**o+(0.763736541034)*x[1]
        arg[0,4,0,1]=(-0.606959691191)*x[0]**o+(0.945562536791)*x[0]+(-0.46271929481)*x[1]**o+(0.929514432096)*x[1]
        arg[0,4,1,0]=(0.953382685939)*x[0]**o+(0.553766359805)*x[0]+(-0.191004010293)*x[1]**o+(0.556369491698)*x[1]
        arg[0,4,1,1]=(-0.269402649165)*x[0]**o+(-0.158926232076)*x[0]+(-0.390038749681)*x[1]**o+(0.553655192666)*x[1]
        arg[0,4,2,0]=(0.0785586372006)*x[0]**o+(-0.778452221862)*x[0]+(-0.927627273292)*x[1]**o+(-0.187742405246)*x[1]
        arg[0,4,2,1]=(-0.965329945447)*x[0]**o+(0.304539006089)*x[0]+(0.213031689568)*x[1]**o+(-0.2807790149)*x[1]
        arg[1,0,0,0]=(0.981534698569)*x[0]**o+(0.466451685806)*x[0]+(0.845322931856)*x[1]**o+(0.582422032334)*x[1]
        arg[1,0,0,1]=(-0.0645798740394)*x[0]**o+(0.0475214206619)*x[0]+(-0.687759433076)*x[1]**o+(0.403440938285)*x[1]
        arg[1,0,1,0]=(-0.317887773392)*x[0]**o+(0.59101163416)*x[0]+(0.658626263291)*x[1]**o+(-0.618167594137)*x[1]
        arg[1,0,1,1]=(-0.314093092813)*x[0]**o+(0.699933899942)*x[0]+(0.824195260822)*x[1]**o+(-0.42875327345)*x[1]
        arg[1,0,2,0]=(-0.254254956887)*x[0]**o+(-0.912137080163)*x[0]+(0.360457121452)*x[1]**o+(-0.902596912588)*x[1]
        arg[1,0,2,1]=(-0.577214593683)*x[0]**o+(-0.419495663008)*x[0]+(-0.803459783331)*x[1]**o+(0.490885513568)*x[1]
        arg[1,1,0,0]=(0.477947766892)*x[0]**o+(-0.853046971581)*x[0]+(0.576044041601)*x[1]**o+(-0.447007701252)*x[1]
        arg[1,1,0,1]=(0.5886933409)*x[0]**o+(0.253650778351)*x[0]+(-0.198070684431)*x[1]**o+(0.189479364528)*x[1]
        arg[1,1,1,0]=(-0.528744478872)*x[0]**o+(-0.504072059557)*x[0]+(-0.0161832474015)*x[1]**o+(0.639592055966)*x[1]
        arg[1,1,1,1]=(-0.581844827169)*x[0]**o+(0.997883224538)*x[0]+(0.9354364489)*x[1]**o+(0.884060785136)*x[1]
        arg[1,1,2,0]=(0.858226955817)*x[0]**o+(0.707261698312)*x[0]+(-0.738309111076)*x[1]**o+(-0.644655976448)*x[1]
        arg[1,1,2,1]=(-0.254976445964)*x[0]**o+(-0.624982185158)*x[0]+(0.304854486996)*x[1]**o+(0.597089811682)*x[1]
        arg[1,2,0,0]=(-0.625190759215)*x[0]**o+(-0.267820197905)*x[0]+(-0.721452024377)*x[1]**o+(-0.830984245819)*x[1]
        arg[1,2,0,1]=(-0.972257608766)*x[0]**o+(0.981368890544)*x[0]+(-0.925886510064)*x[1]**o+(-0.337107356862)*x[1]
        arg[1,2,1,0]=(-0.178932954207)*x[0]**o+(-0.941789507374)*x[0]+(0.360726744794)*x[1]**o+(0.347621863123)*x[1]
        arg[1,2,1,1]=(-0.59790695611)*x[0]**o+(0.552509013166)*x[0]+(0.592412443998)*x[1]**o+(0.105369637971)*x[1]
        arg[1,2,2,0]=(0.300713195225)*x[0]**o+(0.814257503574)*x[0]+(-0.281407483492)*x[1]**o+(-0.803431681095)*x[1]
        arg[1,2,2,1]=(0.80113022942)*x[0]**o+(0.749336171665)*x[0]+(-0.0842928982283)*x[1]**o+(-0.974041792058)*x[1]
        arg[1,3,0,0]=(0.533118876666)*x[0]**o+(-0.00888178987567)*x[0]+(0.372869213762)*x[1]**o+(-0.652513683144)*x[1]
        arg[1,3,0,1]=(-0.295090222537)*x[0]**o+(0.772749124303)*x[0]+(-0.00857029468729)*x[1]**o+(0.927794431125)*x[1]
        arg[1,3,1,0]=(-0.152537465441)*x[0]**o+(0.154900534979)*x[0]+(-0.488743959543)*x[1]**o+(-0.0318497199232)*x[1]
        arg[1,3,1,1]=(-0.754333787921)*x[0]**o+(-0.0583007358701)*x[0]+(-0.050921106891)*x[1]**o+(0.161426452156)*x[1]
        arg[1,3,2,0]=(0.387357469451)*x[0]**o+(-0.213640305885)*x[0]+(-0.844609663023)*x[1]**o+(0.0761914594192)*x[1]
        arg[1,3,2,1]=(0.36964862459)*x[0]**o+(0.887888401952)*x[0]+(-0.933930511093)*x[1]**o+(0.769046614932)*x[1]
        arg[1,4,0,0]=(0.680878887266)*x[0]**o+(0.62930033679)*x[0]+(0.894793542585)*x[1]**o+(-0.636881609067)*x[1]
        arg[1,4,0,1]=(-0.346053756598)*x[0]**o+(0.225254433923)*x[0]+(-0.861052985621)*x[1]**o+(0.415123108136)*x[1]
        arg[1,4,1,0]=(-0.997758408183)*x[0]**o+(-0.265055614341)*x[0]+(-0.7568881617)*x[1]**o+(0.95144584777)*x[1]
        arg[1,4,1,1]=(0.227777912008)*x[0]**o+(-0.83647660817)*x[0]+(-0.275733410165)*x[1]**o+(-0.971560248343)*x[1]
        arg[1,4,2,0]=(0.718803859114)*x[0]**o+(-0.978966510682)*x[0]+(0.463102438649)*x[1]**o+(0.987805610534)*x[1]
        arg[1,4,2,1]=(-0.107543838917)*x[0]**o+(-0.362684018694)*x[0]+(-0.310926939373)*x[1]**o+(-0.571571044712)*x[1]
        arg[2,0,0,0]=(-0.475977416182)*x[0]**o+(-0.627173002609)*x[0]+(-0.369962970984)*x[1]**o+(-0.245077302537)*x[1]
        arg[2,0,0,1]=(0.913949611751)*x[0]**o+(0.645611424617)*x[0]+(0.639304734076)*x[1]**o+(-0.12171441998)*x[1]
        arg[2,0,1,0]=(0.251434098263)*x[0]**o+(-0.119898861407)*x[0]+(-0.365614523709)*x[1]**o+(-0.802928738296)*x[1]
        arg[2,0,1,1]=(0.756390927006)*x[0]**o+(0.246578936199)*x[0]+(0.714968085461)*x[1]**o+(-0.929428808096)*x[1]
        arg[2,0,2,0]=(-0.516888361447)*x[0]**o+(0.0344339250886)*x[0]+(-0.383709788774)*x[1]**o+(-0.437344417623)*x[1]
        arg[2,0,2,1]=(-0.219307162317)*x[0]**o+(0.704909718842)*x[0]+(-0.340454491855)*x[1]**o+(-0.933866231773)*x[1]
        arg[2,1,0,0]=(-0.0618967891627)*x[0]**o+(-0.0590712030789)*x[0]+(0.116776433077)*x[1]**o+(0.0241239764104)*x[1]
        arg[2,1,0,1]=(-0.802207645638)*x[0]**o+(0.590732740531)*x[0]+(-0.25990272933)*x[1]**o+(-0.257633805416)*x[1]
        arg[2,1,1,0]=(0.827623274379)*x[0]**o+(-0.450310573599)*x[0]+(0.213809775826)*x[1]**o+(-0.905410777787)*x[1]
        arg[2,1,1,1]=(-0.624510830356)*x[0]**o+(-0.705682607787)*x[0]+(0.430105716993)*x[1]**o+(0.975510832521)*x[1]
        arg[2,1,2,0]=(-0.297562223309)*x[0]**o+(-0.99090348994)*x[0]+(0.548589416291)*x[1]**o+(0.591718478583)*x[1]
        arg[2,1,2,1]=(-0.531899919376)*x[0]**o+(-0.227453719581)*x[0]+(-0.131664806128)*x[1]**o+(0.0521895817509)*x[1]
        arg[2,2,0,0]=(-0.592291745786)*x[0]**o+(-0.586879961972)*x[0]+(0.563196552327)*x[1]**o+(0.876989321187)*x[1]
        arg[2,2,0,1]=(-0.273360445369)*x[0]**o+(-0.714740527775)*x[0]+(-0.647699497837)*x[1]**o+(-0.0405089612762)*x[1]
        arg[2,2,1,0]=(-0.536629732224)*x[0]**o+(0.72656373966)*x[0]+(0.986630594165)*x[1]**o+(0.0741630711667)*x[1]
        arg[2,2,1,1]=(0.770746088838)*x[0]**o+(-0.219279833426)*x[0]+(0.875445174779)*x[1]**o+(-0.947889926444)*x[1]
        arg[2,2,2,0]=(-0.41428815119)*x[0]**o+(-0.696166268903)*x[0]+(-0.96838062553)*x[1]**o+(0.781394205899)*x[1]
        arg[2,2,2,1]=(-0.810267960752)*x[0]**o+(-0.873632687702)*x[0]+(0.0131971321611)*x[1]**o+(-0.552767192193)*x[1]
        arg[2,3,0,0]=(-0.216091366291)*x[0]**o+(-0.17824047472)*x[0]+(-0.4644515725)*x[1]**o+(0.135819458957)*x[1]
        arg[2,3,0,1]=(-0.0823616523105)*x[0]**o+(0.33121362151)*x[0]+(0.926719919839)*x[1]**o+(0.663475641636)*x[1]
        arg[2,3,1,0]=(-0.278303066047)*x[0]**o+(-0.457286378314)*x[0]+(0.186792859092)*x[1]**o+(-0.734245612464)*x[1]
        arg[2,3,1,1]=(0.810871459779)*x[0]**o+(-0.706641995324)*x[0]+(-0.287827176091)*x[1]**o+(0.453216090282)*x[1]
        arg[2,3,2,0]=(0.83826221049)*x[0]**o+(0.556887792111)*x[0]+(-0.000231102245991)*x[1]**o+(-0.440730958954)*x[1]
        arg[2,3,2,1]=(-0.455087877907)*x[0]**o+(-0.132259634036)*x[0]+(-0.492728335525)*x[1]**o+(-0.880311317777)*x[1]
        arg[2,4,0,0]=(-0.303141352388)*x[0]**o+(0.426137046804)*x[0]+(0.788934776738)*x[1]**o+(0.157477785179)*x[1]
        arg[2,4,0,1]=(0.603411288614)*x[0]**o+(-0.533561451987)*x[0]+(-0.901259210942)*x[1]**o+(0.547372624746)*x[1]
        arg[2,4,1,0]=(0.69517961739)*x[0]**o+(-0.106282888246)*x[0]+(0.257484565681)*x[1]**o+(0.655749633294)*x[1]
        arg[2,4,1,1]=(-0.072748573118)*x[0]**o+(0.905966560113)*x[0]+(0.433145718123)*x[1]**o+(0.187620401615)*x[1]
        arg[2,4,2,0]=(-0.0280176325954)*x[0]**o+(0.489610131347)*x[0]+(-0.740554922134)*x[1]**o+(-0.85045302096)*x[1]
        arg[2,4,2,1]=(-0.158503180709)*x[0]**o+(0.525929643075)*x[0]+(-0.352022652515)*x[1]**o+(-0.706553104932)*x[1]
        arg[3,0,0,0]=(-0.837316508127)*x[0]**o+(-0.89490181779)*x[0]+(0.510084075291)*x[1]**o+(0.321206310024)*x[1]
        arg[3,0,0,1]=(-0.325038591079)*x[0]**o+(0.140342922753)*x[0]+(-0.937466403004)*x[1]**o+(0.580557383497)*x[1]
        arg[3,0,1,0]=(-0.939713654062)*x[0]**o+(-0.394660244166)*x[0]+(0.762376614557)*x[1]**o+(-0.135565547532)*x[1]
        arg[3,0,1,1]=(0.0275332524464)*x[0]**o+(0.368737701882)*x[0]+(0.983641372218)*x[1]**o+(0.573815516366)*x[1]
        arg[3,0,2,0]=(0.816957570314)*x[0]**o+(-0.357626892562)*x[0]+(-0.368933149642)*x[1]**o+(-0.692502486914)*x[1]
        arg[3,0,2,1]=(-0.372408810926)*x[0]**o+(-0.113036390416)*x[0]+(-0.355880297003)*x[1]**o+(0.755478191481)*x[1]
        arg[3,1,0,0]=(0.0399012900832)*x[0]**o+(-0.879576900086)*x[0]+(-0.660858776134)*x[1]**o+(-0.59837040429)*x[1]
        arg[3,1,0,1]=(0.569397128496)*x[0]**o+(-0.138333417746)*x[0]+(0.257769147727)*x[1]**o+(0.174737141027)*x[1]
        arg[3,1,1,0]=(-0.67938408823)*x[0]**o+(0.862731161981)*x[0]+(-0.792538485264)*x[1]**o+(-0.0805197796157)*x[1]
        arg[3,1,1,1]=(0.0113523229279)*x[0]**o+(0.475393597561)*x[0]+(-0.705748021886)*x[1]**o+(0.00836460135285)*x[1]
        arg[3,1,2,0]=(0.463839509799)*x[0]**o+(-0.12663544643)*x[0]+(0.353158399738)*x[1]**o+(0.674929537597)*x[1]
        arg[3,1,2,1]=(-0.6019896019)*x[0]**o+(-0.638280685565)*x[0]+(0.280047072988)*x[1]**o+(0.569239958138)*x[1]
        arg[3,2,0,0]=(-0.113906500346)*x[0]**o+(-0.493494498501)*x[0]+(-0.686419229162)*x[1]**o+(-0.0727046763516)*x[1]
        arg[3,2,0,1]=(-0.302092112795)*x[0]**o+(-0.74819167)*x[0]+(-0.478908975282)*x[1]**o+(0.430024789627)*x[1]
        arg[3,2,1,0]=(0.353535608984)*x[0]**o+(-0.743408549264)*x[0]+(-0.069122881538)*x[1]**o+(-0.0767595089529)*x[1]
        arg[3,2,1,1]=(-0.629946386234)*x[0]**o+(0.851779590894)*x[0]+(0.728577108695)*x[1]**o+(-0.366983040147)*x[1]
        arg[3,2,2,0]=(0.1685167508)*x[0]**o+(0.149999747637)*x[0]+(0.709320430374)*x[1]**o+(-0.280950677222)*x[1]
        arg[3,2,2,1]=(-0.956164974578)*x[0]**o+(0.240460380901)*x[0]+(-0.2820176577)*x[1]**o+(-0.595667156955)*x[1]
        arg[3,3,0,0]=(-0.0711295055938)*x[0]**o+(-0.314513251791)*x[0]+(-0.41924591293)*x[1]**o+(0.0813197780959)*x[1]
        arg[3,3,0,1]=(-0.230592584877)*x[0]**o+(-0.488005144715)*x[0]+(-0.450611759976)*x[1]**o+(-0.0571152769768)*x[1]
        arg[3,3,1,0]=(-0.608255699014)*x[0]**o+(-0.697324108575)*x[0]+(-0.885935900746)*x[1]**o+(-0.884497550016)*x[1]
        arg[3,3,1,1]=(0.857286225292)*x[0]**o+(0.509954550626)*x[0]+(0.182030821433)*x[1]**o+(-0.268076060316)*x[1]
        arg[3,3,2,0]=(-0.77322653479)*x[0]**o+(-0.583013210018)*x[0]+(-0.683867300277)*x[1]**o+(0.754530541251)*x[1]
        arg[3,3,2,1]=(-0.744667985181)*x[0]**o+(0.164165128978)*x[0]+(0.412340155958)*x[1]**o+(-0.38451107118)*x[1]
        arg[3,4,0,0]=(0.216720755049)*x[0]**o+(-0.347803433722)*x[0]+(0.417376268797)*x[1]**o+(0.475787304652)*x[1]
        arg[3,4,0,1]=(-0.737038295974)*x[0]**o+(0.579172519178)*x[0]+(-0.900272993672)*x[1]**o+(0.407018603625)*x[1]
        arg[3,4,1,0]=(-0.898959236474)*x[0]**o+(-0.886636008121)*x[0]+(-0.901972610044)*x[1]**o+(-0.518393782318)*x[1]
        arg[3,4,1,1]=(0.976342884921)*x[0]**o+(-0.273568319493)*x[0]+(-0.332424980846)*x[1]**o+(-0.0146661731139)*x[1]
        arg[3,4,2,0]=(0.794792214027)*x[0]**o+(0.846927984413)*x[0]+(0.535818809709)*x[1]**o+(-0.49324400033)*x[1]
        arg[3,4,2,1]=(0.695766308812)*x[0]**o+(-0.37923153689)*x[0]+(0.0633513026782)*x[1]**o+(0.12334892105)*x[1]
        ref[0,0,0,0]=(0.912518456494)*x_ref[0]**o+(-0.828176151903)*x_ref[0]+(-0.313940298121)*x_ref[1]**o+(-0.850120336531)*x_ref[1]
        ref[0,0,0,1]=(0.14415852783)*x_ref[0]**o+(-0.294155246668)*x_ref[0]+(0.562517248232)*x_ref[1]**o+(-0.204505938286)*x_ref[1]
        ref[0,0,1,0]=(0.555455404368)*x_ref[0]**o+(0.916425343576)*x_ref[0]+(0.547252315693)*x_ref[1]**o+(-0.0992842367479)*x_ref[1]
        ref[0,0,1,1]=(0.603632473928)*x_ref[0]**o+(0.505571549062)*x_ref[0]+(-0.490402921542)*x_ref[1]**o+(0.908624427316)*x_ref[1]
        ref[0,0,2,0]=(0.729542308985)*x_ref[0]**o+(-0.654310868638)*x_ref[0]+(0.0845981276245)*x_ref[1]**o+(-0.95231604686)*x_ref[1]
        ref[0,0,2,1]=(0.0511615606745)*x_ref[0]**o+(-0.939469887975)*x_ref[0]+(-0.206229418155)*x_ref[1]**o+(0.254879844701)*x_ref[1]
        ref[0,1,0,0]=(0.257917344507)*x_ref[0]**o+(0.860729621288)*x_ref[0]+(0.489966193348)*x_ref[1]**o+(0.174897233858)*x_ref[1]
        ref[0,1,0,1]=(0.336588362051)*x_ref[0]**o+(-0.690598635114)*x_ref[0]+(-0.0913735300483)*x_ref[1]**o+(-0.932914439004)*x_ref[1]
        ref[0,1,1,0]=(0.244583093108)*x_ref[0]**o+(-0.827768167758)*x_ref[0]+(0.573065359412)*x_ref[1]**o+(0.176685515596)*x_ref[1]
        ref[0,1,1,1]=(-0.520550726127)*x_ref[0]**o+(-0.605243177286)*x_ref[0]+(0.130549556226)*x_ref[1]**o+(0.750019884067)*x_ref[1]
        ref[0,1,2,0]=(-0.680201627614)*x_ref[0]**o+(0.679925324882)*x_ref[0]+(-0.060408604263)*x_ref[1]**o+(0.568125689359)*x_ref[1]
        ref[0,1,2,1]=(-0.111908859432)*x_ref[0]**o+(0.256521037018)*x_ref[0]+(0.332050616611)*x_ref[1]**o+(-0.0500016397683)*x_ref[1]
        ref[0,2,0,0]=(-0.484953933663)*x_ref[0]**o+(0.628032211831)*x_ref[0]+(-0.63297141454)*x_ref[1]**o+(0.141016662755)*x_ref[1]
        ref[0,2,0,1]=(-0.463920680465)*x_ref[0]**o+(-0.868825520511)*x_ref[0]+(-0.36594425006)*x_ref[1]**o+(-0.823052216578)*x_ref[1]
        ref[0,2,1,0]=(0.972337802357)*x_ref[0]**o+(0.943605157343)*x_ref[0]+(-0.113950717844)*x_ref[1]**o+(-0.839596452897)*x_ref[1]
        ref[0,2,1,1]=(-0.295562401302)*x_ref[0]**o+(0.735770621017)*x_ref[0]+(-0.183838912348)*x_ref[1]**o+(-0.929365380252)*x_ref[1]
        ref[0,2,2,0]=(0.912886298388)*x_ref[0]**o+(0.847507982065)*x_ref[0]+(-0.341306036323)*x_ref[1]**o+(0.393627025978)*x_ref[1]
        ref[0,2,2,1]=(-0.769650638386)*x_ref[0]**o+(-0.619672356488)*x_ref[0]+(-0.126894288906)*x_ref[1]**o+(0.231547632139)*x_ref[1]
        ref[0,3,0,0]=(-0.884442976834)*x_ref[0]**o+(-0.0194505760676)*x_ref[0]+(-0.338764675072)*x_ref[1]**o+(0.723481600171)*x_ref[1]
        ref[0,3,0,1]=(0.655973555091)*x_ref[0]**o+(-0.556230649412)*x_ref[0]+(-0.850356832441)*x_ref[1]**o+(0.115267314853)*x_ref[1]
        ref[0,3,1,0]=(0.659628443559)*x_ref[0]**o+(-0.586677822491)*x_ref[0]+(0.135930588518)*x_ref[1]**o+(0.206866895948)*x_ref[1]
        ref[0,3,1,1]=(-0.700258957557)*x_ref[0]**o+(-0.699774198764)*x_ref[0]+(-0.294009479186)*x_ref[1]**o+(0.848159666503)*x_ref[1]
        ref[0,3,2,0]=(-0.927501754488)*x_ref[0]**o+(-0.981178071331)*x_ref[0]+(0.614074194451)*x_ref[1]**o+(0.711549573086)*x_ref[1]
        ref[0,3,2,1]=(0.312434473198)*x_ref[0]**o+(-0.241506125991)*x_ref[0]+(-0.0840979088912)*x_ref[1]**o+(0.902948463117)*x_ref[1]
        ref[0,4,0,0]=(-0.892269860242)*x_ref[0]**o+(0.823434435855)*x_ref[0]+(0.50499383365)*x_ref[1]**o+(0.763736541034)*x_ref[1]
        ref[0,4,0,1]=(-0.606959691191)*x_ref[0]**o+(0.945562536791)*x_ref[0]+(-0.46271929481)*x_ref[1]**o+(0.929514432096)*x_ref[1]
        ref[0,4,1,0]=(0.953382685939)*x_ref[0]**o+(0.553766359805)*x_ref[0]+(-0.191004010293)*x_ref[1]**o+(0.556369491698)*x_ref[1]
        ref[0,4,1,1]=(-0.269402649165)*x_ref[0]**o+(-0.158926232076)*x_ref[0]+(-0.390038749681)*x_ref[1]**o+(0.553655192666)*x_ref[1]
        ref[0,4,2,0]=(0.0785586372006)*x_ref[0]**o+(-0.778452221862)*x_ref[0]+(-0.927627273292)*x_ref[1]**o+(-0.187742405246)*x_ref[1]
        ref[0,4,2,1]=(-0.965329945447)*x_ref[0]**o+(0.304539006089)*x_ref[0]+(0.213031689568)*x_ref[1]**o+(-0.2807790149)*x_ref[1]
        ref[1,0,0,0]=(0.981534698569)*x_ref[0]**o+(0.466451685806)*x_ref[0]+(0.845322931856)*x_ref[1]**o+(0.582422032334)*x_ref[1]
        ref[1,0,0,1]=(-0.0645798740394)*x_ref[0]**o+(0.0475214206619)*x_ref[0]+(-0.687759433076)*x_ref[1]**o+(0.403440938285)*x_ref[1]
        ref[1,0,1,0]=(-0.317887773392)*x_ref[0]**o+(0.59101163416)*x_ref[0]+(0.658626263291)*x_ref[1]**o+(-0.618167594137)*x_ref[1]
        ref[1,0,1,1]=(-0.314093092813)*x_ref[0]**o+(0.699933899942)*x_ref[0]+(0.824195260822)*x_ref[1]**o+(-0.42875327345)*x_ref[1]
        ref[1,0,2,0]=(-0.254254956887)*x_ref[0]**o+(-0.912137080163)*x_ref[0]+(0.360457121452)*x_ref[1]**o+(-0.902596912588)*x_ref[1]
        ref[1,0,2,1]=(-0.577214593683)*x_ref[0]**o+(-0.419495663008)*x_ref[0]+(-0.803459783331)*x_ref[1]**o+(0.490885513568)*x_ref[1]
        ref[1,1,0,0]=(0.477947766892)*x_ref[0]**o+(-0.853046971581)*x_ref[0]+(0.576044041601)*x_ref[1]**o+(-0.447007701252)*x_ref[1]
        ref[1,1,0,1]=(0.5886933409)*x_ref[0]**o+(0.253650778351)*x_ref[0]+(-0.198070684431)*x_ref[1]**o+(0.189479364528)*x_ref[1]
        ref[1,1,1,0]=(-0.528744478872)*x_ref[0]**o+(-0.504072059557)*x_ref[0]+(-0.0161832474015)*x_ref[1]**o+(0.639592055966)*x_ref[1]
        ref[1,1,1,1]=(-0.581844827169)*x_ref[0]**o+(0.997883224538)*x_ref[0]+(0.9354364489)*x_ref[1]**o+(0.884060785136)*x_ref[1]
        ref[1,1,2,0]=(0.858226955817)*x_ref[0]**o+(0.707261698312)*x_ref[0]+(-0.738309111076)*x_ref[1]**o+(-0.644655976448)*x_ref[1]
        ref[1,1,2,1]=(-0.254976445964)*x_ref[0]**o+(-0.624982185158)*x_ref[0]+(0.304854486996)*x_ref[1]**o+(0.597089811682)*x_ref[1]
        ref[1,2,0,0]=(-0.625190759215)*x_ref[0]**o+(-0.267820197905)*x_ref[0]+(-0.721452024377)*x_ref[1]**o+(-0.830984245819)*x_ref[1]
        ref[1,2,0,1]=(-0.972257608766)*x_ref[0]**o+(0.981368890544)*x_ref[0]+(-0.925886510064)*x_ref[1]**o+(-0.337107356862)*x_ref[1]
        ref[1,2,1,0]=(-0.178932954207)*x_ref[0]**o+(-0.941789507374)*x_ref[0]+(0.360726744794)*x_ref[1]**o+(0.347621863123)*x_ref[1]
        ref[1,2,1,1]=(-0.59790695611)*x_ref[0]**o+(0.552509013166)*x_ref[0]+(0.592412443998)*x_ref[1]**o+(0.105369637971)*x_ref[1]
        ref[1,2,2,0]=(0.300713195225)*x_ref[0]**o+(0.814257503574)*x_ref[0]+(-0.281407483492)*x_ref[1]**o+(-0.803431681095)*x_ref[1]
        ref[1,2,2,1]=(0.80113022942)*x_ref[0]**o+(0.749336171665)*x_ref[0]+(-0.0842928982283)*x_ref[1]**o+(-0.974041792058)*x_ref[1]
        ref[1,3,0,0]=(0.533118876666)*x_ref[0]**o+(-0.00888178987567)*x_ref[0]+(0.372869213762)*x_ref[1]**o+(-0.652513683144)*x_ref[1]
        ref[1,3,0,1]=(-0.295090222537)*x_ref[0]**o+(0.772749124303)*x_ref[0]+(-0.00857029468729)*x_ref[1]**o+(0.927794431125)*x_ref[1]
        ref[1,3,1,0]=(-0.152537465441)*x_ref[0]**o+(0.154900534979)*x_ref[0]+(-0.488743959543)*x_ref[1]**o+(-0.0318497199232)*x_ref[1]
        ref[1,3,1,1]=(-0.754333787921)*x_ref[0]**o+(-0.0583007358701)*x_ref[0]+(-0.050921106891)*x_ref[1]**o+(0.161426452156)*x_ref[1]
        ref[1,3,2,0]=(0.387357469451)*x_ref[0]**o+(-0.213640305885)*x_ref[0]+(-0.844609663023)*x_ref[1]**o+(0.0761914594192)*x_ref[1]
        ref[1,3,2,1]=(0.36964862459)*x_ref[0]**o+(0.887888401952)*x_ref[0]+(-0.933930511093)*x_ref[1]**o+(0.769046614932)*x_ref[1]
        ref[1,4,0,0]=(0.680878887266)*x_ref[0]**o+(0.62930033679)*x_ref[0]+(0.894793542585)*x_ref[1]**o+(-0.636881609067)*x_ref[1]
        ref[1,4,0,1]=(-0.346053756598)*x_ref[0]**o+(0.225254433923)*x_ref[0]+(-0.861052985621)*x_ref[1]**o+(0.415123108136)*x_ref[1]
        ref[1,4,1,0]=(-0.997758408183)*x_ref[0]**o+(-0.265055614341)*x_ref[0]+(-0.7568881617)*x_ref[1]**o+(0.95144584777)*x_ref[1]
        ref[1,4,1,1]=(0.227777912008)*x_ref[0]**o+(-0.83647660817)*x_ref[0]+(-0.275733410165)*x_ref[1]**o+(-0.971560248343)*x_ref[1]
        ref[1,4,2,0]=(0.718803859114)*x_ref[0]**o+(-0.978966510682)*x_ref[0]+(0.463102438649)*x_ref[1]**o+(0.987805610534)*x_ref[1]
        ref[1,4,2,1]=(-0.107543838917)*x_ref[0]**o+(-0.362684018694)*x_ref[0]+(-0.310926939373)*x_ref[1]**o+(-0.571571044712)*x_ref[1]
        ref[2,0,0,0]=(-0.475977416182)*x_ref[0]**o+(-0.627173002609)*x_ref[0]+(-0.369962970984)*x_ref[1]**o+(-0.245077302537)*x_ref[1]
        ref[2,0,0,1]=(0.913949611751)*x_ref[0]**o+(0.645611424617)*x_ref[0]+(0.639304734076)*x_ref[1]**o+(-0.12171441998)*x_ref[1]
        ref[2,0,1,0]=(0.251434098263)*x_ref[0]**o+(-0.119898861407)*x_ref[0]+(-0.365614523709)*x_ref[1]**o+(-0.802928738296)*x_ref[1]
        ref[2,0,1,1]=(0.756390927006)*x_ref[0]**o+(0.246578936199)*x_ref[0]+(0.714968085461)*x_ref[1]**o+(-0.929428808096)*x_ref[1]
        ref[2,0,2,0]=(-0.516888361447)*x_ref[0]**o+(0.0344339250886)*x_ref[0]+(-0.383709788774)*x_ref[1]**o+(-0.437344417623)*x_ref[1]
        ref[2,0,2,1]=(-0.219307162317)*x_ref[0]**o+(0.704909718842)*x_ref[0]+(-0.340454491855)*x_ref[1]**o+(-0.933866231773)*x_ref[1]
        ref[2,1,0,0]=(-0.0618967891627)*x_ref[0]**o+(-0.0590712030789)*x_ref[0]+(0.116776433077)*x_ref[1]**o+(0.0241239764104)*x_ref[1]
        ref[2,1,0,1]=(-0.802207645638)*x_ref[0]**o+(0.590732740531)*x_ref[0]+(-0.25990272933)*x_ref[1]**o+(-0.257633805416)*x_ref[1]
        ref[2,1,1,0]=(0.827623274379)*x_ref[0]**o+(-0.450310573599)*x_ref[0]+(0.213809775826)*x_ref[1]**o+(-0.905410777787)*x_ref[1]
        ref[2,1,1,1]=(-0.624510830356)*x_ref[0]**o+(-0.705682607787)*x_ref[0]+(0.430105716993)*x_ref[1]**o+(0.975510832521)*x_ref[1]
        ref[2,1,2,0]=(-0.297562223309)*x_ref[0]**o+(-0.99090348994)*x_ref[0]+(0.548589416291)*x_ref[1]**o+(0.591718478583)*x_ref[1]
        ref[2,1,2,1]=(-0.531899919376)*x_ref[0]**o+(-0.227453719581)*x_ref[0]+(-0.131664806128)*x_ref[1]**o+(0.0521895817509)*x_ref[1]
        ref[2,2,0,0]=(-0.592291745786)*x_ref[0]**o+(-0.586879961972)*x_ref[0]+(0.563196552327)*x_ref[1]**o+(0.876989321187)*x_ref[1]
        ref[2,2,0,1]=(-0.273360445369)*x_ref[0]**o+(-0.714740527775)*x_ref[0]+(-0.647699497837)*x_ref[1]**o+(-0.0405089612762)*x_ref[1]
        ref[2,2,1,0]=(-0.536629732224)*x_ref[0]**o+(0.72656373966)*x_ref[0]+(0.986630594165)*x_ref[1]**o+(0.0741630711667)*x_ref[1]
        ref[2,2,1,1]=(0.770746088838)*x_ref[0]**o+(-0.219279833426)*x_ref[0]+(0.875445174779)*x_ref[1]**o+(-0.947889926444)*x_ref[1]
        ref[2,2,2,0]=(-0.41428815119)*x_ref[0]**o+(-0.696166268903)*x_ref[0]+(-0.96838062553)*x_ref[1]**o+(0.781394205899)*x_ref[1]
        ref[2,2,2,1]=(-0.810267960752)*x_ref[0]**o+(-0.873632687702)*x_ref[0]+(0.0131971321611)*x_ref[1]**o+(-0.552767192193)*x_ref[1]
        ref[2,3,0,0]=(-0.216091366291)*x_ref[0]**o+(-0.17824047472)*x_ref[0]+(-0.4644515725)*x_ref[1]**o+(0.135819458957)*x_ref[1]
        ref[2,3,0,1]=(-0.0823616523105)*x_ref[0]**o+(0.33121362151)*x_ref[0]+(0.926719919839)*x_ref[1]**o+(0.663475641636)*x_ref[1]
        ref[2,3,1,0]=(-0.278303066047)*x_ref[0]**o+(-0.457286378314)*x_ref[0]+(0.186792859092)*x_ref[1]**o+(-0.734245612464)*x_ref[1]
        ref[2,3,1,1]=(0.810871459779)*x_ref[0]**o+(-0.706641995324)*x_ref[0]+(-0.287827176091)*x_ref[1]**o+(0.453216090282)*x_ref[1]
        ref[2,3,2,0]=(0.83826221049)*x_ref[0]**o+(0.556887792111)*x_ref[0]+(-0.000231102245991)*x_ref[1]**o+(-0.440730958954)*x_ref[1]
        ref[2,3,2,1]=(-0.455087877907)*x_ref[0]**o+(-0.132259634036)*x_ref[0]+(-0.492728335525)*x_ref[1]**o+(-0.880311317777)*x_ref[1]
        ref[2,4,0,0]=(-0.303141352388)*x_ref[0]**o+(0.426137046804)*x_ref[0]+(0.788934776738)*x_ref[1]**o+(0.157477785179)*x_ref[1]
        ref[2,4,0,1]=(0.603411288614)*x_ref[0]**o+(-0.533561451987)*x_ref[0]+(-0.901259210942)*x_ref[1]**o+(0.547372624746)*x_ref[1]
        ref[2,4,1,0]=(0.69517961739)*x_ref[0]**o+(-0.106282888246)*x_ref[0]+(0.257484565681)*x_ref[1]**o+(0.655749633294)*x_ref[1]
        ref[2,4,1,1]=(-0.072748573118)*x_ref[0]**o+(0.905966560113)*x_ref[0]+(0.433145718123)*x_ref[1]**o+(0.187620401615)*x_ref[1]
        ref[2,4,2,0]=(-0.0280176325954)*x_ref[0]**o+(0.489610131347)*x_ref[0]+(-0.740554922134)*x_ref[1]**o+(-0.85045302096)*x_ref[1]
        ref[2,4,2,1]=(-0.158503180709)*x_ref[0]**o+(0.525929643075)*x_ref[0]+(-0.352022652515)*x_ref[1]**o+(-0.706553104932)*x_ref[1]
        ref[3,0,0,0]=(-0.837316508127)*x_ref[0]**o+(-0.89490181779)*x_ref[0]+(0.510084075291)*x_ref[1]**o+(0.321206310024)*x_ref[1]
        ref[3,0,0,1]=(-0.325038591079)*x_ref[0]**o+(0.140342922753)*x_ref[0]+(-0.937466403004)*x_ref[1]**o+(0.580557383497)*x_ref[1]
        ref[3,0,1,0]=(-0.939713654062)*x_ref[0]**o+(-0.394660244166)*x_ref[0]+(0.762376614557)*x_ref[1]**o+(-0.135565547532)*x_ref[1]
        ref[3,0,1,1]=(0.0275332524464)*x_ref[0]**o+(0.368737701882)*x_ref[0]+(0.983641372218)*x_ref[1]**o+(0.573815516366)*x_ref[1]
        ref[3,0,2,0]=(0.816957570314)*x_ref[0]**o+(-0.357626892562)*x_ref[0]+(-0.368933149642)*x_ref[1]**o+(-0.692502486914)*x_ref[1]
        ref[3,0,2,1]=(-0.372408810926)*x_ref[0]**o+(-0.113036390416)*x_ref[0]+(-0.355880297003)*x_ref[1]**o+(0.755478191481)*x_ref[1]
        ref[3,1,0,0]=(0.0399012900832)*x_ref[0]**o+(-0.879576900086)*x_ref[0]+(-0.660858776134)*x_ref[1]**o+(-0.59837040429)*x_ref[1]
        ref[3,1,0,1]=(0.569397128496)*x_ref[0]**o+(-0.138333417746)*x_ref[0]+(0.257769147727)*x_ref[1]**o+(0.174737141027)*x_ref[1]
        ref[3,1,1,0]=(-0.67938408823)*x_ref[0]**o+(0.862731161981)*x_ref[0]+(-0.792538485264)*x_ref[1]**o+(-0.0805197796157)*x_ref[1]
        ref[3,1,1,1]=(0.0113523229279)*x_ref[0]**o+(0.475393597561)*x_ref[0]+(-0.705748021886)*x_ref[1]**o+(0.00836460135285)*x_ref[1]
        ref[3,1,2,0]=(0.463839509799)*x_ref[0]**o+(-0.12663544643)*x_ref[0]+(0.353158399738)*x_ref[1]**o+(0.674929537597)*x_ref[1]
        ref[3,1,2,1]=(-0.6019896019)*x_ref[0]**o+(-0.638280685565)*x_ref[0]+(0.280047072988)*x_ref[1]**o+(0.569239958138)*x_ref[1]
        ref[3,2,0,0]=(-0.113906500346)*x_ref[0]**o+(-0.493494498501)*x_ref[0]+(-0.686419229162)*x_ref[1]**o+(-0.0727046763516)*x_ref[1]
        ref[3,2,0,1]=(-0.302092112795)*x_ref[0]**o+(-0.74819167)*x_ref[0]+(-0.478908975282)*x_ref[1]**o+(0.430024789627)*x_ref[1]
        ref[3,2,1,0]=(0.353535608984)*x_ref[0]**o+(-0.743408549264)*x_ref[0]+(-0.069122881538)*x_ref[1]**o+(-0.0767595089529)*x_ref[1]
        ref[3,2,1,1]=(-0.629946386234)*x_ref[0]**o+(0.851779590894)*x_ref[0]+(0.728577108695)*x_ref[1]**o+(-0.366983040147)*x_ref[1]
        ref[3,2,2,0]=(0.1685167508)*x_ref[0]**o+(0.149999747637)*x_ref[0]+(0.709320430374)*x_ref[1]**o+(-0.280950677222)*x_ref[1]
        ref[3,2,2,1]=(-0.956164974578)*x_ref[0]**o+(0.240460380901)*x_ref[0]+(-0.2820176577)*x_ref[1]**o+(-0.595667156955)*x_ref[1]
        ref[3,3,0,0]=(-0.0711295055938)*x_ref[0]**o+(-0.314513251791)*x_ref[0]+(-0.41924591293)*x_ref[1]**o+(0.0813197780959)*x_ref[1]
        ref[3,3,0,1]=(-0.230592584877)*x_ref[0]**o+(-0.488005144715)*x_ref[0]+(-0.450611759976)*x_ref[1]**o+(-0.0571152769768)*x_ref[1]
        ref[3,3,1,0]=(-0.608255699014)*x_ref[0]**o+(-0.697324108575)*x_ref[0]+(-0.885935900746)*x_ref[1]**o+(-0.884497550016)*x_ref[1]
        ref[3,3,1,1]=(0.857286225292)*x_ref[0]**o+(0.509954550626)*x_ref[0]+(0.182030821433)*x_ref[1]**o+(-0.268076060316)*x_ref[1]
        ref[3,3,2,0]=(-0.77322653479)*x_ref[0]**o+(-0.583013210018)*x_ref[0]+(-0.683867300277)*x_ref[1]**o+(0.754530541251)*x_ref[1]
        ref[3,3,2,1]=(-0.744667985181)*x_ref[0]**o+(0.164165128978)*x_ref[0]+(0.412340155958)*x_ref[1]**o+(-0.38451107118)*x_ref[1]
        ref[3,4,0,0]=(0.216720755049)*x_ref[0]**o+(-0.347803433722)*x_ref[0]+(0.417376268797)*x_ref[1]**o+(0.475787304652)*x_ref[1]
        ref[3,4,0,1]=(-0.737038295974)*x_ref[0]**o+(0.579172519178)*x_ref[0]+(-0.900272993672)*x_ref[1]**o+(0.407018603625)*x_ref[1]
        ref[3,4,1,0]=(-0.898959236474)*x_ref[0]**o+(-0.886636008121)*x_ref[0]+(-0.901972610044)*x_ref[1]**o+(-0.518393782318)*x_ref[1]
        ref[3,4,1,1]=(0.976342884921)*x_ref[0]**o+(-0.273568319493)*x_ref[0]+(-0.332424980846)*x_ref[1]**o+(-0.0146661731139)*x_ref[1]
        ref[3,4,2,0]=(0.794792214027)*x_ref[0]**o+(0.846927984413)*x_ref[0]+(0.535818809709)*x_ref[1]**o+(-0.49324400033)*x_ref[1]
        ref[3,4,2,1]=(0.695766308812)*x_ref[0]**o+(-0.37923153689)*x_ref[0]+(0.0633513026782)*x_ref[1]**o+(0.12334892105)*x_ref[1]
      else:
        arg[0,0,0,0]=(-0.401901998082)*x[0]**o+(-0.805091116808)*x[0]+(0.997659882792)*x[1]**o+(-0.953957389492)*x[1]+(-0.450185408289)*x[2]**o+(0.636472207773)*x[2]
        arg[0,0,0,1]=(-0.786386817128)*x[0]**o+(-0.882559775393)*x[0]+(0.117871400819)*x[1]**o+(-0.625922274774)*x[1]+(0.410431886931)*x[2]**o+(-0.979356501067)*x[2]
        arg[0,0,1,0]=(0.686025409275)*x[0]**o+(-0.19806907659)*x[0]+(-0.710420224449)*x[1]**o+(-0.839236953983)*x[1]+(-0.79971896093)*x[2]**o+(0.378944111319)*x[2]
        arg[0,0,1,1]=(0.771205112484)*x[0]**o+(-0.007083192859)*x[0]+(0.672918588001)*x[1]**o+(-0.351652495793)*x[1]+(-0.841767161187)*x[2]**o+(-0.345642373014)*x[2]
        arg[0,0,2,0]=(0.266930729015)*x[0]**o+(-0.873808152307)*x[0]+(0.206466759135)*x[1]**o+(0.539888649948)*x[1]+(0.17985862191)*x[2]**o+(0.240651460153)*x[2]
        arg[0,0,2,1]=(-0.232569768041)*x[0]**o+(-0.579841454072)*x[0]+(-0.706956104891)*x[1]**o+(0.822863815698)*x[1]+(0.345774991358)*x[2]**o+(-0.0848567668311)*x[2]
        arg[0,1,0,0]=(0.51593851507)*x[0]**o+(-0.233342434349)*x[0]+(-0.92498298026)*x[1]**o+(0.164540234829)*x[1]+(0.356559744499)*x[2]**o+(0.483902511717)*x[2]
        arg[0,1,0,1]=(-0.811509654443)*x[0]**o+(0.73929453189)*x[0]+(0.256205706418)*x[1]**o+(-0.667924913793)*x[1]+(0.266065641257)*x[2]**o+(0.856490258029)*x[2]
        arg[0,1,1,0]=(0.502051590284)*x[0]**o+(0.927283797128)*x[0]+(0.500854521411)*x[1]**o+(-0.486326996369)*x[1]+(-0.327578523685)*x[2]**o+(-0.28106070397)*x[2]
        arg[0,1,1,1]=(0.201740014513)*x[0]**o+(-0.166988241759)*x[0]+(-0.513072955566)*x[1]**o+(0.67179532417)*x[1]+(-0.72135980627)*x[2]**o+(0.763548886467)*x[2]
        arg[0,1,2,0]=(0.3995590338)*x[0]**o+(0.433286929621)*x[0]+(-0.287309453792)*x[1]**o+(-0.0671679443074)*x[1]+(-0.80815399993)*x[2]**o+(-0.398740362114)*x[2]
        arg[0,1,2,1]=(-0.16314640684)*x[0]**o+(-0.698681697367)*x[0]+(-0.11609738418)*x[1]**o+(-0.798465154739)*x[1]+(-0.169052628971)*x[2]**o+(-0.880316962539)*x[2]
        arg[0,2,0,0]=(-0.219731279442)*x[0]**o+(0.731464233112)*x[0]+(0.637811533993)*x[1]**o+(0.381218512692)*x[1]+(-0.181357790048)*x[2]**o+(-0.296793024723)*x[2]
        arg[0,2,0,1]=(0.566451624572)*x[0]**o+(0.302904935559)*x[0]+(-0.771526665232)*x[1]**o+(0.460577683302)*x[1]+(-0.921312455507)*x[2]**o+(0.604299765394)*x[2]
        arg[0,2,1,0]=(-0.0887516895197)*x[0]**o+(0.872208558285)*x[0]+(-0.306564750999)*x[1]**o+(0.841982500263)*x[1]+(-0.550398737046)*x[2]**o+(-0.869697162427)*x[2]
        arg[0,2,1,1]=(-0.652471383912)*x[0]**o+(0.634592304393)*x[0]+(0.0709212473167)*x[1]**o+(-0.671330082579)*x[1]+(-0.232895004478)*x[2]**o+(0.479008351165)*x[2]
        arg[0,2,2,0]=(-0.0764115046136)*x[0]**o+(0.649230346025)*x[0]+(0.544503480999)*x[1]**o+(0.788838981797)*x[1]+(0.416236148267)*x[2]**o+(0.722979541918)*x[2]
        arg[0,2,2,1]=(-0.0776368628837)*x[0]**o+(0.801984674386)*x[0]+(0.443670313486)*x[1]**o+(0.424676792255)*x[1]+(-0.895989937982)*x[2]**o+(0.0774648143589)*x[2]
        arg[0,3,0,0]=(-0.39041318438)*x[0]**o+(-0.174335409262)*x[0]+(0.658666482337)*x[1]**o+(0.23060605921)*x[1]+(-0.336603364419)*x[2]**o+(0.871530192198)*x[2]
        arg[0,3,0,1]=(-0.493104661363)*x[0]**o+(0.590779697499)*x[0]+(-0.903897457443)*x[1]**o+(-0.377640520704)*x[1]+(0.227174544833)*x[2]**o+(-0.860150483539)*x[2]
        arg[0,3,1,0]=(-0.093367253954)*x[0]**o+(0.159230499811)*x[0]+(-0.731396509936)*x[1]**o+(0.600088148838)*x[1]+(0.829175660147)*x[2]**o+(-0.435878280919)*x[2]
        arg[0,3,1,1]=(0.665125540711)*x[0]**o+(-0.92128737118)*x[0]+(-0.33260297543)*x[1]**o+(0.682569476816)*x[1]+(0.999761405058)*x[2]**o+(-0.748303702618)*x[2]
        arg[0,3,2,0]=(-0.430442277525)*x[0]**o+(-0.841437664378)*x[0]+(-0.374267371055)*x[1]**o+(0.667860327373)*x[1]+(0.694185699238)*x[2]**o+(-0.673629629622)*x[2]
        arg[0,3,2,1]=(0.304808617243)*x[0]**o+(-0.894407776037)*x[0]+(-0.758585281699)*x[1]**o+(0.20519963241)*x[1]+(0.538222015181)*x[2]**o+(-0.792962435573)*x[2]
        arg[0,4,0,0]=(0.568300300269)*x[0]**o+(0.407455936625)*x[0]+(0.980898344535)*x[1]**o+(0.988220474461)*x[1]+(0.0598892620089)*x[2]**o+(-0.279555207723)*x[2]
        arg[0,4,0,1]=(-0.208391854989)*x[0]**o+(0.9428019078)*x[0]+(-0.540916877275)*x[1]**o+(-0.427185538555)*x[1]+(-0.879268085857)*x[2]**o+(0.025102321099)*x[2]
        arg[0,4,1,0]=(0.412690112527)*x[0]**o+(-0.957376863734)*x[0]+(-0.913623139719)*x[1]**o+(-0.462418849979)*x[1]+(0.463336747019)*x[2]**o+(0.330747531293)*x[2]
        arg[0,4,1,1]=(-0.632140552118)*x[0]**o+(0.830769577671)*x[0]+(-0.888012251479)*x[1]**o+(-0.0560181569699)*x[1]+(-0.70068099248)*x[2]**o+(0.834483253039)*x[2]
        arg[0,4,2,0]=(0.0429717567385)*x[0]**o+(-0.691784042595)*x[0]+(-0.264988764713)*x[1]**o+(0.359873170931)*x[1]+(-0.116275128437)*x[2]**o+(0.0348873917345)*x[2]
        arg[0,4,2,1]=(-0.203754358311)*x[0]**o+(0.177896174646)*x[0]+(-0.0785920580727)*x[1]**o+(-0.124905949184)*x[1]+(0.968871355367)*x[2]**o+(-0.0369722247589)*x[2]
        arg[1,0,0,0]=(0.594851756786)*x[0]**o+(-0.594400822936)*x[0]+(0.868401350949)*x[1]**o+(0.186739616297)*x[1]+(-0.937219946754)*x[2]**o+(-0.248916422131)*x[2]
        arg[1,0,0,1]=(-0.639865978203)*x[0]**o+(-0.00990375501714)*x[0]+(-0.038266357411)*x[1]**o+(-0.741522380136)*x[1]+(0.460572922772)*x[2]**o+(0.4314205896)*x[2]
        arg[1,0,1,0]=(-0.460110413486)*x[0]**o+(-0.0964934735182)*x[0]+(0.474209024352)*x[1]**o+(-0.928476110805)*x[1]+(0.598621411487)*x[2]**o+(-0.00858812754345)*x[2]
        arg[1,0,1,1]=(0.936466172807)*x[0]**o+(-0.363937334197)*x[0]+(-0.540224651799)*x[1]**o+(0.484530722263)*x[1]+(0.675416993296)*x[2]**o+(0.89657344343)*x[2]
        arg[1,0,2,0]=(-0.713050692555)*x[0]**o+(-0.00851892179229)*x[0]+(0.689494113809)*x[1]**o+(0.633341028044)*x[1]+(0.272726911113)*x[2]**o+(-0.122017566819)*x[2]
        arg[1,0,2,1]=(0.793152080981)*x[0]**o+(0.287379739639)*x[0]+(0.304047372771)*x[1]**o+(-0.509079361053)*x[1]+(0.888302006985)*x[2]**o+(-0.297023045775)*x[2]
        arg[1,1,0,0]=(-0.69695118901)*x[0]**o+(0.505182668512)*x[0]+(0.827555590952)*x[1]**o+(-0.777232447056)*x[1]+(-0.460918138036)*x[2]**o+(0.782090206922)*x[2]
        arg[1,1,0,1]=(-0.435398535566)*x[0]**o+(-0.0997593681984)*x[0]+(0.140250252691)*x[1]**o+(-0.0204469256488)*x[1]+(-0.445836771028)*x[2]**o+(-0.775366037861)*x[2]
        arg[1,1,1,0]=(0.918015347957)*x[0]**o+(-0.491940111489)*x[0]+(0.793071617663)*x[1]**o+(-0.607035294859)*x[1]+(0.610076618566)*x[2]**o+(0.148116733513)*x[2]
        arg[1,1,1,1]=(0.224062518811)*x[0]**o+(-0.145897156735)*x[0]+(0.165380866901)*x[1]**o+(0.910562981814)*x[1]+(-0.406620561548)*x[2]**o+(0.528862812468)*x[2]
        arg[1,1,2,0]=(0.85148444093)*x[0]**o+(0.13210280653)*x[0]+(0.364136256712)*x[1]**o+(0.520388575386)*x[1]+(0.0145641287346)*x[2]**o+(-0.7849036351)*x[2]
        arg[1,1,2,1]=(0.493837193832)*x[0]**o+(-0.193326176569)*x[0]+(-0.665920972154)*x[1]**o+(-0.321645070095)*x[1]+(0.0209354853869)*x[2]**o+(-0.328315981752)*x[2]
        arg[1,2,0,0]=(-0.349281164766)*x[0]**o+(-0.318415198113)*x[0]+(-0.687972781234)*x[1]**o+(0.91825998605)*x[1]+(0.396868773798)*x[2]**o+(0.998204458027)*x[2]
        arg[1,2,0,1]=(-0.967493222935)*x[0]**o+(-0.721128465653)*x[0]+(-0.672178802974)*x[1]**o+(0.0924145224431)*x[1]+(0.114040587594)*x[2]**o+(0.707623895537)*x[2]
        arg[1,2,1,0]=(-0.253327088773)*x[0]**o+(0.206063827625)*x[0]+(0.355152654895)*x[1]**o+(-0.27339285621)*x[1]+(-0.0135567122291)*x[2]**o+(-0.983990446362)*x[2]
        arg[1,2,1,1]=(0.98471687132)*x[0]**o+(0.262902745655)*x[0]+(-0.589359138227)*x[1]**o+(-0.0982548958556)*x[1]+(-0.526161200133)*x[2]**o+(-0.850939484752)*x[2]
        arg[1,2,2,0]=(0.37583596705)*x[0]**o+(0.274845981222)*x[0]+(0.601331110943)*x[1]**o+(0.788294448218)*x[1]+(0.366445166459)*x[2]**o+(-0.554935072394)*x[2]
        arg[1,2,2,1]=(-0.986387850237)*x[0]**o+(0.68772989781)*x[0]+(0.345166604933)*x[1]**o+(0.930376726384)*x[1]+(0.760974421701)*x[2]**o+(0.261797396808)*x[2]
        arg[1,3,0,0]=(-0.171005013523)*x[0]**o+(-0.339229596812)*x[0]+(-0.150317419556)*x[1]**o+(-0.0146533427463)*x[1]+(0.957020663875)*x[2]**o+(0.616513940347)*x[2]
        arg[1,3,0,1]=(-0.0727328269499)*x[0]**o+(-0.635591748073)*x[0]+(-0.502426989774)*x[1]**o+(0.772595119619)*x[1]+(-0.76006511271)*x[2]**o+(-0.665920288084)*x[2]
        arg[1,3,1,0]=(-0.394284205622)*x[0]**o+(-0.105959165811)*x[0]+(0.72649045701)*x[1]**o+(-0.970276379473)*x[1]+(-0.236520349415)*x[2]**o+(0.994973267783)*x[2]
        arg[1,3,1,1]=(-0.444149867906)*x[0]**o+(0.748449201475)*x[0]+(0.419604170756)*x[1]**o+(-0.0291702705178)*x[1]+(-0.109580142628)*x[2]**o+(-0.0791870699838)*x[2]
        arg[1,3,2,0]=(-0.629657820523)*x[0]**o+(-0.995576552624)*x[0]+(0.530135462206)*x[1]**o+(0.197336335845)*x[1]+(0.0224815558807)*x[2]**o+(0.240498463912)*x[2]
        arg[1,3,2,1]=(0.52933948346)*x[0]**o+(0.98353913011)*x[0]+(0.0217614943138)*x[1]**o+(0.0157778453682)*x[1]+(0.177956932986)*x[2]**o+(0.234629778706)*x[2]
        arg[1,4,0,0]=(0.593028367137)*x[0]**o+(0.0340892538994)*x[0]+(-0.0295133170051)*x[1]**o+(0.0865921041963)*x[1]+(0.641131718656)*x[2]**o+(0.788858018609)*x[2]
        arg[1,4,0,1]=(-0.283333820634)*x[0]**o+(0.864482725099)*x[0]+(-0.560218794286)*x[1]**o+(-0.0083115362546)*x[1]+(0.726423285129)*x[2]**o+(0.031531339166)*x[2]
        arg[1,4,1,0]=(0.801452411554)*x[0]**o+(-0.59916393038)*x[0]+(-0.106568863591)*x[1]**o+(0.357442125133)*x[1]+(-0.203380280132)*x[2]**o+(-0.129031606441)*x[2]
        arg[1,4,1,1]=(-0.589220928637)*x[0]**o+(-0.308949389428)*x[0]+(0.785549858338)*x[1]**o+(-0.131153853015)*x[1]+(-0.30571855555)*x[2]**o+(-0.329634802588)*x[2]
        arg[1,4,2,0]=(-0.479686944)*x[0]**o+(-0.707702026873)*x[0]+(0.635894172352)*x[1]**o+(0.370673635069)*x[1]+(0.942862875991)*x[2]**o+(-0.846943456738)*x[2]
        arg[1,4,2,1]=(-0.403737324968)*x[0]**o+(0.101664426734)*x[0]+(-0.677093482354)*x[1]**o+(-0.36526066745)*x[1]+(-0.0889619136182)*x[2]**o+(-0.772491940844)*x[2]
        arg[2,0,0,0]=(0.753148325987)*x[0]**o+(-0.342843083128)*x[0]+(0.812661029553)*x[1]**o+(-0.112683035459)*x[1]+(0.900976470275)*x[2]**o+(0.00934086955102)*x[2]
        arg[2,0,0,1]=(-0.682315007143)*x[0]**o+(-0.694069900861)*x[0]+(-0.469082247747)*x[1]**o+(-0.522165903301)*x[1]+(-0.0997886221924)*x[2]**o+(-0.954161552419)*x[2]
        arg[2,0,1,0]=(0.780125539705)*x[0]**o+(-0.445331450225)*x[0]+(-0.282765510424)*x[1]**o+(-0.0131898361592)*x[1]+(-0.200186433923)*x[2]**o+(0.715139435696)*x[2]
        arg[2,0,1,1]=(-0.132310660786)*x[0]**o+(-0.0981344234369)*x[0]+(0.959297529475)*x[1]**o+(-0.85944894515)*x[1]+(-0.396258659619)*x[2]**o+(-0.09672066185)*x[2]
        arg[2,0,2,0]=(-0.344134014191)*x[0]**o+(0.534345864287)*x[0]+(0.797209509484)*x[1]**o+(0.0479050101499)*x[1]+(0.13620476987)*x[2]**o+(0.82708115383)*x[2]
        arg[2,0,2,1]=(-0.342509328238)*x[0]**o+(-0.833164493006)*x[0]+(0.472047721532)*x[1]**o+(0.301613158637)*x[1]+(0.858691679638)*x[2]**o+(-0.691744164006)*x[2]
        arg[2,1,0,0]=(0.498979390299)*x[0]**o+(0.371513881824)*x[0]+(-0.243506167971)*x[1]**o+(-0.245423890439)*x[1]+(-0.0494707840274)*x[2]**o+(0.0965573709928)*x[2]
        arg[2,1,0,1]=(0.665982455862)*x[0]**o+(-0.409710866336)*x[0]+(0.669257568046)*x[1]**o+(0.711947704788)*x[1]+(-0.0362568635785)*x[2]**o+(-0.30394278188)*x[2]
        arg[2,1,1,0]=(0.296245249928)*x[0]**o+(0.132631019281)*x[0]+(-0.376880152158)*x[1]**o+(0.303184156241)*x[1]+(0.836046560882)*x[2]**o+(0.195205645106)*x[2]
        arg[2,1,1,1]=(-0.971045820953)*x[0]**o+(0.154708896467)*x[0]+(-0.663844656602)*x[1]**o+(-0.0873797588245)*x[1]+(0.985123372542)*x[2]**o+(0.349554922131)*x[2]
        arg[2,1,2,0]=(-0.0117443431348)*x[0]**o+(-0.71980503985)*x[0]+(0.175348078994)*x[1]**o+(-0.883987892161)*x[1]+(0.0587275450458)*x[2]**o+(0.464811715548)*x[2]
        arg[2,1,2,1]=(-0.309202575765)*x[0]**o+(-0.528513248453)*x[0]+(-0.509298303972)*x[1]**o+(-0.228278863496)*x[1]+(0.933917053676)*x[2]**o+(-0.823678942264)*x[2]
        arg[2,2,0,0]=(0.374387386929)*x[0]**o+(0.64969993847)*x[0]+(-0.939822635641)*x[1]**o+(0.578249430422)*x[1]+(-0.973412660635)*x[2]**o+(0.484052718386)*x[2]
        arg[2,2,0,1]=(-0.972635162618)*x[0]**o+(-0.255007433802)*x[0]+(0.134165187097)*x[1]**o+(0.313213615976)*x[1]+(0.143298148686)*x[2]**o+(0.901819440229)*x[2]
        arg[2,2,1,0]=(0.868173256315)*x[0]**o+(-0.532325343453)*x[0]+(-0.712500880845)*x[1]**o+(0.216943462308)*x[1]+(0.733920091212)*x[2]**o+(0.0992400603383)*x[2]
        arg[2,2,1,1]=(0.642839350527)*x[0]**o+(0.127721543559)*x[0]+(-0.152686523032)*x[1]**o+(-0.399032361234)*x[1]+(0.317014518842)*x[2]**o+(0.905538790115)*x[2]
        arg[2,2,2,0]=(0.685267932413)*x[0]**o+(-0.167571547782)*x[0]+(-0.213446144405)*x[1]**o+(0.420270456377)*x[1]+(0.827725350621)*x[2]**o+(-0.920242274399)*x[2]
        arg[2,2,2,1]=(-0.507424593437)*x[0]**o+(-0.694222840019)*x[0]+(-0.904108970188)*x[1]**o+(-0.750899614966)*x[1]+(0.783997553302)*x[2]**o+(0.196327513447)*x[2]
        arg[2,3,0,0]=(-0.473900600479)*x[0]**o+(-0.464347097789)*x[0]+(-0.472423358671)*x[1]**o+(0.373786456364)*x[1]+(-0.989371300414)*x[2]**o+(-0.959508065239)*x[2]
        arg[2,3,0,1]=(0.174375189399)*x[0]**o+(0.925503059886)*x[0]+(-0.647274384723)*x[1]**o+(-0.577149479679)*x[1]+(0.171298877768)*x[2]**o+(-0.936612100882)*x[2]
        arg[2,3,1,0]=(0.482959343396)*x[0]**o+(0.842243554011)*x[0]+(0.814423633314)*x[1]**o+(-0.989445050315)*x[1]+(0.817933433065)*x[2]**o+(0.552396881404)*x[2]
        arg[2,3,1,1]=(0.55641736229)*x[0]**o+(0.498094640058)*x[0]+(0.0890113606037)*x[1]**o+(0.36592723553)*x[1]+(0.458607593386)*x[2]**o+(0.603047420063)*x[2]
        arg[2,3,2,0]=(0.884515080124)*x[0]**o+(-0.561898381022)*x[0]+(-0.292475503834)*x[1]**o+(0.425634423478)*x[1]+(0.65813323252)*x[2]**o+(-0.548850399842)*x[2]
        arg[2,3,2,1]=(-0.72895566982)*x[0]**o+(-0.569405588005)*x[0]+(0.117770875594)*x[1]**o+(-0.33515735969)*x[1]+(0.102062736382)*x[2]**o+(0.611532417683)*x[2]
        arg[2,4,0,0]=(-0.669208630424)*x[0]**o+(0.944393014177)*x[0]+(-0.00345511889683)*x[1]**o+(0.0695279595783)*x[1]+(0.496948250833)*x[2]**o+(-0.899750463971)*x[2]
        arg[2,4,0,1]=(-0.387828717292)*x[0]**o+(0.621172267004)*x[0]+(-0.280632674111)*x[1]**o+(0.401702567575)*x[1]+(-0.65554489378)*x[2]**o+(0.403018047007)*x[2]
        arg[2,4,1,0]=(-0.0770246925803)*x[0]**o+(0.52913302745)*x[0]+(0.922468724357)*x[1]**o+(-0.304442750439)*x[1]+(-0.556575079059)*x[2]**o+(-0.725790672251)*x[2]
        arg[2,4,1,1]=(-0.828378342911)*x[0]**o+(-0.0696370821776)*x[0]+(0.394018581667)*x[1]**o+(-0.251099848269)*x[1]+(-0.29878985656)*x[2]**o+(0.925753371056)*x[2]
        arg[2,4,2,0]=(-0.818277786029)*x[0]**o+(0.225862354706)*x[0]+(-0.629050212595)*x[1]**o+(0.139186107062)*x[1]+(-0.0134835519238)*x[2]**o+(0.301899528641)*x[2]
        arg[2,4,2,1]=(-0.783053684767)*x[0]**o+(-0.0926938783599)*x[0]+(0.519034210352)*x[1]**o+(-0.494564657126)*x[1]+(0.536117310622)*x[2]**o+(0.6278756634)*x[2]
        arg[3,0,0,0]=(-0.694353957685)*x[0]**o+(-0.361037363965)*x[0]+(-0.327004091885)*x[1]**o+(-0.765735142884)*x[1]+(-0.810915729713)*x[2]**o+(0.0640539311609)*x[2]
        arg[3,0,0,1]=(-0.131601548312)*x[0]**o+(0.271587756708)*x[0]+(0.322486168005)*x[1]**o+(0.238086581801)*x[1]+(-0.488081720555)*x[2]**o+(0.607278995635)*x[2]
        arg[3,0,1,0]=(0.903075418102)*x[0]**o+(-0.738609834848)*x[0]+(-0.165097989126)*x[1]**o+(-0.25967140914)*x[1]+(0.271649887428)*x[2]**o+(-0.786131648727)*x[2]
        arg[3,0,1,1]=(-0.963442989157)*x[0]**o+(-0.240205602334)*x[0]+(0.102153389731)*x[1]**o+(-0.106402434995)*x[1]+(-0.383097724211)*x[2]**o+(0.680290963195)*x[2]
        arg[3,0,2,0]=(-0.373162835796)*x[0]**o+(-0.186323761814)*x[0]+(0.661494315705)*x[1]**o+(0.320072189494)*x[1]+(0.0964443431067)*x[2]**o+(0.829015838241)*x[2]
        arg[3,0,2,1]=(0.527719514862)*x[0]**o+(0.0819948261665)*x[0]+(0.826598712178)*x[1]**o+(-0.588102975125)*x[1]+(-0.632584402292)*x[2]**o+(0.0826992233702)*x[2]
        arg[3,1,0,0]=(-0.102243686287)*x[0]**o+(-0.497336112929)*x[0]+(0.941827592628)*x[1]**o+(0.68678991944)*x[1]+(-0.508684841667)*x[2]**o+(-0.752939965161)*x[2]
        arg[3,1,0,1]=(-0.896417523183)*x[0]**o+(0.759224781685)*x[0]+(0.138537695823)*x[1]**o+(0.466727480471)*x[1]+(-0.838380824412)*x[2]**o+(0.285761157398)*x[2]
        arg[3,1,1,0]=(-0.547853182919)*x[0]**o+(-0.41595413414)*x[0]+(0.76894221584)*x[1]**o+(-0.142394275192)*x[1]+(-0.96349294581)*x[2]**o+(-0.873639964693)*x[2]
        arg[3,1,1,1]=(-0.151299403888)*x[0]**o+(0.789406063878)*x[0]+(0.163280670253)*x[1]**o+(-0.287315494745)*x[1]+(-0.627764670218)*x[2]**o+(0.842131632598)*x[2]
        arg[3,1,2,0]=(-0.957096489189)*x[0]**o+(0.0616037805677)*x[0]+(0.255858200166)*x[1]**o+(-0.430555450798)*x[1]+(0.730436930737)*x[2]**o+(0.405094271653)*x[2]
        arg[3,1,2,1]=(-0.924756832665)*x[0]**o+(0.525754162391)*x[0]+(0.281303228038)*x[1]**o+(-0.479500161878)*x[1]+(0.658416209181)*x[2]**o+(-0.0413881595557)*x[2]
        arg[3,2,0,0]=(0.516429381997)*x[0]**o+(-0.8387318467)*x[0]+(0.329773978227)*x[1]**o+(0.599641753085)*x[1]+(0.939937743271)*x[2]**o+(-0.247331169626)*x[2]
        arg[3,2,0,1]=(-0.644982044558)*x[0]**o+(-0.242033352774)*x[0]+(0.531835185779)*x[1]**o+(-0.851565064183)*x[1]+(-0.79213624915)*x[2]**o+(0.0607365690732)*x[2]
        arg[3,2,1,0]=(0.359781383537)*x[0]**o+(-0.0529212842074)*x[0]+(0.845915169452)*x[1]**o+(-0.323979570458)*x[1]+(0.828909997631)*x[2]**o+(-0.839886339138)*x[2]
        arg[3,2,1,1]=(0.559476630421)*x[0]**o+(-0.189725214615)*x[0]+(-0.00927446700413)*x[1]**o+(-0.234703720782)*x[1]+(-0.608362806755)*x[2]**o+(0.968571765534)*x[2]
        arg[3,2,2,0]=(0.924837507913)*x[0]**o+(0.846397611121)*x[0]+(-0.837765472403)*x[1]**o+(0.0225105985057)*x[1]+(-0.834187513844)*x[2]**o+(0.194311845578)*x[2]
        arg[3,2,2,1]=(-0.183561089961)*x[0]**o+(0.0745071729471)*x[0]+(-0.660415615202)*x[1]**o+(-0.528184997345)*x[1]+(0.205554094557)*x[2]**o+(-0.0309246318944)*x[2]
        arg[3,3,0,0]=(-0.67124095974)*x[0]**o+(-0.48501497083)*x[0]+(-0.0394345869094)*x[1]**o+(0.586692270429)*x[1]+(-0.593760749687)*x[2]**o+(-0.124979751327)*x[2]
        arg[3,3,0,1]=(0.778544173855)*x[0]**o+(-0.931466478494)*x[0]+(-0.746894777998)*x[1]**o+(0.806417936297)*x[1]+(0.989460929848)*x[2]**o+(0.666534366369)*x[2]
        arg[3,3,1,0]=(-0.798870838602)*x[0]**o+(-0.294618868937)*x[0]+(0.275029092968)*x[1]**o+(0.45390580364)*x[1]+(-0.311797818508)*x[2]**o+(0.751606624393)*x[2]
        arg[3,3,1,1]=(-0.924260987477)*x[0]**o+(0.771970726902)*x[0]+(-0.909099224213)*x[1]**o+(-0.45204206766)*x[1]+(0.630042614017)*x[2]**o+(-0.990915130464)*x[2]
        arg[3,3,2,0]=(0.734962154852)*x[0]**o+(-0.00621898011475)*x[0]+(-0.786362527189)*x[1]**o+(0.623507947406)*x[1]+(0.867175567104)*x[2]**o+(-0.383283255448)*x[2]
        arg[3,3,2,1]=(-0.955613156717)*x[0]**o+(0.688013754841)*x[0]+(-0.600559375208)*x[1]**o+(0.453342626645)*x[1]+(-0.864890477961)*x[2]**o+(-0.458029873063)*x[2]
        arg[3,4,0,0]=(-0.897010884834)*x[0]**o+(-0.251223576458)*x[0]+(-0.0338052047964)*x[1]**o+(-0.903421466132)*x[1]+(0.836864217874)*x[2]**o+(0.660805905175)*x[2]
        arg[3,4,0,1]=(-0.286025690215)*x[0]**o+(0.746732975604)*x[0]+(0.375810853218)*x[1]**o+(-0.985192974869)*x[1]+(-0.731463214455)*x[2]**o+(-0.465450352444)*x[2]
        arg[3,4,1,0]=(-0.500910000652)*x[0]**o+(0.387502419727)*x[0]+(0.962369164802)*x[1]**o+(-0.10933824251)*x[1]+(-0.33642703989)*x[2]**o+(0.546851133256)*x[2]
        arg[3,4,1,1]=(-0.117959459645)*x[0]**o+(0.867699323392)*x[0]+(0.950506403861)*x[1]**o+(0.613806207745)*x[1]+(0.425723173807)*x[2]**o+(-0.906937548619)*x[2]
        arg[3,4,2,0]=(0.331114892194)*x[0]**o+(-0.168829643424)*x[0]+(-0.855962311982)*x[1]**o+(-0.144202406673)*x[1]+(0.0769985713088)*x[2]**o+(0.856003746423)*x[2]
        arg[3,4,2,1]=(-0.439397439364)*x[0]**o+(0.686116343481)*x[0]+(0.519883854523)*x[1]**o+(0.716231469289)*x[1]+(0.270098231149)*x[2]**o+(-0.448525436427)*x[2]
        ref[0,0,0,0]=(-0.401901998082)*x_ref[0]**o+(-0.805091116808)*x_ref[0]+(0.997659882792)*x_ref[1]**o+(-0.953957389492)*x_ref[1]+(-0.450185408289)*x_ref[2]**o+(0.636472207773)*x_ref[2]
        ref[0,0,0,1]=(-0.786386817128)*x_ref[0]**o+(-0.882559775393)*x_ref[0]+(0.117871400819)*x_ref[1]**o+(-0.625922274774)*x_ref[1]+(0.410431886931)*x_ref[2]**o+(-0.979356501067)*x_ref[2]
        ref[0,0,1,0]=(0.686025409275)*x_ref[0]**o+(-0.19806907659)*x_ref[0]+(-0.710420224449)*x_ref[1]**o+(-0.839236953983)*x_ref[1]+(-0.79971896093)*x_ref[2]**o+(0.378944111319)*x_ref[2]
        ref[0,0,1,1]=(0.771205112484)*x_ref[0]**o+(-0.007083192859)*x_ref[0]+(0.672918588001)*x_ref[1]**o+(-0.351652495793)*x_ref[1]+(-0.841767161187)*x_ref[2]**o+(-0.345642373014)*x_ref[2]
        ref[0,0,2,0]=(0.266930729015)*x_ref[0]**o+(-0.873808152307)*x_ref[0]+(0.206466759135)*x_ref[1]**o+(0.539888649948)*x_ref[1]+(0.17985862191)*x_ref[2]**o+(0.240651460153)*x_ref[2]
        ref[0,0,2,1]=(-0.232569768041)*x_ref[0]**o+(-0.579841454072)*x_ref[0]+(-0.706956104891)*x_ref[1]**o+(0.822863815698)*x_ref[1]+(0.345774991358)*x_ref[2]**o+(-0.0848567668311)*x_ref[2]
        ref[0,1,0,0]=(0.51593851507)*x_ref[0]**o+(-0.233342434349)*x_ref[0]+(-0.92498298026)*x_ref[1]**o+(0.164540234829)*x_ref[1]+(0.356559744499)*x_ref[2]**o+(0.483902511717)*x_ref[2]
        ref[0,1,0,1]=(-0.811509654443)*x_ref[0]**o+(0.73929453189)*x_ref[0]+(0.256205706418)*x_ref[1]**o+(-0.667924913793)*x_ref[1]+(0.266065641257)*x_ref[2]**o+(0.856490258029)*x_ref[2]
        ref[0,1,1,0]=(0.502051590284)*x_ref[0]**o+(0.927283797128)*x_ref[0]+(0.500854521411)*x_ref[1]**o+(-0.486326996369)*x_ref[1]+(-0.327578523685)*x_ref[2]**o+(-0.28106070397)*x_ref[2]
        ref[0,1,1,1]=(0.201740014513)*x_ref[0]**o+(-0.166988241759)*x_ref[0]+(-0.513072955566)*x_ref[1]**o+(0.67179532417)*x_ref[1]+(-0.72135980627)*x_ref[2]**o+(0.763548886467)*x_ref[2]
        ref[0,1,2,0]=(0.3995590338)*x_ref[0]**o+(0.433286929621)*x_ref[0]+(-0.287309453792)*x_ref[1]**o+(-0.0671679443074)*x_ref[1]+(-0.80815399993)*x_ref[2]**o+(-0.398740362114)*x_ref[2]
        ref[0,1,2,1]=(-0.16314640684)*x_ref[0]**o+(-0.698681697367)*x_ref[0]+(-0.11609738418)*x_ref[1]**o+(-0.798465154739)*x_ref[1]+(-0.169052628971)*x_ref[2]**o+(-0.880316962539)*x_ref[2]
        ref[0,2,0,0]=(-0.219731279442)*x_ref[0]**o+(0.731464233112)*x_ref[0]+(0.637811533993)*x_ref[1]**o+(0.381218512692)*x_ref[1]+(-0.181357790048)*x_ref[2]**o+(-0.296793024723)*x_ref[2]
        ref[0,2,0,1]=(0.566451624572)*x_ref[0]**o+(0.302904935559)*x_ref[0]+(-0.771526665232)*x_ref[1]**o+(0.460577683302)*x_ref[1]+(-0.921312455507)*x_ref[2]**o+(0.604299765394)*x_ref[2]
        ref[0,2,1,0]=(-0.0887516895197)*x_ref[0]**o+(0.872208558285)*x_ref[0]+(-0.306564750999)*x_ref[1]**o+(0.841982500263)*x_ref[1]+(-0.550398737046)*x_ref[2]**o+(-0.869697162427)*x_ref[2]
        ref[0,2,1,1]=(-0.652471383912)*x_ref[0]**o+(0.634592304393)*x_ref[0]+(0.0709212473167)*x_ref[1]**o+(-0.671330082579)*x_ref[1]+(-0.232895004478)*x_ref[2]**o+(0.479008351165)*x_ref[2]
        ref[0,2,2,0]=(-0.0764115046136)*x_ref[0]**o+(0.649230346025)*x_ref[0]+(0.544503480999)*x_ref[1]**o+(0.788838981797)*x_ref[1]+(0.416236148267)*x_ref[2]**o+(0.722979541918)*x_ref[2]
        ref[0,2,2,1]=(-0.0776368628837)*x_ref[0]**o+(0.801984674386)*x_ref[0]+(0.443670313486)*x_ref[1]**o+(0.424676792255)*x_ref[1]+(-0.895989937982)*x_ref[2]**o+(0.0774648143589)*x_ref[2]
        ref[0,3,0,0]=(-0.39041318438)*x_ref[0]**o+(-0.174335409262)*x_ref[0]+(0.658666482337)*x_ref[1]**o+(0.23060605921)*x_ref[1]+(-0.336603364419)*x_ref[2]**o+(0.871530192198)*x_ref[2]
        ref[0,3,0,1]=(-0.493104661363)*x_ref[0]**o+(0.590779697499)*x_ref[0]+(-0.903897457443)*x_ref[1]**o+(-0.377640520704)*x_ref[1]+(0.227174544833)*x_ref[2]**o+(-0.860150483539)*x_ref[2]
        ref[0,3,1,0]=(-0.093367253954)*x_ref[0]**o+(0.159230499811)*x_ref[0]+(-0.731396509936)*x_ref[1]**o+(0.600088148838)*x_ref[1]+(0.829175660147)*x_ref[2]**o+(-0.435878280919)*x_ref[2]
        ref[0,3,1,1]=(0.665125540711)*x_ref[0]**o+(-0.92128737118)*x_ref[0]+(-0.33260297543)*x_ref[1]**o+(0.682569476816)*x_ref[1]+(0.999761405058)*x_ref[2]**o+(-0.748303702618)*x_ref[2]
        ref[0,3,2,0]=(-0.430442277525)*x_ref[0]**o+(-0.841437664378)*x_ref[0]+(-0.374267371055)*x_ref[1]**o+(0.667860327373)*x_ref[1]+(0.694185699238)*x_ref[2]**o+(-0.673629629622)*x_ref[2]
        ref[0,3,2,1]=(0.304808617243)*x_ref[0]**o+(-0.894407776037)*x_ref[0]+(-0.758585281699)*x_ref[1]**o+(0.20519963241)*x_ref[1]+(0.538222015181)*x_ref[2]**o+(-0.792962435573)*x_ref[2]
        ref[0,4,0,0]=(0.568300300269)*x_ref[0]**o+(0.407455936625)*x_ref[0]+(0.980898344535)*x_ref[1]**o+(0.988220474461)*x_ref[1]+(0.0598892620089)*x_ref[2]**o+(-0.279555207723)*x_ref[2]
        ref[0,4,0,1]=(-0.208391854989)*x_ref[0]**o+(0.9428019078)*x_ref[0]+(-0.540916877275)*x_ref[1]**o+(-0.427185538555)*x_ref[1]+(-0.879268085857)*x_ref[2]**o+(0.025102321099)*x_ref[2]
        ref[0,4,1,0]=(0.412690112527)*x_ref[0]**o+(-0.957376863734)*x_ref[0]+(-0.913623139719)*x_ref[1]**o+(-0.462418849979)*x_ref[1]+(0.463336747019)*x_ref[2]**o+(0.330747531293)*x_ref[2]
        ref[0,4,1,1]=(-0.632140552118)*x_ref[0]**o+(0.830769577671)*x_ref[0]+(-0.888012251479)*x_ref[1]**o+(-0.0560181569699)*x_ref[1]+(-0.70068099248)*x_ref[2]**o+(0.834483253039)*x_ref[2]
        ref[0,4,2,0]=(0.0429717567385)*x_ref[0]**o+(-0.691784042595)*x_ref[0]+(-0.264988764713)*x_ref[1]**o+(0.359873170931)*x_ref[1]+(-0.116275128437)*x_ref[2]**o+(0.0348873917345)*x_ref[2]
        ref[0,4,2,1]=(-0.203754358311)*x_ref[0]**o+(0.177896174646)*x_ref[0]+(-0.0785920580727)*x_ref[1]**o+(-0.124905949184)*x_ref[1]+(0.968871355367)*x_ref[2]**o+(-0.0369722247589)*x_ref[2]
        ref[1,0,0,0]=(0.594851756786)*x_ref[0]**o+(-0.594400822936)*x_ref[0]+(0.868401350949)*x_ref[1]**o+(0.186739616297)*x_ref[1]+(-0.937219946754)*x_ref[2]**o+(-0.248916422131)*x_ref[2]
        ref[1,0,0,1]=(-0.639865978203)*x_ref[0]**o+(-0.00990375501714)*x_ref[0]+(-0.038266357411)*x_ref[1]**o+(-0.741522380136)*x_ref[1]+(0.460572922772)*x_ref[2]**o+(0.4314205896)*x_ref[2]
        ref[1,0,1,0]=(-0.460110413486)*x_ref[0]**o+(-0.0964934735182)*x_ref[0]+(0.474209024352)*x_ref[1]**o+(-0.928476110805)*x_ref[1]+(0.598621411487)*x_ref[2]**o+(-0.00858812754345)*x_ref[2]
        ref[1,0,1,1]=(0.936466172807)*x_ref[0]**o+(-0.363937334197)*x_ref[0]+(-0.540224651799)*x_ref[1]**o+(0.484530722263)*x_ref[1]+(0.675416993296)*x_ref[2]**o+(0.89657344343)*x_ref[2]
        ref[1,0,2,0]=(-0.713050692555)*x_ref[0]**o+(-0.00851892179229)*x_ref[0]+(0.689494113809)*x_ref[1]**o+(0.633341028044)*x_ref[1]+(0.272726911113)*x_ref[2]**o+(-0.122017566819)*x_ref[2]
        ref[1,0,2,1]=(0.793152080981)*x_ref[0]**o+(0.287379739639)*x_ref[0]+(0.304047372771)*x_ref[1]**o+(-0.509079361053)*x_ref[1]+(0.888302006985)*x_ref[2]**o+(-0.297023045775)*x_ref[2]
        ref[1,1,0,0]=(-0.69695118901)*x_ref[0]**o+(0.505182668512)*x_ref[0]+(0.827555590952)*x_ref[1]**o+(-0.777232447056)*x_ref[1]+(-0.460918138036)*x_ref[2]**o+(0.782090206922)*x_ref[2]
        ref[1,1,0,1]=(-0.435398535566)*x_ref[0]**o+(-0.0997593681984)*x_ref[0]+(0.140250252691)*x_ref[1]**o+(-0.0204469256488)*x_ref[1]+(-0.445836771028)*x_ref[2]**o+(-0.775366037861)*x_ref[2]
        ref[1,1,1,0]=(0.918015347957)*x_ref[0]**o+(-0.491940111489)*x_ref[0]+(0.793071617663)*x_ref[1]**o+(-0.607035294859)*x_ref[1]+(0.610076618566)*x_ref[2]**o+(0.148116733513)*x_ref[2]
        ref[1,1,1,1]=(0.224062518811)*x_ref[0]**o+(-0.145897156735)*x_ref[0]+(0.165380866901)*x_ref[1]**o+(0.910562981814)*x_ref[1]+(-0.406620561548)*x_ref[2]**o+(0.528862812468)*x_ref[2]
        ref[1,1,2,0]=(0.85148444093)*x_ref[0]**o+(0.13210280653)*x_ref[0]+(0.364136256712)*x_ref[1]**o+(0.520388575386)*x_ref[1]+(0.0145641287346)*x_ref[2]**o+(-0.7849036351)*x_ref[2]
        ref[1,1,2,1]=(0.493837193832)*x_ref[0]**o+(-0.193326176569)*x_ref[0]+(-0.665920972154)*x_ref[1]**o+(-0.321645070095)*x_ref[1]+(0.0209354853869)*x_ref[2]**o+(-0.328315981752)*x_ref[2]
        ref[1,2,0,0]=(-0.349281164766)*x_ref[0]**o+(-0.318415198113)*x_ref[0]+(-0.687972781234)*x_ref[1]**o+(0.91825998605)*x_ref[1]+(0.396868773798)*x_ref[2]**o+(0.998204458027)*x_ref[2]
        ref[1,2,0,1]=(-0.967493222935)*x_ref[0]**o+(-0.721128465653)*x_ref[0]+(-0.672178802974)*x_ref[1]**o+(0.0924145224431)*x_ref[1]+(0.114040587594)*x_ref[2]**o+(0.707623895537)*x_ref[2]
        ref[1,2,1,0]=(-0.253327088773)*x_ref[0]**o+(0.206063827625)*x_ref[0]+(0.355152654895)*x_ref[1]**o+(-0.27339285621)*x_ref[1]+(-0.0135567122291)*x_ref[2]**o+(-0.983990446362)*x_ref[2]
        ref[1,2,1,1]=(0.98471687132)*x_ref[0]**o+(0.262902745655)*x_ref[0]+(-0.589359138227)*x_ref[1]**o+(-0.0982548958556)*x_ref[1]+(-0.526161200133)*x_ref[2]**o+(-0.850939484752)*x_ref[2]
        ref[1,2,2,0]=(0.37583596705)*x_ref[0]**o+(0.274845981222)*x_ref[0]+(0.601331110943)*x_ref[1]**o+(0.788294448218)*x_ref[1]+(0.366445166459)*x_ref[2]**o+(-0.554935072394)*x_ref[2]
        ref[1,2,2,1]=(-0.986387850237)*x_ref[0]**o+(0.68772989781)*x_ref[0]+(0.345166604933)*x_ref[1]**o+(0.930376726384)*x_ref[1]+(0.760974421701)*x_ref[2]**o+(0.261797396808)*x_ref[2]
        ref[1,3,0,0]=(-0.171005013523)*x_ref[0]**o+(-0.339229596812)*x_ref[0]+(-0.150317419556)*x_ref[1]**o+(-0.0146533427463)*x_ref[1]+(0.957020663875)*x_ref[2]**o+(0.616513940347)*x_ref[2]
        ref[1,3,0,1]=(-0.0727328269499)*x_ref[0]**o+(-0.635591748073)*x_ref[0]+(-0.502426989774)*x_ref[1]**o+(0.772595119619)*x_ref[1]+(-0.76006511271)*x_ref[2]**o+(-0.665920288084)*x_ref[2]
        ref[1,3,1,0]=(-0.394284205622)*x_ref[0]**o+(-0.105959165811)*x_ref[0]+(0.72649045701)*x_ref[1]**o+(-0.970276379473)*x_ref[1]+(-0.236520349415)*x_ref[2]**o+(0.994973267783)*x_ref[2]
        ref[1,3,1,1]=(-0.444149867906)*x_ref[0]**o+(0.748449201475)*x_ref[0]+(0.419604170756)*x_ref[1]**o+(-0.0291702705178)*x_ref[1]+(-0.109580142628)*x_ref[2]**o+(-0.0791870699838)*x_ref[2]
        ref[1,3,2,0]=(-0.629657820523)*x_ref[0]**o+(-0.995576552624)*x_ref[0]+(0.530135462206)*x_ref[1]**o+(0.197336335845)*x_ref[1]+(0.0224815558807)*x_ref[2]**o+(0.240498463912)*x_ref[2]
        ref[1,3,2,1]=(0.52933948346)*x_ref[0]**o+(0.98353913011)*x_ref[0]+(0.0217614943138)*x_ref[1]**o+(0.0157778453682)*x_ref[1]+(0.177956932986)*x_ref[2]**o+(0.234629778706)*x_ref[2]
        ref[1,4,0,0]=(0.593028367137)*x_ref[0]**o+(0.0340892538994)*x_ref[0]+(-0.0295133170051)*x_ref[1]**o+(0.0865921041963)*x_ref[1]+(0.641131718656)*x_ref[2]**o+(0.788858018609)*x_ref[2]
        ref[1,4,0,1]=(-0.283333820634)*x_ref[0]**o+(0.864482725099)*x_ref[0]+(-0.560218794286)*x_ref[1]**o+(-0.0083115362546)*x_ref[1]+(0.726423285129)*x_ref[2]**o+(0.031531339166)*x_ref[2]
        ref[1,4,1,0]=(0.801452411554)*x_ref[0]**o+(-0.59916393038)*x_ref[0]+(-0.106568863591)*x_ref[1]**o+(0.357442125133)*x_ref[1]+(-0.203380280132)*x_ref[2]**o+(-0.129031606441)*x_ref[2]
        ref[1,4,1,1]=(-0.589220928637)*x_ref[0]**o+(-0.308949389428)*x_ref[0]+(0.785549858338)*x_ref[1]**o+(-0.131153853015)*x_ref[1]+(-0.30571855555)*x_ref[2]**o+(-0.329634802588)*x_ref[2]
        ref[1,4,2,0]=(-0.479686944)*x_ref[0]**o+(-0.707702026873)*x_ref[0]+(0.635894172352)*x_ref[1]**o+(0.370673635069)*x_ref[1]+(0.942862875991)*x_ref[2]**o+(-0.846943456738)*x_ref[2]
        ref[1,4,2,1]=(-0.403737324968)*x_ref[0]**o+(0.101664426734)*x_ref[0]+(-0.677093482354)*x_ref[1]**o+(-0.36526066745)*x_ref[1]+(-0.0889619136182)*x_ref[2]**o+(-0.772491940844)*x_ref[2]
        ref[2,0,0,0]=(0.753148325987)*x_ref[0]**o+(-0.342843083128)*x_ref[0]+(0.812661029553)*x_ref[1]**o+(-0.112683035459)*x_ref[1]+(0.900976470275)*x_ref[2]**o+(0.00934086955102)*x_ref[2]
        ref[2,0,0,1]=(-0.682315007143)*x_ref[0]**o+(-0.694069900861)*x_ref[0]+(-0.469082247747)*x_ref[1]**o+(-0.522165903301)*x_ref[1]+(-0.0997886221924)*x_ref[2]**o+(-0.954161552419)*x_ref[2]
        ref[2,0,1,0]=(0.780125539705)*x_ref[0]**o+(-0.445331450225)*x_ref[0]+(-0.282765510424)*x_ref[1]**o+(-0.0131898361592)*x_ref[1]+(-0.200186433923)*x_ref[2]**o+(0.715139435696)*x_ref[2]
        ref[2,0,1,1]=(-0.132310660786)*x_ref[0]**o+(-0.0981344234369)*x_ref[0]+(0.959297529475)*x_ref[1]**o+(-0.85944894515)*x_ref[1]+(-0.396258659619)*x_ref[2]**o+(-0.09672066185)*x_ref[2]
        ref[2,0,2,0]=(-0.344134014191)*x_ref[0]**o+(0.534345864287)*x_ref[0]+(0.797209509484)*x_ref[1]**o+(0.0479050101499)*x_ref[1]+(0.13620476987)*x_ref[2]**o+(0.82708115383)*x_ref[2]
        ref[2,0,2,1]=(-0.342509328238)*x_ref[0]**o+(-0.833164493006)*x_ref[0]+(0.472047721532)*x_ref[1]**o+(0.301613158637)*x_ref[1]+(0.858691679638)*x_ref[2]**o+(-0.691744164006)*x_ref[2]
        ref[2,1,0,0]=(0.498979390299)*x_ref[0]**o+(0.371513881824)*x_ref[0]+(-0.243506167971)*x_ref[1]**o+(-0.245423890439)*x_ref[1]+(-0.0494707840274)*x_ref[2]**o+(0.0965573709928)*x_ref[2]
        ref[2,1,0,1]=(0.665982455862)*x_ref[0]**o+(-0.409710866336)*x_ref[0]+(0.669257568046)*x_ref[1]**o+(0.711947704788)*x_ref[1]+(-0.0362568635785)*x_ref[2]**o+(-0.30394278188)*x_ref[2]
        ref[2,1,1,0]=(0.296245249928)*x_ref[0]**o+(0.132631019281)*x_ref[0]+(-0.376880152158)*x_ref[1]**o+(0.303184156241)*x_ref[1]+(0.836046560882)*x_ref[2]**o+(0.195205645106)*x_ref[2]
        ref[2,1,1,1]=(-0.971045820953)*x_ref[0]**o+(0.154708896467)*x_ref[0]+(-0.663844656602)*x_ref[1]**o+(-0.0873797588245)*x_ref[1]+(0.985123372542)*x_ref[2]**o+(0.349554922131)*x_ref[2]
        ref[2,1,2,0]=(-0.0117443431348)*x_ref[0]**o+(-0.71980503985)*x_ref[0]+(0.175348078994)*x_ref[1]**o+(-0.883987892161)*x_ref[1]+(0.0587275450458)*x_ref[2]**o+(0.464811715548)*x_ref[2]
        ref[2,1,2,1]=(-0.309202575765)*x_ref[0]**o+(-0.528513248453)*x_ref[0]+(-0.509298303972)*x_ref[1]**o+(-0.228278863496)*x_ref[1]+(0.933917053676)*x_ref[2]**o+(-0.823678942264)*x_ref[2]
        ref[2,2,0,0]=(0.374387386929)*x_ref[0]**o+(0.64969993847)*x_ref[0]+(-0.939822635641)*x_ref[1]**o+(0.578249430422)*x_ref[1]+(-0.973412660635)*x_ref[2]**o+(0.484052718386)*x_ref[2]
        ref[2,2,0,1]=(-0.972635162618)*x_ref[0]**o+(-0.255007433802)*x_ref[0]+(0.134165187097)*x_ref[1]**o+(0.313213615976)*x_ref[1]+(0.143298148686)*x_ref[2]**o+(0.901819440229)*x_ref[2]
        ref[2,2,1,0]=(0.868173256315)*x_ref[0]**o+(-0.532325343453)*x_ref[0]+(-0.712500880845)*x_ref[1]**o+(0.216943462308)*x_ref[1]+(0.733920091212)*x_ref[2]**o+(0.0992400603383)*x_ref[2]
        ref[2,2,1,1]=(0.642839350527)*x_ref[0]**o+(0.127721543559)*x_ref[0]+(-0.152686523032)*x_ref[1]**o+(-0.399032361234)*x_ref[1]+(0.317014518842)*x_ref[2]**o+(0.905538790115)*x_ref[2]
        ref[2,2,2,0]=(0.685267932413)*x_ref[0]**o+(-0.167571547782)*x_ref[0]+(-0.213446144405)*x_ref[1]**o+(0.420270456377)*x_ref[1]+(0.827725350621)*x_ref[2]**o+(-0.920242274399)*x_ref[2]
        ref[2,2,2,1]=(-0.507424593437)*x_ref[0]**o+(-0.694222840019)*x_ref[0]+(-0.904108970188)*x_ref[1]**o+(-0.750899614966)*x_ref[1]+(0.783997553302)*x_ref[2]**o+(0.196327513447)*x_ref[2]
        ref[2,3,0,0]=(-0.473900600479)*x_ref[0]**o+(-0.464347097789)*x_ref[0]+(-0.472423358671)*x_ref[1]**o+(0.373786456364)*x_ref[1]+(-0.989371300414)*x_ref[2]**o+(-0.959508065239)*x_ref[2]
        ref[2,3,0,1]=(0.174375189399)*x_ref[0]**o+(0.925503059886)*x_ref[0]+(-0.647274384723)*x_ref[1]**o+(-0.577149479679)*x_ref[1]+(0.171298877768)*x_ref[2]**o+(-0.936612100882)*x_ref[2]
        ref[2,3,1,0]=(0.482959343396)*x_ref[0]**o+(0.842243554011)*x_ref[0]+(0.814423633314)*x_ref[1]**o+(-0.989445050315)*x_ref[1]+(0.817933433065)*x_ref[2]**o+(0.552396881404)*x_ref[2]
        ref[2,3,1,1]=(0.55641736229)*x_ref[0]**o+(0.498094640058)*x_ref[0]+(0.0890113606037)*x_ref[1]**o+(0.36592723553)*x_ref[1]+(0.458607593386)*x_ref[2]**o+(0.603047420063)*x_ref[2]
        ref[2,3,2,0]=(0.884515080124)*x_ref[0]**o+(-0.561898381022)*x_ref[0]+(-0.292475503834)*x_ref[1]**o+(0.425634423478)*x_ref[1]+(0.65813323252)*x_ref[2]**o+(-0.548850399842)*x_ref[2]
        ref[2,3,2,1]=(-0.72895566982)*x_ref[0]**o+(-0.569405588005)*x_ref[0]+(0.117770875594)*x_ref[1]**o+(-0.33515735969)*x_ref[1]+(0.102062736382)*x_ref[2]**o+(0.611532417683)*x_ref[2]
        ref[2,4,0,0]=(-0.669208630424)*x_ref[0]**o+(0.944393014177)*x_ref[0]+(-0.00345511889683)*x_ref[1]**o+(0.0695279595783)*x_ref[1]+(0.496948250833)*x_ref[2]**o+(-0.899750463971)*x_ref[2]
        ref[2,4,0,1]=(-0.387828717292)*x_ref[0]**o+(0.621172267004)*x_ref[0]+(-0.280632674111)*x_ref[1]**o+(0.401702567575)*x_ref[1]+(-0.65554489378)*x_ref[2]**o+(0.403018047007)*x_ref[2]
        ref[2,4,1,0]=(-0.0770246925803)*x_ref[0]**o+(0.52913302745)*x_ref[0]+(0.922468724357)*x_ref[1]**o+(-0.304442750439)*x_ref[1]+(-0.556575079059)*x_ref[2]**o+(-0.725790672251)*x_ref[2]
        ref[2,4,1,1]=(-0.828378342911)*x_ref[0]**o+(-0.0696370821776)*x_ref[0]+(0.394018581667)*x_ref[1]**o+(-0.251099848269)*x_ref[1]+(-0.29878985656)*x_ref[2]**o+(0.925753371056)*x_ref[2]
        ref[2,4,2,0]=(-0.818277786029)*x_ref[0]**o+(0.225862354706)*x_ref[0]+(-0.629050212595)*x_ref[1]**o+(0.139186107062)*x_ref[1]+(-0.0134835519238)*x_ref[2]**o+(0.301899528641)*x_ref[2]
        ref[2,4,2,1]=(-0.783053684767)*x_ref[0]**o+(-0.0926938783599)*x_ref[0]+(0.519034210352)*x_ref[1]**o+(-0.494564657126)*x_ref[1]+(0.536117310622)*x_ref[2]**o+(0.6278756634)*x_ref[2]
        ref[3,0,0,0]=(-0.694353957685)*x_ref[0]**o+(-0.361037363965)*x_ref[0]+(-0.327004091885)*x_ref[1]**o+(-0.765735142884)*x_ref[1]+(-0.810915729713)*x_ref[2]**o+(0.0640539311609)*x_ref[2]
        ref[3,0,0,1]=(-0.131601548312)*x_ref[0]**o+(0.271587756708)*x_ref[0]+(0.322486168005)*x_ref[1]**o+(0.238086581801)*x_ref[1]+(-0.488081720555)*x_ref[2]**o+(0.607278995635)*x_ref[2]
        ref[3,0,1,0]=(0.903075418102)*x_ref[0]**o+(-0.738609834848)*x_ref[0]+(-0.165097989126)*x_ref[1]**o+(-0.25967140914)*x_ref[1]+(0.271649887428)*x_ref[2]**o+(-0.786131648727)*x_ref[2]
        ref[3,0,1,1]=(-0.963442989157)*x_ref[0]**o+(-0.240205602334)*x_ref[0]+(0.102153389731)*x_ref[1]**o+(-0.106402434995)*x_ref[1]+(-0.383097724211)*x_ref[2]**o+(0.680290963195)*x_ref[2]
        ref[3,0,2,0]=(-0.373162835796)*x_ref[0]**o+(-0.186323761814)*x_ref[0]+(0.661494315705)*x_ref[1]**o+(0.320072189494)*x_ref[1]+(0.0964443431067)*x_ref[2]**o+(0.829015838241)*x_ref[2]
        ref[3,0,2,1]=(0.527719514862)*x_ref[0]**o+(0.0819948261665)*x_ref[0]+(0.826598712178)*x_ref[1]**o+(-0.588102975125)*x_ref[1]+(-0.632584402292)*x_ref[2]**o+(0.0826992233702)*x_ref[2]
        ref[3,1,0,0]=(-0.102243686287)*x_ref[0]**o+(-0.497336112929)*x_ref[0]+(0.941827592628)*x_ref[1]**o+(0.68678991944)*x_ref[1]+(-0.508684841667)*x_ref[2]**o+(-0.752939965161)*x_ref[2]
        ref[3,1,0,1]=(-0.896417523183)*x_ref[0]**o+(0.759224781685)*x_ref[0]+(0.138537695823)*x_ref[1]**o+(0.466727480471)*x_ref[1]+(-0.838380824412)*x_ref[2]**o+(0.285761157398)*x_ref[2]
        ref[3,1,1,0]=(-0.547853182919)*x_ref[0]**o+(-0.41595413414)*x_ref[0]+(0.76894221584)*x_ref[1]**o+(-0.142394275192)*x_ref[1]+(-0.96349294581)*x_ref[2]**o+(-0.873639964693)*x_ref[2]
        ref[3,1,1,1]=(-0.151299403888)*x_ref[0]**o+(0.789406063878)*x_ref[0]+(0.163280670253)*x_ref[1]**o+(-0.287315494745)*x_ref[1]+(-0.627764670218)*x_ref[2]**o+(0.842131632598)*x_ref[2]
        ref[3,1,2,0]=(-0.957096489189)*x_ref[0]**o+(0.0616037805677)*x_ref[0]+(0.255858200166)*x_ref[1]**o+(-0.430555450798)*x_ref[1]+(0.730436930737)*x_ref[2]**o+(0.405094271653)*x_ref[2]
        ref[3,1,2,1]=(-0.924756832665)*x_ref[0]**o+(0.525754162391)*x_ref[0]+(0.281303228038)*x_ref[1]**o+(-0.479500161878)*x_ref[1]+(0.658416209181)*x_ref[2]**o+(-0.0413881595557)*x_ref[2]
        ref[3,2,0,0]=(0.516429381997)*x_ref[0]**o+(-0.8387318467)*x_ref[0]+(0.329773978227)*x_ref[1]**o+(0.599641753085)*x_ref[1]+(0.939937743271)*x_ref[2]**o+(-0.247331169626)*x_ref[2]
        ref[3,2,0,1]=(-0.644982044558)*x_ref[0]**o+(-0.242033352774)*x_ref[0]+(0.531835185779)*x_ref[1]**o+(-0.851565064183)*x_ref[1]+(-0.79213624915)*x_ref[2]**o+(0.0607365690732)*x_ref[2]
        ref[3,2,1,0]=(0.359781383537)*x_ref[0]**o+(-0.0529212842074)*x_ref[0]+(0.845915169452)*x_ref[1]**o+(-0.323979570458)*x_ref[1]+(0.828909997631)*x_ref[2]**o+(-0.839886339138)*x_ref[2]
        ref[3,2,1,1]=(0.559476630421)*x_ref[0]**o+(-0.189725214615)*x_ref[0]+(-0.00927446700413)*x_ref[1]**o+(-0.234703720782)*x_ref[1]+(-0.608362806755)*x_ref[2]**o+(0.968571765534)*x_ref[2]
        ref[3,2,2,0]=(0.924837507913)*x_ref[0]**o+(0.846397611121)*x_ref[0]+(-0.837765472403)*x_ref[1]**o+(0.0225105985057)*x_ref[1]+(-0.834187513844)*x_ref[2]**o+(0.194311845578)*x_ref[2]
        ref[3,2,2,1]=(-0.183561089961)*x_ref[0]**o+(0.0745071729471)*x_ref[0]+(-0.660415615202)*x_ref[1]**o+(-0.528184997345)*x_ref[1]+(0.205554094557)*x_ref[2]**o+(-0.0309246318944)*x_ref[2]
        ref[3,3,0,0]=(-0.67124095974)*x_ref[0]**o+(-0.48501497083)*x_ref[0]+(-0.0394345869094)*x_ref[1]**o+(0.586692270429)*x_ref[1]+(-0.593760749687)*x_ref[2]**o+(-0.124979751327)*x_ref[2]
        ref[3,3,0,1]=(0.778544173855)*x_ref[0]**o+(-0.931466478494)*x_ref[0]+(-0.746894777998)*x_ref[1]**o+(0.806417936297)*x_ref[1]+(0.989460929848)*x_ref[2]**o+(0.666534366369)*x_ref[2]
        ref[3,3,1,0]=(-0.798870838602)*x_ref[0]**o+(-0.294618868937)*x_ref[0]+(0.275029092968)*x_ref[1]**o+(0.45390580364)*x_ref[1]+(-0.311797818508)*x_ref[2]**o+(0.751606624393)*x_ref[2]
        ref[3,3,1,1]=(-0.924260987477)*x_ref[0]**o+(0.771970726902)*x_ref[0]+(-0.909099224213)*x_ref[1]**o+(-0.45204206766)*x_ref[1]+(0.630042614017)*x_ref[2]**o+(-0.990915130464)*x_ref[2]
        ref[3,3,2,0]=(0.734962154852)*x_ref[0]**o+(-0.00621898011475)*x_ref[0]+(-0.786362527189)*x_ref[1]**o+(0.623507947406)*x_ref[1]+(0.867175567104)*x_ref[2]**o+(-0.383283255448)*x_ref[2]
        ref[3,3,2,1]=(-0.955613156717)*x_ref[0]**o+(0.688013754841)*x_ref[0]+(-0.600559375208)*x_ref[1]**o+(0.453342626645)*x_ref[1]+(-0.864890477961)*x_ref[2]**o+(-0.458029873063)*x_ref[2]
        ref[3,4,0,0]=(-0.897010884834)*x_ref[0]**o+(-0.251223576458)*x_ref[0]+(-0.0338052047964)*x_ref[1]**o+(-0.903421466132)*x_ref[1]+(0.836864217874)*x_ref[2]**o+(0.660805905175)*x_ref[2]
        ref[3,4,0,1]=(-0.286025690215)*x_ref[0]**o+(0.746732975604)*x_ref[0]+(0.375810853218)*x_ref[1]**o+(-0.985192974869)*x_ref[1]+(-0.731463214455)*x_ref[2]**o+(-0.465450352444)*x_ref[2]
        ref[3,4,1,0]=(-0.500910000652)*x_ref[0]**o+(0.387502419727)*x_ref[0]+(0.962369164802)*x_ref[1]**o+(-0.10933824251)*x_ref[1]+(-0.33642703989)*x_ref[2]**o+(0.546851133256)*x_ref[2]
        ref[3,4,1,1]=(-0.117959459645)*x_ref[0]**o+(0.867699323392)*x_ref[0]+(0.950506403861)*x_ref[1]**o+(0.613806207745)*x_ref[1]+(0.425723173807)*x_ref[2]**o+(-0.906937548619)*x_ref[2]
        ref[3,4,2,0]=(0.331114892194)*x_ref[0]**o+(-0.168829643424)*x_ref[0]+(-0.855962311982)*x_ref[1]**o+(-0.144202406673)*x_ref[1]+(0.0769985713088)*x_ref[2]**o+(0.856003746423)*x_ref[2]
        ref[3,4,2,1]=(-0.439397439364)*x_ref[0]**o+(0.686116343481)*x_ref[0]+(0.519883854523)*x_ref[1]**o+(0.716231469289)*x_ref[1]+(0.270098231149)*x_ref[2]**o+(-0.448525436427)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_ReducedSolution_rank0(self):
      """
      tests interpolate for rank 0 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedSolution onto ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Data(0,(),w)
      ref=Data(0,(),w_ref)
      if dim==2:
        arg=(-0.787646538574)*x[0]+(0.912422904218)*x[1]
        ref=(-0.787646538574)*x_ref[0]+(0.912422904218)*x_ref[1]
      else:
        arg=(0.934044939024)*x[0]+(0.628477728603)*x[1]+(-0.786339405837)*x[2]
        ref=(0.934044939024)*x_ref[0]+(0.628477728603)*x_ref[1]+(-0.786339405837)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_ReducedSolution_rank1(self):
      """
      tests interpolate for rank 1 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedSolution onto ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Data(0,(2,),w)
      ref=Data(0,(2,),w_ref)
      if dim==2:
        arg[0]=(-0.588326094839)*x[0]+(1.56296941343)*x[1]
        arg[1]=(-0.802191222185)*x[0]+(-1.13471016062)*x[1]
        ref[0]=(-0.588326094839)*x[0]+(1.56296941343)*x[1]
        ref[1]=(-0.802191222185)*x[0]+(-1.13471016062)*x[1]
      else:
        arg[0]=(0.160702771321)*x[0]+(-0.0784425374814)*x[1]+(1.65226639722)*x[2]
        arg[1]=(0.745050820919)*x[0]+(-0.183859360073)*x[1]+(0.399813139488)*x[2]
        ref[0]=(0.160702771321)*x[0]+(-0.0784425374814)*x[1]+(1.65226639722)*x[2]
        ref[1]=(0.745050820919)*x[0]+(-0.183859360073)*x[1]+(0.399813139488)*x[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_ReducedSolution_rank2(self):
      """
      tests interpolate for rank 2 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedSolution onto ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5),w)
      ref=Data(0,(4, 5),w_ref)
      if dim==2:
        arg[0,0]=(-0.797066914479)*x[0]+(-0.0269853164939)*x[1]
        arg[0,1]=(-1.07443661798)*x[0]+(0.818510216043)*x[1]
        arg[0,2]=(0.398735262179)*x[0]+(-0.580387872083)*x[1]
        arg[0,3]=(-0.003185613959)*x[0]+(-0.670506330615)*x[1]
        arg[0,4]=(0.385009570186)*x[0]+(0.858570512936)*x[1]
        arg[1,0]=(1.60917176753)*x[0]+(-0.0866362449992)*x[1]
        arg[1,1]=(-0.883782122452)*x[0]+(-0.402187705578)*x[1]
        arg[1,2]=(0.153893095122)*x[0]+(-1.19636362644)*x[1]
        arg[1,3]=(1.92041961031)*x[0]+(0.325857877636)*x[1]
        arg[1,4]=(-0.0947698843729)*x[0]+(-0.0663785758627)*x[1]
        arg[2,0]=(0.0232949467051)*x[0]+(1.60134210326)*x[1]
        arg[2,1]=(-0.521182882924)*x[0]+(-0.507401571343)*x[1]
        arg[2,2]=(0.833797117616)*x[0]+(-0.268886477194)*x[1]
        arg[2,3]=(-0.867762611227)*x[0]+(1.03727088566)*x[1]
        arg[2,4]=(0.380510651918)*x[0]+(0.350246888456)*x[1]
        arg[3,0]=(-1.10155500839)*x[0]+(0.414683749118)*x[1]
        arg[3,1]=(-0.0306733233552)*x[0]+(-0.351251014237)*x[1]
        arg[3,2]=(-0.0708773001498)*x[0]+(-0.326636587948)*x[1]
        arg[3,3]=(-1.32553119895)*x[0]+(-1.5852157477)*x[1]
        arg[3,4]=(1.40177154802)*x[0]+(-1.24804787994)*x[1]
        ref[0,0]=(-0.797066914479)*x_ref[0]+(-0.0269853164939)*x_ref[1]
        ref[0,1]=(-1.07443661798)*x_ref[0]+(0.818510216043)*x_ref[1]
        ref[0,2]=(0.398735262179)*x_ref[0]+(-0.580387872083)*x_ref[1]
        ref[0,3]=(-0.003185613959)*x_ref[0]+(-0.670506330615)*x_ref[1]
        ref[0,4]=(0.385009570186)*x_ref[0]+(0.858570512936)*x_ref[1]
        ref[1,0]=(1.60917176753)*x_ref[0]+(-0.0866362449992)*x_ref[1]
        ref[1,1]=(-0.883782122452)*x_ref[0]+(-0.402187705578)*x_ref[1]
        ref[1,2]=(0.153893095122)*x_ref[0]+(-1.19636362644)*x_ref[1]
        ref[1,3]=(1.92041961031)*x_ref[0]+(0.325857877636)*x_ref[1]
        ref[1,4]=(-0.0947698843729)*x_ref[0]+(-0.0663785758627)*x_ref[1]
        ref[2,0]=(0.0232949467051)*x_ref[0]+(1.60134210326)*x_ref[1]
        ref[2,1]=(-0.521182882924)*x_ref[0]+(-0.507401571343)*x_ref[1]
        ref[2,2]=(0.833797117616)*x_ref[0]+(-0.268886477194)*x_ref[1]
        ref[2,3]=(-0.867762611227)*x_ref[0]+(1.03727088566)*x_ref[1]
        ref[2,4]=(0.380510651918)*x_ref[0]+(0.350246888456)*x_ref[1]
        ref[3,0]=(-1.10155500839)*x_ref[0]+(0.414683749118)*x_ref[1]
        ref[3,1]=(-0.0306733233552)*x_ref[0]+(-0.351251014237)*x_ref[1]
        ref[3,2]=(-0.0708773001498)*x_ref[0]+(-0.326636587948)*x_ref[1]
        ref[3,3]=(-1.32553119895)*x_ref[0]+(-1.5852157477)*x_ref[1]
        ref[3,4]=(1.40177154802)*x_ref[0]+(-1.24804787994)*x_ref[1]
      else:
        arg[0,0]=(0.000362194678523)*x[0]+(-1.63949220671)*x[1]+(-0.176997156562)*x[2]
        arg[0,1]=(-0.357589049408)*x[0]+(0.828191515325)*x[1]+(-0.269706780136)*x[2]
        arg[0,2]=(0.886395070544)*x[0]+(0.815688451839)*x[1]+(-0.658412270558)*x[2]
        arg[0,3]=(-0.222573318026)*x[0]+(0.137763803092)*x[1]+(0.171902150187)*x[2]
        arg[0,4]=(1.05159741251)*x[0]+(0.923836841038)*x[1]+(-0.426590801731)*x[2]
        arg[1,0]=(-0.00718419057788)*x[0]+(1.55809964998)*x[1]+(-0.912766657496)*x[2]
        arg[1,1]=(1.30769045785)*x[0]+(-0.0356609338389)*x[1]+(0.214543084134)*x[2]
        arg[1,2]=(-1.56763014241)*x[0]+(-0.582970995722)*x[1]+(-0.513972924752)*x[2]
        arg[1,3]=(0.118027736928)*x[0]+(-0.28795014876)*x[1]+(0.620718623034)*x[2]
        arg[1,4]=(-0.784496170193)*x[0]+(0.233037866924)*x[1]+(-0.0500718134572)*x[2]
        arg[2,0]=(0.800851281806)*x[0]+(0.219346484846)*x[1]+(1.19125787701)*x[2]
        arg[2,1]=(-1.45025879429)*x[0]+(-1.32011538154)*x[1]+(0.47946669308)*x[2]
        arg[2,2]=(-0.231377110469)*x[0]+(-0.921551695999)*x[1]+(-0.298964635841)*x[2]
        arg[2,3]=(0.965766335546)*x[0]+(0.424791610089)*x[1]+(1.01860652092)*x[2]
        arg[2,4]=(0.676247093636)*x[0]+(-1.50802741511)*x[1]+(0.0385960918391)*x[2]
        arg[3,0]=(0.322378151756)*x[0]+(-0.483633005509)*x[1]+(-0.852371707865)*x[2]
        arg[3,1]=(-0.241998307153)*x[0]+(-1.08134340082)*x[1]+(-0.849574532537)*x[2]
        arg[3,2]=(-0.0185751405467)*x[0]+(0.179558416001)*x[1]+(-0.488727308548)*x[2]
        arg[3,3]=(0.0984555057845)*x[0]+(1.03085967788)*x[1]+(-0.357009094371)*x[2]
        arg[3,4]=(0.430593980121)*x[0]+(0.689565614489)*x[1]+(-0.0916541808491)*x[2]
        ref[0,0]=(0.000362194678523)*x_ref[0]+(-1.63949220671)*x_ref[1]+(-0.176997156562)*x_ref[2]
        ref[0,1]=(-0.357589049408)*x_ref[0]+(0.828191515325)*x_ref[1]+(-0.269706780136)*x_ref[2]
        ref[0,2]=(0.886395070544)*x_ref[0]+(0.815688451839)*x_ref[1]+(-0.658412270558)*x_ref[2]
        ref[0,3]=(-0.222573318026)*x_ref[0]+(0.137763803092)*x_ref[1]+(0.171902150187)*x_ref[2]
        ref[0,4]=(1.05159741251)*x_ref[0]+(0.923836841038)*x_ref[1]+(-0.426590801731)*x_ref[2]
        ref[1,0]=(-0.00718419057788)*x_ref[0]+(1.55809964998)*x_ref[1]+(-0.912766657496)*x_ref[2]
        ref[1,1]=(1.30769045785)*x_ref[0]+(-0.0356609338389)*x_ref[1]+(0.214543084134)*x_ref[2]
        ref[1,2]=(-1.56763014241)*x_ref[0]+(-0.582970995722)*x_ref[1]+(-0.513972924752)*x_ref[2]
        ref[1,3]=(0.118027736928)*x_ref[0]+(-0.28795014876)*x_ref[1]+(0.620718623034)*x_ref[2]
        ref[1,4]=(-0.784496170193)*x_ref[0]+(0.233037866924)*x_ref[1]+(-0.0500718134572)*x_ref[2]
        ref[2,0]=(0.800851281806)*x_ref[0]+(0.219346484846)*x_ref[1]+(1.19125787701)*x_ref[2]
        ref[2,1]=(-1.45025879429)*x_ref[0]+(-1.32011538154)*x_ref[1]+(0.47946669308)*x_ref[2]
        ref[2,2]=(-0.231377110469)*x_ref[0]+(-0.921551695999)*x_ref[1]+(-0.298964635841)*x_ref[2]
        ref[2,3]=(0.965766335546)*x_ref[0]+(0.424791610089)*x_ref[1]+(1.01860652092)*x_ref[2]
        ref[2,4]=(0.676247093636)*x_ref[0]+(-1.50802741511)*x_ref[1]+(0.0385960918391)*x_ref[2]
        ref[3,0]=(0.322378151756)*x_ref[0]+(-0.483633005509)*x_ref[1]+(-0.852371707865)*x_ref[2]
        ref[3,1]=(-0.241998307153)*x_ref[0]+(-1.08134340082)*x_ref[1]+(-0.849574532537)*x_ref[2]
        ref[3,2]=(-0.0185751405467)*x_ref[0]+(0.179558416001)*x_ref[1]+(-0.488727308548)*x_ref[2]
        ref[3,3]=(0.0984555057845)*x_ref[0]+(1.03085967788)*x_ref[1]+(-0.357009094371)*x_ref[2]
        ref[3,4]=(0.430593980121)*x_ref[0]+(0.689565614489)*x_ref[1]+(-0.0916541808491)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_ReducedSolution_rank3(self):
      """
      tests interpolate for rank 3 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedSolution onto ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2),w_ref)
      if dim==2:
        arg[0,0,0]=(-0.0641544023301)*x[0]+(0.364286357763)*x[1]
        arg[0,0,1]=(-0.858288240411)*x[0]+(-0.847764393559)*x[1]
        arg[0,1,0]=(-0.656948775029)*x[0]+(1.69748608992)*x[1]
        arg[0,1,1]=(0.579399191549)*x[0]+(1.48407368783)*x[1]
        arg[1,0,0]=(-0.35948657403)*x[0]+(0.795436090765)*x[1]
        arg[1,0,1]=(-1.32807156246)*x[0]+(0.921960610294)*x[1]
        arg[1,1,0]=(1.0904085219)*x[0]+(-1.41870997296)*x[1]
        arg[1,1,1]=(-0.677691596793)*x[0]+(-0.395373706333)*x[1]
        arg[2,0,0]=(0.295591917216)*x[0]+(-1.75550830457)*x[1]
        arg[2,0,1]=(0.506665326626)*x[0]+(0.273265713965)*x[1]
        arg[2,1,0]=(0.543899865352)*x[0]+(-0.0678457091657)*x[1]
        arg[2,1,1]=(-0.887824769535)*x[0]+(0.527475538055)*x[1]
        arg[3,0,0]=(-0.788511557365)*x[0]+(-0.635972100675)*x[1]
        arg[3,0,1]=(-1.02799066459)*x[0]+(-0.416241247725)*x[1]
        arg[3,1,0]=(-0.310072483091)*x[0]+(1.2445160445)*x[1]
        arg[3,1,1]=(0.912462196567)*x[0]+(0.631855269783)*x[1]
        arg[4,0,0]=(0.700426861807)*x[0]+(-0.491365913713)*x[1]
        arg[4,0,1]=(0.628791538338)*x[0]+(0.640767645586)*x[1]
        arg[4,1,0]=(0.370886720329)*x[0]+(0.823663448834)*x[1]
        arg[4,1,1]=(1.75857310774)*x[0]+(0.445439424641)*x[1]
        arg[5,0,0]=(1.39637385066)*x[0]+(-0.812211860169)*x[1]
        arg[5,0,1]=(0.865050350195)*x[0]+(-1.25983380375)*x[1]
        arg[5,1,0]=(-0.0429057645445)*x[0]+(-1.08490578934)*x[1]
        arg[5,1,1]=(1.42851264594)*x[0]+(1.52192971749)*x[1]
        ref[0,0,0]=(-0.0641544023301)*x_ref[0]+(0.364286357763)*x_ref[1]
        ref[0,0,1]=(-0.858288240411)*x_ref[0]+(-0.847764393559)*x_ref[1]
        ref[0,1,0]=(-0.656948775029)*x_ref[0]+(1.69748608992)*x_ref[1]
        ref[0,1,1]=(0.579399191549)*x_ref[0]+(1.48407368783)*x_ref[1]
        ref[1,0,0]=(-0.35948657403)*x_ref[0]+(0.795436090765)*x_ref[1]
        ref[1,0,1]=(-1.32807156246)*x_ref[0]+(0.921960610294)*x_ref[1]
        ref[1,1,0]=(1.0904085219)*x_ref[0]+(-1.41870997296)*x_ref[1]
        ref[1,1,1]=(-0.677691596793)*x_ref[0]+(-0.395373706333)*x_ref[1]
        ref[2,0,0]=(0.295591917216)*x_ref[0]+(-1.75550830457)*x_ref[1]
        ref[2,0,1]=(0.506665326626)*x_ref[0]+(0.273265713965)*x_ref[1]
        ref[2,1,0]=(0.543899865352)*x_ref[0]+(-0.0678457091657)*x_ref[1]
        ref[2,1,1]=(-0.887824769535)*x_ref[0]+(0.527475538055)*x_ref[1]
        ref[3,0,0]=(-0.788511557365)*x_ref[0]+(-0.635972100675)*x_ref[1]
        ref[3,0,1]=(-1.02799066459)*x_ref[0]+(-0.416241247725)*x_ref[1]
        ref[3,1,0]=(-0.310072483091)*x_ref[0]+(1.2445160445)*x_ref[1]
        ref[3,1,1]=(0.912462196567)*x_ref[0]+(0.631855269783)*x_ref[1]
        ref[4,0,0]=(0.700426861807)*x_ref[0]+(-0.491365913713)*x_ref[1]
        ref[4,0,1]=(0.628791538338)*x_ref[0]+(0.640767645586)*x_ref[1]
        ref[4,1,0]=(0.370886720329)*x_ref[0]+(0.823663448834)*x_ref[1]
        ref[4,1,1]=(1.75857310774)*x_ref[0]+(0.445439424641)*x_ref[1]
        ref[5,0,0]=(1.39637385066)*x_ref[0]+(-0.812211860169)*x_ref[1]
        ref[5,0,1]=(0.865050350195)*x_ref[0]+(-1.25983380375)*x_ref[1]
        ref[5,1,0]=(-0.0429057645445)*x_ref[0]+(-1.08490578934)*x_ref[1]
        ref[5,1,1]=(1.42851264594)*x_ref[0]+(1.52192971749)*x_ref[1]
      else:
        arg[0,0,0]=(0.731194732468)*x[0]+(-0.0822115660022)*x[1]+(-0.403175091289)*x[2]
        arg[0,0,1]=(0.39292230307)*x[0]+(1.48598681615)*x[1]+(1.05545374107)*x[2]
        arg[0,1,0]=(0.306844260706)*x[0]+(0.836496026763)*x[1]+(-0.121120983155)*x[2]
        arg[0,1,1]=(-0.155245802062)*x[0]+(0.800232934864)*x[1]+(0.373397160636)*x[2]
        arg[1,0,0]=(0.155941790359)*x[0]+(0.826821969281)*x[1]+(0.849596807425)*x[2]
        arg[1,0,1]=(-0.0988854656187)*x[0]+(-0.855032499842)*x[1]+(0.141988288566)*x[2]
        arg[1,1,0]=(-0.761032539296)*x[0]+(-0.0892979073234)*x[1]+(1.93034385132)*x[2]
        arg[1,1,1]=(-1.08303603558)*x[0]+(-0.487250117659)*x[1]+(0.0358501036173)*x[2]
        arg[2,0,0]=(0.148391465413)*x[0]+(-0.731963025625)*x[1]+(0.484236965689)*x[2]
        arg[2,0,1]=(1.06691217722)*x[0]+(-0.638095266244)*x[1]+(-0.971838410779)*x[2]
        arg[2,1,0]=(-0.377750829061)*x[0]+(-0.101467967263)*x[1]+(-1.54241387349)*x[2]
        arg[2,1,1]=(0.62210350231)*x[0]+(0.0435334873911)*x[1]+(-0.299303646623)*x[2]
        arg[3,0,0]=(0.523811147108)*x[0]+(-1.09709839108)*x[1]+(1.83592280935)*x[2]
        arg[3,0,1]=(0.957738143475)*x[0]+(0.282158440867)*x[1]+(0.621694180522)*x[2]
        arg[3,1,0]=(0.366557891822)*x[0]+(0.812194047631)*x[1]+(-0.245067876349)*x[2]
        arg[3,1,1]=(0.604859464982)*x[0]+(1.08415514854)*x[1]+(-1.470128848)*x[2]
        arg[4,0,0]=(-0.13031502499)*x[0]+(-1.20707238733)*x[1]+(0.108382497552)*x[2]
        arg[4,0,1]=(1.30453246317)*x[0]+(1.59696612588)*x[1]+(-0.234259043697)*x[2]
        arg[4,1,0]=(-0.233494640836)*x[0]+(1.10701061816)*x[1]+(-0.409927013674)*x[2]
        arg[4,1,1]=(0.102708240266)*x[0]+(-0.381598345002)*x[1]+(-1.39077748656)*x[2]
        arg[5,0,0]=(1.06217404493)*x[0]+(1.1600791444)*x[1]+(-1.45242735389)*x[2]
        arg[5,0,1]=(-0.423024189506)*x[0]+(-0.0868948651641)*x[1]+(0.0894539043522)*x[2]
        arg[5,1,0]=(1.17650613067)*x[0]+(0.993832919965)*x[1]+(1.16546768569)*x[2]
        arg[5,1,1]=(-0.61549132897)*x[0]+(-0.499619662734)*x[1]+(-0.643422281669)*x[2]
        ref[0,0,0]=(0.731194732468)*x_ref[0]+(-0.0822115660022)*x_ref[1]+(-0.403175091289)*x_ref[2]
        ref[0,0,1]=(0.39292230307)*x_ref[0]+(1.48598681615)*x_ref[1]+(1.05545374107)*x_ref[2]
        ref[0,1,0]=(0.306844260706)*x_ref[0]+(0.836496026763)*x_ref[1]+(-0.121120983155)*x_ref[2]
        ref[0,1,1]=(-0.155245802062)*x_ref[0]+(0.800232934864)*x_ref[1]+(0.373397160636)*x_ref[2]
        ref[1,0,0]=(0.155941790359)*x_ref[0]+(0.826821969281)*x_ref[1]+(0.849596807425)*x_ref[2]
        ref[1,0,1]=(-0.0988854656187)*x_ref[0]+(-0.855032499842)*x_ref[1]+(0.141988288566)*x_ref[2]
        ref[1,1,0]=(-0.761032539296)*x_ref[0]+(-0.0892979073234)*x_ref[1]+(1.93034385132)*x_ref[2]
        ref[1,1,1]=(-1.08303603558)*x_ref[0]+(-0.487250117659)*x_ref[1]+(0.0358501036173)*x_ref[2]
        ref[2,0,0]=(0.148391465413)*x_ref[0]+(-0.731963025625)*x_ref[1]+(0.484236965689)*x_ref[2]
        ref[2,0,1]=(1.06691217722)*x_ref[0]+(-0.638095266244)*x_ref[1]+(-0.971838410779)*x_ref[2]
        ref[2,1,0]=(-0.377750829061)*x_ref[0]+(-0.101467967263)*x_ref[1]+(-1.54241387349)*x_ref[2]
        ref[2,1,1]=(0.62210350231)*x_ref[0]+(0.0435334873911)*x_ref[1]+(-0.299303646623)*x_ref[2]
        ref[3,0,0]=(0.523811147108)*x_ref[0]+(-1.09709839108)*x_ref[1]+(1.83592280935)*x_ref[2]
        ref[3,0,1]=(0.957738143475)*x_ref[0]+(0.282158440867)*x_ref[1]+(0.621694180522)*x_ref[2]
        ref[3,1,0]=(0.366557891822)*x_ref[0]+(0.812194047631)*x_ref[1]+(-0.245067876349)*x_ref[2]
        ref[3,1,1]=(0.604859464982)*x_ref[0]+(1.08415514854)*x_ref[1]+(-1.470128848)*x_ref[2]
        ref[4,0,0]=(-0.13031502499)*x_ref[0]+(-1.20707238733)*x_ref[1]+(0.108382497552)*x_ref[2]
        ref[4,0,1]=(1.30453246317)*x_ref[0]+(1.59696612588)*x_ref[1]+(-0.234259043697)*x_ref[2]
        ref[4,1,0]=(-0.233494640836)*x_ref[0]+(1.10701061816)*x_ref[1]+(-0.409927013674)*x_ref[2]
        ref[4,1,1]=(0.102708240266)*x_ref[0]+(-0.381598345002)*x_ref[1]+(-1.39077748656)*x_ref[2]
        ref[5,0,0]=(1.06217404493)*x_ref[0]+(1.1600791444)*x_ref[1]+(-1.45242735389)*x_ref[2]
        ref[5,0,1]=(-0.423024189506)*x_ref[0]+(-0.0868948651641)*x_ref[1]+(0.0894539043522)*x_ref[2]
        ref[5,1,0]=(1.17650613067)*x_ref[0]+(0.993832919965)*x_ref[1]+(1.16546768569)*x_ref[2]
        ref[5,1,1]=(-0.61549132897)*x_ref[0]+(-0.499619662734)*x_ref[1]+(-0.643422281669)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_ReducedSolution_rank4(self):
      """
      tests interpolate for rank 4 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedSolution onto ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5, 3, 2),w)
      ref=Data(0,(4, 5, 3, 2),w_ref)
      if dim==2:
        arg[0,0,0,0]=(0.227969627252)*x[0]+(0.778177350248)*x[1]
        arg[0,0,0,1]=(-0.684883711494)*x[0]+(-0.574285957132)*x[1]
        arg[0,0,1,0]=(0.483090608974)*x[0]+(0.0779929087959)*x[1]
        arg[0,0,1,1]=(-0.293150814932)*x[0]+(1.35368931993)*x[1]
        arg[0,0,2,0]=(-0.190664734662)*x[0]+(0.910883386754)*x[1]
        arg[0,0,2,1]=(-1.51255575457)*x[0]+(0.421560181501)*x[1]
        arg[0,1,0,0]=(-0.833402828032)*x[0]+(-0.400466806781)*x[1]
        arg[0,1,0,1]=(-0.212099323605)*x[0]+(-0.112478697221)*x[1]
        arg[0,1,1,0]=(0.57687662926)*x[0]+(0.869758954551)*x[1]
        arg[0,1,1,1]=(1.37261769917)*x[0]+(1.37855789907)*x[1]
        arg[0,1,2,0]=(-1.02838589722)*x[0]+(-0.280409246204)*x[1]
        arg[0,1,2,1]=(1.2333373468)*x[0]+(0.337631183798)*x[1]
        arg[0,2,0,0]=(-0.249065448772)*x[0]+(0.834483646317)*x[1]
        arg[0,2,0,1]=(-1.30860297683)*x[0]+(-0.653929967017)*x[1]
        arg[0,2,1,0]=(-0.0155994484453)*x[0]+(-0.274023166937)*x[1]
        arg[0,2,1,1]=(0.31458360335)*x[0]+(1.59507700673)*x[1]
        arg[0,2,2,0]=(-0.11442619553)*x[0]+(1.6577028576)*x[1]
        arg[0,2,2,1]=(0.274261132032)*x[0]+(1.15236962479)*x[1]
        arg[0,3,0,0]=(1.91433860239)*x[0]+(-0.316349747811)*x[1]
        arg[0,3,0,1]=(-1.42457591054)*x[0]+(-0.362812690805)*x[1]
        arg[0,3,1,0]=(0.572734883096)*x[0]+(0.424724787283)*x[1]
        arg[0,3,1,1]=(-0.118538376813)*x[0]+(0.25190312527)*x[1]
        arg[0,3,2,0]=(-0.533409510081)*x[0]+(-0.804349107915)*x[1]
        arg[0,3,2,1]=(-0.290555363666)*x[0]+(-0.912641477145)*x[1]
        arg[0,4,0,0]=(-0.469612775918)*x[0]+(-0.0447411310793)*x[1]
        arg[0,4,0,1]=(0.108538258831)*x[0]+(0.0453962016662)*x[1]
        arg[0,4,1,0]=(-0.130785944028)*x[0]+(0.0887280914032)*x[1]
        arg[0,4,1,1]=(0.166412309585)*x[0]+(-0.75130797706)*x[1]
        arg[0,4,2,0]=(1.22538232455)*x[0]+(0.112925588343)*x[1]
        arg[0,4,2,1]=(-1.02455171854)*x[0]+(-0.725754942749)*x[1]
        arg[1,0,0,0]=(-0.869171086952)*x[0]+(-0.948876783682)*x[1]
        arg[1,0,0,1]=(0.17730673689)*x[0]+(-0.00166180855579)*x[1]
        arg[1,0,1,0]=(0.482072704433)*x[0]+(0.895847096783)*x[1]
        arg[1,0,1,1]=(0.695736834617)*x[0]+(0.358082704558)*x[1]
        arg[1,0,2,0]=(-0.501799636565)*x[0]+(0.363612558025)*x[1]
        arg[1,0,2,1]=(-1.47204447226)*x[0]+(1.91281426121)*x[1]
        arg[1,1,0,0]=(0.241893182978)*x[0]+(0.95199758669)*x[1]
        arg[1,1,0,1]=(-0.68112361827)*x[0]+(-0.241935141301)*x[1]
        arg[1,1,1,0]=(-0.553030972306)*x[0]+(0.0244503690439)*x[1]
        arg[1,1,1,1]=(0.278177176896)*x[0]+(1.58693846427)*x[1]
        arg[1,1,2,0]=(-0.87980015517)*x[0]+(0.315618837123)*x[1]
        arg[1,1,2,1]=(-0.335783961379)*x[0]+(0.368650136138)*x[1]
        arg[1,2,0,0]=(-0.903535146802)*x[0]+(1.68182923194)*x[1]
        arg[1,2,0,1]=(1.06511511382)*x[0]+(-1.08524762302)*x[1]
        arg[1,2,1,0]=(-0.0455274784681)*x[0]+(-0.519576710661)*x[1]
        arg[1,2,1,1]=(0.735100217062)*x[0]+(-0.986268682671)*x[1]
        arg[1,2,2,0]=(-0.544610797462)*x[0]+(0.447237114882)*x[1]
        arg[1,2,2,1]=(0.385242031153)*x[0]+(-0.960538844247)*x[1]
        arg[1,3,0,0]=(-0.354927072897)*x[0]+(0.842574324062)*x[1]
        arg[1,3,0,1]=(0.319036427919)*x[0]+(-0.156282262536)*x[1]
        arg[1,3,1,0]=(0.587123807717)*x[0]+(1.65277990726)*x[1]
        arg[1,3,1,1]=(0.791697370298)*x[0]+(-1.5681680344)*x[1]
        arg[1,3,2,0]=(-1.41931242045)*x[0]+(0.351285136557)*x[1]
        arg[1,3,2,1]=(0.53175704402)*x[0]+(0.987523241324)*x[1]
        arg[1,4,0,0]=(0.00533504818465)*x[0]+(-0.158886930817)*x[1]
        arg[1,4,0,1]=(1.23725297303)*x[0]+(-0.302896452172)*x[1]
        arg[1,4,1,0]=(0.433953146329)*x[0]+(-0.0721602234887)*x[1]
        arg[1,4,1,1]=(-0.383499726865)*x[0]+(-0.253834671235)*x[1]
        arg[1,4,2,0]=(-0.480776695247)*x[0]+(-0.302145524759)*x[1]
        arg[1,4,2,1]=(1.14113180903)*x[0]+(-1.24304726003)*x[1]
        arg[2,0,0,0]=(0.513776013893)*x[0]+(-0.16660259885)*x[1]
        arg[2,0,0,1]=(1.55739209698)*x[0]+(0.0524084654531)*x[1]
        arg[2,0,1,0]=(0.222567881446)*x[0]+(-0.357158470388)*x[1]
        arg[2,0,1,1]=(0.0179816596229)*x[0]+(0.231534460784)*x[1]
        arg[2,0,2,0]=(1.58460345297)*x[0]+(-0.383588433888)*x[1]
        arg[2,0,2,1]=(-1.73386465536)*x[0]+(-0.743326657544)*x[1]
        arg[2,1,0,0]=(-0.737976950246)*x[0]+(-1.01053396224)*x[1]
        arg[2,1,0,1]=(0.926911190393)*x[0]+(0.620251783925)*x[1]
        arg[2,1,1,0]=(-1.34289630129)*x[0]+(-0.429933681523)*x[1]
        arg[2,1,1,1]=(1.79530806415)*x[0]+(-0.539783957021)*x[1]
        arg[2,1,2,0]=(0.237950974938)*x[0]+(1.93783427703)*x[1]
        arg[2,1,2,1]=(-0.439640741973)*x[0]+(-0.61543984673)*x[1]
        arg[2,2,0,0]=(-1.66403612438)*x[0]+(1.00839566441)*x[1]
        arg[2,2,0,1]=(0.249635983351)*x[0]+(-0.722261363675)*x[1]
        arg[2,2,1,0]=(1.34179674459)*x[0]+(0.502542144188)*x[1]
        arg[2,2,1,1]=(-0.815862586074)*x[0]+(-0.110688273707)*x[1]
        arg[2,2,2,0]=(-0.686810719526)*x[0]+(0.500406563421)*x[1]
        arg[2,2,2,1]=(0.681231089598)*x[0]+(0.605219505613)*x[1]
        arg[2,3,0,0]=(-0.953299029956)*x[0]+(-0.223725858372)*x[1]
        arg[2,3,0,1]=(0.831380756684)*x[0]+(-0.0379777076335)*x[1]
        arg[2,3,1,0]=(-1.63804393157)*x[0]+(0.148235731301)*x[1]
        arg[2,3,1,1]=(-0.17786458643)*x[0]+(-1.77342152699)*x[1]
        arg[2,3,2,0]=(0.0210446244192)*x[0]+(1.59778781524)*x[1]
        arg[2,3,2,1]=(0.877162721214)*x[0]+(0.262990448614)*x[1]
        arg[2,4,0,0]=(-0.650265970331)*x[0]+(-1.07003604369)*x[1]
        arg[2,4,0,1]=(1.12940101029)*x[0]+(0.860589290891)*x[1]
        arg[2,4,1,0]=(0.520056210056)*x[0]+(-1.47691193979)*x[1]
        arg[2,4,1,1]=(1.26823535235)*x[0]+(0.496811720647)*x[1]
        arg[2,4,2,0]=(-1.388913443)*x[0]+(0.585343197943)*x[1]
        arg[2,4,2,1]=(-0.762904547594)*x[0]+(0.503438815917)*x[1]
        arg[3,0,0,0]=(0.382862686993)*x[0]+(-0.923525513303)*x[1]
        arg[3,0,0,1]=(-0.54305794485)*x[0]+(0.467513531444)*x[1]
        arg[3,0,1,0]=(1.24228759776)*x[0]+(-1.65740117421)*x[1]
        arg[3,0,1,1]=(-0.463045362742)*x[0]+(0.186438587087)*x[1]
        arg[3,0,2,0]=(0.296881980828)*x[0]+(-0.385878814949)*x[1]
        arg[3,0,2,1]=(-0.299705899564)*x[0]+(0.490736038538)*x[1]
        arg[3,1,0,0]=(-0.786851428658)*x[0]+(-0.760593794308)*x[1]
        arg[3,1,0,1]=(0.506851336483)*x[0]+(0.995163951197)*x[1]
        arg[3,1,1,0]=(-0.678883937341)*x[0]+(0.56996778248)*x[1]
        arg[3,1,1,1]=(-0.401483642528)*x[0]+(0.0476797332537)*x[1]
        arg[3,1,2,0]=(-0.585357955375)*x[0]+(0.330103647539)*x[1]
        arg[3,1,2,1]=(-0.355539976825)*x[0]+(0.416287924349)*x[1]
        arg[3,2,0,0]=(-0.662837130286)*x[0]+(-0.26640899674)*x[1]
        arg[3,2,0,1]=(0.397553900588)*x[0]+(-1.21708620294)*x[1]
        arg[3,2,1,0]=(0.464963326429)*x[0]+(-0.774515114142)*x[1]
        arg[3,2,1,1]=(0.834351469791)*x[0]+(0.487629540715)*x[1]
        arg[3,2,2,0]=(0.27622693716)*x[0]+(0.811338337719)*x[1]
        arg[3,2,2,1]=(0.0384997772653)*x[0]+(1.03724996578)*x[1]
        arg[3,3,0,0]=(-0.805491987874)*x[0]+(-0.476384547853)*x[1]
        arg[3,3,0,1]=(0.615551780195)*x[0]+(-0.312805967691)*x[1]
        arg[3,3,1,0]=(0.348666798955)*x[0]+(-1.31271156524)*x[1]
        arg[3,3,1,1]=(-0.423569975374)*x[0]+(0.930713125389)*x[1]
        arg[3,3,2,0]=(0.325632686234)*x[0]+(-0.166401033259)*x[1]
        arg[3,3,2,1]=(0.126924861797)*x[0]+(-0.425380897969)*x[1]
        arg[3,4,0,0]=(0.169532204538)*x[0]+(0.614632525765)*x[1]
        arg[3,4,0,1]=(-0.938098696851)*x[0]+(0.648854147587)*x[1]
        arg[3,4,1,0]=(-0.545143970434)*x[0]+(-1.59787515966)*x[1]
        arg[3,4,1,1]=(-0.711190914978)*x[0]+(0.897331522911)*x[1]
        arg[3,4,2,0]=(-0.386498124525)*x[0]+(0.299740420711)*x[1]
        arg[3,4,2,1]=(0.617683032062)*x[0]+(-0.449587703061)*x[1]
        ref[0,0,0,0]=(0.227969627252)*x_ref[0]+(0.778177350248)*x_ref[1]
        ref[0,0,0,1]=(-0.684883711494)*x_ref[0]+(-0.574285957132)*x_ref[1]
        ref[0,0,1,0]=(0.483090608974)*x_ref[0]+(0.0779929087959)*x_ref[1]
        ref[0,0,1,1]=(-0.293150814932)*x_ref[0]+(1.35368931993)*x_ref[1]
        ref[0,0,2,0]=(-0.190664734662)*x_ref[0]+(0.910883386754)*x_ref[1]
        ref[0,0,2,1]=(-1.51255575457)*x_ref[0]+(0.421560181501)*x_ref[1]
        ref[0,1,0,0]=(-0.833402828032)*x_ref[0]+(-0.400466806781)*x_ref[1]
        ref[0,1,0,1]=(-0.212099323605)*x_ref[0]+(-0.112478697221)*x_ref[1]
        ref[0,1,1,0]=(0.57687662926)*x_ref[0]+(0.869758954551)*x_ref[1]
        ref[0,1,1,1]=(1.37261769917)*x_ref[0]+(1.37855789907)*x_ref[1]
        ref[0,1,2,0]=(-1.02838589722)*x_ref[0]+(-0.280409246204)*x_ref[1]
        ref[0,1,2,1]=(1.2333373468)*x_ref[0]+(0.337631183798)*x_ref[1]
        ref[0,2,0,0]=(-0.249065448772)*x_ref[0]+(0.834483646317)*x_ref[1]
        ref[0,2,0,1]=(-1.30860297683)*x_ref[0]+(-0.653929967017)*x_ref[1]
        ref[0,2,1,0]=(-0.0155994484453)*x_ref[0]+(-0.274023166937)*x_ref[1]
        ref[0,2,1,1]=(0.31458360335)*x_ref[0]+(1.59507700673)*x_ref[1]
        ref[0,2,2,0]=(-0.11442619553)*x_ref[0]+(1.6577028576)*x_ref[1]
        ref[0,2,2,1]=(0.274261132032)*x_ref[0]+(1.15236962479)*x_ref[1]
        ref[0,3,0,0]=(1.91433860239)*x_ref[0]+(-0.316349747811)*x_ref[1]
        ref[0,3,0,1]=(-1.42457591054)*x_ref[0]+(-0.362812690805)*x_ref[1]
        ref[0,3,1,0]=(0.572734883096)*x_ref[0]+(0.424724787283)*x_ref[1]
        ref[0,3,1,1]=(-0.118538376813)*x_ref[0]+(0.25190312527)*x_ref[1]
        ref[0,3,2,0]=(-0.533409510081)*x_ref[0]+(-0.804349107915)*x_ref[1]
        ref[0,3,2,1]=(-0.290555363666)*x_ref[0]+(-0.912641477145)*x_ref[1]
        ref[0,4,0,0]=(-0.469612775918)*x_ref[0]+(-0.0447411310793)*x_ref[1]
        ref[0,4,0,1]=(0.108538258831)*x_ref[0]+(0.0453962016662)*x_ref[1]
        ref[0,4,1,0]=(-0.130785944028)*x_ref[0]+(0.0887280914032)*x_ref[1]
        ref[0,4,1,1]=(0.166412309585)*x_ref[0]+(-0.75130797706)*x_ref[1]
        ref[0,4,2,0]=(1.22538232455)*x_ref[0]+(0.112925588343)*x_ref[1]
        ref[0,4,2,1]=(-1.02455171854)*x_ref[0]+(-0.725754942749)*x_ref[1]
        ref[1,0,0,0]=(-0.869171086952)*x_ref[0]+(-0.948876783682)*x_ref[1]
        ref[1,0,0,1]=(0.17730673689)*x_ref[0]+(-0.00166180855579)*x_ref[1]
        ref[1,0,1,0]=(0.482072704433)*x_ref[0]+(0.895847096783)*x_ref[1]
        ref[1,0,1,1]=(0.695736834617)*x_ref[0]+(0.358082704558)*x_ref[1]
        ref[1,0,2,0]=(-0.501799636565)*x_ref[0]+(0.363612558025)*x_ref[1]
        ref[1,0,2,1]=(-1.47204447226)*x_ref[0]+(1.91281426121)*x_ref[1]
        ref[1,1,0,0]=(0.241893182978)*x_ref[0]+(0.95199758669)*x_ref[1]
        ref[1,1,0,1]=(-0.68112361827)*x_ref[0]+(-0.241935141301)*x_ref[1]
        ref[1,1,1,0]=(-0.553030972306)*x_ref[0]+(0.0244503690439)*x_ref[1]
        ref[1,1,1,1]=(0.278177176896)*x_ref[0]+(1.58693846427)*x_ref[1]
        ref[1,1,2,0]=(-0.87980015517)*x_ref[0]+(0.315618837123)*x_ref[1]
        ref[1,1,2,1]=(-0.335783961379)*x_ref[0]+(0.368650136138)*x_ref[1]
        ref[1,2,0,0]=(-0.903535146802)*x_ref[0]+(1.68182923194)*x_ref[1]
        ref[1,2,0,1]=(1.06511511382)*x_ref[0]+(-1.08524762302)*x_ref[1]
        ref[1,2,1,0]=(-0.0455274784681)*x_ref[0]+(-0.519576710661)*x_ref[1]
        ref[1,2,1,1]=(0.735100217062)*x_ref[0]+(-0.986268682671)*x_ref[1]
        ref[1,2,2,0]=(-0.544610797462)*x_ref[0]+(0.447237114882)*x_ref[1]
        ref[1,2,2,1]=(0.385242031153)*x_ref[0]+(-0.960538844247)*x_ref[1]
        ref[1,3,0,0]=(-0.354927072897)*x_ref[0]+(0.842574324062)*x_ref[1]
        ref[1,3,0,1]=(0.319036427919)*x_ref[0]+(-0.156282262536)*x_ref[1]
        ref[1,3,1,0]=(0.587123807717)*x_ref[0]+(1.65277990726)*x_ref[1]
        ref[1,3,1,1]=(0.791697370298)*x_ref[0]+(-1.5681680344)*x_ref[1]
        ref[1,3,2,0]=(-1.41931242045)*x_ref[0]+(0.351285136557)*x_ref[1]
        ref[1,3,2,1]=(0.53175704402)*x_ref[0]+(0.987523241324)*x_ref[1]
        ref[1,4,0,0]=(0.00533504818465)*x_ref[0]+(-0.158886930817)*x_ref[1]
        ref[1,4,0,1]=(1.23725297303)*x_ref[0]+(-0.302896452172)*x_ref[1]
        ref[1,4,1,0]=(0.433953146329)*x_ref[0]+(-0.0721602234887)*x_ref[1]
        ref[1,4,1,1]=(-0.383499726865)*x_ref[0]+(-0.253834671235)*x_ref[1]
        ref[1,4,2,0]=(-0.480776695247)*x_ref[0]+(-0.302145524759)*x_ref[1]
        ref[1,4,2,1]=(1.14113180903)*x_ref[0]+(-1.24304726003)*x_ref[1]
        ref[2,0,0,0]=(0.513776013893)*x_ref[0]+(-0.16660259885)*x_ref[1]
        ref[2,0,0,1]=(1.55739209698)*x_ref[0]+(0.0524084654531)*x_ref[1]
        ref[2,0,1,0]=(0.222567881446)*x_ref[0]+(-0.357158470388)*x_ref[1]
        ref[2,0,1,1]=(0.0179816596229)*x_ref[0]+(0.231534460784)*x_ref[1]
        ref[2,0,2,0]=(1.58460345297)*x_ref[0]+(-0.383588433888)*x_ref[1]
        ref[2,0,2,1]=(-1.73386465536)*x_ref[0]+(-0.743326657544)*x_ref[1]
        ref[2,1,0,0]=(-0.737976950246)*x_ref[0]+(-1.01053396224)*x_ref[1]
        ref[2,1,0,1]=(0.926911190393)*x_ref[0]+(0.620251783925)*x_ref[1]
        ref[2,1,1,0]=(-1.34289630129)*x_ref[0]+(-0.429933681523)*x_ref[1]
        ref[2,1,1,1]=(1.79530806415)*x_ref[0]+(-0.539783957021)*x_ref[1]
        ref[2,1,2,0]=(0.237950974938)*x_ref[0]+(1.93783427703)*x_ref[1]
        ref[2,1,2,1]=(-0.439640741973)*x_ref[0]+(-0.61543984673)*x_ref[1]
        ref[2,2,0,0]=(-1.66403612438)*x_ref[0]+(1.00839566441)*x_ref[1]
        ref[2,2,0,1]=(0.249635983351)*x_ref[0]+(-0.722261363675)*x_ref[1]
        ref[2,2,1,0]=(1.34179674459)*x_ref[0]+(0.502542144188)*x_ref[1]
        ref[2,2,1,1]=(-0.815862586074)*x_ref[0]+(-0.110688273707)*x_ref[1]
        ref[2,2,2,0]=(-0.686810719526)*x_ref[0]+(0.500406563421)*x_ref[1]
        ref[2,2,2,1]=(0.681231089598)*x_ref[0]+(0.605219505613)*x_ref[1]
        ref[2,3,0,0]=(-0.953299029956)*x_ref[0]+(-0.223725858372)*x_ref[1]
        ref[2,3,0,1]=(0.831380756684)*x_ref[0]+(-0.0379777076335)*x_ref[1]
        ref[2,3,1,0]=(-1.63804393157)*x_ref[0]+(0.148235731301)*x_ref[1]
        ref[2,3,1,1]=(-0.17786458643)*x_ref[0]+(-1.77342152699)*x_ref[1]
        ref[2,3,2,0]=(0.0210446244192)*x_ref[0]+(1.59778781524)*x_ref[1]
        ref[2,3,2,1]=(0.877162721214)*x_ref[0]+(0.262990448614)*x_ref[1]
        ref[2,4,0,0]=(-0.650265970331)*x_ref[0]+(-1.07003604369)*x_ref[1]
        ref[2,4,0,1]=(1.12940101029)*x_ref[0]+(0.860589290891)*x_ref[1]
        ref[2,4,1,0]=(0.520056210056)*x_ref[0]+(-1.47691193979)*x_ref[1]
        ref[2,4,1,1]=(1.26823535235)*x_ref[0]+(0.496811720647)*x_ref[1]
        ref[2,4,2,0]=(-1.388913443)*x_ref[0]+(0.585343197943)*x_ref[1]
        ref[2,4,2,1]=(-0.762904547594)*x_ref[0]+(0.503438815917)*x_ref[1]
        ref[3,0,0,0]=(0.382862686993)*x_ref[0]+(-0.923525513303)*x_ref[1]
        ref[3,0,0,1]=(-0.54305794485)*x_ref[0]+(0.467513531444)*x_ref[1]
        ref[3,0,1,0]=(1.24228759776)*x_ref[0]+(-1.65740117421)*x_ref[1]
        ref[3,0,1,1]=(-0.463045362742)*x_ref[0]+(0.186438587087)*x_ref[1]
        ref[3,0,2,0]=(0.296881980828)*x_ref[0]+(-0.385878814949)*x_ref[1]
        ref[3,0,2,1]=(-0.299705899564)*x_ref[0]+(0.490736038538)*x_ref[1]
        ref[3,1,0,0]=(-0.786851428658)*x_ref[0]+(-0.760593794308)*x_ref[1]
        ref[3,1,0,1]=(0.506851336483)*x_ref[0]+(0.995163951197)*x_ref[1]
        ref[3,1,1,0]=(-0.678883937341)*x_ref[0]+(0.56996778248)*x_ref[1]
        ref[3,1,1,1]=(-0.401483642528)*x_ref[0]+(0.0476797332537)*x_ref[1]
        ref[3,1,2,0]=(-0.585357955375)*x_ref[0]+(0.330103647539)*x_ref[1]
        ref[3,1,2,1]=(-0.355539976825)*x_ref[0]+(0.416287924349)*x_ref[1]
        ref[3,2,0,0]=(-0.662837130286)*x_ref[0]+(-0.26640899674)*x_ref[1]
        ref[3,2,0,1]=(0.397553900588)*x_ref[0]+(-1.21708620294)*x_ref[1]
        ref[3,2,1,0]=(0.464963326429)*x_ref[0]+(-0.774515114142)*x_ref[1]
        ref[3,2,1,1]=(0.834351469791)*x_ref[0]+(0.487629540715)*x_ref[1]
        ref[3,2,2,0]=(0.27622693716)*x_ref[0]+(0.811338337719)*x_ref[1]
        ref[3,2,2,1]=(0.0384997772653)*x_ref[0]+(1.03724996578)*x_ref[1]
        ref[3,3,0,0]=(-0.805491987874)*x_ref[0]+(-0.476384547853)*x_ref[1]
        ref[3,3,0,1]=(0.615551780195)*x_ref[0]+(-0.312805967691)*x_ref[1]
        ref[3,3,1,0]=(0.348666798955)*x_ref[0]+(-1.31271156524)*x_ref[1]
        ref[3,3,1,1]=(-0.423569975374)*x_ref[0]+(0.930713125389)*x_ref[1]
        ref[3,3,2,0]=(0.325632686234)*x_ref[0]+(-0.166401033259)*x_ref[1]
        ref[3,3,2,1]=(0.126924861797)*x_ref[0]+(-0.425380897969)*x_ref[1]
        ref[3,4,0,0]=(0.169532204538)*x_ref[0]+(0.614632525765)*x_ref[1]
        ref[3,4,0,1]=(-0.938098696851)*x_ref[0]+(0.648854147587)*x_ref[1]
        ref[3,4,1,0]=(-0.545143970434)*x_ref[0]+(-1.59787515966)*x_ref[1]
        ref[3,4,1,1]=(-0.711190914978)*x_ref[0]+(0.897331522911)*x_ref[1]
        ref[3,4,2,0]=(-0.386498124525)*x_ref[0]+(0.299740420711)*x_ref[1]
        ref[3,4,2,1]=(0.617683032062)*x_ref[0]+(-0.449587703061)*x_ref[1]
      else:
        arg[0,0,0,0]=(-0.791620170192)*x[0]+(1.03871460301)*x[1]+(-0.225035657687)*x[2]
        arg[0,0,0,1]=(-0.464422319203)*x[0]+(0.769454931053)*x[1]+(-0.783260637771)*x[2]
        arg[0,0,1,0]=(0.72655538215)*x[0]+(-1.65736312015)*x[1]+(1.08402505927)*x[2]
        arg[0,0,1,1]=(-0.207576709573)*x[0]+(-0.55250594761)*x[1]+(-0.923702938288)*x[2]
        arg[0,0,2,0]=(-0.701991085516)*x[0]+(-1.76863643107)*x[1]+(0.831382933104)*x[2]
        arg[0,0,2,1]=(0.0640035527319)*x[0]+(0.425446342976)*x[1]+(1.64626766696)*x[2]
        arg[0,1,0,0]=(1.01022963801)*x[0]+(-0.0215015059716)*x[1]+(0.849862425592)*x[2]
        arg[0,1,0,1]=(-0.45776932943)*x[0]+(0.7621342151)*x[1]+(-1.40123018391)*x[2]
        arg[0,1,1,0]=(0.737485054127)*x[0]+(-0.147513905858)*x[1]+(-0.616928595213)*x[2]
        arg[0,1,1,1]=(-0.0608005391894)*x[0]+(-0.154978210573)*x[1]+(-1.40322011577)*x[2]
        arg[0,1,2,0]=(1.59908177714)*x[0]+(-0.378738573932)*x[1]+(1.45285315247)*x[2]
        arg[0,1,2,1]=(-0.622552157046)*x[0]+(1.19996417516)*x[1]+(0.728842900843)*x[2]
        arg[0,2,0,0]=(-0.46942034691)*x[0]+(0.333309721372)*x[1]+(-0.280846553832)*x[2]
        arg[0,2,0,1]=(1.22208788109)*x[0]+(0.369339594285)*x[1]+(1.31078425589)*x[2]
        arg[0,2,1,0]=(0.0794165004114)*x[0]+(0.44665451846)*x[1]+(-0.814625708916)*x[2]
        arg[0,2,1,1]=(0.594200807165)*x[0]+(-0.00616720333193)*x[1]+(0.382706175172)*x[2]
        arg[0,2,2,0]=(0.0372230170459)*x[0]+(0.440339923291)*x[1]+(0.137641070028)*x[2]
        arg[0,2,2,1]=(-1.29863078274)*x[0]+(0.283142455171)*x[1]+(0.0844540074659)*x[2]
        arg[0,3,0,0]=(1.72811808036)*x[0]+(0.536456781522)*x[1]+(0.0266737704706)*x[2]
        arg[0,3,0,1]=(0.148219555101)*x[0]+(-0.6887500275)*x[1]+(-0.405613136632)*x[2]
        arg[0,3,1,0]=(0.708262537279)*x[0]+(1.16319635773)*x[1]+(-0.030808721674)*x[2]
        arg[0,3,1,1]=(0.80834615863)*x[0]+(-0.176789000925)*x[1]+(-1.72576120963)*x[2]
        arg[0,3,2,0]=(0.21128315299)*x[0]+(0.770037046126)*x[1]+(0.988028934422)*x[2]
        arg[0,3,2,1]=(1.10522136317)*x[0]+(1.6319351743)*x[1]+(-0.784693713387)*x[2]
        arg[0,4,0,0]=(0.121088121064)*x[0]+(-1.79718067791)*x[1]+(-0.907857173318)*x[2]
        arg[0,4,0,1]=(-0.695887294217)*x[0]+(-1.24166787807)*x[1]+(-1.24968023856)*x[2]
        arg[0,4,1,0]=(-0.202510797291)*x[0]+(0.628310156105)*x[1]+(0.605146447822)*x[2]
        arg[0,4,1,1]=(0.149227458424)*x[0]+(-1.5650392775)*x[1]+(-0.763944892663)*x[2]
        arg[0,4,2,0]=(-0.393218625872)*x[0]+(0.479145914948)*x[1]+(-0.313960447492)*x[2]
        arg[0,4,2,1]=(-0.0191807443799)*x[0]+(0.592992659119)*x[1]+(-0.475633115484)*x[2]
        arg[1,0,0,0]=(-0.964829354261)*x[0]+(0.453014802906)*x[1]+(-0.779502566938)*x[2]
        arg[1,0,0,1]=(0.0151205739508)*x[0]+(-0.153706429594)*x[1]+(0.00866460475724)*x[2]
        arg[1,0,1,0]=(0.92592276366)*x[0]+(0.530787400525)*x[1]+(-0.0577912606876)*x[2]
        arg[1,0,1,1]=(0.948404467077)*x[0]+(-0.265905470751)*x[1]+(0.472349192005)*x[2]
        arg[1,0,2,0]=(0.0176615383702)*x[0]+(0.679605821761)*x[1]+(-0.0333590911941)*x[2]
        arg[1,0,2,1]=(-1.15594952629)*x[0]+(0.264868376508)*x[1]+(-0.883690535499)*x[2]
        arg[1,1,0,0]=(-0.738712703994)*x[0]+(0.545839232808)*x[1]+(0.592474501134)*x[2]
        arg[1,1,0,1]=(-1.3468960453)*x[0]+(-0.00848202443614)*x[1]+(-0.0124138286126)*x[2]
        arg[1,1,1,0]=(0.900228409707)*x[0]+(-1.86848082357)*x[1]+(0.421539431142)*x[2]
        arg[1,1,1,1]=(1.42754013963)*x[0]+(-0.861376440449)*x[1]+(0.0965520362291)*x[2]
        arg[1,1,2,0]=(-0.35873674515)*x[0]+(0.0862592617941)*x[1]+(-0.536106207025)*x[2]
        arg[1,1,2,1]=(0.90712910097)*x[0]+(0.974545253055)*x[1]+(0.570708510858)*x[2]
        arg[1,2,0,0]=(1.19310775669)*x[0]+(0.309414331679)*x[1]+(-0.271298061586)*x[2]
        arg[1,2,0,1]=(0.551778129545)*x[0]+(0.539028430134)*x[1]+(0.239406239262)*x[2]
        arg[1,2,1,0]=(-0.810625054344)*x[0]+(-0.241072190233)*x[1]+(-1.58590369097)*x[2]
        arg[1,2,1,1]=(0.529669075734)*x[0]+(-0.890365024163)*x[1]+(-1.69798509084)*x[2]
        arg[1,2,2,0]=(0.0437087707448)*x[0]+(0.88372190198)*x[1]+(1.86547747806)*x[2]
        arg[1,2,2,1]=(1.30739438898)*x[0]+(0.315850112101)*x[1]+(-0.262001133683)*x[2]
        arg[1,3,0,0]=(-0.143426236837)*x[0]+(-1.1703384826)*x[1]+(0.50608348646)*x[2]
        arg[1,3,0,1]=(-0.0444408776922)*x[0]+(1.26952782036)*x[1]+(1.67192602934)*x[2]
        arg[1,3,1,0]=(0.672742824368)*x[0]+(-0.785246676537)*x[1]+(0.716130664056)*x[2]
        arg[1,3,1,1]=(-0.295182508335)*x[0]+(-1.53658860897)*x[1]+(-1.87133995422)*x[2]
        arg[1,3,2,0]=(-0.440859530081)*x[0]+(0.0730120015821)*x[1]+(-1.02465537353)*x[2]
        arg[1,3,2,1]=(0.838320612003)*x[0]+(1.81233688454)*x[1]+(-1.0633876685)*x[2]
        arg[1,4,0,0]=(0.52440453051)*x[0]+(-0.0182588999022)*x[1]+(1.05556310351)*x[2]
        arg[1,4,0,1]=(0.766612727495)*x[0]+(1.58987590746)*x[1]+(-0.703801484569)*x[2]
        arg[1,4,1,0]=(0.373439757535)*x[0]+(-0.0676963704305)*x[1]+(0.0948034094293)*x[2]
        arg[1,4,1,1]=(-0.542283543375)*x[0]+(-0.454408605761)*x[1]+(-0.398904635316)*x[2]
        arg[1,4,2,0]=(1.3374871143)*x[0]+(0.0067849394587)*x[1]+(-0.319452094882)*x[2]
        arg[1,4,2,1]=(0.38530755393)*x[0]+(1.05356462429)*x[1]+(-0.302179447719)*x[2]
        arg[2,0,0,0]=(-0.123109761583)*x[0]+(0.69223848143)*x[1]+(-1.36903487092)*x[2]
        arg[2,0,0,1]=(0.121957328459)*x[0]+(0.102229685387)*x[1]+(0.962499974737)*x[2]
        arg[2,0,1,0]=(0.478614188653)*x[0]+(-1.23058817023)*x[1]+(1.30950997478)*x[2]
        arg[2,0,1,1]=(-0.529033562461)*x[0]+(0.563030509591)*x[1]+(-0.564939226654)*x[2]
        arg[2,0,2,0]=(-0.377740282406)*x[0]+(1.20532110343)*x[1]+(1.55615174947)*x[2]
        arg[2,0,2,1]=(0.539119710703)*x[0]+(1.91771865857)*x[1]+(0.0809882488048)*x[2]
        arg[2,1,0,0]=(0.00548664737796)*x[0]+(-0.157716099068)*x[1]+(-1.40009044235)*x[2]
        arg[2,1,0,1]=(0.0667557890666)*x[0]+(0.742484448131)*x[1]+(-0.325246111346)*x[2]
        arg[2,1,1,0]=(-0.214965474071)*x[0]+(-0.215055151964)*x[1]+(1.15673960537)*x[2]
        arg[2,1,1,1]=(0.605764831789)*x[0]+(-1.0432964832)*x[1]+(0.645482523161)*x[2]
        arg[2,1,2,0]=(-1.13098364088)*x[0]+(-0.932044089041)*x[1]+(-0.308053828241)*x[2]
        arg[2,1,2,1]=(0.481961305459)*x[0]+(-0.101277472029)*x[1]+(-0.17520207974)*x[2]
        arg[2,2,0,0]=(0.579831852303)*x[0]+(0.538731990254)*x[1]+(-0.0897460374205)*x[2]
        arg[2,2,0,1]=(-1.8763181434)*x[0]+(-0.132710928569)*x[1]+(1.03169751651)*x[2]
        arg[2,2,1,0]=(0.56871511431)*x[0]+(0.075333481656)*x[1]+(1.10238818953)*x[2]
        arg[2,2,1,1]=(1.49347755246)*x[0]+(-1.00500941047)*x[1]+(-0.181733196359)*x[2]
        arg[2,2,2,0]=(0.532881509971)*x[0]+(-0.364333964659)*x[1]+(1.09633720883)*x[2]
        arg[2,2,2,1]=(-0.518419048615)*x[0]+(-0.48091100593)*x[1]+(0.724628840153)*x[2]
        arg[2,3,0,0]=(0.353475859331)*x[0]+(-0.0239879269825)*x[1]+(1.32068893718)*x[2]
        arg[2,3,0,1]=(0.553999122102)*x[0]+(-0.128530869239)*x[1]+(0.134663336287)*x[2]
        arg[2,3,1,0]=(-1.59775882569)*x[0]+(-1.07335424785)*x[1]+(1.39223587988)*x[2]
        arg[2,3,1,1]=(-0.921252080095)*x[0]+(1.46226299067)*x[1]+(-1.13412499932)*x[2]
        arg[2,3,2,0]=(0.417572910178)*x[0]+(0.0814889599489)*x[1]+(0.292159249719)*x[2]
        arg[2,3,2,1]=(1.44620228921)*x[0]+(0.767854577367)*x[1]+(-0.0113709806383)*x[2]
        arg[2,4,0,0]=(-0.156987001681)*x[0]+(-0.922524257833)*x[1]+(0.521128360838)*x[2]
        arg[2,4,0,1]=(0.154198133354)*x[0]+(0.462020370175)*x[1]+(0.966768653665)*x[2]
        arg[2,4,1,0]=(0.507836849242)*x[0]+(1.16701911244)*x[1]+(0.264543038211)*x[2]
        arg[2,4,1,1]=(0.452059436879)*x[0]+(0.409662432778)*x[1]+(-0.403800872595)*x[2]
        arg[2,4,2,0]=(0.521175521008)*x[0]+(-0.362551578816)*x[1]+(0.0922727331623)*x[2]
        arg[2,4,2,1]=(-1.0990361823)*x[0]+(1.73655146851)*x[1]+(-0.867917402922)*x[2]
        arg[3,0,0,0]=(-0.0527010882236)*x[0]+(-1.11528215488)*x[1]+(1.3141906393)*x[2]
        arg[3,0,0,1]=(0.151284271917)*x[0]+(-0.0159077450668)*x[1]+(1.1798338931)*x[2]
        arg[3,0,1,0]=(-1.7648963435)*x[0]+(0.229511437402)*x[1]+(0.924415900312)*x[2]
        arg[3,0,1,1]=(-0.498271821094)*x[0]+(1.58363063019)*x[1]+(1.64871750388)*x[2]
        arg[3,0,2,0]=(-1.11245170725)*x[0]+(1.27662612569)*x[1]+(0.145950372369)*x[2]
        arg[3,0,2,1]=(-0.691643924238)*x[0]+(-0.178211874279)*x[1]+(-0.341071050366)*x[2]
        arg[3,1,0,0]=(-0.929804889461)*x[0]+(0.199087067309)*x[1]+(1.00150168883)*x[2]
        arg[3,1,0,1]=(-1.25152265093)*x[0]+(0.517076294911)*x[1]+(0.103798312362)*x[2]
        arg[3,1,1,0]=(0.797316290623)*x[0]+(0.0239285379293)*x[1]+(-0.836519299418)*x[2]
        arg[3,1,1,1]=(-0.470084592176)*x[0]+(0.844364701728)*x[1]+(-0.027373275093)*x[2]
        arg[3,1,2,0]=(1.0340279373)*x[0]+(-0.0133951276315)*x[1]+(1.17710546194)*x[2]
        arg[3,1,2,1]=(0.0325929541496)*x[0]+(1.13784867184)*x[1]+(-0.0529194266769)*x[2]
        arg[3,2,0,0]=(1.92837561772)*x[0]+(-1.63633468831)*x[1]+(1.4212324281)*x[2]
        arg[3,2,0,1]=(1.09556054408)*x[0]+(1.43286059545)*x[1]+(1.20005604593)*x[2]
        arg[3,2,1,0]=(0.308887169169)*x[0]+(-0.149214224208)*x[1]+(-1.13559614877)*x[2]
        arg[3,2,1,1]=(-0.0648352909004)*x[0]+(-0.5645470596)*x[1]+(-0.525131730514)*x[2]
        arg[3,2,2,0]=(0.21826829974)*x[0]+(1.5950239996)*x[1]+(0.333592425438)*x[2]
        arg[3,2,2,1]=(-1.25613956172)*x[0]+(-0.242338679844)*x[1]+(-0.958875577074)*x[2]
        arg[3,3,0,0]=(1.65005362159)*x[0]+(-1.16713867165)*x[1]+(-0.282495293685)*x[2]
        arg[3,3,0,1]=(1.04854406427)*x[0]+(0.876270949531)*x[1]+(0.291036304743)*x[2]
        arg[3,3,1,0]=(-1.66619451175)*x[0]+(0.656275192484)*x[1]+(-1.27104499942)*x[2]
        arg[3,3,1,1]=(1.55407146907)*x[0]+(-0.322950943159)*x[1]+(-0.287565403896)*x[2]
        arg[3,3,2,0]=(-1.54760268583)*x[0]+(-1.45847761887)*x[1]+(1.13939567009)*x[2]
        arg[3,3,2,1]=(1.24632351699)*x[0]+(0.0265500081594)*x[1]+(1.61925095159)*x[2]
        arg[3,4,0,0]=(-0.0430889433549)*x[0]+(-1.46995559463)*x[1]+(-1.14915159823)*x[2]
        arg[3,4,0,1]=(0.0392641403409)*x[0]+(0.0603053049639)*x[1]+(0.647321169934)*x[2]
        arg[3,4,1,0]=(-0.156300985227)*x[0]+(0.79369911928)*x[1]+(-0.413124762507)*x[2]
        arg[3,4,1,1]=(0.258402059015)*x[0]+(0.00227307988585)*x[1]+(0.70067141295)*x[2]
        arg[3,4,2,0]=(-1.15131298872)*x[0]+(-0.627449043157)*x[1]+(-0.402268345861)*x[2]
        arg[3,4,2,1]=(0.455934238179)*x[0]+(-0.951120759108)*x[1]+(-0.280459367131)*x[2]
        ref[0,0,0,0]=(-0.791620170192)*x_ref[0]+(1.03871460301)*x_ref[1]+(-0.225035657687)*x_ref[2]
        ref[0,0,0,1]=(-0.464422319203)*x_ref[0]+(0.769454931053)*x_ref[1]+(-0.783260637771)*x_ref[2]
        ref[0,0,1,0]=(0.72655538215)*x_ref[0]+(-1.65736312015)*x_ref[1]+(1.08402505927)*x_ref[2]
        ref[0,0,1,1]=(-0.207576709573)*x_ref[0]+(-0.55250594761)*x_ref[1]+(-0.923702938288)*x_ref[2]
        ref[0,0,2,0]=(-0.701991085516)*x_ref[0]+(-1.76863643107)*x_ref[1]+(0.831382933104)*x_ref[2]
        ref[0,0,2,1]=(0.0640035527319)*x_ref[0]+(0.425446342976)*x_ref[1]+(1.64626766696)*x_ref[2]
        ref[0,1,0,0]=(1.01022963801)*x_ref[0]+(-0.0215015059716)*x_ref[1]+(0.849862425592)*x_ref[2]
        ref[0,1,0,1]=(-0.45776932943)*x_ref[0]+(0.7621342151)*x_ref[1]+(-1.40123018391)*x_ref[2]
        ref[0,1,1,0]=(0.737485054127)*x_ref[0]+(-0.147513905858)*x_ref[1]+(-0.616928595213)*x_ref[2]
        ref[0,1,1,1]=(-0.0608005391894)*x_ref[0]+(-0.154978210573)*x_ref[1]+(-1.40322011577)*x_ref[2]
        ref[0,1,2,0]=(1.59908177714)*x_ref[0]+(-0.378738573932)*x_ref[1]+(1.45285315247)*x_ref[2]
        ref[0,1,2,1]=(-0.622552157046)*x_ref[0]+(1.19996417516)*x_ref[1]+(0.728842900843)*x_ref[2]
        ref[0,2,0,0]=(-0.46942034691)*x_ref[0]+(0.333309721372)*x_ref[1]+(-0.280846553832)*x_ref[2]
        ref[0,2,0,1]=(1.22208788109)*x_ref[0]+(0.369339594285)*x_ref[1]+(1.31078425589)*x_ref[2]
        ref[0,2,1,0]=(0.0794165004114)*x_ref[0]+(0.44665451846)*x_ref[1]+(-0.814625708916)*x_ref[2]
        ref[0,2,1,1]=(0.594200807165)*x_ref[0]+(-0.00616720333193)*x_ref[1]+(0.382706175172)*x_ref[2]
        ref[0,2,2,0]=(0.0372230170459)*x_ref[0]+(0.440339923291)*x_ref[1]+(0.137641070028)*x_ref[2]
        ref[0,2,2,1]=(-1.29863078274)*x_ref[0]+(0.283142455171)*x_ref[1]+(0.0844540074659)*x_ref[2]
        ref[0,3,0,0]=(1.72811808036)*x_ref[0]+(0.536456781522)*x_ref[1]+(0.0266737704706)*x_ref[2]
        ref[0,3,0,1]=(0.148219555101)*x_ref[0]+(-0.6887500275)*x_ref[1]+(-0.405613136632)*x_ref[2]
        ref[0,3,1,0]=(0.708262537279)*x_ref[0]+(1.16319635773)*x_ref[1]+(-0.030808721674)*x_ref[2]
        ref[0,3,1,1]=(0.80834615863)*x_ref[0]+(-0.176789000925)*x_ref[1]+(-1.72576120963)*x_ref[2]
        ref[0,3,2,0]=(0.21128315299)*x_ref[0]+(0.770037046126)*x_ref[1]+(0.988028934422)*x_ref[2]
        ref[0,3,2,1]=(1.10522136317)*x_ref[0]+(1.6319351743)*x_ref[1]+(-0.784693713387)*x_ref[2]
        ref[0,4,0,0]=(0.121088121064)*x_ref[0]+(-1.79718067791)*x_ref[1]+(-0.907857173318)*x_ref[2]
        ref[0,4,0,1]=(-0.695887294217)*x_ref[0]+(-1.24166787807)*x_ref[1]+(-1.24968023856)*x_ref[2]
        ref[0,4,1,0]=(-0.202510797291)*x_ref[0]+(0.628310156105)*x_ref[1]+(0.605146447822)*x_ref[2]
        ref[0,4,1,1]=(0.149227458424)*x_ref[0]+(-1.5650392775)*x_ref[1]+(-0.763944892663)*x_ref[2]
        ref[0,4,2,0]=(-0.393218625872)*x_ref[0]+(0.479145914948)*x_ref[1]+(-0.313960447492)*x_ref[2]
        ref[0,4,2,1]=(-0.0191807443799)*x_ref[0]+(0.592992659119)*x_ref[1]+(-0.475633115484)*x_ref[2]
        ref[1,0,0,0]=(-0.964829354261)*x_ref[0]+(0.453014802906)*x_ref[1]+(-0.779502566938)*x_ref[2]
        ref[1,0,0,1]=(0.0151205739508)*x_ref[0]+(-0.153706429594)*x_ref[1]+(0.00866460475724)*x_ref[2]
        ref[1,0,1,0]=(0.92592276366)*x_ref[0]+(0.530787400525)*x_ref[1]+(-0.0577912606876)*x_ref[2]
        ref[1,0,1,1]=(0.948404467077)*x_ref[0]+(-0.265905470751)*x_ref[1]+(0.472349192005)*x_ref[2]
        ref[1,0,2,0]=(0.0176615383702)*x_ref[0]+(0.679605821761)*x_ref[1]+(-0.0333590911941)*x_ref[2]
        ref[1,0,2,1]=(-1.15594952629)*x_ref[0]+(0.264868376508)*x_ref[1]+(-0.883690535499)*x_ref[2]
        ref[1,1,0,0]=(-0.738712703994)*x_ref[0]+(0.545839232808)*x_ref[1]+(0.592474501134)*x_ref[2]
        ref[1,1,0,1]=(-1.3468960453)*x_ref[0]+(-0.00848202443614)*x_ref[1]+(-0.0124138286126)*x_ref[2]
        ref[1,1,1,0]=(0.900228409707)*x_ref[0]+(-1.86848082357)*x_ref[1]+(0.421539431142)*x_ref[2]
        ref[1,1,1,1]=(1.42754013963)*x_ref[0]+(-0.861376440449)*x_ref[1]+(0.0965520362291)*x_ref[2]
        ref[1,1,2,0]=(-0.35873674515)*x_ref[0]+(0.0862592617941)*x_ref[1]+(-0.536106207025)*x_ref[2]
        ref[1,1,2,1]=(0.90712910097)*x_ref[0]+(0.974545253055)*x_ref[1]+(0.570708510858)*x_ref[2]
        ref[1,2,0,0]=(1.19310775669)*x_ref[0]+(0.309414331679)*x_ref[1]+(-0.271298061586)*x_ref[2]
        ref[1,2,0,1]=(0.551778129545)*x_ref[0]+(0.539028430134)*x_ref[1]+(0.239406239262)*x_ref[2]
        ref[1,2,1,0]=(-0.810625054344)*x_ref[0]+(-0.241072190233)*x_ref[1]+(-1.58590369097)*x_ref[2]
        ref[1,2,1,1]=(0.529669075734)*x_ref[0]+(-0.890365024163)*x_ref[1]+(-1.69798509084)*x_ref[2]
        ref[1,2,2,0]=(0.0437087707448)*x_ref[0]+(0.88372190198)*x_ref[1]+(1.86547747806)*x_ref[2]
        ref[1,2,2,1]=(1.30739438898)*x_ref[0]+(0.315850112101)*x_ref[1]+(-0.262001133683)*x_ref[2]
        ref[1,3,0,0]=(-0.143426236837)*x_ref[0]+(-1.1703384826)*x_ref[1]+(0.50608348646)*x_ref[2]
        ref[1,3,0,1]=(-0.0444408776922)*x_ref[0]+(1.26952782036)*x_ref[1]+(1.67192602934)*x_ref[2]
        ref[1,3,1,0]=(0.672742824368)*x_ref[0]+(-0.785246676537)*x_ref[1]+(0.716130664056)*x_ref[2]
        ref[1,3,1,1]=(-0.295182508335)*x_ref[0]+(-1.53658860897)*x_ref[1]+(-1.87133995422)*x_ref[2]
        ref[1,3,2,0]=(-0.440859530081)*x_ref[0]+(0.0730120015821)*x_ref[1]+(-1.02465537353)*x_ref[2]
        ref[1,3,2,1]=(0.838320612003)*x_ref[0]+(1.81233688454)*x_ref[1]+(-1.0633876685)*x_ref[2]
        ref[1,4,0,0]=(0.52440453051)*x_ref[0]+(-0.0182588999022)*x_ref[1]+(1.05556310351)*x_ref[2]
        ref[1,4,0,1]=(0.766612727495)*x_ref[0]+(1.58987590746)*x_ref[1]+(-0.703801484569)*x_ref[2]
        ref[1,4,1,0]=(0.373439757535)*x_ref[0]+(-0.0676963704305)*x_ref[1]+(0.0948034094293)*x_ref[2]
        ref[1,4,1,1]=(-0.542283543375)*x_ref[0]+(-0.454408605761)*x_ref[1]+(-0.398904635316)*x_ref[2]
        ref[1,4,2,0]=(1.3374871143)*x_ref[0]+(0.0067849394587)*x_ref[1]+(-0.319452094882)*x_ref[2]
        ref[1,4,2,1]=(0.38530755393)*x_ref[0]+(1.05356462429)*x_ref[1]+(-0.302179447719)*x_ref[2]
        ref[2,0,0,0]=(-0.123109761583)*x_ref[0]+(0.69223848143)*x_ref[1]+(-1.36903487092)*x_ref[2]
        ref[2,0,0,1]=(0.121957328459)*x_ref[0]+(0.102229685387)*x_ref[1]+(0.962499974737)*x_ref[2]
        ref[2,0,1,0]=(0.478614188653)*x_ref[0]+(-1.23058817023)*x_ref[1]+(1.30950997478)*x_ref[2]
        ref[2,0,1,1]=(-0.529033562461)*x_ref[0]+(0.563030509591)*x_ref[1]+(-0.564939226654)*x_ref[2]
        ref[2,0,2,0]=(-0.377740282406)*x_ref[0]+(1.20532110343)*x_ref[1]+(1.55615174947)*x_ref[2]
        ref[2,0,2,1]=(0.539119710703)*x_ref[0]+(1.91771865857)*x_ref[1]+(0.0809882488048)*x_ref[2]
        ref[2,1,0,0]=(0.00548664737796)*x_ref[0]+(-0.157716099068)*x_ref[1]+(-1.40009044235)*x_ref[2]
        ref[2,1,0,1]=(0.0667557890666)*x_ref[0]+(0.742484448131)*x_ref[1]+(-0.325246111346)*x_ref[2]
        ref[2,1,1,0]=(-0.214965474071)*x_ref[0]+(-0.215055151964)*x_ref[1]+(1.15673960537)*x_ref[2]
        ref[2,1,1,1]=(0.605764831789)*x_ref[0]+(-1.0432964832)*x_ref[1]+(0.645482523161)*x_ref[2]
        ref[2,1,2,0]=(-1.13098364088)*x_ref[0]+(-0.932044089041)*x_ref[1]+(-0.308053828241)*x_ref[2]
        ref[2,1,2,1]=(0.481961305459)*x_ref[0]+(-0.101277472029)*x_ref[1]+(-0.17520207974)*x_ref[2]
        ref[2,2,0,0]=(0.579831852303)*x_ref[0]+(0.538731990254)*x_ref[1]+(-0.0897460374205)*x_ref[2]
        ref[2,2,0,1]=(-1.8763181434)*x_ref[0]+(-0.132710928569)*x_ref[1]+(1.03169751651)*x_ref[2]
        ref[2,2,1,0]=(0.56871511431)*x_ref[0]+(0.075333481656)*x_ref[1]+(1.10238818953)*x_ref[2]
        ref[2,2,1,1]=(1.49347755246)*x_ref[0]+(-1.00500941047)*x_ref[1]+(-0.181733196359)*x_ref[2]
        ref[2,2,2,0]=(0.532881509971)*x_ref[0]+(-0.364333964659)*x_ref[1]+(1.09633720883)*x_ref[2]
        ref[2,2,2,1]=(-0.518419048615)*x_ref[0]+(-0.48091100593)*x_ref[1]+(0.724628840153)*x_ref[2]
        ref[2,3,0,0]=(0.353475859331)*x_ref[0]+(-0.0239879269825)*x_ref[1]+(1.32068893718)*x_ref[2]
        ref[2,3,0,1]=(0.553999122102)*x_ref[0]+(-0.128530869239)*x_ref[1]+(0.134663336287)*x_ref[2]
        ref[2,3,1,0]=(-1.59775882569)*x_ref[0]+(-1.07335424785)*x_ref[1]+(1.39223587988)*x_ref[2]
        ref[2,3,1,1]=(-0.921252080095)*x_ref[0]+(1.46226299067)*x_ref[1]+(-1.13412499932)*x_ref[2]
        ref[2,3,2,0]=(0.417572910178)*x_ref[0]+(0.0814889599489)*x_ref[1]+(0.292159249719)*x_ref[2]
        ref[2,3,2,1]=(1.44620228921)*x_ref[0]+(0.767854577367)*x_ref[1]+(-0.0113709806383)*x_ref[2]
        ref[2,4,0,0]=(-0.156987001681)*x_ref[0]+(-0.922524257833)*x_ref[1]+(0.521128360838)*x_ref[2]
        ref[2,4,0,1]=(0.154198133354)*x_ref[0]+(0.462020370175)*x_ref[1]+(0.966768653665)*x_ref[2]
        ref[2,4,1,0]=(0.507836849242)*x_ref[0]+(1.16701911244)*x_ref[1]+(0.264543038211)*x_ref[2]
        ref[2,4,1,1]=(0.452059436879)*x_ref[0]+(0.409662432778)*x_ref[1]+(-0.403800872595)*x_ref[2]
        ref[2,4,2,0]=(0.521175521008)*x_ref[0]+(-0.362551578816)*x_ref[1]+(0.0922727331623)*x_ref[2]
        ref[2,4,2,1]=(-1.0990361823)*x_ref[0]+(1.73655146851)*x_ref[1]+(-0.867917402922)*x_ref[2]
        ref[3,0,0,0]=(-0.0527010882236)*x_ref[0]+(-1.11528215488)*x_ref[1]+(1.3141906393)*x_ref[2]
        ref[3,0,0,1]=(0.151284271917)*x_ref[0]+(-0.0159077450668)*x_ref[1]+(1.1798338931)*x_ref[2]
        ref[3,0,1,0]=(-1.7648963435)*x_ref[0]+(0.229511437402)*x_ref[1]+(0.924415900312)*x_ref[2]
        ref[3,0,1,1]=(-0.498271821094)*x_ref[0]+(1.58363063019)*x_ref[1]+(1.64871750388)*x_ref[2]
        ref[3,0,2,0]=(-1.11245170725)*x_ref[0]+(1.27662612569)*x_ref[1]+(0.145950372369)*x_ref[2]
        ref[3,0,2,1]=(-0.691643924238)*x_ref[0]+(-0.178211874279)*x_ref[1]+(-0.341071050366)*x_ref[2]
        ref[3,1,0,0]=(-0.929804889461)*x_ref[0]+(0.199087067309)*x_ref[1]+(1.00150168883)*x_ref[2]
        ref[3,1,0,1]=(-1.25152265093)*x_ref[0]+(0.517076294911)*x_ref[1]+(0.103798312362)*x_ref[2]
        ref[3,1,1,0]=(0.797316290623)*x_ref[0]+(0.0239285379293)*x_ref[1]+(-0.836519299418)*x_ref[2]
        ref[3,1,1,1]=(-0.470084592176)*x_ref[0]+(0.844364701728)*x_ref[1]+(-0.027373275093)*x_ref[2]
        ref[3,1,2,0]=(1.0340279373)*x_ref[0]+(-0.0133951276315)*x_ref[1]+(1.17710546194)*x_ref[2]
        ref[3,1,2,1]=(0.0325929541496)*x_ref[0]+(1.13784867184)*x_ref[1]+(-0.0529194266769)*x_ref[2]
        ref[3,2,0,0]=(1.92837561772)*x_ref[0]+(-1.63633468831)*x_ref[1]+(1.4212324281)*x_ref[2]
        ref[3,2,0,1]=(1.09556054408)*x_ref[0]+(1.43286059545)*x_ref[1]+(1.20005604593)*x_ref[2]
        ref[3,2,1,0]=(0.308887169169)*x_ref[0]+(-0.149214224208)*x_ref[1]+(-1.13559614877)*x_ref[2]
        ref[3,2,1,1]=(-0.0648352909004)*x_ref[0]+(-0.5645470596)*x_ref[1]+(-0.525131730514)*x_ref[2]
        ref[3,2,2,0]=(0.21826829974)*x_ref[0]+(1.5950239996)*x_ref[1]+(0.333592425438)*x_ref[2]
        ref[3,2,2,1]=(-1.25613956172)*x_ref[0]+(-0.242338679844)*x_ref[1]+(-0.958875577074)*x_ref[2]
        ref[3,3,0,0]=(1.65005362159)*x_ref[0]+(-1.16713867165)*x_ref[1]+(-0.282495293685)*x_ref[2]
        ref[3,3,0,1]=(1.04854406427)*x_ref[0]+(0.876270949531)*x_ref[1]+(0.291036304743)*x_ref[2]
        ref[3,3,1,0]=(-1.66619451175)*x_ref[0]+(0.656275192484)*x_ref[1]+(-1.27104499942)*x_ref[2]
        ref[3,3,1,1]=(1.55407146907)*x_ref[0]+(-0.322950943159)*x_ref[1]+(-0.287565403896)*x_ref[2]
        ref[3,3,2,0]=(-1.54760268583)*x_ref[0]+(-1.45847761887)*x_ref[1]+(1.13939567009)*x_ref[2]
        ref[3,3,2,1]=(1.24632351699)*x_ref[0]+(0.0265500081594)*x_ref[1]+(1.61925095159)*x_ref[2]
        ref[3,4,0,0]=(-0.0430889433549)*x_ref[0]+(-1.46995559463)*x_ref[1]+(-1.14915159823)*x_ref[2]
        ref[3,4,0,1]=(0.0392641403409)*x_ref[0]+(0.0603053049639)*x_ref[1]+(0.647321169934)*x_ref[2]
        ref[3,4,1,0]=(-0.156300985227)*x_ref[0]+(0.79369911928)*x_ref[1]+(-0.413124762507)*x_ref[2]
        ref[3,4,1,1]=(0.258402059015)*x_ref[0]+(0.00227307988585)*x_ref[1]+(0.70067141295)*x_ref[2]
        ref[3,4,2,0]=(-1.15131298872)*x_ref[0]+(-0.627449043157)*x_ref[1]+(-0.402268345861)*x_ref[2]
        ref[3,4,2,1]=(0.455934238179)*x_ref[0]+(-0.951120759108)*x_ref[1]+(-0.280459367131)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_ReducedContinuousFunction_rank0(self):
      """
      tests interpolate for rank 0 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedContinuousFunction onto ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(),w)
      ref=Data(0,(),w_ref)
      if dim==2:
        arg=(-0.787646538574)*x[0]+(0.912422904218)*x[1]
        ref=(-0.787646538574)*x_ref[0]+(0.912422904218)*x_ref[1]
      else:
        arg=(0.934044939024)*x[0]+(0.628477728603)*x[1]+(-0.786339405837)*x[2]
        ref=(0.934044939024)*x_ref[0]+(0.628477728603)*x_ref[1]+(-0.786339405837)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_ReducedContinuousFunction_rank1(self):
      """
      tests interpolate for rank 1 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedContinuousFunction onto ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(2,),w)
      ref=Data(0,(2,),w_ref)
      if dim==2:
        arg[0]=(-0.588326094839)*x[0]+(1.56296941343)*x[1]
        arg[1]=(-0.802191222185)*x[0]+(-1.13471016062)*x[1]
        ref[0]=(-0.588326094839)*x[0]+(1.56296941343)*x[1]
        ref[1]=(-0.802191222185)*x[0]+(-1.13471016062)*x[1]
      else:
        arg[0]=(0.160702771321)*x[0]+(-0.0784425374814)*x[1]+(1.65226639722)*x[2]
        arg[1]=(0.745050820919)*x[0]+(-0.183859360073)*x[1]+(0.399813139488)*x[2]
        ref[0]=(0.160702771321)*x[0]+(-0.0784425374814)*x[1]+(1.65226639722)*x[2]
        ref[1]=(0.745050820919)*x[0]+(-0.183859360073)*x[1]+(0.399813139488)*x[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_ReducedContinuousFunction_rank2(self):
      """
      tests interpolate for rank 2 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedContinuousFunction onto ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5),w)
      ref=Data(0,(4, 5),w_ref)
      if dim==2:
        arg[0,0]=(-0.797066914479)*x[0]+(-0.0269853164939)*x[1]
        arg[0,1]=(-1.07443661798)*x[0]+(0.818510216043)*x[1]
        arg[0,2]=(0.398735262179)*x[0]+(-0.580387872083)*x[1]
        arg[0,3]=(-0.003185613959)*x[0]+(-0.670506330615)*x[1]
        arg[0,4]=(0.385009570186)*x[0]+(0.858570512936)*x[1]
        arg[1,0]=(1.60917176753)*x[0]+(-0.0866362449992)*x[1]
        arg[1,1]=(-0.883782122452)*x[0]+(-0.402187705578)*x[1]
        arg[1,2]=(0.153893095122)*x[0]+(-1.19636362644)*x[1]
        arg[1,3]=(1.92041961031)*x[0]+(0.325857877636)*x[1]
        arg[1,4]=(-0.0947698843729)*x[0]+(-0.0663785758627)*x[1]
        arg[2,0]=(0.0232949467051)*x[0]+(1.60134210326)*x[1]
        arg[2,1]=(-0.521182882924)*x[0]+(-0.507401571343)*x[1]
        arg[2,2]=(0.833797117616)*x[0]+(-0.268886477194)*x[1]
        arg[2,3]=(-0.867762611227)*x[0]+(1.03727088566)*x[1]
        arg[2,4]=(0.380510651918)*x[0]+(0.350246888456)*x[1]
        arg[3,0]=(-1.10155500839)*x[0]+(0.414683749118)*x[1]
        arg[3,1]=(-0.0306733233552)*x[0]+(-0.351251014237)*x[1]
        arg[3,2]=(-0.0708773001498)*x[0]+(-0.326636587948)*x[1]
        arg[3,3]=(-1.32553119895)*x[0]+(-1.5852157477)*x[1]
        arg[3,4]=(1.40177154802)*x[0]+(-1.24804787994)*x[1]
        ref[0,0]=(-0.797066914479)*x_ref[0]+(-0.0269853164939)*x_ref[1]
        ref[0,1]=(-1.07443661798)*x_ref[0]+(0.818510216043)*x_ref[1]
        ref[0,2]=(0.398735262179)*x_ref[0]+(-0.580387872083)*x_ref[1]
        ref[0,3]=(-0.003185613959)*x_ref[0]+(-0.670506330615)*x_ref[1]
        ref[0,4]=(0.385009570186)*x_ref[0]+(0.858570512936)*x_ref[1]
        ref[1,0]=(1.60917176753)*x_ref[0]+(-0.0866362449992)*x_ref[1]
        ref[1,1]=(-0.883782122452)*x_ref[0]+(-0.402187705578)*x_ref[1]
        ref[1,2]=(0.153893095122)*x_ref[0]+(-1.19636362644)*x_ref[1]
        ref[1,3]=(1.92041961031)*x_ref[0]+(0.325857877636)*x_ref[1]
        ref[1,4]=(-0.0947698843729)*x_ref[0]+(-0.0663785758627)*x_ref[1]
        ref[2,0]=(0.0232949467051)*x_ref[0]+(1.60134210326)*x_ref[1]
        ref[2,1]=(-0.521182882924)*x_ref[0]+(-0.507401571343)*x_ref[1]
        ref[2,2]=(0.833797117616)*x_ref[0]+(-0.268886477194)*x_ref[1]
        ref[2,3]=(-0.867762611227)*x_ref[0]+(1.03727088566)*x_ref[1]
        ref[2,4]=(0.380510651918)*x_ref[0]+(0.350246888456)*x_ref[1]
        ref[3,0]=(-1.10155500839)*x_ref[0]+(0.414683749118)*x_ref[1]
        ref[3,1]=(-0.0306733233552)*x_ref[0]+(-0.351251014237)*x_ref[1]
        ref[3,2]=(-0.0708773001498)*x_ref[0]+(-0.326636587948)*x_ref[1]
        ref[3,3]=(-1.32553119895)*x_ref[0]+(-1.5852157477)*x_ref[1]
        ref[3,4]=(1.40177154802)*x_ref[0]+(-1.24804787994)*x_ref[1]
      else:
        arg[0,0]=(0.000362194678523)*x[0]+(-1.63949220671)*x[1]+(-0.176997156562)*x[2]
        arg[0,1]=(-0.357589049408)*x[0]+(0.828191515325)*x[1]+(-0.269706780136)*x[2]
        arg[0,2]=(0.886395070544)*x[0]+(0.815688451839)*x[1]+(-0.658412270558)*x[2]
        arg[0,3]=(-0.222573318026)*x[0]+(0.137763803092)*x[1]+(0.171902150187)*x[2]
        arg[0,4]=(1.05159741251)*x[0]+(0.923836841038)*x[1]+(-0.426590801731)*x[2]
        arg[1,0]=(-0.00718419057788)*x[0]+(1.55809964998)*x[1]+(-0.912766657496)*x[2]
        arg[1,1]=(1.30769045785)*x[0]+(-0.0356609338389)*x[1]+(0.214543084134)*x[2]
        arg[1,2]=(-1.56763014241)*x[0]+(-0.582970995722)*x[1]+(-0.513972924752)*x[2]
        arg[1,3]=(0.118027736928)*x[0]+(-0.28795014876)*x[1]+(0.620718623034)*x[2]
        arg[1,4]=(-0.784496170193)*x[0]+(0.233037866924)*x[1]+(-0.0500718134572)*x[2]
        arg[2,0]=(0.800851281806)*x[0]+(0.219346484846)*x[1]+(1.19125787701)*x[2]
        arg[2,1]=(-1.45025879429)*x[0]+(-1.32011538154)*x[1]+(0.47946669308)*x[2]
        arg[2,2]=(-0.231377110469)*x[0]+(-0.921551695999)*x[1]+(-0.298964635841)*x[2]
        arg[2,3]=(0.965766335546)*x[0]+(0.424791610089)*x[1]+(1.01860652092)*x[2]
        arg[2,4]=(0.676247093636)*x[0]+(-1.50802741511)*x[1]+(0.0385960918391)*x[2]
        arg[3,0]=(0.322378151756)*x[0]+(-0.483633005509)*x[1]+(-0.852371707865)*x[2]
        arg[3,1]=(-0.241998307153)*x[0]+(-1.08134340082)*x[1]+(-0.849574532537)*x[2]
        arg[3,2]=(-0.0185751405467)*x[0]+(0.179558416001)*x[1]+(-0.488727308548)*x[2]
        arg[3,3]=(0.0984555057845)*x[0]+(1.03085967788)*x[1]+(-0.357009094371)*x[2]
        arg[3,4]=(0.430593980121)*x[0]+(0.689565614489)*x[1]+(-0.0916541808491)*x[2]
        ref[0,0]=(0.000362194678523)*x_ref[0]+(-1.63949220671)*x_ref[1]+(-0.176997156562)*x_ref[2]
        ref[0,1]=(-0.357589049408)*x_ref[0]+(0.828191515325)*x_ref[1]+(-0.269706780136)*x_ref[2]
        ref[0,2]=(0.886395070544)*x_ref[0]+(0.815688451839)*x_ref[1]+(-0.658412270558)*x_ref[2]
        ref[0,3]=(-0.222573318026)*x_ref[0]+(0.137763803092)*x_ref[1]+(0.171902150187)*x_ref[2]
        ref[0,4]=(1.05159741251)*x_ref[0]+(0.923836841038)*x_ref[1]+(-0.426590801731)*x_ref[2]
        ref[1,0]=(-0.00718419057788)*x_ref[0]+(1.55809964998)*x_ref[1]+(-0.912766657496)*x_ref[2]
        ref[1,1]=(1.30769045785)*x_ref[0]+(-0.0356609338389)*x_ref[1]+(0.214543084134)*x_ref[2]
        ref[1,2]=(-1.56763014241)*x_ref[0]+(-0.582970995722)*x_ref[1]+(-0.513972924752)*x_ref[2]
        ref[1,3]=(0.118027736928)*x_ref[0]+(-0.28795014876)*x_ref[1]+(0.620718623034)*x_ref[2]
        ref[1,4]=(-0.784496170193)*x_ref[0]+(0.233037866924)*x_ref[1]+(-0.0500718134572)*x_ref[2]
        ref[2,0]=(0.800851281806)*x_ref[0]+(0.219346484846)*x_ref[1]+(1.19125787701)*x_ref[2]
        ref[2,1]=(-1.45025879429)*x_ref[0]+(-1.32011538154)*x_ref[1]+(0.47946669308)*x_ref[2]
        ref[2,2]=(-0.231377110469)*x_ref[0]+(-0.921551695999)*x_ref[1]+(-0.298964635841)*x_ref[2]
        ref[2,3]=(0.965766335546)*x_ref[0]+(0.424791610089)*x_ref[1]+(1.01860652092)*x_ref[2]
        ref[2,4]=(0.676247093636)*x_ref[0]+(-1.50802741511)*x_ref[1]+(0.0385960918391)*x_ref[2]
        ref[3,0]=(0.322378151756)*x_ref[0]+(-0.483633005509)*x_ref[1]+(-0.852371707865)*x_ref[2]
        ref[3,1]=(-0.241998307153)*x_ref[0]+(-1.08134340082)*x_ref[1]+(-0.849574532537)*x_ref[2]
        ref[3,2]=(-0.0185751405467)*x_ref[0]+(0.179558416001)*x_ref[1]+(-0.488727308548)*x_ref[2]
        ref[3,3]=(0.0984555057845)*x_ref[0]+(1.03085967788)*x_ref[1]+(-0.357009094371)*x_ref[2]
        ref[3,4]=(0.430593980121)*x_ref[0]+(0.689565614489)*x_ref[1]+(-0.0916541808491)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_ReducedContinuousFunction_rank3(self):
      """
      tests interpolate for rank 3 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedContinuousFunction onto ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2),w_ref)
      if dim==2:
        arg[0,0,0]=(-0.0641544023301)*x[0]+(0.364286357763)*x[1]
        arg[0,0,1]=(-0.858288240411)*x[0]+(-0.847764393559)*x[1]
        arg[0,1,0]=(-0.656948775029)*x[0]+(1.69748608992)*x[1]
        arg[0,1,1]=(0.579399191549)*x[0]+(1.48407368783)*x[1]
        arg[1,0,0]=(-0.35948657403)*x[0]+(0.795436090765)*x[1]
        arg[1,0,1]=(-1.32807156246)*x[0]+(0.921960610294)*x[1]
        arg[1,1,0]=(1.0904085219)*x[0]+(-1.41870997296)*x[1]
        arg[1,1,1]=(-0.677691596793)*x[0]+(-0.395373706333)*x[1]
        arg[2,0,0]=(0.295591917216)*x[0]+(-1.75550830457)*x[1]
        arg[2,0,1]=(0.506665326626)*x[0]+(0.273265713965)*x[1]
        arg[2,1,0]=(0.543899865352)*x[0]+(-0.0678457091657)*x[1]
        arg[2,1,1]=(-0.887824769535)*x[0]+(0.527475538055)*x[1]
        arg[3,0,0]=(-0.788511557365)*x[0]+(-0.635972100675)*x[1]
        arg[3,0,1]=(-1.02799066459)*x[0]+(-0.416241247725)*x[1]
        arg[3,1,0]=(-0.310072483091)*x[0]+(1.2445160445)*x[1]
        arg[3,1,1]=(0.912462196567)*x[0]+(0.631855269783)*x[1]
        arg[4,0,0]=(0.700426861807)*x[0]+(-0.491365913713)*x[1]
        arg[4,0,1]=(0.628791538338)*x[0]+(0.640767645586)*x[1]
        arg[4,1,0]=(0.370886720329)*x[0]+(0.823663448834)*x[1]
        arg[4,1,1]=(1.75857310774)*x[0]+(0.445439424641)*x[1]
        arg[5,0,0]=(1.39637385066)*x[0]+(-0.812211860169)*x[1]
        arg[5,0,1]=(0.865050350195)*x[0]+(-1.25983380375)*x[1]
        arg[5,1,0]=(-0.0429057645445)*x[0]+(-1.08490578934)*x[1]
        arg[5,1,1]=(1.42851264594)*x[0]+(1.52192971749)*x[1]
        ref[0,0,0]=(-0.0641544023301)*x_ref[0]+(0.364286357763)*x_ref[1]
        ref[0,0,1]=(-0.858288240411)*x_ref[0]+(-0.847764393559)*x_ref[1]
        ref[0,1,0]=(-0.656948775029)*x_ref[0]+(1.69748608992)*x_ref[1]
        ref[0,1,1]=(0.579399191549)*x_ref[0]+(1.48407368783)*x_ref[1]
        ref[1,0,0]=(-0.35948657403)*x_ref[0]+(0.795436090765)*x_ref[1]
        ref[1,0,1]=(-1.32807156246)*x_ref[0]+(0.921960610294)*x_ref[1]
        ref[1,1,0]=(1.0904085219)*x_ref[0]+(-1.41870997296)*x_ref[1]
        ref[1,1,1]=(-0.677691596793)*x_ref[0]+(-0.395373706333)*x_ref[1]
        ref[2,0,0]=(0.295591917216)*x_ref[0]+(-1.75550830457)*x_ref[1]
        ref[2,0,1]=(0.506665326626)*x_ref[0]+(0.273265713965)*x_ref[1]
        ref[2,1,0]=(0.543899865352)*x_ref[0]+(-0.0678457091657)*x_ref[1]
        ref[2,1,1]=(-0.887824769535)*x_ref[0]+(0.527475538055)*x_ref[1]
        ref[3,0,0]=(-0.788511557365)*x_ref[0]+(-0.635972100675)*x_ref[1]
        ref[3,0,1]=(-1.02799066459)*x_ref[0]+(-0.416241247725)*x_ref[1]
        ref[3,1,0]=(-0.310072483091)*x_ref[0]+(1.2445160445)*x_ref[1]
        ref[3,1,1]=(0.912462196567)*x_ref[0]+(0.631855269783)*x_ref[1]
        ref[4,0,0]=(0.700426861807)*x_ref[0]+(-0.491365913713)*x_ref[1]
        ref[4,0,1]=(0.628791538338)*x_ref[0]+(0.640767645586)*x_ref[1]
        ref[4,1,0]=(0.370886720329)*x_ref[0]+(0.823663448834)*x_ref[1]
        ref[4,1,1]=(1.75857310774)*x_ref[0]+(0.445439424641)*x_ref[1]
        ref[5,0,0]=(1.39637385066)*x_ref[0]+(-0.812211860169)*x_ref[1]
        ref[5,0,1]=(0.865050350195)*x_ref[0]+(-1.25983380375)*x_ref[1]
        ref[5,1,0]=(-0.0429057645445)*x_ref[0]+(-1.08490578934)*x_ref[1]
        ref[5,1,1]=(1.42851264594)*x_ref[0]+(1.52192971749)*x_ref[1]
      else:
        arg[0,0,0]=(0.731194732468)*x[0]+(-0.0822115660022)*x[1]+(-0.403175091289)*x[2]
        arg[0,0,1]=(0.39292230307)*x[0]+(1.48598681615)*x[1]+(1.05545374107)*x[2]
        arg[0,1,0]=(0.306844260706)*x[0]+(0.836496026763)*x[1]+(-0.121120983155)*x[2]
        arg[0,1,1]=(-0.155245802062)*x[0]+(0.800232934864)*x[1]+(0.373397160636)*x[2]
        arg[1,0,0]=(0.155941790359)*x[0]+(0.826821969281)*x[1]+(0.849596807425)*x[2]
        arg[1,0,1]=(-0.0988854656187)*x[0]+(-0.855032499842)*x[1]+(0.141988288566)*x[2]
        arg[1,1,0]=(-0.761032539296)*x[0]+(-0.0892979073234)*x[1]+(1.93034385132)*x[2]
        arg[1,1,1]=(-1.08303603558)*x[0]+(-0.487250117659)*x[1]+(0.0358501036173)*x[2]
        arg[2,0,0]=(0.148391465413)*x[0]+(-0.731963025625)*x[1]+(0.484236965689)*x[2]
        arg[2,0,1]=(1.06691217722)*x[0]+(-0.638095266244)*x[1]+(-0.971838410779)*x[2]
        arg[2,1,0]=(-0.377750829061)*x[0]+(-0.101467967263)*x[1]+(-1.54241387349)*x[2]
        arg[2,1,1]=(0.62210350231)*x[0]+(0.0435334873911)*x[1]+(-0.299303646623)*x[2]
        arg[3,0,0]=(0.523811147108)*x[0]+(-1.09709839108)*x[1]+(1.83592280935)*x[2]
        arg[3,0,1]=(0.957738143475)*x[0]+(0.282158440867)*x[1]+(0.621694180522)*x[2]
        arg[3,1,0]=(0.366557891822)*x[0]+(0.812194047631)*x[1]+(-0.245067876349)*x[2]
        arg[3,1,1]=(0.604859464982)*x[0]+(1.08415514854)*x[1]+(-1.470128848)*x[2]
        arg[4,0,0]=(-0.13031502499)*x[0]+(-1.20707238733)*x[1]+(0.108382497552)*x[2]
        arg[4,0,1]=(1.30453246317)*x[0]+(1.59696612588)*x[1]+(-0.234259043697)*x[2]
        arg[4,1,0]=(-0.233494640836)*x[0]+(1.10701061816)*x[1]+(-0.409927013674)*x[2]
        arg[4,1,1]=(0.102708240266)*x[0]+(-0.381598345002)*x[1]+(-1.39077748656)*x[2]
        arg[5,0,0]=(1.06217404493)*x[0]+(1.1600791444)*x[1]+(-1.45242735389)*x[2]
        arg[5,0,1]=(-0.423024189506)*x[0]+(-0.0868948651641)*x[1]+(0.0894539043522)*x[2]
        arg[5,1,0]=(1.17650613067)*x[0]+(0.993832919965)*x[1]+(1.16546768569)*x[2]
        arg[5,1,1]=(-0.61549132897)*x[0]+(-0.499619662734)*x[1]+(-0.643422281669)*x[2]
        ref[0,0,0]=(0.731194732468)*x_ref[0]+(-0.0822115660022)*x_ref[1]+(-0.403175091289)*x_ref[2]
        ref[0,0,1]=(0.39292230307)*x_ref[0]+(1.48598681615)*x_ref[1]+(1.05545374107)*x_ref[2]
        ref[0,1,0]=(0.306844260706)*x_ref[0]+(0.836496026763)*x_ref[1]+(-0.121120983155)*x_ref[2]
        ref[0,1,1]=(-0.155245802062)*x_ref[0]+(0.800232934864)*x_ref[1]+(0.373397160636)*x_ref[2]
        ref[1,0,0]=(0.155941790359)*x_ref[0]+(0.826821969281)*x_ref[1]+(0.849596807425)*x_ref[2]
        ref[1,0,1]=(-0.0988854656187)*x_ref[0]+(-0.855032499842)*x_ref[1]+(0.141988288566)*x_ref[2]
        ref[1,1,0]=(-0.761032539296)*x_ref[0]+(-0.0892979073234)*x_ref[1]+(1.93034385132)*x_ref[2]
        ref[1,1,1]=(-1.08303603558)*x_ref[0]+(-0.487250117659)*x_ref[1]+(0.0358501036173)*x_ref[2]
        ref[2,0,0]=(0.148391465413)*x_ref[0]+(-0.731963025625)*x_ref[1]+(0.484236965689)*x_ref[2]
        ref[2,0,1]=(1.06691217722)*x_ref[0]+(-0.638095266244)*x_ref[1]+(-0.971838410779)*x_ref[2]
        ref[2,1,0]=(-0.377750829061)*x_ref[0]+(-0.101467967263)*x_ref[1]+(-1.54241387349)*x_ref[2]
        ref[2,1,1]=(0.62210350231)*x_ref[0]+(0.0435334873911)*x_ref[1]+(-0.299303646623)*x_ref[2]
        ref[3,0,0]=(0.523811147108)*x_ref[0]+(-1.09709839108)*x_ref[1]+(1.83592280935)*x_ref[2]
        ref[3,0,1]=(0.957738143475)*x_ref[0]+(0.282158440867)*x_ref[1]+(0.621694180522)*x_ref[2]
        ref[3,1,0]=(0.366557891822)*x_ref[0]+(0.812194047631)*x_ref[1]+(-0.245067876349)*x_ref[2]
        ref[3,1,1]=(0.604859464982)*x_ref[0]+(1.08415514854)*x_ref[1]+(-1.470128848)*x_ref[2]
        ref[4,0,0]=(-0.13031502499)*x_ref[0]+(-1.20707238733)*x_ref[1]+(0.108382497552)*x_ref[2]
        ref[4,0,1]=(1.30453246317)*x_ref[0]+(1.59696612588)*x_ref[1]+(-0.234259043697)*x_ref[2]
        ref[4,1,0]=(-0.233494640836)*x_ref[0]+(1.10701061816)*x_ref[1]+(-0.409927013674)*x_ref[2]
        ref[4,1,1]=(0.102708240266)*x_ref[0]+(-0.381598345002)*x_ref[1]+(-1.39077748656)*x_ref[2]
        ref[5,0,0]=(1.06217404493)*x_ref[0]+(1.1600791444)*x_ref[1]+(-1.45242735389)*x_ref[2]
        ref[5,0,1]=(-0.423024189506)*x_ref[0]+(-0.0868948651641)*x_ref[1]+(0.0894539043522)*x_ref[2]
        ref[5,1,0]=(1.17650613067)*x_ref[0]+(0.993832919965)*x_ref[1]+(1.16546768569)*x_ref[2]
        ref[5,1,1]=(-0.61549132897)*x_ref[0]+(-0.499619662734)*x_ref[1]+(-0.643422281669)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_ReducedContinuousFunction_rank4(self):
      """
      tests interpolate for rank 4 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedContinuousFunction onto ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5, 3, 2),w)
      ref=Data(0,(4, 5, 3, 2),w_ref)
      if dim==2:
        arg[0,0,0,0]=(0.227969627252)*x[0]+(0.778177350248)*x[1]
        arg[0,0,0,1]=(-0.684883711494)*x[0]+(-0.574285957132)*x[1]
        arg[0,0,1,0]=(0.483090608974)*x[0]+(0.0779929087959)*x[1]
        arg[0,0,1,1]=(-0.293150814932)*x[0]+(1.35368931993)*x[1]
        arg[0,0,2,0]=(-0.190664734662)*x[0]+(0.910883386754)*x[1]
        arg[0,0,2,1]=(-1.51255575457)*x[0]+(0.421560181501)*x[1]
        arg[0,1,0,0]=(-0.833402828032)*x[0]+(-0.400466806781)*x[1]
        arg[0,1,0,1]=(-0.212099323605)*x[0]+(-0.112478697221)*x[1]
        arg[0,1,1,0]=(0.57687662926)*x[0]+(0.869758954551)*x[1]
        arg[0,1,1,1]=(1.37261769917)*x[0]+(1.37855789907)*x[1]
        arg[0,1,2,0]=(-1.02838589722)*x[0]+(-0.280409246204)*x[1]
        arg[0,1,2,1]=(1.2333373468)*x[0]+(0.337631183798)*x[1]
        arg[0,2,0,0]=(-0.249065448772)*x[0]+(0.834483646317)*x[1]
        arg[0,2,0,1]=(-1.30860297683)*x[0]+(-0.653929967017)*x[1]
        arg[0,2,1,0]=(-0.0155994484453)*x[0]+(-0.274023166937)*x[1]
        arg[0,2,1,1]=(0.31458360335)*x[0]+(1.59507700673)*x[1]
        arg[0,2,2,0]=(-0.11442619553)*x[0]+(1.6577028576)*x[1]
        arg[0,2,2,1]=(0.274261132032)*x[0]+(1.15236962479)*x[1]
        arg[0,3,0,0]=(1.91433860239)*x[0]+(-0.316349747811)*x[1]
        arg[0,3,0,1]=(-1.42457591054)*x[0]+(-0.362812690805)*x[1]
        arg[0,3,1,0]=(0.572734883096)*x[0]+(0.424724787283)*x[1]
        arg[0,3,1,1]=(-0.118538376813)*x[0]+(0.25190312527)*x[1]
        arg[0,3,2,0]=(-0.533409510081)*x[0]+(-0.804349107915)*x[1]
        arg[0,3,2,1]=(-0.290555363666)*x[0]+(-0.912641477145)*x[1]
        arg[0,4,0,0]=(-0.469612775918)*x[0]+(-0.0447411310793)*x[1]
        arg[0,4,0,1]=(0.108538258831)*x[0]+(0.0453962016662)*x[1]
        arg[0,4,1,0]=(-0.130785944028)*x[0]+(0.0887280914032)*x[1]
        arg[0,4,1,1]=(0.166412309585)*x[0]+(-0.75130797706)*x[1]
        arg[0,4,2,0]=(1.22538232455)*x[0]+(0.112925588343)*x[1]
        arg[0,4,2,1]=(-1.02455171854)*x[0]+(-0.725754942749)*x[1]
        arg[1,0,0,0]=(-0.869171086952)*x[0]+(-0.948876783682)*x[1]
        arg[1,0,0,1]=(0.17730673689)*x[0]+(-0.00166180855579)*x[1]
        arg[1,0,1,0]=(0.482072704433)*x[0]+(0.895847096783)*x[1]
        arg[1,0,1,1]=(0.695736834617)*x[0]+(0.358082704558)*x[1]
        arg[1,0,2,0]=(-0.501799636565)*x[0]+(0.363612558025)*x[1]
        arg[1,0,2,1]=(-1.47204447226)*x[0]+(1.91281426121)*x[1]
        arg[1,1,0,0]=(0.241893182978)*x[0]+(0.95199758669)*x[1]
        arg[1,1,0,1]=(-0.68112361827)*x[0]+(-0.241935141301)*x[1]
        arg[1,1,1,0]=(-0.553030972306)*x[0]+(0.0244503690439)*x[1]
        arg[1,1,1,1]=(0.278177176896)*x[0]+(1.58693846427)*x[1]
        arg[1,1,2,0]=(-0.87980015517)*x[0]+(0.315618837123)*x[1]
        arg[1,1,2,1]=(-0.335783961379)*x[0]+(0.368650136138)*x[1]
        arg[1,2,0,0]=(-0.903535146802)*x[0]+(1.68182923194)*x[1]
        arg[1,2,0,1]=(1.06511511382)*x[0]+(-1.08524762302)*x[1]
        arg[1,2,1,0]=(-0.0455274784681)*x[0]+(-0.519576710661)*x[1]
        arg[1,2,1,1]=(0.735100217062)*x[0]+(-0.986268682671)*x[1]
        arg[1,2,2,0]=(-0.544610797462)*x[0]+(0.447237114882)*x[1]
        arg[1,2,2,1]=(0.385242031153)*x[0]+(-0.960538844247)*x[1]
        arg[1,3,0,0]=(-0.354927072897)*x[0]+(0.842574324062)*x[1]
        arg[1,3,0,1]=(0.319036427919)*x[0]+(-0.156282262536)*x[1]
        arg[1,3,1,0]=(0.587123807717)*x[0]+(1.65277990726)*x[1]
        arg[1,3,1,1]=(0.791697370298)*x[0]+(-1.5681680344)*x[1]
        arg[1,3,2,0]=(-1.41931242045)*x[0]+(0.351285136557)*x[1]
        arg[1,3,2,1]=(0.53175704402)*x[0]+(0.987523241324)*x[1]
        arg[1,4,0,0]=(0.00533504818465)*x[0]+(-0.158886930817)*x[1]
        arg[1,4,0,1]=(1.23725297303)*x[0]+(-0.302896452172)*x[1]
        arg[1,4,1,0]=(0.433953146329)*x[0]+(-0.0721602234887)*x[1]
        arg[1,4,1,1]=(-0.383499726865)*x[0]+(-0.253834671235)*x[1]
        arg[1,4,2,0]=(-0.480776695247)*x[0]+(-0.302145524759)*x[1]
        arg[1,4,2,1]=(1.14113180903)*x[0]+(-1.24304726003)*x[1]
        arg[2,0,0,0]=(0.513776013893)*x[0]+(-0.16660259885)*x[1]
        arg[2,0,0,1]=(1.55739209698)*x[0]+(0.0524084654531)*x[1]
        arg[2,0,1,0]=(0.222567881446)*x[0]+(-0.357158470388)*x[1]
        arg[2,0,1,1]=(0.0179816596229)*x[0]+(0.231534460784)*x[1]
        arg[2,0,2,0]=(1.58460345297)*x[0]+(-0.383588433888)*x[1]
        arg[2,0,2,1]=(-1.73386465536)*x[0]+(-0.743326657544)*x[1]
        arg[2,1,0,0]=(-0.737976950246)*x[0]+(-1.01053396224)*x[1]
        arg[2,1,0,1]=(0.926911190393)*x[0]+(0.620251783925)*x[1]
        arg[2,1,1,0]=(-1.34289630129)*x[0]+(-0.429933681523)*x[1]
        arg[2,1,1,1]=(1.79530806415)*x[0]+(-0.539783957021)*x[1]
        arg[2,1,2,0]=(0.237950974938)*x[0]+(1.93783427703)*x[1]
        arg[2,1,2,1]=(-0.439640741973)*x[0]+(-0.61543984673)*x[1]
        arg[2,2,0,0]=(-1.66403612438)*x[0]+(1.00839566441)*x[1]
        arg[2,2,0,1]=(0.249635983351)*x[0]+(-0.722261363675)*x[1]
        arg[2,2,1,0]=(1.34179674459)*x[0]+(0.502542144188)*x[1]
        arg[2,2,1,1]=(-0.815862586074)*x[0]+(-0.110688273707)*x[1]
        arg[2,2,2,0]=(-0.686810719526)*x[0]+(0.500406563421)*x[1]
        arg[2,2,2,1]=(0.681231089598)*x[0]+(0.605219505613)*x[1]
        arg[2,3,0,0]=(-0.953299029956)*x[0]+(-0.223725858372)*x[1]
        arg[2,3,0,1]=(0.831380756684)*x[0]+(-0.0379777076335)*x[1]
        arg[2,3,1,0]=(-1.63804393157)*x[0]+(0.148235731301)*x[1]
        arg[2,3,1,1]=(-0.17786458643)*x[0]+(-1.77342152699)*x[1]
        arg[2,3,2,0]=(0.0210446244192)*x[0]+(1.59778781524)*x[1]
        arg[2,3,2,1]=(0.877162721214)*x[0]+(0.262990448614)*x[1]
        arg[2,4,0,0]=(-0.650265970331)*x[0]+(-1.07003604369)*x[1]
        arg[2,4,0,1]=(1.12940101029)*x[0]+(0.860589290891)*x[1]
        arg[2,4,1,0]=(0.520056210056)*x[0]+(-1.47691193979)*x[1]
        arg[2,4,1,1]=(1.26823535235)*x[0]+(0.496811720647)*x[1]
        arg[2,4,2,0]=(-1.388913443)*x[0]+(0.585343197943)*x[1]
        arg[2,4,2,1]=(-0.762904547594)*x[0]+(0.503438815917)*x[1]
        arg[3,0,0,0]=(0.382862686993)*x[0]+(-0.923525513303)*x[1]
        arg[3,0,0,1]=(-0.54305794485)*x[0]+(0.467513531444)*x[1]
        arg[3,0,1,0]=(1.24228759776)*x[0]+(-1.65740117421)*x[1]
        arg[3,0,1,1]=(-0.463045362742)*x[0]+(0.186438587087)*x[1]
        arg[3,0,2,0]=(0.296881980828)*x[0]+(-0.385878814949)*x[1]
        arg[3,0,2,1]=(-0.299705899564)*x[0]+(0.490736038538)*x[1]
        arg[3,1,0,0]=(-0.786851428658)*x[0]+(-0.760593794308)*x[1]
        arg[3,1,0,1]=(0.506851336483)*x[0]+(0.995163951197)*x[1]
        arg[3,1,1,0]=(-0.678883937341)*x[0]+(0.56996778248)*x[1]
        arg[3,1,1,1]=(-0.401483642528)*x[0]+(0.0476797332537)*x[1]
        arg[3,1,2,0]=(-0.585357955375)*x[0]+(0.330103647539)*x[1]
        arg[3,1,2,1]=(-0.355539976825)*x[0]+(0.416287924349)*x[1]
        arg[3,2,0,0]=(-0.662837130286)*x[0]+(-0.26640899674)*x[1]
        arg[3,2,0,1]=(0.397553900588)*x[0]+(-1.21708620294)*x[1]
        arg[3,2,1,0]=(0.464963326429)*x[0]+(-0.774515114142)*x[1]
        arg[3,2,1,1]=(0.834351469791)*x[0]+(0.487629540715)*x[1]
        arg[3,2,2,0]=(0.27622693716)*x[0]+(0.811338337719)*x[1]
        arg[3,2,2,1]=(0.0384997772653)*x[0]+(1.03724996578)*x[1]
        arg[3,3,0,0]=(-0.805491987874)*x[0]+(-0.476384547853)*x[1]
        arg[3,3,0,1]=(0.615551780195)*x[0]+(-0.312805967691)*x[1]
        arg[3,3,1,0]=(0.348666798955)*x[0]+(-1.31271156524)*x[1]
        arg[3,3,1,1]=(-0.423569975374)*x[0]+(0.930713125389)*x[1]
        arg[3,3,2,0]=(0.325632686234)*x[0]+(-0.166401033259)*x[1]
        arg[3,3,2,1]=(0.126924861797)*x[0]+(-0.425380897969)*x[1]
        arg[3,4,0,0]=(0.169532204538)*x[0]+(0.614632525765)*x[1]
        arg[3,4,0,1]=(-0.938098696851)*x[0]+(0.648854147587)*x[1]
        arg[3,4,1,0]=(-0.545143970434)*x[0]+(-1.59787515966)*x[1]
        arg[3,4,1,1]=(-0.711190914978)*x[0]+(0.897331522911)*x[1]
        arg[3,4,2,0]=(-0.386498124525)*x[0]+(0.299740420711)*x[1]
        arg[3,4,2,1]=(0.617683032062)*x[0]+(-0.449587703061)*x[1]
        ref[0,0,0,0]=(0.227969627252)*x_ref[0]+(0.778177350248)*x_ref[1]
        ref[0,0,0,1]=(-0.684883711494)*x_ref[0]+(-0.574285957132)*x_ref[1]
        ref[0,0,1,0]=(0.483090608974)*x_ref[0]+(0.0779929087959)*x_ref[1]
        ref[0,0,1,1]=(-0.293150814932)*x_ref[0]+(1.35368931993)*x_ref[1]
        ref[0,0,2,0]=(-0.190664734662)*x_ref[0]+(0.910883386754)*x_ref[1]
        ref[0,0,2,1]=(-1.51255575457)*x_ref[0]+(0.421560181501)*x_ref[1]
        ref[0,1,0,0]=(-0.833402828032)*x_ref[0]+(-0.400466806781)*x_ref[1]
        ref[0,1,0,1]=(-0.212099323605)*x_ref[0]+(-0.112478697221)*x_ref[1]
        ref[0,1,1,0]=(0.57687662926)*x_ref[0]+(0.869758954551)*x_ref[1]
        ref[0,1,1,1]=(1.37261769917)*x_ref[0]+(1.37855789907)*x_ref[1]
        ref[0,1,2,0]=(-1.02838589722)*x_ref[0]+(-0.280409246204)*x_ref[1]
        ref[0,1,2,1]=(1.2333373468)*x_ref[0]+(0.337631183798)*x_ref[1]
        ref[0,2,0,0]=(-0.249065448772)*x_ref[0]+(0.834483646317)*x_ref[1]
        ref[0,2,0,1]=(-1.30860297683)*x_ref[0]+(-0.653929967017)*x_ref[1]
        ref[0,2,1,0]=(-0.0155994484453)*x_ref[0]+(-0.274023166937)*x_ref[1]
        ref[0,2,1,1]=(0.31458360335)*x_ref[0]+(1.59507700673)*x_ref[1]
        ref[0,2,2,0]=(-0.11442619553)*x_ref[0]+(1.6577028576)*x_ref[1]
        ref[0,2,2,1]=(0.274261132032)*x_ref[0]+(1.15236962479)*x_ref[1]
        ref[0,3,0,0]=(1.91433860239)*x_ref[0]+(-0.316349747811)*x_ref[1]
        ref[0,3,0,1]=(-1.42457591054)*x_ref[0]+(-0.362812690805)*x_ref[1]
        ref[0,3,1,0]=(0.572734883096)*x_ref[0]+(0.424724787283)*x_ref[1]
        ref[0,3,1,1]=(-0.118538376813)*x_ref[0]+(0.25190312527)*x_ref[1]
        ref[0,3,2,0]=(-0.533409510081)*x_ref[0]+(-0.804349107915)*x_ref[1]
        ref[0,3,2,1]=(-0.290555363666)*x_ref[0]+(-0.912641477145)*x_ref[1]
        ref[0,4,0,0]=(-0.469612775918)*x_ref[0]+(-0.0447411310793)*x_ref[1]
        ref[0,4,0,1]=(0.108538258831)*x_ref[0]+(0.0453962016662)*x_ref[1]
        ref[0,4,1,0]=(-0.130785944028)*x_ref[0]+(0.0887280914032)*x_ref[1]
        ref[0,4,1,1]=(0.166412309585)*x_ref[0]+(-0.75130797706)*x_ref[1]
        ref[0,4,2,0]=(1.22538232455)*x_ref[0]+(0.112925588343)*x_ref[1]
        ref[0,4,2,1]=(-1.02455171854)*x_ref[0]+(-0.725754942749)*x_ref[1]
        ref[1,0,0,0]=(-0.869171086952)*x_ref[0]+(-0.948876783682)*x_ref[1]
        ref[1,0,0,1]=(0.17730673689)*x_ref[0]+(-0.00166180855579)*x_ref[1]
        ref[1,0,1,0]=(0.482072704433)*x_ref[0]+(0.895847096783)*x_ref[1]
        ref[1,0,1,1]=(0.695736834617)*x_ref[0]+(0.358082704558)*x_ref[1]
        ref[1,0,2,0]=(-0.501799636565)*x_ref[0]+(0.363612558025)*x_ref[1]
        ref[1,0,2,1]=(-1.47204447226)*x_ref[0]+(1.91281426121)*x_ref[1]
        ref[1,1,0,0]=(0.241893182978)*x_ref[0]+(0.95199758669)*x_ref[1]
        ref[1,1,0,1]=(-0.68112361827)*x_ref[0]+(-0.241935141301)*x_ref[1]
        ref[1,1,1,0]=(-0.553030972306)*x_ref[0]+(0.0244503690439)*x_ref[1]
        ref[1,1,1,1]=(0.278177176896)*x_ref[0]+(1.58693846427)*x_ref[1]
        ref[1,1,2,0]=(-0.87980015517)*x_ref[0]+(0.315618837123)*x_ref[1]
        ref[1,1,2,1]=(-0.335783961379)*x_ref[0]+(0.368650136138)*x_ref[1]
        ref[1,2,0,0]=(-0.903535146802)*x_ref[0]+(1.68182923194)*x_ref[1]
        ref[1,2,0,1]=(1.06511511382)*x_ref[0]+(-1.08524762302)*x_ref[1]
        ref[1,2,1,0]=(-0.0455274784681)*x_ref[0]+(-0.519576710661)*x_ref[1]
        ref[1,2,1,1]=(0.735100217062)*x_ref[0]+(-0.986268682671)*x_ref[1]
        ref[1,2,2,0]=(-0.544610797462)*x_ref[0]+(0.447237114882)*x_ref[1]
        ref[1,2,2,1]=(0.385242031153)*x_ref[0]+(-0.960538844247)*x_ref[1]
        ref[1,3,0,0]=(-0.354927072897)*x_ref[0]+(0.842574324062)*x_ref[1]
        ref[1,3,0,1]=(0.319036427919)*x_ref[0]+(-0.156282262536)*x_ref[1]
        ref[1,3,1,0]=(0.587123807717)*x_ref[0]+(1.65277990726)*x_ref[1]
        ref[1,3,1,1]=(0.791697370298)*x_ref[0]+(-1.5681680344)*x_ref[1]
        ref[1,3,2,0]=(-1.41931242045)*x_ref[0]+(0.351285136557)*x_ref[1]
        ref[1,3,2,1]=(0.53175704402)*x_ref[0]+(0.987523241324)*x_ref[1]
        ref[1,4,0,0]=(0.00533504818465)*x_ref[0]+(-0.158886930817)*x_ref[1]
        ref[1,4,0,1]=(1.23725297303)*x_ref[0]+(-0.302896452172)*x_ref[1]
        ref[1,4,1,0]=(0.433953146329)*x_ref[0]+(-0.0721602234887)*x_ref[1]
        ref[1,4,1,1]=(-0.383499726865)*x_ref[0]+(-0.253834671235)*x_ref[1]
        ref[1,4,2,0]=(-0.480776695247)*x_ref[0]+(-0.302145524759)*x_ref[1]
        ref[1,4,2,1]=(1.14113180903)*x_ref[0]+(-1.24304726003)*x_ref[1]
        ref[2,0,0,0]=(0.513776013893)*x_ref[0]+(-0.16660259885)*x_ref[1]
        ref[2,0,0,1]=(1.55739209698)*x_ref[0]+(0.0524084654531)*x_ref[1]
        ref[2,0,1,0]=(0.222567881446)*x_ref[0]+(-0.357158470388)*x_ref[1]
        ref[2,0,1,1]=(0.0179816596229)*x_ref[0]+(0.231534460784)*x_ref[1]
        ref[2,0,2,0]=(1.58460345297)*x_ref[0]+(-0.383588433888)*x_ref[1]
        ref[2,0,2,1]=(-1.73386465536)*x_ref[0]+(-0.743326657544)*x_ref[1]
        ref[2,1,0,0]=(-0.737976950246)*x_ref[0]+(-1.01053396224)*x_ref[1]
        ref[2,1,0,1]=(0.926911190393)*x_ref[0]+(0.620251783925)*x_ref[1]
        ref[2,1,1,0]=(-1.34289630129)*x_ref[0]+(-0.429933681523)*x_ref[1]
        ref[2,1,1,1]=(1.79530806415)*x_ref[0]+(-0.539783957021)*x_ref[1]
        ref[2,1,2,0]=(0.237950974938)*x_ref[0]+(1.93783427703)*x_ref[1]
        ref[2,1,2,1]=(-0.439640741973)*x_ref[0]+(-0.61543984673)*x_ref[1]
        ref[2,2,0,0]=(-1.66403612438)*x_ref[0]+(1.00839566441)*x_ref[1]
        ref[2,2,0,1]=(0.249635983351)*x_ref[0]+(-0.722261363675)*x_ref[1]
        ref[2,2,1,0]=(1.34179674459)*x_ref[0]+(0.502542144188)*x_ref[1]
        ref[2,2,1,1]=(-0.815862586074)*x_ref[0]+(-0.110688273707)*x_ref[1]
        ref[2,2,2,0]=(-0.686810719526)*x_ref[0]+(0.500406563421)*x_ref[1]
        ref[2,2,2,1]=(0.681231089598)*x_ref[0]+(0.605219505613)*x_ref[1]
        ref[2,3,0,0]=(-0.953299029956)*x_ref[0]+(-0.223725858372)*x_ref[1]
        ref[2,3,0,1]=(0.831380756684)*x_ref[0]+(-0.0379777076335)*x_ref[1]
        ref[2,3,1,0]=(-1.63804393157)*x_ref[0]+(0.148235731301)*x_ref[1]
        ref[2,3,1,1]=(-0.17786458643)*x_ref[0]+(-1.77342152699)*x_ref[1]
        ref[2,3,2,0]=(0.0210446244192)*x_ref[0]+(1.59778781524)*x_ref[1]
        ref[2,3,2,1]=(0.877162721214)*x_ref[0]+(0.262990448614)*x_ref[1]
        ref[2,4,0,0]=(-0.650265970331)*x_ref[0]+(-1.07003604369)*x_ref[1]
        ref[2,4,0,1]=(1.12940101029)*x_ref[0]+(0.860589290891)*x_ref[1]
        ref[2,4,1,0]=(0.520056210056)*x_ref[0]+(-1.47691193979)*x_ref[1]
        ref[2,4,1,1]=(1.26823535235)*x_ref[0]+(0.496811720647)*x_ref[1]
        ref[2,4,2,0]=(-1.388913443)*x_ref[0]+(0.585343197943)*x_ref[1]
        ref[2,4,2,1]=(-0.762904547594)*x_ref[0]+(0.503438815917)*x_ref[1]
        ref[3,0,0,0]=(0.382862686993)*x_ref[0]+(-0.923525513303)*x_ref[1]
        ref[3,0,0,1]=(-0.54305794485)*x_ref[0]+(0.467513531444)*x_ref[1]
        ref[3,0,1,0]=(1.24228759776)*x_ref[0]+(-1.65740117421)*x_ref[1]
        ref[3,0,1,1]=(-0.463045362742)*x_ref[0]+(0.186438587087)*x_ref[1]
        ref[3,0,2,0]=(0.296881980828)*x_ref[0]+(-0.385878814949)*x_ref[1]
        ref[3,0,2,1]=(-0.299705899564)*x_ref[0]+(0.490736038538)*x_ref[1]
        ref[3,1,0,0]=(-0.786851428658)*x_ref[0]+(-0.760593794308)*x_ref[1]
        ref[3,1,0,1]=(0.506851336483)*x_ref[0]+(0.995163951197)*x_ref[1]
        ref[3,1,1,0]=(-0.678883937341)*x_ref[0]+(0.56996778248)*x_ref[1]
        ref[3,1,1,1]=(-0.401483642528)*x_ref[0]+(0.0476797332537)*x_ref[1]
        ref[3,1,2,0]=(-0.585357955375)*x_ref[0]+(0.330103647539)*x_ref[1]
        ref[3,1,2,1]=(-0.355539976825)*x_ref[0]+(0.416287924349)*x_ref[1]
        ref[3,2,0,0]=(-0.662837130286)*x_ref[0]+(-0.26640899674)*x_ref[1]
        ref[3,2,0,1]=(0.397553900588)*x_ref[0]+(-1.21708620294)*x_ref[1]
        ref[3,2,1,0]=(0.464963326429)*x_ref[0]+(-0.774515114142)*x_ref[1]
        ref[3,2,1,1]=(0.834351469791)*x_ref[0]+(0.487629540715)*x_ref[1]
        ref[3,2,2,0]=(0.27622693716)*x_ref[0]+(0.811338337719)*x_ref[1]
        ref[3,2,2,1]=(0.0384997772653)*x_ref[0]+(1.03724996578)*x_ref[1]
        ref[3,3,0,0]=(-0.805491987874)*x_ref[0]+(-0.476384547853)*x_ref[1]
        ref[3,3,0,1]=(0.615551780195)*x_ref[0]+(-0.312805967691)*x_ref[1]
        ref[3,3,1,0]=(0.348666798955)*x_ref[0]+(-1.31271156524)*x_ref[1]
        ref[3,3,1,1]=(-0.423569975374)*x_ref[0]+(0.930713125389)*x_ref[1]
        ref[3,3,2,0]=(0.325632686234)*x_ref[0]+(-0.166401033259)*x_ref[1]
        ref[3,3,2,1]=(0.126924861797)*x_ref[0]+(-0.425380897969)*x_ref[1]
        ref[3,4,0,0]=(0.169532204538)*x_ref[0]+(0.614632525765)*x_ref[1]
        ref[3,4,0,1]=(-0.938098696851)*x_ref[0]+(0.648854147587)*x_ref[1]
        ref[3,4,1,0]=(-0.545143970434)*x_ref[0]+(-1.59787515966)*x_ref[1]
        ref[3,4,1,1]=(-0.711190914978)*x_ref[0]+(0.897331522911)*x_ref[1]
        ref[3,4,2,0]=(-0.386498124525)*x_ref[0]+(0.299740420711)*x_ref[1]
        ref[3,4,2,1]=(0.617683032062)*x_ref[0]+(-0.449587703061)*x_ref[1]
      else:
        arg[0,0,0,0]=(-0.791620170192)*x[0]+(1.03871460301)*x[1]+(-0.225035657687)*x[2]
        arg[0,0,0,1]=(-0.464422319203)*x[0]+(0.769454931053)*x[1]+(-0.783260637771)*x[2]
        arg[0,0,1,0]=(0.72655538215)*x[0]+(-1.65736312015)*x[1]+(1.08402505927)*x[2]
        arg[0,0,1,1]=(-0.207576709573)*x[0]+(-0.55250594761)*x[1]+(-0.923702938288)*x[2]
        arg[0,0,2,0]=(-0.701991085516)*x[0]+(-1.76863643107)*x[1]+(0.831382933104)*x[2]
        arg[0,0,2,1]=(0.0640035527319)*x[0]+(0.425446342976)*x[1]+(1.64626766696)*x[2]
        arg[0,1,0,0]=(1.01022963801)*x[0]+(-0.0215015059716)*x[1]+(0.849862425592)*x[2]
        arg[0,1,0,1]=(-0.45776932943)*x[0]+(0.7621342151)*x[1]+(-1.40123018391)*x[2]
        arg[0,1,1,0]=(0.737485054127)*x[0]+(-0.147513905858)*x[1]+(-0.616928595213)*x[2]
        arg[0,1,1,1]=(-0.0608005391894)*x[0]+(-0.154978210573)*x[1]+(-1.40322011577)*x[2]
        arg[0,1,2,0]=(1.59908177714)*x[0]+(-0.378738573932)*x[1]+(1.45285315247)*x[2]
        arg[0,1,2,1]=(-0.622552157046)*x[0]+(1.19996417516)*x[1]+(0.728842900843)*x[2]
        arg[0,2,0,0]=(-0.46942034691)*x[0]+(0.333309721372)*x[1]+(-0.280846553832)*x[2]
        arg[0,2,0,1]=(1.22208788109)*x[0]+(0.369339594285)*x[1]+(1.31078425589)*x[2]
        arg[0,2,1,0]=(0.0794165004114)*x[0]+(0.44665451846)*x[1]+(-0.814625708916)*x[2]
        arg[0,2,1,1]=(0.594200807165)*x[0]+(-0.00616720333193)*x[1]+(0.382706175172)*x[2]
        arg[0,2,2,0]=(0.0372230170459)*x[0]+(0.440339923291)*x[1]+(0.137641070028)*x[2]
        arg[0,2,2,1]=(-1.29863078274)*x[0]+(0.283142455171)*x[1]+(0.0844540074659)*x[2]
        arg[0,3,0,0]=(1.72811808036)*x[0]+(0.536456781522)*x[1]+(0.0266737704706)*x[2]
        arg[0,3,0,1]=(0.148219555101)*x[0]+(-0.6887500275)*x[1]+(-0.405613136632)*x[2]
        arg[0,3,1,0]=(0.708262537279)*x[0]+(1.16319635773)*x[1]+(-0.030808721674)*x[2]
        arg[0,3,1,1]=(0.80834615863)*x[0]+(-0.176789000925)*x[1]+(-1.72576120963)*x[2]
        arg[0,3,2,0]=(0.21128315299)*x[0]+(0.770037046126)*x[1]+(0.988028934422)*x[2]
        arg[0,3,2,1]=(1.10522136317)*x[0]+(1.6319351743)*x[1]+(-0.784693713387)*x[2]
        arg[0,4,0,0]=(0.121088121064)*x[0]+(-1.79718067791)*x[1]+(-0.907857173318)*x[2]
        arg[0,4,0,1]=(-0.695887294217)*x[0]+(-1.24166787807)*x[1]+(-1.24968023856)*x[2]
        arg[0,4,1,0]=(-0.202510797291)*x[0]+(0.628310156105)*x[1]+(0.605146447822)*x[2]
        arg[0,4,1,1]=(0.149227458424)*x[0]+(-1.5650392775)*x[1]+(-0.763944892663)*x[2]
        arg[0,4,2,0]=(-0.393218625872)*x[0]+(0.479145914948)*x[1]+(-0.313960447492)*x[2]
        arg[0,4,2,1]=(-0.0191807443799)*x[0]+(0.592992659119)*x[1]+(-0.475633115484)*x[2]
        arg[1,0,0,0]=(-0.964829354261)*x[0]+(0.453014802906)*x[1]+(-0.779502566938)*x[2]
        arg[1,0,0,1]=(0.0151205739508)*x[0]+(-0.153706429594)*x[1]+(0.00866460475724)*x[2]
        arg[1,0,1,0]=(0.92592276366)*x[0]+(0.530787400525)*x[1]+(-0.0577912606876)*x[2]
        arg[1,0,1,1]=(0.948404467077)*x[0]+(-0.265905470751)*x[1]+(0.472349192005)*x[2]
        arg[1,0,2,0]=(0.0176615383702)*x[0]+(0.679605821761)*x[1]+(-0.0333590911941)*x[2]
        arg[1,0,2,1]=(-1.15594952629)*x[0]+(0.264868376508)*x[1]+(-0.883690535499)*x[2]
        arg[1,1,0,0]=(-0.738712703994)*x[0]+(0.545839232808)*x[1]+(0.592474501134)*x[2]
        arg[1,1,0,1]=(-1.3468960453)*x[0]+(-0.00848202443614)*x[1]+(-0.0124138286126)*x[2]
        arg[1,1,1,0]=(0.900228409707)*x[0]+(-1.86848082357)*x[1]+(0.421539431142)*x[2]
        arg[1,1,1,1]=(1.42754013963)*x[0]+(-0.861376440449)*x[1]+(0.0965520362291)*x[2]
        arg[1,1,2,0]=(-0.35873674515)*x[0]+(0.0862592617941)*x[1]+(-0.536106207025)*x[2]
        arg[1,1,2,1]=(0.90712910097)*x[0]+(0.974545253055)*x[1]+(0.570708510858)*x[2]
        arg[1,2,0,0]=(1.19310775669)*x[0]+(0.309414331679)*x[1]+(-0.271298061586)*x[2]
        arg[1,2,0,1]=(0.551778129545)*x[0]+(0.539028430134)*x[1]+(0.239406239262)*x[2]
        arg[1,2,1,0]=(-0.810625054344)*x[0]+(-0.241072190233)*x[1]+(-1.58590369097)*x[2]
        arg[1,2,1,1]=(0.529669075734)*x[0]+(-0.890365024163)*x[1]+(-1.69798509084)*x[2]
        arg[1,2,2,0]=(0.0437087707448)*x[0]+(0.88372190198)*x[1]+(1.86547747806)*x[2]
        arg[1,2,2,1]=(1.30739438898)*x[0]+(0.315850112101)*x[1]+(-0.262001133683)*x[2]
        arg[1,3,0,0]=(-0.143426236837)*x[0]+(-1.1703384826)*x[1]+(0.50608348646)*x[2]
        arg[1,3,0,1]=(-0.0444408776922)*x[0]+(1.26952782036)*x[1]+(1.67192602934)*x[2]
        arg[1,3,1,0]=(0.672742824368)*x[0]+(-0.785246676537)*x[1]+(0.716130664056)*x[2]
        arg[1,3,1,1]=(-0.295182508335)*x[0]+(-1.53658860897)*x[1]+(-1.87133995422)*x[2]
        arg[1,3,2,0]=(-0.440859530081)*x[0]+(0.0730120015821)*x[1]+(-1.02465537353)*x[2]
        arg[1,3,2,1]=(0.838320612003)*x[0]+(1.81233688454)*x[1]+(-1.0633876685)*x[2]
        arg[1,4,0,0]=(0.52440453051)*x[0]+(-0.0182588999022)*x[1]+(1.05556310351)*x[2]
        arg[1,4,0,1]=(0.766612727495)*x[0]+(1.58987590746)*x[1]+(-0.703801484569)*x[2]
        arg[1,4,1,0]=(0.373439757535)*x[0]+(-0.0676963704305)*x[1]+(0.0948034094293)*x[2]
        arg[1,4,1,1]=(-0.542283543375)*x[0]+(-0.454408605761)*x[1]+(-0.398904635316)*x[2]
        arg[1,4,2,0]=(1.3374871143)*x[0]+(0.0067849394587)*x[1]+(-0.319452094882)*x[2]
        arg[1,4,2,1]=(0.38530755393)*x[0]+(1.05356462429)*x[1]+(-0.302179447719)*x[2]
        arg[2,0,0,0]=(-0.123109761583)*x[0]+(0.69223848143)*x[1]+(-1.36903487092)*x[2]
        arg[2,0,0,1]=(0.121957328459)*x[0]+(0.102229685387)*x[1]+(0.962499974737)*x[2]
        arg[2,0,1,0]=(0.478614188653)*x[0]+(-1.23058817023)*x[1]+(1.30950997478)*x[2]
        arg[2,0,1,1]=(-0.529033562461)*x[0]+(0.563030509591)*x[1]+(-0.564939226654)*x[2]
        arg[2,0,2,0]=(-0.377740282406)*x[0]+(1.20532110343)*x[1]+(1.55615174947)*x[2]
        arg[2,0,2,1]=(0.539119710703)*x[0]+(1.91771865857)*x[1]+(0.0809882488048)*x[2]
        arg[2,1,0,0]=(0.00548664737796)*x[0]+(-0.157716099068)*x[1]+(-1.40009044235)*x[2]
        arg[2,1,0,1]=(0.0667557890666)*x[0]+(0.742484448131)*x[1]+(-0.325246111346)*x[2]
        arg[2,1,1,0]=(-0.214965474071)*x[0]+(-0.215055151964)*x[1]+(1.15673960537)*x[2]
        arg[2,1,1,1]=(0.605764831789)*x[0]+(-1.0432964832)*x[1]+(0.645482523161)*x[2]
        arg[2,1,2,0]=(-1.13098364088)*x[0]+(-0.932044089041)*x[1]+(-0.308053828241)*x[2]
        arg[2,1,2,1]=(0.481961305459)*x[0]+(-0.101277472029)*x[1]+(-0.17520207974)*x[2]
        arg[2,2,0,0]=(0.579831852303)*x[0]+(0.538731990254)*x[1]+(-0.0897460374205)*x[2]
        arg[2,2,0,1]=(-1.8763181434)*x[0]+(-0.132710928569)*x[1]+(1.03169751651)*x[2]
        arg[2,2,1,0]=(0.56871511431)*x[0]+(0.075333481656)*x[1]+(1.10238818953)*x[2]
        arg[2,2,1,1]=(1.49347755246)*x[0]+(-1.00500941047)*x[1]+(-0.181733196359)*x[2]
        arg[2,2,2,0]=(0.532881509971)*x[0]+(-0.364333964659)*x[1]+(1.09633720883)*x[2]
        arg[2,2,2,1]=(-0.518419048615)*x[0]+(-0.48091100593)*x[1]+(0.724628840153)*x[2]
        arg[2,3,0,0]=(0.353475859331)*x[0]+(-0.0239879269825)*x[1]+(1.32068893718)*x[2]
        arg[2,3,0,1]=(0.553999122102)*x[0]+(-0.128530869239)*x[1]+(0.134663336287)*x[2]
        arg[2,3,1,0]=(-1.59775882569)*x[0]+(-1.07335424785)*x[1]+(1.39223587988)*x[2]
        arg[2,3,1,1]=(-0.921252080095)*x[0]+(1.46226299067)*x[1]+(-1.13412499932)*x[2]
        arg[2,3,2,0]=(0.417572910178)*x[0]+(0.0814889599489)*x[1]+(0.292159249719)*x[2]
        arg[2,3,2,1]=(1.44620228921)*x[0]+(0.767854577367)*x[1]+(-0.0113709806383)*x[2]
        arg[2,4,0,0]=(-0.156987001681)*x[0]+(-0.922524257833)*x[1]+(0.521128360838)*x[2]
        arg[2,4,0,1]=(0.154198133354)*x[0]+(0.462020370175)*x[1]+(0.966768653665)*x[2]
        arg[2,4,1,0]=(0.507836849242)*x[0]+(1.16701911244)*x[1]+(0.264543038211)*x[2]
        arg[2,4,1,1]=(0.452059436879)*x[0]+(0.409662432778)*x[1]+(-0.403800872595)*x[2]
        arg[2,4,2,0]=(0.521175521008)*x[0]+(-0.362551578816)*x[1]+(0.0922727331623)*x[2]
        arg[2,4,2,1]=(-1.0990361823)*x[0]+(1.73655146851)*x[1]+(-0.867917402922)*x[2]
        arg[3,0,0,0]=(-0.0527010882236)*x[0]+(-1.11528215488)*x[1]+(1.3141906393)*x[2]
        arg[3,0,0,1]=(0.151284271917)*x[0]+(-0.0159077450668)*x[1]+(1.1798338931)*x[2]
        arg[3,0,1,0]=(-1.7648963435)*x[0]+(0.229511437402)*x[1]+(0.924415900312)*x[2]
        arg[3,0,1,1]=(-0.498271821094)*x[0]+(1.58363063019)*x[1]+(1.64871750388)*x[2]
        arg[3,0,2,0]=(-1.11245170725)*x[0]+(1.27662612569)*x[1]+(0.145950372369)*x[2]
        arg[3,0,2,1]=(-0.691643924238)*x[0]+(-0.178211874279)*x[1]+(-0.341071050366)*x[2]
        arg[3,1,0,0]=(-0.929804889461)*x[0]+(0.199087067309)*x[1]+(1.00150168883)*x[2]
        arg[3,1,0,1]=(-1.25152265093)*x[0]+(0.517076294911)*x[1]+(0.103798312362)*x[2]
        arg[3,1,1,0]=(0.797316290623)*x[0]+(0.0239285379293)*x[1]+(-0.836519299418)*x[2]
        arg[3,1,1,1]=(-0.470084592176)*x[0]+(0.844364701728)*x[1]+(-0.027373275093)*x[2]
        arg[3,1,2,0]=(1.0340279373)*x[0]+(-0.0133951276315)*x[1]+(1.17710546194)*x[2]
        arg[3,1,2,1]=(0.0325929541496)*x[0]+(1.13784867184)*x[1]+(-0.0529194266769)*x[2]
        arg[3,2,0,0]=(1.92837561772)*x[0]+(-1.63633468831)*x[1]+(1.4212324281)*x[2]
        arg[3,2,0,1]=(1.09556054408)*x[0]+(1.43286059545)*x[1]+(1.20005604593)*x[2]
        arg[3,2,1,0]=(0.308887169169)*x[0]+(-0.149214224208)*x[1]+(-1.13559614877)*x[2]
        arg[3,2,1,1]=(-0.0648352909004)*x[0]+(-0.5645470596)*x[1]+(-0.525131730514)*x[2]
        arg[3,2,2,0]=(0.21826829974)*x[0]+(1.5950239996)*x[1]+(0.333592425438)*x[2]
        arg[3,2,2,1]=(-1.25613956172)*x[0]+(-0.242338679844)*x[1]+(-0.958875577074)*x[2]
        arg[3,3,0,0]=(1.65005362159)*x[0]+(-1.16713867165)*x[1]+(-0.282495293685)*x[2]
        arg[3,3,0,1]=(1.04854406427)*x[0]+(0.876270949531)*x[1]+(0.291036304743)*x[2]
        arg[3,3,1,0]=(-1.66619451175)*x[0]+(0.656275192484)*x[1]+(-1.27104499942)*x[2]
        arg[3,3,1,1]=(1.55407146907)*x[0]+(-0.322950943159)*x[1]+(-0.287565403896)*x[2]
        arg[3,3,2,0]=(-1.54760268583)*x[0]+(-1.45847761887)*x[1]+(1.13939567009)*x[2]
        arg[3,3,2,1]=(1.24632351699)*x[0]+(0.0265500081594)*x[1]+(1.61925095159)*x[2]
        arg[3,4,0,0]=(-0.0430889433549)*x[0]+(-1.46995559463)*x[1]+(-1.14915159823)*x[2]
        arg[3,4,0,1]=(0.0392641403409)*x[0]+(0.0603053049639)*x[1]+(0.647321169934)*x[2]
        arg[3,4,1,0]=(-0.156300985227)*x[0]+(0.79369911928)*x[1]+(-0.413124762507)*x[2]
        arg[3,4,1,1]=(0.258402059015)*x[0]+(0.00227307988585)*x[1]+(0.70067141295)*x[2]
        arg[3,4,2,0]=(-1.15131298872)*x[0]+(-0.627449043157)*x[1]+(-0.402268345861)*x[2]
        arg[3,4,2,1]=(0.455934238179)*x[0]+(-0.951120759108)*x[1]+(-0.280459367131)*x[2]
        ref[0,0,0,0]=(-0.791620170192)*x_ref[0]+(1.03871460301)*x_ref[1]+(-0.225035657687)*x_ref[2]
        ref[0,0,0,1]=(-0.464422319203)*x_ref[0]+(0.769454931053)*x_ref[1]+(-0.783260637771)*x_ref[2]
        ref[0,0,1,0]=(0.72655538215)*x_ref[0]+(-1.65736312015)*x_ref[1]+(1.08402505927)*x_ref[2]
        ref[0,0,1,1]=(-0.207576709573)*x_ref[0]+(-0.55250594761)*x_ref[1]+(-0.923702938288)*x_ref[2]
        ref[0,0,2,0]=(-0.701991085516)*x_ref[0]+(-1.76863643107)*x_ref[1]+(0.831382933104)*x_ref[2]
        ref[0,0,2,1]=(0.0640035527319)*x_ref[0]+(0.425446342976)*x_ref[1]+(1.64626766696)*x_ref[2]
        ref[0,1,0,0]=(1.01022963801)*x_ref[0]+(-0.0215015059716)*x_ref[1]+(0.849862425592)*x_ref[2]
        ref[0,1,0,1]=(-0.45776932943)*x_ref[0]+(0.7621342151)*x_ref[1]+(-1.40123018391)*x_ref[2]
        ref[0,1,1,0]=(0.737485054127)*x_ref[0]+(-0.147513905858)*x_ref[1]+(-0.616928595213)*x_ref[2]
        ref[0,1,1,1]=(-0.0608005391894)*x_ref[0]+(-0.154978210573)*x_ref[1]+(-1.40322011577)*x_ref[2]
        ref[0,1,2,0]=(1.59908177714)*x_ref[0]+(-0.378738573932)*x_ref[1]+(1.45285315247)*x_ref[2]
        ref[0,1,2,1]=(-0.622552157046)*x_ref[0]+(1.19996417516)*x_ref[1]+(0.728842900843)*x_ref[2]
        ref[0,2,0,0]=(-0.46942034691)*x_ref[0]+(0.333309721372)*x_ref[1]+(-0.280846553832)*x_ref[2]
        ref[0,2,0,1]=(1.22208788109)*x_ref[0]+(0.369339594285)*x_ref[1]+(1.31078425589)*x_ref[2]
        ref[0,2,1,0]=(0.0794165004114)*x_ref[0]+(0.44665451846)*x_ref[1]+(-0.814625708916)*x_ref[2]
        ref[0,2,1,1]=(0.594200807165)*x_ref[0]+(-0.00616720333193)*x_ref[1]+(0.382706175172)*x_ref[2]
        ref[0,2,2,0]=(0.0372230170459)*x_ref[0]+(0.440339923291)*x_ref[1]+(0.137641070028)*x_ref[2]
        ref[0,2,2,1]=(-1.29863078274)*x_ref[0]+(0.283142455171)*x_ref[1]+(0.0844540074659)*x_ref[2]
        ref[0,3,0,0]=(1.72811808036)*x_ref[0]+(0.536456781522)*x_ref[1]+(0.0266737704706)*x_ref[2]
        ref[0,3,0,1]=(0.148219555101)*x_ref[0]+(-0.6887500275)*x_ref[1]+(-0.405613136632)*x_ref[2]
        ref[0,3,1,0]=(0.708262537279)*x_ref[0]+(1.16319635773)*x_ref[1]+(-0.030808721674)*x_ref[2]
        ref[0,3,1,1]=(0.80834615863)*x_ref[0]+(-0.176789000925)*x_ref[1]+(-1.72576120963)*x_ref[2]
        ref[0,3,2,0]=(0.21128315299)*x_ref[0]+(0.770037046126)*x_ref[1]+(0.988028934422)*x_ref[2]
        ref[0,3,2,1]=(1.10522136317)*x_ref[0]+(1.6319351743)*x_ref[1]+(-0.784693713387)*x_ref[2]
        ref[0,4,0,0]=(0.121088121064)*x_ref[0]+(-1.79718067791)*x_ref[1]+(-0.907857173318)*x_ref[2]
        ref[0,4,0,1]=(-0.695887294217)*x_ref[0]+(-1.24166787807)*x_ref[1]+(-1.24968023856)*x_ref[2]
        ref[0,4,1,0]=(-0.202510797291)*x_ref[0]+(0.628310156105)*x_ref[1]+(0.605146447822)*x_ref[2]
        ref[0,4,1,1]=(0.149227458424)*x_ref[0]+(-1.5650392775)*x_ref[1]+(-0.763944892663)*x_ref[2]
        ref[0,4,2,0]=(-0.393218625872)*x_ref[0]+(0.479145914948)*x_ref[1]+(-0.313960447492)*x_ref[2]
        ref[0,4,2,1]=(-0.0191807443799)*x_ref[0]+(0.592992659119)*x_ref[1]+(-0.475633115484)*x_ref[2]
        ref[1,0,0,0]=(-0.964829354261)*x_ref[0]+(0.453014802906)*x_ref[1]+(-0.779502566938)*x_ref[2]
        ref[1,0,0,1]=(0.0151205739508)*x_ref[0]+(-0.153706429594)*x_ref[1]+(0.00866460475724)*x_ref[2]
        ref[1,0,1,0]=(0.92592276366)*x_ref[0]+(0.530787400525)*x_ref[1]+(-0.0577912606876)*x_ref[2]
        ref[1,0,1,1]=(0.948404467077)*x_ref[0]+(-0.265905470751)*x_ref[1]+(0.472349192005)*x_ref[2]
        ref[1,0,2,0]=(0.0176615383702)*x_ref[0]+(0.679605821761)*x_ref[1]+(-0.0333590911941)*x_ref[2]
        ref[1,0,2,1]=(-1.15594952629)*x_ref[0]+(0.264868376508)*x_ref[1]+(-0.883690535499)*x_ref[2]
        ref[1,1,0,0]=(-0.738712703994)*x_ref[0]+(0.545839232808)*x_ref[1]+(0.592474501134)*x_ref[2]
        ref[1,1,0,1]=(-1.3468960453)*x_ref[0]+(-0.00848202443614)*x_ref[1]+(-0.0124138286126)*x_ref[2]
        ref[1,1,1,0]=(0.900228409707)*x_ref[0]+(-1.86848082357)*x_ref[1]+(0.421539431142)*x_ref[2]
        ref[1,1,1,1]=(1.42754013963)*x_ref[0]+(-0.861376440449)*x_ref[1]+(0.0965520362291)*x_ref[2]
        ref[1,1,2,0]=(-0.35873674515)*x_ref[0]+(0.0862592617941)*x_ref[1]+(-0.536106207025)*x_ref[2]
        ref[1,1,2,1]=(0.90712910097)*x_ref[0]+(0.974545253055)*x_ref[1]+(0.570708510858)*x_ref[2]
        ref[1,2,0,0]=(1.19310775669)*x_ref[0]+(0.309414331679)*x_ref[1]+(-0.271298061586)*x_ref[2]
        ref[1,2,0,1]=(0.551778129545)*x_ref[0]+(0.539028430134)*x_ref[1]+(0.239406239262)*x_ref[2]
        ref[1,2,1,0]=(-0.810625054344)*x_ref[0]+(-0.241072190233)*x_ref[1]+(-1.58590369097)*x_ref[2]
        ref[1,2,1,1]=(0.529669075734)*x_ref[0]+(-0.890365024163)*x_ref[1]+(-1.69798509084)*x_ref[2]
        ref[1,2,2,0]=(0.0437087707448)*x_ref[0]+(0.88372190198)*x_ref[1]+(1.86547747806)*x_ref[2]
        ref[1,2,2,1]=(1.30739438898)*x_ref[0]+(0.315850112101)*x_ref[1]+(-0.262001133683)*x_ref[2]
        ref[1,3,0,0]=(-0.143426236837)*x_ref[0]+(-1.1703384826)*x_ref[1]+(0.50608348646)*x_ref[2]
        ref[1,3,0,1]=(-0.0444408776922)*x_ref[0]+(1.26952782036)*x_ref[1]+(1.67192602934)*x_ref[2]
        ref[1,3,1,0]=(0.672742824368)*x_ref[0]+(-0.785246676537)*x_ref[1]+(0.716130664056)*x_ref[2]
        ref[1,3,1,1]=(-0.295182508335)*x_ref[0]+(-1.53658860897)*x_ref[1]+(-1.87133995422)*x_ref[2]
        ref[1,3,2,0]=(-0.440859530081)*x_ref[0]+(0.0730120015821)*x_ref[1]+(-1.02465537353)*x_ref[2]
        ref[1,3,2,1]=(0.838320612003)*x_ref[0]+(1.81233688454)*x_ref[1]+(-1.0633876685)*x_ref[2]
        ref[1,4,0,0]=(0.52440453051)*x_ref[0]+(-0.0182588999022)*x_ref[1]+(1.05556310351)*x_ref[2]
        ref[1,4,0,1]=(0.766612727495)*x_ref[0]+(1.58987590746)*x_ref[1]+(-0.703801484569)*x_ref[2]
        ref[1,4,1,0]=(0.373439757535)*x_ref[0]+(-0.0676963704305)*x_ref[1]+(0.0948034094293)*x_ref[2]
        ref[1,4,1,1]=(-0.542283543375)*x_ref[0]+(-0.454408605761)*x_ref[1]+(-0.398904635316)*x_ref[2]
        ref[1,4,2,0]=(1.3374871143)*x_ref[0]+(0.0067849394587)*x_ref[1]+(-0.319452094882)*x_ref[2]
        ref[1,4,2,1]=(0.38530755393)*x_ref[0]+(1.05356462429)*x_ref[1]+(-0.302179447719)*x_ref[2]
        ref[2,0,0,0]=(-0.123109761583)*x_ref[0]+(0.69223848143)*x_ref[1]+(-1.36903487092)*x_ref[2]
        ref[2,0,0,1]=(0.121957328459)*x_ref[0]+(0.102229685387)*x_ref[1]+(0.962499974737)*x_ref[2]
        ref[2,0,1,0]=(0.478614188653)*x_ref[0]+(-1.23058817023)*x_ref[1]+(1.30950997478)*x_ref[2]
        ref[2,0,1,1]=(-0.529033562461)*x_ref[0]+(0.563030509591)*x_ref[1]+(-0.564939226654)*x_ref[2]
        ref[2,0,2,0]=(-0.377740282406)*x_ref[0]+(1.20532110343)*x_ref[1]+(1.55615174947)*x_ref[2]
        ref[2,0,2,1]=(0.539119710703)*x_ref[0]+(1.91771865857)*x_ref[1]+(0.0809882488048)*x_ref[2]
        ref[2,1,0,0]=(0.00548664737796)*x_ref[0]+(-0.157716099068)*x_ref[1]+(-1.40009044235)*x_ref[2]
        ref[2,1,0,1]=(0.0667557890666)*x_ref[0]+(0.742484448131)*x_ref[1]+(-0.325246111346)*x_ref[2]
        ref[2,1,1,0]=(-0.214965474071)*x_ref[0]+(-0.215055151964)*x_ref[1]+(1.15673960537)*x_ref[2]
        ref[2,1,1,1]=(0.605764831789)*x_ref[0]+(-1.0432964832)*x_ref[1]+(0.645482523161)*x_ref[2]
        ref[2,1,2,0]=(-1.13098364088)*x_ref[0]+(-0.932044089041)*x_ref[1]+(-0.308053828241)*x_ref[2]
        ref[2,1,2,1]=(0.481961305459)*x_ref[0]+(-0.101277472029)*x_ref[1]+(-0.17520207974)*x_ref[2]
        ref[2,2,0,0]=(0.579831852303)*x_ref[0]+(0.538731990254)*x_ref[1]+(-0.0897460374205)*x_ref[2]
        ref[2,2,0,1]=(-1.8763181434)*x_ref[0]+(-0.132710928569)*x_ref[1]+(1.03169751651)*x_ref[2]
        ref[2,2,1,0]=(0.56871511431)*x_ref[0]+(0.075333481656)*x_ref[1]+(1.10238818953)*x_ref[2]
        ref[2,2,1,1]=(1.49347755246)*x_ref[0]+(-1.00500941047)*x_ref[1]+(-0.181733196359)*x_ref[2]
        ref[2,2,2,0]=(0.532881509971)*x_ref[0]+(-0.364333964659)*x_ref[1]+(1.09633720883)*x_ref[2]
        ref[2,2,2,1]=(-0.518419048615)*x_ref[0]+(-0.48091100593)*x_ref[1]+(0.724628840153)*x_ref[2]
        ref[2,3,0,0]=(0.353475859331)*x_ref[0]+(-0.0239879269825)*x_ref[1]+(1.32068893718)*x_ref[2]
        ref[2,3,0,1]=(0.553999122102)*x_ref[0]+(-0.128530869239)*x_ref[1]+(0.134663336287)*x_ref[2]
        ref[2,3,1,0]=(-1.59775882569)*x_ref[0]+(-1.07335424785)*x_ref[1]+(1.39223587988)*x_ref[2]
        ref[2,3,1,1]=(-0.921252080095)*x_ref[0]+(1.46226299067)*x_ref[1]+(-1.13412499932)*x_ref[2]
        ref[2,3,2,0]=(0.417572910178)*x_ref[0]+(0.0814889599489)*x_ref[1]+(0.292159249719)*x_ref[2]
        ref[2,3,2,1]=(1.44620228921)*x_ref[0]+(0.767854577367)*x_ref[1]+(-0.0113709806383)*x_ref[2]
        ref[2,4,0,0]=(-0.156987001681)*x_ref[0]+(-0.922524257833)*x_ref[1]+(0.521128360838)*x_ref[2]
        ref[2,4,0,1]=(0.154198133354)*x_ref[0]+(0.462020370175)*x_ref[1]+(0.966768653665)*x_ref[2]
        ref[2,4,1,0]=(0.507836849242)*x_ref[0]+(1.16701911244)*x_ref[1]+(0.264543038211)*x_ref[2]
        ref[2,4,1,1]=(0.452059436879)*x_ref[0]+(0.409662432778)*x_ref[1]+(-0.403800872595)*x_ref[2]
        ref[2,4,2,0]=(0.521175521008)*x_ref[0]+(-0.362551578816)*x_ref[1]+(0.0922727331623)*x_ref[2]
        ref[2,4,2,1]=(-1.0990361823)*x_ref[0]+(1.73655146851)*x_ref[1]+(-0.867917402922)*x_ref[2]
        ref[3,0,0,0]=(-0.0527010882236)*x_ref[0]+(-1.11528215488)*x_ref[1]+(1.3141906393)*x_ref[2]
        ref[3,0,0,1]=(0.151284271917)*x_ref[0]+(-0.0159077450668)*x_ref[1]+(1.1798338931)*x_ref[2]
        ref[3,0,1,0]=(-1.7648963435)*x_ref[0]+(0.229511437402)*x_ref[1]+(0.924415900312)*x_ref[2]
        ref[3,0,1,1]=(-0.498271821094)*x_ref[0]+(1.58363063019)*x_ref[1]+(1.64871750388)*x_ref[2]
        ref[3,0,2,0]=(-1.11245170725)*x_ref[0]+(1.27662612569)*x_ref[1]+(0.145950372369)*x_ref[2]
        ref[3,0,2,1]=(-0.691643924238)*x_ref[0]+(-0.178211874279)*x_ref[1]+(-0.341071050366)*x_ref[2]
        ref[3,1,0,0]=(-0.929804889461)*x_ref[0]+(0.199087067309)*x_ref[1]+(1.00150168883)*x_ref[2]
        ref[3,1,0,1]=(-1.25152265093)*x_ref[0]+(0.517076294911)*x_ref[1]+(0.103798312362)*x_ref[2]
        ref[3,1,1,0]=(0.797316290623)*x_ref[0]+(0.0239285379293)*x_ref[1]+(-0.836519299418)*x_ref[2]
        ref[3,1,1,1]=(-0.470084592176)*x_ref[0]+(0.844364701728)*x_ref[1]+(-0.027373275093)*x_ref[2]
        ref[3,1,2,0]=(1.0340279373)*x_ref[0]+(-0.0133951276315)*x_ref[1]+(1.17710546194)*x_ref[2]
        ref[3,1,2,1]=(0.0325929541496)*x_ref[0]+(1.13784867184)*x_ref[1]+(-0.0529194266769)*x_ref[2]
        ref[3,2,0,0]=(1.92837561772)*x_ref[0]+(-1.63633468831)*x_ref[1]+(1.4212324281)*x_ref[2]
        ref[3,2,0,1]=(1.09556054408)*x_ref[0]+(1.43286059545)*x_ref[1]+(1.20005604593)*x_ref[2]
        ref[3,2,1,0]=(0.308887169169)*x_ref[0]+(-0.149214224208)*x_ref[1]+(-1.13559614877)*x_ref[2]
        ref[3,2,1,1]=(-0.0648352909004)*x_ref[0]+(-0.5645470596)*x_ref[1]+(-0.525131730514)*x_ref[2]
        ref[3,2,2,0]=(0.21826829974)*x_ref[0]+(1.5950239996)*x_ref[1]+(0.333592425438)*x_ref[2]
        ref[3,2,2,1]=(-1.25613956172)*x_ref[0]+(-0.242338679844)*x_ref[1]+(-0.958875577074)*x_ref[2]
        ref[3,3,0,0]=(1.65005362159)*x_ref[0]+(-1.16713867165)*x_ref[1]+(-0.282495293685)*x_ref[2]
        ref[3,3,0,1]=(1.04854406427)*x_ref[0]+(0.876270949531)*x_ref[1]+(0.291036304743)*x_ref[2]
        ref[3,3,1,0]=(-1.66619451175)*x_ref[0]+(0.656275192484)*x_ref[1]+(-1.27104499942)*x_ref[2]
        ref[3,3,1,1]=(1.55407146907)*x_ref[0]+(-0.322950943159)*x_ref[1]+(-0.287565403896)*x_ref[2]
        ref[3,3,2,0]=(-1.54760268583)*x_ref[0]+(-1.45847761887)*x_ref[1]+(1.13939567009)*x_ref[2]
        ref[3,3,2,1]=(1.24632351699)*x_ref[0]+(0.0265500081594)*x_ref[1]+(1.61925095159)*x_ref[2]
        ref[3,4,0,0]=(-0.0430889433549)*x_ref[0]+(-1.46995559463)*x_ref[1]+(-1.14915159823)*x_ref[2]
        ref[3,4,0,1]=(0.0392641403409)*x_ref[0]+(0.0603053049639)*x_ref[1]+(0.647321169934)*x_ref[2]
        ref[3,4,1,0]=(-0.156300985227)*x_ref[0]+(0.79369911928)*x_ref[1]+(-0.413124762507)*x_ref[2]
        ref[3,4,1,1]=(0.258402059015)*x_ref[0]+(0.00227307988585)*x_ref[1]+(0.70067141295)*x_ref[2]
        ref[3,4,2,0]=(-1.15131298872)*x_ref[0]+(-0.627449043157)*x_ref[1]+(-0.402268345861)*x_ref[2]
        ref[3,4,2,1]=(0.455934238179)*x_ref[0]+(-0.951120759108)*x_ref[1]+(-0.280459367131)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_ReducedFunctionOnContactZero_rank0(self):
      """
      tests interpolate for rank 0 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactZero onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactZero(self.domain)
      x=w.getX()
      arg=Data(0,(),w)
      ref=Data(0,(),w_ref)
      if dim==2:
        arg=(0.203238386406)*x[0]**o+(0.734542238717)*x[0]+(-0.627248060056)*x[1]**o+(0.347224669854)*x[1]
        ref=(0.203238386406)*x_ref[0]**o+(0.734542238717)*x_ref[0]+(-0.627248060056)*x_ref[1]**o+(0.347224669854)*x_ref[1]
      else:
        arg=(-0.387501489724)*x[0]**o+(0.752554332333)*x[0]+(-0.0831588271355)*x[1]**o+(0.615191870812)*x[1]+(-0.264702923622)*x[2]**o+(0.404594629993)*x[2]
        ref=(-0.387501489724)*x_ref[0]**o+(0.752554332333)*x_ref[0]+(-0.0831588271355)*x_ref[1]**o+(0.615191870812)*x_ref[1]+(-0.264702923622)*x_ref[2]**o+(0.404594629993)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_ReducedFunctionOnContactZero_rank1(self):
      """
      tests interpolate for rank 1 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactZero onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactZero(self.domain)
      x=w.getX()
      arg=Data(0,(2,),w)
      ref=Data(0,(2,),w_ref)
      if dim==2:
        arg[0]=(0.938444792136)*x[0]**o+(-0.809407040724)*x[0]+(-0.963148615762)*x[1]**o+(-0.257000244129)*x[1]
        arg[1]=(-0.791280640039)*x[0]**o+(0.293858623095)*x[0]+(0.964942589784)*x[1]**o+(0.522071353081)*x[1]
        ref[0]=(0.938444792136)*x_ref[0]**o+(-0.809407040724)*x_ref[0]+(-0.963148615762)*x_ref[1]**o+(-0.257000244129)*x_ref[1]
        ref[1]=(-0.791280640039)*x_ref[0]**o+(0.293858623095)*x_ref[0]+(0.964942589784)*x_ref[1]**o+(0.522071353081)*x_ref[1]
      else:
        arg[0]=(0.566110580666)*x[0]**o+(-0.89723913213)*x[0]+(-0.54781629706)*x[1]**o+(-0.0405507445628)*x[1]+(0.435477036501)*x[2]**o+(-0.296404435183)*x[2]
        arg[1]=(-0.436126359475)*x[0]**o+(0.863706244241)*x[0]+(-0.149818409243)*x[1]**o+(-0.654801920098)*x[1]+(-0.281005092809)*x[2]**o+(-0.0257179420057)*x[2]
        ref[0]=(0.566110580666)*x_ref[0]**o+(-0.89723913213)*x_ref[0]+(-0.54781629706)*x_ref[1]**o+(-0.0405507445628)*x_ref[1]+(0.435477036501)*x_ref[2]**o+(-0.296404435183)*x_ref[2]
        ref[1]=(-0.436126359475)*x_ref[0]**o+(0.863706244241)*x_ref[0]+(-0.149818409243)*x_ref[1]**o+(-0.654801920098)*x_ref[1]+(-0.281005092809)*x_ref[2]**o+(-0.0257179420057)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_ReducedFunctionOnContactZero_rank2(self):
      """
      tests interpolate for rank 2 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactZero onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactZero(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5),w)
      ref=Data(0,(4, 5),w_ref)
      if dim==2:
        arg[0,0]=(0.0615713819144)*x[0]**o+(0.325133297443)*x[0]+(-0.529769594257)*x[1]**o+(-0.206126939263)*x[1]
        arg[0,1]=(0.090956391947)*x[0]**o+(0.105841035331)*x[0]+(-0.537033610998)*x[1]**o+(0.429033781495)*x[1]
        arg[0,2]=(-0.568216263139)*x[0]**o+(0.698057796806)*x[0]+(-0.0981614063428)*x[1]**o+(-0.952212332523)*x[1]
        arg[0,3]=(0.128505813477)*x[0]**o+(0.206828305852)*x[0]+(0.938851873532)*x[1]**o+(-0.339234589281)*x[1]
        arg[0,4]=(0.581805477855)*x[0]**o+(-0.621122347153)*x[0]+(-0.601029625565)*x[1]**o+(0.243084366725)*x[1]
        arg[1,0]=(-0.342411991491)*x[0]**o+(0.642434320731)*x[0]+(-0.452657094515)*x[1]**o+(-0.899779109444)*x[1]
        arg[1,1]=(-0.423230603077)*x[0]**o+(0.739600521332)*x[0]+(0.916226480419)*x[1]**o+(0.755264173162)*x[1]
        arg[1,2]=(0.381864221314)*x[0]**o+(0.806402187902)*x[0]+(0.410055116458)*x[1]**o+(-0.264405213472)*x[1]
        arg[1,3]=(0.293401427429)*x[0]**o+(-0.439209675154)*x[0]+(0.299588226145)*x[1]**o+(-0.289652201976)*x[1]
        arg[1,4]=(0.157626502865)*x[0]**o+(0.32264788209)*x[0]+(0.284522039757)*x[1]**o+(-0.460413671191)*x[1]
        arg[2,0]=(0.486148207)*x[0]**o+(0.0366950072658)*x[0]+(0.315299830901)*x[1]**o+(-0.320405042262)*x[1]
        arg[2,1]=(0.207440897542)*x[0]**o+(-0.604659890692)*x[0]+(-0.64841431916)*x[1]**o+(0.0881921209641)*x[1]
        arg[2,2]=(-0.61376840347)*x[0]**o+(0.421837679185)*x[0]+(-0.0445850531916)*x[1]**o+(0.771892018903)*x[1]
        arg[2,3]=(0.632443182263)*x[0]**o+(-0.965297013012)*x[0]+(-0.029677959681)*x[1]**o+(0.598431910551)*x[1]
        arg[2,4]=(-0.457656411094)*x[0]**o+(0.122435925199)*x[0]+(0.772690251937)*x[1]**o+(0.954829922926)*x[1]
        arg[3,0]=(0.929112925461)*x[0]**o+(-0.35689901655)*x[0]+(-0.139192866782)*x[1]**o+(-0.677883436952)*x[1]
        arg[3,1]=(-0.410318721103)*x[0]**o+(0.370830312699)*x[0]+(0.429011619434)*x[1]**o+(0.81780420854)*x[1]
        arg[3,2]=(0.312456682985)*x[0]**o+(0.526447664999)*x[0]+(0.763610570932)*x[1]**o+(-0.631887901017)*x[1]
        arg[3,3]=(0.749121567955)*x[0]**o+(0.168525340464)*x[0]+(-0.838409619895)*x[1]**o+(0.912745137098)*x[1]
        arg[3,4]=(-0.665336604643)*x[0]**o+(0.638293226479)*x[0]+(0.515957084712)*x[1]**o+(-0.261426720977)*x[1]
        ref[0,0]=(0.0615713819144)*x_ref[0]**o+(0.325133297443)*x_ref[0]+(-0.529769594257)*x_ref[1]**o+(-0.206126939263)*x_ref[1]
        ref[0,1]=(0.090956391947)*x_ref[0]**o+(0.105841035331)*x_ref[0]+(-0.537033610998)*x_ref[1]**o+(0.429033781495)*x_ref[1]
        ref[0,2]=(-0.568216263139)*x_ref[0]**o+(0.698057796806)*x_ref[0]+(-0.0981614063428)*x_ref[1]**o+(-0.952212332523)*x_ref[1]
        ref[0,3]=(0.128505813477)*x_ref[0]**o+(0.206828305852)*x_ref[0]+(0.938851873532)*x_ref[1]**o+(-0.339234589281)*x_ref[1]
        ref[0,4]=(0.581805477855)*x_ref[0]**o+(-0.621122347153)*x_ref[0]+(-0.601029625565)*x_ref[1]**o+(0.243084366725)*x_ref[1]
        ref[1,0]=(-0.342411991491)*x_ref[0]**o+(0.642434320731)*x_ref[0]+(-0.452657094515)*x_ref[1]**o+(-0.899779109444)*x_ref[1]
        ref[1,1]=(-0.423230603077)*x_ref[0]**o+(0.739600521332)*x_ref[0]+(0.916226480419)*x_ref[1]**o+(0.755264173162)*x_ref[1]
        ref[1,2]=(0.381864221314)*x_ref[0]**o+(0.806402187902)*x_ref[0]+(0.410055116458)*x_ref[1]**o+(-0.264405213472)*x_ref[1]
        ref[1,3]=(0.293401427429)*x_ref[0]**o+(-0.439209675154)*x_ref[0]+(0.299588226145)*x_ref[1]**o+(-0.289652201976)*x_ref[1]
        ref[1,4]=(0.157626502865)*x_ref[0]**o+(0.32264788209)*x_ref[0]+(0.284522039757)*x_ref[1]**o+(-0.460413671191)*x_ref[1]
        ref[2,0]=(0.486148207)*x_ref[0]**o+(0.0366950072658)*x_ref[0]+(0.315299830901)*x_ref[1]**o+(-0.320405042262)*x_ref[1]
        ref[2,1]=(0.207440897542)*x_ref[0]**o+(-0.604659890692)*x_ref[0]+(-0.64841431916)*x_ref[1]**o+(0.0881921209641)*x_ref[1]
        ref[2,2]=(-0.61376840347)*x_ref[0]**o+(0.421837679185)*x_ref[0]+(-0.0445850531916)*x_ref[1]**o+(0.771892018903)*x_ref[1]
        ref[2,3]=(0.632443182263)*x_ref[0]**o+(-0.965297013012)*x_ref[0]+(-0.029677959681)*x_ref[1]**o+(0.598431910551)*x_ref[1]
        ref[2,4]=(-0.457656411094)*x_ref[0]**o+(0.122435925199)*x_ref[0]+(0.772690251937)*x_ref[1]**o+(0.954829922926)*x_ref[1]
        ref[3,0]=(0.929112925461)*x_ref[0]**o+(-0.35689901655)*x_ref[0]+(-0.139192866782)*x_ref[1]**o+(-0.677883436952)*x_ref[1]
        ref[3,1]=(-0.410318721103)*x_ref[0]**o+(0.370830312699)*x_ref[0]+(0.429011619434)*x_ref[1]**o+(0.81780420854)*x_ref[1]
        ref[3,2]=(0.312456682985)*x_ref[0]**o+(0.526447664999)*x_ref[0]+(0.763610570932)*x_ref[1]**o+(-0.631887901017)*x_ref[1]
        ref[3,3]=(0.749121567955)*x_ref[0]**o+(0.168525340464)*x_ref[0]+(-0.838409619895)*x_ref[1]**o+(0.912745137098)*x_ref[1]
        ref[3,4]=(-0.665336604643)*x_ref[0]**o+(0.638293226479)*x_ref[0]+(0.515957084712)*x_ref[1]**o+(-0.261426720977)*x_ref[1]
      else:
        arg[0,0]=(-0.330165086386)*x[0]**o+(0.60381119065)*x[0]+(0.981018763011)*x[1]**o+(0.0487965999608)*x[1]+(-0.389835388349)*x[2]**o+(0.111199857376)*x[2]
        arg[0,1]=(0.518239033178)*x[0]**o+(0.60358784684)*x[0]+(-0.476990159655)*x[1]**o+(-0.93871565843)*x[1]+(-0.869845295741)*x[2]**o+(0.391409745802)*x[2]
        arg[0,2]=(0.0927396115951)*x[0]**o+(0.546122822928)*x[0]+(-0.557424056733)*x[1]**o+(0.0623374966436)*x[1]+(-0.414595153536)*x[2]**o+(-0.196069073366)*x[2]
        arg[0,3]=(-0.678432580882)*x[0]**o+(0.281850265141)*x[0]+(-0.323978960602)*x[1]**o+(0.655558769921)*x[1]+(-0.295747032089)*x[2]**o+(-0.152719838982)*x[2]
        arg[0,4]=(-0.904177669203)*x[0]**o+(0.750395234205)*x[0]+(0.267431783461)*x[1]**o+(-0.781406235306)*x[1]+(0.856906655757)*x[2]**o+(-0.875554940224)*x[2]
        arg[1,0]=(-0.243144846569)*x[0]**o+(0.970584922467)*x[0]+(-0.977250652473)*x[1]**o+(0.0239563255869)*x[1]+(-0.769423602851)*x[2]**o+(-0.433896361294)*x[2]
        arg[1,1]=(0.31104031046)*x[0]**o+(0.899859709831)*x[0]+(0.692351561919)*x[1]**o+(-0.0838870550002)*x[1]+(-0.276105616798)*x[2]**o+(0.115501348248)*x[2]
        arg[1,2]=(0.56702114595)*x[0]**o+(0.00240284646131)*x[0]+(-0.676314888884)*x[1]**o+(-0.678204346328)*x[1]+(0.472697088984)*x[2]**o+(-0.493949191712)*x[2]
        arg[1,3]=(-0.242548554067)*x[0]**o+(0.980927507053)*x[0]+(-0.955706390436)*x[1]**o+(0.347031550126)*x[1]+(0.234352268238)*x[2]**o+(0.321126345333)*x[2]
        arg[1,4]=(-0.390890566228)*x[0]**o+(-0.958497753144)*x[0]+(0.316397716043)*x[1]**o+(0.952289821769)*x[1]+(-0.027182029514)*x[2]**o+(-0.454422377872)*x[2]
        arg[2,0]=(0.295865166318)*x[0]**o+(-0.301129718468)*x[0]+(-0.748135755269)*x[1]**o+(-0.762413814085)*x[1]+(0.572270067591)*x[2]**o+(-0.393847032684)*x[2]
        arg[2,1]=(0.20229063622)*x[0]**o+(-0.768444028163)*x[0]+(0.378669775381)*x[1]**o+(-0.703384410944)*x[1]+(0.355653859396)*x[2]**o+(-0.228168949639)*x[2]
        arg[2,2]=(-0.31725134742)*x[0]**o+(0.525802867508)*x[0]+(0.13375126758)*x[1]**o+(-0.673551491186)*x[1]+(0.498905883728)*x[2]**o+(-0.102794461221)*x[2]
        arg[2,3]=(-0.396813838168)*x[0]**o+(0.0892605807843)*x[0]+(0.244133298111)*x[1]**o+(0.310900636168)*x[1]+(-0.256807063422)*x[2]**o+(0.565370697688)*x[2]
        arg[2,4]=(0.42011835881)*x[0]**o+(0.439847594375)*x[0]+(0.0731897272006)*x[1]**o+(-0.373911054753)*x[1]+(-0.50313315963)*x[2]**o+(0.356362677601)*x[2]
        arg[3,0]=(-0.444672845607)*x[0]**o+(-0.978424347535)*x[0]+(0.421699748442)*x[1]**o+(-0.0111798084161)*x[1]+(0.206042685881)*x[2]**o+(0.990523831215)*x[2]
        arg[3,1]=(0.942429627068)*x[0]**o+(-0.769989484282)*x[0]+(-0.879916738075)*x[1]**o+(-0.746448112871)*x[1]+(0.280824650861)*x[2]**o+(0.0632084823022)*x[2]
        arg[3,2]=(0.281070481177)*x[0]**o+(0.628020001876)*x[0]+(0.347767303962)*x[1]**o+(-0.276761252142)*x[1]+(0.950345260734)*x[2]**o+(-0.00148186335195)*x[2]
        arg[3,3]=(-0.37590210465)*x[0]**o+(0.034147350068)*x[0]+(0.245173418336)*x[1]**o+(0.0564051241759)*x[1]+(-0.157928106127)*x[2]**o+(-0.611677441847)*x[2]
        arg[3,4]=(-0.279066532276)*x[0]**o+(0.316033119063)*x[0]+(0.655948999472)*x[1]**o+(-0.520504180943)*x[1]+(0.531738158819)*x[2]**o+(0.899490827443)*x[2]
        ref[0,0]=(-0.330165086386)*x_ref[0]**o+(0.60381119065)*x_ref[0]+(0.981018763011)*x_ref[1]**o+(0.0487965999608)*x_ref[1]+(-0.389835388349)*x_ref[2]**o+(0.111199857376)*x_ref[2]
        ref[0,1]=(0.518239033178)*x_ref[0]**o+(0.60358784684)*x_ref[0]+(-0.476990159655)*x_ref[1]**o+(-0.93871565843)*x_ref[1]+(-0.869845295741)*x_ref[2]**o+(0.391409745802)*x_ref[2]
        ref[0,2]=(0.0927396115951)*x_ref[0]**o+(0.546122822928)*x_ref[0]+(-0.557424056733)*x_ref[1]**o+(0.0623374966436)*x_ref[1]+(-0.414595153536)*x_ref[2]**o+(-0.196069073366)*x_ref[2]
        ref[0,3]=(-0.678432580882)*x_ref[0]**o+(0.281850265141)*x_ref[0]+(-0.323978960602)*x_ref[1]**o+(0.655558769921)*x_ref[1]+(-0.295747032089)*x_ref[2]**o+(-0.152719838982)*x_ref[2]
        ref[0,4]=(-0.904177669203)*x_ref[0]**o+(0.750395234205)*x_ref[0]+(0.267431783461)*x_ref[1]**o+(-0.781406235306)*x_ref[1]+(0.856906655757)*x_ref[2]**o+(-0.875554940224)*x_ref[2]
        ref[1,0]=(-0.243144846569)*x_ref[0]**o+(0.970584922467)*x_ref[0]+(-0.977250652473)*x_ref[1]**o+(0.0239563255869)*x_ref[1]+(-0.769423602851)*x_ref[2]**o+(-0.433896361294)*x_ref[2]
        ref[1,1]=(0.31104031046)*x_ref[0]**o+(0.899859709831)*x_ref[0]+(0.692351561919)*x_ref[1]**o+(-0.0838870550002)*x_ref[1]+(-0.276105616798)*x_ref[2]**o+(0.115501348248)*x_ref[2]
        ref[1,2]=(0.56702114595)*x_ref[0]**o+(0.00240284646131)*x_ref[0]+(-0.676314888884)*x_ref[1]**o+(-0.678204346328)*x_ref[1]+(0.472697088984)*x_ref[2]**o+(-0.493949191712)*x_ref[2]
        ref[1,3]=(-0.242548554067)*x_ref[0]**o+(0.980927507053)*x_ref[0]+(-0.955706390436)*x_ref[1]**o+(0.347031550126)*x_ref[1]+(0.234352268238)*x_ref[2]**o+(0.321126345333)*x_ref[2]
        ref[1,4]=(-0.390890566228)*x_ref[0]**o+(-0.958497753144)*x_ref[0]+(0.316397716043)*x_ref[1]**o+(0.952289821769)*x_ref[1]+(-0.027182029514)*x_ref[2]**o+(-0.454422377872)*x_ref[2]
        ref[2,0]=(0.295865166318)*x_ref[0]**o+(-0.301129718468)*x_ref[0]+(-0.748135755269)*x_ref[1]**o+(-0.762413814085)*x_ref[1]+(0.572270067591)*x_ref[2]**o+(-0.393847032684)*x_ref[2]
        ref[2,1]=(0.20229063622)*x_ref[0]**o+(-0.768444028163)*x_ref[0]+(0.378669775381)*x_ref[1]**o+(-0.703384410944)*x_ref[1]+(0.355653859396)*x_ref[2]**o+(-0.228168949639)*x_ref[2]
        ref[2,2]=(-0.31725134742)*x_ref[0]**o+(0.525802867508)*x_ref[0]+(0.13375126758)*x_ref[1]**o+(-0.673551491186)*x_ref[1]+(0.498905883728)*x_ref[2]**o+(-0.102794461221)*x_ref[2]
        ref[2,3]=(-0.396813838168)*x_ref[0]**o+(0.0892605807843)*x_ref[0]+(0.244133298111)*x_ref[1]**o+(0.310900636168)*x_ref[1]+(-0.256807063422)*x_ref[2]**o+(0.565370697688)*x_ref[2]
        ref[2,4]=(0.42011835881)*x_ref[0]**o+(0.439847594375)*x_ref[0]+(0.0731897272006)*x_ref[1]**o+(-0.373911054753)*x_ref[1]+(-0.50313315963)*x_ref[2]**o+(0.356362677601)*x_ref[2]
        ref[3,0]=(-0.444672845607)*x_ref[0]**o+(-0.978424347535)*x_ref[0]+(0.421699748442)*x_ref[1]**o+(-0.0111798084161)*x_ref[1]+(0.206042685881)*x_ref[2]**o+(0.990523831215)*x_ref[2]
        ref[3,1]=(0.942429627068)*x_ref[0]**o+(-0.769989484282)*x_ref[0]+(-0.879916738075)*x_ref[1]**o+(-0.746448112871)*x_ref[1]+(0.280824650861)*x_ref[2]**o+(0.0632084823022)*x_ref[2]
        ref[3,2]=(0.281070481177)*x_ref[0]**o+(0.628020001876)*x_ref[0]+(0.347767303962)*x_ref[1]**o+(-0.276761252142)*x_ref[1]+(0.950345260734)*x_ref[2]**o+(-0.00148186335195)*x_ref[2]
        ref[3,3]=(-0.37590210465)*x_ref[0]**o+(0.034147350068)*x_ref[0]+(0.245173418336)*x_ref[1]**o+(0.0564051241759)*x_ref[1]+(-0.157928106127)*x_ref[2]**o+(-0.611677441847)*x_ref[2]
        ref[3,4]=(-0.279066532276)*x_ref[0]**o+(0.316033119063)*x_ref[0]+(0.655948999472)*x_ref[1]**o+(-0.520504180943)*x_ref[1]+(0.531738158819)*x_ref[2]**o+(0.899490827443)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_ReducedFunctionOnContactZero_rank3(self):
      """
      tests interpolate for rank 3 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactZero onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactZero(self.domain)
      x=w.getX()
      arg=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2),w_ref)
      if dim==2:
        arg[0,0,0]=(0.415438516191)*x[0]**o+(0.374659323173)*x[0]+(0.688470746611)*x[1]**o+(0.498183692176)*x[1]
        arg[0,0,1]=(-0.846725329522)*x[0]**o+(0.414376816326)*x[0]+(0.651609661615)*x[1]**o+(0.357606396667)*x[1]
        arg[0,1,0]=(0.728029172023)*x[0]**o+(0.975728459112)*x[0]+(-0.658211371639)*x[1]**o+(0.411894140351)*x[1]
        arg[0,1,1]=(0.807350281817)*x[0]**o+(-0.369755733522)*x[0]+(-0.519483139314)*x[1]**o+(-0.709353537648)*x[1]
        arg[1,0,0]=(0.0588259329479)*x[0]**o+(0.218362870441)*x[0]+(-0.0279239318934)*x[1]**o+(-0.75783635329)*x[1]
        arg[1,0,1]=(-0.429573558505)*x[0]**o+(-0.339046134618)*x[0]+(0.412777825743)*x[1]**o+(-0.306607333437)*x[1]
        arg[1,1,0]=(0.0222661407543)*x[0]**o+(0.927923674781)*x[0]+(-0.767468570033)*x[1]**o+(-0.0659150590367)*x[1]
        arg[1,1,1]=(0.372811733787)*x[0]**o+(-0.286500260061)*x[0]+(-0.807504375569)*x[1]**o+(-0.992850148234)*x[1]
        arg[2,0,0]=(0.368229828596)*x[0]**o+(0.672540290379)*x[0]+(-0.760462595095)*x[1]**o+(-0.761242492425)*x[1]
        arg[2,0,1]=(0.306249523133)*x[0]**o+(0.769928924717)*x[0]+(0.610293800742)*x[1]**o+(-0.221996271153)*x[1]
        arg[2,1,0]=(-0.909148832991)*x[0]**o+(-0.0293038712703)*x[0]+(-0.161707473269)*x[1]**o+(-0.534345319122)*x[1]
        arg[2,1,1]=(-0.146432487737)*x[0]**o+(0.93729266958)*x[0]+(-0.561919238789)*x[1]**o+(-0.449440501171)*x[1]
        arg[3,0,0]=(0.411711661035)*x[0]**o+(-0.0699878047274)*x[0]+(-0.0227688826645)*x[1]**o+(-0.939787633547)*x[1]
        arg[3,0,1]=(-0.82108820245)*x[0]**o+(0.225501168847)*x[0]+(0.622921674682)*x[1]**o+(0.61092413094)*x[1]
        arg[3,1,0]=(0.758116155912)*x[0]**o+(-0.117316615447)*x[0]+(0.0702245119124)*x[1]**o+(-0.275114508561)*x[1]
        arg[3,1,1]=(-0.32568550821)*x[0]**o+(-0.280144846326)*x[0]+(0.194971187876)*x[1]**o+(-0.728447244746)*x[1]
        arg[4,0,0]=(0.880042622981)*x[0]**o+(-0.395684607563)*x[0]+(-0.563304061265)*x[1]**o+(0.968663606772)*x[1]
        arg[4,0,1]=(-0.515392712458)*x[0]**o+(0.820857647342)*x[0]+(-0.476289639495)*x[1]**o+(-0.478062589609)*x[1]
        arg[4,1,0]=(0.534322425269)*x[0]**o+(0.975618695265)*x[0]+(-0.195764005045)*x[1]**o+(0.561996200643)*x[1]
        arg[4,1,1]=(-0.906243138672)*x[0]**o+(-0.580985047294)*x[0]+(-0.094609405051)*x[1]**o+(0.734326711649)*x[1]
        arg[5,0,0]=(-0.0655516121632)*x[0]**o+(0.785493123174)*x[0]+(-0.760068333675)*x[1]**o+(0.693869247554)*x[1]
        arg[5,0,1]=(-0.931019838892)*x[0]**o+(0.80958291621)*x[0]+(-0.590591206111)*x[1]**o+(-0.800900346601)*x[1]
        arg[5,1,0]=(0.448779048952)*x[0]**o+(-0.793450503268)*x[0]+(0.161067403301)*x[1]**o+(0.432054043544)*x[1]
        arg[5,1,1]=(0.0423607361972)*x[0]**o+(-0.664958045702)*x[0]+(-0.174728816918)*x[1]**o+(-0.394390068699)*x[1]
        ref[0,0,0]=(0.415438516191)*x_ref[0]**o+(0.374659323173)*x_ref[0]+(0.688470746611)*x_ref[1]**o+(0.498183692176)*x_ref[1]
        ref[0,0,1]=(-0.846725329522)*x_ref[0]**o+(0.414376816326)*x_ref[0]+(0.651609661615)*x_ref[1]**o+(0.357606396667)*x_ref[1]
        ref[0,1,0]=(0.728029172023)*x_ref[0]**o+(0.975728459112)*x_ref[0]+(-0.658211371639)*x_ref[1]**o+(0.411894140351)*x_ref[1]
        ref[0,1,1]=(0.807350281817)*x_ref[0]**o+(-0.369755733522)*x_ref[0]+(-0.519483139314)*x_ref[1]**o+(-0.709353537648)*x_ref[1]
        ref[1,0,0]=(0.0588259329479)*x_ref[0]**o+(0.218362870441)*x_ref[0]+(-0.0279239318934)*x_ref[1]**o+(-0.75783635329)*x_ref[1]
        ref[1,0,1]=(-0.429573558505)*x_ref[0]**o+(-0.339046134618)*x_ref[0]+(0.412777825743)*x_ref[1]**o+(-0.306607333437)*x_ref[1]
        ref[1,1,0]=(0.0222661407543)*x_ref[0]**o+(0.927923674781)*x_ref[0]+(-0.767468570033)*x_ref[1]**o+(-0.0659150590367)*x_ref[1]
        ref[1,1,1]=(0.372811733787)*x_ref[0]**o+(-0.286500260061)*x_ref[0]+(-0.807504375569)*x_ref[1]**o+(-0.992850148234)*x_ref[1]
        ref[2,0,0]=(0.368229828596)*x_ref[0]**o+(0.672540290379)*x_ref[0]+(-0.760462595095)*x_ref[1]**o+(-0.761242492425)*x_ref[1]
        ref[2,0,1]=(0.306249523133)*x_ref[0]**o+(0.769928924717)*x_ref[0]+(0.610293800742)*x_ref[1]**o+(-0.221996271153)*x_ref[1]
        ref[2,1,0]=(-0.909148832991)*x_ref[0]**o+(-0.0293038712703)*x_ref[0]+(-0.161707473269)*x_ref[1]**o+(-0.534345319122)*x_ref[1]
        ref[2,1,1]=(-0.146432487737)*x_ref[0]**o+(0.93729266958)*x_ref[0]+(-0.561919238789)*x_ref[1]**o+(-0.449440501171)*x_ref[1]
        ref[3,0,0]=(0.411711661035)*x_ref[0]**o+(-0.0699878047274)*x_ref[0]+(-0.0227688826645)*x_ref[1]**o+(-0.939787633547)*x_ref[1]
        ref[3,0,1]=(-0.82108820245)*x_ref[0]**o+(0.225501168847)*x_ref[0]+(0.622921674682)*x_ref[1]**o+(0.61092413094)*x_ref[1]
        ref[3,1,0]=(0.758116155912)*x_ref[0]**o+(-0.117316615447)*x_ref[0]+(0.0702245119124)*x_ref[1]**o+(-0.275114508561)*x_ref[1]
        ref[3,1,1]=(-0.32568550821)*x_ref[0]**o+(-0.280144846326)*x_ref[0]+(0.194971187876)*x_ref[1]**o+(-0.728447244746)*x_ref[1]
        ref[4,0,0]=(0.880042622981)*x_ref[0]**o+(-0.395684607563)*x_ref[0]+(-0.563304061265)*x_ref[1]**o+(0.968663606772)*x_ref[1]
        ref[4,0,1]=(-0.515392712458)*x_ref[0]**o+(0.820857647342)*x_ref[0]+(-0.476289639495)*x_ref[1]**o+(-0.478062589609)*x_ref[1]
        ref[4,1,0]=(0.534322425269)*x_ref[0]**o+(0.975618695265)*x_ref[0]+(-0.195764005045)*x_ref[1]**o+(0.561996200643)*x_ref[1]
        ref[4,1,1]=(-0.906243138672)*x_ref[0]**o+(-0.580985047294)*x_ref[0]+(-0.094609405051)*x_ref[1]**o+(0.734326711649)*x_ref[1]
        ref[5,0,0]=(-0.0655516121632)*x_ref[0]**o+(0.785493123174)*x_ref[0]+(-0.760068333675)*x_ref[1]**o+(0.693869247554)*x_ref[1]
        ref[5,0,1]=(-0.931019838892)*x_ref[0]**o+(0.80958291621)*x_ref[0]+(-0.590591206111)*x_ref[1]**o+(-0.800900346601)*x_ref[1]
        ref[5,1,0]=(0.448779048952)*x_ref[0]**o+(-0.793450503268)*x_ref[0]+(0.161067403301)*x_ref[1]**o+(0.432054043544)*x_ref[1]
        ref[5,1,1]=(0.0423607361972)*x_ref[0]**o+(-0.664958045702)*x_ref[0]+(-0.174728816918)*x_ref[1]**o+(-0.394390068699)*x_ref[1]
      else:
        arg[0,0,0]=(0.34355250678)*x[0]**o+(0.554590991175)*x[0]+(0.00673862595967)*x[1]**o+(-0.405764224977)*x[1]+(0.0865564883661)*x[2]**o+(0.102294137607)*x[2]
        arg[0,0,1]=(0.855234460708)*x[0]**o+(-0.710901486015)*x[0]+(0.546393138087)*x[1]**o+(0.847113479239)*x[1]+(-0.650701604407)*x[2]**o+(-0.914755510262)*x[2]
        arg[0,1,0]=(0.252544568503)*x[0]**o+(-0.473471976093)*x[0]+(-0.587315274764)*x[1]**o+(-0.269940894375)*x[1]+(0.360340735507)*x[2]**o+(-0.745440004827)*x[2]
        arg[0,1,1]=(0.11967506555)*x[0]**o+(0.696836453942)*x[0]+(-0.871162187396)*x[1]**o+(-0.315170111067)*x[1]+(0.232066109266)*x[2]**o+(-0.464730524838)*x[2]
        arg[1,0,0]=(-0.790535662997)*x[0]**o+(-0.964779159884)*x[0]+(-0.689937881374)*x[1]**o+(0.422799231262)*x[1]+(-0.97315874804)*x[2]**o+(-0.532103115205)*x[2]
        arg[1,0,1]=(-0.0487589554945)*x[0]**o+(-0.622735713658)*x[0]+(-0.520842613265)*x[1]**o+(0.803711422599)*x[1]+(0.440648875556)*x[2]**o+(0.98786672574)*x[2]
        arg[1,1,0]=(0.811189977538)*x[0]**o+(0.0175150486788)*x[0]+(-0.124328899976)*x[1]**o+(-0.942398680503)*x[1]+(-0.149270209895)*x[2]**o+(-0.853084407696)*x[2]
        arg[1,1,1]=(0.688378159744)*x[0]**o+(0.408766927878)*x[0]+(0.834128919609)*x[1]**o+(0.253605923886)*x[1]+(0.424992121054)*x[2]**o+(0.21016117963)*x[2]
        arg[2,0,0]=(0.277469067087)*x[0]**o+(0.539821557611)*x[0]+(0.348679402521)*x[1]**o+(-0.469772506537)*x[1]+(0.454583229137)*x[2]**o+(0.653681481133)*x[2]
        arg[2,0,1]=(-0.934067178483)*x[0]**o+(-0.172735098497)*x[0]+(-0.0347256850948)*x[1]**o+(-0.459971424944)*x[1]+(0.932661853019)*x[2]**o+(-0.186822590955)*x[2]
        arg[2,1,0]=(0.71698574321)*x[0]**o+(-0.197244831261)*x[0]+(0.183419576182)*x[1]**o+(-0.871451480853)*x[1]+(-0.349396269955)*x[2]**o+(0.095909246936)*x[2]
        arg[2,1,1]=(-0.00972985962106)*x[0]**o+(-0.919204152811)*x[0]+(-0.751744404365)*x[1]**o+(0.377096436852)*x[1]+(-0.924228885362)*x[2]**o+(0.451053813959)*x[2]
        arg[3,0,0]=(0.951119968373)*x[0]**o+(-0.0680027504528)*x[0]+(-0.113660990287)*x[1]**o+(-0.428999453003)*x[1]+(0.171804544103)*x[2]**o+(0.677635036297)*x[2]
        arg[3,0,1]=(0.62534864581)*x[0]**o+(0.389496246771)*x[0]+(0.621295327948)*x[1]**o+(-0.684165739023)*x[1]+(0.258637161492)*x[2]**o+(0.0441460764076)*x[2]
        arg[3,1,0]=(0.0348922721145)*x[0]**o+(0.091201286767)*x[0]+(0.421079169132)*x[1]**o+(-0.616504862591)*x[1]+(0.563927673176)*x[2]**o+(-0.183993394822)*x[2]
        arg[3,1,1]=(0.657959811783)*x[0]**o+(0.229729671382)*x[0]+(0.549853155207)*x[1]**o+(-0.847412930565)*x[1]+(-0.0780682500337)*x[2]**o+(0.574435563406)*x[2]
        arg[4,0,0]=(-0.679988515344)*x[0]**o+(0.0517331630343)*x[0]+(-0.160281064879)*x[1]**o+(-0.0672405944261)*x[1]+(-0.118968444607)*x[2]**o+(0.00980065834148)*x[2]
        arg[4,0,1]=(-0.150457972951)*x[0]**o+(0.671586511406)*x[0]+(-0.844112675259)*x[1]**o+(0.42766255101)*x[1]+(-0.22449721018)*x[2]**o+(-0.150416151059)*x[2]
        arg[4,1,0]=(0.0768992597753)*x[0]**o+(0.0139605476692)*x[0]+(-0.831269554863)*x[1]**o+(-0.279339344511)*x[1]+(0.387318821185)*x[2]**o+(0.967687121678)*x[2]
        arg[4,1,1]=(-0.670585948598)*x[0]**o+(0.422715818654)*x[0]+(0.623702390878)*x[1]**o+(0.922646081646)*x[1]+(-0.714191287039)*x[2]**o+(-0.99018084869)*x[2]
        arg[5,0,0]=(0.100273819257)*x[0]**o+(-0.769422291863)*x[0]+(-0.152611272068)*x[1]**o+(-0.396798631305)*x[1]+(0.502251047462)*x[2]**o+(0.993791216449)*x[2]
        arg[5,0,1]=(-0.0222540900657)*x[0]**o+(-0.339810353987)*x[0]+(0.241665078696)*x[1]**o+(0.111020568564)*x[1]+(0.298147316931)*x[2]**o+(0.391278660594)*x[2]
        arg[5,1,0]=(0.828297599555)*x[0]**o+(0.237678875371)*x[0]+(0.924551126274)*x[1]**o+(-0.663695049485)*x[1]+(-0.547123493132)*x[2]**o+(-0.788635745118)*x[2]
        arg[5,1,1]=(0.0726334370982)*x[0]**o+(-0.875688492957)*x[0]+(0.473726235108)*x[1]**o+(0.023025596151)*x[1]+(0.827449396402)*x[2]**o+(-0.154698941604)*x[2]
        ref[0,0,0]=(0.34355250678)*x_ref[0]**o+(0.554590991175)*x_ref[0]+(0.00673862595967)*x_ref[1]**o+(-0.405764224977)*x_ref[1]+(0.0865564883661)*x_ref[2]**o+(0.102294137607)*x_ref[2]
        ref[0,0,1]=(0.855234460708)*x_ref[0]**o+(-0.710901486015)*x_ref[0]+(0.546393138087)*x_ref[1]**o+(0.847113479239)*x_ref[1]+(-0.650701604407)*x_ref[2]**o+(-0.914755510262)*x_ref[2]
        ref[0,1,0]=(0.252544568503)*x_ref[0]**o+(-0.473471976093)*x_ref[0]+(-0.587315274764)*x_ref[1]**o+(-0.269940894375)*x_ref[1]+(0.360340735507)*x_ref[2]**o+(-0.745440004827)*x_ref[2]
        ref[0,1,1]=(0.11967506555)*x_ref[0]**o+(0.696836453942)*x_ref[0]+(-0.871162187396)*x_ref[1]**o+(-0.315170111067)*x_ref[1]+(0.232066109266)*x_ref[2]**o+(-0.464730524838)*x_ref[2]
        ref[1,0,0]=(-0.790535662997)*x_ref[0]**o+(-0.964779159884)*x_ref[0]+(-0.689937881374)*x_ref[1]**o+(0.422799231262)*x_ref[1]+(-0.97315874804)*x_ref[2]**o+(-0.532103115205)*x_ref[2]
        ref[1,0,1]=(-0.0487589554945)*x_ref[0]**o+(-0.622735713658)*x_ref[0]+(-0.520842613265)*x_ref[1]**o+(0.803711422599)*x_ref[1]+(0.440648875556)*x_ref[2]**o+(0.98786672574)*x_ref[2]
        ref[1,1,0]=(0.811189977538)*x_ref[0]**o+(0.0175150486788)*x_ref[0]+(-0.124328899976)*x_ref[1]**o+(-0.942398680503)*x_ref[1]+(-0.149270209895)*x_ref[2]**o+(-0.853084407696)*x_ref[2]
        ref[1,1,1]=(0.688378159744)*x_ref[0]**o+(0.408766927878)*x_ref[0]+(0.834128919609)*x_ref[1]**o+(0.253605923886)*x_ref[1]+(0.424992121054)*x_ref[2]**o+(0.21016117963)*x_ref[2]
        ref[2,0,0]=(0.277469067087)*x_ref[0]**o+(0.539821557611)*x_ref[0]+(0.348679402521)*x_ref[1]**o+(-0.469772506537)*x_ref[1]+(0.454583229137)*x_ref[2]**o+(0.653681481133)*x_ref[2]
        ref[2,0,1]=(-0.934067178483)*x_ref[0]**o+(-0.172735098497)*x_ref[0]+(-0.0347256850948)*x_ref[1]**o+(-0.459971424944)*x_ref[1]+(0.932661853019)*x_ref[2]**o+(-0.186822590955)*x_ref[2]
        ref[2,1,0]=(0.71698574321)*x_ref[0]**o+(-0.197244831261)*x_ref[0]+(0.183419576182)*x_ref[1]**o+(-0.871451480853)*x_ref[1]+(-0.349396269955)*x_ref[2]**o+(0.095909246936)*x_ref[2]
        ref[2,1,1]=(-0.00972985962106)*x_ref[0]**o+(-0.919204152811)*x_ref[0]+(-0.751744404365)*x_ref[1]**o+(0.377096436852)*x_ref[1]+(-0.924228885362)*x_ref[2]**o+(0.451053813959)*x_ref[2]
        ref[3,0,0]=(0.951119968373)*x_ref[0]**o+(-0.0680027504528)*x_ref[0]+(-0.113660990287)*x_ref[1]**o+(-0.428999453003)*x_ref[1]+(0.171804544103)*x_ref[2]**o+(0.677635036297)*x_ref[2]
        ref[3,0,1]=(0.62534864581)*x_ref[0]**o+(0.389496246771)*x_ref[0]+(0.621295327948)*x_ref[1]**o+(-0.684165739023)*x_ref[1]+(0.258637161492)*x_ref[2]**o+(0.0441460764076)*x_ref[2]
        ref[3,1,0]=(0.0348922721145)*x_ref[0]**o+(0.091201286767)*x_ref[0]+(0.421079169132)*x_ref[1]**o+(-0.616504862591)*x_ref[1]+(0.563927673176)*x_ref[2]**o+(-0.183993394822)*x_ref[2]
        ref[3,1,1]=(0.657959811783)*x_ref[0]**o+(0.229729671382)*x_ref[0]+(0.549853155207)*x_ref[1]**o+(-0.847412930565)*x_ref[1]+(-0.0780682500337)*x_ref[2]**o+(0.574435563406)*x_ref[2]
        ref[4,0,0]=(-0.679988515344)*x_ref[0]**o+(0.0517331630343)*x_ref[0]+(-0.160281064879)*x_ref[1]**o+(-0.0672405944261)*x_ref[1]+(-0.118968444607)*x_ref[2]**o+(0.00980065834148)*x_ref[2]
        ref[4,0,1]=(-0.150457972951)*x_ref[0]**o+(0.671586511406)*x_ref[0]+(-0.844112675259)*x_ref[1]**o+(0.42766255101)*x_ref[1]+(-0.22449721018)*x_ref[2]**o+(-0.150416151059)*x_ref[2]
        ref[4,1,0]=(0.0768992597753)*x_ref[0]**o+(0.0139605476692)*x_ref[0]+(-0.831269554863)*x_ref[1]**o+(-0.279339344511)*x_ref[1]+(0.387318821185)*x_ref[2]**o+(0.967687121678)*x_ref[2]
        ref[4,1,1]=(-0.670585948598)*x_ref[0]**o+(0.422715818654)*x_ref[0]+(0.623702390878)*x_ref[1]**o+(0.922646081646)*x_ref[1]+(-0.714191287039)*x_ref[2]**o+(-0.99018084869)*x_ref[2]
        ref[5,0,0]=(0.100273819257)*x_ref[0]**o+(-0.769422291863)*x_ref[0]+(-0.152611272068)*x_ref[1]**o+(-0.396798631305)*x_ref[1]+(0.502251047462)*x_ref[2]**o+(0.993791216449)*x_ref[2]
        ref[5,0,1]=(-0.0222540900657)*x_ref[0]**o+(-0.339810353987)*x_ref[0]+(0.241665078696)*x_ref[1]**o+(0.111020568564)*x_ref[1]+(0.298147316931)*x_ref[2]**o+(0.391278660594)*x_ref[2]
        ref[5,1,0]=(0.828297599555)*x_ref[0]**o+(0.237678875371)*x_ref[0]+(0.924551126274)*x_ref[1]**o+(-0.663695049485)*x_ref[1]+(-0.547123493132)*x_ref[2]**o+(-0.788635745118)*x_ref[2]
        ref[5,1,1]=(0.0726334370982)*x_ref[0]**o+(-0.875688492957)*x_ref[0]+(0.473726235108)*x_ref[1]**o+(0.023025596151)*x_ref[1]+(0.827449396402)*x_ref[2]**o+(-0.154698941604)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_ReducedFunctionOnContactZero_rank4(self):
      """
      tests interpolate for rank 4 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactZero onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactZero(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5, 3, 2),w)
      ref=Data(0,(4, 5, 3, 2),w_ref)
      if dim==2:
        arg[0,0,0,0]=(-0.0338299192888)*x[0]**o+(-0.114906274359)*x[0]+(0.15010339398)*x[1]**o+(0.625295991328)*x[1]
        arg[0,0,0,1]=(-0.1311901039)*x[0]**o+(-0.551712861217)*x[0]+(-0.287411404506)*x[1]**o+(0.0242820629572)*x[1]
        arg[0,0,1,0]=(0.121962094111)*x[0]**o+(0.203169622345)*x[0]+(0.241504218053)*x[1]**o+(-0.0696356239498)*x[1]
        arg[0,0,1,1]=(0.0461537352329)*x[0]**o+(-0.275908938567)*x[0]+(-0.271184121027)*x[1]**o+(-0.364024729567)*x[1]
        arg[0,0,2,0]=(0.199272234315)*x[0]**o+(-0.906364258855)*x[0]+(-0.92102930037)*x[1]**o+(0.0718981903454)*x[1]
        arg[0,0,2,1]=(-0.742950958383)*x[0]**o+(0.0301464847722)*x[0]+(0.466993477687)*x[1]**o+(-0.402131750698)*x[1]
        arg[0,1,0,0]=(-0.0653776546375)*x[0]**o+(-0.679360612372)*x[0]+(-0.32783837768)*x[1]**o+(-0.113753016428)*x[1]
        arg[0,1,0,1]=(0.58610119414)*x[0]**o+(0.142613166231)*x[0]+(0.0409105619986)*x[1]**o+(-0.660641663558)*x[1]
        arg[0,1,1,0]=(0.212172308585)*x[0]**o+(-0.452767768344)*x[0]+(0.963053969889)*x[1]**o+(0.752691773953)*x[1]
        arg[0,1,1,1]=(-0.514998377887)*x[0]**o+(0.0872464329801)*x[0]+(0.265590295505)*x[1]**o+(0.557238826549)*x[1]
        arg[0,1,2,0]=(0.360523954724)*x[0]**o+(0.550238402994)*x[0]+(-0.206349415539)*x[1]**o+(0.357304730511)*x[1]
        arg[0,1,2,1]=(-0.872755217435)*x[0]**o+(0.190943683664)*x[0]+(-0.245454292354)*x[1]**o+(0.507229578506)*x[1]
        arg[0,2,0,0]=(0.994619505257)*x[0]**o+(0.33482820568)*x[0]+(-0.12721791236)*x[1]**o+(-0.505873171195)*x[1]
        arg[0,2,0,1]=(0.224805131522)*x[0]**o+(0.332022434603)*x[0]+(0.0583494507268)*x[1]**o+(0.857185410994)*x[1]
        arg[0,2,1,0]=(-0.425367077516)*x[0]**o+(-0.25207101061)*x[0]+(-0.11164491753)*x[1]**o+(0.44752084285)*x[1]
        arg[0,2,1,1]=(-0.158725210751)*x[0]**o+(-0.0692363138932)*x[0]+(0.214803128256)*x[1]**o+(0.886699106748)*x[1]
        arg[0,2,2,0]=(-0.332146331257)*x[0]**o+(0.114845050128)*x[0]+(-0.0781411611124)*x[1]**o+(0.474972057387)*x[1]
        arg[0,2,2,1]=(-0.481949630694)*x[0]**o+(0.644984972151)*x[0]+(0.171146519527)*x[1]**o+(0.57534803966)*x[1]
        arg[0,3,0,0]=(0.7580633024)*x[0]**o+(0.0280511776577)*x[0]+(0.375583652078)*x[1]**o+(-0.809112787284)*x[1]
        arg[0,3,0,1]=(-0.466737159934)*x[0]**o+(0.410032775686)*x[0]+(0.176141189584)*x[1]**o+(-0.228288309456)*x[1]
        arg[0,3,1,0]=(0.427129582196)*x[0]**o+(-0.718385859483)*x[0]+(-0.172261212989)*x[1]**o+(0.913157476188)*x[1]
        arg[0,3,1,1]=(0.952506706487)*x[0]**o+(-0.497478163899)*x[0]+(-0.930273038716)*x[1]**o+(0.673573149025)*x[1]
        arg[0,3,2,0]=(-0.0074302930747)*x[0]**o+(0.226424842035)*x[0]+(0.871190620671)*x[1]**o+(0.277695333679)*x[1]
        arg[0,3,2,1]=(-0.540820239946)*x[0]**o+(0.33044763211)*x[0]+(-0.772648820924)*x[1]**o+(-0.362410088021)*x[1]
        arg[0,4,0,0]=(-0.748396039111)*x[0]**o+(0.215131518212)*x[0]+(0.514087824403)*x[1]**o+(0.335563854205)*x[1]
        arg[0,4,0,1]=(-0.443263590391)*x[0]**o+(-0.523167282542)*x[0]+(0.122612452799)*x[1]**o+(0.216527857615)*x[1]
        arg[0,4,1,0]=(0.448176296777)*x[0]**o+(0.478292146131)*x[0]+(-0.558921083502)*x[1]**o+(-0.17018258654)*x[1]
        arg[0,4,1,1]=(-0.912765673953)*x[0]**o+(0.751746623223)*x[0]+(-0.104811579057)*x[1]**o+(0.456436870323)*x[1]
        arg[0,4,2,0]=(0.157946450932)*x[0]**o+(0.147858587857)*x[0]+(0.775747362874)*x[1]**o+(0.478478862463)*x[1]
        arg[0,4,2,1]=(-0.819625058185)*x[0]**o+(-0.0231980393919)*x[0]+(-0.186647585522)*x[1]**o+(0.513350371371)*x[1]
        arg[1,0,0,0]=(-0.942373946995)*x[0]**o+(-0.761326160931)*x[0]+(0.192511867167)*x[1]**o+(-0.50895969579)*x[1]
        arg[1,0,0,1]=(0.0670813349951)*x[0]**o+(0.55602953512)*x[0]+(0.757621178635)*x[1]**o+(0.741550201759)*x[1]
        arg[1,0,1,0]=(-0.114642609493)*x[0]**o+(-0.639500048447)*x[0]+(-0.861813039243)*x[1]**o+(0.36413406744)*x[1]
        arg[1,0,1,1]=(0.126555451594)*x[0]**o+(-0.885172557082)*x[0]+(-0.376073252921)*x[1]**o+(0.166102850653)*x[1]
        arg[1,0,2,0]=(-0.381517068219)*x[0]**o+(0.94870328267)*x[0]+(-0.129934777755)*x[1]**o+(0.78090715758)*x[1]
        arg[1,0,2,1]=(-0.535852302247)*x[0]**o+(0.63751039318)*x[0]+(-0.537556159925)*x[1]**o+(0.147766714221)*x[1]
        arg[1,1,0,0]=(-0.605543047533)*x[0]**o+(-0.366612785372)*x[0]+(0.544900149212)*x[1]**o+(0.722108900447)*x[1]
        arg[1,1,0,1]=(0.133378561142)*x[0]**o+(-0.227808772494)*x[0]+(-0.729399624175)*x[1]**o+(0.307571743455)*x[1]
        arg[1,1,1,0]=(0.0458830538139)*x[0]**o+(0.0585044493026)*x[0]+(-0.136630547605)*x[1]**o+(-0.0823288389309)*x[1]
        arg[1,1,1,1]=(0.339263808784)*x[0]**o+(0.00988917109076)*x[0]+(-0.786591838083)*x[1]**o+(-0.361686799179)*x[1]
        arg[1,1,2,0]=(0.789942037592)*x[0]**o+(-0.266689758676)*x[0]+(0.464856327495)*x[1]**o+(0.138022774293)*x[1]
        arg[1,1,2,1]=(-0.723544867217)*x[0]**o+(-0.523023674205)*x[0]+(0.629726026331)*x[1]**o+(-0.867808120997)*x[1]
        arg[1,2,0,0]=(-0.495589518787)*x[0]**o+(-0.561664880262)*x[0]+(0.0319320417078)*x[1]**o+(-0.153720661498)*x[1]
        arg[1,2,0,1]=(0.293168114024)*x[0]**o+(-0.344107163981)*x[0]+(-0.275629670068)*x[1]**o+(0.06103653487)*x[1]
        arg[1,2,1,0]=(0.495640927339)*x[0]**o+(0.596540095043)*x[0]+(0.208792567756)*x[1]**o+(0.886040724145)*x[1]
        arg[1,2,1,1]=(0.331476398093)*x[0]**o+(-0.472588572455)*x[0]+(-0.455670457285)*x[1]**o+(-0.731223615642)*x[1]
        arg[1,2,2,0]=(-0.127107829674)*x[0]**o+(-0.21113128456)*x[0]+(-0.733928194168)*x[1]**o+(-0.174127696853)*x[1]
        arg[1,2,2,1]=(0.521879316151)*x[0]**o+(0.983424557865)*x[0]+(-0.288659660182)*x[1]**o+(-0.431045203113)*x[1]
        arg[1,3,0,0]=(0.292348765746)*x[0]**o+(0.0730741240915)*x[0]+(0.079624022683)*x[1]**o+(0.237483071633)*x[1]
        arg[1,3,0,1]=(0.291035771848)*x[0]**o+(-0.482909477855)*x[0]+(-0.999142129615)*x[1]**o+(-0.559723367197)*x[1]
        arg[1,3,1,0]=(0.747096275653)*x[0]**o+(0.762757649624)*x[0]+(-0.821650889507)*x[1]**o+(0.301128670189)*x[1]
        arg[1,3,1,1]=(0.82703681414)*x[0]**o+(-0.336570070436)*x[0]+(0.798131952979)*x[1]**o+(0.603562737956)*x[1]
        arg[1,3,2,0]=(-0.164875914126)*x[0]**o+(0.219409873902)*x[0]+(0.784253160678)*x[1]**o+(0.0219597460707)*x[1]
        arg[1,3,2,1]=(0.124778899794)*x[0]**o+(0.785367842145)*x[0]+(0.948111125908)*x[1]**o+(-0.887427847089)*x[1]
        arg[1,4,0,0]=(0.0911816002883)*x[0]**o+(0.645410022441)*x[0]+(0.720551531965)*x[1]**o+(-0.388407683413)*x[1]
        arg[1,4,0,1]=(-0.0416762613062)*x[0]**o+(-0.548874829834)*x[0]+(-0.377952555557)*x[1]**o+(0.742795234361)*x[1]
        arg[1,4,1,0]=(-0.896929017971)*x[0]**o+(0.678343686516)*x[0]+(0.711572763082)*x[1]**o+(-0.43397331651)*x[1]
        arg[1,4,1,1]=(-0.190804421566)*x[0]**o+(-0.0848405124789)*x[0]+(0.36246957905)*x[1]**o+(0.963599086227)*x[1]
        arg[1,4,2,0]=(-0.0210248567982)*x[0]**o+(0.545977814581)*x[0]+(0.858962864401)*x[1]**o+(0.113792851997)*x[1]
        arg[1,4,2,1]=(0.229641892622)*x[0]**o+(-0.739561617198)*x[0]+(-0.164372740617)*x[1]**o+(0.537097868843)*x[1]
        arg[2,0,0,0]=(-0.13321389343)*x[0]**o+(-0.668403350503)*x[0]+(-0.924879105983)*x[1]**o+(-0.5161790717)*x[1]
        arg[2,0,0,1]=(0.26582844685)*x[0]**o+(0.89048660684)*x[0]+(-0.377225981719)*x[1]**o+(-0.303790710194)*x[1]
        arg[2,0,1,0]=(0.0997470860283)*x[0]**o+(0.0385427264217)*x[0]+(-0.71387347273)*x[1]**o+(-0.621376499192)*x[1]
        arg[2,0,1,1]=(0.595544616326)*x[0]**o+(-0.850529503975)*x[0]+(0.602203957938)*x[1]**o+(-0.329484843907)*x[1]
        arg[2,0,2,0]=(-0.106031967029)*x[0]**o+(-0.165122821622)*x[0]+(0.647473815919)*x[1]**o+(0.28942226062)*x[1]
        arg[2,0,2,1]=(0.585955875472)*x[0]**o+(0.922667710403)*x[0]+(-0.0710905957841)*x[1]**o+(-0.669138163697)*x[1]
        arg[2,1,0,0]=(0.461941211081)*x[0]**o+(0.723022466263)*x[0]+(-0.118551893472)*x[1]**o+(0.00445187353637)*x[1]
        arg[2,1,0,1]=(-0.253168248108)*x[0]**o+(0.476938842605)*x[0]+(0.476641658034)*x[1]**o+(-0.916886387595)*x[1]
        arg[2,1,1,0]=(0.722029309503)*x[0]**o+(0.0588848593425)*x[0]+(0.84737370043)*x[1]**o+(-0.674604415213)*x[1]
        arg[2,1,1,1]=(-0.644736759298)*x[0]**o+(-0.212847303893)*x[0]+(0.25394587552)*x[1]**o+(-0.366955175081)*x[1]
        arg[2,1,2,0]=(-0.657395296801)*x[0]**o+(-0.967570027599)*x[0]+(0.180926628541)*x[1]**o+(-0.940599897942)*x[1]
        arg[2,1,2,1]=(0.849897328801)*x[0]**o+(-0.299199474051)*x[0]+(0.427221207827)*x[1]**o+(0.212027985441)*x[1]
        arg[2,2,0,0]=(-0.561573507381)*x[0]**o+(-0.446925224908)*x[0]+(0.132091224948)*x[1]**o+(-0.673375661849)*x[1]
        arg[2,2,0,1]=(-0.386657291574)*x[0]**o+(0.356784637112)*x[0]+(0.644292006233)*x[1]**o+(0.922937167842)*x[1]
        arg[2,2,1,0]=(0.484446757285)*x[0]**o+(-0.822469870455)*x[0]+(-0.89605890123)*x[1]**o+(-0.895891500031)*x[1]
        arg[2,2,1,1]=(-0.0930740520192)*x[0]**o+(0.953356473678)*x[0]+(-0.714418458806)*x[1]**o+(-0.100584950946)*x[1]
        arg[2,2,2,0]=(0.945572242667)*x[0]**o+(-0.259280849605)*x[0]+(-0.925459846341)*x[1]**o+(0.906002758873)*x[1]
        arg[2,2,2,1]=(-0.638993332975)*x[0]**o+(-0.668594641703)*x[0]+(0.012666231369)*x[1]**o+(0.89732685856)*x[1]
        arg[2,3,0,0]=(-0.885061116144)*x[0]**o+(-0.576097536483)*x[0]+(-0.43166075788)*x[1]**o+(0.538821620917)*x[1]
        arg[2,3,0,1]=(-0.0743638381797)*x[0]**o+(0.251037500035)*x[0]+(-0.905989928076)*x[1]**o+(0.243771000867)*x[1]
        arg[2,3,1,0]=(-0.016045622351)*x[0]**o+(-0.632944072174)*x[0]+(-0.296175918793)*x[1]**o+(-0.00120123641365)*x[1]
        arg[2,3,1,1]=(-0.896354921812)*x[0]**o+(-0.331348989027)*x[0]+(-0.450865285201)*x[1]**o+(-0.243268582009)*x[1]
        arg[2,3,2,0]=(0.609332927431)*x[0]**o+(-0.0815856802115)*x[0]+(0.201924792273)*x[1]**o+(-0.890424451348)*x[1]
        arg[2,3,2,1]=(0.395090137264)*x[0]**o+(0.294230442573)*x[0]+(-0.345371510364)*x[1]**o+(0.181229663241)*x[1]
        arg[2,4,0,0]=(-0.823684509034)*x[0]**o+(0.431019301942)*x[0]+(0.554707015054)*x[1]**o+(-0.846322903003)*x[1]
        arg[2,4,0,1]=(0.862984476658)*x[0]**o+(0.20512500102)*x[0]+(0.0247756159287)*x[1]**o+(-0.374795100946)*x[1]
        arg[2,4,1,0]=(-0.0849914269517)*x[0]**o+(0.754908200339)*x[0]+(0.593363754807)*x[1]**o+(0.124923961126)*x[1]
        arg[2,4,1,1]=(-0.691440264234)*x[0]**o+(0.825739298015)*x[0]+(0.426452231478)*x[1]**o+(0.0995610721341)*x[1]
        arg[2,4,2,0]=(-0.175322750567)*x[0]**o+(0.419750865569)*x[0]+(0.208448987712)*x[1]**o+(0.150584798952)*x[1]
        arg[2,4,2,1]=(0.931680152344)*x[0]**o+(0.390947206307)*x[0]+(0.388437348378)*x[1]**o+(-0.808413079032)*x[1]
        arg[3,0,0,0]=(0.58855431402)*x[0]**o+(0.383865913364)*x[0]+(-0.691445323219)*x[1]**o+(-0.443370658523)*x[1]
        arg[3,0,0,1]=(-0.657371481478)*x[0]**o+(0.746891082673)*x[0]+(-0.829229947773)*x[1]**o+(0.761915670404)*x[1]
        arg[3,0,1,0]=(-0.431502932714)*x[0]**o+(-0.961472285716)*x[0]+(0.0423384388031)*x[1]**o+(0.339373458118)*x[1]
        arg[3,0,1,1]=(-0.272831163259)*x[0]**o+(-0.533438904494)*x[0]+(-0.595625384875)*x[1]**o+(0.0366857022323)*x[1]
        arg[3,0,2,0]=(-0.319684662306)*x[0]**o+(0.672185828912)*x[0]+(0.483126548994)*x[1]**o+(0.745872629604)*x[1]
        arg[3,0,2,1]=(0.437895277404)*x[0]**o+(-0.337209416963)*x[0]+(0.101080522788)*x[1]**o+(0.892022488644)*x[1]
        arg[3,1,0,0]=(-0.582325078455)*x[0]**o+(-0.854689347157)*x[0]+(0.0308638312259)*x[1]**o+(0.633382365704)*x[1]
        arg[3,1,0,1]=(0.319735178376)*x[0]**o+(-0.782764593952)*x[0]+(0.082230132244)*x[1]**o+(-0.946010937852)*x[1]
        arg[3,1,1,0]=(-0.698490410666)*x[0]**o+(-0.801358802689)*x[0]+(0.20695648235)*x[1]**o+(0.75257523471)*x[1]
        arg[3,1,1,1]=(0.66321696527)*x[0]**o+(0.704572918868)*x[0]+(0.528208879002)*x[1]**o+(-0.408378325513)*x[1]
        arg[3,1,2,0]=(-0.925177198417)*x[0]**o+(-0.152427811549)*x[0]+(0.341781993583)*x[1]**o+(-0.589976456377)*x[1]
        arg[3,1,2,1]=(0.206706628353)*x[0]**o+(0.673777173329)*x[0]+(-0.994138314254)*x[1]**o+(-0.843696565863)*x[1]
        arg[3,2,0,0]=(-0.588881067333)*x[0]**o+(-0.537572388341)*x[0]+(-0.206570371675)*x[1]**o+(-0.127468554294)*x[1]
        arg[3,2,0,1]=(0.498638284096)*x[0]**o+(-0.488278831086)*x[0]+(-0.28320874594)*x[1]**o+(-0.638241576615)*x[1]
        arg[3,2,1,0]=(-0.445780790822)*x[0]**o+(-0.347493953089)*x[0]+(-0.558010086232)*x[1]**o+(-0.0287339307689)*x[1]
        arg[3,2,1,1]=(-0.361727811272)*x[0]**o+(-0.559565378007)*x[0]+(-0.211685558972)*x[1]**o+(0.330088396811)*x[1]
        arg[3,2,2,0]=(-0.42600117831)*x[0]**o+(-0.488997111988)*x[0]+(0.127421278051)*x[1]**o+(-0.48233167806)*x[1]
        arg[3,2,2,1]=(0.120468829686)*x[0]**o+(0.0675603906118)*x[0]+(-0.60674361864)*x[1]**o+(0.639115708258)*x[1]
        arg[3,3,0,0]=(0.369343033015)*x[0]**o+(0.707735743593)*x[0]+(-0.426275591836)*x[1]**o+(-0.733377765625)*x[1]
        arg[3,3,0,1]=(-0.931245338156)*x[0]**o+(-0.465861794185)*x[0]+(-0.650664500687)*x[1]**o+(0.103077662402)*x[1]
        arg[3,3,1,0]=(0.731834744512)*x[0]**o+(0.267840571071)*x[0]+(0.952106457495)*x[1]**o+(-0.943845364366)*x[1]
        arg[3,3,1,1]=(-0.21692720916)*x[0]**o+(0.580841374997)*x[0]+(-0.412129341223)*x[1]**o+(0.40414445304)*x[1]
        arg[3,3,2,0]=(-0.447758598726)*x[0]**o+(0.0616926352912)*x[0]+(-0.824624226122)*x[1]**o+(0.19099833104)*x[1]
        arg[3,3,2,1]=(-0.615519138165)*x[0]**o+(0.233926822164)*x[0]+(-0.969014060236)*x[1]**o+(0.835103627893)*x[1]
        arg[3,4,0,0]=(0.550778983301)*x[0]**o+(0.0198253530882)*x[0]+(-0.39383849187)*x[1]**o+(0.367721826304)*x[1]
        arg[3,4,0,1]=(0.444877923149)*x[0]**o+(-0.899159835223)*x[0]+(0.923392540452)*x[1]**o+(0.512905141718)*x[1]
        arg[3,4,1,0]=(-0.563807675145)*x[0]**o+(0.994405383899)*x[0]+(-0.777352948218)*x[1]**o+(0.865994906227)*x[1]
        arg[3,4,1,1]=(-0.593800452139)*x[0]**o+(0.613778803237)*x[0]+(0.582200979718)*x[1]**o+(0.448026243271)*x[1]
        arg[3,4,2,0]=(-0.900331125416)*x[0]**o+(0.234613965424)*x[0]+(0.510400273805)*x[1]**o+(0.0836773641661)*x[1]
        arg[3,4,2,1]=(0.710999097307)*x[0]**o+(0.886603988265)*x[0]+(-0.405644887679)*x[1]**o+(0.362257258809)*x[1]
        ref[0,0,0,0]=(-0.0338299192888)*x_ref[0]**o+(-0.114906274359)*x_ref[0]+(0.15010339398)*x_ref[1]**o+(0.625295991328)*x_ref[1]
        ref[0,0,0,1]=(-0.1311901039)*x_ref[0]**o+(-0.551712861217)*x_ref[0]+(-0.287411404506)*x_ref[1]**o+(0.0242820629572)*x_ref[1]
        ref[0,0,1,0]=(0.121962094111)*x_ref[0]**o+(0.203169622345)*x_ref[0]+(0.241504218053)*x_ref[1]**o+(-0.0696356239498)*x_ref[1]
        ref[0,0,1,1]=(0.0461537352329)*x_ref[0]**o+(-0.275908938567)*x_ref[0]+(-0.271184121027)*x_ref[1]**o+(-0.364024729567)*x_ref[1]
        ref[0,0,2,0]=(0.199272234315)*x_ref[0]**o+(-0.906364258855)*x_ref[0]+(-0.92102930037)*x_ref[1]**o+(0.0718981903454)*x_ref[1]
        ref[0,0,2,1]=(-0.742950958383)*x_ref[0]**o+(0.0301464847722)*x_ref[0]+(0.466993477687)*x_ref[1]**o+(-0.402131750698)*x_ref[1]
        ref[0,1,0,0]=(-0.0653776546375)*x_ref[0]**o+(-0.679360612372)*x_ref[0]+(-0.32783837768)*x_ref[1]**o+(-0.113753016428)*x_ref[1]
        ref[0,1,0,1]=(0.58610119414)*x_ref[0]**o+(0.142613166231)*x_ref[0]+(0.0409105619986)*x_ref[1]**o+(-0.660641663558)*x_ref[1]
        ref[0,1,1,0]=(0.212172308585)*x_ref[0]**o+(-0.452767768344)*x_ref[0]+(0.963053969889)*x_ref[1]**o+(0.752691773953)*x_ref[1]
        ref[0,1,1,1]=(-0.514998377887)*x_ref[0]**o+(0.0872464329801)*x_ref[0]+(0.265590295505)*x_ref[1]**o+(0.557238826549)*x_ref[1]
        ref[0,1,2,0]=(0.360523954724)*x_ref[0]**o+(0.550238402994)*x_ref[0]+(-0.206349415539)*x_ref[1]**o+(0.357304730511)*x_ref[1]
        ref[0,1,2,1]=(-0.872755217435)*x_ref[0]**o+(0.190943683664)*x_ref[0]+(-0.245454292354)*x_ref[1]**o+(0.507229578506)*x_ref[1]
        ref[0,2,0,0]=(0.994619505257)*x_ref[0]**o+(0.33482820568)*x_ref[0]+(-0.12721791236)*x_ref[1]**o+(-0.505873171195)*x_ref[1]
        ref[0,2,0,1]=(0.224805131522)*x_ref[0]**o+(0.332022434603)*x_ref[0]+(0.0583494507268)*x_ref[1]**o+(0.857185410994)*x_ref[1]
        ref[0,2,1,0]=(-0.425367077516)*x_ref[0]**o+(-0.25207101061)*x_ref[0]+(-0.11164491753)*x_ref[1]**o+(0.44752084285)*x_ref[1]
        ref[0,2,1,1]=(-0.158725210751)*x_ref[0]**o+(-0.0692363138932)*x_ref[0]+(0.214803128256)*x_ref[1]**o+(0.886699106748)*x_ref[1]
        ref[0,2,2,0]=(-0.332146331257)*x_ref[0]**o+(0.114845050128)*x_ref[0]+(-0.0781411611124)*x_ref[1]**o+(0.474972057387)*x_ref[1]
        ref[0,2,2,1]=(-0.481949630694)*x_ref[0]**o+(0.644984972151)*x_ref[0]+(0.171146519527)*x_ref[1]**o+(0.57534803966)*x_ref[1]
        ref[0,3,0,0]=(0.7580633024)*x_ref[0]**o+(0.0280511776577)*x_ref[0]+(0.375583652078)*x_ref[1]**o+(-0.809112787284)*x_ref[1]
        ref[0,3,0,1]=(-0.466737159934)*x_ref[0]**o+(0.410032775686)*x_ref[0]+(0.176141189584)*x_ref[1]**o+(-0.228288309456)*x_ref[1]
        ref[0,3,1,0]=(0.427129582196)*x_ref[0]**o+(-0.718385859483)*x_ref[0]+(-0.172261212989)*x_ref[1]**o+(0.913157476188)*x_ref[1]
        ref[0,3,1,1]=(0.952506706487)*x_ref[0]**o+(-0.497478163899)*x_ref[0]+(-0.930273038716)*x_ref[1]**o+(0.673573149025)*x_ref[1]
        ref[0,3,2,0]=(-0.0074302930747)*x_ref[0]**o+(0.226424842035)*x_ref[0]+(0.871190620671)*x_ref[1]**o+(0.277695333679)*x_ref[1]
        ref[0,3,2,1]=(-0.540820239946)*x_ref[0]**o+(0.33044763211)*x_ref[0]+(-0.772648820924)*x_ref[1]**o+(-0.362410088021)*x_ref[1]
        ref[0,4,0,0]=(-0.748396039111)*x_ref[0]**o+(0.215131518212)*x_ref[0]+(0.514087824403)*x_ref[1]**o+(0.335563854205)*x_ref[1]
        ref[0,4,0,1]=(-0.443263590391)*x_ref[0]**o+(-0.523167282542)*x_ref[0]+(0.122612452799)*x_ref[1]**o+(0.216527857615)*x_ref[1]
        ref[0,4,1,0]=(0.448176296777)*x_ref[0]**o+(0.478292146131)*x_ref[0]+(-0.558921083502)*x_ref[1]**o+(-0.17018258654)*x_ref[1]
        ref[0,4,1,1]=(-0.912765673953)*x_ref[0]**o+(0.751746623223)*x_ref[0]+(-0.104811579057)*x_ref[1]**o+(0.456436870323)*x_ref[1]
        ref[0,4,2,0]=(0.157946450932)*x_ref[0]**o+(0.147858587857)*x_ref[0]+(0.775747362874)*x_ref[1]**o+(0.478478862463)*x_ref[1]
        ref[0,4,2,1]=(-0.819625058185)*x_ref[0]**o+(-0.0231980393919)*x_ref[0]+(-0.186647585522)*x_ref[1]**o+(0.513350371371)*x_ref[1]
        ref[1,0,0,0]=(-0.942373946995)*x_ref[0]**o+(-0.761326160931)*x_ref[0]+(0.192511867167)*x_ref[1]**o+(-0.50895969579)*x_ref[1]
        ref[1,0,0,1]=(0.0670813349951)*x_ref[0]**o+(0.55602953512)*x_ref[0]+(0.757621178635)*x_ref[1]**o+(0.741550201759)*x_ref[1]
        ref[1,0,1,0]=(-0.114642609493)*x_ref[0]**o+(-0.639500048447)*x_ref[0]+(-0.861813039243)*x_ref[1]**o+(0.36413406744)*x_ref[1]
        ref[1,0,1,1]=(0.126555451594)*x_ref[0]**o+(-0.885172557082)*x_ref[0]+(-0.376073252921)*x_ref[1]**o+(0.166102850653)*x_ref[1]
        ref[1,0,2,0]=(-0.381517068219)*x_ref[0]**o+(0.94870328267)*x_ref[0]+(-0.129934777755)*x_ref[1]**o+(0.78090715758)*x_ref[1]
        ref[1,0,2,1]=(-0.535852302247)*x_ref[0]**o+(0.63751039318)*x_ref[0]+(-0.537556159925)*x_ref[1]**o+(0.147766714221)*x_ref[1]
        ref[1,1,0,0]=(-0.605543047533)*x_ref[0]**o+(-0.366612785372)*x_ref[0]+(0.544900149212)*x_ref[1]**o+(0.722108900447)*x_ref[1]
        ref[1,1,0,1]=(0.133378561142)*x_ref[0]**o+(-0.227808772494)*x_ref[0]+(-0.729399624175)*x_ref[1]**o+(0.307571743455)*x_ref[1]
        ref[1,1,1,0]=(0.0458830538139)*x_ref[0]**o+(0.0585044493026)*x_ref[0]+(-0.136630547605)*x_ref[1]**o+(-0.0823288389309)*x_ref[1]
        ref[1,1,1,1]=(0.339263808784)*x_ref[0]**o+(0.00988917109076)*x_ref[0]+(-0.786591838083)*x_ref[1]**o+(-0.361686799179)*x_ref[1]
        ref[1,1,2,0]=(0.789942037592)*x_ref[0]**o+(-0.266689758676)*x_ref[0]+(0.464856327495)*x_ref[1]**o+(0.138022774293)*x_ref[1]
        ref[1,1,2,1]=(-0.723544867217)*x_ref[0]**o+(-0.523023674205)*x_ref[0]+(0.629726026331)*x_ref[1]**o+(-0.867808120997)*x_ref[1]
        ref[1,2,0,0]=(-0.495589518787)*x_ref[0]**o+(-0.561664880262)*x_ref[0]+(0.0319320417078)*x_ref[1]**o+(-0.153720661498)*x_ref[1]
        ref[1,2,0,1]=(0.293168114024)*x_ref[0]**o+(-0.344107163981)*x_ref[0]+(-0.275629670068)*x_ref[1]**o+(0.06103653487)*x_ref[1]
        ref[1,2,1,0]=(0.495640927339)*x_ref[0]**o+(0.596540095043)*x_ref[0]+(0.208792567756)*x_ref[1]**o+(0.886040724145)*x_ref[1]
        ref[1,2,1,1]=(0.331476398093)*x_ref[0]**o+(-0.472588572455)*x_ref[0]+(-0.455670457285)*x_ref[1]**o+(-0.731223615642)*x_ref[1]
        ref[1,2,2,0]=(-0.127107829674)*x_ref[0]**o+(-0.21113128456)*x_ref[0]+(-0.733928194168)*x_ref[1]**o+(-0.174127696853)*x_ref[1]
        ref[1,2,2,1]=(0.521879316151)*x_ref[0]**o+(0.983424557865)*x_ref[0]+(-0.288659660182)*x_ref[1]**o+(-0.431045203113)*x_ref[1]
        ref[1,3,0,0]=(0.292348765746)*x_ref[0]**o+(0.0730741240915)*x_ref[0]+(0.079624022683)*x_ref[1]**o+(0.237483071633)*x_ref[1]
        ref[1,3,0,1]=(0.291035771848)*x_ref[0]**o+(-0.482909477855)*x_ref[0]+(-0.999142129615)*x_ref[1]**o+(-0.559723367197)*x_ref[1]
        ref[1,3,1,0]=(0.747096275653)*x_ref[0]**o+(0.762757649624)*x_ref[0]+(-0.821650889507)*x_ref[1]**o+(0.301128670189)*x_ref[1]
        ref[1,3,1,1]=(0.82703681414)*x_ref[0]**o+(-0.336570070436)*x_ref[0]+(0.798131952979)*x_ref[1]**o+(0.603562737956)*x_ref[1]
        ref[1,3,2,0]=(-0.164875914126)*x_ref[0]**o+(0.219409873902)*x_ref[0]+(0.784253160678)*x_ref[1]**o+(0.0219597460707)*x_ref[1]
        ref[1,3,2,1]=(0.124778899794)*x_ref[0]**o+(0.785367842145)*x_ref[0]+(0.948111125908)*x_ref[1]**o+(-0.887427847089)*x_ref[1]
        ref[1,4,0,0]=(0.0911816002883)*x_ref[0]**o+(0.645410022441)*x_ref[0]+(0.720551531965)*x_ref[1]**o+(-0.388407683413)*x_ref[1]
        ref[1,4,0,1]=(-0.0416762613062)*x_ref[0]**o+(-0.548874829834)*x_ref[0]+(-0.377952555557)*x_ref[1]**o+(0.742795234361)*x_ref[1]
        ref[1,4,1,0]=(-0.896929017971)*x_ref[0]**o+(0.678343686516)*x_ref[0]+(0.711572763082)*x_ref[1]**o+(-0.43397331651)*x_ref[1]
        ref[1,4,1,1]=(-0.190804421566)*x_ref[0]**o+(-0.0848405124789)*x_ref[0]+(0.36246957905)*x_ref[1]**o+(0.963599086227)*x_ref[1]
        ref[1,4,2,0]=(-0.0210248567982)*x_ref[0]**o+(0.545977814581)*x_ref[0]+(0.858962864401)*x_ref[1]**o+(0.113792851997)*x_ref[1]
        ref[1,4,2,1]=(0.229641892622)*x_ref[0]**o+(-0.739561617198)*x_ref[0]+(-0.164372740617)*x_ref[1]**o+(0.537097868843)*x_ref[1]
        ref[2,0,0,0]=(-0.13321389343)*x_ref[0]**o+(-0.668403350503)*x_ref[0]+(-0.924879105983)*x_ref[1]**o+(-0.5161790717)*x_ref[1]
        ref[2,0,0,1]=(0.26582844685)*x_ref[0]**o+(0.89048660684)*x_ref[0]+(-0.377225981719)*x_ref[1]**o+(-0.303790710194)*x_ref[1]
        ref[2,0,1,0]=(0.0997470860283)*x_ref[0]**o+(0.0385427264217)*x_ref[0]+(-0.71387347273)*x_ref[1]**o+(-0.621376499192)*x_ref[1]
        ref[2,0,1,1]=(0.595544616326)*x_ref[0]**o+(-0.850529503975)*x_ref[0]+(0.602203957938)*x_ref[1]**o+(-0.329484843907)*x_ref[1]
        ref[2,0,2,0]=(-0.106031967029)*x_ref[0]**o+(-0.165122821622)*x_ref[0]+(0.647473815919)*x_ref[1]**o+(0.28942226062)*x_ref[1]
        ref[2,0,2,1]=(0.585955875472)*x_ref[0]**o+(0.922667710403)*x_ref[0]+(-0.0710905957841)*x_ref[1]**o+(-0.669138163697)*x_ref[1]
        ref[2,1,0,0]=(0.461941211081)*x_ref[0]**o+(0.723022466263)*x_ref[0]+(-0.118551893472)*x_ref[1]**o+(0.00445187353637)*x_ref[1]
        ref[2,1,0,1]=(-0.253168248108)*x_ref[0]**o+(0.476938842605)*x_ref[0]+(0.476641658034)*x_ref[1]**o+(-0.916886387595)*x_ref[1]
        ref[2,1,1,0]=(0.722029309503)*x_ref[0]**o+(0.0588848593425)*x_ref[0]+(0.84737370043)*x_ref[1]**o+(-0.674604415213)*x_ref[1]
        ref[2,1,1,1]=(-0.644736759298)*x_ref[0]**o+(-0.212847303893)*x_ref[0]+(0.25394587552)*x_ref[1]**o+(-0.366955175081)*x_ref[1]
        ref[2,1,2,0]=(-0.657395296801)*x_ref[0]**o+(-0.967570027599)*x_ref[0]+(0.180926628541)*x_ref[1]**o+(-0.940599897942)*x_ref[1]
        ref[2,1,2,1]=(0.849897328801)*x_ref[0]**o+(-0.299199474051)*x_ref[0]+(0.427221207827)*x_ref[1]**o+(0.212027985441)*x_ref[1]
        ref[2,2,0,0]=(-0.561573507381)*x_ref[0]**o+(-0.446925224908)*x_ref[0]+(0.132091224948)*x_ref[1]**o+(-0.673375661849)*x_ref[1]
        ref[2,2,0,1]=(-0.386657291574)*x_ref[0]**o+(0.356784637112)*x_ref[0]+(0.644292006233)*x_ref[1]**o+(0.922937167842)*x_ref[1]
        ref[2,2,1,0]=(0.484446757285)*x_ref[0]**o+(-0.822469870455)*x_ref[0]+(-0.89605890123)*x_ref[1]**o+(-0.895891500031)*x_ref[1]
        ref[2,2,1,1]=(-0.0930740520192)*x_ref[0]**o+(0.953356473678)*x_ref[0]+(-0.714418458806)*x_ref[1]**o+(-0.100584950946)*x_ref[1]
        ref[2,2,2,0]=(0.945572242667)*x_ref[0]**o+(-0.259280849605)*x_ref[0]+(-0.925459846341)*x_ref[1]**o+(0.906002758873)*x_ref[1]
        ref[2,2,2,1]=(-0.638993332975)*x_ref[0]**o+(-0.668594641703)*x_ref[0]+(0.012666231369)*x_ref[1]**o+(0.89732685856)*x_ref[1]
        ref[2,3,0,0]=(-0.885061116144)*x_ref[0]**o+(-0.576097536483)*x_ref[0]+(-0.43166075788)*x_ref[1]**o+(0.538821620917)*x_ref[1]
        ref[2,3,0,1]=(-0.0743638381797)*x_ref[0]**o+(0.251037500035)*x_ref[0]+(-0.905989928076)*x_ref[1]**o+(0.243771000867)*x_ref[1]
        ref[2,3,1,0]=(-0.016045622351)*x_ref[0]**o+(-0.632944072174)*x_ref[0]+(-0.296175918793)*x_ref[1]**o+(-0.00120123641365)*x_ref[1]
        ref[2,3,1,1]=(-0.896354921812)*x_ref[0]**o+(-0.331348989027)*x_ref[0]+(-0.450865285201)*x_ref[1]**o+(-0.243268582009)*x_ref[1]
        ref[2,3,2,0]=(0.609332927431)*x_ref[0]**o+(-0.0815856802115)*x_ref[0]+(0.201924792273)*x_ref[1]**o+(-0.890424451348)*x_ref[1]
        ref[2,3,2,1]=(0.395090137264)*x_ref[0]**o+(0.294230442573)*x_ref[0]+(-0.345371510364)*x_ref[1]**o+(0.181229663241)*x_ref[1]
        ref[2,4,0,0]=(-0.823684509034)*x_ref[0]**o+(0.431019301942)*x_ref[0]+(0.554707015054)*x_ref[1]**o+(-0.846322903003)*x_ref[1]
        ref[2,4,0,1]=(0.862984476658)*x_ref[0]**o+(0.20512500102)*x_ref[0]+(0.0247756159287)*x_ref[1]**o+(-0.374795100946)*x_ref[1]
        ref[2,4,1,0]=(-0.0849914269517)*x_ref[0]**o+(0.754908200339)*x_ref[0]+(0.593363754807)*x_ref[1]**o+(0.124923961126)*x_ref[1]
        ref[2,4,1,1]=(-0.691440264234)*x_ref[0]**o+(0.825739298015)*x_ref[0]+(0.426452231478)*x_ref[1]**o+(0.0995610721341)*x_ref[1]
        ref[2,4,2,0]=(-0.175322750567)*x_ref[0]**o+(0.419750865569)*x_ref[0]+(0.208448987712)*x_ref[1]**o+(0.150584798952)*x_ref[1]
        ref[2,4,2,1]=(0.931680152344)*x_ref[0]**o+(0.390947206307)*x_ref[0]+(0.388437348378)*x_ref[1]**o+(-0.808413079032)*x_ref[1]
        ref[3,0,0,0]=(0.58855431402)*x_ref[0]**o+(0.383865913364)*x_ref[0]+(-0.691445323219)*x_ref[1]**o+(-0.443370658523)*x_ref[1]
        ref[3,0,0,1]=(-0.657371481478)*x_ref[0]**o+(0.746891082673)*x_ref[0]+(-0.829229947773)*x_ref[1]**o+(0.761915670404)*x_ref[1]
        ref[3,0,1,0]=(-0.431502932714)*x_ref[0]**o+(-0.961472285716)*x_ref[0]+(0.0423384388031)*x_ref[1]**o+(0.339373458118)*x_ref[1]
        ref[3,0,1,1]=(-0.272831163259)*x_ref[0]**o+(-0.533438904494)*x_ref[0]+(-0.595625384875)*x_ref[1]**o+(0.0366857022323)*x_ref[1]
        ref[3,0,2,0]=(-0.319684662306)*x_ref[0]**o+(0.672185828912)*x_ref[0]+(0.483126548994)*x_ref[1]**o+(0.745872629604)*x_ref[1]
        ref[3,0,2,1]=(0.437895277404)*x_ref[0]**o+(-0.337209416963)*x_ref[0]+(0.101080522788)*x_ref[1]**o+(0.892022488644)*x_ref[1]
        ref[3,1,0,0]=(-0.582325078455)*x_ref[0]**o+(-0.854689347157)*x_ref[0]+(0.0308638312259)*x_ref[1]**o+(0.633382365704)*x_ref[1]
        ref[3,1,0,1]=(0.319735178376)*x_ref[0]**o+(-0.782764593952)*x_ref[0]+(0.082230132244)*x_ref[1]**o+(-0.946010937852)*x_ref[1]
        ref[3,1,1,0]=(-0.698490410666)*x_ref[0]**o+(-0.801358802689)*x_ref[0]+(0.20695648235)*x_ref[1]**o+(0.75257523471)*x_ref[1]
        ref[3,1,1,1]=(0.66321696527)*x_ref[0]**o+(0.704572918868)*x_ref[0]+(0.528208879002)*x_ref[1]**o+(-0.408378325513)*x_ref[1]
        ref[3,1,2,0]=(-0.925177198417)*x_ref[0]**o+(-0.152427811549)*x_ref[0]+(0.341781993583)*x_ref[1]**o+(-0.589976456377)*x_ref[1]
        ref[3,1,2,1]=(0.206706628353)*x_ref[0]**o+(0.673777173329)*x_ref[0]+(-0.994138314254)*x_ref[1]**o+(-0.843696565863)*x_ref[1]
        ref[3,2,0,0]=(-0.588881067333)*x_ref[0]**o+(-0.537572388341)*x_ref[0]+(-0.206570371675)*x_ref[1]**o+(-0.127468554294)*x_ref[1]
        ref[3,2,0,1]=(0.498638284096)*x_ref[0]**o+(-0.488278831086)*x_ref[0]+(-0.28320874594)*x_ref[1]**o+(-0.638241576615)*x_ref[1]
        ref[3,2,1,0]=(-0.445780790822)*x_ref[0]**o+(-0.347493953089)*x_ref[0]+(-0.558010086232)*x_ref[1]**o+(-0.0287339307689)*x_ref[1]
        ref[3,2,1,1]=(-0.361727811272)*x_ref[0]**o+(-0.559565378007)*x_ref[0]+(-0.211685558972)*x_ref[1]**o+(0.330088396811)*x_ref[1]
        ref[3,2,2,0]=(-0.42600117831)*x_ref[0]**o+(-0.488997111988)*x_ref[0]+(0.127421278051)*x_ref[1]**o+(-0.48233167806)*x_ref[1]
        ref[3,2,2,1]=(0.120468829686)*x_ref[0]**o+(0.0675603906118)*x_ref[0]+(-0.60674361864)*x_ref[1]**o+(0.639115708258)*x_ref[1]
        ref[3,3,0,0]=(0.369343033015)*x_ref[0]**o+(0.707735743593)*x_ref[0]+(-0.426275591836)*x_ref[1]**o+(-0.733377765625)*x_ref[1]
        ref[3,3,0,1]=(-0.931245338156)*x_ref[0]**o+(-0.465861794185)*x_ref[0]+(-0.650664500687)*x_ref[1]**o+(0.103077662402)*x_ref[1]
        ref[3,3,1,0]=(0.731834744512)*x_ref[0]**o+(0.267840571071)*x_ref[0]+(0.952106457495)*x_ref[1]**o+(-0.943845364366)*x_ref[1]
        ref[3,3,1,1]=(-0.21692720916)*x_ref[0]**o+(0.580841374997)*x_ref[0]+(-0.412129341223)*x_ref[1]**o+(0.40414445304)*x_ref[1]
        ref[3,3,2,0]=(-0.447758598726)*x_ref[0]**o+(0.0616926352912)*x_ref[0]+(-0.824624226122)*x_ref[1]**o+(0.19099833104)*x_ref[1]
        ref[3,3,2,1]=(-0.615519138165)*x_ref[0]**o+(0.233926822164)*x_ref[0]+(-0.969014060236)*x_ref[1]**o+(0.835103627893)*x_ref[1]
        ref[3,4,0,0]=(0.550778983301)*x_ref[0]**o+(0.0198253530882)*x_ref[0]+(-0.39383849187)*x_ref[1]**o+(0.367721826304)*x_ref[1]
        ref[3,4,0,1]=(0.444877923149)*x_ref[0]**o+(-0.899159835223)*x_ref[0]+(0.923392540452)*x_ref[1]**o+(0.512905141718)*x_ref[1]
        ref[3,4,1,0]=(-0.563807675145)*x_ref[0]**o+(0.994405383899)*x_ref[0]+(-0.777352948218)*x_ref[1]**o+(0.865994906227)*x_ref[1]
        ref[3,4,1,1]=(-0.593800452139)*x_ref[0]**o+(0.613778803237)*x_ref[0]+(0.582200979718)*x_ref[1]**o+(0.448026243271)*x_ref[1]
        ref[3,4,2,0]=(-0.900331125416)*x_ref[0]**o+(0.234613965424)*x_ref[0]+(0.510400273805)*x_ref[1]**o+(0.0836773641661)*x_ref[1]
        ref[3,4,2,1]=(0.710999097307)*x_ref[0]**o+(0.886603988265)*x_ref[0]+(-0.405644887679)*x_ref[1]**o+(0.362257258809)*x_ref[1]
      else:
        arg[0,0,0,0]=(0.310272924207)*x[0]**o+(-0.664497806351)*x[0]+(0.741942132609)*x[1]**o+(0.636051298779)*x[1]+(0.76627159146)*x[2]**o+(0.492960732588)*x[2]
        arg[0,0,0,1]=(-0.513740801773)*x[0]**o+(-0.633330750909)*x[0]+(0.177934618983)*x[1]**o+(-0.323745723166)*x[1]+(0.694167180904)*x[2]**o+(0.441239251435)*x[2]
        arg[0,0,1,0]=(0.417477695848)*x[0]**o+(-0.923415976002)*x[0]+(0.0765555276662)*x[1]**o+(-0.277290890834)*x[1]+(0.0177191326614)*x[2]**o+(-0.0842520100387)*x[2]
        arg[0,0,1,1]=(-0.761916249894)*x[0]**o+(0.169673139552)*x[0]+(0.0306290828795)*x[1]**o+(0.842234592613)*x[1]+(-0.350342692669)*x[2]**o+(0.321292270658)*x[2]
        arg[0,0,2,0]=(-0.707494137624)*x[0]**o+(0.346873490395)*x[0]+(-0.68281925138)*x[1]**o+(-0.830538083274)*x[1]+(0.329372141613)*x[2]**o+(-0.121366157163)*x[2]
        arg[0,0,2,1]=(-0.708018186507)*x[0]**o+(-0.778051239314)*x[0]+(-0.43106291563)*x[1]**o+(-0.0115914251143)*x[1]+(0.487334215892)*x[2]**o+(-0.289232284658)*x[2]
        arg[0,1,0,0]=(0.530614704199)*x[0]**o+(0.518622028317)*x[0]+(-0.0143129589551)*x[1]**o+(0.808469352257)*x[1]+(-0.156787373225)*x[2]**o+(0.668887813143)*x[2]
        arg[0,1,0,1]=(0.953223233706)*x[0]**o+(-0.164323838353)*x[0]+(0.336717320858)*x[1]**o+(-0.520230488172)*x[1]+(0.689960738002)*x[2]**o+(0.487630947578)*x[2]
        arg[0,1,1,0]=(-0.418095373429)*x[0]**o+(-0.941817844939)*x[0]+(0.88105115607)*x[1]**o+(-0.820502501439)*x[1]+(-0.693858267667)*x[2]**o+(0.530567713127)*x[2]
        arg[0,1,1,1]=(-0.716861320218)*x[0]**o+(-0.736619859467)*x[0]+(-0.885478605601)*x[1]**o+(-0.129375578967)*x[1]+(-0.0181433907318)*x[2]**o+(0.373255537959)*x[2]
        arg[0,1,2,0]=(-0.174742583167)*x[0]**o+(-0.0887967480112)*x[0]+(-0.291168698662)*x[1]**o+(-0.188608506769)*x[1]+(0.22479613454)*x[2]**o+(0.34173928582)*x[2]
        arg[0,1,2,1]=(0.117208929504)*x[0]**o+(-0.420969282079)*x[0]+(0.585298109836)*x[1]**o+(0.771355162745)*x[1]+(-0.51374493747)*x[2]**o+(0.785016548855)*x[2]
        arg[0,2,0,0]=(0.737394887886)*x[0]**o+(0.966080271342)*x[0]+(-0.373178384492)*x[1]**o+(-0.845505386142)*x[1]+(-0.274654519371)*x[2]**o+(0.709356557887)*x[2]
        arg[0,2,0,1]=(0.0365864090636)*x[0]**o+(-0.393382479787)*x[0]+(0.018273360977)*x[1]**o+(-0.509201909328)*x[1]+(-0.777852392534)*x[2]**o+(0.0417280247507)*x[2]
        arg[0,2,1,0]=(0.637829302751)*x[0]**o+(-0.767824790186)*x[0]+(0.843382009511)*x[1]**o+(-0.599633234403)*x[1]+(0.14283138941)*x[2]**o+(-0.354404633991)*x[2]
        arg[0,2,1,1]=(-0.465258613599)*x[0]**o+(0.09341154383)*x[0]+(-0.629114782988)*x[1]**o+(0.91608655383)*x[1]+(-0.692901135848)*x[2]**o+(-0.707225310667)*x[2]
        arg[0,2,2,0]=(0.9935958759)*x[0]**o+(-0.385008742466)*x[0]+(0.153885356854)*x[1]**o+(0.146270680656)*x[1]+(0.305838410343)*x[2]**o+(0.141193552717)*x[2]
        arg[0,2,2,1]=(-0.487538023894)*x[0]**o+(-0.814283442647)*x[0]+(0.166008520199)*x[1]**o+(0.313655774416)*x[1]+(0.282308782973)*x[2]**o+(0.309176062484)*x[2]
        arg[0,3,0,0]=(0.00203507798913)*x[0]**o+(-0.19902542604)*x[0]+(0.88110918624)*x[1]**o+(0.223549219288)*x[1]+(0.135758952369)*x[2]**o+(0.59116783639)*x[2]
        arg[0,3,0,1]=(-0.185063700767)*x[0]**o+(0.833862155173)*x[0]+(-0.0939117509645)*x[1]**o+(0.288003692231)*x[1]+(0.580196157839)*x[2]**o+(0.299320177754)*x[2]
        arg[0,3,1,0]=(-0.039100718826)*x[0]**o+(0.423179869395)*x[0]+(-0.013669578975)*x[1]**o+(-0.355053343085)*x[1]+(0.89285158975)*x[2]**o+(0.392984367877)*x[2]
        arg[0,3,1,1]=(0.0988989416429)*x[0]**o+(-0.446243235174)*x[0]+(-0.691253313617)*x[1]**o+(0.338823693086)*x[1]+(0.707054126334)*x[2]**o+(0.601121388278)*x[2]
        arg[0,3,2,0]=(-0.191366770484)*x[0]**o+(0.961791275903)*x[0]+(-0.103547475035)*x[1]**o+(-0.745236510079)*x[1]+(-0.157506981712)*x[2]**o+(0.986229506787)*x[2]
        arg[0,3,2,1]=(-0.886091883647)*x[0]**o+(0.6979804248)*x[0]+(-0.366575419428)*x[1]**o+(-0.958601153259)*x[1]+(-0.0119724511494)*x[2]**o+(-0.76101132076)*x[2]
        arg[0,4,0,0]=(0.923290554434)*x[0]**o+(0.411359492198)*x[0]+(-0.977999562875)*x[1]**o+(0.352085161527)*x[1]+(0.955745235349)*x[2]**o+(-0.978885609257)*x[2]
        arg[0,4,0,1]=(0.226973479105)*x[0]**o+(-0.423881897251)*x[0]+(0.889101969481)*x[1]**o+(-0.994236389028)*x[1]+(0.0159169033361)*x[2]**o+(-0.559935154203)*x[2]
        arg[0,4,1,0]=(-0.225427291797)*x[0]**o+(-0.557334621848)*x[0]+(-0.486252586142)*x[1]**o+(0.0497120905216)*x[1]+(0.84165736691)*x[2]**o+(-0.185010156797)*x[2]
        arg[0,4,1,1]=(0.762694602609)*x[0]**o+(-0.447367170727)*x[0]+(-0.61103181818)*x[1]**o+(0.689761057656)*x[1]+(-0.631855120122)*x[2]**o+(0.33274655602)*x[2]
        arg[0,4,2,0]=(-0.198884868914)*x[0]**o+(0.716371959278)*x[0]+(0.260871768919)*x[1]**o+(0.622241740082)*x[1]+(0.774103524035)*x[2]**o+(0.186132703676)*x[2]
        arg[0,4,2,1]=(-0.309450645317)*x[0]**o+(0.204754571336)*x[0]+(0.483402810171)*x[1]**o+(0.404620744188)*x[1]+(-0.699137246959)*x[2]**o+(0.467428706196)*x[2]
        arg[1,0,0,0]=(0.642289506448)*x[0]**o+(0.383990304394)*x[0]+(0.556940973318)*x[1]**o+(0.135219729041)*x[1]+(0.577195836358)*x[2]**o+(-0.338924419674)*x[2]
        arg[1,0,0,1]=(-0.920873911295)*x[0]**o+(-0.330610192304)*x[0]+(-0.117226803856)*x[1]**o+(0.172482867684)*x[1]+(0.909140902709)*x[2]**o+(-0.590494167913)*x[2]
        arg[1,0,1,0]=(0.210161538861)*x[0]**o+(-0.83860241761)*x[0]+(0.0257180382661)*x[1]**o+(0.877633854348)*x[1]+(0.26045311754)*x[2]**o+(-0.971349902714)*x[2]
        arg[1,0,1,1]=(0.70736843125)*x[0]**o+(-0.765121985431)*x[0]+(-0.196525225943)*x[1]**o+(-0.175928729346)*x[1]+(0.945523028748)*x[2]**o+(0.877830322823)*x[2]
        arg[1,0,2,0]=(-0.963513016065)*x[0]**o+(0.0820537856351)*x[0]+(0.0122070196118)*x[1]**o+(-0.154147449592)*x[1]+(0.323844633268)*x[2]**o+(-0.124897698214)*x[2]
        arg[1,0,2,1]=(0.656267566405)*x[0]**o+(0.0174786620768)*x[0]+(0.763902435618)*x[1]**o+(-0.986021506043)*x[1]+(0.632625555994)*x[2]**o+(-0.110657743523)*x[2]
        arg[1,1,0,0]=(0.00354144688732)*x[0]**o+(-0.813515808171)*x[0]+(-0.100717717446)*x[1]**o+(0.0543708163629)*x[1]+(0.785927880762)*x[2]**o+(0.0109166652227)*x[2]
        arg[1,1,0,1]=(-0.239705688631)*x[0]**o+(0.402915642509)*x[0]+(0.463371068163)*x[1]**o+(-0.507302462928)*x[1]+(-0.0789844128295)*x[2]**o+(0.669144891499)*x[2]
        arg[1,1,1,0]=(-0.491891367001)*x[0]**o+(-0.524377636928)*x[0]+(0.903091863917)*x[1]**o+(-0.554368807183)*x[1]+(0.332899195339)*x[2]**o+(-0.645216147049)*x[2]
        arg[1,1,1,1]=(-0.915843941243)*x[0]**o+(-0.233175654748)*x[0]+(0.467730425037)*x[1]**o+(-0.765957936683)*x[1]+(0.726678877276)*x[2]**o+(0.770533947942)*x[2]
        arg[1,1,2,0]=(0.678071852032)*x[0]**o+(0.266537844237)*x[0]+(0.324335300151)*x[1]**o+(-0.286713971373)*x[1]+(-0.702384414014)*x[2]**o+(0.887198276519)*x[2]
        arg[1,1,2,1]=(0.0466444959648)*x[0]**o+(0.689162912677)*x[0]+(-0.35033923676)*x[1]**o+(-0.0536380766876)*x[1]+(-0.511833442517)*x[2]**o+(0.125566299829)*x[2]
        arg[1,2,0,0]=(-0.803170431622)*x[0]**o+(0.476211169752)*x[0]+(-0.170510267953)*x[1]**o+(-0.765945735317)*x[1]+(-0.169966329291)*x[2]**o+(-0.988525918724)*x[2]
        arg[1,2,0,1]=(0.289618751212)*x[0]**o+(0.294379040976)*x[0]+(-0.922478416838)*x[1]**o+(0.210546495537)*x[1]+(-0.87266381496)*x[2]**o+(-0.865571669577)*x[2]
        arg[1,2,1,0]=(-0.292811054005)*x[0]**o+(0.651280833746)*x[0]+(-0.625004203736)*x[1]**o+(0.372078169172)*x[1]+(-0.0539099055169)*x[2]**o+(0.0911869270008)*x[2]
        arg[1,2,1,1]=(0.496217698294)*x[0]**o+(-0.986021787133)*x[0]+(0.246417059532)*x[1]**o+(0.610226802401)*x[1]+(0.991534665298)*x[2]**o+(-0.507441185982)*x[2]
        arg[1,2,2,0]=(-0.839902565947)*x[0]**o+(-0.279607147278)*x[0]+(0.278212307963)*x[1]**o+(0.609793640036)*x[1]+(0.940471913774)*x[2]**o+(0.21860828705)*x[2]
        arg[1,2,2,1]=(0.789578112278)*x[0]**o+(-0.0226697818608)*x[0]+(0.216321340291)*x[1]**o+(-0.701818463454)*x[1]+(-0.166689030646)*x[2]**o+(0.966131542292)*x[2]
        arg[1,3,0,0]=(0.149337042551)*x[0]**o+(-0.785050579674)*x[0]+(0.530219924721)*x[1]**o+(-0.971927239398)*x[1]+(0.950917908603)*x[2]**o+(0.132668077447)*x[2]
        arg[1,3,0,1]=(0.983081559724)*x[0]**o+(-0.358694498455)*x[0]+(0.181080099281)*x[1]**o+(-0.699756291643)*x[1]+(-0.744335953304)*x[2]**o+(0.00262043221382)*x[2]
        arg[1,3,1,0]=(0.31776032045)*x[0]**o+(-0.0726184220029)*x[0]+(-0.759883735971)*x[1]**o+(-0.6389710873)*x[1]+(-0.607858499502)*x[2]**o+(-0.397169895784)*x[2]
        arg[1,3,1,1]=(-0.950468389728)*x[0]**o+(0.831121915521)*x[0]+(0.552857011083)*x[1]**o+(0.848599672594)*x[1]+(0.432183564231)*x[2]**o+(0.57537721362)*x[2]
        arg[1,3,2,0]=(0.736080143601)*x[0]**o+(-0.804140307747)*x[0]+(-0.163864867884)*x[1]**o+(0.950874042315)*x[1]+(-0.99676139412)*x[2]**o+(0.332320294246)*x[2]
        arg[1,3,2,1]=(0.119082582041)*x[0]**o+(-0.969740778852)*x[0]+(0.828432583532)*x[1]**o+(0.190324050395)*x[1]+(-0.42639142674)*x[2]**o+(0.315519578192)*x[2]
        arg[1,4,0,0]=(-0.798465141775)*x[0]**o+(-0.717282977908)*x[0]+(0.935655148713)*x[1]**o+(-0.655752964484)*x[1]+(0.159990149099)*x[2]**o+(-0.537115062687)*x[2]
        arg[1,4,0,1]=(0.445151775007)*x[0]**o+(-0.867956302051)*x[0]+(-0.809856879825)*x[1]**o+(-0.326897750981)*x[1]+(0.756352311987)*x[2]**o+(0.577585513459)*x[2]
        arg[1,4,1,0]=(0.294243347591)*x[0]**o+(0.774741656931)*x[0]+(-0.0799985737984)*x[1]**o+(0.179789107412)*x[1]+(-0.612198025189)*x[2]**o+(-0.07086009611)*x[2]
        arg[1,4,1,1]=(0.0692586287045)*x[0]**o+(-0.671956300122)*x[0]+(-0.948667104683)*x[1]**o+(0.68656246801)*x[1]+(-0.352975297696)*x[2]**o+(0.277952113716)*x[2]
        arg[1,4,2,0]=(-0.491316095517)*x[0]**o+(0.725899087239)*x[0]+(0.634113963431)*x[1]**o+(0.235694393453)*x[1]+(-0.15600720713)*x[2]**o+(0.954814976539)*x[2]
        arg[1,4,2,1]=(0.120206996339)*x[0]**o+(-0.86151411823)*x[0]+(-0.137093650097)*x[1]**o+(0.633402978246)*x[1]+(-0.42515364728)*x[2]**o+(-0.987634210617)*x[2]
        arg[2,0,0,0]=(0.539463971222)*x[0]**o+(-0.03435650579)*x[0]+(-0.884395295544)*x[1]**o+(0.215454879022)*x[1]+(0.842377328246)*x[2]**o+(0.370920136871)*x[2]
        arg[2,0,0,1]=(-0.411505463039)*x[0]**o+(0.446494039439)*x[0]+(0.0313771969556)*x[1]**o+(0.602893223061)*x[1]+(-0.30383490021)*x[2]**o+(-0.788028756758)*x[2]
        arg[2,0,1,0]=(0.489009615999)*x[0]**o+(-0.95508248016)*x[0]+(-0.21489475022)*x[1]**o+(-0.857263478431)*x[1]+(-0.123703409045)*x[2]**o+(0.663389562431)*x[2]
        arg[2,0,1,1]=(-0.0224763310929)*x[0]**o+(-0.315886249237)*x[0]+(-0.318201849695)*x[1]**o+(0.423732682462)*x[1]+(0.396065636381)*x[2]**o+(0.0816717878865)*x[2]
        arg[2,0,2,0]=(-0.191432618224)*x[0]**o+(0.688389390795)*x[0]+(-0.382948112159)*x[1]**o+(0.562905173818)*x[1]+(-0.118396714029)*x[2]**o+(0.234478642577)*x[2]
        arg[2,0,2,1]=(-0.348008173171)*x[0]**o+(0.191994166636)*x[0]+(0.450556057147)*x[1]**o+(-0.710821693343)*x[1]+(-0.245119333857)*x[2]**o+(0.262869525239)*x[2]
        arg[2,1,0,0]=(-0.160444506203)*x[0]**o+(-0.816142837432)*x[0]+(-0.619722480492)*x[1]**o+(-0.989077039675)*x[1]+(-0.0282349103977)*x[2]**o+(-0.629849819691)*x[2]
        arg[2,1,0,1]=(0.379513765297)*x[0]**o+(0.934652105586)*x[0]+(-0.632636427868)*x[1]**o+(-0.0492550496553)*x[1]+(0.137733572307)*x[2]**o+(0.779782637942)*x[2]
        arg[2,1,1,0]=(0.469499320417)*x[0]**o+(0.570880924125)*x[0]+(-0.0329833251419)*x[1]**o+(-0.0388751034604)*x[1]+(-0.878520094389)*x[2]**o+(0.992656649048)*x[2]
        arg[2,1,1,1]=(0.592005181995)*x[0]**o+(-0.553980493526)*x[0]+(0.359837665019)*x[1]**o+(0.811107941179)*x[1]+(-0.441929650912)*x[2]**o+(-0.768618378636)*x[2]
        arg[2,1,2,0]=(0.741376258816)*x[0]**o+(-0.238909756717)*x[0]+(0.204631082449)*x[1]**o+(0.970501662949)*x[1]+(-0.615638092239)*x[2]**o+(-0.575948750844)*x[2]
        arg[2,1,2,1]=(0.774050933112)*x[0]**o+(-0.872234076721)*x[0]+(-0.0936189240061)*x[1]**o+(-0.773608756961)*x[1]+(-0.119171907808)*x[2]**o+(-0.256124796845)*x[2]
        arg[2,2,0,0]=(0.459481754425)*x[0]**o+(0.478364929493)*x[0]+(0.460199733287)*x[1]**o+(0.66880294659)*x[1]+(-0.859525058389)*x[2]**o+(0.77960284414)*x[2]
        arg[2,2,0,1]=(0.690557835207)*x[0]**o+(-0.812305038592)*x[0]+(0.837924124791)*x[1]**o+(-0.785979206364)*x[1]+(-0.295553949819)*x[2]**o+(0.575096295897)*x[2]
        arg[2,2,1,0]=(0.443193683205)*x[0]**o+(0.0355033222253)*x[0]+(0.497641002766)*x[1]**o+(-0.636955142451)*x[1]+(-0.265324357454)*x[2]**o+(0.734148569078)*x[2]
        arg[2,2,1,1]=(-0.088707242599)*x[0]**o+(0.513403518669)*x[0]+(0.337741737651)*x[1]**o+(-0.174843924478)*x[1]+(0.687291255986)*x[2]**o+(0.473850502258)*x[2]
        arg[2,2,2,0]=(0.5305624642)*x[0]**o+(-0.972997835953)*x[0]+(0.00197107287888)*x[1]**o+(-0.75490872634)*x[1]+(-0.0569624569435)*x[2]**o+(-0.0780532868631)*x[2]
        arg[2,2,2,1]=(-0.744584471866)*x[0]**o+(0.915494909272)*x[0]+(0.239235050038)*x[1]**o+(-0.192466753187)*x[1]+(0.933556550824)*x[2]**o+(-0.786420251487)*x[2]
        arg[2,3,0,0]=(0.472114530627)*x[0]**o+(0.0693211807557)*x[0]+(-0.274273425713)*x[1]**o+(0.136242554894)*x[1]+(-0.329240675785)*x[2]**o+(0.0212539247343)*x[2]
        arg[2,3,0,1]=(-0.780426108099)*x[0]**o+(-0.632382551025)*x[0]+(0.554926949239)*x[1]**o+(0.264204009059)*x[1]+(-0.868007525824)*x[2]**o+(-0.937631120201)*x[2]
        arg[2,3,1,0]=(-0.826692080947)*x[0]**o+(0.915241858538)*x[0]+(-0.49986160859)*x[1]**o+(0.921686464658)*x[1]+(0.71996666304)*x[2]**o+(0.297609501096)*x[2]
        arg[2,3,1,1]=(0.991896994614)*x[0]**o+(-0.543310008103)*x[0]+(0.0791899631111)*x[1]**o+(-0.914088200145)*x[1]+(-0.496337624951)*x[2]**o+(0.708950898614)*x[2]
        arg[2,3,2,0]=(-0.235116966432)*x[0]**o+(-0.492848730207)*x[0]+(0.428728646789)*x[1]**o+(-0.410747634691)*x[1]+(0.705059815458)*x[2]**o+(-0.185604633831)*x[2]
        arg[2,3,2,1]=(0.838829037117)*x[0]**o+(0.670561147563)*x[0]+(0.621407965075)*x[1]**o+(-0.10624946305)*x[1]+(0.893703402687)*x[2]**o+(0.985504995821)*x[2]
        arg[2,4,0,0]=(0.635072504682)*x[0]**o+(-0.681962427555)*x[0]+(-0.00774518011081)*x[1]**o+(-0.264996228668)*x[1]+(0.690668403198)*x[2]**o+(0.883573761062)*x[2]
        arg[2,4,0,1]=(0.545632268651)*x[0]**o+(0.439668519394)*x[0]+(-0.482479808491)*x[1]**o+(0.752028962391)*x[1]+(0.0596773623939)*x[2]**o+(-0.369987181127)*x[2]
        arg[2,4,1,0]=(0.659005322796)*x[0]**o+(0.503940833428)*x[0]+(0.643633321586)*x[1]**o+(0.927082323252)*x[1]+(0.354072146184)*x[2]**o+(-0.638737560608)*x[2]
        arg[2,4,1,1]=(-0.443548860913)*x[0]**o+(0.707607048458)*x[0]+(-0.955225746402)*x[1]**o+(0.515995563585)*x[1]+(-0.288406637165)*x[2]**o+(-0.760864937318)*x[2]
        arg[2,4,2,0]=(0.898146580355)*x[0]**o+(-0.590650787106)*x[0]+(0.036095789753)*x[1]**o+(0.326040260581)*x[1]+(0.680069361306)*x[2]**o+(0.333839099341)*x[2]
        arg[2,4,2,1]=(-0.151364260271)*x[0]**o+(-0.675198183832)*x[0]+(-0.183626298978)*x[1]**o+(-0.182565217146)*x[1]+(0.0345632129814)*x[2]**o+(-0.581534100763)*x[2]
        arg[3,0,0,0]=(-0.942351703246)*x[0]**o+(-0.699567649453)*x[0]+(-0.832068665702)*x[1]**o+(0.163439416504)*x[1]+(0.742500339519)*x[2]**o+(-0.492882887489)*x[2]
        arg[3,0,0,1]=(0.990078296484)*x[0]**o+(-0.999837336587)*x[0]+(0.197589456459)*x[1]**o+(0.431403059019)*x[1]+(0.859864015871)*x[2]**o+(-0.267474829282)*x[2]
        arg[3,0,1,0]=(-0.776773660723)*x[0]**o+(-0.189048370486)*x[0]+(0.926953856802)*x[1]**o+(-0.72314744652)*x[1]+(0.330064363809)*x[2]**o+(0.188533334359)*x[2]
        arg[3,0,1,1]=(-0.854830058762)*x[0]**o+(0.832073883276)*x[0]+(-0.425691165124)*x[1]**o+(0.162351500316)*x[1]+(0.370604864512)*x[2]**o+(-0.581907071484)*x[2]
        arg[3,0,2,0]=(-0.316716909548)*x[0]**o+(0.335417284383)*x[0]+(-0.783061160352)*x[1]**o+(-0.253972519239)*x[1]+(-0.826432933721)*x[2]**o+(0.267368378363)*x[2]
        arg[3,0,2,1]=(-0.765899705462)*x[0]**o+(-0.0993875861476)*x[0]+(-0.211785004352)*x[1]**o+(-0.635330151497)*x[1]+(0.414575412599)*x[2]**o+(0.586882274623)*x[2]
        arg[3,1,0,0]=(0.987776377237)*x[0]**o+(0.796660669191)*x[0]+(0.580724891862)*x[1]**o+(-0.24149253995)*x[1]+(-0.633421016909)*x[2]**o+(-0.220110913711)*x[2]
        arg[3,1,0,1]=(-0.182240918081)*x[0]**o+(-0.290526050572)*x[0]+(0.00283230959976)*x[1]**o+(-0.933282166805)*x[1]+(-0.857893235883)*x[2]**o+(0.551414074127)*x[2]
        arg[3,1,1,0]=(-0.854435997816)*x[0]**o+(0.706455144263)*x[0]+(0.406237965254)*x[1]**o+(0.192071142415)*x[1]+(0.357966796834)*x[2]**o+(0.887011087987)*x[2]
        arg[3,1,1,1]=(0.832013396996)*x[0]**o+(0.696133072321)*x[0]+(0.682726968712)*x[1]**o+(0.613316240514)*x[1]+(0.12600339887)*x[2]**o+(-0.620088231382)*x[2]
        arg[3,1,2,0]=(-0.178681615221)*x[0]**o+(-0.952835986168)*x[0]+(0.299692693746)*x[1]**o+(0.892451908416)*x[1]+(0.202403273634)*x[2]**o+(0.677212897943)*x[2]
        arg[3,1,2,1]=(0.0592594053342)*x[0]**o+(0.56391153959)*x[0]+(-0.874866627667)*x[1]**o+(0.432102900769)*x[1]+(-0.49073126285)*x[2]**o+(-0.010495860028)*x[2]
        arg[3,2,0,0]=(0.548006445814)*x[0]**o+(0.667313392202)*x[0]+(0.397805355499)*x[1]**o+(-0.788100267808)*x[1]+(0.869234055258)*x[2]**o+(-0.830241520401)*x[2]
        arg[3,2,0,1]=(0.72586139216)*x[0]**o+(-0.931878957878)*x[0]+(-0.94119555858)*x[1]**o+(-0.133544806169)*x[1]+(0.754263323382)*x[2]**o+(-0.446604740157)*x[2]
        arg[3,2,1,0]=(-0.28135830707)*x[0]**o+(-0.478744058518)*x[0]+(0.88219033731)*x[1]**o+(0.460587286945)*x[1]+(-0.849692146096)*x[2]**o+(0.0927701345107)*x[2]
        arg[3,2,1,1]=(-0.681859641611)*x[0]**o+(-0.670164293088)*x[0]+(0.796366374435)*x[1]**o+(0.837230317054)*x[1]+(-0.413283915904)*x[2]**o+(0.779624227187)*x[2]
        arg[3,2,2,0]=(-0.803242771255)*x[0]**o+(-0.0687616271776)*x[0]+(-0.948856027999)*x[1]**o+(0.762568790931)*x[1]+(0.937588292604)*x[2]**o+(-0.0224232611283)*x[2]
        arg[3,2,2,1]=(-0.215752944937)*x[0]**o+(-0.807114296075)*x[0]+(0.970625375964)*x[1]**o+(0.68578124692)*x[1]+(-0.165007047419)*x[2]**o+(0.942718439981)*x[2]
        arg[3,3,0,0]=(0.152001104716)*x[0]**o+(-0.657242342357)*x[0]+(0.559608661705)*x[1]**o+(0.0764403273251)*x[1]+(0.633414754695)*x[2]**o+(0.430777868344)*x[2]
        arg[3,3,0,1]=(-0.898109497759)*x[0]**o+(0.736574249406)*x[0]+(-0.569584184574)*x[1]**o+(-0.145501395506)*x[1]+(-0.412424116475)*x[2]**o+(-0.416350923096)*x[2]
        arg[3,3,1,0]=(0.602022795614)*x[0]**o+(-0.848525569825)*x[0]+(-0.713426287505)*x[1]**o+(-0.829278452687)*x[1]+(-0.491257659316)*x[2]**o+(0.90219908999)*x[2]
        arg[3,3,1,1]=(-0.769110216066)*x[0]**o+(-0.789464037338)*x[0]+(-0.363686172584)*x[1]**o+(-0.86190787068)*x[1]+(0.275438119232)*x[2]**o+(0.285209834248)*x[2]
        arg[3,3,2,0]=(0.351662405853)*x[0]**o+(0.8403302307)*x[0]+(-0.449869332519)*x[1]**o+(0.936043614838)*x[1]+(-0.656908777335)*x[2]**o+(-0.231923872531)*x[2]
        arg[3,3,2,1]=(0.873848945653)*x[0]**o+(0.551429883959)*x[0]+(-0.577942548701)*x[1]**o+(-0.254317163049)*x[1]+(0.64878089878)*x[2]**o+(0.7135339515)*x[2]
        arg[3,4,0,0]=(0.891435610494)*x[0]**o+(0.598893704498)*x[0]+(0.24682094756)*x[1]**o+(0.0895612710338)*x[1]+(-0.513686244953)*x[2]**o+(0.307205505502)*x[2]
        arg[3,4,0,1]=(-0.577752944196)*x[0]**o+(-0.458383751623)*x[0]+(-0.913956988143)*x[1]**o+(0.470857602235)*x[1]+(0.631726850665)*x[2]**o+(-0.112962289904)*x[2]
        arg[3,4,1,0]=(-0.65648177489)*x[0]**o+(-0.843530972747)*x[0]+(-0.0332153723809)*x[1]**o+(0.230484263355)*x[1]+(-0.865687508592)*x[2]**o+(0.122751735082)*x[2]
        arg[3,4,1,1]=(-0.353502640734)*x[0]**o+(-0.816020900282)*x[0]+(0.943688796868)*x[1]**o+(-0.526707881958)*x[1]+(-0.790258298117)*x[2]**o+(-0.748706809042)*x[2]
        arg[3,4,2,0]=(-0.596345912175)*x[0]**o+(0.872771121837)*x[0]+(0.100107395575)*x[1]**o+(0.950482249217)*x[1]+(0.0487634582783)*x[2]**o+(-0.0285218097887)*x[2]
        arg[3,4,2,1]=(-0.827443553933)*x[0]**o+(-0.922564800149)*x[0]+(-0.810354385287)*x[1]**o+(0.630079217293)*x[1]+(-0.316280559723)*x[2]**o+(0.977830435871)*x[2]
        ref[0,0,0,0]=(0.310272924207)*x_ref[0]**o+(-0.664497806351)*x_ref[0]+(0.741942132609)*x_ref[1]**o+(0.636051298779)*x_ref[1]+(0.76627159146)*x_ref[2]**o+(0.492960732588)*x_ref[2]
        ref[0,0,0,1]=(-0.513740801773)*x_ref[0]**o+(-0.633330750909)*x_ref[0]+(0.177934618983)*x_ref[1]**o+(-0.323745723166)*x_ref[1]+(0.694167180904)*x_ref[2]**o+(0.441239251435)*x_ref[2]
        ref[0,0,1,0]=(0.417477695848)*x_ref[0]**o+(-0.923415976002)*x_ref[0]+(0.0765555276662)*x_ref[1]**o+(-0.277290890834)*x_ref[1]+(0.0177191326614)*x_ref[2]**o+(-0.0842520100387)*x_ref[2]
        ref[0,0,1,1]=(-0.761916249894)*x_ref[0]**o+(0.169673139552)*x_ref[0]+(0.0306290828795)*x_ref[1]**o+(0.842234592613)*x_ref[1]+(-0.350342692669)*x_ref[2]**o+(0.321292270658)*x_ref[2]
        ref[0,0,2,0]=(-0.707494137624)*x_ref[0]**o+(0.346873490395)*x_ref[0]+(-0.68281925138)*x_ref[1]**o+(-0.830538083274)*x_ref[1]+(0.329372141613)*x_ref[2]**o+(-0.121366157163)*x_ref[2]
        ref[0,0,2,1]=(-0.708018186507)*x_ref[0]**o+(-0.778051239314)*x_ref[0]+(-0.43106291563)*x_ref[1]**o+(-0.0115914251143)*x_ref[1]+(0.487334215892)*x_ref[2]**o+(-0.289232284658)*x_ref[2]
        ref[0,1,0,0]=(0.530614704199)*x_ref[0]**o+(0.518622028317)*x_ref[0]+(-0.0143129589551)*x_ref[1]**o+(0.808469352257)*x_ref[1]+(-0.156787373225)*x_ref[2]**o+(0.668887813143)*x_ref[2]
        ref[0,1,0,1]=(0.953223233706)*x_ref[0]**o+(-0.164323838353)*x_ref[0]+(0.336717320858)*x_ref[1]**o+(-0.520230488172)*x_ref[1]+(0.689960738002)*x_ref[2]**o+(0.487630947578)*x_ref[2]
        ref[0,1,1,0]=(-0.418095373429)*x_ref[0]**o+(-0.941817844939)*x_ref[0]+(0.88105115607)*x_ref[1]**o+(-0.820502501439)*x_ref[1]+(-0.693858267667)*x_ref[2]**o+(0.530567713127)*x_ref[2]
        ref[0,1,1,1]=(-0.716861320218)*x_ref[0]**o+(-0.736619859467)*x_ref[0]+(-0.885478605601)*x_ref[1]**o+(-0.129375578967)*x_ref[1]+(-0.0181433907318)*x_ref[2]**o+(0.373255537959)*x_ref[2]
        ref[0,1,2,0]=(-0.174742583167)*x_ref[0]**o+(-0.0887967480112)*x_ref[0]+(-0.291168698662)*x_ref[1]**o+(-0.188608506769)*x_ref[1]+(0.22479613454)*x_ref[2]**o+(0.34173928582)*x_ref[2]
        ref[0,1,2,1]=(0.117208929504)*x_ref[0]**o+(-0.420969282079)*x_ref[0]+(0.585298109836)*x_ref[1]**o+(0.771355162745)*x_ref[1]+(-0.51374493747)*x_ref[2]**o+(0.785016548855)*x_ref[2]
        ref[0,2,0,0]=(0.737394887886)*x_ref[0]**o+(0.966080271342)*x_ref[0]+(-0.373178384492)*x_ref[1]**o+(-0.845505386142)*x_ref[1]+(-0.274654519371)*x_ref[2]**o+(0.709356557887)*x_ref[2]
        ref[0,2,0,1]=(0.0365864090636)*x_ref[0]**o+(-0.393382479787)*x_ref[0]+(0.018273360977)*x_ref[1]**o+(-0.509201909328)*x_ref[1]+(-0.777852392534)*x_ref[2]**o+(0.0417280247507)*x_ref[2]
        ref[0,2,1,0]=(0.637829302751)*x_ref[0]**o+(-0.767824790186)*x_ref[0]+(0.843382009511)*x_ref[1]**o+(-0.599633234403)*x_ref[1]+(0.14283138941)*x_ref[2]**o+(-0.354404633991)*x_ref[2]
        ref[0,2,1,1]=(-0.465258613599)*x_ref[0]**o+(0.09341154383)*x_ref[0]+(-0.629114782988)*x_ref[1]**o+(0.91608655383)*x_ref[1]+(-0.692901135848)*x_ref[2]**o+(-0.707225310667)*x_ref[2]
        ref[0,2,2,0]=(0.9935958759)*x_ref[0]**o+(-0.385008742466)*x_ref[0]+(0.153885356854)*x_ref[1]**o+(0.146270680656)*x_ref[1]+(0.305838410343)*x_ref[2]**o+(0.141193552717)*x_ref[2]
        ref[0,2,2,1]=(-0.487538023894)*x_ref[0]**o+(-0.814283442647)*x_ref[0]+(0.166008520199)*x_ref[1]**o+(0.313655774416)*x_ref[1]+(0.282308782973)*x_ref[2]**o+(0.309176062484)*x_ref[2]
        ref[0,3,0,0]=(0.00203507798913)*x_ref[0]**o+(-0.19902542604)*x_ref[0]+(0.88110918624)*x_ref[1]**o+(0.223549219288)*x_ref[1]+(0.135758952369)*x_ref[2]**o+(0.59116783639)*x_ref[2]
        ref[0,3,0,1]=(-0.185063700767)*x_ref[0]**o+(0.833862155173)*x_ref[0]+(-0.0939117509645)*x_ref[1]**o+(0.288003692231)*x_ref[1]+(0.580196157839)*x_ref[2]**o+(0.299320177754)*x_ref[2]
        ref[0,3,1,0]=(-0.039100718826)*x_ref[0]**o+(0.423179869395)*x_ref[0]+(-0.013669578975)*x_ref[1]**o+(-0.355053343085)*x_ref[1]+(0.89285158975)*x_ref[2]**o+(0.392984367877)*x_ref[2]
        ref[0,3,1,1]=(0.0988989416429)*x_ref[0]**o+(-0.446243235174)*x_ref[0]+(-0.691253313617)*x_ref[1]**o+(0.338823693086)*x_ref[1]+(0.707054126334)*x_ref[2]**o+(0.601121388278)*x_ref[2]
        ref[0,3,2,0]=(-0.191366770484)*x_ref[0]**o+(0.961791275903)*x_ref[0]+(-0.103547475035)*x_ref[1]**o+(-0.745236510079)*x_ref[1]+(-0.157506981712)*x_ref[2]**o+(0.986229506787)*x_ref[2]
        ref[0,3,2,1]=(-0.886091883647)*x_ref[0]**o+(0.6979804248)*x_ref[0]+(-0.366575419428)*x_ref[1]**o+(-0.958601153259)*x_ref[1]+(-0.0119724511494)*x_ref[2]**o+(-0.76101132076)*x_ref[2]
        ref[0,4,0,0]=(0.923290554434)*x_ref[0]**o+(0.411359492198)*x_ref[0]+(-0.977999562875)*x_ref[1]**o+(0.352085161527)*x_ref[1]+(0.955745235349)*x_ref[2]**o+(-0.978885609257)*x_ref[2]
        ref[0,4,0,1]=(0.226973479105)*x_ref[0]**o+(-0.423881897251)*x_ref[0]+(0.889101969481)*x_ref[1]**o+(-0.994236389028)*x_ref[1]+(0.0159169033361)*x_ref[2]**o+(-0.559935154203)*x_ref[2]
        ref[0,4,1,0]=(-0.225427291797)*x_ref[0]**o+(-0.557334621848)*x_ref[0]+(-0.486252586142)*x_ref[1]**o+(0.0497120905216)*x_ref[1]+(0.84165736691)*x_ref[2]**o+(-0.185010156797)*x_ref[2]
        ref[0,4,1,1]=(0.762694602609)*x_ref[0]**o+(-0.447367170727)*x_ref[0]+(-0.61103181818)*x_ref[1]**o+(0.689761057656)*x_ref[1]+(-0.631855120122)*x_ref[2]**o+(0.33274655602)*x_ref[2]
        ref[0,4,2,0]=(-0.198884868914)*x_ref[0]**o+(0.716371959278)*x_ref[0]+(0.260871768919)*x_ref[1]**o+(0.622241740082)*x_ref[1]+(0.774103524035)*x_ref[2]**o+(0.186132703676)*x_ref[2]
        ref[0,4,2,1]=(-0.309450645317)*x_ref[0]**o+(0.204754571336)*x_ref[0]+(0.483402810171)*x_ref[1]**o+(0.404620744188)*x_ref[1]+(-0.699137246959)*x_ref[2]**o+(0.467428706196)*x_ref[2]
        ref[1,0,0,0]=(0.642289506448)*x_ref[0]**o+(0.383990304394)*x_ref[0]+(0.556940973318)*x_ref[1]**o+(0.135219729041)*x_ref[1]+(0.577195836358)*x_ref[2]**o+(-0.338924419674)*x_ref[2]
        ref[1,0,0,1]=(-0.920873911295)*x_ref[0]**o+(-0.330610192304)*x_ref[0]+(-0.117226803856)*x_ref[1]**o+(0.172482867684)*x_ref[1]+(0.909140902709)*x_ref[2]**o+(-0.590494167913)*x_ref[2]
        ref[1,0,1,0]=(0.210161538861)*x_ref[0]**o+(-0.83860241761)*x_ref[0]+(0.0257180382661)*x_ref[1]**o+(0.877633854348)*x_ref[1]+(0.26045311754)*x_ref[2]**o+(-0.971349902714)*x_ref[2]
        ref[1,0,1,1]=(0.70736843125)*x_ref[0]**o+(-0.765121985431)*x_ref[0]+(-0.196525225943)*x_ref[1]**o+(-0.175928729346)*x_ref[1]+(0.945523028748)*x_ref[2]**o+(0.877830322823)*x_ref[2]
        ref[1,0,2,0]=(-0.963513016065)*x_ref[0]**o+(0.0820537856351)*x_ref[0]+(0.0122070196118)*x_ref[1]**o+(-0.154147449592)*x_ref[1]+(0.323844633268)*x_ref[2]**o+(-0.124897698214)*x_ref[2]
        ref[1,0,2,1]=(0.656267566405)*x_ref[0]**o+(0.0174786620768)*x_ref[0]+(0.763902435618)*x_ref[1]**o+(-0.986021506043)*x_ref[1]+(0.632625555994)*x_ref[2]**o+(-0.110657743523)*x_ref[2]
        ref[1,1,0,0]=(0.00354144688732)*x_ref[0]**o+(-0.813515808171)*x_ref[0]+(-0.100717717446)*x_ref[1]**o+(0.0543708163629)*x_ref[1]+(0.785927880762)*x_ref[2]**o+(0.0109166652227)*x_ref[2]
        ref[1,1,0,1]=(-0.239705688631)*x_ref[0]**o+(0.402915642509)*x_ref[0]+(0.463371068163)*x_ref[1]**o+(-0.507302462928)*x_ref[1]+(-0.0789844128295)*x_ref[2]**o+(0.669144891499)*x_ref[2]
        ref[1,1,1,0]=(-0.491891367001)*x_ref[0]**o+(-0.524377636928)*x_ref[0]+(0.903091863917)*x_ref[1]**o+(-0.554368807183)*x_ref[1]+(0.332899195339)*x_ref[2]**o+(-0.645216147049)*x_ref[2]
        ref[1,1,1,1]=(-0.915843941243)*x_ref[0]**o+(-0.233175654748)*x_ref[0]+(0.467730425037)*x_ref[1]**o+(-0.765957936683)*x_ref[1]+(0.726678877276)*x_ref[2]**o+(0.770533947942)*x_ref[2]
        ref[1,1,2,0]=(0.678071852032)*x_ref[0]**o+(0.266537844237)*x_ref[0]+(0.324335300151)*x_ref[1]**o+(-0.286713971373)*x_ref[1]+(-0.702384414014)*x_ref[2]**o+(0.887198276519)*x_ref[2]
        ref[1,1,2,1]=(0.0466444959648)*x_ref[0]**o+(0.689162912677)*x_ref[0]+(-0.35033923676)*x_ref[1]**o+(-0.0536380766876)*x_ref[1]+(-0.511833442517)*x_ref[2]**o+(0.125566299829)*x_ref[2]
        ref[1,2,0,0]=(-0.803170431622)*x_ref[0]**o+(0.476211169752)*x_ref[0]+(-0.170510267953)*x_ref[1]**o+(-0.765945735317)*x_ref[1]+(-0.169966329291)*x_ref[2]**o+(-0.988525918724)*x_ref[2]
        ref[1,2,0,1]=(0.289618751212)*x_ref[0]**o+(0.294379040976)*x_ref[0]+(-0.922478416838)*x_ref[1]**o+(0.210546495537)*x_ref[1]+(-0.87266381496)*x_ref[2]**o+(-0.865571669577)*x_ref[2]
        ref[1,2,1,0]=(-0.292811054005)*x_ref[0]**o+(0.651280833746)*x_ref[0]+(-0.625004203736)*x_ref[1]**o+(0.372078169172)*x_ref[1]+(-0.0539099055169)*x_ref[2]**o+(0.0911869270008)*x_ref[2]
        ref[1,2,1,1]=(0.496217698294)*x_ref[0]**o+(-0.986021787133)*x_ref[0]+(0.246417059532)*x_ref[1]**o+(0.610226802401)*x_ref[1]+(0.991534665298)*x_ref[2]**o+(-0.507441185982)*x_ref[2]
        ref[1,2,2,0]=(-0.839902565947)*x_ref[0]**o+(-0.279607147278)*x_ref[0]+(0.278212307963)*x_ref[1]**o+(0.609793640036)*x_ref[1]+(0.940471913774)*x_ref[2]**o+(0.21860828705)*x_ref[2]
        ref[1,2,2,1]=(0.789578112278)*x_ref[0]**o+(-0.0226697818608)*x_ref[0]+(0.216321340291)*x_ref[1]**o+(-0.701818463454)*x_ref[1]+(-0.166689030646)*x_ref[2]**o+(0.966131542292)*x_ref[2]
        ref[1,3,0,0]=(0.149337042551)*x_ref[0]**o+(-0.785050579674)*x_ref[0]+(0.530219924721)*x_ref[1]**o+(-0.971927239398)*x_ref[1]+(0.950917908603)*x_ref[2]**o+(0.132668077447)*x_ref[2]
        ref[1,3,0,1]=(0.983081559724)*x_ref[0]**o+(-0.358694498455)*x_ref[0]+(0.181080099281)*x_ref[1]**o+(-0.699756291643)*x_ref[1]+(-0.744335953304)*x_ref[2]**o+(0.00262043221382)*x_ref[2]
        ref[1,3,1,0]=(0.31776032045)*x_ref[0]**o+(-0.0726184220029)*x_ref[0]+(-0.759883735971)*x_ref[1]**o+(-0.6389710873)*x_ref[1]+(-0.607858499502)*x_ref[2]**o+(-0.397169895784)*x_ref[2]
        ref[1,3,1,1]=(-0.950468389728)*x_ref[0]**o+(0.831121915521)*x_ref[0]+(0.552857011083)*x_ref[1]**o+(0.848599672594)*x_ref[1]+(0.432183564231)*x_ref[2]**o+(0.57537721362)*x_ref[2]
        ref[1,3,2,0]=(0.736080143601)*x_ref[0]**o+(-0.804140307747)*x_ref[0]+(-0.163864867884)*x_ref[1]**o+(0.950874042315)*x_ref[1]+(-0.99676139412)*x_ref[2]**o+(0.332320294246)*x_ref[2]
        ref[1,3,2,1]=(0.119082582041)*x_ref[0]**o+(-0.969740778852)*x_ref[0]+(0.828432583532)*x_ref[1]**o+(0.190324050395)*x_ref[1]+(-0.42639142674)*x_ref[2]**o+(0.315519578192)*x_ref[2]
        ref[1,4,0,0]=(-0.798465141775)*x_ref[0]**o+(-0.717282977908)*x_ref[0]+(0.935655148713)*x_ref[1]**o+(-0.655752964484)*x_ref[1]+(0.159990149099)*x_ref[2]**o+(-0.537115062687)*x_ref[2]
        ref[1,4,0,1]=(0.445151775007)*x_ref[0]**o+(-0.867956302051)*x_ref[0]+(-0.809856879825)*x_ref[1]**o+(-0.326897750981)*x_ref[1]+(0.756352311987)*x_ref[2]**o+(0.577585513459)*x_ref[2]
        ref[1,4,1,0]=(0.294243347591)*x_ref[0]**o+(0.774741656931)*x_ref[0]+(-0.0799985737984)*x_ref[1]**o+(0.179789107412)*x_ref[1]+(-0.612198025189)*x_ref[2]**o+(-0.07086009611)*x_ref[2]
        ref[1,4,1,1]=(0.0692586287045)*x_ref[0]**o+(-0.671956300122)*x_ref[0]+(-0.948667104683)*x_ref[1]**o+(0.68656246801)*x_ref[1]+(-0.352975297696)*x_ref[2]**o+(0.277952113716)*x_ref[2]
        ref[1,4,2,0]=(-0.491316095517)*x_ref[0]**o+(0.725899087239)*x_ref[0]+(0.634113963431)*x_ref[1]**o+(0.235694393453)*x_ref[1]+(-0.15600720713)*x_ref[2]**o+(0.954814976539)*x_ref[2]
        ref[1,4,2,1]=(0.120206996339)*x_ref[0]**o+(-0.86151411823)*x_ref[0]+(-0.137093650097)*x_ref[1]**o+(0.633402978246)*x_ref[1]+(-0.42515364728)*x_ref[2]**o+(-0.987634210617)*x_ref[2]
        ref[2,0,0,0]=(0.539463971222)*x_ref[0]**o+(-0.03435650579)*x_ref[0]+(-0.884395295544)*x_ref[1]**o+(0.215454879022)*x_ref[1]+(0.842377328246)*x_ref[2]**o+(0.370920136871)*x_ref[2]
        ref[2,0,0,1]=(-0.411505463039)*x_ref[0]**o+(0.446494039439)*x_ref[0]+(0.0313771969556)*x_ref[1]**o+(0.602893223061)*x_ref[1]+(-0.30383490021)*x_ref[2]**o+(-0.788028756758)*x_ref[2]
        ref[2,0,1,0]=(0.489009615999)*x_ref[0]**o+(-0.95508248016)*x_ref[0]+(-0.21489475022)*x_ref[1]**o+(-0.857263478431)*x_ref[1]+(-0.123703409045)*x_ref[2]**o+(0.663389562431)*x_ref[2]
        ref[2,0,1,1]=(-0.0224763310929)*x_ref[0]**o+(-0.315886249237)*x_ref[0]+(-0.318201849695)*x_ref[1]**o+(0.423732682462)*x_ref[1]+(0.396065636381)*x_ref[2]**o+(0.0816717878865)*x_ref[2]
        ref[2,0,2,0]=(-0.191432618224)*x_ref[0]**o+(0.688389390795)*x_ref[0]+(-0.382948112159)*x_ref[1]**o+(0.562905173818)*x_ref[1]+(-0.118396714029)*x_ref[2]**o+(0.234478642577)*x_ref[2]
        ref[2,0,2,1]=(-0.348008173171)*x_ref[0]**o+(0.191994166636)*x_ref[0]+(0.450556057147)*x_ref[1]**o+(-0.710821693343)*x_ref[1]+(-0.245119333857)*x_ref[2]**o+(0.262869525239)*x_ref[2]
        ref[2,1,0,0]=(-0.160444506203)*x_ref[0]**o+(-0.816142837432)*x_ref[0]+(-0.619722480492)*x_ref[1]**o+(-0.989077039675)*x_ref[1]+(-0.0282349103977)*x_ref[2]**o+(-0.629849819691)*x_ref[2]
        ref[2,1,0,1]=(0.379513765297)*x_ref[0]**o+(0.934652105586)*x_ref[0]+(-0.632636427868)*x_ref[1]**o+(-0.0492550496553)*x_ref[1]+(0.137733572307)*x_ref[2]**o+(0.779782637942)*x_ref[2]
        ref[2,1,1,0]=(0.469499320417)*x_ref[0]**o+(0.570880924125)*x_ref[0]+(-0.0329833251419)*x_ref[1]**o+(-0.0388751034604)*x_ref[1]+(-0.878520094389)*x_ref[2]**o+(0.992656649048)*x_ref[2]
        ref[2,1,1,1]=(0.592005181995)*x_ref[0]**o+(-0.553980493526)*x_ref[0]+(0.359837665019)*x_ref[1]**o+(0.811107941179)*x_ref[1]+(-0.441929650912)*x_ref[2]**o+(-0.768618378636)*x_ref[2]
        ref[2,1,2,0]=(0.741376258816)*x_ref[0]**o+(-0.238909756717)*x_ref[0]+(0.204631082449)*x_ref[1]**o+(0.970501662949)*x_ref[1]+(-0.615638092239)*x_ref[2]**o+(-0.575948750844)*x_ref[2]
        ref[2,1,2,1]=(0.774050933112)*x_ref[0]**o+(-0.872234076721)*x_ref[0]+(-0.0936189240061)*x_ref[1]**o+(-0.773608756961)*x_ref[1]+(-0.119171907808)*x_ref[2]**o+(-0.256124796845)*x_ref[2]
        ref[2,2,0,0]=(0.459481754425)*x_ref[0]**o+(0.478364929493)*x_ref[0]+(0.460199733287)*x_ref[1]**o+(0.66880294659)*x_ref[1]+(-0.859525058389)*x_ref[2]**o+(0.77960284414)*x_ref[2]
        ref[2,2,0,1]=(0.690557835207)*x_ref[0]**o+(-0.812305038592)*x_ref[0]+(0.837924124791)*x_ref[1]**o+(-0.785979206364)*x_ref[1]+(-0.295553949819)*x_ref[2]**o+(0.575096295897)*x_ref[2]
        ref[2,2,1,0]=(0.443193683205)*x_ref[0]**o+(0.0355033222253)*x_ref[0]+(0.497641002766)*x_ref[1]**o+(-0.636955142451)*x_ref[1]+(-0.265324357454)*x_ref[2]**o+(0.734148569078)*x_ref[2]
        ref[2,2,1,1]=(-0.088707242599)*x_ref[0]**o+(0.513403518669)*x_ref[0]+(0.337741737651)*x_ref[1]**o+(-0.174843924478)*x_ref[1]+(0.687291255986)*x_ref[2]**o+(0.473850502258)*x_ref[2]
        ref[2,2,2,0]=(0.5305624642)*x_ref[0]**o+(-0.972997835953)*x_ref[0]+(0.00197107287888)*x_ref[1]**o+(-0.75490872634)*x_ref[1]+(-0.0569624569435)*x_ref[2]**o+(-0.0780532868631)*x_ref[2]
        ref[2,2,2,1]=(-0.744584471866)*x_ref[0]**o+(0.915494909272)*x_ref[0]+(0.239235050038)*x_ref[1]**o+(-0.192466753187)*x_ref[1]+(0.933556550824)*x_ref[2]**o+(-0.786420251487)*x_ref[2]
        ref[2,3,0,0]=(0.472114530627)*x_ref[0]**o+(0.0693211807557)*x_ref[0]+(-0.274273425713)*x_ref[1]**o+(0.136242554894)*x_ref[1]+(-0.329240675785)*x_ref[2]**o+(0.0212539247343)*x_ref[2]
        ref[2,3,0,1]=(-0.780426108099)*x_ref[0]**o+(-0.632382551025)*x_ref[0]+(0.554926949239)*x_ref[1]**o+(0.264204009059)*x_ref[1]+(-0.868007525824)*x_ref[2]**o+(-0.937631120201)*x_ref[2]
        ref[2,3,1,0]=(-0.826692080947)*x_ref[0]**o+(0.915241858538)*x_ref[0]+(-0.49986160859)*x_ref[1]**o+(0.921686464658)*x_ref[1]+(0.71996666304)*x_ref[2]**o+(0.297609501096)*x_ref[2]
        ref[2,3,1,1]=(0.991896994614)*x_ref[0]**o+(-0.543310008103)*x_ref[0]+(0.0791899631111)*x_ref[1]**o+(-0.914088200145)*x_ref[1]+(-0.496337624951)*x_ref[2]**o+(0.708950898614)*x_ref[2]
        ref[2,3,2,0]=(-0.235116966432)*x_ref[0]**o+(-0.492848730207)*x_ref[0]+(0.428728646789)*x_ref[1]**o+(-0.410747634691)*x_ref[1]+(0.705059815458)*x_ref[2]**o+(-0.185604633831)*x_ref[2]
        ref[2,3,2,1]=(0.838829037117)*x_ref[0]**o+(0.670561147563)*x_ref[0]+(0.621407965075)*x_ref[1]**o+(-0.10624946305)*x_ref[1]+(0.893703402687)*x_ref[2]**o+(0.985504995821)*x_ref[2]
        ref[2,4,0,0]=(0.635072504682)*x_ref[0]**o+(-0.681962427555)*x_ref[0]+(-0.00774518011081)*x_ref[1]**o+(-0.264996228668)*x_ref[1]+(0.690668403198)*x_ref[2]**o+(0.883573761062)*x_ref[2]
        ref[2,4,0,1]=(0.545632268651)*x_ref[0]**o+(0.439668519394)*x_ref[0]+(-0.482479808491)*x_ref[1]**o+(0.752028962391)*x_ref[1]+(0.0596773623939)*x_ref[2]**o+(-0.369987181127)*x_ref[2]
        ref[2,4,1,0]=(0.659005322796)*x_ref[0]**o+(0.503940833428)*x_ref[0]+(0.643633321586)*x_ref[1]**o+(0.927082323252)*x_ref[1]+(0.354072146184)*x_ref[2]**o+(-0.638737560608)*x_ref[2]
        ref[2,4,1,1]=(-0.443548860913)*x_ref[0]**o+(0.707607048458)*x_ref[0]+(-0.955225746402)*x_ref[1]**o+(0.515995563585)*x_ref[1]+(-0.288406637165)*x_ref[2]**o+(-0.760864937318)*x_ref[2]
        ref[2,4,2,0]=(0.898146580355)*x_ref[0]**o+(-0.590650787106)*x_ref[0]+(0.036095789753)*x_ref[1]**o+(0.326040260581)*x_ref[1]+(0.680069361306)*x_ref[2]**o+(0.333839099341)*x_ref[2]
        ref[2,4,2,1]=(-0.151364260271)*x_ref[0]**o+(-0.675198183832)*x_ref[0]+(-0.183626298978)*x_ref[1]**o+(-0.182565217146)*x_ref[1]+(0.0345632129814)*x_ref[2]**o+(-0.581534100763)*x_ref[2]
        ref[3,0,0,0]=(-0.942351703246)*x_ref[0]**o+(-0.699567649453)*x_ref[0]+(-0.832068665702)*x_ref[1]**o+(0.163439416504)*x_ref[1]+(0.742500339519)*x_ref[2]**o+(-0.492882887489)*x_ref[2]
        ref[3,0,0,1]=(0.990078296484)*x_ref[0]**o+(-0.999837336587)*x_ref[0]+(0.197589456459)*x_ref[1]**o+(0.431403059019)*x_ref[1]+(0.859864015871)*x_ref[2]**o+(-0.267474829282)*x_ref[2]
        ref[3,0,1,0]=(-0.776773660723)*x_ref[0]**o+(-0.189048370486)*x_ref[0]+(0.926953856802)*x_ref[1]**o+(-0.72314744652)*x_ref[1]+(0.330064363809)*x_ref[2]**o+(0.188533334359)*x_ref[2]
        ref[3,0,1,1]=(-0.854830058762)*x_ref[0]**o+(0.832073883276)*x_ref[0]+(-0.425691165124)*x_ref[1]**o+(0.162351500316)*x_ref[1]+(0.370604864512)*x_ref[2]**o+(-0.581907071484)*x_ref[2]
        ref[3,0,2,0]=(-0.316716909548)*x_ref[0]**o+(0.335417284383)*x_ref[0]+(-0.783061160352)*x_ref[1]**o+(-0.253972519239)*x_ref[1]+(-0.826432933721)*x_ref[2]**o+(0.267368378363)*x_ref[2]
        ref[3,0,2,1]=(-0.765899705462)*x_ref[0]**o+(-0.0993875861476)*x_ref[0]+(-0.211785004352)*x_ref[1]**o+(-0.635330151497)*x_ref[1]+(0.414575412599)*x_ref[2]**o+(0.586882274623)*x_ref[2]
        ref[3,1,0,0]=(0.987776377237)*x_ref[0]**o+(0.796660669191)*x_ref[0]+(0.580724891862)*x_ref[1]**o+(-0.24149253995)*x_ref[1]+(-0.633421016909)*x_ref[2]**o+(-0.220110913711)*x_ref[2]
        ref[3,1,0,1]=(-0.182240918081)*x_ref[0]**o+(-0.290526050572)*x_ref[0]+(0.00283230959976)*x_ref[1]**o+(-0.933282166805)*x_ref[1]+(-0.857893235883)*x_ref[2]**o+(0.551414074127)*x_ref[2]
        ref[3,1,1,0]=(-0.854435997816)*x_ref[0]**o+(0.706455144263)*x_ref[0]+(0.406237965254)*x_ref[1]**o+(0.192071142415)*x_ref[1]+(0.357966796834)*x_ref[2]**o+(0.887011087987)*x_ref[2]
        ref[3,1,1,1]=(0.832013396996)*x_ref[0]**o+(0.696133072321)*x_ref[0]+(0.682726968712)*x_ref[1]**o+(0.613316240514)*x_ref[1]+(0.12600339887)*x_ref[2]**o+(-0.620088231382)*x_ref[2]
        ref[3,1,2,0]=(-0.178681615221)*x_ref[0]**o+(-0.952835986168)*x_ref[0]+(0.299692693746)*x_ref[1]**o+(0.892451908416)*x_ref[1]+(0.202403273634)*x_ref[2]**o+(0.677212897943)*x_ref[2]
        ref[3,1,2,1]=(0.0592594053342)*x_ref[0]**o+(0.56391153959)*x_ref[0]+(-0.874866627667)*x_ref[1]**o+(0.432102900769)*x_ref[1]+(-0.49073126285)*x_ref[2]**o+(-0.010495860028)*x_ref[2]
        ref[3,2,0,0]=(0.548006445814)*x_ref[0]**o+(0.667313392202)*x_ref[0]+(0.397805355499)*x_ref[1]**o+(-0.788100267808)*x_ref[1]+(0.869234055258)*x_ref[2]**o+(-0.830241520401)*x_ref[2]
        ref[3,2,0,1]=(0.72586139216)*x_ref[0]**o+(-0.931878957878)*x_ref[0]+(-0.94119555858)*x_ref[1]**o+(-0.133544806169)*x_ref[1]+(0.754263323382)*x_ref[2]**o+(-0.446604740157)*x_ref[2]
        ref[3,2,1,0]=(-0.28135830707)*x_ref[0]**o+(-0.478744058518)*x_ref[0]+(0.88219033731)*x_ref[1]**o+(0.460587286945)*x_ref[1]+(-0.849692146096)*x_ref[2]**o+(0.0927701345107)*x_ref[2]
        ref[3,2,1,1]=(-0.681859641611)*x_ref[0]**o+(-0.670164293088)*x_ref[0]+(0.796366374435)*x_ref[1]**o+(0.837230317054)*x_ref[1]+(-0.413283915904)*x_ref[2]**o+(0.779624227187)*x_ref[2]
        ref[3,2,2,0]=(-0.803242771255)*x_ref[0]**o+(-0.0687616271776)*x_ref[0]+(-0.948856027999)*x_ref[1]**o+(0.762568790931)*x_ref[1]+(0.937588292604)*x_ref[2]**o+(-0.0224232611283)*x_ref[2]
        ref[3,2,2,1]=(-0.215752944937)*x_ref[0]**o+(-0.807114296075)*x_ref[0]+(0.970625375964)*x_ref[1]**o+(0.68578124692)*x_ref[1]+(-0.165007047419)*x_ref[2]**o+(0.942718439981)*x_ref[2]
        ref[3,3,0,0]=(0.152001104716)*x_ref[0]**o+(-0.657242342357)*x_ref[0]+(0.559608661705)*x_ref[1]**o+(0.0764403273251)*x_ref[1]+(0.633414754695)*x_ref[2]**o+(0.430777868344)*x_ref[2]
        ref[3,3,0,1]=(-0.898109497759)*x_ref[0]**o+(0.736574249406)*x_ref[0]+(-0.569584184574)*x_ref[1]**o+(-0.145501395506)*x_ref[1]+(-0.412424116475)*x_ref[2]**o+(-0.416350923096)*x_ref[2]
        ref[3,3,1,0]=(0.602022795614)*x_ref[0]**o+(-0.848525569825)*x_ref[0]+(-0.713426287505)*x_ref[1]**o+(-0.829278452687)*x_ref[1]+(-0.491257659316)*x_ref[2]**o+(0.90219908999)*x_ref[2]
        ref[3,3,1,1]=(-0.769110216066)*x_ref[0]**o+(-0.789464037338)*x_ref[0]+(-0.363686172584)*x_ref[1]**o+(-0.86190787068)*x_ref[1]+(0.275438119232)*x_ref[2]**o+(0.285209834248)*x_ref[2]
        ref[3,3,2,0]=(0.351662405853)*x_ref[0]**o+(0.8403302307)*x_ref[0]+(-0.449869332519)*x_ref[1]**o+(0.936043614838)*x_ref[1]+(-0.656908777335)*x_ref[2]**o+(-0.231923872531)*x_ref[2]
        ref[3,3,2,1]=(0.873848945653)*x_ref[0]**o+(0.551429883959)*x_ref[0]+(-0.577942548701)*x_ref[1]**o+(-0.254317163049)*x_ref[1]+(0.64878089878)*x_ref[2]**o+(0.7135339515)*x_ref[2]
        ref[3,4,0,0]=(0.891435610494)*x_ref[0]**o+(0.598893704498)*x_ref[0]+(0.24682094756)*x_ref[1]**o+(0.0895612710338)*x_ref[1]+(-0.513686244953)*x_ref[2]**o+(0.307205505502)*x_ref[2]
        ref[3,4,0,1]=(-0.577752944196)*x_ref[0]**o+(-0.458383751623)*x_ref[0]+(-0.913956988143)*x_ref[1]**o+(0.470857602235)*x_ref[1]+(0.631726850665)*x_ref[2]**o+(-0.112962289904)*x_ref[2]
        ref[3,4,1,0]=(-0.65648177489)*x_ref[0]**o+(-0.843530972747)*x_ref[0]+(-0.0332153723809)*x_ref[1]**o+(0.230484263355)*x_ref[1]+(-0.865687508592)*x_ref[2]**o+(0.122751735082)*x_ref[2]
        ref[3,4,1,1]=(-0.353502640734)*x_ref[0]**o+(-0.816020900282)*x_ref[0]+(0.943688796868)*x_ref[1]**o+(-0.526707881958)*x_ref[1]+(-0.790258298117)*x_ref[2]**o+(-0.748706809042)*x_ref[2]
        ref[3,4,2,0]=(-0.596345912175)*x_ref[0]**o+(0.872771121837)*x_ref[0]+(0.100107395575)*x_ref[1]**o+(0.950482249217)*x_ref[1]+(0.0487634582783)*x_ref[2]**o+(-0.0285218097887)*x_ref[2]
        ref[3,4,2,1]=(-0.827443553933)*x_ref[0]**o+(-0.922564800149)*x_ref[0]+(-0.810354385287)*x_ref[1]**o+(0.630079217293)*x_ref[1]+(-0.316280559723)*x_ref[2]**o+(0.977830435871)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_ReducedFunctionOnContactOne_rank0(self):
      """
      tests interpolate for rank 0 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactOne onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactOne(self.domain)
      x=w.getX()
      arg=Data(0,(),w)
      ref=Data(0,(),w_ref)
      if dim==2:
        arg=(0.142445811229)*x[0]**o+(-0.170111870439)*x[0]+(-0.571150729608)*x[1]**o+(-0.799596368319)*x[1]
        ref=(0.142445811229)*x_ref[0]**o+(-0.170111870439)*x_ref[0]+(-0.571150729608)*x_ref[1]**o+(-0.799596368319)*x_ref[1]
      else:
        arg=(0.131269935442)*x[0]**o+(0.67764849927)*x[0]+(0.199946665)*x[1]**o+(0.775678673404)*x[1]+(-0.818505707274)*x[2]**o+(-0.141160005914)*x[2]
        ref=(0.131269935442)*x_ref[0]**o+(0.67764849927)*x_ref[0]+(0.199946665)*x_ref[1]**o+(0.775678673404)*x_ref[1]+(-0.818505707274)*x_ref[2]**o+(-0.141160005914)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_ReducedFunctionOnContactOne_rank1(self):
      """
      tests interpolate for rank 1 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactOne onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactOne(self.domain)
      x=w.getX()
      arg=Data(0,(2,),w)
      ref=Data(0,(2,),w_ref)
      if dim==2:
        arg[0]=(-0.271030346059)*x[0]**o+(-0.604054986035)*x[0]+(-0.856492962015)*x[1]**o+(0.229582187423)*x[1]
        arg[1]=(-0.152539485929)*x[0]**o+(0.936634973224)*x[0]+(0.805016132937)*x[1]**o+(-0.6832854412)*x[1]
        ref[0]=(-0.271030346059)*x_ref[0]**o+(-0.604054986035)*x_ref[0]+(-0.856492962015)*x_ref[1]**o+(0.229582187423)*x_ref[1]
        ref[1]=(-0.152539485929)*x_ref[0]**o+(0.936634973224)*x_ref[0]+(0.805016132937)*x_ref[1]**o+(-0.6832854412)*x_ref[1]
      else:
        arg[0]=(-0.484580515247)*x[0]**o+(-0.134539997002)*x[0]+(0.589742989514)*x[1]**o+(-0.950803827747)*x[1]+(-0.852976687869)*x[2]**o+(0.0999526415345)*x[2]
        arg[1]=(-0.863620642149)*x[0]**o+(-0.16255544441)*x[0]+(-0.4963232161)*x[1]**o+(0.180988811217)*x[1]+(-0.787186716126)*x[2]**o+(-0.359825851725)*x[2]
        ref[0]=(-0.484580515247)*x_ref[0]**o+(-0.134539997002)*x_ref[0]+(0.589742989514)*x_ref[1]**o+(-0.950803827747)*x_ref[1]+(-0.852976687869)*x_ref[2]**o+(0.0999526415345)*x_ref[2]
        ref[1]=(-0.863620642149)*x_ref[0]**o+(-0.16255544441)*x_ref[0]+(-0.4963232161)*x_ref[1]**o+(0.180988811217)*x_ref[1]+(-0.787186716126)*x_ref[2]**o+(-0.359825851725)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_ReducedFunctionOnContactOne_rank2(self):
      """
      tests interpolate for rank 2 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactOne onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactOne(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5),w)
      ref=Data(0,(4, 5),w_ref)
      if dim==2:
        arg[0,0]=(-0.88064835903)*x[0]**o+(-0.140271048585)*x[0]+(-0.363467917345)*x[1]**o+(0.382530733181)*x[1]
        arg[0,1]=(0.835096280471)*x[0]**o+(0.132077921375)*x[0]+(-0.814392115734)*x[1]**o+(-0.379227781873)*x[1]
        arg[0,2]=(0.257391174596)*x[0]**o+(-0.135017751364)*x[0]+(-0.462760793484)*x[1]**o+(-0.332510860616)*x[1]
        arg[0,3]=(0.303541666957)*x[0]**o+(0.729054207976)*x[0]+(-0.0474210800089)*x[1]**o+(0.213859363809)*x[1]
        arg[0,4]=(0.564466740909)*x[0]**o+(0.684761789227)*x[0]+(0.432934636771)*x[1]**o+(0.261269614075)*x[1]
        arg[1,0]=(0.83080596334)*x[0]**o+(0.968795153198)*x[0]+(0.835115581928)*x[1]**o+(-0.134688885728)*x[1]
        arg[1,1]=(-0.876235093042)*x[0]**o+(-0.43923440557)*x[0]+(-0.641186633503)*x[1]**o+(-0.987129244994)*x[1]
        arg[1,2]=(-0.994007922096)*x[0]**o+(0.534914510762)*x[0]+(0.754420969331)*x[1]**o+(0.241533891038)*x[1]
        arg[1,3]=(-0.196027247194)*x[0]**o+(0.962231147494)*x[0]+(-0.819146467532)*x[1]**o+(0.966024528731)*x[1]
        arg[1,4]=(-0.577014723723)*x[0]**o+(0.350310620855)*x[0]+(0.16266475329)*x[1]**o+(-0.926739533525)*x[1]
        arg[2,0]=(-0.262964780787)*x[0]**o+(-0.824445469277)*x[0]+(-0.982497605841)*x[1]**o+(0.900080022932)*x[1]
        arg[2,1]=(0.54409272334)*x[0]**o+(0.789421020125)*x[0]+(0.672239490644)*x[1]**o+(-0.0306885937363)*x[1]
        arg[2,2]=(-0.130272075526)*x[0]**o+(0.51213564573)*x[0]+(-0.819817699321)*x[1]**o+(-0.259068167835)*x[1]
        arg[2,3]=(-0.61526267092)*x[0]**o+(-0.504362726269)*x[0]+(0.362429515671)*x[1]**o+(-0.0921852465615)*x[1]
        arg[2,4]=(-0.344772989987)*x[0]**o+(0.260747743828)*x[0]+(0.474457474131)*x[1]**o+(0.436008642452)*x[1]
        arg[3,0]=(-0.72740499107)*x[0]**o+(-0.297524430077)*x[0]+(-0.809254720524)*x[1]**o+(-0.230695822532)*x[1]
        arg[3,1]=(-0.0817042383885)*x[0]**o+(-0.324890823023)*x[0]+(0.923635342281)*x[1]**o+(-0.336825317481)*x[1]
        arg[3,2]=(-0.872954045409)*x[0]**o+(0.242165239629)*x[0]+(0.919264824591)*x[1]**o+(0.424247774945)*x[1]
        arg[3,3]=(0.146344350418)*x[0]**o+(-0.705285807962)*x[0]+(-0.0735095471635)*x[1]**o+(-0.464180824267)*x[1]
        arg[3,4]=(0.624085113245)*x[0]**o+(0.719452515121)*x[0]+(0.28331108038)*x[1]**o+(-0.897435702442)*x[1]
        ref[0,0]=(-0.88064835903)*x_ref[0]**o+(-0.140271048585)*x_ref[0]+(-0.363467917345)*x_ref[1]**o+(0.382530733181)*x_ref[1]
        ref[0,1]=(0.835096280471)*x_ref[0]**o+(0.132077921375)*x_ref[0]+(-0.814392115734)*x_ref[1]**o+(-0.379227781873)*x_ref[1]
        ref[0,2]=(0.257391174596)*x_ref[0]**o+(-0.135017751364)*x_ref[0]+(-0.462760793484)*x_ref[1]**o+(-0.332510860616)*x_ref[1]
        ref[0,3]=(0.303541666957)*x_ref[0]**o+(0.729054207976)*x_ref[0]+(-0.0474210800089)*x_ref[1]**o+(0.213859363809)*x_ref[1]
        ref[0,4]=(0.564466740909)*x_ref[0]**o+(0.684761789227)*x_ref[0]+(0.432934636771)*x_ref[1]**o+(0.261269614075)*x_ref[1]
        ref[1,0]=(0.83080596334)*x_ref[0]**o+(0.968795153198)*x_ref[0]+(0.835115581928)*x_ref[1]**o+(-0.134688885728)*x_ref[1]
        ref[1,1]=(-0.876235093042)*x_ref[0]**o+(-0.43923440557)*x_ref[0]+(-0.641186633503)*x_ref[1]**o+(-0.987129244994)*x_ref[1]
        ref[1,2]=(-0.994007922096)*x_ref[0]**o+(0.534914510762)*x_ref[0]+(0.754420969331)*x_ref[1]**o+(0.241533891038)*x_ref[1]
        ref[1,3]=(-0.196027247194)*x_ref[0]**o+(0.962231147494)*x_ref[0]+(-0.819146467532)*x_ref[1]**o+(0.966024528731)*x_ref[1]
        ref[1,4]=(-0.577014723723)*x_ref[0]**o+(0.350310620855)*x_ref[0]+(0.16266475329)*x_ref[1]**o+(-0.926739533525)*x_ref[1]
        ref[2,0]=(-0.262964780787)*x_ref[0]**o+(-0.824445469277)*x_ref[0]+(-0.982497605841)*x_ref[1]**o+(0.900080022932)*x_ref[1]
        ref[2,1]=(0.54409272334)*x_ref[0]**o+(0.789421020125)*x_ref[0]+(0.672239490644)*x_ref[1]**o+(-0.0306885937363)*x_ref[1]
        ref[2,2]=(-0.130272075526)*x_ref[0]**o+(0.51213564573)*x_ref[0]+(-0.819817699321)*x_ref[1]**o+(-0.259068167835)*x_ref[1]
        ref[2,3]=(-0.61526267092)*x_ref[0]**o+(-0.504362726269)*x_ref[0]+(0.362429515671)*x_ref[1]**o+(-0.0921852465615)*x_ref[1]
        ref[2,4]=(-0.344772989987)*x_ref[0]**o+(0.260747743828)*x_ref[0]+(0.474457474131)*x_ref[1]**o+(0.436008642452)*x_ref[1]
        ref[3,0]=(-0.72740499107)*x_ref[0]**o+(-0.297524430077)*x_ref[0]+(-0.809254720524)*x_ref[1]**o+(-0.230695822532)*x_ref[1]
        ref[3,1]=(-0.0817042383885)*x_ref[0]**o+(-0.324890823023)*x_ref[0]+(0.923635342281)*x_ref[1]**o+(-0.336825317481)*x_ref[1]
        ref[3,2]=(-0.872954045409)*x_ref[0]**o+(0.242165239629)*x_ref[0]+(0.919264824591)*x_ref[1]**o+(0.424247774945)*x_ref[1]
        ref[3,3]=(0.146344350418)*x_ref[0]**o+(-0.705285807962)*x_ref[0]+(-0.0735095471635)*x_ref[1]**o+(-0.464180824267)*x_ref[1]
        ref[3,4]=(0.624085113245)*x_ref[0]**o+(0.719452515121)*x_ref[0]+(0.28331108038)*x_ref[1]**o+(-0.897435702442)*x_ref[1]
      else:
        arg[0,0]=(0.47332206873)*x[0]**o+(0.20564453167)*x[0]+(-0.175485469473)*x[1]**o+(-0.25166656853)*x[1]+(-0.242693318919)*x[2]**o+(-0.773417228526)*x[2]
        arg[0,1]=(0.788563488587)*x[0]**o+(0.598162739778)*x[0]+(-0.736463777722)*x[1]**o+(-0.161555683484)*x[1]+(0.480782882175)*x[2]**o+(-0.500552084345)*x[2]
        arg[0,2]=(-0.207893050318)*x[0]**o+(0.206221241577)*x[0]+(0.306329036151)*x[1]**o+(0.585237890094)*x[1]+(0.0276469766188)*x[2]**o+(0.0502503670529)*x[2]
        arg[0,3]=(0.914950534537)*x[0]**o+(0.401315333176)*x[0]+(0.245224334898)*x[1]**o+(0.526049350134)*x[1]+(-0.735617023846)*x[2]**o+(-0.674692140531)*x[2]
        arg[0,4]=(-0.359566330106)*x[0]**o+(-0.279627730979)*x[0]+(-0.08398829656)*x[1]**o+(-0.789678621983)*x[1]+(-0.357374658443)*x[2]**o+(0.795724504883)*x[2]
        arg[1,0]=(-0.0715278193161)*x[0]**o+(-0.562120168502)*x[0]+(0.428614380101)*x[1]**o+(0.115408373591)*x[1]+(-0.154674397872)*x[2]**o+(0.233468972682)*x[2]
        arg[1,1]=(0.707445640039)*x[0]**o+(-0.719649334705)*x[0]+(-0.349743923514)*x[1]**o+(0.369262203159)*x[1]+(0.712786695739)*x[2]**o+(-0.962450154391)*x[2]
        arg[1,2]=(-0.306983094408)*x[0]**o+(-0.933322822964)*x[0]+(-0.525515899443)*x[1]**o+(0.642320930433)*x[1]+(-0.950421802297)*x[2]**o+(-0.392142385683)*x[2]
        arg[1,3]=(-0.765158324235)*x[0]**o+(-0.362010493356)*x[0]+(-0.918673930909)*x[1]**o+(0.72772784184)*x[1]+(-0.392848438408)*x[2]**o+(0.566119231816)*x[2]
        arg[1,4]=(0.429984662315)*x[0]**o+(-0.165476415697)*x[0]+(0.778539911928)*x[1]**o+(0.411518911472)*x[1]+(0.418323432603)*x[2]**o+(-0.846495913126)*x[2]
        arg[2,0]=(-0.836556750858)*x[0]**o+(0.976027316021)*x[0]+(-0.556499096672)*x[1]**o+(0.320038395915)*x[1]+(0.377663884516)*x[2]**o+(0.491202727923)*x[2]
        arg[2,1]=(-0.329597826951)*x[0]**o+(-0.555819984044)*x[0]+(0.110029358927)*x[1]**o+(-0.737070187836)*x[1]+(-0.587142473737)*x[2]**o+(-0.0388834177112)*x[2]
        arg[2,2]=(0.442251030646)*x[0]**o+(0.285010574287)*x[0]+(0.603267537331)*x[1]**o+(-0.0776884302384)*x[1]+(-0.123541484708)*x[2]**o+(0.44717125529)*x[2]
        arg[2,3]=(0.474458930313)*x[0]**o+(0.415947401939)*x[0]+(0.381539850696)*x[1]**o+(-0.307590909422)*x[1]+(0.488348451586)*x[2]**o+(0.336883810306)*x[2]
        arg[2,4]=(0.804466542611)*x[0]**o+(0.891337158669)*x[0]+(-0.599119487307)*x[1]**o+(-0.92047732727)*x[1]+(-0.251489480657)*x[2]**o+(-0.266483991526)*x[2]
        arg[3,0]=(-0.733633520827)*x[0]**o+(0.494805537624)*x[0]+(-0.117702558274)*x[1]**o+(-0.000461693303442)*x[1]+(-0.102033981202)*x[2]**o+(-0.772865985272)*x[2]
        arg[3,1]=(-0.636901276465)*x[0]**o+(-0.436330072041)*x[0]+(0.403048850222)*x[1]**o+(-0.94737100009)*x[1]+(0.781059874101)*x[2]**o+(-0.952781516037)*x[2]
        arg[3,2]=(-0.489372336164)*x[0]**o+(0.221211043152)*x[0]+(0.793975373906)*x[1]**o+(0.424349965288)*x[1]+(0.453332885662)*x[2]**o+(-0.513541753005)*x[2]
        arg[3,3]=(-0.700899677836)*x[0]**o+(-0.984271896306)*x[0]+(-0.663909486898)*x[1]**o+(0.461302174316)*x[1]+(-0.906457707724)*x[2]**o+(-0.0440003164038)*x[2]
        arg[3,4]=(0.838326295366)*x[0]**o+(-0.483407593962)*x[0]+(-0.122458752397)*x[1]**o+(0.607337951095)*x[1]+(0.750481384434)*x[2]**o+(-0.677155633204)*x[2]
        ref[0,0]=(0.47332206873)*x_ref[0]**o+(0.20564453167)*x_ref[0]+(-0.175485469473)*x_ref[1]**o+(-0.25166656853)*x_ref[1]+(-0.242693318919)*x_ref[2]**o+(-0.773417228526)*x_ref[2]
        ref[0,1]=(0.788563488587)*x_ref[0]**o+(0.598162739778)*x_ref[0]+(-0.736463777722)*x_ref[1]**o+(-0.161555683484)*x_ref[1]+(0.480782882175)*x_ref[2]**o+(-0.500552084345)*x_ref[2]
        ref[0,2]=(-0.207893050318)*x_ref[0]**o+(0.206221241577)*x_ref[0]+(0.306329036151)*x_ref[1]**o+(0.585237890094)*x_ref[1]+(0.0276469766188)*x_ref[2]**o+(0.0502503670529)*x_ref[2]
        ref[0,3]=(0.914950534537)*x_ref[0]**o+(0.401315333176)*x_ref[0]+(0.245224334898)*x_ref[1]**o+(0.526049350134)*x_ref[1]+(-0.735617023846)*x_ref[2]**o+(-0.674692140531)*x_ref[2]
        ref[0,4]=(-0.359566330106)*x_ref[0]**o+(-0.279627730979)*x_ref[0]+(-0.08398829656)*x_ref[1]**o+(-0.789678621983)*x_ref[1]+(-0.357374658443)*x_ref[2]**o+(0.795724504883)*x_ref[2]
        ref[1,0]=(-0.0715278193161)*x_ref[0]**o+(-0.562120168502)*x_ref[0]+(0.428614380101)*x_ref[1]**o+(0.115408373591)*x_ref[1]+(-0.154674397872)*x_ref[2]**o+(0.233468972682)*x_ref[2]
        ref[1,1]=(0.707445640039)*x_ref[0]**o+(-0.719649334705)*x_ref[0]+(-0.349743923514)*x_ref[1]**o+(0.369262203159)*x_ref[1]+(0.712786695739)*x_ref[2]**o+(-0.962450154391)*x_ref[2]
        ref[1,2]=(-0.306983094408)*x_ref[0]**o+(-0.933322822964)*x_ref[0]+(-0.525515899443)*x_ref[1]**o+(0.642320930433)*x_ref[1]+(-0.950421802297)*x_ref[2]**o+(-0.392142385683)*x_ref[2]
        ref[1,3]=(-0.765158324235)*x_ref[0]**o+(-0.362010493356)*x_ref[0]+(-0.918673930909)*x_ref[1]**o+(0.72772784184)*x_ref[1]+(-0.392848438408)*x_ref[2]**o+(0.566119231816)*x_ref[2]
        ref[1,4]=(0.429984662315)*x_ref[0]**o+(-0.165476415697)*x_ref[0]+(0.778539911928)*x_ref[1]**o+(0.411518911472)*x_ref[1]+(0.418323432603)*x_ref[2]**o+(-0.846495913126)*x_ref[2]
        ref[2,0]=(-0.836556750858)*x_ref[0]**o+(0.976027316021)*x_ref[0]+(-0.556499096672)*x_ref[1]**o+(0.320038395915)*x_ref[1]+(0.377663884516)*x_ref[2]**o+(0.491202727923)*x_ref[2]
        ref[2,1]=(-0.329597826951)*x_ref[0]**o+(-0.555819984044)*x_ref[0]+(0.110029358927)*x_ref[1]**o+(-0.737070187836)*x_ref[1]+(-0.587142473737)*x_ref[2]**o+(-0.0388834177112)*x_ref[2]
        ref[2,2]=(0.442251030646)*x_ref[0]**o+(0.285010574287)*x_ref[0]+(0.603267537331)*x_ref[1]**o+(-0.0776884302384)*x_ref[1]+(-0.123541484708)*x_ref[2]**o+(0.44717125529)*x_ref[2]
        ref[2,3]=(0.474458930313)*x_ref[0]**o+(0.415947401939)*x_ref[0]+(0.381539850696)*x_ref[1]**o+(-0.307590909422)*x_ref[1]+(0.488348451586)*x_ref[2]**o+(0.336883810306)*x_ref[2]
        ref[2,4]=(0.804466542611)*x_ref[0]**o+(0.891337158669)*x_ref[0]+(-0.599119487307)*x_ref[1]**o+(-0.92047732727)*x_ref[1]+(-0.251489480657)*x_ref[2]**o+(-0.266483991526)*x_ref[2]
        ref[3,0]=(-0.733633520827)*x_ref[0]**o+(0.494805537624)*x_ref[0]+(-0.117702558274)*x_ref[1]**o+(-0.000461693303442)*x_ref[1]+(-0.102033981202)*x_ref[2]**o+(-0.772865985272)*x_ref[2]
        ref[3,1]=(-0.636901276465)*x_ref[0]**o+(-0.436330072041)*x_ref[0]+(0.403048850222)*x_ref[1]**o+(-0.94737100009)*x_ref[1]+(0.781059874101)*x_ref[2]**o+(-0.952781516037)*x_ref[2]
        ref[3,2]=(-0.489372336164)*x_ref[0]**o+(0.221211043152)*x_ref[0]+(0.793975373906)*x_ref[1]**o+(0.424349965288)*x_ref[1]+(0.453332885662)*x_ref[2]**o+(-0.513541753005)*x_ref[2]
        ref[3,3]=(-0.700899677836)*x_ref[0]**o+(-0.984271896306)*x_ref[0]+(-0.663909486898)*x_ref[1]**o+(0.461302174316)*x_ref[1]+(-0.906457707724)*x_ref[2]**o+(-0.0440003164038)*x_ref[2]
        ref[3,4]=(0.838326295366)*x_ref[0]**o+(-0.483407593962)*x_ref[0]+(-0.122458752397)*x_ref[1]**o+(0.607337951095)*x_ref[1]+(0.750481384434)*x_ref[2]**o+(-0.677155633204)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_ReducedFunctionOnContactOne_rank3(self):
      """
      tests interpolate for rank 3 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactOne onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactOne(self.domain)
      x=w.getX()
      arg=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2),w_ref)
      if dim==2:
        arg[0,0,0]=(0.111473275114)*x[0]**o+(-0.552264448532)*x[0]+(0.67142534913)*x[1]**o+(-0.642305751076)*x[1]
        arg[0,0,1]=(-0.431075800974)*x[0]**o+(0.911382078454)*x[0]+(0.546541644612)*x[1]**o+(-0.67679652536)*x[1]
        arg[0,1,0]=(0.103056292292)*x[0]**o+(-0.640858804416)*x[0]+(0.159596964992)*x[1]**o+(-0.237782007316)*x[1]
        arg[0,1,1]=(0.975898170389)*x[0]**o+(0.255916326383)*x[0]+(-0.692770462322)*x[1]**o+(0.786355114846)*x[1]
        arg[1,0,0]=(-0.669378370759)*x[0]**o+(-0.0153788709146)*x[0]+(0.924383188557)*x[1]**o+(0.885037556829)*x[1]
        arg[1,0,1]=(-0.509891819881)*x[0]**o+(-0.377779514697)*x[0]+(-0.343176473036)*x[1]**o+(-0.39978416725)*x[1]
        arg[1,1,0]=(0.457959467665)*x[0]**o+(0.786511967341)*x[0]+(0.256561665146)*x[1]**o+(-0.261300099643)*x[1]
        arg[1,1,1]=(0.219754515189)*x[0]**o+(-0.131747762867)*x[0]+(0.736017818893)*x[1]**o+(-0.450963700533)*x[1]
        arg[2,0,0]=(-0.182946832789)*x[0]**o+(-0.103064731935)*x[0]+(-0.223584334222)*x[1]**o+(-0.578088201628)*x[1]
        arg[2,0,1]=(-0.830744157425)*x[0]**o+(0.590627167207)*x[0]+(-0.538048378049)*x[1]**o+(-0.0529793352741)*x[1]
        arg[2,1,0]=(-0.307908670388)*x[0]**o+(-0.929987734045)*x[0]+(-0.167190563092)*x[1]**o+(0.126466628227)*x[1]
        arg[2,1,1]=(-0.812045378916)*x[0]**o+(0.676973938072)*x[0]+(0.528243909204)*x[1]**o+(-0.594241387691)*x[1]
        arg[3,0,0]=(-0.0198606164114)*x[0]**o+(-0.00574708685554)*x[0]+(0.908946312745)*x[1]**o+(0.775054099788)*x[1]
        arg[3,0,1]=(-0.810078120518)*x[0]**o+(-0.960585888485)*x[0]+(-0.0747272427198)*x[1]**o+(0.164107641817)*x[1]
        arg[3,1,0]=(-0.535791917139)*x[0]**o+(0.0424226440074)*x[0]+(0.701479263902)*x[1]**o+(0.787745423088)*x[1]
        arg[3,1,1]=(-0.896532152793)*x[0]**o+(-0.0224566920251)*x[0]+(-0.554655626689)*x[1]**o+(0.41012867952)*x[1]
        arg[4,0,0]=(0.972433817496)*x[0]**o+(-0.440662536033)*x[0]+(-0.910456530943)*x[1]**o+(-0.333429535028)*x[1]
        arg[4,0,1]=(-0.415824313942)*x[0]**o+(0.00255557712922)*x[0]+(0.304897639195)*x[1]**o+(-0.866600848052)*x[1]
        arg[4,1,0]=(-0.866277739076)*x[0]**o+(0.282976169776)*x[0]+(0.477528278501)*x[1]**o+(-0.233506497695)*x[1]
        arg[4,1,1]=(-0.12729626381)*x[0]**o+(-0.723204796785)*x[0]+(0.0856495713442)*x[1]**o+(-0.214819437731)*x[1]
        arg[5,0,0]=(0.208133503644)*x[0]**o+(-0.955529202895)*x[0]+(0.0291569127831)*x[1]**o+(0.489913918305)*x[1]
        arg[5,0,1]=(0.96923667477)*x[0]**o+(0.543776525315)*x[0]+(0.38666450029)*x[1]**o+(-0.193493550272)*x[1]
        arg[5,1,0]=(-0.0912940016888)*x[0]**o+(-0.477561072681)*x[0]+(0.474816021613)*x[1]**o+(-0.252664229253)*x[1]
        arg[5,1,1]=(0.748337265159)*x[0]**o+(-0.706194454618)*x[0]+(-0.504466281507)*x[1]**o+(0.582500817679)*x[1]
        ref[0,0,0]=(0.111473275114)*x_ref[0]**o+(-0.552264448532)*x_ref[0]+(0.67142534913)*x_ref[1]**o+(-0.642305751076)*x_ref[1]
        ref[0,0,1]=(-0.431075800974)*x_ref[0]**o+(0.911382078454)*x_ref[0]+(0.546541644612)*x_ref[1]**o+(-0.67679652536)*x_ref[1]
        ref[0,1,0]=(0.103056292292)*x_ref[0]**o+(-0.640858804416)*x_ref[0]+(0.159596964992)*x_ref[1]**o+(-0.237782007316)*x_ref[1]
        ref[0,1,1]=(0.975898170389)*x_ref[0]**o+(0.255916326383)*x_ref[0]+(-0.692770462322)*x_ref[1]**o+(0.786355114846)*x_ref[1]
        ref[1,0,0]=(-0.669378370759)*x_ref[0]**o+(-0.0153788709146)*x_ref[0]+(0.924383188557)*x_ref[1]**o+(0.885037556829)*x_ref[1]
        ref[1,0,1]=(-0.509891819881)*x_ref[0]**o+(-0.377779514697)*x_ref[0]+(-0.343176473036)*x_ref[1]**o+(-0.39978416725)*x_ref[1]
        ref[1,1,0]=(0.457959467665)*x_ref[0]**o+(0.786511967341)*x_ref[0]+(0.256561665146)*x_ref[1]**o+(-0.261300099643)*x_ref[1]
        ref[1,1,1]=(0.219754515189)*x_ref[0]**o+(-0.131747762867)*x_ref[0]+(0.736017818893)*x_ref[1]**o+(-0.450963700533)*x_ref[1]
        ref[2,0,0]=(-0.182946832789)*x_ref[0]**o+(-0.103064731935)*x_ref[0]+(-0.223584334222)*x_ref[1]**o+(-0.578088201628)*x_ref[1]
        ref[2,0,1]=(-0.830744157425)*x_ref[0]**o+(0.590627167207)*x_ref[0]+(-0.538048378049)*x_ref[1]**o+(-0.0529793352741)*x_ref[1]
        ref[2,1,0]=(-0.307908670388)*x_ref[0]**o+(-0.929987734045)*x_ref[0]+(-0.167190563092)*x_ref[1]**o+(0.126466628227)*x_ref[1]
        ref[2,1,1]=(-0.812045378916)*x_ref[0]**o+(0.676973938072)*x_ref[0]+(0.528243909204)*x_ref[1]**o+(-0.594241387691)*x_ref[1]
        ref[3,0,0]=(-0.0198606164114)*x_ref[0]**o+(-0.00574708685554)*x_ref[0]+(0.908946312745)*x_ref[1]**o+(0.775054099788)*x_ref[1]
        ref[3,0,1]=(-0.810078120518)*x_ref[0]**o+(-0.960585888485)*x_ref[0]+(-0.0747272427198)*x_ref[1]**o+(0.164107641817)*x_ref[1]
        ref[3,1,0]=(-0.535791917139)*x_ref[0]**o+(0.0424226440074)*x_ref[0]+(0.701479263902)*x_ref[1]**o+(0.787745423088)*x_ref[1]
        ref[3,1,1]=(-0.896532152793)*x_ref[0]**o+(-0.0224566920251)*x_ref[0]+(-0.554655626689)*x_ref[1]**o+(0.41012867952)*x_ref[1]
        ref[4,0,0]=(0.972433817496)*x_ref[0]**o+(-0.440662536033)*x_ref[0]+(-0.910456530943)*x_ref[1]**o+(-0.333429535028)*x_ref[1]
        ref[4,0,1]=(-0.415824313942)*x_ref[0]**o+(0.00255557712922)*x_ref[0]+(0.304897639195)*x_ref[1]**o+(-0.866600848052)*x_ref[1]
        ref[4,1,0]=(-0.866277739076)*x_ref[0]**o+(0.282976169776)*x_ref[0]+(0.477528278501)*x_ref[1]**o+(-0.233506497695)*x_ref[1]
        ref[4,1,1]=(-0.12729626381)*x_ref[0]**o+(-0.723204796785)*x_ref[0]+(0.0856495713442)*x_ref[1]**o+(-0.214819437731)*x_ref[1]
        ref[5,0,0]=(0.208133503644)*x_ref[0]**o+(-0.955529202895)*x_ref[0]+(0.0291569127831)*x_ref[1]**o+(0.489913918305)*x_ref[1]
        ref[5,0,1]=(0.96923667477)*x_ref[0]**o+(0.543776525315)*x_ref[0]+(0.38666450029)*x_ref[1]**o+(-0.193493550272)*x_ref[1]
        ref[5,1,0]=(-0.0912940016888)*x_ref[0]**o+(-0.477561072681)*x_ref[0]+(0.474816021613)*x_ref[1]**o+(-0.252664229253)*x_ref[1]
        ref[5,1,1]=(0.748337265159)*x_ref[0]**o+(-0.706194454618)*x_ref[0]+(-0.504466281507)*x_ref[1]**o+(0.582500817679)*x_ref[1]
      else:
        arg[0,0,0]=(-0.179972720095)*x[0]**o+(-0.546715268523)*x[0]+(0.835867431319)*x[1]**o+(0.626798499259)*x[1]+(0.397356013278)*x[2]**o+(0.0771586784158)*x[2]
        arg[0,0,1]=(0.625676031039)*x[0]**o+(0.65843849475)*x[0]+(0.456469199307)*x[1]**o+(-0.970709759479)*x[1]+(-0.699606143992)*x[2]**o+(0.76429400724)*x[2]
        arg[0,1,0]=(0.613916424165)*x[0]**o+(0.556778588303)*x[0]+(-0.509658743529)*x[1]**o+(-0.978718968218)*x[1]+(-0.603928369428)*x[2]**o+(-0.550813953692)*x[2]
        arg[0,1,1]=(-0.935214755436)*x[0]**o+(0.44745901813)*x[0]+(-0.582828846453)*x[1]**o+(-0.181547183233)*x[1]+(-0.183912053485)*x[2]**o+(0.446152494296)*x[2]
        arg[1,0,0]=(-0.486345219779)*x[0]**o+(0.753529436833)*x[0]+(0.138157856552)*x[1]**o+(-0.874849020539)*x[1]+(0.374396543792)*x[2]**o+(0.26548456046)*x[2]
        arg[1,0,1]=(0.492908753036)*x[0]**o+(-0.418118685105)*x[0]+(0.880817569823)*x[1]**o+(-0.827691145524)*x[1]+(0.238633156133)*x[2]**o+(-0.463687731341)*x[2]
        arg[1,1,0]=(-0.397373578442)*x[0]**o+(-0.316321105607)*x[0]+(0.647511225459)*x[1]**o+(-0.82553752195)*x[1]+(-0.715549303171)*x[2]**o+(0.804623957459)*x[2]
        arg[1,1,1]=(0.414743130234)*x[0]**o+(0.0201794413683)*x[0]+(-0.636337599584)*x[1]**o+(-0.230053616026)*x[1]+(-0.4302153311)*x[2]**o+(-0.387468029626)*x[2]
        arg[2,0,0]=(0.715100517336)*x[0]**o+(-0.4286120178)*x[0]+(0.0987858542117)*x[1]**o+(0.815093470301)*x[1]+(0.293455353257)*x[2]**o+(0.200576090379)*x[2]
        arg[2,0,1]=(-0.158330015822)*x[0]**o+(0.363339023457)*x[0]+(-0.796055594971)*x[1]**o+(-0.213299205306)*x[1]+(-0.136620988812)*x[2]**o+(0.10733886947)*x[2]
        arg[2,1,0]=(-0.258729405118)*x[0]**o+(0.0669553480038)*x[0]+(0.997663166613)*x[1]**o+(0.00398861063931)*x[1]+(-0.989459856803)*x[2]**o+(0.932687384325)*x[2]
        arg[2,1,1]=(-0.974431322319)*x[0]**o+(-0.486447431804)*x[0]+(0.871784873347)*x[1]**o+(-0.978904038765)*x[1]+(0.647341201506)*x[2]**o+(-0.269683556528)*x[2]
        arg[3,0,0]=(-0.369507398073)*x[0]**o+(-0.772085607074)*x[0]+(0.417550881937)*x[1]**o+(0.271743719652)*x[1]+(0.881807154737)*x[2]**o+(0.54725428039)*x[2]
        arg[3,0,1]=(0.440089335941)*x[0]**o+(0.0844171421294)*x[0]+(0.926389704362)*x[1]**o+(-0.0709182998297)*x[1]+(0.903978660317)*x[2]**o+(-0.160556206716)*x[2]
        arg[3,1,0]=(-0.49936391282)*x[0]**o+(-0.632753931881)*x[0]+(-0.604686084554)*x[1]**o+(-0.954205199531)*x[1]+(-0.535102184845)*x[2]**o+(0.621683814123)*x[2]
        arg[3,1,1]=(-0.117790973132)*x[0]**o+(-0.649091727896)*x[0]+(0.925682916785)*x[1]**o+(0.852053515469)*x[1]+(-0.479218429897)*x[2]**o+(0.426823478575)*x[2]
        arg[4,0,0]=(0.454174564319)*x[0]**o+(-0.855975647406)*x[0]+(0.775972969288)*x[1]**o+(0.267900999893)*x[1]+(0.221387501995)*x[2]**o+(-0.26413791201)*x[2]
        arg[4,0,1]=(0.875191302431)*x[0]**o+(-0.362782502152)*x[0]+(0.24600070339)*x[1]**o+(-0.566709533298)*x[1]+(-0.82187455531)*x[2]**o+(-0.943929640301)*x[2]
        arg[4,1,0]=(-0.409313259973)*x[0]**o+(-0.784833509001)*x[0]+(-0.939690505056)*x[1]**o+(-0.0736280226974)*x[1]+(0.939072745486)*x[2]**o+(0.611697159959)*x[2]
        arg[4,1,1]=(0.893374246808)*x[0]**o+(0.36113513561)*x[0]+(0.227899271271)*x[1]**o+(-0.847221657012)*x[1]+(-0.0106368947417)*x[2]**o+(0.0358896817595)*x[2]
        arg[5,0,0]=(0.0329230524924)*x[0]**o+(0.0565192084014)*x[0]+(-0.68398448658)*x[1]**o+(0.909035618076)*x[1]+(-0.900060440872)*x[2]**o+(-0.481014787833)*x[2]
        arg[5,0,1]=(0.404966692391)*x[0]**o+(0.647468764373)*x[0]+(-0.723847499346)*x[1]**o+(-0.0740896957606)*x[1]+(-0.00628515079923)*x[2]**o+(0.906667863539)*x[2]
        arg[5,1,0]=(0.983181781531)*x[0]**o+(-0.297325990236)*x[0]+(0.00489064543352)*x[1]**o+(0.667560767038)*x[1]+(-0.665768141301)*x[2]**o+(0.772761393608)*x[2]
        arg[5,1,1]=(-0.496581691354)*x[0]**o+(0.0121337979505)*x[0]+(0.492281382726)*x[1]**o+(0.545232714025)*x[1]+(0.418321481532)*x[2]**o+(0.489635969764)*x[2]
        ref[0,0,0]=(-0.179972720095)*x_ref[0]**o+(-0.546715268523)*x_ref[0]+(0.835867431319)*x_ref[1]**o+(0.626798499259)*x_ref[1]+(0.397356013278)*x_ref[2]**o+(0.0771586784158)*x_ref[2]
        ref[0,0,1]=(0.625676031039)*x_ref[0]**o+(0.65843849475)*x_ref[0]+(0.456469199307)*x_ref[1]**o+(-0.970709759479)*x_ref[1]+(-0.699606143992)*x_ref[2]**o+(0.76429400724)*x_ref[2]
        ref[0,1,0]=(0.613916424165)*x_ref[0]**o+(0.556778588303)*x_ref[0]+(-0.509658743529)*x_ref[1]**o+(-0.978718968218)*x_ref[1]+(-0.603928369428)*x_ref[2]**o+(-0.550813953692)*x_ref[2]
        ref[0,1,1]=(-0.935214755436)*x_ref[0]**o+(0.44745901813)*x_ref[0]+(-0.582828846453)*x_ref[1]**o+(-0.181547183233)*x_ref[1]+(-0.183912053485)*x_ref[2]**o+(0.446152494296)*x_ref[2]
        ref[1,0,0]=(-0.486345219779)*x_ref[0]**o+(0.753529436833)*x_ref[0]+(0.138157856552)*x_ref[1]**o+(-0.874849020539)*x_ref[1]+(0.374396543792)*x_ref[2]**o+(0.26548456046)*x_ref[2]
        ref[1,0,1]=(0.492908753036)*x_ref[0]**o+(-0.418118685105)*x_ref[0]+(0.880817569823)*x_ref[1]**o+(-0.827691145524)*x_ref[1]+(0.238633156133)*x_ref[2]**o+(-0.463687731341)*x_ref[2]
        ref[1,1,0]=(-0.397373578442)*x_ref[0]**o+(-0.316321105607)*x_ref[0]+(0.647511225459)*x_ref[1]**o+(-0.82553752195)*x_ref[1]+(-0.715549303171)*x_ref[2]**o+(0.804623957459)*x_ref[2]
        ref[1,1,1]=(0.414743130234)*x_ref[0]**o+(0.0201794413683)*x_ref[0]+(-0.636337599584)*x_ref[1]**o+(-0.230053616026)*x_ref[1]+(-0.4302153311)*x_ref[2]**o+(-0.387468029626)*x_ref[2]
        ref[2,0,0]=(0.715100517336)*x_ref[0]**o+(-0.4286120178)*x_ref[0]+(0.0987858542117)*x_ref[1]**o+(0.815093470301)*x_ref[1]+(0.293455353257)*x_ref[2]**o+(0.200576090379)*x_ref[2]
        ref[2,0,1]=(-0.158330015822)*x_ref[0]**o+(0.363339023457)*x_ref[0]+(-0.796055594971)*x_ref[1]**o+(-0.213299205306)*x_ref[1]+(-0.136620988812)*x_ref[2]**o+(0.10733886947)*x_ref[2]
        ref[2,1,0]=(-0.258729405118)*x_ref[0]**o+(0.0669553480038)*x_ref[0]+(0.997663166613)*x_ref[1]**o+(0.00398861063931)*x_ref[1]+(-0.989459856803)*x_ref[2]**o+(0.932687384325)*x_ref[2]
        ref[2,1,1]=(-0.974431322319)*x_ref[0]**o+(-0.486447431804)*x_ref[0]+(0.871784873347)*x_ref[1]**o+(-0.978904038765)*x_ref[1]+(0.647341201506)*x_ref[2]**o+(-0.269683556528)*x_ref[2]
        ref[3,0,0]=(-0.369507398073)*x_ref[0]**o+(-0.772085607074)*x_ref[0]+(0.417550881937)*x_ref[1]**o+(0.271743719652)*x_ref[1]+(0.881807154737)*x_ref[2]**o+(0.54725428039)*x_ref[2]
        ref[3,0,1]=(0.440089335941)*x_ref[0]**o+(0.0844171421294)*x_ref[0]+(0.926389704362)*x_ref[1]**o+(-0.0709182998297)*x_ref[1]+(0.903978660317)*x_ref[2]**o+(-0.160556206716)*x_ref[2]
        ref[3,1,0]=(-0.49936391282)*x_ref[0]**o+(-0.632753931881)*x_ref[0]+(-0.604686084554)*x_ref[1]**o+(-0.954205199531)*x_ref[1]+(-0.535102184845)*x_ref[2]**o+(0.621683814123)*x_ref[2]
        ref[3,1,1]=(-0.117790973132)*x_ref[0]**o+(-0.649091727896)*x_ref[0]+(0.925682916785)*x_ref[1]**o+(0.852053515469)*x_ref[1]+(-0.479218429897)*x_ref[2]**o+(0.426823478575)*x_ref[2]
        ref[4,0,0]=(0.454174564319)*x_ref[0]**o+(-0.855975647406)*x_ref[0]+(0.775972969288)*x_ref[1]**o+(0.267900999893)*x_ref[1]+(0.221387501995)*x_ref[2]**o+(-0.26413791201)*x_ref[2]
        ref[4,0,1]=(0.875191302431)*x_ref[0]**o+(-0.362782502152)*x_ref[0]+(0.24600070339)*x_ref[1]**o+(-0.566709533298)*x_ref[1]+(-0.82187455531)*x_ref[2]**o+(-0.943929640301)*x_ref[2]
        ref[4,1,0]=(-0.409313259973)*x_ref[0]**o+(-0.784833509001)*x_ref[0]+(-0.939690505056)*x_ref[1]**o+(-0.0736280226974)*x_ref[1]+(0.939072745486)*x_ref[2]**o+(0.611697159959)*x_ref[2]
        ref[4,1,1]=(0.893374246808)*x_ref[0]**o+(0.36113513561)*x_ref[0]+(0.227899271271)*x_ref[1]**o+(-0.847221657012)*x_ref[1]+(-0.0106368947417)*x_ref[2]**o+(0.0358896817595)*x_ref[2]
        ref[5,0,0]=(0.0329230524924)*x_ref[0]**o+(0.0565192084014)*x_ref[0]+(-0.68398448658)*x_ref[1]**o+(0.909035618076)*x_ref[1]+(-0.900060440872)*x_ref[2]**o+(-0.481014787833)*x_ref[2]
        ref[5,0,1]=(0.404966692391)*x_ref[0]**o+(0.647468764373)*x_ref[0]+(-0.723847499346)*x_ref[1]**o+(-0.0740896957606)*x_ref[1]+(-0.00628515079923)*x_ref[2]**o+(0.906667863539)*x_ref[2]
        ref[5,1,0]=(0.983181781531)*x_ref[0]**o+(-0.297325990236)*x_ref[0]+(0.00489064543352)*x_ref[1]**o+(0.667560767038)*x_ref[1]+(-0.665768141301)*x_ref[2]**o+(0.772761393608)*x_ref[2]
        ref[5,1,1]=(-0.496581691354)*x_ref[0]**o+(0.0121337979505)*x_ref[0]+(0.492281382726)*x_ref[1]**o+(0.545232714025)*x_ref[1]+(0.418321481532)*x_ref[2]**o+(0.489635969764)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_ReducedFunctionOnContactOne_rank4(self):
      """
      tests interpolate for rank 4 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactOne onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactOne(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5, 3, 2),w)
      ref=Data(0,(4, 5, 3, 2),w_ref)
      if dim==2:
        arg[0,0,0,0]=(-0.282394950702)*x[0]**o+(0.836580157584)*x[0]+(0.337652262648)*x[1]**o+(-0.442694263325)*x[1]
        arg[0,0,0,1]=(0.85150774028)*x[0]**o+(0.998297296593)*x[0]+(-0.710029954342)*x[1]**o+(0.0816168287837)*x[1]
        arg[0,0,1,0]=(-0.636435102838)*x[0]**o+(-0.613479863553)*x[0]+(-0.861506513168)*x[1]**o+(-0.242998961478)*x[1]
        arg[0,0,1,1]=(-0.00671863978521)*x[0]**o+(0.909404084068)*x[0]+(-0.734956067026)*x[1]**o+(0.256530538264)*x[1]
        arg[0,0,2,0]=(-0.646631608999)*x[0]**o+(-0.521130455843)*x[0]+(-0.44328404557)*x[1]**o+(-0.156860325817)*x[1]
        arg[0,0,2,1]=(-0.817550008949)*x[0]**o+(0.702414497795)*x[0]+(0.946449680763)*x[1]**o+(-0.61576361925)*x[1]
        arg[0,1,0,0]=(-0.527206691595)*x[0]**o+(-0.43469216984)*x[0]+(-0.108455901154)*x[1]**o+(0.435646962156)*x[1]
        arg[0,1,0,1]=(-0.148522219535)*x[0]**o+(0.146912249755)*x[0]+(-0.414934945665)*x[1]**o+(-0.422754988714)*x[1]
        arg[0,1,1,0]=(0.177760972947)*x[0]**o+(-0.938522530862)*x[0]+(0.566229652264)*x[1]**o+(0.255269743628)*x[1]
        arg[0,1,1,1]=(-0.317807933792)*x[0]**o+(0.432181451246)*x[0]+(-0.358821513362)*x[1]**o+(-0.470517232585)*x[1]
        arg[0,1,2,0]=(0.486536430227)*x[0]**o+(-0.533653809349)*x[0]+(-0.126612514045)*x[1]**o+(0.103088115487)*x[1]
        arg[0,1,2,1]=(-0.995976559589)*x[0]**o+(0.965537167421)*x[0]+(-0.351307699531)*x[1]**o+(-0.684367434666)*x[1]
        arg[0,2,0,0]=(0.976008613768)*x[0]**o+(-0.890573007002)*x[0]+(-0.440709423944)*x[1]**o+(0.315724459367)*x[1]
        arg[0,2,0,1]=(-0.595275811962)*x[0]**o+(-0.599634144509)*x[0]+(0.646466002346)*x[1]**o+(-0.94529568651)*x[1]
        arg[0,2,1,0]=(0.977691521821)*x[0]**o+(-0.0730997643682)*x[0]+(-0.338897498737)*x[1]**o+(0.246308678824)*x[1]
        arg[0,2,1,1]=(-0.111317410732)*x[0]**o+(-0.574381793398)*x[0]+(-0.0314431899843)*x[1]**o+(0.393105047096)*x[1]
        arg[0,2,2,0]=(0.886320020757)*x[0]**o+(-0.785042003544)*x[0]+(0.82609269979)*x[1]**o+(-0.677169484337)*x[1]
        arg[0,2,2,1]=(0.811027138079)*x[0]**o+(-0.375108470948)*x[0]+(0.781472464253)*x[1]**o+(0.217640794143)*x[1]
        arg[0,3,0,0]=(0.264884117091)*x[0]**o+(0.116591319557)*x[0]+(0.282667679763)*x[1]**o+(0.111123454337)*x[1]
        arg[0,3,0,1]=(0.0451626985922)*x[0]**o+(-0.93771279728)*x[0]+(0.129171764072)*x[1]**o+(0.598367954162)*x[1]
        arg[0,3,1,0]=(0.905243511086)*x[0]**o+(0.0837062573586)*x[0]+(0.0657223248433)*x[1]**o+(-0.746644690072)*x[1]
        arg[0,3,1,1]=(-0.0547856755485)*x[0]**o+(-0.22157806755)*x[0]+(0.124623625108)*x[1]**o+(0.0436590157081)*x[1]
        arg[0,3,2,0]=(-0.558323744577)*x[0]**o+(-0.646043519007)*x[0]+(0.991885458004)*x[1]**o+(-0.634140808163)*x[1]
        arg[0,3,2,1]=(-0.27247983028)*x[0]**o+(0.803906404485)*x[0]+(-0.816826609924)*x[1]**o+(-0.969532783336)*x[1]
        arg[0,4,0,0]=(0.638733842891)*x[0]**o+(-0.972793270208)*x[0]+(0.714611008099)*x[1]**o+(0.586988595847)*x[1]
        arg[0,4,0,1]=(0.63584760734)*x[0]**o+(-0.653740236747)*x[0]+(-0.596304731446)*x[1]**o+(0.42519786429)*x[1]
        arg[0,4,1,0]=(0.0665562973529)*x[0]**o+(0.362152092093)*x[0]+(0.732530335739)*x[1]**o+(0.618536326941)*x[1]
        arg[0,4,1,1]=(0.486060667608)*x[0]**o+(0.59489013279)*x[0]+(-0.547237017498)*x[1]**o+(0.0371894705857)*x[1]
        arg[0,4,2,0]=(0.395505844397)*x[0]**o+(-0.869090434535)*x[0]+(-0.606640827437)*x[1]**o+(0.269995459083)*x[1]
        arg[0,4,2,1]=(0.304671547176)*x[0]**o+(-0.515772523849)*x[0]+(0.546114300876)*x[1]**o+(0.530247404341)*x[1]
        arg[1,0,0,0]=(0.408332677433)*x[0]**o+(-0.0778807359918)*x[0]+(-0.803337489297)*x[1]**o+(-0.123835718422)*x[1]
        arg[1,0,0,1]=(-0.00441620254944)*x[0]**o+(0.0321259598831)*x[0]+(-0.998381508472)*x[1]**o+(0.800767747511)*x[1]
        arg[1,0,1,0]=(-0.923683057418)*x[0]**o+(0.579405114902)*x[0]+(-0.939639823585)*x[1]**o+(-0.138423358134)*x[1]
        arg[1,0,1,1]=(-0.558420234848)*x[0]**o+(0.133316569813)*x[0]+(-0.294134717329)*x[1]**o+(-0.750897749044)*x[1]
        arg[1,0,2,0]=(0.478951298545)*x[0]**o+(-0.807173978799)*x[0]+(0.852438146763)*x[1]**o+(-0.103814229563)*x[1]
        arg[1,0,2,1]=(-0.875250726603)*x[0]**o+(-0.934803428491)*x[0]+(0.143352873672)*x[1]**o+(0.600934881206)*x[1]
        arg[1,1,0,0]=(-0.0539476321018)*x[0]**o+(0.194177663417)*x[0]+(-0.862877991009)*x[1]**o+(-0.438337993664)*x[1]
        arg[1,1,0,1]=(-0.623379355474)*x[0]**o+(-0.0746994739748)*x[0]+(0.365005438956)*x[1]**o+(-0.431737688205)*x[1]
        arg[1,1,1,0]=(0.0936054461735)*x[0]**o+(-0.974594269446)*x[0]+(-0.121917598428)*x[1]**o+(-0.84429083727)*x[1]
        arg[1,1,1,1]=(0.967763348672)*x[0]**o+(0.331698826475)*x[0]+(0.66448515877)*x[1]**o+(0.00214985930661)*x[1]
        arg[1,1,2,0]=(-0.312146580762)*x[0]**o+(0.998152287706)*x[0]+(-0.474242942777)*x[1]**o+(0.637827347846)*x[1]
        arg[1,1,2,1]=(-0.556765443436)*x[0]**o+(0.227327929774)*x[0]+(0.739457845569)*x[1]**o+(-0.615070017663)*x[1]
        arg[1,2,0,0]=(-0.23030660552)*x[0]**o+(0.750479465953)*x[0]+(0.278215408392)*x[1]**o+(-0.640618698224)*x[1]
        arg[1,2,0,1]=(-0.0713217101129)*x[0]**o+(0.500737477887)*x[0]+(-0.972834770005)*x[1]**o+(-0.420386056033)*x[1]
        arg[1,2,1,0]=(-0.497322989847)*x[0]**o+(-0.528685124841)*x[0]+(-0.934386348496)*x[1]**o+(0.191930553839)*x[1]
        arg[1,2,1,1]=(0.124362419781)*x[0]**o+(-0.0457849787456)*x[0]+(-0.360596154712)*x[1]**o+(0.799736253715)*x[1]
        arg[1,2,2,0]=(-0.346294878893)*x[0]**o+(0.427772309361)*x[0]+(-0.0642666791012)*x[1]**o+(-0.214442238311)*x[1]
        arg[1,2,2,1]=(0.24434837411)*x[0]**o+(-0.913373159786)*x[0]+(0.158750862391)*x[1]**o+(-0.544328807576)*x[1]
        arg[1,3,0,0]=(0.870847002687)*x[0]**o+(-0.466884462301)*x[0]+(-0.343358829445)*x[1]**o+(-0.180956419766)*x[1]
        arg[1,3,0,1]=(-0.813518850863)*x[0]**o+(0.960810467501)*x[0]+(0.35091416904)*x[1]**o+(0.84534544948)*x[1]
        arg[1,3,1,0]=(-0.284687751218)*x[0]**o+(-0.274583519355)*x[0]+(0.338892764102)*x[1]**o+(-0.145142443753)*x[1]
        arg[1,3,1,1]=(0.921064262137)*x[0]**o+(0.152552421678)*x[0]+(0.827247693375)*x[1]**o+(-0.569529848619)*x[1]
        arg[1,3,2,0]=(0.133567630356)*x[0]**o+(-0.0429397358364)*x[0]+(-0.175383170663)*x[1]**o+(0.584407631334)*x[1]
        arg[1,3,2,1]=(0.203880167371)*x[0]**o+(-0.422978073825)*x[0]+(-0.210109267588)*x[1]**o+(-0.902327373219)*x[1]
        arg[1,4,0,0]=(-0.566386864876)*x[0]**o+(-0.139795085483)*x[0]+(-0.00852913464238)*x[1]**o+(0.569151381884)*x[1]
        arg[1,4,0,1]=(-0.937256625212)*x[0]**o+(0.563438253128)*x[0]+(-0.586182238715)*x[1]**o+(-0.566169678723)*x[1]
        arg[1,4,1,0]=(-0.73296890712)*x[0]**o+(0.359249594406)*x[0]+(0.66050790616)*x[1]**o+(-0.367919797426)*x[1]
        arg[1,4,1,1]=(-0.81165734248)*x[0]**o+(0.460592817838)*x[0]+(0.953573909492)*x[1]**o+(0.329659021274)*x[1]
        arg[1,4,2,0]=(0.345994483595)*x[0]**o+(-0.18547324343)*x[0]+(0.669452926238)*x[1]**o+(0.42513249272)*x[1]
        arg[1,4,2,1]=(0.250677642616)*x[0]**o+(0.466118496803)*x[0]+(-0.776608543595)*x[1]**o+(0.449144243682)*x[1]
        arg[2,0,0,0]=(0.244172909021)*x[0]**o+(0.0311068322898)*x[0]+(-0.909348039508)*x[1]**o+(0.376500719414)*x[1]
        arg[2,0,0,1]=(-0.033516640667)*x[0]**o+(0.698424718328)*x[0]+(-0.282814785542)*x[1]**o+(0.61118022947)*x[1]
        arg[2,0,1,0]=(-0.946672649538)*x[0]**o+(0.814325244591)*x[0]+(-0.353215562825)*x[1]**o+(-0.138150257429)*x[1]
        arg[2,0,1,1]=(-0.886509051692)*x[0]**o+(-0.0468334385819)*x[0]+(0.71984719189)*x[1]**o+(-0.324744971493)*x[1]
        arg[2,0,2,0]=(-0.780921647053)*x[0]**o+(-0.401418496932)*x[0]+(0.647847274332)*x[1]**o+(-0.831834322278)*x[1]
        arg[2,0,2,1]=(0.835155004862)*x[0]**o+(0.684459986902)*x[0]+(-0.301244240677)*x[1]**o+(-0.161292780187)*x[1]
        arg[2,1,0,0]=(-0.652199398394)*x[0]**o+(0.62480722178)*x[0]+(-0.474535712914)*x[1]**o+(0.926061797766)*x[1]
        arg[2,1,0,1]=(-0.813365478568)*x[0]**o+(-0.38795792417)*x[0]+(0.378161353697)*x[1]**o+(0.70581806211)*x[1]
        arg[2,1,1,0]=(0.729265594934)*x[0]**o+(-0.469397842359)*x[0]+(0.642472199926)*x[1]**o+(-0.779527283011)*x[1]
        arg[2,1,1,1]=(0.0517342917574)*x[0]**o+(-0.414111007917)*x[0]+(-0.89437375241)*x[1]**o+(-0.830119585842)*x[1]
        arg[2,1,2,0]=(0.769898486257)*x[0]**o+(0.791725624832)*x[0]+(0.297519329396)*x[1]**o+(-0.959462006611)*x[1]
        arg[2,1,2,1]=(0.817802144251)*x[0]**o+(0.69851258911)*x[0]+(-0.174401735228)*x[1]**o+(0.13449696498)*x[1]
        arg[2,2,0,0]=(0.547972662256)*x[0]**o+(-0.66491929979)*x[0]+(0.330783041672)*x[1]**o+(0.816196730606)*x[1]
        arg[2,2,0,1]=(0.245971097763)*x[0]**o+(0.213087481481)*x[0]+(-0.840766291392)*x[1]**o+(0.388414557376)*x[1]
        arg[2,2,1,0]=(0.523151676444)*x[0]**o+(0.854817486027)*x[0]+(-0.53570188528)*x[1]**o+(-0.68149362242)*x[1]
        arg[2,2,1,1]=(-0.984200247787)*x[0]**o+(0.83324365092)*x[0]+(-0.896184408816)*x[1]**o+(0.102276243487)*x[1]
        arg[2,2,2,0]=(-0.216357164308)*x[0]**o+(-0.597984882107)*x[0]+(-0.205435921511)*x[1]**o+(0.484947652759)*x[1]
        arg[2,2,2,1]=(-0.650046253868)*x[0]**o+(-0.624878186105)*x[0]+(0.786519914406)*x[1]**o+(0.28027273095)*x[1]
        arg[2,3,0,0]=(-0.0821718346579)*x[0]**o+(-0.687049917467)*x[0]+(0.380792333736)*x[1]**o+(0.151185837715)*x[1]
        arg[2,3,0,1]=(0.61796113854)*x[0]**o+(0.844624413065)*x[0]+(-0.980619401098)*x[1]**o+(-0.854924862641)*x[1]
        arg[2,3,1,0]=(0.812293906883)*x[0]**o+(-0.225593208768)*x[0]+(0.964175374163)*x[1]**o+(0.848850064926)*x[1]
        arg[2,3,1,1]=(0.376453572937)*x[0]**o+(0.994818812059)*x[0]+(0.58872018303)*x[1]**o+(0.68332473361)*x[1]
        arg[2,3,2,0]=(-0.00948421972234)*x[0]**o+(-0.680685462738)*x[0]+(-0.767076278167)*x[1]**o+(0.210165437284)*x[1]
        arg[2,3,2,1]=(-0.455199698263)*x[0]**o+(-0.763430401419)*x[0]+(0.183773350212)*x[1]**o+(0.43105940923)*x[1]
        arg[2,4,0,0]=(-0.673962025988)*x[0]**o+(-0.679073069725)*x[0]+(0.11018431715)*x[1]**o+(0.349020527482)*x[1]
        arg[2,4,0,1]=(-0.876691282076)*x[0]**o+(0.920902226631)*x[0]+(-0.477417470246)*x[1]**o+(0.836417982216)*x[1]
        arg[2,4,1,0]=(0.567966851024)*x[0]**o+(0.583241431716)*x[0]+(-0.271851271008)*x[1]**o+(0.678727902863)*x[1]
        arg[2,4,1,1]=(0.707400094)*x[0]**o+(0.802820856444)*x[0]+(0.84518937616)*x[1]**o+(-0.0267193533403)*x[1]
        arg[2,4,2,0]=(0.213268920036)*x[0]**o+(0.780721415328)*x[0]+(0.839292948154)*x[1]**o+(-0.0376334744443)*x[1]
        arg[2,4,2,1]=(0.136255687031)*x[0]**o+(-0.582766278693)*x[0]+(0.257662521321)*x[1]**o+(0.808337521979)*x[1]
        arg[3,0,0,0]=(-0.524068770822)*x[0]**o+(0.942236878842)*x[0]+(0.889496255793)*x[1]**o+(-0.904810742393)*x[1]
        arg[3,0,0,1]=(0.780157465475)*x[0]**o+(-0.763811668832)*x[0]+(-0.517482070639)*x[1]**o+(0.662770780676)*x[1]
        arg[3,0,1,0]=(-0.693227564812)*x[0]**o+(-0.763506211841)*x[0]+(-0.427709881126)*x[1]**o+(0.159109827204)*x[1]
        arg[3,0,1,1]=(0.644975629084)*x[0]**o+(-0.928112761899)*x[0]+(-0.530772740083)*x[1]**o+(-0.723174790087)*x[1]
        arg[3,0,2,0]=(-0.923467274097)*x[0]**o+(0.806128117288)*x[0]+(0.177779363207)*x[1]**o+(0.862243885542)*x[1]
        arg[3,0,2,1]=(-0.0823252901358)*x[0]**o+(0.289022916284)*x[0]+(-0.518508771021)*x[1]**o+(0.174564769845)*x[1]
        arg[3,1,0,0]=(0.608837578444)*x[0]**o+(0.898216758361)*x[0]+(-0.844730109761)*x[1]**o+(0.745974173207)*x[1]
        arg[3,1,0,1]=(-0.11776411078)*x[0]**o+(0.186256502356)*x[0]+(0.565289400333)*x[1]**o+(-0.776173716206)*x[1]
        arg[3,1,1,0]=(-0.170736580152)*x[0]**o+(0.829581661683)*x[0]+(0.310135833859)*x[1]**o+(-0.509629545687)*x[1]
        arg[3,1,1,1]=(-0.225246122751)*x[0]**o+(0.586754377294)*x[0]+(-0.726064725635)*x[1]**o+(0.136952605704)*x[1]
        arg[3,1,2,0]=(-0.287932115478)*x[0]**o+(0.858031396153)*x[0]+(-0.493441745353)*x[1]**o+(0.601827664994)*x[1]
        arg[3,1,2,1]=(0.665503842862)*x[0]**o+(-0.515998970452)*x[0]+(-0.890055558624)*x[1]**o+(-0.0446371451457)*x[1]
        arg[3,2,0,0]=(-0.33741557585)*x[0]**o+(-0.416183752333)*x[0]+(-0.92430073538)*x[1]**o+(-0.486265090904)*x[1]
        arg[3,2,0,1]=(0.910860925994)*x[0]**o+(-0.896869755389)*x[0]+(0.0634658185324)*x[1]**o+(-0.383136455739)*x[1]
        arg[3,2,1,0]=(0.386839297976)*x[0]**o+(-0.190659927071)*x[0]+(-0.304846978185)*x[1]**o+(-0.92705511899)*x[1]
        arg[3,2,1,1]=(-0.0197812018683)*x[0]**o+(-0.479083095775)*x[0]+(0.502489976089)*x[1]**o+(-0.859440665475)*x[1]
        arg[3,2,2,0]=(-0.507672045299)*x[0]**o+(0.498689676072)*x[0]+(-0.719735804645)*x[1]**o+(-0.522053306674)*x[1]
        arg[3,2,2,1]=(0.46207629211)*x[0]**o+(-0.496697066039)*x[0]+(0.994069339904)*x[1]**o+(0.873222754449)*x[1]
        arg[3,3,0,0]=(0.126410029692)*x[0]**o+(0.51067050196)*x[0]+(-0.978384824088)*x[1]**o+(-0.68606930503)*x[1]
        arg[3,3,0,1]=(0.376888157008)*x[0]**o+(-0.14015764025)*x[0]+(0.523667700758)*x[1]**o+(0.0277726855258)*x[1]
        arg[3,3,1,0]=(-0.105865233294)*x[0]**o+(-0.0515230741203)*x[0]+(0.203766525036)*x[1]**o+(-0.0431893014387)*x[1]
        arg[3,3,1,1]=(-0.404351448486)*x[0]**o+(-0.639161429157)*x[0]+(-0.957341091708)*x[1]**o+(-0.103346466984)*x[1]
        arg[3,3,2,0]=(-0.0523508780623)*x[0]**o+(-0.563589575544)*x[0]+(0.158487727325)*x[1]**o+(-0.519455492178)*x[1]
        arg[3,3,2,1]=(-0.823286342218)*x[0]**o+(-0.493663499535)*x[0]+(-0.409825574064)*x[1]**o+(-0.0406101279806)*x[1]
        arg[3,4,0,0]=(0.926212910387)*x[0]**o+(-0.0944507963)*x[0]+(0.640740499391)*x[1]**o+(-0.106969076047)*x[1]
        arg[3,4,0,1]=(-0.912044993941)*x[0]**o+(-0.495323777918)*x[0]+(-0.595831845108)*x[1]**o+(-0.313723426318)*x[1]
        arg[3,4,1,0]=(-0.0730898742548)*x[0]**o+(0.0524008484492)*x[0]+(-0.948232935497)*x[1]**o+(0.952670824572)*x[1]
        arg[3,4,1,1]=(-0.81869438138)*x[0]**o+(-0.894654266952)*x[0]+(0.414078621872)*x[1]**o+(-0.876690817847)*x[1]
        arg[3,4,2,0]=(0.00688155078441)*x[0]**o+(0.845964586551)*x[0]+(0.0183149371892)*x[1]**o+(-0.882931394537)*x[1]
        arg[3,4,2,1]=(-0.734830097149)*x[0]**o+(-0.37824588859)*x[0]+(-0.815402620311)*x[1]**o+(-0.357905685517)*x[1]
        ref[0,0,0,0]=(-0.282394950702)*x_ref[0]**o+(0.836580157584)*x_ref[0]+(0.337652262648)*x_ref[1]**o+(-0.442694263325)*x_ref[1]
        ref[0,0,0,1]=(0.85150774028)*x_ref[0]**o+(0.998297296593)*x_ref[0]+(-0.710029954342)*x_ref[1]**o+(0.0816168287837)*x_ref[1]
        ref[0,0,1,0]=(-0.636435102838)*x_ref[0]**o+(-0.613479863553)*x_ref[0]+(-0.861506513168)*x_ref[1]**o+(-0.242998961478)*x_ref[1]
        ref[0,0,1,1]=(-0.00671863978521)*x_ref[0]**o+(0.909404084068)*x_ref[0]+(-0.734956067026)*x_ref[1]**o+(0.256530538264)*x_ref[1]
        ref[0,0,2,0]=(-0.646631608999)*x_ref[0]**o+(-0.521130455843)*x_ref[0]+(-0.44328404557)*x_ref[1]**o+(-0.156860325817)*x_ref[1]
        ref[0,0,2,1]=(-0.817550008949)*x_ref[0]**o+(0.702414497795)*x_ref[0]+(0.946449680763)*x_ref[1]**o+(-0.61576361925)*x_ref[1]
        ref[0,1,0,0]=(-0.527206691595)*x_ref[0]**o+(-0.43469216984)*x_ref[0]+(-0.108455901154)*x_ref[1]**o+(0.435646962156)*x_ref[1]
        ref[0,1,0,1]=(-0.148522219535)*x_ref[0]**o+(0.146912249755)*x_ref[0]+(-0.414934945665)*x_ref[1]**o+(-0.422754988714)*x_ref[1]
        ref[0,1,1,0]=(0.177760972947)*x_ref[0]**o+(-0.938522530862)*x_ref[0]+(0.566229652264)*x_ref[1]**o+(0.255269743628)*x_ref[1]
        ref[0,1,1,1]=(-0.317807933792)*x_ref[0]**o+(0.432181451246)*x_ref[0]+(-0.358821513362)*x_ref[1]**o+(-0.470517232585)*x_ref[1]
        ref[0,1,2,0]=(0.486536430227)*x_ref[0]**o+(-0.533653809349)*x_ref[0]+(-0.126612514045)*x_ref[1]**o+(0.103088115487)*x_ref[1]
        ref[0,1,2,1]=(-0.995976559589)*x_ref[0]**o+(0.965537167421)*x_ref[0]+(-0.351307699531)*x_ref[1]**o+(-0.684367434666)*x_ref[1]
        ref[0,2,0,0]=(0.976008613768)*x_ref[0]**o+(-0.890573007002)*x_ref[0]+(-0.440709423944)*x_ref[1]**o+(0.315724459367)*x_ref[1]
        ref[0,2,0,1]=(-0.595275811962)*x_ref[0]**o+(-0.599634144509)*x_ref[0]+(0.646466002346)*x_ref[1]**o+(-0.94529568651)*x_ref[1]
        ref[0,2,1,0]=(0.977691521821)*x_ref[0]**o+(-0.0730997643682)*x_ref[0]+(-0.338897498737)*x_ref[1]**o+(0.246308678824)*x_ref[1]
        ref[0,2,1,1]=(-0.111317410732)*x_ref[0]**o+(-0.574381793398)*x_ref[0]+(-0.0314431899843)*x_ref[1]**o+(0.393105047096)*x_ref[1]
        ref[0,2,2,0]=(0.886320020757)*x_ref[0]**o+(-0.785042003544)*x_ref[0]+(0.82609269979)*x_ref[1]**o+(-0.677169484337)*x_ref[1]
        ref[0,2,2,1]=(0.811027138079)*x_ref[0]**o+(-0.375108470948)*x_ref[0]+(0.781472464253)*x_ref[1]**o+(0.217640794143)*x_ref[1]
        ref[0,3,0,0]=(0.264884117091)*x_ref[0]**o+(0.116591319557)*x_ref[0]+(0.282667679763)*x_ref[1]**o+(0.111123454337)*x_ref[1]
        ref[0,3,0,1]=(0.0451626985922)*x_ref[0]**o+(-0.93771279728)*x_ref[0]+(0.129171764072)*x_ref[1]**o+(0.598367954162)*x_ref[1]
        ref[0,3,1,0]=(0.905243511086)*x_ref[0]**o+(0.0837062573586)*x_ref[0]+(0.0657223248433)*x_ref[1]**o+(-0.746644690072)*x_ref[1]
        ref[0,3,1,1]=(-0.0547856755485)*x_ref[0]**o+(-0.22157806755)*x_ref[0]+(0.124623625108)*x_ref[1]**o+(0.0436590157081)*x_ref[1]
        ref[0,3,2,0]=(-0.558323744577)*x_ref[0]**o+(-0.646043519007)*x_ref[0]+(0.991885458004)*x_ref[1]**o+(-0.634140808163)*x_ref[1]
        ref[0,3,2,1]=(-0.27247983028)*x_ref[0]**o+(0.803906404485)*x_ref[0]+(-0.816826609924)*x_ref[1]**o+(-0.969532783336)*x_ref[1]
        ref[0,4,0,0]=(0.638733842891)*x_ref[0]**o+(-0.972793270208)*x_ref[0]+(0.714611008099)*x_ref[1]**o+(0.586988595847)*x_ref[1]
        ref[0,4,0,1]=(0.63584760734)*x_ref[0]**o+(-0.653740236747)*x_ref[0]+(-0.596304731446)*x_ref[1]**o+(0.42519786429)*x_ref[1]
        ref[0,4,1,0]=(0.0665562973529)*x_ref[0]**o+(0.362152092093)*x_ref[0]+(0.732530335739)*x_ref[1]**o+(0.618536326941)*x_ref[1]
        ref[0,4,1,1]=(0.486060667608)*x_ref[0]**o+(0.59489013279)*x_ref[0]+(-0.547237017498)*x_ref[1]**o+(0.0371894705857)*x_ref[1]
        ref[0,4,2,0]=(0.395505844397)*x_ref[0]**o+(-0.869090434535)*x_ref[0]+(-0.606640827437)*x_ref[1]**o+(0.269995459083)*x_ref[1]
        ref[0,4,2,1]=(0.304671547176)*x_ref[0]**o+(-0.515772523849)*x_ref[0]+(0.546114300876)*x_ref[1]**o+(0.530247404341)*x_ref[1]
        ref[1,0,0,0]=(0.408332677433)*x_ref[0]**o+(-0.0778807359918)*x_ref[0]+(-0.803337489297)*x_ref[1]**o+(-0.123835718422)*x_ref[1]
        ref[1,0,0,1]=(-0.00441620254944)*x_ref[0]**o+(0.0321259598831)*x_ref[0]+(-0.998381508472)*x_ref[1]**o+(0.800767747511)*x_ref[1]
        ref[1,0,1,0]=(-0.923683057418)*x_ref[0]**o+(0.579405114902)*x_ref[0]+(-0.939639823585)*x_ref[1]**o+(-0.138423358134)*x_ref[1]
        ref[1,0,1,1]=(-0.558420234848)*x_ref[0]**o+(0.133316569813)*x_ref[0]+(-0.294134717329)*x_ref[1]**o+(-0.750897749044)*x_ref[1]
        ref[1,0,2,0]=(0.478951298545)*x_ref[0]**o+(-0.807173978799)*x_ref[0]+(0.852438146763)*x_ref[1]**o+(-0.103814229563)*x_ref[1]
        ref[1,0,2,1]=(-0.875250726603)*x_ref[0]**o+(-0.934803428491)*x_ref[0]+(0.143352873672)*x_ref[1]**o+(0.600934881206)*x_ref[1]
        ref[1,1,0,0]=(-0.0539476321018)*x_ref[0]**o+(0.194177663417)*x_ref[0]+(-0.862877991009)*x_ref[1]**o+(-0.438337993664)*x_ref[1]
        ref[1,1,0,1]=(-0.623379355474)*x_ref[0]**o+(-0.0746994739748)*x_ref[0]+(0.365005438956)*x_ref[1]**o+(-0.431737688205)*x_ref[1]
        ref[1,1,1,0]=(0.0936054461735)*x_ref[0]**o+(-0.974594269446)*x_ref[0]+(-0.121917598428)*x_ref[1]**o+(-0.84429083727)*x_ref[1]
        ref[1,1,1,1]=(0.967763348672)*x_ref[0]**o+(0.331698826475)*x_ref[0]+(0.66448515877)*x_ref[1]**o+(0.00214985930661)*x_ref[1]
        ref[1,1,2,0]=(-0.312146580762)*x_ref[0]**o+(0.998152287706)*x_ref[0]+(-0.474242942777)*x_ref[1]**o+(0.637827347846)*x_ref[1]
        ref[1,1,2,1]=(-0.556765443436)*x_ref[0]**o+(0.227327929774)*x_ref[0]+(0.739457845569)*x_ref[1]**o+(-0.615070017663)*x_ref[1]
        ref[1,2,0,0]=(-0.23030660552)*x_ref[0]**o+(0.750479465953)*x_ref[0]+(0.278215408392)*x_ref[1]**o+(-0.640618698224)*x_ref[1]
        ref[1,2,0,1]=(-0.0713217101129)*x_ref[0]**o+(0.500737477887)*x_ref[0]+(-0.972834770005)*x_ref[1]**o+(-0.420386056033)*x_ref[1]
        ref[1,2,1,0]=(-0.497322989847)*x_ref[0]**o+(-0.528685124841)*x_ref[0]+(-0.934386348496)*x_ref[1]**o+(0.191930553839)*x_ref[1]
        ref[1,2,1,1]=(0.124362419781)*x_ref[0]**o+(-0.0457849787456)*x_ref[0]+(-0.360596154712)*x_ref[1]**o+(0.799736253715)*x_ref[1]
        ref[1,2,2,0]=(-0.346294878893)*x_ref[0]**o+(0.427772309361)*x_ref[0]+(-0.0642666791012)*x_ref[1]**o+(-0.214442238311)*x_ref[1]
        ref[1,2,2,1]=(0.24434837411)*x_ref[0]**o+(-0.913373159786)*x_ref[0]+(0.158750862391)*x_ref[1]**o+(-0.544328807576)*x_ref[1]
        ref[1,3,0,0]=(0.870847002687)*x_ref[0]**o+(-0.466884462301)*x_ref[0]+(-0.343358829445)*x_ref[1]**o+(-0.180956419766)*x_ref[1]
        ref[1,3,0,1]=(-0.813518850863)*x_ref[0]**o+(0.960810467501)*x_ref[0]+(0.35091416904)*x_ref[1]**o+(0.84534544948)*x_ref[1]
        ref[1,3,1,0]=(-0.284687751218)*x_ref[0]**o+(-0.274583519355)*x_ref[0]+(0.338892764102)*x_ref[1]**o+(-0.145142443753)*x_ref[1]
        ref[1,3,1,1]=(0.921064262137)*x_ref[0]**o+(0.152552421678)*x_ref[0]+(0.827247693375)*x_ref[1]**o+(-0.569529848619)*x_ref[1]
        ref[1,3,2,0]=(0.133567630356)*x_ref[0]**o+(-0.0429397358364)*x_ref[0]+(-0.175383170663)*x_ref[1]**o+(0.584407631334)*x_ref[1]
        ref[1,3,2,1]=(0.203880167371)*x_ref[0]**o+(-0.422978073825)*x_ref[0]+(-0.210109267588)*x_ref[1]**o+(-0.902327373219)*x_ref[1]
        ref[1,4,0,0]=(-0.566386864876)*x_ref[0]**o+(-0.139795085483)*x_ref[0]+(-0.00852913464238)*x_ref[1]**o+(0.569151381884)*x_ref[1]
        ref[1,4,0,1]=(-0.937256625212)*x_ref[0]**o+(0.563438253128)*x_ref[0]+(-0.586182238715)*x_ref[1]**o+(-0.566169678723)*x_ref[1]
        ref[1,4,1,0]=(-0.73296890712)*x_ref[0]**o+(0.359249594406)*x_ref[0]+(0.66050790616)*x_ref[1]**o+(-0.367919797426)*x_ref[1]
        ref[1,4,1,1]=(-0.81165734248)*x_ref[0]**o+(0.460592817838)*x_ref[0]+(0.953573909492)*x_ref[1]**o+(0.329659021274)*x_ref[1]
        ref[1,4,2,0]=(0.345994483595)*x_ref[0]**o+(-0.18547324343)*x_ref[0]+(0.669452926238)*x_ref[1]**o+(0.42513249272)*x_ref[1]
        ref[1,4,2,1]=(0.250677642616)*x_ref[0]**o+(0.466118496803)*x_ref[0]+(-0.776608543595)*x_ref[1]**o+(0.449144243682)*x_ref[1]
        ref[2,0,0,0]=(0.244172909021)*x_ref[0]**o+(0.0311068322898)*x_ref[0]+(-0.909348039508)*x_ref[1]**o+(0.376500719414)*x_ref[1]
        ref[2,0,0,1]=(-0.033516640667)*x_ref[0]**o+(0.698424718328)*x_ref[0]+(-0.282814785542)*x_ref[1]**o+(0.61118022947)*x_ref[1]
        ref[2,0,1,0]=(-0.946672649538)*x_ref[0]**o+(0.814325244591)*x_ref[0]+(-0.353215562825)*x_ref[1]**o+(-0.138150257429)*x_ref[1]
        ref[2,0,1,1]=(-0.886509051692)*x_ref[0]**o+(-0.0468334385819)*x_ref[0]+(0.71984719189)*x_ref[1]**o+(-0.324744971493)*x_ref[1]
        ref[2,0,2,0]=(-0.780921647053)*x_ref[0]**o+(-0.401418496932)*x_ref[0]+(0.647847274332)*x_ref[1]**o+(-0.831834322278)*x_ref[1]
        ref[2,0,2,1]=(0.835155004862)*x_ref[0]**o+(0.684459986902)*x_ref[0]+(-0.301244240677)*x_ref[1]**o+(-0.161292780187)*x_ref[1]
        ref[2,1,0,0]=(-0.652199398394)*x_ref[0]**o+(0.62480722178)*x_ref[0]+(-0.474535712914)*x_ref[1]**o+(0.926061797766)*x_ref[1]
        ref[2,1,0,1]=(-0.813365478568)*x_ref[0]**o+(-0.38795792417)*x_ref[0]+(0.378161353697)*x_ref[1]**o+(0.70581806211)*x_ref[1]
        ref[2,1,1,0]=(0.729265594934)*x_ref[0]**o+(-0.469397842359)*x_ref[0]+(0.642472199926)*x_ref[1]**o+(-0.779527283011)*x_ref[1]
        ref[2,1,1,1]=(0.0517342917574)*x_ref[0]**o+(-0.414111007917)*x_ref[0]+(-0.89437375241)*x_ref[1]**o+(-0.830119585842)*x_ref[1]
        ref[2,1,2,0]=(0.769898486257)*x_ref[0]**o+(0.791725624832)*x_ref[0]+(0.297519329396)*x_ref[1]**o+(-0.959462006611)*x_ref[1]
        ref[2,1,2,1]=(0.817802144251)*x_ref[0]**o+(0.69851258911)*x_ref[0]+(-0.174401735228)*x_ref[1]**o+(0.13449696498)*x_ref[1]
        ref[2,2,0,0]=(0.547972662256)*x_ref[0]**o+(-0.66491929979)*x_ref[0]+(0.330783041672)*x_ref[1]**o+(0.816196730606)*x_ref[1]
        ref[2,2,0,1]=(0.245971097763)*x_ref[0]**o+(0.213087481481)*x_ref[0]+(-0.840766291392)*x_ref[1]**o+(0.388414557376)*x_ref[1]
        ref[2,2,1,0]=(0.523151676444)*x_ref[0]**o+(0.854817486027)*x_ref[0]+(-0.53570188528)*x_ref[1]**o+(-0.68149362242)*x_ref[1]
        ref[2,2,1,1]=(-0.984200247787)*x_ref[0]**o+(0.83324365092)*x_ref[0]+(-0.896184408816)*x_ref[1]**o+(0.102276243487)*x_ref[1]
        ref[2,2,2,0]=(-0.216357164308)*x_ref[0]**o+(-0.597984882107)*x_ref[0]+(-0.205435921511)*x_ref[1]**o+(0.484947652759)*x_ref[1]
        ref[2,2,2,1]=(-0.650046253868)*x_ref[0]**o+(-0.624878186105)*x_ref[0]+(0.786519914406)*x_ref[1]**o+(0.28027273095)*x_ref[1]
        ref[2,3,0,0]=(-0.0821718346579)*x_ref[0]**o+(-0.687049917467)*x_ref[0]+(0.380792333736)*x_ref[1]**o+(0.151185837715)*x_ref[1]
        ref[2,3,0,1]=(0.61796113854)*x_ref[0]**o+(0.844624413065)*x_ref[0]+(-0.980619401098)*x_ref[1]**o+(-0.854924862641)*x_ref[1]
        ref[2,3,1,0]=(0.812293906883)*x_ref[0]**o+(-0.225593208768)*x_ref[0]+(0.964175374163)*x_ref[1]**o+(0.848850064926)*x_ref[1]
        ref[2,3,1,1]=(0.376453572937)*x_ref[0]**o+(0.994818812059)*x_ref[0]+(0.58872018303)*x_ref[1]**o+(0.68332473361)*x_ref[1]
        ref[2,3,2,0]=(-0.00948421972234)*x_ref[0]**o+(-0.680685462738)*x_ref[0]+(-0.767076278167)*x_ref[1]**o+(0.210165437284)*x_ref[1]
        ref[2,3,2,1]=(-0.455199698263)*x_ref[0]**o+(-0.763430401419)*x_ref[0]+(0.183773350212)*x_ref[1]**o+(0.43105940923)*x_ref[1]
        ref[2,4,0,0]=(-0.673962025988)*x_ref[0]**o+(-0.679073069725)*x_ref[0]+(0.11018431715)*x_ref[1]**o+(0.349020527482)*x_ref[1]
        ref[2,4,0,1]=(-0.876691282076)*x_ref[0]**o+(0.920902226631)*x_ref[0]+(-0.477417470246)*x_ref[1]**o+(0.836417982216)*x_ref[1]
        ref[2,4,1,0]=(0.567966851024)*x_ref[0]**o+(0.583241431716)*x_ref[0]+(-0.271851271008)*x_ref[1]**o+(0.678727902863)*x_ref[1]
        ref[2,4,1,1]=(0.707400094)*x_ref[0]**o+(0.802820856444)*x_ref[0]+(0.84518937616)*x_ref[1]**o+(-0.0267193533403)*x_ref[1]
        ref[2,4,2,0]=(0.213268920036)*x_ref[0]**o+(0.780721415328)*x_ref[0]+(0.839292948154)*x_ref[1]**o+(-0.0376334744443)*x_ref[1]
        ref[2,4,2,1]=(0.136255687031)*x_ref[0]**o+(-0.582766278693)*x_ref[0]+(0.257662521321)*x_ref[1]**o+(0.808337521979)*x_ref[1]
        ref[3,0,0,0]=(-0.524068770822)*x_ref[0]**o+(0.942236878842)*x_ref[0]+(0.889496255793)*x_ref[1]**o+(-0.904810742393)*x_ref[1]
        ref[3,0,0,1]=(0.780157465475)*x_ref[0]**o+(-0.763811668832)*x_ref[0]+(-0.517482070639)*x_ref[1]**o+(0.662770780676)*x_ref[1]
        ref[3,0,1,0]=(-0.693227564812)*x_ref[0]**o+(-0.763506211841)*x_ref[0]+(-0.427709881126)*x_ref[1]**o+(0.159109827204)*x_ref[1]
        ref[3,0,1,1]=(0.644975629084)*x_ref[0]**o+(-0.928112761899)*x_ref[0]+(-0.530772740083)*x_ref[1]**o+(-0.723174790087)*x_ref[1]
        ref[3,0,2,0]=(-0.923467274097)*x_ref[0]**o+(0.806128117288)*x_ref[0]+(0.177779363207)*x_ref[1]**o+(0.862243885542)*x_ref[1]
        ref[3,0,2,1]=(-0.0823252901358)*x_ref[0]**o+(0.289022916284)*x_ref[0]+(-0.518508771021)*x_ref[1]**o+(0.174564769845)*x_ref[1]
        ref[3,1,0,0]=(0.608837578444)*x_ref[0]**o+(0.898216758361)*x_ref[0]+(-0.844730109761)*x_ref[1]**o+(0.745974173207)*x_ref[1]
        ref[3,1,0,1]=(-0.11776411078)*x_ref[0]**o+(0.186256502356)*x_ref[0]+(0.565289400333)*x_ref[1]**o+(-0.776173716206)*x_ref[1]
        ref[3,1,1,0]=(-0.170736580152)*x_ref[0]**o+(0.829581661683)*x_ref[0]+(0.310135833859)*x_ref[1]**o+(-0.509629545687)*x_ref[1]
        ref[3,1,1,1]=(-0.225246122751)*x_ref[0]**o+(0.586754377294)*x_ref[0]+(-0.726064725635)*x_ref[1]**o+(0.136952605704)*x_ref[1]
        ref[3,1,2,0]=(-0.287932115478)*x_ref[0]**o+(0.858031396153)*x_ref[0]+(-0.493441745353)*x_ref[1]**o+(0.601827664994)*x_ref[1]
        ref[3,1,2,1]=(0.665503842862)*x_ref[0]**o+(-0.515998970452)*x_ref[0]+(-0.890055558624)*x_ref[1]**o+(-0.0446371451457)*x_ref[1]
        ref[3,2,0,0]=(-0.33741557585)*x_ref[0]**o+(-0.416183752333)*x_ref[0]+(-0.92430073538)*x_ref[1]**o+(-0.486265090904)*x_ref[1]
        ref[3,2,0,1]=(0.910860925994)*x_ref[0]**o+(-0.896869755389)*x_ref[0]+(0.0634658185324)*x_ref[1]**o+(-0.383136455739)*x_ref[1]
        ref[3,2,1,0]=(0.386839297976)*x_ref[0]**o+(-0.190659927071)*x_ref[0]+(-0.304846978185)*x_ref[1]**o+(-0.92705511899)*x_ref[1]
        ref[3,2,1,1]=(-0.0197812018683)*x_ref[0]**o+(-0.479083095775)*x_ref[0]+(0.502489976089)*x_ref[1]**o+(-0.859440665475)*x_ref[1]
        ref[3,2,2,0]=(-0.507672045299)*x_ref[0]**o+(0.498689676072)*x_ref[0]+(-0.719735804645)*x_ref[1]**o+(-0.522053306674)*x_ref[1]
        ref[3,2,2,1]=(0.46207629211)*x_ref[0]**o+(-0.496697066039)*x_ref[0]+(0.994069339904)*x_ref[1]**o+(0.873222754449)*x_ref[1]
        ref[3,3,0,0]=(0.126410029692)*x_ref[0]**o+(0.51067050196)*x_ref[0]+(-0.978384824088)*x_ref[1]**o+(-0.68606930503)*x_ref[1]
        ref[3,3,0,1]=(0.376888157008)*x_ref[0]**o+(-0.14015764025)*x_ref[0]+(0.523667700758)*x_ref[1]**o+(0.0277726855258)*x_ref[1]
        ref[3,3,1,0]=(-0.105865233294)*x_ref[0]**o+(-0.0515230741203)*x_ref[0]+(0.203766525036)*x_ref[1]**o+(-0.0431893014387)*x_ref[1]
        ref[3,3,1,1]=(-0.404351448486)*x_ref[0]**o+(-0.639161429157)*x_ref[0]+(-0.957341091708)*x_ref[1]**o+(-0.103346466984)*x_ref[1]
        ref[3,3,2,0]=(-0.0523508780623)*x_ref[0]**o+(-0.563589575544)*x_ref[0]+(0.158487727325)*x_ref[1]**o+(-0.519455492178)*x_ref[1]
        ref[3,3,2,1]=(-0.823286342218)*x_ref[0]**o+(-0.493663499535)*x_ref[0]+(-0.409825574064)*x_ref[1]**o+(-0.0406101279806)*x_ref[1]
        ref[3,4,0,0]=(0.926212910387)*x_ref[0]**o+(-0.0944507963)*x_ref[0]+(0.640740499391)*x_ref[1]**o+(-0.106969076047)*x_ref[1]
        ref[3,4,0,1]=(-0.912044993941)*x_ref[0]**o+(-0.495323777918)*x_ref[0]+(-0.595831845108)*x_ref[1]**o+(-0.313723426318)*x_ref[1]
        ref[3,4,1,0]=(-0.0730898742548)*x_ref[0]**o+(0.0524008484492)*x_ref[0]+(-0.948232935497)*x_ref[1]**o+(0.952670824572)*x_ref[1]
        ref[3,4,1,1]=(-0.81869438138)*x_ref[0]**o+(-0.894654266952)*x_ref[0]+(0.414078621872)*x_ref[1]**o+(-0.876690817847)*x_ref[1]
        ref[3,4,2,0]=(0.00688155078441)*x_ref[0]**o+(0.845964586551)*x_ref[0]+(0.0183149371892)*x_ref[1]**o+(-0.882931394537)*x_ref[1]
        ref[3,4,2,1]=(-0.734830097149)*x_ref[0]**o+(-0.37824588859)*x_ref[0]+(-0.815402620311)*x_ref[1]**o+(-0.357905685517)*x_ref[1]
      else:
        arg[0,0,0,0]=(-0.110519315354)*x[0]**o+(0.904227374292)*x[0]+(0.459005710707)*x[1]**o+(-0.980231422256)*x[1]+(-0.819713355475)*x[2]**o+(-0.93459517989)*x[2]
        arg[0,0,0,1]=(-0.326902946413)*x[0]**o+(0.489327766219)*x[0]+(-0.965308741061)*x[1]**o+(0.0351399111471)*x[1]+(0.257760096595)*x[2]**o+(0.125123023244)*x[2]
        arg[0,0,1,0]=(-0.584360710004)*x[0]**o+(0.681892732544)*x[0]+(-0.118257741614)*x[1]**o+(0.839875738892)*x[1]+(0.107934154241)*x[2]**o+(0.279757054665)*x[2]
        arg[0,0,1,1]=(0.90973214794)*x[0]**o+(-0.776804618826)*x[0]+(-0.910633853381)*x[1]**o+(-0.25678788529)*x[1]+(0.783709363646)*x[2]**o+(-0.922660855915)*x[2]
        arg[0,0,2,0]=(-0.802273263394)*x[0]**o+(-0.917749665952)*x[0]+(0.796206817226)*x[1]**o+(0.917284637885)*x[1]+(0.219841508893)*x[2]**o+(0.0655848200083)*x[2]
        arg[0,0,2,1]=(0.550596792343)*x[0]**o+(-0.760207057617)*x[0]+(0.0368469683098)*x[1]**o+(0.509347717331)*x[1]+(0.813780778487)*x[2]**o+(-0.432656298451)*x[2]
        arg[0,1,0,0]=(-0.238502042941)*x[0]**o+(-0.966632221032)*x[0]+(-0.373745652933)*x[1]**o+(-0.436176336389)*x[1]+(0.124280197699)*x[2]**o+(0.552700562803)*x[2]
        arg[0,1,0,1]=(-0.19848053276)*x[0]**o+(0.110167634957)*x[0]+(-0.748553857473)*x[1]**o+(0.355932508959)*x[1]+(-0.226987914753)*x[2]**o+(-0.68870238197)*x[2]
        arg[0,1,1,0]=(0.365822518882)*x[0]**o+(0.327285102897)*x[0]+(0.549212504635)*x[1]**o+(0.918179946504)*x[1]+(0.20644004679)*x[2]**o+(0.821299409756)*x[2]
        arg[0,1,1,1]=(-0.366167489101)*x[0]**o+(0.529933186572)*x[0]+(-0.604965310967)*x[1]**o+(-0.833093092352)*x[1]+(-0.73593913912)*x[2]**o+(0.872325614453)*x[2]
        arg[0,1,2,0]=(0.289123646902)*x[0]**o+(0.705862153789)*x[0]+(-0.319756886822)*x[1]**o+(-0.453220232524)*x[1]+(-0.192336374167)*x[2]**o+(0.363575495006)*x[2]
        arg[0,1,2,1]=(-0.213014833353)*x[0]**o+(-0.0706784230627)*x[0]+(-0.941272982229)*x[1]**o+(-0.96641361036)*x[1]+(0.289919014964)*x[2]**o+(0.57658772066)*x[2]
        arg[0,2,0,0]=(-0.406683991704)*x[0]**o+(-0.0110042213592)*x[0]+(-0.273148083258)*x[1]**o+(0.054257030447)*x[1]+(0.387598147681)*x[2]**o+(0.480398109069)*x[2]
        arg[0,2,0,1]=(0.318713057562)*x[0]**o+(-0.449467749585)*x[0]+(0.669991267829)*x[1]**o+(0.237986860249)*x[1]+(0.894370523471)*x[2]**o+(-0.397832436233)*x[2]
        arg[0,2,1,0]=(-0.858603253642)*x[0]**o+(-0.713122015919)*x[0]+(0.870251906188)*x[1]**o+(-0.781601254578)*x[1]+(-0.504531525822)*x[2]**o+(-0.699819349556)*x[2]
        arg[0,2,1,1]=(-0.825591116361)*x[0]**o+(-0.552452876394)*x[0]+(-0.262627261618)*x[1]**o+(-0.147127256184)*x[1]+(-0.171485665103)*x[2]**o+(0.823279741501)*x[2]
        arg[0,2,2,0]=(-0.0416279689068)*x[0]**o+(-0.0903338000001)*x[0]+(0.911673780635)*x[1]**o+(0.86298673049)*x[1]+(-0.60251179847)*x[2]**o+(0.378303877746)*x[2]
        arg[0,2,2,1]=(0.261384106387)*x[0]**o+(0.0326416199811)*x[0]+(-0.426740102269)*x[1]**o+(-0.445401626921)*x[1]+(-0.388668254526)*x[2]**o+(-0.977942913284)*x[2]
        arg[0,3,0,0]=(-0.0195053813483)*x[0]**o+(-0.294061998664)*x[0]+(0.790461098426)*x[1]**o+(-0.506334268838)*x[1]+(0.394015541605)*x[2]**o+(-0.562248105432)*x[2]
        arg[0,3,0,1]=(0.390726516735)*x[0]**o+(-0.757597437859)*x[0]+(0.861882149861)*x[1]**o+(0.621870451314)*x[1]+(0.900777666162)*x[2]**o+(0.271507131938)*x[2]
        arg[0,3,1,0]=(-0.725635966475)*x[0]**o+(-0.695837548737)*x[0]+(-0.163078176328)*x[1]**o+(-0.0435808418729)*x[1]+(0.0890756544611)*x[2]**o+(-0.270422627949)*x[2]
        arg[0,3,1,1]=(-0.175498720647)*x[0]**o+(-0.213605441632)*x[0]+(-0.358632620362)*x[1]**o+(-0.193273004152)*x[1]+(0.937704823805)*x[2]**o+(0.861707832291)*x[2]
        arg[0,3,2,0]=(0.0976010265611)*x[0]**o+(0.995135925341)*x[0]+(0.755339287387)*x[1]**o+(-0.924722068193)*x[1]+(-0.480212181904)*x[2]**o+(-0.40237760439)*x[2]
        arg[0,3,2,1]=(-0.679215392952)*x[0]**o+(-0.00796096253297)*x[0]+(-0.964972599421)*x[1]**o+(-0.410415435656)*x[1]+(-0.292638067785)*x[2]**o+(-0.950383976561)*x[2]
        arg[0,4,0,0]=(0.245586594928)*x[0]**o+(-0.593420401896)*x[0]+(0.553450486534)*x[1]**o+(0.173814965689)*x[1]+(0.0719392610563)*x[2]**o+(0.676231437486)*x[2]
        arg[0,4,0,1]=(0.154382756425)*x[0]**o+(-0.0655740293432)*x[0]+(0.924689745142)*x[1]**o+(-0.246876531175)*x[1]+(-0.241460713543)*x[2]**o+(-0.489840108568)*x[2]
        arg[0,4,1,0]=(-0.482705840359)*x[0]**o+(-0.392960380114)*x[0]+(-0.503463823972)*x[1]**o+(0.317163204738)*x[1]+(0.775048877356)*x[2]**o+(0.599889732326)*x[2]
        arg[0,4,1,1]=(-0.122555861716)*x[0]**o+(0.930175770449)*x[0]+(0.483503764941)*x[1]**o+(0.340001033217)*x[1]+(0.672924280979)*x[2]**o+(0.746489359683)*x[2]
        arg[0,4,2,0]=(-0.339421141033)*x[0]**o+(-0.604966575169)*x[0]+(0.862413017648)*x[1]**o+(0.936752078454)*x[1]+(-0.0549628353285)*x[2]**o+(-0.0858885835314)*x[2]
        arg[0,4,2,1]=(-0.581767357421)*x[0]**o+(-0.644286027548)*x[0]+(0.76948396865)*x[1]**o+(0.692265904744)*x[1]+(0.955871091531)*x[2]**o+(-0.378888503457)*x[2]
        arg[1,0,0,0]=(-0.0407846595046)*x[0]**o+(-0.293709943251)*x[0]+(-0.310552931582)*x[1]**o+(-0.696244178466)*x[1]+(0.962985284252)*x[2]**o+(-0.0266457621404)*x[2]
        arg[1,0,0,1]=(-0.480942811584)*x[0]**o+(-0.400158759679)*x[0]+(0.311171869807)*x[1]**o+(-0.79518809427)*x[1]+(-0.497004450478)*x[2]**o+(-0.639822005752)*x[2]
        arg[1,0,1,0]=(0.158738648483)*x[0]**o+(0.274859816468)*x[0]+(-0.208677612654)*x[1]**o+(0.911107598366)*x[1]+(-0.400163007778)*x[2]**o+(0.276296890267)*x[2]
        arg[1,0,1,1]=(0.613312276864)*x[0]**o+(-0.644237218776)*x[0]+(0.81927272895)*x[1]**o+(-0.12971860556)*x[1]+(0.214379406799)*x[2]**o+(0.856335675043)*x[2]
        arg[1,0,2,0]=(-0.699925173766)*x[0]**o+(-0.560613547952)*x[0]+(-0.355325107637)*x[1]**o+(-0.598747822041)*x[1]+(0.243087037553)*x[2]**o+(-0.484176993015)*x[2]
        arg[1,0,2,1]=(-0.218912300848)*x[0]**o+(-0.259483312969)*x[0]+(-0.863788186295)*x[1]**o+(0.924423774757)*x[1]+(-0.0823752192204)*x[2]**o+(-0.928377900602)*x[2]
        arg[1,1,0,0]=(-0.368901144425)*x[0]**o+(0.544104655592)*x[0]+(0.321295151176)*x[1]**o+(0.395692754134)*x[1]+(-0.721319111754)*x[2]**o+(-0.698518442137)*x[2]
        arg[1,1,0,1]=(-0.592867336047)*x[0]**o+(0.145120231038)*x[0]+(0.110390771366)*x[1]**o+(0.168333368086)*x[1]+(-0.577866419921)*x[2]**o+(0.705027708469)*x[2]
        arg[1,1,1,0]=(-0.210603428235)*x[0]**o+(-0.250156447116)*x[0]+(-0.397803248976)*x[1]**o+(0.332163380867)*x[1]+(-0.250648216541)*x[2]**o+(-0.0693729055697)*x[2]
        arg[1,1,1,1]=(0.324927558096)*x[0]**o+(-0.262726116223)*x[0]+(-0.555439758488)*x[1]**o+(-0.865019248609)*x[1]+(-0.0971907425857)*x[2]**o+(-0.742359236711)*x[2]
        arg[1,1,2,0]=(0.161058869414)*x[0]**o+(0.349191500791)*x[0]+(0.468336473315)*x[1]**o+(-0.856364775995)*x[1]+(0.491736696689)*x[2]**o+(0.352984104173)*x[2]
        arg[1,1,2,1]=(-0.243275976508)*x[0]**o+(-0.785605388878)*x[0]+(-0.282478919296)*x[1]**o+(-0.728851522341)*x[1]+(0.375134848237)*x[2]**o+(-0.486200561256)*x[2]
        arg[1,2,0,0]=(-0.172241451405)*x[0]**o+(0.422361498196)*x[0]+(0.464862755164)*x[1]**o+(0.466661717105)*x[1]+(-0.886209677512)*x[2]**o+(-0.43529076789)*x[2]
        arg[1,2,0,1]=(0.946582222405)*x[0]**o+(0.513640973894)*x[0]+(-0.923788000633)*x[1]**o+(-0.358163567195)*x[1]+(-0.709999132129)*x[2]**o+(-0.761163299694)*x[2]
        arg[1,2,1,0]=(0.236339981889)*x[0]**o+(0.259874866134)*x[0]+(-0.32460762691)*x[1]**o+(-0.629786950313)*x[1]+(-0.679261685402)*x[2]**o+(0.593069192918)*x[2]
        arg[1,2,1,1]=(0.11006800009)*x[0]**o+(0.970532991787)*x[0]+(-0.467608188361)*x[1]**o+(-0.959281984552)*x[1]+(0.610396622519)*x[2]**o+(0.282468600058)*x[2]
        arg[1,2,2,0]=(-0.656419157742)*x[0]**o+(0.234027308171)*x[0]+(-0.67027206978)*x[1]**o+(-0.746034436817)*x[1]+(-0.771894073652)*x[2]**o+(0.949545830287)*x[2]
        arg[1,2,2,1]=(0.615884236811)*x[0]**o+(-0.670156612555)*x[0]+(0.543165493345)*x[1]**o+(-0.618294277594)*x[1]+(-0.974114255061)*x[2]**o+(-0.762320542298)*x[2]
        arg[1,3,0,0]=(0.096201957648)*x[0]**o+(0.440870816208)*x[0]+(0.124448859607)*x[1]**o+(0.507919407561)*x[1]+(-0.401666263687)*x[2]**o+(-0.437653713723)*x[2]
        arg[1,3,0,1]=(0.165129229294)*x[0]**o+(-0.353217428955)*x[0]+(0.473666065504)*x[1]**o+(0.907667166988)*x[1]+(-0.864298533288)*x[2]**o+(0.641505117281)*x[2]
        arg[1,3,1,0]=(0.381900624425)*x[0]**o+(0.427520178597)*x[0]+(-0.560279540293)*x[1]**o+(0.190814532271)*x[1]+(-0.187947688629)*x[2]**o+(-0.127694148593)*x[2]
        arg[1,3,1,1]=(-0.903921363895)*x[0]**o+(-0.659942040828)*x[0]+(-0.549460213492)*x[1]**o+(0.869909651895)*x[1]+(-0.962273439484)*x[2]**o+(-0.046365857751)*x[2]
        arg[1,3,2,0]=(-0.021688482187)*x[0]**o+(-0.209531000717)*x[0]+(0.668564395133)*x[1]**o+(-0.0347978700975)*x[1]+(0.947677147488)*x[2]**o+(-0.0152385052475)*x[2]
        arg[1,3,2,1]=(0.880839485817)*x[0]**o+(-0.319275929312)*x[0]+(-0.940056657485)*x[1]**o+(0.570855397726)*x[1]+(0.279710812074)*x[2]**o+(0.496622215051)*x[2]
        arg[1,4,0,0]=(-0.139033645615)*x[0]**o+(-0.271453722366)*x[0]+(-0.268516720244)*x[1]**o+(-0.0467555017438)*x[1]+(-0.41618262265)*x[2]**o+(0.315320363813)*x[2]
        arg[1,4,0,1]=(0.296108607204)*x[0]**o+(-0.184582165661)*x[0]+(0.121416543744)*x[1]**o+(-0.922284926751)*x[1]+(-0.437600089668)*x[2]**o+(-0.608800626513)*x[2]
        arg[1,4,1,0]=(0.8942028423)*x[0]**o+(0.673402831188)*x[0]+(-0.622529195562)*x[1]**o+(-0.266052338798)*x[1]+(-0.273167851702)*x[2]**o+(-0.312211982747)*x[2]
        arg[1,4,1,1]=(-0.102315711178)*x[0]**o+(-0.885515719371)*x[0]+(-0.532910101204)*x[1]**o+(0.798525944348)*x[1]+(0.640586802303)*x[2]**o+(0.36308806536)*x[2]
        arg[1,4,2,0]=(0.376793506541)*x[0]**o+(-0.667063564875)*x[0]+(0.609591985816)*x[1]**o+(0.28347745142)*x[1]+(-0.751832936302)*x[2]**o+(-0.683738856965)*x[2]
        arg[1,4,2,1]=(-0.51412252819)*x[0]**o+(0.559560297206)*x[0]+(0.751134603429)*x[1]**o+(-0.0031812310797)*x[1]+(-0.525776122178)*x[2]**o+(0.338471942848)*x[2]
        arg[2,0,0,0]=(0.937572244905)*x[0]**o+(-0.293020870379)*x[0]+(-0.918591520817)*x[1]**o+(0.622194477261)*x[1]+(0.31641482851)*x[2]**o+(0.276912266864)*x[2]
        arg[2,0,0,1]=(0.853041599998)*x[0]**o+(-0.0220623724171)*x[0]+(0.89909044296)*x[1]**o+(-0.838782642732)*x[1]+(0.0133197592042)*x[2]**o+(-0.868977618993)*x[2]
        arg[2,0,1,0]=(-0.325799042976)*x[0]**o+(0.746269971994)*x[0]+(0.882671061757)*x[1]**o+(-0.284133373252)*x[1]+(0.120086549521)*x[2]**o+(-0.397352610969)*x[2]
        arg[2,0,1,1]=(0.282336934951)*x[0]**o+(-0.339893978547)*x[0]+(-0.0861660125386)*x[1]**o+(0.283063579798)*x[1]+(0.714762478569)*x[2]**o+(-0.497021718549)*x[2]
        arg[2,0,2,0]=(0.891782779491)*x[0]**o+(-0.817647577644)*x[0]+(0.364088142085)*x[1]**o+(-0.950577637048)*x[1]+(0.4070663557)*x[2]**o+(0.270753179246)*x[2]
        arg[2,0,2,1]=(0.316594913808)*x[0]**o+(0.310731486851)*x[0]+(0.250421828765)*x[1]**o+(0.772786141458)*x[1]+(0.64843040679)*x[2]**o+(-0.691903332511)*x[2]
        arg[2,1,0,0]=(0.87180715028)*x[0]**o+(-0.953173230375)*x[0]+(-0.823516344548)*x[1]**o+(0.934653756548)*x[1]+(-0.537124962489)*x[2]**o+(-0.233365939659)*x[2]
        arg[2,1,0,1]=(-0.407189744631)*x[0]**o+(-0.754425720288)*x[0]+(0.737526767897)*x[1]**o+(0.745393943502)*x[1]+(0.811744293685)*x[2]**o+(-0.459556659177)*x[2]
        arg[2,1,1,0]=(-0.693529259877)*x[0]**o+(0.580197938048)*x[0]+(0.506055440405)*x[1]**o+(0.122531014759)*x[1]+(0.427736137775)*x[2]**o+(-0.83260140401)*x[2]
        arg[2,1,1,1]=(0.542621975026)*x[0]**o+(-0.568425713106)*x[0]+(0.303002043446)*x[1]**o+(0.363286261651)*x[1]+(0.2729476289)*x[2]**o+(-0.0113629744659)*x[2]
        arg[2,1,2,0]=(-0.266750235694)*x[0]**o+(0.10116400306)*x[0]+(0.315977962897)*x[1]**o+(-0.406236201454)*x[1]+(-0.586845072374)*x[2]**o+(0.513937047282)*x[2]
        arg[2,1,2,1]=(-0.518780811975)*x[0]**o+(0.275053640829)*x[0]+(0.4234970646)*x[1]**o+(0.389585163376)*x[1]+(-0.609771068778)*x[2]**o+(-0.121467061755)*x[2]
        arg[2,2,0,0]=(0.138823207466)*x[0]**o+(-0.779837455014)*x[0]+(0.70167049555)*x[1]**o+(0.766428813369)*x[1]+(0.114023530366)*x[2]**o+(0.650098536187)*x[2]
        arg[2,2,0,1]=(0.974998667633)*x[0]**o+(0.958471050654)*x[0]+(0.193515162503)*x[1]**o+(-0.252148167253)*x[1]+(0.495475460125)*x[2]**o+(0.562693347983)*x[2]
        arg[2,2,1,0]=(-0.552379548988)*x[0]**o+(-0.831399971618)*x[0]+(0.437581216647)*x[1]**o+(-0.775331267696)*x[1]+(0.729041628844)*x[2]**o+(0.633113359893)*x[2]
        arg[2,2,1,1]=(0.379858437049)*x[0]**o+(0.923586504203)*x[0]+(0.707244783969)*x[1]**o+(-0.293157375059)*x[1]+(-0.368717941876)*x[2]**o+(-0.202610288616)*x[2]
        arg[2,2,2,0]=(-0.804826108231)*x[0]**o+(-0.0208712096312)*x[0]+(0.0482547072498)*x[1]**o+(0.852366543609)*x[1]+(0.328962500136)*x[2]**o+(-0.132777835512)*x[2]
        arg[2,2,2,1]=(0.91373752217)*x[0]**o+(-0.792112120729)*x[0]+(-0.0457564087072)*x[1]**o+(0.309694852949)*x[1]+(0.089219285337)*x[2]**o+(-0.516000514614)*x[2]
        arg[2,3,0,0]=(0.343215129115)*x[0]**o+(0.40804099837)*x[0]+(-0.865942442914)*x[1]**o+(0.842241683043)*x[1]+(0.0727803297601)*x[2]**o+(-0.395225682949)*x[2]
        arg[2,3,0,1]=(-0.965340522771)*x[0]**o+(-0.897056478677)*x[0]+(-0.305205944062)*x[1]**o+(0.393941789685)*x[1]+(-0.253328379131)*x[2]**o+(0.0951510469271)*x[2]
        arg[2,3,1,0]=(0.536251514752)*x[0]**o+(0.0629776386779)*x[0]+(-0.706311222379)*x[1]**o+(-0.84362242029)*x[1]+(0.00110678760539)*x[2]**o+(-0.30286059941)*x[2]
        arg[2,3,1,1]=(0.114584737295)*x[0]**o+(0.0708763738067)*x[0]+(-0.454760062397)*x[1]**o+(0.228742539922)*x[1]+(0.514960810352)*x[2]**o+(-0.815749211085)*x[2]
        arg[2,3,2,0]=(0.246335688008)*x[0]**o+(-0.239199299885)*x[0]+(-0.447390151659)*x[1]**o+(0.918949356391)*x[1]+(-0.307864840175)*x[2]**o+(0.560139080481)*x[2]
        arg[2,3,2,1]=(0.118766035723)*x[0]**o+(0.610711164899)*x[0]+(0.336628771776)*x[1]**o+(0.833699268751)*x[1]+(-0.424694647429)*x[2]**o+(-0.639929688104)*x[2]
        arg[2,4,0,0]=(0.814543506987)*x[0]**o+(-0.797684321368)*x[0]+(0.337242635783)*x[1]**o+(0.278139837072)*x[1]+(0.633511526441)*x[2]**o+(0.372175615028)*x[2]
        arg[2,4,0,1]=(-0.0590668694094)*x[0]**o+(0.256732067267)*x[0]+(0.519737187577)*x[1]**o+(-0.376391855948)*x[1]+(-0.810934658651)*x[2]**o+(0.826899476928)*x[2]
        arg[2,4,1,0]=(-0.428976843661)*x[0]**o+(0.672667833313)*x[0]+(0.263194890946)*x[1]**o+(-0.894790773661)*x[1]+(-0.550298557295)*x[2]**o+(-0.00560384035502)*x[2]
        arg[2,4,1,1]=(-0.818372430144)*x[0]**o+(-0.829920833073)*x[0]+(-0.359560636119)*x[1]**o+(-0.113484563205)*x[1]+(0.586796110365)*x[2]**o+(0.296820898426)*x[2]
        arg[2,4,2,0]=(-0.691597969566)*x[0]**o+(0.715813153538)*x[0]+(0.954093612503)*x[1]**o+(0.88803420754)*x[1]+(0.697447667765)*x[2]**o+(0.793730496625)*x[2]
        arg[2,4,2,1]=(0.351099833336)*x[0]**o+(-0.716419272807)*x[0]+(0.318726674998)*x[1]**o+(0.246151914414)*x[1]+(-0.945520673884)*x[2]**o+(-0.489026233954)*x[2]
        arg[3,0,0,0]=(0.511598022804)*x[0]**o+(0.567617269582)*x[0]+(-0.526293319137)*x[1]**o+(0.333750098146)*x[1]+(-0.0869228849144)*x[2]**o+(-0.204072834867)*x[2]
        arg[3,0,0,1]=(0.905384007172)*x[0]**o+(-0.661067390314)*x[0]+(-0.470790632295)*x[1]**o+(-0.149608017616)*x[1]+(0.427879278267)*x[2]**o+(0.987040179184)*x[2]
        arg[3,0,1,0]=(-0.946562006809)*x[0]**o+(0.64141909633)*x[0]+(-0.762076826276)*x[1]**o+(-0.642912850794)*x[1]+(0.712567898559)*x[2]**o+(-0.292128944205)*x[2]
        arg[3,0,1,1]=(-0.955087949361)*x[0]**o+(0.426588041133)*x[0]+(0.339313104127)*x[1]**o+(-0.375147344045)*x[1]+(0.967259014679)*x[2]**o+(0.504521081858)*x[2]
        arg[3,0,2,0]=(0.141876598344)*x[0]**o+(-0.335579252825)*x[0]+(0.514552993602)*x[1]**o+(-0.531116948499)*x[1]+(-0.369319413141)*x[2]**o+(0.0374413262362)*x[2]
        arg[3,0,2,1]=(0.678360463658)*x[0]**o+(-0.357900119646)*x[0]+(0.300750717059)*x[1]**o+(0.516196189448)*x[1]+(0.0535466018203)*x[2]**o+(-0.149961940786)*x[2]
        arg[3,1,0,0]=(0.0975994351079)*x[0]**o+(0.634544068499)*x[0]+(-0.963217046352)*x[1]**o+(-0.804658091456)*x[1]+(-0.620132720823)*x[2]**o+(0.862250619369)*x[2]
        arg[3,1,0,1]=(0.210306328879)*x[0]**o+(-0.457341187966)*x[0]+(0.957556084801)*x[1]**o+(0.704073170928)*x[1]+(0.423134492133)*x[2]**o+(-0.167268375412)*x[2]
        arg[3,1,1,0]=(-0.909228817744)*x[0]**o+(0.110665018921)*x[0]+(0.36275630147)*x[1]**o+(0.346866824763)*x[1]+(-0.535600547489)*x[2]**o+(0.627638814375)*x[2]
        arg[3,1,1,1]=(0.328780933823)*x[0]**o+(-0.249563994161)*x[0]+(-0.555789062892)*x[1]**o+(-0.917287525036)*x[1]+(0.386191175017)*x[2]**o+(0.123904319061)*x[2]
        arg[3,1,2,0]=(-0.838409901623)*x[0]**o+(0.673479705393)*x[0]+(0.482921104917)*x[1]**o+(0.136411388588)*x[1]+(0.583674116225)*x[2]**o+(0.305100836877)*x[2]
        arg[3,1,2,1]=(0.952961008312)*x[0]**o+(-0.0501981079532)*x[0]+(0.661434464642)*x[1]**o+(0.65030469399)*x[1]+(-0.157562001295)*x[2]**o+(-0.249551892917)*x[2]
        arg[3,2,0,0]=(-0.0232804500286)*x[0]**o+(-0.559229392203)*x[0]+(0.127967930273)*x[1]**o+(-0.635827181645)*x[1]+(-0.0426703224383)*x[2]**o+(0.47720230258)*x[2]
        arg[3,2,0,1]=(-0.421658996106)*x[0]**o+(-0.468260038077)*x[0]+(0.479083572554)*x[1]**o+(-0.214417881841)*x[1]+(0.372430534261)*x[2]**o+(0.133063789546)*x[2]
        arg[3,2,1,0]=(0.905811586567)*x[0]**o+(0.245447340384)*x[0]+(-0.337650999146)*x[1]**o+(0.908188235976)*x[1]+(-0.804029751277)*x[2]**o+(0.0512676456643)*x[2]
        arg[3,2,1,1]=(-0.891486839409)*x[0]**o+(0.20753794329)*x[0]+(0.879183464252)*x[1]**o+(0.902028906992)*x[1]+(0.416776813598)*x[2]**o+(0.491964374373)*x[2]
        arg[3,2,2,0]=(0.653067027119)*x[0]**o+(0.765293890745)*x[0]+(-0.348902169508)*x[1]**o+(0.324429035849)*x[1]+(-0.363026365065)*x[2]**o+(0.0816186405511)*x[2]
        arg[3,2,2,1]=(-0.0415375052325)*x[0]**o+(-0.480779075784)*x[0]+(-0.137223552938)*x[1]**o+(-0.915733625512)*x[1]+(0.750199445159)*x[2]**o+(-0.789818945171)*x[2]
        arg[3,3,0,0]=(0.0744921239083)*x[0]**o+(0.0700589144159)*x[0]+(-0.327105425297)*x[1]**o+(0.469843257765)*x[1]+(-0.170191600844)*x[2]**o+(0.732071534155)*x[2]
        arg[3,3,0,1]=(0.172643927441)*x[0]**o+(0.848924114383)*x[0]+(-0.79125762228)*x[1]**o+(-0.694725775976)*x[1]+(-0.309710503985)*x[2]**o+(-0.405310234927)*x[2]
        arg[3,3,1,0]=(-0.465911787595)*x[0]**o+(-0.71080474709)*x[0]+(0.959925934203)*x[1]**o+(-0.111110644825)*x[1]+(-0.145349925218)*x[2]**o+(0.122955165811)*x[2]
        arg[3,3,1,1]=(-0.769111863754)*x[0]**o+(0.348036938532)*x[0]+(0.485890856474)*x[1]**o+(0.382595218985)*x[1]+(-0.897488692981)*x[2]**o+(0.830822333819)*x[2]
        arg[3,3,2,0]=(0.341890793681)*x[0]**o+(-0.667145936571)*x[0]+(-0.145094108268)*x[1]**o+(0.771433911212)*x[1]+(-0.299752164325)*x[2]**o+(-0.357856549677)*x[2]
        arg[3,3,2,1]=(0.26381838796)*x[0]**o+(-0.44474724998)*x[0]+(0.310283988195)*x[1]**o+(0.698675181314)*x[1]+(0.437624133238)*x[2]**o+(0.394916054445)*x[2]
        arg[3,4,0,0]=(-0.576499406674)*x[0]**o+(0.770936634959)*x[0]+(0.707441735783)*x[1]**o+(-0.794953257043)*x[1]+(-0.535342943209)*x[2]**o+(-0.509446811787)*x[2]
        arg[3,4,0,1]=(0.161115012113)*x[0]**o+(0.11079092252)*x[0]+(0.0633804181164)*x[1]**o+(-0.406049293019)*x[1]+(-0.824283994336)*x[2]**o+(-0.481107560499)*x[2]
        arg[3,4,1,0]=(-0.411133810295)*x[0]**o+(0.988486625099)*x[0]+(-0.36050191367)*x[1]**o+(0.469479821965)*x[1]+(-0.431175357325)*x[2]**o+(-0.966295015263)*x[2]
        arg[3,4,1,1]=(0.233731410367)*x[0]**o+(-0.821900740766)*x[0]+(-0.823123461385)*x[1]**o+(-0.380537914421)*x[1]+(-0.896146300355)*x[2]**o+(-0.775172990257)*x[2]
        arg[3,4,2,0]=(-0.96323363586)*x[0]**o+(0.0935412405245)*x[0]+(0.929333026259)*x[1]**o+(-0.322236627348)*x[1]+(-0.390898135964)*x[2]**o+(-0.766178290162)*x[2]
        arg[3,4,2,1]=(-0.773670533268)*x[0]**o+(-0.156975157098)*x[0]+(-0.91512996795)*x[1]**o+(0.0247803682126)*x[1]+(0.995159733782)*x[2]**o+(-0.669224144225)*x[2]
        ref[0,0,0,0]=(-0.110519315354)*x_ref[0]**o+(0.904227374292)*x_ref[0]+(0.459005710707)*x_ref[1]**o+(-0.980231422256)*x_ref[1]+(-0.819713355475)*x_ref[2]**o+(-0.93459517989)*x_ref[2]
        ref[0,0,0,1]=(-0.326902946413)*x_ref[0]**o+(0.489327766219)*x_ref[0]+(-0.965308741061)*x_ref[1]**o+(0.0351399111471)*x_ref[1]+(0.257760096595)*x_ref[2]**o+(0.125123023244)*x_ref[2]
        ref[0,0,1,0]=(-0.584360710004)*x_ref[0]**o+(0.681892732544)*x_ref[0]+(-0.118257741614)*x_ref[1]**o+(0.839875738892)*x_ref[1]+(0.107934154241)*x_ref[2]**o+(0.279757054665)*x_ref[2]
        ref[0,0,1,1]=(0.90973214794)*x_ref[0]**o+(-0.776804618826)*x_ref[0]+(-0.910633853381)*x_ref[1]**o+(-0.25678788529)*x_ref[1]+(0.783709363646)*x_ref[2]**o+(-0.922660855915)*x_ref[2]
        ref[0,0,2,0]=(-0.802273263394)*x_ref[0]**o+(-0.917749665952)*x_ref[0]+(0.796206817226)*x_ref[1]**o+(0.917284637885)*x_ref[1]+(0.219841508893)*x_ref[2]**o+(0.0655848200083)*x_ref[2]
        ref[0,0,2,1]=(0.550596792343)*x_ref[0]**o+(-0.760207057617)*x_ref[0]+(0.0368469683098)*x_ref[1]**o+(0.509347717331)*x_ref[1]+(0.813780778487)*x_ref[2]**o+(-0.432656298451)*x_ref[2]
        ref[0,1,0,0]=(-0.238502042941)*x_ref[0]**o+(-0.966632221032)*x_ref[0]+(-0.373745652933)*x_ref[1]**o+(-0.436176336389)*x_ref[1]+(0.124280197699)*x_ref[2]**o+(0.552700562803)*x_ref[2]
        ref[0,1,0,1]=(-0.19848053276)*x_ref[0]**o+(0.110167634957)*x_ref[0]+(-0.748553857473)*x_ref[1]**o+(0.355932508959)*x_ref[1]+(-0.226987914753)*x_ref[2]**o+(-0.68870238197)*x_ref[2]
        ref[0,1,1,0]=(0.365822518882)*x_ref[0]**o+(0.327285102897)*x_ref[0]+(0.549212504635)*x_ref[1]**o+(0.918179946504)*x_ref[1]+(0.20644004679)*x_ref[2]**o+(0.821299409756)*x_ref[2]
        ref[0,1,1,1]=(-0.366167489101)*x_ref[0]**o+(0.529933186572)*x_ref[0]+(-0.604965310967)*x_ref[1]**o+(-0.833093092352)*x_ref[1]+(-0.73593913912)*x_ref[2]**o+(0.872325614453)*x_ref[2]
        ref[0,1,2,0]=(0.289123646902)*x_ref[0]**o+(0.705862153789)*x_ref[0]+(-0.319756886822)*x_ref[1]**o+(-0.453220232524)*x_ref[1]+(-0.192336374167)*x_ref[2]**o+(0.363575495006)*x_ref[2]
        ref[0,1,2,1]=(-0.213014833353)*x_ref[0]**o+(-0.0706784230627)*x_ref[0]+(-0.941272982229)*x_ref[1]**o+(-0.96641361036)*x_ref[1]+(0.289919014964)*x_ref[2]**o+(0.57658772066)*x_ref[2]
        ref[0,2,0,0]=(-0.406683991704)*x_ref[0]**o+(-0.0110042213592)*x_ref[0]+(-0.273148083258)*x_ref[1]**o+(0.054257030447)*x_ref[1]+(0.387598147681)*x_ref[2]**o+(0.480398109069)*x_ref[2]
        ref[0,2,0,1]=(0.318713057562)*x_ref[0]**o+(-0.449467749585)*x_ref[0]+(0.669991267829)*x_ref[1]**o+(0.237986860249)*x_ref[1]+(0.894370523471)*x_ref[2]**o+(-0.397832436233)*x_ref[2]
        ref[0,2,1,0]=(-0.858603253642)*x_ref[0]**o+(-0.713122015919)*x_ref[0]+(0.870251906188)*x_ref[1]**o+(-0.781601254578)*x_ref[1]+(-0.504531525822)*x_ref[2]**o+(-0.699819349556)*x_ref[2]
        ref[0,2,1,1]=(-0.825591116361)*x_ref[0]**o+(-0.552452876394)*x_ref[0]+(-0.262627261618)*x_ref[1]**o+(-0.147127256184)*x_ref[1]+(-0.171485665103)*x_ref[2]**o+(0.823279741501)*x_ref[2]
        ref[0,2,2,0]=(-0.0416279689068)*x_ref[0]**o+(-0.0903338000001)*x_ref[0]+(0.911673780635)*x_ref[1]**o+(0.86298673049)*x_ref[1]+(-0.60251179847)*x_ref[2]**o+(0.378303877746)*x_ref[2]
        ref[0,2,2,1]=(0.261384106387)*x_ref[0]**o+(0.0326416199811)*x_ref[0]+(-0.426740102269)*x_ref[1]**o+(-0.445401626921)*x_ref[1]+(-0.388668254526)*x_ref[2]**o+(-0.977942913284)*x_ref[2]
        ref[0,3,0,0]=(-0.0195053813483)*x_ref[0]**o+(-0.294061998664)*x_ref[0]+(0.790461098426)*x_ref[1]**o+(-0.506334268838)*x_ref[1]+(0.394015541605)*x_ref[2]**o+(-0.562248105432)*x_ref[2]
        ref[0,3,0,1]=(0.390726516735)*x_ref[0]**o+(-0.757597437859)*x_ref[0]+(0.861882149861)*x_ref[1]**o+(0.621870451314)*x_ref[1]+(0.900777666162)*x_ref[2]**o+(0.271507131938)*x_ref[2]
        ref[0,3,1,0]=(-0.725635966475)*x_ref[0]**o+(-0.695837548737)*x_ref[0]+(-0.163078176328)*x_ref[1]**o+(-0.0435808418729)*x_ref[1]+(0.0890756544611)*x_ref[2]**o+(-0.270422627949)*x_ref[2]
        ref[0,3,1,1]=(-0.175498720647)*x_ref[0]**o+(-0.213605441632)*x_ref[0]+(-0.358632620362)*x_ref[1]**o+(-0.193273004152)*x_ref[1]+(0.937704823805)*x_ref[2]**o+(0.861707832291)*x_ref[2]
        ref[0,3,2,0]=(0.0976010265611)*x_ref[0]**o+(0.995135925341)*x_ref[0]+(0.755339287387)*x_ref[1]**o+(-0.924722068193)*x_ref[1]+(-0.480212181904)*x_ref[2]**o+(-0.40237760439)*x_ref[2]
        ref[0,3,2,1]=(-0.679215392952)*x_ref[0]**o+(-0.00796096253297)*x_ref[0]+(-0.964972599421)*x_ref[1]**o+(-0.410415435656)*x_ref[1]+(-0.292638067785)*x_ref[2]**o+(-0.950383976561)*x_ref[2]
        ref[0,4,0,0]=(0.245586594928)*x_ref[0]**o+(-0.593420401896)*x_ref[0]+(0.553450486534)*x_ref[1]**o+(0.173814965689)*x_ref[1]+(0.0719392610563)*x_ref[2]**o+(0.676231437486)*x_ref[2]
        ref[0,4,0,1]=(0.154382756425)*x_ref[0]**o+(-0.0655740293432)*x_ref[0]+(0.924689745142)*x_ref[1]**o+(-0.246876531175)*x_ref[1]+(-0.241460713543)*x_ref[2]**o+(-0.489840108568)*x_ref[2]
        ref[0,4,1,0]=(-0.482705840359)*x_ref[0]**o+(-0.392960380114)*x_ref[0]+(-0.503463823972)*x_ref[1]**o+(0.317163204738)*x_ref[1]+(0.775048877356)*x_ref[2]**o+(0.599889732326)*x_ref[2]
        ref[0,4,1,1]=(-0.122555861716)*x_ref[0]**o+(0.930175770449)*x_ref[0]+(0.483503764941)*x_ref[1]**o+(0.340001033217)*x_ref[1]+(0.672924280979)*x_ref[2]**o+(0.746489359683)*x_ref[2]
        ref[0,4,2,0]=(-0.339421141033)*x_ref[0]**o+(-0.604966575169)*x_ref[0]+(0.862413017648)*x_ref[1]**o+(0.936752078454)*x_ref[1]+(-0.0549628353285)*x_ref[2]**o+(-0.0858885835314)*x_ref[2]
        ref[0,4,2,1]=(-0.581767357421)*x_ref[0]**o+(-0.644286027548)*x_ref[0]+(0.76948396865)*x_ref[1]**o+(0.692265904744)*x_ref[1]+(0.955871091531)*x_ref[2]**o+(-0.378888503457)*x_ref[2]
        ref[1,0,0,0]=(-0.0407846595046)*x_ref[0]**o+(-0.293709943251)*x_ref[0]+(-0.310552931582)*x_ref[1]**o+(-0.696244178466)*x_ref[1]+(0.962985284252)*x_ref[2]**o+(-0.0266457621404)*x_ref[2]
        ref[1,0,0,1]=(-0.480942811584)*x_ref[0]**o+(-0.400158759679)*x_ref[0]+(0.311171869807)*x_ref[1]**o+(-0.79518809427)*x_ref[1]+(-0.497004450478)*x_ref[2]**o+(-0.639822005752)*x_ref[2]
        ref[1,0,1,0]=(0.158738648483)*x_ref[0]**o+(0.274859816468)*x_ref[0]+(-0.208677612654)*x_ref[1]**o+(0.911107598366)*x_ref[1]+(-0.400163007778)*x_ref[2]**o+(0.276296890267)*x_ref[2]
        ref[1,0,1,1]=(0.613312276864)*x_ref[0]**o+(-0.644237218776)*x_ref[0]+(0.81927272895)*x_ref[1]**o+(-0.12971860556)*x_ref[1]+(0.214379406799)*x_ref[2]**o+(0.856335675043)*x_ref[2]
        ref[1,0,2,0]=(-0.699925173766)*x_ref[0]**o+(-0.560613547952)*x_ref[0]+(-0.355325107637)*x_ref[1]**o+(-0.598747822041)*x_ref[1]+(0.243087037553)*x_ref[2]**o+(-0.484176993015)*x_ref[2]
        ref[1,0,2,1]=(-0.218912300848)*x_ref[0]**o+(-0.259483312969)*x_ref[0]+(-0.863788186295)*x_ref[1]**o+(0.924423774757)*x_ref[1]+(-0.0823752192204)*x_ref[2]**o+(-0.928377900602)*x_ref[2]
        ref[1,1,0,0]=(-0.368901144425)*x_ref[0]**o+(0.544104655592)*x_ref[0]+(0.321295151176)*x_ref[1]**o+(0.395692754134)*x_ref[1]+(-0.721319111754)*x_ref[2]**o+(-0.698518442137)*x_ref[2]
        ref[1,1,0,1]=(-0.592867336047)*x_ref[0]**o+(0.145120231038)*x_ref[0]+(0.110390771366)*x_ref[1]**o+(0.168333368086)*x_ref[1]+(-0.577866419921)*x_ref[2]**o+(0.705027708469)*x_ref[2]
        ref[1,1,1,0]=(-0.210603428235)*x_ref[0]**o+(-0.250156447116)*x_ref[0]+(-0.397803248976)*x_ref[1]**o+(0.332163380867)*x_ref[1]+(-0.250648216541)*x_ref[2]**o+(-0.0693729055697)*x_ref[2]
        ref[1,1,1,1]=(0.324927558096)*x_ref[0]**o+(-0.262726116223)*x_ref[0]+(-0.555439758488)*x_ref[1]**o+(-0.865019248609)*x_ref[1]+(-0.0971907425857)*x_ref[2]**o+(-0.742359236711)*x_ref[2]
        ref[1,1,2,0]=(0.161058869414)*x_ref[0]**o+(0.349191500791)*x_ref[0]+(0.468336473315)*x_ref[1]**o+(-0.856364775995)*x_ref[1]+(0.491736696689)*x_ref[2]**o+(0.352984104173)*x_ref[2]
        ref[1,1,2,1]=(-0.243275976508)*x_ref[0]**o+(-0.785605388878)*x_ref[0]+(-0.282478919296)*x_ref[1]**o+(-0.728851522341)*x_ref[1]+(0.375134848237)*x_ref[2]**o+(-0.486200561256)*x_ref[2]
        ref[1,2,0,0]=(-0.172241451405)*x_ref[0]**o+(0.422361498196)*x_ref[0]+(0.464862755164)*x_ref[1]**o+(0.466661717105)*x_ref[1]+(-0.886209677512)*x_ref[2]**o+(-0.43529076789)*x_ref[2]
        ref[1,2,0,1]=(0.946582222405)*x_ref[0]**o+(0.513640973894)*x_ref[0]+(-0.923788000633)*x_ref[1]**o+(-0.358163567195)*x_ref[1]+(-0.709999132129)*x_ref[2]**o+(-0.761163299694)*x_ref[2]
        ref[1,2,1,0]=(0.236339981889)*x_ref[0]**o+(0.259874866134)*x_ref[0]+(-0.32460762691)*x_ref[1]**o+(-0.629786950313)*x_ref[1]+(-0.679261685402)*x_ref[2]**o+(0.593069192918)*x_ref[2]
        ref[1,2,1,1]=(0.11006800009)*x_ref[0]**o+(0.970532991787)*x_ref[0]+(-0.467608188361)*x_ref[1]**o+(-0.959281984552)*x_ref[1]+(0.610396622519)*x_ref[2]**o+(0.282468600058)*x_ref[2]
        ref[1,2,2,0]=(-0.656419157742)*x_ref[0]**o+(0.234027308171)*x_ref[0]+(-0.67027206978)*x_ref[1]**o+(-0.746034436817)*x_ref[1]+(-0.771894073652)*x_ref[2]**o+(0.949545830287)*x_ref[2]
        ref[1,2,2,1]=(0.615884236811)*x_ref[0]**o+(-0.670156612555)*x_ref[0]+(0.543165493345)*x_ref[1]**o+(-0.618294277594)*x_ref[1]+(-0.974114255061)*x_ref[2]**o+(-0.762320542298)*x_ref[2]
        ref[1,3,0,0]=(0.096201957648)*x_ref[0]**o+(0.440870816208)*x_ref[0]+(0.124448859607)*x_ref[1]**o+(0.507919407561)*x_ref[1]+(-0.401666263687)*x_ref[2]**o+(-0.437653713723)*x_ref[2]
        ref[1,3,0,1]=(0.165129229294)*x_ref[0]**o+(-0.353217428955)*x_ref[0]+(0.473666065504)*x_ref[1]**o+(0.907667166988)*x_ref[1]+(-0.864298533288)*x_ref[2]**o+(0.641505117281)*x_ref[2]
        ref[1,3,1,0]=(0.381900624425)*x_ref[0]**o+(0.427520178597)*x_ref[0]+(-0.560279540293)*x_ref[1]**o+(0.190814532271)*x_ref[1]+(-0.187947688629)*x_ref[2]**o+(-0.127694148593)*x_ref[2]
        ref[1,3,1,1]=(-0.903921363895)*x_ref[0]**o+(-0.659942040828)*x_ref[0]+(-0.549460213492)*x_ref[1]**o+(0.869909651895)*x_ref[1]+(-0.962273439484)*x_ref[2]**o+(-0.046365857751)*x_ref[2]
        ref[1,3,2,0]=(-0.021688482187)*x_ref[0]**o+(-0.209531000717)*x_ref[0]+(0.668564395133)*x_ref[1]**o+(-0.0347978700975)*x_ref[1]+(0.947677147488)*x_ref[2]**o+(-0.0152385052475)*x_ref[2]
        ref[1,3,2,1]=(0.880839485817)*x_ref[0]**o+(-0.319275929312)*x_ref[0]+(-0.940056657485)*x_ref[1]**o+(0.570855397726)*x_ref[1]+(0.279710812074)*x_ref[2]**o+(0.496622215051)*x_ref[2]
        ref[1,4,0,0]=(-0.139033645615)*x_ref[0]**o+(-0.271453722366)*x_ref[0]+(-0.268516720244)*x_ref[1]**o+(-0.0467555017438)*x_ref[1]+(-0.41618262265)*x_ref[2]**o+(0.315320363813)*x_ref[2]
        ref[1,4,0,1]=(0.296108607204)*x_ref[0]**o+(-0.184582165661)*x_ref[0]+(0.121416543744)*x_ref[1]**o+(-0.922284926751)*x_ref[1]+(-0.437600089668)*x_ref[2]**o+(-0.608800626513)*x_ref[2]
        ref[1,4,1,0]=(0.8942028423)*x_ref[0]**o+(0.673402831188)*x_ref[0]+(-0.622529195562)*x_ref[1]**o+(-0.266052338798)*x_ref[1]+(-0.273167851702)*x_ref[2]**o+(-0.312211982747)*x_ref[2]
        ref[1,4,1,1]=(-0.102315711178)*x_ref[0]**o+(-0.885515719371)*x_ref[0]+(-0.532910101204)*x_ref[1]**o+(0.798525944348)*x_ref[1]+(0.640586802303)*x_ref[2]**o+(0.36308806536)*x_ref[2]
        ref[1,4,2,0]=(0.376793506541)*x_ref[0]**o+(-0.667063564875)*x_ref[0]+(0.609591985816)*x_ref[1]**o+(0.28347745142)*x_ref[1]+(-0.751832936302)*x_ref[2]**o+(-0.683738856965)*x_ref[2]
        ref[1,4,2,1]=(-0.51412252819)*x_ref[0]**o+(0.559560297206)*x_ref[0]+(0.751134603429)*x_ref[1]**o+(-0.0031812310797)*x_ref[1]+(-0.525776122178)*x_ref[2]**o+(0.338471942848)*x_ref[2]
        ref[2,0,0,0]=(0.937572244905)*x_ref[0]**o+(-0.293020870379)*x_ref[0]+(-0.918591520817)*x_ref[1]**o+(0.622194477261)*x_ref[1]+(0.31641482851)*x_ref[2]**o+(0.276912266864)*x_ref[2]
        ref[2,0,0,1]=(0.853041599998)*x_ref[0]**o+(-0.0220623724171)*x_ref[0]+(0.89909044296)*x_ref[1]**o+(-0.838782642732)*x_ref[1]+(0.0133197592042)*x_ref[2]**o+(-0.868977618993)*x_ref[2]
        ref[2,0,1,0]=(-0.325799042976)*x_ref[0]**o+(0.746269971994)*x_ref[0]+(0.882671061757)*x_ref[1]**o+(-0.284133373252)*x_ref[1]+(0.120086549521)*x_ref[2]**o+(-0.397352610969)*x_ref[2]
        ref[2,0,1,1]=(0.282336934951)*x_ref[0]**o+(-0.339893978547)*x_ref[0]+(-0.0861660125386)*x_ref[1]**o+(0.283063579798)*x_ref[1]+(0.714762478569)*x_ref[2]**o+(-0.497021718549)*x_ref[2]
        ref[2,0,2,0]=(0.891782779491)*x_ref[0]**o+(-0.817647577644)*x_ref[0]+(0.364088142085)*x_ref[1]**o+(-0.950577637048)*x_ref[1]+(0.4070663557)*x_ref[2]**o+(0.270753179246)*x_ref[2]
        ref[2,0,2,1]=(0.316594913808)*x_ref[0]**o+(0.310731486851)*x_ref[0]+(0.250421828765)*x_ref[1]**o+(0.772786141458)*x_ref[1]+(0.64843040679)*x_ref[2]**o+(-0.691903332511)*x_ref[2]
        ref[2,1,0,0]=(0.87180715028)*x_ref[0]**o+(-0.953173230375)*x_ref[0]+(-0.823516344548)*x_ref[1]**o+(0.934653756548)*x_ref[1]+(-0.537124962489)*x_ref[2]**o+(-0.233365939659)*x_ref[2]
        ref[2,1,0,1]=(-0.407189744631)*x_ref[0]**o+(-0.754425720288)*x_ref[0]+(0.737526767897)*x_ref[1]**o+(0.745393943502)*x_ref[1]+(0.811744293685)*x_ref[2]**o+(-0.459556659177)*x_ref[2]
        ref[2,1,1,0]=(-0.693529259877)*x_ref[0]**o+(0.580197938048)*x_ref[0]+(0.506055440405)*x_ref[1]**o+(0.122531014759)*x_ref[1]+(0.427736137775)*x_ref[2]**o+(-0.83260140401)*x_ref[2]
        ref[2,1,1,1]=(0.542621975026)*x_ref[0]**o+(-0.568425713106)*x_ref[0]+(0.303002043446)*x_ref[1]**o+(0.363286261651)*x_ref[1]+(0.2729476289)*x_ref[2]**o+(-0.0113629744659)*x_ref[2]
        ref[2,1,2,0]=(-0.266750235694)*x_ref[0]**o+(0.10116400306)*x_ref[0]+(0.315977962897)*x_ref[1]**o+(-0.406236201454)*x_ref[1]+(-0.586845072374)*x_ref[2]**o+(0.513937047282)*x_ref[2]
        ref[2,1,2,1]=(-0.518780811975)*x_ref[0]**o+(0.275053640829)*x_ref[0]+(0.4234970646)*x_ref[1]**o+(0.389585163376)*x_ref[1]+(-0.609771068778)*x_ref[2]**o+(-0.121467061755)*x_ref[2]
        ref[2,2,0,0]=(0.138823207466)*x_ref[0]**o+(-0.779837455014)*x_ref[0]+(0.70167049555)*x_ref[1]**o+(0.766428813369)*x_ref[1]+(0.114023530366)*x_ref[2]**o+(0.650098536187)*x_ref[2]
        ref[2,2,0,1]=(0.974998667633)*x_ref[0]**o+(0.958471050654)*x_ref[0]+(0.193515162503)*x_ref[1]**o+(-0.252148167253)*x_ref[1]+(0.495475460125)*x_ref[2]**o+(0.562693347983)*x_ref[2]
        ref[2,2,1,0]=(-0.552379548988)*x_ref[0]**o+(-0.831399971618)*x_ref[0]+(0.437581216647)*x_ref[1]**o+(-0.775331267696)*x_ref[1]+(0.729041628844)*x_ref[2]**o+(0.633113359893)*x_ref[2]
        ref[2,2,1,1]=(0.379858437049)*x_ref[0]**o+(0.923586504203)*x_ref[0]+(0.707244783969)*x_ref[1]**o+(-0.293157375059)*x_ref[1]+(-0.368717941876)*x_ref[2]**o+(-0.202610288616)*x_ref[2]
        ref[2,2,2,0]=(-0.804826108231)*x_ref[0]**o+(-0.0208712096312)*x_ref[0]+(0.0482547072498)*x_ref[1]**o+(0.852366543609)*x_ref[1]+(0.328962500136)*x_ref[2]**o+(-0.132777835512)*x_ref[2]
        ref[2,2,2,1]=(0.91373752217)*x_ref[0]**o+(-0.792112120729)*x_ref[0]+(-0.0457564087072)*x_ref[1]**o+(0.309694852949)*x_ref[1]+(0.089219285337)*x_ref[2]**o+(-0.516000514614)*x_ref[2]
        ref[2,3,0,0]=(0.343215129115)*x_ref[0]**o+(0.40804099837)*x_ref[0]+(-0.865942442914)*x_ref[1]**o+(0.842241683043)*x_ref[1]+(0.0727803297601)*x_ref[2]**o+(-0.395225682949)*x_ref[2]
        ref[2,3,0,1]=(-0.965340522771)*x_ref[0]**o+(-0.897056478677)*x_ref[0]+(-0.305205944062)*x_ref[1]**o+(0.393941789685)*x_ref[1]+(-0.253328379131)*x_ref[2]**o+(0.0951510469271)*x_ref[2]
        ref[2,3,1,0]=(0.536251514752)*x_ref[0]**o+(0.0629776386779)*x_ref[0]+(-0.706311222379)*x_ref[1]**o+(-0.84362242029)*x_ref[1]+(0.00110678760539)*x_ref[2]**o+(-0.30286059941)*x_ref[2]
        ref[2,3,1,1]=(0.114584737295)*x_ref[0]**o+(0.0708763738067)*x_ref[0]+(-0.454760062397)*x_ref[1]**o+(0.228742539922)*x_ref[1]+(0.514960810352)*x_ref[2]**o+(-0.815749211085)*x_ref[2]
        ref[2,3,2,0]=(0.246335688008)*x_ref[0]**o+(-0.239199299885)*x_ref[0]+(-0.447390151659)*x_ref[1]**o+(0.918949356391)*x_ref[1]+(-0.307864840175)*x_ref[2]**o+(0.560139080481)*x_ref[2]
        ref[2,3,2,1]=(0.118766035723)*x_ref[0]**o+(0.610711164899)*x_ref[0]+(0.336628771776)*x_ref[1]**o+(0.833699268751)*x_ref[1]+(-0.424694647429)*x_ref[2]**o+(-0.639929688104)*x_ref[2]
        ref[2,4,0,0]=(0.814543506987)*x_ref[0]**o+(-0.797684321368)*x_ref[0]+(0.337242635783)*x_ref[1]**o+(0.278139837072)*x_ref[1]+(0.633511526441)*x_ref[2]**o+(0.372175615028)*x_ref[2]
        ref[2,4,0,1]=(-0.0590668694094)*x_ref[0]**o+(0.256732067267)*x_ref[0]+(0.519737187577)*x_ref[1]**o+(-0.376391855948)*x_ref[1]+(-0.810934658651)*x_ref[2]**o+(0.826899476928)*x_ref[2]
        ref[2,4,1,0]=(-0.428976843661)*x_ref[0]**o+(0.672667833313)*x_ref[0]+(0.263194890946)*x_ref[1]**o+(-0.894790773661)*x_ref[1]+(-0.550298557295)*x_ref[2]**o+(-0.00560384035502)*x_ref[2]
        ref[2,4,1,1]=(-0.818372430144)*x_ref[0]**o+(-0.829920833073)*x_ref[0]+(-0.359560636119)*x_ref[1]**o+(-0.113484563205)*x_ref[1]+(0.586796110365)*x_ref[2]**o+(0.296820898426)*x_ref[2]
        ref[2,4,2,0]=(-0.691597969566)*x_ref[0]**o+(0.715813153538)*x_ref[0]+(0.954093612503)*x_ref[1]**o+(0.88803420754)*x_ref[1]+(0.697447667765)*x_ref[2]**o+(0.793730496625)*x_ref[2]
        ref[2,4,2,1]=(0.351099833336)*x_ref[0]**o+(-0.716419272807)*x_ref[0]+(0.318726674998)*x_ref[1]**o+(0.246151914414)*x_ref[1]+(-0.945520673884)*x_ref[2]**o+(-0.489026233954)*x_ref[2]
        ref[3,0,0,0]=(0.511598022804)*x_ref[0]**o+(0.567617269582)*x_ref[0]+(-0.526293319137)*x_ref[1]**o+(0.333750098146)*x_ref[1]+(-0.0869228849144)*x_ref[2]**o+(-0.204072834867)*x_ref[2]
        ref[3,0,0,1]=(0.905384007172)*x_ref[0]**o+(-0.661067390314)*x_ref[0]+(-0.470790632295)*x_ref[1]**o+(-0.149608017616)*x_ref[1]+(0.427879278267)*x_ref[2]**o+(0.987040179184)*x_ref[2]
        ref[3,0,1,0]=(-0.946562006809)*x_ref[0]**o+(0.64141909633)*x_ref[0]+(-0.762076826276)*x_ref[1]**o+(-0.642912850794)*x_ref[1]+(0.712567898559)*x_ref[2]**o+(-0.292128944205)*x_ref[2]
        ref[3,0,1,1]=(-0.955087949361)*x_ref[0]**o+(0.426588041133)*x_ref[0]+(0.339313104127)*x_ref[1]**o+(-0.375147344045)*x_ref[1]+(0.967259014679)*x_ref[2]**o+(0.504521081858)*x_ref[2]
        ref[3,0,2,0]=(0.141876598344)*x_ref[0]**o+(-0.335579252825)*x_ref[0]+(0.514552993602)*x_ref[1]**o+(-0.531116948499)*x_ref[1]+(-0.369319413141)*x_ref[2]**o+(0.0374413262362)*x_ref[2]
        ref[3,0,2,1]=(0.678360463658)*x_ref[0]**o+(-0.357900119646)*x_ref[0]+(0.300750717059)*x_ref[1]**o+(0.516196189448)*x_ref[1]+(0.0535466018203)*x_ref[2]**o+(-0.149961940786)*x_ref[2]
        ref[3,1,0,0]=(0.0975994351079)*x_ref[0]**o+(0.634544068499)*x_ref[0]+(-0.963217046352)*x_ref[1]**o+(-0.804658091456)*x_ref[1]+(-0.620132720823)*x_ref[2]**o+(0.862250619369)*x_ref[2]
        ref[3,1,0,1]=(0.210306328879)*x_ref[0]**o+(-0.457341187966)*x_ref[0]+(0.957556084801)*x_ref[1]**o+(0.704073170928)*x_ref[1]+(0.423134492133)*x_ref[2]**o+(-0.167268375412)*x_ref[2]
        ref[3,1,1,0]=(-0.909228817744)*x_ref[0]**o+(0.110665018921)*x_ref[0]+(0.36275630147)*x_ref[1]**o+(0.346866824763)*x_ref[1]+(-0.535600547489)*x_ref[2]**o+(0.627638814375)*x_ref[2]
        ref[3,1,1,1]=(0.328780933823)*x_ref[0]**o+(-0.249563994161)*x_ref[0]+(-0.555789062892)*x_ref[1]**o+(-0.917287525036)*x_ref[1]+(0.386191175017)*x_ref[2]**o+(0.123904319061)*x_ref[2]
        ref[3,1,2,0]=(-0.838409901623)*x_ref[0]**o+(0.673479705393)*x_ref[0]+(0.482921104917)*x_ref[1]**o+(0.136411388588)*x_ref[1]+(0.583674116225)*x_ref[2]**o+(0.305100836877)*x_ref[2]
        ref[3,1,2,1]=(0.952961008312)*x_ref[0]**o+(-0.0501981079532)*x_ref[0]+(0.661434464642)*x_ref[1]**o+(0.65030469399)*x_ref[1]+(-0.157562001295)*x_ref[2]**o+(-0.249551892917)*x_ref[2]
        ref[3,2,0,0]=(-0.0232804500286)*x_ref[0]**o+(-0.559229392203)*x_ref[0]+(0.127967930273)*x_ref[1]**o+(-0.635827181645)*x_ref[1]+(-0.0426703224383)*x_ref[2]**o+(0.47720230258)*x_ref[2]
        ref[3,2,0,1]=(-0.421658996106)*x_ref[0]**o+(-0.468260038077)*x_ref[0]+(0.479083572554)*x_ref[1]**o+(-0.214417881841)*x_ref[1]+(0.372430534261)*x_ref[2]**o+(0.133063789546)*x_ref[2]
        ref[3,2,1,0]=(0.905811586567)*x_ref[0]**o+(0.245447340384)*x_ref[0]+(-0.337650999146)*x_ref[1]**o+(0.908188235976)*x_ref[1]+(-0.804029751277)*x_ref[2]**o+(0.0512676456643)*x_ref[2]
        ref[3,2,1,1]=(-0.891486839409)*x_ref[0]**o+(0.20753794329)*x_ref[0]+(0.879183464252)*x_ref[1]**o+(0.902028906992)*x_ref[1]+(0.416776813598)*x_ref[2]**o+(0.491964374373)*x_ref[2]
        ref[3,2,2,0]=(0.653067027119)*x_ref[0]**o+(0.765293890745)*x_ref[0]+(-0.348902169508)*x_ref[1]**o+(0.324429035849)*x_ref[1]+(-0.363026365065)*x_ref[2]**o+(0.0816186405511)*x_ref[2]
        ref[3,2,2,1]=(-0.0415375052325)*x_ref[0]**o+(-0.480779075784)*x_ref[0]+(-0.137223552938)*x_ref[1]**o+(-0.915733625512)*x_ref[1]+(0.750199445159)*x_ref[2]**o+(-0.789818945171)*x_ref[2]
        ref[3,3,0,0]=(0.0744921239083)*x_ref[0]**o+(0.0700589144159)*x_ref[0]+(-0.327105425297)*x_ref[1]**o+(0.469843257765)*x_ref[1]+(-0.170191600844)*x_ref[2]**o+(0.732071534155)*x_ref[2]
        ref[3,3,0,1]=(0.172643927441)*x_ref[0]**o+(0.848924114383)*x_ref[0]+(-0.79125762228)*x_ref[1]**o+(-0.694725775976)*x_ref[1]+(-0.309710503985)*x_ref[2]**o+(-0.405310234927)*x_ref[2]
        ref[3,3,1,0]=(-0.465911787595)*x_ref[0]**o+(-0.71080474709)*x_ref[0]+(0.959925934203)*x_ref[1]**o+(-0.111110644825)*x_ref[1]+(-0.145349925218)*x_ref[2]**o+(0.122955165811)*x_ref[2]
        ref[3,3,1,1]=(-0.769111863754)*x_ref[0]**o+(0.348036938532)*x_ref[0]+(0.485890856474)*x_ref[1]**o+(0.382595218985)*x_ref[1]+(-0.897488692981)*x_ref[2]**o+(0.830822333819)*x_ref[2]
        ref[3,3,2,0]=(0.341890793681)*x_ref[0]**o+(-0.667145936571)*x_ref[0]+(-0.145094108268)*x_ref[1]**o+(0.771433911212)*x_ref[1]+(-0.299752164325)*x_ref[2]**o+(-0.357856549677)*x_ref[2]
        ref[3,3,2,1]=(0.26381838796)*x_ref[0]**o+(-0.44474724998)*x_ref[0]+(0.310283988195)*x_ref[1]**o+(0.698675181314)*x_ref[1]+(0.437624133238)*x_ref[2]**o+(0.394916054445)*x_ref[2]
        ref[3,4,0,0]=(-0.576499406674)*x_ref[0]**o+(0.770936634959)*x_ref[0]+(0.707441735783)*x_ref[1]**o+(-0.794953257043)*x_ref[1]+(-0.535342943209)*x_ref[2]**o+(-0.509446811787)*x_ref[2]
        ref[3,4,0,1]=(0.161115012113)*x_ref[0]**o+(0.11079092252)*x_ref[0]+(0.0633804181164)*x_ref[1]**o+(-0.406049293019)*x_ref[1]+(-0.824283994336)*x_ref[2]**o+(-0.481107560499)*x_ref[2]
        ref[3,4,1,0]=(-0.411133810295)*x_ref[0]**o+(0.988486625099)*x_ref[0]+(-0.36050191367)*x_ref[1]**o+(0.469479821965)*x_ref[1]+(-0.431175357325)*x_ref[2]**o+(-0.966295015263)*x_ref[2]
        ref[3,4,1,1]=(0.233731410367)*x_ref[0]**o+(-0.821900740766)*x_ref[0]+(-0.823123461385)*x_ref[1]**o+(-0.380537914421)*x_ref[1]+(-0.896146300355)*x_ref[2]**o+(-0.775172990257)*x_ref[2]
        ref[3,4,2,0]=(-0.96323363586)*x_ref[0]**o+(0.0935412405245)*x_ref[0]+(0.929333026259)*x_ref[1]**o+(-0.322236627348)*x_ref[1]+(-0.390898135964)*x_ref[2]**o+(-0.766178290162)*x_ref[2]
        ref[3,4,2,1]=(-0.773670533268)*x_ref[0]**o+(-0.156975157098)*x_ref[0]+(-0.91512996795)*x_ref[1]**o+(0.0247803682126)*x_ref[1]+(0.995159733782)*x_ref[2]**o+(-0.669224144225)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_ContinuousFunction_rank0(self):
      """
      tests interpolate for rank 0 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ContinuousFunction onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(),dim=dim)
      s=Data(0,(),w)
      ref=Data(0,(),w_ref)
      if dim==2:
        s=(-0.0411847356752)*x[0]**o+(-0.978252807465)*x[0]+(-0.765516202331)*x[1]**o+(-0.60628403943)*x[1]
        ref=(-0.0411847356752)*x_ref[0]**o+(-0.978252807465)*x_ref[0]+(-0.765516202331)*x_ref[1]**o+(-0.60628403943)*x_ref[1]
      else:
        s=(-0.520571545539)*x[0]**o+(0.516384538651)*x[0]+(-0.865529136972)*x[1]**o+(0.80164386579)*x[1]+(-0.330695181072)*x[2]**o+(-0.758708438741)*x[2]
        ref=(-0.520571545539)*x_ref[0]**o+(0.516384538651)*x_ref[0]+(-0.865529136972)*x_ref[1]**o+(0.80164386579)*x_ref[1]+(-0.330695181072)*x_ref[2]**o+(-0.758708438741)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_ContinuousFunction_rank1(self):
      """
      tests interpolate for rank 1 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ContinuousFunction onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(2,),dim=dim)
      s=Data(0,(2,),w)
      ref=Data(0,(2,),w_ref)
      if dim==2:
        s[0]=(0.505196018365)*x[0]**o+(0.731510032507)*x[0]+(-0.0776298756069)*x[1]**o+(0.223327557462)*x[1]
        s[1]=(-0.239558790149)*x[0]**o+(0.608302150075)*x[0]+(-0.840461486286)*x[1]**o+(-0.0797379857546)*x[1]
        ref[0]=(0.505196018365)*x_ref[0]**o+(0.731510032507)*x_ref[0]+(-0.0776298756069)*x_ref[1]**o+(0.223327557462)*x_ref[1]
        ref[1]=(-0.239558790149)*x_ref[0]**o+(0.608302150075)*x_ref[0]+(-0.840461486286)*x_ref[1]**o+(-0.0797379857546)*x_ref[1]
      else:
        s[0]=(-0.853964974211)*x[0]**o+(-0.436921639846)*x[0]+(-0.356712309058)*x[1]**o+(0.324151975277)*x[1]+(0.338538264809)*x[2]**o+(0.254654297562)*x[2]
        s[1]=(0.46616823299)*x[0]**o+(0.64792464198)*x[0]+(-0.655892247116)*x[1]**o+(-0.9837201382)*x[1]+(0.36604513951)*x[2]**o+(0.343454378544)*x[2]
        ref[0]=(-0.853964974211)*x_ref[0]**o+(-0.436921639846)*x_ref[0]+(-0.356712309058)*x_ref[1]**o+(0.324151975277)*x_ref[1]+(0.338538264809)*x_ref[2]**o+(0.254654297562)*x_ref[2]
        ref[1]=(0.46616823299)*x_ref[0]**o+(0.64792464198)*x_ref[0]+(-0.655892247116)*x_ref[1]**o+(-0.9837201382)*x_ref[1]+(0.36604513951)*x_ref[2]**o+(0.343454378544)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_ContinuousFunction_rank2(self):
      """
      tests interpolate for rank 2 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ContinuousFunction onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5),dim=dim)
      s=Data(0,(4, 5),w)
      ref=Data(0,(4, 5),w_ref)
      if dim==2:
        s[0,0]=(0.403739435229)*x[0]**o+(-0.976543623588)*x[0]+(0.408638664295)*x[1]**o+(0.290441090188)*x[1]
        s[0,1]=(-0.186112372212)*x[0]**o+(-0.885114653425)*x[0]+(0.79020447405)*x[1]**o+(-0.988138900337)*x[1]
        s[0,2]=(-0.749718519557)*x[0]**o+(-0.556662006553)*x[0]+(-0.388628750048)*x[1]**o+(0.488811308506)*x[1]
        s[0,3]=(0.55958060511)*x[0]**o+(-0.692678975478)*x[0]+(-0.507024092372)*x[1]**o+(0.547618022755)*x[1]
        s[0,4]=(0.393523537188)*x[0]**o+(-0.278215081058)*x[0]+(0.524241376975)*x[1]**o+(-0.18209236848)*x[1]
        s[1,0]=(0.277364605261)*x[0]**o+(0.867787382941)*x[0]+(-0.468812178843)*x[1]**o+(-0.915273555727)*x[1]
        s[1,1]=(-0.883469445503)*x[0]**o+(0.949713147084)*x[0]+(-0.195972546782)*x[1]**o+(-0.550840574214)*x[1]
        s[1,2]=(-0.0612993770053)*x[0]**o+(0.349066961111)*x[0]+(0.117833853269)*x[1]**o+(0.915145781122)*x[1]
        s[1,3]=(-0.221469359751)*x[0]**o+(0.860157157768)*x[0]+(0.334660915861)*x[1]**o+(0.0202241427453)*x[1]
        s[1,4]=(-0.859671577801)*x[0]**o+(-0.751878816346)*x[0]+(0.970178380651)*x[1]**o+(-0.852998297383)*x[1]
        s[2,0]=(-0.273019645199)*x[0]**o+(0.77546843291)*x[0]+(0.40198451057)*x[1]**o+(0.719427949335)*x[1]
        s[2,1]=(-0.615814919857)*x[0]**o+(-0.804830902046)*x[0]+(-0.55666668286)*x[1]**o+(-0.436274362037)*x[1]
        s[2,2]=(-0.257042626769)*x[0]**o+(-0.0674177291518)*x[0]+(-0.487890885639)*x[1]**o+(-0.276346354771)*x[1]
        s[2,3]=(-0.0863705553448)*x[0]**o+(-0.456256145846)*x[0]+(0.279365337349)*x[1]**o+(-0.788366887305)*x[1]
        s[2,4]=(0.00696265740333)*x[0]**o+(-0.903180556586)*x[0]+(0.299924222843)*x[1]**o+(0.432554144375)*x[1]
        s[3,0]=(0.928471341856)*x[0]**o+(-0.58681191854)*x[0]+(0.592280256168)*x[1]**o+(-0.486182643564)*x[1]
        s[3,1]=(-0.949679045727)*x[0]**o+(0.28219904154)*x[0]+(0.742879063098)*x[1]**o+(-0.0189753136959)*x[1]
        s[3,2]=(-0.386608884833)*x[0]**o+(0.771169732795)*x[0]+(-0.864253528029)*x[1]**o+(-0.0196378783985)*x[1]
        s[3,3]=(0.744817366451)*x[0]**o+(0.973334266028)*x[0]+(0.879085180711)*x[1]**o+(-0.682866003591)*x[1]
        s[3,4]=(-0.453687552098)*x[0]**o+(-0.421237560962)*x[0]+(0.55363269981)*x[1]**o+(-0.446292920116)*x[1]
        ref[0,0]=(0.403739435229)*x_ref[0]**o+(-0.976543623588)*x_ref[0]+(0.408638664295)*x_ref[1]**o+(0.290441090188)*x_ref[1]
        ref[0,1]=(-0.186112372212)*x_ref[0]**o+(-0.885114653425)*x_ref[0]+(0.79020447405)*x_ref[1]**o+(-0.988138900337)*x_ref[1]
        ref[0,2]=(-0.749718519557)*x_ref[0]**o+(-0.556662006553)*x_ref[0]+(-0.388628750048)*x_ref[1]**o+(0.488811308506)*x_ref[1]
        ref[0,3]=(0.55958060511)*x_ref[0]**o+(-0.692678975478)*x_ref[0]+(-0.507024092372)*x_ref[1]**o+(0.547618022755)*x_ref[1]
        ref[0,4]=(0.393523537188)*x_ref[0]**o+(-0.278215081058)*x_ref[0]+(0.524241376975)*x_ref[1]**o+(-0.18209236848)*x_ref[1]
        ref[1,0]=(0.277364605261)*x_ref[0]**o+(0.867787382941)*x_ref[0]+(-0.468812178843)*x_ref[1]**o+(-0.915273555727)*x_ref[1]
        ref[1,1]=(-0.883469445503)*x_ref[0]**o+(0.949713147084)*x_ref[0]+(-0.195972546782)*x_ref[1]**o+(-0.550840574214)*x_ref[1]
        ref[1,2]=(-0.0612993770053)*x_ref[0]**o+(0.349066961111)*x_ref[0]+(0.117833853269)*x_ref[1]**o+(0.915145781122)*x_ref[1]
        ref[1,3]=(-0.221469359751)*x_ref[0]**o+(0.860157157768)*x_ref[0]+(0.334660915861)*x_ref[1]**o+(0.0202241427453)*x_ref[1]
        ref[1,4]=(-0.859671577801)*x_ref[0]**o+(-0.751878816346)*x_ref[0]+(0.970178380651)*x_ref[1]**o+(-0.852998297383)*x_ref[1]
        ref[2,0]=(-0.273019645199)*x_ref[0]**o+(0.77546843291)*x_ref[0]+(0.40198451057)*x_ref[1]**o+(0.719427949335)*x_ref[1]
        ref[2,1]=(-0.615814919857)*x_ref[0]**o+(-0.804830902046)*x_ref[0]+(-0.55666668286)*x_ref[1]**o+(-0.436274362037)*x_ref[1]
        ref[2,2]=(-0.257042626769)*x_ref[0]**o+(-0.0674177291518)*x_ref[0]+(-0.487890885639)*x_ref[1]**o+(-0.276346354771)*x_ref[1]
        ref[2,3]=(-0.0863705553448)*x_ref[0]**o+(-0.456256145846)*x_ref[0]+(0.279365337349)*x_ref[1]**o+(-0.788366887305)*x_ref[1]
        ref[2,4]=(0.00696265740333)*x_ref[0]**o+(-0.903180556586)*x_ref[0]+(0.299924222843)*x_ref[1]**o+(0.432554144375)*x_ref[1]
        ref[3,0]=(0.928471341856)*x_ref[0]**o+(-0.58681191854)*x_ref[0]+(0.592280256168)*x_ref[1]**o+(-0.486182643564)*x_ref[1]
        ref[3,1]=(-0.949679045727)*x_ref[0]**o+(0.28219904154)*x_ref[0]+(0.742879063098)*x_ref[1]**o+(-0.0189753136959)*x_ref[1]
        ref[3,2]=(-0.386608884833)*x_ref[0]**o+(0.771169732795)*x_ref[0]+(-0.864253528029)*x_ref[1]**o+(-0.0196378783985)*x_ref[1]
        ref[3,3]=(0.744817366451)*x_ref[0]**o+(0.973334266028)*x_ref[0]+(0.879085180711)*x_ref[1]**o+(-0.682866003591)*x_ref[1]
        ref[3,4]=(-0.453687552098)*x_ref[0]**o+(-0.421237560962)*x_ref[0]+(0.55363269981)*x_ref[1]**o+(-0.446292920116)*x_ref[1]
      else:
        s[0,0]=(-0.200149935444)*x[0]**o+(0.507524859849)*x[0]+(0.619745083854)*x[1]**o+(0.627105339142)*x[1]+(0.563386979803)*x[2]**o+(-0.961618026889)*x[2]
        s[0,1]=(0.363899424005)*x[0]**o+(0.924425901309)*x[0]+(-0.658883398191)*x[1]**o+(0.481606823837)*x[1]+(-0.159370395541)*x[2]**o+(-0.311388746226)*x[2]
        s[0,2]=(-0.0293410199192)*x[0]**o+(0.256908390845)*x[0]+(-0.909877958762)*x[1]**o+(0.105112556474)*x[1]+(0.252859198357)*x[2]**o+(0.92186026337)*x[2]
        s[0,3]=(-0.00170582078327)*x[0]**o+(0.730720627097)*x[0]+(-0.801575927568)*x[1]**o+(0.455500499782)*x[1]+(0.380759901094)*x[2]**o+(-0.823963841034)*x[2]
        s[0,4]=(0.259808988031)*x[0]**o+(0.863791713598)*x[0]+(0.553188235768)*x[1]**o+(-0.464509312659)*x[1]+(0.42570171)*x[2]**o+(0.192687787615)*x[2]
        s[1,0]=(0.994753432059)*x[0]**o+(-0.68864217861)*x[0]+(-0.0204526591618)*x[1]**o+(-0.306443480795)*x[1]+(0.148406858899)*x[2]**o+(0.854009274563)*x[2]
        s[1,1]=(0.173721730295)*x[0]**o+(-0.987657366279)*x[0]+(0.253868939396)*x[1]**o+(-0.771588800243)*x[1]+(0.53766861285)*x[2]**o+(0.719877605356)*x[2]
        s[1,2]=(0.430924077377)*x[0]**o+(-0.746769688215)*x[0]+(0.828996359633)*x[1]**o+(-0.192590851955)*x[1]+(0.669402098684)*x[2]**o+(-0.962105583491)*x[2]
        s[1,3]=(-0.410727896031)*x[0]**o+(-0.833008011026)*x[0]+(-0.927201509946)*x[1]**o+(-0.876016513532)*x[1]+(0.0914553334551)*x[2]**o+(-0.348128799709)*x[2]
        s[1,4]=(-0.73394473562)*x[0]**o+(0.00706331438319)*x[0]+(-0.205588695357)*x[1]**o+(0.714894523918)*x[1]+(-0.713385984004)*x[2]**o+(-0.385550060437)*x[2]
        s[2,0]=(-0.0853783479307)*x[0]**o+(0.596547791334)*x[0]+(-0.0925317449093)*x[1]**o+(0.751414290883)*x[1]+(0.838736367666)*x[2]**o+(0.735536663934)*x[2]
        s[2,1]=(-0.159026214794)*x[0]**o+(0.949032823761)*x[0]+(-0.422829286588)*x[1]**o+(0.985896164837)*x[1]+(-0.758176201469)*x[2]**o+(0.283868871122)*x[2]
        s[2,2]=(-0.14639325109)*x[0]**o+(0.346468853582)*x[0]+(0.563105800299)*x[1]**o+(-0.531036765601)*x[1]+(-0.0990976207418)*x[2]**o+(0.750636702702)*x[2]
        s[2,3]=(0.849599418085)*x[0]**o+(-0.779136748258)*x[0]+(0.730951130588)*x[1]**o+(0.53628857622)*x[1]+(-0.947230186497)*x[2]**o+(-0.72450102135)*x[2]
        s[2,4]=(-0.700617249374)*x[0]**o+(-0.10080480368)*x[0]+(-0.507852228465)*x[1]**o+(-0.857417192918)*x[1]+(0.640292095556)*x[2]**o+(-0.929256110195)*x[2]
        s[3,0]=(0.653772333481)*x[0]**o+(0.000343679690948)*x[0]+(0.668997254669)*x[1]**o+(0.362901119538)*x[1]+(-0.830987535544)*x[2]**o+(-0.423720904628)*x[2]
        s[3,1]=(-0.0420273808908)*x[0]**o+(0.323011823157)*x[0]+(-0.24882314712)*x[1]**o+(-0.423985714012)*x[1]+(0.752345053106)*x[2]**o+(-0.160413317211)*x[2]
        s[3,2]=(0.890620617817)*x[0]**o+(0.783464544239)*x[0]+(0.790371129121)*x[1]**o+(0.237259212081)*x[1]+(0.610840739105)*x[2]**o+(0.973783326469)*x[2]
        s[3,3]=(-0.379379310577)*x[0]**o+(0.690137408246)*x[0]+(-0.779634423108)*x[1]**o+(-0.974077688357)*x[1]+(0.788216664671)*x[2]**o+(0.101022285352)*x[2]
        s[3,4]=(-0.505116742845)*x[0]**o+(0.923132757962)*x[0]+(-0.0146493711199)*x[1]**o+(0.994421781292)*x[1]+(-0.20080218328)*x[2]**o+(0.555271515554)*x[2]
        ref[0,0]=(-0.200149935444)*x_ref[0]**o+(0.507524859849)*x_ref[0]+(0.619745083854)*x_ref[1]**o+(0.627105339142)*x_ref[1]+(0.563386979803)*x_ref[2]**o+(-0.961618026889)*x_ref[2]
        ref[0,1]=(0.363899424005)*x_ref[0]**o+(0.924425901309)*x_ref[0]+(-0.658883398191)*x_ref[1]**o+(0.481606823837)*x_ref[1]+(-0.159370395541)*x_ref[2]**o+(-0.311388746226)*x_ref[2]
        ref[0,2]=(-0.0293410199192)*x_ref[0]**o+(0.256908390845)*x_ref[0]+(-0.909877958762)*x_ref[1]**o+(0.105112556474)*x_ref[1]+(0.252859198357)*x_ref[2]**o+(0.92186026337)*x_ref[2]
        ref[0,3]=(-0.00170582078327)*x_ref[0]**o+(0.730720627097)*x_ref[0]+(-0.801575927568)*x_ref[1]**o+(0.455500499782)*x_ref[1]+(0.380759901094)*x_ref[2]**o+(-0.823963841034)*x_ref[2]
        ref[0,4]=(0.259808988031)*x_ref[0]**o+(0.863791713598)*x_ref[0]+(0.553188235768)*x_ref[1]**o+(-0.464509312659)*x_ref[1]+(0.42570171)*x_ref[2]**o+(0.192687787615)*x_ref[2]
        ref[1,0]=(0.994753432059)*x_ref[0]**o+(-0.68864217861)*x_ref[0]+(-0.0204526591618)*x_ref[1]**o+(-0.306443480795)*x_ref[1]+(0.148406858899)*x_ref[2]**o+(0.854009274563)*x_ref[2]
        ref[1,1]=(0.173721730295)*x_ref[0]**o+(-0.987657366279)*x_ref[0]+(0.253868939396)*x_ref[1]**o+(-0.771588800243)*x_ref[1]+(0.53766861285)*x_ref[2]**o+(0.719877605356)*x_ref[2]
        ref[1,2]=(0.430924077377)*x_ref[0]**o+(-0.746769688215)*x_ref[0]+(0.828996359633)*x_ref[1]**o+(-0.192590851955)*x_ref[1]+(0.669402098684)*x_ref[2]**o+(-0.962105583491)*x_ref[2]
        ref[1,3]=(-0.410727896031)*x_ref[0]**o+(-0.833008011026)*x_ref[0]+(-0.927201509946)*x_ref[1]**o+(-0.876016513532)*x_ref[1]+(0.0914553334551)*x_ref[2]**o+(-0.348128799709)*x_ref[2]
        ref[1,4]=(-0.73394473562)*x_ref[0]**o+(0.00706331438319)*x_ref[0]+(-0.205588695357)*x_ref[1]**o+(0.714894523918)*x_ref[1]+(-0.713385984004)*x_ref[2]**o+(-0.385550060437)*x_ref[2]
        ref[2,0]=(-0.0853783479307)*x_ref[0]**o+(0.596547791334)*x_ref[0]+(-0.0925317449093)*x_ref[1]**o+(0.751414290883)*x_ref[1]+(0.838736367666)*x_ref[2]**o+(0.735536663934)*x_ref[2]
        ref[2,1]=(-0.159026214794)*x_ref[0]**o+(0.949032823761)*x_ref[0]+(-0.422829286588)*x_ref[1]**o+(0.985896164837)*x_ref[1]+(-0.758176201469)*x_ref[2]**o+(0.283868871122)*x_ref[2]
        ref[2,2]=(-0.14639325109)*x_ref[0]**o+(0.346468853582)*x_ref[0]+(0.563105800299)*x_ref[1]**o+(-0.531036765601)*x_ref[1]+(-0.0990976207418)*x_ref[2]**o+(0.750636702702)*x_ref[2]
        ref[2,3]=(0.849599418085)*x_ref[0]**o+(-0.779136748258)*x_ref[0]+(0.730951130588)*x_ref[1]**o+(0.53628857622)*x_ref[1]+(-0.947230186497)*x_ref[2]**o+(-0.72450102135)*x_ref[2]
        ref[2,4]=(-0.700617249374)*x_ref[0]**o+(-0.10080480368)*x_ref[0]+(-0.507852228465)*x_ref[1]**o+(-0.857417192918)*x_ref[1]+(0.640292095556)*x_ref[2]**o+(-0.929256110195)*x_ref[2]
        ref[3,0]=(0.653772333481)*x_ref[0]**o+(0.000343679690948)*x_ref[0]+(0.668997254669)*x_ref[1]**o+(0.362901119538)*x_ref[1]+(-0.830987535544)*x_ref[2]**o+(-0.423720904628)*x_ref[2]
        ref[3,1]=(-0.0420273808908)*x_ref[0]**o+(0.323011823157)*x_ref[0]+(-0.24882314712)*x_ref[1]**o+(-0.423985714012)*x_ref[1]+(0.752345053106)*x_ref[2]**o+(-0.160413317211)*x_ref[2]
        ref[3,2]=(0.890620617817)*x_ref[0]**o+(0.783464544239)*x_ref[0]+(0.790371129121)*x_ref[1]**o+(0.237259212081)*x_ref[1]+(0.610840739105)*x_ref[2]**o+(0.973783326469)*x_ref[2]
        ref[3,3]=(-0.379379310577)*x_ref[0]**o+(0.690137408246)*x_ref[0]+(-0.779634423108)*x_ref[1]**o+(-0.974077688357)*x_ref[1]+(0.788216664671)*x_ref[2]**o+(0.101022285352)*x_ref[2]
        ref[3,4]=(-0.505116742845)*x_ref[0]**o+(0.923132757962)*x_ref[0]+(-0.0146493711199)*x_ref[1]**o+(0.994421781292)*x_ref[1]+(-0.20080218328)*x_ref[2]**o+(0.555271515554)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_ContinuousFunction_rank3(self):
      """
      tests interpolate for rank 3 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ContinuousFunction onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(6, 2, 2),dim=dim)
      s=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2),w_ref)
      if dim==2:
        s[0,0,0]=(-0.564155350537)*x[0]**o+(-0.800380058082)*x[0]+(0.689149742912)*x[1]**o+(-0.730209879145)*x[1]
        s[0,0,1]=(-0.875752239419)*x[0]**o+(0.620770279709)*x[0]+(-0.405669056556)*x[1]**o+(-0.650461825052)*x[1]
        s[0,1,0]=(-0.986148210247)*x[0]**o+(-0.915312621763)*x[0]+(0.96862065918)*x[1]**o+(-0.789867091704)*x[1]
        s[0,1,1]=(0.937339541643)*x[0]**o+(0.707719614445)*x[0]+(-0.0185139390834)*x[1]**o+(0.370474130738)*x[1]
        s[1,0,0]=(0.351845644061)*x[0]**o+(-0.790772449987)*x[0]+(0.446801930114)*x[1]**o+(-0.79526180504)*x[1]
        s[1,0,1]=(0.807251261875)*x[0]**o+(-0.955059117804)*x[0]+(-0.797368998645)*x[1]**o+(-0.334719758479)*x[1]
        s[1,1,0]=(0.896257416662)*x[0]**o+(-0.630142892036)*x[0]+(0.697410593951)*x[1]**o+(-0.719230278941)*x[1]
        s[1,1,1]=(-0.628604182238)*x[0]**o+(-0.98443009117)*x[0]+(0.213668228562)*x[1]**o+(0.67702641015)*x[1]
        s[2,0,0]=(0.14317864629)*x[0]**o+(-0.668340038203)*x[0]+(-0.624330217214)*x[1]**o+(-0.18225231407)*x[1]
        s[2,0,1]=(-0.120451964275)*x[0]**o+(0.476453331533)*x[0]+(0.428548558072)*x[1]**o+(0.487245871651)*x[1]
        s[2,1,0]=(0.957821516578)*x[0]**o+(0.245067538044)*x[0]+(0.59057574623)*x[1]**o+(0.799481912975)*x[1]
        s[2,1,1]=(0.157267505975)*x[0]**o+(0.486829437266)*x[0]+(0.72573406263)*x[1]**o+(-0.854913668061)*x[1]
        s[3,0,0]=(0.981080241547)*x[0]**o+(-0.475318323052)*x[0]+(-0.309438112923)*x[1]**o+(0.432416200997)*x[1]
        s[3,0,1]=(0.598520022432)*x[0]**o+(-0.384489046077)*x[0]+(-0.563263372262)*x[1]**o+(-0.787292999177)*x[1]
        s[3,1,0]=(-0.581355275119)*x[0]**o+(-0.404673888552)*x[0]+(-0.481631833004)*x[1]**o+(-0.873167338173)*x[1]
        s[3,1,1]=(-0.809669221808)*x[0]**o+(0.271025961643)*x[0]+(0.532318218814)*x[1]**o+(0.892267754107)*x[1]
        s[4,0,0]=(-0.654686292022)*x[0]**o+(-0.21708498353)*x[0]+(-0.277205071363)*x[1]**o+(-0.427753053795)*x[1]
        s[4,0,1]=(-0.540515032413)*x[0]**o+(-0.902343725594)*x[0]+(-0.880588922705)*x[1]**o+(0.449347508903)*x[1]
        s[4,1,0]=(-0.429101982872)*x[0]**o+(0.537796527897)*x[0]+(0.417004167989)*x[1]**o+(-0.0986410894762)*x[1]
        s[4,1,1]=(0.760414524096)*x[0]**o+(0.587360702148)*x[0]+(-0.977847482262)*x[1]**o+(0.233948889011)*x[1]
        s[5,0,0]=(0.489553684109)*x[0]**o+(-0.576529833035)*x[0]+(0.534315888534)*x[1]**o+(-0.27340402533)*x[1]
        s[5,0,1]=(-0.399620876418)*x[0]**o+(-0.313714298147)*x[0]+(0.180876722218)*x[1]**o+(-0.783972277649)*x[1]
        s[5,1,0]=(-0.35422230106)*x[0]**o+(0.015751912455)*x[0]+(0.10875420172)*x[1]**o+(-0.264805104189)*x[1]
        s[5,1,1]=(0.275494542521)*x[0]**o+(-0.758423085522)*x[0]+(0.223846690671)*x[1]**o+(0.717226976294)*x[1]
        ref[0,0,0]=(-0.564155350537)*x_ref[0]**o+(-0.800380058082)*x_ref[0]+(0.689149742912)*x_ref[1]**o+(-0.730209879145)*x_ref[1]
        ref[0,0,1]=(-0.875752239419)*x_ref[0]**o+(0.620770279709)*x_ref[0]+(-0.405669056556)*x_ref[1]**o+(-0.650461825052)*x_ref[1]
        ref[0,1,0]=(-0.986148210247)*x_ref[0]**o+(-0.915312621763)*x_ref[0]+(0.96862065918)*x_ref[1]**o+(-0.789867091704)*x_ref[1]
        ref[0,1,1]=(0.937339541643)*x_ref[0]**o+(0.707719614445)*x_ref[0]+(-0.0185139390834)*x_ref[1]**o+(0.370474130738)*x_ref[1]
        ref[1,0,0]=(0.351845644061)*x_ref[0]**o+(-0.790772449987)*x_ref[0]+(0.446801930114)*x_ref[1]**o+(-0.79526180504)*x_ref[1]
        ref[1,0,1]=(0.807251261875)*x_ref[0]**o+(-0.955059117804)*x_ref[0]+(-0.797368998645)*x_ref[1]**o+(-0.334719758479)*x_ref[1]
        ref[1,1,0]=(0.896257416662)*x_ref[0]**o+(-0.630142892036)*x_ref[0]+(0.697410593951)*x_ref[1]**o+(-0.719230278941)*x_ref[1]
        ref[1,1,1]=(-0.628604182238)*x_ref[0]**o+(-0.98443009117)*x_ref[0]+(0.213668228562)*x_ref[1]**o+(0.67702641015)*x_ref[1]
        ref[2,0,0]=(0.14317864629)*x_ref[0]**o+(-0.668340038203)*x_ref[0]+(-0.624330217214)*x_ref[1]**o+(-0.18225231407)*x_ref[1]
        ref[2,0,1]=(-0.120451964275)*x_ref[0]**o+(0.476453331533)*x_ref[0]+(0.428548558072)*x_ref[1]**o+(0.487245871651)*x_ref[1]
        ref[2,1,0]=(0.957821516578)*x_ref[0]**o+(0.245067538044)*x_ref[0]+(0.59057574623)*x_ref[1]**o+(0.799481912975)*x_ref[1]
        ref[2,1,1]=(0.157267505975)*x_ref[0]**o+(0.486829437266)*x_ref[0]+(0.72573406263)*x_ref[1]**o+(-0.854913668061)*x_ref[1]
        ref[3,0,0]=(0.981080241547)*x_ref[0]**o+(-0.475318323052)*x_ref[0]+(-0.309438112923)*x_ref[1]**o+(0.432416200997)*x_ref[1]
        ref[3,0,1]=(0.598520022432)*x_ref[0]**o+(-0.384489046077)*x_ref[0]+(-0.563263372262)*x_ref[1]**o+(-0.787292999177)*x_ref[1]
        ref[3,1,0]=(-0.581355275119)*x_ref[0]**o+(-0.404673888552)*x_ref[0]+(-0.481631833004)*x_ref[1]**o+(-0.873167338173)*x_ref[1]
        ref[3,1,1]=(-0.809669221808)*x_ref[0]**o+(0.271025961643)*x_ref[0]+(0.532318218814)*x_ref[1]**o+(0.892267754107)*x_ref[1]
        ref[4,0,0]=(-0.654686292022)*x_ref[0]**o+(-0.21708498353)*x_ref[0]+(-0.277205071363)*x_ref[1]**o+(-0.427753053795)*x_ref[1]
        ref[4,0,1]=(-0.540515032413)*x_ref[0]**o+(-0.902343725594)*x_ref[0]+(-0.880588922705)*x_ref[1]**o+(0.449347508903)*x_ref[1]
        ref[4,1,0]=(-0.429101982872)*x_ref[0]**o+(0.537796527897)*x_ref[0]+(0.417004167989)*x_ref[1]**o+(-0.0986410894762)*x_ref[1]
        ref[4,1,1]=(0.760414524096)*x_ref[0]**o+(0.587360702148)*x_ref[0]+(-0.977847482262)*x_ref[1]**o+(0.233948889011)*x_ref[1]
        ref[5,0,0]=(0.489553684109)*x_ref[0]**o+(-0.576529833035)*x_ref[0]+(0.534315888534)*x_ref[1]**o+(-0.27340402533)*x_ref[1]
        ref[5,0,1]=(-0.399620876418)*x_ref[0]**o+(-0.313714298147)*x_ref[0]+(0.180876722218)*x_ref[1]**o+(-0.783972277649)*x_ref[1]
        ref[5,1,0]=(-0.35422230106)*x_ref[0]**o+(0.015751912455)*x_ref[0]+(0.10875420172)*x_ref[1]**o+(-0.264805104189)*x_ref[1]
        ref[5,1,1]=(0.275494542521)*x_ref[0]**o+(-0.758423085522)*x_ref[0]+(0.223846690671)*x_ref[1]**o+(0.717226976294)*x_ref[1]
      else:
        s[0,0,0]=(-0.774622440694)*x[0]**o+(-0.871495007805)*x[0]+(0.462893985101)*x[1]**o+(0.97809884411)*x[1]+(-0.294782632822)*x[2]**o+(0.758086059139)*x[2]
        s[0,0,1]=(0.593542531653)*x[0]**o+(-0.623508645258)*x[0]+(-0.869924719996)*x[1]**o+(0.683987028818)*x[1]+(0.214983363659)*x[2]**o+(0.937703017041)*x[2]
        s[0,1,0]=(0.683415745305)*x[0]**o+(0.514284128555)*x[0]+(-0.495949103899)*x[1]**o+(0.0846543741985)*x[1]+(0.713593706483)*x[2]**o+(0.68727976891)*x[2]
        s[0,1,1]=(0.890309435361)*x[0]**o+(-0.971477359218)*x[0]+(0.832140831851)*x[1]**o+(-0.208499413601)*x[1]+(0.632915577047)*x[2]**o+(0.255341853742)*x[2]
        s[1,0,0]=(0.180392491602)*x[0]**o+(-0.535445127886)*x[0]+(-0.785876938514)*x[1]**o+(0.0816315522405)*x[1]+(0.648551332906)*x[2]**o+(0.0429567512952)*x[2]
        s[1,0,1]=(-0.662744890855)*x[0]**o+(0.961507967025)*x[0]+(-0.535104990223)*x[1]**o+(-0.100555677207)*x[1]+(0.254286176467)*x[2]**o+(0.23822457275)*x[2]
        s[1,1,0]=(0.0618685350699)*x[0]**o+(0.801588371288)*x[0]+(0.775832395529)*x[1]**o+(0.76315781257)*x[1]+(0.201501889875)*x[2]**o+(-0.594080987915)*x[2]
        s[1,1,1]=(-0.472648941872)*x[0]**o+(-0.268595087901)*x[0]+(-0.634032640513)*x[1]**o+(0.895658995237)*x[1]+(-0.311243350516)*x[2]**o+(-0.941899709416)*x[2]
        s[2,0,0]=(-0.937843815317)*x[0]**o+(-0.732052451935)*x[0]+(-0.195729427352)*x[1]**o+(-0.686479979743)*x[1]+(0.233916070094)*x[2]**o+(-0.0840356688737)*x[2]
        s[2,0,1]=(0.010530087739)*x[0]**o+(-0.790921976555)*x[0]+(0.0748807338793)*x[1]**o+(-0.903727762342)*x[1]+(0.627134883108)*x[2]**o+(-0.0387413340061)*x[2]
        s[2,1,0]=(0.546735016177)*x[0]**o+(0.315849018818)*x[0]+(-0.95747551189)*x[1]**o+(0.0565452708853)*x[1]+(0.305498701829)*x[2]**o+(0.122314734589)*x[2]
        s[2,1,1]=(-0.838026718661)*x[0]**o+(0.128953039794)*x[0]+(0.624067160575)*x[1]**o+(0.444434080889)*x[1]+(-0.835923583772)*x[2]**o+(-0.286578073481)*x[2]
        s[3,0,0]=(0.00360603498589)*x[0]**o+(0.842043331421)*x[0]+(0.362497876527)*x[1]**o+(0.0681458369022)*x[1]+(0.75852693193)*x[2]**o+(-0.53716235)*x[2]
        s[3,0,1]=(-0.159678757928)*x[0]**o+(0.883308054535)*x[0]+(0.21591905568)*x[1]**o+(-0.898265348642)*x[1]+(-0.951423349936)*x[2]**o+(-0.240034084504)*x[2]
        s[3,1,0]=(0.692230687386)*x[0]**o+(0.0335587781762)*x[0]+(-0.766571197965)*x[1]**o+(0.128927596751)*x[1]+(0.986505471196)*x[2]**o+(0.149177674029)*x[2]
        s[3,1,1]=(0.135591222661)*x[0]**o+(0.931657866742)*x[0]+(-0.4633142936)*x[1]**o+(0.868618051626)*x[1]+(-0.563215256065)*x[2]**o+(0.937159665351)*x[2]
        s[4,0,0]=(-0.00496897513325)*x[0]**o+(0.64627613088)*x[0]+(-0.63362415209)*x[1]**o+(0.875299573404)*x[1]+(0.00915422996533)*x[2]**o+(-0.898719885994)*x[2]
        s[4,0,1]=(0.531986621993)*x[0]**o+(-0.592503622446)*x[0]+(0.168725128639)*x[1]**o+(-0.415718155368)*x[1]+(0.153807804669)*x[2]**o+(0.0517662389301)*x[2]
        s[4,1,0]=(-0.40167686609)*x[0]**o+(0.765205907898)*x[0]+(-0.330400749167)*x[1]**o+(-0.227287643007)*x[1]+(0.86248041007)*x[2]**o+(0.475472570688)*x[2]
        s[4,1,1]=(-0.379429448909)*x[0]**o+(-0.746283001238)*x[0]+(0.451902471866)*x[1]**o+(0.975850402176)*x[1]+(-0.752666040136)*x[2]**o+(0.310437725938)*x[2]
        s[5,0,0]=(-0.790243943683)*x[0]**o+(-0.483698083361)*x[0]+(0.298354468428)*x[1]**o+(-0.430878231617)*x[1]+(-0.970790186654)*x[2]**o+(0.295573821695)*x[2]
        s[5,0,1]=(0.342850991179)*x[0]**o+(0.716485409508)*x[0]+(0.0929941902588)*x[1]**o+(-0.804594470152)*x[1]+(-0.6262904765)*x[2]**o+(0.1281559082)*x[2]
        s[5,1,0]=(0.6989819417)*x[0]**o+(0.513436682661)*x[0]+(-0.398488542709)*x[1]**o+(0.429421472572)*x[1]+(0.0491769595898)*x[2]**o+(0.402704297546)*x[2]
        s[5,1,1]=(-0.638592369916)*x[0]**o+(0.164631869734)*x[0]+(0.306302338543)*x[1]**o+(0.293774443655)*x[1]+(-0.536801505818)*x[2]**o+(0.758326129903)*x[2]
        ref[0,0,0]=(-0.774622440694)*x_ref[0]**o+(-0.871495007805)*x_ref[0]+(0.462893985101)*x_ref[1]**o+(0.97809884411)*x_ref[1]+(-0.294782632822)*x_ref[2]**o+(0.758086059139)*x_ref[2]
        ref[0,0,1]=(0.593542531653)*x_ref[0]**o+(-0.623508645258)*x_ref[0]+(-0.869924719996)*x_ref[1]**o+(0.683987028818)*x_ref[1]+(0.214983363659)*x_ref[2]**o+(0.937703017041)*x_ref[2]
        ref[0,1,0]=(0.683415745305)*x_ref[0]**o+(0.514284128555)*x_ref[0]+(-0.495949103899)*x_ref[1]**o+(0.0846543741985)*x_ref[1]+(0.713593706483)*x_ref[2]**o+(0.68727976891)*x_ref[2]
        ref[0,1,1]=(0.890309435361)*x_ref[0]**o+(-0.971477359218)*x_ref[0]+(0.832140831851)*x_ref[1]**o+(-0.208499413601)*x_ref[1]+(0.632915577047)*x_ref[2]**o+(0.255341853742)*x_ref[2]
        ref[1,0,0]=(0.180392491602)*x_ref[0]**o+(-0.535445127886)*x_ref[0]+(-0.785876938514)*x_ref[1]**o+(0.0816315522405)*x_ref[1]+(0.648551332906)*x_ref[2]**o+(0.0429567512952)*x_ref[2]
        ref[1,0,1]=(-0.662744890855)*x_ref[0]**o+(0.961507967025)*x_ref[0]+(-0.535104990223)*x_ref[1]**o+(-0.100555677207)*x_ref[1]+(0.254286176467)*x_ref[2]**o+(0.23822457275)*x_ref[2]
        ref[1,1,0]=(0.0618685350699)*x_ref[0]**o+(0.801588371288)*x_ref[0]+(0.775832395529)*x_ref[1]**o+(0.76315781257)*x_ref[1]+(0.201501889875)*x_ref[2]**o+(-0.594080987915)*x_ref[2]
        ref[1,1,1]=(-0.472648941872)*x_ref[0]**o+(-0.268595087901)*x_ref[0]+(-0.634032640513)*x_ref[1]**o+(0.895658995237)*x_ref[1]+(-0.311243350516)*x_ref[2]**o+(-0.941899709416)*x_ref[2]
        ref[2,0,0]=(-0.937843815317)*x_ref[0]**o+(-0.732052451935)*x_ref[0]+(-0.195729427352)*x_ref[1]**o+(-0.686479979743)*x_ref[1]+(0.233916070094)*x_ref[2]**o+(-0.0840356688737)*x_ref[2]
        ref[2,0,1]=(0.010530087739)*x_ref[0]**o+(-0.790921976555)*x_ref[0]+(0.0748807338793)*x_ref[1]**o+(-0.903727762342)*x_ref[1]+(0.627134883108)*x_ref[2]**o+(-0.0387413340061)*x_ref[2]
        ref[2,1,0]=(0.546735016177)*x_ref[0]**o+(0.315849018818)*x_ref[0]+(-0.95747551189)*x_ref[1]**o+(0.0565452708853)*x_ref[1]+(0.305498701829)*x_ref[2]**o+(0.122314734589)*x_ref[2]
        ref[2,1,1]=(-0.838026718661)*x_ref[0]**o+(0.128953039794)*x_ref[0]+(0.624067160575)*x_ref[1]**o+(0.444434080889)*x_ref[1]+(-0.835923583772)*x_ref[2]**o+(-0.286578073481)*x_ref[2]
        ref[3,0,0]=(0.00360603498589)*x_ref[0]**o+(0.842043331421)*x_ref[0]+(0.362497876527)*x_ref[1]**o+(0.0681458369022)*x_ref[1]+(0.75852693193)*x_ref[2]**o+(-0.53716235)*x_ref[2]
        ref[3,0,1]=(-0.159678757928)*x_ref[0]**o+(0.883308054535)*x_ref[0]+(0.21591905568)*x_ref[1]**o+(-0.898265348642)*x_ref[1]+(-0.951423349936)*x_ref[2]**o+(-0.240034084504)*x_ref[2]
        ref[3,1,0]=(0.692230687386)*x_ref[0]**o+(0.0335587781762)*x_ref[0]+(-0.766571197965)*x_ref[1]**o+(0.128927596751)*x_ref[1]+(0.986505471196)*x_ref[2]**o+(0.149177674029)*x_ref[2]
        ref[3,1,1]=(0.135591222661)*x_ref[0]**o+(0.931657866742)*x_ref[0]+(-0.4633142936)*x_ref[1]**o+(0.868618051626)*x_ref[1]+(-0.563215256065)*x_ref[2]**o+(0.937159665351)*x_ref[2]
        ref[4,0,0]=(-0.00496897513325)*x_ref[0]**o+(0.64627613088)*x_ref[0]+(-0.63362415209)*x_ref[1]**o+(0.875299573404)*x_ref[1]+(0.00915422996533)*x_ref[2]**o+(-0.898719885994)*x_ref[2]
        ref[4,0,1]=(0.531986621993)*x_ref[0]**o+(-0.592503622446)*x_ref[0]+(0.168725128639)*x_ref[1]**o+(-0.415718155368)*x_ref[1]+(0.153807804669)*x_ref[2]**o+(0.0517662389301)*x_ref[2]
        ref[4,1,0]=(-0.40167686609)*x_ref[0]**o+(0.765205907898)*x_ref[0]+(-0.330400749167)*x_ref[1]**o+(-0.227287643007)*x_ref[1]+(0.86248041007)*x_ref[2]**o+(0.475472570688)*x_ref[2]
        ref[4,1,1]=(-0.379429448909)*x_ref[0]**o+(-0.746283001238)*x_ref[0]+(0.451902471866)*x_ref[1]**o+(0.975850402176)*x_ref[1]+(-0.752666040136)*x_ref[2]**o+(0.310437725938)*x_ref[2]
        ref[5,0,0]=(-0.790243943683)*x_ref[0]**o+(-0.483698083361)*x_ref[0]+(0.298354468428)*x_ref[1]**o+(-0.430878231617)*x_ref[1]+(-0.970790186654)*x_ref[2]**o+(0.295573821695)*x_ref[2]
        ref[5,0,1]=(0.342850991179)*x_ref[0]**o+(0.716485409508)*x_ref[0]+(0.0929941902588)*x_ref[1]**o+(-0.804594470152)*x_ref[1]+(-0.6262904765)*x_ref[2]**o+(0.1281559082)*x_ref[2]
        ref[5,1,0]=(0.6989819417)*x_ref[0]**o+(0.513436682661)*x_ref[0]+(-0.398488542709)*x_ref[1]**o+(0.429421472572)*x_ref[1]+(0.0491769595898)*x_ref[2]**o+(0.402704297546)*x_ref[2]
        ref[5,1,1]=(-0.638592369916)*x_ref[0]**o+(0.164631869734)*x_ref[0]+(0.306302338543)*x_ref[1]**o+(0.293774443655)*x_ref[1]+(-0.536801505818)*x_ref[2]**o+(0.758326129903)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_ContinuousFunction_rank4(self):
      """
      tests interpolate for rank 4 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ContinuousFunction onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5, 3, 2),dim=dim)
      s=Data(0,(4, 5, 3, 2),w)
      ref=Data(0,(4, 5, 3, 2),w_ref)
      if dim==2:
        s[0,0,0,0]=(-0.474386706114)*x[0]**o+(-0.0739442523179)*x[0]+(-0.934839682165)*x[1]**o+(-0.874748715361)*x[1]
        s[0,0,0,1]=(0.970835666471)*x[0]**o+(-0.210304396563)*x[0]+(-0.245577454075)*x[1]**o+(0.706874293911)*x[1]
        s[0,0,1,0]=(-0.886508211386)*x[0]**o+(0.40749471331)*x[0]+(-0.575539262859)*x[1]**o+(-0.714242182567)*x[1]
        s[0,0,1,1]=(0.0925249369772)*x[0]**o+(-0.326000469119)*x[0]+(0.427385578633)*x[1]**o+(0.723913449012)*x[1]
        s[0,0,2,0]=(0.178301174425)*x[0]**o+(-0.935408769107)*x[0]+(0.185756314077)*x[1]**o+(0.779639226265)*x[1]
        s[0,0,2,1]=(0.517627638569)*x[0]**o+(0.833228507097)*x[0]+(0.509792014643)*x[1]**o+(0.512759085233)*x[1]
        s[0,1,0,0]=(0.943220804246)*x[0]**o+(0.358695377859)*x[0]+(0.483282522344)*x[1]**o+(-0.780850766978)*x[1]
        s[0,1,0,1]=(0.228281969805)*x[0]**o+(0.378112450475)*x[0]+(-0.265569820367)*x[1]**o+(0.793063241358)*x[1]
        s[0,1,1,0]=(-0.991239767457)*x[0]**o+(0.584083573937)*x[0]+(0.00907947673395)*x[1]**o+(-0.0949125419342)*x[1]
        s[0,1,1,1]=(0.312976776903)*x[0]**o+(-0.572058365439)*x[0]+(0.337493203667)*x[1]**o+(-0.559749276661)*x[1]
        s[0,1,2,0]=(-0.154473081077)*x[0]**o+(0.946407875413)*x[0]+(0.721108882799)*x[1]**o+(0.409146005045)*x[1]
        s[0,1,2,1]=(0.42027715889)*x[0]**o+(-0.892090996743)*x[0]+(-0.718807145454)*x[1]**o+(0.229975539898)*x[1]
        s[0,2,0,0]=(0.378117082397)*x[0]**o+(-0.613916810385)*x[0]+(-0.564348246694)*x[1]**o+(0.101261637507)*x[1]
        s[0,2,0,1]=(0.839696159524)*x[0]**o+(0.24184619932)*x[0]+(-0.095338864129)*x[1]**o+(0.903106275587)*x[1]
        s[0,2,1,0]=(0.620479230354)*x[0]**o+(-0.34112105179)*x[0]+(0.624902166524)*x[1]**o+(-0.693289910121)*x[1]
        s[0,2,1,1]=(-0.39760464469)*x[0]**o+(-0.921383269336)*x[0]+(0.753657132321)*x[1]**o+(0.324768021371)*x[1]
        s[0,2,2,0]=(0.0243534816004)*x[0]**o+(0.290663121221)*x[0]+(0.0305666628617)*x[1]**o+(0.0446693318891)*x[1]
        s[0,2,2,1]=(-0.14154275886)*x[0]**o+(0.881207887999)*x[0]+(0.399029780289)*x[1]**o+(0.168497381665)*x[1]
        s[0,3,0,0]=(-0.233073561791)*x[0]**o+(-0.513212321666)*x[0]+(-0.651480028725)*x[1]**o+(0.147498573261)*x[1]
        s[0,3,0,1]=(-0.918136406734)*x[0]**o+(-0.279161242114)*x[0]+(0.983432543886)*x[1]**o+(-0.723257302563)*x[1]
        s[0,3,1,0]=(0.290070399137)*x[0]**o+(0.485832779323)*x[0]+(0.955571801349)*x[1]**o+(0.552993034163)*x[1]
        s[0,3,1,1]=(0.688570077213)*x[0]**o+(0.578716466405)*x[0]+(-0.40829418423)*x[1]**o+(-0.918196317732)*x[1]
        s[0,3,2,0]=(0.403069347997)*x[0]**o+(0.119390454784)*x[0]+(-0.322018782913)*x[1]**o+(0.224814772968)*x[1]
        s[0,3,2,1]=(-0.23084264112)*x[0]**o+(-0.398235674194)*x[0]+(-0.730556055687)*x[1]**o+(0.293981158644)*x[1]
        s[0,4,0,0]=(-0.0984201873925)*x[0]**o+(0.018359257988)*x[0]+(-0.0781876875229)*x[1]**o+(0.253344618889)*x[1]
        s[0,4,0,1]=(-0.20899724864)*x[0]**o+(-0.722019204308)*x[0]+(0.555162582715)*x[1]**o+(-0.958068826429)*x[1]
        s[0,4,1,0]=(0.477725976915)*x[0]**o+(0.295872229679)*x[0]+(-0.828208555398)*x[1]**o+(-0.931582818519)*x[1]
        s[0,4,1,1]=(0.721285453077)*x[0]**o+(-0.042927949943)*x[0]+(0.680231405034)*x[1]**o+(-0.16043805248)*x[1]
        s[0,4,2,0]=(0.985988751136)*x[0]**o+(0.885785903078)*x[0]+(-0.0410955856477)*x[1]**o+(-0.552089008201)*x[1]
        s[0,4,2,1]=(0.847466818225)*x[0]**o+(-0.51057282448)*x[0]+(0.163775791256)*x[1]**o+(0.0152553094575)*x[1]
        s[1,0,0,0]=(0.0343855486006)*x[0]**o+(-0.214334464898)*x[0]+(-0.573344070879)*x[1]**o+(0.24905956136)*x[1]
        s[1,0,0,1]=(0.783338978316)*x[0]**o+(0.781324842059)*x[0]+(-0.847509450777)*x[1]**o+(-0.858994488444)*x[1]
        s[1,0,1,0]=(0.59719044885)*x[0]**o+(0.347038111035)*x[0]+(-0.385761256521)*x[1]**o+(-0.571166989318)*x[1]
        s[1,0,1,1]=(0.636911008507)*x[0]**o+(-0.825376916398)*x[0]+(0.32394534216)*x[1]**o+(0.653119502004)*x[1]
        s[1,0,2,0]=(-0.669732532783)*x[0]**o+(-0.145427031253)*x[0]+(-0.303693857831)*x[1]**o+(-0.727878524833)*x[1]
        s[1,0,2,1]=(0.621061585764)*x[0]**o+(0.243432217146)*x[0]+(-0.888610099037)*x[1]**o+(0.298051589144)*x[1]
        s[1,1,0,0]=(0.357341721813)*x[0]**o+(0.346506498237)*x[0]+(-0.555625437961)*x[1]**o+(-0.0662579456147)*x[1]
        s[1,1,0,1]=(0.96955420495)*x[0]**o+(-0.190486134786)*x[0]+(-0.900124677941)*x[1]**o+(0.659993963277)*x[1]
        s[1,1,1,0]=(-0.517837108406)*x[0]**o+(0.00779665389526)*x[0]+(-0.0544022169324)*x[1]**o+(0.888217855388)*x[1]
        s[1,1,1,1]=(0.252515646862)*x[0]**o+(0.328948655235)*x[0]+(0.265916563145)*x[1]**o+(-0.491865215095)*x[1]
        s[1,1,2,0]=(0.922520219928)*x[0]**o+(0.463811222486)*x[0]+(0.651244201962)*x[1]**o+(0.982565677231)*x[1]
        s[1,1,2,1]=(0.257711278265)*x[0]**o+(0.23971381197)*x[0]+(-0.168559977915)*x[1]**o+(0.453909616568)*x[1]
        s[1,2,0,0]=(0.0426933948917)*x[0]**o+(-0.477381944712)*x[0]+(0.503797983762)*x[1]**o+(-0.943092701839)*x[1]
        s[1,2,0,1]=(0.913591544613)*x[0]**o+(-0.895063818412)*x[0]+(0.252264576564)*x[1]**o+(0.424846745491)*x[1]
        s[1,2,1,0]=(0.4199934247)*x[0]**o+(-0.128668379749)*x[0]+(-0.157997092493)*x[1]**o+(-0.174476338072)*x[1]
        s[1,2,1,1]=(0.394310091468)*x[0]**o+(0.599539798636)*x[0]+(0.133087615453)*x[1]**o+(-0.22271509754)*x[1]
        s[1,2,2,0]=(0.559521330007)*x[0]**o+(-0.411300693025)*x[0]+(-0.543921885179)*x[1]**o+(-0.0506834937614)*x[1]
        s[1,2,2,1]=(-0.423996886775)*x[0]**o+(0.14409349326)*x[0]+(-0.970632608145)*x[1]**o+(0.582937355294)*x[1]
        s[1,3,0,0]=(-0.884972026022)*x[0]**o+(-0.872260393893)*x[0]+(0.224463013859)*x[1]**o+(-0.245013144519)*x[1]
        s[1,3,0,1]=(-0.834873836296)*x[0]**o+(-0.778611513205)*x[0]+(0.989113281842)*x[1]**o+(0.847047912646)*x[1]
        s[1,3,1,0]=(-0.0900703847239)*x[0]**o+(-0.0295919916877)*x[0]+(0.510380741505)*x[1]**o+(-0.00354494905936)*x[1]
        s[1,3,1,1]=(0.0422103052661)*x[0]**o+(0.31376324435)*x[0]+(0.831890412519)*x[1]**o+(0.764406560706)*x[1]
        s[1,3,2,0]=(0.496699862489)*x[0]**o+(0.477514051712)*x[0]+(0.156435695163)*x[1]**o+(0.339443828388)*x[1]
        s[1,3,2,1]=(0.396689832399)*x[0]**o+(0.243605505552)*x[0]+(-0.0357728699149)*x[1]**o+(0.930734881576)*x[1]
        s[1,4,0,0]=(-0.90788627878)*x[0]**o+(0.82597715373)*x[0]+(0.287988132693)*x[1]**o+(-0.834307302134)*x[1]
        s[1,4,0,1]=(-0.942882799669)*x[0]**o+(-0.149609905045)*x[0]+(-0.340025012164)*x[1]**o+(0.539546162654)*x[1]
        s[1,4,1,0]=(-0.244377326727)*x[0]**o+(-0.586439384775)*x[0]+(0.89506494604)*x[1]**o+(0.233226803834)*x[1]
        s[1,4,1,1]=(0.864582224514)*x[0]**o+(0.0687754700765)*x[0]+(-0.641556536115)*x[1]**o+(0.22036806322)*x[1]
        s[1,4,2,0]=(-0.552589117461)*x[0]**o+(-0.705244093999)*x[0]+(0.378837921567)*x[1]**o+(-0.173968977986)*x[1]
        s[1,4,2,1]=(0.846900000181)*x[0]**o+(-0.52589472084)*x[0]+(0.872400573242)*x[1]**o+(0.182683546911)*x[1]
        s[2,0,0,0]=(0.305006901177)*x[0]**o+(-0.732858762152)*x[0]+(-0.882239786812)*x[1]**o+(0.481712295538)*x[1]
        s[2,0,0,1]=(0.836332373747)*x[0]**o+(0.63905574965)*x[0]+(0.039226709213)*x[1]**o+(0.704758586936)*x[1]
        s[2,0,1,0]=(0.51805564252)*x[0]**o+(0.641883965761)*x[0]+(0.901663292208)*x[1]**o+(0.375272715842)*x[1]
        s[2,0,1,1]=(-0.980058913052)*x[0]**o+(0.385619663167)*x[0]+(0.0722192483217)*x[1]**o+(0.00650731529315)*x[1]
        s[2,0,2,0]=(-0.25791454536)*x[0]**o+(0.460015385804)*x[0]+(0.234970544559)*x[1]**o+(-0.11962323176)*x[1]
        s[2,0,2,1]=(0.023390322577)*x[0]**o+(-0.104449150638)*x[0]+(0.149724956284)*x[1]**o+(-0.309848596985)*x[1]
        s[2,1,0,0]=(0.603565629307)*x[0]**o+(-0.143206239367)*x[0]+(-0.0430666954645)*x[1]**o+(-0.154546675756)*x[1]
        s[2,1,0,1]=(-0.980112660961)*x[0]**o+(0.419145835585)*x[0]+(-0.76202669209)*x[1]**o+(-0.648615698697)*x[1]
        s[2,1,1,0]=(0.805964945679)*x[0]**o+(0.650144650504)*x[0]+(0.308110033125)*x[1]**o+(0.0666476104838)*x[1]
        s[2,1,1,1]=(0.42561398203)*x[0]**o+(0.259276259654)*x[0]+(-0.215194398301)*x[1]**o+(0.880282506885)*x[1]
        s[2,1,2,0]=(-0.413779899229)*x[0]**o+(-0.796859182258)*x[0]+(0.606865226329)*x[1]**o+(-0.783313517188)*x[1]
        s[2,1,2,1]=(0.402767439742)*x[0]**o+(-0.692468937875)*x[0]+(0.27866251803)*x[1]**o+(0.95092559773)*x[1]
        s[2,2,0,0]=(0.749107532378)*x[0]**o+(-0.364633672479)*x[0]+(0.970710337513)*x[1]**o+(0.1986749977)*x[1]
        s[2,2,0,1]=(-0.722832876805)*x[0]**o+(-0.238825576333)*x[0]+(-0.101752751414)*x[1]**o+(-0.960067435044)*x[1]
        s[2,2,1,0]=(-0.704825996403)*x[0]**o+(0.74244530617)*x[0]+(-0.586571188279)*x[1]**o+(0.841297962355)*x[1]
        s[2,2,1,1]=(0.586127464995)*x[0]**o+(0.454050196541)*x[0]+(0.225294694159)*x[1]**o+(0.705925824031)*x[1]
        s[2,2,2,0]=(0.535446366333)*x[0]**o+(-0.787565531495)*x[0]+(-0.566053725536)*x[1]**o+(0.468968266813)*x[1]
        s[2,2,2,1]=(0.449236149776)*x[0]**o+(0.589799099557)*x[0]+(0.189540632578)*x[1]**o+(-0.775036139482)*x[1]
        s[2,3,0,0]=(0.979572074948)*x[0]**o+(-0.806420343846)*x[0]+(0.661682487968)*x[1]**o+(-0.615846811222)*x[1]
        s[2,3,0,1]=(0.021072244205)*x[0]**o+(0.878344628134)*x[0]+(-0.381067816502)*x[1]**o+(0.229027446937)*x[1]
        s[2,3,1,0]=(0.0182291979593)*x[0]**o+(0.625595584968)*x[0]+(-0.824161018488)*x[1]**o+(-0.316191235785)*x[1]
        s[2,3,1,1]=(-0.0168799900253)*x[0]**o+(-0.137230078645)*x[0]+(0.0240280311645)*x[1]**o+(-0.364099905801)*x[1]
        s[2,3,2,0]=(0.374193672582)*x[0]**o+(0.429814892173)*x[0]+(0.0194417431204)*x[1]**o+(0.830453174612)*x[1]
        s[2,3,2,1]=(-0.683758441576)*x[0]**o+(-0.759635036652)*x[0]+(-0.50102074202)*x[1]**o+(0.123764523628)*x[1]
        s[2,4,0,0]=(-0.0817296101008)*x[0]**o+(0.730474427452)*x[0]+(-0.0733544873209)*x[1]**o+(0.416303527556)*x[1]
        s[2,4,0,1]=(0.258598296267)*x[0]**o+(0.686740404407)*x[0]+(0.543414259362)*x[1]**o+(0.518071987904)*x[1]
        s[2,4,1,0]=(-0.32475099592)*x[0]**o+(-0.0460901003729)*x[0]+(0.275611466137)*x[1]**o+(0.8415303665)*x[1]
        s[2,4,1,1]=(0.5015310862)*x[0]**o+(-0.71451126551)*x[0]+(0.897703909295)*x[1]**o+(-0.0476780709524)*x[1]
        s[2,4,2,0]=(0.612722779576)*x[0]**o+(0.424485459817)*x[0]+(-0.136026922168)*x[1]**o+(0.16727850944)*x[1]
        s[2,4,2,1]=(-0.841159017194)*x[0]**o+(0.856949657441)*x[0]+(-0.975786438976)*x[1]**o+(-0.407229683889)*x[1]
        s[3,0,0,0]=(0.833459943775)*x[0]**o+(-0.230178834842)*x[0]+(-0.660801597291)*x[1]**o+(-0.738869823127)*x[1]
        s[3,0,0,1]=(0.30987970183)*x[0]**o+(-0.610707485471)*x[0]+(0.628798231358)*x[1]**o+(-0.0491116840572)*x[1]
        s[3,0,1,0]=(0.649739974341)*x[0]**o+(-0.450666468651)*x[0]+(0.901638992166)*x[1]**o+(-0.787549067401)*x[1]
        s[3,0,1,1]=(0.934583955853)*x[0]**o+(-0.859086605977)*x[0]+(0.0975143570756)*x[1]**o+(0.0350097808797)*x[1]
        s[3,0,2,0]=(0.837723849965)*x[0]**o+(-0.665696123522)*x[0]+(-0.70629678851)*x[1]**o+(-0.468945853667)*x[1]
        s[3,0,2,1]=(0.526302645633)*x[0]**o+(-0.0979568077223)*x[0]+(-0.767623484018)*x[1]**o+(-0.21912588217)*x[1]
        s[3,1,0,0]=(0.18832505785)*x[0]**o+(0.45004009293)*x[0]+(-0.823702397711)*x[1]**o+(-0.912642057295)*x[1]
        s[3,1,0,1]=(0.554483721388)*x[0]**o+(-0.607925140409)*x[0]+(-0.998308584629)*x[1]**o+(-0.538184347898)*x[1]
        s[3,1,1,0]=(-0.682363035722)*x[0]**o+(0.55318030401)*x[0]+(0.562408350319)*x[1]**o+(0.220145757421)*x[1]
        s[3,1,1,1]=(0.924816884714)*x[0]**o+(-0.295767203325)*x[0]+(0.230343937118)*x[1]**o+(0.957747143338)*x[1]
        s[3,1,2,0]=(0.235422171194)*x[0]**o+(0.264750603603)*x[0]+(0.13053017764)*x[1]**o+(0.628805980195)*x[1]
        s[3,1,2,1]=(-0.603111233811)*x[0]**o+(-0.325257245906)*x[0]+(-0.879402225173)*x[1]**o+(0.977736365104)*x[1]
        s[3,2,0,0]=(0.463242222753)*x[0]**o+(-0.802230749276)*x[0]+(0.085387156258)*x[1]**o+(0.18309223952)*x[1]
        s[3,2,0,1]=(0.778060805693)*x[0]**o+(0.32136550684)*x[0]+(0.775284717696)*x[1]**o+(0.151449430741)*x[1]
        s[3,2,1,0]=(0.288473735099)*x[0]**o+(0.675142038399)*x[0]+(-0.313098988069)*x[1]**o+(-0.771883975974)*x[1]
        s[3,2,1,1]=(0.715304130316)*x[0]**o+(0.994375393616)*x[0]+(-0.874667784061)*x[1]**o+(-0.29510377129)*x[1]
        s[3,2,2,0]=(-0.211496384269)*x[0]**o+(0.146746732763)*x[0]+(0.234317983556)*x[1]**o+(0.844710015939)*x[1]
        s[3,2,2,1]=(0.557426563891)*x[0]**o+(0.181380049852)*x[0]+(-0.068830884747)*x[1]**o+(0.424778582562)*x[1]
        s[3,3,0,0]=(-0.784389821334)*x[0]**o+(0.533073578323)*x[0]+(-0.172622226211)*x[1]**o+(-0.0882056556263)*x[1]
        s[3,3,0,1]=(-0.369034971239)*x[0]**o+(-0.323409858793)*x[0]+(0.408766889438)*x[1]**o+(-0.653470515807)*x[1]
        s[3,3,1,0]=(0.239411265848)*x[0]**o+(-0.672719205684)*x[0]+(-0.751715345243)*x[1]**o+(-0.4586462848)*x[1]
        s[3,3,1,1]=(-0.937724235502)*x[0]**o+(-0.9284117207)*x[0]+(-0.584507468424)*x[1]**o+(0.779562243368)*x[1]
        s[3,3,2,0]=(-0.563406066207)*x[0]**o+(0.115504525413)*x[0]+(0.0230187346201)*x[1]**o+(0.960113641875)*x[1]
        s[3,3,2,1]=(0.792686350755)*x[0]**o+(0.560920910518)*x[0]+(0.0766597516323)*x[1]**o+(-0.728345800422)*x[1]
        s[3,4,0,0]=(-0.182383283107)*x[0]**o+(0.604818623768)*x[0]+(0.756868334363)*x[1]**o+(0.0259669270239)*x[1]
        s[3,4,0,1]=(0.547592534695)*x[0]**o+(0.666130932562)*x[0]+(-0.922839241546)*x[1]**o+(-0.143547846716)*x[1]
        s[3,4,1,0]=(-0.43295353947)*x[0]**o+(0.342981294189)*x[0]+(0.785780092257)*x[1]**o+(0.480176709076)*x[1]
        s[3,4,1,1]=(-0.859691385907)*x[0]**o+(0.124338939229)*x[0]+(-0.883988405006)*x[1]**o+(0.0683789905855)*x[1]
        s[3,4,2,0]=(-0.106494101633)*x[0]**o+(0.276437588849)*x[0]+(0.408900924262)*x[1]**o+(-0.479798281555)*x[1]
        s[3,4,2,1]=(-0.226633020661)*x[0]**o+(0.487277344856)*x[0]+(-0.0241726779927)*x[1]**o+(-0.778823212273)*x[1]
        ref[0,0,0,0]=(-0.474386706114)*x_ref[0]**o+(-0.0739442523179)*x_ref[0]+(-0.934839682165)*x_ref[1]**o+(-0.874748715361)*x_ref[1]
        ref[0,0,0,1]=(0.970835666471)*x_ref[0]**o+(-0.210304396563)*x_ref[0]+(-0.245577454075)*x_ref[1]**o+(0.706874293911)*x_ref[1]
        ref[0,0,1,0]=(-0.886508211386)*x_ref[0]**o+(0.40749471331)*x_ref[0]+(-0.575539262859)*x_ref[1]**o+(-0.714242182567)*x_ref[1]
        ref[0,0,1,1]=(0.0925249369772)*x_ref[0]**o+(-0.326000469119)*x_ref[0]+(0.427385578633)*x_ref[1]**o+(0.723913449012)*x_ref[1]
        ref[0,0,2,0]=(0.178301174425)*x_ref[0]**o+(-0.935408769107)*x_ref[0]+(0.185756314077)*x_ref[1]**o+(0.779639226265)*x_ref[1]
        ref[0,0,2,1]=(0.517627638569)*x_ref[0]**o+(0.833228507097)*x_ref[0]+(0.509792014643)*x_ref[1]**o+(0.512759085233)*x_ref[1]
        ref[0,1,0,0]=(0.943220804246)*x_ref[0]**o+(0.358695377859)*x_ref[0]+(0.483282522344)*x_ref[1]**o+(-0.780850766978)*x_ref[1]
        ref[0,1,0,1]=(0.228281969805)*x_ref[0]**o+(0.378112450475)*x_ref[0]+(-0.265569820367)*x_ref[1]**o+(0.793063241358)*x_ref[1]
        ref[0,1,1,0]=(-0.991239767457)*x_ref[0]**o+(0.584083573937)*x_ref[0]+(0.00907947673395)*x_ref[1]**o+(-0.0949125419342)*x_ref[1]
        ref[0,1,1,1]=(0.312976776903)*x_ref[0]**o+(-0.572058365439)*x_ref[0]+(0.337493203667)*x_ref[1]**o+(-0.559749276661)*x_ref[1]
        ref[0,1,2,0]=(-0.154473081077)*x_ref[0]**o+(0.946407875413)*x_ref[0]+(0.721108882799)*x_ref[1]**o+(0.409146005045)*x_ref[1]
        ref[0,1,2,1]=(0.42027715889)*x_ref[0]**o+(-0.892090996743)*x_ref[0]+(-0.718807145454)*x_ref[1]**o+(0.229975539898)*x_ref[1]
        ref[0,2,0,0]=(0.378117082397)*x_ref[0]**o+(-0.613916810385)*x_ref[0]+(-0.564348246694)*x_ref[1]**o+(0.101261637507)*x_ref[1]
        ref[0,2,0,1]=(0.839696159524)*x_ref[0]**o+(0.24184619932)*x_ref[0]+(-0.095338864129)*x_ref[1]**o+(0.903106275587)*x_ref[1]
        ref[0,2,1,0]=(0.620479230354)*x_ref[0]**o+(-0.34112105179)*x_ref[0]+(0.624902166524)*x_ref[1]**o+(-0.693289910121)*x_ref[1]
        ref[0,2,1,1]=(-0.39760464469)*x_ref[0]**o+(-0.921383269336)*x_ref[0]+(0.753657132321)*x_ref[1]**o+(0.324768021371)*x_ref[1]
        ref[0,2,2,0]=(0.0243534816004)*x_ref[0]**o+(0.290663121221)*x_ref[0]+(0.0305666628617)*x_ref[1]**o+(0.0446693318891)*x_ref[1]
        ref[0,2,2,1]=(-0.14154275886)*x_ref[0]**o+(0.881207887999)*x_ref[0]+(0.399029780289)*x_ref[1]**o+(0.168497381665)*x_ref[1]
        ref[0,3,0,0]=(-0.233073561791)*x_ref[0]**o+(-0.513212321666)*x_ref[0]+(-0.651480028725)*x_ref[1]**o+(0.147498573261)*x_ref[1]
        ref[0,3,0,1]=(-0.918136406734)*x_ref[0]**o+(-0.279161242114)*x_ref[0]+(0.983432543886)*x_ref[1]**o+(-0.723257302563)*x_ref[1]
        ref[0,3,1,0]=(0.290070399137)*x_ref[0]**o+(0.485832779323)*x_ref[0]+(0.955571801349)*x_ref[1]**o+(0.552993034163)*x_ref[1]
        ref[0,3,1,1]=(0.688570077213)*x_ref[0]**o+(0.578716466405)*x_ref[0]+(-0.40829418423)*x_ref[1]**o+(-0.918196317732)*x_ref[1]
        ref[0,3,2,0]=(0.403069347997)*x_ref[0]**o+(0.119390454784)*x_ref[0]+(-0.322018782913)*x_ref[1]**o+(0.224814772968)*x_ref[1]
        ref[0,3,2,1]=(-0.23084264112)*x_ref[0]**o+(-0.398235674194)*x_ref[0]+(-0.730556055687)*x_ref[1]**o+(0.293981158644)*x_ref[1]
        ref[0,4,0,0]=(-0.0984201873925)*x_ref[0]**o+(0.018359257988)*x_ref[0]+(-0.0781876875229)*x_ref[1]**o+(0.253344618889)*x_ref[1]
        ref[0,4,0,1]=(-0.20899724864)*x_ref[0]**o+(-0.722019204308)*x_ref[0]+(0.555162582715)*x_ref[1]**o+(-0.958068826429)*x_ref[1]
        ref[0,4,1,0]=(0.477725976915)*x_ref[0]**o+(0.295872229679)*x_ref[0]+(-0.828208555398)*x_ref[1]**o+(-0.931582818519)*x_ref[1]
        ref[0,4,1,1]=(0.721285453077)*x_ref[0]**o+(-0.042927949943)*x_ref[0]+(0.680231405034)*x_ref[1]**o+(-0.16043805248)*x_ref[1]
        ref[0,4,2,0]=(0.985988751136)*x_ref[0]**o+(0.885785903078)*x_ref[0]+(-0.0410955856477)*x_ref[1]**o+(-0.552089008201)*x_ref[1]
        ref[0,4,2,1]=(0.847466818225)*x_ref[0]**o+(-0.51057282448)*x_ref[0]+(0.163775791256)*x_ref[1]**o+(0.0152553094575)*x_ref[1]
        ref[1,0,0,0]=(0.0343855486006)*x_ref[0]**o+(-0.214334464898)*x_ref[0]+(-0.573344070879)*x_ref[1]**o+(0.24905956136)*x_ref[1]
        ref[1,0,0,1]=(0.783338978316)*x_ref[0]**o+(0.781324842059)*x_ref[0]+(-0.847509450777)*x_ref[1]**o+(-0.858994488444)*x_ref[1]
        ref[1,0,1,0]=(0.59719044885)*x_ref[0]**o+(0.347038111035)*x_ref[0]+(-0.385761256521)*x_ref[1]**o+(-0.571166989318)*x_ref[1]
        ref[1,0,1,1]=(0.636911008507)*x_ref[0]**o+(-0.825376916398)*x_ref[0]+(0.32394534216)*x_ref[1]**o+(0.653119502004)*x_ref[1]
        ref[1,0,2,0]=(-0.669732532783)*x_ref[0]**o+(-0.145427031253)*x_ref[0]+(-0.303693857831)*x_ref[1]**o+(-0.727878524833)*x_ref[1]
        ref[1,0,2,1]=(0.621061585764)*x_ref[0]**o+(0.243432217146)*x_ref[0]+(-0.888610099037)*x_ref[1]**o+(0.298051589144)*x_ref[1]
        ref[1,1,0,0]=(0.357341721813)*x_ref[0]**o+(0.346506498237)*x_ref[0]+(-0.555625437961)*x_ref[1]**o+(-0.0662579456147)*x_ref[1]
        ref[1,1,0,1]=(0.96955420495)*x_ref[0]**o+(-0.190486134786)*x_ref[0]+(-0.900124677941)*x_ref[1]**o+(0.659993963277)*x_ref[1]
        ref[1,1,1,0]=(-0.517837108406)*x_ref[0]**o+(0.00779665389526)*x_ref[0]+(-0.0544022169324)*x_ref[1]**o+(0.888217855388)*x_ref[1]
        ref[1,1,1,1]=(0.252515646862)*x_ref[0]**o+(0.328948655235)*x_ref[0]+(0.265916563145)*x_ref[1]**o+(-0.491865215095)*x_ref[1]
        ref[1,1,2,0]=(0.922520219928)*x_ref[0]**o+(0.463811222486)*x_ref[0]+(0.651244201962)*x_ref[1]**o+(0.982565677231)*x_ref[1]
        ref[1,1,2,1]=(0.257711278265)*x_ref[0]**o+(0.23971381197)*x_ref[0]+(-0.168559977915)*x_ref[1]**o+(0.453909616568)*x_ref[1]
        ref[1,2,0,0]=(0.0426933948917)*x_ref[0]**o+(-0.477381944712)*x_ref[0]+(0.503797983762)*x_ref[1]**o+(-0.943092701839)*x_ref[1]
        ref[1,2,0,1]=(0.913591544613)*x_ref[0]**o+(-0.895063818412)*x_ref[0]+(0.252264576564)*x_ref[1]**o+(0.424846745491)*x_ref[1]
        ref[1,2,1,0]=(0.4199934247)*x_ref[0]**o+(-0.128668379749)*x_ref[0]+(-0.157997092493)*x_ref[1]**o+(-0.174476338072)*x_ref[1]
        ref[1,2,1,1]=(0.394310091468)*x_ref[0]**o+(0.599539798636)*x_ref[0]+(0.133087615453)*x_ref[1]**o+(-0.22271509754)*x_ref[1]
        ref[1,2,2,0]=(0.559521330007)*x_ref[0]**o+(-0.411300693025)*x_ref[0]+(-0.543921885179)*x_ref[1]**o+(-0.0506834937614)*x_ref[1]
        ref[1,2,2,1]=(-0.423996886775)*x_ref[0]**o+(0.14409349326)*x_ref[0]+(-0.970632608145)*x_ref[1]**o+(0.582937355294)*x_ref[1]
        ref[1,3,0,0]=(-0.884972026022)*x_ref[0]**o+(-0.872260393893)*x_ref[0]+(0.224463013859)*x_ref[1]**o+(-0.245013144519)*x_ref[1]
        ref[1,3,0,1]=(-0.834873836296)*x_ref[0]**o+(-0.778611513205)*x_ref[0]+(0.989113281842)*x_ref[1]**o+(0.847047912646)*x_ref[1]
        ref[1,3,1,0]=(-0.0900703847239)*x_ref[0]**o+(-0.0295919916877)*x_ref[0]+(0.510380741505)*x_ref[1]**o+(-0.00354494905936)*x_ref[1]
        ref[1,3,1,1]=(0.0422103052661)*x_ref[0]**o+(0.31376324435)*x_ref[0]+(0.831890412519)*x_ref[1]**o+(0.764406560706)*x_ref[1]
        ref[1,3,2,0]=(0.496699862489)*x_ref[0]**o+(0.477514051712)*x_ref[0]+(0.156435695163)*x_ref[1]**o+(0.339443828388)*x_ref[1]
        ref[1,3,2,1]=(0.396689832399)*x_ref[0]**o+(0.243605505552)*x_ref[0]+(-0.0357728699149)*x_ref[1]**o+(0.930734881576)*x_ref[1]
        ref[1,4,0,0]=(-0.90788627878)*x_ref[0]**o+(0.82597715373)*x_ref[0]+(0.287988132693)*x_ref[1]**o+(-0.834307302134)*x_ref[1]
        ref[1,4,0,1]=(-0.942882799669)*x_ref[0]**o+(-0.149609905045)*x_ref[0]+(-0.340025012164)*x_ref[1]**o+(0.539546162654)*x_ref[1]
        ref[1,4,1,0]=(-0.244377326727)*x_ref[0]**o+(-0.586439384775)*x_ref[0]+(0.89506494604)*x_ref[1]**o+(0.233226803834)*x_ref[1]
        ref[1,4,1,1]=(0.864582224514)*x_ref[0]**o+(0.0687754700765)*x_ref[0]+(-0.641556536115)*x_ref[1]**o+(0.22036806322)*x_ref[1]
        ref[1,4,2,0]=(-0.552589117461)*x_ref[0]**o+(-0.705244093999)*x_ref[0]+(0.378837921567)*x_ref[1]**o+(-0.173968977986)*x_ref[1]
        ref[1,4,2,1]=(0.846900000181)*x_ref[0]**o+(-0.52589472084)*x_ref[0]+(0.872400573242)*x_ref[1]**o+(0.182683546911)*x_ref[1]
        ref[2,0,0,0]=(0.305006901177)*x_ref[0]**o+(-0.732858762152)*x_ref[0]+(-0.882239786812)*x_ref[1]**o+(0.481712295538)*x_ref[1]
        ref[2,0,0,1]=(0.836332373747)*x_ref[0]**o+(0.63905574965)*x_ref[0]+(0.039226709213)*x_ref[1]**o+(0.704758586936)*x_ref[1]
        ref[2,0,1,0]=(0.51805564252)*x_ref[0]**o+(0.641883965761)*x_ref[0]+(0.901663292208)*x_ref[1]**o+(0.375272715842)*x_ref[1]
        ref[2,0,1,1]=(-0.980058913052)*x_ref[0]**o+(0.385619663167)*x_ref[0]+(0.0722192483217)*x_ref[1]**o+(0.00650731529315)*x_ref[1]
        ref[2,0,2,0]=(-0.25791454536)*x_ref[0]**o+(0.460015385804)*x_ref[0]+(0.234970544559)*x_ref[1]**o+(-0.11962323176)*x_ref[1]
        ref[2,0,2,1]=(0.023390322577)*x_ref[0]**o+(-0.104449150638)*x_ref[0]+(0.149724956284)*x_ref[1]**o+(-0.309848596985)*x_ref[1]
        ref[2,1,0,0]=(0.603565629307)*x_ref[0]**o+(-0.143206239367)*x_ref[0]+(-0.0430666954645)*x_ref[1]**o+(-0.154546675756)*x_ref[1]
        ref[2,1,0,1]=(-0.980112660961)*x_ref[0]**o+(0.419145835585)*x_ref[0]+(-0.76202669209)*x_ref[1]**o+(-0.648615698697)*x_ref[1]
        ref[2,1,1,0]=(0.805964945679)*x_ref[0]**o+(0.650144650504)*x_ref[0]+(0.308110033125)*x_ref[1]**o+(0.0666476104838)*x_ref[1]
        ref[2,1,1,1]=(0.42561398203)*x_ref[0]**o+(0.259276259654)*x_ref[0]+(-0.215194398301)*x_ref[1]**o+(0.880282506885)*x_ref[1]
        ref[2,1,2,0]=(-0.413779899229)*x_ref[0]**o+(-0.796859182258)*x_ref[0]+(0.606865226329)*x_ref[1]**o+(-0.783313517188)*x_ref[1]
        ref[2,1,2,1]=(0.402767439742)*x_ref[0]**o+(-0.692468937875)*x_ref[0]+(0.27866251803)*x_ref[1]**o+(0.95092559773)*x_ref[1]
        ref[2,2,0,0]=(0.749107532378)*x_ref[0]**o+(-0.364633672479)*x_ref[0]+(0.970710337513)*x_ref[1]**o+(0.1986749977)*x_ref[1]
        ref[2,2,0,1]=(-0.722832876805)*x_ref[0]**o+(-0.238825576333)*x_ref[0]+(-0.101752751414)*x_ref[1]**o+(-0.960067435044)*x_ref[1]
        ref[2,2,1,0]=(-0.704825996403)*x_ref[0]**o+(0.74244530617)*x_ref[0]+(-0.586571188279)*x_ref[1]**o+(0.841297962355)*x_ref[1]
        ref[2,2,1,1]=(0.586127464995)*x_ref[0]**o+(0.454050196541)*x_ref[0]+(0.225294694159)*x_ref[1]**o+(0.705925824031)*x_ref[1]
        ref[2,2,2,0]=(0.535446366333)*x_ref[0]**o+(-0.787565531495)*x_ref[0]+(-0.566053725536)*x_ref[1]**o+(0.468968266813)*x_ref[1]
        ref[2,2,2,1]=(0.449236149776)*x_ref[0]**o+(0.589799099557)*x_ref[0]+(0.189540632578)*x_ref[1]**o+(-0.775036139482)*x_ref[1]
        ref[2,3,0,0]=(0.979572074948)*x_ref[0]**o+(-0.806420343846)*x_ref[0]+(0.661682487968)*x_ref[1]**o+(-0.615846811222)*x_ref[1]
        ref[2,3,0,1]=(0.021072244205)*x_ref[0]**o+(0.878344628134)*x_ref[0]+(-0.381067816502)*x_ref[1]**o+(0.229027446937)*x_ref[1]
        ref[2,3,1,0]=(0.0182291979593)*x_ref[0]**o+(0.625595584968)*x_ref[0]+(-0.824161018488)*x_ref[1]**o+(-0.316191235785)*x_ref[1]
        ref[2,3,1,1]=(-0.0168799900253)*x_ref[0]**o+(-0.137230078645)*x_ref[0]+(0.0240280311645)*x_ref[1]**o+(-0.364099905801)*x_ref[1]
        ref[2,3,2,0]=(0.374193672582)*x_ref[0]**o+(0.429814892173)*x_ref[0]+(0.0194417431204)*x_ref[1]**o+(0.830453174612)*x_ref[1]
        ref[2,3,2,1]=(-0.683758441576)*x_ref[0]**o+(-0.759635036652)*x_ref[0]+(-0.50102074202)*x_ref[1]**o+(0.123764523628)*x_ref[1]
        ref[2,4,0,0]=(-0.0817296101008)*x_ref[0]**o+(0.730474427452)*x_ref[0]+(-0.0733544873209)*x_ref[1]**o+(0.416303527556)*x_ref[1]
        ref[2,4,0,1]=(0.258598296267)*x_ref[0]**o+(0.686740404407)*x_ref[0]+(0.543414259362)*x_ref[1]**o+(0.518071987904)*x_ref[1]
        ref[2,4,1,0]=(-0.32475099592)*x_ref[0]**o+(-0.0460901003729)*x_ref[0]+(0.275611466137)*x_ref[1]**o+(0.8415303665)*x_ref[1]
        ref[2,4,1,1]=(0.5015310862)*x_ref[0]**o+(-0.71451126551)*x_ref[0]+(0.897703909295)*x_ref[1]**o+(-0.0476780709524)*x_ref[1]
        ref[2,4,2,0]=(0.612722779576)*x_ref[0]**o+(0.424485459817)*x_ref[0]+(-0.136026922168)*x_ref[1]**o+(0.16727850944)*x_ref[1]
        ref[2,4,2,1]=(-0.841159017194)*x_ref[0]**o+(0.856949657441)*x_ref[0]+(-0.975786438976)*x_ref[1]**o+(-0.407229683889)*x_ref[1]
        ref[3,0,0,0]=(0.833459943775)*x_ref[0]**o+(-0.230178834842)*x_ref[0]+(-0.660801597291)*x_ref[1]**o+(-0.738869823127)*x_ref[1]
        ref[3,0,0,1]=(0.30987970183)*x_ref[0]**o+(-0.610707485471)*x_ref[0]+(0.628798231358)*x_ref[1]**o+(-0.0491116840572)*x_ref[1]
        ref[3,0,1,0]=(0.649739974341)*x_ref[0]**o+(-0.450666468651)*x_ref[0]+(0.901638992166)*x_ref[1]**o+(-0.787549067401)*x_ref[1]
        ref[3,0,1,1]=(0.934583955853)*x_ref[0]**o+(-0.859086605977)*x_ref[0]+(0.0975143570756)*x_ref[1]**o+(0.0350097808797)*x_ref[1]
        ref[3,0,2,0]=(0.837723849965)*x_ref[0]**o+(-0.665696123522)*x_ref[0]+(-0.70629678851)*x_ref[1]**o+(-0.468945853667)*x_ref[1]
        ref[3,0,2,1]=(0.526302645633)*x_ref[0]**o+(-0.0979568077223)*x_ref[0]+(-0.767623484018)*x_ref[1]**o+(-0.21912588217)*x_ref[1]
        ref[3,1,0,0]=(0.18832505785)*x_ref[0]**o+(0.45004009293)*x_ref[0]+(-0.823702397711)*x_ref[1]**o+(-0.912642057295)*x_ref[1]
        ref[3,1,0,1]=(0.554483721388)*x_ref[0]**o+(-0.607925140409)*x_ref[0]+(-0.998308584629)*x_ref[1]**o+(-0.538184347898)*x_ref[1]
        ref[3,1,1,0]=(-0.682363035722)*x_ref[0]**o+(0.55318030401)*x_ref[0]+(0.562408350319)*x_ref[1]**o+(0.220145757421)*x_ref[1]
        ref[3,1,1,1]=(0.924816884714)*x_ref[0]**o+(-0.295767203325)*x_ref[0]+(0.230343937118)*x_ref[1]**o+(0.957747143338)*x_ref[1]
        ref[3,1,2,0]=(0.235422171194)*x_ref[0]**o+(0.264750603603)*x_ref[0]+(0.13053017764)*x_ref[1]**o+(0.628805980195)*x_ref[1]
        ref[3,1,2,1]=(-0.603111233811)*x_ref[0]**o+(-0.325257245906)*x_ref[0]+(-0.879402225173)*x_ref[1]**o+(0.977736365104)*x_ref[1]
        ref[3,2,0,0]=(0.463242222753)*x_ref[0]**o+(-0.802230749276)*x_ref[0]+(0.085387156258)*x_ref[1]**o+(0.18309223952)*x_ref[1]
        ref[3,2,0,1]=(0.778060805693)*x_ref[0]**o+(0.32136550684)*x_ref[0]+(0.775284717696)*x_ref[1]**o+(0.151449430741)*x_ref[1]
        ref[3,2,1,0]=(0.288473735099)*x_ref[0]**o+(0.675142038399)*x_ref[0]+(-0.313098988069)*x_ref[1]**o+(-0.771883975974)*x_ref[1]
        ref[3,2,1,1]=(0.715304130316)*x_ref[0]**o+(0.994375393616)*x_ref[0]+(-0.874667784061)*x_ref[1]**o+(-0.29510377129)*x_ref[1]
        ref[3,2,2,0]=(-0.211496384269)*x_ref[0]**o+(0.146746732763)*x_ref[0]+(0.234317983556)*x_ref[1]**o+(0.844710015939)*x_ref[1]
        ref[3,2,2,1]=(0.557426563891)*x_ref[0]**o+(0.181380049852)*x_ref[0]+(-0.068830884747)*x_ref[1]**o+(0.424778582562)*x_ref[1]
        ref[3,3,0,0]=(-0.784389821334)*x_ref[0]**o+(0.533073578323)*x_ref[0]+(-0.172622226211)*x_ref[1]**o+(-0.0882056556263)*x_ref[1]
        ref[3,3,0,1]=(-0.369034971239)*x_ref[0]**o+(-0.323409858793)*x_ref[0]+(0.408766889438)*x_ref[1]**o+(-0.653470515807)*x_ref[1]
        ref[3,3,1,0]=(0.239411265848)*x_ref[0]**o+(-0.672719205684)*x_ref[0]+(-0.751715345243)*x_ref[1]**o+(-0.4586462848)*x_ref[1]
        ref[3,3,1,1]=(-0.937724235502)*x_ref[0]**o+(-0.9284117207)*x_ref[0]+(-0.584507468424)*x_ref[1]**o+(0.779562243368)*x_ref[1]
        ref[3,3,2,0]=(-0.563406066207)*x_ref[0]**o+(0.115504525413)*x_ref[0]+(0.0230187346201)*x_ref[1]**o+(0.960113641875)*x_ref[1]
        ref[3,3,2,1]=(0.792686350755)*x_ref[0]**o+(0.560920910518)*x_ref[0]+(0.0766597516323)*x_ref[1]**o+(-0.728345800422)*x_ref[1]
        ref[3,4,0,0]=(-0.182383283107)*x_ref[0]**o+(0.604818623768)*x_ref[0]+(0.756868334363)*x_ref[1]**o+(0.0259669270239)*x_ref[1]
        ref[3,4,0,1]=(0.547592534695)*x_ref[0]**o+(0.666130932562)*x_ref[0]+(-0.922839241546)*x_ref[1]**o+(-0.143547846716)*x_ref[1]
        ref[3,4,1,0]=(-0.43295353947)*x_ref[0]**o+(0.342981294189)*x_ref[0]+(0.785780092257)*x_ref[1]**o+(0.480176709076)*x_ref[1]
        ref[3,4,1,1]=(-0.859691385907)*x_ref[0]**o+(0.124338939229)*x_ref[0]+(-0.883988405006)*x_ref[1]**o+(0.0683789905855)*x_ref[1]
        ref[3,4,2,0]=(-0.106494101633)*x_ref[0]**o+(0.276437588849)*x_ref[0]+(0.408900924262)*x_ref[1]**o+(-0.479798281555)*x_ref[1]
        ref[3,4,2,1]=(-0.226633020661)*x_ref[0]**o+(0.487277344856)*x_ref[0]+(-0.0241726779927)*x_ref[1]**o+(-0.778823212273)*x_ref[1]
      else:
        s[0,0,0,0]=(-0.400665270553)*x[0]**o+(-0.465656794893)*x[0]+(-0.103120094839)*x[1]**o+(0.0464026120794)*x[1]+(0.030596268742)*x[2]**o+(0.00334628754684)*x[2]
        s[0,0,0,1]=(0.937480902128)*x[0]**o+(0.0466658012652)*x[0]+(0.817801316276)*x[1]**o+(-0.142948501497)*x[1]+(-0.946943305156)*x[2]**o+(-0.633505380306)*x[2]
        s[0,0,1,0]=(-0.441314521722)*x[0]**o+(-0.42022483938)*x[0]+(-0.833228022905)*x[1]**o+(-0.225426650714)*x[1]+(0.332718715913)*x[2]**o+(0.123261640846)*x[2]
        s[0,0,1,1]=(0.335875498268)*x[0]**o+(0.983700594491)*x[0]+(-0.214325088125)*x[1]**o+(-0.893543563881)*x[1]+(0.0631361789468)*x[2]**o+(0.4467209081)*x[2]
        s[0,0,2,0]=(0.658215366848)*x[0]**o+(-0.672295829784)*x[0]+(-0.0137993447332)*x[1]**o+(0.698632148806)*x[1]+(-0.221572355182)*x[2]**o+(0.956096660679)*x[2]
        s[0,0,2,1]=(-0.578795938537)*x[0]**o+(0.750684927067)*x[0]+(0.333363761953)*x[1]**o+(-0.617804591638)*x[1]+(-0.0469494394831)*x[2]**o+(0.16187318118)*x[2]
        s[0,1,0,0]=(-0.0638097634115)*x[0]**o+(-0.38507312448)*x[0]+(-0.663736729005)*x[1]**o+(-0.989965410511)*x[1]+(0.230748674134)*x[2]**o+(-0.830580398633)*x[2]
        s[0,1,0,1]=(-0.658956286059)*x[0]**o+(0.345180478933)*x[0]+(-0.544808816473)*x[1]**o+(0.799798879144)*x[1]+(-0.862695252289)*x[2]**o+(0.757134401664)*x[2]
        s[0,1,1,0]=(0.456144819828)*x[0]**o+(0.969408065204)*x[0]+(0.488759322896)*x[1]**o+(0.191259701658)*x[1]+(-0.211530905801)*x[2]**o+(0.0976530925065)*x[2]
        s[0,1,1,1]=(-0.160993701908)*x[0]**o+(-0.768084723399)*x[0]+(0.200776153747)*x[1]**o+(0.380923444552)*x[1]+(0.152388183214)*x[2]**o+(0.0345495856869)*x[2]
        s[0,1,2,0]=(-0.249731094314)*x[0]**o+(0.420921348364)*x[0]+(-0.745813017956)*x[1]**o+(-0.128319964472)*x[1]+(-0.134590180255)*x[2]**o+(-0.598364990007)*x[2]
        s[0,1,2,1]=(0.608497167751)*x[0]**o+(0.134066300157)*x[0]+(-0.219556933024)*x[1]**o+(0.845095947325)*x[1]+(-0.0910112208065)*x[2]**o+(-0.919387374125)*x[2]
        s[0,2,0,0]=(-0.603722566149)*x[0]**o+(-0.141049572403)*x[0]+(-0.433072627843)*x[1]**o+(-0.266640930301)*x[1]+(-0.0520863163599)*x[2]**o+(0.113826044084)*x[2]
        s[0,2,0,1]=(0.735600439472)*x[0]**o+(-0.683666203432)*x[0]+(0.439527311107)*x[1]**o+(0.167685906293)*x[1]+(-0.526256697628)*x[2]**o+(-0.152028404784)*x[2]
        s[0,2,1,0]=(-0.484268475407)*x[0]**o+(-0.1302369788)*x[0]+(0.0996963541291)*x[1]**o+(0.525432686771)*x[1]+(0.915056396412)*x[2]**o+(-0.93142015197)*x[2]
        s[0,2,1,1]=(0.103016217069)*x[0]**o+(0.00517836018812)*x[0]+(-0.28331822161)*x[1]**o+(0.413800832478)*x[1]+(0.94613674069)*x[2]**o+(-0.287932854599)*x[2]
        s[0,2,2,0]=(0.512958587386)*x[0]**o+(-0.851145503923)*x[0]+(-0.535377708818)*x[1]**o+(-0.379448444669)*x[1]+(-0.238808818844)*x[2]**o+(0.567146071008)*x[2]
        s[0,2,2,1]=(-0.141490825422)*x[0]**o+(0.133948738844)*x[0]+(0.336733111602)*x[1]**o+(-0.223425201179)*x[1]+(-0.646454138519)*x[2]**o+(0.823936105317)*x[2]
        s[0,3,0,0]=(-0.938725695354)*x[0]**o+(0.988131864396)*x[0]+(-0.82501934096)*x[1]**o+(-0.257156351526)*x[1]+(-0.00349993655072)*x[2]**o+(0.580041226419)*x[2]
        s[0,3,0,1]=(0.814169577155)*x[0]**o+(-0.686109074376)*x[0]+(-0.965459022554)*x[1]**o+(0.10941355421)*x[1]+(0.609050101353)*x[2]**o+(-0.965588393623)*x[2]
        s[0,3,1,0]=(-0.820571213003)*x[0]**o+(0.746749194774)*x[0]+(0.710345174116)*x[1]**o+(-0.892921665205)*x[1]+(0.4004855541)*x[2]**o+(-0.695117880583)*x[2]
        s[0,3,1,1]=(-0.122273772977)*x[0]**o+(0.570499591543)*x[0]+(0.795058034707)*x[1]**o+(0.925412519293)*x[1]+(-0.0932686873001)*x[2]**o+(-0.686679792186)*x[2]
        s[0,3,2,0]=(-0.0042793185541)*x[0]**o+(-0.821480764508)*x[0]+(0.875015113726)*x[1]**o+(0.649383518184)*x[1]+(-0.288257569005)*x[2]**o+(-0.743941827161)*x[2]
        s[0,3,2,1]=(0.974182567966)*x[0]**o+(0.733657486071)*x[0]+(0.780035494556)*x[1]**o+(0.749708949611)*x[1]+(-0.521673379431)*x[2]**o+(0.673844867694)*x[2]
        s[0,4,0,0]=(-0.150057729107)*x[0]**o+(0.102611114471)*x[0]+(-0.0633864241155)*x[1]**o+(-0.329289187178)*x[1]+(0.315706711936)*x[2]**o+(-0.554443679394)*x[2]
        s[0,4,0,1]=(0.564883543338)*x[0]**o+(0.535008108501)*x[0]+(0.815233609718)*x[1]**o+(-0.766536697052)*x[1]+(0.997595942191)*x[2]**o+(0.991881374395)*x[2]
        s[0,4,1,0]=(-0.725748156838)*x[0]**o+(-0.817693342658)*x[0]+(0.501105656184)*x[1]**o+(-0.331771590595)*x[1]+(-0.757619653898)*x[2]**o+(-0.885657271086)*x[2]
        s[0,4,1,1]=(-0.559814749411)*x[0]**o+(0.665634435712)*x[0]+(0.787002371365)*x[1]**o+(-0.762034037705)*x[1]+(0.508112385178)*x[2]**o+(-0.985681857439)*x[2]
        s[0,4,2,0]=(0.504217223699)*x[0]**o+(-0.361750429828)*x[0]+(0.19299099109)*x[1]**o+(0.338521927796)*x[1]+(0.549199682759)*x[2]**o+(0.891662590379)*x[2]
        s[0,4,2,1]=(-0.251870804657)*x[0]**o+(0.776378068034)*x[0]+(-0.769493074808)*x[1]**o+(0.929429944731)*x[1]+(0.679845951817)*x[2]**o+(0.261179050197)*x[2]
        s[1,0,0,0]=(0.339823121204)*x[0]**o+(0.212075564821)*x[0]+(0.103901436655)*x[1]**o+(0.887698805047)*x[1]+(-0.467697404092)*x[2]**o+(0.71153584324)*x[2]
        s[1,0,0,1]=(0.610147759489)*x[0]**o+(-0.820866072636)*x[0]+(0.505294661936)*x[1]**o+(-0.206382410595)*x[1]+(0.343889298437)*x[2]**o+(0.165680393811)*x[2]
        s[1,0,1,0]=(-0.155342751495)*x[0]**o+(-0.638727698114)*x[0]+(-0.245328510129)*x[1]**o+(0.816689513534)*x[1]+(0.138847830114)*x[2]**o+(0.286577642911)*x[2]
        s[1,0,1,1]=(0.804058022285)*x[0]**o+(-0.922679564915)*x[0]+(-0.755679292664)*x[1]**o+(-0.409251039141)*x[1]+(-0.697869565514)*x[2]**o+(-0.0749517480557)*x[2]
        s[1,0,2,0]=(-0.345641262813)*x[0]**o+(0.31066537094)*x[0]+(0.165733093512)*x[1]**o+(-0.733996040753)*x[1]+(0.240017268146)*x[2]**o+(0.981970338884)*x[2]
        s[1,0,2,1]=(-0.0394605078977)*x[0]**o+(-0.671255389736)*x[0]+(-0.58457671038)*x[1]**o+(-0.308411795131)*x[1]+(0.944299000242)*x[2]**o+(0.0274884386004)*x[2]
        s[1,1,0,0]=(0.285078336337)*x[0]**o+(-0.29609479737)*x[0]+(0.851762966843)*x[1]**o+(-0.56978612625)*x[1]+(-0.416902609766)*x[2]**o+(-0.209352333275)*x[2]
        s[1,1,0,1]=(0.165305243827)*x[0]**o+(0.569657682766)*x[0]+(-0.158936758165)*x[1]**o+(-0.826897725956)*x[1]+(-0.906153311902)*x[2]**o+(-0.508208690461)*x[2]
        s[1,1,1,0]=(-0.168856536186)*x[0]**o+(0.600931452022)*x[0]+(0.515269032144)*x[1]**o+(-0.615629628046)*x[1]+(-0.248016004788)*x[2]**o+(-0.236340748995)*x[2]
        s[1,1,1,1]=(-0.265342410313)*x[0]**o+(0.546304098539)*x[0]+(0.44451795278)*x[1]**o+(0.53863749512)*x[1]+(-0.400353954694)*x[2]**o+(0.762091735577)*x[2]
        s[1,1,2,0]=(-0.828507588085)*x[0]**o+(0.310151266525)*x[0]+(0.767699919156)*x[1]**o+(-0.802526284756)*x[1]+(-0.12986443685)*x[2]**o+(0.555810425336)*x[2]
        s[1,1,2,1]=(-0.0843531370201)*x[0]**o+(0.138067935996)*x[0]+(-0.819663672158)*x[1]**o+(0.227177408663)*x[1]+(-0.481488807321)*x[2]**o+(0.277780608851)*x[2]
        s[1,2,0,0]=(0.843912054965)*x[0]**o+(-0.532942686099)*x[0]+(-0.12263392247)*x[1]**o+(-0.918649820864)*x[1]+(-0.898969788765)*x[2]**o+(0.401718907796)*x[2]
        s[1,2,0,1]=(0.500181344385)*x[0]**o+(-0.695081419586)*x[0]+(0.49127001192)*x[1]**o+(0.424707942699)*x[1]+(-0.170525943009)*x[2]**o+(-0.908857558417)*x[2]
        s[1,2,1,0]=(0.374108805296)*x[0]**o+(-0.778311131014)*x[0]+(-0.619173488016)*x[1]**o+(-0.0906082150795)*x[1]+(0.312783760308)*x[2]**o+(0.50073883273)*x[2]
        s[1,2,1,1]=(0.474622980263)*x[0]**o+(0.00968769992516)*x[0]+(-0.246566756628)*x[1]**o+(0.868983907653)*x[1]+(-0.318924531295)*x[2]**o+(-0.340314263979)*x[2]
        s[1,2,2,0]=(0.621111695462)*x[0]**o+(0.655354521723)*x[0]+(-0.992404590253)*x[1]**o+(-0.497860913536)*x[1]+(-0.963211421906)*x[2]**o+(0.493347857543)*x[2]
        s[1,2,2,1]=(-0.32035342079)*x[0]**o+(0.351692950156)*x[0]+(-0.715457409869)*x[1]**o+(0.317941180953)*x[1]+(0.831453341583)*x[2]**o+(0.330720124015)*x[2]
        s[1,3,0,0]=(-0.775605593731)*x[0]**o+(-0.985544306967)*x[0]+(0.838313790256)*x[1]**o+(-0.0515692297878)*x[1]+(0.6757488996)*x[2]**o+(-0.398401163936)*x[2]
        s[1,3,0,1]=(0.129462614525)*x[0]**o+(-0.225290874002)*x[0]+(0.1627137973)*x[1]**o+(0.96490154212)*x[1]+(-0.0348196505137)*x[2]**o+(0.00501682452837)*x[2]
        s[1,3,1,0]=(-0.774906442617)*x[0]**o+(0.554579514554)*x[0]+(0.68683898073)*x[1]**o+(0.687422147497)*x[1]+(0.812740760401)*x[2]**o+(0.650574541761)*x[2]
        s[1,3,1,1]=(0.371414654481)*x[0]**o+(-0.460895148602)*x[0]+(-0.60035497386)*x[1]**o+(0.382490052311)*x[1]+(0.348365164241)*x[2]**o+(0.700564991456)*x[2]
        s[1,3,2,0]=(0.0627821130997)*x[0]**o+(-0.108341459129)*x[0]+(-0.459931858236)*x[1]**o+(-0.84999694423)*x[1]+(-0.369143500965)*x[2]**o+(-0.204962250898)*x[2]
        s[1,3,2,1]=(0.940231653623)*x[0]**o+(0.432366518466)*x[0]+(0.965573462127)*x[1]**o+(-0.400536850676)*x[1]+(0.627651239821)*x[2]**o+(0.541970384011)*x[2]
        s[1,4,0,0]=(-0.905529267496)*x[0]**o+(-0.728637228667)*x[0]+(0.528565304374)*x[1]**o+(-0.0791953805449)*x[1]+(0.278340538308)*x[2]**o+(0.0829173042009)*x[2]
        s[1,4,0,1]=(0.196476386515)*x[0]**o+(0.960964133173)*x[0]+(-0.975854821639)*x[1]**o+(-0.231600097119)*x[1]+(0.462289007605)*x[2]**o+(-0.41548631733)*x[2]
        s[1,4,1,0]=(0.133593658437)*x[0]**o+(0.27713134658)*x[0]+(-0.626190133661)*x[1]**o+(0.22128472818)*x[1]+(-0.0310045745464)*x[2]**o+(-0.308952151364)*x[2]
        s[1,4,1,1]=(-0.540782388402)*x[0]**o+(0.782610613508)*x[0]+(-0.121818568845)*x[1]**o+(-0.481514588156)*x[1]+(0.92789834048)*x[2]**o+(-0.053669614999)*x[2]
        s[1,4,2,0]=(0.228180064246)*x[0]**o+(-0.387791183012)*x[0]+(-0.809517349857)*x[1]**o+(0.104233744095)*x[1]+(-0.010561196572)*x[2]**o+(0.812403786777)*x[2]
        s[1,4,2,1]=(-0.213527688719)*x[0]**o+(0.444377334059)*x[0]+(0.250474600963)*x[1]**o+(-0.628454953053)*x[1]+(-0.149400612708)*x[2]**o+(-0.362500181895)*x[2]
        s[2,0,0,0]=(0.424970215816)*x[0]**o+(-0.409634055175)*x[0]+(-0.209831967682)*x[1]**o+(0.777759135142)*x[1]+(0.904286801356)*x[2]**o+(-0.533118859622)*x[2]
        s[2,0,0,1]=(-0.0634924164631)*x[0]**o+(0.61665557894)*x[0]+(0.886000765027)*x[1]**o+(-0.283672331088)*x[1]+(-0.104923941729)*x[2]**o+(0.647966459575)*x[2]
        s[2,0,1,0]=(-0.601180947882)*x[0]**o+(-0.853176409106)*x[0]+(0.589049472252)*x[1]**o+(0.947188905512)*x[1]+(0.937016925257)*x[2]**o+(-0.528933622785)*x[2]
        s[2,0,1,1]=(-0.493108989214)*x[0]**o+(0.123452031172)*x[0]+(0.579015108934)*x[1]**o+(-0.224656610786)*x[1]+(-0.351966996224)*x[2]**o+(-0.58667858467)*x[2]
        s[2,0,2,0]=(0.342923581511)*x[0]**o+(-0.787209375031)*x[0]+(0.0689438410554)*x[1]**o+(0.624797184447)*x[1]+(0.508789531503)*x[2]**o+(0.17151319396)*x[2]
        s[2,0,2,1]=(-0.875568056219)*x[0]**o+(-0.113031987088)*x[0]+(-0.0609746766081)*x[1]**o+(0.196111602161)*x[1]+(-0.00186601407674)*x[2]**o+(-0.695244625545)*x[2]
        s[2,1,0,0]=(0.0760124938281)*x[0]**o+(0.878269236128)*x[0]+(0.424404663038)*x[1]**o+(0.382191251895)*x[1]+(-0.799519816473)*x[2]**o+(0.756150518781)*x[2]
        s[2,1,0,1]=(-0.620026715848)*x[0]**o+(0.120584354877)*x[0]+(0.794571749541)*x[1]**o+(0.775886892396)*x[1]+(-0.455364446354)*x[2]**o+(-0.25890753368)*x[2]
        s[2,1,1,0]=(-0.434917779388)*x[0]**o+(0.893805701213)*x[0]+(-0.359966318659)*x[1]**o+(-0.92549769048)*x[1]+(0.632194530492)*x[2]**o+(0.783055504124)*x[2]
        s[2,1,1,1]=(-0.30435434794)*x[0]**o+(-0.132800060169)*x[0]+(0.200039404417)*x[1]**o+(-0.690955124262)*x[1]+(-0.984472889901)*x[2]**o+(0.0242811209734)*x[2]
        s[2,1,2,0]=(-0.223815583384)*x[0]**o+(0.592762849244)*x[0]+(0.281436806248)*x[1]**o+(0.822974431302)*x[1]+(-0.0648638855118)*x[2]**o+(-0.798035298439)*x[2]
        s[2,1,2,1]=(-0.847339863019)*x[0]**o+(0.603483610026)*x[0]+(0.865918681346)*x[1]**o+(-0.0625995244022)*x[1]+(0.767004105382)*x[2]**o+(0.536886296262)*x[2]
        s[2,2,0,0]=(0.0907868991281)*x[0]**o+(0.0361485757336)*x[0]+(0.725787806874)*x[1]**o+(-0.955232659581)*x[1]+(0.015387691978)*x[2]**o+(-0.969939112182)*x[2]
        s[2,2,0,1]=(-0.702771278477)*x[0]**o+(-0.376611311235)*x[0]+(0.321676098666)*x[1]**o+(0.2741544323)*x[1]+(-0.490015464554)*x[2]**o+(-0.287278266636)*x[2]
        s[2,2,1,0]=(-0.239838875645)*x[0]**o+(-0.46816615613)*x[0]+(0.208010055752)*x[1]**o+(0.343192409907)*x[1]+(-0.752447682834)*x[2]**o+(-0.450307042264)*x[2]
        s[2,2,1,1]=(-0.377645472017)*x[0]**o+(0.979388261808)*x[0]+(0.899484011996)*x[1]**o+(0.278589875985)*x[1]+(-0.0566572805419)*x[2]**o+(-0.329814353957)*x[2]
        s[2,2,2,0]=(0.329972082761)*x[0]**o+(-0.40893298634)*x[0]+(0.14025500949)*x[1]**o+(-0.915605553253)*x[1]+(-0.743877103988)*x[2]**o+(0.0625993015596)*x[2]
        s[2,2,2,1]=(-0.426922382741)*x[0]**o+(0.638893012106)*x[0]+(0.775217826454)*x[1]**o+(0.64924098439)*x[1]+(-0.157863119278)*x[2]**o+(-0.785891322569)*x[2]
        s[2,3,0,0]=(-0.573635249481)*x[0]**o+(0.466378353545)*x[0]+(-0.968280361551)*x[1]**o+(-0.521040950167)*x[1]+(-0.979861738542)*x[2]**o+(0.293316009588)*x[2]
        s[2,3,0,1]=(-0.987445004264)*x[0]**o+(-0.855558575123)*x[0]+(0.703881586225)*x[1]**o+(0.393607926606)*x[1]+(0.615218365579)*x[2]**o+(0.865070564586)*x[2]
        s[2,3,1,0]=(0.243032241627)*x[0]**o+(0.447193793692)*x[0]+(0.121835082448)*x[1]**o+(0.253731889763)*x[1]+(0.482861064484)*x[2]**o+(0.0857661253915)*x[2]
        s[2,3,1,1]=(0.630205092814)*x[0]**o+(-0.230885016198)*x[0]+(-0.00423203911773)*x[1]**o+(0.560066624606)*x[1]+(-0.850065237172)*x[2]**o+(-0.0608202441365)*x[2]
        s[2,3,2,0]=(0.956674282321)*x[0]**o+(0.478516276613)*x[0]+(-0.142801297861)*x[1]**o+(-0.470299296114)*x[1]+(0.947735333619)*x[2]**o+(0.343691324696)*x[2]
        s[2,3,2,1]=(0.366910635243)*x[0]**o+(-0.0831991185812)*x[0]+(-0.828901851379)*x[1]**o+(-0.537917965468)*x[1]+(-0.581342127867)*x[2]**o+(-0.307660153071)*x[2]
        s[2,4,0,0]=(0.340464857467)*x[0]**o+(0.881022180176)*x[0]+(-0.223178224315)*x[1]**o+(0.0108952156846)*x[1]+(-0.130766988014)*x[2]**o+(-0.655755053638)*x[2]
        s[2,4,0,1]=(-0.142352024958)*x[0]**o+(-0.279134821704)*x[0]+(0.696577046583)*x[1]**o+(-0.913765778235)*x[1]+(0.674571789555)*x[2]**o+(-0.236086993066)*x[2]
        s[2,4,1,0]=(0.993240530889)*x[0]**o+(0.152584699472)*x[0]+(0.364843511953)*x[1]**o+(0.900205378533)*x[1]+(0.397560553857)*x[2]**o+(0.334533062647)*x[2]
        s[2,4,1,1]=(-0.261396791645)*x[0]**o+(0.96949114295)*x[0]+(0.604187009028)*x[1]**o+(-0.667369171782)*x[1]+(-0.313439370957)*x[2]**o+(0.448859840994)*x[2]
        s[2,4,2,0]=(-0.508273068959)*x[0]**o+(0.473127945457)*x[0]+(0.512686326084)*x[1]**o+(-0.872890815913)*x[1]+(-0.515333454444)*x[2]**o+(-0.578402077636)*x[2]
        s[2,4,2,1]=(-0.78684369593)*x[0]**o+(0.719254214286)*x[0]+(-0.857414812359)*x[1]**o+(-0.260538452293)*x[1]+(0.940982479653)*x[2]**o+(-0.94620100965)*x[2]
        s[3,0,0,0]=(0.741419615809)*x[0]**o+(0.535310119259)*x[0]+(-0.396521376661)*x[1]**o+(0.238209214692)*x[1]+(0.550440543811)*x[2]**o+(0.592067475252)*x[2]
        s[3,0,0,1]=(0.423030333878)*x[0]**o+(0.434870473967)*x[0]+(-0.599556117755)*x[1]**o+(0.299434424134)*x[1]+(0.120835992748)*x[2]**o+(-0.314011916578)*x[2]
        s[3,0,1,0]=(-0.326589197488)*x[0]**o+(-0.5670140013)*x[0]+(0.0548382156857)*x[1]**o+(0.689609176876)*x[1]+(0.257405458159)*x[2]**o+(-0.896175631279)*x[2]
        s[3,0,1,1]=(0.221120334733)*x[0]**o+(-0.273395088002)*x[0]+(0.678993292365)*x[1]**o+(-0.261592406628)*x[1]+(0.0129202430825)*x[2]**o+(0.148164299785)*x[2]
        s[3,0,2,0]=(0.894503166055)*x[0]**o+(-0.468004653303)*x[0]+(-0.103574702833)*x[1]**o+(-0.390725885766)*x[1]+(0.081712057468)*x[2]**o+(-0.507015846204)*x[2]
        s[3,0,2,1]=(0.712909263678)*x[0]**o+(-0.412420758959)*x[0]+(-0.620377453769)*x[1]**o+(-0.166241669671)*x[1]+(-0.738733737609)*x[2]**o+(-0.757426176741)*x[2]
        s[3,1,0,0]=(-0.166116194562)*x[0]**o+(0.988666848357)*x[0]+(-0.483399298528)*x[1]**o+(0.777960698485)*x[1]+(-0.374839973139)*x[2]**o+(0.914310220752)*x[2]
        s[3,1,0,1]=(-0.578374538082)*x[0]**o+(0.228605767869)*x[0]+(0.120418953818)*x[1]**o+(-0.693317296354)*x[1]+(-0.691997757818)*x[2]**o+(0.551252364781)*x[2]
        s[3,1,1,0]=(-0.384863493904)*x[0]**o+(0.300215711827)*x[0]+(0.948187183083)*x[1]**o+(-0.184464558346)*x[1]+(-0.7671733816)*x[2]**o+(0.307217058571)*x[2]
        s[3,1,1,1]=(0.578689722929)*x[0]**o+(0.885714287205)*x[0]+(0.722247905148)*x[1]**o+(0.933475647861)*x[1]+(0.211727836913)*x[2]**o+(0.711589357201)*x[2]
        s[3,1,2,0]=(-0.116046568217)*x[0]**o+(-0.735782802049)*x[0]+(-0.968041163959)*x[1]**o+(0.314957910843)*x[1]+(0.639581098814)*x[2]**o+(-0.566524915874)*x[2]
        s[3,1,2,1]=(0.0690920573575)*x[0]**o+(-0.0344562175408)*x[0]+(-0.729181462751)*x[1]**o+(-0.352992891115)*x[1]+(-0.230186276771)*x[2]**o+(-0.392393764843)*x[2]
        s[3,2,0,0]=(-0.897572361738)*x[0]**o+(0.42292848631)*x[0]+(0.0402102445405)*x[1]**o+(-0.695803229507)*x[1]+(0.657341764505)*x[2]**o+(0.534681573441)*x[2]
        s[3,2,0,1]=(-0.883898106985)*x[0]**o+(0.563240147064)*x[0]+(0.596180998828)*x[1]**o+(0.278590998454)*x[1]+(-0.686188881246)*x[2]**o+(0.827407211045)*x[2]
        s[3,2,1,0]=(-0.896992911933)*x[0]**o+(0.349871189445)*x[0]+(0.959261344864)*x[1]**o+(0.567834800455)*x[1]+(0.624336079204)*x[2]**o+(-0.853743864993)*x[2]
        s[3,2,1,1]=(0.947681143718)*x[0]**o+(0.563362524754)*x[0]+(0.398321082655)*x[1]**o+(0.217952803297)*x[1]+(0.243654787599)*x[2]**o+(-0.927517802601)*x[2]
        s[3,2,2,0]=(0.330842075788)*x[0]**o+(0.630950656819)*x[0]+(-0.469619054343)*x[1]**o+(-0.354739266808)*x[1]+(0.253515569858)*x[2]**o+(0.635440000647)*x[2]
        s[3,2,2,1]=(0.729280956878)*x[0]**o+(0.734399392463)*x[0]+(0.800616353182)*x[1]**o+(-0.764367237486)*x[1]+(0.90307570667)*x[2]**o+(-0.957618675411)*x[2]
        s[3,3,0,0]=(-0.783664290925)*x[0]**o+(-0.609524665456)*x[0]+(0.0126829605834)*x[1]**o+(0.396479443362)*x[1]+(0.262779944509)*x[2]**o+(-0.666191777094)*x[2]
        s[3,3,0,1]=(0.412930217814)*x[0]**o+(0.993234310311)*x[0]+(-0.123107776537)*x[1]**o+(-0.156132010898)*x[1]+(-0.72881870001)*x[2]**o+(0.757371069942)*x[2]
        s[3,3,1,0]=(-0.829528778744)*x[0]**o+(-0.676499470344)*x[0]+(-0.65512195457)*x[1]**o+(0.758109011229)*x[1]+(-0.5027027576)*x[2]**o+(-0.139493760854)*x[2]
        s[3,3,1,1]=(-0.489794277367)*x[0]**o+(-0.58772339266)*x[0]+(-0.920341394628)*x[1]**o+(-0.0302202079576)*x[1]+(-0.874433835334)*x[2]**o+(-0.382644983451)*x[2]
        s[3,3,2,0]=(-0.729507420376)*x[0]**o+(0.785351449428)*x[0]+(0.134356586244)*x[1]**o+(-0.786187017995)*x[1]+(-0.460746337928)*x[2]**o+(0.24750031597)*x[2]
        s[3,3,2,1]=(0.422343530499)*x[0]**o+(0.951298731532)*x[0]+(0.746519769375)*x[1]**o+(-0.787687730811)*x[1]+(-0.189033858381)*x[2]**o+(0.875049681221)*x[2]
        s[3,4,0,0]=(-0.784862616375)*x[0]**o+(0.605595736907)*x[0]+(-0.758269958875)*x[1]**o+(0.983503797662)*x[1]+(-0.897215719033)*x[2]**o+(0.434216381093)*x[2]
        s[3,4,0,1]=(0.959540582812)*x[0]**o+(0.74585782554)*x[0]+(0.00254441839877)*x[1]**o+(0.026781582564)*x[1]+(-0.0121835909437)*x[2]**o+(-0.166652448502)*x[2]
        s[3,4,1,0]=(-0.875170954413)*x[0]**o+(0.963228995313)*x[0]+(-0.198757196024)*x[1]**o+(-0.16115897441)*x[1]+(0.668387545829)*x[2]**o+(0.784441651668)*x[2]
        s[3,4,1,1]=(-0.0966644403845)*x[0]**o+(-0.0128070214998)*x[0]+(0.471059883662)*x[1]**o+(0.792978130351)*x[1]+(-0.709010757124)*x[2]**o+(-0.535329042416)*x[2]
        s[3,4,2,0]=(0.390795392416)*x[0]**o+(0.0465531497232)*x[0]+(-0.665065120082)*x[1]**o+(-0.637116189764)*x[1]+(0.671385406225)*x[2]**o+(-0.663428041906)*x[2]
        s[3,4,2,1]=(-0.962153478958)*x[0]**o+(0.255601758555)*x[0]+(-0.843511999734)*x[1]**o+(0.677818749903)*x[1]+(-0.273341883139)*x[2]**o+(0.846596068215)*x[2]
        ref[0,0,0,0]=(-0.400665270553)*x_ref[0]**o+(-0.465656794893)*x_ref[0]+(-0.103120094839)*x_ref[1]**o+(0.0464026120794)*x_ref[1]+(0.030596268742)*x_ref[2]**o+(0.00334628754684)*x_ref[2]
        ref[0,0,0,1]=(0.937480902128)*x_ref[0]**o+(0.0466658012652)*x_ref[0]+(0.817801316276)*x_ref[1]**o+(-0.142948501497)*x_ref[1]+(-0.946943305156)*x_ref[2]**o+(-0.633505380306)*x_ref[2]
        ref[0,0,1,0]=(-0.441314521722)*x_ref[0]**o+(-0.42022483938)*x_ref[0]+(-0.833228022905)*x_ref[1]**o+(-0.225426650714)*x_ref[1]+(0.332718715913)*x_ref[2]**o+(0.123261640846)*x_ref[2]
        ref[0,0,1,1]=(0.335875498268)*x_ref[0]**o+(0.983700594491)*x_ref[0]+(-0.214325088125)*x_ref[1]**o+(-0.893543563881)*x_ref[1]+(0.0631361789468)*x_ref[2]**o+(0.4467209081)*x_ref[2]
        ref[0,0,2,0]=(0.658215366848)*x_ref[0]**o+(-0.672295829784)*x_ref[0]+(-0.0137993447332)*x_ref[1]**o+(0.698632148806)*x_ref[1]+(-0.221572355182)*x_ref[2]**o+(0.956096660679)*x_ref[2]
        ref[0,0,2,1]=(-0.578795938537)*x_ref[0]**o+(0.750684927067)*x_ref[0]+(0.333363761953)*x_ref[1]**o+(-0.617804591638)*x_ref[1]+(-0.0469494394831)*x_ref[2]**o+(0.16187318118)*x_ref[2]
        ref[0,1,0,0]=(-0.0638097634115)*x_ref[0]**o+(-0.38507312448)*x_ref[0]+(-0.663736729005)*x_ref[1]**o+(-0.989965410511)*x_ref[1]+(0.230748674134)*x_ref[2]**o+(-0.830580398633)*x_ref[2]
        ref[0,1,0,1]=(-0.658956286059)*x_ref[0]**o+(0.345180478933)*x_ref[0]+(-0.544808816473)*x_ref[1]**o+(0.799798879144)*x_ref[1]+(-0.862695252289)*x_ref[2]**o+(0.757134401664)*x_ref[2]
        ref[0,1,1,0]=(0.456144819828)*x_ref[0]**o+(0.969408065204)*x_ref[0]+(0.488759322896)*x_ref[1]**o+(0.191259701658)*x_ref[1]+(-0.211530905801)*x_ref[2]**o+(0.0976530925065)*x_ref[2]
        ref[0,1,1,1]=(-0.160993701908)*x_ref[0]**o+(-0.768084723399)*x_ref[0]+(0.200776153747)*x_ref[1]**o+(0.380923444552)*x_ref[1]+(0.152388183214)*x_ref[2]**o+(0.0345495856869)*x_ref[2]
        ref[0,1,2,0]=(-0.249731094314)*x_ref[0]**o+(0.420921348364)*x_ref[0]+(-0.745813017956)*x_ref[1]**o+(-0.128319964472)*x_ref[1]+(-0.134590180255)*x_ref[2]**o+(-0.598364990007)*x_ref[2]
        ref[0,1,2,1]=(0.608497167751)*x_ref[0]**o+(0.134066300157)*x_ref[0]+(-0.219556933024)*x_ref[1]**o+(0.845095947325)*x_ref[1]+(-0.0910112208065)*x_ref[2]**o+(-0.919387374125)*x_ref[2]
        ref[0,2,0,0]=(-0.603722566149)*x_ref[0]**o+(-0.141049572403)*x_ref[0]+(-0.433072627843)*x_ref[1]**o+(-0.266640930301)*x_ref[1]+(-0.0520863163599)*x_ref[2]**o+(0.113826044084)*x_ref[2]
        ref[0,2,0,1]=(0.735600439472)*x_ref[0]**o+(-0.683666203432)*x_ref[0]+(0.439527311107)*x_ref[1]**o+(0.167685906293)*x_ref[1]+(-0.526256697628)*x_ref[2]**o+(-0.152028404784)*x_ref[2]
        ref[0,2,1,0]=(-0.484268475407)*x_ref[0]**o+(-0.1302369788)*x_ref[0]+(0.0996963541291)*x_ref[1]**o+(0.525432686771)*x_ref[1]+(0.915056396412)*x_ref[2]**o+(-0.93142015197)*x_ref[2]
        ref[0,2,1,1]=(0.103016217069)*x_ref[0]**o+(0.00517836018812)*x_ref[0]+(-0.28331822161)*x_ref[1]**o+(0.413800832478)*x_ref[1]+(0.94613674069)*x_ref[2]**o+(-0.287932854599)*x_ref[2]
        ref[0,2,2,0]=(0.512958587386)*x_ref[0]**o+(-0.851145503923)*x_ref[0]+(-0.535377708818)*x_ref[1]**o+(-0.379448444669)*x_ref[1]+(-0.238808818844)*x_ref[2]**o+(0.567146071008)*x_ref[2]
        ref[0,2,2,1]=(-0.141490825422)*x_ref[0]**o+(0.133948738844)*x_ref[0]+(0.336733111602)*x_ref[1]**o+(-0.223425201179)*x_ref[1]+(-0.646454138519)*x_ref[2]**o+(0.823936105317)*x_ref[2]
        ref[0,3,0,0]=(-0.938725695354)*x_ref[0]**o+(0.988131864396)*x_ref[0]+(-0.82501934096)*x_ref[1]**o+(-0.257156351526)*x_ref[1]+(-0.00349993655072)*x_ref[2]**o+(0.580041226419)*x_ref[2]
        ref[0,3,0,1]=(0.814169577155)*x_ref[0]**o+(-0.686109074376)*x_ref[0]+(-0.965459022554)*x_ref[1]**o+(0.10941355421)*x_ref[1]+(0.609050101353)*x_ref[2]**o+(-0.965588393623)*x_ref[2]
        ref[0,3,1,0]=(-0.820571213003)*x_ref[0]**o+(0.746749194774)*x_ref[0]+(0.710345174116)*x_ref[1]**o+(-0.892921665205)*x_ref[1]+(0.4004855541)*x_ref[2]**o+(-0.695117880583)*x_ref[2]
        ref[0,3,1,1]=(-0.122273772977)*x_ref[0]**o+(0.570499591543)*x_ref[0]+(0.795058034707)*x_ref[1]**o+(0.925412519293)*x_ref[1]+(-0.0932686873001)*x_ref[2]**o+(-0.686679792186)*x_ref[2]
        ref[0,3,2,0]=(-0.0042793185541)*x_ref[0]**o+(-0.821480764508)*x_ref[0]+(0.875015113726)*x_ref[1]**o+(0.649383518184)*x_ref[1]+(-0.288257569005)*x_ref[2]**o+(-0.743941827161)*x_ref[2]
        ref[0,3,2,1]=(0.974182567966)*x_ref[0]**o+(0.733657486071)*x_ref[0]+(0.780035494556)*x_ref[1]**o+(0.749708949611)*x_ref[1]+(-0.521673379431)*x_ref[2]**o+(0.673844867694)*x_ref[2]
        ref[0,4,0,0]=(-0.150057729107)*x_ref[0]**o+(0.102611114471)*x_ref[0]+(-0.0633864241155)*x_ref[1]**o+(-0.329289187178)*x_ref[1]+(0.315706711936)*x_ref[2]**o+(-0.554443679394)*x_ref[2]
        ref[0,4,0,1]=(0.564883543338)*x_ref[0]**o+(0.535008108501)*x_ref[0]+(0.815233609718)*x_ref[1]**o+(-0.766536697052)*x_ref[1]+(0.997595942191)*x_ref[2]**o+(0.991881374395)*x_ref[2]
        ref[0,4,1,0]=(-0.725748156838)*x_ref[0]**o+(-0.817693342658)*x_ref[0]+(0.501105656184)*x_ref[1]**o+(-0.331771590595)*x_ref[1]+(-0.757619653898)*x_ref[2]**o+(-0.885657271086)*x_ref[2]
        ref[0,4,1,1]=(-0.559814749411)*x_ref[0]**o+(0.665634435712)*x_ref[0]+(0.787002371365)*x_ref[1]**o+(-0.762034037705)*x_ref[1]+(0.508112385178)*x_ref[2]**o+(-0.985681857439)*x_ref[2]
        ref[0,4,2,0]=(0.504217223699)*x_ref[0]**o+(-0.361750429828)*x_ref[0]+(0.19299099109)*x_ref[1]**o+(0.338521927796)*x_ref[1]+(0.549199682759)*x_ref[2]**o+(0.891662590379)*x_ref[2]
        ref[0,4,2,1]=(-0.251870804657)*x_ref[0]**o+(0.776378068034)*x_ref[0]+(-0.769493074808)*x_ref[1]**o+(0.929429944731)*x_ref[1]+(0.679845951817)*x_ref[2]**o+(0.261179050197)*x_ref[2]
        ref[1,0,0,0]=(0.339823121204)*x_ref[0]**o+(0.212075564821)*x_ref[0]+(0.103901436655)*x_ref[1]**o+(0.887698805047)*x_ref[1]+(-0.467697404092)*x_ref[2]**o+(0.71153584324)*x_ref[2]
        ref[1,0,0,1]=(0.610147759489)*x_ref[0]**o+(-0.820866072636)*x_ref[0]+(0.505294661936)*x_ref[1]**o+(-0.206382410595)*x_ref[1]+(0.343889298437)*x_ref[2]**o+(0.165680393811)*x_ref[2]
        ref[1,0,1,0]=(-0.155342751495)*x_ref[0]**o+(-0.638727698114)*x_ref[0]+(-0.245328510129)*x_ref[1]**o+(0.816689513534)*x_ref[1]+(0.138847830114)*x_ref[2]**o+(0.286577642911)*x_ref[2]
        ref[1,0,1,1]=(0.804058022285)*x_ref[0]**o+(-0.922679564915)*x_ref[0]+(-0.755679292664)*x_ref[1]**o+(-0.409251039141)*x_ref[1]+(-0.697869565514)*x_ref[2]**o+(-0.0749517480557)*x_ref[2]
        ref[1,0,2,0]=(-0.345641262813)*x_ref[0]**o+(0.31066537094)*x_ref[0]+(0.165733093512)*x_ref[1]**o+(-0.733996040753)*x_ref[1]+(0.240017268146)*x_ref[2]**o+(0.981970338884)*x_ref[2]
        ref[1,0,2,1]=(-0.0394605078977)*x_ref[0]**o+(-0.671255389736)*x_ref[0]+(-0.58457671038)*x_ref[1]**o+(-0.308411795131)*x_ref[1]+(0.944299000242)*x_ref[2]**o+(0.0274884386004)*x_ref[2]
        ref[1,1,0,0]=(0.285078336337)*x_ref[0]**o+(-0.29609479737)*x_ref[0]+(0.851762966843)*x_ref[1]**o+(-0.56978612625)*x_ref[1]+(-0.416902609766)*x_ref[2]**o+(-0.209352333275)*x_ref[2]
        ref[1,1,0,1]=(0.165305243827)*x_ref[0]**o+(0.569657682766)*x_ref[0]+(-0.158936758165)*x_ref[1]**o+(-0.826897725956)*x_ref[1]+(-0.906153311902)*x_ref[2]**o+(-0.508208690461)*x_ref[2]
        ref[1,1,1,0]=(-0.168856536186)*x_ref[0]**o+(0.600931452022)*x_ref[0]+(0.515269032144)*x_ref[1]**o+(-0.615629628046)*x_ref[1]+(-0.248016004788)*x_ref[2]**o+(-0.236340748995)*x_ref[2]
        ref[1,1,1,1]=(-0.265342410313)*x_ref[0]**o+(0.546304098539)*x_ref[0]+(0.44451795278)*x_ref[1]**o+(0.53863749512)*x_ref[1]+(-0.400353954694)*x_ref[2]**o+(0.762091735577)*x_ref[2]
        ref[1,1,2,0]=(-0.828507588085)*x_ref[0]**o+(0.310151266525)*x_ref[0]+(0.767699919156)*x_ref[1]**o+(-0.802526284756)*x_ref[1]+(-0.12986443685)*x_ref[2]**o+(0.555810425336)*x_ref[2]
        ref[1,1,2,1]=(-0.0843531370201)*x_ref[0]**o+(0.138067935996)*x_ref[0]+(-0.819663672158)*x_ref[1]**o+(0.227177408663)*x_ref[1]+(-0.481488807321)*x_ref[2]**o+(0.277780608851)*x_ref[2]
        ref[1,2,0,0]=(0.843912054965)*x_ref[0]**o+(-0.532942686099)*x_ref[0]+(-0.12263392247)*x_ref[1]**o+(-0.918649820864)*x_ref[1]+(-0.898969788765)*x_ref[2]**o+(0.401718907796)*x_ref[2]
        ref[1,2,0,1]=(0.500181344385)*x_ref[0]**o+(-0.695081419586)*x_ref[0]+(0.49127001192)*x_ref[1]**o+(0.424707942699)*x_ref[1]+(-0.170525943009)*x_ref[2]**o+(-0.908857558417)*x_ref[2]
        ref[1,2,1,0]=(0.374108805296)*x_ref[0]**o+(-0.778311131014)*x_ref[0]+(-0.619173488016)*x_ref[1]**o+(-0.0906082150795)*x_ref[1]+(0.312783760308)*x_ref[2]**o+(0.50073883273)*x_ref[2]
        ref[1,2,1,1]=(0.474622980263)*x_ref[0]**o+(0.00968769992516)*x_ref[0]+(-0.246566756628)*x_ref[1]**o+(0.868983907653)*x_ref[1]+(-0.318924531295)*x_ref[2]**o+(-0.340314263979)*x_ref[2]
        ref[1,2,2,0]=(0.621111695462)*x_ref[0]**o+(0.655354521723)*x_ref[0]+(-0.992404590253)*x_ref[1]**o+(-0.497860913536)*x_ref[1]+(-0.963211421906)*x_ref[2]**o+(0.493347857543)*x_ref[2]
        ref[1,2,2,1]=(-0.32035342079)*x_ref[0]**o+(0.351692950156)*x_ref[0]+(-0.715457409869)*x_ref[1]**o+(0.317941180953)*x_ref[1]+(0.831453341583)*x_ref[2]**o+(0.330720124015)*x_ref[2]
        ref[1,3,0,0]=(-0.775605593731)*x_ref[0]**o+(-0.985544306967)*x_ref[0]+(0.838313790256)*x_ref[1]**o+(-0.0515692297878)*x_ref[1]+(0.6757488996)*x_ref[2]**o+(-0.398401163936)*x_ref[2]
        ref[1,3,0,1]=(0.129462614525)*x_ref[0]**o+(-0.225290874002)*x_ref[0]+(0.1627137973)*x_ref[1]**o+(0.96490154212)*x_ref[1]+(-0.0348196505137)*x_ref[2]**o+(0.00501682452837)*x_ref[2]
        ref[1,3,1,0]=(-0.774906442617)*x_ref[0]**o+(0.554579514554)*x_ref[0]+(0.68683898073)*x_ref[1]**o+(0.687422147497)*x_ref[1]+(0.812740760401)*x_ref[2]**o+(0.650574541761)*x_ref[2]
        ref[1,3,1,1]=(0.371414654481)*x_ref[0]**o+(-0.460895148602)*x_ref[0]+(-0.60035497386)*x_ref[1]**o+(0.382490052311)*x_ref[1]+(0.348365164241)*x_ref[2]**o+(0.700564991456)*x_ref[2]
        ref[1,3,2,0]=(0.0627821130997)*x_ref[0]**o+(-0.108341459129)*x_ref[0]+(-0.459931858236)*x_ref[1]**o+(-0.84999694423)*x_ref[1]+(-0.369143500965)*x_ref[2]**o+(-0.204962250898)*x_ref[2]
        ref[1,3,2,1]=(0.940231653623)*x_ref[0]**o+(0.432366518466)*x_ref[0]+(0.965573462127)*x_ref[1]**o+(-0.400536850676)*x_ref[1]+(0.627651239821)*x_ref[2]**o+(0.541970384011)*x_ref[2]
        ref[1,4,0,0]=(-0.905529267496)*x_ref[0]**o+(-0.728637228667)*x_ref[0]+(0.528565304374)*x_ref[1]**o+(-0.0791953805449)*x_ref[1]+(0.278340538308)*x_ref[2]**o+(0.0829173042009)*x_ref[2]
        ref[1,4,0,1]=(0.196476386515)*x_ref[0]**o+(0.960964133173)*x_ref[0]+(-0.975854821639)*x_ref[1]**o+(-0.231600097119)*x_ref[1]+(0.462289007605)*x_ref[2]**o+(-0.41548631733)*x_ref[2]
        ref[1,4,1,0]=(0.133593658437)*x_ref[0]**o+(0.27713134658)*x_ref[0]+(-0.626190133661)*x_ref[1]**o+(0.22128472818)*x_ref[1]+(-0.0310045745464)*x_ref[2]**o+(-0.308952151364)*x_ref[2]
        ref[1,4,1,1]=(-0.540782388402)*x_ref[0]**o+(0.782610613508)*x_ref[0]+(-0.121818568845)*x_ref[1]**o+(-0.481514588156)*x_ref[1]+(0.92789834048)*x_ref[2]**o+(-0.053669614999)*x_ref[2]
        ref[1,4,2,0]=(0.228180064246)*x_ref[0]**o+(-0.387791183012)*x_ref[0]+(-0.809517349857)*x_ref[1]**o+(0.104233744095)*x_ref[1]+(-0.010561196572)*x_ref[2]**o+(0.812403786777)*x_ref[2]
        ref[1,4,2,1]=(-0.213527688719)*x_ref[0]**o+(0.444377334059)*x_ref[0]+(0.250474600963)*x_ref[1]**o+(-0.628454953053)*x_ref[1]+(-0.149400612708)*x_ref[2]**o+(-0.362500181895)*x_ref[2]
        ref[2,0,0,0]=(0.424970215816)*x_ref[0]**o+(-0.409634055175)*x_ref[0]+(-0.209831967682)*x_ref[1]**o+(0.777759135142)*x_ref[1]+(0.904286801356)*x_ref[2]**o+(-0.533118859622)*x_ref[2]
        ref[2,0,0,1]=(-0.0634924164631)*x_ref[0]**o+(0.61665557894)*x_ref[0]+(0.886000765027)*x_ref[1]**o+(-0.283672331088)*x_ref[1]+(-0.104923941729)*x_ref[2]**o+(0.647966459575)*x_ref[2]
        ref[2,0,1,0]=(-0.601180947882)*x_ref[0]**o+(-0.853176409106)*x_ref[0]+(0.589049472252)*x_ref[1]**o+(0.947188905512)*x_ref[1]+(0.937016925257)*x_ref[2]**o+(-0.528933622785)*x_ref[2]
        ref[2,0,1,1]=(-0.493108989214)*x_ref[0]**o+(0.123452031172)*x_ref[0]+(0.579015108934)*x_ref[1]**o+(-0.224656610786)*x_ref[1]+(-0.351966996224)*x_ref[2]**o+(-0.58667858467)*x_ref[2]
        ref[2,0,2,0]=(0.342923581511)*x_ref[0]**o+(-0.787209375031)*x_ref[0]+(0.0689438410554)*x_ref[1]**o+(0.624797184447)*x_ref[1]+(0.508789531503)*x_ref[2]**o+(0.17151319396)*x_ref[2]
        ref[2,0,2,1]=(-0.875568056219)*x_ref[0]**o+(-0.113031987088)*x_ref[0]+(-0.0609746766081)*x_ref[1]**o+(0.196111602161)*x_ref[1]+(-0.00186601407674)*x_ref[2]**o+(-0.695244625545)*x_ref[2]
        ref[2,1,0,0]=(0.0760124938281)*x_ref[0]**o+(0.878269236128)*x_ref[0]+(0.424404663038)*x_ref[1]**o+(0.382191251895)*x_ref[1]+(-0.799519816473)*x_ref[2]**o+(0.756150518781)*x_ref[2]
        ref[2,1,0,1]=(-0.620026715848)*x_ref[0]**o+(0.120584354877)*x_ref[0]+(0.794571749541)*x_ref[1]**o+(0.775886892396)*x_ref[1]+(-0.455364446354)*x_ref[2]**o+(-0.25890753368)*x_ref[2]
        ref[2,1,1,0]=(-0.434917779388)*x_ref[0]**o+(0.893805701213)*x_ref[0]+(-0.359966318659)*x_ref[1]**o+(-0.92549769048)*x_ref[1]+(0.632194530492)*x_ref[2]**o+(0.783055504124)*x_ref[2]
        ref[2,1,1,1]=(-0.30435434794)*x_ref[0]**o+(-0.132800060169)*x_ref[0]+(0.200039404417)*x_ref[1]**o+(-0.690955124262)*x_ref[1]+(-0.984472889901)*x_ref[2]**o+(0.0242811209734)*x_ref[2]
        ref[2,1,2,0]=(-0.223815583384)*x_ref[0]**o+(0.592762849244)*x_ref[0]+(0.281436806248)*x_ref[1]**o+(0.822974431302)*x_ref[1]+(-0.0648638855118)*x_ref[2]**o+(-0.798035298439)*x_ref[2]
        ref[2,1,2,1]=(-0.847339863019)*x_ref[0]**o+(0.603483610026)*x_ref[0]+(0.865918681346)*x_ref[1]**o+(-0.0625995244022)*x_ref[1]+(0.767004105382)*x_ref[2]**o+(0.536886296262)*x_ref[2]
        ref[2,2,0,0]=(0.0907868991281)*x_ref[0]**o+(0.0361485757336)*x_ref[0]+(0.725787806874)*x_ref[1]**o+(-0.955232659581)*x_ref[1]+(0.015387691978)*x_ref[2]**o+(-0.969939112182)*x_ref[2]
        ref[2,2,0,1]=(-0.702771278477)*x_ref[0]**o+(-0.376611311235)*x_ref[0]+(0.321676098666)*x_ref[1]**o+(0.2741544323)*x_ref[1]+(-0.490015464554)*x_ref[2]**o+(-0.287278266636)*x_ref[2]
        ref[2,2,1,0]=(-0.239838875645)*x_ref[0]**o+(-0.46816615613)*x_ref[0]+(0.208010055752)*x_ref[1]**o+(0.343192409907)*x_ref[1]+(-0.752447682834)*x_ref[2]**o+(-0.450307042264)*x_ref[2]
        ref[2,2,1,1]=(-0.377645472017)*x_ref[0]**o+(0.979388261808)*x_ref[0]+(0.899484011996)*x_ref[1]**o+(0.278589875985)*x_ref[1]+(-0.0566572805419)*x_ref[2]**o+(-0.329814353957)*x_ref[2]
        ref[2,2,2,0]=(0.329972082761)*x_ref[0]**o+(-0.40893298634)*x_ref[0]+(0.14025500949)*x_ref[1]**o+(-0.915605553253)*x_ref[1]+(-0.743877103988)*x_ref[2]**o+(0.0625993015596)*x_ref[2]
        ref[2,2,2,1]=(-0.426922382741)*x_ref[0]**o+(0.638893012106)*x_ref[0]+(0.775217826454)*x_ref[1]**o+(0.64924098439)*x_ref[1]+(-0.157863119278)*x_ref[2]**o+(-0.785891322569)*x_ref[2]
        ref[2,3,0,0]=(-0.573635249481)*x_ref[0]**o+(0.466378353545)*x_ref[0]+(-0.968280361551)*x_ref[1]**o+(-0.521040950167)*x_ref[1]+(-0.979861738542)*x_ref[2]**o+(0.293316009588)*x_ref[2]
        ref[2,3,0,1]=(-0.987445004264)*x_ref[0]**o+(-0.855558575123)*x_ref[0]+(0.703881586225)*x_ref[1]**o+(0.393607926606)*x_ref[1]+(0.615218365579)*x_ref[2]**o+(0.865070564586)*x_ref[2]
        ref[2,3,1,0]=(0.243032241627)*x_ref[0]**o+(0.447193793692)*x_ref[0]+(0.121835082448)*x_ref[1]**o+(0.253731889763)*x_ref[1]+(0.482861064484)*x_ref[2]**o+(0.0857661253915)*x_ref[2]
        ref[2,3,1,1]=(0.630205092814)*x_ref[0]**o+(-0.230885016198)*x_ref[0]+(-0.00423203911773)*x_ref[1]**o+(0.560066624606)*x_ref[1]+(-0.850065237172)*x_ref[2]**o+(-0.0608202441365)*x_ref[2]
        ref[2,3,2,0]=(0.956674282321)*x_ref[0]**o+(0.478516276613)*x_ref[0]+(-0.142801297861)*x_ref[1]**o+(-0.470299296114)*x_ref[1]+(0.947735333619)*x_ref[2]**o+(0.343691324696)*x_ref[2]
        ref[2,3,2,1]=(0.366910635243)*x_ref[0]**o+(-0.0831991185812)*x_ref[0]+(-0.828901851379)*x_ref[1]**o+(-0.537917965468)*x_ref[1]+(-0.581342127867)*x_ref[2]**o+(-0.307660153071)*x_ref[2]
        ref[2,4,0,0]=(0.340464857467)*x_ref[0]**o+(0.881022180176)*x_ref[0]+(-0.223178224315)*x_ref[1]**o+(0.0108952156846)*x_ref[1]+(-0.130766988014)*x_ref[2]**o+(-0.655755053638)*x_ref[2]
        ref[2,4,0,1]=(-0.142352024958)*x_ref[0]**o+(-0.279134821704)*x_ref[0]+(0.696577046583)*x_ref[1]**o+(-0.913765778235)*x_ref[1]+(0.674571789555)*x_ref[2]**o+(-0.236086993066)*x_ref[2]
        ref[2,4,1,0]=(0.993240530889)*x_ref[0]**o+(0.152584699472)*x_ref[0]+(0.364843511953)*x_ref[1]**o+(0.900205378533)*x_ref[1]+(0.397560553857)*x_ref[2]**o+(0.334533062647)*x_ref[2]
        ref[2,4,1,1]=(-0.261396791645)*x_ref[0]**o+(0.96949114295)*x_ref[0]+(0.604187009028)*x_ref[1]**o+(-0.667369171782)*x_ref[1]+(-0.313439370957)*x_ref[2]**o+(0.448859840994)*x_ref[2]
        ref[2,4,2,0]=(-0.508273068959)*x_ref[0]**o+(0.473127945457)*x_ref[0]+(0.512686326084)*x_ref[1]**o+(-0.872890815913)*x_ref[1]+(-0.515333454444)*x_ref[2]**o+(-0.578402077636)*x_ref[2]
        ref[2,4,2,1]=(-0.78684369593)*x_ref[0]**o+(0.719254214286)*x_ref[0]+(-0.857414812359)*x_ref[1]**o+(-0.260538452293)*x_ref[1]+(0.940982479653)*x_ref[2]**o+(-0.94620100965)*x_ref[2]
        ref[3,0,0,0]=(0.741419615809)*x_ref[0]**o+(0.535310119259)*x_ref[0]+(-0.396521376661)*x_ref[1]**o+(0.238209214692)*x_ref[1]+(0.550440543811)*x_ref[2]**o+(0.592067475252)*x_ref[2]
        ref[3,0,0,1]=(0.423030333878)*x_ref[0]**o+(0.434870473967)*x_ref[0]+(-0.599556117755)*x_ref[1]**o+(0.299434424134)*x_ref[1]+(0.120835992748)*x_ref[2]**o+(-0.314011916578)*x_ref[2]
        ref[3,0,1,0]=(-0.326589197488)*x_ref[0]**o+(-0.5670140013)*x_ref[0]+(0.0548382156857)*x_ref[1]**o+(0.689609176876)*x_ref[1]+(0.257405458159)*x_ref[2]**o+(-0.896175631279)*x_ref[2]
        ref[3,0,1,1]=(0.221120334733)*x_ref[0]**o+(-0.273395088002)*x_ref[0]+(0.678993292365)*x_ref[1]**o+(-0.261592406628)*x_ref[1]+(0.0129202430825)*x_ref[2]**o+(0.148164299785)*x_ref[2]
        ref[3,0,2,0]=(0.894503166055)*x_ref[0]**o+(-0.468004653303)*x_ref[0]+(-0.103574702833)*x_ref[1]**o+(-0.390725885766)*x_ref[1]+(0.081712057468)*x_ref[2]**o+(-0.507015846204)*x_ref[2]
        ref[3,0,2,1]=(0.712909263678)*x_ref[0]**o+(-0.412420758959)*x_ref[0]+(-0.620377453769)*x_ref[1]**o+(-0.166241669671)*x_ref[1]+(-0.738733737609)*x_ref[2]**o+(-0.757426176741)*x_ref[2]
        ref[3,1,0,0]=(-0.166116194562)*x_ref[0]**o+(0.988666848357)*x_ref[0]+(-0.483399298528)*x_ref[1]**o+(0.777960698485)*x_ref[1]+(-0.374839973139)*x_ref[2]**o+(0.914310220752)*x_ref[2]
        ref[3,1,0,1]=(-0.578374538082)*x_ref[0]**o+(0.228605767869)*x_ref[0]+(0.120418953818)*x_ref[1]**o+(-0.693317296354)*x_ref[1]+(-0.691997757818)*x_ref[2]**o+(0.551252364781)*x_ref[2]
        ref[3,1,1,0]=(-0.384863493904)*x_ref[0]**o+(0.300215711827)*x_ref[0]+(0.948187183083)*x_ref[1]**o+(-0.184464558346)*x_ref[1]+(-0.7671733816)*x_ref[2]**o+(0.307217058571)*x_ref[2]
        ref[3,1,1,1]=(0.578689722929)*x_ref[0]**o+(0.885714287205)*x_ref[0]+(0.722247905148)*x_ref[1]**o+(0.933475647861)*x_ref[1]+(0.211727836913)*x_ref[2]**o+(0.711589357201)*x_ref[2]
        ref[3,1,2,0]=(-0.116046568217)*x_ref[0]**o+(-0.735782802049)*x_ref[0]+(-0.968041163959)*x_ref[1]**o+(0.314957910843)*x_ref[1]+(0.639581098814)*x_ref[2]**o+(-0.566524915874)*x_ref[2]
        ref[3,1,2,1]=(0.0690920573575)*x_ref[0]**o+(-0.0344562175408)*x_ref[0]+(-0.729181462751)*x_ref[1]**o+(-0.352992891115)*x_ref[1]+(-0.230186276771)*x_ref[2]**o+(-0.392393764843)*x_ref[2]
        ref[3,2,0,0]=(-0.897572361738)*x_ref[0]**o+(0.42292848631)*x_ref[0]+(0.0402102445405)*x_ref[1]**o+(-0.695803229507)*x_ref[1]+(0.657341764505)*x_ref[2]**o+(0.534681573441)*x_ref[2]
        ref[3,2,0,1]=(-0.883898106985)*x_ref[0]**o+(0.563240147064)*x_ref[0]+(0.596180998828)*x_ref[1]**o+(0.278590998454)*x_ref[1]+(-0.686188881246)*x_ref[2]**o+(0.827407211045)*x_ref[2]
        ref[3,2,1,0]=(-0.896992911933)*x_ref[0]**o+(0.349871189445)*x_ref[0]+(0.959261344864)*x_ref[1]**o+(0.567834800455)*x_ref[1]+(0.624336079204)*x_ref[2]**o+(-0.853743864993)*x_ref[2]
        ref[3,2,1,1]=(0.947681143718)*x_ref[0]**o+(0.563362524754)*x_ref[0]+(0.398321082655)*x_ref[1]**o+(0.217952803297)*x_ref[1]+(0.243654787599)*x_ref[2]**o+(-0.927517802601)*x_ref[2]
        ref[3,2,2,0]=(0.330842075788)*x_ref[0]**o+(0.630950656819)*x_ref[0]+(-0.469619054343)*x_ref[1]**o+(-0.354739266808)*x_ref[1]+(0.253515569858)*x_ref[2]**o+(0.635440000647)*x_ref[2]
        ref[3,2,2,1]=(0.729280956878)*x_ref[0]**o+(0.734399392463)*x_ref[0]+(0.800616353182)*x_ref[1]**o+(-0.764367237486)*x_ref[1]+(0.90307570667)*x_ref[2]**o+(-0.957618675411)*x_ref[2]
        ref[3,3,0,0]=(-0.783664290925)*x_ref[0]**o+(-0.609524665456)*x_ref[0]+(0.0126829605834)*x_ref[1]**o+(0.396479443362)*x_ref[1]+(0.262779944509)*x_ref[2]**o+(-0.666191777094)*x_ref[2]
        ref[3,3,0,1]=(0.412930217814)*x_ref[0]**o+(0.993234310311)*x_ref[0]+(-0.123107776537)*x_ref[1]**o+(-0.156132010898)*x_ref[1]+(-0.72881870001)*x_ref[2]**o+(0.757371069942)*x_ref[2]
        ref[3,3,1,0]=(-0.829528778744)*x_ref[0]**o+(-0.676499470344)*x_ref[0]+(-0.65512195457)*x_ref[1]**o+(0.758109011229)*x_ref[1]+(-0.5027027576)*x_ref[2]**o+(-0.139493760854)*x_ref[2]
        ref[3,3,1,1]=(-0.489794277367)*x_ref[0]**o+(-0.58772339266)*x_ref[0]+(-0.920341394628)*x_ref[1]**o+(-0.0302202079576)*x_ref[1]+(-0.874433835334)*x_ref[2]**o+(-0.382644983451)*x_ref[2]
        ref[3,3,2,0]=(-0.729507420376)*x_ref[0]**o+(0.785351449428)*x_ref[0]+(0.134356586244)*x_ref[1]**o+(-0.786187017995)*x_ref[1]+(-0.460746337928)*x_ref[2]**o+(0.24750031597)*x_ref[2]
        ref[3,3,2,1]=(0.422343530499)*x_ref[0]**o+(0.951298731532)*x_ref[0]+(0.746519769375)*x_ref[1]**o+(-0.787687730811)*x_ref[1]+(-0.189033858381)*x_ref[2]**o+(0.875049681221)*x_ref[2]
        ref[3,4,0,0]=(-0.784862616375)*x_ref[0]**o+(0.605595736907)*x_ref[0]+(-0.758269958875)*x_ref[1]**o+(0.983503797662)*x_ref[1]+(-0.897215719033)*x_ref[2]**o+(0.434216381093)*x_ref[2]
        ref[3,4,0,1]=(0.959540582812)*x_ref[0]**o+(0.74585782554)*x_ref[0]+(0.00254441839877)*x_ref[1]**o+(0.026781582564)*x_ref[1]+(-0.0121835909437)*x_ref[2]**o+(-0.166652448502)*x_ref[2]
        ref[3,4,1,0]=(-0.875170954413)*x_ref[0]**o+(0.963228995313)*x_ref[0]+(-0.198757196024)*x_ref[1]**o+(-0.16115897441)*x_ref[1]+(0.668387545829)*x_ref[2]**o+(0.784441651668)*x_ref[2]
        ref[3,4,1,1]=(-0.0966644403845)*x_ref[0]**o+(-0.0128070214998)*x_ref[0]+(0.471059883662)*x_ref[1]**o+(0.792978130351)*x_ref[1]+(-0.709010757124)*x_ref[2]**o+(-0.535329042416)*x_ref[2]
        ref[3,4,2,0]=(0.390795392416)*x_ref[0]**o+(0.0465531497232)*x_ref[0]+(-0.665065120082)*x_ref[1]**o+(-0.637116189764)*x_ref[1]+(0.671385406225)*x_ref[2]**o+(-0.663428041906)*x_ref[2]
        ref[3,4,2,1]=(-0.962153478958)*x_ref[0]**o+(0.255601758555)*x_ref[0]+(-0.843511999734)*x_ref[1]**o+(0.677818749903)*x_ref[1]+(-0.273341883139)*x_ref[2]**o+(0.846596068215)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_Solution_rank0(self):
      """
      tests interpolate for rank 0 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from Solution onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(),dim=dim)
      s=Data(0,(),w)
      ref=Data(0,(),w_ref)
      if dim==2:
        s=(0.887413974092)*x[0]**o+(-0.7148023493)*x[0]+(0.39224588792)*x[1]**o+(0.712669303726)*x[1]
        ref=(0.887413974092)*x_ref[0]**o+(-0.7148023493)*x_ref[0]+(0.39224588792)*x_ref[1]**o+(0.712669303726)*x_ref[1]
      else:
        s=(0.435927213379)*x[0]**o+(-0.406836076828)*x[0]+(0.240829091966)*x[1]**o+(-0.782431907383)*x[1]+(0.968499178848)*x[2]**o+(0.11248632948)*x[2]
        ref=(0.435927213379)*x_ref[0]**o+(-0.406836076828)*x_ref[0]+(0.240829091966)*x_ref[1]**o+(-0.782431907383)*x_ref[1]+(0.968499178848)*x_ref[2]**o+(0.11248632948)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_Solution_rank1(self):
      """
      tests interpolate for rank 1 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from Solution onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(2,),dim=dim)
      s=Data(0,(2,),w)
      ref=Data(0,(2,),w_ref)
      if dim==2:
        s[0]=(0.497549313651)*x[0]**o+(0.342704471642)*x[0]+(0.576164780397)*x[1]**o+(0.331889751836)*x[1]
        s[1]=(0.245818716931)*x[0]**o+(0.275654781295)*x[0]+(0.444526351413)*x[1]**o+(-0.783404039317)*x[1]
        ref[0]=(0.497549313651)*x_ref[0]**o+(0.342704471642)*x_ref[0]+(0.576164780397)*x_ref[1]**o+(0.331889751836)*x_ref[1]
        ref[1]=(0.245818716931)*x_ref[0]**o+(0.275654781295)*x_ref[0]+(0.444526351413)*x_ref[1]**o+(-0.783404039317)*x_ref[1]
      else:
        s[0]=(0.234557077894)*x[0]**o+(-0.201831350239)*x[0]+(-0.869318965642)*x[1]**o+(-0.715212786062)*x[1]+(0.45304187029)*x[2]**o+(0.744681443133)*x[2]
        s[1]=(0.00903347119997)*x[0]**o+(0.746691772493)*x[0]+(-0.538042235565)*x[1]**o+(0.932594018318)*x[1]+(0.58848513662)*x[2]**o+(-0.977139295615)*x[2]
        ref[0]=(0.234557077894)*x_ref[0]**o+(-0.201831350239)*x_ref[0]+(-0.869318965642)*x_ref[1]**o+(-0.715212786062)*x_ref[1]+(0.45304187029)*x_ref[2]**o+(0.744681443133)*x_ref[2]
        ref[1]=(0.00903347119997)*x_ref[0]**o+(0.746691772493)*x_ref[0]+(-0.538042235565)*x_ref[1]**o+(0.932594018318)*x_ref[1]+(0.58848513662)*x_ref[2]**o+(-0.977139295615)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_Solution_rank2(self):
      """
      tests interpolate for rank 2 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from Solution onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5),dim=dim)
      s=Data(0,(4, 5),w)
      ref=Data(0,(4, 5),w_ref)
      if dim==2:
        s[0,0]=(-0.662397963931)*x[0]**o+(-0.398404018087)*x[0]+(-0.725256273358)*x[1]**o+(0.423211349101)*x[1]
        s[0,1]=(-0.167952506192)*x[0]**o+(-0.718540804013)*x[0]+(-0.781553467855)*x[1]**o+(0.0228480720358)*x[1]
        s[0,2]=(-0.13358806923)*x[0]**o+(0.398616852411)*x[0]+(-0.311696313693)*x[1]**o+(-0.631789739932)*x[1]
        s[0,3]=(0.855686693303)*x[0]**o+(0.630881886632)*x[0]+(-0.814763888676)*x[1]**o+(-0.614872151046)*x[1]
        s[0,4]=(0.937592800976)*x[0]**o+(-0.348259700685)*x[0]+(0.506034550337)*x[1]**o+(0.0144403536426)*x[1]
        s[1,0]=(-0.994330568705)*x[0]**o+(0.0966914505526)*x[0]+(0.353136362033)*x[1]**o+(0.241811294194)*x[1]
        s[1,1]=(-0.88991407014)*x[0]**o+(0.0439813622179)*x[0]+(0.922533826895)*x[1]**o+(0.0792543536698)*x[1]
        s[1,2]=(-0.57254546238)*x[0]**o+(0.132011764714)*x[0]+(0.919045869867)*x[1]**o+(-0.781631515229)*x[1]
        s[1,3]=(0.125157999689)*x[0]**o+(-0.239227948738)*x[0]+(-0.904449723359)*x[1]**o+(-0.726384322833)*x[1]
        s[1,4]=(0.157119099313)*x[0]**o+(0.149352102671)*x[0]+(-0.715944585517)*x[1]**o+(0.559674967195)*x[1]
        s[2,0]=(0.655983809801)*x[0]**o+(-0.0230135667792)*x[0]+(-0.963391677483)*x[1]**o+(-0.705626017783)*x[1]
        s[2,1]=(-0.405526943481)*x[0]**o+(0.528349592338)*x[0]+(0.881082721559)*x[1]**o+(0.683763434255)*x[1]
        s[2,2]=(0.747664855298)*x[0]**o+(-0.336598258345)*x[0]+(-0.15756896687)*x[1]**o+(0.657585259369)*x[1]
        s[2,3]=(-0.206021660879)*x[0]**o+(0.496139719147)*x[0]+(-0.0609353622346)*x[1]**o+(0.532090076594)*x[1]
        s[2,4]=(-0.870347013263)*x[0]**o+(0.540121560488)*x[0]+(0.373986056701)*x[1]**o+(0.586491974916)*x[1]
        s[3,0]=(-0.981237374017)*x[0]**o+(0.22435680807)*x[0]+(0.988571281429)*x[1]**o+(-0.66346591076)*x[1]
        s[3,1]=(-0.939588996339)*x[0]**o+(-0.350214307841)*x[0]+(-0.269044584808)*x[1]**o+(0.115043285059)*x[1]
        s[3,2]=(-0.401479061703)*x[0]**o+(-0.922855198541)*x[0]+(0.148477007737)*x[1]**o+(-0.64051890925)*x[1]
        s[3,3]=(0.324132581573)*x[0]**o+(0.328081603393)*x[0]+(0.938536399063)*x[1]**o+(0.742951802582)*x[1]
        s[3,4]=(-0.476640426723)*x[0]**o+(0.485808406776)*x[0]+(-0.649177668213)*x[1]**o+(0.925209215964)*x[1]
        ref[0,0]=(-0.662397963931)*x_ref[0]**o+(-0.398404018087)*x_ref[0]+(-0.725256273358)*x_ref[1]**o+(0.423211349101)*x_ref[1]
        ref[0,1]=(-0.167952506192)*x_ref[0]**o+(-0.718540804013)*x_ref[0]+(-0.781553467855)*x_ref[1]**o+(0.0228480720358)*x_ref[1]
        ref[0,2]=(-0.13358806923)*x_ref[0]**o+(0.398616852411)*x_ref[0]+(-0.311696313693)*x_ref[1]**o+(-0.631789739932)*x_ref[1]
        ref[0,3]=(0.855686693303)*x_ref[0]**o+(0.630881886632)*x_ref[0]+(-0.814763888676)*x_ref[1]**o+(-0.614872151046)*x_ref[1]
        ref[0,4]=(0.937592800976)*x_ref[0]**o+(-0.348259700685)*x_ref[0]+(0.506034550337)*x_ref[1]**o+(0.0144403536426)*x_ref[1]
        ref[1,0]=(-0.994330568705)*x_ref[0]**o+(0.0966914505526)*x_ref[0]+(0.353136362033)*x_ref[1]**o+(0.241811294194)*x_ref[1]
        ref[1,1]=(-0.88991407014)*x_ref[0]**o+(0.0439813622179)*x_ref[0]+(0.922533826895)*x_ref[1]**o+(0.0792543536698)*x_ref[1]
        ref[1,2]=(-0.57254546238)*x_ref[0]**o+(0.132011764714)*x_ref[0]+(0.919045869867)*x_ref[1]**o+(-0.781631515229)*x_ref[1]
        ref[1,3]=(0.125157999689)*x_ref[0]**o+(-0.239227948738)*x_ref[0]+(-0.904449723359)*x_ref[1]**o+(-0.726384322833)*x_ref[1]
        ref[1,4]=(0.157119099313)*x_ref[0]**o+(0.149352102671)*x_ref[0]+(-0.715944585517)*x_ref[1]**o+(0.559674967195)*x_ref[1]
        ref[2,0]=(0.655983809801)*x_ref[0]**o+(-0.0230135667792)*x_ref[0]+(-0.963391677483)*x_ref[1]**o+(-0.705626017783)*x_ref[1]
        ref[2,1]=(-0.405526943481)*x_ref[0]**o+(0.528349592338)*x_ref[0]+(0.881082721559)*x_ref[1]**o+(0.683763434255)*x_ref[1]
        ref[2,2]=(0.747664855298)*x_ref[0]**o+(-0.336598258345)*x_ref[0]+(-0.15756896687)*x_ref[1]**o+(0.657585259369)*x_ref[1]
        ref[2,3]=(-0.206021660879)*x_ref[0]**o+(0.496139719147)*x_ref[0]+(-0.0609353622346)*x_ref[1]**o+(0.532090076594)*x_ref[1]
        ref[2,4]=(-0.870347013263)*x_ref[0]**o+(0.540121560488)*x_ref[0]+(0.373986056701)*x_ref[1]**o+(0.586491974916)*x_ref[1]
        ref[3,0]=(-0.981237374017)*x_ref[0]**o+(0.22435680807)*x_ref[0]+(0.988571281429)*x_ref[1]**o+(-0.66346591076)*x_ref[1]
        ref[3,1]=(-0.939588996339)*x_ref[0]**o+(-0.350214307841)*x_ref[0]+(-0.269044584808)*x_ref[1]**o+(0.115043285059)*x_ref[1]
        ref[3,2]=(-0.401479061703)*x_ref[0]**o+(-0.922855198541)*x_ref[0]+(0.148477007737)*x_ref[1]**o+(-0.64051890925)*x_ref[1]
        ref[3,3]=(0.324132581573)*x_ref[0]**o+(0.328081603393)*x_ref[0]+(0.938536399063)*x_ref[1]**o+(0.742951802582)*x_ref[1]
        ref[3,4]=(-0.476640426723)*x_ref[0]**o+(0.485808406776)*x_ref[0]+(-0.649177668213)*x_ref[1]**o+(0.925209215964)*x_ref[1]
      else:
        s[0,0]=(-0.564914581723)*x[0]**o+(0.258795214127)*x[0]+(-0.220020909772)*x[1]**o+(0.810531687745)*x[1]+(0.454950462482)*x[2]**o+(0.601383225786)*x[2]
        s[0,1]=(-0.11600536761)*x[0]**o+(-0.690336577243)*x[0]+(-0.835351354712)*x[1]**o+(-0.158331093203)*x[1]+(0.645069697527)*x[2]**o+(0.126534640518)*x[2]
        s[0,2]=(-0.350053630252)*x[0]**o+(-0.890479764661)*x[0]+(-0.280608934641)*x[1]**o+(-0.855574890474)*x[1]+(-0.988708278729)*x[2]**o+(0.0791783723801)*x[2]
        s[0,3]=(0.987927769717)*x[0]**o+(-0.347061742182)*x[0]+(0.501082056996)*x[1]**o+(-0.708442600496)*x[1]+(0.948114708446)*x[2]**o+(0.211831323577)*x[2]
        s[0,4]=(-0.930773949241)*x[0]**o+(-0.545284390651)*x[0]+(0.198879874511)*x[1]**o+(-0.221794025714)*x[1]+(-0.447838984339)*x[2]**o+(-0.610640502396)*x[2]
        s[1,0]=(-0.156742047015)*x[0]**o+(0.843077944384)*x[0]+(-0.28777246252)*x[1]**o+(0.638858613604)*x[1]+(-0.503228005834)*x[2]**o+(-0.960025936087)*x[2]
        s[1,1]=(-0.270691254249)*x[0]**o+(0.0299500422959)*x[0]+(-0.894145166738)*x[1]**o+(0.85820171409)*x[1]+(0.61650626344)*x[2]**o+(-0.480895585635)*x[2]
        s[1,2]=(0.896861224301)*x[0]**o+(0.00816715231085)*x[0]+(-0.203903225872)*x[1]**o+(-0.126928738689)*x[1]+(-0.805120272595)*x[2]**o+(0.98588145789)*x[2]
        s[1,3]=(-0.21275125769)*x[0]**o+(0.458722054632)*x[0]+(0.153520900666)*x[1]**o+(0.512505902794)*x[1]+(0.576294451799)*x[2]**o+(0.604008619969)*x[2]
        s[1,4]=(-0.882533391878)*x[0]**o+(0.616362261559)*x[0]+(0.155750337317)*x[1]**o+(-0.0162797277104)*x[1]+(0.226933640416)*x[2]**o+(-0.118618269284)*x[2]
        s[2,0]=(-0.149844951974)*x[0]**o+(0.601452947295)*x[0]+(-0.447624109479)*x[1]**o+(-0.854580861366)*x[1]+(0.392948004882)*x[2]**o+(0.0820844858757)*x[2]
        s[2,1]=(-0.256158991676)*x[0]**o+(0.161885127371)*x[0]+(0.702594613311)*x[1]**o+(0.598617952297)*x[1]+(-0.338391564856)*x[2]**o+(0.669697240278)*x[2]
        s[2,2]=(0.441512025449)*x[0]**o+(-0.761281221569)*x[0]+(-0.445161879502)*x[1]**o+(0.268033115176)*x[1]+(-0.968003905235)*x[2]**o+(0.597022138075)*x[2]
        s[2,3]=(-0.214740575251)*x[0]**o+(0.318615769596)*x[0]+(0.554289329072)*x[1]**o+(-0.785944656086)*x[1]+(0.599828055452)*x[2]**o+(-0.221979522444)*x[2]
        s[2,4]=(-0.195666204125)*x[0]**o+(-0.0924232910586)*x[0]+(-0.442263606773)*x[1]**o+(0.114036215441)*x[1]+(-0.92582505345)*x[2]**o+(-0.138685852496)*x[2]
        s[3,0]=(0.376151237664)*x[0]**o+(0.352792436289)*x[0]+(0.755168859218)*x[1]**o+(-0.888367652073)*x[1]+(-0.0969244305571)*x[2]**o+(-0.210556734576)*x[2]
        s[3,1]=(-0.802814538265)*x[0]**o+(-0.512660557023)*x[0]+(-0.574936227852)*x[1]**o+(0.587382818046)*x[1]+(-0.500271058925)*x[2]**o+(-0.385863400163)*x[2]
        s[3,2]=(-0.487420035559)*x[0]**o+(-0.18302859198)*x[0]+(-0.477510753173)*x[1]**o+(-0.438221120428)*x[1]+(0.551062942324)*x[2]**o+(-0.0447436082813)*x[2]
        s[3,3]=(-0.934219151253)*x[0]**o+(0.398356052198)*x[0]+(0.317803529004)*x[1]**o+(0.996396715585)*x[1]+(-0.0662920794517)*x[2]**o+(-0.514553218432)*x[2]
        s[3,4]=(-0.775751006525)*x[0]**o+(-0.846022290911)*x[0]+(0.394673920051)*x[1]**o+(0.332106638437)*x[1]+(-0.925799206735)*x[2]**o+(0.908724437176)*x[2]
        ref[0,0]=(-0.564914581723)*x_ref[0]**o+(0.258795214127)*x_ref[0]+(-0.220020909772)*x_ref[1]**o+(0.810531687745)*x_ref[1]+(0.454950462482)*x_ref[2]**o+(0.601383225786)*x_ref[2]
        ref[0,1]=(-0.11600536761)*x_ref[0]**o+(-0.690336577243)*x_ref[0]+(-0.835351354712)*x_ref[1]**o+(-0.158331093203)*x_ref[1]+(0.645069697527)*x_ref[2]**o+(0.126534640518)*x_ref[2]
        ref[0,2]=(-0.350053630252)*x_ref[0]**o+(-0.890479764661)*x_ref[0]+(-0.280608934641)*x_ref[1]**o+(-0.855574890474)*x_ref[1]+(-0.988708278729)*x_ref[2]**o+(0.0791783723801)*x_ref[2]
        ref[0,3]=(0.987927769717)*x_ref[0]**o+(-0.347061742182)*x_ref[0]+(0.501082056996)*x_ref[1]**o+(-0.708442600496)*x_ref[1]+(0.948114708446)*x_ref[2]**o+(0.211831323577)*x_ref[2]
        ref[0,4]=(-0.930773949241)*x_ref[0]**o+(-0.545284390651)*x_ref[0]+(0.198879874511)*x_ref[1]**o+(-0.221794025714)*x_ref[1]+(-0.447838984339)*x_ref[2]**o+(-0.610640502396)*x_ref[2]
        ref[1,0]=(-0.156742047015)*x_ref[0]**o+(0.843077944384)*x_ref[0]+(-0.28777246252)*x_ref[1]**o+(0.638858613604)*x_ref[1]+(-0.503228005834)*x_ref[2]**o+(-0.960025936087)*x_ref[2]
        ref[1,1]=(-0.270691254249)*x_ref[0]**o+(0.0299500422959)*x_ref[0]+(-0.894145166738)*x_ref[1]**o+(0.85820171409)*x_ref[1]+(0.61650626344)*x_ref[2]**o+(-0.480895585635)*x_ref[2]
        ref[1,2]=(0.896861224301)*x_ref[0]**o+(0.00816715231085)*x_ref[0]+(-0.203903225872)*x_ref[1]**o+(-0.126928738689)*x_ref[1]+(-0.805120272595)*x_ref[2]**o+(0.98588145789)*x_ref[2]
        ref[1,3]=(-0.21275125769)*x_ref[0]**o+(0.458722054632)*x_ref[0]+(0.153520900666)*x_ref[1]**o+(0.512505902794)*x_ref[1]+(0.576294451799)*x_ref[2]**o+(0.604008619969)*x_ref[2]
        ref[1,4]=(-0.882533391878)*x_ref[0]**o+(0.616362261559)*x_ref[0]+(0.155750337317)*x_ref[1]**o+(-0.0162797277104)*x_ref[1]+(0.226933640416)*x_ref[2]**o+(-0.118618269284)*x_ref[2]
        ref[2,0]=(-0.149844951974)*x_ref[0]**o+(0.601452947295)*x_ref[0]+(-0.447624109479)*x_ref[1]**o+(-0.854580861366)*x_ref[1]+(0.392948004882)*x_ref[2]**o+(0.0820844858757)*x_ref[2]
        ref[2,1]=(-0.256158991676)*x_ref[0]**o+(0.161885127371)*x_ref[0]+(0.702594613311)*x_ref[1]**o+(0.598617952297)*x_ref[1]+(-0.338391564856)*x_ref[2]**o+(0.669697240278)*x_ref[2]
        ref[2,2]=(0.441512025449)*x_ref[0]**o+(-0.761281221569)*x_ref[0]+(-0.445161879502)*x_ref[1]**o+(0.268033115176)*x_ref[1]+(-0.968003905235)*x_ref[2]**o+(0.597022138075)*x_ref[2]
        ref[2,3]=(-0.214740575251)*x_ref[0]**o+(0.318615769596)*x_ref[0]+(0.554289329072)*x_ref[1]**o+(-0.785944656086)*x_ref[1]+(0.599828055452)*x_ref[2]**o+(-0.221979522444)*x_ref[2]
        ref[2,4]=(-0.195666204125)*x_ref[0]**o+(-0.0924232910586)*x_ref[0]+(-0.442263606773)*x_ref[1]**o+(0.114036215441)*x_ref[1]+(-0.92582505345)*x_ref[2]**o+(-0.138685852496)*x_ref[2]
        ref[3,0]=(0.376151237664)*x_ref[0]**o+(0.352792436289)*x_ref[0]+(0.755168859218)*x_ref[1]**o+(-0.888367652073)*x_ref[1]+(-0.0969244305571)*x_ref[2]**o+(-0.210556734576)*x_ref[2]
        ref[3,1]=(-0.802814538265)*x_ref[0]**o+(-0.512660557023)*x_ref[0]+(-0.574936227852)*x_ref[1]**o+(0.587382818046)*x_ref[1]+(-0.500271058925)*x_ref[2]**o+(-0.385863400163)*x_ref[2]
        ref[3,2]=(-0.487420035559)*x_ref[0]**o+(-0.18302859198)*x_ref[0]+(-0.477510753173)*x_ref[1]**o+(-0.438221120428)*x_ref[1]+(0.551062942324)*x_ref[2]**o+(-0.0447436082813)*x_ref[2]
        ref[3,3]=(-0.934219151253)*x_ref[0]**o+(0.398356052198)*x_ref[0]+(0.317803529004)*x_ref[1]**o+(0.996396715585)*x_ref[1]+(-0.0662920794517)*x_ref[2]**o+(-0.514553218432)*x_ref[2]
        ref[3,4]=(-0.775751006525)*x_ref[0]**o+(-0.846022290911)*x_ref[0]+(0.394673920051)*x_ref[1]**o+(0.332106638437)*x_ref[1]+(-0.925799206735)*x_ref[2]**o+(0.908724437176)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_Solution_rank3(self):
      """
      tests interpolate for rank 3 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from Solution onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(6, 2, 2),dim=dim)
      s=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2),w_ref)
      if dim==2:
        s[0,0,0]=(0.166904084074)*x[0]**o+(0.528859695436)*x[0]+(0.767492755038)*x[1]**o+(0.219756088661)*x[1]
        s[0,0,1]=(0.116354249122)*x[0]**o+(-0.489054596883)*x[0]+(0.843432052987)*x[1]**o+(0.792328551842)*x[1]
        s[0,1,0]=(0.316210380258)*x[0]**o+(0.567201270085)*x[0]+(-0.998412810858)*x[1]**o+(-0.523766343794)*x[1]
        s[0,1,1]=(-0.131361133794)*x[0]**o+(-0.415781217922)*x[0]+(-0.743444932389)*x[1]**o+(0.0440356388227)*x[1]
        s[1,0,0]=(-0.13886008398)*x[0]**o+(0.94188356362)*x[0]+(-0.310067426785)*x[1]**o+(-0.0123016016596)*x[1]
        s[1,0,1]=(-0.451586142333)*x[0]**o+(-0.00942276847807)*x[0]+(0.141967698659)*x[1]**o+(0.0763178193995)*x[1]
        s[1,1,0]=(-0.959190563677)*x[0]**o+(0.954437590409)*x[0]+(0.862816899262)*x[1]**o+(-0.849746515492)*x[1]
        s[1,1,1]=(0.277945454335)*x[0]**o+(-0.269801922972)*x[0]+(0.991405174196)*x[1]**o+(-0.481494765175)*x[1]
        s[2,0,0]=(-0.449455660138)*x[0]**o+(0.903463595246)*x[0]+(-0.796159239153)*x[1]**o+(0.178892715811)*x[1]
        s[2,0,1]=(-0.361748699382)*x[0]**o+(-0.380839684794)*x[0]+(0.167962669517)*x[1]**o+(-0.657300482681)*x[1]
        s[2,1,0]=(-0.00835791778047)*x[0]**o+(-0.964628782216)*x[0]+(-0.242890776847)*x[1]**o+(0.580549752444)*x[1]
        s[2,1,1]=(0.31424829894)*x[0]**o+(-0.902559989702)*x[0]+(0.11028478987)*x[1]**o+(-0.408366814277)*x[1]
        s[3,0,0]=(0.0269640344016)*x[0]**o+(0.742724075544)*x[0]+(-0.275698627658)*x[1]**o+(-0.666351552089)*x[1]
        s[3,0,1]=(0.634533702145)*x[0]**o+(-0.962956780064)*x[0]+(-0.117282164464)*x[1]**o+(0.116533409993)*x[1]
        s[3,1,0]=(0.0889039574007)*x[0]**o+(-0.302373214302)*x[0]+(0.0840411077509)*x[1]**o+(0.938857669151)*x[1]
        s[3,1,1]=(-0.814069429474)*x[0]**o+(-0.157632753675)*x[0]+(0.0908314139661)*x[1]**o+(0.918730301934)*x[1]
        s[4,0,0]=(0.535565487565)*x[0]**o+(-0.136652909062)*x[0]+(0.805198026614)*x[1]**o+(0.643003183224)*x[1]
        s[4,0,1]=(-0.80530904582)*x[0]**o+(0.966761926917)*x[0]+(0.427549028006)*x[1]**o+(0.262827083069)*x[1]
        s[4,1,0]=(-0.71456682036)*x[0]**o+(-0.0739127178784)*x[0]+(-0.438432370229)*x[1]**o+(-0.550981212497)*x[1]
        s[4,1,1]=(-0.536067456171)*x[0]**o+(0.343971454511)*x[0]+(-0.738912708731)*x[1]**o+(-0.721452063307)*x[1]
        s[5,0,0]=(-0.474192484949)*x[0]**o+(0.202037328764)*x[0]+(0.899258010081)*x[1]**o+(0.409255983049)*x[1]
        s[5,0,1]=(-0.0106650866193)*x[0]**o+(0.16386057287)*x[0]+(-0.027569820513)*x[1]**o+(0.49082096942)*x[1]
        s[5,1,0]=(0.888476355004)*x[0]**o+(-0.605284684678)*x[0]+(-0.767736685512)*x[1]**o+(0.443614918182)*x[1]
        s[5,1,1]=(0.538075122922)*x[0]**o+(0.0902918352337)*x[0]+(-0.848299366243)*x[1]**o+(-0.222800083729)*x[1]
        ref[0,0,0]=(0.166904084074)*x_ref[0]**o+(0.528859695436)*x_ref[0]+(0.767492755038)*x_ref[1]**o+(0.219756088661)*x_ref[1]
        ref[0,0,1]=(0.116354249122)*x_ref[0]**o+(-0.489054596883)*x_ref[0]+(0.843432052987)*x_ref[1]**o+(0.792328551842)*x_ref[1]
        ref[0,1,0]=(0.316210380258)*x_ref[0]**o+(0.567201270085)*x_ref[0]+(-0.998412810858)*x_ref[1]**o+(-0.523766343794)*x_ref[1]
        ref[0,1,1]=(-0.131361133794)*x_ref[0]**o+(-0.415781217922)*x_ref[0]+(-0.743444932389)*x_ref[1]**o+(0.0440356388227)*x_ref[1]
        ref[1,0,0]=(-0.13886008398)*x_ref[0]**o+(0.94188356362)*x_ref[0]+(-0.310067426785)*x_ref[1]**o+(-0.0123016016596)*x_ref[1]
        ref[1,0,1]=(-0.451586142333)*x_ref[0]**o+(-0.00942276847807)*x_ref[0]+(0.141967698659)*x_ref[1]**o+(0.0763178193995)*x_ref[1]
        ref[1,1,0]=(-0.959190563677)*x_ref[0]**o+(0.954437590409)*x_ref[0]+(0.862816899262)*x_ref[1]**o+(-0.849746515492)*x_ref[1]
        ref[1,1,1]=(0.277945454335)*x_ref[0]**o+(-0.269801922972)*x_ref[0]+(0.991405174196)*x_ref[1]**o+(-0.481494765175)*x_ref[1]
        ref[2,0,0]=(-0.449455660138)*x_ref[0]**o+(0.903463595246)*x_ref[0]+(-0.796159239153)*x_ref[1]**o+(0.178892715811)*x_ref[1]
        ref[2,0,1]=(-0.361748699382)*x_ref[0]**o+(-0.380839684794)*x_ref[0]+(0.167962669517)*x_ref[1]**o+(-0.657300482681)*x_ref[1]
        ref[2,1,0]=(-0.00835791778047)*x_ref[0]**o+(-0.964628782216)*x_ref[0]+(-0.242890776847)*x_ref[1]**o+(0.580549752444)*x_ref[1]
        ref[2,1,1]=(0.31424829894)*x_ref[0]**o+(-0.902559989702)*x_ref[0]+(0.11028478987)*x_ref[1]**o+(-0.408366814277)*x_ref[1]
        ref[3,0,0]=(0.0269640344016)*x_ref[0]**o+(0.742724075544)*x_ref[0]+(-0.275698627658)*x_ref[1]**o+(-0.666351552089)*x_ref[1]
        ref[3,0,1]=(0.634533702145)*x_ref[0]**o+(-0.962956780064)*x_ref[0]+(-0.117282164464)*x_ref[1]**o+(0.116533409993)*x_ref[1]
        ref[3,1,0]=(0.0889039574007)*x_ref[0]**o+(-0.302373214302)*x_ref[0]+(0.0840411077509)*x_ref[1]**o+(0.938857669151)*x_ref[1]
        ref[3,1,1]=(-0.814069429474)*x_ref[0]**o+(-0.157632753675)*x_ref[0]+(0.0908314139661)*x_ref[1]**o+(0.918730301934)*x_ref[1]
        ref[4,0,0]=(0.535565487565)*x_ref[0]**o+(-0.136652909062)*x_ref[0]+(0.805198026614)*x_ref[1]**o+(0.643003183224)*x_ref[1]
        ref[4,0,1]=(-0.80530904582)*x_ref[0]**o+(0.966761926917)*x_ref[0]+(0.427549028006)*x_ref[1]**o+(0.262827083069)*x_ref[1]
        ref[4,1,0]=(-0.71456682036)*x_ref[0]**o+(-0.0739127178784)*x_ref[0]+(-0.438432370229)*x_ref[1]**o+(-0.550981212497)*x_ref[1]
        ref[4,1,1]=(-0.536067456171)*x_ref[0]**o+(0.343971454511)*x_ref[0]+(-0.738912708731)*x_ref[1]**o+(-0.721452063307)*x_ref[1]
        ref[5,0,0]=(-0.474192484949)*x_ref[0]**o+(0.202037328764)*x_ref[0]+(0.899258010081)*x_ref[1]**o+(0.409255983049)*x_ref[1]
        ref[5,0,1]=(-0.0106650866193)*x_ref[0]**o+(0.16386057287)*x_ref[0]+(-0.027569820513)*x_ref[1]**o+(0.49082096942)*x_ref[1]
        ref[5,1,0]=(0.888476355004)*x_ref[0]**o+(-0.605284684678)*x_ref[0]+(-0.767736685512)*x_ref[1]**o+(0.443614918182)*x_ref[1]
        ref[5,1,1]=(0.538075122922)*x_ref[0]**o+(0.0902918352337)*x_ref[0]+(-0.848299366243)*x_ref[1]**o+(-0.222800083729)*x_ref[1]
      else:
        s[0,0,0]=(0.100353793751)*x[0]**o+(-0.00847651366844)*x[0]+(-0.916659095312)*x[1]**o+(0.43123854291)*x[1]+(-0.325548712866)*x[2]**o+(0.436984953031)*x[2]
        s[0,0,1]=(-0.450705520207)*x[0]**o+(-0.0420094946238)*x[0]+(0.0471679698967)*x[1]**o+(-0.666536961424)*x[1]+(0.629675854589)*x[2]**o+(0.789343254309)*x[2]
        s[0,1,0]=(-0.753618765099)*x[0]**o+(0.327794192265)*x[0]+(-0.230331681868)*x[1]**o+(0.414256616776)*x[1]+(0.861119329276)*x[2]**o+(-0.663194921545)*x[2]
        s[0,1,1]=(-0.897978378096)*x[0]**o+(-0.758509805972)*x[0]+(0.527577710213)*x[1]**o+(-0.668678399218)*x[1]+(0.274556839547)*x[2]**o+(0.838582981011)*x[2]
        s[1,0,0]=(-0.983732811569)*x[0]**o+(0.681831497881)*x[0]+(-0.834619144249)*x[1]**o+(-0.585649970853)*x[1]+(-0.443723853248)*x[2]**o+(-0.24230119834)*x[2]
        s[1,0,1]=(0.223700452689)*x[0]**o+(-0.976082665433)*x[0]+(0.682274036244)*x[1]**o+(0.513516465519)*x[1]+(0.604542635213)*x[2]**o+(0.461982295722)*x[2]
        s[1,1,0]=(-0.124528740755)*x[0]**o+(0.982942403581)*x[0]+(0.168657059059)*x[1]**o+(0.468525342776)*x[1]+(-0.628181131076)*x[2]**o+(-0.351466961112)*x[2]
        s[1,1,1]=(-0.147571648722)*x[0]**o+(0.823854564267)*x[0]+(0.808529030428)*x[1]**o+(0.809937338033)*x[1]+(0.86059984675)*x[2]**o+(-0.245256089602)*x[2]
        s[2,0,0]=(-0.421589109694)*x[0]**o+(0.739843964299)*x[0]+(-0.807211114691)*x[1]**o+(0.926253044589)*x[1]+(0.0247961654116)*x[2]**o+(0.372268514736)*x[2]
        s[2,0,1]=(-0.271986963767)*x[0]**o+(0.653599242745)*x[0]+(-0.950340669539)*x[1]**o+(0.486306280058)*x[1]+(-0.289166423358)*x[2]**o+(-0.05193840419)*x[2]
        s[2,1,0]=(-0.439847329001)*x[0]**o+(-0.0414982844889)*x[0]+(0.724606145337)*x[1]**o+(0.800164759919)*x[1]+(-0.948978159606)*x[2]**o+(0.330443199192)*x[2]
        s[2,1,1]=(0.145027947835)*x[0]**o+(-0.227814589191)*x[0]+(0.0969921713223)*x[1]**o+(0.70541078023)*x[1]+(0.536631305638)*x[2]**o+(0.761601818309)*x[2]
        s[3,0,0]=(-0.382247778154)*x[0]**o+(0.0243544380509)*x[0]+(0.515439251148)*x[1]**o+(0.531106239532)*x[1]+(-0.303961766943)*x[2]**o+(0.463673819842)*x[2]
        s[3,0,1]=(0.375551089648)*x[0]**o+(-0.0674026512267)*x[0]+(0.0797417786103)*x[1]**o+(0.90997116985)*x[1]+(-0.613366372871)*x[2]**o+(-0.0981062035417)*x[2]
        s[3,1,0]=(-0.467358089029)*x[0]**o+(-0.656202312608)*x[0]+(-0.00790864600562)*x[1]**o+(0.501618787933)*x[1]+(0.118835067496)*x[2]**o+(0.253600511775)*x[2]
        s[3,1,1]=(-0.0613387848811)*x[0]**o+(0.0555375789746)*x[0]+(0.467925348711)*x[1]**o+(-0.521361813902)*x[1]+(-0.890969702071)*x[2]**o+(-0.0588424357829)*x[2]
        s[4,0,0]=(-0.585761522751)*x[0]**o+(0.362766428544)*x[0]+(0.405833998664)*x[1]**o+(0.927768276759)*x[1]+(-0.515236556184)*x[2]**o+(0.903733254638)*x[2]
        s[4,0,1]=(-0.443373751826)*x[0]**o+(0.589695272429)*x[0]+(-0.547332598122)*x[1]**o+(0.547140256879)*x[1]+(0.0276104869591)*x[2]**o+(-0.00901656145103)*x[2]
        s[4,1,0]=(0.327714447478)*x[0]**o+(0.333181860896)*x[0]+(0.00402719775545)*x[1]**o+(0.119030826449)*x[1]+(0.121265643664)*x[2]**o+(0.3209309716)*x[2]
        s[4,1,1]=(-0.428606778799)*x[0]**o+(-0.0240488914911)*x[0]+(-0.617222089407)*x[1]**o+(0.326500165958)*x[1]+(0.695161972341)*x[2]**o+(-0.674385407299)*x[2]
        s[5,0,0]=(0.96840017886)*x[0]**o+(0.977460834915)*x[0]+(-0.0847928163657)*x[1]**o+(-0.289814405309)*x[1]+(-0.556056635259)*x[2]**o+(-0.127899784078)*x[2]
        s[5,0,1]=(-0.535056372252)*x[0]**o+(-0.557336923293)*x[0]+(-0.982050329504)*x[1]**o+(-0.186847262795)*x[1]+(0.859894693504)*x[2]**o+(-0.993989085549)*x[2]
        s[5,1,0]=(0.846310277081)*x[0]**o+(-0.996136776377)*x[0]+(0.221846701921)*x[1]**o+(0.947752560707)*x[1]+(-0.674009316486)*x[2]**o+(0.850085324498)*x[2]
        s[5,1,1]=(-0.0244426977213)*x[0]**o+(-0.768717160785)*x[0]+(0.42356601994)*x[1]**o+(-0.359166983456)*x[1]+(-0.671384564312)*x[2]**o+(0.850466416962)*x[2]
        ref[0,0,0]=(0.100353793751)*x_ref[0]**o+(-0.00847651366844)*x_ref[0]+(-0.916659095312)*x_ref[1]**o+(0.43123854291)*x_ref[1]+(-0.325548712866)*x_ref[2]**o+(0.436984953031)*x_ref[2]
        ref[0,0,1]=(-0.450705520207)*x_ref[0]**o+(-0.0420094946238)*x_ref[0]+(0.0471679698967)*x_ref[1]**o+(-0.666536961424)*x_ref[1]+(0.629675854589)*x_ref[2]**o+(0.789343254309)*x_ref[2]
        ref[0,1,0]=(-0.753618765099)*x_ref[0]**o+(0.327794192265)*x_ref[0]+(-0.230331681868)*x_ref[1]**o+(0.414256616776)*x_ref[1]+(0.861119329276)*x_ref[2]**o+(-0.663194921545)*x_ref[2]
        ref[0,1,1]=(-0.897978378096)*x_ref[0]**o+(-0.758509805972)*x_ref[0]+(0.527577710213)*x_ref[1]**o+(-0.668678399218)*x_ref[1]+(0.274556839547)*x_ref[2]**o+(0.838582981011)*x_ref[2]
        ref[1,0,0]=(-0.983732811569)*x_ref[0]**o+(0.681831497881)*x_ref[0]+(-0.834619144249)*x_ref[1]**o+(-0.585649970853)*x_ref[1]+(-0.443723853248)*x_ref[2]**o+(-0.24230119834)*x_ref[2]
        ref[1,0,1]=(0.223700452689)*x_ref[0]**o+(-0.976082665433)*x_ref[0]+(0.682274036244)*x_ref[1]**o+(0.513516465519)*x_ref[1]+(0.604542635213)*x_ref[2]**o+(0.461982295722)*x_ref[2]
        ref[1,1,0]=(-0.124528740755)*x_ref[0]**o+(0.982942403581)*x_ref[0]+(0.168657059059)*x_ref[1]**o+(0.468525342776)*x_ref[1]+(-0.628181131076)*x_ref[2]**o+(-0.351466961112)*x_ref[2]
        ref[1,1,1]=(-0.147571648722)*x_ref[0]**o+(0.823854564267)*x_ref[0]+(0.808529030428)*x_ref[1]**o+(0.809937338033)*x_ref[1]+(0.86059984675)*x_ref[2]**o+(-0.245256089602)*x_ref[2]
        ref[2,0,0]=(-0.421589109694)*x_ref[0]**o+(0.739843964299)*x_ref[0]+(-0.807211114691)*x_ref[1]**o+(0.926253044589)*x_ref[1]+(0.0247961654116)*x_ref[2]**o+(0.372268514736)*x_ref[2]
        ref[2,0,1]=(-0.271986963767)*x_ref[0]**o+(0.653599242745)*x_ref[0]+(-0.950340669539)*x_ref[1]**o+(0.486306280058)*x_ref[1]+(-0.289166423358)*x_ref[2]**o+(-0.05193840419)*x_ref[2]
        ref[2,1,0]=(-0.439847329001)*x_ref[0]**o+(-0.0414982844889)*x_ref[0]+(0.724606145337)*x_ref[1]**o+(0.800164759919)*x_ref[1]+(-0.948978159606)*x_ref[2]**o+(0.330443199192)*x_ref[2]
        ref[2,1,1]=(0.145027947835)*x_ref[0]**o+(-0.227814589191)*x_ref[0]+(0.0969921713223)*x_ref[1]**o+(0.70541078023)*x_ref[1]+(0.536631305638)*x_ref[2]**o+(0.761601818309)*x_ref[2]
        ref[3,0,0]=(-0.382247778154)*x_ref[0]**o+(0.0243544380509)*x_ref[0]+(0.515439251148)*x_ref[1]**o+(0.531106239532)*x_ref[1]+(-0.303961766943)*x_ref[2]**o+(0.463673819842)*x_ref[2]
        ref[3,0,1]=(0.375551089648)*x_ref[0]**o+(-0.0674026512267)*x_ref[0]+(0.0797417786103)*x_ref[1]**o+(0.90997116985)*x_ref[1]+(-0.613366372871)*x_ref[2]**o+(-0.0981062035417)*x_ref[2]
        ref[3,1,0]=(-0.467358089029)*x_ref[0]**o+(-0.656202312608)*x_ref[0]+(-0.00790864600562)*x_ref[1]**o+(0.501618787933)*x_ref[1]+(0.118835067496)*x_ref[2]**o+(0.253600511775)*x_ref[2]
        ref[3,1,1]=(-0.0613387848811)*x_ref[0]**o+(0.0555375789746)*x_ref[0]+(0.467925348711)*x_ref[1]**o+(-0.521361813902)*x_ref[1]+(-0.890969702071)*x_ref[2]**o+(-0.0588424357829)*x_ref[2]
        ref[4,0,0]=(-0.585761522751)*x_ref[0]**o+(0.362766428544)*x_ref[0]+(0.405833998664)*x_ref[1]**o+(0.927768276759)*x_ref[1]+(-0.515236556184)*x_ref[2]**o+(0.903733254638)*x_ref[2]
        ref[4,0,1]=(-0.443373751826)*x_ref[0]**o+(0.589695272429)*x_ref[0]+(-0.547332598122)*x_ref[1]**o+(0.547140256879)*x_ref[1]+(0.0276104869591)*x_ref[2]**o+(-0.00901656145103)*x_ref[2]
        ref[4,1,0]=(0.327714447478)*x_ref[0]**o+(0.333181860896)*x_ref[0]+(0.00402719775545)*x_ref[1]**o+(0.119030826449)*x_ref[1]+(0.121265643664)*x_ref[2]**o+(0.3209309716)*x_ref[2]
        ref[4,1,1]=(-0.428606778799)*x_ref[0]**o+(-0.0240488914911)*x_ref[0]+(-0.617222089407)*x_ref[1]**o+(0.326500165958)*x_ref[1]+(0.695161972341)*x_ref[2]**o+(-0.674385407299)*x_ref[2]
        ref[5,0,0]=(0.96840017886)*x_ref[0]**o+(0.977460834915)*x_ref[0]+(-0.0847928163657)*x_ref[1]**o+(-0.289814405309)*x_ref[1]+(-0.556056635259)*x_ref[2]**o+(-0.127899784078)*x_ref[2]
        ref[5,0,1]=(-0.535056372252)*x_ref[0]**o+(-0.557336923293)*x_ref[0]+(-0.982050329504)*x_ref[1]**o+(-0.186847262795)*x_ref[1]+(0.859894693504)*x_ref[2]**o+(-0.993989085549)*x_ref[2]
        ref[5,1,0]=(0.846310277081)*x_ref[0]**o+(-0.996136776377)*x_ref[0]+(0.221846701921)*x_ref[1]**o+(0.947752560707)*x_ref[1]+(-0.674009316486)*x_ref[2]**o+(0.850085324498)*x_ref[2]
        ref[5,1,1]=(-0.0244426977213)*x_ref[0]**o+(-0.768717160785)*x_ref[0]+(0.42356601994)*x_ref[1]**o+(-0.359166983456)*x_ref[1]+(-0.671384564312)*x_ref[2]**o+(0.850466416962)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_Solution_rank4(self):
      """
      tests interpolate for rank 4 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from Solution onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5, 3, 2),dim=dim)
      s=Data(0,(4, 5, 3, 2),w)
      ref=Data(0,(4, 5, 3, 2),w_ref)
      if dim==2:
        s[0,0,0,0]=(-0.099815403496)*x[0]**o+(-0.918641975262)*x[0]+(-0.643800270537)*x[1]**o+(-0.188947131246)*x[1]
        s[0,0,0,1]=(-0.654158499426)*x[0]**o+(-0.158917070952)*x[0]+(0.482580361439)*x[1]**o+(0.654085120986)*x[1]
        s[0,0,1,0]=(-0.82639526996)*x[0]**o+(-0.852788225935)*x[0]+(-0.883559061247)*x[1]**o+(-0.762093398747)*x[1]
        s[0,0,1,1]=(-0.410064006917)*x[0]**o+(-0.358595400023)*x[0]+(-0.744803691415)*x[1]**o+(-0.418642435874)*x[1]
        s[0,0,2,0]=(0.627425019462)*x[0]**o+(-0.759635228096)*x[0]+(-0.225478401461)*x[1]**o+(-0.394678152175)*x[1]
        s[0,0,2,1]=(0.282477965976)*x[0]**o+(0.591668341745)*x[0]+(0.997081550686)*x[1]**o+(-0.16137686071)*x[1]
        s[0,1,0,0]=(0.503328443933)*x[0]**o+(-0.349809904995)*x[0]+(0.675840382965)*x[1]**o+(0.122847940652)*x[1]
        s[0,1,0,1]=(-0.693842907429)*x[0]**o+(0.242153763819)*x[0]+(0.669516208483)*x[1]**o+(-0.966639099473)*x[1]
        s[0,1,1,0]=(-0.143996595411)*x[0]**o+(-0.0789367221661)*x[0]+(0.44978165828)*x[1]**o+(0.982272605823)*x[1]
        s[0,1,1,1]=(0.217504837315)*x[0]**o+(0.62345834313)*x[0]+(0.421663416031)*x[1]**o+(0.298214724947)*x[1]
        s[0,1,2,0]=(-0.192788493597)*x[0]**o+(0.59360099119)*x[0]+(0.798640786481)*x[1]**o+(-0.0618029984487)*x[1]
        s[0,1,2,1]=(-0.448474976435)*x[0]**o+(-0.136832009015)*x[0]+(-0.597189414931)*x[1]**o+(0.502732362863)*x[1]
        s[0,2,0,0]=(0.252384046343)*x[0]**o+(-0.400123690831)*x[0]+(-0.545481308678)*x[1]**o+(-0.388790715614)*x[1]
        s[0,2,0,1]=(-0.66240802679)*x[0]**o+(0.77064500558)*x[0]+(0.5239431782)*x[1]**o+(0.467410945489)*x[1]
        s[0,2,1,0]=(-0.85322788336)*x[0]**o+(-0.690030962118)*x[0]+(0.699888315472)*x[1]**o+(-0.89805156241)*x[1]
        s[0,2,1,1]=(-0.255889046261)*x[0]**o+(0.0541259188422)*x[0]+(-0.749587408291)*x[1]**o+(0.791965754891)*x[1]
        s[0,2,2,0]=(0.496462410445)*x[0]**o+(0.772074605719)*x[0]+(0.180850727108)*x[1]**o+(-0.487496085214)*x[1]
        s[0,2,2,1]=(0.706055709638)*x[0]**o+(-0.598906910405)*x[0]+(-0.152966385845)*x[1]**o+(0.871383558924)*x[1]
        s[0,3,0,0]=(-0.951858396669)*x[0]**o+(-0.794420515046)*x[0]+(0.834659431502)*x[1]**o+(0.481994252391)*x[1]
        s[0,3,0,1]=(-0.578325718357)*x[0]**o+(-0.843177109188)*x[0]+(0.38052630466)*x[1]**o+(0.0999564588392)*x[1]
        s[0,3,1,0]=(0.251016293115)*x[0]**o+(-0.846784412909)*x[0]+(0.615130638894)*x[1]**o+(-0.545472677499)*x[1]
        s[0,3,1,1]=(0.0242041003245)*x[0]**o+(-0.558470673629)*x[0]+(0.0355389532695)*x[1]**o+(-0.82512767128)*x[1]
        s[0,3,2,0]=(0.104901847054)*x[0]**o+(0.176002645436)*x[0]+(-0.0529040105457)*x[1]**o+(-0.497309382471)*x[1]
        s[0,3,2,1]=(-0.19187265394)*x[0]**o+(-0.459102146282)*x[0]+(0.263116634482)*x[1]**o+(0.573723666261)*x[1]
        s[0,4,0,0]=(-0.605664936504)*x[0]**o+(0.236623083899)*x[0]+(-0.573965234185)*x[1]**o+(-0.191541238111)*x[1]
        s[0,4,0,1]=(0.735297405377)*x[0]**o+(-0.863752806793)*x[0]+(0.130990772115)*x[1]**o+(0.153051776075)*x[1]
        s[0,4,1,0]=(0.83806722488)*x[0]**o+(0.952102086801)*x[0]+(0.921278259866)*x[1]**o+(0.100889262792)*x[1]
        s[0,4,1,1]=(0.422551160597)*x[0]**o+(-0.68105266886)*x[0]+(-0.0230668740384)*x[1]**o+(0.608853763751)*x[1]
        s[0,4,2,0]=(-0.509106957279)*x[0]**o+(0.535714529222)*x[0]+(0.407059274864)*x[1]**o+(0.0304780664648)*x[1]
        s[0,4,2,1]=(-0.899880077882)*x[0]**o+(-0.150489820435)*x[0]+(-0.329228777137)*x[1]**o+(-0.325670285955)*x[1]
        s[1,0,0,0]=(0.723148414781)*x[0]**o+(0.276861527427)*x[0]+(-0.457164017461)*x[1]**o+(0.587623869716)*x[1]
        s[1,0,0,1]=(-0.656042247607)*x[0]**o+(0.599876456972)*x[0]+(-0.487723393567)*x[1]**o+(-0.293330841893)*x[1]
        s[1,0,1,0]=(0.498385487237)*x[0]**o+(0.823826806076)*x[0]+(-0.260001108163)*x[1]**o+(-0.520083127954)*x[1]
        s[1,0,1,1]=(0.0511021547178)*x[0]**o+(0.0752387451382)*x[0]+(-0.295018309613)*x[1]**o+(0.410655676791)*x[1]
        s[1,0,2,0]=(-0.926038025877)*x[0]**o+(-0.101726746627)*x[0]+(-0.714060837737)*x[1]**o+(0.291653139474)*x[1]
        s[1,0,2,1]=(0.416231421476)*x[0]**o+(0.968331479531)*x[0]+(0.13575381957)*x[1]**o+(0.0581635556746)*x[1]
        s[1,1,0,0]=(0.483218423871)*x[0]**o+(0.837199391237)*x[0]+(-0.332232511767)*x[1]**o+(0.861714372392)*x[1]
        s[1,1,0,1]=(-0.861985810928)*x[0]**o+(0.704494207363)*x[0]+(0.0120560011404)*x[1]**o+(-0.922350499146)*x[1]
        s[1,1,1,0]=(0.710471809896)*x[0]**o+(0.10838308437)*x[0]+(-0.936553657991)*x[1]**o+(-0.157956217106)*x[1]
        s[1,1,1,1]=(0.61604545132)*x[0]**o+(0.161202252636)*x[0]+(-0.208993778286)*x[1]**o+(0.150797102116)*x[1]
        s[1,1,2,0]=(0.237396232488)*x[0]**o+(-0.260415056277)*x[0]+(-0.760219527005)*x[1]**o+(0.203694444805)*x[1]
        s[1,1,2,1]=(-0.656645187501)*x[0]**o+(-0.84629755936)*x[0]+(-0.465896055143)*x[1]**o+(0.335524534826)*x[1]
        s[1,2,0,0]=(-0.546393675559)*x[0]**o+(0.0344401682615)*x[0]+(-0.267054613816)*x[1]**o+(-0.345211291755)*x[1]
        s[1,2,0,1]=(0.807935620456)*x[0]**o+(0.89820026562)*x[0]+(-0.471804773843)*x[1]**o+(0.0183931405417)*x[1]
        s[1,2,1,0]=(0.347457777403)*x[0]**o+(0.894578611023)*x[0]+(0.506086327041)*x[1]**o+(-0.0552731455772)*x[1]
        s[1,2,1,1]=(0.898402940142)*x[0]**o+(-0.850905749447)*x[0]+(0.421790306199)*x[1]**o+(0.560435305462)*x[1]
        s[1,2,2,0]=(0.417854170807)*x[0]**o+(0.0808123920631)*x[0]+(0.685114818766)*x[1]**o+(0.627248749954)*x[1]
        s[1,2,2,1]=(0.608689720682)*x[0]**o+(-0.215111751974)*x[0]+(-0.157391477249)*x[1]**o+(-0.157398505688)*x[1]
        s[1,3,0,0]=(0.453375349378)*x[0]**o+(-0.232490378969)*x[0]+(0.370581316376)*x[1]**o+(0.2321645519)*x[1]
        s[1,3,0,1]=(-0.921453677654)*x[0]**o+(-0.272608399386)*x[0]+(0.980108178061)*x[1]**o+(0.0304202383868)*x[1]
        s[1,3,1,0]=(-0.703951869845)*x[0]**o+(0.232888375306)*x[0]+(0.737108419956)*x[1]**o+(-0.815317463111)*x[1]
        s[1,3,1,1]=(-0.849808179245)*x[0]**o+(-0.848051887981)*x[0]+(0.452157961656)*x[1]**o+(0.942533763098)*x[1]
        s[1,3,2,0]=(0.276310583942)*x[0]**o+(-0.426217969193)*x[0]+(0.383056703597)*x[1]**o+(-0.100608872707)*x[1]
        s[1,3,2,1]=(0.792743379061)*x[0]**o+(-0.638898354)*x[0]+(0.0900755365083)*x[1]**o+(0.346700761651)*x[1]
        s[1,4,0,0]=(0.227940749448)*x[0]**o+(-0.723141488706)*x[0]+(-0.597539261807)*x[1]**o+(-0.0547634570298)*x[1]
        s[1,4,0,1]=(-0.00150193441347)*x[0]**o+(-0.875771867697)*x[0]+(0.554295315299)*x[1]**o+(0.179291021879)*x[1]
        s[1,4,1,0]=(0.665263753178)*x[0]**o+(-0.261999229902)*x[0]+(0.177708351596)*x[1]**o+(-0.71011990376)*x[1]
        s[1,4,1,1]=(0.416094331837)*x[0]**o+(0.809191029905)*x[0]+(-0.631251155279)*x[1]**o+(0.758593347696)*x[1]
        s[1,4,2,0]=(0.50941363283)*x[0]**o+(-0.652536088555)*x[0]+(-0.0994076235125)*x[1]**o+(0.186614952474)*x[1]
        s[1,4,2,1]=(0.129263234225)*x[0]**o+(0.115870119224)*x[0]+(-0.25581908442)*x[1]**o+(0.295990548791)*x[1]
        s[2,0,0,0]=(-0.432850417862)*x[0]**o+(-0.491964804361)*x[0]+(0.0106143845453)*x[1]**o+(0.787362504175)*x[1]
        s[2,0,0,1]=(0.31617556259)*x[0]**o+(0.255445781486)*x[0]+(-0.302678692069)*x[1]**o+(0.840840019713)*x[1]
        s[2,0,1,0]=(-0.855359978332)*x[0]**o+(0.410554550223)*x[0]+(-0.703253338388)*x[1]**o+(0.59010970561)*x[1]
        s[2,0,1,1]=(-0.573270259685)*x[0]**o+(0.928396842624)*x[0]+(0.94155331127)*x[1]**o+(0.160140257961)*x[1]
        s[2,0,2,0]=(0.0524505638003)*x[0]**o+(-0.736508989499)*x[0]+(-0.565060891384)*x[1]**o+(-0.613736274563)*x[1]
        s[2,0,2,1]=(-0.776619329318)*x[0]**o+(0.485104655735)*x[0]+(-0.135863414015)*x[1]**o+(-0.14173677679)*x[1]
        s[2,1,0,0]=(0.662345907479)*x[0]**o+(-0.900993470049)*x[0]+(-0.882752065723)*x[1]**o+(0.642775104186)*x[1]
        s[2,1,0,1]=(-0.189295597666)*x[0]**o+(0.0165605763413)*x[0]+(0.989119296804)*x[1]**o+(0.262678778013)*x[1]
        s[2,1,1,0]=(0.927673133577)*x[0]**o+(-0.0248539427539)*x[0]+(0.504610255651)*x[1]**o+(-0.593911741472)*x[1]
        s[2,1,1,1]=(0.899525029448)*x[0]**o+(-0.97195963201)*x[0]+(-0.461931155897)*x[1]**o+(0.91354314585)*x[1]
        s[2,1,2,0]=(0.398654077456)*x[0]**o+(-0.429950726187)*x[0]+(0.378910374088)*x[1]**o+(-0.600308536915)*x[1]
        s[2,1,2,1]=(0.684159561112)*x[0]**o+(-0.408717922991)*x[0]+(0.0986838286044)*x[1]**o+(0.446743772828)*x[1]
        s[2,2,0,0]=(-0.657789003001)*x[0]**o+(-0.696243850831)*x[0]+(0.143074391914)*x[1]**o+(-0.784498727037)*x[1]
        s[2,2,0,1]=(-0.035450676833)*x[0]**o+(-0.453531837262)*x[0]+(0.59955708994)*x[1]**o+(0.35879284257)*x[1]
        s[2,2,1,0]=(0.302411531858)*x[0]**o+(-0.120739744552)*x[0]+(0.23248426438)*x[1]**o+(-0.39024395986)*x[1]
        s[2,2,1,1]=(0.957811592451)*x[0]**o+(-0.444133483391)*x[0]+(-0.917299241507)*x[1]**o+(0.66053932766)*x[1]
        s[2,2,2,0]=(0.361654572167)*x[0]**o+(-0.860498450204)*x[0]+(0.839403953375)*x[1]**o+(-0.386466411338)*x[1]
        s[2,2,2,1]=(-0.0143243836126)*x[0]**o+(0.444553204161)*x[0]+(0.0214936163191)*x[1]**o+(-0.257450046824)*x[1]
        s[2,3,0,0]=(0.841171215625)*x[0]**o+(-0.342091657341)*x[0]+(0.429215544167)*x[1]**o+(0.291784964305)*x[1]
        s[2,3,0,1]=(0.650468319582)*x[0]**o+(-0.121782834777)*x[0]+(0.887022664242)*x[1]**o+(0.957311577092)*x[1]
        s[2,3,1,0]=(0.156495854578)*x[0]**o+(0.855814547392)*x[0]+(0.320209813846)*x[1]**o+(0.458769463001)*x[1]
        s[2,3,1,1]=(0.34344265441)*x[0]**o+(-0.992393068373)*x[0]+(-0.0554086726336)*x[1]**o+(-0.985825317842)*x[1]
        s[2,3,2,0]=(-0.866276931509)*x[0]**o+(-0.343662754778)*x[0]+(0.0302242797749)*x[1]**o+(-0.652051073003)*x[1]
        s[2,3,2,1]=(-0.68451689603)*x[0]**o+(0.564121866438)*x[0]+(-0.136564076086)*x[1]**o+(0.648701900333)*x[1]
        s[2,4,0,0]=(-0.430429138775)*x[0]**o+(0.721222678082)*x[0]+(-0.0375465605006)*x[1]**o+(-0.462643638758)*x[1]
        s[2,4,0,1]=(-0.321870157728)*x[0]**o+(0.884979965704)*x[0]+(0.210984337677)*x[1]**o+(-0.796486469869)*x[1]
        s[2,4,1,0]=(0.365646287696)*x[0]**o+(-0.600693071289)*x[0]+(-0.998407218964)*x[1]**o+(-0.119949305545)*x[1]
        s[2,4,1,1]=(-0.763923462892)*x[0]**o+(-0.831011577325)*x[0]+(0.547670679354)*x[1]**o+(-0.856456618287)*x[1]
        s[2,4,2,0]=(0.107211541473)*x[0]**o+(-0.787737471224)*x[0]+(0.00381729482611)*x[1]**o+(-0.737759009165)*x[1]
        s[2,4,2,1]=(0.233196464862)*x[0]**o+(-0.464864449316)*x[0]+(0.229081432725)*x[1]**o+(-0.316321259146)*x[1]
        s[3,0,0,0]=(0.84633152323)*x[0]**o+(0.107225324454)*x[0]+(-0.768659465295)*x[1]**o+(0.441758939429)*x[1]
        s[3,0,0,1]=(-0.317081751011)*x[0]**o+(-0.557807443325)*x[0]+(-0.513518142998)*x[1]**o+(0.969406024047)*x[1]
        s[3,0,1,0]=(0.470133545453)*x[0]**o+(0.689603661125)*x[0]+(-0.358563691038)*x[1]**o+(0.0930824872861)*x[1]
        s[3,0,1,1]=(0.31919974075)*x[0]**o+(-0.555789020025)*x[0]+(0.0108880065973)*x[1]**o+(0.519548687393)*x[1]
        s[3,0,2,0]=(0.663881497435)*x[0]**o+(-0.668884360184)*x[0]+(-0.249669522726)*x[1]**o+(0.978543111491)*x[1]
        s[3,0,2,1]=(0.119167252087)*x[0]**o+(0.80927599582)*x[0]+(0.526489556971)*x[1]**o+(-0.440104597696)*x[1]
        s[3,1,0,0]=(-0.367446660377)*x[0]**o+(-0.326052348072)*x[0]+(0.841697795111)*x[1]**o+(0.403206833675)*x[1]
        s[3,1,0,1]=(-0.647927382559)*x[0]**o+(0.096852636085)*x[0]+(-0.126798636274)*x[1]**o+(0.117295901868)*x[1]
        s[3,1,1,0]=(0.827676755777)*x[0]**o+(-0.922576416262)*x[0]+(-0.3927169369)*x[1]**o+(0.828141309451)*x[1]
        s[3,1,1,1]=(-0.0442514478276)*x[0]**o+(-0.17904563214)*x[0]+(0.0870253733863)*x[1]**o+(-0.521249847127)*x[1]
        s[3,1,2,0]=(0.295531599451)*x[0]**o+(-0.125313746461)*x[0]+(0.982602753269)*x[1]**o+(-0.469029795983)*x[1]
        s[3,1,2,1]=(0.950273049304)*x[0]**o+(0.859212132249)*x[0]+(0.728565507333)*x[1]**o+(-0.274531603828)*x[1]
        s[3,2,0,0]=(-0.885845551336)*x[0]**o+(-0.704449229328)*x[0]+(-0.571836258871)*x[1]**o+(-0.0476396264682)*x[1]
        s[3,2,0,1]=(0.0713925637733)*x[0]**o+(0.294650464749)*x[0]+(-0.917378997351)*x[1]**o+(-0.715770468859)*x[1]
        s[3,2,1,0]=(-0.715304376467)*x[0]**o+(0.252625168227)*x[0]+(0.942098162684)*x[1]**o+(-0.0579654780836)*x[1]
        s[3,2,1,1]=(-0.937718965835)*x[0]**o+(0.649603348472)*x[0]+(-0.293284658463)*x[1]**o+(-0.347005762941)*x[1]
        s[3,2,2,0]=(-0.042358031525)*x[0]**o+(0.036891446424)*x[0]+(-0.229597055696)*x[1]**o+(0.0283282394156)*x[1]
        s[3,2,2,1]=(-0.0186366313091)*x[0]**o+(-0.0127649188124)*x[0]+(0.803780828202)*x[1]**o+(0.961045979755)*x[1]
        s[3,3,0,0]=(-0.27020299765)*x[0]**o+(-0.558034547573)*x[0]+(-0.524049515519)*x[1]**o+(-0.276595466979)*x[1]
        s[3,3,0,1]=(-0.510463688955)*x[0]**o+(-0.267341166705)*x[0]+(0.547679624943)*x[1]**o+(0.521119242662)*x[1]
        s[3,3,1,0]=(-0.155255975174)*x[0]**o+(0.365872110897)*x[0]+(-0.725704463467)*x[1]**o+(-0.790632654371)*x[1]
        s[3,3,1,1]=(-0.5111339808)*x[0]**o+(-0.0339146931841)*x[0]+(-0.696520447897)*x[1]**o+(-0.229679747746)*x[1]
        s[3,3,2,0]=(-0.552425399042)*x[0]**o+(-0.971205157087)*x[0]+(0.394260185879)*x[1]**o+(-0.164097031641)*x[1]
        s[3,3,2,1]=(-0.386876641825)*x[0]**o+(0.215459781811)*x[0]+(-0.332800684344)*x[1]**o+(0.419984531651)*x[1]
        s[3,4,0,0]=(-0.430256150077)*x[0]**o+(-0.110854674619)*x[0]+(0.687593230685)*x[1]**o+(-0.686895166454)*x[1]
        s[3,4,0,1]=(0.240753477602)*x[0]**o+(-0.815224522564)*x[0]+(0.499109852429)*x[1]**o+(-0.106870206363)*x[1]
        s[3,4,1,0]=(0.168547621411)*x[0]**o+(-0.509465054282)*x[0]+(-0.806347768707)*x[1]**o+(-0.799422054705)*x[1]
        s[3,4,1,1]=(-0.634806382514)*x[0]**o+(-0.125122793997)*x[0]+(-0.918019563645)*x[1]**o+(-0.341625694299)*x[1]
        s[3,4,2,0]=(-0.136108140149)*x[0]**o+(-0.780175946944)*x[0]+(0.0441301051696)*x[1]**o+(-0.326789608501)*x[1]
        s[3,4,2,1]=(0.181450357705)*x[0]**o+(0.467047306052)*x[0]+(0.182531831827)*x[1]**o+(-0.598701482526)*x[1]
        ref[0,0,0,0]=(-0.099815403496)*x_ref[0]**o+(-0.918641975262)*x_ref[0]+(-0.643800270537)*x_ref[1]**o+(-0.188947131246)*x_ref[1]
        ref[0,0,0,1]=(-0.654158499426)*x_ref[0]**o+(-0.158917070952)*x_ref[0]+(0.482580361439)*x_ref[1]**o+(0.654085120986)*x_ref[1]
        ref[0,0,1,0]=(-0.82639526996)*x_ref[0]**o+(-0.852788225935)*x_ref[0]+(-0.883559061247)*x_ref[1]**o+(-0.762093398747)*x_ref[1]
        ref[0,0,1,1]=(-0.410064006917)*x_ref[0]**o+(-0.358595400023)*x_ref[0]+(-0.744803691415)*x_ref[1]**o+(-0.418642435874)*x_ref[1]
        ref[0,0,2,0]=(0.627425019462)*x_ref[0]**o+(-0.759635228096)*x_ref[0]+(-0.225478401461)*x_ref[1]**o+(-0.394678152175)*x_ref[1]
        ref[0,0,2,1]=(0.282477965976)*x_ref[0]**o+(0.591668341745)*x_ref[0]+(0.997081550686)*x_ref[1]**o+(-0.16137686071)*x_ref[1]
        ref[0,1,0,0]=(0.503328443933)*x_ref[0]**o+(-0.349809904995)*x_ref[0]+(0.675840382965)*x_ref[1]**o+(0.122847940652)*x_ref[1]
        ref[0,1,0,1]=(-0.693842907429)*x_ref[0]**o+(0.242153763819)*x_ref[0]+(0.669516208483)*x_ref[1]**o+(-0.966639099473)*x_ref[1]
        ref[0,1,1,0]=(-0.143996595411)*x_ref[0]**o+(-0.0789367221661)*x_ref[0]+(0.44978165828)*x_ref[1]**o+(0.982272605823)*x_ref[1]
        ref[0,1,1,1]=(0.217504837315)*x_ref[0]**o+(0.62345834313)*x_ref[0]+(0.421663416031)*x_ref[1]**o+(0.298214724947)*x_ref[1]
        ref[0,1,2,0]=(-0.192788493597)*x_ref[0]**o+(0.59360099119)*x_ref[0]+(0.798640786481)*x_ref[1]**o+(-0.0618029984487)*x_ref[1]
        ref[0,1,2,1]=(-0.448474976435)*x_ref[0]**o+(-0.136832009015)*x_ref[0]+(-0.597189414931)*x_ref[1]**o+(0.502732362863)*x_ref[1]
        ref[0,2,0,0]=(0.252384046343)*x_ref[0]**o+(-0.400123690831)*x_ref[0]+(-0.545481308678)*x_ref[1]**o+(-0.388790715614)*x_ref[1]
        ref[0,2,0,1]=(-0.66240802679)*x_ref[0]**o+(0.77064500558)*x_ref[0]+(0.5239431782)*x_ref[1]**o+(0.467410945489)*x_ref[1]
        ref[0,2,1,0]=(-0.85322788336)*x_ref[0]**o+(-0.690030962118)*x_ref[0]+(0.699888315472)*x_ref[1]**o+(-0.89805156241)*x_ref[1]
        ref[0,2,1,1]=(-0.255889046261)*x_ref[0]**o+(0.0541259188422)*x_ref[0]+(-0.749587408291)*x_ref[1]**o+(0.791965754891)*x_ref[1]
        ref[0,2,2,0]=(0.496462410445)*x_ref[0]**o+(0.772074605719)*x_ref[0]+(0.180850727108)*x_ref[1]**o+(-0.487496085214)*x_ref[1]
        ref[0,2,2,1]=(0.706055709638)*x_ref[0]**o+(-0.598906910405)*x_ref[0]+(-0.152966385845)*x_ref[1]**o+(0.871383558924)*x_ref[1]
        ref[0,3,0,0]=(-0.951858396669)*x_ref[0]**o+(-0.794420515046)*x_ref[0]+(0.834659431502)*x_ref[1]**o+(0.481994252391)*x_ref[1]
        ref[0,3,0,1]=(-0.578325718357)*x_ref[0]**o+(-0.843177109188)*x_ref[0]+(0.38052630466)*x_ref[1]**o+(0.0999564588392)*x_ref[1]
        ref[0,3,1,0]=(0.251016293115)*x_ref[0]**o+(-0.846784412909)*x_ref[0]+(0.615130638894)*x_ref[1]**o+(-0.545472677499)*x_ref[1]
        ref[0,3,1,1]=(0.0242041003245)*x_ref[0]**o+(-0.558470673629)*x_ref[0]+(0.0355389532695)*x_ref[1]**o+(-0.82512767128)*x_ref[1]
        ref[0,3,2,0]=(0.104901847054)*x_ref[0]**o+(0.176002645436)*x_ref[0]+(-0.0529040105457)*x_ref[1]**o+(-0.497309382471)*x_ref[1]
        ref[0,3,2,1]=(-0.19187265394)*x_ref[0]**o+(-0.459102146282)*x_ref[0]+(0.263116634482)*x_ref[1]**o+(0.573723666261)*x_ref[1]
        ref[0,4,0,0]=(-0.605664936504)*x_ref[0]**o+(0.236623083899)*x_ref[0]+(-0.573965234185)*x_ref[1]**o+(-0.191541238111)*x_ref[1]
        ref[0,4,0,1]=(0.735297405377)*x_ref[0]**o+(-0.863752806793)*x_ref[0]+(0.130990772115)*x_ref[1]**o+(0.153051776075)*x_ref[1]
        ref[0,4,1,0]=(0.83806722488)*x_ref[0]**o+(0.952102086801)*x_ref[0]+(0.921278259866)*x_ref[1]**o+(0.100889262792)*x_ref[1]
        ref[0,4,1,1]=(0.422551160597)*x_ref[0]**o+(-0.68105266886)*x_ref[0]+(-0.0230668740384)*x_ref[1]**o+(0.608853763751)*x_ref[1]
        ref[0,4,2,0]=(-0.509106957279)*x_ref[0]**o+(0.535714529222)*x_ref[0]+(0.407059274864)*x_ref[1]**o+(0.0304780664648)*x_ref[1]
        ref[0,4,2,1]=(-0.899880077882)*x_ref[0]**o+(-0.150489820435)*x_ref[0]+(-0.329228777137)*x_ref[1]**o+(-0.325670285955)*x_ref[1]
        ref[1,0,0,0]=(0.723148414781)*x_ref[0]**o+(0.276861527427)*x_ref[0]+(-0.457164017461)*x_ref[1]**o+(0.587623869716)*x_ref[1]
        ref[1,0,0,1]=(-0.656042247607)*x_ref[0]**o+(0.599876456972)*x_ref[0]+(-0.487723393567)*x_ref[1]**o+(-0.293330841893)*x_ref[1]
        ref[1,0,1,0]=(0.498385487237)*x_ref[0]**o+(0.823826806076)*x_ref[0]+(-0.260001108163)*x_ref[1]**o+(-0.520083127954)*x_ref[1]
        ref[1,0,1,1]=(0.0511021547178)*x_ref[0]**o+(0.0752387451382)*x_ref[0]+(-0.295018309613)*x_ref[1]**o+(0.410655676791)*x_ref[1]
        ref[1,0,2,0]=(-0.926038025877)*x_ref[0]**o+(-0.101726746627)*x_ref[0]+(-0.714060837737)*x_ref[1]**o+(0.291653139474)*x_ref[1]
        ref[1,0,2,1]=(0.416231421476)*x_ref[0]**o+(0.968331479531)*x_ref[0]+(0.13575381957)*x_ref[1]**o+(0.0581635556746)*x_ref[1]
        ref[1,1,0,0]=(0.483218423871)*x_ref[0]**o+(0.837199391237)*x_ref[0]+(-0.332232511767)*x_ref[1]**o+(0.861714372392)*x_ref[1]
        ref[1,1,0,1]=(-0.861985810928)*x_ref[0]**o+(0.704494207363)*x_ref[0]+(0.0120560011404)*x_ref[1]**o+(-0.922350499146)*x_ref[1]
        ref[1,1,1,0]=(0.710471809896)*x_ref[0]**o+(0.10838308437)*x_ref[0]+(-0.936553657991)*x_ref[1]**o+(-0.157956217106)*x_ref[1]
        ref[1,1,1,1]=(0.61604545132)*x_ref[0]**o+(0.161202252636)*x_ref[0]+(-0.208993778286)*x_ref[1]**o+(0.150797102116)*x_ref[1]
        ref[1,1,2,0]=(0.237396232488)*x_ref[0]**o+(-0.260415056277)*x_ref[0]+(-0.760219527005)*x_ref[1]**o+(0.203694444805)*x_ref[1]
        ref[1,1,2,1]=(-0.656645187501)*x_ref[0]**o+(-0.84629755936)*x_ref[0]+(-0.465896055143)*x_ref[1]**o+(0.335524534826)*x_ref[1]
        ref[1,2,0,0]=(-0.546393675559)*x_ref[0]**o+(0.0344401682615)*x_ref[0]+(-0.267054613816)*x_ref[1]**o+(-0.345211291755)*x_ref[1]
        ref[1,2,0,1]=(0.807935620456)*x_ref[0]**o+(0.89820026562)*x_ref[0]+(-0.471804773843)*x_ref[1]**o+(0.0183931405417)*x_ref[1]
        ref[1,2,1,0]=(0.347457777403)*x_ref[0]**o+(0.894578611023)*x_ref[0]+(0.506086327041)*x_ref[1]**o+(-0.0552731455772)*x_ref[1]
        ref[1,2,1,1]=(0.898402940142)*x_ref[0]**o+(-0.850905749447)*x_ref[0]+(0.421790306199)*x_ref[1]**o+(0.560435305462)*x_ref[1]
        ref[1,2,2,0]=(0.417854170807)*x_ref[0]**o+(0.0808123920631)*x_ref[0]+(0.685114818766)*x_ref[1]**o+(0.627248749954)*x_ref[1]
        ref[1,2,2,1]=(0.608689720682)*x_ref[0]**o+(-0.215111751974)*x_ref[0]+(-0.157391477249)*x_ref[1]**o+(-0.157398505688)*x_ref[1]
        ref[1,3,0,0]=(0.453375349378)*x_ref[0]**o+(-0.232490378969)*x_ref[0]+(0.370581316376)*x_ref[1]**o+(0.2321645519)*x_ref[1]
        ref[1,3,0,1]=(-0.921453677654)*x_ref[0]**o+(-0.272608399386)*x_ref[0]+(0.980108178061)*x_ref[1]**o+(0.0304202383868)*x_ref[1]
        ref[1,3,1,0]=(-0.703951869845)*x_ref[0]**o+(0.232888375306)*x_ref[0]+(0.737108419956)*x_ref[1]**o+(-0.815317463111)*x_ref[1]
        ref[1,3,1,1]=(-0.849808179245)*x_ref[0]**o+(-0.848051887981)*x_ref[0]+(0.452157961656)*x_ref[1]**o+(0.942533763098)*x_ref[1]
        ref[1,3,2,0]=(0.276310583942)*x_ref[0]**o+(-0.426217969193)*x_ref[0]+(0.383056703597)*x_ref[1]**o+(-0.100608872707)*x_ref[1]
        ref[1,3,2,1]=(0.792743379061)*x_ref[0]**o+(-0.638898354)*x_ref[0]+(0.0900755365083)*x_ref[1]**o+(0.346700761651)*x_ref[1]
        ref[1,4,0,0]=(0.227940749448)*x_ref[0]**o+(-0.723141488706)*x_ref[0]+(-0.597539261807)*x_ref[1]**o+(-0.0547634570298)*x_ref[1]
        ref[1,4,0,1]=(-0.00150193441347)*x_ref[0]**o+(-0.875771867697)*x_ref[0]+(0.554295315299)*x_ref[1]**o+(0.179291021879)*x_ref[1]
        ref[1,4,1,0]=(0.665263753178)*x_ref[0]**o+(-0.261999229902)*x_ref[0]+(0.177708351596)*x_ref[1]**o+(-0.71011990376)*x_ref[1]
        ref[1,4,1,1]=(0.416094331837)*x_ref[0]**o+(0.809191029905)*x_ref[0]+(-0.631251155279)*x_ref[1]**o+(0.758593347696)*x_ref[1]
        ref[1,4,2,0]=(0.50941363283)*x_ref[0]**o+(-0.652536088555)*x_ref[0]+(-0.0994076235125)*x_ref[1]**o+(0.186614952474)*x_ref[1]
        ref[1,4,2,1]=(0.129263234225)*x_ref[0]**o+(0.115870119224)*x_ref[0]+(-0.25581908442)*x_ref[1]**o+(0.295990548791)*x_ref[1]
        ref[2,0,0,0]=(-0.432850417862)*x_ref[0]**o+(-0.491964804361)*x_ref[0]+(0.0106143845453)*x_ref[1]**o+(0.787362504175)*x_ref[1]
        ref[2,0,0,1]=(0.31617556259)*x_ref[0]**o+(0.255445781486)*x_ref[0]+(-0.302678692069)*x_ref[1]**o+(0.840840019713)*x_ref[1]
        ref[2,0,1,0]=(-0.855359978332)*x_ref[0]**o+(0.410554550223)*x_ref[0]+(-0.703253338388)*x_ref[1]**o+(0.59010970561)*x_ref[1]
        ref[2,0,1,1]=(-0.573270259685)*x_ref[0]**o+(0.928396842624)*x_ref[0]+(0.94155331127)*x_ref[1]**o+(0.160140257961)*x_ref[1]
        ref[2,0,2,0]=(0.0524505638003)*x_ref[0]**o+(-0.736508989499)*x_ref[0]+(-0.565060891384)*x_ref[1]**o+(-0.613736274563)*x_ref[1]
        ref[2,0,2,1]=(-0.776619329318)*x_ref[0]**o+(0.485104655735)*x_ref[0]+(-0.135863414015)*x_ref[1]**o+(-0.14173677679)*x_ref[1]
        ref[2,1,0,0]=(0.662345907479)*x_ref[0]**o+(-0.900993470049)*x_ref[0]+(-0.882752065723)*x_ref[1]**o+(0.642775104186)*x_ref[1]
        ref[2,1,0,1]=(-0.189295597666)*x_ref[0]**o+(0.0165605763413)*x_ref[0]+(0.989119296804)*x_ref[1]**o+(0.262678778013)*x_ref[1]
        ref[2,1,1,0]=(0.927673133577)*x_ref[0]**o+(-0.0248539427539)*x_ref[0]+(0.504610255651)*x_ref[1]**o+(-0.593911741472)*x_ref[1]
        ref[2,1,1,1]=(0.899525029448)*x_ref[0]**o+(-0.97195963201)*x_ref[0]+(-0.461931155897)*x_ref[1]**o+(0.91354314585)*x_ref[1]
        ref[2,1,2,0]=(0.398654077456)*x_ref[0]**o+(-0.429950726187)*x_ref[0]+(0.378910374088)*x_ref[1]**o+(-0.600308536915)*x_ref[1]
        ref[2,1,2,1]=(0.684159561112)*x_ref[0]**o+(-0.408717922991)*x_ref[0]+(0.0986838286044)*x_ref[1]**o+(0.446743772828)*x_ref[1]
        ref[2,2,0,0]=(-0.657789003001)*x_ref[0]**o+(-0.696243850831)*x_ref[0]+(0.143074391914)*x_ref[1]**o+(-0.784498727037)*x_ref[1]
        ref[2,2,0,1]=(-0.035450676833)*x_ref[0]**o+(-0.453531837262)*x_ref[0]+(0.59955708994)*x_ref[1]**o+(0.35879284257)*x_ref[1]
        ref[2,2,1,0]=(0.302411531858)*x_ref[0]**o+(-0.120739744552)*x_ref[0]+(0.23248426438)*x_ref[1]**o+(-0.39024395986)*x_ref[1]
        ref[2,2,1,1]=(0.957811592451)*x_ref[0]**o+(-0.444133483391)*x_ref[0]+(-0.917299241507)*x_ref[1]**o+(0.66053932766)*x_ref[1]
        ref[2,2,2,0]=(0.361654572167)*x_ref[0]**o+(-0.860498450204)*x_ref[0]+(0.839403953375)*x_ref[1]**o+(-0.386466411338)*x_ref[1]
        ref[2,2,2,1]=(-0.0143243836126)*x_ref[0]**o+(0.444553204161)*x_ref[0]+(0.0214936163191)*x_ref[1]**o+(-0.257450046824)*x_ref[1]
        ref[2,3,0,0]=(0.841171215625)*x_ref[0]**o+(-0.342091657341)*x_ref[0]+(0.429215544167)*x_ref[1]**o+(0.291784964305)*x_ref[1]
        ref[2,3,0,1]=(0.650468319582)*x_ref[0]**o+(-0.121782834777)*x_ref[0]+(0.887022664242)*x_ref[1]**o+(0.957311577092)*x_ref[1]
        ref[2,3,1,0]=(0.156495854578)*x_ref[0]**o+(0.855814547392)*x_ref[0]+(0.320209813846)*x_ref[1]**o+(0.458769463001)*x_ref[1]
        ref[2,3,1,1]=(0.34344265441)*x_ref[0]**o+(-0.992393068373)*x_ref[0]+(-0.0554086726336)*x_ref[1]**o+(-0.985825317842)*x_ref[1]
        ref[2,3,2,0]=(-0.866276931509)*x_ref[0]**o+(-0.343662754778)*x_ref[0]+(0.0302242797749)*x_ref[1]**o+(-0.652051073003)*x_ref[1]
        ref[2,3,2,1]=(-0.68451689603)*x_ref[0]**o+(0.564121866438)*x_ref[0]+(-0.136564076086)*x_ref[1]**o+(0.648701900333)*x_ref[1]
        ref[2,4,0,0]=(-0.430429138775)*x_ref[0]**o+(0.721222678082)*x_ref[0]+(-0.0375465605006)*x_ref[1]**o+(-0.462643638758)*x_ref[1]
        ref[2,4,0,1]=(-0.321870157728)*x_ref[0]**o+(0.884979965704)*x_ref[0]+(0.210984337677)*x_ref[1]**o+(-0.796486469869)*x_ref[1]
        ref[2,4,1,0]=(0.365646287696)*x_ref[0]**o+(-0.600693071289)*x_ref[0]+(-0.998407218964)*x_ref[1]**o+(-0.119949305545)*x_ref[1]
        ref[2,4,1,1]=(-0.763923462892)*x_ref[0]**o+(-0.831011577325)*x_ref[0]+(0.547670679354)*x_ref[1]**o+(-0.856456618287)*x_ref[1]
        ref[2,4,2,0]=(0.107211541473)*x_ref[0]**o+(-0.787737471224)*x_ref[0]+(0.00381729482611)*x_ref[1]**o+(-0.737759009165)*x_ref[1]
        ref[2,4,2,1]=(0.233196464862)*x_ref[0]**o+(-0.464864449316)*x_ref[0]+(0.229081432725)*x_ref[1]**o+(-0.316321259146)*x_ref[1]
        ref[3,0,0,0]=(0.84633152323)*x_ref[0]**o+(0.107225324454)*x_ref[0]+(-0.768659465295)*x_ref[1]**o+(0.441758939429)*x_ref[1]
        ref[3,0,0,1]=(-0.317081751011)*x_ref[0]**o+(-0.557807443325)*x_ref[0]+(-0.513518142998)*x_ref[1]**o+(0.969406024047)*x_ref[1]
        ref[3,0,1,0]=(0.470133545453)*x_ref[0]**o+(0.689603661125)*x_ref[0]+(-0.358563691038)*x_ref[1]**o+(0.0930824872861)*x_ref[1]
        ref[3,0,1,1]=(0.31919974075)*x_ref[0]**o+(-0.555789020025)*x_ref[0]+(0.0108880065973)*x_ref[1]**o+(0.519548687393)*x_ref[1]
        ref[3,0,2,0]=(0.663881497435)*x_ref[0]**o+(-0.668884360184)*x_ref[0]+(-0.249669522726)*x_ref[1]**o+(0.978543111491)*x_ref[1]
        ref[3,0,2,1]=(0.119167252087)*x_ref[0]**o+(0.80927599582)*x_ref[0]+(0.526489556971)*x_ref[1]**o+(-0.440104597696)*x_ref[1]
        ref[3,1,0,0]=(-0.367446660377)*x_ref[0]**o+(-0.326052348072)*x_ref[0]+(0.841697795111)*x_ref[1]**o+(0.403206833675)*x_ref[1]
        ref[3,1,0,1]=(-0.647927382559)*x_ref[0]**o+(0.096852636085)*x_ref[0]+(-0.126798636274)*x_ref[1]**o+(0.117295901868)*x_ref[1]
        ref[3,1,1,0]=(0.827676755777)*x_ref[0]**o+(-0.922576416262)*x_ref[0]+(-0.3927169369)*x_ref[1]**o+(0.828141309451)*x_ref[1]
        ref[3,1,1,1]=(-0.0442514478276)*x_ref[0]**o+(-0.17904563214)*x_ref[0]+(0.0870253733863)*x_ref[1]**o+(-0.521249847127)*x_ref[1]
        ref[3,1,2,0]=(0.295531599451)*x_ref[0]**o+(-0.125313746461)*x_ref[0]+(0.982602753269)*x_ref[1]**o+(-0.469029795983)*x_ref[1]
        ref[3,1,2,1]=(0.950273049304)*x_ref[0]**o+(0.859212132249)*x_ref[0]+(0.728565507333)*x_ref[1]**o+(-0.274531603828)*x_ref[1]
        ref[3,2,0,0]=(-0.885845551336)*x_ref[0]**o+(-0.704449229328)*x_ref[0]+(-0.571836258871)*x_ref[1]**o+(-0.0476396264682)*x_ref[1]
        ref[3,2,0,1]=(0.0713925637733)*x_ref[0]**o+(0.294650464749)*x_ref[0]+(-0.917378997351)*x_ref[1]**o+(-0.715770468859)*x_ref[1]
        ref[3,2,1,0]=(-0.715304376467)*x_ref[0]**o+(0.252625168227)*x_ref[0]+(0.942098162684)*x_ref[1]**o+(-0.0579654780836)*x_ref[1]
        ref[3,2,1,1]=(-0.937718965835)*x_ref[0]**o+(0.649603348472)*x_ref[0]+(-0.293284658463)*x_ref[1]**o+(-0.347005762941)*x_ref[1]
        ref[3,2,2,0]=(-0.042358031525)*x_ref[0]**o+(0.036891446424)*x_ref[0]+(-0.229597055696)*x_ref[1]**o+(0.0283282394156)*x_ref[1]
        ref[3,2,2,1]=(-0.0186366313091)*x_ref[0]**o+(-0.0127649188124)*x_ref[0]+(0.803780828202)*x_ref[1]**o+(0.961045979755)*x_ref[1]
        ref[3,3,0,0]=(-0.27020299765)*x_ref[0]**o+(-0.558034547573)*x_ref[0]+(-0.524049515519)*x_ref[1]**o+(-0.276595466979)*x_ref[1]
        ref[3,3,0,1]=(-0.510463688955)*x_ref[0]**o+(-0.267341166705)*x_ref[0]+(0.547679624943)*x_ref[1]**o+(0.521119242662)*x_ref[1]
        ref[3,3,1,0]=(-0.155255975174)*x_ref[0]**o+(0.365872110897)*x_ref[0]+(-0.725704463467)*x_ref[1]**o+(-0.790632654371)*x_ref[1]
        ref[3,3,1,1]=(-0.5111339808)*x_ref[0]**o+(-0.0339146931841)*x_ref[0]+(-0.696520447897)*x_ref[1]**o+(-0.229679747746)*x_ref[1]
        ref[3,3,2,0]=(-0.552425399042)*x_ref[0]**o+(-0.971205157087)*x_ref[0]+(0.394260185879)*x_ref[1]**o+(-0.164097031641)*x_ref[1]
        ref[3,3,2,1]=(-0.386876641825)*x_ref[0]**o+(0.215459781811)*x_ref[0]+(-0.332800684344)*x_ref[1]**o+(0.419984531651)*x_ref[1]
        ref[3,4,0,0]=(-0.430256150077)*x_ref[0]**o+(-0.110854674619)*x_ref[0]+(0.687593230685)*x_ref[1]**o+(-0.686895166454)*x_ref[1]
        ref[3,4,0,1]=(0.240753477602)*x_ref[0]**o+(-0.815224522564)*x_ref[0]+(0.499109852429)*x_ref[1]**o+(-0.106870206363)*x_ref[1]
        ref[3,4,1,0]=(0.168547621411)*x_ref[0]**o+(-0.509465054282)*x_ref[0]+(-0.806347768707)*x_ref[1]**o+(-0.799422054705)*x_ref[1]
        ref[3,4,1,1]=(-0.634806382514)*x_ref[0]**o+(-0.125122793997)*x_ref[0]+(-0.918019563645)*x_ref[1]**o+(-0.341625694299)*x_ref[1]
        ref[3,4,2,0]=(-0.136108140149)*x_ref[0]**o+(-0.780175946944)*x_ref[0]+(0.0441301051696)*x_ref[1]**o+(-0.326789608501)*x_ref[1]
        ref[3,4,2,1]=(0.181450357705)*x_ref[0]**o+(0.467047306052)*x_ref[0]+(0.182531831827)*x_ref[1]**o+(-0.598701482526)*x_ref[1]
      else:
        s[0,0,0,0]=(-0.0362500650077)*x[0]**o+(0.692407811012)*x[0]+(-0.855107417704)*x[1]**o+(-0.0397854200762)*x[1]+(0.488177999316)*x[2]**o+(0.946478266036)*x[2]
        s[0,0,0,1]=(-0.533284434038)*x[0]**o+(-0.599290705017)*x[0]+(0.198922961617)*x[1]**o+(0.652839426276)*x[1]+(0.560109621832)*x[2]**o+(0.167310997616)*x[2]
        s[0,0,1,0]=(-0.21016675446)*x[0]**o+(0.812498932794)*x[0]+(-0.611566184558)*x[1]**o+(0.626231122402)*x[1]+(0.536374628942)*x[2]**o+(0.8520547245)*x[2]
        s[0,0,1,1]=(0.26308571414)*x[0]**o+(-0.359237396089)*x[0]+(0.146879292748)*x[1]**o+(-0.241831631389)*x[1]+(0.812820522163)*x[2]**o+(0.824191692943)*x[2]
        s[0,0,2,0]=(-0.585682391332)*x[0]**o+(0.82273803111)*x[0]+(-0.715688315155)*x[1]**o+(-0.997893564593)*x[1]+(0.39588322481)*x[2]**o+(0.410447772982)*x[2]
        s[0,0,2,1]=(0.491817354047)*x[0]**o+(0.105289616977)*x[0]+(0.0581932965006)*x[1]**o+(-0.584153248946)*x[1]+(-0.837305609444)*x[2]**o+(-0.140188544108)*x[2]
        s[0,1,0,0]=(-0.158174387659)*x[0]**o+(0.774512978962)*x[0]+(0.953772629099)*x[1]**o+(-0.0474582135671)*x[1]+(-0.691917863221)*x[2]**o+(0.978550525536)*x[2]
        s[0,1,0,1]=(-0.696002534433)*x[0]**o+(-0.904833590237)*x[0]+(-0.991879336542)*x[1]**o+(-0.00301423741703)*x[1]+(-0.495919957691)*x[2]**o+(0.923872653662)*x[2]
        s[0,1,1,0]=(-0.617187346815)*x[0]**o+(-0.0283413268672)*x[0]+(0.439716253425)*x[1]**o+(0.679148169861)*x[1]+(-0.5494839537)*x[2]**o+(0.780531698733)*x[2]
        s[0,1,1,1]=(0.0763086894431)*x[0]**o+(0.149024988807)*x[0]+(0.660061724201)*x[1]**o+(0.227260856773)*x[1]+(0.604941557505)*x[2]**o+(-0.32106743372)*x[2]
        s[0,1,2,0]=(-0.710651974651)*x[0]**o+(0.729233023268)*x[0]+(0.830588938931)*x[1]**o+(0.930100931807)*x[1]+(0.576259601615)*x[2]**o+(0.995388406293)*x[2]
        s[0,1,2,1]=(0.635415205147)*x[0]**o+(-0.0475242382825)*x[0]+(0.779184290843)*x[1]**o+(0.161316071106)*x[1]+(-0.321426515539)*x[2]**o+(-0.188018565147)*x[2]
        s[0,2,0,0]=(-0.234573769896)*x[0]**o+(-0.35461213258)*x[0]+(0.322664245412)*x[1]**o+(-0.36584459555)*x[1]+(0.656560085187)*x[2]**o+(-0.380544830082)*x[2]
        s[0,2,0,1]=(0.314171153213)*x[0]**o+(-0.871055875712)*x[0]+(0.600399452124)*x[1]**o+(-0.710939791885)*x[1]+(-0.189029597187)*x[2]**o+(-0.29466779727)*x[2]
        s[0,2,1,0]=(0.911500533914)*x[0]**o+(-0.217077039937)*x[0]+(0.488325722425)*x[1]**o+(0.781071969291)*x[1]+(-0.883567213205)*x[2]**o+(0.938457876454)*x[2]
        s[0,2,1,1]=(0.947554813988)*x[0]**o+(0.108161094029)*x[0]+(0.0814722282926)*x[1]**o+(0.394738648377)*x[1]+(-0.585604351258)*x[2]**o+(0.424262882559)*x[2]
        s[0,2,2,0]=(0.130793077586)*x[0]**o+(-0.0759869806951)*x[0]+(0.524572237994)*x[1]**o+(-0.0494098932334)*x[1]+(-0.162573831498)*x[2]**o+(0.950665975392)*x[2]
        s[0,2,2,1]=(0.0636110845427)*x[0]**o+(-0.152586932248)*x[0]+(-0.428472289058)*x[1]**o+(0.943093556156)*x[1]+(-0.925181631256)*x[2]**o+(-0.850747416138)*x[2]
        s[0,3,0,0]=(0.803654693645)*x[0]**o+(-0.852793510169)*x[0]+(0.372325502382)*x[1]**o+(0.437191610865)*x[1]+(-0.626819779481)*x[2]**o+(-0.969085192013)*x[2]
        s[0,3,0,1]=(-0.498449247588)*x[0]**o+(-0.174254842954)*x[0]+(-0.751570511608)*x[1]**o+(0.0278714978689)*x[1]+(0.732061097781)*x[2]**o+(0.434922200273)*x[2]
        s[0,3,1,0]=(0.356966448806)*x[0]**o+(-0.754346917896)*x[0]+(-0.200096880735)*x[1]**o+(0.726202986575)*x[1]+(-0.273975127289)*x[2]**o+(0.922054852117)*x[2]
        s[0,3,1,1]=(0.294033872459)*x[0]**o+(0.435258372604)*x[0]+(-0.606222342027)*x[1]**o+(-0.418222134998)*x[1]+(-0.617263336049)*x[2]**o+(-0.655912764464)*x[2]
        s[0,3,2,0]=(-0.552805093123)*x[0]**o+(0.814985531695)*x[0]+(0.201701287999)*x[1]**o+(0.449378802496)*x[1]+(0.106877031271)*x[2]**o+(0.972523159602)*x[2]
        s[0,3,2,1]=(-0.123112956117)*x[0]**o+(-0.9669799128)*x[0]+(-0.365437347923)*x[1]**o+(0.95077682916)*x[1]+(-0.68795960948)*x[2]**o+(0.282323391952)*x[2]
        s[0,4,0,0]=(0.762949486092)*x[0]**o+(0.275310784251)*x[0]+(0.90824072581)*x[1]**o+(0.267277710972)*x[1]+(-0.387183781105)*x[2]**o+(-0.919714964098)*x[2]
        s[0,4,0,1]=(-0.219860124563)*x[0]**o+(0.458617314398)*x[0]+(-0.253950962469)*x[1]**o+(-0.213326656667)*x[1]+(-0.914189191308)*x[2]**o+(-0.88808493928)*x[2]
        s[0,4,1,0]=(0.0111879656405)*x[0]**o+(0.857083093627)*x[0]+(0.397087247889)*x[1]**o+(-0.411624885427)*x[1]+(-0.533535312833)*x[2]**o+(-0.473765290954)*x[2]
        s[0,4,1,1]=(0.408477302242)*x[0]**o+(-0.336255491206)*x[0]+(0.535004316057)*x[1]**o+(0.646300732798)*x[1]+(-0.683570698066)*x[2]**o+(-0.691723219637)*x[2]
        s[0,4,2,0]=(0.229517234528)*x[0]**o+(-0.327638251614)*x[0]+(0.752536339629)*x[1]**o+(0.440600000033)*x[1]+(-0.166203457183)*x[2]**o+(-0.632615828058)*x[2]
        s[0,4,2,1]=(0.128453646267)*x[0]**o+(-0.152501701527)*x[0]+(0.131616948331)*x[1]**o+(-0.560356219781)*x[1]+(0.369923630213)*x[2]**o+(-0.405937225725)*x[2]
        s[1,0,0,0]=(-0.498802287823)*x[0]**o+(0.974250100105)*x[0]+(-0.752335049339)*x[1]**o+(0.460106742976)*x[1]+(0.878914200699)*x[2]**o+(-0.85963539732)*x[2]
        s[1,0,0,1]=(0.937345235384)*x[0]**o+(0.0616042350221)*x[0]+(-0.63597655214)*x[1]**o+(0.200363730821)*x[1]+(0.677520498976)*x[2]**o+(0.310367850218)*x[2]
        s[1,0,1,0]=(-0.328289664191)*x[0]**o+(-0.439002316683)*x[0]+(-0.753405240499)*x[1]**o+(-0.749020227582)*x[1]+(0.200333899797)*x[2]**o+(0.63068525957)*x[2]
        s[1,0,1,1]=(-0.522381373851)*x[0]**o+(-0.877707833729)*x[0]+(-0.0334841793702)*x[1]**o+(0.00769816693416)*x[1]+(-0.81697893166)*x[2]**o+(-0.47349699259)*x[2]
        s[1,0,2,0]=(0.0786863277722)*x[0]**o+(-0.47462306827)*x[0]+(-0.850065158625)*x[1]**o+(0.0481804762762)*x[1]+(0.239465148926)*x[2]**o+(0.484166919183)*x[2]
        s[1,0,2,1]=(-0.0411433282485)*x[0]**o+(-0.851343034379)*x[0]+(-0.518404507699)*x[1]**o+(-0.357342937948)*x[1]+(-0.57398214485)*x[2]**o+(-0.0989103206187)*x[2]
        s[1,1,0,0]=(-0.060737698278)*x[0]**o+(-0.103483640965)*x[0]+(-0.206004957088)*x[1]**o+(0.556901021479)*x[1]+(0.951054830915)*x[2]**o+(0.668118573076)*x[2]
        s[1,1,0,1]=(0.0668860744649)*x[0]**o+(0.0490910382588)*x[0]+(0.817265796062)*x[1]**o+(-0.939419683478)*x[1]+(-0.800905110922)*x[2]**o+(-0.546647838589)*x[2]
        s[1,1,1,0]=(-0.437101637549)*x[0]**o+(0.269260162519)*x[0]+(-0.026802186298)*x[1]**o+(-0.932589891991)*x[1]+(-0.313814820129)*x[2]**o+(-0.436056943196)*x[2]
        s[1,1,1,1]=(-0.893118860321)*x[0]**o+(-0.837193118481)*x[0]+(-0.58313090725)*x[1]**o+(-0.657017739851)*x[1]+(-0.0158333033033)*x[2]**o+(0.286927848108)*x[2]
        s[1,1,2,0]=(-0.414914106956)*x[0]**o+(-0.295021056636)*x[0]+(-0.100513024651)*x[1]**o+(0.119321919323)*x[1]+(-0.911626057698)*x[2]**o+(0.0715733685275)*x[2]
        s[1,1,2,1]=(0.415073119201)*x[0]**o+(-0.79060072691)*x[0]+(-0.280560285152)*x[1]**o+(-0.651271923585)*x[1]+(-0.783523725291)*x[2]**o+(-0.165699899479)*x[2]
        s[1,2,0,0]=(0.204696440606)*x[0]**o+(0.58070882041)*x[0]+(0.498396642996)*x[1]**o+(0.270570176185)*x[1]+(-0.261604910878)*x[2]**o+(0.788402570665)*x[2]
        s[1,2,0,1]=(-0.163227090863)*x[0]**o+(0.17193387282)*x[0]+(0.559640507601)*x[1]**o+(0.39180422655)*x[1]+(0.182426715866)*x[2]**o+(0.52544860689)*x[2]
        s[1,2,1,0]=(-0.0826612289833)*x[0]**o+(-0.244737952715)*x[0]+(0.887738370973)*x[1]**o+(0.593438512127)*x[1]+(0.73128277327)*x[2]**o+(0.229067764431)*x[2]
        s[1,2,1,1]=(-0.117634040351)*x[0]**o+(-0.490365253501)*x[0]+(0.654067544802)*x[1]**o+(-0.862866217687)*x[1]+(-0.563450922747)*x[2]**o+(-0.785940469253)*x[2]
        s[1,2,2,0]=(0.486381246904)*x[0]**o+(-0.119137386359)*x[0]+(0.251388315083)*x[1]**o+(-0.142435376157)*x[1]+(-0.882418773242)*x[2]**o+(0.509436056026)*x[2]
        s[1,2,2,1]=(-0.69532427028)*x[0]**o+(-0.587331382396)*x[0]+(0.750092712907)*x[1]**o+(0.864188183197)*x[1]+(-0.0336735158745)*x[2]**o+(-0.118692941484)*x[2]
        s[1,3,0,0]=(0.287194598069)*x[0]**o+(0.775480786321)*x[0]+(0.24820537283)*x[1]**o+(0.579330345046)*x[1]+(0.641058615756)*x[2]**o+(-0.358716444584)*x[2]
        s[1,3,0,1]=(-0.218876040408)*x[0]**o+(-0.581134176647)*x[0]+(0.47937196949)*x[1]**o+(0.0529063680602)*x[1]+(-0.42659429504)*x[2]**o+(-0.445845798304)*x[2]
        s[1,3,1,0]=(0.500331134532)*x[0]**o+(0.764920932873)*x[0]+(0.256972231189)*x[1]**o+(0.279967260143)*x[1]+(0.90143027778)*x[2]**o+(-0.753075337423)*x[2]
        s[1,3,1,1]=(0.0478484319999)*x[0]**o+(0.239476880568)*x[0]+(-0.286765457909)*x[1]**o+(-0.952926972714)*x[1]+(-0.117698931152)*x[2]**o+(-0.950120391675)*x[2]
        s[1,3,2,0]=(-0.404765291554)*x[0]**o+(0.0796360913293)*x[0]+(0.743564239837)*x[1]**o+(-0.0431456147708)*x[1]+(-0.82536795966)*x[2]**o+(-0.770711063789)*x[2]
        s[1,3,2,1]=(-0.6113113129)*x[0]**o+(0.330568908804)*x[0]+(-0.391402722722)*x[1]**o+(0.415093002807)*x[1]+(-0.893617348746)*x[2]**o+(0.520968946942)*x[2]
        s[1,4,0,0]=(-0.209574438366)*x[0]**o+(0.644246054163)*x[0]+(-0.0664046919847)*x[1]**o+(0.214820911697)*x[1]+(-0.10545535704)*x[2]**o+(0.655063184877)*x[2]
        s[1,4,0,1]=(-0.38446029244)*x[0]**o+(-0.802691253671)*x[0]+(-0.306211901233)*x[1]**o+(-0.755184562233)*x[1]+(-0.704583961281)*x[2]**o+(0.0410695954026)*x[2]
        s[1,4,1,0]=(-0.512877649374)*x[0]**o+(-0.0190184293267)*x[0]+(0.588904418509)*x[1]**o+(-0.343320898588)*x[1]+(-0.16543066117)*x[2]**o+(-0.118363009949)*x[2]
        s[1,4,1,1]=(-0.410758135711)*x[0]**o+(0.302846037727)*x[0]+(0.674701250578)*x[1]**o+(-0.96295102282)*x[1]+(0.787271275616)*x[2]**o+(0.925818905258)*x[2]
        s[1,4,2,0]=(0.629642717166)*x[0]**o+(-0.696019558128)*x[0]+(0.465609587281)*x[1]**o+(0.191961667012)*x[1]+(-0.0182853530912)*x[2]**o+(0.504352699288)*x[2]
        s[1,4,2,1]=(-0.561295424723)*x[0]**o+(0.830136434947)*x[0]+(0.981595445935)*x[1]**o+(-0.0881426921172)*x[1]+(-0.581790941676)*x[2]**o+(-0.489101727188)*x[2]
        s[2,0,0,0]=(0.173249525275)*x[0]**o+(0.376871332154)*x[0]+(-0.379974081425)*x[1]**o+(-0.118151578515)*x[1]+(0.371715379047)*x[2]**o+(0.115273894116)*x[2]
        s[2,0,0,1]=(0.656339759814)*x[0]**o+(-0.0783529441613)*x[0]+(0.988608900721)*x[1]**o+(-0.498370704737)*x[1]+(0.886731679317)*x[2]**o+(0.251026122418)*x[2]
        s[2,0,1,0]=(-0.350592561402)*x[0]**o+(-0.592294993708)*x[0]+(0.860912193414)*x[1]**o+(0.678591565883)*x[1]+(-0.823185915223)*x[2]**o+(-0.882500202366)*x[2]
        s[2,0,1,1]=(0.84096537128)*x[0]**o+(-0.78582850022)*x[0]+(-0.872411932976)*x[1]**o+(0.208867262377)*x[1]+(0.795227125238)*x[2]**o+(0.434019201479)*x[2]
        s[2,0,2,0]=(0.670038316075)*x[0]**o+(0.221065948244)*x[0]+(-0.0801945207329)*x[1]**o+(0.35963157451)*x[1]+(-0.523905944746)*x[2]**o+(0.113327871154)*x[2]
        s[2,0,2,1]=(-0.790165918677)*x[0]**o+(0.826330759592)*x[0]+(0.816532564314)*x[1]**o+(-0.947088135238)*x[1]+(0.498519891556)*x[2]**o+(-0.221796346503)*x[2]
        s[2,1,0,0]=(0.679814393957)*x[0]**o+(-0.287910028702)*x[0]+(-0.179717372152)*x[1]**o+(-0.895091171351)*x[1]+(0.453459163192)*x[2]**o+(-0.732993783456)*x[2]
        s[2,1,0,1]=(-0.917014070487)*x[0]**o+(-0.28427666054)*x[0]+(0.520172057178)*x[1]**o+(0.973381422092)*x[1]+(-0.785604082848)*x[2]**o+(-0.280246362235)*x[2]
        s[2,1,1,0]=(-0.975543038311)*x[0]**o+(0.0471394295003)*x[0]+(-0.744014701)*x[1]**o+(-0.309281686644)*x[1]+(0.970209409582)*x[2]**o+(0.182638690597)*x[2]
        s[2,1,1,1]=(0.754879118472)*x[0]**o+(-0.345631769168)*x[0]+(0.864272677704)*x[1]**o+(-0.634026748154)*x[1]+(-0.25656646391)*x[2]**o+(-0.976807129917)*x[2]
        s[2,1,2,0]=(0.0378913818133)*x[0]**o+(-0.589418434757)*x[0]+(0.500291612315)*x[1]**o+(0.831173369231)*x[1]+(0.0773865261454)*x[2]**o+(-0.170449265388)*x[2]
        s[2,1,2,1]=(0.363668102897)*x[0]**o+(0.0152347755192)*x[0]+(0.295985339318)*x[1]**o+(-0.695348667793)*x[1]+(0.654035675777)*x[2]**o+(-0.998480773881)*x[2]
        s[2,2,0,0]=(-0.248968924843)*x[0]**o+(-0.454040417829)*x[0]+(0.618989286259)*x[1]**o+(-0.284627664137)*x[1]+(0.597922789444)*x[2]**o+(-0.0544707719136)*x[2]
        s[2,2,0,1]=(-0.564540493714)*x[0]**o+(0.948462945081)*x[0]+(-0.554305925084)*x[1]**o+(-0.683216336448)*x[1]+(-0.393720863511)*x[2]**o+(-0.454706611434)*x[2]
        s[2,2,1,0]=(0.00267926298461)*x[0]**o+(0.69057345924)*x[0]+(0.972990086053)*x[1]**o+(0.334585200812)*x[1]+(0.458913228071)*x[2]**o+(-0.927443691914)*x[2]
        s[2,2,1,1]=(-0.869817596024)*x[0]**o+(0.183295661514)*x[0]+(-0.215314084509)*x[1]**o+(0.274879624616)*x[1]+(-0.872459015851)*x[2]**o+(-0.43371203901)*x[2]
        s[2,2,2,0]=(-0.59916469262)*x[0]**o+(-0.635232312962)*x[0]+(-0.730547369574)*x[1]**o+(-0.199910106304)*x[1]+(-0.81824698793)*x[2]**o+(-0.351742356136)*x[2]
        s[2,2,2,1]=(0.52115654452)*x[0]**o+(-0.648726400371)*x[0]+(0.0705029035387)*x[1]**o+(0.82069581623)*x[1]+(0.262897099948)*x[2]**o+(0.302543200334)*x[2]
        s[2,3,0,0]=(-0.887996543825)*x[0]**o+(-0.141917392835)*x[0]+(0.458379292046)*x[1]**o+(0.279749722412)*x[1]+(-0.185582606045)*x[2]**o+(-0.408626717355)*x[2]
        s[2,3,0,1]=(-0.0708646458785)*x[0]**o+(-0.723087450876)*x[0]+(0.801053779885)*x[1]**o+(-0.316692437663)*x[1]+(0.313620016387)*x[2]**o+(-0.802237492322)*x[2]
        s[2,3,1,0]=(-0.413969268245)*x[0]**o+(-0.419189503807)*x[0]+(-0.150655589102)*x[1]**o+(-0.451064114786)*x[1]+(-0.11752330372)*x[2]**o+(-0.564979364641)*x[2]
        s[2,3,1,1]=(-0.127258808057)*x[0]**o+(-0.166240866424)*x[0]+(0.482435262134)*x[1]**o+(0.210083811762)*x[1]+(0.496523704735)*x[2]**o+(-0.528305427062)*x[2]
        s[2,3,2,0]=(-0.619368230074)*x[0]**o+(0.196929842772)*x[0]+(0.45226305027)*x[1]**o+(-0.949509654146)*x[1]+(-0.200199092979)*x[2]**o+(0.879368741494)*x[2]
        s[2,3,2,1]=(0.727974979035)*x[0]**o+(-0.486118068882)*x[0]+(0.330990540492)*x[1]**o+(0.627547710515)*x[1]+(-0.00312667743686)*x[2]**o+(0.34180677866)*x[2]
        s[2,4,0,0]=(-0.849340738634)*x[0]**o+(0.105575131464)*x[0]+(0.531365100651)*x[1]**o+(-0.885829562648)*x[1]+(-0.959105112582)*x[2]**o+(0.6210710685)*x[2]
        s[2,4,0,1]=(-0.525287919972)*x[0]**o+(0.685382709266)*x[0]+(0.706077262452)*x[1]**o+(-0.577859356102)*x[1]+(0.494281341217)*x[2]**o+(0.936337201867)*x[2]
        s[2,4,1,0]=(-0.0263664228096)*x[0]**o+(0.609335710155)*x[0]+(-0.417542795725)*x[1]**o+(0.127890280292)*x[1]+(0.0530446261715)*x[2]**o+(-0.203436845419)*x[2]
        s[2,4,1,1]=(0.58910765961)*x[0]**o+(-0.233711911277)*x[0]+(-0.68506108197)*x[1]**o+(-0.796608219305)*x[1]+(-0.55051026522)*x[2]**o+(-0.663284299983)*x[2]
        s[2,4,2,0]=(0.407867058666)*x[0]**o+(-0.155472572874)*x[0]+(0.0574795179339)*x[1]**o+(0.534562635965)*x[1]+(-0.35303239373)*x[2]**o+(0.690463242713)*x[2]
        s[2,4,2,1]=(0.458514172477)*x[0]**o+(0.493538792362)*x[0]+(-0.534087073406)*x[1]**o+(0.140190942794)*x[1]+(0.53904059561)*x[2]**o+(0.511924245166)*x[2]
        s[3,0,0,0]=(0.80688664483)*x[0]**o+(0.641990457435)*x[0]+(0.258439350653)*x[1]**o+(0.360949512497)*x[1]+(0.71055262439)*x[2]**o+(-0.14812833383)*x[2]
        s[3,0,0,1]=(0.349076196526)*x[0]**o+(-0.250426445203)*x[0]+(0.363350736371)*x[1]**o+(0.62331683755)*x[1]+(-0.799481730613)*x[2]**o+(0.86946545402)*x[2]
        s[3,0,1,0]=(-0.461900551752)*x[0]**o+(-0.530275233976)*x[0]+(0.778112655963)*x[1]**o+(-0.548160947996)*x[1]+(0.27643541045)*x[2]**o+(0.558519627971)*x[2]
        s[3,0,1,1]=(-0.870845434837)*x[0]**o+(0.90473098031)*x[0]+(0.356324338565)*x[1]**o+(-0.173579775601)*x[1]+(0.866960109024)*x[2]**o+(0.22810880895)*x[2]
        s[3,0,2,0]=(0.449562471445)*x[0]**o+(-0.133977676845)*x[0]+(-0.945220890263)*x[1]**o+(0.999013496171)*x[1]+(-0.612837266133)*x[2]**o+(-0.695797874487)*x[2]
        s[3,0,2,1]=(-0.588105630499)*x[0]**o+(0.176592342414)*x[0]+(-0.820991814796)*x[1]**o+(0.0010811086852)*x[1]+(0.498174763907)*x[2]**o+(-0.0223969132485)*x[2]
        s[3,1,0,0]=(0.653483773188)*x[0]**o+(0.460215104331)*x[0]+(-0.287173677147)*x[1]**o+(-0.837222104854)*x[1]+(0.457524748826)*x[2]**o+(-0.638268772518)*x[2]
        s[3,1,0,1]=(-0.0999951386915)*x[0]**o+(0.294288175142)*x[0]+(0.159178572562)*x[1]**o+(0.13437803751)*x[1]+(-0.411939992038)*x[2]**o+(0.283003770703)*x[2]
        s[3,1,1,0]=(-0.862929516639)*x[0]**o+(-0.455798937374)*x[0]+(0.972383754049)*x[1]**o+(0.251968652813)*x[1]+(-0.631965845999)*x[2]**o+(-0.418936218512)*x[2]
        s[3,1,1,1]=(-0.0265533722079)*x[0]**o+(-0.720054910456)*x[0]+(-0.134379948062)*x[1]**o+(-0.535223772178)*x[1]+(-0.711365379454)*x[2]**o+(0.39963284442)*x[2]
        s[3,1,2,0]=(-0.9691936764)*x[0]**o+(-0.220678275504)*x[0]+(0.423830480119)*x[1]**o+(-0.286816976029)*x[1]+(-0.637835943762)*x[2]**o+(0.389581647726)*x[2]
        s[3,1,2,1]=(-0.375225433223)*x[0]**o+(0.339950113552)*x[0]+(0.272089140767)*x[1]**o+(-0.671811329081)*x[1]+(-0.341834415939)*x[2]**o+(-0.974852723909)*x[2]
        s[3,2,0,0]=(-0.13212102165)*x[0]**o+(-0.529435682355)*x[0]+(0.57321409695)*x[1]**o+(0.123050260287)*x[1]+(-0.50608599056)*x[2]**o+(-0.258999018643)*x[2]
        s[3,2,0,1]=(-0.594320198162)*x[0]**o+(0.59617231387)*x[0]+(-0.132055038377)*x[1]**o+(0.0267055057967)*x[1]+(-0.0909023410879)*x[2]**o+(-0.327689716933)*x[2]
        s[3,2,1,0]=(0.352248348539)*x[0]**o+(-0.937172168074)*x[0]+(0.492884278868)*x[1]**o+(0.599164015797)*x[1]+(-0.306391881602)*x[2]**o+(-0.124914738757)*x[2]
        s[3,2,1,1]=(-0.124766143355)*x[0]**o+(-0.201629423485)*x[0]+(0.69199028272)*x[1]**o+(0.0909253140313)*x[1]+(-0.0807463990698)*x[2]**o+(0.274206212499)*x[2]
        s[3,2,2,0]=(-0.939257892238)*x[0]**o+(-0.0905023289297)*x[0]+(-0.101189471915)*x[1]**o+(-0.850452366024)*x[1]+(0.706374760942)*x[2]**o+(-0.0741989595692)*x[2]
        s[3,2,2,1]=(-0.956776001745)*x[0]**o+(0.313587985523)*x[0]+(0.501857466469)*x[1]**o+(0.866521730616)*x[1]+(-0.113777691526)*x[2]**o+(-0.43724474427)*x[2]
        s[3,3,0,0]=(-0.794240122389)*x[0]**o+(0.117246192404)*x[0]+(0.730901073854)*x[1]**o+(0.406753741761)*x[1]+(-0.987815418361)*x[2]**o+(0.543031203217)*x[2]
        s[3,3,0,1]=(-0.539899807208)*x[0]**o+(0.425096558373)*x[0]+(-0.261756005984)*x[1]**o+(0.219196945781)*x[1]+(-0.458238548056)*x[2]**o+(-0.396891207522)*x[2]
        s[3,3,1,0]=(-0.512322191935)*x[0]**o+(0.158356684082)*x[0]+(-0.702591537714)*x[1]**o+(0.202771352983)*x[1]+(-0.346772902785)*x[2]**o+(0.260366433089)*x[2]
        s[3,3,1,1]=(0.839858482584)*x[0]**o+(0.132933571004)*x[0]+(0.840682050408)*x[1]**o+(0.490904029834)*x[1]+(-0.298290729461)*x[2]**o+(-0.467229890802)*x[2]
        s[3,3,2,0]=(-0.691489850326)*x[0]**o+(0.852014173253)*x[0]+(0.484568640775)*x[1]**o+(0.0788611533234)*x[1]+(0.335927774531)*x[2]**o+(-0.507648906088)*x[2]
        s[3,3,2,1]=(0.140576758316)*x[0]**o+(-0.783440190349)*x[0]+(-0.408503163319)*x[1]**o+(0.0871935010894)*x[1]+(0.478151373206)*x[2]**o+(0.837020241411)*x[2]
        s[3,4,0,0]=(-0.956822615698)*x[0]**o+(0.594397877716)*x[0]+(0.844357571956)*x[1]**o+(0.173315874424)*x[1]+(0.96181728365)*x[2]**o+(-0.900445532048)*x[2]
        s[3,4,0,1]=(0.887824966747)*x[0]**o+(-0.579258071661)*x[0]+(0.983311456722)*x[1]**o+(0.996052998934)*x[1]+(0.215867343112)*x[2]**o+(0.614855622355)*x[2]
        s[3,4,1,0]=(0.482666172028)*x[0]**o+(0.0258626246719)*x[0]+(-0.394140399663)*x[1]**o+(0.192974809281)*x[1]+(-0.265879383852)*x[2]**o+(-0.0409441022179)*x[2]
        s[3,4,1,1]=(0.0600752930856)*x[0]**o+(-0.885644457634)*x[0]+(-0.54869761625)*x[1]**o+(0.4745411417)*x[1]+(-0.554764519536)*x[2]**o+(-0.159514159559)*x[2]
        s[3,4,2,0]=(-0.804675220841)*x[0]**o+(0.586839632342)*x[0]+(-0.329997060623)*x[1]**o+(0.242316008423)*x[1]+(0.272064229179)*x[2]**o+(0.24348627845)*x[2]
        s[3,4,2,1]=(0.847664079563)*x[0]**o+(0.927723091901)*x[0]+(-0.861048553467)*x[1]**o+(-0.413216499555)*x[1]+(0.546938488463)*x[2]**o+(0.0558689143145)*x[2]
        ref[0,0,0,0]=(-0.0362500650077)*x_ref[0]**o+(0.692407811012)*x_ref[0]+(-0.855107417704)*x_ref[1]**o+(-0.0397854200762)*x_ref[1]+(0.488177999316)*x_ref[2]**o+(0.946478266036)*x_ref[2]
        ref[0,0,0,1]=(-0.533284434038)*x_ref[0]**o+(-0.599290705017)*x_ref[0]+(0.198922961617)*x_ref[1]**o+(0.652839426276)*x_ref[1]+(0.560109621832)*x_ref[2]**o+(0.167310997616)*x_ref[2]
        ref[0,0,1,0]=(-0.21016675446)*x_ref[0]**o+(0.812498932794)*x_ref[0]+(-0.611566184558)*x_ref[1]**o+(0.626231122402)*x_ref[1]+(0.536374628942)*x_ref[2]**o+(0.8520547245)*x_ref[2]
        ref[0,0,1,1]=(0.26308571414)*x_ref[0]**o+(-0.359237396089)*x_ref[0]+(0.146879292748)*x_ref[1]**o+(-0.241831631389)*x_ref[1]+(0.812820522163)*x_ref[2]**o+(0.824191692943)*x_ref[2]
        ref[0,0,2,0]=(-0.585682391332)*x_ref[0]**o+(0.82273803111)*x_ref[0]+(-0.715688315155)*x_ref[1]**o+(-0.997893564593)*x_ref[1]+(0.39588322481)*x_ref[2]**o+(0.410447772982)*x_ref[2]
        ref[0,0,2,1]=(0.491817354047)*x_ref[0]**o+(0.105289616977)*x_ref[0]+(0.0581932965006)*x_ref[1]**o+(-0.584153248946)*x_ref[1]+(-0.837305609444)*x_ref[2]**o+(-0.140188544108)*x_ref[2]
        ref[0,1,0,0]=(-0.158174387659)*x_ref[0]**o+(0.774512978962)*x_ref[0]+(0.953772629099)*x_ref[1]**o+(-0.0474582135671)*x_ref[1]+(-0.691917863221)*x_ref[2]**o+(0.978550525536)*x_ref[2]
        ref[0,1,0,1]=(-0.696002534433)*x_ref[0]**o+(-0.904833590237)*x_ref[0]+(-0.991879336542)*x_ref[1]**o+(-0.00301423741703)*x_ref[1]+(-0.495919957691)*x_ref[2]**o+(0.923872653662)*x_ref[2]
        ref[0,1,1,0]=(-0.617187346815)*x_ref[0]**o+(-0.0283413268672)*x_ref[0]+(0.439716253425)*x_ref[1]**o+(0.679148169861)*x_ref[1]+(-0.5494839537)*x_ref[2]**o+(0.780531698733)*x_ref[2]
        ref[0,1,1,1]=(0.0763086894431)*x_ref[0]**o+(0.149024988807)*x_ref[0]+(0.660061724201)*x_ref[1]**o+(0.227260856773)*x_ref[1]+(0.604941557505)*x_ref[2]**o+(-0.32106743372)*x_ref[2]
        ref[0,1,2,0]=(-0.710651974651)*x_ref[0]**o+(0.729233023268)*x_ref[0]+(0.830588938931)*x_ref[1]**o+(0.930100931807)*x_ref[1]+(0.576259601615)*x_ref[2]**o+(0.995388406293)*x_ref[2]
        ref[0,1,2,1]=(0.635415205147)*x_ref[0]**o+(-0.0475242382825)*x_ref[0]+(0.779184290843)*x_ref[1]**o+(0.161316071106)*x_ref[1]+(-0.321426515539)*x_ref[2]**o+(-0.188018565147)*x_ref[2]
        ref[0,2,0,0]=(-0.234573769896)*x_ref[0]**o+(-0.35461213258)*x_ref[0]+(0.322664245412)*x_ref[1]**o+(-0.36584459555)*x_ref[1]+(0.656560085187)*x_ref[2]**o+(-0.380544830082)*x_ref[2]
        ref[0,2,0,1]=(0.314171153213)*x_ref[0]**o+(-0.871055875712)*x_ref[0]+(0.600399452124)*x_ref[1]**o+(-0.710939791885)*x_ref[1]+(-0.189029597187)*x_ref[2]**o+(-0.29466779727)*x_ref[2]
        ref[0,2,1,0]=(0.911500533914)*x_ref[0]**o+(-0.217077039937)*x_ref[0]+(0.488325722425)*x_ref[1]**o+(0.781071969291)*x_ref[1]+(-0.883567213205)*x_ref[2]**o+(0.938457876454)*x_ref[2]
        ref[0,2,1,1]=(0.947554813988)*x_ref[0]**o+(0.108161094029)*x_ref[0]+(0.0814722282926)*x_ref[1]**o+(0.394738648377)*x_ref[1]+(-0.585604351258)*x_ref[2]**o+(0.424262882559)*x_ref[2]
        ref[0,2,2,0]=(0.130793077586)*x_ref[0]**o+(-0.0759869806951)*x_ref[0]+(0.524572237994)*x_ref[1]**o+(-0.0494098932334)*x_ref[1]+(-0.162573831498)*x_ref[2]**o+(0.950665975392)*x_ref[2]
        ref[0,2,2,1]=(0.0636110845427)*x_ref[0]**o+(-0.152586932248)*x_ref[0]+(-0.428472289058)*x_ref[1]**o+(0.943093556156)*x_ref[1]+(-0.925181631256)*x_ref[2]**o+(-0.850747416138)*x_ref[2]
        ref[0,3,0,0]=(0.803654693645)*x_ref[0]**o+(-0.852793510169)*x_ref[0]+(0.372325502382)*x_ref[1]**o+(0.437191610865)*x_ref[1]+(-0.626819779481)*x_ref[2]**o+(-0.969085192013)*x_ref[2]
        ref[0,3,0,1]=(-0.498449247588)*x_ref[0]**o+(-0.174254842954)*x_ref[0]+(-0.751570511608)*x_ref[1]**o+(0.0278714978689)*x_ref[1]+(0.732061097781)*x_ref[2]**o+(0.434922200273)*x_ref[2]
        ref[0,3,1,0]=(0.356966448806)*x_ref[0]**o+(-0.754346917896)*x_ref[0]+(-0.200096880735)*x_ref[1]**o+(0.726202986575)*x_ref[1]+(-0.273975127289)*x_ref[2]**o+(0.922054852117)*x_ref[2]
        ref[0,3,1,1]=(0.294033872459)*x_ref[0]**o+(0.435258372604)*x_ref[0]+(-0.606222342027)*x_ref[1]**o+(-0.418222134998)*x_ref[1]+(-0.617263336049)*x_ref[2]**o+(-0.655912764464)*x_ref[2]
        ref[0,3,2,0]=(-0.552805093123)*x_ref[0]**o+(0.814985531695)*x_ref[0]+(0.201701287999)*x_ref[1]**o+(0.449378802496)*x_ref[1]+(0.106877031271)*x_ref[2]**o+(0.972523159602)*x_ref[2]
        ref[0,3,2,1]=(-0.123112956117)*x_ref[0]**o+(-0.9669799128)*x_ref[0]+(-0.365437347923)*x_ref[1]**o+(0.95077682916)*x_ref[1]+(-0.68795960948)*x_ref[2]**o+(0.282323391952)*x_ref[2]
        ref[0,4,0,0]=(0.762949486092)*x_ref[0]**o+(0.275310784251)*x_ref[0]+(0.90824072581)*x_ref[1]**o+(0.267277710972)*x_ref[1]+(-0.387183781105)*x_ref[2]**o+(-0.919714964098)*x_ref[2]
        ref[0,4,0,1]=(-0.219860124563)*x_ref[0]**o+(0.458617314398)*x_ref[0]+(-0.253950962469)*x_ref[1]**o+(-0.213326656667)*x_ref[1]+(-0.914189191308)*x_ref[2]**o+(-0.88808493928)*x_ref[2]
        ref[0,4,1,0]=(0.0111879656405)*x_ref[0]**o+(0.857083093627)*x_ref[0]+(0.397087247889)*x_ref[1]**o+(-0.411624885427)*x_ref[1]+(-0.533535312833)*x_ref[2]**o+(-0.473765290954)*x_ref[2]
        ref[0,4,1,1]=(0.408477302242)*x_ref[0]**o+(-0.336255491206)*x_ref[0]+(0.535004316057)*x_ref[1]**o+(0.646300732798)*x_ref[1]+(-0.683570698066)*x_ref[2]**o+(-0.691723219637)*x_ref[2]
        ref[0,4,2,0]=(0.229517234528)*x_ref[0]**o+(-0.327638251614)*x_ref[0]+(0.752536339629)*x_ref[1]**o+(0.440600000033)*x_ref[1]+(-0.166203457183)*x_ref[2]**o+(-0.632615828058)*x_ref[2]
        ref[0,4,2,1]=(0.128453646267)*x_ref[0]**o+(-0.152501701527)*x_ref[0]+(0.131616948331)*x_ref[1]**o+(-0.560356219781)*x_ref[1]+(0.369923630213)*x_ref[2]**o+(-0.405937225725)*x_ref[2]
        ref[1,0,0,0]=(-0.498802287823)*x_ref[0]**o+(0.974250100105)*x_ref[0]+(-0.752335049339)*x_ref[1]**o+(0.460106742976)*x_ref[1]+(0.878914200699)*x_ref[2]**o+(-0.85963539732)*x_ref[2]
        ref[1,0,0,1]=(0.937345235384)*x_ref[0]**o+(0.0616042350221)*x_ref[0]+(-0.63597655214)*x_ref[1]**o+(0.200363730821)*x_ref[1]+(0.677520498976)*x_ref[2]**o+(0.310367850218)*x_ref[2]
        ref[1,0,1,0]=(-0.328289664191)*x_ref[0]**o+(-0.439002316683)*x_ref[0]+(-0.753405240499)*x_ref[1]**o+(-0.749020227582)*x_ref[1]+(0.200333899797)*x_ref[2]**o+(0.63068525957)*x_ref[2]
        ref[1,0,1,1]=(-0.522381373851)*x_ref[0]**o+(-0.877707833729)*x_ref[0]+(-0.0334841793702)*x_ref[1]**o+(0.00769816693416)*x_ref[1]+(-0.81697893166)*x_ref[2]**o+(-0.47349699259)*x_ref[2]
        ref[1,0,2,0]=(0.0786863277722)*x_ref[0]**o+(-0.47462306827)*x_ref[0]+(-0.850065158625)*x_ref[1]**o+(0.0481804762762)*x_ref[1]+(0.239465148926)*x_ref[2]**o+(0.484166919183)*x_ref[2]
        ref[1,0,2,1]=(-0.0411433282485)*x_ref[0]**o+(-0.851343034379)*x_ref[0]+(-0.518404507699)*x_ref[1]**o+(-0.357342937948)*x_ref[1]+(-0.57398214485)*x_ref[2]**o+(-0.0989103206187)*x_ref[2]
        ref[1,1,0,0]=(-0.060737698278)*x_ref[0]**o+(-0.103483640965)*x_ref[0]+(-0.206004957088)*x_ref[1]**o+(0.556901021479)*x_ref[1]+(0.951054830915)*x_ref[2]**o+(0.668118573076)*x_ref[2]
        ref[1,1,0,1]=(0.0668860744649)*x_ref[0]**o+(0.0490910382588)*x_ref[0]+(0.817265796062)*x_ref[1]**o+(-0.939419683478)*x_ref[1]+(-0.800905110922)*x_ref[2]**o+(-0.546647838589)*x_ref[2]
        ref[1,1,1,0]=(-0.437101637549)*x_ref[0]**o+(0.269260162519)*x_ref[0]+(-0.026802186298)*x_ref[1]**o+(-0.932589891991)*x_ref[1]+(-0.313814820129)*x_ref[2]**o+(-0.436056943196)*x_ref[2]
        ref[1,1,1,1]=(-0.893118860321)*x_ref[0]**o+(-0.837193118481)*x_ref[0]+(-0.58313090725)*x_ref[1]**o+(-0.657017739851)*x_ref[1]+(-0.0158333033033)*x_ref[2]**o+(0.286927848108)*x_ref[2]
        ref[1,1,2,0]=(-0.414914106956)*x_ref[0]**o+(-0.295021056636)*x_ref[0]+(-0.100513024651)*x_ref[1]**o+(0.119321919323)*x_ref[1]+(-0.911626057698)*x_ref[2]**o+(0.0715733685275)*x_ref[2]
        ref[1,1,2,1]=(0.415073119201)*x_ref[0]**o+(-0.79060072691)*x_ref[0]+(-0.280560285152)*x_ref[1]**o+(-0.651271923585)*x_ref[1]+(-0.783523725291)*x_ref[2]**o+(-0.165699899479)*x_ref[2]
        ref[1,2,0,0]=(0.204696440606)*x_ref[0]**o+(0.58070882041)*x_ref[0]+(0.498396642996)*x_ref[1]**o+(0.270570176185)*x_ref[1]+(-0.261604910878)*x_ref[2]**o+(0.788402570665)*x_ref[2]
        ref[1,2,0,1]=(-0.163227090863)*x_ref[0]**o+(0.17193387282)*x_ref[0]+(0.559640507601)*x_ref[1]**o+(0.39180422655)*x_ref[1]+(0.182426715866)*x_ref[2]**o+(0.52544860689)*x_ref[2]
        ref[1,2,1,0]=(-0.0826612289833)*x_ref[0]**o+(-0.244737952715)*x_ref[0]+(0.887738370973)*x_ref[1]**o+(0.593438512127)*x_ref[1]+(0.73128277327)*x_ref[2]**o+(0.229067764431)*x_ref[2]
        ref[1,2,1,1]=(-0.117634040351)*x_ref[0]**o+(-0.490365253501)*x_ref[0]+(0.654067544802)*x_ref[1]**o+(-0.862866217687)*x_ref[1]+(-0.563450922747)*x_ref[2]**o+(-0.785940469253)*x_ref[2]
        ref[1,2,2,0]=(0.486381246904)*x_ref[0]**o+(-0.119137386359)*x_ref[0]+(0.251388315083)*x_ref[1]**o+(-0.142435376157)*x_ref[1]+(-0.882418773242)*x_ref[2]**o+(0.509436056026)*x_ref[2]
        ref[1,2,2,1]=(-0.69532427028)*x_ref[0]**o+(-0.587331382396)*x_ref[0]+(0.750092712907)*x_ref[1]**o+(0.864188183197)*x_ref[1]+(-0.0336735158745)*x_ref[2]**o+(-0.118692941484)*x_ref[2]
        ref[1,3,0,0]=(0.287194598069)*x_ref[0]**o+(0.775480786321)*x_ref[0]+(0.24820537283)*x_ref[1]**o+(0.579330345046)*x_ref[1]+(0.641058615756)*x_ref[2]**o+(-0.358716444584)*x_ref[2]
        ref[1,3,0,1]=(-0.218876040408)*x_ref[0]**o+(-0.581134176647)*x_ref[0]+(0.47937196949)*x_ref[1]**o+(0.0529063680602)*x_ref[1]+(-0.42659429504)*x_ref[2]**o+(-0.445845798304)*x_ref[2]
        ref[1,3,1,0]=(0.500331134532)*x_ref[0]**o+(0.764920932873)*x_ref[0]+(0.256972231189)*x_ref[1]**o+(0.279967260143)*x_ref[1]+(0.90143027778)*x_ref[2]**o+(-0.753075337423)*x_ref[2]
        ref[1,3,1,1]=(0.0478484319999)*x_ref[0]**o+(0.239476880568)*x_ref[0]+(-0.286765457909)*x_ref[1]**o+(-0.952926972714)*x_ref[1]+(-0.117698931152)*x_ref[2]**o+(-0.950120391675)*x_ref[2]
        ref[1,3,2,0]=(-0.404765291554)*x_ref[0]**o+(0.0796360913293)*x_ref[0]+(0.743564239837)*x_ref[1]**o+(-0.0431456147708)*x_ref[1]+(-0.82536795966)*x_ref[2]**o+(-0.770711063789)*x_ref[2]
        ref[1,3,2,1]=(-0.6113113129)*x_ref[0]**o+(0.330568908804)*x_ref[0]+(-0.391402722722)*x_ref[1]**o+(0.415093002807)*x_ref[1]+(-0.893617348746)*x_ref[2]**o+(0.520968946942)*x_ref[2]
        ref[1,4,0,0]=(-0.209574438366)*x_ref[0]**o+(0.644246054163)*x_ref[0]+(-0.0664046919847)*x_ref[1]**o+(0.214820911697)*x_ref[1]+(-0.10545535704)*x_ref[2]**o+(0.655063184877)*x_ref[2]
        ref[1,4,0,1]=(-0.38446029244)*x_ref[0]**o+(-0.802691253671)*x_ref[0]+(-0.306211901233)*x_ref[1]**o+(-0.755184562233)*x_ref[1]+(-0.704583961281)*x_ref[2]**o+(0.0410695954026)*x_ref[2]
        ref[1,4,1,0]=(-0.512877649374)*x_ref[0]**o+(-0.0190184293267)*x_ref[0]+(0.588904418509)*x_ref[1]**o+(-0.343320898588)*x_ref[1]+(-0.16543066117)*x_ref[2]**o+(-0.118363009949)*x_ref[2]
        ref[1,4,1,1]=(-0.410758135711)*x_ref[0]**o+(0.302846037727)*x_ref[0]+(0.674701250578)*x_ref[1]**o+(-0.96295102282)*x_ref[1]+(0.787271275616)*x_ref[2]**o+(0.925818905258)*x_ref[2]
        ref[1,4,2,0]=(0.629642717166)*x_ref[0]**o+(-0.696019558128)*x_ref[0]+(0.465609587281)*x_ref[1]**o+(0.191961667012)*x_ref[1]+(-0.0182853530912)*x_ref[2]**o+(0.504352699288)*x_ref[2]
        ref[1,4,2,1]=(-0.561295424723)*x_ref[0]**o+(0.830136434947)*x_ref[0]+(0.981595445935)*x_ref[1]**o+(-0.0881426921172)*x_ref[1]+(-0.581790941676)*x_ref[2]**o+(-0.489101727188)*x_ref[2]
        ref[2,0,0,0]=(0.173249525275)*x_ref[0]**o+(0.376871332154)*x_ref[0]+(-0.379974081425)*x_ref[1]**o+(-0.118151578515)*x_ref[1]+(0.371715379047)*x_ref[2]**o+(0.115273894116)*x_ref[2]
        ref[2,0,0,1]=(0.656339759814)*x_ref[0]**o+(-0.0783529441613)*x_ref[0]+(0.988608900721)*x_ref[1]**o+(-0.498370704737)*x_ref[1]+(0.886731679317)*x_ref[2]**o+(0.251026122418)*x_ref[2]
        ref[2,0,1,0]=(-0.350592561402)*x_ref[0]**o+(-0.592294993708)*x_ref[0]+(0.860912193414)*x_ref[1]**o+(0.678591565883)*x_ref[1]+(-0.823185915223)*x_ref[2]**o+(-0.882500202366)*x_ref[2]
        ref[2,0,1,1]=(0.84096537128)*x_ref[0]**o+(-0.78582850022)*x_ref[0]+(-0.872411932976)*x_ref[1]**o+(0.208867262377)*x_ref[1]+(0.795227125238)*x_ref[2]**o+(0.434019201479)*x_ref[2]
        ref[2,0,2,0]=(0.670038316075)*x_ref[0]**o+(0.221065948244)*x_ref[0]+(-0.0801945207329)*x_ref[1]**o+(0.35963157451)*x_ref[1]+(-0.523905944746)*x_ref[2]**o+(0.113327871154)*x_ref[2]
        ref[2,0,2,1]=(-0.790165918677)*x_ref[0]**o+(0.826330759592)*x_ref[0]+(0.816532564314)*x_ref[1]**o+(-0.947088135238)*x_ref[1]+(0.498519891556)*x_ref[2]**o+(-0.221796346503)*x_ref[2]
        ref[2,1,0,0]=(0.679814393957)*x_ref[0]**o+(-0.287910028702)*x_ref[0]+(-0.179717372152)*x_ref[1]**o+(-0.895091171351)*x_ref[1]+(0.453459163192)*x_ref[2]**o+(-0.732993783456)*x_ref[2]
        ref[2,1,0,1]=(-0.917014070487)*x_ref[0]**o+(-0.28427666054)*x_ref[0]+(0.520172057178)*x_ref[1]**o+(0.973381422092)*x_ref[1]+(-0.785604082848)*x_ref[2]**o+(-0.280246362235)*x_ref[2]
        ref[2,1,1,0]=(-0.975543038311)*x_ref[0]**o+(0.0471394295003)*x_ref[0]+(-0.744014701)*x_ref[1]**o+(-0.309281686644)*x_ref[1]+(0.970209409582)*x_ref[2]**o+(0.182638690597)*x_ref[2]
        ref[2,1,1,1]=(0.754879118472)*x_ref[0]**o+(-0.345631769168)*x_ref[0]+(0.864272677704)*x_ref[1]**o+(-0.634026748154)*x_ref[1]+(-0.25656646391)*x_ref[2]**o+(-0.976807129917)*x_ref[2]
        ref[2,1,2,0]=(0.0378913818133)*x_ref[0]**o+(-0.589418434757)*x_ref[0]+(0.500291612315)*x_ref[1]**o+(0.831173369231)*x_ref[1]+(0.0773865261454)*x_ref[2]**o+(-0.170449265388)*x_ref[2]
        ref[2,1,2,1]=(0.363668102897)*x_ref[0]**o+(0.0152347755192)*x_ref[0]+(0.295985339318)*x_ref[1]**o+(-0.695348667793)*x_ref[1]+(0.654035675777)*x_ref[2]**o+(-0.998480773881)*x_ref[2]
        ref[2,2,0,0]=(-0.248968924843)*x_ref[0]**o+(-0.454040417829)*x_ref[0]+(0.618989286259)*x_ref[1]**o+(-0.284627664137)*x_ref[1]+(0.597922789444)*x_ref[2]**o+(-0.0544707719136)*x_ref[2]
        ref[2,2,0,1]=(-0.564540493714)*x_ref[0]**o+(0.948462945081)*x_ref[0]+(-0.554305925084)*x_ref[1]**o+(-0.683216336448)*x_ref[1]+(-0.393720863511)*x_ref[2]**o+(-0.454706611434)*x_ref[2]
        ref[2,2,1,0]=(0.00267926298461)*x_ref[0]**o+(0.69057345924)*x_ref[0]+(0.972990086053)*x_ref[1]**o+(0.334585200812)*x_ref[1]+(0.458913228071)*x_ref[2]**o+(-0.927443691914)*x_ref[2]
        ref[2,2,1,1]=(-0.869817596024)*x_ref[0]**o+(0.183295661514)*x_ref[0]+(-0.215314084509)*x_ref[1]**o+(0.274879624616)*x_ref[1]+(-0.872459015851)*x_ref[2]**o+(-0.43371203901)*x_ref[2]
        ref[2,2,2,0]=(-0.59916469262)*x_ref[0]**o+(-0.635232312962)*x_ref[0]+(-0.730547369574)*x_ref[1]**o+(-0.199910106304)*x_ref[1]+(-0.81824698793)*x_ref[2]**o+(-0.351742356136)*x_ref[2]
        ref[2,2,2,1]=(0.52115654452)*x_ref[0]**o+(-0.648726400371)*x_ref[0]+(0.0705029035387)*x_ref[1]**o+(0.82069581623)*x_ref[1]+(0.262897099948)*x_ref[2]**o+(0.302543200334)*x_ref[2]
        ref[2,3,0,0]=(-0.887996543825)*x_ref[0]**o+(-0.141917392835)*x_ref[0]+(0.458379292046)*x_ref[1]**o+(0.279749722412)*x_ref[1]+(-0.185582606045)*x_ref[2]**o+(-0.408626717355)*x_ref[2]
        ref[2,3,0,1]=(-0.0708646458785)*x_ref[0]**o+(-0.723087450876)*x_ref[0]+(0.801053779885)*x_ref[1]**o+(-0.316692437663)*x_ref[1]+(0.313620016387)*x_ref[2]**o+(-0.802237492322)*x_ref[2]
        ref[2,3,1,0]=(-0.413969268245)*x_ref[0]**o+(-0.419189503807)*x_ref[0]+(-0.150655589102)*x_ref[1]**o+(-0.451064114786)*x_ref[1]+(-0.11752330372)*x_ref[2]**o+(-0.564979364641)*x_ref[2]
        ref[2,3,1,1]=(-0.127258808057)*x_ref[0]**o+(-0.166240866424)*x_ref[0]+(0.482435262134)*x_ref[1]**o+(0.210083811762)*x_ref[1]+(0.496523704735)*x_ref[2]**o+(-0.528305427062)*x_ref[2]
        ref[2,3,2,0]=(-0.619368230074)*x_ref[0]**o+(0.196929842772)*x_ref[0]+(0.45226305027)*x_ref[1]**o+(-0.949509654146)*x_ref[1]+(-0.200199092979)*x_ref[2]**o+(0.879368741494)*x_ref[2]
        ref[2,3,2,1]=(0.727974979035)*x_ref[0]**o+(-0.486118068882)*x_ref[0]+(0.330990540492)*x_ref[1]**o+(0.627547710515)*x_ref[1]+(-0.00312667743686)*x_ref[2]**o+(0.34180677866)*x_ref[2]
        ref[2,4,0,0]=(-0.849340738634)*x_ref[0]**o+(0.105575131464)*x_ref[0]+(0.531365100651)*x_ref[1]**o+(-0.885829562648)*x_ref[1]+(-0.959105112582)*x_ref[2]**o+(0.6210710685)*x_ref[2]
        ref[2,4,0,1]=(-0.525287919972)*x_ref[0]**o+(0.685382709266)*x_ref[0]+(0.706077262452)*x_ref[1]**o+(-0.577859356102)*x_ref[1]+(0.494281341217)*x_ref[2]**o+(0.936337201867)*x_ref[2]
        ref[2,4,1,0]=(-0.0263664228096)*x_ref[0]**o+(0.609335710155)*x_ref[0]+(-0.417542795725)*x_ref[1]**o+(0.127890280292)*x_ref[1]+(0.0530446261715)*x_ref[2]**o+(-0.203436845419)*x_ref[2]
        ref[2,4,1,1]=(0.58910765961)*x_ref[0]**o+(-0.233711911277)*x_ref[0]+(-0.68506108197)*x_ref[1]**o+(-0.796608219305)*x_ref[1]+(-0.55051026522)*x_ref[2]**o+(-0.663284299983)*x_ref[2]
        ref[2,4,2,0]=(0.407867058666)*x_ref[0]**o+(-0.155472572874)*x_ref[0]+(0.0574795179339)*x_ref[1]**o+(0.534562635965)*x_ref[1]+(-0.35303239373)*x_ref[2]**o+(0.690463242713)*x_ref[2]
        ref[2,4,2,1]=(0.458514172477)*x_ref[0]**o+(0.493538792362)*x_ref[0]+(-0.534087073406)*x_ref[1]**o+(0.140190942794)*x_ref[1]+(0.53904059561)*x_ref[2]**o+(0.511924245166)*x_ref[2]
        ref[3,0,0,0]=(0.80688664483)*x_ref[0]**o+(0.641990457435)*x_ref[0]+(0.258439350653)*x_ref[1]**o+(0.360949512497)*x_ref[1]+(0.71055262439)*x_ref[2]**o+(-0.14812833383)*x_ref[2]
        ref[3,0,0,1]=(0.349076196526)*x_ref[0]**o+(-0.250426445203)*x_ref[0]+(0.363350736371)*x_ref[1]**o+(0.62331683755)*x_ref[1]+(-0.799481730613)*x_ref[2]**o+(0.86946545402)*x_ref[2]
        ref[3,0,1,0]=(-0.461900551752)*x_ref[0]**o+(-0.530275233976)*x_ref[0]+(0.778112655963)*x_ref[1]**o+(-0.548160947996)*x_ref[1]+(0.27643541045)*x_ref[2]**o+(0.558519627971)*x_ref[2]
        ref[3,0,1,1]=(-0.870845434837)*x_ref[0]**o+(0.90473098031)*x_ref[0]+(0.356324338565)*x_ref[1]**o+(-0.173579775601)*x_ref[1]+(0.866960109024)*x_ref[2]**o+(0.22810880895)*x_ref[2]
        ref[3,0,2,0]=(0.449562471445)*x_ref[0]**o+(-0.133977676845)*x_ref[0]+(-0.945220890263)*x_ref[1]**o+(0.999013496171)*x_ref[1]+(-0.612837266133)*x_ref[2]**o+(-0.695797874487)*x_ref[2]
        ref[3,0,2,1]=(-0.588105630499)*x_ref[0]**o+(0.176592342414)*x_ref[0]+(-0.820991814796)*x_ref[1]**o+(0.0010811086852)*x_ref[1]+(0.498174763907)*x_ref[2]**o+(-0.0223969132485)*x_ref[2]
        ref[3,1,0,0]=(0.653483773188)*x_ref[0]**o+(0.460215104331)*x_ref[0]+(-0.287173677147)*x_ref[1]**o+(-0.837222104854)*x_ref[1]+(0.457524748826)*x_ref[2]**o+(-0.638268772518)*x_ref[2]
        ref[3,1,0,1]=(-0.0999951386915)*x_ref[0]**o+(0.294288175142)*x_ref[0]+(0.159178572562)*x_ref[1]**o+(0.13437803751)*x_ref[1]+(-0.411939992038)*x_ref[2]**o+(0.283003770703)*x_ref[2]
        ref[3,1,1,0]=(-0.862929516639)*x_ref[0]**o+(-0.455798937374)*x_ref[0]+(0.972383754049)*x_ref[1]**o+(0.251968652813)*x_ref[1]+(-0.631965845999)*x_ref[2]**o+(-0.418936218512)*x_ref[2]
        ref[3,1,1,1]=(-0.0265533722079)*x_ref[0]**o+(-0.720054910456)*x_ref[0]+(-0.134379948062)*x_ref[1]**o+(-0.535223772178)*x_ref[1]+(-0.711365379454)*x_ref[2]**o+(0.39963284442)*x_ref[2]
        ref[3,1,2,0]=(-0.9691936764)*x_ref[0]**o+(-0.220678275504)*x_ref[0]+(0.423830480119)*x_ref[1]**o+(-0.286816976029)*x_ref[1]+(-0.637835943762)*x_ref[2]**o+(0.389581647726)*x_ref[2]
        ref[3,1,2,1]=(-0.375225433223)*x_ref[0]**o+(0.339950113552)*x_ref[0]+(0.272089140767)*x_ref[1]**o+(-0.671811329081)*x_ref[1]+(-0.341834415939)*x_ref[2]**o+(-0.974852723909)*x_ref[2]
        ref[3,2,0,0]=(-0.13212102165)*x_ref[0]**o+(-0.529435682355)*x_ref[0]+(0.57321409695)*x_ref[1]**o+(0.123050260287)*x_ref[1]+(-0.50608599056)*x_ref[2]**o+(-0.258999018643)*x_ref[2]
        ref[3,2,0,1]=(-0.594320198162)*x_ref[0]**o+(0.59617231387)*x_ref[0]+(-0.132055038377)*x_ref[1]**o+(0.0267055057967)*x_ref[1]+(-0.0909023410879)*x_ref[2]**o+(-0.327689716933)*x_ref[2]
        ref[3,2,1,0]=(0.352248348539)*x_ref[0]**o+(-0.937172168074)*x_ref[0]+(0.492884278868)*x_ref[1]**o+(0.599164015797)*x_ref[1]+(-0.306391881602)*x_ref[2]**o+(-0.124914738757)*x_ref[2]
        ref[3,2,1,1]=(-0.124766143355)*x_ref[0]**o+(-0.201629423485)*x_ref[0]+(0.69199028272)*x_ref[1]**o+(0.0909253140313)*x_ref[1]+(-0.0807463990698)*x_ref[2]**o+(0.274206212499)*x_ref[2]
        ref[3,2,2,0]=(-0.939257892238)*x_ref[0]**o+(-0.0905023289297)*x_ref[0]+(-0.101189471915)*x_ref[1]**o+(-0.850452366024)*x_ref[1]+(0.706374760942)*x_ref[2]**o+(-0.0741989595692)*x_ref[2]
        ref[3,2,2,1]=(-0.956776001745)*x_ref[0]**o+(0.313587985523)*x_ref[0]+(0.501857466469)*x_ref[1]**o+(0.866521730616)*x_ref[1]+(-0.113777691526)*x_ref[2]**o+(-0.43724474427)*x_ref[2]
        ref[3,3,0,0]=(-0.794240122389)*x_ref[0]**o+(0.117246192404)*x_ref[0]+(0.730901073854)*x_ref[1]**o+(0.406753741761)*x_ref[1]+(-0.987815418361)*x_ref[2]**o+(0.543031203217)*x_ref[2]
        ref[3,3,0,1]=(-0.539899807208)*x_ref[0]**o+(0.425096558373)*x_ref[0]+(-0.261756005984)*x_ref[1]**o+(0.219196945781)*x_ref[1]+(-0.458238548056)*x_ref[2]**o+(-0.396891207522)*x_ref[2]
        ref[3,3,1,0]=(-0.512322191935)*x_ref[0]**o+(0.158356684082)*x_ref[0]+(-0.702591537714)*x_ref[1]**o+(0.202771352983)*x_ref[1]+(-0.346772902785)*x_ref[2]**o+(0.260366433089)*x_ref[2]
        ref[3,3,1,1]=(0.839858482584)*x_ref[0]**o+(0.132933571004)*x_ref[0]+(0.840682050408)*x_ref[1]**o+(0.490904029834)*x_ref[1]+(-0.298290729461)*x_ref[2]**o+(-0.467229890802)*x_ref[2]
        ref[3,3,2,0]=(-0.691489850326)*x_ref[0]**o+(0.852014173253)*x_ref[0]+(0.484568640775)*x_ref[1]**o+(0.0788611533234)*x_ref[1]+(0.335927774531)*x_ref[2]**o+(-0.507648906088)*x_ref[2]
        ref[3,3,2,1]=(0.140576758316)*x_ref[0]**o+(-0.783440190349)*x_ref[0]+(-0.408503163319)*x_ref[1]**o+(0.0871935010894)*x_ref[1]+(0.478151373206)*x_ref[2]**o+(0.837020241411)*x_ref[2]
        ref[3,4,0,0]=(-0.956822615698)*x_ref[0]**o+(0.594397877716)*x_ref[0]+(0.844357571956)*x_ref[1]**o+(0.173315874424)*x_ref[1]+(0.96181728365)*x_ref[2]**o+(-0.900445532048)*x_ref[2]
        ref[3,4,0,1]=(0.887824966747)*x_ref[0]**o+(-0.579258071661)*x_ref[0]+(0.983311456722)*x_ref[1]**o+(0.996052998934)*x_ref[1]+(0.215867343112)*x_ref[2]**o+(0.614855622355)*x_ref[2]
        ref[3,4,1,0]=(0.482666172028)*x_ref[0]**o+(0.0258626246719)*x_ref[0]+(-0.394140399663)*x_ref[1]**o+(0.192974809281)*x_ref[1]+(-0.265879383852)*x_ref[2]**o+(-0.0409441022179)*x_ref[2]
        ref[3,4,1,1]=(0.0600752930856)*x_ref[0]**o+(-0.885644457634)*x_ref[0]+(-0.54869761625)*x_ref[1]**o+(0.4745411417)*x_ref[1]+(-0.554764519536)*x_ref[2]**o+(-0.159514159559)*x_ref[2]
        ref[3,4,2,0]=(-0.804675220841)*x_ref[0]**o+(0.586839632342)*x_ref[0]+(-0.329997060623)*x_ref[1]**o+(0.242316008423)*x_ref[1]+(0.272064229179)*x_ref[2]**o+(0.24348627845)*x_ref[2]
        ref[3,4,2,1]=(0.847664079563)*x_ref[0]**o+(0.927723091901)*x_ref[0]+(-0.861048553467)*x_ref[1]**o+(-0.413216499555)*x_ref[1]+(0.546938488463)*x_ref[2]**o+(0.0558689143145)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_ReducedSolution_rank0(self):
      """
      tests interpolate for rank 0 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedSolution onto ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(),dim=dim)
      s=Data(0,(),w)
      ref=Data(0,(),w_ref)
      if dim==2:
        s=(-1.64974193328)*x[0]+(1.68675911612)*x[1]
        ref=(-1.64974193328)*x_ref[0]+(1.68675911612)*x_ref[1]
      else:
        s=(-0.517718083008)*x[0]+(0.479679773182)*x[1]+(0.72459612383)*x[2]
        ref=(-0.517718083008)*x_ref[0]+(0.479679773182)*x_ref[1]+(0.72459612383)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_ReducedSolution_rank1(self):
      """
      tests interpolate for rank 1 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedSolution onto ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(2,),dim=dim)
      s=Data(0,(2,),w)
      ref=Data(0,(2,),w_ref)
      if dim==2:
        s[0]=(-0.415290633997)*x[0]+(0.118775036022)*x[1]
        s[1]=(-0.833749289762)*x[0]+(0.121297221921)*x[1]
        ref[0]=(-0.415290633997)*x[0]+(0.118775036022)*x[1]
        ref[1]=(-0.833749289762)*x[0]+(0.121297221921)*x[1]
      else:
        s[0]=(-0.336174941008)*x[0]+(-0.844918885743)*x[1]+(-0.410649979186)*x[2]
        s[1]=(0.112527237498)*x[0]+(-0.867632388528)*x[1]+(1.42359104479)*x[2]
        ref[0]=(-0.336174941008)*x[0]+(-0.844918885743)*x[1]+(-0.410649979186)*x[2]
        ref[1]=(0.112527237498)*x[0]+(-0.867632388528)*x[1]+(1.42359104479)*x[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_ReducedSolution_rank2(self):
      """
      tests interpolate for rank 2 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedSolution onto ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5),dim=dim)
      s=Data(0,(4, 5),w)
      ref=Data(0,(4, 5),w_ref)
      if dim==2:
        s[0,0]=(-0.833920166667)*x[0]+(-0.105734397941)*x[1]
        s[0,1]=(-0.0535194301789)*x[0]+(1.2869199411)*x[1]
        s[0,2]=(-1.81783608134)*x[0]+(-0.0858549172679)*x[1]
        s[0,3]=(0.133570573696)*x[0]+(0.398281712973)*x[1]
        s[0,4]=(-1.2514060292)*x[0]+(-0.24743946144)*x[1]
        s[1,0]=(0.917760776668)*x[0]+(0.179758313228)*x[1]
        s[1,1]=(0.677772149724)*x[0]+(0.762705763999)*x[1]
        s[1,2]=(-0.505172381789)*x[0]+(0.412365083581)*x[1]
        s[1,3]=(0.0481873177823)*x[0]+(-0.449871653505)*x[1]
        s[1,4]=(-1.43052283016)*x[0]+(0.553426132082)*x[1]
        s[2,0]=(0.337172157552)*x[0]+(1.56154138911)*x[1]
        s[2,1]=(-0.338189138128)*x[0]+(-0.62614063612)*x[1]
        s[2,2]=(-1.51606676783)*x[0]+(-0.13283454533)*x[1]
        s[2,3]=(0.727068106091)*x[0]+(1.00949960235)*x[1]
        s[2,4]=(0.297941246402)*x[0]+(0.952119168452)*x[1]
        s[3,0]=(0.724157315025)*x[0]+(0.962405533793)*x[1]
        s[3,1]=(-0.239168209123)*x[0]+(1.28280794616)*x[1]
        s[3,2]=(-0.0336024791682)*x[0]+(-0.775860593407)*x[1]
        s[3,3]=(-0.118809020482)*x[0]+(0.51890714417)*x[1]
        s[3,4]=(0.374315302545)*x[0]+(-0.258942994)*x[1]
        ref[0,0]=(-0.833920166667)*x_ref[0]+(-0.105734397941)*x_ref[1]
        ref[0,1]=(-0.0535194301789)*x_ref[0]+(1.2869199411)*x_ref[1]
        ref[0,2]=(-1.81783608134)*x_ref[0]+(-0.0858549172679)*x_ref[1]
        ref[0,3]=(0.133570573696)*x_ref[0]+(0.398281712973)*x_ref[1]
        ref[0,4]=(-1.2514060292)*x_ref[0]+(-0.24743946144)*x_ref[1]
        ref[1,0]=(0.917760776668)*x_ref[0]+(0.179758313228)*x_ref[1]
        ref[1,1]=(0.677772149724)*x_ref[0]+(0.762705763999)*x_ref[1]
        ref[1,2]=(-0.505172381789)*x_ref[0]+(0.412365083581)*x_ref[1]
        ref[1,3]=(0.0481873177823)*x_ref[0]+(-0.449871653505)*x_ref[1]
        ref[1,4]=(-1.43052283016)*x_ref[0]+(0.553426132082)*x_ref[1]
        ref[2,0]=(0.337172157552)*x_ref[0]+(1.56154138911)*x_ref[1]
        ref[2,1]=(-0.338189138128)*x_ref[0]+(-0.62614063612)*x_ref[1]
        ref[2,2]=(-1.51606676783)*x_ref[0]+(-0.13283454533)*x_ref[1]
        ref[2,3]=(0.727068106091)*x_ref[0]+(1.00949960235)*x_ref[1]
        ref[2,4]=(0.297941246402)*x_ref[0]+(0.952119168452)*x_ref[1]
        ref[3,0]=(0.724157315025)*x_ref[0]+(0.962405533793)*x_ref[1]
        ref[3,1]=(-0.239168209123)*x_ref[0]+(1.28280794616)*x_ref[1]
        ref[3,2]=(-0.0336024791682)*x_ref[0]+(-0.775860593407)*x_ref[1]
        ref[3,3]=(-0.118809020482)*x_ref[0]+(0.51890714417)*x_ref[1]
        ref[3,4]=(0.374315302545)*x_ref[0]+(-0.258942994)*x_ref[1]
      else:
        s[0,0]=(1.18033303246)*x[0]+(0.59372740835)*x[1]+(-0.684959540977)*x[2]
        s[0,1]=(0.65301033503)*x[0]+(-1.17147722359)*x[1]+(0.455404166733)*x[2]
        s[0,2]=(-0.412048337979)*x[0]+(-1.24933561194)*x[1]+(-0.636287202752)*x[2]
        s[0,3]=(0.122156598551)*x[0]+(0.103952485283)*x[1]+(-0.564471099255)*x[2]
        s[0,4]=(0.737873603859)*x[0]+(0.139615778712)*x[1]+(0.523544473477)*x[2]
        s[1,0]=(0.0198124265514)*x[0]+(-0.335073882197)*x[1]+(-0.805965691618)*x[2]
        s[1,1]=(-1.28485879121)*x[0]+(-0.618847596536)*x[1]+(1.32879899911)*x[2]
        s[1,2]=(1.09319515176)*x[0]+(0.543230471078)*x[1]+(1.56430808151)*x[2]
        s[1,3]=(0.541425658238)*x[0]+(0.0140217060596)*x[1]+(0.603558416867)*x[2]
        s[1,4]=(0.303110106189)*x[0]+(-0.103009794147)*x[1]+(-1.68324105571)*x[2]
        s[2,0]=(-1.65609235169)*x[0]+(-0.290040263496)*x[1]+(0.477073850071)*x[2]
        s[2,1]=(-0.897378807931)*x[0]+(0.191583115578)*x[1]+(0.213226828436)*x[2]
        s[2,2]=(0.523724313062)*x[0]+(0.214413673529)*x[1]+(0.533345615094)*x[2]
        s[2,3]=(-0.994043729754)*x[0]+(-0.307957055273)*x[1]+(0.0422995421427)*x[2]
        s[2,4]=(1.22590973976)*x[0]+(-1.65014223339)*x[1]+(-1.59726996804)*x[2]
        s[3,0]=(-0.0870943824713)*x[0]+(-0.45766718732)*x[1]+(-0.878874753774)*x[2]
        s[3,1]=(0.244470697307)*x[0]+(-0.0429568231364)*x[1]+(-1.57098356615)*x[2]
        s[3,2]=(1.35091699523)*x[0]+(-0.127297590922)*x[1]+(-0.556702799475)*x[2]
        s[3,3]=(0.189117166393)*x[0]+(0.317562328009)*x[1]+(-0.59788285723)*x[2]
        s[3,4]=(-0.155355776878)*x[0]+(-0.616350184089)*x[1]+(0.296083681054)*x[2]
        ref[0,0]=(1.18033303246)*x_ref[0]+(0.59372740835)*x_ref[1]+(-0.684959540977)*x_ref[2]
        ref[0,1]=(0.65301033503)*x_ref[0]+(-1.17147722359)*x_ref[1]+(0.455404166733)*x_ref[2]
        ref[0,2]=(-0.412048337979)*x_ref[0]+(-1.24933561194)*x_ref[1]+(-0.636287202752)*x_ref[2]
        ref[0,3]=(0.122156598551)*x_ref[0]+(0.103952485283)*x_ref[1]+(-0.564471099255)*x_ref[2]
        ref[0,4]=(0.737873603859)*x_ref[0]+(0.139615778712)*x_ref[1]+(0.523544473477)*x_ref[2]
        ref[1,0]=(0.0198124265514)*x_ref[0]+(-0.335073882197)*x_ref[1]+(-0.805965691618)*x_ref[2]
        ref[1,1]=(-1.28485879121)*x_ref[0]+(-0.618847596536)*x_ref[1]+(1.32879899911)*x_ref[2]
        ref[1,2]=(1.09319515176)*x_ref[0]+(0.543230471078)*x_ref[1]+(1.56430808151)*x_ref[2]
        ref[1,3]=(0.541425658238)*x_ref[0]+(0.0140217060596)*x_ref[1]+(0.603558416867)*x_ref[2]
        ref[1,4]=(0.303110106189)*x_ref[0]+(-0.103009794147)*x_ref[1]+(-1.68324105571)*x_ref[2]
        ref[2,0]=(-1.65609235169)*x_ref[0]+(-0.290040263496)*x_ref[1]+(0.477073850071)*x_ref[2]
        ref[2,1]=(-0.897378807931)*x_ref[0]+(0.191583115578)*x_ref[1]+(0.213226828436)*x_ref[2]
        ref[2,2]=(0.523724313062)*x_ref[0]+(0.214413673529)*x_ref[1]+(0.533345615094)*x_ref[2]
        ref[2,3]=(-0.994043729754)*x_ref[0]+(-0.307957055273)*x_ref[1]+(0.0422995421427)*x_ref[2]
        ref[2,4]=(1.22590973976)*x_ref[0]+(-1.65014223339)*x_ref[1]+(-1.59726996804)*x_ref[2]
        ref[3,0]=(-0.0870943824713)*x_ref[0]+(-0.45766718732)*x_ref[1]+(-0.878874753774)*x_ref[2]
        ref[3,1]=(0.244470697307)*x_ref[0]+(-0.0429568231364)*x_ref[1]+(-1.57098356615)*x_ref[2]
        ref[3,2]=(1.35091699523)*x_ref[0]+(-0.127297590922)*x_ref[1]+(-0.556702799475)*x_ref[2]
        ref[3,3]=(0.189117166393)*x_ref[0]+(0.317562328009)*x_ref[1]+(-0.59788285723)*x_ref[2]
        ref[3,4]=(-0.155355776878)*x_ref[0]+(-0.616350184089)*x_ref[1]+(0.296083681054)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_ReducedSolution_rank3(self):
      """
      tests interpolate for rank 3 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedSolution onto ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(6, 2, 2),dim=dim)
      s=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2),w_ref)
      if dim==2:
        s[0,0,0]=(0.0730549738553)*x[0]+(0.585649475666)*x[1]
        s[0,0,1]=(-0.449075310223)*x[0]+(1.39116964942)*x[1]
        s[0,1,0]=(0.0210032238157)*x[0]+(1.33904896697)*x[1]
        s[0,1,1]=(0.556125399236)*x[0]+(-0.00311484949343)*x[1]
        s[1,0,0]=(0.938296971476)*x[0]+(1.13223213262)*x[1]
        s[1,0,1]=(0.744133292867)*x[0]+(0.0271311625871)*x[1]
        s[1,1,0]=(-0.0478922372672)*x[0]+(-1.08882574207)*x[1]
        s[1,1,1]=(0.738328994373)*x[0]+(-0.730449948444)*x[1]
        s[2,0,0]=(0.66593099915)*x[0]+(-0.185918605751)*x[1]
        s[2,0,1]=(0.450972410432)*x[0]+(1.3393675839)*x[1]
        s[2,1,0]=(0.897370528228)*x[0]+(0.765127571621)*x[1]
        s[2,1,1]=(-0.349658725406)*x[0]+(-0.536332219959)*x[1]
        s[3,0,0]=(1.11112898963)*x[0]+(1.50253301112)*x[1]
        s[3,0,1]=(0.238957188525)*x[0]+(0.775368032877)*x[1]
        s[3,1,0]=(0.612991596596)*x[0]+(0.0718037077356)*x[1]
        s[3,1,1]=(-0.40039392925)*x[0]+(-1.58712410632)*x[1]
        s[4,0,0]=(0.23241254342)*x[0]+(-0.547054930927)*x[1]
        s[4,0,1]=(-0.976995206698)*x[0]+(1.251791534)*x[1]
        s[4,1,0]=(-1.13243662763)*x[0]+(0.56739947752)*x[1]
        s[4,1,1]=(-0.543613180121)*x[0]+(0.649805921416)*x[1]
        s[5,0,0]=(-0.0690689383083)*x[0]+(0.818563064557)*x[1]
        s[5,0,1]=(0.362899720121)*x[0]+(-0.112604112729)*x[1]
        s[5,1,0]=(-0.291756903179)*x[0]+(-0.538221294595)*x[1]
        s[5,1,1]=(0.60406331286)*x[0]+(0.228744896658)*x[1]
        ref[0,0,0]=(0.0730549738553)*x_ref[0]+(0.585649475666)*x_ref[1]
        ref[0,0,1]=(-0.449075310223)*x_ref[0]+(1.39116964942)*x_ref[1]
        ref[0,1,0]=(0.0210032238157)*x_ref[0]+(1.33904896697)*x_ref[1]
        ref[0,1,1]=(0.556125399236)*x_ref[0]+(-0.00311484949343)*x_ref[1]
        ref[1,0,0]=(0.938296971476)*x_ref[0]+(1.13223213262)*x_ref[1]
        ref[1,0,1]=(0.744133292867)*x_ref[0]+(0.0271311625871)*x_ref[1]
        ref[1,1,0]=(-0.0478922372672)*x_ref[0]+(-1.08882574207)*x_ref[1]
        ref[1,1,1]=(0.738328994373)*x_ref[0]+(-0.730449948444)*x_ref[1]
        ref[2,0,0]=(0.66593099915)*x_ref[0]+(-0.185918605751)*x_ref[1]
        ref[2,0,1]=(0.450972410432)*x_ref[0]+(1.3393675839)*x_ref[1]
        ref[2,1,0]=(0.897370528228)*x_ref[0]+(0.765127571621)*x_ref[1]
        ref[2,1,1]=(-0.349658725406)*x_ref[0]+(-0.536332219959)*x_ref[1]
        ref[3,0,0]=(1.11112898963)*x_ref[0]+(1.50253301112)*x_ref[1]
        ref[3,0,1]=(0.238957188525)*x_ref[0]+(0.775368032877)*x_ref[1]
        ref[3,1,0]=(0.612991596596)*x_ref[0]+(0.0718037077356)*x_ref[1]
        ref[3,1,1]=(-0.40039392925)*x_ref[0]+(-1.58712410632)*x_ref[1]
        ref[4,0,0]=(0.23241254342)*x_ref[0]+(-0.547054930927)*x_ref[1]
        ref[4,0,1]=(-0.976995206698)*x_ref[0]+(1.251791534)*x_ref[1]
        ref[4,1,0]=(-1.13243662763)*x_ref[0]+(0.56739947752)*x_ref[1]
        ref[4,1,1]=(-0.543613180121)*x_ref[0]+(0.649805921416)*x_ref[1]
        ref[5,0,0]=(-0.0690689383083)*x_ref[0]+(0.818563064557)*x_ref[1]
        ref[5,0,1]=(0.362899720121)*x_ref[0]+(-0.112604112729)*x_ref[1]
        ref[5,1,0]=(-0.291756903179)*x_ref[0]+(-0.538221294595)*x_ref[1]
        ref[5,1,1]=(0.60406331286)*x_ref[0]+(0.228744896658)*x_ref[1]
      else:
        s[0,0,0]=(0.811538724853)*x[0]+(-0.347250364239)*x[1]+(-0.116011094391)*x[2]
        s[0,0,1]=(1.69112205586)*x[0]+(-1.04197300041)*x[1]+(0.849118123462)*x[2]
        s[0,1,0]=(1.30816943778)*x[0]+(1.01636941747)*x[1]+(-0.849746116393)*x[2]
        s[0,1,1]=(0.888683598925)*x[0]+(-1.60930356378)*x[1]+(0.522151570265)*x[2]
        s[1,0,0]=(1.03821316703)*x[0]+(-1.19647941504)*x[1]+(-1.08543246224)*x[2]
        s[1,0,1]=(-0.314182278588)*x[0]+(0.144825414517)*x[1]+(0.162613879231)*x[2]
        s[1,1,0]=(0.0416209427382)*x[0]+(0.0390092175082)*x[1]+(1.55616366687)*x[2]
        s[1,1,1]=(-0.577251716148)*x[0]+(0.717529140134)*x[1]+(-0.567199280535)*x[2]
        s[2,0,0]=(1.26716303575)*x[0]+(0.119576983216)*x[1]+(-0.602181033831)*x[2]
        s[2,0,1]=(0.533935330879)*x[0]+(0.292003869006)*x[1]+(-0.0917835680855)*x[2]
        s[2,1,0]=(-1.50218600857)*x[0]+(1.23418550746)*x[1]+(-0.600240530145)*x[2]
        s[2,1,1]=(0.689151741161)*x[0]+(0.909077564156)*x[1]+(0.924638961344)*x[2]
        s[3,0,0]=(-0.763411711557)*x[0]+(-0.862413521307)*x[1]+(1.49095932554)*x[2]
        s[3,0,1]=(-0.215589144977)*x[0]+(0.305301450314)*x[1]+(-0.468853406603)*x[2]
        s[3,1,0]=(-0.433045929433)*x[0]+(-0.907669929171)*x[1]+(0.917462300039)*x[2]
        s[3,1,1]=(0.229593212783)*x[0]+(-1.79371444035)*x[1]+(-0.848409349534)*x[2]
        s[4,0,0]=(1.40249316312)*x[0]+(-0.766609553683)*x[1]+(0.337911083882)*x[2]
        s[4,0,1]=(0.728650674867)*x[0]+(0.631222570369)*x[1]+(1.2491315131)*x[2]
        s[4,1,0]=(1.03998601102)*x[0]+(-1.81204917532)*x[1]+(0.781499164268)*x[2]
        s[4,1,1]=(-0.151999545303)*x[0]+(-0.767585861167)*x[1]+(1.17261654125)*x[2]
        s[5,0,0]=(0.0632249575346)*x[0]+(0.333565970302)*x[1]+(-1.66679001354)*x[2]
        s[5,0,1]=(0.945677231929)*x[0]+(-1.15936834775)*x[1]+(-0.644891644457)*x[2]
        s[5,1,0]=(1.08939087839)*x[0]+(0.687932839014)*x[1]+(-0.473426758917)*x[2]
        s[5,1,1]=(0.0349915672648)*x[0]+(-1.48593425129)*x[1]+(0.145421756236)*x[2]
        ref[0,0,0]=(0.811538724853)*x_ref[0]+(-0.347250364239)*x_ref[1]+(-0.116011094391)*x_ref[2]
        ref[0,0,1]=(1.69112205586)*x_ref[0]+(-1.04197300041)*x_ref[1]+(0.849118123462)*x_ref[2]
        ref[0,1,0]=(1.30816943778)*x_ref[0]+(1.01636941747)*x_ref[1]+(-0.849746116393)*x_ref[2]
        ref[0,1,1]=(0.888683598925)*x_ref[0]+(-1.60930356378)*x_ref[1]+(0.522151570265)*x_ref[2]
        ref[1,0,0]=(1.03821316703)*x_ref[0]+(-1.19647941504)*x_ref[1]+(-1.08543246224)*x_ref[2]
        ref[1,0,1]=(-0.314182278588)*x_ref[0]+(0.144825414517)*x_ref[1]+(0.162613879231)*x_ref[2]
        ref[1,1,0]=(0.0416209427382)*x_ref[0]+(0.0390092175082)*x_ref[1]+(1.55616366687)*x_ref[2]
        ref[1,1,1]=(-0.577251716148)*x_ref[0]+(0.717529140134)*x_ref[1]+(-0.567199280535)*x_ref[2]
        ref[2,0,0]=(1.26716303575)*x_ref[0]+(0.119576983216)*x_ref[1]+(-0.602181033831)*x_ref[2]
        ref[2,0,1]=(0.533935330879)*x_ref[0]+(0.292003869006)*x_ref[1]+(-0.0917835680855)*x_ref[2]
        ref[2,1,0]=(-1.50218600857)*x_ref[0]+(1.23418550746)*x_ref[1]+(-0.600240530145)*x_ref[2]
        ref[2,1,1]=(0.689151741161)*x_ref[0]+(0.909077564156)*x_ref[1]+(0.924638961344)*x_ref[2]
        ref[3,0,0]=(-0.763411711557)*x_ref[0]+(-0.862413521307)*x_ref[1]+(1.49095932554)*x_ref[2]
        ref[3,0,1]=(-0.215589144977)*x_ref[0]+(0.305301450314)*x_ref[1]+(-0.468853406603)*x_ref[2]
        ref[3,1,0]=(-0.433045929433)*x_ref[0]+(-0.907669929171)*x_ref[1]+(0.917462300039)*x_ref[2]
        ref[3,1,1]=(0.229593212783)*x_ref[0]+(-1.79371444035)*x_ref[1]+(-0.848409349534)*x_ref[2]
        ref[4,0,0]=(1.40249316312)*x_ref[0]+(-0.766609553683)*x_ref[1]+(0.337911083882)*x_ref[2]
        ref[4,0,1]=(0.728650674867)*x_ref[0]+(0.631222570369)*x_ref[1]+(1.2491315131)*x_ref[2]
        ref[4,1,0]=(1.03998601102)*x_ref[0]+(-1.81204917532)*x_ref[1]+(0.781499164268)*x_ref[2]
        ref[4,1,1]=(-0.151999545303)*x_ref[0]+(-0.767585861167)*x_ref[1]+(1.17261654125)*x_ref[2]
        ref[5,0,0]=(0.0632249575346)*x_ref[0]+(0.333565970302)*x_ref[1]+(-1.66679001354)*x_ref[2]
        ref[5,0,1]=(0.945677231929)*x_ref[0]+(-1.15936834775)*x_ref[1]+(-0.644891644457)*x_ref[2]
        ref[5,1,0]=(1.08939087839)*x_ref[0]+(0.687932839014)*x_ref[1]+(-0.473426758917)*x_ref[2]
        ref[5,1,1]=(0.0349915672648)*x_ref[0]+(-1.48593425129)*x_ref[1]+(0.145421756236)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_ReducedSolution_rank4(self):
      """
      tests interpolate for rank 4 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedSolution onto ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5, 3, 2),dim=dim)
      s=Data(0,(4, 5, 3, 2),w)
      ref=Data(0,(4, 5, 3, 2),w_ref)
      if dim==2:
        s[0,0,0,0]=(0.609727033659)*x[0]+(-0.901858940633)*x[1]
        s[0,0,0,1]=(-0.221259461243)*x[0]+(0.351136340908)*x[1]
        s[0,0,1,0]=(0.497173734282)*x[0]+(0.899142906899)*x[1]
        s[0,0,1,1]=(-1.55206649551)*x[0]+(-0.680257384044)*x[1]
        s[0,0,2,0]=(-0.932033638033)*x[0]+(-0.46503758126)*x[1]
        s[0,0,2,1]=(-0.0372650123525)*x[0]+(0.0173714029098)*x[1]
        s[0,1,0,0]=(0.855963098465)*x[0]+(0.760176320898)*x[1]
        s[0,1,0,1]=(1.37546120327)*x[0]+(1.72916158331)*x[1]
        s[0,1,1,0]=(0.25469861613)*x[0]+(0.71445716033)*x[1]
        s[0,1,1,1]=(0.198455659765)*x[0]+(0.0410276829495)*x[1]
        s[0,1,2,0]=(-0.628080427667)*x[0]+(-0.0197206025984)*x[1]
        s[0,1,2,1]=(1.26759029895)*x[0]+(1.15914953253)*x[1]
        s[0,2,0,0]=(-1.43203053441)*x[0]+(0.366301170069)*x[1]
        s[0,2,0,1]=(0.420527802229)*x[0]+(-0.709209747415)*x[1]
        s[0,2,1,0]=(0.399042264829)*x[0]+(0.0453132754731)*x[1]
        s[0,2,1,1]=(1.2074346062)*x[0]+(-1.14163410992)*x[1]
        s[0,2,2,0]=(-0.00270984158569)*x[0]+(-1.63216200182)*x[1]
        s[0,2,2,1]=(0.174276088847)*x[0]+(-1.04789472231)*x[1]
        s[0,3,0,0]=(-0.47704729036)*x[0]+(-1.2550637204)*x[1]
        s[0,3,0,1]=(0.348356873191)*x[0]+(-0.796668578265)*x[1]
        s[0,3,1,0]=(-0.52724725097)*x[0]+(1.26163123592)*x[1]
        s[0,3,1,1]=(-1.52681799261)*x[0]+(0.936041931735)*x[1]
        s[0,3,2,0]=(1.33689673219)*x[0]+(-1.34901214018)*x[1]
        s[0,3,2,1]=(1.57651286047)*x[0]+(0.245413762864)*x[1]
        s[0,4,0,0]=(0.194500181498)*x[0]+(-0.0231951844441)*x[1]
        s[0,4,0,1]=(-1.49372076697)*x[0]+(-0.111369678181)*x[1]
        s[0,4,1,0]=(-0.74121646836)*x[0]+(-0.817740156307)*x[1]
        s[0,4,1,1]=(-0.233643397123)*x[0]+(0.672715068796)*x[1]
        s[0,4,2,0]=(1.05850947579)*x[0]+(0.386484386191)*x[1]
        s[0,4,2,1]=(1.45105922963)*x[0]+(-0.367458625996)*x[1]
        s[1,0,0,0]=(-0.391803788669)*x[0]+(0.797592300662)*x[1]
        s[1,0,0,1]=(-0.205299072883)*x[0]+(0.677427935558)*x[1]
        s[1,0,1,0]=(1.56225329031)*x[0]+(1.17468092515)*x[1]
        s[1,0,1,1]=(1.4586005658)*x[0]+(1.36481062943)*x[1]
        s[1,0,2,0]=(-1.53281881413)*x[0]+(0.207028833974)*x[1]
        s[1,0,2,1]=(0.0469239691648)*x[0]+(1.03547812705)*x[1]
        s[1,1,0,0]=(1.39614335897)*x[0]+(0.82046489695)*x[1]
        s[1,1,0,1]=(-0.775462741637)*x[0]+(-0.275081761646)*x[1]
        s[1,1,1,0]=(0.173353028608)*x[0]+(-1.05620431816)*x[1]
        s[1,1,1,1]=(-0.464192870247)*x[0]+(0.212204784206)*x[1]
        s[1,1,2,0]=(-0.383813531866)*x[0]+(-0.0773759374234)*x[1]
        s[1,1,2,1]=(-1.07811385759)*x[0]+(-0.457232330819)*x[1]
        s[1,2,0,0]=(-0.295869147576)*x[0]+(0.166843915631)*x[1]
        s[1,2,0,1]=(0.571421034325)*x[0]+(1.46722583245)*x[1]
        s[1,2,1,0]=(-0.34494944257)*x[0]+(-0.286867052491)*x[1]
        s[1,2,1,1]=(1.43128885761)*x[0]+(0.21277887035)*x[1]
        s[1,2,2,0]=(-1.19397187873)*x[0]+(0.126397665227)*x[1]
        s[1,2,2,1]=(0.171368751954)*x[0]+(-1.12100254625)*x[1]
        s[1,3,0,0]=(0.0975333077604)*x[0]+(0.583524660188)*x[1]
        s[1,3,0,1]=(0.0271450955458)*x[0]+(0.368978332867)*x[1]
        s[1,3,1,0]=(1.22806968486)*x[0]+(-0.569932748468)*x[1]
        s[1,3,1,1]=(-0.691989839521)*x[0]+(-0.829822182922)*x[1]
        s[1,3,2,0]=(0.743776534098)*x[0]+(1.3668937127)*x[1]
        s[1,3,2,1]=(0.192206575204)*x[0]+(0.96576702824)*x[1]
        s[1,4,0,0]=(0.149381460079)*x[0]+(-0.987875538036)*x[1]
        s[1,4,0,1]=(-0.706855836683)*x[0]+(0.0410884211631)*x[1]
        s[1,4,1,0]=(-0.222650274)*x[0]+(0.59589242048)*x[1]
        s[1,4,1,1]=(0.541489186743)*x[0]+(-1.22543996074)*x[1]
        s[1,4,2,0]=(0.630234824652)*x[0]+(0.508436575179)*x[1]
        s[1,4,2,1]=(0.990062359294)*x[0]+(-0.639884693407)*x[1]
        s[2,0,0,0]=(0.378887461733)*x[0]+(1.71999809344)*x[1]
        s[2,0,0,1]=(0.182412683544)*x[0]+(-0.401183243877)*x[1]
        s[2,0,1,0]=(0.325775290396)*x[0]+(-0.656088786126)*x[1]
        s[2,0,1,1]=(0.316355229742)*x[0]+(1.54087545195)*x[1]
        s[2,0,2,0]=(1.14203881183)*x[0]+(0.703207191364)*x[1]
        s[2,0,2,1]=(-1.05211790226)*x[0]+(1.68596483036)*x[1]
        s[2,1,0,0]=(0.704109387934)*x[0]+(-0.589003921574)*x[1]
        s[2,1,0,1]=(-0.382489922028)*x[0]+(0.387647642942)*x[1]
        s[2,1,1,0]=(-0.0343434958891)*x[0]+(-0.119176404279)*x[1]
        s[2,1,1,1]=(0.567853527929)*x[0]+(1.32523261969)*x[1]
        s[2,1,2,0]=(1.03496000653)*x[0]+(0.110203591459)*x[1]
        s[2,1,2,1]=(0.560149997806)*x[0]+(-1.62893742873)*x[1]
        s[2,2,0,0]=(1.1801792312)*x[0]+(1.1912385935)*x[1]
        s[2,2,0,1]=(0.936932817634)*x[0]+(-1.49082021974)*x[1]
        s[2,2,1,0]=(-0.305582892069)*x[0]+(-0.267235698563)*x[1]
        s[2,2,1,1]=(0.107449730184)*x[0]+(-0.325267993615)*x[1]
        s[2,2,2,0]=(-1.08118986347)*x[0]+(0.144190727334)*x[1]
        s[2,2,2,1]=(1.44755060146)*x[0]+(-0.127612338817)*x[1]
        s[2,3,0,0]=(-0.666943112999)*x[0]+(-0.780402371989)*x[1]
        s[2,3,0,1]=(0.724206717771)*x[0]+(-0.523593908278)*x[1]
        s[2,3,1,0]=(1.86101197199)*x[0]+(-0.710894911302)*x[1]
        s[2,3,1,1]=(-1.74409232023)*x[0]+(0.407821816271)*x[1]
        s[2,3,2,0]=(0.646304172575)*x[0]+(0.0132628778543)*x[1]
        s[2,3,2,1]=(-0.746041438673)*x[0]+(0.516304872741)*x[1]
        s[2,4,0,0]=(0.0326900201283)*x[0]+(-0.533021156401)*x[1]
        s[2,4,0,1]=(0.088598956981)*x[0]+(0.764307970269)*x[1]
        s[2,4,1,0]=(-0.749553477756)*x[0]+(0.379646792939)*x[1]
        s[2,4,1,1]=(-0.243635321378)*x[0]+(-0.470022600261)*x[1]
        s[2,4,2,0]=(0.586398757247)*x[0]+(-0.116982897683)*x[1]
        s[2,4,2,1]=(-0.989012121879)*x[0]+(-1.25006535257)*x[1]
        s[3,0,0,0]=(1.25228306081)*x[0]+(-0.0932717035896)*x[1]
        s[3,0,0,1]=(1.5975516771)*x[0]+(-0.177733286111)*x[1]
        s[3,0,1,0]=(-0.659910070911)*x[0]+(-0.0999498944812)*x[1]
        s[3,0,1,1]=(-0.299870273662)*x[0]+(-1.22977656744)*x[1]
        s[3,0,2,0]=(-0.158211581143)*x[0]+(-0.141421380503)*x[1]
        s[3,0,2,1]=(-1.48226452788)*x[0]+(-0.731264943309)*x[1]
        s[3,1,0,0]=(-1.64589827853)*x[0]+(1.19692573998)*x[1]
        s[3,1,0,1]=(0.361329655357)*x[0]+(0.234310482171)*x[1]
        s[3,1,1,0]=(-1.14077649516)*x[0]+(0.132657084065)*x[1]
        s[3,1,1,1]=(1.41067975663)*x[0]+(0.818243117884)*x[1]
        s[3,1,2,0]=(-1.53137884342)*x[0]+(-0.0127805333093)*x[1]
        s[3,1,2,1]=(0.240455755486)*x[0]+(-1.02361753893)*x[1]
        s[3,2,0,0]=(-0.334803473072)*x[0]+(-0.807452510423)*x[1]
        s[3,2,0,1]=(0.617324536974)*x[0]+(-0.733163471774)*x[1]
        s[3,2,1,0]=(-0.634580050897)*x[0]+(0.362106278168)*x[1]
        s[3,2,1,1]=(-0.21525531818)*x[0]+(0.982902540898)*x[1]
        s[3,2,2,0]=(-0.590682803195)*x[0]+(-1.27537880632)*x[1]
        s[3,2,2,1]=(0.142578313438)*x[0]+(0.212341687291)*x[1]
        s[3,3,0,0]=(-0.758345317216)*x[0]+(-0.971914631359)*x[1]
        s[3,3,0,1]=(0.569463909234)*x[0]+(0.498848917338)*x[1]
        s[3,3,1,0]=(-0.771456575632)*x[0]+(-0.130456072944)*x[1]
        s[3,3,1,1]=(0.811799269245)*x[0]+(-0.829647203396)*x[1]
        s[3,3,2,0]=(-0.814979427693)*x[0]+(-0.0416092796813)*x[1]
        s[3,3,2,1]=(0.933434189127)*x[0]+(0.60794704936)*x[1]
        s[3,4,0,0]=(0.160280566478)*x[0]+(-0.0141616040181)*x[1]
        s[3,4,0,1]=(-1.34835766268)*x[0]+(0.178951225861)*x[1]
        s[3,4,1,0]=(1.67632814658)*x[0]+(1.31316096483)*x[1]
        s[3,4,1,1]=(-0.603403275376)*x[0]+(0.0607753526431)*x[1]
        s[3,4,2,0]=(-0.178440295473)*x[0]+(0.318102694371)*x[1]
        s[3,4,2,1]=(0.781285160749)*x[0]+(0.625651915981)*x[1]
        ref[0,0,0,0]=(0.609727033659)*x_ref[0]+(-0.901858940633)*x_ref[1]
        ref[0,0,0,1]=(-0.221259461243)*x_ref[0]+(0.351136340908)*x_ref[1]
        ref[0,0,1,0]=(0.497173734282)*x_ref[0]+(0.899142906899)*x_ref[1]
        ref[0,0,1,1]=(-1.55206649551)*x_ref[0]+(-0.680257384044)*x_ref[1]
        ref[0,0,2,0]=(-0.932033638033)*x_ref[0]+(-0.46503758126)*x_ref[1]
        ref[0,0,2,1]=(-0.0372650123525)*x_ref[0]+(0.0173714029098)*x_ref[1]
        ref[0,1,0,0]=(0.855963098465)*x_ref[0]+(0.760176320898)*x_ref[1]
        ref[0,1,0,1]=(1.37546120327)*x_ref[0]+(1.72916158331)*x_ref[1]
        ref[0,1,1,0]=(0.25469861613)*x_ref[0]+(0.71445716033)*x_ref[1]
        ref[0,1,1,1]=(0.198455659765)*x_ref[0]+(0.0410276829495)*x_ref[1]
        ref[0,1,2,0]=(-0.628080427667)*x_ref[0]+(-0.0197206025984)*x_ref[1]
        ref[0,1,2,1]=(1.26759029895)*x_ref[0]+(1.15914953253)*x_ref[1]
        ref[0,2,0,0]=(-1.43203053441)*x_ref[0]+(0.366301170069)*x_ref[1]
        ref[0,2,0,1]=(0.420527802229)*x_ref[0]+(-0.709209747415)*x_ref[1]
        ref[0,2,1,0]=(0.399042264829)*x_ref[0]+(0.0453132754731)*x_ref[1]
        ref[0,2,1,1]=(1.2074346062)*x_ref[0]+(-1.14163410992)*x_ref[1]
        ref[0,2,2,0]=(-0.00270984158569)*x_ref[0]+(-1.63216200182)*x_ref[1]
        ref[0,2,2,1]=(0.174276088847)*x_ref[0]+(-1.04789472231)*x_ref[1]
        ref[0,3,0,0]=(-0.47704729036)*x_ref[0]+(-1.2550637204)*x_ref[1]
        ref[0,3,0,1]=(0.348356873191)*x_ref[0]+(-0.796668578265)*x_ref[1]
        ref[0,3,1,0]=(-0.52724725097)*x_ref[0]+(1.26163123592)*x_ref[1]
        ref[0,3,1,1]=(-1.52681799261)*x_ref[0]+(0.936041931735)*x_ref[1]
        ref[0,3,2,0]=(1.33689673219)*x_ref[0]+(-1.34901214018)*x_ref[1]
        ref[0,3,2,1]=(1.57651286047)*x_ref[0]+(0.245413762864)*x_ref[1]
        ref[0,4,0,0]=(0.194500181498)*x_ref[0]+(-0.0231951844441)*x_ref[1]
        ref[0,4,0,1]=(-1.49372076697)*x_ref[0]+(-0.111369678181)*x_ref[1]
        ref[0,4,1,0]=(-0.74121646836)*x_ref[0]+(-0.817740156307)*x_ref[1]
        ref[0,4,1,1]=(-0.233643397123)*x_ref[0]+(0.672715068796)*x_ref[1]
        ref[0,4,2,0]=(1.05850947579)*x_ref[0]+(0.386484386191)*x_ref[1]
        ref[0,4,2,1]=(1.45105922963)*x_ref[0]+(-0.367458625996)*x_ref[1]
        ref[1,0,0,0]=(-0.391803788669)*x_ref[0]+(0.797592300662)*x_ref[1]
        ref[1,0,0,1]=(-0.205299072883)*x_ref[0]+(0.677427935558)*x_ref[1]
        ref[1,0,1,0]=(1.56225329031)*x_ref[0]+(1.17468092515)*x_ref[1]
        ref[1,0,1,1]=(1.4586005658)*x_ref[0]+(1.36481062943)*x_ref[1]
        ref[1,0,2,0]=(-1.53281881413)*x_ref[0]+(0.207028833974)*x_ref[1]
        ref[1,0,2,1]=(0.0469239691648)*x_ref[0]+(1.03547812705)*x_ref[1]
        ref[1,1,0,0]=(1.39614335897)*x_ref[0]+(0.82046489695)*x_ref[1]
        ref[1,1,0,1]=(-0.775462741637)*x_ref[0]+(-0.275081761646)*x_ref[1]
        ref[1,1,1,0]=(0.173353028608)*x_ref[0]+(-1.05620431816)*x_ref[1]
        ref[1,1,1,1]=(-0.464192870247)*x_ref[0]+(0.212204784206)*x_ref[1]
        ref[1,1,2,0]=(-0.383813531866)*x_ref[0]+(-0.0773759374234)*x_ref[1]
        ref[1,1,2,1]=(-1.07811385759)*x_ref[0]+(-0.457232330819)*x_ref[1]
        ref[1,2,0,0]=(-0.295869147576)*x_ref[0]+(0.166843915631)*x_ref[1]
        ref[1,2,0,1]=(0.571421034325)*x_ref[0]+(1.46722583245)*x_ref[1]
        ref[1,2,1,0]=(-0.34494944257)*x_ref[0]+(-0.286867052491)*x_ref[1]
        ref[1,2,1,1]=(1.43128885761)*x_ref[0]+(0.21277887035)*x_ref[1]
        ref[1,2,2,0]=(-1.19397187873)*x_ref[0]+(0.126397665227)*x_ref[1]
        ref[1,2,2,1]=(0.171368751954)*x_ref[0]+(-1.12100254625)*x_ref[1]
        ref[1,3,0,0]=(0.0975333077604)*x_ref[0]+(0.583524660188)*x_ref[1]
        ref[1,3,0,1]=(0.0271450955458)*x_ref[0]+(0.368978332867)*x_ref[1]
        ref[1,3,1,0]=(1.22806968486)*x_ref[0]+(-0.569932748468)*x_ref[1]
        ref[1,3,1,1]=(-0.691989839521)*x_ref[0]+(-0.829822182922)*x_ref[1]
        ref[1,3,2,0]=(0.743776534098)*x_ref[0]+(1.3668937127)*x_ref[1]
        ref[1,3,2,1]=(0.192206575204)*x_ref[0]+(0.96576702824)*x_ref[1]
        ref[1,4,0,0]=(0.149381460079)*x_ref[0]+(-0.987875538036)*x_ref[1]
        ref[1,4,0,1]=(-0.706855836683)*x_ref[0]+(0.0410884211631)*x_ref[1]
        ref[1,4,1,0]=(-0.222650274)*x_ref[0]+(0.59589242048)*x_ref[1]
        ref[1,4,1,1]=(0.541489186743)*x_ref[0]+(-1.22543996074)*x_ref[1]
        ref[1,4,2,0]=(0.630234824652)*x_ref[0]+(0.508436575179)*x_ref[1]
        ref[1,4,2,1]=(0.990062359294)*x_ref[0]+(-0.639884693407)*x_ref[1]
        ref[2,0,0,0]=(0.378887461733)*x_ref[0]+(1.71999809344)*x_ref[1]
        ref[2,0,0,1]=(0.182412683544)*x_ref[0]+(-0.401183243877)*x_ref[1]
        ref[2,0,1,0]=(0.325775290396)*x_ref[0]+(-0.656088786126)*x_ref[1]
        ref[2,0,1,1]=(0.316355229742)*x_ref[0]+(1.54087545195)*x_ref[1]
        ref[2,0,2,0]=(1.14203881183)*x_ref[0]+(0.703207191364)*x_ref[1]
        ref[2,0,2,1]=(-1.05211790226)*x_ref[0]+(1.68596483036)*x_ref[1]
        ref[2,1,0,0]=(0.704109387934)*x_ref[0]+(-0.589003921574)*x_ref[1]
        ref[2,1,0,1]=(-0.382489922028)*x_ref[0]+(0.387647642942)*x_ref[1]
        ref[2,1,1,0]=(-0.0343434958891)*x_ref[0]+(-0.119176404279)*x_ref[1]
        ref[2,1,1,1]=(0.567853527929)*x_ref[0]+(1.32523261969)*x_ref[1]
        ref[2,1,2,0]=(1.03496000653)*x_ref[0]+(0.110203591459)*x_ref[1]
        ref[2,1,2,1]=(0.560149997806)*x_ref[0]+(-1.62893742873)*x_ref[1]
        ref[2,2,0,0]=(1.1801792312)*x_ref[0]+(1.1912385935)*x_ref[1]
        ref[2,2,0,1]=(0.936932817634)*x_ref[0]+(-1.49082021974)*x_ref[1]
        ref[2,2,1,0]=(-0.305582892069)*x_ref[0]+(-0.267235698563)*x_ref[1]
        ref[2,2,1,1]=(0.107449730184)*x_ref[0]+(-0.325267993615)*x_ref[1]
        ref[2,2,2,0]=(-1.08118986347)*x_ref[0]+(0.144190727334)*x_ref[1]
        ref[2,2,2,1]=(1.44755060146)*x_ref[0]+(-0.127612338817)*x_ref[1]
        ref[2,3,0,0]=(-0.666943112999)*x_ref[0]+(-0.780402371989)*x_ref[1]
        ref[2,3,0,1]=(0.724206717771)*x_ref[0]+(-0.523593908278)*x_ref[1]
        ref[2,3,1,0]=(1.86101197199)*x_ref[0]+(-0.710894911302)*x_ref[1]
        ref[2,3,1,1]=(-1.74409232023)*x_ref[0]+(0.407821816271)*x_ref[1]
        ref[2,3,2,0]=(0.646304172575)*x_ref[0]+(0.0132628778543)*x_ref[1]
        ref[2,3,2,1]=(-0.746041438673)*x_ref[0]+(0.516304872741)*x_ref[1]
        ref[2,4,0,0]=(0.0326900201283)*x_ref[0]+(-0.533021156401)*x_ref[1]
        ref[2,4,0,1]=(0.088598956981)*x_ref[0]+(0.764307970269)*x_ref[1]
        ref[2,4,1,0]=(-0.749553477756)*x_ref[0]+(0.379646792939)*x_ref[1]
        ref[2,4,1,1]=(-0.243635321378)*x_ref[0]+(-0.470022600261)*x_ref[1]
        ref[2,4,2,0]=(0.586398757247)*x_ref[0]+(-0.116982897683)*x_ref[1]
        ref[2,4,2,1]=(-0.989012121879)*x_ref[0]+(-1.25006535257)*x_ref[1]
        ref[3,0,0,0]=(1.25228306081)*x_ref[0]+(-0.0932717035896)*x_ref[1]
        ref[3,0,0,1]=(1.5975516771)*x_ref[0]+(-0.177733286111)*x_ref[1]
        ref[3,0,1,0]=(-0.659910070911)*x_ref[0]+(-0.0999498944812)*x_ref[1]
        ref[3,0,1,1]=(-0.299870273662)*x_ref[0]+(-1.22977656744)*x_ref[1]
        ref[3,0,2,0]=(-0.158211581143)*x_ref[0]+(-0.141421380503)*x_ref[1]
        ref[3,0,2,1]=(-1.48226452788)*x_ref[0]+(-0.731264943309)*x_ref[1]
        ref[3,1,0,0]=(-1.64589827853)*x_ref[0]+(1.19692573998)*x_ref[1]
        ref[3,1,0,1]=(0.361329655357)*x_ref[0]+(0.234310482171)*x_ref[1]
        ref[3,1,1,0]=(-1.14077649516)*x_ref[0]+(0.132657084065)*x_ref[1]
        ref[3,1,1,1]=(1.41067975663)*x_ref[0]+(0.818243117884)*x_ref[1]
        ref[3,1,2,0]=(-1.53137884342)*x_ref[0]+(-0.0127805333093)*x_ref[1]
        ref[3,1,2,1]=(0.240455755486)*x_ref[0]+(-1.02361753893)*x_ref[1]
        ref[3,2,0,0]=(-0.334803473072)*x_ref[0]+(-0.807452510423)*x_ref[1]
        ref[3,2,0,1]=(0.617324536974)*x_ref[0]+(-0.733163471774)*x_ref[1]
        ref[3,2,1,0]=(-0.634580050897)*x_ref[0]+(0.362106278168)*x_ref[1]
        ref[3,2,1,1]=(-0.21525531818)*x_ref[0]+(0.982902540898)*x_ref[1]
        ref[3,2,2,0]=(-0.590682803195)*x_ref[0]+(-1.27537880632)*x_ref[1]
        ref[3,2,2,1]=(0.142578313438)*x_ref[0]+(0.212341687291)*x_ref[1]
        ref[3,3,0,0]=(-0.758345317216)*x_ref[0]+(-0.971914631359)*x_ref[1]
        ref[3,3,0,1]=(0.569463909234)*x_ref[0]+(0.498848917338)*x_ref[1]
        ref[3,3,1,0]=(-0.771456575632)*x_ref[0]+(-0.130456072944)*x_ref[1]
        ref[3,3,1,1]=(0.811799269245)*x_ref[0]+(-0.829647203396)*x_ref[1]
        ref[3,3,2,0]=(-0.814979427693)*x_ref[0]+(-0.0416092796813)*x_ref[1]
        ref[3,3,2,1]=(0.933434189127)*x_ref[0]+(0.60794704936)*x_ref[1]
        ref[3,4,0,0]=(0.160280566478)*x_ref[0]+(-0.0141616040181)*x_ref[1]
        ref[3,4,0,1]=(-1.34835766268)*x_ref[0]+(0.178951225861)*x_ref[1]
        ref[3,4,1,0]=(1.67632814658)*x_ref[0]+(1.31316096483)*x_ref[1]
        ref[3,4,1,1]=(-0.603403275376)*x_ref[0]+(0.0607753526431)*x_ref[1]
        ref[3,4,2,0]=(-0.178440295473)*x_ref[0]+(0.318102694371)*x_ref[1]
        ref[3,4,2,1]=(0.781285160749)*x_ref[0]+(0.625651915981)*x_ref[1]
      else:
        s[0,0,0,0]=(0.847845815599)*x[0]+(-0.610234211476)*x[1]+(-0.627625510082)*x[2]
        s[0,0,0,1]=(0.0126184651041)*x[0]+(-0.0651294063564)*x[1]+(-0.274615935201)*x[2]
        s[0,0,1,0]=(-0.545925417969)*x[0]+(0.421929302201)*x[1]+(-0.910418625383)*x[2]
        s[0,0,1,1]=(1.58754252937)*x[0]+(0.110111591047)*x[1]+(-1.64780563736)*x[2]
        s[0,0,2,0]=(-0.510331643992)*x[0]+(0.827685326542)*x[1]+(1.31447962917)*x[2]
        s[0,0,2,1]=(-1.29575211142)*x[0]+(-0.318096803401)*x[1]+(-0.274660613777)*x[2]
        s[0,1,0,0]=(0.77444944835)*x[0]+(-1.0893411122)*x[1]+(0.331702744143)*x[2]
        s[0,1,0,1]=(0.624693916432)*x[0]+(-0.306552086758)*x[1]+(-0.845524417807)*x[2]
        s[0,1,1,0]=(0.0219492668609)*x[0]+(-1.13712865204)*x[1]+(-0.193051406533)*x[2]
        s[0,1,1,1]=(-1.32696889246)*x[0]+(1.63857177904)*x[1]+(0.194095476936)*x[2]
        s[0,1,2,0]=(1.19169978642)*x[0]+(-1.88274910505)*x[1]+(0.321438561021)*x[2]
        s[0,1,2,1]=(-0.765748081809)*x[0]+(-0.163422665563)*x[1]+(0.256721885141)*x[2]
        s[0,2,0,0]=(-0.863281319991)*x[0]+(-0.021828308413)*x[1]+(0.868312889989)*x[2]
        s[0,2,0,1]=(-0.57479341478)*x[0]+(0.404861121887)*x[1]+(-0.274037007699)*x[2]
        s[0,2,1,0]=(-0.529171478131)*x[0]+(0.836001872018)*x[1]+(-1.12226189724)*x[2]
        s[0,2,1,1]=(-0.821929160411)*x[0]+(0.0466762602824)*x[1]+(-0.0514331500765)*x[2]
        s[0,2,2,0]=(-0.605940618544)*x[0]+(-1.54379919132)*x[1]+(1.63243021269)*x[2]
        s[0,2,2,1]=(-1.49695860655)*x[0]+(-0.601345463475)*x[1]+(-0.753443592162)*x[2]
        s[0,3,0,0]=(1.33514081849)*x[0]+(0.955849232345)*x[1]+(-0.872221354599)*x[2]
        s[0,3,0,1]=(0.939731890901)*x[0]+(-1.16116012822)*x[1]+(-0.601602002743)*x[2]
        s[0,3,1,0]=(-0.675545889094)*x[0]+(0.736755090077)*x[1]+(-0.437370345974)*x[2]
        s[0,3,1,1]=(-1.17899966213)*x[0]+(0.422125349488)*x[1]+(-0.815649229407)*x[2]
        s[0,3,2,0]=(0.0773790438387)*x[0]+(0.148378160687)*x[1]+(-1.0156040466)*x[2]
        s[0,3,2,1]=(0.519422925349)*x[0]+(0.189993892084)*x[1]+(1.59654399673)*x[2]
        s[0,4,0,0]=(-0.0593352727817)*x[0]+(0.0865384232804)*x[1]+(-0.316122229956)*x[2]
        s[0,4,0,1]=(0.935220504817)*x[0]+(0.868143343952)*x[1]+(-0.470286174039)*x[2]
        s[0,4,1,0]=(0.763506086212)*x[0]+(-0.0756795330833)*x[1]+(1.06812672934)*x[2]
        s[0,4,1,1]=(-0.446580256351)*x[0]+(0.653986880233)*x[1]+(0.634364696873)*x[2]
        s[0,4,2,0]=(-0.72871248403)*x[0]+(0.0839150289011)*x[1]+(0.178899292496)*x[2]
        s[0,4,2,1]=(1.30808288445)*x[0]+(0.548931211195)*x[1]+(-0.971585668771)*x[2]
        s[1,0,0,0]=(-0.737171903787)*x[0]+(-0.137110160891)*x[1]+(1.24817883553)*x[2]
        s[1,0,0,1]=(0.589824248199)*x[0]+(-0.191688810075)*x[1]+(1.33820051734)*x[2]
        s[1,0,1,0]=(-0.598063103136)*x[0]+(0.347705502684)*x[1]+(0.490747627936)*x[2]
        s[1,0,1,1]=(0.287829427303)*x[0]+(-0.853875346674)*x[1]+(-0.227722773909)*x[2]
        s[1,0,2,0]=(-1.22183236343)*x[0]+(-0.162406713796)*x[1]+(-0.436433884201)*x[2]
        s[1,0,2,1]=(0.929372688401)*x[0]+(0.469931776805)*x[1]+(0.739110972137)*x[2]
        s[1,1,0,0]=(0.391824586)*x[0]+(-0.551498198462)*x[1]+(-0.588935046796)*x[2]
        s[1,1,0,1]=(0.849805745827)*x[0]+(0.557121315313)*x[1]+(1.80284061615)*x[2]
        s[1,1,1,0]=(-1.62270432051)*x[0]+(-1.55943636923)*x[1]+(0.629204169009)*x[2]
        s[1,1,1,1]=(-0.191271118514)*x[0]+(-0.454718665581)*x[1]+(0.405403171722)*x[2]
        s[1,1,2,0]=(0.461750123507)*x[0]+(0.537594040674)*x[1]+(-0.500046587877)*x[2]
        s[1,1,2,1]=(1.07800193455)*x[0]+(0.0757746249706)*x[1]+(0.358223454759)*x[2]
        s[1,2,0,0]=(-1.31360669142)*x[0]+(0.908413426737)*x[1]+(0.986904165818)*x[2]
        s[1,2,0,1]=(-0.250344247716)*x[0]+(0.596746427887)*x[1]+(0.846846599186)*x[2]
        s[1,2,1,0]=(1.65761753214)*x[0]+(0.0969407749043)*x[1]+(0.197913502281)*x[2]
        s[1,2,1,1]=(0.808454446977)*x[0]+(-0.107176416129)*x[1]+(-0.115562549181)*x[2]
        s[1,2,2,0]=(-0.410108442793)*x[0]+(0.250496842214)*x[1]+(0.403806076052)*x[2]
        s[1,2,2,1]=(1.58110488679)*x[0]+(-0.171583112102)*x[1]+(0.441813895222)*x[2]
        s[1,3,0,0]=(-0.519973841295)*x[0]+(0.14862500493)*x[1]+(-0.0257159960311)*x[2]
        s[1,3,0,1]=(-0.710423889723)*x[0]+(0.0941990549739)*x[1]+(-0.147877062943)*x[2]
        s[1,3,1,0]=(-0.19917762038)*x[0]+(-1.06873791711)*x[1]+(1.37688275432)*x[2]
        s[1,3,1,1]=(-0.570477493665)*x[0]+(0.0705814270652)*x[1]+(-1.10186231057)*x[2]
        s[1,3,2,0]=(-1.04891017002)*x[0]+(-0.136784107636)*x[1]+(-1.65953016693)*x[2]
        s[1,3,2,1]=(-0.674805716398)*x[0]+(-0.211803508128)*x[1]+(-0.116463503974)*x[2]
        s[1,4,0,0]=(0.269217220297)*x[0]+(0.810722194772)*x[1]+(1.60977315112)*x[2]
        s[1,4,0,1]=(-0.0213286766746)*x[0]+(1.0974274553)*x[1]+(0.275633207025)*x[2]
        s[1,4,1,0]=(-0.601230245409)*x[0]+(-0.756418955933)*x[1]+(-0.46194303915)*x[2]
        s[1,4,1,1]=(-0.0907667209143)*x[0]+(-1.86623564673)*x[1]+(0.552729474701)*x[2]
        s[1,4,2,0]=(-1.09694541489)*x[0]+(-0.496193881555)*x[1]+(-0.0126133249168)*x[2]
        s[1,4,2,1]=(-0.648959246633)*x[0]+(-0.60655325639)*x[1]+(0.438258051102)*x[2]
        s[2,0,0,0]=(-0.100512547019)*x[0]+(-1.37291269079)*x[1]+(1.63084319876)*x[2]
        s[2,0,0,1]=(-0.50510326916)*x[0]+(0.308799737351)*x[1]+(1.63506761477)*x[2]
        s[2,0,1,0]=(0.708408519723)*x[0]+(-0.0759833841695)*x[1]+(-0.980974095374)*x[2]
        s[2,0,1,1]=(-1.59727324143)*x[0]+(0.995250195962)*x[1]+(0.169527316083)*x[2]
        s[2,0,2,0]=(0.766935817054)*x[0]+(0.455564399031)*x[1]+(0.605772951782)*x[2]
        s[2,0,2,1]=(-0.820755214894)*x[0]+(-0.43359371038)*x[1]+(-0.0942398466964)*x[2]
        s[2,1,0,0]=(0.390476682801)*x[0]+(1.11627447827)*x[1]+(0.670843405466)*x[2]
        s[2,1,0,1]=(-0.433594870483)*x[0]+(-0.940710004402)*x[1]+(1.10756041487)*x[2]
        s[2,1,1,0]=(-1.21880018868)*x[0]+(0.380422307344)*x[1]+(-0.864017731789)*x[2]
        s[2,1,1,1]=(-0.194168346974)*x[0]+(0.324183983028)*x[1]+(0.741456579501)*x[2]
        s[2,1,2,0]=(-0.134211025912)*x[0]+(0.497311041207)*x[1]+(0.33014196376)*x[2]
        s[2,1,2,1]=(-1.3501981937)*x[0]+(1.67601514859)*x[1]+(-1.60084725544)*x[2]
        s[2,2,0,0]=(-0.8367457236)*x[0]+(-1.15628524562)*x[1]+(0.342983125417)*x[2]
        s[2,2,0,1]=(0.956733778283)*x[0]+(-0.404880336577)*x[1]+(-0.350927185105)*x[2]
        s[2,2,1,0]=(0.854842449281)*x[0]+(-0.185001304765)*x[1]+(1.15245157168)*x[2]
        s[2,2,1,1]=(0.00584831242623)*x[0]+(-0.562950548984)*x[1]+(0.785766513431)*x[2]
        s[2,2,2,0]=(0.272759945164)*x[0]+(-0.568207376307)*x[1]+(-0.155997758854)*x[2]
        s[2,2,2,1]=(-0.105324133175)*x[0]+(1.11807221711)*x[1]+(0.122751975298)*x[2]
        s[2,3,0,0]=(0.685932400265)*x[0]+(-0.666847052302)*x[1]+(0.596321369477)*x[2]
        s[2,3,0,1]=(0.335067153518)*x[0]+(0.268453607431)*x[1]+(0.247881738828)*x[2]
        s[2,3,1,0]=(-1.89084909179)*x[0]+(-0.529751828744)*x[1]+(-0.959358202828)*x[2]
        s[2,3,1,1]=(1.06725689989)*x[0]+(-0.709878517637)*x[1]+(0.968886366535)*x[2]
        s[2,3,2,0]=(-1.63466018652)*x[0]+(0.517481248484)*x[1]+(0.109232845638)*x[2]
        s[2,3,2,1]=(0.0716425266603)*x[0]+(-0.58040972115)*x[1]+(0.346948738894)*x[2]
        s[2,4,0,0]=(0.193599854213)*x[0]+(-1.52744079327)*x[1]+(-0.0377038338183)*x[2]
        s[2,4,0,1]=(-0.0296855161658)*x[0]+(0.266976994363)*x[1]+(-0.832771979863)*x[2]
        s[2,4,1,0]=(-0.885217283694)*x[0]+(0.657425725373)*x[1]+(0.385584782531)*x[2]
        s[2,4,1,1]=(-0.249912733602)*x[0]+(0.156704487203)*x[1]+(0.000393408500501)*x[2]
        s[2,4,2,0]=(-0.083303562971)*x[0]+(0.748009334975)*x[1]+(-0.411323946112)*x[2]
        s[2,4,2,1]=(0.143782087293)*x[0]+(-1.86497054401)*x[1]+(-0.398161751889)*x[2]
        s[3,0,0,0]=(-0.531041035506)*x[0]+(0.998737023878)*x[1]+(-0.719326852701)*x[2]
        s[3,0,0,1]=(-1.10320774278)*x[0]+(-1.27795876669)*x[1]+(0.57451462957)*x[2]
        s[3,0,1,0]=(-0.319275578502)*x[0]+(-1.51721910203)*x[1]+(-0.677470578443)*x[2]
        s[3,0,1,1]=(0.7584807917)*x[0]+(0.0523998746885)*x[1]+(0.502182688228)*x[2]
        s[3,0,2,0]=(-0.905809892125)*x[0]+(0.479751975382)*x[1]+(-0.0778539387977)*x[2]
        s[3,0,2,1]=(-0.724891388357)*x[0]+(0.941493554841)*x[1]+(-1.51514565838)*x[2]
        s[3,1,0,0]=(0.815118793475)*x[0]+(0.0503368907581)*x[1]+(-0.502091253213)*x[2]
        s[3,1,0,1]=(-1.50859117948)*x[0]+(0.31279517213)*x[1]+(-0.339295035594)*x[2]
        s[3,1,1,0]=(0.246825549192)*x[0]+(0.687251908372)*x[1]+(0.133971781776)*x[2]
        s[3,1,1,1]=(0.470105585328)*x[0]+(0.382971101979)*x[1]+(-0.469397699886)*x[2]
        s[3,1,2,0]=(-1.45842815542)*x[0]+(-1.00979255513)*x[1]+(1.45386034482)*x[2]
        s[3,1,2,1]=(0.172130164577)*x[0]+(1.25034739991)*x[1]+(-1.15953423359)*x[2]
        s[3,2,0,0]=(-0.58261393055)*x[0]+(0.730381032663)*x[1]+(1.29377076249)*x[2]
        s[3,2,0,1]=(0.696715362351)*x[0]+(-1.37598351546)*x[1]+(-1.26545085967)*x[2]
        s[3,2,1,0]=(-0.0511282610422)*x[0]+(0.58041679118)*x[1]+(-0.0810357412151)*x[2]
        s[3,2,1,1]=(-0.750057228803)*x[0]+(-0.750703125074)*x[1]+(-1.21748956536)*x[2]
        s[3,2,2,0]=(-0.455746666635)*x[0]+(-0.720020627516)*x[1]+(-0.0911281889971)*x[2]
        s[3,2,2,1]=(-0.0681894163236)*x[0]+(0.101592029278)*x[1]+(1.10954864406)*x[2]
        s[3,3,0,0]=(0.433898443791)*x[0]+(0.901180030496)*x[1]+(0.795641263874)*x[2]
        s[3,3,0,1]=(-0.213306810738)*x[0]+(-0.852539239286)*x[1]+(-1.25905416259)*x[2]
        s[3,3,1,0]=(-0.668307384473)*x[0]+(-1.37753879862)*x[1]+(-0.593731800903)*x[2]
        s[3,3,1,1]=(0.815464294514)*x[0]+(0.835389640785)*x[1]+(0.933029468258)*x[2]
        s[3,3,2,0]=(0.565194274444)*x[0]+(0.584119707001)*x[1]+(-0.53812189952)*x[2]
        s[3,3,2,1]=(-0.903604209314)*x[0]+(0.370887007483)*x[1]+(-0.548135915608)*x[2]
        s[3,4,0,0]=(-0.819290139492)*x[0]+(-0.256800498012)*x[1]+(0.350363336935)*x[2]
        s[3,4,0,1]=(0.863170182723)*x[0]+(0.0101766489337)*x[1]+(1.34009635664)*x[2]
        s[3,4,1,0]=(1.03350734136)*x[0]+(1.37672500233)*x[1]+(1.35134434663)*x[2]
        s[3,4,1,1]=(1.4883144839)*x[0]+(0.410688244039)*x[1]+(0.759267998436)*x[2]
        s[3,4,2,0]=(0.373190738096)*x[0]+(1.22136515279)*x[1]+(-0.0365030690962)*x[2]
        s[3,4,2,1]=(-0.607437402737)*x[0]+(0.750130654364)*x[1]+(0.084778884789)*x[2]
        ref[0,0,0,0]=(0.847845815599)*x_ref[0]+(-0.610234211476)*x_ref[1]+(-0.627625510082)*x_ref[2]
        ref[0,0,0,1]=(0.0126184651041)*x_ref[0]+(-0.0651294063564)*x_ref[1]+(-0.274615935201)*x_ref[2]
        ref[0,0,1,0]=(-0.545925417969)*x_ref[0]+(0.421929302201)*x_ref[1]+(-0.910418625383)*x_ref[2]
        ref[0,0,1,1]=(1.58754252937)*x_ref[0]+(0.110111591047)*x_ref[1]+(-1.64780563736)*x_ref[2]
        ref[0,0,2,0]=(-0.510331643992)*x_ref[0]+(0.827685326542)*x_ref[1]+(1.31447962917)*x_ref[2]
        ref[0,0,2,1]=(-1.29575211142)*x_ref[0]+(-0.318096803401)*x_ref[1]+(-0.274660613777)*x_ref[2]
        ref[0,1,0,0]=(0.77444944835)*x_ref[0]+(-1.0893411122)*x_ref[1]+(0.331702744143)*x_ref[2]
        ref[0,1,0,1]=(0.624693916432)*x_ref[0]+(-0.306552086758)*x_ref[1]+(-0.845524417807)*x_ref[2]
        ref[0,1,1,0]=(0.0219492668609)*x_ref[0]+(-1.13712865204)*x_ref[1]+(-0.193051406533)*x_ref[2]
        ref[0,1,1,1]=(-1.32696889246)*x_ref[0]+(1.63857177904)*x_ref[1]+(0.194095476936)*x_ref[2]
        ref[0,1,2,0]=(1.19169978642)*x_ref[0]+(-1.88274910505)*x_ref[1]+(0.321438561021)*x_ref[2]
        ref[0,1,2,1]=(-0.765748081809)*x_ref[0]+(-0.163422665563)*x_ref[1]+(0.256721885141)*x_ref[2]
        ref[0,2,0,0]=(-0.863281319991)*x_ref[0]+(-0.021828308413)*x_ref[1]+(0.868312889989)*x_ref[2]
        ref[0,2,0,1]=(-0.57479341478)*x_ref[0]+(0.404861121887)*x_ref[1]+(-0.274037007699)*x_ref[2]
        ref[0,2,1,0]=(-0.529171478131)*x_ref[0]+(0.836001872018)*x_ref[1]+(-1.12226189724)*x_ref[2]
        ref[0,2,1,1]=(-0.821929160411)*x_ref[0]+(0.0466762602824)*x_ref[1]+(-0.0514331500765)*x_ref[2]
        ref[0,2,2,0]=(-0.605940618544)*x_ref[0]+(-1.54379919132)*x_ref[1]+(1.63243021269)*x_ref[2]
        ref[0,2,2,1]=(-1.49695860655)*x_ref[0]+(-0.601345463475)*x_ref[1]+(-0.753443592162)*x_ref[2]
        ref[0,3,0,0]=(1.33514081849)*x_ref[0]+(0.955849232345)*x_ref[1]+(-0.872221354599)*x_ref[2]
        ref[0,3,0,1]=(0.939731890901)*x_ref[0]+(-1.16116012822)*x_ref[1]+(-0.601602002743)*x_ref[2]
        ref[0,3,1,0]=(-0.675545889094)*x_ref[0]+(0.736755090077)*x_ref[1]+(-0.437370345974)*x_ref[2]
        ref[0,3,1,1]=(-1.17899966213)*x_ref[0]+(0.422125349488)*x_ref[1]+(-0.815649229407)*x_ref[2]
        ref[0,3,2,0]=(0.0773790438387)*x_ref[0]+(0.148378160687)*x_ref[1]+(-1.0156040466)*x_ref[2]
        ref[0,3,2,1]=(0.519422925349)*x_ref[0]+(0.189993892084)*x_ref[1]+(1.59654399673)*x_ref[2]
        ref[0,4,0,0]=(-0.0593352727817)*x_ref[0]+(0.0865384232804)*x_ref[1]+(-0.316122229956)*x_ref[2]
        ref[0,4,0,1]=(0.935220504817)*x_ref[0]+(0.868143343952)*x_ref[1]+(-0.470286174039)*x_ref[2]
        ref[0,4,1,0]=(0.763506086212)*x_ref[0]+(-0.0756795330833)*x_ref[1]+(1.06812672934)*x_ref[2]
        ref[0,4,1,1]=(-0.446580256351)*x_ref[0]+(0.653986880233)*x_ref[1]+(0.634364696873)*x_ref[2]
        ref[0,4,2,0]=(-0.72871248403)*x_ref[0]+(0.0839150289011)*x_ref[1]+(0.178899292496)*x_ref[2]
        ref[0,4,2,1]=(1.30808288445)*x_ref[0]+(0.548931211195)*x_ref[1]+(-0.971585668771)*x_ref[2]
        ref[1,0,0,0]=(-0.737171903787)*x_ref[0]+(-0.137110160891)*x_ref[1]+(1.24817883553)*x_ref[2]
        ref[1,0,0,1]=(0.589824248199)*x_ref[0]+(-0.191688810075)*x_ref[1]+(1.33820051734)*x_ref[2]
        ref[1,0,1,0]=(-0.598063103136)*x_ref[0]+(0.347705502684)*x_ref[1]+(0.490747627936)*x_ref[2]
        ref[1,0,1,1]=(0.287829427303)*x_ref[0]+(-0.853875346674)*x_ref[1]+(-0.227722773909)*x_ref[2]
        ref[1,0,2,0]=(-1.22183236343)*x_ref[0]+(-0.162406713796)*x_ref[1]+(-0.436433884201)*x_ref[2]
        ref[1,0,2,1]=(0.929372688401)*x_ref[0]+(0.469931776805)*x_ref[1]+(0.739110972137)*x_ref[2]
        ref[1,1,0,0]=(0.391824586)*x_ref[0]+(-0.551498198462)*x_ref[1]+(-0.588935046796)*x_ref[2]
        ref[1,1,0,1]=(0.849805745827)*x_ref[0]+(0.557121315313)*x_ref[1]+(1.80284061615)*x_ref[2]
        ref[1,1,1,0]=(-1.62270432051)*x_ref[0]+(-1.55943636923)*x_ref[1]+(0.629204169009)*x_ref[2]
        ref[1,1,1,1]=(-0.191271118514)*x_ref[0]+(-0.454718665581)*x_ref[1]+(0.405403171722)*x_ref[2]
        ref[1,1,2,0]=(0.461750123507)*x_ref[0]+(0.537594040674)*x_ref[1]+(-0.500046587877)*x_ref[2]
        ref[1,1,2,1]=(1.07800193455)*x_ref[0]+(0.0757746249706)*x_ref[1]+(0.358223454759)*x_ref[2]
        ref[1,2,0,0]=(-1.31360669142)*x_ref[0]+(0.908413426737)*x_ref[1]+(0.986904165818)*x_ref[2]
        ref[1,2,0,1]=(-0.250344247716)*x_ref[0]+(0.596746427887)*x_ref[1]+(0.846846599186)*x_ref[2]
        ref[1,2,1,0]=(1.65761753214)*x_ref[0]+(0.0969407749043)*x_ref[1]+(0.197913502281)*x_ref[2]
        ref[1,2,1,1]=(0.808454446977)*x_ref[0]+(-0.107176416129)*x_ref[1]+(-0.115562549181)*x_ref[2]
        ref[1,2,2,0]=(-0.410108442793)*x_ref[0]+(0.250496842214)*x_ref[1]+(0.403806076052)*x_ref[2]
        ref[1,2,2,1]=(1.58110488679)*x_ref[0]+(-0.171583112102)*x_ref[1]+(0.441813895222)*x_ref[2]
        ref[1,3,0,0]=(-0.519973841295)*x_ref[0]+(0.14862500493)*x_ref[1]+(-0.0257159960311)*x_ref[2]
        ref[1,3,0,1]=(-0.710423889723)*x_ref[0]+(0.0941990549739)*x_ref[1]+(-0.147877062943)*x_ref[2]
        ref[1,3,1,0]=(-0.19917762038)*x_ref[0]+(-1.06873791711)*x_ref[1]+(1.37688275432)*x_ref[2]
        ref[1,3,1,1]=(-0.570477493665)*x_ref[0]+(0.0705814270652)*x_ref[1]+(-1.10186231057)*x_ref[2]
        ref[1,3,2,0]=(-1.04891017002)*x_ref[0]+(-0.136784107636)*x_ref[1]+(-1.65953016693)*x_ref[2]
        ref[1,3,2,1]=(-0.674805716398)*x_ref[0]+(-0.211803508128)*x_ref[1]+(-0.116463503974)*x_ref[2]
        ref[1,4,0,0]=(0.269217220297)*x_ref[0]+(0.810722194772)*x_ref[1]+(1.60977315112)*x_ref[2]
        ref[1,4,0,1]=(-0.0213286766746)*x_ref[0]+(1.0974274553)*x_ref[1]+(0.275633207025)*x_ref[2]
        ref[1,4,1,0]=(-0.601230245409)*x_ref[0]+(-0.756418955933)*x_ref[1]+(-0.46194303915)*x_ref[2]
        ref[1,4,1,1]=(-0.0907667209143)*x_ref[0]+(-1.86623564673)*x_ref[1]+(0.552729474701)*x_ref[2]
        ref[1,4,2,0]=(-1.09694541489)*x_ref[0]+(-0.496193881555)*x_ref[1]+(-0.0126133249168)*x_ref[2]
        ref[1,4,2,1]=(-0.648959246633)*x_ref[0]+(-0.60655325639)*x_ref[1]+(0.438258051102)*x_ref[2]
        ref[2,0,0,0]=(-0.100512547019)*x_ref[0]+(-1.37291269079)*x_ref[1]+(1.63084319876)*x_ref[2]
        ref[2,0,0,1]=(-0.50510326916)*x_ref[0]+(0.308799737351)*x_ref[1]+(1.63506761477)*x_ref[2]
        ref[2,0,1,0]=(0.708408519723)*x_ref[0]+(-0.0759833841695)*x_ref[1]+(-0.980974095374)*x_ref[2]
        ref[2,0,1,1]=(-1.59727324143)*x_ref[0]+(0.995250195962)*x_ref[1]+(0.169527316083)*x_ref[2]
        ref[2,0,2,0]=(0.766935817054)*x_ref[0]+(0.455564399031)*x_ref[1]+(0.605772951782)*x_ref[2]
        ref[2,0,2,1]=(-0.820755214894)*x_ref[0]+(-0.43359371038)*x_ref[1]+(-0.0942398466964)*x_ref[2]
        ref[2,1,0,0]=(0.390476682801)*x_ref[0]+(1.11627447827)*x_ref[1]+(0.670843405466)*x_ref[2]
        ref[2,1,0,1]=(-0.433594870483)*x_ref[0]+(-0.940710004402)*x_ref[1]+(1.10756041487)*x_ref[2]
        ref[2,1,1,0]=(-1.21880018868)*x_ref[0]+(0.380422307344)*x_ref[1]+(-0.864017731789)*x_ref[2]
        ref[2,1,1,1]=(-0.194168346974)*x_ref[0]+(0.324183983028)*x_ref[1]+(0.741456579501)*x_ref[2]
        ref[2,1,2,0]=(-0.134211025912)*x_ref[0]+(0.497311041207)*x_ref[1]+(0.33014196376)*x_ref[2]
        ref[2,1,2,1]=(-1.3501981937)*x_ref[0]+(1.67601514859)*x_ref[1]+(-1.60084725544)*x_ref[2]
        ref[2,2,0,0]=(-0.8367457236)*x_ref[0]+(-1.15628524562)*x_ref[1]+(0.342983125417)*x_ref[2]
        ref[2,2,0,1]=(0.956733778283)*x_ref[0]+(-0.404880336577)*x_ref[1]+(-0.350927185105)*x_ref[2]
        ref[2,2,1,0]=(0.854842449281)*x_ref[0]+(-0.185001304765)*x_ref[1]+(1.15245157168)*x_ref[2]
        ref[2,2,1,1]=(0.00584831242623)*x_ref[0]+(-0.562950548984)*x_ref[1]+(0.785766513431)*x_ref[2]
        ref[2,2,2,0]=(0.272759945164)*x_ref[0]+(-0.568207376307)*x_ref[1]+(-0.155997758854)*x_ref[2]
        ref[2,2,2,1]=(-0.105324133175)*x_ref[0]+(1.11807221711)*x_ref[1]+(0.122751975298)*x_ref[2]
        ref[2,3,0,0]=(0.685932400265)*x_ref[0]+(-0.666847052302)*x_ref[1]+(0.596321369477)*x_ref[2]
        ref[2,3,0,1]=(0.335067153518)*x_ref[0]+(0.268453607431)*x_ref[1]+(0.247881738828)*x_ref[2]
        ref[2,3,1,0]=(-1.89084909179)*x_ref[0]+(-0.529751828744)*x_ref[1]+(-0.959358202828)*x_ref[2]
        ref[2,3,1,1]=(1.06725689989)*x_ref[0]+(-0.709878517637)*x_ref[1]+(0.968886366535)*x_ref[2]
        ref[2,3,2,0]=(-1.63466018652)*x_ref[0]+(0.517481248484)*x_ref[1]+(0.109232845638)*x_ref[2]
        ref[2,3,2,1]=(0.0716425266603)*x_ref[0]+(-0.58040972115)*x_ref[1]+(0.346948738894)*x_ref[2]
        ref[2,4,0,0]=(0.193599854213)*x_ref[0]+(-1.52744079327)*x_ref[1]+(-0.0377038338183)*x_ref[2]
        ref[2,4,0,1]=(-0.0296855161658)*x_ref[0]+(0.266976994363)*x_ref[1]+(-0.832771979863)*x_ref[2]
        ref[2,4,1,0]=(-0.885217283694)*x_ref[0]+(0.657425725373)*x_ref[1]+(0.385584782531)*x_ref[2]
        ref[2,4,1,1]=(-0.249912733602)*x_ref[0]+(0.156704487203)*x_ref[1]+(0.000393408500501)*x_ref[2]
        ref[2,4,2,0]=(-0.083303562971)*x_ref[0]+(0.748009334975)*x_ref[1]+(-0.411323946112)*x_ref[2]
        ref[2,4,2,1]=(0.143782087293)*x_ref[0]+(-1.86497054401)*x_ref[1]+(-0.398161751889)*x_ref[2]
        ref[3,0,0,0]=(-0.531041035506)*x_ref[0]+(0.998737023878)*x_ref[1]+(-0.719326852701)*x_ref[2]
        ref[3,0,0,1]=(-1.10320774278)*x_ref[0]+(-1.27795876669)*x_ref[1]+(0.57451462957)*x_ref[2]
        ref[3,0,1,0]=(-0.319275578502)*x_ref[0]+(-1.51721910203)*x_ref[1]+(-0.677470578443)*x_ref[2]
        ref[3,0,1,1]=(0.7584807917)*x_ref[0]+(0.0523998746885)*x_ref[1]+(0.502182688228)*x_ref[2]
        ref[3,0,2,0]=(-0.905809892125)*x_ref[0]+(0.479751975382)*x_ref[1]+(-0.0778539387977)*x_ref[2]
        ref[3,0,2,1]=(-0.724891388357)*x_ref[0]+(0.941493554841)*x_ref[1]+(-1.51514565838)*x_ref[2]
        ref[3,1,0,0]=(0.815118793475)*x_ref[0]+(0.0503368907581)*x_ref[1]+(-0.502091253213)*x_ref[2]
        ref[3,1,0,1]=(-1.50859117948)*x_ref[0]+(0.31279517213)*x_ref[1]+(-0.339295035594)*x_ref[2]
        ref[3,1,1,0]=(0.246825549192)*x_ref[0]+(0.687251908372)*x_ref[1]+(0.133971781776)*x_ref[2]
        ref[3,1,1,1]=(0.470105585328)*x_ref[0]+(0.382971101979)*x_ref[1]+(-0.469397699886)*x_ref[2]
        ref[3,1,2,0]=(-1.45842815542)*x_ref[0]+(-1.00979255513)*x_ref[1]+(1.45386034482)*x_ref[2]
        ref[3,1,2,1]=(0.172130164577)*x_ref[0]+(1.25034739991)*x_ref[1]+(-1.15953423359)*x_ref[2]
        ref[3,2,0,0]=(-0.58261393055)*x_ref[0]+(0.730381032663)*x_ref[1]+(1.29377076249)*x_ref[2]
        ref[3,2,0,1]=(0.696715362351)*x_ref[0]+(-1.37598351546)*x_ref[1]+(-1.26545085967)*x_ref[2]
        ref[3,2,1,0]=(-0.0511282610422)*x_ref[0]+(0.58041679118)*x_ref[1]+(-0.0810357412151)*x_ref[2]
        ref[3,2,1,1]=(-0.750057228803)*x_ref[0]+(-0.750703125074)*x_ref[1]+(-1.21748956536)*x_ref[2]
        ref[3,2,2,0]=(-0.455746666635)*x_ref[0]+(-0.720020627516)*x_ref[1]+(-0.0911281889971)*x_ref[2]
        ref[3,2,2,1]=(-0.0681894163236)*x_ref[0]+(0.101592029278)*x_ref[1]+(1.10954864406)*x_ref[2]
        ref[3,3,0,0]=(0.433898443791)*x_ref[0]+(0.901180030496)*x_ref[1]+(0.795641263874)*x_ref[2]
        ref[3,3,0,1]=(-0.213306810738)*x_ref[0]+(-0.852539239286)*x_ref[1]+(-1.25905416259)*x_ref[2]
        ref[3,3,1,0]=(-0.668307384473)*x_ref[0]+(-1.37753879862)*x_ref[1]+(-0.593731800903)*x_ref[2]
        ref[3,3,1,1]=(0.815464294514)*x_ref[0]+(0.835389640785)*x_ref[1]+(0.933029468258)*x_ref[2]
        ref[3,3,2,0]=(0.565194274444)*x_ref[0]+(0.584119707001)*x_ref[1]+(-0.53812189952)*x_ref[2]
        ref[3,3,2,1]=(-0.903604209314)*x_ref[0]+(0.370887007483)*x_ref[1]+(-0.548135915608)*x_ref[2]
        ref[3,4,0,0]=(-0.819290139492)*x_ref[0]+(-0.256800498012)*x_ref[1]+(0.350363336935)*x_ref[2]
        ref[3,4,0,1]=(0.863170182723)*x_ref[0]+(0.0101766489337)*x_ref[1]+(1.34009635664)*x_ref[2]
        ref[3,4,1,0]=(1.03350734136)*x_ref[0]+(1.37672500233)*x_ref[1]+(1.35134434663)*x_ref[2]
        ref[3,4,1,1]=(1.4883144839)*x_ref[0]+(0.410688244039)*x_ref[1]+(0.759267998436)*x_ref[2]
        ref[3,4,2,0]=(0.373190738096)*x_ref[0]+(1.22136515279)*x_ref[1]+(-0.0365030690962)*x_ref[2]
        ref[3,4,2,1]=(-0.607437402737)*x_ref[0]+(0.750130654364)*x_ref[1]+(0.084778884789)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_ReducedContinuousFunction_rank0(self):
      """
      tests interpolate for rank 0 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedContinuousFunction onto ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(),dim=dim)
      s=Data(0,(),w)
      ref=Data(0,(),w_ref)
      if dim==2:
        s=(-1.64974193328)*x[0]+(1.68675911612)*x[1]
        ref=(-1.64974193328)*x_ref[0]+(1.68675911612)*x_ref[1]
      else:
        s=(-0.517718083008)*x[0]+(0.479679773182)*x[1]+(0.72459612383)*x[2]
        ref=(-0.517718083008)*x_ref[0]+(0.479679773182)*x_ref[1]+(0.72459612383)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_ReducedContinuousFunction_rank1(self):
      """
      tests interpolate for rank 1 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedContinuousFunction onto ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(2,),dim=dim)
      s=Data(0,(2,),w)
      ref=Data(0,(2,),w_ref)
      if dim==2:
        s[0]=(-0.415290633997)*x[0]+(0.118775036022)*x[1]
        s[1]=(-0.833749289762)*x[0]+(0.121297221921)*x[1]
        ref[0]=(-0.415290633997)*x[0]+(0.118775036022)*x[1]
        ref[1]=(-0.833749289762)*x[0]+(0.121297221921)*x[1]
      else:
        s[0]=(-0.336174941008)*x[0]+(-0.844918885743)*x[1]+(-0.410649979186)*x[2]
        s[1]=(0.112527237498)*x[0]+(-0.867632388528)*x[1]+(1.42359104479)*x[2]
        ref[0]=(-0.336174941008)*x[0]+(-0.844918885743)*x[1]+(-0.410649979186)*x[2]
        ref[1]=(0.112527237498)*x[0]+(-0.867632388528)*x[1]+(1.42359104479)*x[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_ReducedContinuousFunction_rank2(self):
      """
      tests interpolate for rank 2 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedContinuousFunction onto ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5),dim=dim)
      s=Data(0,(4, 5),w)
      ref=Data(0,(4, 5),w_ref)
      if dim==2:
        s[0,0]=(-0.833920166667)*x[0]+(-0.105734397941)*x[1]
        s[0,1]=(-0.0535194301789)*x[0]+(1.2869199411)*x[1]
        s[0,2]=(-1.81783608134)*x[0]+(-0.0858549172679)*x[1]
        s[0,3]=(0.133570573696)*x[0]+(0.398281712973)*x[1]
        s[0,4]=(-1.2514060292)*x[0]+(-0.24743946144)*x[1]
        s[1,0]=(0.917760776668)*x[0]+(0.179758313228)*x[1]
        s[1,1]=(0.677772149724)*x[0]+(0.762705763999)*x[1]
        s[1,2]=(-0.505172381789)*x[0]+(0.412365083581)*x[1]
        s[1,3]=(0.0481873177823)*x[0]+(-0.449871653505)*x[1]
        s[1,4]=(-1.43052283016)*x[0]+(0.553426132082)*x[1]
        s[2,0]=(0.337172157552)*x[0]+(1.56154138911)*x[1]
        s[2,1]=(-0.338189138128)*x[0]+(-0.62614063612)*x[1]
        s[2,2]=(-1.51606676783)*x[0]+(-0.13283454533)*x[1]
        s[2,3]=(0.727068106091)*x[0]+(1.00949960235)*x[1]
        s[2,4]=(0.297941246402)*x[0]+(0.952119168452)*x[1]
        s[3,0]=(0.724157315025)*x[0]+(0.962405533793)*x[1]
        s[3,1]=(-0.239168209123)*x[0]+(1.28280794616)*x[1]
        s[3,2]=(-0.0336024791682)*x[0]+(-0.775860593407)*x[1]
        s[3,3]=(-0.118809020482)*x[0]+(0.51890714417)*x[1]
        s[3,4]=(0.374315302545)*x[0]+(-0.258942994)*x[1]
        ref[0,0]=(-0.833920166667)*x_ref[0]+(-0.105734397941)*x_ref[1]
        ref[0,1]=(-0.0535194301789)*x_ref[0]+(1.2869199411)*x_ref[1]
        ref[0,2]=(-1.81783608134)*x_ref[0]+(-0.0858549172679)*x_ref[1]
        ref[0,3]=(0.133570573696)*x_ref[0]+(0.398281712973)*x_ref[1]
        ref[0,4]=(-1.2514060292)*x_ref[0]+(-0.24743946144)*x_ref[1]
        ref[1,0]=(0.917760776668)*x_ref[0]+(0.179758313228)*x_ref[1]
        ref[1,1]=(0.677772149724)*x_ref[0]+(0.762705763999)*x_ref[1]
        ref[1,2]=(-0.505172381789)*x_ref[0]+(0.412365083581)*x_ref[1]
        ref[1,3]=(0.0481873177823)*x_ref[0]+(-0.449871653505)*x_ref[1]
        ref[1,4]=(-1.43052283016)*x_ref[0]+(0.553426132082)*x_ref[1]
        ref[2,0]=(0.337172157552)*x_ref[0]+(1.56154138911)*x_ref[1]
        ref[2,1]=(-0.338189138128)*x_ref[0]+(-0.62614063612)*x_ref[1]
        ref[2,2]=(-1.51606676783)*x_ref[0]+(-0.13283454533)*x_ref[1]
        ref[2,3]=(0.727068106091)*x_ref[0]+(1.00949960235)*x_ref[1]
        ref[2,4]=(0.297941246402)*x_ref[0]+(0.952119168452)*x_ref[1]
        ref[3,0]=(0.724157315025)*x_ref[0]+(0.962405533793)*x_ref[1]
        ref[3,1]=(-0.239168209123)*x_ref[0]+(1.28280794616)*x_ref[1]
        ref[3,2]=(-0.0336024791682)*x_ref[0]+(-0.775860593407)*x_ref[1]
        ref[3,3]=(-0.118809020482)*x_ref[0]+(0.51890714417)*x_ref[1]
        ref[3,4]=(0.374315302545)*x_ref[0]+(-0.258942994)*x_ref[1]
      else:
        s[0,0]=(1.18033303246)*x[0]+(0.59372740835)*x[1]+(-0.684959540977)*x[2]
        s[0,1]=(0.65301033503)*x[0]+(-1.17147722359)*x[1]+(0.455404166733)*x[2]
        s[0,2]=(-0.412048337979)*x[0]+(-1.24933561194)*x[1]+(-0.636287202752)*x[2]
        s[0,3]=(0.122156598551)*x[0]+(0.103952485283)*x[1]+(-0.564471099255)*x[2]
        s[0,4]=(0.737873603859)*x[0]+(0.139615778712)*x[1]+(0.523544473477)*x[2]
        s[1,0]=(0.0198124265514)*x[0]+(-0.335073882197)*x[1]+(-0.805965691618)*x[2]
        s[1,1]=(-1.28485879121)*x[0]+(-0.618847596536)*x[1]+(1.32879899911)*x[2]
        s[1,2]=(1.09319515176)*x[0]+(0.543230471078)*x[1]+(1.56430808151)*x[2]
        s[1,3]=(0.541425658238)*x[0]+(0.0140217060596)*x[1]+(0.603558416867)*x[2]
        s[1,4]=(0.303110106189)*x[0]+(-0.103009794147)*x[1]+(-1.68324105571)*x[2]
        s[2,0]=(-1.65609235169)*x[0]+(-0.290040263496)*x[1]+(0.477073850071)*x[2]
        s[2,1]=(-0.897378807931)*x[0]+(0.191583115578)*x[1]+(0.213226828436)*x[2]
        s[2,2]=(0.523724313062)*x[0]+(0.214413673529)*x[1]+(0.533345615094)*x[2]
        s[2,3]=(-0.994043729754)*x[0]+(-0.307957055273)*x[1]+(0.0422995421427)*x[2]
        s[2,4]=(1.22590973976)*x[0]+(-1.65014223339)*x[1]+(-1.59726996804)*x[2]
        s[3,0]=(-0.0870943824713)*x[0]+(-0.45766718732)*x[1]+(-0.878874753774)*x[2]
        s[3,1]=(0.244470697307)*x[0]+(-0.0429568231364)*x[1]+(-1.57098356615)*x[2]
        s[3,2]=(1.35091699523)*x[0]+(-0.127297590922)*x[1]+(-0.556702799475)*x[2]
        s[3,3]=(0.189117166393)*x[0]+(0.317562328009)*x[1]+(-0.59788285723)*x[2]
        s[3,4]=(-0.155355776878)*x[0]+(-0.616350184089)*x[1]+(0.296083681054)*x[2]
        ref[0,0]=(1.18033303246)*x_ref[0]+(0.59372740835)*x_ref[1]+(-0.684959540977)*x_ref[2]
        ref[0,1]=(0.65301033503)*x_ref[0]+(-1.17147722359)*x_ref[1]+(0.455404166733)*x_ref[2]
        ref[0,2]=(-0.412048337979)*x_ref[0]+(-1.24933561194)*x_ref[1]+(-0.636287202752)*x_ref[2]
        ref[0,3]=(0.122156598551)*x_ref[0]+(0.103952485283)*x_ref[1]+(-0.564471099255)*x_ref[2]
        ref[0,4]=(0.737873603859)*x_ref[0]+(0.139615778712)*x_ref[1]+(0.523544473477)*x_ref[2]
        ref[1,0]=(0.0198124265514)*x_ref[0]+(-0.335073882197)*x_ref[1]+(-0.805965691618)*x_ref[2]
        ref[1,1]=(-1.28485879121)*x_ref[0]+(-0.618847596536)*x_ref[1]+(1.32879899911)*x_ref[2]
        ref[1,2]=(1.09319515176)*x_ref[0]+(0.543230471078)*x_ref[1]+(1.56430808151)*x_ref[2]
        ref[1,3]=(0.541425658238)*x_ref[0]+(0.0140217060596)*x_ref[1]+(0.603558416867)*x_ref[2]
        ref[1,4]=(0.303110106189)*x_ref[0]+(-0.103009794147)*x_ref[1]+(-1.68324105571)*x_ref[2]
        ref[2,0]=(-1.65609235169)*x_ref[0]+(-0.290040263496)*x_ref[1]+(0.477073850071)*x_ref[2]
        ref[2,1]=(-0.897378807931)*x_ref[0]+(0.191583115578)*x_ref[1]+(0.213226828436)*x_ref[2]
        ref[2,2]=(0.523724313062)*x_ref[0]+(0.214413673529)*x_ref[1]+(0.533345615094)*x_ref[2]
        ref[2,3]=(-0.994043729754)*x_ref[0]+(-0.307957055273)*x_ref[1]+(0.0422995421427)*x_ref[2]
        ref[2,4]=(1.22590973976)*x_ref[0]+(-1.65014223339)*x_ref[1]+(-1.59726996804)*x_ref[2]
        ref[3,0]=(-0.0870943824713)*x_ref[0]+(-0.45766718732)*x_ref[1]+(-0.878874753774)*x_ref[2]
        ref[3,1]=(0.244470697307)*x_ref[0]+(-0.0429568231364)*x_ref[1]+(-1.57098356615)*x_ref[2]
        ref[3,2]=(1.35091699523)*x_ref[0]+(-0.127297590922)*x_ref[1]+(-0.556702799475)*x_ref[2]
        ref[3,3]=(0.189117166393)*x_ref[0]+(0.317562328009)*x_ref[1]+(-0.59788285723)*x_ref[2]
        ref[3,4]=(-0.155355776878)*x_ref[0]+(-0.616350184089)*x_ref[1]+(0.296083681054)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_ReducedContinuousFunction_rank3(self):
      """
      tests interpolate for rank 3 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedContinuousFunction onto ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(6, 2, 2),dim=dim)
      s=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2),w_ref)
      if dim==2:
        s[0,0,0]=(0.0730549738553)*x[0]+(0.585649475666)*x[1]
        s[0,0,1]=(-0.449075310223)*x[0]+(1.39116964942)*x[1]
        s[0,1,0]=(0.0210032238157)*x[0]+(1.33904896697)*x[1]
        s[0,1,1]=(0.556125399236)*x[0]+(-0.00311484949343)*x[1]
        s[1,0,0]=(0.938296971476)*x[0]+(1.13223213262)*x[1]
        s[1,0,1]=(0.744133292867)*x[0]+(0.0271311625871)*x[1]
        s[1,1,0]=(-0.0478922372672)*x[0]+(-1.08882574207)*x[1]
        s[1,1,1]=(0.738328994373)*x[0]+(-0.730449948444)*x[1]
        s[2,0,0]=(0.66593099915)*x[0]+(-0.185918605751)*x[1]
        s[2,0,1]=(0.450972410432)*x[0]+(1.3393675839)*x[1]
        s[2,1,0]=(0.897370528228)*x[0]+(0.765127571621)*x[1]
        s[2,1,1]=(-0.349658725406)*x[0]+(-0.536332219959)*x[1]
        s[3,0,0]=(1.11112898963)*x[0]+(1.50253301112)*x[1]
        s[3,0,1]=(0.238957188525)*x[0]+(0.775368032877)*x[1]
        s[3,1,0]=(0.612991596596)*x[0]+(0.0718037077356)*x[1]
        s[3,1,1]=(-0.40039392925)*x[0]+(-1.58712410632)*x[1]
        s[4,0,0]=(0.23241254342)*x[0]+(-0.547054930927)*x[1]
        s[4,0,1]=(-0.976995206698)*x[0]+(1.251791534)*x[1]
        s[4,1,0]=(-1.13243662763)*x[0]+(0.56739947752)*x[1]
        s[4,1,1]=(-0.543613180121)*x[0]+(0.649805921416)*x[1]
        s[5,0,0]=(-0.0690689383083)*x[0]+(0.818563064557)*x[1]
        s[5,0,1]=(0.362899720121)*x[0]+(-0.112604112729)*x[1]
        s[5,1,0]=(-0.291756903179)*x[0]+(-0.538221294595)*x[1]
        s[5,1,1]=(0.60406331286)*x[0]+(0.228744896658)*x[1]
        ref[0,0,0]=(0.0730549738553)*x_ref[0]+(0.585649475666)*x_ref[1]
        ref[0,0,1]=(-0.449075310223)*x_ref[0]+(1.39116964942)*x_ref[1]
        ref[0,1,0]=(0.0210032238157)*x_ref[0]+(1.33904896697)*x_ref[1]
        ref[0,1,1]=(0.556125399236)*x_ref[0]+(-0.00311484949343)*x_ref[1]
        ref[1,0,0]=(0.938296971476)*x_ref[0]+(1.13223213262)*x_ref[1]
        ref[1,0,1]=(0.744133292867)*x_ref[0]+(0.0271311625871)*x_ref[1]
        ref[1,1,0]=(-0.0478922372672)*x_ref[0]+(-1.08882574207)*x_ref[1]
        ref[1,1,1]=(0.738328994373)*x_ref[0]+(-0.730449948444)*x_ref[1]
        ref[2,0,0]=(0.66593099915)*x_ref[0]+(-0.185918605751)*x_ref[1]
        ref[2,0,1]=(0.450972410432)*x_ref[0]+(1.3393675839)*x_ref[1]
        ref[2,1,0]=(0.897370528228)*x_ref[0]+(0.765127571621)*x_ref[1]
        ref[2,1,1]=(-0.349658725406)*x_ref[0]+(-0.536332219959)*x_ref[1]
        ref[3,0,0]=(1.11112898963)*x_ref[0]+(1.50253301112)*x_ref[1]
        ref[3,0,1]=(0.238957188525)*x_ref[0]+(0.775368032877)*x_ref[1]
        ref[3,1,0]=(0.612991596596)*x_ref[0]+(0.0718037077356)*x_ref[1]
        ref[3,1,1]=(-0.40039392925)*x_ref[0]+(-1.58712410632)*x_ref[1]
        ref[4,0,0]=(0.23241254342)*x_ref[0]+(-0.547054930927)*x_ref[1]
        ref[4,0,1]=(-0.976995206698)*x_ref[0]+(1.251791534)*x_ref[1]
        ref[4,1,0]=(-1.13243662763)*x_ref[0]+(0.56739947752)*x_ref[1]
        ref[4,1,1]=(-0.543613180121)*x_ref[0]+(0.649805921416)*x_ref[1]
        ref[5,0,0]=(-0.0690689383083)*x_ref[0]+(0.818563064557)*x_ref[1]
        ref[5,0,1]=(0.362899720121)*x_ref[0]+(-0.112604112729)*x_ref[1]
        ref[5,1,0]=(-0.291756903179)*x_ref[0]+(-0.538221294595)*x_ref[1]
        ref[5,1,1]=(0.60406331286)*x_ref[0]+(0.228744896658)*x_ref[1]
      else:
        s[0,0,0]=(0.811538724853)*x[0]+(-0.347250364239)*x[1]+(-0.116011094391)*x[2]
        s[0,0,1]=(1.69112205586)*x[0]+(-1.04197300041)*x[1]+(0.849118123462)*x[2]
        s[0,1,0]=(1.30816943778)*x[0]+(1.01636941747)*x[1]+(-0.849746116393)*x[2]
        s[0,1,1]=(0.888683598925)*x[0]+(-1.60930356378)*x[1]+(0.522151570265)*x[2]
        s[1,0,0]=(1.03821316703)*x[0]+(-1.19647941504)*x[1]+(-1.08543246224)*x[2]
        s[1,0,1]=(-0.314182278588)*x[0]+(0.144825414517)*x[1]+(0.162613879231)*x[2]
        s[1,1,0]=(0.0416209427382)*x[0]+(0.0390092175082)*x[1]+(1.55616366687)*x[2]
        s[1,1,1]=(-0.577251716148)*x[0]+(0.717529140134)*x[1]+(-0.567199280535)*x[2]
        s[2,0,0]=(1.26716303575)*x[0]+(0.119576983216)*x[1]+(-0.602181033831)*x[2]
        s[2,0,1]=(0.533935330879)*x[0]+(0.292003869006)*x[1]+(-0.0917835680855)*x[2]
        s[2,1,0]=(-1.50218600857)*x[0]+(1.23418550746)*x[1]+(-0.600240530145)*x[2]
        s[2,1,1]=(0.689151741161)*x[0]+(0.909077564156)*x[1]+(0.924638961344)*x[2]
        s[3,0,0]=(-0.763411711557)*x[0]+(-0.862413521307)*x[1]+(1.49095932554)*x[2]
        s[3,0,1]=(-0.215589144977)*x[0]+(0.305301450314)*x[1]+(-0.468853406603)*x[2]
        s[3,1,0]=(-0.433045929433)*x[0]+(-0.907669929171)*x[1]+(0.917462300039)*x[2]
        s[3,1,1]=(0.229593212783)*x[0]+(-1.79371444035)*x[1]+(-0.848409349534)*x[2]
        s[4,0,0]=(1.40249316312)*x[0]+(-0.766609553683)*x[1]+(0.337911083882)*x[2]
        s[4,0,1]=(0.728650674867)*x[0]+(0.631222570369)*x[1]+(1.2491315131)*x[2]
        s[4,1,0]=(1.03998601102)*x[0]+(-1.81204917532)*x[1]+(0.781499164268)*x[2]
        s[4,1,1]=(-0.151999545303)*x[0]+(-0.767585861167)*x[1]+(1.17261654125)*x[2]
        s[5,0,0]=(0.0632249575346)*x[0]+(0.333565970302)*x[1]+(-1.66679001354)*x[2]
        s[5,0,1]=(0.945677231929)*x[0]+(-1.15936834775)*x[1]+(-0.644891644457)*x[2]
        s[5,1,0]=(1.08939087839)*x[0]+(0.687932839014)*x[1]+(-0.473426758917)*x[2]
        s[5,1,1]=(0.0349915672648)*x[0]+(-1.48593425129)*x[1]+(0.145421756236)*x[2]
        ref[0,0,0]=(0.811538724853)*x_ref[0]+(-0.347250364239)*x_ref[1]+(-0.116011094391)*x_ref[2]
        ref[0,0,1]=(1.69112205586)*x_ref[0]+(-1.04197300041)*x_ref[1]+(0.849118123462)*x_ref[2]
        ref[0,1,0]=(1.30816943778)*x_ref[0]+(1.01636941747)*x_ref[1]+(-0.849746116393)*x_ref[2]
        ref[0,1,1]=(0.888683598925)*x_ref[0]+(-1.60930356378)*x_ref[1]+(0.522151570265)*x_ref[2]
        ref[1,0,0]=(1.03821316703)*x_ref[0]+(-1.19647941504)*x_ref[1]+(-1.08543246224)*x_ref[2]
        ref[1,0,1]=(-0.314182278588)*x_ref[0]+(0.144825414517)*x_ref[1]+(0.162613879231)*x_ref[2]
        ref[1,1,0]=(0.0416209427382)*x_ref[0]+(0.0390092175082)*x_ref[1]+(1.55616366687)*x_ref[2]
        ref[1,1,1]=(-0.577251716148)*x_ref[0]+(0.717529140134)*x_ref[1]+(-0.567199280535)*x_ref[2]
        ref[2,0,0]=(1.26716303575)*x_ref[0]+(0.119576983216)*x_ref[1]+(-0.602181033831)*x_ref[2]
        ref[2,0,1]=(0.533935330879)*x_ref[0]+(0.292003869006)*x_ref[1]+(-0.0917835680855)*x_ref[2]
        ref[2,1,0]=(-1.50218600857)*x_ref[0]+(1.23418550746)*x_ref[1]+(-0.600240530145)*x_ref[2]
        ref[2,1,1]=(0.689151741161)*x_ref[0]+(0.909077564156)*x_ref[1]+(0.924638961344)*x_ref[2]
        ref[3,0,0]=(-0.763411711557)*x_ref[0]+(-0.862413521307)*x_ref[1]+(1.49095932554)*x_ref[2]
        ref[3,0,1]=(-0.215589144977)*x_ref[0]+(0.305301450314)*x_ref[1]+(-0.468853406603)*x_ref[2]
        ref[3,1,0]=(-0.433045929433)*x_ref[0]+(-0.907669929171)*x_ref[1]+(0.917462300039)*x_ref[2]
        ref[3,1,1]=(0.229593212783)*x_ref[0]+(-1.79371444035)*x_ref[1]+(-0.848409349534)*x_ref[2]
        ref[4,0,0]=(1.40249316312)*x_ref[0]+(-0.766609553683)*x_ref[1]+(0.337911083882)*x_ref[2]
        ref[4,0,1]=(0.728650674867)*x_ref[0]+(0.631222570369)*x_ref[1]+(1.2491315131)*x_ref[2]
        ref[4,1,0]=(1.03998601102)*x_ref[0]+(-1.81204917532)*x_ref[1]+(0.781499164268)*x_ref[2]
        ref[4,1,1]=(-0.151999545303)*x_ref[0]+(-0.767585861167)*x_ref[1]+(1.17261654125)*x_ref[2]
        ref[5,0,0]=(0.0632249575346)*x_ref[0]+(0.333565970302)*x_ref[1]+(-1.66679001354)*x_ref[2]
        ref[5,0,1]=(0.945677231929)*x_ref[0]+(-1.15936834775)*x_ref[1]+(-0.644891644457)*x_ref[2]
        ref[5,1,0]=(1.08939087839)*x_ref[0]+(0.687932839014)*x_ref[1]+(-0.473426758917)*x_ref[2]
        ref[5,1,1]=(0.0349915672648)*x_ref[0]+(-1.48593425129)*x_ref[1]+(0.145421756236)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_ReducedContinuousFunction_rank4(self):
      """
      tests interpolate for rank 4 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedContinuousFunction onto ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5, 3, 2),dim=dim)
      s=Data(0,(4, 5, 3, 2),w)
      ref=Data(0,(4, 5, 3, 2),w_ref)
      if dim==2:
        s[0,0,0,0]=(0.609727033659)*x[0]+(-0.901858940633)*x[1]
        s[0,0,0,1]=(-0.221259461243)*x[0]+(0.351136340908)*x[1]
        s[0,0,1,0]=(0.497173734282)*x[0]+(0.899142906899)*x[1]
        s[0,0,1,1]=(-1.55206649551)*x[0]+(-0.680257384044)*x[1]
        s[0,0,2,0]=(-0.932033638033)*x[0]+(-0.46503758126)*x[1]
        s[0,0,2,1]=(-0.0372650123525)*x[0]+(0.0173714029098)*x[1]
        s[0,1,0,0]=(0.855963098465)*x[0]+(0.760176320898)*x[1]
        s[0,1,0,1]=(1.37546120327)*x[0]+(1.72916158331)*x[1]
        s[0,1,1,0]=(0.25469861613)*x[0]+(0.71445716033)*x[1]
        s[0,1,1,1]=(0.198455659765)*x[0]+(0.0410276829495)*x[1]
        s[0,1,2,0]=(-0.628080427667)*x[0]+(-0.0197206025984)*x[1]
        s[0,1,2,1]=(1.26759029895)*x[0]+(1.15914953253)*x[1]
        s[0,2,0,0]=(-1.43203053441)*x[0]+(0.366301170069)*x[1]
        s[0,2,0,1]=(0.420527802229)*x[0]+(-0.709209747415)*x[1]
        s[0,2,1,0]=(0.399042264829)*x[0]+(0.0453132754731)*x[1]
        s[0,2,1,1]=(1.2074346062)*x[0]+(-1.14163410992)*x[1]
        s[0,2,2,0]=(-0.00270984158569)*x[0]+(-1.63216200182)*x[1]
        s[0,2,2,1]=(0.174276088847)*x[0]+(-1.04789472231)*x[1]
        s[0,3,0,0]=(-0.47704729036)*x[0]+(-1.2550637204)*x[1]
        s[0,3,0,1]=(0.348356873191)*x[0]+(-0.796668578265)*x[1]
        s[0,3,1,0]=(-0.52724725097)*x[0]+(1.26163123592)*x[1]
        s[0,3,1,1]=(-1.52681799261)*x[0]+(0.936041931735)*x[1]
        s[0,3,2,0]=(1.33689673219)*x[0]+(-1.34901214018)*x[1]
        s[0,3,2,1]=(1.57651286047)*x[0]+(0.245413762864)*x[1]
        s[0,4,0,0]=(0.194500181498)*x[0]+(-0.0231951844441)*x[1]
        s[0,4,0,1]=(-1.49372076697)*x[0]+(-0.111369678181)*x[1]
        s[0,4,1,0]=(-0.74121646836)*x[0]+(-0.817740156307)*x[1]
        s[0,4,1,1]=(-0.233643397123)*x[0]+(0.672715068796)*x[1]
        s[0,4,2,0]=(1.05850947579)*x[0]+(0.386484386191)*x[1]
        s[0,4,2,1]=(1.45105922963)*x[0]+(-0.367458625996)*x[1]
        s[1,0,0,0]=(-0.391803788669)*x[0]+(0.797592300662)*x[1]
        s[1,0,0,1]=(-0.205299072883)*x[0]+(0.677427935558)*x[1]
        s[1,0,1,0]=(1.56225329031)*x[0]+(1.17468092515)*x[1]
        s[1,0,1,1]=(1.4586005658)*x[0]+(1.36481062943)*x[1]
        s[1,0,2,0]=(-1.53281881413)*x[0]+(0.207028833974)*x[1]
        s[1,0,2,1]=(0.0469239691648)*x[0]+(1.03547812705)*x[1]
        s[1,1,0,0]=(1.39614335897)*x[0]+(0.82046489695)*x[1]
        s[1,1,0,1]=(-0.775462741637)*x[0]+(-0.275081761646)*x[1]
        s[1,1,1,0]=(0.173353028608)*x[0]+(-1.05620431816)*x[1]
        s[1,1,1,1]=(-0.464192870247)*x[0]+(0.212204784206)*x[1]
        s[1,1,2,0]=(-0.383813531866)*x[0]+(-0.0773759374234)*x[1]
        s[1,1,2,1]=(-1.07811385759)*x[0]+(-0.457232330819)*x[1]
        s[1,2,0,0]=(-0.295869147576)*x[0]+(0.166843915631)*x[1]
        s[1,2,0,1]=(0.571421034325)*x[0]+(1.46722583245)*x[1]
        s[1,2,1,0]=(-0.34494944257)*x[0]+(-0.286867052491)*x[1]
        s[1,2,1,1]=(1.43128885761)*x[0]+(0.21277887035)*x[1]
        s[1,2,2,0]=(-1.19397187873)*x[0]+(0.126397665227)*x[1]
        s[1,2,2,1]=(0.171368751954)*x[0]+(-1.12100254625)*x[1]
        s[1,3,0,0]=(0.0975333077604)*x[0]+(0.583524660188)*x[1]
        s[1,3,0,1]=(0.0271450955458)*x[0]+(0.368978332867)*x[1]
        s[1,3,1,0]=(1.22806968486)*x[0]+(-0.569932748468)*x[1]
        s[1,3,1,1]=(-0.691989839521)*x[0]+(-0.829822182922)*x[1]
        s[1,3,2,0]=(0.743776534098)*x[0]+(1.3668937127)*x[1]
        s[1,3,2,1]=(0.192206575204)*x[0]+(0.96576702824)*x[1]
        s[1,4,0,0]=(0.149381460079)*x[0]+(-0.987875538036)*x[1]
        s[1,4,0,1]=(-0.706855836683)*x[0]+(0.0410884211631)*x[1]
        s[1,4,1,0]=(-0.222650274)*x[0]+(0.59589242048)*x[1]
        s[1,4,1,1]=(0.541489186743)*x[0]+(-1.22543996074)*x[1]
        s[1,4,2,0]=(0.630234824652)*x[0]+(0.508436575179)*x[1]
        s[1,4,2,1]=(0.990062359294)*x[0]+(-0.639884693407)*x[1]
        s[2,0,0,0]=(0.378887461733)*x[0]+(1.71999809344)*x[1]
        s[2,0,0,1]=(0.182412683544)*x[0]+(-0.401183243877)*x[1]
        s[2,0,1,0]=(0.325775290396)*x[0]+(-0.656088786126)*x[1]
        s[2,0,1,1]=(0.316355229742)*x[0]+(1.54087545195)*x[1]
        s[2,0,2,0]=(1.14203881183)*x[0]+(0.703207191364)*x[1]
        s[2,0,2,1]=(-1.05211790226)*x[0]+(1.68596483036)*x[1]
        s[2,1,0,0]=(0.704109387934)*x[0]+(-0.589003921574)*x[1]
        s[2,1,0,1]=(-0.382489922028)*x[0]+(0.387647642942)*x[1]
        s[2,1,1,0]=(-0.0343434958891)*x[0]+(-0.119176404279)*x[1]
        s[2,1,1,1]=(0.567853527929)*x[0]+(1.32523261969)*x[1]
        s[2,1,2,0]=(1.03496000653)*x[0]+(0.110203591459)*x[1]
        s[2,1,2,1]=(0.560149997806)*x[0]+(-1.62893742873)*x[1]
        s[2,2,0,0]=(1.1801792312)*x[0]+(1.1912385935)*x[1]
        s[2,2,0,1]=(0.936932817634)*x[0]+(-1.49082021974)*x[1]
        s[2,2,1,0]=(-0.305582892069)*x[0]+(-0.267235698563)*x[1]
        s[2,2,1,1]=(0.107449730184)*x[0]+(-0.325267993615)*x[1]
        s[2,2,2,0]=(-1.08118986347)*x[0]+(0.144190727334)*x[1]
        s[2,2,2,1]=(1.44755060146)*x[0]+(-0.127612338817)*x[1]
        s[2,3,0,0]=(-0.666943112999)*x[0]+(-0.780402371989)*x[1]
        s[2,3,0,1]=(0.724206717771)*x[0]+(-0.523593908278)*x[1]
        s[2,3,1,0]=(1.86101197199)*x[0]+(-0.710894911302)*x[1]
        s[2,3,1,1]=(-1.74409232023)*x[0]+(0.407821816271)*x[1]
        s[2,3,2,0]=(0.646304172575)*x[0]+(0.0132628778543)*x[1]
        s[2,3,2,1]=(-0.746041438673)*x[0]+(0.516304872741)*x[1]
        s[2,4,0,0]=(0.0326900201283)*x[0]+(-0.533021156401)*x[1]
        s[2,4,0,1]=(0.088598956981)*x[0]+(0.764307970269)*x[1]
        s[2,4,1,0]=(-0.749553477756)*x[0]+(0.379646792939)*x[1]
        s[2,4,1,1]=(-0.243635321378)*x[0]+(-0.470022600261)*x[1]
        s[2,4,2,0]=(0.586398757247)*x[0]+(-0.116982897683)*x[1]
        s[2,4,2,1]=(-0.989012121879)*x[0]+(-1.25006535257)*x[1]
        s[3,0,0,0]=(1.25228306081)*x[0]+(-0.0932717035896)*x[1]
        s[3,0,0,1]=(1.5975516771)*x[0]+(-0.177733286111)*x[1]
        s[3,0,1,0]=(-0.659910070911)*x[0]+(-0.0999498944812)*x[1]
        s[3,0,1,1]=(-0.299870273662)*x[0]+(-1.22977656744)*x[1]
        s[3,0,2,0]=(-0.158211581143)*x[0]+(-0.141421380503)*x[1]
        s[3,0,2,1]=(-1.48226452788)*x[0]+(-0.731264943309)*x[1]
        s[3,1,0,0]=(-1.64589827853)*x[0]+(1.19692573998)*x[1]
        s[3,1,0,1]=(0.361329655357)*x[0]+(0.234310482171)*x[1]
        s[3,1,1,0]=(-1.14077649516)*x[0]+(0.132657084065)*x[1]
        s[3,1,1,1]=(1.41067975663)*x[0]+(0.818243117884)*x[1]
        s[3,1,2,0]=(-1.53137884342)*x[0]+(-0.0127805333093)*x[1]
        s[3,1,2,1]=(0.240455755486)*x[0]+(-1.02361753893)*x[1]
        s[3,2,0,0]=(-0.334803473072)*x[0]+(-0.807452510423)*x[1]
        s[3,2,0,1]=(0.617324536974)*x[0]+(-0.733163471774)*x[1]
        s[3,2,1,0]=(-0.634580050897)*x[0]+(0.362106278168)*x[1]
        s[3,2,1,1]=(-0.21525531818)*x[0]+(0.982902540898)*x[1]
        s[3,2,2,0]=(-0.590682803195)*x[0]+(-1.27537880632)*x[1]
        s[3,2,2,1]=(0.142578313438)*x[0]+(0.212341687291)*x[1]
        s[3,3,0,0]=(-0.758345317216)*x[0]+(-0.971914631359)*x[1]
        s[3,3,0,1]=(0.569463909234)*x[0]+(0.498848917338)*x[1]
        s[3,3,1,0]=(-0.771456575632)*x[0]+(-0.130456072944)*x[1]
        s[3,3,1,1]=(0.811799269245)*x[0]+(-0.829647203396)*x[1]
        s[3,3,2,0]=(-0.814979427693)*x[0]+(-0.0416092796813)*x[1]
        s[3,3,2,1]=(0.933434189127)*x[0]+(0.60794704936)*x[1]
        s[3,4,0,0]=(0.160280566478)*x[0]+(-0.0141616040181)*x[1]
        s[3,4,0,1]=(-1.34835766268)*x[0]+(0.178951225861)*x[1]
        s[3,4,1,0]=(1.67632814658)*x[0]+(1.31316096483)*x[1]
        s[3,4,1,1]=(-0.603403275376)*x[0]+(0.0607753526431)*x[1]
        s[3,4,2,0]=(-0.178440295473)*x[0]+(0.318102694371)*x[1]
        s[3,4,2,1]=(0.781285160749)*x[0]+(0.625651915981)*x[1]
        ref[0,0,0,0]=(0.609727033659)*x_ref[0]+(-0.901858940633)*x_ref[1]
        ref[0,0,0,1]=(-0.221259461243)*x_ref[0]+(0.351136340908)*x_ref[1]
        ref[0,0,1,0]=(0.497173734282)*x_ref[0]+(0.899142906899)*x_ref[1]
        ref[0,0,1,1]=(-1.55206649551)*x_ref[0]+(-0.680257384044)*x_ref[1]
        ref[0,0,2,0]=(-0.932033638033)*x_ref[0]+(-0.46503758126)*x_ref[1]
        ref[0,0,2,1]=(-0.0372650123525)*x_ref[0]+(0.0173714029098)*x_ref[1]
        ref[0,1,0,0]=(0.855963098465)*x_ref[0]+(0.760176320898)*x_ref[1]
        ref[0,1,0,1]=(1.37546120327)*x_ref[0]+(1.72916158331)*x_ref[1]
        ref[0,1,1,0]=(0.25469861613)*x_ref[0]+(0.71445716033)*x_ref[1]
        ref[0,1,1,1]=(0.198455659765)*x_ref[0]+(0.0410276829495)*x_ref[1]
        ref[0,1,2,0]=(-0.628080427667)*x_ref[0]+(-0.0197206025984)*x_ref[1]
        ref[0,1,2,1]=(1.26759029895)*x_ref[0]+(1.15914953253)*x_ref[1]
        ref[0,2,0,0]=(-1.43203053441)*x_ref[0]+(0.366301170069)*x_ref[1]
        ref[0,2,0,1]=(0.420527802229)*x_ref[0]+(-0.709209747415)*x_ref[1]
        ref[0,2,1,0]=(0.399042264829)*x_ref[0]+(0.0453132754731)*x_ref[1]
        ref[0,2,1,1]=(1.2074346062)*x_ref[0]+(-1.14163410992)*x_ref[1]
        ref[0,2,2,0]=(-0.00270984158569)*x_ref[0]+(-1.63216200182)*x_ref[1]
        ref[0,2,2,1]=(0.174276088847)*x_ref[0]+(-1.04789472231)*x_ref[1]
        ref[0,3,0,0]=(-0.47704729036)*x_ref[0]+(-1.2550637204)*x_ref[1]
        ref[0,3,0,1]=(0.348356873191)*x_ref[0]+(-0.796668578265)*x_ref[1]
        ref[0,3,1,0]=(-0.52724725097)*x_ref[0]+(1.26163123592)*x_ref[1]
        ref[0,3,1,1]=(-1.52681799261)*x_ref[0]+(0.936041931735)*x_ref[1]
        ref[0,3,2,0]=(1.33689673219)*x_ref[0]+(-1.34901214018)*x_ref[1]
        ref[0,3,2,1]=(1.57651286047)*x_ref[0]+(0.245413762864)*x_ref[1]
        ref[0,4,0,0]=(0.194500181498)*x_ref[0]+(-0.0231951844441)*x_ref[1]
        ref[0,4,0,1]=(-1.49372076697)*x_ref[0]+(-0.111369678181)*x_ref[1]
        ref[0,4,1,0]=(-0.74121646836)*x_ref[0]+(-0.817740156307)*x_ref[1]
        ref[0,4,1,1]=(-0.233643397123)*x_ref[0]+(0.672715068796)*x_ref[1]
        ref[0,4,2,0]=(1.05850947579)*x_ref[0]+(0.386484386191)*x_ref[1]
        ref[0,4,2,1]=(1.45105922963)*x_ref[0]+(-0.367458625996)*x_ref[1]
        ref[1,0,0,0]=(-0.391803788669)*x_ref[0]+(0.797592300662)*x_ref[1]
        ref[1,0,0,1]=(-0.205299072883)*x_ref[0]+(0.677427935558)*x_ref[1]
        ref[1,0,1,0]=(1.56225329031)*x_ref[0]+(1.17468092515)*x_ref[1]
        ref[1,0,1,1]=(1.4586005658)*x_ref[0]+(1.36481062943)*x_ref[1]
        ref[1,0,2,0]=(-1.53281881413)*x_ref[0]+(0.207028833974)*x_ref[1]
        ref[1,0,2,1]=(0.0469239691648)*x_ref[0]+(1.03547812705)*x_ref[1]
        ref[1,1,0,0]=(1.39614335897)*x_ref[0]+(0.82046489695)*x_ref[1]
        ref[1,1,0,1]=(-0.775462741637)*x_ref[0]+(-0.275081761646)*x_ref[1]
        ref[1,1,1,0]=(0.173353028608)*x_ref[0]+(-1.05620431816)*x_ref[1]
        ref[1,1,1,1]=(-0.464192870247)*x_ref[0]+(0.212204784206)*x_ref[1]
        ref[1,1,2,0]=(-0.383813531866)*x_ref[0]+(-0.0773759374234)*x_ref[1]
        ref[1,1,2,1]=(-1.07811385759)*x_ref[0]+(-0.457232330819)*x_ref[1]
        ref[1,2,0,0]=(-0.295869147576)*x_ref[0]+(0.166843915631)*x_ref[1]
        ref[1,2,0,1]=(0.571421034325)*x_ref[0]+(1.46722583245)*x_ref[1]
        ref[1,2,1,0]=(-0.34494944257)*x_ref[0]+(-0.286867052491)*x_ref[1]
        ref[1,2,1,1]=(1.43128885761)*x_ref[0]+(0.21277887035)*x_ref[1]
        ref[1,2,2,0]=(-1.19397187873)*x_ref[0]+(0.126397665227)*x_ref[1]
        ref[1,2,2,1]=(0.171368751954)*x_ref[0]+(-1.12100254625)*x_ref[1]
        ref[1,3,0,0]=(0.0975333077604)*x_ref[0]+(0.583524660188)*x_ref[1]
        ref[1,3,0,1]=(0.0271450955458)*x_ref[0]+(0.368978332867)*x_ref[1]
        ref[1,3,1,0]=(1.22806968486)*x_ref[0]+(-0.569932748468)*x_ref[1]
        ref[1,3,1,1]=(-0.691989839521)*x_ref[0]+(-0.829822182922)*x_ref[1]
        ref[1,3,2,0]=(0.743776534098)*x_ref[0]+(1.3668937127)*x_ref[1]
        ref[1,3,2,1]=(0.192206575204)*x_ref[0]+(0.96576702824)*x_ref[1]
        ref[1,4,0,0]=(0.149381460079)*x_ref[0]+(-0.987875538036)*x_ref[1]
        ref[1,4,0,1]=(-0.706855836683)*x_ref[0]+(0.0410884211631)*x_ref[1]
        ref[1,4,1,0]=(-0.222650274)*x_ref[0]+(0.59589242048)*x_ref[1]
        ref[1,4,1,1]=(0.541489186743)*x_ref[0]+(-1.22543996074)*x_ref[1]
        ref[1,4,2,0]=(0.630234824652)*x_ref[0]+(0.508436575179)*x_ref[1]
        ref[1,4,2,1]=(0.990062359294)*x_ref[0]+(-0.639884693407)*x_ref[1]
        ref[2,0,0,0]=(0.378887461733)*x_ref[0]+(1.71999809344)*x_ref[1]
        ref[2,0,0,1]=(0.182412683544)*x_ref[0]+(-0.401183243877)*x_ref[1]
        ref[2,0,1,0]=(0.325775290396)*x_ref[0]+(-0.656088786126)*x_ref[1]
        ref[2,0,1,1]=(0.316355229742)*x_ref[0]+(1.54087545195)*x_ref[1]
        ref[2,0,2,0]=(1.14203881183)*x_ref[0]+(0.703207191364)*x_ref[1]
        ref[2,0,2,1]=(-1.05211790226)*x_ref[0]+(1.68596483036)*x_ref[1]
        ref[2,1,0,0]=(0.704109387934)*x_ref[0]+(-0.589003921574)*x_ref[1]
        ref[2,1,0,1]=(-0.382489922028)*x_ref[0]+(0.387647642942)*x_ref[1]
        ref[2,1,1,0]=(-0.0343434958891)*x_ref[0]+(-0.119176404279)*x_ref[1]
        ref[2,1,1,1]=(0.567853527929)*x_ref[0]+(1.32523261969)*x_ref[1]
        ref[2,1,2,0]=(1.03496000653)*x_ref[0]+(0.110203591459)*x_ref[1]
        ref[2,1,2,1]=(0.560149997806)*x_ref[0]+(-1.62893742873)*x_ref[1]
        ref[2,2,0,0]=(1.1801792312)*x_ref[0]+(1.1912385935)*x_ref[1]
        ref[2,2,0,1]=(0.936932817634)*x_ref[0]+(-1.49082021974)*x_ref[1]
        ref[2,2,1,0]=(-0.305582892069)*x_ref[0]+(-0.267235698563)*x_ref[1]
        ref[2,2,1,1]=(0.107449730184)*x_ref[0]+(-0.325267993615)*x_ref[1]
        ref[2,2,2,0]=(-1.08118986347)*x_ref[0]+(0.144190727334)*x_ref[1]
        ref[2,2,2,1]=(1.44755060146)*x_ref[0]+(-0.127612338817)*x_ref[1]
        ref[2,3,0,0]=(-0.666943112999)*x_ref[0]+(-0.780402371989)*x_ref[1]
        ref[2,3,0,1]=(0.724206717771)*x_ref[0]+(-0.523593908278)*x_ref[1]
        ref[2,3,1,0]=(1.86101197199)*x_ref[0]+(-0.710894911302)*x_ref[1]
        ref[2,3,1,1]=(-1.74409232023)*x_ref[0]+(0.407821816271)*x_ref[1]
        ref[2,3,2,0]=(0.646304172575)*x_ref[0]+(0.0132628778543)*x_ref[1]
        ref[2,3,2,1]=(-0.746041438673)*x_ref[0]+(0.516304872741)*x_ref[1]
        ref[2,4,0,0]=(0.0326900201283)*x_ref[0]+(-0.533021156401)*x_ref[1]
        ref[2,4,0,1]=(0.088598956981)*x_ref[0]+(0.764307970269)*x_ref[1]
        ref[2,4,1,0]=(-0.749553477756)*x_ref[0]+(0.379646792939)*x_ref[1]
        ref[2,4,1,1]=(-0.243635321378)*x_ref[0]+(-0.470022600261)*x_ref[1]
        ref[2,4,2,0]=(0.586398757247)*x_ref[0]+(-0.116982897683)*x_ref[1]
        ref[2,4,2,1]=(-0.989012121879)*x_ref[0]+(-1.25006535257)*x_ref[1]
        ref[3,0,0,0]=(1.25228306081)*x_ref[0]+(-0.0932717035896)*x_ref[1]
        ref[3,0,0,1]=(1.5975516771)*x_ref[0]+(-0.177733286111)*x_ref[1]
        ref[3,0,1,0]=(-0.659910070911)*x_ref[0]+(-0.0999498944812)*x_ref[1]
        ref[3,0,1,1]=(-0.299870273662)*x_ref[0]+(-1.22977656744)*x_ref[1]
        ref[3,0,2,0]=(-0.158211581143)*x_ref[0]+(-0.141421380503)*x_ref[1]
        ref[3,0,2,1]=(-1.48226452788)*x_ref[0]+(-0.731264943309)*x_ref[1]
        ref[3,1,0,0]=(-1.64589827853)*x_ref[0]+(1.19692573998)*x_ref[1]
        ref[3,1,0,1]=(0.361329655357)*x_ref[0]+(0.234310482171)*x_ref[1]
        ref[3,1,1,0]=(-1.14077649516)*x_ref[0]+(0.132657084065)*x_ref[1]
        ref[3,1,1,1]=(1.41067975663)*x_ref[0]+(0.818243117884)*x_ref[1]
        ref[3,1,2,0]=(-1.53137884342)*x_ref[0]+(-0.0127805333093)*x_ref[1]
        ref[3,1,2,1]=(0.240455755486)*x_ref[0]+(-1.02361753893)*x_ref[1]
        ref[3,2,0,0]=(-0.334803473072)*x_ref[0]+(-0.807452510423)*x_ref[1]
        ref[3,2,0,1]=(0.617324536974)*x_ref[0]+(-0.733163471774)*x_ref[1]
        ref[3,2,1,0]=(-0.634580050897)*x_ref[0]+(0.362106278168)*x_ref[1]
        ref[3,2,1,1]=(-0.21525531818)*x_ref[0]+(0.982902540898)*x_ref[1]
        ref[3,2,2,0]=(-0.590682803195)*x_ref[0]+(-1.27537880632)*x_ref[1]
        ref[3,2,2,1]=(0.142578313438)*x_ref[0]+(0.212341687291)*x_ref[1]
        ref[3,3,0,0]=(-0.758345317216)*x_ref[0]+(-0.971914631359)*x_ref[1]
        ref[3,3,0,1]=(0.569463909234)*x_ref[0]+(0.498848917338)*x_ref[1]
        ref[3,3,1,0]=(-0.771456575632)*x_ref[0]+(-0.130456072944)*x_ref[1]
        ref[3,3,1,1]=(0.811799269245)*x_ref[0]+(-0.829647203396)*x_ref[1]
        ref[3,3,2,0]=(-0.814979427693)*x_ref[0]+(-0.0416092796813)*x_ref[1]
        ref[3,3,2,1]=(0.933434189127)*x_ref[0]+(0.60794704936)*x_ref[1]
        ref[3,4,0,0]=(0.160280566478)*x_ref[0]+(-0.0141616040181)*x_ref[1]
        ref[3,4,0,1]=(-1.34835766268)*x_ref[0]+(0.178951225861)*x_ref[1]
        ref[3,4,1,0]=(1.67632814658)*x_ref[0]+(1.31316096483)*x_ref[1]
        ref[3,4,1,1]=(-0.603403275376)*x_ref[0]+(0.0607753526431)*x_ref[1]
        ref[3,4,2,0]=(-0.178440295473)*x_ref[0]+(0.318102694371)*x_ref[1]
        ref[3,4,2,1]=(0.781285160749)*x_ref[0]+(0.625651915981)*x_ref[1]
      else:
        s[0,0,0,0]=(0.847845815599)*x[0]+(-0.610234211476)*x[1]+(-0.627625510082)*x[2]
        s[0,0,0,1]=(0.0126184651041)*x[0]+(-0.0651294063564)*x[1]+(-0.274615935201)*x[2]
        s[0,0,1,0]=(-0.545925417969)*x[0]+(0.421929302201)*x[1]+(-0.910418625383)*x[2]
        s[0,0,1,1]=(1.58754252937)*x[0]+(0.110111591047)*x[1]+(-1.64780563736)*x[2]
        s[0,0,2,0]=(-0.510331643992)*x[0]+(0.827685326542)*x[1]+(1.31447962917)*x[2]
        s[0,0,2,1]=(-1.29575211142)*x[0]+(-0.318096803401)*x[1]+(-0.274660613777)*x[2]
        s[0,1,0,0]=(0.77444944835)*x[0]+(-1.0893411122)*x[1]+(0.331702744143)*x[2]
        s[0,1,0,1]=(0.624693916432)*x[0]+(-0.306552086758)*x[1]+(-0.845524417807)*x[2]
        s[0,1,1,0]=(0.0219492668609)*x[0]+(-1.13712865204)*x[1]+(-0.193051406533)*x[2]
        s[0,1,1,1]=(-1.32696889246)*x[0]+(1.63857177904)*x[1]+(0.194095476936)*x[2]
        s[0,1,2,0]=(1.19169978642)*x[0]+(-1.88274910505)*x[1]+(0.321438561021)*x[2]
        s[0,1,2,1]=(-0.765748081809)*x[0]+(-0.163422665563)*x[1]+(0.256721885141)*x[2]
        s[0,2,0,0]=(-0.863281319991)*x[0]+(-0.021828308413)*x[1]+(0.868312889989)*x[2]
        s[0,2,0,1]=(-0.57479341478)*x[0]+(0.404861121887)*x[1]+(-0.274037007699)*x[2]
        s[0,2,1,0]=(-0.529171478131)*x[0]+(0.836001872018)*x[1]+(-1.12226189724)*x[2]
        s[0,2,1,1]=(-0.821929160411)*x[0]+(0.0466762602824)*x[1]+(-0.0514331500765)*x[2]
        s[0,2,2,0]=(-0.605940618544)*x[0]+(-1.54379919132)*x[1]+(1.63243021269)*x[2]
        s[0,2,2,1]=(-1.49695860655)*x[0]+(-0.601345463475)*x[1]+(-0.753443592162)*x[2]
        s[0,3,0,0]=(1.33514081849)*x[0]+(0.955849232345)*x[1]+(-0.872221354599)*x[2]
        s[0,3,0,1]=(0.939731890901)*x[0]+(-1.16116012822)*x[1]+(-0.601602002743)*x[2]
        s[0,3,1,0]=(-0.675545889094)*x[0]+(0.736755090077)*x[1]+(-0.437370345974)*x[2]
        s[0,3,1,1]=(-1.17899966213)*x[0]+(0.422125349488)*x[1]+(-0.815649229407)*x[2]
        s[0,3,2,0]=(0.0773790438387)*x[0]+(0.148378160687)*x[1]+(-1.0156040466)*x[2]
        s[0,3,2,1]=(0.519422925349)*x[0]+(0.189993892084)*x[1]+(1.59654399673)*x[2]
        s[0,4,0,0]=(-0.0593352727817)*x[0]+(0.0865384232804)*x[1]+(-0.316122229956)*x[2]
        s[0,4,0,1]=(0.935220504817)*x[0]+(0.868143343952)*x[1]+(-0.470286174039)*x[2]
        s[0,4,1,0]=(0.763506086212)*x[0]+(-0.0756795330833)*x[1]+(1.06812672934)*x[2]
        s[0,4,1,1]=(-0.446580256351)*x[0]+(0.653986880233)*x[1]+(0.634364696873)*x[2]
        s[0,4,2,0]=(-0.72871248403)*x[0]+(0.0839150289011)*x[1]+(0.178899292496)*x[2]
        s[0,4,2,1]=(1.30808288445)*x[0]+(0.548931211195)*x[1]+(-0.971585668771)*x[2]
        s[1,0,0,0]=(-0.737171903787)*x[0]+(-0.137110160891)*x[1]+(1.24817883553)*x[2]
        s[1,0,0,1]=(0.589824248199)*x[0]+(-0.191688810075)*x[1]+(1.33820051734)*x[2]
        s[1,0,1,0]=(-0.598063103136)*x[0]+(0.347705502684)*x[1]+(0.490747627936)*x[2]
        s[1,0,1,1]=(0.287829427303)*x[0]+(-0.853875346674)*x[1]+(-0.227722773909)*x[2]
        s[1,0,2,0]=(-1.22183236343)*x[0]+(-0.162406713796)*x[1]+(-0.436433884201)*x[2]
        s[1,0,2,1]=(0.929372688401)*x[0]+(0.469931776805)*x[1]+(0.739110972137)*x[2]
        s[1,1,0,0]=(0.391824586)*x[0]+(-0.551498198462)*x[1]+(-0.588935046796)*x[2]
        s[1,1,0,1]=(0.849805745827)*x[0]+(0.557121315313)*x[1]+(1.80284061615)*x[2]
        s[1,1,1,0]=(-1.62270432051)*x[0]+(-1.55943636923)*x[1]+(0.629204169009)*x[2]
        s[1,1,1,1]=(-0.191271118514)*x[0]+(-0.454718665581)*x[1]+(0.405403171722)*x[2]
        s[1,1,2,0]=(0.461750123507)*x[0]+(0.537594040674)*x[1]+(-0.500046587877)*x[2]
        s[1,1,2,1]=(1.07800193455)*x[0]+(0.0757746249706)*x[1]+(0.358223454759)*x[2]
        s[1,2,0,0]=(-1.31360669142)*x[0]+(0.908413426737)*x[1]+(0.986904165818)*x[2]
        s[1,2,0,1]=(-0.250344247716)*x[0]+(0.596746427887)*x[1]+(0.846846599186)*x[2]
        s[1,2,1,0]=(1.65761753214)*x[0]+(0.0969407749043)*x[1]+(0.197913502281)*x[2]
        s[1,2,1,1]=(0.808454446977)*x[0]+(-0.107176416129)*x[1]+(-0.115562549181)*x[2]
        s[1,2,2,0]=(-0.410108442793)*x[0]+(0.250496842214)*x[1]+(0.403806076052)*x[2]
        s[1,2,2,1]=(1.58110488679)*x[0]+(-0.171583112102)*x[1]+(0.441813895222)*x[2]
        s[1,3,0,0]=(-0.519973841295)*x[0]+(0.14862500493)*x[1]+(-0.0257159960311)*x[2]
        s[1,3,0,1]=(-0.710423889723)*x[0]+(0.0941990549739)*x[1]+(-0.147877062943)*x[2]
        s[1,3,1,0]=(-0.19917762038)*x[0]+(-1.06873791711)*x[1]+(1.37688275432)*x[2]
        s[1,3,1,1]=(-0.570477493665)*x[0]+(0.0705814270652)*x[1]+(-1.10186231057)*x[2]
        s[1,3,2,0]=(-1.04891017002)*x[0]+(-0.136784107636)*x[1]+(-1.65953016693)*x[2]
        s[1,3,2,1]=(-0.674805716398)*x[0]+(-0.211803508128)*x[1]+(-0.116463503974)*x[2]
        s[1,4,0,0]=(0.269217220297)*x[0]+(0.810722194772)*x[1]+(1.60977315112)*x[2]
        s[1,4,0,1]=(-0.0213286766746)*x[0]+(1.0974274553)*x[1]+(0.275633207025)*x[2]
        s[1,4,1,0]=(-0.601230245409)*x[0]+(-0.756418955933)*x[1]+(-0.46194303915)*x[2]
        s[1,4,1,1]=(-0.0907667209143)*x[0]+(-1.86623564673)*x[1]+(0.552729474701)*x[2]
        s[1,4,2,0]=(-1.09694541489)*x[0]+(-0.496193881555)*x[1]+(-0.0126133249168)*x[2]
        s[1,4,2,1]=(-0.648959246633)*x[0]+(-0.60655325639)*x[1]+(0.438258051102)*x[2]
        s[2,0,0,0]=(-0.100512547019)*x[0]+(-1.37291269079)*x[1]+(1.63084319876)*x[2]
        s[2,0,0,1]=(-0.50510326916)*x[0]+(0.308799737351)*x[1]+(1.63506761477)*x[2]
        s[2,0,1,0]=(0.708408519723)*x[0]+(-0.0759833841695)*x[1]+(-0.980974095374)*x[2]
        s[2,0,1,1]=(-1.59727324143)*x[0]+(0.995250195962)*x[1]+(0.169527316083)*x[2]
        s[2,0,2,0]=(0.766935817054)*x[0]+(0.455564399031)*x[1]+(0.605772951782)*x[2]
        s[2,0,2,1]=(-0.820755214894)*x[0]+(-0.43359371038)*x[1]+(-0.0942398466964)*x[2]
        s[2,1,0,0]=(0.390476682801)*x[0]+(1.11627447827)*x[1]+(0.670843405466)*x[2]
        s[2,1,0,1]=(-0.433594870483)*x[0]+(-0.940710004402)*x[1]+(1.10756041487)*x[2]
        s[2,1,1,0]=(-1.21880018868)*x[0]+(0.380422307344)*x[1]+(-0.864017731789)*x[2]
        s[2,1,1,1]=(-0.194168346974)*x[0]+(0.324183983028)*x[1]+(0.741456579501)*x[2]
        s[2,1,2,0]=(-0.134211025912)*x[0]+(0.497311041207)*x[1]+(0.33014196376)*x[2]
        s[2,1,2,1]=(-1.3501981937)*x[0]+(1.67601514859)*x[1]+(-1.60084725544)*x[2]
        s[2,2,0,0]=(-0.8367457236)*x[0]+(-1.15628524562)*x[1]+(0.342983125417)*x[2]
        s[2,2,0,1]=(0.956733778283)*x[0]+(-0.404880336577)*x[1]+(-0.350927185105)*x[2]
        s[2,2,1,0]=(0.854842449281)*x[0]+(-0.185001304765)*x[1]+(1.15245157168)*x[2]
        s[2,2,1,1]=(0.00584831242623)*x[0]+(-0.562950548984)*x[1]+(0.785766513431)*x[2]
        s[2,2,2,0]=(0.272759945164)*x[0]+(-0.568207376307)*x[1]+(-0.155997758854)*x[2]
        s[2,2,2,1]=(-0.105324133175)*x[0]+(1.11807221711)*x[1]+(0.122751975298)*x[2]
        s[2,3,0,0]=(0.685932400265)*x[0]+(-0.666847052302)*x[1]+(0.596321369477)*x[2]
        s[2,3,0,1]=(0.335067153518)*x[0]+(0.268453607431)*x[1]+(0.247881738828)*x[2]
        s[2,3,1,0]=(-1.89084909179)*x[0]+(-0.529751828744)*x[1]+(-0.959358202828)*x[2]
        s[2,3,1,1]=(1.06725689989)*x[0]+(-0.709878517637)*x[1]+(0.968886366535)*x[2]
        s[2,3,2,0]=(-1.63466018652)*x[0]+(0.517481248484)*x[1]+(0.109232845638)*x[2]
        s[2,3,2,1]=(0.0716425266603)*x[0]+(-0.58040972115)*x[1]+(0.346948738894)*x[2]
        s[2,4,0,0]=(0.193599854213)*x[0]+(-1.52744079327)*x[1]+(-0.0377038338183)*x[2]
        s[2,4,0,1]=(-0.0296855161658)*x[0]+(0.266976994363)*x[1]+(-0.832771979863)*x[2]
        s[2,4,1,0]=(-0.885217283694)*x[0]+(0.657425725373)*x[1]+(0.385584782531)*x[2]
        s[2,4,1,1]=(-0.249912733602)*x[0]+(0.156704487203)*x[1]+(0.000393408500501)*x[2]
        s[2,4,2,0]=(-0.083303562971)*x[0]+(0.748009334975)*x[1]+(-0.411323946112)*x[2]
        s[2,4,2,1]=(0.143782087293)*x[0]+(-1.86497054401)*x[1]+(-0.398161751889)*x[2]
        s[3,0,0,0]=(-0.531041035506)*x[0]+(0.998737023878)*x[1]+(-0.719326852701)*x[2]
        s[3,0,0,1]=(-1.10320774278)*x[0]+(-1.27795876669)*x[1]+(0.57451462957)*x[2]
        s[3,0,1,0]=(-0.319275578502)*x[0]+(-1.51721910203)*x[1]+(-0.677470578443)*x[2]
        s[3,0,1,1]=(0.7584807917)*x[0]+(0.0523998746885)*x[1]+(0.502182688228)*x[2]
        s[3,0,2,0]=(-0.905809892125)*x[0]+(0.479751975382)*x[1]+(-0.0778539387977)*x[2]
        s[3,0,2,1]=(-0.724891388357)*x[0]+(0.941493554841)*x[1]+(-1.51514565838)*x[2]
        s[3,1,0,0]=(0.815118793475)*x[0]+(0.0503368907581)*x[1]+(-0.502091253213)*x[2]
        s[3,1,0,1]=(-1.50859117948)*x[0]+(0.31279517213)*x[1]+(-0.339295035594)*x[2]
        s[3,1,1,0]=(0.246825549192)*x[0]+(0.687251908372)*x[1]+(0.133971781776)*x[2]
        s[3,1,1,1]=(0.470105585328)*x[0]+(0.382971101979)*x[1]+(-0.469397699886)*x[2]
        s[3,1,2,0]=(-1.45842815542)*x[0]+(-1.00979255513)*x[1]+(1.45386034482)*x[2]
        s[3,1,2,1]=(0.172130164577)*x[0]+(1.25034739991)*x[1]+(-1.15953423359)*x[2]
        s[3,2,0,0]=(-0.58261393055)*x[0]+(0.730381032663)*x[1]+(1.29377076249)*x[2]
        s[3,2,0,1]=(0.696715362351)*x[0]+(-1.37598351546)*x[1]+(-1.26545085967)*x[2]
        s[3,2,1,0]=(-0.0511282610422)*x[0]+(0.58041679118)*x[1]+(-0.0810357412151)*x[2]
        s[3,2,1,1]=(-0.750057228803)*x[0]+(-0.750703125074)*x[1]+(-1.21748956536)*x[2]
        s[3,2,2,0]=(-0.455746666635)*x[0]+(-0.720020627516)*x[1]+(-0.0911281889971)*x[2]
        s[3,2,2,1]=(-0.0681894163236)*x[0]+(0.101592029278)*x[1]+(1.10954864406)*x[2]
        s[3,3,0,0]=(0.433898443791)*x[0]+(0.901180030496)*x[1]+(0.795641263874)*x[2]
        s[3,3,0,1]=(-0.213306810738)*x[0]+(-0.852539239286)*x[1]+(-1.25905416259)*x[2]
        s[3,3,1,0]=(-0.668307384473)*x[0]+(-1.37753879862)*x[1]+(-0.593731800903)*x[2]
        s[3,3,1,1]=(0.815464294514)*x[0]+(0.835389640785)*x[1]+(0.933029468258)*x[2]
        s[3,3,2,0]=(0.565194274444)*x[0]+(0.584119707001)*x[1]+(-0.53812189952)*x[2]
        s[3,3,2,1]=(-0.903604209314)*x[0]+(0.370887007483)*x[1]+(-0.548135915608)*x[2]
        s[3,4,0,0]=(-0.819290139492)*x[0]+(-0.256800498012)*x[1]+(0.350363336935)*x[2]
        s[3,4,0,1]=(0.863170182723)*x[0]+(0.0101766489337)*x[1]+(1.34009635664)*x[2]
        s[3,4,1,0]=(1.03350734136)*x[0]+(1.37672500233)*x[1]+(1.35134434663)*x[2]
        s[3,4,1,1]=(1.4883144839)*x[0]+(0.410688244039)*x[1]+(0.759267998436)*x[2]
        s[3,4,2,0]=(0.373190738096)*x[0]+(1.22136515279)*x[1]+(-0.0365030690962)*x[2]
        s[3,4,2,1]=(-0.607437402737)*x[0]+(0.750130654364)*x[1]+(0.084778884789)*x[2]
        ref[0,0,0,0]=(0.847845815599)*x_ref[0]+(-0.610234211476)*x_ref[1]+(-0.627625510082)*x_ref[2]
        ref[0,0,0,1]=(0.0126184651041)*x_ref[0]+(-0.0651294063564)*x_ref[1]+(-0.274615935201)*x_ref[2]
        ref[0,0,1,0]=(-0.545925417969)*x_ref[0]+(0.421929302201)*x_ref[1]+(-0.910418625383)*x_ref[2]
        ref[0,0,1,1]=(1.58754252937)*x_ref[0]+(0.110111591047)*x_ref[1]+(-1.64780563736)*x_ref[2]
        ref[0,0,2,0]=(-0.510331643992)*x_ref[0]+(0.827685326542)*x_ref[1]+(1.31447962917)*x_ref[2]
        ref[0,0,2,1]=(-1.29575211142)*x_ref[0]+(-0.318096803401)*x_ref[1]+(-0.274660613777)*x_ref[2]
        ref[0,1,0,0]=(0.77444944835)*x_ref[0]+(-1.0893411122)*x_ref[1]+(0.331702744143)*x_ref[2]
        ref[0,1,0,1]=(0.624693916432)*x_ref[0]+(-0.306552086758)*x_ref[1]+(-0.845524417807)*x_ref[2]
        ref[0,1,1,0]=(0.0219492668609)*x_ref[0]+(-1.13712865204)*x_ref[1]+(-0.193051406533)*x_ref[2]
        ref[0,1,1,1]=(-1.32696889246)*x_ref[0]+(1.63857177904)*x_ref[1]+(0.194095476936)*x_ref[2]
        ref[0,1,2,0]=(1.19169978642)*x_ref[0]+(-1.88274910505)*x_ref[1]+(0.321438561021)*x_ref[2]
        ref[0,1,2,1]=(-0.765748081809)*x_ref[0]+(-0.163422665563)*x_ref[1]+(0.256721885141)*x_ref[2]
        ref[0,2,0,0]=(-0.863281319991)*x_ref[0]+(-0.021828308413)*x_ref[1]+(0.868312889989)*x_ref[2]
        ref[0,2,0,1]=(-0.57479341478)*x_ref[0]+(0.404861121887)*x_ref[1]+(-0.274037007699)*x_ref[2]
        ref[0,2,1,0]=(-0.529171478131)*x_ref[0]+(0.836001872018)*x_ref[1]+(-1.12226189724)*x_ref[2]
        ref[0,2,1,1]=(-0.821929160411)*x_ref[0]+(0.0466762602824)*x_ref[1]+(-0.0514331500765)*x_ref[2]
        ref[0,2,2,0]=(-0.605940618544)*x_ref[0]+(-1.54379919132)*x_ref[1]+(1.63243021269)*x_ref[2]
        ref[0,2,2,1]=(-1.49695860655)*x_ref[0]+(-0.601345463475)*x_ref[1]+(-0.753443592162)*x_ref[2]
        ref[0,3,0,0]=(1.33514081849)*x_ref[0]+(0.955849232345)*x_ref[1]+(-0.872221354599)*x_ref[2]
        ref[0,3,0,1]=(0.939731890901)*x_ref[0]+(-1.16116012822)*x_ref[1]+(-0.601602002743)*x_ref[2]
        ref[0,3,1,0]=(-0.675545889094)*x_ref[0]+(0.736755090077)*x_ref[1]+(-0.437370345974)*x_ref[2]
        ref[0,3,1,1]=(-1.17899966213)*x_ref[0]+(0.422125349488)*x_ref[1]+(-0.815649229407)*x_ref[2]
        ref[0,3,2,0]=(0.0773790438387)*x_ref[0]+(0.148378160687)*x_ref[1]+(-1.0156040466)*x_ref[2]
        ref[0,3,2,1]=(0.519422925349)*x_ref[0]+(0.189993892084)*x_ref[1]+(1.59654399673)*x_ref[2]
        ref[0,4,0,0]=(-0.0593352727817)*x_ref[0]+(0.0865384232804)*x_ref[1]+(-0.316122229956)*x_ref[2]
        ref[0,4,0,1]=(0.935220504817)*x_ref[0]+(0.868143343952)*x_ref[1]+(-0.470286174039)*x_ref[2]
        ref[0,4,1,0]=(0.763506086212)*x_ref[0]+(-0.0756795330833)*x_ref[1]+(1.06812672934)*x_ref[2]
        ref[0,4,1,1]=(-0.446580256351)*x_ref[0]+(0.653986880233)*x_ref[1]+(0.634364696873)*x_ref[2]
        ref[0,4,2,0]=(-0.72871248403)*x_ref[0]+(0.0839150289011)*x_ref[1]+(0.178899292496)*x_ref[2]
        ref[0,4,2,1]=(1.30808288445)*x_ref[0]+(0.548931211195)*x_ref[1]+(-0.971585668771)*x_ref[2]
        ref[1,0,0,0]=(-0.737171903787)*x_ref[0]+(-0.137110160891)*x_ref[1]+(1.24817883553)*x_ref[2]
        ref[1,0,0,1]=(0.589824248199)*x_ref[0]+(-0.191688810075)*x_ref[1]+(1.33820051734)*x_ref[2]
        ref[1,0,1,0]=(-0.598063103136)*x_ref[0]+(0.347705502684)*x_ref[1]+(0.490747627936)*x_ref[2]
        ref[1,0,1,1]=(0.287829427303)*x_ref[0]+(-0.853875346674)*x_ref[1]+(-0.227722773909)*x_ref[2]
        ref[1,0,2,0]=(-1.22183236343)*x_ref[0]+(-0.162406713796)*x_ref[1]+(-0.436433884201)*x_ref[2]
        ref[1,0,2,1]=(0.929372688401)*x_ref[0]+(0.469931776805)*x_ref[1]+(0.739110972137)*x_ref[2]
        ref[1,1,0,0]=(0.391824586)*x_ref[0]+(-0.551498198462)*x_ref[1]+(-0.588935046796)*x_ref[2]
        ref[1,1,0,1]=(0.849805745827)*x_ref[0]+(0.557121315313)*x_ref[1]+(1.80284061615)*x_ref[2]
        ref[1,1,1,0]=(-1.62270432051)*x_ref[0]+(-1.55943636923)*x_ref[1]+(0.629204169009)*x_ref[2]
        ref[1,1,1,1]=(-0.191271118514)*x_ref[0]+(-0.454718665581)*x_ref[1]+(0.405403171722)*x_ref[2]
        ref[1,1,2,0]=(0.461750123507)*x_ref[0]+(0.537594040674)*x_ref[1]+(-0.500046587877)*x_ref[2]
        ref[1,1,2,1]=(1.07800193455)*x_ref[0]+(0.0757746249706)*x_ref[1]+(0.358223454759)*x_ref[2]
        ref[1,2,0,0]=(-1.31360669142)*x_ref[0]+(0.908413426737)*x_ref[1]+(0.986904165818)*x_ref[2]
        ref[1,2,0,1]=(-0.250344247716)*x_ref[0]+(0.596746427887)*x_ref[1]+(0.846846599186)*x_ref[2]
        ref[1,2,1,0]=(1.65761753214)*x_ref[0]+(0.0969407749043)*x_ref[1]+(0.197913502281)*x_ref[2]
        ref[1,2,1,1]=(0.808454446977)*x_ref[0]+(-0.107176416129)*x_ref[1]+(-0.115562549181)*x_ref[2]
        ref[1,2,2,0]=(-0.410108442793)*x_ref[0]+(0.250496842214)*x_ref[1]+(0.403806076052)*x_ref[2]
        ref[1,2,2,1]=(1.58110488679)*x_ref[0]+(-0.171583112102)*x_ref[1]+(0.441813895222)*x_ref[2]
        ref[1,3,0,0]=(-0.519973841295)*x_ref[0]+(0.14862500493)*x_ref[1]+(-0.0257159960311)*x_ref[2]
        ref[1,3,0,1]=(-0.710423889723)*x_ref[0]+(0.0941990549739)*x_ref[1]+(-0.147877062943)*x_ref[2]
        ref[1,3,1,0]=(-0.19917762038)*x_ref[0]+(-1.06873791711)*x_ref[1]+(1.37688275432)*x_ref[2]
        ref[1,3,1,1]=(-0.570477493665)*x_ref[0]+(0.0705814270652)*x_ref[1]+(-1.10186231057)*x_ref[2]
        ref[1,3,2,0]=(-1.04891017002)*x_ref[0]+(-0.136784107636)*x_ref[1]+(-1.65953016693)*x_ref[2]
        ref[1,3,2,1]=(-0.674805716398)*x_ref[0]+(-0.211803508128)*x_ref[1]+(-0.116463503974)*x_ref[2]
        ref[1,4,0,0]=(0.269217220297)*x_ref[0]+(0.810722194772)*x_ref[1]+(1.60977315112)*x_ref[2]
        ref[1,4,0,1]=(-0.0213286766746)*x_ref[0]+(1.0974274553)*x_ref[1]+(0.275633207025)*x_ref[2]
        ref[1,4,1,0]=(-0.601230245409)*x_ref[0]+(-0.756418955933)*x_ref[1]+(-0.46194303915)*x_ref[2]
        ref[1,4,1,1]=(-0.0907667209143)*x_ref[0]+(-1.86623564673)*x_ref[1]+(0.552729474701)*x_ref[2]
        ref[1,4,2,0]=(-1.09694541489)*x_ref[0]+(-0.496193881555)*x_ref[1]+(-0.0126133249168)*x_ref[2]
        ref[1,4,2,1]=(-0.648959246633)*x_ref[0]+(-0.60655325639)*x_ref[1]+(0.438258051102)*x_ref[2]
        ref[2,0,0,0]=(-0.100512547019)*x_ref[0]+(-1.37291269079)*x_ref[1]+(1.63084319876)*x_ref[2]
        ref[2,0,0,1]=(-0.50510326916)*x_ref[0]+(0.308799737351)*x_ref[1]+(1.63506761477)*x_ref[2]
        ref[2,0,1,0]=(0.708408519723)*x_ref[0]+(-0.0759833841695)*x_ref[1]+(-0.980974095374)*x_ref[2]
        ref[2,0,1,1]=(-1.59727324143)*x_ref[0]+(0.995250195962)*x_ref[1]+(0.169527316083)*x_ref[2]
        ref[2,0,2,0]=(0.766935817054)*x_ref[0]+(0.455564399031)*x_ref[1]+(0.605772951782)*x_ref[2]
        ref[2,0,2,1]=(-0.820755214894)*x_ref[0]+(-0.43359371038)*x_ref[1]+(-0.0942398466964)*x_ref[2]
        ref[2,1,0,0]=(0.390476682801)*x_ref[0]+(1.11627447827)*x_ref[1]+(0.670843405466)*x_ref[2]
        ref[2,1,0,1]=(-0.433594870483)*x_ref[0]+(-0.940710004402)*x_ref[1]+(1.10756041487)*x_ref[2]
        ref[2,1,1,0]=(-1.21880018868)*x_ref[0]+(0.380422307344)*x_ref[1]+(-0.864017731789)*x_ref[2]
        ref[2,1,1,1]=(-0.194168346974)*x_ref[0]+(0.324183983028)*x_ref[1]+(0.741456579501)*x_ref[2]
        ref[2,1,2,0]=(-0.134211025912)*x_ref[0]+(0.497311041207)*x_ref[1]+(0.33014196376)*x_ref[2]
        ref[2,1,2,1]=(-1.3501981937)*x_ref[0]+(1.67601514859)*x_ref[1]+(-1.60084725544)*x_ref[2]
        ref[2,2,0,0]=(-0.8367457236)*x_ref[0]+(-1.15628524562)*x_ref[1]+(0.342983125417)*x_ref[2]
        ref[2,2,0,1]=(0.956733778283)*x_ref[0]+(-0.404880336577)*x_ref[1]+(-0.350927185105)*x_ref[2]
        ref[2,2,1,0]=(0.854842449281)*x_ref[0]+(-0.185001304765)*x_ref[1]+(1.15245157168)*x_ref[2]
        ref[2,2,1,1]=(0.00584831242623)*x_ref[0]+(-0.562950548984)*x_ref[1]+(0.785766513431)*x_ref[2]
        ref[2,2,2,0]=(0.272759945164)*x_ref[0]+(-0.568207376307)*x_ref[1]+(-0.155997758854)*x_ref[2]
        ref[2,2,2,1]=(-0.105324133175)*x_ref[0]+(1.11807221711)*x_ref[1]+(0.122751975298)*x_ref[2]
        ref[2,3,0,0]=(0.685932400265)*x_ref[0]+(-0.666847052302)*x_ref[1]+(0.596321369477)*x_ref[2]
        ref[2,3,0,1]=(0.335067153518)*x_ref[0]+(0.268453607431)*x_ref[1]+(0.247881738828)*x_ref[2]
        ref[2,3,1,0]=(-1.89084909179)*x_ref[0]+(-0.529751828744)*x_ref[1]+(-0.959358202828)*x_ref[2]
        ref[2,3,1,1]=(1.06725689989)*x_ref[0]+(-0.709878517637)*x_ref[1]+(0.968886366535)*x_ref[2]
        ref[2,3,2,0]=(-1.63466018652)*x_ref[0]+(0.517481248484)*x_ref[1]+(0.109232845638)*x_ref[2]
        ref[2,3,2,1]=(0.0716425266603)*x_ref[0]+(-0.58040972115)*x_ref[1]+(0.346948738894)*x_ref[2]
        ref[2,4,0,0]=(0.193599854213)*x_ref[0]+(-1.52744079327)*x_ref[1]+(-0.0377038338183)*x_ref[2]
        ref[2,4,0,1]=(-0.0296855161658)*x_ref[0]+(0.266976994363)*x_ref[1]+(-0.832771979863)*x_ref[2]
        ref[2,4,1,0]=(-0.885217283694)*x_ref[0]+(0.657425725373)*x_ref[1]+(0.385584782531)*x_ref[2]
        ref[2,4,1,1]=(-0.249912733602)*x_ref[0]+(0.156704487203)*x_ref[1]+(0.000393408500501)*x_ref[2]
        ref[2,4,2,0]=(-0.083303562971)*x_ref[0]+(0.748009334975)*x_ref[1]+(-0.411323946112)*x_ref[2]
        ref[2,4,2,1]=(0.143782087293)*x_ref[0]+(-1.86497054401)*x_ref[1]+(-0.398161751889)*x_ref[2]
        ref[3,0,0,0]=(-0.531041035506)*x_ref[0]+(0.998737023878)*x_ref[1]+(-0.719326852701)*x_ref[2]
        ref[3,0,0,1]=(-1.10320774278)*x_ref[0]+(-1.27795876669)*x_ref[1]+(0.57451462957)*x_ref[2]
        ref[3,0,1,0]=(-0.319275578502)*x_ref[0]+(-1.51721910203)*x_ref[1]+(-0.677470578443)*x_ref[2]
        ref[3,0,1,1]=(0.7584807917)*x_ref[0]+(0.0523998746885)*x_ref[1]+(0.502182688228)*x_ref[2]
        ref[3,0,2,0]=(-0.905809892125)*x_ref[0]+(0.479751975382)*x_ref[1]+(-0.0778539387977)*x_ref[2]
        ref[3,0,2,1]=(-0.724891388357)*x_ref[0]+(0.941493554841)*x_ref[1]+(-1.51514565838)*x_ref[2]
        ref[3,1,0,0]=(0.815118793475)*x_ref[0]+(0.0503368907581)*x_ref[1]+(-0.502091253213)*x_ref[2]
        ref[3,1,0,1]=(-1.50859117948)*x_ref[0]+(0.31279517213)*x_ref[1]+(-0.339295035594)*x_ref[2]
        ref[3,1,1,0]=(0.246825549192)*x_ref[0]+(0.687251908372)*x_ref[1]+(0.133971781776)*x_ref[2]
        ref[3,1,1,1]=(0.470105585328)*x_ref[0]+(0.382971101979)*x_ref[1]+(-0.469397699886)*x_ref[2]
        ref[3,1,2,0]=(-1.45842815542)*x_ref[0]+(-1.00979255513)*x_ref[1]+(1.45386034482)*x_ref[2]
        ref[3,1,2,1]=(0.172130164577)*x_ref[0]+(1.25034739991)*x_ref[1]+(-1.15953423359)*x_ref[2]
        ref[3,2,0,0]=(-0.58261393055)*x_ref[0]+(0.730381032663)*x_ref[1]+(1.29377076249)*x_ref[2]
        ref[3,2,0,1]=(0.696715362351)*x_ref[0]+(-1.37598351546)*x_ref[1]+(-1.26545085967)*x_ref[2]
        ref[3,2,1,0]=(-0.0511282610422)*x_ref[0]+(0.58041679118)*x_ref[1]+(-0.0810357412151)*x_ref[2]
        ref[3,2,1,1]=(-0.750057228803)*x_ref[0]+(-0.750703125074)*x_ref[1]+(-1.21748956536)*x_ref[2]
        ref[3,2,2,0]=(-0.455746666635)*x_ref[0]+(-0.720020627516)*x_ref[1]+(-0.0911281889971)*x_ref[2]
        ref[3,2,2,1]=(-0.0681894163236)*x_ref[0]+(0.101592029278)*x_ref[1]+(1.10954864406)*x_ref[2]
        ref[3,3,0,0]=(0.433898443791)*x_ref[0]+(0.901180030496)*x_ref[1]+(0.795641263874)*x_ref[2]
        ref[3,3,0,1]=(-0.213306810738)*x_ref[0]+(-0.852539239286)*x_ref[1]+(-1.25905416259)*x_ref[2]
        ref[3,3,1,0]=(-0.668307384473)*x_ref[0]+(-1.37753879862)*x_ref[1]+(-0.593731800903)*x_ref[2]
        ref[3,3,1,1]=(0.815464294514)*x_ref[0]+(0.835389640785)*x_ref[1]+(0.933029468258)*x_ref[2]
        ref[3,3,2,0]=(0.565194274444)*x_ref[0]+(0.584119707001)*x_ref[1]+(-0.53812189952)*x_ref[2]
        ref[3,3,2,1]=(-0.903604209314)*x_ref[0]+(0.370887007483)*x_ref[1]+(-0.548135915608)*x_ref[2]
        ref[3,4,0,0]=(-0.819290139492)*x_ref[0]+(-0.256800498012)*x_ref[1]+(0.350363336935)*x_ref[2]
        ref[3,4,0,1]=(0.863170182723)*x_ref[0]+(0.0101766489337)*x_ref[1]+(1.34009635664)*x_ref[2]
        ref[3,4,1,0]=(1.03350734136)*x_ref[0]+(1.37672500233)*x_ref[1]+(1.35134434663)*x_ref[2]
        ref[3,4,1,1]=(1.4883144839)*x_ref[0]+(0.410688244039)*x_ref[1]+(0.759267998436)*x_ref[2]
        ref[3,4,2,0]=(0.373190738096)*x_ref[0]+(1.22136515279)*x_ref[1]+(-0.0365030690962)*x_ref[2]
        ref[3,4,2,1]=(-0.607437402737)*x_ref[0]+(0.750130654364)*x_ref[1]+(0.084778884789)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_ReducedFunctionOnContactZero_rank0(self):
      """
      tests interpolate for rank 0 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactZero onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactZero(self.domain)
      x=w.getX()
      arg=Symbol(shape=(),dim=dim)
      s=Data(0,(),w)
      ref=Data(0,(),w_ref)
      if dim==2:
        s=(-0.478526643653)*x[0]**o+(-0.798079781236)*x[0]+(-0.499553919498)*x[1]**o+(0.336723069414)*x[1]
        ref=(-0.478526643653)*x_ref[0]**o+(-0.798079781236)*x_ref[0]+(-0.499553919498)*x_ref[1]**o+(0.336723069414)*x_ref[1]
      else:
        s=(-0.35871218678)*x[0]**o+(0.954679093985)*x[0]+(-0.127722494434)*x[1]**o+(0.0119904093364)*x[1]+(-0.647695475743)*x[2]**o+(-0.522381915873)*x[2]
        ref=(-0.35871218678)*x_ref[0]**o+(0.954679093985)*x_ref[0]+(-0.127722494434)*x_ref[1]**o+(0.0119904093364)*x_ref[1]+(-0.647695475743)*x_ref[2]**o+(-0.522381915873)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_ReducedFunctionOnContactZero_rank1(self):
      """
      tests interpolate for rank 1 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactZero onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactZero(self.domain)
      x=w.getX()
      arg=Symbol(shape=(2,),dim=dim)
      s=Data(0,(2,),w)
      ref=Data(0,(2,),w_ref)
      if dim==2:
        s[0]=(0.0333528524993)*x[0]**o+(-0.534169045864)*x[0]+(0.234758321366)*x[1]**o+(0.395702014106)*x[1]
        s[1]=(-0.559512507686)*x[0]**o+(0.473831121701)*x[0]+(0.315500482589)*x[1]**o+(0.659175367015)*x[1]
        ref[0]=(0.0333528524993)*x_ref[0]**o+(-0.534169045864)*x_ref[0]+(0.234758321366)*x_ref[1]**o+(0.395702014106)*x_ref[1]
        ref[1]=(-0.559512507686)*x_ref[0]**o+(0.473831121701)*x_ref[0]+(0.315500482589)*x_ref[1]**o+(0.659175367015)*x_ref[1]
      else:
        s[0]=(0.920431380976)*x[0]**o+(0.203182015633)*x[0]+(-0.30119291643)*x[1]**o+(0.743123673038)*x[1]+(0.158345217723)*x[2]**o+(-0.106151059285)*x[2]
        s[1]=(-0.246067927964)*x[0]**o+(0.249705802315)*x[0]+(0.87511193184)*x[1]**o+(0.174791560577)*x[1]+(-0.717945042302)*x[2]**o+(0.558392243221)*x[2]
        ref[0]=(0.920431380976)*x_ref[0]**o+(0.203182015633)*x_ref[0]+(-0.30119291643)*x_ref[1]**o+(0.743123673038)*x_ref[1]+(0.158345217723)*x_ref[2]**o+(-0.106151059285)*x_ref[2]
        ref[1]=(-0.246067927964)*x_ref[0]**o+(0.249705802315)*x_ref[0]+(0.87511193184)*x_ref[1]**o+(0.174791560577)*x_ref[1]+(-0.717945042302)*x_ref[2]**o+(0.558392243221)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_ReducedFunctionOnContactZero_rank2(self):
      """
      tests interpolate for rank 2 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactZero onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactZero(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5),dim=dim)
      s=Data(0,(4, 5),w)
      ref=Data(0,(4, 5),w_ref)
      if dim==2:
        s[0,0]=(-0.238674589068)*x[0]**o+(0.845146310144)*x[0]+(0.51393009023)*x[1]**o+(0.987659698068)*x[1]
        s[0,1]=(0.525738867968)*x[0]**o+(-0.541924459594)*x[0]+(-0.345083945286)*x[1]**o+(0.892536321695)*x[1]
        s[0,2]=(0.467434483897)*x[0]**o+(0.673356246793)*x[0]+(0.373066330502)*x[1]**o+(0.455273174229)*x[1]
        s[0,3]=(-0.39771044233)*x[0]**o+(0.890234781078)*x[0]+(-0.11624722564)*x[1]**o+(0.995857794085)*x[1]
        s[0,4]=(0.795987536438)*x[0]**o+(-0.683413643902)*x[0]+(-0.0517246803869)*x[1]**o+(0.149282877675)*x[1]
        s[1,0]=(0.215953459935)*x[0]**o+(0.0124143229743)*x[0]+(0.719573519094)*x[1]**o+(0.120838088026)*x[1]
        s[1,1]=(0.195894110038)*x[0]**o+(-0.481212957341)*x[0]+(-0.579369726661)*x[1]**o+(-0.825868817932)*x[1]
        s[1,2]=(0.242091565308)*x[0]**o+(-0.342866600818)*x[0]+(0.871948019343)*x[1]**o+(0.925599757496)*x[1]
        s[1,3]=(-0.228955457627)*x[0]**o+(-0.618007692175)*x[0]+(0.641736066096)*x[1]**o+(-0.648940539231)*x[1]
        s[1,4]=(0.949258368604)*x[0]**o+(0.46037118633)*x[0]+(0.264262232255)*x[1]**o+(-0.204687228041)*x[1]
        s[2,0]=(0.201719829764)*x[0]**o+(-0.758921537456)*x[0]+(0.809565809407)*x[1]**o+(0.18932187303)*x[1]
        s[2,1]=(0.0400837555441)*x[0]**o+(0.703393740665)*x[0]+(0.989844901722)*x[1]**o+(-0.920311703188)*x[1]
        s[2,2]=(-0.1127643095)*x[0]**o+(-0.181508597843)*x[0]+(0.845854218374)*x[1]**o+(-0.401564965436)*x[1]
        s[2,3]=(0.725120155616)*x[0]**o+(0.0408162472823)*x[0]+(0.849016789586)*x[1]**o+(-0.0894725097851)*x[1]
        s[2,4]=(0.00712438942747)*x[0]**o+(0.430728482242)*x[0]+(-0.630571782961)*x[1]**o+(-0.292289123038)*x[1]
        s[3,0]=(0.604980765238)*x[0]**o+(-0.545924520431)*x[0]+(0.365986250184)*x[1]**o+(-0.717572291319)*x[1]
        s[3,1]=(-0.105418444539)*x[0]**o+(-0.301186945237)*x[0]+(0.298798693172)*x[1]**o+(0.305159917178)*x[1]
        s[3,2]=(-0.116290679999)*x[0]**o+(-0.814717358777)*x[0]+(0.463687518523)*x[1]**o+(-0.142856072677)*x[1]
        s[3,3]=(-0.0933248161262)*x[0]**o+(-0.745036857332)*x[0]+(-0.143025048876)*x[1]**o+(-0.317322119337)*x[1]
        s[3,4]=(0.340645005623)*x[0]**o+(-0.369235909154)*x[0]+(-0.739698569483)*x[1]**o+(0.296590339275)*x[1]
        ref[0,0]=(-0.238674589068)*x_ref[0]**o+(0.845146310144)*x_ref[0]+(0.51393009023)*x_ref[1]**o+(0.987659698068)*x_ref[1]
        ref[0,1]=(0.525738867968)*x_ref[0]**o+(-0.541924459594)*x_ref[0]+(-0.345083945286)*x_ref[1]**o+(0.892536321695)*x_ref[1]
        ref[0,2]=(0.467434483897)*x_ref[0]**o+(0.673356246793)*x_ref[0]+(0.373066330502)*x_ref[1]**o+(0.455273174229)*x_ref[1]
        ref[0,3]=(-0.39771044233)*x_ref[0]**o+(0.890234781078)*x_ref[0]+(-0.11624722564)*x_ref[1]**o+(0.995857794085)*x_ref[1]
        ref[0,4]=(0.795987536438)*x_ref[0]**o+(-0.683413643902)*x_ref[0]+(-0.0517246803869)*x_ref[1]**o+(0.149282877675)*x_ref[1]
        ref[1,0]=(0.215953459935)*x_ref[0]**o+(0.0124143229743)*x_ref[0]+(0.719573519094)*x_ref[1]**o+(0.120838088026)*x_ref[1]
        ref[1,1]=(0.195894110038)*x_ref[0]**o+(-0.481212957341)*x_ref[0]+(-0.579369726661)*x_ref[1]**o+(-0.825868817932)*x_ref[1]
        ref[1,2]=(0.242091565308)*x_ref[0]**o+(-0.342866600818)*x_ref[0]+(0.871948019343)*x_ref[1]**o+(0.925599757496)*x_ref[1]
        ref[1,3]=(-0.228955457627)*x_ref[0]**o+(-0.618007692175)*x_ref[0]+(0.641736066096)*x_ref[1]**o+(-0.648940539231)*x_ref[1]
        ref[1,4]=(0.949258368604)*x_ref[0]**o+(0.46037118633)*x_ref[0]+(0.264262232255)*x_ref[1]**o+(-0.204687228041)*x_ref[1]
        ref[2,0]=(0.201719829764)*x_ref[0]**o+(-0.758921537456)*x_ref[0]+(0.809565809407)*x_ref[1]**o+(0.18932187303)*x_ref[1]
        ref[2,1]=(0.0400837555441)*x_ref[0]**o+(0.703393740665)*x_ref[0]+(0.989844901722)*x_ref[1]**o+(-0.920311703188)*x_ref[1]
        ref[2,2]=(-0.1127643095)*x_ref[0]**o+(-0.181508597843)*x_ref[0]+(0.845854218374)*x_ref[1]**o+(-0.401564965436)*x_ref[1]
        ref[2,3]=(0.725120155616)*x_ref[0]**o+(0.0408162472823)*x_ref[0]+(0.849016789586)*x_ref[1]**o+(-0.0894725097851)*x_ref[1]
        ref[2,4]=(0.00712438942747)*x_ref[0]**o+(0.430728482242)*x_ref[0]+(-0.630571782961)*x_ref[1]**o+(-0.292289123038)*x_ref[1]
        ref[3,0]=(0.604980765238)*x_ref[0]**o+(-0.545924520431)*x_ref[0]+(0.365986250184)*x_ref[1]**o+(-0.717572291319)*x_ref[1]
        ref[3,1]=(-0.105418444539)*x_ref[0]**o+(-0.301186945237)*x_ref[0]+(0.298798693172)*x_ref[1]**o+(0.305159917178)*x_ref[1]
        ref[3,2]=(-0.116290679999)*x_ref[0]**o+(-0.814717358777)*x_ref[0]+(0.463687518523)*x_ref[1]**o+(-0.142856072677)*x_ref[1]
        ref[3,3]=(-0.0933248161262)*x_ref[0]**o+(-0.745036857332)*x_ref[0]+(-0.143025048876)*x_ref[1]**o+(-0.317322119337)*x_ref[1]
        ref[3,4]=(0.340645005623)*x_ref[0]**o+(-0.369235909154)*x_ref[0]+(-0.739698569483)*x_ref[1]**o+(0.296590339275)*x_ref[1]
      else:
        s[0,0]=(-0.343988908393)*x[0]**o+(-0.249046543972)*x[0]+(-0.94669255683)*x[1]**o+(-0.125519622379)*x[1]+(-0.238439423996)*x[2]**o+(-0.270484426474)*x[2]
        s[0,1]=(-0.81960628266)*x[0]**o+(-0.752592323006)*x[0]+(-0.608566310221)*x[1]**o+(0.136425226211)*x[1]+(-0.417282843542)*x[2]**o+(0.510888775001)*x[2]
        s[0,2]=(-0.469706099858)*x[0]**o+(0.948757046189)*x[0]+(-0.752290984326)*x[1]**o+(0.423515758928)*x[1]+(0.031409477618)*x[2]**o+(0.234409805802)*x[2]
        s[0,3]=(-0.900755988795)*x[0]**o+(-0.767761655222)*x[0]+(0.569180403754)*x[1]**o+(-0.233362758546)*x[1]+(0.569347237378)*x[2]**o+(0.612139565636)*x[2]
        s[0,4]=(0.885682511121)*x[0]**o+(0.332415602786)*x[0]+(0.350335006384)*x[1]**o+(0.613490643585)*x[1]+(0.444593860725)*x[2]**o+(-0.000225980787608)*x[2]
        s[1,0]=(-0.584959823379)*x[0]**o+(-0.964957485122)*x[0]+(0.289141634104)*x[1]**o+(0.263120358818)*x[1]+(-0.028745699583)*x[2]**o+(0.982902290525)*x[2]
        s[1,1]=(-0.191593204495)*x[0]**o+(-0.604908583526)*x[0]+(0.804988763905)*x[1]**o+(0.741296673588)*x[1]+(0.808775189028)*x[2]**o+(0.320306927549)*x[2]
        s[1,2]=(0.658017563103)*x[0]**o+(0.150282440579)*x[0]+(0.988184604156)*x[1]**o+(-0.168752859833)*x[1]+(0.0248149744777)*x[2]**o+(0.158431888663)*x[2]
        s[1,3]=(0.482286527364)*x[0]**o+(0.794842099146)*x[0]+(-0.103524158001)*x[1]**o+(0.834955012893)*x[1]+(-0.724721525621)*x[2]**o+(-0.581451689942)*x[2]
        s[1,4]=(0.987701027083)*x[0]**o+(0.358477587611)*x[0]+(0.0447031116415)*x[1]**o+(-0.63585414781)*x[1]+(-0.493573694258)*x[2]**o+(-0.670666093134)*x[2]
        s[2,0]=(0.156348690681)*x[0]**o+(0.181907398102)*x[0]+(0.810783963046)*x[1]**o+(-0.0420809525478)*x[1]+(0.455236439578)*x[2]**o+(0.68359015036)*x[2]
        s[2,1]=(-0.277203778658)*x[0]**o+(0.906301662318)*x[0]+(0.507233384623)*x[1]**o+(-0.905644178764)*x[1]+(-0.116730257787)*x[2]**o+(-0.205065773315)*x[2]
        s[2,2]=(-0.951427408426)*x[0]**o+(0.418017573994)*x[0]+(-0.707741913019)*x[1]**o+(-0.517554096797)*x[1]+(-0.612052430927)*x[2]**o+(-0.831045221937)*x[2]
        s[2,3]=(0.0142781625544)*x[0]**o+(-0.0207359350934)*x[0]+(0.69086283301)*x[1]**o+(-0.487396868956)*x[1]+(0.868531537118)*x[2]**o+(-0.269368930177)*x[2]
        s[2,4]=(-0.657537261147)*x[0]**o+(0.411006377043)*x[0]+(0.641800881279)*x[1]**o+(-0.179106096535)*x[1]+(0.413169503059)*x[2]**o+(0.0796148767789)*x[2]
        s[3,0]=(-0.0465615058741)*x[0]**o+(0.497229745315)*x[0]+(0.048895565684)*x[1]**o+(0.881074858803)*x[1]+(-0.0904556886937)*x[2]**o+(-0.0486834982676)*x[2]
        s[3,1]=(0.472154117103)*x[0]**o+(-0.841976825688)*x[0]+(-0.840760456295)*x[1]**o+(-0.461023286336)*x[1]+(0.680506149471)*x[2]**o+(0.503845111856)*x[2]
        s[3,2]=(0.166611223306)*x[0]**o+(0.308064312352)*x[0]+(0.850239399334)*x[1]**o+(0.127827758007)*x[1]+(0.306675924384)*x[2]**o+(-0.35563117529)*x[2]
        s[3,3]=(0.0854397842075)*x[0]**o+(0.696191334187)*x[0]+(0.719296439336)*x[1]**o+(0.703852484675)*x[1]+(-0.841440898463)*x[2]**o+(-0.688038437262)*x[2]
        s[3,4]=(0.775596800509)*x[0]**o+(-0.0380327958147)*x[0]+(0.0582036420384)*x[1]**o+(0.0396609528486)*x[1]+(-0.449051573608)*x[2]**o+(0.139175147549)*x[2]
        ref[0,0]=(-0.343988908393)*x_ref[0]**o+(-0.249046543972)*x_ref[0]+(-0.94669255683)*x_ref[1]**o+(-0.125519622379)*x_ref[1]+(-0.238439423996)*x_ref[2]**o+(-0.270484426474)*x_ref[2]
        ref[0,1]=(-0.81960628266)*x_ref[0]**o+(-0.752592323006)*x_ref[0]+(-0.608566310221)*x_ref[1]**o+(0.136425226211)*x_ref[1]+(-0.417282843542)*x_ref[2]**o+(0.510888775001)*x_ref[2]
        ref[0,2]=(-0.469706099858)*x_ref[0]**o+(0.948757046189)*x_ref[0]+(-0.752290984326)*x_ref[1]**o+(0.423515758928)*x_ref[1]+(0.031409477618)*x_ref[2]**o+(0.234409805802)*x_ref[2]
        ref[0,3]=(-0.900755988795)*x_ref[0]**o+(-0.767761655222)*x_ref[0]+(0.569180403754)*x_ref[1]**o+(-0.233362758546)*x_ref[1]+(0.569347237378)*x_ref[2]**o+(0.612139565636)*x_ref[2]
        ref[0,4]=(0.885682511121)*x_ref[0]**o+(0.332415602786)*x_ref[0]+(0.350335006384)*x_ref[1]**o+(0.613490643585)*x_ref[1]+(0.444593860725)*x_ref[2]**o+(-0.000225980787608)*x_ref[2]
        ref[1,0]=(-0.584959823379)*x_ref[0]**o+(-0.964957485122)*x_ref[0]+(0.289141634104)*x_ref[1]**o+(0.263120358818)*x_ref[1]+(-0.028745699583)*x_ref[2]**o+(0.982902290525)*x_ref[2]
        ref[1,1]=(-0.191593204495)*x_ref[0]**o+(-0.604908583526)*x_ref[0]+(0.804988763905)*x_ref[1]**o+(0.741296673588)*x_ref[1]+(0.808775189028)*x_ref[2]**o+(0.320306927549)*x_ref[2]
        ref[1,2]=(0.658017563103)*x_ref[0]**o+(0.150282440579)*x_ref[0]+(0.988184604156)*x_ref[1]**o+(-0.168752859833)*x_ref[1]+(0.0248149744777)*x_ref[2]**o+(0.158431888663)*x_ref[2]
        ref[1,3]=(0.482286527364)*x_ref[0]**o+(0.794842099146)*x_ref[0]+(-0.103524158001)*x_ref[1]**o+(0.834955012893)*x_ref[1]+(-0.724721525621)*x_ref[2]**o+(-0.581451689942)*x_ref[2]
        ref[1,4]=(0.987701027083)*x_ref[0]**o+(0.358477587611)*x_ref[0]+(0.0447031116415)*x_ref[1]**o+(-0.63585414781)*x_ref[1]+(-0.493573694258)*x_ref[2]**o+(-0.670666093134)*x_ref[2]
        ref[2,0]=(0.156348690681)*x_ref[0]**o+(0.181907398102)*x_ref[0]+(0.810783963046)*x_ref[1]**o+(-0.0420809525478)*x_ref[1]+(0.455236439578)*x_ref[2]**o+(0.68359015036)*x_ref[2]
        ref[2,1]=(-0.277203778658)*x_ref[0]**o+(0.906301662318)*x_ref[0]+(0.507233384623)*x_ref[1]**o+(-0.905644178764)*x_ref[1]+(-0.116730257787)*x_ref[2]**o+(-0.205065773315)*x_ref[2]
        ref[2,2]=(-0.951427408426)*x_ref[0]**o+(0.418017573994)*x_ref[0]+(-0.707741913019)*x_ref[1]**o+(-0.517554096797)*x_ref[1]+(-0.612052430927)*x_ref[2]**o+(-0.831045221937)*x_ref[2]
        ref[2,3]=(0.0142781625544)*x_ref[0]**o+(-0.0207359350934)*x_ref[0]+(0.69086283301)*x_ref[1]**o+(-0.487396868956)*x_ref[1]+(0.868531537118)*x_ref[2]**o+(-0.269368930177)*x_ref[2]
        ref[2,4]=(-0.657537261147)*x_ref[0]**o+(0.411006377043)*x_ref[0]+(0.641800881279)*x_ref[1]**o+(-0.179106096535)*x_ref[1]+(0.413169503059)*x_ref[2]**o+(0.0796148767789)*x_ref[2]
        ref[3,0]=(-0.0465615058741)*x_ref[0]**o+(0.497229745315)*x_ref[0]+(0.048895565684)*x_ref[1]**o+(0.881074858803)*x_ref[1]+(-0.0904556886937)*x_ref[2]**o+(-0.0486834982676)*x_ref[2]
        ref[3,1]=(0.472154117103)*x_ref[0]**o+(-0.841976825688)*x_ref[0]+(-0.840760456295)*x_ref[1]**o+(-0.461023286336)*x_ref[1]+(0.680506149471)*x_ref[2]**o+(0.503845111856)*x_ref[2]
        ref[3,2]=(0.166611223306)*x_ref[0]**o+(0.308064312352)*x_ref[0]+(0.850239399334)*x_ref[1]**o+(0.127827758007)*x_ref[1]+(0.306675924384)*x_ref[2]**o+(-0.35563117529)*x_ref[2]
        ref[3,3]=(0.0854397842075)*x_ref[0]**o+(0.696191334187)*x_ref[0]+(0.719296439336)*x_ref[1]**o+(0.703852484675)*x_ref[1]+(-0.841440898463)*x_ref[2]**o+(-0.688038437262)*x_ref[2]
        ref[3,4]=(0.775596800509)*x_ref[0]**o+(-0.0380327958147)*x_ref[0]+(0.0582036420384)*x_ref[1]**o+(0.0396609528486)*x_ref[1]+(-0.449051573608)*x_ref[2]**o+(0.139175147549)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_ReducedFunctionOnContactZero_rank3(self):
      """
      tests interpolate for rank 3 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactZero onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactZero(self.domain)
      x=w.getX()
      arg=Symbol(shape=(6, 2, 2),dim=dim)
      s=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2),w_ref)
      if dim==2:
        s[0,0,0]=(0.748658213139)*x[0]**o+(0.588706109747)*x[0]+(-0.462395788697)*x[1]**o+(0.558348215492)*x[1]
        s[0,0,1]=(0.629957777572)*x[0]**o+(0.714493090646)*x[0]+(-0.491054832228)*x[1]**o+(0.710860562012)*x[1]
        s[0,1,0]=(-0.775354422191)*x[0]**o+(0.152096744898)*x[0]+(0.717568022676)*x[1]**o+(0.749449532926)*x[1]
        s[0,1,1]=(0.883871444687)*x[0]**o+(0.149367558371)*x[0]+(0.986279594706)*x[1]**o+(0.537229955955)*x[1]
        s[1,0,0]=(0.564688441996)*x[0]**o+(-0.206862774902)*x[0]+(0.522967069439)*x[1]**o+(-0.0390132413525)*x[1]
        s[1,0,1]=(0.706775860383)*x[0]**o+(0.973359689319)*x[0]+(0.567062178556)*x[1]**o+(-0.00264874871685)*x[1]
        s[1,1,0]=(-0.385131437493)*x[0]**o+(0.93223208004)*x[0]+(0.915604601744)*x[1]**o+(-0.604136431942)*x[1]
        s[1,1,1]=(-0.58509656422)*x[0]**o+(-0.46833032365)*x[0]+(0.143097299013)*x[1]**o+(-0.685071549088)*x[1]
        s[2,0,0]=(-0.987452159168)*x[0]**o+(0.609343916634)*x[0]+(0.425443014801)*x[1]**o+(0.58099336693)*x[1]
        s[2,0,1]=(0.957582337028)*x[0]**o+(0.62772269136)*x[0]+(-0.783891315097)*x[1]**o+(-0.77280642716)*x[1]
        s[2,1,0]=(0.327068296677)*x[0]**o+(0.0579535708335)*x[0]+(-0.798372147412)*x[1]**o+(-0.517396772711)*x[1]
        s[2,1,1]=(-0.863293845828)*x[0]**o+(0.688257458796)*x[0]+(0.777044937444)*x[1]**o+(0.210894669552)*x[1]
        s[3,0,0]=(0.641389559724)*x[0]**o+(-0.115831890978)*x[0]+(0.0852026159583)*x[1]**o+(0.101596282721)*x[1]
        s[3,0,1]=(0.0869751746538)*x[0]**o+(0.8310099269)*x[0]+(0.915409474965)*x[1]**o+(-0.140400418634)*x[1]
        s[3,1,0]=(-0.782218646979)*x[0]**o+(0.259242630384)*x[0]+(0.474259881734)*x[1]**o+(-0.339247837819)*x[1]
        s[3,1,1]=(0.110237392658)*x[0]**o+(-0.631167781024)*x[0]+(-0.901066615275)*x[1]**o+(0.380918387854)*x[1]
        s[4,0,0]=(-0.377519984827)*x[0]**o+(-0.77772320212)*x[0]+(0.441380184654)*x[1]**o+(0.601046552584)*x[1]
        s[4,0,1]=(-0.563858988542)*x[0]**o+(0.00445139485154)*x[0]+(-0.294713455734)*x[1]**o+(-0.863742009505)*x[1]
        s[4,1,0]=(-0.0472447099394)*x[0]**o+(-0.192880217627)*x[0]+(-0.299507318615)*x[1]**o+(-0.372320397063)*x[1]
        s[4,1,1]=(-0.81522394336)*x[0]**o+(0.822036737098)*x[0]+(0.944437317648)*x[1]**o+(-0.614529825138)*x[1]
        s[5,0,0]=(0.817613090618)*x[0]**o+(-0.611860825889)*x[0]+(-0.425821334232)*x[1]**o+(0.657829198468)*x[1]
        s[5,0,1]=(-0.562766866656)*x[0]**o+(0.374639711066)*x[0]+(0.5381903866)*x[1]**o+(0.642409148669)*x[1]
        s[5,1,0]=(0.98365300543)*x[0]**o+(0.95143935497)*x[0]+(-0.186391221276)*x[1]**o+(0.99144952063)*x[1]
        s[5,1,1]=(0.422131909022)*x[0]**o+(-0.503167260902)*x[0]+(-0.708590674746)*x[1]**o+(0.164597708191)*x[1]
        ref[0,0,0]=(0.748658213139)*x_ref[0]**o+(0.588706109747)*x_ref[0]+(-0.462395788697)*x_ref[1]**o+(0.558348215492)*x_ref[1]
        ref[0,0,1]=(0.629957777572)*x_ref[0]**o+(0.714493090646)*x_ref[0]+(-0.491054832228)*x_ref[1]**o+(0.710860562012)*x_ref[1]
        ref[0,1,0]=(-0.775354422191)*x_ref[0]**o+(0.152096744898)*x_ref[0]+(0.717568022676)*x_ref[1]**o+(0.749449532926)*x_ref[1]
        ref[0,1,1]=(0.883871444687)*x_ref[0]**o+(0.149367558371)*x_ref[0]+(0.986279594706)*x_ref[1]**o+(0.537229955955)*x_ref[1]
        ref[1,0,0]=(0.564688441996)*x_ref[0]**o+(-0.206862774902)*x_ref[0]+(0.522967069439)*x_ref[1]**o+(-0.0390132413525)*x_ref[1]
        ref[1,0,1]=(0.706775860383)*x_ref[0]**o+(0.973359689319)*x_ref[0]+(0.567062178556)*x_ref[1]**o+(-0.00264874871685)*x_ref[1]
        ref[1,1,0]=(-0.385131437493)*x_ref[0]**o+(0.93223208004)*x_ref[0]+(0.915604601744)*x_ref[1]**o+(-0.604136431942)*x_ref[1]
        ref[1,1,1]=(-0.58509656422)*x_ref[0]**o+(-0.46833032365)*x_ref[0]+(0.143097299013)*x_ref[1]**o+(-0.685071549088)*x_ref[1]
        ref[2,0,0]=(-0.987452159168)*x_ref[0]**o+(0.609343916634)*x_ref[0]+(0.425443014801)*x_ref[1]**o+(0.58099336693)*x_ref[1]
        ref[2,0,1]=(0.957582337028)*x_ref[0]**o+(0.62772269136)*x_ref[0]+(-0.783891315097)*x_ref[1]**o+(-0.77280642716)*x_ref[1]
        ref[2,1,0]=(0.327068296677)*x_ref[0]**o+(0.0579535708335)*x_ref[0]+(-0.798372147412)*x_ref[1]**o+(-0.517396772711)*x_ref[1]
        ref[2,1,1]=(-0.863293845828)*x_ref[0]**o+(0.688257458796)*x_ref[0]+(0.777044937444)*x_ref[1]**o+(0.210894669552)*x_ref[1]
        ref[3,0,0]=(0.641389559724)*x_ref[0]**o+(-0.115831890978)*x_ref[0]+(0.0852026159583)*x_ref[1]**o+(0.101596282721)*x_ref[1]
        ref[3,0,1]=(0.0869751746538)*x_ref[0]**o+(0.8310099269)*x_ref[0]+(0.915409474965)*x_ref[1]**o+(-0.140400418634)*x_ref[1]
        ref[3,1,0]=(-0.782218646979)*x_ref[0]**o+(0.259242630384)*x_ref[0]+(0.474259881734)*x_ref[1]**o+(-0.339247837819)*x_ref[1]
        ref[3,1,1]=(0.110237392658)*x_ref[0]**o+(-0.631167781024)*x_ref[0]+(-0.901066615275)*x_ref[1]**o+(0.380918387854)*x_ref[1]
        ref[4,0,0]=(-0.377519984827)*x_ref[0]**o+(-0.77772320212)*x_ref[0]+(0.441380184654)*x_ref[1]**o+(0.601046552584)*x_ref[1]
        ref[4,0,1]=(-0.563858988542)*x_ref[0]**o+(0.00445139485154)*x_ref[0]+(-0.294713455734)*x_ref[1]**o+(-0.863742009505)*x_ref[1]
        ref[4,1,0]=(-0.0472447099394)*x_ref[0]**o+(-0.192880217627)*x_ref[0]+(-0.299507318615)*x_ref[1]**o+(-0.372320397063)*x_ref[1]
        ref[4,1,1]=(-0.81522394336)*x_ref[0]**o+(0.822036737098)*x_ref[0]+(0.944437317648)*x_ref[1]**o+(-0.614529825138)*x_ref[1]
        ref[5,0,0]=(0.817613090618)*x_ref[0]**o+(-0.611860825889)*x_ref[0]+(-0.425821334232)*x_ref[1]**o+(0.657829198468)*x_ref[1]
        ref[5,0,1]=(-0.562766866656)*x_ref[0]**o+(0.374639711066)*x_ref[0]+(0.5381903866)*x_ref[1]**o+(0.642409148669)*x_ref[1]
        ref[5,1,0]=(0.98365300543)*x_ref[0]**o+(0.95143935497)*x_ref[0]+(-0.186391221276)*x_ref[1]**o+(0.99144952063)*x_ref[1]
        ref[5,1,1]=(0.422131909022)*x_ref[0]**o+(-0.503167260902)*x_ref[0]+(-0.708590674746)*x_ref[1]**o+(0.164597708191)*x_ref[1]
      else:
        s[0,0,0]=(0.286638887843)*x[0]**o+(0.522689277676)*x[0]+(0.467954875136)*x[1]**o+(0.445711366876)*x[1]+(-0.669973962002)*x[2]**o+(0.181395269699)*x[2]
        s[0,0,1]=(0.353016816809)*x[0]**o+(0.991829804601)*x[0]+(-0.258315747172)*x[1]**o+(0.543990055925)*x[1]+(-0.769057570955)*x[2]**o+(0.233571124807)*x[2]
        s[0,1,0]=(0.918139015102)*x[0]**o+(-0.947988526027)*x[0]+(0.331622282377)*x[1]**o+(-0.777541428786)*x[1]+(-0.664961721637)*x[2]**o+(-0.484643482585)*x[2]
        s[0,1,1]=(0.140004476479)*x[0]**o+(-0.610763924636)*x[0]+(0.789189906539)*x[1]**o+(-0.590757211504)*x[1]+(-0.0725209360897)*x[2]**o+(-0.766615663399)*x[2]
        s[1,0,0]=(0.725731333346)*x[0]**o+(-0.131718022099)*x[0]+(0.627777941816)*x[1]**o+(-0.482136262357)*x[1]+(0.413942360645)*x[2]**o+(-0.987161567454)*x[2]
        s[1,0,1]=(0.975906575301)*x[0]**o+(0.642044782609)*x[0]+(0.670119948857)*x[1]**o+(-0.93125168252)*x[1]+(-0.553725567396)*x[2]**o+(0.609585560905)*x[2]
        s[1,1,0]=(-0.073682338522)*x[0]**o+(-0.391983504903)*x[0]+(0.252884640925)*x[1]**o+(0.474040995664)*x[1]+(-0.763628395477)*x[2]**o+(-0.753583810078)*x[2]
        s[1,1,1]=(-0.680881408584)*x[0]**o+(-0.546411376016)*x[0]+(0.85433594417)*x[1]**o+(-0.990738365487)*x[1]+(-0.202162243501)*x[2]**o+(0.256411615615)*x[2]
        s[2,0,0]=(0.79289192889)*x[0]**o+(0.243831874798)*x[0]+(0.681415436664)*x[1]**o+(0.223205387356)*x[1]+(0.517155768633)*x[2]**o+(-0.283058167273)*x[2]
        s[2,0,1]=(-0.608869102346)*x[0]**o+(0.496990925284)*x[0]+(0.925407693967)*x[1]**o+(-0.358933716971)*x[1]+(-0.328654182706)*x[2]**o+(0.835965636453)*x[2]
        s[2,1,0]=(-0.830133878676)*x[0]**o+(-0.123104434072)*x[0]+(-0.662614589611)*x[1]**o+(-0.288827930561)*x[1]+(-0.411343484191)*x[2]**o+(0.632500606896)*x[2]
        s[2,1,1]=(0.272639418094)*x[0]**o+(-0.897989787007)*x[0]+(-0.838744589589)*x[1]**o+(0.492441488123)*x[1]+(-0.634424482735)*x[2]**o+(-0.509179635903)*x[2]
        s[3,0,0]=(-0.810786146104)*x[0]**o+(-0.582309522585)*x[0]+(-0.0537085055431)*x[1]**o+(0.230345910939)*x[1]+(0.482750960353)*x[2]**o+(0.248433602737)*x[2]
        s[3,0,1]=(0.148959585193)*x[0]**o+(0.222768449067)*x[0]+(0.819219237595)*x[1]**o+(0.386479159121)*x[1]+(-0.0808464606991)*x[2]**o+(-0.0578651077709)*x[2]
        s[3,1,0]=(-0.665745862321)*x[0]**o+(-0.864332853905)*x[0]+(-0.622626855999)*x[1]**o+(-0.961729702089)*x[1]+(-0.0528887263986)*x[2]**o+(0.203930539621)*x[2]
        s[3,1,1]=(0.216834395994)*x[0]**o+(0.14686771427)*x[0]+(-0.634838825712)*x[1]**o+(-0.0741875710555)*x[1]+(0.49863557383)*x[2]**o+(0.128134148051)*x[2]
        s[4,0,0]=(-0.0429185773768)*x[0]**o+(-0.884386434806)*x[0]+(0.590953595756)*x[1]**o+(0.229045075384)*x[1]+(0.289339613114)*x[2]**o+(0.799614281102)*x[2]
        s[4,0,1]=(0.401237094734)*x[0]**o+(0.374532696487)*x[0]+(0.382784705093)*x[1]**o+(-0.00752495997413)*x[1]+(-0.693554885405)*x[2]**o+(-0.0030745706685)*x[2]
        s[4,1,0]=(0.746124554182)*x[0]**o+(-0.863459707092)*x[0]+(0.586647115827)*x[1]**o+(-0.94164686768)*x[1]+(0.700937522698)*x[2]**o+(-0.0464264819588)*x[2]
        s[4,1,1]=(0.33270904188)*x[0]**o+(0.90735243167)*x[0]+(-0.479962340673)*x[1]**o+(-0.860540809179)*x[1]+(0.396295831574)*x[2]**o+(-0.47164342577)*x[2]
        s[5,0,0]=(-0.518396400617)*x[0]**o+(0.0893385548496)*x[0]+(0.695842336307)*x[1]**o+(0.552511723893)*x[1]+(0.0636197708084)*x[2]**o+(0.0221143500177)*x[2]
        s[5,0,1]=(0.612931629659)*x[0]**o+(-0.898561600235)*x[0]+(0.231387395725)*x[1]**o+(0.728648837106)*x[1]+(-0.819722749156)*x[2]**o+(-0.987410960592)*x[2]
        s[5,1,0]=(0.228541632142)*x[0]**o+(0.0352301546347)*x[0]+(-0.457859957282)*x[1]**o+(0.126487825769)*x[1]+(-0.685267391542)*x[2]**o+(-0.695662688997)*x[2]
        s[5,1,1]=(0.892154759933)*x[0]**o+(0.0404556267055)*x[0]+(-0.509790912326)*x[1]**o+(0.544457102938)*x[1]+(-0.110692172028)*x[2]**o+(-0.111481287291)*x[2]
        ref[0,0,0]=(0.286638887843)*x_ref[0]**o+(0.522689277676)*x_ref[0]+(0.467954875136)*x_ref[1]**o+(0.445711366876)*x_ref[1]+(-0.669973962002)*x_ref[2]**o+(0.181395269699)*x_ref[2]
        ref[0,0,1]=(0.353016816809)*x_ref[0]**o+(0.991829804601)*x_ref[0]+(-0.258315747172)*x_ref[1]**o+(0.543990055925)*x_ref[1]+(-0.769057570955)*x_ref[2]**o+(0.233571124807)*x_ref[2]
        ref[0,1,0]=(0.918139015102)*x_ref[0]**o+(-0.947988526027)*x_ref[0]+(0.331622282377)*x_ref[1]**o+(-0.777541428786)*x_ref[1]+(-0.664961721637)*x_ref[2]**o+(-0.484643482585)*x_ref[2]
        ref[0,1,1]=(0.140004476479)*x_ref[0]**o+(-0.610763924636)*x_ref[0]+(0.789189906539)*x_ref[1]**o+(-0.590757211504)*x_ref[1]+(-0.0725209360897)*x_ref[2]**o+(-0.766615663399)*x_ref[2]
        ref[1,0,0]=(0.725731333346)*x_ref[0]**o+(-0.131718022099)*x_ref[0]+(0.627777941816)*x_ref[1]**o+(-0.482136262357)*x_ref[1]+(0.413942360645)*x_ref[2]**o+(-0.987161567454)*x_ref[2]
        ref[1,0,1]=(0.975906575301)*x_ref[0]**o+(0.642044782609)*x_ref[0]+(0.670119948857)*x_ref[1]**o+(-0.93125168252)*x_ref[1]+(-0.553725567396)*x_ref[2]**o+(0.609585560905)*x_ref[2]
        ref[1,1,0]=(-0.073682338522)*x_ref[0]**o+(-0.391983504903)*x_ref[0]+(0.252884640925)*x_ref[1]**o+(0.474040995664)*x_ref[1]+(-0.763628395477)*x_ref[2]**o+(-0.753583810078)*x_ref[2]
        ref[1,1,1]=(-0.680881408584)*x_ref[0]**o+(-0.546411376016)*x_ref[0]+(0.85433594417)*x_ref[1]**o+(-0.990738365487)*x_ref[1]+(-0.202162243501)*x_ref[2]**o+(0.256411615615)*x_ref[2]
        ref[2,0,0]=(0.79289192889)*x_ref[0]**o+(0.243831874798)*x_ref[0]+(0.681415436664)*x_ref[1]**o+(0.223205387356)*x_ref[1]+(0.517155768633)*x_ref[2]**o+(-0.283058167273)*x_ref[2]
        ref[2,0,1]=(-0.608869102346)*x_ref[0]**o+(0.496990925284)*x_ref[0]+(0.925407693967)*x_ref[1]**o+(-0.358933716971)*x_ref[1]+(-0.328654182706)*x_ref[2]**o+(0.835965636453)*x_ref[2]
        ref[2,1,0]=(-0.830133878676)*x_ref[0]**o+(-0.123104434072)*x_ref[0]+(-0.662614589611)*x_ref[1]**o+(-0.288827930561)*x_ref[1]+(-0.411343484191)*x_ref[2]**o+(0.632500606896)*x_ref[2]
        ref[2,1,1]=(0.272639418094)*x_ref[0]**o+(-0.897989787007)*x_ref[0]+(-0.838744589589)*x_ref[1]**o+(0.492441488123)*x_ref[1]+(-0.634424482735)*x_ref[2]**o+(-0.509179635903)*x_ref[2]
        ref[3,0,0]=(-0.810786146104)*x_ref[0]**o+(-0.582309522585)*x_ref[0]+(-0.0537085055431)*x_ref[1]**o+(0.230345910939)*x_ref[1]+(0.482750960353)*x_ref[2]**o+(0.248433602737)*x_ref[2]
        ref[3,0,1]=(0.148959585193)*x_ref[0]**o+(0.222768449067)*x_ref[0]+(0.819219237595)*x_ref[1]**o+(0.386479159121)*x_ref[1]+(-0.0808464606991)*x_ref[2]**o+(-0.0578651077709)*x_ref[2]
        ref[3,1,0]=(-0.665745862321)*x_ref[0]**o+(-0.864332853905)*x_ref[0]+(-0.622626855999)*x_ref[1]**o+(-0.961729702089)*x_ref[1]+(-0.0528887263986)*x_ref[2]**o+(0.203930539621)*x_ref[2]
        ref[3,1,1]=(0.216834395994)*x_ref[0]**o+(0.14686771427)*x_ref[0]+(-0.634838825712)*x_ref[1]**o+(-0.0741875710555)*x_ref[1]+(0.49863557383)*x_ref[2]**o+(0.128134148051)*x_ref[2]
        ref[4,0,0]=(-0.0429185773768)*x_ref[0]**o+(-0.884386434806)*x_ref[0]+(0.590953595756)*x_ref[1]**o+(0.229045075384)*x_ref[1]+(0.289339613114)*x_ref[2]**o+(0.799614281102)*x_ref[2]
        ref[4,0,1]=(0.401237094734)*x_ref[0]**o+(0.374532696487)*x_ref[0]+(0.382784705093)*x_ref[1]**o+(-0.00752495997413)*x_ref[1]+(-0.693554885405)*x_ref[2]**o+(-0.0030745706685)*x_ref[2]
        ref[4,1,0]=(0.746124554182)*x_ref[0]**o+(-0.863459707092)*x_ref[0]+(0.586647115827)*x_ref[1]**o+(-0.94164686768)*x_ref[1]+(0.700937522698)*x_ref[2]**o+(-0.0464264819588)*x_ref[2]
        ref[4,1,1]=(0.33270904188)*x_ref[0]**o+(0.90735243167)*x_ref[0]+(-0.479962340673)*x_ref[1]**o+(-0.860540809179)*x_ref[1]+(0.396295831574)*x_ref[2]**o+(-0.47164342577)*x_ref[2]
        ref[5,0,0]=(-0.518396400617)*x_ref[0]**o+(0.0893385548496)*x_ref[0]+(0.695842336307)*x_ref[1]**o+(0.552511723893)*x_ref[1]+(0.0636197708084)*x_ref[2]**o+(0.0221143500177)*x_ref[2]
        ref[5,0,1]=(0.612931629659)*x_ref[0]**o+(-0.898561600235)*x_ref[0]+(0.231387395725)*x_ref[1]**o+(0.728648837106)*x_ref[1]+(-0.819722749156)*x_ref[2]**o+(-0.987410960592)*x_ref[2]
        ref[5,1,0]=(0.228541632142)*x_ref[0]**o+(0.0352301546347)*x_ref[0]+(-0.457859957282)*x_ref[1]**o+(0.126487825769)*x_ref[1]+(-0.685267391542)*x_ref[2]**o+(-0.695662688997)*x_ref[2]
        ref[5,1,1]=(0.892154759933)*x_ref[0]**o+(0.0404556267055)*x_ref[0]+(-0.509790912326)*x_ref[1]**o+(0.544457102938)*x_ref[1]+(-0.110692172028)*x_ref[2]**o+(-0.111481287291)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_ReducedFunctionOnContactZero_rank4(self):
      """
      tests interpolate for rank 4 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactZero onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactZero(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5, 3, 2),dim=dim)
      s=Data(0,(4, 5, 3, 2),w)
      ref=Data(0,(4, 5, 3, 2),w_ref)
      if dim==2:
        s[0,0,0,0]=(-0.0573009878147)*x[0]**o+(-0.350301437886)*x[0]+(0.253659046023)*x[1]**o+(0.658249302609)*x[1]
        s[0,0,0,1]=(0.830676080978)*x[0]**o+(-0.809626423382)*x[0]+(0.370613645219)*x[1]**o+(-0.172834681384)*x[1]
        s[0,0,1,0]=(-0.735478785962)*x[0]**o+(0.512724546806)*x[0]+(-0.717033770258)*x[1]**o+(-0.0946841286843)*x[1]
        s[0,0,1,1]=(-0.715337226505)*x[0]**o+(0.1253433121)*x[0]+(0.230909585243)*x[1]**o+(0.470778883904)*x[1]
        s[0,0,2,0]=(0.949793126568)*x[0]**o+(-0.12899598886)*x[0]+(-0.749733392851)*x[1]**o+(-0.340807421479)*x[1]
        s[0,0,2,1]=(0.261609659356)*x[0]**o+(0.336166427172)*x[0]+(0.657111532027)*x[1]**o+(-0.614814002143)*x[1]
        s[0,1,0,0]=(-0.0275060361172)*x[0]**o+(0.966092296743)*x[0]+(-0.532890777604)*x[1]**o+(0.517696025589)*x[1]
        s[0,1,0,1]=(0.00166569160565)*x[0]**o+(0.236625486597)*x[0]+(0.893101928693)*x[1]**o+(-0.574929038375)*x[1]
        s[0,1,1,0]=(0.872269789656)*x[0]**o+(-0.905740937959)*x[0]+(0.719538254619)*x[1]**o+(0.711635238494)*x[1]
        s[0,1,1,1]=(0.801801659725)*x[0]**o+(0.139019380599)*x[0]+(0.708524489966)*x[1]**o+(-0.77122453375)*x[1]
        s[0,1,2,0]=(-0.0481701009478)*x[0]**o+(0.167249653575)*x[0]+(0.0551704085528)*x[1]**o+(0.505944176073)*x[1]
        s[0,1,2,1]=(0.185898813821)*x[0]**o+(-0.750605793976)*x[0]+(-0.172006188497)*x[1]**o+(-0.0840963686282)*x[1]
        s[0,2,0,0]=(0.499076840583)*x[0]**o+(-0.230482058386)*x[0]+(0.522046309086)*x[1]**o+(-0.445372865956)*x[1]
        s[0,2,0,1]=(0.643144964505)*x[0]**o+(0.754412438054)*x[0]+(-0.527488849658)*x[1]**o+(-0.781951420648)*x[1]
        s[0,2,1,0]=(-0.428090027626)*x[0]**o+(0.941724629359)*x[0]+(-0.888361304631)*x[1]**o+(-0.193082322033)*x[1]
        s[0,2,1,1]=(0.737854520423)*x[0]**o+(-0.600462005647)*x[0]+(0.523944735668)*x[1]**o+(-0.88314904941)*x[1]
        s[0,2,2,0]=(0.401651360188)*x[0]**o+(0.605266809429)*x[0]+(0.114190031414)*x[1]**o+(-0.0554731448399)*x[1]
        s[0,2,2,1]=(0.49575503694)*x[0]**o+(0.842572557458)*x[0]+(-0.65557628937)*x[1]**o+(-0.110654902831)*x[1]
        s[0,3,0,0]=(-0.914580503709)*x[0]**o+(0.568110262697)*x[0]+(0.465998898456)*x[1]**o+(0.668680449802)*x[1]
        s[0,3,0,1]=(-0.406517611176)*x[0]**o+(0.751773194945)*x[0]+(-0.546709674237)*x[1]**o+(0.830905534576)*x[1]
        s[0,3,1,0]=(0.256170881332)*x[0]**o+(0.785200644094)*x[0]+(-0.0949323527814)*x[1]**o+(0.437799246842)*x[1]
        s[0,3,1,1]=(0.388926885643)*x[0]**o+(0.0649842722847)*x[0]+(-0.570138074228)*x[1]**o+(0.0125008958832)*x[1]
        s[0,3,2,0]=(-0.693124111229)*x[0]**o+(-0.135531989861)*x[0]+(-0.484364583498)*x[1]**o+(0.142028440685)*x[1]
        s[0,3,2,1]=(0.162852074905)*x[0]**o+(-0.211709648647)*x[0]+(-0.424964818278)*x[1]**o+(-0.0311289175964)*x[1]
        s[0,4,0,0]=(-0.893934248272)*x[0]**o+(-0.883334046545)*x[0]+(0.141127275532)*x[1]**o+(0.790251212849)*x[1]
        s[0,4,0,1]=(0.751030754129)*x[0]**o+(-0.0463112594115)*x[0]+(0.894754601054)*x[1]**o+(-0.986137408191)*x[1]
        s[0,4,1,0]=(-0.994216054583)*x[0]**o+(0.989450915377)*x[0]+(0.202383847504)*x[1]**o+(-0.829413728527)*x[1]
        s[0,4,1,1]=(-0.70581665488)*x[0]**o+(0.387180461053)*x[0]+(0.925669527701)*x[1]**o+(0.422052343171)*x[1]
        s[0,4,2,0]=(-0.498553799214)*x[0]**o+(0.70559513684)*x[0]+(-0.595097550839)*x[1]**o+(-0.177703117092)*x[1]
        s[0,4,2,1]=(-0.703437824074)*x[0]**o+(-0.574613675229)*x[0]+(-0.699959931134)*x[1]**o+(-0.503745753701)*x[1]
        s[1,0,0,0]=(-0.0673123845899)*x[0]**o+(-0.0603073393325)*x[0]+(0.724127339391)*x[1]**o+(-0.0875500989225)*x[1]
        s[1,0,0,1]=(0.0766117819496)*x[0]**o+(-0.489029658255)*x[0]+(0.602970700568)*x[1]**o+(0.624180485873)*x[1]
        s[1,0,1,0]=(0.537236166297)*x[0]**o+(0.805657082915)*x[0]+(-0.0263572051431)*x[1]**o+(0.683790156906)*x[1]
        s[1,0,1,1]=(0.55427198724)*x[0]**o+(0.631937641401)*x[0]+(-0.634744046324)*x[1]**o+(0.824382447181)*x[1]
        s[1,0,2,0]=(0.694551886505)*x[0]**o+(-0.123561028797)*x[0]+(-0.0237233372904)*x[1]**o+(-0.822938415295)*x[1]
        s[1,0,2,1]=(0.293944558832)*x[0]**o+(-0.34741820803)*x[0]+(-0.471269455638)*x[1]**o+(0.716959839747)*x[1]
        s[1,1,0,0]=(0.409876005614)*x[0]**o+(-0.322385353356)*x[0]+(0.362023900558)*x[1]**o+(-0.974915188765)*x[1]
        s[1,1,0,1]=(-0.823587457611)*x[0]**o+(-0.315566397595)*x[0]+(-0.681951254575)*x[1]**o+(-0.945803349426)*x[1]
        s[1,1,1,0]=(0.782950007967)*x[0]**o+(0.767220278425)*x[0]+(0.109277917483)*x[1]**o+(-0.336228598202)*x[1]
        s[1,1,1,1]=(-0.969811850973)*x[0]**o+(0.453588680715)*x[0]+(-0.296932776592)*x[1]**o+(-0.460668458023)*x[1]
        s[1,1,2,0]=(0.124616545015)*x[0]**o+(0.180238150583)*x[0]+(0.283723867146)*x[1]**o+(0.873774600692)*x[1]
        s[1,1,2,1]=(0.00334202091213)*x[0]**o+(-0.711821714868)*x[0]+(0.638247490899)*x[1]**o+(-0.409696943449)*x[1]
        s[1,2,0,0]=(-0.206885052591)*x[0]**o+(-0.677245253776)*x[0]+(-0.508637512178)*x[1]**o+(0.0862259084563)*x[1]
        s[1,2,0,1]=(0.402289447161)*x[0]**o+(0.242246771389)*x[0]+(0.775293224518)*x[1]**o+(-0.668881714475)*x[1]
        s[1,2,1,0]=(-0.094983969904)*x[0]**o+(0.615306379484)*x[0]+(-0.737839325648)*x[1]**o+(0.457810359747)*x[1]
        s[1,2,1,1]=(0.488668557736)*x[0]**o+(0.188170233286)*x[0]+(0.0947759637418)*x[1]**o+(-0.510206729955)*x[1]
        s[1,2,2,0]=(-0.102204043127)*x[0]**o+(0.584307889877)*x[0]+(-0.779746892074)*x[1]**o+(-0.767757863288)*x[1]
        s[1,2,2,1]=(-0.00230617324952)*x[0]**o+(-0.184632788675)*x[0]+(-0.779494086898)*x[1]**o+(-0.105311421755)*x[1]
        s[1,3,0,0]=(0.373541105531)*x[0]**o+(-0.895909586816)*x[0]+(0.853523284159)*x[1]**o+(0.915229883403)*x[1]
        s[1,3,0,1]=(-0.888315748473)*x[0]**o+(0.877859466924)*x[0]+(-0.830380928425)*x[1]**o+(-0.25922026388)*x[1]
        s[1,3,1,0]=(-0.944775455365)*x[0]**o+(0.694978915121)*x[0]+(0.00346060944841)*x[1]**o+(0.610096191934)*x[1]
        s[1,3,1,1]=(-0.175304757445)*x[0]**o+(0.883688655763)*x[0]+(-0.0260273718271)*x[1]**o+(0.440569935385)*x[1]
        s[1,3,2,0]=(0.973418372218)*x[0]**o+(-0.0725489149125)*x[0]+(-0.790861964708)*x[1]**o+(-0.764398676127)*x[1]
        s[1,3,2,1]=(0.188665821479)*x[0]**o+(-0.694573366343)*x[0]+(-0.860244164818)*x[1]**o+(0.642736876841)*x[1]
        s[1,4,0,0]=(-0.619221357105)*x[0]**o+(-0.0366612230467)*x[0]+(0.920044684933)*x[1]**o+(-0.901158565327)*x[1]
        s[1,4,0,1]=(-0.774227719954)*x[0]**o+(0.471966691541)*x[0]+(0.0706379157745)*x[1]**o+(-0.238713884878)*x[1]
        s[1,4,1,0]=(0.863551422475)*x[0]**o+(0.131078367733)*x[0]+(0.458326651999)*x[1]**o+(-0.750431841864)*x[1]
        s[1,4,1,1]=(0.105763306948)*x[0]**o+(-0.254538567936)*x[0]+(-0.559297954287)*x[1]**o+(0.21981759162)*x[1]
        s[1,4,2,0]=(0.922873574086)*x[0]**o+(-0.853230802788)*x[0]+(-0.982798222021)*x[1]**o+(-0.0866961947026)*x[1]
        s[1,4,2,1]=(-0.21664472336)*x[0]**o+(-0.915876043584)*x[0]+(0.6182964805)*x[1]**o+(0.665515072302)*x[1]
        s[2,0,0,0]=(-0.491230924471)*x[0]**o+(0.509670721146)*x[0]+(-0.44580759248)*x[1]**o+(-0.943168309946)*x[1]
        s[2,0,0,1]=(-0.0895802535424)*x[0]**o+(0.0780292649662)*x[0]+(-0.42293922141)*x[1]**o+(-0.76928974668)*x[1]
        s[2,0,1,0]=(-0.404348433578)*x[0]**o+(-0.986117091427)*x[0]+(0.562569938626)*x[1]**o+(-0.41119675761)*x[1]
        s[2,0,1,1]=(0.517587873308)*x[0]**o+(0.0731347448674)*x[0]+(0.830876774432)*x[1]**o+(-0.690590335528)*x[1]
        s[2,0,2,0]=(0.600931766711)*x[0]**o+(-0.786774940006)*x[0]+(-0.285975800855)*x[1]**o+(0.348122549519)*x[1]
        s[2,0,2,1]=(0.825033481951)*x[0]**o+(-0.744551393479)*x[0]+(0.452879660703)*x[1]**o+(-0.257025658518)*x[1]
        s[2,1,0,0]=(-0.0822184021881)*x[0]**o+(0.955001628077)*x[0]+(-0.878009442237)*x[1]**o+(-0.816970793701)*x[1]
        s[2,1,0,1]=(-0.151473282662)*x[0]**o+(0.937636279558)*x[0]+(-0.969793512643)*x[1]**o+(0.704581030148)*x[1]
        s[2,1,1,0]=(0.76728625844)*x[0]**o+(-0.90294390262)*x[0]+(-0.375443980814)*x[1]**o+(0.276219230561)*x[1]
        s[2,1,1,1]=(-0.994260771722)*x[0]**o+(0.64595057257)*x[0]+(0.486576661947)*x[1]**o+(0.731738848734)*x[1]
        s[2,1,2,0]=(-0.52048273513)*x[0]**o+(-0.51915057929)*x[0]+(0.184855975415)*x[1]**o+(0.417869171454)*x[1]
        s[2,1,2,1]=(-0.371019640816)*x[0]**o+(-0.617014410896)*x[0]+(0.725259821636)*x[1]**o+(-0.0393667759542)*x[1]
        s[2,2,0,0]=(0.070044943001)*x[0]**o+(-0.663066522147)*x[0]+(0.761006491068)*x[1]**o+(-0.137480662114)*x[1]
        s[2,2,0,1]=(-0.394035586896)*x[0]**o+(0.802334732427)*x[0]+(-0.191154645494)*x[1]**o+(-0.278490557185)*x[1]
        s[2,2,1,0]=(0.0664932052956)*x[0]**o+(0.938378455125)*x[0]+(0.997119543949)*x[1]**o+(-0.696820719616)*x[1]
        s[2,2,1,1]=(0.79758586389)*x[0]**o+(0.0393291272874)*x[0]+(0.644980805887)*x[1]**o+(0.61925875842)*x[1]
        s[2,2,2,0]=(0.0353810358089)*x[0]**o+(-0.0134652375886)*x[0]+(-0.942381983803)*x[1]**o+(0.288697669014)*x[1]
        s[2,2,2,1]=(-0.327577257504)*x[0]**o+(0.816374902745)*x[0]+(0.676853003934)*x[1]**o+(0.581394632094)*x[1]
        s[2,3,0,0]=(0.63036222221)*x[0]**o+(-0.899724810809)*x[0]+(0.276882104091)*x[1]**o+(-0.00448820301083)*x[1]
        s[2,3,0,1]=(-0.219899069159)*x[0]**o+(-0.439244974227)*x[0]+(-0.369366024932)*x[1]**o+(-0.161805301574)*x[1]
        s[2,3,1,0]=(0.165202872055)*x[0]**o+(-0.21521932039)*x[0]+(0.635188695229)*x[1]**o+(-0.254785217427)*x[1]
        s[2,3,1,1]=(0.743546803617)*x[0]**o+(0.876160636955)*x[0]+(-0.094730937905)*x[1]**o+(-0.898315912992)*x[1]
        s[2,3,2,0]=(0.946538368626)*x[0]**o+(0.715488078243)*x[0]+(0.830112020529)*x[1]**o+(-0.320605753644)*x[1]
        s[2,3,2,1]=(0.52106888624)*x[0]**o+(-0.938072665733)*x[0]+(-0.682830472037)*x[1]**o+(-0.787442211636)*x[1]
        s[2,4,0,0]=(0.802634158133)*x[0]**o+(0.350923640202)*x[0]+(-0.142397660222)*x[1]**o+(0.768827179039)*x[1]
        s[2,4,0,1]=(0.912207226957)*x[0]**o+(0.302955742414)*x[0]+(-0.125141454725)*x[1]**o+(0.115375955845)*x[1]
        s[2,4,1,0]=(0.88629189066)*x[0]**o+(-0.431145868333)*x[0]+(0.521817592458)*x[1]**o+(0.757372657079)*x[1]
        s[2,4,1,1]=(-0.34619470039)*x[0]**o+(0.0971941022378)*x[0]+(-0.235321219876)*x[1]**o+(0.919340221916)*x[1]
        s[2,4,2,0]=(-0.453744205858)*x[0]**o+(0.386107152737)*x[0]+(-0.53829794769)*x[1]**o+(-0.955985738202)*x[1]
        s[2,4,2,1]=(0.327564798989)*x[0]**o+(0.234176645121)*x[0]+(-0.647242055069)*x[1]**o+(-0.914279617484)*x[1]
        s[3,0,0,0]=(-0.765850994681)*x[0]**o+(0.0969021602734)*x[0]+(-0.003697715708)*x[1]**o+(0.642293903037)*x[1]
        s[3,0,0,1]=(-0.736912760299)*x[0]**o+(-0.496756935434)*x[0]+(0.00160221880808)*x[1]**o+(-0.0275314787811)*x[1]
        s[3,0,1,0]=(0.266302600859)*x[0]**o+(0.962232443249)*x[0]+(0.588094142167)*x[1]**o+(-0.740113379551)*x[1]
        s[3,0,1,1]=(-0.701204282707)*x[0]**o+(0.16377449778)*x[0]+(-0.636725536715)*x[1]**o+(0.925340403623)*x[1]
        s[3,0,2,0]=(-0.473492482064)*x[0]**o+(0.356976796364)*x[0]+(0.28490257392)*x[1]**o+(0.558859192591)*x[1]
        s[3,0,2,1]=(-0.398155056481)*x[0]**o+(-0.693010191754)*x[0]+(-0.457570700744)*x[1]**o+(0.899032980684)*x[1]
        s[3,1,0,0]=(0.676626616862)*x[0]**o+(-0.142127205425)*x[0]+(-0.0703163466342)*x[1]**o+(-0.603767804268)*x[1]
        s[3,1,0,1]=(0.901299559593)*x[0]**o+(-0.319119050502)*x[0]+(-0.230855076893)*x[1]**o+(0.923789250137)*x[1]
        s[3,1,1,0]=(0.0125691057912)*x[0]**o+(0.827401151454)*x[0]+(0.0881382970418)*x[1]**o+(0.12796232574)*x[1]
        s[3,1,1,1]=(-0.544917801861)*x[0]**o+(-0.163923780568)*x[0]+(0.107833995954)*x[1]**o+(-0.347470857919)*x[1]
        s[3,1,2,0]=(-0.127281534891)*x[0]**o+(0.410172113868)*x[0]+(0.516300378909)*x[1]**o+(-0.454031679856)*x[1]
        s[3,1,2,1]=(-0.797619552686)*x[0]**o+(0.0212633210895)*x[0]+(-0.0890328154993)*x[1]**o+(0.755392852372)*x[1]
        s[3,2,0,0]=(0.00976392699076)*x[0]**o+(-0.608813040196)*x[0]+(0.786997931081)*x[1]**o+(0.637450406747)*x[1]
        s[3,2,0,1]=(-0.594915968211)*x[0]**o+(0.909106492284)*x[0]+(0.209774113888)*x[1]**o+(0.926156671099)*x[1]
        s[3,2,1,0]=(0.226007901841)*x[0]**o+(-0.43873163217)*x[0]+(-0.414778768373)*x[1]**o+(0.681053132578)*x[1]
        s[3,2,1,1]=(0.733375971867)*x[0]**o+(0.861188515019)*x[0]+(0.036782797193)*x[1]**o+(0.145181529782)*x[1]
        s[3,2,2,0]=(0.635727397799)*x[0]**o+(0.874128725841)*x[0]+(-0.810169847222)*x[1]**o+(0.234369619159)*x[1]
        s[3,2,2,1]=(-0.137252742544)*x[0]**o+(0.817564886208)*x[0]+(0.524706682726)*x[1]**o+(0.381735970369)*x[1]
        s[3,3,0,0]=(0.537368746859)*x[0]**o+(-0.298760793455)*x[0]+(-0.890807690497)*x[1]**o+(-0.662222369966)*x[1]
        s[3,3,0,1]=(-0.67672238667)*x[0]**o+(-0.305431277061)*x[0]+(-0.435551021687)*x[1]**o+(0.0777923105041)*x[1]
        s[3,3,1,0]=(-0.163284186346)*x[0]**o+(-0.764058727495)*x[0]+(-0.255689425142)*x[1]**o+(0.285528254098)*x[1]
        s[3,3,1,1]=(-0.842634439814)*x[0]**o+(-0.194811378723)*x[0]+(0.814424721344)*x[1]**o+(-0.236826785563)*x[1]
        s[3,3,2,0]=(0.532936591284)*x[0]**o+(-0.920822945577)*x[0]+(-0.677841412043)*x[1]**o+(0.579981868038)*x[1]
        s[3,3,2,1]=(-0.470357673624)*x[0]**o+(-0.972798479564)*x[0]+(0.489692897505)*x[1]**o+(0.213937860142)*x[1]
        s[3,4,0,0]=(-0.169170544642)*x[0]**o+(0.543113611166)*x[0]+(0.536912197355)*x[1]**o+(0.304230092305)*x[1]
        s[3,4,0,1]=(-0.170701528868)*x[0]**o+(0.340606153319)*x[0]+(-0.470528666825)*x[1]**o+(-0.522028583228)*x[1]
        s[3,4,1,0]=(0.0194532320642)*x[0]**o+(0.335898702694)*x[0]+(-0.917025993458)*x[1]**o+(-0.539748486826)*x[1]
        s[3,4,1,1]=(-0.576465573138)*x[0]**o+(-0.906917910142)*x[0]+(-0.313000868178)*x[1]**o+(-0.20607256753)*x[1]
        s[3,4,2,0]=(-0.75394368612)*x[0]**o+(0.658866075222)*x[0]+(0.402200040518)*x[1]**o+(-0.544252067119)*x[1]
        s[3,4,2,1]=(0.0490609014064)*x[0]**o+(-0.873887848405)*x[0]+(0.91186722933)*x[1]**o+(0.375269571672)*x[1]
        ref[0,0,0,0]=(-0.0573009878147)*x_ref[0]**o+(-0.350301437886)*x_ref[0]+(0.253659046023)*x_ref[1]**o+(0.658249302609)*x_ref[1]
        ref[0,0,0,1]=(0.830676080978)*x_ref[0]**o+(-0.809626423382)*x_ref[0]+(0.370613645219)*x_ref[1]**o+(-0.172834681384)*x_ref[1]
        ref[0,0,1,0]=(-0.735478785962)*x_ref[0]**o+(0.512724546806)*x_ref[0]+(-0.717033770258)*x_ref[1]**o+(-0.0946841286843)*x_ref[1]
        ref[0,0,1,1]=(-0.715337226505)*x_ref[0]**o+(0.1253433121)*x_ref[0]+(0.230909585243)*x_ref[1]**o+(0.470778883904)*x_ref[1]
        ref[0,0,2,0]=(0.949793126568)*x_ref[0]**o+(-0.12899598886)*x_ref[0]+(-0.749733392851)*x_ref[1]**o+(-0.340807421479)*x_ref[1]
        ref[0,0,2,1]=(0.261609659356)*x_ref[0]**o+(0.336166427172)*x_ref[0]+(0.657111532027)*x_ref[1]**o+(-0.614814002143)*x_ref[1]
        ref[0,1,0,0]=(-0.0275060361172)*x_ref[0]**o+(0.966092296743)*x_ref[0]+(-0.532890777604)*x_ref[1]**o+(0.517696025589)*x_ref[1]
        ref[0,1,0,1]=(0.00166569160565)*x_ref[0]**o+(0.236625486597)*x_ref[0]+(0.893101928693)*x_ref[1]**o+(-0.574929038375)*x_ref[1]
        ref[0,1,1,0]=(0.872269789656)*x_ref[0]**o+(-0.905740937959)*x_ref[0]+(0.719538254619)*x_ref[1]**o+(0.711635238494)*x_ref[1]
        ref[0,1,1,1]=(0.801801659725)*x_ref[0]**o+(0.139019380599)*x_ref[0]+(0.708524489966)*x_ref[1]**o+(-0.77122453375)*x_ref[1]
        ref[0,1,2,0]=(-0.0481701009478)*x_ref[0]**o+(0.167249653575)*x_ref[0]+(0.0551704085528)*x_ref[1]**o+(0.505944176073)*x_ref[1]
        ref[0,1,2,1]=(0.185898813821)*x_ref[0]**o+(-0.750605793976)*x_ref[0]+(-0.172006188497)*x_ref[1]**o+(-0.0840963686282)*x_ref[1]
        ref[0,2,0,0]=(0.499076840583)*x_ref[0]**o+(-0.230482058386)*x_ref[0]+(0.522046309086)*x_ref[1]**o+(-0.445372865956)*x_ref[1]
        ref[0,2,0,1]=(0.643144964505)*x_ref[0]**o+(0.754412438054)*x_ref[0]+(-0.527488849658)*x_ref[1]**o+(-0.781951420648)*x_ref[1]
        ref[0,2,1,0]=(-0.428090027626)*x_ref[0]**o+(0.941724629359)*x_ref[0]+(-0.888361304631)*x_ref[1]**o+(-0.193082322033)*x_ref[1]
        ref[0,2,1,1]=(0.737854520423)*x_ref[0]**o+(-0.600462005647)*x_ref[0]+(0.523944735668)*x_ref[1]**o+(-0.88314904941)*x_ref[1]
        ref[0,2,2,0]=(0.401651360188)*x_ref[0]**o+(0.605266809429)*x_ref[0]+(0.114190031414)*x_ref[1]**o+(-0.0554731448399)*x_ref[1]
        ref[0,2,2,1]=(0.49575503694)*x_ref[0]**o+(0.842572557458)*x_ref[0]+(-0.65557628937)*x_ref[1]**o+(-0.110654902831)*x_ref[1]
        ref[0,3,0,0]=(-0.914580503709)*x_ref[0]**o+(0.568110262697)*x_ref[0]+(0.465998898456)*x_ref[1]**o+(0.668680449802)*x_ref[1]
        ref[0,3,0,1]=(-0.406517611176)*x_ref[0]**o+(0.751773194945)*x_ref[0]+(-0.546709674237)*x_ref[1]**o+(0.830905534576)*x_ref[1]
        ref[0,3,1,0]=(0.256170881332)*x_ref[0]**o+(0.785200644094)*x_ref[0]+(-0.0949323527814)*x_ref[1]**o+(0.437799246842)*x_ref[1]
        ref[0,3,1,1]=(0.388926885643)*x_ref[0]**o+(0.0649842722847)*x_ref[0]+(-0.570138074228)*x_ref[1]**o+(0.0125008958832)*x_ref[1]
        ref[0,3,2,0]=(-0.693124111229)*x_ref[0]**o+(-0.135531989861)*x_ref[0]+(-0.484364583498)*x_ref[1]**o+(0.142028440685)*x_ref[1]
        ref[0,3,2,1]=(0.162852074905)*x_ref[0]**o+(-0.211709648647)*x_ref[0]+(-0.424964818278)*x_ref[1]**o+(-0.0311289175964)*x_ref[1]
        ref[0,4,0,0]=(-0.893934248272)*x_ref[0]**o+(-0.883334046545)*x_ref[0]+(0.141127275532)*x_ref[1]**o+(0.790251212849)*x_ref[1]
        ref[0,4,0,1]=(0.751030754129)*x_ref[0]**o+(-0.0463112594115)*x_ref[0]+(0.894754601054)*x_ref[1]**o+(-0.986137408191)*x_ref[1]
        ref[0,4,1,0]=(-0.994216054583)*x_ref[0]**o+(0.989450915377)*x_ref[0]+(0.202383847504)*x_ref[1]**o+(-0.829413728527)*x_ref[1]
        ref[0,4,1,1]=(-0.70581665488)*x_ref[0]**o+(0.387180461053)*x_ref[0]+(0.925669527701)*x_ref[1]**o+(0.422052343171)*x_ref[1]
        ref[0,4,2,0]=(-0.498553799214)*x_ref[0]**o+(0.70559513684)*x_ref[0]+(-0.595097550839)*x_ref[1]**o+(-0.177703117092)*x_ref[1]
        ref[0,4,2,1]=(-0.703437824074)*x_ref[0]**o+(-0.574613675229)*x_ref[0]+(-0.699959931134)*x_ref[1]**o+(-0.503745753701)*x_ref[1]
        ref[1,0,0,0]=(-0.0673123845899)*x_ref[0]**o+(-0.0603073393325)*x_ref[0]+(0.724127339391)*x_ref[1]**o+(-0.0875500989225)*x_ref[1]
        ref[1,0,0,1]=(0.0766117819496)*x_ref[0]**o+(-0.489029658255)*x_ref[0]+(0.602970700568)*x_ref[1]**o+(0.624180485873)*x_ref[1]
        ref[1,0,1,0]=(0.537236166297)*x_ref[0]**o+(0.805657082915)*x_ref[0]+(-0.0263572051431)*x_ref[1]**o+(0.683790156906)*x_ref[1]
        ref[1,0,1,1]=(0.55427198724)*x_ref[0]**o+(0.631937641401)*x_ref[0]+(-0.634744046324)*x_ref[1]**o+(0.824382447181)*x_ref[1]
        ref[1,0,2,0]=(0.694551886505)*x_ref[0]**o+(-0.123561028797)*x_ref[0]+(-0.0237233372904)*x_ref[1]**o+(-0.822938415295)*x_ref[1]
        ref[1,0,2,1]=(0.293944558832)*x_ref[0]**o+(-0.34741820803)*x_ref[0]+(-0.471269455638)*x_ref[1]**o+(0.716959839747)*x_ref[1]
        ref[1,1,0,0]=(0.409876005614)*x_ref[0]**o+(-0.322385353356)*x_ref[0]+(0.362023900558)*x_ref[1]**o+(-0.974915188765)*x_ref[1]
        ref[1,1,0,1]=(-0.823587457611)*x_ref[0]**o+(-0.315566397595)*x_ref[0]+(-0.681951254575)*x_ref[1]**o+(-0.945803349426)*x_ref[1]
        ref[1,1,1,0]=(0.782950007967)*x_ref[0]**o+(0.767220278425)*x_ref[0]+(0.109277917483)*x_ref[1]**o+(-0.336228598202)*x_ref[1]
        ref[1,1,1,1]=(-0.969811850973)*x_ref[0]**o+(0.453588680715)*x_ref[0]+(-0.296932776592)*x_ref[1]**o+(-0.460668458023)*x_ref[1]
        ref[1,1,2,0]=(0.124616545015)*x_ref[0]**o+(0.180238150583)*x_ref[0]+(0.283723867146)*x_ref[1]**o+(0.873774600692)*x_ref[1]
        ref[1,1,2,1]=(0.00334202091213)*x_ref[0]**o+(-0.711821714868)*x_ref[0]+(0.638247490899)*x_ref[1]**o+(-0.409696943449)*x_ref[1]
        ref[1,2,0,0]=(-0.206885052591)*x_ref[0]**o+(-0.677245253776)*x_ref[0]+(-0.508637512178)*x_ref[1]**o+(0.0862259084563)*x_ref[1]
        ref[1,2,0,1]=(0.402289447161)*x_ref[0]**o+(0.242246771389)*x_ref[0]+(0.775293224518)*x_ref[1]**o+(-0.668881714475)*x_ref[1]
        ref[1,2,1,0]=(-0.094983969904)*x_ref[0]**o+(0.615306379484)*x_ref[0]+(-0.737839325648)*x_ref[1]**o+(0.457810359747)*x_ref[1]
        ref[1,2,1,1]=(0.488668557736)*x_ref[0]**o+(0.188170233286)*x_ref[0]+(0.0947759637418)*x_ref[1]**o+(-0.510206729955)*x_ref[1]
        ref[1,2,2,0]=(-0.102204043127)*x_ref[0]**o+(0.584307889877)*x_ref[0]+(-0.779746892074)*x_ref[1]**o+(-0.767757863288)*x_ref[1]
        ref[1,2,2,1]=(-0.00230617324952)*x_ref[0]**o+(-0.184632788675)*x_ref[0]+(-0.779494086898)*x_ref[1]**o+(-0.105311421755)*x_ref[1]
        ref[1,3,0,0]=(0.373541105531)*x_ref[0]**o+(-0.895909586816)*x_ref[0]+(0.853523284159)*x_ref[1]**o+(0.915229883403)*x_ref[1]
        ref[1,3,0,1]=(-0.888315748473)*x_ref[0]**o+(0.877859466924)*x_ref[0]+(-0.830380928425)*x_ref[1]**o+(-0.25922026388)*x_ref[1]
        ref[1,3,1,0]=(-0.944775455365)*x_ref[0]**o+(0.694978915121)*x_ref[0]+(0.00346060944841)*x_ref[1]**o+(0.610096191934)*x_ref[1]
        ref[1,3,1,1]=(-0.175304757445)*x_ref[0]**o+(0.883688655763)*x_ref[0]+(-0.0260273718271)*x_ref[1]**o+(0.440569935385)*x_ref[1]
        ref[1,3,2,0]=(0.973418372218)*x_ref[0]**o+(-0.0725489149125)*x_ref[0]+(-0.790861964708)*x_ref[1]**o+(-0.764398676127)*x_ref[1]
        ref[1,3,2,1]=(0.188665821479)*x_ref[0]**o+(-0.694573366343)*x_ref[0]+(-0.860244164818)*x_ref[1]**o+(0.642736876841)*x_ref[1]
        ref[1,4,0,0]=(-0.619221357105)*x_ref[0]**o+(-0.0366612230467)*x_ref[0]+(0.920044684933)*x_ref[1]**o+(-0.901158565327)*x_ref[1]
        ref[1,4,0,1]=(-0.774227719954)*x_ref[0]**o+(0.471966691541)*x_ref[0]+(0.0706379157745)*x_ref[1]**o+(-0.238713884878)*x_ref[1]
        ref[1,4,1,0]=(0.863551422475)*x_ref[0]**o+(0.131078367733)*x_ref[0]+(0.458326651999)*x_ref[1]**o+(-0.750431841864)*x_ref[1]
        ref[1,4,1,1]=(0.105763306948)*x_ref[0]**o+(-0.254538567936)*x_ref[0]+(-0.559297954287)*x_ref[1]**o+(0.21981759162)*x_ref[1]
        ref[1,4,2,0]=(0.922873574086)*x_ref[0]**o+(-0.853230802788)*x_ref[0]+(-0.982798222021)*x_ref[1]**o+(-0.0866961947026)*x_ref[1]
        ref[1,4,2,1]=(-0.21664472336)*x_ref[0]**o+(-0.915876043584)*x_ref[0]+(0.6182964805)*x_ref[1]**o+(0.665515072302)*x_ref[1]
        ref[2,0,0,0]=(-0.491230924471)*x_ref[0]**o+(0.509670721146)*x_ref[0]+(-0.44580759248)*x_ref[1]**o+(-0.943168309946)*x_ref[1]
        ref[2,0,0,1]=(-0.0895802535424)*x_ref[0]**o+(0.0780292649662)*x_ref[0]+(-0.42293922141)*x_ref[1]**o+(-0.76928974668)*x_ref[1]
        ref[2,0,1,0]=(-0.404348433578)*x_ref[0]**o+(-0.986117091427)*x_ref[0]+(0.562569938626)*x_ref[1]**o+(-0.41119675761)*x_ref[1]
        ref[2,0,1,1]=(0.517587873308)*x_ref[0]**o+(0.0731347448674)*x_ref[0]+(0.830876774432)*x_ref[1]**o+(-0.690590335528)*x_ref[1]
        ref[2,0,2,0]=(0.600931766711)*x_ref[0]**o+(-0.786774940006)*x_ref[0]+(-0.285975800855)*x_ref[1]**o+(0.348122549519)*x_ref[1]
        ref[2,0,2,1]=(0.825033481951)*x_ref[0]**o+(-0.744551393479)*x_ref[0]+(0.452879660703)*x_ref[1]**o+(-0.257025658518)*x_ref[1]
        ref[2,1,0,0]=(-0.0822184021881)*x_ref[0]**o+(0.955001628077)*x_ref[0]+(-0.878009442237)*x_ref[1]**o+(-0.816970793701)*x_ref[1]
        ref[2,1,0,1]=(-0.151473282662)*x_ref[0]**o+(0.937636279558)*x_ref[0]+(-0.969793512643)*x_ref[1]**o+(0.704581030148)*x_ref[1]
        ref[2,1,1,0]=(0.76728625844)*x_ref[0]**o+(-0.90294390262)*x_ref[0]+(-0.375443980814)*x_ref[1]**o+(0.276219230561)*x_ref[1]
        ref[2,1,1,1]=(-0.994260771722)*x_ref[0]**o+(0.64595057257)*x_ref[0]+(0.486576661947)*x_ref[1]**o+(0.731738848734)*x_ref[1]
        ref[2,1,2,0]=(-0.52048273513)*x_ref[0]**o+(-0.51915057929)*x_ref[0]+(0.184855975415)*x_ref[1]**o+(0.417869171454)*x_ref[1]
        ref[2,1,2,1]=(-0.371019640816)*x_ref[0]**o+(-0.617014410896)*x_ref[0]+(0.725259821636)*x_ref[1]**o+(-0.0393667759542)*x_ref[1]
        ref[2,2,0,0]=(0.070044943001)*x_ref[0]**o+(-0.663066522147)*x_ref[0]+(0.761006491068)*x_ref[1]**o+(-0.137480662114)*x_ref[1]
        ref[2,2,0,1]=(-0.394035586896)*x_ref[0]**o+(0.802334732427)*x_ref[0]+(-0.191154645494)*x_ref[1]**o+(-0.278490557185)*x_ref[1]
        ref[2,2,1,0]=(0.0664932052956)*x_ref[0]**o+(0.938378455125)*x_ref[0]+(0.997119543949)*x_ref[1]**o+(-0.696820719616)*x_ref[1]
        ref[2,2,1,1]=(0.79758586389)*x_ref[0]**o+(0.0393291272874)*x_ref[0]+(0.644980805887)*x_ref[1]**o+(0.61925875842)*x_ref[1]
        ref[2,2,2,0]=(0.0353810358089)*x_ref[0]**o+(-0.0134652375886)*x_ref[0]+(-0.942381983803)*x_ref[1]**o+(0.288697669014)*x_ref[1]
        ref[2,2,2,1]=(-0.327577257504)*x_ref[0]**o+(0.816374902745)*x_ref[0]+(0.676853003934)*x_ref[1]**o+(0.581394632094)*x_ref[1]
        ref[2,3,0,0]=(0.63036222221)*x_ref[0]**o+(-0.899724810809)*x_ref[0]+(0.276882104091)*x_ref[1]**o+(-0.00448820301083)*x_ref[1]
        ref[2,3,0,1]=(-0.219899069159)*x_ref[0]**o+(-0.439244974227)*x_ref[0]+(-0.369366024932)*x_ref[1]**o+(-0.161805301574)*x_ref[1]
        ref[2,3,1,0]=(0.165202872055)*x_ref[0]**o+(-0.21521932039)*x_ref[0]+(0.635188695229)*x_ref[1]**o+(-0.254785217427)*x_ref[1]
        ref[2,3,1,1]=(0.743546803617)*x_ref[0]**o+(0.876160636955)*x_ref[0]+(-0.094730937905)*x_ref[1]**o+(-0.898315912992)*x_ref[1]
        ref[2,3,2,0]=(0.946538368626)*x_ref[0]**o+(0.715488078243)*x_ref[0]+(0.830112020529)*x_ref[1]**o+(-0.320605753644)*x_ref[1]
        ref[2,3,2,1]=(0.52106888624)*x_ref[0]**o+(-0.938072665733)*x_ref[0]+(-0.682830472037)*x_ref[1]**o+(-0.787442211636)*x_ref[1]
        ref[2,4,0,0]=(0.802634158133)*x_ref[0]**o+(0.350923640202)*x_ref[0]+(-0.142397660222)*x_ref[1]**o+(0.768827179039)*x_ref[1]
        ref[2,4,0,1]=(0.912207226957)*x_ref[0]**o+(0.302955742414)*x_ref[0]+(-0.125141454725)*x_ref[1]**o+(0.115375955845)*x_ref[1]
        ref[2,4,1,0]=(0.88629189066)*x_ref[0]**o+(-0.431145868333)*x_ref[0]+(0.521817592458)*x_ref[1]**o+(0.757372657079)*x_ref[1]
        ref[2,4,1,1]=(-0.34619470039)*x_ref[0]**o+(0.0971941022378)*x_ref[0]+(-0.235321219876)*x_ref[1]**o+(0.919340221916)*x_ref[1]
        ref[2,4,2,0]=(-0.453744205858)*x_ref[0]**o+(0.386107152737)*x_ref[0]+(-0.53829794769)*x_ref[1]**o+(-0.955985738202)*x_ref[1]
        ref[2,4,2,1]=(0.327564798989)*x_ref[0]**o+(0.234176645121)*x_ref[0]+(-0.647242055069)*x_ref[1]**o+(-0.914279617484)*x_ref[1]
        ref[3,0,0,0]=(-0.765850994681)*x_ref[0]**o+(0.0969021602734)*x_ref[0]+(-0.003697715708)*x_ref[1]**o+(0.642293903037)*x_ref[1]
        ref[3,0,0,1]=(-0.736912760299)*x_ref[0]**o+(-0.496756935434)*x_ref[0]+(0.00160221880808)*x_ref[1]**o+(-0.0275314787811)*x_ref[1]
        ref[3,0,1,0]=(0.266302600859)*x_ref[0]**o+(0.962232443249)*x_ref[0]+(0.588094142167)*x_ref[1]**o+(-0.740113379551)*x_ref[1]
        ref[3,0,1,1]=(-0.701204282707)*x_ref[0]**o+(0.16377449778)*x_ref[0]+(-0.636725536715)*x_ref[1]**o+(0.925340403623)*x_ref[1]
        ref[3,0,2,0]=(-0.473492482064)*x_ref[0]**o+(0.356976796364)*x_ref[0]+(0.28490257392)*x_ref[1]**o+(0.558859192591)*x_ref[1]
        ref[3,0,2,1]=(-0.398155056481)*x_ref[0]**o+(-0.693010191754)*x_ref[0]+(-0.457570700744)*x_ref[1]**o+(0.899032980684)*x_ref[1]
        ref[3,1,0,0]=(0.676626616862)*x_ref[0]**o+(-0.142127205425)*x_ref[0]+(-0.0703163466342)*x_ref[1]**o+(-0.603767804268)*x_ref[1]
        ref[3,1,0,1]=(0.901299559593)*x_ref[0]**o+(-0.319119050502)*x_ref[0]+(-0.230855076893)*x_ref[1]**o+(0.923789250137)*x_ref[1]
        ref[3,1,1,0]=(0.0125691057912)*x_ref[0]**o+(0.827401151454)*x_ref[0]+(0.0881382970418)*x_ref[1]**o+(0.12796232574)*x_ref[1]
        ref[3,1,1,1]=(-0.544917801861)*x_ref[0]**o+(-0.163923780568)*x_ref[0]+(0.107833995954)*x_ref[1]**o+(-0.347470857919)*x_ref[1]
        ref[3,1,2,0]=(-0.127281534891)*x_ref[0]**o+(0.410172113868)*x_ref[0]+(0.516300378909)*x_ref[1]**o+(-0.454031679856)*x_ref[1]
        ref[3,1,2,1]=(-0.797619552686)*x_ref[0]**o+(0.0212633210895)*x_ref[0]+(-0.0890328154993)*x_ref[1]**o+(0.755392852372)*x_ref[1]
        ref[3,2,0,0]=(0.00976392699076)*x_ref[0]**o+(-0.608813040196)*x_ref[0]+(0.786997931081)*x_ref[1]**o+(0.637450406747)*x_ref[1]
        ref[3,2,0,1]=(-0.594915968211)*x_ref[0]**o+(0.909106492284)*x_ref[0]+(0.209774113888)*x_ref[1]**o+(0.926156671099)*x_ref[1]
        ref[3,2,1,0]=(0.226007901841)*x_ref[0]**o+(-0.43873163217)*x_ref[0]+(-0.414778768373)*x_ref[1]**o+(0.681053132578)*x_ref[1]
        ref[3,2,1,1]=(0.733375971867)*x_ref[0]**o+(0.861188515019)*x_ref[0]+(0.036782797193)*x_ref[1]**o+(0.145181529782)*x_ref[1]
        ref[3,2,2,0]=(0.635727397799)*x_ref[0]**o+(0.874128725841)*x_ref[0]+(-0.810169847222)*x_ref[1]**o+(0.234369619159)*x_ref[1]
        ref[3,2,2,1]=(-0.137252742544)*x_ref[0]**o+(0.817564886208)*x_ref[0]+(0.524706682726)*x_ref[1]**o+(0.381735970369)*x_ref[1]
        ref[3,3,0,0]=(0.537368746859)*x_ref[0]**o+(-0.298760793455)*x_ref[0]+(-0.890807690497)*x_ref[1]**o+(-0.662222369966)*x_ref[1]
        ref[3,3,0,1]=(-0.67672238667)*x_ref[0]**o+(-0.305431277061)*x_ref[0]+(-0.435551021687)*x_ref[1]**o+(0.0777923105041)*x_ref[1]
        ref[3,3,1,0]=(-0.163284186346)*x_ref[0]**o+(-0.764058727495)*x_ref[0]+(-0.255689425142)*x_ref[1]**o+(0.285528254098)*x_ref[1]
        ref[3,3,1,1]=(-0.842634439814)*x_ref[0]**o+(-0.194811378723)*x_ref[0]+(0.814424721344)*x_ref[1]**o+(-0.236826785563)*x_ref[1]
        ref[3,3,2,0]=(0.532936591284)*x_ref[0]**o+(-0.920822945577)*x_ref[0]+(-0.677841412043)*x_ref[1]**o+(0.579981868038)*x_ref[1]
        ref[3,3,2,1]=(-0.470357673624)*x_ref[0]**o+(-0.972798479564)*x_ref[0]+(0.489692897505)*x_ref[1]**o+(0.213937860142)*x_ref[1]
        ref[3,4,0,0]=(-0.169170544642)*x_ref[0]**o+(0.543113611166)*x_ref[0]+(0.536912197355)*x_ref[1]**o+(0.304230092305)*x_ref[1]
        ref[3,4,0,1]=(-0.170701528868)*x_ref[0]**o+(0.340606153319)*x_ref[0]+(-0.470528666825)*x_ref[1]**o+(-0.522028583228)*x_ref[1]
        ref[3,4,1,0]=(0.0194532320642)*x_ref[0]**o+(0.335898702694)*x_ref[0]+(-0.917025993458)*x_ref[1]**o+(-0.539748486826)*x_ref[1]
        ref[3,4,1,1]=(-0.576465573138)*x_ref[0]**o+(-0.906917910142)*x_ref[0]+(-0.313000868178)*x_ref[1]**o+(-0.20607256753)*x_ref[1]
        ref[3,4,2,0]=(-0.75394368612)*x_ref[0]**o+(0.658866075222)*x_ref[0]+(0.402200040518)*x_ref[1]**o+(-0.544252067119)*x_ref[1]
        ref[3,4,2,1]=(0.0490609014064)*x_ref[0]**o+(-0.873887848405)*x_ref[0]+(0.91186722933)*x_ref[1]**o+(0.375269571672)*x_ref[1]
      else:
        s[0,0,0,0]=(-0.853604830323)*x[0]**o+(0.378465791865)*x[0]+(-0.83364075876)*x[1]**o+(0.212018195408)*x[1]+(0.48856925163)*x[2]**o+(-0.179831169531)*x[2]
        s[0,0,0,1]=(0.868673673397)*x[0]**o+(-0.238560476072)*x[0]+(0.828785612257)*x[1]**o+(0.176759782363)*x[1]+(0.525888840201)*x[2]**o+(0.436158744893)*x[2]
        s[0,0,1,0]=(0.887888476899)*x[0]**o+(0.458452579078)*x[0]+(0.572312971484)*x[1]**o+(0.694648297654)*x[1]+(0.244770581638)*x[2]**o+(-0.326433505221)*x[2]
        s[0,0,1,1]=(0.958595740398)*x[0]**o+(-0.448016561436)*x[0]+(-0.898608139656)*x[1]**o+(0.0189044802348)*x[1]+(0.664802638624)*x[2]**o+(0.675631565244)*x[2]
        s[0,0,2,0]=(-0.0968613941718)*x[0]**o+(-0.580198871721)*x[0]+(-0.695139695389)*x[1]**o+(0.670627088913)*x[1]+(-0.466392817977)*x[2]**o+(-0.414367914263)*x[2]
        s[0,0,2,1]=(0.0563724518255)*x[0]**o+(0.141716006991)*x[0]+(-0.889611317888)*x[1]**o+(0.274193692615)*x[1]+(0.81766066569)*x[2]**o+(0.260242356947)*x[2]
        s[0,1,0,0]=(0.729916171245)*x[0]**o+(-0.0512883787314)*x[0]+(-0.395879068642)*x[1]**o+(0.65153799151)*x[1]+(-0.90747241683)*x[2]**o+(0.556056661876)*x[2]
        s[0,1,0,1]=(-0.336962711878)*x[0]**o+(0.494126376731)*x[0]+(-0.0180540415242)*x[1]**o+(-0.281208191895)*x[1]+(-0.891281821502)*x[2]**o+(0.434438114521)*x[2]
        s[0,1,1,0]=(-0.0203012608797)*x[0]**o+(-0.592593200931)*x[0]+(-0.908091999823)*x[1]**o+(0.917265173937)*x[1]+(0.0113338093354)*x[2]**o+(0.832522528967)*x[2]
        s[0,1,1,1]=(-0.287082151578)*x[0]**o+(-0.882563505538)*x[0]+(-0.805577758903)*x[1]**o+(-0.0877632778656)*x[1]+(-0.4141073805)*x[2]**o+(-0.0809945294401)*x[2]
        s[0,1,2,0]=(-0.745887649438)*x[0]**o+(0.105423480913)*x[0]+(0.642374896965)*x[1]**o+(0.479730302425)*x[1]+(0.334298930575)*x[2]**o+(0.214588275036)*x[2]
        s[0,1,2,1]=(-0.987939537144)*x[0]**o+(-0.889107799492)*x[0]+(-0.133927268357)*x[1]**o+(-0.991450009918)*x[1]+(-0.201645805359)*x[2]**o+(0.481104989077)*x[2]
        s[0,2,0,0]=(0.535918222555)*x[0]**o+(0.243041294608)*x[0]+(-0.205082148634)*x[1]**o+(0.233470577382)*x[1]+(0.398471417689)*x[2]**o+(-0.56437758006)*x[2]
        s[0,2,0,1]=(0.374626206936)*x[0]**o+(-0.800840698958)*x[0]+(0.336427483065)*x[1]**o+(0.143368857985)*x[1]+(0.822396957468)*x[2]**o+(-0.288884862626)*x[2]
        s[0,2,1,0]=(-0.152217915896)*x[0]**o+(0.868324413095)*x[0]+(0.93760108412)*x[1]**o+(-0.584628102458)*x[1]+(-0.678653990131)*x[2]**o+(-0.130768977951)*x[2]
        s[0,2,1,1]=(-0.962809494754)*x[0]**o+(0.057963920559)*x[0]+(-0.68685683212)*x[1]**o+(0.869986883492)*x[1]+(-0.193987500788)*x[2]**o+(0.0357315899177)*x[2]
        s[0,2,2,0]=(-0.296222468619)*x[0]**o+(0.853293489381)*x[0]+(0.886622498642)*x[1]**o+(-0.22952370037)*x[1]+(0.444962301355)*x[2]**o+(0.498735104677)*x[2]
        s[0,2,2,1]=(-0.556796221789)*x[0]**o+(-0.98717630759)*x[0]+(-0.56578841851)*x[1]**o+(-0.769428379953)*x[1]+(0.41417947942)*x[2]**o+(-0.99075609451)*x[2]
        s[0,3,0,0]=(-0.312173122533)*x[0]**o+(-0.0183061811911)*x[0]+(0.632028466217)*x[1]**o+(-0.516798544207)*x[1]+(0.467776726567)*x[2]**o+(-0.0750898474112)*x[2]
        s[0,3,0,1]=(0.151818577102)*x[0]**o+(0.275659634397)*x[0]+(-0.695866418429)*x[1]**o+(-0.158484337329)*x[1]+(-0.717635621103)*x[2]**o+(0.96218471765)*x[2]
        s[0,3,1,0]=(-0.959269035858)*x[0]**o+(-0.694952577107)*x[0]+(0.133713794285)*x[1]**o+(0.945801566586)*x[1]+(0.480976843012)*x[2]**o+(0.514096820167)*x[2]
        s[0,3,1,1]=(-0.696429878723)*x[0]**o+(0.844116956105)*x[0]+(0.986596486924)*x[1]**o+(0.213400513251)*x[1]+(0.168567391983)*x[2]**o+(-0.96577037709)*x[2]
        s[0,3,2,0]=(0.272554167042)*x[0]**o+(-0.134445947881)*x[0]+(-0.620927727232)*x[1]**o+(0.774446825459)*x[1]+(-0.140360641548)*x[2]**o+(-0.393011913595)*x[2]
        s[0,3,2,1]=(0.973937360802)*x[0]**o+(-0.780203033305)*x[0]+(0.62508420402)*x[1]**o+(-0.957359234026)*x[1]+(0.301032273116)*x[2]**o+(0.238130119107)*x[2]
        s[0,4,0,0]=(0.550167138562)*x[0]**o+(-0.975271434354)*x[0]+(-0.90226839553)*x[1]**o+(0.765360109563)*x[1]+(-0.0539842939871)*x[2]**o+(-0.992811018253)*x[2]
        s[0,4,0,1]=(0.115680644235)*x[0]**o+(0.6504959069)*x[0]+(0.0711117194347)*x[1]**o+(0.765247742051)*x[1]+(0.651478097229)*x[2]**o+(0.0605973413193)*x[2]
        s[0,4,1,0]=(-0.339495686607)*x[0]**o+(0.98910452173)*x[0]+(-0.97593906201)*x[1]**o+(-0.961098427995)*x[1]+(-0.970981798475)*x[2]**o+(-0.655846307864)*x[2]
        s[0,4,1,1]=(-0.676422431554)*x[0]**o+(0.17779634387)*x[0]+(-0.101241446408)*x[1]**o+(0.592319052036)*x[1]+(-0.328471629455)*x[2]**o+(0.842981957639)*x[2]
        s[0,4,2,0]=(0.475154370559)*x[0]**o+(-0.861452809594)*x[0]+(0.021220095967)*x[1]**o+(-0.167324332634)*x[1]+(-0.420334858996)*x[2]**o+(-0.243503287995)*x[2]
        s[0,4,2,1]=(0.507818170756)*x[0]**o+(0.0305556062843)*x[0]+(0.657465103712)*x[1]**o+(-0.133306218785)*x[1]+(0.596182931411)*x[2]**o+(0.26354220091)*x[2]
        s[1,0,0,0]=(0.141665798979)*x[0]**o+(0.257839463098)*x[0]+(0.20842719092)*x[1]**o+(0.0606481804876)*x[1]+(0.564188524366)*x[2]**o+(0.276431563089)*x[2]
        s[1,0,0,1]=(-0.475871120292)*x[0]**o+(-0.945285882278)*x[0]+(-0.852943036141)*x[1]**o+(-0.393053044641)*x[1]+(0.894360138187)*x[2]**o+(0.945580362088)*x[2]
        s[1,0,1,0]=(-0.269113046186)*x[0]**o+(0.979226351783)*x[0]+(0.59588973168)*x[1]**o+(0.709182601617)*x[1]+(0.203809169118)*x[2]**o+(0.843792678336)*x[2]
        s[1,0,1,1]=(0.625152197275)*x[0]**o+(0.700601491781)*x[0]+(-0.556266788732)*x[1]**o+(0.621296929469)*x[1]+(0.370627217553)*x[2]**o+(-0.86981081047)*x[2]
        s[1,0,2,0]=(0.337314024742)*x[0]**o+(0.4936522561)*x[0]+(-0.121004948433)*x[1]**o+(0.682816106721)*x[1]+(0.135016952475)*x[2]**o+(0.447788511803)*x[2]
        s[1,0,2,1]=(0.627048609871)*x[0]**o+(0.464430671751)*x[0]+(0.248071099972)*x[1]**o+(0.49700045944)*x[1]+(0.764133255754)*x[2]**o+(-0.407441090409)*x[2]
        s[1,1,0,0]=(-0.891648124769)*x[0]**o+(-0.852500694418)*x[0]+(-0.501966394474)*x[1]**o+(-0.951197952585)*x[1]+(-0.293731401112)*x[2]**o+(0.340542703849)*x[2]
        s[1,1,0,1]=(0.047745963453)*x[0]**o+(0.605499497949)*x[0]+(0.603025931833)*x[1]**o+(0.412711498009)*x[1]+(-0.706970380941)*x[2]**o+(-0.865479677059)*x[2]
        s[1,1,1,0]=(-0.809326436683)*x[0]**o+(0.50103064436)*x[0]+(-0.726029943072)*x[1]**o+(-0.0136545501065)*x[1]+(0.869298450797)*x[2]**o+(-0.0539795856427)*x[2]
        s[1,1,1,1]=(0.822265437858)*x[0]**o+(-0.13686562101)*x[0]+(-0.921877204478)*x[1]**o+(0.436216049238)*x[1]+(0.167449871886)*x[2]**o+(-0.376043518724)*x[2]
        s[1,1,2,0]=(-0.240904110506)*x[0]**o+(-0.709486977651)*x[0]+(0.589748137401)*x[1]**o+(0.997759434219)*x[1]+(0.211495210823)*x[2]**o+(0.857954370359)*x[2]
        s[1,1,2,1]=(-0.872650055395)*x[0]**o+(0.488392921139)*x[0]+(0.456778341472)*x[1]**o+(0.533360514682)*x[1]+(0.660463672454)*x[2]**o+(-0.287728736994)*x[2]
        s[1,2,0,0]=(-0.56865288172)*x[0]**o+(-0.257707278342)*x[0]+(0.492925973387)*x[1]**o+(-0.433521245223)*x[1]+(-0.214711127244)*x[2]**o+(-0.578235847487)*x[2]
        s[1,2,0,1]=(0.502020345167)*x[0]**o+(-0.898730931974)*x[0]+(0.152379872963)*x[1]**o+(0.963247300695)*x[1]+(-0.935219139549)*x[2]**o+(-0.322095550311)*x[2]
        s[1,2,1,0]=(0.755425499434)*x[0]**o+(0.922722006611)*x[0]+(-0.854913831787)*x[1]**o+(0.28200116496)*x[1]+(0.654754619366)*x[2]**o+(-0.87246359532)*x[2]
        s[1,2,1,1]=(0.476533428226)*x[0]**o+(-0.21427962663)*x[0]+(-0.42807986959)*x[1]**o+(-0.453270655163)*x[1]+(-0.0618352146195)*x[2]**o+(-0.0578564702081)*x[2]
        s[1,2,2,0]=(0.836242039415)*x[0]**o+(0.0582519660745)*x[0]+(-0.736245953793)*x[1]**o+(-0.0216606630383)*x[1]+(-0.241691127038)*x[2]**o+(-0.273769285415)*x[2]
        s[1,2,2,1]=(0.956981101891)*x[0]**o+(-0.619600309731)*x[0]+(0.255999571352)*x[1]**o+(0.931443107924)*x[1]+(0.803455662831)*x[2]**o+(0.587378690917)*x[2]
        s[1,3,0,0]=(0.84725602389)*x[0]**o+(0.099929556134)*x[0]+(0.918513623927)*x[1]**o+(-0.540239895987)*x[1]+(0.543336053141)*x[2]**o+(-0.35339148993)*x[2]
        s[1,3,0,1]=(0.0790066953583)*x[0]**o+(-0.622984122413)*x[0]+(0.844477077556)*x[1]**o+(-0.843967484949)*x[1]+(0.382810149795)*x[2]**o+(0.693483174672)*x[2]
        s[1,3,1,0]=(-0.606850566406)*x[0]**o+(-0.613588487235)*x[0]+(-0.982801043706)*x[1]**o+(-0.200157044491)*x[1]+(0.865010142689)*x[2]**o+(-0.926404146796)*x[2]
        s[1,3,1,1]=(-0.926332516404)*x[0]**o+(-0.644403030771)*x[0]+(-0.391941644842)*x[1]**o+(-0.628555138555)*x[1]+(0.584646630734)*x[2]**o+(0.627561535294)*x[2]
        s[1,3,2,0]=(0.777960693446)*x[0]**o+(-0.580698013493)*x[0]+(-0.061588850878)*x[1]**o+(0.909167097211)*x[1]+(-0.346386067681)*x[2]**o+(-0.698593831635)*x[2]
        s[1,3,2,1]=(-0.25740731816)*x[0]**o+(0.0609560824945)*x[0]+(-0.866636970162)*x[1]**o+(0.352496912986)*x[1]+(0.212276243322)*x[2]**o+(-0.29499231054)*x[2]
        s[1,4,0,0]=(0.176828476382)*x[0]**o+(0.592116728712)*x[0]+(-0.969737040463)*x[1]**o+(-0.0140144291993)*x[1]+(-0.203791892462)*x[2]**o+(0.687043221294)*x[2]
        s[1,4,0,1]=(-0.0673562914955)*x[0]**o+(0.104813124019)*x[0]+(0.766386480162)*x[1]**o+(-0.499215759851)*x[1]+(0.163490109505)*x[2]**o+(0.0477449791182)*x[2]
        s[1,4,1,0]=(-0.186436883357)*x[0]**o+(0.771658464504)*x[0]+(0.115845567912)*x[1]**o+(-0.573016245745)*x[1]+(0.759680038826)*x[2]**o+(0.0660145528929)*x[2]
        s[1,4,1,1]=(-0.983746295741)*x[0]**o+(-0.370146292969)*x[0]+(-0.276398856773)*x[1]**o+(-0.765483196375)*x[1]+(-0.0802845394538)*x[2]**o+(0.762085198906)*x[2]
        s[1,4,2,0]=(-0.437323501168)*x[0]**o+(-0.076976193125)*x[0]+(0.780179691457)*x[1]**o+(0.941748527645)*x[1]+(0.921220023779)*x[2]**o+(-0.611350278287)*x[2]
        s[1,4,2,1]=(-0.811077839585)*x[0]**o+(-0.0781699097656)*x[0]+(-0.955979549084)*x[1]**o+(0.742581422414)*x[1]+(-0.905829236752)*x[2]**o+(-0.671257774729)*x[2]
        s[2,0,0,0]=(0.3602498958)*x[0]**o+(0.567952303965)*x[0]+(-0.72126509422)*x[1]**o+(0.652912038045)*x[1]+(-0.448081819441)*x[2]**o+(-0.628140847819)*x[2]
        s[2,0,0,1]=(0.823191679062)*x[0]**o+(-0.12280206123)*x[0]+(0.202149797631)*x[1]**o+(-0.751457177895)*x[1]+(0.367057556408)*x[2]**o+(0.71681510921)*x[2]
        s[2,0,1,0]=(-0.553277801352)*x[0]**o+(-0.695114786088)*x[0]+(-0.506695185331)*x[1]**o+(0.947900606544)*x[1]+(0.393574900465)*x[2]**o+(-0.131907977332)*x[2]
        s[2,0,1,1]=(-0.683260439471)*x[0]**o+(0.108921790681)*x[0]+(0.694361429808)*x[1]**o+(-0.879827908202)*x[1]+(-0.768400141399)*x[2]**o+(-0.869535466479)*x[2]
        s[2,0,2,0]=(-0.377844410367)*x[0]**o+(-0.486876364642)*x[0]+(-0.139990170975)*x[1]**o+(0.521294626989)*x[1]+(0.384721944946)*x[2]**o+(0.912206678795)*x[2]
        s[2,0,2,1]=(0.624159317849)*x[0]**o+(0.910540130065)*x[0]+(-0.879166404339)*x[1]**o+(0.710229335366)*x[1]+(-0.756806360688)*x[2]**o+(-0.169809484076)*x[2]
        s[2,1,0,0]=(-0.887833054092)*x[0]**o+(-0.541562581783)*x[0]+(-0.439734700726)*x[1]**o+(-0.294103567325)*x[1]+(0.621145429347)*x[2]**o+(0.65618994558)*x[2]
        s[2,1,0,1]=(-0.199707709698)*x[0]**o+(0.952895238716)*x[0]+(-0.113807788254)*x[1]**o+(-0.209497731554)*x[1]+(-0.201935349119)*x[2]**o+(0.896172767252)*x[2]
        s[2,1,1,0]=(0.961864846935)*x[0]**o+(0.34037006989)*x[0]+(0.97866208125)*x[1]**o+(0.949937962054)*x[1]+(-0.943237740253)*x[2]**o+(0.492751804379)*x[2]
        s[2,1,1,1]=(0.0364913119154)*x[0]**o+(-0.706276345936)*x[0]+(-0.0562330252965)*x[1]**o+(-0.919546510941)*x[1]+(0.376680654917)*x[2]**o+(-0.184153093526)*x[2]
        s[2,1,2,0]=(-0.135265768495)*x[0]**o+(0.650164259074)*x[0]+(0.224627083284)*x[1]**o+(0.918537471241)*x[1]+(0.662719594601)*x[2]**o+(-0.4451558313)*x[2]
        s[2,1,2,1]=(0.968484347906)*x[0]**o+(0.261215594011)*x[0]+(-0.478839261077)*x[1]**o+(-0.806554334141)*x[1]+(0.576441463783)*x[2]**o+(-0.407338671799)*x[2]
        s[2,2,0,0]=(-0.691105566226)*x[0]**o+(0.860347396649)*x[0]+(0.354066286829)*x[1]**o+(-0.0402175279153)*x[1]+(0.415237302648)*x[2]**o+(0.905209054794)*x[2]
        s[2,2,0,1]=(-0.548678761348)*x[0]**o+(-0.294803736974)*x[0]+(-0.960170319863)*x[1]**o+(-0.27333394559)*x[1]+(0.866348038561)*x[2]**o+(-0.977985070269)*x[2]
        s[2,2,1,0]=(-0.942843839038)*x[0]**o+(-0.868858708934)*x[0]+(-0.208988345792)*x[1]**o+(-0.628702405302)*x[1]+(0.856538967932)*x[2]**o+(-0.940751252093)*x[2]
        s[2,2,1,1]=(0.493835163998)*x[0]**o+(0.236452180515)*x[0]+(0.92903313506)*x[1]**o+(0.817122194244)*x[1]+(-0.498919897775)*x[2]**o+(0.177040227977)*x[2]
        s[2,2,2,0]=(-0.750931164998)*x[0]**o+(0.859659424304)*x[0]+(0.415426727903)*x[1]**o+(-0.468797574814)*x[1]+(-0.294847888148)*x[2]**o+(-0.992000733445)*x[2]
        s[2,2,2,1]=(0.0182583094659)*x[0]**o+(-0.830672513808)*x[0]+(0.37453609414)*x[1]**o+(-0.57449821779)*x[1]+(0.134490719543)*x[2]**o+(-0.609352246473)*x[2]
        s[2,3,0,0]=(-0.872131310391)*x[0]**o+(0.0825093683843)*x[0]+(-0.982804116245)*x[1]**o+(0.22341420851)*x[1]+(-0.231362455053)*x[2]**o+(-0.904631856625)*x[2]
        s[2,3,0,1]=(-0.0582824887741)*x[0]**o+(-0.399156766464)*x[0]+(0.0772313065438)*x[1]**o+(0.00426296268846)*x[1]+(-0.803184293158)*x[2]**o+(-0.17731453976)*x[2]
        s[2,3,1,0]=(0.252580558693)*x[0]**o+(-0.670939935472)*x[0]+(-0.396636904351)*x[1]**o+(-0.774693006552)*x[1]+(0.451637181152)*x[2]**o+(-0.463784221351)*x[2]
        s[2,3,1,1]=(0.126767645109)*x[0]**o+(-0.395030113635)*x[0]+(-0.955454976686)*x[1]**o+(-0.710220994617)*x[1]+(-0.397874215052)*x[2]**o+(-0.0217615150269)*x[2]
        s[2,3,2,0]=(0.990529883771)*x[0]**o+(0.278013247448)*x[0]+(-0.430442262972)*x[1]**o+(-0.720399995623)*x[1]+(0.610560781511)*x[2]**o+(-0.264692262808)*x[2]
        s[2,3,2,1]=(-0.776369460759)*x[0]**o+(0.397198841934)*x[0]+(-0.604527190952)*x[1]**o+(0.94368439705)*x[1]+(-0.88032288089)*x[2]**o+(0.989155014613)*x[2]
        s[2,4,0,0]=(0.457573740068)*x[0]**o+(-0.352003681775)*x[0]+(0.996428644418)*x[1]**o+(-0.348011873806)*x[1]+(-0.58023500034)*x[2]**o+(0.727228005186)*x[2]
        s[2,4,0,1]=(0.50989726873)*x[0]**o+(0.83338864798)*x[0]+(-0.111380969484)*x[1]**o+(-0.278057155647)*x[1]+(0.0022286233284)*x[2]**o+(0.841662666668)*x[2]
        s[2,4,1,0]=(-0.901603581146)*x[0]**o+(0.475205009348)*x[0]+(-0.430007910717)*x[1]**o+(-0.114746630672)*x[1]+(0.847548245931)*x[2]**o+(-0.539570230562)*x[2]
        s[2,4,1,1]=(0.539830165452)*x[0]**o+(-0.415758645187)*x[0]+(0.712970160554)*x[1]**o+(0.833164796207)*x[1]+(-0.65521876468)*x[2]**o+(-0.478072278167)*x[2]
        s[2,4,2,0]=(0.651879536878)*x[0]**o+(0.782082841616)*x[0]+(0.0416994850732)*x[1]**o+(0.0989265074417)*x[1]+(0.417171388449)*x[2]**o+(0.204544481473)*x[2]
        s[2,4,2,1]=(-0.957695309447)*x[0]**o+(0.41057815979)*x[0]+(0.847095166862)*x[1]**o+(0.937811945321)*x[1]+(-0.794606224039)*x[2]**o+(0.021545807724)*x[2]
        s[3,0,0,0]=(-0.521599558272)*x[0]**o+(-0.640095375476)*x[0]+(-0.51854438692)*x[1]**o+(-0.87210774891)*x[1]+(-0.693848689324)*x[2]**o+(-0.261635833944)*x[2]
        s[3,0,0,1]=(0.620197170573)*x[0]**o+(-0.841665901801)*x[0]+(-0.546759437483)*x[1]**o+(-0.168579145925)*x[1]+(-0.576567932552)*x[2]**o+(-0.645039556295)*x[2]
        s[3,0,1,0]=(0.581444084796)*x[0]**o+(-0.0462338578443)*x[0]+(0.117244760003)*x[1]**o+(0.732980884473)*x[1]+(-0.765006403884)*x[2]**o+(-0.543045061256)*x[2]
        s[3,0,1,1]=(0.695840005801)*x[0]**o+(-0.650014497536)*x[0]+(-0.801799125938)*x[1]**o+(-0.482643409576)*x[1]+(-0.416787187079)*x[2]**o+(0.584087905418)*x[2]
        s[3,0,2,0]=(-0.474618797242)*x[0]**o+(0.0189438474293)*x[0]+(-0.982174873996)*x[1]**o+(0.866178328676)*x[1]+(0.208673392242)*x[2]**o+(0.260241441013)*x[2]
        s[3,0,2,1]=(0.764105959702)*x[0]**o+(-0.578329182229)*x[0]+(0.556936430537)*x[1]**o+(0.0927824162943)*x[1]+(0.541941913665)*x[2]**o+(-0.20051953035)*x[2]
        s[3,1,0,0]=(0.467656474356)*x[0]**o+(0.20979187037)*x[0]+(0.830045363621)*x[1]**o+(0.305144956736)*x[1]+(-0.75827792525)*x[2]**o+(-0.336399882938)*x[2]
        s[3,1,0,1]=(-0.608709029175)*x[0]**o+(0.856296131715)*x[0]+(0.740185602478)*x[1]**o+(-0.899456455583)*x[1]+(-0.876657902509)*x[2]**o+(0.364216124717)*x[2]
        s[3,1,1,0]=(-0.19192395812)*x[0]**o+(0.106118249552)*x[0]+(-0.894241926881)*x[1]**o+(-0.985455697871)*x[1]+(-0.782102817165)*x[2]**o+(0.600080360854)*x[2]
        s[3,1,1,1]=(0.721311552863)*x[0]**o+(0.882962028856)*x[0]+(0.150876040487)*x[1]**o+(0.726055027812)*x[1]+(0.379204844464)*x[2]**o+(0.451150402164)*x[2]
        s[3,1,2,0]=(-0.694311490249)*x[0]**o+(-0.0875834002548)*x[0]+(-0.496200108766)*x[1]**o+(-0.151348317151)*x[1]+(0.854949924292)*x[2]**o+(0.915644549228)*x[2]
        s[3,1,2,1]=(0.470664286879)*x[0]**o+(0.706777924428)*x[0]+(-0.402719482127)*x[1]**o+(0.354894757591)*x[1]+(-0.472386640978)*x[2]**o+(0.947399400151)*x[2]
        s[3,2,0,0]=(-0.348423303161)*x[0]**o+(0.555948931484)*x[0]+(-0.43100597992)*x[1]**o+(-0.616049833968)*x[1]+(-0.150797584473)*x[2]**o+(0.520748537644)*x[2]
        s[3,2,0,1]=(0.138677687017)*x[0]**o+(0.18650212002)*x[0]+(0.260025448474)*x[1]**o+(0.0500586777217)*x[1]+(0.588917447067)*x[2]**o+(0.631694471884)*x[2]
        s[3,2,1,0]=(0.732017093416)*x[0]**o+(0.454265152556)*x[0]+(0.656152173314)*x[1]**o+(-0.214651040504)*x[1]+(0.606594898698)*x[2]**o+(-0.944529888173)*x[2]
        s[3,2,1,1]=(0.81276769768)*x[0]**o+(0.955839641144)*x[0]+(-0.597801282701)*x[1]**o+(0.203084441108)*x[1]+(0.371816930827)*x[2]**o+(-0.271843831057)*x[2]
        s[3,2,2,0]=(-0.0175874259272)*x[0]**o+(0.714416611293)*x[0]+(-0.446272670147)*x[1]**o+(-0.749224258055)*x[1]+(0.0434115533854)*x[2]**o+(-0.988008357199)*x[2]
        s[3,2,2,1]=(-0.219583104946)*x[0]**o+(0.207359013184)*x[0]+(-0.891027011221)*x[1]**o+(0.587776096493)*x[1]+(-0.545546707003)*x[2]**o+(-0.964582011253)*x[2]
        s[3,3,0,0]=(-0.540327832705)*x[0]**o+(0.489344676308)*x[0]+(0.663864603046)*x[1]**o+(-0.382248762805)*x[1]+(-0.729410073862)*x[2]**o+(-0.0325319636233)*x[2]
        s[3,3,0,1]=(0.259904022099)*x[0]**o+(-0.954974552252)*x[0]+(0.807969897038)*x[1]**o+(-0.535211587892)*x[1]+(0.401555747708)*x[2]**o+(0.80508329455)*x[2]
        s[3,3,1,0]=(0.172742587833)*x[0]**o+(-0.548601545874)*x[0]+(-0.991112338996)*x[1]**o+(-0.43054209437)*x[1]+(-0.32739577851)*x[2]**o+(-0.774125262769)*x[2]
        s[3,3,1,1]=(0.845744554328)*x[0]**o+(-0.258867050163)*x[0]+(-0.204629368407)*x[1]**o+(-0.821603302256)*x[1]+(-0.828098528219)*x[2]**o+(0.777673858589)*x[2]
        s[3,3,2,0]=(0.852991896102)*x[0]**o+(-0.745196256354)*x[0]+(0.220492132429)*x[1]**o+(-0.953335053523)*x[1]+(-0.324748822468)*x[2]**o+(-0.271392188657)*x[2]
        s[3,3,2,1]=(-0.256699792962)*x[0]**o+(0.670574693398)*x[0]+(0.538127693066)*x[1]**o+(-0.8291145795)*x[1]+(-0.152733016837)*x[2]**o+(0.401659214289)*x[2]
        s[3,4,0,0]=(-0.185294031294)*x[0]**o+(0.0683648982912)*x[0]+(-0.375470928752)*x[1]**o+(-0.67810003631)*x[1]+(-0.189259339657)*x[2]**o+(-0.174734934234)*x[2]
        s[3,4,0,1]=(-0.690376152222)*x[0]**o+(-0.021959548712)*x[0]+(0.816545400525)*x[1]**o+(0.743074566253)*x[1]+(0.907319374762)*x[2]**o+(-0.399104880981)*x[2]
        s[3,4,1,0]=(0.00764653168626)*x[0]**o+(0.418055838525)*x[0]+(-0.711231894711)*x[1]**o+(-0.319907118923)*x[1]+(0.230294344053)*x[2]**o+(0.852841918128)*x[2]
        s[3,4,1,1]=(0.773576800189)*x[0]**o+(-0.258243482679)*x[0]+(-0.217450374857)*x[1]**o+(0.0765524835501)*x[1]+(0.602823749066)*x[2]**o+(0.960046729433)*x[2]
        s[3,4,2,0]=(0.049879518732)*x[0]**o+(0.780027362911)*x[0]+(0.352479979369)*x[1]**o+(-0.960180931719)*x[1]+(0.220657155288)*x[2]**o+(-0.534538076664)*x[2]
        s[3,4,2,1]=(-0.260286357506)*x[0]**o+(0.319391110493)*x[0]+(0.0657794068301)*x[1]**o+(0.254797964539)*x[1]+(-0.495401153451)*x[2]**o+(-0.339881488587)*x[2]
        ref[0,0,0,0]=(-0.853604830323)*x_ref[0]**o+(0.378465791865)*x_ref[0]+(-0.83364075876)*x_ref[1]**o+(0.212018195408)*x_ref[1]+(0.48856925163)*x_ref[2]**o+(-0.179831169531)*x_ref[2]
        ref[0,0,0,1]=(0.868673673397)*x_ref[0]**o+(-0.238560476072)*x_ref[0]+(0.828785612257)*x_ref[1]**o+(0.176759782363)*x_ref[1]+(0.525888840201)*x_ref[2]**o+(0.436158744893)*x_ref[2]
        ref[0,0,1,0]=(0.887888476899)*x_ref[0]**o+(0.458452579078)*x_ref[0]+(0.572312971484)*x_ref[1]**o+(0.694648297654)*x_ref[1]+(0.244770581638)*x_ref[2]**o+(-0.326433505221)*x_ref[2]
        ref[0,0,1,1]=(0.958595740398)*x_ref[0]**o+(-0.448016561436)*x_ref[0]+(-0.898608139656)*x_ref[1]**o+(0.0189044802348)*x_ref[1]+(0.664802638624)*x_ref[2]**o+(0.675631565244)*x_ref[2]
        ref[0,0,2,0]=(-0.0968613941718)*x_ref[0]**o+(-0.580198871721)*x_ref[0]+(-0.695139695389)*x_ref[1]**o+(0.670627088913)*x_ref[1]+(-0.466392817977)*x_ref[2]**o+(-0.414367914263)*x_ref[2]
        ref[0,0,2,1]=(0.0563724518255)*x_ref[0]**o+(0.141716006991)*x_ref[0]+(-0.889611317888)*x_ref[1]**o+(0.274193692615)*x_ref[1]+(0.81766066569)*x_ref[2]**o+(0.260242356947)*x_ref[2]
        ref[0,1,0,0]=(0.729916171245)*x_ref[0]**o+(-0.0512883787314)*x_ref[0]+(-0.395879068642)*x_ref[1]**o+(0.65153799151)*x_ref[1]+(-0.90747241683)*x_ref[2]**o+(0.556056661876)*x_ref[2]
        ref[0,1,0,1]=(-0.336962711878)*x_ref[0]**o+(0.494126376731)*x_ref[0]+(-0.0180540415242)*x_ref[1]**o+(-0.281208191895)*x_ref[1]+(-0.891281821502)*x_ref[2]**o+(0.434438114521)*x_ref[2]
        ref[0,1,1,0]=(-0.0203012608797)*x_ref[0]**o+(-0.592593200931)*x_ref[0]+(-0.908091999823)*x_ref[1]**o+(0.917265173937)*x_ref[1]+(0.0113338093354)*x_ref[2]**o+(0.832522528967)*x_ref[2]
        ref[0,1,1,1]=(-0.287082151578)*x_ref[0]**o+(-0.882563505538)*x_ref[0]+(-0.805577758903)*x_ref[1]**o+(-0.0877632778656)*x_ref[1]+(-0.4141073805)*x_ref[2]**o+(-0.0809945294401)*x_ref[2]
        ref[0,1,2,0]=(-0.745887649438)*x_ref[0]**o+(0.105423480913)*x_ref[0]+(0.642374896965)*x_ref[1]**o+(0.479730302425)*x_ref[1]+(0.334298930575)*x_ref[2]**o+(0.214588275036)*x_ref[2]
        ref[0,1,2,1]=(-0.987939537144)*x_ref[0]**o+(-0.889107799492)*x_ref[0]+(-0.133927268357)*x_ref[1]**o+(-0.991450009918)*x_ref[1]+(-0.201645805359)*x_ref[2]**o+(0.481104989077)*x_ref[2]
        ref[0,2,0,0]=(0.535918222555)*x_ref[0]**o+(0.243041294608)*x_ref[0]+(-0.205082148634)*x_ref[1]**o+(0.233470577382)*x_ref[1]+(0.398471417689)*x_ref[2]**o+(-0.56437758006)*x_ref[2]
        ref[0,2,0,1]=(0.374626206936)*x_ref[0]**o+(-0.800840698958)*x_ref[0]+(0.336427483065)*x_ref[1]**o+(0.143368857985)*x_ref[1]+(0.822396957468)*x_ref[2]**o+(-0.288884862626)*x_ref[2]
        ref[0,2,1,0]=(-0.152217915896)*x_ref[0]**o+(0.868324413095)*x_ref[0]+(0.93760108412)*x_ref[1]**o+(-0.584628102458)*x_ref[1]+(-0.678653990131)*x_ref[2]**o+(-0.130768977951)*x_ref[2]
        ref[0,2,1,1]=(-0.962809494754)*x_ref[0]**o+(0.057963920559)*x_ref[0]+(-0.68685683212)*x_ref[1]**o+(0.869986883492)*x_ref[1]+(-0.193987500788)*x_ref[2]**o+(0.0357315899177)*x_ref[2]
        ref[0,2,2,0]=(-0.296222468619)*x_ref[0]**o+(0.853293489381)*x_ref[0]+(0.886622498642)*x_ref[1]**o+(-0.22952370037)*x_ref[1]+(0.444962301355)*x_ref[2]**o+(0.498735104677)*x_ref[2]
        ref[0,2,2,1]=(-0.556796221789)*x_ref[0]**o+(-0.98717630759)*x_ref[0]+(-0.56578841851)*x_ref[1]**o+(-0.769428379953)*x_ref[1]+(0.41417947942)*x_ref[2]**o+(-0.99075609451)*x_ref[2]
        ref[0,3,0,0]=(-0.312173122533)*x_ref[0]**o+(-0.0183061811911)*x_ref[0]+(0.632028466217)*x_ref[1]**o+(-0.516798544207)*x_ref[1]+(0.467776726567)*x_ref[2]**o+(-0.0750898474112)*x_ref[2]
        ref[0,3,0,1]=(0.151818577102)*x_ref[0]**o+(0.275659634397)*x_ref[0]+(-0.695866418429)*x_ref[1]**o+(-0.158484337329)*x_ref[1]+(-0.717635621103)*x_ref[2]**o+(0.96218471765)*x_ref[2]
        ref[0,3,1,0]=(-0.959269035858)*x_ref[0]**o+(-0.694952577107)*x_ref[0]+(0.133713794285)*x_ref[1]**o+(0.945801566586)*x_ref[1]+(0.480976843012)*x_ref[2]**o+(0.514096820167)*x_ref[2]
        ref[0,3,1,1]=(-0.696429878723)*x_ref[0]**o+(0.844116956105)*x_ref[0]+(0.986596486924)*x_ref[1]**o+(0.213400513251)*x_ref[1]+(0.168567391983)*x_ref[2]**o+(-0.96577037709)*x_ref[2]
        ref[0,3,2,0]=(0.272554167042)*x_ref[0]**o+(-0.134445947881)*x_ref[0]+(-0.620927727232)*x_ref[1]**o+(0.774446825459)*x_ref[1]+(-0.140360641548)*x_ref[2]**o+(-0.393011913595)*x_ref[2]
        ref[0,3,2,1]=(0.973937360802)*x_ref[0]**o+(-0.780203033305)*x_ref[0]+(0.62508420402)*x_ref[1]**o+(-0.957359234026)*x_ref[1]+(0.301032273116)*x_ref[2]**o+(0.238130119107)*x_ref[2]
        ref[0,4,0,0]=(0.550167138562)*x_ref[0]**o+(-0.975271434354)*x_ref[0]+(-0.90226839553)*x_ref[1]**o+(0.765360109563)*x_ref[1]+(-0.0539842939871)*x_ref[2]**o+(-0.992811018253)*x_ref[2]
        ref[0,4,0,1]=(0.115680644235)*x_ref[0]**o+(0.6504959069)*x_ref[0]+(0.0711117194347)*x_ref[1]**o+(0.765247742051)*x_ref[1]+(0.651478097229)*x_ref[2]**o+(0.0605973413193)*x_ref[2]
        ref[0,4,1,0]=(-0.339495686607)*x_ref[0]**o+(0.98910452173)*x_ref[0]+(-0.97593906201)*x_ref[1]**o+(-0.961098427995)*x_ref[1]+(-0.970981798475)*x_ref[2]**o+(-0.655846307864)*x_ref[2]
        ref[0,4,1,1]=(-0.676422431554)*x_ref[0]**o+(0.17779634387)*x_ref[0]+(-0.101241446408)*x_ref[1]**o+(0.592319052036)*x_ref[1]+(-0.328471629455)*x_ref[2]**o+(0.842981957639)*x_ref[2]
        ref[0,4,2,0]=(0.475154370559)*x_ref[0]**o+(-0.861452809594)*x_ref[0]+(0.021220095967)*x_ref[1]**o+(-0.167324332634)*x_ref[1]+(-0.420334858996)*x_ref[2]**o+(-0.243503287995)*x_ref[2]
        ref[0,4,2,1]=(0.507818170756)*x_ref[0]**o+(0.0305556062843)*x_ref[0]+(0.657465103712)*x_ref[1]**o+(-0.133306218785)*x_ref[1]+(0.596182931411)*x_ref[2]**o+(0.26354220091)*x_ref[2]
        ref[1,0,0,0]=(0.141665798979)*x_ref[0]**o+(0.257839463098)*x_ref[0]+(0.20842719092)*x_ref[1]**o+(0.0606481804876)*x_ref[1]+(0.564188524366)*x_ref[2]**o+(0.276431563089)*x_ref[2]
        ref[1,0,0,1]=(-0.475871120292)*x_ref[0]**o+(-0.945285882278)*x_ref[0]+(-0.852943036141)*x_ref[1]**o+(-0.393053044641)*x_ref[1]+(0.894360138187)*x_ref[2]**o+(0.945580362088)*x_ref[2]
        ref[1,0,1,0]=(-0.269113046186)*x_ref[0]**o+(0.979226351783)*x_ref[0]+(0.59588973168)*x_ref[1]**o+(0.709182601617)*x_ref[1]+(0.203809169118)*x_ref[2]**o+(0.843792678336)*x_ref[2]
        ref[1,0,1,1]=(0.625152197275)*x_ref[0]**o+(0.700601491781)*x_ref[0]+(-0.556266788732)*x_ref[1]**o+(0.621296929469)*x_ref[1]+(0.370627217553)*x_ref[2]**o+(-0.86981081047)*x_ref[2]
        ref[1,0,2,0]=(0.337314024742)*x_ref[0]**o+(0.4936522561)*x_ref[0]+(-0.121004948433)*x_ref[1]**o+(0.682816106721)*x_ref[1]+(0.135016952475)*x_ref[2]**o+(0.447788511803)*x_ref[2]
        ref[1,0,2,1]=(0.627048609871)*x_ref[0]**o+(0.464430671751)*x_ref[0]+(0.248071099972)*x_ref[1]**o+(0.49700045944)*x_ref[1]+(0.764133255754)*x_ref[2]**o+(-0.407441090409)*x_ref[2]
        ref[1,1,0,0]=(-0.891648124769)*x_ref[0]**o+(-0.852500694418)*x_ref[0]+(-0.501966394474)*x_ref[1]**o+(-0.951197952585)*x_ref[1]+(-0.293731401112)*x_ref[2]**o+(0.340542703849)*x_ref[2]
        ref[1,1,0,1]=(0.047745963453)*x_ref[0]**o+(0.605499497949)*x_ref[0]+(0.603025931833)*x_ref[1]**o+(0.412711498009)*x_ref[1]+(-0.706970380941)*x_ref[2]**o+(-0.865479677059)*x_ref[2]
        ref[1,1,1,0]=(-0.809326436683)*x_ref[0]**o+(0.50103064436)*x_ref[0]+(-0.726029943072)*x_ref[1]**o+(-0.0136545501065)*x_ref[1]+(0.869298450797)*x_ref[2]**o+(-0.0539795856427)*x_ref[2]
        ref[1,1,1,1]=(0.822265437858)*x_ref[0]**o+(-0.13686562101)*x_ref[0]+(-0.921877204478)*x_ref[1]**o+(0.436216049238)*x_ref[1]+(0.167449871886)*x_ref[2]**o+(-0.376043518724)*x_ref[2]
        ref[1,1,2,0]=(-0.240904110506)*x_ref[0]**o+(-0.709486977651)*x_ref[0]+(0.589748137401)*x_ref[1]**o+(0.997759434219)*x_ref[1]+(0.211495210823)*x_ref[2]**o+(0.857954370359)*x_ref[2]
        ref[1,1,2,1]=(-0.872650055395)*x_ref[0]**o+(0.488392921139)*x_ref[0]+(0.456778341472)*x_ref[1]**o+(0.533360514682)*x_ref[1]+(0.660463672454)*x_ref[2]**o+(-0.287728736994)*x_ref[2]
        ref[1,2,0,0]=(-0.56865288172)*x_ref[0]**o+(-0.257707278342)*x_ref[0]+(0.492925973387)*x_ref[1]**o+(-0.433521245223)*x_ref[1]+(-0.214711127244)*x_ref[2]**o+(-0.578235847487)*x_ref[2]
        ref[1,2,0,1]=(0.502020345167)*x_ref[0]**o+(-0.898730931974)*x_ref[0]+(0.152379872963)*x_ref[1]**o+(0.963247300695)*x_ref[1]+(-0.935219139549)*x_ref[2]**o+(-0.322095550311)*x_ref[2]
        ref[1,2,1,0]=(0.755425499434)*x_ref[0]**o+(0.922722006611)*x_ref[0]+(-0.854913831787)*x_ref[1]**o+(0.28200116496)*x_ref[1]+(0.654754619366)*x_ref[2]**o+(-0.87246359532)*x_ref[2]
        ref[1,2,1,1]=(0.476533428226)*x_ref[0]**o+(-0.21427962663)*x_ref[0]+(-0.42807986959)*x_ref[1]**o+(-0.453270655163)*x_ref[1]+(-0.0618352146195)*x_ref[2]**o+(-0.0578564702081)*x_ref[2]
        ref[1,2,2,0]=(0.836242039415)*x_ref[0]**o+(0.0582519660745)*x_ref[0]+(-0.736245953793)*x_ref[1]**o+(-0.0216606630383)*x_ref[1]+(-0.241691127038)*x_ref[2]**o+(-0.273769285415)*x_ref[2]
        ref[1,2,2,1]=(0.956981101891)*x_ref[0]**o+(-0.619600309731)*x_ref[0]+(0.255999571352)*x_ref[1]**o+(0.931443107924)*x_ref[1]+(0.803455662831)*x_ref[2]**o+(0.587378690917)*x_ref[2]
        ref[1,3,0,0]=(0.84725602389)*x_ref[0]**o+(0.099929556134)*x_ref[0]+(0.918513623927)*x_ref[1]**o+(-0.540239895987)*x_ref[1]+(0.543336053141)*x_ref[2]**o+(-0.35339148993)*x_ref[2]
        ref[1,3,0,1]=(0.0790066953583)*x_ref[0]**o+(-0.622984122413)*x_ref[0]+(0.844477077556)*x_ref[1]**o+(-0.843967484949)*x_ref[1]+(0.382810149795)*x_ref[2]**o+(0.693483174672)*x_ref[2]
        ref[1,3,1,0]=(-0.606850566406)*x_ref[0]**o+(-0.613588487235)*x_ref[0]+(-0.982801043706)*x_ref[1]**o+(-0.200157044491)*x_ref[1]+(0.865010142689)*x_ref[2]**o+(-0.926404146796)*x_ref[2]
        ref[1,3,1,1]=(-0.926332516404)*x_ref[0]**o+(-0.644403030771)*x_ref[0]+(-0.391941644842)*x_ref[1]**o+(-0.628555138555)*x_ref[1]+(0.584646630734)*x_ref[2]**o+(0.627561535294)*x_ref[2]
        ref[1,3,2,0]=(0.777960693446)*x_ref[0]**o+(-0.580698013493)*x_ref[0]+(-0.061588850878)*x_ref[1]**o+(0.909167097211)*x_ref[1]+(-0.346386067681)*x_ref[2]**o+(-0.698593831635)*x_ref[2]
        ref[1,3,2,1]=(-0.25740731816)*x_ref[0]**o+(0.0609560824945)*x_ref[0]+(-0.866636970162)*x_ref[1]**o+(0.352496912986)*x_ref[1]+(0.212276243322)*x_ref[2]**o+(-0.29499231054)*x_ref[2]
        ref[1,4,0,0]=(0.176828476382)*x_ref[0]**o+(0.592116728712)*x_ref[0]+(-0.969737040463)*x_ref[1]**o+(-0.0140144291993)*x_ref[1]+(-0.203791892462)*x_ref[2]**o+(0.687043221294)*x_ref[2]
        ref[1,4,0,1]=(-0.0673562914955)*x_ref[0]**o+(0.104813124019)*x_ref[0]+(0.766386480162)*x_ref[1]**o+(-0.499215759851)*x_ref[1]+(0.163490109505)*x_ref[2]**o+(0.0477449791182)*x_ref[2]
        ref[1,4,1,0]=(-0.186436883357)*x_ref[0]**o+(0.771658464504)*x_ref[0]+(0.115845567912)*x_ref[1]**o+(-0.573016245745)*x_ref[1]+(0.759680038826)*x_ref[2]**o+(0.0660145528929)*x_ref[2]
        ref[1,4,1,1]=(-0.983746295741)*x_ref[0]**o+(-0.370146292969)*x_ref[0]+(-0.276398856773)*x_ref[1]**o+(-0.765483196375)*x_ref[1]+(-0.0802845394538)*x_ref[2]**o+(0.762085198906)*x_ref[2]
        ref[1,4,2,0]=(-0.437323501168)*x_ref[0]**o+(-0.076976193125)*x_ref[0]+(0.780179691457)*x_ref[1]**o+(0.941748527645)*x_ref[1]+(0.921220023779)*x_ref[2]**o+(-0.611350278287)*x_ref[2]
        ref[1,4,2,1]=(-0.811077839585)*x_ref[0]**o+(-0.0781699097656)*x_ref[0]+(-0.955979549084)*x_ref[1]**o+(0.742581422414)*x_ref[1]+(-0.905829236752)*x_ref[2]**o+(-0.671257774729)*x_ref[2]
        ref[2,0,0,0]=(0.3602498958)*x_ref[0]**o+(0.567952303965)*x_ref[0]+(-0.72126509422)*x_ref[1]**o+(0.652912038045)*x_ref[1]+(-0.448081819441)*x_ref[2]**o+(-0.628140847819)*x_ref[2]
        ref[2,0,0,1]=(0.823191679062)*x_ref[0]**o+(-0.12280206123)*x_ref[0]+(0.202149797631)*x_ref[1]**o+(-0.751457177895)*x_ref[1]+(0.367057556408)*x_ref[2]**o+(0.71681510921)*x_ref[2]
        ref[2,0,1,0]=(-0.553277801352)*x_ref[0]**o+(-0.695114786088)*x_ref[0]+(-0.506695185331)*x_ref[1]**o+(0.947900606544)*x_ref[1]+(0.393574900465)*x_ref[2]**o+(-0.131907977332)*x_ref[2]
        ref[2,0,1,1]=(-0.683260439471)*x_ref[0]**o+(0.108921790681)*x_ref[0]+(0.694361429808)*x_ref[1]**o+(-0.879827908202)*x_ref[1]+(-0.768400141399)*x_ref[2]**o+(-0.869535466479)*x_ref[2]
        ref[2,0,2,0]=(-0.377844410367)*x_ref[0]**o+(-0.486876364642)*x_ref[0]+(-0.139990170975)*x_ref[1]**o+(0.521294626989)*x_ref[1]+(0.384721944946)*x_ref[2]**o+(0.912206678795)*x_ref[2]
        ref[2,0,2,1]=(0.624159317849)*x_ref[0]**o+(0.910540130065)*x_ref[0]+(-0.879166404339)*x_ref[1]**o+(0.710229335366)*x_ref[1]+(-0.756806360688)*x_ref[2]**o+(-0.169809484076)*x_ref[2]
        ref[2,1,0,0]=(-0.887833054092)*x_ref[0]**o+(-0.541562581783)*x_ref[0]+(-0.439734700726)*x_ref[1]**o+(-0.294103567325)*x_ref[1]+(0.621145429347)*x_ref[2]**o+(0.65618994558)*x_ref[2]
        ref[2,1,0,1]=(-0.199707709698)*x_ref[0]**o+(0.952895238716)*x_ref[0]+(-0.113807788254)*x_ref[1]**o+(-0.209497731554)*x_ref[1]+(-0.201935349119)*x_ref[2]**o+(0.896172767252)*x_ref[2]
        ref[2,1,1,0]=(0.961864846935)*x_ref[0]**o+(0.34037006989)*x_ref[0]+(0.97866208125)*x_ref[1]**o+(0.949937962054)*x_ref[1]+(-0.943237740253)*x_ref[2]**o+(0.492751804379)*x_ref[2]
        ref[2,1,1,1]=(0.0364913119154)*x_ref[0]**o+(-0.706276345936)*x_ref[0]+(-0.0562330252965)*x_ref[1]**o+(-0.919546510941)*x_ref[1]+(0.376680654917)*x_ref[2]**o+(-0.184153093526)*x_ref[2]
        ref[2,1,2,0]=(-0.135265768495)*x_ref[0]**o+(0.650164259074)*x_ref[0]+(0.224627083284)*x_ref[1]**o+(0.918537471241)*x_ref[1]+(0.662719594601)*x_ref[2]**o+(-0.4451558313)*x_ref[2]
        ref[2,1,2,1]=(0.968484347906)*x_ref[0]**o+(0.261215594011)*x_ref[0]+(-0.478839261077)*x_ref[1]**o+(-0.806554334141)*x_ref[1]+(0.576441463783)*x_ref[2]**o+(-0.407338671799)*x_ref[2]
        ref[2,2,0,0]=(-0.691105566226)*x_ref[0]**o+(0.860347396649)*x_ref[0]+(0.354066286829)*x_ref[1]**o+(-0.0402175279153)*x_ref[1]+(0.415237302648)*x_ref[2]**o+(0.905209054794)*x_ref[2]
        ref[2,2,0,1]=(-0.548678761348)*x_ref[0]**o+(-0.294803736974)*x_ref[0]+(-0.960170319863)*x_ref[1]**o+(-0.27333394559)*x_ref[1]+(0.866348038561)*x_ref[2]**o+(-0.977985070269)*x_ref[2]
        ref[2,2,1,0]=(-0.942843839038)*x_ref[0]**o+(-0.868858708934)*x_ref[0]+(-0.208988345792)*x_ref[1]**o+(-0.628702405302)*x_ref[1]+(0.856538967932)*x_ref[2]**o+(-0.940751252093)*x_ref[2]
        ref[2,2,1,1]=(0.493835163998)*x_ref[0]**o+(0.236452180515)*x_ref[0]+(0.92903313506)*x_ref[1]**o+(0.817122194244)*x_ref[1]+(-0.498919897775)*x_ref[2]**o+(0.177040227977)*x_ref[2]
        ref[2,2,2,0]=(-0.750931164998)*x_ref[0]**o+(0.859659424304)*x_ref[0]+(0.415426727903)*x_ref[1]**o+(-0.468797574814)*x_ref[1]+(-0.294847888148)*x_ref[2]**o+(-0.992000733445)*x_ref[2]
        ref[2,2,2,1]=(0.0182583094659)*x_ref[0]**o+(-0.830672513808)*x_ref[0]+(0.37453609414)*x_ref[1]**o+(-0.57449821779)*x_ref[1]+(0.134490719543)*x_ref[2]**o+(-0.609352246473)*x_ref[2]
        ref[2,3,0,0]=(-0.872131310391)*x_ref[0]**o+(0.0825093683843)*x_ref[0]+(-0.982804116245)*x_ref[1]**o+(0.22341420851)*x_ref[1]+(-0.231362455053)*x_ref[2]**o+(-0.904631856625)*x_ref[2]
        ref[2,3,0,1]=(-0.0582824887741)*x_ref[0]**o+(-0.399156766464)*x_ref[0]+(0.0772313065438)*x_ref[1]**o+(0.00426296268846)*x_ref[1]+(-0.803184293158)*x_ref[2]**o+(-0.17731453976)*x_ref[2]
        ref[2,3,1,0]=(0.252580558693)*x_ref[0]**o+(-0.670939935472)*x_ref[0]+(-0.396636904351)*x_ref[1]**o+(-0.774693006552)*x_ref[1]+(0.451637181152)*x_ref[2]**o+(-0.463784221351)*x_ref[2]
        ref[2,3,1,1]=(0.126767645109)*x_ref[0]**o+(-0.395030113635)*x_ref[0]+(-0.955454976686)*x_ref[1]**o+(-0.710220994617)*x_ref[1]+(-0.397874215052)*x_ref[2]**o+(-0.0217615150269)*x_ref[2]
        ref[2,3,2,0]=(0.990529883771)*x_ref[0]**o+(0.278013247448)*x_ref[0]+(-0.430442262972)*x_ref[1]**o+(-0.720399995623)*x_ref[1]+(0.610560781511)*x_ref[2]**o+(-0.264692262808)*x_ref[2]
        ref[2,3,2,1]=(-0.776369460759)*x_ref[0]**o+(0.397198841934)*x_ref[0]+(-0.604527190952)*x_ref[1]**o+(0.94368439705)*x_ref[1]+(-0.88032288089)*x_ref[2]**o+(0.989155014613)*x_ref[2]
        ref[2,4,0,0]=(0.457573740068)*x_ref[0]**o+(-0.352003681775)*x_ref[0]+(0.996428644418)*x_ref[1]**o+(-0.348011873806)*x_ref[1]+(-0.58023500034)*x_ref[2]**o+(0.727228005186)*x_ref[2]
        ref[2,4,0,1]=(0.50989726873)*x_ref[0]**o+(0.83338864798)*x_ref[0]+(-0.111380969484)*x_ref[1]**o+(-0.278057155647)*x_ref[1]+(0.0022286233284)*x_ref[2]**o+(0.841662666668)*x_ref[2]
        ref[2,4,1,0]=(-0.901603581146)*x_ref[0]**o+(0.475205009348)*x_ref[0]+(-0.430007910717)*x_ref[1]**o+(-0.114746630672)*x_ref[1]+(0.847548245931)*x_ref[2]**o+(-0.539570230562)*x_ref[2]
        ref[2,4,1,1]=(0.539830165452)*x_ref[0]**o+(-0.415758645187)*x_ref[0]+(0.712970160554)*x_ref[1]**o+(0.833164796207)*x_ref[1]+(-0.65521876468)*x_ref[2]**o+(-0.478072278167)*x_ref[2]
        ref[2,4,2,0]=(0.651879536878)*x_ref[0]**o+(0.782082841616)*x_ref[0]+(0.0416994850732)*x_ref[1]**o+(0.0989265074417)*x_ref[1]+(0.417171388449)*x_ref[2]**o+(0.204544481473)*x_ref[2]
        ref[2,4,2,1]=(-0.957695309447)*x_ref[0]**o+(0.41057815979)*x_ref[0]+(0.847095166862)*x_ref[1]**o+(0.937811945321)*x_ref[1]+(-0.794606224039)*x_ref[2]**o+(0.021545807724)*x_ref[2]
        ref[3,0,0,0]=(-0.521599558272)*x_ref[0]**o+(-0.640095375476)*x_ref[0]+(-0.51854438692)*x_ref[1]**o+(-0.87210774891)*x_ref[1]+(-0.693848689324)*x_ref[2]**o+(-0.261635833944)*x_ref[2]
        ref[3,0,0,1]=(0.620197170573)*x_ref[0]**o+(-0.841665901801)*x_ref[0]+(-0.546759437483)*x_ref[1]**o+(-0.168579145925)*x_ref[1]+(-0.576567932552)*x_ref[2]**o+(-0.645039556295)*x_ref[2]
        ref[3,0,1,0]=(0.581444084796)*x_ref[0]**o+(-0.0462338578443)*x_ref[0]+(0.117244760003)*x_ref[1]**o+(0.732980884473)*x_ref[1]+(-0.765006403884)*x_ref[2]**o+(-0.543045061256)*x_ref[2]
        ref[3,0,1,1]=(0.695840005801)*x_ref[0]**o+(-0.650014497536)*x_ref[0]+(-0.801799125938)*x_ref[1]**o+(-0.482643409576)*x_ref[1]+(-0.416787187079)*x_ref[2]**o+(0.584087905418)*x_ref[2]
        ref[3,0,2,0]=(-0.474618797242)*x_ref[0]**o+(0.0189438474293)*x_ref[0]+(-0.982174873996)*x_ref[1]**o+(0.866178328676)*x_ref[1]+(0.208673392242)*x_ref[2]**o+(0.260241441013)*x_ref[2]
        ref[3,0,2,1]=(0.764105959702)*x_ref[0]**o+(-0.578329182229)*x_ref[0]+(0.556936430537)*x_ref[1]**o+(0.0927824162943)*x_ref[1]+(0.541941913665)*x_ref[2]**o+(-0.20051953035)*x_ref[2]
        ref[3,1,0,0]=(0.467656474356)*x_ref[0]**o+(0.20979187037)*x_ref[0]+(0.830045363621)*x_ref[1]**o+(0.305144956736)*x_ref[1]+(-0.75827792525)*x_ref[2]**o+(-0.336399882938)*x_ref[2]
        ref[3,1,0,1]=(-0.608709029175)*x_ref[0]**o+(0.856296131715)*x_ref[0]+(0.740185602478)*x_ref[1]**o+(-0.899456455583)*x_ref[1]+(-0.876657902509)*x_ref[2]**o+(0.364216124717)*x_ref[2]
        ref[3,1,1,0]=(-0.19192395812)*x_ref[0]**o+(0.106118249552)*x_ref[0]+(-0.894241926881)*x_ref[1]**o+(-0.985455697871)*x_ref[1]+(-0.782102817165)*x_ref[2]**o+(0.600080360854)*x_ref[2]
        ref[3,1,1,1]=(0.721311552863)*x_ref[0]**o+(0.882962028856)*x_ref[0]+(0.150876040487)*x_ref[1]**o+(0.726055027812)*x_ref[1]+(0.379204844464)*x_ref[2]**o+(0.451150402164)*x_ref[2]
        ref[3,1,2,0]=(-0.694311490249)*x_ref[0]**o+(-0.0875834002548)*x_ref[0]+(-0.496200108766)*x_ref[1]**o+(-0.151348317151)*x_ref[1]+(0.854949924292)*x_ref[2]**o+(0.915644549228)*x_ref[2]
        ref[3,1,2,1]=(0.470664286879)*x_ref[0]**o+(0.706777924428)*x_ref[0]+(-0.402719482127)*x_ref[1]**o+(0.354894757591)*x_ref[1]+(-0.472386640978)*x_ref[2]**o+(0.947399400151)*x_ref[2]
        ref[3,2,0,0]=(-0.348423303161)*x_ref[0]**o+(0.555948931484)*x_ref[0]+(-0.43100597992)*x_ref[1]**o+(-0.616049833968)*x_ref[1]+(-0.150797584473)*x_ref[2]**o+(0.520748537644)*x_ref[2]
        ref[3,2,0,1]=(0.138677687017)*x_ref[0]**o+(0.18650212002)*x_ref[0]+(0.260025448474)*x_ref[1]**o+(0.0500586777217)*x_ref[1]+(0.588917447067)*x_ref[2]**o+(0.631694471884)*x_ref[2]
        ref[3,2,1,0]=(0.732017093416)*x_ref[0]**o+(0.454265152556)*x_ref[0]+(0.656152173314)*x_ref[1]**o+(-0.214651040504)*x_ref[1]+(0.606594898698)*x_ref[2]**o+(-0.944529888173)*x_ref[2]
        ref[3,2,1,1]=(0.81276769768)*x_ref[0]**o+(0.955839641144)*x_ref[0]+(-0.597801282701)*x_ref[1]**o+(0.203084441108)*x_ref[1]+(0.371816930827)*x_ref[2]**o+(-0.271843831057)*x_ref[2]
        ref[3,2,2,0]=(-0.0175874259272)*x_ref[0]**o+(0.714416611293)*x_ref[0]+(-0.446272670147)*x_ref[1]**o+(-0.749224258055)*x_ref[1]+(0.0434115533854)*x_ref[2]**o+(-0.988008357199)*x_ref[2]
        ref[3,2,2,1]=(-0.219583104946)*x_ref[0]**o+(0.207359013184)*x_ref[0]+(-0.891027011221)*x_ref[1]**o+(0.587776096493)*x_ref[1]+(-0.545546707003)*x_ref[2]**o+(-0.964582011253)*x_ref[2]
        ref[3,3,0,0]=(-0.540327832705)*x_ref[0]**o+(0.489344676308)*x_ref[0]+(0.663864603046)*x_ref[1]**o+(-0.382248762805)*x_ref[1]+(-0.729410073862)*x_ref[2]**o+(-0.0325319636233)*x_ref[2]
        ref[3,3,0,1]=(0.259904022099)*x_ref[0]**o+(-0.954974552252)*x_ref[0]+(0.807969897038)*x_ref[1]**o+(-0.535211587892)*x_ref[1]+(0.401555747708)*x_ref[2]**o+(0.80508329455)*x_ref[2]
        ref[3,3,1,0]=(0.172742587833)*x_ref[0]**o+(-0.548601545874)*x_ref[0]+(-0.991112338996)*x_ref[1]**o+(-0.43054209437)*x_ref[1]+(-0.32739577851)*x_ref[2]**o+(-0.774125262769)*x_ref[2]
        ref[3,3,1,1]=(0.845744554328)*x_ref[0]**o+(-0.258867050163)*x_ref[0]+(-0.204629368407)*x_ref[1]**o+(-0.821603302256)*x_ref[1]+(-0.828098528219)*x_ref[2]**o+(0.777673858589)*x_ref[2]
        ref[3,3,2,0]=(0.852991896102)*x_ref[0]**o+(-0.745196256354)*x_ref[0]+(0.220492132429)*x_ref[1]**o+(-0.953335053523)*x_ref[1]+(-0.324748822468)*x_ref[2]**o+(-0.271392188657)*x_ref[2]
        ref[3,3,2,1]=(-0.256699792962)*x_ref[0]**o+(0.670574693398)*x_ref[0]+(0.538127693066)*x_ref[1]**o+(-0.8291145795)*x_ref[1]+(-0.152733016837)*x_ref[2]**o+(0.401659214289)*x_ref[2]
        ref[3,4,0,0]=(-0.185294031294)*x_ref[0]**o+(0.0683648982912)*x_ref[0]+(-0.375470928752)*x_ref[1]**o+(-0.67810003631)*x_ref[1]+(-0.189259339657)*x_ref[2]**o+(-0.174734934234)*x_ref[2]
        ref[3,4,0,1]=(-0.690376152222)*x_ref[0]**o+(-0.021959548712)*x_ref[0]+(0.816545400525)*x_ref[1]**o+(0.743074566253)*x_ref[1]+(0.907319374762)*x_ref[2]**o+(-0.399104880981)*x_ref[2]
        ref[3,4,1,0]=(0.00764653168626)*x_ref[0]**o+(0.418055838525)*x_ref[0]+(-0.711231894711)*x_ref[1]**o+(-0.319907118923)*x_ref[1]+(0.230294344053)*x_ref[2]**o+(0.852841918128)*x_ref[2]
        ref[3,4,1,1]=(0.773576800189)*x_ref[0]**o+(-0.258243482679)*x_ref[0]+(-0.217450374857)*x_ref[1]**o+(0.0765524835501)*x_ref[1]+(0.602823749066)*x_ref[2]**o+(0.960046729433)*x_ref[2]
        ref[3,4,2,0]=(0.049879518732)*x_ref[0]**o+(0.780027362911)*x_ref[0]+(0.352479979369)*x_ref[1]**o+(-0.960180931719)*x_ref[1]+(0.220657155288)*x_ref[2]**o+(-0.534538076664)*x_ref[2]
        ref[3,4,2,1]=(-0.260286357506)*x_ref[0]**o+(0.319391110493)*x_ref[0]+(0.0657794068301)*x_ref[1]**o+(0.254797964539)*x_ref[1]+(-0.495401153451)*x_ref[2]**o+(-0.339881488587)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_ReducedFunctionOnContactOne_rank0(self):
      """
      tests interpolate for rank 0 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactOne onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactOne(self.domain)
      x=w.getX()
      arg=Symbol(shape=(),dim=dim)
      s=Data(0,(),w)
      ref=Data(0,(),w_ref)
      if dim==2:
        s=(0.992324887623)*x[0]**o+(-0.399021408175)*x[0]+(-0.82044668182)*x[1]**o+(-0.115930766833)*x[1]
        ref=(0.992324887623)*x_ref[0]**o+(-0.399021408175)*x_ref[0]+(-0.82044668182)*x_ref[1]**o+(-0.115930766833)*x_ref[1]
      else:
        s=(-0.917591389532)*x[0]**o+(-0.636147522121)*x[0]+(-0.624726334305)*x[1]**o+(0.253824967861)*x[1]+(0.391376692454)*x[2]**o+(-0.635535244093)*x[2]
        ref=(-0.917591389532)*x_ref[0]**o+(-0.636147522121)*x_ref[0]+(-0.624726334305)*x_ref[1]**o+(0.253824967861)*x_ref[1]+(0.391376692454)*x_ref[2]**o+(-0.635535244093)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_ReducedFunctionOnContactOne_rank1(self):
      """
      tests interpolate for rank 1 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactOne onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactOne(self.domain)
      x=w.getX()
      arg=Symbol(shape=(2,),dim=dim)
      s=Data(0,(2,),w)
      ref=Data(0,(2,),w_ref)
      if dim==2:
        s[0]=(-0.420392705289)*x[0]**o+(-0.988154722022)*x[0]+(-0.865378713801)*x[1]**o+(0.992312897132)*x[1]
        s[1]=(-0.48354228784)*x[0]**o+(0.07109068872)*x[0]+(0.757923528294)*x[1]**o+(0.903265965794)*x[1]
        ref[0]=(-0.420392705289)*x_ref[0]**o+(-0.988154722022)*x_ref[0]+(-0.865378713801)*x_ref[1]**o+(0.992312897132)*x_ref[1]
        ref[1]=(-0.48354228784)*x_ref[0]**o+(0.07109068872)*x_ref[0]+(0.757923528294)*x_ref[1]**o+(0.903265965794)*x_ref[1]
      else:
        s[0]=(-0.119618139601)*x[0]**o+(0.674458706959)*x[0]+(0.283506026525)*x[1]**o+(0.867334648745)*x[1]+(0.793263586089)*x[2]**o+(-0.225225247588)*x[2]
        s[1]=(-0.458483133737)*x[0]**o+(0.636334230541)*x[0]+(0.433152852396)*x[1]**o+(0.946877061205)*x[1]+(-0.633441461314)*x[2]**o+(-0.00536782448735)*x[2]
        ref[0]=(-0.119618139601)*x_ref[0]**o+(0.674458706959)*x_ref[0]+(0.283506026525)*x_ref[1]**o+(0.867334648745)*x_ref[1]+(0.793263586089)*x_ref[2]**o+(-0.225225247588)*x_ref[2]
        ref[1]=(-0.458483133737)*x_ref[0]**o+(0.636334230541)*x_ref[0]+(0.433152852396)*x_ref[1]**o+(0.946877061205)*x_ref[1]+(-0.633441461314)*x_ref[2]**o+(-0.00536782448735)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_ReducedFunctionOnContactOne_rank2(self):
      """
      tests interpolate for rank 2 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactOne onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactOne(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5),dim=dim)
      s=Data(0,(4, 5),w)
      ref=Data(0,(4, 5),w_ref)
      if dim==2:
        s[0,0]=(-0.230877307866)*x[0]**o+(0.215021920364)*x[0]+(0.72160607128)*x[1]**o+(0.288782620524)*x[1]
        s[0,1]=(-0.397089347754)*x[0]**o+(-0.859308303321)*x[0]+(-0.655908162636)*x[1]**o+(-0.75086945631)*x[1]
        s[0,2]=(0.17849428647)*x[0]**o+(0.190815421543)*x[0]+(0.972002884101)*x[1]**o+(0.4108901607)*x[1]
        s[0,3]=(-0.288371936253)*x[0]**o+(-0.394846982668)*x[0]+(0.00326108093218)*x[1]**o+(0.218771269587)*x[1]
        s[0,4]=(0.629096144369)*x[0]**o+(0.820908159894)*x[0]+(0.691458623728)*x[1]**o+(0.214095362021)*x[1]
        s[1,0]=(0.709456114123)*x[0]**o+(-0.837635170883)*x[0]+(0.469741659937)*x[1]**o+(-0.225676816505)*x[1]
        s[1,1]=(-0.429024930015)*x[0]**o+(0.332522034373)*x[0]+(-0.14085265632)*x[1]**o+(0.739644399991)*x[1]
        s[1,2]=(-0.550855064281)*x[0]**o+(0.950491676704)*x[0]+(0.468631317567)*x[1]**o+(-0.78396330626)*x[1]
        s[1,3]=(0.0713337235427)*x[0]**o+(-0.371870909904)*x[0]+(-0.329491599348)*x[1]**o+(0.0765857177272)*x[1]
        s[1,4]=(-0.619922856297)*x[0]**o+(-0.230137321592)*x[0]+(0.883918096123)*x[1]**o+(-0.164797122993)*x[1]
        s[2,0]=(0.577972603997)*x[0]**o+(-0.535427767528)*x[0]+(0.105653927239)*x[1]**o+(-0.803115354047)*x[1]
        s[2,1]=(-0.941298066772)*x[0]**o+(0.633879774328)*x[0]+(-0.769467220886)*x[1]**o+(-0.830635312894)*x[1]
        s[2,2]=(-0.143658254898)*x[0]**o+(0.612045970085)*x[0]+(-0.234784912386)*x[1]**o+(-0.111427283432)*x[1]
        s[2,3]=(-0.0806971603544)*x[0]**o+(-0.478260278718)*x[0]+(0.405214636774)*x[1]**o+(-0.0171544079949)*x[1]
        s[2,4]=(-0.619407716965)*x[0]**o+(0.292456447643)*x[0]+(-0.404895807117)*x[1]**o+(0.955367226581)*x[1]
        s[3,0]=(-0.762723098154)*x[0]**o+(-0.547004242664)*x[0]+(-0.266917906271)*x[1]**o+(0.600697811809)*x[1]
        s[3,1]=(-0.567148999707)*x[0]**o+(0.374593601186)*x[0]+(-0.440814971395)*x[1]**o+(-0.737145147183)*x[1]
        s[3,2]=(0.630637925234)*x[0]**o+(-0.0884747935175)*x[0]+(-0.0319938816017)*x[1]**o+(0.865297163842)*x[1]
        s[3,3]=(-0.811769976054)*x[0]**o+(0.533258797244)*x[0]+(0.735904279517)*x[1]**o+(0.362604687369)*x[1]
        s[3,4]=(-0.711314210671)*x[0]**o+(-0.270182873191)*x[0]+(-0.965300380176)*x[1]**o+(0.50703939773)*x[1]
        ref[0,0]=(-0.230877307866)*x_ref[0]**o+(0.215021920364)*x_ref[0]+(0.72160607128)*x_ref[1]**o+(0.288782620524)*x_ref[1]
        ref[0,1]=(-0.397089347754)*x_ref[0]**o+(-0.859308303321)*x_ref[0]+(-0.655908162636)*x_ref[1]**o+(-0.75086945631)*x_ref[1]
        ref[0,2]=(0.17849428647)*x_ref[0]**o+(0.190815421543)*x_ref[0]+(0.972002884101)*x_ref[1]**o+(0.4108901607)*x_ref[1]
        ref[0,3]=(-0.288371936253)*x_ref[0]**o+(-0.394846982668)*x_ref[0]+(0.00326108093218)*x_ref[1]**o+(0.218771269587)*x_ref[1]
        ref[0,4]=(0.629096144369)*x_ref[0]**o+(0.820908159894)*x_ref[0]+(0.691458623728)*x_ref[1]**o+(0.214095362021)*x_ref[1]
        ref[1,0]=(0.709456114123)*x_ref[0]**o+(-0.837635170883)*x_ref[0]+(0.469741659937)*x_ref[1]**o+(-0.225676816505)*x_ref[1]
        ref[1,1]=(-0.429024930015)*x_ref[0]**o+(0.332522034373)*x_ref[0]+(-0.14085265632)*x_ref[1]**o+(0.739644399991)*x_ref[1]
        ref[1,2]=(-0.550855064281)*x_ref[0]**o+(0.950491676704)*x_ref[0]+(0.468631317567)*x_ref[1]**o+(-0.78396330626)*x_ref[1]
        ref[1,3]=(0.0713337235427)*x_ref[0]**o+(-0.371870909904)*x_ref[0]+(-0.329491599348)*x_ref[1]**o+(0.0765857177272)*x_ref[1]
        ref[1,4]=(-0.619922856297)*x_ref[0]**o+(-0.230137321592)*x_ref[0]+(0.883918096123)*x_ref[1]**o+(-0.164797122993)*x_ref[1]
        ref[2,0]=(0.577972603997)*x_ref[0]**o+(-0.535427767528)*x_ref[0]+(0.105653927239)*x_ref[1]**o+(-0.803115354047)*x_ref[1]
        ref[2,1]=(-0.941298066772)*x_ref[0]**o+(0.633879774328)*x_ref[0]+(-0.769467220886)*x_ref[1]**o+(-0.830635312894)*x_ref[1]
        ref[2,2]=(-0.143658254898)*x_ref[0]**o+(0.612045970085)*x_ref[0]+(-0.234784912386)*x_ref[1]**o+(-0.111427283432)*x_ref[1]
        ref[2,3]=(-0.0806971603544)*x_ref[0]**o+(-0.478260278718)*x_ref[0]+(0.405214636774)*x_ref[1]**o+(-0.0171544079949)*x_ref[1]
        ref[2,4]=(-0.619407716965)*x_ref[0]**o+(0.292456447643)*x_ref[0]+(-0.404895807117)*x_ref[1]**o+(0.955367226581)*x_ref[1]
        ref[3,0]=(-0.762723098154)*x_ref[0]**o+(-0.547004242664)*x_ref[0]+(-0.266917906271)*x_ref[1]**o+(0.600697811809)*x_ref[1]
        ref[3,1]=(-0.567148999707)*x_ref[0]**o+(0.374593601186)*x_ref[0]+(-0.440814971395)*x_ref[1]**o+(-0.737145147183)*x_ref[1]
        ref[3,2]=(0.630637925234)*x_ref[0]**o+(-0.0884747935175)*x_ref[0]+(-0.0319938816017)*x_ref[1]**o+(0.865297163842)*x_ref[1]
        ref[3,3]=(-0.811769976054)*x_ref[0]**o+(0.533258797244)*x_ref[0]+(0.735904279517)*x_ref[1]**o+(0.362604687369)*x_ref[1]
        ref[3,4]=(-0.711314210671)*x_ref[0]**o+(-0.270182873191)*x_ref[0]+(-0.965300380176)*x_ref[1]**o+(0.50703939773)*x_ref[1]
      else:
        s[0,0]=(0.817784424934)*x[0]**o+(0.140829586694)*x[0]+(0.395723095233)*x[1]**o+(-0.0916449245097)*x[1]+(-0.686149802471)*x[2]**o+(0.231879617817)*x[2]
        s[0,1]=(-0.753947196887)*x[0]**o+(0.0184422308138)*x[0]+(0.38127599678)*x[1]**o+(-0.958287760938)*x[1]+(-0.230273819981)*x[2]**o+(-0.922603290238)*x[2]
        s[0,2]=(0.44304320962)*x[0]**o+(0.35396422013)*x[0]+(-0.479605019935)*x[1]**o+(0.289724501875)*x[1]+(0.657449032305)*x[2]**o+(-0.706689994866)*x[2]
        s[0,3]=(0.0214930482601)*x[0]**o+(-0.915710233423)*x[0]+(-0.571112565179)*x[1]**o+(0.837855622741)*x[1]+(-0.402178902773)*x[2]**o+(-0.113536055723)*x[2]
        s[0,4]=(0.0332368088464)*x[0]**o+(0.488860583014)*x[0]+(0.687230732879)*x[1]**o+(0.417177289784)*x[1]+(-0.138583225765)*x[2]**o+(-0.288206976222)*x[2]
        s[1,0]=(-0.571959772672)*x[0]**o+(-0.806968233432)*x[0]+(-0.951330341336)*x[1]**o+(-0.725871940445)*x[1]+(0.473279498914)*x[2]**o+(0.693882912652)*x[2]
        s[1,1]=(-0.925147598095)*x[0]**o+(0.612875097386)*x[0]+(0.625107884558)*x[1]**o+(0.665316395744)*x[1]+(-0.012158847541)*x[2]**o+(0.114880783467)*x[2]
        s[1,2]=(-0.361803346001)*x[0]**o+(0.630266012707)*x[0]+(-0.698127168765)*x[1]**o+(-0.534017552333)*x[1]+(0.968004682982)*x[2]**o+(-0.607970906679)*x[2]
        s[1,3]=(-0.122803946559)*x[0]**o+(-0.506542460419)*x[0]+(-0.993759502512)*x[1]**o+(-0.335687805433)*x[1]+(0.143963376404)*x[2]**o+(-0.980235207299)*x[2]
        s[1,4]=(-0.998831548478)*x[0]**o+(0.985193839232)*x[0]+(-0.310482397827)*x[1]**o+(0.487407650791)*x[1]+(-0.974332702676)*x[2]**o+(-0.0921055039133)*x[2]
        s[2,0]=(-0.160666729317)*x[0]**o+(0.250220171076)*x[0]+(-0.868648917809)*x[1]**o+(-0.711318604851)*x[1]+(0.115301101729)*x[2]**o+(0.231696383247)*x[2]
        s[2,1]=(-0.775210051641)*x[0]**o+(0.586897738775)*x[0]+(0.449891876198)*x[1]**o+(-0.326417063196)*x[1]+(-0.693635538015)*x[2]**o+(-0.180803736532)*x[2]
        s[2,2]=(-0.208290352953)*x[0]**o+(-0.447870020368)*x[0]+(-0.518392174915)*x[1]**o+(-0.183558767418)*x[1]+(-0.496457835849)*x[2]**o+(-0.423284073396)*x[2]
        s[2,3]=(-0.994564109782)*x[0]**o+(-0.229677912537)*x[0]+(0.563307213686)*x[1]**o+(0.922013593154)*x[1]+(-0.371937088648)*x[2]**o+(-0.243995790101)*x[2]
        s[2,4]=(0.108680732756)*x[0]**o+(0.8978812368)*x[0]+(0.672348022342)*x[1]**o+(0.316624749349)*x[1]+(0.304319187223)*x[2]**o+(-0.604575730332)*x[2]
        s[3,0]=(0.0734716079145)*x[0]**o+(-0.627088377782)*x[0]+(0.620453085026)*x[1]**o+(0.331742115021)*x[1]+(-0.166118547603)*x[2]**o+(0.808314484323)*x[2]
        s[3,1]=(-0.0808492395011)*x[0]**o+(-0.0938687820054)*x[0]+(-0.888326387847)*x[1]**o+(-0.608268617177)*x[1]+(-0.316327102759)*x[2]**o+(-0.592763199764)*x[2]
        s[3,2]=(-0.255058846163)*x[0]**o+(0.103273543614)*x[0]+(-0.976707756405)*x[1]**o+(-0.723115640951)*x[1]+(-0.871816562933)*x[2]**o+(0.0248677291123)*x[2]
        s[3,3]=(-0.128009946516)*x[0]**o+(0.285430888656)*x[0]+(0.660763158647)*x[1]**o+(-0.96689913729)*x[1]+(-0.512425011889)*x[2]**o+(0.12926532516)*x[2]
        s[3,4]=(0.374659980563)*x[0]**o+(-0.13005773825)*x[0]+(-0.491643647934)*x[1]**o+(0.536781460391)*x[1]+(0.484891534824)*x[2]**o+(0.764854714727)*x[2]
        ref[0,0]=(0.817784424934)*x_ref[0]**o+(0.140829586694)*x_ref[0]+(0.395723095233)*x_ref[1]**o+(-0.0916449245097)*x_ref[1]+(-0.686149802471)*x_ref[2]**o+(0.231879617817)*x_ref[2]
        ref[0,1]=(-0.753947196887)*x_ref[0]**o+(0.0184422308138)*x_ref[0]+(0.38127599678)*x_ref[1]**o+(-0.958287760938)*x_ref[1]+(-0.230273819981)*x_ref[2]**o+(-0.922603290238)*x_ref[2]
        ref[0,2]=(0.44304320962)*x_ref[0]**o+(0.35396422013)*x_ref[0]+(-0.479605019935)*x_ref[1]**o+(0.289724501875)*x_ref[1]+(0.657449032305)*x_ref[2]**o+(-0.706689994866)*x_ref[2]
        ref[0,3]=(0.0214930482601)*x_ref[0]**o+(-0.915710233423)*x_ref[0]+(-0.571112565179)*x_ref[1]**o+(0.837855622741)*x_ref[1]+(-0.402178902773)*x_ref[2]**o+(-0.113536055723)*x_ref[2]
        ref[0,4]=(0.0332368088464)*x_ref[0]**o+(0.488860583014)*x_ref[0]+(0.687230732879)*x_ref[1]**o+(0.417177289784)*x_ref[1]+(-0.138583225765)*x_ref[2]**o+(-0.288206976222)*x_ref[2]
        ref[1,0]=(-0.571959772672)*x_ref[0]**o+(-0.806968233432)*x_ref[0]+(-0.951330341336)*x_ref[1]**o+(-0.725871940445)*x_ref[1]+(0.473279498914)*x_ref[2]**o+(0.693882912652)*x_ref[2]
        ref[1,1]=(-0.925147598095)*x_ref[0]**o+(0.612875097386)*x_ref[0]+(0.625107884558)*x_ref[1]**o+(0.665316395744)*x_ref[1]+(-0.012158847541)*x_ref[2]**o+(0.114880783467)*x_ref[2]
        ref[1,2]=(-0.361803346001)*x_ref[0]**o+(0.630266012707)*x_ref[0]+(-0.698127168765)*x_ref[1]**o+(-0.534017552333)*x_ref[1]+(0.968004682982)*x_ref[2]**o+(-0.607970906679)*x_ref[2]
        ref[1,3]=(-0.122803946559)*x_ref[0]**o+(-0.506542460419)*x_ref[0]+(-0.993759502512)*x_ref[1]**o+(-0.335687805433)*x_ref[1]+(0.143963376404)*x_ref[2]**o+(-0.980235207299)*x_ref[2]
        ref[1,4]=(-0.998831548478)*x_ref[0]**o+(0.985193839232)*x_ref[0]+(-0.310482397827)*x_ref[1]**o+(0.487407650791)*x_ref[1]+(-0.974332702676)*x_ref[2]**o+(-0.0921055039133)*x_ref[2]
        ref[2,0]=(-0.160666729317)*x_ref[0]**o+(0.250220171076)*x_ref[0]+(-0.868648917809)*x_ref[1]**o+(-0.711318604851)*x_ref[1]+(0.115301101729)*x_ref[2]**o+(0.231696383247)*x_ref[2]
        ref[2,1]=(-0.775210051641)*x_ref[0]**o+(0.586897738775)*x_ref[0]+(0.449891876198)*x_ref[1]**o+(-0.326417063196)*x_ref[1]+(-0.693635538015)*x_ref[2]**o+(-0.180803736532)*x_ref[2]
        ref[2,2]=(-0.208290352953)*x_ref[0]**o+(-0.447870020368)*x_ref[0]+(-0.518392174915)*x_ref[1]**o+(-0.183558767418)*x_ref[1]+(-0.496457835849)*x_ref[2]**o+(-0.423284073396)*x_ref[2]
        ref[2,3]=(-0.994564109782)*x_ref[0]**o+(-0.229677912537)*x_ref[0]+(0.563307213686)*x_ref[1]**o+(0.922013593154)*x_ref[1]+(-0.371937088648)*x_ref[2]**o+(-0.243995790101)*x_ref[2]
        ref[2,4]=(0.108680732756)*x_ref[0]**o+(0.8978812368)*x_ref[0]+(0.672348022342)*x_ref[1]**o+(0.316624749349)*x_ref[1]+(0.304319187223)*x_ref[2]**o+(-0.604575730332)*x_ref[2]
        ref[3,0]=(0.0734716079145)*x_ref[0]**o+(-0.627088377782)*x_ref[0]+(0.620453085026)*x_ref[1]**o+(0.331742115021)*x_ref[1]+(-0.166118547603)*x_ref[2]**o+(0.808314484323)*x_ref[2]
        ref[3,1]=(-0.0808492395011)*x_ref[0]**o+(-0.0938687820054)*x_ref[0]+(-0.888326387847)*x_ref[1]**o+(-0.608268617177)*x_ref[1]+(-0.316327102759)*x_ref[2]**o+(-0.592763199764)*x_ref[2]
        ref[3,2]=(-0.255058846163)*x_ref[0]**o+(0.103273543614)*x_ref[0]+(-0.976707756405)*x_ref[1]**o+(-0.723115640951)*x_ref[1]+(-0.871816562933)*x_ref[2]**o+(0.0248677291123)*x_ref[2]
        ref[3,3]=(-0.128009946516)*x_ref[0]**o+(0.285430888656)*x_ref[0]+(0.660763158647)*x_ref[1]**o+(-0.96689913729)*x_ref[1]+(-0.512425011889)*x_ref[2]**o+(0.12926532516)*x_ref[2]
        ref[3,4]=(0.374659980563)*x_ref[0]**o+(-0.13005773825)*x_ref[0]+(-0.491643647934)*x_ref[1]**o+(0.536781460391)*x_ref[1]+(0.484891534824)*x_ref[2]**o+(0.764854714727)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_ReducedFunctionOnContactOne_rank3(self):
      """
      tests interpolate for rank 3 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactOne onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactOne(self.domain)
      x=w.getX()
      arg=Symbol(shape=(6, 2, 2),dim=dim)
      s=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2),w_ref)
      if dim==2:
        s[0,0,0]=(-0.395079492423)*x[0]**o+(-0.627751628747)*x[0]+(-0.754320526654)*x[1]**o+(-0.498408852477)*x[1]
        s[0,0,1]=(0.498446911285)*x[0]**o+(0.0596046404343)*x[0]+(0.667944256533)*x[1]**o+(0.80046807163)*x[1]
        s[0,1,0]=(-0.0130333830081)*x[0]**o+(0.649913235604)*x[0]+(-0.374473401736)*x[1]**o+(-0.0186132358452)*x[1]
        s[0,1,1]=(-0.390035543793)*x[0]**o+(0.673194387531)*x[0]+(-0.867455391881)*x[1]**o+(-0.510668203571)*x[1]
        s[1,0,0]=(0.978997680408)*x[0]**o+(-0.343461179415)*x[0]+(0.582713911591)*x[1]**o+(-0.165697613343)*x[1]
        s[1,0,1]=(0.0923732379474)*x[0]**o+(-0.585219326869)*x[0]+(0.540073721296)*x[1]**o+(0.027063080312)*x[1]
        s[1,1,0]=(0.590126162972)*x[0]**o+(-0.136675336981)*x[0]+(-0.376003854007)*x[1]**o+(0.966535168787)*x[1]
        s[1,1,1]=(-0.493032436853)*x[0]**o+(0.239853630037)*x[0]+(-0.602522863181)*x[1]**o+(0.33785945991)*x[1]
        s[2,0,0]=(-0.529642904818)*x[0]**o+(-0.43628377125)*x[0]+(-0.996498099393)*x[1]**o+(-0.0591954433263)*x[1]
        s[2,0,1]=(-0.235182234411)*x[0]**o+(0.811387084032)*x[0]+(-0.443296230671)*x[1]**o+(-0.3314686181)*x[1]
        s[2,1,0]=(-0.110109607717)*x[0]**o+(-0.1409604355)*x[0]+(0.284844298465)*x[1]**o+(0.714735331234)*x[1]
        s[2,1,1]=(-0.936889720851)*x[0]**o+(-0.00858197667802)*x[0]+(0.680058166799)*x[1]**o+(0.436911645255)*x[1]
        s[3,0,0]=(-0.478297335601)*x[0]**o+(0.686114547145)*x[0]+(0.234700452186)*x[1]**o+(-0.330970013839)*x[1]
        s[3,0,1]=(0.975338382165)*x[0]**o+(0.607523981612)*x[0]+(-0.616521819955)*x[1]**o+(-0.292932591377)*x[1]
        s[3,1,0]=(0.987717426508)*x[0]**o+(0.6507515036)*x[0]+(-0.195762979545)*x[1]**o+(-0.667650116614)*x[1]
        s[3,1,1]=(0.53886053148)*x[0]**o+(-0.491668085193)*x[0]+(0.486615664224)*x[1]**o+(0.741560383409)*x[1]
        s[4,0,0]=(-0.936467387625)*x[0]**o+(-0.142421597802)*x[0]+(0.733595719048)*x[1]**o+(0.402826551348)*x[1]
        s[4,0,1]=(0.64501254965)*x[0]**o+(-0.660424630011)*x[0]+(-0.221052224259)*x[1]**o+(-0.199679968927)*x[1]
        s[4,1,0]=(0.809693682851)*x[0]**o+(-0.614404151383)*x[0]+(-0.754545121479)*x[1]**o+(-0.603231220423)*x[1]
        s[4,1,1]=(0.709936703222)*x[0]**o+(-0.389296803297)*x[0]+(-0.271161950977)*x[1]**o+(-0.929259781943)*x[1]
        s[5,0,0]=(-0.267950850602)*x[0]**o+(-0.660075303261)*x[0]+(0.0304564867312)*x[1]**o+(-0.42168319765)*x[1]
        s[5,0,1]=(0.137764595135)*x[0]**o+(0.265554426237)*x[0]+(-0.435216862466)*x[1]**o+(-0.338253702328)*x[1]
        s[5,1,0]=(0.858568660192)*x[0]**o+(-0.18371104351)*x[0]+(0.665662082165)*x[1]**o+(0.0684606399813)*x[1]
        s[5,1,1]=(0.897687912811)*x[0]**o+(0.675844284819)*x[0]+(0.0430574862378)*x[1]**o+(0.371728813053)*x[1]
        ref[0,0,0]=(-0.395079492423)*x_ref[0]**o+(-0.627751628747)*x_ref[0]+(-0.754320526654)*x_ref[1]**o+(-0.498408852477)*x_ref[1]
        ref[0,0,1]=(0.498446911285)*x_ref[0]**o+(0.0596046404343)*x_ref[0]+(0.667944256533)*x_ref[1]**o+(0.80046807163)*x_ref[1]
        ref[0,1,0]=(-0.0130333830081)*x_ref[0]**o+(0.649913235604)*x_ref[0]+(-0.374473401736)*x_ref[1]**o+(-0.0186132358452)*x_ref[1]
        ref[0,1,1]=(-0.390035543793)*x_ref[0]**o+(0.673194387531)*x_ref[0]+(-0.867455391881)*x_ref[1]**o+(-0.510668203571)*x_ref[1]
        ref[1,0,0]=(0.978997680408)*x_ref[0]**o+(-0.343461179415)*x_ref[0]+(0.582713911591)*x_ref[1]**o+(-0.165697613343)*x_ref[1]
        ref[1,0,1]=(0.0923732379474)*x_ref[0]**o+(-0.585219326869)*x_ref[0]+(0.540073721296)*x_ref[1]**o+(0.027063080312)*x_ref[1]
        ref[1,1,0]=(0.590126162972)*x_ref[0]**o+(-0.136675336981)*x_ref[0]+(-0.376003854007)*x_ref[1]**o+(0.966535168787)*x_ref[1]
        ref[1,1,1]=(-0.493032436853)*x_ref[0]**o+(0.239853630037)*x_ref[0]+(-0.602522863181)*x_ref[1]**o+(0.33785945991)*x_ref[1]
        ref[2,0,0]=(-0.529642904818)*x_ref[0]**o+(-0.43628377125)*x_ref[0]+(-0.996498099393)*x_ref[1]**o+(-0.0591954433263)*x_ref[1]
        ref[2,0,1]=(-0.235182234411)*x_ref[0]**o+(0.811387084032)*x_ref[0]+(-0.443296230671)*x_ref[1]**o+(-0.3314686181)*x_ref[1]
        ref[2,1,0]=(-0.110109607717)*x_ref[0]**o+(-0.1409604355)*x_ref[0]+(0.284844298465)*x_ref[1]**o+(0.714735331234)*x_ref[1]
        ref[2,1,1]=(-0.936889720851)*x_ref[0]**o+(-0.00858197667802)*x_ref[0]+(0.680058166799)*x_ref[1]**o+(0.436911645255)*x_ref[1]
        ref[3,0,0]=(-0.478297335601)*x_ref[0]**o+(0.686114547145)*x_ref[0]+(0.234700452186)*x_ref[1]**o+(-0.330970013839)*x_ref[1]
        ref[3,0,1]=(0.975338382165)*x_ref[0]**o+(0.607523981612)*x_ref[0]+(-0.616521819955)*x_ref[1]**o+(-0.292932591377)*x_ref[1]
        ref[3,1,0]=(0.987717426508)*x_ref[0]**o+(0.6507515036)*x_ref[0]+(-0.195762979545)*x_ref[1]**o+(-0.667650116614)*x_ref[1]
        ref[3,1,1]=(0.53886053148)*x_ref[0]**o+(-0.491668085193)*x_ref[0]+(0.486615664224)*x_ref[1]**o+(0.741560383409)*x_ref[1]
        ref[4,0,0]=(-0.936467387625)*x_ref[0]**o+(-0.142421597802)*x_ref[0]+(0.733595719048)*x_ref[1]**o+(0.402826551348)*x_ref[1]
        ref[4,0,1]=(0.64501254965)*x_ref[0]**o+(-0.660424630011)*x_ref[0]+(-0.221052224259)*x_ref[1]**o+(-0.199679968927)*x_ref[1]
        ref[4,1,0]=(0.809693682851)*x_ref[0]**o+(-0.614404151383)*x_ref[0]+(-0.754545121479)*x_ref[1]**o+(-0.603231220423)*x_ref[1]
        ref[4,1,1]=(0.709936703222)*x_ref[0]**o+(-0.389296803297)*x_ref[0]+(-0.271161950977)*x_ref[1]**o+(-0.929259781943)*x_ref[1]
        ref[5,0,0]=(-0.267950850602)*x_ref[0]**o+(-0.660075303261)*x_ref[0]+(0.0304564867312)*x_ref[1]**o+(-0.42168319765)*x_ref[1]
        ref[5,0,1]=(0.137764595135)*x_ref[0]**o+(0.265554426237)*x_ref[0]+(-0.435216862466)*x_ref[1]**o+(-0.338253702328)*x_ref[1]
        ref[5,1,0]=(0.858568660192)*x_ref[0]**o+(-0.18371104351)*x_ref[0]+(0.665662082165)*x_ref[1]**o+(0.0684606399813)*x_ref[1]
        ref[5,1,1]=(0.897687912811)*x_ref[0]**o+(0.675844284819)*x_ref[0]+(0.0430574862378)*x_ref[1]**o+(0.371728813053)*x_ref[1]
      else:
        s[0,0,0]=(-0.589511962709)*x[0]**o+(0.159792477428)*x[0]+(0.766030256721)*x[1]**o+(-0.312062526449)*x[1]+(0.64700443985)*x[2]**o+(0.0518757184544)*x[2]
        s[0,0,1]=(-0.894062305961)*x[0]**o+(0.864404945522)*x[0]+(0.560280553158)*x[1]**o+(0.918283462394)*x[1]+(-0.490840733192)*x[2]**o+(0.697691943154)*x[2]
        s[0,1,0]=(-0.0882043414483)*x[0]**o+(-0.096537953899)*x[0]+(0.927057867899)*x[1]**o+(0.77613572949)*x[1]+(-0.987702345599)*x[2]**o+(-0.574645805199)*x[2]
        s[0,1,1]=(-0.66012117034)*x[0]**o+(-0.189179227113)*x[0]+(0.292721630825)*x[1]**o+(0.0170456979372)*x[1]+(0.0183109554333)*x[2]**o+(-0.917454875678)*x[2]
        s[1,0,0]=(0.654605700477)*x[0]**o+(-0.170526279436)*x[0]+(0.343326885913)*x[1]**o+(0.210563849271)*x[1]+(0.534291586724)*x[2]**o+(-0.755048841391)*x[2]
        s[1,0,1]=(-0.364686989491)*x[0]**o+(0.552984183344)*x[0]+(-0.470357667845)*x[1]**o+(-0.464300631313)*x[1]+(-0.438895038261)*x[2]**o+(0.518824978532)*x[2]
        s[1,1,0]=(-0.0456454803782)*x[0]**o+(-0.696772845921)*x[0]+(-0.587520555693)*x[1]**o+(0.676523672069)*x[1]+(-0.905996237596)*x[2]**o+(-0.474417867255)*x[2]
        s[1,1,1]=(0.360939068763)*x[0]**o+(-0.743105635563)*x[0]+(-0.615952199646)*x[1]**o+(-0.538063201399)*x[1]+(-0.898283738708)*x[2]**o+(-0.97634117952)*x[2]
        s[2,0,0]=(-0.185178359078)*x[0]**o+(0.224443281068)*x[0]+(0.754102418405)*x[1]**o+(-0.411697272338)*x[1]+(-0.528900607926)*x[2]**o+(-0.807089685705)*x[2]
        s[2,0,1]=(-0.784480844715)*x[0]**o+(-0.823404986088)*x[0]+(0.749752559055)*x[1]**o+(-0.993803702434)*x[1]+(-0.03194828585)*x[2]**o+(-0.104230991195)*x[2]
        s[2,1,0]=(-0.3975495117)*x[0]**o+(-0.921611510193)*x[0]+(0.364776109304)*x[1]**o+(0.568483551949)*x[1]+(0.92620487977)*x[2]**o+(-0.203075275658)*x[2]
        s[2,1,1]=(0.483830827389)*x[0]**o+(-0.16426527248)*x[0]+(0.829148557161)*x[1]**o+(-0.476785306719)*x[1]+(0.498338738131)*x[2]**o+(-0.335483518227)*x[2]
        s[3,0,0]=(-0.673980780716)*x[0]**o+(0.565441119757)*x[0]+(-0.440135530887)*x[1]**o+(-0.505046260809)*x[1]+(-0.37380888836)*x[2]**o+(0.405503742087)*x[2]
        s[3,0,1]=(-0.330871586339)*x[0]**o+(-0.590160971509)*x[0]+(-0.737041178139)*x[1]**o+(0.998222421798)*x[1]+(0.910277176468)*x[2]**o+(0.429491322427)*x[2]
        s[3,1,0]=(-0.166111866877)*x[0]**o+(-0.758927317121)*x[0]+(0.287263609892)*x[1]**o+(-0.322563818793)*x[1]+(-0.430564368296)*x[2]**o+(0.174322344232)*x[2]
        s[3,1,1]=(-0.661309380996)*x[0]**o+(0.139785030015)*x[0]+(-0.682140312514)*x[1]**o+(-0.654285952086)*x[1]+(0.622815328424)*x[2]**o+(0.485021479372)*x[2]
        s[4,0,0]=(-0.594196524006)*x[0]**o+(-0.539707458079)*x[0]+(-0.793206045899)*x[1]**o+(-0.65733164652)*x[1]+(0.853898902594)*x[2]**o+(-0.68578755422)*x[2]
        s[4,0,1]=(-0.368655085475)*x[0]**o+(0.965754821996)*x[0]+(0.796746162388)*x[1]**o+(-0.0792750556783)*x[1]+(0.865822523139)*x[2]**o+(0.824557599127)*x[2]
        s[4,1,0]=(0.0309342911768)*x[0]**o+(-0.57039195842)*x[0]+(0.314105965219)*x[1]**o+(0.991995648934)*x[1]+(-0.344915004359)*x[2]**o+(0.461898675894)*x[2]
        s[4,1,1]=(0.287549691013)*x[0]**o+(0.342272217628)*x[0]+(-0.469131330709)*x[1]**o+(0.505146423329)*x[1]+(0.347751545765)*x[2]**o+(0.547278978185)*x[2]
        s[5,0,0]=(-0.884852695364)*x[0]**o+(-0.642157963952)*x[0]+(-0.787121736521)*x[1]**o+(0.187833667108)*x[1]+(-0.339842450451)*x[2]**o+(0.174012710957)*x[2]
        s[5,0,1]=(0.28878191697)*x[0]**o+(-0.360576326583)*x[0]+(-0.0945722807072)*x[1]**o+(-0.587351566921)*x[1]+(-0.933568527838)*x[2]**o+(-0.60716101577)*x[2]
        s[5,1,0]=(-0.581405265173)*x[0]**o+(-0.015810740193)*x[0]+(0.919122906998)*x[1]**o+(-0.676080492473)*x[1]+(0.0255111996852)*x[2]**o+(-0.693486219028)*x[2]
        s[5,1,1]=(-0.941020084511)*x[0]**o+(-0.330527617459)*x[0]+(0.98542398649)*x[1]**o+(-0.154193758572)*x[1]+(0.876454677452)*x[2]**o+(-0.278575401803)*x[2]
        ref[0,0,0]=(-0.589511962709)*x_ref[0]**o+(0.159792477428)*x_ref[0]+(0.766030256721)*x_ref[1]**o+(-0.312062526449)*x_ref[1]+(0.64700443985)*x_ref[2]**o+(0.0518757184544)*x_ref[2]
        ref[0,0,1]=(-0.894062305961)*x_ref[0]**o+(0.864404945522)*x_ref[0]+(0.560280553158)*x_ref[1]**o+(0.918283462394)*x_ref[1]+(-0.490840733192)*x_ref[2]**o+(0.697691943154)*x_ref[2]
        ref[0,1,0]=(-0.0882043414483)*x_ref[0]**o+(-0.096537953899)*x_ref[0]+(0.927057867899)*x_ref[1]**o+(0.77613572949)*x_ref[1]+(-0.987702345599)*x_ref[2]**o+(-0.574645805199)*x_ref[2]
        ref[0,1,1]=(-0.66012117034)*x_ref[0]**o+(-0.189179227113)*x_ref[0]+(0.292721630825)*x_ref[1]**o+(0.0170456979372)*x_ref[1]+(0.0183109554333)*x_ref[2]**o+(-0.917454875678)*x_ref[2]
        ref[1,0,0]=(0.654605700477)*x_ref[0]**o+(-0.170526279436)*x_ref[0]+(0.343326885913)*x_ref[1]**o+(0.210563849271)*x_ref[1]+(0.534291586724)*x_ref[2]**o+(-0.755048841391)*x_ref[2]
        ref[1,0,1]=(-0.364686989491)*x_ref[0]**o+(0.552984183344)*x_ref[0]+(-0.470357667845)*x_ref[1]**o+(-0.464300631313)*x_ref[1]+(-0.438895038261)*x_ref[2]**o+(0.518824978532)*x_ref[2]
        ref[1,1,0]=(-0.0456454803782)*x_ref[0]**o+(-0.696772845921)*x_ref[0]+(-0.587520555693)*x_ref[1]**o+(0.676523672069)*x_ref[1]+(-0.905996237596)*x_ref[2]**o+(-0.474417867255)*x_ref[2]
        ref[1,1,1]=(0.360939068763)*x_ref[0]**o+(-0.743105635563)*x_ref[0]+(-0.615952199646)*x_ref[1]**o+(-0.538063201399)*x_ref[1]+(-0.898283738708)*x_ref[2]**o+(-0.97634117952)*x_ref[2]
        ref[2,0,0]=(-0.185178359078)*x_ref[0]**o+(0.224443281068)*x_ref[0]+(0.754102418405)*x_ref[1]**o+(-0.411697272338)*x_ref[1]+(-0.528900607926)*x_ref[2]**o+(-0.807089685705)*x_ref[2]
        ref[2,0,1]=(-0.784480844715)*x_ref[0]**o+(-0.823404986088)*x_ref[0]+(0.749752559055)*x_ref[1]**o+(-0.993803702434)*x_ref[1]+(-0.03194828585)*x_ref[2]**o+(-0.104230991195)*x_ref[2]
        ref[2,1,0]=(-0.3975495117)*x_ref[0]**o+(-0.921611510193)*x_ref[0]+(0.364776109304)*x_ref[1]**o+(0.568483551949)*x_ref[1]+(0.92620487977)*x_ref[2]**o+(-0.203075275658)*x_ref[2]
        ref[2,1,1]=(0.483830827389)*x_ref[0]**o+(-0.16426527248)*x_ref[0]+(0.829148557161)*x_ref[1]**o+(-0.476785306719)*x_ref[1]+(0.498338738131)*x_ref[2]**o+(-0.335483518227)*x_ref[2]
        ref[3,0,0]=(-0.673980780716)*x_ref[0]**o+(0.565441119757)*x_ref[0]+(-0.440135530887)*x_ref[1]**o+(-0.505046260809)*x_ref[1]+(-0.37380888836)*x_ref[2]**o+(0.405503742087)*x_ref[2]
        ref[3,0,1]=(-0.330871586339)*x_ref[0]**o+(-0.590160971509)*x_ref[0]+(-0.737041178139)*x_ref[1]**o+(0.998222421798)*x_ref[1]+(0.910277176468)*x_ref[2]**o+(0.429491322427)*x_ref[2]
        ref[3,1,0]=(-0.166111866877)*x_ref[0]**o+(-0.758927317121)*x_ref[0]+(0.287263609892)*x_ref[1]**o+(-0.322563818793)*x_ref[1]+(-0.430564368296)*x_ref[2]**o+(0.174322344232)*x_ref[2]
        ref[3,1,1]=(-0.661309380996)*x_ref[0]**o+(0.139785030015)*x_ref[0]+(-0.682140312514)*x_ref[1]**o+(-0.654285952086)*x_ref[1]+(0.622815328424)*x_ref[2]**o+(0.485021479372)*x_ref[2]
        ref[4,0,0]=(-0.594196524006)*x_ref[0]**o+(-0.539707458079)*x_ref[0]+(-0.793206045899)*x_ref[1]**o+(-0.65733164652)*x_ref[1]+(0.853898902594)*x_ref[2]**o+(-0.68578755422)*x_ref[2]
        ref[4,0,1]=(-0.368655085475)*x_ref[0]**o+(0.965754821996)*x_ref[0]+(0.796746162388)*x_ref[1]**o+(-0.0792750556783)*x_ref[1]+(0.865822523139)*x_ref[2]**o+(0.824557599127)*x_ref[2]
        ref[4,1,0]=(0.0309342911768)*x_ref[0]**o+(-0.57039195842)*x_ref[0]+(0.314105965219)*x_ref[1]**o+(0.991995648934)*x_ref[1]+(-0.344915004359)*x_ref[2]**o+(0.461898675894)*x_ref[2]
        ref[4,1,1]=(0.287549691013)*x_ref[0]**o+(0.342272217628)*x_ref[0]+(-0.469131330709)*x_ref[1]**o+(0.505146423329)*x_ref[1]+(0.347751545765)*x_ref[2]**o+(0.547278978185)*x_ref[2]
        ref[5,0,0]=(-0.884852695364)*x_ref[0]**o+(-0.642157963952)*x_ref[0]+(-0.787121736521)*x_ref[1]**o+(0.187833667108)*x_ref[1]+(-0.339842450451)*x_ref[2]**o+(0.174012710957)*x_ref[2]
        ref[5,0,1]=(0.28878191697)*x_ref[0]**o+(-0.360576326583)*x_ref[0]+(-0.0945722807072)*x_ref[1]**o+(-0.587351566921)*x_ref[1]+(-0.933568527838)*x_ref[2]**o+(-0.60716101577)*x_ref[2]
        ref[5,1,0]=(-0.581405265173)*x_ref[0]**o+(-0.015810740193)*x_ref[0]+(0.919122906998)*x_ref[1]**o+(-0.676080492473)*x_ref[1]+(0.0255111996852)*x_ref[2]**o+(-0.693486219028)*x_ref[2]
        ref[5,1,1]=(-0.941020084511)*x_ref[0]**o+(-0.330527617459)*x_ref[0]+(0.98542398649)*x_ref[1]**o+(-0.154193758572)*x_ref[1]+(0.876454677452)*x_ref[2]**o+(-0.278575401803)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_ReducedFunctionOnContactOne_rank4(self):
      """
      tests interpolate for rank 4 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactOne onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactOne(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5, 3, 2),dim=dim)
      s=Data(0,(4, 5, 3, 2),w)
      ref=Data(0,(4, 5, 3, 2),w_ref)
      if dim==2:
        s[0,0,0,0]=(-0.939524201884)*x[0]**o+(-0.294114596857)*x[0]+(-0.662571514204)*x[1]**o+(-0.295055230761)*x[1]
        s[0,0,0,1]=(0.174447802399)*x[0]**o+(0.313121933505)*x[0]+(0.780423776572)*x[1]**o+(0.260329316622)*x[1]
        s[0,0,1,0]=(-0.307907192496)*x[0]**o+(0.924386500426)*x[0]+(-0.34849235247)*x[1]**o+(0.42450311854)*x[1]
        s[0,0,1,1]=(-0.221818525078)*x[0]**o+(0.334658153768)*x[0]+(0.730236845709)*x[1]**o+(0.923645750095)*x[1]
        s[0,0,2,0]=(-0.0573462957426)*x[0]**o+(0.10400906253)*x[0]+(-0.606757296289)*x[1]**o+(-0.171453259849)*x[1]
        s[0,0,2,1]=(-0.653190260251)*x[0]**o+(0.392209523496)*x[0]+(0.984792984469)*x[1]**o+(-0.310847119118)*x[1]
        s[0,1,0,0]=(-0.0980551705202)*x[0]**o+(0.475337517365)*x[0]+(-0.93300381011)*x[1]**o+(0.0593095200419)*x[1]
        s[0,1,0,1]=(0.833302217025)*x[0]**o+(-0.866194985925)*x[0]+(0.337967734176)*x[1]**o+(0.0581478569311)*x[1]
        s[0,1,1,0]=(0.936751875211)*x[0]**o+(-0.217226318173)*x[0]+(0.477011461247)*x[1]**o+(0.221436266198)*x[1]
        s[0,1,1,1]=(-0.537625551791)*x[0]**o+(0.718351842961)*x[0]+(0.0286304587001)*x[1]**o+(-0.721442785761)*x[1]
        s[0,1,2,0]=(-0.738645216931)*x[0]**o+(0.0195980002286)*x[0]+(0.827363980625)*x[1]**o+(-0.743955943985)*x[1]
        s[0,1,2,1]=(-0.145033137282)*x[0]**o+(0.98069402199)*x[0]+(-0.167153611864)*x[1]**o+(-0.577697516598)*x[1]
        s[0,2,0,0]=(0.594926068713)*x[0]**o+(-0.124581510742)*x[0]+(-0.392512448989)*x[1]**o+(-0.989702605289)*x[1]
        s[0,2,0,1]=(-0.208680726694)*x[0]**o+(0.0226857988067)*x[0]+(0.0127574486126)*x[1]**o+(-0.661169282937)*x[1]
        s[0,2,1,0]=(0.43109296375)*x[0]**o+(-0.966243011304)*x[0]+(-0.448863362387)*x[1]**o+(-0.531981406137)*x[1]
        s[0,2,1,1]=(0.427441889618)*x[0]**o+(-0.730863645031)*x[0]+(0.0948615738498)*x[1]**o+(0.349409965532)*x[1]
        s[0,2,2,0]=(0.0857935500808)*x[0]**o+(0.916612852535)*x[0]+(-0.5434740164)*x[1]**o+(0.651670987189)*x[1]
        s[0,2,2,1]=(0.269775748844)*x[0]**o+(-0.351727930666)*x[0]+(0.648234694172)*x[1]**o+(0.714331370173)*x[1]
        s[0,3,0,0]=(0.862594479045)*x[0]**o+(-0.787345711339)*x[0]+(0.1934849213)*x[1]**o+(-0.977224028721)*x[1]
        s[0,3,0,1]=(0.166215962959)*x[0]**o+(0.734752140512)*x[0]+(0.373558020776)*x[1]**o+(-0.589819927874)*x[1]
        s[0,3,1,0]=(0.0289724459782)*x[0]**o+(0.517327391578)*x[0]+(0.109956624336)*x[1]**o+(-0.290855234976)*x[1]
        s[0,3,1,1]=(0.369852037916)*x[0]**o+(0.184556562048)*x[0]+(-0.694630650679)*x[1]**o+(-0.526020870229)*x[1]
        s[0,3,2,0]=(0.397498255324)*x[0]**o+(-0.375327133358)*x[0]+(0.564571187906)*x[1]**o+(-0.295820211045)*x[1]
        s[0,3,2,1]=(0.550532413074)*x[0]**o+(-0.751307255234)*x[0]+(0.323446174542)*x[1]**o+(0.968475158376)*x[1]
        s[0,4,0,0]=(-0.112506956442)*x[0]**o+(-0.432032518771)*x[0]+(0.0205217950343)*x[1]**o+(-0.507714722498)*x[1]
        s[0,4,0,1]=(-0.501932335824)*x[0]**o+(-0.147371573026)*x[0]+(-0.421130372732)*x[1]**o+(-0.958029815205)*x[1]
        s[0,4,1,0]=(0.172226819831)*x[0]**o+(-0.712313983987)*x[0]+(0.859887204781)*x[1]**o+(0.0661613029313)*x[1]
        s[0,4,1,1]=(-0.863367396226)*x[0]**o+(0.150655310131)*x[0]+(-0.446461969683)*x[1]**o+(0.247377395525)*x[1]
        s[0,4,2,0]=(-0.0431980822723)*x[0]**o+(0.676680822195)*x[0]+(-0.613277443463)*x[1]**o+(0.787356918727)*x[1]
        s[0,4,2,1]=(-0.0569719722636)*x[0]**o+(0.641187476483)*x[0]+(-0.303617110099)*x[1]**o+(-0.888820557309)*x[1]
        s[1,0,0,0]=(-0.0552307803637)*x[0]**o+(0.306010657529)*x[0]+(0.488223930134)*x[1]**o+(-0.682392555784)*x[1]
        s[1,0,0,1]=(-0.253901726526)*x[0]**o+(0.73343629071)*x[0]+(-0.390491326134)*x[1]**o+(0.774605582697)*x[1]
        s[1,0,1,0]=(-0.42967954588)*x[0]**o+(-0.751284399784)*x[0]+(0.813738133871)*x[1]**o+(-0.0995163058125)*x[1]
        s[1,0,1,1]=(-0.0880959500234)*x[0]**o+(-0.781527901178)*x[0]+(-0.496899067642)*x[1]**o+(0.55524107606)*x[1]
        s[1,0,2,0]=(0.256546278479)*x[0]**o+(-0.595310894415)*x[0]+(-0.281823345122)*x[1]**o+(-0.953457305769)*x[1]
        s[1,0,2,1]=(0.365266273739)*x[0]**o+(-0.935192596652)*x[0]+(0.740921701447)*x[1]**o+(0.843620235195)*x[1]
        s[1,1,0,0]=(-0.992209271955)*x[0]**o+(0.267499041637)*x[0]+(-0.446633252289)*x[1]**o+(0.357054618942)*x[1]
        s[1,1,0,1]=(0.863355358433)*x[0]**o+(-0.104881968217)*x[0]+(0.0776056200756)*x[1]**o+(0.485205982195)*x[1]
        s[1,1,1,0]=(0.29822878169)*x[0]**o+(0.766559198241)*x[0]+(0.489839749671)*x[1]**o+(-0.694316951865)*x[1]
        s[1,1,1,1]=(-0.947135250126)*x[0]**o+(-0.952537408063)*x[0]+(-0.840569482634)*x[1]**o+(-0.0881389491948)*x[1]
        s[1,1,2,0]=(0.269982185854)*x[0]**o+(0.290850786316)*x[0]+(0.707550793741)*x[1]**o+(-0.583787562553)*x[1]
        s[1,1,2,1]=(-0.998478389198)*x[0]**o+(-0.902713626135)*x[0]+(-0.0809583498397)*x[1]**o+(0.242726761269)*x[1]
        s[1,2,0,0]=(-0.0752124155425)*x[0]**o+(0.287338426631)*x[0]+(-0.873281064449)*x[1]**o+(0.86906507097)*x[1]
        s[1,2,0,1]=(-0.451012489095)*x[0]**o+(0.692877058818)*x[0]+(-0.771278511209)*x[1]**o+(0.325983000966)*x[1]
        s[1,2,1,0]=(-0.633239034154)*x[0]**o+(0.751673044983)*x[0]+(-0.799706248119)*x[1]**o+(0.0783814978548)*x[1]
        s[1,2,1,1]=(-0.496449080562)*x[0]**o+(-0.382869354097)*x[0]+(-0.658792080006)*x[1]**o+(0.578499479656)*x[1]
        s[1,2,2,0]=(0.41041079346)*x[0]**o+(0.552454636943)*x[0]+(0.236050520221)*x[1]**o+(0.477587390293)*x[1]
        s[1,2,2,1]=(0.649795041558)*x[0]**o+(-0.947868231928)*x[0]+(-0.529895928082)*x[1]**o+(0.715695540026)*x[1]
        s[1,3,0,0]=(0.948112901731)*x[0]**o+(-0.374624145588)*x[0]+(0.806640283077)*x[1]**o+(0.729074938843)*x[1]
        s[1,3,0,1]=(0.266956301954)*x[0]**o+(-0.878004354786)*x[0]+(-0.422298760963)*x[1]**o+(-0.251922913127)*x[1]
        s[1,3,1,0]=(0.751159198422)*x[0]**o+(-0.576983418224)*x[0]+(0.427653287761)*x[1]**o+(0.351116956206)*x[1]
        s[1,3,1,1]=(-0.804702781998)*x[0]**o+(-0.218650676384)*x[0]+(-0.832220670207)*x[1]**o+(0.997469993794)*x[1]
        s[1,3,2,0]=(0.111110638087)*x[0]**o+(-0.435831664904)*x[0]+(-0.144481003079)*x[1]**o+(-0.604349850119)*x[1]
        s[1,3,2,1]=(0.470311907248)*x[0]**o+(-0.591709930358)*x[0]+(-0.562662398681)*x[1]**o+(0.0163307873851)*x[1]
        s[1,4,0,0]=(0.684349728229)*x[0]**o+(0.404890565831)*x[0]+(0.922438828147)*x[1]**o+(0.939539648586)*x[1]
        s[1,4,0,1]=(-0.179161968364)*x[0]**o+(0.983188684135)*x[0]+(-0.722578271046)*x[1]**o+(-0.450472366251)*x[1]
        s[1,4,1,0]=(0.243718494318)*x[0]**o+(-0.0251174594804)*x[0]+(0.952890549291)*x[1]**o+(-0.398408844924)*x[1]
        s[1,4,1,1]=(-0.883090916963)*x[0]**o+(-0.236073018867)*x[0]+(-0.245092152479)*x[1]**o+(0.846958498274)*x[1]
        s[1,4,2,0]=(-0.839928016329)*x[0]**o+(-0.677734468188)*x[0]+(-0.123639765438)*x[1]**o+(-0.680672570927)*x[1]
        s[1,4,2,1]=(-0.234629335025)*x[0]**o+(-0.567381745605)*x[0]+(0.623480113756)*x[1]**o+(0.2049427452)*x[1]
        s[2,0,0,0]=(-0.297265663727)*x[0]**o+(0.00899195467866)*x[0]+(0.992475896282)*x[1]**o+(0.863451618621)*x[1]
        s[2,0,0,1]=(-0.24914517208)*x[0]**o+(0.104091943078)*x[0]+(0.762436787588)*x[1]**o+(0.89363657955)*x[1]
        s[2,0,1,0]=(0.264734056412)*x[0]**o+(-0.716868164934)*x[0]+(-0.905285068877)*x[1]**o+(0.0227117696273)*x[1]
        s[2,0,1,1]=(-0.932617617299)*x[0]**o+(0.271634450254)*x[0]+(0.51812940988)*x[1]**o+(-0.484525267705)*x[1]
        s[2,0,2,0]=(0.364599957461)*x[0]**o+(0.818588026862)*x[0]+(0.70366241863)*x[1]**o+(0.43910943526)*x[1]
        s[2,0,2,1]=(0.557891341305)*x[0]**o+(-0.891810635437)*x[0]+(0.649138154483)*x[1]**o+(-0.213951745119)*x[1]
        s[2,1,0,0]=(0.994667066111)*x[0]**o+(0.433110731778)*x[0]+(0.787983438535)*x[1]**o+(0.979717998129)*x[1]
        s[2,1,0,1]=(0.619284628164)*x[0]**o+(0.662998143683)*x[0]+(0.0328849267442)*x[1]**o+(0.765881868994)*x[1]
        s[2,1,1,0]=(0.335939940328)*x[0]**o+(0.556758825457)*x[0]+(-0.524870937642)*x[1]**o+(0.0316796705422)*x[1]
        s[2,1,1,1]=(-0.702249603437)*x[0]**o+(0.544642480464)*x[0]+(0.330520285989)*x[1]**o+(0.313003559197)*x[1]
        s[2,1,2,0]=(0.0294247925115)*x[0]**o+(0.10067678181)*x[0]+(0.608148681239)*x[1]**o+(-0.95697997936)*x[1]
        s[2,1,2,1]=(0.101913506022)*x[0]**o+(-0.423061068258)*x[0]+(0.303518988539)*x[1]**o+(0.0108182161414)*x[1]
        s[2,2,0,0]=(0.390642040424)*x[0]**o+(0.0578342521895)*x[0]+(0.228499563768)*x[1]**o+(-0.944433241107)*x[1]
        s[2,2,0,1]=(0.522633870922)*x[0]**o+(0.724655140578)*x[0]+(-0.664760217849)*x[1]**o+(0.812535008015)*x[1]
        s[2,2,1,0]=(0.27608720491)*x[0]**o+(0.42350834695)*x[0]+(-0.732116287856)*x[1]**o+(0.930674651671)*x[1]
        s[2,2,1,1]=(0.214308063462)*x[0]**o+(0.815707217077)*x[0]+(0.231785784747)*x[1]**o+(-0.954429150944)*x[1]
        s[2,2,2,0]=(0.402814098347)*x[0]**o+(-0.650491293555)*x[0]+(-0.359159927669)*x[1]**o+(0.503842188774)*x[1]
        s[2,2,2,1]=(-0.191195415522)*x[0]**o+(0.980791691347)*x[0]+(0.228597878156)*x[1]**o+(0.535975172993)*x[1]
        s[2,3,0,0]=(0.495749327231)*x[0]**o+(0.311034011895)*x[0]+(-0.532686805928)*x[1]**o+(0.92438969176)*x[1]
        s[2,3,0,1]=(-0.38513599341)*x[0]**o+(-0.463099237819)*x[0]+(-0.992606328662)*x[1]**o+(-0.927920439344)*x[1]
        s[2,3,1,0]=(-0.868022271483)*x[0]**o+(-0.540096192748)*x[0]+(-0.0879858139685)*x[1]**o+(0.534604810517)*x[1]
        s[2,3,1,1]=(-0.686837754683)*x[0]**o+(-0.505585075528)*x[0]+(-0.218155668559)*x[1]**o+(-0.241972887878)*x[1]
        s[2,3,2,0]=(0.86187325655)*x[0]**o+(-0.733958307606)*x[0]+(-0.107393817742)*x[1]**o+(-0.830306745614)*x[1]
        s[2,3,2,1]=(0.404834271017)*x[0]**o+(0.44646270613)*x[0]+(-0.657038236282)*x[1]**o+(-0.540570795414)*x[1]
        s[2,4,0,0]=(-0.216828677229)*x[0]**o+(0.129702603598)*x[0]+(0.556920803376)*x[1]**o+(0.960021941421)*x[1]
        s[2,4,0,1]=(-0.21625137654)*x[0]**o+(-0.19604294366)*x[0]+(-0.889082170851)*x[1]**o+(-0.834967241538)*x[1]
        s[2,4,1,0]=(0.123104286845)*x[0]**o+(-0.43197703752)*x[0]+(0.737850827826)*x[1]**o+(0.344689639656)*x[1]
        s[2,4,1,1]=(0.119471348292)*x[0]**o+(0.948353928609)*x[0]+(-0.726863566036)*x[1]**o+(-0.957158639686)*x[1]
        s[2,4,2,0]=(0.606624368242)*x[0]**o+(-0.129636738653)*x[0]+(0.117469089776)*x[1]**o+(0.766024628396)*x[1]
        s[2,4,2,1]=(-0.83631907859)*x[0]**o+(-0.168433036335)*x[0]+(-0.500663355802)*x[1]**o+(0.0114866397792)*x[1]
        s[3,0,0,0]=(-0.629399168531)*x[0]**o+(-0.464524331386)*x[0]+(0.0406391749081)*x[1]**o+(-0.380881295086)*x[1]
        s[3,0,0,1]=(-0.614191170121)*x[0]**o+(0.90297284642)*x[0]+(0.949653922227)*x[1]**o+(0.947014467705)*x[1]
        s[3,0,1,0]=(0.128075669906)*x[0]**o+(-0.215027529464)*x[0]+(-0.230820603567)*x[1]**o+(0.656672788071)*x[1]
        s[3,0,1,1]=(0.11006414768)*x[0]**o+(0.575548749986)*x[0]+(0.803886215346)*x[1]**o+(0.627503866105)*x[1]
        s[3,0,2,0]=(-0.824524708364)*x[0]**o+(-0.399533304607)*x[0]+(-0.278996789484)*x[1]**o+(-0.139967848513)*x[1]
        s[3,0,2,1]=(-0.657392776251)*x[0]**o+(0.959283975752)*x[0]+(0.990715521701)*x[1]**o+(0.0999777335871)*x[1]
        s[3,1,0,0]=(-0.691053621801)*x[0]**o+(0.189005136467)*x[0]+(0.789186325959)*x[1]**o+(-0.981079749179)*x[1]
        s[3,1,0,1]=(0.228390867858)*x[0]**o+(-0.463941202938)*x[0]+(-0.782094886569)*x[1]**o+(0.464512245943)*x[1]
        s[3,1,1,0]=(-0.819255906497)*x[0]**o+(-0.204510874222)*x[0]+(-0.193473362908)*x[1]**o+(-0.0250165714593)*x[1]
        s[3,1,1,1]=(0.331636523937)*x[0]**o+(-0.0429730777685)*x[0]+(-0.547743299325)*x[1]**o+(-0.187953226002)*x[1]
        s[3,1,2,0]=(0.159370360993)*x[0]**o+(0.883315520778)*x[0]+(0.0933439209551)*x[1]**o+(-0.458319201953)*x[1]
        s[3,1,2,1]=(-0.273991258)*x[0]**o+(0.209739105337)*x[0]+(0.335210395431)*x[1]**o+(-0.91838746555)*x[1]
        s[3,2,0,0]=(-0.0618039563518)*x[0]**o+(-0.59799143228)*x[0]+(0.93203030688)*x[1]**o+(0.581545424351)*x[1]
        s[3,2,0,1]=(-0.321101903544)*x[0]**o+(-0.781260985212)*x[0]+(-0.106128211217)*x[1]**o+(0.913142993455)*x[1]
        s[3,2,1,0]=(0.584486365293)*x[0]**o+(-0.654479654421)*x[0]+(0.579992761397)*x[1]**o+(-0.984753896904)*x[1]
        s[3,2,1,1]=(0.186056629749)*x[0]**o+(-0.889254599383)*x[0]+(0.134406174831)*x[1]**o+(0.843880331117)*x[1]
        s[3,2,2,0]=(0.578447040133)*x[0]**o+(-0.354828599745)*x[0]+(0.943853840713)*x[1]**o+(0.668565941699)*x[1]
        s[3,2,2,1]=(-0.657678353823)*x[0]**o+(0.657623646562)*x[0]+(0.670144253782)*x[1]**o+(0.489533260867)*x[1]
        s[3,3,0,0]=(0.518241767069)*x[0]**o+(0.717113307079)*x[0]+(-0.986081844812)*x[1]**o+(-0.922915734137)*x[1]
        s[3,3,0,1]=(0.934651660463)*x[0]**o+(0.524253175591)*x[0]+(0.519169286504)*x[1]**o+(0.995798377114)*x[1]
        s[3,3,1,0]=(0.396606116937)*x[0]**o+(-0.479342680739)*x[0]+(0.677502711338)*x[1]**o+(-0.591869615963)*x[1]
        s[3,3,1,1]=(0.339631149039)*x[0]**o+(0.44124607198)*x[0]+(0.786875844013)*x[1]**o+(-0.925255814639)*x[1]
        s[3,3,2,0]=(-0.779510842649)*x[0]**o+(0.562505114965)*x[0]+(0.582094163828)*x[1]**o+(0.910521768351)*x[1]
        s[3,3,2,1]=(-0.661315883103)*x[0]**o+(0.226495727911)*x[0]+(-0.464093148756)*x[1]**o+(0.696758174053)*x[1]
        s[3,4,0,0]=(-0.0366658565966)*x[0]**o+(0.78669216474)*x[0]+(-0.784779466718)*x[1]**o+(0.137721843502)*x[1]
        s[3,4,0,1]=(-0.00884301625831)*x[0]**o+(0.25582694645)*x[0]+(-0.73173489905)*x[1]**o+(-0.956569853331)*x[1]
        s[3,4,1,0]=(-0.0884743360746)*x[0]**o+(-0.484605933801)*x[0]+(0.773028853802)*x[1]**o+(-0.13255190562)*x[1]
        s[3,4,1,1]=(0.597859646547)*x[0]**o+(-0.254684950139)*x[0]+(0.292389327091)*x[1]**o+(0.942836804031)*x[1]
        s[3,4,2,0]=(-0.82916540636)*x[0]**o+(0.533192478501)*x[0]+(0.577760569513)*x[1]**o+(-0.812132995663)*x[1]
        s[3,4,2,1]=(-0.0918171874792)*x[0]**o+(-0.232778513191)*x[0]+(-0.57345566951)*x[1]**o+(0.870122102)*x[1]
        ref[0,0,0,0]=(-0.939524201884)*x_ref[0]**o+(-0.294114596857)*x_ref[0]+(-0.662571514204)*x_ref[1]**o+(-0.295055230761)*x_ref[1]
        ref[0,0,0,1]=(0.174447802399)*x_ref[0]**o+(0.313121933505)*x_ref[0]+(0.780423776572)*x_ref[1]**o+(0.260329316622)*x_ref[1]
        ref[0,0,1,0]=(-0.307907192496)*x_ref[0]**o+(0.924386500426)*x_ref[0]+(-0.34849235247)*x_ref[1]**o+(0.42450311854)*x_ref[1]
        ref[0,0,1,1]=(-0.221818525078)*x_ref[0]**o+(0.334658153768)*x_ref[0]+(0.730236845709)*x_ref[1]**o+(0.923645750095)*x_ref[1]
        ref[0,0,2,0]=(-0.0573462957426)*x_ref[0]**o+(0.10400906253)*x_ref[0]+(-0.606757296289)*x_ref[1]**o+(-0.171453259849)*x_ref[1]
        ref[0,0,2,1]=(-0.653190260251)*x_ref[0]**o+(0.392209523496)*x_ref[0]+(0.984792984469)*x_ref[1]**o+(-0.310847119118)*x_ref[1]
        ref[0,1,0,0]=(-0.0980551705202)*x_ref[0]**o+(0.475337517365)*x_ref[0]+(-0.93300381011)*x_ref[1]**o+(0.0593095200419)*x_ref[1]
        ref[0,1,0,1]=(0.833302217025)*x_ref[0]**o+(-0.866194985925)*x_ref[0]+(0.337967734176)*x_ref[1]**o+(0.0581478569311)*x_ref[1]
        ref[0,1,1,0]=(0.936751875211)*x_ref[0]**o+(-0.217226318173)*x_ref[0]+(0.477011461247)*x_ref[1]**o+(0.221436266198)*x_ref[1]
        ref[0,1,1,1]=(-0.537625551791)*x_ref[0]**o+(0.718351842961)*x_ref[0]+(0.0286304587001)*x_ref[1]**o+(-0.721442785761)*x_ref[1]
        ref[0,1,2,0]=(-0.738645216931)*x_ref[0]**o+(0.0195980002286)*x_ref[0]+(0.827363980625)*x_ref[1]**o+(-0.743955943985)*x_ref[1]
        ref[0,1,2,1]=(-0.145033137282)*x_ref[0]**o+(0.98069402199)*x_ref[0]+(-0.167153611864)*x_ref[1]**o+(-0.577697516598)*x_ref[1]
        ref[0,2,0,0]=(0.594926068713)*x_ref[0]**o+(-0.124581510742)*x_ref[0]+(-0.392512448989)*x_ref[1]**o+(-0.989702605289)*x_ref[1]
        ref[0,2,0,1]=(-0.208680726694)*x_ref[0]**o+(0.0226857988067)*x_ref[0]+(0.0127574486126)*x_ref[1]**o+(-0.661169282937)*x_ref[1]
        ref[0,2,1,0]=(0.43109296375)*x_ref[0]**o+(-0.966243011304)*x_ref[0]+(-0.448863362387)*x_ref[1]**o+(-0.531981406137)*x_ref[1]
        ref[0,2,1,1]=(0.427441889618)*x_ref[0]**o+(-0.730863645031)*x_ref[0]+(0.0948615738498)*x_ref[1]**o+(0.349409965532)*x_ref[1]
        ref[0,2,2,0]=(0.0857935500808)*x_ref[0]**o+(0.916612852535)*x_ref[0]+(-0.5434740164)*x_ref[1]**o+(0.651670987189)*x_ref[1]
        ref[0,2,2,1]=(0.269775748844)*x_ref[0]**o+(-0.351727930666)*x_ref[0]+(0.648234694172)*x_ref[1]**o+(0.714331370173)*x_ref[1]
        ref[0,3,0,0]=(0.862594479045)*x_ref[0]**o+(-0.787345711339)*x_ref[0]+(0.1934849213)*x_ref[1]**o+(-0.977224028721)*x_ref[1]
        ref[0,3,0,1]=(0.166215962959)*x_ref[0]**o+(0.734752140512)*x_ref[0]+(0.373558020776)*x_ref[1]**o+(-0.589819927874)*x_ref[1]
        ref[0,3,1,0]=(0.0289724459782)*x_ref[0]**o+(0.517327391578)*x_ref[0]+(0.109956624336)*x_ref[1]**o+(-0.290855234976)*x_ref[1]
        ref[0,3,1,1]=(0.369852037916)*x_ref[0]**o+(0.184556562048)*x_ref[0]+(-0.694630650679)*x_ref[1]**o+(-0.526020870229)*x_ref[1]
        ref[0,3,2,0]=(0.397498255324)*x_ref[0]**o+(-0.375327133358)*x_ref[0]+(0.564571187906)*x_ref[1]**o+(-0.295820211045)*x_ref[1]
        ref[0,3,2,1]=(0.550532413074)*x_ref[0]**o+(-0.751307255234)*x_ref[0]+(0.323446174542)*x_ref[1]**o+(0.968475158376)*x_ref[1]
        ref[0,4,0,0]=(-0.112506956442)*x_ref[0]**o+(-0.432032518771)*x_ref[0]+(0.0205217950343)*x_ref[1]**o+(-0.507714722498)*x_ref[1]
        ref[0,4,0,1]=(-0.501932335824)*x_ref[0]**o+(-0.147371573026)*x_ref[0]+(-0.421130372732)*x_ref[1]**o+(-0.958029815205)*x_ref[1]
        ref[0,4,1,0]=(0.172226819831)*x_ref[0]**o+(-0.712313983987)*x_ref[0]+(0.859887204781)*x_ref[1]**o+(0.0661613029313)*x_ref[1]
        ref[0,4,1,1]=(-0.863367396226)*x_ref[0]**o+(0.150655310131)*x_ref[0]+(-0.446461969683)*x_ref[1]**o+(0.247377395525)*x_ref[1]
        ref[0,4,2,0]=(-0.0431980822723)*x_ref[0]**o+(0.676680822195)*x_ref[0]+(-0.613277443463)*x_ref[1]**o+(0.787356918727)*x_ref[1]
        ref[0,4,2,1]=(-0.0569719722636)*x_ref[0]**o+(0.641187476483)*x_ref[0]+(-0.303617110099)*x_ref[1]**o+(-0.888820557309)*x_ref[1]
        ref[1,0,0,0]=(-0.0552307803637)*x_ref[0]**o+(0.306010657529)*x_ref[0]+(0.488223930134)*x_ref[1]**o+(-0.682392555784)*x_ref[1]
        ref[1,0,0,1]=(-0.253901726526)*x_ref[0]**o+(0.73343629071)*x_ref[0]+(-0.390491326134)*x_ref[1]**o+(0.774605582697)*x_ref[1]
        ref[1,0,1,0]=(-0.42967954588)*x_ref[0]**o+(-0.751284399784)*x_ref[0]+(0.813738133871)*x_ref[1]**o+(-0.0995163058125)*x_ref[1]
        ref[1,0,1,1]=(-0.0880959500234)*x_ref[0]**o+(-0.781527901178)*x_ref[0]+(-0.496899067642)*x_ref[1]**o+(0.55524107606)*x_ref[1]
        ref[1,0,2,0]=(0.256546278479)*x_ref[0]**o+(-0.595310894415)*x_ref[0]+(-0.281823345122)*x_ref[1]**o+(-0.953457305769)*x_ref[1]
        ref[1,0,2,1]=(0.365266273739)*x_ref[0]**o+(-0.935192596652)*x_ref[0]+(0.740921701447)*x_ref[1]**o+(0.843620235195)*x_ref[1]
        ref[1,1,0,0]=(-0.992209271955)*x_ref[0]**o+(0.267499041637)*x_ref[0]+(-0.446633252289)*x_ref[1]**o+(0.357054618942)*x_ref[1]
        ref[1,1,0,1]=(0.863355358433)*x_ref[0]**o+(-0.104881968217)*x_ref[0]+(0.0776056200756)*x_ref[1]**o+(0.485205982195)*x_ref[1]
        ref[1,1,1,0]=(0.29822878169)*x_ref[0]**o+(0.766559198241)*x_ref[0]+(0.489839749671)*x_ref[1]**o+(-0.694316951865)*x_ref[1]
        ref[1,1,1,1]=(-0.947135250126)*x_ref[0]**o+(-0.952537408063)*x_ref[0]+(-0.840569482634)*x_ref[1]**o+(-0.0881389491948)*x_ref[1]
        ref[1,1,2,0]=(0.269982185854)*x_ref[0]**o+(0.290850786316)*x_ref[0]+(0.707550793741)*x_ref[1]**o+(-0.583787562553)*x_ref[1]
        ref[1,1,2,1]=(-0.998478389198)*x_ref[0]**o+(-0.902713626135)*x_ref[0]+(-0.0809583498397)*x_ref[1]**o+(0.242726761269)*x_ref[1]
        ref[1,2,0,0]=(-0.0752124155425)*x_ref[0]**o+(0.287338426631)*x_ref[0]+(-0.873281064449)*x_ref[1]**o+(0.86906507097)*x_ref[1]
        ref[1,2,0,1]=(-0.451012489095)*x_ref[0]**o+(0.692877058818)*x_ref[0]+(-0.771278511209)*x_ref[1]**o+(0.325983000966)*x_ref[1]
        ref[1,2,1,0]=(-0.633239034154)*x_ref[0]**o+(0.751673044983)*x_ref[0]+(-0.799706248119)*x_ref[1]**o+(0.0783814978548)*x_ref[1]
        ref[1,2,1,1]=(-0.496449080562)*x_ref[0]**o+(-0.382869354097)*x_ref[0]+(-0.658792080006)*x_ref[1]**o+(0.578499479656)*x_ref[1]
        ref[1,2,2,0]=(0.41041079346)*x_ref[0]**o+(0.552454636943)*x_ref[0]+(0.236050520221)*x_ref[1]**o+(0.477587390293)*x_ref[1]
        ref[1,2,2,1]=(0.649795041558)*x_ref[0]**o+(-0.947868231928)*x_ref[0]+(-0.529895928082)*x_ref[1]**o+(0.715695540026)*x_ref[1]
        ref[1,3,0,0]=(0.948112901731)*x_ref[0]**o+(-0.374624145588)*x_ref[0]+(0.806640283077)*x_ref[1]**o+(0.729074938843)*x_ref[1]
        ref[1,3,0,1]=(0.266956301954)*x_ref[0]**o+(-0.878004354786)*x_ref[0]+(-0.422298760963)*x_ref[1]**o+(-0.251922913127)*x_ref[1]
        ref[1,3,1,0]=(0.751159198422)*x_ref[0]**o+(-0.576983418224)*x_ref[0]+(0.427653287761)*x_ref[1]**o+(0.351116956206)*x_ref[1]
        ref[1,3,1,1]=(-0.804702781998)*x_ref[0]**o+(-0.218650676384)*x_ref[0]+(-0.832220670207)*x_ref[1]**o+(0.997469993794)*x_ref[1]
        ref[1,3,2,0]=(0.111110638087)*x_ref[0]**o+(-0.435831664904)*x_ref[0]+(-0.144481003079)*x_ref[1]**o+(-0.604349850119)*x_ref[1]
        ref[1,3,2,1]=(0.470311907248)*x_ref[0]**o+(-0.591709930358)*x_ref[0]+(-0.562662398681)*x_ref[1]**o+(0.0163307873851)*x_ref[1]
        ref[1,4,0,0]=(0.684349728229)*x_ref[0]**o+(0.404890565831)*x_ref[0]+(0.922438828147)*x_ref[1]**o+(0.939539648586)*x_ref[1]
        ref[1,4,0,1]=(-0.179161968364)*x_ref[0]**o+(0.983188684135)*x_ref[0]+(-0.722578271046)*x_ref[1]**o+(-0.450472366251)*x_ref[1]
        ref[1,4,1,0]=(0.243718494318)*x_ref[0]**o+(-0.0251174594804)*x_ref[0]+(0.952890549291)*x_ref[1]**o+(-0.398408844924)*x_ref[1]
        ref[1,4,1,1]=(-0.883090916963)*x_ref[0]**o+(-0.236073018867)*x_ref[0]+(-0.245092152479)*x_ref[1]**o+(0.846958498274)*x_ref[1]
        ref[1,4,2,0]=(-0.839928016329)*x_ref[0]**o+(-0.677734468188)*x_ref[0]+(-0.123639765438)*x_ref[1]**o+(-0.680672570927)*x_ref[1]
        ref[1,4,2,1]=(-0.234629335025)*x_ref[0]**o+(-0.567381745605)*x_ref[0]+(0.623480113756)*x_ref[1]**o+(0.2049427452)*x_ref[1]
        ref[2,0,0,0]=(-0.297265663727)*x_ref[0]**o+(0.00899195467866)*x_ref[0]+(0.992475896282)*x_ref[1]**o+(0.863451618621)*x_ref[1]
        ref[2,0,0,1]=(-0.24914517208)*x_ref[0]**o+(0.104091943078)*x_ref[0]+(0.762436787588)*x_ref[1]**o+(0.89363657955)*x_ref[1]
        ref[2,0,1,0]=(0.264734056412)*x_ref[0]**o+(-0.716868164934)*x_ref[0]+(-0.905285068877)*x_ref[1]**o+(0.0227117696273)*x_ref[1]
        ref[2,0,1,1]=(-0.932617617299)*x_ref[0]**o+(0.271634450254)*x_ref[0]+(0.51812940988)*x_ref[1]**o+(-0.484525267705)*x_ref[1]
        ref[2,0,2,0]=(0.364599957461)*x_ref[0]**o+(0.818588026862)*x_ref[0]+(0.70366241863)*x_ref[1]**o+(0.43910943526)*x_ref[1]
        ref[2,0,2,1]=(0.557891341305)*x_ref[0]**o+(-0.891810635437)*x_ref[0]+(0.649138154483)*x_ref[1]**o+(-0.213951745119)*x_ref[1]
        ref[2,1,0,0]=(0.994667066111)*x_ref[0]**o+(0.433110731778)*x_ref[0]+(0.787983438535)*x_ref[1]**o+(0.979717998129)*x_ref[1]
        ref[2,1,0,1]=(0.619284628164)*x_ref[0]**o+(0.662998143683)*x_ref[0]+(0.0328849267442)*x_ref[1]**o+(0.765881868994)*x_ref[1]
        ref[2,1,1,0]=(0.335939940328)*x_ref[0]**o+(0.556758825457)*x_ref[0]+(-0.524870937642)*x_ref[1]**o+(0.0316796705422)*x_ref[1]
        ref[2,1,1,1]=(-0.702249603437)*x_ref[0]**o+(0.544642480464)*x_ref[0]+(0.330520285989)*x_ref[1]**o+(0.313003559197)*x_ref[1]
        ref[2,1,2,0]=(0.0294247925115)*x_ref[0]**o+(0.10067678181)*x_ref[0]+(0.608148681239)*x_ref[1]**o+(-0.95697997936)*x_ref[1]
        ref[2,1,2,1]=(0.101913506022)*x_ref[0]**o+(-0.423061068258)*x_ref[0]+(0.303518988539)*x_ref[1]**o+(0.0108182161414)*x_ref[1]
        ref[2,2,0,0]=(0.390642040424)*x_ref[0]**o+(0.0578342521895)*x_ref[0]+(0.228499563768)*x_ref[1]**o+(-0.944433241107)*x_ref[1]
        ref[2,2,0,1]=(0.522633870922)*x_ref[0]**o+(0.724655140578)*x_ref[0]+(-0.664760217849)*x_ref[1]**o+(0.812535008015)*x_ref[1]
        ref[2,2,1,0]=(0.27608720491)*x_ref[0]**o+(0.42350834695)*x_ref[0]+(-0.732116287856)*x_ref[1]**o+(0.930674651671)*x_ref[1]
        ref[2,2,1,1]=(0.214308063462)*x_ref[0]**o+(0.815707217077)*x_ref[0]+(0.231785784747)*x_ref[1]**o+(-0.954429150944)*x_ref[1]
        ref[2,2,2,0]=(0.402814098347)*x_ref[0]**o+(-0.650491293555)*x_ref[0]+(-0.359159927669)*x_ref[1]**o+(0.503842188774)*x_ref[1]
        ref[2,2,2,1]=(-0.191195415522)*x_ref[0]**o+(0.980791691347)*x_ref[0]+(0.228597878156)*x_ref[1]**o+(0.535975172993)*x_ref[1]
        ref[2,3,0,0]=(0.495749327231)*x_ref[0]**o+(0.311034011895)*x_ref[0]+(-0.532686805928)*x_ref[1]**o+(0.92438969176)*x_ref[1]
        ref[2,3,0,1]=(-0.38513599341)*x_ref[0]**o+(-0.463099237819)*x_ref[0]+(-0.992606328662)*x_ref[1]**o+(-0.927920439344)*x_ref[1]
        ref[2,3,1,0]=(-0.868022271483)*x_ref[0]**o+(-0.540096192748)*x_ref[0]+(-0.0879858139685)*x_ref[1]**o+(0.534604810517)*x_ref[1]
        ref[2,3,1,1]=(-0.686837754683)*x_ref[0]**o+(-0.505585075528)*x_ref[0]+(-0.218155668559)*x_ref[1]**o+(-0.241972887878)*x_ref[1]
        ref[2,3,2,0]=(0.86187325655)*x_ref[0]**o+(-0.733958307606)*x_ref[0]+(-0.107393817742)*x_ref[1]**o+(-0.830306745614)*x_ref[1]
        ref[2,3,2,1]=(0.404834271017)*x_ref[0]**o+(0.44646270613)*x_ref[0]+(-0.657038236282)*x_ref[1]**o+(-0.540570795414)*x_ref[1]
        ref[2,4,0,0]=(-0.216828677229)*x_ref[0]**o+(0.129702603598)*x_ref[0]+(0.556920803376)*x_ref[1]**o+(0.960021941421)*x_ref[1]
        ref[2,4,0,1]=(-0.21625137654)*x_ref[0]**o+(-0.19604294366)*x_ref[0]+(-0.889082170851)*x_ref[1]**o+(-0.834967241538)*x_ref[1]
        ref[2,4,1,0]=(0.123104286845)*x_ref[0]**o+(-0.43197703752)*x_ref[0]+(0.737850827826)*x_ref[1]**o+(0.344689639656)*x_ref[1]
        ref[2,4,1,1]=(0.119471348292)*x_ref[0]**o+(0.948353928609)*x_ref[0]+(-0.726863566036)*x_ref[1]**o+(-0.957158639686)*x_ref[1]
        ref[2,4,2,0]=(0.606624368242)*x_ref[0]**o+(-0.129636738653)*x_ref[0]+(0.117469089776)*x_ref[1]**o+(0.766024628396)*x_ref[1]
        ref[2,4,2,1]=(-0.83631907859)*x_ref[0]**o+(-0.168433036335)*x_ref[0]+(-0.500663355802)*x_ref[1]**o+(0.0114866397792)*x_ref[1]
        ref[3,0,0,0]=(-0.629399168531)*x_ref[0]**o+(-0.464524331386)*x_ref[0]+(0.0406391749081)*x_ref[1]**o+(-0.380881295086)*x_ref[1]
        ref[3,0,0,1]=(-0.614191170121)*x_ref[0]**o+(0.90297284642)*x_ref[0]+(0.949653922227)*x_ref[1]**o+(0.947014467705)*x_ref[1]
        ref[3,0,1,0]=(0.128075669906)*x_ref[0]**o+(-0.215027529464)*x_ref[0]+(-0.230820603567)*x_ref[1]**o+(0.656672788071)*x_ref[1]
        ref[3,0,1,1]=(0.11006414768)*x_ref[0]**o+(0.575548749986)*x_ref[0]+(0.803886215346)*x_ref[1]**o+(0.627503866105)*x_ref[1]
        ref[3,0,2,0]=(-0.824524708364)*x_ref[0]**o+(-0.399533304607)*x_ref[0]+(-0.278996789484)*x_ref[1]**o+(-0.139967848513)*x_ref[1]
        ref[3,0,2,1]=(-0.657392776251)*x_ref[0]**o+(0.959283975752)*x_ref[0]+(0.990715521701)*x_ref[1]**o+(0.0999777335871)*x_ref[1]
        ref[3,1,0,0]=(-0.691053621801)*x_ref[0]**o+(0.189005136467)*x_ref[0]+(0.789186325959)*x_ref[1]**o+(-0.981079749179)*x_ref[1]
        ref[3,1,0,1]=(0.228390867858)*x_ref[0]**o+(-0.463941202938)*x_ref[0]+(-0.782094886569)*x_ref[1]**o+(0.464512245943)*x_ref[1]
        ref[3,1,1,0]=(-0.819255906497)*x_ref[0]**o+(-0.204510874222)*x_ref[0]+(-0.193473362908)*x_ref[1]**o+(-0.0250165714593)*x_ref[1]
        ref[3,1,1,1]=(0.331636523937)*x_ref[0]**o+(-0.0429730777685)*x_ref[0]+(-0.547743299325)*x_ref[1]**o+(-0.187953226002)*x_ref[1]
        ref[3,1,2,0]=(0.159370360993)*x_ref[0]**o+(0.883315520778)*x_ref[0]+(0.0933439209551)*x_ref[1]**o+(-0.458319201953)*x_ref[1]
        ref[3,1,2,1]=(-0.273991258)*x_ref[0]**o+(0.209739105337)*x_ref[0]+(0.335210395431)*x_ref[1]**o+(-0.91838746555)*x_ref[1]
        ref[3,2,0,0]=(-0.0618039563518)*x_ref[0]**o+(-0.59799143228)*x_ref[0]+(0.93203030688)*x_ref[1]**o+(0.581545424351)*x_ref[1]
        ref[3,2,0,1]=(-0.321101903544)*x_ref[0]**o+(-0.781260985212)*x_ref[0]+(-0.106128211217)*x_ref[1]**o+(0.913142993455)*x_ref[1]
        ref[3,2,1,0]=(0.584486365293)*x_ref[0]**o+(-0.654479654421)*x_ref[0]+(0.579992761397)*x_ref[1]**o+(-0.984753896904)*x_ref[1]
        ref[3,2,1,1]=(0.186056629749)*x_ref[0]**o+(-0.889254599383)*x_ref[0]+(0.134406174831)*x_ref[1]**o+(0.843880331117)*x_ref[1]
        ref[3,2,2,0]=(0.578447040133)*x_ref[0]**o+(-0.354828599745)*x_ref[0]+(0.943853840713)*x_ref[1]**o+(0.668565941699)*x_ref[1]
        ref[3,2,2,1]=(-0.657678353823)*x_ref[0]**o+(0.657623646562)*x_ref[0]+(0.670144253782)*x_ref[1]**o+(0.489533260867)*x_ref[1]
        ref[3,3,0,0]=(0.518241767069)*x_ref[0]**o+(0.717113307079)*x_ref[0]+(-0.986081844812)*x_ref[1]**o+(-0.922915734137)*x_ref[1]
        ref[3,3,0,1]=(0.934651660463)*x_ref[0]**o+(0.524253175591)*x_ref[0]+(0.519169286504)*x_ref[1]**o+(0.995798377114)*x_ref[1]
        ref[3,3,1,0]=(0.396606116937)*x_ref[0]**o+(-0.479342680739)*x_ref[0]+(0.677502711338)*x_ref[1]**o+(-0.591869615963)*x_ref[1]
        ref[3,3,1,1]=(0.339631149039)*x_ref[0]**o+(0.44124607198)*x_ref[0]+(0.786875844013)*x_ref[1]**o+(-0.925255814639)*x_ref[1]
        ref[3,3,2,0]=(-0.779510842649)*x_ref[0]**o+(0.562505114965)*x_ref[0]+(0.582094163828)*x_ref[1]**o+(0.910521768351)*x_ref[1]
        ref[3,3,2,1]=(-0.661315883103)*x_ref[0]**o+(0.226495727911)*x_ref[0]+(-0.464093148756)*x_ref[1]**o+(0.696758174053)*x_ref[1]
        ref[3,4,0,0]=(-0.0366658565966)*x_ref[0]**o+(0.78669216474)*x_ref[0]+(-0.784779466718)*x_ref[1]**o+(0.137721843502)*x_ref[1]
        ref[3,4,0,1]=(-0.00884301625831)*x_ref[0]**o+(0.25582694645)*x_ref[0]+(-0.73173489905)*x_ref[1]**o+(-0.956569853331)*x_ref[1]
        ref[3,4,1,0]=(-0.0884743360746)*x_ref[0]**o+(-0.484605933801)*x_ref[0]+(0.773028853802)*x_ref[1]**o+(-0.13255190562)*x_ref[1]
        ref[3,4,1,1]=(0.597859646547)*x_ref[0]**o+(-0.254684950139)*x_ref[0]+(0.292389327091)*x_ref[1]**o+(0.942836804031)*x_ref[1]
        ref[3,4,2,0]=(-0.82916540636)*x_ref[0]**o+(0.533192478501)*x_ref[0]+(0.577760569513)*x_ref[1]**o+(-0.812132995663)*x_ref[1]
        ref[3,4,2,1]=(-0.0918171874792)*x_ref[0]**o+(-0.232778513191)*x_ref[0]+(-0.57345566951)*x_ref[1]**o+(0.870122102)*x_ref[1]
      else:
        s[0,0,0,0]=(-0.419262547354)*x[0]**o+(-0.607792358178)*x[0]+(0.539129711536)*x[1]**o+(-0.954002022975)*x[1]+(0.52016184412)*x[2]**o+(-0.905599414589)*x[2]
        s[0,0,0,1]=(0.552249219683)*x[0]**o+(0.0747535682119)*x[0]+(-0.0550858389381)*x[1]**o+(-0.700756838491)*x[1]+(-0.62569620016)*x[2]**o+(-0.557287205831)*x[2]
        s[0,0,1,0]=(-0.84674266979)*x[0]**o+(-0.266089976589)*x[0]+(-0.994821395425)*x[1]**o+(-0.567614298529)*x[1]+(-0.814756268883)*x[2]**o+(-0.524622381582)*x[2]
        s[0,0,1,1]=(0.160195104873)*x[0]**o+(0.161596910068)*x[0]+(-0.0404024431157)*x[1]**o+(0.299140544701)*x[1]+(0.479120924271)*x[2]**o+(0.375793387197)*x[2]
        s[0,0,2,0]=(0.619011883068)*x[0]**o+(0.453196321806)*x[0]+(0.596546848845)*x[1]**o+(-0.148655559308)*x[1]+(-0.276643963879)*x[2]**o+(-0.556233199969)*x[2]
        s[0,0,2,1]=(-0.44936519278)*x[0]**o+(0.279187353573)*x[0]+(-0.626699833624)*x[1]**o+(0.172543451997)*x[1]+(0.324942859052)*x[2]**o+(0.227082355579)*x[2]
        s[0,1,0,0]=(0.348983572344)*x[0]**o+(-0.692310510135)*x[0]+(0.127542726356)*x[1]**o+(-0.719693998448)*x[1]+(0.190533765375)*x[2]**o+(-0.44411272509)*x[2]
        s[0,1,0,1]=(0.755193842627)*x[0]**o+(-0.0380801278139)*x[0]+(-0.416973165389)*x[1]**o+(-0.407050428042)*x[1]+(-0.803402606403)*x[2]**o+(0.860761604165)*x[2]
        s[0,1,1,0]=(0.903125811803)*x[0]**o+(-0.370434864257)*x[0]+(-0.810880082994)*x[1]**o+(0.214166691855)*x[1]+(-0.62621693665)*x[2]**o+(-0.043360368073)*x[2]
        s[0,1,1,1]=(-0.442516790862)*x[0]**o+(0.912139701807)*x[0]+(0.871611995743)*x[1]**o+(0.0291580448949)*x[1]+(-0.489427959855)*x[2]**o+(-0.950170294127)*x[2]
        s[0,1,2,0]=(-0.0867645917569)*x[0]**o+(0.556462347924)*x[0]+(-0.816554443299)*x[1]**o+(0.818773437796)*x[1]+(0.073905470924)*x[2]**o+(-0.714497094507)*x[2]
        s[0,1,2,1]=(0.699491100428)*x[0]**o+(-0.200275804269)*x[0]+(0.994316913102)*x[1]**o+(0.832890328003)*x[1]+(0.385381865669)*x[2]**o+(0.153056606745)*x[2]
        s[0,2,0,0]=(0.612319496073)*x[0]**o+(-0.301991428044)*x[0]+(-0.536866971844)*x[1]**o+(-0.375311119699)*x[1]+(0.733545441876)*x[2]**o+(0.0177187199528)*x[2]
        s[0,2,0,1]=(0.358991602948)*x[0]**o+(-0.862573091669)*x[0]+(0.464891723814)*x[1]**o+(0.602366461601)*x[1]+(-0.668916509347)*x[2]**o+(0.76048108017)*x[2]
        s[0,2,1,0]=(0.188195665573)*x[0]**o+(-0.927407688437)*x[0]+(-0.473014301986)*x[1]**o+(-0.775114885201)*x[1]+(-0.453403342886)*x[2]**o+(0.006776981492)*x[2]
        s[0,2,1,1]=(-0.700181448543)*x[0]**o+(-0.289180138599)*x[0]+(-0.867022883968)*x[1]**o+(-0.359339607516)*x[1]+(-0.809558496152)*x[2]**o+(-0.576358976651)*x[2]
        s[0,2,2,0]=(-0.557333080207)*x[0]**o+(0.724962853865)*x[0]+(0.995041226791)*x[1]**o+(-0.610460710065)*x[1]+(0.620016428769)*x[2]**o+(-0.449182254452)*x[2]
        s[0,2,2,1]=(-0.937207840292)*x[0]**o+(-0.66582207891)*x[0]+(0.0951881913165)*x[1]**o+(-0.0770694628972)*x[1]+(0.034224640398)*x[2]**o+(0.652057466224)*x[2]
        s[0,3,0,0]=(-0.0236750483037)*x[0]**o+(0.956130793675)*x[0]+(-0.589425964387)*x[1]**o+(-0.204770941511)*x[1]+(0.695255730164)*x[2]**o+(-0.796222694557)*x[2]
        s[0,3,0,1]=(0.138589818728)*x[0]**o+(0.493237559501)*x[0]+(0.139281181588)*x[1]**o+(-0.974201990591)*x[1]+(-0.15401887402)*x[2]**o+(0.23794136041)*x[2]
        s[0,3,1,0]=(-0.332886758751)*x[0]**o+(-0.231549718295)*x[0]+(0.581536450397)*x[1]**o+(0.790063092494)*x[1]+(0.358814129932)*x[2]**o+(-0.110547525643)*x[2]
        s[0,3,1,1]=(0.122775146487)*x[0]**o+(-0.00818530079912)*x[0]+(0.0510575666357)*x[1]**o+(-0.96911992662)*x[1]+(-0.820269176326)*x[2]**o+(0.875945222846)*x[2]
        s[0,3,2,0]=(0.761874596666)*x[0]**o+(0.621369218231)*x[0]+(0.960271412263)*x[1]**o+(-0.808832785376)*x[1]+(0.981428454481)*x[2]**o+(0.393331122254)*x[2]
        s[0,3,2,1]=(-0.646477042209)*x[0]**o+(-0.26169646294)*x[0]+(0.992265690785)*x[1]**o+(-0.678558332435)*x[1]+(-0.285531525349)*x[2]**o+(-0.571647919822)*x[2]
        s[0,4,0,0]=(0.492930260015)*x[0]**o+(-0.647489772768)*x[0]+(-0.381225997009)*x[1]**o+(0.560462598779)*x[1]+(-0.63571677055)*x[2]**o+(0.103213895387)*x[2]
        s[0,4,0,1]=(-0.0362476689653)*x[0]**o+(-0.478992488756)*x[0]+(0.544907192925)*x[1]**o+(-0.678350370776)*x[1]+(-0.0776363475051)*x[2]**o+(-0.803140341684)*x[2]
        s[0,4,1,0]=(0.0787683344872)*x[0]**o+(0.688990188782)*x[0]+(0.257089245341)*x[1]**o+(-0.648663765213)*x[1]+(-0.140632312728)*x[2]**o+(0.676209090063)*x[2]
        s[0,4,1,1]=(0.804994807205)*x[0]**o+(0.74750498671)*x[0]+(-0.00532761506025)*x[1]**o+(-0.295700694165)*x[1]+(-0.209175099115)*x[2]**o+(-0.0126407798374)*x[2]
        s[0,4,2,0]=(-0.789160064467)*x[0]**o+(0.332412023778)*x[0]+(0.242860020473)*x[1]**o+(0.0284658318544)*x[1]+(-0.357559965972)*x[2]**o+(0.57056582293)*x[2]
        s[0,4,2,1]=(-0.861027741096)*x[0]**o+(0.545166238342)*x[0]+(0.950975873381)*x[1]**o+(-0.436709572856)*x[1]+(0.93636977927)*x[2]**o+(-0.405205657349)*x[2]
        s[1,0,0,0]=(0.47880791873)*x[0]**o+(0.037486909403)*x[0]+(0.732218957477)*x[1]**o+(0.274811688271)*x[1]+(0.0686323288694)*x[2]**o+(-0.492347744678)*x[2]
        s[1,0,0,1]=(-0.618259414329)*x[0]**o+(0.630883795613)*x[0]+(0.269141377847)*x[1]**o+(-0.490916952452)*x[1]+(-0.333070678904)*x[2]**o+(0.0484209401349)*x[2]
        s[1,0,1,0]=(-0.906059152197)*x[0]**o+(0.0322820078282)*x[0]+(-0.47807076381)*x[1]**o+(0.0296380181906)*x[1]+(0.553686905633)*x[2]**o+(-0.408333784569)*x[2]
        s[1,0,1,1]=(0.282962689091)*x[0]**o+(-0.11495842442)*x[0]+(0.818142653998)*x[1]**o+(0.563460324128)*x[1]+(-0.993782402791)*x[2]**o+(-0.0193690715193)*x[2]
        s[1,0,2,0]=(0.287962506263)*x[0]**o+(-0.401293374286)*x[0]+(0.173559459587)*x[1]**o+(0.822343918838)*x[1]+(0.888309195417)*x[2]**o+(0.687878298228)*x[2]
        s[1,0,2,1]=(-0.926775125361)*x[0]**o+(0.396396669159)*x[0]+(0.632962652289)*x[1]**o+(-0.915667885738)*x[1]+(-0.31075960693)*x[2]**o+(0.920135418244)*x[2]
        s[1,1,0,0]=(-0.0915500113579)*x[0]**o+(0.489036843145)*x[0]+(0.477982330324)*x[1]**o+(0.703371804742)*x[1]+(-0.471773636091)*x[2]**o+(0.0665811350713)*x[2]
        s[1,1,0,1]=(0.525411480834)*x[0]**o+(-0.369843393433)*x[0]+(-0.7770705966)*x[1]**o+(0.534153802448)*x[1]+(0.309828458492)*x[2]**o+(0.627690678711)*x[2]
        s[1,1,1,0]=(0.149039257889)*x[0]**o+(-0.0585283560518)*x[0]+(-0.0505893024858)*x[1]**o+(0.677281229531)*x[1]+(-0.451950028252)*x[2]**o+(0.356600216864)*x[2]
        s[1,1,1,1]=(-0.528237952467)*x[0]**o+(0.533554438129)*x[0]+(0.741981923136)*x[1]**o+(-0.596922185095)*x[1]+(0.50521173977)*x[2]**o+(-0.247100318829)*x[2]
        s[1,1,2,0]=(0.667977944954)*x[0]**o+(0.306325804541)*x[0]+(-0.654643365837)*x[1]**o+(0.0176380527447)*x[1]+(-0.218012920296)*x[2]**o+(-0.181740681902)*x[2]
        s[1,1,2,1]=(0.440430981908)*x[0]**o+(0.994608754477)*x[0]+(0.114426402744)*x[1]**o+(-0.896222828258)*x[1]+(0.997911423594)*x[2]**o+(0.648328801403)*x[2]
        s[1,2,0,0]=(0.397328767103)*x[0]**o+(0.780110953188)*x[0]+(0.453955649367)*x[1]**o+(0.953839185994)*x[1]+(0.995536671917)*x[2]**o+(0.177278490615)*x[2]
        s[1,2,0,1]=(-0.154393442621)*x[0]**o+(-0.100267841021)*x[0]+(-0.781347482738)*x[1]**o+(0.391441261298)*x[1]+(0.469832428629)*x[2]**o+(0.608531800159)*x[2]
        s[1,2,1,0]=(-0.534167310466)*x[0]**o+(0.414564904301)*x[0]+(-0.0674568638795)*x[1]**o+(0.577721681963)*x[1]+(-0.00750850180411)*x[2]**o+(0.0817658166565)*x[2]
        s[1,2,1,1]=(-0.529547188418)*x[0]**o+(0.267917121456)*x[0]+(0.372449729423)*x[1]**o+(0.887072823755)*x[1]+(0.887821539758)*x[2]**o+(0.249771090594)*x[2]
        s[1,2,2,0]=(0.949056994737)*x[0]**o+(-0.174523839411)*x[0]+(-0.646085737348)*x[1]**o+(0.449358235114)*x[1]+(-0.970748494721)*x[2]**o+(-0.410800591341)*x[2]
        s[1,2,2,1]=(0.879006069022)*x[0]**o+(-0.510106021113)*x[0]+(0.0203703221469)*x[1]**o+(-0.844338135123)*x[1]+(0.752168319634)*x[2]**o+(0.158760999046)*x[2]
        s[1,3,0,0]=(0.845321813876)*x[0]**o+(0.405224606418)*x[0]+(-0.89159606951)*x[1]**o+(-0.949499370695)*x[1]+(0.501499536159)*x[2]**o+(-0.869583954204)*x[2]
        s[1,3,0,1]=(0.0887248259912)*x[0]**o+(0.232925681778)*x[0]+(0.0315176777236)*x[1]**o+(0.00568107770424)*x[1]+(-0.821963787641)*x[2]**o+(-0.634962715535)*x[2]
        s[1,3,1,0]=(0.546210053059)*x[0]**o+(-0.688171108228)*x[0]+(-0.699621490343)*x[1]**o+(-0.850445270177)*x[1]+(-0.725131419114)*x[2]**o+(0.484336843024)*x[2]
        s[1,3,1,1]=(0.278594806693)*x[0]**o+(-0.266710093152)*x[0]+(-0.151515190688)*x[1]**o+(-0.705005965054)*x[1]+(0.811008835405)*x[2]**o+(-0.342637057553)*x[2]
        s[1,3,2,0]=(0.4748294578)*x[0]**o+(-0.0741346092747)*x[0]+(0.148026337115)*x[1]**o+(0.0252686427245)*x[1]+(-0.479120476874)*x[2]**o+(0.0855008343792)*x[2]
        s[1,3,2,1]=(-0.337007818128)*x[0]**o+(-0.690464368815)*x[0]+(0.481925609432)*x[1]**o+(-0.0981705702921)*x[1]+(0.19218353071)*x[2]**o+(0.988337158745)*x[2]
        s[1,4,0,0]=(-0.757351506535)*x[0]**o+(-0.824203825425)*x[0]+(-0.998647750787)*x[1]**o+(-0.111654698732)*x[1]+(-0.713287873387)*x[2]**o+(0.779671510732)*x[2]
        s[1,4,0,1]=(-0.756370998169)*x[0]**o+(0.826863767951)*x[0]+(0.358310449165)*x[1]**o+(0.882430292099)*x[1]+(0.880156909975)*x[2]**o+(0.360411043274)*x[2]
        s[1,4,1,0]=(0.226080011536)*x[0]**o+(-0.740536947501)*x[0]+(-0.650527457572)*x[1]**o+(-0.890015156497)*x[1]+(0.688973071404)*x[2]**o+(0.29237820526)*x[2]
        s[1,4,1,1]=(0.574015626188)*x[0]**o+(-0.675641095131)*x[0]+(0.814289365677)*x[1]**o+(-0.625021588943)*x[1]+(-0.862079110641)*x[2]**o+(0.0363949889967)*x[2]
        s[1,4,2,0]=(0.126659193582)*x[0]**o+(0.16976937315)*x[0]+(0.18822384431)*x[1]**o+(0.546082142359)*x[1]+(0.551365996908)*x[2]**o+(0.395213783677)*x[2]
        s[1,4,2,1]=(-0.292436267133)*x[0]**o+(-0.442787521929)*x[0]+(-0.310488595304)*x[1]**o+(0.0918858003303)*x[1]+(-0.986695921261)*x[2]**o+(-0.52979957068)*x[2]
        s[2,0,0,0]=(0.396669972067)*x[0]**o+(-0.711415400722)*x[0]+(-0.717006050735)*x[1]**o+(-0.0935757817421)*x[1]+(0.743477556176)*x[2]**o+(0.823967976857)*x[2]
        s[2,0,0,1]=(0.488395228667)*x[0]**o+(0.230144035899)*x[0]+(0.125851877066)*x[1]**o+(0.0536284998009)*x[1]+(-0.0114515128334)*x[2]**o+(-0.494039400348)*x[2]
        s[2,0,1,0]=(0.900615108208)*x[0]**o+(0.432765237128)*x[0]+(0.981213121806)*x[1]**o+(-0.116934702769)*x[1]+(-0.612499086627)*x[2]**o+(-0.365572411348)*x[2]
        s[2,0,1,1]=(0.0342670705338)*x[0]**o+(0.54954404771)*x[0]+(0.428653302401)*x[1]**o+(0.537787906346)*x[1]+(-0.64287424457)*x[2]**o+(-0.173877644259)*x[2]
        s[2,0,2,0]=(-0.0510871045727)*x[0]**o+(-0.908007271573)*x[0]+(0.00194326164542)*x[1]**o+(0.466243299714)*x[1]+(-0.433962263339)*x[2]**o+(0.874724229969)*x[2]
        s[2,0,2,1]=(0.666308090297)*x[0]**o+(0.933188439742)*x[0]+(-0.0479445062129)*x[1]**o+(0.0747806162498)*x[1]+(0.911844507192)*x[2]**o+(-0.934538480024)*x[2]
        s[2,1,0,0]=(-0.516290157366)*x[0]**o+(0.437519025532)*x[0]+(-0.170999588789)*x[1]**o+(-0.934096554106)*x[1]+(0.88575800375)*x[2]**o+(0.636966058116)*x[2]
        s[2,1,0,1]=(0.0172669412005)*x[0]**o+(-0.867857912246)*x[0]+(-0.972747832278)*x[1]**o+(-0.780969292213)*x[1]+(-0.418154913379)*x[2]**o+(0.805902597544)*x[2]
        s[2,1,1,0]=(-0.964006922451)*x[0]**o+(0.236797578253)*x[0]+(-0.78802971076)*x[1]**o+(0.704165381448)*x[1]+(0.00112348607329)*x[2]**o+(-0.689065853536)*x[2]
        s[2,1,1,1]=(-0.722392335335)*x[0]**o+(-0.120018753634)*x[0]+(0.216541173904)*x[1]**o+(-0.374710223309)*x[1]+(0.532158723682)*x[2]**o+(0.464637663705)*x[2]
        s[2,1,2,0]=(-0.183637646888)*x[0]**o+(-0.0680674034088)*x[0]+(-0.114129008735)*x[1]**o+(-0.78666547603)*x[1]+(0.0956689943304)*x[2]**o+(0.743296994432)*x[2]
        s[2,1,2,1]=(0.611382565923)*x[0]**o+(-0.768682259494)*x[0]+(-0.540943249593)*x[1]**o+(0.753523431999)*x[1]+(0.610689642267)*x[2]**o+(0.0122811249496)*x[2]
        s[2,2,0,0]=(-0.789273374112)*x[0]**o+(0.0977540682694)*x[0]+(-0.829719260296)*x[1]**o+(0.824773509135)*x[1]+(0.216669854484)*x[2]**o+(-0.764947843182)*x[2]
        s[2,2,0,1]=(-0.340784170043)*x[0]**o+(-0.267437208067)*x[0]+(-0.430973997706)*x[1]**o+(-0.0846177320866)*x[1]+(0.688599546971)*x[2]**o+(0.265513958383)*x[2]
        s[2,2,1,0]=(-0.237677411545)*x[0]**o+(-0.792144835449)*x[0]+(-0.435763499999)*x[1]**o+(0.682920743804)*x[1]+(0.116459199169)*x[2]**o+(0.254527225097)*x[2]
        s[2,2,1,1]=(0.342906428734)*x[0]**o+(0.206470310722)*x[0]+(-0.352429884648)*x[1]**o+(-0.168905312607)*x[1]+(-0.105529665095)*x[2]**o+(0.199873788005)*x[2]
        s[2,2,2,0]=(0.677855511296)*x[0]**o+(0.983953993003)*x[0]+(-0.64997591911)*x[1]**o+(-0.629221981454)*x[1]+(0.990621105566)*x[2]**o+(0.785766133934)*x[2]
        s[2,2,2,1]=(-0.570482617922)*x[0]**o+(-0.151066051792)*x[0]+(-0.511453364326)*x[1]**o+(0.128824691419)*x[1]+(-0.520127324328)*x[2]**o+(-0.157450052117)*x[2]
        s[2,3,0,0]=(-0.423827660795)*x[0]**o+(-0.815943363397)*x[0]+(-0.636070023986)*x[1]**o+(0.251327239067)*x[1]+(-0.401641739847)*x[2]**o+(0.130280903755)*x[2]
        s[2,3,0,1]=(0.0411439121625)*x[0]**o+(0.621609981629)*x[0]+(-0.589059932668)*x[1]**o+(0.543095349334)*x[1]+(-0.0122421869863)*x[2]**o+(-0.776924587359)*x[2]
        s[2,3,1,0]=(0.63252327363)*x[0]**o+(0.275597518383)*x[0]+(0.832198526441)*x[1]**o+(-0.92855918951)*x[1]+(0.490343890624)*x[2]**o+(-0.412580022625)*x[2]
        s[2,3,1,1]=(-0.560907436774)*x[0]**o+(0.413956414518)*x[0]+(-0.849736975879)*x[1]**o+(0.167861053833)*x[1]+(-0.316526371498)*x[2]**o+(-0.431245755915)*x[2]
        s[2,3,2,0]=(0.883757549647)*x[0]**o+(-0.614133317417)*x[0]+(0.923442235162)*x[1]**o+(0.483998522495)*x[1]+(0.649003328717)*x[2]**o+(-0.320181801832)*x[2]
        s[2,3,2,1]=(0.181982086686)*x[0]**o+(-0.88761330228)*x[0]+(-0.752108545831)*x[1]**o+(0.105681418232)*x[1]+(0.211053107745)*x[2]**o+(-0.840650090772)*x[2]
        s[2,4,0,0]=(0.774235220873)*x[0]**o+(-0.394430371883)*x[0]+(-0.82551531411)*x[1]**o+(-0.397374538567)*x[1]+(0.51757884795)*x[2]**o+(-0.878919341817)*x[2]
        s[2,4,0,1]=(-0.282445913776)*x[0]**o+(-0.252614209563)*x[0]+(-0.66810962709)*x[1]**o+(0.912660025157)*x[1]+(0.786200789061)*x[2]**o+(-0.271329907892)*x[2]
        s[2,4,1,0]=(-0.415061697718)*x[0]**o+(-0.577077815899)*x[0]+(-0.479115469184)*x[1]**o+(0.285424287206)*x[1]+(0.580737671113)*x[2]**o+(0.345716282261)*x[2]
        s[2,4,1,1]=(0.103116885191)*x[0]**o+(0.472934825259)*x[0]+(0.192747945705)*x[1]**o+(0.106949097334)*x[1]+(-0.109457290729)*x[2]**o+(0.843345964596)*x[2]
        s[2,4,2,0]=(0.600281530422)*x[0]**o+(-0.427230650838)*x[0]+(-0.868150408946)*x[1]**o+(-0.203647711262)*x[1]+(-0.550477804035)*x[2]**o+(0.415736214172)*x[2]
        s[2,4,2,1]=(0.739232217202)*x[0]**o+(-0.476864848171)*x[0]+(0.367627911492)*x[1]**o+(0.105176582927)*x[1]+(0.482113806667)*x[2]**o+(0.782020005885)*x[2]
        s[3,0,0,0]=(0.938839141593)*x[0]**o+(-0.655793939395)*x[0]+(0.0224780413611)*x[1]**o+(0.252535317383)*x[1]+(-0.887022478757)*x[2]**o+(-0.451912746232)*x[2]
        s[3,0,0,1]=(-0.196994797427)*x[0]**o+(-0.222816920678)*x[0]+(0.37131460044)*x[1]**o+(0.124872956754)*x[1]+(-0.872680499884)*x[2]**o+(-0.490521736465)*x[2]
        s[3,0,1,0]=(-0.977712672434)*x[0]**o+(-0.665877989129)*x[0]+(-0.541103000349)*x[1]**o+(-0.527523471522)*x[1]+(0.469063433285)*x[2]**o+(-0.410680406273)*x[2]
        s[3,0,1,1]=(0.371612180735)*x[0]**o+(-0.882397978651)*x[0]+(-0.333119569273)*x[1]**o+(0.0439599202688)*x[1]+(-0.378698960758)*x[2]**o+(0.373830212832)*x[2]
        s[3,0,2,0]=(0.13552136805)*x[0]**o+(-0.347572305079)*x[0]+(0.194173745453)*x[1]**o+(-0.536374783086)*x[1]+(-0.0928402883882)*x[2]**o+(0.972386889251)*x[2]
        s[3,0,2,1]=(-0.673716579315)*x[0]**o+(0.497683603572)*x[0]+(0.0894501300871)*x[1]**o+(-0.691905501083)*x[1]+(-0.819373609736)*x[2]**o+(0.876955815163)*x[2]
        s[3,1,0,0]=(-0.612778977625)*x[0]**o+(0.898593519513)*x[0]+(-0.579505051936)*x[1]**o+(0.294087297529)*x[1]+(0.713022674221)*x[2]**o+(-0.0994065928128)*x[2]
        s[3,1,0,1]=(-0.141600251938)*x[0]**o+(-0.677815682935)*x[0]+(-0.408920459122)*x[1]**o+(-0.723029003437)*x[1]+(0.481057659884)*x[2]**o+(0.427909771498)*x[2]
        s[3,1,1,0]=(-0.516486224771)*x[0]**o+(0.526484181999)*x[0]+(0.911793059151)*x[1]**o+(0.409808964766)*x[1]+(-0.413591542253)*x[2]**o+(-0.759664326823)*x[2]
        s[3,1,1,1]=(0.905324377656)*x[0]**o+(-0.208896172985)*x[0]+(-0.422109371498)*x[1]**o+(0.491883832748)*x[1]+(-0.203413002012)*x[2]**o+(0.406737219074)*x[2]
        s[3,1,2,0]=(-0.573692277755)*x[0]**o+(0.895762539503)*x[0]+(0.176538736347)*x[1]**o+(-0.847640766515)*x[1]+(0.5650742337)*x[2]**o+(0.503165369442)*x[2]
        s[3,1,2,1]=(0.308521434074)*x[0]**o+(0.616776036606)*x[0]+(-0.405321909102)*x[1]**o+(0.497367404288)*x[1]+(-0.798636270652)*x[2]**o+(-0.337423592684)*x[2]
        s[3,2,0,0]=(-0.394455434548)*x[0]**o+(-0.292212771133)*x[0]+(-0.941613233015)*x[1]**o+(-0.404522244202)*x[1]+(-0.0112893909573)*x[2]**o+(-0.9706166943)*x[2]
        s[3,2,0,1]=(0.627774279675)*x[0]**o+(-0.645334179444)*x[0]+(0.691389142649)*x[1]**o+(0.763379715146)*x[1]+(0.610766030943)*x[2]**o+(-0.786459853785)*x[2]
        s[3,2,1,0]=(-0.0101413740932)*x[0]**o+(-0.0970748510013)*x[0]+(-0.52181886114)*x[1]**o+(-0.612494396935)*x[1]+(0.941988727244)*x[2]**o+(0.0361447091017)*x[2]
        s[3,2,1,1]=(-0.0943447427863)*x[0]**o+(0.708735490064)*x[0]+(-0.352853510872)*x[1]**o+(-0.545593672805)*x[1]+(-0.0106851492926)*x[2]**o+(0.712976402827)*x[2]
        s[3,2,2,0]=(-0.522341854545)*x[0]**o+(0.320365455548)*x[0]+(-0.111682800076)*x[1]**o+(-0.337351991987)*x[1]+(-0.33503967624)*x[2]**o+(-0.215395174878)*x[2]
        s[3,2,2,1]=(-0.191634883402)*x[0]**o+(-0.79050407204)*x[0]+(-0.881959919487)*x[1]**o+(0.962597211649)*x[1]+(0.602181506788)*x[2]**o+(-0.430152854642)*x[2]
        s[3,3,0,0]=(-0.458551188388)*x[0]**o+(0.62195840207)*x[0]+(0.646839412108)*x[1]**o+(-0.597989589884)*x[1]+(-0.160578706146)*x[2]**o+(0.885101198188)*x[2]
        s[3,3,0,1]=(-0.806015868617)*x[0]**o+(0.0617182982024)*x[0]+(0.296661747576)*x[1]**o+(0.381687206096)*x[1]+(0.954310501938)*x[2]**o+(0.112880684912)*x[2]
        s[3,3,1,0]=(0.485202154902)*x[0]**o+(0.528514725346)*x[0]+(0.550444453321)*x[1]**o+(-0.816313566191)*x[1]+(0.945151247472)*x[2]**o+(-0.201955849739)*x[2]
        s[3,3,1,1]=(0.245157893191)*x[0]**o+(-0.281606458201)*x[0]+(-0.312808561115)*x[1]**o+(0.185786713188)*x[1]+(0.996302657312)*x[2]**o+(-0.227884060488)*x[2]
        s[3,3,2,0]=(0.297297617074)*x[0]**o+(0.156994991329)*x[0]+(-0.403988935033)*x[1]**o+(-0.0894931373916)*x[1]+(-0.979214022227)*x[2]**o+(0.287684161706)*x[2]
        s[3,3,2,1]=(-0.287794860211)*x[0]**o+(0.169671378363)*x[0]+(0.0172804553692)*x[1]**o+(-0.973449618671)*x[1]+(0.0652371831894)*x[2]**o+(0.775978144983)*x[2]
        s[3,4,0,0]=(0.54203905332)*x[0]**o+(0.66731094672)*x[0]+(-0.865207826406)*x[1]**o+(0.466596502355)*x[1]+(-0.45336027189)*x[2]**o+(-0.266404323774)*x[2]
        s[3,4,0,1]=(-0.0264326659602)*x[0]**o+(0.102421850054)*x[0]+(-0.672364919167)*x[1]**o+(-0.461816126462)*x[1]+(0.11436848926)*x[2]**o+(-0.78486118431)*x[2]
        s[3,4,1,0]=(-0.941616359303)*x[0]**o+(-0.0519153474588)*x[0]+(0.8597678151)*x[1]**o+(0.164278414756)*x[1]+(0.496859863828)*x[2]**o+(-0.413785115519)*x[2]
        s[3,4,1,1]=(-0.726802133704)*x[0]**o+(0.685989768699)*x[0]+(0.499728992214)*x[1]**o+(0.675799006108)*x[1]+(0.205370350943)*x[2]**o+(0.983979039059)*x[2]
        s[3,4,2,0]=(-0.121723702011)*x[0]**o+(0.350456030225)*x[0]+(0.211968006265)*x[1]**o+(-0.960265509868)*x[1]+(-0.593090834374)*x[2]**o+(-0.631506211212)*x[2]
        s[3,4,2,1]=(0.279791655852)*x[0]**o+(0.236927673275)*x[0]+(0.84188823482)*x[1]**o+(0.208410505206)*x[1]+(0.392354982825)*x[2]**o+(0.671826571516)*x[2]
        ref[0,0,0,0]=(-0.419262547354)*x_ref[0]**o+(-0.607792358178)*x_ref[0]+(0.539129711536)*x_ref[1]**o+(-0.954002022975)*x_ref[1]+(0.52016184412)*x_ref[2]**o+(-0.905599414589)*x_ref[2]
        ref[0,0,0,1]=(0.552249219683)*x_ref[0]**o+(0.0747535682119)*x_ref[0]+(-0.0550858389381)*x_ref[1]**o+(-0.700756838491)*x_ref[1]+(-0.62569620016)*x_ref[2]**o+(-0.557287205831)*x_ref[2]
        ref[0,0,1,0]=(-0.84674266979)*x_ref[0]**o+(-0.266089976589)*x_ref[0]+(-0.994821395425)*x_ref[1]**o+(-0.567614298529)*x_ref[1]+(-0.814756268883)*x_ref[2]**o+(-0.524622381582)*x_ref[2]
        ref[0,0,1,1]=(0.160195104873)*x_ref[0]**o+(0.161596910068)*x_ref[0]+(-0.0404024431157)*x_ref[1]**o+(0.299140544701)*x_ref[1]+(0.479120924271)*x_ref[2]**o+(0.375793387197)*x_ref[2]
        ref[0,0,2,0]=(0.619011883068)*x_ref[0]**o+(0.453196321806)*x_ref[0]+(0.596546848845)*x_ref[1]**o+(-0.148655559308)*x_ref[1]+(-0.276643963879)*x_ref[2]**o+(-0.556233199969)*x_ref[2]
        ref[0,0,2,1]=(-0.44936519278)*x_ref[0]**o+(0.279187353573)*x_ref[0]+(-0.626699833624)*x_ref[1]**o+(0.172543451997)*x_ref[1]+(0.324942859052)*x_ref[2]**o+(0.227082355579)*x_ref[2]
        ref[0,1,0,0]=(0.348983572344)*x_ref[0]**o+(-0.692310510135)*x_ref[0]+(0.127542726356)*x_ref[1]**o+(-0.719693998448)*x_ref[1]+(0.190533765375)*x_ref[2]**o+(-0.44411272509)*x_ref[2]
        ref[0,1,0,1]=(0.755193842627)*x_ref[0]**o+(-0.0380801278139)*x_ref[0]+(-0.416973165389)*x_ref[1]**o+(-0.407050428042)*x_ref[1]+(-0.803402606403)*x_ref[2]**o+(0.860761604165)*x_ref[2]
        ref[0,1,1,0]=(0.903125811803)*x_ref[0]**o+(-0.370434864257)*x_ref[0]+(-0.810880082994)*x_ref[1]**o+(0.214166691855)*x_ref[1]+(-0.62621693665)*x_ref[2]**o+(-0.043360368073)*x_ref[2]
        ref[0,1,1,1]=(-0.442516790862)*x_ref[0]**o+(0.912139701807)*x_ref[0]+(0.871611995743)*x_ref[1]**o+(0.0291580448949)*x_ref[1]+(-0.489427959855)*x_ref[2]**o+(-0.950170294127)*x_ref[2]
        ref[0,1,2,0]=(-0.0867645917569)*x_ref[0]**o+(0.556462347924)*x_ref[0]+(-0.816554443299)*x_ref[1]**o+(0.818773437796)*x_ref[1]+(0.073905470924)*x_ref[2]**o+(-0.714497094507)*x_ref[2]
        ref[0,1,2,1]=(0.699491100428)*x_ref[0]**o+(-0.200275804269)*x_ref[0]+(0.994316913102)*x_ref[1]**o+(0.832890328003)*x_ref[1]+(0.385381865669)*x_ref[2]**o+(0.153056606745)*x_ref[2]
        ref[0,2,0,0]=(0.612319496073)*x_ref[0]**o+(-0.301991428044)*x_ref[0]+(-0.536866971844)*x_ref[1]**o+(-0.375311119699)*x_ref[1]+(0.733545441876)*x_ref[2]**o+(0.0177187199528)*x_ref[2]
        ref[0,2,0,1]=(0.358991602948)*x_ref[0]**o+(-0.862573091669)*x_ref[0]+(0.464891723814)*x_ref[1]**o+(0.602366461601)*x_ref[1]+(-0.668916509347)*x_ref[2]**o+(0.76048108017)*x_ref[2]
        ref[0,2,1,0]=(0.188195665573)*x_ref[0]**o+(-0.927407688437)*x_ref[0]+(-0.473014301986)*x_ref[1]**o+(-0.775114885201)*x_ref[1]+(-0.453403342886)*x_ref[2]**o+(0.006776981492)*x_ref[2]
        ref[0,2,1,1]=(-0.700181448543)*x_ref[0]**o+(-0.289180138599)*x_ref[0]+(-0.867022883968)*x_ref[1]**o+(-0.359339607516)*x_ref[1]+(-0.809558496152)*x_ref[2]**o+(-0.576358976651)*x_ref[2]
        ref[0,2,2,0]=(-0.557333080207)*x_ref[0]**o+(0.724962853865)*x_ref[0]+(0.995041226791)*x_ref[1]**o+(-0.610460710065)*x_ref[1]+(0.620016428769)*x_ref[2]**o+(-0.449182254452)*x_ref[2]
        ref[0,2,2,1]=(-0.937207840292)*x_ref[0]**o+(-0.66582207891)*x_ref[0]+(0.0951881913165)*x_ref[1]**o+(-0.0770694628972)*x_ref[1]+(0.034224640398)*x_ref[2]**o+(0.652057466224)*x_ref[2]
        ref[0,3,0,0]=(-0.0236750483037)*x_ref[0]**o+(0.956130793675)*x_ref[0]+(-0.589425964387)*x_ref[1]**o+(-0.204770941511)*x_ref[1]+(0.695255730164)*x_ref[2]**o+(-0.796222694557)*x_ref[2]
        ref[0,3,0,1]=(0.138589818728)*x_ref[0]**o+(0.493237559501)*x_ref[0]+(0.139281181588)*x_ref[1]**o+(-0.974201990591)*x_ref[1]+(-0.15401887402)*x_ref[2]**o+(0.23794136041)*x_ref[2]
        ref[0,3,1,0]=(-0.332886758751)*x_ref[0]**o+(-0.231549718295)*x_ref[0]+(0.581536450397)*x_ref[1]**o+(0.790063092494)*x_ref[1]+(0.358814129932)*x_ref[2]**o+(-0.110547525643)*x_ref[2]
        ref[0,3,1,1]=(0.122775146487)*x_ref[0]**o+(-0.00818530079912)*x_ref[0]+(0.0510575666357)*x_ref[1]**o+(-0.96911992662)*x_ref[1]+(-0.820269176326)*x_ref[2]**o+(0.875945222846)*x_ref[2]
        ref[0,3,2,0]=(0.761874596666)*x_ref[0]**o+(0.621369218231)*x_ref[0]+(0.960271412263)*x_ref[1]**o+(-0.808832785376)*x_ref[1]+(0.981428454481)*x_ref[2]**o+(0.393331122254)*x_ref[2]
        ref[0,3,2,1]=(-0.646477042209)*x_ref[0]**o+(-0.26169646294)*x_ref[0]+(0.992265690785)*x_ref[1]**o+(-0.678558332435)*x_ref[1]+(-0.285531525349)*x_ref[2]**o+(-0.571647919822)*x_ref[2]
        ref[0,4,0,0]=(0.492930260015)*x_ref[0]**o+(-0.647489772768)*x_ref[0]+(-0.381225997009)*x_ref[1]**o+(0.560462598779)*x_ref[1]+(-0.63571677055)*x_ref[2]**o+(0.103213895387)*x_ref[2]
        ref[0,4,0,1]=(-0.0362476689653)*x_ref[0]**o+(-0.478992488756)*x_ref[0]+(0.544907192925)*x_ref[1]**o+(-0.678350370776)*x_ref[1]+(-0.0776363475051)*x_ref[2]**o+(-0.803140341684)*x_ref[2]
        ref[0,4,1,0]=(0.0787683344872)*x_ref[0]**o+(0.688990188782)*x_ref[0]+(0.257089245341)*x_ref[1]**o+(-0.648663765213)*x_ref[1]+(-0.140632312728)*x_ref[2]**o+(0.676209090063)*x_ref[2]
        ref[0,4,1,1]=(0.804994807205)*x_ref[0]**o+(0.74750498671)*x_ref[0]+(-0.00532761506025)*x_ref[1]**o+(-0.295700694165)*x_ref[1]+(-0.209175099115)*x_ref[2]**o+(-0.0126407798374)*x_ref[2]
        ref[0,4,2,0]=(-0.789160064467)*x_ref[0]**o+(0.332412023778)*x_ref[0]+(0.242860020473)*x_ref[1]**o+(0.0284658318544)*x_ref[1]+(-0.357559965972)*x_ref[2]**o+(0.57056582293)*x_ref[2]
        ref[0,4,2,1]=(-0.861027741096)*x_ref[0]**o+(0.545166238342)*x_ref[0]+(0.950975873381)*x_ref[1]**o+(-0.436709572856)*x_ref[1]+(0.93636977927)*x_ref[2]**o+(-0.405205657349)*x_ref[2]
        ref[1,0,0,0]=(0.47880791873)*x_ref[0]**o+(0.037486909403)*x_ref[0]+(0.732218957477)*x_ref[1]**o+(0.274811688271)*x_ref[1]+(0.0686323288694)*x_ref[2]**o+(-0.492347744678)*x_ref[2]
        ref[1,0,0,1]=(-0.618259414329)*x_ref[0]**o+(0.630883795613)*x_ref[0]+(0.269141377847)*x_ref[1]**o+(-0.490916952452)*x_ref[1]+(-0.333070678904)*x_ref[2]**o+(0.0484209401349)*x_ref[2]
        ref[1,0,1,0]=(-0.906059152197)*x_ref[0]**o+(0.0322820078282)*x_ref[0]+(-0.47807076381)*x_ref[1]**o+(0.0296380181906)*x_ref[1]+(0.553686905633)*x_ref[2]**o+(-0.408333784569)*x_ref[2]
        ref[1,0,1,1]=(0.282962689091)*x_ref[0]**o+(-0.11495842442)*x_ref[0]+(0.818142653998)*x_ref[1]**o+(0.563460324128)*x_ref[1]+(-0.993782402791)*x_ref[2]**o+(-0.0193690715193)*x_ref[2]
        ref[1,0,2,0]=(0.287962506263)*x_ref[0]**o+(-0.401293374286)*x_ref[0]+(0.173559459587)*x_ref[1]**o+(0.822343918838)*x_ref[1]+(0.888309195417)*x_ref[2]**o+(0.687878298228)*x_ref[2]
        ref[1,0,2,1]=(-0.926775125361)*x_ref[0]**o+(0.396396669159)*x_ref[0]+(0.632962652289)*x_ref[1]**o+(-0.915667885738)*x_ref[1]+(-0.31075960693)*x_ref[2]**o+(0.920135418244)*x_ref[2]
        ref[1,1,0,0]=(-0.0915500113579)*x_ref[0]**o+(0.489036843145)*x_ref[0]+(0.477982330324)*x_ref[1]**o+(0.703371804742)*x_ref[1]+(-0.471773636091)*x_ref[2]**o+(0.0665811350713)*x_ref[2]
        ref[1,1,0,1]=(0.525411480834)*x_ref[0]**o+(-0.369843393433)*x_ref[0]+(-0.7770705966)*x_ref[1]**o+(0.534153802448)*x_ref[1]+(0.309828458492)*x_ref[2]**o+(0.627690678711)*x_ref[2]
        ref[1,1,1,0]=(0.149039257889)*x_ref[0]**o+(-0.0585283560518)*x_ref[0]+(-0.0505893024858)*x_ref[1]**o+(0.677281229531)*x_ref[1]+(-0.451950028252)*x_ref[2]**o+(0.356600216864)*x_ref[2]
        ref[1,1,1,1]=(-0.528237952467)*x_ref[0]**o+(0.533554438129)*x_ref[0]+(0.741981923136)*x_ref[1]**o+(-0.596922185095)*x_ref[1]+(0.50521173977)*x_ref[2]**o+(-0.247100318829)*x_ref[2]
        ref[1,1,2,0]=(0.667977944954)*x_ref[0]**o+(0.306325804541)*x_ref[0]+(-0.654643365837)*x_ref[1]**o+(0.0176380527447)*x_ref[1]+(-0.218012920296)*x_ref[2]**o+(-0.181740681902)*x_ref[2]
        ref[1,1,2,1]=(0.440430981908)*x_ref[0]**o+(0.994608754477)*x_ref[0]+(0.114426402744)*x_ref[1]**o+(-0.896222828258)*x_ref[1]+(0.997911423594)*x_ref[2]**o+(0.648328801403)*x_ref[2]
        ref[1,2,0,0]=(0.397328767103)*x_ref[0]**o+(0.780110953188)*x_ref[0]+(0.453955649367)*x_ref[1]**o+(0.953839185994)*x_ref[1]+(0.995536671917)*x_ref[2]**o+(0.177278490615)*x_ref[2]
        ref[1,2,0,1]=(-0.154393442621)*x_ref[0]**o+(-0.100267841021)*x_ref[0]+(-0.781347482738)*x_ref[1]**o+(0.391441261298)*x_ref[1]+(0.469832428629)*x_ref[2]**o+(0.608531800159)*x_ref[2]
        ref[1,2,1,0]=(-0.534167310466)*x_ref[0]**o+(0.414564904301)*x_ref[0]+(-0.0674568638795)*x_ref[1]**o+(0.577721681963)*x_ref[1]+(-0.00750850180411)*x_ref[2]**o+(0.0817658166565)*x_ref[2]
        ref[1,2,1,1]=(-0.529547188418)*x_ref[0]**o+(0.267917121456)*x_ref[0]+(0.372449729423)*x_ref[1]**o+(0.887072823755)*x_ref[1]+(0.887821539758)*x_ref[2]**o+(0.249771090594)*x_ref[2]
        ref[1,2,2,0]=(0.949056994737)*x_ref[0]**o+(-0.174523839411)*x_ref[0]+(-0.646085737348)*x_ref[1]**o+(0.449358235114)*x_ref[1]+(-0.970748494721)*x_ref[2]**o+(-0.410800591341)*x_ref[2]
        ref[1,2,2,1]=(0.879006069022)*x_ref[0]**o+(-0.510106021113)*x_ref[0]+(0.0203703221469)*x_ref[1]**o+(-0.844338135123)*x_ref[1]+(0.752168319634)*x_ref[2]**o+(0.158760999046)*x_ref[2]
        ref[1,3,0,0]=(0.845321813876)*x_ref[0]**o+(0.405224606418)*x_ref[0]+(-0.89159606951)*x_ref[1]**o+(-0.949499370695)*x_ref[1]+(0.501499536159)*x_ref[2]**o+(-0.869583954204)*x_ref[2]
        ref[1,3,0,1]=(0.0887248259912)*x_ref[0]**o+(0.232925681778)*x_ref[0]+(0.0315176777236)*x_ref[1]**o+(0.00568107770424)*x_ref[1]+(-0.821963787641)*x_ref[2]**o+(-0.634962715535)*x_ref[2]
        ref[1,3,1,0]=(0.546210053059)*x_ref[0]**o+(-0.688171108228)*x_ref[0]+(-0.699621490343)*x_ref[1]**o+(-0.850445270177)*x_ref[1]+(-0.725131419114)*x_ref[2]**o+(0.484336843024)*x_ref[2]
        ref[1,3,1,1]=(0.278594806693)*x_ref[0]**o+(-0.266710093152)*x_ref[0]+(-0.151515190688)*x_ref[1]**o+(-0.705005965054)*x_ref[1]+(0.811008835405)*x_ref[2]**o+(-0.342637057553)*x_ref[2]
        ref[1,3,2,0]=(0.4748294578)*x_ref[0]**o+(-0.0741346092747)*x_ref[0]+(0.148026337115)*x_ref[1]**o+(0.0252686427245)*x_ref[1]+(-0.479120476874)*x_ref[2]**o+(0.0855008343792)*x_ref[2]
        ref[1,3,2,1]=(-0.337007818128)*x_ref[0]**o+(-0.690464368815)*x_ref[0]+(0.481925609432)*x_ref[1]**o+(-0.0981705702921)*x_ref[1]+(0.19218353071)*x_ref[2]**o+(0.988337158745)*x_ref[2]
        ref[1,4,0,0]=(-0.757351506535)*x_ref[0]**o+(-0.824203825425)*x_ref[0]+(-0.998647750787)*x_ref[1]**o+(-0.111654698732)*x_ref[1]+(-0.713287873387)*x_ref[2]**o+(0.779671510732)*x_ref[2]
        ref[1,4,0,1]=(-0.756370998169)*x_ref[0]**o+(0.826863767951)*x_ref[0]+(0.358310449165)*x_ref[1]**o+(0.882430292099)*x_ref[1]+(0.880156909975)*x_ref[2]**o+(0.360411043274)*x_ref[2]
        ref[1,4,1,0]=(0.226080011536)*x_ref[0]**o+(-0.740536947501)*x_ref[0]+(-0.650527457572)*x_ref[1]**o+(-0.890015156497)*x_ref[1]+(0.688973071404)*x_ref[2]**o+(0.29237820526)*x_ref[2]
        ref[1,4,1,1]=(0.574015626188)*x_ref[0]**o+(-0.675641095131)*x_ref[0]+(0.814289365677)*x_ref[1]**o+(-0.625021588943)*x_ref[1]+(-0.862079110641)*x_ref[2]**o+(0.0363949889967)*x_ref[2]
        ref[1,4,2,0]=(0.126659193582)*x_ref[0]**o+(0.16976937315)*x_ref[0]+(0.18822384431)*x_ref[1]**o+(0.546082142359)*x_ref[1]+(0.551365996908)*x_ref[2]**o+(0.395213783677)*x_ref[2]
        ref[1,4,2,1]=(-0.292436267133)*x_ref[0]**o+(-0.442787521929)*x_ref[0]+(-0.310488595304)*x_ref[1]**o+(0.0918858003303)*x_ref[1]+(-0.986695921261)*x_ref[2]**o+(-0.52979957068)*x_ref[2]
        ref[2,0,0,0]=(0.396669972067)*x_ref[0]**o+(-0.711415400722)*x_ref[0]+(-0.717006050735)*x_ref[1]**o+(-0.0935757817421)*x_ref[1]+(0.743477556176)*x_ref[2]**o+(0.823967976857)*x_ref[2]
        ref[2,0,0,1]=(0.488395228667)*x_ref[0]**o+(0.230144035899)*x_ref[0]+(0.125851877066)*x_ref[1]**o+(0.0536284998009)*x_ref[1]+(-0.0114515128334)*x_ref[2]**o+(-0.494039400348)*x_ref[2]
        ref[2,0,1,0]=(0.900615108208)*x_ref[0]**o+(0.432765237128)*x_ref[0]+(0.981213121806)*x_ref[1]**o+(-0.116934702769)*x_ref[1]+(-0.612499086627)*x_ref[2]**o+(-0.365572411348)*x_ref[2]
        ref[2,0,1,1]=(0.0342670705338)*x_ref[0]**o+(0.54954404771)*x_ref[0]+(0.428653302401)*x_ref[1]**o+(0.537787906346)*x_ref[1]+(-0.64287424457)*x_ref[2]**o+(-0.173877644259)*x_ref[2]
        ref[2,0,2,0]=(-0.0510871045727)*x_ref[0]**o+(-0.908007271573)*x_ref[0]+(0.00194326164542)*x_ref[1]**o+(0.466243299714)*x_ref[1]+(-0.433962263339)*x_ref[2]**o+(0.874724229969)*x_ref[2]
        ref[2,0,2,1]=(0.666308090297)*x_ref[0]**o+(0.933188439742)*x_ref[0]+(-0.0479445062129)*x_ref[1]**o+(0.0747806162498)*x_ref[1]+(0.911844507192)*x_ref[2]**o+(-0.934538480024)*x_ref[2]
        ref[2,1,0,0]=(-0.516290157366)*x_ref[0]**o+(0.437519025532)*x_ref[0]+(-0.170999588789)*x_ref[1]**o+(-0.934096554106)*x_ref[1]+(0.88575800375)*x_ref[2]**o+(0.636966058116)*x_ref[2]
        ref[2,1,0,1]=(0.0172669412005)*x_ref[0]**o+(-0.867857912246)*x_ref[0]+(-0.972747832278)*x_ref[1]**o+(-0.780969292213)*x_ref[1]+(-0.418154913379)*x_ref[2]**o+(0.805902597544)*x_ref[2]
        ref[2,1,1,0]=(-0.964006922451)*x_ref[0]**o+(0.236797578253)*x_ref[0]+(-0.78802971076)*x_ref[1]**o+(0.704165381448)*x_ref[1]+(0.00112348607329)*x_ref[2]**o+(-0.689065853536)*x_ref[2]
        ref[2,1,1,1]=(-0.722392335335)*x_ref[0]**o+(-0.120018753634)*x_ref[0]+(0.216541173904)*x_ref[1]**o+(-0.374710223309)*x_ref[1]+(0.532158723682)*x_ref[2]**o+(0.464637663705)*x_ref[2]
        ref[2,1,2,0]=(-0.183637646888)*x_ref[0]**o+(-0.0680674034088)*x_ref[0]+(-0.114129008735)*x_ref[1]**o+(-0.78666547603)*x_ref[1]+(0.0956689943304)*x_ref[2]**o+(0.743296994432)*x_ref[2]
        ref[2,1,2,1]=(0.611382565923)*x_ref[0]**o+(-0.768682259494)*x_ref[0]+(-0.540943249593)*x_ref[1]**o+(0.753523431999)*x_ref[1]+(0.610689642267)*x_ref[2]**o+(0.0122811249496)*x_ref[2]
        ref[2,2,0,0]=(-0.789273374112)*x_ref[0]**o+(0.0977540682694)*x_ref[0]+(-0.829719260296)*x_ref[1]**o+(0.824773509135)*x_ref[1]+(0.216669854484)*x_ref[2]**o+(-0.764947843182)*x_ref[2]
        ref[2,2,0,1]=(-0.340784170043)*x_ref[0]**o+(-0.267437208067)*x_ref[0]+(-0.430973997706)*x_ref[1]**o+(-0.0846177320866)*x_ref[1]+(0.688599546971)*x_ref[2]**o+(0.265513958383)*x_ref[2]
        ref[2,2,1,0]=(-0.237677411545)*x_ref[0]**o+(-0.792144835449)*x_ref[0]+(-0.435763499999)*x_ref[1]**o+(0.682920743804)*x_ref[1]+(0.116459199169)*x_ref[2]**o+(0.254527225097)*x_ref[2]
        ref[2,2,1,1]=(0.342906428734)*x_ref[0]**o+(0.206470310722)*x_ref[0]+(-0.352429884648)*x_ref[1]**o+(-0.168905312607)*x_ref[1]+(-0.105529665095)*x_ref[2]**o+(0.199873788005)*x_ref[2]
        ref[2,2,2,0]=(0.677855511296)*x_ref[0]**o+(0.983953993003)*x_ref[0]+(-0.64997591911)*x_ref[1]**o+(-0.629221981454)*x_ref[1]+(0.990621105566)*x_ref[2]**o+(0.785766133934)*x_ref[2]
        ref[2,2,2,1]=(-0.570482617922)*x_ref[0]**o+(-0.151066051792)*x_ref[0]+(-0.511453364326)*x_ref[1]**o+(0.128824691419)*x_ref[1]+(-0.520127324328)*x_ref[2]**o+(-0.157450052117)*x_ref[2]
        ref[2,3,0,0]=(-0.423827660795)*x_ref[0]**o+(-0.815943363397)*x_ref[0]+(-0.636070023986)*x_ref[1]**o+(0.251327239067)*x_ref[1]+(-0.401641739847)*x_ref[2]**o+(0.130280903755)*x_ref[2]
        ref[2,3,0,1]=(0.0411439121625)*x_ref[0]**o+(0.621609981629)*x_ref[0]+(-0.589059932668)*x_ref[1]**o+(0.543095349334)*x_ref[1]+(-0.0122421869863)*x_ref[2]**o+(-0.776924587359)*x_ref[2]
        ref[2,3,1,0]=(0.63252327363)*x_ref[0]**o+(0.275597518383)*x_ref[0]+(0.832198526441)*x_ref[1]**o+(-0.92855918951)*x_ref[1]+(0.490343890624)*x_ref[2]**o+(-0.412580022625)*x_ref[2]
        ref[2,3,1,1]=(-0.560907436774)*x_ref[0]**o+(0.413956414518)*x_ref[0]+(-0.849736975879)*x_ref[1]**o+(0.167861053833)*x_ref[1]+(-0.316526371498)*x_ref[2]**o+(-0.431245755915)*x_ref[2]
        ref[2,3,2,0]=(0.883757549647)*x_ref[0]**o+(-0.614133317417)*x_ref[0]+(0.923442235162)*x_ref[1]**o+(0.483998522495)*x_ref[1]+(0.649003328717)*x_ref[2]**o+(-0.320181801832)*x_ref[2]
        ref[2,3,2,1]=(0.181982086686)*x_ref[0]**o+(-0.88761330228)*x_ref[0]+(-0.752108545831)*x_ref[1]**o+(0.105681418232)*x_ref[1]+(0.211053107745)*x_ref[2]**o+(-0.840650090772)*x_ref[2]
        ref[2,4,0,0]=(0.774235220873)*x_ref[0]**o+(-0.394430371883)*x_ref[0]+(-0.82551531411)*x_ref[1]**o+(-0.397374538567)*x_ref[1]+(0.51757884795)*x_ref[2]**o+(-0.878919341817)*x_ref[2]
        ref[2,4,0,1]=(-0.282445913776)*x_ref[0]**o+(-0.252614209563)*x_ref[0]+(-0.66810962709)*x_ref[1]**o+(0.912660025157)*x_ref[1]+(0.786200789061)*x_ref[2]**o+(-0.271329907892)*x_ref[2]
        ref[2,4,1,0]=(-0.415061697718)*x_ref[0]**o+(-0.577077815899)*x_ref[0]+(-0.479115469184)*x_ref[1]**o+(0.285424287206)*x_ref[1]+(0.580737671113)*x_ref[2]**o+(0.345716282261)*x_ref[2]
        ref[2,4,1,1]=(0.103116885191)*x_ref[0]**o+(0.472934825259)*x_ref[0]+(0.192747945705)*x_ref[1]**o+(0.106949097334)*x_ref[1]+(-0.109457290729)*x_ref[2]**o+(0.843345964596)*x_ref[2]
        ref[2,4,2,0]=(0.600281530422)*x_ref[0]**o+(-0.427230650838)*x_ref[0]+(-0.868150408946)*x_ref[1]**o+(-0.203647711262)*x_ref[1]+(-0.550477804035)*x_ref[2]**o+(0.415736214172)*x_ref[2]
        ref[2,4,2,1]=(0.739232217202)*x_ref[0]**o+(-0.476864848171)*x_ref[0]+(0.367627911492)*x_ref[1]**o+(0.105176582927)*x_ref[1]+(0.482113806667)*x_ref[2]**o+(0.782020005885)*x_ref[2]
        ref[3,0,0,0]=(0.938839141593)*x_ref[0]**o+(-0.655793939395)*x_ref[0]+(0.0224780413611)*x_ref[1]**o+(0.252535317383)*x_ref[1]+(-0.887022478757)*x_ref[2]**o+(-0.451912746232)*x_ref[2]
        ref[3,0,0,1]=(-0.196994797427)*x_ref[0]**o+(-0.222816920678)*x_ref[0]+(0.37131460044)*x_ref[1]**o+(0.124872956754)*x_ref[1]+(-0.872680499884)*x_ref[2]**o+(-0.490521736465)*x_ref[2]
        ref[3,0,1,0]=(-0.977712672434)*x_ref[0]**o+(-0.665877989129)*x_ref[0]+(-0.541103000349)*x_ref[1]**o+(-0.527523471522)*x_ref[1]+(0.469063433285)*x_ref[2]**o+(-0.410680406273)*x_ref[2]
        ref[3,0,1,1]=(0.371612180735)*x_ref[0]**o+(-0.882397978651)*x_ref[0]+(-0.333119569273)*x_ref[1]**o+(0.0439599202688)*x_ref[1]+(-0.378698960758)*x_ref[2]**o+(0.373830212832)*x_ref[2]
        ref[3,0,2,0]=(0.13552136805)*x_ref[0]**o+(-0.347572305079)*x_ref[0]+(0.194173745453)*x_ref[1]**o+(-0.536374783086)*x_ref[1]+(-0.0928402883882)*x_ref[2]**o+(0.972386889251)*x_ref[2]
        ref[3,0,2,1]=(-0.673716579315)*x_ref[0]**o+(0.497683603572)*x_ref[0]+(0.0894501300871)*x_ref[1]**o+(-0.691905501083)*x_ref[1]+(-0.819373609736)*x_ref[2]**o+(0.876955815163)*x_ref[2]
        ref[3,1,0,0]=(-0.612778977625)*x_ref[0]**o+(0.898593519513)*x_ref[0]+(-0.579505051936)*x_ref[1]**o+(0.294087297529)*x_ref[1]+(0.713022674221)*x_ref[2]**o+(-0.0994065928128)*x_ref[2]
        ref[3,1,0,1]=(-0.141600251938)*x_ref[0]**o+(-0.677815682935)*x_ref[0]+(-0.408920459122)*x_ref[1]**o+(-0.723029003437)*x_ref[1]+(0.481057659884)*x_ref[2]**o+(0.427909771498)*x_ref[2]
        ref[3,1,1,0]=(-0.516486224771)*x_ref[0]**o+(0.526484181999)*x_ref[0]+(0.911793059151)*x_ref[1]**o+(0.409808964766)*x_ref[1]+(-0.413591542253)*x_ref[2]**o+(-0.759664326823)*x_ref[2]
        ref[3,1,1,1]=(0.905324377656)*x_ref[0]**o+(-0.208896172985)*x_ref[0]+(-0.422109371498)*x_ref[1]**o+(0.491883832748)*x_ref[1]+(-0.203413002012)*x_ref[2]**o+(0.406737219074)*x_ref[2]
        ref[3,1,2,0]=(-0.573692277755)*x_ref[0]**o+(0.895762539503)*x_ref[0]+(0.176538736347)*x_ref[1]**o+(-0.847640766515)*x_ref[1]+(0.5650742337)*x_ref[2]**o+(0.503165369442)*x_ref[2]
        ref[3,1,2,1]=(0.308521434074)*x_ref[0]**o+(0.616776036606)*x_ref[0]+(-0.405321909102)*x_ref[1]**o+(0.497367404288)*x_ref[1]+(-0.798636270652)*x_ref[2]**o+(-0.337423592684)*x_ref[2]
        ref[3,2,0,0]=(-0.394455434548)*x_ref[0]**o+(-0.292212771133)*x_ref[0]+(-0.941613233015)*x_ref[1]**o+(-0.404522244202)*x_ref[1]+(-0.0112893909573)*x_ref[2]**o+(-0.9706166943)*x_ref[2]
        ref[3,2,0,1]=(0.627774279675)*x_ref[0]**o+(-0.645334179444)*x_ref[0]+(0.691389142649)*x_ref[1]**o+(0.763379715146)*x_ref[1]+(0.610766030943)*x_ref[2]**o+(-0.786459853785)*x_ref[2]
        ref[3,2,1,0]=(-0.0101413740932)*x_ref[0]**o+(-0.0970748510013)*x_ref[0]+(-0.52181886114)*x_ref[1]**o+(-0.612494396935)*x_ref[1]+(0.941988727244)*x_ref[2]**o+(0.0361447091017)*x_ref[2]
        ref[3,2,1,1]=(-0.0943447427863)*x_ref[0]**o+(0.708735490064)*x_ref[0]+(-0.352853510872)*x_ref[1]**o+(-0.545593672805)*x_ref[1]+(-0.0106851492926)*x_ref[2]**o+(0.712976402827)*x_ref[2]
        ref[3,2,2,0]=(-0.522341854545)*x_ref[0]**o+(0.320365455548)*x_ref[0]+(-0.111682800076)*x_ref[1]**o+(-0.337351991987)*x_ref[1]+(-0.33503967624)*x_ref[2]**o+(-0.215395174878)*x_ref[2]
        ref[3,2,2,1]=(-0.191634883402)*x_ref[0]**o+(-0.79050407204)*x_ref[0]+(-0.881959919487)*x_ref[1]**o+(0.962597211649)*x_ref[1]+(0.602181506788)*x_ref[2]**o+(-0.430152854642)*x_ref[2]
        ref[3,3,0,0]=(-0.458551188388)*x_ref[0]**o+(0.62195840207)*x_ref[0]+(0.646839412108)*x_ref[1]**o+(-0.597989589884)*x_ref[1]+(-0.160578706146)*x_ref[2]**o+(0.885101198188)*x_ref[2]
        ref[3,3,0,1]=(-0.806015868617)*x_ref[0]**o+(0.0617182982024)*x_ref[0]+(0.296661747576)*x_ref[1]**o+(0.381687206096)*x_ref[1]+(0.954310501938)*x_ref[2]**o+(0.112880684912)*x_ref[2]
        ref[3,3,1,0]=(0.485202154902)*x_ref[0]**o+(0.528514725346)*x_ref[0]+(0.550444453321)*x_ref[1]**o+(-0.816313566191)*x_ref[1]+(0.945151247472)*x_ref[2]**o+(-0.201955849739)*x_ref[2]
        ref[3,3,1,1]=(0.245157893191)*x_ref[0]**o+(-0.281606458201)*x_ref[0]+(-0.312808561115)*x_ref[1]**o+(0.185786713188)*x_ref[1]+(0.996302657312)*x_ref[2]**o+(-0.227884060488)*x_ref[2]
        ref[3,3,2,0]=(0.297297617074)*x_ref[0]**o+(0.156994991329)*x_ref[0]+(-0.403988935033)*x_ref[1]**o+(-0.0894931373916)*x_ref[1]+(-0.979214022227)*x_ref[2]**o+(0.287684161706)*x_ref[2]
        ref[3,3,2,1]=(-0.287794860211)*x_ref[0]**o+(0.169671378363)*x_ref[0]+(0.0172804553692)*x_ref[1]**o+(-0.973449618671)*x_ref[1]+(0.0652371831894)*x_ref[2]**o+(0.775978144983)*x_ref[2]
        ref[3,4,0,0]=(0.54203905332)*x_ref[0]**o+(0.66731094672)*x_ref[0]+(-0.865207826406)*x_ref[1]**o+(0.466596502355)*x_ref[1]+(-0.45336027189)*x_ref[2]**o+(-0.266404323774)*x_ref[2]
        ref[3,4,0,1]=(-0.0264326659602)*x_ref[0]**o+(0.102421850054)*x_ref[0]+(-0.672364919167)*x_ref[1]**o+(-0.461816126462)*x_ref[1]+(0.11436848926)*x_ref[2]**o+(-0.78486118431)*x_ref[2]
        ref[3,4,1,0]=(-0.941616359303)*x_ref[0]**o+(-0.0519153474588)*x_ref[0]+(0.8597678151)*x_ref[1]**o+(0.164278414756)*x_ref[1]+(0.496859863828)*x_ref[2]**o+(-0.413785115519)*x_ref[2]
        ref[3,4,1,1]=(-0.726802133704)*x_ref[0]**o+(0.685989768699)*x_ref[0]+(0.499728992214)*x_ref[1]**o+(0.675799006108)*x_ref[1]+(0.205370350943)*x_ref[2]**o+(0.983979039059)*x_ref[2]
        ref[3,4,2,0]=(-0.121723702011)*x_ref[0]**o+(0.350456030225)*x_ref[0]+(0.211968006265)*x_ref[1]**o+(-0.960265509868)*x_ref[1]+(-0.593090834374)*x_ref[2]**o+(-0.631506211212)*x_ref[2]
        ref[3,4,2,1]=(0.279791655852)*x_ref[0]**o+(0.236927673275)*x_ref[0]+(0.84188823482)*x_ref[1]**o+(0.208410505206)*x_ref[1]+(0.392354982825)*x_ref[2]**o+(0.671826571516)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromData_ContinuousFunction_rank0(self):
      """
      tests interpolate for rank 0 Data onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ContinuousFunction onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(),w)
      ref=Data(0,(),w_ref)
      if dim==2:
        arg=(-0.0071553637759)*x[0]**o+(0.357615094637)*x[0]+(0.980385970972)*x[1]**o+(0.423505273491)*x[1]
        ref=(-0.0071553637759)*x_ref[0]**o+(0.357615094637)*x_ref[0]+(0.980385970972)*x_ref[1]**o+(0.423505273491)*x_ref[1]
      else:
        arg=(0.546251547514)*x[0]**o+(-0.530405447164)*x[0]+(-0.901465213708)*x[1]**o+(-0.986019835171)*x[1]+(-0.731836172089)*x[2]**o+(0.377317674082)*x[2]
        ref=(0.546251547514)*x_ref[0]**o+(-0.530405447164)*x_ref[0]+(-0.901465213708)*x_ref[1]**o+(-0.986019835171)*x_ref[1]+(-0.731836172089)*x_ref[2]**o+(0.377317674082)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromData_ContinuousFunction_rank1(self):
      """
      tests interpolate for rank 1 Data onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ContinuousFunction onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(2,),w)
      ref=Data(0,(2,),w_ref)
      if dim==2:
        arg[0]=(-0.138216822552)*x[0]**o+(0.908302047805)*x[0]+(-0.247600939209)*x[1]**o+(0.776239000167)*x[1]
        arg[1]=(-0.577926899073)*x[0]**o+(0.714263102346)*x[0]+(-0.343808625475)*x[1]**o+(0.786295563423)*x[1]
        ref[0]=(-0.138216822552)*x_ref[0]**o+(0.908302047805)*x_ref[0]+(-0.247600939209)*x_ref[1]**o+(0.776239000167)*x_ref[1]
        ref[1]=(-0.577926899073)*x_ref[0]**o+(0.714263102346)*x_ref[0]+(-0.343808625475)*x_ref[1]**o+(0.786295563423)*x_ref[1]
      else:
        arg[0]=(0.020232467504)*x[0]**o+(-0.0449727381653)*x[0]+(0.00132143537494)*x[1]**o+(-0.870292414095)*x[1]+(-0.926731511382)*x[2]**o+(-0.398915622332)*x[2]
        arg[1]=(0.203864545604)*x[0]**o+(0.150501673382)*x[0]+(0.132132290114)*x[1]**o+(-0.720617456803)*x[1]+(0.673909788445)*x[2]**o+(0.145376368499)*x[2]
        ref[0]=(0.020232467504)*x_ref[0]**o+(-0.0449727381653)*x_ref[0]+(0.00132143537494)*x_ref[1]**o+(-0.870292414095)*x_ref[1]+(-0.926731511382)*x_ref[2]**o+(-0.398915622332)*x_ref[2]
        ref[1]=(0.203864545604)*x_ref[0]**o+(0.150501673382)*x_ref[0]+(0.132132290114)*x_ref[1]**o+(-0.720617456803)*x_ref[1]+(0.673909788445)*x_ref[2]**o+(0.145376368499)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromData_ContinuousFunction_rank2(self):
      """
      tests interpolate for rank 2 Data onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ContinuousFunction onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5),w)
      ref=Data(0,(4, 5),w_ref)
      if dim==2:
        arg[0,0]=(-0.344685653483)*x[0]**o+(-0.618443209029)*x[0]+(-0.338246837583)*x[1]**o+(-0.709311881709)*x[1]
        arg[0,1]=(-0.176341370817)*x[0]**o+(-0.230030832268)*x[0]+(-0.335163038431)*x[1]**o+(0.278627093006)*x[1]
        arg[0,2]=(0.376270607923)*x[0]**o+(0.895822309578)*x[0]+(0.418316736671)*x[1]**o+(-0.951461687587)*x[1]
        arg[0,3]=(0.69925470532)*x[0]**o+(-0.829917904706)*x[0]+(0.60851481091)*x[1]**o+(-0.880924691692)*x[1]
        arg[0,4]=(0.537570501499)*x[0]**o+(0.584091505535)*x[0]+(-0.0045669307163)*x[1]**o+(0.162714723047)*x[1]
        arg[1,0]=(-0.244615538677)*x[0]**o+(-0.0168359545772)*x[0]+(-0.511097014251)*x[1]**o+(-0.476830089968)*x[1]
        arg[1,1]=(0.350953788855)*x[0]**o+(0.0859931352733)*x[0]+(-0.565664846934)*x[1]**o+(-0.230383210864)*x[1]
        arg[1,2]=(0.787537227138)*x[0]**o+(-0.97589028834)*x[0]+(0.978870857538)*x[1]**o+(0.436540016432)*x[1]
        arg[1,3]=(-0.806777674064)*x[0]**o+(-0.384971723467)*x[0]+(-0.423836322283)*x[1]**o+(0.470609842503)*x[1]
        arg[1,4]=(0.288422170494)*x[0]**o+(-0.467670464119)*x[0]+(-0.616952048135)*x[1]**o+(0.604831335817)*x[1]
        arg[2,0]=(0.0461969205986)*x[0]**o+(0.663083190578)*x[0]+(-0.673597938582)*x[1]**o+(0.217389040208)*x[1]
        arg[2,1]=(-0.32008443048)*x[0]**o+(-0.970854886309)*x[0]+(0.630220968872)*x[1]**o+(0.827824011117)*x[1]
        arg[2,2]=(-0.232867522682)*x[0]**o+(-0.17921445933)*x[0]+(-0.565289194319)*x[1]**o+(0.0251729876325)*x[1]
        arg[2,3]=(-0.862002880133)*x[0]**o+(0.64790108714)*x[0]+(-0.469428882373)*x[1]**o+(0.50581485158)*x[1]
        arg[2,4]=(-0.957389923449)*x[0]**o+(-0.716972488151)*x[0]+(0.848168933491)*x[1]**o+(0.78614576485)*x[1]
        arg[3,0]=(0.417890378265)*x[0]**o+(-0.128884271967)*x[0]+(0.981251819451)*x[1]**o+(0.837945499182)*x[1]
        arg[3,1]=(-0.296791907211)*x[0]**o+(0.913966659909)*x[0]+(0.0171079547828)*x[1]**o+(0.472922286638)*x[1]
        arg[3,2]=(-0.549504779354)*x[0]**o+(0.356989069435)*x[0]+(-0.0978494385269)*x[1]**o+(0.59354694599)*x[1]
        arg[3,3]=(0.0272055308458)*x[0]**o+(0.560184544705)*x[0]+(-0.825452907104)*x[1]**o+(0.701919616153)*x[1]
        arg[3,4]=(-0.33094753848)*x[0]**o+(0.877723269108)*x[0]+(-0.491915661514)*x[1]**o+(-0.98784807255)*x[1]
        ref[0,0]=(-0.344685653483)*x_ref[0]**o+(-0.618443209029)*x_ref[0]+(-0.338246837583)*x_ref[1]**o+(-0.709311881709)*x_ref[1]
        ref[0,1]=(-0.176341370817)*x_ref[0]**o+(-0.230030832268)*x_ref[0]+(-0.335163038431)*x_ref[1]**o+(0.278627093006)*x_ref[1]
        ref[0,2]=(0.376270607923)*x_ref[0]**o+(0.895822309578)*x_ref[0]+(0.418316736671)*x_ref[1]**o+(-0.951461687587)*x_ref[1]
        ref[0,3]=(0.69925470532)*x_ref[0]**o+(-0.829917904706)*x_ref[0]+(0.60851481091)*x_ref[1]**o+(-0.880924691692)*x_ref[1]
        ref[0,4]=(0.537570501499)*x_ref[0]**o+(0.584091505535)*x_ref[0]+(-0.0045669307163)*x_ref[1]**o+(0.162714723047)*x_ref[1]
        ref[1,0]=(-0.244615538677)*x_ref[0]**o+(-0.0168359545772)*x_ref[0]+(-0.511097014251)*x_ref[1]**o+(-0.476830089968)*x_ref[1]
        ref[1,1]=(0.350953788855)*x_ref[0]**o+(0.0859931352733)*x_ref[0]+(-0.565664846934)*x_ref[1]**o+(-0.230383210864)*x_ref[1]
        ref[1,2]=(0.787537227138)*x_ref[0]**o+(-0.97589028834)*x_ref[0]+(0.978870857538)*x_ref[1]**o+(0.436540016432)*x_ref[1]
        ref[1,3]=(-0.806777674064)*x_ref[0]**o+(-0.384971723467)*x_ref[0]+(-0.423836322283)*x_ref[1]**o+(0.470609842503)*x_ref[1]
        ref[1,4]=(0.288422170494)*x_ref[0]**o+(-0.467670464119)*x_ref[0]+(-0.616952048135)*x_ref[1]**o+(0.604831335817)*x_ref[1]
        ref[2,0]=(0.0461969205986)*x_ref[0]**o+(0.663083190578)*x_ref[0]+(-0.673597938582)*x_ref[1]**o+(0.217389040208)*x_ref[1]
        ref[2,1]=(-0.32008443048)*x_ref[0]**o+(-0.970854886309)*x_ref[0]+(0.630220968872)*x_ref[1]**o+(0.827824011117)*x_ref[1]
        ref[2,2]=(-0.232867522682)*x_ref[0]**o+(-0.17921445933)*x_ref[0]+(-0.565289194319)*x_ref[1]**o+(0.0251729876325)*x_ref[1]
        ref[2,3]=(-0.862002880133)*x_ref[0]**o+(0.64790108714)*x_ref[0]+(-0.469428882373)*x_ref[1]**o+(0.50581485158)*x_ref[1]
        ref[2,4]=(-0.957389923449)*x_ref[0]**o+(-0.716972488151)*x_ref[0]+(0.848168933491)*x_ref[1]**o+(0.78614576485)*x_ref[1]
        ref[3,0]=(0.417890378265)*x_ref[0]**o+(-0.128884271967)*x_ref[0]+(0.981251819451)*x_ref[1]**o+(0.837945499182)*x_ref[1]
        ref[3,1]=(-0.296791907211)*x_ref[0]**o+(0.913966659909)*x_ref[0]+(0.0171079547828)*x_ref[1]**o+(0.472922286638)*x_ref[1]
        ref[3,2]=(-0.549504779354)*x_ref[0]**o+(0.356989069435)*x_ref[0]+(-0.0978494385269)*x_ref[1]**o+(0.59354694599)*x_ref[1]
        ref[3,3]=(0.0272055308458)*x_ref[0]**o+(0.560184544705)*x_ref[0]+(-0.825452907104)*x_ref[1]**o+(0.701919616153)*x_ref[1]
        ref[3,4]=(-0.33094753848)*x_ref[0]**o+(0.877723269108)*x_ref[0]+(-0.491915661514)*x_ref[1]**o+(-0.98784807255)*x_ref[1]
      else:
        arg[0,0]=(-0.434721514594)*x[0]**o+(-0.28074158231)*x[0]+(0.102692621316)*x[1]**o+(0.399677492352)*x[1]+(0.39799987301)*x[2]**o+(-0.134547332796)*x[2]
        arg[0,1]=(-0.814630031471)*x[0]**o+(-0.0722898377573)*x[0]+(0.356831166041)*x[1]**o+(0.0556663518903)*x[1]+(-0.335764152958)*x[2]**o+(0.344564519072)*x[2]
        arg[0,2]=(-0.881698115197)*x[0]**o+(-0.030759288922)*x[0]+(-0.23126177081)*x[1]**o+(0.865011678258)*x[1]+(0.961867471932)*x[2]**o+(-0.04809978604)*x[2]
        arg[0,3]=(-0.931319970946)*x[0]**o+(0.807195614255)*x[0]+(0.116305510032)*x[1]**o+(0.337873586448)*x[1]+(0.523766276637)*x[2]**o+(-0.890502808117)*x[2]
        arg[0,4]=(0.952369561411)*x[0]**o+(0.518186569223)*x[0]+(0.184454818734)*x[1]**o+(-0.198618411156)*x[1]+(0.891317797589)*x[2]**o+(-0.312268228065)*x[2]
        arg[1,0]=(-0.770379397538)*x[0]**o+(0.11068893635)*x[0]+(0.435574518842)*x[1]**o+(-0.895094805657)*x[1]+(0.719886389608)*x[2]**o+(0.617254266829)*x[2]
        arg[1,1]=(-0.968566082536)*x[0]**o+(0.70331821608)*x[0]+(-0.256092697223)*x[1]**o+(0.214482838048)*x[1]+(-0.20817108493)*x[2]**o+(-0.741961694045)*x[2]
        arg[1,2]=(0.928194134464)*x[0]**o+(0.748278982508)*x[0]+(0.459001496702)*x[1]**o+(-0.333954496033)*x[1]+(0.0451659439418)*x[2]**o+(0.0355669781059)*x[2]
        arg[1,3]=(-0.285564661682)*x[0]**o+(-0.780972389596)*x[0]+(0.20442278849)*x[1]**o+(-0.425673038027)*x[1]+(-0.886021022849)*x[2]**o+(0.2231241391)*x[2]
        arg[1,4]=(0.48756511391)*x[0]**o+(0.904219456651)*x[0]+(-0.723245954496)*x[1]**o+(-0.876483077689)*x[1]+(0.454913273237)*x[2]**o+(0.907651543551)*x[2]
        arg[2,0]=(-0.490623674798)*x[0]**o+(0.179561344392)*x[0]+(-0.257796709265)*x[1]**o+(0.470216680739)*x[1]+(-0.334946805819)*x[2]**o+(-0.295308708488)*x[2]
        arg[2,1]=(-0.0753731003603)*x[0]**o+(0.848622766023)*x[0]+(-0.168762035291)*x[1]**o+(-0.132048839333)*x[1]+(0.229089231051)*x[2]**o+(0.280032186468)*x[2]
        arg[2,2]=(0.601432167354)*x[0]**o+(-0.0857688429764)*x[0]+(0.556763499853)*x[1]**o+(-0.0953568285359)*x[1]+(-0.396589175262)*x[2]**o+(0.437257052414)*x[2]
        arg[2,3]=(0.239710792478)*x[0]**o+(0.255863660922)*x[0]+(-0.860831814227)*x[1]**o+(0.416057968294)*x[1]+(0.284515440811)*x[2]**o+(-0.0737992548315)*x[2]
        arg[2,4]=(-0.86884373286)*x[0]**o+(0.306811019609)*x[0]+(0.955007347168)*x[1]**o+(0.731959544948)*x[1]+(0.326947842619)*x[2]**o+(0.430618227986)*x[2]
        arg[3,0]=(0.450341287007)*x[0]**o+(-0.0318463261779)*x[0]+(-0.755083793007)*x[1]**o+(0.427706480425)*x[1]+(0.224383931841)*x[2]**o+(-0.801200151542)*x[2]
        arg[3,1]=(-0.750405028813)*x[0]**o+(-0.531057162102)*x[0]+(0.120116357521)*x[1]**o+(-0.247640504917)*x[1]+(-0.360494371461)*x[2]**o+(-0.496404068337)*x[2]
        arg[3,2]=(-0.202775617449)*x[0]**o+(0.833337604045)*x[0]+(0.551285270602)*x[1]**o+(0.266630410234)*x[1]+(0.903041333927)*x[2]**o+(0.527233939028)*x[2]
        arg[3,3]=(0.12892683791)*x[0]**o+(-0.486726243104)*x[0]+(-0.340728397711)*x[1]**o+(-0.670692588507)*x[1]+(0.832250132781)*x[2]**o+(0.276897913372)*x[2]
        arg[3,4]=(0.839441639419)*x[0]**o+(0.362150671815)*x[0]+(0.728370029046)*x[1]**o+(0.149540296968)*x[1]+(-0.0418719598944)*x[2]**o+(-0.331805297414)*x[2]
        ref[0,0]=(-0.434721514594)*x_ref[0]**o+(-0.28074158231)*x_ref[0]+(0.102692621316)*x_ref[1]**o+(0.399677492352)*x_ref[1]+(0.39799987301)*x_ref[2]**o+(-0.134547332796)*x_ref[2]
        ref[0,1]=(-0.814630031471)*x_ref[0]**o+(-0.0722898377573)*x_ref[0]+(0.356831166041)*x_ref[1]**o+(0.0556663518903)*x_ref[1]+(-0.335764152958)*x_ref[2]**o+(0.344564519072)*x_ref[2]
        ref[0,2]=(-0.881698115197)*x_ref[0]**o+(-0.030759288922)*x_ref[0]+(-0.23126177081)*x_ref[1]**o+(0.865011678258)*x_ref[1]+(0.961867471932)*x_ref[2]**o+(-0.04809978604)*x_ref[2]
        ref[0,3]=(-0.931319970946)*x_ref[0]**o+(0.807195614255)*x_ref[0]+(0.116305510032)*x_ref[1]**o+(0.337873586448)*x_ref[1]+(0.523766276637)*x_ref[2]**o+(-0.890502808117)*x_ref[2]
        ref[0,4]=(0.952369561411)*x_ref[0]**o+(0.518186569223)*x_ref[0]+(0.184454818734)*x_ref[1]**o+(-0.198618411156)*x_ref[1]+(0.891317797589)*x_ref[2]**o+(-0.312268228065)*x_ref[2]
        ref[1,0]=(-0.770379397538)*x_ref[0]**o+(0.11068893635)*x_ref[0]+(0.435574518842)*x_ref[1]**o+(-0.895094805657)*x_ref[1]+(0.719886389608)*x_ref[2]**o+(0.617254266829)*x_ref[2]
        ref[1,1]=(-0.968566082536)*x_ref[0]**o+(0.70331821608)*x_ref[0]+(-0.256092697223)*x_ref[1]**o+(0.214482838048)*x_ref[1]+(-0.20817108493)*x_ref[2]**o+(-0.741961694045)*x_ref[2]
        ref[1,2]=(0.928194134464)*x_ref[0]**o+(0.748278982508)*x_ref[0]+(0.459001496702)*x_ref[1]**o+(-0.333954496033)*x_ref[1]+(0.0451659439418)*x_ref[2]**o+(0.0355669781059)*x_ref[2]
        ref[1,3]=(-0.285564661682)*x_ref[0]**o+(-0.780972389596)*x_ref[0]+(0.20442278849)*x_ref[1]**o+(-0.425673038027)*x_ref[1]+(-0.886021022849)*x_ref[2]**o+(0.2231241391)*x_ref[2]
        ref[1,4]=(0.48756511391)*x_ref[0]**o+(0.904219456651)*x_ref[0]+(-0.723245954496)*x_ref[1]**o+(-0.876483077689)*x_ref[1]+(0.454913273237)*x_ref[2]**o+(0.907651543551)*x_ref[2]
        ref[2,0]=(-0.490623674798)*x_ref[0]**o+(0.179561344392)*x_ref[0]+(-0.257796709265)*x_ref[1]**o+(0.470216680739)*x_ref[1]+(-0.334946805819)*x_ref[2]**o+(-0.295308708488)*x_ref[2]
        ref[2,1]=(-0.0753731003603)*x_ref[0]**o+(0.848622766023)*x_ref[0]+(-0.168762035291)*x_ref[1]**o+(-0.132048839333)*x_ref[1]+(0.229089231051)*x_ref[2]**o+(0.280032186468)*x_ref[2]
        ref[2,2]=(0.601432167354)*x_ref[0]**o+(-0.0857688429764)*x_ref[0]+(0.556763499853)*x_ref[1]**o+(-0.0953568285359)*x_ref[1]+(-0.396589175262)*x_ref[2]**o+(0.437257052414)*x_ref[2]
        ref[2,3]=(0.239710792478)*x_ref[0]**o+(0.255863660922)*x_ref[0]+(-0.860831814227)*x_ref[1]**o+(0.416057968294)*x_ref[1]+(0.284515440811)*x_ref[2]**o+(-0.0737992548315)*x_ref[2]
        ref[2,4]=(-0.86884373286)*x_ref[0]**o+(0.306811019609)*x_ref[0]+(0.955007347168)*x_ref[1]**o+(0.731959544948)*x_ref[1]+(0.326947842619)*x_ref[2]**o+(0.430618227986)*x_ref[2]
        ref[3,0]=(0.450341287007)*x_ref[0]**o+(-0.0318463261779)*x_ref[0]+(-0.755083793007)*x_ref[1]**o+(0.427706480425)*x_ref[1]+(0.224383931841)*x_ref[2]**o+(-0.801200151542)*x_ref[2]
        ref[3,1]=(-0.750405028813)*x_ref[0]**o+(-0.531057162102)*x_ref[0]+(0.120116357521)*x_ref[1]**o+(-0.247640504917)*x_ref[1]+(-0.360494371461)*x_ref[2]**o+(-0.496404068337)*x_ref[2]
        ref[3,2]=(-0.202775617449)*x_ref[0]**o+(0.833337604045)*x_ref[0]+(0.551285270602)*x_ref[1]**o+(0.266630410234)*x_ref[1]+(0.903041333927)*x_ref[2]**o+(0.527233939028)*x_ref[2]
        ref[3,3]=(0.12892683791)*x_ref[0]**o+(-0.486726243104)*x_ref[0]+(-0.340728397711)*x_ref[1]**o+(-0.670692588507)*x_ref[1]+(0.832250132781)*x_ref[2]**o+(0.276897913372)*x_ref[2]
        ref[3,4]=(0.839441639419)*x_ref[0]**o+(0.362150671815)*x_ref[0]+(0.728370029046)*x_ref[1]**o+(0.149540296968)*x_ref[1]+(-0.0418719598944)*x_ref[2]**o+(-0.331805297414)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromData_ContinuousFunction_rank3(self):
      """
      tests interpolate for rank 3 Data onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ContinuousFunction onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2),w_ref)
      if dim==2:
        arg[0,0,0]=(-0.475615092269)*x[0]**o+(-0.0893751722575)*x[0]+(0.595881277166)*x[1]**o+(0.397900173915)*x[1]
        arg[0,0,1]=(-0.552614805712)*x[0]**o+(0.14072435445)*x[0]+(0.290231845094)*x[1]**o+(0.573878059041)*x[1]
        arg[0,1,0]=(-0.226148312038)*x[0]**o+(-0.737524507359)*x[0]+(-0.704362709293)*x[1]**o+(-0.197223346266)*x[1]
        arg[0,1,1]=(0.844313529642)*x[0]**o+(-0.948514286846)*x[0]+(0.383754990882)*x[1]**o+(0.629819148632)*x[1]
        arg[1,0,0]=(-0.519122333314)*x[0]**o+(-0.895333584981)*x[0]+(-0.941503934561)*x[1]**o+(0.574440495425)*x[1]
        arg[1,0,1]=(0.34967649382)*x[0]**o+(-0.332598318229)*x[0]+(0.0587818420974)*x[1]**o+(0.0848419508498)*x[1]
        arg[1,1,0]=(0.126943263196)*x[0]**o+(0.20287950383)*x[0]+(-0.305454685691)*x[1]**o+(-0.357695790052)*x[1]
        arg[1,1,1]=(0.104154708153)*x[0]**o+(0.867435765411)*x[0]+(0.656946071961)*x[1]**o+(0.96433811362)*x[1]
        arg[2,0,0]=(-0.407362949457)*x[0]**o+(-0.171021521273)*x[0]+(0.901342300398)*x[1]**o+(-0.0360476097399)*x[1]
        arg[2,0,1]=(-0.709421035652)*x[0]**o+(0.949318093375)*x[0]+(0.155061840049)*x[1]**o+(0.828999317354)*x[1]
        arg[2,1,0]=(-0.573031939083)*x[0]**o+(-0.766186602896)*x[0]+(-0.0483071013367)*x[1]**o+(0.741988754559)*x[1]
        arg[2,1,1]=(-0.826747727748)*x[0]**o+(0.481970220735)*x[0]+(0.735617076451)*x[1]**o+(-0.64482586429)*x[1]
        arg[3,0,0]=(0.0834619008603)*x[0]**o+(-0.122260004323)*x[0]+(-0.193792114997)*x[1]**o+(0.635792828013)*x[1]
        arg[3,0,1]=(0.868630978759)*x[0]**o+(0.54474614716)*x[0]+(-0.514377753448)*x[1]**o+(0.58889412844)*x[1]
        arg[3,1,0]=(-0.233744744882)*x[0]**o+(0.883601383004)*x[0]+(0.575012113003)*x[1]**o+(0.185704140149)*x[1]
        arg[3,1,1]=(-0.948893544046)*x[0]**o+(0.851870038501)*x[0]+(-0.591835259957)*x[1]**o+(-0.958375628808)*x[1]
        arg[4,0,0]=(0.981685480559)*x[0]**o+(0.718249744763)*x[0]+(0.589534430546)*x[1]**o+(0.72294148429)*x[1]
        arg[4,0,1]=(-0.603601154465)*x[0]**o+(0.293688738803)*x[0]+(0.151726724518)*x[1]**o+(0.56316377967)*x[1]
        arg[4,1,0]=(0.539033109011)*x[0]**o+(0.791686885872)*x[0]+(0.878973377358)*x[1]**o+(0.869606341625)*x[1]
        arg[4,1,1]=(-0.0920873290752)*x[0]**o+(-0.132682060309)*x[0]+(0.34023826981)*x[1]**o+(0.585476816741)*x[1]
        arg[5,0,0]=(-0.430989548175)*x[0]**o+(-0.525387682044)*x[0]+(-0.996483829778)*x[1]**o+(-0.564503119535)*x[1]
        arg[5,0,1]=(0.602911018749)*x[0]**o+(-0.348972500704)*x[0]+(0.174681048071)*x[1]**o+(-0.94551825287)*x[1]
        arg[5,1,0]=(0.780926392259)*x[0]**o+(-0.164977652547)*x[0]+(0.224726701017)*x[1]**o+(-0.779109318124)*x[1]
        arg[5,1,1]=(0.470928696402)*x[0]**o+(-0.505871268699)*x[0]+(0.407158974816)*x[1]**o+(0.392856659954)*x[1]
        ref[0,0,0]=(-0.475615092269)*x_ref[0]**o+(-0.0893751722575)*x_ref[0]+(0.595881277166)*x_ref[1]**o+(0.397900173915)*x_ref[1]
        ref[0,0,1]=(-0.552614805712)*x_ref[0]**o+(0.14072435445)*x_ref[0]+(0.290231845094)*x_ref[1]**o+(0.573878059041)*x_ref[1]
        ref[0,1,0]=(-0.226148312038)*x_ref[0]**o+(-0.737524507359)*x_ref[0]+(-0.704362709293)*x_ref[1]**o+(-0.197223346266)*x_ref[1]
        ref[0,1,1]=(0.844313529642)*x_ref[0]**o+(-0.948514286846)*x_ref[0]+(0.383754990882)*x_ref[1]**o+(0.629819148632)*x_ref[1]
        ref[1,0,0]=(-0.519122333314)*x_ref[0]**o+(-0.895333584981)*x_ref[0]+(-0.941503934561)*x_ref[1]**o+(0.574440495425)*x_ref[1]
        ref[1,0,1]=(0.34967649382)*x_ref[0]**o+(-0.332598318229)*x_ref[0]+(0.0587818420974)*x_ref[1]**o+(0.0848419508498)*x_ref[1]
        ref[1,1,0]=(0.126943263196)*x_ref[0]**o+(0.20287950383)*x_ref[0]+(-0.305454685691)*x_ref[1]**o+(-0.357695790052)*x_ref[1]
        ref[1,1,1]=(0.104154708153)*x_ref[0]**o+(0.867435765411)*x_ref[0]+(0.656946071961)*x_ref[1]**o+(0.96433811362)*x_ref[1]
        ref[2,0,0]=(-0.407362949457)*x_ref[0]**o+(-0.171021521273)*x_ref[0]+(0.901342300398)*x_ref[1]**o+(-0.0360476097399)*x_ref[1]
        ref[2,0,1]=(-0.709421035652)*x_ref[0]**o+(0.949318093375)*x_ref[0]+(0.155061840049)*x_ref[1]**o+(0.828999317354)*x_ref[1]
        ref[2,1,0]=(-0.573031939083)*x_ref[0]**o+(-0.766186602896)*x_ref[0]+(-0.0483071013367)*x_ref[1]**o+(0.741988754559)*x_ref[1]
        ref[2,1,1]=(-0.826747727748)*x_ref[0]**o+(0.481970220735)*x_ref[0]+(0.735617076451)*x_ref[1]**o+(-0.64482586429)*x_ref[1]
        ref[3,0,0]=(0.0834619008603)*x_ref[0]**o+(-0.122260004323)*x_ref[0]+(-0.193792114997)*x_ref[1]**o+(0.635792828013)*x_ref[1]
        ref[3,0,1]=(0.868630978759)*x_ref[0]**o+(0.54474614716)*x_ref[0]+(-0.514377753448)*x_ref[1]**o+(0.58889412844)*x_ref[1]
        ref[3,1,0]=(-0.233744744882)*x_ref[0]**o+(0.883601383004)*x_ref[0]+(0.575012113003)*x_ref[1]**o+(0.185704140149)*x_ref[1]
        ref[3,1,1]=(-0.948893544046)*x_ref[0]**o+(0.851870038501)*x_ref[0]+(-0.591835259957)*x_ref[1]**o+(-0.958375628808)*x_ref[1]
        ref[4,0,0]=(0.981685480559)*x_ref[0]**o+(0.718249744763)*x_ref[0]+(0.589534430546)*x_ref[1]**o+(0.72294148429)*x_ref[1]
        ref[4,0,1]=(-0.603601154465)*x_ref[0]**o+(0.293688738803)*x_ref[0]+(0.151726724518)*x_ref[1]**o+(0.56316377967)*x_ref[1]
        ref[4,1,0]=(0.539033109011)*x_ref[0]**o+(0.791686885872)*x_ref[0]+(0.878973377358)*x_ref[1]**o+(0.869606341625)*x_ref[1]
        ref[4,1,1]=(-0.0920873290752)*x_ref[0]**o+(-0.132682060309)*x_ref[0]+(0.34023826981)*x_ref[1]**o+(0.585476816741)*x_ref[1]
        ref[5,0,0]=(-0.430989548175)*x_ref[0]**o+(-0.525387682044)*x_ref[0]+(-0.996483829778)*x_ref[1]**o+(-0.564503119535)*x_ref[1]
        ref[5,0,1]=(0.602911018749)*x_ref[0]**o+(-0.348972500704)*x_ref[0]+(0.174681048071)*x_ref[1]**o+(-0.94551825287)*x_ref[1]
        ref[5,1,0]=(0.780926392259)*x_ref[0]**o+(-0.164977652547)*x_ref[0]+(0.224726701017)*x_ref[1]**o+(-0.779109318124)*x_ref[1]
        ref[5,1,1]=(0.470928696402)*x_ref[0]**o+(-0.505871268699)*x_ref[0]+(0.407158974816)*x_ref[1]**o+(0.392856659954)*x_ref[1]
      else:
        arg[0,0,0]=(0.759590374826)*x[0]**o+(-0.776182110267)*x[0]+(0.713871101133)*x[1]**o+(-0.0991655752543)*x[1]+(0.339491998788)*x[2]**o+(-0.624141085669)*x[2]
        arg[0,0,1]=(-0.948679111098)*x[0]**o+(0.0918139408797)*x[0]+(-0.519481784001)*x[1]**o+(-0.480754367303)*x[1]+(0.996421472879)*x[2]**o+(-0.933032853651)*x[2]
        arg[0,1,0]=(-0.824750788431)*x[0]**o+(0.846735099726)*x[0]+(-0.436835905895)*x[1]**o+(-0.732363323611)*x[1]+(-0.865610972073)*x[2]**o+(0.754759759319)*x[2]
        arg[0,1,1]=(-0.403783782613)*x[0]**o+(0.918875237045)*x[0]+(0.0934155750366)*x[1]**o+(0.832477256884)*x[1]+(0.615080190605)*x[2]**o+(0.000920074523901)*x[2]
        arg[1,0,0]=(0.260531941775)*x[0]**o+(-0.514647294043)*x[0]+(0.237113978946)*x[1]**o+(-0.116626766689)*x[1]+(-0.670422853592)*x[2]**o+(0.574941151693)*x[2]
        arg[1,0,1]=(-0.0133773984155)*x[0]**o+(0.286273972547)*x[0]+(0.0493985332233)*x[1]**o+(-0.567835671454)*x[1]+(0.508314781519)*x[2]**o+(-0.603506084784)*x[2]
        arg[1,1,0]=(0.0237413792908)*x[0]**o+(-0.758893281773)*x[0]+(0.150670689461)*x[1]**o+(0.136386516616)*x[1]+(-0.440203817756)*x[2]**o+(0.796610908529)*x[2]
        arg[1,1,1]=(-0.223549634917)*x[0]**o+(0.396472946824)*x[0]+(0.47830083818)*x[1]**o+(0.059933592631)*x[1]+(0.709371074379)*x[2]**o+(-0.129442424949)*x[2]
        arg[2,0,0]=(0.686170679191)*x[0]**o+(-0.118668021586)*x[0]+(0.033954449978)*x[1]**o+(-0.0508661210104)*x[1]+(0.352824724076)*x[2]**o+(-0.772874133692)*x[2]
        arg[2,0,1]=(0.434879772789)*x[0]**o+(-0.148309931356)*x[0]+(-0.013792688421)*x[1]**o+(0.597899709109)*x[1]+(-0.280336465565)*x[2]**o+(-0.177134087728)*x[2]
        arg[2,1,0]=(-0.636844238715)*x[0]**o+(-0.646269255495)*x[0]+(0.243236542558)*x[1]**o+(0.203480511801)*x[1]+(0.693290035164)*x[2]**o+(0.773354114662)*x[2]
        arg[2,1,1]=(-0.161998867128)*x[0]**o+(0.193608078775)*x[0]+(0.607954851405)*x[1]**o+(0.648078354644)*x[1]+(-0.488891750936)*x[2]**o+(0.912860597431)*x[2]
        arg[3,0,0]=(-0.696909279028)*x[0]**o+(0.039558795649)*x[0]+(0.441121646773)*x[1]**o+(-0.0681790925087)*x[1]+(-0.792437910441)*x[2]**o+(-0.512942395878)*x[2]
        arg[3,0,1]=(-0.264832490825)*x[0]**o+(-0.955976365474)*x[0]+(-0.884641387524)*x[1]**o+(-0.736970630699)*x[1]+(-0.0525378155791)*x[2]**o+(0.149164361449)*x[2]
        arg[3,1,0]=(0.434888951175)*x[0]**o+(-0.218511432369)*x[0]+(0.564377624401)*x[1]**o+(-0.0978716780106)*x[1]+(0.447857191989)*x[2]**o+(-0.466572586856)*x[2]
        arg[3,1,1]=(0.457689985738)*x[0]**o+(0.00322996620178)*x[0]+(0.307787601144)*x[1]**o+(-0.119455639704)*x[1]+(0.70367723461)*x[2]**o+(0.561913096313)*x[2]
        arg[4,0,0]=(-0.759802700221)*x[0]**o+(0.469649690329)*x[0]+(0.0913972297116)*x[1]**o+(0.310277900593)*x[1]+(-0.590266889319)*x[2]**o+(-0.89711266859)*x[2]
        arg[4,0,1]=(0.85506147544)*x[0]**o+(-0.357476737069)*x[0]+(-0.646898911915)*x[1]**o+(-0.674811608049)*x[1]+(0.865265982147)*x[2]**o+(0.720153126392)*x[2]
        arg[4,1,0]=(0.848146446583)*x[0]**o+(0.868166234685)*x[0]+(0.0477110054774)*x[1]**o+(-0.0338098833736)*x[1]+(-0.360024439914)*x[2]**o+(-0.634359083766)*x[2]
        arg[4,1,1]=(-0.672530624126)*x[0]**o+(0.354170615043)*x[0]+(0.486028226561)*x[1]**o+(0.857532578327)*x[1]+(0.15798171243)*x[2]**o+(0.0579857791862)*x[2]
        arg[5,0,0]=(0.653171799956)*x[0]**o+(0.605427034155)*x[0]+(0.7414424011)*x[1]**o+(-0.807505211605)*x[1]+(-0.0411776868808)*x[2]**o+(0.24301961063)*x[2]
        arg[5,0,1]=(-0.336860102074)*x[0]**o+(0.235697646908)*x[0]+(-0.173217953629)*x[1]**o+(0.989643649714)*x[1]+(0.977460054831)*x[2]**o+(-0.743825027125)*x[2]
        arg[5,1,0]=(0.794904875581)*x[0]**o+(0.607548574046)*x[0]+(0.603180304387)*x[1]**o+(0.276995429843)*x[1]+(0.698731315308)*x[2]**o+(-0.966744108727)*x[2]
        arg[5,1,1]=(-0.0230004766002)*x[0]**o+(0.489585418104)*x[0]+(0.382764947116)*x[1]**o+(0.143873532935)*x[1]+(0.932000957196)*x[2]**o+(0.739975795806)*x[2]
        ref[0,0,0]=(0.759590374826)*x_ref[0]**o+(-0.776182110267)*x_ref[0]+(0.713871101133)*x_ref[1]**o+(-0.0991655752543)*x_ref[1]+(0.339491998788)*x_ref[2]**o+(-0.624141085669)*x_ref[2]
        ref[0,0,1]=(-0.948679111098)*x_ref[0]**o+(0.0918139408797)*x_ref[0]+(-0.519481784001)*x_ref[1]**o+(-0.480754367303)*x_ref[1]+(0.996421472879)*x_ref[2]**o+(-0.933032853651)*x_ref[2]
        ref[0,1,0]=(-0.824750788431)*x_ref[0]**o+(0.846735099726)*x_ref[0]+(-0.436835905895)*x_ref[1]**o+(-0.732363323611)*x_ref[1]+(-0.865610972073)*x_ref[2]**o+(0.754759759319)*x_ref[2]
        ref[0,1,1]=(-0.403783782613)*x_ref[0]**o+(0.918875237045)*x_ref[0]+(0.0934155750366)*x_ref[1]**o+(0.832477256884)*x_ref[1]+(0.615080190605)*x_ref[2]**o+(0.000920074523901)*x_ref[2]
        ref[1,0,0]=(0.260531941775)*x_ref[0]**o+(-0.514647294043)*x_ref[0]+(0.237113978946)*x_ref[1]**o+(-0.116626766689)*x_ref[1]+(-0.670422853592)*x_ref[2]**o+(0.574941151693)*x_ref[2]
        ref[1,0,1]=(-0.0133773984155)*x_ref[0]**o+(0.286273972547)*x_ref[0]+(0.0493985332233)*x_ref[1]**o+(-0.567835671454)*x_ref[1]+(0.508314781519)*x_ref[2]**o+(-0.603506084784)*x_ref[2]
        ref[1,1,0]=(0.0237413792908)*x_ref[0]**o+(-0.758893281773)*x_ref[0]+(0.150670689461)*x_ref[1]**o+(0.136386516616)*x_ref[1]+(-0.440203817756)*x_ref[2]**o+(0.796610908529)*x_ref[2]
        ref[1,1,1]=(-0.223549634917)*x_ref[0]**o+(0.396472946824)*x_ref[0]+(0.47830083818)*x_ref[1]**o+(0.059933592631)*x_ref[1]+(0.709371074379)*x_ref[2]**o+(-0.129442424949)*x_ref[2]
        ref[2,0,0]=(0.686170679191)*x_ref[0]**o+(-0.118668021586)*x_ref[0]+(0.033954449978)*x_ref[1]**o+(-0.0508661210104)*x_ref[1]+(0.352824724076)*x_ref[2]**o+(-0.772874133692)*x_ref[2]
        ref[2,0,1]=(0.434879772789)*x_ref[0]**o+(-0.148309931356)*x_ref[0]+(-0.013792688421)*x_ref[1]**o+(0.597899709109)*x_ref[1]+(-0.280336465565)*x_ref[2]**o+(-0.177134087728)*x_ref[2]
        ref[2,1,0]=(-0.636844238715)*x_ref[0]**o+(-0.646269255495)*x_ref[0]+(0.243236542558)*x_ref[1]**o+(0.203480511801)*x_ref[1]+(0.693290035164)*x_ref[2]**o+(0.773354114662)*x_ref[2]
        ref[2,1,1]=(-0.161998867128)*x_ref[0]**o+(0.193608078775)*x_ref[0]+(0.607954851405)*x_ref[1]**o+(0.648078354644)*x_ref[1]+(-0.488891750936)*x_ref[2]**o+(0.912860597431)*x_ref[2]
        ref[3,0,0]=(-0.696909279028)*x_ref[0]**o+(0.039558795649)*x_ref[0]+(0.441121646773)*x_ref[1]**o+(-0.0681790925087)*x_ref[1]+(-0.792437910441)*x_ref[2]**o+(-0.512942395878)*x_ref[2]
        ref[3,0,1]=(-0.264832490825)*x_ref[0]**o+(-0.955976365474)*x_ref[0]+(-0.884641387524)*x_ref[1]**o+(-0.736970630699)*x_ref[1]+(-0.0525378155791)*x_ref[2]**o+(0.149164361449)*x_ref[2]
        ref[3,1,0]=(0.434888951175)*x_ref[0]**o+(-0.218511432369)*x_ref[0]+(0.564377624401)*x_ref[1]**o+(-0.0978716780106)*x_ref[1]+(0.447857191989)*x_ref[2]**o+(-0.466572586856)*x_ref[2]
        ref[3,1,1]=(0.457689985738)*x_ref[0]**o+(0.00322996620178)*x_ref[0]+(0.307787601144)*x_ref[1]**o+(-0.119455639704)*x_ref[1]+(0.70367723461)*x_ref[2]**o+(0.561913096313)*x_ref[2]
        ref[4,0,0]=(-0.759802700221)*x_ref[0]**o+(0.469649690329)*x_ref[0]+(0.0913972297116)*x_ref[1]**o+(0.310277900593)*x_ref[1]+(-0.590266889319)*x_ref[2]**o+(-0.89711266859)*x_ref[2]
        ref[4,0,1]=(0.85506147544)*x_ref[0]**o+(-0.357476737069)*x_ref[0]+(-0.646898911915)*x_ref[1]**o+(-0.674811608049)*x_ref[1]+(0.865265982147)*x_ref[2]**o+(0.720153126392)*x_ref[2]
        ref[4,1,0]=(0.848146446583)*x_ref[0]**o+(0.868166234685)*x_ref[0]+(0.0477110054774)*x_ref[1]**o+(-0.0338098833736)*x_ref[1]+(-0.360024439914)*x_ref[2]**o+(-0.634359083766)*x_ref[2]
        ref[4,1,1]=(-0.672530624126)*x_ref[0]**o+(0.354170615043)*x_ref[0]+(0.486028226561)*x_ref[1]**o+(0.857532578327)*x_ref[1]+(0.15798171243)*x_ref[2]**o+(0.0579857791862)*x_ref[2]
        ref[5,0,0]=(0.653171799956)*x_ref[0]**o+(0.605427034155)*x_ref[0]+(0.7414424011)*x_ref[1]**o+(-0.807505211605)*x_ref[1]+(-0.0411776868808)*x_ref[2]**o+(0.24301961063)*x_ref[2]
        ref[5,0,1]=(-0.336860102074)*x_ref[0]**o+(0.235697646908)*x_ref[0]+(-0.173217953629)*x_ref[1]**o+(0.989643649714)*x_ref[1]+(0.977460054831)*x_ref[2]**o+(-0.743825027125)*x_ref[2]
        ref[5,1,0]=(0.794904875581)*x_ref[0]**o+(0.607548574046)*x_ref[0]+(0.603180304387)*x_ref[1]**o+(0.276995429843)*x_ref[1]+(0.698731315308)*x_ref[2]**o+(-0.966744108727)*x_ref[2]
        ref[5,1,1]=(-0.0230004766002)*x_ref[0]**o+(0.489585418104)*x_ref[0]+(0.382764947116)*x_ref[1]**o+(0.143873532935)*x_ref[1]+(0.932000957196)*x_ref[2]**o+(0.739975795806)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromData_ContinuousFunction_rank4(self):
      """
      tests interpolate for rank 4 Data onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ContinuousFunction onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5, 3, 2),w)
      ref=Data(0,(4, 5, 3, 2),w_ref)
      if dim==2:
        arg[0,0,0,0]=(0.922177773381)*x[0]**o+(0.37324175781)*x[0]+(-0.169208776011)*x[1]**o+(0.72134249236)*x[1]
        arg[0,0,0,1]=(0.85312270129)*x[0]**o+(0.0512919651836)*x[0]+(0.968880249675)*x[1]**o+(0.740058819096)*x[1]
        arg[0,0,1,0]=(-0.018205657901)*x[0]**o+(0.601838710779)*x[0]+(0.211002282731)*x[1]**o+(0.801382123792)*x[1]
        arg[0,0,1,1]=(-0.249128714489)*x[0]**o+(-0.408809542089)*x[0]+(-0.684434920261)*x[1]**o+(0.223674849818)*x[1]
        arg[0,0,2,0]=(0.396627551183)*x[0]**o+(-0.223592778052)*x[0]+(0.227333834259)*x[1]**o+(0.66580805616)*x[1]
        arg[0,0,2,1]=(0.365216692731)*x[0]**o+(-0.670947594503)*x[0]+(0.400260365509)*x[1]**o+(0.10903299253)*x[1]
        arg[0,1,0,0]=(0.823297765985)*x[0]**o+(0.60337946486)*x[0]+(-0.314265083693)*x[1]**o+(-0.00636816227929)*x[1]
        arg[0,1,0,1]=(-0.26216625982)*x[0]**o+(0.767412606588)*x[0]+(-0.284917238441)*x[1]**o+(0.56385025143)*x[1]
        arg[0,1,1,0]=(-0.732326448281)*x[0]**o+(-0.669197323856)*x[0]+(-0.608751752289)*x[1]**o+(-0.853458985188)*x[1]
        arg[0,1,1,1]=(0.637275594334)*x[0]**o+(0.696570850488)*x[0]+(0.012026820657)*x[1]**o+(0.558317439186)*x[1]
        arg[0,1,2,0]=(0.925899092402)*x[0]**o+(0.568045453255)*x[0]+(-0.766701931715)*x[1]**o+(-0.95082586036)*x[1]
        arg[0,1,2,1]=(0.127524211574)*x[0]**o+(-0.65150542164)*x[0]+(-0.365773940068)*x[1]**o+(0.990465711754)*x[1]
        arg[0,2,0,0]=(0.456469052217)*x[0]**o+(0.403391798397)*x[0]+(-0.21921915949)*x[1]**o+(0.171493695945)*x[1]
        arg[0,2,0,1]=(-0.980242506743)*x[0]**o+(0.880225959645)*x[0]+(0.682653186307)*x[1]**o+(-0.561867916377)*x[1]
        arg[0,2,1,0]=(-0.329881998938)*x[0]**o+(0.464976480407)*x[0]+(-0.812583693347)*x[1]**o+(0.0103356333568)*x[1]
        arg[0,2,1,1]=(-0.710350374606)*x[0]**o+(-0.445981415127)*x[0]+(-0.159054554695)*x[1]**o+(-0.0306514318922)*x[1]
        arg[0,2,2,0]=(0.0903142465124)*x[0]**o+(0.872680271888)*x[0]+(0.230242338668)*x[1]**o+(0.557126778217)*x[1]
        arg[0,2,2,1]=(-0.842046475241)*x[0]**o+(0.350544067128)*x[0]+(0.486186516633)*x[1]**o+(-0.550801066658)*x[1]
        arg[0,3,0,0]=(0.740313870334)*x[0]**o+(-0.177105021967)*x[0]+(0.00114640526837)*x[1]**o+(0.349602955151)*x[1]
        arg[0,3,0,1]=(0.898906724506)*x[0]**o+(-0.882729119203)*x[0]+(0.000278645863091)*x[1]**o+(-0.450247260284)*x[1]
        arg[0,3,1,0]=(0.411917691981)*x[0]**o+(0.0461900014479)*x[0]+(0.1608400364)*x[1]**o+(0.130550626888)*x[1]
        arg[0,3,1,1]=(0.610378099059)*x[0]**o+(-0.903971943528)*x[0]+(-0.0170960427518)*x[1]**o+(0.765729630725)*x[1]
        arg[0,3,2,0]=(-0.811421756909)*x[0]**o+(-0.220065414196)*x[0]+(0.529467921065)*x[1]**o+(0.13896086667)*x[1]
        arg[0,3,2,1]=(-0.621021589728)*x[0]**o+(0.181903892108)*x[0]+(0.724296638902)*x[1]**o+(0.651705107858)*x[1]
        arg[0,4,0,0]=(0.323254788695)*x[0]**o+(0.758728125147)*x[0]+(0.0719056895797)*x[1]**o+(-0.486299017994)*x[1]
        arg[0,4,0,1]=(-0.426525215977)*x[0]**o+(-0.70344592953)*x[0]+(-0.467799544116)*x[1]**o+(0.0394017649712)*x[1]
        arg[0,4,1,0]=(-0.559984242808)*x[0]**o+(-0.963639424043)*x[0]+(0.884232194806)*x[1]**o+(0.948111505594)*x[1]
        arg[0,4,1,1]=(-0.825627656302)*x[0]**o+(0.167871540452)*x[0]+(0.0969950885663)*x[1]**o+(-0.477024385485)*x[1]
        arg[0,4,2,0]=(-0.890191205005)*x[0]**o+(-0.291891624031)*x[0]+(-0.924812934321)*x[1]**o+(-0.898567598264)*x[1]
        arg[0,4,2,1]=(0.0552241756422)*x[0]**o+(-0.47878688784)*x[0]+(-0.208893161197)*x[1]**o+(-0.671733441515)*x[1]
        arg[1,0,0,0]=(0.609961055126)*x[0]**o+(0.389225845794)*x[0]+(0.147977819654)*x[1]**o+(-0.75096024248)*x[1]
        arg[1,0,0,1]=(-0.407376527411)*x[0]**o+(-0.696660373977)*x[0]+(-0.803862178008)*x[1]**o+(-0.367087112506)*x[1]
        arg[1,0,1,0]=(0.89146134332)*x[0]**o+(0.388607608477)*x[0]+(0.763190367241)*x[1]**o+(0.169275499463)*x[1]
        arg[1,0,1,1]=(-0.870871474063)*x[0]**o+(-0.21526798243)*x[0]+(0.159168404985)*x[1]**o+(0.715049896538)*x[1]
        arg[1,0,2,0]=(-0.842978554795)*x[0]**o+(0.209649486001)*x[0]+(-0.767629750054)*x[1]**o+(-0.126634026874)*x[1]
        arg[1,0,2,1]=(-0.728571756044)*x[0]**o+(-0.247852313042)*x[0]+(0.818809803616)*x[1]**o+(0.736163163331)*x[1]
        arg[1,1,0,0]=(-0.670608139072)*x[0]**o+(0.330167259903)*x[0]+(0.275974484086)*x[1]**o+(0.492110644529)*x[1]
        arg[1,1,0,1]=(0.724664287226)*x[0]**o+(0.738855592065)*x[0]+(0.511369798951)*x[1]**o+(0.455794112842)*x[1]
        arg[1,1,1,0]=(0.783504599076)*x[0]**o+(-0.558579246854)*x[0]+(0.608044912637)*x[1]**o+(-0.401058633268)*x[1]
        arg[1,1,1,1]=(0.406697861117)*x[0]**o+(-0.31302718156)*x[0]+(-0.999553049864)*x[1]**o+(0.647698243869)*x[1]
        arg[1,1,2,0]=(-0.788346252365)*x[0]**o+(-0.809895265253)*x[0]+(0.441738551485)*x[1]**o+(-0.686785865679)*x[1]
        arg[1,1,2,1]=(0.794571346519)*x[0]**o+(-0.552071697712)*x[0]+(0.77260218173)*x[1]**o+(-0.969584549528)*x[1]
        arg[1,2,0,0]=(-0.867948229325)*x[0]**o+(-0.582387320474)*x[0]+(0.0387726756335)*x[1]**o+(0.4143770831)*x[1]
        arg[1,2,0,1]=(-0.913557875303)*x[0]**o+(-0.517970834439)*x[0]+(-0.922517499063)*x[1]**o+(0.858552054359)*x[1]
        arg[1,2,1,0]=(0.409302016181)*x[0]**o+(-0.684529177441)*x[0]+(0.818122023218)*x[1]**o+(-0.583589419623)*x[1]
        arg[1,2,1,1]=(0.750795883056)*x[0]**o+(-0.710882726624)*x[0]+(0.915210035315)*x[1]**o+(0.802588664027)*x[1]
        arg[1,2,2,0]=(0.639216566804)*x[0]**o+(-0.300954214125)*x[0]+(-0.26724393342)*x[1]**o+(0.426742851293)*x[1]
        arg[1,2,2,1]=(-0.00337538020121)*x[0]**o+(0.283012169215)*x[0]+(0.725040731001)*x[1]**o+(-0.708238265029)*x[1]
        arg[1,3,0,0]=(0.217404922432)*x[0]**o+(-0.590883003502)*x[0]+(0.451142849723)*x[1]**o+(0.624698767392)*x[1]
        arg[1,3,0,1]=(0.0351258739828)*x[0]**o+(-0.784068492686)*x[0]+(0.469956184167)*x[1]**o+(0.634124784387)*x[1]
        arg[1,3,1,0]=(0.998414775431)*x[0]**o+(0.388202946918)*x[0]+(-0.849223443255)*x[1]**o+(-0.804590028218)*x[1]
        arg[1,3,1,1]=(0.0219748851856)*x[0]**o+(0.464069403336)*x[0]+(-0.470759541095)*x[1]**o+(-0.21773573235)*x[1]
        arg[1,3,2,0]=(-0.879175905661)*x[0]**o+(0.100238280861)*x[0]+(0.331472085056)*x[1]**o+(0.0310947651268)*x[1]
        arg[1,3,2,1]=(-0.195155740038)*x[0]**o+(0.532078676231)*x[0]+(-0.863288082011)*x[1]**o+(-0.0905878788265)*x[1]
        arg[1,4,0,0]=(0.470883637487)*x[0]**o+(0.0291028245963)*x[0]+(-0.152462235474)*x[1]**o+(0.890883963301)*x[1]
        arg[1,4,0,1]=(-0.0780797534409)*x[0]**o+(-0.999649003413)*x[0]+(-0.79446765004)*x[1]**o+(-0.820520881428)*x[1]
        arg[1,4,1,0]=(-0.82479629638)*x[0]**o+(-0.28924829922)*x[0]+(-0.960430867949)*x[1]**o+(-0.514730558673)*x[1]
        arg[1,4,1,1]=(-0.0853911110391)*x[0]**o+(0.726747270178)*x[0]+(-0.70925711586)*x[1]**o+(-0.331433386035)*x[1]
        arg[1,4,2,0]=(-0.434350343402)*x[0]**o+(0.335341284415)*x[0]+(0.496818240476)*x[1]**o+(-0.925006191184)*x[1]
        arg[1,4,2,1]=(0.0753516047957)*x[0]**o+(0.171090641529)*x[0]+(-0.836002762987)*x[1]**o+(-0.753836171615)*x[1]
        arg[2,0,0,0]=(-0.331008073553)*x[0]**o+(-0.299051880715)*x[0]+(-0.84520124413)*x[1]**o+(0.780471473975)*x[1]
        arg[2,0,0,1]=(-0.947973851318)*x[0]**o+(0.579142212501)*x[0]+(0.137317760268)*x[1]**o+(-0.379509530229)*x[1]
        arg[2,0,1,0]=(-0.741320730287)*x[0]**o+(0.642110710824)*x[0]+(-0.632595944259)*x[1]**o+(0.944980046528)*x[1]
        arg[2,0,1,1]=(0.616943682269)*x[0]**o+(-0.348189507611)*x[0]+(-0.414080070092)*x[1]**o+(-0.733041081362)*x[1]
        arg[2,0,2,0]=(-0.021261659709)*x[0]**o+(0.790041129031)*x[0]+(0.788378634137)*x[1]**o+(0.391057560074)*x[1]
        arg[2,0,2,1]=(0.563737321295)*x[0]**o+(-0.110516278477)*x[0]+(0.383563652425)*x[1]**o+(-0.857100387101)*x[1]
        arg[2,1,0,0]=(-0.0839346911978)*x[0]**o+(0.699255867073)*x[0]+(-0.698488781835)*x[1]**o+(0.225940657322)*x[1]
        arg[2,1,0,1]=(0.93638601214)*x[0]**o+(-0.593516691147)*x[0]+(-0.445863239521)*x[1]**o+(0.601067357746)*x[1]
        arg[2,1,1,0]=(0.114814804735)*x[0]**o+(0.717600499991)*x[0]+(0.7371911818)*x[1]**o+(0.136012416552)*x[1]
        arg[2,1,1,1]=(0.794499830148)*x[0]**o+(0.0581947415855)*x[0]+(0.626382859976)*x[1]**o+(-0.668272098674)*x[1]
        arg[2,1,2,0]=(0.564628771073)*x[0]**o+(0.102820241267)*x[0]+(0.397960234363)*x[1]**o+(0.379632827896)*x[1]
        arg[2,1,2,1]=(-0.0880639775913)*x[0]**o+(0.325978376801)*x[0]+(-0.505498229733)*x[1]**o+(-0.595922646655)*x[1]
        arg[2,2,0,0]=(-0.132672737928)*x[0]**o+(-0.0291240195523)*x[0]+(-0.00825944211706)*x[1]**o+(0.985993963145)*x[1]
        arg[2,2,0,1]=(0.244321497515)*x[0]**o+(-0.111329005236)*x[0]+(0.769353471123)*x[1]**o+(-0.174257027136)*x[1]
        arg[2,2,1,0]=(0.252286330282)*x[0]**o+(0.450975615825)*x[0]+(-0.515016578606)*x[1]**o+(0.834736757555)*x[1]
        arg[2,2,1,1]=(0.0588273338962)*x[0]**o+(-0.865406338788)*x[0]+(0.482095645149)*x[1]**o+(-0.595126490156)*x[1]
        arg[2,2,2,0]=(0.244580063555)*x[0]**o+(0.576709884892)*x[0]+(-0.212616562122)*x[1]**o+(-0.407508050723)*x[1]
        arg[2,2,2,1]=(0.416825783755)*x[0]**o+(0.430959078988)*x[0]+(-0.398840301481)*x[1]**o+(0.889105964295)*x[1]
        arg[2,3,0,0]=(-0.731872136283)*x[0]**o+(0.237034931745)*x[0]+(0.547737117698)*x[1]**o+(0.364733861661)*x[1]
        arg[2,3,0,1]=(0.971716549689)*x[0]**o+(0.466300970081)*x[0]+(-0.51802162747)*x[1]**o+(-0.859910778018)*x[1]
        arg[2,3,1,0]=(-0.562918219893)*x[0]**o+(-0.0532549959391)*x[0]+(0.842033495359)*x[1]**o+(-0.708554283815)*x[1]
        arg[2,3,1,1]=(0.437125534104)*x[0]**o+(-0.376751615116)*x[0]+(-0.330889383623)*x[1]**o+(0.958884035289)*x[1]
        arg[2,3,2,0]=(-0.194290501566)*x[0]**o+(-0.368131904292)*x[0]+(0.802917650409)*x[1]**o+(0.628603987906)*x[1]
        arg[2,3,2,1]=(0.70219875881)*x[0]**o+(0.822181339005)*x[0]+(0.768635447234)*x[1]**o+(0.871676010439)*x[1]
        arg[2,4,0,0]=(-0.654498700662)*x[0]**o+(0.851143676147)*x[0]+(0.468468395415)*x[1]**o+(-0.447240713576)*x[1]
        arg[2,4,0,1]=(0.893177052112)*x[0]**o+(0.652743480988)*x[0]+(0.726646163682)*x[1]**o+(-0.854895731892)*x[1]
        arg[2,4,1,0]=(0.0540542301334)*x[0]**o+(-0.208654443984)*x[0]+(0.925127576853)*x[1]**o+(-0.932489814868)*x[1]
        arg[2,4,1,1]=(-0.460268214104)*x[0]**o+(-0.303569308977)*x[0]+(-0.381458880316)*x[1]**o+(0.690537661673)*x[1]
        arg[2,4,2,0]=(0.401012366366)*x[0]**o+(-0.970993441679)*x[0]+(0.0249535421254)*x[1]**o+(-0.499737254014)*x[1]
        arg[2,4,2,1]=(-0.692604790859)*x[0]**o+(-0.685292098725)*x[0]+(0.22550357354)*x[1]**o+(0.908152783462)*x[1]
        arg[3,0,0,0]=(-0.634246407254)*x[0]**o+(-0.571223033336)*x[0]+(0.0574319920181)*x[1]**o+(0.123763929908)*x[1]
        arg[3,0,0,1]=(-0.569358219213)*x[0]**o+(-0.107237509632)*x[0]+(0.97672999497)*x[1]**o+(-0.114939366813)*x[1]
        arg[3,0,1,0]=(-0.767885152479)*x[0]**o+(-0.105924102555)*x[0]+(0.539361607398)*x[1]**o+(0.546186446469)*x[1]
        arg[3,0,1,1]=(-0.58818696897)*x[0]**o+(0.629371516566)*x[0]+(-0.598173904376)*x[1]**o+(0.959043112088)*x[1]
        arg[3,0,2,0]=(0.751165393927)*x[0]**o+(0.732151619397)*x[0]+(-0.435276735735)*x[1]**o+(0.947142140722)*x[1]
        arg[3,0,2,1]=(0.463904349547)*x[0]**o+(-0.947270841014)*x[0]+(0.304239969784)*x[1]**o+(-0.233566944292)*x[1]
        arg[3,1,0,0]=(0.961122232395)*x[0]**o+(-0.989040215156)*x[0]+(0.833186746232)*x[1]**o+(0.226673105305)*x[1]
        arg[3,1,0,1]=(-0.156957616578)*x[0]**o+(-0.361665587314)*x[0]+(0.987490273947)*x[1]**o+(-0.890271058746)*x[1]
        arg[3,1,1,0]=(-0.259959222953)*x[0]**o+(-0.696135598749)*x[0]+(0.234411151165)*x[1]**o+(0.783105897256)*x[1]
        arg[3,1,1,1]=(0.020886322199)*x[0]**o+(0.887445555864)*x[0]+(0.709489180254)*x[1]**o+(0.990590112507)*x[1]
        arg[3,1,2,0]=(-0.963790247866)*x[0]**o+(-0.417180971158)*x[0]+(0.303859261646)*x[1]**o+(0.745552024399)*x[1]
        arg[3,1,2,1]=(-0.364448135641)*x[0]**o+(0.760841571414)*x[0]+(-0.0283374186763)*x[1]**o+(-0.491433855748)*x[1]
        arg[3,2,0,0]=(-0.649766142291)*x[0]**o+(-0.177410238297)*x[0]+(0.137316641289)*x[1]**o+(-0.40890299152)*x[1]
        arg[3,2,0,1]=(0.282922984657)*x[0]**o+(-0.660238152943)*x[0]+(0.481726052488)*x[1]**o+(0.976011254218)*x[1]
        arg[3,2,1,0]=(0.989761520473)*x[0]**o+(0.194082255647)*x[0]+(-0.648435682551)*x[1]**o+(-0.721569599311)*x[1]
        arg[3,2,1,1]=(0.734634540883)*x[0]**o+(0.524638529049)*x[0]+(-0.737636791717)*x[1]**o+(-0.995904525822)*x[1]
        arg[3,2,2,0]=(0.422396808663)*x[0]**o+(-0.820639154105)*x[0]+(-0.566207315373)*x[1]**o+(-0.224174711984)*x[1]
        arg[3,2,2,1]=(0.719145374997)*x[0]**o+(-0.885860499468)*x[0]+(0.184062878609)*x[1]**o+(0.47660543007)*x[1]
        arg[3,3,0,0]=(-0.0122290849915)*x[0]**o+(-0.0314298339876)*x[0]+(0.937255407134)*x[1]**o+(-0.754858547072)*x[1]
        arg[3,3,0,1]=(-0.507826956669)*x[0]**o+(0.951205020689)*x[0]+(0.837096245109)*x[1]**o+(-0.576692399518)*x[1]
        arg[3,3,1,0]=(0.602119808238)*x[0]**o+(-0.943937134298)*x[0]+(-0.244250254626)*x[1]**o+(-0.814103542727)*x[1]
        arg[3,3,1,1]=(-0.51811661822)*x[0]**o+(-0.188969078762)*x[0]+(-0.697470816837)*x[1]**o+(-0.741593014841)*x[1]
        arg[3,3,2,0]=(-0.673948933786)*x[0]**o+(0.816404336566)*x[0]+(0.339239090092)*x[1]**o+(-0.0173550947113)*x[1]
        arg[3,3,2,1]=(0.499109852716)*x[0]**o+(0.515501552893)*x[0]+(0.926442939974)*x[1]**o+(0.157227173254)*x[1]
        arg[3,4,0,0]=(-0.1837389273)*x[0]**o+(0.78490229721)*x[0]+(-0.248033788374)*x[1]**o+(0.735136386405)*x[1]
        arg[3,4,0,1]=(0.121169550194)*x[0]**o+(-0.854027732077)*x[0]+(0.937680848475)*x[1]**o+(-0.447184197264)*x[1]
        arg[3,4,1,0]=(-0.788610934694)*x[0]**o+(-0.397825388144)*x[0]+(0.604421361449)*x[1]**o+(0.937184552821)*x[1]
        arg[3,4,1,1]=(-0.493380143496)*x[0]**o+(0.958197000398)*x[0]+(0.282372561751)*x[1]**o+(-0.55322669826)*x[1]
        arg[3,4,2,0]=(0.11333865923)*x[0]**o+(-0.542805511569)*x[0]+(-0.640646701928)*x[1]**o+(-0.342030716597)*x[1]
        arg[3,4,2,1]=(0.505356266297)*x[0]**o+(-0.107422782294)*x[0]+(0.542833447968)*x[1]**o+(-0.615275716592)*x[1]
        ref[0,0,0,0]=(0.922177773381)*x_ref[0]**o+(0.37324175781)*x_ref[0]+(-0.169208776011)*x_ref[1]**o+(0.72134249236)*x_ref[1]
        ref[0,0,0,1]=(0.85312270129)*x_ref[0]**o+(0.0512919651836)*x_ref[0]+(0.968880249675)*x_ref[1]**o+(0.740058819096)*x_ref[1]
        ref[0,0,1,0]=(-0.018205657901)*x_ref[0]**o+(0.601838710779)*x_ref[0]+(0.211002282731)*x_ref[1]**o+(0.801382123792)*x_ref[1]
        ref[0,0,1,1]=(-0.249128714489)*x_ref[0]**o+(-0.408809542089)*x_ref[0]+(-0.684434920261)*x_ref[1]**o+(0.223674849818)*x_ref[1]
        ref[0,0,2,0]=(0.396627551183)*x_ref[0]**o+(-0.223592778052)*x_ref[0]+(0.227333834259)*x_ref[1]**o+(0.66580805616)*x_ref[1]
        ref[0,0,2,1]=(0.365216692731)*x_ref[0]**o+(-0.670947594503)*x_ref[0]+(0.400260365509)*x_ref[1]**o+(0.10903299253)*x_ref[1]
        ref[0,1,0,0]=(0.823297765985)*x_ref[0]**o+(0.60337946486)*x_ref[0]+(-0.314265083693)*x_ref[1]**o+(-0.00636816227929)*x_ref[1]
        ref[0,1,0,1]=(-0.26216625982)*x_ref[0]**o+(0.767412606588)*x_ref[0]+(-0.284917238441)*x_ref[1]**o+(0.56385025143)*x_ref[1]
        ref[0,1,1,0]=(-0.732326448281)*x_ref[0]**o+(-0.669197323856)*x_ref[0]+(-0.608751752289)*x_ref[1]**o+(-0.853458985188)*x_ref[1]
        ref[0,1,1,1]=(0.637275594334)*x_ref[0]**o+(0.696570850488)*x_ref[0]+(0.012026820657)*x_ref[1]**o+(0.558317439186)*x_ref[1]
        ref[0,1,2,0]=(0.925899092402)*x_ref[0]**o+(0.568045453255)*x_ref[0]+(-0.766701931715)*x_ref[1]**o+(-0.95082586036)*x_ref[1]
        ref[0,1,2,1]=(0.127524211574)*x_ref[0]**o+(-0.65150542164)*x_ref[0]+(-0.365773940068)*x_ref[1]**o+(0.990465711754)*x_ref[1]
        ref[0,2,0,0]=(0.456469052217)*x_ref[0]**o+(0.403391798397)*x_ref[0]+(-0.21921915949)*x_ref[1]**o+(0.171493695945)*x_ref[1]
        ref[0,2,0,1]=(-0.980242506743)*x_ref[0]**o+(0.880225959645)*x_ref[0]+(0.682653186307)*x_ref[1]**o+(-0.561867916377)*x_ref[1]
        ref[0,2,1,0]=(-0.329881998938)*x_ref[0]**o+(0.464976480407)*x_ref[0]+(-0.812583693347)*x_ref[1]**o+(0.0103356333568)*x_ref[1]
        ref[0,2,1,1]=(-0.710350374606)*x_ref[0]**o+(-0.445981415127)*x_ref[0]+(-0.159054554695)*x_ref[1]**o+(-0.0306514318922)*x_ref[1]
        ref[0,2,2,0]=(0.0903142465124)*x_ref[0]**o+(0.872680271888)*x_ref[0]+(0.230242338668)*x_ref[1]**o+(0.557126778217)*x_ref[1]
        ref[0,2,2,1]=(-0.842046475241)*x_ref[0]**o+(0.350544067128)*x_ref[0]+(0.486186516633)*x_ref[1]**o+(-0.550801066658)*x_ref[1]
        ref[0,3,0,0]=(0.740313870334)*x_ref[0]**o+(-0.177105021967)*x_ref[0]+(0.00114640526837)*x_ref[1]**o+(0.349602955151)*x_ref[1]
        ref[0,3,0,1]=(0.898906724506)*x_ref[0]**o+(-0.882729119203)*x_ref[0]+(0.000278645863091)*x_ref[1]**o+(-0.450247260284)*x_ref[1]
        ref[0,3,1,0]=(0.411917691981)*x_ref[0]**o+(0.0461900014479)*x_ref[0]+(0.1608400364)*x_ref[1]**o+(0.130550626888)*x_ref[1]
        ref[0,3,1,1]=(0.610378099059)*x_ref[0]**o+(-0.903971943528)*x_ref[0]+(-0.0170960427518)*x_ref[1]**o+(0.765729630725)*x_ref[1]
        ref[0,3,2,0]=(-0.811421756909)*x_ref[0]**o+(-0.220065414196)*x_ref[0]+(0.529467921065)*x_ref[1]**o+(0.13896086667)*x_ref[1]
        ref[0,3,2,1]=(-0.621021589728)*x_ref[0]**o+(0.181903892108)*x_ref[0]+(0.724296638902)*x_ref[1]**o+(0.651705107858)*x_ref[1]
        ref[0,4,0,0]=(0.323254788695)*x_ref[0]**o+(0.758728125147)*x_ref[0]+(0.0719056895797)*x_ref[1]**o+(-0.486299017994)*x_ref[1]
        ref[0,4,0,1]=(-0.426525215977)*x_ref[0]**o+(-0.70344592953)*x_ref[0]+(-0.467799544116)*x_ref[1]**o+(0.0394017649712)*x_ref[1]
        ref[0,4,1,0]=(-0.559984242808)*x_ref[0]**o+(-0.963639424043)*x_ref[0]+(0.884232194806)*x_ref[1]**o+(0.948111505594)*x_ref[1]
        ref[0,4,1,1]=(-0.825627656302)*x_ref[0]**o+(0.167871540452)*x_ref[0]+(0.0969950885663)*x_ref[1]**o+(-0.477024385485)*x_ref[1]
        ref[0,4,2,0]=(-0.890191205005)*x_ref[0]**o+(-0.291891624031)*x_ref[0]+(-0.924812934321)*x_ref[1]**o+(-0.898567598264)*x_ref[1]
        ref[0,4,2,1]=(0.0552241756422)*x_ref[0]**o+(-0.47878688784)*x_ref[0]+(-0.208893161197)*x_ref[1]**o+(-0.671733441515)*x_ref[1]
        ref[1,0,0,0]=(0.609961055126)*x_ref[0]**o+(0.389225845794)*x_ref[0]+(0.147977819654)*x_ref[1]**o+(-0.75096024248)*x_ref[1]
        ref[1,0,0,1]=(-0.407376527411)*x_ref[0]**o+(-0.696660373977)*x_ref[0]+(-0.803862178008)*x_ref[1]**o+(-0.367087112506)*x_ref[1]
        ref[1,0,1,0]=(0.89146134332)*x_ref[0]**o+(0.388607608477)*x_ref[0]+(0.763190367241)*x_ref[1]**o+(0.169275499463)*x_ref[1]
        ref[1,0,1,1]=(-0.870871474063)*x_ref[0]**o+(-0.21526798243)*x_ref[0]+(0.159168404985)*x_ref[1]**o+(0.715049896538)*x_ref[1]
        ref[1,0,2,0]=(-0.842978554795)*x_ref[0]**o+(0.209649486001)*x_ref[0]+(-0.767629750054)*x_ref[1]**o+(-0.126634026874)*x_ref[1]
        ref[1,0,2,1]=(-0.728571756044)*x_ref[0]**o+(-0.247852313042)*x_ref[0]+(0.818809803616)*x_ref[1]**o+(0.736163163331)*x_ref[1]
        ref[1,1,0,0]=(-0.670608139072)*x_ref[0]**o+(0.330167259903)*x_ref[0]+(0.275974484086)*x_ref[1]**o+(0.492110644529)*x_ref[1]
        ref[1,1,0,1]=(0.724664287226)*x_ref[0]**o+(0.738855592065)*x_ref[0]+(0.511369798951)*x_ref[1]**o+(0.455794112842)*x_ref[1]
        ref[1,1,1,0]=(0.783504599076)*x_ref[0]**o+(-0.558579246854)*x_ref[0]+(0.608044912637)*x_ref[1]**o+(-0.401058633268)*x_ref[1]
        ref[1,1,1,1]=(0.406697861117)*x_ref[0]**o+(-0.31302718156)*x_ref[0]+(-0.999553049864)*x_ref[1]**o+(0.647698243869)*x_ref[1]
        ref[1,1,2,0]=(-0.788346252365)*x_ref[0]**o+(-0.809895265253)*x_ref[0]+(0.441738551485)*x_ref[1]**o+(-0.686785865679)*x_ref[1]
        ref[1,1,2,1]=(0.794571346519)*x_ref[0]**o+(-0.552071697712)*x_ref[0]+(0.77260218173)*x_ref[1]**o+(-0.969584549528)*x_ref[1]
        ref[1,2,0,0]=(-0.867948229325)*x_ref[0]**o+(-0.582387320474)*x_ref[0]+(0.0387726756335)*x_ref[1]**o+(0.4143770831)*x_ref[1]
        ref[1,2,0,1]=(-0.913557875303)*x_ref[0]**o+(-0.517970834439)*x_ref[0]+(-0.922517499063)*x_ref[1]**o+(0.858552054359)*x_ref[1]
        ref[1,2,1,0]=(0.409302016181)*x_ref[0]**o+(-0.684529177441)*x_ref[0]+(0.818122023218)*x_ref[1]**o+(-0.583589419623)*x_ref[1]
        ref[1,2,1,1]=(0.750795883056)*x_ref[0]**o+(-0.710882726624)*x_ref[0]+(0.915210035315)*x_ref[1]**o+(0.802588664027)*x_ref[1]
        ref[1,2,2,0]=(0.639216566804)*x_ref[0]**o+(-0.300954214125)*x_ref[0]+(-0.26724393342)*x_ref[1]**o+(0.426742851293)*x_ref[1]
        ref[1,2,2,1]=(-0.00337538020121)*x_ref[0]**o+(0.283012169215)*x_ref[0]+(0.725040731001)*x_ref[1]**o+(-0.708238265029)*x_ref[1]
        ref[1,3,0,0]=(0.217404922432)*x_ref[0]**o+(-0.590883003502)*x_ref[0]+(0.451142849723)*x_ref[1]**o+(0.624698767392)*x_ref[1]
        ref[1,3,0,1]=(0.0351258739828)*x_ref[0]**o+(-0.784068492686)*x_ref[0]+(0.469956184167)*x_ref[1]**o+(0.634124784387)*x_ref[1]
        ref[1,3,1,0]=(0.998414775431)*x_ref[0]**o+(0.388202946918)*x_ref[0]+(-0.849223443255)*x_ref[1]**o+(-0.804590028218)*x_ref[1]
        ref[1,3,1,1]=(0.0219748851856)*x_ref[0]**o+(0.464069403336)*x_ref[0]+(-0.470759541095)*x_ref[1]**o+(-0.21773573235)*x_ref[1]
        ref[1,3,2,0]=(-0.879175905661)*x_ref[0]**o+(0.100238280861)*x_ref[0]+(0.331472085056)*x_ref[1]**o+(0.0310947651268)*x_ref[1]
        ref[1,3,2,1]=(-0.195155740038)*x_ref[0]**o+(0.532078676231)*x_ref[0]+(-0.863288082011)*x_ref[1]**o+(-0.0905878788265)*x_ref[1]
        ref[1,4,0,0]=(0.470883637487)*x_ref[0]**o+(0.0291028245963)*x_ref[0]+(-0.152462235474)*x_ref[1]**o+(0.890883963301)*x_ref[1]
        ref[1,4,0,1]=(-0.0780797534409)*x_ref[0]**o+(-0.999649003413)*x_ref[0]+(-0.79446765004)*x_ref[1]**o+(-0.820520881428)*x_ref[1]
        ref[1,4,1,0]=(-0.82479629638)*x_ref[0]**o+(-0.28924829922)*x_ref[0]+(-0.960430867949)*x_ref[1]**o+(-0.514730558673)*x_ref[1]
        ref[1,4,1,1]=(-0.0853911110391)*x_ref[0]**o+(0.726747270178)*x_ref[0]+(-0.70925711586)*x_ref[1]**o+(-0.331433386035)*x_ref[1]
        ref[1,4,2,0]=(-0.434350343402)*x_ref[0]**o+(0.335341284415)*x_ref[0]+(0.496818240476)*x_ref[1]**o+(-0.925006191184)*x_ref[1]
        ref[1,4,2,1]=(0.0753516047957)*x_ref[0]**o+(0.171090641529)*x_ref[0]+(-0.836002762987)*x_ref[1]**o+(-0.753836171615)*x_ref[1]
        ref[2,0,0,0]=(-0.331008073553)*x_ref[0]**o+(-0.299051880715)*x_ref[0]+(-0.84520124413)*x_ref[1]**o+(0.780471473975)*x_ref[1]
        ref[2,0,0,1]=(-0.947973851318)*x_ref[0]**o+(0.579142212501)*x_ref[0]+(0.137317760268)*x_ref[1]**o+(-0.379509530229)*x_ref[1]
        ref[2,0,1,0]=(-0.741320730287)*x_ref[0]**o+(0.642110710824)*x_ref[0]+(-0.632595944259)*x_ref[1]**o+(0.944980046528)*x_ref[1]
        ref[2,0,1,1]=(0.616943682269)*x_ref[0]**o+(-0.348189507611)*x_ref[0]+(-0.414080070092)*x_ref[1]**o+(-0.733041081362)*x_ref[1]
        ref[2,0,2,0]=(-0.021261659709)*x_ref[0]**o+(0.790041129031)*x_ref[0]+(0.788378634137)*x_ref[1]**o+(0.391057560074)*x_ref[1]
        ref[2,0,2,1]=(0.563737321295)*x_ref[0]**o+(-0.110516278477)*x_ref[0]+(0.383563652425)*x_ref[1]**o+(-0.857100387101)*x_ref[1]
        ref[2,1,0,0]=(-0.0839346911978)*x_ref[0]**o+(0.699255867073)*x_ref[0]+(-0.698488781835)*x_ref[1]**o+(0.225940657322)*x_ref[1]
        ref[2,1,0,1]=(0.93638601214)*x_ref[0]**o+(-0.593516691147)*x_ref[0]+(-0.445863239521)*x_ref[1]**o+(0.601067357746)*x_ref[1]
        ref[2,1,1,0]=(0.114814804735)*x_ref[0]**o+(0.717600499991)*x_ref[0]+(0.7371911818)*x_ref[1]**o+(0.136012416552)*x_ref[1]
        ref[2,1,1,1]=(0.794499830148)*x_ref[0]**o+(0.0581947415855)*x_ref[0]+(0.626382859976)*x_ref[1]**o+(-0.668272098674)*x_ref[1]
        ref[2,1,2,0]=(0.564628771073)*x_ref[0]**o+(0.102820241267)*x_ref[0]+(0.397960234363)*x_ref[1]**o+(0.379632827896)*x_ref[1]
        ref[2,1,2,1]=(-0.0880639775913)*x_ref[0]**o+(0.325978376801)*x_ref[0]+(-0.505498229733)*x_ref[1]**o+(-0.595922646655)*x_ref[1]
        ref[2,2,0,0]=(-0.132672737928)*x_ref[0]**o+(-0.0291240195523)*x_ref[0]+(-0.00825944211706)*x_ref[1]**o+(0.985993963145)*x_ref[1]
        ref[2,2,0,1]=(0.244321497515)*x_ref[0]**o+(-0.111329005236)*x_ref[0]+(0.769353471123)*x_ref[1]**o+(-0.174257027136)*x_ref[1]
        ref[2,2,1,0]=(0.252286330282)*x_ref[0]**o+(0.450975615825)*x_ref[0]+(-0.515016578606)*x_ref[1]**o+(0.834736757555)*x_ref[1]
        ref[2,2,1,1]=(0.0588273338962)*x_ref[0]**o+(-0.865406338788)*x_ref[0]+(0.482095645149)*x_ref[1]**o+(-0.595126490156)*x_ref[1]
        ref[2,2,2,0]=(0.244580063555)*x_ref[0]**o+(0.576709884892)*x_ref[0]+(-0.212616562122)*x_ref[1]**o+(-0.407508050723)*x_ref[1]
        ref[2,2,2,1]=(0.416825783755)*x_ref[0]**o+(0.430959078988)*x_ref[0]+(-0.398840301481)*x_ref[1]**o+(0.889105964295)*x_ref[1]
        ref[2,3,0,0]=(-0.731872136283)*x_ref[0]**o+(0.237034931745)*x_ref[0]+(0.547737117698)*x_ref[1]**o+(0.364733861661)*x_ref[1]
        ref[2,3,0,1]=(0.971716549689)*x_ref[0]**o+(0.466300970081)*x_ref[0]+(-0.51802162747)*x_ref[1]**o+(-0.859910778018)*x_ref[1]
        ref[2,3,1,0]=(-0.562918219893)*x_ref[0]**o+(-0.0532549959391)*x_ref[0]+(0.842033495359)*x_ref[1]**o+(-0.708554283815)*x_ref[1]
        ref[2,3,1,1]=(0.437125534104)*x_ref[0]**o+(-0.376751615116)*x_ref[0]+(-0.330889383623)*x_ref[1]**o+(0.958884035289)*x_ref[1]
        ref[2,3,2,0]=(-0.194290501566)*x_ref[0]**o+(-0.368131904292)*x_ref[0]+(0.802917650409)*x_ref[1]**o+(0.628603987906)*x_ref[1]
        ref[2,3,2,1]=(0.70219875881)*x_ref[0]**o+(0.822181339005)*x_ref[0]+(0.768635447234)*x_ref[1]**o+(0.871676010439)*x_ref[1]
        ref[2,4,0,0]=(-0.654498700662)*x_ref[0]**o+(0.851143676147)*x_ref[0]+(0.468468395415)*x_ref[1]**o+(-0.447240713576)*x_ref[1]
        ref[2,4,0,1]=(0.893177052112)*x_ref[0]**o+(0.652743480988)*x_ref[0]+(0.726646163682)*x_ref[1]**o+(-0.854895731892)*x_ref[1]
        ref[2,4,1,0]=(0.0540542301334)*x_ref[0]**o+(-0.208654443984)*x_ref[0]+(0.925127576853)*x_ref[1]**o+(-0.932489814868)*x_ref[1]
        ref[2,4,1,1]=(-0.460268214104)*x_ref[0]**o+(-0.303569308977)*x_ref[0]+(-0.381458880316)*x_ref[1]**o+(0.690537661673)*x_ref[1]
        ref[2,4,2,0]=(0.401012366366)*x_ref[0]**o+(-0.970993441679)*x_ref[0]+(0.0249535421254)*x_ref[1]**o+(-0.499737254014)*x_ref[1]
        ref[2,4,2,1]=(-0.692604790859)*x_ref[0]**o+(-0.685292098725)*x_ref[0]+(0.22550357354)*x_ref[1]**o+(0.908152783462)*x_ref[1]
        ref[3,0,0,0]=(-0.634246407254)*x_ref[0]**o+(-0.571223033336)*x_ref[0]+(0.0574319920181)*x_ref[1]**o+(0.123763929908)*x_ref[1]
        ref[3,0,0,1]=(-0.569358219213)*x_ref[0]**o+(-0.107237509632)*x_ref[0]+(0.97672999497)*x_ref[1]**o+(-0.114939366813)*x_ref[1]
        ref[3,0,1,0]=(-0.767885152479)*x_ref[0]**o+(-0.105924102555)*x_ref[0]+(0.539361607398)*x_ref[1]**o+(0.546186446469)*x_ref[1]
        ref[3,0,1,1]=(-0.58818696897)*x_ref[0]**o+(0.629371516566)*x_ref[0]+(-0.598173904376)*x_ref[1]**o+(0.959043112088)*x_ref[1]
        ref[3,0,2,0]=(0.751165393927)*x_ref[0]**o+(0.732151619397)*x_ref[0]+(-0.435276735735)*x_ref[1]**o+(0.947142140722)*x_ref[1]
        ref[3,0,2,1]=(0.463904349547)*x_ref[0]**o+(-0.947270841014)*x_ref[0]+(0.304239969784)*x_ref[1]**o+(-0.233566944292)*x_ref[1]
        ref[3,1,0,0]=(0.961122232395)*x_ref[0]**o+(-0.989040215156)*x_ref[0]+(0.833186746232)*x_ref[1]**o+(0.226673105305)*x_ref[1]
        ref[3,1,0,1]=(-0.156957616578)*x_ref[0]**o+(-0.361665587314)*x_ref[0]+(0.987490273947)*x_ref[1]**o+(-0.890271058746)*x_ref[1]
        ref[3,1,1,0]=(-0.259959222953)*x_ref[0]**o+(-0.696135598749)*x_ref[0]+(0.234411151165)*x_ref[1]**o+(0.783105897256)*x_ref[1]
        ref[3,1,1,1]=(0.020886322199)*x_ref[0]**o+(0.887445555864)*x_ref[0]+(0.709489180254)*x_ref[1]**o+(0.990590112507)*x_ref[1]
        ref[3,1,2,0]=(-0.963790247866)*x_ref[0]**o+(-0.417180971158)*x_ref[0]+(0.303859261646)*x_ref[1]**o+(0.745552024399)*x_ref[1]
        ref[3,1,2,1]=(-0.364448135641)*x_ref[0]**o+(0.760841571414)*x_ref[0]+(-0.0283374186763)*x_ref[1]**o+(-0.491433855748)*x_ref[1]
        ref[3,2,0,0]=(-0.649766142291)*x_ref[0]**o+(-0.177410238297)*x_ref[0]+(0.137316641289)*x_ref[1]**o+(-0.40890299152)*x_ref[1]
        ref[3,2,0,1]=(0.282922984657)*x_ref[0]**o+(-0.660238152943)*x_ref[0]+(0.481726052488)*x_ref[1]**o+(0.976011254218)*x_ref[1]
        ref[3,2,1,0]=(0.989761520473)*x_ref[0]**o+(0.194082255647)*x_ref[0]+(-0.648435682551)*x_ref[1]**o+(-0.721569599311)*x_ref[1]
        ref[3,2,1,1]=(0.734634540883)*x_ref[0]**o+(0.524638529049)*x_ref[0]+(-0.737636791717)*x_ref[1]**o+(-0.995904525822)*x_ref[1]
        ref[3,2,2,0]=(0.422396808663)*x_ref[0]**o+(-0.820639154105)*x_ref[0]+(-0.566207315373)*x_ref[1]**o+(-0.224174711984)*x_ref[1]
        ref[3,2,2,1]=(0.719145374997)*x_ref[0]**o+(-0.885860499468)*x_ref[0]+(0.184062878609)*x_ref[1]**o+(0.47660543007)*x_ref[1]
        ref[3,3,0,0]=(-0.0122290849915)*x_ref[0]**o+(-0.0314298339876)*x_ref[0]+(0.937255407134)*x_ref[1]**o+(-0.754858547072)*x_ref[1]
        ref[3,3,0,1]=(-0.507826956669)*x_ref[0]**o+(0.951205020689)*x_ref[0]+(0.837096245109)*x_ref[1]**o+(-0.576692399518)*x_ref[1]
        ref[3,3,1,0]=(0.602119808238)*x_ref[0]**o+(-0.943937134298)*x_ref[0]+(-0.244250254626)*x_ref[1]**o+(-0.814103542727)*x_ref[1]
        ref[3,3,1,1]=(-0.51811661822)*x_ref[0]**o+(-0.188969078762)*x_ref[0]+(-0.697470816837)*x_ref[1]**o+(-0.741593014841)*x_ref[1]
        ref[3,3,2,0]=(-0.673948933786)*x_ref[0]**o+(0.816404336566)*x_ref[0]+(0.339239090092)*x_ref[1]**o+(-0.0173550947113)*x_ref[1]
        ref[3,3,2,1]=(0.499109852716)*x_ref[0]**o+(0.515501552893)*x_ref[0]+(0.926442939974)*x_ref[1]**o+(0.157227173254)*x_ref[1]
        ref[3,4,0,0]=(-0.1837389273)*x_ref[0]**o+(0.78490229721)*x_ref[0]+(-0.248033788374)*x_ref[1]**o+(0.735136386405)*x_ref[1]
        ref[3,4,0,1]=(0.121169550194)*x_ref[0]**o+(-0.854027732077)*x_ref[0]+(0.937680848475)*x_ref[1]**o+(-0.447184197264)*x_ref[1]
        ref[3,4,1,0]=(-0.788610934694)*x_ref[0]**o+(-0.397825388144)*x_ref[0]+(0.604421361449)*x_ref[1]**o+(0.937184552821)*x_ref[1]
        ref[3,4,1,1]=(-0.493380143496)*x_ref[0]**o+(0.958197000398)*x_ref[0]+(0.282372561751)*x_ref[1]**o+(-0.55322669826)*x_ref[1]
        ref[3,4,2,0]=(0.11333865923)*x_ref[0]**o+(-0.542805511569)*x_ref[0]+(-0.640646701928)*x_ref[1]**o+(-0.342030716597)*x_ref[1]
        ref[3,4,2,1]=(0.505356266297)*x_ref[0]**o+(-0.107422782294)*x_ref[0]+(0.542833447968)*x_ref[1]**o+(-0.615275716592)*x_ref[1]
      else:
        arg[0,0,0,0]=(0.507422888369)*x[0]**o+(0.808798513305)*x[0]+(0.51493073805)*x[1]**o+(-0.928527686947)*x[1]+(-0.788010447498)*x[2]**o+(-0.378389833131)*x[2]
        arg[0,0,0,1]=(-0.475851243177)*x[0]**o+(0.746578446988)*x[0]+(0.504778622494)*x[1]**o+(-0.677770398899)*x[1]+(-0.862535648403)*x[2]**o+(0.620414141691)*x[2]
        arg[0,0,1,0]=(-0.361726879582)*x[0]**o+(0.554584127587)*x[0]+(0.436610414772)*x[1]**o+(-0.686534492717)*x[1]+(0.757226061419)*x[2]**o+(0.0948537725389)*x[2]
        arg[0,0,1,1]=(-0.362160696355)*x[0]**o+(0.766524051703)*x[0]+(0.209980857795)*x[1]**o+(-0.478398061437)*x[1]+(-0.410960179122)*x[2]**o+(-0.213272138462)*x[2]
        arg[0,0,2,0]=(0.87154872554)*x[0]**o+(-0.727915301904)*x[0]+(0.683580698056)*x[1]**o+(0.215052521277)*x[1]+(0.267231507898)*x[2]**o+(-0.0124189912449)*x[2]
        arg[0,0,2,1]=(0.631400944876)*x[0]**o+(0.537293628606)*x[0]+(0.944754350994)*x[1]**o+(0.108068908012)*x[1]+(0.890117096365)*x[2]**o+(0.164706867235)*x[2]
        arg[0,1,0,0]=(-0.935871759207)*x[0]**o+(0.350133758077)*x[0]+(-0.966482181015)*x[1]**o+(0.440902701528)*x[1]+(-0.893329541322)*x[2]**o+(0.350317827938)*x[2]
        arg[0,1,0,1]=(0.465305203601)*x[0]**o+(0.711131797928)*x[0]+(0.104091536498)*x[1]**o+(0.484811628127)*x[1]+(0.676878954516)*x[2]**o+(-0.99327016995)*x[2]
        arg[0,1,1,0]=(-0.361189016151)*x[0]**o+(0.962468573492)*x[0]+(-0.158571122564)*x[1]**o+(-0.293583660912)*x[1]+(-0.713699173608)*x[2]**o+(-0.507430163361)*x[2]
        arg[0,1,1,1]=(0.172564527054)*x[0]**o+(0.273438145027)*x[0]+(0.137378266239)*x[1]**o+(-0.675788606841)*x[1]+(-0.362143416121)*x[2]**o+(0.0703897749775)*x[2]
        arg[0,1,2,0]=(-0.941374717271)*x[0]**o+(0.289433165458)*x[0]+(0.318615227028)*x[1]**o+(0.524362568204)*x[1]+(-0.612590305095)*x[2]**o+(-0.117555538957)*x[2]
        arg[0,1,2,1]=(-0.442920825551)*x[0]**o+(-0.454982146579)*x[0]+(0.355353737589)*x[1]**o+(-0.729623917004)*x[1]+(-0.567220335937)*x[2]**o+(0.977652639976)*x[2]
        arg[0,2,0,0]=(0.942485930373)*x[0]**o+(-0.605823961033)*x[0]+(-0.143465779338)*x[1]**o+(0.327354308987)*x[1]+(0.691900417542)*x[2]**o+(-0.514165213849)*x[2]
        arg[0,2,0,1]=(0.018029272977)*x[0]**o+(-0.209992705469)*x[0]+(-0.224614727554)*x[1]**o+(-0.862697462048)*x[1]+(0.274791387928)*x[2]**o+(-0.907445869506)*x[2]
        arg[0,2,1,0]=(0.862755248265)*x[0]**o+(0.020831641939)*x[0]+(0.496662599261)*x[1]**o+(0.79318975953)*x[1]+(0.0912166895541)*x[2]**o+(0.556337610316)*x[2]
        arg[0,2,1,1]=(-0.09652997071)*x[0]**o+(-0.954279421394)*x[0]+(-0.601053524018)*x[1]**o+(0.642528012068)*x[1]+(0.295748113888)*x[2]**o+(-0.318166346186)*x[2]
        arg[0,2,2,0]=(-0.914878854129)*x[0]**o+(0.159201123642)*x[0]+(0.589819056368)*x[1]**o+(-0.715763226823)*x[1]+(-0.444572583567)*x[2]**o+(-0.487866651367)*x[2]
        arg[0,2,2,1]=(-0.71271969175)*x[0]**o+(0.454616308594)*x[0]+(0.510270095021)*x[1]**o+(-0.0385822967955)*x[1]+(-0.519736540313)*x[2]**o+(0.0556733091556)*x[2]
        arg[0,3,0,0]=(0.355768034846)*x[0]**o+(0.152240594627)*x[0]+(-0.81081131843)*x[1]**o+(-0.913373483156)*x[1]+(-0.319859744799)*x[2]**o+(-0.306218435393)*x[2]
        arg[0,3,0,1]=(-0.656932063693)*x[0]**o+(0.832886861783)*x[0]+(-0.417112750683)*x[1]**o+(-0.388190265957)*x[1]+(0.246534947096)*x[2]**o+(-0.896822839189)*x[2]
        arg[0,3,1,0]=(0.204713418891)*x[0]**o+(-0.150097488817)*x[0]+(0.806578426941)*x[1]**o+(-0.590052708995)*x[1]+(-0.758958608419)*x[2]**o+(-0.221973528232)*x[2]
        arg[0,3,1,1]=(-0.854308184618)*x[0]**o+(-0.292526203646)*x[0]+(-0.662403504833)*x[1]**o+(0.0955657982227)*x[1]+(0.69471076173)*x[2]**o+(0.384077309264)*x[2]
        arg[0,3,2,0]=(0.982057200864)*x[0]**o+(-0.64758727704)*x[0]+(0.15941493222)*x[1]**o+(-0.17745437115)*x[1]+(-0.72953252955)*x[2]**o+(0.986979270802)*x[2]
        arg[0,3,2,1]=(0.764526698779)*x[0]**o+(0.22809521115)*x[0]+(-0.0762092087998)*x[1]**o+(0.415212410471)*x[1]+(-0.263638988463)*x[2]**o+(-0.934669942336)*x[2]
        arg[0,4,0,0]=(0.0812656932707)*x[0]**o+(0.0397235062518)*x[0]+(-0.0496912823983)*x[1]**o+(0.189891585189)*x[1]+(0.243537823293)*x[2]**o+(0.461256134208)*x[2]
        arg[0,4,0,1]=(0.167596358861)*x[0]**o+(0.441650690481)*x[0]+(0.638100632205)*x[1]**o+(-0.69240615765)*x[1]+(-0.501855369512)*x[2]**o+(0.769465683782)*x[2]
        arg[0,4,1,0]=(-0.137149398927)*x[0]**o+(-0.186548325285)*x[0]+(-0.504656948352)*x[1]**o+(0.228896656134)*x[1]+(-0.472425358061)*x[2]**o+(-0.0769834754996)*x[2]
        arg[0,4,1,1]=(0.311655648626)*x[0]**o+(0.856513683959)*x[0]+(0.47173626537)*x[1]**o+(0.245497333467)*x[1]+(-0.513727691305)*x[2]**o+(-0.588614785682)*x[2]
        arg[0,4,2,0]=(-0.152907854625)*x[0]**o+(-0.86229950575)*x[0]+(-0.931356414564)*x[1]**o+(-0.915667991864)*x[1]+(0.322316118527)*x[2]**o+(-0.987559462539)*x[2]
        arg[0,4,2,1]=(-0.705947769164)*x[0]**o+(0.104081366627)*x[0]+(-0.540454644392)*x[1]**o+(0.404522880836)*x[1]+(0.146464264141)*x[2]**o+(0.586755030006)*x[2]
        arg[1,0,0,0]=(-0.466614768106)*x[0]**o+(0.482681414943)*x[0]+(-0.176553738108)*x[1]**o+(-0.39162272813)*x[1]+(0.74312023324)*x[2]**o+(0.824538905894)*x[2]
        arg[1,0,0,1]=(-0.9351073631)*x[0]**o+(-0.640226640449)*x[0]+(0.966208762054)*x[1]**o+(0.101792167944)*x[1]+(-0.732983657038)*x[2]**o+(-0.112313375937)*x[2]
        arg[1,0,1,0]=(-0.305403470434)*x[0]**o+(-0.414397930347)*x[0]+(0.356093254322)*x[1]**o+(-0.52607560749)*x[1]+(-0.70070674473)*x[2]**o+(-0.0192901616015)*x[2]
        arg[1,0,1,1]=(0.450046989211)*x[0]**o+(0.602770057268)*x[0]+(0.0193812509113)*x[1]**o+(-0.275615149191)*x[1]+(0.0180468267836)*x[2]**o+(-0.81388255113)*x[2]
        arg[1,0,2,0]=(0.913651483249)*x[0]**o+(-0.92188313743)*x[0]+(0.680007150195)*x[1]**o+(-0.300145584708)*x[1]+(-0.398883520002)*x[2]**o+(-0.752781496463)*x[2]
        arg[1,0,2,1]=(-0.265749827845)*x[0]**o+(0.594379728352)*x[0]+(0.463409943587)*x[1]**o+(-0.85840420426)*x[1]+(-0.705963756674)*x[2]**o+(0.251711112887)*x[2]
        arg[1,1,0,0]=(-0.28895690781)*x[0]**o+(0.476461656166)*x[0]+(0.0692567954597)*x[1]**o+(0.88763437878)*x[1]+(0.0872256197554)*x[2]**o+(0.19467039694)*x[2]
        arg[1,1,0,1]=(0.741045418981)*x[0]**o+(-0.726935151332)*x[0]+(0.899418662131)*x[1]**o+(0.27516497286)*x[1]+(-0.715927474971)*x[2]**o+(-0.709283098732)*x[2]
        arg[1,1,1,0]=(0.991407130174)*x[0]**o+(-0.163484265852)*x[0]+(-0.983254548659)*x[1]**o+(0.0568443855477)*x[1]+(0.309664644588)*x[2]**o+(0.904334718845)*x[2]
        arg[1,1,1,1]=(0.237261672833)*x[0]**o+(0.0861047746345)*x[0]+(-0.494898018662)*x[1]**o+(-0.433075334646)*x[1]+(0.588597346505)*x[2]**o+(-0.185559567553)*x[2]
        arg[1,1,2,0]=(-0.969384091709)*x[0]**o+(-0.0229777223491)*x[0]+(0.0945623059666)*x[1]**o+(0.303543647034)*x[1]+(0.659000135381)*x[2]**o+(0.00873791711254)*x[2]
        arg[1,1,2,1]=(-0.421406731882)*x[0]**o+(-0.5017394377)*x[0]+(0.027747130694)*x[1]**o+(0.327278873996)*x[1]+(-0.754133439304)*x[2]**o+(0.673525803845)*x[2]
        arg[1,2,0,0]=(-0.104620319685)*x[0]**o+(-0.685602393278)*x[0]+(0.972173717014)*x[1]**o+(-0.440573017063)*x[1]+(-0.709529395743)*x[2]**o+(0.634170741457)*x[2]
        arg[1,2,0,1]=(-0.560386754264)*x[0]**o+(0.570894623654)*x[0]+(-0.455122833362)*x[1]**o+(-0.00322505937935)*x[1]+(0.959400336149)*x[2]**o+(0.64014859954)*x[2]
        arg[1,2,1,0]=(0.100585233221)*x[0]**o+(0.939532360957)*x[0]+(-0.900301955112)*x[1]**o+(-0.776375896786)*x[1]+(-0.779373294746)*x[2]**o+(0.546157461116)*x[2]
        arg[1,2,1,1]=(-0.680457315071)*x[0]**o+(-0.0278844539208)*x[0]+(-0.629673590044)*x[1]**o+(0.813218376348)*x[1]+(-0.449484131841)*x[2]**o+(0.641682150048)*x[2]
        arg[1,2,2,0]=(0.60056954386)*x[0]**o+(-0.763169434657)*x[0]+(-0.672890915035)*x[1]**o+(-0.368711154571)*x[1]+(0.476810195318)*x[2]**o+(-0.935071124382)*x[2]
        arg[1,2,2,1]=(0.994100148333)*x[0]**o+(0.471647033381)*x[0]+(0.341677296654)*x[1]**o+(0.466442262068)*x[1]+(-0.0634919249117)*x[2]**o+(0.0637348830912)*x[2]
        arg[1,3,0,0]=(-0.120744286726)*x[0]**o+(0.630378984537)*x[0]+(0.0837817796507)*x[1]**o+(-0.850115274881)*x[1]+(0.574342626561)*x[2]**o+(-0.705409915626)*x[2]
        arg[1,3,0,1]=(-0.51181061391)*x[0]**o+(0.711479747616)*x[0]+(-0.00800928076734)*x[1]**o+(-0.703424580277)*x[1]+(-0.925935026157)*x[2]**o+(0.488175992827)*x[2]
        arg[1,3,1,0]=(0.531879615751)*x[0]**o+(-0.214270521934)*x[0]+(-0.432344726015)*x[1]**o+(-0.107660826449)*x[1]+(0.763406427398)*x[2]**o+(-0.511000150429)*x[2]
        arg[1,3,1,1]=(-0.867467568206)*x[0]**o+(-0.738425961596)*x[0]+(-0.925337667706)*x[1]**o+(-0.586227765107)*x[1]+(0.909987211462)*x[2]**o+(0.533347022582)*x[2]
        arg[1,3,2,0]=(-0.47713113862)*x[0]**o+(-0.130876138883)*x[0]+(-0.607996835981)*x[1]**o+(-0.0306261551172)*x[1]+(0.956207500475)*x[2]**o+(-0.287731719382)*x[2]
        arg[1,3,2,1]=(0.179479486943)*x[0]**o+(0.361266562479)*x[0]+(0.374904472267)*x[1]**o+(0.00788379106301)*x[1]+(-0.34090684265)*x[2]**o+(-0.190875910791)*x[2]
        arg[1,4,0,0]=(0.685464203729)*x[0]**o+(0.0634561930987)*x[0]+(-0.662534110385)*x[1]**o+(0.383849898086)*x[1]+(0.948230524205)*x[2]**o+(-0.347042281976)*x[2]
        arg[1,4,0,1]=(-0.484499755322)*x[0]**o+(-0.765655349243)*x[0]+(0.805331514606)*x[1]**o+(-0.0635912547496)*x[1]+(-0.139457134772)*x[2]**o+(0.227446734221)*x[2]
        arg[1,4,1,0]=(-0.913971229647)*x[0]**o+(0.042531889985)*x[0]+(0.143956410289)*x[1]**o+(0.597990264168)*x[1]+(-0.85926903006)*x[2]**o+(0.205900361792)*x[2]
        arg[1,4,1,1]=(-0.955540264653)*x[0]**o+(0.983439073814)*x[0]+(0.246870351555)*x[1]**o+(-0.610917180843)*x[1]+(-0.0418134447092)*x[2]**o+(-0.58407787472)*x[2]
        arg[1,4,2,0]=(0.787274711453)*x[0]**o+(-0.899651339467)*x[0]+(-0.62195254225)*x[1]**o+(-0.516447289251)*x[1]+(0.267082643601)*x[2]**o+(0.0408186039688)*x[2]
        arg[1,4,2,1]=(-0.485773801895)*x[0]**o+(0.663478067655)*x[0]+(0.928973237869)*x[1]**o+(0.353735145156)*x[1]+(0.128409748978)*x[2]**o+(-0.476479648592)*x[2]
        arg[2,0,0,0]=(0.491574821811)*x[0]**o+(0.455303820526)*x[0]+(-0.747773112075)*x[1]**o+(-0.653255987137)*x[1]+(-0.738293040346)*x[2]**o+(0.948070477272)*x[2]
        arg[2,0,0,1]=(-0.697412782403)*x[0]**o+(-0.738389544586)*x[0]+(0.231728579935)*x[1]**o+(0.248503931032)*x[1]+(-0.561155590703)*x[2]**o+(0.324682113147)*x[2]
        arg[2,0,1,0]=(-0.00507638382064)*x[0]**o+(-0.983105866448)*x[0]+(-0.253141627484)*x[1]**o+(-0.0664178091069)*x[1]+(0.194460220144)*x[2]**o+(-0.174257469198)*x[2]
        arg[2,0,1,1]=(-0.644447560463)*x[0]**o+(0.658703761133)*x[0]+(-0.685392825104)*x[1]**o+(-0.766295260796)*x[1]+(-0.922004388493)*x[2]**o+(-0.0745707720923)*x[2]
        arg[2,0,2,0]=(-0.69924009606)*x[0]**o+(-0.398212597328)*x[0]+(-0.364991550186)*x[1]**o+(0.243474288977)*x[1]+(0.129545162767)*x[2]**o+(0.920611954845)*x[2]
        arg[2,0,2,1]=(-0.494476126875)*x[0]**o+(-0.131503103148)*x[0]+(-0.241670500874)*x[1]**o+(0.40439125905)*x[1]+(0.134166779399)*x[2]**o+(-0.507385751809)*x[2]
        arg[2,1,0,0]=(0.301148115913)*x[0]**o+(0.189635133395)*x[0]+(0.744167404774)*x[1]**o+(0.995754344203)*x[1]+(-0.334760662218)*x[2]**o+(-0.182462923741)*x[2]
        arg[2,1,0,1]=(0.523500022508)*x[0]**o+(-0.257042661558)*x[0]+(-0.940233100381)*x[1]**o+(-0.202431731891)*x[1]+(-0.653480190287)*x[2]**o+(0.993156915081)*x[2]
        arg[2,1,1,0]=(-0.502452234661)*x[0]**o+(-0.711581805303)*x[0]+(0.696450091152)*x[1]**o+(0.436746953802)*x[1]+(0.695969418438)*x[2]**o+(-0.671400022595)*x[2]
        arg[2,1,1,1]=(0.0705944125182)*x[0]**o+(-0.865755834485)*x[0]+(0.921555996641)*x[1]**o+(-0.567855814858)*x[1]+(-0.213455192388)*x[2]**o+(0.820868399259)*x[2]
        arg[2,1,2,0]=(0.744232126707)*x[0]**o+(0.336969176485)*x[0]+(0.339407729288)*x[1]**o+(-0.465154968917)*x[1]+(0.389524167202)*x[2]**o+(0.645394954477)*x[2]
        arg[2,1,2,1]=(-0.299085687516)*x[0]**o+(0.220795864743)*x[0]+(-0.19353913538)*x[1]**o+(0.445131396297)*x[1]+(-0.779605318636)*x[2]**o+(-0.76707264371)*x[2]
        arg[2,2,0,0]=(-0.357585997171)*x[0]**o+(-0.248795296406)*x[0]+(0.0876364714478)*x[1]**o+(-0.682833482179)*x[1]+(0.497052575986)*x[2]**o+(0.893378007889)*x[2]
        arg[2,2,0,1]=(0.38067999218)*x[0]**o+(-0.857246232026)*x[0]+(0.0785818284359)*x[1]**o+(-0.473287028123)*x[1]+(-0.763041318956)*x[2]**o+(-0.1119710251)*x[2]
        arg[2,2,1,0]=(-0.0278856926369)*x[0]**o+(0.435955626553)*x[0]+(-0.676274741782)*x[1]**o+(-0.719721825467)*x[1]+(0.195007734703)*x[2]**o+(0.504771278902)*x[2]
        arg[2,2,1,1]=(0.459547475772)*x[0]**o+(-0.669041681613)*x[0]+(-0.741957874364)*x[1]**o+(0.604585337407)*x[1]+(-0.886718742046)*x[2]**o+(0.338113237761)*x[2]
        arg[2,2,2,0]=(0.392926733678)*x[0]**o+(-0.923680589861)*x[0]+(-0.425503328457)*x[1]**o+(-0.0902669250117)*x[1]+(-0.346726072703)*x[2]**o+(0.541482976709)*x[2]
        arg[2,2,2,1]=(0.134867023616)*x[0]**o+(0.478349607348)*x[0]+(-0.334617836366)*x[1]**o+(-0.74587411744)*x[1]+(-0.880365937567)*x[2]**o+(0.820743064452)*x[2]
        arg[2,3,0,0]=(0.081184173895)*x[0]**o+(0.315011929905)*x[0]+(-0.137083926616)*x[1]**o+(0.103926223299)*x[1]+(0.363270989968)*x[2]**o+(-0.454075563606)*x[2]
        arg[2,3,0,1]=(0.457730701321)*x[0]**o+(0.922345625691)*x[0]+(-0.412933933474)*x[1]**o+(-0.546359655103)*x[1]+(0.719101029933)*x[2]**o+(-0.790522859322)*x[2]
        arg[2,3,1,0]=(-0.811816419841)*x[0]**o+(0.0379298408788)*x[0]+(-0.852643125131)*x[1]**o+(0.76881274254)*x[1]+(0.75214553496)*x[2]**o+(0.800382814403)*x[2]
        arg[2,3,1,1]=(0.524802143618)*x[0]**o+(-0.717457078371)*x[0]+(0.389026887965)*x[1]**o+(-0.231912558721)*x[1]+(-0.394705562333)*x[2]**o+(-0.794723802655)*x[2]
        arg[2,3,2,0]=(-0.232215520303)*x[0]**o+(0.994932250617)*x[0]+(-0.760425274222)*x[1]**o+(0.0910727760744)*x[1]+(0.621642024132)*x[2]**o+(0.263978719304)*x[2]
        arg[2,3,2,1]=(0.515269404424)*x[0]**o+(-0.998266958288)*x[0]+(-0.485078033534)*x[1]**o+(-0.960043247553)*x[1]+(-0.926562413314)*x[2]**o+(0.612710762712)*x[2]
        arg[2,4,0,0]=(0.419808446591)*x[0]**o+(0.132792190191)*x[0]+(-0.690603562736)*x[1]**o+(0.45137757885)*x[1]+(-0.262249041567)*x[2]**o+(-0.432995254783)*x[2]
        arg[2,4,0,1]=(0.62413323935)*x[0]**o+(0.963559148607)*x[0]+(-0.879044261015)*x[1]**o+(-0.504981574957)*x[1]+(-0.575204029973)*x[2]**o+(0.066855791448)*x[2]
        arg[2,4,1,0]=(0.321218662791)*x[0]**o+(0.975103526588)*x[0]+(0.783716185844)*x[1]**o+(-0.816368139389)*x[1]+(0.778169798128)*x[2]**o+(-0.527443068085)*x[2]
        arg[2,4,1,1]=(-0.8801082648)*x[0]**o+(-0.0313845929272)*x[0]+(0.485262237838)*x[1]**o+(0.0778888535573)*x[1]+(-0.900948214929)*x[2]**o+(-0.98830300089)*x[2]
        arg[2,4,2,0]=(-0.996395042226)*x[0]**o+(0.253635852642)*x[0]+(0.180965173659)*x[1]**o+(-0.488294662459)*x[1]+(-0.144583511741)*x[2]**o+(0.285854081388)*x[2]
        arg[2,4,2,1]=(0.00988506292061)*x[0]**o+(-0.00192786315452)*x[0]+(0.720775470647)*x[1]**o+(-0.587673311506)*x[1]+(0.313836238675)*x[2]**o+(0.45975125457)*x[2]
        arg[3,0,0,0]=(-0.822049489749)*x[0]**o+(-0.989589435175)*x[0]+(-0.0943830434118)*x[1]**o+(-0.789229900035)*x[1]+(0.311382619317)*x[2]**o+(-0.316646938685)*x[2]
        arg[3,0,0,1]=(0.501517916677)*x[0]**o+(-0.704502365436)*x[0]+(0.913184728006)*x[1]**o+(-0.843712848791)*x[1]+(0.901479696878)*x[2]**o+(-0.713893336542)*x[2]
        arg[3,0,1,0]=(0.0771592811086)*x[0]**o+(0.796038032206)*x[0]+(0.566256212871)*x[1]**o+(0.394136659986)*x[1]+(-0.794422065388)*x[2]**o+(0.662174992848)*x[2]
        arg[3,0,1,1]=(0.180225115167)*x[0]**o+(0.366965867971)*x[0]+(-0.244015756153)*x[1]**o+(0.433571067335)*x[1]+(0.129075150185)*x[2]**o+(-0.216304414605)*x[2]
        arg[3,0,2,0]=(0.100277967034)*x[0]**o+(-0.590583546786)*x[0]+(-0.595553697152)*x[1]**o+(0.923573252239)*x[1]+(0.578966663184)*x[2]**o+(-0.411082913097)*x[2]
        arg[3,0,2,1]=(-0.575315696811)*x[0]**o+(0.608760285918)*x[0]+(-0.3592074605)*x[1]**o+(0.515119945308)*x[1]+(-0.4000138306)*x[2]**o+(0.477685324937)*x[2]
        arg[3,1,0,0]=(0.0780014276919)*x[0]**o+(-0.0932830671471)*x[0]+(0.39477236642)*x[1]**o+(0.406673135556)*x[1]+(-0.235480707568)*x[2]**o+(-0.872302257144)*x[2]
        arg[3,1,0,1]=(0.170573072646)*x[0]**o+(0.868146561326)*x[0]+(0.262136343896)*x[1]**o+(0.192343529912)*x[1]+(0.157400965693)*x[2]**o+(-0.932981619473)*x[2]
        arg[3,1,1,0]=(0.891433179291)*x[0]**o+(-0.281396449997)*x[0]+(-0.0803173619772)*x[1]**o+(-0.378460054356)*x[1]+(-0.581578656014)*x[2]**o+(-0.65856067509)*x[2]
        arg[3,1,1,1]=(0.343599956626)*x[0]**o+(-0.92063694125)*x[0]+(0.440698102072)*x[1]**o+(0.767124166388)*x[1]+(0.050964317543)*x[2]**o+(1.35605562239e-05)*x[2]
        arg[3,1,2,0]=(-0.614268150987)*x[0]**o+(0.363806471556)*x[0]+(0.92193457905)*x[1]**o+(0.157919580642)*x[1]+(-0.955575321124)*x[2]**o+(-0.984808437666)*x[2]
        arg[3,1,2,1]=(0.030099648966)*x[0]**o+(-0.375999369491)*x[0]+(-0.0992643926008)*x[1]**o+(0.109209161093)*x[1]+(-0.974043780088)*x[2]**o+(0.908049658745)*x[2]
        arg[3,2,0,0]=(0.297771341106)*x[0]**o+(-0.637802228169)*x[0]+(0.398100441783)*x[1]**o+(-0.0643425181651)*x[1]+(0.662775339033)*x[2]**o+(0.744835182052)*x[2]
        arg[3,2,0,1]=(0.312026420583)*x[0]**o+(0.865855052574)*x[0]+(0.0195001783165)*x[1]**o+(0.537899238473)*x[1]+(0.554893357481)*x[2]**o+(0.71555610145)*x[2]
        arg[3,2,1,0]=(-0.199138146186)*x[0]**o+(0.827546489855)*x[0]+(-0.480965464736)*x[1]**o+(-0.473796093187)*x[1]+(0.880264675049)*x[2]**o+(0.544506670844)*x[2]
        arg[3,2,1,1]=(0.773798005008)*x[0]**o+(0.744945241634)*x[0]+(0.548170564608)*x[1]**o+(-0.560503692823)*x[1]+(-0.435630215427)*x[2]**o+(0.080015730707)*x[2]
        arg[3,2,2,0]=(0.36735714937)*x[0]**o+(-0.867071557873)*x[0]+(0.936135650431)*x[1]**o+(-0.432533600524)*x[1]+(-0.0224430204069)*x[2]**o+(0.218590302621)*x[2]
        arg[3,2,2,1]=(0.932808382499)*x[0]**o+(-0.0970016713319)*x[0]+(0.458943467579)*x[1]**o+(-0.134338789898)*x[1]+(-0.426892557314)*x[2]**o+(0.627071826863)*x[2]
        arg[3,3,0,0]=(-0.623804149075)*x[0]**o+(0.910423590172)*x[0]+(0.465938181097)*x[1]**o+(0.495479245391)*x[1]+(0.693745442185)*x[2]**o+(-0.437014226757)*x[2]
        arg[3,3,0,1]=(0.598807017496)*x[0]**o+(-0.112361840211)*x[0]+(-0.550899685213)*x[1]**o+(-0.0478222019564)*x[1]+(-0.0110129517693)*x[2]**o+(-0.059534786374)*x[2]
        arg[3,3,1,0]=(-0.561155633409)*x[0]**o+(-0.463563057636)*x[0]+(-0.170463996009)*x[1]**o+(-0.214509204942)*x[1]+(0.851200198666)*x[2]**o+(-0.721036329689)*x[2]
        arg[3,3,1,1]=(0.608666240405)*x[0]**o+(0.814232455115)*x[0]+(-0.604645629689)*x[1]**o+(0.960976509673)*x[1]+(0.365807996219)*x[2]**o+(-0.87491909238)*x[2]
        arg[3,3,2,0]=(0.120126988418)*x[0]**o+(-0.745687314825)*x[0]+(-0.332697826984)*x[1]**o+(-0.823683406516)*x[1]+(0.305444079613)*x[2]**o+(0.901972115441)*x[2]
        arg[3,3,2,1]=(0.0470293848904)*x[0]**o+(-0.195496754814)*x[0]+(0.671870114105)*x[1]**o+(0.170273146371)*x[1]+(-0.851927986188)*x[2]**o+(0.772192863973)*x[2]
        arg[3,4,0,0]=(-0.95392341727)*x[0]**o+(-0.665694215521)*x[0]+(-0.0247396499378)*x[1]**o+(0.441352857387)*x[1]+(0.767156547791)*x[2]**o+(-0.14696372018)*x[2]
        arg[3,4,0,1]=(0.209761166815)*x[0]**o+(0.978934473425)*x[0]+(0.943806891053)*x[1]**o+(-0.756619917601)*x[1]+(0.635699949764)*x[2]**o+(-0.345248269568)*x[2]
        arg[3,4,1,0]=(0.748020785785)*x[0]**o+(0.311104549542)*x[0]+(0.739127551248)*x[1]**o+(-0.635238575859)*x[1]+(0.623898875653)*x[2]**o+(0.343407392068)*x[2]
        arg[3,4,1,1]=(-0.422500892471)*x[0]**o+(0.0688495013346)*x[0]+(-0.919653906232)*x[1]**o+(0.531036586396)*x[1]+(-0.22487910672)*x[2]**o+(-0.848225994541)*x[2]
        arg[3,4,2,0]=(0.723965744673)*x[0]**o+(0.558602959224)*x[0]+(0.699539243308)*x[1]**o+(-0.597426541715)*x[1]+(-0.0128675070522)*x[2]**o+(-0.874796626242)*x[2]
        arg[3,4,2,1]=(-0.962970021216)*x[0]**o+(0.532886481874)*x[0]+(-0.523159705965)*x[1]**o+(-0.82887407638)*x[1]+(0.874288116843)*x[2]**o+(0.520024515804)*x[2]
        ref[0,0,0,0]=(0.507422888369)*x_ref[0]**o+(0.808798513305)*x_ref[0]+(0.51493073805)*x_ref[1]**o+(-0.928527686947)*x_ref[1]+(-0.788010447498)*x_ref[2]**o+(-0.378389833131)*x_ref[2]
        ref[0,0,0,1]=(-0.475851243177)*x_ref[0]**o+(0.746578446988)*x_ref[0]+(0.504778622494)*x_ref[1]**o+(-0.677770398899)*x_ref[1]+(-0.862535648403)*x_ref[2]**o+(0.620414141691)*x_ref[2]
        ref[0,0,1,0]=(-0.361726879582)*x_ref[0]**o+(0.554584127587)*x_ref[0]+(0.436610414772)*x_ref[1]**o+(-0.686534492717)*x_ref[1]+(0.757226061419)*x_ref[2]**o+(0.0948537725389)*x_ref[2]
        ref[0,0,1,1]=(-0.362160696355)*x_ref[0]**o+(0.766524051703)*x_ref[0]+(0.209980857795)*x_ref[1]**o+(-0.478398061437)*x_ref[1]+(-0.410960179122)*x_ref[2]**o+(-0.213272138462)*x_ref[2]
        ref[0,0,2,0]=(0.87154872554)*x_ref[0]**o+(-0.727915301904)*x_ref[0]+(0.683580698056)*x_ref[1]**o+(0.215052521277)*x_ref[1]+(0.267231507898)*x_ref[2]**o+(-0.0124189912449)*x_ref[2]
        ref[0,0,2,1]=(0.631400944876)*x_ref[0]**o+(0.537293628606)*x_ref[0]+(0.944754350994)*x_ref[1]**o+(0.108068908012)*x_ref[1]+(0.890117096365)*x_ref[2]**o+(0.164706867235)*x_ref[2]
        ref[0,1,0,0]=(-0.935871759207)*x_ref[0]**o+(0.350133758077)*x_ref[0]+(-0.966482181015)*x_ref[1]**o+(0.440902701528)*x_ref[1]+(-0.893329541322)*x_ref[2]**o+(0.350317827938)*x_ref[2]
        ref[0,1,0,1]=(0.465305203601)*x_ref[0]**o+(0.711131797928)*x_ref[0]+(0.104091536498)*x_ref[1]**o+(0.484811628127)*x_ref[1]+(0.676878954516)*x_ref[2]**o+(-0.99327016995)*x_ref[2]
        ref[0,1,1,0]=(-0.361189016151)*x_ref[0]**o+(0.962468573492)*x_ref[0]+(-0.158571122564)*x_ref[1]**o+(-0.293583660912)*x_ref[1]+(-0.713699173608)*x_ref[2]**o+(-0.507430163361)*x_ref[2]
        ref[0,1,1,1]=(0.172564527054)*x_ref[0]**o+(0.273438145027)*x_ref[0]+(0.137378266239)*x_ref[1]**o+(-0.675788606841)*x_ref[1]+(-0.362143416121)*x_ref[2]**o+(0.0703897749775)*x_ref[2]
        ref[0,1,2,0]=(-0.941374717271)*x_ref[0]**o+(0.289433165458)*x_ref[0]+(0.318615227028)*x_ref[1]**o+(0.524362568204)*x_ref[1]+(-0.612590305095)*x_ref[2]**o+(-0.117555538957)*x_ref[2]
        ref[0,1,2,1]=(-0.442920825551)*x_ref[0]**o+(-0.454982146579)*x_ref[0]+(0.355353737589)*x_ref[1]**o+(-0.729623917004)*x_ref[1]+(-0.567220335937)*x_ref[2]**o+(0.977652639976)*x_ref[2]
        ref[0,2,0,0]=(0.942485930373)*x_ref[0]**o+(-0.605823961033)*x_ref[0]+(-0.143465779338)*x_ref[1]**o+(0.327354308987)*x_ref[1]+(0.691900417542)*x_ref[2]**o+(-0.514165213849)*x_ref[2]
        ref[0,2,0,1]=(0.018029272977)*x_ref[0]**o+(-0.209992705469)*x_ref[0]+(-0.224614727554)*x_ref[1]**o+(-0.862697462048)*x_ref[1]+(0.274791387928)*x_ref[2]**o+(-0.907445869506)*x_ref[2]
        ref[0,2,1,0]=(0.862755248265)*x_ref[0]**o+(0.020831641939)*x_ref[0]+(0.496662599261)*x_ref[1]**o+(0.79318975953)*x_ref[1]+(0.0912166895541)*x_ref[2]**o+(0.556337610316)*x_ref[2]
        ref[0,2,1,1]=(-0.09652997071)*x_ref[0]**o+(-0.954279421394)*x_ref[0]+(-0.601053524018)*x_ref[1]**o+(0.642528012068)*x_ref[1]+(0.295748113888)*x_ref[2]**o+(-0.318166346186)*x_ref[2]
        ref[0,2,2,0]=(-0.914878854129)*x_ref[0]**o+(0.159201123642)*x_ref[0]+(0.589819056368)*x_ref[1]**o+(-0.715763226823)*x_ref[1]+(-0.444572583567)*x_ref[2]**o+(-0.487866651367)*x_ref[2]
        ref[0,2,2,1]=(-0.71271969175)*x_ref[0]**o+(0.454616308594)*x_ref[0]+(0.510270095021)*x_ref[1]**o+(-0.0385822967955)*x_ref[1]+(-0.519736540313)*x_ref[2]**o+(0.0556733091556)*x_ref[2]
        ref[0,3,0,0]=(0.355768034846)*x_ref[0]**o+(0.152240594627)*x_ref[0]+(-0.81081131843)*x_ref[1]**o+(-0.913373483156)*x_ref[1]+(-0.319859744799)*x_ref[2]**o+(-0.306218435393)*x_ref[2]
        ref[0,3,0,1]=(-0.656932063693)*x_ref[0]**o+(0.832886861783)*x_ref[0]+(-0.417112750683)*x_ref[1]**o+(-0.388190265957)*x_ref[1]+(0.246534947096)*x_ref[2]**o+(-0.896822839189)*x_ref[2]
        ref[0,3,1,0]=(0.204713418891)*x_ref[0]**o+(-0.150097488817)*x_ref[0]+(0.806578426941)*x_ref[1]**o+(-0.590052708995)*x_ref[1]+(-0.758958608419)*x_ref[2]**o+(-0.221973528232)*x_ref[2]
        ref[0,3,1,1]=(-0.854308184618)*x_ref[0]**o+(-0.292526203646)*x_ref[0]+(-0.662403504833)*x_ref[1]**o+(0.0955657982227)*x_ref[1]+(0.69471076173)*x_ref[2]**o+(0.384077309264)*x_ref[2]
        ref[0,3,2,0]=(0.982057200864)*x_ref[0]**o+(-0.64758727704)*x_ref[0]+(0.15941493222)*x_ref[1]**o+(-0.17745437115)*x_ref[1]+(-0.72953252955)*x_ref[2]**o+(0.986979270802)*x_ref[2]
        ref[0,3,2,1]=(0.764526698779)*x_ref[0]**o+(0.22809521115)*x_ref[0]+(-0.0762092087998)*x_ref[1]**o+(0.415212410471)*x_ref[1]+(-0.263638988463)*x_ref[2]**o+(-0.934669942336)*x_ref[2]
        ref[0,4,0,0]=(0.0812656932707)*x_ref[0]**o+(0.0397235062518)*x_ref[0]+(-0.0496912823983)*x_ref[1]**o+(0.189891585189)*x_ref[1]+(0.243537823293)*x_ref[2]**o+(0.461256134208)*x_ref[2]
        ref[0,4,0,1]=(0.167596358861)*x_ref[0]**o+(0.441650690481)*x_ref[0]+(0.638100632205)*x_ref[1]**o+(-0.69240615765)*x_ref[1]+(-0.501855369512)*x_ref[2]**o+(0.769465683782)*x_ref[2]
        ref[0,4,1,0]=(-0.137149398927)*x_ref[0]**o+(-0.186548325285)*x_ref[0]+(-0.504656948352)*x_ref[1]**o+(0.228896656134)*x_ref[1]+(-0.472425358061)*x_ref[2]**o+(-0.0769834754996)*x_ref[2]
        ref[0,4,1,1]=(0.311655648626)*x_ref[0]**o+(0.856513683959)*x_ref[0]+(0.47173626537)*x_ref[1]**o+(0.245497333467)*x_ref[1]+(-0.513727691305)*x_ref[2]**o+(-0.588614785682)*x_ref[2]
        ref[0,4,2,0]=(-0.152907854625)*x_ref[0]**o+(-0.86229950575)*x_ref[0]+(-0.931356414564)*x_ref[1]**o+(-0.915667991864)*x_ref[1]+(0.322316118527)*x_ref[2]**o+(-0.987559462539)*x_ref[2]
        ref[0,4,2,1]=(-0.705947769164)*x_ref[0]**o+(0.104081366627)*x_ref[0]+(-0.540454644392)*x_ref[1]**o+(0.404522880836)*x_ref[1]+(0.146464264141)*x_ref[2]**o+(0.586755030006)*x_ref[2]
        ref[1,0,0,0]=(-0.466614768106)*x_ref[0]**o+(0.482681414943)*x_ref[0]+(-0.176553738108)*x_ref[1]**o+(-0.39162272813)*x_ref[1]+(0.74312023324)*x_ref[2]**o+(0.824538905894)*x_ref[2]
        ref[1,0,0,1]=(-0.9351073631)*x_ref[0]**o+(-0.640226640449)*x_ref[0]+(0.966208762054)*x_ref[1]**o+(0.101792167944)*x_ref[1]+(-0.732983657038)*x_ref[2]**o+(-0.112313375937)*x_ref[2]
        ref[1,0,1,0]=(-0.305403470434)*x_ref[0]**o+(-0.414397930347)*x_ref[0]+(0.356093254322)*x_ref[1]**o+(-0.52607560749)*x_ref[1]+(-0.70070674473)*x_ref[2]**o+(-0.0192901616015)*x_ref[2]
        ref[1,0,1,1]=(0.450046989211)*x_ref[0]**o+(0.602770057268)*x_ref[0]+(0.0193812509113)*x_ref[1]**o+(-0.275615149191)*x_ref[1]+(0.0180468267836)*x_ref[2]**o+(-0.81388255113)*x_ref[2]
        ref[1,0,2,0]=(0.913651483249)*x_ref[0]**o+(-0.92188313743)*x_ref[0]+(0.680007150195)*x_ref[1]**o+(-0.300145584708)*x_ref[1]+(-0.398883520002)*x_ref[2]**o+(-0.752781496463)*x_ref[2]
        ref[1,0,2,1]=(-0.265749827845)*x_ref[0]**o+(0.594379728352)*x_ref[0]+(0.463409943587)*x_ref[1]**o+(-0.85840420426)*x_ref[1]+(-0.705963756674)*x_ref[2]**o+(0.251711112887)*x_ref[2]
        ref[1,1,0,0]=(-0.28895690781)*x_ref[0]**o+(0.476461656166)*x_ref[0]+(0.0692567954597)*x_ref[1]**o+(0.88763437878)*x_ref[1]+(0.0872256197554)*x_ref[2]**o+(0.19467039694)*x_ref[2]
        ref[1,1,0,1]=(0.741045418981)*x_ref[0]**o+(-0.726935151332)*x_ref[0]+(0.899418662131)*x_ref[1]**o+(0.27516497286)*x_ref[1]+(-0.715927474971)*x_ref[2]**o+(-0.709283098732)*x_ref[2]
        ref[1,1,1,0]=(0.991407130174)*x_ref[0]**o+(-0.163484265852)*x_ref[0]+(-0.983254548659)*x_ref[1]**o+(0.0568443855477)*x_ref[1]+(0.309664644588)*x_ref[2]**o+(0.904334718845)*x_ref[2]
        ref[1,1,1,1]=(0.237261672833)*x_ref[0]**o+(0.0861047746345)*x_ref[0]+(-0.494898018662)*x_ref[1]**o+(-0.433075334646)*x_ref[1]+(0.588597346505)*x_ref[2]**o+(-0.185559567553)*x_ref[2]
        ref[1,1,2,0]=(-0.969384091709)*x_ref[0]**o+(-0.0229777223491)*x_ref[0]+(0.0945623059666)*x_ref[1]**o+(0.303543647034)*x_ref[1]+(0.659000135381)*x_ref[2]**o+(0.00873791711254)*x_ref[2]
        ref[1,1,2,1]=(-0.421406731882)*x_ref[0]**o+(-0.5017394377)*x_ref[0]+(0.027747130694)*x_ref[1]**o+(0.327278873996)*x_ref[1]+(-0.754133439304)*x_ref[2]**o+(0.673525803845)*x_ref[2]
        ref[1,2,0,0]=(-0.104620319685)*x_ref[0]**o+(-0.685602393278)*x_ref[0]+(0.972173717014)*x_ref[1]**o+(-0.440573017063)*x_ref[1]+(-0.709529395743)*x_ref[2]**o+(0.634170741457)*x_ref[2]
        ref[1,2,0,1]=(-0.560386754264)*x_ref[0]**o+(0.570894623654)*x_ref[0]+(-0.455122833362)*x_ref[1]**o+(-0.00322505937935)*x_ref[1]+(0.959400336149)*x_ref[2]**o+(0.64014859954)*x_ref[2]
        ref[1,2,1,0]=(0.100585233221)*x_ref[0]**o+(0.939532360957)*x_ref[0]+(-0.900301955112)*x_ref[1]**o+(-0.776375896786)*x_ref[1]+(-0.779373294746)*x_ref[2]**o+(0.546157461116)*x_ref[2]
        ref[1,2,1,1]=(-0.680457315071)*x_ref[0]**o+(-0.0278844539208)*x_ref[0]+(-0.629673590044)*x_ref[1]**o+(0.813218376348)*x_ref[1]+(-0.449484131841)*x_ref[2]**o+(0.641682150048)*x_ref[2]
        ref[1,2,2,0]=(0.60056954386)*x_ref[0]**o+(-0.763169434657)*x_ref[0]+(-0.672890915035)*x_ref[1]**o+(-0.368711154571)*x_ref[1]+(0.476810195318)*x_ref[2]**o+(-0.935071124382)*x_ref[2]
        ref[1,2,2,1]=(0.994100148333)*x_ref[0]**o+(0.471647033381)*x_ref[0]+(0.341677296654)*x_ref[1]**o+(0.466442262068)*x_ref[1]+(-0.0634919249117)*x_ref[2]**o+(0.0637348830912)*x_ref[2]
        ref[1,3,0,0]=(-0.120744286726)*x_ref[0]**o+(0.630378984537)*x_ref[0]+(0.0837817796507)*x_ref[1]**o+(-0.850115274881)*x_ref[1]+(0.574342626561)*x_ref[2]**o+(-0.705409915626)*x_ref[2]
        ref[1,3,0,1]=(-0.51181061391)*x_ref[0]**o+(0.711479747616)*x_ref[0]+(-0.00800928076734)*x_ref[1]**o+(-0.703424580277)*x_ref[1]+(-0.925935026157)*x_ref[2]**o+(0.488175992827)*x_ref[2]
        ref[1,3,1,0]=(0.531879615751)*x_ref[0]**o+(-0.214270521934)*x_ref[0]+(-0.432344726015)*x_ref[1]**o+(-0.107660826449)*x_ref[1]+(0.763406427398)*x_ref[2]**o+(-0.511000150429)*x_ref[2]
        ref[1,3,1,1]=(-0.867467568206)*x_ref[0]**o+(-0.738425961596)*x_ref[0]+(-0.925337667706)*x_ref[1]**o+(-0.586227765107)*x_ref[1]+(0.909987211462)*x_ref[2]**o+(0.533347022582)*x_ref[2]
        ref[1,3,2,0]=(-0.47713113862)*x_ref[0]**o+(-0.130876138883)*x_ref[0]+(-0.607996835981)*x_ref[1]**o+(-0.0306261551172)*x_ref[1]+(0.956207500475)*x_ref[2]**o+(-0.287731719382)*x_ref[2]
        ref[1,3,2,1]=(0.179479486943)*x_ref[0]**o+(0.361266562479)*x_ref[0]+(0.374904472267)*x_ref[1]**o+(0.00788379106301)*x_ref[1]+(-0.34090684265)*x_ref[2]**o+(-0.190875910791)*x_ref[2]
        ref[1,4,0,0]=(0.685464203729)*x_ref[0]**o+(0.0634561930987)*x_ref[0]+(-0.662534110385)*x_ref[1]**o+(0.383849898086)*x_ref[1]+(0.948230524205)*x_ref[2]**o+(-0.347042281976)*x_ref[2]
        ref[1,4,0,1]=(-0.484499755322)*x_ref[0]**o+(-0.765655349243)*x_ref[0]+(0.805331514606)*x_ref[1]**o+(-0.0635912547496)*x_ref[1]+(-0.139457134772)*x_ref[2]**o+(0.227446734221)*x_ref[2]
        ref[1,4,1,0]=(-0.913971229647)*x_ref[0]**o+(0.042531889985)*x_ref[0]+(0.143956410289)*x_ref[1]**o+(0.597990264168)*x_ref[1]+(-0.85926903006)*x_ref[2]**o+(0.205900361792)*x_ref[2]
        ref[1,4,1,1]=(-0.955540264653)*x_ref[0]**o+(0.983439073814)*x_ref[0]+(0.246870351555)*x_ref[1]**o+(-0.610917180843)*x_ref[1]+(-0.0418134447092)*x_ref[2]**o+(-0.58407787472)*x_ref[2]
        ref[1,4,2,0]=(0.787274711453)*x_ref[0]**o+(-0.899651339467)*x_ref[0]+(-0.62195254225)*x_ref[1]**o+(-0.516447289251)*x_ref[1]+(0.267082643601)*x_ref[2]**o+(0.0408186039688)*x_ref[2]
        ref[1,4,2,1]=(-0.485773801895)*x_ref[0]**o+(0.663478067655)*x_ref[0]+(0.928973237869)*x_ref[1]**o+(0.353735145156)*x_ref[1]+(0.128409748978)*x_ref[2]**o+(-0.476479648592)*x_ref[2]
        ref[2,0,0,0]=(0.491574821811)*x_ref[0]**o+(0.455303820526)*x_ref[0]+(-0.747773112075)*x_ref[1]**o+(-0.653255987137)*x_ref[1]+(-0.738293040346)*x_ref[2]**o+(0.948070477272)*x_ref[2]
        ref[2,0,0,1]=(-0.697412782403)*x_ref[0]**o+(-0.738389544586)*x_ref[0]+(0.231728579935)*x_ref[1]**o+(0.248503931032)*x_ref[1]+(-0.561155590703)*x_ref[2]**o+(0.324682113147)*x_ref[2]
        ref[2,0,1,0]=(-0.00507638382064)*x_ref[0]**o+(-0.983105866448)*x_ref[0]+(-0.253141627484)*x_ref[1]**o+(-0.0664178091069)*x_ref[1]+(0.194460220144)*x_ref[2]**o+(-0.174257469198)*x_ref[2]
        ref[2,0,1,1]=(-0.644447560463)*x_ref[0]**o+(0.658703761133)*x_ref[0]+(-0.685392825104)*x_ref[1]**o+(-0.766295260796)*x_ref[1]+(-0.922004388493)*x_ref[2]**o+(-0.0745707720923)*x_ref[2]
        ref[2,0,2,0]=(-0.69924009606)*x_ref[0]**o+(-0.398212597328)*x_ref[0]+(-0.364991550186)*x_ref[1]**o+(0.243474288977)*x_ref[1]+(0.129545162767)*x_ref[2]**o+(0.920611954845)*x_ref[2]
        ref[2,0,2,1]=(-0.494476126875)*x_ref[0]**o+(-0.131503103148)*x_ref[0]+(-0.241670500874)*x_ref[1]**o+(0.40439125905)*x_ref[1]+(0.134166779399)*x_ref[2]**o+(-0.507385751809)*x_ref[2]
        ref[2,1,0,0]=(0.301148115913)*x_ref[0]**o+(0.189635133395)*x_ref[0]+(0.744167404774)*x_ref[1]**o+(0.995754344203)*x_ref[1]+(-0.334760662218)*x_ref[2]**o+(-0.182462923741)*x_ref[2]
        ref[2,1,0,1]=(0.523500022508)*x_ref[0]**o+(-0.257042661558)*x_ref[0]+(-0.940233100381)*x_ref[1]**o+(-0.202431731891)*x_ref[1]+(-0.653480190287)*x_ref[2]**o+(0.993156915081)*x_ref[2]
        ref[2,1,1,0]=(-0.502452234661)*x_ref[0]**o+(-0.711581805303)*x_ref[0]+(0.696450091152)*x_ref[1]**o+(0.436746953802)*x_ref[1]+(0.695969418438)*x_ref[2]**o+(-0.671400022595)*x_ref[2]
        ref[2,1,1,1]=(0.0705944125182)*x_ref[0]**o+(-0.865755834485)*x_ref[0]+(0.921555996641)*x_ref[1]**o+(-0.567855814858)*x_ref[1]+(-0.213455192388)*x_ref[2]**o+(0.820868399259)*x_ref[2]
        ref[2,1,2,0]=(0.744232126707)*x_ref[0]**o+(0.336969176485)*x_ref[0]+(0.339407729288)*x_ref[1]**o+(-0.465154968917)*x_ref[1]+(0.389524167202)*x_ref[2]**o+(0.645394954477)*x_ref[2]
        ref[2,1,2,1]=(-0.299085687516)*x_ref[0]**o+(0.220795864743)*x_ref[0]+(-0.19353913538)*x_ref[1]**o+(0.445131396297)*x_ref[1]+(-0.779605318636)*x_ref[2]**o+(-0.76707264371)*x_ref[2]
        ref[2,2,0,0]=(-0.357585997171)*x_ref[0]**o+(-0.248795296406)*x_ref[0]+(0.0876364714478)*x_ref[1]**o+(-0.682833482179)*x_ref[1]+(0.497052575986)*x_ref[2]**o+(0.893378007889)*x_ref[2]
        ref[2,2,0,1]=(0.38067999218)*x_ref[0]**o+(-0.857246232026)*x_ref[0]+(0.0785818284359)*x_ref[1]**o+(-0.473287028123)*x_ref[1]+(-0.763041318956)*x_ref[2]**o+(-0.1119710251)*x_ref[2]
        ref[2,2,1,0]=(-0.0278856926369)*x_ref[0]**o+(0.435955626553)*x_ref[0]+(-0.676274741782)*x_ref[1]**o+(-0.719721825467)*x_ref[1]+(0.195007734703)*x_ref[2]**o+(0.504771278902)*x_ref[2]
        ref[2,2,1,1]=(0.459547475772)*x_ref[0]**o+(-0.669041681613)*x_ref[0]+(-0.741957874364)*x_ref[1]**o+(0.604585337407)*x_ref[1]+(-0.886718742046)*x_ref[2]**o+(0.338113237761)*x_ref[2]
        ref[2,2,2,0]=(0.392926733678)*x_ref[0]**o+(-0.923680589861)*x_ref[0]+(-0.425503328457)*x_ref[1]**o+(-0.0902669250117)*x_ref[1]+(-0.346726072703)*x_ref[2]**o+(0.541482976709)*x_ref[2]
        ref[2,2,2,1]=(0.134867023616)*x_ref[0]**o+(0.478349607348)*x_ref[0]+(-0.334617836366)*x_ref[1]**o+(-0.74587411744)*x_ref[1]+(-0.880365937567)*x_ref[2]**o+(0.820743064452)*x_ref[2]
        ref[2,3,0,0]=(0.081184173895)*x_ref[0]**o+(0.315011929905)*x_ref[0]+(-0.137083926616)*x_ref[1]**o+(0.103926223299)*x_ref[1]+(0.363270989968)*x_ref[2]**o+(-0.454075563606)*x_ref[2]
        ref[2,3,0,1]=(0.457730701321)*x_ref[0]**o+(0.922345625691)*x_ref[0]+(-0.412933933474)*x_ref[1]**o+(-0.546359655103)*x_ref[1]+(0.719101029933)*x_ref[2]**o+(-0.790522859322)*x_ref[2]
        ref[2,3,1,0]=(-0.811816419841)*x_ref[0]**o+(0.0379298408788)*x_ref[0]+(-0.852643125131)*x_ref[1]**o+(0.76881274254)*x_ref[1]+(0.75214553496)*x_ref[2]**o+(0.800382814403)*x_ref[2]
        ref[2,3,1,1]=(0.524802143618)*x_ref[0]**o+(-0.717457078371)*x_ref[0]+(0.389026887965)*x_ref[1]**o+(-0.231912558721)*x_ref[1]+(-0.394705562333)*x_ref[2]**o+(-0.794723802655)*x_ref[2]
        ref[2,3,2,0]=(-0.232215520303)*x_ref[0]**o+(0.994932250617)*x_ref[0]+(-0.760425274222)*x_ref[1]**o+(0.0910727760744)*x_ref[1]+(0.621642024132)*x_ref[2]**o+(0.263978719304)*x_ref[2]
        ref[2,3,2,1]=(0.515269404424)*x_ref[0]**o+(-0.998266958288)*x_ref[0]+(-0.485078033534)*x_ref[1]**o+(-0.960043247553)*x_ref[1]+(-0.926562413314)*x_ref[2]**o+(0.612710762712)*x_ref[2]
        ref[2,4,0,0]=(0.419808446591)*x_ref[0]**o+(0.132792190191)*x_ref[0]+(-0.690603562736)*x_ref[1]**o+(0.45137757885)*x_ref[1]+(-0.262249041567)*x_ref[2]**o+(-0.432995254783)*x_ref[2]
        ref[2,4,0,1]=(0.62413323935)*x_ref[0]**o+(0.963559148607)*x_ref[0]+(-0.879044261015)*x_ref[1]**o+(-0.504981574957)*x_ref[1]+(-0.575204029973)*x_ref[2]**o+(0.066855791448)*x_ref[2]
        ref[2,4,1,0]=(0.321218662791)*x_ref[0]**o+(0.975103526588)*x_ref[0]+(0.783716185844)*x_ref[1]**o+(-0.816368139389)*x_ref[1]+(0.778169798128)*x_ref[2]**o+(-0.527443068085)*x_ref[2]
        ref[2,4,1,1]=(-0.8801082648)*x_ref[0]**o+(-0.0313845929272)*x_ref[0]+(0.485262237838)*x_ref[1]**o+(0.0778888535573)*x_ref[1]+(-0.900948214929)*x_ref[2]**o+(-0.98830300089)*x_ref[2]
        ref[2,4,2,0]=(-0.996395042226)*x_ref[0]**o+(0.253635852642)*x_ref[0]+(0.180965173659)*x_ref[1]**o+(-0.488294662459)*x_ref[1]+(-0.144583511741)*x_ref[2]**o+(0.285854081388)*x_ref[2]
        ref[2,4,2,1]=(0.00988506292061)*x_ref[0]**o+(-0.00192786315452)*x_ref[0]+(0.720775470647)*x_ref[1]**o+(-0.587673311506)*x_ref[1]+(0.313836238675)*x_ref[2]**o+(0.45975125457)*x_ref[2]
        ref[3,0,0,0]=(-0.822049489749)*x_ref[0]**o+(-0.989589435175)*x_ref[0]+(-0.0943830434118)*x_ref[1]**o+(-0.789229900035)*x_ref[1]+(0.311382619317)*x_ref[2]**o+(-0.316646938685)*x_ref[2]
        ref[3,0,0,1]=(0.501517916677)*x_ref[0]**o+(-0.704502365436)*x_ref[0]+(0.913184728006)*x_ref[1]**o+(-0.843712848791)*x_ref[1]+(0.901479696878)*x_ref[2]**o+(-0.713893336542)*x_ref[2]
        ref[3,0,1,0]=(0.0771592811086)*x_ref[0]**o+(0.796038032206)*x_ref[0]+(0.566256212871)*x_ref[1]**o+(0.394136659986)*x_ref[1]+(-0.794422065388)*x_ref[2]**o+(0.662174992848)*x_ref[2]
        ref[3,0,1,1]=(0.180225115167)*x_ref[0]**o+(0.366965867971)*x_ref[0]+(-0.244015756153)*x_ref[1]**o+(0.433571067335)*x_ref[1]+(0.129075150185)*x_ref[2]**o+(-0.216304414605)*x_ref[2]
        ref[3,0,2,0]=(0.100277967034)*x_ref[0]**o+(-0.590583546786)*x_ref[0]+(-0.595553697152)*x_ref[1]**o+(0.923573252239)*x_ref[1]+(0.578966663184)*x_ref[2]**o+(-0.411082913097)*x_ref[2]
        ref[3,0,2,1]=(-0.575315696811)*x_ref[0]**o+(0.608760285918)*x_ref[0]+(-0.3592074605)*x_ref[1]**o+(0.515119945308)*x_ref[1]+(-0.4000138306)*x_ref[2]**o+(0.477685324937)*x_ref[2]
        ref[3,1,0,0]=(0.0780014276919)*x_ref[0]**o+(-0.0932830671471)*x_ref[0]+(0.39477236642)*x_ref[1]**o+(0.406673135556)*x_ref[1]+(-0.235480707568)*x_ref[2]**o+(-0.872302257144)*x_ref[2]
        ref[3,1,0,1]=(0.170573072646)*x_ref[0]**o+(0.868146561326)*x_ref[0]+(0.262136343896)*x_ref[1]**o+(0.192343529912)*x_ref[1]+(0.157400965693)*x_ref[2]**o+(-0.932981619473)*x_ref[2]
        ref[3,1,1,0]=(0.891433179291)*x_ref[0]**o+(-0.281396449997)*x_ref[0]+(-0.0803173619772)*x_ref[1]**o+(-0.378460054356)*x_ref[1]+(-0.581578656014)*x_ref[2]**o+(-0.65856067509)*x_ref[2]
        ref[3,1,1,1]=(0.343599956626)*x_ref[0]**o+(-0.92063694125)*x_ref[0]+(0.440698102072)*x_ref[1]**o+(0.767124166388)*x_ref[1]+(0.050964317543)*x_ref[2]**o+(1.35605562239e-05)*x_ref[2]
        ref[3,1,2,0]=(-0.614268150987)*x_ref[0]**o+(0.363806471556)*x_ref[0]+(0.92193457905)*x_ref[1]**o+(0.157919580642)*x_ref[1]+(-0.955575321124)*x_ref[2]**o+(-0.984808437666)*x_ref[2]
        ref[3,1,2,1]=(0.030099648966)*x_ref[0]**o+(-0.375999369491)*x_ref[0]+(-0.0992643926008)*x_ref[1]**o+(0.109209161093)*x_ref[1]+(-0.974043780088)*x_ref[2]**o+(0.908049658745)*x_ref[2]
        ref[3,2,0,0]=(0.297771341106)*x_ref[0]**o+(-0.637802228169)*x_ref[0]+(0.398100441783)*x_ref[1]**o+(-0.0643425181651)*x_ref[1]+(0.662775339033)*x_ref[2]**o+(0.744835182052)*x_ref[2]
        ref[3,2,0,1]=(0.312026420583)*x_ref[0]**o+(0.865855052574)*x_ref[0]+(0.0195001783165)*x_ref[1]**o+(0.537899238473)*x_ref[1]+(0.554893357481)*x_ref[2]**o+(0.71555610145)*x_ref[2]
        ref[3,2,1,0]=(-0.199138146186)*x_ref[0]**o+(0.827546489855)*x_ref[0]+(-0.480965464736)*x_ref[1]**o+(-0.473796093187)*x_ref[1]+(0.880264675049)*x_ref[2]**o+(0.544506670844)*x_ref[2]
        ref[3,2,1,1]=(0.773798005008)*x_ref[0]**o+(0.744945241634)*x_ref[0]+(0.548170564608)*x_ref[1]**o+(-0.560503692823)*x_ref[1]+(-0.435630215427)*x_ref[2]**o+(0.080015730707)*x_ref[2]
        ref[3,2,2,0]=(0.36735714937)*x_ref[0]**o+(-0.867071557873)*x_ref[0]+(0.936135650431)*x_ref[1]**o+(-0.432533600524)*x_ref[1]+(-0.0224430204069)*x_ref[2]**o+(0.218590302621)*x_ref[2]
        ref[3,2,2,1]=(0.932808382499)*x_ref[0]**o+(-0.0970016713319)*x_ref[0]+(0.458943467579)*x_ref[1]**o+(-0.134338789898)*x_ref[1]+(-0.426892557314)*x_ref[2]**o+(0.627071826863)*x_ref[2]
        ref[3,3,0,0]=(-0.623804149075)*x_ref[0]**o+(0.910423590172)*x_ref[0]+(0.465938181097)*x_ref[1]**o+(0.495479245391)*x_ref[1]+(0.693745442185)*x_ref[2]**o+(-0.437014226757)*x_ref[2]
        ref[3,3,0,1]=(0.598807017496)*x_ref[0]**o+(-0.112361840211)*x_ref[0]+(-0.550899685213)*x_ref[1]**o+(-0.0478222019564)*x_ref[1]+(-0.0110129517693)*x_ref[2]**o+(-0.059534786374)*x_ref[2]
        ref[3,3,1,0]=(-0.561155633409)*x_ref[0]**o+(-0.463563057636)*x_ref[0]+(-0.170463996009)*x_ref[1]**o+(-0.214509204942)*x_ref[1]+(0.851200198666)*x_ref[2]**o+(-0.721036329689)*x_ref[2]
        ref[3,3,1,1]=(0.608666240405)*x_ref[0]**o+(0.814232455115)*x_ref[0]+(-0.604645629689)*x_ref[1]**o+(0.960976509673)*x_ref[1]+(0.365807996219)*x_ref[2]**o+(-0.87491909238)*x_ref[2]
        ref[3,3,2,0]=(0.120126988418)*x_ref[0]**o+(-0.745687314825)*x_ref[0]+(-0.332697826984)*x_ref[1]**o+(-0.823683406516)*x_ref[1]+(0.305444079613)*x_ref[2]**o+(0.901972115441)*x_ref[2]
        ref[3,3,2,1]=(0.0470293848904)*x_ref[0]**o+(-0.195496754814)*x_ref[0]+(0.671870114105)*x_ref[1]**o+(0.170273146371)*x_ref[1]+(-0.851927986188)*x_ref[2]**o+(0.772192863973)*x_ref[2]
        ref[3,4,0,0]=(-0.95392341727)*x_ref[0]**o+(-0.665694215521)*x_ref[0]+(-0.0247396499378)*x_ref[1]**o+(0.441352857387)*x_ref[1]+(0.767156547791)*x_ref[2]**o+(-0.14696372018)*x_ref[2]
        ref[3,4,0,1]=(0.209761166815)*x_ref[0]**o+(0.978934473425)*x_ref[0]+(0.943806891053)*x_ref[1]**o+(-0.756619917601)*x_ref[1]+(0.635699949764)*x_ref[2]**o+(-0.345248269568)*x_ref[2]
        ref[3,4,1,0]=(0.748020785785)*x_ref[0]**o+(0.311104549542)*x_ref[0]+(0.739127551248)*x_ref[1]**o+(-0.635238575859)*x_ref[1]+(0.623898875653)*x_ref[2]**o+(0.343407392068)*x_ref[2]
        ref[3,4,1,1]=(-0.422500892471)*x_ref[0]**o+(0.0688495013346)*x_ref[0]+(-0.919653906232)*x_ref[1]**o+(0.531036586396)*x_ref[1]+(-0.22487910672)*x_ref[2]**o+(-0.848225994541)*x_ref[2]
        ref[3,4,2,0]=(0.723965744673)*x_ref[0]**o+(0.558602959224)*x_ref[0]+(0.699539243308)*x_ref[1]**o+(-0.597426541715)*x_ref[1]+(-0.0128675070522)*x_ref[2]**o+(-0.874796626242)*x_ref[2]
        ref[3,4,2,1]=(-0.962970021216)*x_ref[0]**o+(0.532886481874)*x_ref[0]+(-0.523159705965)*x_ref[1]**o+(-0.82887407638)*x_ref[1]+(0.874288116843)*x_ref[2]**o+(0.520024515804)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromData_Solution_rank0(self):
      """
      tests interpolate for rank 0 Data onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from Solution onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Data(0,(),w)
      ref=Data(0,(),w_ref)
      if dim==2:
        arg=(-0.0603413935246)*x[0]**o+(-0.70184894912)*x[0]+(-0.206341392562)*x[1]**o+(0.892585857385)*x[1]
        ref=(-0.0603413935246)*x_ref[0]**o+(-0.70184894912)*x_ref[0]+(-0.206341392562)*x_ref[1]**o+(0.892585857385)*x_ref[1]
      else:
        arg=(0.120763612793)*x[0]**o+(-0.298506957699)*x[0]+(0.0620633729217)*x[1]**o+(0.807000465544)*x[1]+(-0.564443939995)*x[2]**o+(0.820783035273)*x[2]
        ref=(0.120763612793)*x_ref[0]**o+(-0.298506957699)*x_ref[0]+(0.0620633729217)*x_ref[1]**o+(0.807000465544)*x_ref[1]+(-0.564443939995)*x_ref[2]**o+(0.820783035273)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromData_Solution_rank1(self):
      """
      tests interpolate for rank 1 Data onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from Solution onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Data(0,(2,),w)
      ref=Data(0,(2,),w_ref)
      if dim==2:
        arg[0]=(-0.69193390973)*x[0]**o+(-0.768888031033)*x[0]+(-0.813493496124)*x[1]**o+(-0.767059499773)*x[1]
        arg[1]=(0.111870073987)*x[0]**o+(-0.196406293238)*x[0]+(-0.10683601134)*x[1]**o+(0.69972228684)*x[1]
        ref[0]=(-0.69193390973)*x_ref[0]**o+(-0.768888031033)*x_ref[0]+(-0.813493496124)*x_ref[1]**o+(-0.767059499773)*x_ref[1]
        ref[1]=(0.111870073987)*x_ref[0]**o+(-0.196406293238)*x_ref[0]+(-0.10683601134)*x_ref[1]**o+(0.69972228684)*x_ref[1]
      else:
        arg[0]=(-0.20606732369)*x[0]**o+(0.773916734719)*x[0]+(-0.915594370888)*x[1]**o+(-0.684202475504)*x[1]+(0.161644377631)*x[2]**o+(-0.834568364942)*x[2]
        arg[1]=(-0.475848931399)*x[0]**o+(-0.967325026091)*x[0]+(0.371032251427)*x[1]**o+(0.976933923378)*x[1]+(0.326121409381)*x[2]**o+(0.880666151318)*x[2]
        ref[0]=(-0.20606732369)*x_ref[0]**o+(0.773916734719)*x_ref[0]+(-0.915594370888)*x_ref[1]**o+(-0.684202475504)*x_ref[1]+(0.161644377631)*x_ref[2]**o+(-0.834568364942)*x_ref[2]
        ref[1]=(-0.475848931399)*x_ref[0]**o+(-0.967325026091)*x_ref[0]+(0.371032251427)*x_ref[1]**o+(0.976933923378)*x_ref[1]+(0.326121409381)*x_ref[2]**o+(0.880666151318)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromData_Solution_rank2(self):
      """
      tests interpolate for rank 2 Data onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from Solution onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5),w)
      ref=Data(0,(4, 5),w_ref)
      if dim==2:
        arg[0,0]=(-0.664616598004)*x[0]**o+(0.159791014359)*x[0]+(-0.3747223278)*x[1]**o+(0.302802720584)*x[1]
        arg[0,1]=(-0.334311194265)*x[0]**o+(0.865855840771)*x[0]+(-0.0184933229609)*x[1]**o+(0.589277746683)*x[1]
        arg[0,2]=(0.685266031246)*x[0]**o+(0.846357835994)*x[0]+(0.233422031025)*x[1]**o+(0.391225449794)*x[1]
        arg[0,3]=(-0.889412463645)*x[0]**o+(0.336985584587)*x[0]+(-0.311629085429)*x[1]**o+(-0.574756125055)*x[1]
        arg[0,4]=(-0.786434255596)*x[0]**o+(0.897529643329)*x[0]+(0.112631455142)*x[1]**o+(0.501957817255)*x[1]
        arg[1,0]=(-0.841995669352)*x[0]**o+(-0.372241861224)*x[0]+(0.669429619157)*x[1]**o+(0.913007793035)*x[1]
        arg[1,1]=(-0.965486931849)*x[0]**o+(-0.163330142621)*x[0]+(0.302996628649)*x[1]**o+(-0.214176383433)*x[1]
        arg[1,2]=(-0.717841677595)*x[0]**o+(-0.560453345617)*x[0]+(0.885221610478)*x[1]**o+(0.0925933897521)*x[1]
        arg[1,3]=(0.827380987548)*x[0]**o+(-0.185460343082)*x[0]+(-0.618848726331)*x[1]**o+(-0.25427152262)*x[1]
        arg[1,4]=(-0.232848766689)*x[0]**o+(-0.812427149527)*x[0]+(-0.6328125174)*x[1]**o+(0.377849173478)*x[1]
        arg[2,0]=(0.725568000689)*x[0]**o+(-0.0119648722611)*x[0]+(0.0551179058345)*x[1]**o+(-0.313601095535)*x[1]
        arg[2,1]=(-0.371331280463)*x[0]**o+(0.164812693414)*x[0]+(0.140689113041)*x[1]**o+(0.866863122864)*x[1]
        arg[2,2]=(0.445443472696)*x[0]**o+(0.885420986537)*x[0]+(0.434326737693)*x[1]**o+(0.013677800309)*x[1]
        arg[2,3]=(0.186221146816)*x[0]**o+(0.919506151782)*x[0]+(-0.163957717851)*x[1]**o+(-0.533309231769)*x[1]
        arg[2,4]=(0.137164729623)*x[0]**o+(0.561715777859)*x[0]+(0.33724606372)*x[1]**o+(-0.554531877504)*x[1]
        arg[3,0]=(-0.0632399630146)*x[0]**o+(0.712014652956)*x[0]+(0.107109261787)*x[1]**o+(0.228670262705)*x[1]
        arg[3,1]=(-0.973793135038)*x[0]**o+(0.923465546039)*x[0]+(0.189825544105)*x[1]**o+(-0.33784496447)*x[1]
        arg[3,2]=(-0.12686258826)*x[0]**o+(-0.582653698893)*x[0]+(0.860503162549)*x[1]**o+(-0.477305983221)*x[1]
        arg[3,3]=(-0.407988306429)*x[0]**o+(-0.174396602789)*x[0]+(0.646815890222)*x[1]**o+(-0.450974163079)*x[1]
        arg[3,4]=(0.162458379489)*x[0]**o+(-0.0569354582388)*x[0]+(0.0463216806125)*x[1]**o+(-0.381777259635)*x[1]
        ref[0,0]=(-0.664616598004)*x_ref[0]**o+(0.159791014359)*x_ref[0]+(-0.3747223278)*x_ref[1]**o+(0.302802720584)*x_ref[1]
        ref[0,1]=(-0.334311194265)*x_ref[0]**o+(0.865855840771)*x_ref[0]+(-0.0184933229609)*x_ref[1]**o+(0.589277746683)*x_ref[1]
        ref[0,2]=(0.685266031246)*x_ref[0]**o+(0.846357835994)*x_ref[0]+(0.233422031025)*x_ref[1]**o+(0.391225449794)*x_ref[1]
        ref[0,3]=(-0.889412463645)*x_ref[0]**o+(0.336985584587)*x_ref[0]+(-0.311629085429)*x_ref[1]**o+(-0.574756125055)*x_ref[1]
        ref[0,4]=(-0.786434255596)*x_ref[0]**o+(0.897529643329)*x_ref[0]+(0.112631455142)*x_ref[1]**o+(0.501957817255)*x_ref[1]
        ref[1,0]=(-0.841995669352)*x_ref[0]**o+(-0.372241861224)*x_ref[0]+(0.669429619157)*x_ref[1]**o+(0.913007793035)*x_ref[1]
        ref[1,1]=(-0.965486931849)*x_ref[0]**o+(-0.163330142621)*x_ref[0]+(0.302996628649)*x_ref[1]**o+(-0.214176383433)*x_ref[1]
        ref[1,2]=(-0.717841677595)*x_ref[0]**o+(-0.560453345617)*x_ref[0]+(0.885221610478)*x_ref[1]**o+(0.0925933897521)*x_ref[1]
        ref[1,3]=(0.827380987548)*x_ref[0]**o+(-0.185460343082)*x_ref[0]+(-0.618848726331)*x_ref[1]**o+(-0.25427152262)*x_ref[1]
        ref[1,4]=(-0.232848766689)*x_ref[0]**o+(-0.812427149527)*x_ref[0]+(-0.6328125174)*x_ref[1]**o+(0.377849173478)*x_ref[1]
        ref[2,0]=(0.725568000689)*x_ref[0]**o+(-0.0119648722611)*x_ref[0]+(0.0551179058345)*x_ref[1]**o+(-0.313601095535)*x_ref[1]
        ref[2,1]=(-0.371331280463)*x_ref[0]**o+(0.164812693414)*x_ref[0]+(0.140689113041)*x_ref[1]**o+(0.866863122864)*x_ref[1]
        ref[2,2]=(0.445443472696)*x_ref[0]**o+(0.885420986537)*x_ref[0]+(0.434326737693)*x_ref[1]**o+(0.013677800309)*x_ref[1]
        ref[2,3]=(0.186221146816)*x_ref[0]**o+(0.919506151782)*x_ref[0]+(-0.163957717851)*x_ref[1]**o+(-0.533309231769)*x_ref[1]
        ref[2,4]=(0.137164729623)*x_ref[0]**o+(0.561715777859)*x_ref[0]+(0.33724606372)*x_ref[1]**o+(-0.554531877504)*x_ref[1]
        ref[3,0]=(-0.0632399630146)*x_ref[0]**o+(0.712014652956)*x_ref[0]+(0.107109261787)*x_ref[1]**o+(0.228670262705)*x_ref[1]
        ref[3,1]=(-0.973793135038)*x_ref[0]**o+(0.923465546039)*x_ref[0]+(0.189825544105)*x_ref[1]**o+(-0.33784496447)*x_ref[1]
        ref[3,2]=(-0.12686258826)*x_ref[0]**o+(-0.582653698893)*x_ref[0]+(0.860503162549)*x_ref[1]**o+(-0.477305983221)*x_ref[1]
        ref[3,3]=(-0.407988306429)*x_ref[0]**o+(-0.174396602789)*x_ref[0]+(0.646815890222)*x_ref[1]**o+(-0.450974163079)*x_ref[1]
        ref[3,4]=(0.162458379489)*x_ref[0]**o+(-0.0569354582388)*x_ref[0]+(0.0463216806125)*x_ref[1]**o+(-0.381777259635)*x_ref[1]
      else:
        arg[0,0]=(0.865079887449)*x[0]**o+(-0.833757275547)*x[0]+(-0.967658088049)*x[1]**o+(-0.168314490396)*x[1]+(0.151618201772)*x[2]**o+(-0.251062354355)*x[2]
        arg[0,1]=(0.110207997589)*x[0]**o+(-0.510994295641)*x[0]+(0.855215902178)*x[1]**o+(0.941502315255)*x[1]+(-0.794874945971)*x[2]**o+(-0.309190709938)*x[2]
        arg[0,2]=(0.71701217173)*x[0]**o+(-0.221333357165)*x[0]+(0.689963224026)*x[1]**o+(0.221517236085)*x[1]+(-0.52608140393)*x[2]**o+(0.371488496481)*x[2]
        arg[0,3]=(0.618194041415)*x[0]**o+(0.454387091678)*x[0]+(-0.707329629946)*x[1]**o+(0.41972715681)*x[1]+(0.0180789814415)*x[2]**o+(-0.176487980225)*x[2]
        arg[0,4]=(0.329275243989)*x[0]**o+(-0.679005993002)*x[0]+(-0.261017436418)*x[1]**o+(0.151780687062)*x[1]+(-0.480936330475)*x[2]**o+(-0.697077234587)*x[2]
        arg[1,0]=(0.720913763906)*x[0]**o+(0.283486705277)*x[0]+(0.498289077088)*x[1]**o+(0.657543220863)*x[1]+(-0.666293219679)*x[2]**o+(0.901456618353)*x[2]
        arg[1,1]=(-0.984526350563)*x[0]**o+(0.677118292603)*x[0]+(0.614707818107)*x[1]**o+(0.272173514554)*x[1]+(0.41492971828)*x[2]**o+(0.274511869871)*x[2]
        arg[1,2]=(0.322224985373)*x[0]**o+(0.567272204545)*x[0]+(-0.437313912332)*x[1]**o+(0.847486025896)*x[1]+(0.769434908781)*x[2]**o+(0.325917607497)*x[2]
        arg[1,3]=(0.709357837264)*x[0]**o+(0.927425103914)*x[0]+(-0.622876388501)*x[1]**o+(-0.296141603555)*x[1]+(-0.038448878923)*x[2]**o+(-0.320692580169)*x[2]
        arg[1,4]=(-0.27890333885)*x[0]**o+(0.803346569394)*x[0]+(0.00665934218255)*x[1]**o+(-0.89057979428)*x[1]+(-0.727135553706)*x[2]**o+(0.892946022497)*x[2]
        arg[2,0]=(0.579951394388)*x[0]**o+(0.0406506913385)*x[0]+(0.363944368798)*x[1]**o+(0.582732756951)*x[1]+(0.542548838027)*x[2]**o+(0.365742797343)*x[2]
        arg[2,1]=(0.253260777874)*x[0]**o+(-0.69077350875)*x[0]+(0.458080700492)*x[1]**o+(-0.79224006672)*x[1]+(-0.504865679524)*x[2]**o+(0.681425632809)*x[2]
        arg[2,2]=(0.600427718261)*x[0]**o+(0.91761242151)*x[0]+(0.602316047463)*x[1]**o+(0.890383614185)*x[1]+(-0.386691039525)*x[2]**o+(0.688659949109)*x[2]
        arg[2,3]=(-0.181338810914)*x[0]**o+(0.933744241999)*x[0]+(0.0212774093137)*x[1]**o+(0.284972854884)*x[1]+(-0.150219922419)*x[2]**o+(0.0878011402107)*x[2]
        arg[2,4]=(-0.848985879538)*x[0]**o+(0.298962025531)*x[0]+(0.593300576179)*x[1]**o+(0.301509253541)*x[1]+(0.971171084219)*x[2]**o+(-0.973386482857)*x[2]
        arg[3,0]=(-0.735608389534)*x[0]**o+(0.44902436426)*x[0]+(0.0536456095512)*x[1]**o+(0.208969935548)*x[1]+(-0.787609300211)*x[2]**o+(0.726024812477)*x[2]
        arg[3,1]=(0.200882171943)*x[0]**o+(-0.644461780745)*x[0]+(-0.0995704510264)*x[1]**o+(-0.857730241944)*x[1]+(-0.372302969302)*x[2]**o+(-0.171367954739)*x[2]
        arg[3,2]=(-0.673606332627)*x[0]**o+(-0.828183678442)*x[0]+(-0.236856099365)*x[1]**o+(-0.266339053288)*x[1]+(-0.972034740204)*x[2]**o+(-0.33151750348)*x[2]
        arg[3,3]=(-0.312470720017)*x[0]**o+(-0.657433718629)*x[0]+(-0.984940036612)*x[1]**o+(0.197707053145)*x[1]+(0.512009064643)*x[2]**o+(0.855601033248)*x[2]
        arg[3,4]=(-0.674854508228)*x[0]**o+(-0.394035682831)*x[0]+(-0.749446232511)*x[1]**o+(0.866992763518)*x[1]+(-0.945837361158)*x[2]**o+(0.593490782481)*x[2]
        ref[0,0]=(0.865079887449)*x_ref[0]**o+(-0.833757275547)*x_ref[0]+(-0.967658088049)*x_ref[1]**o+(-0.168314490396)*x_ref[1]+(0.151618201772)*x_ref[2]**o+(-0.251062354355)*x_ref[2]
        ref[0,1]=(0.110207997589)*x_ref[0]**o+(-0.510994295641)*x_ref[0]+(0.855215902178)*x_ref[1]**o+(0.941502315255)*x_ref[1]+(-0.794874945971)*x_ref[2]**o+(-0.309190709938)*x_ref[2]
        ref[0,2]=(0.71701217173)*x_ref[0]**o+(-0.221333357165)*x_ref[0]+(0.689963224026)*x_ref[1]**o+(0.221517236085)*x_ref[1]+(-0.52608140393)*x_ref[2]**o+(0.371488496481)*x_ref[2]
        ref[0,3]=(0.618194041415)*x_ref[0]**o+(0.454387091678)*x_ref[0]+(-0.707329629946)*x_ref[1]**o+(0.41972715681)*x_ref[1]+(0.0180789814415)*x_ref[2]**o+(-0.176487980225)*x_ref[2]
        ref[0,4]=(0.329275243989)*x_ref[0]**o+(-0.679005993002)*x_ref[0]+(-0.261017436418)*x_ref[1]**o+(0.151780687062)*x_ref[1]+(-0.480936330475)*x_ref[2]**o+(-0.697077234587)*x_ref[2]
        ref[1,0]=(0.720913763906)*x_ref[0]**o+(0.283486705277)*x_ref[0]+(0.498289077088)*x_ref[1]**o+(0.657543220863)*x_ref[1]+(-0.666293219679)*x_ref[2]**o+(0.901456618353)*x_ref[2]
        ref[1,1]=(-0.984526350563)*x_ref[0]**o+(0.677118292603)*x_ref[0]+(0.614707818107)*x_ref[1]**o+(0.272173514554)*x_ref[1]+(0.41492971828)*x_ref[2]**o+(0.274511869871)*x_ref[2]
        ref[1,2]=(0.322224985373)*x_ref[0]**o+(0.567272204545)*x_ref[0]+(-0.437313912332)*x_ref[1]**o+(0.847486025896)*x_ref[1]+(0.769434908781)*x_ref[2]**o+(0.325917607497)*x_ref[2]
        ref[1,3]=(0.709357837264)*x_ref[0]**o+(0.927425103914)*x_ref[0]+(-0.622876388501)*x_ref[1]**o+(-0.296141603555)*x_ref[1]+(-0.038448878923)*x_ref[2]**o+(-0.320692580169)*x_ref[2]
        ref[1,4]=(-0.27890333885)*x_ref[0]**o+(0.803346569394)*x_ref[0]+(0.00665934218255)*x_ref[1]**o+(-0.89057979428)*x_ref[1]+(-0.727135553706)*x_ref[2]**o+(0.892946022497)*x_ref[2]
        ref[2,0]=(0.579951394388)*x_ref[0]**o+(0.0406506913385)*x_ref[0]+(0.363944368798)*x_ref[1]**o+(0.582732756951)*x_ref[1]+(0.542548838027)*x_ref[2]**o+(0.365742797343)*x_ref[2]
        ref[2,1]=(0.253260777874)*x_ref[0]**o+(-0.69077350875)*x_ref[0]+(0.458080700492)*x_ref[1]**o+(-0.79224006672)*x_ref[1]+(-0.504865679524)*x_ref[2]**o+(0.681425632809)*x_ref[2]
        ref[2,2]=(0.600427718261)*x_ref[0]**o+(0.91761242151)*x_ref[0]+(0.602316047463)*x_ref[1]**o+(0.890383614185)*x_ref[1]+(-0.386691039525)*x_ref[2]**o+(0.688659949109)*x_ref[2]
        ref[2,3]=(-0.181338810914)*x_ref[0]**o+(0.933744241999)*x_ref[0]+(0.0212774093137)*x_ref[1]**o+(0.284972854884)*x_ref[1]+(-0.150219922419)*x_ref[2]**o+(0.0878011402107)*x_ref[2]
        ref[2,4]=(-0.848985879538)*x_ref[0]**o+(0.298962025531)*x_ref[0]+(0.593300576179)*x_ref[1]**o+(0.301509253541)*x_ref[1]+(0.971171084219)*x_ref[2]**o+(-0.973386482857)*x_ref[2]
        ref[3,0]=(-0.735608389534)*x_ref[0]**o+(0.44902436426)*x_ref[0]+(0.0536456095512)*x_ref[1]**o+(0.208969935548)*x_ref[1]+(-0.787609300211)*x_ref[2]**o+(0.726024812477)*x_ref[2]
        ref[3,1]=(0.200882171943)*x_ref[0]**o+(-0.644461780745)*x_ref[0]+(-0.0995704510264)*x_ref[1]**o+(-0.857730241944)*x_ref[1]+(-0.372302969302)*x_ref[2]**o+(-0.171367954739)*x_ref[2]
        ref[3,2]=(-0.673606332627)*x_ref[0]**o+(-0.828183678442)*x_ref[0]+(-0.236856099365)*x_ref[1]**o+(-0.266339053288)*x_ref[1]+(-0.972034740204)*x_ref[2]**o+(-0.33151750348)*x_ref[2]
        ref[3,3]=(-0.312470720017)*x_ref[0]**o+(-0.657433718629)*x_ref[0]+(-0.984940036612)*x_ref[1]**o+(0.197707053145)*x_ref[1]+(0.512009064643)*x_ref[2]**o+(0.855601033248)*x_ref[2]
        ref[3,4]=(-0.674854508228)*x_ref[0]**o+(-0.394035682831)*x_ref[0]+(-0.749446232511)*x_ref[1]**o+(0.866992763518)*x_ref[1]+(-0.945837361158)*x_ref[2]**o+(0.593490782481)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromData_Solution_rank3(self):
      """
      tests interpolate for rank 3 Data onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from Solution onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2),w_ref)
      if dim==2:
        arg[0,0,0]=(-0.511844892619)*x[0]**o+(0.72447480361)*x[0]+(-0.415725587152)*x[1]**o+(-0.495233818217)*x[1]
        arg[0,0,1]=(0.513861713949)*x[0]**o+(-0.918192893835)*x[0]+(0.563871531176)*x[1]**o+(0.885768393284)*x[1]
        arg[0,1,0]=(-0.412320762031)*x[0]**o+(0.576776422795)*x[0]+(0.0103535112247)*x[1]**o+(-0.738144219037)*x[1]
        arg[0,1,1]=(0.69941105604)*x[0]**o+(0.923470022558)*x[0]+(0.568449180547)*x[1]**o+(0.361429609343)*x[1]
        arg[1,0,0]=(-0.574139432427)*x[0]**o+(0.968980609212)*x[0]+(-0.198809151818)*x[1]**o+(0.751623099192)*x[1]
        arg[1,0,1]=(-0.743433955873)*x[0]**o+(-0.608013190658)*x[0]+(-0.0625473402572)*x[1]**o+(0.168418241981)*x[1]
        arg[1,1,0]=(-0.632180449517)*x[0]**o+(-0.892666803222)*x[0]+(-0.558127678438)*x[1]**o+(0.212462582041)*x[1]
        arg[1,1,1]=(-0.666381769622)*x[0]**o+(-0.0299308846938)*x[0]+(-0.450221481906)*x[1]**o+(0.701099654576)*x[1]
        arg[2,0,0]=(-0.927832262778)*x[0]**o+(0.901665098705)*x[0]+(-0.0462955000541)*x[1]**o+(-0.92139600993)*x[1]
        arg[2,0,1]=(-0.813732112672)*x[0]**o+(0.569194058283)*x[0]+(-0.527298120198)*x[1]**o+(-0.531607593369)*x[1]
        arg[2,1,0]=(-0.760238058111)*x[0]**o+(0.628718839289)*x[0]+(-0.420066582474)*x[1]**o+(-0.205537665056)*x[1]
        arg[2,1,1]=(0.421784629744)*x[0]**o+(-0.781451031751)*x[0]+(-0.0401156726902)*x[1]**o+(0.437044655245)*x[1]
        arg[3,0,0]=(0.324136561188)*x[0]**o+(-0.920235828584)*x[0]+(0.677675509043)*x[1]**o+(0.322337884254)*x[1]
        arg[3,0,1]=(-0.0711813236306)*x[0]**o+(0.0770312838088)*x[0]+(0.259613502243)*x[1]**o+(0.508154611576)*x[1]
        arg[3,1,0]=(0.944048224599)*x[0]**o+(0.039954080407)*x[0]+(-0.503309434151)*x[1]**o+(0.747109874504)*x[1]
        arg[3,1,1]=(0.692616323513)*x[0]**o+(0.0114568310884)*x[0]+(0.0708318026263)*x[1]**o+(0.822167043168)*x[1]
        arg[4,0,0]=(0.392645136008)*x[0]**o+(0.160054219263)*x[0]+(-0.32320145108)*x[1]**o+(0.598963113077)*x[1]
        arg[4,0,1]=(0.124778736846)*x[0]**o+(0.949589553291)*x[0]+(0.0985585514202)*x[1]**o+(-0.536684039884)*x[1]
        arg[4,1,0]=(-0.983690591076)*x[0]**o+(-0.115321512368)*x[0]+(0.0875590878092)*x[1]**o+(0.992162964289)*x[1]
        arg[4,1,1]=(-0.89264628952)*x[0]**o+(0.414589880453)*x[0]+(-0.40959764673)*x[1]**o+(-0.261016337239)*x[1]
        arg[5,0,0]=(-0.613553175002)*x[0]**o+(0.320685999723)*x[0]+(-0.523099455238)*x[1]**o+(0.855780524769)*x[1]
        arg[5,0,1]=(-0.45911360029)*x[0]**o+(-0.285942390871)*x[0]+(0.7279702976)*x[1]**o+(0.981483922949)*x[1]
        arg[5,1,0]=(0.577254626037)*x[0]**o+(0.146911866492)*x[0]+(-0.414545328014)*x[1]**o+(-0.979179708006)*x[1]
        arg[5,1,1]=(-0.838392720256)*x[0]**o+(-0.51717223091)*x[0]+(0.635616756614)*x[1]**o+(0.112711970543)*x[1]
        ref[0,0,0]=(-0.511844892619)*x_ref[0]**o+(0.72447480361)*x_ref[0]+(-0.415725587152)*x_ref[1]**o+(-0.495233818217)*x_ref[1]
        ref[0,0,1]=(0.513861713949)*x_ref[0]**o+(-0.918192893835)*x_ref[0]+(0.563871531176)*x_ref[1]**o+(0.885768393284)*x_ref[1]
        ref[0,1,0]=(-0.412320762031)*x_ref[0]**o+(0.576776422795)*x_ref[0]+(0.0103535112247)*x_ref[1]**o+(-0.738144219037)*x_ref[1]
        ref[0,1,1]=(0.69941105604)*x_ref[0]**o+(0.923470022558)*x_ref[0]+(0.568449180547)*x_ref[1]**o+(0.361429609343)*x_ref[1]
        ref[1,0,0]=(-0.574139432427)*x_ref[0]**o+(0.968980609212)*x_ref[0]+(-0.198809151818)*x_ref[1]**o+(0.751623099192)*x_ref[1]
        ref[1,0,1]=(-0.743433955873)*x_ref[0]**o+(-0.608013190658)*x_ref[0]+(-0.0625473402572)*x_ref[1]**o+(0.168418241981)*x_ref[1]
        ref[1,1,0]=(-0.632180449517)*x_ref[0]**o+(-0.892666803222)*x_ref[0]+(-0.558127678438)*x_ref[1]**o+(0.212462582041)*x_ref[1]
        ref[1,1,1]=(-0.666381769622)*x_ref[0]**o+(-0.0299308846938)*x_ref[0]+(-0.450221481906)*x_ref[1]**o+(0.701099654576)*x_ref[1]
        ref[2,0,0]=(-0.927832262778)*x_ref[0]**o+(0.901665098705)*x_ref[0]+(-0.0462955000541)*x_ref[1]**o+(-0.92139600993)*x_ref[1]
        ref[2,0,1]=(-0.813732112672)*x_ref[0]**o+(0.569194058283)*x_ref[0]+(-0.527298120198)*x_ref[1]**o+(-0.531607593369)*x_ref[1]
        ref[2,1,0]=(-0.760238058111)*x_ref[0]**o+(0.628718839289)*x_ref[0]+(-0.420066582474)*x_ref[1]**o+(-0.205537665056)*x_ref[1]
        ref[2,1,1]=(0.421784629744)*x_ref[0]**o+(-0.781451031751)*x_ref[0]+(-0.0401156726902)*x_ref[1]**o+(0.437044655245)*x_ref[1]
        ref[3,0,0]=(0.324136561188)*x_ref[0]**o+(-0.920235828584)*x_ref[0]+(0.677675509043)*x_ref[1]**o+(0.322337884254)*x_ref[1]
        ref[3,0,1]=(-0.0711813236306)*x_ref[0]**o+(0.0770312838088)*x_ref[0]+(0.259613502243)*x_ref[1]**o+(0.508154611576)*x_ref[1]
        ref[3,1,0]=(0.944048224599)*x_ref[0]**o+(0.039954080407)*x_ref[0]+(-0.503309434151)*x_ref[1]**o+(0.747109874504)*x_ref[1]
        ref[3,1,1]=(0.692616323513)*x_ref[0]**o+(0.0114568310884)*x_ref[0]+(0.0708318026263)*x_ref[1]**o+(0.822167043168)*x_ref[1]
        ref[4,0,0]=(0.392645136008)*x_ref[0]**o+(0.160054219263)*x_ref[0]+(-0.32320145108)*x_ref[1]**o+(0.598963113077)*x_ref[1]
        ref[4,0,1]=(0.124778736846)*x_ref[0]**o+(0.949589553291)*x_ref[0]+(0.0985585514202)*x_ref[1]**o+(-0.536684039884)*x_ref[1]
        ref[4,1,0]=(-0.983690591076)*x_ref[0]**o+(-0.115321512368)*x_ref[0]+(0.0875590878092)*x_ref[1]**o+(0.992162964289)*x_ref[1]
        ref[4,1,1]=(-0.89264628952)*x_ref[0]**o+(0.414589880453)*x_ref[0]+(-0.40959764673)*x_ref[1]**o+(-0.261016337239)*x_ref[1]
        ref[5,0,0]=(-0.613553175002)*x_ref[0]**o+(0.320685999723)*x_ref[0]+(-0.523099455238)*x_ref[1]**o+(0.855780524769)*x_ref[1]
        ref[5,0,1]=(-0.45911360029)*x_ref[0]**o+(-0.285942390871)*x_ref[0]+(0.7279702976)*x_ref[1]**o+(0.981483922949)*x_ref[1]
        ref[5,1,0]=(0.577254626037)*x_ref[0]**o+(0.146911866492)*x_ref[0]+(-0.414545328014)*x_ref[1]**o+(-0.979179708006)*x_ref[1]
        ref[5,1,1]=(-0.838392720256)*x_ref[0]**o+(-0.51717223091)*x_ref[0]+(0.635616756614)*x_ref[1]**o+(0.112711970543)*x_ref[1]
      else:
        arg[0,0,0]=(0.807360140818)*x[0]**o+(-0.399141641865)*x[0]+(0.752910658124)*x[1]**o+(0.0998734354996)*x[1]+(-0.146785318133)*x[2]**o+(-0.393796605729)*x[2]
        arg[0,0,1]=(-0.502209573844)*x[0]**o+(0.501684846477)*x[0]+(0.634537290794)*x[1]**o+(0.481836589517)*x[1]+(-0.0221536339712)*x[2]**o+(-0.868693970024)*x[2]
        arg[0,1,0]=(-0.268854282014)*x[0]**o+(0.0516535086968)*x[0]+(-0.505997013698)*x[1]**o+(0.414205349613)*x[1]+(0.0119995662599)*x[2]**o+(0.584354222675)*x[2]
        arg[0,1,1]=(0.81923283939)*x[0]**o+(-0.174637610292)*x[0]+(0.283785251937)*x[1]**o+(0.716260504685)*x[1]+(-0.957279283093)*x[2]**o+(0.632750238692)*x[2]
        arg[1,0,0]=(0.601003888536)*x[0]**o+(-0.575011763358)*x[0]+(0.095159775614)*x[1]**o+(0.141200996742)*x[1]+(0.0173869440863)*x[2]**o+(0.478733688259)*x[2]
        arg[1,0,1]=(-0.783826426764)*x[0]**o+(-0.40604430175)*x[0]+(-0.4574879786)*x[1]**o+(0.020222886424)*x[1]+(0.225419750344)*x[2]**o+(-0.530026190372)*x[2]
        arg[1,1,0]=(0.542744341911)*x[0]**o+(-0.501169367089)*x[0]+(0.447661946532)*x[1]**o+(0.384528043888)*x[1]+(-0.599271261928)*x[2]**o+(-0.535447844217)*x[2]
        arg[1,1,1]=(-0.457163465824)*x[0]**o+(0.0475382663524)*x[0]+(0.369626614251)*x[1]**o+(0.1125159395)*x[1]+(-0.252442923096)*x[2]**o+(0.0717205095822)*x[2]
        arg[2,0,0]=(0.341536174626)*x[0]**o+(0.4408059057)*x[0]+(0.196877383546)*x[1]**o+(0.903072917866)*x[1]+(0.423403716275)*x[2]**o+(-0.521563534689)*x[2]
        arg[2,0,1]=(0.553187361262)*x[0]**o+(-0.539057536895)*x[0]+(-0.814990210449)*x[1]**o+(-0.418614789609)*x[1]+(0.857298489434)*x[2]**o+(0.629708672)*x[2]
        arg[2,1,0]=(0.494803348038)*x[0]**o+(0.143644562028)*x[0]+(-0.0425813919235)*x[1]**o+(-0.818562153097)*x[1]+(-0.730690839032)*x[2]**o+(-0.569426137866)*x[2]
        arg[2,1,1]=(-0.573254727715)*x[0]**o+(-0.536459946225)*x[0]+(-0.244717967781)*x[1]**o+(-0.67463285343)*x[1]+(-0.383959754049)*x[2]**o+(-0.87595126443)*x[2]
        arg[3,0,0]=(0.758065154533)*x[0]**o+(0.634049823109)*x[0]+(0.851352543082)*x[1]**o+(0.302790462394)*x[1]+(-0.920790730269)*x[2]**o+(0.460419670912)*x[2]
        arg[3,0,1]=(0.415682634939)*x[0]**o+(-0.664821834506)*x[0]+(-0.314068887264)*x[1]**o+(-0.322413102029)*x[1]+(-0.498768865759)*x[2]**o+(-0.165338012626)*x[2]
        arg[3,1,0]=(-0.398599595126)*x[0]**o+(0.744397086526)*x[0]+(-0.02321839482)*x[1]**o+(0.989743131098)*x[1]+(0.0713981107777)*x[2]**o+(-0.7862942966)*x[2]
        arg[3,1,1]=(0.90835870295)*x[0]**o+(0.285965355995)*x[0]+(-0.302780058578)*x[1]**o+(0.396923771953)*x[1]+(0.717312324488)*x[2]**o+(0.967488138744)*x[2]
        arg[4,0,0]=(-0.286827561624)*x[0]**o+(0.67973673275)*x[0]+(0.325169520671)*x[1]**o+(0.482002981846)*x[1]+(-0.826037251282)*x[2]**o+(-0.0567054906659)*x[2]
        arg[4,0,1]=(-0.667297458554)*x[0]**o+(-0.207369757609)*x[0]+(-0.982746021703)*x[1]**o+(0.963677089363)*x[1]+(0.917328237047)*x[2]**o+(-0.684386577728)*x[2]
        arg[4,1,0]=(-0.835525763762)*x[0]**o+(-0.113324184435)*x[0]+(0.465855661969)*x[1]**o+(0.656047307997)*x[1]+(0.331675778117)*x[2]**o+(-0.686067177565)*x[2]
        arg[4,1,1]=(-0.468668496499)*x[0]**o+(0.295197251458)*x[0]+(0.065102699741)*x[1]**o+(-0.378381176309)*x[1]+(0.933627973066)*x[2]**o+(-0.91462136931)*x[2]
        arg[5,0,0]=(-0.161916994488)*x[0]**o+(0.3070005414)*x[0]+(0.195946524325)*x[1]**o+(-0.453290146527)*x[1]+(0.751532065651)*x[2]**o+(0.82676125654)*x[2]
        arg[5,0,1]=(-0.925362363492)*x[0]**o+(-0.0295351854097)*x[0]+(0.533980764998)*x[1]**o+(0.128080489262)*x[1]+(0.502858052015)*x[2]**o+(0.813196540163)*x[2]
        arg[5,1,0]=(-0.740146099318)*x[0]**o+(0.27623528724)*x[0]+(-0.361620073401)*x[1]**o+(-0.310534085108)*x[1]+(-0.0363428185757)*x[2]**o+(-0.145397849357)*x[2]
        arg[5,1,1]=(0.362089087335)*x[0]**o+(0.539359659929)*x[0]+(0.00904990097799)*x[1]**o+(-0.150401175722)*x[1]+(-0.20465882105)*x[2]**o+(-0.519331232192)*x[2]
        ref[0,0,0]=(0.807360140818)*x_ref[0]**o+(-0.399141641865)*x_ref[0]+(0.752910658124)*x_ref[1]**o+(0.0998734354996)*x_ref[1]+(-0.146785318133)*x_ref[2]**o+(-0.393796605729)*x_ref[2]
        ref[0,0,1]=(-0.502209573844)*x_ref[0]**o+(0.501684846477)*x_ref[0]+(0.634537290794)*x_ref[1]**o+(0.481836589517)*x_ref[1]+(-0.0221536339712)*x_ref[2]**o+(-0.868693970024)*x_ref[2]
        ref[0,1,0]=(-0.268854282014)*x_ref[0]**o+(0.0516535086968)*x_ref[0]+(-0.505997013698)*x_ref[1]**o+(0.414205349613)*x_ref[1]+(0.0119995662599)*x_ref[2]**o+(0.584354222675)*x_ref[2]
        ref[0,1,1]=(0.81923283939)*x_ref[0]**o+(-0.174637610292)*x_ref[0]+(0.283785251937)*x_ref[1]**o+(0.716260504685)*x_ref[1]+(-0.957279283093)*x_ref[2]**o+(0.632750238692)*x_ref[2]
        ref[1,0,0]=(0.601003888536)*x_ref[0]**o+(-0.575011763358)*x_ref[0]+(0.095159775614)*x_ref[1]**o+(0.141200996742)*x_ref[1]+(0.0173869440863)*x_ref[2]**o+(0.478733688259)*x_ref[2]
        ref[1,0,1]=(-0.783826426764)*x_ref[0]**o+(-0.40604430175)*x_ref[0]+(-0.4574879786)*x_ref[1]**o+(0.020222886424)*x_ref[1]+(0.225419750344)*x_ref[2]**o+(-0.530026190372)*x_ref[2]
        ref[1,1,0]=(0.542744341911)*x_ref[0]**o+(-0.501169367089)*x_ref[0]+(0.447661946532)*x_ref[1]**o+(0.384528043888)*x_ref[1]+(-0.599271261928)*x_ref[2]**o+(-0.535447844217)*x_ref[2]
        ref[1,1,1]=(-0.457163465824)*x_ref[0]**o+(0.0475382663524)*x_ref[0]+(0.369626614251)*x_ref[1]**o+(0.1125159395)*x_ref[1]+(-0.252442923096)*x_ref[2]**o+(0.0717205095822)*x_ref[2]
        ref[2,0,0]=(0.341536174626)*x_ref[0]**o+(0.4408059057)*x_ref[0]+(0.196877383546)*x_ref[1]**o+(0.903072917866)*x_ref[1]+(0.423403716275)*x_ref[2]**o+(-0.521563534689)*x_ref[2]
        ref[2,0,1]=(0.553187361262)*x_ref[0]**o+(-0.539057536895)*x_ref[0]+(-0.814990210449)*x_ref[1]**o+(-0.418614789609)*x_ref[1]+(0.857298489434)*x_ref[2]**o+(0.629708672)*x_ref[2]
        ref[2,1,0]=(0.494803348038)*x_ref[0]**o+(0.143644562028)*x_ref[0]+(-0.0425813919235)*x_ref[1]**o+(-0.818562153097)*x_ref[1]+(-0.730690839032)*x_ref[2]**o+(-0.569426137866)*x_ref[2]
        ref[2,1,1]=(-0.573254727715)*x_ref[0]**o+(-0.536459946225)*x_ref[0]+(-0.244717967781)*x_ref[1]**o+(-0.67463285343)*x_ref[1]+(-0.383959754049)*x_ref[2]**o+(-0.87595126443)*x_ref[2]
        ref[3,0,0]=(0.758065154533)*x_ref[0]**o+(0.634049823109)*x_ref[0]+(0.851352543082)*x_ref[1]**o+(0.302790462394)*x_ref[1]+(-0.920790730269)*x_ref[2]**o+(0.460419670912)*x_ref[2]
        ref[3,0,1]=(0.415682634939)*x_ref[0]**o+(-0.664821834506)*x_ref[0]+(-0.314068887264)*x_ref[1]**o+(-0.322413102029)*x_ref[1]+(-0.498768865759)*x_ref[2]**o+(-0.165338012626)*x_ref[2]
        ref[3,1,0]=(-0.398599595126)*x_ref[0]**o+(0.744397086526)*x_ref[0]+(-0.02321839482)*x_ref[1]**o+(0.989743131098)*x_ref[1]+(0.0713981107777)*x_ref[2]**o+(-0.7862942966)*x_ref[2]
        ref[3,1,1]=(0.90835870295)*x_ref[0]**o+(0.285965355995)*x_ref[0]+(-0.302780058578)*x_ref[1]**o+(0.396923771953)*x_ref[1]+(0.717312324488)*x_ref[2]**o+(0.967488138744)*x_ref[2]
        ref[4,0,0]=(-0.286827561624)*x_ref[0]**o+(0.67973673275)*x_ref[0]+(0.325169520671)*x_ref[1]**o+(0.482002981846)*x_ref[1]+(-0.826037251282)*x_ref[2]**o+(-0.0567054906659)*x_ref[2]
        ref[4,0,1]=(-0.667297458554)*x_ref[0]**o+(-0.207369757609)*x_ref[0]+(-0.982746021703)*x_ref[1]**o+(0.963677089363)*x_ref[1]+(0.917328237047)*x_ref[2]**o+(-0.684386577728)*x_ref[2]
        ref[4,1,0]=(-0.835525763762)*x_ref[0]**o+(-0.113324184435)*x_ref[0]+(0.465855661969)*x_ref[1]**o+(0.656047307997)*x_ref[1]+(0.331675778117)*x_ref[2]**o+(-0.686067177565)*x_ref[2]
        ref[4,1,1]=(-0.468668496499)*x_ref[0]**o+(0.295197251458)*x_ref[0]+(0.065102699741)*x_ref[1]**o+(-0.378381176309)*x_ref[1]+(0.933627973066)*x_ref[2]**o+(-0.91462136931)*x_ref[2]
        ref[5,0,0]=(-0.161916994488)*x_ref[0]**o+(0.3070005414)*x_ref[0]+(0.195946524325)*x_ref[1]**o+(-0.453290146527)*x_ref[1]+(0.751532065651)*x_ref[2]**o+(0.82676125654)*x_ref[2]
        ref[5,0,1]=(-0.925362363492)*x_ref[0]**o+(-0.0295351854097)*x_ref[0]+(0.533980764998)*x_ref[1]**o+(0.128080489262)*x_ref[1]+(0.502858052015)*x_ref[2]**o+(0.813196540163)*x_ref[2]
        ref[5,1,0]=(-0.740146099318)*x_ref[0]**o+(0.27623528724)*x_ref[0]+(-0.361620073401)*x_ref[1]**o+(-0.310534085108)*x_ref[1]+(-0.0363428185757)*x_ref[2]**o+(-0.145397849357)*x_ref[2]
        ref[5,1,1]=(0.362089087335)*x_ref[0]**o+(0.539359659929)*x_ref[0]+(0.00904990097799)*x_ref[1]**o+(-0.150401175722)*x_ref[1]+(-0.20465882105)*x_ref[2]**o+(-0.519331232192)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromData_Solution_rank4(self):
      """
      tests interpolate for rank 4 Data onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from Solution onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5, 3, 2),w)
      ref=Data(0,(4, 5, 3, 2),w_ref)
      if dim==2:
        arg[0,0,0,0]=(0.14016992168)*x[0]**o+(-0.396856222091)*x[0]+(0.747446299477)*x[1]**o+(-0.299063042467)*x[1]
        arg[0,0,0,1]=(0.126297805139)*x[0]**o+(0.317089000483)*x[0]+(0.0418478916266)*x[1]**o+(-0.892825689806)*x[1]
        arg[0,0,1,0]=(0.127263485532)*x[0]**o+(0.400880552184)*x[0]+(-0.858713979216)*x[1]**o+(0.238693631738)*x[1]
        arg[0,0,1,1]=(-0.985703521737)*x[0]**o+(-0.0192845851979)*x[0]+(-0.442493677109)*x[1]**o+(0.188038174612)*x[1]
        arg[0,0,2,0]=(0.814303535641)*x[0]**o+(-0.201938227136)*x[0]+(-0.134135136581)*x[1]**o+(0.939272592089)*x[1]
        arg[0,0,2,1]=(-0.609570412575)*x[0]**o+(0.181669701944)*x[0]+(0.882708213835)*x[1]**o+(0.315561722535)*x[1]
        arg[0,1,0,0]=(0.180272217085)*x[0]**o+(-0.921231698951)*x[0]+(-0.670170775864)*x[1]**o+(0.36959006719)*x[1]
        arg[0,1,0,1]=(-0.907447754449)*x[0]**o+(0.984356686322)*x[0]+(0.300621254386)*x[1]**o+(0.491995830235)*x[1]
        arg[0,1,1,0]=(-0.535808887861)*x[0]**o+(0.991647965865)*x[0]+(-0.505667913884)*x[1]**o+(-0.403245981754)*x[1]
        arg[0,1,1,1]=(-0.315687983887)*x[0]**o+(-0.570118136173)*x[0]+(0.327842038508)*x[1]**o+(-0.475833876716)*x[1]
        arg[0,1,2,0]=(0.598143043008)*x[0]**o+(0.119789453354)*x[0]+(-0.307994361027)*x[1]**o+(-0.747443620209)*x[1]
        arg[0,1,2,1]=(-0.384576138426)*x[0]**o+(-0.21431195392)*x[0]+(0.622157923623)*x[1]**o+(-0.535976986322)*x[1]
        arg[0,2,0,0]=(-0.823640169824)*x[0]**o+(-0.946436729867)*x[0]+(0.158819942161)*x[1]**o+(-0.175783146941)*x[1]
        arg[0,2,0,1]=(0.31586435609)*x[0]**o+(0.480823809413)*x[0]+(-0.155590673671)*x[1]**o+(-0.283774897002)*x[1]
        arg[0,2,1,0]=(0.973677960338)*x[0]**o+(-0.42119441385)*x[0]+(0.936434159723)*x[1]**o+(-0.271328279678)*x[1]
        arg[0,2,1,1]=(0.995977746066)*x[0]**o+(0.115566554312)*x[0]+(0.6023712777)*x[1]**o+(0.523708594385)*x[1]
        arg[0,2,2,0]=(0.690331991941)*x[0]**o+(0.708864222789)*x[0]+(-0.0209681974418)*x[1]**o+(-0.402664547241)*x[1]
        arg[0,2,2,1]=(0.793403379571)*x[0]**o+(-0.44195907862)*x[0]+(-0.750943552401)*x[1]**o+(0.87817105061)*x[1]
        arg[0,3,0,0]=(-0.960650344452)*x[0]**o+(0.918969286865)*x[0]+(-0.788611475275)*x[1]**o+(-0.917265819096)*x[1]
        arg[0,3,0,1]=(0.379234695969)*x[0]**o+(0.288784268667)*x[0]+(-0.231788371542)*x[1]**o+(-0.521488838978)*x[1]
        arg[0,3,1,0]=(-0.482353410226)*x[0]**o+(0.666269361063)*x[0]+(0.207628082266)*x[1]**o+(0.890536697637)*x[1]
        arg[0,3,1,1]=(-0.256826971067)*x[0]**o+(0.244573109457)*x[0]+(0.435182409565)*x[1]**o+(-0.98882467576)*x[1]
        arg[0,3,2,0]=(0.727681501266)*x[0]**o+(0.174117939756)*x[0]+(0.626998345198)*x[1]**o+(0.666518756047)*x[1]
        arg[0,3,2,1]=(-0.125855300157)*x[0]**o+(-0.750291117974)*x[0]+(0.840722993443)*x[1]**o+(0.0798848819291)*x[1]
        arg[0,4,0,0]=(-0.156808480304)*x[0]**o+(-0.864855000875)*x[0]+(-0.825786767174)*x[1]**o+(-0.563999207624)*x[1]
        arg[0,4,0,1]=(-0.890116284779)*x[0]**o+(0.795471417269)*x[0]+(0.326454331959)*x[1]**o+(-0.376644334859)*x[1]
        arg[0,4,1,0]=(-0.453181887597)*x[0]**o+(0.488149970589)*x[0]+(0.506352954527)*x[1]**o+(0.971016325918)*x[1]
        arg[0,4,1,1]=(0.156976484311)*x[0]**o+(0.988818166411)*x[0]+(0.948579296281)*x[1]**o+(-0.80303322078)*x[1]
        arg[0,4,2,0]=(-0.0625333956516)*x[0]**o+(0.357609677642)*x[0]+(0.00245182913993)*x[1]**o+(-0.162058851234)*x[1]
        arg[0,4,2,1]=(0.645504631508)*x[0]**o+(-0.0107543576265)*x[0]+(0.15490632437)*x[1]**o+(0.841985786269)*x[1]
        arg[1,0,0,0]=(0.703923364197)*x[0]**o+(-0.424818178123)*x[0]+(-0.744399116071)*x[1]**o+(0.303429987787)*x[1]
        arg[1,0,0,1]=(-0.92849671168)*x[0]**o+(0.050597602551)*x[0]+(0.91000243103)*x[1]**o+(-0.742775174195)*x[1]
        arg[1,0,1,0]=(0.281218131435)*x[0]**o+(0.171029842909)*x[0]+(-0.65806576709)*x[1]**o+(-0.718069456213)*x[1]
        arg[1,0,1,1]=(0.312255067232)*x[0]**o+(0.566401030665)*x[0]+(-0.941709776271)*x[1]**o+(0.0111896041808)*x[1]
        arg[1,0,2,0]=(-0.886019321094)*x[0]**o+(-0.688815756004)*x[0]+(-0.595491288141)*x[1]**o+(0.343402495621)*x[1]
        arg[1,0,2,1]=(-0.681063845863)*x[0]**o+(-0.676550403066)*x[0]+(0.186106532969)*x[1]**o+(0.181779537727)*x[1]
        arg[1,1,0,0]=(0.128588939989)*x[0]**o+(0.477711214734)*x[0]+(-0.17697460919)*x[1]**o+(0.0217208670244)*x[1]
        arg[1,1,0,1]=(-0.652028522296)*x[0]**o+(-0.503146425209)*x[0]+(-0.427973128189)*x[1]**o+(-0.182775422425)*x[1]
        arg[1,1,1,0]=(-0.0499349942904)*x[0]**o+(-0.0183804745613)*x[0]+(-0.912604094211)*x[1]**o+(-0.949181938689)*x[1]
        arg[1,1,1,1]=(-0.472737626621)*x[0]**o+(-0.142195314687)*x[0]+(0.580775749383)*x[1]**o+(-0.832549796746)*x[1]
        arg[1,1,2,0]=(-0.288756468146)*x[0]**o+(-0.50657056474)*x[0]+(-0.809713268333)*x[1]**o+(0.598450102365)*x[1]
        arg[1,1,2,1]=(-0.259779636241)*x[0]**o+(-0.227925001879)*x[0]+(-0.0752800702413)*x[1]**o+(-0.438841969161)*x[1]
        arg[1,2,0,0]=(-0.617835522076)*x[0]**o+(-0.969275786274)*x[0]+(0.899717033555)*x[1]**o+(0.77648944909)*x[1]
        arg[1,2,0,1]=(-0.0619395581419)*x[0]**o+(0.926394874574)*x[0]+(0.984605961354)*x[1]**o+(0.253347734181)*x[1]
        arg[1,2,1,0]=(0.748474342269)*x[0]**o+(0.0794003341104)*x[0]+(-0.244607682532)*x[1]**o+(-0.228787873564)*x[1]
        arg[1,2,1,1]=(-0.266853205536)*x[0]**o+(0.529714058158)*x[0]+(-0.421599080872)*x[1]**o+(-0.595557599245)*x[1]
        arg[1,2,2,0]=(-0.377962443753)*x[0]**o+(0.609550409525)*x[0]+(0.951904771097)*x[1]**o+(-0.123551801933)*x[1]
        arg[1,2,2,1]=(0.3262671136)*x[0]**o+(-0.574281078833)*x[0]+(0.308261022622)*x[1]**o+(-0.784762723843)*x[1]
        arg[1,3,0,0]=(-0.419601410829)*x[0]**o+(-0.576064062071)*x[0]+(-0.431574278091)*x[1]**o+(-0.264303418881)*x[1]
        arg[1,3,0,1]=(0.176058414693)*x[0]**o+(0.672752046411)*x[0]+(-0.8579992002)*x[1]**o+(-0.135904959185)*x[1]
        arg[1,3,1,0]=(0.84480795105)*x[0]**o+(0.846846799225)*x[0]+(0.134417432017)*x[1]**o+(-0.725316162128)*x[1]
        arg[1,3,1,1]=(-0.102162279538)*x[0]**o+(0.451180031789)*x[0]+(0.490175190736)*x[1]**o+(0.811823281177)*x[1]
        arg[1,3,2,0]=(-0.582177348293)*x[0]**o+(-0.74631626712)*x[0]+(-0.275524221012)*x[1]**o+(-0.658276720144)*x[1]
        arg[1,3,2,1]=(0.304511057297)*x[0]**o+(-0.354531411968)*x[0]+(0.0944491594546)*x[1]**o+(0.619039121597)*x[1]
        arg[1,4,0,0]=(0.754898520324)*x[0]**o+(-0.28914218119)*x[0]+(0.8660675945)*x[1]**o+(0.526777161228)*x[1]
        arg[1,4,0,1]=(-0.820150637429)*x[0]**o+(-0.399203443857)*x[0]+(0.118863382167)*x[1]**o+(0.269335037373)*x[1]
        arg[1,4,1,0]=(-0.669389232967)*x[0]**o+(-0.119080399209)*x[0]+(0.999919873616)*x[1]**o+(0.733390578378)*x[1]
        arg[1,4,1,1]=(-0.85147101348)*x[0]**o+(-0.0453996523642)*x[0]+(0.00891698330334)*x[1]**o+(-0.27128670805)*x[1]
        arg[1,4,2,0]=(-0.946558097166)*x[0]**o+(0.0141300604882)*x[0]+(0.04075090617)*x[1]**o+(-0.141095386138)*x[1]
        arg[1,4,2,1]=(0.613700159605)*x[0]**o+(0.115107829201)*x[0]+(0.907429968641)*x[1]**o+(0.901820725424)*x[1]
        arg[2,0,0,0]=(-0.629486588984)*x[0]**o+(-0.590908868493)*x[0]+(0.689333052519)*x[1]**o+(0.22096464697)*x[1]
        arg[2,0,0,1]=(-0.475364708492)*x[0]**o+(-0.3195126467)*x[0]+(-0.688082519853)*x[1]**o+(-0.889722517035)*x[1]
        arg[2,0,1,0]=(0.931261765284)*x[0]**o+(0.399101548845)*x[0]+(-0.185342345742)*x[1]**o+(0.188927724838)*x[1]
        arg[2,0,1,1]=(-0.195889209768)*x[0]**o+(-0.0878952476483)*x[0]+(0.636980893598)*x[1]**o+(0.827709469156)*x[1]
        arg[2,0,2,0]=(0.644658731705)*x[0]**o+(0.275771474382)*x[0]+(0.0802903363405)*x[1]**o+(-0.157785300253)*x[1]
        arg[2,0,2,1]=(0.324153739483)*x[0]**o+(-0.978043993375)*x[0]+(0.440060321393)*x[1]**o+(0.103952166755)*x[1]
        arg[2,1,0,0]=(-0.233567312787)*x[0]**o+(0.139529308019)*x[0]+(-0.200168759944)*x[1]**o+(-0.629057762953)*x[1]
        arg[2,1,0,1]=(-0.407808993281)*x[0]**o+(-0.208049914815)*x[0]+(0.676817990818)*x[1]**o+(-0.27174220248)*x[1]
        arg[2,1,1,0]=(-0.159113456499)*x[0]**o+(0.586749846436)*x[0]+(0.112658884741)*x[1]**o+(0.521917298039)*x[1]
        arg[2,1,1,1]=(-0.377986619436)*x[0]**o+(0.920912593255)*x[0]+(0.61285598265)*x[1]**o+(0.491316381102)*x[1]
        arg[2,1,2,0]=(-0.594573349393)*x[0]**o+(-0.784940398408)*x[0]+(0.0602427709094)*x[1]**o+(0.381334187216)*x[1]
        arg[2,1,2,1]=(-0.359402310466)*x[0]**o+(0.243014104221)*x[0]+(0.960587701667)*x[1]**o+(0.653124687952)*x[1]
        arg[2,2,0,0]=(0.667031182369)*x[0]**o+(0.099374903559)*x[0]+(0.00279841725989)*x[1]**o+(-0.413751543412)*x[1]
        arg[2,2,0,1]=(0.793557898241)*x[0]**o+(0.146572317359)*x[0]+(0.812964170008)*x[1]**o+(-0.988110793397)*x[1]
        arg[2,2,1,0]=(0.900661246137)*x[0]**o+(-0.997975978586)*x[0]+(0.63300171387)*x[1]**o+(-0.103878548778)*x[1]
        arg[2,2,1,1]=(-0.456464050469)*x[0]**o+(0.830699811714)*x[0]+(-0.186689927425)*x[1]**o+(-0.100170151164)*x[1]
        arg[2,2,2,0]=(0.732282428164)*x[0]**o+(0.928947589228)*x[0]+(-0.282773641579)*x[1]**o+(-0.927533112027)*x[1]
        arg[2,2,2,1]=(0.768111868708)*x[0]**o+(-0.308231472858)*x[0]+(0.890807565689)*x[1]**o+(-0.731196891492)*x[1]
        arg[2,3,0,0]=(0.559658559574)*x[0]**o+(0.104536615096)*x[0]+(-0.162594824125)*x[1]**o+(-0.172859200721)*x[1]
        arg[2,3,0,1]=(0.769297853152)*x[0]**o+(0.346598673904)*x[0]+(0.0313124779153)*x[1]**o+(-0.751767979251)*x[1]
        arg[2,3,1,0]=(0.870887050816)*x[0]**o+(0.593904859463)*x[0]+(-0.108083837899)*x[1]**o+(0.5467383682)*x[1]
        arg[2,3,1,1]=(0.244253227645)*x[0]**o+(0.911176738654)*x[0]+(-0.497855487714)*x[1]**o+(0.226443212585)*x[1]
        arg[2,3,2,0]=(-0.599893560596)*x[0]**o+(-0.893030581717)*x[0]+(-0.745249903413)*x[1]**o+(0.563740068619)*x[1]
        arg[2,3,2,1]=(-0.198088241103)*x[0]**o+(-0.171306730647)*x[0]+(-0.917417138299)*x[1]**o+(0.233920617125)*x[1]
        arg[2,4,0,0]=(0.804585270618)*x[0]**o+(0.3784879961)*x[0]+(0.621076476928)*x[1]**o+(-0.534671153952)*x[1]
        arg[2,4,0,1]=(0.380555159947)*x[0]**o+(-0.630487045841)*x[0]+(0.917413683072)*x[1]**o+(0.491925783834)*x[1]
        arg[2,4,1,0]=(0.946642644964)*x[0]**o+(-0.236438579482)*x[0]+(0.525585953739)*x[1]**o+(0.499940951133)*x[1]
        arg[2,4,1,1]=(0.827450889548)*x[0]**o+(-0.73264543626)*x[0]+(0.0626492901072)*x[1]**o+(-0.882759632862)*x[1]
        arg[2,4,2,0]=(-0.0671814496498)*x[0]**o+(0.108634446052)*x[0]+(-0.421081640868)*x[1]**o+(-0.546455336395)*x[1]
        arg[2,4,2,1]=(-0.601339784479)*x[0]**o+(-0.206937965916)*x[0]+(-0.849120792543)*x[1]**o+(0.615842037441)*x[1]
        arg[3,0,0,0]=(0.640466650913)*x[0]**o+(-0.936176165762)*x[0]+(0.12758686525)*x[1]**o+(-0.104771286099)*x[1]
        arg[3,0,0,1]=(-0.868109993006)*x[0]**o+(0.470815003671)*x[0]+(-0.301208743474)*x[1]**o+(-0.296271408095)*x[1]
        arg[3,0,1,0]=(0.66584252787)*x[0]**o+(-0.889549573986)*x[0]+(-0.0876076820512)*x[1]**o+(-0.249000029178)*x[1]
        arg[3,0,1,1]=(0.289388044606)*x[0]**o+(0.674519856916)*x[0]+(-0.721031236925)*x[1]**o+(0.093281544052)*x[1]
        arg[3,0,2,0]=(-0.0558863543119)*x[0]**o+(-0.367544430651)*x[0]+(-0.85580152565)*x[1]**o+(-0.140166367203)*x[1]
        arg[3,0,2,1]=(0.600709511675)*x[0]**o+(0.286322916564)*x[0]+(-0.544094007372)*x[1]**o+(0.470305833347)*x[1]
        arg[3,1,0,0]=(-0.828780197168)*x[0]**o+(-0.568096532278)*x[0]+(0.435127414299)*x[1]**o+(0.96890877255)*x[1]
        arg[3,1,0,1]=(0.191455614175)*x[0]**o+(-0.99507613827)*x[0]+(0.0735829936134)*x[1]**o+(0.635849313153)*x[1]
        arg[3,1,1,0]=(-0.839049013591)*x[0]**o+(-0.718731394184)*x[0]+(0.575853740085)*x[1]**o+(-0.57924857477)*x[1]
        arg[3,1,1,1]=(0.116876810453)*x[0]**o+(0.145727923885)*x[0]+(0.0155069936368)*x[1]**o+(0.223214885433)*x[1]
        arg[3,1,2,0]=(-0.674811720223)*x[0]**o+(0.651640807063)*x[0]+(-0.652022625703)*x[1]**o+(-0.119110788344)*x[1]
        arg[3,1,2,1]=(-0.544748794193)*x[0]**o+(-0.689440165483)*x[0]+(0.936900319026)*x[1]**o+(-0.337287311564)*x[1]
        arg[3,2,0,0]=(-0.355059804554)*x[0]**o+(0.458306329535)*x[0]+(0.172261907494)*x[1]**o+(0.498548138594)*x[1]
        arg[3,2,0,1]=(0.624420187129)*x[0]**o+(0.808931844722)*x[0]+(-0.690635894194)*x[1]**o+(-0.156122444554)*x[1]
        arg[3,2,1,0]=(-0.190029529776)*x[0]**o+(0.925600418137)*x[0]+(0.601040324953)*x[1]**o+(0.615485749173)*x[1]
        arg[3,2,1,1]=(0.679796674662)*x[0]**o+(0.879285148869)*x[0]+(0.0476373514288)*x[1]**o+(0.37655760501)*x[1]
        arg[3,2,2,0]=(0.231748550393)*x[0]**o+(0.209858138221)*x[0]+(-0.88372579866)*x[1]**o+(0.30534744967)*x[1]
        arg[3,2,2,1]=(-0.772177697671)*x[0]**o+(0.012439044435)*x[0]+(-0.293218976284)*x[1]**o+(0.306955066595)*x[1]
        arg[3,3,0,0]=(0.387256929042)*x[0]**o+(-0.434015953267)*x[0]+(-0.098736946868)*x[1]**o+(-0.0837712534186)*x[1]
        arg[3,3,0,1]=(0.396246252967)*x[0]**o+(-0.338955181605)*x[0]+(-0.353504371554)*x[1]**o+(-0.969811402293)*x[1]
        arg[3,3,1,0]=(-0.336508964723)*x[0]**o+(-0.411882233342)*x[0]+(-0.603965321132)*x[1]**o+(0.177221705416)*x[1]
        arg[3,3,1,1]=(0.786971174387)*x[0]**o+(0.759003522166)*x[0]+(-0.0961289050409)*x[1]**o+(-0.0811258682982)*x[1]
        arg[3,3,2,0]=(0.526266925685)*x[0]**o+(-0.113145219611)*x[0]+(-0.208427816884)*x[1]**o+(0.291484710079)*x[1]
        arg[3,3,2,1]=(-0.591153907468)*x[0]**o+(-0.342892047641)*x[0]+(0.865954283634)*x[1]**o+(-0.22112030903)*x[1]
        arg[3,4,0,0]=(0.755162128186)*x[0]**o+(0.494457469003)*x[0]+(0.889254517137)*x[1]**o+(0.880654056969)*x[1]
        arg[3,4,0,1]=(0.450983401629)*x[0]**o+(-0.0855489110466)*x[0]+(0.0601625202676)*x[1]**o+(-0.0730383237708)*x[1]
        arg[3,4,1,0]=(0.495514588456)*x[0]**o+(-0.0895269876334)*x[0]+(-0.89729630673)*x[1]**o+(0.759465768614)*x[1]
        arg[3,4,1,1]=(-0.666472982468)*x[0]**o+(-0.251715028171)*x[0]+(-0.0623287375223)*x[1]**o+(-0.895663326995)*x[1]
        arg[3,4,2,0]=(0.489737923112)*x[0]**o+(0.868066981889)*x[0]+(0.0371747456704)*x[1]**o+(-0.528951636028)*x[1]
        arg[3,4,2,1]=(0.556312598455)*x[0]**o+(-0.0979773350236)*x[0]+(-0.82785928691)*x[1]**o+(-0.567596765858)*x[1]
        ref[0,0,0,0]=(0.14016992168)*x_ref[0]**o+(-0.396856222091)*x_ref[0]+(0.747446299477)*x_ref[1]**o+(-0.299063042467)*x_ref[1]
        ref[0,0,0,1]=(0.126297805139)*x_ref[0]**o+(0.317089000483)*x_ref[0]+(0.0418478916266)*x_ref[1]**o+(-0.892825689806)*x_ref[1]
        ref[0,0,1,0]=(0.127263485532)*x_ref[0]**o+(0.400880552184)*x_ref[0]+(-0.858713979216)*x_ref[1]**o+(0.238693631738)*x_ref[1]
        ref[0,0,1,1]=(-0.985703521737)*x_ref[0]**o+(-0.0192845851979)*x_ref[0]+(-0.442493677109)*x_ref[1]**o+(0.188038174612)*x_ref[1]
        ref[0,0,2,0]=(0.814303535641)*x_ref[0]**o+(-0.201938227136)*x_ref[0]+(-0.134135136581)*x_ref[1]**o+(0.939272592089)*x_ref[1]
        ref[0,0,2,1]=(-0.609570412575)*x_ref[0]**o+(0.181669701944)*x_ref[0]+(0.882708213835)*x_ref[1]**o+(0.315561722535)*x_ref[1]
        ref[0,1,0,0]=(0.180272217085)*x_ref[0]**o+(-0.921231698951)*x_ref[0]+(-0.670170775864)*x_ref[1]**o+(0.36959006719)*x_ref[1]
        ref[0,1,0,1]=(-0.907447754449)*x_ref[0]**o+(0.984356686322)*x_ref[0]+(0.300621254386)*x_ref[1]**o+(0.491995830235)*x_ref[1]
        ref[0,1,1,0]=(-0.535808887861)*x_ref[0]**o+(0.991647965865)*x_ref[0]+(-0.505667913884)*x_ref[1]**o+(-0.403245981754)*x_ref[1]
        ref[0,1,1,1]=(-0.315687983887)*x_ref[0]**o+(-0.570118136173)*x_ref[0]+(0.327842038508)*x_ref[1]**o+(-0.475833876716)*x_ref[1]
        ref[0,1,2,0]=(0.598143043008)*x_ref[0]**o+(0.119789453354)*x_ref[0]+(-0.307994361027)*x_ref[1]**o+(-0.747443620209)*x_ref[1]
        ref[0,1,2,1]=(-0.384576138426)*x_ref[0]**o+(-0.21431195392)*x_ref[0]+(0.622157923623)*x_ref[1]**o+(-0.535976986322)*x_ref[1]
        ref[0,2,0,0]=(-0.823640169824)*x_ref[0]**o+(-0.946436729867)*x_ref[0]+(0.158819942161)*x_ref[1]**o+(-0.175783146941)*x_ref[1]
        ref[0,2,0,1]=(0.31586435609)*x_ref[0]**o+(0.480823809413)*x_ref[0]+(-0.155590673671)*x_ref[1]**o+(-0.283774897002)*x_ref[1]
        ref[0,2,1,0]=(0.973677960338)*x_ref[0]**o+(-0.42119441385)*x_ref[0]+(0.936434159723)*x_ref[1]**o+(-0.271328279678)*x_ref[1]
        ref[0,2,1,1]=(0.995977746066)*x_ref[0]**o+(0.115566554312)*x_ref[0]+(0.6023712777)*x_ref[1]**o+(0.523708594385)*x_ref[1]
        ref[0,2,2,0]=(0.690331991941)*x_ref[0]**o+(0.708864222789)*x_ref[0]+(-0.0209681974418)*x_ref[1]**o+(-0.402664547241)*x_ref[1]
        ref[0,2,2,1]=(0.793403379571)*x_ref[0]**o+(-0.44195907862)*x_ref[0]+(-0.750943552401)*x_ref[1]**o+(0.87817105061)*x_ref[1]
        ref[0,3,0,0]=(-0.960650344452)*x_ref[0]**o+(0.918969286865)*x_ref[0]+(-0.788611475275)*x_ref[1]**o+(-0.917265819096)*x_ref[1]
        ref[0,3,0,1]=(0.379234695969)*x_ref[0]**o+(0.288784268667)*x_ref[0]+(-0.231788371542)*x_ref[1]**o+(-0.521488838978)*x_ref[1]
        ref[0,3,1,0]=(-0.482353410226)*x_ref[0]**o+(0.666269361063)*x_ref[0]+(0.207628082266)*x_ref[1]**o+(0.890536697637)*x_ref[1]
        ref[0,3,1,1]=(-0.256826971067)*x_ref[0]**o+(0.244573109457)*x_ref[0]+(0.435182409565)*x_ref[1]**o+(-0.98882467576)*x_ref[1]
        ref[0,3,2,0]=(0.727681501266)*x_ref[0]**o+(0.174117939756)*x_ref[0]+(0.626998345198)*x_ref[1]**o+(0.666518756047)*x_ref[1]
        ref[0,3,2,1]=(-0.125855300157)*x_ref[0]**o+(-0.750291117974)*x_ref[0]+(0.840722993443)*x_ref[1]**o+(0.0798848819291)*x_ref[1]
        ref[0,4,0,0]=(-0.156808480304)*x_ref[0]**o+(-0.864855000875)*x_ref[0]+(-0.825786767174)*x_ref[1]**o+(-0.563999207624)*x_ref[1]
        ref[0,4,0,1]=(-0.890116284779)*x_ref[0]**o+(0.795471417269)*x_ref[0]+(0.326454331959)*x_ref[1]**o+(-0.376644334859)*x_ref[1]
        ref[0,4,1,0]=(-0.453181887597)*x_ref[0]**o+(0.488149970589)*x_ref[0]+(0.506352954527)*x_ref[1]**o+(0.971016325918)*x_ref[1]
        ref[0,4,1,1]=(0.156976484311)*x_ref[0]**o+(0.988818166411)*x_ref[0]+(0.948579296281)*x_ref[1]**o+(-0.80303322078)*x_ref[1]
        ref[0,4,2,0]=(-0.0625333956516)*x_ref[0]**o+(0.357609677642)*x_ref[0]+(0.00245182913993)*x_ref[1]**o+(-0.162058851234)*x_ref[1]
        ref[0,4,2,1]=(0.645504631508)*x_ref[0]**o+(-0.0107543576265)*x_ref[0]+(0.15490632437)*x_ref[1]**o+(0.841985786269)*x_ref[1]
        ref[1,0,0,0]=(0.703923364197)*x_ref[0]**o+(-0.424818178123)*x_ref[0]+(-0.744399116071)*x_ref[1]**o+(0.303429987787)*x_ref[1]
        ref[1,0,0,1]=(-0.92849671168)*x_ref[0]**o+(0.050597602551)*x_ref[0]+(0.91000243103)*x_ref[1]**o+(-0.742775174195)*x_ref[1]
        ref[1,0,1,0]=(0.281218131435)*x_ref[0]**o+(0.171029842909)*x_ref[0]+(-0.65806576709)*x_ref[1]**o+(-0.718069456213)*x_ref[1]
        ref[1,0,1,1]=(0.312255067232)*x_ref[0]**o+(0.566401030665)*x_ref[0]+(-0.941709776271)*x_ref[1]**o+(0.0111896041808)*x_ref[1]
        ref[1,0,2,0]=(-0.886019321094)*x_ref[0]**o+(-0.688815756004)*x_ref[0]+(-0.595491288141)*x_ref[1]**o+(0.343402495621)*x_ref[1]
        ref[1,0,2,1]=(-0.681063845863)*x_ref[0]**o+(-0.676550403066)*x_ref[0]+(0.186106532969)*x_ref[1]**o+(0.181779537727)*x_ref[1]
        ref[1,1,0,0]=(0.128588939989)*x_ref[0]**o+(0.477711214734)*x_ref[0]+(-0.17697460919)*x_ref[1]**o+(0.0217208670244)*x_ref[1]
        ref[1,1,0,1]=(-0.652028522296)*x_ref[0]**o+(-0.503146425209)*x_ref[0]+(-0.427973128189)*x_ref[1]**o+(-0.182775422425)*x_ref[1]
        ref[1,1,1,0]=(-0.0499349942904)*x_ref[0]**o+(-0.0183804745613)*x_ref[0]+(-0.912604094211)*x_ref[1]**o+(-0.949181938689)*x_ref[1]
        ref[1,1,1,1]=(-0.472737626621)*x_ref[0]**o+(-0.142195314687)*x_ref[0]+(0.580775749383)*x_ref[1]**o+(-0.832549796746)*x_ref[1]
        ref[1,1,2,0]=(-0.288756468146)*x_ref[0]**o+(-0.50657056474)*x_ref[0]+(-0.809713268333)*x_ref[1]**o+(0.598450102365)*x_ref[1]
        ref[1,1,2,1]=(-0.259779636241)*x_ref[0]**o+(-0.227925001879)*x_ref[0]+(-0.0752800702413)*x_ref[1]**o+(-0.438841969161)*x_ref[1]
        ref[1,2,0,0]=(-0.617835522076)*x_ref[0]**o+(-0.969275786274)*x_ref[0]+(0.899717033555)*x_ref[1]**o+(0.77648944909)*x_ref[1]
        ref[1,2,0,1]=(-0.0619395581419)*x_ref[0]**o+(0.926394874574)*x_ref[0]+(0.984605961354)*x_ref[1]**o+(0.253347734181)*x_ref[1]
        ref[1,2,1,0]=(0.748474342269)*x_ref[0]**o+(0.0794003341104)*x_ref[0]+(-0.244607682532)*x_ref[1]**o+(-0.228787873564)*x_ref[1]
        ref[1,2,1,1]=(-0.266853205536)*x_ref[0]**o+(0.529714058158)*x_ref[0]+(-0.421599080872)*x_ref[1]**o+(-0.595557599245)*x_ref[1]
        ref[1,2,2,0]=(-0.377962443753)*x_ref[0]**o+(0.609550409525)*x_ref[0]+(0.951904771097)*x_ref[1]**o+(-0.123551801933)*x_ref[1]
        ref[1,2,2,1]=(0.3262671136)*x_ref[0]**o+(-0.574281078833)*x_ref[0]+(0.308261022622)*x_ref[1]**o+(-0.784762723843)*x_ref[1]
        ref[1,3,0,0]=(-0.419601410829)*x_ref[0]**o+(-0.576064062071)*x_ref[0]+(-0.431574278091)*x_ref[1]**o+(-0.264303418881)*x_ref[1]
        ref[1,3,0,1]=(0.176058414693)*x_ref[0]**o+(0.672752046411)*x_ref[0]+(-0.8579992002)*x_ref[1]**o+(-0.135904959185)*x_ref[1]
        ref[1,3,1,0]=(0.84480795105)*x_ref[0]**o+(0.846846799225)*x_ref[0]+(0.134417432017)*x_ref[1]**o+(-0.725316162128)*x_ref[1]
        ref[1,3,1,1]=(-0.102162279538)*x_ref[0]**o+(0.451180031789)*x_ref[0]+(0.490175190736)*x_ref[1]**o+(0.811823281177)*x_ref[1]
        ref[1,3,2,0]=(-0.582177348293)*x_ref[0]**o+(-0.74631626712)*x_ref[0]+(-0.275524221012)*x_ref[1]**o+(-0.658276720144)*x_ref[1]
        ref[1,3,2,1]=(0.304511057297)*x_ref[0]**o+(-0.354531411968)*x_ref[0]+(0.0944491594546)*x_ref[1]**o+(0.619039121597)*x_ref[1]
        ref[1,4,0,0]=(0.754898520324)*x_ref[0]**o+(-0.28914218119)*x_ref[0]+(0.8660675945)*x_ref[1]**o+(0.526777161228)*x_ref[1]
        ref[1,4,0,1]=(-0.820150637429)*x_ref[0]**o+(-0.399203443857)*x_ref[0]+(0.118863382167)*x_ref[1]**o+(0.269335037373)*x_ref[1]
        ref[1,4,1,0]=(-0.669389232967)*x_ref[0]**o+(-0.119080399209)*x_ref[0]+(0.999919873616)*x_ref[1]**o+(0.733390578378)*x_ref[1]
        ref[1,4,1,1]=(-0.85147101348)*x_ref[0]**o+(-0.0453996523642)*x_ref[0]+(0.00891698330334)*x_ref[1]**o+(-0.27128670805)*x_ref[1]
        ref[1,4,2,0]=(-0.946558097166)*x_ref[0]**o+(0.0141300604882)*x_ref[0]+(0.04075090617)*x_ref[1]**o+(-0.141095386138)*x_ref[1]
        ref[1,4,2,1]=(0.613700159605)*x_ref[0]**o+(0.115107829201)*x_ref[0]+(0.907429968641)*x_ref[1]**o+(0.901820725424)*x_ref[1]
        ref[2,0,0,0]=(-0.629486588984)*x_ref[0]**o+(-0.590908868493)*x_ref[0]+(0.689333052519)*x_ref[1]**o+(0.22096464697)*x_ref[1]
        ref[2,0,0,1]=(-0.475364708492)*x_ref[0]**o+(-0.3195126467)*x_ref[0]+(-0.688082519853)*x_ref[1]**o+(-0.889722517035)*x_ref[1]
        ref[2,0,1,0]=(0.931261765284)*x_ref[0]**o+(0.399101548845)*x_ref[0]+(-0.185342345742)*x_ref[1]**o+(0.188927724838)*x_ref[1]
        ref[2,0,1,1]=(-0.195889209768)*x_ref[0]**o+(-0.0878952476483)*x_ref[0]+(0.636980893598)*x_ref[1]**o+(0.827709469156)*x_ref[1]
        ref[2,0,2,0]=(0.644658731705)*x_ref[0]**o+(0.275771474382)*x_ref[0]+(0.0802903363405)*x_ref[1]**o+(-0.157785300253)*x_ref[1]
        ref[2,0,2,1]=(0.324153739483)*x_ref[0]**o+(-0.978043993375)*x_ref[0]+(0.440060321393)*x_ref[1]**o+(0.103952166755)*x_ref[1]
        ref[2,1,0,0]=(-0.233567312787)*x_ref[0]**o+(0.139529308019)*x_ref[0]+(-0.200168759944)*x_ref[1]**o+(-0.629057762953)*x_ref[1]
        ref[2,1,0,1]=(-0.407808993281)*x_ref[0]**o+(-0.208049914815)*x_ref[0]+(0.676817990818)*x_ref[1]**o+(-0.27174220248)*x_ref[1]
        ref[2,1,1,0]=(-0.159113456499)*x_ref[0]**o+(0.586749846436)*x_ref[0]+(0.112658884741)*x_ref[1]**o+(0.521917298039)*x_ref[1]
        ref[2,1,1,1]=(-0.377986619436)*x_ref[0]**o+(0.920912593255)*x_ref[0]+(0.61285598265)*x_ref[1]**o+(0.491316381102)*x_ref[1]
        ref[2,1,2,0]=(-0.594573349393)*x_ref[0]**o+(-0.784940398408)*x_ref[0]+(0.0602427709094)*x_ref[1]**o+(0.381334187216)*x_ref[1]
        ref[2,1,2,1]=(-0.359402310466)*x_ref[0]**o+(0.243014104221)*x_ref[0]+(0.960587701667)*x_ref[1]**o+(0.653124687952)*x_ref[1]
        ref[2,2,0,0]=(0.667031182369)*x_ref[0]**o+(0.099374903559)*x_ref[0]+(0.00279841725989)*x_ref[1]**o+(-0.413751543412)*x_ref[1]
        ref[2,2,0,1]=(0.793557898241)*x_ref[0]**o+(0.146572317359)*x_ref[0]+(0.812964170008)*x_ref[1]**o+(-0.988110793397)*x_ref[1]
        ref[2,2,1,0]=(0.900661246137)*x_ref[0]**o+(-0.997975978586)*x_ref[0]+(0.63300171387)*x_ref[1]**o+(-0.103878548778)*x_ref[1]
        ref[2,2,1,1]=(-0.456464050469)*x_ref[0]**o+(0.830699811714)*x_ref[0]+(-0.186689927425)*x_ref[1]**o+(-0.100170151164)*x_ref[1]
        ref[2,2,2,0]=(0.732282428164)*x_ref[0]**o+(0.928947589228)*x_ref[0]+(-0.282773641579)*x_ref[1]**o+(-0.927533112027)*x_ref[1]
        ref[2,2,2,1]=(0.768111868708)*x_ref[0]**o+(-0.308231472858)*x_ref[0]+(0.890807565689)*x_ref[1]**o+(-0.731196891492)*x_ref[1]
        ref[2,3,0,0]=(0.559658559574)*x_ref[0]**o+(0.104536615096)*x_ref[0]+(-0.162594824125)*x_ref[1]**o+(-0.172859200721)*x_ref[1]
        ref[2,3,0,1]=(0.769297853152)*x_ref[0]**o+(0.346598673904)*x_ref[0]+(0.0313124779153)*x_ref[1]**o+(-0.751767979251)*x_ref[1]
        ref[2,3,1,0]=(0.870887050816)*x_ref[0]**o+(0.593904859463)*x_ref[0]+(-0.108083837899)*x_ref[1]**o+(0.5467383682)*x_ref[1]
        ref[2,3,1,1]=(0.244253227645)*x_ref[0]**o+(0.911176738654)*x_ref[0]+(-0.497855487714)*x_ref[1]**o+(0.226443212585)*x_ref[1]
        ref[2,3,2,0]=(-0.599893560596)*x_ref[0]**o+(-0.893030581717)*x_ref[0]+(-0.745249903413)*x_ref[1]**o+(0.563740068619)*x_ref[1]
        ref[2,3,2,1]=(-0.198088241103)*x_ref[0]**o+(-0.171306730647)*x_ref[0]+(-0.917417138299)*x_ref[1]**o+(0.233920617125)*x_ref[1]
        ref[2,4,0,0]=(0.804585270618)*x_ref[0]**o+(0.3784879961)*x_ref[0]+(0.621076476928)*x_ref[1]**o+(-0.534671153952)*x_ref[1]
        ref[2,4,0,1]=(0.380555159947)*x_ref[0]**o+(-0.630487045841)*x_ref[0]+(0.917413683072)*x_ref[1]**o+(0.491925783834)*x_ref[1]
        ref[2,4,1,0]=(0.946642644964)*x_ref[0]**o+(-0.236438579482)*x_ref[0]+(0.525585953739)*x_ref[1]**o+(0.499940951133)*x_ref[1]
        ref[2,4,1,1]=(0.827450889548)*x_ref[0]**o+(-0.73264543626)*x_ref[0]+(0.0626492901072)*x_ref[1]**o+(-0.882759632862)*x_ref[1]
        ref[2,4,2,0]=(-0.0671814496498)*x_ref[0]**o+(0.108634446052)*x_ref[0]+(-0.421081640868)*x_ref[1]**o+(-0.546455336395)*x_ref[1]
        ref[2,4,2,1]=(-0.601339784479)*x_ref[0]**o+(-0.206937965916)*x_ref[0]+(-0.849120792543)*x_ref[1]**o+(0.615842037441)*x_ref[1]
        ref[3,0,0,0]=(0.640466650913)*x_ref[0]**o+(-0.936176165762)*x_ref[0]+(0.12758686525)*x_ref[1]**o+(-0.104771286099)*x_ref[1]
        ref[3,0,0,1]=(-0.868109993006)*x_ref[0]**o+(0.470815003671)*x_ref[0]+(-0.301208743474)*x_ref[1]**o+(-0.296271408095)*x_ref[1]
        ref[3,0,1,0]=(0.66584252787)*x_ref[0]**o+(-0.889549573986)*x_ref[0]+(-0.0876076820512)*x_ref[1]**o+(-0.249000029178)*x_ref[1]
        ref[3,0,1,1]=(0.289388044606)*x_ref[0]**o+(0.674519856916)*x_ref[0]+(-0.721031236925)*x_ref[1]**o+(0.093281544052)*x_ref[1]
        ref[3,0,2,0]=(-0.0558863543119)*x_ref[0]**o+(-0.367544430651)*x_ref[0]+(-0.85580152565)*x_ref[1]**o+(-0.140166367203)*x_ref[1]
        ref[3,0,2,1]=(0.600709511675)*x_ref[0]**o+(0.286322916564)*x_ref[0]+(-0.544094007372)*x_ref[1]**o+(0.470305833347)*x_ref[1]
        ref[3,1,0,0]=(-0.828780197168)*x_ref[0]**o+(-0.568096532278)*x_ref[0]+(0.435127414299)*x_ref[1]**o+(0.96890877255)*x_ref[1]
        ref[3,1,0,1]=(0.191455614175)*x_ref[0]**o+(-0.99507613827)*x_ref[0]+(0.0735829936134)*x_ref[1]**o+(0.635849313153)*x_ref[1]
        ref[3,1,1,0]=(-0.839049013591)*x_ref[0]**o+(-0.718731394184)*x_ref[0]+(0.575853740085)*x_ref[1]**o+(-0.57924857477)*x_ref[1]
        ref[3,1,1,1]=(0.116876810453)*x_ref[0]**o+(0.145727923885)*x_ref[0]+(0.0155069936368)*x_ref[1]**o+(0.223214885433)*x_ref[1]
        ref[3,1,2,0]=(-0.674811720223)*x_ref[0]**o+(0.651640807063)*x_ref[0]+(-0.652022625703)*x_ref[1]**o+(-0.119110788344)*x_ref[1]
        ref[3,1,2,1]=(-0.544748794193)*x_ref[0]**o+(-0.689440165483)*x_ref[0]+(0.936900319026)*x_ref[1]**o+(-0.337287311564)*x_ref[1]
        ref[3,2,0,0]=(-0.355059804554)*x_ref[0]**o+(0.458306329535)*x_ref[0]+(0.172261907494)*x_ref[1]**o+(0.498548138594)*x_ref[1]
        ref[3,2,0,1]=(0.624420187129)*x_ref[0]**o+(0.808931844722)*x_ref[0]+(-0.690635894194)*x_ref[1]**o+(-0.156122444554)*x_ref[1]
        ref[3,2,1,0]=(-0.190029529776)*x_ref[0]**o+(0.925600418137)*x_ref[0]+(0.601040324953)*x_ref[1]**o+(0.615485749173)*x_ref[1]
        ref[3,2,1,1]=(0.679796674662)*x_ref[0]**o+(0.879285148869)*x_ref[0]+(0.0476373514288)*x_ref[1]**o+(0.37655760501)*x_ref[1]
        ref[3,2,2,0]=(0.231748550393)*x_ref[0]**o+(0.209858138221)*x_ref[0]+(-0.88372579866)*x_ref[1]**o+(0.30534744967)*x_ref[1]
        ref[3,2,2,1]=(-0.772177697671)*x_ref[0]**o+(0.012439044435)*x_ref[0]+(-0.293218976284)*x_ref[1]**o+(0.306955066595)*x_ref[1]
        ref[3,3,0,0]=(0.387256929042)*x_ref[0]**o+(-0.434015953267)*x_ref[0]+(-0.098736946868)*x_ref[1]**o+(-0.0837712534186)*x_ref[1]
        ref[3,3,0,1]=(0.396246252967)*x_ref[0]**o+(-0.338955181605)*x_ref[0]+(-0.353504371554)*x_ref[1]**o+(-0.969811402293)*x_ref[1]
        ref[3,3,1,0]=(-0.336508964723)*x_ref[0]**o+(-0.411882233342)*x_ref[0]+(-0.603965321132)*x_ref[1]**o+(0.177221705416)*x_ref[1]
        ref[3,3,1,1]=(0.786971174387)*x_ref[0]**o+(0.759003522166)*x_ref[0]+(-0.0961289050409)*x_ref[1]**o+(-0.0811258682982)*x_ref[1]
        ref[3,3,2,0]=(0.526266925685)*x_ref[0]**o+(-0.113145219611)*x_ref[0]+(-0.208427816884)*x_ref[1]**o+(0.291484710079)*x_ref[1]
        ref[3,3,2,1]=(-0.591153907468)*x_ref[0]**o+(-0.342892047641)*x_ref[0]+(0.865954283634)*x_ref[1]**o+(-0.22112030903)*x_ref[1]
        ref[3,4,0,0]=(0.755162128186)*x_ref[0]**o+(0.494457469003)*x_ref[0]+(0.889254517137)*x_ref[1]**o+(0.880654056969)*x_ref[1]
        ref[3,4,0,1]=(0.450983401629)*x_ref[0]**o+(-0.0855489110466)*x_ref[0]+(0.0601625202676)*x_ref[1]**o+(-0.0730383237708)*x_ref[1]
        ref[3,4,1,0]=(0.495514588456)*x_ref[0]**o+(-0.0895269876334)*x_ref[0]+(-0.89729630673)*x_ref[1]**o+(0.759465768614)*x_ref[1]
        ref[3,4,1,1]=(-0.666472982468)*x_ref[0]**o+(-0.251715028171)*x_ref[0]+(-0.0623287375223)*x_ref[1]**o+(-0.895663326995)*x_ref[1]
        ref[3,4,2,0]=(0.489737923112)*x_ref[0]**o+(0.868066981889)*x_ref[0]+(0.0371747456704)*x_ref[1]**o+(-0.528951636028)*x_ref[1]
        ref[3,4,2,1]=(0.556312598455)*x_ref[0]**o+(-0.0979773350236)*x_ref[0]+(-0.82785928691)*x_ref[1]**o+(-0.567596765858)*x_ref[1]
      else:
        arg[0,0,0,0]=(0.952750275738)*x[0]**o+(0.971489204087)*x[0]+(-0.320706487584)*x[1]**o+(-0.853260471578)*x[1]+(-0.696434862068)*x[2]**o+(0.641824397433)*x[2]
        arg[0,0,0,1]=(0.115484192786)*x[0]**o+(0.147813687309)*x[0]+(-0.998010548186)*x[1]**o+(0.0274712088882)*x[1]+(0.136937971489)*x[2]**o+(-0.657986135444)*x[2]
        arg[0,0,1,0]=(0.805857504779)*x[0]**o+(0.196278916938)*x[0]+(-0.969340785263)*x[1]**o+(-0.543890174888)*x[1]+(0.49118423357)*x[2]**o+(-0.883493800593)*x[2]
        arg[0,0,1,1]=(-0.808802136474)*x[0]**o+(-0.866039395649)*x[0]+(0.0339207025941)*x[1]**o+(-0.441368773534)*x[1]+(0.673688211653)*x[2]**o+(0.917894389398)*x[2]
        arg[0,0,2,0]=(0.0459902210078)*x[0]**o+(0.862332666528)*x[0]+(-0.547792306389)*x[1]**o+(0.722176431732)*x[1]+(-0.511278989032)*x[2]**o+(-0.692775336577)*x[2]
        arg[0,0,2,1]=(-0.641904406513)*x[0]**o+(-0.854450656063)*x[0]+(0.81138894849)*x[1]**o+(0.328886625074)*x[1]+(0.400935831363)*x[2]**o+(-0.25184059766)*x[2]
        arg[0,1,0,0]=(0.69922840942)*x[0]**o+(-0.798085508443)*x[0]+(-0.85191383431)*x[1]**o+(-0.162940829331)*x[1]+(0.504020642931)*x[2]**o+(0.0950957436173)*x[2]
        arg[0,1,0,1]=(0.28901429037)*x[0]**o+(-0.624508745729)*x[0]+(0.921288077478)*x[1]**o+(-0.769207901119)*x[1]+(0.0852374599763)*x[2]**o+(-0.402583033986)*x[2]
        arg[0,1,1,0]=(0.715817282215)*x[0]**o+(0.68735256665)*x[0]+(0.610212058649)*x[1]**o+(0.53560824609)*x[1]+(-0.238908897264)*x[2]**o+(0.619799158474)*x[2]
        arg[0,1,1,1]=(-0.371797414191)*x[0]**o+(0.106263350514)*x[0]+(0.583768355362)*x[1]**o+(0.818797667369)*x[1]+(0.0567571152493)*x[2]**o+(-0.242244867169)*x[2]
        arg[0,1,2,0]=(0.190480659729)*x[0]**o+(-0.632415604771)*x[0]+(0.252513826138)*x[1]**o+(0.874732840469)*x[1]+(-0.367769326078)*x[2]**o+(0.364453292404)*x[2]
        arg[0,1,2,1]=(-0.704507136581)*x[0]**o+(-0.614211737547)*x[0]+(0.533383507313)*x[1]**o+(-0.940007583478)*x[1]+(0.558260002746)*x[2]**o+(0.742565692751)*x[2]
        arg[0,2,0,0]=(0.132581156766)*x[0]**o+(0.671793183183)*x[0]+(-0.308255110769)*x[1]**o+(-0.956002957061)*x[1]+(0.300091541301)*x[2]**o+(-0.714260746791)*x[2]
        arg[0,2,0,1]=(-0.836648080859)*x[0]**o+(0.123813861017)*x[0]+(0.866569934894)*x[1]**o+(-0.957824572351)*x[1]+(-0.904478410952)*x[2]**o+(0.691673758521)*x[2]
        arg[0,2,1,0]=(0.411624721878)*x[0]**o+(0.594054722352)*x[0]+(0.150177701135)*x[1]**o+(0.945855261084)*x[1]+(0.295930158817)*x[2]**o+(-0.607980954424)*x[2]
        arg[0,2,1,1]=(-0.0777741308373)*x[0]**o+(0.697501295655)*x[0]+(0.981797797329)*x[1]**o+(-0.671827077949)*x[1]+(0.292310146229)*x[2]**o+(0.484652357857)*x[2]
        arg[0,2,2,0]=(0.194910469807)*x[0]**o+(-0.70008515623)*x[0]+(0.192808336639)*x[1]**o+(-0.962764477371)*x[1]+(-0.34292240753)*x[2]**o+(0.525470829202)*x[2]
        arg[0,2,2,1]=(0.850368420228)*x[0]**o+(-0.516355386368)*x[0]+(0.395764367873)*x[1]**o+(0.320830752607)*x[1]+(-0.26288679044)*x[2]**o+(0.174457723191)*x[2]
        arg[0,3,0,0]=(-0.713987556845)*x[0]**o+(0.0391066248253)*x[0]+(-0.63911733479)*x[1]**o+(-0.119953032121)*x[1]+(0.697879836447)*x[2]**o+(-0.474957057702)*x[2]
        arg[0,3,0,1]=(-0.128614897423)*x[0]**o+(0.7166036403)*x[0]+(0.678415978447)*x[1]**o+(-0.191453169618)*x[1]+(-0.755261361956)*x[2]**o+(-0.839956004196)*x[2]
        arg[0,3,1,0]=(0.258158740934)*x[0]**o+(-0.412199346967)*x[0]+(-0.781106341272)*x[1]**o+(-0.561162823101)*x[1]+(-0.59549864377)*x[2]**o+(-0.748038415721)*x[2]
        arg[0,3,1,1]=(0.68950222305)*x[0]**o+(-0.904027774445)*x[0]+(0.769744453441)*x[1]**o+(-0.265549152752)*x[1]+(-0.375978837131)*x[2]**o+(-0.387010981761)*x[2]
        arg[0,3,2,0]=(0.46852681319)*x[0]**o+(-0.887650258615)*x[0]+(-0.780275624319)*x[1]**o+(0.527897372599)*x[1]+(0.889281778411)*x[2]**o+(0.82538563877)*x[2]
        arg[0,3,2,1]=(-0.817514448184)*x[0]**o+(-0.230190768519)*x[0]+(-0.101550849344)*x[1]**o+(0.1654885857)*x[1]+(-0.231317749498)*x[2]**o+(0.580153306395)*x[2]
        arg[0,4,0,0]=(0.394993007242)*x[0]**o+(0.305948764395)*x[0]+(-0.66853441368)*x[1]**o+(0.279027498094)*x[1]+(0.161366934709)*x[2]**o+(-0.742002471552)*x[2]
        arg[0,4,0,1]=(-0.661214709471)*x[0]**o+(0.548880317935)*x[0]+(0.472682748503)*x[1]**o+(0.266797505699)*x[1]+(0.405618413683)*x[2]**o+(-0.0539677542667)*x[2]
        arg[0,4,1,0]=(-0.713892424429)*x[0]**o+(0.568075831592)*x[0]+(-0.956180895599)*x[1]**o+(0.63618957695)*x[1]+(0.983696160047)*x[2]**o+(-0.308157929002)*x[2]
        arg[0,4,1,1]=(-0.666582919353)*x[0]**o+(0.0816884254484)*x[0]+(0.190810048896)*x[1]**o+(0.482322905132)*x[1]+(-0.576597070954)*x[2]**o+(-0.722381611553)*x[2]
        arg[0,4,2,0]=(-0.0229650731041)*x[0]**o+(-0.0867816754979)*x[0]+(0.0905042102365)*x[1]**o+(-0.994963424765)*x[1]+(0.74504757815)*x[2]**o+(0.71462549644)*x[2]
        arg[0,4,2,1]=(-0.0464278496147)*x[0]**o+(0.122501026996)*x[0]+(-0.049450141901)*x[1]**o+(-0.388739464086)*x[1]+(-0.151389885067)*x[2]**o+(-0.0316948088803)*x[2]
        arg[1,0,0,0]=(-0.303653386851)*x[0]**o+(-0.435866753659)*x[0]+(-0.419078175082)*x[1]**o+(-0.907595244917)*x[1]+(0.442156528837)*x[2]**o+(-0.76481785523)*x[2]
        arg[1,0,0,1]=(0.879454551618)*x[0]**o+(-0.530789858588)*x[0]+(0.942203776542)*x[1]**o+(0.864362935436)*x[1]+(-0.252832164588)*x[2]**o+(0.00758252581015)*x[2]
        arg[1,0,1,0]=(0.522433142898)*x[0]**o+(0.564002257569)*x[0]+(0.149811689377)*x[1]**o+(-0.757726948165)*x[1]+(-0.767052869719)*x[2]**o+(0.743541806001)*x[2]
        arg[1,0,1,1]=(0.535612198186)*x[0]**o+(0.0874480088401)*x[0]+(0.913151292296)*x[1]**o+(-0.495024655308)*x[1]+(-0.42308905182)*x[2]**o+(-0.983623551153)*x[2]
        arg[1,0,2,0]=(0.587221456352)*x[0]**o+(0.0384438535009)*x[0]+(0.301554087927)*x[1]**o+(-0.944074061806)*x[1]+(0.808195432991)*x[2]**o+(-0.46545743768)*x[2]
        arg[1,0,2,1]=(0.136251169669)*x[0]**o+(0.772190271875)*x[0]+(-0.490981224022)*x[1]**o+(-0.191915588448)*x[1]+(0.792617864405)*x[2]**o+(0.139425359773)*x[2]
        arg[1,1,0,0]=(0.0440493146329)*x[0]**o+(-0.948951787109)*x[0]+(-0.564104026105)*x[1]**o+(0.177457883377)*x[1]+(0.578169345377)*x[2]**o+(0.35574407049)*x[2]
        arg[1,1,0,1]=(-0.740305651819)*x[0]**o+(-0.340976006932)*x[0]+(-0.657257828033)*x[1]**o+(-0.456240473558)*x[1]+(0.526236681039)*x[2]**o+(0.804762801115)*x[2]
        arg[1,1,1,0]=(-0.753480536304)*x[0]**o+(0.424080516297)*x[0]+(-0.97529611006)*x[1]**o+(-0.511450162624)*x[1]+(-0.248982226385)*x[2]**o+(0.497803766242)*x[2]
        arg[1,1,1,1]=(-0.273280180629)*x[0]**o+(0.798945176097)*x[0]+(0.552152546131)*x[1]**o+(0.737594404744)*x[1]+(0.0339269287984)*x[2]**o+(-0.773791176515)*x[2]
        arg[1,1,2,0]=(0.553404075378)*x[0]**o+(-0.630969888585)*x[0]+(0.455072287913)*x[1]**o+(0.145928503781)*x[1]+(0.632480470504)*x[2]**o+(0.363815684296)*x[2]
        arg[1,1,2,1]=(-0.426756293656)*x[0]**o+(-0.449867410734)*x[0]+(-0.875891527959)*x[1]**o+(-0.125243159763)*x[1]+(0.887344501669)*x[2]**o+(-0.0569005112264)*x[2]
        arg[1,2,0,0]=(0.762313320036)*x[0]**o+(-0.514266376348)*x[0]+(-0.915248032301)*x[1]**o+(0.50418081892)*x[1]+(-0.867662577318)*x[2]**o+(0.731635970592)*x[2]
        arg[1,2,0,1]=(-0.0866106252969)*x[0]**o+(0.063250309716)*x[0]+(0.598723588976)*x[1]**o+(0.317235331686)*x[1]+(-0.412892601258)*x[2]**o+(0.771147670726)*x[2]
        arg[1,2,1,0]=(0.57639148624)*x[0]**o+(0.773916349481)*x[0]+(0.401319543026)*x[1]**o+(-0.510387398986)*x[1]+(-0.796773375436)*x[2]**o+(-0.765544756429)*x[2]
        arg[1,2,1,1]=(-0.686952109389)*x[0]**o+(0.428892331428)*x[0]+(-0.139971762047)*x[1]**o+(-0.747581454837)*x[1]+(0.152196734461)*x[2]**o+(-0.778424993847)*x[2]
        arg[1,2,2,0]=(0.729199771391)*x[0]**o+(0.185870258363)*x[0]+(0.996198489286)*x[1]**o+(0.0343703739018)*x[1]+(-0.6637833208)*x[2]**o+(-0.738510210409)*x[2]
        arg[1,2,2,1]=(0.705773326438)*x[0]**o+(-0.633406525448)*x[0]+(0.252620923822)*x[1]**o+(-0.760785834478)*x[1]+(0.995411456622)*x[2]**o+(-0.365718663741)*x[2]
        arg[1,3,0,0]=(0.826142335301)*x[0]**o+(-0.869335317945)*x[0]+(0.660360312658)*x[1]**o+(0.96157015332)*x[1]+(-0.268246324116)*x[2]**o+(-0.575288823295)*x[2]
        arg[1,3,0,1]=(0.620117943072)*x[0]**o+(0.51267609202)*x[0]+(-0.882468485772)*x[1]**o+(0.771021246475)*x[1]+(0.398548927211)*x[2]**o+(0.275983192446)*x[2]
        arg[1,3,1,0]=(0.716523839796)*x[0]**o+(-0.0868143348547)*x[0]+(-0.456384710114)*x[1]**o+(-0.646489063979)*x[1]+(0.594500214771)*x[2]**o+(-0.299956296467)*x[2]
        arg[1,3,1,1]=(-0.453257208906)*x[0]**o+(0.583260425119)*x[0]+(0.263463648634)*x[1]**o+(0.493693446665)*x[1]+(-0.529884773858)*x[2]**o+(0.220733486264)*x[2]
        arg[1,3,2,0]=(0.669052014654)*x[0]**o+(-0.247902780475)*x[0]+(0.44071786167)*x[1]**o+(-0.21199653848)*x[1]+(-0.0713603448391)*x[2]**o+(-0.66681439981)*x[2]
        arg[1,3,2,1]=(-0.341533813758)*x[0]**o+(0.770581995272)*x[0]+(0.03376913861)*x[1]**o+(0.000270719073091)*x[1]+(-0.774936686899)*x[2]**o+(-0.0196300167016)*x[2]
        arg[1,4,0,0]=(-0.869695667108)*x[0]**o+(0.515676789402)*x[0]+(0.67731822534)*x[1]**o+(0.126887846346)*x[1]+(0.843200966455)*x[2]**o+(-0.59930363301)*x[2]
        arg[1,4,0,1]=(0.909369352728)*x[0]**o+(-0.110828876636)*x[0]+(-0.65561953478)*x[1]**o+(0.598491879954)*x[1]+(-0.557567770163)*x[2]**o+(-0.391108926502)*x[2]
        arg[1,4,1,0]=(0.320646291934)*x[0]**o+(0.164062007697)*x[0]+(-0.546323998218)*x[1]**o+(0.72651711346)*x[1]+(-0.613496738898)*x[2]**o+(0.430761082137)*x[2]
        arg[1,4,1,1]=(-0.721651561555)*x[0]**o+(-0.0647310624509)*x[0]+(-0.360525345485)*x[1]**o+(0.408239277468)*x[1]+(0.600551705094)*x[2]**o+(0.506237401392)*x[2]
        arg[1,4,2,0]=(-0.391702965605)*x[0]**o+(-0.574470582913)*x[0]+(-0.729724609696)*x[1]**o+(0.91381180302)*x[1]+(-0.544328104929)*x[2]**o+(-0.516992534607)*x[2]
        arg[1,4,2,1]=(-0.0409666962711)*x[0]**o+(-0.524913112823)*x[0]+(-0.677030607856)*x[1]**o+(-0.530859496807)*x[1]+(0.137890059737)*x[2]**o+(0.0669856870322)*x[2]
        arg[2,0,0,0]=(0.10369318023)*x[0]**o+(0.345691611083)*x[0]+(-0.222795950561)*x[1]**o+(-0.797690270935)*x[1]+(-0.703457203234)*x[2]**o+(-0.751193209871)*x[2]
        arg[2,0,0,1]=(0.166457278787)*x[0]**o+(-0.0304766240065)*x[0]+(0.159987374581)*x[1]**o+(0.828012214748)*x[1]+(0.0670696566332)*x[2]**o+(-0.498252024306)*x[2]
        arg[2,0,1,0]=(0.901095992383)*x[0]**o+(0.400296735981)*x[0]+(0.394687295672)*x[1]**o+(-0.986496611224)*x[1]+(-0.768945602918)*x[2]**o+(-0.782762026735)*x[2]
        arg[2,0,1,1]=(0.47969549703)*x[0]**o+(0.981251232699)*x[0]+(-0.69807700503)*x[1]**o+(-0.813549271424)*x[1]+(-0.135970806025)*x[2]**o+(0.288738861745)*x[2]
        arg[2,0,2,0]=(0.315262113284)*x[0]**o+(0.0380265323361)*x[0]+(0.259894600802)*x[1]**o+(0.538712801332)*x[1]+(-0.385783597257)*x[2]**o+(0.666670257561)*x[2]
        arg[2,0,2,1]=(-0.756715539091)*x[0]**o+(-0.823809574081)*x[0]+(-0.283842954238)*x[1]**o+(-0.640166311159)*x[1]+(-0.39947011053)*x[2]**o+(-0.393543798757)*x[2]
        arg[2,1,0,0]=(-0.292033523499)*x[0]**o+(-0.940246412652)*x[0]+(-0.327571437608)*x[1]**o+(0.837077706655)*x[1]+(0.446471499818)*x[2]**o+(-0.747249149016)*x[2]
        arg[2,1,0,1]=(-0.867504469067)*x[0]**o+(0.635684140727)*x[0]+(0.488043128661)*x[1]**o+(-0.163389002402)*x[1]+(0.0440527250714)*x[2]**o+(0.605766722432)*x[2]
        arg[2,1,1,0]=(-0.0622304802635)*x[0]**o+(0.607299318698)*x[0]+(0.974933906045)*x[1]**o+(0.0632442428777)*x[1]+(0.0672808299851)*x[2]**o+(-0.555700803374)*x[2]
        arg[2,1,1,1]=(-0.350438453721)*x[0]**o+(-0.826289294567)*x[0]+(-0.473764081208)*x[1]**o+(-0.247804269494)*x[1]+(0.834301955859)*x[2]**o+(0.443058363661)*x[2]
        arg[2,1,2,0]=(0.5046026606)*x[0]**o+(0.305502438052)*x[0]+(-0.201816724695)*x[1]**o+(-0.101544576856)*x[1]+(0.729817329051)*x[2]**o+(-0.851479834334)*x[2]
        arg[2,1,2,1]=(0.917039424926)*x[0]**o+(0.260565966637)*x[0]+(-0.595846674057)*x[1]**o+(0.654773238108)*x[1]+(0.544664592565)*x[2]**o+(0.363343885358)*x[2]
        arg[2,2,0,0]=(0.243879200917)*x[0]**o+(0.335612613868)*x[0]+(-0.287066716076)*x[1]**o+(0.506206687739)*x[1]+(-0.902513036448)*x[2]**o+(-0.427017746025)*x[2]
        arg[2,2,0,1]=(0.965102871162)*x[0]**o+(0.161476912964)*x[0]+(0.97007160306)*x[1]**o+(0.870104939402)*x[1]+(0.552404991536)*x[2]**o+(-0.319082395552)*x[2]
        arg[2,2,1,0]=(0.17611459545)*x[0]**o+(-0.457419791602)*x[0]+(-0.00217752259743)*x[1]**o+(-0.632177103931)*x[1]+(0.521123797224)*x[2]**o+(0.427935307605)*x[2]
        arg[2,2,1,1]=(-0.993298850316)*x[0]**o+(0.277763727112)*x[0]+(0.849464768611)*x[1]**o+(0.890919639607)*x[1]+(0.01030894969)*x[2]**o+(-0.76051772768)*x[2]
        arg[2,2,2,0]=(-0.622887788117)*x[0]**o+(0.40187802561)*x[0]+(0.836685367934)*x[1]**o+(0.700379534201)*x[1]+(0.683627950354)*x[2]**o+(0.445351804296)*x[2]
        arg[2,2,2,1]=(0.124965536288)*x[0]**o+(-0.198462168996)*x[0]+(-0.657084126986)*x[1]**o+(-0.420469996385)*x[1]+(0.237164091597)*x[2]**o+(0.161926160295)*x[2]
        arg[2,3,0,0]=(-0.622682941897)*x[0]**o+(-0.6279729398)*x[0]+(0.189086870792)*x[1]**o+(-0.350822257394)*x[1]+(-0.505481371359)*x[2]**o+(0.812944216714)*x[2]
        arg[2,3,0,1]=(-0.246704719057)*x[0]**o+(0.513716384141)*x[0]+(-0.264914192193)*x[1]**o+(-0.32984984663)*x[1]+(-0.80409813849)*x[2]**o+(-0.825568736974)*x[2]
        arg[2,3,1,0]=(-0.534782640399)*x[0]**o+(-0.917823259622)*x[0]+(0.691827074405)*x[1]**o+(-0.818629872154)*x[1]+(0.870046992385)*x[2]**o+(0.00423218787803)*x[2]
        arg[2,3,1,1]=(0.633068676613)*x[0]**o+(-0.895613811402)*x[0]+(0.62242069294)*x[1]**o+(-0.272136050955)*x[1]+(-0.199008776357)*x[2]**o+(0.836119124508)*x[2]
        arg[2,3,2,0]=(0.335884106464)*x[0]**o+(-0.857917873801)*x[0]+(-0.42653278071)*x[1]**o+(0.375200781448)*x[1]+(0.378427453722)*x[2]**o+(0.398603623445)*x[2]
        arg[2,3,2,1]=(0.0602499604703)*x[0]**o+(-0.921459358089)*x[0]+(0.558180675902)*x[1]**o+(0.0863734870743)*x[1]+(0.215167095483)*x[2]**o+(-0.37908830738)*x[2]
        arg[2,4,0,0]=(0.866321987007)*x[0]**o+(0.81950553744)*x[0]+(0.0422552872889)*x[1]**o+(0.369328458453)*x[1]+(0.0547909452885)*x[2]**o+(-0.396367133883)*x[2]
        arg[2,4,0,1]=(0.438824337571)*x[0]**o+(0.211435787811)*x[0]+(-0.801273340861)*x[1]**o+(-0.181654271172)*x[1]+(0.374171148243)*x[2]**o+(0.945633108881)*x[2]
        arg[2,4,1,0]=(0.588044951473)*x[0]**o+(0.401384824924)*x[0]+(-0.191996429616)*x[1]**o+(0.0472579697166)*x[1]+(0.470962958517)*x[2]**o+(0.52718875619)*x[2]
        arg[2,4,1,1]=(0.371613549998)*x[0]**o+(-0.414998926249)*x[0]+(-0.762013182749)*x[1]**o+(0.796144230249)*x[1]+(-0.555041053148)*x[2]**o+(-0.595298275916)*x[2]
        arg[2,4,2,0]=(-0.992527136862)*x[0]**o+(-0.995041267483)*x[0]+(-0.730305990844)*x[1]**o+(-0.52470798566)*x[1]+(-0.938945944067)*x[2]**o+(0.33396135548)*x[2]
        arg[2,4,2,1]=(0.0333921786723)*x[0]**o+(0.707990374515)*x[0]+(-0.106667413755)*x[1]**o+(0.282989655082)*x[1]+(-0.152535982788)*x[2]**o+(-0.75130096727)*x[2]
        arg[3,0,0,0]=(-0.822675771493)*x[0]**o+(0.726468488157)*x[0]+(0.972567560814)*x[1]**o+(0.308266449093)*x[1]+(0.71069864654)*x[2]**o+(-0.683195491271)*x[2]
        arg[3,0,0,1]=(-0.505155072199)*x[0]**o+(0.339013729552)*x[0]+(-0.81565931012)*x[1]**o+(0.362458680494)*x[1]+(0.960221320064)*x[2]**o+(-0.17234580875)*x[2]
        arg[3,0,1,0]=(0.67972657926)*x[0]**o+(0.143085543693)*x[0]+(-0.699516563264)*x[1]**o+(-0.892226133751)*x[1]+(-0.898541541049)*x[2]**o+(-0.736249681714)*x[2]
        arg[3,0,1,1]=(-0.144423319215)*x[0]**o+(-0.509713525676)*x[0]+(0.347584762861)*x[1]**o+(0.666169972794)*x[1]+(0.691734080163)*x[2]**o+(0.9338062148)*x[2]
        arg[3,0,2,0]=(-0.47323843589)*x[0]**o+(0.0819655059363)*x[0]+(0.0863030211133)*x[1]**o+(0.127803089307)*x[1]+(-0.155852489756)*x[2]**o+(-0.00470398386987)*x[2]
        arg[3,0,2,1]=(0.372452726232)*x[0]**o+(0.133808609857)*x[0]+(0.962945642209)*x[1]**o+(0.0446609213929)*x[1]+(-0.446207951293)*x[2]**o+(0.0400733439868)*x[2]
        arg[3,1,0,0]=(-0.668575734235)*x[0]**o+(0.905913572459)*x[0]+(-0.321327880802)*x[1]**o+(0.0263209456188)*x[1]+(-0.0355325780368)*x[2]**o+(-0.624130395105)*x[2]
        arg[3,1,0,1]=(0.859810828483)*x[0]**o+(-0.235372125226)*x[0]+(-0.16715244612)*x[1]**o+(-0.793399872713)*x[1]+(0.303973795556)*x[2]**o+(0.176740655638)*x[2]
        arg[3,1,1,0]=(0.736448874258)*x[0]**o+(-0.448956585466)*x[0]+(0.782095902992)*x[1]**o+(0.796524196866)*x[1]+(-0.732812289541)*x[2]**o+(0.687089667291)*x[2]
        arg[3,1,1,1]=(-0.797708068194)*x[0]**o+(-0.691308116296)*x[0]+(-0.644065410151)*x[1]**o+(-0.929624672017)*x[1]+(0.265040940279)*x[2]**o+(0.424929324889)*x[2]
        arg[3,1,2,0]=(0.555032482777)*x[0]**o+(-0.564617314931)*x[0]+(-0.612721853869)*x[1]**o+(-0.048466699593)*x[1]+(0.753265037897)*x[2]**o+(0.913086239874)*x[2]
        arg[3,1,2,1]=(0.580881807771)*x[0]**o+(0.970973128604)*x[0]+(-0.603610563753)*x[1]**o+(0.979985814438)*x[1]+(-0.639531987694)*x[2]**o+(-0.773094138073)*x[2]
        arg[3,2,0,0]=(0.574306840544)*x[0]**o+(0.449094769988)*x[0]+(-0.406303076614)*x[1]**o+(-0.776648781906)*x[1]+(-0.59829476647)*x[2]**o+(0.257632446666)*x[2]
        arg[3,2,0,1]=(0.689106127051)*x[0]**o+(0.717184851708)*x[0]+(-0.495581943615)*x[1]**o+(-0.30329139788)*x[1]+(0.901753128535)*x[2]**o+(0.48496505727)*x[2]
        arg[3,2,1,0]=(0.252050902256)*x[0]**o+(-0.0414281706943)*x[0]+(-0.284125627654)*x[1]**o+(0.477450554269)*x[1]+(0.849202242148)*x[2]**o+(0.211826565889)*x[2]
        arg[3,2,1,1]=(-0.319900673571)*x[0]**o+(0.965993723739)*x[0]+(0.671360092314)*x[1]**o+(-0.652805407934)*x[1]+(-0.0352316646853)*x[2]**o+(-0.442493499306)*x[2]
        arg[3,2,2,0]=(0.525577660439)*x[0]**o+(0.660882378915)*x[0]+(-0.419826738105)*x[1]**o+(-0.798251889077)*x[1]+(-0.430382393461)*x[2]**o+(0.714357242496)*x[2]
        arg[3,2,2,1]=(-0.200634944034)*x[0]**o+(0.211086366034)*x[0]+(0.116289218188)*x[1]**o+(-0.785792860501)*x[1]+(0.763320327438)*x[2]**o+(-0.457355826175)*x[2]
        arg[3,3,0,0]=(-0.890873783782)*x[0]**o+(0.322610697657)*x[0]+(-0.351437390332)*x[1]**o+(-0.720698169708)*x[1]+(-0.664444440194)*x[2]**o+(0.274865394829)*x[2]
        arg[3,3,0,1]=(-0.384885652342)*x[0]**o+(-0.409799972422)*x[0]+(0.679315041587)*x[1]**o+(0.0859788488759)*x[1]+(-0.136173937432)*x[2]**o+(-0.136591429212)*x[2]
        arg[3,3,1,0]=(0.147934741321)*x[0]**o+(0.246941956666)*x[0]+(0.696831123708)*x[1]**o+(-0.905966322958)*x[1]+(0.672705695255)*x[2]**o+(-0.776017262395)*x[2]
        arg[3,3,1,1]=(0.933982654679)*x[0]**o+(-0.367246348547)*x[0]+(-0.254076834401)*x[1]**o+(0.148617981234)*x[1]+(-0.486881004358)*x[2]**o+(0.636371570082)*x[2]
        arg[3,3,2,0]=(-0.536125392071)*x[0]**o+(-0.663380779479)*x[0]+(0.0655694689011)*x[1]**o+(0.0390002326894)*x[1]+(-0.731730129742)*x[2]**o+(-0.578791485067)*x[2]
        arg[3,3,2,1]=(0.705873902409)*x[0]**o+(-0.0444303506955)*x[0]+(-0.978519191225)*x[1]**o+(-0.0280878186958)*x[1]+(0.182378726121)*x[2]**o+(-0.555345919696)*x[2]
        arg[3,4,0,0]=(0.945945862395)*x[0]**o+(-0.448487950632)*x[0]+(-0.477570573229)*x[1]**o+(0.313871838624)*x[1]+(0.237944220287)*x[2]**o+(-0.218095130482)*x[2]
        arg[3,4,0,1]=(-0.758647277269)*x[0]**o+(-0.550604667208)*x[0]+(-0.262944399053)*x[1]**o+(0.994302652859)*x[1]+(0.363440310008)*x[2]**o+(-0.127863970613)*x[2]
        arg[3,4,1,0]=(0.667202433965)*x[0]**o+(0.954901068821)*x[0]+(0.719745510318)*x[1]**o+(-0.763882742007)*x[1]+(0.880927476123)*x[2]**o+(0.102034630893)*x[2]
        arg[3,4,1,1]=(-0.684644024456)*x[0]**o+(0.88037275093)*x[0]+(-0.772213766289)*x[1]**o+(-0.702615214094)*x[1]+(-0.25661708053)*x[2]**o+(0.126779040694)*x[2]
        arg[3,4,2,0]=(-0.589611385303)*x[0]**o+(0.638712635252)*x[0]+(0.745450741856)*x[1]**o+(0.399494092815)*x[1]+(-0.717293809038)*x[2]**o+(0.305662737751)*x[2]
        arg[3,4,2,1]=(0.0572426997796)*x[0]**o+(-0.764600144578)*x[0]+(0.482545219565)*x[1]**o+(0.525883069411)*x[1]+(-0.612224327195)*x[2]**o+(0.354344820256)*x[2]
        ref[0,0,0,0]=(0.952750275738)*x_ref[0]**o+(0.971489204087)*x_ref[0]+(-0.320706487584)*x_ref[1]**o+(-0.853260471578)*x_ref[1]+(-0.696434862068)*x_ref[2]**o+(0.641824397433)*x_ref[2]
        ref[0,0,0,1]=(0.115484192786)*x_ref[0]**o+(0.147813687309)*x_ref[0]+(-0.998010548186)*x_ref[1]**o+(0.0274712088882)*x_ref[1]+(0.136937971489)*x_ref[2]**o+(-0.657986135444)*x_ref[2]
        ref[0,0,1,0]=(0.805857504779)*x_ref[0]**o+(0.196278916938)*x_ref[0]+(-0.969340785263)*x_ref[1]**o+(-0.543890174888)*x_ref[1]+(0.49118423357)*x_ref[2]**o+(-0.883493800593)*x_ref[2]
        ref[0,0,1,1]=(-0.808802136474)*x_ref[0]**o+(-0.866039395649)*x_ref[0]+(0.0339207025941)*x_ref[1]**o+(-0.441368773534)*x_ref[1]+(0.673688211653)*x_ref[2]**o+(0.917894389398)*x_ref[2]
        ref[0,0,2,0]=(0.0459902210078)*x_ref[0]**o+(0.862332666528)*x_ref[0]+(-0.547792306389)*x_ref[1]**o+(0.722176431732)*x_ref[1]+(-0.511278989032)*x_ref[2]**o+(-0.692775336577)*x_ref[2]
        ref[0,0,2,1]=(-0.641904406513)*x_ref[0]**o+(-0.854450656063)*x_ref[0]+(0.81138894849)*x_ref[1]**o+(0.328886625074)*x_ref[1]+(0.400935831363)*x_ref[2]**o+(-0.25184059766)*x_ref[2]
        ref[0,1,0,0]=(0.69922840942)*x_ref[0]**o+(-0.798085508443)*x_ref[0]+(-0.85191383431)*x_ref[1]**o+(-0.162940829331)*x_ref[1]+(0.504020642931)*x_ref[2]**o+(0.0950957436173)*x_ref[2]
        ref[0,1,0,1]=(0.28901429037)*x_ref[0]**o+(-0.624508745729)*x_ref[0]+(0.921288077478)*x_ref[1]**o+(-0.769207901119)*x_ref[1]+(0.0852374599763)*x_ref[2]**o+(-0.402583033986)*x_ref[2]
        ref[0,1,1,0]=(0.715817282215)*x_ref[0]**o+(0.68735256665)*x_ref[0]+(0.610212058649)*x_ref[1]**o+(0.53560824609)*x_ref[1]+(-0.238908897264)*x_ref[2]**o+(0.619799158474)*x_ref[2]
        ref[0,1,1,1]=(-0.371797414191)*x_ref[0]**o+(0.106263350514)*x_ref[0]+(0.583768355362)*x_ref[1]**o+(0.818797667369)*x_ref[1]+(0.0567571152493)*x_ref[2]**o+(-0.242244867169)*x_ref[2]
        ref[0,1,2,0]=(0.190480659729)*x_ref[0]**o+(-0.632415604771)*x_ref[0]+(0.252513826138)*x_ref[1]**o+(0.874732840469)*x_ref[1]+(-0.367769326078)*x_ref[2]**o+(0.364453292404)*x_ref[2]
        ref[0,1,2,1]=(-0.704507136581)*x_ref[0]**o+(-0.614211737547)*x_ref[0]+(0.533383507313)*x_ref[1]**o+(-0.940007583478)*x_ref[1]+(0.558260002746)*x_ref[2]**o+(0.742565692751)*x_ref[2]
        ref[0,2,0,0]=(0.132581156766)*x_ref[0]**o+(0.671793183183)*x_ref[0]+(-0.308255110769)*x_ref[1]**o+(-0.956002957061)*x_ref[1]+(0.300091541301)*x_ref[2]**o+(-0.714260746791)*x_ref[2]
        ref[0,2,0,1]=(-0.836648080859)*x_ref[0]**o+(0.123813861017)*x_ref[0]+(0.866569934894)*x_ref[1]**o+(-0.957824572351)*x_ref[1]+(-0.904478410952)*x_ref[2]**o+(0.691673758521)*x_ref[2]
        ref[0,2,1,0]=(0.411624721878)*x_ref[0]**o+(0.594054722352)*x_ref[0]+(0.150177701135)*x_ref[1]**o+(0.945855261084)*x_ref[1]+(0.295930158817)*x_ref[2]**o+(-0.607980954424)*x_ref[2]
        ref[0,2,1,1]=(-0.0777741308373)*x_ref[0]**o+(0.697501295655)*x_ref[0]+(0.981797797329)*x_ref[1]**o+(-0.671827077949)*x_ref[1]+(0.292310146229)*x_ref[2]**o+(0.484652357857)*x_ref[2]
        ref[0,2,2,0]=(0.194910469807)*x_ref[0]**o+(-0.70008515623)*x_ref[0]+(0.192808336639)*x_ref[1]**o+(-0.962764477371)*x_ref[1]+(-0.34292240753)*x_ref[2]**o+(0.525470829202)*x_ref[2]
        ref[0,2,2,1]=(0.850368420228)*x_ref[0]**o+(-0.516355386368)*x_ref[0]+(0.395764367873)*x_ref[1]**o+(0.320830752607)*x_ref[1]+(-0.26288679044)*x_ref[2]**o+(0.174457723191)*x_ref[2]
        ref[0,3,0,0]=(-0.713987556845)*x_ref[0]**o+(0.0391066248253)*x_ref[0]+(-0.63911733479)*x_ref[1]**o+(-0.119953032121)*x_ref[1]+(0.697879836447)*x_ref[2]**o+(-0.474957057702)*x_ref[2]
        ref[0,3,0,1]=(-0.128614897423)*x_ref[0]**o+(0.7166036403)*x_ref[0]+(0.678415978447)*x_ref[1]**o+(-0.191453169618)*x_ref[1]+(-0.755261361956)*x_ref[2]**o+(-0.839956004196)*x_ref[2]
        ref[0,3,1,0]=(0.258158740934)*x_ref[0]**o+(-0.412199346967)*x_ref[0]+(-0.781106341272)*x_ref[1]**o+(-0.561162823101)*x_ref[1]+(-0.59549864377)*x_ref[2]**o+(-0.748038415721)*x_ref[2]
        ref[0,3,1,1]=(0.68950222305)*x_ref[0]**o+(-0.904027774445)*x_ref[0]+(0.769744453441)*x_ref[1]**o+(-0.265549152752)*x_ref[1]+(-0.375978837131)*x_ref[2]**o+(-0.387010981761)*x_ref[2]
        ref[0,3,2,0]=(0.46852681319)*x_ref[0]**o+(-0.887650258615)*x_ref[0]+(-0.780275624319)*x_ref[1]**o+(0.527897372599)*x_ref[1]+(0.889281778411)*x_ref[2]**o+(0.82538563877)*x_ref[2]
        ref[0,3,2,1]=(-0.817514448184)*x_ref[0]**o+(-0.230190768519)*x_ref[0]+(-0.101550849344)*x_ref[1]**o+(0.1654885857)*x_ref[1]+(-0.231317749498)*x_ref[2]**o+(0.580153306395)*x_ref[2]
        ref[0,4,0,0]=(0.394993007242)*x_ref[0]**o+(0.305948764395)*x_ref[0]+(-0.66853441368)*x_ref[1]**o+(0.279027498094)*x_ref[1]+(0.161366934709)*x_ref[2]**o+(-0.742002471552)*x_ref[2]
        ref[0,4,0,1]=(-0.661214709471)*x_ref[0]**o+(0.548880317935)*x_ref[0]+(0.472682748503)*x_ref[1]**o+(0.266797505699)*x_ref[1]+(0.405618413683)*x_ref[2]**o+(-0.0539677542667)*x_ref[2]
        ref[0,4,1,0]=(-0.713892424429)*x_ref[0]**o+(0.568075831592)*x_ref[0]+(-0.956180895599)*x_ref[1]**o+(0.63618957695)*x_ref[1]+(0.983696160047)*x_ref[2]**o+(-0.308157929002)*x_ref[2]
        ref[0,4,1,1]=(-0.666582919353)*x_ref[0]**o+(0.0816884254484)*x_ref[0]+(0.190810048896)*x_ref[1]**o+(0.482322905132)*x_ref[1]+(-0.576597070954)*x_ref[2]**o+(-0.722381611553)*x_ref[2]
        ref[0,4,2,0]=(-0.0229650731041)*x_ref[0]**o+(-0.0867816754979)*x_ref[0]+(0.0905042102365)*x_ref[1]**o+(-0.994963424765)*x_ref[1]+(0.74504757815)*x_ref[2]**o+(0.71462549644)*x_ref[2]
        ref[0,4,2,1]=(-0.0464278496147)*x_ref[0]**o+(0.122501026996)*x_ref[0]+(-0.049450141901)*x_ref[1]**o+(-0.388739464086)*x_ref[1]+(-0.151389885067)*x_ref[2]**o+(-0.0316948088803)*x_ref[2]
        ref[1,0,0,0]=(-0.303653386851)*x_ref[0]**o+(-0.435866753659)*x_ref[0]+(-0.419078175082)*x_ref[1]**o+(-0.907595244917)*x_ref[1]+(0.442156528837)*x_ref[2]**o+(-0.76481785523)*x_ref[2]
        ref[1,0,0,1]=(0.879454551618)*x_ref[0]**o+(-0.530789858588)*x_ref[0]+(0.942203776542)*x_ref[1]**o+(0.864362935436)*x_ref[1]+(-0.252832164588)*x_ref[2]**o+(0.00758252581015)*x_ref[2]
        ref[1,0,1,0]=(0.522433142898)*x_ref[0]**o+(0.564002257569)*x_ref[0]+(0.149811689377)*x_ref[1]**o+(-0.757726948165)*x_ref[1]+(-0.767052869719)*x_ref[2]**o+(0.743541806001)*x_ref[2]
        ref[1,0,1,1]=(0.535612198186)*x_ref[0]**o+(0.0874480088401)*x_ref[0]+(0.913151292296)*x_ref[1]**o+(-0.495024655308)*x_ref[1]+(-0.42308905182)*x_ref[2]**o+(-0.983623551153)*x_ref[2]
        ref[1,0,2,0]=(0.587221456352)*x_ref[0]**o+(0.0384438535009)*x_ref[0]+(0.301554087927)*x_ref[1]**o+(-0.944074061806)*x_ref[1]+(0.808195432991)*x_ref[2]**o+(-0.46545743768)*x_ref[2]
        ref[1,0,2,1]=(0.136251169669)*x_ref[0]**o+(0.772190271875)*x_ref[0]+(-0.490981224022)*x_ref[1]**o+(-0.191915588448)*x_ref[1]+(0.792617864405)*x_ref[2]**o+(0.139425359773)*x_ref[2]
        ref[1,1,0,0]=(0.0440493146329)*x_ref[0]**o+(-0.948951787109)*x_ref[0]+(-0.564104026105)*x_ref[1]**o+(0.177457883377)*x_ref[1]+(0.578169345377)*x_ref[2]**o+(0.35574407049)*x_ref[2]
        ref[1,1,0,1]=(-0.740305651819)*x_ref[0]**o+(-0.340976006932)*x_ref[0]+(-0.657257828033)*x_ref[1]**o+(-0.456240473558)*x_ref[1]+(0.526236681039)*x_ref[2]**o+(0.804762801115)*x_ref[2]
        ref[1,1,1,0]=(-0.753480536304)*x_ref[0]**o+(0.424080516297)*x_ref[0]+(-0.97529611006)*x_ref[1]**o+(-0.511450162624)*x_ref[1]+(-0.248982226385)*x_ref[2]**o+(0.497803766242)*x_ref[2]
        ref[1,1,1,1]=(-0.273280180629)*x_ref[0]**o+(0.798945176097)*x_ref[0]+(0.552152546131)*x_ref[1]**o+(0.737594404744)*x_ref[1]+(0.0339269287984)*x_ref[2]**o+(-0.773791176515)*x_ref[2]
        ref[1,1,2,0]=(0.553404075378)*x_ref[0]**o+(-0.630969888585)*x_ref[0]+(0.455072287913)*x_ref[1]**o+(0.145928503781)*x_ref[1]+(0.632480470504)*x_ref[2]**o+(0.363815684296)*x_ref[2]
        ref[1,1,2,1]=(-0.426756293656)*x_ref[0]**o+(-0.449867410734)*x_ref[0]+(-0.875891527959)*x_ref[1]**o+(-0.125243159763)*x_ref[1]+(0.887344501669)*x_ref[2]**o+(-0.0569005112264)*x_ref[2]
        ref[1,2,0,0]=(0.762313320036)*x_ref[0]**o+(-0.514266376348)*x_ref[0]+(-0.915248032301)*x_ref[1]**o+(0.50418081892)*x_ref[1]+(-0.867662577318)*x_ref[2]**o+(0.731635970592)*x_ref[2]
        ref[1,2,0,1]=(-0.0866106252969)*x_ref[0]**o+(0.063250309716)*x_ref[0]+(0.598723588976)*x_ref[1]**o+(0.317235331686)*x_ref[1]+(-0.412892601258)*x_ref[2]**o+(0.771147670726)*x_ref[2]
        ref[1,2,1,0]=(0.57639148624)*x_ref[0]**o+(0.773916349481)*x_ref[0]+(0.401319543026)*x_ref[1]**o+(-0.510387398986)*x_ref[1]+(-0.796773375436)*x_ref[2]**o+(-0.765544756429)*x_ref[2]
        ref[1,2,1,1]=(-0.686952109389)*x_ref[0]**o+(0.428892331428)*x_ref[0]+(-0.139971762047)*x_ref[1]**o+(-0.747581454837)*x_ref[1]+(0.152196734461)*x_ref[2]**o+(-0.778424993847)*x_ref[2]
        ref[1,2,2,0]=(0.729199771391)*x_ref[0]**o+(0.185870258363)*x_ref[0]+(0.996198489286)*x_ref[1]**o+(0.0343703739018)*x_ref[1]+(-0.6637833208)*x_ref[2]**o+(-0.738510210409)*x_ref[2]
        ref[1,2,2,1]=(0.705773326438)*x_ref[0]**o+(-0.633406525448)*x_ref[0]+(0.252620923822)*x_ref[1]**o+(-0.760785834478)*x_ref[1]+(0.995411456622)*x_ref[2]**o+(-0.365718663741)*x_ref[2]
        ref[1,3,0,0]=(0.826142335301)*x_ref[0]**o+(-0.869335317945)*x_ref[0]+(0.660360312658)*x_ref[1]**o+(0.96157015332)*x_ref[1]+(-0.268246324116)*x_ref[2]**o+(-0.575288823295)*x_ref[2]
        ref[1,3,0,1]=(0.620117943072)*x_ref[0]**o+(0.51267609202)*x_ref[0]+(-0.882468485772)*x_ref[1]**o+(0.771021246475)*x_ref[1]+(0.398548927211)*x_ref[2]**o+(0.275983192446)*x_ref[2]
        ref[1,3,1,0]=(0.716523839796)*x_ref[0]**o+(-0.0868143348547)*x_ref[0]+(-0.456384710114)*x_ref[1]**o+(-0.646489063979)*x_ref[1]+(0.594500214771)*x_ref[2]**o+(-0.299956296467)*x_ref[2]
        ref[1,3,1,1]=(-0.453257208906)*x_ref[0]**o+(0.583260425119)*x_ref[0]+(0.263463648634)*x_ref[1]**o+(0.493693446665)*x_ref[1]+(-0.529884773858)*x_ref[2]**o+(0.220733486264)*x_ref[2]
        ref[1,3,2,0]=(0.669052014654)*x_ref[0]**o+(-0.247902780475)*x_ref[0]+(0.44071786167)*x_ref[1]**o+(-0.21199653848)*x_ref[1]+(-0.0713603448391)*x_ref[2]**o+(-0.66681439981)*x_ref[2]
        ref[1,3,2,1]=(-0.341533813758)*x_ref[0]**o+(0.770581995272)*x_ref[0]+(0.03376913861)*x_ref[1]**o+(0.000270719073091)*x_ref[1]+(-0.774936686899)*x_ref[2]**o+(-0.0196300167016)*x_ref[2]
        ref[1,4,0,0]=(-0.869695667108)*x_ref[0]**o+(0.515676789402)*x_ref[0]+(0.67731822534)*x_ref[1]**o+(0.126887846346)*x_ref[1]+(0.843200966455)*x_ref[2]**o+(-0.59930363301)*x_ref[2]
        ref[1,4,0,1]=(0.909369352728)*x_ref[0]**o+(-0.110828876636)*x_ref[0]+(-0.65561953478)*x_ref[1]**o+(0.598491879954)*x_ref[1]+(-0.557567770163)*x_ref[2]**o+(-0.391108926502)*x_ref[2]
        ref[1,4,1,0]=(0.320646291934)*x_ref[0]**o+(0.164062007697)*x_ref[0]+(-0.546323998218)*x_ref[1]**o+(0.72651711346)*x_ref[1]+(-0.613496738898)*x_ref[2]**o+(0.430761082137)*x_ref[2]
        ref[1,4,1,1]=(-0.721651561555)*x_ref[0]**o+(-0.0647310624509)*x_ref[0]+(-0.360525345485)*x_ref[1]**o+(0.408239277468)*x_ref[1]+(0.600551705094)*x_ref[2]**o+(0.506237401392)*x_ref[2]
        ref[1,4,2,0]=(-0.391702965605)*x_ref[0]**o+(-0.574470582913)*x_ref[0]+(-0.729724609696)*x_ref[1]**o+(0.91381180302)*x_ref[1]+(-0.544328104929)*x_ref[2]**o+(-0.516992534607)*x_ref[2]
        ref[1,4,2,1]=(-0.0409666962711)*x_ref[0]**o+(-0.524913112823)*x_ref[0]+(-0.677030607856)*x_ref[1]**o+(-0.530859496807)*x_ref[1]+(0.137890059737)*x_ref[2]**o+(0.0669856870322)*x_ref[2]
        ref[2,0,0,0]=(0.10369318023)*x_ref[0]**o+(0.345691611083)*x_ref[0]+(-0.222795950561)*x_ref[1]**o+(-0.797690270935)*x_ref[1]+(-0.703457203234)*x_ref[2]**o+(-0.751193209871)*x_ref[2]
        ref[2,0,0,1]=(0.166457278787)*x_ref[0]**o+(-0.0304766240065)*x_ref[0]+(0.159987374581)*x_ref[1]**o+(0.828012214748)*x_ref[1]+(0.0670696566332)*x_ref[2]**o+(-0.498252024306)*x_ref[2]
        ref[2,0,1,0]=(0.901095992383)*x_ref[0]**o+(0.400296735981)*x_ref[0]+(0.394687295672)*x_ref[1]**o+(-0.986496611224)*x_ref[1]+(-0.768945602918)*x_ref[2]**o+(-0.782762026735)*x_ref[2]
        ref[2,0,1,1]=(0.47969549703)*x_ref[0]**o+(0.981251232699)*x_ref[0]+(-0.69807700503)*x_ref[1]**o+(-0.813549271424)*x_ref[1]+(-0.135970806025)*x_ref[2]**o+(0.288738861745)*x_ref[2]
        ref[2,0,2,0]=(0.315262113284)*x_ref[0]**o+(0.0380265323361)*x_ref[0]+(0.259894600802)*x_ref[1]**o+(0.538712801332)*x_ref[1]+(-0.385783597257)*x_ref[2]**o+(0.666670257561)*x_ref[2]
        ref[2,0,2,1]=(-0.756715539091)*x_ref[0]**o+(-0.823809574081)*x_ref[0]+(-0.283842954238)*x_ref[1]**o+(-0.640166311159)*x_ref[1]+(-0.39947011053)*x_ref[2]**o+(-0.393543798757)*x_ref[2]
        ref[2,1,0,0]=(-0.292033523499)*x_ref[0]**o+(-0.940246412652)*x_ref[0]+(-0.327571437608)*x_ref[1]**o+(0.837077706655)*x_ref[1]+(0.446471499818)*x_ref[2]**o+(-0.747249149016)*x_ref[2]
        ref[2,1,0,1]=(-0.867504469067)*x_ref[0]**o+(0.635684140727)*x_ref[0]+(0.488043128661)*x_ref[1]**o+(-0.163389002402)*x_ref[1]+(0.0440527250714)*x_ref[2]**o+(0.605766722432)*x_ref[2]
        ref[2,1,1,0]=(-0.0622304802635)*x_ref[0]**o+(0.607299318698)*x_ref[0]+(0.974933906045)*x_ref[1]**o+(0.0632442428777)*x_ref[1]+(0.0672808299851)*x_ref[2]**o+(-0.555700803374)*x_ref[2]
        ref[2,1,1,1]=(-0.350438453721)*x_ref[0]**o+(-0.826289294567)*x_ref[0]+(-0.473764081208)*x_ref[1]**o+(-0.247804269494)*x_ref[1]+(0.834301955859)*x_ref[2]**o+(0.443058363661)*x_ref[2]
        ref[2,1,2,0]=(0.5046026606)*x_ref[0]**o+(0.305502438052)*x_ref[0]+(-0.201816724695)*x_ref[1]**o+(-0.101544576856)*x_ref[1]+(0.729817329051)*x_ref[2]**o+(-0.851479834334)*x_ref[2]
        ref[2,1,2,1]=(0.917039424926)*x_ref[0]**o+(0.260565966637)*x_ref[0]+(-0.595846674057)*x_ref[1]**o+(0.654773238108)*x_ref[1]+(0.544664592565)*x_ref[2]**o+(0.363343885358)*x_ref[2]
        ref[2,2,0,0]=(0.243879200917)*x_ref[0]**o+(0.335612613868)*x_ref[0]+(-0.287066716076)*x_ref[1]**o+(0.506206687739)*x_ref[1]+(-0.902513036448)*x_ref[2]**o+(-0.427017746025)*x_ref[2]
        ref[2,2,0,1]=(0.965102871162)*x_ref[0]**o+(0.161476912964)*x_ref[0]+(0.97007160306)*x_ref[1]**o+(0.870104939402)*x_ref[1]+(0.552404991536)*x_ref[2]**o+(-0.319082395552)*x_ref[2]
        ref[2,2,1,0]=(0.17611459545)*x_ref[0]**o+(-0.457419791602)*x_ref[0]+(-0.00217752259743)*x_ref[1]**o+(-0.632177103931)*x_ref[1]+(0.521123797224)*x_ref[2]**o+(0.427935307605)*x_ref[2]
        ref[2,2,1,1]=(-0.993298850316)*x_ref[0]**o+(0.277763727112)*x_ref[0]+(0.849464768611)*x_ref[1]**o+(0.890919639607)*x_ref[1]+(0.01030894969)*x_ref[2]**o+(-0.76051772768)*x_ref[2]
        ref[2,2,2,0]=(-0.622887788117)*x_ref[0]**o+(0.40187802561)*x_ref[0]+(0.836685367934)*x_ref[1]**o+(0.700379534201)*x_ref[1]+(0.683627950354)*x_ref[2]**o+(0.445351804296)*x_ref[2]
        ref[2,2,2,1]=(0.124965536288)*x_ref[0]**o+(-0.198462168996)*x_ref[0]+(-0.657084126986)*x_ref[1]**o+(-0.420469996385)*x_ref[1]+(0.237164091597)*x_ref[2]**o+(0.161926160295)*x_ref[2]
        ref[2,3,0,0]=(-0.622682941897)*x_ref[0]**o+(-0.6279729398)*x_ref[0]+(0.189086870792)*x_ref[1]**o+(-0.350822257394)*x_ref[1]+(-0.505481371359)*x_ref[2]**o+(0.812944216714)*x_ref[2]
        ref[2,3,0,1]=(-0.246704719057)*x_ref[0]**o+(0.513716384141)*x_ref[0]+(-0.264914192193)*x_ref[1]**o+(-0.32984984663)*x_ref[1]+(-0.80409813849)*x_ref[2]**o+(-0.825568736974)*x_ref[2]
        ref[2,3,1,0]=(-0.534782640399)*x_ref[0]**o+(-0.917823259622)*x_ref[0]+(0.691827074405)*x_ref[1]**o+(-0.818629872154)*x_ref[1]+(0.870046992385)*x_ref[2]**o+(0.00423218787803)*x_ref[2]
        ref[2,3,1,1]=(0.633068676613)*x_ref[0]**o+(-0.895613811402)*x_ref[0]+(0.62242069294)*x_ref[1]**o+(-0.272136050955)*x_ref[1]+(-0.199008776357)*x_ref[2]**o+(0.836119124508)*x_ref[2]
        ref[2,3,2,0]=(0.335884106464)*x_ref[0]**o+(-0.857917873801)*x_ref[0]+(-0.42653278071)*x_ref[1]**o+(0.375200781448)*x_ref[1]+(0.378427453722)*x_ref[2]**o+(0.398603623445)*x_ref[2]
        ref[2,3,2,1]=(0.0602499604703)*x_ref[0]**o+(-0.921459358089)*x_ref[0]+(0.558180675902)*x_ref[1]**o+(0.0863734870743)*x_ref[1]+(0.215167095483)*x_ref[2]**o+(-0.37908830738)*x_ref[2]
        ref[2,4,0,0]=(0.866321987007)*x_ref[0]**o+(0.81950553744)*x_ref[0]+(0.0422552872889)*x_ref[1]**o+(0.369328458453)*x_ref[1]+(0.0547909452885)*x_ref[2]**o+(-0.396367133883)*x_ref[2]
        ref[2,4,0,1]=(0.438824337571)*x_ref[0]**o+(0.211435787811)*x_ref[0]+(-0.801273340861)*x_ref[1]**o+(-0.181654271172)*x_ref[1]+(0.374171148243)*x_ref[2]**o+(0.945633108881)*x_ref[2]
        ref[2,4,1,0]=(0.588044951473)*x_ref[0]**o+(0.401384824924)*x_ref[0]+(-0.191996429616)*x_ref[1]**o+(0.0472579697166)*x_ref[1]+(0.470962958517)*x_ref[2]**o+(0.52718875619)*x_ref[2]
        ref[2,4,1,1]=(0.371613549998)*x_ref[0]**o+(-0.414998926249)*x_ref[0]+(-0.762013182749)*x_ref[1]**o+(0.796144230249)*x_ref[1]+(-0.555041053148)*x_ref[2]**o+(-0.595298275916)*x_ref[2]
        ref[2,4,2,0]=(-0.992527136862)*x_ref[0]**o+(-0.995041267483)*x_ref[0]+(-0.730305990844)*x_ref[1]**o+(-0.52470798566)*x_ref[1]+(-0.938945944067)*x_ref[2]**o+(0.33396135548)*x_ref[2]
        ref[2,4,2,1]=(0.0333921786723)*x_ref[0]**o+(0.707990374515)*x_ref[0]+(-0.106667413755)*x_ref[1]**o+(0.282989655082)*x_ref[1]+(-0.152535982788)*x_ref[2]**o+(-0.75130096727)*x_ref[2]
        ref[3,0,0,0]=(-0.822675771493)*x_ref[0]**o+(0.726468488157)*x_ref[0]+(0.972567560814)*x_ref[1]**o+(0.308266449093)*x_ref[1]+(0.71069864654)*x_ref[2]**o+(-0.683195491271)*x_ref[2]
        ref[3,0,0,1]=(-0.505155072199)*x_ref[0]**o+(0.339013729552)*x_ref[0]+(-0.81565931012)*x_ref[1]**o+(0.362458680494)*x_ref[1]+(0.960221320064)*x_ref[2]**o+(-0.17234580875)*x_ref[2]
        ref[3,0,1,0]=(0.67972657926)*x_ref[0]**o+(0.143085543693)*x_ref[0]+(-0.699516563264)*x_ref[1]**o+(-0.892226133751)*x_ref[1]+(-0.898541541049)*x_ref[2]**o+(-0.736249681714)*x_ref[2]
        ref[3,0,1,1]=(-0.144423319215)*x_ref[0]**o+(-0.509713525676)*x_ref[0]+(0.347584762861)*x_ref[1]**o+(0.666169972794)*x_ref[1]+(0.691734080163)*x_ref[2]**o+(0.9338062148)*x_ref[2]
        ref[3,0,2,0]=(-0.47323843589)*x_ref[0]**o+(0.0819655059363)*x_ref[0]+(0.0863030211133)*x_ref[1]**o+(0.127803089307)*x_ref[1]+(-0.155852489756)*x_ref[2]**o+(-0.00470398386987)*x_ref[2]
        ref[3,0,2,1]=(0.372452726232)*x_ref[0]**o+(0.133808609857)*x_ref[0]+(0.962945642209)*x_ref[1]**o+(0.0446609213929)*x_ref[1]+(-0.446207951293)*x_ref[2]**o+(0.0400733439868)*x_ref[2]
        ref[3,1,0,0]=(-0.668575734235)*x_ref[0]**o+(0.905913572459)*x_ref[0]+(-0.321327880802)*x_ref[1]**o+(0.0263209456188)*x_ref[1]+(-0.0355325780368)*x_ref[2]**o+(-0.624130395105)*x_ref[2]
        ref[3,1,0,1]=(0.859810828483)*x_ref[0]**o+(-0.235372125226)*x_ref[0]+(-0.16715244612)*x_ref[1]**o+(-0.793399872713)*x_ref[1]+(0.303973795556)*x_ref[2]**o+(0.176740655638)*x_ref[2]
        ref[3,1,1,0]=(0.736448874258)*x_ref[0]**o+(-0.448956585466)*x_ref[0]+(0.782095902992)*x_ref[1]**o+(0.796524196866)*x_ref[1]+(-0.732812289541)*x_ref[2]**o+(0.687089667291)*x_ref[2]
        ref[3,1,1,1]=(-0.797708068194)*x_ref[0]**o+(-0.691308116296)*x_ref[0]+(-0.644065410151)*x_ref[1]**o+(-0.929624672017)*x_ref[1]+(0.265040940279)*x_ref[2]**o+(0.424929324889)*x_ref[2]
        ref[3,1,2,0]=(0.555032482777)*x_ref[0]**o+(-0.564617314931)*x_ref[0]+(-0.612721853869)*x_ref[1]**o+(-0.048466699593)*x_ref[1]+(0.753265037897)*x_ref[2]**o+(0.913086239874)*x_ref[2]
        ref[3,1,2,1]=(0.580881807771)*x_ref[0]**o+(0.970973128604)*x_ref[0]+(-0.603610563753)*x_ref[1]**o+(0.979985814438)*x_ref[1]+(-0.639531987694)*x_ref[2]**o+(-0.773094138073)*x_ref[2]
        ref[3,2,0,0]=(0.574306840544)*x_ref[0]**o+(0.449094769988)*x_ref[0]+(-0.406303076614)*x_ref[1]**o+(-0.776648781906)*x_ref[1]+(-0.59829476647)*x_ref[2]**o+(0.257632446666)*x_ref[2]
        ref[3,2,0,1]=(0.689106127051)*x_ref[0]**o+(0.717184851708)*x_ref[0]+(-0.495581943615)*x_ref[1]**o+(-0.30329139788)*x_ref[1]+(0.901753128535)*x_ref[2]**o+(0.48496505727)*x_ref[2]
        ref[3,2,1,0]=(0.252050902256)*x_ref[0]**o+(-0.0414281706943)*x_ref[0]+(-0.284125627654)*x_ref[1]**o+(0.477450554269)*x_ref[1]+(0.849202242148)*x_ref[2]**o+(0.211826565889)*x_ref[2]
        ref[3,2,1,1]=(-0.319900673571)*x_ref[0]**o+(0.965993723739)*x_ref[0]+(0.671360092314)*x_ref[1]**o+(-0.652805407934)*x_ref[1]+(-0.0352316646853)*x_ref[2]**o+(-0.442493499306)*x_ref[2]
        ref[3,2,2,0]=(0.525577660439)*x_ref[0]**o+(0.660882378915)*x_ref[0]+(-0.419826738105)*x_ref[1]**o+(-0.798251889077)*x_ref[1]+(-0.430382393461)*x_ref[2]**o+(0.714357242496)*x_ref[2]
        ref[3,2,2,1]=(-0.200634944034)*x_ref[0]**o+(0.211086366034)*x_ref[0]+(0.116289218188)*x_ref[1]**o+(-0.785792860501)*x_ref[1]+(0.763320327438)*x_ref[2]**o+(-0.457355826175)*x_ref[2]
        ref[3,3,0,0]=(-0.890873783782)*x_ref[0]**o+(0.322610697657)*x_ref[0]+(-0.351437390332)*x_ref[1]**o+(-0.720698169708)*x_ref[1]+(-0.664444440194)*x_ref[2]**o+(0.274865394829)*x_ref[2]
        ref[3,3,0,1]=(-0.384885652342)*x_ref[0]**o+(-0.409799972422)*x_ref[0]+(0.679315041587)*x_ref[1]**o+(0.0859788488759)*x_ref[1]+(-0.136173937432)*x_ref[2]**o+(-0.136591429212)*x_ref[2]
        ref[3,3,1,0]=(0.147934741321)*x_ref[0]**o+(0.246941956666)*x_ref[0]+(0.696831123708)*x_ref[1]**o+(-0.905966322958)*x_ref[1]+(0.672705695255)*x_ref[2]**o+(-0.776017262395)*x_ref[2]
        ref[3,3,1,1]=(0.933982654679)*x_ref[0]**o+(-0.367246348547)*x_ref[0]+(-0.254076834401)*x_ref[1]**o+(0.148617981234)*x_ref[1]+(-0.486881004358)*x_ref[2]**o+(0.636371570082)*x_ref[2]
        ref[3,3,2,0]=(-0.536125392071)*x_ref[0]**o+(-0.663380779479)*x_ref[0]+(0.0655694689011)*x_ref[1]**o+(0.0390002326894)*x_ref[1]+(-0.731730129742)*x_ref[2]**o+(-0.578791485067)*x_ref[2]
        ref[3,3,2,1]=(0.705873902409)*x_ref[0]**o+(-0.0444303506955)*x_ref[0]+(-0.978519191225)*x_ref[1]**o+(-0.0280878186958)*x_ref[1]+(0.182378726121)*x_ref[2]**o+(-0.555345919696)*x_ref[2]
        ref[3,4,0,0]=(0.945945862395)*x_ref[0]**o+(-0.448487950632)*x_ref[0]+(-0.477570573229)*x_ref[1]**o+(0.313871838624)*x_ref[1]+(0.237944220287)*x_ref[2]**o+(-0.218095130482)*x_ref[2]
        ref[3,4,0,1]=(-0.758647277269)*x_ref[0]**o+(-0.550604667208)*x_ref[0]+(-0.262944399053)*x_ref[1]**o+(0.994302652859)*x_ref[1]+(0.363440310008)*x_ref[2]**o+(-0.127863970613)*x_ref[2]
        ref[3,4,1,0]=(0.667202433965)*x_ref[0]**o+(0.954901068821)*x_ref[0]+(0.719745510318)*x_ref[1]**o+(-0.763882742007)*x_ref[1]+(0.880927476123)*x_ref[2]**o+(0.102034630893)*x_ref[2]
        ref[3,4,1,1]=(-0.684644024456)*x_ref[0]**o+(0.88037275093)*x_ref[0]+(-0.772213766289)*x_ref[1]**o+(-0.702615214094)*x_ref[1]+(-0.25661708053)*x_ref[2]**o+(0.126779040694)*x_ref[2]
        ref[3,4,2,0]=(-0.589611385303)*x_ref[0]**o+(0.638712635252)*x_ref[0]+(0.745450741856)*x_ref[1]**o+(0.399494092815)*x_ref[1]+(-0.717293809038)*x_ref[2]**o+(0.305662737751)*x_ref[2]
        ref[3,4,2,1]=(0.0572426997796)*x_ref[0]**o+(-0.764600144578)*x_ref[0]+(0.482545219565)*x_ref[1]**o+(0.525883069411)*x_ref[1]+(-0.612224327195)*x_ref[2]**o+(0.354344820256)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromData_ReducedSolution_rank0(self):
      """
      tests interpolate for rank 0 Data onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedSolution onto ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Data(0,(),w)
      ref=Data(0,(),w_ref)
      if dim==2:
        arg=(-0.100067148208)*x[0]+(-0.8375743725)*x[1]
        ref=(-0.100067148208)*x_ref[0]+(-0.8375743725)*x_ref[1]
      else:
        arg=(-0.247343416726)*x[0]+(0.604193484164)*x[1]+(-0.219899438287)*x[2]
        ref=(-0.247343416726)*x_ref[0]+(0.604193484164)*x_ref[1]+(-0.219899438287)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromData_ReducedSolution_rank1(self):
      """
      tests interpolate for rank 1 Data onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedSolution onto ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Data(0,(2,),w)
      ref=Data(0,(2,),w_ref)
      if dim==2:
        arg[0]=(-1.66196653104)*x[0]+(1.29891335227)*x[1]
        arg[1]=(-0.347844771248)*x[0]+(-0.404726300584)*x[1]
        ref[0]=(-1.66196653104)*x[0]+(1.29891335227)*x[1]
        ref[1]=(-0.347844771248)*x[0]+(-0.404726300584)*x[1]
      else:
        arg[0]=(0.334094742067)*x[0]+(-0.619063646554)*x[1]+(0.538572422001)*x[2]
        arg[1]=(0.673290433004)*x[0]+(0.169501972843)*x[1]+(-0.833663493893)*x[2]
        ref[0]=(0.334094742067)*x[0]+(-0.619063646554)*x[1]+(0.538572422001)*x[2]
        ref[1]=(0.673290433004)*x[0]+(0.169501972843)*x[1]+(-0.833663493893)*x[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromData_ReducedSolution_rank2(self):
      """
      tests interpolate for rank 2 Data onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedSolution onto ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5),w)
      ref=Data(0,(4, 5),w_ref)
      if dim==2:
        arg[0,0]=(0.00153698933322)*x[0]+(0.392281645764)*x[1]
        arg[0,1]=(-1.2794655544)*x[0]+(-0.373519639818)*x[1]
        arg[0,2]=(0.30127033321)*x[0]+(-0.0996865853738)*x[1]
        arg[0,3]=(1.23405799349)*x[0]+(0.498558200339)*x[1]
        arg[0,4]=(0.85519350742)*x[0]+(-0.19188046135)*x[1]
        arg[1,0]=(0.417367834962)*x[0]+(0.992318201639)*x[1]
        arg[1,1]=(-0.646432128833)*x[0]+(-0.872750321582)*x[1]
        arg[1,2]=(-0.806117185061)*x[0]+(-0.89396408325)*x[1]
        arg[1,3]=(0.768017237627)*x[0]+(-1.2904111912)*x[1]
        arg[1,4]=(-0.0159685523188)*x[0]+(0.401410193169)*x[1]
        arg[2,0]=(-0.844501716181)*x[0]+(-0.123115343819)*x[1]
        arg[2,1]=(1.08441909409)*x[0]+(0.152432282539)*x[1]
        arg[2,2]=(-0.60551037413)*x[0]+(1.15692828826)*x[1]
        arg[2,3]=(-1.21509168541)*x[0]+(1.03683236377)*x[1]
        arg[2,4]=(1.16850730134)*x[0]+(0.162202615909)*x[1]
        arg[3,0]=(0.727772166897)*x[0]+(-0.296815840935)*x[1]
        arg[3,1]=(-0.0880577151489)*x[0]+(1.66427691426)*x[1]
        arg[3,2]=(0.415144534458)*x[0]+(-0.555833723295)*x[1]
        arg[3,3]=(1.09444635829)*x[0]+(-0.335209417536)*x[1]
        arg[3,4]=(-1.05047769585)*x[0]+(0.684427039986)*x[1]
        ref[0,0]=(0.00153698933322)*x_ref[0]+(0.392281645764)*x_ref[1]
        ref[0,1]=(-1.2794655544)*x_ref[0]+(-0.373519639818)*x_ref[1]
        ref[0,2]=(0.30127033321)*x_ref[0]+(-0.0996865853738)*x_ref[1]
        ref[0,3]=(1.23405799349)*x_ref[0]+(0.498558200339)*x_ref[1]
        ref[0,4]=(0.85519350742)*x_ref[0]+(-0.19188046135)*x_ref[1]
        ref[1,0]=(0.417367834962)*x_ref[0]+(0.992318201639)*x_ref[1]
        ref[1,1]=(-0.646432128833)*x_ref[0]+(-0.872750321582)*x_ref[1]
        ref[1,2]=(-0.806117185061)*x_ref[0]+(-0.89396408325)*x_ref[1]
        ref[1,3]=(0.768017237627)*x_ref[0]+(-1.2904111912)*x_ref[1]
        ref[1,4]=(-0.0159685523188)*x_ref[0]+(0.401410193169)*x_ref[1]
        ref[2,0]=(-0.844501716181)*x_ref[0]+(-0.123115343819)*x_ref[1]
        ref[2,1]=(1.08441909409)*x_ref[0]+(0.152432282539)*x_ref[1]
        ref[2,2]=(-0.60551037413)*x_ref[0]+(1.15692828826)*x_ref[1]
        ref[2,3]=(-1.21509168541)*x_ref[0]+(1.03683236377)*x_ref[1]
        ref[2,4]=(1.16850730134)*x_ref[0]+(0.162202615909)*x_ref[1]
        ref[3,0]=(0.727772166897)*x_ref[0]+(-0.296815840935)*x_ref[1]
        ref[3,1]=(-0.0880577151489)*x_ref[0]+(1.66427691426)*x_ref[1]
        ref[3,2]=(0.415144534458)*x_ref[0]+(-0.555833723295)*x_ref[1]
        ref[3,3]=(1.09444635829)*x_ref[0]+(-0.335209417536)*x_ref[1]
        ref[3,4]=(-1.05047769585)*x_ref[0]+(0.684427039986)*x_ref[1]
      else:
        arg[0,0]=(-0.611607640773)*x[0]+(0.584485971628)*x[1]+(0.0090316337643)*x[2]
        arg[0,1]=(0.905278285857)*x[0]+(0.332523151242)*x[1]+(0.9909841807)*x[2]
        arg[0,2]=(-1.06629333132)*x[0]+(-0.38478180064)*x[1]+(-0.63309211256)*x[2]
        arg[0,3]=(0.122436386344)*x[0]+(-0.218752671204)*x[1]+(-0.746695172216)*x[2]
        arg[0,4]=(1.03152124874)*x[0]+(-1.11327271563)*x[1]+(-0.257591112135)*x[2]
        arg[1,0]=(0.969019600299)*x[0]+(-0.320279309689)*x[1]+(0.327767746545)*x[2]
        arg[1,1]=(0.291023069052)*x[0]+(0.78112901064)*x[1]+(0.879257239042)*x[2]
        arg[1,2]=(-0.131262891536)*x[0]+(-0.0109690229153)*x[1]+(-0.171900332406)*x[2]
        arg[1,3]=(-1.32272684486)*x[0]+(-0.256869536085)*x[1]+(0.0308524567462)*x[2]
        arg[1,4]=(1.30707061279)*x[0]+(0.972866282634)*x[1]+(-0.474948537882)*x[2]
        arg[2,0]=(0.169034210421)*x[0]+(-1.08714896226)*x[1]+(-0.636838450708)*x[2]
        arg[2,1]=(-0.470752780773)*x[0]+(-0.695286088059)*x[1]+(-1.19197058712)*x[2]
        arg[2,2]=(0.701403721228)*x[0]+(0.551002482436)*x[1]+(-0.295853851234)*x[2]
        arg[2,3]=(-0.796431361745)*x[0]+(0.0581884766856)*x[1]+(-1.13668563089)*x[2]
        arg[2,4]=(-0.733185798017)*x[0]+(0.178900103735)*x[1]+(-0.845671526781)*x[2]
        arg[3,0]=(-1.47064927107)*x[0]+(1.33393324965)*x[1]+(-0.767393460937)*x[2]
        arg[3,1]=(1.36784513226)*x[0]+(0.100008910431)*x[1]+(-1.09210256713)*x[2]
        arg[3,2]=(-0.873076669977)*x[0]+(0.367213631778)*x[1]+(0.0698296920039)*x[2]
        arg[3,3]=(-1.76609214158)*x[0]+(-1.26186286566)*x[1]+(0.377817035113)*x[2]
        arg[3,4]=(-1.28261709396)*x[0]+(0.240480767533)*x[1]+(0.307813279357)*x[2]
        ref[0,0]=(-0.611607640773)*x_ref[0]+(0.584485971628)*x_ref[1]+(0.0090316337643)*x_ref[2]
        ref[0,1]=(0.905278285857)*x_ref[0]+(0.332523151242)*x_ref[1]+(0.9909841807)*x_ref[2]
        ref[0,2]=(-1.06629333132)*x_ref[0]+(-0.38478180064)*x_ref[1]+(-0.63309211256)*x_ref[2]
        ref[0,3]=(0.122436386344)*x_ref[0]+(-0.218752671204)*x_ref[1]+(-0.746695172216)*x_ref[2]
        ref[0,4]=(1.03152124874)*x_ref[0]+(-1.11327271563)*x_ref[1]+(-0.257591112135)*x_ref[2]
        ref[1,0]=(0.969019600299)*x_ref[0]+(-0.320279309689)*x_ref[1]+(0.327767746545)*x_ref[2]
        ref[1,1]=(0.291023069052)*x_ref[0]+(0.78112901064)*x_ref[1]+(0.879257239042)*x_ref[2]
        ref[1,2]=(-0.131262891536)*x_ref[0]+(-0.0109690229153)*x_ref[1]+(-0.171900332406)*x_ref[2]
        ref[1,3]=(-1.32272684486)*x_ref[0]+(-0.256869536085)*x_ref[1]+(0.0308524567462)*x_ref[2]
        ref[1,4]=(1.30707061279)*x_ref[0]+(0.972866282634)*x_ref[1]+(-0.474948537882)*x_ref[2]
        ref[2,0]=(0.169034210421)*x_ref[0]+(-1.08714896226)*x_ref[1]+(-0.636838450708)*x_ref[2]
        ref[2,1]=(-0.470752780773)*x_ref[0]+(-0.695286088059)*x_ref[1]+(-1.19197058712)*x_ref[2]
        ref[2,2]=(0.701403721228)*x_ref[0]+(0.551002482436)*x_ref[1]+(-0.295853851234)*x_ref[2]
        ref[2,3]=(-0.796431361745)*x_ref[0]+(0.0581884766856)*x_ref[1]+(-1.13668563089)*x_ref[2]
        ref[2,4]=(-0.733185798017)*x_ref[0]+(0.178900103735)*x_ref[1]+(-0.845671526781)*x_ref[2]
        ref[3,0]=(-1.47064927107)*x_ref[0]+(1.33393324965)*x_ref[1]+(-0.767393460937)*x_ref[2]
        ref[3,1]=(1.36784513226)*x_ref[0]+(0.100008910431)*x_ref[1]+(-1.09210256713)*x_ref[2]
        ref[3,2]=(-0.873076669977)*x_ref[0]+(0.367213631778)*x_ref[1]+(0.0698296920039)*x_ref[2]
        ref[3,3]=(-1.76609214158)*x_ref[0]+(-1.26186286566)*x_ref[1]+(0.377817035113)*x_ref[2]
        ref[3,4]=(-1.28261709396)*x_ref[0]+(0.240480767533)*x_ref[1]+(0.307813279357)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromData_ReducedSolution_rank3(self):
      """
      tests interpolate for rank 3 Data onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedSolution onto ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2),w_ref)
      if dim==2:
        arg[0,0,0]=(-0.423505169404)*x[0]+(-1.10390334264)*x[1]
        arg[0,0,1]=(-0.181820089254)*x[0]+(-0.374602268791)*x[1]
        arg[0,1,0]=(-1.36906917075)*x[0]+(0.26554385908)*x[1]
        arg[0,1,1]=(-0.280661572374)*x[0]+(-0.239232524287)*x[1]
        arg[1,0,0]=(-1.25670251934)*x[0]+(-0.429353977735)*x[1]
        arg[1,0,1]=(-0.747928970735)*x[0]+(1.17874317611)*x[1]
        arg[1,1,0]=(-0.548814611802)*x[0]+(-0.0249315764285)*x[1]
        arg[1,1,1]=(-0.481123117522)*x[0]+(1.23402203879)*x[1]
        arg[2,0,0]=(-0.0639512689884)*x[0]+(-0.0278213371376)*x[1]
        arg[2,0,1]=(-1.17292797621)*x[0]+(0.42140600304)*x[1]
        arg[2,1,0]=(-0.358052601083)*x[0]+(-1.66807880791)*x[1]
        arg[2,1,1]=(-0.291433251349)*x[0]+(0.111473162108)*x[1]
        arg[3,0,0]=(0.742643625989)*x[0]+(-1.29214106228)*x[1]
        arg[3,0,1]=(0.319887321826)*x[0]+(-0.40213369238)*x[1]
        arg[3,1,0]=(-0.449412064097)*x[0]+(-1.10654262009)*x[1]
        arg[3,1,1]=(1.19722227632)*x[0]+(1.3455799026)*x[1]
        arg[4,0,0]=(-1.77453188487)*x[0]+(0.634838793872)*x[1]
        arg[4,0,1]=(-0.215461066907)*x[0]+(1.11635233844)*x[1]
        arg[4,1,0]=(0.835816652592)*x[0]+(0.668378554214)*x[1]
        arg[4,1,1]=(-1.22133225406)*x[0]+(-0.148780252398)*x[1]
        arg[5,0,0]=(0.320245409639)*x[0]+(-1.08828513776)*x[1]
        arg[5,0,1]=(-0.1865835417)*x[0]+(-0.353282061874)*x[1]
        arg[5,1,0]=(1.38858040685)*x[0]+(0.166398261098)*x[1]
        arg[5,1,1]=(0.947718875755)*x[0]+(-1.06171934302)*x[1]
        ref[0,0,0]=(-0.423505169404)*x_ref[0]+(-1.10390334264)*x_ref[1]
        ref[0,0,1]=(-0.181820089254)*x_ref[0]+(-0.374602268791)*x_ref[1]
        ref[0,1,0]=(-1.36906917075)*x_ref[0]+(0.26554385908)*x_ref[1]
        ref[0,1,1]=(-0.280661572374)*x_ref[0]+(-0.239232524287)*x_ref[1]
        ref[1,0,0]=(-1.25670251934)*x_ref[0]+(-0.429353977735)*x_ref[1]
        ref[1,0,1]=(-0.747928970735)*x_ref[0]+(1.17874317611)*x_ref[1]
        ref[1,1,0]=(-0.548814611802)*x_ref[0]+(-0.0249315764285)*x_ref[1]
        ref[1,1,1]=(-0.481123117522)*x_ref[0]+(1.23402203879)*x_ref[1]
        ref[2,0,0]=(-0.0639512689884)*x_ref[0]+(-0.0278213371376)*x_ref[1]
        ref[2,0,1]=(-1.17292797621)*x_ref[0]+(0.42140600304)*x_ref[1]
        ref[2,1,0]=(-0.358052601083)*x_ref[0]+(-1.66807880791)*x_ref[1]
        ref[2,1,1]=(-0.291433251349)*x_ref[0]+(0.111473162108)*x_ref[1]
        ref[3,0,0]=(0.742643625989)*x_ref[0]+(-1.29214106228)*x_ref[1]
        ref[3,0,1]=(0.319887321826)*x_ref[0]+(-0.40213369238)*x_ref[1]
        ref[3,1,0]=(-0.449412064097)*x_ref[0]+(-1.10654262009)*x_ref[1]
        ref[3,1,1]=(1.19722227632)*x_ref[0]+(1.3455799026)*x_ref[1]
        ref[4,0,0]=(-1.77453188487)*x_ref[0]+(0.634838793872)*x_ref[1]
        ref[4,0,1]=(-0.215461066907)*x_ref[0]+(1.11635233844)*x_ref[1]
        ref[4,1,0]=(0.835816652592)*x_ref[0]+(0.668378554214)*x_ref[1]
        ref[4,1,1]=(-1.22133225406)*x_ref[0]+(-0.148780252398)*x_ref[1]
        ref[5,0,0]=(0.320245409639)*x_ref[0]+(-1.08828513776)*x_ref[1]
        ref[5,0,1]=(-0.1865835417)*x_ref[0]+(-0.353282061874)*x_ref[1]
        ref[5,1,0]=(1.38858040685)*x_ref[0]+(0.166398261098)*x_ref[1]
        ref[5,1,1]=(0.947718875755)*x_ref[0]+(-1.06171934302)*x_ref[1]
      else:
        arg[0,0,0]=(-0.892096322478)*x[0]+(0.720363183451)*x[1]+(0.600609272191)*x[2]
        arg[0,0,1]=(-0.418024461935)*x[0]+(-0.819275292035)*x[1]+(0.134962557692)*x[2]
        arg[0,1,0]=(-0.449783838361)*x[0]+(0.943677173533)*x[1]+(0.209424190991)*x[2]
        arg[0,1,1]=(-0.424386775004)*x[0]+(-0.213024388094)*x[1]+(-1.05927943384)*x[2]
        arg[1,0,0]=(-0.699504096539)*x[0]+(-0.153570308377)*x[1]+(0.576902392658)*x[2]
        arg[1,0,1]=(-0.426914055661)*x[0]+(-0.330405267059)*x[1]+(1.07656632155)*x[2]
        arg[1,1,0]=(0.775238005352)*x[0]+(1.11012072575)*x[1]+(0.764627098603)*x[2]
        arg[1,1,1]=(0.0130424708408)*x[0]+(-0.671778102107)*x[1]+(0.780684368983)*x[2]
        arg[2,0,0]=(-0.0512224258498)*x[0]+(-1.50515803984)*x[1]+(0.342824143946)*x[2]
        arg[2,0,1]=(0.117541624176)*x[0]+(0.372415524552)*x[1]+(0.255218095016)*x[2]
        arg[2,1,0]=(-0.574508895703)*x[0]+(0.345217292359)*x[1]+(0.209211956821)*x[2]
        arg[2,1,1]=(0.93643587147)*x[0]+(-0.096209009902)*x[1]+(0.205150859527)*x[2]
        arg[3,0,0]=(1.7541929884)*x[0]+(0.169061478447)*x[1]+(0.534746695515)*x[2]
        arg[3,0,1]=(0.442329266782)*x[0]+(0.334971710414)*x[1]+(1.66892557698)*x[2]
        arg[3,1,0]=(0.316436716758)*x[0]+(-1.25442030987)*x[1]+(0.372183105374)*x[2]
        arg[3,1,1]=(-0.254157820265)*x[0]+(0.287381300023)*x[1]+(0.0574117423352)*x[2]
        arg[4,0,0]=(-1.08380257521)*x[0]+(-1.31461890032)*x[1]+(-0.503802775099)*x[2]
        arg[4,0,1]=(-0.558293486798)*x[0]+(0.702317635925)*x[1]+(0.463814829137)*x[2]
        arg[4,1,0]=(-0.677160971974)*x[0]+(-0.14921275312)*x[1]+(-1.6501459435)*x[2]
        arg[4,1,1]=(-0.176672532474)*x[0]+(0.945017605811)*x[1]+(-0.790342939788)*x[2]
        arg[5,0,0]=(-1.45668070675)*x[0]+(-0.404957129204)*x[1]+(1.37383065954)*x[2]
        arg[5,0,1]=(0.53829326756)*x[0]+(0.543333031479)*x[1]+(-0.854057296362)*x[2]
        arg[5,1,0]=(0.805864283921)*x[0]+(-0.00186634953513)*x[1]+(-1.97111362539)*x[2]
        arg[5,1,1]=(-0.387746238967)*x[0]+(-0.739158353882)*x[1]+(-0.134082808118)*x[2]
        ref[0,0,0]=(-0.892096322478)*x_ref[0]+(0.720363183451)*x_ref[1]+(0.600609272191)*x_ref[2]
        ref[0,0,1]=(-0.418024461935)*x_ref[0]+(-0.819275292035)*x_ref[1]+(0.134962557692)*x_ref[2]
        ref[0,1,0]=(-0.449783838361)*x_ref[0]+(0.943677173533)*x_ref[1]+(0.209424190991)*x_ref[2]
        ref[0,1,1]=(-0.424386775004)*x_ref[0]+(-0.213024388094)*x_ref[1]+(-1.05927943384)*x_ref[2]
        ref[1,0,0]=(-0.699504096539)*x_ref[0]+(-0.153570308377)*x_ref[1]+(0.576902392658)*x_ref[2]
        ref[1,0,1]=(-0.426914055661)*x_ref[0]+(-0.330405267059)*x_ref[1]+(1.07656632155)*x_ref[2]
        ref[1,1,0]=(0.775238005352)*x_ref[0]+(1.11012072575)*x_ref[1]+(0.764627098603)*x_ref[2]
        ref[1,1,1]=(0.0130424708408)*x_ref[0]+(-0.671778102107)*x_ref[1]+(0.780684368983)*x_ref[2]
        ref[2,0,0]=(-0.0512224258498)*x_ref[0]+(-1.50515803984)*x_ref[1]+(0.342824143946)*x_ref[2]
        ref[2,0,1]=(0.117541624176)*x_ref[0]+(0.372415524552)*x_ref[1]+(0.255218095016)*x_ref[2]
        ref[2,1,0]=(-0.574508895703)*x_ref[0]+(0.345217292359)*x_ref[1]+(0.209211956821)*x_ref[2]
        ref[2,1,1]=(0.93643587147)*x_ref[0]+(-0.096209009902)*x_ref[1]+(0.205150859527)*x_ref[2]
        ref[3,0,0]=(1.7541929884)*x_ref[0]+(0.169061478447)*x_ref[1]+(0.534746695515)*x_ref[2]
        ref[3,0,1]=(0.442329266782)*x_ref[0]+(0.334971710414)*x_ref[1]+(1.66892557698)*x_ref[2]
        ref[3,1,0]=(0.316436716758)*x_ref[0]+(-1.25442030987)*x_ref[1]+(0.372183105374)*x_ref[2]
        ref[3,1,1]=(-0.254157820265)*x_ref[0]+(0.287381300023)*x_ref[1]+(0.0574117423352)*x_ref[2]
        ref[4,0,0]=(-1.08380257521)*x_ref[0]+(-1.31461890032)*x_ref[1]+(-0.503802775099)*x_ref[2]
        ref[4,0,1]=(-0.558293486798)*x_ref[0]+(0.702317635925)*x_ref[1]+(0.463814829137)*x_ref[2]
        ref[4,1,0]=(-0.677160971974)*x_ref[0]+(-0.14921275312)*x_ref[1]+(-1.6501459435)*x_ref[2]
        ref[4,1,1]=(-0.176672532474)*x_ref[0]+(0.945017605811)*x_ref[1]+(-0.790342939788)*x_ref[2]
        ref[5,0,0]=(-1.45668070675)*x_ref[0]+(-0.404957129204)*x_ref[1]+(1.37383065954)*x_ref[2]
        ref[5,0,1]=(0.53829326756)*x_ref[0]+(0.543333031479)*x_ref[1]+(-0.854057296362)*x_ref[2]
        ref[5,1,0]=(0.805864283921)*x_ref[0]+(-0.00186634953513)*x_ref[1]+(-1.97111362539)*x_ref[2]
        ref[5,1,1]=(-0.387746238967)*x_ref[0]+(-0.739158353882)*x_ref[1]+(-0.134082808118)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromData_ReducedSolution_rank4(self):
      """
      tests interpolate for rank 4 Data onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedSolution onto ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5, 3, 2),w)
      ref=Data(0,(4, 5, 3, 2),w_ref)
      if dim==2:
        arg[0,0,0,0]=(-0.754311658086)*x[0]+(0.235157034342)*x[1]
        arg[0,0,0,1]=(-1.33786315024)*x[0]+(0.243349279742)*x[1]
        arg[0,0,1,0]=(1.48657750088)*x[0]+(1.09853108305)*x[1]
        arg[0,0,1,1]=(1.4501362266)*x[0]+(0.824811567155)*x[1]
        arg[0,0,2,0]=(1.12567566082)*x[0]+(0.663525681174)*x[1]
        arg[0,0,2,1]=(1.48991565019)*x[0]+(-0.680795365011)*x[1]
        arg[0,1,0,0]=(0.268571449528)*x[0]+(0.818595834672)*x[1]
        arg[0,1,0,1]=(-1.55490945555)*x[0]+(0.783595197888)*x[1]
        arg[0,1,1,0]=(0.913201382585)*x[0]+(0.855491379376)*x[1]
        arg[0,1,1,1]=(-0.179192075116)*x[0]+(-0.690415079523)*x[1]
        arg[0,1,2,0]=(-0.0393574502912)*x[0]+(-0.690157650634)*x[1]
        arg[0,1,2,1]=(0.784216863217)*x[0]+(0.498735486159)*x[1]
        arg[0,2,0,0]=(0.137939554165)*x[0]+(-1.33961519113)*x[1]
        arg[0,2,0,1]=(-1.03897311736)*x[0]+(-1.37797405312)*x[1]
        arg[0,2,1,0]=(0.199812003756)*x[0]+(-0.0388499674037)*x[1]
        arg[0,2,1,1]=(1.15530534202)*x[0]+(-1.28280869078)*x[1]
        arg[0,2,2,0]=(0.836447791155)*x[0]+(1.2627066026)*x[1]
        arg[0,2,2,1]=(-0.584608492688)*x[0]+(-1.55305974305)*x[1]
        arg[0,3,0,0]=(-0.661952381799)*x[0]+(-1.27062403778)*x[1]
        arg[0,3,0,1]=(-1.47789532259)*x[0]+(-0.279583524936)*x[1]
        arg[0,3,1,0]=(1.40729771357)*x[0]+(0.0971284144528)*x[1]
        arg[0,3,1,1]=(-0.459167955911)*x[0]+(-1.29033281524)*x[1]
        arg[0,3,2,0]=(-1.63189436824)*x[0]+(0.75729808422)*x[1]
        arg[0,3,2,1]=(0.522479828718)*x[0]+(-0.0231299515383)*x[1]
        arg[0,4,0,0]=(0.241891446615)*x[0]+(0.803327107651)*x[1]
        arg[0,4,0,1]=(0.273613502836)*x[0]+(-0.0115180464171)*x[1]
        arg[0,4,1,0]=(1.96219818696)*x[0]+(-0.349137094743)*x[1]
        arg[0,4,1,1]=(0.365690821719)*x[0]+(-1.6943347181)*x[1]
        arg[0,4,2,0]=(-0.283970779872)*x[0]+(-0.258537179804)*x[1]
        arg[0,4,2,1]=(-0.973333430598)*x[0]+(1.48706276212)*x[1]
        arg[1,0,0,0]=(1.18757466971)*x[0]+(0.838804656357)*x[1]
        arg[1,0,0,1]=(0.840859904702)*x[0]+(1.09955195682)*x[1]
        arg[1,0,1,0]=(1.46190513227)*x[0]+(-0.596959830422)*x[1]
        arg[1,0,1,1]=(0.718689296745)*x[0]+(-0.628074620222)*x[1]
        arg[1,0,2,0]=(0.440761293041)*x[0]+(0.115800854194)*x[1]
        arg[1,0,2,1]=(1.34891146904)*x[0]+(0.813690260652)*x[1]
        arg[1,1,0,0]=(0.183071638098)*x[0]+(-1.37996210755)*x[1]
        arg[1,1,0,1]=(0.258142115988)*x[0]+(-0.757709381529)*x[1]
        arg[1,1,1,0]=(1.72392829309)*x[0]+(0.444995102781)*x[1]
        arg[1,1,1,1]=(-0.380898209988)*x[0]+(-0.0950246860347)*x[1]
        arg[1,1,2,0]=(0.203007226016)*x[0]+(1.09786001832)*x[1]
        arg[1,1,2,1]=(-0.502500865341)*x[0]+(-0.629980527995)*x[1]
        arg[1,2,0,0]=(0.482549518436)*x[0]+(0.565840824146)*x[1]
        arg[1,2,0,1]=(0.355818565068)*x[0]+(-0.78273295574)*x[1]
        arg[1,2,1,0]=(1.79204033071)*x[0]+(-0.698040851585)*x[1]
        arg[1,2,1,1]=(-0.534827741044)*x[0]+(0.00943131627498)*x[1]
        arg[1,2,2,0]=(-1.00663524444)*x[0]+(0.0207854553456)*x[1]
        arg[1,2,2,1]=(-0.338048040159)*x[0]+(-0.380607406217)*x[1]
        arg[1,3,0,0]=(0.586310549044)*x[0]+(0.276326798805)*x[1]
        arg[1,3,0,1]=(-0.260482580702)*x[0]+(-0.349279395321)*x[1]
        arg[1,3,1,0]=(-0.196533829895)*x[0]+(0.614805272738)*x[1]
        arg[1,3,1,1]=(-0.363176483895)*x[0]+(-0.925817017731)*x[1]
        arg[1,3,2,0]=(-1.24182075421)*x[0]+(0.215966446013)*x[1]
        arg[1,3,2,1]=(1.07546652642)*x[0]+(0.383803991974)*x[1]
        arg[1,4,0,0]=(0.199312303525)*x[0]+(-0.314520883859)*x[1]
        arg[1,4,0,1]=(-1.56890826152)*x[0]+(-0.133086662241)*x[1]
        arg[1,4,1,0]=(1.1307241936)*x[0]+(-1.32905744984)*x[1]
        arg[1,4,1,1]=(-0.385991455298)*x[0]+(0.292913024507)*x[1]
        arg[1,4,2,0]=(1.14561678056)*x[0]+(-0.662696162276)*x[1]
        arg[1,4,2,1]=(-1.49533948954)*x[0]+(1.43396138744)*x[1]
        arg[2,0,0,0]=(-0.63160396493)*x[0]+(0.813120998776)*x[1]
        arg[2,0,0,1]=(0.65598998434)*x[0]+(-1.02445537553)*x[1]
        arg[2,0,1,0]=(-1.13473104075)*x[0]+(0.057331183449)*x[1]
        arg[2,0,1,1]=(-0.154892482107)*x[0]+(0.481969957579)*x[1]
        arg[2,0,2,0]=(0.521916671496)*x[0]+(0.546676943059)*x[1]
        arg[2,0,2,1]=(-1.68919379028)*x[0]+(1.08292038981)*x[1]
        arg[2,1,0,0]=(-0.643846595888)*x[0]+(0.528306097755)*x[1]
        arg[2,1,0,1]=(1.3745090497)*x[0]+(-0.0801648509184)*x[1]
        arg[2,1,1,0]=(0.129096817054)*x[0]+(0.90750370855)*x[1]
        arg[2,1,1,1]=(0.539309513305)*x[0]+(1.34949470961)*x[1]
        arg[2,1,2,0]=(1.4189703896)*x[0]+(-1.75011201893)*x[1]
        arg[2,1,2,1]=(-1.09325548386)*x[0]+(0.148279301716)*x[1]
        arg[2,2,0,0]=(-1.25785133743)*x[0]+(0.275071306594)*x[1]
        arg[2,2,0,1]=(-0.550152068761)*x[0]+(-1.20422484275)*x[1]
        arg[2,2,1,0]=(-0.102062863668)*x[0]+(-1.34431887541)*x[1]
        arg[2,2,1,1]=(0.103354724169)*x[0]+(-0.941952180446)*x[1]
        arg[2,2,2,0]=(1.2090339604)*x[0]+(-0.330357976796)*x[1]
        arg[2,2,2,1]=(0.203162449188)*x[0]+(-1.03640702014)*x[1]
        arg[2,3,0,0]=(-0.324576353514)*x[0]+(0.582614539267)*x[1]
        arg[2,3,0,1]=(-0.213823136215)*x[0]+(-0.473199105214)*x[1]
        arg[2,3,1,0]=(0.395144702599)*x[0]+(0.0358194467152)*x[1]
        arg[2,3,1,1]=(1.32860563396)*x[0]+(-0.577308310732)*x[1]
        arg[2,3,2,0]=(0.994205029432)*x[0]+(1.13433517636)*x[1]
        arg[2,3,2,1]=(-0.488499286742)*x[0]+(0.224253820738)*x[1]
        arg[2,4,0,0]=(1.48035663008)*x[0]+(-1.09697986914)*x[1]
        arg[2,4,0,1]=(-1.09991915151)*x[0]+(-0.557951199985)*x[1]
        arg[2,4,1,0]=(-1.23862699804)*x[0]+(-0.230602187713)*x[1]
        arg[2,4,1,1]=(-0.922609370448)*x[0]+(1.76673953413)*x[1]
        arg[2,4,2,0]=(0.731769419709)*x[0]+(-1.1094516608)*x[1]
        arg[2,4,2,1]=(-0.145672504805)*x[0]+(0.168194421355)*x[1]
        arg[3,0,0,0]=(1.02559707602)*x[0]+(0.260431368523)*x[1]
        arg[3,0,0,1]=(0.111899568824)*x[0]+(-0.90725475423)*x[1]
        arg[3,0,1,0]=(-0.226547745683)*x[0]+(-0.232830393372)*x[1]
        arg[3,0,1,1]=(0.347004318471)*x[0]+(-0.196130970001)*x[1]
        arg[3,0,2,0]=(-1.72155871442)*x[0]+(-1.28372407452)*x[1]
        arg[3,0,2,1]=(1.43817914693)*x[0]+(-0.354093253684)*x[1]
        arg[3,1,0,0]=(-0.293939245102)*x[0]+(0.0255562139103)*x[1]
        arg[3,1,0,1]=(0.139306213018)*x[0]+(0.595062226642)*x[1]
        arg[3,1,1,0]=(-0.670339920672)*x[0]+(0.387520583192)*x[1]
        arg[3,1,1,1]=(-0.0323325470954)*x[0]+(-0.681162999279)*x[1]
        arg[3,1,2,0]=(0.235275048739)*x[0]+(0.144725473773)*x[1]
        arg[3,1,2,1]=(0.132211795214)*x[0]+(0.262780216845)*x[1]
        arg[3,2,0,0]=(0.590514326163)*x[0]+(-0.84273677918)*x[1]
        arg[3,2,0,1]=(-0.925181556185)*x[0]+(-0.300244447001)*x[1]
        arg[3,2,1,0]=(-0.0509700795112)*x[0]+(-1.31919389616)*x[1]
        arg[3,2,1,1]=(1.50577291846)*x[0]+(-0.219284607769)*x[1]
        arg[3,2,2,0]=(-0.0460616673346)*x[0]+(-0.690312685466)*x[1]
        arg[3,2,2,1]=(-0.673765477936)*x[0]+(-0.0965860475641)*x[1]
        arg[3,3,0,0]=(-0.6577594174)*x[0]+(0.171886277016)*x[1]
        arg[3,3,0,1]=(-1.08810434718)*x[0]+(0.333153250183)*x[1]
        arg[3,3,1,0]=(0.150968371983)*x[0]+(0.290859317391)*x[1]
        arg[3,3,1,1]=(1.62096265131)*x[0]+(-1.10309602741)*x[1]
        arg[3,3,2,0]=(-0.932063639972)*x[0]+(-0.897131561126)*x[1]
        arg[3,3,2,1]=(-0.819110834345)*x[0]+(1.59159109562)*x[1]
        arg[3,4,0,0]=(0.272662931155)*x[0]+(0.69026285756)*x[1]
        arg[3,4,0,1]=(-0.600556370757)*x[0]+(-0.0853533546302)*x[1]
        arg[3,4,1,0]=(1.18544735281)*x[0]+(0.217751189034)*x[1]
        arg[3,4,1,1]=(-0.594609531358)*x[0]+(0.882158349788)*x[1]
        arg[3,4,2,0]=(1.18876765552)*x[0]+(-0.411481586979)*x[1]
        arg[3,4,2,1]=(-1.093067792)*x[0]+(-0.266744260768)*x[1]
        ref[0,0,0,0]=(-0.754311658086)*x_ref[0]+(0.235157034342)*x_ref[1]
        ref[0,0,0,1]=(-1.33786315024)*x_ref[0]+(0.243349279742)*x_ref[1]
        ref[0,0,1,0]=(1.48657750088)*x_ref[0]+(1.09853108305)*x_ref[1]
        ref[0,0,1,1]=(1.4501362266)*x_ref[0]+(0.824811567155)*x_ref[1]
        ref[0,0,2,0]=(1.12567566082)*x_ref[0]+(0.663525681174)*x_ref[1]
        ref[0,0,2,1]=(1.48991565019)*x_ref[0]+(-0.680795365011)*x_ref[1]
        ref[0,1,0,0]=(0.268571449528)*x_ref[0]+(0.818595834672)*x_ref[1]
        ref[0,1,0,1]=(-1.55490945555)*x_ref[0]+(0.783595197888)*x_ref[1]
        ref[0,1,1,0]=(0.913201382585)*x_ref[0]+(0.855491379376)*x_ref[1]
        ref[0,1,1,1]=(-0.179192075116)*x_ref[0]+(-0.690415079523)*x_ref[1]
        ref[0,1,2,0]=(-0.0393574502912)*x_ref[0]+(-0.690157650634)*x_ref[1]
        ref[0,1,2,1]=(0.784216863217)*x_ref[0]+(0.498735486159)*x_ref[1]
        ref[0,2,0,0]=(0.137939554165)*x_ref[0]+(-1.33961519113)*x_ref[1]
        ref[0,2,0,1]=(-1.03897311736)*x_ref[0]+(-1.37797405312)*x_ref[1]
        ref[0,2,1,0]=(0.199812003756)*x_ref[0]+(-0.0388499674037)*x_ref[1]
        ref[0,2,1,1]=(1.15530534202)*x_ref[0]+(-1.28280869078)*x_ref[1]
        ref[0,2,2,0]=(0.836447791155)*x_ref[0]+(1.2627066026)*x_ref[1]
        ref[0,2,2,1]=(-0.584608492688)*x_ref[0]+(-1.55305974305)*x_ref[1]
        ref[0,3,0,0]=(-0.661952381799)*x_ref[0]+(-1.27062403778)*x_ref[1]
        ref[0,3,0,1]=(-1.47789532259)*x_ref[0]+(-0.279583524936)*x_ref[1]
        ref[0,3,1,0]=(1.40729771357)*x_ref[0]+(0.0971284144528)*x_ref[1]
        ref[0,3,1,1]=(-0.459167955911)*x_ref[0]+(-1.29033281524)*x_ref[1]
        ref[0,3,2,0]=(-1.63189436824)*x_ref[0]+(0.75729808422)*x_ref[1]
        ref[0,3,2,1]=(0.522479828718)*x_ref[0]+(-0.0231299515383)*x_ref[1]
        ref[0,4,0,0]=(0.241891446615)*x_ref[0]+(0.803327107651)*x_ref[1]
        ref[0,4,0,1]=(0.273613502836)*x_ref[0]+(-0.0115180464171)*x_ref[1]
        ref[0,4,1,0]=(1.96219818696)*x_ref[0]+(-0.349137094743)*x_ref[1]
        ref[0,4,1,1]=(0.365690821719)*x_ref[0]+(-1.6943347181)*x_ref[1]
        ref[0,4,2,0]=(-0.283970779872)*x_ref[0]+(-0.258537179804)*x_ref[1]
        ref[0,4,2,1]=(-0.973333430598)*x_ref[0]+(1.48706276212)*x_ref[1]
        ref[1,0,0,0]=(1.18757466971)*x_ref[0]+(0.838804656357)*x_ref[1]
        ref[1,0,0,1]=(0.840859904702)*x_ref[0]+(1.09955195682)*x_ref[1]
        ref[1,0,1,0]=(1.46190513227)*x_ref[0]+(-0.596959830422)*x_ref[1]
        ref[1,0,1,1]=(0.718689296745)*x_ref[0]+(-0.628074620222)*x_ref[1]
        ref[1,0,2,0]=(0.440761293041)*x_ref[0]+(0.115800854194)*x_ref[1]
        ref[1,0,2,1]=(1.34891146904)*x_ref[0]+(0.813690260652)*x_ref[1]
        ref[1,1,0,0]=(0.183071638098)*x_ref[0]+(-1.37996210755)*x_ref[1]
        ref[1,1,0,1]=(0.258142115988)*x_ref[0]+(-0.757709381529)*x_ref[1]
        ref[1,1,1,0]=(1.72392829309)*x_ref[0]+(0.444995102781)*x_ref[1]
        ref[1,1,1,1]=(-0.380898209988)*x_ref[0]+(-0.0950246860347)*x_ref[1]
        ref[1,1,2,0]=(0.203007226016)*x_ref[0]+(1.09786001832)*x_ref[1]
        ref[1,1,2,1]=(-0.502500865341)*x_ref[0]+(-0.629980527995)*x_ref[1]
        ref[1,2,0,0]=(0.482549518436)*x_ref[0]+(0.565840824146)*x_ref[1]
        ref[1,2,0,1]=(0.355818565068)*x_ref[0]+(-0.78273295574)*x_ref[1]
        ref[1,2,1,0]=(1.79204033071)*x_ref[0]+(-0.698040851585)*x_ref[1]
        ref[1,2,1,1]=(-0.534827741044)*x_ref[0]+(0.00943131627498)*x_ref[1]
        ref[1,2,2,0]=(-1.00663524444)*x_ref[0]+(0.0207854553456)*x_ref[1]
        ref[1,2,2,1]=(-0.338048040159)*x_ref[0]+(-0.380607406217)*x_ref[1]
        ref[1,3,0,0]=(0.586310549044)*x_ref[0]+(0.276326798805)*x_ref[1]
        ref[1,3,0,1]=(-0.260482580702)*x_ref[0]+(-0.349279395321)*x_ref[1]
        ref[1,3,1,0]=(-0.196533829895)*x_ref[0]+(0.614805272738)*x_ref[1]
        ref[1,3,1,1]=(-0.363176483895)*x_ref[0]+(-0.925817017731)*x_ref[1]
        ref[1,3,2,0]=(-1.24182075421)*x_ref[0]+(0.215966446013)*x_ref[1]
        ref[1,3,2,1]=(1.07546652642)*x_ref[0]+(0.383803991974)*x_ref[1]
        ref[1,4,0,0]=(0.199312303525)*x_ref[0]+(-0.314520883859)*x_ref[1]
        ref[1,4,0,1]=(-1.56890826152)*x_ref[0]+(-0.133086662241)*x_ref[1]
        ref[1,4,1,0]=(1.1307241936)*x_ref[0]+(-1.32905744984)*x_ref[1]
        ref[1,4,1,1]=(-0.385991455298)*x_ref[0]+(0.292913024507)*x_ref[1]
        ref[1,4,2,0]=(1.14561678056)*x_ref[0]+(-0.662696162276)*x_ref[1]
        ref[1,4,2,1]=(-1.49533948954)*x_ref[0]+(1.43396138744)*x_ref[1]
        ref[2,0,0,0]=(-0.63160396493)*x_ref[0]+(0.813120998776)*x_ref[1]
        ref[2,0,0,1]=(0.65598998434)*x_ref[0]+(-1.02445537553)*x_ref[1]
        ref[2,0,1,0]=(-1.13473104075)*x_ref[0]+(0.057331183449)*x_ref[1]
        ref[2,0,1,1]=(-0.154892482107)*x_ref[0]+(0.481969957579)*x_ref[1]
        ref[2,0,2,0]=(0.521916671496)*x_ref[0]+(0.546676943059)*x_ref[1]
        ref[2,0,2,1]=(-1.68919379028)*x_ref[0]+(1.08292038981)*x_ref[1]
        ref[2,1,0,0]=(-0.643846595888)*x_ref[0]+(0.528306097755)*x_ref[1]
        ref[2,1,0,1]=(1.3745090497)*x_ref[0]+(-0.0801648509184)*x_ref[1]
        ref[2,1,1,0]=(0.129096817054)*x_ref[0]+(0.90750370855)*x_ref[1]
        ref[2,1,1,1]=(0.539309513305)*x_ref[0]+(1.34949470961)*x_ref[1]
        ref[2,1,2,0]=(1.4189703896)*x_ref[0]+(-1.75011201893)*x_ref[1]
        ref[2,1,2,1]=(-1.09325548386)*x_ref[0]+(0.148279301716)*x_ref[1]
        ref[2,2,0,0]=(-1.25785133743)*x_ref[0]+(0.275071306594)*x_ref[1]
        ref[2,2,0,1]=(-0.550152068761)*x_ref[0]+(-1.20422484275)*x_ref[1]
        ref[2,2,1,0]=(-0.102062863668)*x_ref[0]+(-1.34431887541)*x_ref[1]
        ref[2,2,1,1]=(0.103354724169)*x_ref[0]+(-0.941952180446)*x_ref[1]
        ref[2,2,2,0]=(1.2090339604)*x_ref[0]+(-0.330357976796)*x_ref[1]
        ref[2,2,2,1]=(0.203162449188)*x_ref[0]+(-1.03640702014)*x_ref[1]
        ref[2,3,0,0]=(-0.324576353514)*x_ref[0]+(0.582614539267)*x_ref[1]
        ref[2,3,0,1]=(-0.213823136215)*x_ref[0]+(-0.473199105214)*x_ref[1]
        ref[2,3,1,0]=(0.395144702599)*x_ref[0]+(0.0358194467152)*x_ref[1]
        ref[2,3,1,1]=(1.32860563396)*x_ref[0]+(-0.577308310732)*x_ref[1]
        ref[2,3,2,0]=(0.994205029432)*x_ref[0]+(1.13433517636)*x_ref[1]
        ref[2,3,2,1]=(-0.488499286742)*x_ref[0]+(0.224253820738)*x_ref[1]
        ref[2,4,0,0]=(1.48035663008)*x_ref[0]+(-1.09697986914)*x_ref[1]
        ref[2,4,0,1]=(-1.09991915151)*x_ref[0]+(-0.557951199985)*x_ref[1]
        ref[2,4,1,0]=(-1.23862699804)*x_ref[0]+(-0.230602187713)*x_ref[1]
        ref[2,4,1,1]=(-0.922609370448)*x_ref[0]+(1.76673953413)*x_ref[1]
        ref[2,4,2,0]=(0.731769419709)*x_ref[0]+(-1.1094516608)*x_ref[1]
        ref[2,4,2,1]=(-0.145672504805)*x_ref[0]+(0.168194421355)*x_ref[1]
        ref[3,0,0,0]=(1.02559707602)*x_ref[0]+(0.260431368523)*x_ref[1]
        ref[3,0,0,1]=(0.111899568824)*x_ref[0]+(-0.90725475423)*x_ref[1]
        ref[3,0,1,0]=(-0.226547745683)*x_ref[0]+(-0.232830393372)*x_ref[1]
        ref[3,0,1,1]=(0.347004318471)*x_ref[0]+(-0.196130970001)*x_ref[1]
        ref[3,0,2,0]=(-1.72155871442)*x_ref[0]+(-1.28372407452)*x_ref[1]
        ref[3,0,2,1]=(1.43817914693)*x_ref[0]+(-0.354093253684)*x_ref[1]
        ref[3,1,0,0]=(-0.293939245102)*x_ref[0]+(0.0255562139103)*x_ref[1]
        ref[3,1,0,1]=(0.139306213018)*x_ref[0]+(0.595062226642)*x_ref[1]
        ref[3,1,1,0]=(-0.670339920672)*x_ref[0]+(0.387520583192)*x_ref[1]
        ref[3,1,1,1]=(-0.0323325470954)*x_ref[0]+(-0.681162999279)*x_ref[1]
        ref[3,1,2,0]=(0.235275048739)*x_ref[0]+(0.144725473773)*x_ref[1]
        ref[3,1,2,1]=(0.132211795214)*x_ref[0]+(0.262780216845)*x_ref[1]
        ref[3,2,0,0]=(0.590514326163)*x_ref[0]+(-0.84273677918)*x_ref[1]
        ref[3,2,0,1]=(-0.925181556185)*x_ref[0]+(-0.300244447001)*x_ref[1]
        ref[3,2,1,0]=(-0.0509700795112)*x_ref[0]+(-1.31919389616)*x_ref[1]
        ref[3,2,1,1]=(1.50577291846)*x_ref[0]+(-0.219284607769)*x_ref[1]
        ref[3,2,2,0]=(-0.0460616673346)*x_ref[0]+(-0.690312685466)*x_ref[1]
        ref[3,2,2,1]=(-0.673765477936)*x_ref[0]+(-0.0965860475641)*x_ref[1]
        ref[3,3,0,0]=(-0.6577594174)*x_ref[0]+(0.171886277016)*x_ref[1]
        ref[3,3,0,1]=(-1.08810434718)*x_ref[0]+(0.333153250183)*x_ref[1]
        ref[3,3,1,0]=(0.150968371983)*x_ref[0]+(0.290859317391)*x_ref[1]
        ref[3,3,1,1]=(1.62096265131)*x_ref[0]+(-1.10309602741)*x_ref[1]
        ref[3,3,2,0]=(-0.932063639972)*x_ref[0]+(-0.897131561126)*x_ref[1]
        ref[3,3,2,1]=(-0.819110834345)*x_ref[0]+(1.59159109562)*x_ref[1]
        ref[3,4,0,0]=(0.272662931155)*x_ref[0]+(0.69026285756)*x_ref[1]
        ref[3,4,0,1]=(-0.600556370757)*x_ref[0]+(-0.0853533546302)*x_ref[1]
        ref[3,4,1,0]=(1.18544735281)*x_ref[0]+(0.217751189034)*x_ref[1]
        ref[3,4,1,1]=(-0.594609531358)*x_ref[0]+(0.882158349788)*x_ref[1]
        ref[3,4,2,0]=(1.18876765552)*x_ref[0]+(-0.411481586979)*x_ref[1]
        ref[3,4,2,1]=(-1.093067792)*x_ref[0]+(-0.266744260768)*x_ref[1]
      else:
        arg[0,0,0,0]=(-1.29218217256)*x[0]+(-0.382937892213)*x[1]+(0.14363896447)*x[2]
        arg[0,0,0,1]=(1.39856942421)*x[0]+(0.302082267835)*x[1]+(-0.40870736584)*x[2]
        arg[0,0,1,0]=(0.523846108043)*x[0]+(1.19961531204)*x[1]+(0.838881761349)*x[2]
        arg[0,0,1,1]=(0.634542282764)*x[0]+(-0.475514686466)*x[1]+(-1.68745428486)*x[2]
        arg[0,0,2,0]=(1.16379585731)*x[0]+(-0.928037595416)*x[1]+(0.998857178353)*x[2]
        arg[0,0,2,1]=(-0.804515607022)*x[0]+(-0.915469493439)*x[1]+(-0.143888000623)*x[2]
        arg[0,1,0,0]=(0.925843155486)*x[0]+(-0.168243418953)*x[1]+(-0.887839555187)*x[2]
        arg[0,1,0,1]=(-0.913103105329)*x[0]+(1.38011681732)*x[1]+(-0.992478793291)*x[2]
        arg[0,1,1,0]=(0.759685289903)*x[0]+(-0.579481939982)*x[1]+(0.577682929138)*x[2]
        arg[0,1,1,1]=(0.813155548228)*x[0]+(-0.178317843217)*x[1]+(-0.0460113194526)*x[2]
        arg[0,1,2,0]=(-0.195764860141)*x[0]+(1.44256979628)*x[1]+(0.283992441343)*x[2]
        arg[0,1,2,1]=(-0.921229459072)*x[0]+(-1.19295761547)*x[1]+(0.215736801944)*x[2]
        arg[0,2,0,0]=(-0.741726331752)*x[0]+(-0.225585937382)*x[1]+(1.12654556466)*x[2]
        arg[0,2,0,1]=(-0.709656162421)*x[0]+(0.298862010425)*x[1]+(-0.502443284827)*x[2]
        arg[0,2,1,0]=(-1.01166188669)*x[0]+(-1.43763408953)*x[1]+(-0.835160398488)*x[2]
        arg[0,2,1,1]=(1.80968424545)*x[0]+(-0.802666103495)*x[1]+(0.290283606115)*x[2]
        arg[0,2,2,0]=(0.346481981585)*x[0]+(-0.444176962215)*x[1]+(0.689127612227)*x[2]
        arg[0,2,2,1]=(1.06925494095)*x[0]+(0.452854988449)*x[1]+(-1.19974690364)*x[2]
        arg[0,3,0,0]=(-0.534896036328)*x[0]+(-0.312000770615)*x[1]+(-0.440033746693)*x[2]
        arg[0,3,0,1]=(0.580351431838)*x[0]+(-0.36623848451)*x[1]+(-0.507837490037)*x[2]
        arg[0,3,1,0]=(0.383180910522)*x[0]+(-0.0907152443563)*x[1]+(-0.200103743943)*x[2]
        arg[0,3,1,1]=(0.0160194687513)*x[0]+(0.862509612939)*x[1]+(-1.14643594474)*x[2]
        arg[0,3,2,0]=(-0.224390850199)*x[0]+(-0.176927715447)*x[1]+(0.213480525831)*x[2]
        arg[0,3,2,1]=(-0.0508511448148)*x[0]+(-1.28943467389)*x[1]+(0.902285302487)*x[2]
        arg[0,4,0,0]=(-0.633106625232)*x[0]+(-0.430651917485)*x[1]+(0.583149841952)*x[2]
        arg[0,4,0,1]=(0.579035528956)*x[0]+(0.159439682519)*x[1]+(-0.0926277063021)*x[2]
        arg[0,4,1,0]=(0.00484756366973)*x[0]+(0.0324822270755)*x[1]+(1.20052413378)*x[2]
        arg[0,4,1,1]=(-0.337383994688)*x[0]+(0.902076367981)*x[1]+(-0.1956377395)*x[2]
        arg[0,4,2,0]=(0.928434343456)*x[0]+(0.0376569884355)*x[1]+(-0.155274118568)*x[2]
        arg[0,4,2,1]=(-0.541990213238)*x[0]+(-0.0165076697829)*x[1]+(-0.210319224859)*x[2]
        arg[1,0,0,0]=(-1.67762025181)*x[0]+(1.61434755579)*x[1]+(0.357272049999)*x[2]
        arg[1,0,0,1]=(-1.62532544549)*x[0]+(-0.271426231027)*x[1]+(0.338680991778)*x[2]
        arg[1,0,1,0]=(1.73523408033)*x[0]+(1.11987092368)*x[1]+(0.126769364273)*x[2]
        arg[1,0,1,1]=(-1.49220786558)*x[0]+(1.51915566092)*x[1]+(1.24788192157)*x[2]
        arg[1,0,2,0]=(-0.687808766765)*x[0]+(-1.02318754737)*x[1]+(0.590653972416)*x[2]
        arg[1,0,2,1]=(0.756067354802)*x[0]+(-0.66964706965)*x[1]+(-1.18986499158)*x[2]
        arg[1,1,0,0]=(0.371345732177)*x[0]+(0.212739617115)*x[1]+(-1.13439047405)*x[2]
        arg[1,1,0,1]=(0.525761804111)*x[0]+(0.144130399691)*x[1]+(0.715517587677)*x[2]
        arg[1,1,1,0]=(-0.376160293215)*x[0]+(-0.469964935263)*x[1]+(0.413518194707)*x[2]
        arg[1,1,1,1]=(-0.787748398144)*x[0]+(-0.810090402237)*x[1]+(-0.50127109256)*x[2]
        arg[1,1,2,0]=(0.654555083692)*x[0]+(-0.142134700515)*x[1]+(-1.55891930521)*x[2]
        arg[1,1,2,1]=(-0.423058885456)*x[0]+(-0.65930158641)*x[1]+(0.94440057744)*x[2]
        arg[1,2,0,0]=(-0.924832007053)*x[0]+(1.0154890382)*x[1]+(0.11938079748)*x[2]
        arg[1,2,0,1]=(0.134642686931)*x[0]+(-1.30908067998)*x[1]+(0.367206139074)*x[2]
        arg[1,2,1,0]=(1.55524055201)*x[0]+(0.727208660637)*x[1]+(0.809810917377)*x[2]
        arg[1,2,1,1]=(0.94813391462)*x[0]+(-1.49402524308)*x[1]+(0.367824542434)*x[2]
        arg[1,2,2,0]=(-0.608295798731)*x[0]+(-0.142302252814)*x[1]+(-0.986529251308)*x[2]
        arg[1,2,2,1]=(0.0259394619607)*x[0]+(-1.60728202467)*x[1]+(0.312534656619)*x[2]
        arg[1,3,0,0]=(-1.34063373729)*x[0]+(-0.268499839329)*x[1]+(-0.335125886299)*x[2]
        arg[1,3,0,1]=(1.53280698072)*x[0]+(1.24369684711)*x[1]+(0.766369156132)*x[2]
        arg[1,3,1,0]=(0.902067176876)*x[0]+(0.791102994814)*x[1]+(-0.89266430493)*x[2]
        arg[1,3,1,1]=(0.704997629232)*x[0]+(-0.551611360332)*x[1]+(1.39134537491)*x[2]
        arg[1,3,2,0]=(-0.151690675211)*x[0]+(-1.01762975893)*x[1]+(-0.627493214284)*x[2]
        arg[1,3,2,1]=(1.00936645922)*x[0]+(0.0322458997248)*x[1]+(1.45039938194)*x[2]
        arg[1,4,0,0]=(-0.0916390388904)*x[0]+(-0.783111746469)*x[1]+(0.389725342225)*x[2]
        arg[1,4,0,1]=(1.18859328243)*x[0]+(0.871689661558)*x[1]+(-0.775464713448)*x[2]
        arg[1,4,1,0]=(0.913118342488)*x[0]+(1.53285648884)*x[1]+(0.0947365101243)*x[2]
        arg[1,4,1,1]=(-1.00735841224)*x[0]+(-1.12544193607)*x[1]+(0.267347229372)*x[2]
        arg[1,4,2,0]=(0.285943438284)*x[0]+(-1.0889389244)*x[1]+(-1.03054842327)*x[2]
        arg[1,4,2,1]=(-1.5768940139)*x[0]+(0.91237754587)*x[1]+(1.30220394199)*x[2]
        arg[2,0,0,0]=(-0.0634935883277)*x[0]+(0.157304131712)*x[1]+(-1.2995042074)*x[2]
        arg[2,0,0,1]=(-0.802705266871)*x[0]+(0.238233162774)*x[1]+(1.29463414013)*x[2]
        arg[2,0,1,0]=(-0.925468462671)*x[0]+(0.870960795966)*x[1]+(0.156522765713)*x[2]
        arg[2,0,1,1]=(-0.546666833309)*x[0]+(-1.14649607995)*x[1]+(-1.53906505048)*x[2]
        arg[2,0,2,0]=(0.464952910857)*x[0]+(0.408605786027)*x[1]+(-0.886974881114)*x[2]
        arg[2,0,2,1]=(0.491723247948)*x[0]+(0.194616420077)*x[1]+(0.409049782749)*x[2]
        arg[2,1,0,0]=(-1.19836883114)*x[0]+(1.0622722337)*x[1]+(-0.109193867727)*x[2]
        arg[2,1,0,1]=(1.01973600383)*x[0]+(-0.154482220539)*x[1]+(0.378696371956)*x[2]
        arg[2,1,1,0]=(-0.0812015296246)*x[0]+(-1.10252794568)*x[1]+(-0.00585426581857)*x[2]
        arg[2,1,1,1]=(0.536042665272)*x[0]+(0.685507117255)*x[1]+(-0.179652683139)*x[2]
        arg[2,1,2,0]=(-0.36750467811)*x[0]+(-1.23674550869)*x[1]+(-0.762905702801)*x[2]
        arg[2,1,2,1]=(0.329665965651)*x[0]+(0.101195810737)*x[1]+(-0.610015052375)*x[2]
        arg[2,2,0,0]=(-0.81644635185)*x[0]+(-0.604578476306)*x[1]+(0.0440925706991)*x[2]
        arg[2,2,0,1]=(-0.958241437971)*x[0]+(0.38563498206)*x[1]+(0.913406762318)*x[2]
        arg[2,2,1,0]=(1.22027492224)*x[0]+(-1.00726209939)*x[1]+(-1.05083068303)*x[2]
        arg[2,2,1,1]=(-1.47561192645)*x[0]+(-0.903549355608)*x[1]+(0.236771324671)*x[2]
        arg[2,2,2,0]=(0.284995864449)*x[0]+(-0.175760829921)*x[1]+(-0.720607220001)*x[2]
        arg[2,2,2,1]=(-0.535894515963)*x[0]+(-0.821302957162)*x[1]+(-1.75141540241)*x[2]
        arg[2,3,0,0]=(-0.333082106674)*x[0]+(0.579471826412)*x[1]+(0.561184138108)*x[2]
        arg[2,3,0,1]=(1.34803519557)*x[0]+(-0.441299088849)*x[1]+(-1.04945951616)*x[2]
        arg[2,3,1,0]=(0.299011096602)*x[0]+(1.32726458235)*x[1]+(-0.196298687359)*x[2]
        arg[2,3,1,1]=(1.48698834204)*x[0]+(0.527221496971)*x[1]+(0.0819813519422)*x[2]
        arg[2,3,2,0]=(0.681680955153)*x[0]+(0.503653972357)*x[1]+(0.434669347891)*x[2]
        arg[2,3,2,1]=(-0.152586789311)*x[0]+(-0.765242635579)*x[1]+(0.712663994387)*x[2]
        arg[2,4,0,0]=(0.630152300463)*x[0]+(1.12366733588)*x[1]+(-0.662966700638)*x[2]
        arg[2,4,0,1]=(1.02796709755)*x[0]+(-0.24093881584)*x[1]+(1.79445417019)*x[2]
        arg[2,4,1,0]=(-0.954575760758)*x[0]+(0.16331782608)*x[1]+(1.38309921277)*x[2]
        arg[2,4,1,1]=(0.536092514281)*x[0]+(-0.276364952673)*x[1]+(0.779619701528)*x[2]
        arg[2,4,2,0]=(0.267270444141)*x[0]+(-0.810746424331)*x[1]+(-0.733877853264)*x[2]
        arg[2,4,2,1]=(-0.0641688847923)*x[0]+(-0.386279280667)*x[1]+(0.0637571595255)*x[2]
        arg[3,0,0,0]=(0.469903180687)*x[0]+(-0.793812397342)*x[1]+(1.60841921779)*x[2]
        arg[3,0,0,1]=(0.720057600102)*x[0]+(0.502582253261)*x[1]+(0.460147791003)*x[2]
        arg[3,0,1,0]=(0.0327862564137)*x[0]+(-1.142962965)*x[1]+(0.114799131894)*x[2]
        arg[3,0,1,1]=(-0.997412656978)*x[0]+(-0.678175468576)*x[1]+(-0.152663147682)*x[2]
        arg[3,0,2,0]=(0.810424599078)*x[0]+(0.75522727786)*x[1]+(-0.205880941501)*x[2]
        arg[3,0,2,1]=(0.353601949476)*x[0]+(0.0963200813379)*x[1]+(0.0578829287146)*x[2]
        arg[3,1,0,0]=(0.346590730062)*x[0]+(1.07953790092)*x[1]+(0.647586851217)*x[2]
        arg[3,1,0,1]=(0.889879195334)*x[0]+(-0.352788186249)*x[1]+(0.434900135803)*x[2]
        arg[3,1,1,0]=(-0.721742734959)*x[0]+(0.166066672134)*x[1]+(-0.489266924809)*x[2]
        arg[3,1,1,1]=(-0.810487755181)*x[0]+(0.0354916979253)*x[1]+(0.0735520654395)*x[2]
        arg[3,1,2,0]=(1.02414338939)*x[0]+(-0.851076284556)*x[1]+(-0.232174494405)*x[2]
        arg[3,1,2,1]=(-0.327246566026)*x[0]+(-0.0499396023605)*x[1]+(0.611585402379)*x[2]
        arg[3,2,0,0]=(-0.365757628557)*x[0]+(0.868697342115)*x[1]+(-1.102545972)*x[2]
        arg[3,2,0,1]=(0.628592129227)*x[0]+(1.25545515311)*x[1]+(-1.0577476264)*x[2]
        arg[3,2,1,0]=(0.160995978044)*x[0]+(-0.947389499556)*x[1]+(-0.870476160656)*x[2]
        arg[3,2,1,1]=(0.910787139938)*x[0]+(-0.0681493345139)*x[1]+(-0.561910153781)*x[2]
        arg[3,2,2,0]=(0.0761644634428)*x[0]+(-0.700940967343)*x[1]+(1.19407210904)*x[2]
        arg[3,2,2,1]=(1.75717437065)*x[0]+(-0.144457317494)*x[1]+(-0.523233453079)*x[2]
        arg[3,3,0,0]=(0.448782872606)*x[0]+(0.650464256547)*x[1]+(1.19476547456)*x[2]
        arg[3,3,0,1]=(0.118586027201)*x[0]+(0.449971816918)*x[1]+(-0.510812733389)*x[2]
        arg[3,3,1,0]=(-0.330032265081)*x[0]+(-0.13657534891)*x[1]+(0.456365289556)*x[2]
        arg[3,3,1,1]=(0.697584212635)*x[0]+(0.897604241724)*x[1]+(1.31517116663)*x[2]
        arg[3,3,2,0]=(-0.940538707365)*x[0]+(-0.334862221665)*x[1]+(0.0969742096754)*x[2]
        arg[3,3,2,1]=(-0.985788813516)*x[0]+(0.348389725548)*x[1]+(-0.260816590361)*x[2]
        arg[3,4,0,0]=(0.754692421417)*x[0]+(0.775985781769)*x[1]+(-0.0866159222457)*x[2]
        arg[3,4,0,1]=(-1.17670502429)*x[0]+(1.12268279393)*x[1]+(-1.1279822095)*x[2]
        arg[3,4,1,0]=(-1.48674370666)*x[0]+(1.26685445713)*x[1]+(-0.0522828809306)*x[2]
        arg[3,4,1,1]=(0.401760689843)*x[0]+(-1.35707797971)*x[1]+(-0.0580654273828)*x[2]
        arg[3,4,2,0]=(-1.40697830852)*x[0]+(0.933923369958)*x[1]+(0.230152169159)*x[2]
        arg[3,4,2,1]=(-1.21242130555)*x[0]+(0.858316339663)*x[1]+(-0.839747078526)*x[2]
        ref[0,0,0,0]=(-1.29218217256)*x_ref[0]+(-0.382937892213)*x_ref[1]+(0.14363896447)*x_ref[2]
        ref[0,0,0,1]=(1.39856942421)*x_ref[0]+(0.302082267835)*x_ref[1]+(-0.40870736584)*x_ref[2]
        ref[0,0,1,0]=(0.523846108043)*x_ref[0]+(1.19961531204)*x_ref[1]+(0.838881761349)*x_ref[2]
        ref[0,0,1,1]=(0.634542282764)*x_ref[0]+(-0.475514686466)*x_ref[1]+(-1.68745428486)*x_ref[2]
        ref[0,0,2,0]=(1.16379585731)*x_ref[0]+(-0.928037595416)*x_ref[1]+(0.998857178353)*x_ref[2]
        ref[0,0,2,1]=(-0.804515607022)*x_ref[0]+(-0.915469493439)*x_ref[1]+(-0.143888000623)*x_ref[2]
        ref[0,1,0,0]=(0.925843155486)*x_ref[0]+(-0.168243418953)*x_ref[1]+(-0.887839555187)*x_ref[2]
        ref[0,1,0,1]=(-0.913103105329)*x_ref[0]+(1.38011681732)*x_ref[1]+(-0.992478793291)*x_ref[2]
        ref[0,1,1,0]=(0.759685289903)*x_ref[0]+(-0.579481939982)*x_ref[1]+(0.577682929138)*x_ref[2]
        ref[0,1,1,1]=(0.813155548228)*x_ref[0]+(-0.178317843217)*x_ref[1]+(-0.0460113194526)*x_ref[2]
        ref[0,1,2,0]=(-0.195764860141)*x_ref[0]+(1.44256979628)*x_ref[1]+(0.283992441343)*x_ref[2]
        ref[0,1,2,1]=(-0.921229459072)*x_ref[0]+(-1.19295761547)*x_ref[1]+(0.215736801944)*x_ref[2]
        ref[0,2,0,0]=(-0.741726331752)*x_ref[0]+(-0.225585937382)*x_ref[1]+(1.12654556466)*x_ref[2]
        ref[0,2,0,1]=(-0.709656162421)*x_ref[0]+(0.298862010425)*x_ref[1]+(-0.502443284827)*x_ref[2]
        ref[0,2,1,0]=(-1.01166188669)*x_ref[0]+(-1.43763408953)*x_ref[1]+(-0.835160398488)*x_ref[2]
        ref[0,2,1,1]=(1.80968424545)*x_ref[0]+(-0.802666103495)*x_ref[1]+(0.290283606115)*x_ref[2]
        ref[0,2,2,0]=(0.346481981585)*x_ref[0]+(-0.444176962215)*x_ref[1]+(0.689127612227)*x_ref[2]
        ref[0,2,2,1]=(1.06925494095)*x_ref[0]+(0.452854988449)*x_ref[1]+(-1.19974690364)*x_ref[2]
        ref[0,3,0,0]=(-0.534896036328)*x_ref[0]+(-0.312000770615)*x_ref[1]+(-0.440033746693)*x_ref[2]
        ref[0,3,0,1]=(0.580351431838)*x_ref[0]+(-0.36623848451)*x_ref[1]+(-0.507837490037)*x_ref[2]
        ref[0,3,1,0]=(0.383180910522)*x_ref[0]+(-0.0907152443563)*x_ref[1]+(-0.200103743943)*x_ref[2]
        ref[0,3,1,1]=(0.0160194687513)*x_ref[0]+(0.862509612939)*x_ref[1]+(-1.14643594474)*x_ref[2]
        ref[0,3,2,0]=(-0.224390850199)*x_ref[0]+(-0.176927715447)*x_ref[1]+(0.213480525831)*x_ref[2]
        ref[0,3,2,1]=(-0.0508511448148)*x_ref[0]+(-1.28943467389)*x_ref[1]+(0.902285302487)*x_ref[2]
        ref[0,4,0,0]=(-0.633106625232)*x_ref[0]+(-0.430651917485)*x_ref[1]+(0.583149841952)*x_ref[2]
        ref[0,4,0,1]=(0.579035528956)*x_ref[0]+(0.159439682519)*x_ref[1]+(-0.0926277063021)*x_ref[2]
        ref[0,4,1,0]=(0.00484756366973)*x_ref[0]+(0.0324822270755)*x_ref[1]+(1.20052413378)*x_ref[2]
        ref[0,4,1,1]=(-0.337383994688)*x_ref[0]+(0.902076367981)*x_ref[1]+(-0.1956377395)*x_ref[2]
        ref[0,4,2,0]=(0.928434343456)*x_ref[0]+(0.0376569884355)*x_ref[1]+(-0.155274118568)*x_ref[2]
        ref[0,4,2,1]=(-0.541990213238)*x_ref[0]+(-0.0165076697829)*x_ref[1]+(-0.210319224859)*x_ref[2]
        ref[1,0,0,0]=(-1.67762025181)*x_ref[0]+(1.61434755579)*x_ref[1]+(0.357272049999)*x_ref[2]
        ref[1,0,0,1]=(-1.62532544549)*x_ref[0]+(-0.271426231027)*x_ref[1]+(0.338680991778)*x_ref[2]
        ref[1,0,1,0]=(1.73523408033)*x_ref[0]+(1.11987092368)*x_ref[1]+(0.126769364273)*x_ref[2]
        ref[1,0,1,1]=(-1.49220786558)*x_ref[0]+(1.51915566092)*x_ref[1]+(1.24788192157)*x_ref[2]
        ref[1,0,2,0]=(-0.687808766765)*x_ref[0]+(-1.02318754737)*x_ref[1]+(0.590653972416)*x_ref[2]
        ref[1,0,2,1]=(0.756067354802)*x_ref[0]+(-0.66964706965)*x_ref[1]+(-1.18986499158)*x_ref[2]
        ref[1,1,0,0]=(0.371345732177)*x_ref[0]+(0.212739617115)*x_ref[1]+(-1.13439047405)*x_ref[2]
        ref[1,1,0,1]=(0.525761804111)*x_ref[0]+(0.144130399691)*x_ref[1]+(0.715517587677)*x_ref[2]
        ref[1,1,1,0]=(-0.376160293215)*x_ref[0]+(-0.469964935263)*x_ref[1]+(0.413518194707)*x_ref[2]
        ref[1,1,1,1]=(-0.787748398144)*x_ref[0]+(-0.810090402237)*x_ref[1]+(-0.50127109256)*x_ref[2]
        ref[1,1,2,0]=(0.654555083692)*x_ref[0]+(-0.142134700515)*x_ref[1]+(-1.55891930521)*x_ref[2]
        ref[1,1,2,1]=(-0.423058885456)*x_ref[0]+(-0.65930158641)*x_ref[1]+(0.94440057744)*x_ref[2]
        ref[1,2,0,0]=(-0.924832007053)*x_ref[0]+(1.0154890382)*x_ref[1]+(0.11938079748)*x_ref[2]
        ref[1,2,0,1]=(0.134642686931)*x_ref[0]+(-1.30908067998)*x_ref[1]+(0.367206139074)*x_ref[2]
        ref[1,2,1,0]=(1.55524055201)*x_ref[0]+(0.727208660637)*x_ref[1]+(0.809810917377)*x_ref[2]
        ref[1,2,1,1]=(0.94813391462)*x_ref[0]+(-1.49402524308)*x_ref[1]+(0.367824542434)*x_ref[2]
        ref[1,2,2,0]=(-0.608295798731)*x_ref[0]+(-0.142302252814)*x_ref[1]+(-0.986529251308)*x_ref[2]
        ref[1,2,2,1]=(0.0259394619607)*x_ref[0]+(-1.60728202467)*x_ref[1]+(0.312534656619)*x_ref[2]
        ref[1,3,0,0]=(-1.34063373729)*x_ref[0]+(-0.268499839329)*x_ref[1]+(-0.335125886299)*x_ref[2]
        ref[1,3,0,1]=(1.53280698072)*x_ref[0]+(1.24369684711)*x_ref[1]+(0.766369156132)*x_ref[2]
        ref[1,3,1,0]=(0.902067176876)*x_ref[0]+(0.791102994814)*x_ref[1]+(-0.89266430493)*x_ref[2]
        ref[1,3,1,1]=(0.704997629232)*x_ref[0]+(-0.551611360332)*x_ref[1]+(1.39134537491)*x_ref[2]
        ref[1,3,2,0]=(-0.151690675211)*x_ref[0]+(-1.01762975893)*x_ref[1]+(-0.627493214284)*x_ref[2]
        ref[1,3,2,1]=(1.00936645922)*x_ref[0]+(0.0322458997248)*x_ref[1]+(1.45039938194)*x_ref[2]
        ref[1,4,0,0]=(-0.0916390388904)*x_ref[0]+(-0.783111746469)*x_ref[1]+(0.389725342225)*x_ref[2]
        ref[1,4,0,1]=(1.18859328243)*x_ref[0]+(0.871689661558)*x_ref[1]+(-0.775464713448)*x_ref[2]
        ref[1,4,1,0]=(0.913118342488)*x_ref[0]+(1.53285648884)*x_ref[1]+(0.0947365101243)*x_ref[2]
        ref[1,4,1,1]=(-1.00735841224)*x_ref[0]+(-1.12544193607)*x_ref[1]+(0.267347229372)*x_ref[2]
        ref[1,4,2,0]=(0.285943438284)*x_ref[0]+(-1.0889389244)*x_ref[1]+(-1.03054842327)*x_ref[2]
        ref[1,4,2,1]=(-1.5768940139)*x_ref[0]+(0.91237754587)*x_ref[1]+(1.30220394199)*x_ref[2]
        ref[2,0,0,0]=(-0.0634935883277)*x_ref[0]+(0.157304131712)*x_ref[1]+(-1.2995042074)*x_ref[2]
        ref[2,0,0,1]=(-0.802705266871)*x_ref[0]+(0.238233162774)*x_ref[1]+(1.29463414013)*x_ref[2]
        ref[2,0,1,0]=(-0.925468462671)*x_ref[0]+(0.870960795966)*x_ref[1]+(0.156522765713)*x_ref[2]
        ref[2,0,1,1]=(-0.546666833309)*x_ref[0]+(-1.14649607995)*x_ref[1]+(-1.53906505048)*x_ref[2]
        ref[2,0,2,0]=(0.464952910857)*x_ref[0]+(0.408605786027)*x_ref[1]+(-0.886974881114)*x_ref[2]
        ref[2,0,2,1]=(0.491723247948)*x_ref[0]+(0.194616420077)*x_ref[1]+(0.409049782749)*x_ref[2]
        ref[2,1,0,0]=(-1.19836883114)*x_ref[0]+(1.0622722337)*x_ref[1]+(-0.109193867727)*x_ref[2]
        ref[2,1,0,1]=(1.01973600383)*x_ref[0]+(-0.154482220539)*x_ref[1]+(0.378696371956)*x_ref[2]
        ref[2,1,1,0]=(-0.0812015296246)*x_ref[0]+(-1.10252794568)*x_ref[1]+(-0.00585426581857)*x_ref[2]
        ref[2,1,1,1]=(0.536042665272)*x_ref[0]+(0.685507117255)*x_ref[1]+(-0.179652683139)*x_ref[2]
        ref[2,1,2,0]=(-0.36750467811)*x_ref[0]+(-1.23674550869)*x_ref[1]+(-0.762905702801)*x_ref[2]
        ref[2,1,2,1]=(0.329665965651)*x_ref[0]+(0.101195810737)*x_ref[1]+(-0.610015052375)*x_ref[2]
        ref[2,2,0,0]=(-0.81644635185)*x_ref[0]+(-0.604578476306)*x_ref[1]+(0.0440925706991)*x_ref[2]
        ref[2,2,0,1]=(-0.958241437971)*x_ref[0]+(0.38563498206)*x_ref[1]+(0.913406762318)*x_ref[2]
        ref[2,2,1,0]=(1.22027492224)*x_ref[0]+(-1.00726209939)*x_ref[1]+(-1.05083068303)*x_ref[2]
        ref[2,2,1,1]=(-1.47561192645)*x_ref[0]+(-0.903549355608)*x_ref[1]+(0.236771324671)*x_ref[2]
        ref[2,2,2,0]=(0.284995864449)*x_ref[0]+(-0.175760829921)*x_ref[1]+(-0.720607220001)*x_ref[2]
        ref[2,2,2,1]=(-0.535894515963)*x_ref[0]+(-0.821302957162)*x_ref[1]+(-1.75141540241)*x_ref[2]
        ref[2,3,0,0]=(-0.333082106674)*x_ref[0]+(0.579471826412)*x_ref[1]+(0.561184138108)*x_ref[2]
        ref[2,3,0,1]=(1.34803519557)*x_ref[0]+(-0.441299088849)*x_ref[1]+(-1.04945951616)*x_ref[2]
        ref[2,3,1,0]=(0.299011096602)*x_ref[0]+(1.32726458235)*x_ref[1]+(-0.196298687359)*x_ref[2]
        ref[2,3,1,1]=(1.48698834204)*x_ref[0]+(0.527221496971)*x_ref[1]+(0.0819813519422)*x_ref[2]
        ref[2,3,2,0]=(0.681680955153)*x_ref[0]+(0.503653972357)*x_ref[1]+(0.434669347891)*x_ref[2]
        ref[2,3,2,1]=(-0.152586789311)*x_ref[0]+(-0.765242635579)*x_ref[1]+(0.712663994387)*x_ref[2]
        ref[2,4,0,0]=(0.630152300463)*x_ref[0]+(1.12366733588)*x_ref[1]+(-0.662966700638)*x_ref[2]
        ref[2,4,0,1]=(1.02796709755)*x_ref[0]+(-0.24093881584)*x_ref[1]+(1.79445417019)*x_ref[2]
        ref[2,4,1,0]=(-0.954575760758)*x_ref[0]+(0.16331782608)*x_ref[1]+(1.38309921277)*x_ref[2]
        ref[2,4,1,1]=(0.536092514281)*x_ref[0]+(-0.276364952673)*x_ref[1]+(0.779619701528)*x_ref[2]
        ref[2,4,2,0]=(0.267270444141)*x_ref[0]+(-0.810746424331)*x_ref[1]+(-0.733877853264)*x_ref[2]
        ref[2,4,2,1]=(-0.0641688847923)*x_ref[0]+(-0.386279280667)*x_ref[1]+(0.0637571595255)*x_ref[2]
        ref[3,0,0,0]=(0.469903180687)*x_ref[0]+(-0.793812397342)*x_ref[1]+(1.60841921779)*x_ref[2]
        ref[3,0,0,1]=(0.720057600102)*x_ref[0]+(0.502582253261)*x_ref[1]+(0.460147791003)*x_ref[2]
        ref[3,0,1,0]=(0.0327862564137)*x_ref[0]+(-1.142962965)*x_ref[1]+(0.114799131894)*x_ref[2]
        ref[3,0,1,1]=(-0.997412656978)*x_ref[0]+(-0.678175468576)*x_ref[1]+(-0.152663147682)*x_ref[2]
        ref[3,0,2,0]=(0.810424599078)*x_ref[0]+(0.75522727786)*x_ref[1]+(-0.205880941501)*x_ref[2]
        ref[3,0,2,1]=(0.353601949476)*x_ref[0]+(0.0963200813379)*x_ref[1]+(0.0578829287146)*x_ref[2]
        ref[3,1,0,0]=(0.346590730062)*x_ref[0]+(1.07953790092)*x_ref[1]+(0.647586851217)*x_ref[2]
        ref[3,1,0,1]=(0.889879195334)*x_ref[0]+(-0.352788186249)*x_ref[1]+(0.434900135803)*x_ref[2]
        ref[3,1,1,0]=(-0.721742734959)*x_ref[0]+(0.166066672134)*x_ref[1]+(-0.489266924809)*x_ref[2]
        ref[3,1,1,1]=(-0.810487755181)*x_ref[0]+(0.0354916979253)*x_ref[1]+(0.0735520654395)*x_ref[2]
        ref[3,1,2,0]=(1.02414338939)*x_ref[0]+(-0.851076284556)*x_ref[1]+(-0.232174494405)*x_ref[2]
        ref[3,1,2,1]=(-0.327246566026)*x_ref[0]+(-0.0499396023605)*x_ref[1]+(0.611585402379)*x_ref[2]
        ref[3,2,0,0]=(-0.365757628557)*x_ref[0]+(0.868697342115)*x_ref[1]+(-1.102545972)*x_ref[2]
        ref[3,2,0,1]=(0.628592129227)*x_ref[0]+(1.25545515311)*x_ref[1]+(-1.0577476264)*x_ref[2]
        ref[3,2,1,0]=(0.160995978044)*x_ref[0]+(-0.947389499556)*x_ref[1]+(-0.870476160656)*x_ref[2]
        ref[3,2,1,1]=(0.910787139938)*x_ref[0]+(-0.0681493345139)*x_ref[1]+(-0.561910153781)*x_ref[2]
        ref[3,2,2,0]=(0.0761644634428)*x_ref[0]+(-0.700940967343)*x_ref[1]+(1.19407210904)*x_ref[2]
        ref[3,2,2,1]=(1.75717437065)*x_ref[0]+(-0.144457317494)*x_ref[1]+(-0.523233453079)*x_ref[2]
        ref[3,3,0,0]=(0.448782872606)*x_ref[0]+(0.650464256547)*x_ref[1]+(1.19476547456)*x_ref[2]
        ref[3,3,0,1]=(0.118586027201)*x_ref[0]+(0.449971816918)*x_ref[1]+(-0.510812733389)*x_ref[2]
        ref[3,3,1,0]=(-0.330032265081)*x_ref[0]+(-0.13657534891)*x_ref[1]+(0.456365289556)*x_ref[2]
        ref[3,3,1,1]=(0.697584212635)*x_ref[0]+(0.897604241724)*x_ref[1]+(1.31517116663)*x_ref[2]
        ref[3,3,2,0]=(-0.940538707365)*x_ref[0]+(-0.334862221665)*x_ref[1]+(0.0969742096754)*x_ref[2]
        ref[3,3,2,1]=(-0.985788813516)*x_ref[0]+(0.348389725548)*x_ref[1]+(-0.260816590361)*x_ref[2]
        ref[3,4,0,0]=(0.754692421417)*x_ref[0]+(0.775985781769)*x_ref[1]+(-0.0866159222457)*x_ref[2]
        ref[3,4,0,1]=(-1.17670502429)*x_ref[0]+(1.12268279393)*x_ref[1]+(-1.1279822095)*x_ref[2]
        ref[3,4,1,0]=(-1.48674370666)*x_ref[0]+(1.26685445713)*x_ref[1]+(-0.0522828809306)*x_ref[2]
        ref[3,4,1,1]=(0.401760689843)*x_ref[0]+(-1.35707797971)*x_ref[1]+(-0.0580654273828)*x_ref[2]
        ref[3,4,2,0]=(-1.40697830852)*x_ref[0]+(0.933923369958)*x_ref[1]+(0.230152169159)*x_ref[2]
        ref[3,4,2,1]=(-1.21242130555)*x_ref[0]+(0.858316339663)*x_ref[1]+(-0.839747078526)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromData_ReducedContinuousFunction_rank0(self):
      """
      tests interpolate for rank 0 Data onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedContinuousFunction onto ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(),w)
      ref=Data(0,(),w_ref)
      if dim==2:
        arg=(-0.100067148208)*x[0]+(-0.8375743725)*x[1]
        ref=(-0.100067148208)*x_ref[0]+(-0.8375743725)*x_ref[1]
      else:
        arg=(-0.247343416726)*x[0]+(0.604193484164)*x[1]+(-0.219899438287)*x[2]
        ref=(-0.247343416726)*x_ref[0]+(0.604193484164)*x_ref[1]+(-0.219899438287)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromData_ReducedContinuousFunction_rank1(self):
      """
      tests interpolate for rank 1 Data onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedContinuousFunction onto ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(2,),w)
      ref=Data(0,(2,),w_ref)
      if dim==2:
        arg[0]=(-1.66196653104)*x[0]+(1.29891335227)*x[1]
        arg[1]=(-0.347844771248)*x[0]+(-0.404726300584)*x[1]
        ref[0]=(-1.66196653104)*x[0]+(1.29891335227)*x[1]
        ref[1]=(-0.347844771248)*x[0]+(-0.404726300584)*x[1]
      else:
        arg[0]=(0.334094742067)*x[0]+(-0.619063646554)*x[1]+(0.538572422001)*x[2]
        arg[1]=(0.673290433004)*x[0]+(0.169501972843)*x[1]+(-0.833663493893)*x[2]
        ref[0]=(0.334094742067)*x[0]+(-0.619063646554)*x[1]+(0.538572422001)*x[2]
        ref[1]=(0.673290433004)*x[0]+(0.169501972843)*x[1]+(-0.833663493893)*x[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromData_ReducedContinuousFunction_rank2(self):
      """
      tests interpolate for rank 2 Data onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedContinuousFunction onto ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5),w)
      ref=Data(0,(4, 5),w_ref)
      if dim==2:
        arg[0,0]=(0.00153698933322)*x[0]+(0.392281645764)*x[1]
        arg[0,1]=(-1.2794655544)*x[0]+(-0.373519639818)*x[1]
        arg[0,2]=(0.30127033321)*x[0]+(-0.0996865853738)*x[1]
        arg[0,3]=(1.23405799349)*x[0]+(0.498558200339)*x[1]
        arg[0,4]=(0.85519350742)*x[0]+(-0.19188046135)*x[1]
        arg[1,0]=(0.417367834962)*x[0]+(0.992318201639)*x[1]
        arg[1,1]=(-0.646432128833)*x[0]+(-0.872750321582)*x[1]
        arg[1,2]=(-0.806117185061)*x[0]+(-0.89396408325)*x[1]
        arg[1,3]=(0.768017237627)*x[0]+(-1.2904111912)*x[1]
        arg[1,4]=(-0.0159685523188)*x[0]+(0.401410193169)*x[1]
        arg[2,0]=(-0.844501716181)*x[0]+(-0.123115343819)*x[1]
        arg[2,1]=(1.08441909409)*x[0]+(0.152432282539)*x[1]
        arg[2,2]=(-0.60551037413)*x[0]+(1.15692828826)*x[1]
        arg[2,3]=(-1.21509168541)*x[0]+(1.03683236377)*x[1]
        arg[2,4]=(1.16850730134)*x[0]+(0.162202615909)*x[1]
        arg[3,0]=(0.727772166897)*x[0]+(-0.296815840935)*x[1]
        arg[3,1]=(-0.0880577151489)*x[0]+(1.66427691426)*x[1]
        arg[3,2]=(0.415144534458)*x[0]+(-0.555833723295)*x[1]
        arg[3,3]=(1.09444635829)*x[0]+(-0.335209417536)*x[1]
        arg[3,4]=(-1.05047769585)*x[0]+(0.684427039986)*x[1]
        ref[0,0]=(0.00153698933322)*x_ref[0]+(0.392281645764)*x_ref[1]
        ref[0,1]=(-1.2794655544)*x_ref[0]+(-0.373519639818)*x_ref[1]
        ref[0,2]=(0.30127033321)*x_ref[0]+(-0.0996865853738)*x_ref[1]
        ref[0,3]=(1.23405799349)*x_ref[0]+(0.498558200339)*x_ref[1]
        ref[0,4]=(0.85519350742)*x_ref[0]+(-0.19188046135)*x_ref[1]
        ref[1,0]=(0.417367834962)*x_ref[0]+(0.992318201639)*x_ref[1]
        ref[1,1]=(-0.646432128833)*x_ref[0]+(-0.872750321582)*x_ref[1]
        ref[1,2]=(-0.806117185061)*x_ref[0]+(-0.89396408325)*x_ref[1]
        ref[1,3]=(0.768017237627)*x_ref[0]+(-1.2904111912)*x_ref[1]
        ref[1,4]=(-0.0159685523188)*x_ref[0]+(0.401410193169)*x_ref[1]
        ref[2,0]=(-0.844501716181)*x_ref[0]+(-0.123115343819)*x_ref[1]
        ref[2,1]=(1.08441909409)*x_ref[0]+(0.152432282539)*x_ref[1]
        ref[2,2]=(-0.60551037413)*x_ref[0]+(1.15692828826)*x_ref[1]
        ref[2,3]=(-1.21509168541)*x_ref[0]+(1.03683236377)*x_ref[1]
        ref[2,4]=(1.16850730134)*x_ref[0]+(0.162202615909)*x_ref[1]
        ref[3,0]=(0.727772166897)*x_ref[0]+(-0.296815840935)*x_ref[1]
        ref[3,1]=(-0.0880577151489)*x_ref[0]+(1.66427691426)*x_ref[1]
        ref[3,2]=(0.415144534458)*x_ref[0]+(-0.555833723295)*x_ref[1]
        ref[3,3]=(1.09444635829)*x_ref[0]+(-0.335209417536)*x_ref[1]
        ref[3,4]=(-1.05047769585)*x_ref[0]+(0.684427039986)*x_ref[1]
      else:
        arg[0,0]=(-0.611607640773)*x[0]+(0.584485971628)*x[1]+(0.0090316337643)*x[2]
        arg[0,1]=(0.905278285857)*x[0]+(0.332523151242)*x[1]+(0.9909841807)*x[2]
        arg[0,2]=(-1.06629333132)*x[0]+(-0.38478180064)*x[1]+(-0.63309211256)*x[2]
        arg[0,3]=(0.122436386344)*x[0]+(-0.218752671204)*x[1]+(-0.746695172216)*x[2]
        arg[0,4]=(1.03152124874)*x[0]+(-1.11327271563)*x[1]+(-0.257591112135)*x[2]
        arg[1,0]=(0.969019600299)*x[0]+(-0.320279309689)*x[1]+(0.327767746545)*x[2]
        arg[1,1]=(0.291023069052)*x[0]+(0.78112901064)*x[1]+(0.879257239042)*x[2]
        arg[1,2]=(-0.131262891536)*x[0]+(-0.0109690229153)*x[1]+(-0.171900332406)*x[2]
        arg[1,3]=(-1.32272684486)*x[0]+(-0.256869536085)*x[1]+(0.0308524567462)*x[2]
        arg[1,4]=(1.30707061279)*x[0]+(0.972866282634)*x[1]+(-0.474948537882)*x[2]
        arg[2,0]=(0.169034210421)*x[0]+(-1.08714896226)*x[1]+(-0.636838450708)*x[2]
        arg[2,1]=(-0.470752780773)*x[0]+(-0.695286088059)*x[1]+(-1.19197058712)*x[2]
        arg[2,2]=(0.701403721228)*x[0]+(0.551002482436)*x[1]+(-0.295853851234)*x[2]
        arg[2,3]=(-0.796431361745)*x[0]+(0.0581884766856)*x[1]+(-1.13668563089)*x[2]
        arg[2,4]=(-0.733185798017)*x[0]+(0.178900103735)*x[1]+(-0.845671526781)*x[2]
        arg[3,0]=(-1.47064927107)*x[0]+(1.33393324965)*x[1]+(-0.767393460937)*x[2]
        arg[3,1]=(1.36784513226)*x[0]+(0.100008910431)*x[1]+(-1.09210256713)*x[2]
        arg[3,2]=(-0.873076669977)*x[0]+(0.367213631778)*x[1]+(0.0698296920039)*x[2]
        arg[3,3]=(-1.76609214158)*x[0]+(-1.26186286566)*x[1]+(0.377817035113)*x[2]
        arg[3,4]=(-1.28261709396)*x[0]+(0.240480767533)*x[1]+(0.307813279357)*x[2]
        ref[0,0]=(-0.611607640773)*x_ref[0]+(0.584485971628)*x_ref[1]+(0.0090316337643)*x_ref[2]
        ref[0,1]=(0.905278285857)*x_ref[0]+(0.332523151242)*x_ref[1]+(0.9909841807)*x_ref[2]
        ref[0,2]=(-1.06629333132)*x_ref[0]+(-0.38478180064)*x_ref[1]+(-0.63309211256)*x_ref[2]
        ref[0,3]=(0.122436386344)*x_ref[0]+(-0.218752671204)*x_ref[1]+(-0.746695172216)*x_ref[2]
        ref[0,4]=(1.03152124874)*x_ref[0]+(-1.11327271563)*x_ref[1]+(-0.257591112135)*x_ref[2]
        ref[1,0]=(0.969019600299)*x_ref[0]+(-0.320279309689)*x_ref[1]+(0.327767746545)*x_ref[2]
        ref[1,1]=(0.291023069052)*x_ref[0]+(0.78112901064)*x_ref[1]+(0.879257239042)*x_ref[2]
        ref[1,2]=(-0.131262891536)*x_ref[0]+(-0.0109690229153)*x_ref[1]+(-0.171900332406)*x_ref[2]
        ref[1,3]=(-1.32272684486)*x_ref[0]+(-0.256869536085)*x_ref[1]+(0.0308524567462)*x_ref[2]
        ref[1,4]=(1.30707061279)*x_ref[0]+(0.972866282634)*x_ref[1]+(-0.474948537882)*x_ref[2]
        ref[2,0]=(0.169034210421)*x_ref[0]+(-1.08714896226)*x_ref[1]+(-0.636838450708)*x_ref[2]
        ref[2,1]=(-0.470752780773)*x_ref[0]+(-0.695286088059)*x_ref[1]+(-1.19197058712)*x_ref[2]
        ref[2,2]=(0.701403721228)*x_ref[0]+(0.551002482436)*x_ref[1]+(-0.295853851234)*x_ref[2]
        ref[2,3]=(-0.796431361745)*x_ref[0]+(0.0581884766856)*x_ref[1]+(-1.13668563089)*x_ref[2]
        ref[2,4]=(-0.733185798017)*x_ref[0]+(0.178900103735)*x_ref[1]+(-0.845671526781)*x_ref[2]
        ref[3,0]=(-1.47064927107)*x_ref[0]+(1.33393324965)*x_ref[1]+(-0.767393460937)*x_ref[2]
        ref[3,1]=(1.36784513226)*x_ref[0]+(0.100008910431)*x_ref[1]+(-1.09210256713)*x_ref[2]
        ref[3,2]=(-0.873076669977)*x_ref[0]+(0.367213631778)*x_ref[1]+(0.0698296920039)*x_ref[2]
        ref[3,3]=(-1.76609214158)*x_ref[0]+(-1.26186286566)*x_ref[1]+(0.377817035113)*x_ref[2]
        ref[3,4]=(-1.28261709396)*x_ref[0]+(0.240480767533)*x_ref[1]+(0.307813279357)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromData_ReducedContinuousFunction_rank3(self):
      """
      tests interpolate for rank 3 Data onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedContinuousFunction onto ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2),w_ref)
      if dim==2:
        arg[0,0,0]=(-0.423505169404)*x[0]+(-1.10390334264)*x[1]
        arg[0,0,1]=(-0.181820089254)*x[0]+(-0.374602268791)*x[1]
        arg[0,1,0]=(-1.36906917075)*x[0]+(0.26554385908)*x[1]
        arg[0,1,1]=(-0.280661572374)*x[0]+(-0.239232524287)*x[1]
        arg[1,0,0]=(-1.25670251934)*x[0]+(-0.429353977735)*x[1]
        arg[1,0,1]=(-0.747928970735)*x[0]+(1.17874317611)*x[1]
        arg[1,1,0]=(-0.548814611802)*x[0]+(-0.0249315764285)*x[1]
        arg[1,1,1]=(-0.481123117522)*x[0]+(1.23402203879)*x[1]
        arg[2,0,0]=(-0.0639512689884)*x[0]+(-0.0278213371376)*x[1]
        arg[2,0,1]=(-1.17292797621)*x[0]+(0.42140600304)*x[1]
        arg[2,1,0]=(-0.358052601083)*x[0]+(-1.66807880791)*x[1]
        arg[2,1,1]=(-0.291433251349)*x[0]+(0.111473162108)*x[1]
        arg[3,0,0]=(0.742643625989)*x[0]+(-1.29214106228)*x[1]
        arg[3,0,1]=(0.319887321826)*x[0]+(-0.40213369238)*x[1]
        arg[3,1,0]=(-0.449412064097)*x[0]+(-1.10654262009)*x[1]
        arg[3,1,1]=(1.19722227632)*x[0]+(1.3455799026)*x[1]
        arg[4,0,0]=(-1.77453188487)*x[0]+(0.634838793872)*x[1]
        arg[4,0,1]=(-0.215461066907)*x[0]+(1.11635233844)*x[1]
        arg[4,1,0]=(0.835816652592)*x[0]+(0.668378554214)*x[1]
        arg[4,1,1]=(-1.22133225406)*x[0]+(-0.148780252398)*x[1]
        arg[5,0,0]=(0.320245409639)*x[0]+(-1.08828513776)*x[1]
        arg[5,0,1]=(-0.1865835417)*x[0]+(-0.353282061874)*x[1]
        arg[5,1,0]=(1.38858040685)*x[0]+(0.166398261098)*x[1]
        arg[5,1,1]=(0.947718875755)*x[0]+(-1.06171934302)*x[1]
        ref[0,0,0]=(-0.423505169404)*x_ref[0]+(-1.10390334264)*x_ref[1]
        ref[0,0,1]=(-0.181820089254)*x_ref[0]+(-0.374602268791)*x_ref[1]
        ref[0,1,0]=(-1.36906917075)*x_ref[0]+(0.26554385908)*x_ref[1]
        ref[0,1,1]=(-0.280661572374)*x_ref[0]+(-0.239232524287)*x_ref[1]
        ref[1,0,0]=(-1.25670251934)*x_ref[0]+(-0.429353977735)*x_ref[1]
        ref[1,0,1]=(-0.747928970735)*x_ref[0]+(1.17874317611)*x_ref[1]
        ref[1,1,0]=(-0.548814611802)*x_ref[0]+(-0.0249315764285)*x_ref[1]
        ref[1,1,1]=(-0.481123117522)*x_ref[0]+(1.23402203879)*x_ref[1]
        ref[2,0,0]=(-0.0639512689884)*x_ref[0]+(-0.0278213371376)*x_ref[1]
        ref[2,0,1]=(-1.17292797621)*x_ref[0]+(0.42140600304)*x_ref[1]
        ref[2,1,0]=(-0.358052601083)*x_ref[0]+(-1.66807880791)*x_ref[1]
        ref[2,1,1]=(-0.291433251349)*x_ref[0]+(0.111473162108)*x_ref[1]
        ref[3,0,0]=(0.742643625989)*x_ref[0]+(-1.29214106228)*x_ref[1]
        ref[3,0,1]=(0.319887321826)*x_ref[0]+(-0.40213369238)*x_ref[1]
        ref[3,1,0]=(-0.449412064097)*x_ref[0]+(-1.10654262009)*x_ref[1]
        ref[3,1,1]=(1.19722227632)*x_ref[0]+(1.3455799026)*x_ref[1]
        ref[4,0,0]=(-1.77453188487)*x_ref[0]+(0.634838793872)*x_ref[1]
        ref[4,0,1]=(-0.215461066907)*x_ref[0]+(1.11635233844)*x_ref[1]
        ref[4,1,0]=(0.835816652592)*x_ref[0]+(0.668378554214)*x_ref[1]
        ref[4,1,1]=(-1.22133225406)*x_ref[0]+(-0.148780252398)*x_ref[1]
        ref[5,0,0]=(0.320245409639)*x_ref[0]+(-1.08828513776)*x_ref[1]
        ref[5,0,1]=(-0.1865835417)*x_ref[0]+(-0.353282061874)*x_ref[1]
        ref[5,1,0]=(1.38858040685)*x_ref[0]+(0.166398261098)*x_ref[1]
        ref[5,1,1]=(0.947718875755)*x_ref[0]+(-1.06171934302)*x_ref[1]
      else:
        arg[0,0,0]=(-0.892096322478)*x[0]+(0.720363183451)*x[1]+(0.600609272191)*x[2]
        arg[0,0,1]=(-0.418024461935)*x[0]+(-0.819275292035)*x[1]+(0.134962557692)*x[2]
        arg[0,1,0]=(-0.449783838361)*x[0]+(0.943677173533)*x[1]+(0.209424190991)*x[2]
        arg[0,1,1]=(-0.424386775004)*x[0]+(-0.213024388094)*x[1]+(-1.05927943384)*x[2]
        arg[1,0,0]=(-0.699504096539)*x[0]+(-0.153570308377)*x[1]+(0.576902392658)*x[2]
        arg[1,0,1]=(-0.426914055661)*x[0]+(-0.330405267059)*x[1]+(1.07656632155)*x[2]
        arg[1,1,0]=(0.775238005352)*x[0]+(1.11012072575)*x[1]+(0.764627098603)*x[2]
        arg[1,1,1]=(0.0130424708408)*x[0]+(-0.671778102107)*x[1]+(0.780684368983)*x[2]
        arg[2,0,0]=(-0.0512224258498)*x[0]+(-1.50515803984)*x[1]+(0.342824143946)*x[2]
        arg[2,0,1]=(0.117541624176)*x[0]+(0.372415524552)*x[1]+(0.255218095016)*x[2]
        arg[2,1,0]=(-0.574508895703)*x[0]+(0.345217292359)*x[1]+(0.209211956821)*x[2]
        arg[2,1,1]=(0.93643587147)*x[0]+(-0.096209009902)*x[1]+(0.205150859527)*x[2]
        arg[3,0,0]=(1.7541929884)*x[0]+(0.169061478447)*x[1]+(0.534746695515)*x[2]
        arg[3,0,1]=(0.442329266782)*x[0]+(0.334971710414)*x[1]+(1.66892557698)*x[2]
        arg[3,1,0]=(0.316436716758)*x[0]+(-1.25442030987)*x[1]+(0.372183105374)*x[2]
        arg[3,1,1]=(-0.254157820265)*x[0]+(0.287381300023)*x[1]+(0.0574117423352)*x[2]
        arg[4,0,0]=(-1.08380257521)*x[0]+(-1.31461890032)*x[1]+(-0.503802775099)*x[2]
        arg[4,0,1]=(-0.558293486798)*x[0]+(0.702317635925)*x[1]+(0.463814829137)*x[2]
        arg[4,1,0]=(-0.677160971974)*x[0]+(-0.14921275312)*x[1]+(-1.6501459435)*x[2]
        arg[4,1,1]=(-0.176672532474)*x[0]+(0.945017605811)*x[1]+(-0.790342939788)*x[2]
        arg[5,0,0]=(-1.45668070675)*x[0]+(-0.404957129204)*x[1]+(1.37383065954)*x[2]
        arg[5,0,1]=(0.53829326756)*x[0]+(0.543333031479)*x[1]+(-0.854057296362)*x[2]
        arg[5,1,0]=(0.805864283921)*x[0]+(-0.00186634953513)*x[1]+(-1.97111362539)*x[2]
        arg[5,1,1]=(-0.387746238967)*x[0]+(-0.739158353882)*x[1]+(-0.134082808118)*x[2]
        ref[0,0,0]=(-0.892096322478)*x_ref[0]+(0.720363183451)*x_ref[1]+(0.600609272191)*x_ref[2]
        ref[0,0,1]=(-0.418024461935)*x_ref[0]+(-0.819275292035)*x_ref[1]+(0.134962557692)*x_ref[2]
        ref[0,1,0]=(-0.449783838361)*x_ref[0]+(0.943677173533)*x_ref[1]+(0.209424190991)*x_ref[2]
        ref[0,1,1]=(-0.424386775004)*x_ref[0]+(-0.213024388094)*x_ref[1]+(-1.05927943384)*x_ref[2]
        ref[1,0,0]=(-0.699504096539)*x_ref[0]+(-0.153570308377)*x_ref[1]+(0.576902392658)*x_ref[2]
        ref[1,0,1]=(-0.426914055661)*x_ref[0]+(-0.330405267059)*x_ref[1]+(1.07656632155)*x_ref[2]
        ref[1,1,0]=(0.775238005352)*x_ref[0]+(1.11012072575)*x_ref[1]+(0.764627098603)*x_ref[2]
        ref[1,1,1]=(0.0130424708408)*x_ref[0]+(-0.671778102107)*x_ref[1]+(0.780684368983)*x_ref[2]
        ref[2,0,0]=(-0.0512224258498)*x_ref[0]+(-1.50515803984)*x_ref[1]+(0.342824143946)*x_ref[2]
        ref[2,0,1]=(0.117541624176)*x_ref[0]+(0.372415524552)*x_ref[1]+(0.255218095016)*x_ref[2]
        ref[2,1,0]=(-0.574508895703)*x_ref[0]+(0.345217292359)*x_ref[1]+(0.209211956821)*x_ref[2]
        ref[2,1,1]=(0.93643587147)*x_ref[0]+(-0.096209009902)*x_ref[1]+(0.205150859527)*x_ref[2]
        ref[3,0,0]=(1.7541929884)*x_ref[0]+(0.169061478447)*x_ref[1]+(0.534746695515)*x_ref[2]
        ref[3,0,1]=(0.442329266782)*x_ref[0]+(0.334971710414)*x_ref[1]+(1.66892557698)*x_ref[2]
        ref[3,1,0]=(0.316436716758)*x_ref[0]+(-1.25442030987)*x_ref[1]+(0.372183105374)*x_ref[2]
        ref[3,1,1]=(-0.254157820265)*x_ref[0]+(0.287381300023)*x_ref[1]+(0.0574117423352)*x_ref[2]
        ref[4,0,0]=(-1.08380257521)*x_ref[0]+(-1.31461890032)*x_ref[1]+(-0.503802775099)*x_ref[2]
        ref[4,0,1]=(-0.558293486798)*x_ref[0]+(0.702317635925)*x_ref[1]+(0.463814829137)*x_ref[2]
        ref[4,1,0]=(-0.677160971974)*x_ref[0]+(-0.14921275312)*x_ref[1]+(-1.6501459435)*x_ref[2]
        ref[4,1,1]=(-0.176672532474)*x_ref[0]+(0.945017605811)*x_ref[1]+(-0.790342939788)*x_ref[2]
        ref[5,0,0]=(-1.45668070675)*x_ref[0]+(-0.404957129204)*x_ref[1]+(1.37383065954)*x_ref[2]
        ref[5,0,1]=(0.53829326756)*x_ref[0]+(0.543333031479)*x_ref[1]+(-0.854057296362)*x_ref[2]
        ref[5,1,0]=(0.805864283921)*x_ref[0]+(-0.00186634953513)*x_ref[1]+(-1.97111362539)*x_ref[2]
        ref[5,1,1]=(-0.387746238967)*x_ref[0]+(-0.739158353882)*x_ref[1]+(-0.134082808118)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromData_ReducedContinuousFunction_rank4(self):
      """
      tests interpolate for rank 4 Data onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedContinuousFunction onto ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5, 3, 2),w)
      ref=Data(0,(4, 5, 3, 2),w_ref)
      if dim==2:
        arg[0,0,0,0]=(-0.754311658086)*x[0]+(0.235157034342)*x[1]
        arg[0,0,0,1]=(-1.33786315024)*x[0]+(0.243349279742)*x[1]
        arg[0,0,1,0]=(1.48657750088)*x[0]+(1.09853108305)*x[1]
        arg[0,0,1,1]=(1.4501362266)*x[0]+(0.824811567155)*x[1]
        arg[0,0,2,0]=(1.12567566082)*x[0]+(0.663525681174)*x[1]
        arg[0,0,2,1]=(1.48991565019)*x[0]+(-0.680795365011)*x[1]
        arg[0,1,0,0]=(0.268571449528)*x[0]+(0.818595834672)*x[1]
        arg[0,1,0,1]=(-1.55490945555)*x[0]+(0.783595197888)*x[1]
        arg[0,1,1,0]=(0.913201382585)*x[0]+(0.855491379376)*x[1]
        arg[0,1,1,1]=(-0.179192075116)*x[0]+(-0.690415079523)*x[1]
        arg[0,1,2,0]=(-0.0393574502912)*x[0]+(-0.690157650634)*x[1]
        arg[0,1,2,1]=(0.784216863217)*x[0]+(0.498735486159)*x[1]
        arg[0,2,0,0]=(0.137939554165)*x[0]+(-1.33961519113)*x[1]
        arg[0,2,0,1]=(-1.03897311736)*x[0]+(-1.37797405312)*x[1]
        arg[0,2,1,0]=(0.199812003756)*x[0]+(-0.0388499674037)*x[1]
        arg[0,2,1,1]=(1.15530534202)*x[0]+(-1.28280869078)*x[1]
        arg[0,2,2,0]=(0.836447791155)*x[0]+(1.2627066026)*x[1]
        arg[0,2,2,1]=(-0.584608492688)*x[0]+(-1.55305974305)*x[1]
        arg[0,3,0,0]=(-0.661952381799)*x[0]+(-1.27062403778)*x[1]
        arg[0,3,0,1]=(-1.47789532259)*x[0]+(-0.279583524936)*x[1]
        arg[0,3,1,0]=(1.40729771357)*x[0]+(0.0971284144528)*x[1]
        arg[0,3,1,1]=(-0.459167955911)*x[0]+(-1.29033281524)*x[1]
        arg[0,3,2,0]=(-1.63189436824)*x[0]+(0.75729808422)*x[1]
        arg[0,3,2,1]=(0.522479828718)*x[0]+(-0.0231299515383)*x[1]
        arg[0,4,0,0]=(0.241891446615)*x[0]+(0.803327107651)*x[1]
        arg[0,4,0,1]=(0.273613502836)*x[0]+(-0.0115180464171)*x[1]
        arg[0,4,1,0]=(1.96219818696)*x[0]+(-0.349137094743)*x[1]
        arg[0,4,1,1]=(0.365690821719)*x[0]+(-1.6943347181)*x[1]
        arg[0,4,2,0]=(-0.283970779872)*x[0]+(-0.258537179804)*x[1]
        arg[0,4,2,1]=(-0.973333430598)*x[0]+(1.48706276212)*x[1]
        arg[1,0,0,0]=(1.18757466971)*x[0]+(0.838804656357)*x[1]
        arg[1,0,0,1]=(0.840859904702)*x[0]+(1.09955195682)*x[1]
        arg[1,0,1,0]=(1.46190513227)*x[0]+(-0.596959830422)*x[1]
        arg[1,0,1,1]=(0.718689296745)*x[0]+(-0.628074620222)*x[1]
        arg[1,0,2,0]=(0.440761293041)*x[0]+(0.115800854194)*x[1]
        arg[1,0,2,1]=(1.34891146904)*x[0]+(0.813690260652)*x[1]
        arg[1,1,0,0]=(0.183071638098)*x[0]+(-1.37996210755)*x[1]
        arg[1,1,0,1]=(0.258142115988)*x[0]+(-0.757709381529)*x[1]
        arg[1,1,1,0]=(1.72392829309)*x[0]+(0.444995102781)*x[1]
        arg[1,1,1,1]=(-0.380898209988)*x[0]+(-0.0950246860347)*x[1]
        arg[1,1,2,0]=(0.203007226016)*x[0]+(1.09786001832)*x[1]
        arg[1,1,2,1]=(-0.502500865341)*x[0]+(-0.629980527995)*x[1]
        arg[1,2,0,0]=(0.482549518436)*x[0]+(0.565840824146)*x[1]
        arg[1,2,0,1]=(0.355818565068)*x[0]+(-0.78273295574)*x[1]
        arg[1,2,1,0]=(1.79204033071)*x[0]+(-0.698040851585)*x[1]
        arg[1,2,1,1]=(-0.534827741044)*x[0]+(0.00943131627498)*x[1]
        arg[1,2,2,0]=(-1.00663524444)*x[0]+(0.0207854553456)*x[1]
        arg[1,2,2,1]=(-0.338048040159)*x[0]+(-0.380607406217)*x[1]
        arg[1,3,0,0]=(0.586310549044)*x[0]+(0.276326798805)*x[1]
        arg[1,3,0,1]=(-0.260482580702)*x[0]+(-0.349279395321)*x[1]
        arg[1,3,1,0]=(-0.196533829895)*x[0]+(0.614805272738)*x[1]
        arg[1,3,1,1]=(-0.363176483895)*x[0]+(-0.925817017731)*x[1]
        arg[1,3,2,0]=(-1.24182075421)*x[0]+(0.215966446013)*x[1]
        arg[1,3,2,1]=(1.07546652642)*x[0]+(0.383803991974)*x[1]
        arg[1,4,0,0]=(0.199312303525)*x[0]+(-0.314520883859)*x[1]
        arg[1,4,0,1]=(-1.56890826152)*x[0]+(-0.133086662241)*x[1]
        arg[1,4,1,0]=(1.1307241936)*x[0]+(-1.32905744984)*x[1]
        arg[1,4,1,1]=(-0.385991455298)*x[0]+(0.292913024507)*x[1]
        arg[1,4,2,0]=(1.14561678056)*x[0]+(-0.662696162276)*x[1]
        arg[1,4,2,1]=(-1.49533948954)*x[0]+(1.43396138744)*x[1]
        arg[2,0,0,0]=(-0.63160396493)*x[0]+(0.813120998776)*x[1]
        arg[2,0,0,1]=(0.65598998434)*x[0]+(-1.02445537553)*x[1]
        arg[2,0,1,0]=(-1.13473104075)*x[0]+(0.057331183449)*x[1]
        arg[2,0,1,1]=(-0.154892482107)*x[0]+(0.481969957579)*x[1]
        arg[2,0,2,0]=(0.521916671496)*x[0]+(0.546676943059)*x[1]
        arg[2,0,2,1]=(-1.68919379028)*x[0]+(1.08292038981)*x[1]
        arg[2,1,0,0]=(-0.643846595888)*x[0]+(0.528306097755)*x[1]
        arg[2,1,0,1]=(1.3745090497)*x[0]+(-0.0801648509184)*x[1]
        arg[2,1,1,0]=(0.129096817054)*x[0]+(0.90750370855)*x[1]
        arg[2,1,1,1]=(0.539309513305)*x[0]+(1.34949470961)*x[1]
        arg[2,1,2,0]=(1.4189703896)*x[0]+(-1.75011201893)*x[1]
        arg[2,1,2,1]=(-1.09325548386)*x[0]+(0.148279301716)*x[1]
        arg[2,2,0,0]=(-1.25785133743)*x[0]+(0.275071306594)*x[1]
        arg[2,2,0,1]=(-0.550152068761)*x[0]+(-1.20422484275)*x[1]
        arg[2,2,1,0]=(-0.102062863668)*x[0]+(-1.34431887541)*x[1]
        arg[2,2,1,1]=(0.103354724169)*x[0]+(-0.941952180446)*x[1]
        arg[2,2,2,0]=(1.2090339604)*x[0]+(-0.330357976796)*x[1]
        arg[2,2,2,1]=(0.203162449188)*x[0]+(-1.03640702014)*x[1]
        arg[2,3,0,0]=(-0.324576353514)*x[0]+(0.582614539267)*x[1]
        arg[2,3,0,1]=(-0.213823136215)*x[0]+(-0.473199105214)*x[1]
        arg[2,3,1,0]=(0.395144702599)*x[0]+(0.0358194467152)*x[1]
        arg[2,3,1,1]=(1.32860563396)*x[0]+(-0.577308310732)*x[1]
        arg[2,3,2,0]=(0.994205029432)*x[0]+(1.13433517636)*x[1]
        arg[2,3,2,1]=(-0.488499286742)*x[0]+(0.224253820738)*x[1]
        arg[2,4,0,0]=(1.48035663008)*x[0]+(-1.09697986914)*x[1]
        arg[2,4,0,1]=(-1.09991915151)*x[0]+(-0.557951199985)*x[1]
        arg[2,4,1,0]=(-1.23862699804)*x[0]+(-0.230602187713)*x[1]
        arg[2,4,1,1]=(-0.922609370448)*x[0]+(1.76673953413)*x[1]
        arg[2,4,2,0]=(0.731769419709)*x[0]+(-1.1094516608)*x[1]
        arg[2,4,2,1]=(-0.145672504805)*x[0]+(0.168194421355)*x[1]
        arg[3,0,0,0]=(1.02559707602)*x[0]+(0.260431368523)*x[1]
        arg[3,0,0,1]=(0.111899568824)*x[0]+(-0.90725475423)*x[1]
        arg[3,0,1,0]=(-0.226547745683)*x[0]+(-0.232830393372)*x[1]
        arg[3,0,1,1]=(0.347004318471)*x[0]+(-0.196130970001)*x[1]
        arg[3,0,2,0]=(-1.72155871442)*x[0]+(-1.28372407452)*x[1]
        arg[3,0,2,1]=(1.43817914693)*x[0]+(-0.354093253684)*x[1]
        arg[3,1,0,0]=(-0.293939245102)*x[0]+(0.0255562139103)*x[1]
        arg[3,1,0,1]=(0.139306213018)*x[0]+(0.595062226642)*x[1]
        arg[3,1,1,0]=(-0.670339920672)*x[0]+(0.387520583192)*x[1]
        arg[3,1,1,1]=(-0.0323325470954)*x[0]+(-0.681162999279)*x[1]
        arg[3,1,2,0]=(0.235275048739)*x[0]+(0.144725473773)*x[1]
        arg[3,1,2,1]=(0.132211795214)*x[0]+(0.262780216845)*x[1]
        arg[3,2,0,0]=(0.590514326163)*x[0]+(-0.84273677918)*x[1]
        arg[3,2,0,1]=(-0.925181556185)*x[0]+(-0.300244447001)*x[1]
        arg[3,2,1,0]=(-0.0509700795112)*x[0]+(-1.31919389616)*x[1]
        arg[3,2,1,1]=(1.50577291846)*x[0]+(-0.219284607769)*x[1]
        arg[3,2,2,0]=(-0.0460616673346)*x[0]+(-0.690312685466)*x[1]
        arg[3,2,2,1]=(-0.673765477936)*x[0]+(-0.0965860475641)*x[1]
        arg[3,3,0,0]=(-0.6577594174)*x[0]+(0.171886277016)*x[1]
        arg[3,3,0,1]=(-1.08810434718)*x[0]+(0.333153250183)*x[1]
        arg[3,3,1,0]=(0.150968371983)*x[0]+(0.290859317391)*x[1]
        arg[3,3,1,1]=(1.62096265131)*x[0]+(-1.10309602741)*x[1]
        arg[3,3,2,0]=(-0.932063639972)*x[0]+(-0.897131561126)*x[1]
        arg[3,3,2,1]=(-0.819110834345)*x[0]+(1.59159109562)*x[1]
        arg[3,4,0,0]=(0.272662931155)*x[0]+(0.69026285756)*x[1]
        arg[3,4,0,1]=(-0.600556370757)*x[0]+(-0.0853533546302)*x[1]
        arg[3,4,1,0]=(1.18544735281)*x[0]+(0.217751189034)*x[1]
        arg[3,4,1,1]=(-0.594609531358)*x[0]+(0.882158349788)*x[1]
        arg[3,4,2,0]=(1.18876765552)*x[0]+(-0.411481586979)*x[1]
        arg[3,4,2,1]=(-1.093067792)*x[0]+(-0.266744260768)*x[1]
        ref[0,0,0,0]=(-0.754311658086)*x_ref[0]+(0.235157034342)*x_ref[1]
        ref[0,0,0,1]=(-1.33786315024)*x_ref[0]+(0.243349279742)*x_ref[1]
        ref[0,0,1,0]=(1.48657750088)*x_ref[0]+(1.09853108305)*x_ref[1]
        ref[0,0,1,1]=(1.4501362266)*x_ref[0]+(0.824811567155)*x_ref[1]
        ref[0,0,2,0]=(1.12567566082)*x_ref[0]+(0.663525681174)*x_ref[1]
        ref[0,0,2,1]=(1.48991565019)*x_ref[0]+(-0.680795365011)*x_ref[1]
        ref[0,1,0,0]=(0.268571449528)*x_ref[0]+(0.818595834672)*x_ref[1]
        ref[0,1,0,1]=(-1.55490945555)*x_ref[0]+(0.783595197888)*x_ref[1]
        ref[0,1,1,0]=(0.913201382585)*x_ref[0]+(0.855491379376)*x_ref[1]
        ref[0,1,1,1]=(-0.179192075116)*x_ref[0]+(-0.690415079523)*x_ref[1]
        ref[0,1,2,0]=(-0.0393574502912)*x_ref[0]+(-0.690157650634)*x_ref[1]
        ref[0,1,2,1]=(0.784216863217)*x_ref[0]+(0.498735486159)*x_ref[1]
        ref[0,2,0,0]=(0.137939554165)*x_ref[0]+(-1.33961519113)*x_ref[1]
        ref[0,2,0,1]=(-1.03897311736)*x_ref[0]+(-1.37797405312)*x_ref[1]
        ref[0,2,1,0]=(0.199812003756)*x_ref[0]+(-0.0388499674037)*x_ref[1]
        ref[0,2,1,1]=(1.15530534202)*x_ref[0]+(-1.28280869078)*x_ref[1]
        ref[0,2,2,0]=(0.836447791155)*x_ref[0]+(1.2627066026)*x_ref[1]
        ref[0,2,2,1]=(-0.584608492688)*x_ref[0]+(-1.55305974305)*x_ref[1]
        ref[0,3,0,0]=(-0.661952381799)*x_ref[0]+(-1.27062403778)*x_ref[1]
        ref[0,3,0,1]=(-1.47789532259)*x_ref[0]+(-0.279583524936)*x_ref[1]
        ref[0,3,1,0]=(1.40729771357)*x_ref[0]+(0.0971284144528)*x_ref[1]
        ref[0,3,1,1]=(-0.459167955911)*x_ref[0]+(-1.29033281524)*x_ref[1]
        ref[0,3,2,0]=(-1.63189436824)*x_ref[0]+(0.75729808422)*x_ref[1]
        ref[0,3,2,1]=(0.522479828718)*x_ref[0]+(-0.0231299515383)*x_ref[1]
        ref[0,4,0,0]=(0.241891446615)*x_ref[0]+(0.803327107651)*x_ref[1]
        ref[0,4,0,1]=(0.273613502836)*x_ref[0]+(-0.0115180464171)*x_ref[1]
        ref[0,4,1,0]=(1.96219818696)*x_ref[0]+(-0.349137094743)*x_ref[1]
        ref[0,4,1,1]=(0.365690821719)*x_ref[0]+(-1.6943347181)*x_ref[1]
        ref[0,4,2,0]=(-0.283970779872)*x_ref[0]+(-0.258537179804)*x_ref[1]
        ref[0,4,2,1]=(-0.973333430598)*x_ref[0]+(1.48706276212)*x_ref[1]
        ref[1,0,0,0]=(1.18757466971)*x_ref[0]+(0.838804656357)*x_ref[1]
        ref[1,0,0,1]=(0.840859904702)*x_ref[0]+(1.09955195682)*x_ref[1]
        ref[1,0,1,0]=(1.46190513227)*x_ref[0]+(-0.596959830422)*x_ref[1]
        ref[1,0,1,1]=(0.718689296745)*x_ref[0]+(-0.628074620222)*x_ref[1]
        ref[1,0,2,0]=(0.440761293041)*x_ref[0]+(0.115800854194)*x_ref[1]
        ref[1,0,2,1]=(1.34891146904)*x_ref[0]+(0.813690260652)*x_ref[1]
        ref[1,1,0,0]=(0.183071638098)*x_ref[0]+(-1.37996210755)*x_ref[1]
        ref[1,1,0,1]=(0.258142115988)*x_ref[0]+(-0.757709381529)*x_ref[1]
        ref[1,1,1,0]=(1.72392829309)*x_ref[0]+(0.444995102781)*x_ref[1]
        ref[1,1,1,1]=(-0.380898209988)*x_ref[0]+(-0.0950246860347)*x_ref[1]
        ref[1,1,2,0]=(0.203007226016)*x_ref[0]+(1.09786001832)*x_ref[1]
        ref[1,1,2,1]=(-0.502500865341)*x_ref[0]+(-0.629980527995)*x_ref[1]
        ref[1,2,0,0]=(0.482549518436)*x_ref[0]+(0.565840824146)*x_ref[1]
        ref[1,2,0,1]=(0.355818565068)*x_ref[0]+(-0.78273295574)*x_ref[1]
        ref[1,2,1,0]=(1.79204033071)*x_ref[0]+(-0.698040851585)*x_ref[1]
        ref[1,2,1,1]=(-0.534827741044)*x_ref[0]+(0.00943131627498)*x_ref[1]
        ref[1,2,2,0]=(-1.00663524444)*x_ref[0]+(0.0207854553456)*x_ref[1]
        ref[1,2,2,1]=(-0.338048040159)*x_ref[0]+(-0.380607406217)*x_ref[1]
        ref[1,3,0,0]=(0.586310549044)*x_ref[0]+(0.276326798805)*x_ref[1]
        ref[1,3,0,1]=(-0.260482580702)*x_ref[0]+(-0.349279395321)*x_ref[1]
        ref[1,3,1,0]=(-0.196533829895)*x_ref[0]+(0.614805272738)*x_ref[1]
        ref[1,3,1,1]=(-0.363176483895)*x_ref[0]+(-0.925817017731)*x_ref[1]
        ref[1,3,2,0]=(-1.24182075421)*x_ref[0]+(0.215966446013)*x_ref[1]
        ref[1,3,2,1]=(1.07546652642)*x_ref[0]+(0.383803991974)*x_ref[1]
        ref[1,4,0,0]=(0.199312303525)*x_ref[0]+(-0.314520883859)*x_ref[1]
        ref[1,4,0,1]=(-1.56890826152)*x_ref[0]+(-0.133086662241)*x_ref[1]
        ref[1,4,1,0]=(1.1307241936)*x_ref[0]+(-1.32905744984)*x_ref[1]
        ref[1,4,1,1]=(-0.385991455298)*x_ref[0]+(0.292913024507)*x_ref[1]
        ref[1,4,2,0]=(1.14561678056)*x_ref[0]+(-0.662696162276)*x_ref[1]
        ref[1,4,2,1]=(-1.49533948954)*x_ref[0]+(1.43396138744)*x_ref[1]
        ref[2,0,0,0]=(-0.63160396493)*x_ref[0]+(0.813120998776)*x_ref[1]
        ref[2,0,0,1]=(0.65598998434)*x_ref[0]+(-1.02445537553)*x_ref[1]
        ref[2,0,1,0]=(-1.13473104075)*x_ref[0]+(0.057331183449)*x_ref[1]
        ref[2,0,1,1]=(-0.154892482107)*x_ref[0]+(0.481969957579)*x_ref[1]
        ref[2,0,2,0]=(0.521916671496)*x_ref[0]+(0.546676943059)*x_ref[1]
        ref[2,0,2,1]=(-1.68919379028)*x_ref[0]+(1.08292038981)*x_ref[1]
        ref[2,1,0,0]=(-0.643846595888)*x_ref[0]+(0.528306097755)*x_ref[1]
        ref[2,1,0,1]=(1.3745090497)*x_ref[0]+(-0.0801648509184)*x_ref[1]
        ref[2,1,1,0]=(0.129096817054)*x_ref[0]+(0.90750370855)*x_ref[1]
        ref[2,1,1,1]=(0.539309513305)*x_ref[0]+(1.34949470961)*x_ref[1]
        ref[2,1,2,0]=(1.4189703896)*x_ref[0]+(-1.75011201893)*x_ref[1]
        ref[2,1,2,1]=(-1.09325548386)*x_ref[0]+(0.148279301716)*x_ref[1]
        ref[2,2,0,0]=(-1.25785133743)*x_ref[0]+(0.275071306594)*x_ref[1]
        ref[2,2,0,1]=(-0.550152068761)*x_ref[0]+(-1.20422484275)*x_ref[1]
        ref[2,2,1,0]=(-0.102062863668)*x_ref[0]+(-1.34431887541)*x_ref[1]
        ref[2,2,1,1]=(0.103354724169)*x_ref[0]+(-0.941952180446)*x_ref[1]
        ref[2,2,2,0]=(1.2090339604)*x_ref[0]+(-0.330357976796)*x_ref[1]
        ref[2,2,2,1]=(0.203162449188)*x_ref[0]+(-1.03640702014)*x_ref[1]
        ref[2,3,0,0]=(-0.324576353514)*x_ref[0]+(0.582614539267)*x_ref[1]
        ref[2,3,0,1]=(-0.213823136215)*x_ref[0]+(-0.473199105214)*x_ref[1]
        ref[2,3,1,0]=(0.395144702599)*x_ref[0]+(0.0358194467152)*x_ref[1]
        ref[2,3,1,1]=(1.32860563396)*x_ref[0]+(-0.577308310732)*x_ref[1]
        ref[2,3,2,0]=(0.994205029432)*x_ref[0]+(1.13433517636)*x_ref[1]
        ref[2,3,2,1]=(-0.488499286742)*x_ref[0]+(0.224253820738)*x_ref[1]
        ref[2,4,0,0]=(1.48035663008)*x_ref[0]+(-1.09697986914)*x_ref[1]
        ref[2,4,0,1]=(-1.09991915151)*x_ref[0]+(-0.557951199985)*x_ref[1]
        ref[2,4,1,0]=(-1.23862699804)*x_ref[0]+(-0.230602187713)*x_ref[1]
        ref[2,4,1,1]=(-0.922609370448)*x_ref[0]+(1.76673953413)*x_ref[1]
        ref[2,4,2,0]=(0.731769419709)*x_ref[0]+(-1.1094516608)*x_ref[1]
        ref[2,4,2,1]=(-0.145672504805)*x_ref[0]+(0.168194421355)*x_ref[1]
        ref[3,0,0,0]=(1.02559707602)*x_ref[0]+(0.260431368523)*x_ref[1]
        ref[3,0,0,1]=(0.111899568824)*x_ref[0]+(-0.90725475423)*x_ref[1]
        ref[3,0,1,0]=(-0.226547745683)*x_ref[0]+(-0.232830393372)*x_ref[1]
        ref[3,0,1,1]=(0.347004318471)*x_ref[0]+(-0.196130970001)*x_ref[1]
        ref[3,0,2,0]=(-1.72155871442)*x_ref[0]+(-1.28372407452)*x_ref[1]
        ref[3,0,2,1]=(1.43817914693)*x_ref[0]+(-0.354093253684)*x_ref[1]
        ref[3,1,0,0]=(-0.293939245102)*x_ref[0]+(0.0255562139103)*x_ref[1]
        ref[3,1,0,1]=(0.139306213018)*x_ref[0]+(0.595062226642)*x_ref[1]
        ref[3,1,1,0]=(-0.670339920672)*x_ref[0]+(0.387520583192)*x_ref[1]
        ref[3,1,1,1]=(-0.0323325470954)*x_ref[0]+(-0.681162999279)*x_ref[1]
        ref[3,1,2,0]=(0.235275048739)*x_ref[0]+(0.144725473773)*x_ref[1]
        ref[3,1,2,1]=(0.132211795214)*x_ref[0]+(0.262780216845)*x_ref[1]
        ref[3,2,0,0]=(0.590514326163)*x_ref[0]+(-0.84273677918)*x_ref[1]
        ref[3,2,0,1]=(-0.925181556185)*x_ref[0]+(-0.300244447001)*x_ref[1]
        ref[3,2,1,0]=(-0.0509700795112)*x_ref[0]+(-1.31919389616)*x_ref[1]
        ref[3,2,1,1]=(1.50577291846)*x_ref[0]+(-0.219284607769)*x_ref[1]
        ref[3,2,2,0]=(-0.0460616673346)*x_ref[0]+(-0.690312685466)*x_ref[1]
        ref[3,2,2,1]=(-0.673765477936)*x_ref[0]+(-0.0965860475641)*x_ref[1]
        ref[3,3,0,0]=(-0.6577594174)*x_ref[0]+(0.171886277016)*x_ref[1]
        ref[3,3,0,1]=(-1.08810434718)*x_ref[0]+(0.333153250183)*x_ref[1]
        ref[3,3,1,0]=(0.150968371983)*x_ref[0]+(0.290859317391)*x_ref[1]
        ref[3,3,1,1]=(1.62096265131)*x_ref[0]+(-1.10309602741)*x_ref[1]
        ref[3,3,2,0]=(-0.932063639972)*x_ref[0]+(-0.897131561126)*x_ref[1]
        ref[3,3,2,1]=(-0.819110834345)*x_ref[0]+(1.59159109562)*x_ref[1]
        ref[3,4,0,0]=(0.272662931155)*x_ref[0]+(0.69026285756)*x_ref[1]
        ref[3,4,0,1]=(-0.600556370757)*x_ref[0]+(-0.0853533546302)*x_ref[1]
        ref[3,4,1,0]=(1.18544735281)*x_ref[0]+(0.217751189034)*x_ref[1]
        ref[3,4,1,1]=(-0.594609531358)*x_ref[0]+(0.882158349788)*x_ref[1]
        ref[3,4,2,0]=(1.18876765552)*x_ref[0]+(-0.411481586979)*x_ref[1]
        ref[3,4,2,1]=(-1.093067792)*x_ref[0]+(-0.266744260768)*x_ref[1]
      else:
        arg[0,0,0,0]=(-1.29218217256)*x[0]+(-0.382937892213)*x[1]+(0.14363896447)*x[2]
        arg[0,0,0,1]=(1.39856942421)*x[0]+(0.302082267835)*x[1]+(-0.40870736584)*x[2]
        arg[0,0,1,0]=(0.523846108043)*x[0]+(1.19961531204)*x[1]+(0.838881761349)*x[2]
        arg[0,0,1,1]=(0.634542282764)*x[0]+(-0.475514686466)*x[1]+(-1.68745428486)*x[2]
        arg[0,0,2,0]=(1.16379585731)*x[0]+(-0.928037595416)*x[1]+(0.998857178353)*x[2]
        arg[0,0,2,1]=(-0.804515607022)*x[0]+(-0.915469493439)*x[1]+(-0.143888000623)*x[2]
        arg[0,1,0,0]=(0.925843155486)*x[0]+(-0.168243418953)*x[1]+(-0.887839555187)*x[2]
        arg[0,1,0,1]=(-0.913103105329)*x[0]+(1.38011681732)*x[1]+(-0.992478793291)*x[2]
        arg[0,1,1,0]=(0.759685289903)*x[0]+(-0.579481939982)*x[1]+(0.577682929138)*x[2]
        arg[0,1,1,1]=(0.813155548228)*x[0]+(-0.178317843217)*x[1]+(-0.0460113194526)*x[2]
        arg[0,1,2,0]=(-0.195764860141)*x[0]+(1.44256979628)*x[1]+(0.283992441343)*x[2]
        arg[0,1,2,1]=(-0.921229459072)*x[0]+(-1.19295761547)*x[1]+(0.215736801944)*x[2]
        arg[0,2,0,0]=(-0.741726331752)*x[0]+(-0.225585937382)*x[1]+(1.12654556466)*x[2]
        arg[0,2,0,1]=(-0.709656162421)*x[0]+(0.298862010425)*x[1]+(-0.502443284827)*x[2]
        arg[0,2,1,0]=(-1.01166188669)*x[0]+(-1.43763408953)*x[1]+(-0.835160398488)*x[2]
        arg[0,2,1,1]=(1.80968424545)*x[0]+(-0.802666103495)*x[1]+(0.290283606115)*x[2]
        arg[0,2,2,0]=(0.346481981585)*x[0]+(-0.444176962215)*x[1]+(0.689127612227)*x[2]
        arg[0,2,2,1]=(1.06925494095)*x[0]+(0.452854988449)*x[1]+(-1.19974690364)*x[2]
        arg[0,3,0,0]=(-0.534896036328)*x[0]+(-0.312000770615)*x[1]+(-0.440033746693)*x[2]
        arg[0,3,0,1]=(0.580351431838)*x[0]+(-0.36623848451)*x[1]+(-0.507837490037)*x[2]
        arg[0,3,1,0]=(0.383180910522)*x[0]+(-0.0907152443563)*x[1]+(-0.200103743943)*x[2]
        arg[0,3,1,1]=(0.0160194687513)*x[0]+(0.862509612939)*x[1]+(-1.14643594474)*x[2]
        arg[0,3,2,0]=(-0.224390850199)*x[0]+(-0.176927715447)*x[1]+(0.213480525831)*x[2]
        arg[0,3,2,1]=(-0.0508511448148)*x[0]+(-1.28943467389)*x[1]+(0.902285302487)*x[2]
        arg[0,4,0,0]=(-0.633106625232)*x[0]+(-0.430651917485)*x[1]+(0.583149841952)*x[2]
        arg[0,4,0,1]=(0.579035528956)*x[0]+(0.159439682519)*x[1]+(-0.0926277063021)*x[2]
        arg[0,4,1,0]=(0.00484756366973)*x[0]+(0.0324822270755)*x[1]+(1.20052413378)*x[2]
        arg[0,4,1,1]=(-0.337383994688)*x[0]+(0.902076367981)*x[1]+(-0.1956377395)*x[2]
        arg[0,4,2,0]=(0.928434343456)*x[0]+(0.0376569884355)*x[1]+(-0.155274118568)*x[2]
        arg[0,4,2,1]=(-0.541990213238)*x[0]+(-0.0165076697829)*x[1]+(-0.210319224859)*x[2]
        arg[1,0,0,0]=(-1.67762025181)*x[0]+(1.61434755579)*x[1]+(0.357272049999)*x[2]
        arg[1,0,0,1]=(-1.62532544549)*x[0]+(-0.271426231027)*x[1]+(0.338680991778)*x[2]
        arg[1,0,1,0]=(1.73523408033)*x[0]+(1.11987092368)*x[1]+(0.126769364273)*x[2]
        arg[1,0,1,1]=(-1.49220786558)*x[0]+(1.51915566092)*x[1]+(1.24788192157)*x[2]
        arg[1,0,2,0]=(-0.687808766765)*x[0]+(-1.02318754737)*x[1]+(0.590653972416)*x[2]
        arg[1,0,2,1]=(0.756067354802)*x[0]+(-0.66964706965)*x[1]+(-1.18986499158)*x[2]
        arg[1,1,0,0]=(0.371345732177)*x[0]+(0.212739617115)*x[1]+(-1.13439047405)*x[2]
        arg[1,1,0,1]=(0.525761804111)*x[0]+(0.144130399691)*x[1]+(0.715517587677)*x[2]
        arg[1,1,1,0]=(-0.376160293215)*x[0]+(-0.469964935263)*x[1]+(0.413518194707)*x[2]
        arg[1,1,1,1]=(-0.787748398144)*x[0]+(-0.810090402237)*x[1]+(-0.50127109256)*x[2]
        arg[1,1,2,0]=(0.654555083692)*x[0]+(-0.142134700515)*x[1]+(-1.55891930521)*x[2]
        arg[1,1,2,1]=(-0.423058885456)*x[0]+(-0.65930158641)*x[1]+(0.94440057744)*x[2]
        arg[1,2,0,0]=(-0.924832007053)*x[0]+(1.0154890382)*x[1]+(0.11938079748)*x[2]
        arg[1,2,0,1]=(0.134642686931)*x[0]+(-1.30908067998)*x[1]+(0.367206139074)*x[2]
        arg[1,2,1,0]=(1.55524055201)*x[0]+(0.727208660637)*x[1]+(0.809810917377)*x[2]
        arg[1,2,1,1]=(0.94813391462)*x[0]+(-1.49402524308)*x[1]+(0.367824542434)*x[2]
        arg[1,2,2,0]=(-0.608295798731)*x[0]+(-0.142302252814)*x[1]+(-0.986529251308)*x[2]
        arg[1,2,2,1]=(0.0259394619607)*x[0]+(-1.60728202467)*x[1]+(0.312534656619)*x[2]
        arg[1,3,0,0]=(-1.34063373729)*x[0]+(-0.268499839329)*x[1]+(-0.335125886299)*x[2]
        arg[1,3,0,1]=(1.53280698072)*x[0]+(1.24369684711)*x[1]+(0.766369156132)*x[2]
        arg[1,3,1,0]=(0.902067176876)*x[0]+(0.791102994814)*x[1]+(-0.89266430493)*x[2]
        arg[1,3,1,1]=(0.704997629232)*x[0]+(-0.551611360332)*x[1]+(1.39134537491)*x[2]
        arg[1,3,2,0]=(-0.151690675211)*x[0]+(-1.01762975893)*x[1]+(-0.627493214284)*x[2]
        arg[1,3,2,1]=(1.00936645922)*x[0]+(0.0322458997248)*x[1]+(1.45039938194)*x[2]
        arg[1,4,0,0]=(-0.0916390388904)*x[0]+(-0.783111746469)*x[1]+(0.389725342225)*x[2]
        arg[1,4,0,1]=(1.18859328243)*x[0]+(0.871689661558)*x[1]+(-0.775464713448)*x[2]
        arg[1,4,1,0]=(0.913118342488)*x[0]+(1.53285648884)*x[1]+(0.0947365101243)*x[2]
        arg[1,4,1,1]=(-1.00735841224)*x[0]+(-1.12544193607)*x[1]+(0.267347229372)*x[2]
        arg[1,4,2,0]=(0.285943438284)*x[0]+(-1.0889389244)*x[1]+(-1.03054842327)*x[2]
        arg[1,4,2,1]=(-1.5768940139)*x[0]+(0.91237754587)*x[1]+(1.30220394199)*x[2]
        arg[2,0,0,0]=(-0.0634935883277)*x[0]+(0.157304131712)*x[1]+(-1.2995042074)*x[2]
        arg[2,0,0,1]=(-0.802705266871)*x[0]+(0.238233162774)*x[1]+(1.29463414013)*x[2]
        arg[2,0,1,0]=(-0.925468462671)*x[0]+(0.870960795966)*x[1]+(0.156522765713)*x[2]
        arg[2,0,1,1]=(-0.546666833309)*x[0]+(-1.14649607995)*x[1]+(-1.53906505048)*x[2]
        arg[2,0,2,0]=(0.464952910857)*x[0]+(0.408605786027)*x[1]+(-0.886974881114)*x[2]
        arg[2,0,2,1]=(0.491723247948)*x[0]+(0.194616420077)*x[1]+(0.409049782749)*x[2]
        arg[2,1,0,0]=(-1.19836883114)*x[0]+(1.0622722337)*x[1]+(-0.109193867727)*x[2]
        arg[2,1,0,1]=(1.01973600383)*x[0]+(-0.154482220539)*x[1]+(0.378696371956)*x[2]
        arg[2,1,1,0]=(-0.0812015296246)*x[0]+(-1.10252794568)*x[1]+(-0.00585426581857)*x[2]
        arg[2,1,1,1]=(0.536042665272)*x[0]+(0.685507117255)*x[1]+(-0.179652683139)*x[2]
        arg[2,1,2,0]=(-0.36750467811)*x[0]+(-1.23674550869)*x[1]+(-0.762905702801)*x[2]
        arg[2,1,2,1]=(0.329665965651)*x[0]+(0.101195810737)*x[1]+(-0.610015052375)*x[2]
        arg[2,2,0,0]=(-0.81644635185)*x[0]+(-0.604578476306)*x[1]+(0.0440925706991)*x[2]
        arg[2,2,0,1]=(-0.958241437971)*x[0]+(0.38563498206)*x[1]+(0.913406762318)*x[2]
        arg[2,2,1,0]=(1.22027492224)*x[0]+(-1.00726209939)*x[1]+(-1.05083068303)*x[2]
        arg[2,2,1,1]=(-1.47561192645)*x[0]+(-0.903549355608)*x[1]+(0.236771324671)*x[2]
        arg[2,2,2,0]=(0.284995864449)*x[0]+(-0.175760829921)*x[1]+(-0.720607220001)*x[2]
        arg[2,2,2,1]=(-0.535894515963)*x[0]+(-0.821302957162)*x[1]+(-1.75141540241)*x[2]
        arg[2,3,0,0]=(-0.333082106674)*x[0]+(0.579471826412)*x[1]+(0.561184138108)*x[2]
        arg[2,3,0,1]=(1.34803519557)*x[0]+(-0.441299088849)*x[1]+(-1.04945951616)*x[2]
        arg[2,3,1,0]=(0.299011096602)*x[0]+(1.32726458235)*x[1]+(-0.196298687359)*x[2]
        arg[2,3,1,1]=(1.48698834204)*x[0]+(0.527221496971)*x[1]+(0.0819813519422)*x[2]
        arg[2,3,2,0]=(0.681680955153)*x[0]+(0.503653972357)*x[1]+(0.434669347891)*x[2]
        arg[2,3,2,1]=(-0.152586789311)*x[0]+(-0.765242635579)*x[1]+(0.712663994387)*x[2]
        arg[2,4,0,0]=(0.630152300463)*x[0]+(1.12366733588)*x[1]+(-0.662966700638)*x[2]
        arg[2,4,0,1]=(1.02796709755)*x[0]+(-0.24093881584)*x[1]+(1.79445417019)*x[2]
        arg[2,4,1,0]=(-0.954575760758)*x[0]+(0.16331782608)*x[1]+(1.38309921277)*x[2]
        arg[2,4,1,1]=(0.536092514281)*x[0]+(-0.276364952673)*x[1]+(0.779619701528)*x[2]
        arg[2,4,2,0]=(0.267270444141)*x[0]+(-0.810746424331)*x[1]+(-0.733877853264)*x[2]
        arg[2,4,2,1]=(-0.0641688847923)*x[0]+(-0.386279280667)*x[1]+(0.0637571595255)*x[2]
        arg[3,0,0,0]=(0.469903180687)*x[0]+(-0.793812397342)*x[1]+(1.60841921779)*x[2]
        arg[3,0,0,1]=(0.720057600102)*x[0]+(0.502582253261)*x[1]+(0.460147791003)*x[2]
        arg[3,0,1,0]=(0.0327862564137)*x[0]+(-1.142962965)*x[1]+(0.114799131894)*x[2]
        arg[3,0,1,1]=(-0.997412656978)*x[0]+(-0.678175468576)*x[1]+(-0.152663147682)*x[2]
        arg[3,0,2,0]=(0.810424599078)*x[0]+(0.75522727786)*x[1]+(-0.205880941501)*x[2]
        arg[3,0,2,1]=(0.353601949476)*x[0]+(0.0963200813379)*x[1]+(0.0578829287146)*x[2]
        arg[3,1,0,0]=(0.346590730062)*x[0]+(1.07953790092)*x[1]+(0.647586851217)*x[2]
        arg[3,1,0,1]=(0.889879195334)*x[0]+(-0.352788186249)*x[1]+(0.434900135803)*x[2]
        arg[3,1,1,0]=(-0.721742734959)*x[0]+(0.166066672134)*x[1]+(-0.489266924809)*x[2]
        arg[3,1,1,1]=(-0.810487755181)*x[0]+(0.0354916979253)*x[1]+(0.0735520654395)*x[2]
        arg[3,1,2,0]=(1.02414338939)*x[0]+(-0.851076284556)*x[1]+(-0.232174494405)*x[2]
        arg[3,1,2,1]=(-0.327246566026)*x[0]+(-0.0499396023605)*x[1]+(0.611585402379)*x[2]
        arg[3,2,0,0]=(-0.365757628557)*x[0]+(0.868697342115)*x[1]+(-1.102545972)*x[2]
        arg[3,2,0,1]=(0.628592129227)*x[0]+(1.25545515311)*x[1]+(-1.0577476264)*x[2]
        arg[3,2,1,0]=(0.160995978044)*x[0]+(-0.947389499556)*x[1]+(-0.870476160656)*x[2]
        arg[3,2,1,1]=(0.910787139938)*x[0]+(-0.0681493345139)*x[1]+(-0.561910153781)*x[2]
        arg[3,2,2,0]=(0.0761644634428)*x[0]+(-0.700940967343)*x[1]+(1.19407210904)*x[2]
        arg[3,2,2,1]=(1.75717437065)*x[0]+(-0.144457317494)*x[1]+(-0.523233453079)*x[2]
        arg[3,3,0,0]=(0.448782872606)*x[0]+(0.650464256547)*x[1]+(1.19476547456)*x[2]
        arg[3,3,0,1]=(0.118586027201)*x[0]+(0.449971816918)*x[1]+(-0.510812733389)*x[2]
        arg[3,3,1,0]=(-0.330032265081)*x[0]+(-0.13657534891)*x[1]+(0.456365289556)*x[2]
        arg[3,3,1,1]=(0.697584212635)*x[0]+(0.897604241724)*x[1]+(1.31517116663)*x[2]
        arg[3,3,2,0]=(-0.940538707365)*x[0]+(-0.334862221665)*x[1]+(0.0969742096754)*x[2]
        arg[3,3,2,1]=(-0.985788813516)*x[0]+(0.348389725548)*x[1]+(-0.260816590361)*x[2]
        arg[3,4,0,0]=(0.754692421417)*x[0]+(0.775985781769)*x[1]+(-0.0866159222457)*x[2]
        arg[3,4,0,1]=(-1.17670502429)*x[0]+(1.12268279393)*x[1]+(-1.1279822095)*x[2]
        arg[3,4,1,0]=(-1.48674370666)*x[0]+(1.26685445713)*x[1]+(-0.0522828809306)*x[2]
        arg[3,4,1,1]=(0.401760689843)*x[0]+(-1.35707797971)*x[1]+(-0.0580654273828)*x[2]
        arg[3,4,2,0]=(-1.40697830852)*x[0]+(0.933923369958)*x[1]+(0.230152169159)*x[2]
        arg[3,4,2,1]=(-1.21242130555)*x[0]+(0.858316339663)*x[1]+(-0.839747078526)*x[2]
        ref[0,0,0,0]=(-1.29218217256)*x_ref[0]+(-0.382937892213)*x_ref[1]+(0.14363896447)*x_ref[2]
        ref[0,0,0,1]=(1.39856942421)*x_ref[0]+(0.302082267835)*x_ref[1]+(-0.40870736584)*x_ref[2]
        ref[0,0,1,0]=(0.523846108043)*x_ref[0]+(1.19961531204)*x_ref[1]+(0.838881761349)*x_ref[2]
        ref[0,0,1,1]=(0.634542282764)*x_ref[0]+(-0.475514686466)*x_ref[1]+(-1.68745428486)*x_ref[2]
        ref[0,0,2,0]=(1.16379585731)*x_ref[0]+(-0.928037595416)*x_ref[1]+(0.998857178353)*x_ref[2]
        ref[0,0,2,1]=(-0.804515607022)*x_ref[0]+(-0.915469493439)*x_ref[1]+(-0.143888000623)*x_ref[2]
        ref[0,1,0,0]=(0.925843155486)*x_ref[0]+(-0.168243418953)*x_ref[1]+(-0.887839555187)*x_ref[2]
        ref[0,1,0,1]=(-0.913103105329)*x_ref[0]+(1.38011681732)*x_ref[1]+(-0.992478793291)*x_ref[2]
        ref[0,1,1,0]=(0.759685289903)*x_ref[0]+(-0.579481939982)*x_ref[1]+(0.577682929138)*x_ref[2]
        ref[0,1,1,1]=(0.813155548228)*x_ref[0]+(-0.178317843217)*x_ref[1]+(-0.0460113194526)*x_ref[2]
        ref[0,1,2,0]=(-0.195764860141)*x_ref[0]+(1.44256979628)*x_ref[1]+(0.283992441343)*x_ref[2]
        ref[0,1,2,1]=(-0.921229459072)*x_ref[0]+(-1.19295761547)*x_ref[1]+(0.215736801944)*x_ref[2]
        ref[0,2,0,0]=(-0.741726331752)*x_ref[0]+(-0.225585937382)*x_ref[1]+(1.12654556466)*x_ref[2]
        ref[0,2,0,1]=(-0.709656162421)*x_ref[0]+(0.298862010425)*x_ref[1]+(-0.502443284827)*x_ref[2]
        ref[0,2,1,0]=(-1.01166188669)*x_ref[0]+(-1.43763408953)*x_ref[1]+(-0.835160398488)*x_ref[2]
        ref[0,2,1,1]=(1.80968424545)*x_ref[0]+(-0.802666103495)*x_ref[1]+(0.290283606115)*x_ref[2]
        ref[0,2,2,0]=(0.346481981585)*x_ref[0]+(-0.444176962215)*x_ref[1]+(0.689127612227)*x_ref[2]
        ref[0,2,2,1]=(1.06925494095)*x_ref[0]+(0.452854988449)*x_ref[1]+(-1.19974690364)*x_ref[2]
        ref[0,3,0,0]=(-0.534896036328)*x_ref[0]+(-0.312000770615)*x_ref[1]+(-0.440033746693)*x_ref[2]
        ref[0,3,0,1]=(0.580351431838)*x_ref[0]+(-0.36623848451)*x_ref[1]+(-0.507837490037)*x_ref[2]
        ref[0,3,1,0]=(0.383180910522)*x_ref[0]+(-0.0907152443563)*x_ref[1]+(-0.200103743943)*x_ref[2]
        ref[0,3,1,1]=(0.0160194687513)*x_ref[0]+(0.862509612939)*x_ref[1]+(-1.14643594474)*x_ref[2]
        ref[0,3,2,0]=(-0.224390850199)*x_ref[0]+(-0.176927715447)*x_ref[1]+(0.213480525831)*x_ref[2]
        ref[0,3,2,1]=(-0.0508511448148)*x_ref[0]+(-1.28943467389)*x_ref[1]+(0.902285302487)*x_ref[2]
        ref[0,4,0,0]=(-0.633106625232)*x_ref[0]+(-0.430651917485)*x_ref[1]+(0.583149841952)*x_ref[2]
        ref[0,4,0,1]=(0.579035528956)*x_ref[0]+(0.159439682519)*x_ref[1]+(-0.0926277063021)*x_ref[2]
        ref[0,4,1,0]=(0.00484756366973)*x_ref[0]+(0.0324822270755)*x_ref[1]+(1.20052413378)*x_ref[2]
        ref[0,4,1,1]=(-0.337383994688)*x_ref[0]+(0.902076367981)*x_ref[1]+(-0.1956377395)*x_ref[2]
        ref[0,4,2,0]=(0.928434343456)*x_ref[0]+(0.0376569884355)*x_ref[1]+(-0.155274118568)*x_ref[2]
        ref[0,4,2,1]=(-0.541990213238)*x_ref[0]+(-0.0165076697829)*x_ref[1]+(-0.210319224859)*x_ref[2]
        ref[1,0,0,0]=(-1.67762025181)*x_ref[0]+(1.61434755579)*x_ref[1]+(0.357272049999)*x_ref[2]
        ref[1,0,0,1]=(-1.62532544549)*x_ref[0]+(-0.271426231027)*x_ref[1]+(0.338680991778)*x_ref[2]
        ref[1,0,1,0]=(1.73523408033)*x_ref[0]+(1.11987092368)*x_ref[1]+(0.126769364273)*x_ref[2]
        ref[1,0,1,1]=(-1.49220786558)*x_ref[0]+(1.51915566092)*x_ref[1]+(1.24788192157)*x_ref[2]
        ref[1,0,2,0]=(-0.687808766765)*x_ref[0]+(-1.02318754737)*x_ref[1]+(0.590653972416)*x_ref[2]
        ref[1,0,2,1]=(0.756067354802)*x_ref[0]+(-0.66964706965)*x_ref[1]+(-1.18986499158)*x_ref[2]
        ref[1,1,0,0]=(0.371345732177)*x_ref[0]+(0.212739617115)*x_ref[1]+(-1.13439047405)*x_ref[2]
        ref[1,1,0,1]=(0.525761804111)*x_ref[0]+(0.144130399691)*x_ref[1]+(0.715517587677)*x_ref[2]
        ref[1,1,1,0]=(-0.376160293215)*x_ref[0]+(-0.469964935263)*x_ref[1]+(0.413518194707)*x_ref[2]
        ref[1,1,1,1]=(-0.787748398144)*x_ref[0]+(-0.810090402237)*x_ref[1]+(-0.50127109256)*x_ref[2]
        ref[1,1,2,0]=(0.654555083692)*x_ref[0]+(-0.142134700515)*x_ref[1]+(-1.55891930521)*x_ref[2]
        ref[1,1,2,1]=(-0.423058885456)*x_ref[0]+(-0.65930158641)*x_ref[1]+(0.94440057744)*x_ref[2]
        ref[1,2,0,0]=(-0.924832007053)*x_ref[0]+(1.0154890382)*x_ref[1]+(0.11938079748)*x_ref[2]
        ref[1,2,0,1]=(0.134642686931)*x_ref[0]+(-1.30908067998)*x_ref[1]+(0.367206139074)*x_ref[2]
        ref[1,2,1,0]=(1.55524055201)*x_ref[0]+(0.727208660637)*x_ref[1]+(0.809810917377)*x_ref[2]
        ref[1,2,1,1]=(0.94813391462)*x_ref[0]+(-1.49402524308)*x_ref[1]+(0.367824542434)*x_ref[2]
        ref[1,2,2,0]=(-0.608295798731)*x_ref[0]+(-0.142302252814)*x_ref[1]+(-0.986529251308)*x_ref[2]
        ref[1,2,2,1]=(0.0259394619607)*x_ref[0]+(-1.60728202467)*x_ref[1]+(0.312534656619)*x_ref[2]
        ref[1,3,0,0]=(-1.34063373729)*x_ref[0]+(-0.268499839329)*x_ref[1]+(-0.335125886299)*x_ref[2]
        ref[1,3,0,1]=(1.53280698072)*x_ref[0]+(1.24369684711)*x_ref[1]+(0.766369156132)*x_ref[2]
        ref[1,3,1,0]=(0.902067176876)*x_ref[0]+(0.791102994814)*x_ref[1]+(-0.89266430493)*x_ref[2]
        ref[1,3,1,1]=(0.704997629232)*x_ref[0]+(-0.551611360332)*x_ref[1]+(1.39134537491)*x_ref[2]
        ref[1,3,2,0]=(-0.151690675211)*x_ref[0]+(-1.01762975893)*x_ref[1]+(-0.627493214284)*x_ref[2]
        ref[1,3,2,1]=(1.00936645922)*x_ref[0]+(0.0322458997248)*x_ref[1]+(1.45039938194)*x_ref[2]
        ref[1,4,0,0]=(-0.0916390388904)*x_ref[0]+(-0.783111746469)*x_ref[1]+(0.389725342225)*x_ref[2]
        ref[1,4,0,1]=(1.18859328243)*x_ref[0]+(0.871689661558)*x_ref[1]+(-0.775464713448)*x_ref[2]
        ref[1,4,1,0]=(0.913118342488)*x_ref[0]+(1.53285648884)*x_ref[1]+(0.0947365101243)*x_ref[2]
        ref[1,4,1,1]=(-1.00735841224)*x_ref[0]+(-1.12544193607)*x_ref[1]+(0.267347229372)*x_ref[2]
        ref[1,4,2,0]=(0.285943438284)*x_ref[0]+(-1.0889389244)*x_ref[1]+(-1.03054842327)*x_ref[2]
        ref[1,4,2,1]=(-1.5768940139)*x_ref[0]+(0.91237754587)*x_ref[1]+(1.30220394199)*x_ref[2]
        ref[2,0,0,0]=(-0.0634935883277)*x_ref[0]+(0.157304131712)*x_ref[1]+(-1.2995042074)*x_ref[2]
        ref[2,0,0,1]=(-0.802705266871)*x_ref[0]+(0.238233162774)*x_ref[1]+(1.29463414013)*x_ref[2]
        ref[2,0,1,0]=(-0.925468462671)*x_ref[0]+(0.870960795966)*x_ref[1]+(0.156522765713)*x_ref[2]
        ref[2,0,1,1]=(-0.546666833309)*x_ref[0]+(-1.14649607995)*x_ref[1]+(-1.53906505048)*x_ref[2]
        ref[2,0,2,0]=(0.464952910857)*x_ref[0]+(0.408605786027)*x_ref[1]+(-0.886974881114)*x_ref[2]
        ref[2,0,2,1]=(0.491723247948)*x_ref[0]+(0.194616420077)*x_ref[1]+(0.409049782749)*x_ref[2]
        ref[2,1,0,0]=(-1.19836883114)*x_ref[0]+(1.0622722337)*x_ref[1]+(-0.109193867727)*x_ref[2]
        ref[2,1,0,1]=(1.01973600383)*x_ref[0]+(-0.154482220539)*x_ref[1]+(0.378696371956)*x_ref[2]
        ref[2,1,1,0]=(-0.0812015296246)*x_ref[0]+(-1.10252794568)*x_ref[1]+(-0.00585426581857)*x_ref[2]
        ref[2,1,1,1]=(0.536042665272)*x_ref[0]+(0.685507117255)*x_ref[1]+(-0.179652683139)*x_ref[2]
        ref[2,1,2,0]=(-0.36750467811)*x_ref[0]+(-1.23674550869)*x_ref[1]+(-0.762905702801)*x_ref[2]
        ref[2,1,2,1]=(0.329665965651)*x_ref[0]+(0.101195810737)*x_ref[1]+(-0.610015052375)*x_ref[2]
        ref[2,2,0,0]=(-0.81644635185)*x_ref[0]+(-0.604578476306)*x_ref[1]+(0.0440925706991)*x_ref[2]
        ref[2,2,0,1]=(-0.958241437971)*x_ref[0]+(0.38563498206)*x_ref[1]+(0.913406762318)*x_ref[2]
        ref[2,2,1,0]=(1.22027492224)*x_ref[0]+(-1.00726209939)*x_ref[1]+(-1.05083068303)*x_ref[2]
        ref[2,2,1,1]=(-1.47561192645)*x_ref[0]+(-0.903549355608)*x_ref[1]+(0.236771324671)*x_ref[2]
        ref[2,2,2,0]=(0.284995864449)*x_ref[0]+(-0.175760829921)*x_ref[1]+(-0.720607220001)*x_ref[2]
        ref[2,2,2,1]=(-0.535894515963)*x_ref[0]+(-0.821302957162)*x_ref[1]+(-1.75141540241)*x_ref[2]
        ref[2,3,0,0]=(-0.333082106674)*x_ref[0]+(0.579471826412)*x_ref[1]+(0.561184138108)*x_ref[2]
        ref[2,3,0,1]=(1.34803519557)*x_ref[0]+(-0.441299088849)*x_ref[1]+(-1.04945951616)*x_ref[2]
        ref[2,3,1,0]=(0.299011096602)*x_ref[0]+(1.32726458235)*x_ref[1]+(-0.196298687359)*x_ref[2]
        ref[2,3,1,1]=(1.48698834204)*x_ref[0]+(0.527221496971)*x_ref[1]+(0.0819813519422)*x_ref[2]
        ref[2,3,2,0]=(0.681680955153)*x_ref[0]+(0.503653972357)*x_ref[1]+(0.434669347891)*x_ref[2]
        ref[2,3,2,1]=(-0.152586789311)*x_ref[0]+(-0.765242635579)*x_ref[1]+(0.712663994387)*x_ref[2]
        ref[2,4,0,0]=(0.630152300463)*x_ref[0]+(1.12366733588)*x_ref[1]+(-0.662966700638)*x_ref[2]
        ref[2,4,0,1]=(1.02796709755)*x_ref[0]+(-0.24093881584)*x_ref[1]+(1.79445417019)*x_ref[2]
        ref[2,4,1,0]=(-0.954575760758)*x_ref[0]+(0.16331782608)*x_ref[1]+(1.38309921277)*x_ref[2]
        ref[2,4,1,1]=(0.536092514281)*x_ref[0]+(-0.276364952673)*x_ref[1]+(0.779619701528)*x_ref[2]
        ref[2,4,2,0]=(0.267270444141)*x_ref[0]+(-0.810746424331)*x_ref[1]+(-0.733877853264)*x_ref[2]
        ref[2,4,2,1]=(-0.0641688847923)*x_ref[0]+(-0.386279280667)*x_ref[1]+(0.0637571595255)*x_ref[2]
        ref[3,0,0,0]=(0.469903180687)*x_ref[0]+(-0.793812397342)*x_ref[1]+(1.60841921779)*x_ref[2]
        ref[3,0,0,1]=(0.720057600102)*x_ref[0]+(0.502582253261)*x_ref[1]+(0.460147791003)*x_ref[2]
        ref[3,0,1,0]=(0.0327862564137)*x_ref[0]+(-1.142962965)*x_ref[1]+(0.114799131894)*x_ref[2]
        ref[3,0,1,1]=(-0.997412656978)*x_ref[0]+(-0.678175468576)*x_ref[1]+(-0.152663147682)*x_ref[2]
        ref[3,0,2,0]=(0.810424599078)*x_ref[0]+(0.75522727786)*x_ref[1]+(-0.205880941501)*x_ref[2]
        ref[3,0,2,1]=(0.353601949476)*x_ref[0]+(0.0963200813379)*x_ref[1]+(0.0578829287146)*x_ref[2]
        ref[3,1,0,0]=(0.346590730062)*x_ref[0]+(1.07953790092)*x_ref[1]+(0.647586851217)*x_ref[2]
        ref[3,1,0,1]=(0.889879195334)*x_ref[0]+(-0.352788186249)*x_ref[1]+(0.434900135803)*x_ref[2]
        ref[3,1,1,0]=(-0.721742734959)*x_ref[0]+(0.166066672134)*x_ref[1]+(-0.489266924809)*x_ref[2]
        ref[3,1,1,1]=(-0.810487755181)*x_ref[0]+(0.0354916979253)*x_ref[1]+(0.0735520654395)*x_ref[2]
        ref[3,1,2,0]=(1.02414338939)*x_ref[0]+(-0.851076284556)*x_ref[1]+(-0.232174494405)*x_ref[2]
        ref[3,1,2,1]=(-0.327246566026)*x_ref[0]+(-0.0499396023605)*x_ref[1]+(0.611585402379)*x_ref[2]
        ref[3,2,0,0]=(-0.365757628557)*x_ref[0]+(0.868697342115)*x_ref[1]+(-1.102545972)*x_ref[2]
        ref[3,2,0,1]=(0.628592129227)*x_ref[0]+(1.25545515311)*x_ref[1]+(-1.0577476264)*x_ref[2]
        ref[3,2,1,0]=(0.160995978044)*x_ref[0]+(-0.947389499556)*x_ref[1]+(-0.870476160656)*x_ref[2]
        ref[3,2,1,1]=(0.910787139938)*x_ref[0]+(-0.0681493345139)*x_ref[1]+(-0.561910153781)*x_ref[2]
        ref[3,2,2,0]=(0.0761644634428)*x_ref[0]+(-0.700940967343)*x_ref[1]+(1.19407210904)*x_ref[2]
        ref[3,2,2,1]=(1.75717437065)*x_ref[0]+(-0.144457317494)*x_ref[1]+(-0.523233453079)*x_ref[2]
        ref[3,3,0,0]=(0.448782872606)*x_ref[0]+(0.650464256547)*x_ref[1]+(1.19476547456)*x_ref[2]
        ref[3,3,0,1]=(0.118586027201)*x_ref[0]+(0.449971816918)*x_ref[1]+(-0.510812733389)*x_ref[2]
        ref[3,3,1,0]=(-0.330032265081)*x_ref[0]+(-0.13657534891)*x_ref[1]+(0.456365289556)*x_ref[2]
        ref[3,3,1,1]=(0.697584212635)*x_ref[0]+(0.897604241724)*x_ref[1]+(1.31517116663)*x_ref[2]
        ref[3,3,2,0]=(-0.940538707365)*x_ref[0]+(-0.334862221665)*x_ref[1]+(0.0969742096754)*x_ref[2]
        ref[3,3,2,1]=(-0.985788813516)*x_ref[0]+(0.348389725548)*x_ref[1]+(-0.260816590361)*x_ref[2]
        ref[3,4,0,0]=(0.754692421417)*x_ref[0]+(0.775985781769)*x_ref[1]+(-0.0866159222457)*x_ref[2]
        ref[3,4,0,1]=(-1.17670502429)*x_ref[0]+(1.12268279393)*x_ref[1]+(-1.1279822095)*x_ref[2]
        ref[3,4,1,0]=(-1.48674370666)*x_ref[0]+(1.26685445713)*x_ref[1]+(-0.0522828809306)*x_ref[2]
        ref[3,4,1,1]=(0.401760689843)*x_ref[0]+(-1.35707797971)*x_ref[1]+(-0.0580654273828)*x_ref[2]
        ref[3,4,2,0]=(-1.40697830852)*x_ref[0]+(0.933923369958)*x_ref[1]+(0.230152169159)*x_ref[2]
        ref[3,4,2,1]=(-1.21242130555)*x_ref[0]+(0.858316339663)*x_ref[1]+(-0.839747078526)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromData_ReducedFunctionOnContactZero_rank0(self):
      """
      tests interpolate for rank 0 Data onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactZero onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactZero(self.domain)
      x=w.getX()
      arg=Data(0,(),w)
      ref=Data(0,(),w_ref)
      if dim==2:
        arg=(-0.485106931655)*x[0]**o+(-0.45911691249)*x[0]+(-0.780654139819)*x[1]**o+(-0.496275334303)*x[1]
        ref=(-0.485106931655)*x_ref[0]**o+(-0.45911691249)*x_ref[0]+(-0.780654139819)*x_ref[1]**o+(-0.496275334303)*x_ref[1]
      else:
        arg=(-0.312233545328)*x[0]**o+(0.386652391857)*x[0]+(0.893938213685)*x[1]**o+(-0.109836308832)*x[1]+(0.293419600866)*x[2]**o+(-0.802363407879)*x[2]
        ref=(-0.312233545328)*x_ref[0]**o+(0.386652391857)*x_ref[0]+(0.893938213685)*x_ref[1]**o+(-0.109836308832)*x_ref[1]+(0.293419600866)*x_ref[2]**o+(-0.802363407879)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromData_ReducedFunctionOnContactZero_rank1(self):
      """
      tests interpolate for rank 1 Data onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactZero onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactZero(self.domain)
      x=w.getX()
      arg=Data(0,(2,),w)
      ref=Data(0,(2,),w_ref)
      if dim==2:
        arg[0]=(0.383147346126)*x[0]**o+(-0.103098043084)*x[0]+(0.389581571033)*x[1]**o+(0.647441523744)*x[1]
        arg[1]=(-0.0718078186662)*x[0]**o+(-0.975209175853)*x[0]+(-0.62913306056)*x[1]**o+(0.648038487393)*x[1]
        ref[0]=(0.383147346126)*x_ref[0]**o+(-0.103098043084)*x_ref[0]+(0.389581571033)*x_ref[1]**o+(0.647441523744)*x_ref[1]
        ref[1]=(-0.0718078186662)*x_ref[0]**o+(-0.975209175853)*x_ref[0]+(-0.62913306056)*x_ref[1]**o+(0.648038487393)*x_ref[1]
      else:
        arg[0]=(-0.392958184101)*x[0]**o+(-0.529709692826)*x[0]+(-0.834422900831)*x[1]**o+(0.52562214773)*x[1]+(-0.726762801989)*x[2]**o+(-0.329225625652)*x[2]
        arg[1]=(-0.388486489727)*x[0]**o+(0.62353995706)*x[0]+(0.0198506797091)*x[1]**o+(0.744336375443)*x[1]+(0.115966573066)*x[2]**o+(-0.195838697299)*x[2]
        ref[0]=(-0.392958184101)*x_ref[0]**o+(-0.529709692826)*x_ref[0]+(-0.834422900831)*x_ref[1]**o+(0.52562214773)*x_ref[1]+(-0.726762801989)*x_ref[2]**o+(-0.329225625652)*x_ref[2]
        ref[1]=(-0.388486489727)*x_ref[0]**o+(0.62353995706)*x_ref[0]+(0.0198506797091)*x_ref[1]**o+(0.744336375443)*x_ref[1]+(0.115966573066)*x_ref[2]**o+(-0.195838697299)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromData_ReducedFunctionOnContactZero_rank2(self):
      """
      tests interpolate for rank 2 Data onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactZero onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactZero(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5),w)
      ref=Data(0,(4, 5),w_ref)
      if dim==2:
        arg[0,0]=(-0.657915437562)*x[0]**o+(-0.545113889022)*x[0]+(0.531900702747)*x[1]**o+(0.593791195717)*x[1]
        arg[0,1]=(0.051080053648)*x[0]**o+(-0.693135614033)*x[0]+(0.132124625293)*x[1]**o+(0.790120778986)*x[1]
        arg[0,2]=(-0.474326324304)*x[0]**o+(0.844119715746)*x[0]+(-0.107139623866)*x[1]**o+(0.456623369855)*x[1]
        arg[0,3]=(-0.582421376463)*x[0]**o+(0.972810074845)*x[0]+(0.875278994526)*x[1]**o+(-0.382656553829)*x[1]
        arg[0,4]=(-0.904956623057)*x[0]**o+(0.701968065151)*x[0]+(-0.710231782855)*x[1]**o+(-0.123456684979)*x[1]
        arg[1,0]=(0.361017539866)*x[0]**o+(0.865249575899)*x[0]+(0.529515655309)*x[1]**o+(0.161402780958)*x[1]
        arg[1,1]=(0.813945406612)*x[0]**o+(0.801153720706)*x[0]+(-0.254227034266)*x[1]**o+(-0.422848409284)*x[1]
        arg[1,2]=(0.56207473833)*x[0]**o+(-0.831638855484)*x[0]+(-0.584619648965)*x[1]**o+(0.453944553892)*x[1]
        arg[1,3]=(0.25526821881)*x[0]**o+(-0.436135120045)*x[0]+(0.524842439137)*x[1]**o+(0.568007117728)*x[1]
        arg[1,4]=(-0.0587551077421)*x[0]**o+(-0.434635762185)*x[0]+(-0.592094375986)*x[1]**o+(0.873123434395)*x[1]
        arg[2,0]=(0.266819439236)*x[0]**o+(-0.45156140915)*x[0]+(0.436756372775)*x[1]**o+(0.12295128383)*x[1]
        arg[2,1]=(0.347128677451)*x[0]**o+(-0.0983169467916)*x[0]+(-0.414573209624)*x[1]**o+(-0.171300108487)*x[1]
        arg[2,2]=(-0.279695154296)*x[0]**o+(-0.911450144779)*x[0]+(0.5055340448)*x[1]**o+(-0.858058864152)*x[1]
        arg[2,3]=(-0.53768906863)*x[0]**o+(-0.0626923004303)*x[0]+(0.72295545269)*x[1]**o+(-0.77070425804)*x[1]
        arg[2,4]=(0.18215032008)*x[0]**o+(-0.965014882526)*x[0]+(0.596564813119)*x[1]**o+(0.0817024840554)*x[1]
        arg[3,0]=(-0.572404596368)*x[0]**o+(0.488365635829)*x[0]+(0.145439800826)*x[1]**o+(-0.337589667899)*x[1]
        arg[3,1]=(-0.628977426654)*x[0]**o+(0.22731945375)*x[0]+(0.989224773975)*x[1]**o+(0.0691553279499)*x[1]
        arg[3,2]=(-0.43609079916)*x[0]**o+(0.966113801416)*x[0]+(-0.853136416278)*x[1]**o+(-0.0125151911906)*x[1]
        arg[3,3]=(0.340159347855)*x[0]**o+(-0.254775013843)*x[0]+(0.795261224144)*x[1]**o+(-0.692255020659)*x[1]
        arg[3,4]=(-0.153521724504)*x[0]**o+(-0.681779299881)*x[0]+(0.693860779041)*x[1]**o+(0.882012604794)*x[1]
        ref[0,0]=(-0.657915437562)*x_ref[0]**o+(-0.545113889022)*x_ref[0]+(0.531900702747)*x_ref[1]**o+(0.593791195717)*x_ref[1]
        ref[0,1]=(0.051080053648)*x_ref[0]**o+(-0.693135614033)*x_ref[0]+(0.132124625293)*x_ref[1]**o+(0.790120778986)*x_ref[1]
        ref[0,2]=(-0.474326324304)*x_ref[0]**o+(0.844119715746)*x_ref[0]+(-0.107139623866)*x_ref[1]**o+(0.456623369855)*x_ref[1]
        ref[0,3]=(-0.582421376463)*x_ref[0]**o+(0.972810074845)*x_ref[0]+(0.875278994526)*x_ref[1]**o+(-0.382656553829)*x_ref[1]
        ref[0,4]=(-0.904956623057)*x_ref[0]**o+(0.701968065151)*x_ref[0]+(-0.710231782855)*x_ref[1]**o+(-0.123456684979)*x_ref[1]
        ref[1,0]=(0.361017539866)*x_ref[0]**o+(0.865249575899)*x_ref[0]+(0.529515655309)*x_ref[1]**o+(0.161402780958)*x_ref[1]
        ref[1,1]=(0.813945406612)*x_ref[0]**o+(0.801153720706)*x_ref[0]+(-0.254227034266)*x_ref[1]**o+(-0.422848409284)*x_ref[1]
        ref[1,2]=(0.56207473833)*x_ref[0]**o+(-0.831638855484)*x_ref[0]+(-0.584619648965)*x_ref[1]**o+(0.453944553892)*x_ref[1]
        ref[1,3]=(0.25526821881)*x_ref[0]**o+(-0.436135120045)*x_ref[0]+(0.524842439137)*x_ref[1]**o+(0.568007117728)*x_ref[1]
        ref[1,4]=(-0.0587551077421)*x_ref[0]**o+(-0.434635762185)*x_ref[0]+(-0.592094375986)*x_ref[1]**o+(0.873123434395)*x_ref[1]
        ref[2,0]=(0.266819439236)*x_ref[0]**o+(-0.45156140915)*x_ref[0]+(0.436756372775)*x_ref[1]**o+(0.12295128383)*x_ref[1]
        ref[2,1]=(0.347128677451)*x_ref[0]**o+(-0.0983169467916)*x_ref[0]+(-0.414573209624)*x_ref[1]**o+(-0.171300108487)*x_ref[1]
        ref[2,2]=(-0.279695154296)*x_ref[0]**o+(-0.911450144779)*x_ref[0]+(0.5055340448)*x_ref[1]**o+(-0.858058864152)*x_ref[1]
        ref[2,3]=(-0.53768906863)*x_ref[0]**o+(-0.0626923004303)*x_ref[0]+(0.72295545269)*x_ref[1]**o+(-0.77070425804)*x_ref[1]
        ref[2,4]=(0.18215032008)*x_ref[0]**o+(-0.965014882526)*x_ref[0]+(0.596564813119)*x_ref[1]**o+(0.0817024840554)*x_ref[1]
        ref[3,0]=(-0.572404596368)*x_ref[0]**o+(0.488365635829)*x_ref[0]+(0.145439800826)*x_ref[1]**o+(-0.337589667899)*x_ref[1]
        ref[3,1]=(-0.628977426654)*x_ref[0]**o+(0.22731945375)*x_ref[0]+(0.989224773975)*x_ref[1]**o+(0.0691553279499)*x_ref[1]
        ref[3,2]=(-0.43609079916)*x_ref[0]**o+(0.966113801416)*x_ref[0]+(-0.853136416278)*x_ref[1]**o+(-0.0125151911906)*x_ref[1]
        ref[3,3]=(0.340159347855)*x_ref[0]**o+(-0.254775013843)*x_ref[0]+(0.795261224144)*x_ref[1]**o+(-0.692255020659)*x_ref[1]
        ref[3,4]=(-0.153521724504)*x_ref[0]**o+(-0.681779299881)*x_ref[0]+(0.693860779041)*x_ref[1]**o+(0.882012604794)*x_ref[1]
      else:
        arg[0,0]=(-0.0201751669244)*x[0]**o+(0.337967826628)*x[0]+(0.262482967798)*x[1]**o+(-0.192863458488)*x[1]+(-0.696822606165)*x[2]**o+(-0.476556518002)*x[2]
        arg[0,1]=(-0.546277895941)*x[0]**o+(-0.118793125394)*x[0]+(0.987786322041)*x[1]**o+(-0.241472898602)*x[1]+(-0.454308453322)*x[2]**o+(-0.27057260464)*x[2]
        arg[0,2]=(0.383631846628)*x[0]**o+(0.979909217026)*x[0]+(-0.0503694684178)*x[1]**o+(-0.748153155067)*x[1]+(-0.721832849635)*x[2]**o+(0.144937710994)*x[2]
        arg[0,3]=(-0.540144273345)*x[0]**o+(-0.986853732479)*x[0]+(-0.14527387031)*x[1]**o+(0.329815126012)*x[1]+(0.566431907689)*x[2]**o+(-0.323179211167)*x[2]
        arg[0,4]=(-0.448624394254)*x[0]**o+(-0.272742080619)*x[0]+(-0.451405020267)*x[1]**o+(0.173204890481)*x[1]+(-0.0122463571841)*x[2]**o+(-0.595932249794)*x[2]
        arg[1,0]=(0.984160677776)*x[0]**o+(-0.243512460216)*x[0]+(0.82590496761)*x[1]**o+(-0.448270326787)*x[1]+(0.286008153265)*x[2]**o+(0.804229889769)*x[2]
        arg[1,1]=(0.531762548093)*x[0]**o+(0.211455783615)*x[0]+(0.836058347792)*x[1]**o+(0.806904570574)*x[1]+(-0.052512287992)*x[2]**o+(0.998679219355)*x[2]
        arg[1,2]=(0.462615968956)*x[0]**o+(0.960862689218)*x[0]+(-0.693932866844)*x[1]**o+(0.881110014782)*x[1]+(0.976329091354)*x[2]**o+(-0.647483316997)*x[2]
        arg[1,3]=(-0.764181748932)*x[0]**o+(-0.962197871121)*x[0]+(-0.143747029438)*x[1]**o+(0.913962833322)*x[1]+(0.165013597835)*x[2]**o+(0.166877976184)*x[2]
        arg[1,4]=(0.687688905493)*x[0]**o+(-0.152816071208)*x[0]+(-0.277974702822)*x[1]**o+(-0.389918839086)*x[1]+(0.229658530129)*x[2]**o+(-0.69924627348)*x[2]
        arg[2,0]=(0.0598215047949)*x[0]**o+(0.592389000293)*x[0]+(0.00886573296364)*x[1]**o+(0.339086619934)*x[1]+(0.476600446349)*x[2]**o+(0.767524427684)*x[2]
        arg[2,1]=(0.525936212445)*x[0]**o+(0.44607860423)*x[0]+(0.0397184541609)*x[1]**o+(-0.424118244789)*x[1]+(-0.273077104391)*x[2]**o+(0.435027185784)*x[2]
        arg[2,2]=(0.672992314995)*x[0]**o+(0.993339100191)*x[0]+(0.488983819898)*x[1]**o+(0.568579529177)*x[1]+(-0.321282052211)*x[2]**o+(-0.268847753649)*x[2]
        arg[2,3]=(-0.0536748741467)*x[0]**o+(-0.195658514433)*x[0]+(-0.931652264383)*x[1]**o+(-0.89957282695)*x[1]+(-0.811088135231)*x[2]**o+(0.0699177832759)*x[2]
        arg[2,4]=(-0.707875141752)*x[0]**o+(-0.00297935832343)*x[0]+(-0.959580991547)*x[1]**o+(0.0787110832272)*x[1]+(-0.754646316678)*x[2]**o+(-0.432731727467)*x[2]
        arg[3,0]=(0.728853463111)*x[0]**o+(-0.681375546727)*x[0]+(-0.283722278591)*x[1]**o+(0.950522890527)*x[1]+(0.608194817491)*x[2]**o+(0.571156129477)*x[2]
        arg[3,1]=(-0.0208969501228)*x[0]**o+(0.572596028151)*x[0]+(0.811179009214)*x[1]**o+(0.519098352581)*x[1]+(0.154527510954)*x[2]**o+(-0.21413742337)*x[2]
        arg[3,2]=(0.347209642455)*x[0]**o+(-0.114911324326)*x[0]+(-0.104561779357)*x[1]**o+(0.848703366841)*x[1]+(0.680312939858)*x[2]**o+(-0.680566404979)*x[2]
        arg[3,3]=(0.342687985669)*x[0]**o+(0.443470953221)*x[0]+(-0.849668173915)*x[1]**o+(0.337689225078)*x[1]+(-0.35083505383)*x[2]**o+(-0.284627678739)*x[2]
        arg[3,4]=(0.661690111131)*x[0]**o+(-0.0918926979359)*x[0]+(-0.587753600932)*x[1]**o+(-0.383115718043)*x[1]+(-0.0696588695623)*x[2]**o+(-0.799630628972)*x[2]
        ref[0,0]=(-0.0201751669244)*x_ref[0]**o+(0.337967826628)*x_ref[0]+(0.262482967798)*x_ref[1]**o+(-0.192863458488)*x_ref[1]+(-0.696822606165)*x_ref[2]**o+(-0.476556518002)*x_ref[2]
        ref[0,1]=(-0.546277895941)*x_ref[0]**o+(-0.118793125394)*x_ref[0]+(0.987786322041)*x_ref[1]**o+(-0.241472898602)*x_ref[1]+(-0.454308453322)*x_ref[2]**o+(-0.27057260464)*x_ref[2]
        ref[0,2]=(0.383631846628)*x_ref[0]**o+(0.979909217026)*x_ref[0]+(-0.0503694684178)*x_ref[1]**o+(-0.748153155067)*x_ref[1]+(-0.721832849635)*x_ref[2]**o+(0.144937710994)*x_ref[2]
        ref[0,3]=(-0.540144273345)*x_ref[0]**o+(-0.986853732479)*x_ref[0]+(-0.14527387031)*x_ref[1]**o+(0.329815126012)*x_ref[1]+(0.566431907689)*x_ref[2]**o+(-0.323179211167)*x_ref[2]
        ref[0,4]=(-0.448624394254)*x_ref[0]**o+(-0.272742080619)*x_ref[0]+(-0.451405020267)*x_ref[1]**o+(0.173204890481)*x_ref[1]+(-0.0122463571841)*x_ref[2]**o+(-0.595932249794)*x_ref[2]
        ref[1,0]=(0.984160677776)*x_ref[0]**o+(-0.243512460216)*x_ref[0]+(0.82590496761)*x_ref[1]**o+(-0.448270326787)*x_ref[1]+(0.286008153265)*x_ref[2]**o+(0.804229889769)*x_ref[2]
        ref[1,1]=(0.531762548093)*x_ref[0]**o+(0.211455783615)*x_ref[0]+(0.836058347792)*x_ref[1]**o+(0.806904570574)*x_ref[1]+(-0.052512287992)*x_ref[2]**o+(0.998679219355)*x_ref[2]
        ref[1,2]=(0.462615968956)*x_ref[0]**o+(0.960862689218)*x_ref[0]+(-0.693932866844)*x_ref[1]**o+(0.881110014782)*x_ref[1]+(0.976329091354)*x_ref[2]**o+(-0.647483316997)*x_ref[2]
        ref[1,3]=(-0.764181748932)*x_ref[0]**o+(-0.962197871121)*x_ref[0]+(-0.143747029438)*x_ref[1]**o+(0.913962833322)*x_ref[1]+(0.165013597835)*x_ref[2]**o+(0.166877976184)*x_ref[2]
        ref[1,4]=(0.687688905493)*x_ref[0]**o+(-0.152816071208)*x_ref[0]+(-0.277974702822)*x_ref[1]**o+(-0.389918839086)*x_ref[1]+(0.229658530129)*x_ref[2]**o+(-0.69924627348)*x_ref[2]
        ref[2,0]=(0.0598215047949)*x_ref[0]**o+(0.592389000293)*x_ref[0]+(0.00886573296364)*x_ref[1]**o+(0.339086619934)*x_ref[1]+(0.476600446349)*x_ref[2]**o+(0.767524427684)*x_ref[2]
        ref[2,1]=(0.525936212445)*x_ref[0]**o+(0.44607860423)*x_ref[0]+(0.0397184541609)*x_ref[1]**o+(-0.424118244789)*x_ref[1]+(-0.273077104391)*x_ref[2]**o+(0.435027185784)*x_ref[2]
        ref[2,2]=(0.672992314995)*x_ref[0]**o+(0.993339100191)*x_ref[0]+(0.488983819898)*x_ref[1]**o+(0.568579529177)*x_ref[1]+(-0.321282052211)*x_ref[2]**o+(-0.268847753649)*x_ref[2]
        ref[2,3]=(-0.0536748741467)*x_ref[0]**o+(-0.195658514433)*x_ref[0]+(-0.931652264383)*x_ref[1]**o+(-0.89957282695)*x_ref[1]+(-0.811088135231)*x_ref[2]**o+(0.0699177832759)*x_ref[2]
        ref[2,4]=(-0.707875141752)*x_ref[0]**o+(-0.00297935832343)*x_ref[0]+(-0.959580991547)*x_ref[1]**o+(0.0787110832272)*x_ref[1]+(-0.754646316678)*x_ref[2]**o+(-0.432731727467)*x_ref[2]
        ref[3,0]=(0.728853463111)*x_ref[0]**o+(-0.681375546727)*x_ref[0]+(-0.283722278591)*x_ref[1]**o+(0.950522890527)*x_ref[1]+(0.608194817491)*x_ref[2]**o+(0.571156129477)*x_ref[2]
        ref[3,1]=(-0.0208969501228)*x_ref[0]**o+(0.572596028151)*x_ref[0]+(0.811179009214)*x_ref[1]**o+(0.519098352581)*x_ref[1]+(0.154527510954)*x_ref[2]**o+(-0.21413742337)*x_ref[2]
        ref[3,2]=(0.347209642455)*x_ref[0]**o+(-0.114911324326)*x_ref[0]+(-0.104561779357)*x_ref[1]**o+(0.848703366841)*x_ref[1]+(0.680312939858)*x_ref[2]**o+(-0.680566404979)*x_ref[2]
        ref[3,3]=(0.342687985669)*x_ref[0]**o+(0.443470953221)*x_ref[0]+(-0.849668173915)*x_ref[1]**o+(0.337689225078)*x_ref[1]+(-0.35083505383)*x_ref[2]**o+(-0.284627678739)*x_ref[2]
        ref[3,4]=(0.661690111131)*x_ref[0]**o+(-0.0918926979359)*x_ref[0]+(-0.587753600932)*x_ref[1]**o+(-0.383115718043)*x_ref[1]+(-0.0696588695623)*x_ref[2]**o+(-0.799630628972)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromData_ReducedFunctionOnContactZero_rank3(self):
      """
      tests interpolate for rank 3 Data onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactZero onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactZero(self.domain)
      x=w.getX()
      arg=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2),w_ref)
      if dim==2:
        arg[0,0,0]=(-0.906705618142)*x[0]**o+(0.458982439103)*x[0]+(0.556464353903)*x[1]**o+(0.733236686234)*x[1]
        arg[0,0,1]=(-0.192889954523)*x[0]**o+(-0.814332539133)*x[0]+(0.446164214282)*x[1]**o+(0.032291315446)*x[1]
        arg[0,1,0]=(0.757081929199)*x[0]**o+(-0.3938524931)*x[0]+(-0.458060132115)*x[1]**o+(0.471831491182)*x[1]
        arg[0,1,1]=(0.459868785407)*x[0]**o+(0.513707183316)*x[0]+(-0.9682413377)*x[1]**o+(-0.49496265187)*x[1]
        arg[1,0,0]=(0.546655212697)*x[0]**o+(-0.296125788768)*x[0]+(0.570033305927)*x[1]**o+(-0.818919094317)*x[1]
        arg[1,0,1]=(-0.954422252892)*x[0]**o+(-0.925035286684)*x[0]+(0.171311127546)*x[1]**o+(-0.584606597796)*x[1]
        arg[1,1,0]=(-0.203956024883)*x[0]**o+(-0.933117428128)*x[0]+(-0.745101180472)*x[1]**o+(0.0390208509919)*x[1]
        arg[1,1,1]=(-0.67467593628)*x[0]**o+(0.42281003355)*x[0]+(-0.495401176467)*x[1]**o+(0.993257306842)*x[1]
        arg[2,0,0]=(-0.204011890956)*x[0]**o+(0.730452092671)*x[0]+(-0.0486718200506)*x[1]**o+(0.691928726795)*x[1]
        arg[2,0,1]=(0.381571380631)*x[0]**o+(0.504896621541)*x[0]+(-0.229805983176)*x[1]**o+(0.281401365705)*x[1]
        arg[2,1,0]=(0.578852751326)*x[0]**o+(-0.38588684985)*x[0]+(-0.630908284751)*x[1]**o+(0.237972319424)*x[1]
        arg[2,1,1]=(-0.928366568501)*x[0]**o+(0.254484026833)*x[0]+(0.903931940538)*x[1]**o+(-0.370818194785)*x[1]
        arg[3,0,0]=(-0.216012465475)*x[0]**o+(-0.938274177001)*x[0]+(0.0674408710328)*x[1]**o+(-0.731594784098)*x[1]
        arg[3,0,1]=(-0.156790905207)*x[0]**o+(0.284403081253)*x[0]+(0.3754228052)*x[1]**o+(-0.772758808958)*x[1]
        arg[3,1,0]=(0.89987768188)*x[0]**o+(0.0177886564195)*x[0]+(0.102329016274)*x[1]**o+(0.474039949682)*x[1]
        arg[3,1,1]=(0.0416243959618)*x[0]**o+(0.739593496294)*x[0]+(-0.0215969068381)*x[1]**o+(0.203954403681)*x[1]
        arg[4,0,0]=(-0.479380091881)*x[0]**o+(0.973630227471)*x[0]+(0.372617682727)*x[1]**o+(0.721993565094)*x[1]
        arg[4,0,1]=(0.360211306746)*x[0]**o+(0.260993313325)*x[0]+(0.252635752916)*x[1]**o+(0.534875843061)*x[1]
        arg[4,1,0]=(-0.609695114069)*x[0]**o+(0.283865161572)*x[0]+(-0.607441168016)*x[1]**o+(-0.838279309054)*x[1]
        arg[4,1,1]=(0.984921561958)*x[0]**o+(-0.352255390774)*x[0]+(-0.770706841576)*x[1]**o+(0.456415771191)*x[1]
        arg[5,0,0]=(0.903410389973)*x[0]**o+(0.609651342058)*x[0]+(-0.0890705788805)*x[1]**o+(0.613493629434)*x[1]
        arg[5,0,1]=(-0.977979577058)*x[0]**o+(-0.121665631763)*x[0]+(0.466897174395)*x[1]**o+(-0.438819940752)*x[1]
        arg[5,1,0]=(-0.537313389905)*x[0]**o+(-0.0401503571218)*x[0]+(-0.454563705252)*x[1]**o+(-0.728276295145)*x[1]
        arg[5,1,1]=(0.858855332454)*x[0]**o+(0.378938106473)*x[0]+(0.908957690039)*x[1]**o+(0.989040980713)*x[1]
        ref[0,0,0]=(-0.906705618142)*x_ref[0]**o+(0.458982439103)*x_ref[0]+(0.556464353903)*x_ref[1]**o+(0.733236686234)*x_ref[1]
        ref[0,0,1]=(-0.192889954523)*x_ref[0]**o+(-0.814332539133)*x_ref[0]+(0.446164214282)*x_ref[1]**o+(0.032291315446)*x_ref[1]
        ref[0,1,0]=(0.757081929199)*x_ref[0]**o+(-0.3938524931)*x_ref[0]+(-0.458060132115)*x_ref[1]**o+(0.471831491182)*x_ref[1]
        ref[0,1,1]=(0.459868785407)*x_ref[0]**o+(0.513707183316)*x_ref[0]+(-0.9682413377)*x_ref[1]**o+(-0.49496265187)*x_ref[1]
        ref[1,0,0]=(0.546655212697)*x_ref[0]**o+(-0.296125788768)*x_ref[0]+(0.570033305927)*x_ref[1]**o+(-0.818919094317)*x_ref[1]
        ref[1,0,1]=(-0.954422252892)*x_ref[0]**o+(-0.925035286684)*x_ref[0]+(0.171311127546)*x_ref[1]**o+(-0.584606597796)*x_ref[1]
        ref[1,1,0]=(-0.203956024883)*x_ref[0]**o+(-0.933117428128)*x_ref[0]+(-0.745101180472)*x_ref[1]**o+(0.0390208509919)*x_ref[1]
        ref[1,1,1]=(-0.67467593628)*x_ref[0]**o+(0.42281003355)*x_ref[0]+(-0.495401176467)*x_ref[1]**o+(0.993257306842)*x_ref[1]
        ref[2,0,0]=(-0.204011890956)*x_ref[0]**o+(0.730452092671)*x_ref[0]+(-0.0486718200506)*x_ref[1]**o+(0.691928726795)*x_ref[1]
        ref[2,0,1]=(0.381571380631)*x_ref[0]**o+(0.504896621541)*x_ref[0]+(-0.229805983176)*x_ref[1]**o+(0.281401365705)*x_ref[1]
        ref[2,1,0]=(0.578852751326)*x_ref[0]**o+(-0.38588684985)*x_ref[0]+(-0.630908284751)*x_ref[1]**o+(0.237972319424)*x_ref[1]
        ref[2,1,1]=(-0.928366568501)*x_ref[0]**o+(0.254484026833)*x_ref[0]+(0.903931940538)*x_ref[1]**o+(-0.370818194785)*x_ref[1]
        ref[3,0,0]=(-0.216012465475)*x_ref[0]**o+(-0.938274177001)*x_ref[0]+(0.0674408710328)*x_ref[1]**o+(-0.731594784098)*x_ref[1]
        ref[3,0,1]=(-0.156790905207)*x_ref[0]**o+(0.284403081253)*x_ref[0]+(0.3754228052)*x_ref[1]**o+(-0.772758808958)*x_ref[1]
        ref[3,1,0]=(0.89987768188)*x_ref[0]**o+(0.0177886564195)*x_ref[0]+(0.102329016274)*x_ref[1]**o+(0.474039949682)*x_ref[1]
        ref[3,1,1]=(0.0416243959618)*x_ref[0]**o+(0.739593496294)*x_ref[0]+(-0.0215969068381)*x_ref[1]**o+(0.203954403681)*x_ref[1]
        ref[4,0,0]=(-0.479380091881)*x_ref[0]**o+(0.973630227471)*x_ref[0]+(0.372617682727)*x_ref[1]**o+(0.721993565094)*x_ref[1]
        ref[4,0,1]=(0.360211306746)*x_ref[0]**o+(0.260993313325)*x_ref[0]+(0.252635752916)*x_ref[1]**o+(0.534875843061)*x_ref[1]
        ref[4,1,0]=(-0.609695114069)*x_ref[0]**o+(0.283865161572)*x_ref[0]+(-0.607441168016)*x_ref[1]**o+(-0.838279309054)*x_ref[1]
        ref[4,1,1]=(0.984921561958)*x_ref[0]**o+(-0.352255390774)*x_ref[0]+(-0.770706841576)*x_ref[1]**o+(0.456415771191)*x_ref[1]
        ref[5,0,0]=(0.903410389973)*x_ref[0]**o+(0.609651342058)*x_ref[0]+(-0.0890705788805)*x_ref[1]**o+(0.613493629434)*x_ref[1]
        ref[5,0,1]=(-0.977979577058)*x_ref[0]**o+(-0.121665631763)*x_ref[0]+(0.466897174395)*x_ref[1]**o+(-0.438819940752)*x_ref[1]
        ref[5,1,0]=(-0.537313389905)*x_ref[0]**o+(-0.0401503571218)*x_ref[0]+(-0.454563705252)*x_ref[1]**o+(-0.728276295145)*x_ref[1]
        ref[5,1,1]=(0.858855332454)*x_ref[0]**o+(0.378938106473)*x_ref[0]+(0.908957690039)*x_ref[1]**o+(0.989040980713)*x_ref[1]
      else:
        arg[0,0,0]=(-0.550852990025)*x[0]**o+(0.578705971531)*x[0]+(0.9533401901)*x[1]**o+(0.233181910221)*x[1]+(-0.143544272432)*x[2]**o+(-0.986513253736)*x[2]
        arg[0,0,1]=(-0.326417636931)*x[0]**o+(0.0910236969351)*x[0]+(-0.8240642668)*x[1]**o+(-0.242359389714)*x[1]+(0.266813299647)*x[2]**o+(-0.364956839435)*x[2]
        arg[0,1,0]=(-0.777107068999)*x[0]**o+(0.201120303703)*x[0]+(-0.684906707463)*x[1]**o+(-0.9620762843)*x[1]+(-0.314789913974)*x[2]**o+(-0.70682592319)*x[2]
        arg[0,1,1]=(-0.980292769796)*x[0]**o+(-0.827763529178)*x[0]+(0.256770279946)*x[1]**o+(0.619665292369)*x[1]+(0.014752087873)*x[2]**o+(-0.810044916486)*x[2]
        arg[1,0,0]=(-0.51164541516)*x[0]**o+(-0.429646605692)*x[0]+(-0.844516540906)*x[1]**o+(-0.0334629941654)*x[1]+(0.942164301071)*x[2]**o+(0.129008643344)*x[2]
        arg[1,0,1]=(-0.545329352463)*x[0]**o+(-0.0601985885543)*x[0]+(0.959752895562)*x[1]**o+(-0.617790988468)*x[1]+(-0.352323774737)*x[2]**o+(0.0854549755648)*x[2]
        arg[1,1,0]=(0.163270290732)*x[0]**o+(0.715418689409)*x[0]+(-0.892736094727)*x[1]**o+(-0.250085969902)*x[1]+(-0.323686607391)*x[2]**o+(-0.808822044566)*x[2]
        arg[1,1,1]=(-0.582858228848)*x[0]**o+(0.245356494646)*x[0]+(0.253867860186)*x[1]**o+(0.294533212339)*x[1]+(0.802766375942)*x[2]**o+(-0.142246487978)*x[2]
        arg[2,0,0]=(0.559682931749)*x[0]**o+(0.349781768104)*x[0]+(-0.903942828138)*x[1]**o+(0.285790428402)*x[1]+(0.334765408827)*x[2]**o+(-0.319553167552)*x[2]
        arg[2,0,1]=(-0.33958138616)*x[0]**o+(-0.476356251551)*x[0]+(0.222685919591)*x[1]**o+(0.947677379274)*x[1]+(-0.238198083505)*x[2]**o+(-0.158835521934)*x[2]
        arg[2,1,0]=(0.233005925554)*x[0]**o+(-0.0885827512025)*x[0]+(0.99298235997)*x[1]**o+(-0.972864413346)*x[1]+(-0.328337965331)*x[2]**o+(0.548449226427)*x[2]
        arg[2,1,1]=(-0.682434615287)*x[0]**o+(-0.681560497743)*x[0]+(-0.0427998840708)*x[1]**o+(-0.640532507401)*x[1]+(0.0600572641903)*x[2]**o+(0.580127097835)*x[2]
        arg[3,0,0]=(0.627556962303)*x[0]**o+(-0.2282646452)*x[0]+(0.201159933134)*x[1]**o+(-0.0874921144181)*x[1]+(0.487026890349)*x[2]**o+(-0.044256665504)*x[2]
        arg[3,0,1]=(-0.202969525043)*x[0]**o+(0.0445621391034)*x[0]+(-0.763479658456)*x[1]**o+(-0.600818445386)*x[1]+(-0.442800171339)*x[2]**o+(0.0422443400421)*x[2]
        arg[3,1,0]=(-0.422348404642)*x[0]**o+(0.723277196982)*x[0]+(0.882658191301)*x[1]**o+(-0.415218341669)*x[1]+(0.834911332561)*x[2]**o+(-0.843638850779)*x[2]
        arg[3,1,1]=(-0.685037113836)*x[0]**o+(0.119526696821)*x[0]+(0.732784440424)*x[1]**o+(0.156993935952)*x[1]+(-0.565837827825)*x[2]**o+(0.667635603478)*x[2]
        arg[4,0,0]=(-0.18210082527)*x[0]**o+(-0.00639006601355)*x[0]+(0.633771301081)*x[1]**o+(0.765450011643)*x[1]+(-0.323701472535)*x[2]**o+(0.841246809469)*x[2]
        arg[4,0,1]=(0.345847635107)*x[0]**o+(0.910917657816)*x[0]+(0.329667803527)*x[1]**o+(-0.0638797846725)*x[1]+(-0.705758696202)*x[2]**o+(0.39567870447)*x[2]
        arg[4,1,0]=(0.700805557734)*x[0]**o+(0.215376719066)*x[0]+(0.057212023986)*x[1]**o+(-0.601972647211)*x[1]+(-0.0163323501628)*x[2]**o+(-0.182314452762)*x[2]
        arg[4,1,1]=(-0.25633784052)*x[0]**o+(-0.36128209808)*x[0]+(-0.580660119759)*x[1]**o+(-0.718361413439)*x[1]+(0.294752937469)*x[2]**o+(0.479425008353)*x[2]
        arg[5,0,0]=(0.0250768411622)*x[0]**o+(-0.954437161073)*x[0]+(0.314623130846)*x[1]**o+(-0.915911802082)*x[1]+(-0.293481557155)*x[2]**o+(0.193040417512)*x[2]
        arg[5,0,1]=(-0.260552760825)*x[0]**o+(-0.993318746791)*x[0]+(-0.0888187740235)*x[1]**o+(0.228899511722)*x[1]+(0.667467747688)*x[2]**o+(0.210051895803)*x[2]
        arg[5,1,0]=(-0.925551425372)*x[0]**o+(-0.051063099903)*x[0]+(0.0607054862843)*x[1]**o+(0.778043999849)*x[1]+(-0.393429559505)*x[2]**o+(0.235480458425)*x[2]
        arg[5,1,1]=(0.00188621198193)*x[0]**o+(-0.529444982511)*x[0]+(0.766032106394)*x[1]**o+(-0.554844276584)*x[1]+(0.793956883291)*x[2]**o+(0.221253426455)*x[2]
        ref[0,0,0]=(-0.550852990025)*x_ref[0]**o+(0.578705971531)*x_ref[0]+(0.9533401901)*x_ref[1]**o+(0.233181910221)*x_ref[1]+(-0.143544272432)*x_ref[2]**o+(-0.986513253736)*x_ref[2]
        ref[0,0,1]=(-0.326417636931)*x_ref[0]**o+(0.0910236969351)*x_ref[0]+(-0.8240642668)*x_ref[1]**o+(-0.242359389714)*x_ref[1]+(0.266813299647)*x_ref[2]**o+(-0.364956839435)*x_ref[2]
        ref[0,1,0]=(-0.777107068999)*x_ref[0]**o+(0.201120303703)*x_ref[0]+(-0.684906707463)*x_ref[1]**o+(-0.9620762843)*x_ref[1]+(-0.314789913974)*x_ref[2]**o+(-0.70682592319)*x_ref[2]
        ref[0,1,1]=(-0.980292769796)*x_ref[0]**o+(-0.827763529178)*x_ref[0]+(0.256770279946)*x_ref[1]**o+(0.619665292369)*x_ref[1]+(0.014752087873)*x_ref[2]**o+(-0.810044916486)*x_ref[2]
        ref[1,0,0]=(-0.51164541516)*x_ref[0]**o+(-0.429646605692)*x_ref[0]+(-0.844516540906)*x_ref[1]**o+(-0.0334629941654)*x_ref[1]+(0.942164301071)*x_ref[2]**o+(0.129008643344)*x_ref[2]
        ref[1,0,1]=(-0.545329352463)*x_ref[0]**o+(-0.0601985885543)*x_ref[0]+(0.959752895562)*x_ref[1]**o+(-0.617790988468)*x_ref[1]+(-0.352323774737)*x_ref[2]**o+(0.0854549755648)*x_ref[2]
        ref[1,1,0]=(0.163270290732)*x_ref[0]**o+(0.715418689409)*x_ref[0]+(-0.892736094727)*x_ref[1]**o+(-0.250085969902)*x_ref[1]+(-0.323686607391)*x_ref[2]**o+(-0.808822044566)*x_ref[2]
        ref[1,1,1]=(-0.582858228848)*x_ref[0]**o+(0.245356494646)*x_ref[0]+(0.253867860186)*x_ref[1]**o+(0.294533212339)*x_ref[1]+(0.802766375942)*x_ref[2]**o+(-0.142246487978)*x_ref[2]
        ref[2,0,0]=(0.559682931749)*x_ref[0]**o+(0.349781768104)*x_ref[0]+(-0.903942828138)*x_ref[1]**o+(0.285790428402)*x_ref[1]+(0.334765408827)*x_ref[2]**o+(-0.319553167552)*x_ref[2]
        ref[2,0,1]=(-0.33958138616)*x_ref[0]**o+(-0.476356251551)*x_ref[0]+(0.222685919591)*x_ref[1]**o+(0.947677379274)*x_ref[1]+(-0.238198083505)*x_ref[2]**o+(-0.158835521934)*x_ref[2]
        ref[2,1,0]=(0.233005925554)*x_ref[0]**o+(-0.0885827512025)*x_ref[0]+(0.99298235997)*x_ref[1]**o+(-0.972864413346)*x_ref[1]+(-0.328337965331)*x_ref[2]**o+(0.548449226427)*x_ref[2]
        ref[2,1,1]=(-0.682434615287)*x_ref[0]**o+(-0.681560497743)*x_ref[0]+(-0.0427998840708)*x_ref[1]**o+(-0.640532507401)*x_ref[1]+(0.0600572641903)*x_ref[2]**o+(0.580127097835)*x_ref[2]
        ref[3,0,0]=(0.627556962303)*x_ref[0]**o+(-0.2282646452)*x_ref[0]+(0.201159933134)*x_ref[1]**o+(-0.0874921144181)*x_ref[1]+(0.487026890349)*x_ref[2]**o+(-0.044256665504)*x_ref[2]
        ref[3,0,1]=(-0.202969525043)*x_ref[0]**o+(0.0445621391034)*x_ref[0]+(-0.763479658456)*x_ref[1]**o+(-0.600818445386)*x_ref[1]+(-0.442800171339)*x_ref[2]**o+(0.0422443400421)*x_ref[2]
        ref[3,1,0]=(-0.422348404642)*x_ref[0]**o+(0.723277196982)*x_ref[0]+(0.882658191301)*x_ref[1]**o+(-0.415218341669)*x_ref[1]+(0.834911332561)*x_ref[2]**o+(-0.843638850779)*x_ref[2]
        ref[3,1,1]=(-0.685037113836)*x_ref[0]**o+(0.119526696821)*x_ref[0]+(0.732784440424)*x_ref[1]**o+(0.156993935952)*x_ref[1]+(-0.565837827825)*x_ref[2]**o+(0.667635603478)*x_ref[2]
        ref[4,0,0]=(-0.18210082527)*x_ref[0]**o+(-0.00639006601355)*x_ref[0]+(0.633771301081)*x_ref[1]**o+(0.765450011643)*x_ref[1]+(-0.323701472535)*x_ref[2]**o+(0.841246809469)*x_ref[2]
        ref[4,0,1]=(0.345847635107)*x_ref[0]**o+(0.910917657816)*x_ref[0]+(0.329667803527)*x_ref[1]**o+(-0.0638797846725)*x_ref[1]+(-0.705758696202)*x_ref[2]**o+(0.39567870447)*x_ref[2]
        ref[4,1,0]=(0.700805557734)*x_ref[0]**o+(0.215376719066)*x_ref[0]+(0.057212023986)*x_ref[1]**o+(-0.601972647211)*x_ref[1]+(-0.0163323501628)*x_ref[2]**o+(-0.182314452762)*x_ref[2]
        ref[4,1,1]=(-0.25633784052)*x_ref[0]**o+(-0.36128209808)*x_ref[0]+(-0.580660119759)*x_ref[1]**o+(-0.718361413439)*x_ref[1]+(0.294752937469)*x_ref[2]**o+(0.479425008353)*x_ref[2]
        ref[5,0,0]=(0.0250768411622)*x_ref[0]**o+(-0.954437161073)*x_ref[0]+(0.314623130846)*x_ref[1]**o+(-0.915911802082)*x_ref[1]+(-0.293481557155)*x_ref[2]**o+(0.193040417512)*x_ref[2]
        ref[5,0,1]=(-0.260552760825)*x_ref[0]**o+(-0.993318746791)*x_ref[0]+(-0.0888187740235)*x_ref[1]**o+(0.228899511722)*x_ref[1]+(0.667467747688)*x_ref[2]**o+(0.210051895803)*x_ref[2]
        ref[5,1,0]=(-0.925551425372)*x_ref[0]**o+(-0.051063099903)*x_ref[0]+(0.0607054862843)*x_ref[1]**o+(0.778043999849)*x_ref[1]+(-0.393429559505)*x_ref[2]**o+(0.235480458425)*x_ref[2]
        ref[5,1,1]=(0.00188621198193)*x_ref[0]**o+(-0.529444982511)*x_ref[0]+(0.766032106394)*x_ref[1]**o+(-0.554844276584)*x_ref[1]+(0.793956883291)*x_ref[2]**o+(0.221253426455)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromData_ReducedFunctionOnContactZero_rank4(self):
      """
      tests interpolate for rank 4 Data onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactZero onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactZero(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5, 3, 2),w)
      ref=Data(0,(4, 5, 3, 2),w_ref)
      if dim==2:
        arg[0,0,0,0]=(0.358768976574)*x[0]**o+(-0.231637053165)*x[0]+(0.798891506345)*x[1]**o+(0.292325388353)*x[1]
        arg[0,0,0,1]=(0.53301689922)*x[0]**o+(-0.761786870664)*x[0]+(0.284067900826)*x[1]**o+(-0.688662082205)*x[1]
        arg[0,0,1,0]=(-0.721988036113)*x[0]**o+(-0.743892757815)*x[0]+(-0.796787022375)*x[1]**o+(-0.499468412523)*x[1]
        arg[0,0,1,1]=(0.0710691488306)*x[0]**o+(0.0210385141456)*x[0]+(-0.709342417683)*x[1]**o+(0.901794071115)*x[1]
        arg[0,0,2,0]=(-0.800537263339)*x[0]**o+(0.583114786539)*x[0]+(0.422752622522)*x[1]**o+(-0.0954981460748)*x[1]
        arg[0,0,2,1]=(-0.0805357065445)*x[0]**o+(0.476377895547)*x[0]+(0.242203912566)*x[1]**o+(-0.479971540043)*x[1]
        arg[0,1,0,0]=(-0.476356823162)*x[0]**o+(-0.438853622252)*x[0]+(-0.937228141746)*x[1]**o+(0.366496742588)*x[1]
        arg[0,1,0,1]=(-0.496918253956)*x[0]**o+(-0.0227549061268)*x[0]+(0.830195584768)*x[1]**o+(-0.114624902423)*x[1]
        arg[0,1,1,0]=(0.431291036997)*x[0]**o+(0.807713720258)*x[0]+(0.730057543333)*x[1]**o+(0.635171725232)*x[1]
        arg[0,1,1,1]=(0.784425995411)*x[0]**o+(-0.397510494899)*x[0]+(-0.579911740115)*x[1]**o+(0.130426014607)*x[1]
        arg[0,1,2,0]=(0.473613650641)*x[0]**o+(-0.155617741177)*x[0]+(0.873237852103)*x[1]**o+(-0.749395152414)*x[1]
        arg[0,1,2,1]=(0.367649559431)*x[0]**o+(0.830181309644)*x[0]+(0.87726968147)*x[1]**o+(-0.450304793682)*x[1]
        arg[0,2,0,0]=(-0.700522876654)*x[0]**o+(-0.998187126003)*x[0]+(0.732491610173)*x[1]**o+(0.648679267)*x[1]
        arg[0,2,0,1]=(0.499746141285)*x[0]**o+(-0.730090975806)*x[0]+(-0.525566438654)*x[1]**o+(0.212469850421)*x[1]
        arg[0,2,1,0]=(-0.847752916298)*x[0]**o+(0.43510472619)*x[0]+(0.815305936548)*x[1]**o+(-0.143802676552)*x[1]
        arg[0,2,1,1]=(-0.453572376958)*x[0]**o+(-0.913023531929)*x[0]+(0.743328127665)*x[1]**o+(0.152586122591)*x[1]
        arg[0,2,2,0]=(0.526939334508)*x[0]**o+(0.247268722057)*x[0]+(-0.176432621481)*x[1]**o+(-0.510110202196)*x[1]
        arg[0,2,2,1]=(0.139685732003)*x[0]**o+(0.278304455867)*x[0]+(0.517953769616)*x[1]**o+(-0.272384261701)*x[1]
        arg[0,3,0,0]=(-0.38867907688)*x[0]**o+(-0.73043975055)*x[0]+(-0.14293243901)*x[1]**o+(0.0601042854043)*x[1]
        arg[0,3,0,1]=(-0.163055949188)*x[0]**o+(-0.035877062482)*x[0]+(0.0396615423106)*x[1]**o+(-0.870062303275)*x[1]
        arg[0,3,1,0]=(0.283839779914)*x[0]**o+(-0.519316925197)*x[0]+(-0.437401973959)*x[1]**o+(-0.823707232663)*x[1]
        arg[0,3,1,1]=(0.700305425892)*x[0]**o+(0.0299761521094)*x[0]+(-0.993032151025)*x[1]**o+(0.633293413221)*x[1]
        arg[0,3,2,0]=(-0.745357256365)*x[0]**o+(-0.451553456961)*x[0]+(0.05789646302)*x[1]**o+(-0.755901155251)*x[1]
        arg[0,3,2,1]=(-0.57330840403)*x[0]**o+(-0.589361483751)*x[0]+(-0.038677123513)*x[1]**o+(0.417332032168)*x[1]
        arg[0,4,0,0]=(0.759655161531)*x[0]**o+(0.737483461328)*x[0]+(0.573158629937)*x[1]**o+(-0.454191109106)*x[1]
        arg[0,4,0,1]=(-0.232982406645)*x[0]**o+(-0.926030038179)*x[0]+(-0.504256834465)*x[1]**o+(-0.107703527058)*x[1]
        arg[0,4,1,0]=(0.898411697016)*x[0]**o+(0.642045536742)*x[0]+(-0.953059791895)*x[1]**o+(-0.67501710093)*x[1]
        arg[0,4,1,1]=(0.300380574165)*x[0]**o+(0.204591960115)*x[0]+(-0.308849697562)*x[1]**o+(-0.489495887357)*x[1]
        arg[0,4,2,0]=(-0.814560286716)*x[0]**o+(0.154235599953)*x[0]+(0.279329273498)*x[1]**o+(-0.674063978926)*x[1]
        arg[0,4,2,1]=(-0.718018870365)*x[0]**o+(-0.735982858424)*x[0]+(-0.0132197132926)*x[1]**o+(0.398939587543)*x[1]
        arg[1,0,0,0]=(0.0559995351725)*x[0]**o+(-0.164549619892)*x[0]+(-0.794242529408)*x[1]**o+(0.0927822314785)*x[1]
        arg[1,0,0,1]=(-0.894238542341)*x[0]**o+(0.378559247473)*x[0]+(-0.183957272639)*x[1]**o+(0.486358968942)*x[1]
        arg[1,0,1,0]=(0.931288801001)*x[0]**o+(0.223815769968)*x[0]+(-0.719591906328)*x[1]**o+(-0.743644741105)*x[1]
        arg[1,0,1,1]=(0.156646029123)*x[0]**o+(0.306942879419)*x[0]+(0.747006830674)*x[1]**o+(-0.876534912274)*x[1]
        arg[1,0,2,0]=(-0.496819081999)*x[0]**o+(-0.939227402425)*x[0]+(0.0566331712182)*x[1]**o+(0.565724364661)*x[1]
        arg[1,0,2,1]=(0.903706429954)*x[0]**o+(0.667947320307)*x[0]+(0.97140855373)*x[1]**o+(0.924095235752)*x[1]
        arg[1,1,0,0]=(0.526180197299)*x[0]**o+(0.435493351674)*x[0]+(0.400107077848)*x[1]**o+(-0.704760833758)*x[1]
        arg[1,1,0,1]=(0.0723740015004)*x[0]**o+(-0.405859051371)*x[0]+(0.200787757511)*x[1]**o+(0.41819184563)*x[1]
        arg[1,1,1,0]=(-0.0320951655627)*x[0]**o+(0.235068445821)*x[0]+(0.450383349599)*x[1]**o+(-0.801487424765)*x[1]
        arg[1,1,1,1]=(-0.52478892744)*x[0]**o+(-0.449111297125)*x[0]+(0.731062182014)*x[1]**o+(0.533619608577)*x[1]
        arg[1,1,2,0]=(0.0889794057057)*x[0]**o+(-0.971856981571)*x[0]+(0.662216408922)*x[1]**o+(0.164089024468)*x[1]
        arg[1,1,2,1]=(-0.196374963647)*x[0]**o+(-0.739360211802)*x[0]+(0.916611650753)*x[1]**o+(-0.260102318115)*x[1]
        arg[1,2,0,0]=(0.627763572103)*x[0]**o+(-0.893900847319)*x[0]+(0.907548446417)*x[1]**o+(0.0687906203663)*x[1]
        arg[1,2,0,1]=(0.59755498682)*x[0]**o+(-0.457108009787)*x[0]+(0.71562139654)*x[1]**o+(-0.300872832559)*x[1]
        arg[1,2,1,0]=(-0.479641829834)*x[0]**o+(0.297188513722)*x[0]+(0.0227730978493)*x[1]**o+(0.199329665406)*x[1]
        arg[1,2,1,1]=(0.918974940101)*x[0]**o+(0.595194981816)*x[0]+(0.760030152554)*x[1]**o+(-0.798899878486)*x[1]
        arg[1,2,2,0]=(-0.878334511842)*x[0]**o+(0.910857236287)*x[0]+(0.0164747567997)*x[1]**o+(-0.876970888445)*x[1]
        arg[1,2,2,1]=(0.29445809181)*x[0]**o+(0.837849454931)*x[0]+(0.0759054371211)*x[1]**o+(0.501243080372)*x[1]
        arg[1,3,0,0]=(-0.529006436258)*x[0]**o+(-0.932325855538)*x[0]+(-0.561472972374)*x[1]**o+(0.845125736656)*x[1]
        arg[1,3,0,1]=(-0.843008257488)*x[0]**o+(0.614118623012)*x[0]+(-0.167015137447)*x[1]**o+(0.158701952625)*x[1]
        arg[1,3,1,0]=(-0.662966916148)*x[0]**o+(-0.443297576512)*x[0]+(0.295988608077)*x[1]**o+(0.059506908615)*x[1]
        arg[1,3,1,1]=(0.864199978014)*x[0]**o+(0.138723519602)*x[0]+(0.144964839302)*x[1]**o+(0.239863709389)*x[1]
        arg[1,3,2,0]=(0.479006097431)*x[0]**o+(0.542168797821)*x[0]+(0.561074562163)*x[1]**o+(-0.133365787275)*x[1]
        arg[1,3,2,1]=(-0.901033449452)*x[0]**o+(-0.620245150074)*x[0]+(0.598461569697)*x[1]**o+(-0.2213246854)*x[1]
        arg[1,4,0,0]=(-0.577067401348)*x[0]**o+(0.486154324874)*x[0]+(0.809205150442)*x[1]**o+(-0.218206418459)*x[1]
        arg[1,4,0,1]=(0.726556862066)*x[0]**o+(0.0293695509867)*x[0]+(0.601938348193)*x[1]**o+(0.967332697531)*x[1]
        arg[1,4,1,0]=(-0.675624388054)*x[0]**o+(-0.871613723214)*x[0]+(0.605519387693)*x[1]**o+(0.287186894411)*x[1]
        arg[1,4,1,1]=(-0.689904898214)*x[0]**o+(-0.918236705665)*x[0]+(-0.892168622177)*x[1]**o+(0.507441662612)*x[1]
        arg[1,4,2,0]=(0.845768980712)*x[0]**o+(-0.458467508114)*x[0]+(-0.518185249391)*x[1]**o+(0.829945330327)*x[1]
        arg[1,4,2,1]=(0.776852312178)*x[0]**o+(-0.268810999469)*x[0]+(-0.576757148841)*x[1]**o+(-0.50628294802)*x[1]
        arg[2,0,0,0]=(-0.686840219939)*x[0]**o+(0.6861172847)*x[0]+(0.240362263052)*x[1]**o+(0.962361523884)*x[1]
        arg[2,0,0,1]=(-0.943837188119)*x[0]**o+(0.829832768268)*x[0]+(0.377590124133)*x[1]**o+(-0.171714144962)*x[1]
        arg[2,0,1,0]=(0.862809148447)*x[0]**o+(0.0573402136489)*x[0]+(-0.743311826013)*x[1]**o+(0.427809313437)*x[1]
        arg[2,0,1,1]=(0.377380819576)*x[0]**o+(-0.223280285041)*x[0]+(0.178628658562)*x[1]**o+(0.121964611877)*x[1]
        arg[2,0,2,0]=(0.41723758689)*x[0]**o+(0.256039203883)*x[0]+(0.255358776676)*x[1]**o+(-0.881665964974)*x[1]
        arg[2,0,2,1]=(-0.0797103024499)*x[0]**o+(0.128068931414)*x[0]+(-0.758317834409)*x[1]**o+(0.654657809183)*x[1]
        arg[2,1,0,0]=(0.603648218133)*x[0]**o+(-0.286780270729)*x[0]+(-0.341108067214)*x[1]**o+(-0.769667356087)*x[1]
        arg[2,1,0,1]=(0.844044270035)*x[0]**o+(0.0585907857764)*x[0]+(-0.578232785252)*x[1]**o+(-0.40959017859)*x[1]
        arg[2,1,1,0]=(0.599709567393)*x[0]**o+(0.307092449739)*x[0]+(-0.402950586215)*x[1]**o+(0.0674239470451)*x[1]
        arg[2,1,1,1]=(0.182608249961)*x[0]**o+(0.373533584601)*x[0]+(0.40949869082)*x[1]**o+(0.696333617699)*x[1]
        arg[2,1,2,0]=(0.0462891111054)*x[0]**o+(0.371258293292)*x[0]+(0.782976187716)*x[1]**o+(0.117976017999)*x[1]
        arg[2,1,2,1]=(0.891841121526)*x[0]**o+(-0.998971173204)*x[0]+(0.0286712507928)*x[1]**o+(-0.0198507022516)*x[1]
        arg[2,2,0,0]=(0.284620902703)*x[0]**o+(0.936231203073)*x[0]+(0.829209752046)*x[1]**o+(-0.94940982125)*x[1]
        arg[2,2,0,1]=(-0.647956027376)*x[0]**o+(0.923690718922)*x[0]+(-0.0322423837346)*x[1]**o+(0.124962508606)*x[1]
        arg[2,2,1,0]=(-0.0696010411705)*x[0]**o+(-0.161672936739)*x[0]+(-0.997445762186)*x[1]**o+(0.177962302014)*x[1]
        arg[2,2,1,1]=(-0.897657700296)*x[0]**o+(-0.467866721175)*x[0]+(-0.560034332007)*x[1]**o+(-0.861509898295)*x[1]
        arg[2,2,2,0]=(-0.100880174048)*x[0]**o+(0.678568254695)*x[0]+(-0.83289411684)*x[1]**o+(0.680564393149)*x[1]
        arg[2,2,2,1]=(0.546159181491)*x[0]**o+(0.720296622966)*x[0]+(0.939046941118)*x[1]**o+(-0.716147554733)*x[1]
        arg[2,3,0,0]=(-0.921073162235)*x[0]**o+(0.979925882289)*x[0]+(-0.0938642966949)*x[1]**o+(0.482093047223)*x[1]
        arg[2,3,0,1]=(0.0709940956822)*x[0]**o+(-0.691401017243)*x[0]+(0.447000814029)*x[1]**o+(0.0575856240533)*x[1]
        arg[2,3,1,0]=(0.562800393365)*x[0]**o+(0.657646549915)*x[0]+(-0.835326361689)*x[1]**o+(0.3487960687)*x[1]
        arg[2,3,1,1]=(0.106668867409)*x[0]**o+(0.436327240765)*x[0]+(-0.585051261259)*x[1]**o+(0.616655338667)*x[1]
        arg[2,3,2,0]=(-0.501849628176)*x[0]**o+(-0.284722503916)*x[0]+(-0.892470199837)*x[1]**o+(-0.979942541795)*x[1]
        arg[2,3,2,1]=(0.236759540073)*x[0]**o+(-0.806931113769)*x[0]+(-0.225220003475)*x[1]**o+(-0.95183239844)*x[1]
        arg[2,4,0,0]=(-0.684823909581)*x[0]**o+(0.445615228059)*x[0]+(-0.638879895767)*x[1]**o+(-0.335505874088)*x[1]
        arg[2,4,0,1]=(-0.196708460612)*x[0]**o+(0.981592996809)*x[0]+(0.435774317028)*x[1]**o+(-0.0952200663109)*x[1]
        arg[2,4,1,0]=(-0.552759474764)*x[0]**o+(-0.614031173545)*x[0]+(0.304375542871)*x[1]**o+(-0.285131901533)*x[1]
        arg[2,4,1,1]=(0.147812951421)*x[0]**o+(-0.78533487681)*x[0]+(0.942205524401)*x[1]**o+(0.166280215075)*x[1]
        arg[2,4,2,0]=(0.843067982563)*x[0]**o+(0.0116520710289)*x[0]+(0.927948889059)*x[1]**o+(-0.395251919577)*x[1]
        arg[2,4,2,1]=(-0.316538010007)*x[0]**o+(-0.139262497126)*x[0]+(0.645409249217)*x[1]**o+(0.192949521992)*x[1]
        arg[3,0,0,0]=(0.88179290666)*x[0]**o+(0.337853287365)*x[0]+(-0.590219587193)*x[1]**o+(-0.827051504196)*x[1]
        arg[3,0,0,1]=(-0.104526336997)*x[0]**o+(0.299097012282)*x[0]+(-0.348493995853)*x[1]**o+(-0.780034322304)*x[1]
        arg[3,0,1,0]=(0.358724159445)*x[0]**o+(-0.0338089482488)*x[0]+(-0.0423054506456)*x[1]**o+(0.67180489557)*x[1]
        arg[3,0,1,1]=(-0.778880095665)*x[0]**o+(-0.448812173928)*x[0]+(-0.823686122382)*x[1]**o+(0.636028922656)*x[1]
        arg[3,0,2,0]=(0.146891002069)*x[0]**o+(-0.368903199628)*x[0]+(-0.596272077557)*x[1]**o+(0.703510683225)*x[1]
        arg[3,0,2,1]=(0.357416423547)*x[0]**o+(-0.103754905142)*x[0]+(0.32026011189)*x[1]**o+(0.755648597627)*x[1]
        arg[3,1,0,0]=(0.399082464238)*x[0]**o+(-0.12611921797)*x[0]+(0.426552225948)*x[1]**o+(-0.228714297286)*x[1]
        arg[3,1,0,1]=(-0.638411026717)*x[0]**o+(-0.342143922306)*x[0]+(-0.502107802626)*x[1]**o+(-0.881045304848)*x[1]
        arg[3,1,1,0]=(0.411134247295)*x[0]**o+(-0.216585727086)*x[0]+(0.270013339156)*x[1]**o+(0.251581243494)*x[1]
        arg[3,1,1,1]=(0.788758848294)*x[0]**o+(0.345181621902)*x[0]+(0.102843710598)*x[1]**o+(0.0561071077516)*x[1]
        arg[3,1,2,0]=(0.232441162347)*x[0]**o+(-0.687544250386)*x[0]+(0.369065746372)*x[1]**o+(0.628109564412)*x[1]
        arg[3,1,2,1]=(-0.912295044312)*x[0]**o+(0.847830497468)*x[0]+(0.776648618003)*x[1]**o+(0.522638388084)*x[1]
        arg[3,2,0,0]=(0.779333120942)*x[0]**o+(-0.0747119967046)*x[0]+(-0.594363282198)*x[1]**o+(0.513922159444)*x[1]
        arg[3,2,0,1]=(0.877492427404)*x[0]**o+(-0.191727825399)*x[0]+(0.0881296554818)*x[1]**o+(-0.408407503067)*x[1]
        arg[3,2,1,0]=(0.29696771215)*x[0]**o+(-0.914882727138)*x[0]+(0.684156831385)*x[1]**o+(0.142293411327)*x[1]
        arg[3,2,1,1]=(0.105927449304)*x[0]**o+(-0.863339661598)*x[0]+(-0.815550662607)*x[1]**o+(-0.261034715274)*x[1]
        arg[3,2,2,0]=(0.944082529871)*x[0]**o+(-0.449259627627)*x[0]+(-0.560526159016)*x[1]**o+(0.609050500798)*x[1]
        arg[3,2,2,1]=(0.529508870267)*x[0]**o+(0.627216712239)*x[0]+(-0.0148122228382)*x[1]**o+(0.0743274653452)*x[1]
        arg[3,3,0,0]=(-0.30609697448)*x[0]**o+(0.910972095956)*x[0]+(0.256274444924)*x[1]**o+(0.229096821635)*x[1]
        arg[3,3,0,1]=(-0.510323627078)*x[0]**o+(0.308418491165)*x[0]+(0.786418296678)*x[1]**o+(-0.248788086479)*x[1]
        arg[3,3,1,0]=(-0.981107944688)*x[0]**o+(-0.331637054173)*x[0]+(-0.972981902152)*x[1]**o+(-0.389364980465)*x[1]
        arg[3,3,1,1]=(-0.839880147367)*x[0]**o+(-0.911569972467)*x[0]+(0.209642332468)*x[1]**o+(0.0926875442137)*x[1]
        arg[3,3,2,0]=(-0.22724314378)*x[0]**o+(-0.0396604383619)*x[0]+(0.906829234649)*x[1]**o+(-0.862410550066)*x[1]
        arg[3,3,2,1]=(-0.412816401935)*x[0]**o+(-0.848442294241)*x[0]+(-0.599302206188)*x[1]**o+(0.719403009819)*x[1]
        arg[3,4,0,0]=(0.259283054664)*x[0]**o+(0.0983272120339)*x[0]+(-0.538373082787)*x[1]**o+(0.950164435617)*x[1]
        arg[3,4,0,1]=(0.633594236794)*x[0]**o+(0.120984469179)*x[0]+(0.472771297982)*x[1]**o+(0.873015785724)*x[1]
        arg[3,4,1,0]=(0.913524970057)*x[0]**o+(0.106590018424)*x[0]+(0.124026494061)*x[1]**o+(-0.254989594971)*x[1]
        arg[3,4,1,1]=(0.945528885492)*x[0]**o+(-0.323205519576)*x[0]+(-0.467294290307)*x[1]**o+(-0.557033486306)*x[1]
        arg[3,4,2,0]=(-0.268364682764)*x[0]**o+(0.0289685280025)*x[0]+(0.0493964636013)*x[1]**o+(0.872120137365)*x[1]
        arg[3,4,2,1]=(-0.266698796897)*x[0]**o+(-0.679148300702)*x[0]+(0.337653394795)*x[1]**o+(0.186866149716)*x[1]
        ref[0,0,0,0]=(0.358768976574)*x_ref[0]**o+(-0.231637053165)*x_ref[0]+(0.798891506345)*x_ref[1]**o+(0.292325388353)*x_ref[1]
        ref[0,0,0,1]=(0.53301689922)*x_ref[0]**o+(-0.761786870664)*x_ref[0]+(0.284067900826)*x_ref[1]**o+(-0.688662082205)*x_ref[1]
        ref[0,0,1,0]=(-0.721988036113)*x_ref[0]**o+(-0.743892757815)*x_ref[0]+(-0.796787022375)*x_ref[1]**o+(-0.499468412523)*x_ref[1]
        ref[0,0,1,1]=(0.0710691488306)*x_ref[0]**o+(0.0210385141456)*x_ref[0]+(-0.709342417683)*x_ref[1]**o+(0.901794071115)*x_ref[1]
        ref[0,0,2,0]=(-0.800537263339)*x_ref[0]**o+(0.583114786539)*x_ref[0]+(0.422752622522)*x_ref[1]**o+(-0.0954981460748)*x_ref[1]
        ref[0,0,2,1]=(-0.0805357065445)*x_ref[0]**o+(0.476377895547)*x_ref[0]+(0.242203912566)*x_ref[1]**o+(-0.479971540043)*x_ref[1]
        ref[0,1,0,0]=(-0.476356823162)*x_ref[0]**o+(-0.438853622252)*x_ref[0]+(-0.937228141746)*x_ref[1]**o+(0.366496742588)*x_ref[1]
        ref[0,1,0,1]=(-0.496918253956)*x_ref[0]**o+(-0.0227549061268)*x_ref[0]+(0.830195584768)*x_ref[1]**o+(-0.114624902423)*x_ref[1]
        ref[0,1,1,0]=(0.431291036997)*x_ref[0]**o+(0.807713720258)*x_ref[0]+(0.730057543333)*x_ref[1]**o+(0.635171725232)*x_ref[1]
        ref[0,1,1,1]=(0.784425995411)*x_ref[0]**o+(-0.397510494899)*x_ref[0]+(-0.579911740115)*x_ref[1]**o+(0.130426014607)*x_ref[1]
        ref[0,1,2,0]=(0.473613650641)*x_ref[0]**o+(-0.155617741177)*x_ref[0]+(0.873237852103)*x_ref[1]**o+(-0.749395152414)*x_ref[1]
        ref[0,1,2,1]=(0.367649559431)*x_ref[0]**o+(0.830181309644)*x_ref[0]+(0.87726968147)*x_ref[1]**o+(-0.450304793682)*x_ref[1]
        ref[0,2,0,0]=(-0.700522876654)*x_ref[0]**o+(-0.998187126003)*x_ref[0]+(0.732491610173)*x_ref[1]**o+(0.648679267)*x_ref[1]
        ref[0,2,0,1]=(0.499746141285)*x_ref[0]**o+(-0.730090975806)*x_ref[0]+(-0.525566438654)*x_ref[1]**o+(0.212469850421)*x_ref[1]
        ref[0,2,1,0]=(-0.847752916298)*x_ref[0]**o+(0.43510472619)*x_ref[0]+(0.815305936548)*x_ref[1]**o+(-0.143802676552)*x_ref[1]
        ref[0,2,1,1]=(-0.453572376958)*x_ref[0]**o+(-0.913023531929)*x_ref[0]+(0.743328127665)*x_ref[1]**o+(0.152586122591)*x_ref[1]
        ref[0,2,2,0]=(0.526939334508)*x_ref[0]**o+(0.247268722057)*x_ref[0]+(-0.176432621481)*x_ref[1]**o+(-0.510110202196)*x_ref[1]
        ref[0,2,2,1]=(0.139685732003)*x_ref[0]**o+(0.278304455867)*x_ref[0]+(0.517953769616)*x_ref[1]**o+(-0.272384261701)*x_ref[1]
        ref[0,3,0,0]=(-0.38867907688)*x_ref[0]**o+(-0.73043975055)*x_ref[0]+(-0.14293243901)*x_ref[1]**o+(0.0601042854043)*x_ref[1]
        ref[0,3,0,1]=(-0.163055949188)*x_ref[0]**o+(-0.035877062482)*x_ref[0]+(0.0396615423106)*x_ref[1]**o+(-0.870062303275)*x_ref[1]
        ref[0,3,1,0]=(0.283839779914)*x_ref[0]**o+(-0.519316925197)*x_ref[0]+(-0.437401973959)*x_ref[1]**o+(-0.823707232663)*x_ref[1]
        ref[0,3,1,1]=(0.700305425892)*x_ref[0]**o+(0.0299761521094)*x_ref[0]+(-0.993032151025)*x_ref[1]**o+(0.633293413221)*x_ref[1]
        ref[0,3,2,0]=(-0.745357256365)*x_ref[0]**o+(-0.451553456961)*x_ref[0]+(0.05789646302)*x_ref[1]**o+(-0.755901155251)*x_ref[1]
        ref[0,3,2,1]=(-0.57330840403)*x_ref[0]**o+(-0.589361483751)*x_ref[0]+(-0.038677123513)*x_ref[1]**o+(0.417332032168)*x_ref[1]
        ref[0,4,0,0]=(0.759655161531)*x_ref[0]**o+(0.737483461328)*x_ref[0]+(0.573158629937)*x_ref[1]**o+(-0.454191109106)*x_ref[1]
        ref[0,4,0,1]=(-0.232982406645)*x_ref[0]**o+(-0.926030038179)*x_ref[0]+(-0.504256834465)*x_ref[1]**o+(-0.107703527058)*x_ref[1]
        ref[0,4,1,0]=(0.898411697016)*x_ref[0]**o+(0.642045536742)*x_ref[0]+(-0.953059791895)*x_ref[1]**o+(-0.67501710093)*x_ref[1]
        ref[0,4,1,1]=(0.300380574165)*x_ref[0]**o+(0.204591960115)*x_ref[0]+(-0.308849697562)*x_ref[1]**o+(-0.489495887357)*x_ref[1]
        ref[0,4,2,0]=(-0.814560286716)*x_ref[0]**o+(0.154235599953)*x_ref[0]+(0.279329273498)*x_ref[1]**o+(-0.674063978926)*x_ref[1]
        ref[0,4,2,1]=(-0.718018870365)*x_ref[0]**o+(-0.735982858424)*x_ref[0]+(-0.0132197132926)*x_ref[1]**o+(0.398939587543)*x_ref[1]
        ref[1,0,0,0]=(0.0559995351725)*x_ref[0]**o+(-0.164549619892)*x_ref[0]+(-0.794242529408)*x_ref[1]**o+(0.0927822314785)*x_ref[1]
        ref[1,0,0,1]=(-0.894238542341)*x_ref[0]**o+(0.378559247473)*x_ref[0]+(-0.183957272639)*x_ref[1]**o+(0.486358968942)*x_ref[1]
        ref[1,0,1,0]=(0.931288801001)*x_ref[0]**o+(0.223815769968)*x_ref[0]+(-0.719591906328)*x_ref[1]**o+(-0.743644741105)*x_ref[1]
        ref[1,0,1,1]=(0.156646029123)*x_ref[0]**o+(0.306942879419)*x_ref[0]+(0.747006830674)*x_ref[1]**o+(-0.876534912274)*x_ref[1]
        ref[1,0,2,0]=(-0.496819081999)*x_ref[0]**o+(-0.939227402425)*x_ref[0]+(0.0566331712182)*x_ref[1]**o+(0.565724364661)*x_ref[1]
        ref[1,0,2,1]=(0.903706429954)*x_ref[0]**o+(0.667947320307)*x_ref[0]+(0.97140855373)*x_ref[1]**o+(0.924095235752)*x_ref[1]
        ref[1,1,0,0]=(0.526180197299)*x_ref[0]**o+(0.435493351674)*x_ref[0]+(0.400107077848)*x_ref[1]**o+(-0.704760833758)*x_ref[1]
        ref[1,1,0,1]=(0.0723740015004)*x_ref[0]**o+(-0.405859051371)*x_ref[0]+(0.200787757511)*x_ref[1]**o+(0.41819184563)*x_ref[1]
        ref[1,1,1,0]=(-0.0320951655627)*x_ref[0]**o+(0.235068445821)*x_ref[0]+(0.450383349599)*x_ref[1]**o+(-0.801487424765)*x_ref[1]
        ref[1,1,1,1]=(-0.52478892744)*x_ref[0]**o+(-0.449111297125)*x_ref[0]+(0.731062182014)*x_ref[1]**o+(0.533619608577)*x_ref[1]
        ref[1,1,2,0]=(0.0889794057057)*x_ref[0]**o+(-0.971856981571)*x_ref[0]+(0.662216408922)*x_ref[1]**o+(0.164089024468)*x_ref[1]
        ref[1,1,2,1]=(-0.196374963647)*x_ref[0]**o+(-0.739360211802)*x_ref[0]+(0.916611650753)*x_ref[1]**o+(-0.260102318115)*x_ref[1]
        ref[1,2,0,0]=(0.627763572103)*x_ref[0]**o+(-0.893900847319)*x_ref[0]+(0.907548446417)*x_ref[1]**o+(0.0687906203663)*x_ref[1]
        ref[1,2,0,1]=(0.59755498682)*x_ref[0]**o+(-0.457108009787)*x_ref[0]+(0.71562139654)*x_ref[1]**o+(-0.300872832559)*x_ref[1]
        ref[1,2,1,0]=(-0.479641829834)*x_ref[0]**o+(0.297188513722)*x_ref[0]+(0.0227730978493)*x_ref[1]**o+(0.199329665406)*x_ref[1]
        ref[1,2,1,1]=(0.918974940101)*x_ref[0]**o+(0.595194981816)*x_ref[0]+(0.760030152554)*x_ref[1]**o+(-0.798899878486)*x_ref[1]
        ref[1,2,2,0]=(-0.878334511842)*x_ref[0]**o+(0.910857236287)*x_ref[0]+(0.0164747567997)*x_ref[1]**o+(-0.876970888445)*x_ref[1]
        ref[1,2,2,1]=(0.29445809181)*x_ref[0]**o+(0.837849454931)*x_ref[0]+(0.0759054371211)*x_ref[1]**o+(0.501243080372)*x_ref[1]
        ref[1,3,0,0]=(-0.529006436258)*x_ref[0]**o+(-0.932325855538)*x_ref[0]+(-0.561472972374)*x_ref[1]**o+(0.845125736656)*x_ref[1]
        ref[1,3,0,1]=(-0.843008257488)*x_ref[0]**o+(0.614118623012)*x_ref[0]+(-0.167015137447)*x_ref[1]**o+(0.158701952625)*x_ref[1]
        ref[1,3,1,0]=(-0.662966916148)*x_ref[0]**o+(-0.443297576512)*x_ref[0]+(0.295988608077)*x_ref[1]**o+(0.059506908615)*x_ref[1]
        ref[1,3,1,1]=(0.864199978014)*x_ref[0]**o+(0.138723519602)*x_ref[0]+(0.144964839302)*x_ref[1]**o+(0.239863709389)*x_ref[1]
        ref[1,3,2,0]=(0.479006097431)*x_ref[0]**o+(0.542168797821)*x_ref[0]+(0.561074562163)*x_ref[1]**o+(-0.133365787275)*x_ref[1]
        ref[1,3,2,1]=(-0.901033449452)*x_ref[0]**o+(-0.620245150074)*x_ref[0]+(0.598461569697)*x_ref[1]**o+(-0.2213246854)*x_ref[1]
        ref[1,4,0,0]=(-0.577067401348)*x_ref[0]**o+(0.486154324874)*x_ref[0]+(0.809205150442)*x_ref[1]**o+(-0.218206418459)*x_ref[1]
        ref[1,4,0,1]=(0.726556862066)*x_ref[0]**o+(0.0293695509867)*x_ref[0]+(0.601938348193)*x_ref[1]**o+(0.967332697531)*x_ref[1]
        ref[1,4,1,0]=(-0.675624388054)*x_ref[0]**o+(-0.871613723214)*x_ref[0]+(0.605519387693)*x_ref[1]**o+(0.287186894411)*x_ref[1]
        ref[1,4,1,1]=(-0.689904898214)*x_ref[0]**o+(-0.918236705665)*x_ref[0]+(-0.892168622177)*x_ref[1]**o+(0.507441662612)*x_ref[1]
        ref[1,4,2,0]=(0.845768980712)*x_ref[0]**o+(-0.458467508114)*x_ref[0]+(-0.518185249391)*x_ref[1]**o+(0.829945330327)*x_ref[1]
        ref[1,4,2,1]=(0.776852312178)*x_ref[0]**o+(-0.268810999469)*x_ref[0]+(-0.576757148841)*x_ref[1]**o+(-0.50628294802)*x_ref[1]
        ref[2,0,0,0]=(-0.686840219939)*x_ref[0]**o+(0.6861172847)*x_ref[0]+(0.240362263052)*x_ref[1]**o+(0.962361523884)*x_ref[1]
        ref[2,0,0,1]=(-0.943837188119)*x_ref[0]**o+(0.829832768268)*x_ref[0]+(0.377590124133)*x_ref[1]**o+(-0.171714144962)*x_ref[1]
        ref[2,0,1,0]=(0.862809148447)*x_ref[0]**o+(0.0573402136489)*x_ref[0]+(-0.743311826013)*x_ref[1]**o+(0.427809313437)*x_ref[1]
        ref[2,0,1,1]=(0.377380819576)*x_ref[0]**o+(-0.223280285041)*x_ref[0]+(0.178628658562)*x_ref[1]**o+(0.121964611877)*x_ref[1]
        ref[2,0,2,0]=(0.41723758689)*x_ref[0]**o+(0.256039203883)*x_ref[0]+(0.255358776676)*x_ref[1]**o+(-0.881665964974)*x_ref[1]
        ref[2,0,2,1]=(-0.0797103024499)*x_ref[0]**o+(0.128068931414)*x_ref[0]+(-0.758317834409)*x_ref[1]**o+(0.654657809183)*x_ref[1]
        ref[2,1,0,0]=(0.603648218133)*x_ref[0]**o+(-0.286780270729)*x_ref[0]+(-0.341108067214)*x_ref[1]**o+(-0.769667356087)*x_ref[1]
        ref[2,1,0,1]=(0.844044270035)*x_ref[0]**o+(0.0585907857764)*x_ref[0]+(-0.578232785252)*x_ref[1]**o+(-0.40959017859)*x_ref[1]
        ref[2,1,1,0]=(0.599709567393)*x_ref[0]**o+(0.307092449739)*x_ref[0]+(-0.402950586215)*x_ref[1]**o+(0.0674239470451)*x_ref[1]
        ref[2,1,1,1]=(0.182608249961)*x_ref[0]**o+(0.373533584601)*x_ref[0]+(0.40949869082)*x_ref[1]**o+(0.696333617699)*x_ref[1]
        ref[2,1,2,0]=(0.0462891111054)*x_ref[0]**o+(0.371258293292)*x_ref[0]+(0.782976187716)*x_ref[1]**o+(0.117976017999)*x_ref[1]
        ref[2,1,2,1]=(0.891841121526)*x_ref[0]**o+(-0.998971173204)*x_ref[0]+(0.0286712507928)*x_ref[1]**o+(-0.0198507022516)*x_ref[1]
        ref[2,2,0,0]=(0.284620902703)*x_ref[0]**o+(0.936231203073)*x_ref[0]+(0.829209752046)*x_ref[1]**o+(-0.94940982125)*x_ref[1]
        ref[2,2,0,1]=(-0.647956027376)*x_ref[0]**o+(0.923690718922)*x_ref[0]+(-0.0322423837346)*x_ref[1]**o+(0.124962508606)*x_ref[1]
        ref[2,2,1,0]=(-0.0696010411705)*x_ref[0]**o+(-0.161672936739)*x_ref[0]+(-0.997445762186)*x_ref[1]**o+(0.177962302014)*x_ref[1]
        ref[2,2,1,1]=(-0.897657700296)*x_ref[0]**o+(-0.467866721175)*x_ref[0]+(-0.560034332007)*x_ref[1]**o+(-0.861509898295)*x_ref[1]
        ref[2,2,2,0]=(-0.100880174048)*x_ref[0]**o+(0.678568254695)*x_ref[0]+(-0.83289411684)*x_ref[1]**o+(0.680564393149)*x_ref[1]
        ref[2,2,2,1]=(0.546159181491)*x_ref[0]**o+(0.720296622966)*x_ref[0]+(0.939046941118)*x_ref[1]**o+(-0.716147554733)*x_ref[1]
        ref[2,3,0,0]=(-0.921073162235)*x_ref[0]**o+(0.979925882289)*x_ref[0]+(-0.0938642966949)*x_ref[1]**o+(0.482093047223)*x_ref[1]
        ref[2,3,0,1]=(0.0709940956822)*x_ref[0]**o+(-0.691401017243)*x_ref[0]+(0.447000814029)*x_ref[1]**o+(0.0575856240533)*x_ref[1]
        ref[2,3,1,0]=(0.562800393365)*x_ref[0]**o+(0.657646549915)*x_ref[0]+(-0.835326361689)*x_ref[1]**o+(0.3487960687)*x_ref[1]
        ref[2,3,1,1]=(0.106668867409)*x_ref[0]**o+(0.436327240765)*x_ref[0]+(-0.585051261259)*x_ref[1]**o+(0.616655338667)*x_ref[1]
        ref[2,3,2,0]=(-0.501849628176)*x_ref[0]**o+(-0.284722503916)*x_ref[0]+(-0.892470199837)*x_ref[1]**o+(-0.979942541795)*x_ref[1]
        ref[2,3,2,1]=(0.236759540073)*x_ref[0]**o+(-0.806931113769)*x_ref[0]+(-0.225220003475)*x_ref[1]**o+(-0.95183239844)*x_ref[1]
        ref[2,4,0,0]=(-0.684823909581)*x_ref[0]**o+(0.445615228059)*x_ref[0]+(-0.638879895767)*x_ref[1]**o+(-0.335505874088)*x_ref[1]
        ref[2,4,0,1]=(-0.196708460612)*x_ref[0]**o+(0.981592996809)*x_ref[0]+(0.435774317028)*x_ref[1]**o+(-0.0952200663109)*x_ref[1]
        ref[2,4,1,0]=(-0.552759474764)*x_ref[0]**o+(-0.614031173545)*x_ref[0]+(0.304375542871)*x_ref[1]**o+(-0.285131901533)*x_ref[1]
        ref[2,4,1,1]=(0.147812951421)*x_ref[0]**o+(-0.78533487681)*x_ref[0]+(0.942205524401)*x_ref[1]**o+(0.166280215075)*x_ref[1]
        ref[2,4,2,0]=(0.843067982563)*x_ref[0]**o+(0.0116520710289)*x_ref[0]+(0.927948889059)*x_ref[1]**o+(-0.395251919577)*x_ref[1]
        ref[2,4,2,1]=(-0.316538010007)*x_ref[0]**o+(-0.139262497126)*x_ref[0]+(0.645409249217)*x_ref[1]**o+(0.192949521992)*x_ref[1]
        ref[3,0,0,0]=(0.88179290666)*x_ref[0]**o+(0.337853287365)*x_ref[0]+(-0.590219587193)*x_ref[1]**o+(-0.827051504196)*x_ref[1]
        ref[3,0,0,1]=(-0.104526336997)*x_ref[0]**o+(0.299097012282)*x_ref[0]+(-0.348493995853)*x_ref[1]**o+(-0.780034322304)*x_ref[1]
        ref[3,0,1,0]=(0.358724159445)*x_ref[0]**o+(-0.0338089482488)*x_ref[0]+(-0.0423054506456)*x_ref[1]**o+(0.67180489557)*x_ref[1]
        ref[3,0,1,1]=(-0.778880095665)*x_ref[0]**o+(-0.448812173928)*x_ref[0]+(-0.823686122382)*x_ref[1]**o+(0.636028922656)*x_ref[1]
        ref[3,0,2,0]=(0.146891002069)*x_ref[0]**o+(-0.368903199628)*x_ref[0]+(-0.596272077557)*x_ref[1]**o+(0.703510683225)*x_ref[1]
        ref[3,0,2,1]=(0.357416423547)*x_ref[0]**o+(-0.103754905142)*x_ref[0]+(0.32026011189)*x_ref[1]**o+(0.755648597627)*x_ref[1]
        ref[3,1,0,0]=(0.399082464238)*x_ref[0]**o+(-0.12611921797)*x_ref[0]+(0.426552225948)*x_ref[1]**o+(-0.228714297286)*x_ref[1]
        ref[3,1,0,1]=(-0.638411026717)*x_ref[0]**o+(-0.342143922306)*x_ref[0]+(-0.502107802626)*x_ref[1]**o+(-0.881045304848)*x_ref[1]
        ref[3,1,1,0]=(0.411134247295)*x_ref[0]**o+(-0.216585727086)*x_ref[0]+(0.270013339156)*x_ref[1]**o+(0.251581243494)*x_ref[1]
        ref[3,1,1,1]=(0.788758848294)*x_ref[0]**o+(0.345181621902)*x_ref[0]+(0.102843710598)*x_ref[1]**o+(0.0561071077516)*x_ref[1]
        ref[3,1,2,0]=(0.232441162347)*x_ref[0]**o+(-0.687544250386)*x_ref[0]+(0.369065746372)*x_ref[1]**o+(0.628109564412)*x_ref[1]
        ref[3,1,2,1]=(-0.912295044312)*x_ref[0]**o+(0.847830497468)*x_ref[0]+(0.776648618003)*x_ref[1]**o+(0.522638388084)*x_ref[1]
        ref[3,2,0,0]=(0.779333120942)*x_ref[0]**o+(-0.0747119967046)*x_ref[0]+(-0.594363282198)*x_ref[1]**o+(0.513922159444)*x_ref[1]
        ref[3,2,0,1]=(0.877492427404)*x_ref[0]**o+(-0.191727825399)*x_ref[0]+(0.0881296554818)*x_ref[1]**o+(-0.408407503067)*x_ref[1]
        ref[3,2,1,0]=(0.29696771215)*x_ref[0]**o+(-0.914882727138)*x_ref[0]+(0.684156831385)*x_ref[1]**o+(0.142293411327)*x_ref[1]
        ref[3,2,1,1]=(0.105927449304)*x_ref[0]**o+(-0.863339661598)*x_ref[0]+(-0.815550662607)*x_ref[1]**o+(-0.261034715274)*x_ref[1]
        ref[3,2,2,0]=(0.944082529871)*x_ref[0]**o+(-0.449259627627)*x_ref[0]+(-0.560526159016)*x_ref[1]**o+(0.609050500798)*x_ref[1]
        ref[3,2,2,1]=(0.529508870267)*x_ref[0]**o+(0.627216712239)*x_ref[0]+(-0.0148122228382)*x_ref[1]**o+(0.0743274653452)*x_ref[1]
        ref[3,3,0,0]=(-0.30609697448)*x_ref[0]**o+(0.910972095956)*x_ref[0]+(0.256274444924)*x_ref[1]**o+(0.229096821635)*x_ref[1]
        ref[3,3,0,1]=(-0.510323627078)*x_ref[0]**o+(0.308418491165)*x_ref[0]+(0.786418296678)*x_ref[1]**o+(-0.248788086479)*x_ref[1]
        ref[3,3,1,0]=(-0.981107944688)*x_ref[0]**o+(-0.331637054173)*x_ref[0]+(-0.972981902152)*x_ref[1]**o+(-0.389364980465)*x_ref[1]
        ref[3,3,1,1]=(-0.839880147367)*x_ref[0]**o+(-0.911569972467)*x_ref[0]+(0.209642332468)*x_ref[1]**o+(0.0926875442137)*x_ref[1]
        ref[3,3,2,0]=(-0.22724314378)*x_ref[0]**o+(-0.0396604383619)*x_ref[0]+(0.906829234649)*x_ref[1]**o+(-0.862410550066)*x_ref[1]
        ref[3,3,2,1]=(-0.412816401935)*x_ref[0]**o+(-0.848442294241)*x_ref[0]+(-0.599302206188)*x_ref[1]**o+(0.719403009819)*x_ref[1]
        ref[3,4,0,0]=(0.259283054664)*x_ref[0]**o+(0.0983272120339)*x_ref[0]+(-0.538373082787)*x_ref[1]**o+(0.950164435617)*x_ref[1]
        ref[3,4,0,1]=(0.633594236794)*x_ref[0]**o+(0.120984469179)*x_ref[0]+(0.472771297982)*x_ref[1]**o+(0.873015785724)*x_ref[1]
        ref[3,4,1,0]=(0.913524970057)*x_ref[0]**o+(0.106590018424)*x_ref[0]+(0.124026494061)*x_ref[1]**o+(-0.254989594971)*x_ref[1]
        ref[3,4,1,1]=(0.945528885492)*x_ref[0]**o+(-0.323205519576)*x_ref[0]+(-0.467294290307)*x_ref[1]**o+(-0.557033486306)*x_ref[1]
        ref[3,4,2,0]=(-0.268364682764)*x_ref[0]**o+(0.0289685280025)*x_ref[0]+(0.0493964636013)*x_ref[1]**o+(0.872120137365)*x_ref[1]
        ref[3,4,2,1]=(-0.266698796897)*x_ref[0]**o+(-0.679148300702)*x_ref[0]+(0.337653394795)*x_ref[1]**o+(0.186866149716)*x_ref[1]
      else:
        arg[0,0,0,0]=(-0.928996629952)*x[0]**o+(-0.171383074114)*x[0]+(-0.324423798901)*x[1]**o+(-0.801354053962)*x[1]+(0.365256142808)*x[2]**o+(0.126262208857)*x[2]
        arg[0,0,0,1]=(0.326024843626)*x[0]**o+(-0.0091566240426)*x[0]+(-0.0396235158589)*x[1]**o+(-0.753302577837)*x[1]+(-0.38709916587)*x[2]**o+(-0.28660316595)*x[2]
        arg[0,0,1,0]=(0.755338255506)*x[0]**o+(0.851895434343)*x[0]+(0.419119464402)*x[1]**o+(0.670733085023)*x[1]+(0.819398310958)*x[2]**o+(0.476185335773)*x[2]
        arg[0,0,1,1]=(-0.981416259131)*x[0]**o+(-0.97000252267)*x[0]+(-0.132624451699)*x[1]**o+(-0.261428370649)*x[1]+(-0.349245951727)*x[2]**o+(-0.717343195877)*x[2]
        arg[0,0,2,0]=(-0.874826075266)*x[0]**o+(-0.399307905707)*x[0]+(0.350873599317)*x[1]**o+(0.15861651992)*x[1]+(0.396279933216)*x[2]**o+(0.92332515708)*x[2]
        arg[0,0,2,1]=(-0.183596683527)*x[0]**o+(-0.115194639348)*x[0]+(0.616111723618)*x[1]**o+(-0.402157657669)*x[1]+(0.44145594193)*x[2]**o+(0.686052407784)*x[2]
        arg[0,1,0,0]=(-0.669169178244)*x[0]**o+(0.835937801609)*x[0]+(0.468805302181)*x[1]**o+(-0.627292634023)*x[1]+(-0.674361231979)*x[2]**o+(-0.00929181785129)*x[2]
        arg[0,1,0,1]=(-0.568131655705)*x[0]**o+(0.690366535852)*x[0]+(-0.212524396914)*x[1]**o+(-0.767983769407)*x[1]+(0.477500128569)*x[2]**o+(-0.0190887598644)*x[2]
        arg[0,1,1,0]=(-0.374170635822)*x[0]**o+(0.0169009974558)*x[0]+(-0.162519103357)*x[1]**o+(0.786991144449)*x[1]+(0.0590294918131)*x[2]**o+(-0.213442509309)*x[2]
        arg[0,1,1,1]=(-0.558268657439)*x[0]**o+(0.584734933669)*x[0]+(-0.650790013689)*x[1]**o+(-0.192849048407)*x[1]+(0.505458636714)*x[2]**o+(-0.585174281371)*x[2]
        arg[0,1,2,0]=(-0.933243590775)*x[0]**o+(0.528060249767)*x[0]+(-0.763255746252)*x[1]**o+(0.771589118163)*x[1]+(-0.641287604338)*x[2]**o+(0.379566401585)*x[2]
        arg[0,1,2,1]=(-0.836976373935)*x[0]**o+(-0.969434110974)*x[0]+(0.758148372888)*x[1]**o+(-0.89212062535)*x[1]+(0.788693820015)*x[2]**o+(-0.620704367414)*x[2]
        arg[0,2,0,0]=(0.0163117360814)*x[0]**o+(-0.390867752068)*x[0]+(-0.725068297982)*x[1]**o+(-0.680312496646)*x[1]+(0.069273494969)*x[2]**o+(-0.576122275696)*x[2]
        arg[0,2,0,1]=(0.461771197614)*x[0]**o+(-0.767541404109)*x[0]+(-0.694279263622)*x[1]**o+(-0.721448812749)*x[1]+(0.248787896584)*x[2]**o+(0.237075401327)*x[2]
        arg[0,2,1,0]=(0.381987012965)*x[0]**o+(-0.288236233238)*x[0]+(-0.519012025084)*x[1]**o+(-0.256658293082)*x[1]+(0.807322167368)*x[2]**o+(-0.261916179859)*x[2]
        arg[0,2,1,1]=(0.257241267407)*x[0]**o+(-0.470612990721)*x[0]+(-0.208612656448)*x[1]**o+(0.839646962914)*x[1]+(0.916675150494)*x[2]**o+(-0.165468351005)*x[2]
        arg[0,2,2,0]=(0.92569232521)*x[0]**o+(0.809982458308)*x[0]+(-0.860530305336)*x[1]**o+(0.261687622817)*x[1]+(0.948003684651)*x[2]**o+(-0.473133742361)*x[2]
        arg[0,2,2,1]=(-0.113846814874)*x[0]**o+(0.680846459377)*x[0]+(0.920219149741)*x[1]**o+(0.172027554503)*x[1]+(0.475330041115)*x[2]**o+(-0.176218891429)*x[2]
        arg[0,3,0,0]=(-0.948638294297)*x[0]**o+(-0.0411621070703)*x[0]+(-0.294759271027)*x[1]**o+(-0.903733455086)*x[1]+(-0.995253733593)*x[2]**o+(0.106726960462)*x[2]
        arg[0,3,0,1]=(0.1592709614)*x[0]**o+(0.0683266445293)*x[0]+(0.336092506745)*x[1]**o+(-0.209047739881)*x[1]+(-0.330417243459)*x[2]**o+(-0.809242647755)*x[2]
        arg[0,3,1,0]=(-0.680637516359)*x[0]**o+(-0.713405815404)*x[0]+(-0.484624435043)*x[1]**o+(-0.279314462332)*x[1]+(-0.458856066504)*x[2]**o+(0.0588111305227)*x[2]
        arg[0,3,1,1]=(-0.853676338655)*x[0]**o+(-0.651539814195)*x[0]+(-0.109727277725)*x[1]**o+(0.0139006407942)*x[1]+(0.231943245426)*x[2]**o+(0.42972517233)*x[2]
        arg[0,3,2,0]=(-0.60799269765)*x[0]**o+(-0.802413269128)*x[0]+(0.151362498863)*x[1]**o+(-0.678226252875)*x[1]+(-0.867117685659)*x[2]**o+(-0.711982116071)*x[2]
        arg[0,3,2,1]=(-0.243012743941)*x[0]**o+(-0.692614579016)*x[0]+(0.0986817641914)*x[1]**o+(-0.495735862695)*x[1]+(-0.430040681653)*x[2]**o+(0.0108928722284)*x[2]
        arg[0,4,0,0]=(0.0989402181558)*x[0]**o+(-0.60096719882)*x[0]+(-0.324253928432)*x[1]**o+(-0.862723687559)*x[1]+(-0.34745416427)*x[2]**o+(0.936559074873)*x[2]
        arg[0,4,0,1]=(0.711021967566)*x[0]**o+(-0.939408401113)*x[0]+(-0.218100921633)*x[1]**o+(0.92835194875)*x[1]+(-0.353936913865)*x[2]**o+(-0.547266090928)*x[2]
        arg[0,4,1,0]=(0.172231614188)*x[0]**o+(-0.734622090671)*x[0]+(0.432686916647)*x[1]**o+(0.552595350492)*x[1]+(0.779662641076)*x[2]**o+(-0.0933220492779)*x[2]
        arg[0,4,1,1]=(0.996776716866)*x[0]**o+(-0.340786520686)*x[0]+(0.260075214171)*x[1]**o+(0.00385504852496)*x[1]+(0.377516772127)*x[2]**o+(-0.341751634181)*x[2]
        arg[0,4,2,0]=(-0.655642896664)*x[0]**o+(-0.840525173737)*x[0]+(-0.855343721564)*x[1]**o+(0.253558571524)*x[1]+(0.0463733059925)*x[2]**o+(-0.116359866912)*x[2]
        arg[0,4,2,1]=(0.131619661414)*x[0]**o+(-0.162067255954)*x[0]+(0.562536831367)*x[1]**o+(-0.940412472035)*x[1]+(-0.282265762783)*x[2]**o+(0.634711607923)*x[2]
        arg[1,0,0,0]=(0.484071808097)*x[0]**o+(0.566763524691)*x[0]+(-0.658614173653)*x[1]**o+(-0.140192232895)*x[1]+(-0.310114236409)*x[2]**o+(-0.918532401595)*x[2]
        arg[1,0,0,1]=(-0.81190380441)*x[0]**o+(0.644439878497)*x[0]+(0.479583403289)*x[1]**o+(-0.493318882779)*x[1]+(-0.424704492175)*x[2]**o+(0.700624317732)*x[2]
        arg[1,0,1,0]=(0.766650182435)*x[0]**o+(0.226365545129)*x[0]+(0.510828945971)*x[1]**o+(0.283221954498)*x[1]+(0.817303877886)*x[2]**o+(-0.245709044628)*x[2]
        arg[1,0,1,1]=(0.194787112664)*x[0]**o+(-0.879306736922)*x[0]+(0.185381291627)*x[1]**o+(-0.510692743975)*x[1]+(-0.422555014055)*x[2]**o+(0.556629919959)*x[2]
        arg[1,0,2,0]=(-0.958345828895)*x[0]**o+(-0.667735551013)*x[0]+(0.278271284225)*x[1]**o+(-0.0847995194757)*x[1]+(0.902184171441)*x[2]**o+(0.763125168854)*x[2]
        arg[1,0,2,1]=(-0.0766429670195)*x[0]**o+(0.894421139029)*x[0]+(-0.331560314101)*x[1]**o+(-0.0253297623997)*x[1]+(0.13406045967)*x[2]**o+(-0.326681111625)*x[2]
        arg[1,1,0,0]=(-0.979406855838)*x[0]**o+(0.528849909402)*x[0]+(-0.0486159476737)*x[1]**o+(-0.898392512695)*x[1]+(-0.678328327407)*x[2]**o+(0.644201802057)*x[2]
        arg[1,1,0,1]=(-0.977485554605)*x[0]**o+(0.794799941538)*x[0]+(-0.00237183848504)*x[1]**o+(-0.633497483149)*x[1]+(0.746593853568)*x[2]**o+(0.197741126365)*x[2]
        arg[1,1,1,0]=(0.379985506182)*x[0]**o+(0.69889144234)*x[0]+(-0.507293887409)*x[1]**o+(-0.727675545291)*x[1]+(-0.467064531693)*x[2]**o+(0.130273930089)*x[2]
        arg[1,1,1,1]=(-0.330627830819)*x[0]**o+(-0.0311542935889)*x[0]+(-0.0671494148749)*x[1]**o+(0.583605469163)*x[1]+(-0.172083996069)*x[2]**o+(0.666718011074)*x[2]
        arg[1,1,2,0]=(0.539618381063)*x[0]**o+(-0.546796237888)*x[0]+(0.857911750392)*x[1]**o+(0.119552731463)*x[1]+(0.329286820109)*x[2]**o+(0.637240649087)*x[2]
        arg[1,1,2,1]=(-0.828603071334)*x[0]**o+(0.861799149065)*x[0]+(0.615224040538)*x[1]**o+(-0.0868985543263)*x[1]+(-0.206168997809)*x[2]**o+(0.311103719286)*x[2]
        arg[1,2,0,0]=(0.695106867715)*x[0]**o+(-0.533303418727)*x[0]+(-0.393936955207)*x[1]**o+(-0.407635261686)*x[1]+(0.604346269147)*x[2]**o+(-0.10213061533)*x[2]
        arg[1,2,0,1]=(0.338035309929)*x[0]**o+(0.517135412671)*x[0]+(0.277863735829)*x[1]**o+(-0.703417079398)*x[1]+(0.369398635984)*x[2]**o+(-0.952905070601)*x[2]
        arg[1,2,1,0]=(0.937866603297)*x[0]**o+(0.436121631708)*x[0]+(0.568362098461)*x[1]**o+(0.223837722984)*x[1]+(-0.236896910941)*x[2]**o+(0.316114408308)*x[2]
        arg[1,2,1,1]=(0.285714491932)*x[0]**o+(0.633321537987)*x[0]+(0.953737237202)*x[1]**o+(0.732065555924)*x[1]+(-0.515759068197)*x[2]**o+(-0.519775334889)*x[2]
        arg[1,2,2,0]=(-0.0610222303479)*x[0]**o+(0.135974983944)*x[0]+(0.450213393447)*x[1]**o+(-0.44747541715)*x[1]+(-0.640660207187)*x[2]**o+(0.348457261119)*x[2]
        arg[1,2,2,1]=(0.131753033369)*x[0]**o+(-0.311359549998)*x[0]+(0.445707767235)*x[1]**o+(-0.710795569938)*x[1]+(-0.746038648477)*x[2]**o+(-0.210414773188)*x[2]
        arg[1,3,0,0]=(-0.323684061289)*x[0]**o+(-0.0937091220282)*x[0]+(-0.827884898323)*x[1]**o+(0.604442907568)*x[1]+(0.620923596379)*x[2]**o+(0.851758170763)*x[2]
        arg[1,3,0,1]=(-0.269239872573)*x[0]**o+(-0.603248280988)*x[0]+(0.603804530326)*x[1]**o+(-0.331910095046)*x[1]+(-0.653877236308)*x[2]**o+(0.115434865132)*x[2]
        arg[1,3,1,0]=(0.0531798412869)*x[0]**o+(0.292950346899)*x[0]+(-0.634376081823)*x[1]**o+(-0.775427771578)*x[1]+(-0.987113467149)*x[2]**o+(0.757980444447)*x[2]
        arg[1,3,1,1]=(0.757807292214)*x[0]**o+(0.81778478971)*x[0]+(-0.619873148405)*x[1]**o+(0.0129855659138)*x[1]+(-0.282861093681)*x[2]**o+(0.667211104045)*x[2]
        arg[1,3,2,0]=(-0.915205606853)*x[0]**o+(0.918704643459)*x[0]+(0.143238155504)*x[1]**o+(0.0285686377214)*x[1]+(-0.848712843243)*x[2]**o+(0.058647124862)*x[2]
        arg[1,3,2,1]=(0.441487712831)*x[0]**o+(-0.626878272507)*x[0]+(-0.862579662295)*x[1]**o+(0.204629243832)*x[1]+(0.964143347265)*x[2]**o+(-0.399125844282)*x[2]
        arg[1,4,0,0]=(-0.349791275126)*x[0]**o+(-0.140293844776)*x[0]+(-0.693879671138)*x[1]**o+(0.429290435637)*x[1]+(-0.892670314048)*x[2]**o+(-0.380777380409)*x[2]
        arg[1,4,0,1]=(0.528712424647)*x[0]**o+(0.449268002671)*x[0]+(-0.202472214961)*x[1]**o+(-0.374762208851)*x[1]+(-0.625590920648)*x[2]**o+(-0.000776205555531)*x[2]
        arg[1,4,1,0]=(0.00425881397184)*x[0]**o+(0.265831686248)*x[0]+(0.757626547866)*x[1]**o+(-0.934207064756)*x[1]+(0.541356932538)*x[2]**o+(0.217841597089)*x[2]
        arg[1,4,1,1]=(-0.02422207106)*x[0]**o+(-0.807018149115)*x[0]+(-0.402347046418)*x[1]**o+(-0.0548851919048)*x[1]+(0.850202044077)*x[2]**o+(-0.889576261979)*x[2]
        arg[1,4,2,0]=(0.487059068561)*x[0]**o+(0.338254525003)*x[0]+(-0.387019663154)*x[1]**o+(-0.436969552236)*x[1]+(-0.747690766603)*x[2]**o+(0.265854907003)*x[2]
        arg[1,4,2,1]=(0.687562221132)*x[0]**o+(0.4755782976)*x[0]+(-0.456287111804)*x[1]**o+(0.412138123928)*x[1]+(0.637920334647)*x[2]**o+(0.817007725525)*x[2]
        arg[2,0,0,0]=(-0.728166422776)*x[0]**o+(0.108904177195)*x[0]+(0.383848108635)*x[1]**o+(-0.19252707292)*x[1]+(-0.279319122495)*x[2]**o+(-0.0530727745585)*x[2]
        arg[2,0,0,1]=(0.380827452272)*x[0]**o+(-0.133776280687)*x[0]+(-0.174731551283)*x[1]**o+(-0.369231086652)*x[1]+(-0.0190771818046)*x[2]**o+(-0.331873068919)*x[2]
        arg[2,0,1,0]=(0.653644298826)*x[0]**o+(-0.916110182689)*x[0]+(0.0510855252492)*x[1]**o+(-0.934192949208)*x[1]+(0.572048462784)*x[2]**o+(-0.451401856863)*x[2]
        arg[2,0,1,1]=(-0.394958497502)*x[0]**o+(-0.690752972227)*x[0]+(-0.436352069349)*x[1]**o+(0.933524576573)*x[1]+(-0.524344161699)*x[2]**o+(0.526748792776)*x[2]
        arg[2,0,2,0]=(0.845649523782)*x[0]**o+(0.63927747558)*x[0]+(-0.534192218216)*x[1]**o+(0.714500925459)*x[1]+(0.156077814898)*x[2]**o+(-0.193712778792)*x[2]
        arg[2,0,2,1]=(0.809636640413)*x[0]**o+(-0.930654037481)*x[0]+(0.509441692942)*x[1]**o+(0.881563824585)*x[1]+(0.101333749057)*x[2]**o+(0.418791722704)*x[2]
        arg[2,1,0,0]=(0.289481281745)*x[0]**o+(-0.98279193802)*x[0]+(-0.443382747319)*x[1]**o+(-0.281710992472)*x[1]+(0.426349350391)*x[2]**o+(-0.870192121251)*x[2]
        arg[2,1,0,1]=(0.837046002753)*x[0]**o+(-0.0254166983371)*x[0]+(0.30178635814)*x[1]**o+(0.973915444924)*x[1]+(0.298631524734)*x[2]**o+(-0.422490970248)*x[2]
        arg[2,1,1,0]=(0.869799082394)*x[0]**o+(0.376436513628)*x[0]+(-0.74735679118)*x[1]**o+(0.477184335901)*x[1]+(-0.797472924916)*x[2]**o+(-0.568827873964)*x[2]
        arg[2,1,1,1]=(0.993441009779)*x[0]**o+(-0.264888299493)*x[0]+(0.861197560367)*x[1]**o+(-0.769058084144)*x[1]+(-0.86221947415)*x[2]**o+(-0.617024169672)*x[2]
        arg[2,1,2,0]=(-0.89303691749)*x[0]**o+(-0.390287412175)*x[0]+(-0.312883084291)*x[1]**o+(-0.432368502641)*x[1]+(-0.121476666357)*x[2]**o+(0.416361314288)*x[2]
        arg[2,1,2,1]=(-0.622214426504)*x[0]**o+(-0.595279134338)*x[0]+(0.000927173498715)*x[1]**o+(0.783001179117)*x[1]+(-0.956003738061)*x[2]**o+(0.482321214492)*x[2]
        arg[2,2,0,0]=(-0.0047994639068)*x[0]**o+(0.478805093492)*x[0]+(0.747825846519)*x[1]**o+(0.0756064430807)*x[1]+(-0.470534826525)*x[2]**o+(-0.421524571831)*x[2]
        arg[2,2,0,1]=(-0.0569198078126)*x[0]**o+(-0.36284856463)*x[0]+(-0.374343856016)*x[1]**o+(0.193470734451)*x[1]+(0.425350990791)*x[2]**o+(-0.903309757487)*x[2]
        arg[2,2,1,0]=(0.602075251229)*x[0]**o+(-0.558163518701)*x[0]+(-0.00146699467501)*x[1]**o+(0.437116756605)*x[1]+(-0.252696658318)*x[2]**o+(0.154017654456)*x[2]
        arg[2,2,1,1]=(0.226970288166)*x[0]**o+(0.844076458869)*x[0]+(0.231874137232)*x[1]**o+(-0.270233945335)*x[1]+(0.365979082276)*x[2]**o+(-0.937144802776)*x[2]
        arg[2,2,2,0]=(0.662440813722)*x[0]**o+(-0.600830429698)*x[0]+(0.248436486724)*x[1]**o+(0.73439963812)*x[1]+(-0.406317484259)*x[2]**o+(-0.406641354653)*x[2]
        arg[2,2,2,1]=(0.948630940404)*x[0]**o+(-0.695098513788)*x[0]+(-0.819843883615)*x[1]**o+(0.0479609386435)*x[1]+(-0.968033045837)*x[2]**o+(0.199490071238)*x[2]
        arg[2,3,0,0]=(0.967996364186)*x[0]**o+(0.562562580668)*x[0]+(0.660658005197)*x[1]**o+(-0.00126241998808)*x[1]+(-0.0188273819622)*x[2]**o+(-0.798370566647)*x[2]
        arg[2,3,0,1]=(0.108137383906)*x[0]**o+(0.230815090379)*x[0]+(-0.0817434914408)*x[1]**o+(0.256977068172)*x[1]+(0.323454379512)*x[2]**o+(0.0382049771532)*x[2]
        arg[2,3,1,0]=(-0.279443973124)*x[0]**o+(-0.948014830608)*x[0]+(-0.237787814838)*x[1]**o+(-0.880632864876)*x[1]+(0.305230465688)*x[2]**o+(0.722139326346)*x[2]
        arg[2,3,1,1]=(-0.192752669063)*x[0]**o+(0.763575125553)*x[0]+(-0.317096602795)*x[1]**o+(-0.241479822867)*x[1]+(0.643154065804)*x[2]**o+(0.903250562165)*x[2]
        arg[2,3,2,0]=(0.58153852366)*x[0]**o+(-0.131284078682)*x[0]+(0.803022288461)*x[1]**o+(0.862074208938)*x[1]+(0.934648335783)*x[2]**o+(0.720337200812)*x[2]
        arg[2,3,2,1]=(-0.866755142782)*x[0]**o+(0.299995782189)*x[0]+(0.830022439718)*x[1]**o+(0.787775630269)*x[1]+(-0.436980900353)*x[2]**o+(0.436719598692)*x[2]
        arg[2,4,0,0]=(-0.416946111972)*x[0]**o+(-0.342692819134)*x[0]+(-0.265712927184)*x[1]**o+(-0.503154497528)*x[1]+(0.436447837569)*x[2]**o+(-0.514226068117)*x[2]
        arg[2,4,0,1]=(-0.915995315837)*x[0]**o+(-0.562870475868)*x[0]+(0.158881525561)*x[1]**o+(-0.458461594412)*x[1]+(-0.117927681987)*x[2]**o+(-0.697751714189)*x[2]
        arg[2,4,1,0]=(-0.961291698815)*x[0]**o+(0.457487466485)*x[0]+(0.170987701633)*x[1]**o+(-0.14226156627)*x[1]+(-0.663462703057)*x[2]**o+(-0.0307934454283)*x[2]
        arg[2,4,1,1]=(0.588162542138)*x[0]**o+(-0.761783611043)*x[0]+(0.510241246326)*x[1]**o+(0.805053446677)*x[1]+(-0.851391511147)*x[2]**o+(-0.0976037736598)*x[2]
        arg[2,4,2,0]=(0.649047985858)*x[0]**o+(0.677435498282)*x[0]+(-0.620777977748)*x[1]**o+(-0.209440794016)*x[1]+(0.58009322661)*x[2]**o+(0.56317180284)*x[2]
        arg[2,4,2,1]=(-0.975596464331)*x[0]**o+(0.759364271658)*x[0]+(0.43142780149)*x[1]**o+(-0.701043203786)*x[1]+(-0.648159388134)*x[2]**o+(-0.129618261144)*x[2]
        arg[3,0,0,0]=(-0.0207100058532)*x[0]**o+(-0.829637327952)*x[0]+(-0.0326040806041)*x[1]**o+(0.652320712227)*x[1]+(-0.509435869246)*x[2]**o+(-0.66044392521)*x[2]
        arg[3,0,0,1]=(-0.312677151974)*x[0]**o+(0.0204494340128)*x[0]+(-0.989457740146)*x[1]**o+(-0.0606077687298)*x[1]+(-0.527267756606)*x[2]**o+(0.496006274201)*x[2]
        arg[3,0,1,0]=(0.842871894549)*x[0]**o+(-0.577687338895)*x[0]+(0.414019506131)*x[1]**o+(0.0410521331015)*x[1]+(-0.830482660184)*x[2]**o+(0.688692187619)*x[2]
        arg[3,0,1,1]=(-0.920070939376)*x[0]**o+(0.632972952755)*x[0]+(0.944186412149)*x[1]**o+(0.620319712728)*x[1]+(-0.171433727161)*x[2]**o+(0.267125744191)*x[2]
        arg[3,0,2,0]=(-0.0669535338492)*x[0]**o+(0.249181230746)*x[0]+(-0.147448553328)*x[1]**o+(-0.459701695839)*x[1]+(0.378458172384)*x[2]**o+(0.916594147889)*x[2]
        arg[3,0,2,1]=(0.0720111269432)*x[0]**o+(-0.390032127319)*x[0]+(-0.115565740734)*x[1]**o+(0.501210832815)*x[1]+(0.568129023658)*x[2]**o+(-0.303909080042)*x[2]
        arg[3,1,0,0]=(0.205675480731)*x[0]**o+(-0.843093328795)*x[0]+(-0.812415573635)*x[1]**o+(-0.706806177394)*x[1]+(-0.158442718286)*x[2]**o+(0.15187176613)*x[2]
        arg[3,1,0,1]=(0.149669308017)*x[0]**o+(0.487445503149)*x[0]+(-0.27973905833)*x[1]**o+(-0.504992462225)*x[1]+(-0.728166691522)*x[2]**o+(0.734846267391)*x[2]
        arg[3,1,1,0]=(0.0410578015251)*x[0]**o+(-0.525001495676)*x[0]+(-0.512685697472)*x[1]**o+(-0.284314686987)*x[1]+(0.888112049081)*x[2]**o+(0.425466793802)*x[2]
        arg[3,1,1,1]=(-0.374237047043)*x[0]**o+(-0.00078724690461)*x[0]+(0.849952487348)*x[1]**o+(-0.654945893799)*x[1]+(-0.536739132177)*x[2]**o+(0.20919294066)*x[2]
        arg[3,1,2,0]=(0.685749084068)*x[0]**o+(-0.730378478543)*x[0]+(-0.918072746761)*x[1]**o+(-0.706713872062)*x[1]+(-0.486100957526)*x[2]**o+(0.95736939257)*x[2]
        arg[3,1,2,1]=(-0.287859736321)*x[0]**o+(-0.271882215207)*x[0]+(-0.673357024623)*x[1]**o+(0.702867457627)*x[1]+(0.78445294416)*x[2]**o+(0.613888365259)*x[2]
        arg[3,2,0,0]=(0.568506008135)*x[0]**o+(-0.825947283734)*x[0]+(0.687369924271)*x[1]**o+(-0.866597782569)*x[1]+(-0.332810902355)*x[2]**o+(0.499502049139)*x[2]
        arg[3,2,0,1]=(0.757251773732)*x[0]**o+(0.293337812465)*x[0]+(0.686709285382)*x[1]**o+(-0.084997189402)*x[1]+(-0.338903817381)*x[2]**o+(-0.787646799698)*x[2]
        arg[3,2,1,0]=(-0.580327419983)*x[0]**o+(-0.603952498996)*x[0]+(0.523057988618)*x[1]**o+(-0.0835187394107)*x[1]+(0.459827887147)*x[2]**o+(0.689021201042)*x[2]
        arg[3,2,1,1]=(-0.41923416619)*x[0]**o+(0.0406426957237)*x[0]+(-0.908041008186)*x[1]**o+(0.382097138462)*x[1]+(0.66835706849)*x[2]**o+(0.355849275378)*x[2]
        arg[3,2,2,0]=(-0.133756767108)*x[0]**o+(-0.233743496333)*x[0]+(-0.661420120856)*x[1]**o+(0.788722576584)*x[1]+(-0.131764585707)*x[2]**o+(-0.781004479871)*x[2]
        arg[3,2,2,1]=(0.0308645139784)*x[0]**o+(-0.194986246344)*x[0]+(-0.267418187303)*x[1]**o+(0.332143733168)*x[1]+(0.513266431894)*x[2]**o+(0.594016981473)*x[2]
        arg[3,3,0,0]=(0.644314102613)*x[0]**o+(-0.977355279476)*x[0]+(0.516452323667)*x[1]**o+(0.172873579919)*x[1]+(0.778904819969)*x[2]**o+(0.559689750425)*x[2]
        arg[3,3,0,1]=(0.736135986201)*x[0]**o+(0.366918250826)*x[0]+(-0.0547007988818)*x[1]**o+(-0.830174436433)*x[1]+(0.613440124338)*x[2]**o+(0.574992469591)*x[2]
        arg[3,3,1,0]=(0.324504938146)*x[0]**o+(-0.915240176689)*x[0]+(-0.398105336249)*x[1]**o+(-0.526498343533)*x[1]+(0.43264573336)*x[2]**o+(0.393008086529)*x[2]
        arg[3,3,1,1]=(0.926367438323)*x[0]**o+(0.526364305462)*x[0]+(0.15596582236)*x[1]**o+(0.165158439711)*x[1]+(-0.508505348973)*x[2]**o+(-0.603637833992)*x[2]
        arg[3,3,2,0]=(0.661333046004)*x[0]**o+(0.425320373512)*x[0]+(-0.810809051016)*x[1]**o+(0.20446843023)*x[1]+(0.685136412413)*x[2]**o+(-0.0281127019894)*x[2]
        arg[3,3,2,1]=(0.851742499007)*x[0]**o+(-0.405506974975)*x[0]+(-0.296768983731)*x[1]**o+(0.389557354929)*x[1]+(0.335108341463)*x[2]**o+(0.0313094433351)*x[2]
        arg[3,4,0,0]=(0.0415035498649)*x[0]**o+(0.827106496882)*x[0]+(-0.481831487377)*x[1]**o+(-0.14935685817)*x[1]+(0.354765095566)*x[2]**o+(0.521898910137)*x[2]
        arg[3,4,0,1]=(-0.36328099035)*x[0]**o+(-0.243342386595)*x[0]+(0.0475935188475)*x[1]**o+(0.845308417697)*x[1]+(0.315259620082)*x[2]**o+(0.541313742759)*x[2]
        arg[3,4,1,0]=(-0.66364149672)*x[0]**o+(0.987505978353)*x[0]+(-0.337659775957)*x[1]**o+(0.197277151989)*x[1]+(-0.431260870605)*x[2]**o+(-0.803375909471)*x[2]
        arg[3,4,1,1]=(0.722104365428)*x[0]**o+(0.168416682879)*x[0]+(-0.415740544748)*x[1]**o+(-0.437836696476)*x[1]+(-0.280581752598)*x[2]**o+(0.434326371762)*x[2]
        arg[3,4,2,0]=(0.673144889684)*x[0]**o+(-0.0628529591764)*x[0]+(0.0161458704408)*x[1]**o+(-0.439966404278)*x[1]+(-0.963402070447)*x[2]**o+(0.308312287021)*x[2]
        arg[3,4,2,1]=(0.649131384887)*x[0]**o+(-0.825086043001)*x[0]+(0.204615530143)*x[1]**o+(0.00801105023644)*x[1]+(-0.234754758061)*x[2]**o+(0.659650083864)*x[2]
        ref[0,0,0,0]=(-0.928996629952)*x_ref[0]**o+(-0.171383074114)*x_ref[0]+(-0.324423798901)*x_ref[1]**o+(-0.801354053962)*x_ref[1]+(0.365256142808)*x_ref[2]**o+(0.126262208857)*x_ref[2]
        ref[0,0,0,1]=(0.326024843626)*x_ref[0]**o+(-0.0091566240426)*x_ref[0]+(-0.0396235158589)*x_ref[1]**o+(-0.753302577837)*x_ref[1]+(-0.38709916587)*x_ref[2]**o+(-0.28660316595)*x_ref[2]
        ref[0,0,1,0]=(0.755338255506)*x_ref[0]**o+(0.851895434343)*x_ref[0]+(0.419119464402)*x_ref[1]**o+(0.670733085023)*x_ref[1]+(0.819398310958)*x_ref[2]**o+(0.476185335773)*x_ref[2]
        ref[0,0,1,1]=(-0.981416259131)*x_ref[0]**o+(-0.97000252267)*x_ref[0]+(-0.132624451699)*x_ref[1]**o+(-0.261428370649)*x_ref[1]+(-0.349245951727)*x_ref[2]**o+(-0.717343195877)*x_ref[2]
        ref[0,0,2,0]=(-0.874826075266)*x_ref[0]**o+(-0.399307905707)*x_ref[0]+(0.350873599317)*x_ref[1]**o+(0.15861651992)*x_ref[1]+(0.396279933216)*x_ref[2]**o+(0.92332515708)*x_ref[2]
        ref[0,0,2,1]=(-0.183596683527)*x_ref[0]**o+(-0.115194639348)*x_ref[0]+(0.616111723618)*x_ref[1]**o+(-0.402157657669)*x_ref[1]+(0.44145594193)*x_ref[2]**o+(0.686052407784)*x_ref[2]
        ref[0,1,0,0]=(-0.669169178244)*x_ref[0]**o+(0.835937801609)*x_ref[0]+(0.468805302181)*x_ref[1]**o+(-0.627292634023)*x_ref[1]+(-0.674361231979)*x_ref[2]**o+(-0.00929181785129)*x_ref[2]
        ref[0,1,0,1]=(-0.568131655705)*x_ref[0]**o+(0.690366535852)*x_ref[0]+(-0.212524396914)*x_ref[1]**o+(-0.767983769407)*x_ref[1]+(0.477500128569)*x_ref[2]**o+(-0.0190887598644)*x_ref[2]
        ref[0,1,1,0]=(-0.374170635822)*x_ref[0]**o+(0.0169009974558)*x_ref[0]+(-0.162519103357)*x_ref[1]**o+(0.786991144449)*x_ref[1]+(0.0590294918131)*x_ref[2]**o+(-0.213442509309)*x_ref[2]
        ref[0,1,1,1]=(-0.558268657439)*x_ref[0]**o+(0.584734933669)*x_ref[0]+(-0.650790013689)*x_ref[1]**o+(-0.192849048407)*x_ref[1]+(0.505458636714)*x_ref[2]**o+(-0.585174281371)*x_ref[2]
        ref[0,1,2,0]=(-0.933243590775)*x_ref[0]**o+(0.528060249767)*x_ref[0]+(-0.763255746252)*x_ref[1]**o+(0.771589118163)*x_ref[1]+(-0.641287604338)*x_ref[2]**o+(0.379566401585)*x_ref[2]
        ref[0,1,2,1]=(-0.836976373935)*x_ref[0]**o+(-0.969434110974)*x_ref[0]+(0.758148372888)*x_ref[1]**o+(-0.89212062535)*x_ref[1]+(0.788693820015)*x_ref[2]**o+(-0.620704367414)*x_ref[2]
        ref[0,2,0,0]=(0.0163117360814)*x_ref[0]**o+(-0.390867752068)*x_ref[0]+(-0.725068297982)*x_ref[1]**o+(-0.680312496646)*x_ref[1]+(0.069273494969)*x_ref[2]**o+(-0.576122275696)*x_ref[2]
        ref[0,2,0,1]=(0.461771197614)*x_ref[0]**o+(-0.767541404109)*x_ref[0]+(-0.694279263622)*x_ref[1]**o+(-0.721448812749)*x_ref[1]+(0.248787896584)*x_ref[2]**o+(0.237075401327)*x_ref[2]
        ref[0,2,1,0]=(0.381987012965)*x_ref[0]**o+(-0.288236233238)*x_ref[0]+(-0.519012025084)*x_ref[1]**o+(-0.256658293082)*x_ref[1]+(0.807322167368)*x_ref[2]**o+(-0.261916179859)*x_ref[2]
        ref[0,2,1,1]=(0.257241267407)*x_ref[0]**o+(-0.470612990721)*x_ref[0]+(-0.208612656448)*x_ref[1]**o+(0.839646962914)*x_ref[1]+(0.916675150494)*x_ref[2]**o+(-0.165468351005)*x_ref[2]
        ref[0,2,2,0]=(0.92569232521)*x_ref[0]**o+(0.809982458308)*x_ref[0]+(-0.860530305336)*x_ref[1]**o+(0.261687622817)*x_ref[1]+(0.948003684651)*x_ref[2]**o+(-0.473133742361)*x_ref[2]
        ref[0,2,2,1]=(-0.113846814874)*x_ref[0]**o+(0.680846459377)*x_ref[0]+(0.920219149741)*x_ref[1]**o+(0.172027554503)*x_ref[1]+(0.475330041115)*x_ref[2]**o+(-0.176218891429)*x_ref[2]
        ref[0,3,0,0]=(-0.948638294297)*x_ref[0]**o+(-0.0411621070703)*x_ref[0]+(-0.294759271027)*x_ref[1]**o+(-0.903733455086)*x_ref[1]+(-0.995253733593)*x_ref[2]**o+(0.106726960462)*x_ref[2]
        ref[0,3,0,1]=(0.1592709614)*x_ref[0]**o+(0.0683266445293)*x_ref[0]+(0.336092506745)*x_ref[1]**o+(-0.209047739881)*x_ref[1]+(-0.330417243459)*x_ref[2]**o+(-0.809242647755)*x_ref[2]
        ref[0,3,1,0]=(-0.680637516359)*x_ref[0]**o+(-0.713405815404)*x_ref[0]+(-0.484624435043)*x_ref[1]**o+(-0.279314462332)*x_ref[1]+(-0.458856066504)*x_ref[2]**o+(0.0588111305227)*x_ref[2]
        ref[0,3,1,1]=(-0.853676338655)*x_ref[0]**o+(-0.651539814195)*x_ref[0]+(-0.109727277725)*x_ref[1]**o+(0.0139006407942)*x_ref[1]+(0.231943245426)*x_ref[2]**o+(0.42972517233)*x_ref[2]
        ref[0,3,2,0]=(-0.60799269765)*x_ref[0]**o+(-0.802413269128)*x_ref[0]+(0.151362498863)*x_ref[1]**o+(-0.678226252875)*x_ref[1]+(-0.867117685659)*x_ref[2]**o+(-0.711982116071)*x_ref[2]
        ref[0,3,2,1]=(-0.243012743941)*x_ref[0]**o+(-0.692614579016)*x_ref[0]+(0.0986817641914)*x_ref[1]**o+(-0.495735862695)*x_ref[1]+(-0.430040681653)*x_ref[2]**o+(0.0108928722284)*x_ref[2]
        ref[0,4,0,0]=(0.0989402181558)*x_ref[0]**o+(-0.60096719882)*x_ref[0]+(-0.324253928432)*x_ref[1]**o+(-0.862723687559)*x_ref[1]+(-0.34745416427)*x_ref[2]**o+(0.936559074873)*x_ref[2]
        ref[0,4,0,1]=(0.711021967566)*x_ref[0]**o+(-0.939408401113)*x_ref[0]+(-0.218100921633)*x_ref[1]**o+(0.92835194875)*x_ref[1]+(-0.353936913865)*x_ref[2]**o+(-0.547266090928)*x_ref[2]
        ref[0,4,1,0]=(0.172231614188)*x_ref[0]**o+(-0.734622090671)*x_ref[0]+(0.432686916647)*x_ref[1]**o+(0.552595350492)*x_ref[1]+(0.779662641076)*x_ref[2]**o+(-0.0933220492779)*x_ref[2]
        ref[0,4,1,1]=(0.996776716866)*x_ref[0]**o+(-0.340786520686)*x_ref[0]+(0.260075214171)*x_ref[1]**o+(0.00385504852496)*x_ref[1]+(0.377516772127)*x_ref[2]**o+(-0.341751634181)*x_ref[2]
        ref[0,4,2,0]=(-0.655642896664)*x_ref[0]**o+(-0.840525173737)*x_ref[0]+(-0.855343721564)*x_ref[1]**o+(0.253558571524)*x_ref[1]+(0.0463733059925)*x_ref[2]**o+(-0.116359866912)*x_ref[2]
        ref[0,4,2,1]=(0.131619661414)*x_ref[0]**o+(-0.162067255954)*x_ref[0]+(0.562536831367)*x_ref[1]**o+(-0.940412472035)*x_ref[1]+(-0.282265762783)*x_ref[2]**o+(0.634711607923)*x_ref[2]
        ref[1,0,0,0]=(0.484071808097)*x_ref[0]**o+(0.566763524691)*x_ref[0]+(-0.658614173653)*x_ref[1]**o+(-0.140192232895)*x_ref[1]+(-0.310114236409)*x_ref[2]**o+(-0.918532401595)*x_ref[2]
        ref[1,0,0,1]=(-0.81190380441)*x_ref[0]**o+(0.644439878497)*x_ref[0]+(0.479583403289)*x_ref[1]**o+(-0.493318882779)*x_ref[1]+(-0.424704492175)*x_ref[2]**o+(0.700624317732)*x_ref[2]
        ref[1,0,1,0]=(0.766650182435)*x_ref[0]**o+(0.226365545129)*x_ref[0]+(0.510828945971)*x_ref[1]**o+(0.283221954498)*x_ref[1]+(0.817303877886)*x_ref[2]**o+(-0.245709044628)*x_ref[2]
        ref[1,0,1,1]=(0.194787112664)*x_ref[0]**o+(-0.879306736922)*x_ref[0]+(0.185381291627)*x_ref[1]**o+(-0.510692743975)*x_ref[1]+(-0.422555014055)*x_ref[2]**o+(0.556629919959)*x_ref[2]
        ref[1,0,2,0]=(-0.958345828895)*x_ref[0]**o+(-0.667735551013)*x_ref[0]+(0.278271284225)*x_ref[1]**o+(-0.0847995194757)*x_ref[1]+(0.902184171441)*x_ref[2]**o+(0.763125168854)*x_ref[2]
        ref[1,0,2,1]=(-0.0766429670195)*x_ref[0]**o+(0.894421139029)*x_ref[0]+(-0.331560314101)*x_ref[1]**o+(-0.0253297623997)*x_ref[1]+(0.13406045967)*x_ref[2]**o+(-0.326681111625)*x_ref[2]
        ref[1,1,0,0]=(-0.979406855838)*x_ref[0]**o+(0.528849909402)*x_ref[0]+(-0.0486159476737)*x_ref[1]**o+(-0.898392512695)*x_ref[1]+(-0.678328327407)*x_ref[2]**o+(0.644201802057)*x_ref[2]
        ref[1,1,0,1]=(-0.977485554605)*x_ref[0]**o+(0.794799941538)*x_ref[0]+(-0.00237183848504)*x_ref[1]**o+(-0.633497483149)*x_ref[1]+(0.746593853568)*x_ref[2]**o+(0.197741126365)*x_ref[2]
        ref[1,1,1,0]=(0.379985506182)*x_ref[0]**o+(0.69889144234)*x_ref[0]+(-0.507293887409)*x_ref[1]**o+(-0.727675545291)*x_ref[1]+(-0.467064531693)*x_ref[2]**o+(0.130273930089)*x_ref[2]
        ref[1,1,1,1]=(-0.330627830819)*x_ref[0]**o+(-0.0311542935889)*x_ref[0]+(-0.0671494148749)*x_ref[1]**o+(0.583605469163)*x_ref[1]+(-0.172083996069)*x_ref[2]**o+(0.666718011074)*x_ref[2]
        ref[1,1,2,0]=(0.539618381063)*x_ref[0]**o+(-0.546796237888)*x_ref[0]+(0.857911750392)*x_ref[1]**o+(0.119552731463)*x_ref[1]+(0.329286820109)*x_ref[2]**o+(0.637240649087)*x_ref[2]
        ref[1,1,2,1]=(-0.828603071334)*x_ref[0]**o+(0.861799149065)*x_ref[0]+(0.615224040538)*x_ref[1]**o+(-0.0868985543263)*x_ref[1]+(-0.206168997809)*x_ref[2]**o+(0.311103719286)*x_ref[2]
        ref[1,2,0,0]=(0.695106867715)*x_ref[0]**o+(-0.533303418727)*x_ref[0]+(-0.393936955207)*x_ref[1]**o+(-0.407635261686)*x_ref[1]+(0.604346269147)*x_ref[2]**o+(-0.10213061533)*x_ref[2]
        ref[1,2,0,1]=(0.338035309929)*x_ref[0]**o+(0.517135412671)*x_ref[0]+(0.277863735829)*x_ref[1]**o+(-0.703417079398)*x_ref[1]+(0.369398635984)*x_ref[2]**o+(-0.952905070601)*x_ref[2]
        ref[1,2,1,0]=(0.937866603297)*x_ref[0]**o+(0.436121631708)*x_ref[0]+(0.568362098461)*x_ref[1]**o+(0.223837722984)*x_ref[1]+(-0.236896910941)*x_ref[2]**o+(0.316114408308)*x_ref[2]
        ref[1,2,1,1]=(0.285714491932)*x_ref[0]**o+(0.633321537987)*x_ref[0]+(0.953737237202)*x_ref[1]**o+(0.732065555924)*x_ref[1]+(-0.515759068197)*x_ref[2]**o+(-0.519775334889)*x_ref[2]
        ref[1,2,2,0]=(-0.0610222303479)*x_ref[0]**o+(0.135974983944)*x_ref[0]+(0.450213393447)*x_ref[1]**o+(-0.44747541715)*x_ref[1]+(-0.640660207187)*x_ref[2]**o+(0.348457261119)*x_ref[2]
        ref[1,2,2,1]=(0.131753033369)*x_ref[0]**o+(-0.311359549998)*x_ref[0]+(0.445707767235)*x_ref[1]**o+(-0.710795569938)*x_ref[1]+(-0.746038648477)*x_ref[2]**o+(-0.210414773188)*x_ref[2]
        ref[1,3,0,0]=(-0.323684061289)*x_ref[0]**o+(-0.0937091220282)*x_ref[0]+(-0.827884898323)*x_ref[1]**o+(0.604442907568)*x_ref[1]+(0.620923596379)*x_ref[2]**o+(0.851758170763)*x_ref[2]
        ref[1,3,0,1]=(-0.269239872573)*x_ref[0]**o+(-0.603248280988)*x_ref[0]+(0.603804530326)*x_ref[1]**o+(-0.331910095046)*x_ref[1]+(-0.653877236308)*x_ref[2]**o+(0.115434865132)*x_ref[2]
        ref[1,3,1,0]=(0.0531798412869)*x_ref[0]**o+(0.292950346899)*x_ref[0]+(-0.634376081823)*x_ref[1]**o+(-0.775427771578)*x_ref[1]+(-0.987113467149)*x_ref[2]**o+(0.757980444447)*x_ref[2]
        ref[1,3,1,1]=(0.757807292214)*x_ref[0]**o+(0.81778478971)*x_ref[0]+(-0.619873148405)*x_ref[1]**o+(0.0129855659138)*x_ref[1]+(-0.282861093681)*x_ref[2]**o+(0.667211104045)*x_ref[2]
        ref[1,3,2,0]=(-0.915205606853)*x_ref[0]**o+(0.918704643459)*x_ref[0]+(0.143238155504)*x_ref[1]**o+(0.0285686377214)*x_ref[1]+(-0.848712843243)*x_ref[2]**o+(0.058647124862)*x_ref[2]
        ref[1,3,2,1]=(0.441487712831)*x_ref[0]**o+(-0.626878272507)*x_ref[0]+(-0.862579662295)*x_ref[1]**o+(0.204629243832)*x_ref[1]+(0.964143347265)*x_ref[2]**o+(-0.399125844282)*x_ref[2]
        ref[1,4,0,0]=(-0.349791275126)*x_ref[0]**o+(-0.140293844776)*x_ref[0]+(-0.693879671138)*x_ref[1]**o+(0.429290435637)*x_ref[1]+(-0.892670314048)*x_ref[2]**o+(-0.380777380409)*x_ref[2]
        ref[1,4,0,1]=(0.528712424647)*x_ref[0]**o+(0.449268002671)*x_ref[0]+(-0.202472214961)*x_ref[1]**o+(-0.374762208851)*x_ref[1]+(-0.625590920648)*x_ref[2]**o+(-0.000776205555531)*x_ref[2]
        ref[1,4,1,0]=(0.00425881397184)*x_ref[0]**o+(0.265831686248)*x_ref[0]+(0.757626547866)*x_ref[1]**o+(-0.934207064756)*x_ref[1]+(0.541356932538)*x_ref[2]**o+(0.217841597089)*x_ref[2]
        ref[1,4,1,1]=(-0.02422207106)*x_ref[0]**o+(-0.807018149115)*x_ref[0]+(-0.402347046418)*x_ref[1]**o+(-0.0548851919048)*x_ref[1]+(0.850202044077)*x_ref[2]**o+(-0.889576261979)*x_ref[2]
        ref[1,4,2,0]=(0.487059068561)*x_ref[0]**o+(0.338254525003)*x_ref[0]+(-0.387019663154)*x_ref[1]**o+(-0.436969552236)*x_ref[1]+(-0.747690766603)*x_ref[2]**o+(0.265854907003)*x_ref[2]
        ref[1,4,2,1]=(0.687562221132)*x_ref[0]**o+(0.4755782976)*x_ref[0]+(-0.456287111804)*x_ref[1]**o+(0.412138123928)*x_ref[1]+(0.637920334647)*x_ref[2]**o+(0.817007725525)*x_ref[2]
        ref[2,0,0,0]=(-0.728166422776)*x_ref[0]**o+(0.108904177195)*x_ref[0]+(0.383848108635)*x_ref[1]**o+(-0.19252707292)*x_ref[1]+(-0.279319122495)*x_ref[2]**o+(-0.0530727745585)*x_ref[2]
        ref[2,0,0,1]=(0.380827452272)*x_ref[0]**o+(-0.133776280687)*x_ref[0]+(-0.174731551283)*x_ref[1]**o+(-0.369231086652)*x_ref[1]+(-0.0190771818046)*x_ref[2]**o+(-0.331873068919)*x_ref[2]
        ref[2,0,1,0]=(0.653644298826)*x_ref[0]**o+(-0.916110182689)*x_ref[0]+(0.0510855252492)*x_ref[1]**o+(-0.934192949208)*x_ref[1]+(0.572048462784)*x_ref[2]**o+(-0.451401856863)*x_ref[2]
        ref[2,0,1,1]=(-0.394958497502)*x_ref[0]**o+(-0.690752972227)*x_ref[0]+(-0.436352069349)*x_ref[1]**o+(0.933524576573)*x_ref[1]+(-0.524344161699)*x_ref[2]**o+(0.526748792776)*x_ref[2]
        ref[2,0,2,0]=(0.845649523782)*x_ref[0]**o+(0.63927747558)*x_ref[0]+(-0.534192218216)*x_ref[1]**o+(0.714500925459)*x_ref[1]+(0.156077814898)*x_ref[2]**o+(-0.193712778792)*x_ref[2]
        ref[2,0,2,1]=(0.809636640413)*x_ref[0]**o+(-0.930654037481)*x_ref[0]+(0.509441692942)*x_ref[1]**o+(0.881563824585)*x_ref[1]+(0.101333749057)*x_ref[2]**o+(0.418791722704)*x_ref[2]
        ref[2,1,0,0]=(0.289481281745)*x_ref[0]**o+(-0.98279193802)*x_ref[0]+(-0.443382747319)*x_ref[1]**o+(-0.281710992472)*x_ref[1]+(0.426349350391)*x_ref[2]**o+(-0.870192121251)*x_ref[2]
        ref[2,1,0,1]=(0.837046002753)*x_ref[0]**o+(-0.0254166983371)*x_ref[0]+(0.30178635814)*x_ref[1]**o+(0.973915444924)*x_ref[1]+(0.298631524734)*x_ref[2]**o+(-0.422490970248)*x_ref[2]
        ref[2,1,1,0]=(0.869799082394)*x_ref[0]**o+(0.376436513628)*x_ref[0]+(-0.74735679118)*x_ref[1]**o+(0.477184335901)*x_ref[1]+(-0.797472924916)*x_ref[2]**o+(-0.568827873964)*x_ref[2]
        ref[2,1,1,1]=(0.993441009779)*x_ref[0]**o+(-0.264888299493)*x_ref[0]+(0.861197560367)*x_ref[1]**o+(-0.769058084144)*x_ref[1]+(-0.86221947415)*x_ref[2]**o+(-0.617024169672)*x_ref[2]
        ref[2,1,2,0]=(-0.89303691749)*x_ref[0]**o+(-0.390287412175)*x_ref[0]+(-0.312883084291)*x_ref[1]**o+(-0.432368502641)*x_ref[1]+(-0.121476666357)*x_ref[2]**o+(0.416361314288)*x_ref[2]
        ref[2,1,2,1]=(-0.622214426504)*x_ref[0]**o+(-0.595279134338)*x_ref[0]+(0.000927173498715)*x_ref[1]**o+(0.783001179117)*x_ref[1]+(-0.956003738061)*x_ref[2]**o+(0.482321214492)*x_ref[2]
        ref[2,2,0,0]=(-0.0047994639068)*x_ref[0]**o+(0.478805093492)*x_ref[0]+(0.747825846519)*x_ref[1]**o+(0.0756064430807)*x_ref[1]+(-0.470534826525)*x_ref[2]**o+(-0.421524571831)*x_ref[2]
        ref[2,2,0,1]=(-0.0569198078126)*x_ref[0]**o+(-0.36284856463)*x_ref[0]+(-0.374343856016)*x_ref[1]**o+(0.193470734451)*x_ref[1]+(0.425350990791)*x_ref[2]**o+(-0.903309757487)*x_ref[2]
        ref[2,2,1,0]=(0.602075251229)*x_ref[0]**o+(-0.558163518701)*x_ref[0]+(-0.00146699467501)*x_ref[1]**o+(0.437116756605)*x_ref[1]+(-0.252696658318)*x_ref[2]**o+(0.154017654456)*x_ref[2]
        ref[2,2,1,1]=(0.226970288166)*x_ref[0]**o+(0.844076458869)*x_ref[0]+(0.231874137232)*x_ref[1]**o+(-0.270233945335)*x_ref[1]+(0.365979082276)*x_ref[2]**o+(-0.937144802776)*x_ref[2]
        ref[2,2,2,0]=(0.662440813722)*x_ref[0]**o+(-0.600830429698)*x_ref[0]+(0.248436486724)*x_ref[1]**o+(0.73439963812)*x_ref[1]+(-0.406317484259)*x_ref[2]**o+(-0.406641354653)*x_ref[2]
        ref[2,2,2,1]=(0.948630940404)*x_ref[0]**o+(-0.695098513788)*x_ref[0]+(-0.819843883615)*x_ref[1]**o+(0.0479609386435)*x_ref[1]+(-0.968033045837)*x_ref[2]**o+(0.199490071238)*x_ref[2]
        ref[2,3,0,0]=(0.967996364186)*x_ref[0]**o+(0.562562580668)*x_ref[0]+(0.660658005197)*x_ref[1]**o+(-0.00126241998808)*x_ref[1]+(-0.0188273819622)*x_ref[2]**o+(-0.798370566647)*x_ref[2]
        ref[2,3,0,1]=(0.108137383906)*x_ref[0]**o+(0.230815090379)*x_ref[0]+(-0.0817434914408)*x_ref[1]**o+(0.256977068172)*x_ref[1]+(0.323454379512)*x_ref[2]**o+(0.0382049771532)*x_ref[2]
        ref[2,3,1,0]=(-0.279443973124)*x_ref[0]**o+(-0.948014830608)*x_ref[0]+(-0.237787814838)*x_ref[1]**o+(-0.880632864876)*x_ref[1]+(0.305230465688)*x_ref[2]**o+(0.722139326346)*x_ref[2]
        ref[2,3,1,1]=(-0.192752669063)*x_ref[0]**o+(0.763575125553)*x_ref[0]+(-0.317096602795)*x_ref[1]**o+(-0.241479822867)*x_ref[1]+(0.643154065804)*x_ref[2]**o+(0.903250562165)*x_ref[2]
        ref[2,3,2,0]=(0.58153852366)*x_ref[0]**o+(-0.131284078682)*x_ref[0]+(0.803022288461)*x_ref[1]**o+(0.862074208938)*x_ref[1]+(0.934648335783)*x_ref[2]**o+(0.720337200812)*x_ref[2]
        ref[2,3,2,1]=(-0.866755142782)*x_ref[0]**o+(0.299995782189)*x_ref[0]+(0.830022439718)*x_ref[1]**o+(0.787775630269)*x_ref[1]+(-0.436980900353)*x_ref[2]**o+(0.436719598692)*x_ref[2]
        ref[2,4,0,0]=(-0.416946111972)*x_ref[0]**o+(-0.342692819134)*x_ref[0]+(-0.265712927184)*x_ref[1]**o+(-0.503154497528)*x_ref[1]+(0.436447837569)*x_ref[2]**o+(-0.514226068117)*x_ref[2]
        ref[2,4,0,1]=(-0.915995315837)*x_ref[0]**o+(-0.562870475868)*x_ref[0]+(0.158881525561)*x_ref[1]**o+(-0.458461594412)*x_ref[1]+(-0.117927681987)*x_ref[2]**o+(-0.697751714189)*x_ref[2]
        ref[2,4,1,0]=(-0.961291698815)*x_ref[0]**o+(0.457487466485)*x_ref[0]+(0.170987701633)*x_ref[1]**o+(-0.14226156627)*x_ref[1]+(-0.663462703057)*x_ref[2]**o+(-0.0307934454283)*x_ref[2]
        ref[2,4,1,1]=(0.588162542138)*x_ref[0]**o+(-0.761783611043)*x_ref[0]+(0.510241246326)*x_ref[1]**o+(0.805053446677)*x_ref[1]+(-0.851391511147)*x_ref[2]**o+(-0.0976037736598)*x_ref[2]
        ref[2,4,2,0]=(0.649047985858)*x_ref[0]**o+(0.677435498282)*x_ref[0]+(-0.620777977748)*x_ref[1]**o+(-0.209440794016)*x_ref[1]+(0.58009322661)*x_ref[2]**o+(0.56317180284)*x_ref[2]
        ref[2,4,2,1]=(-0.975596464331)*x_ref[0]**o+(0.759364271658)*x_ref[0]+(0.43142780149)*x_ref[1]**o+(-0.701043203786)*x_ref[1]+(-0.648159388134)*x_ref[2]**o+(-0.129618261144)*x_ref[2]
        ref[3,0,0,0]=(-0.0207100058532)*x_ref[0]**o+(-0.829637327952)*x_ref[0]+(-0.0326040806041)*x_ref[1]**o+(0.652320712227)*x_ref[1]+(-0.509435869246)*x_ref[2]**o+(-0.66044392521)*x_ref[2]
        ref[3,0,0,1]=(-0.312677151974)*x_ref[0]**o+(0.0204494340128)*x_ref[0]+(-0.989457740146)*x_ref[1]**o+(-0.0606077687298)*x_ref[1]+(-0.527267756606)*x_ref[2]**o+(0.496006274201)*x_ref[2]
        ref[3,0,1,0]=(0.842871894549)*x_ref[0]**o+(-0.577687338895)*x_ref[0]+(0.414019506131)*x_ref[1]**o+(0.0410521331015)*x_ref[1]+(-0.830482660184)*x_ref[2]**o+(0.688692187619)*x_ref[2]
        ref[3,0,1,1]=(-0.920070939376)*x_ref[0]**o+(0.632972952755)*x_ref[0]+(0.944186412149)*x_ref[1]**o+(0.620319712728)*x_ref[1]+(-0.171433727161)*x_ref[2]**o+(0.267125744191)*x_ref[2]
        ref[3,0,2,0]=(-0.0669535338492)*x_ref[0]**o+(0.249181230746)*x_ref[0]+(-0.147448553328)*x_ref[1]**o+(-0.459701695839)*x_ref[1]+(0.378458172384)*x_ref[2]**o+(0.916594147889)*x_ref[2]
        ref[3,0,2,1]=(0.0720111269432)*x_ref[0]**o+(-0.390032127319)*x_ref[0]+(-0.115565740734)*x_ref[1]**o+(0.501210832815)*x_ref[1]+(0.568129023658)*x_ref[2]**o+(-0.303909080042)*x_ref[2]
        ref[3,1,0,0]=(0.205675480731)*x_ref[0]**o+(-0.843093328795)*x_ref[0]+(-0.812415573635)*x_ref[1]**o+(-0.706806177394)*x_ref[1]+(-0.158442718286)*x_ref[2]**o+(0.15187176613)*x_ref[2]
        ref[3,1,0,1]=(0.149669308017)*x_ref[0]**o+(0.487445503149)*x_ref[0]+(-0.27973905833)*x_ref[1]**o+(-0.504992462225)*x_ref[1]+(-0.728166691522)*x_ref[2]**o+(0.734846267391)*x_ref[2]
        ref[3,1,1,0]=(0.0410578015251)*x_ref[0]**o+(-0.525001495676)*x_ref[0]+(-0.512685697472)*x_ref[1]**o+(-0.284314686987)*x_ref[1]+(0.888112049081)*x_ref[2]**o+(0.425466793802)*x_ref[2]
        ref[3,1,1,1]=(-0.374237047043)*x_ref[0]**o+(-0.00078724690461)*x_ref[0]+(0.849952487348)*x_ref[1]**o+(-0.654945893799)*x_ref[1]+(-0.536739132177)*x_ref[2]**o+(0.20919294066)*x_ref[2]
        ref[3,1,2,0]=(0.685749084068)*x_ref[0]**o+(-0.730378478543)*x_ref[0]+(-0.918072746761)*x_ref[1]**o+(-0.706713872062)*x_ref[1]+(-0.486100957526)*x_ref[2]**o+(0.95736939257)*x_ref[2]
        ref[3,1,2,1]=(-0.287859736321)*x_ref[0]**o+(-0.271882215207)*x_ref[0]+(-0.673357024623)*x_ref[1]**o+(0.702867457627)*x_ref[1]+(0.78445294416)*x_ref[2]**o+(0.613888365259)*x_ref[2]
        ref[3,2,0,0]=(0.568506008135)*x_ref[0]**o+(-0.825947283734)*x_ref[0]+(0.687369924271)*x_ref[1]**o+(-0.866597782569)*x_ref[1]+(-0.332810902355)*x_ref[2]**o+(0.499502049139)*x_ref[2]
        ref[3,2,0,1]=(0.757251773732)*x_ref[0]**o+(0.293337812465)*x_ref[0]+(0.686709285382)*x_ref[1]**o+(-0.084997189402)*x_ref[1]+(-0.338903817381)*x_ref[2]**o+(-0.787646799698)*x_ref[2]
        ref[3,2,1,0]=(-0.580327419983)*x_ref[0]**o+(-0.603952498996)*x_ref[0]+(0.523057988618)*x_ref[1]**o+(-0.0835187394107)*x_ref[1]+(0.459827887147)*x_ref[2]**o+(0.689021201042)*x_ref[2]
        ref[3,2,1,1]=(-0.41923416619)*x_ref[0]**o+(0.0406426957237)*x_ref[0]+(-0.908041008186)*x_ref[1]**o+(0.382097138462)*x_ref[1]+(0.66835706849)*x_ref[2]**o+(0.355849275378)*x_ref[2]
        ref[3,2,2,0]=(-0.133756767108)*x_ref[0]**o+(-0.233743496333)*x_ref[0]+(-0.661420120856)*x_ref[1]**o+(0.788722576584)*x_ref[1]+(-0.131764585707)*x_ref[2]**o+(-0.781004479871)*x_ref[2]
        ref[3,2,2,1]=(0.0308645139784)*x_ref[0]**o+(-0.194986246344)*x_ref[0]+(-0.267418187303)*x_ref[1]**o+(0.332143733168)*x_ref[1]+(0.513266431894)*x_ref[2]**o+(0.594016981473)*x_ref[2]
        ref[3,3,0,0]=(0.644314102613)*x_ref[0]**o+(-0.977355279476)*x_ref[0]+(0.516452323667)*x_ref[1]**o+(0.172873579919)*x_ref[1]+(0.778904819969)*x_ref[2]**o+(0.559689750425)*x_ref[2]
        ref[3,3,0,1]=(0.736135986201)*x_ref[0]**o+(0.366918250826)*x_ref[0]+(-0.0547007988818)*x_ref[1]**o+(-0.830174436433)*x_ref[1]+(0.613440124338)*x_ref[2]**o+(0.574992469591)*x_ref[2]
        ref[3,3,1,0]=(0.324504938146)*x_ref[0]**o+(-0.915240176689)*x_ref[0]+(-0.398105336249)*x_ref[1]**o+(-0.526498343533)*x_ref[1]+(0.43264573336)*x_ref[2]**o+(0.393008086529)*x_ref[2]
        ref[3,3,1,1]=(0.926367438323)*x_ref[0]**o+(0.526364305462)*x_ref[0]+(0.15596582236)*x_ref[1]**o+(0.165158439711)*x_ref[1]+(-0.508505348973)*x_ref[2]**o+(-0.603637833992)*x_ref[2]
        ref[3,3,2,0]=(0.661333046004)*x_ref[0]**o+(0.425320373512)*x_ref[0]+(-0.810809051016)*x_ref[1]**o+(0.20446843023)*x_ref[1]+(0.685136412413)*x_ref[2]**o+(-0.0281127019894)*x_ref[2]
        ref[3,3,2,1]=(0.851742499007)*x_ref[0]**o+(-0.405506974975)*x_ref[0]+(-0.296768983731)*x_ref[1]**o+(0.389557354929)*x_ref[1]+(0.335108341463)*x_ref[2]**o+(0.0313094433351)*x_ref[2]
        ref[3,4,0,0]=(0.0415035498649)*x_ref[0]**o+(0.827106496882)*x_ref[0]+(-0.481831487377)*x_ref[1]**o+(-0.14935685817)*x_ref[1]+(0.354765095566)*x_ref[2]**o+(0.521898910137)*x_ref[2]
        ref[3,4,0,1]=(-0.36328099035)*x_ref[0]**o+(-0.243342386595)*x_ref[0]+(0.0475935188475)*x_ref[1]**o+(0.845308417697)*x_ref[1]+(0.315259620082)*x_ref[2]**o+(0.541313742759)*x_ref[2]
        ref[3,4,1,0]=(-0.66364149672)*x_ref[0]**o+(0.987505978353)*x_ref[0]+(-0.337659775957)*x_ref[1]**o+(0.197277151989)*x_ref[1]+(-0.431260870605)*x_ref[2]**o+(-0.803375909471)*x_ref[2]
        ref[3,4,1,1]=(0.722104365428)*x_ref[0]**o+(0.168416682879)*x_ref[0]+(-0.415740544748)*x_ref[1]**o+(-0.437836696476)*x_ref[1]+(-0.280581752598)*x_ref[2]**o+(0.434326371762)*x_ref[2]
        ref[3,4,2,0]=(0.673144889684)*x_ref[0]**o+(-0.0628529591764)*x_ref[0]+(0.0161458704408)*x_ref[1]**o+(-0.439966404278)*x_ref[1]+(-0.963402070447)*x_ref[2]**o+(0.308312287021)*x_ref[2]
        ref[3,4,2,1]=(0.649131384887)*x_ref[0]**o+(-0.825086043001)*x_ref[0]+(0.204615530143)*x_ref[1]**o+(0.00801105023644)*x_ref[1]+(-0.234754758061)*x_ref[2]**o+(0.659650083864)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromData_ReducedFunctionOnContactOne_rank0(self):
      """
      tests interpolate for rank 0 Data onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactOne onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactOne(self.domain)
      x=w.getX()
      arg=Data(0,(),w)
      ref=Data(0,(),w_ref)
      if dim==2:
        arg=(-0.935068705503)*x[0]**o+(-0.843768568975)*x[0]+(0.220151879481)*x[1]**o+(0.629247769917)*x[1]
        ref=(-0.935068705503)*x_ref[0]**o+(-0.843768568975)*x_ref[0]+(0.220151879481)*x_ref[1]**o+(0.629247769917)*x_ref[1]
      else:
        arg=(0.204715133488)*x[0]**o+(-0.933589384176)*x[0]+(-0.651477613601)*x[1]**o+(-0.128099401086)*x[1]+(0.0507915334433)*x[2]**o+(0.63225604515)*x[2]
        ref=(0.204715133488)*x_ref[0]**o+(-0.933589384176)*x_ref[0]+(-0.651477613601)*x_ref[1]**o+(-0.128099401086)*x_ref[1]+(0.0507915334433)*x_ref[2]**o+(0.63225604515)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromData_ReducedFunctionOnContactOne_rank1(self):
      """
      tests interpolate for rank 1 Data onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactOne onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactOne(self.domain)
      x=w.getX()
      arg=Data(0,(2,),w)
      ref=Data(0,(2,),w_ref)
      if dim==2:
        arg[0]=(0.693671772608)*x[0]**o+(0.260200526395)*x[0]+(-0.99202032752)*x[1]**o+(0.698503413133)*x[1]
        arg[1]=(-0.549448913905)*x[0]**o+(-0.728769679965)*x[0]+(0.415132041969)*x[1]**o+(-0.106391482954)*x[1]
        ref[0]=(0.693671772608)*x_ref[0]**o+(0.260200526395)*x_ref[0]+(-0.99202032752)*x_ref[1]**o+(0.698503413133)*x_ref[1]
        ref[1]=(-0.549448913905)*x_ref[0]**o+(-0.728769679965)*x_ref[0]+(0.415132041969)*x_ref[1]**o+(-0.106391482954)*x_ref[1]
      else:
        arg[0]=(0.479813279739)*x[0]**o+(0.940795516779)*x[0]+(-0.671143421639)*x[1]**o+(-0.485412476854)*x[1]+(-0.962114397463)*x[2]**o+(0.0250734884375)*x[2]
        arg[1]=(0.703481492279)*x[0]**o+(0.298972367312)*x[0]+(0.22386003834)*x[1]**o+(-0.726486516907)*x[1]+(0.0556568111911)*x[2]**o+(-0.603711754869)*x[2]
        ref[0]=(0.479813279739)*x_ref[0]**o+(0.940795516779)*x_ref[0]+(-0.671143421639)*x_ref[1]**o+(-0.485412476854)*x_ref[1]+(-0.962114397463)*x_ref[2]**o+(0.0250734884375)*x_ref[2]
        ref[1]=(0.703481492279)*x_ref[0]**o+(0.298972367312)*x_ref[0]+(0.22386003834)*x_ref[1]**o+(-0.726486516907)*x_ref[1]+(0.0556568111911)*x_ref[2]**o+(-0.603711754869)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromData_ReducedFunctionOnContactOne_rank2(self):
      """
      tests interpolate for rank 2 Data onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactOne onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactOne(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5),w)
      ref=Data(0,(4, 5),w_ref)
      if dim==2:
        arg[0,0]=(0.436439283545)*x[0]**o+(0.705849609672)*x[0]+(-0.261476237133)*x[1]**o+(-0.10504570237)*x[1]
        arg[0,1]=(0.035208834696)*x[0]**o+(0.590058421274)*x[0]+(-0.532905126944)*x[1]**o+(0.126572388319)*x[1]
        arg[0,2]=(0.985266120604)*x[0]**o+(-0.465951670003)*x[0]+(0.855881157118)*x[1]**o+(0.00932326399668)*x[1]
        arg[0,3]=(0.867525798771)*x[0]**o+(-0.480556561488)*x[0]+(-0.958556900472)*x[1]**o+(-0.764939529718)*x[1]
        arg[0,4]=(-0.51062937871)*x[0]**o+(-0.135415437913)*x[0]+(0.340077694039)*x[1]**o+(0.874347806916)*x[1]
        arg[1,0]=(0.636810684661)*x[0]**o+(0.774651780754)*x[0]+(-0.314130997812)*x[1]**o+(-0.422706240721)*x[1]
        arg[1,1]=(0.495122166216)*x[0]**o+(0.757099858142)*x[0]+(-0.25861054515)*x[1]**o+(0.612180732826)*x[1]
        arg[1,2]=(-0.909636123949)*x[0]**o+(-0.0434375137409)*x[0]+(0.100612174358)*x[1]**o+(0.787104124194)*x[1]
        arg[1,3]=(-0.499307726848)*x[0]**o+(0.013205984499)*x[0]+(0.858074302957)*x[1]**o+(-0.630926261217)*x[1]
        arg[1,4]=(-0.410815849264)*x[0]**o+(0.820939326325)*x[0]+(-0.415370205615)*x[1]**o+(0.0602438669371)*x[1]
        arg[2,0]=(0.642545275281)*x[0]**o+(0.872239548588)*x[0]+(0.446294659843)*x[1]**o+(0.929350720888)*x[1]
        arg[2,1]=(0.182218495486)*x[0]**o+(-0.393184556589)*x[0]+(0.42311545961)*x[1]**o+(-0.479636250878)*x[1]
        arg[2,2]=(0.874574618473)*x[0]**o+(0.879031610251)*x[0]+(0.686157301617)*x[1]**o+(0.706917527697)*x[1]
        arg[2,3]=(0.614189029029)*x[0]**o+(0.0973950074653)*x[0]+(0.626052996278)*x[1]**o+(0.75043535006)*x[1]
        arg[2,4]=(0.386209196978)*x[0]**o+(0.644169312279)*x[0]+(-0.554562346917)*x[1]**o+(-0.474292025255)*x[1]
        arg[3,0]=(0.015012073686)*x[0]**o+(-0.665804159938)*x[0]+(-0.849030247328)*x[1]**o+(-0.936779648988)*x[1]
        arg[3,1]=(-0.818194334698)*x[0]**o+(-0.569993362998)*x[0]+(-0.806235611766)*x[1]**o+(0.35347475408)*x[1]
        arg[3,2]=(-0.227117779134)*x[0]**o+(-0.981285175536)*x[0]+(-0.226804205345)*x[1]**o+(-0.0134258740334)*x[1]
        arg[3,3]=(0.126716245369)*x[0]**o+(0.135476076345)*x[0]+(-0.641421845681)*x[1]**o+(0.0595388928019)*x[1]
        arg[3,4]=(-0.658260624128)*x[0]**o+(0.124989215683)*x[0]+(-0.949146047463)*x[1]**o+(0.805277938593)*x[1]
        ref[0,0]=(0.436439283545)*x_ref[0]**o+(0.705849609672)*x_ref[0]+(-0.261476237133)*x_ref[1]**o+(-0.10504570237)*x_ref[1]
        ref[0,1]=(0.035208834696)*x_ref[0]**o+(0.590058421274)*x_ref[0]+(-0.532905126944)*x_ref[1]**o+(0.126572388319)*x_ref[1]
        ref[0,2]=(0.985266120604)*x_ref[0]**o+(-0.465951670003)*x_ref[0]+(0.855881157118)*x_ref[1]**o+(0.00932326399668)*x_ref[1]
        ref[0,3]=(0.867525798771)*x_ref[0]**o+(-0.480556561488)*x_ref[0]+(-0.958556900472)*x_ref[1]**o+(-0.764939529718)*x_ref[1]
        ref[0,4]=(-0.51062937871)*x_ref[0]**o+(-0.135415437913)*x_ref[0]+(0.340077694039)*x_ref[1]**o+(0.874347806916)*x_ref[1]
        ref[1,0]=(0.636810684661)*x_ref[0]**o+(0.774651780754)*x_ref[0]+(-0.314130997812)*x_ref[1]**o+(-0.422706240721)*x_ref[1]
        ref[1,1]=(0.495122166216)*x_ref[0]**o+(0.757099858142)*x_ref[0]+(-0.25861054515)*x_ref[1]**o+(0.612180732826)*x_ref[1]
        ref[1,2]=(-0.909636123949)*x_ref[0]**o+(-0.0434375137409)*x_ref[0]+(0.100612174358)*x_ref[1]**o+(0.787104124194)*x_ref[1]
        ref[1,3]=(-0.499307726848)*x_ref[0]**o+(0.013205984499)*x_ref[0]+(0.858074302957)*x_ref[1]**o+(-0.630926261217)*x_ref[1]
        ref[1,4]=(-0.410815849264)*x_ref[0]**o+(0.820939326325)*x_ref[0]+(-0.415370205615)*x_ref[1]**o+(0.0602438669371)*x_ref[1]
        ref[2,0]=(0.642545275281)*x_ref[0]**o+(0.872239548588)*x_ref[0]+(0.446294659843)*x_ref[1]**o+(0.929350720888)*x_ref[1]
        ref[2,1]=(0.182218495486)*x_ref[0]**o+(-0.393184556589)*x_ref[0]+(0.42311545961)*x_ref[1]**o+(-0.479636250878)*x_ref[1]
        ref[2,2]=(0.874574618473)*x_ref[0]**o+(0.879031610251)*x_ref[0]+(0.686157301617)*x_ref[1]**o+(0.706917527697)*x_ref[1]
        ref[2,3]=(0.614189029029)*x_ref[0]**o+(0.0973950074653)*x_ref[0]+(0.626052996278)*x_ref[1]**o+(0.75043535006)*x_ref[1]
        ref[2,4]=(0.386209196978)*x_ref[0]**o+(0.644169312279)*x_ref[0]+(-0.554562346917)*x_ref[1]**o+(-0.474292025255)*x_ref[1]
        ref[3,0]=(0.015012073686)*x_ref[0]**o+(-0.665804159938)*x_ref[0]+(-0.849030247328)*x_ref[1]**o+(-0.936779648988)*x_ref[1]
        ref[3,1]=(-0.818194334698)*x_ref[0]**o+(-0.569993362998)*x_ref[0]+(-0.806235611766)*x_ref[1]**o+(0.35347475408)*x_ref[1]
        ref[3,2]=(-0.227117779134)*x_ref[0]**o+(-0.981285175536)*x_ref[0]+(-0.226804205345)*x_ref[1]**o+(-0.0134258740334)*x_ref[1]
        ref[3,3]=(0.126716245369)*x_ref[0]**o+(0.135476076345)*x_ref[0]+(-0.641421845681)*x_ref[1]**o+(0.0595388928019)*x_ref[1]
        ref[3,4]=(-0.658260624128)*x_ref[0]**o+(0.124989215683)*x_ref[0]+(-0.949146047463)*x_ref[1]**o+(0.805277938593)*x_ref[1]
      else:
        arg[0,0]=(-0.859332605724)*x[0]**o+(-0.239005430756)*x[0]+(-0.52959833033)*x[1]**o+(0.725877930655)*x[1]+(-0.0348436114161)*x[2]**o+(0.47287744745)*x[2]
        arg[0,1]=(-0.0816886854463)*x[0]**o+(-0.626507446322)*x[0]+(-0.93804868959)*x[1]**o+(-0.450134311929)*x[1]+(0.0957946938715)*x[2]**o+(-0.236994496198)*x[2]
        arg[0,2]=(-0.669441310039)*x[0]**o+(-0.406451428536)*x[0]+(-0.292718530458)*x[1]**o+(-0.0948016380336)*x[1]+(-0.0943083566957)*x[2]**o+(0.747978324024)*x[2]
        arg[0,3]=(-0.0350228116485)*x[0]**o+(-0.967728170529)*x[0]+(-0.16425180976)*x[1]**o+(-0.141334783297)*x[1]+(0.179458630309)*x[2]**o+(0.45080506919)*x[2]
        arg[0,4]=(0.909578721407)*x[0]**o+(-0.341028350711)*x[0]+(-0.937703389458)*x[1]**o+(-0.845025015446)*x[1]+(-0.762836324997)*x[2]**o+(0.772817997464)*x[2]
        arg[1,0]=(0.75675349915)*x[0]**o+(-0.208827138169)*x[0]+(-0.0146742840091)*x[1]**o+(-0.1486215734)*x[1]+(0.205344647252)*x[2]**o+(-0.409293867732)*x[2]
        arg[1,1]=(-0.215359027624)*x[0]**o+(0.220661567012)*x[0]+(0.122954614324)*x[1]**o+(-0.888828539035)*x[1]+(-0.860535003713)*x[2]**o+(-0.582396229154)*x[2]
        arg[1,2]=(-0.252703089432)*x[0]**o+(0.294168447756)*x[0]+(0.323290819691)*x[1]**o+(-0.0440370164755)*x[1]+(0.487110700327)*x[2]**o+(-0.269109129573)*x[2]
        arg[1,3]=(0.0653521784751)*x[0]**o+(0.328048279094)*x[0]+(-0.414467594535)*x[1]**o+(0.436061182815)*x[1]+(0.483513283452)*x[2]**o+(-0.744560862926)*x[2]
        arg[1,4]=(0.0102955000926)*x[0]**o+(-0.636994970624)*x[0]+(0.83732393089)*x[1]**o+(0.914683806871)*x[1]+(0.574439856221)*x[2]**o+(-0.096821482591)*x[2]
        arg[2,0]=(-0.129692996812)*x[0]**o+(-0.617534967242)*x[0]+(0.299770969796)*x[1]**o+(0.93870530376)*x[1]+(-0.864448551767)*x[2]**o+(0.755129965839)*x[2]
        arg[2,1]=(-0.354122778719)*x[0]**o+(0.576654786045)*x[0]+(-0.965116346607)*x[1]**o+(0.0705856628847)*x[1]+(0.817059960545)*x[2]**o+(-0.597731384232)*x[2]
        arg[2,2]=(-0.705190107591)*x[0]**o+(0.876019062019)*x[0]+(-0.732733986501)*x[1]**o+(0.505334922081)*x[1]+(-0.616601928994)*x[2]**o+(-0.332268983863)*x[2]
        arg[2,3]=(0.663768017414)*x[0]**o+(-0.431682444208)*x[0]+(0.186159158088)*x[1]**o+(-0.689766933274)*x[1]+(-0.765854811539)*x[2]**o+(-0.402774599632)*x[2]
        arg[2,4]=(-0.698031877485)*x[0]**o+(0.914418269242)*x[0]+(-0.693564630363)*x[1]**o+(-0.18289238868)*x[1]+(0.988158500287)*x[2]**o+(-0.33952288549)*x[2]
        arg[3,0]=(-0.188499634873)*x[0]**o+(-0.10427816897)*x[0]+(-0.772866404794)*x[1]**o+(0.841928986479)*x[1]+(-0.866955639695)*x[2]**o+(-0.152515560713)*x[2]
        arg[3,1]=(0.937695460564)*x[0]**o+(-0.551079526492)*x[0]+(0.353043727721)*x[1]**o+(-0.604814667118)*x[1]+(0.200913251642)*x[2]**o+(-0.0497427852682)*x[2]
        arg[3,2]=(-0.49130784927)*x[0]**o+(0.537445967432)*x[0]+(-0.408371200391)*x[1]**o+(0.945593173362)*x[1]+(-0.0956969348675)*x[2]**o+(-0.174133549441)*x[2]
        arg[3,3]=(0.847790369559)*x[0]**o+(0.743993523287)*x[0]+(-0.810126540693)*x[1]**o+(-0.775699501886)*x[1]+(-0.479687101313)*x[2]**o+(0.627116825116)*x[2]
        arg[3,4]=(0.352455163407)*x[0]**o+(0.86343850667)*x[0]+(0.126742974996)*x[1]**o+(0.0175699888323)*x[1]+(0.964899127449)*x[2]**o+(-0.206749046822)*x[2]
        ref[0,0]=(-0.859332605724)*x_ref[0]**o+(-0.239005430756)*x_ref[0]+(-0.52959833033)*x_ref[1]**o+(0.725877930655)*x_ref[1]+(-0.0348436114161)*x_ref[2]**o+(0.47287744745)*x_ref[2]
        ref[0,1]=(-0.0816886854463)*x_ref[0]**o+(-0.626507446322)*x_ref[0]+(-0.93804868959)*x_ref[1]**o+(-0.450134311929)*x_ref[1]+(0.0957946938715)*x_ref[2]**o+(-0.236994496198)*x_ref[2]
        ref[0,2]=(-0.669441310039)*x_ref[0]**o+(-0.406451428536)*x_ref[0]+(-0.292718530458)*x_ref[1]**o+(-0.0948016380336)*x_ref[1]+(-0.0943083566957)*x_ref[2]**o+(0.747978324024)*x_ref[2]
        ref[0,3]=(-0.0350228116485)*x_ref[0]**o+(-0.967728170529)*x_ref[0]+(-0.16425180976)*x_ref[1]**o+(-0.141334783297)*x_ref[1]+(0.179458630309)*x_ref[2]**o+(0.45080506919)*x_ref[2]
        ref[0,4]=(0.909578721407)*x_ref[0]**o+(-0.341028350711)*x_ref[0]+(-0.937703389458)*x_ref[1]**o+(-0.845025015446)*x_ref[1]+(-0.762836324997)*x_ref[2]**o+(0.772817997464)*x_ref[2]
        ref[1,0]=(0.75675349915)*x_ref[0]**o+(-0.208827138169)*x_ref[0]+(-0.0146742840091)*x_ref[1]**o+(-0.1486215734)*x_ref[1]+(0.205344647252)*x_ref[2]**o+(-0.409293867732)*x_ref[2]
        ref[1,1]=(-0.215359027624)*x_ref[0]**o+(0.220661567012)*x_ref[0]+(0.122954614324)*x_ref[1]**o+(-0.888828539035)*x_ref[1]+(-0.860535003713)*x_ref[2]**o+(-0.582396229154)*x_ref[2]
        ref[1,2]=(-0.252703089432)*x_ref[0]**o+(0.294168447756)*x_ref[0]+(0.323290819691)*x_ref[1]**o+(-0.0440370164755)*x_ref[1]+(0.487110700327)*x_ref[2]**o+(-0.269109129573)*x_ref[2]
        ref[1,3]=(0.0653521784751)*x_ref[0]**o+(0.328048279094)*x_ref[0]+(-0.414467594535)*x_ref[1]**o+(0.436061182815)*x_ref[1]+(0.483513283452)*x_ref[2]**o+(-0.744560862926)*x_ref[2]
        ref[1,4]=(0.0102955000926)*x_ref[0]**o+(-0.636994970624)*x_ref[0]+(0.83732393089)*x_ref[1]**o+(0.914683806871)*x_ref[1]+(0.574439856221)*x_ref[2]**o+(-0.096821482591)*x_ref[2]
        ref[2,0]=(-0.129692996812)*x_ref[0]**o+(-0.617534967242)*x_ref[0]+(0.299770969796)*x_ref[1]**o+(0.93870530376)*x_ref[1]+(-0.864448551767)*x_ref[2]**o+(0.755129965839)*x_ref[2]
        ref[2,1]=(-0.354122778719)*x_ref[0]**o+(0.576654786045)*x_ref[0]+(-0.965116346607)*x_ref[1]**o+(0.0705856628847)*x_ref[1]+(0.817059960545)*x_ref[2]**o+(-0.597731384232)*x_ref[2]
        ref[2,2]=(-0.705190107591)*x_ref[0]**o+(0.876019062019)*x_ref[0]+(-0.732733986501)*x_ref[1]**o+(0.505334922081)*x_ref[1]+(-0.616601928994)*x_ref[2]**o+(-0.332268983863)*x_ref[2]
        ref[2,3]=(0.663768017414)*x_ref[0]**o+(-0.431682444208)*x_ref[0]+(0.186159158088)*x_ref[1]**o+(-0.689766933274)*x_ref[1]+(-0.765854811539)*x_ref[2]**o+(-0.402774599632)*x_ref[2]
        ref[2,4]=(-0.698031877485)*x_ref[0]**o+(0.914418269242)*x_ref[0]+(-0.693564630363)*x_ref[1]**o+(-0.18289238868)*x_ref[1]+(0.988158500287)*x_ref[2]**o+(-0.33952288549)*x_ref[2]
        ref[3,0]=(-0.188499634873)*x_ref[0]**o+(-0.10427816897)*x_ref[0]+(-0.772866404794)*x_ref[1]**o+(0.841928986479)*x_ref[1]+(-0.866955639695)*x_ref[2]**o+(-0.152515560713)*x_ref[2]
        ref[3,1]=(0.937695460564)*x_ref[0]**o+(-0.551079526492)*x_ref[0]+(0.353043727721)*x_ref[1]**o+(-0.604814667118)*x_ref[1]+(0.200913251642)*x_ref[2]**o+(-0.0497427852682)*x_ref[2]
        ref[3,2]=(-0.49130784927)*x_ref[0]**o+(0.537445967432)*x_ref[0]+(-0.408371200391)*x_ref[1]**o+(0.945593173362)*x_ref[1]+(-0.0956969348675)*x_ref[2]**o+(-0.174133549441)*x_ref[2]
        ref[3,3]=(0.847790369559)*x_ref[0]**o+(0.743993523287)*x_ref[0]+(-0.810126540693)*x_ref[1]**o+(-0.775699501886)*x_ref[1]+(-0.479687101313)*x_ref[2]**o+(0.627116825116)*x_ref[2]
        ref[3,4]=(0.352455163407)*x_ref[0]**o+(0.86343850667)*x_ref[0]+(0.126742974996)*x_ref[1]**o+(0.0175699888323)*x_ref[1]+(0.964899127449)*x_ref[2]**o+(-0.206749046822)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromData_ReducedFunctionOnContactOne_rank3(self):
      """
      tests interpolate for rank 3 Data onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactOne onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactOne(self.domain)
      x=w.getX()
      arg=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2),w_ref)
      if dim==2:
        arg[0,0,0]=(-0.396745847367)*x[0]**o+(-0.700719490691)*x[0]+(0.117558242545)*x[1]**o+(-0.161976142849)*x[1]
        arg[0,0,1]=(-0.490732277107)*x[0]**o+(0.526833149216)*x[0]+(-0.136309303122)*x[1]**o+(0.629465649533)*x[1]
        arg[0,1,0]=(-0.242474968121)*x[0]**o+(-0.76270698332)*x[0]+(-0.0979504547153)*x[1]**o+(-0.0743521177641)*x[1]
        arg[0,1,1]=(0.641246476843)*x[0]**o+(0.0158147797433)*x[0]+(-0.213898399703)*x[1]**o+(-0.378401519675)*x[1]
        arg[1,0,0]=(0.603700544573)*x[0]**o+(-0.993751144334)*x[0]+(0.466222025692)*x[1]**o+(0.457689495241)*x[1]
        arg[1,0,1]=(-0.727287858841)*x[0]**o+(-0.177841994955)*x[0]+(0.720237448692)*x[1]**o+(0.716175629178)*x[1]
        arg[1,1,0]=(0.784514682242)*x[0]**o+(0.681214612353)*x[0]+(0.404266860166)*x[1]**o+(0.871218157219)*x[1]
        arg[1,1,1]=(-0.918634586367)*x[0]**o+(-0.722365105889)*x[0]+(-0.0337202979587)*x[1]**o+(-0.487790376172)*x[1]
        arg[2,0,0]=(0.159515902499)*x[0]**o+(-0.0650417831188)*x[0]+(0.107260481333)*x[1]**o+(0.469096212511)*x[1]
        arg[2,0,1]=(0.0436003865409)*x[0]**o+(-0.67488837592)*x[0]+(0.547761443272)*x[1]**o+(-0.69222031017)*x[1]
        arg[2,1,0]=(-0.153636036391)*x[0]**o+(0.0415911821723)*x[0]+(0.421990370678)*x[1]**o+(-0.128883832414)*x[1]
        arg[2,1,1]=(0.513420393955)*x[0]**o+(-0.769636964711)*x[0]+(0.874383588901)*x[1]**o+(-0.357923227767)*x[1]
        arg[3,0,0]=(0.943275907357)*x[0]**o+(0.372378913231)*x[0]+(0.559656674007)*x[1]**o+(-0.878007664293)*x[1]
        arg[3,0,1]=(-0.541535388034)*x[0]**o+(-0.0510200932939)*x[0]+(-0.448618747155)*x[1]**o+(0.693514427768)*x[1]
        arg[3,1,0]=(-0.287622342268)*x[0]**o+(0.8333134988)*x[0]+(-0.691930501782)*x[1]**o+(-0.659618596505)*x[1]
        arg[3,1,1]=(0.44446133556)*x[0]**o+(0.697704081094)*x[0]+(0.0679863572322)*x[1]**o+(-0.949443102916)*x[1]
        arg[4,0,0]=(0.546245770076)*x[0]**o+(-0.466891879347)*x[0]+(-0.608261901126)*x[1]**o+(0.669561643667)*x[1]
        arg[4,0,1]=(-0.522497485476)*x[0]**o+(0.70886716903)*x[0]+(-0.296815361132)*x[1]**o+(0.500540583082)*x[1]
        arg[4,1,0]=(-0.841653490079)*x[0]**o+(0.400589086976)*x[0]+(0.865994660457)*x[1]**o+(0.0205817210969)*x[1]
        arg[4,1,1]=(-0.64369513787)*x[0]**o+(0.883032618928)*x[0]+(-0.178766897224)*x[1]**o+(0.748041632322)*x[1]
        arg[5,0,0]=(0.471810615096)*x[0]**o+(-0.140340964274)*x[0]+(-0.7063314968)*x[1]**o+(0.0524231879837)*x[1]
        arg[5,0,1]=(-0.51339482213)*x[0]**o+(-0.81447661092)*x[0]+(0.437085868301)*x[1]**o+(0.862678017842)*x[1]
        arg[5,1,0]=(-0.752858172473)*x[0]**o+(0.325827525243)*x[0]+(-0.702286757187)*x[1]**o+(-0.467810951898)*x[1]
        arg[5,1,1]=(-0.256879797442)*x[0]**o+(0.257067666229)*x[0]+(0.912308244411)*x[1]**o+(0.0620337856352)*x[1]
        ref[0,0,0]=(-0.396745847367)*x_ref[0]**o+(-0.700719490691)*x_ref[0]+(0.117558242545)*x_ref[1]**o+(-0.161976142849)*x_ref[1]
        ref[0,0,1]=(-0.490732277107)*x_ref[0]**o+(0.526833149216)*x_ref[0]+(-0.136309303122)*x_ref[1]**o+(0.629465649533)*x_ref[1]
        ref[0,1,0]=(-0.242474968121)*x_ref[0]**o+(-0.76270698332)*x_ref[0]+(-0.0979504547153)*x_ref[1]**o+(-0.0743521177641)*x_ref[1]
        ref[0,1,1]=(0.641246476843)*x_ref[0]**o+(0.0158147797433)*x_ref[0]+(-0.213898399703)*x_ref[1]**o+(-0.378401519675)*x_ref[1]
        ref[1,0,0]=(0.603700544573)*x_ref[0]**o+(-0.993751144334)*x_ref[0]+(0.466222025692)*x_ref[1]**o+(0.457689495241)*x_ref[1]
        ref[1,0,1]=(-0.727287858841)*x_ref[0]**o+(-0.177841994955)*x_ref[0]+(0.720237448692)*x_ref[1]**o+(0.716175629178)*x_ref[1]
        ref[1,1,0]=(0.784514682242)*x_ref[0]**o+(0.681214612353)*x_ref[0]+(0.404266860166)*x_ref[1]**o+(0.871218157219)*x_ref[1]
        ref[1,1,1]=(-0.918634586367)*x_ref[0]**o+(-0.722365105889)*x_ref[0]+(-0.0337202979587)*x_ref[1]**o+(-0.487790376172)*x_ref[1]
        ref[2,0,0]=(0.159515902499)*x_ref[0]**o+(-0.0650417831188)*x_ref[0]+(0.107260481333)*x_ref[1]**o+(0.469096212511)*x_ref[1]
        ref[2,0,1]=(0.0436003865409)*x_ref[0]**o+(-0.67488837592)*x_ref[0]+(0.547761443272)*x_ref[1]**o+(-0.69222031017)*x_ref[1]
        ref[2,1,0]=(-0.153636036391)*x_ref[0]**o+(0.0415911821723)*x_ref[0]+(0.421990370678)*x_ref[1]**o+(-0.128883832414)*x_ref[1]
        ref[2,1,1]=(0.513420393955)*x_ref[0]**o+(-0.769636964711)*x_ref[0]+(0.874383588901)*x_ref[1]**o+(-0.357923227767)*x_ref[1]
        ref[3,0,0]=(0.943275907357)*x_ref[0]**o+(0.372378913231)*x_ref[0]+(0.559656674007)*x_ref[1]**o+(-0.878007664293)*x_ref[1]
        ref[3,0,1]=(-0.541535388034)*x_ref[0]**o+(-0.0510200932939)*x_ref[0]+(-0.448618747155)*x_ref[1]**o+(0.693514427768)*x_ref[1]
        ref[3,1,0]=(-0.287622342268)*x_ref[0]**o+(0.8333134988)*x_ref[0]+(-0.691930501782)*x_ref[1]**o+(-0.659618596505)*x_ref[1]
        ref[3,1,1]=(0.44446133556)*x_ref[0]**o+(0.697704081094)*x_ref[0]+(0.0679863572322)*x_ref[1]**o+(-0.949443102916)*x_ref[1]
        ref[4,0,0]=(0.546245770076)*x_ref[0]**o+(-0.466891879347)*x_ref[0]+(-0.608261901126)*x_ref[1]**o+(0.669561643667)*x_ref[1]
        ref[4,0,1]=(-0.522497485476)*x_ref[0]**o+(0.70886716903)*x_ref[0]+(-0.296815361132)*x_ref[1]**o+(0.500540583082)*x_ref[1]
        ref[4,1,0]=(-0.841653490079)*x_ref[0]**o+(0.400589086976)*x_ref[0]+(0.865994660457)*x_ref[1]**o+(0.0205817210969)*x_ref[1]
        ref[4,1,1]=(-0.64369513787)*x_ref[0]**o+(0.883032618928)*x_ref[0]+(-0.178766897224)*x_ref[1]**o+(0.748041632322)*x_ref[1]
        ref[5,0,0]=(0.471810615096)*x_ref[0]**o+(-0.140340964274)*x_ref[0]+(-0.7063314968)*x_ref[1]**o+(0.0524231879837)*x_ref[1]
        ref[5,0,1]=(-0.51339482213)*x_ref[0]**o+(-0.81447661092)*x_ref[0]+(0.437085868301)*x_ref[1]**o+(0.862678017842)*x_ref[1]
        ref[5,1,0]=(-0.752858172473)*x_ref[0]**o+(0.325827525243)*x_ref[0]+(-0.702286757187)*x_ref[1]**o+(-0.467810951898)*x_ref[1]
        ref[5,1,1]=(-0.256879797442)*x_ref[0]**o+(0.257067666229)*x_ref[0]+(0.912308244411)*x_ref[1]**o+(0.0620337856352)*x_ref[1]
      else:
        arg[0,0,0]=(0.564502533309)*x[0]**o+(-0.237721559752)*x[0]+(0.225601658644)*x[1]**o+(0.967190188292)*x[1]+(0.471760897542)*x[2]**o+(0.79404420728)*x[2]
        arg[0,0,1]=(-0.373762346136)*x[0]**o+(-0.643766021731)*x[0]+(0.088028383792)*x[1]**o+(-0.541163410313)*x[1]+(0.463315911637)*x[2]**o+(-0.596740807595)*x[2]
        arg[0,1,0]=(-0.526694453842)*x[0]**o+(0.405010889931)*x[0]+(-0.0153807962972)*x[1]**o+(-0.0446620712949)*x[1]+(0.00553918668197)*x[2]**o+(0.72170514564)*x[2]
        arg[0,1,1]=(-0.00750073616089)*x[0]**o+(-0.60431903208)*x[0]+(0.790032415931)*x[1]**o+(0.272709398336)*x[1]+(0.228539600712)*x[2]**o+(0.62284140325)*x[2]
        arg[1,0,0]=(-0.892797595418)*x[0]**o+(-0.151608141828)*x[0]+(0.313142074823)*x[1]**o+(0.80280439466)*x[1]+(-0.593056273594)*x[2]**o+(-0.41551679707)*x[2]
        arg[1,0,1]=(-0.385715092974)*x[0]**o+(-0.77884107065)*x[0]+(-0.0804019137225)*x[1]**o+(-0.240129932101)*x[1]+(0.238145788042)*x[2]**o+(-0.806337666628)*x[2]
        arg[1,1,0]=(0.940492459417)*x[0]**o+(-0.0054460073194)*x[0]+(-0.697916565168)*x[1]**o+(-0.949456184725)*x[1]+(-0.115461214789)*x[2]**o+(-0.403742409948)*x[2]
        arg[1,1,1]=(-0.711716124415)*x[0]**o+(0.586131021871)*x[0]+(0.625988666837)*x[1]**o+(0.638433219635)*x[1]+(0.971169174015)*x[2]**o+(-0.064949626884)*x[2]
        arg[2,0,0]=(-0.891973333325)*x[0]**o+(0.76090478364)*x[0]+(0.439956136239)*x[1]**o+(-0.74807071296)*x[1]+(-0.209999803467)*x[2]**o+(0.404396254503)*x[2]
        arg[2,0,1]=(0.559776288735)*x[0]**o+(0.284643618707)*x[0]+(0.0527799359967)*x[1]**o+(-0.201660204363)*x[1]+(0.590722655898)*x[2]**o+(-0.24940757474)*x[2]
        arg[2,1,0]=(-0.747588234432)*x[0]**o+(0.0196702046746)*x[0]+(0.0454238912973)*x[1]**o+(0.174053915643)*x[1]+(-0.868554688657)*x[2]**o+(-0.127425849813)*x[2]
        arg[2,1,1]=(-0.324276984008)*x[0]**o+(-0.494912096003)*x[0]+(-0.780953487415)*x[1]**o+(0.225787179491)*x[1]+(-0.760330660035)*x[2]**o+(0.0440722700759)*x[2]
        arg[3,0,0]=(0.37479028021)*x[0]**o+(-0.869009335783)*x[0]+(0.811673580204)*x[1]**o+(0.521877505351)*x[1]+(-0.508710979256)*x[2]**o+(0.558085136638)*x[2]
        arg[3,0,1]=(0.815143535066)*x[0]**o+(0.867977183071)*x[0]+(0.0315543942614)*x[1]**o+(0.786379597601)*x[1]+(0.0433279381339)*x[2]**o+(-0.56554445624)*x[2]
        arg[3,1,0]=(-0.948224238625)*x[0]**o+(-0.759282528831)*x[0]+(0.511802336753)*x[1]**o+(0.331073342499)*x[1]+(0.295241697427)*x[2]**o+(0.154864664771)*x[2]
        arg[3,1,1]=(-0.982634132939)*x[0]**o+(-0.0762014431695)*x[0]+(-0.0797462613351)*x[1]**o+(-0.512305271957)*x[1]+(-0.348883505418)*x[2]**o+(0.00580412526927)*x[2]
        arg[4,0,0]=(0.186597109908)*x[0]**o+(0.051957206238)*x[0]+(0.572882754553)*x[1]**o+(-0.114338954084)*x[1]+(-0.517239439578)*x[2]**o+(-0.298287404607)*x[2]
        arg[4,0,1]=(0.637628569107)*x[0]**o+(0.228562669958)*x[0]+(0.053657321561)*x[1]**o+(0.762909608846)*x[1]+(0.873936819574)*x[2]**o+(-0.20000969514)*x[2]
        arg[4,1,0]=(-0.717459489066)*x[0]**o+(-0.462174191129)*x[0]+(-0.70491180102)*x[1]**o+(-0.406072010622)*x[1]+(-0.903872259447)*x[2]**o+(0.487491537826)*x[2]
        arg[4,1,1]=(0.947654286923)*x[0]**o+(-0.228830597313)*x[0]+(-0.501442713655)*x[1]**o+(0.282437368749)*x[1]+(0.543592950002)*x[2]**o+(0.5400198247)*x[2]
        arg[5,0,0]=(0.144349494033)*x[0]**o+(-0.635328703132)*x[0]+(-0.797851162499)*x[1]**o+(0.356874780185)*x[1]+(-0.205725457569)*x[2]**o+(-0.674962512018)*x[2]
        arg[5,0,1]=(-0.728948504419)*x[0]**o+(0.944337871242)*x[0]+(-0.790388944678)*x[1]**o+(0.59119693076)*x[1]+(0.9569152246)*x[2]**o+(-0.205404245177)*x[2]
        arg[5,1,0]=(0.477597678348)*x[0]**o+(-0.318993929605)*x[0]+(0.481029970286)*x[1]**o+(0.708696918395)*x[1]+(0.0531716082887)*x[2]**o+(-0.847697491061)*x[2]
        arg[5,1,1]=(-0.458572001979)*x[0]**o+(0.582048244354)*x[0]+(-0.040756132547)*x[1]**o+(-0.953201198076)*x[1]+(-0.971859723822)*x[2]**o+(-0.796426143181)*x[2]
        ref[0,0,0]=(0.564502533309)*x_ref[0]**o+(-0.237721559752)*x_ref[0]+(0.225601658644)*x_ref[1]**o+(0.967190188292)*x_ref[1]+(0.471760897542)*x_ref[2]**o+(0.79404420728)*x_ref[2]
        ref[0,0,1]=(-0.373762346136)*x_ref[0]**o+(-0.643766021731)*x_ref[0]+(0.088028383792)*x_ref[1]**o+(-0.541163410313)*x_ref[1]+(0.463315911637)*x_ref[2]**o+(-0.596740807595)*x_ref[2]
        ref[0,1,0]=(-0.526694453842)*x_ref[0]**o+(0.405010889931)*x_ref[0]+(-0.0153807962972)*x_ref[1]**o+(-0.0446620712949)*x_ref[1]+(0.00553918668197)*x_ref[2]**o+(0.72170514564)*x_ref[2]
        ref[0,1,1]=(-0.00750073616089)*x_ref[0]**o+(-0.60431903208)*x_ref[0]+(0.790032415931)*x_ref[1]**o+(0.272709398336)*x_ref[1]+(0.228539600712)*x_ref[2]**o+(0.62284140325)*x_ref[2]
        ref[1,0,0]=(-0.892797595418)*x_ref[0]**o+(-0.151608141828)*x_ref[0]+(0.313142074823)*x_ref[1]**o+(0.80280439466)*x_ref[1]+(-0.593056273594)*x_ref[2]**o+(-0.41551679707)*x_ref[2]
        ref[1,0,1]=(-0.385715092974)*x_ref[0]**o+(-0.77884107065)*x_ref[0]+(-0.0804019137225)*x_ref[1]**o+(-0.240129932101)*x_ref[1]+(0.238145788042)*x_ref[2]**o+(-0.806337666628)*x_ref[2]
        ref[1,1,0]=(0.940492459417)*x_ref[0]**o+(-0.0054460073194)*x_ref[0]+(-0.697916565168)*x_ref[1]**o+(-0.949456184725)*x_ref[1]+(-0.115461214789)*x_ref[2]**o+(-0.403742409948)*x_ref[2]
        ref[1,1,1]=(-0.711716124415)*x_ref[0]**o+(0.586131021871)*x_ref[0]+(0.625988666837)*x_ref[1]**o+(0.638433219635)*x_ref[1]+(0.971169174015)*x_ref[2]**o+(-0.064949626884)*x_ref[2]
        ref[2,0,0]=(-0.891973333325)*x_ref[0]**o+(0.76090478364)*x_ref[0]+(0.439956136239)*x_ref[1]**o+(-0.74807071296)*x_ref[1]+(-0.209999803467)*x_ref[2]**o+(0.404396254503)*x_ref[2]
        ref[2,0,1]=(0.559776288735)*x_ref[0]**o+(0.284643618707)*x_ref[0]+(0.0527799359967)*x_ref[1]**o+(-0.201660204363)*x_ref[1]+(0.590722655898)*x_ref[2]**o+(-0.24940757474)*x_ref[2]
        ref[2,1,0]=(-0.747588234432)*x_ref[0]**o+(0.0196702046746)*x_ref[0]+(0.0454238912973)*x_ref[1]**o+(0.174053915643)*x_ref[1]+(-0.868554688657)*x_ref[2]**o+(-0.127425849813)*x_ref[2]
        ref[2,1,1]=(-0.324276984008)*x_ref[0]**o+(-0.494912096003)*x_ref[0]+(-0.780953487415)*x_ref[1]**o+(0.225787179491)*x_ref[1]+(-0.760330660035)*x_ref[2]**o+(0.0440722700759)*x_ref[2]
        ref[3,0,0]=(0.37479028021)*x_ref[0]**o+(-0.869009335783)*x_ref[0]+(0.811673580204)*x_ref[1]**o+(0.521877505351)*x_ref[1]+(-0.508710979256)*x_ref[2]**o+(0.558085136638)*x_ref[2]
        ref[3,0,1]=(0.815143535066)*x_ref[0]**o+(0.867977183071)*x_ref[0]+(0.0315543942614)*x_ref[1]**o+(0.786379597601)*x_ref[1]+(0.0433279381339)*x_ref[2]**o+(-0.56554445624)*x_ref[2]
        ref[3,1,0]=(-0.948224238625)*x_ref[0]**o+(-0.759282528831)*x_ref[0]+(0.511802336753)*x_ref[1]**o+(0.331073342499)*x_ref[1]+(0.295241697427)*x_ref[2]**o+(0.154864664771)*x_ref[2]
        ref[3,1,1]=(-0.982634132939)*x_ref[0]**o+(-0.0762014431695)*x_ref[0]+(-0.0797462613351)*x_ref[1]**o+(-0.512305271957)*x_ref[1]+(-0.348883505418)*x_ref[2]**o+(0.00580412526927)*x_ref[2]
        ref[4,0,0]=(0.186597109908)*x_ref[0]**o+(0.051957206238)*x_ref[0]+(0.572882754553)*x_ref[1]**o+(-0.114338954084)*x_ref[1]+(-0.517239439578)*x_ref[2]**o+(-0.298287404607)*x_ref[2]
        ref[4,0,1]=(0.637628569107)*x_ref[0]**o+(0.228562669958)*x_ref[0]+(0.053657321561)*x_ref[1]**o+(0.762909608846)*x_ref[1]+(0.873936819574)*x_ref[2]**o+(-0.20000969514)*x_ref[2]
        ref[4,1,0]=(-0.717459489066)*x_ref[0]**o+(-0.462174191129)*x_ref[0]+(-0.70491180102)*x_ref[1]**o+(-0.406072010622)*x_ref[1]+(-0.903872259447)*x_ref[2]**o+(0.487491537826)*x_ref[2]
        ref[4,1,1]=(0.947654286923)*x_ref[0]**o+(-0.228830597313)*x_ref[0]+(-0.501442713655)*x_ref[1]**o+(0.282437368749)*x_ref[1]+(0.543592950002)*x_ref[2]**o+(0.5400198247)*x_ref[2]
        ref[5,0,0]=(0.144349494033)*x_ref[0]**o+(-0.635328703132)*x_ref[0]+(-0.797851162499)*x_ref[1]**o+(0.356874780185)*x_ref[1]+(-0.205725457569)*x_ref[2]**o+(-0.674962512018)*x_ref[2]
        ref[5,0,1]=(-0.728948504419)*x_ref[0]**o+(0.944337871242)*x_ref[0]+(-0.790388944678)*x_ref[1]**o+(0.59119693076)*x_ref[1]+(0.9569152246)*x_ref[2]**o+(-0.205404245177)*x_ref[2]
        ref[5,1,0]=(0.477597678348)*x_ref[0]**o+(-0.318993929605)*x_ref[0]+(0.481029970286)*x_ref[1]**o+(0.708696918395)*x_ref[1]+(0.0531716082887)*x_ref[2]**o+(-0.847697491061)*x_ref[2]
        ref[5,1,1]=(-0.458572001979)*x_ref[0]**o+(0.582048244354)*x_ref[0]+(-0.040756132547)*x_ref[1]**o+(-0.953201198076)*x_ref[1]+(-0.971859723822)*x_ref[2]**o+(-0.796426143181)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromData_ReducedFunctionOnContactOne_rank4(self):
      """
      tests interpolate for rank 4 Data onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactOne onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactOne(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5, 3, 2),w)
      ref=Data(0,(4, 5, 3, 2),w_ref)
      if dim==2:
        arg[0,0,0,0]=(-0.937777610663)*x[0]**o+(-0.0228520946632)*x[0]+(0.642385396583)*x[1]**o+(-0.803821945812)*x[1]
        arg[0,0,0,1]=(-0.398785307492)*x[0]**o+(0.0450288952883)*x[0]+(-0.777780548133)*x[1]**o+(0.0273772099414)*x[1]
        arg[0,0,1,0]=(-0.667280086925)*x[0]**o+(-0.934642710274)*x[0]+(0.565327494491)*x[1]**o+(0.988521769056)*x[1]
        arg[0,0,1,1]=(-0.225562383498)*x[0]**o+(-0.280566031134)*x[0]+(-0.295768954831)*x[1]**o+(0.102759013802)*x[1]
        arg[0,0,2,0]=(-0.179577502546)*x[0]**o+(-0.156932777074)*x[0]+(-0.145616153633)*x[1]**o+(0.971876899135)*x[1]
        arg[0,0,2,1]=(-0.840266409118)*x[0]**o+(0.704356785022)*x[0]+(0.740091470711)*x[1]**o+(0.708478499023)*x[1]
        arg[0,1,0,0]=(-0.487161662806)*x[0]**o+(0.565313908399)*x[0]+(-0.33708697293)*x[1]**o+(0.731675966546)*x[1]
        arg[0,1,0,1]=(-0.135615857963)*x[0]**o+(0.265408777025)*x[0]+(-0.168207260227)*x[1]**o+(-0.598974659317)*x[1]
        arg[0,1,1,0]=(0.0533944010258)*x[0]**o+(0.819248906738)*x[0]+(-0.837005251524)*x[1]**o+(0.119613618595)*x[1]
        arg[0,1,1,1]=(-0.257520627047)*x[0]**o+(0.339264545713)*x[0]+(0.137721228188)*x[1]**o+(-0.512251030979)*x[1]
        arg[0,1,2,0]=(-0.797070396749)*x[0]**o+(0.690173285077)*x[0]+(-0.213206187271)*x[1]**o+(-0.873999125945)*x[1]
        arg[0,1,2,1]=(-0.847662300367)*x[0]**o+(0.785781114266)*x[0]+(-0.456918398811)*x[1]**o+(0.458881079947)*x[1]
        arg[0,2,0,0]=(-0.385269163334)*x[0]**o+(0.135941228964)*x[0]+(0.233245620648)*x[1]**o+(-0.246282209626)*x[1]
        arg[0,2,0,1]=(0.372914636218)*x[0]**o+(-0.160283657003)*x[0]+(0.0565043889781)*x[1]**o+(0.199715522021)*x[1]
        arg[0,2,1,0]=(0.676198390598)*x[0]**o+(0.938025122014)*x[0]+(0.161074696651)*x[1]**o+(-0.155495646964)*x[1]
        arg[0,2,1,1]=(-0.709286924155)*x[0]**o+(-0.173716284776)*x[0]+(0.0576435484694)*x[1]**o+(0.673924697832)*x[1]
        arg[0,2,2,0]=(-0.436468652969)*x[0]**o+(0.117279009082)*x[0]+(-0.731436369191)*x[1]**o+(0.919578503469)*x[1]
        arg[0,2,2,1]=(-0.691042646151)*x[0]**o+(-0.606293725574)*x[0]+(0.657291692759)*x[1]**o+(-0.0146398994729)*x[1]
        arg[0,3,0,0]=(0.365939313719)*x[0]**o+(-0.221236913792)*x[0]+(-0.729594967883)*x[1]**o+(0.0179024719114)*x[1]
        arg[0,3,0,1]=(0.135892133258)*x[0]**o+(0.728069461936)*x[0]+(-0.365691795279)*x[1]**o+(0.877708599922)*x[1]
        arg[0,3,1,0]=(-0.853312456817)*x[0]**o+(-0.713031969049)*x[0]+(0.274755827464)*x[1]**o+(-0.527330699033)*x[1]
        arg[0,3,1,1]=(-0.30358584687)*x[0]**o+(0.871629391968)*x[0]+(0.906915162645)*x[1]**o+(0.499955429041)*x[1]
        arg[0,3,2,0]=(-0.880532587427)*x[0]**o+(0.265729672258)*x[0]+(0.528677136349)*x[1]**o+(-0.224602964925)*x[1]
        arg[0,3,2,1]=(0.158632463532)*x[0]**o+(-0.665418329643)*x[0]+(0.957661069067)*x[1]**o+(-0.23771309434)*x[1]
        arg[0,4,0,0]=(-0.128073514905)*x[0]**o+(-0.452543798055)*x[0]+(0.303808626277)*x[1]**o+(0.841041779759)*x[1]
        arg[0,4,0,1]=(0.0219309964781)*x[0]**o+(-0.949488393443)*x[0]+(0.0436905404663)*x[1]**o+(-0.304527242795)*x[1]
        arg[0,4,1,0]=(-0.792275137775)*x[0]**o+(0.450079829644)*x[0]+(0.162879981637)*x[1]**o+(-0.185839015805)*x[1]
        arg[0,4,1,1]=(0.761932713893)*x[0]**o+(0.932601232945)*x[0]+(-0.606328305783)*x[1]**o+(-0.123973545682)*x[1]
        arg[0,4,2,0]=(-0.514088304908)*x[0]**o+(0.987668774807)*x[0]+(0.866078887168)*x[1]**o+(-0.608518224484)*x[1]
        arg[0,4,2,1]=(-0.427564608028)*x[0]**o+(0.231599614445)*x[0]+(-0.406862229819)*x[1]**o+(0.358456864265)*x[1]
        arg[1,0,0,0]=(0.342092171801)*x[0]**o+(-0.264522675185)*x[0]+(-0.37976917867)*x[1]**o+(0.778596602561)*x[1]
        arg[1,0,0,1]=(0.672156473235)*x[0]**o+(0.320317500856)*x[0]+(0.514206881296)*x[1]**o+(0.602594631705)*x[1]
        arg[1,0,1,0]=(0.914472920293)*x[0]**o+(0.525647943447)*x[0]+(0.685613621597)*x[1]**o+(0.482308039231)*x[1]
        arg[1,0,1,1]=(-0.727982874425)*x[0]**o+(-0.417964463065)*x[0]+(0.825853988063)*x[1]**o+(0.236134760252)*x[1]
        arg[1,0,2,0]=(0.922186381756)*x[0]**o+(-0.140522762044)*x[0]+(-0.282169431666)*x[1]**o+(0.778383988397)*x[1]
        arg[1,0,2,1]=(0.307296736289)*x[0]**o+(0.589844868569)*x[0]+(0.64571457752)*x[1]**o+(0.749333079485)*x[1]
        arg[1,1,0,0]=(0.908345176703)*x[0]**o+(-0.476053147164)*x[0]+(0.0828501189289)*x[1]**o+(0.280657117705)*x[1]
        arg[1,1,0,1]=(0.396128814097)*x[0]**o+(0.368821575482)*x[0]+(-0.916584602334)*x[1]**o+(-0.149377992159)*x[1]
        arg[1,1,1,0]=(-0.38468474313)*x[0]**o+(0.304742499301)*x[0]+(0.638083865846)*x[1]**o+(-0.332046460663)*x[1]
        arg[1,1,1,1]=(0.845097787138)*x[0]**o+(-0.34949498164)*x[0]+(0.229157202572)*x[1]**o+(0.677584672856)*x[1]
        arg[1,1,2,0]=(0.0280692825358)*x[0]**o+(0.567451370107)*x[0]+(0.66934188305)*x[1]**o+(-0.477197213188)*x[1]
        arg[1,1,2,1]=(0.375411487264)*x[0]**o+(0.55086548839)*x[0]+(-0.392135351366)*x[1]**o+(0.91971334589)*x[1]
        arg[1,2,0,0]=(-0.7659270368)*x[0]**o+(-0.338082792509)*x[0]+(-0.26272975232)*x[1]**o+(0.267936454568)*x[1]
        arg[1,2,0,1]=(0.918125455771)*x[0]**o+(0.182896655906)*x[0]+(0.525396835337)*x[1]**o+(0.201918789801)*x[1]
        arg[1,2,1,0]=(-0.54472490266)*x[0]**o+(0.693714832775)*x[0]+(0.898278265794)*x[1]**o+(0.345095187159)*x[1]
        arg[1,2,1,1]=(-0.498607752195)*x[0]**o+(0.84678344253)*x[0]+(0.678992455984)*x[1]**o+(-0.597396262326)*x[1]
        arg[1,2,2,0]=(0.215267998554)*x[0]**o+(0.384479388838)*x[0]+(-0.09078862662)*x[1]**o+(-0.0162185695524)*x[1]
        arg[1,2,2,1]=(-0.335574750173)*x[0]**o+(-0.439933690512)*x[0]+(0.194601817488)*x[1]**o+(0.795181445633)*x[1]
        arg[1,3,0,0]=(0.862180273604)*x[0]**o+(0.276662657248)*x[0]+(-0.908827123328)*x[1]**o+(0.290406540669)*x[1]
        arg[1,3,0,1]=(-0.620441636176)*x[0]**o+(0.448236189204)*x[0]+(-0.503817375869)*x[1]**o+(0.689447469467)*x[1]
        arg[1,3,1,0]=(0.126327572417)*x[0]**o+(-0.476552913848)*x[0]+(0.362273273226)*x[1]**o+(-0.816228830372)*x[1]
        arg[1,3,1,1]=(-0.65091977023)*x[0]**o+(0.0316560718061)*x[0]+(-0.922906273452)*x[1]**o+(-0.584811809954)*x[1]
        arg[1,3,2,0]=(-0.675982262546)*x[0]**o+(0.23518662745)*x[0]+(-0.270229269162)*x[1]**o+(-0.00300656227686)*x[1]
        arg[1,3,2,1]=(0.55691633063)*x[0]**o+(-0.0643123929496)*x[0]+(0.212825067376)*x[1]**o+(-0.0807098593265)*x[1]
        arg[1,4,0,0]=(0.199100268435)*x[0]**o+(-0.358028564112)*x[0]+(0.130301315926)*x[1]**o+(-0.255752658178)*x[1]
        arg[1,4,0,1]=(0.821720568968)*x[0]**o+(0.193634063182)*x[0]+(0.862353589302)*x[1]**o+(-0.42971617129)*x[1]
        arg[1,4,1,0]=(0.51160977767)*x[0]**o+(0.326743318954)*x[0]+(-0.866878934278)*x[1]**o+(-0.927585776234)*x[1]
        arg[1,4,1,1]=(0.299835915861)*x[0]**o+(-0.969027084981)*x[0]+(-0.589772230832)*x[1]**o+(0.382433302029)*x[1]
        arg[1,4,2,0]=(0.473101160384)*x[0]**o+(-0.591124830283)*x[0]+(0.0929141413982)*x[1]**o+(-0.529811769054)*x[1]
        arg[1,4,2,1]=(-0.712363897962)*x[0]**o+(-0.259229026597)*x[0]+(0.00983651162685)*x[1]**o+(-0.329312308231)*x[1]
        arg[2,0,0,0]=(-0.507414918232)*x[0]**o+(0.668167664289)*x[0]+(0.654870826953)*x[1]**o+(-0.411963068213)*x[1]
        arg[2,0,0,1]=(-0.894915678458)*x[0]**o+(0.647579368695)*x[0]+(0.222274441569)*x[1]**o+(0.737972296743)*x[1]
        arg[2,0,1,0]=(-0.457176106717)*x[0]**o+(0.99997698232)*x[0]+(0.524381951875)*x[1]**o+(0.796302331418)*x[1]
        arg[2,0,1,1]=(-0.761912711581)*x[0]**o+(-0.995235985398)*x[0]+(-0.371926591709)*x[1]**o+(-0.37949019966)*x[1]
        arg[2,0,2,0]=(0.402826798397)*x[0]**o+(0.473547871218)*x[0]+(0.801636706721)*x[1]**o+(0.314221308918)*x[1]
        arg[2,0,2,1]=(0.0416846902183)*x[0]**o+(0.815306853966)*x[0]+(-0.526315166764)*x[1]**o+(-0.327935702076)*x[1]
        arg[2,1,0,0]=(-0.53829655239)*x[0]**o+(-0.467914397053)*x[0]+(0.392659650792)*x[1]**o+(0.309531884089)*x[1]
        arg[2,1,0,1]=(-0.768324371377)*x[0]**o+(0.136336455054)*x[0]+(0.0653485752909)*x[1]**o+(-0.0181923467911)*x[1]
        arg[2,1,1,0]=(0.121561771322)*x[0]**o+(-0.794575616334)*x[0]+(0.038194779569)*x[1]**o+(-0.695864614551)*x[1]
        arg[2,1,1,1]=(0.941872575674)*x[0]**o+(0.322067879395)*x[0]+(0.176873583908)*x[1]**o+(0.266964433323)*x[1]
        arg[2,1,2,0]=(0.324335817101)*x[0]**o+(0.609048475305)*x[0]+(0.350973789003)*x[1]**o+(-0.284995830372)*x[1]
        arg[2,1,2,1]=(0.939941132515)*x[0]**o+(0.782468124836)*x[0]+(-0.238539378134)*x[1]**o+(-0.515203802574)*x[1]
        arg[2,2,0,0]=(0.080179317168)*x[0]**o+(-0.555769741953)*x[0]+(0.283304726007)*x[1]**o+(0.844935300942)*x[1]
        arg[2,2,0,1]=(0.52001819752)*x[0]**o+(-0.105912273152)*x[0]+(-0.360523424917)*x[1]**o+(-0.179135541646)*x[1]
        arg[2,2,1,0]=(0.719593478448)*x[0]**o+(-0.722945454687)*x[0]+(0.0316404625439)*x[1]**o+(0.154290328891)*x[1]
        arg[2,2,1,1]=(0.906382302919)*x[0]**o+(-0.137753840588)*x[0]+(-0.626417865101)*x[1]**o+(-0.890642745149)*x[1]
        arg[2,2,2,0]=(0.618763086671)*x[0]**o+(-0.267876013516)*x[0]+(0.582353922702)*x[1]**o+(-0.190278788501)*x[1]
        arg[2,2,2,1]=(0.778824546516)*x[0]**o+(-0.00837218512008)*x[0]+(0.576376019084)*x[1]**o+(-0.173470710629)*x[1]
        arg[2,3,0,0]=(0.351742111948)*x[0]**o+(-0.641801256932)*x[0]+(-0.979232109277)*x[1]**o+(0.973197715414)*x[1]
        arg[2,3,0,1]=(-0.194493570182)*x[0]**o+(-0.24088269914)*x[0]+(-0.594521666623)*x[1]**o+(0.442357460052)*x[1]
        arg[2,3,1,0]=(-0.0677209625916)*x[0]**o+(-0.329246651504)*x[0]+(-0.200109031557)*x[1]**o+(0.0666064469254)*x[1]
        arg[2,3,1,1]=(-0.31026797404)*x[0]**o+(-0.0420778789656)*x[0]+(-0.534104800904)*x[1]**o+(0.940011883171)*x[1]
        arg[2,3,2,0]=(0.572825859789)*x[0]**o+(-0.439819027899)*x[0]+(-0.641214796724)*x[1]**o+(0.281317897435)*x[1]
        arg[2,3,2,1]=(0.410920070755)*x[0]**o+(-0.517259918184)*x[0]+(0.436420469387)*x[1]**o+(0.212276585342)*x[1]
        arg[2,4,0,0]=(0.0537540090315)*x[0]**o+(-0.362577123782)*x[0]+(-0.0134992480023)*x[1]**o+(0.59055317928)*x[1]
        arg[2,4,0,1]=(0.894896469792)*x[0]**o+(-0.594219811149)*x[0]+(0.755490579022)*x[1]**o+(-0.222913251691)*x[1]
        arg[2,4,1,0]=(-0.95471806813)*x[0]**o+(0.364260591437)*x[0]+(-0.51232745259)*x[1]**o+(-0.874795288001)*x[1]
        arg[2,4,1,1]=(-0.146483465086)*x[0]**o+(0.182495641134)*x[0]+(-0.132211746968)*x[1]**o+(0.39915899917)*x[1]
        arg[2,4,2,0]=(0.630856642463)*x[0]**o+(0.568438094437)*x[0]+(0.283670966088)*x[1]**o+(0.200206672418)*x[1]
        arg[2,4,2,1]=(-0.181938802717)*x[0]**o+(0.850684926475)*x[0]+(-0.765889361148)*x[1]**o+(0.750762858672)*x[1]
        arg[3,0,0,0]=(0.618955933151)*x[0]**o+(0.0280502885803)*x[0]+(-0.518282623282)*x[1]**o+(-0.741257199588)*x[1]
        arg[3,0,0,1]=(0.122462861306)*x[0]**o+(-0.103190806023)*x[0]+(0.647959918767)*x[1]**o+(0.531727405032)*x[1]
        arg[3,0,1,0]=(-0.129468664704)*x[0]**o+(0.342288003693)*x[0]+(-0.0268246756133)*x[1]**o+(0.671147920273)*x[1]
        arg[3,0,1,1]=(-0.99153395274)*x[0]**o+(0.584874556271)*x[0]+(-0.29273067599)*x[1]**o+(-0.802100630404)*x[1]
        arg[3,0,2,0]=(0.406033817569)*x[0]**o+(-0.664112618189)*x[0]+(0.970253482906)*x[1]**o+(0.979709129918)*x[1]
        arg[3,0,2,1]=(-0.708798148641)*x[0]**o+(-0.574259437301)*x[0]+(0.305353886124)*x[1]**o+(0.90520799029)*x[1]
        arg[3,1,0,0]=(-0.296777292529)*x[0]**o+(0.788817860427)*x[0]+(-0.817934426475)*x[1]**o+(-0.480405944642)*x[1]
        arg[3,1,0,1]=(0.382866653892)*x[0]**o+(-0.0911340124698)*x[0]+(0.449555911305)*x[1]**o+(0.972590525904)*x[1]
        arg[3,1,1,0]=(0.150349675921)*x[0]**o+(0.316401916978)*x[0]+(0.176755455021)*x[1]**o+(-0.895165552576)*x[1]
        arg[3,1,1,1]=(-0.334194603044)*x[0]**o+(0.40353334527)*x[0]+(0.510619331296)*x[1]**o+(-0.423857458769)*x[1]
        arg[3,1,2,0]=(0.876266380991)*x[0]**o+(-0.1146693887)*x[0]+(0.0388933985336)*x[1]**o+(0.537276050219)*x[1]
        arg[3,1,2,1]=(0.953489127278)*x[0]**o+(-0.40088917811)*x[0]+(0.322112015322)*x[1]**o+(0.570451651396)*x[1]
        arg[3,2,0,0]=(-0.392674815471)*x[0]**o+(0.805725364633)*x[0]+(-0.550131544344)*x[1]**o+(-0.902042986324)*x[1]
        arg[3,2,0,1]=(0.996009652754)*x[0]**o+(0.534022968708)*x[0]+(-0.251810035763)*x[1]**o+(-0.324231235105)*x[1]
        arg[3,2,1,0]=(-0.280691594637)*x[0]**o+(0.716441336919)*x[0]+(0.444717298591)*x[1]**o+(-0.417682576675)*x[1]
        arg[3,2,1,1]=(-0.528033696021)*x[0]**o+(0.105118392268)*x[0]+(0.219345332231)*x[1]**o+(-0.554011705858)*x[1]
        arg[3,2,2,0]=(0.869967952526)*x[0]**o+(-0.285204973916)*x[0]+(0.676958471189)*x[1]**o+(-0.109813007662)*x[1]
        arg[3,2,2,1]=(-0.183338615818)*x[0]**o+(0.124214512553)*x[0]+(-0.00262420395052)*x[1]**o+(0.505099322191)*x[1]
        arg[3,3,0,0]=(-0.77019083751)*x[0]**o+(-0.288304317474)*x[0]+(-0.674724525333)*x[1]**o+(0.6230658909)*x[1]
        arg[3,3,0,1]=(0.4027332007)*x[0]**o+(0.0181383553658)*x[0]+(0.0346834190431)*x[1]**o+(-0.343302420746)*x[1]
        arg[3,3,1,0]=(0.0907452191096)*x[0]**o+(-0.734197819823)*x[0]+(-0.262536956938)*x[1]**o+(-0.838964523281)*x[1]
        arg[3,3,1,1]=(0.362001847336)*x[0]**o+(-0.99196319473)*x[0]+(-0.544565811529)*x[1]**o+(0.0460139495686)*x[1]
        arg[3,3,2,0]=(0.43659913886)*x[0]**o+(-0.42986865456)*x[0]+(-0.298749218949)*x[1]**o+(0.927334871957)*x[1]
        arg[3,3,2,1]=(0.400272592051)*x[0]**o+(0.332756541456)*x[0]+(0.971127734155)*x[1]**o+(0.653965425034)*x[1]
        arg[3,4,0,0]=(0.663781122153)*x[0]**o+(0.697898766724)*x[0]+(0.157432061303)*x[1]**o+(-0.0526003145497)*x[1]
        arg[3,4,0,1]=(-0.463791084222)*x[0]**o+(0.384914785855)*x[0]+(-0.996485418809)*x[1]**o+(-0.519106290889)*x[1]
        arg[3,4,1,0]=(-0.598068247392)*x[0]**o+(0.553482523952)*x[0]+(0.202485340343)*x[1]**o+(-0.540515409196)*x[1]
        arg[3,4,1,1]=(0.134764979841)*x[0]**o+(0.0141136152488)*x[0]+(-0.536476467366)*x[1]**o+(0.323150057577)*x[1]
        arg[3,4,2,0]=(-0.73096888708)*x[0]**o+(0.485529166207)*x[0]+(-0.541219829073)*x[1]**o+(0.471443044424)*x[1]
        arg[3,4,2,1]=(-0.815808733094)*x[0]**o+(0.694490460113)*x[0]+(0.292301816313)*x[1]**o+(0.700014734893)*x[1]
        ref[0,0,0,0]=(-0.937777610663)*x_ref[0]**o+(-0.0228520946632)*x_ref[0]+(0.642385396583)*x_ref[1]**o+(-0.803821945812)*x_ref[1]
        ref[0,0,0,1]=(-0.398785307492)*x_ref[0]**o+(0.0450288952883)*x_ref[0]+(-0.777780548133)*x_ref[1]**o+(0.0273772099414)*x_ref[1]
        ref[0,0,1,0]=(-0.667280086925)*x_ref[0]**o+(-0.934642710274)*x_ref[0]+(0.565327494491)*x_ref[1]**o+(0.988521769056)*x_ref[1]
        ref[0,0,1,1]=(-0.225562383498)*x_ref[0]**o+(-0.280566031134)*x_ref[0]+(-0.295768954831)*x_ref[1]**o+(0.102759013802)*x_ref[1]
        ref[0,0,2,0]=(-0.179577502546)*x_ref[0]**o+(-0.156932777074)*x_ref[0]+(-0.145616153633)*x_ref[1]**o+(0.971876899135)*x_ref[1]
        ref[0,0,2,1]=(-0.840266409118)*x_ref[0]**o+(0.704356785022)*x_ref[0]+(0.740091470711)*x_ref[1]**o+(0.708478499023)*x_ref[1]
        ref[0,1,0,0]=(-0.487161662806)*x_ref[0]**o+(0.565313908399)*x_ref[0]+(-0.33708697293)*x_ref[1]**o+(0.731675966546)*x_ref[1]
        ref[0,1,0,1]=(-0.135615857963)*x_ref[0]**o+(0.265408777025)*x_ref[0]+(-0.168207260227)*x_ref[1]**o+(-0.598974659317)*x_ref[1]
        ref[0,1,1,0]=(0.0533944010258)*x_ref[0]**o+(0.819248906738)*x_ref[0]+(-0.837005251524)*x_ref[1]**o+(0.119613618595)*x_ref[1]
        ref[0,1,1,1]=(-0.257520627047)*x_ref[0]**o+(0.339264545713)*x_ref[0]+(0.137721228188)*x_ref[1]**o+(-0.512251030979)*x_ref[1]
        ref[0,1,2,0]=(-0.797070396749)*x_ref[0]**o+(0.690173285077)*x_ref[0]+(-0.213206187271)*x_ref[1]**o+(-0.873999125945)*x_ref[1]
        ref[0,1,2,1]=(-0.847662300367)*x_ref[0]**o+(0.785781114266)*x_ref[0]+(-0.456918398811)*x_ref[1]**o+(0.458881079947)*x_ref[1]
        ref[0,2,0,0]=(-0.385269163334)*x_ref[0]**o+(0.135941228964)*x_ref[0]+(0.233245620648)*x_ref[1]**o+(-0.246282209626)*x_ref[1]
        ref[0,2,0,1]=(0.372914636218)*x_ref[0]**o+(-0.160283657003)*x_ref[0]+(0.0565043889781)*x_ref[1]**o+(0.199715522021)*x_ref[1]
        ref[0,2,1,0]=(0.676198390598)*x_ref[0]**o+(0.938025122014)*x_ref[0]+(0.161074696651)*x_ref[1]**o+(-0.155495646964)*x_ref[1]
        ref[0,2,1,1]=(-0.709286924155)*x_ref[0]**o+(-0.173716284776)*x_ref[0]+(0.0576435484694)*x_ref[1]**o+(0.673924697832)*x_ref[1]
        ref[0,2,2,0]=(-0.436468652969)*x_ref[0]**o+(0.117279009082)*x_ref[0]+(-0.731436369191)*x_ref[1]**o+(0.919578503469)*x_ref[1]
        ref[0,2,2,1]=(-0.691042646151)*x_ref[0]**o+(-0.606293725574)*x_ref[0]+(0.657291692759)*x_ref[1]**o+(-0.0146398994729)*x_ref[1]
        ref[0,3,0,0]=(0.365939313719)*x_ref[0]**o+(-0.221236913792)*x_ref[0]+(-0.729594967883)*x_ref[1]**o+(0.0179024719114)*x_ref[1]
        ref[0,3,0,1]=(0.135892133258)*x_ref[0]**o+(0.728069461936)*x_ref[0]+(-0.365691795279)*x_ref[1]**o+(0.877708599922)*x_ref[1]
        ref[0,3,1,0]=(-0.853312456817)*x_ref[0]**o+(-0.713031969049)*x_ref[0]+(0.274755827464)*x_ref[1]**o+(-0.527330699033)*x_ref[1]
        ref[0,3,1,1]=(-0.30358584687)*x_ref[0]**o+(0.871629391968)*x_ref[0]+(0.906915162645)*x_ref[1]**o+(0.499955429041)*x_ref[1]
        ref[0,3,2,0]=(-0.880532587427)*x_ref[0]**o+(0.265729672258)*x_ref[0]+(0.528677136349)*x_ref[1]**o+(-0.224602964925)*x_ref[1]
        ref[0,3,2,1]=(0.158632463532)*x_ref[0]**o+(-0.665418329643)*x_ref[0]+(0.957661069067)*x_ref[1]**o+(-0.23771309434)*x_ref[1]
        ref[0,4,0,0]=(-0.128073514905)*x_ref[0]**o+(-0.452543798055)*x_ref[0]+(0.303808626277)*x_ref[1]**o+(0.841041779759)*x_ref[1]
        ref[0,4,0,1]=(0.0219309964781)*x_ref[0]**o+(-0.949488393443)*x_ref[0]+(0.0436905404663)*x_ref[1]**o+(-0.304527242795)*x_ref[1]
        ref[0,4,1,0]=(-0.792275137775)*x_ref[0]**o+(0.450079829644)*x_ref[0]+(0.162879981637)*x_ref[1]**o+(-0.185839015805)*x_ref[1]
        ref[0,4,1,1]=(0.761932713893)*x_ref[0]**o+(0.932601232945)*x_ref[0]+(-0.606328305783)*x_ref[1]**o+(-0.123973545682)*x_ref[1]
        ref[0,4,2,0]=(-0.514088304908)*x_ref[0]**o+(0.987668774807)*x_ref[0]+(0.866078887168)*x_ref[1]**o+(-0.608518224484)*x_ref[1]
        ref[0,4,2,1]=(-0.427564608028)*x_ref[0]**o+(0.231599614445)*x_ref[0]+(-0.406862229819)*x_ref[1]**o+(0.358456864265)*x_ref[1]
        ref[1,0,0,0]=(0.342092171801)*x_ref[0]**o+(-0.264522675185)*x_ref[0]+(-0.37976917867)*x_ref[1]**o+(0.778596602561)*x_ref[1]
        ref[1,0,0,1]=(0.672156473235)*x_ref[0]**o+(0.320317500856)*x_ref[0]+(0.514206881296)*x_ref[1]**o+(0.602594631705)*x_ref[1]
        ref[1,0,1,0]=(0.914472920293)*x_ref[0]**o+(0.525647943447)*x_ref[0]+(0.685613621597)*x_ref[1]**o+(0.482308039231)*x_ref[1]
        ref[1,0,1,1]=(-0.727982874425)*x_ref[0]**o+(-0.417964463065)*x_ref[0]+(0.825853988063)*x_ref[1]**o+(0.236134760252)*x_ref[1]
        ref[1,0,2,0]=(0.922186381756)*x_ref[0]**o+(-0.140522762044)*x_ref[0]+(-0.282169431666)*x_ref[1]**o+(0.778383988397)*x_ref[1]
        ref[1,0,2,1]=(0.307296736289)*x_ref[0]**o+(0.589844868569)*x_ref[0]+(0.64571457752)*x_ref[1]**o+(0.749333079485)*x_ref[1]
        ref[1,1,0,0]=(0.908345176703)*x_ref[0]**o+(-0.476053147164)*x_ref[0]+(0.0828501189289)*x_ref[1]**o+(0.280657117705)*x_ref[1]
        ref[1,1,0,1]=(0.396128814097)*x_ref[0]**o+(0.368821575482)*x_ref[0]+(-0.916584602334)*x_ref[1]**o+(-0.149377992159)*x_ref[1]
        ref[1,1,1,0]=(-0.38468474313)*x_ref[0]**o+(0.304742499301)*x_ref[0]+(0.638083865846)*x_ref[1]**o+(-0.332046460663)*x_ref[1]
        ref[1,1,1,1]=(0.845097787138)*x_ref[0]**o+(-0.34949498164)*x_ref[0]+(0.229157202572)*x_ref[1]**o+(0.677584672856)*x_ref[1]
        ref[1,1,2,0]=(0.0280692825358)*x_ref[0]**o+(0.567451370107)*x_ref[0]+(0.66934188305)*x_ref[1]**o+(-0.477197213188)*x_ref[1]
        ref[1,1,2,1]=(0.375411487264)*x_ref[0]**o+(0.55086548839)*x_ref[0]+(-0.392135351366)*x_ref[1]**o+(0.91971334589)*x_ref[1]
        ref[1,2,0,0]=(-0.7659270368)*x_ref[0]**o+(-0.338082792509)*x_ref[0]+(-0.26272975232)*x_ref[1]**o+(0.267936454568)*x_ref[1]
        ref[1,2,0,1]=(0.918125455771)*x_ref[0]**o+(0.182896655906)*x_ref[0]+(0.525396835337)*x_ref[1]**o+(0.201918789801)*x_ref[1]
        ref[1,2,1,0]=(-0.54472490266)*x_ref[0]**o+(0.693714832775)*x_ref[0]+(0.898278265794)*x_ref[1]**o+(0.345095187159)*x_ref[1]
        ref[1,2,1,1]=(-0.498607752195)*x_ref[0]**o+(0.84678344253)*x_ref[0]+(0.678992455984)*x_ref[1]**o+(-0.597396262326)*x_ref[1]
        ref[1,2,2,0]=(0.215267998554)*x_ref[0]**o+(0.384479388838)*x_ref[0]+(-0.09078862662)*x_ref[1]**o+(-0.0162185695524)*x_ref[1]
        ref[1,2,2,1]=(-0.335574750173)*x_ref[0]**o+(-0.439933690512)*x_ref[0]+(0.194601817488)*x_ref[1]**o+(0.795181445633)*x_ref[1]
        ref[1,3,0,0]=(0.862180273604)*x_ref[0]**o+(0.276662657248)*x_ref[0]+(-0.908827123328)*x_ref[1]**o+(0.290406540669)*x_ref[1]
        ref[1,3,0,1]=(-0.620441636176)*x_ref[0]**o+(0.448236189204)*x_ref[0]+(-0.503817375869)*x_ref[1]**o+(0.689447469467)*x_ref[1]
        ref[1,3,1,0]=(0.126327572417)*x_ref[0]**o+(-0.476552913848)*x_ref[0]+(0.362273273226)*x_ref[1]**o+(-0.816228830372)*x_ref[1]
        ref[1,3,1,1]=(-0.65091977023)*x_ref[0]**o+(0.0316560718061)*x_ref[0]+(-0.922906273452)*x_ref[1]**o+(-0.584811809954)*x_ref[1]
        ref[1,3,2,0]=(-0.675982262546)*x_ref[0]**o+(0.23518662745)*x_ref[0]+(-0.270229269162)*x_ref[1]**o+(-0.00300656227686)*x_ref[1]
        ref[1,3,2,1]=(0.55691633063)*x_ref[0]**o+(-0.0643123929496)*x_ref[0]+(0.212825067376)*x_ref[1]**o+(-0.0807098593265)*x_ref[1]
        ref[1,4,0,0]=(0.199100268435)*x_ref[0]**o+(-0.358028564112)*x_ref[0]+(0.130301315926)*x_ref[1]**o+(-0.255752658178)*x_ref[1]
        ref[1,4,0,1]=(0.821720568968)*x_ref[0]**o+(0.193634063182)*x_ref[0]+(0.862353589302)*x_ref[1]**o+(-0.42971617129)*x_ref[1]
        ref[1,4,1,0]=(0.51160977767)*x_ref[0]**o+(0.326743318954)*x_ref[0]+(-0.866878934278)*x_ref[1]**o+(-0.927585776234)*x_ref[1]
        ref[1,4,1,1]=(0.299835915861)*x_ref[0]**o+(-0.969027084981)*x_ref[0]+(-0.589772230832)*x_ref[1]**o+(0.382433302029)*x_ref[1]
        ref[1,4,2,0]=(0.473101160384)*x_ref[0]**o+(-0.591124830283)*x_ref[0]+(0.0929141413982)*x_ref[1]**o+(-0.529811769054)*x_ref[1]
        ref[1,4,2,1]=(-0.712363897962)*x_ref[0]**o+(-0.259229026597)*x_ref[0]+(0.00983651162685)*x_ref[1]**o+(-0.329312308231)*x_ref[1]
        ref[2,0,0,0]=(-0.507414918232)*x_ref[0]**o+(0.668167664289)*x_ref[0]+(0.654870826953)*x_ref[1]**o+(-0.411963068213)*x_ref[1]
        ref[2,0,0,1]=(-0.894915678458)*x_ref[0]**o+(0.647579368695)*x_ref[0]+(0.222274441569)*x_ref[1]**o+(0.737972296743)*x_ref[1]
        ref[2,0,1,0]=(-0.457176106717)*x_ref[0]**o+(0.99997698232)*x_ref[0]+(0.524381951875)*x_ref[1]**o+(0.796302331418)*x_ref[1]
        ref[2,0,1,1]=(-0.761912711581)*x_ref[0]**o+(-0.995235985398)*x_ref[0]+(-0.371926591709)*x_ref[1]**o+(-0.37949019966)*x_ref[1]
        ref[2,0,2,0]=(0.402826798397)*x_ref[0]**o+(0.473547871218)*x_ref[0]+(0.801636706721)*x_ref[1]**o+(0.314221308918)*x_ref[1]
        ref[2,0,2,1]=(0.0416846902183)*x_ref[0]**o+(0.815306853966)*x_ref[0]+(-0.526315166764)*x_ref[1]**o+(-0.327935702076)*x_ref[1]
        ref[2,1,0,0]=(-0.53829655239)*x_ref[0]**o+(-0.467914397053)*x_ref[0]+(0.392659650792)*x_ref[1]**o+(0.309531884089)*x_ref[1]
        ref[2,1,0,1]=(-0.768324371377)*x_ref[0]**o+(0.136336455054)*x_ref[0]+(0.0653485752909)*x_ref[1]**o+(-0.0181923467911)*x_ref[1]
        ref[2,1,1,0]=(0.121561771322)*x_ref[0]**o+(-0.794575616334)*x_ref[0]+(0.038194779569)*x_ref[1]**o+(-0.695864614551)*x_ref[1]
        ref[2,1,1,1]=(0.941872575674)*x_ref[0]**o+(0.322067879395)*x_ref[0]+(0.176873583908)*x_ref[1]**o+(0.266964433323)*x_ref[1]
        ref[2,1,2,0]=(0.324335817101)*x_ref[0]**o+(0.609048475305)*x_ref[0]+(0.350973789003)*x_ref[1]**o+(-0.284995830372)*x_ref[1]
        ref[2,1,2,1]=(0.939941132515)*x_ref[0]**o+(0.782468124836)*x_ref[0]+(-0.238539378134)*x_ref[1]**o+(-0.515203802574)*x_ref[1]
        ref[2,2,0,0]=(0.080179317168)*x_ref[0]**o+(-0.555769741953)*x_ref[0]+(0.283304726007)*x_ref[1]**o+(0.844935300942)*x_ref[1]
        ref[2,2,0,1]=(0.52001819752)*x_ref[0]**o+(-0.105912273152)*x_ref[0]+(-0.360523424917)*x_ref[1]**o+(-0.179135541646)*x_ref[1]
        ref[2,2,1,0]=(0.719593478448)*x_ref[0]**o+(-0.722945454687)*x_ref[0]+(0.0316404625439)*x_ref[1]**o+(0.154290328891)*x_ref[1]
        ref[2,2,1,1]=(0.906382302919)*x_ref[0]**o+(-0.137753840588)*x_ref[0]+(-0.626417865101)*x_ref[1]**o+(-0.890642745149)*x_ref[1]
        ref[2,2,2,0]=(0.618763086671)*x_ref[0]**o+(-0.267876013516)*x_ref[0]+(0.582353922702)*x_ref[1]**o+(-0.190278788501)*x_ref[1]
        ref[2,2,2,1]=(0.778824546516)*x_ref[0]**o+(-0.00837218512008)*x_ref[0]+(0.576376019084)*x_ref[1]**o+(-0.173470710629)*x_ref[1]
        ref[2,3,0,0]=(0.351742111948)*x_ref[0]**o+(-0.641801256932)*x_ref[0]+(-0.979232109277)*x_ref[1]**o+(0.973197715414)*x_ref[1]
        ref[2,3,0,1]=(-0.194493570182)*x_ref[0]**o+(-0.24088269914)*x_ref[0]+(-0.594521666623)*x_ref[1]**o+(0.442357460052)*x_ref[1]
        ref[2,3,1,0]=(-0.0677209625916)*x_ref[0]**o+(-0.329246651504)*x_ref[0]+(-0.200109031557)*x_ref[1]**o+(0.0666064469254)*x_ref[1]
        ref[2,3,1,1]=(-0.31026797404)*x_ref[0]**o+(-0.0420778789656)*x_ref[0]+(-0.534104800904)*x_ref[1]**o+(0.940011883171)*x_ref[1]
        ref[2,3,2,0]=(0.572825859789)*x_ref[0]**o+(-0.439819027899)*x_ref[0]+(-0.641214796724)*x_ref[1]**o+(0.281317897435)*x_ref[1]
        ref[2,3,2,1]=(0.410920070755)*x_ref[0]**o+(-0.517259918184)*x_ref[0]+(0.436420469387)*x_ref[1]**o+(0.212276585342)*x_ref[1]
        ref[2,4,0,0]=(0.0537540090315)*x_ref[0]**o+(-0.362577123782)*x_ref[0]+(-0.0134992480023)*x_ref[1]**o+(0.59055317928)*x_ref[1]
        ref[2,4,0,1]=(0.894896469792)*x_ref[0]**o+(-0.594219811149)*x_ref[0]+(0.755490579022)*x_ref[1]**o+(-0.222913251691)*x_ref[1]
        ref[2,4,1,0]=(-0.95471806813)*x_ref[0]**o+(0.364260591437)*x_ref[0]+(-0.51232745259)*x_ref[1]**o+(-0.874795288001)*x_ref[1]
        ref[2,4,1,1]=(-0.146483465086)*x_ref[0]**o+(0.182495641134)*x_ref[0]+(-0.132211746968)*x_ref[1]**o+(0.39915899917)*x_ref[1]
        ref[2,4,2,0]=(0.630856642463)*x_ref[0]**o+(0.568438094437)*x_ref[0]+(0.283670966088)*x_ref[1]**o+(0.200206672418)*x_ref[1]
        ref[2,4,2,1]=(-0.181938802717)*x_ref[0]**o+(0.850684926475)*x_ref[0]+(-0.765889361148)*x_ref[1]**o+(0.750762858672)*x_ref[1]
        ref[3,0,0,0]=(0.618955933151)*x_ref[0]**o+(0.0280502885803)*x_ref[0]+(-0.518282623282)*x_ref[1]**o+(-0.741257199588)*x_ref[1]
        ref[3,0,0,1]=(0.122462861306)*x_ref[0]**o+(-0.103190806023)*x_ref[0]+(0.647959918767)*x_ref[1]**o+(0.531727405032)*x_ref[1]
        ref[3,0,1,0]=(-0.129468664704)*x_ref[0]**o+(0.342288003693)*x_ref[0]+(-0.0268246756133)*x_ref[1]**o+(0.671147920273)*x_ref[1]
        ref[3,0,1,1]=(-0.99153395274)*x_ref[0]**o+(0.584874556271)*x_ref[0]+(-0.29273067599)*x_ref[1]**o+(-0.802100630404)*x_ref[1]
        ref[3,0,2,0]=(0.406033817569)*x_ref[0]**o+(-0.664112618189)*x_ref[0]+(0.970253482906)*x_ref[1]**o+(0.979709129918)*x_ref[1]
        ref[3,0,2,1]=(-0.708798148641)*x_ref[0]**o+(-0.574259437301)*x_ref[0]+(0.305353886124)*x_ref[1]**o+(0.90520799029)*x_ref[1]
        ref[3,1,0,0]=(-0.296777292529)*x_ref[0]**o+(0.788817860427)*x_ref[0]+(-0.817934426475)*x_ref[1]**o+(-0.480405944642)*x_ref[1]
        ref[3,1,0,1]=(0.382866653892)*x_ref[0]**o+(-0.0911340124698)*x_ref[0]+(0.449555911305)*x_ref[1]**o+(0.972590525904)*x_ref[1]
        ref[3,1,1,0]=(0.150349675921)*x_ref[0]**o+(0.316401916978)*x_ref[0]+(0.176755455021)*x_ref[1]**o+(-0.895165552576)*x_ref[1]
        ref[3,1,1,1]=(-0.334194603044)*x_ref[0]**o+(0.40353334527)*x_ref[0]+(0.510619331296)*x_ref[1]**o+(-0.423857458769)*x_ref[1]
        ref[3,1,2,0]=(0.876266380991)*x_ref[0]**o+(-0.1146693887)*x_ref[0]+(0.0388933985336)*x_ref[1]**o+(0.537276050219)*x_ref[1]
        ref[3,1,2,1]=(0.953489127278)*x_ref[0]**o+(-0.40088917811)*x_ref[0]+(0.322112015322)*x_ref[1]**o+(0.570451651396)*x_ref[1]
        ref[3,2,0,0]=(-0.392674815471)*x_ref[0]**o+(0.805725364633)*x_ref[0]+(-0.550131544344)*x_ref[1]**o+(-0.902042986324)*x_ref[1]
        ref[3,2,0,1]=(0.996009652754)*x_ref[0]**o+(0.534022968708)*x_ref[0]+(-0.251810035763)*x_ref[1]**o+(-0.324231235105)*x_ref[1]
        ref[3,2,1,0]=(-0.280691594637)*x_ref[0]**o+(0.716441336919)*x_ref[0]+(0.444717298591)*x_ref[1]**o+(-0.417682576675)*x_ref[1]
        ref[3,2,1,1]=(-0.528033696021)*x_ref[0]**o+(0.105118392268)*x_ref[0]+(0.219345332231)*x_ref[1]**o+(-0.554011705858)*x_ref[1]
        ref[3,2,2,0]=(0.869967952526)*x_ref[0]**o+(-0.285204973916)*x_ref[0]+(0.676958471189)*x_ref[1]**o+(-0.109813007662)*x_ref[1]
        ref[3,2,2,1]=(-0.183338615818)*x_ref[0]**o+(0.124214512553)*x_ref[0]+(-0.00262420395052)*x_ref[1]**o+(0.505099322191)*x_ref[1]
        ref[3,3,0,0]=(-0.77019083751)*x_ref[0]**o+(-0.288304317474)*x_ref[0]+(-0.674724525333)*x_ref[1]**o+(0.6230658909)*x_ref[1]
        ref[3,3,0,1]=(0.4027332007)*x_ref[0]**o+(0.0181383553658)*x_ref[0]+(0.0346834190431)*x_ref[1]**o+(-0.343302420746)*x_ref[1]
        ref[3,3,1,0]=(0.0907452191096)*x_ref[0]**o+(-0.734197819823)*x_ref[0]+(-0.262536956938)*x_ref[1]**o+(-0.838964523281)*x_ref[1]
        ref[3,3,1,1]=(0.362001847336)*x_ref[0]**o+(-0.99196319473)*x_ref[0]+(-0.544565811529)*x_ref[1]**o+(0.0460139495686)*x_ref[1]
        ref[3,3,2,0]=(0.43659913886)*x_ref[0]**o+(-0.42986865456)*x_ref[0]+(-0.298749218949)*x_ref[1]**o+(0.927334871957)*x_ref[1]
        ref[3,3,2,1]=(0.400272592051)*x_ref[0]**o+(0.332756541456)*x_ref[0]+(0.971127734155)*x_ref[1]**o+(0.653965425034)*x_ref[1]
        ref[3,4,0,0]=(0.663781122153)*x_ref[0]**o+(0.697898766724)*x_ref[0]+(0.157432061303)*x_ref[1]**o+(-0.0526003145497)*x_ref[1]
        ref[3,4,0,1]=(-0.463791084222)*x_ref[0]**o+(0.384914785855)*x_ref[0]+(-0.996485418809)*x_ref[1]**o+(-0.519106290889)*x_ref[1]
        ref[3,4,1,0]=(-0.598068247392)*x_ref[0]**o+(0.553482523952)*x_ref[0]+(0.202485340343)*x_ref[1]**o+(-0.540515409196)*x_ref[1]
        ref[3,4,1,1]=(0.134764979841)*x_ref[0]**o+(0.0141136152488)*x_ref[0]+(-0.536476467366)*x_ref[1]**o+(0.323150057577)*x_ref[1]
        ref[3,4,2,0]=(-0.73096888708)*x_ref[0]**o+(0.485529166207)*x_ref[0]+(-0.541219829073)*x_ref[1]**o+(0.471443044424)*x_ref[1]
        ref[3,4,2,1]=(-0.815808733094)*x_ref[0]**o+(0.694490460113)*x_ref[0]+(0.292301816313)*x_ref[1]**o+(0.700014734893)*x_ref[1]
      else:
        arg[0,0,0,0]=(0.244520699379)*x[0]**o+(-0.125771252842)*x[0]+(0.654557798928)*x[1]**o+(0.438110948503)*x[1]+(0.399996169353)*x[2]**o+(0.37037141324)*x[2]
        arg[0,0,0,1]=(0.019693348021)*x[0]**o+(-0.509938412509)*x[0]+(-0.425852907243)*x[1]**o+(0.351462945072)*x[1]+(0.628410872426)*x[2]**o+(0.4769252017)*x[2]
        arg[0,0,1,0]=(0.623813767602)*x[0]**o+(-0.940818977456)*x[0]+(-0.00111063869285)*x[1]**o+(-0.577558081394)*x[1]+(-0.30810971068)*x[2]**o+(-0.660149007373)*x[2]
        arg[0,0,1,1]=(0.994105796371)*x[0]**o+(0.199970906966)*x[0]+(0.961509001434)*x[1]**o+(-0.783069602127)*x[1]+(-0.179922863021)*x[2]**o+(-0.386974503089)*x[2]
        arg[0,0,2,0]=(0.932745137122)*x[0]**o+(0.627995830114)*x[0]+(-0.334344490736)*x[1]**o+(0.733210590815)*x[1]+(-0.23424959051)*x[2]**o+(0.140644214958)*x[2]
        arg[0,0,2,1]=(-0.282836519247)*x[0]**o+(-0.744077669768)*x[0]+(0.123898574243)*x[1]**o+(0.562384289004)*x[1]+(0.0757927394637)*x[2]**o+(0.767684067921)*x[2]
        arg[0,1,0,0]=(0.212366202901)*x[0]**o+(-0.679228304269)*x[0]+(0.829706652419)*x[1]**o+(0.24389996073)*x[1]+(0.909367025978)*x[2]**o+(0.280617790407)*x[2]
        arg[0,1,0,1]=(0.760070952632)*x[0]**o+(-0.678309252372)*x[0]+(0.0372281910363)*x[1]**o+(0.0284582814297)*x[1]+(-0.092349250941)*x[2]**o+(0.347713645921)*x[2]
        arg[0,1,1,0]=(0.248410230191)*x[0]**o+(-0.833983202295)*x[0]+(-0.759357278899)*x[1]**o+(0.880038285893)*x[1]+(-0.734799540652)*x[2]**o+(0.493330231104)*x[2]
        arg[0,1,1,1]=(0.0359919724228)*x[0]**o+(-0.0393899014057)*x[0]+(0.185155782488)*x[1]**o+(-0.92692925719)*x[1]+(0.74718037404)*x[2]**o+(-0.283900681763)*x[2]
        arg[0,1,2,0]=(-0.208643544667)*x[0]**o+(0.368200149096)*x[0]+(-0.327032725172)*x[1]**o+(-0.245547905792)*x[1]+(0.0886488214242)*x[2]**o+(0.179139897691)*x[2]
        arg[0,1,2,1]=(-0.0618762861459)*x[0]**o+(0.253419217897)*x[0]+(0.840270125508)*x[1]**o+(0.0223556878387)*x[1]+(0.0560904700176)*x[2]**o+(-0.646231049424)*x[2]
        arg[0,2,0,0]=(0.568966969865)*x[0]**o+(0.786776511348)*x[0]+(0.150595926116)*x[1]**o+(-0.780164417762)*x[1]+(0.510149831242)*x[2]**o+(-0.902871929903)*x[2]
        arg[0,2,0,1]=(-0.817062669957)*x[0]**o+(0.626805486868)*x[0]+(-0.566129786829)*x[1]**o+(0.248418548905)*x[1]+(-0.273995326019)*x[2]**o+(0.402023521018)*x[2]
        arg[0,2,1,0]=(-0.160276235113)*x[0]**o+(-0.0732543937053)*x[0]+(-0.0651595982698)*x[1]**o+(-0.529463476237)*x[1]+(0.346098812945)*x[2]**o+(0.418367016439)*x[2]
        arg[0,2,1,1]=(0.736907408498)*x[0]**o+(-0.123612748157)*x[0]+(0.000834659108651)*x[1]**o+(0.568833761363)*x[1]+(-0.542676953582)*x[2]**o+(0.026182553484)*x[2]
        arg[0,2,2,0]=(0.320985918151)*x[0]**o+(0.0512654311644)*x[0]+(-0.0313412913713)*x[1]**o+(0.295659930913)*x[1]+(-0.989560405329)*x[2]**o+(-0.107344246298)*x[2]
        arg[0,2,2,1]=(-0.957793939949)*x[0]**o+(-0.177591880789)*x[0]+(0.0785829772689)*x[1]**o+(0.121445956612)*x[1]+(-0.384134176508)*x[2]**o+(-0.775433914622)*x[2]
        arg[0,3,0,0]=(-0.130620609021)*x[0]**o+(0.268283814564)*x[0]+(-0.192835222037)*x[1]**o+(0.235341688541)*x[1]+(0.957069315916)*x[2]**o+(-0.708853567529)*x[2]
        arg[0,3,0,1]=(-0.377028600614)*x[0]**o+(-0.858446162753)*x[0]+(-0.850630580491)*x[1]**o+(-0.810676898206)*x[1]+(-0.613330062817)*x[2]**o+(0.714335738947)*x[2]
        arg[0,3,1,0]=(-0.602336649415)*x[0]**o+(-0.166667559558)*x[0]+(0.962948866081)*x[1]**o+(0.939531483467)*x[1]+(0.267767294244)*x[2]**o+(0.247155872428)*x[2]
        arg[0,3,1,1]=(0.838505391049)*x[0]**o+(0.115204438919)*x[0]+(0.989010581013)*x[1]**o+(-0.649789818418)*x[1]+(0.757328444925)*x[2]**o+(0.825158129449)*x[2]
        arg[0,3,2,0]=(0.724488034506)*x[0]**o+(0.0333093325925)*x[0]+(0.864643652652)*x[1]**o+(-0.557517585925)*x[1]+(-0.340534672063)*x[2]**o+(-0.00425705176271)*x[2]
        arg[0,3,2,1]=(-0.366459483792)*x[0]**o+(0.257706980646)*x[0]+(0.821731531023)*x[1]**o+(-0.212501303617)*x[1]+(-0.128977067795)*x[2]**o+(-0.931439847041)*x[2]
        arg[0,4,0,0]=(0.398430239547)*x[0]**o+(-0.00991917699933)*x[0]+(-0.182471763172)*x[1]**o+(0.218327764946)*x[1]+(-0.484510127696)*x[2]**o+(-0.360678193996)*x[2]
        arg[0,4,0,1]=(0.0640680415135)*x[0]**o+(0.671150063087)*x[0]+(0.208610130381)*x[1]**o+(-0.441252393083)*x[1]+(0.365150146582)*x[2]**o+(-0.228346558142)*x[2]
        arg[0,4,1,0]=(-0.852694824303)*x[0]**o+(0.181041494463)*x[0]+(0.898413617086)*x[1]**o+(-0.918988862595)*x[1]+(0.169010143324)*x[2]**o+(0.540458809834)*x[2]
        arg[0,4,1,1]=(-0.213782550366)*x[0]**o+(-0.180557891041)*x[0]+(0.23144059382)*x[1]**o+(0.0609482143429)*x[1]+(0.89256221946)*x[2]**o+(0.977857158982)*x[2]
        arg[0,4,2,0]=(0.320772898841)*x[0]**o+(0.371610189518)*x[0]+(0.692264150983)*x[1]**o+(-0.751154935415)*x[1]+(-0.163272902772)*x[2]**o+(0.0635386628764)*x[2]
        arg[0,4,2,1]=(-0.316885908577)*x[0]**o+(0.476104431241)*x[0]+(-0.647713072489)*x[1]**o+(0.683813449908)*x[1]+(0.699293342945)*x[2]**o+(-0.0373906770097)*x[2]
        arg[1,0,0,0]=(-0.030329265733)*x[0]**o+(-0.0115761157375)*x[0]+(-0.30993823088)*x[1]**o+(0.628877975389)*x[1]+(-0.582940961417)*x[2]**o+(0.154786570457)*x[2]
        arg[1,0,0,1]=(-0.593610753495)*x[0]**o+(-0.822232097764)*x[0]+(-0.465144000698)*x[1]**o+(-0.62576868204)*x[1]+(-0.53588531407)*x[2]**o+(0.119119578421)*x[2]
        arg[1,0,1,0]=(-0.15668734059)*x[0]**o+(-0.950658811047)*x[0]+(-0.267969935748)*x[1]**o+(0.0919415340304)*x[1]+(-0.0445382452662)*x[2]**o+(0.949984936459)*x[2]
        arg[1,0,1,1]=(-0.762474809436)*x[0]**o+(0.794314442281)*x[0]+(0.617742930372)*x[1]**o+(-0.502057530669)*x[1]+(0.42532154189)*x[2]**o+(0.343417687179)*x[2]
        arg[1,0,2,0]=(0.259583418084)*x[0]**o+(0.460122326434)*x[0]+(0.875185242587)*x[1]**o+(-0.621232091699)*x[1]+(0.922536397008)*x[2]**o+(-0.494729334323)*x[2]
        arg[1,0,2,1]=(0.79625863156)*x[0]**o+(-0.980360881506)*x[0]+(0.947721778185)*x[1]**o+(-0.234201025597)*x[1]+(-0.974251961452)*x[2]**o+(-0.300484605111)*x[2]
        arg[1,1,0,0]=(0.719927358834)*x[0]**o+(-0.296944527139)*x[0]+(0.970290716632)*x[1]**o+(0.45656092862)*x[1]+(0.657023751979)*x[2]**o+(-0.703898365316)*x[2]
        arg[1,1,0,1]=(-0.0600633107738)*x[0]**o+(0.525293730667)*x[0]+(0.17955637601)*x[1]**o+(-0.625782540859)*x[1]+(0.205255859488)*x[2]**o+(-0.963128832741)*x[2]
        arg[1,1,1,0]=(0.119721921725)*x[0]**o+(0.957918016103)*x[0]+(-0.574883179285)*x[1]**o+(0.915580622743)*x[1]+(-0.432538225757)*x[2]**o+(-0.937539574986)*x[2]
        arg[1,1,1,1]=(-0.295639299942)*x[0]**o+(-0.703954000058)*x[0]+(-0.185001166055)*x[1]**o+(-0.943933820897)*x[1]+(-0.12692007135)*x[2]**o+(0.476481616585)*x[2]
        arg[1,1,2,0]=(0.247892010246)*x[0]**o+(0.901198196531)*x[0]+(-0.586162224974)*x[1]**o+(-0.934177552379)*x[1]+(0.704265144051)*x[2]**o+(-0.688632953989)*x[2]
        arg[1,1,2,1]=(-0.116039842334)*x[0]**o+(-0.300373025101)*x[0]+(-0.286684372551)*x[1]**o+(-0.452337760059)*x[1]+(0.883210026596)*x[2]**o+(-0.319739612634)*x[2]
        arg[1,2,0,0]=(-0.0326555626249)*x[0]**o+(-0.0933559852573)*x[0]+(0.807443822382)*x[1]**o+(-0.148842350195)*x[1]+(-0.118904471983)*x[2]**o+(-0.740440145747)*x[2]
        arg[1,2,0,1]=(0.313623768488)*x[0]**o+(0.524958399998)*x[0]+(0.876747001712)*x[1]**o+(-0.468458894694)*x[1]+(0.52118935613)*x[2]**o+(-0.776405429712)*x[2]
        arg[1,2,1,0]=(0.1328367779)*x[0]**o+(-0.959628068869)*x[0]+(0.746000325891)*x[1]**o+(-0.454705589649)*x[1]+(0.08534697763)*x[2]**o+(0.174892567303)*x[2]
        arg[1,2,1,1]=(-0.482594997843)*x[0]**o+(0.765628170342)*x[0]+(0.103333249513)*x[1]**o+(0.840778499017)*x[1]+(0.136748193255)*x[2]**o+(0.771613240623)*x[2]
        arg[1,2,2,0]=(-0.305745376076)*x[0]**o+(-0.0464234361529)*x[0]+(-0.163528204043)*x[1]**o+(0.80782719974)*x[1]+(0.131046315567)*x[2]**o+(0.0849217124905)*x[2]
        arg[1,2,2,1]=(0.564915635291)*x[0]**o+(-0.162467524139)*x[0]+(-0.696721556806)*x[1]**o+(0.0172839552576)*x[1]+(-0.630040402586)*x[2]**o+(-0.181232689117)*x[2]
        arg[1,3,0,0]=(-0.768631958549)*x[0]**o+(0.537076196504)*x[0]+(-0.403489871013)*x[1]**o+(0.831966318632)*x[1]+(0.602387061638)*x[2]**o+(0.330495470463)*x[2]
        arg[1,3,0,1]=(0.770878177942)*x[0]**o+(-0.678609489017)*x[0]+(-0.895200519099)*x[1]**o+(0.935093561205)*x[1]+(0.0497551733469)*x[2]**o+(0.820432808383)*x[2]
        arg[1,3,1,0]=(0.052867263447)*x[0]**o+(0.566167926663)*x[0]+(-0.00397870350455)*x[1]**o+(-0.780374460006)*x[1]+(-0.166645199933)*x[2]**o+(-0.867274057327)*x[2]
        arg[1,3,1,1]=(0.0573446476183)*x[0]**o+(-0.699343104682)*x[0]+(0.428624411932)*x[1]**o+(-0.400834963806)*x[1]+(-0.722750021776)*x[2]**o+(0.386821944027)*x[2]
        arg[1,3,2,0]=(0.390777333115)*x[0]**o+(0.191564882434)*x[0]+(-0.774547811812)*x[1]**o+(0.747678394063)*x[1]+(0.662273739757)*x[2]**o+(-0.295629568405)*x[2]
        arg[1,3,2,1]=(-0.160077661734)*x[0]**o+(-0.512286879466)*x[0]+(0.616824388099)*x[1]**o+(0.0257995114324)*x[1]+(0.644270949659)*x[2]**o+(0.243520087896)*x[2]
        arg[1,4,0,0]=(-0.980501571755)*x[0]**o+(0.844184850883)*x[0]+(-0.608509998911)*x[1]**o+(-0.347687895783)*x[1]+(-0.536534228926)*x[2]**o+(0.632821444072)*x[2]
        arg[1,4,0,1]=(-0.881676821515)*x[0]**o+(-0.858728104026)*x[0]+(-0.576516592622)*x[1]**o+(0.695748958418)*x[1]+(0.856547670877)*x[2]**o+(-0.932960091743)*x[2]
        arg[1,4,1,0]=(-0.451435812217)*x[0]**o+(0.963886330308)*x[0]+(0.888532771273)*x[1]**o+(0.807924978471)*x[1]+(0.789117121354)*x[2]**o+(0.577647188972)*x[2]
        arg[1,4,1,1]=(-0.315320308371)*x[0]**o+(0.182798184972)*x[0]+(0.936963228212)*x[1]**o+(-0.634429372225)*x[1]+(0.634343556662)*x[2]**o+(0.975851394002)*x[2]
        arg[1,4,2,0]=(0.858246326961)*x[0]**o+(0.0955812000864)*x[0]+(0.16143064785)*x[1]**o+(0.397013086552)*x[1]+(-0.507188929963)*x[2]**o+(0.49789827039)*x[2]
        arg[1,4,2,1]=(-0.524634529078)*x[0]**o+(0.675491034226)*x[0]+(0.858857975014)*x[1]**o+(-0.57874207047)*x[1]+(0.476551353232)*x[2]**o+(-0.622601386038)*x[2]
        arg[2,0,0,0]=(0.976563609)*x[0]**o+(0.127133211238)*x[0]+(-0.410155025204)*x[1]**o+(-0.909032700787)*x[1]+(-0.197014506331)*x[2]**o+(0.557171133933)*x[2]
        arg[2,0,0,1]=(0.434874625301)*x[0]**o+(-0.576955025374)*x[0]+(-0.466686136319)*x[1]**o+(-0.359616702831)*x[1]+(-0.916050768111)*x[2]**o+(0.900510057507)*x[2]
        arg[2,0,1,0]=(0.615259852986)*x[0]**o+(-0.685399646912)*x[0]+(-0.222742007887)*x[1]**o+(-0.487739486778)*x[1]+(-0.675225266903)*x[2]**o+(0.494623157154)*x[2]
        arg[2,0,1,1]=(-0.89129010918)*x[0]**o+(0.0051571255592)*x[0]+(0.681003954208)*x[1]**o+(0.634041720719)*x[1]+(-0.308587458722)*x[2]**o+(-0.107801539193)*x[2]
        arg[2,0,2,0]=(-0.660496698398)*x[0]**o+(-0.889315862375)*x[0]+(0.64248433679)*x[1]**o+(-0.941990511222)*x[1]+(-0.173668816201)*x[2]**o+(0.237812802184)*x[2]
        arg[2,0,2,1]=(0.847365004226)*x[0]**o+(0.802449170085)*x[0]+(0.504737931993)*x[1]**o+(-0.695546680342)*x[1]+(0.0742668699455)*x[2]**o+(0.396565506083)*x[2]
        arg[2,1,0,0]=(0.812115742561)*x[0]**o+(-0.528091349418)*x[0]+(-0.442834134691)*x[1]**o+(-0.574414379092)*x[1]+(-0.734077338221)*x[2]**o+(0.159306515061)*x[2]
        arg[2,1,0,1]=(-0.259422255041)*x[0]**o+(-0.104016445541)*x[0]+(0.3560854635)*x[1]**o+(0.997015276785)*x[1]+(-0.707278312293)*x[2]**o+(-0.975235879836)*x[2]
        arg[2,1,1,0]=(0.402099188227)*x[0]**o+(0.0320354114274)*x[0]+(0.203384927114)*x[1]**o+(-0.847804170327)*x[1]+(-0.898459903546)*x[2]**o+(0.639388498254)*x[2]
        arg[2,1,1,1]=(-0.0477209380768)*x[0]**o+(0.531574667575)*x[0]+(0.280889682833)*x[1]**o+(-0.444963671116)*x[1]+(0.33935324652)*x[2]**o+(0.943089036339)*x[2]
        arg[2,1,2,0]=(0.396038211375)*x[0]**o+(-0.54994373568)*x[0]+(0.851017234687)*x[1]**o+(-0.739787661885)*x[1]+(-0.368325897478)*x[2]**o+(-0.449096507837)*x[2]
        arg[2,1,2,1]=(0.857714720302)*x[0]**o+(-0.750668366832)*x[0]+(-0.587901076381)*x[1]**o+(-0.24906126763)*x[1]+(0.131707530536)*x[2]**o+(0.900975507373)*x[2]
        arg[2,2,0,0]=(0.783919932392)*x[0]**o+(-0.967669966647)*x[0]+(0.82052580972)*x[1]**o+(-0.711919131449)*x[1]+(-0.396059626351)*x[2]**o+(-0.265805391355)*x[2]
        arg[2,2,0,1]=(-0.0398160197307)*x[0]**o+(-0.238193541)*x[0]+(0.282277679599)*x[1]**o+(-0.0526835304963)*x[1]+(0.590871489195)*x[2]**o+(-0.925835585742)*x[2]
        arg[2,2,1,0]=(-0.825757767738)*x[0]**o+(0.238488063696)*x[0]+(0.109616380527)*x[1]**o+(-0.942941715764)*x[1]+(0.632013079754)*x[2]**o+(0.0517997920543)*x[2]
        arg[2,2,1,1]=(-0.85343230326)*x[0]**o+(0.225247381624)*x[0]+(0.43171428489)*x[1]**o+(0.0937487532231)*x[1]+(0.894064129869)*x[2]**o+(-0.0876265498234)*x[2]
        arg[2,2,2,0]=(0.349294373111)*x[0]**o+(-0.734159680014)*x[0]+(0.0489227393033)*x[1]**o+(-0.245284468757)*x[1]+(-0.346469825649)*x[2]**o+(0.547054093324)*x[2]
        arg[2,2,2,1]=(-0.0478095998047)*x[0]**o+(0.440261344405)*x[0]+(-0.590330499679)*x[1]**o+(-0.961009986328)*x[1]+(0.546190002905)*x[2]**o+(-0.027965996519)*x[2]
        arg[2,3,0,0]=(0.148563550644)*x[0]**o+(0.576704219545)*x[0]+(-0.782168581109)*x[1]**o+(-0.8301245678)*x[1]+(0.937936245682)*x[2]**o+(0.846740948371)*x[2]
        arg[2,3,0,1]=(-0.447832058524)*x[0]**o+(0.385431388609)*x[0]+(0.193985676964)*x[1]**o+(-0.91717180275)*x[1]+(-0.408732230908)*x[2]**o+(0.548173152118)*x[2]
        arg[2,3,1,0]=(0.197592567864)*x[0]**o+(-0.77657127384)*x[0]+(0.266355135001)*x[1]**o+(0.604009981584)*x[1]+(-0.379930450459)*x[2]**o+(0.46952844479)*x[2]
        arg[2,3,1,1]=(0.996803794962)*x[0]**o+(0.319115676981)*x[0]+(0.255552997875)*x[1]**o+(0.21834578367)*x[1]+(0.389966036243)*x[2]**o+(-0.115308080726)*x[2]
        arg[2,3,2,0]=(-0.0244303880675)*x[0]**o+(-0.128929049017)*x[0]+(-0.350933414406)*x[1]**o+(-0.341716440852)*x[1]+(-0.724579076377)*x[2]**o+(0.21963465519)*x[2]
        arg[2,3,2,1]=(-0.92899971478)*x[0]**o+(-0.304496074717)*x[0]+(0.568151427024)*x[1]**o+(0.504629517922)*x[1]+(0.846158711477)*x[2]**o+(0.35936792568)*x[2]
        arg[2,4,0,0]=(-0.350964834352)*x[0]**o+(0.387667833884)*x[0]+(0.547101274223)*x[1]**o+(0.257218757373)*x[1]+(-0.00316748527841)*x[2]**o+(0.314561238615)*x[2]
        arg[2,4,0,1]=(0.408941739151)*x[0]**o+(0.819113792537)*x[0]+(0.532891848781)*x[1]**o+(-0.30688659725)*x[1]+(-0.736611937618)*x[2]**o+(-0.875783706051)*x[2]
        arg[2,4,1,0]=(-0.730885165985)*x[0]**o+(0.453908506302)*x[0]+(-0.617385752505)*x[1]**o+(0.357459789592)*x[1]+(0.286323175805)*x[2]**o+(-0.448879127031)*x[2]
        arg[2,4,1,1]=(-0.295377341281)*x[0]**o+(0.423424371051)*x[0]+(0.524695076927)*x[1]**o+(-0.698499625038)*x[1]+(0.617308008395)*x[2]**o+(-0.423469866581)*x[2]
        arg[2,4,2,0]=(0.104251379635)*x[0]**o+(0.989709098795)*x[0]+(-0.422075396624)*x[1]**o+(-0.254328561231)*x[1]+(-0.502052795074)*x[2]**o+(-0.0851411652168)*x[2]
        arg[2,4,2,1]=(0.379332842082)*x[0]**o+(-0.707504861783)*x[0]+(0.00855075647146)*x[1]**o+(0.792851159033)*x[1]+(-0.692042160791)*x[2]**o+(-0.18327165243)*x[2]
        arg[3,0,0,0]=(0.310700158718)*x[0]**o+(0.285647240187)*x[0]+(0.817947571838)*x[1]**o+(0.808698040916)*x[1]+(0.844184006443)*x[2]**o+(-0.310383215636)*x[2]
        arg[3,0,0,1]=(-0.0173109952616)*x[0]**o+(0.672034452037)*x[0]+(-0.464572256298)*x[1]**o+(0.205799523299)*x[1]+(-0.971979841338)*x[2]**o+(0.261515966001)*x[2]
        arg[3,0,1,0]=(0.0329384377028)*x[0]**o+(0.571501185072)*x[0]+(-0.902222556642)*x[1]**o+(-0.528317784373)*x[1]+(0.331989640881)*x[2]**o+(0.0427505624318)*x[2]
        arg[3,0,1,1]=(0.964350410761)*x[0]**o+(-0.327198989999)*x[0]+(-0.195271523191)*x[1]**o+(0.534663014601)*x[1]+(-0.0208261208706)*x[2]**o+(-0.158019249052)*x[2]
        arg[3,0,2,0]=(-0.622082726008)*x[0]**o+(0.559463883442)*x[0]+(0.841439693313)*x[1]**o+(-0.196363987516)*x[1]+(-0.0143926547396)*x[2]**o+(0.144732603689)*x[2]
        arg[3,0,2,1]=(-0.149531653278)*x[0]**o+(0.354229138322)*x[0]+(-0.507916088829)*x[1]**o+(0.221251580047)*x[1]+(0.197213706274)*x[2]**o+(-0.555645149276)*x[2]
        arg[3,1,0,0]=(0.331020785374)*x[0]**o+(-0.377647773513)*x[0]+(0.563355928949)*x[1]**o+(0.789309216711)*x[1]+(0.203663383875)*x[2]**o+(-0.0333894540645)*x[2]
        arg[3,1,0,1]=(-0.0965613913882)*x[0]**o+(0.18656484323)*x[0]+(-0.138245941339)*x[1]**o+(-0.34549538745)*x[1]+(0.827173403167)*x[2]**o+(-0.873380503939)*x[2]
        arg[3,1,1,0]=(0.932940890033)*x[0]**o+(-0.564883830288)*x[0]+(-0.505052725295)*x[1]**o+(-0.669420210206)*x[1]+(-0.799638096039)*x[2]**o+(0.469400233749)*x[2]
        arg[3,1,1,1]=(-0.9741761069)*x[0]**o+(-0.748125328781)*x[0]+(0.664448241931)*x[1]**o+(0.0957643296979)*x[1]+(0.112211322019)*x[2]**o+(-0.131870290434)*x[2]
        arg[3,1,2,0]=(-0.438609127704)*x[0]**o+(-0.36618286515)*x[0]+(-0.376420011115)*x[1]**o+(-0.67084624599)*x[1]+(0.821679796644)*x[2]**o+(0.80111319897)*x[2]
        arg[3,1,2,1]=(-0.409859432085)*x[0]**o+(0.848767816929)*x[0]+(0.382358459835)*x[1]**o+(-0.750139843631)*x[1]+(0.587152586559)*x[2]**o+(-0.26162952001)*x[2]
        arg[3,2,0,0]=(-0.506329785074)*x[0]**o+(-0.0967745752004)*x[0]+(-0.163316795268)*x[1]**o+(0.0628481921732)*x[1]+(0.395355494367)*x[2]**o+(0.0978901916332)*x[2]
        arg[3,2,0,1]=(0.803590309587)*x[0]**o+(0.470388053389)*x[0]+(0.674942922168)*x[1]**o+(0.223472834762)*x[1]+(0.552332209028)*x[2]**o+(-0.396443498843)*x[2]
        arg[3,2,1,0]=(-0.312384723839)*x[0]**o+(0.392213828608)*x[0]+(0.449465316671)*x[1]**o+(-0.249103089401)*x[1]+(0.601412913458)*x[2]**o+(0.209899368039)*x[2]
        arg[3,2,1,1]=(-0.583776179557)*x[0]**o+(0.485007047754)*x[0]+(-0.0354153633589)*x[1]**o+(-0.898630969916)*x[1]+(-0.266581492795)*x[2]**o+(0.822086680281)*x[2]
        arg[3,2,2,0]=(0.944617094397)*x[0]**o+(-0.173057994833)*x[0]+(0.927832160048)*x[1]**o+(-0.218599594252)*x[1]+(0.349957755011)*x[2]**o+(0.392602744545)*x[2]
        arg[3,2,2,1]=(0.461648084674)*x[0]**o+(0.826213376949)*x[0]+(0.143429054731)*x[1]**o+(-0.214265128007)*x[1]+(0.0317208127607)*x[2]**o+(-0.858361078381)*x[2]
        arg[3,3,0,0]=(0.306607920368)*x[0]**o+(0.147692891688)*x[0]+(-0.562661766456)*x[1]**o+(0.448595564727)*x[1]+(-0.707178218373)*x[2]**o+(0.0655554587866)*x[2]
        arg[3,3,0,1]=(0.135475820362)*x[0]**o+(0.51420718793)*x[0]+(0.343291220736)*x[1]**o+(-0.797532312602)*x[1]+(-0.273431011273)*x[2]**o+(0.135307779253)*x[2]
        arg[3,3,1,0]=(0.488028231073)*x[0]**o+(0.542021806829)*x[0]+(0.91916268609)*x[1]**o+(0.317368273365)*x[1]+(0.314807039395)*x[2]**o+(-0.841583308025)*x[2]
        arg[3,3,1,1]=(-0.182783814917)*x[0]**o+(0.250863481201)*x[0]+(0.755551457003)*x[1]**o+(0.946171548529)*x[1]+(0.0350225436682)*x[2]**o+(0.776059830052)*x[2]
        arg[3,3,2,0]=(-0.116707124994)*x[0]**o+(-0.727010280588)*x[0]+(0.172065389268)*x[1]**o+(0.754164564639)*x[1]+(0.803777635621)*x[2]**o+(0.293490264417)*x[2]
        arg[3,3,2,1]=(0.154235982797)*x[0]**o+(-0.124286021486)*x[0]+(-0.737324748116)*x[1]**o+(-0.136803541581)*x[1]+(-0.608370191604)*x[2]**o+(-0.933599034128)*x[2]
        arg[3,4,0,0]=(0.818061677328)*x[0]**o+(-0.771719613166)*x[0]+(0.742772795251)*x[1]**o+(0.0359285091312)*x[1]+(-0.985656851837)*x[2]**o+(-0.884598206512)*x[2]
        arg[3,4,0,1]=(0.198371732741)*x[0]**o+(0.646496618681)*x[0]+(0.232912446494)*x[1]**o+(0.945619492023)*x[1]+(-0.344370341731)*x[2]**o+(0.31156840357)*x[2]
        arg[3,4,1,0]=(-0.609298834606)*x[0]**o+(0.784697392412)*x[0]+(0.996065619079)*x[1]**o+(-0.284498300043)*x[1]+(-0.652795706803)*x[2]**o+(-0.254746840396)*x[2]
        arg[3,4,1,1]=(-0.0083632822507)*x[0]**o+(0.200923618933)*x[0]+(0.353883930064)*x[1]**o+(0.153257380179)*x[1]+(0.843965464205)*x[2]**o+(0.0418501400703)*x[2]
        arg[3,4,2,0]=(-0.226055622191)*x[0]**o+(-0.701789166696)*x[0]+(0.274080957143)*x[1]**o+(0.0596744660556)*x[1]+(-0.342083152987)*x[2]**o+(0.593518464697)*x[2]
        arg[3,4,2,1]=(-0.288168945023)*x[0]**o+(-0.742841678774)*x[0]+(0.972629532424)*x[1]**o+(0.892100560733)*x[1]+(0.803880474721)*x[2]**o+(-0.0638336211224)*x[2]
        ref[0,0,0,0]=(0.244520699379)*x_ref[0]**o+(-0.125771252842)*x_ref[0]+(0.654557798928)*x_ref[1]**o+(0.438110948503)*x_ref[1]+(0.399996169353)*x_ref[2]**o+(0.37037141324)*x_ref[2]
        ref[0,0,0,1]=(0.019693348021)*x_ref[0]**o+(-0.509938412509)*x_ref[0]+(-0.425852907243)*x_ref[1]**o+(0.351462945072)*x_ref[1]+(0.628410872426)*x_ref[2]**o+(0.4769252017)*x_ref[2]
        ref[0,0,1,0]=(0.623813767602)*x_ref[0]**o+(-0.940818977456)*x_ref[0]+(-0.00111063869285)*x_ref[1]**o+(-0.577558081394)*x_ref[1]+(-0.30810971068)*x_ref[2]**o+(-0.660149007373)*x_ref[2]
        ref[0,0,1,1]=(0.994105796371)*x_ref[0]**o+(0.199970906966)*x_ref[0]+(0.961509001434)*x_ref[1]**o+(-0.783069602127)*x_ref[1]+(-0.179922863021)*x_ref[2]**o+(-0.386974503089)*x_ref[2]
        ref[0,0,2,0]=(0.932745137122)*x_ref[0]**o+(0.627995830114)*x_ref[0]+(-0.334344490736)*x_ref[1]**o+(0.733210590815)*x_ref[1]+(-0.23424959051)*x_ref[2]**o+(0.140644214958)*x_ref[2]
        ref[0,0,2,1]=(-0.282836519247)*x_ref[0]**o+(-0.744077669768)*x_ref[0]+(0.123898574243)*x_ref[1]**o+(0.562384289004)*x_ref[1]+(0.0757927394637)*x_ref[2]**o+(0.767684067921)*x_ref[2]
        ref[0,1,0,0]=(0.212366202901)*x_ref[0]**o+(-0.679228304269)*x_ref[0]+(0.829706652419)*x_ref[1]**o+(0.24389996073)*x_ref[1]+(0.909367025978)*x_ref[2]**o+(0.280617790407)*x_ref[2]
        ref[0,1,0,1]=(0.760070952632)*x_ref[0]**o+(-0.678309252372)*x_ref[0]+(0.0372281910363)*x_ref[1]**o+(0.0284582814297)*x_ref[1]+(-0.092349250941)*x_ref[2]**o+(0.347713645921)*x_ref[2]
        ref[0,1,1,0]=(0.248410230191)*x_ref[0]**o+(-0.833983202295)*x_ref[0]+(-0.759357278899)*x_ref[1]**o+(0.880038285893)*x_ref[1]+(-0.734799540652)*x_ref[2]**o+(0.493330231104)*x_ref[2]
        ref[0,1,1,1]=(0.0359919724228)*x_ref[0]**o+(-0.0393899014057)*x_ref[0]+(0.185155782488)*x_ref[1]**o+(-0.92692925719)*x_ref[1]+(0.74718037404)*x_ref[2]**o+(-0.283900681763)*x_ref[2]
        ref[0,1,2,0]=(-0.208643544667)*x_ref[0]**o+(0.368200149096)*x_ref[0]+(-0.327032725172)*x_ref[1]**o+(-0.245547905792)*x_ref[1]+(0.0886488214242)*x_ref[2]**o+(0.179139897691)*x_ref[2]
        ref[0,1,2,1]=(-0.0618762861459)*x_ref[0]**o+(0.253419217897)*x_ref[0]+(0.840270125508)*x_ref[1]**o+(0.0223556878387)*x_ref[1]+(0.0560904700176)*x_ref[2]**o+(-0.646231049424)*x_ref[2]
        ref[0,2,0,0]=(0.568966969865)*x_ref[0]**o+(0.786776511348)*x_ref[0]+(0.150595926116)*x_ref[1]**o+(-0.780164417762)*x_ref[1]+(0.510149831242)*x_ref[2]**o+(-0.902871929903)*x_ref[2]
        ref[0,2,0,1]=(-0.817062669957)*x_ref[0]**o+(0.626805486868)*x_ref[0]+(-0.566129786829)*x_ref[1]**o+(0.248418548905)*x_ref[1]+(-0.273995326019)*x_ref[2]**o+(0.402023521018)*x_ref[2]
        ref[0,2,1,0]=(-0.160276235113)*x_ref[0]**o+(-0.0732543937053)*x_ref[0]+(-0.0651595982698)*x_ref[1]**o+(-0.529463476237)*x_ref[1]+(0.346098812945)*x_ref[2]**o+(0.418367016439)*x_ref[2]
        ref[0,2,1,1]=(0.736907408498)*x_ref[0]**o+(-0.123612748157)*x_ref[0]+(0.000834659108651)*x_ref[1]**o+(0.568833761363)*x_ref[1]+(-0.542676953582)*x_ref[2]**o+(0.026182553484)*x_ref[2]
        ref[0,2,2,0]=(0.320985918151)*x_ref[0]**o+(0.0512654311644)*x_ref[0]+(-0.0313412913713)*x_ref[1]**o+(0.295659930913)*x_ref[1]+(-0.989560405329)*x_ref[2]**o+(-0.107344246298)*x_ref[2]
        ref[0,2,2,1]=(-0.957793939949)*x_ref[0]**o+(-0.177591880789)*x_ref[0]+(0.0785829772689)*x_ref[1]**o+(0.121445956612)*x_ref[1]+(-0.384134176508)*x_ref[2]**o+(-0.775433914622)*x_ref[2]
        ref[0,3,0,0]=(-0.130620609021)*x_ref[0]**o+(0.268283814564)*x_ref[0]+(-0.192835222037)*x_ref[1]**o+(0.235341688541)*x_ref[1]+(0.957069315916)*x_ref[2]**o+(-0.708853567529)*x_ref[2]
        ref[0,3,0,1]=(-0.377028600614)*x_ref[0]**o+(-0.858446162753)*x_ref[0]+(-0.850630580491)*x_ref[1]**o+(-0.810676898206)*x_ref[1]+(-0.613330062817)*x_ref[2]**o+(0.714335738947)*x_ref[2]
        ref[0,3,1,0]=(-0.602336649415)*x_ref[0]**o+(-0.166667559558)*x_ref[0]+(0.962948866081)*x_ref[1]**o+(0.939531483467)*x_ref[1]+(0.267767294244)*x_ref[2]**o+(0.247155872428)*x_ref[2]
        ref[0,3,1,1]=(0.838505391049)*x_ref[0]**o+(0.115204438919)*x_ref[0]+(0.989010581013)*x_ref[1]**o+(-0.649789818418)*x_ref[1]+(0.757328444925)*x_ref[2]**o+(0.825158129449)*x_ref[2]
        ref[0,3,2,0]=(0.724488034506)*x_ref[0]**o+(0.0333093325925)*x_ref[0]+(0.864643652652)*x_ref[1]**o+(-0.557517585925)*x_ref[1]+(-0.340534672063)*x_ref[2]**o+(-0.00425705176271)*x_ref[2]
        ref[0,3,2,1]=(-0.366459483792)*x_ref[0]**o+(0.257706980646)*x_ref[0]+(0.821731531023)*x_ref[1]**o+(-0.212501303617)*x_ref[1]+(-0.128977067795)*x_ref[2]**o+(-0.931439847041)*x_ref[2]
        ref[0,4,0,0]=(0.398430239547)*x_ref[0]**o+(-0.00991917699933)*x_ref[0]+(-0.182471763172)*x_ref[1]**o+(0.218327764946)*x_ref[1]+(-0.484510127696)*x_ref[2]**o+(-0.360678193996)*x_ref[2]
        ref[0,4,0,1]=(0.0640680415135)*x_ref[0]**o+(0.671150063087)*x_ref[0]+(0.208610130381)*x_ref[1]**o+(-0.441252393083)*x_ref[1]+(0.365150146582)*x_ref[2]**o+(-0.228346558142)*x_ref[2]
        ref[0,4,1,0]=(-0.852694824303)*x_ref[0]**o+(0.181041494463)*x_ref[0]+(0.898413617086)*x_ref[1]**o+(-0.918988862595)*x_ref[1]+(0.169010143324)*x_ref[2]**o+(0.540458809834)*x_ref[2]
        ref[0,4,1,1]=(-0.213782550366)*x_ref[0]**o+(-0.180557891041)*x_ref[0]+(0.23144059382)*x_ref[1]**o+(0.0609482143429)*x_ref[1]+(0.89256221946)*x_ref[2]**o+(0.977857158982)*x_ref[2]
        ref[0,4,2,0]=(0.320772898841)*x_ref[0]**o+(0.371610189518)*x_ref[0]+(0.692264150983)*x_ref[1]**o+(-0.751154935415)*x_ref[1]+(-0.163272902772)*x_ref[2]**o+(0.0635386628764)*x_ref[2]
        ref[0,4,2,1]=(-0.316885908577)*x_ref[0]**o+(0.476104431241)*x_ref[0]+(-0.647713072489)*x_ref[1]**o+(0.683813449908)*x_ref[1]+(0.699293342945)*x_ref[2]**o+(-0.0373906770097)*x_ref[2]
        ref[1,0,0,0]=(-0.030329265733)*x_ref[0]**o+(-0.0115761157375)*x_ref[0]+(-0.30993823088)*x_ref[1]**o+(0.628877975389)*x_ref[1]+(-0.582940961417)*x_ref[2]**o+(0.154786570457)*x_ref[2]
        ref[1,0,0,1]=(-0.593610753495)*x_ref[0]**o+(-0.822232097764)*x_ref[0]+(-0.465144000698)*x_ref[1]**o+(-0.62576868204)*x_ref[1]+(-0.53588531407)*x_ref[2]**o+(0.119119578421)*x_ref[2]
        ref[1,0,1,0]=(-0.15668734059)*x_ref[0]**o+(-0.950658811047)*x_ref[0]+(-0.267969935748)*x_ref[1]**o+(0.0919415340304)*x_ref[1]+(-0.0445382452662)*x_ref[2]**o+(0.949984936459)*x_ref[2]
        ref[1,0,1,1]=(-0.762474809436)*x_ref[0]**o+(0.794314442281)*x_ref[0]+(0.617742930372)*x_ref[1]**o+(-0.502057530669)*x_ref[1]+(0.42532154189)*x_ref[2]**o+(0.343417687179)*x_ref[2]
        ref[1,0,2,0]=(0.259583418084)*x_ref[0]**o+(0.460122326434)*x_ref[0]+(0.875185242587)*x_ref[1]**o+(-0.621232091699)*x_ref[1]+(0.922536397008)*x_ref[2]**o+(-0.494729334323)*x_ref[2]
        ref[1,0,2,1]=(0.79625863156)*x_ref[0]**o+(-0.980360881506)*x_ref[0]+(0.947721778185)*x_ref[1]**o+(-0.234201025597)*x_ref[1]+(-0.974251961452)*x_ref[2]**o+(-0.300484605111)*x_ref[2]
        ref[1,1,0,0]=(0.719927358834)*x_ref[0]**o+(-0.296944527139)*x_ref[0]+(0.970290716632)*x_ref[1]**o+(0.45656092862)*x_ref[1]+(0.657023751979)*x_ref[2]**o+(-0.703898365316)*x_ref[2]
        ref[1,1,0,1]=(-0.0600633107738)*x_ref[0]**o+(0.525293730667)*x_ref[0]+(0.17955637601)*x_ref[1]**o+(-0.625782540859)*x_ref[1]+(0.205255859488)*x_ref[2]**o+(-0.963128832741)*x_ref[2]
        ref[1,1,1,0]=(0.119721921725)*x_ref[0]**o+(0.957918016103)*x_ref[0]+(-0.574883179285)*x_ref[1]**o+(0.915580622743)*x_ref[1]+(-0.432538225757)*x_ref[2]**o+(-0.937539574986)*x_ref[2]
        ref[1,1,1,1]=(-0.295639299942)*x_ref[0]**o+(-0.703954000058)*x_ref[0]+(-0.185001166055)*x_ref[1]**o+(-0.943933820897)*x_ref[1]+(-0.12692007135)*x_ref[2]**o+(0.476481616585)*x_ref[2]
        ref[1,1,2,0]=(0.247892010246)*x_ref[0]**o+(0.901198196531)*x_ref[0]+(-0.586162224974)*x_ref[1]**o+(-0.934177552379)*x_ref[1]+(0.704265144051)*x_ref[2]**o+(-0.688632953989)*x_ref[2]
        ref[1,1,2,1]=(-0.116039842334)*x_ref[0]**o+(-0.300373025101)*x_ref[0]+(-0.286684372551)*x_ref[1]**o+(-0.452337760059)*x_ref[1]+(0.883210026596)*x_ref[2]**o+(-0.319739612634)*x_ref[2]
        ref[1,2,0,0]=(-0.0326555626249)*x_ref[0]**o+(-0.0933559852573)*x_ref[0]+(0.807443822382)*x_ref[1]**o+(-0.148842350195)*x_ref[1]+(-0.118904471983)*x_ref[2]**o+(-0.740440145747)*x_ref[2]
        ref[1,2,0,1]=(0.313623768488)*x_ref[0]**o+(0.524958399998)*x_ref[0]+(0.876747001712)*x_ref[1]**o+(-0.468458894694)*x_ref[1]+(0.52118935613)*x_ref[2]**o+(-0.776405429712)*x_ref[2]
        ref[1,2,1,0]=(0.1328367779)*x_ref[0]**o+(-0.959628068869)*x_ref[0]+(0.746000325891)*x_ref[1]**o+(-0.454705589649)*x_ref[1]+(0.08534697763)*x_ref[2]**o+(0.174892567303)*x_ref[2]
        ref[1,2,1,1]=(-0.482594997843)*x_ref[0]**o+(0.765628170342)*x_ref[0]+(0.103333249513)*x_ref[1]**o+(0.840778499017)*x_ref[1]+(0.136748193255)*x_ref[2]**o+(0.771613240623)*x_ref[2]
        ref[1,2,2,0]=(-0.305745376076)*x_ref[0]**o+(-0.0464234361529)*x_ref[0]+(-0.163528204043)*x_ref[1]**o+(0.80782719974)*x_ref[1]+(0.131046315567)*x_ref[2]**o+(0.0849217124905)*x_ref[2]
        ref[1,2,2,1]=(0.564915635291)*x_ref[0]**o+(-0.162467524139)*x_ref[0]+(-0.696721556806)*x_ref[1]**o+(0.0172839552576)*x_ref[1]+(-0.630040402586)*x_ref[2]**o+(-0.181232689117)*x_ref[2]
        ref[1,3,0,0]=(-0.768631958549)*x_ref[0]**o+(0.537076196504)*x_ref[0]+(-0.403489871013)*x_ref[1]**o+(0.831966318632)*x_ref[1]+(0.602387061638)*x_ref[2]**o+(0.330495470463)*x_ref[2]
        ref[1,3,0,1]=(0.770878177942)*x_ref[0]**o+(-0.678609489017)*x_ref[0]+(-0.895200519099)*x_ref[1]**o+(0.935093561205)*x_ref[1]+(0.0497551733469)*x_ref[2]**o+(0.820432808383)*x_ref[2]
        ref[1,3,1,0]=(0.052867263447)*x_ref[0]**o+(0.566167926663)*x_ref[0]+(-0.00397870350455)*x_ref[1]**o+(-0.780374460006)*x_ref[1]+(-0.166645199933)*x_ref[2]**o+(-0.867274057327)*x_ref[2]
        ref[1,3,1,1]=(0.0573446476183)*x_ref[0]**o+(-0.699343104682)*x_ref[0]+(0.428624411932)*x_ref[1]**o+(-0.400834963806)*x_ref[1]+(-0.722750021776)*x_ref[2]**o+(0.386821944027)*x_ref[2]
        ref[1,3,2,0]=(0.390777333115)*x_ref[0]**o+(0.191564882434)*x_ref[0]+(-0.774547811812)*x_ref[1]**o+(0.747678394063)*x_ref[1]+(0.662273739757)*x_ref[2]**o+(-0.295629568405)*x_ref[2]
        ref[1,3,2,1]=(-0.160077661734)*x_ref[0]**o+(-0.512286879466)*x_ref[0]+(0.616824388099)*x_ref[1]**o+(0.0257995114324)*x_ref[1]+(0.644270949659)*x_ref[2]**o+(0.243520087896)*x_ref[2]
        ref[1,4,0,0]=(-0.980501571755)*x_ref[0]**o+(0.844184850883)*x_ref[0]+(-0.608509998911)*x_ref[1]**o+(-0.347687895783)*x_ref[1]+(-0.536534228926)*x_ref[2]**o+(0.632821444072)*x_ref[2]
        ref[1,4,0,1]=(-0.881676821515)*x_ref[0]**o+(-0.858728104026)*x_ref[0]+(-0.576516592622)*x_ref[1]**o+(0.695748958418)*x_ref[1]+(0.856547670877)*x_ref[2]**o+(-0.932960091743)*x_ref[2]
        ref[1,4,1,0]=(-0.451435812217)*x_ref[0]**o+(0.963886330308)*x_ref[0]+(0.888532771273)*x_ref[1]**o+(0.807924978471)*x_ref[1]+(0.789117121354)*x_ref[2]**o+(0.577647188972)*x_ref[2]
        ref[1,4,1,1]=(-0.315320308371)*x_ref[0]**o+(0.182798184972)*x_ref[0]+(0.936963228212)*x_ref[1]**o+(-0.634429372225)*x_ref[1]+(0.634343556662)*x_ref[2]**o+(0.975851394002)*x_ref[2]
        ref[1,4,2,0]=(0.858246326961)*x_ref[0]**o+(0.0955812000864)*x_ref[0]+(0.16143064785)*x_ref[1]**o+(0.397013086552)*x_ref[1]+(-0.507188929963)*x_ref[2]**o+(0.49789827039)*x_ref[2]
        ref[1,4,2,1]=(-0.524634529078)*x_ref[0]**o+(0.675491034226)*x_ref[0]+(0.858857975014)*x_ref[1]**o+(-0.57874207047)*x_ref[1]+(0.476551353232)*x_ref[2]**o+(-0.622601386038)*x_ref[2]
        ref[2,0,0,0]=(0.976563609)*x_ref[0]**o+(0.127133211238)*x_ref[0]+(-0.410155025204)*x_ref[1]**o+(-0.909032700787)*x_ref[1]+(-0.197014506331)*x_ref[2]**o+(0.557171133933)*x_ref[2]
        ref[2,0,0,1]=(0.434874625301)*x_ref[0]**o+(-0.576955025374)*x_ref[0]+(-0.466686136319)*x_ref[1]**o+(-0.359616702831)*x_ref[1]+(-0.916050768111)*x_ref[2]**o+(0.900510057507)*x_ref[2]
        ref[2,0,1,0]=(0.615259852986)*x_ref[0]**o+(-0.685399646912)*x_ref[0]+(-0.222742007887)*x_ref[1]**o+(-0.487739486778)*x_ref[1]+(-0.675225266903)*x_ref[2]**o+(0.494623157154)*x_ref[2]
        ref[2,0,1,1]=(-0.89129010918)*x_ref[0]**o+(0.0051571255592)*x_ref[0]+(0.681003954208)*x_ref[1]**o+(0.634041720719)*x_ref[1]+(-0.308587458722)*x_ref[2]**o+(-0.107801539193)*x_ref[2]
        ref[2,0,2,0]=(-0.660496698398)*x_ref[0]**o+(-0.889315862375)*x_ref[0]+(0.64248433679)*x_ref[1]**o+(-0.941990511222)*x_ref[1]+(-0.173668816201)*x_ref[2]**o+(0.237812802184)*x_ref[2]
        ref[2,0,2,1]=(0.847365004226)*x_ref[0]**o+(0.802449170085)*x_ref[0]+(0.504737931993)*x_ref[1]**o+(-0.695546680342)*x_ref[1]+(0.0742668699455)*x_ref[2]**o+(0.396565506083)*x_ref[2]
        ref[2,1,0,0]=(0.812115742561)*x_ref[0]**o+(-0.528091349418)*x_ref[0]+(-0.442834134691)*x_ref[1]**o+(-0.574414379092)*x_ref[1]+(-0.734077338221)*x_ref[2]**o+(0.159306515061)*x_ref[2]
        ref[2,1,0,1]=(-0.259422255041)*x_ref[0]**o+(-0.104016445541)*x_ref[0]+(0.3560854635)*x_ref[1]**o+(0.997015276785)*x_ref[1]+(-0.707278312293)*x_ref[2]**o+(-0.975235879836)*x_ref[2]
        ref[2,1,1,0]=(0.402099188227)*x_ref[0]**o+(0.0320354114274)*x_ref[0]+(0.203384927114)*x_ref[1]**o+(-0.847804170327)*x_ref[1]+(-0.898459903546)*x_ref[2]**o+(0.639388498254)*x_ref[2]
        ref[2,1,1,1]=(-0.0477209380768)*x_ref[0]**o+(0.531574667575)*x_ref[0]+(0.280889682833)*x_ref[1]**o+(-0.444963671116)*x_ref[1]+(0.33935324652)*x_ref[2]**o+(0.943089036339)*x_ref[2]
        ref[2,1,2,0]=(0.396038211375)*x_ref[0]**o+(-0.54994373568)*x_ref[0]+(0.851017234687)*x_ref[1]**o+(-0.739787661885)*x_ref[1]+(-0.368325897478)*x_ref[2]**o+(-0.449096507837)*x_ref[2]
        ref[2,1,2,1]=(0.857714720302)*x_ref[0]**o+(-0.750668366832)*x_ref[0]+(-0.587901076381)*x_ref[1]**o+(-0.24906126763)*x_ref[1]+(0.131707530536)*x_ref[2]**o+(0.900975507373)*x_ref[2]
        ref[2,2,0,0]=(0.783919932392)*x_ref[0]**o+(-0.967669966647)*x_ref[0]+(0.82052580972)*x_ref[1]**o+(-0.711919131449)*x_ref[1]+(-0.396059626351)*x_ref[2]**o+(-0.265805391355)*x_ref[2]
        ref[2,2,0,1]=(-0.0398160197307)*x_ref[0]**o+(-0.238193541)*x_ref[0]+(0.282277679599)*x_ref[1]**o+(-0.0526835304963)*x_ref[1]+(0.590871489195)*x_ref[2]**o+(-0.925835585742)*x_ref[2]
        ref[2,2,1,0]=(-0.825757767738)*x_ref[0]**o+(0.238488063696)*x_ref[0]+(0.109616380527)*x_ref[1]**o+(-0.942941715764)*x_ref[1]+(0.632013079754)*x_ref[2]**o+(0.0517997920543)*x_ref[2]
        ref[2,2,1,1]=(-0.85343230326)*x_ref[0]**o+(0.225247381624)*x_ref[0]+(0.43171428489)*x_ref[1]**o+(0.0937487532231)*x_ref[1]+(0.894064129869)*x_ref[2]**o+(-0.0876265498234)*x_ref[2]
        ref[2,2,2,0]=(0.349294373111)*x_ref[0]**o+(-0.734159680014)*x_ref[0]+(0.0489227393033)*x_ref[1]**o+(-0.245284468757)*x_ref[1]+(-0.346469825649)*x_ref[2]**o+(0.547054093324)*x_ref[2]
        ref[2,2,2,1]=(-0.0478095998047)*x_ref[0]**o+(0.440261344405)*x_ref[0]+(-0.590330499679)*x_ref[1]**o+(-0.961009986328)*x_ref[1]+(0.546190002905)*x_ref[2]**o+(-0.027965996519)*x_ref[2]
        ref[2,3,0,0]=(0.148563550644)*x_ref[0]**o+(0.576704219545)*x_ref[0]+(-0.782168581109)*x_ref[1]**o+(-0.8301245678)*x_ref[1]+(0.937936245682)*x_ref[2]**o+(0.846740948371)*x_ref[2]
        ref[2,3,0,1]=(-0.447832058524)*x_ref[0]**o+(0.385431388609)*x_ref[0]+(0.193985676964)*x_ref[1]**o+(-0.91717180275)*x_ref[1]+(-0.408732230908)*x_ref[2]**o+(0.548173152118)*x_ref[2]
        ref[2,3,1,0]=(0.197592567864)*x_ref[0]**o+(-0.77657127384)*x_ref[0]+(0.266355135001)*x_ref[1]**o+(0.604009981584)*x_ref[1]+(-0.379930450459)*x_ref[2]**o+(0.46952844479)*x_ref[2]
        ref[2,3,1,1]=(0.996803794962)*x_ref[0]**o+(0.319115676981)*x_ref[0]+(0.255552997875)*x_ref[1]**o+(0.21834578367)*x_ref[1]+(0.389966036243)*x_ref[2]**o+(-0.115308080726)*x_ref[2]
        ref[2,3,2,0]=(-0.0244303880675)*x_ref[0]**o+(-0.128929049017)*x_ref[0]+(-0.350933414406)*x_ref[1]**o+(-0.341716440852)*x_ref[1]+(-0.724579076377)*x_ref[2]**o+(0.21963465519)*x_ref[2]
        ref[2,3,2,1]=(-0.92899971478)*x_ref[0]**o+(-0.304496074717)*x_ref[0]+(0.568151427024)*x_ref[1]**o+(0.504629517922)*x_ref[1]+(0.846158711477)*x_ref[2]**o+(0.35936792568)*x_ref[2]
        ref[2,4,0,0]=(-0.350964834352)*x_ref[0]**o+(0.387667833884)*x_ref[0]+(0.547101274223)*x_ref[1]**o+(0.257218757373)*x_ref[1]+(-0.00316748527841)*x_ref[2]**o+(0.314561238615)*x_ref[2]
        ref[2,4,0,1]=(0.408941739151)*x_ref[0]**o+(0.819113792537)*x_ref[0]+(0.532891848781)*x_ref[1]**o+(-0.30688659725)*x_ref[1]+(-0.736611937618)*x_ref[2]**o+(-0.875783706051)*x_ref[2]
        ref[2,4,1,0]=(-0.730885165985)*x_ref[0]**o+(0.453908506302)*x_ref[0]+(-0.617385752505)*x_ref[1]**o+(0.357459789592)*x_ref[1]+(0.286323175805)*x_ref[2]**o+(-0.448879127031)*x_ref[2]
        ref[2,4,1,1]=(-0.295377341281)*x_ref[0]**o+(0.423424371051)*x_ref[0]+(0.524695076927)*x_ref[1]**o+(-0.698499625038)*x_ref[1]+(0.617308008395)*x_ref[2]**o+(-0.423469866581)*x_ref[2]
        ref[2,4,2,0]=(0.104251379635)*x_ref[0]**o+(0.989709098795)*x_ref[0]+(-0.422075396624)*x_ref[1]**o+(-0.254328561231)*x_ref[1]+(-0.502052795074)*x_ref[2]**o+(-0.0851411652168)*x_ref[2]
        ref[2,4,2,1]=(0.379332842082)*x_ref[0]**o+(-0.707504861783)*x_ref[0]+(0.00855075647146)*x_ref[1]**o+(0.792851159033)*x_ref[1]+(-0.692042160791)*x_ref[2]**o+(-0.18327165243)*x_ref[2]
        ref[3,0,0,0]=(0.310700158718)*x_ref[0]**o+(0.285647240187)*x_ref[0]+(0.817947571838)*x_ref[1]**o+(0.808698040916)*x_ref[1]+(0.844184006443)*x_ref[2]**o+(-0.310383215636)*x_ref[2]
        ref[3,0,0,1]=(-0.0173109952616)*x_ref[0]**o+(0.672034452037)*x_ref[0]+(-0.464572256298)*x_ref[1]**o+(0.205799523299)*x_ref[1]+(-0.971979841338)*x_ref[2]**o+(0.261515966001)*x_ref[2]
        ref[3,0,1,0]=(0.0329384377028)*x_ref[0]**o+(0.571501185072)*x_ref[0]+(-0.902222556642)*x_ref[1]**o+(-0.528317784373)*x_ref[1]+(0.331989640881)*x_ref[2]**o+(0.0427505624318)*x_ref[2]
        ref[3,0,1,1]=(0.964350410761)*x_ref[0]**o+(-0.327198989999)*x_ref[0]+(-0.195271523191)*x_ref[1]**o+(0.534663014601)*x_ref[1]+(-0.0208261208706)*x_ref[2]**o+(-0.158019249052)*x_ref[2]
        ref[3,0,2,0]=(-0.622082726008)*x_ref[0]**o+(0.559463883442)*x_ref[0]+(0.841439693313)*x_ref[1]**o+(-0.196363987516)*x_ref[1]+(-0.0143926547396)*x_ref[2]**o+(0.144732603689)*x_ref[2]
        ref[3,0,2,1]=(-0.149531653278)*x_ref[0]**o+(0.354229138322)*x_ref[0]+(-0.507916088829)*x_ref[1]**o+(0.221251580047)*x_ref[1]+(0.197213706274)*x_ref[2]**o+(-0.555645149276)*x_ref[2]
        ref[3,1,0,0]=(0.331020785374)*x_ref[0]**o+(-0.377647773513)*x_ref[0]+(0.563355928949)*x_ref[1]**o+(0.789309216711)*x_ref[1]+(0.203663383875)*x_ref[2]**o+(-0.0333894540645)*x_ref[2]
        ref[3,1,0,1]=(-0.0965613913882)*x_ref[0]**o+(0.18656484323)*x_ref[0]+(-0.138245941339)*x_ref[1]**o+(-0.34549538745)*x_ref[1]+(0.827173403167)*x_ref[2]**o+(-0.873380503939)*x_ref[2]
        ref[3,1,1,0]=(0.932940890033)*x_ref[0]**o+(-0.564883830288)*x_ref[0]+(-0.505052725295)*x_ref[1]**o+(-0.669420210206)*x_ref[1]+(-0.799638096039)*x_ref[2]**o+(0.469400233749)*x_ref[2]
        ref[3,1,1,1]=(-0.9741761069)*x_ref[0]**o+(-0.748125328781)*x_ref[0]+(0.664448241931)*x_ref[1]**o+(0.0957643296979)*x_ref[1]+(0.112211322019)*x_ref[2]**o+(-0.131870290434)*x_ref[2]
        ref[3,1,2,0]=(-0.438609127704)*x_ref[0]**o+(-0.36618286515)*x_ref[0]+(-0.376420011115)*x_ref[1]**o+(-0.67084624599)*x_ref[1]+(0.821679796644)*x_ref[2]**o+(0.80111319897)*x_ref[2]
        ref[3,1,2,1]=(-0.409859432085)*x_ref[0]**o+(0.848767816929)*x_ref[0]+(0.382358459835)*x_ref[1]**o+(-0.750139843631)*x_ref[1]+(0.587152586559)*x_ref[2]**o+(-0.26162952001)*x_ref[2]
        ref[3,2,0,0]=(-0.506329785074)*x_ref[0]**o+(-0.0967745752004)*x_ref[0]+(-0.163316795268)*x_ref[1]**o+(0.0628481921732)*x_ref[1]+(0.395355494367)*x_ref[2]**o+(0.0978901916332)*x_ref[2]
        ref[3,2,0,1]=(0.803590309587)*x_ref[0]**o+(0.470388053389)*x_ref[0]+(0.674942922168)*x_ref[1]**o+(0.223472834762)*x_ref[1]+(0.552332209028)*x_ref[2]**o+(-0.396443498843)*x_ref[2]
        ref[3,2,1,0]=(-0.312384723839)*x_ref[0]**o+(0.392213828608)*x_ref[0]+(0.449465316671)*x_ref[1]**o+(-0.249103089401)*x_ref[1]+(0.601412913458)*x_ref[2]**o+(0.209899368039)*x_ref[2]
        ref[3,2,1,1]=(-0.583776179557)*x_ref[0]**o+(0.485007047754)*x_ref[0]+(-0.0354153633589)*x_ref[1]**o+(-0.898630969916)*x_ref[1]+(-0.266581492795)*x_ref[2]**o+(0.822086680281)*x_ref[2]
        ref[3,2,2,0]=(0.944617094397)*x_ref[0]**o+(-0.173057994833)*x_ref[0]+(0.927832160048)*x_ref[1]**o+(-0.218599594252)*x_ref[1]+(0.349957755011)*x_ref[2]**o+(0.392602744545)*x_ref[2]
        ref[3,2,2,1]=(0.461648084674)*x_ref[0]**o+(0.826213376949)*x_ref[0]+(0.143429054731)*x_ref[1]**o+(-0.214265128007)*x_ref[1]+(0.0317208127607)*x_ref[2]**o+(-0.858361078381)*x_ref[2]
        ref[3,3,0,0]=(0.306607920368)*x_ref[0]**o+(0.147692891688)*x_ref[0]+(-0.562661766456)*x_ref[1]**o+(0.448595564727)*x_ref[1]+(-0.707178218373)*x_ref[2]**o+(0.0655554587866)*x_ref[2]
        ref[3,3,0,1]=(0.135475820362)*x_ref[0]**o+(0.51420718793)*x_ref[0]+(0.343291220736)*x_ref[1]**o+(-0.797532312602)*x_ref[1]+(-0.273431011273)*x_ref[2]**o+(0.135307779253)*x_ref[2]
        ref[3,3,1,0]=(0.488028231073)*x_ref[0]**o+(0.542021806829)*x_ref[0]+(0.91916268609)*x_ref[1]**o+(0.317368273365)*x_ref[1]+(0.314807039395)*x_ref[2]**o+(-0.841583308025)*x_ref[2]
        ref[3,3,1,1]=(-0.182783814917)*x_ref[0]**o+(0.250863481201)*x_ref[0]+(0.755551457003)*x_ref[1]**o+(0.946171548529)*x_ref[1]+(0.0350225436682)*x_ref[2]**o+(0.776059830052)*x_ref[2]
        ref[3,3,2,0]=(-0.116707124994)*x_ref[0]**o+(-0.727010280588)*x_ref[0]+(0.172065389268)*x_ref[1]**o+(0.754164564639)*x_ref[1]+(0.803777635621)*x_ref[2]**o+(0.293490264417)*x_ref[2]
        ref[3,3,2,1]=(0.154235982797)*x_ref[0]**o+(-0.124286021486)*x_ref[0]+(-0.737324748116)*x_ref[1]**o+(-0.136803541581)*x_ref[1]+(-0.608370191604)*x_ref[2]**o+(-0.933599034128)*x_ref[2]
        ref[3,4,0,0]=(0.818061677328)*x_ref[0]**o+(-0.771719613166)*x_ref[0]+(0.742772795251)*x_ref[1]**o+(0.0359285091312)*x_ref[1]+(-0.985656851837)*x_ref[2]**o+(-0.884598206512)*x_ref[2]
        ref[3,4,0,1]=(0.198371732741)*x_ref[0]**o+(0.646496618681)*x_ref[0]+(0.232912446494)*x_ref[1]**o+(0.945619492023)*x_ref[1]+(-0.344370341731)*x_ref[2]**o+(0.31156840357)*x_ref[2]
        ref[3,4,1,0]=(-0.609298834606)*x_ref[0]**o+(0.784697392412)*x_ref[0]+(0.996065619079)*x_ref[1]**o+(-0.284498300043)*x_ref[1]+(-0.652795706803)*x_ref[2]**o+(-0.254746840396)*x_ref[2]
        ref[3,4,1,1]=(-0.0083632822507)*x_ref[0]**o+(0.200923618933)*x_ref[0]+(0.353883930064)*x_ref[1]**o+(0.153257380179)*x_ref[1]+(0.843965464205)*x_ref[2]**o+(0.0418501400703)*x_ref[2]
        ref[3,4,2,0]=(-0.226055622191)*x_ref[0]**o+(-0.701789166696)*x_ref[0]+(0.274080957143)*x_ref[1]**o+(0.0596744660556)*x_ref[1]+(-0.342083152987)*x_ref[2]**o+(0.593518464697)*x_ref[2]
        ref[3,4,2,1]=(-0.288168945023)*x_ref[0]**o+(-0.742841678774)*x_ref[0]+(0.972629532424)*x_ref[1]**o+(0.892100560733)*x_ref[1]+(0.803880474721)*x_ref[2]**o+(-0.0638336211224)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromSymbol_ContinuousFunction_rank0(self):
      """
      tests interpolate for rank 0 Symbol onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ContinuousFunction onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(),dim=dim)
      s=Data(0,(),w)
      ref=Data(0,(),w_ref)
      if dim==2:
        s=(0.207629351539)*x[0]**o+(0.601153935253)*x[0]+(0.280159096838)*x[1]**o+(-0.327076958891)*x[1]
        ref=(0.207629351539)*x_ref[0]**o+(0.601153935253)*x_ref[0]+(0.280159096838)*x_ref[1]**o+(-0.327076958891)*x_ref[1]
      else:
        s=(0.870140941982)*x[0]**o+(0.698352761373)*x[0]+(0.143773825221)*x[1]**o+(-0.906122820343)*x[1]+(-0.479637063866)*x[2]**o+(0.76395958357)*x[2]
        ref=(0.870140941982)*x_ref[0]**o+(0.698352761373)*x_ref[0]+(0.143773825221)*x_ref[1]**o+(-0.906122820343)*x_ref[1]+(-0.479637063866)*x_ref[2]**o+(0.76395958357)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromSymbol_ContinuousFunction_rank1(self):
      """
      tests interpolate for rank 1 Symbol onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ContinuousFunction onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(2,),dim=dim)
      s=Data(0,(2,),w)
      ref=Data(0,(2,),w_ref)
      if dim==2:
        s[0]=(-0.218806478928)*x[0]**o+(0.377016390463)*x[0]+(-0.821557051729)*x[1]**o+(-0.291850215951)*x[1]
        s[1]=(0.268489780039)*x[0]**o+(0.682467861682)*x[0]+(0.247435448778)*x[1]**o+(0.299818969839)*x[1]
        ref[0]=(-0.218806478928)*x_ref[0]**o+(0.377016390463)*x_ref[0]+(-0.821557051729)*x_ref[1]**o+(-0.291850215951)*x_ref[1]
        ref[1]=(0.268489780039)*x_ref[0]**o+(0.682467861682)*x_ref[0]+(0.247435448778)*x_ref[1]**o+(0.299818969839)*x_ref[1]
      else:
        s[0]=(0.45901051382)*x[0]**o+(-0.79161426979)*x[0]+(0.447653516381)*x[1]**o+(-0.50384199989)*x[1]+(0.20809418393)*x[2]**o+(0.978047066867)*x[2]
        s[1]=(-0.383595164108)*x[0]**o+(-0.343736899245)*x[0]+(0.0396873129716)*x[1]**o+(-0.385915355146)*x[1]+(-0.853533231426)*x[2]**o+(0.792420068965)*x[2]
        ref[0]=(0.45901051382)*x_ref[0]**o+(-0.79161426979)*x_ref[0]+(0.447653516381)*x_ref[1]**o+(-0.50384199989)*x_ref[1]+(0.20809418393)*x_ref[2]**o+(0.978047066867)*x_ref[2]
        ref[1]=(-0.383595164108)*x_ref[0]**o+(-0.343736899245)*x_ref[0]+(0.0396873129716)*x_ref[1]**o+(-0.385915355146)*x_ref[1]+(-0.853533231426)*x_ref[2]**o+(0.792420068965)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromSymbol_ContinuousFunction_rank2(self):
      """
      tests interpolate for rank 2 Symbol onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ContinuousFunction onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5),dim=dim)
      s=Data(0,(4, 5),w)
      ref=Data(0,(4, 5),w_ref)
      if dim==2:
        s[0,0]=(-0.0353181613333)*x[0]**o+(0.878876352154)*x[0]+(-0.904935867476)*x[1]**o+(-0.321624539514)*x[1]
        s[0,1]=(-0.288092527799)*x[0]**o+(0.928073219222)*x[0]+(-0.110185302788)*x[1]**o+(-0.891915948247)*x[1]
        s[0,2]=(0.0954349617972)*x[0]**o+(-0.140083515966)*x[0]+(-0.827213026204)*x[1]**o+(-0.910012731257)*x[1]
        s[0,3]=(-0.565938692108)*x[0]**o+(0.991031920541)*x[0]+(-0.582526646689)*x[1]**o+(0.0715276554003)*x[1]
        s[0,4]=(0.621703736822)*x[0]**o+(-0.123891976545)*x[0]+(0.284804346655)*x[1]**o+(0.712402142502)*x[1]
        s[1,0]=(0.406855284727)*x[0]**o+(0.481423856624)*x[0]+(0.196078127047)*x[1]**o+(-0.514004426398)*x[1]
        s[1,1]=(-0.378073499305)*x[0]**o+(0.473337934464)*x[0]+(-0.055111166928)*x[1]**o+(-0.0436034347619)*x[1]
        s[1,2]=(-0.124152152221)*x[0]**o+(-0.420813454028)*x[0]+(0.569674875796)*x[1]**o+(-0.101882853127)*x[1]
        s[1,3]=(-0.528314793712)*x[0]**o+(-0.693704904591)*x[0]+(0.676975909426)*x[1]**o+(0.150922079384)*x[1]
        s[1,4]=(-0.738713706919)*x[0]**o+(-0.366958606319)*x[0]+(0.634632816865)*x[1]**o+(0.793822443349)*x[1]
        s[2,0]=(-0.743224960117)*x[0]**o+(0.885444739748)*x[0]+(-0.569356473669)*x[1]**o+(0.0554721350962)*x[1]
        s[2,1]=(-0.685934253863)*x[0]**o+(0.00101896345091)*x[0]+(0.756736504806)*x[1]**o+(-0.823095371277)*x[1]
        s[2,2]=(-0.220514216102)*x[0]**o+(-0.710824329182)*x[0]+(0.838443592125)*x[1]**o+(0.737338692368)*x[1]
        s[2,3]=(0.386990571268)*x[0]**o+(-0.754808523044)*x[0]+(0.259729871523)*x[1]**o+(0.757370896746)*x[1]
        s[2,4]=(0.942933704541)*x[0]**o+(-0.37055099742)*x[0]+(-0.807891788882)*x[1]**o+(0.875578199856)*x[1]
        s[3,0]=(-0.288294610358)*x[0]**o+(-0.547708565934)*x[0]+(0.875419437633)*x[1]**o+(0.25641477553)*x[1]
        s[3,1]=(-0.283600102851)*x[0]**o+(0.682131874062)*x[0]+(-0.124142296952)*x[1]**o+(0.122898656305)*x[1]
        s[3,2]=(0.714735953181)*x[0]**o+(-0.190821251504)*x[0]+(0.273433212059)*x[1]**o+(0.812718970292)*x[1]
        s[3,3]=(0.338475492958)*x[0]**o+(0.863340300202)*x[0]+(-0.512598335671)*x[1]**o+(0.219919160346)*x[1]
        s[3,4]=(-0.735913321392)*x[0]**o+(-0.599322689823)*x[0]+(-0.69249277704)*x[1]**o+(-0.665352731149)*x[1]
        ref[0,0]=(-0.0353181613333)*x_ref[0]**o+(0.878876352154)*x_ref[0]+(-0.904935867476)*x_ref[1]**o+(-0.321624539514)*x_ref[1]
        ref[0,1]=(-0.288092527799)*x_ref[0]**o+(0.928073219222)*x_ref[0]+(-0.110185302788)*x_ref[1]**o+(-0.891915948247)*x_ref[1]
        ref[0,2]=(0.0954349617972)*x_ref[0]**o+(-0.140083515966)*x_ref[0]+(-0.827213026204)*x_ref[1]**o+(-0.910012731257)*x_ref[1]
        ref[0,3]=(-0.565938692108)*x_ref[0]**o+(0.991031920541)*x_ref[0]+(-0.582526646689)*x_ref[1]**o+(0.0715276554003)*x_ref[1]
        ref[0,4]=(0.621703736822)*x_ref[0]**o+(-0.123891976545)*x_ref[0]+(0.284804346655)*x_ref[1]**o+(0.712402142502)*x_ref[1]
        ref[1,0]=(0.406855284727)*x_ref[0]**o+(0.481423856624)*x_ref[0]+(0.196078127047)*x_ref[1]**o+(-0.514004426398)*x_ref[1]
        ref[1,1]=(-0.378073499305)*x_ref[0]**o+(0.473337934464)*x_ref[0]+(-0.055111166928)*x_ref[1]**o+(-0.0436034347619)*x_ref[1]
        ref[1,2]=(-0.124152152221)*x_ref[0]**o+(-0.420813454028)*x_ref[0]+(0.569674875796)*x_ref[1]**o+(-0.101882853127)*x_ref[1]
        ref[1,3]=(-0.528314793712)*x_ref[0]**o+(-0.693704904591)*x_ref[0]+(0.676975909426)*x_ref[1]**o+(0.150922079384)*x_ref[1]
        ref[1,4]=(-0.738713706919)*x_ref[0]**o+(-0.366958606319)*x_ref[0]+(0.634632816865)*x_ref[1]**o+(0.793822443349)*x_ref[1]
        ref[2,0]=(-0.743224960117)*x_ref[0]**o+(0.885444739748)*x_ref[0]+(-0.569356473669)*x_ref[1]**o+(0.0554721350962)*x_ref[1]
        ref[2,1]=(-0.685934253863)*x_ref[0]**o+(0.00101896345091)*x_ref[0]+(0.756736504806)*x_ref[1]**o+(-0.823095371277)*x_ref[1]
        ref[2,2]=(-0.220514216102)*x_ref[0]**o+(-0.710824329182)*x_ref[0]+(0.838443592125)*x_ref[1]**o+(0.737338692368)*x_ref[1]
        ref[2,3]=(0.386990571268)*x_ref[0]**o+(-0.754808523044)*x_ref[0]+(0.259729871523)*x_ref[1]**o+(0.757370896746)*x_ref[1]
        ref[2,4]=(0.942933704541)*x_ref[0]**o+(-0.37055099742)*x_ref[0]+(-0.807891788882)*x_ref[1]**o+(0.875578199856)*x_ref[1]
        ref[3,0]=(-0.288294610358)*x_ref[0]**o+(-0.547708565934)*x_ref[0]+(0.875419437633)*x_ref[1]**o+(0.25641477553)*x_ref[1]
        ref[3,1]=(-0.283600102851)*x_ref[0]**o+(0.682131874062)*x_ref[0]+(-0.124142296952)*x_ref[1]**o+(0.122898656305)*x_ref[1]
        ref[3,2]=(0.714735953181)*x_ref[0]**o+(-0.190821251504)*x_ref[0]+(0.273433212059)*x_ref[1]**o+(0.812718970292)*x_ref[1]
        ref[3,3]=(0.338475492958)*x_ref[0]**o+(0.863340300202)*x_ref[0]+(-0.512598335671)*x_ref[1]**o+(0.219919160346)*x_ref[1]
        ref[3,4]=(-0.735913321392)*x_ref[0]**o+(-0.599322689823)*x_ref[0]+(-0.69249277704)*x_ref[1]**o+(-0.665352731149)*x_ref[1]
      else:
        s[0,0]=(0.430293433102)*x[0]**o+(-0.857172626308)*x[0]+(0.868470915818)*x[1]**o+(-0.882235724566)*x[1]+(0.284271991649)*x[2]**o+(-0.459504355652)*x[2]
        s[0,1]=(-0.679633702459)*x[0]**o+(-0.222491254643)*x[0]+(0.286851801882)*x[1]**o+(-0.844507424335)*x[1]+(-0.808810961797)*x[2]**o+(0.139670580428)*x[2]
        s[0,2]=(0.682748402275)*x[0]**o+(-0.392534760514)*x[0]+(0.716754522284)*x[1]**o+(-0.151837258954)*x[1]+(0.0826469913138)*x[2]**o+(-0.316631816663)*x[2]
        s[0,3]=(0.551943486674)*x[0]**o+(0.946320480258)*x[0]+(0.75350645127)*x[1]**o+(0.543509582247)*x[1]+(-0.97668108246)*x[2]**o+(-0.910891750491)*x[2]
        s[0,4]=(-0.317631622813)*x[0]**o+(0.647217229231)*x[0]+(0.770238984569)*x[1]**o+(0.873217492832)*x[1]+(0.512276250922)*x[2]**o+(0.282068518751)*x[2]
        s[1,0]=(-0.23327675725)*x[0]**o+(0.562413317314)*x[0]+(-0.0812857523424)*x[1]**o+(-0.721924983109)*x[1]+(0.775846134529)*x[2]**o+(-0.14640332198)*x[2]
        s[1,1]=(0.00265339866254)*x[0]**o+(0.486208331625)*x[0]+(-0.530584135285)*x[1]**o+(-0.84550455983)*x[1]+(-0.296142439781)*x[2]**o+(-0.595262357569)*x[2]
        s[1,2]=(0.985928898395)*x[0]**o+(-0.681211427726)*x[0]+(0.926263452555)*x[1]**o+(-0.435493892249)*x[1]+(-0.312630433899)*x[2]**o+(0.584197447454)*x[2]
        s[1,3]=(-0.558671284798)*x[0]**o+(0.837591478004)*x[0]+(0.00535393855178)*x[1]**o+(-0.662459914024)*x[1]+(0.558541008138)*x[2]**o+(-0.314047626792)*x[2]
        s[1,4]=(-0.300759983439)*x[0]**o+(0.354828936226)*x[0]+(-0.986139955736)*x[1]**o+(0.202600852457)*x[1]+(-0.927794980602)*x[2]**o+(-0.359202012865)*x[2]
        s[2,0]=(0.829016217959)*x[0]**o+(-0.423599278647)*x[0]+(-0.40846564457)*x[1]**o+(-0.869126229204)*x[1]+(0.473022094753)*x[2]**o+(0.910852581174)*x[2]
        s[2,1]=(-0.706119428033)*x[0]**o+(0.0566333827809)*x[0]+(0.809800273677)*x[1]**o+(-0.588604664572)*x[1]+(-0.285988770457)*x[2]**o+(-0.904317124653)*x[2]
        s[2,2]=(0.449032734821)*x[0]**o+(0.896383171746)*x[0]+(-0.852643286424)*x[1]**o+(0.650126054768)*x[1]+(0.255148390617)*x[2]**o+(0.38493564187)*x[2]
        s[2,3]=(0.269546556328)*x[0]**o+(0.217209115989)*x[0]+(0.0299948255851)*x[1]**o+(0.966502659562)*x[1]+(-0.837125730275)*x[2]**o+(0.192194381152)*x[2]
        s[2,4]=(0.749886797267)*x[0]**o+(-0.358784003038)*x[0]+(0.458790223998)*x[1]**o+(-0.868481406026)*x[1]+(-0.856425916274)*x[2]**o+(-0.177865937666)*x[2]
        s[3,0]=(0.944647107347)*x[0]**o+(-0.599969226039)*x[0]+(0.606271189912)*x[1]**o+(0.515347418734)*x[1]+(-0.239547882059)*x[2]**o+(0.179383932895)*x[2]
        s[3,1]=(0.202273995087)*x[0]**o+(0.559445783871)*x[0]+(-0.358032565476)*x[1]**o+(-0.512085149684)*x[1]+(0.888993422451)*x[2]**o+(0.0854392659772)*x[2]
        s[3,2]=(0.367717494858)*x[0]**o+(-0.14816870753)*x[0]+(0.99339026962)*x[1]**o+(0.705222667312)*x[1]+(0.700295737391)*x[2]**o+(0.403996497394)*x[2]
        s[3,3]=(-0.104867279309)*x[0]**o+(0.814918041051)*x[0]+(0.252610473279)*x[1]**o+(-0.744193407199)*x[1]+(0.0189009064209)*x[2]**o+(0.314571922242)*x[2]
        s[3,4]=(-0.622691514266)*x[0]**o+(-0.964201372823)*x[0]+(-0.411043099228)*x[1]**o+(0.142020161136)*x[1]+(-0.587923250915)*x[2]**o+(0.37852533939)*x[2]
        ref[0,0]=(0.430293433102)*x_ref[0]**o+(-0.857172626308)*x_ref[0]+(0.868470915818)*x_ref[1]**o+(-0.882235724566)*x_ref[1]+(0.284271991649)*x_ref[2]**o+(-0.459504355652)*x_ref[2]
        ref[0,1]=(-0.679633702459)*x_ref[0]**o+(-0.222491254643)*x_ref[0]+(0.286851801882)*x_ref[1]**o+(-0.844507424335)*x_ref[1]+(-0.808810961797)*x_ref[2]**o+(0.139670580428)*x_ref[2]
        ref[0,2]=(0.682748402275)*x_ref[0]**o+(-0.392534760514)*x_ref[0]+(0.716754522284)*x_ref[1]**o+(-0.151837258954)*x_ref[1]+(0.0826469913138)*x_ref[2]**o+(-0.316631816663)*x_ref[2]
        ref[0,3]=(0.551943486674)*x_ref[0]**o+(0.946320480258)*x_ref[0]+(0.75350645127)*x_ref[1]**o+(0.543509582247)*x_ref[1]+(-0.97668108246)*x_ref[2]**o+(-0.910891750491)*x_ref[2]
        ref[0,4]=(-0.317631622813)*x_ref[0]**o+(0.647217229231)*x_ref[0]+(0.770238984569)*x_ref[1]**o+(0.873217492832)*x_ref[1]+(0.512276250922)*x_ref[2]**o+(0.282068518751)*x_ref[2]
        ref[1,0]=(-0.23327675725)*x_ref[0]**o+(0.562413317314)*x_ref[0]+(-0.0812857523424)*x_ref[1]**o+(-0.721924983109)*x_ref[1]+(0.775846134529)*x_ref[2]**o+(-0.14640332198)*x_ref[2]
        ref[1,1]=(0.00265339866254)*x_ref[0]**o+(0.486208331625)*x_ref[0]+(-0.530584135285)*x_ref[1]**o+(-0.84550455983)*x_ref[1]+(-0.296142439781)*x_ref[2]**o+(-0.595262357569)*x_ref[2]
        ref[1,2]=(0.985928898395)*x_ref[0]**o+(-0.681211427726)*x_ref[0]+(0.926263452555)*x_ref[1]**o+(-0.435493892249)*x_ref[1]+(-0.312630433899)*x_ref[2]**o+(0.584197447454)*x_ref[2]
        ref[1,3]=(-0.558671284798)*x_ref[0]**o+(0.837591478004)*x_ref[0]+(0.00535393855178)*x_ref[1]**o+(-0.662459914024)*x_ref[1]+(0.558541008138)*x_ref[2]**o+(-0.314047626792)*x_ref[2]
        ref[1,4]=(-0.300759983439)*x_ref[0]**o+(0.354828936226)*x_ref[0]+(-0.986139955736)*x_ref[1]**o+(0.202600852457)*x_ref[1]+(-0.927794980602)*x_ref[2]**o+(-0.359202012865)*x_ref[2]
        ref[2,0]=(0.829016217959)*x_ref[0]**o+(-0.423599278647)*x_ref[0]+(-0.40846564457)*x_ref[1]**o+(-0.869126229204)*x_ref[1]+(0.473022094753)*x_ref[2]**o+(0.910852581174)*x_ref[2]
        ref[2,1]=(-0.706119428033)*x_ref[0]**o+(0.0566333827809)*x_ref[0]+(0.809800273677)*x_ref[1]**o+(-0.588604664572)*x_ref[1]+(-0.285988770457)*x_ref[2]**o+(-0.904317124653)*x_ref[2]
        ref[2,2]=(0.449032734821)*x_ref[0]**o+(0.896383171746)*x_ref[0]+(-0.852643286424)*x_ref[1]**o+(0.650126054768)*x_ref[1]+(0.255148390617)*x_ref[2]**o+(0.38493564187)*x_ref[2]
        ref[2,3]=(0.269546556328)*x_ref[0]**o+(0.217209115989)*x_ref[0]+(0.0299948255851)*x_ref[1]**o+(0.966502659562)*x_ref[1]+(-0.837125730275)*x_ref[2]**o+(0.192194381152)*x_ref[2]
        ref[2,4]=(0.749886797267)*x_ref[0]**o+(-0.358784003038)*x_ref[0]+(0.458790223998)*x_ref[1]**o+(-0.868481406026)*x_ref[1]+(-0.856425916274)*x_ref[2]**o+(-0.177865937666)*x_ref[2]
        ref[3,0]=(0.944647107347)*x_ref[0]**o+(-0.599969226039)*x_ref[0]+(0.606271189912)*x_ref[1]**o+(0.515347418734)*x_ref[1]+(-0.239547882059)*x_ref[2]**o+(0.179383932895)*x_ref[2]
        ref[3,1]=(0.202273995087)*x_ref[0]**o+(0.559445783871)*x_ref[0]+(-0.358032565476)*x_ref[1]**o+(-0.512085149684)*x_ref[1]+(0.888993422451)*x_ref[2]**o+(0.0854392659772)*x_ref[2]
        ref[3,2]=(0.367717494858)*x_ref[0]**o+(-0.14816870753)*x_ref[0]+(0.99339026962)*x_ref[1]**o+(0.705222667312)*x_ref[1]+(0.700295737391)*x_ref[2]**o+(0.403996497394)*x_ref[2]
        ref[3,3]=(-0.104867279309)*x_ref[0]**o+(0.814918041051)*x_ref[0]+(0.252610473279)*x_ref[1]**o+(-0.744193407199)*x_ref[1]+(0.0189009064209)*x_ref[2]**o+(0.314571922242)*x_ref[2]
        ref[3,4]=(-0.622691514266)*x_ref[0]**o+(-0.964201372823)*x_ref[0]+(-0.411043099228)*x_ref[1]**o+(0.142020161136)*x_ref[1]+(-0.587923250915)*x_ref[2]**o+(0.37852533939)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromSymbol_ContinuousFunction_rank3(self):
      """
      tests interpolate for rank 3 Symbol onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ContinuousFunction onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(6, 2, 2),dim=dim)
      s=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2),w_ref)
      if dim==2:
        s[0,0,0]=(0.486126148472)*x[0]**o+(-0.225402743005)*x[0]+(0.182927976639)*x[1]**o+(0.620824619579)*x[1]
        s[0,0,1]=(-0.407409056531)*x[0]**o+(-0.833838057228)*x[0]+(0.644035353162)*x[1]**o+(-0.672679458606)*x[1]
        s[0,1,0]=(-0.581856850287)*x[0]**o+(-0.0618093338242)*x[0]+(0.684919787457)*x[1]**o+(-0.788915767932)*x[1]
        s[0,1,1]=(-0.224040348858)*x[0]**o+(-0.103067046916)*x[0]+(-0.578696291301)*x[1]**o+(0.00211758484966)*x[1]
        s[1,0,0]=(0.979651701027)*x[0]**o+(-0.106752935345)*x[0]+(0.109182674529)*x[1]**o+(-0.487877441364)*x[1]
        s[1,0,1]=(0.108662013179)*x[0]**o+(0.309241035214)*x[0]+(0.666605783286)*x[1]**o+(-0.631752854289)*x[1]
        s[1,1,0]=(0.171413355921)*x[0]**o+(-0.468708755515)*x[0]+(0.668398470024)*x[1]**o+(0.12499650775)*x[1]
        s[1,1,1]=(0.203364381723)*x[0]**o+(0.149271421925)*x[0]+(0.0393327688768)*x[1]**o+(-0.342283671904)*x[1]
        s[2,0,0]=(0.426437033887)*x[0]**o+(-0.0681592307883)*x[0]+(-0.96231688885)*x[1]**o+(-0.74406098717)*x[1]
        s[2,0,1]=(0.817824016472)*x[0]**o+(-0.0332805444397)*x[0]+(0.274976215212)*x[1]**o+(0.0486739871191)*x[1]
        s[2,1,0]=(0.201535943822)*x[0]**o+(-0.620687964831)*x[0]+(0.675814320754)*x[1]**o+(-0.865640523715)*x[1]
        s[2,1,1]=(-0.89105770032)*x[0]**o+(0.730291041637)*x[0]+(-0.0379223326195)*x[1]**o+(0.874877694224)*x[1]
        s[3,0,0]=(-0.562138233363)*x[0]**o+(-0.211931551199)*x[0]+(0.245533602059)*x[1]**o+(-0.752149846924)*x[1]
        s[3,0,1]=(-0.54251114713)*x[0]**o+(0.663375258793)*x[0]+(0.787787067042)*x[1]**o+(-0.128869775715)*x[1]
        s[3,1,0]=(-0.317250597186)*x[0]**o+(-0.92812680953)*x[0]+(-0.787378775549)*x[1]**o+(0.0495460276628)*x[1]
        s[3,1,1]=(0.41215674863)*x[0]**o+(-0.647351893065)*x[0]+(0.910056539934)*x[1]**o+(-0.931747263108)*x[1]
        s[4,0,0]=(-0.239940821453)*x[0]**o+(0.811323165415)*x[0]+(0.524171970463)*x[1]**o+(0.919862325215)*x[1]
        s[4,0,1]=(-0.812864948023)*x[0]**o+(0.966779570342)*x[0]+(-0.528240223073)*x[1]**o+(0.532264572013)*x[1]
        s[4,1,0]=(-0.905620729062)*x[0]**o+(-0.532558330506)*x[0]+(-0.828115659854)*x[1]**o+(-0.0539571069974)*x[1]
        s[4,1,1]=(-0.456328687883)*x[0]**o+(0.198344289976)*x[0]+(0.255309661612)*x[1]**o+(-0.530390597461)*x[1]
        s[5,0,0]=(-0.525932317875)*x[0]**o+(0.959390874707)*x[0]+(-0.860006819486)*x[1]**o+(0.485364243112)*x[1]
        s[5,0,1]=(0.41405164663)*x[0]**o+(0.738240577168)*x[0]+(-0.158776067456)*x[1]**o+(-0.0120083238952)*x[1]
        s[5,1,0]=(0.915095618736)*x[0]**o+(-0.180612310391)*x[0]+(0.859456959672)*x[1]**o+(-0.230245385712)*x[1]
        s[5,1,1]=(0.195626998508)*x[0]**o+(-0.895957896446)*x[0]+(-0.674564217018)*x[1]**o+(0.182147320643)*x[1]
        ref[0,0,0]=(0.486126148472)*x_ref[0]**o+(-0.225402743005)*x_ref[0]+(0.182927976639)*x_ref[1]**o+(0.620824619579)*x_ref[1]
        ref[0,0,1]=(-0.407409056531)*x_ref[0]**o+(-0.833838057228)*x_ref[0]+(0.644035353162)*x_ref[1]**o+(-0.672679458606)*x_ref[1]
        ref[0,1,0]=(-0.581856850287)*x_ref[0]**o+(-0.0618093338242)*x_ref[0]+(0.684919787457)*x_ref[1]**o+(-0.788915767932)*x_ref[1]
        ref[0,1,1]=(-0.224040348858)*x_ref[0]**o+(-0.103067046916)*x_ref[0]+(-0.578696291301)*x_ref[1]**o+(0.00211758484966)*x_ref[1]
        ref[1,0,0]=(0.979651701027)*x_ref[0]**o+(-0.106752935345)*x_ref[0]+(0.109182674529)*x_ref[1]**o+(-0.487877441364)*x_ref[1]
        ref[1,0,1]=(0.108662013179)*x_ref[0]**o+(0.309241035214)*x_ref[0]+(0.666605783286)*x_ref[1]**o+(-0.631752854289)*x_ref[1]
        ref[1,1,0]=(0.171413355921)*x_ref[0]**o+(-0.468708755515)*x_ref[0]+(0.668398470024)*x_ref[1]**o+(0.12499650775)*x_ref[1]
        ref[1,1,1]=(0.203364381723)*x_ref[0]**o+(0.149271421925)*x_ref[0]+(0.0393327688768)*x_ref[1]**o+(-0.342283671904)*x_ref[1]
        ref[2,0,0]=(0.426437033887)*x_ref[0]**o+(-0.0681592307883)*x_ref[0]+(-0.96231688885)*x_ref[1]**o+(-0.74406098717)*x_ref[1]
        ref[2,0,1]=(0.817824016472)*x_ref[0]**o+(-0.0332805444397)*x_ref[0]+(0.274976215212)*x_ref[1]**o+(0.0486739871191)*x_ref[1]
        ref[2,1,0]=(0.201535943822)*x_ref[0]**o+(-0.620687964831)*x_ref[0]+(0.675814320754)*x_ref[1]**o+(-0.865640523715)*x_ref[1]
        ref[2,1,1]=(-0.89105770032)*x_ref[0]**o+(0.730291041637)*x_ref[0]+(-0.0379223326195)*x_ref[1]**o+(0.874877694224)*x_ref[1]
        ref[3,0,0]=(-0.562138233363)*x_ref[0]**o+(-0.211931551199)*x_ref[0]+(0.245533602059)*x_ref[1]**o+(-0.752149846924)*x_ref[1]
        ref[3,0,1]=(-0.54251114713)*x_ref[0]**o+(0.663375258793)*x_ref[0]+(0.787787067042)*x_ref[1]**o+(-0.128869775715)*x_ref[1]
        ref[3,1,0]=(-0.317250597186)*x_ref[0]**o+(-0.92812680953)*x_ref[0]+(-0.787378775549)*x_ref[1]**o+(0.0495460276628)*x_ref[1]
        ref[3,1,1]=(0.41215674863)*x_ref[0]**o+(-0.647351893065)*x_ref[0]+(0.910056539934)*x_ref[1]**o+(-0.931747263108)*x_ref[1]
        ref[4,0,0]=(-0.239940821453)*x_ref[0]**o+(0.811323165415)*x_ref[0]+(0.524171970463)*x_ref[1]**o+(0.919862325215)*x_ref[1]
        ref[4,0,1]=(-0.812864948023)*x_ref[0]**o+(0.966779570342)*x_ref[0]+(-0.528240223073)*x_ref[1]**o+(0.532264572013)*x_ref[1]
        ref[4,1,0]=(-0.905620729062)*x_ref[0]**o+(-0.532558330506)*x_ref[0]+(-0.828115659854)*x_ref[1]**o+(-0.0539571069974)*x_ref[1]
        ref[4,1,1]=(-0.456328687883)*x_ref[0]**o+(0.198344289976)*x_ref[0]+(0.255309661612)*x_ref[1]**o+(-0.530390597461)*x_ref[1]
        ref[5,0,0]=(-0.525932317875)*x_ref[0]**o+(0.959390874707)*x_ref[0]+(-0.860006819486)*x_ref[1]**o+(0.485364243112)*x_ref[1]
        ref[5,0,1]=(0.41405164663)*x_ref[0]**o+(0.738240577168)*x_ref[0]+(-0.158776067456)*x_ref[1]**o+(-0.0120083238952)*x_ref[1]
        ref[5,1,0]=(0.915095618736)*x_ref[0]**o+(-0.180612310391)*x_ref[0]+(0.859456959672)*x_ref[1]**o+(-0.230245385712)*x_ref[1]
        ref[5,1,1]=(0.195626998508)*x_ref[0]**o+(-0.895957896446)*x_ref[0]+(-0.674564217018)*x_ref[1]**o+(0.182147320643)*x_ref[1]
      else:
        s[0,0,0]=(-0.386562865086)*x[0]**o+(-0.101923658127)*x[0]+(0.366033434213)*x[1]**o+(-0.458022094261)*x[1]+(-0.269807341125)*x[2]**o+(0.210015891017)*x[2]
        s[0,0,1]=(-0.232605620109)*x[0]**o+(-0.89635125143)*x[0]+(-0.344907210539)*x[1]**o+(-0.117272497457)*x[1]+(0.86877312028)*x[2]**o+(-0.984463097169)*x[2]
        s[0,1,0]=(-0.734186269208)*x[0]**o+(-0.955220234973)*x[0]+(0.355373019012)*x[1]**o+(-0.0259995318533)*x[1]+(-0.923817606426)*x[2]**o+(0.707611624363)*x[2]
        s[0,1,1]=(-0.65807304305)*x[0]**o+(-0.546854762897)*x[0]+(-0.041383885518)*x[1]**o+(-0.847656164394)*x[1]+(-0.69366771235)*x[2]**o+(-0.00594720800349)*x[2]
        s[1,0,0]=(0.4916488707)*x[0]**o+(0.610932307599)*x[0]+(-0.634836490924)*x[1]**o+(0.190377750049)*x[1]+(0.839648718767)*x[2]**o+(-0.697199112832)*x[2]
        s[1,0,1]=(-0.733462530162)*x[0]**o+(0.897438124381)*x[0]+(-0.284428270428)*x[1]**o+(-0.490325551864)*x[1]+(-0.425270047571)*x[2]**o+(-0.0169167898593)*x[2]
        s[1,1,0]=(-0.1134205794)*x[0]**o+(-0.589938051075)*x[0]+(-0.551954550243)*x[1]**o+(-0.828713609462)*x[1]+(-0.14740902465)*x[2]**o+(-0.0643506708023)*x[2]
        s[1,1,1]=(0.875176575479)*x[0]**o+(-0.436379792297)*x[0]+(-0.238769521024)*x[1]**o+(-0.0883649176657)*x[1]+(-0.268393504724)*x[2]**o+(-0.331629361371)*x[2]
        s[2,0,0]=(-0.445061932808)*x[0]**o+(-0.136283305342)*x[0]+(0.109782590986)*x[1]**o+(0.664151679353)*x[1]+(0.77415319639)*x[2]**o+(0.918600325668)*x[2]
        s[2,0,1]=(-0.966582148735)*x[0]**o+(0.90850899638)*x[0]+(0.860043581705)*x[1]**o+(0.555164541705)*x[1]+(-0.0881641948474)*x[2]**o+(-0.437393798496)*x[2]
        s[2,1,0]=(0.424061230847)*x[0]**o+(0.426231869756)*x[0]+(0.20042430919)*x[1]**o+(-0.392259834985)*x[1]+(0.56903598931)*x[2]**o+(-0.713346863994)*x[2]
        s[2,1,1]=(0.391005749137)*x[0]**o+(-0.141533299622)*x[0]+(-0.54549941972)*x[1]**o+(-0.551542113521)*x[1]+(0.461175241452)*x[2]**o+(0.946400145746)*x[2]
        s[3,0,0]=(0.690778432543)*x[0]**o+(0.0987975519633)*x[0]+(0.600232607252)*x[1]**o+(-0.403160580412)*x[1]+(-0.259142718629)*x[2]**o+(0.125408478863)*x[2]
        s[3,0,1]=(0.45235548032)*x[0]**o+(0.465495192061)*x[0]+(0.605495566594)*x[1]**o+(0.0665669455329)*x[1]+(-0.528374124117)*x[2]**o+(0.971371028924)*x[2]
        s[3,1,0]=(-0.868740120635)*x[0]**o+(0.94078043392)*x[0]+(-0.924836120128)*x[1]**o+(0.0436824783429)*x[1]+(0.436490992322)*x[2]**o+(0.563162883509)*x[2]
        s[3,1,1]=(-0.148701893094)*x[0]**o+(0.114671183607)*x[0]+(0.40845144069)*x[1]**o+(-0.35749472879)*x[1]+(-0.240456109591)*x[2]**o+(-0.959187876519)*x[2]
        s[4,0,0]=(-0.902322503602)*x[0]**o+(-0.0870613571753)*x[0]+(0.604597216194)*x[1]**o+(0.484694096025)*x[1]+(0.733216575545)*x[2]**o+(0.981143903885)*x[2]
        s[4,0,1]=(-0.9183984444)*x[0]**o+(0.222613734722)*x[0]+(0.472361095251)*x[1]**o+(-0.182413993184)*x[1]+(0.404737782329)*x[2]**o+(-0.462634159071)*x[2]
        s[4,1,0]=(-0.577317659134)*x[0]**o+(0.863519222199)*x[0]+(-0.955859899586)*x[1]**o+(0.898032633657)*x[1]+(0.000203504014658)*x[2]**o+(0.84858789073)*x[2]
        s[4,1,1]=(0.103931145317)*x[0]**o+(-0.491733208237)*x[0]+(-0.679616928853)*x[1]**o+(0.779966695883)*x[1]+(0.19391391729)*x[2]**o+(0.237260204789)*x[2]
        s[5,0,0]=(-0.587657719833)*x[0]**o+(0.0677059438625)*x[0]+(-0.070159254762)*x[1]**o+(-0.725676771128)*x[1]+(0.563139108018)*x[2]**o+(-0.182136289146)*x[2]
        s[5,0,1]=(0.647754539947)*x[0]**o+(-0.238569428669)*x[0]+(-0.0352701585821)*x[1]**o+(0.117211931696)*x[1]+(-0.749719337272)*x[2]**o+(-0.0572910818861)*x[2]
        s[5,1,0]=(0.0802652788129)*x[0]**o+(-0.320772823979)*x[0]+(0.875989601034)*x[1]**o+(0.56803751787)*x[1]+(0.115080146257)*x[2]**o+(0.960937482841)*x[2]
        s[5,1,1]=(0.374333508365)*x[0]**o+(-0.198321408411)*x[0]+(0.550730384308)*x[1]**o+(0.980522161961)*x[1]+(-0.523593924556)*x[2]**o+(-0.0882497539881)*x[2]
        ref[0,0,0]=(-0.386562865086)*x_ref[0]**o+(-0.101923658127)*x_ref[0]+(0.366033434213)*x_ref[1]**o+(-0.458022094261)*x_ref[1]+(-0.269807341125)*x_ref[2]**o+(0.210015891017)*x_ref[2]
        ref[0,0,1]=(-0.232605620109)*x_ref[0]**o+(-0.89635125143)*x_ref[0]+(-0.344907210539)*x_ref[1]**o+(-0.117272497457)*x_ref[1]+(0.86877312028)*x_ref[2]**o+(-0.984463097169)*x_ref[2]
        ref[0,1,0]=(-0.734186269208)*x_ref[0]**o+(-0.955220234973)*x_ref[0]+(0.355373019012)*x_ref[1]**o+(-0.0259995318533)*x_ref[1]+(-0.923817606426)*x_ref[2]**o+(0.707611624363)*x_ref[2]
        ref[0,1,1]=(-0.65807304305)*x_ref[0]**o+(-0.546854762897)*x_ref[0]+(-0.041383885518)*x_ref[1]**o+(-0.847656164394)*x_ref[1]+(-0.69366771235)*x_ref[2]**o+(-0.00594720800349)*x_ref[2]
        ref[1,0,0]=(0.4916488707)*x_ref[0]**o+(0.610932307599)*x_ref[0]+(-0.634836490924)*x_ref[1]**o+(0.190377750049)*x_ref[1]+(0.839648718767)*x_ref[2]**o+(-0.697199112832)*x_ref[2]
        ref[1,0,1]=(-0.733462530162)*x_ref[0]**o+(0.897438124381)*x_ref[0]+(-0.284428270428)*x_ref[1]**o+(-0.490325551864)*x_ref[1]+(-0.425270047571)*x_ref[2]**o+(-0.0169167898593)*x_ref[2]
        ref[1,1,0]=(-0.1134205794)*x_ref[0]**o+(-0.589938051075)*x_ref[0]+(-0.551954550243)*x_ref[1]**o+(-0.828713609462)*x_ref[1]+(-0.14740902465)*x_ref[2]**o+(-0.0643506708023)*x_ref[2]
        ref[1,1,1]=(0.875176575479)*x_ref[0]**o+(-0.436379792297)*x_ref[0]+(-0.238769521024)*x_ref[1]**o+(-0.0883649176657)*x_ref[1]+(-0.268393504724)*x_ref[2]**o+(-0.331629361371)*x_ref[2]
        ref[2,0,0]=(-0.445061932808)*x_ref[0]**o+(-0.136283305342)*x_ref[0]+(0.109782590986)*x_ref[1]**o+(0.664151679353)*x_ref[1]+(0.77415319639)*x_ref[2]**o+(0.918600325668)*x_ref[2]
        ref[2,0,1]=(-0.966582148735)*x_ref[0]**o+(0.90850899638)*x_ref[0]+(0.860043581705)*x_ref[1]**o+(0.555164541705)*x_ref[1]+(-0.0881641948474)*x_ref[2]**o+(-0.437393798496)*x_ref[2]
        ref[2,1,0]=(0.424061230847)*x_ref[0]**o+(0.426231869756)*x_ref[0]+(0.20042430919)*x_ref[1]**o+(-0.392259834985)*x_ref[1]+(0.56903598931)*x_ref[2]**o+(-0.713346863994)*x_ref[2]
        ref[2,1,1]=(0.391005749137)*x_ref[0]**o+(-0.141533299622)*x_ref[0]+(-0.54549941972)*x_ref[1]**o+(-0.551542113521)*x_ref[1]+(0.461175241452)*x_ref[2]**o+(0.946400145746)*x_ref[2]
        ref[3,0,0]=(0.690778432543)*x_ref[0]**o+(0.0987975519633)*x_ref[0]+(0.600232607252)*x_ref[1]**o+(-0.403160580412)*x_ref[1]+(-0.259142718629)*x_ref[2]**o+(0.125408478863)*x_ref[2]
        ref[3,0,1]=(0.45235548032)*x_ref[0]**o+(0.465495192061)*x_ref[0]+(0.605495566594)*x_ref[1]**o+(0.0665669455329)*x_ref[1]+(-0.528374124117)*x_ref[2]**o+(0.971371028924)*x_ref[2]
        ref[3,1,0]=(-0.868740120635)*x_ref[0]**o+(0.94078043392)*x_ref[0]+(-0.924836120128)*x_ref[1]**o+(0.0436824783429)*x_ref[1]+(0.436490992322)*x_ref[2]**o+(0.563162883509)*x_ref[2]
        ref[3,1,1]=(-0.148701893094)*x_ref[0]**o+(0.114671183607)*x_ref[0]+(0.40845144069)*x_ref[1]**o+(-0.35749472879)*x_ref[1]+(-0.240456109591)*x_ref[2]**o+(-0.959187876519)*x_ref[2]
        ref[4,0,0]=(-0.902322503602)*x_ref[0]**o+(-0.0870613571753)*x_ref[0]+(0.604597216194)*x_ref[1]**o+(0.484694096025)*x_ref[1]+(0.733216575545)*x_ref[2]**o+(0.981143903885)*x_ref[2]
        ref[4,0,1]=(-0.9183984444)*x_ref[0]**o+(0.222613734722)*x_ref[0]+(0.472361095251)*x_ref[1]**o+(-0.182413993184)*x_ref[1]+(0.404737782329)*x_ref[2]**o+(-0.462634159071)*x_ref[2]
        ref[4,1,0]=(-0.577317659134)*x_ref[0]**o+(0.863519222199)*x_ref[0]+(-0.955859899586)*x_ref[1]**o+(0.898032633657)*x_ref[1]+(0.000203504014658)*x_ref[2]**o+(0.84858789073)*x_ref[2]
        ref[4,1,1]=(0.103931145317)*x_ref[0]**o+(-0.491733208237)*x_ref[0]+(-0.679616928853)*x_ref[1]**o+(0.779966695883)*x_ref[1]+(0.19391391729)*x_ref[2]**o+(0.237260204789)*x_ref[2]
        ref[5,0,0]=(-0.587657719833)*x_ref[0]**o+(0.0677059438625)*x_ref[0]+(-0.070159254762)*x_ref[1]**o+(-0.725676771128)*x_ref[1]+(0.563139108018)*x_ref[2]**o+(-0.182136289146)*x_ref[2]
        ref[5,0,1]=(0.647754539947)*x_ref[0]**o+(-0.238569428669)*x_ref[0]+(-0.0352701585821)*x_ref[1]**o+(0.117211931696)*x_ref[1]+(-0.749719337272)*x_ref[2]**o+(-0.0572910818861)*x_ref[2]
        ref[5,1,0]=(0.0802652788129)*x_ref[0]**o+(-0.320772823979)*x_ref[0]+(0.875989601034)*x_ref[1]**o+(0.56803751787)*x_ref[1]+(0.115080146257)*x_ref[2]**o+(0.960937482841)*x_ref[2]
        ref[5,1,1]=(0.374333508365)*x_ref[0]**o+(-0.198321408411)*x_ref[0]+(0.550730384308)*x_ref[1]**o+(0.980522161961)*x_ref[1]+(-0.523593924556)*x_ref[2]**o+(-0.0882497539881)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromSymbol_ContinuousFunction_rank4(self):
      """
      tests interpolate for rank 4 Symbol onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ContinuousFunction onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5, 3, 2),dim=dim)
      s=Data(0,(4, 5, 3, 2),w)
      ref=Data(0,(4, 5, 3, 2),w_ref)
      if dim==2:
        s[0,0,0,0]=(0.222648011315)*x[0]**o+(0.42721508374)*x[0]+(-0.895044476361)*x[1]**o+(-0.42954458773)*x[1]
        s[0,0,0,1]=(0.141549825966)*x[0]**o+(0.28788545297)*x[0]+(0.595616465531)*x[1]**o+(-0.780212218036)*x[1]
        s[0,0,1,0]=(-0.206706340827)*x[0]**o+(-0.133810198949)*x[0]+(0.257781241939)*x[1]**o+(0.597910683263)*x[1]
        s[0,0,1,1]=(-0.121459945992)*x[0]**o+(-0.766623739533)*x[0]+(0.239967670821)*x[1]**o+(-0.388414165885)*x[1]
        s[0,0,2,0]=(0.146250309234)*x[0]**o+(-0.49783173353)*x[0]+(-0.244780515634)*x[1]**o+(-0.314249479259)*x[1]
        s[0,0,2,1]=(-0.270630697677)*x[0]**o+(0.0407254161321)*x[0]+(0.433386393453)*x[1]**o+(0.0801094288602)*x[1]
        s[0,1,0,0]=(0.748843675191)*x[0]**o+(-0.0720203817557)*x[0]+(-0.593596340641)*x[1]**o+(0.347464092831)*x[1]
        s[0,1,0,1]=(-0.395277643253)*x[0]**o+(0.664246198151)*x[0]+(-0.531558963654)*x[1]**o+(-0.809864143072)*x[1]
        s[0,1,1,0]=(0.193097468131)*x[0]**o+(-0.630186168256)*x[0]+(0.0615850267982)*x[1]**o+(-0.570479656519)*x[1]
        s[0,1,1,1]=(-0.368327911525)*x[0]**o+(-0.407914196122)*x[0]+(0.979471770727)*x[1]**o+(0.983952824078)*x[1]
        s[0,1,2,0]=(0.0767864730507)*x[0]**o+(0.114382875063)*x[0]+(-0.147666698894)*x[1]**o+(0.857285661664)*x[1]
        s[0,1,2,1]=(-0.122027577603)*x[0]**o+(-0.590353974558)*x[0]+(-0.667221707908)*x[1]**o+(0.745800225071)*x[1]
        s[0,2,0,0]=(-0.88730273713)*x[0]**o+(0.190743538467)*x[0]+(-0.938661510873)*x[1]**o+(-0.338310116385)*x[1]
        s[0,2,0,1]=(-0.141121464144)*x[0]**o+(0.850394068231)*x[0]+(0.0971484151963)*x[1]**o+(-0.960454373066)*x[1]
        s[0,2,1,0]=(0.824172674079)*x[0]**o+(0.26884586085)*x[0]+(-0.126014056897)*x[1]**o+(0.2784109285)*x[1]
        s[0,2,1,1]=(-0.360983815477)*x[0]**o+(0.714915900407)*x[0]+(0.767809732657)*x[1]**o+(-0.37952152499)*x[1]
        s[0,2,2,0]=(0.161042364518)*x[0]**o+(-0.643042351924)*x[0]+(0.252496991843)*x[1]**o+(-0.979633048076)*x[1]
        s[0,2,2,1]=(0.603846956362)*x[0]**o+(0.700541009601)*x[0]+(-0.63935941149)*x[1]**o+(-0.933149153774)*x[1]
        s[0,3,0,0]=(0.079280423825)*x[0]**o+(-0.716376532932)*x[0]+(-0.502244268586)*x[1]**o+(-0.959402022113)*x[1]
        s[0,3,0,1]=(-0.00511781253292)*x[0]**o+(-0.450787404264)*x[0]+(-0.474024181711)*x[1]**o+(0.456830901009)*x[1]
        s[0,3,1,0]=(0.111583663671)*x[0]**o+(0.102696091218)*x[0]+(0.523533956986)*x[1]**o+(-0.588614638579)*x[1]
        s[0,3,1,1]=(0.58778089432)*x[0]**o+(0.0740732890571)*x[0]+(-0.836011518186)*x[1]**o+(-0.108700168991)*x[1]
        s[0,3,2,0]=(-0.476011451886)*x[0]**o+(-0.773655516146)*x[0]+(-0.682784970237)*x[1]**o+(0.138275892754)*x[1]
        s[0,3,2,1]=(-0.214407387481)*x[0]**o+(-0.418064963679)*x[0]+(0.261699528297)*x[1]**o+(0.0441030472265)*x[1]
        s[0,4,0,0]=(-0.184136692471)*x[0]**o+(0.757765208488)*x[0]+(-0.873091970596)*x[1]**o+(0.350453919065)*x[1]
        s[0,4,0,1]=(-0.885266586543)*x[0]**o+(0.191822948672)*x[0]+(0.333201768343)*x[1]**o+(0.56090642241)*x[1]
        s[0,4,1,0]=(0.22567370118)*x[0]**o+(0.477010002185)*x[0]+(-0.135849171523)*x[1]**o+(-0.399167662154)*x[1]
        s[0,4,1,1]=(-0.830942601229)*x[0]**o+(0.996133387607)*x[0]+(0.518365254824)*x[1]**o+(-0.316479216888)*x[1]
        s[0,4,2,0]=(-0.21777913967)*x[0]**o+(-0.234792212377)*x[0]+(-0.551676836277)*x[1]**o+(-0.594255963868)*x[1]
        s[0,4,2,1]=(0.261945319278)*x[0]**o+(-0.0895552648139)*x[0]+(0.419366366098)*x[1]**o+(0.952305260972)*x[1]
        s[1,0,0,0]=(-0.940686110562)*x[0]**o+(-0.283753475589)*x[0]+(-0.38170291279)*x[1]**o+(0.800953830123)*x[1]
        s[1,0,0,1]=(-0.674895560431)*x[0]**o+(0.246943746198)*x[0]+(0.33525110471)*x[1]**o+(-0.365874383263)*x[1]
        s[1,0,1,0]=(0.00215505252567)*x[0]**o+(-0.859263824986)*x[0]+(-0.509512919301)*x[1]**o+(-0.413848698442)*x[1]
        s[1,0,1,1]=(0.700734957272)*x[0]**o+(0.974253812785)*x[0]+(-0.161730087557)*x[1]**o+(0.163662044371)*x[1]
        s[1,0,2,0]=(-0.961772561059)*x[0]**o+(0.949683155983)*x[0]+(-0.85122085071)*x[1]**o+(0.480287755677)*x[1]
        s[1,0,2,1]=(-0.560545819014)*x[0]**o+(0.574588583212)*x[0]+(-0.0973249975595)*x[1]**o+(0.635962665135)*x[1]
        s[1,1,0,0]=(0.362894470984)*x[0]**o+(-0.550857559086)*x[0]+(0.250693089264)*x[1]**o+(0.620977354637)*x[1]
        s[1,1,0,1]=(0.843905294027)*x[0]**o+(0.533693790759)*x[0]+(-0.29073001293)*x[1]**o+(-0.0166976969594)*x[1]
        s[1,1,1,0]=(-0.710809602785)*x[0]**o+(-0.886308169232)*x[0]+(-0.0578420674111)*x[1]**o+(0.113973788995)*x[1]
        s[1,1,1,1]=(-0.206206827837)*x[0]**o+(0.666323404382)*x[0]+(-0.602962857934)*x[1]**o+(0.0520638745337)*x[1]
        s[1,1,2,0]=(0.0999097614108)*x[0]**o+(-0.813782256512)*x[0]+(0.543364309707)*x[1]**o+(0.526696582979)*x[1]
        s[1,1,2,1]=(0.9692802149)*x[0]**o+(0.822581430545)*x[0]+(-0.42796571194)*x[1]**o+(-0.98187357708)*x[1]
        s[1,2,0,0]=(0.546276944309)*x[0]**o+(-0.374798696965)*x[0]+(-0.531415779779)*x[1]**o+(0.636501153213)*x[1]
        s[1,2,0,1]=(0.591914536156)*x[0]**o+(0.436968785918)*x[0]+(-0.0643795810318)*x[1]**o+(-0.103071630867)*x[1]
        s[1,2,1,0]=(0.0556003374382)*x[0]**o+(0.321314284536)*x[0]+(0.428484115069)*x[1]**o+(-0.788818848309)*x[1]
        s[1,2,1,1]=(0.708201805858)*x[0]**o+(-0.447922785627)*x[0]+(0.720006160805)*x[1]**o+(-0.195158798158)*x[1]
        s[1,2,2,0]=(-0.191158036071)*x[0]**o+(-0.24071498725)*x[0]+(0.0867135968142)*x[1]**o+(-0.381118927393)*x[1]
        s[1,2,2,1]=(0.101200398285)*x[0]**o+(0.319160878245)*x[0]+(0.823111493698)*x[1]**o+(0.373915251689)*x[1]
        s[1,3,0,0]=(0.418534190311)*x[0]**o+(0.38426953402)*x[0]+(0.202103889162)*x[1]**o+(-0.987162870942)*x[1]
        s[1,3,0,1]=(0.506778960729)*x[0]**o+(0.66657084314)*x[0]+(-0.348650547674)*x[1]**o+(-0.175750694648)*x[1]
        s[1,3,1,0]=(-0.723440967066)*x[0]**o+(-0.0953195217426)*x[0]+(0.701375987002)*x[1]**o+(0.789666038518)*x[1]
        s[1,3,1,1]=(-0.298531731302)*x[0]**o+(0.700679845377)*x[0]+(0.940832077343)*x[1]**o+(0.950445857347)*x[1]
        s[1,3,2,0]=(-0.891108074117)*x[0]**o+(0.864924848307)*x[0]+(0.266041420151)*x[1]**o+(0.552064290037)*x[1]
        s[1,3,2,1]=(-0.457816415507)*x[0]**o+(-0.0349980783656)*x[0]+(-0.487164584531)*x[1]**o+(0.781198707595)*x[1]
        s[1,4,0,0]=(-0.165226035876)*x[0]**o+(0.744411614612)*x[0]+(-0.583420396138)*x[1]**o+(0.0705229778455)*x[1]
        s[1,4,0,1]=(0.0891155162281)*x[0]**o+(-0.880960952088)*x[0]+(0.886639784408)*x[1]**o+(0.882570908394)*x[1]
        s[1,4,1,0]=(0.165376342322)*x[0]**o+(0.946226681425)*x[0]+(-0.692718922043)*x[1]**o+(-0.743234216196)*x[1]
        s[1,4,1,1]=(0.710740507812)*x[0]**o+(0.997738475549)*x[0]+(-0.465912186923)*x[1]**o+(-0.00201986786631)*x[1]
        s[1,4,2,0]=(0.374989766617)*x[0]**o+(-0.405268600342)*x[0]+(0.952553167679)*x[1]**o+(-0.0300346590056)*x[1]
        s[1,4,2,1]=(0.596218440945)*x[0]**o+(0.792458254573)*x[0]+(0.277317899686)*x[1]**o+(0.959183907598)*x[1]
        s[2,0,0,0]=(0.755363954311)*x[0]**o+(0.110444956303)*x[0]+(-0.992020571874)*x[1]**o+(-0.0319378081217)*x[1]
        s[2,0,0,1]=(0.955184354029)*x[0]**o+(0.695896419253)*x[0]+(0.36332796725)*x[1]**o+(-0.317618535271)*x[1]
        s[2,0,1,0]=(-0.609579319942)*x[0]**o+(0.964032314592)*x[0]+(-0.997676133302)*x[1]**o+(0.882837207968)*x[1]
        s[2,0,1,1]=(-0.856804288862)*x[0]**o+(-0.28105270207)*x[0]+(-0.630216743999)*x[1]**o+(-0.698401891968)*x[1]
        s[2,0,2,0]=(0.284068678209)*x[0]**o+(-0.207077555458)*x[0]+(0.602851480909)*x[1]**o+(-0.754913042436)*x[1]
        s[2,0,2,1]=(-0.250223707591)*x[0]**o+(0.35320467306)*x[0]+(0.389011278513)*x[1]**o+(0.226530933139)*x[1]
        s[2,1,0,0]=(-0.941260683043)*x[0]**o+(0.873373110513)*x[0]+(-0.766222917713)*x[1]**o+(-0.553221037926)*x[1]
        s[2,1,0,1]=(-0.0826999421931)*x[0]**o+(0.912535931146)*x[0]+(0.757104222181)*x[1]**o+(-0.285277618808)*x[1]
        s[2,1,1,0]=(0.3338465047)*x[0]**o+(0.781271516683)*x[0]+(-0.879533691783)*x[1]**o+(-0.714418961015)*x[1]
        s[2,1,1,1]=(0.149738013539)*x[0]**o+(0.591368145868)*x[0]+(0.0850187448147)*x[1]**o+(-0.197535932669)*x[1]
        s[2,1,2,0]=(0.919954979514)*x[0]**o+(0.910109312902)*x[0]+(-0.066392739623)*x[1]**o+(0.72593933626)*x[1]
        s[2,1,2,1]=(0.132310708599)*x[0]**o+(-0.651825160189)*x[0]+(-0.763887217289)*x[1]**o+(0.693036971149)*x[1]
        s[2,2,0,0]=(0.444176449376)*x[0]**o+(0.0925744377816)*x[0]+(-0.552171800039)*x[1]**o+(-0.0561642176587)*x[1]
        s[2,2,0,1]=(-0.319291518868)*x[0]**o+(-0.937039777249)*x[0]+(0.977602989302)*x[1]**o+(0.232130154058)*x[1]
        s[2,2,1,0]=(-0.0903492311881)*x[0]**o+(0.7908165014)*x[0]+(-0.427663715489)*x[1]**o+(-0.644083387831)*x[1]
        s[2,2,1,1]=(0.866462742024)*x[0]**o+(-0.321161526982)*x[0]+(0.467946139031)*x[1]**o+(0.981155836279)*x[1]
        s[2,2,2,0]=(-0.946044628595)*x[0]**o+(-0.513431569734)*x[0]+(0.205205524474)*x[1]**o+(-0.299199260153)*x[1]
        s[2,2,2,1]=(0.0871796517538)*x[0]**o+(-0.499117781496)*x[0]+(-0.395420669586)*x[1]**o+(-0.740740744789)*x[1]
        s[2,3,0,0]=(-0.663617561023)*x[0]**o+(0.436824463783)*x[0]+(-0.627409472686)*x[1]**o+(-0.68586382338)*x[1]
        s[2,3,0,1]=(-0.100973464748)*x[0]**o+(-0.376217782462)*x[0]+(0.358787883332)*x[1]**o+(0.041205949572)*x[1]
        s[2,3,1,0]=(-0.18138462543)*x[0]**o+(0.212059079029)*x[0]+(0.259368203273)*x[1]**o+(0.0823245655952)*x[1]
        s[2,3,1,1]=(0.143918794807)*x[0]**o+(0.41064856854)*x[0]+(0.591128358341)*x[1]**o+(0.974911718087)*x[1]
        s[2,3,2,0]=(0.215121419628)*x[0]**o+(-0.202266227576)*x[0]+(0.360645018302)*x[1]**o+(-0.530217372293)*x[1]
        s[2,3,2,1]=(-0.645683652927)*x[0]**o+(0.0783849074368)*x[0]+(0.446495488766)*x[1]**o+(-0.510345834163)*x[1]
        s[2,4,0,0]=(-0.889866222706)*x[0]**o+(0.167795911139)*x[0]+(0.342469422898)*x[1]**o+(0.829595836798)*x[1]
        s[2,4,0,1]=(0.149554083927)*x[0]**o+(-0.822909234878)*x[0]+(0.252083851924)*x[1]**o+(-0.316759440069)*x[1]
        s[2,4,1,0]=(0.903989662182)*x[0]**o+(-0.00199692638164)*x[0]+(0.183216868853)*x[1]**o+(0.291683747052)*x[1]
        s[2,4,1,1]=(-0.837481381147)*x[0]**o+(-0.49767333079)*x[0]+(0.379317825858)*x[1]**o+(0.656734345135)*x[1]
        s[2,4,2,0]=(0.575327981246)*x[0]**o+(-0.32472740434)*x[0]+(-0.615235966928)*x[1]**o+(0.859062569875)*x[1]
        s[2,4,2,1]=(0.997096808384)*x[0]**o+(0.176877547637)*x[0]+(0.944330419866)*x[1]**o+(0.09534223881)*x[1]
        s[3,0,0,0]=(-0.0767412809661)*x[0]**o+(0.184419353055)*x[0]+(0.450559868822)*x[1]**o+(-0.482074341734)*x[1]
        s[3,0,0,1]=(0.500618673994)*x[0]**o+(-0.428845431851)*x[0]+(-0.593972292401)*x[1]**o+(-0.590749326466)*x[1]
        s[3,0,1,0]=(-0.274065531705)*x[0]**o+(-0.0749212487474)*x[0]+(0.179752364175)*x[1]**o+(-0.960772511585)*x[1]
        s[3,0,1,1]=(-0.238772579502)*x[0]**o+(0.489035124726)*x[0]+(-0.664791553185)*x[1]**o+(-0.104025806782)*x[1]
        s[3,0,2,0]=(0.982870092816)*x[0]**o+(-0.888195654437)*x[0]+(0.440388652872)*x[1]**o+(0.805316867841)*x[1]
        s[3,0,2,1]=(0.0228989287015)*x[0]**o+(0.148291709924)*x[0]+(-0.686198603954)*x[1]**o+(-0.821746591235)*x[1]
        s[3,1,0,0]=(0.00240454431321)*x[0]**o+(0.61269586693)*x[0]+(0.341226969118)*x[1]**o+(0.124208016094)*x[1]
        s[3,1,0,1]=(0.0701234542733)*x[0]**o+(0.895384612471)*x[0]+(-0.922625286693)*x[1]**o+(-0.752021767621)*x[1]
        s[3,1,1,0]=(0.771977634017)*x[0]**o+(-0.912502253275)*x[0]+(-0.320855123076)*x[1]**o+(-0.521053436714)*x[1]
        s[3,1,1,1]=(0.291967128777)*x[0]**o+(0.0136675345425)*x[0]+(-0.240499641569)*x[1]**o+(0.120459293301)*x[1]
        s[3,1,2,0]=(-0.165796720823)*x[0]**o+(-0.564849404668)*x[0]+(-0.745995713866)*x[1]**o+(0.105168429062)*x[1]
        s[3,1,2,1]=(0.703303894233)*x[0]**o+(0.198015398782)*x[0]+(-0.399667205564)*x[1]**o+(-0.431944087803)*x[1]
        s[3,2,0,0]=(-0.859984329126)*x[0]**o+(-0.1985647639)*x[0]+(0.0785405575628)*x[1]**o+(-0.91703866491)*x[1]
        s[3,2,0,1]=(0.142256934845)*x[0]**o+(-0.9608102834)*x[0]+(-0.573917350955)*x[1]**o+(-0.115513923505)*x[1]
        s[3,2,1,0]=(0.0929359330483)*x[0]**o+(-0.870599213251)*x[0]+(-0.47846799921)*x[1]**o+(-0.560581415535)*x[1]
        s[3,2,1,1]=(0.148783020471)*x[0]**o+(0.735514599027)*x[0]+(0.839043206017)*x[1]**o+(0.89788839722)*x[1]
        s[3,2,2,0]=(0.838672369672)*x[0]**o+(0.508289848942)*x[0]+(-0.726948781664)*x[1]**o+(-0.766210549368)*x[1]
        s[3,2,2,1]=(-0.934406814466)*x[0]**o+(0.782704000359)*x[0]+(0.813704047884)*x[1]**o+(0.810651175197)*x[1]
        s[3,3,0,0]=(0.0808252741173)*x[0]**o+(-0.863997858352)*x[0]+(-0.486533766186)*x[1]**o+(0.696138206258)*x[1]
        s[3,3,0,1]=(0.828675714346)*x[0]**o+(0.533631275087)*x[0]+(0.767869622158)*x[1]**o+(0.127624967621)*x[1]
        s[3,3,1,0]=(0.486100829039)*x[0]**o+(-0.0150390173975)*x[0]+(0.0799449336303)*x[1]**o+(-0.680174222069)*x[1]
        s[3,3,1,1]=(-0.163578245514)*x[0]**o+(0.664275094053)*x[0]+(-0.983443827553)*x[1]**o+(0.0384576043866)*x[1]
        s[3,3,2,0]=(0.098732247123)*x[0]**o+(-0.500859620961)*x[0]+(0.781489169465)*x[1]**o+(-0.593888761671)*x[1]
        s[3,3,2,1]=(0.207503649878)*x[0]**o+(-0.782987281123)*x[0]+(0.401857033567)*x[1]**o+(-0.561255499698)*x[1]
        s[3,4,0,0]=(-0.703304703621)*x[0]**o+(0.900854331554)*x[0]+(-0.660621668968)*x[1]**o+(-0.339993374651)*x[1]
        s[3,4,0,1]=(-0.113871007693)*x[0]**o+(-0.91823246062)*x[0]+(-0.328204536528)*x[1]**o+(0.811186991056)*x[1]
        s[3,4,1,0]=(0.118365746728)*x[0]**o+(0.966348151963)*x[0]+(0.734158908606)*x[1]**o+(-0.793152919357)*x[1]
        s[3,4,1,1]=(-0.865632981044)*x[0]**o+(-0.533667091846)*x[0]+(0.539354750567)*x[1]**o+(-0.436274385593)*x[1]
        s[3,4,2,0]=(0.0308218777522)*x[0]**o+(0.135254149275)*x[0]+(0.541297536585)*x[1]**o+(0.969869993827)*x[1]
        s[3,4,2,1]=(-0.0120021630149)*x[0]**o+(-0.641342077878)*x[0]+(-0.605491166275)*x[1]**o+(-0.170165570072)*x[1]
        ref[0,0,0,0]=(0.222648011315)*x_ref[0]**o+(0.42721508374)*x_ref[0]+(-0.895044476361)*x_ref[1]**o+(-0.42954458773)*x_ref[1]
        ref[0,0,0,1]=(0.141549825966)*x_ref[0]**o+(0.28788545297)*x_ref[0]+(0.595616465531)*x_ref[1]**o+(-0.780212218036)*x_ref[1]
        ref[0,0,1,0]=(-0.206706340827)*x_ref[0]**o+(-0.133810198949)*x_ref[0]+(0.257781241939)*x_ref[1]**o+(0.597910683263)*x_ref[1]
        ref[0,0,1,1]=(-0.121459945992)*x_ref[0]**o+(-0.766623739533)*x_ref[0]+(0.239967670821)*x_ref[1]**o+(-0.388414165885)*x_ref[1]
        ref[0,0,2,0]=(0.146250309234)*x_ref[0]**o+(-0.49783173353)*x_ref[0]+(-0.244780515634)*x_ref[1]**o+(-0.314249479259)*x_ref[1]
        ref[0,0,2,1]=(-0.270630697677)*x_ref[0]**o+(0.0407254161321)*x_ref[0]+(0.433386393453)*x_ref[1]**o+(0.0801094288602)*x_ref[1]
        ref[0,1,0,0]=(0.748843675191)*x_ref[0]**o+(-0.0720203817557)*x_ref[0]+(-0.593596340641)*x_ref[1]**o+(0.347464092831)*x_ref[1]
        ref[0,1,0,1]=(-0.395277643253)*x_ref[0]**o+(0.664246198151)*x_ref[0]+(-0.531558963654)*x_ref[1]**o+(-0.809864143072)*x_ref[1]
        ref[0,1,1,0]=(0.193097468131)*x_ref[0]**o+(-0.630186168256)*x_ref[0]+(0.0615850267982)*x_ref[1]**o+(-0.570479656519)*x_ref[1]
        ref[0,1,1,1]=(-0.368327911525)*x_ref[0]**o+(-0.407914196122)*x_ref[0]+(0.979471770727)*x_ref[1]**o+(0.983952824078)*x_ref[1]
        ref[0,1,2,0]=(0.0767864730507)*x_ref[0]**o+(0.114382875063)*x_ref[0]+(-0.147666698894)*x_ref[1]**o+(0.857285661664)*x_ref[1]
        ref[0,1,2,1]=(-0.122027577603)*x_ref[0]**o+(-0.590353974558)*x_ref[0]+(-0.667221707908)*x_ref[1]**o+(0.745800225071)*x_ref[1]
        ref[0,2,0,0]=(-0.88730273713)*x_ref[0]**o+(0.190743538467)*x_ref[0]+(-0.938661510873)*x_ref[1]**o+(-0.338310116385)*x_ref[1]
        ref[0,2,0,1]=(-0.141121464144)*x_ref[0]**o+(0.850394068231)*x_ref[0]+(0.0971484151963)*x_ref[1]**o+(-0.960454373066)*x_ref[1]
        ref[0,2,1,0]=(0.824172674079)*x_ref[0]**o+(0.26884586085)*x_ref[0]+(-0.126014056897)*x_ref[1]**o+(0.2784109285)*x_ref[1]
        ref[0,2,1,1]=(-0.360983815477)*x_ref[0]**o+(0.714915900407)*x_ref[0]+(0.767809732657)*x_ref[1]**o+(-0.37952152499)*x_ref[1]
        ref[0,2,2,0]=(0.161042364518)*x_ref[0]**o+(-0.643042351924)*x_ref[0]+(0.252496991843)*x_ref[1]**o+(-0.979633048076)*x_ref[1]
        ref[0,2,2,1]=(0.603846956362)*x_ref[0]**o+(0.700541009601)*x_ref[0]+(-0.63935941149)*x_ref[1]**o+(-0.933149153774)*x_ref[1]
        ref[0,3,0,0]=(0.079280423825)*x_ref[0]**o+(-0.716376532932)*x_ref[0]+(-0.502244268586)*x_ref[1]**o+(-0.959402022113)*x_ref[1]
        ref[0,3,0,1]=(-0.00511781253292)*x_ref[0]**o+(-0.450787404264)*x_ref[0]+(-0.474024181711)*x_ref[1]**o+(0.456830901009)*x_ref[1]
        ref[0,3,1,0]=(0.111583663671)*x_ref[0]**o+(0.102696091218)*x_ref[0]+(0.523533956986)*x_ref[1]**o+(-0.588614638579)*x_ref[1]
        ref[0,3,1,1]=(0.58778089432)*x_ref[0]**o+(0.0740732890571)*x_ref[0]+(-0.836011518186)*x_ref[1]**o+(-0.108700168991)*x_ref[1]
        ref[0,3,2,0]=(-0.476011451886)*x_ref[0]**o+(-0.773655516146)*x_ref[0]+(-0.682784970237)*x_ref[1]**o+(0.138275892754)*x_ref[1]
        ref[0,3,2,1]=(-0.214407387481)*x_ref[0]**o+(-0.418064963679)*x_ref[0]+(0.261699528297)*x_ref[1]**o+(0.0441030472265)*x_ref[1]
        ref[0,4,0,0]=(-0.184136692471)*x_ref[0]**o+(0.757765208488)*x_ref[0]+(-0.873091970596)*x_ref[1]**o+(0.350453919065)*x_ref[1]
        ref[0,4,0,1]=(-0.885266586543)*x_ref[0]**o+(0.191822948672)*x_ref[0]+(0.333201768343)*x_ref[1]**o+(0.56090642241)*x_ref[1]
        ref[0,4,1,0]=(0.22567370118)*x_ref[0]**o+(0.477010002185)*x_ref[0]+(-0.135849171523)*x_ref[1]**o+(-0.399167662154)*x_ref[1]
        ref[0,4,1,1]=(-0.830942601229)*x_ref[0]**o+(0.996133387607)*x_ref[0]+(0.518365254824)*x_ref[1]**o+(-0.316479216888)*x_ref[1]
        ref[0,4,2,0]=(-0.21777913967)*x_ref[0]**o+(-0.234792212377)*x_ref[0]+(-0.551676836277)*x_ref[1]**o+(-0.594255963868)*x_ref[1]
        ref[0,4,2,1]=(0.261945319278)*x_ref[0]**o+(-0.0895552648139)*x_ref[0]+(0.419366366098)*x_ref[1]**o+(0.952305260972)*x_ref[1]
        ref[1,0,0,0]=(-0.940686110562)*x_ref[0]**o+(-0.283753475589)*x_ref[0]+(-0.38170291279)*x_ref[1]**o+(0.800953830123)*x_ref[1]
        ref[1,0,0,1]=(-0.674895560431)*x_ref[0]**o+(0.246943746198)*x_ref[0]+(0.33525110471)*x_ref[1]**o+(-0.365874383263)*x_ref[1]
        ref[1,0,1,0]=(0.00215505252567)*x_ref[0]**o+(-0.859263824986)*x_ref[0]+(-0.509512919301)*x_ref[1]**o+(-0.413848698442)*x_ref[1]
        ref[1,0,1,1]=(0.700734957272)*x_ref[0]**o+(0.974253812785)*x_ref[0]+(-0.161730087557)*x_ref[1]**o+(0.163662044371)*x_ref[1]
        ref[1,0,2,0]=(-0.961772561059)*x_ref[0]**o+(0.949683155983)*x_ref[0]+(-0.85122085071)*x_ref[1]**o+(0.480287755677)*x_ref[1]
        ref[1,0,2,1]=(-0.560545819014)*x_ref[0]**o+(0.574588583212)*x_ref[0]+(-0.0973249975595)*x_ref[1]**o+(0.635962665135)*x_ref[1]
        ref[1,1,0,0]=(0.362894470984)*x_ref[0]**o+(-0.550857559086)*x_ref[0]+(0.250693089264)*x_ref[1]**o+(0.620977354637)*x_ref[1]
        ref[1,1,0,1]=(0.843905294027)*x_ref[0]**o+(0.533693790759)*x_ref[0]+(-0.29073001293)*x_ref[1]**o+(-0.0166976969594)*x_ref[1]
        ref[1,1,1,0]=(-0.710809602785)*x_ref[0]**o+(-0.886308169232)*x_ref[0]+(-0.0578420674111)*x_ref[1]**o+(0.113973788995)*x_ref[1]
        ref[1,1,1,1]=(-0.206206827837)*x_ref[0]**o+(0.666323404382)*x_ref[0]+(-0.602962857934)*x_ref[1]**o+(0.0520638745337)*x_ref[1]
        ref[1,1,2,0]=(0.0999097614108)*x_ref[0]**o+(-0.813782256512)*x_ref[0]+(0.543364309707)*x_ref[1]**o+(0.526696582979)*x_ref[1]
        ref[1,1,2,1]=(0.9692802149)*x_ref[0]**o+(0.822581430545)*x_ref[0]+(-0.42796571194)*x_ref[1]**o+(-0.98187357708)*x_ref[1]
        ref[1,2,0,0]=(0.546276944309)*x_ref[0]**o+(-0.374798696965)*x_ref[0]+(-0.531415779779)*x_ref[1]**o+(0.636501153213)*x_ref[1]
        ref[1,2,0,1]=(0.591914536156)*x_ref[0]**o+(0.436968785918)*x_ref[0]+(-0.0643795810318)*x_ref[1]**o+(-0.103071630867)*x_ref[1]
        ref[1,2,1,0]=(0.0556003374382)*x_ref[0]**o+(0.321314284536)*x_ref[0]+(0.428484115069)*x_ref[1]**o+(-0.788818848309)*x_ref[1]
        ref[1,2,1,1]=(0.708201805858)*x_ref[0]**o+(-0.447922785627)*x_ref[0]+(0.720006160805)*x_ref[1]**o+(-0.195158798158)*x_ref[1]
        ref[1,2,2,0]=(-0.191158036071)*x_ref[0]**o+(-0.24071498725)*x_ref[0]+(0.0867135968142)*x_ref[1]**o+(-0.381118927393)*x_ref[1]
        ref[1,2,2,1]=(0.101200398285)*x_ref[0]**o+(0.319160878245)*x_ref[0]+(0.823111493698)*x_ref[1]**o+(0.373915251689)*x_ref[1]
        ref[1,3,0,0]=(0.418534190311)*x_ref[0]**o+(0.38426953402)*x_ref[0]+(0.202103889162)*x_ref[1]**o+(-0.987162870942)*x_ref[1]
        ref[1,3,0,1]=(0.506778960729)*x_ref[0]**o+(0.66657084314)*x_ref[0]+(-0.348650547674)*x_ref[1]**o+(-0.175750694648)*x_ref[1]
        ref[1,3,1,0]=(-0.723440967066)*x_ref[0]**o+(-0.0953195217426)*x_ref[0]+(0.701375987002)*x_ref[1]**o+(0.789666038518)*x_ref[1]
        ref[1,3,1,1]=(-0.298531731302)*x_ref[0]**o+(0.700679845377)*x_ref[0]+(0.940832077343)*x_ref[1]**o+(0.950445857347)*x_ref[1]
        ref[1,3,2,0]=(-0.891108074117)*x_ref[0]**o+(0.864924848307)*x_ref[0]+(0.266041420151)*x_ref[1]**o+(0.552064290037)*x_ref[1]
        ref[1,3,2,1]=(-0.457816415507)*x_ref[0]**o+(-0.0349980783656)*x_ref[0]+(-0.487164584531)*x_ref[1]**o+(0.781198707595)*x_ref[1]
        ref[1,4,0,0]=(-0.165226035876)*x_ref[0]**o+(0.744411614612)*x_ref[0]+(-0.583420396138)*x_ref[1]**o+(0.0705229778455)*x_ref[1]
        ref[1,4,0,1]=(0.0891155162281)*x_ref[0]**o+(-0.880960952088)*x_ref[0]+(0.886639784408)*x_ref[1]**o+(0.882570908394)*x_ref[1]
        ref[1,4,1,0]=(0.165376342322)*x_ref[0]**o+(0.946226681425)*x_ref[0]+(-0.692718922043)*x_ref[1]**o+(-0.743234216196)*x_ref[1]
        ref[1,4,1,1]=(0.710740507812)*x_ref[0]**o+(0.997738475549)*x_ref[0]+(-0.465912186923)*x_ref[1]**o+(-0.00201986786631)*x_ref[1]
        ref[1,4,2,0]=(0.374989766617)*x_ref[0]**o+(-0.405268600342)*x_ref[0]+(0.952553167679)*x_ref[1]**o+(-0.0300346590056)*x_ref[1]
        ref[1,4,2,1]=(0.596218440945)*x_ref[0]**o+(0.792458254573)*x_ref[0]+(0.277317899686)*x_ref[1]**o+(0.959183907598)*x_ref[1]
        ref[2,0,0,0]=(0.755363954311)*x_ref[0]**o+(0.110444956303)*x_ref[0]+(-0.992020571874)*x_ref[1]**o+(-0.0319378081217)*x_ref[1]
        ref[2,0,0,1]=(0.955184354029)*x_ref[0]**o+(0.695896419253)*x_ref[0]+(0.36332796725)*x_ref[1]**o+(-0.317618535271)*x_ref[1]
        ref[2,0,1,0]=(-0.609579319942)*x_ref[0]**o+(0.964032314592)*x_ref[0]+(-0.997676133302)*x_ref[1]**o+(0.882837207968)*x_ref[1]
        ref[2,0,1,1]=(-0.856804288862)*x_ref[0]**o+(-0.28105270207)*x_ref[0]+(-0.630216743999)*x_ref[1]**o+(-0.698401891968)*x_ref[1]
        ref[2,0,2,0]=(0.284068678209)*x_ref[0]**o+(-0.207077555458)*x_ref[0]+(0.602851480909)*x_ref[1]**o+(-0.754913042436)*x_ref[1]
        ref[2,0,2,1]=(-0.250223707591)*x_ref[0]**o+(0.35320467306)*x_ref[0]+(0.389011278513)*x_ref[1]**o+(0.226530933139)*x_ref[1]
        ref[2,1,0,0]=(-0.941260683043)*x_ref[0]**o+(0.873373110513)*x_ref[0]+(-0.766222917713)*x_ref[1]**o+(-0.553221037926)*x_ref[1]
        ref[2,1,0,1]=(-0.0826999421931)*x_ref[0]**o+(0.912535931146)*x_ref[0]+(0.757104222181)*x_ref[1]**o+(-0.285277618808)*x_ref[1]
        ref[2,1,1,0]=(0.3338465047)*x_ref[0]**o+(0.781271516683)*x_ref[0]+(-0.879533691783)*x_ref[1]**o+(-0.714418961015)*x_ref[1]
        ref[2,1,1,1]=(0.149738013539)*x_ref[0]**o+(0.591368145868)*x_ref[0]+(0.0850187448147)*x_ref[1]**o+(-0.197535932669)*x_ref[1]
        ref[2,1,2,0]=(0.919954979514)*x_ref[0]**o+(0.910109312902)*x_ref[0]+(-0.066392739623)*x_ref[1]**o+(0.72593933626)*x_ref[1]
        ref[2,1,2,1]=(0.132310708599)*x_ref[0]**o+(-0.651825160189)*x_ref[0]+(-0.763887217289)*x_ref[1]**o+(0.693036971149)*x_ref[1]
        ref[2,2,0,0]=(0.444176449376)*x_ref[0]**o+(0.0925744377816)*x_ref[0]+(-0.552171800039)*x_ref[1]**o+(-0.0561642176587)*x_ref[1]
        ref[2,2,0,1]=(-0.319291518868)*x_ref[0]**o+(-0.937039777249)*x_ref[0]+(0.977602989302)*x_ref[1]**o+(0.232130154058)*x_ref[1]
        ref[2,2,1,0]=(-0.0903492311881)*x_ref[0]**o+(0.7908165014)*x_ref[0]+(-0.427663715489)*x_ref[1]**o+(-0.644083387831)*x_ref[1]
        ref[2,2,1,1]=(0.866462742024)*x_ref[0]**o+(-0.321161526982)*x_ref[0]+(0.467946139031)*x_ref[1]**o+(0.981155836279)*x_ref[1]
        ref[2,2,2,0]=(-0.946044628595)*x_ref[0]**o+(-0.513431569734)*x_ref[0]+(0.205205524474)*x_ref[1]**o+(-0.299199260153)*x_ref[1]
        ref[2,2,2,1]=(0.0871796517538)*x_ref[0]**o+(-0.499117781496)*x_ref[0]+(-0.395420669586)*x_ref[1]**o+(-0.740740744789)*x_ref[1]
        ref[2,3,0,0]=(-0.663617561023)*x_ref[0]**o+(0.436824463783)*x_ref[0]+(-0.627409472686)*x_ref[1]**o+(-0.68586382338)*x_ref[1]
        ref[2,3,0,1]=(-0.100973464748)*x_ref[0]**o+(-0.376217782462)*x_ref[0]+(0.358787883332)*x_ref[1]**o+(0.041205949572)*x_ref[1]
        ref[2,3,1,0]=(-0.18138462543)*x_ref[0]**o+(0.212059079029)*x_ref[0]+(0.259368203273)*x_ref[1]**o+(0.0823245655952)*x_ref[1]
        ref[2,3,1,1]=(0.143918794807)*x_ref[0]**o+(0.41064856854)*x_ref[0]+(0.591128358341)*x_ref[1]**o+(0.974911718087)*x_ref[1]
        ref[2,3,2,0]=(0.215121419628)*x_ref[0]**o+(-0.202266227576)*x_ref[0]+(0.360645018302)*x_ref[1]**o+(-0.530217372293)*x_ref[1]
        ref[2,3,2,1]=(-0.645683652927)*x_ref[0]**o+(0.0783849074368)*x_ref[0]+(0.446495488766)*x_ref[1]**o+(-0.510345834163)*x_ref[1]
        ref[2,4,0,0]=(-0.889866222706)*x_ref[0]**o+(0.167795911139)*x_ref[0]+(0.342469422898)*x_ref[1]**o+(0.829595836798)*x_ref[1]
        ref[2,4,0,1]=(0.149554083927)*x_ref[0]**o+(-0.822909234878)*x_ref[0]+(0.252083851924)*x_ref[1]**o+(-0.316759440069)*x_ref[1]
        ref[2,4,1,0]=(0.903989662182)*x_ref[0]**o+(-0.00199692638164)*x_ref[0]+(0.183216868853)*x_ref[1]**o+(0.291683747052)*x_ref[1]
        ref[2,4,1,1]=(-0.837481381147)*x_ref[0]**o+(-0.49767333079)*x_ref[0]+(0.379317825858)*x_ref[1]**o+(0.656734345135)*x_ref[1]
        ref[2,4,2,0]=(0.575327981246)*x_ref[0]**o+(-0.32472740434)*x_ref[0]+(-0.615235966928)*x_ref[1]**o+(0.859062569875)*x_ref[1]
        ref[2,4,2,1]=(0.997096808384)*x_ref[0]**o+(0.176877547637)*x_ref[0]+(0.944330419866)*x_ref[1]**o+(0.09534223881)*x_ref[1]
        ref[3,0,0,0]=(-0.0767412809661)*x_ref[0]**o+(0.184419353055)*x_ref[0]+(0.450559868822)*x_ref[1]**o+(-0.482074341734)*x_ref[1]
        ref[3,0,0,1]=(0.500618673994)*x_ref[0]**o+(-0.428845431851)*x_ref[0]+(-0.593972292401)*x_ref[1]**o+(-0.590749326466)*x_ref[1]
        ref[3,0,1,0]=(-0.274065531705)*x_ref[0]**o+(-0.0749212487474)*x_ref[0]+(0.179752364175)*x_ref[1]**o+(-0.960772511585)*x_ref[1]
        ref[3,0,1,1]=(-0.238772579502)*x_ref[0]**o+(0.489035124726)*x_ref[0]+(-0.664791553185)*x_ref[1]**o+(-0.104025806782)*x_ref[1]
        ref[3,0,2,0]=(0.982870092816)*x_ref[0]**o+(-0.888195654437)*x_ref[0]+(0.440388652872)*x_ref[1]**o+(0.805316867841)*x_ref[1]
        ref[3,0,2,1]=(0.0228989287015)*x_ref[0]**o+(0.148291709924)*x_ref[0]+(-0.686198603954)*x_ref[1]**o+(-0.821746591235)*x_ref[1]
        ref[3,1,0,0]=(0.00240454431321)*x_ref[0]**o+(0.61269586693)*x_ref[0]+(0.341226969118)*x_ref[1]**o+(0.124208016094)*x_ref[1]
        ref[3,1,0,1]=(0.0701234542733)*x_ref[0]**o+(0.895384612471)*x_ref[0]+(-0.922625286693)*x_ref[1]**o+(-0.752021767621)*x_ref[1]
        ref[3,1,1,0]=(0.771977634017)*x_ref[0]**o+(-0.912502253275)*x_ref[0]+(-0.320855123076)*x_ref[1]**o+(-0.521053436714)*x_ref[1]
        ref[3,1,1,1]=(0.291967128777)*x_ref[0]**o+(0.0136675345425)*x_ref[0]+(-0.240499641569)*x_ref[1]**o+(0.120459293301)*x_ref[1]
        ref[3,1,2,0]=(-0.165796720823)*x_ref[0]**o+(-0.564849404668)*x_ref[0]+(-0.745995713866)*x_ref[1]**o+(0.105168429062)*x_ref[1]
        ref[3,1,2,1]=(0.703303894233)*x_ref[0]**o+(0.198015398782)*x_ref[0]+(-0.399667205564)*x_ref[1]**o+(-0.431944087803)*x_ref[1]
        ref[3,2,0,0]=(-0.859984329126)*x_ref[0]**o+(-0.1985647639)*x_ref[0]+(0.0785405575628)*x_ref[1]**o+(-0.91703866491)*x_ref[1]
        ref[3,2,0,1]=(0.142256934845)*x_ref[0]**o+(-0.9608102834)*x_ref[0]+(-0.573917350955)*x_ref[1]**o+(-0.115513923505)*x_ref[1]
        ref[3,2,1,0]=(0.0929359330483)*x_ref[0]**o+(-0.870599213251)*x_ref[0]+(-0.47846799921)*x_ref[1]**o+(-0.560581415535)*x_ref[1]
        ref[3,2,1,1]=(0.148783020471)*x_ref[0]**o+(0.735514599027)*x_ref[0]+(0.839043206017)*x_ref[1]**o+(0.89788839722)*x_ref[1]
        ref[3,2,2,0]=(0.838672369672)*x_ref[0]**o+(0.508289848942)*x_ref[0]+(-0.726948781664)*x_ref[1]**o+(-0.766210549368)*x_ref[1]
        ref[3,2,2,1]=(-0.934406814466)*x_ref[0]**o+(0.782704000359)*x_ref[0]+(0.813704047884)*x_ref[1]**o+(0.810651175197)*x_ref[1]
        ref[3,3,0,0]=(0.0808252741173)*x_ref[0]**o+(-0.863997858352)*x_ref[0]+(-0.486533766186)*x_ref[1]**o+(0.696138206258)*x_ref[1]
        ref[3,3,0,1]=(0.828675714346)*x_ref[0]**o+(0.533631275087)*x_ref[0]+(0.767869622158)*x_ref[1]**o+(0.127624967621)*x_ref[1]
        ref[3,3,1,0]=(0.486100829039)*x_ref[0]**o+(-0.0150390173975)*x_ref[0]+(0.0799449336303)*x_ref[1]**o+(-0.680174222069)*x_ref[1]
        ref[3,3,1,1]=(-0.163578245514)*x_ref[0]**o+(0.664275094053)*x_ref[0]+(-0.983443827553)*x_ref[1]**o+(0.0384576043866)*x_ref[1]
        ref[3,3,2,0]=(0.098732247123)*x_ref[0]**o+(-0.500859620961)*x_ref[0]+(0.781489169465)*x_ref[1]**o+(-0.593888761671)*x_ref[1]
        ref[3,3,2,1]=(0.207503649878)*x_ref[0]**o+(-0.782987281123)*x_ref[0]+(0.401857033567)*x_ref[1]**o+(-0.561255499698)*x_ref[1]
        ref[3,4,0,0]=(-0.703304703621)*x_ref[0]**o+(0.900854331554)*x_ref[0]+(-0.660621668968)*x_ref[1]**o+(-0.339993374651)*x_ref[1]
        ref[3,4,0,1]=(-0.113871007693)*x_ref[0]**o+(-0.91823246062)*x_ref[0]+(-0.328204536528)*x_ref[1]**o+(0.811186991056)*x_ref[1]
        ref[3,4,1,0]=(0.118365746728)*x_ref[0]**o+(0.966348151963)*x_ref[0]+(0.734158908606)*x_ref[1]**o+(-0.793152919357)*x_ref[1]
        ref[3,4,1,1]=(-0.865632981044)*x_ref[0]**o+(-0.533667091846)*x_ref[0]+(0.539354750567)*x_ref[1]**o+(-0.436274385593)*x_ref[1]
        ref[3,4,2,0]=(0.0308218777522)*x_ref[0]**o+(0.135254149275)*x_ref[0]+(0.541297536585)*x_ref[1]**o+(0.969869993827)*x_ref[1]
        ref[3,4,2,1]=(-0.0120021630149)*x_ref[0]**o+(-0.641342077878)*x_ref[0]+(-0.605491166275)*x_ref[1]**o+(-0.170165570072)*x_ref[1]
      else:
        s[0,0,0,0]=(-0.645055264384)*x[0]**o+(-0.193054732885)*x[0]+(0.0857819007815)*x[1]**o+(-0.461528553026)*x[1]+(-0.57134367637)*x[2]**o+(-0.74286427921)*x[2]
        s[0,0,0,1]=(0.487034525946)*x[0]**o+(-0.714187726361)*x[0]+(-0.606957760182)*x[1]**o+(0.0779147207958)*x[1]+(-0.29526614633)*x[2]**o+(-0.458434997173)*x[2]
        s[0,0,1,0]=(0.156747343036)*x[0]**o+(-0.341557748989)*x[0]+(0.158761045238)*x[1]**o+(-0.801472587987)*x[1]+(-0.271446861129)*x[2]**o+(0.900123872896)*x[2]
        s[0,0,1,1]=(0.573494874527)*x[0]**o+(-0.0243920948082)*x[0]+(0.0948894747825)*x[1]**o+(0.65093963776)*x[1]+(0.671075295673)*x[2]**o+(-0.864463024946)*x[2]
        s[0,0,2,0]=(0.725244561373)*x[0]**o+(-0.996337933154)*x[0]+(0.767557173446)*x[1]**o+(0.949472898375)*x[1]+(0.34694891892)*x[2]**o+(-0.0275257881948)*x[2]
        s[0,0,2,1]=(0.172147320675)*x[0]**o+(-0.308830513267)*x[0]+(-0.186533796188)*x[1]**o+(-0.253183031716)*x[1]+(-0.0558420387348)*x[2]**o+(0.639581162188)*x[2]
        s[0,1,0,0]=(-0.201128976272)*x[0]**o+(-0.0604529403788)*x[0]+(0.687516667804)*x[1]**o+(0.0463366971989)*x[1]+(-0.512099164154)*x[2]**o+(-0.562496472369)*x[2]
        s[0,1,0,1]=(-0.579867899667)*x[0]**o+(0.481671792856)*x[0]+(-0.534848750787)*x[1]**o+(-0.681855536562)*x[1]+(-0.138906106494)*x[2]**o+(0.85816397707)*x[2]
        s[0,1,1,0]=(0.361813239327)*x[0]**o+(-0.371707128692)*x[0]+(0.1413931258)*x[1]**o+(-0.222686191434)*x[1]+(0.429860839989)*x[2]**o+(-0.854238845295)*x[2]
        s[0,1,1,1]=(0.777481754216)*x[0]**o+(0.559920012089)*x[0]+(-0.708494735928)*x[1]**o+(-0.340573378074)*x[1]+(-0.658897461649)*x[2]**o+(0.0686343535795)*x[2]
        s[0,1,2,0]=(0.548617122525)*x[0]**o+(0.142683302153)*x[0]+(0.599377799663)*x[1]**o+(-0.0941501509169)*x[1]+(0.899613162974)*x[2]**o+(0.437942658389)*x[2]
        s[0,1,2,1]=(0.055538446966)*x[0]**o+(0.213360227501)*x[0]+(-0.929499951666)*x[1]**o+(0.336982626422)*x[1]+(-0.656888335372)*x[2]**o+(0.407882483181)*x[2]
        s[0,2,0,0]=(-0.874997203733)*x[0]**o+(0.759216265112)*x[0]+(0.948642776931)*x[1]**o+(-0.242980793949)*x[1]+(0.677616559342)*x[2]**o+(0.341859949626)*x[2]
        s[0,2,0,1]=(-0.181175585757)*x[0]**o+(0.335483796366)*x[0]+(-0.664259907913)*x[1]**o+(0.906747760538)*x[1]+(-0.984393889605)*x[2]**o+(-0.812569900067)*x[2]
        s[0,2,1,0]=(0.851036684294)*x[0]**o+(-0.751387632375)*x[0]+(0.724388989617)*x[1]**o+(0.384230212684)*x[1]+(-0.449333509528)*x[2]**o+(0.784460454553)*x[2]
        s[0,2,1,1]=(-0.609413988379)*x[0]**o+(-0.493285779523)*x[0]+(-0.590691722184)*x[1]**o+(-0.607626442728)*x[1]+(0.0625828744206)*x[2]**o+(-0.15945730222)*x[2]
        s[0,2,2,0]=(0.766872578253)*x[0]**o+(-0.590526688805)*x[0]+(0.222323591845)*x[1]**o+(0.512451311011)*x[1]+(-0.160540733556)*x[2]**o+(0.5770077018)*x[2]
        s[0,2,2,1]=(-0.729525005374)*x[0]**o+(0.986018027885)*x[0]+(0.25197251588)*x[1]**o+(0.0450480112291)*x[1]+(-0.869546576401)*x[2]**o+(-0.290965290154)*x[2]
        s[0,3,0,0]=(-0.184322684185)*x[0]**o+(0.751826009862)*x[0]+(-0.697266959303)*x[1]**o+(0.924909596435)*x[1]+(-0.638157491307)*x[2]**o+(0.236651962157)*x[2]
        s[0,3,0,1]=(0.706826305243)*x[0]**o+(-0.946439184553)*x[0]+(0.168150324173)*x[1]**o+(-0.792980290572)*x[1]+(0.61272398393)*x[2]**o+(0.961138901449)*x[2]
        s[0,3,1,0]=(0.933288270936)*x[0]**o+(-0.55682855224)*x[0]+(0.883604318263)*x[1]**o+(0.816211062971)*x[1]+(0.260246780311)*x[2]**o+(0.905516381287)*x[2]
        s[0,3,1,1]=(0.404420727507)*x[0]**o+(0.471967111659)*x[0]+(0.980687681241)*x[1]**o+(-0.0887974405176)*x[1]+(0.0934146142552)*x[2]**o+(0.646580252159)*x[2]
        s[0,3,2,0]=(-0.921777598275)*x[0]**o+(0.499527698309)*x[0]+(0.664601445292)*x[1]**o+(-0.851079448204)*x[1]+(-0.769256802656)*x[2]**o+(0.730819279497)*x[2]
        s[0,3,2,1]=(-0.245507455509)*x[0]**o+(0.357764745233)*x[0]+(0.88892212086)*x[1]**o+(-0.819843178794)*x[1]+(-0.203314043822)*x[2]**o+(0.224643592554)*x[2]
        s[0,4,0,0]=(0.343174611172)*x[0]**o+(-0.701162898726)*x[0]+(-0.447921000153)*x[1]**o+(0.206599993047)*x[1]+(-0.461911660893)*x[2]**o+(-0.757271316865)*x[2]
        s[0,4,0,1]=(-0.55800009078)*x[0]**o+(-0.39370288163)*x[0]+(-0.35379408894)*x[1]**o+(-0.214762670331)*x[1]+(0.725231115203)*x[2]**o+(0.765815232027)*x[2]
        s[0,4,1,0]=(0.440761988298)*x[0]**o+(-0.798551732493)*x[0]+(-0.955319652492)*x[1]**o+(-0.50319720075)*x[1]+(-0.213217943103)*x[2]**o+(-0.882735934069)*x[2]
        s[0,4,1,1]=(0.441117080069)*x[0]**o+(-0.748338538161)*x[0]+(0.956112036119)*x[1]**o+(-0.960896721051)*x[1]+(-0.291350828791)*x[2]**o+(-0.488896289566)*x[2]
        s[0,4,2,0]=(-0.855035257555)*x[0]**o+(-0.579064017558)*x[0]+(-0.501257230663)*x[1]**o+(-0.956216916996)*x[1]+(-0.860898494221)*x[2]**o+(-0.830087013431)*x[2]
        s[0,4,2,1]=(-0.868978503525)*x[0]**o+(0.162013861935)*x[0]+(-0.311759384766)*x[1]**o+(0.399068885633)*x[1]+(-0.569888913523)*x[2]**o+(-0.394173563136)*x[2]
        s[1,0,0,0]=(0.215242187817)*x[0]**o+(-0.215197471124)*x[0]+(0.779932221493)*x[1]**o+(-0.365740975053)*x[1]+(0.601368178784)*x[2]**o+(-0.170706718888)*x[2]
        s[1,0,0,1]=(0.862102162389)*x[0]**o+(-0.815403119428)*x[0]+(0.269334729254)*x[1]**o+(-0.511457037588)*x[1]+(0.00982312570019)*x[2]**o+(-0.407471705546)*x[2]
        s[1,0,1,0]=(-0.728800252524)*x[0]**o+(-0.178562348078)*x[0]+(0.711141895031)*x[1]**o+(-0.0037290743948)*x[1]+(-0.353455013195)*x[2]**o+(-0.0812715065083)*x[2]
        s[1,0,1,1]=(-0.689013409151)*x[0]**o+(-0.467030398964)*x[0]+(0.295472079898)*x[1]**o+(0.803549469534)*x[1]+(-0.360809851719)*x[2]**o+(-0.563821123318)*x[2]
        s[1,0,2,0]=(0.238079356258)*x[0]**o+(-0.385019487483)*x[0]+(0.864174173897)*x[1]**o+(-0.338065985154)*x[1]+(-0.516837345006)*x[2]**o+(0.611410140553)*x[2]
        s[1,0,2,1]=(0.937340721244)*x[0]**o+(-0.0936947975478)*x[0]+(0.313991879475)*x[1]**o+(-0.126811107707)*x[1]+(-0.627872015529)*x[2]**o+(-0.981891626964)*x[2]
        s[1,1,0,0]=(-0.0675382298198)*x[0]**o+(-0.783093574363)*x[0]+(-0.377941739502)*x[1]**o+(-0.0833668456557)*x[1]+(-0.157483560616)*x[2]**o+(0.686957855465)*x[2]
        s[1,1,0,1]=(0.151028535239)*x[0]**o+(0.992218304228)*x[0]+(-0.686192558465)*x[1]**o+(-0.0130549001993)*x[1]+(0.578982765961)*x[2]**o+(0.814271616163)*x[2]
        s[1,1,1,0]=(0.384343377969)*x[0]**o+(0.463218368911)*x[0]+(0.35576368147)*x[1]**o+(-0.834867049786)*x[1]+(-0.274552267393)*x[2]**o+(0.623371271421)*x[2]
        s[1,1,1,1]=(-0.61717606851)*x[0]**o+(-0.578562595163)*x[0]+(0.0518387545717)*x[1]**o+(-0.30815303855)*x[1]+(0.556738906305)*x[2]**o+(0.38164058851)*x[2]
        s[1,1,2,0]=(0.52042073171)*x[0]**o+(0.698894225253)*x[0]+(0.120169831848)*x[1]**o+(0.238450852883)*x[1]+(-0.552966574246)*x[2]**o+(0.661290976968)*x[2]
        s[1,1,2,1]=(-0.428668260768)*x[0]**o+(0.568751515596)*x[0]+(-0.0768903249884)*x[1]**o+(0.768814424639)*x[1]+(0.424759560455)*x[2]**o+(-0.690938863922)*x[2]
        s[1,2,0,0]=(-0.5443917305)*x[0]**o+(-0.0595999390634)*x[0]+(0.901507044764)*x[1]**o+(-0.681191541382)*x[1]+(-0.123122907072)*x[2]**o+(-0.152869512816)*x[2]
        s[1,2,0,1]=(-0.0145795630242)*x[0]**o+(-0.53313358583)*x[0]+(0.629028937665)*x[1]**o+(-0.539546800632)*x[1]+(-0.252160197397)*x[2]**o+(0.0133469506662)*x[2]
        s[1,2,1,0]=(0.00594869891786)*x[0]**o+(-0.56813890169)*x[0]+(-0.685679553166)*x[1]**o+(0.81727698867)*x[1]+(-0.895427226202)*x[2]**o+(0.140096007363)*x[2]
        s[1,2,1,1]=(-0.770028732259)*x[0]**o+(0.827053138879)*x[0]+(-0.917989309552)*x[1]**o+(-0.0517956582083)*x[1]+(-0.266844746028)*x[2]**o+(-0.99294686081)*x[2]
        s[1,2,2,0]=(0.0529054005828)*x[0]**o+(-0.807219927076)*x[0]+(-0.296037864436)*x[1]**o+(-0.617547075266)*x[1]+(-0.966016304179)*x[2]**o+(-0.168793187561)*x[2]
        s[1,2,2,1]=(0.868690089119)*x[0]**o+(-0.610624824567)*x[0]+(-0.411113678451)*x[1]**o+(0.103726193007)*x[1]+(-0.148892296878)*x[2]**o+(-0.195079797362)*x[2]
        s[1,3,0,0]=(0.248806387929)*x[0]**o+(-0.683762743914)*x[0]+(0.460928199336)*x[1]**o+(0.338430932923)*x[1]+(0.589813218945)*x[2]**o+(-0.632423181868)*x[2]
        s[1,3,0,1]=(-0.338476160834)*x[0]**o+(0.491753392304)*x[0]+(-0.590351153797)*x[1]**o+(-0.448026649636)*x[1]+(-0.806930505259)*x[2]**o+(-0.262565462369)*x[2]
        s[1,3,1,0]=(0.808414907549)*x[0]**o+(0.156426917662)*x[0]+(0.33216874106)*x[1]**o+(-0.714791665494)*x[1]+(-0.163168903301)*x[2]**o+(-0.0423061478551)*x[2]
        s[1,3,1,1]=(0.72498761831)*x[0]**o+(0.08194542863)*x[0]+(-0.566316087403)*x[1]**o+(0.49486476938)*x[1]+(0.744201752122)*x[2]**o+(0.766413325553)*x[2]
        s[1,3,2,0]=(0.793853329178)*x[0]**o+(-0.883754372376)*x[0]+(-0.991903018183)*x[1]**o+(0.279576133295)*x[1]+(-0.0364449658341)*x[2]**o+(-0.85024249755)*x[2]
        s[1,3,2,1]=(0.308927107876)*x[0]**o+(-0.995603136748)*x[0]+(-0.15217214087)*x[1]**o+(-0.554310725129)*x[1]+(0.69107000546)*x[2]**o+(0.353155450955)*x[2]
        s[1,4,0,0]=(0.696506340076)*x[0]**o+(0.889728425166)*x[0]+(-0.858268204558)*x[1]**o+(0.353251100246)*x[1]+(-0.974872511432)*x[2]**o+(-0.878064243518)*x[2]
        s[1,4,0,1]=(-0.232678527901)*x[0]**o+(-0.60738965282)*x[0]+(-0.629841924447)*x[1]**o+(0.0731692547228)*x[1]+(-0.457280478523)*x[2]**o+(-0.309371862287)*x[2]
        s[1,4,1,0]=(-0.0254189247267)*x[0]**o+(-0.160538018478)*x[0]+(-0.757800049379)*x[1]**o+(0.108707596517)*x[1]+(-0.182095912638)*x[2]**o+(0.565356253873)*x[2]
        s[1,4,1,1]=(0.353936491579)*x[0]**o+(0.368146351094)*x[0]+(-0.0271653916784)*x[1]**o+(0.7863197856)*x[1]+(-0.0642828131827)*x[2]**o+(0.946295880643)*x[2]
        s[1,4,2,0]=(-0.151347070208)*x[0]**o+(-0.490039530717)*x[0]+(-0.222861393521)*x[1]**o+(0.160218722103)*x[1]+(-0.881185613005)*x[2]**o+(0.453889724466)*x[2]
        s[1,4,2,1]=(0.0185253314435)*x[0]**o+(0.824691270585)*x[0]+(-0.175025465282)*x[1]**o+(-0.946563236595)*x[1]+(0.915012725055)*x[2]**o+(-0.14949108932)*x[2]
        s[2,0,0,0]=(0.608065273397)*x[0]**o+(0.103537849738)*x[0]+(0.640722456034)*x[1]**o+(0.707059952842)*x[1]+(0.958670726841)*x[2]**o+(0.178333168266)*x[2]
        s[2,0,0,1]=(0.762096192615)*x[0]**o+(-0.91311774327)*x[0]+(-0.636481030154)*x[1]**o+(0.0485431700913)*x[1]+(-0.76456752608)*x[2]**o+(0.988366648174)*x[2]
        s[2,0,1,0]=(-0.997545907902)*x[0]**o+(0.0386694282297)*x[0]+(-0.95454977224)*x[1]**o+(-0.472500001853)*x[1]+(-0.315668831265)*x[2]**o+(-0.0911950672323)*x[2]
        s[2,0,1,1]=(0.758639431658)*x[0]**o+(-0.35267267476)*x[0]+(0.733519690739)*x[1]**o+(0.687429543066)*x[1]+(0.748932363975)*x[2]**o+(-0.360330977046)*x[2]
        s[2,0,2,0]=(0.748882354686)*x[0]**o+(0.398195068825)*x[0]+(0.268707244918)*x[1]**o+(-0.906848889273)*x[1]+(0.8203042296)*x[2]**o+(-0.388209014017)*x[2]
        s[2,0,2,1]=(-0.622158650545)*x[0]**o+(0.757162985883)*x[0]+(0.213276305935)*x[1]**o+(0.376995936453)*x[1]+(-0.919851919774)*x[2]**o+(-0.341006967087)*x[2]
        s[2,1,0,0]=(0.45283405067)*x[0]**o+(-0.456183034235)*x[0]+(-0.211030502465)*x[1]**o+(0.874532774776)*x[1]+(-0.245927412434)*x[2]**o+(0.426470240035)*x[2]
        s[2,1,0,1]=(-0.167484852552)*x[0]**o+(0.23618650325)*x[0]+(-0.256431628436)*x[1]**o+(-0.758314815145)*x[1]+(-0.553126534983)*x[2]**o+(-0.993390924398)*x[2]
        s[2,1,1,0]=(-0.579896706714)*x[0]**o+(0.0247821970864)*x[0]+(-0.485603356124)*x[1]**o+(0.610550482437)*x[1]+(-0.788558005559)*x[2]**o+(-0.716855983085)*x[2]
        s[2,1,1,1]=(0.011213932334)*x[0]**o+(-0.606541681102)*x[0]+(-0.783918973118)*x[1]**o+(-0.129337676239)*x[1]+(-0.681878469428)*x[2]**o+(-0.818559911243)*x[2]
        s[2,1,2,0]=(0.806311020506)*x[0]**o+(0.348481209449)*x[0]+(0.147356878845)*x[1]**o+(0.481921509802)*x[1]+(0.445429036477)*x[2]**o+(0.113745424091)*x[2]
        s[2,1,2,1]=(-0.587849278852)*x[0]**o+(-0.141739080759)*x[0]+(-0.0431697185243)*x[1]**o+(0.845205068629)*x[1]+(0.485473344628)*x[2]**o+(0.416700333568)*x[2]
        s[2,2,0,0]=(-0.36580939154)*x[0]**o+(-0.826777569794)*x[0]+(-0.322507767896)*x[1]**o+(0.697307401686)*x[1]+(0.52326834016)*x[2]**o+(-0.31919043817)*x[2]
        s[2,2,0,1]=(-0.130664405569)*x[0]**o+(0.823599150158)*x[0]+(0.751486085263)*x[1]**o+(0.150119789292)*x[1]+(-0.307393365311)*x[2]**o+(0.919517717122)*x[2]
        s[2,2,1,0]=(0.254288128084)*x[0]**o+(0.580902444535)*x[0]+(0.90113516563)*x[1]**o+(0.805502401301)*x[1]+(-0.483775735536)*x[2]**o+(-0.846660440246)*x[2]
        s[2,2,1,1]=(-0.481100104989)*x[0]**o+(0.459159658041)*x[0]+(-0.886023770838)*x[1]**o+(0.693304415006)*x[1]+(-0.490424960617)*x[2]**o+(0.618464834808)*x[2]
        s[2,2,2,0]=(0.851037708736)*x[0]**o+(0.973238080753)*x[0]+(0.617606926402)*x[1]**o+(-0.0129774413768)*x[1]+(0.342094147972)*x[2]**o+(-0.698988316685)*x[2]
        s[2,2,2,1]=(-0.571806691565)*x[0]**o+(-0.488244261444)*x[0]+(-0.408779266509)*x[1]**o+(-0.688083559849)*x[1]+(-0.668712586564)*x[2]**o+(0.811908851853)*x[2]
        s[2,3,0,0]=(0.305354422638)*x[0]**o+(-0.347372904868)*x[0]+(0.147767334473)*x[1]**o+(0.597885123439)*x[1]+(-0.532183927826)*x[2]**o+(0.778189138615)*x[2]
        s[2,3,0,1]=(0.986328831124)*x[0]**o+(0.237847460579)*x[0]+(0.350775306824)*x[1]**o+(-0.750995843201)*x[1]+(0.85575809431)*x[2]**o+(0.624236482292)*x[2]
        s[2,3,1,0]=(-0.418600918765)*x[0]**o+(-0.958489157688)*x[0]+(-0.524398996544)*x[1]**o+(0.186795580218)*x[1]+(-0.364079386452)*x[2]**o+(-0.330796017113)*x[2]
        s[2,3,1,1]=(0.632410928458)*x[0]**o+(-0.557450605906)*x[0]+(-0.699741830058)*x[1]**o+(0.867693032849)*x[1]+(-0.770879368311)*x[2]**o+(0.972808988148)*x[2]
        s[2,3,2,0]=(0.283379195721)*x[0]**o+(-0.636902680473)*x[0]+(0.609885794066)*x[1]**o+(0.831554040093)*x[1]+(-0.330474761682)*x[2]**o+(0.000310556070561)*x[2]
        s[2,3,2,1]=(0.363575347708)*x[0]**o+(-0.327653113631)*x[0]+(-0.842370028303)*x[1]**o+(0.115573924346)*x[1]+(0.0903633504982)*x[2]**o+(-0.619568111717)*x[2]
        s[2,4,0,0]=(0.319164526641)*x[0]**o+(0.45922058507)*x[0]+(0.26273868718)*x[1]**o+(0.919180620189)*x[1]+(0.68231305095)*x[2]**o+(-0.0793024563191)*x[2]
        s[2,4,0,1]=(0.966152397311)*x[0]**o+(-0.323284283817)*x[0]+(0.68073876378)*x[1]**o+(0.353025011982)*x[1]+(0.907154492113)*x[2]**o+(0.371499834679)*x[2]
        s[2,4,1,0]=(0.441142086658)*x[0]**o+(-0.374316420849)*x[0]+(0.886357340085)*x[1]**o+(0.172424726539)*x[1]+(0.416195908431)*x[2]**o+(0.374779208985)*x[2]
        s[2,4,1,1]=(0.809132101144)*x[0]**o+(0.33952471205)*x[0]+(0.159476692789)*x[1]**o+(-0.719297128466)*x[1]+(-0.395244031022)*x[2]**o+(-0.942445288252)*x[2]
        s[2,4,2,0]=(-0.0269665566962)*x[0]**o+(-0.932688309921)*x[0]+(0.980071332595)*x[1]**o+(0.778605742399)*x[1]+(-0.69781881904)*x[2]**o+(0.919684867278)*x[2]
        s[2,4,2,1]=(0.440810804674)*x[0]**o+(0.642802841983)*x[0]+(-0.151896393227)*x[1]**o+(0.5114517147)*x[1]+(0.202606346779)*x[2]**o+(0.145680766426)*x[2]
        s[3,0,0,0]=(0.854905410474)*x[0]**o+(-0.883160499153)*x[0]+(-0.0518436529324)*x[1]**o+(0.472365408955)*x[1]+(-0.49219002513)*x[2]**o+(0.232861430592)*x[2]
        s[3,0,0,1]=(-0.046116184691)*x[0]**o+(0.596829797988)*x[0]+(-0.144691397205)*x[1]**o+(-0.252432054327)*x[1]+(0.0799656699566)*x[2]**o+(-0.220413254006)*x[2]
        s[3,0,1,0]=(0.675157805468)*x[0]**o+(0.655696700297)*x[0]+(0.474709137832)*x[1]**o+(0.121302534807)*x[1]+(-0.718410825564)*x[2]**o+(-0.671917363185)*x[2]
        s[3,0,1,1]=(-0.514174913964)*x[0]**o+(-0.0161999364854)*x[0]+(-0.38933612074)*x[1]**o+(-0.629975655537)*x[1]+(0.488451340059)*x[2]**o+(-0.583580208023)*x[2]
        s[3,0,2,0]=(-0.56947915393)*x[0]**o+(0.434845155767)*x[0]+(-0.968993748561)*x[1]**o+(-0.510861471635)*x[1]+(-0.288591065135)*x[2]**o+(-0.230425132856)*x[2]
        s[3,0,2,1]=(-0.374979517517)*x[0]**o+(-0.0163569866605)*x[0]+(0.75837797441)*x[1]**o+(-0.632240609496)*x[1]+(0.422914146097)*x[2]**o+(0.951370365751)*x[2]
        s[3,1,0,0]=(-0.991849908442)*x[0]**o+(-0.00797307615614)*x[0]+(-0.464488377523)*x[1]**o+(0.183107151355)*x[1]+(-0.470200932584)*x[2]**o+(-0.37271407313)*x[2]
        s[3,1,0,1]=(0.714765401281)*x[0]**o+(0.604856800205)*x[0]+(-0.550067009901)*x[1]**o+(-0.5869512121)*x[1]+(0.620821251646)*x[2]**o+(-0.897762101083)*x[2]
        s[3,1,1,0]=(0.347567608921)*x[0]**o+(0.425122715827)*x[0]+(-0.802475224849)*x[1]**o+(-0.522837292632)*x[1]+(-0.160768805832)*x[2]**o+(-0.445406093759)*x[2]
        s[3,1,1,1]=(0.320284660978)*x[0]**o+(0.252323047651)*x[0]+(-0.615677083087)*x[1]**o+(-0.85623985966)*x[1]+(-0.0480138503943)*x[2]**o+(0.106970090808)*x[2]
        s[3,1,2,0]=(0.744223828984)*x[0]**o+(0.0646702358772)*x[0]+(-0.780189264654)*x[1]**o+(-0.946938785995)*x[1]+(-0.73270156813)*x[2]**o+(-0.240670757875)*x[2]
        s[3,1,2,1]=(0.24593668579)*x[0]**o+(0.0666411656061)*x[0]+(0.278029304996)*x[1]**o+(-0.543159254692)*x[1]+(-0.394304458971)*x[2]**o+(0.0789963203485)*x[2]
        s[3,2,0,0]=(-0.129597165729)*x[0]**o+(-0.375126607602)*x[0]+(-0.133573059675)*x[1]**o+(-0.222965614248)*x[1]+(0.482498045295)*x[2]**o+(0.666380815962)*x[2]
        s[3,2,0,1]=(-0.298314889544)*x[0]**o+(-0.200460718422)*x[0]+(0.969934227091)*x[1]**o+(0.982204235633)*x[1]+(-0.875400986072)*x[2]**o+(-0.962005163625)*x[2]
        s[3,2,1,0]=(-0.744314645975)*x[0]**o+(0.899141755704)*x[0]+(0.738788130658)*x[1]**o+(-0.465548783282)*x[1]+(0.27881485753)*x[2]**o+(-0.711524090298)*x[2]
        s[3,2,1,1]=(-0.698617989028)*x[0]**o+(0.082546181429)*x[0]+(-0.611282109662)*x[1]**o+(-0.28088009072)*x[1]+(-0.260638676131)*x[2]**o+(-0.554757759687)*x[2]
        s[3,2,2,0]=(-0.934985159158)*x[0]**o+(0.366271859175)*x[0]+(0.55018247768)*x[1]**o+(0.114993902724)*x[1]+(0.196571047024)*x[2]**o+(-0.889655441435)*x[2]
        s[3,2,2,1]=(-0.76297489993)*x[0]**o+(-0.00479337140756)*x[0]+(0.853999531704)*x[1]**o+(-0.563106979943)*x[1]+(-0.925873572722)*x[2]**o+(-0.0277052508628)*x[2]
        s[3,3,0,0]=(-0.138137971158)*x[0]**o+(-0.41896080496)*x[0]+(0.0482713349343)*x[1]**o+(-0.21852867848)*x[1]+(0.0190850076189)*x[2]**o+(0.583969066138)*x[2]
        s[3,3,0,1]=(0.806413890451)*x[0]**o+(-0.863828929679)*x[0]+(-0.152284946303)*x[1]**o+(-0.41164748505)*x[1]+(-0.206330733354)*x[2]**o+(0.127114872982)*x[2]
        s[3,3,1,0]=(-0.418785729088)*x[0]**o+(-0.0658589519492)*x[0]+(-0.696602204394)*x[1]**o+(-0.725906543227)*x[1]+(0.968658365691)*x[2]**o+(-0.262448319217)*x[2]
        s[3,3,1,1]=(-0.189305081379)*x[0]**o+(-0.709824241653)*x[0]+(-0.799048663139)*x[1]**o+(0.0420441850736)*x[1]+(-0.443839189643)*x[2]**o+(0.867954924509)*x[2]
        s[3,3,2,0]=(0.56856531819)*x[0]**o+(0.227835800084)*x[0]+(-0.521859405163)*x[1]**o+(-0.869003176683)*x[1]+(-0.684378595493)*x[2]**o+(0.544243688121)*x[2]
        s[3,3,2,1]=(0.714635391612)*x[0]**o+(-0.718360328239)*x[0]+(-0.510003226793)*x[1]**o+(-0.857292216621)*x[1]+(-0.295240904568)*x[2]**o+(0.340409128873)*x[2]
        s[3,4,0,0]=(-0.715361279417)*x[0]**o+(-0.829022388935)*x[0]+(0.56459861288)*x[1]**o+(0.879337406251)*x[1]+(0.423698189396)*x[2]**o+(-0.763053018627)*x[2]
        s[3,4,0,1]=(0.282251024454)*x[0]**o+(-0.275549571404)*x[0]+(0.416099858947)*x[1]**o+(0.304537400335)*x[1]+(0.846027568045)*x[2]**o+(-0.00236635940042)*x[2]
        s[3,4,1,0]=(-0.396026824114)*x[0]**o+(-0.967573299876)*x[0]+(0.419395952325)*x[1]**o+(-0.0769940862315)*x[1]+(-0.451079361856)*x[2]**o+(0.891465362708)*x[2]
        s[3,4,1,1]=(-0.629373368312)*x[0]**o+(-0.0463131060729)*x[0]+(-0.285992266736)*x[1]**o+(0.852693491625)*x[1]+(0.101111799967)*x[2]**o+(-0.982099200191)*x[2]
        s[3,4,2,0]=(-0.840902275944)*x[0]**o+(0.578695985974)*x[0]+(0.533595808779)*x[1]**o+(0.970844141257)*x[1]+(-0.942252307966)*x[2]**o+(-0.051129339745)*x[2]
        s[3,4,2,1]=(-0.981944223126)*x[0]**o+(-0.0930019659212)*x[0]+(-0.239359393705)*x[1]**o+(-0.277892188253)*x[1]+(0.447927318111)*x[2]**o+(-0.860595690016)*x[2]
        ref[0,0,0,0]=(-0.645055264384)*x_ref[0]**o+(-0.193054732885)*x_ref[0]+(0.0857819007815)*x_ref[1]**o+(-0.461528553026)*x_ref[1]+(-0.57134367637)*x_ref[2]**o+(-0.74286427921)*x_ref[2]
        ref[0,0,0,1]=(0.487034525946)*x_ref[0]**o+(-0.714187726361)*x_ref[0]+(-0.606957760182)*x_ref[1]**o+(0.0779147207958)*x_ref[1]+(-0.29526614633)*x_ref[2]**o+(-0.458434997173)*x_ref[2]
        ref[0,0,1,0]=(0.156747343036)*x_ref[0]**o+(-0.341557748989)*x_ref[0]+(0.158761045238)*x_ref[1]**o+(-0.801472587987)*x_ref[1]+(-0.271446861129)*x_ref[2]**o+(0.900123872896)*x_ref[2]
        ref[0,0,1,1]=(0.573494874527)*x_ref[0]**o+(-0.0243920948082)*x_ref[0]+(0.0948894747825)*x_ref[1]**o+(0.65093963776)*x_ref[1]+(0.671075295673)*x_ref[2]**o+(-0.864463024946)*x_ref[2]
        ref[0,0,2,0]=(0.725244561373)*x_ref[0]**o+(-0.996337933154)*x_ref[0]+(0.767557173446)*x_ref[1]**o+(0.949472898375)*x_ref[1]+(0.34694891892)*x_ref[2]**o+(-0.0275257881948)*x_ref[2]
        ref[0,0,2,1]=(0.172147320675)*x_ref[0]**o+(-0.308830513267)*x_ref[0]+(-0.186533796188)*x_ref[1]**o+(-0.253183031716)*x_ref[1]+(-0.0558420387348)*x_ref[2]**o+(0.639581162188)*x_ref[2]
        ref[0,1,0,0]=(-0.201128976272)*x_ref[0]**o+(-0.0604529403788)*x_ref[0]+(0.687516667804)*x_ref[1]**o+(0.0463366971989)*x_ref[1]+(-0.512099164154)*x_ref[2]**o+(-0.562496472369)*x_ref[2]
        ref[0,1,0,1]=(-0.579867899667)*x_ref[0]**o+(0.481671792856)*x_ref[0]+(-0.534848750787)*x_ref[1]**o+(-0.681855536562)*x_ref[1]+(-0.138906106494)*x_ref[2]**o+(0.85816397707)*x_ref[2]
        ref[0,1,1,0]=(0.361813239327)*x_ref[0]**o+(-0.371707128692)*x_ref[0]+(0.1413931258)*x_ref[1]**o+(-0.222686191434)*x_ref[1]+(0.429860839989)*x_ref[2]**o+(-0.854238845295)*x_ref[2]
        ref[0,1,1,1]=(0.777481754216)*x_ref[0]**o+(0.559920012089)*x_ref[0]+(-0.708494735928)*x_ref[1]**o+(-0.340573378074)*x_ref[1]+(-0.658897461649)*x_ref[2]**o+(0.0686343535795)*x_ref[2]
        ref[0,1,2,0]=(0.548617122525)*x_ref[0]**o+(0.142683302153)*x_ref[0]+(0.599377799663)*x_ref[1]**o+(-0.0941501509169)*x_ref[1]+(0.899613162974)*x_ref[2]**o+(0.437942658389)*x_ref[2]
        ref[0,1,2,1]=(0.055538446966)*x_ref[0]**o+(0.213360227501)*x_ref[0]+(-0.929499951666)*x_ref[1]**o+(0.336982626422)*x_ref[1]+(-0.656888335372)*x_ref[2]**o+(0.407882483181)*x_ref[2]
        ref[0,2,0,0]=(-0.874997203733)*x_ref[0]**o+(0.759216265112)*x_ref[0]+(0.948642776931)*x_ref[1]**o+(-0.242980793949)*x_ref[1]+(0.677616559342)*x_ref[2]**o+(0.341859949626)*x_ref[2]
        ref[0,2,0,1]=(-0.181175585757)*x_ref[0]**o+(0.335483796366)*x_ref[0]+(-0.664259907913)*x_ref[1]**o+(0.906747760538)*x_ref[1]+(-0.984393889605)*x_ref[2]**o+(-0.812569900067)*x_ref[2]
        ref[0,2,1,0]=(0.851036684294)*x_ref[0]**o+(-0.751387632375)*x_ref[0]+(0.724388989617)*x_ref[1]**o+(0.384230212684)*x_ref[1]+(-0.449333509528)*x_ref[2]**o+(0.784460454553)*x_ref[2]
        ref[0,2,1,1]=(-0.609413988379)*x_ref[0]**o+(-0.493285779523)*x_ref[0]+(-0.590691722184)*x_ref[1]**o+(-0.607626442728)*x_ref[1]+(0.0625828744206)*x_ref[2]**o+(-0.15945730222)*x_ref[2]
        ref[0,2,2,0]=(0.766872578253)*x_ref[0]**o+(-0.590526688805)*x_ref[0]+(0.222323591845)*x_ref[1]**o+(0.512451311011)*x_ref[1]+(-0.160540733556)*x_ref[2]**o+(0.5770077018)*x_ref[2]
        ref[0,2,2,1]=(-0.729525005374)*x_ref[0]**o+(0.986018027885)*x_ref[0]+(0.25197251588)*x_ref[1]**o+(0.0450480112291)*x_ref[1]+(-0.869546576401)*x_ref[2]**o+(-0.290965290154)*x_ref[2]
        ref[0,3,0,0]=(-0.184322684185)*x_ref[0]**o+(0.751826009862)*x_ref[0]+(-0.697266959303)*x_ref[1]**o+(0.924909596435)*x_ref[1]+(-0.638157491307)*x_ref[2]**o+(0.236651962157)*x_ref[2]
        ref[0,3,0,1]=(0.706826305243)*x_ref[0]**o+(-0.946439184553)*x_ref[0]+(0.168150324173)*x_ref[1]**o+(-0.792980290572)*x_ref[1]+(0.61272398393)*x_ref[2]**o+(0.961138901449)*x_ref[2]
        ref[0,3,1,0]=(0.933288270936)*x_ref[0]**o+(-0.55682855224)*x_ref[0]+(0.883604318263)*x_ref[1]**o+(0.816211062971)*x_ref[1]+(0.260246780311)*x_ref[2]**o+(0.905516381287)*x_ref[2]
        ref[0,3,1,1]=(0.404420727507)*x_ref[0]**o+(0.471967111659)*x_ref[0]+(0.980687681241)*x_ref[1]**o+(-0.0887974405176)*x_ref[1]+(0.0934146142552)*x_ref[2]**o+(0.646580252159)*x_ref[2]
        ref[0,3,2,0]=(-0.921777598275)*x_ref[0]**o+(0.499527698309)*x_ref[0]+(0.664601445292)*x_ref[1]**o+(-0.851079448204)*x_ref[1]+(-0.769256802656)*x_ref[2]**o+(0.730819279497)*x_ref[2]
        ref[0,3,2,1]=(-0.245507455509)*x_ref[0]**o+(0.357764745233)*x_ref[0]+(0.88892212086)*x_ref[1]**o+(-0.819843178794)*x_ref[1]+(-0.203314043822)*x_ref[2]**o+(0.224643592554)*x_ref[2]
        ref[0,4,0,0]=(0.343174611172)*x_ref[0]**o+(-0.701162898726)*x_ref[0]+(-0.447921000153)*x_ref[1]**o+(0.206599993047)*x_ref[1]+(-0.461911660893)*x_ref[2]**o+(-0.757271316865)*x_ref[2]
        ref[0,4,0,1]=(-0.55800009078)*x_ref[0]**o+(-0.39370288163)*x_ref[0]+(-0.35379408894)*x_ref[1]**o+(-0.214762670331)*x_ref[1]+(0.725231115203)*x_ref[2]**o+(0.765815232027)*x_ref[2]
        ref[0,4,1,0]=(0.440761988298)*x_ref[0]**o+(-0.798551732493)*x_ref[0]+(-0.955319652492)*x_ref[1]**o+(-0.50319720075)*x_ref[1]+(-0.213217943103)*x_ref[2]**o+(-0.882735934069)*x_ref[2]
        ref[0,4,1,1]=(0.441117080069)*x_ref[0]**o+(-0.748338538161)*x_ref[0]+(0.956112036119)*x_ref[1]**o+(-0.960896721051)*x_ref[1]+(-0.291350828791)*x_ref[2]**o+(-0.488896289566)*x_ref[2]
        ref[0,4,2,0]=(-0.855035257555)*x_ref[0]**o+(-0.579064017558)*x_ref[0]+(-0.501257230663)*x_ref[1]**o+(-0.956216916996)*x_ref[1]+(-0.860898494221)*x_ref[2]**o+(-0.830087013431)*x_ref[2]
        ref[0,4,2,1]=(-0.868978503525)*x_ref[0]**o+(0.162013861935)*x_ref[0]+(-0.311759384766)*x_ref[1]**o+(0.399068885633)*x_ref[1]+(-0.569888913523)*x_ref[2]**o+(-0.394173563136)*x_ref[2]
        ref[1,0,0,0]=(0.215242187817)*x_ref[0]**o+(-0.215197471124)*x_ref[0]+(0.779932221493)*x_ref[1]**o+(-0.365740975053)*x_ref[1]+(0.601368178784)*x_ref[2]**o+(-0.170706718888)*x_ref[2]
        ref[1,0,0,1]=(0.862102162389)*x_ref[0]**o+(-0.815403119428)*x_ref[0]+(0.269334729254)*x_ref[1]**o+(-0.511457037588)*x_ref[1]+(0.00982312570019)*x_ref[2]**o+(-0.407471705546)*x_ref[2]
        ref[1,0,1,0]=(-0.728800252524)*x_ref[0]**o+(-0.178562348078)*x_ref[0]+(0.711141895031)*x_ref[1]**o+(-0.0037290743948)*x_ref[1]+(-0.353455013195)*x_ref[2]**o+(-0.0812715065083)*x_ref[2]
        ref[1,0,1,1]=(-0.689013409151)*x_ref[0]**o+(-0.467030398964)*x_ref[0]+(0.295472079898)*x_ref[1]**o+(0.803549469534)*x_ref[1]+(-0.360809851719)*x_ref[2]**o+(-0.563821123318)*x_ref[2]
        ref[1,0,2,0]=(0.238079356258)*x_ref[0]**o+(-0.385019487483)*x_ref[0]+(0.864174173897)*x_ref[1]**o+(-0.338065985154)*x_ref[1]+(-0.516837345006)*x_ref[2]**o+(0.611410140553)*x_ref[2]
        ref[1,0,2,1]=(0.937340721244)*x_ref[0]**o+(-0.0936947975478)*x_ref[0]+(0.313991879475)*x_ref[1]**o+(-0.126811107707)*x_ref[1]+(-0.627872015529)*x_ref[2]**o+(-0.981891626964)*x_ref[2]
        ref[1,1,0,0]=(-0.0675382298198)*x_ref[0]**o+(-0.783093574363)*x_ref[0]+(-0.377941739502)*x_ref[1]**o+(-0.0833668456557)*x_ref[1]+(-0.157483560616)*x_ref[2]**o+(0.686957855465)*x_ref[2]
        ref[1,1,0,1]=(0.151028535239)*x_ref[0]**o+(0.992218304228)*x_ref[0]+(-0.686192558465)*x_ref[1]**o+(-0.0130549001993)*x_ref[1]+(0.578982765961)*x_ref[2]**o+(0.814271616163)*x_ref[2]
        ref[1,1,1,0]=(0.384343377969)*x_ref[0]**o+(0.463218368911)*x_ref[0]+(0.35576368147)*x_ref[1]**o+(-0.834867049786)*x_ref[1]+(-0.274552267393)*x_ref[2]**o+(0.623371271421)*x_ref[2]
        ref[1,1,1,1]=(-0.61717606851)*x_ref[0]**o+(-0.578562595163)*x_ref[0]+(0.0518387545717)*x_ref[1]**o+(-0.30815303855)*x_ref[1]+(0.556738906305)*x_ref[2]**o+(0.38164058851)*x_ref[2]
        ref[1,1,2,0]=(0.52042073171)*x_ref[0]**o+(0.698894225253)*x_ref[0]+(0.120169831848)*x_ref[1]**o+(0.238450852883)*x_ref[1]+(-0.552966574246)*x_ref[2]**o+(0.661290976968)*x_ref[2]
        ref[1,1,2,1]=(-0.428668260768)*x_ref[0]**o+(0.568751515596)*x_ref[0]+(-0.0768903249884)*x_ref[1]**o+(0.768814424639)*x_ref[1]+(0.424759560455)*x_ref[2]**o+(-0.690938863922)*x_ref[2]
        ref[1,2,0,0]=(-0.5443917305)*x_ref[0]**o+(-0.0595999390634)*x_ref[0]+(0.901507044764)*x_ref[1]**o+(-0.681191541382)*x_ref[1]+(-0.123122907072)*x_ref[2]**o+(-0.152869512816)*x_ref[2]
        ref[1,2,0,1]=(-0.0145795630242)*x_ref[0]**o+(-0.53313358583)*x_ref[0]+(0.629028937665)*x_ref[1]**o+(-0.539546800632)*x_ref[1]+(-0.252160197397)*x_ref[2]**o+(0.0133469506662)*x_ref[2]
        ref[1,2,1,0]=(0.00594869891786)*x_ref[0]**o+(-0.56813890169)*x_ref[0]+(-0.685679553166)*x_ref[1]**o+(0.81727698867)*x_ref[1]+(-0.895427226202)*x_ref[2]**o+(0.140096007363)*x_ref[2]
        ref[1,2,1,1]=(-0.770028732259)*x_ref[0]**o+(0.827053138879)*x_ref[0]+(-0.917989309552)*x_ref[1]**o+(-0.0517956582083)*x_ref[1]+(-0.266844746028)*x_ref[2]**o+(-0.99294686081)*x_ref[2]
        ref[1,2,2,0]=(0.0529054005828)*x_ref[0]**o+(-0.807219927076)*x_ref[0]+(-0.296037864436)*x_ref[1]**o+(-0.617547075266)*x_ref[1]+(-0.966016304179)*x_ref[2]**o+(-0.168793187561)*x_ref[2]
        ref[1,2,2,1]=(0.868690089119)*x_ref[0]**o+(-0.610624824567)*x_ref[0]+(-0.411113678451)*x_ref[1]**o+(0.103726193007)*x_ref[1]+(-0.148892296878)*x_ref[2]**o+(-0.195079797362)*x_ref[2]
        ref[1,3,0,0]=(0.248806387929)*x_ref[0]**o+(-0.683762743914)*x_ref[0]+(0.460928199336)*x_ref[1]**o+(0.338430932923)*x_ref[1]+(0.589813218945)*x_ref[2]**o+(-0.632423181868)*x_ref[2]
        ref[1,3,0,1]=(-0.338476160834)*x_ref[0]**o+(0.491753392304)*x_ref[0]+(-0.590351153797)*x_ref[1]**o+(-0.448026649636)*x_ref[1]+(-0.806930505259)*x_ref[2]**o+(-0.262565462369)*x_ref[2]
        ref[1,3,1,0]=(0.808414907549)*x_ref[0]**o+(0.156426917662)*x_ref[0]+(0.33216874106)*x_ref[1]**o+(-0.714791665494)*x_ref[1]+(-0.163168903301)*x_ref[2]**o+(-0.0423061478551)*x_ref[2]
        ref[1,3,1,1]=(0.72498761831)*x_ref[0]**o+(0.08194542863)*x_ref[0]+(-0.566316087403)*x_ref[1]**o+(0.49486476938)*x_ref[1]+(0.744201752122)*x_ref[2]**o+(0.766413325553)*x_ref[2]
        ref[1,3,2,0]=(0.793853329178)*x_ref[0]**o+(-0.883754372376)*x_ref[0]+(-0.991903018183)*x_ref[1]**o+(0.279576133295)*x_ref[1]+(-0.0364449658341)*x_ref[2]**o+(-0.85024249755)*x_ref[2]
        ref[1,3,2,1]=(0.308927107876)*x_ref[0]**o+(-0.995603136748)*x_ref[0]+(-0.15217214087)*x_ref[1]**o+(-0.554310725129)*x_ref[1]+(0.69107000546)*x_ref[2]**o+(0.353155450955)*x_ref[2]
        ref[1,4,0,0]=(0.696506340076)*x_ref[0]**o+(0.889728425166)*x_ref[0]+(-0.858268204558)*x_ref[1]**o+(0.353251100246)*x_ref[1]+(-0.974872511432)*x_ref[2]**o+(-0.878064243518)*x_ref[2]
        ref[1,4,0,1]=(-0.232678527901)*x_ref[0]**o+(-0.60738965282)*x_ref[0]+(-0.629841924447)*x_ref[1]**o+(0.0731692547228)*x_ref[1]+(-0.457280478523)*x_ref[2]**o+(-0.309371862287)*x_ref[2]
        ref[1,4,1,0]=(-0.0254189247267)*x_ref[0]**o+(-0.160538018478)*x_ref[0]+(-0.757800049379)*x_ref[1]**o+(0.108707596517)*x_ref[1]+(-0.182095912638)*x_ref[2]**o+(0.565356253873)*x_ref[2]
        ref[1,4,1,1]=(0.353936491579)*x_ref[0]**o+(0.368146351094)*x_ref[0]+(-0.0271653916784)*x_ref[1]**o+(0.7863197856)*x_ref[1]+(-0.0642828131827)*x_ref[2]**o+(0.946295880643)*x_ref[2]
        ref[1,4,2,0]=(-0.151347070208)*x_ref[0]**o+(-0.490039530717)*x_ref[0]+(-0.222861393521)*x_ref[1]**o+(0.160218722103)*x_ref[1]+(-0.881185613005)*x_ref[2]**o+(0.453889724466)*x_ref[2]
        ref[1,4,2,1]=(0.0185253314435)*x_ref[0]**o+(0.824691270585)*x_ref[0]+(-0.175025465282)*x_ref[1]**o+(-0.946563236595)*x_ref[1]+(0.915012725055)*x_ref[2]**o+(-0.14949108932)*x_ref[2]
        ref[2,0,0,0]=(0.608065273397)*x_ref[0]**o+(0.103537849738)*x_ref[0]+(0.640722456034)*x_ref[1]**o+(0.707059952842)*x_ref[1]+(0.958670726841)*x_ref[2]**o+(0.178333168266)*x_ref[2]
        ref[2,0,0,1]=(0.762096192615)*x_ref[0]**o+(-0.91311774327)*x_ref[0]+(-0.636481030154)*x_ref[1]**o+(0.0485431700913)*x_ref[1]+(-0.76456752608)*x_ref[2]**o+(0.988366648174)*x_ref[2]
        ref[2,0,1,0]=(-0.997545907902)*x_ref[0]**o+(0.0386694282297)*x_ref[0]+(-0.95454977224)*x_ref[1]**o+(-0.472500001853)*x_ref[1]+(-0.315668831265)*x_ref[2]**o+(-0.0911950672323)*x_ref[2]
        ref[2,0,1,1]=(0.758639431658)*x_ref[0]**o+(-0.35267267476)*x_ref[0]+(0.733519690739)*x_ref[1]**o+(0.687429543066)*x_ref[1]+(0.748932363975)*x_ref[2]**o+(-0.360330977046)*x_ref[2]
        ref[2,0,2,0]=(0.748882354686)*x_ref[0]**o+(0.398195068825)*x_ref[0]+(0.268707244918)*x_ref[1]**o+(-0.906848889273)*x_ref[1]+(0.8203042296)*x_ref[2]**o+(-0.388209014017)*x_ref[2]
        ref[2,0,2,1]=(-0.622158650545)*x_ref[0]**o+(0.757162985883)*x_ref[0]+(0.213276305935)*x_ref[1]**o+(0.376995936453)*x_ref[1]+(-0.919851919774)*x_ref[2]**o+(-0.341006967087)*x_ref[2]
        ref[2,1,0,0]=(0.45283405067)*x_ref[0]**o+(-0.456183034235)*x_ref[0]+(-0.211030502465)*x_ref[1]**o+(0.874532774776)*x_ref[1]+(-0.245927412434)*x_ref[2]**o+(0.426470240035)*x_ref[2]
        ref[2,1,0,1]=(-0.167484852552)*x_ref[0]**o+(0.23618650325)*x_ref[0]+(-0.256431628436)*x_ref[1]**o+(-0.758314815145)*x_ref[1]+(-0.553126534983)*x_ref[2]**o+(-0.993390924398)*x_ref[2]
        ref[2,1,1,0]=(-0.579896706714)*x_ref[0]**o+(0.0247821970864)*x_ref[0]+(-0.485603356124)*x_ref[1]**o+(0.610550482437)*x_ref[1]+(-0.788558005559)*x_ref[2]**o+(-0.716855983085)*x_ref[2]
        ref[2,1,1,1]=(0.011213932334)*x_ref[0]**o+(-0.606541681102)*x_ref[0]+(-0.783918973118)*x_ref[1]**o+(-0.129337676239)*x_ref[1]+(-0.681878469428)*x_ref[2]**o+(-0.818559911243)*x_ref[2]
        ref[2,1,2,0]=(0.806311020506)*x_ref[0]**o+(0.348481209449)*x_ref[0]+(0.147356878845)*x_ref[1]**o+(0.481921509802)*x_ref[1]+(0.445429036477)*x_ref[2]**o+(0.113745424091)*x_ref[2]
        ref[2,1,2,1]=(-0.587849278852)*x_ref[0]**o+(-0.141739080759)*x_ref[0]+(-0.0431697185243)*x_ref[1]**o+(0.845205068629)*x_ref[1]+(0.485473344628)*x_ref[2]**o+(0.416700333568)*x_ref[2]
        ref[2,2,0,0]=(-0.36580939154)*x_ref[0]**o+(-0.826777569794)*x_ref[0]+(-0.322507767896)*x_ref[1]**o+(0.697307401686)*x_ref[1]+(0.52326834016)*x_ref[2]**o+(-0.31919043817)*x_ref[2]
        ref[2,2,0,1]=(-0.130664405569)*x_ref[0]**o+(0.823599150158)*x_ref[0]+(0.751486085263)*x_ref[1]**o+(0.150119789292)*x_ref[1]+(-0.307393365311)*x_ref[2]**o+(0.919517717122)*x_ref[2]
        ref[2,2,1,0]=(0.254288128084)*x_ref[0]**o+(0.580902444535)*x_ref[0]+(0.90113516563)*x_ref[1]**o+(0.805502401301)*x_ref[1]+(-0.483775735536)*x_ref[2]**o+(-0.846660440246)*x_ref[2]
        ref[2,2,1,1]=(-0.481100104989)*x_ref[0]**o+(0.459159658041)*x_ref[0]+(-0.886023770838)*x_ref[1]**o+(0.693304415006)*x_ref[1]+(-0.490424960617)*x_ref[2]**o+(0.618464834808)*x_ref[2]
        ref[2,2,2,0]=(0.851037708736)*x_ref[0]**o+(0.973238080753)*x_ref[0]+(0.617606926402)*x_ref[1]**o+(-0.0129774413768)*x_ref[1]+(0.342094147972)*x_ref[2]**o+(-0.698988316685)*x_ref[2]
        ref[2,2,2,1]=(-0.571806691565)*x_ref[0]**o+(-0.488244261444)*x_ref[0]+(-0.408779266509)*x_ref[1]**o+(-0.688083559849)*x_ref[1]+(-0.668712586564)*x_ref[2]**o+(0.811908851853)*x_ref[2]
        ref[2,3,0,0]=(0.305354422638)*x_ref[0]**o+(-0.347372904868)*x_ref[0]+(0.147767334473)*x_ref[1]**o+(0.597885123439)*x_ref[1]+(-0.532183927826)*x_ref[2]**o+(0.778189138615)*x_ref[2]
        ref[2,3,0,1]=(0.986328831124)*x_ref[0]**o+(0.237847460579)*x_ref[0]+(0.350775306824)*x_ref[1]**o+(-0.750995843201)*x_ref[1]+(0.85575809431)*x_ref[2]**o+(0.624236482292)*x_ref[2]
        ref[2,3,1,0]=(-0.418600918765)*x_ref[0]**o+(-0.958489157688)*x_ref[0]+(-0.524398996544)*x_ref[1]**o+(0.186795580218)*x_ref[1]+(-0.364079386452)*x_ref[2]**o+(-0.330796017113)*x_ref[2]
        ref[2,3,1,1]=(0.632410928458)*x_ref[0]**o+(-0.557450605906)*x_ref[0]+(-0.699741830058)*x_ref[1]**o+(0.867693032849)*x_ref[1]+(-0.770879368311)*x_ref[2]**o+(0.972808988148)*x_ref[2]
        ref[2,3,2,0]=(0.283379195721)*x_ref[0]**o+(-0.636902680473)*x_ref[0]+(0.609885794066)*x_ref[1]**o+(0.831554040093)*x_ref[1]+(-0.330474761682)*x_ref[2]**o+(0.000310556070561)*x_ref[2]
        ref[2,3,2,1]=(0.363575347708)*x_ref[0]**o+(-0.327653113631)*x_ref[0]+(-0.842370028303)*x_ref[1]**o+(0.115573924346)*x_ref[1]+(0.0903633504982)*x_ref[2]**o+(-0.619568111717)*x_ref[2]
        ref[2,4,0,0]=(0.319164526641)*x_ref[0]**o+(0.45922058507)*x_ref[0]+(0.26273868718)*x_ref[1]**o+(0.919180620189)*x_ref[1]+(0.68231305095)*x_ref[2]**o+(-0.0793024563191)*x_ref[2]
        ref[2,4,0,1]=(0.966152397311)*x_ref[0]**o+(-0.323284283817)*x_ref[0]+(0.68073876378)*x_ref[1]**o+(0.353025011982)*x_ref[1]+(0.907154492113)*x_ref[2]**o+(0.371499834679)*x_ref[2]
        ref[2,4,1,0]=(0.441142086658)*x_ref[0]**o+(-0.374316420849)*x_ref[0]+(0.886357340085)*x_ref[1]**o+(0.172424726539)*x_ref[1]+(0.416195908431)*x_ref[2]**o+(0.374779208985)*x_ref[2]
        ref[2,4,1,1]=(0.809132101144)*x_ref[0]**o+(0.33952471205)*x_ref[0]+(0.159476692789)*x_ref[1]**o+(-0.719297128466)*x_ref[1]+(-0.395244031022)*x_ref[2]**o+(-0.942445288252)*x_ref[2]
        ref[2,4,2,0]=(-0.0269665566962)*x_ref[0]**o+(-0.932688309921)*x_ref[0]+(0.980071332595)*x_ref[1]**o+(0.778605742399)*x_ref[1]+(-0.69781881904)*x_ref[2]**o+(0.919684867278)*x_ref[2]
        ref[2,4,2,1]=(0.440810804674)*x_ref[0]**o+(0.642802841983)*x_ref[0]+(-0.151896393227)*x_ref[1]**o+(0.5114517147)*x_ref[1]+(0.202606346779)*x_ref[2]**o+(0.145680766426)*x_ref[2]
        ref[3,0,0,0]=(0.854905410474)*x_ref[0]**o+(-0.883160499153)*x_ref[0]+(-0.0518436529324)*x_ref[1]**o+(0.472365408955)*x_ref[1]+(-0.49219002513)*x_ref[2]**o+(0.232861430592)*x_ref[2]
        ref[3,0,0,1]=(-0.046116184691)*x_ref[0]**o+(0.596829797988)*x_ref[0]+(-0.144691397205)*x_ref[1]**o+(-0.252432054327)*x_ref[1]+(0.0799656699566)*x_ref[2]**o+(-0.220413254006)*x_ref[2]
        ref[3,0,1,0]=(0.675157805468)*x_ref[0]**o+(0.655696700297)*x_ref[0]+(0.474709137832)*x_ref[1]**o+(0.121302534807)*x_ref[1]+(-0.718410825564)*x_ref[2]**o+(-0.671917363185)*x_ref[2]
        ref[3,0,1,1]=(-0.514174913964)*x_ref[0]**o+(-0.0161999364854)*x_ref[0]+(-0.38933612074)*x_ref[1]**o+(-0.629975655537)*x_ref[1]+(0.488451340059)*x_ref[2]**o+(-0.583580208023)*x_ref[2]
        ref[3,0,2,0]=(-0.56947915393)*x_ref[0]**o+(0.434845155767)*x_ref[0]+(-0.968993748561)*x_ref[1]**o+(-0.510861471635)*x_ref[1]+(-0.288591065135)*x_ref[2]**o+(-0.230425132856)*x_ref[2]
        ref[3,0,2,1]=(-0.374979517517)*x_ref[0]**o+(-0.0163569866605)*x_ref[0]+(0.75837797441)*x_ref[1]**o+(-0.632240609496)*x_ref[1]+(0.422914146097)*x_ref[2]**o+(0.951370365751)*x_ref[2]
        ref[3,1,0,0]=(-0.991849908442)*x_ref[0]**o+(-0.00797307615614)*x_ref[0]+(-0.464488377523)*x_ref[1]**o+(0.183107151355)*x_ref[1]+(-0.470200932584)*x_ref[2]**o+(-0.37271407313)*x_ref[2]
        ref[3,1,0,1]=(0.714765401281)*x_ref[0]**o+(0.604856800205)*x_ref[0]+(-0.550067009901)*x_ref[1]**o+(-0.5869512121)*x_ref[1]+(0.620821251646)*x_ref[2]**o+(-0.897762101083)*x_ref[2]
        ref[3,1,1,0]=(0.347567608921)*x_ref[0]**o+(0.425122715827)*x_ref[0]+(-0.802475224849)*x_ref[1]**o+(-0.522837292632)*x_ref[1]+(-0.160768805832)*x_ref[2]**o+(-0.445406093759)*x_ref[2]
        ref[3,1,1,1]=(0.320284660978)*x_ref[0]**o+(0.252323047651)*x_ref[0]+(-0.615677083087)*x_ref[1]**o+(-0.85623985966)*x_ref[1]+(-0.0480138503943)*x_ref[2]**o+(0.106970090808)*x_ref[2]
        ref[3,1,2,0]=(0.744223828984)*x_ref[0]**o+(0.0646702358772)*x_ref[0]+(-0.780189264654)*x_ref[1]**o+(-0.946938785995)*x_ref[1]+(-0.73270156813)*x_ref[2]**o+(-0.240670757875)*x_ref[2]
        ref[3,1,2,1]=(0.24593668579)*x_ref[0]**o+(0.0666411656061)*x_ref[0]+(0.278029304996)*x_ref[1]**o+(-0.543159254692)*x_ref[1]+(-0.394304458971)*x_ref[2]**o+(0.0789963203485)*x_ref[2]
        ref[3,2,0,0]=(-0.129597165729)*x_ref[0]**o+(-0.375126607602)*x_ref[0]+(-0.133573059675)*x_ref[1]**o+(-0.222965614248)*x_ref[1]+(0.482498045295)*x_ref[2]**o+(0.666380815962)*x_ref[2]
        ref[3,2,0,1]=(-0.298314889544)*x_ref[0]**o+(-0.200460718422)*x_ref[0]+(0.969934227091)*x_ref[1]**o+(0.982204235633)*x_ref[1]+(-0.875400986072)*x_ref[2]**o+(-0.962005163625)*x_ref[2]
        ref[3,2,1,0]=(-0.744314645975)*x_ref[0]**o+(0.899141755704)*x_ref[0]+(0.738788130658)*x_ref[1]**o+(-0.465548783282)*x_ref[1]+(0.27881485753)*x_ref[2]**o+(-0.711524090298)*x_ref[2]
        ref[3,2,1,1]=(-0.698617989028)*x_ref[0]**o+(0.082546181429)*x_ref[0]+(-0.611282109662)*x_ref[1]**o+(-0.28088009072)*x_ref[1]+(-0.260638676131)*x_ref[2]**o+(-0.554757759687)*x_ref[2]
        ref[3,2,2,0]=(-0.934985159158)*x_ref[0]**o+(0.366271859175)*x_ref[0]+(0.55018247768)*x_ref[1]**o+(0.114993902724)*x_ref[1]+(0.196571047024)*x_ref[2]**o+(-0.889655441435)*x_ref[2]
        ref[3,2,2,1]=(-0.76297489993)*x_ref[0]**o+(-0.00479337140756)*x_ref[0]+(0.853999531704)*x_ref[1]**o+(-0.563106979943)*x_ref[1]+(-0.925873572722)*x_ref[2]**o+(-0.0277052508628)*x_ref[2]
        ref[3,3,0,0]=(-0.138137971158)*x_ref[0]**o+(-0.41896080496)*x_ref[0]+(0.0482713349343)*x_ref[1]**o+(-0.21852867848)*x_ref[1]+(0.0190850076189)*x_ref[2]**o+(0.583969066138)*x_ref[2]
        ref[3,3,0,1]=(0.806413890451)*x_ref[0]**o+(-0.863828929679)*x_ref[0]+(-0.152284946303)*x_ref[1]**o+(-0.41164748505)*x_ref[1]+(-0.206330733354)*x_ref[2]**o+(0.127114872982)*x_ref[2]
        ref[3,3,1,0]=(-0.418785729088)*x_ref[0]**o+(-0.0658589519492)*x_ref[0]+(-0.696602204394)*x_ref[1]**o+(-0.725906543227)*x_ref[1]+(0.968658365691)*x_ref[2]**o+(-0.262448319217)*x_ref[2]
        ref[3,3,1,1]=(-0.189305081379)*x_ref[0]**o+(-0.709824241653)*x_ref[0]+(-0.799048663139)*x_ref[1]**o+(0.0420441850736)*x_ref[1]+(-0.443839189643)*x_ref[2]**o+(0.867954924509)*x_ref[2]
        ref[3,3,2,0]=(0.56856531819)*x_ref[0]**o+(0.227835800084)*x_ref[0]+(-0.521859405163)*x_ref[1]**o+(-0.869003176683)*x_ref[1]+(-0.684378595493)*x_ref[2]**o+(0.544243688121)*x_ref[2]
        ref[3,3,2,1]=(0.714635391612)*x_ref[0]**o+(-0.718360328239)*x_ref[0]+(-0.510003226793)*x_ref[1]**o+(-0.857292216621)*x_ref[1]+(-0.295240904568)*x_ref[2]**o+(0.340409128873)*x_ref[2]
        ref[3,4,0,0]=(-0.715361279417)*x_ref[0]**o+(-0.829022388935)*x_ref[0]+(0.56459861288)*x_ref[1]**o+(0.879337406251)*x_ref[1]+(0.423698189396)*x_ref[2]**o+(-0.763053018627)*x_ref[2]
        ref[3,4,0,1]=(0.282251024454)*x_ref[0]**o+(-0.275549571404)*x_ref[0]+(0.416099858947)*x_ref[1]**o+(0.304537400335)*x_ref[1]+(0.846027568045)*x_ref[2]**o+(-0.00236635940042)*x_ref[2]
        ref[3,4,1,0]=(-0.396026824114)*x_ref[0]**o+(-0.967573299876)*x_ref[0]+(0.419395952325)*x_ref[1]**o+(-0.0769940862315)*x_ref[1]+(-0.451079361856)*x_ref[2]**o+(0.891465362708)*x_ref[2]
        ref[3,4,1,1]=(-0.629373368312)*x_ref[0]**o+(-0.0463131060729)*x_ref[0]+(-0.285992266736)*x_ref[1]**o+(0.852693491625)*x_ref[1]+(0.101111799967)*x_ref[2]**o+(-0.982099200191)*x_ref[2]
        ref[3,4,2,0]=(-0.840902275944)*x_ref[0]**o+(0.578695985974)*x_ref[0]+(0.533595808779)*x_ref[1]**o+(0.970844141257)*x_ref[1]+(-0.942252307966)*x_ref[2]**o+(-0.051129339745)*x_ref[2]
        ref[3,4,2,1]=(-0.981944223126)*x_ref[0]**o+(-0.0930019659212)*x_ref[0]+(-0.239359393705)*x_ref[1]**o+(-0.277892188253)*x_ref[1]+(0.447927318111)*x_ref[2]**o+(-0.860595690016)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromSymbol_Solution_rank0(self):
      """
      tests interpolate for rank 0 Symbol onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from Solution onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(),dim=dim)
      s=Data(0,(),w)
      ref=Data(0,(),w_ref)
      if dim==2:
        s=(-0.420247061675)*x[0]**o+(0.845150024832)*x[0]+(-0.70586438118)*x[1]**o+(0.336975044874)*x[1]
        ref=(-0.420247061675)*x_ref[0]**o+(0.845150024832)*x_ref[0]+(-0.70586438118)*x_ref[1]**o+(0.336975044874)*x_ref[1]
      else:
        s=(-0.280517761942)*x[0]**o+(-0.612238168916)*x[0]+(-0.103863621547)*x[1]**o+(-0.0303139932438)*x[1]+(0.216524064885)*x[2]**o+(-0.722384143847)*x[2]
        ref=(-0.280517761942)*x_ref[0]**o+(-0.612238168916)*x_ref[0]+(-0.103863621547)*x_ref[1]**o+(-0.0303139932438)*x_ref[1]+(0.216524064885)*x_ref[2]**o+(-0.722384143847)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromSymbol_Solution_rank1(self):
      """
      tests interpolate for rank 1 Symbol onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from Solution onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(2,),dim=dim)
      s=Data(0,(2,),w)
      ref=Data(0,(2,),w_ref)
      if dim==2:
        s[0]=(0.572752317912)*x[0]**o+(-0.557996031811)*x[0]+(-0.105666248913)*x[1]**o+(0.285149455719)*x[1]
        s[1]=(0.83173456891)*x[0]**o+(-0.347928678237)*x[0]+(0.326013379698)*x[1]**o+(0.949427685784)*x[1]
        ref[0]=(0.572752317912)*x_ref[0]**o+(-0.557996031811)*x_ref[0]+(-0.105666248913)*x_ref[1]**o+(0.285149455719)*x_ref[1]
        ref[1]=(0.83173456891)*x_ref[0]**o+(-0.347928678237)*x_ref[0]+(0.326013379698)*x_ref[1]**o+(0.949427685784)*x_ref[1]
      else:
        s[0]=(0.254569308395)*x[0]**o+(-0.199165607071)*x[0]+(0.368026751091)*x[1]**o+(0.749363957437)*x[1]+(0.784765582138)*x[2]**o+(-0.385360238329)*x[2]
        s[1]=(0.915245931081)*x[0]**o+(-0.427166268095)*x[0]+(-0.310327593132)*x[1]**o+(0.0685458597459)*x[1]+(0.951457086556)*x[2]**o+(-0.103882183326)*x[2]
        ref[0]=(0.254569308395)*x_ref[0]**o+(-0.199165607071)*x_ref[0]+(0.368026751091)*x_ref[1]**o+(0.749363957437)*x_ref[1]+(0.784765582138)*x_ref[2]**o+(-0.385360238329)*x_ref[2]
        ref[1]=(0.915245931081)*x_ref[0]**o+(-0.427166268095)*x_ref[0]+(-0.310327593132)*x_ref[1]**o+(0.0685458597459)*x_ref[1]+(0.951457086556)*x_ref[2]**o+(-0.103882183326)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromSymbol_Solution_rank2(self):
      """
      tests interpolate for rank 2 Symbol onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from Solution onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5),dim=dim)
      s=Data(0,(4, 5),w)
      ref=Data(0,(4, 5),w_ref)
      if dim==2:
        s[0,0]=(0.835384508525)*x[0]**o+(-0.0883356828618)*x[0]+(0.540787143442)*x[1]**o+(-0.844850006869)*x[1]
        s[0,1]=(-0.604551369282)*x[0]**o+(0.364670860569)*x[0]+(0.0483050521734)*x[1]**o+(-0.203895463114)*x[1]
        s[0,2]=(-0.815610974203)*x[0]**o+(0.833865082306)*x[0]+(0.0720866771967)*x[1]**o+(-0.247530114868)*x[1]
        s[0,3]=(-0.735317110832)*x[0]**o+(-0.148571378133)*x[0]+(-0.231962284467)*x[1]**o+(0.636603292819)*x[1]
        s[0,4]=(-0.649798394504)*x[0]**o+(0.185726064609)*x[0]+(0.484279786833)*x[1]**o+(0.519453975218)*x[1]
        s[1,0]=(-0.79002150838)*x[0]**o+(-0.899574857585)*x[0]+(-0.233297202379)*x[1]**o+(0.352419375812)*x[1]
        s[1,1]=(0.607990106144)*x[0]**o+(-0.916411570588)*x[0]+(-0.172031442436)*x[1]**o+(0.795245984279)*x[1]
        s[1,2]=(-0.534291044354)*x[0]**o+(0.0230033818057)*x[0]+(0.399956972297)*x[1]**o+(-0.115654728987)*x[1]
        s[1,3]=(0.0920349813015)*x[0]**o+(-0.0312648034967)*x[0]+(0.347084284907)*x[1]**o+(-0.796084479949)*x[1]
        s[1,4]=(0.341416390322)*x[0]**o+(0.400112572704)*x[0]+(-0.0833860476359)*x[1]**o+(0.999568411991)*x[1]
        s[2,0]=(0.128164199552)*x[0]**o+(0.812514973535)*x[0]+(-0.118931795809)*x[1]**o+(-0.375359555032)*x[1]
        s[2,1]=(-0.288738444027)*x[0]**o+(0.572163772906)*x[0]+(0.184909580584)*x[1]**o+(-0.727127325602)*x[1]
        s[2,2]=(0.170486550408)*x[0]**o+(0.493448396565)*x[0]+(0.00435735862836)*x[1]**o+(0.250237688279)*x[1]
        s[2,3]=(-0.7062351645)*x[0]**o+(-0.0122464096259)*x[0]+(-0.976360268944)*x[1]**o+(0.265587257351)*x[1]
        s[2,4]=(-0.710741742377)*x[0]**o+(0.111377096886)*x[0]+(0.518621968579)*x[1]**o+(-0.264799841387)*x[1]
        s[3,0]=(0.661089560632)*x[0]**o+(-0.175284355246)*x[0]+(0.434737404117)*x[1]**o+(-0.569800530193)*x[1]
        s[3,1]=(-0.581369919535)*x[0]**o+(-0.599162778688)*x[0]+(-0.772449383214)*x[1]**o+(-0.286495247533)*x[1]
        s[3,2]=(0.350919520995)*x[0]**o+(0.2325739414)*x[0]+(0.744285468619)*x[1]**o+(-0.603444606663)*x[1]
        s[3,3]=(-0.946270717928)*x[0]**o+(0.944211313875)*x[0]+(0.128830208798)*x[1]**o+(-0.108311954532)*x[1]
        s[3,4]=(-0.0704013779853)*x[0]**o+(-0.334719508353)*x[0]+(0.39364248466)*x[1]**o+(-0.894987529202)*x[1]
        ref[0,0]=(0.835384508525)*x_ref[0]**o+(-0.0883356828618)*x_ref[0]+(0.540787143442)*x_ref[1]**o+(-0.844850006869)*x_ref[1]
        ref[0,1]=(-0.604551369282)*x_ref[0]**o+(0.364670860569)*x_ref[0]+(0.0483050521734)*x_ref[1]**o+(-0.203895463114)*x_ref[1]
        ref[0,2]=(-0.815610974203)*x_ref[0]**o+(0.833865082306)*x_ref[0]+(0.0720866771967)*x_ref[1]**o+(-0.247530114868)*x_ref[1]
        ref[0,3]=(-0.735317110832)*x_ref[0]**o+(-0.148571378133)*x_ref[0]+(-0.231962284467)*x_ref[1]**o+(0.636603292819)*x_ref[1]
        ref[0,4]=(-0.649798394504)*x_ref[0]**o+(0.185726064609)*x_ref[0]+(0.484279786833)*x_ref[1]**o+(0.519453975218)*x_ref[1]
        ref[1,0]=(-0.79002150838)*x_ref[0]**o+(-0.899574857585)*x_ref[0]+(-0.233297202379)*x_ref[1]**o+(0.352419375812)*x_ref[1]
        ref[1,1]=(0.607990106144)*x_ref[0]**o+(-0.916411570588)*x_ref[0]+(-0.172031442436)*x_ref[1]**o+(0.795245984279)*x_ref[1]
        ref[1,2]=(-0.534291044354)*x_ref[0]**o+(0.0230033818057)*x_ref[0]+(0.399956972297)*x_ref[1]**o+(-0.115654728987)*x_ref[1]
        ref[1,3]=(0.0920349813015)*x_ref[0]**o+(-0.0312648034967)*x_ref[0]+(0.347084284907)*x_ref[1]**o+(-0.796084479949)*x_ref[1]
        ref[1,4]=(0.341416390322)*x_ref[0]**o+(0.400112572704)*x_ref[0]+(-0.0833860476359)*x_ref[1]**o+(0.999568411991)*x_ref[1]
        ref[2,0]=(0.128164199552)*x_ref[0]**o+(0.812514973535)*x_ref[0]+(-0.118931795809)*x_ref[1]**o+(-0.375359555032)*x_ref[1]
        ref[2,1]=(-0.288738444027)*x_ref[0]**o+(0.572163772906)*x_ref[0]+(0.184909580584)*x_ref[1]**o+(-0.727127325602)*x_ref[1]
        ref[2,2]=(0.170486550408)*x_ref[0]**o+(0.493448396565)*x_ref[0]+(0.00435735862836)*x_ref[1]**o+(0.250237688279)*x_ref[1]
        ref[2,3]=(-0.7062351645)*x_ref[0]**o+(-0.0122464096259)*x_ref[0]+(-0.976360268944)*x_ref[1]**o+(0.265587257351)*x_ref[1]
        ref[2,4]=(-0.710741742377)*x_ref[0]**o+(0.111377096886)*x_ref[0]+(0.518621968579)*x_ref[1]**o+(-0.264799841387)*x_ref[1]
        ref[3,0]=(0.661089560632)*x_ref[0]**o+(-0.175284355246)*x_ref[0]+(0.434737404117)*x_ref[1]**o+(-0.569800530193)*x_ref[1]
        ref[3,1]=(-0.581369919535)*x_ref[0]**o+(-0.599162778688)*x_ref[0]+(-0.772449383214)*x_ref[1]**o+(-0.286495247533)*x_ref[1]
        ref[3,2]=(0.350919520995)*x_ref[0]**o+(0.2325739414)*x_ref[0]+(0.744285468619)*x_ref[1]**o+(-0.603444606663)*x_ref[1]
        ref[3,3]=(-0.946270717928)*x_ref[0]**o+(0.944211313875)*x_ref[0]+(0.128830208798)*x_ref[1]**o+(-0.108311954532)*x_ref[1]
        ref[3,4]=(-0.0704013779853)*x_ref[0]**o+(-0.334719508353)*x_ref[0]+(0.39364248466)*x_ref[1]**o+(-0.894987529202)*x_ref[1]
      else:
        s[0,0]=(0.99056578049)*x[0]**o+(0.509128967554)*x[0]+(0.171080190069)*x[1]**o+(0.522380534005)*x[1]+(0.431016370389)*x[2]**o+(-0.993934787463)*x[2]
        s[0,1]=(0.440865158692)*x[0]**o+(0.469110688262)*x[0]+(-0.44415137348)*x[1]**o+(-0.592580419437)*x[1]+(0.725915122511)*x[2]**o+(0.978265572749)*x[2]
        s[0,2]=(0.759448293392)*x[0]**o+(-0.673851848255)*x[0]+(-0.850110053326)*x[1]**o+(0.542563968866)*x[1]+(-0.593647236451)*x[2]**o+(0.238446783033)*x[2]
        s[0,3]=(0.124244333648)*x[0]**o+(-0.72755263687)*x[0]+(-0.694167850492)*x[1]**o+(-0.613367096258)*x[1]+(0.750711571202)*x[2]**o+(0.931327127719)*x[2]
        s[0,4]=(-0.501005188112)*x[0]**o+(-0.844862164905)*x[0]+(-0.771531270162)*x[1]**o+(-0.379614684646)*x[1]+(0.668772926463)*x[2]**o+(0.452222970598)*x[2]
        s[1,0]=(-0.903660331121)*x[0]**o+(0.885970144397)*x[0]+(-0.589427430828)*x[1]**o+(0.149545558395)*x[1]+(-0.672063110451)*x[2]**o+(0.809659958716)*x[2]
        s[1,1]=(0.443858030252)*x[0]**o+(0.268545734898)*x[0]+(-0.723221753395)*x[1]**o+(-0.497164094332)*x[1]+(-0.38727254966)*x[2]**o+(-0.713966847727)*x[2]
        s[1,2]=(-0.141155739442)*x[0]**o+(0.147814163801)*x[0]+(0.30705733457)*x[1]**o+(0.671290544062)*x[1]+(-0.48022640662)*x[2]**o+(-0.258276084908)*x[2]
        s[1,3]=(-0.23260749408)*x[0]**o+(0.685806077106)*x[0]+(-0.802672585899)*x[1]**o+(0.220698760546)*x[1]+(-0.930587997597)*x[2]**o+(0.131923598812)*x[2]
        s[1,4]=(-0.0116512039336)*x[0]**o+(0.949770706957)*x[0]+(-0.315276290308)*x[1]**o+(0.385326039063)*x[1]+(0.508964415622)*x[2]**o+(0.813022533544)*x[2]
        s[2,0]=(0.214620768745)*x[0]**o+(-0.378543435195)*x[0]+(-0.415588213697)*x[1]**o+(0.390408636588)*x[1]+(0.89210598718)*x[2]**o+(0.649105001473)*x[2]
        s[2,1]=(-0.9290010077)*x[0]**o+(-0.819789531059)*x[0]+(-0.31733087944)*x[1]**o+(0.18679256152)*x[1]+(-0.595489801854)*x[2]**o+(0.376682692017)*x[2]
        s[2,2]=(-0.396100840781)*x[0]**o+(-0.962832400773)*x[0]+(-0.97170822186)*x[1]**o+(0.153092378797)*x[1]+(-0.102354054108)*x[2]**o+(0.0569186680057)*x[2]
        s[2,3]=(-0.736586445243)*x[0]**o+(-0.453505210464)*x[0]+(0.0894043232592)*x[1]**o+(0.352561026525)*x[1]+(0.950731094588)*x[2]**o+(0.170774557675)*x[2]
        s[2,4]=(-0.742839390272)*x[0]**o+(0.62294122589)*x[0]+(-0.543260730767)*x[1]**o+(0.189330809244)*x[1]+(-0.603776416284)*x[2]**o+(0.875279918494)*x[2]
        s[3,0]=(-0.409012740744)*x[0]**o+(-0.245267437205)*x[0]+(0.818223917843)*x[1]**o+(-0.962578887383)*x[1]+(-0.51108067434)*x[2]**o+(-0.0692224100799)*x[2]
        s[3,1]=(0.701052191719)*x[0]**o+(-0.611563296923)*x[0]+(0.584279125155)*x[1]**o+(0.324940331516)*x[1]+(-0.0474533585109)*x[2]**o+(-0.226269391305)*x[2]
        s[3,2]=(0.601966725574)*x[0]**o+(0.698971273869)*x[0]+(0.832499644008)*x[1]**o+(-0.680484279276)*x[1]+(0.557754387767)*x[2]**o+(0.589582740674)*x[2]
        s[3,3]=(0.555182612378)*x[0]**o+(-0.548416585377)*x[0]+(0.171087446868)*x[1]**o+(0.252969734012)*x[1]+(-0.291200177721)*x[2]**o+(0.591022017095)*x[2]
        s[3,4]=(0.358263224328)*x[0]**o+(0.618229036593)*x[0]+(-0.853797452875)*x[1]**o+(-0.689076935276)*x[1]+(-0.705975774167)*x[2]**o+(-0.767101969914)*x[2]
        ref[0,0]=(0.99056578049)*x_ref[0]**o+(0.509128967554)*x_ref[0]+(0.171080190069)*x_ref[1]**o+(0.522380534005)*x_ref[1]+(0.431016370389)*x_ref[2]**o+(-0.993934787463)*x_ref[2]
        ref[0,1]=(0.440865158692)*x_ref[0]**o+(0.469110688262)*x_ref[0]+(-0.44415137348)*x_ref[1]**o+(-0.592580419437)*x_ref[1]+(0.725915122511)*x_ref[2]**o+(0.978265572749)*x_ref[2]
        ref[0,2]=(0.759448293392)*x_ref[0]**o+(-0.673851848255)*x_ref[0]+(-0.850110053326)*x_ref[1]**o+(0.542563968866)*x_ref[1]+(-0.593647236451)*x_ref[2]**o+(0.238446783033)*x_ref[2]
        ref[0,3]=(0.124244333648)*x_ref[0]**o+(-0.72755263687)*x_ref[0]+(-0.694167850492)*x_ref[1]**o+(-0.613367096258)*x_ref[1]+(0.750711571202)*x_ref[2]**o+(0.931327127719)*x_ref[2]
        ref[0,4]=(-0.501005188112)*x_ref[0]**o+(-0.844862164905)*x_ref[0]+(-0.771531270162)*x_ref[1]**o+(-0.379614684646)*x_ref[1]+(0.668772926463)*x_ref[2]**o+(0.452222970598)*x_ref[2]
        ref[1,0]=(-0.903660331121)*x_ref[0]**o+(0.885970144397)*x_ref[0]+(-0.589427430828)*x_ref[1]**o+(0.149545558395)*x_ref[1]+(-0.672063110451)*x_ref[2]**o+(0.809659958716)*x_ref[2]
        ref[1,1]=(0.443858030252)*x_ref[0]**o+(0.268545734898)*x_ref[0]+(-0.723221753395)*x_ref[1]**o+(-0.497164094332)*x_ref[1]+(-0.38727254966)*x_ref[2]**o+(-0.713966847727)*x_ref[2]
        ref[1,2]=(-0.141155739442)*x_ref[0]**o+(0.147814163801)*x_ref[0]+(0.30705733457)*x_ref[1]**o+(0.671290544062)*x_ref[1]+(-0.48022640662)*x_ref[2]**o+(-0.258276084908)*x_ref[2]
        ref[1,3]=(-0.23260749408)*x_ref[0]**o+(0.685806077106)*x_ref[0]+(-0.802672585899)*x_ref[1]**o+(0.220698760546)*x_ref[1]+(-0.930587997597)*x_ref[2]**o+(0.131923598812)*x_ref[2]
        ref[1,4]=(-0.0116512039336)*x_ref[0]**o+(0.949770706957)*x_ref[0]+(-0.315276290308)*x_ref[1]**o+(0.385326039063)*x_ref[1]+(0.508964415622)*x_ref[2]**o+(0.813022533544)*x_ref[2]
        ref[2,0]=(0.214620768745)*x_ref[0]**o+(-0.378543435195)*x_ref[0]+(-0.415588213697)*x_ref[1]**o+(0.390408636588)*x_ref[1]+(0.89210598718)*x_ref[2]**o+(0.649105001473)*x_ref[2]
        ref[2,1]=(-0.9290010077)*x_ref[0]**o+(-0.819789531059)*x_ref[0]+(-0.31733087944)*x_ref[1]**o+(0.18679256152)*x_ref[1]+(-0.595489801854)*x_ref[2]**o+(0.376682692017)*x_ref[2]
        ref[2,2]=(-0.396100840781)*x_ref[0]**o+(-0.962832400773)*x_ref[0]+(-0.97170822186)*x_ref[1]**o+(0.153092378797)*x_ref[1]+(-0.102354054108)*x_ref[2]**o+(0.0569186680057)*x_ref[2]
        ref[2,3]=(-0.736586445243)*x_ref[0]**o+(-0.453505210464)*x_ref[0]+(0.0894043232592)*x_ref[1]**o+(0.352561026525)*x_ref[1]+(0.950731094588)*x_ref[2]**o+(0.170774557675)*x_ref[2]
        ref[2,4]=(-0.742839390272)*x_ref[0]**o+(0.62294122589)*x_ref[0]+(-0.543260730767)*x_ref[1]**o+(0.189330809244)*x_ref[1]+(-0.603776416284)*x_ref[2]**o+(0.875279918494)*x_ref[2]
        ref[3,0]=(-0.409012740744)*x_ref[0]**o+(-0.245267437205)*x_ref[0]+(0.818223917843)*x_ref[1]**o+(-0.962578887383)*x_ref[1]+(-0.51108067434)*x_ref[2]**o+(-0.0692224100799)*x_ref[2]
        ref[3,1]=(0.701052191719)*x_ref[0]**o+(-0.611563296923)*x_ref[0]+(0.584279125155)*x_ref[1]**o+(0.324940331516)*x_ref[1]+(-0.0474533585109)*x_ref[2]**o+(-0.226269391305)*x_ref[2]
        ref[3,2]=(0.601966725574)*x_ref[0]**o+(0.698971273869)*x_ref[0]+(0.832499644008)*x_ref[1]**o+(-0.680484279276)*x_ref[1]+(0.557754387767)*x_ref[2]**o+(0.589582740674)*x_ref[2]
        ref[3,3]=(0.555182612378)*x_ref[0]**o+(-0.548416585377)*x_ref[0]+(0.171087446868)*x_ref[1]**o+(0.252969734012)*x_ref[1]+(-0.291200177721)*x_ref[2]**o+(0.591022017095)*x_ref[2]
        ref[3,4]=(0.358263224328)*x_ref[0]**o+(0.618229036593)*x_ref[0]+(-0.853797452875)*x_ref[1]**o+(-0.689076935276)*x_ref[1]+(-0.705975774167)*x_ref[2]**o+(-0.767101969914)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromSymbol_Solution_rank3(self):
      """
      tests interpolate for rank 3 Symbol onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from Solution onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(6, 2, 2),dim=dim)
      s=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2),w_ref)
      if dim==2:
        s[0,0,0]=(0.551903191753)*x[0]**o+(0.134167484401)*x[0]+(-0.970029226665)*x[1]**o+(-0.505769142138)*x[1]
        s[0,0,1]=(0.527481367323)*x[0]**o+(0.383023560586)*x[0]+(0.534661490175)*x[1]**o+(-0.454003082098)*x[1]
        s[0,1,0]=(-0.532872138974)*x[0]**o+(-0.858154622463)*x[0]+(-0.334227178563)*x[1]**o+(0.818853150256)*x[1]
        s[0,1,1]=(-0.402982114556)*x[0]**o+(-0.415574274216)*x[0]+(-0.41876444792)*x[1]**o+(0.944724142146)*x[1]
        s[1,0,0]=(0.10201141991)*x[0]**o+(-0.524044051877)*x[0]+(-0.143688302936)*x[1]**o+(0.669583775819)*x[1]
        s[1,0,1]=(-0.207189538013)*x[0]**o+(-0.293524466257)*x[0]+(0.742423576418)*x[1]**o+(0.698550598519)*x[1]
        s[1,1,0]=(0.589159582266)*x[0]**o+(-0.118585184595)*x[0]+(-0.289302989425)*x[1]**o+(0.408124429254)*x[1]
        s[1,1,1]=(0.432211891066)*x[0]**o+(0.606915869583)*x[0]+(0.945265653758)*x[1]**o+(-0.555752925873)*x[1]
        s[2,0,0]=(-0.0432578004806)*x[0]**o+(0.536208710346)*x[0]+(-0.765816194908)*x[1]**o+(0.0558566372534)*x[1]
        s[2,0,1]=(0.596807480612)*x[0]**o+(-0.0117541181841)*x[0]+(-0.167126829104)*x[1]**o+(0.810853013382)*x[1]
        s[2,1,0]=(0.1865308646)*x[0]**o+(-0.35633893825)*x[0]+(0.0578054987569)*x[1]**o+(0.454879409627)*x[1]
        s[2,1,1]=(0.977442292262)*x[0]**o+(-0.639419194896)*x[0]+(0.660710610339)*x[1]**o+(0.737541315152)*x[1]
        s[3,0,0]=(-0.448512485148)*x[0]**o+(0.0195770337028)*x[0]+(0.476589731682)*x[1]**o+(0.252822613388)*x[1]
        s[3,0,1]=(0.168702994111)*x[0]**o+(-0.262429046534)*x[0]+(-0.336163311284)*x[1]**o+(-0.496715703947)*x[1]
        s[3,1,0]=(0.527929126799)*x[0]**o+(-0.524648338276)*x[0]+(-0.639355259144)*x[1]**o+(0.944748719208)*x[1]
        s[3,1,1]=(0.544347152911)*x[0]**o+(0.411148837164)*x[0]+(-0.0708716357747)*x[1]**o+(0.83241698331)*x[1]
        s[4,0,0]=(-0.633639608573)*x[0]**o+(-0.956364543284)*x[0]+(-0.785908625875)*x[1]**o+(0.930061167858)*x[1]
        s[4,0,1]=(0.609309892405)*x[0]**o+(0.230873371053)*x[0]+(0.787017185481)*x[1]**o+(0.198757600328)*x[1]
        s[4,1,0]=(-0.697537179974)*x[0]**o+(-0.0523509220171)*x[0]+(0.279820592879)*x[1]**o+(0.763446870577)*x[1]
        s[4,1,1]=(-0.489219146209)*x[0]**o+(-0.145758299279)*x[0]+(0.565938821575)*x[1]**o+(-0.98703168748)*x[1]
        s[5,0,0]=(0.0839742626164)*x[0]**o+(0.195476369019)*x[0]+(-0.856490669921)*x[1]**o+(-0.471313686326)*x[1]
        s[5,0,1]=(-0.612045283736)*x[0]**o+(0.9330978976)*x[0]+(0.554623000698)*x[1]**o+(-0.0447734703966)*x[1]
        s[5,1,0]=(0.137284580652)*x[0]**o+(-0.576384951707)*x[0]+(-0.18207486832)*x[1]**o+(0.581595310151)*x[1]
        s[5,1,1]=(0.0169341997249)*x[0]**o+(0.0681222507261)*x[0]+(0.0252651671358)*x[1]**o+(-0.884122676141)*x[1]
        ref[0,0,0]=(0.551903191753)*x_ref[0]**o+(0.134167484401)*x_ref[0]+(-0.970029226665)*x_ref[1]**o+(-0.505769142138)*x_ref[1]
        ref[0,0,1]=(0.527481367323)*x_ref[0]**o+(0.383023560586)*x_ref[0]+(0.534661490175)*x_ref[1]**o+(-0.454003082098)*x_ref[1]
        ref[0,1,0]=(-0.532872138974)*x_ref[0]**o+(-0.858154622463)*x_ref[0]+(-0.334227178563)*x_ref[1]**o+(0.818853150256)*x_ref[1]
        ref[0,1,1]=(-0.402982114556)*x_ref[0]**o+(-0.415574274216)*x_ref[0]+(-0.41876444792)*x_ref[1]**o+(0.944724142146)*x_ref[1]
        ref[1,0,0]=(0.10201141991)*x_ref[0]**o+(-0.524044051877)*x_ref[0]+(-0.143688302936)*x_ref[1]**o+(0.669583775819)*x_ref[1]
        ref[1,0,1]=(-0.207189538013)*x_ref[0]**o+(-0.293524466257)*x_ref[0]+(0.742423576418)*x_ref[1]**o+(0.698550598519)*x_ref[1]
        ref[1,1,0]=(0.589159582266)*x_ref[0]**o+(-0.118585184595)*x_ref[0]+(-0.289302989425)*x_ref[1]**o+(0.408124429254)*x_ref[1]
        ref[1,1,1]=(0.432211891066)*x_ref[0]**o+(0.606915869583)*x_ref[0]+(0.945265653758)*x_ref[1]**o+(-0.555752925873)*x_ref[1]
        ref[2,0,0]=(-0.0432578004806)*x_ref[0]**o+(0.536208710346)*x_ref[0]+(-0.765816194908)*x_ref[1]**o+(0.0558566372534)*x_ref[1]
        ref[2,0,1]=(0.596807480612)*x_ref[0]**o+(-0.0117541181841)*x_ref[0]+(-0.167126829104)*x_ref[1]**o+(0.810853013382)*x_ref[1]
        ref[2,1,0]=(0.1865308646)*x_ref[0]**o+(-0.35633893825)*x_ref[0]+(0.0578054987569)*x_ref[1]**o+(0.454879409627)*x_ref[1]
        ref[2,1,1]=(0.977442292262)*x_ref[0]**o+(-0.639419194896)*x_ref[0]+(0.660710610339)*x_ref[1]**o+(0.737541315152)*x_ref[1]
        ref[3,0,0]=(-0.448512485148)*x_ref[0]**o+(0.0195770337028)*x_ref[0]+(0.476589731682)*x_ref[1]**o+(0.252822613388)*x_ref[1]
        ref[3,0,1]=(0.168702994111)*x_ref[0]**o+(-0.262429046534)*x_ref[0]+(-0.336163311284)*x_ref[1]**o+(-0.496715703947)*x_ref[1]
        ref[3,1,0]=(0.527929126799)*x_ref[0]**o+(-0.524648338276)*x_ref[0]+(-0.639355259144)*x_ref[1]**o+(0.944748719208)*x_ref[1]
        ref[3,1,1]=(0.544347152911)*x_ref[0]**o+(0.411148837164)*x_ref[0]+(-0.0708716357747)*x_ref[1]**o+(0.83241698331)*x_ref[1]
        ref[4,0,0]=(-0.633639608573)*x_ref[0]**o+(-0.956364543284)*x_ref[0]+(-0.785908625875)*x_ref[1]**o+(0.930061167858)*x_ref[1]
        ref[4,0,1]=(0.609309892405)*x_ref[0]**o+(0.230873371053)*x_ref[0]+(0.787017185481)*x_ref[1]**o+(0.198757600328)*x_ref[1]
        ref[4,1,0]=(-0.697537179974)*x_ref[0]**o+(-0.0523509220171)*x_ref[0]+(0.279820592879)*x_ref[1]**o+(0.763446870577)*x_ref[1]
        ref[4,1,1]=(-0.489219146209)*x_ref[0]**o+(-0.145758299279)*x_ref[0]+(0.565938821575)*x_ref[1]**o+(-0.98703168748)*x_ref[1]
        ref[5,0,0]=(0.0839742626164)*x_ref[0]**o+(0.195476369019)*x_ref[0]+(-0.856490669921)*x_ref[1]**o+(-0.471313686326)*x_ref[1]
        ref[5,0,1]=(-0.612045283736)*x_ref[0]**o+(0.9330978976)*x_ref[0]+(0.554623000698)*x_ref[1]**o+(-0.0447734703966)*x_ref[1]
        ref[5,1,0]=(0.137284580652)*x_ref[0]**o+(-0.576384951707)*x_ref[0]+(-0.18207486832)*x_ref[1]**o+(0.581595310151)*x_ref[1]
        ref[5,1,1]=(0.0169341997249)*x_ref[0]**o+(0.0681222507261)*x_ref[0]+(0.0252651671358)*x_ref[1]**o+(-0.884122676141)*x_ref[1]
      else:
        s[0,0,0]=(0.692580506423)*x[0]**o+(-0.739196213794)*x[0]+(-0.754727396858)*x[1]**o+(0.85754978429)*x[1]+(0.320776017021)*x[2]**o+(0.496406747735)*x[2]
        s[0,0,1]=(-0.368178763519)*x[0]**o+(0.443073964944)*x[0]+(-0.441300717299)*x[1]**o+(-0.642365449833)*x[1]+(-0.411121382093)*x[2]**o+(0.298351368421)*x[2]
        s[0,1,0]=(0.124028838378)*x[0]**o+(0.364040911128)*x[0]+(-0.816006030587)*x[1]**o+(-0.342165428091)*x[1]+(-0.565296255652)*x[2]**o+(0.10324169783)*x[2]
        s[0,1,1]=(-0.310115517857)*x[0]**o+(0.408326426387)*x[0]+(0.522912752025)*x[1]**o+(0.637782282686)*x[1]+(-0.386389984613)*x[2]**o+(0.566324706521)*x[2]
        s[1,0,0]=(-0.154331234746)*x[0]**o+(-0.436808593217)*x[0]+(0.00168591908065)*x[1]**o+(-0.786939679968)*x[1]+(-0.914492955074)*x[2]**o+(0.268713986856)*x[2]
        s[1,0,1]=(-0.583647316282)*x[0]**o+(0.893582962788)*x[0]+(0.764771722881)*x[1]**o+(-0.358865375935)*x[1]+(-0.157963402459)*x[2]**o+(-0.606181853567)*x[2]
        s[1,1,0]=(-0.655042363999)*x[0]**o+(0.746261777599)*x[0]+(-0.263881826688)*x[1]**o+(0.996006722883)*x[1]+(0.363648087969)*x[2]**o+(-0.448676062094)*x[2]
        s[1,1,1]=(0.123310902924)*x[0]**o+(0.18934896301)*x[0]+(0.520444019722)*x[1]**o+(0.163046210167)*x[1]+(0.191271174203)*x[2]**o+(-0.603399653729)*x[2]
        s[2,0,0]=(0.556843419598)*x[0]**o+(0.352145721987)*x[0]+(0.311744047933)*x[1]**o+(0.393374763098)*x[1]+(-0.411522712826)*x[2]**o+(-0.857749946733)*x[2]
        s[2,0,1]=(0.716861034223)*x[0]**o+(0.574178184871)*x[0]+(0.0537394109557)*x[1]**o+(0.468155056849)*x[1]+(0.579460976425)*x[2]**o+(0.699492008017)*x[2]
        s[2,1,0]=(-0.93398841558)*x[0]**o+(-0.697231862042)*x[0]+(0.109026477391)*x[1]**o+(0.634335059223)*x[1]+(-0.960274632681)*x[2]**o+(0.111578852692)*x[2]
        s[2,1,1]=(-0.183177179838)*x[0]**o+(0.507798545721)*x[0]+(0.762549005034)*x[1]**o+(0.135498023818)*x[1]+(-0.766065132764)*x[2]**o+(-0.622877452314)*x[2]
        s[3,0,0]=(0.873113007323)*x[0]**o+(-0.140853976021)*x[0]+(0.784413539889)*x[1]**o+(-0.488650797745)*x[1]+(0.452977221701)*x[2]**o+(0.913380695175)*x[2]
        s[3,0,1]=(0.282641502309)*x[0]**o+(-0.340827819728)*x[0]+(-0.54633943683)*x[1]**o+(-0.121403331732)*x[1]+(-0.915298514083)*x[2]**o+(-0.226945838638)*x[2]
        s[3,1,0]=(0.697235623378)*x[0]**o+(-0.124366296878)*x[0]+(-0.402714117894)*x[1]**o+(0.847633014733)*x[1]+(-0.112276438729)*x[2]**o+(-0.563344547848)*x[2]
        s[3,1,1]=(0.029600044738)*x[0]**o+(0.410269123343)*x[0]+(-0.92058774924)*x[1]**o+(-0.764026992207)*x[1]+(0.941619693688)*x[2]**o+(0.59508687872)*x[2]
        s[4,0,0]=(-0.782484223278)*x[0]**o+(-0.463642056406)*x[0]+(0.693714378966)*x[1]**o+(-0.409388955972)*x[1]+(-0.684017609615)*x[2]**o+(0.212037484357)*x[2]
        s[4,0,1]=(-0.507499040314)*x[0]**o+(0.628756911098)*x[0]+(0.582941469574)*x[1]**o+(-0.907597519745)*x[1]+(-0.674162640518)*x[2]**o+(0.343602419355)*x[2]
        s[4,1,0]=(0.718884212471)*x[0]**o+(-0.773542451968)*x[0]+(-0.558458416647)*x[1]**o+(0.246124177952)*x[1]+(0.301868653091)*x[2]**o+(-0.747134856013)*x[2]
        s[4,1,1]=(-0.826106273057)*x[0]**o+(0.810523783505)*x[0]+(-0.654586275299)*x[1]**o+(-0.616385894964)*x[1]+(0.254151903925)*x[2]**o+(0.890878608639)*x[2]
        s[5,0,0]=(0.674499531922)*x[0]**o+(-0.780390926945)*x[0]+(0.950129766744)*x[1]**o+(-0.60369977837)*x[1]+(-0.012121074813)*x[2]**o+(0.281422122495)*x[2]
        s[5,0,1]=(0.834522181864)*x[0]**o+(0.594872843083)*x[0]+(-0.921495736486)*x[1]**o+(0.873833748135)*x[1]+(-0.683068734294)*x[2]**o+(-0.234163027333)*x[2]
        s[5,1,0]=(-0.762343975994)*x[0]**o+(0.328522804445)*x[0]+(-0.684121978071)*x[1]**o+(0.240663011598)*x[1]+(0.781441947029)*x[2]**o+(0.63503560345)*x[2]
        s[5,1,1]=(-0.967543985688)*x[0]**o+(-0.141323317442)*x[0]+(-0.306422683038)*x[1]**o+(-0.935081886909)*x[1]+(-0.142084729038)*x[2]**o+(0.668337592761)*x[2]
        ref[0,0,0]=(0.692580506423)*x_ref[0]**o+(-0.739196213794)*x_ref[0]+(-0.754727396858)*x_ref[1]**o+(0.85754978429)*x_ref[1]+(0.320776017021)*x_ref[2]**o+(0.496406747735)*x_ref[2]
        ref[0,0,1]=(-0.368178763519)*x_ref[0]**o+(0.443073964944)*x_ref[0]+(-0.441300717299)*x_ref[1]**o+(-0.642365449833)*x_ref[1]+(-0.411121382093)*x_ref[2]**o+(0.298351368421)*x_ref[2]
        ref[0,1,0]=(0.124028838378)*x_ref[0]**o+(0.364040911128)*x_ref[0]+(-0.816006030587)*x_ref[1]**o+(-0.342165428091)*x_ref[1]+(-0.565296255652)*x_ref[2]**o+(0.10324169783)*x_ref[2]
        ref[0,1,1]=(-0.310115517857)*x_ref[0]**o+(0.408326426387)*x_ref[0]+(0.522912752025)*x_ref[1]**o+(0.637782282686)*x_ref[1]+(-0.386389984613)*x_ref[2]**o+(0.566324706521)*x_ref[2]
        ref[1,0,0]=(-0.154331234746)*x_ref[0]**o+(-0.436808593217)*x_ref[0]+(0.00168591908065)*x_ref[1]**o+(-0.786939679968)*x_ref[1]+(-0.914492955074)*x_ref[2]**o+(0.268713986856)*x_ref[2]
        ref[1,0,1]=(-0.583647316282)*x_ref[0]**o+(0.893582962788)*x_ref[0]+(0.764771722881)*x_ref[1]**o+(-0.358865375935)*x_ref[1]+(-0.157963402459)*x_ref[2]**o+(-0.606181853567)*x_ref[2]
        ref[1,1,0]=(-0.655042363999)*x_ref[0]**o+(0.746261777599)*x_ref[0]+(-0.263881826688)*x_ref[1]**o+(0.996006722883)*x_ref[1]+(0.363648087969)*x_ref[2]**o+(-0.448676062094)*x_ref[2]
        ref[1,1,1]=(0.123310902924)*x_ref[0]**o+(0.18934896301)*x_ref[0]+(0.520444019722)*x_ref[1]**o+(0.163046210167)*x_ref[1]+(0.191271174203)*x_ref[2]**o+(-0.603399653729)*x_ref[2]
        ref[2,0,0]=(0.556843419598)*x_ref[0]**o+(0.352145721987)*x_ref[0]+(0.311744047933)*x_ref[1]**o+(0.393374763098)*x_ref[1]+(-0.411522712826)*x_ref[2]**o+(-0.857749946733)*x_ref[2]
        ref[2,0,1]=(0.716861034223)*x_ref[0]**o+(0.574178184871)*x_ref[0]+(0.0537394109557)*x_ref[1]**o+(0.468155056849)*x_ref[1]+(0.579460976425)*x_ref[2]**o+(0.699492008017)*x_ref[2]
        ref[2,1,0]=(-0.93398841558)*x_ref[0]**o+(-0.697231862042)*x_ref[0]+(0.109026477391)*x_ref[1]**o+(0.634335059223)*x_ref[1]+(-0.960274632681)*x_ref[2]**o+(0.111578852692)*x_ref[2]
        ref[2,1,1]=(-0.183177179838)*x_ref[0]**o+(0.507798545721)*x_ref[0]+(0.762549005034)*x_ref[1]**o+(0.135498023818)*x_ref[1]+(-0.766065132764)*x_ref[2]**o+(-0.622877452314)*x_ref[2]
        ref[3,0,0]=(0.873113007323)*x_ref[0]**o+(-0.140853976021)*x_ref[0]+(0.784413539889)*x_ref[1]**o+(-0.488650797745)*x_ref[1]+(0.452977221701)*x_ref[2]**o+(0.913380695175)*x_ref[2]
        ref[3,0,1]=(0.282641502309)*x_ref[0]**o+(-0.340827819728)*x_ref[0]+(-0.54633943683)*x_ref[1]**o+(-0.121403331732)*x_ref[1]+(-0.915298514083)*x_ref[2]**o+(-0.226945838638)*x_ref[2]
        ref[3,1,0]=(0.697235623378)*x_ref[0]**o+(-0.124366296878)*x_ref[0]+(-0.402714117894)*x_ref[1]**o+(0.847633014733)*x_ref[1]+(-0.112276438729)*x_ref[2]**o+(-0.563344547848)*x_ref[2]
        ref[3,1,1]=(0.029600044738)*x_ref[0]**o+(0.410269123343)*x_ref[0]+(-0.92058774924)*x_ref[1]**o+(-0.764026992207)*x_ref[1]+(0.941619693688)*x_ref[2]**o+(0.59508687872)*x_ref[2]
        ref[4,0,0]=(-0.782484223278)*x_ref[0]**o+(-0.463642056406)*x_ref[0]+(0.693714378966)*x_ref[1]**o+(-0.409388955972)*x_ref[1]+(-0.684017609615)*x_ref[2]**o+(0.212037484357)*x_ref[2]
        ref[4,0,1]=(-0.507499040314)*x_ref[0]**o+(0.628756911098)*x_ref[0]+(0.582941469574)*x_ref[1]**o+(-0.907597519745)*x_ref[1]+(-0.674162640518)*x_ref[2]**o+(0.343602419355)*x_ref[2]
        ref[4,1,0]=(0.718884212471)*x_ref[0]**o+(-0.773542451968)*x_ref[0]+(-0.558458416647)*x_ref[1]**o+(0.246124177952)*x_ref[1]+(0.301868653091)*x_ref[2]**o+(-0.747134856013)*x_ref[2]
        ref[4,1,1]=(-0.826106273057)*x_ref[0]**o+(0.810523783505)*x_ref[0]+(-0.654586275299)*x_ref[1]**o+(-0.616385894964)*x_ref[1]+(0.254151903925)*x_ref[2]**o+(0.890878608639)*x_ref[2]
        ref[5,0,0]=(0.674499531922)*x_ref[0]**o+(-0.780390926945)*x_ref[0]+(0.950129766744)*x_ref[1]**o+(-0.60369977837)*x_ref[1]+(-0.012121074813)*x_ref[2]**o+(0.281422122495)*x_ref[2]
        ref[5,0,1]=(0.834522181864)*x_ref[0]**o+(0.594872843083)*x_ref[0]+(-0.921495736486)*x_ref[1]**o+(0.873833748135)*x_ref[1]+(-0.683068734294)*x_ref[2]**o+(-0.234163027333)*x_ref[2]
        ref[5,1,0]=(-0.762343975994)*x_ref[0]**o+(0.328522804445)*x_ref[0]+(-0.684121978071)*x_ref[1]**o+(0.240663011598)*x_ref[1]+(0.781441947029)*x_ref[2]**o+(0.63503560345)*x_ref[2]
        ref[5,1,1]=(-0.967543985688)*x_ref[0]**o+(-0.141323317442)*x_ref[0]+(-0.306422683038)*x_ref[1]**o+(-0.935081886909)*x_ref[1]+(-0.142084729038)*x_ref[2]**o+(0.668337592761)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromSymbol_Solution_rank4(self):
      """
      tests interpolate for rank 4 Symbol onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from Solution onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5, 3, 2),dim=dim)
      s=Data(0,(4, 5, 3, 2),w)
      ref=Data(0,(4, 5, 3, 2),w_ref)
      if dim==2:
        s[0,0,0,0]=(-0.547754951171)*x[0]**o+(0.356610809044)*x[0]+(0.452044983539)*x[1]**o+(-0.821341147164)*x[1]
        s[0,0,0,1]=(0.509267661384)*x[0]**o+(0.667069615676)*x[0]+(0.344083346522)*x[1]**o+(-0.281070233626)*x[1]
        s[0,0,1,0]=(0.910153339127)*x[0]**o+(-0.134026992189)*x[0]+(0.283156493736)*x[1]**o+(-0.236635156141)*x[1]
        s[0,0,1,1]=(-0.486341299233)*x[0]**o+(0.969619867206)*x[0]+(0.145365694427)*x[1]**o+(0.395899354311)*x[1]
        s[0,0,2,0]=(-0.683524112594)*x[0]**o+(-0.191053076108)*x[0]+(-0.156617335102)*x[1]**o+(-0.908072509242)*x[1]
        s[0,0,2,1]=(-0.329288530406)*x[0]**o+(0.861655757717)*x[0]+(-0.423717545177)*x[1]**o+(0.357896568135)*x[1]
        s[0,1,0,0]=(0.454682074738)*x[0]**o+(0.242640789048)*x[0]+(-0.465273320791)*x[1]**o+(-0.523776617273)*x[1]
        s[0,1,0,1]=(0.299812377446)*x[0]**o+(0.258743885127)*x[0]+(-0.545559145397)*x[1]**o+(-0.703973878214)*x[1]
        s[0,1,1,0]=(-0.240052819623)*x[0]**o+(-0.648241629125)*x[0]+(0.550226821237)*x[1]**o+(0.586254427297)*x[1]
        s[0,1,1,1]=(0.232594672088)*x[0]**o+(-0.082742546667)*x[0]+(0.695512543415)*x[1]**o+(0.695609364591)*x[1]
        s[0,1,2,0]=(-0.337129430207)*x[0]**o+(-0.0996108162262)*x[0]+(0.520279135444)*x[1]**o+(-0.0419680101154)*x[1]
        s[0,1,2,1]=(0.107050704386)*x[0]**o+(-0.0108163145609)*x[0]+(-0.555986163214)*x[1]**o+(-0.264372389969)*x[1]
        s[0,2,0,0]=(0.558185714977)*x[0]**o+(0.718930754697)*x[0]+(0.661967682247)*x[1]**o+(0.968289790196)*x[1]
        s[0,2,0,1]=(-0.885308413126)*x[0]**o+(-0.000196750280455)*x[0]+(0.162337198564)*x[1]**o+(-0.372085076741)*x[1]
        s[0,2,1,0]=(0.732654846787)*x[0]**o+(0.490322343819)*x[0]+(0.253518225543)*x[1]**o+(0.755915810471)*x[1]
        s[0,2,1,1]=(0.686091929471)*x[0]**o+(0.968804203166)*x[0]+(-0.874263118503)*x[1]**o+(0.792892684533)*x[1]
        s[0,2,2,0]=(-0.538165611113)*x[0]**o+(0.435640729152)*x[0]+(0.53052383117)*x[1]**o+(-0.152917182113)*x[1]
        s[0,2,2,1]=(0.959376322991)*x[0]**o+(0.295428309997)*x[0]+(0.774755280624)*x[1]**o+(0.699112429694)*x[1]
        s[0,3,0,0]=(-0.879327491632)*x[0]**o+(-0.210915363483)*x[0]+(0.677720758543)*x[1]**o+(-0.156843338639)*x[1]
        s[0,3,0,1]=(-0.877749774331)*x[0]**o+(-0.0590688859445)*x[0]+(0.372427864404)*x[1]**o+(0.865604489942)*x[1]
        s[0,3,1,0]=(-0.479636139873)*x[0]**o+(-0.2440095302)*x[0]+(0.291946443647)*x[1]**o+(-0.446912115557)*x[1]
        s[0,3,1,1]=(-0.666968887345)*x[0]**o+(-0.530382039479)*x[0]+(-0.954360470015)*x[1]**o+(0.827870552445)*x[1]
        s[0,3,2,0]=(-0.697008931895)*x[0]**o+(-0.506024200287)*x[0]+(-0.284218723212)*x[1]**o+(-0.12154006973)*x[1]
        s[0,3,2,1]=(0.182079912323)*x[0]**o+(0.128753570425)*x[0]+(-0.610041297244)*x[1]**o+(0.167312633663)*x[1]
        s[0,4,0,0]=(0.905082824851)*x[0]**o+(-0.850946080497)*x[0]+(-0.234019076337)*x[1]**o+(-0.746004283581)*x[1]
        s[0,4,0,1]=(0.252751160265)*x[0]**o+(0.783081625822)*x[0]+(-0.18400975082)*x[1]**o+(0.729096084753)*x[1]
        s[0,4,1,0]=(-0.646745752044)*x[0]**o+(-0.95019218163)*x[0]+(-0.739473363344)*x[1]**o+(0.53887852033)*x[1]
        s[0,4,1,1]=(0.190710071596)*x[0]**o+(0.363498800572)*x[0]+(-0.166712144832)*x[1]**o+(0.383842690034)*x[1]
        s[0,4,2,0]=(-0.252312977399)*x[0]**o+(-0.181431558994)*x[0]+(-0.496223516025)*x[1]**o+(0.262053197788)*x[1]
        s[0,4,2,1]=(0.740944095813)*x[0]**o+(0.350982435679)*x[0]+(0.540773567147)*x[1]**o+(0.154742779719)*x[1]
        s[1,0,0,0]=(-0.204518679107)*x[0]**o+(0.224833545775)*x[0]+(0.701576397345)*x[1]**o+(0.955758264121)*x[1]
        s[1,0,0,1]=(0.98822877512)*x[0]**o+(-0.252532184922)*x[0]+(-0.999543053077)*x[1]**o+(0.20420747276)*x[1]
        s[1,0,1,0]=(-0.435694974363)*x[0]**o+(0.106116746803)*x[0]+(-0.614925089687)*x[1]**o+(-0.433482250787)*x[1]
        s[1,0,1,1]=(-0.967905352631)*x[0]**o+(0.733630963983)*x[0]+(0.561497813177)*x[1]**o+(-0.254199898967)*x[1]
        s[1,0,2,0]=(0.192509982098)*x[0]**o+(0.30844145669)*x[0]+(-0.537990978303)*x[1]**o+(-0.971691014587)*x[1]
        s[1,0,2,1]=(-0.642601469866)*x[0]**o+(0.828976360334)*x[0]+(-0.166247518535)*x[1]**o+(-0.394356610568)*x[1]
        s[1,1,0,0]=(-0.190023582376)*x[0]**o+(0.00308098708653)*x[0]+(0.616327311292)*x[1]**o+(0.50233410734)*x[1]
        s[1,1,0,1]=(-0.421733782097)*x[0]**o+(-0.81738567824)*x[0]+(0.0102253671566)*x[1]**o+(0.977732802241)*x[1]
        s[1,1,1,0]=(-0.932299813732)*x[0]**o+(-0.570728228193)*x[0]+(0.906998960465)*x[1]**o+(0.276967285338)*x[1]
        s[1,1,1,1]=(0.361866174533)*x[0]**o+(-0.948833321247)*x[0]+(0.0267950613442)*x[1]**o+(0.305540794357)*x[1]
        s[1,1,2,0]=(-0.107562715773)*x[0]**o+(0.884548510332)*x[0]+(-0.641236013271)*x[1]**o+(0.835011288403)*x[1]
        s[1,1,2,1]=(-0.447112184269)*x[0]**o+(0.68668033563)*x[0]+(-0.643123622354)*x[1]**o+(-0.373398061272)*x[1]
        s[1,2,0,0]=(0.566476213386)*x[0]**o+(-0.660978862942)*x[0]+(-0.250064101238)*x[1]**o+(0.0617654537116)*x[1]
        s[1,2,0,1]=(-0.0220375595157)*x[0]**o+(-0.664682506405)*x[0]+(-0.810443003238)*x[1]**o+(-0.704430911362)*x[1]
        s[1,2,1,0]=(-0.66837513872)*x[0]**o+(0.518574783899)*x[0]+(0.456254450127)*x[1]**o+(-0.0264418287977)*x[1]
        s[1,2,1,1]=(0.518371474153)*x[0]**o+(0.305863788515)*x[0]+(0.641786832956)*x[1]**o+(0.957403546249)*x[1]
        s[1,2,2,0]=(-0.702485305168)*x[0]**o+(0.810129172394)*x[0]+(0.481384977464)*x[1]**o+(-0.552283897592)*x[1]
        s[1,2,2,1]=(-0.220714257324)*x[0]**o+(0.658520871538)*x[0]+(-0.156197468389)*x[1]**o+(0.406317861261)*x[1]
        s[1,3,0,0]=(-0.833421663125)*x[0]**o+(-0.23885966657)*x[0]+(-0.0853110945734)*x[1]**o+(0.134587357668)*x[1]
        s[1,3,0,1]=(0.983003072543)*x[0]**o+(-0.444298839588)*x[0]+(-0.555821641978)*x[1]**o+(-0.934198804069)*x[1]
        s[1,3,1,0]=(-0.321902025108)*x[0]**o+(0.373296267434)*x[0]+(-0.963121149124)*x[1]**o+(0.0147111484841)*x[1]
        s[1,3,1,1]=(-0.686361701679)*x[0]**o+(0.322135914568)*x[0]+(0.0577830897835)*x[1]**o+(0.577239387766)*x[1]
        s[1,3,2,0]=(0.722944586976)*x[0]**o+(0.789741801778)*x[0]+(0.175208515485)*x[1]**o+(-0.71289312398)*x[1]
        s[1,3,2,1]=(-0.280608530342)*x[0]**o+(0.0860239950894)*x[0]+(0.695636585879)*x[1]**o+(-0.960594121577)*x[1]
        s[1,4,0,0]=(-0.852503330892)*x[0]**o+(-0.602234774377)*x[0]+(0.198091265715)*x[1]**o+(0.777503337768)*x[1]
        s[1,4,0,1]=(-0.177137751594)*x[0]**o+(-0.490189065267)*x[0]+(-0.8576459288)*x[1]**o+(-0.074100447778)*x[1]
        s[1,4,1,0]=(-0.334129920953)*x[0]**o+(-0.337245487943)*x[0]+(-0.839457764245)*x[1]**o+(-0.58152129024)*x[1]
        s[1,4,1,1]=(0.0937571716669)*x[0]**o+(0.592103688833)*x[0]+(0.15448940004)*x[1]**o+(-0.203463329576)*x[1]
        s[1,4,2,0]=(0.139741892415)*x[0]**o+(0.611539882432)*x[0]+(-0.407889172421)*x[1]**o+(0.891469847051)*x[1]
        s[1,4,2,1]=(0.95147411312)*x[0]**o+(0.0998931787148)*x[0]+(0.466902306753)*x[1]**o+(-0.718472606772)*x[1]
        s[2,0,0,0]=(-0.130008770209)*x[0]**o+(0.809915931169)*x[0]+(-0.732276845472)*x[1]**o+(-0.192903892405)*x[1]
        s[2,0,0,1]=(-0.0119712505975)*x[0]**o+(0.656694120973)*x[0]+(0.6862601083)*x[1]**o+(-0.528666986412)*x[1]
        s[2,0,1,0]=(0.28678233655)*x[0]**o+(-0.37339891868)*x[0]+(-0.946168471623)*x[1]**o+(0.762233026568)*x[1]
        s[2,0,1,1]=(0.778843713994)*x[0]**o+(0.817766330368)*x[0]+(-0.300918496467)*x[1]**o+(0.442988971838)*x[1]
        s[2,0,2,0]=(0.207449688136)*x[0]**o+(0.10275570242)*x[0]+(-0.520466147913)*x[1]**o+(0.33346010447)*x[1]
        s[2,0,2,1]=(0.709221342424)*x[0]**o+(0.159144243448)*x[0]+(0.685824812808)*x[1]**o+(0.50901941527)*x[1]
        s[2,1,0,0]=(0.0244924098009)*x[0]**o+(-0.770942809125)*x[0]+(0.372626872676)*x[1]**o+(-0.714320503926)*x[1]
        s[2,1,0,1]=(-0.105052119318)*x[0]**o+(0.573319234378)*x[0]+(0.0995762579843)*x[1]**o+(0.579088439928)*x[1]
        s[2,1,1,0]=(-0.572394543041)*x[0]**o+(0.1606374212)*x[0]+(-0.801731532404)*x[1]**o+(-0.644110219424)*x[1]
        s[2,1,1,1]=(0.914053075068)*x[0]**o+(-0.230478556116)*x[0]+(-0.00343924084847)*x[1]**o+(-0.53144169108)*x[1]
        s[2,1,2,0]=(-0.743362569446)*x[0]**o+(-0.0428045574774)*x[0]+(-0.503190273025)*x[1]**o+(0.831658185042)*x[1]
        s[2,1,2,1]=(0.176721891893)*x[0]**o+(-0.460950049697)*x[0]+(0.45677887739)*x[1]**o+(-0.853283022367)*x[1]
        s[2,2,0,0]=(-0.585337031786)*x[0]**o+(0.901183590413)*x[0]+(-0.746305431471)*x[1]**o+(0.0439043730531)*x[1]
        s[2,2,0,1]=(0.594349646931)*x[0]**o+(-0.431546660621)*x[0]+(-0.962096864047)*x[1]**o+(0.955112341273)*x[1]
        s[2,2,1,0]=(-0.857877499011)*x[0]**o+(0.803223671127)*x[0]+(-0.995806495668)*x[1]**o+(-0.267707433165)*x[1]
        s[2,2,1,1]=(0.968406110131)*x[0]**o+(-0.0208982523366)*x[0]+(-0.000406237500393)*x[1]**o+(-0.334429552696)*x[1]
        s[2,2,2,0]=(-0.960828307746)*x[0]**o+(0.235004476043)*x[0]+(0.0456041210872)*x[1]**o+(0.323144610527)*x[1]
        s[2,2,2,1]=(0.538691072674)*x[0]**o+(-0.737490055104)*x[0]+(-0.96703538134)*x[1]**o+(0.727057872325)*x[1]
        s[2,3,0,0]=(0.475319703318)*x[0]**o+(0.863857783409)*x[0]+(0.0891089574388)*x[1]**o+(0.121619153024)*x[1]
        s[2,3,0,1]=(0.68881433038)*x[0]**o+(-0.812908863657)*x[0]+(0.687129220133)*x[1]**o+(0.580728857382)*x[1]
        s[2,3,1,0]=(0.798247442811)*x[0]**o+(0.600332572583)*x[0]+(0.629676907787)*x[1]**o+(0.0214779823831)*x[1]
        s[2,3,1,1]=(0.446783692926)*x[0]**o+(-0.579857496036)*x[0]+(0.669355002891)*x[1]**o+(-0.104502660981)*x[1]
        s[2,3,2,0]=(0.874126641785)*x[0]**o+(0.244866909003)*x[0]+(-0.0181287421091)*x[1]**o+(-0.728707148938)*x[1]
        s[2,3,2,1]=(-0.716069945597)*x[0]**o+(0.632201538129)*x[0]+(-0.538839892852)*x[1]**o+(-0.886572197711)*x[1]
        s[2,4,0,0]=(-0.422214071919)*x[0]**o+(0.276563507343)*x[0]+(0.481338034453)*x[1]**o+(-0.972870285915)*x[1]
        s[2,4,0,1]=(0.740441652804)*x[0]**o+(0.447067551217)*x[0]+(-0.310919183692)*x[1]**o+(0.636513538883)*x[1]
        s[2,4,1,0]=(-0.599111504454)*x[0]**o+(-0.250476629717)*x[0]+(-0.658311629855)*x[1]**o+(0.492577876045)*x[1]
        s[2,4,1,1]=(0.107735805283)*x[0]**o+(0.208970264385)*x[0]+(-0.147382019475)*x[1]**o+(-0.406553806193)*x[1]
        s[2,4,2,0]=(0.925310626624)*x[0]**o+(0.0400900248412)*x[0]+(-0.431657024276)*x[1]**o+(-0.153834430713)*x[1]
        s[2,4,2,1]=(-0.713453580612)*x[0]**o+(0.391780861442)*x[0]+(-0.940123468631)*x[1]**o+(-0.937804820594)*x[1]
        s[3,0,0,0]=(-0.272038820171)*x[0]**o+(0.0722677637085)*x[0]+(-0.81552430022)*x[1]**o+(-0.605100343524)*x[1]
        s[3,0,0,1]=(-0.869234792389)*x[0]**o+(0.133360720775)*x[0]+(0.334759295473)*x[1]**o+(0.891704395665)*x[1]
        s[3,0,1,0]=(0.283196831063)*x[0]**o+(-0.433641717885)*x[0]+(0.821493597367)*x[1]**o+(0.0757494621194)*x[1]
        s[3,0,1,1]=(-0.970851846713)*x[0]**o+(0.420332346058)*x[0]+(-0.128433226815)*x[1]**o+(0.355540827779)*x[1]
        s[3,0,2,0]=(-0.855684873323)*x[0]**o+(-0.896780938471)*x[0]+(-0.910323225744)*x[1]**o+(-0.648707265693)*x[1]
        s[3,0,2,1]=(0.224013589567)*x[0]**o+(-0.109904216055)*x[0]+(0.86374013156)*x[1]**o+(0.474691072866)*x[1]
        s[3,1,0,0]=(-0.497944848072)*x[0]**o+(0.146776775946)*x[0]+(0.526474245297)*x[1]**o+(-0.835737531991)*x[1]
        s[3,1,0,1]=(-0.421150013578)*x[0]**o+(-0.784402607666)*x[0]+(-0.0805148903284)*x[1]**o+(-0.52347503704)*x[1]
        s[3,1,1,0]=(0.671180085936)*x[0]**o+(-0.0708908978891)*x[0]+(0.80005635402)*x[1]**o+(0.250773812305)*x[1]
        s[3,1,1,1]=(-0.740517458065)*x[0]**o+(-0.164975700504)*x[0]+(-0.954776148663)*x[1]**o+(0.0662528804032)*x[1]
        s[3,1,2,0]=(-0.0487118351329)*x[0]**o+(0.0172491914156)*x[0]+(0.459297824842)*x[1]**o+(0.931441363698)*x[1]
        s[3,1,2,1]=(0.914541867519)*x[0]**o+(-0.421726383007)*x[0]+(0.932990305149)*x[1]**o+(0.00708080928476)*x[1]
        s[3,2,0,0]=(-0.268505299798)*x[0]**o+(-0.884105078203)*x[0]+(-0.343177738426)*x[1]**o+(0.925594916146)*x[1]
        s[3,2,0,1]=(0.792964128575)*x[0]**o+(0.45312934456)*x[0]+(-0.190272595857)*x[1]**o+(-0.714484743775)*x[1]
        s[3,2,1,0]=(-0.389811244023)*x[0]**o+(-0.230272427621)*x[0]+(-0.062642727789)*x[1]**o+(0.333179046361)*x[1]
        s[3,2,1,1]=(0.468980019323)*x[0]**o+(-0.213426322608)*x[0]+(0.806778675985)*x[1]**o+(-0.031962815429)*x[1]
        s[3,2,2,0]=(0.187153153145)*x[0]**o+(0.440459591208)*x[0]+(-0.766229604202)*x[1]**o+(0.630228576743)*x[1]
        s[3,2,2,1]=(-0.362635752315)*x[0]**o+(-0.356960434705)*x[0]+(-0.209248043764)*x[1]**o+(0.70828385566)*x[1]
        s[3,3,0,0]=(-0.680482417515)*x[0]**o+(-0.90459474138)*x[0]+(-0.568552053392)*x[1]**o+(0.394198411912)*x[1]
        s[3,3,0,1]=(-0.0932148057029)*x[0]**o+(0.618628605947)*x[0]+(0.0470711457335)*x[1]**o+(0.802136993857)*x[1]
        s[3,3,1,0]=(-0.442943896794)*x[0]**o+(0.57385228082)*x[0]+(0.916092493461)*x[1]**o+(-0.913579167434)*x[1]
        s[3,3,1,1]=(0.976285532615)*x[0]**o+(-0.327052093737)*x[0]+(0.731622827093)*x[1]**o+(0.198510813385)*x[1]
        s[3,3,2,0]=(-0.921649918257)*x[0]**o+(-0.954803772809)*x[0]+(0.64789210876)*x[1]**o+(-0.928730411729)*x[1]
        s[3,3,2,1]=(0.227719394295)*x[0]**o+(-0.512300073514)*x[0]+(-0.401469395128)*x[1]**o+(-0.229248271712)*x[1]
        s[3,4,0,0]=(0.655903201215)*x[0]**o+(0.285189680631)*x[0]+(-0.183566536361)*x[1]**o+(-0.18428580448)*x[1]
        s[3,4,0,1]=(-0.0282806177857)*x[0]**o+(-0.29856051731)*x[0]+(0.120325396195)*x[1]**o+(-0.29397064071)*x[1]
        s[3,4,1,0]=(-0.640500611083)*x[0]**o+(0.979817975078)*x[0]+(0.577156862636)*x[1]**o+(-0.00170003402221)*x[1]
        s[3,4,1,1]=(0.681055102015)*x[0]**o+(-0.656903522238)*x[0]+(0.470845699959)*x[1]**o+(-0.973850290239)*x[1]
        s[3,4,2,0]=(-0.941893654596)*x[0]**o+(0.96514461809)*x[0]+(0.822665841321)*x[1]**o+(-0.826621500352)*x[1]
        s[3,4,2,1]=(0.0808254384437)*x[0]**o+(0.191567292517)*x[0]+(-0.48838584117)*x[1]**o+(-0.472758040235)*x[1]
        ref[0,0,0,0]=(-0.547754951171)*x_ref[0]**o+(0.356610809044)*x_ref[0]+(0.452044983539)*x_ref[1]**o+(-0.821341147164)*x_ref[1]
        ref[0,0,0,1]=(0.509267661384)*x_ref[0]**o+(0.667069615676)*x_ref[0]+(0.344083346522)*x_ref[1]**o+(-0.281070233626)*x_ref[1]
        ref[0,0,1,0]=(0.910153339127)*x_ref[0]**o+(-0.134026992189)*x_ref[0]+(0.283156493736)*x_ref[1]**o+(-0.236635156141)*x_ref[1]
        ref[0,0,1,1]=(-0.486341299233)*x_ref[0]**o+(0.969619867206)*x_ref[0]+(0.145365694427)*x_ref[1]**o+(0.395899354311)*x_ref[1]
        ref[0,0,2,0]=(-0.683524112594)*x_ref[0]**o+(-0.191053076108)*x_ref[0]+(-0.156617335102)*x_ref[1]**o+(-0.908072509242)*x_ref[1]
        ref[0,0,2,1]=(-0.329288530406)*x_ref[0]**o+(0.861655757717)*x_ref[0]+(-0.423717545177)*x_ref[1]**o+(0.357896568135)*x_ref[1]
        ref[0,1,0,0]=(0.454682074738)*x_ref[0]**o+(0.242640789048)*x_ref[0]+(-0.465273320791)*x_ref[1]**o+(-0.523776617273)*x_ref[1]
        ref[0,1,0,1]=(0.299812377446)*x_ref[0]**o+(0.258743885127)*x_ref[0]+(-0.545559145397)*x_ref[1]**o+(-0.703973878214)*x_ref[1]
        ref[0,1,1,0]=(-0.240052819623)*x_ref[0]**o+(-0.648241629125)*x_ref[0]+(0.550226821237)*x_ref[1]**o+(0.586254427297)*x_ref[1]
        ref[0,1,1,1]=(0.232594672088)*x_ref[0]**o+(-0.082742546667)*x_ref[0]+(0.695512543415)*x_ref[1]**o+(0.695609364591)*x_ref[1]
        ref[0,1,2,0]=(-0.337129430207)*x_ref[0]**o+(-0.0996108162262)*x_ref[0]+(0.520279135444)*x_ref[1]**o+(-0.0419680101154)*x_ref[1]
        ref[0,1,2,1]=(0.107050704386)*x_ref[0]**o+(-0.0108163145609)*x_ref[0]+(-0.555986163214)*x_ref[1]**o+(-0.264372389969)*x_ref[1]
        ref[0,2,0,0]=(0.558185714977)*x_ref[0]**o+(0.718930754697)*x_ref[0]+(0.661967682247)*x_ref[1]**o+(0.968289790196)*x_ref[1]
        ref[0,2,0,1]=(-0.885308413126)*x_ref[0]**o+(-0.000196750280455)*x_ref[0]+(0.162337198564)*x_ref[1]**o+(-0.372085076741)*x_ref[1]
        ref[0,2,1,0]=(0.732654846787)*x_ref[0]**o+(0.490322343819)*x_ref[0]+(0.253518225543)*x_ref[1]**o+(0.755915810471)*x_ref[1]
        ref[0,2,1,1]=(0.686091929471)*x_ref[0]**o+(0.968804203166)*x_ref[0]+(-0.874263118503)*x_ref[1]**o+(0.792892684533)*x_ref[1]
        ref[0,2,2,0]=(-0.538165611113)*x_ref[0]**o+(0.435640729152)*x_ref[0]+(0.53052383117)*x_ref[1]**o+(-0.152917182113)*x_ref[1]
        ref[0,2,2,1]=(0.959376322991)*x_ref[0]**o+(0.295428309997)*x_ref[0]+(0.774755280624)*x_ref[1]**o+(0.699112429694)*x_ref[1]
        ref[0,3,0,0]=(-0.879327491632)*x_ref[0]**o+(-0.210915363483)*x_ref[0]+(0.677720758543)*x_ref[1]**o+(-0.156843338639)*x_ref[1]
        ref[0,3,0,1]=(-0.877749774331)*x_ref[0]**o+(-0.0590688859445)*x_ref[0]+(0.372427864404)*x_ref[1]**o+(0.865604489942)*x_ref[1]
        ref[0,3,1,0]=(-0.479636139873)*x_ref[0]**o+(-0.2440095302)*x_ref[0]+(0.291946443647)*x_ref[1]**o+(-0.446912115557)*x_ref[1]
        ref[0,3,1,1]=(-0.666968887345)*x_ref[0]**o+(-0.530382039479)*x_ref[0]+(-0.954360470015)*x_ref[1]**o+(0.827870552445)*x_ref[1]
        ref[0,3,2,0]=(-0.697008931895)*x_ref[0]**o+(-0.506024200287)*x_ref[0]+(-0.284218723212)*x_ref[1]**o+(-0.12154006973)*x_ref[1]
        ref[0,3,2,1]=(0.182079912323)*x_ref[0]**o+(0.128753570425)*x_ref[0]+(-0.610041297244)*x_ref[1]**o+(0.167312633663)*x_ref[1]
        ref[0,4,0,0]=(0.905082824851)*x_ref[0]**o+(-0.850946080497)*x_ref[0]+(-0.234019076337)*x_ref[1]**o+(-0.746004283581)*x_ref[1]
        ref[0,4,0,1]=(0.252751160265)*x_ref[0]**o+(0.783081625822)*x_ref[0]+(-0.18400975082)*x_ref[1]**o+(0.729096084753)*x_ref[1]
        ref[0,4,1,0]=(-0.646745752044)*x_ref[0]**o+(-0.95019218163)*x_ref[0]+(-0.739473363344)*x_ref[1]**o+(0.53887852033)*x_ref[1]
        ref[0,4,1,1]=(0.190710071596)*x_ref[0]**o+(0.363498800572)*x_ref[0]+(-0.166712144832)*x_ref[1]**o+(0.383842690034)*x_ref[1]
        ref[0,4,2,0]=(-0.252312977399)*x_ref[0]**o+(-0.181431558994)*x_ref[0]+(-0.496223516025)*x_ref[1]**o+(0.262053197788)*x_ref[1]
        ref[0,4,2,1]=(0.740944095813)*x_ref[0]**o+(0.350982435679)*x_ref[0]+(0.540773567147)*x_ref[1]**o+(0.154742779719)*x_ref[1]
        ref[1,0,0,0]=(-0.204518679107)*x_ref[0]**o+(0.224833545775)*x_ref[0]+(0.701576397345)*x_ref[1]**o+(0.955758264121)*x_ref[1]
        ref[1,0,0,1]=(0.98822877512)*x_ref[0]**o+(-0.252532184922)*x_ref[0]+(-0.999543053077)*x_ref[1]**o+(0.20420747276)*x_ref[1]
        ref[1,0,1,0]=(-0.435694974363)*x_ref[0]**o+(0.106116746803)*x_ref[0]+(-0.614925089687)*x_ref[1]**o+(-0.433482250787)*x_ref[1]
        ref[1,0,1,1]=(-0.967905352631)*x_ref[0]**o+(0.733630963983)*x_ref[0]+(0.561497813177)*x_ref[1]**o+(-0.254199898967)*x_ref[1]
        ref[1,0,2,0]=(0.192509982098)*x_ref[0]**o+(0.30844145669)*x_ref[0]+(-0.537990978303)*x_ref[1]**o+(-0.971691014587)*x_ref[1]
        ref[1,0,2,1]=(-0.642601469866)*x_ref[0]**o+(0.828976360334)*x_ref[0]+(-0.166247518535)*x_ref[1]**o+(-0.394356610568)*x_ref[1]
        ref[1,1,0,0]=(-0.190023582376)*x_ref[0]**o+(0.00308098708653)*x_ref[0]+(0.616327311292)*x_ref[1]**o+(0.50233410734)*x_ref[1]
        ref[1,1,0,1]=(-0.421733782097)*x_ref[0]**o+(-0.81738567824)*x_ref[0]+(0.0102253671566)*x_ref[1]**o+(0.977732802241)*x_ref[1]
        ref[1,1,1,0]=(-0.932299813732)*x_ref[0]**o+(-0.570728228193)*x_ref[0]+(0.906998960465)*x_ref[1]**o+(0.276967285338)*x_ref[1]
        ref[1,1,1,1]=(0.361866174533)*x_ref[0]**o+(-0.948833321247)*x_ref[0]+(0.0267950613442)*x_ref[1]**o+(0.305540794357)*x_ref[1]
        ref[1,1,2,0]=(-0.107562715773)*x_ref[0]**o+(0.884548510332)*x_ref[0]+(-0.641236013271)*x_ref[1]**o+(0.835011288403)*x_ref[1]
        ref[1,1,2,1]=(-0.447112184269)*x_ref[0]**o+(0.68668033563)*x_ref[0]+(-0.643123622354)*x_ref[1]**o+(-0.373398061272)*x_ref[1]
        ref[1,2,0,0]=(0.566476213386)*x_ref[0]**o+(-0.660978862942)*x_ref[0]+(-0.250064101238)*x_ref[1]**o+(0.0617654537116)*x_ref[1]
        ref[1,2,0,1]=(-0.0220375595157)*x_ref[0]**o+(-0.664682506405)*x_ref[0]+(-0.810443003238)*x_ref[1]**o+(-0.704430911362)*x_ref[1]
        ref[1,2,1,0]=(-0.66837513872)*x_ref[0]**o+(0.518574783899)*x_ref[0]+(0.456254450127)*x_ref[1]**o+(-0.0264418287977)*x_ref[1]
        ref[1,2,1,1]=(0.518371474153)*x_ref[0]**o+(0.305863788515)*x_ref[0]+(0.641786832956)*x_ref[1]**o+(0.957403546249)*x_ref[1]
        ref[1,2,2,0]=(-0.702485305168)*x_ref[0]**o+(0.810129172394)*x_ref[0]+(0.481384977464)*x_ref[1]**o+(-0.552283897592)*x_ref[1]
        ref[1,2,2,1]=(-0.220714257324)*x_ref[0]**o+(0.658520871538)*x_ref[0]+(-0.156197468389)*x_ref[1]**o+(0.406317861261)*x_ref[1]
        ref[1,3,0,0]=(-0.833421663125)*x_ref[0]**o+(-0.23885966657)*x_ref[0]+(-0.0853110945734)*x_ref[1]**o+(0.134587357668)*x_ref[1]
        ref[1,3,0,1]=(0.983003072543)*x_ref[0]**o+(-0.444298839588)*x_ref[0]+(-0.555821641978)*x_ref[1]**o+(-0.934198804069)*x_ref[1]
        ref[1,3,1,0]=(-0.321902025108)*x_ref[0]**o+(0.373296267434)*x_ref[0]+(-0.963121149124)*x_ref[1]**o+(0.0147111484841)*x_ref[1]
        ref[1,3,1,1]=(-0.686361701679)*x_ref[0]**o+(0.322135914568)*x_ref[0]+(0.0577830897835)*x_ref[1]**o+(0.577239387766)*x_ref[1]
        ref[1,3,2,0]=(0.722944586976)*x_ref[0]**o+(0.789741801778)*x_ref[0]+(0.175208515485)*x_ref[1]**o+(-0.71289312398)*x_ref[1]
        ref[1,3,2,1]=(-0.280608530342)*x_ref[0]**o+(0.0860239950894)*x_ref[0]+(0.695636585879)*x_ref[1]**o+(-0.960594121577)*x_ref[1]
        ref[1,4,0,0]=(-0.852503330892)*x_ref[0]**o+(-0.602234774377)*x_ref[0]+(0.198091265715)*x_ref[1]**o+(0.777503337768)*x_ref[1]
        ref[1,4,0,1]=(-0.177137751594)*x_ref[0]**o+(-0.490189065267)*x_ref[0]+(-0.8576459288)*x_ref[1]**o+(-0.074100447778)*x_ref[1]
        ref[1,4,1,0]=(-0.334129920953)*x_ref[0]**o+(-0.337245487943)*x_ref[0]+(-0.839457764245)*x_ref[1]**o+(-0.58152129024)*x_ref[1]
        ref[1,4,1,1]=(0.0937571716669)*x_ref[0]**o+(0.592103688833)*x_ref[0]+(0.15448940004)*x_ref[1]**o+(-0.203463329576)*x_ref[1]
        ref[1,4,2,0]=(0.139741892415)*x_ref[0]**o+(0.611539882432)*x_ref[0]+(-0.407889172421)*x_ref[1]**o+(0.891469847051)*x_ref[1]
        ref[1,4,2,1]=(0.95147411312)*x_ref[0]**o+(0.0998931787148)*x_ref[0]+(0.466902306753)*x_ref[1]**o+(-0.718472606772)*x_ref[1]
        ref[2,0,0,0]=(-0.130008770209)*x_ref[0]**o+(0.809915931169)*x_ref[0]+(-0.732276845472)*x_ref[1]**o+(-0.192903892405)*x_ref[1]
        ref[2,0,0,1]=(-0.0119712505975)*x_ref[0]**o+(0.656694120973)*x_ref[0]+(0.6862601083)*x_ref[1]**o+(-0.528666986412)*x_ref[1]
        ref[2,0,1,0]=(0.28678233655)*x_ref[0]**o+(-0.37339891868)*x_ref[0]+(-0.946168471623)*x_ref[1]**o+(0.762233026568)*x_ref[1]
        ref[2,0,1,1]=(0.778843713994)*x_ref[0]**o+(0.817766330368)*x_ref[0]+(-0.300918496467)*x_ref[1]**o+(0.442988971838)*x_ref[1]
        ref[2,0,2,0]=(0.207449688136)*x_ref[0]**o+(0.10275570242)*x_ref[0]+(-0.520466147913)*x_ref[1]**o+(0.33346010447)*x_ref[1]
        ref[2,0,2,1]=(0.709221342424)*x_ref[0]**o+(0.159144243448)*x_ref[0]+(0.685824812808)*x_ref[1]**o+(0.50901941527)*x_ref[1]
        ref[2,1,0,0]=(0.0244924098009)*x_ref[0]**o+(-0.770942809125)*x_ref[0]+(0.372626872676)*x_ref[1]**o+(-0.714320503926)*x_ref[1]
        ref[2,1,0,1]=(-0.105052119318)*x_ref[0]**o+(0.573319234378)*x_ref[0]+(0.0995762579843)*x_ref[1]**o+(0.579088439928)*x_ref[1]
        ref[2,1,1,0]=(-0.572394543041)*x_ref[0]**o+(0.1606374212)*x_ref[0]+(-0.801731532404)*x_ref[1]**o+(-0.644110219424)*x_ref[1]
        ref[2,1,1,1]=(0.914053075068)*x_ref[0]**o+(-0.230478556116)*x_ref[0]+(-0.00343924084847)*x_ref[1]**o+(-0.53144169108)*x_ref[1]
        ref[2,1,2,0]=(-0.743362569446)*x_ref[0]**o+(-0.0428045574774)*x_ref[0]+(-0.503190273025)*x_ref[1]**o+(0.831658185042)*x_ref[1]
        ref[2,1,2,1]=(0.176721891893)*x_ref[0]**o+(-0.460950049697)*x_ref[0]+(0.45677887739)*x_ref[1]**o+(-0.853283022367)*x_ref[1]
        ref[2,2,0,0]=(-0.585337031786)*x_ref[0]**o+(0.901183590413)*x_ref[0]+(-0.746305431471)*x_ref[1]**o+(0.0439043730531)*x_ref[1]
        ref[2,2,0,1]=(0.594349646931)*x_ref[0]**o+(-0.431546660621)*x_ref[0]+(-0.962096864047)*x_ref[1]**o+(0.955112341273)*x_ref[1]
        ref[2,2,1,0]=(-0.857877499011)*x_ref[0]**o+(0.803223671127)*x_ref[0]+(-0.995806495668)*x_ref[1]**o+(-0.267707433165)*x_ref[1]
        ref[2,2,1,1]=(0.968406110131)*x_ref[0]**o+(-0.0208982523366)*x_ref[0]+(-0.000406237500393)*x_ref[1]**o+(-0.334429552696)*x_ref[1]
        ref[2,2,2,0]=(-0.960828307746)*x_ref[0]**o+(0.235004476043)*x_ref[0]+(0.0456041210872)*x_ref[1]**o+(0.323144610527)*x_ref[1]
        ref[2,2,2,1]=(0.538691072674)*x_ref[0]**o+(-0.737490055104)*x_ref[0]+(-0.96703538134)*x_ref[1]**o+(0.727057872325)*x_ref[1]
        ref[2,3,0,0]=(0.475319703318)*x_ref[0]**o+(0.863857783409)*x_ref[0]+(0.0891089574388)*x_ref[1]**o+(0.121619153024)*x_ref[1]
        ref[2,3,0,1]=(0.68881433038)*x_ref[0]**o+(-0.812908863657)*x_ref[0]+(0.687129220133)*x_ref[1]**o+(0.580728857382)*x_ref[1]
        ref[2,3,1,0]=(0.798247442811)*x_ref[0]**o+(0.600332572583)*x_ref[0]+(0.629676907787)*x_ref[1]**o+(0.0214779823831)*x_ref[1]
        ref[2,3,1,1]=(0.446783692926)*x_ref[0]**o+(-0.579857496036)*x_ref[0]+(0.669355002891)*x_ref[1]**o+(-0.104502660981)*x_ref[1]
        ref[2,3,2,0]=(0.874126641785)*x_ref[0]**o+(0.244866909003)*x_ref[0]+(-0.0181287421091)*x_ref[1]**o+(-0.728707148938)*x_ref[1]
        ref[2,3,2,1]=(-0.716069945597)*x_ref[0]**o+(0.632201538129)*x_ref[0]+(-0.538839892852)*x_ref[1]**o+(-0.886572197711)*x_ref[1]
        ref[2,4,0,0]=(-0.422214071919)*x_ref[0]**o+(0.276563507343)*x_ref[0]+(0.481338034453)*x_ref[1]**o+(-0.972870285915)*x_ref[1]
        ref[2,4,0,1]=(0.740441652804)*x_ref[0]**o+(0.447067551217)*x_ref[0]+(-0.310919183692)*x_ref[1]**o+(0.636513538883)*x_ref[1]
        ref[2,4,1,0]=(-0.599111504454)*x_ref[0]**o+(-0.250476629717)*x_ref[0]+(-0.658311629855)*x_ref[1]**o+(0.492577876045)*x_ref[1]
        ref[2,4,1,1]=(0.107735805283)*x_ref[0]**o+(0.208970264385)*x_ref[0]+(-0.147382019475)*x_ref[1]**o+(-0.406553806193)*x_ref[1]
        ref[2,4,2,0]=(0.925310626624)*x_ref[0]**o+(0.0400900248412)*x_ref[0]+(-0.431657024276)*x_ref[1]**o+(-0.153834430713)*x_ref[1]
        ref[2,4,2,1]=(-0.713453580612)*x_ref[0]**o+(0.391780861442)*x_ref[0]+(-0.940123468631)*x_ref[1]**o+(-0.937804820594)*x_ref[1]
        ref[3,0,0,0]=(-0.272038820171)*x_ref[0]**o+(0.0722677637085)*x_ref[0]+(-0.81552430022)*x_ref[1]**o+(-0.605100343524)*x_ref[1]
        ref[3,0,0,1]=(-0.869234792389)*x_ref[0]**o+(0.133360720775)*x_ref[0]+(0.334759295473)*x_ref[1]**o+(0.891704395665)*x_ref[1]
        ref[3,0,1,0]=(0.283196831063)*x_ref[0]**o+(-0.433641717885)*x_ref[0]+(0.821493597367)*x_ref[1]**o+(0.0757494621194)*x_ref[1]
        ref[3,0,1,1]=(-0.970851846713)*x_ref[0]**o+(0.420332346058)*x_ref[0]+(-0.128433226815)*x_ref[1]**o+(0.355540827779)*x_ref[1]
        ref[3,0,2,0]=(-0.855684873323)*x_ref[0]**o+(-0.896780938471)*x_ref[0]+(-0.910323225744)*x_ref[1]**o+(-0.648707265693)*x_ref[1]
        ref[3,0,2,1]=(0.224013589567)*x_ref[0]**o+(-0.109904216055)*x_ref[0]+(0.86374013156)*x_ref[1]**o+(0.474691072866)*x_ref[1]
        ref[3,1,0,0]=(-0.497944848072)*x_ref[0]**o+(0.146776775946)*x_ref[0]+(0.526474245297)*x_ref[1]**o+(-0.835737531991)*x_ref[1]
        ref[3,1,0,1]=(-0.421150013578)*x_ref[0]**o+(-0.784402607666)*x_ref[0]+(-0.0805148903284)*x_ref[1]**o+(-0.52347503704)*x_ref[1]
        ref[3,1,1,0]=(0.671180085936)*x_ref[0]**o+(-0.0708908978891)*x_ref[0]+(0.80005635402)*x_ref[1]**o+(0.250773812305)*x_ref[1]
        ref[3,1,1,1]=(-0.740517458065)*x_ref[0]**o+(-0.164975700504)*x_ref[0]+(-0.954776148663)*x_ref[1]**o+(0.0662528804032)*x_ref[1]
        ref[3,1,2,0]=(-0.0487118351329)*x_ref[0]**o+(0.0172491914156)*x_ref[0]+(0.459297824842)*x_ref[1]**o+(0.931441363698)*x_ref[1]
        ref[3,1,2,1]=(0.914541867519)*x_ref[0]**o+(-0.421726383007)*x_ref[0]+(0.932990305149)*x_ref[1]**o+(0.00708080928476)*x_ref[1]
        ref[3,2,0,0]=(-0.268505299798)*x_ref[0]**o+(-0.884105078203)*x_ref[0]+(-0.343177738426)*x_ref[1]**o+(0.925594916146)*x_ref[1]
        ref[3,2,0,1]=(0.792964128575)*x_ref[0]**o+(0.45312934456)*x_ref[0]+(-0.190272595857)*x_ref[1]**o+(-0.714484743775)*x_ref[1]
        ref[3,2,1,0]=(-0.389811244023)*x_ref[0]**o+(-0.230272427621)*x_ref[0]+(-0.062642727789)*x_ref[1]**o+(0.333179046361)*x_ref[1]
        ref[3,2,1,1]=(0.468980019323)*x_ref[0]**o+(-0.213426322608)*x_ref[0]+(0.806778675985)*x_ref[1]**o+(-0.031962815429)*x_ref[1]
        ref[3,2,2,0]=(0.187153153145)*x_ref[0]**o+(0.440459591208)*x_ref[0]+(-0.766229604202)*x_ref[1]**o+(0.630228576743)*x_ref[1]
        ref[3,2,2,1]=(-0.362635752315)*x_ref[0]**o+(-0.356960434705)*x_ref[0]+(-0.209248043764)*x_ref[1]**o+(0.70828385566)*x_ref[1]
        ref[3,3,0,0]=(-0.680482417515)*x_ref[0]**o+(-0.90459474138)*x_ref[0]+(-0.568552053392)*x_ref[1]**o+(0.394198411912)*x_ref[1]
        ref[3,3,0,1]=(-0.0932148057029)*x_ref[0]**o+(0.618628605947)*x_ref[0]+(0.0470711457335)*x_ref[1]**o+(0.802136993857)*x_ref[1]
        ref[3,3,1,0]=(-0.442943896794)*x_ref[0]**o+(0.57385228082)*x_ref[0]+(0.916092493461)*x_ref[1]**o+(-0.913579167434)*x_ref[1]
        ref[3,3,1,1]=(0.976285532615)*x_ref[0]**o+(-0.327052093737)*x_ref[0]+(0.731622827093)*x_ref[1]**o+(0.198510813385)*x_ref[1]
        ref[3,3,2,0]=(-0.921649918257)*x_ref[0]**o+(-0.954803772809)*x_ref[0]+(0.64789210876)*x_ref[1]**o+(-0.928730411729)*x_ref[1]
        ref[3,3,2,1]=(0.227719394295)*x_ref[0]**o+(-0.512300073514)*x_ref[0]+(-0.401469395128)*x_ref[1]**o+(-0.229248271712)*x_ref[1]
        ref[3,4,0,0]=(0.655903201215)*x_ref[0]**o+(0.285189680631)*x_ref[0]+(-0.183566536361)*x_ref[1]**o+(-0.18428580448)*x_ref[1]
        ref[3,4,0,1]=(-0.0282806177857)*x_ref[0]**o+(-0.29856051731)*x_ref[0]+(0.120325396195)*x_ref[1]**o+(-0.29397064071)*x_ref[1]
        ref[3,4,1,0]=(-0.640500611083)*x_ref[0]**o+(0.979817975078)*x_ref[0]+(0.577156862636)*x_ref[1]**o+(-0.00170003402221)*x_ref[1]
        ref[3,4,1,1]=(0.681055102015)*x_ref[0]**o+(-0.656903522238)*x_ref[0]+(0.470845699959)*x_ref[1]**o+(-0.973850290239)*x_ref[1]
        ref[3,4,2,0]=(-0.941893654596)*x_ref[0]**o+(0.96514461809)*x_ref[0]+(0.822665841321)*x_ref[1]**o+(-0.826621500352)*x_ref[1]
        ref[3,4,2,1]=(0.0808254384437)*x_ref[0]**o+(0.191567292517)*x_ref[0]+(-0.48838584117)*x_ref[1]**o+(-0.472758040235)*x_ref[1]
      else:
        s[0,0,0,0]=(0.186525987907)*x[0]**o+(-0.690917154378)*x[0]+(0.832803193217)*x[1]**o+(-0.0440294807629)*x[1]+(0.936927757561)*x[2]**o+(-0.120763864664)*x[2]
        s[0,0,0,1]=(0.0113224706745)*x[0]**o+(0.522015747536)*x[0]+(-0.281875411547)*x[1]**o+(0.605176091896)*x[1]+(-0.448130660869)*x[2]**o+(0.557667991786)*x[2]
        s[0,0,1,0]=(-0.5115411507)*x[0]**o+(0.568051998295)*x[0]+(0.722442651334)*x[1]**o+(0.410822384229)*x[1]+(0.235364385145)*x[2]**o+(0.881645768144)*x[2]
        s[0,0,1,1]=(0.365191539085)*x[0]**o+(0.0518813698411)*x[0]+(0.179359856471)*x[1]**o+(-0.634486389415)*x[1]+(0.311578648976)*x[2]**o+(-0.157034397126)*x[2]
        s[0,0,2,0]=(0.340474667312)*x[0]**o+(-0.761761779074)*x[0]+(-0.118701824049)*x[1]**o+(0.0922924561625)*x[1]+(0.132924567807)*x[2]**o+(-0.0694502802118)*x[2]
        s[0,0,2,1]=(0.21666407868)*x[0]**o+(0.65101030565)*x[0]+(0.0978745307399)*x[1]**o+(-0.592357988084)*x[1]+(-0.605382873404)*x[2]**o+(-0.899153475467)*x[2]
        s[0,1,0,0]=(-0.239693597874)*x[0]**o+(-0.265899323452)*x[0]+(-0.210985518909)*x[1]**o+(-0.408445623906)*x[1]+(0.350622495806)*x[2]**o+(-0.534920839922)*x[2]
        s[0,1,0,1]=(-0.596708903276)*x[0]**o+(0.241677381533)*x[0]+(-0.327063865807)*x[1]**o+(0.731700322335)*x[1]+(-0.29224310646)*x[2]**o+(-0.177855845822)*x[2]
        s[0,1,1,0]=(0.13119146515)*x[0]**o+(0.24155441457)*x[0]+(-0.730354339458)*x[1]**o+(0.17509896433)*x[1]+(0.602886357198)*x[2]**o+(-0.300613826457)*x[2]
        s[0,1,1,1]=(-0.528650609789)*x[0]**o+(0.730375830185)*x[0]+(0.706889523794)*x[1]**o+(-0.280144821714)*x[1]+(0.261124676699)*x[2]**o+(0.230842251108)*x[2]
        s[0,1,2,0]=(0.0867102525865)*x[0]**o+(0.238707033898)*x[0]+(-0.914453619505)*x[1]**o+(0.224533380834)*x[1]+(0.406716858519)*x[2]**o+(-0.347408293194)*x[2]
        s[0,1,2,1]=(-0.567166350757)*x[0]**o+(-0.156169071272)*x[0]+(0.15846212695)*x[1]**o+(-0.0254016167751)*x[1]+(0.707956392472)*x[2]**o+(-0.510965658867)*x[2]
        s[0,2,0,0]=(-0.181235844201)*x[0]**o+(0.936656137873)*x[0]+(0.830895834923)*x[1]**o+(0.752514827093)*x[1]+(0.478651458614)*x[2]**o+(0.353659575913)*x[2]
        s[0,2,0,1]=(-0.0466341076413)*x[0]**o+(-0.137913043533)*x[0]+(0.569602506318)*x[1]**o+(0.453951166085)*x[1]+(-0.736500514341)*x[2]**o+(0.494877047573)*x[2]
        s[0,2,1,0]=(-0.774800275665)*x[0]**o+(0.320634390869)*x[0]+(-0.801898551402)*x[1]**o+(0.680379650404)*x[1]+(0.115138386556)*x[2]**o+(0.847285566367)*x[2]
        s[0,2,1,1]=(-0.948363366443)*x[0]**o+(-0.401155889062)*x[0]+(0.602001652794)*x[1]**o+(0.881111991966)*x[1]+(-0.407826312421)*x[2]**o+(-0.385793087535)*x[2]
        s[0,2,2,0]=(-0.0498782718626)*x[0]**o+(0.278903888089)*x[0]+(0.28455096215)*x[1]**o+(0.165496315478)*x[1]+(-0.516044235314)*x[2]**o+(-0.320130768929)*x[2]
        s[0,2,2,1]=(-0.0313287552123)*x[0]**o+(0.795534322833)*x[0]+(0.0672014159699)*x[1]**o+(-0.232632859958)*x[1]+(0.968980662087)*x[2]**o+(-0.0437975661275)*x[2]
        s[0,3,0,0]=(0.879723765854)*x[0]**o+(0.616502035178)*x[0]+(-0.888143209368)*x[1]**o+(-0.954918894054)*x[1]+(-0.0649197025038)*x[2]**o+(-0.865957161833)*x[2]
        s[0,3,0,1]=(0.550691528997)*x[0]**o+(0.136724942999)*x[0]+(0.474412879455)*x[1]**o+(0.677263937069)*x[1]+(0.453101604697)*x[2]**o+(0.133742079633)*x[2]
        s[0,3,1,0]=(-0.970027293215)*x[0]**o+(-0.610696832357)*x[0]+(-0.248732897836)*x[1]**o+(0.957868248702)*x[1]+(0.334784026675)*x[2]**o+(0.696895349867)*x[2]
        s[0,3,1,1]=(-0.672808837142)*x[0]**o+(-0.0553936755153)*x[0]+(-0.00732320498388)*x[1]**o+(0.49021088588)*x[1]+(0.907554472651)*x[2]**o+(-0.058244899423)*x[2]
        s[0,3,2,0]=(0.0159001870399)*x[0]**o+(0.0227678146243)*x[0]+(0.805708466496)*x[1]**o+(0.785990910247)*x[1]+(-0.725672039232)*x[2]**o+(-0.162870251036)*x[2]
        s[0,3,2,1]=(0.520705647829)*x[0]**o+(-0.310310041859)*x[0]+(-0.317118647815)*x[1]**o+(0.534773934168)*x[1]+(-0.268620331386)*x[2]**o+(0.334022726024)*x[2]
        s[0,4,0,0]=(0.529035051582)*x[0]**o+(0.566819545868)*x[0]+(0.618896518995)*x[1]**o+(0.263612275958)*x[1]+(-0.618741920385)*x[2]**o+(-0.0264643721073)*x[2]
        s[0,4,0,1]=(0.860757010858)*x[0]**o+(-0.706974137596)*x[0]+(0.0612556623739)*x[1]**o+(0.722512189141)*x[1]+(-0.676611378845)*x[2]**o+(-0.615875915177)*x[2]
        s[0,4,1,0]=(-0.186509714673)*x[0]**o+(0.0351164428631)*x[0]+(-0.173768366165)*x[1]**o+(-0.426776871363)*x[1]+(-0.0678988589984)*x[2]**o+(-0.0164388017801)*x[2]
        s[0,4,1,1]=(-0.528750419399)*x[0]**o+(-0.0188958007868)*x[0]+(-0.381850037026)*x[1]**o+(-0.366474181406)*x[1]+(-0.0157097200919)*x[2]**o+(0.750818168768)*x[2]
        s[0,4,2,0]=(0.962983200785)*x[0]**o+(-0.904671719523)*x[0]+(0.83012715029)*x[1]**o+(0.20786351761)*x[1]+(0.61032830083)*x[2]**o+(0.999068101126)*x[2]
        s[0,4,2,1]=(0.369225553113)*x[0]**o+(0.370643799864)*x[0]+(0.562458197002)*x[1]**o+(0.864829790167)*x[1]+(-0.507288312866)*x[2]**o+(-0.615656522393)*x[2]
        s[1,0,0,0]=(0.0208552161791)*x[0]**o+(0.64767817505)*x[0]+(-0.913625079536)*x[1]**o+(0.214341575088)*x[1]+(-0.369358777546)*x[2]**o+(-0.744597104081)*x[2]
        s[1,0,0,1]=(0.688956912909)*x[0]**o+(0.130856033181)*x[0]+(-0.754570690839)*x[1]**o+(-0.892591311673)*x[1]+(0.121513821098)*x[2]**o+(0.0499651928086)*x[2]
        s[1,0,1,0]=(0.488371732492)*x[0]**o+(-0.0899662322511)*x[0]+(-0.184052688998)*x[1]**o+(-0.504791015013)*x[1]+(0.939413170478)*x[2]**o+(-0.371392738292)*x[2]
        s[1,0,1,1]=(-0.634015136517)*x[0]**o+(0.189031198575)*x[0]+(-0.187597065324)*x[1]**o+(0.417035820606)*x[1]+(-0.640693898356)*x[2]**o+(-0.356999823488)*x[2]
        s[1,0,2,0]=(-0.299739907169)*x[0]**o+(0.97973415315)*x[0]+(0.172728079372)*x[1]**o+(-0.219250264223)*x[1]+(0.0377842392779)*x[2]**o+(0.568605450772)*x[2]
        s[1,0,2,1]=(0.668822572798)*x[0]**o+(-0.464904703232)*x[0]+(-0.0438179224843)*x[1]**o+(0.972577571624)*x[1]+(0.200473119295)*x[2]**o+(0.769536551131)*x[2]
        s[1,1,0,0]=(-0.5952291144)*x[0]**o+(0.215912343317)*x[0]+(-0.187871783507)*x[1]**o+(0.000141658119764)*x[1]+(0.652350178895)*x[2]**o+(0.748942594082)*x[2]
        s[1,1,0,1]=(0.978202121028)*x[0]**o+(-0.270952075217)*x[0]+(0.379211795706)*x[1]**o+(0.570614905933)*x[1]+(0.531348311358)*x[2]**o+(-0.774802192871)*x[2]
        s[1,1,1,0]=(-0.173449381518)*x[0]**o+(-0.788533061983)*x[0]+(-0.159503791119)*x[1]**o+(-0.182774291708)*x[1]+(0.493985220422)*x[2]**o+(-0.442523611538)*x[2]
        s[1,1,1,1]=(-0.196642522444)*x[0]**o+(0.353038689926)*x[0]+(0.739133358128)*x[1]**o+(0.384564329164)*x[1]+(0.00724850200888)*x[2]**o+(-0.534791252863)*x[2]
        s[1,1,2,0]=(-0.742601262273)*x[0]**o+(0.870936220364)*x[0]+(0.173515343981)*x[1]**o+(-0.924818265666)*x[1]+(-0.786225297604)*x[2]**o+(0.723191951357)*x[2]
        s[1,1,2,1]=(0.584821708581)*x[0]**o+(0.233692617647)*x[0]+(0.917976018627)*x[1]**o+(0.0411719886893)*x[1]+(-0.731518066297)*x[2]**o+(-0.886641339023)*x[2]
        s[1,2,0,0]=(0.879019029124)*x[0]**o+(0.198224253024)*x[0]+(-0.508879741455)*x[1]**o+(-0.0493693577221)*x[1]+(-0.200204490189)*x[2]**o+(-0.558819359406)*x[2]
        s[1,2,0,1]=(0.343696269765)*x[0]**o+(-0.694690909901)*x[0]+(0.562857727334)*x[1]**o+(0.170105867024)*x[1]+(0.300442565793)*x[2]**o+(-0.78443966073)*x[2]
        s[1,2,1,0]=(-0.104227942738)*x[0]**o+(-0.631808216742)*x[0]+(-0.688416357176)*x[1]**o+(0.543474609002)*x[1]+(-0.475674253864)*x[2]**o+(-0.622363009599)*x[2]
        s[1,2,1,1]=(-0.187993381022)*x[0]**o+(-0.112771930635)*x[0]+(0.231591342365)*x[1]**o+(-0.619972979708)*x[1]+(-0.633470156859)*x[2]**o+(0.726102617311)*x[2]
        s[1,2,2,0]=(-0.0929871065489)*x[0]**o+(-0.741493547909)*x[0]+(-0.500399374101)*x[1]**o+(0.489221111027)*x[1]+(-0.538850386612)*x[2]**o+(-0.366843628744)*x[2]
        s[1,2,2,1]=(-0.449265426222)*x[0]**o+(-0.711133901403)*x[0]+(-0.671933617866)*x[1]**o+(-0.75650179505)*x[1]+(-0.441794206618)*x[2]**o+(-0.679799182186)*x[2]
        s[1,3,0,0]=(0.0794555795393)*x[0]**o+(-0.591466474089)*x[0]+(0.537807349728)*x[1]**o+(-0.0295904103007)*x[1]+(-0.206212961776)*x[2]**o+(0.633837648242)*x[2]
        s[1,3,0,1]=(0.232707718354)*x[0]**o+(-0.310438878448)*x[0]+(-0.0844732229089)*x[1]**o+(0.507469871796)*x[1]+(-0.391898028012)*x[2]**o+(-0.374582716512)*x[2]
        s[1,3,1,0]=(-0.166926427703)*x[0]**o+(-0.926298908928)*x[0]+(0.244425001632)*x[1]**o+(0.284615289051)*x[1]+(-0.675054769717)*x[2]**o+(-0.0765552057049)*x[2]
        s[1,3,1,1]=(-0.469538130468)*x[0]**o+(-0.110820901289)*x[0]+(-0.608743940572)*x[1]**o+(0.0606943645263)*x[1]+(-0.211127814891)*x[2]**o+(-0.0474195944349)*x[2]
        s[1,3,2,0]=(0.769147852508)*x[0]**o+(-0.500794487532)*x[0]+(-0.0971530930356)*x[1]**o+(0.282178880991)*x[1]+(-0.175125963785)*x[2]**o+(0.0567340005296)*x[2]
        s[1,3,2,1]=(0.14422447964)*x[0]**o+(-0.9056785988)*x[0]+(-0.962346923857)*x[1]**o+(0.64076498247)*x[1]+(0.77680599777)*x[2]**o+(0.450703438151)*x[2]
        s[1,4,0,0]=(0.421930869494)*x[0]**o+(0.889038749248)*x[0]+(-0.989075156799)*x[1]**o+(-0.614721833409)*x[1]+(0.112500578762)*x[2]**o+(-0.47083438608)*x[2]
        s[1,4,0,1]=(-0.0180470731329)*x[0]**o+(-0.47539878993)*x[0]+(0.317084518451)*x[1]**o+(0.237846416515)*x[1]+(0.375600671255)*x[2]**o+(0.66276020876)*x[2]
        s[1,4,1,0]=(0.125983672606)*x[0]**o+(0.595653397427)*x[0]+(0.31474692021)*x[1]**o+(0.234488164569)*x[1]+(0.618714006094)*x[2]**o+(-0.160776384119)*x[2]
        s[1,4,1,1]=(0.951917864448)*x[0]**o+(-0.33473954392)*x[0]+(-0.0378707336313)*x[1]**o+(-0.238158748773)*x[1]+(0.574999874285)*x[2]**o+(0.232302306417)*x[2]
        s[1,4,2,0]=(0.505450486119)*x[0]**o+(-0.370259193476)*x[0]+(-0.663053563021)*x[1]**o+(0.669872208578)*x[1]+(0.931794239064)*x[2]**o+(0.133184889523)*x[2]
        s[1,4,2,1]=(-0.937476724928)*x[0]**o+(0.110843923089)*x[0]+(-0.425245370966)*x[1]**o+(0.409116775141)*x[1]+(-0.273590511289)*x[2]**o+(0.441443512397)*x[2]
        s[2,0,0,0]=(0.0599017312855)*x[0]**o+(-0.941401999676)*x[0]+(0.379283945566)*x[1]**o+(-0.336352358178)*x[1]+(-0.514408042486)*x[2]**o+(-0.169011791163)*x[2]
        s[2,0,0,1]=(-0.269712116236)*x[0]**o+(-0.349623235155)*x[0]+(0.522466035838)*x[1]**o+(0.427111359806)*x[1]+(-0.0296663851565)*x[2]**o+(-0.0191832326161)*x[2]
        s[2,0,1,0]=(0.14892181923)*x[0]**o+(0.269460608852)*x[0]+(-0.0343896163306)*x[1]**o+(0.590044957948)*x[1]+(0.946719737501)*x[2]**o+(0.315742744827)*x[2]
        s[2,0,1,1]=(-0.388655118673)*x[0]**o+(-0.00180659903075)*x[0]+(0.969087798155)*x[1]**o+(0.86635225912)*x[1]+(-0.761434218549)*x[2]**o+(-0.323637696199)*x[2]
        s[2,0,2,0]=(0.588641788156)*x[0]**o+(-0.00912456965621)*x[0]+(-0.0266085527484)*x[1]**o+(0.0598075913127)*x[1]+(-0.658095450927)*x[2]**o+(0.110978909962)*x[2]
        s[2,0,2,1]=(0.139641588755)*x[0]**o+(0.593809622136)*x[0]+(-0.259839406352)*x[1]**o+(0.848541675808)*x[1]+(-0.860886035055)*x[2]**o+(-0.489752174489)*x[2]
        s[2,1,0,0]=(0.850328373883)*x[0]**o+(0.675383216776)*x[0]+(-0.117397818856)*x[1]**o+(-0.344801013716)*x[1]+(-0.887091513121)*x[2]**o+(0.584872468856)*x[2]
        s[2,1,0,1]=(-0.745901868498)*x[0]**o+(-0.407149004368)*x[0]+(-0.182253232897)*x[1]**o+(-0.203000014779)*x[1]+(-0.172710156372)*x[2]**o+(0.127673047341)*x[2]
        s[2,1,1,0]=(0.655182501192)*x[0]**o+(-0.17668945533)*x[0]+(0.717711469841)*x[1]**o+(0.595777555767)*x[1]+(-0.974184895311)*x[2]**o+(-0.0299700820305)*x[2]
        s[2,1,1,1]=(0.707793528102)*x[0]**o+(-0.0624977281364)*x[0]+(0.623777123073)*x[1]**o+(-0.794217090978)*x[1]+(0.542320017675)*x[2]**o+(0.903852498061)*x[2]
        s[2,1,2,0]=(0.713295120131)*x[0]**o+(-0.174892054913)*x[0]+(-0.251138558622)*x[1]**o+(-0.77735156875)*x[1]+(-0.94176928221)*x[2]**o+(-0.0244708586622)*x[2]
        s[2,1,2,1]=(0.933976789519)*x[0]**o+(0.90177842055)*x[0]+(-0.343254953267)*x[1]**o+(0.713448014636)*x[1]+(-0.468875743251)*x[2]**o+(0.350869684481)*x[2]
        s[2,2,0,0]=(0.390367713904)*x[0]**o+(-0.601608385293)*x[0]+(-0.757340195841)*x[1]**o+(0.360436452755)*x[1]+(0.291087135717)*x[2]**o+(0.316481151314)*x[2]
        s[2,2,0,1]=(0.111600306678)*x[0]**o+(0.823704129888)*x[0]+(0.682856997317)*x[1]**o+(-0.523736197366)*x[1]+(-0.100121183783)*x[2]**o+(-0.666230008262)*x[2]
        s[2,2,1,0]=(0.173360884658)*x[0]**o+(0.949511779713)*x[0]+(0.259889008248)*x[1]**o+(-0.942214191047)*x[1]+(-0.48690925565)*x[2]**o+(-0.264798193102)*x[2]
        s[2,2,1,1]=(0.483759255276)*x[0]**o+(-0.657669281632)*x[0]+(-0.154764754082)*x[1]**o+(-0.881550854822)*x[1]+(0.523812678208)*x[2]**o+(-0.979914768652)*x[2]
        s[2,2,2,0]=(0.137127593491)*x[0]**o+(-0.104908009627)*x[0]+(0.579522653005)*x[1]**o+(0.844822805269)*x[1]+(0.0328079119318)*x[2]**o+(-0.26897713517)*x[2]
        s[2,2,2,1]=(0.377116909347)*x[0]**o+(0.752578358605)*x[0]+(-0.467992979597)*x[1]**o+(0.254761405884)*x[1]+(-0.391559118991)*x[2]**o+(-0.61268413168)*x[2]
        s[2,3,0,0]=(-0.989947766953)*x[0]**o+(0.830717454791)*x[0]+(-0.473161731716)*x[1]**o+(0.625028238242)*x[1]+(-0.546708564025)*x[2]**o+(-0.691027642154)*x[2]
        s[2,3,0,1]=(0.650495990975)*x[0]**o+(-0.654863347162)*x[0]+(0.638342908241)*x[1]**o+(-0.0485838638503)*x[1]+(-0.632091721289)*x[2]**o+(0.791203973276)*x[2]
        s[2,3,1,0]=(0.266156252436)*x[0]**o+(0.272153723938)*x[0]+(-0.159895797204)*x[1]**o+(-0.640350200995)*x[1]+(0.990601940876)*x[2]**o+(-0.0816835237084)*x[2]
        s[2,3,1,1]=(0.562807345614)*x[0]**o+(0.544038968862)*x[0]+(0.878928928869)*x[1]**o+(0.404794709914)*x[1]+(0.269928450336)*x[2]**o+(0.869832826526)*x[2]
        s[2,3,2,0]=(-0.899170106614)*x[0]**o+(-0.031251129011)*x[0]+(0.0696989801411)*x[1]**o+(0.89839882525)*x[1]+(0.99905365333)*x[2]**o+(-0.856781333049)*x[2]
        s[2,3,2,1]=(0.980091885122)*x[0]**o+(-0.12832421228)*x[0]+(0.0798899702085)*x[1]**o+(-0.729861980386)*x[1]+(-0.970268210161)*x[2]**o+(0.319025854228)*x[2]
        s[2,4,0,0]=(0.891703788308)*x[0]**o+(-0.0724026917568)*x[0]+(-0.74794796925)*x[1]**o+(-0.339878454118)*x[1]+(0.413228256742)*x[2]**o+(-0.371024671724)*x[2]
        s[2,4,0,1]=(0.640892618319)*x[0]**o+(-0.44510403082)*x[0]+(-0.100468205238)*x[1]**o+(0.923901932578)*x[1]+(0.731059415968)*x[2]**o+(-0.609224901759)*x[2]
        s[2,4,1,0]=(0.130333662915)*x[0]**o+(0.394752475547)*x[0]+(-0.11866957093)*x[1]**o+(0.980649245967)*x[1]+(-0.602353311004)*x[2]**o+(0.406199522673)*x[2]
        s[2,4,1,1]=(0.0665437323779)*x[0]**o+(0.952105166153)*x[0]+(0.488465603925)*x[1]**o+(0.0178900500949)*x[1]+(0.206584014652)*x[2]**o+(0.446123239912)*x[2]
        s[2,4,2,0]=(-0.792540868222)*x[0]**o+(0.0265488816348)*x[0]+(-0.693338065664)*x[1]**o+(-0.682629782298)*x[1]+(-0.778972053935)*x[2]**o+(-0.0274648446373)*x[2]
        s[2,4,2,1]=(0.916389263123)*x[0]**o+(-0.662528447313)*x[0]+(-0.0703995406002)*x[1]**o+(0.385180574963)*x[1]+(-0.855872398416)*x[2]**o+(0.5530510507)*x[2]
        s[3,0,0,0]=(-0.306821143583)*x[0]**o+(-0.962173177288)*x[0]+(0.993016856406)*x[1]**o+(-0.579698238302)*x[1]+(0.571895736521)*x[2]**o+(-0.88966756316)*x[2]
        s[3,0,0,1]=(0.719926546559)*x[0]**o+(-0.0858941607816)*x[0]+(0.807242676458)*x[1]**o+(-0.190440184356)*x[1]+(-0.136927444751)*x[2]**o+(0.869886135543)*x[2]
        s[3,0,1,0]=(0.892148659939)*x[0]**o+(-0.482479220378)*x[0]+(0.503671753254)*x[1]**o+(-0.552769467575)*x[1]+(0.138159512264)*x[2]**o+(0.102674771614)*x[2]
        s[3,0,1,1]=(0.185256299506)*x[0]**o+(-0.369351326706)*x[0]+(0.220953483691)*x[1]**o+(-0.0131034510397)*x[1]+(0.956037877433)*x[2]**o+(0.568289950328)*x[2]
        s[3,0,2,0]=(0.045864662475)*x[0]**o+(0.0183434079897)*x[0]+(-0.428111425773)*x[1]**o+(0.254667250574)*x[1]+(-0.983322740032)*x[2]**o+(0.877555405709)*x[2]
        s[3,0,2,1]=(-0.408521811841)*x[0]**o+(0.00640043590699)*x[0]+(-0.105754378317)*x[1]**o+(-0.952486645506)*x[1]+(-0.725225188014)*x[2]**o+(-0.205556627888)*x[2]
        s[3,1,0,0]=(-0.944493178271)*x[0]**o+(0.810364339849)*x[0]+(0.705003782307)*x[1]**o+(-0.880195742246)*x[1]+(-0.843196216793)*x[2]**o+(-0.971449880924)*x[2]
        s[3,1,0,1]=(0.150616114959)*x[0]**o+(-0.717321007987)*x[0]+(0.194099301116)*x[1]**o+(-0.0345362551173)*x[1]+(0.375587089675)*x[2]**o+(-0.409625637147)*x[2]
        s[3,1,1,0]=(0.0802340794764)*x[0]**o+(0.0153666337202)*x[0]+(-0.782772322072)*x[1]**o+(-0.0965866811611)*x[1]+(0.725231231559)*x[2]**o+(-0.452016239379)*x[2]
        s[3,1,1,1]=(0.546956847386)*x[0]**o+(0.839322197764)*x[0]+(-0.735023211892)*x[1]**o+(-0.41772545319)*x[1]+(-0.843610604975)*x[2]**o+(-0.145546519925)*x[2]
        s[3,1,2,0]=(0.729170600381)*x[0]**o+(-0.551244794813)*x[0]+(0.846396830038)*x[1]**o+(-0.759556129789)*x[1]+(0.136640189225)*x[2]**o+(-0.379654525677)*x[2]
        s[3,1,2,1]=(-0.953857643507)*x[0]**o+(-0.304074878681)*x[0]+(0.472716648295)*x[1]**o+(-0.179292020611)*x[1]+(0.909646409116)*x[2]**o+(-0.149337148625)*x[2]
        s[3,2,0,0]=(-0.165651506554)*x[0]**o+(0.410286556434)*x[0]+(-0.385291761561)*x[1]**o+(-0.932290142267)*x[1]+(0.562269759333)*x[2]**o+(-0.516760168836)*x[2]
        s[3,2,0,1]=(-0.874391348276)*x[0]**o+(-0.626614642393)*x[0]+(0.135326235872)*x[1]**o+(-0.169497542734)*x[1]+(0.453312712391)*x[2]**o+(0.699294330345)*x[2]
        s[3,2,1,0]=(0.30374985921)*x[0]**o+(-0.66286753305)*x[0]+(-0.504388407872)*x[1]**o+(-0.0864778873713)*x[1]+(0.7780825189)*x[2]**o+(0.607545865231)*x[2]
        s[3,2,1,1]=(0.323217898368)*x[0]**o+(0.256285070904)*x[0]+(0.32326826477)*x[1]**o+(-0.983183590335)*x[1]+(0.530605958132)*x[2]**o+(-0.109640854019)*x[2]
        s[3,2,2,0]=(0.587862067621)*x[0]**o+(-0.951653317735)*x[0]+(0.089519431923)*x[1]**o+(-0.780330592384)*x[1]+(-0.0483676475341)*x[2]**o+(0.766538955993)*x[2]
        s[3,2,2,1]=(-0.957533418498)*x[0]**o+(-0.681231272445)*x[0]+(-0.0918289718767)*x[1]**o+(0.38725634145)*x[1]+(0.26451874534)*x[2]**o+(-0.901046807078)*x[2]
        s[3,3,0,0]=(0.604215052532)*x[0]**o+(-0.0188225127963)*x[0]+(-0.544365413309)*x[1]**o+(-0.837547618266)*x[1]+(0.506665562924)*x[2]**o+(0.868580855034)*x[2]
        s[3,3,0,1]=(0.307620045896)*x[0]**o+(-0.941702619274)*x[0]+(0.467078433616)*x[1]**o+(-0.525971373276)*x[1]+(-0.438681578224)*x[2]**o+(-0.865538708837)*x[2]
        s[3,3,1,0]=(-0.128299566207)*x[0]**o+(0.0526714707626)*x[0]+(-0.194464943133)*x[1]**o+(0.848343510106)*x[1]+(-0.843042290362)*x[2]**o+(-0.384441684551)*x[2]
        s[3,3,1,1]=(-0.0202567344895)*x[0]**o+(0.919037490786)*x[0]+(0.953616123775)*x[1]**o+(-0.359010584029)*x[1]+(-0.892651116514)*x[2]**o+(0.900116594426)*x[2]
        s[3,3,2,0]=(-0.196701204528)*x[0]**o+(0.173992581268)*x[0]+(0.0304200566544)*x[1]**o+(-0.562311121578)*x[1]+(-0.349902986645)*x[2]**o+(0.137392211876)*x[2]
        s[3,3,2,1]=(-0.264756448582)*x[0]**o+(0.104810865018)*x[0]+(-0.550559232481)*x[1]**o+(-0.621292330017)*x[1]+(-0.697096182063)*x[2]**o+(-0.137686685739)*x[2]
        s[3,4,0,0]=(-0.0500780621237)*x[0]**o+(-0.842196022641)*x[0]+(-0.375688956182)*x[1]**o+(0.293383514141)*x[1]+(-0.679863519997)*x[2]**o+(-0.108799395703)*x[2]
        s[3,4,0,1]=(-0.825663711773)*x[0]**o+(0.0811924383936)*x[0]+(0.658753764102)*x[1]**o+(0.213557161034)*x[1]+(0.048778548639)*x[2]**o+(0.308220587221)*x[2]
        s[3,4,1,0]=(-0.0366332011016)*x[0]**o+(-0.935060469197)*x[0]+(0.52021729799)*x[1]**o+(0.270580190954)*x[1]+(0.690657153532)*x[2]**o+(0.572985687766)*x[2]
        s[3,4,1,1]=(-0.358041857957)*x[0]**o+(0.151420239879)*x[0]+(0.578841565624)*x[1]**o+(0.379085642891)*x[1]+(0.231447762893)*x[2]**o+(-0.494885835203)*x[2]
        s[3,4,2,0]=(0.689670410742)*x[0]**o+(-0.469898987295)*x[0]+(-0.617664742358)*x[1]**o+(-0.92098566662)*x[1]+(0.535497980872)*x[2]**o+(-0.660036711933)*x[2]
        s[3,4,2,1]=(-0.443846183672)*x[0]**o+(-0.0606185138497)*x[0]+(0.902723111093)*x[1]**o+(-0.201894548349)*x[1]+(0.650998336304)*x[2]**o+(-0.936010399236)*x[2]
        ref[0,0,0,0]=(0.186525987907)*x_ref[0]**o+(-0.690917154378)*x_ref[0]+(0.832803193217)*x_ref[1]**o+(-0.0440294807629)*x_ref[1]+(0.936927757561)*x_ref[2]**o+(-0.120763864664)*x_ref[2]
        ref[0,0,0,1]=(0.0113224706745)*x_ref[0]**o+(0.522015747536)*x_ref[0]+(-0.281875411547)*x_ref[1]**o+(0.605176091896)*x_ref[1]+(-0.448130660869)*x_ref[2]**o+(0.557667991786)*x_ref[2]
        ref[0,0,1,0]=(-0.5115411507)*x_ref[0]**o+(0.568051998295)*x_ref[0]+(0.722442651334)*x_ref[1]**o+(0.410822384229)*x_ref[1]+(0.235364385145)*x_ref[2]**o+(0.881645768144)*x_ref[2]
        ref[0,0,1,1]=(0.365191539085)*x_ref[0]**o+(0.0518813698411)*x_ref[0]+(0.179359856471)*x_ref[1]**o+(-0.634486389415)*x_ref[1]+(0.311578648976)*x_ref[2]**o+(-0.157034397126)*x_ref[2]
        ref[0,0,2,0]=(0.340474667312)*x_ref[0]**o+(-0.761761779074)*x_ref[0]+(-0.118701824049)*x_ref[1]**o+(0.0922924561625)*x_ref[1]+(0.132924567807)*x_ref[2]**o+(-0.0694502802118)*x_ref[2]
        ref[0,0,2,1]=(0.21666407868)*x_ref[0]**o+(0.65101030565)*x_ref[0]+(0.0978745307399)*x_ref[1]**o+(-0.592357988084)*x_ref[1]+(-0.605382873404)*x_ref[2]**o+(-0.899153475467)*x_ref[2]
        ref[0,1,0,0]=(-0.239693597874)*x_ref[0]**o+(-0.265899323452)*x_ref[0]+(-0.210985518909)*x_ref[1]**o+(-0.408445623906)*x_ref[1]+(0.350622495806)*x_ref[2]**o+(-0.534920839922)*x_ref[2]
        ref[0,1,0,1]=(-0.596708903276)*x_ref[0]**o+(0.241677381533)*x_ref[0]+(-0.327063865807)*x_ref[1]**o+(0.731700322335)*x_ref[1]+(-0.29224310646)*x_ref[2]**o+(-0.177855845822)*x_ref[2]
        ref[0,1,1,0]=(0.13119146515)*x_ref[0]**o+(0.24155441457)*x_ref[0]+(-0.730354339458)*x_ref[1]**o+(0.17509896433)*x_ref[1]+(0.602886357198)*x_ref[2]**o+(-0.300613826457)*x_ref[2]
        ref[0,1,1,1]=(-0.528650609789)*x_ref[0]**o+(0.730375830185)*x_ref[0]+(0.706889523794)*x_ref[1]**o+(-0.280144821714)*x_ref[1]+(0.261124676699)*x_ref[2]**o+(0.230842251108)*x_ref[2]
        ref[0,1,2,0]=(0.0867102525865)*x_ref[0]**o+(0.238707033898)*x_ref[0]+(-0.914453619505)*x_ref[1]**o+(0.224533380834)*x_ref[1]+(0.406716858519)*x_ref[2]**o+(-0.347408293194)*x_ref[2]
        ref[0,1,2,1]=(-0.567166350757)*x_ref[0]**o+(-0.156169071272)*x_ref[0]+(0.15846212695)*x_ref[1]**o+(-0.0254016167751)*x_ref[1]+(0.707956392472)*x_ref[2]**o+(-0.510965658867)*x_ref[2]
        ref[0,2,0,0]=(-0.181235844201)*x_ref[0]**o+(0.936656137873)*x_ref[0]+(0.830895834923)*x_ref[1]**o+(0.752514827093)*x_ref[1]+(0.478651458614)*x_ref[2]**o+(0.353659575913)*x_ref[2]
        ref[0,2,0,1]=(-0.0466341076413)*x_ref[0]**o+(-0.137913043533)*x_ref[0]+(0.569602506318)*x_ref[1]**o+(0.453951166085)*x_ref[1]+(-0.736500514341)*x_ref[2]**o+(0.494877047573)*x_ref[2]
        ref[0,2,1,0]=(-0.774800275665)*x_ref[0]**o+(0.320634390869)*x_ref[0]+(-0.801898551402)*x_ref[1]**o+(0.680379650404)*x_ref[1]+(0.115138386556)*x_ref[2]**o+(0.847285566367)*x_ref[2]
        ref[0,2,1,1]=(-0.948363366443)*x_ref[0]**o+(-0.401155889062)*x_ref[0]+(0.602001652794)*x_ref[1]**o+(0.881111991966)*x_ref[1]+(-0.407826312421)*x_ref[2]**o+(-0.385793087535)*x_ref[2]
        ref[0,2,2,0]=(-0.0498782718626)*x_ref[0]**o+(0.278903888089)*x_ref[0]+(0.28455096215)*x_ref[1]**o+(0.165496315478)*x_ref[1]+(-0.516044235314)*x_ref[2]**o+(-0.320130768929)*x_ref[2]
        ref[0,2,2,1]=(-0.0313287552123)*x_ref[0]**o+(0.795534322833)*x_ref[0]+(0.0672014159699)*x_ref[1]**o+(-0.232632859958)*x_ref[1]+(0.968980662087)*x_ref[2]**o+(-0.0437975661275)*x_ref[2]
        ref[0,3,0,0]=(0.879723765854)*x_ref[0]**o+(0.616502035178)*x_ref[0]+(-0.888143209368)*x_ref[1]**o+(-0.954918894054)*x_ref[1]+(-0.0649197025038)*x_ref[2]**o+(-0.865957161833)*x_ref[2]
        ref[0,3,0,1]=(0.550691528997)*x_ref[0]**o+(0.136724942999)*x_ref[0]+(0.474412879455)*x_ref[1]**o+(0.677263937069)*x_ref[1]+(0.453101604697)*x_ref[2]**o+(0.133742079633)*x_ref[2]
        ref[0,3,1,0]=(-0.970027293215)*x_ref[0]**o+(-0.610696832357)*x_ref[0]+(-0.248732897836)*x_ref[1]**o+(0.957868248702)*x_ref[1]+(0.334784026675)*x_ref[2]**o+(0.696895349867)*x_ref[2]
        ref[0,3,1,1]=(-0.672808837142)*x_ref[0]**o+(-0.0553936755153)*x_ref[0]+(-0.00732320498388)*x_ref[1]**o+(0.49021088588)*x_ref[1]+(0.907554472651)*x_ref[2]**o+(-0.058244899423)*x_ref[2]
        ref[0,3,2,0]=(0.0159001870399)*x_ref[0]**o+(0.0227678146243)*x_ref[0]+(0.805708466496)*x_ref[1]**o+(0.785990910247)*x_ref[1]+(-0.725672039232)*x_ref[2]**o+(-0.162870251036)*x_ref[2]
        ref[0,3,2,1]=(0.520705647829)*x_ref[0]**o+(-0.310310041859)*x_ref[0]+(-0.317118647815)*x_ref[1]**o+(0.534773934168)*x_ref[1]+(-0.268620331386)*x_ref[2]**o+(0.334022726024)*x_ref[2]
        ref[0,4,0,0]=(0.529035051582)*x_ref[0]**o+(0.566819545868)*x_ref[0]+(0.618896518995)*x_ref[1]**o+(0.263612275958)*x_ref[1]+(-0.618741920385)*x_ref[2]**o+(-0.0264643721073)*x_ref[2]
        ref[0,4,0,1]=(0.860757010858)*x_ref[0]**o+(-0.706974137596)*x_ref[0]+(0.0612556623739)*x_ref[1]**o+(0.722512189141)*x_ref[1]+(-0.676611378845)*x_ref[2]**o+(-0.615875915177)*x_ref[2]
        ref[0,4,1,0]=(-0.186509714673)*x_ref[0]**o+(0.0351164428631)*x_ref[0]+(-0.173768366165)*x_ref[1]**o+(-0.426776871363)*x_ref[1]+(-0.0678988589984)*x_ref[2]**o+(-0.0164388017801)*x_ref[2]
        ref[0,4,1,1]=(-0.528750419399)*x_ref[0]**o+(-0.0188958007868)*x_ref[0]+(-0.381850037026)*x_ref[1]**o+(-0.366474181406)*x_ref[1]+(-0.0157097200919)*x_ref[2]**o+(0.750818168768)*x_ref[2]
        ref[0,4,2,0]=(0.962983200785)*x_ref[0]**o+(-0.904671719523)*x_ref[0]+(0.83012715029)*x_ref[1]**o+(0.20786351761)*x_ref[1]+(0.61032830083)*x_ref[2]**o+(0.999068101126)*x_ref[2]
        ref[0,4,2,1]=(0.369225553113)*x_ref[0]**o+(0.370643799864)*x_ref[0]+(0.562458197002)*x_ref[1]**o+(0.864829790167)*x_ref[1]+(-0.507288312866)*x_ref[2]**o+(-0.615656522393)*x_ref[2]
        ref[1,0,0,0]=(0.0208552161791)*x_ref[0]**o+(0.64767817505)*x_ref[0]+(-0.913625079536)*x_ref[1]**o+(0.214341575088)*x_ref[1]+(-0.369358777546)*x_ref[2]**o+(-0.744597104081)*x_ref[2]
        ref[1,0,0,1]=(0.688956912909)*x_ref[0]**o+(0.130856033181)*x_ref[0]+(-0.754570690839)*x_ref[1]**o+(-0.892591311673)*x_ref[1]+(0.121513821098)*x_ref[2]**o+(0.0499651928086)*x_ref[2]
        ref[1,0,1,0]=(0.488371732492)*x_ref[0]**o+(-0.0899662322511)*x_ref[0]+(-0.184052688998)*x_ref[1]**o+(-0.504791015013)*x_ref[1]+(0.939413170478)*x_ref[2]**o+(-0.371392738292)*x_ref[2]
        ref[1,0,1,1]=(-0.634015136517)*x_ref[0]**o+(0.189031198575)*x_ref[0]+(-0.187597065324)*x_ref[1]**o+(0.417035820606)*x_ref[1]+(-0.640693898356)*x_ref[2]**o+(-0.356999823488)*x_ref[2]
        ref[1,0,2,0]=(-0.299739907169)*x_ref[0]**o+(0.97973415315)*x_ref[0]+(0.172728079372)*x_ref[1]**o+(-0.219250264223)*x_ref[1]+(0.0377842392779)*x_ref[2]**o+(0.568605450772)*x_ref[2]
        ref[1,0,2,1]=(0.668822572798)*x_ref[0]**o+(-0.464904703232)*x_ref[0]+(-0.0438179224843)*x_ref[1]**o+(0.972577571624)*x_ref[1]+(0.200473119295)*x_ref[2]**o+(0.769536551131)*x_ref[2]
        ref[1,1,0,0]=(-0.5952291144)*x_ref[0]**o+(0.215912343317)*x_ref[0]+(-0.187871783507)*x_ref[1]**o+(0.000141658119764)*x_ref[1]+(0.652350178895)*x_ref[2]**o+(0.748942594082)*x_ref[2]
        ref[1,1,0,1]=(0.978202121028)*x_ref[0]**o+(-0.270952075217)*x_ref[0]+(0.379211795706)*x_ref[1]**o+(0.570614905933)*x_ref[1]+(0.531348311358)*x_ref[2]**o+(-0.774802192871)*x_ref[2]
        ref[1,1,1,0]=(-0.173449381518)*x_ref[0]**o+(-0.788533061983)*x_ref[0]+(-0.159503791119)*x_ref[1]**o+(-0.182774291708)*x_ref[1]+(0.493985220422)*x_ref[2]**o+(-0.442523611538)*x_ref[2]
        ref[1,1,1,1]=(-0.196642522444)*x_ref[0]**o+(0.353038689926)*x_ref[0]+(0.739133358128)*x_ref[1]**o+(0.384564329164)*x_ref[1]+(0.00724850200888)*x_ref[2]**o+(-0.534791252863)*x_ref[2]
        ref[1,1,2,0]=(-0.742601262273)*x_ref[0]**o+(0.870936220364)*x_ref[0]+(0.173515343981)*x_ref[1]**o+(-0.924818265666)*x_ref[1]+(-0.786225297604)*x_ref[2]**o+(0.723191951357)*x_ref[2]
        ref[1,1,2,1]=(0.584821708581)*x_ref[0]**o+(0.233692617647)*x_ref[0]+(0.917976018627)*x_ref[1]**o+(0.0411719886893)*x_ref[1]+(-0.731518066297)*x_ref[2]**o+(-0.886641339023)*x_ref[2]
        ref[1,2,0,0]=(0.879019029124)*x_ref[0]**o+(0.198224253024)*x_ref[0]+(-0.508879741455)*x_ref[1]**o+(-0.0493693577221)*x_ref[1]+(-0.200204490189)*x_ref[2]**o+(-0.558819359406)*x_ref[2]
        ref[1,2,0,1]=(0.343696269765)*x_ref[0]**o+(-0.694690909901)*x_ref[0]+(0.562857727334)*x_ref[1]**o+(0.170105867024)*x_ref[1]+(0.300442565793)*x_ref[2]**o+(-0.78443966073)*x_ref[2]
        ref[1,2,1,0]=(-0.104227942738)*x_ref[0]**o+(-0.631808216742)*x_ref[0]+(-0.688416357176)*x_ref[1]**o+(0.543474609002)*x_ref[1]+(-0.475674253864)*x_ref[2]**o+(-0.622363009599)*x_ref[2]
        ref[1,2,1,1]=(-0.187993381022)*x_ref[0]**o+(-0.112771930635)*x_ref[0]+(0.231591342365)*x_ref[1]**o+(-0.619972979708)*x_ref[1]+(-0.633470156859)*x_ref[2]**o+(0.726102617311)*x_ref[2]
        ref[1,2,2,0]=(-0.0929871065489)*x_ref[0]**o+(-0.741493547909)*x_ref[0]+(-0.500399374101)*x_ref[1]**o+(0.489221111027)*x_ref[1]+(-0.538850386612)*x_ref[2]**o+(-0.366843628744)*x_ref[2]
        ref[1,2,2,1]=(-0.449265426222)*x_ref[0]**o+(-0.711133901403)*x_ref[0]+(-0.671933617866)*x_ref[1]**o+(-0.75650179505)*x_ref[1]+(-0.441794206618)*x_ref[2]**o+(-0.679799182186)*x_ref[2]
        ref[1,3,0,0]=(0.0794555795393)*x_ref[0]**o+(-0.591466474089)*x_ref[0]+(0.537807349728)*x_ref[1]**o+(-0.0295904103007)*x_ref[1]+(-0.206212961776)*x_ref[2]**o+(0.633837648242)*x_ref[2]
        ref[1,3,0,1]=(0.232707718354)*x_ref[0]**o+(-0.310438878448)*x_ref[0]+(-0.0844732229089)*x_ref[1]**o+(0.507469871796)*x_ref[1]+(-0.391898028012)*x_ref[2]**o+(-0.374582716512)*x_ref[2]
        ref[1,3,1,0]=(-0.166926427703)*x_ref[0]**o+(-0.926298908928)*x_ref[0]+(0.244425001632)*x_ref[1]**o+(0.284615289051)*x_ref[1]+(-0.675054769717)*x_ref[2]**o+(-0.0765552057049)*x_ref[2]
        ref[1,3,1,1]=(-0.469538130468)*x_ref[0]**o+(-0.110820901289)*x_ref[0]+(-0.608743940572)*x_ref[1]**o+(0.0606943645263)*x_ref[1]+(-0.211127814891)*x_ref[2]**o+(-0.0474195944349)*x_ref[2]
        ref[1,3,2,0]=(0.769147852508)*x_ref[0]**o+(-0.500794487532)*x_ref[0]+(-0.0971530930356)*x_ref[1]**o+(0.282178880991)*x_ref[1]+(-0.175125963785)*x_ref[2]**o+(0.0567340005296)*x_ref[2]
        ref[1,3,2,1]=(0.14422447964)*x_ref[0]**o+(-0.9056785988)*x_ref[0]+(-0.962346923857)*x_ref[1]**o+(0.64076498247)*x_ref[1]+(0.77680599777)*x_ref[2]**o+(0.450703438151)*x_ref[2]
        ref[1,4,0,0]=(0.421930869494)*x_ref[0]**o+(0.889038749248)*x_ref[0]+(-0.989075156799)*x_ref[1]**o+(-0.614721833409)*x_ref[1]+(0.112500578762)*x_ref[2]**o+(-0.47083438608)*x_ref[2]
        ref[1,4,0,1]=(-0.0180470731329)*x_ref[0]**o+(-0.47539878993)*x_ref[0]+(0.317084518451)*x_ref[1]**o+(0.237846416515)*x_ref[1]+(0.375600671255)*x_ref[2]**o+(0.66276020876)*x_ref[2]
        ref[1,4,1,0]=(0.125983672606)*x_ref[0]**o+(0.595653397427)*x_ref[0]+(0.31474692021)*x_ref[1]**o+(0.234488164569)*x_ref[1]+(0.618714006094)*x_ref[2]**o+(-0.160776384119)*x_ref[2]
        ref[1,4,1,1]=(0.951917864448)*x_ref[0]**o+(-0.33473954392)*x_ref[0]+(-0.0378707336313)*x_ref[1]**o+(-0.238158748773)*x_ref[1]+(0.574999874285)*x_ref[2]**o+(0.232302306417)*x_ref[2]
        ref[1,4,2,0]=(0.505450486119)*x_ref[0]**o+(-0.370259193476)*x_ref[0]+(-0.663053563021)*x_ref[1]**o+(0.669872208578)*x_ref[1]+(0.931794239064)*x_ref[2]**o+(0.133184889523)*x_ref[2]
        ref[1,4,2,1]=(-0.937476724928)*x_ref[0]**o+(0.110843923089)*x_ref[0]+(-0.425245370966)*x_ref[1]**o+(0.409116775141)*x_ref[1]+(-0.273590511289)*x_ref[2]**o+(0.441443512397)*x_ref[2]
        ref[2,0,0,0]=(0.0599017312855)*x_ref[0]**o+(-0.941401999676)*x_ref[0]+(0.379283945566)*x_ref[1]**o+(-0.336352358178)*x_ref[1]+(-0.514408042486)*x_ref[2]**o+(-0.169011791163)*x_ref[2]
        ref[2,0,0,1]=(-0.269712116236)*x_ref[0]**o+(-0.349623235155)*x_ref[0]+(0.522466035838)*x_ref[1]**o+(0.427111359806)*x_ref[1]+(-0.0296663851565)*x_ref[2]**o+(-0.0191832326161)*x_ref[2]
        ref[2,0,1,0]=(0.14892181923)*x_ref[0]**o+(0.269460608852)*x_ref[0]+(-0.0343896163306)*x_ref[1]**o+(0.590044957948)*x_ref[1]+(0.946719737501)*x_ref[2]**o+(0.315742744827)*x_ref[2]
        ref[2,0,1,1]=(-0.388655118673)*x_ref[0]**o+(-0.00180659903075)*x_ref[0]+(0.969087798155)*x_ref[1]**o+(0.86635225912)*x_ref[1]+(-0.761434218549)*x_ref[2]**o+(-0.323637696199)*x_ref[2]
        ref[2,0,2,0]=(0.588641788156)*x_ref[0]**o+(-0.00912456965621)*x_ref[0]+(-0.0266085527484)*x_ref[1]**o+(0.0598075913127)*x_ref[1]+(-0.658095450927)*x_ref[2]**o+(0.110978909962)*x_ref[2]
        ref[2,0,2,1]=(0.139641588755)*x_ref[0]**o+(0.593809622136)*x_ref[0]+(-0.259839406352)*x_ref[1]**o+(0.848541675808)*x_ref[1]+(-0.860886035055)*x_ref[2]**o+(-0.489752174489)*x_ref[2]
        ref[2,1,0,0]=(0.850328373883)*x_ref[0]**o+(0.675383216776)*x_ref[0]+(-0.117397818856)*x_ref[1]**o+(-0.344801013716)*x_ref[1]+(-0.887091513121)*x_ref[2]**o+(0.584872468856)*x_ref[2]
        ref[2,1,0,1]=(-0.745901868498)*x_ref[0]**o+(-0.407149004368)*x_ref[0]+(-0.182253232897)*x_ref[1]**o+(-0.203000014779)*x_ref[1]+(-0.172710156372)*x_ref[2]**o+(0.127673047341)*x_ref[2]
        ref[2,1,1,0]=(0.655182501192)*x_ref[0]**o+(-0.17668945533)*x_ref[0]+(0.717711469841)*x_ref[1]**o+(0.595777555767)*x_ref[1]+(-0.974184895311)*x_ref[2]**o+(-0.0299700820305)*x_ref[2]
        ref[2,1,1,1]=(0.707793528102)*x_ref[0]**o+(-0.0624977281364)*x_ref[0]+(0.623777123073)*x_ref[1]**o+(-0.794217090978)*x_ref[1]+(0.542320017675)*x_ref[2]**o+(0.903852498061)*x_ref[2]
        ref[2,1,2,0]=(0.713295120131)*x_ref[0]**o+(-0.174892054913)*x_ref[0]+(-0.251138558622)*x_ref[1]**o+(-0.77735156875)*x_ref[1]+(-0.94176928221)*x_ref[2]**o+(-0.0244708586622)*x_ref[2]
        ref[2,1,2,1]=(0.933976789519)*x_ref[0]**o+(0.90177842055)*x_ref[0]+(-0.343254953267)*x_ref[1]**o+(0.713448014636)*x_ref[1]+(-0.468875743251)*x_ref[2]**o+(0.350869684481)*x_ref[2]
        ref[2,2,0,0]=(0.390367713904)*x_ref[0]**o+(-0.601608385293)*x_ref[0]+(-0.757340195841)*x_ref[1]**o+(0.360436452755)*x_ref[1]+(0.291087135717)*x_ref[2]**o+(0.316481151314)*x_ref[2]
        ref[2,2,0,1]=(0.111600306678)*x_ref[0]**o+(0.823704129888)*x_ref[0]+(0.682856997317)*x_ref[1]**o+(-0.523736197366)*x_ref[1]+(-0.100121183783)*x_ref[2]**o+(-0.666230008262)*x_ref[2]
        ref[2,2,1,0]=(0.173360884658)*x_ref[0]**o+(0.949511779713)*x_ref[0]+(0.259889008248)*x_ref[1]**o+(-0.942214191047)*x_ref[1]+(-0.48690925565)*x_ref[2]**o+(-0.264798193102)*x_ref[2]
        ref[2,2,1,1]=(0.483759255276)*x_ref[0]**o+(-0.657669281632)*x_ref[0]+(-0.154764754082)*x_ref[1]**o+(-0.881550854822)*x_ref[1]+(0.523812678208)*x_ref[2]**o+(-0.979914768652)*x_ref[2]
        ref[2,2,2,0]=(0.137127593491)*x_ref[0]**o+(-0.104908009627)*x_ref[0]+(0.579522653005)*x_ref[1]**o+(0.844822805269)*x_ref[1]+(0.0328079119318)*x_ref[2]**o+(-0.26897713517)*x_ref[2]
        ref[2,2,2,1]=(0.377116909347)*x_ref[0]**o+(0.752578358605)*x_ref[0]+(-0.467992979597)*x_ref[1]**o+(0.254761405884)*x_ref[1]+(-0.391559118991)*x_ref[2]**o+(-0.61268413168)*x_ref[2]
        ref[2,3,0,0]=(-0.989947766953)*x_ref[0]**o+(0.830717454791)*x_ref[0]+(-0.473161731716)*x_ref[1]**o+(0.625028238242)*x_ref[1]+(-0.546708564025)*x_ref[2]**o+(-0.691027642154)*x_ref[2]
        ref[2,3,0,1]=(0.650495990975)*x_ref[0]**o+(-0.654863347162)*x_ref[0]+(0.638342908241)*x_ref[1]**o+(-0.0485838638503)*x_ref[1]+(-0.632091721289)*x_ref[2]**o+(0.791203973276)*x_ref[2]
        ref[2,3,1,0]=(0.266156252436)*x_ref[0]**o+(0.272153723938)*x_ref[0]+(-0.159895797204)*x_ref[1]**o+(-0.640350200995)*x_ref[1]+(0.990601940876)*x_ref[2]**o+(-0.0816835237084)*x_ref[2]
        ref[2,3,1,1]=(0.562807345614)*x_ref[0]**o+(0.544038968862)*x_ref[0]+(0.878928928869)*x_ref[1]**o+(0.404794709914)*x_ref[1]+(0.269928450336)*x_ref[2]**o+(0.869832826526)*x_ref[2]
        ref[2,3,2,0]=(-0.899170106614)*x_ref[0]**o+(-0.031251129011)*x_ref[0]+(0.0696989801411)*x_ref[1]**o+(0.89839882525)*x_ref[1]+(0.99905365333)*x_ref[2]**o+(-0.856781333049)*x_ref[2]
        ref[2,3,2,1]=(0.980091885122)*x_ref[0]**o+(-0.12832421228)*x_ref[0]+(0.0798899702085)*x_ref[1]**o+(-0.729861980386)*x_ref[1]+(-0.970268210161)*x_ref[2]**o+(0.319025854228)*x_ref[2]
        ref[2,4,0,0]=(0.891703788308)*x_ref[0]**o+(-0.0724026917568)*x_ref[0]+(-0.74794796925)*x_ref[1]**o+(-0.339878454118)*x_ref[1]+(0.413228256742)*x_ref[2]**o+(-0.371024671724)*x_ref[2]
        ref[2,4,0,1]=(0.640892618319)*x_ref[0]**o+(-0.44510403082)*x_ref[0]+(-0.100468205238)*x_ref[1]**o+(0.923901932578)*x_ref[1]+(0.731059415968)*x_ref[2]**o+(-0.609224901759)*x_ref[2]
        ref[2,4,1,0]=(0.130333662915)*x_ref[0]**o+(0.394752475547)*x_ref[0]+(-0.11866957093)*x_ref[1]**o+(0.980649245967)*x_ref[1]+(-0.602353311004)*x_ref[2]**o+(0.406199522673)*x_ref[2]
        ref[2,4,1,1]=(0.0665437323779)*x_ref[0]**o+(0.952105166153)*x_ref[0]+(0.488465603925)*x_ref[1]**o+(0.0178900500949)*x_ref[1]+(0.206584014652)*x_ref[2]**o+(0.446123239912)*x_ref[2]
        ref[2,4,2,0]=(-0.792540868222)*x_ref[0]**o+(0.0265488816348)*x_ref[0]+(-0.693338065664)*x_ref[1]**o+(-0.682629782298)*x_ref[1]+(-0.778972053935)*x_ref[2]**o+(-0.0274648446373)*x_ref[2]
        ref[2,4,2,1]=(0.916389263123)*x_ref[0]**o+(-0.662528447313)*x_ref[0]+(-0.0703995406002)*x_ref[1]**o+(0.385180574963)*x_ref[1]+(-0.855872398416)*x_ref[2]**o+(0.5530510507)*x_ref[2]
        ref[3,0,0,0]=(-0.306821143583)*x_ref[0]**o+(-0.962173177288)*x_ref[0]+(0.993016856406)*x_ref[1]**o+(-0.579698238302)*x_ref[1]+(0.571895736521)*x_ref[2]**o+(-0.88966756316)*x_ref[2]
        ref[3,0,0,1]=(0.719926546559)*x_ref[0]**o+(-0.0858941607816)*x_ref[0]+(0.807242676458)*x_ref[1]**o+(-0.190440184356)*x_ref[1]+(-0.136927444751)*x_ref[2]**o+(0.869886135543)*x_ref[2]
        ref[3,0,1,0]=(0.892148659939)*x_ref[0]**o+(-0.482479220378)*x_ref[0]+(0.503671753254)*x_ref[1]**o+(-0.552769467575)*x_ref[1]+(0.138159512264)*x_ref[2]**o+(0.102674771614)*x_ref[2]
        ref[3,0,1,1]=(0.185256299506)*x_ref[0]**o+(-0.369351326706)*x_ref[0]+(0.220953483691)*x_ref[1]**o+(-0.0131034510397)*x_ref[1]+(0.956037877433)*x_ref[2]**o+(0.568289950328)*x_ref[2]
        ref[3,0,2,0]=(0.045864662475)*x_ref[0]**o+(0.0183434079897)*x_ref[0]+(-0.428111425773)*x_ref[1]**o+(0.254667250574)*x_ref[1]+(-0.983322740032)*x_ref[2]**o+(0.877555405709)*x_ref[2]
        ref[3,0,2,1]=(-0.408521811841)*x_ref[0]**o+(0.00640043590699)*x_ref[0]+(-0.105754378317)*x_ref[1]**o+(-0.952486645506)*x_ref[1]+(-0.725225188014)*x_ref[2]**o+(-0.205556627888)*x_ref[2]
        ref[3,1,0,0]=(-0.944493178271)*x_ref[0]**o+(0.810364339849)*x_ref[0]+(0.705003782307)*x_ref[1]**o+(-0.880195742246)*x_ref[1]+(-0.843196216793)*x_ref[2]**o+(-0.971449880924)*x_ref[2]
        ref[3,1,0,1]=(0.150616114959)*x_ref[0]**o+(-0.717321007987)*x_ref[0]+(0.194099301116)*x_ref[1]**o+(-0.0345362551173)*x_ref[1]+(0.375587089675)*x_ref[2]**o+(-0.409625637147)*x_ref[2]
        ref[3,1,1,0]=(0.0802340794764)*x_ref[0]**o+(0.0153666337202)*x_ref[0]+(-0.782772322072)*x_ref[1]**o+(-0.0965866811611)*x_ref[1]+(0.725231231559)*x_ref[2]**o+(-0.452016239379)*x_ref[2]
        ref[3,1,1,1]=(0.546956847386)*x_ref[0]**o+(0.839322197764)*x_ref[0]+(-0.735023211892)*x_ref[1]**o+(-0.41772545319)*x_ref[1]+(-0.843610604975)*x_ref[2]**o+(-0.145546519925)*x_ref[2]
        ref[3,1,2,0]=(0.729170600381)*x_ref[0]**o+(-0.551244794813)*x_ref[0]+(0.846396830038)*x_ref[1]**o+(-0.759556129789)*x_ref[1]+(0.136640189225)*x_ref[2]**o+(-0.379654525677)*x_ref[2]
        ref[3,1,2,1]=(-0.953857643507)*x_ref[0]**o+(-0.304074878681)*x_ref[0]+(0.472716648295)*x_ref[1]**o+(-0.179292020611)*x_ref[1]+(0.909646409116)*x_ref[2]**o+(-0.149337148625)*x_ref[2]
        ref[3,2,0,0]=(-0.165651506554)*x_ref[0]**o+(0.410286556434)*x_ref[0]+(-0.385291761561)*x_ref[1]**o+(-0.932290142267)*x_ref[1]+(0.562269759333)*x_ref[2]**o+(-0.516760168836)*x_ref[2]
        ref[3,2,0,1]=(-0.874391348276)*x_ref[0]**o+(-0.626614642393)*x_ref[0]+(0.135326235872)*x_ref[1]**o+(-0.169497542734)*x_ref[1]+(0.453312712391)*x_ref[2]**o+(0.699294330345)*x_ref[2]
        ref[3,2,1,0]=(0.30374985921)*x_ref[0]**o+(-0.66286753305)*x_ref[0]+(-0.504388407872)*x_ref[1]**o+(-0.0864778873713)*x_ref[1]+(0.7780825189)*x_ref[2]**o+(0.607545865231)*x_ref[2]
        ref[3,2,1,1]=(0.323217898368)*x_ref[0]**o+(0.256285070904)*x_ref[0]+(0.32326826477)*x_ref[1]**o+(-0.983183590335)*x_ref[1]+(0.530605958132)*x_ref[2]**o+(-0.109640854019)*x_ref[2]
        ref[3,2,2,0]=(0.587862067621)*x_ref[0]**o+(-0.951653317735)*x_ref[0]+(0.089519431923)*x_ref[1]**o+(-0.780330592384)*x_ref[1]+(-0.0483676475341)*x_ref[2]**o+(0.766538955993)*x_ref[2]
        ref[3,2,2,1]=(-0.957533418498)*x_ref[0]**o+(-0.681231272445)*x_ref[0]+(-0.0918289718767)*x_ref[1]**o+(0.38725634145)*x_ref[1]+(0.26451874534)*x_ref[2]**o+(-0.901046807078)*x_ref[2]
        ref[3,3,0,0]=(0.604215052532)*x_ref[0]**o+(-0.0188225127963)*x_ref[0]+(-0.544365413309)*x_ref[1]**o+(-0.837547618266)*x_ref[1]+(0.506665562924)*x_ref[2]**o+(0.868580855034)*x_ref[2]
        ref[3,3,0,1]=(0.307620045896)*x_ref[0]**o+(-0.941702619274)*x_ref[0]+(0.467078433616)*x_ref[1]**o+(-0.525971373276)*x_ref[1]+(-0.438681578224)*x_ref[2]**o+(-0.865538708837)*x_ref[2]
        ref[3,3,1,0]=(-0.128299566207)*x_ref[0]**o+(0.0526714707626)*x_ref[0]+(-0.194464943133)*x_ref[1]**o+(0.848343510106)*x_ref[1]+(-0.843042290362)*x_ref[2]**o+(-0.384441684551)*x_ref[2]
        ref[3,3,1,1]=(-0.0202567344895)*x_ref[0]**o+(0.919037490786)*x_ref[0]+(0.953616123775)*x_ref[1]**o+(-0.359010584029)*x_ref[1]+(-0.892651116514)*x_ref[2]**o+(0.900116594426)*x_ref[2]
        ref[3,3,2,0]=(-0.196701204528)*x_ref[0]**o+(0.173992581268)*x_ref[0]+(0.0304200566544)*x_ref[1]**o+(-0.562311121578)*x_ref[1]+(-0.349902986645)*x_ref[2]**o+(0.137392211876)*x_ref[2]
        ref[3,3,2,1]=(-0.264756448582)*x_ref[0]**o+(0.104810865018)*x_ref[0]+(-0.550559232481)*x_ref[1]**o+(-0.621292330017)*x_ref[1]+(-0.697096182063)*x_ref[2]**o+(-0.137686685739)*x_ref[2]
        ref[3,4,0,0]=(-0.0500780621237)*x_ref[0]**o+(-0.842196022641)*x_ref[0]+(-0.375688956182)*x_ref[1]**o+(0.293383514141)*x_ref[1]+(-0.679863519997)*x_ref[2]**o+(-0.108799395703)*x_ref[2]
        ref[3,4,0,1]=(-0.825663711773)*x_ref[0]**o+(0.0811924383936)*x_ref[0]+(0.658753764102)*x_ref[1]**o+(0.213557161034)*x_ref[1]+(0.048778548639)*x_ref[2]**o+(0.308220587221)*x_ref[2]
        ref[3,4,1,0]=(-0.0366332011016)*x_ref[0]**o+(-0.935060469197)*x_ref[0]+(0.52021729799)*x_ref[1]**o+(0.270580190954)*x_ref[1]+(0.690657153532)*x_ref[2]**o+(0.572985687766)*x_ref[2]
        ref[3,4,1,1]=(-0.358041857957)*x_ref[0]**o+(0.151420239879)*x_ref[0]+(0.578841565624)*x_ref[1]**o+(0.379085642891)*x_ref[1]+(0.231447762893)*x_ref[2]**o+(-0.494885835203)*x_ref[2]
        ref[3,4,2,0]=(0.689670410742)*x_ref[0]**o+(-0.469898987295)*x_ref[0]+(-0.617664742358)*x_ref[1]**o+(-0.92098566662)*x_ref[1]+(0.535497980872)*x_ref[2]**o+(-0.660036711933)*x_ref[2]
        ref[3,4,2,1]=(-0.443846183672)*x_ref[0]**o+(-0.0606185138497)*x_ref[0]+(0.902723111093)*x_ref[1]**o+(-0.201894548349)*x_ref[1]+(0.650998336304)*x_ref[2]**o+(-0.936010399236)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromSymbol_ReducedSolution_rank0(self):
      """
      tests interpolate for rank 0 Symbol onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedSolution onto ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(),dim=dim)
      s=Data(0,(),w)
      ref=Data(0,(),w_ref)
      if dim==2:
        s=(0.14488970011)*x[0]+(-0.430489267095)*x[1]
        ref=(0.14488970011)*x_ref[0]+(-0.430489267095)*x_ref[1]
      else:
        s=(-0.0442386353051)*x[0]+(0.0394615944862)*x[1]+(-1.29904883856)*x[2]
        ref=(-0.0442386353051)*x_ref[0]+(0.0394615944862)*x_ref[1]+(-1.29904883856)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromSymbol_ReducedSolution_rank1(self):
      """
      tests interpolate for rank 1 Symbol onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedSolution onto ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(2,),dim=dim)
      s=Data(0,(2,),w)
      ref=Data(0,(2,),w_ref)
      if dim==2:
        s[0]=(0.0426949725928)*x[0]+(0.796580332975)*x[1]
        s[1]=(0.297643604255)*x[0]+(0.501403166746)*x[1]
        ref[0]=(0.0426949725928)*x[0]+(0.796580332975)*x[1]
        ref[1]=(0.297643604255)*x[0]+(0.501403166746)*x[1]
      else:
        s[0]=(0.820713117949)*x[0]+(0.0164537336201)*x[1]+(-0.130931974587)*x[2]
        s[1]=(-0.155153784636)*x[0]+(-0.48729229521)*x[1]+(-0.875514015614)*x[2]
        ref[0]=(0.820713117949)*x[0]+(0.0164537336201)*x[1]+(-0.130931974587)*x[2]
        ref[1]=(-0.155153784636)*x[0]+(-0.48729229521)*x[1]+(-0.875514015614)*x[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromSymbol_ReducedSolution_rank2(self):
      """
      tests interpolate for rank 2 Symbol onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedSolution onto ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5),dim=dim)
      s=Data(0,(4, 5),w)
      ref=Data(0,(4, 5),w_ref)
      if dim==2:
        s[0,0]=(0.918593737351)*x[0]+(-0.183298779344)*x[1]
        s[0,1]=(1.01407474954)*x[0]+(-1.14888338084)*x[1]
        s[0,2]=(-0.0702606564619)*x[0]+(-0.104586136782)*x[1]
        s[0,3]=(-1.00520814193)*x[0]+(-0.959854266357)*x[1]
        s[0,4]=(-1.11463618446)*x[0]+(-0.794825170571)*x[1]
        s[1,0]=(-1.14418607702)*x[0]+(-0.273235396058)*x[1]
        s[1,1]=(0.641683109978)*x[0]+(0.685097028758)*x[1]
        s[1,2]=(0.833533353805)*x[0]+(-0.769590460713)*x[1]
        s[1,3]=(1.44918354254)*x[0]+(0.78941323689)*x[1]
        s[1,4]=(0.968133426719)*x[0]+(-0.923562755675)*x[1]
        s[2,0]=(1.58053224399)*x[0]+(0.600967207116)*x[1]
        s[2,1]=(1.04054361996)*x[0]+(-1.71519167959)*x[1]
        s[2,2]=(-1.55202351352)*x[0]+(-1.06650593034)*x[1]
        s[2,3]=(0.894986308301)*x[0]+(-0.767078277329)*x[1]
        s[2,4]=(-0.804853431103)*x[0]+(-0.44572378652)*x[1]
        s[3,0]=(0.90347844754)*x[0]+(0.357148881796)*x[1]
        s[3,1]=(0.0552482783049)*x[0]+(-0.652692619264)*x[1]
        s[3,2]=(-0.251809701815)*x[0]+(1.89155992551)*x[1]
        s[3,3]=(0.43181695535)*x[0]+(-0.228997029489)*x[1]
        s[3,4]=(-0.522831785326)*x[0]+(-0.372437316342)*x[1]
        ref[0,0]=(0.918593737351)*x_ref[0]+(-0.183298779344)*x_ref[1]
        ref[0,1]=(1.01407474954)*x_ref[0]+(-1.14888338084)*x_ref[1]
        ref[0,2]=(-0.0702606564619)*x_ref[0]+(-0.104586136782)*x_ref[1]
        ref[0,3]=(-1.00520814193)*x_ref[0]+(-0.959854266357)*x_ref[1]
        ref[0,4]=(-1.11463618446)*x_ref[0]+(-0.794825170571)*x_ref[1]
        ref[1,0]=(-1.14418607702)*x_ref[0]+(-0.273235396058)*x_ref[1]
        ref[1,1]=(0.641683109978)*x_ref[0]+(0.685097028758)*x_ref[1]
        ref[1,2]=(0.833533353805)*x_ref[0]+(-0.769590460713)*x_ref[1]
        ref[1,3]=(1.44918354254)*x_ref[0]+(0.78941323689)*x_ref[1]
        ref[1,4]=(0.968133426719)*x_ref[0]+(-0.923562755675)*x_ref[1]
        ref[2,0]=(1.58053224399)*x_ref[0]+(0.600967207116)*x_ref[1]
        ref[2,1]=(1.04054361996)*x_ref[0]+(-1.71519167959)*x_ref[1]
        ref[2,2]=(-1.55202351352)*x_ref[0]+(-1.06650593034)*x_ref[1]
        ref[2,3]=(0.894986308301)*x_ref[0]+(-0.767078277329)*x_ref[1]
        ref[2,4]=(-0.804853431103)*x_ref[0]+(-0.44572378652)*x_ref[1]
        ref[3,0]=(0.90347844754)*x_ref[0]+(0.357148881796)*x_ref[1]
        ref[3,1]=(0.0552482783049)*x_ref[0]+(-0.652692619264)*x_ref[1]
        ref[3,2]=(-0.251809701815)*x_ref[0]+(1.89155992551)*x_ref[1]
        ref[3,3]=(0.43181695535)*x_ref[0]+(-0.228997029489)*x_ref[1]
        ref[3,4]=(-0.522831785326)*x_ref[0]+(-0.372437316342)*x_ref[1]
      else:
        s[0,0]=(0.221916152348)*x[0]+(1.46835470229)*x[1]+(0.300487757142)*x[2]
        s[0,1]=(-0.15334206717)*x[0]+(0.506590749852)*x[1]+(0.477134329524)*x[2]
        s[0,2]=(0.786821788538)*x[0]+(-0.0141797716133)*x[1]+(-1.67421275557)*x[2]
        s[0,3]=(-1.14438881368)*x[0]+(-0.621323583056)*x[1]+(-0.0198865298096)*x[2]
        s[0,4]=(-0.195154511305)*x[0]+(-0.237596745083)*x[1]+(-0.36680113021)*x[2]
        s[1,0]=(-0.0476767967515)*x[0]+(-1.19950396856)*x[1]+(0.12823672742)*x[2]
        s[1,1]=(-0.420573984429)*x[0]+(0.100192190413)*x[1]+(1.30995115077)*x[2]
        s[1,2]=(1.34087691694)*x[0]+(-0.151927765846)*x[1]+(0.417185746786)*x[2]
        s[1,3]=(-0.383933595451)*x[0]+(0.208340861893)*x[1]+(0.947878917669)*x[2]
        s[1,4]=(-0.143686438189)*x[0]+(-0.00211348684153)*x[1]+(-0.859362566827)*x[2]
        s[2,0]=(0.447423746179)*x[0]+(-0.636241203952)*x[1]+(-0.0949114725339)*x[2]
        s[2,1]=(-0.0413740104938)*x[0]+(0.587001644427)*x[1]+(-0.544886520536)*x[2]
        s[2,2]=(-1.07017617383)*x[0]+(-0.762147671415)*x[1]+(-0.248360379513)*x[2]
        s[2,3]=(0.391720300962)*x[0]+(1.6521113321)*x[1]+(-0.408205550425)*x[2]
        s[2,4]=(1.2369249321)*x[0]+(0.952522466347)*x[1]+(-0.404645424067)*x[2]
        s[3,0]=(0.62410800627)*x[0]+(-0.308929187117)*x[1]+(-0.703796873161)*x[2]
        s[3,1]=(0.676590956976)*x[0]+(-1.39642714827)*x[1]+(0.198267797885)*x[2]
        s[3,2]=(-0.202804694134)*x[0]+(0.187577851273)*x[1]+(-0.700960966206)*x[2]
        s[3,3]=(-0.799833564134)*x[0]+(-0.455781630028)*x[1]+(0.453451623154)*x[2]
        s[3,4]=(-0.660373484066)*x[0]+(-1.55472528981)*x[1]+(0.0751348530851)*x[2]
        ref[0,0]=(0.221916152348)*x_ref[0]+(1.46835470229)*x_ref[1]+(0.300487757142)*x_ref[2]
        ref[0,1]=(-0.15334206717)*x_ref[0]+(0.506590749852)*x_ref[1]+(0.477134329524)*x_ref[2]
        ref[0,2]=(0.786821788538)*x_ref[0]+(-0.0141797716133)*x_ref[1]+(-1.67421275557)*x_ref[2]
        ref[0,3]=(-1.14438881368)*x_ref[0]+(-0.621323583056)*x_ref[1]+(-0.0198865298096)*x_ref[2]
        ref[0,4]=(-0.195154511305)*x_ref[0]+(-0.237596745083)*x_ref[1]+(-0.36680113021)*x_ref[2]
        ref[1,0]=(-0.0476767967515)*x_ref[0]+(-1.19950396856)*x_ref[1]+(0.12823672742)*x_ref[2]
        ref[1,1]=(-0.420573984429)*x_ref[0]+(0.100192190413)*x_ref[1]+(1.30995115077)*x_ref[2]
        ref[1,2]=(1.34087691694)*x_ref[0]+(-0.151927765846)*x_ref[1]+(0.417185746786)*x_ref[2]
        ref[1,3]=(-0.383933595451)*x_ref[0]+(0.208340861893)*x_ref[1]+(0.947878917669)*x_ref[2]
        ref[1,4]=(-0.143686438189)*x_ref[0]+(-0.00211348684153)*x_ref[1]+(-0.859362566827)*x_ref[2]
        ref[2,0]=(0.447423746179)*x_ref[0]+(-0.636241203952)*x_ref[1]+(-0.0949114725339)*x_ref[2]
        ref[2,1]=(-0.0413740104938)*x_ref[0]+(0.587001644427)*x_ref[1]+(-0.544886520536)*x_ref[2]
        ref[2,2]=(-1.07017617383)*x_ref[0]+(-0.762147671415)*x_ref[1]+(-0.248360379513)*x_ref[2]
        ref[2,3]=(0.391720300962)*x_ref[0]+(1.6521113321)*x_ref[1]+(-0.408205550425)*x_ref[2]
        ref[2,4]=(1.2369249321)*x_ref[0]+(0.952522466347)*x_ref[1]+(-0.404645424067)*x_ref[2]
        ref[3,0]=(0.62410800627)*x_ref[0]+(-0.308929187117)*x_ref[1]+(-0.703796873161)*x_ref[2]
        ref[3,1]=(0.676590956976)*x_ref[0]+(-1.39642714827)*x_ref[1]+(0.198267797885)*x_ref[2]
        ref[3,2]=(-0.202804694134)*x_ref[0]+(0.187577851273)*x_ref[1]+(-0.700960966206)*x_ref[2]
        ref[3,3]=(-0.799833564134)*x_ref[0]+(-0.455781630028)*x_ref[1]+(0.453451623154)*x_ref[2]
        ref[3,4]=(-0.660373484066)*x_ref[0]+(-1.55472528981)*x_ref[1]+(0.0751348530851)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromSymbol_ReducedSolution_rank3(self):
      """
      tests interpolate for rank 3 Symbol onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedSolution onto ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(6, 2, 2),dim=dim)
      s=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2),w_ref)
      if dim==2:
        s[0,0,0]=(-1.78813432343)*x[0]+(1.60161751633)*x[1]
        s[0,0,1]=(-1.49849771585)*x[0]+(-0.635435863643)*x[1]
        s[0,1,0]=(-0.906731346391)*x[0]+(0.533678119416)*x[1]
        s[0,1,1]=(-0.792521183513)*x[0]+(0.260998720452)*x[1]
        s[1,0,0]=(-0.710279314945)*x[0]+(0.38834997642)*x[1]
        s[1,0,1]=(-0.168938079833)*x[0]+(-0.502425332838)*x[1]
        s[1,1,0]=(-0.358550156297)*x[0]+(-0.650005639384)*x[1]
        s[1,1,1]=(-1.57793968521)*x[0]+(-0.46781871273)*x[1]
        s[2,0,0]=(0.0112676061889)*x[0]+(0.925017077862)*x[1]
        s[2,0,1]=(-0.109253916051)*x[0]+(-0.38189031862)*x[1]
        s[2,1,0]=(0.777175068327)*x[0]+(1.00196875887)*x[1]
        s[2,1,1]=(-0.27343070108)*x[0]+(-0.11181493242)*x[1]
        s[3,0,0]=(0.321226287504)*x[0]+(0.173667688124)*x[1]
        s[3,0,1]=(1.67405481183)*x[0]+(-0.981404244518)*x[1]
        s[3,1,0]=(0.129874894836)*x[0]+(0.341748165587)*x[1]
        s[3,1,1]=(-0.545737004242)*x[0]+(-1.20936137818)*x[1]
        s[4,0,0]=(0.389007738117)*x[0]+(-0.435046969057)*x[1]
        s[4,0,1]=(1.50296802873)*x[0]+(-0.878616436543)*x[1]
        s[4,1,0]=(1.16067694875)*x[0]+(-0.954317497512)*x[1]
        s[4,1,1]=(1.14634534638)*x[0]+(-0.63981762551)*x[1]
        s[5,0,0]=(0.803750299983)*x[0]+(-0.383611041681)*x[1]
        s[5,0,1]=(0.51863884687)*x[0]+(1.32891347835)*x[1]
        s[5,1,0]=(-0.0775425248593)*x[0]+(-0.0375631918508)*x[1]
        s[5,1,1]=(1.57167118663)*x[0]+(-0.649960686486)*x[1]
        ref[0,0,0]=(-1.78813432343)*x_ref[0]+(1.60161751633)*x_ref[1]
        ref[0,0,1]=(-1.49849771585)*x_ref[0]+(-0.635435863643)*x_ref[1]
        ref[0,1,0]=(-0.906731346391)*x_ref[0]+(0.533678119416)*x_ref[1]
        ref[0,1,1]=(-0.792521183513)*x_ref[0]+(0.260998720452)*x_ref[1]
        ref[1,0,0]=(-0.710279314945)*x_ref[0]+(0.38834997642)*x_ref[1]
        ref[1,0,1]=(-0.168938079833)*x_ref[0]+(-0.502425332838)*x_ref[1]
        ref[1,1,0]=(-0.358550156297)*x_ref[0]+(-0.650005639384)*x_ref[1]
        ref[1,1,1]=(-1.57793968521)*x_ref[0]+(-0.46781871273)*x_ref[1]
        ref[2,0,0]=(0.0112676061889)*x_ref[0]+(0.925017077862)*x_ref[1]
        ref[2,0,1]=(-0.109253916051)*x_ref[0]+(-0.38189031862)*x_ref[1]
        ref[2,1,0]=(0.777175068327)*x_ref[0]+(1.00196875887)*x_ref[1]
        ref[2,1,1]=(-0.27343070108)*x_ref[0]+(-0.11181493242)*x_ref[1]
        ref[3,0,0]=(0.321226287504)*x_ref[0]+(0.173667688124)*x_ref[1]
        ref[3,0,1]=(1.67405481183)*x_ref[0]+(-0.981404244518)*x_ref[1]
        ref[3,1,0]=(0.129874894836)*x_ref[0]+(0.341748165587)*x_ref[1]
        ref[3,1,1]=(-0.545737004242)*x_ref[0]+(-1.20936137818)*x_ref[1]
        ref[4,0,0]=(0.389007738117)*x_ref[0]+(-0.435046969057)*x_ref[1]
        ref[4,0,1]=(1.50296802873)*x_ref[0]+(-0.878616436543)*x_ref[1]
        ref[4,1,0]=(1.16067694875)*x_ref[0]+(-0.954317497512)*x_ref[1]
        ref[4,1,1]=(1.14634534638)*x_ref[0]+(-0.63981762551)*x_ref[1]
        ref[5,0,0]=(0.803750299983)*x_ref[0]+(-0.383611041681)*x_ref[1]
        ref[5,0,1]=(0.51863884687)*x_ref[0]+(1.32891347835)*x_ref[1]
        ref[5,1,0]=(-0.0775425248593)*x_ref[0]+(-0.0375631918508)*x_ref[1]
        ref[5,1,1]=(1.57167118663)*x_ref[0]+(-0.649960686486)*x_ref[1]
      else:
        s[0,0,0]=(1.00224477774)*x[0]+(0.276541980132)*x[1]+(-1.61268557504)*x[2]
        s[0,0,1]=(0.182197948486)*x[0]+(-0.314887566213)*x[1]+(0.172753958549)*x[2]
        s[0,1,0]=(-0.225255532228)*x[0]+(1.64823628561)*x[1]+(0.00289115479583)*x[2]
        s[0,1,1]=(-0.0081734432798)*x[0]+(-1.40127403863)*x[1]+(-0.878765016785)*x[2]
        s[1,0,0]=(-0.88802398859)*x[0]+(0.980539857729)*x[1]+(0.106008000545)*x[2]
        s[1,0,1]=(0.413338006112)*x[0]+(0.238889884193)*x[1]+(0.216982176844)*x[2]
        s[1,1,0]=(-0.375359689292)*x[0]+(0.656302411812)*x[1]+(-1.13744732251)*x[2]
        s[1,1,1]=(-0.768638146497)*x[0]+(0.382131491456)*x[1]+(-0.0473187661226)*x[2]
        s[2,0,0]=(0.727474830988)*x[0]+(-1.15897383904)*x[1]+(1.03107954393)*x[2]
        s[2,0,1]=(-0.565279703477)*x[0]+(0.952275155472)*x[1]+(1.72082557166)*x[2]
        s[2,1,0]=(-0.504951356904)*x[0]+(0.0945851323338)*x[1]+(-1.18137582843)*x[2]
        s[2,1,1]=(0.441559000376)*x[0]+(0.808481091433)*x[1]+(-0.185790180692)*x[2]
        s[3,0,0]=(1.097525477)*x[0]+(1.09598373235)*x[1]+(-0.225656362018)*x[2]
        s[3,0,1]=(-1.64387091178)*x[0]+(0.161580847642)*x[1]+(0.616741154516)*x[2]
        s[3,1,0]=(0.518052869517)*x[0]+(-0.491037975466)*x[1]+(-0.320485227716)*x[2]
        s[3,1,1]=(0.0299906539271)*x[0]+(-0.161268046678)*x[1]+(-0.933417226819)*x[2]
        s[4,0,0]=(-1.45814177886)*x[0]+(-1.01057484083)*x[1]+(1.05668957085)*x[2]
        s[4,0,1]=(0.335720229493)*x[0]+(0.155778050975)*x[1]+(-0.479943899005)*x[2]
        s[4,1,0]=(0.424166905697)*x[0]+(1.04188647689)*x[1]+(-0.154173690871)*x[2]
        s[4,1,1]=(-0.663564713586)*x[0]+(-0.744257330117)*x[1]+(-0.586452598078)*x[2]
        s[5,0,0]=(-0.308954778807)*x[0]+(0.0799719560375)*x[1]+(0.128085782808)*x[2]
        s[5,0,1]=(0.548010504324)*x[0]+(-0.676431304167)*x[1]+(0.12659732457)*x[2]
        s[5,1,0]=(0.17131336527)*x[0]+(-1.54908008145)*x[1]+(0.212836465224)*x[2]
        s[5,1,1]=(-0.834644454471)*x[0]+(1.52867678218)*x[1]+(1.19656096059)*x[2]
        ref[0,0,0]=(1.00224477774)*x_ref[0]+(0.276541980132)*x_ref[1]+(-1.61268557504)*x_ref[2]
        ref[0,0,1]=(0.182197948486)*x_ref[0]+(-0.314887566213)*x_ref[1]+(0.172753958549)*x_ref[2]
        ref[0,1,0]=(-0.225255532228)*x_ref[0]+(1.64823628561)*x_ref[1]+(0.00289115479583)*x_ref[2]
        ref[0,1,1]=(-0.0081734432798)*x_ref[0]+(-1.40127403863)*x_ref[1]+(-0.878765016785)*x_ref[2]
        ref[1,0,0]=(-0.88802398859)*x_ref[0]+(0.980539857729)*x_ref[1]+(0.106008000545)*x_ref[2]
        ref[1,0,1]=(0.413338006112)*x_ref[0]+(0.238889884193)*x_ref[1]+(0.216982176844)*x_ref[2]
        ref[1,1,0]=(-0.375359689292)*x_ref[0]+(0.656302411812)*x_ref[1]+(-1.13744732251)*x_ref[2]
        ref[1,1,1]=(-0.768638146497)*x_ref[0]+(0.382131491456)*x_ref[1]+(-0.0473187661226)*x_ref[2]
        ref[2,0,0]=(0.727474830988)*x_ref[0]+(-1.15897383904)*x_ref[1]+(1.03107954393)*x_ref[2]
        ref[2,0,1]=(-0.565279703477)*x_ref[0]+(0.952275155472)*x_ref[1]+(1.72082557166)*x_ref[2]
        ref[2,1,0]=(-0.504951356904)*x_ref[0]+(0.0945851323338)*x_ref[1]+(-1.18137582843)*x_ref[2]
        ref[2,1,1]=(0.441559000376)*x_ref[0]+(0.808481091433)*x_ref[1]+(-0.185790180692)*x_ref[2]
        ref[3,0,0]=(1.097525477)*x_ref[0]+(1.09598373235)*x_ref[1]+(-0.225656362018)*x_ref[2]
        ref[3,0,1]=(-1.64387091178)*x_ref[0]+(0.161580847642)*x_ref[1]+(0.616741154516)*x_ref[2]
        ref[3,1,0]=(0.518052869517)*x_ref[0]+(-0.491037975466)*x_ref[1]+(-0.320485227716)*x_ref[2]
        ref[3,1,1]=(0.0299906539271)*x_ref[0]+(-0.161268046678)*x_ref[1]+(-0.933417226819)*x_ref[2]
        ref[4,0,0]=(-1.45814177886)*x_ref[0]+(-1.01057484083)*x_ref[1]+(1.05668957085)*x_ref[2]
        ref[4,0,1]=(0.335720229493)*x_ref[0]+(0.155778050975)*x_ref[1]+(-0.479943899005)*x_ref[2]
        ref[4,1,0]=(0.424166905697)*x_ref[0]+(1.04188647689)*x_ref[1]+(-0.154173690871)*x_ref[2]
        ref[4,1,1]=(-0.663564713586)*x_ref[0]+(-0.744257330117)*x_ref[1]+(-0.586452598078)*x_ref[2]
        ref[5,0,0]=(-0.308954778807)*x_ref[0]+(0.0799719560375)*x_ref[1]+(0.128085782808)*x_ref[2]
        ref[5,0,1]=(0.548010504324)*x_ref[0]+(-0.676431304167)*x_ref[1]+(0.12659732457)*x_ref[2]
        ref[5,1,0]=(0.17131336527)*x_ref[0]+(-1.54908008145)*x_ref[1]+(0.212836465224)*x_ref[2]
        ref[5,1,1]=(-0.834644454471)*x_ref[0]+(1.52867678218)*x_ref[1]+(1.19656096059)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromSymbol_ReducedSolution_rank4(self):
      """
      tests interpolate for rank 4 Symbol onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedSolution onto ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5, 3, 2),dim=dim)
      s=Data(0,(4, 5, 3, 2),w)
      ref=Data(0,(4, 5, 3, 2),w_ref)
      if dim==2:
        s[0,0,0,0]=(-0.110140466592)*x[0]+(0.758992308989)*x[1]
        s[0,0,0,1]=(-1.27611018159)*x[0]+(-1.21469838588)*x[1]
        s[0,0,1,0]=(-0.695719260931)*x[0]+(-0.631650774678)*x[1]
        s[0,0,1,1]=(-0.0446786678677)*x[0]+(-0.547249041592)*x[1]
        s[0,0,2,0]=(0.824673646909)*x[0]+(-1.53499816213)*x[1]
        s[0,0,2,1]=(0.902244689111)*x[0]+(0.194498326222)*x[1]
        s[0,1,0,0]=(-1.29097330044)*x[0]+(0.0328620774092)*x[1]
        s[0,1,0,1]=(1.20419739576)*x[0]+(-1.25415670694)*x[1]
        s[0,1,1,0]=(0.825311334936)*x[0]+(0.732241589298)*x[1]
        s[0,1,1,1]=(0.239743697336)*x[0]+(0.446701903969)*x[1]
        s[0,1,2,0]=(-0.208525758105)*x[0]+(0.564727160849)*x[1]
        s[0,1,2,1]=(0.425352221432)*x[0]+(-0.439885368017)*x[1]
        s[0,2,0,0]=(0.089403990032)*x[0]+(0.927312736507)*x[1]
        s[0,2,0,1]=(-1.98342026057)*x[0]+(0.908824462154)*x[1]
        s[0,2,1,0]=(-1.02677966987)*x[0]+(1.64009929611)*x[1]
        s[0,2,1,1]=(1.18506229959)*x[0]+(-0.336041064484)*x[1]
        s[0,2,2,0]=(0.124121400983)*x[0]+(0.169858827797)*x[1]
        s[0,2,2,1]=(0.0171983721215)*x[0]+(-0.679377487833)*x[1]
        s[0,3,0,0]=(0.707474586567)*x[0]+(-0.327636474016)*x[1]
        s[0,3,0,1]=(-0.183596474381)*x[0]+(-1.07263498957)*x[1]
        s[0,3,1,0]=(1.49812660538)*x[0]+(-0.313209529867)*x[1]
        s[0,3,1,1]=(-0.678162796237)*x[0]+(-0.823724533082)*x[1]
        s[0,3,2,0]=(-0.79806833841)*x[0]+(0.865576104504)*x[1]
        s[0,3,2,1]=(0.941864201983)*x[0]+(0.734846756813)*x[1]
        s[0,4,0,0]=(-0.205171531689)*x[0]+(0.299610959481)*x[1]
        s[0,4,0,1]=(0.643430728461)*x[0]+(0.584994687875)*x[1]
        s[0,4,1,0]=(-0.408681959359)*x[0]+(0.601772358172)*x[1]
        s[0,4,1,1]=(-1.44472725121)*x[0]+(-0.0899809192849)*x[1]
        s[0,4,2,0]=(0.722764720152)*x[0]+(-0.407449476647)*x[1]
        s[0,4,2,1]=(0.166269375199)*x[0]+(1.20016653459)*x[1]
        s[1,0,0,0]=(-0.764665577343)*x[0]+(0.15492015266)*x[1]
        s[1,0,0,1]=(0.822023595189)*x[0]+(0.00511531586673)*x[1]
        s[1,0,1,0]=(0.0840294388025)*x[0]+(0.393606253186)*x[1]
        s[1,0,1,1]=(-0.702650062915)*x[0]+(-1.10361836979)*x[1]
        s[1,0,2,0]=(-1.34569564836)*x[0]+(-1.50816431101)*x[1]
        s[1,0,2,1]=(1.11337280315)*x[0]+(-0.0771805114838)*x[1]
        s[1,1,0,0]=(-0.0695861126454)*x[0]+(-0.437020174051)*x[1]
        s[1,1,0,1]=(1.68924343517)*x[0]+(0.706853885933)*x[1]
        s[1,1,1,0]=(0.0806933170819)*x[0]+(-0.974818406182)*x[1]
        s[1,1,1,1]=(-0.236836420312)*x[0]+(-0.352629546416)*x[1]
        s[1,1,2,0]=(-0.312697720062)*x[0]+(-0.977312197678)*x[1]
        s[1,1,2,1]=(-0.263563591998)*x[0]+(-0.899465194216)*x[1]
        s[1,2,0,0]=(1.0599010032)*x[0]+(0.625641179018)*x[1]
        s[1,2,0,1]=(0.42589369081)*x[0]+(0.63136683361)*x[1]
        s[1,2,1,0]=(0.629519575902)*x[0]+(-1.03125707286)*x[1]
        s[1,2,1,1]=(0.479272369893)*x[0]+(-1.63724584249)*x[1]
        s[1,2,2,0]=(1.48877937845)*x[0]+(-0.802308310767)*x[1]
        s[1,2,2,1]=(0.374636882204)*x[0]+(0.187733613588)*x[1]
        s[1,3,0,0]=(0.360148707925)*x[0]+(1.48869239048)*x[1]
        s[1,3,0,1]=(-1.29194970756)*x[0]+(-0.433561540189)*x[1]
        s[1,3,1,0]=(0.696118719028)*x[0]+(1.73199962229)*x[1]
        s[1,3,1,1]=(1.63831809651)*x[0]+(0.70687058079)*x[1]
        s[1,3,2,0]=(1.03614006811)*x[0]+(-0.783037005502)*x[1]
        s[1,3,2,1]=(0.351931393443)*x[0]+(-0.720060584954)*x[1]
        s[1,4,0,0]=(0.856138152361)*x[0]+(-0.0733689526147)*x[1]
        s[1,4,0,1]=(-0.256696810525)*x[0]+(1.57547133796)*x[1]
        s[1,4,1,0]=(0.260933620175)*x[0]+(-0.528418816229)*x[1]
        s[1,4,1,1]=(-1.14624698517)*x[0]+(0.537831890429)*x[1]
        s[1,4,2,0]=(-1.11328030308)*x[0]+(0.477599527943)*x[1]
        s[1,4,2,1]=(-1.52135700331)*x[0]+(1.18803329651)*x[1]
        s[2,0,0,0]=(1.02103992957)*x[0]+(1.3428520083)*x[1]
        s[2,0,0,1]=(0.74126089694)*x[0]+(-0.747092126031)*x[1]
        s[2,0,1,0]=(-0.300534057945)*x[0]+(-0.0679226717228)*x[1]
        s[2,0,1,1]=(0.205780243466)*x[0]+(1.89346679377)*x[1]
        s[2,0,2,0]=(0.711997044871)*x[0]+(0.614186591236)*x[1]
        s[2,0,2,1]=(0.562225764659)*x[0]+(-0.75210737336)*x[1]
        s[2,1,0,0]=(1.51024763079)*x[0]+(0.0655411043245)*x[1]
        s[2,1,0,1]=(-1.60270171758)*x[0]+(-0.555919918868)*x[1]
        s[2,1,1,0]=(0.0117250633883)*x[0]+(1.06089889984)*x[1]
        s[2,1,1,1]=(1.41839826051)*x[0]+(-1.11883387918)*x[1]
        s[2,1,2,0]=(-1.47481010167)*x[0]+(0.21986710996)*x[1]
        s[2,1,2,1]=(-0.890780004667)*x[0]+(0.68206435926)*x[1]
        s[2,2,0,0]=(0.00647580761798)*x[0]+(1.08988646609)*x[1]
        s[2,2,0,1]=(0.107576626365)*x[0]+(0.569541010093)*x[1]
        s[2,2,1,0]=(0.655745883134)*x[0]+(1.59781027174)*x[1]
        s[2,2,1,1]=(0.423699701576)*x[0]+(-0.854187892248)*x[1]
        s[2,2,2,0]=(-0.904849730929)*x[0]+(0.235074145696)*x[1]
        s[2,2,2,1]=(0.981525463443)*x[0]+(0.701832385796)*x[1]
        s[2,3,0,0]=(-0.489533883573)*x[0]+(-0.0213717056714)*x[1]
        s[2,3,0,1]=(0.516832539105)*x[0]+(-1.11995329355)*x[1]
        s[2,3,1,0]=(0.287740517981)*x[0]+(-1.3210942817)*x[1]
        s[2,3,1,1]=(0.179604184524)*x[0]+(-1.63664715586)*x[1]
        s[2,3,2,0]=(-0.492296320142)*x[0]+(-0.502235503805)*x[1]
        s[2,3,2,1]=(-0.653587673689)*x[0]+(0.5466761379)*x[1]
        s[2,4,0,0]=(1.37393629094)*x[0]+(0.645991033265)*x[1]
        s[2,4,0,1]=(-0.108057947979)*x[0]+(-0.761267645781)*x[1]
        s[2,4,1,0]=(-0.0900524038235)*x[0]+(0.16773443574)*x[1]
        s[2,4,1,1]=(0.578749842767)*x[0]+(1.18687780739)*x[1]
        s[2,4,2,0]=(-0.349879331342)*x[0]+(-0.214411558047)*x[1]
        s[2,4,2,1]=(-0.875945724127)*x[0]+(-1.36065436643)*x[1]
        s[3,0,0,0]=(0.0488555315168)*x[0]+(-0.82382271801)*x[1]
        s[3,0,0,1]=(1.24017449448)*x[0]+(-0.911198047419)*x[1]
        s[3,0,1,0]=(0.30200654088)*x[0]+(-1.11559310554)*x[1]
        s[3,0,1,1]=(0.67275005347)*x[0]+(-0.645034711632)*x[1]
        s[3,0,2,0]=(-0.670229614054)*x[0]+(0.359423039794)*x[1]
        s[3,0,2,1]=(1.14988130646)*x[0]+(0.86614224885)*x[1]
        s[3,1,0,0]=(0.705716005963)*x[0]+(-1.15357209623)*x[1]
        s[3,1,0,1]=(0.31067508882)*x[0]+(0.960823286011)*x[1]
        s[3,1,1,0]=(0.866232494898)*x[0]+(-0.65162103259)*x[1]
        s[3,1,1,1]=(-0.452071783562)*x[0]+(-0.211956993156)*x[1]
        s[3,1,2,0]=(-1.32137973484)*x[0]+(1.49152904496)*x[1]
        s[3,1,2,1]=(0.854595989445)*x[0]+(1.11534387313)*x[1]
        s[3,2,0,0]=(0.789093645939)*x[0]+(-0.623866327905)*x[1]
        s[3,2,0,1]=(-0.649118397874)*x[0]+(-0.510923012103)*x[1]
        s[3,2,1,0]=(-0.263103480764)*x[0]+(0.0353171690366)*x[1]
        s[3,2,1,1]=(-0.910441885706)*x[0]+(-0.572413584482)*x[1]
        s[3,2,2,0]=(-0.764615254979)*x[0]+(-0.257894213166)*x[1]
        s[3,2,2,1]=(0.82110723418)*x[0]+(-0.881337133364)*x[1]
        s[3,3,0,0]=(-0.978909443882)*x[0]+(0.95340950915)*x[1]
        s[3,3,0,1]=(-0.121221461759)*x[0]+(-1.68773270918)*x[1]
        s[3,3,1,0]=(-0.150032516264)*x[0]+(0.441210519728)*x[1]
        s[3,3,1,1]=(-0.618694838164)*x[0]+(0.379748674791)*x[1]
        s[3,3,2,0]=(0.208366138002)*x[0]+(1.04701688993)*x[1]
        s[3,3,2,1]=(0.207672316371)*x[0]+(-1.04311602421)*x[1]
        s[3,4,0,0]=(0.404190185535)*x[0]+(0.206083873066)*x[1]
        s[3,4,0,1]=(0.928741940014)*x[0]+(0.126856418039)*x[1]
        s[3,4,1,0]=(0.0831637965324)*x[0]+(0.198288732425)*x[1]
        s[3,4,1,1]=(0.602722196536)*x[0]+(-0.256860860163)*x[1]
        s[3,4,2,0]=(1.11330980537)*x[0]+(0.130706767542)*x[1]
        s[3,4,2,1]=(0.638478712945)*x[0]+(0.558803659636)*x[1]
        ref[0,0,0,0]=(-0.110140466592)*x_ref[0]+(0.758992308989)*x_ref[1]
        ref[0,0,0,1]=(-1.27611018159)*x_ref[0]+(-1.21469838588)*x_ref[1]
        ref[0,0,1,0]=(-0.695719260931)*x_ref[0]+(-0.631650774678)*x_ref[1]
        ref[0,0,1,1]=(-0.0446786678677)*x_ref[0]+(-0.547249041592)*x_ref[1]
        ref[0,0,2,0]=(0.824673646909)*x_ref[0]+(-1.53499816213)*x_ref[1]
        ref[0,0,2,1]=(0.902244689111)*x_ref[0]+(0.194498326222)*x_ref[1]
        ref[0,1,0,0]=(-1.29097330044)*x_ref[0]+(0.0328620774092)*x_ref[1]
        ref[0,1,0,1]=(1.20419739576)*x_ref[0]+(-1.25415670694)*x_ref[1]
        ref[0,1,1,0]=(0.825311334936)*x_ref[0]+(0.732241589298)*x_ref[1]
        ref[0,1,1,1]=(0.239743697336)*x_ref[0]+(0.446701903969)*x_ref[1]
        ref[0,1,2,0]=(-0.208525758105)*x_ref[0]+(0.564727160849)*x_ref[1]
        ref[0,1,2,1]=(0.425352221432)*x_ref[0]+(-0.439885368017)*x_ref[1]
        ref[0,2,0,0]=(0.089403990032)*x_ref[0]+(0.927312736507)*x_ref[1]
        ref[0,2,0,1]=(-1.98342026057)*x_ref[0]+(0.908824462154)*x_ref[1]
        ref[0,2,1,0]=(-1.02677966987)*x_ref[0]+(1.64009929611)*x_ref[1]
        ref[0,2,1,1]=(1.18506229959)*x_ref[0]+(-0.336041064484)*x_ref[1]
        ref[0,2,2,0]=(0.124121400983)*x_ref[0]+(0.169858827797)*x_ref[1]
        ref[0,2,2,1]=(0.0171983721215)*x_ref[0]+(-0.679377487833)*x_ref[1]
        ref[0,3,0,0]=(0.707474586567)*x_ref[0]+(-0.327636474016)*x_ref[1]
        ref[0,3,0,1]=(-0.183596474381)*x_ref[0]+(-1.07263498957)*x_ref[1]
        ref[0,3,1,0]=(1.49812660538)*x_ref[0]+(-0.313209529867)*x_ref[1]
        ref[0,3,1,1]=(-0.678162796237)*x_ref[0]+(-0.823724533082)*x_ref[1]
        ref[0,3,2,0]=(-0.79806833841)*x_ref[0]+(0.865576104504)*x_ref[1]
        ref[0,3,2,1]=(0.941864201983)*x_ref[0]+(0.734846756813)*x_ref[1]
        ref[0,4,0,0]=(-0.205171531689)*x_ref[0]+(0.299610959481)*x_ref[1]
        ref[0,4,0,1]=(0.643430728461)*x_ref[0]+(0.584994687875)*x_ref[1]
        ref[0,4,1,0]=(-0.408681959359)*x_ref[0]+(0.601772358172)*x_ref[1]
        ref[0,4,1,1]=(-1.44472725121)*x_ref[0]+(-0.0899809192849)*x_ref[1]
        ref[0,4,2,0]=(0.722764720152)*x_ref[0]+(-0.407449476647)*x_ref[1]
        ref[0,4,2,1]=(0.166269375199)*x_ref[0]+(1.20016653459)*x_ref[1]
        ref[1,0,0,0]=(-0.764665577343)*x_ref[0]+(0.15492015266)*x_ref[1]
        ref[1,0,0,1]=(0.822023595189)*x_ref[0]+(0.00511531586673)*x_ref[1]
        ref[1,0,1,0]=(0.0840294388025)*x_ref[0]+(0.393606253186)*x_ref[1]
        ref[1,0,1,1]=(-0.702650062915)*x_ref[0]+(-1.10361836979)*x_ref[1]
        ref[1,0,2,0]=(-1.34569564836)*x_ref[0]+(-1.50816431101)*x_ref[1]
        ref[1,0,2,1]=(1.11337280315)*x_ref[0]+(-0.0771805114838)*x_ref[1]
        ref[1,1,0,0]=(-0.0695861126454)*x_ref[0]+(-0.437020174051)*x_ref[1]
        ref[1,1,0,1]=(1.68924343517)*x_ref[0]+(0.706853885933)*x_ref[1]
        ref[1,1,1,0]=(0.0806933170819)*x_ref[0]+(-0.974818406182)*x_ref[1]
        ref[1,1,1,1]=(-0.236836420312)*x_ref[0]+(-0.352629546416)*x_ref[1]
        ref[1,1,2,0]=(-0.312697720062)*x_ref[0]+(-0.977312197678)*x_ref[1]
        ref[1,1,2,1]=(-0.263563591998)*x_ref[0]+(-0.899465194216)*x_ref[1]
        ref[1,2,0,0]=(1.0599010032)*x_ref[0]+(0.625641179018)*x_ref[1]
        ref[1,2,0,1]=(0.42589369081)*x_ref[0]+(0.63136683361)*x_ref[1]
        ref[1,2,1,0]=(0.629519575902)*x_ref[0]+(-1.03125707286)*x_ref[1]
        ref[1,2,1,1]=(0.479272369893)*x_ref[0]+(-1.63724584249)*x_ref[1]
        ref[1,2,2,0]=(1.48877937845)*x_ref[0]+(-0.802308310767)*x_ref[1]
        ref[1,2,2,1]=(0.374636882204)*x_ref[0]+(0.187733613588)*x_ref[1]
        ref[1,3,0,0]=(0.360148707925)*x_ref[0]+(1.48869239048)*x_ref[1]
        ref[1,3,0,1]=(-1.29194970756)*x_ref[0]+(-0.433561540189)*x_ref[1]
        ref[1,3,1,0]=(0.696118719028)*x_ref[0]+(1.73199962229)*x_ref[1]
        ref[1,3,1,1]=(1.63831809651)*x_ref[0]+(0.70687058079)*x_ref[1]
        ref[1,3,2,0]=(1.03614006811)*x_ref[0]+(-0.783037005502)*x_ref[1]
        ref[1,3,2,1]=(0.351931393443)*x_ref[0]+(-0.720060584954)*x_ref[1]
        ref[1,4,0,0]=(0.856138152361)*x_ref[0]+(-0.0733689526147)*x_ref[1]
        ref[1,4,0,1]=(-0.256696810525)*x_ref[0]+(1.57547133796)*x_ref[1]
        ref[1,4,1,0]=(0.260933620175)*x_ref[0]+(-0.528418816229)*x_ref[1]
        ref[1,4,1,1]=(-1.14624698517)*x_ref[0]+(0.537831890429)*x_ref[1]
        ref[1,4,2,0]=(-1.11328030308)*x_ref[0]+(0.477599527943)*x_ref[1]
        ref[1,4,2,1]=(-1.52135700331)*x_ref[0]+(1.18803329651)*x_ref[1]
        ref[2,0,0,0]=(1.02103992957)*x_ref[0]+(1.3428520083)*x_ref[1]
        ref[2,0,0,1]=(0.74126089694)*x_ref[0]+(-0.747092126031)*x_ref[1]
        ref[2,0,1,0]=(-0.300534057945)*x_ref[0]+(-0.0679226717228)*x_ref[1]
        ref[2,0,1,1]=(0.205780243466)*x_ref[0]+(1.89346679377)*x_ref[1]
        ref[2,0,2,0]=(0.711997044871)*x_ref[0]+(0.614186591236)*x_ref[1]
        ref[2,0,2,1]=(0.562225764659)*x_ref[0]+(-0.75210737336)*x_ref[1]
        ref[2,1,0,0]=(1.51024763079)*x_ref[0]+(0.0655411043245)*x_ref[1]
        ref[2,1,0,1]=(-1.60270171758)*x_ref[0]+(-0.555919918868)*x_ref[1]
        ref[2,1,1,0]=(0.0117250633883)*x_ref[0]+(1.06089889984)*x_ref[1]
        ref[2,1,1,1]=(1.41839826051)*x_ref[0]+(-1.11883387918)*x_ref[1]
        ref[2,1,2,0]=(-1.47481010167)*x_ref[0]+(0.21986710996)*x_ref[1]
        ref[2,1,2,1]=(-0.890780004667)*x_ref[0]+(0.68206435926)*x_ref[1]
        ref[2,2,0,0]=(0.00647580761798)*x_ref[0]+(1.08988646609)*x_ref[1]
        ref[2,2,0,1]=(0.107576626365)*x_ref[0]+(0.569541010093)*x_ref[1]
        ref[2,2,1,0]=(0.655745883134)*x_ref[0]+(1.59781027174)*x_ref[1]
        ref[2,2,1,1]=(0.423699701576)*x_ref[0]+(-0.854187892248)*x_ref[1]
        ref[2,2,2,0]=(-0.904849730929)*x_ref[0]+(0.235074145696)*x_ref[1]
        ref[2,2,2,1]=(0.981525463443)*x_ref[0]+(0.701832385796)*x_ref[1]
        ref[2,3,0,0]=(-0.489533883573)*x_ref[0]+(-0.0213717056714)*x_ref[1]
        ref[2,3,0,1]=(0.516832539105)*x_ref[0]+(-1.11995329355)*x_ref[1]
        ref[2,3,1,0]=(0.287740517981)*x_ref[0]+(-1.3210942817)*x_ref[1]
        ref[2,3,1,1]=(0.179604184524)*x_ref[0]+(-1.63664715586)*x_ref[1]
        ref[2,3,2,0]=(-0.492296320142)*x_ref[0]+(-0.502235503805)*x_ref[1]
        ref[2,3,2,1]=(-0.653587673689)*x_ref[0]+(0.5466761379)*x_ref[1]
        ref[2,4,0,0]=(1.37393629094)*x_ref[0]+(0.645991033265)*x_ref[1]
        ref[2,4,0,1]=(-0.108057947979)*x_ref[0]+(-0.761267645781)*x_ref[1]
        ref[2,4,1,0]=(-0.0900524038235)*x_ref[0]+(0.16773443574)*x_ref[1]
        ref[2,4,1,1]=(0.578749842767)*x_ref[0]+(1.18687780739)*x_ref[1]
        ref[2,4,2,0]=(-0.349879331342)*x_ref[0]+(-0.214411558047)*x_ref[1]
        ref[2,4,2,1]=(-0.875945724127)*x_ref[0]+(-1.36065436643)*x_ref[1]
        ref[3,0,0,0]=(0.0488555315168)*x_ref[0]+(-0.82382271801)*x_ref[1]
        ref[3,0,0,1]=(1.24017449448)*x_ref[0]+(-0.911198047419)*x_ref[1]
        ref[3,0,1,0]=(0.30200654088)*x_ref[0]+(-1.11559310554)*x_ref[1]
        ref[3,0,1,1]=(0.67275005347)*x_ref[0]+(-0.645034711632)*x_ref[1]
        ref[3,0,2,0]=(-0.670229614054)*x_ref[0]+(0.359423039794)*x_ref[1]
        ref[3,0,2,1]=(1.14988130646)*x_ref[0]+(0.86614224885)*x_ref[1]
        ref[3,1,0,0]=(0.705716005963)*x_ref[0]+(-1.15357209623)*x_ref[1]
        ref[3,1,0,1]=(0.31067508882)*x_ref[0]+(0.960823286011)*x_ref[1]
        ref[3,1,1,0]=(0.866232494898)*x_ref[0]+(-0.65162103259)*x_ref[1]
        ref[3,1,1,1]=(-0.452071783562)*x_ref[0]+(-0.211956993156)*x_ref[1]
        ref[3,1,2,0]=(-1.32137973484)*x_ref[0]+(1.49152904496)*x_ref[1]
        ref[3,1,2,1]=(0.854595989445)*x_ref[0]+(1.11534387313)*x_ref[1]
        ref[3,2,0,0]=(0.789093645939)*x_ref[0]+(-0.623866327905)*x_ref[1]
        ref[3,2,0,1]=(-0.649118397874)*x_ref[0]+(-0.510923012103)*x_ref[1]
        ref[3,2,1,0]=(-0.263103480764)*x_ref[0]+(0.0353171690366)*x_ref[1]
        ref[3,2,1,1]=(-0.910441885706)*x_ref[0]+(-0.572413584482)*x_ref[1]
        ref[3,2,2,0]=(-0.764615254979)*x_ref[0]+(-0.257894213166)*x_ref[1]
        ref[3,2,2,1]=(0.82110723418)*x_ref[0]+(-0.881337133364)*x_ref[1]
        ref[3,3,0,0]=(-0.978909443882)*x_ref[0]+(0.95340950915)*x_ref[1]
        ref[3,3,0,1]=(-0.121221461759)*x_ref[0]+(-1.68773270918)*x_ref[1]
        ref[3,3,1,0]=(-0.150032516264)*x_ref[0]+(0.441210519728)*x_ref[1]
        ref[3,3,1,1]=(-0.618694838164)*x_ref[0]+(0.379748674791)*x_ref[1]
        ref[3,3,2,0]=(0.208366138002)*x_ref[0]+(1.04701688993)*x_ref[1]
        ref[3,3,2,1]=(0.207672316371)*x_ref[0]+(-1.04311602421)*x_ref[1]
        ref[3,4,0,0]=(0.404190185535)*x_ref[0]+(0.206083873066)*x_ref[1]
        ref[3,4,0,1]=(0.928741940014)*x_ref[0]+(0.126856418039)*x_ref[1]
        ref[3,4,1,0]=(0.0831637965324)*x_ref[0]+(0.198288732425)*x_ref[1]
        ref[3,4,1,1]=(0.602722196536)*x_ref[0]+(-0.256860860163)*x_ref[1]
        ref[3,4,2,0]=(1.11330980537)*x_ref[0]+(0.130706767542)*x_ref[1]
        ref[3,4,2,1]=(0.638478712945)*x_ref[0]+(0.558803659636)*x_ref[1]
      else:
        s[0,0,0,0]=(-0.678750626234)*x[0]+(0.707898134345)*x[1]+(1.47682355575)*x[2]
        s[0,0,0,1]=(-1.08665921875)*x[0]+(-1.03746577329)*x[1]+(0.459220112015)*x[2]
        s[0,0,1,0]=(-0.0924850687831)*x[0]+(-1.21392495875)*x[1]+(-0.335482829106)*x[2]
        s[0,0,1,1]=(-0.796044372195)*x[0]+(-1.61600945946)*x[1]+(-0.917678370431)*x[2]
        s[0,0,2,0]=(-1.17314148071)*x[0]+(0.594053121363)*x[1]+(-1.0751981685)*x[2]
        s[0,0,2,1]=(-0.993747266312)*x[0]+(0.647546031964)*x[1]+(0.789233440797)*x[2]
        s[0,1,0,0]=(-0.538620007403)*x[0]+(0.572432466547)*x[1]+(1.59224293356)*x[2]
        s[0,1,0,1]=(-0.315217413376)*x[0]+(1.30492005409)*x[1]+(-0.04944666559)*x[2]
        s[0,1,1,0]=(-0.734247343541)*x[0]+(1.14104633611)*x[1]+(-0.0424483055941)*x[2]
        s[0,1,1,1]=(-0.521727902651)*x[0]+(-0.0302866863619)*x[1]+(-0.0378416654735)*x[2]
        s[0,1,2,0]=(0.319532436452)*x[0]+(0.312388869947)*x[1]+(0.237025751349)*x[2]
        s[0,1,2,1]=(0.713884175591)*x[0]+(1.10911840343)*x[1]+(-1.4943567087)*x[2]
        s[0,2,0,0]=(-0.599254236697)*x[0]+(-0.190373139957)*x[1]+(-1.75168866737)*x[2]
        s[0,2,0,1]=(0.396782073165)*x[0]+(-0.252470841512)*x[1]+(-0.463651254287)*x[2]
        s[0,2,1,0]=(1.03549889185)*x[0]+(0.225894922761)*x[1]+(-0.609139538729)*x[2]
        s[0,2,1,1]=(0.289458873955)*x[0]+(-0.136668874286)*x[1]+(0.474827071819)*x[2]
        s[0,2,2,0]=(0.654253727057)*x[0]+(-0.998748120229)*x[1]+(-0.107718066457)*x[2]
        s[0,2,2,1]=(0.256710826049)*x[0]+(0.309515575678)*x[1]+(-1.60310813555)*x[2]
        s[0,3,0,0]=(1.70779242771)*x[0]+(-0.530818399689)*x[1]+(-0.516799319419)*x[2]
        s[0,3,0,1]=(-0.848868447677)*x[0]+(-0.301212439564)*x[1]+(-0.423135852525)*x[2]
        s[0,3,1,0]=(1.0807833809)*x[0]+(-1.20726021059)*x[1]+(0.565314004443)*x[2]
        s[0,3,1,1]=(-0.660234771003)*x[0]+(-0.443966617085)*x[1]+(-0.185381740077)*x[2]
        s[0,3,2,0]=(1.6260513125)*x[0]+(0.145626479867)*x[1]+(-0.808965403823)*x[2]
        s[0,3,2,1]=(-0.503772598493)*x[0]+(-1.63975947666)*x[1]+(0.255681416016)*x[2]
        s[0,4,0,0]=(0.0818777313252)*x[0]+(0.428155907605)*x[1]+(-0.642991998051)*x[2]
        s[0,4,0,1]=(-0.145032953266)*x[0]+(-0.7645425853)*x[1]+(0.458784780224)*x[2]
        s[0,4,1,0]=(-0.589583511871)*x[0]+(-0.0743175116482)*x[1]+(0.285113485271)*x[2]
        s[0,4,1,1]=(-1.37305892557)*x[0]+(0.917822397515)*x[1]+(-0.378127728174)*x[2]
        s[0,4,2,0]=(1.16505232525)*x[0]+(1.10944900772)*x[1]+(-0.95491955738)*x[2]
        s[0,4,2,1]=(-1.12539432931)*x[0]+(-0.290265022179)*x[1]+(-1.20809121853)*x[2]
        s[1,0,0,0]=(-0.859420732972)*x[0]+(0.564058561557)*x[1]+(-0.795155516002)*x[2]
        s[1,0,0,1]=(-0.314313522693)*x[0]+(1.46275003784)*x[1]+(0.285596389281)*x[2]
        s[1,0,1,0]=(-0.513360534744)*x[0]+(-0.0532410726117)*x[1]+(-0.122433325652)*x[2]
        s[1,0,1,1]=(0.527488940208)*x[0]+(0.0760609834598)*x[1]+(-0.612300815838)*x[2]
        s[1,0,2,0]=(0.302549971797)*x[0]+(-0.710901255568)*x[1]+(0.955062818656)*x[2]
        s[1,0,2,1]=(0.126520802001)*x[0]+(-1.07564234283)*x[1]+(-0.669526781403)*x[2]
        s[1,1,0,0]=(1.50804296428)*x[0]+(0.111903566342)*x[1]+(0.465876365517)*x[2]
        s[1,1,0,1]=(-1.04380726708)*x[0]+(0.148245186406)*x[1]+(-1.06801443458)*x[2]
        s[1,1,1,0]=(1.48236374583)*x[0]+(-0.00794102905726)*x[1]+(-1.35613430084)*x[2]
        s[1,1,1,1]=(1.04119256641)*x[0]+(-1.84075261029)*x[1]+(-0.776969168414)*x[2]
        s[1,1,2,0]=(-0.356539020393)*x[0]+(1.70797570799)*x[1]+(-0.525736126605)*x[2]
        s[1,1,2,1]=(1.62559049194)*x[0]+(-0.482482804647)*x[1]+(0.631790498783)*x[2]
        s[1,2,0,0]=(0.499315452031)*x[0]+(-0.296700233772)*x[1]+(1.00111627057)*x[2]
        s[1,2,0,1]=(1.0433119055)*x[0]+(-0.269957745366)*x[1]+(0.591153091169)*x[2]
        s[1,2,1,0]=(0.502068262492)*x[0]+(-0.849037462803)*x[1]+(-1.56692020458)*x[2]
        s[1,2,1,1]=(1.2984795003)*x[0]+(-1.07653573134)*x[1]+(0.487232329161)*x[2]
        s[1,2,2,0]=(-0.109333568443)*x[0]+(0.687162555279)*x[1]+(-0.812636431387)*x[2]
        s[1,2,2,1]=(0.202326014817)*x[0]+(1.23297194202)*x[1]+(1.41163217274)*x[2]
        s[1,3,0,0]=(0.124740742544)*x[0]+(-0.500878894409)*x[1]+(-0.376438702019)*x[2]
        s[1,3,0,1]=(-0.540234624582)*x[0]+(-0.400577181452)*x[1]+(0.879215681428)*x[2]
        s[1,3,1,0]=(0.0426835015338)*x[0]+(-0.726976626459)*x[1]+(1.32655296741)*x[2]
        s[1,3,1,1]=(-0.120969369352)*x[0]+(-0.426853376816)*x[1]+(1.88787704976)*x[2]
        s[1,3,2,0]=(-0.0895115836038)*x[0]+(-1.17409640569)*x[1]+(-0.874853748928)*x[2]
        s[1,3,2,1]=(-0.0664677570269)*x[0]+(0.708204452159)*x[1]+(-0.26397721498)*x[2]
        s[1,4,0,0]=(0.0236750184919)*x[0]+(-1.42225228911)*x[1]+(0.399502903376)*x[2]
        s[1,4,0,1]=(1.03400366274)*x[0]+(-0.442018365874)*x[1]+(1.02491120541)*x[2]
        s[1,4,1,0]=(-0.150528717616)*x[0]+(-0.512236709632)*x[1]+(-0.0644728802768)*x[2]
        s[1,4,1,1]=(-0.998520493595)*x[0]+(-0.507010096027)*x[1]+(-0.889168150742)*x[2]
        s[1,4,2,0]=(0.179165278401)*x[0]+(1.07136931045)*x[1]+(-0.790101000847)*x[2]
        s[1,4,2,1]=(0.376877479598)*x[0]+(-0.25979387806)*x[1]+(0.446682478544)*x[2]
        s[2,0,0,0]=(0.365668617405)*x[0]+(1.44268125148)*x[1]+(1.65785163405)*x[2]
        s[2,0,0,1]=(0.576180639993)*x[0]+(-0.0571374651916)*x[1]+(-0.0854307657892)*x[2]
        s[2,0,1,0]=(0.0474860699991)*x[0]+(-1.47716457199)*x[1]+(0.87300241949)*x[2]
        s[2,0,1,1]=(-0.293608884991)*x[0]+(0.0963236497693)*x[1]+(-1.31217901681)*x[2]
        s[2,0,2,0]=(-0.725125904522)*x[0]+(0.15259525725)*x[1]+(0.0624842579957)*x[2]
        s[2,0,2,1]=(-0.374667932076)*x[0]+(-0.586704716252)*x[1]+(-0.208259155061)*x[2]
        s[2,1,0,0]=(0.480010407761)*x[0]+(-1.78856630141)*x[1]+(0.296814589368)*x[2]
        s[2,1,0,1]=(-0.131871389585)*x[0]+(0.0984653022698)*x[1]+(0.222785695838)*x[2]
        s[2,1,1,0]=(-0.4095594348)*x[0]+(-0.746059065385)*x[1]+(1.72282248984)*x[2]
        s[2,1,1,1]=(0.841882742791)*x[0]+(0.698803891138)*x[1]+(0.201279240318)*x[2]
        s[2,1,2,0]=(-1.52503851146)*x[0]+(0.746778574384)*x[1]+(0.820127798751)*x[2]
        s[2,1,2,1]=(0.578616447592)*x[0]+(-0.232884112626)*x[1]+(0.719949134498)*x[2]
        s[2,2,0,0]=(-0.250869865455)*x[0]+(0.0457838553118)*x[1]+(0.694605224088)*x[2]
        s[2,2,0,1]=(-0.344703134894)*x[0]+(-0.945296587028)*x[1]+(-0.408991625518)*x[2]
        s[2,2,1,0]=(1.15842118069)*x[0]+(0.385018100988)*x[1]+(-0.298867420422)*x[2]
        s[2,2,1,1]=(-0.992212925836)*x[0]+(-0.303921757453)*x[1]+(1.90996417539)*x[2]
        s[2,2,2,0]=(0.139346086335)*x[0]+(-1.5255000175)*x[1]+(-0.418223280061)*x[2]
        s[2,2,2,1]=(-0.287165354844)*x[0]+(-0.563470113789)*x[1]+(-0.267078242542)*x[2]
        s[2,3,0,0]=(-0.877956931038)*x[0]+(-0.587379404687)*x[1]+(0.223772989101)*x[2]
        s[2,3,0,1]=(0.450240028907)*x[0]+(0.933649504645)*x[1]+(0.922982912622)*x[2]
        s[2,3,1,0]=(-0.695647633954)*x[0]+(0.675123237641)*x[1]+(0.333013288603)*x[2]
        s[2,3,1,1]=(-0.638319791165)*x[0]+(-1.01905860587)*x[1]+(-0.0829897514576)*x[2]
        s[2,3,2,0]=(-0.85507363839)*x[0]+(0.213154292163)*x[1]+(-1.19634907609)*x[2]
        s[2,3,2,1]=(0.51969848742)*x[0]+(-0.103353752424)*x[1]+(1.38100947757)*x[2]
        s[2,4,0,0]=(-1.42264362114)*x[0]+(-0.0622861012095)*x[1]+(1.11415860946)*x[2]
        s[2,4,0,1]=(1.03146522202)*x[0]+(-0.59775537826)*x[1]+(0.455451132416)*x[2]
        s[2,4,1,0]=(-1.42367400988)*x[0]+(0.468019196451)*x[1]+(-0.0449222277683)*x[2]
        s[2,4,1,1]=(-0.337745267456)*x[0]+(-1.24573098041)*x[1]+(0.591411642381)*x[2]
        s[2,4,2,0]=(-0.773220236551)*x[0]+(-0.92934639777)*x[1]+(-0.323666905212)*x[2]
        s[2,4,2,1]=(-1.25416565442)*x[0]+(0.10804460369)*x[1]+(1.3398483009)*x[2]
        s[3,0,0,0]=(-0.187928573247)*x[0]+(0.952793598709)*x[1]+(-0.651544295118)*x[2]
        s[3,0,0,1]=(-1.18183645576)*x[0]+(-0.767480258477)*x[1]+(0.0733909569555)*x[2]
        s[3,0,1,0]=(0.825156828887)*x[0]+(0.900307840844)*x[1]+(-1.03153118659)*x[2]
        s[3,0,1,1]=(-0.150209523447)*x[0]+(1.07490578747)*x[1]+(-1.6916184583)*x[2]
        s[3,0,2,0]=(-1.07041023688)*x[0]+(-0.420347448824)*x[1]+(-0.556885268114)*x[2]
        s[3,0,2,1]=(-1.30609527635)*x[0]+(-0.10753520843)*x[1]+(0.240258152808)*x[2]
        s[3,1,0,0]=(0.899651818251)*x[0]+(0.00909698197696)*x[1]+(-1.64985946319)*x[2]
        s[3,1,0,1]=(-0.880471412731)*x[0]+(0.678893404261)*x[1]+(0.288497390335)*x[2]
        s[3,1,1,0]=(-0.00910924813519)*x[0]+(-1.30365942504)*x[1]+(-0.568919628746)*x[2]
        s[3,1,1,1]=(-0.811256892653)*x[0]+(-0.0284400402628)*x[1]+(0.727386097819)*x[2]
        s[3,1,2,0]=(0.105755186759)*x[0]+(0.196242692495)*x[1]+(0.358516596542)*x[2]
        s[3,1,2,1]=(-0.37985806916)*x[0]+(0.44810760494)*x[1]+(0.633807267038)*x[2]
        s[3,2,0,0]=(1.34921360449)*x[0]+(-0.9349137549)*x[1]+(-0.272245661092)*x[2]
        s[3,2,0,1]=(1.21020688092)*x[0]+(1.1296074258)*x[1]+(0.000391430715188)*x[2]
        s[3,2,1,0]=(-1.25769018387)*x[0]+(-0.786491629941)*x[1]+(-0.862105545664)*x[2]
        s[3,2,1,1]=(0.668529227552)*x[0]+(-0.983024344824)*x[1]+(0.74846424728)*x[2]
        s[3,2,2,0]=(0.45487084108)*x[0]+(0.855491649411)*x[1]+(-0.104621648479)*x[2]
        s[3,2,2,1]=(0.802451481866)*x[0]+(-0.137058520895)*x[1]+(0.573940225359)*x[2]
        s[3,3,0,0]=(0.468999526003)*x[0]+(0.504917415809)*x[1]+(-1.39187140958)*x[2]
        s[3,3,0,1]=(0.501007631838)*x[0]+(-0.285379909295)*x[1]+(0.257752560803)*x[2]
        s[3,3,1,0]=(-1.01423550219)*x[0]+(-0.749951834747)*x[1]+(-0.156550300135)*x[2]
        s[3,3,1,1]=(0.190954159364)*x[0]+(0.985690213898)*x[1]+(-0.942349242057)*x[2]
        s[3,3,2,0]=(-1.04776017317)*x[0]+(1.3978257599)*x[1]+(-0.727385312507)*x[2]
        s[3,3,2,1]=(0.118574662458)*x[0]+(0.763657521414)*x[1]+(-0.947854404873)*x[2]
        s[3,4,0,0]=(-1.42375158871)*x[0]+(1.02577770584)*x[1]+(1.7262667856)*x[2]
        s[3,4,0,1]=(0.819198798755)*x[0]+(-0.177472792322)*x[1]+(-0.495119104783)*x[2]
        s[3,4,1,0]=(-1.44070928104)*x[0]+(-0.67376984043)*x[1]+(0.314203511381)*x[2]
        s[3,4,1,1]=(-0.0417551422406)*x[0]+(-0.446166510311)*x[1]+(-0.303922120635)*x[2]
        s[3,4,2,0]=(0.161905825888)*x[0]+(-0.433001417416)*x[1]+(0.713602984094)*x[2]
        s[3,4,2,1]=(1.29579174225)*x[0]+(-0.136051917273)*x[1]+(0.270836816064)*x[2]
        ref[0,0,0,0]=(-0.678750626234)*x_ref[0]+(0.707898134345)*x_ref[1]+(1.47682355575)*x_ref[2]
        ref[0,0,0,1]=(-1.08665921875)*x_ref[0]+(-1.03746577329)*x_ref[1]+(0.459220112015)*x_ref[2]
        ref[0,0,1,0]=(-0.0924850687831)*x_ref[0]+(-1.21392495875)*x_ref[1]+(-0.335482829106)*x_ref[2]
        ref[0,0,1,1]=(-0.796044372195)*x_ref[0]+(-1.61600945946)*x_ref[1]+(-0.917678370431)*x_ref[2]
        ref[0,0,2,0]=(-1.17314148071)*x_ref[0]+(0.594053121363)*x_ref[1]+(-1.0751981685)*x_ref[2]
        ref[0,0,2,1]=(-0.993747266312)*x_ref[0]+(0.647546031964)*x_ref[1]+(0.789233440797)*x_ref[2]
        ref[0,1,0,0]=(-0.538620007403)*x_ref[0]+(0.572432466547)*x_ref[1]+(1.59224293356)*x_ref[2]
        ref[0,1,0,1]=(-0.315217413376)*x_ref[0]+(1.30492005409)*x_ref[1]+(-0.04944666559)*x_ref[2]
        ref[0,1,1,0]=(-0.734247343541)*x_ref[0]+(1.14104633611)*x_ref[1]+(-0.0424483055941)*x_ref[2]
        ref[0,1,1,1]=(-0.521727902651)*x_ref[0]+(-0.0302866863619)*x_ref[1]+(-0.0378416654735)*x_ref[2]
        ref[0,1,2,0]=(0.319532436452)*x_ref[0]+(0.312388869947)*x_ref[1]+(0.237025751349)*x_ref[2]
        ref[0,1,2,1]=(0.713884175591)*x_ref[0]+(1.10911840343)*x_ref[1]+(-1.4943567087)*x_ref[2]
        ref[0,2,0,0]=(-0.599254236697)*x_ref[0]+(-0.190373139957)*x_ref[1]+(-1.75168866737)*x_ref[2]
        ref[0,2,0,1]=(0.396782073165)*x_ref[0]+(-0.252470841512)*x_ref[1]+(-0.463651254287)*x_ref[2]
        ref[0,2,1,0]=(1.03549889185)*x_ref[0]+(0.225894922761)*x_ref[1]+(-0.609139538729)*x_ref[2]
        ref[0,2,1,1]=(0.289458873955)*x_ref[0]+(-0.136668874286)*x_ref[1]+(0.474827071819)*x_ref[2]
        ref[0,2,2,0]=(0.654253727057)*x_ref[0]+(-0.998748120229)*x_ref[1]+(-0.107718066457)*x_ref[2]
        ref[0,2,2,1]=(0.256710826049)*x_ref[0]+(0.309515575678)*x_ref[1]+(-1.60310813555)*x_ref[2]
        ref[0,3,0,0]=(1.70779242771)*x_ref[0]+(-0.530818399689)*x_ref[1]+(-0.516799319419)*x_ref[2]
        ref[0,3,0,1]=(-0.848868447677)*x_ref[0]+(-0.301212439564)*x_ref[1]+(-0.423135852525)*x_ref[2]
        ref[0,3,1,0]=(1.0807833809)*x_ref[0]+(-1.20726021059)*x_ref[1]+(0.565314004443)*x_ref[2]
        ref[0,3,1,1]=(-0.660234771003)*x_ref[0]+(-0.443966617085)*x_ref[1]+(-0.185381740077)*x_ref[2]
        ref[0,3,2,0]=(1.6260513125)*x_ref[0]+(0.145626479867)*x_ref[1]+(-0.808965403823)*x_ref[2]
        ref[0,3,2,1]=(-0.503772598493)*x_ref[0]+(-1.63975947666)*x_ref[1]+(0.255681416016)*x_ref[2]
        ref[0,4,0,0]=(0.0818777313252)*x_ref[0]+(0.428155907605)*x_ref[1]+(-0.642991998051)*x_ref[2]
        ref[0,4,0,1]=(-0.145032953266)*x_ref[0]+(-0.7645425853)*x_ref[1]+(0.458784780224)*x_ref[2]
        ref[0,4,1,0]=(-0.589583511871)*x_ref[0]+(-0.0743175116482)*x_ref[1]+(0.285113485271)*x_ref[2]
        ref[0,4,1,1]=(-1.37305892557)*x_ref[0]+(0.917822397515)*x_ref[1]+(-0.378127728174)*x_ref[2]
        ref[0,4,2,0]=(1.16505232525)*x_ref[0]+(1.10944900772)*x_ref[1]+(-0.95491955738)*x_ref[2]
        ref[0,4,2,1]=(-1.12539432931)*x_ref[0]+(-0.290265022179)*x_ref[1]+(-1.20809121853)*x_ref[2]
        ref[1,0,0,0]=(-0.859420732972)*x_ref[0]+(0.564058561557)*x_ref[1]+(-0.795155516002)*x_ref[2]
        ref[1,0,0,1]=(-0.314313522693)*x_ref[0]+(1.46275003784)*x_ref[1]+(0.285596389281)*x_ref[2]
        ref[1,0,1,0]=(-0.513360534744)*x_ref[0]+(-0.0532410726117)*x_ref[1]+(-0.122433325652)*x_ref[2]
        ref[1,0,1,1]=(0.527488940208)*x_ref[0]+(0.0760609834598)*x_ref[1]+(-0.612300815838)*x_ref[2]
        ref[1,0,2,0]=(0.302549971797)*x_ref[0]+(-0.710901255568)*x_ref[1]+(0.955062818656)*x_ref[2]
        ref[1,0,2,1]=(0.126520802001)*x_ref[0]+(-1.07564234283)*x_ref[1]+(-0.669526781403)*x_ref[2]
        ref[1,1,0,0]=(1.50804296428)*x_ref[0]+(0.111903566342)*x_ref[1]+(0.465876365517)*x_ref[2]
        ref[1,1,0,1]=(-1.04380726708)*x_ref[0]+(0.148245186406)*x_ref[1]+(-1.06801443458)*x_ref[2]
        ref[1,1,1,0]=(1.48236374583)*x_ref[0]+(-0.00794102905726)*x_ref[1]+(-1.35613430084)*x_ref[2]
        ref[1,1,1,1]=(1.04119256641)*x_ref[0]+(-1.84075261029)*x_ref[1]+(-0.776969168414)*x_ref[2]
        ref[1,1,2,0]=(-0.356539020393)*x_ref[0]+(1.70797570799)*x_ref[1]+(-0.525736126605)*x_ref[2]
        ref[1,1,2,1]=(1.62559049194)*x_ref[0]+(-0.482482804647)*x_ref[1]+(0.631790498783)*x_ref[2]
        ref[1,2,0,0]=(0.499315452031)*x_ref[0]+(-0.296700233772)*x_ref[1]+(1.00111627057)*x_ref[2]
        ref[1,2,0,1]=(1.0433119055)*x_ref[0]+(-0.269957745366)*x_ref[1]+(0.591153091169)*x_ref[2]
        ref[1,2,1,0]=(0.502068262492)*x_ref[0]+(-0.849037462803)*x_ref[1]+(-1.56692020458)*x_ref[2]
        ref[1,2,1,1]=(1.2984795003)*x_ref[0]+(-1.07653573134)*x_ref[1]+(0.487232329161)*x_ref[2]
        ref[1,2,2,0]=(-0.109333568443)*x_ref[0]+(0.687162555279)*x_ref[1]+(-0.812636431387)*x_ref[2]
        ref[1,2,2,1]=(0.202326014817)*x_ref[0]+(1.23297194202)*x_ref[1]+(1.41163217274)*x_ref[2]
        ref[1,3,0,0]=(0.124740742544)*x_ref[0]+(-0.500878894409)*x_ref[1]+(-0.376438702019)*x_ref[2]
        ref[1,3,0,1]=(-0.540234624582)*x_ref[0]+(-0.400577181452)*x_ref[1]+(0.879215681428)*x_ref[2]
        ref[1,3,1,0]=(0.0426835015338)*x_ref[0]+(-0.726976626459)*x_ref[1]+(1.32655296741)*x_ref[2]
        ref[1,3,1,1]=(-0.120969369352)*x_ref[0]+(-0.426853376816)*x_ref[1]+(1.88787704976)*x_ref[2]
        ref[1,3,2,0]=(-0.0895115836038)*x_ref[0]+(-1.17409640569)*x_ref[1]+(-0.874853748928)*x_ref[2]
        ref[1,3,2,1]=(-0.0664677570269)*x_ref[0]+(0.708204452159)*x_ref[1]+(-0.26397721498)*x_ref[2]
        ref[1,4,0,0]=(0.0236750184919)*x_ref[0]+(-1.42225228911)*x_ref[1]+(0.399502903376)*x_ref[2]
        ref[1,4,0,1]=(1.03400366274)*x_ref[0]+(-0.442018365874)*x_ref[1]+(1.02491120541)*x_ref[2]
        ref[1,4,1,0]=(-0.150528717616)*x_ref[0]+(-0.512236709632)*x_ref[1]+(-0.0644728802768)*x_ref[2]
        ref[1,4,1,1]=(-0.998520493595)*x_ref[0]+(-0.507010096027)*x_ref[1]+(-0.889168150742)*x_ref[2]
        ref[1,4,2,0]=(0.179165278401)*x_ref[0]+(1.07136931045)*x_ref[1]+(-0.790101000847)*x_ref[2]
        ref[1,4,2,1]=(0.376877479598)*x_ref[0]+(-0.25979387806)*x_ref[1]+(0.446682478544)*x_ref[2]
        ref[2,0,0,0]=(0.365668617405)*x_ref[0]+(1.44268125148)*x_ref[1]+(1.65785163405)*x_ref[2]
        ref[2,0,0,1]=(0.576180639993)*x_ref[0]+(-0.0571374651916)*x_ref[1]+(-0.0854307657892)*x_ref[2]
        ref[2,0,1,0]=(0.0474860699991)*x_ref[0]+(-1.47716457199)*x_ref[1]+(0.87300241949)*x_ref[2]
        ref[2,0,1,1]=(-0.293608884991)*x_ref[0]+(0.0963236497693)*x_ref[1]+(-1.31217901681)*x_ref[2]
        ref[2,0,2,0]=(-0.725125904522)*x_ref[0]+(0.15259525725)*x_ref[1]+(0.0624842579957)*x_ref[2]
        ref[2,0,2,1]=(-0.374667932076)*x_ref[0]+(-0.586704716252)*x_ref[1]+(-0.208259155061)*x_ref[2]
        ref[2,1,0,0]=(0.480010407761)*x_ref[0]+(-1.78856630141)*x_ref[1]+(0.296814589368)*x_ref[2]
        ref[2,1,0,1]=(-0.131871389585)*x_ref[0]+(0.0984653022698)*x_ref[1]+(0.222785695838)*x_ref[2]
        ref[2,1,1,0]=(-0.4095594348)*x_ref[0]+(-0.746059065385)*x_ref[1]+(1.72282248984)*x_ref[2]
        ref[2,1,1,1]=(0.841882742791)*x_ref[0]+(0.698803891138)*x_ref[1]+(0.201279240318)*x_ref[2]
        ref[2,1,2,0]=(-1.52503851146)*x_ref[0]+(0.746778574384)*x_ref[1]+(0.820127798751)*x_ref[2]
        ref[2,1,2,1]=(0.578616447592)*x_ref[0]+(-0.232884112626)*x_ref[1]+(0.719949134498)*x_ref[2]
        ref[2,2,0,0]=(-0.250869865455)*x_ref[0]+(0.0457838553118)*x_ref[1]+(0.694605224088)*x_ref[2]
        ref[2,2,0,1]=(-0.344703134894)*x_ref[0]+(-0.945296587028)*x_ref[1]+(-0.408991625518)*x_ref[2]
        ref[2,2,1,0]=(1.15842118069)*x_ref[0]+(0.385018100988)*x_ref[1]+(-0.298867420422)*x_ref[2]
        ref[2,2,1,1]=(-0.992212925836)*x_ref[0]+(-0.303921757453)*x_ref[1]+(1.90996417539)*x_ref[2]
        ref[2,2,2,0]=(0.139346086335)*x_ref[0]+(-1.5255000175)*x_ref[1]+(-0.418223280061)*x_ref[2]
        ref[2,2,2,1]=(-0.287165354844)*x_ref[0]+(-0.563470113789)*x_ref[1]+(-0.267078242542)*x_ref[2]
        ref[2,3,0,0]=(-0.877956931038)*x_ref[0]+(-0.587379404687)*x_ref[1]+(0.223772989101)*x_ref[2]
        ref[2,3,0,1]=(0.450240028907)*x_ref[0]+(0.933649504645)*x_ref[1]+(0.922982912622)*x_ref[2]
        ref[2,3,1,0]=(-0.695647633954)*x_ref[0]+(0.675123237641)*x_ref[1]+(0.333013288603)*x_ref[2]
        ref[2,3,1,1]=(-0.638319791165)*x_ref[0]+(-1.01905860587)*x_ref[1]+(-0.0829897514576)*x_ref[2]
        ref[2,3,2,0]=(-0.85507363839)*x_ref[0]+(0.213154292163)*x_ref[1]+(-1.19634907609)*x_ref[2]
        ref[2,3,2,1]=(0.51969848742)*x_ref[0]+(-0.103353752424)*x_ref[1]+(1.38100947757)*x_ref[2]
        ref[2,4,0,0]=(-1.42264362114)*x_ref[0]+(-0.0622861012095)*x_ref[1]+(1.11415860946)*x_ref[2]
        ref[2,4,0,1]=(1.03146522202)*x_ref[0]+(-0.59775537826)*x_ref[1]+(0.455451132416)*x_ref[2]
        ref[2,4,1,0]=(-1.42367400988)*x_ref[0]+(0.468019196451)*x_ref[1]+(-0.0449222277683)*x_ref[2]
        ref[2,4,1,1]=(-0.337745267456)*x_ref[0]+(-1.24573098041)*x_ref[1]+(0.591411642381)*x_ref[2]
        ref[2,4,2,0]=(-0.773220236551)*x_ref[0]+(-0.92934639777)*x_ref[1]+(-0.323666905212)*x_ref[2]
        ref[2,4,2,1]=(-1.25416565442)*x_ref[0]+(0.10804460369)*x_ref[1]+(1.3398483009)*x_ref[2]
        ref[3,0,0,0]=(-0.187928573247)*x_ref[0]+(0.952793598709)*x_ref[1]+(-0.651544295118)*x_ref[2]
        ref[3,0,0,1]=(-1.18183645576)*x_ref[0]+(-0.767480258477)*x_ref[1]+(0.0733909569555)*x_ref[2]
        ref[3,0,1,0]=(0.825156828887)*x_ref[0]+(0.900307840844)*x_ref[1]+(-1.03153118659)*x_ref[2]
        ref[3,0,1,1]=(-0.150209523447)*x_ref[0]+(1.07490578747)*x_ref[1]+(-1.6916184583)*x_ref[2]
        ref[3,0,2,0]=(-1.07041023688)*x_ref[0]+(-0.420347448824)*x_ref[1]+(-0.556885268114)*x_ref[2]
        ref[3,0,2,1]=(-1.30609527635)*x_ref[0]+(-0.10753520843)*x_ref[1]+(0.240258152808)*x_ref[2]
        ref[3,1,0,0]=(0.899651818251)*x_ref[0]+(0.00909698197696)*x_ref[1]+(-1.64985946319)*x_ref[2]
        ref[3,1,0,1]=(-0.880471412731)*x_ref[0]+(0.678893404261)*x_ref[1]+(0.288497390335)*x_ref[2]
        ref[3,1,1,0]=(-0.00910924813519)*x_ref[0]+(-1.30365942504)*x_ref[1]+(-0.568919628746)*x_ref[2]
        ref[3,1,1,1]=(-0.811256892653)*x_ref[0]+(-0.0284400402628)*x_ref[1]+(0.727386097819)*x_ref[2]
        ref[3,1,2,0]=(0.105755186759)*x_ref[0]+(0.196242692495)*x_ref[1]+(0.358516596542)*x_ref[2]
        ref[3,1,2,1]=(-0.37985806916)*x_ref[0]+(0.44810760494)*x_ref[1]+(0.633807267038)*x_ref[2]
        ref[3,2,0,0]=(1.34921360449)*x_ref[0]+(-0.9349137549)*x_ref[1]+(-0.272245661092)*x_ref[2]
        ref[3,2,0,1]=(1.21020688092)*x_ref[0]+(1.1296074258)*x_ref[1]+(0.000391430715188)*x_ref[2]
        ref[3,2,1,0]=(-1.25769018387)*x_ref[0]+(-0.786491629941)*x_ref[1]+(-0.862105545664)*x_ref[2]
        ref[3,2,1,1]=(0.668529227552)*x_ref[0]+(-0.983024344824)*x_ref[1]+(0.74846424728)*x_ref[2]
        ref[3,2,2,0]=(0.45487084108)*x_ref[0]+(0.855491649411)*x_ref[1]+(-0.104621648479)*x_ref[2]
        ref[3,2,2,1]=(0.802451481866)*x_ref[0]+(-0.137058520895)*x_ref[1]+(0.573940225359)*x_ref[2]
        ref[3,3,0,0]=(0.468999526003)*x_ref[0]+(0.504917415809)*x_ref[1]+(-1.39187140958)*x_ref[2]
        ref[3,3,0,1]=(0.501007631838)*x_ref[0]+(-0.285379909295)*x_ref[1]+(0.257752560803)*x_ref[2]
        ref[3,3,1,0]=(-1.01423550219)*x_ref[0]+(-0.749951834747)*x_ref[1]+(-0.156550300135)*x_ref[2]
        ref[3,3,1,1]=(0.190954159364)*x_ref[0]+(0.985690213898)*x_ref[1]+(-0.942349242057)*x_ref[2]
        ref[3,3,2,0]=(-1.04776017317)*x_ref[0]+(1.3978257599)*x_ref[1]+(-0.727385312507)*x_ref[2]
        ref[3,3,2,1]=(0.118574662458)*x_ref[0]+(0.763657521414)*x_ref[1]+(-0.947854404873)*x_ref[2]
        ref[3,4,0,0]=(-1.42375158871)*x_ref[0]+(1.02577770584)*x_ref[1]+(1.7262667856)*x_ref[2]
        ref[3,4,0,1]=(0.819198798755)*x_ref[0]+(-0.177472792322)*x_ref[1]+(-0.495119104783)*x_ref[2]
        ref[3,4,1,0]=(-1.44070928104)*x_ref[0]+(-0.67376984043)*x_ref[1]+(0.314203511381)*x_ref[2]
        ref[3,4,1,1]=(-0.0417551422406)*x_ref[0]+(-0.446166510311)*x_ref[1]+(-0.303922120635)*x_ref[2]
        ref[3,4,2,0]=(0.161905825888)*x_ref[0]+(-0.433001417416)*x_ref[1]+(0.713602984094)*x_ref[2]
        ref[3,4,2,1]=(1.29579174225)*x_ref[0]+(-0.136051917273)*x_ref[1]+(0.270836816064)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromSymbol_ReducedContinuousFunction_rank0(self):
      """
      tests interpolate for rank 0 Symbol onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedContinuousFunction onto ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(),dim=dim)
      s=Data(0,(),w)
      ref=Data(0,(),w_ref)
      if dim==2:
        s=(0.14488970011)*x[0]+(-0.430489267095)*x[1]
        ref=(0.14488970011)*x_ref[0]+(-0.430489267095)*x_ref[1]
      else:
        s=(-0.0442386353051)*x[0]+(0.0394615944862)*x[1]+(-1.29904883856)*x[2]
        ref=(-0.0442386353051)*x_ref[0]+(0.0394615944862)*x_ref[1]+(-1.29904883856)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromSymbol_ReducedContinuousFunction_rank1(self):
      """
      tests interpolate for rank 1 Symbol onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedContinuousFunction onto ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(2,),dim=dim)
      s=Data(0,(2,),w)
      ref=Data(0,(2,),w_ref)
      if dim==2:
        s[0]=(0.0426949725928)*x[0]+(0.796580332975)*x[1]
        s[1]=(0.297643604255)*x[0]+(0.501403166746)*x[1]
        ref[0]=(0.0426949725928)*x[0]+(0.796580332975)*x[1]
        ref[1]=(0.297643604255)*x[0]+(0.501403166746)*x[1]
      else:
        s[0]=(0.820713117949)*x[0]+(0.0164537336201)*x[1]+(-0.130931974587)*x[2]
        s[1]=(-0.155153784636)*x[0]+(-0.48729229521)*x[1]+(-0.875514015614)*x[2]
        ref[0]=(0.820713117949)*x[0]+(0.0164537336201)*x[1]+(-0.130931974587)*x[2]
        ref[1]=(-0.155153784636)*x[0]+(-0.48729229521)*x[1]+(-0.875514015614)*x[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromSymbol_ReducedContinuousFunction_rank2(self):
      """
      tests interpolate for rank 2 Symbol onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedContinuousFunction onto ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5),dim=dim)
      s=Data(0,(4, 5),w)
      ref=Data(0,(4, 5),w_ref)
      if dim==2:
        s[0,0]=(0.918593737351)*x[0]+(-0.183298779344)*x[1]
        s[0,1]=(1.01407474954)*x[0]+(-1.14888338084)*x[1]
        s[0,2]=(-0.0702606564619)*x[0]+(-0.104586136782)*x[1]
        s[0,3]=(-1.00520814193)*x[0]+(-0.959854266357)*x[1]
        s[0,4]=(-1.11463618446)*x[0]+(-0.794825170571)*x[1]
        s[1,0]=(-1.14418607702)*x[0]+(-0.273235396058)*x[1]
        s[1,1]=(0.641683109978)*x[0]+(0.685097028758)*x[1]
        s[1,2]=(0.833533353805)*x[0]+(-0.769590460713)*x[1]
        s[1,3]=(1.44918354254)*x[0]+(0.78941323689)*x[1]
        s[1,4]=(0.968133426719)*x[0]+(-0.923562755675)*x[1]
        s[2,0]=(1.58053224399)*x[0]+(0.600967207116)*x[1]
        s[2,1]=(1.04054361996)*x[0]+(-1.71519167959)*x[1]
        s[2,2]=(-1.55202351352)*x[0]+(-1.06650593034)*x[1]
        s[2,3]=(0.894986308301)*x[0]+(-0.767078277329)*x[1]
        s[2,4]=(-0.804853431103)*x[0]+(-0.44572378652)*x[1]
        s[3,0]=(0.90347844754)*x[0]+(0.357148881796)*x[1]
        s[3,1]=(0.0552482783049)*x[0]+(-0.652692619264)*x[1]
        s[3,2]=(-0.251809701815)*x[0]+(1.89155992551)*x[1]
        s[3,3]=(0.43181695535)*x[0]+(-0.228997029489)*x[1]
        s[3,4]=(-0.522831785326)*x[0]+(-0.372437316342)*x[1]
        ref[0,0]=(0.918593737351)*x_ref[0]+(-0.183298779344)*x_ref[1]
        ref[0,1]=(1.01407474954)*x_ref[0]+(-1.14888338084)*x_ref[1]
        ref[0,2]=(-0.0702606564619)*x_ref[0]+(-0.104586136782)*x_ref[1]
        ref[0,3]=(-1.00520814193)*x_ref[0]+(-0.959854266357)*x_ref[1]
        ref[0,4]=(-1.11463618446)*x_ref[0]+(-0.794825170571)*x_ref[1]
        ref[1,0]=(-1.14418607702)*x_ref[0]+(-0.273235396058)*x_ref[1]
        ref[1,1]=(0.641683109978)*x_ref[0]+(0.685097028758)*x_ref[1]
        ref[1,2]=(0.833533353805)*x_ref[0]+(-0.769590460713)*x_ref[1]
        ref[1,3]=(1.44918354254)*x_ref[0]+(0.78941323689)*x_ref[1]
        ref[1,4]=(0.968133426719)*x_ref[0]+(-0.923562755675)*x_ref[1]
        ref[2,0]=(1.58053224399)*x_ref[0]+(0.600967207116)*x_ref[1]
        ref[2,1]=(1.04054361996)*x_ref[0]+(-1.71519167959)*x_ref[1]
        ref[2,2]=(-1.55202351352)*x_ref[0]+(-1.06650593034)*x_ref[1]
        ref[2,3]=(0.894986308301)*x_ref[0]+(-0.767078277329)*x_ref[1]
        ref[2,4]=(-0.804853431103)*x_ref[0]+(-0.44572378652)*x_ref[1]
        ref[3,0]=(0.90347844754)*x_ref[0]+(0.357148881796)*x_ref[1]
        ref[3,1]=(0.0552482783049)*x_ref[0]+(-0.652692619264)*x_ref[1]
        ref[3,2]=(-0.251809701815)*x_ref[0]+(1.89155992551)*x_ref[1]
        ref[3,3]=(0.43181695535)*x_ref[0]+(-0.228997029489)*x_ref[1]
        ref[3,4]=(-0.522831785326)*x_ref[0]+(-0.372437316342)*x_ref[1]
      else:
        s[0,0]=(0.221916152348)*x[0]+(1.46835470229)*x[1]+(0.300487757142)*x[2]
        s[0,1]=(-0.15334206717)*x[0]+(0.506590749852)*x[1]+(0.477134329524)*x[2]
        s[0,2]=(0.786821788538)*x[0]+(-0.0141797716133)*x[1]+(-1.67421275557)*x[2]
        s[0,3]=(-1.14438881368)*x[0]+(-0.621323583056)*x[1]+(-0.0198865298096)*x[2]
        s[0,4]=(-0.195154511305)*x[0]+(-0.237596745083)*x[1]+(-0.36680113021)*x[2]
        s[1,0]=(-0.0476767967515)*x[0]+(-1.19950396856)*x[1]+(0.12823672742)*x[2]
        s[1,1]=(-0.420573984429)*x[0]+(0.100192190413)*x[1]+(1.30995115077)*x[2]
        s[1,2]=(1.34087691694)*x[0]+(-0.151927765846)*x[1]+(0.417185746786)*x[2]
        s[1,3]=(-0.383933595451)*x[0]+(0.208340861893)*x[1]+(0.947878917669)*x[2]
        s[1,4]=(-0.143686438189)*x[0]+(-0.00211348684153)*x[1]+(-0.859362566827)*x[2]
        s[2,0]=(0.447423746179)*x[0]+(-0.636241203952)*x[1]+(-0.0949114725339)*x[2]
        s[2,1]=(-0.0413740104938)*x[0]+(0.587001644427)*x[1]+(-0.544886520536)*x[2]
        s[2,2]=(-1.07017617383)*x[0]+(-0.762147671415)*x[1]+(-0.248360379513)*x[2]
        s[2,3]=(0.391720300962)*x[0]+(1.6521113321)*x[1]+(-0.408205550425)*x[2]
        s[2,4]=(1.2369249321)*x[0]+(0.952522466347)*x[1]+(-0.404645424067)*x[2]
        s[3,0]=(0.62410800627)*x[0]+(-0.308929187117)*x[1]+(-0.703796873161)*x[2]
        s[3,1]=(0.676590956976)*x[0]+(-1.39642714827)*x[1]+(0.198267797885)*x[2]
        s[3,2]=(-0.202804694134)*x[0]+(0.187577851273)*x[1]+(-0.700960966206)*x[2]
        s[3,3]=(-0.799833564134)*x[0]+(-0.455781630028)*x[1]+(0.453451623154)*x[2]
        s[3,4]=(-0.660373484066)*x[0]+(-1.55472528981)*x[1]+(0.0751348530851)*x[2]
        ref[0,0]=(0.221916152348)*x_ref[0]+(1.46835470229)*x_ref[1]+(0.300487757142)*x_ref[2]
        ref[0,1]=(-0.15334206717)*x_ref[0]+(0.506590749852)*x_ref[1]+(0.477134329524)*x_ref[2]
        ref[0,2]=(0.786821788538)*x_ref[0]+(-0.0141797716133)*x_ref[1]+(-1.67421275557)*x_ref[2]
        ref[0,3]=(-1.14438881368)*x_ref[0]+(-0.621323583056)*x_ref[1]+(-0.0198865298096)*x_ref[2]
        ref[0,4]=(-0.195154511305)*x_ref[0]+(-0.237596745083)*x_ref[1]+(-0.36680113021)*x_ref[2]
        ref[1,0]=(-0.0476767967515)*x_ref[0]+(-1.19950396856)*x_ref[1]+(0.12823672742)*x_ref[2]
        ref[1,1]=(-0.420573984429)*x_ref[0]+(0.100192190413)*x_ref[1]+(1.30995115077)*x_ref[2]
        ref[1,2]=(1.34087691694)*x_ref[0]+(-0.151927765846)*x_ref[1]+(0.417185746786)*x_ref[2]
        ref[1,3]=(-0.383933595451)*x_ref[0]+(0.208340861893)*x_ref[1]+(0.947878917669)*x_ref[2]
        ref[1,4]=(-0.143686438189)*x_ref[0]+(-0.00211348684153)*x_ref[1]+(-0.859362566827)*x_ref[2]
        ref[2,0]=(0.447423746179)*x_ref[0]+(-0.636241203952)*x_ref[1]+(-0.0949114725339)*x_ref[2]
        ref[2,1]=(-0.0413740104938)*x_ref[0]+(0.587001644427)*x_ref[1]+(-0.544886520536)*x_ref[2]
        ref[2,2]=(-1.07017617383)*x_ref[0]+(-0.762147671415)*x_ref[1]+(-0.248360379513)*x_ref[2]
        ref[2,3]=(0.391720300962)*x_ref[0]+(1.6521113321)*x_ref[1]+(-0.408205550425)*x_ref[2]
        ref[2,4]=(1.2369249321)*x_ref[0]+(0.952522466347)*x_ref[1]+(-0.404645424067)*x_ref[2]
        ref[3,0]=(0.62410800627)*x_ref[0]+(-0.308929187117)*x_ref[1]+(-0.703796873161)*x_ref[2]
        ref[3,1]=(0.676590956976)*x_ref[0]+(-1.39642714827)*x_ref[1]+(0.198267797885)*x_ref[2]
        ref[3,2]=(-0.202804694134)*x_ref[0]+(0.187577851273)*x_ref[1]+(-0.700960966206)*x_ref[2]
        ref[3,3]=(-0.799833564134)*x_ref[0]+(-0.455781630028)*x_ref[1]+(0.453451623154)*x_ref[2]
        ref[3,4]=(-0.660373484066)*x_ref[0]+(-1.55472528981)*x_ref[1]+(0.0751348530851)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromSymbol_ReducedContinuousFunction_rank3(self):
      """
      tests interpolate for rank 3 Symbol onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedContinuousFunction onto ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(6, 2, 2),dim=dim)
      s=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2),w_ref)
      if dim==2:
        s[0,0,0]=(-1.78813432343)*x[0]+(1.60161751633)*x[1]
        s[0,0,1]=(-1.49849771585)*x[0]+(-0.635435863643)*x[1]
        s[0,1,0]=(-0.906731346391)*x[0]+(0.533678119416)*x[1]
        s[0,1,1]=(-0.792521183513)*x[0]+(0.260998720452)*x[1]
        s[1,0,0]=(-0.710279314945)*x[0]+(0.38834997642)*x[1]
        s[1,0,1]=(-0.168938079833)*x[0]+(-0.502425332838)*x[1]
        s[1,1,0]=(-0.358550156297)*x[0]+(-0.650005639384)*x[1]
        s[1,1,1]=(-1.57793968521)*x[0]+(-0.46781871273)*x[1]
        s[2,0,0]=(0.0112676061889)*x[0]+(0.925017077862)*x[1]
        s[2,0,1]=(-0.109253916051)*x[0]+(-0.38189031862)*x[1]
        s[2,1,0]=(0.777175068327)*x[0]+(1.00196875887)*x[1]
        s[2,1,1]=(-0.27343070108)*x[0]+(-0.11181493242)*x[1]
        s[3,0,0]=(0.321226287504)*x[0]+(0.173667688124)*x[1]
        s[3,0,1]=(1.67405481183)*x[0]+(-0.981404244518)*x[1]
        s[3,1,0]=(0.129874894836)*x[0]+(0.341748165587)*x[1]
        s[3,1,1]=(-0.545737004242)*x[0]+(-1.20936137818)*x[1]
        s[4,0,0]=(0.389007738117)*x[0]+(-0.435046969057)*x[1]
        s[4,0,1]=(1.50296802873)*x[0]+(-0.878616436543)*x[1]
        s[4,1,0]=(1.16067694875)*x[0]+(-0.954317497512)*x[1]
        s[4,1,1]=(1.14634534638)*x[0]+(-0.63981762551)*x[1]
        s[5,0,0]=(0.803750299983)*x[0]+(-0.383611041681)*x[1]
        s[5,0,1]=(0.51863884687)*x[0]+(1.32891347835)*x[1]
        s[5,1,0]=(-0.0775425248593)*x[0]+(-0.0375631918508)*x[1]
        s[5,1,1]=(1.57167118663)*x[0]+(-0.649960686486)*x[1]
        ref[0,0,0]=(-1.78813432343)*x_ref[0]+(1.60161751633)*x_ref[1]
        ref[0,0,1]=(-1.49849771585)*x_ref[0]+(-0.635435863643)*x_ref[1]
        ref[0,1,0]=(-0.906731346391)*x_ref[0]+(0.533678119416)*x_ref[1]
        ref[0,1,1]=(-0.792521183513)*x_ref[0]+(0.260998720452)*x_ref[1]
        ref[1,0,0]=(-0.710279314945)*x_ref[0]+(0.38834997642)*x_ref[1]
        ref[1,0,1]=(-0.168938079833)*x_ref[0]+(-0.502425332838)*x_ref[1]
        ref[1,1,0]=(-0.358550156297)*x_ref[0]+(-0.650005639384)*x_ref[1]
        ref[1,1,1]=(-1.57793968521)*x_ref[0]+(-0.46781871273)*x_ref[1]
        ref[2,0,0]=(0.0112676061889)*x_ref[0]+(0.925017077862)*x_ref[1]
        ref[2,0,1]=(-0.109253916051)*x_ref[0]+(-0.38189031862)*x_ref[1]
        ref[2,1,0]=(0.777175068327)*x_ref[0]+(1.00196875887)*x_ref[1]
        ref[2,1,1]=(-0.27343070108)*x_ref[0]+(-0.11181493242)*x_ref[1]
        ref[3,0,0]=(0.321226287504)*x_ref[0]+(0.173667688124)*x_ref[1]
        ref[3,0,1]=(1.67405481183)*x_ref[0]+(-0.981404244518)*x_ref[1]
        ref[3,1,0]=(0.129874894836)*x_ref[0]+(0.341748165587)*x_ref[1]
        ref[3,1,1]=(-0.545737004242)*x_ref[0]+(-1.20936137818)*x_ref[1]
        ref[4,0,0]=(0.389007738117)*x_ref[0]+(-0.435046969057)*x_ref[1]
        ref[4,0,1]=(1.50296802873)*x_ref[0]+(-0.878616436543)*x_ref[1]
        ref[4,1,0]=(1.16067694875)*x_ref[0]+(-0.954317497512)*x_ref[1]
        ref[4,1,1]=(1.14634534638)*x_ref[0]+(-0.63981762551)*x_ref[1]
        ref[5,0,0]=(0.803750299983)*x_ref[0]+(-0.383611041681)*x_ref[1]
        ref[5,0,1]=(0.51863884687)*x_ref[0]+(1.32891347835)*x_ref[1]
        ref[5,1,0]=(-0.0775425248593)*x_ref[0]+(-0.0375631918508)*x_ref[1]
        ref[5,1,1]=(1.57167118663)*x_ref[0]+(-0.649960686486)*x_ref[1]
      else:
        s[0,0,0]=(1.00224477774)*x[0]+(0.276541980132)*x[1]+(-1.61268557504)*x[2]
        s[0,0,1]=(0.182197948486)*x[0]+(-0.314887566213)*x[1]+(0.172753958549)*x[2]
        s[0,1,0]=(-0.225255532228)*x[0]+(1.64823628561)*x[1]+(0.00289115479583)*x[2]
        s[0,1,1]=(-0.0081734432798)*x[0]+(-1.40127403863)*x[1]+(-0.878765016785)*x[2]
        s[1,0,0]=(-0.88802398859)*x[0]+(0.980539857729)*x[1]+(0.106008000545)*x[2]
        s[1,0,1]=(0.413338006112)*x[0]+(0.238889884193)*x[1]+(0.216982176844)*x[2]
        s[1,1,0]=(-0.375359689292)*x[0]+(0.656302411812)*x[1]+(-1.13744732251)*x[2]
        s[1,1,1]=(-0.768638146497)*x[0]+(0.382131491456)*x[1]+(-0.0473187661226)*x[2]
        s[2,0,0]=(0.727474830988)*x[0]+(-1.15897383904)*x[1]+(1.03107954393)*x[2]
        s[2,0,1]=(-0.565279703477)*x[0]+(0.952275155472)*x[1]+(1.72082557166)*x[2]
        s[2,1,0]=(-0.504951356904)*x[0]+(0.0945851323338)*x[1]+(-1.18137582843)*x[2]
        s[2,1,1]=(0.441559000376)*x[0]+(0.808481091433)*x[1]+(-0.185790180692)*x[2]
        s[3,0,0]=(1.097525477)*x[0]+(1.09598373235)*x[1]+(-0.225656362018)*x[2]
        s[3,0,1]=(-1.64387091178)*x[0]+(0.161580847642)*x[1]+(0.616741154516)*x[2]
        s[3,1,0]=(0.518052869517)*x[0]+(-0.491037975466)*x[1]+(-0.320485227716)*x[2]
        s[3,1,1]=(0.0299906539271)*x[0]+(-0.161268046678)*x[1]+(-0.933417226819)*x[2]
        s[4,0,0]=(-1.45814177886)*x[0]+(-1.01057484083)*x[1]+(1.05668957085)*x[2]
        s[4,0,1]=(0.335720229493)*x[0]+(0.155778050975)*x[1]+(-0.479943899005)*x[2]
        s[4,1,0]=(0.424166905697)*x[0]+(1.04188647689)*x[1]+(-0.154173690871)*x[2]
        s[4,1,1]=(-0.663564713586)*x[0]+(-0.744257330117)*x[1]+(-0.586452598078)*x[2]
        s[5,0,0]=(-0.308954778807)*x[0]+(0.0799719560375)*x[1]+(0.128085782808)*x[2]
        s[5,0,1]=(0.548010504324)*x[0]+(-0.676431304167)*x[1]+(0.12659732457)*x[2]
        s[5,1,0]=(0.17131336527)*x[0]+(-1.54908008145)*x[1]+(0.212836465224)*x[2]
        s[5,1,1]=(-0.834644454471)*x[0]+(1.52867678218)*x[1]+(1.19656096059)*x[2]
        ref[0,0,0]=(1.00224477774)*x_ref[0]+(0.276541980132)*x_ref[1]+(-1.61268557504)*x_ref[2]
        ref[0,0,1]=(0.182197948486)*x_ref[0]+(-0.314887566213)*x_ref[1]+(0.172753958549)*x_ref[2]
        ref[0,1,0]=(-0.225255532228)*x_ref[0]+(1.64823628561)*x_ref[1]+(0.00289115479583)*x_ref[2]
        ref[0,1,1]=(-0.0081734432798)*x_ref[0]+(-1.40127403863)*x_ref[1]+(-0.878765016785)*x_ref[2]
        ref[1,0,0]=(-0.88802398859)*x_ref[0]+(0.980539857729)*x_ref[1]+(0.106008000545)*x_ref[2]
        ref[1,0,1]=(0.413338006112)*x_ref[0]+(0.238889884193)*x_ref[1]+(0.216982176844)*x_ref[2]
        ref[1,1,0]=(-0.375359689292)*x_ref[0]+(0.656302411812)*x_ref[1]+(-1.13744732251)*x_ref[2]
        ref[1,1,1]=(-0.768638146497)*x_ref[0]+(0.382131491456)*x_ref[1]+(-0.0473187661226)*x_ref[2]
        ref[2,0,0]=(0.727474830988)*x_ref[0]+(-1.15897383904)*x_ref[1]+(1.03107954393)*x_ref[2]
        ref[2,0,1]=(-0.565279703477)*x_ref[0]+(0.952275155472)*x_ref[1]+(1.72082557166)*x_ref[2]
        ref[2,1,0]=(-0.504951356904)*x_ref[0]+(0.0945851323338)*x_ref[1]+(-1.18137582843)*x_ref[2]
        ref[2,1,1]=(0.441559000376)*x_ref[0]+(0.808481091433)*x_ref[1]+(-0.185790180692)*x_ref[2]
        ref[3,0,0]=(1.097525477)*x_ref[0]+(1.09598373235)*x_ref[1]+(-0.225656362018)*x_ref[2]
        ref[3,0,1]=(-1.64387091178)*x_ref[0]+(0.161580847642)*x_ref[1]+(0.616741154516)*x_ref[2]
        ref[3,1,0]=(0.518052869517)*x_ref[0]+(-0.491037975466)*x_ref[1]+(-0.320485227716)*x_ref[2]
        ref[3,1,1]=(0.0299906539271)*x_ref[0]+(-0.161268046678)*x_ref[1]+(-0.933417226819)*x_ref[2]
        ref[4,0,0]=(-1.45814177886)*x_ref[0]+(-1.01057484083)*x_ref[1]+(1.05668957085)*x_ref[2]
        ref[4,0,1]=(0.335720229493)*x_ref[0]+(0.155778050975)*x_ref[1]+(-0.479943899005)*x_ref[2]
        ref[4,1,0]=(0.424166905697)*x_ref[0]+(1.04188647689)*x_ref[1]+(-0.154173690871)*x_ref[2]
        ref[4,1,1]=(-0.663564713586)*x_ref[0]+(-0.744257330117)*x_ref[1]+(-0.586452598078)*x_ref[2]
        ref[5,0,0]=(-0.308954778807)*x_ref[0]+(0.0799719560375)*x_ref[1]+(0.128085782808)*x_ref[2]
        ref[5,0,1]=(0.548010504324)*x_ref[0]+(-0.676431304167)*x_ref[1]+(0.12659732457)*x_ref[2]
        ref[5,1,0]=(0.17131336527)*x_ref[0]+(-1.54908008145)*x_ref[1]+(0.212836465224)*x_ref[2]
        ref[5,1,1]=(-0.834644454471)*x_ref[0]+(1.52867678218)*x_ref[1]+(1.19656096059)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromSymbol_ReducedContinuousFunction_rank4(self):
      """
      tests interpolate for rank 4 Symbol onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedContinuousFunction onto ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5, 3, 2),dim=dim)
      s=Data(0,(4, 5, 3, 2),w)
      ref=Data(0,(4, 5, 3, 2),w_ref)
      if dim==2:
        s[0,0,0,0]=(-0.110140466592)*x[0]+(0.758992308989)*x[1]
        s[0,0,0,1]=(-1.27611018159)*x[0]+(-1.21469838588)*x[1]
        s[0,0,1,0]=(-0.695719260931)*x[0]+(-0.631650774678)*x[1]
        s[0,0,1,1]=(-0.0446786678677)*x[0]+(-0.547249041592)*x[1]
        s[0,0,2,0]=(0.824673646909)*x[0]+(-1.53499816213)*x[1]
        s[0,0,2,1]=(0.902244689111)*x[0]+(0.194498326222)*x[1]
        s[0,1,0,0]=(-1.29097330044)*x[0]+(0.0328620774092)*x[1]
        s[0,1,0,1]=(1.20419739576)*x[0]+(-1.25415670694)*x[1]
        s[0,1,1,0]=(0.825311334936)*x[0]+(0.732241589298)*x[1]
        s[0,1,1,1]=(0.239743697336)*x[0]+(0.446701903969)*x[1]
        s[0,1,2,0]=(-0.208525758105)*x[0]+(0.564727160849)*x[1]
        s[0,1,2,1]=(0.425352221432)*x[0]+(-0.439885368017)*x[1]
        s[0,2,0,0]=(0.089403990032)*x[0]+(0.927312736507)*x[1]
        s[0,2,0,1]=(-1.98342026057)*x[0]+(0.908824462154)*x[1]
        s[0,2,1,0]=(-1.02677966987)*x[0]+(1.64009929611)*x[1]
        s[0,2,1,1]=(1.18506229959)*x[0]+(-0.336041064484)*x[1]
        s[0,2,2,0]=(0.124121400983)*x[0]+(0.169858827797)*x[1]
        s[0,2,2,1]=(0.0171983721215)*x[0]+(-0.679377487833)*x[1]
        s[0,3,0,0]=(0.707474586567)*x[0]+(-0.327636474016)*x[1]
        s[0,3,0,1]=(-0.183596474381)*x[0]+(-1.07263498957)*x[1]
        s[0,3,1,0]=(1.49812660538)*x[0]+(-0.313209529867)*x[1]
        s[0,3,1,1]=(-0.678162796237)*x[0]+(-0.823724533082)*x[1]
        s[0,3,2,0]=(-0.79806833841)*x[0]+(0.865576104504)*x[1]
        s[0,3,2,1]=(0.941864201983)*x[0]+(0.734846756813)*x[1]
        s[0,4,0,0]=(-0.205171531689)*x[0]+(0.299610959481)*x[1]
        s[0,4,0,1]=(0.643430728461)*x[0]+(0.584994687875)*x[1]
        s[0,4,1,0]=(-0.408681959359)*x[0]+(0.601772358172)*x[1]
        s[0,4,1,1]=(-1.44472725121)*x[0]+(-0.0899809192849)*x[1]
        s[0,4,2,0]=(0.722764720152)*x[0]+(-0.407449476647)*x[1]
        s[0,4,2,1]=(0.166269375199)*x[0]+(1.20016653459)*x[1]
        s[1,0,0,0]=(-0.764665577343)*x[0]+(0.15492015266)*x[1]
        s[1,0,0,1]=(0.822023595189)*x[0]+(0.00511531586673)*x[1]
        s[1,0,1,0]=(0.0840294388025)*x[0]+(0.393606253186)*x[1]
        s[1,0,1,1]=(-0.702650062915)*x[0]+(-1.10361836979)*x[1]
        s[1,0,2,0]=(-1.34569564836)*x[0]+(-1.50816431101)*x[1]
        s[1,0,2,1]=(1.11337280315)*x[0]+(-0.0771805114838)*x[1]
        s[1,1,0,0]=(-0.0695861126454)*x[0]+(-0.437020174051)*x[1]
        s[1,1,0,1]=(1.68924343517)*x[0]+(0.706853885933)*x[1]
        s[1,1,1,0]=(0.0806933170819)*x[0]+(-0.974818406182)*x[1]
        s[1,1,1,1]=(-0.236836420312)*x[0]+(-0.352629546416)*x[1]
        s[1,1,2,0]=(-0.312697720062)*x[0]+(-0.977312197678)*x[1]
        s[1,1,2,1]=(-0.263563591998)*x[0]+(-0.899465194216)*x[1]
        s[1,2,0,0]=(1.0599010032)*x[0]+(0.625641179018)*x[1]
        s[1,2,0,1]=(0.42589369081)*x[0]+(0.63136683361)*x[1]
        s[1,2,1,0]=(0.629519575902)*x[0]+(-1.03125707286)*x[1]
        s[1,2,1,1]=(0.479272369893)*x[0]+(-1.63724584249)*x[1]
        s[1,2,2,0]=(1.48877937845)*x[0]+(-0.802308310767)*x[1]
        s[1,2,2,1]=(0.374636882204)*x[0]+(0.187733613588)*x[1]
        s[1,3,0,0]=(0.360148707925)*x[0]+(1.48869239048)*x[1]
        s[1,3,0,1]=(-1.29194970756)*x[0]+(-0.433561540189)*x[1]
        s[1,3,1,0]=(0.696118719028)*x[0]+(1.73199962229)*x[1]
        s[1,3,1,1]=(1.63831809651)*x[0]+(0.70687058079)*x[1]
        s[1,3,2,0]=(1.03614006811)*x[0]+(-0.783037005502)*x[1]
        s[1,3,2,1]=(0.351931393443)*x[0]+(-0.720060584954)*x[1]
        s[1,4,0,0]=(0.856138152361)*x[0]+(-0.0733689526147)*x[1]
        s[1,4,0,1]=(-0.256696810525)*x[0]+(1.57547133796)*x[1]
        s[1,4,1,0]=(0.260933620175)*x[0]+(-0.528418816229)*x[1]
        s[1,4,1,1]=(-1.14624698517)*x[0]+(0.537831890429)*x[1]
        s[1,4,2,0]=(-1.11328030308)*x[0]+(0.477599527943)*x[1]
        s[1,4,2,1]=(-1.52135700331)*x[0]+(1.18803329651)*x[1]
        s[2,0,0,0]=(1.02103992957)*x[0]+(1.3428520083)*x[1]
        s[2,0,0,1]=(0.74126089694)*x[0]+(-0.747092126031)*x[1]
        s[2,0,1,0]=(-0.300534057945)*x[0]+(-0.0679226717228)*x[1]
        s[2,0,1,1]=(0.205780243466)*x[0]+(1.89346679377)*x[1]
        s[2,0,2,0]=(0.711997044871)*x[0]+(0.614186591236)*x[1]
        s[2,0,2,1]=(0.562225764659)*x[0]+(-0.75210737336)*x[1]
        s[2,1,0,0]=(1.51024763079)*x[0]+(0.0655411043245)*x[1]
        s[2,1,0,1]=(-1.60270171758)*x[0]+(-0.555919918868)*x[1]
        s[2,1,1,0]=(0.0117250633883)*x[0]+(1.06089889984)*x[1]
        s[2,1,1,1]=(1.41839826051)*x[0]+(-1.11883387918)*x[1]
        s[2,1,2,0]=(-1.47481010167)*x[0]+(0.21986710996)*x[1]
        s[2,1,2,1]=(-0.890780004667)*x[0]+(0.68206435926)*x[1]
        s[2,2,0,0]=(0.00647580761798)*x[0]+(1.08988646609)*x[1]
        s[2,2,0,1]=(0.107576626365)*x[0]+(0.569541010093)*x[1]
        s[2,2,1,0]=(0.655745883134)*x[0]+(1.59781027174)*x[1]
        s[2,2,1,1]=(0.423699701576)*x[0]+(-0.854187892248)*x[1]
        s[2,2,2,0]=(-0.904849730929)*x[0]+(0.235074145696)*x[1]
        s[2,2,2,1]=(0.981525463443)*x[0]+(0.701832385796)*x[1]
        s[2,3,0,0]=(-0.489533883573)*x[0]+(-0.0213717056714)*x[1]
        s[2,3,0,1]=(0.516832539105)*x[0]+(-1.11995329355)*x[1]
        s[2,3,1,0]=(0.287740517981)*x[0]+(-1.3210942817)*x[1]
        s[2,3,1,1]=(0.179604184524)*x[0]+(-1.63664715586)*x[1]
        s[2,3,2,0]=(-0.492296320142)*x[0]+(-0.502235503805)*x[1]
        s[2,3,2,1]=(-0.653587673689)*x[0]+(0.5466761379)*x[1]
        s[2,4,0,0]=(1.37393629094)*x[0]+(0.645991033265)*x[1]
        s[2,4,0,1]=(-0.108057947979)*x[0]+(-0.761267645781)*x[1]
        s[2,4,1,0]=(-0.0900524038235)*x[0]+(0.16773443574)*x[1]
        s[2,4,1,1]=(0.578749842767)*x[0]+(1.18687780739)*x[1]
        s[2,4,2,0]=(-0.349879331342)*x[0]+(-0.214411558047)*x[1]
        s[2,4,2,1]=(-0.875945724127)*x[0]+(-1.36065436643)*x[1]
        s[3,0,0,0]=(0.0488555315168)*x[0]+(-0.82382271801)*x[1]
        s[3,0,0,1]=(1.24017449448)*x[0]+(-0.911198047419)*x[1]
        s[3,0,1,0]=(0.30200654088)*x[0]+(-1.11559310554)*x[1]
        s[3,0,1,1]=(0.67275005347)*x[0]+(-0.645034711632)*x[1]
        s[3,0,2,0]=(-0.670229614054)*x[0]+(0.359423039794)*x[1]
        s[3,0,2,1]=(1.14988130646)*x[0]+(0.86614224885)*x[1]
        s[3,1,0,0]=(0.705716005963)*x[0]+(-1.15357209623)*x[1]
        s[3,1,0,1]=(0.31067508882)*x[0]+(0.960823286011)*x[1]
        s[3,1,1,0]=(0.866232494898)*x[0]+(-0.65162103259)*x[1]
        s[3,1,1,1]=(-0.452071783562)*x[0]+(-0.211956993156)*x[1]
        s[3,1,2,0]=(-1.32137973484)*x[0]+(1.49152904496)*x[1]
        s[3,1,2,1]=(0.854595989445)*x[0]+(1.11534387313)*x[1]
        s[3,2,0,0]=(0.789093645939)*x[0]+(-0.623866327905)*x[1]
        s[3,2,0,1]=(-0.649118397874)*x[0]+(-0.510923012103)*x[1]
        s[3,2,1,0]=(-0.263103480764)*x[0]+(0.0353171690366)*x[1]
        s[3,2,1,1]=(-0.910441885706)*x[0]+(-0.572413584482)*x[1]
        s[3,2,2,0]=(-0.764615254979)*x[0]+(-0.257894213166)*x[1]
        s[3,2,2,1]=(0.82110723418)*x[0]+(-0.881337133364)*x[1]
        s[3,3,0,0]=(-0.978909443882)*x[0]+(0.95340950915)*x[1]
        s[3,3,0,1]=(-0.121221461759)*x[0]+(-1.68773270918)*x[1]
        s[3,3,1,0]=(-0.150032516264)*x[0]+(0.441210519728)*x[1]
        s[3,3,1,1]=(-0.618694838164)*x[0]+(0.379748674791)*x[1]
        s[3,3,2,0]=(0.208366138002)*x[0]+(1.04701688993)*x[1]
        s[3,3,2,1]=(0.207672316371)*x[0]+(-1.04311602421)*x[1]
        s[3,4,0,0]=(0.404190185535)*x[0]+(0.206083873066)*x[1]
        s[3,4,0,1]=(0.928741940014)*x[0]+(0.126856418039)*x[1]
        s[3,4,1,0]=(0.0831637965324)*x[0]+(0.198288732425)*x[1]
        s[3,4,1,1]=(0.602722196536)*x[0]+(-0.256860860163)*x[1]
        s[3,4,2,0]=(1.11330980537)*x[0]+(0.130706767542)*x[1]
        s[3,4,2,1]=(0.638478712945)*x[0]+(0.558803659636)*x[1]
        ref[0,0,0,0]=(-0.110140466592)*x_ref[0]+(0.758992308989)*x_ref[1]
        ref[0,0,0,1]=(-1.27611018159)*x_ref[0]+(-1.21469838588)*x_ref[1]
        ref[0,0,1,0]=(-0.695719260931)*x_ref[0]+(-0.631650774678)*x_ref[1]
        ref[0,0,1,1]=(-0.0446786678677)*x_ref[0]+(-0.547249041592)*x_ref[1]
        ref[0,0,2,0]=(0.824673646909)*x_ref[0]+(-1.53499816213)*x_ref[1]
        ref[0,0,2,1]=(0.902244689111)*x_ref[0]+(0.194498326222)*x_ref[1]
        ref[0,1,0,0]=(-1.29097330044)*x_ref[0]+(0.0328620774092)*x_ref[1]
        ref[0,1,0,1]=(1.20419739576)*x_ref[0]+(-1.25415670694)*x_ref[1]
        ref[0,1,1,0]=(0.825311334936)*x_ref[0]+(0.732241589298)*x_ref[1]
        ref[0,1,1,1]=(0.239743697336)*x_ref[0]+(0.446701903969)*x_ref[1]
        ref[0,1,2,0]=(-0.208525758105)*x_ref[0]+(0.564727160849)*x_ref[1]
        ref[0,1,2,1]=(0.425352221432)*x_ref[0]+(-0.439885368017)*x_ref[1]
        ref[0,2,0,0]=(0.089403990032)*x_ref[0]+(0.927312736507)*x_ref[1]
        ref[0,2,0,1]=(-1.98342026057)*x_ref[0]+(0.908824462154)*x_ref[1]
        ref[0,2,1,0]=(-1.02677966987)*x_ref[0]+(1.64009929611)*x_ref[1]
        ref[0,2,1,1]=(1.18506229959)*x_ref[0]+(-0.336041064484)*x_ref[1]
        ref[0,2,2,0]=(0.124121400983)*x_ref[0]+(0.169858827797)*x_ref[1]
        ref[0,2,2,1]=(0.0171983721215)*x_ref[0]+(-0.679377487833)*x_ref[1]
        ref[0,3,0,0]=(0.707474586567)*x_ref[0]+(-0.327636474016)*x_ref[1]
        ref[0,3,0,1]=(-0.183596474381)*x_ref[0]+(-1.07263498957)*x_ref[1]
        ref[0,3,1,0]=(1.49812660538)*x_ref[0]+(-0.313209529867)*x_ref[1]
        ref[0,3,1,1]=(-0.678162796237)*x_ref[0]+(-0.823724533082)*x_ref[1]
        ref[0,3,2,0]=(-0.79806833841)*x_ref[0]+(0.865576104504)*x_ref[1]
        ref[0,3,2,1]=(0.941864201983)*x_ref[0]+(0.734846756813)*x_ref[1]
        ref[0,4,0,0]=(-0.205171531689)*x_ref[0]+(0.299610959481)*x_ref[1]
        ref[0,4,0,1]=(0.643430728461)*x_ref[0]+(0.584994687875)*x_ref[1]
        ref[0,4,1,0]=(-0.408681959359)*x_ref[0]+(0.601772358172)*x_ref[1]
        ref[0,4,1,1]=(-1.44472725121)*x_ref[0]+(-0.0899809192849)*x_ref[1]
        ref[0,4,2,0]=(0.722764720152)*x_ref[0]+(-0.407449476647)*x_ref[1]
        ref[0,4,2,1]=(0.166269375199)*x_ref[0]+(1.20016653459)*x_ref[1]
        ref[1,0,0,0]=(-0.764665577343)*x_ref[0]+(0.15492015266)*x_ref[1]
        ref[1,0,0,1]=(0.822023595189)*x_ref[0]+(0.00511531586673)*x_ref[1]
        ref[1,0,1,0]=(0.0840294388025)*x_ref[0]+(0.393606253186)*x_ref[1]
        ref[1,0,1,1]=(-0.702650062915)*x_ref[0]+(-1.10361836979)*x_ref[1]
        ref[1,0,2,0]=(-1.34569564836)*x_ref[0]+(-1.50816431101)*x_ref[1]
        ref[1,0,2,1]=(1.11337280315)*x_ref[0]+(-0.0771805114838)*x_ref[1]
        ref[1,1,0,0]=(-0.0695861126454)*x_ref[0]+(-0.437020174051)*x_ref[1]
        ref[1,1,0,1]=(1.68924343517)*x_ref[0]+(0.706853885933)*x_ref[1]
        ref[1,1,1,0]=(0.0806933170819)*x_ref[0]+(-0.974818406182)*x_ref[1]
        ref[1,1,1,1]=(-0.236836420312)*x_ref[0]+(-0.352629546416)*x_ref[1]
        ref[1,1,2,0]=(-0.312697720062)*x_ref[0]+(-0.977312197678)*x_ref[1]
        ref[1,1,2,1]=(-0.263563591998)*x_ref[0]+(-0.899465194216)*x_ref[1]
        ref[1,2,0,0]=(1.0599010032)*x_ref[0]+(0.625641179018)*x_ref[1]
        ref[1,2,0,1]=(0.42589369081)*x_ref[0]+(0.63136683361)*x_ref[1]
        ref[1,2,1,0]=(0.629519575902)*x_ref[0]+(-1.03125707286)*x_ref[1]
        ref[1,2,1,1]=(0.479272369893)*x_ref[0]+(-1.63724584249)*x_ref[1]
        ref[1,2,2,0]=(1.48877937845)*x_ref[0]+(-0.802308310767)*x_ref[1]
        ref[1,2,2,1]=(0.374636882204)*x_ref[0]+(0.187733613588)*x_ref[1]
        ref[1,3,0,0]=(0.360148707925)*x_ref[0]+(1.48869239048)*x_ref[1]
        ref[1,3,0,1]=(-1.29194970756)*x_ref[0]+(-0.433561540189)*x_ref[1]
        ref[1,3,1,0]=(0.696118719028)*x_ref[0]+(1.73199962229)*x_ref[1]
        ref[1,3,1,1]=(1.63831809651)*x_ref[0]+(0.70687058079)*x_ref[1]
        ref[1,3,2,0]=(1.03614006811)*x_ref[0]+(-0.783037005502)*x_ref[1]
        ref[1,3,2,1]=(0.351931393443)*x_ref[0]+(-0.720060584954)*x_ref[1]
        ref[1,4,0,0]=(0.856138152361)*x_ref[0]+(-0.0733689526147)*x_ref[1]
        ref[1,4,0,1]=(-0.256696810525)*x_ref[0]+(1.57547133796)*x_ref[1]
        ref[1,4,1,0]=(0.260933620175)*x_ref[0]+(-0.528418816229)*x_ref[1]
        ref[1,4,1,1]=(-1.14624698517)*x_ref[0]+(0.537831890429)*x_ref[1]
        ref[1,4,2,0]=(-1.11328030308)*x_ref[0]+(0.477599527943)*x_ref[1]
        ref[1,4,2,1]=(-1.52135700331)*x_ref[0]+(1.18803329651)*x_ref[1]
        ref[2,0,0,0]=(1.02103992957)*x_ref[0]+(1.3428520083)*x_ref[1]
        ref[2,0,0,1]=(0.74126089694)*x_ref[0]+(-0.747092126031)*x_ref[1]
        ref[2,0,1,0]=(-0.300534057945)*x_ref[0]+(-0.0679226717228)*x_ref[1]
        ref[2,0,1,1]=(0.205780243466)*x_ref[0]+(1.89346679377)*x_ref[1]
        ref[2,0,2,0]=(0.711997044871)*x_ref[0]+(0.614186591236)*x_ref[1]
        ref[2,0,2,1]=(0.562225764659)*x_ref[0]+(-0.75210737336)*x_ref[1]
        ref[2,1,0,0]=(1.51024763079)*x_ref[0]+(0.0655411043245)*x_ref[1]
        ref[2,1,0,1]=(-1.60270171758)*x_ref[0]+(-0.555919918868)*x_ref[1]
        ref[2,1,1,0]=(0.0117250633883)*x_ref[0]+(1.06089889984)*x_ref[1]
        ref[2,1,1,1]=(1.41839826051)*x_ref[0]+(-1.11883387918)*x_ref[1]
        ref[2,1,2,0]=(-1.47481010167)*x_ref[0]+(0.21986710996)*x_ref[1]
        ref[2,1,2,1]=(-0.890780004667)*x_ref[0]+(0.68206435926)*x_ref[1]
        ref[2,2,0,0]=(0.00647580761798)*x_ref[0]+(1.08988646609)*x_ref[1]
        ref[2,2,0,1]=(0.107576626365)*x_ref[0]+(0.569541010093)*x_ref[1]
        ref[2,2,1,0]=(0.655745883134)*x_ref[0]+(1.59781027174)*x_ref[1]
        ref[2,2,1,1]=(0.423699701576)*x_ref[0]+(-0.854187892248)*x_ref[1]
        ref[2,2,2,0]=(-0.904849730929)*x_ref[0]+(0.235074145696)*x_ref[1]
        ref[2,2,2,1]=(0.981525463443)*x_ref[0]+(0.701832385796)*x_ref[1]
        ref[2,3,0,0]=(-0.489533883573)*x_ref[0]+(-0.0213717056714)*x_ref[1]
        ref[2,3,0,1]=(0.516832539105)*x_ref[0]+(-1.11995329355)*x_ref[1]
        ref[2,3,1,0]=(0.287740517981)*x_ref[0]+(-1.3210942817)*x_ref[1]
        ref[2,3,1,1]=(0.179604184524)*x_ref[0]+(-1.63664715586)*x_ref[1]
        ref[2,3,2,0]=(-0.492296320142)*x_ref[0]+(-0.502235503805)*x_ref[1]
        ref[2,3,2,1]=(-0.653587673689)*x_ref[0]+(0.5466761379)*x_ref[1]
        ref[2,4,0,0]=(1.37393629094)*x_ref[0]+(0.645991033265)*x_ref[1]
        ref[2,4,0,1]=(-0.108057947979)*x_ref[0]+(-0.761267645781)*x_ref[1]
        ref[2,4,1,0]=(-0.0900524038235)*x_ref[0]+(0.16773443574)*x_ref[1]
        ref[2,4,1,1]=(0.578749842767)*x_ref[0]+(1.18687780739)*x_ref[1]
        ref[2,4,2,0]=(-0.349879331342)*x_ref[0]+(-0.214411558047)*x_ref[1]
        ref[2,4,2,1]=(-0.875945724127)*x_ref[0]+(-1.36065436643)*x_ref[1]
        ref[3,0,0,0]=(0.0488555315168)*x_ref[0]+(-0.82382271801)*x_ref[1]
        ref[3,0,0,1]=(1.24017449448)*x_ref[0]+(-0.911198047419)*x_ref[1]
        ref[3,0,1,0]=(0.30200654088)*x_ref[0]+(-1.11559310554)*x_ref[1]
        ref[3,0,1,1]=(0.67275005347)*x_ref[0]+(-0.645034711632)*x_ref[1]
        ref[3,0,2,0]=(-0.670229614054)*x_ref[0]+(0.359423039794)*x_ref[1]
        ref[3,0,2,1]=(1.14988130646)*x_ref[0]+(0.86614224885)*x_ref[1]
        ref[3,1,0,0]=(0.705716005963)*x_ref[0]+(-1.15357209623)*x_ref[1]
        ref[3,1,0,1]=(0.31067508882)*x_ref[0]+(0.960823286011)*x_ref[1]
        ref[3,1,1,0]=(0.866232494898)*x_ref[0]+(-0.65162103259)*x_ref[1]
        ref[3,1,1,1]=(-0.452071783562)*x_ref[0]+(-0.211956993156)*x_ref[1]
        ref[3,1,2,0]=(-1.32137973484)*x_ref[0]+(1.49152904496)*x_ref[1]
        ref[3,1,2,1]=(0.854595989445)*x_ref[0]+(1.11534387313)*x_ref[1]
        ref[3,2,0,0]=(0.789093645939)*x_ref[0]+(-0.623866327905)*x_ref[1]
        ref[3,2,0,1]=(-0.649118397874)*x_ref[0]+(-0.510923012103)*x_ref[1]
        ref[3,2,1,0]=(-0.263103480764)*x_ref[0]+(0.0353171690366)*x_ref[1]
        ref[3,2,1,1]=(-0.910441885706)*x_ref[0]+(-0.572413584482)*x_ref[1]
        ref[3,2,2,0]=(-0.764615254979)*x_ref[0]+(-0.257894213166)*x_ref[1]
        ref[3,2,2,1]=(0.82110723418)*x_ref[0]+(-0.881337133364)*x_ref[1]
        ref[3,3,0,0]=(-0.978909443882)*x_ref[0]+(0.95340950915)*x_ref[1]
        ref[3,3,0,1]=(-0.121221461759)*x_ref[0]+(-1.68773270918)*x_ref[1]
        ref[3,3,1,0]=(-0.150032516264)*x_ref[0]+(0.441210519728)*x_ref[1]
        ref[3,3,1,1]=(-0.618694838164)*x_ref[0]+(0.379748674791)*x_ref[1]
        ref[3,3,2,0]=(0.208366138002)*x_ref[0]+(1.04701688993)*x_ref[1]
        ref[3,3,2,1]=(0.207672316371)*x_ref[0]+(-1.04311602421)*x_ref[1]
        ref[3,4,0,0]=(0.404190185535)*x_ref[0]+(0.206083873066)*x_ref[1]
        ref[3,4,0,1]=(0.928741940014)*x_ref[0]+(0.126856418039)*x_ref[1]
        ref[3,4,1,0]=(0.0831637965324)*x_ref[0]+(0.198288732425)*x_ref[1]
        ref[3,4,1,1]=(0.602722196536)*x_ref[0]+(-0.256860860163)*x_ref[1]
        ref[3,4,2,0]=(1.11330980537)*x_ref[0]+(0.130706767542)*x_ref[1]
        ref[3,4,2,1]=(0.638478712945)*x_ref[0]+(0.558803659636)*x_ref[1]
      else:
        s[0,0,0,0]=(-0.678750626234)*x[0]+(0.707898134345)*x[1]+(1.47682355575)*x[2]
        s[0,0,0,1]=(-1.08665921875)*x[0]+(-1.03746577329)*x[1]+(0.459220112015)*x[2]
        s[0,0,1,0]=(-0.0924850687831)*x[0]+(-1.21392495875)*x[1]+(-0.335482829106)*x[2]
        s[0,0,1,1]=(-0.796044372195)*x[0]+(-1.61600945946)*x[1]+(-0.917678370431)*x[2]
        s[0,0,2,0]=(-1.17314148071)*x[0]+(0.594053121363)*x[1]+(-1.0751981685)*x[2]
        s[0,0,2,1]=(-0.993747266312)*x[0]+(0.647546031964)*x[1]+(0.789233440797)*x[2]
        s[0,1,0,0]=(-0.538620007403)*x[0]+(0.572432466547)*x[1]+(1.59224293356)*x[2]
        s[0,1,0,1]=(-0.315217413376)*x[0]+(1.30492005409)*x[1]+(-0.04944666559)*x[2]
        s[0,1,1,0]=(-0.734247343541)*x[0]+(1.14104633611)*x[1]+(-0.0424483055941)*x[2]
        s[0,1,1,1]=(-0.521727902651)*x[0]+(-0.0302866863619)*x[1]+(-0.0378416654735)*x[2]
        s[0,1,2,0]=(0.319532436452)*x[0]+(0.312388869947)*x[1]+(0.237025751349)*x[2]
        s[0,1,2,1]=(0.713884175591)*x[0]+(1.10911840343)*x[1]+(-1.4943567087)*x[2]
        s[0,2,0,0]=(-0.599254236697)*x[0]+(-0.190373139957)*x[1]+(-1.75168866737)*x[2]
        s[0,2,0,1]=(0.396782073165)*x[0]+(-0.252470841512)*x[1]+(-0.463651254287)*x[2]
        s[0,2,1,0]=(1.03549889185)*x[0]+(0.225894922761)*x[1]+(-0.609139538729)*x[2]
        s[0,2,1,1]=(0.289458873955)*x[0]+(-0.136668874286)*x[1]+(0.474827071819)*x[2]
        s[0,2,2,0]=(0.654253727057)*x[0]+(-0.998748120229)*x[1]+(-0.107718066457)*x[2]
        s[0,2,2,1]=(0.256710826049)*x[0]+(0.309515575678)*x[1]+(-1.60310813555)*x[2]
        s[0,3,0,0]=(1.70779242771)*x[0]+(-0.530818399689)*x[1]+(-0.516799319419)*x[2]
        s[0,3,0,1]=(-0.848868447677)*x[0]+(-0.301212439564)*x[1]+(-0.423135852525)*x[2]
        s[0,3,1,0]=(1.0807833809)*x[0]+(-1.20726021059)*x[1]+(0.565314004443)*x[2]
        s[0,3,1,1]=(-0.660234771003)*x[0]+(-0.443966617085)*x[1]+(-0.185381740077)*x[2]
        s[0,3,2,0]=(1.6260513125)*x[0]+(0.145626479867)*x[1]+(-0.808965403823)*x[2]
        s[0,3,2,1]=(-0.503772598493)*x[0]+(-1.63975947666)*x[1]+(0.255681416016)*x[2]
        s[0,4,0,0]=(0.0818777313252)*x[0]+(0.428155907605)*x[1]+(-0.642991998051)*x[2]
        s[0,4,0,1]=(-0.145032953266)*x[0]+(-0.7645425853)*x[1]+(0.458784780224)*x[2]
        s[0,4,1,0]=(-0.589583511871)*x[0]+(-0.0743175116482)*x[1]+(0.285113485271)*x[2]
        s[0,4,1,1]=(-1.37305892557)*x[0]+(0.917822397515)*x[1]+(-0.378127728174)*x[2]
        s[0,4,2,0]=(1.16505232525)*x[0]+(1.10944900772)*x[1]+(-0.95491955738)*x[2]
        s[0,4,2,1]=(-1.12539432931)*x[0]+(-0.290265022179)*x[1]+(-1.20809121853)*x[2]
        s[1,0,0,0]=(-0.859420732972)*x[0]+(0.564058561557)*x[1]+(-0.795155516002)*x[2]
        s[1,0,0,1]=(-0.314313522693)*x[0]+(1.46275003784)*x[1]+(0.285596389281)*x[2]
        s[1,0,1,0]=(-0.513360534744)*x[0]+(-0.0532410726117)*x[1]+(-0.122433325652)*x[2]
        s[1,0,1,1]=(0.527488940208)*x[0]+(0.0760609834598)*x[1]+(-0.612300815838)*x[2]
        s[1,0,2,0]=(0.302549971797)*x[0]+(-0.710901255568)*x[1]+(0.955062818656)*x[2]
        s[1,0,2,1]=(0.126520802001)*x[0]+(-1.07564234283)*x[1]+(-0.669526781403)*x[2]
        s[1,1,0,0]=(1.50804296428)*x[0]+(0.111903566342)*x[1]+(0.465876365517)*x[2]
        s[1,1,0,1]=(-1.04380726708)*x[0]+(0.148245186406)*x[1]+(-1.06801443458)*x[2]
        s[1,1,1,0]=(1.48236374583)*x[0]+(-0.00794102905726)*x[1]+(-1.35613430084)*x[2]
        s[1,1,1,1]=(1.04119256641)*x[0]+(-1.84075261029)*x[1]+(-0.776969168414)*x[2]
        s[1,1,2,0]=(-0.356539020393)*x[0]+(1.70797570799)*x[1]+(-0.525736126605)*x[2]
        s[1,1,2,1]=(1.62559049194)*x[0]+(-0.482482804647)*x[1]+(0.631790498783)*x[2]
        s[1,2,0,0]=(0.499315452031)*x[0]+(-0.296700233772)*x[1]+(1.00111627057)*x[2]
        s[1,2,0,1]=(1.0433119055)*x[0]+(-0.269957745366)*x[1]+(0.591153091169)*x[2]
        s[1,2,1,0]=(0.502068262492)*x[0]+(-0.849037462803)*x[1]+(-1.56692020458)*x[2]
        s[1,2,1,1]=(1.2984795003)*x[0]+(-1.07653573134)*x[1]+(0.487232329161)*x[2]
        s[1,2,2,0]=(-0.109333568443)*x[0]+(0.687162555279)*x[1]+(-0.812636431387)*x[2]
        s[1,2,2,1]=(0.202326014817)*x[0]+(1.23297194202)*x[1]+(1.41163217274)*x[2]
        s[1,3,0,0]=(0.124740742544)*x[0]+(-0.500878894409)*x[1]+(-0.376438702019)*x[2]
        s[1,3,0,1]=(-0.540234624582)*x[0]+(-0.400577181452)*x[1]+(0.879215681428)*x[2]
        s[1,3,1,0]=(0.0426835015338)*x[0]+(-0.726976626459)*x[1]+(1.32655296741)*x[2]
        s[1,3,1,1]=(-0.120969369352)*x[0]+(-0.426853376816)*x[1]+(1.88787704976)*x[2]
        s[1,3,2,0]=(-0.0895115836038)*x[0]+(-1.17409640569)*x[1]+(-0.874853748928)*x[2]
        s[1,3,2,1]=(-0.0664677570269)*x[0]+(0.708204452159)*x[1]+(-0.26397721498)*x[2]
        s[1,4,0,0]=(0.0236750184919)*x[0]+(-1.42225228911)*x[1]+(0.399502903376)*x[2]
        s[1,4,0,1]=(1.03400366274)*x[0]+(-0.442018365874)*x[1]+(1.02491120541)*x[2]
        s[1,4,1,0]=(-0.150528717616)*x[0]+(-0.512236709632)*x[1]+(-0.0644728802768)*x[2]
        s[1,4,1,1]=(-0.998520493595)*x[0]+(-0.507010096027)*x[1]+(-0.889168150742)*x[2]
        s[1,4,2,0]=(0.179165278401)*x[0]+(1.07136931045)*x[1]+(-0.790101000847)*x[2]
        s[1,4,2,1]=(0.376877479598)*x[0]+(-0.25979387806)*x[1]+(0.446682478544)*x[2]
        s[2,0,0,0]=(0.365668617405)*x[0]+(1.44268125148)*x[1]+(1.65785163405)*x[2]
        s[2,0,0,1]=(0.576180639993)*x[0]+(-0.0571374651916)*x[1]+(-0.0854307657892)*x[2]
        s[2,0,1,0]=(0.0474860699991)*x[0]+(-1.47716457199)*x[1]+(0.87300241949)*x[2]
        s[2,0,1,1]=(-0.293608884991)*x[0]+(0.0963236497693)*x[1]+(-1.31217901681)*x[2]
        s[2,0,2,0]=(-0.725125904522)*x[0]+(0.15259525725)*x[1]+(0.0624842579957)*x[2]
        s[2,0,2,1]=(-0.374667932076)*x[0]+(-0.586704716252)*x[1]+(-0.208259155061)*x[2]
        s[2,1,0,0]=(0.480010407761)*x[0]+(-1.78856630141)*x[1]+(0.296814589368)*x[2]
        s[2,1,0,1]=(-0.131871389585)*x[0]+(0.0984653022698)*x[1]+(0.222785695838)*x[2]
        s[2,1,1,0]=(-0.4095594348)*x[0]+(-0.746059065385)*x[1]+(1.72282248984)*x[2]
        s[2,1,1,1]=(0.841882742791)*x[0]+(0.698803891138)*x[1]+(0.201279240318)*x[2]
        s[2,1,2,0]=(-1.52503851146)*x[0]+(0.746778574384)*x[1]+(0.820127798751)*x[2]
        s[2,1,2,1]=(0.578616447592)*x[0]+(-0.232884112626)*x[1]+(0.719949134498)*x[2]
        s[2,2,0,0]=(-0.250869865455)*x[0]+(0.0457838553118)*x[1]+(0.694605224088)*x[2]
        s[2,2,0,1]=(-0.344703134894)*x[0]+(-0.945296587028)*x[1]+(-0.408991625518)*x[2]
        s[2,2,1,0]=(1.15842118069)*x[0]+(0.385018100988)*x[1]+(-0.298867420422)*x[2]
        s[2,2,1,1]=(-0.992212925836)*x[0]+(-0.303921757453)*x[1]+(1.90996417539)*x[2]
        s[2,2,2,0]=(0.139346086335)*x[0]+(-1.5255000175)*x[1]+(-0.418223280061)*x[2]
        s[2,2,2,1]=(-0.287165354844)*x[0]+(-0.563470113789)*x[1]+(-0.267078242542)*x[2]
        s[2,3,0,0]=(-0.877956931038)*x[0]+(-0.587379404687)*x[1]+(0.223772989101)*x[2]
        s[2,3,0,1]=(0.450240028907)*x[0]+(0.933649504645)*x[1]+(0.922982912622)*x[2]
        s[2,3,1,0]=(-0.695647633954)*x[0]+(0.675123237641)*x[1]+(0.333013288603)*x[2]
        s[2,3,1,1]=(-0.638319791165)*x[0]+(-1.01905860587)*x[1]+(-0.0829897514576)*x[2]
        s[2,3,2,0]=(-0.85507363839)*x[0]+(0.213154292163)*x[1]+(-1.19634907609)*x[2]
        s[2,3,2,1]=(0.51969848742)*x[0]+(-0.103353752424)*x[1]+(1.38100947757)*x[2]
        s[2,4,0,0]=(-1.42264362114)*x[0]+(-0.0622861012095)*x[1]+(1.11415860946)*x[2]
        s[2,4,0,1]=(1.03146522202)*x[0]+(-0.59775537826)*x[1]+(0.455451132416)*x[2]
        s[2,4,1,0]=(-1.42367400988)*x[0]+(0.468019196451)*x[1]+(-0.0449222277683)*x[2]
        s[2,4,1,1]=(-0.337745267456)*x[0]+(-1.24573098041)*x[1]+(0.591411642381)*x[2]
        s[2,4,2,0]=(-0.773220236551)*x[0]+(-0.92934639777)*x[1]+(-0.323666905212)*x[2]
        s[2,4,2,1]=(-1.25416565442)*x[0]+(0.10804460369)*x[1]+(1.3398483009)*x[2]
        s[3,0,0,0]=(-0.187928573247)*x[0]+(0.952793598709)*x[1]+(-0.651544295118)*x[2]
        s[3,0,0,1]=(-1.18183645576)*x[0]+(-0.767480258477)*x[1]+(0.0733909569555)*x[2]
        s[3,0,1,0]=(0.825156828887)*x[0]+(0.900307840844)*x[1]+(-1.03153118659)*x[2]
        s[3,0,1,1]=(-0.150209523447)*x[0]+(1.07490578747)*x[1]+(-1.6916184583)*x[2]
        s[3,0,2,0]=(-1.07041023688)*x[0]+(-0.420347448824)*x[1]+(-0.556885268114)*x[2]
        s[3,0,2,1]=(-1.30609527635)*x[0]+(-0.10753520843)*x[1]+(0.240258152808)*x[2]
        s[3,1,0,0]=(0.899651818251)*x[0]+(0.00909698197696)*x[1]+(-1.64985946319)*x[2]
        s[3,1,0,1]=(-0.880471412731)*x[0]+(0.678893404261)*x[1]+(0.288497390335)*x[2]
        s[3,1,1,0]=(-0.00910924813519)*x[0]+(-1.30365942504)*x[1]+(-0.568919628746)*x[2]
        s[3,1,1,1]=(-0.811256892653)*x[0]+(-0.0284400402628)*x[1]+(0.727386097819)*x[2]
        s[3,1,2,0]=(0.105755186759)*x[0]+(0.196242692495)*x[1]+(0.358516596542)*x[2]
        s[3,1,2,1]=(-0.37985806916)*x[0]+(0.44810760494)*x[1]+(0.633807267038)*x[2]
        s[3,2,0,0]=(1.34921360449)*x[0]+(-0.9349137549)*x[1]+(-0.272245661092)*x[2]
        s[3,2,0,1]=(1.21020688092)*x[0]+(1.1296074258)*x[1]+(0.000391430715188)*x[2]
        s[3,2,1,0]=(-1.25769018387)*x[0]+(-0.786491629941)*x[1]+(-0.862105545664)*x[2]
        s[3,2,1,1]=(0.668529227552)*x[0]+(-0.983024344824)*x[1]+(0.74846424728)*x[2]
        s[3,2,2,0]=(0.45487084108)*x[0]+(0.855491649411)*x[1]+(-0.104621648479)*x[2]
        s[3,2,2,1]=(0.802451481866)*x[0]+(-0.137058520895)*x[1]+(0.573940225359)*x[2]
        s[3,3,0,0]=(0.468999526003)*x[0]+(0.504917415809)*x[1]+(-1.39187140958)*x[2]
        s[3,3,0,1]=(0.501007631838)*x[0]+(-0.285379909295)*x[1]+(0.257752560803)*x[2]
        s[3,3,1,0]=(-1.01423550219)*x[0]+(-0.749951834747)*x[1]+(-0.156550300135)*x[2]
        s[3,3,1,1]=(0.190954159364)*x[0]+(0.985690213898)*x[1]+(-0.942349242057)*x[2]
        s[3,3,2,0]=(-1.04776017317)*x[0]+(1.3978257599)*x[1]+(-0.727385312507)*x[2]
        s[3,3,2,1]=(0.118574662458)*x[0]+(0.763657521414)*x[1]+(-0.947854404873)*x[2]
        s[3,4,0,0]=(-1.42375158871)*x[0]+(1.02577770584)*x[1]+(1.7262667856)*x[2]
        s[3,4,0,1]=(0.819198798755)*x[0]+(-0.177472792322)*x[1]+(-0.495119104783)*x[2]
        s[3,4,1,0]=(-1.44070928104)*x[0]+(-0.67376984043)*x[1]+(0.314203511381)*x[2]
        s[3,4,1,1]=(-0.0417551422406)*x[0]+(-0.446166510311)*x[1]+(-0.303922120635)*x[2]
        s[3,4,2,0]=(0.161905825888)*x[0]+(-0.433001417416)*x[1]+(0.713602984094)*x[2]
        s[3,4,2,1]=(1.29579174225)*x[0]+(-0.136051917273)*x[1]+(0.270836816064)*x[2]
        ref[0,0,0,0]=(-0.678750626234)*x_ref[0]+(0.707898134345)*x_ref[1]+(1.47682355575)*x_ref[2]
        ref[0,0,0,1]=(-1.08665921875)*x_ref[0]+(-1.03746577329)*x_ref[1]+(0.459220112015)*x_ref[2]
        ref[0,0,1,0]=(-0.0924850687831)*x_ref[0]+(-1.21392495875)*x_ref[1]+(-0.335482829106)*x_ref[2]
        ref[0,0,1,1]=(-0.796044372195)*x_ref[0]+(-1.61600945946)*x_ref[1]+(-0.917678370431)*x_ref[2]
        ref[0,0,2,0]=(-1.17314148071)*x_ref[0]+(0.594053121363)*x_ref[1]+(-1.0751981685)*x_ref[2]
        ref[0,0,2,1]=(-0.993747266312)*x_ref[0]+(0.647546031964)*x_ref[1]+(0.789233440797)*x_ref[2]
        ref[0,1,0,0]=(-0.538620007403)*x_ref[0]+(0.572432466547)*x_ref[1]+(1.59224293356)*x_ref[2]
        ref[0,1,0,1]=(-0.315217413376)*x_ref[0]+(1.30492005409)*x_ref[1]+(-0.04944666559)*x_ref[2]
        ref[0,1,1,0]=(-0.734247343541)*x_ref[0]+(1.14104633611)*x_ref[1]+(-0.0424483055941)*x_ref[2]
        ref[0,1,1,1]=(-0.521727902651)*x_ref[0]+(-0.0302866863619)*x_ref[1]+(-0.0378416654735)*x_ref[2]
        ref[0,1,2,0]=(0.319532436452)*x_ref[0]+(0.312388869947)*x_ref[1]+(0.237025751349)*x_ref[2]
        ref[0,1,2,1]=(0.713884175591)*x_ref[0]+(1.10911840343)*x_ref[1]+(-1.4943567087)*x_ref[2]
        ref[0,2,0,0]=(-0.599254236697)*x_ref[0]+(-0.190373139957)*x_ref[1]+(-1.75168866737)*x_ref[2]
        ref[0,2,0,1]=(0.396782073165)*x_ref[0]+(-0.252470841512)*x_ref[1]+(-0.463651254287)*x_ref[2]
        ref[0,2,1,0]=(1.03549889185)*x_ref[0]+(0.225894922761)*x_ref[1]+(-0.609139538729)*x_ref[2]
        ref[0,2,1,1]=(0.289458873955)*x_ref[0]+(-0.136668874286)*x_ref[1]+(0.474827071819)*x_ref[2]
        ref[0,2,2,0]=(0.654253727057)*x_ref[0]+(-0.998748120229)*x_ref[1]+(-0.107718066457)*x_ref[2]
        ref[0,2,2,1]=(0.256710826049)*x_ref[0]+(0.309515575678)*x_ref[1]+(-1.60310813555)*x_ref[2]
        ref[0,3,0,0]=(1.70779242771)*x_ref[0]+(-0.530818399689)*x_ref[1]+(-0.516799319419)*x_ref[2]
        ref[0,3,0,1]=(-0.848868447677)*x_ref[0]+(-0.301212439564)*x_ref[1]+(-0.423135852525)*x_ref[2]
        ref[0,3,1,0]=(1.0807833809)*x_ref[0]+(-1.20726021059)*x_ref[1]+(0.565314004443)*x_ref[2]
        ref[0,3,1,1]=(-0.660234771003)*x_ref[0]+(-0.443966617085)*x_ref[1]+(-0.185381740077)*x_ref[2]
        ref[0,3,2,0]=(1.6260513125)*x_ref[0]+(0.145626479867)*x_ref[1]+(-0.808965403823)*x_ref[2]
        ref[0,3,2,1]=(-0.503772598493)*x_ref[0]+(-1.63975947666)*x_ref[1]+(0.255681416016)*x_ref[2]
        ref[0,4,0,0]=(0.0818777313252)*x_ref[0]+(0.428155907605)*x_ref[1]+(-0.642991998051)*x_ref[2]
        ref[0,4,0,1]=(-0.145032953266)*x_ref[0]+(-0.7645425853)*x_ref[1]+(0.458784780224)*x_ref[2]
        ref[0,4,1,0]=(-0.589583511871)*x_ref[0]+(-0.0743175116482)*x_ref[1]+(0.285113485271)*x_ref[2]
        ref[0,4,1,1]=(-1.37305892557)*x_ref[0]+(0.917822397515)*x_ref[1]+(-0.378127728174)*x_ref[2]
        ref[0,4,2,0]=(1.16505232525)*x_ref[0]+(1.10944900772)*x_ref[1]+(-0.95491955738)*x_ref[2]
        ref[0,4,2,1]=(-1.12539432931)*x_ref[0]+(-0.290265022179)*x_ref[1]+(-1.20809121853)*x_ref[2]
        ref[1,0,0,0]=(-0.859420732972)*x_ref[0]+(0.564058561557)*x_ref[1]+(-0.795155516002)*x_ref[2]
        ref[1,0,0,1]=(-0.314313522693)*x_ref[0]+(1.46275003784)*x_ref[1]+(0.285596389281)*x_ref[2]
        ref[1,0,1,0]=(-0.513360534744)*x_ref[0]+(-0.0532410726117)*x_ref[1]+(-0.122433325652)*x_ref[2]
        ref[1,0,1,1]=(0.527488940208)*x_ref[0]+(0.0760609834598)*x_ref[1]+(-0.612300815838)*x_ref[2]
        ref[1,0,2,0]=(0.302549971797)*x_ref[0]+(-0.710901255568)*x_ref[1]+(0.955062818656)*x_ref[2]
        ref[1,0,2,1]=(0.126520802001)*x_ref[0]+(-1.07564234283)*x_ref[1]+(-0.669526781403)*x_ref[2]
        ref[1,1,0,0]=(1.50804296428)*x_ref[0]+(0.111903566342)*x_ref[1]+(0.465876365517)*x_ref[2]
        ref[1,1,0,1]=(-1.04380726708)*x_ref[0]+(0.148245186406)*x_ref[1]+(-1.06801443458)*x_ref[2]
        ref[1,1,1,0]=(1.48236374583)*x_ref[0]+(-0.00794102905726)*x_ref[1]+(-1.35613430084)*x_ref[2]
        ref[1,1,1,1]=(1.04119256641)*x_ref[0]+(-1.84075261029)*x_ref[1]+(-0.776969168414)*x_ref[2]
        ref[1,1,2,0]=(-0.356539020393)*x_ref[0]+(1.70797570799)*x_ref[1]+(-0.525736126605)*x_ref[2]
        ref[1,1,2,1]=(1.62559049194)*x_ref[0]+(-0.482482804647)*x_ref[1]+(0.631790498783)*x_ref[2]
        ref[1,2,0,0]=(0.499315452031)*x_ref[0]+(-0.296700233772)*x_ref[1]+(1.00111627057)*x_ref[2]
        ref[1,2,0,1]=(1.0433119055)*x_ref[0]+(-0.269957745366)*x_ref[1]+(0.591153091169)*x_ref[2]
        ref[1,2,1,0]=(0.502068262492)*x_ref[0]+(-0.849037462803)*x_ref[1]+(-1.56692020458)*x_ref[2]
        ref[1,2,1,1]=(1.2984795003)*x_ref[0]+(-1.07653573134)*x_ref[1]+(0.487232329161)*x_ref[2]
        ref[1,2,2,0]=(-0.109333568443)*x_ref[0]+(0.687162555279)*x_ref[1]+(-0.812636431387)*x_ref[2]
        ref[1,2,2,1]=(0.202326014817)*x_ref[0]+(1.23297194202)*x_ref[1]+(1.41163217274)*x_ref[2]
        ref[1,3,0,0]=(0.124740742544)*x_ref[0]+(-0.500878894409)*x_ref[1]+(-0.376438702019)*x_ref[2]
        ref[1,3,0,1]=(-0.540234624582)*x_ref[0]+(-0.400577181452)*x_ref[1]+(0.879215681428)*x_ref[2]
        ref[1,3,1,0]=(0.0426835015338)*x_ref[0]+(-0.726976626459)*x_ref[1]+(1.32655296741)*x_ref[2]
        ref[1,3,1,1]=(-0.120969369352)*x_ref[0]+(-0.426853376816)*x_ref[1]+(1.88787704976)*x_ref[2]
        ref[1,3,2,0]=(-0.0895115836038)*x_ref[0]+(-1.17409640569)*x_ref[1]+(-0.874853748928)*x_ref[2]
        ref[1,3,2,1]=(-0.0664677570269)*x_ref[0]+(0.708204452159)*x_ref[1]+(-0.26397721498)*x_ref[2]
        ref[1,4,0,0]=(0.0236750184919)*x_ref[0]+(-1.42225228911)*x_ref[1]+(0.399502903376)*x_ref[2]
        ref[1,4,0,1]=(1.03400366274)*x_ref[0]+(-0.442018365874)*x_ref[1]+(1.02491120541)*x_ref[2]
        ref[1,4,1,0]=(-0.150528717616)*x_ref[0]+(-0.512236709632)*x_ref[1]+(-0.0644728802768)*x_ref[2]
        ref[1,4,1,1]=(-0.998520493595)*x_ref[0]+(-0.507010096027)*x_ref[1]+(-0.889168150742)*x_ref[2]
        ref[1,4,2,0]=(0.179165278401)*x_ref[0]+(1.07136931045)*x_ref[1]+(-0.790101000847)*x_ref[2]
        ref[1,4,2,1]=(0.376877479598)*x_ref[0]+(-0.25979387806)*x_ref[1]+(0.446682478544)*x_ref[2]
        ref[2,0,0,0]=(0.365668617405)*x_ref[0]+(1.44268125148)*x_ref[1]+(1.65785163405)*x_ref[2]
        ref[2,0,0,1]=(0.576180639993)*x_ref[0]+(-0.0571374651916)*x_ref[1]+(-0.0854307657892)*x_ref[2]
        ref[2,0,1,0]=(0.0474860699991)*x_ref[0]+(-1.47716457199)*x_ref[1]+(0.87300241949)*x_ref[2]
        ref[2,0,1,1]=(-0.293608884991)*x_ref[0]+(0.0963236497693)*x_ref[1]+(-1.31217901681)*x_ref[2]
        ref[2,0,2,0]=(-0.725125904522)*x_ref[0]+(0.15259525725)*x_ref[1]+(0.0624842579957)*x_ref[2]
        ref[2,0,2,1]=(-0.374667932076)*x_ref[0]+(-0.586704716252)*x_ref[1]+(-0.208259155061)*x_ref[2]
        ref[2,1,0,0]=(0.480010407761)*x_ref[0]+(-1.78856630141)*x_ref[1]+(0.296814589368)*x_ref[2]
        ref[2,1,0,1]=(-0.131871389585)*x_ref[0]+(0.0984653022698)*x_ref[1]+(0.222785695838)*x_ref[2]
        ref[2,1,1,0]=(-0.4095594348)*x_ref[0]+(-0.746059065385)*x_ref[1]+(1.72282248984)*x_ref[2]
        ref[2,1,1,1]=(0.841882742791)*x_ref[0]+(0.698803891138)*x_ref[1]+(0.201279240318)*x_ref[2]
        ref[2,1,2,0]=(-1.52503851146)*x_ref[0]+(0.746778574384)*x_ref[1]+(0.820127798751)*x_ref[2]
        ref[2,1,2,1]=(0.578616447592)*x_ref[0]+(-0.232884112626)*x_ref[1]+(0.719949134498)*x_ref[2]
        ref[2,2,0,0]=(-0.250869865455)*x_ref[0]+(0.0457838553118)*x_ref[1]+(0.694605224088)*x_ref[2]
        ref[2,2,0,1]=(-0.344703134894)*x_ref[0]+(-0.945296587028)*x_ref[1]+(-0.408991625518)*x_ref[2]
        ref[2,2,1,0]=(1.15842118069)*x_ref[0]+(0.385018100988)*x_ref[1]+(-0.298867420422)*x_ref[2]
        ref[2,2,1,1]=(-0.992212925836)*x_ref[0]+(-0.303921757453)*x_ref[1]+(1.90996417539)*x_ref[2]
        ref[2,2,2,0]=(0.139346086335)*x_ref[0]+(-1.5255000175)*x_ref[1]+(-0.418223280061)*x_ref[2]
        ref[2,2,2,1]=(-0.287165354844)*x_ref[0]+(-0.563470113789)*x_ref[1]+(-0.267078242542)*x_ref[2]
        ref[2,3,0,0]=(-0.877956931038)*x_ref[0]+(-0.587379404687)*x_ref[1]+(0.223772989101)*x_ref[2]
        ref[2,3,0,1]=(0.450240028907)*x_ref[0]+(0.933649504645)*x_ref[1]+(0.922982912622)*x_ref[2]
        ref[2,3,1,0]=(-0.695647633954)*x_ref[0]+(0.675123237641)*x_ref[1]+(0.333013288603)*x_ref[2]
        ref[2,3,1,1]=(-0.638319791165)*x_ref[0]+(-1.01905860587)*x_ref[1]+(-0.0829897514576)*x_ref[2]
        ref[2,3,2,0]=(-0.85507363839)*x_ref[0]+(0.213154292163)*x_ref[1]+(-1.19634907609)*x_ref[2]
        ref[2,3,2,1]=(0.51969848742)*x_ref[0]+(-0.103353752424)*x_ref[1]+(1.38100947757)*x_ref[2]
        ref[2,4,0,0]=(-1.42264362114)*x_ref[0]+(-0.0622861012095)*x_ref[1]+(1.11415860946)*x_ref[2]
        ref[2,4,0,1]=(1.03146522202)*x_ref[0]+(-0.59775537826)*x_ref[1]+(0.455451132416)*x_ref[2]
        ref[2,4,1,0]=(-1.42367400988)*x_ref[0]+(0.468019196451)*x_ref[1]+(-0.0449222277683)*x_ref[2]
        ref[2,4,1,1]=(-0.337745267456)*x_ref[0]+(-1.24573098041)*x_ref[1]+(0.591411642381)*x_ref[2]
        ref[2,4,2,0]=(-0.773220236551)*x_ref[0]+(-0.92934639777)*x_ref[1]+(-0.323666905212)*x_ref[2]
        ref[2,4,2,1]=(-1.25416565442)*x_ref[0]+(0.10804460369)*x_ref[1]+(1.3398483009)*x_ref[2]
        ref[3,0,0,0]=(-0.187928573247)*x_ref[0]+(0.952793598709)*x_ref[1]+(-0.651544295118)*x_ref[2]
        ref[3,0,0,1]=(-1.18183645576)*x_ref[0]+(-0.767480258477)*x_ref[1]+(0.0733909569555)*x_ref[2]
        ref[3,0,1,0]=(0.825156828887)*x_ref[0]+(0.900307840844)*x_ref[1]+(-1.03153118659)*x_ref[2]
        ref[3,0,1,1]=(-0.150209523447)*x_ref[0]+(1.07490578747)*x_ref[1]+(-1.6916184583)*x_ref[2]
        ref[3,0,2,0]=(-1.07041023688)*x_ref[0]+(-0.420347448824)*x_ref[1]+(-0.556885268114)*x_ref[2]
        ref[3,0,2,1]=(-1.30609527635)*x_ref[0]+(-0.10753520843)*x_ref[1]+(0.240258152808)*x_ref[2]
        ref[3,1,0,0]=(0.899651818251)*x_ref[0]+(0.00909698197696)*x_ref[1]+(-1.64985946319)*x_ref[2]
        ref[3,1,0,1]=(-0.880471412731)*x_ref[0]+(0.678893404261)*x_ref[1]+(0.288497390335)*x_ref[2]
        ref[3,1,1,0]=(-0.00910924813519)*x_ref[0]+(-1.30365942504)*x_ref[1]+(-0.568919628746)*x_ref[2]
        ref[3,1,1,1]=(-0.811256892653)*x_ref[0]+(-0.0284400402628)*x_ref[1]+(0.727386097819)*x_ref[2]
        ref[3,1,2,0]=(0.105755186759)*x_ref[0]+(0.196242692495)*x_ref[1]+(0.358516596542)*x_ref[2]
        ref[3,1,2,1]=(-0.37985806916)*x_ref[0]+(0.44810760494)*x_ref[1]+(0.633807267038)*x_ref[2]
        ref[3,2,0,0]=(1.34921360449)*x_ref[0]+(-0.9349137549)*x_ref[1]+(-0.272245661092)*x_ref[2]
        ref[3,2,0,1]=(1.21020688092)*x_ref[0]+(1.1296074258)*x_ref[1]+(0.000391430715188)*x_ref[2]
        ref[3,2,1,0]=(-1.25769018387)*x_ref[0]+(-0.786491629941)*x_ref[1]+(-0.862105545664)*x_ref[2]
        ref[3,2,1,1]=(0.668529227552)*x_ref[0]+(-0.983024344824)*x_ref[1]+(0.74846424728)*x_ref[2]
        ref[3,2,2,0]=(0.45487084108)*x_ref[0]+(0.855491649411)*x_ref[1]+(-0.104621648479)*x_ref[2]
        ref[3,2,2,1]=(0.802451481866)*x_ref[0]+(-0.137058520895)*x_ref[1]+(0.573940225359)*x_ref[2]
        ref[3,3,0,0]=(0.468999526003)*x_ref[0]+(0.504917415809)*x_ref[1]+(-1.39187140958)*x_ref[2]
        ref[3,3,0,1]=(0.501007631838)*x_ref[0]+(-0.285379909295)*x_ref[1]+(0.257752560803)*x_ref[2]
        ref[3,3,1,0]=(-1.01423550219)*x_ref[0]+(-0.749951834747)*x_ref[1]+(-0.156550300135)*x_ref[2]
        ref[3,3,1,1]=(0.190954159364)*x_ref[0]+(0.985690213898)*x_ref[1]+(-0.942349242057)*x_ref[2]
        ref[3,3,2,0]=(-1.04776017317)*x_ref[0]+(1.3978257599)*x_ref[1]+(-0.727385312507)*x_ref[2]
        ref[3,3,2,1]=(0.118574662458)*x_ref[0]+(0.763657521414)*x_ref[1]+(-0.947854404873)*x_ref[2]
        ref[3,4,0,0]=(-1.42375158871)*x_ref[0]+(1.02577770584)*x_ref[1]+(1.7262667856)*x_ref[2]
        ref[3,4,0,1]=(0.819198798755)*x_ref[0]+(-0.177472792322)*x_ref[1]+(-0.495119104783)*x_ref[2]
        ref[3,4,1,0]=(-1.44070928104)*x_ref[0]+(-0.67376984043)*x_ref[1]+(0.314203511381)*x_ref[2]
        ref[3,4,1,1]=(-0.0417551422406)*x_ref[0]+(-0.446166510311)*x_ref[1]+(-0.303922120635)*x_ref[2]
        ref[3,4,2,0]=(0.161905825888)*x_ref[0]+(-0.433001417416)*x_ref[1]+(0.713602984094)*x_ref[2]
        ref[3,4,2,1]=(1.29579174225)*x_ref[0]+(-0.136051917273)*x_ref[1]+(0.270836816064)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromSymbol_ReducedFunctionOnContactZero_rank0(self):
      """
      tests interpolate for rank 0 Symbol onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactZero onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactZero(self.domain)
      x=w.getX()
      arg=Symbol(shape=(),dim=dim)
      s=Data(0,(),w)
      ref=Data(0,(),w_ref)
      if dim==2:
        s=(0.52559525257)*x[0]**o+(0.229391539263)*x[0]+(0.948594885998)*x[1]**o+(0.247647100433)*x[1]
        ref=(0.52559525257)*x_ref[0]**o+(0.229391539263)*x_ref[0]+(0.948594885998)*x_ref[1]**o+(0.247647100433)*x_ref[1]
      else:
        s=(-0.386040522546)*x[0]**o+(0.390865081371)*x[0]+(-0.194909055558)*x[1]**o+(0.890024298101)*x[1]+(-0.660390723258)*x[2]**o+(-0.191783645278)*x[2]
        ref=(-0.386040522546)*x_ref[0]**o+(0.390865081371)*x_ref[0]+(-0.194909055558)*x_ref[1]**o+(0.890024298101)*x_ref[1]+(-0.660390723258)*x_ref[2]**o+(-0.191783645278)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromSymbol_ReducedFunctionOnContactZero_rank1(self):
      """
      tests interpolate for rank 1 Symbol onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactZero onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactZero(self.domain)
      x=w.getX()
      arg=Symbol(shape=(2,),dim=dim)
      s=Data(0,(2,),w)
      ref=Data(0,(2,),w_ref)
      if dim==2:
        s[0]=(0.886118109323)*x[0]**o+(-0.430472191748)*x[0]+(0.188986990863)*x[1]**o+(0.123299635287)*x[1]
        s[1]=(-0.406991760476)*x[0]**o+(0.940756190869)*x[0]+(-0.926329857206)*x[1]**o+(-0.752111183385)*x[1]
        ref[0]=(0.886118109323)*x_ref[0]**o+(-0.430472191748)*x_ref[0]+(0.188986990863)*x_ref[1]**o+(0.123299635287)*x_ref[1]
        ref[1]=(-0.406991760476)*x_ref[0]**o+(0.940756190869)*x_ref[0]+(-0.926329857206)*x_ref[1]**o+(-0.752111183385)*x_ref[1]
      else:
        s[0]=(-0.888491776167)*x[0]**o+(0.568362216099)*x[0]+(0.92434143189)*x[1]**o+(0.686002203387)*x[1]+(0.323992600729)*x[2]**o+(-0.150051242182)*x[2]
        s[1]=(0.286049958958)*x[0]**o+(-0.947739219713)*x[0]+(-0.216646074031)*x[1]**o+(0.67646631851)*x[1]+(0.0591555372338)*x[2]**o+(0.949863355854)*x[2]
        ref[0]=(-0.888491776167)*x_ref[0]**o+(0.568362216099)*x_ref[0]+(0.92434143189)*x_ref[1]**o+(0.686002203387)*x_ref[1]+(0.323992600729)*x_ref[2]**o+(-0.150051242182)*x_ref[2]
        ref[1]=(0.286049958958)*x_ref[0]**o+(-0.947739219713)*x_ref[0]+(-0.216646074031)*x_ref[1]**o+(0.67646631851)*x_ref[1]+(0.0591555372338)*x_ref[2]**o+(0.949863355854)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromSymbol_ReducedFunctionOnContactZero_rank2(self):
      """
      tests interpolate for rank 2 Symbol onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactZero onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactZero(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5),dim=dim)
      s=Data(0,(4, 5),w)
      ref=Data(0,(4, 5),w_ref)
      if dim==2:
        s[0,0]=(0.777675758098)*x[0]**o+(-0.473156188055)*x[0]+(-0.83738893374)*x[1]**o+(-0.772375433518)*x[1]
        s[0,1]=(0.690632604654)*x[0]**o+(-0.596947428521)*x[0]+(-0.927206976637)*x[1]**o+(0.416133449904)*x[1]
        s[0,2]=(0.159353873068)*x[0]**o+(-0.544617444157)*x[0]+(-0.384185707939)*x[1]**o+(0.630137005341)*x[1]
        s[0,3]=(-0.677069489949)*x[0]**o+(0.121886164808)*x[0]+(-0.243464410455)*x[1]**o+(-0.424481834693)*x[1]
        s[0,4]=(-0.0586502823616)*x[0]**o+(-0.952781957131)*x[0]+(-0.638027009478)*x[1]**o+(-0.939836467104)*x[1]
        s[1,0]=(0.946604826689)*x[0]**o+(-0.344506753444)*x[0]+(-0.366256116023)*x[1]**o+(0.66571445335)*x[1]
        s[1,1]=(-0.148616405902)*x[0]**o+(0.24049619296)*x[0]+(-0.59987968565)*x[1]**o+(-0.30617564854)*x[1]
        s[1,2]=(-0.0215096239827)*x[0]**o+(0.92491389564)*x[0]+(0.820884930171)*x[1]**o+(0.907170170451)*x[1]
        s[1,3]=(0.151439258285)*x[0]**o+(-0.619065403082)*x[0]+(-0.931701693869)*x[1]**o+(-0.636007349209)*x[1]
        s[1,4]=(0.599937864144)*x[0]**o+(0.856401446192)*x[0]+(0.0498004863454)*x[1]**o+(-0.542473731625)*x[1]
        s[2,0]=(0.114304895833)*x[0]**o+(-0.688107927554)*x[0]+(0.391000230103)*x[1]**o+(-0.406487512048)*x[1]
        s[2,1]=(-0.745374769533)*x[0]**o+(0.797317533966)*x[0]+(-0.667327745105)*x[1]**o+(-0.356428295255)*x[1]
        s[2,2]=(-0.381300049096)*x[0]**o+(-0.187744971624)*x[0]+(0.361881540737)*x[1]**o+(-0.116712686284)*x[1]
        s[2,3]=(-0.156007331641)*x[0]**o+(0.887169440804)*x[0]+(-0.324903456097)*x[1]**o+(0.360748003763)*x[1]
        s[2,4]=(0.712524916579)*x[0]**o+(-0.638314673897)*x[0]+(-0.0308273014936)*x[1]**o+(-0.602816096773)*x[1]
        s[3,0]=(0.269003703906)*x[0]**o+(-0.648611537709)*x[0]+(0.0450380502202)*x[1]**o+(0.261193188134)*x[1]
        s[3,1]=(0.84586203363)*x[0]**o+(0.698542521798)*x[0]+(0.827757021664)*x[1]**o+(-0.297626737453)*x[1]
        s[3,2]=(-0.346844241342)*x[0]**o+(-0.609694742721)*x[0]+(-0.34363112844)*x[1]**o+(0.898190611922)*x[1]
        s[3,3]=(-0.150818980857)*x[0]**o+(-0.0555804912754)*x[0]+(0.543245030743)*x[1]**o+(0.862313486187)*x[1]
        s[3,4]=(0.191954299926)*x[0]**o+(0.218021623666)*x[0]+(-0.354607040653)*x[1]**o+(-0.382222211827)*x[1]
        ref[0,0]=(0.777675758098)*x_ref[0]**o+(-0.473156188055)*x_ref[0]+(-0.83738893374)*x_ref[1]**o+(-0.772375433518)*x_ref[1]
        ref[0,1]=(0.690632604654)*x_ref[0]**o+(-0.596947428521)*x_ref[0]+(-0.927206976637)*x_ref[1]**o+(0.416133449904)*x_ref[1]
        ref[0,2]=(0.159353873068)*x_ref[0]**o+(-0.544617444157)*x_ref[0]+(-0.384185707939)*x_ref[1]**o+(0.630137005341)*x_ref[1]
        ref[0,3]=(-0.677069489949)*x_ref[0]**o+(0.121886164808)*x_ref[0]+(-0.243464410455)*x_ref[1]**o+(-0.424481834693)*x_ref[1]
        ref[0,4]=(-0.0586502823616)*x_ref[0]**o+(-0.952781957131)*x_ref[0]+(-0.638027009478)*x_ref[1]**o+(-0.939836467104)*x_ref[1]
        ref[1,0]=(0.946604826689)*x_ref[0]**o+(-0.344506753444)*x_ref[0]+(-0.366256116023)*x_ref[1]**o+(0.66571445335)*x_ref[1]
        ref[1,1]=(-0.148616405902)*x_ref[0]**o+(0.24049619296)*x_ref[0]+(-0.59987968565)*x_ref[1]**o+(-0.30617564854)*x_ref[1]
        ref[1,2]=(-0.0215096239827)*x_ref[0]**o+(0.92491389564)*x_ref[0]+(0.820884930171)*x_ref[1]**o+(0.907170170451)*x_ref[1]
        ref[1,3]=(0.151439258285)*x_ref[0]**o+(-0.619065403082)*x_ref[0]+(-0.931701693869)*x_ref[1]**o+(-0.636007349209)*x_ref[1]
        ref[1,4]=(0.599937864144)*x_ref[0]**o+(0.856401446192)*x_ref[0]+(0.0498004863454)*x_ref[1]**o+(-0.542473731625)*x_ref[1]
        ref[2,0]=(0.114304895833)*x_ref[0]**o+(-0.688107927554)*x_ref[0]+(0.391000230103)*x_ref[1]**o+(-0.406487512048)*x_ref[1]
        ref[2,1]=(-0.745374769533)*x_ref[0]**o+(0.797317533966)*x_ref[0]+(-0.667327745105)*x_ref[1]**o+(-0.356428295255)*x_ref[1]
        ref[2,2]=(-0.381300049096)*x_ref[0]**o+(-0.187744971624)*x_ref[0]+(0.361881540737)*x_ref[1]**o+(-0.116712686284)*x_ref[1]
        ref[2,3]=(-0.156007331641)*x_ref[0]**o+(0.887169440804)*x_ref[0]+(-0.324903456097)*x_ref[1]**o+(0.360748003763)*x_ref[1]
        ref[2,4]=(0.712524916579)*x_ref[0]**o+(-0.638314673897)*x_ref[0]+(-0.0308273014936)*x_ref[1]**o+(-0.602816096773)*x_ref[1]
        ref[3,0]=(0.269003703906)*x_ref[0]**o+(-0.648611537709)*x_ref[0]+(0.0450380502202)*x_ref[1]**o+(0.261193188134)*x_ref[1]
        ref[3,1]=(0.84586203363)*x_ref[0]**o+(0.698542521798)*x_ref[0]+(0.827757021664)*x_ref[1]**o+(-0.297626737453)*x_ref[1]
        ref[3,2]=(-0.346844241342)*x_ref[0]**o+(-0.609694742721)*x_ref[0]+(-0.34363112844)*x_ref[1]**o+(0.898190611922)*x_ref[1]
        ref[3,3]=(-0.150818980857)*x_ref[0]**o+(-0.0555804912754)*x_ref[0]+(0.543245030743)*x_ref[1]**o+(0.862313486187)*x_ref[1]
        ref[3,4]=(0.191954299926)*x_ref[0]**o+(0.218021623666)*x_ref[0]+(-0.354607040653)*x_ref[1]**o+(-0.382222211827)*x_ref[1]
      else:
        s[0,0]=(0.264155606808)*x[0]**o+(0.865349854552)*x[0]+(0.290185544602)*x[1]**o+(-0.881187362254)*x[1]+(0.0974065632758)*x[2]**o+(-0.0146386375323)*x[2]
        s[0,1]=(-0.553039444966)*x[0]**o+(-0.225032077709)*x[0]+(-0.114709392843)*x[1]**o+(0.476316128194)*x[1]+(-0.157256007297)*x[2]**o+(-0.881306470969)*x[2]
        s[0,2]=(0.204050245804)*x[0]**o+(0.843162287468)*x[0]+(0.479364138104)*x[1]**o+(0.584277416031)*x[1]+(-0.728022519952)*x[2]**o+(0.455950432012)*x[2]
        s[0,3]=(0.337825383533)*x[0]**o+(-0.727944018401)*x[0]+(-0.884967739733)*x[1]**o+(0.0927602193719)*x[1]+(0.693987105436)*x[2]**o+(-0.212308959739)*x[2]
        s[0,4]=(-0.663473315009)*x[0]**o+(-0.534399657707)*x[0]+(0.00834675143069)*x[1]**o+(-0.323653373064)*x[1]+(0.138014603409)*x[2]**o+(-0.833018982749)*x[2]
        s[1,0]=(0.18898839339)*x[0]**o+(-0.0348685084852)*x[0]+(0.822229497116)*x[1]**o+(0.512525434163)*x[1]+(-0.136217897358)*x[2]**o+(0.503370342407)*x[2]
        s[1,1]=(-0.628784998358)*x[0]**o+(-0.253509454631)*x[0]+(-0.192305899603)*x[1]**o+(-0.415871165125)*x[1]+(0.709676594594)*x[2]**o+(0.331428469664)*x[2]
        s[1,2]=(0.459380888002)*x[0]**o+(-0.0693389935675)*x[0]+(-0.853473419272)*x[1]**o+(-0.221441495424)*x[1]+(0.649741007162)*x[2]**o+(0.138511312385)*x[2]
        s[1,3]=(-0.130673271522)*x[0]**o+(-0.273321361436)*x[0]+(-0.799741846799)*x[1]**o+(-0.950206189845)*x[1]+(-0.502011070117)*x[2]**o+(0.202315792805)*x[2]
        s[1,4]=(-0.131491315255)*x[0]**o+(0.814802856108)*x[0]+(0.12368780731)*x[1]**o+(-0.184462314547)*x[1]+(0.430222618358)*x[2]**o+(-0.74005756053)*x[2]
        s[2,0]=(-0.713274359703)*x[0]**o+(-0.0251491873794)*x[0]+(-0.622142303525)*x[1]**o+(0.933649343169)*x[1]+(0.312383777738)*x[2]**o+(0.533383231604)*x[2]
        s[2,1]=(0.311750294112)*x[0]**o+(0.144605685448)*x[0]+(-0.980716557512)*x[1]**o+(0.575451487258)*x[1]+(-0.0552788567322)*x[2]**o+(0.416492300045)*x[2]
        s[2,2]=(-0.543301125737)*x[0]**o+(0.788016744944)*x[0]+(0.244619454713)*x[1]**o+(0.570164703338)*x[1]+(-0.448168767253)*x[2]**o+(-0.346442604379)*x[2]
        s[2,3]=(0.731364271175)*x[0]**o+(-0.787549613302)*x[0]+(0.119788530606)*x[1]**o+(0.287282876919)*x[1]+(-0.235292928704)*x[2]**o+(-0.953636574666)*x[2]
        s[2,4]=(0.449796191117)*x[0]**o+(-0.648747759536)*x[0]+(0.601089847127)*x[1]**o+(0.593778367156)*x[1]+(-0.800274848868)*x[2]**o+(-0.576472670212)*x[2]
        s[3,0]=(0.951920248404)*x[0]**o+(-0.42219089138)*x[0]+(0.728667162964)*x[1]**o+(-0.882554153116)*x[1]+(-0.820351735649)*x[2]**o+(0.169030492382)*x[2]
        s[3,1]=(-0.497772815911)*x[0]**o+(0.798040706328)*x[0]+(-0.197945158571)*x[1]**o+(0.974647527948)*x[1]+(0.903944658321)*x[2]**o+(-0.249832296182)*x[2]
        s[3,2]=(0.98668376271)*x[0]**o+(-0.102390184426)*x[0]+(0.70946055219)*x[1]**o+(-0.493091573905)*x[1]+(-0.778257510928)*x[2]**o+(-0.413832616402)*x[2]
        s[3,3]=(0.62445606614)*x[0]**o+(-0.0898301047789)*x[0]+(0.788538530992)*x[1]**o+(0.702286686509)*x[1]+(-0.487082450751)*x[2]**o+(0.875348478204)*x[2]
        s[3,4]=(0.957689119464)*x[0]**o+(-0.855933895119)*x[0]+(0.119002293838)*x[1]**o+(-0.00826726591134)*x[1]+(0.145310022693)*x[2]**o+(0.842408432594)*x[2]
        ref[0,0]=(0.264155606808)*x_ref[0]**o+(0.865349854552)*x_ref[0]+(0.290185544602)*x_ref[1]**o+(-0.881187362254)*x_ref[1]+(0.0974065632758)*x_ref[2]**o+(-0.0146386375323)*x_ref[2]
        ref[0,1]=(-0.553039444966)*x_ref[0]**o+(-0.225032077709)*x_ref[0]+(-0.114709392843)*x_ref[1]**o+(0.476316128194)*x_ref[1]+(-0.157256007297)*x_ref[2]**o+(-0.881306470969)*x_ref[2]
        ref[0,2]=(0.204050245804)*x_ref[0]**o+(0.843162287468)*x_ref[0]+(0.479364138104)*x_ref[1]**o+(0.584277416031)*x_ref[1]+(-0.728022519952)*x_ref[2]**o+(0.455950432012)*x_ref[2]
        ref[0,3]=(0.337825383533)*x_ref[0]**o+(-0.727944018401)*x_ref[0]+(-0.884967739733)*x_ref[1]**o+(0.0927602193719)*x_ref[1]+(0.693987105436)*x_ref[2]**o+(-0.212308959739)*x_ref[2]
        ref[0,4]=(-0.663473315009)*x_ref[0]**o+(-0.534399657707)*x_ref[0]+(0.00834675143069)*x_ref[1]**o+(-0.323653373064)*x_ref[1]+(0.138014603409)*x_ref[2]**o+(-0.833018982749)*x_ref[2]
        ref[1,0]=(0.18898839339)*x_ref[0]**o+(-0.0348685084852)*x_ref[0]+(0.822229497116)*x_ref[1]**o+(0.512525434163)*x_ref[1]+(-0.136217897358)*x_ref[2]**o+(0.503370342407)*x_ref[2]
        ref[1,1]=(-0.628784998358)*x_ref[0]**o+(-0.253509454631)*x_ref[0]+(-0.192305899603)*x_ref[1]**o+(-0.415871165125)*x_ref[1]+(0.709676594594)*x_ref[2]**o+(0.331428469664)*x_ref[2]
        ref[1,2]=(0.459380888002)*x_ref[0]**o+(-0.0693389935675)*x_ref[0]+(-0.853473419272)*x_ref[1]**o+(-0.221441495424)*x_ref[1]+(0.649741007162)*x_ref[2]**o+(0.138511312385)*x_ref[2]
        ref[1,3]=(-0.130673271522)*x_ref[0]**o+(-0.273321361436)*x_ref[0]+(-0.799741846799)*x_ref[1]**o+(-0.950206189845)*x_ref[1]+(-0.502011070117)*x_ref[2]**o+(0.202315792805)*x_ref[2]
        ref[1,4]=(-0.131491315255)*x_ref[0]**o+(0.814802856108)*x_ref[0]+(0.12368780731)*x_ref[1]**o+(-0.184462314547)*x_ref[1]+(0.430222618358)*x_ref[2]**o+(-0.74005756053)*x_ref[2]
        ref[2,0]=(-0.713274359703)*x_ref[0]**o+(-0.0251491873794)*x_ref[0]+(-0.622142303525)*x_ref[1]**o+(0.933649343169)*x_ref[1]+(0.312383777738)*x_ref[2]**o+(0.533383231604)*x_ref[2]
        ref[2,1]=(0.311750294112)*x_ref[0]**o+(0.144605685448)*x_ref[0]+(-0.980716557512)*x_ref[1]**o+(0.575451487258)*x_ref[1]+(-0.0552788567322)*x_ref[2]**o+(0.416492300045)*x_ref[2]
        ref[2,2]=(-0.543301125737)*x_ref[0]**o+(0.788016744944)*x_ref[0]+(0.244619454713)*x_ref[1]**o+(0.570164703338)*x_ref[1]+(-0.448168767253)*x_ref[2]**o+(-0.346442604379)*x_ref[2]
        ref[2,3]=(0.731364271175)*x_ref[0]**o+(-0.787549613302)*x_ref[0]+(0.119788530606)*x_ref[1]**o+(0.287282876919)*x_ref[1]+(-0.235292928704)*x_ref[2]**o+(-0.953636574666)*x_ref[2]
        ref[2,4]=(0.449796191117)*x_ref[0]**o+(-0.648747759536)*x_ref[0]+(0.601089847127)*x_ref[1]**o+(0.593778367156)*x_ref[1]+(-0.800274848868)*x_ref[2]**o+(-0.576472670212)*x_ref[2]
        ref[3,0]=(0.951920248404)*x_ref[0]**o+(-0.42219089138)*x_ref[0]+(0.728667162964)*x_ref[1]**o+(-0.882554153116)*x_ref[1]+(-0.820351735649)*x_ref[2]**o+(0.169030492382)*x_ref[2]
        ref[3,1]=(-0.497772815911)*x_ref[0]**o+(0.798040706328)*x_ref[0]+(-0.197945158571)*x_ref[1]**o+(0.974647527948)*x_ref[1]+(0.903944658321)*x_ref[2]**o+(-0.249832296182)*x_ref[2]
        ref[3,2]=(0.98668376271)*x_ref[0]**o+(-0.102390184426)*x_ref[0]+(0.70946055219)*x_ref[1]**o+(-0.493091573905)*x_ref[1]+(-0.778257510928)*x_ref[2]**o+(-0.413832616402)*x_ref[2]
        ref[3,3]=(0.62445606614)*x_ref[0]**o+(-0.0898301047789)*x_ref[0]+(0.788538530992)*x_ref[1]**o+(0.702286686509)*x_ref[1]+(-0.487082450751)*x_ref[2]**o+(0.875348478204)*x_ref[2]
        ref[3,4]=(0.957689119464)*x_ref[0]**o+(-0.855933895119)*x_ref[0]+(0.119002293838)*x_ref[1]**o+(-0.00826726591134)*x_ref[1]+(0.145310022693)*x_ref[2]**o+(0.842408432594)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromSymbol_ReducedFunctionOnContactZero_rank3(self):
      """
      tests interpolate for rank 3 Symbol onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactZero onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactZero(self.domain)
      x=w.getX()
      arg=Symbol(shape=(6, 2, 2),dim=dim)
      s=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2),w_ref)
      if dim==2:
        s[0,0,0]=(-0.712152806182)*x[0]**o+(0.553286790231)*x[0]+(-0.725363109396)*x[1]**o+(-0.788986615699)*x[1]
        s[0,0,1]=(0.0651260964364)*x[0]**o+(0.846234983166)*x[0]+(0.220346387636)*x[1]**o+(-0.169715997542)*x[1]
        s[0,1,0]=(-0.312770289544)*x[0]**o+(-0.830517475738)*x[0]+(0.532769330513)*x[1]**o+(-0.502035152634)*x[1]
        s[0,1,1]=(-0.268414866022)*x[0]**o+(0.653788083346)*x[0]+(0.845119607511)*x[1]**o+(0.928640589437)*x[1]
        s[1,0,0]=(0.0174302886823)*x[0]**o+(0.567027146136)*x[0]+(0.658004848684)*x[1]**o+(0.238897231665)*x[1]
        s[1,0,1]=(0.0275230087777)*x[0]**o+(-0.909065705806)*x[0]+(0.616873751656)*x[1]**o+(-0.750647747368)*x[1]
        s[1,1,0]=(0.968790737085)*x[0]**o+(-0.717354080994)*x[0]+(0.322354780033)*x[1]**o+(0.297005507117)*x[1]
        s[1,1,1]=(0.260782453626)*x[0]**o+(-0.294785381256)*x[0]+(0.249751113945)*x[1]**o+(-0.334065019247)*x[1]
        s[2,0,0]=(-0.443277964789)*x[0]**o+(-0.951540439846)*x[0]+(0.714190838936)*x[1]**o+(0.745547080519)*x[1]
        s[2,0,1]=(0.835967278262)*x[0]**o+(0.849721181077)*x[0]+(-0.45248294099)*x[1]**o+(-0.91527746844)*x[1]
        s[2,1,0]=(-0.550753310799)*x[0]**o+(0.0991213939269)*x[0]+(-0.990618565235)*x[1]**o+(-0.101473335456)*x[1]
        s[2,1,1]=(-0.992001639983)*x[0]**o+(0.582614881862)*x[0]+(0.649744791891)*x[1]**o+(0.910908482017)*x[1]
        s[3,0,0]=(-0.615118121008)*x[0]**o+(0.908419887637)*x[0]+(0.764202178909)*x[1]**o+(-0.10825645538)*x[1]
        s[3,0,1]=(0.494014349582)*x[0]**o+(0.340651430733)*x[0]+(0.837654928126)*x[1]**o+(0.118115058337)*x[1]
        s[3,1,0]=(0.800835589492)*x[0]**o+(0.358498940072)*x[0]+(0.347138646821)*x[1]**o+(-0.760451735532)*x[1]
        s[3,1,1]=(0.221808818118)*x[0]**o+(0.76998621836)*x[0]+(-0.184775740422)*x[1]**o+(0.891454523108)*x[1]
        s[4,0,0]=(-0.552924785414)*x[0]**o+(0.00903957955798)*x[0]+(-0.0176781743627)*x[1]**o+(-0.788145401649)*x[1]
        s[4,0,1]=(0.0557272322524)*x[0]**o+(0.37477815689)*x[0]+(0.920309703574)*x[1]**o+(0.478950854112)*x[1]
        s[4,1,0]=(0.282037005502)*x[0]**o+(-0.42297688723)*x[0]+(0.544702991352)*x[1]**o+(0.418508536472)*x[1]
        s[4,1,1]=(-0.0983696927616)*x[0]**o+(-0.814684556832)*x[0]+(0.291195735479)*x[1]**o+(-0.840465495063)*x[1]
        s[5,0,0]=(-0.653946625568)*x[0]**o+(-0.419703492576)*x[0]+(-0.00132821148354)*x[1]**o+(0.845340315129)*x[1]
        s[5,0,1]=(0.389254602951)*x[0]**o+(0.269315721817)*x[0]+(-0.666779865472)*x[1]**o+(-0.721924390904)*x[1]
        s[5,1,0]=(0.0273782807939)*x[0]**o+(-0.629004324686)*x[0]+(0.469241132749)*x[1]**o+(0.101031019558)*x[1]
        s[5,1,1]=(-0.642652252134)*x[0]**o+(0.055584656243)*x[0]+(0.591952970815)*x[1]**o+(0.0560172089007)*x[1]
        ref[0,0,0]=(-0.712152806182)*x_ref[0]**o+(0.553286790231)*x_ref[0]+(-0.725363109396)*x_ref[1]**o+(-0.788986615699)*x_ref[1]
        ref[0,0,1]=(0.0651260964364)*x_ref[0]**o+(0.846234983166)*x_ref[0]+(0.220346387636)*x_ref[1]**o+(-0.169715997542)*x_ref[1]
        ref[0,1,0]=(-0.312770289544)*x_ref[0]**o+(-0.830517475738)*x_ref[0]+(0.532769330513)*x_ref[1]**o+(-0.502035152634)*x_ref[1]
        ref[0,1,1]=(-0.268414866022)*x_ref[0]**o+(0.653788083346)*x_ref[0]+(0.845119607511)*x_ref[1]**o+(0.928640589437)*x_ref[1]
        ref[1,0,0]=(0.0174302886823)*x_ref[0]**o+(0.567027146136)*x_ref[0]+(0.658004848684)*x_ref[1]**o+(0.238897231665)*x_ref[1]
        ref[1,0,1]=(0.0275230087777)*x_ref[0]**o+(-0.909065705806)*x_ref[0]+(0.616873751656)*x_ref[1]**o+(-0.750647747368)*x_ref[1]
        ref[1,1,0]=(0.968790737085)*x_ref[0]**o+(-0.717354080994)*x_ref[0]+(0.322354780033)*x_ref[1]**o+(0.297005507117)*x_ref[1]
        ref[1,1,1]=(0.260782453626)*x_ref[0]**o+(-0.294785381256)*x_ref[0]+(0.249751113945)*x_ref[1]**o+(-0.334065019247)*x_ref[1]
        ref[2,0,0]=(-0.443277964789)*x_ref[0]**o+(-0.951540439846)*x_ref[0]+(0.714190838936)*x_ref[1]**o+(0.745547080519)*x_ref[1]
        ref[2,0,1]=(0.835967278262)*x_ref[0]**o+(0.849721181077)*x_ref[0]+(-0.45248294099)*x_ref[1]**o+(-0.91527746844)*x_ref[1]
        ref[2,1,0]=(-0.550753310799)*x_ref[0]**o+(0.0991213939269)*x_ref[0]+(-0.990618565235)*x_ref[1]**o+(-0.101473335456)*x_ref[1]
        ref[2,1,1]=(-0.992001639983)*x_ref[0]**o+(0.582614881862)*x_ref[0]+(0.649744791891)*x_ref[1]**o+(0.910908482017)*x_ref[1]
        ref[3,0,0]=(-0.615118121008)*x_ref[0]**o+(0.908419887637)*x_ref[0]+(0.764202178909)*x_ref[1]**o+(-0.10825645538)*x_ref[1]
        ref[3,0,1]=(0.494014349582)*x_ref[0]**o+(0.340651430733)*x_ref[0]+(0.837654928126)*x_ref[1]**o+(0.118115058337)*x_ref[1]
        ref[3,1,0]=(0.800835589492)*x_ref[0]**o+(0.358498940072)*x_ref[0]+(0.347138646821)*x_ref[1]**o+(-0.760451735532)*x_ref[1]
        ref[3,1,1]=(0.221808818118)*x_ref[0]**o+(0.76998621836)*x_ref[0]+(-0.184775740422)*x_ref[1]**o+(0.891454523108)*x_ref[1]
        ref[4,0,0]=(-0.552924785414)*x_ref[0]**o+(0.00903957955798)*x_ref[0]+(-0.0176781743627)*x_ref[1]**o+(-0.788145401649)*x_ref[1]
        ref[4,0,1]=(0.0557272322524)*x_ref[0]**o+(0.37477815689)*x_ref[0]+(0.920309703574)*x_ref[1]**o+(0.478950854112)*x_ref[1]
        ref[4,1,0]=(0.282037005502)*x_ref[0]**o+(-0.42297688723)*x_ref[0]+(0.544702991352)*x_ref[1]**o+(0.418508536472)*x_ref[1]
        ref[4,1,1]=(-0.0983696927616)*x_ref[0]**o+(-0.814684556832)*x_ref[0]+(0.291195735479)*x_ref[1]**o+(-0.840465495063)*x_ref[1]
        ref[5,0,0]=(-0.653946625568)*x_ref[0]**o+(-0.419703492576)*x_ref[0]+(-0.00132821148354)*x_ref[1]**o+(0.845340315129)*x_ref[1]
        ref[5,0,1]=(0.389254602951)*x_ref[0]**o+(0.269315721817)*x_ref[0]+(-0.666779865472)*x_ref[1]**o+(-0.721924390904)*x_ref[1]
        ref[5,1,0]=(0.0273782807939)*x_ref[0]**o+(-0.629004324686)*x_ref[0]+(0.469241132749)*x_ref[1]**o+(0.101031019558)*x_ref[1]
        ref[5,1,1]=(-0.642652252134)*x_ref[0]**o+(0.055584656243)*x_ref[0]+(0.591952970815)*x_ref[1]**o+(0.0560172089007)*x_ref[1]
      else:
        s[0,0,0]=(-0.484756778033)*x[0]**o+(-0.231779672771)*x[0]+(0.919660538872)*x[1]**o+(0.695050320072)*x[1]+(-0.705779426734)*x[2]**o+(-0.422079798585)*x[2]
        s[0,0,1]=(-0.274490489231)*x[0]**o+(0.387197647284)*x[0]+(-0.366490824762)*x[1]**o+(-0.145891361722)*x[1]+(0.397994935942)*x[2]**o+(0.945093224566)*x[2]
        s[0,1,0]=(-0.0703394741878)*x[0]**o+(0.772542317636)*x[0]+(-0.798490294743)*x[1]**o+(0.425742369552)*x[1]+(0.876889140151)*x[2]**o+(-0.796003635544)*x[2]
        s[0,1,1]=(0.96374816047)*x[0]**o+(-0.784131619847)*x[0]+(-0.93299245221)*x[1]**o+(0.0531089993902)*x[1]+(0.0843073749033)*x[2]**o+(0.391297554637)*x[2]
        s[1,0,0]=(-0.481999451884)*x[0]**o+(0.751216783461)*x[0]+(-0.771590406041)*x[1]**o+(0.623595993735)*x[1]+(-0.921210206711)*x[2]**o+(-0.744578883939)*x[2]
        s[1,0,1]=(-0.812200901916)*x[0]**o+(-0.208483140217)*x[0]+(0.413360073886)*x[1]**o+(-0.169471108437)*x[1]+(-0.428293613451)*x[2]**o+(-0.444321672841)*x[2]
        s[1,1,0]=(0.434668673877)*x[0]**o+(-0.227506663361)*x[0]+(0.396429343706)*x[1]**o+(-0.513657758836)*x[1]+(-0.957101683551)*x[2]**o+(-0.736199356445)*x[2]
        s[1,1,1]=(0.414698036573)*x[0]**o+(0.664686946926)*x[0]+(0.813892586594)*x[1]**o+(-0.991059972133)*x[1]+(-0.966403253116)*x[2]**o+(0.87196549345)*x[2]
        s[2,0,0]=(0.412695063456)*x[0]**o+(0.500236015073)*x[0]+(0.264707290309)*x[1]**o+(-0.877912823257)*x[1]+(0.702553921733)*x[2]**o+(-0.866356407426)*x[2]
        s[2,0,1]=(-0.736177808565)*x[0]**o+(-0.881438411498)*x[0]+(-0.92649028732)*x[1]**o+(-0.296241009813)*x[1]+(0.435099826882)*x[2]**o+(0.868673212655)*x[2]
        s[2,1,0]=(-0.812274827825)*x[0]**o+(0.808913617552)*x[0]+(-0.30994330994)*x[1]**o+(0.425208195893)*x[1]+(0.155091457675)*x[2]**o+(0.989636493062)*x[2]
        s[2,1,1]=(0.704418763542)*x[0]**o+(0.714721207479)*x[0]+(-0.772435541761)*x[1]**o+(-0.701532388352)*x[1]+(-0.0556099968738)*x[2]**o+(-0.385766816311)*x[2]
        s[3,0,0]=(0.329616772304)*x[0]**o+(0.497122208449)*x[0]+(0.803535729001)*x[1]**o+(-0.0936214856199)*x[1]+(0.568734100002)*x[2]**o+(-0.357279001712)*x[2]
        s[3,0,1]=(-0.885441319041)*x[0]**o+(-0.9962223468)*x[0]+(0.875311867648)*x[1]**o+(0.421410419839)*x[1]+(0.612998402692)*x[2]**o+(0.77679279321)*x[2]
        s[3,1,0]=(0.70421388911)*x[0]**o+(-0.0190681099318)*x[0]+(-0.841932997223)*x[1]**o+(-0.0250788950858)*x[1]+(-0.222261918598)*x[2]**o+(-0.271140805403)*x[2]
        s[3,1,1]=(0.0694954416869)*x[0]**o+(-0.391253608839)*x[0]+(0.0431700251114)*x[1]**o+(0.405848258278)*x[1]+(-0.615615286047)*x[2]**o+(-0.619571625435)*x[2]
        s[4,0,0]=(-0.686666614457)*x[0]**o+(0.545505992881)*x[0]+(0.334621135483)*x[1]**o+(0.751135339301)*x[1]+(0.61064481817)*x[2]**o+(0.482901826784)*x[2]
        s[4,0,1]=(0.775613779362)*x[0]**o+(0.614521704508)*x[0]+(-0.0189207080695)*x[1]**o+(0.228118267253)*x[1]+(-0.4140063649)*x[2]**o+(0.944802560539)*x[2]
        s[4,1,0]=(0.118375936359)*x[0]**o+(-0.889963092371)*x[0]+(-0.511745894234)*x[1]**o+(0.545576927355)*x[1]+(-0.11589270826)*x[2]**o+(0.559538534484)*x[2]
        s[4,1,1]=(-0.933623754993)*x[0]**o+(-0.174346861161)*x[0]+(-0.101529487111)*x[1]**o+(-0.943622750193)*x[1]+(-0.625527859586)*x[2]**o+(-0.463443945073)*x[2]
        s[5,0,0]=(-0.60504471632)*x[0]**o+(-0.783562086238)*x[0]+(0.614590790432)*x[1]**o+(0.421598441357)*x[1]+(0.433099495473)*x[2]**o+(0.658822060627)*x[2]
        s[5,0,1]=(0.951028673162)*x[0]**o+(0.757383296595)*x[0]+(-0.671328274248)*x[1]**o+(0.30758815695)*x[1]+(0.429507454083)*x[2]**o+(-0.0784898035186)*x[2]
        s[5,1,0]=(0.357359564527)*x[0]**o+(0.402138711471)*x[0]+(0.397488404381)*x[1]**o+(0.532261812787)*x[1]+(-0.379695545138)*x[2]**o+(-0.351422772499)*x[2]
        s[5,1,1]=(0.776246461598)*x[0]**o+(-0.552578551)*x[0]+(0.770179118746)*x[1]**o+(0.51956785933)*x[1]+(-0.182066259842)*x[2]**o+(-0.915270193133)*x[2]
        ref[0,0,0]=(-0.484756778033)*x_ref[0]**o+(-0.231779672771)*x_ref[0]+(0.919660538872)*x_ref[1]**o+(0.695050320072)*x_ref[1]+(-0.705779426734)*x_ref[2]**o+(-0.422079798585)*x_ref[2]
        ref[0,0,1]=(-0.274490489231)*x_ref[0]**o+(0.387197647284)*x_ref[0]+(-0.366490824762)*x_ref[1]**o+(-0.145891361722)*x_ref[1]+(0.397994935942)*x_ref[2]**o+(0.945093224566)*x_ref[2]
        ref[0,1,0]=(-0.0703394741878)*x_ref[0]**o+(0.772542317636)*x_ref[0]+(-0.798490294743)*x_ref[1]**o+(0.425742369552)*x_ref[1]+(0.876889140151)*x_ref[2]**o+(-0.796003635544)*x_ref[2]
        ref[0,1,1]=(0.96374816047)*x_ref[0]**o+(-0.784131619847)*x_ref[0]+(-0.93299245221)*x_ref[1]**o+(0.0531089993902)*x_ref[1]+(0.0843073749033)*x_ref[2]**o+(0.391297554637)*x_ref[2]
        ref[1,0,0]=(-0.481999451884)*x_ref[0]**o+(0.751216783461)*x_ref[0]+(-0.771590406041)*x_ref[1]**o+(0.623595993735)*x_ref[1]+(-0.921210206711)*x_ref[2]**o+(-0.744578883939)*x_ref[2]
        ref[1,0,1]=(-0.812200901916)*x_ref[0]**o+(-0.208483140217)*x_ref[0]+(0.413360073886)*x_ref[1]**o+(-0.169471108437)*x_ref[1]+(-0.428293613451)*x_ref[2]**o+(-0.444321672841)*x_ref[2]
        ref[1,1,0]=(0.434668673877)*x_ref[0]**o+(-0.227506663361)*x_ref[0]+(0.396429343706)*x_ref[1]**o+(-0.513657758836)*x_ref[1]+(-0.957101683551)*x_ref[2]**o+(-0.736199356445)*x_ref[2]
        ref[1,1,1]=(0.414698036573)*x_ref[0]**o+(0.664686946926)*x_ref[0]+(0.813892586594)*x_ref[1]**o+(-0.991059972133)*x_ref[1]+(-0.966403253116)*x_ref[2]**o+(0.87196549345)*x_ref[2]
        ref[2,0,0]=(0.412695063456)*x_ref[0]**o+(0.500236015073)*x_ref[0]+(0.264707290309)*x_ref[1]**o+(-0.877912823257)*x_ref[1]+(0.702553921733)*x_ref[2]**o+(-0.866356407426)*x_ref[2]
        ref[2,0,1]=(-0.736177808565)*x_ref[0]**o+(-0.881438411498)*x_ref[0]+(-0.92649028732)*x_ref[1]**o+(-0.296241009813)*x_ref[1]+(0.435099826882)*x_ref[2]**o+(0.868673212655)*x_ref[2]
        ref[2,1,0]=(-0.812274827825)*x_ref[0]**o+(0.808913617552)*x_ref[0]+(-0.30994330994)*x_ref[1]**o+(0.425208195893)*x_ref[1]+(0.155091457675)*x_ref[2]**o+(0.989636493062)*x_ref[2]
        ref[2,1,1]=(0.704418763542)*x_ref[0]**o+(0.714721207479)*x_ref[0]+(-0.772435541761)*x_ref[1]**o+(-0.701532388352)*x_ref[1]+(-0.0556099968738)*x_ref[2]**o+(-0.385766816311)*x_ref[2]
        ref[3,0,0]=(0.329616772304)*x_ref[0]**o+(0.497122208449)*x_ref[0]+(0.803535729001)*x_ref[1]**o+(-0.0936214856199)*x_ref[1]+(0.568734100002)*x_ref[2]**o+(-0.357279001712)*x_ref[2]
        ref[3,0,1]=(-0.885441319041)*x_ref[0]**o+(-0.9962223468)*x_ref[0]+(0.875311867648)*x_ref[1]**o+(0.421410419839)*x_ref[1]+(0.612998402692)*x_ref[2]**o+(0.77679279321)*x_ref[2]
        ref[3,1,0]=(0.70421388911)*x_ref[0]**o+(-0.0190681099318)*x_ref[0]+(-0.841932997223)*x_ref[1]**o+(-0.0250788950858)*x_ref[1]+(-0.222261918598)*x_ref[2]**o+(-0.271140805403)*x_ref[2]
        ref[3,1,1]=(0.0694954416869)*x_ref[0]**o+(-0.391253608839)*x_ref[0]+(0.0431700251114)*x_ref[1]**o+(0.405848258278)*x_ref[1]+(-0.615615286047)*x_ref[2]**o+(-0.619571625435)*x_ref[2]
        ref[4,0,0]=(-0.686666614457)*x_ref[0]**o+(0.545505992881)*x_ref[0]+(0.334621135483)*x_ref[1]**o+(0.751135339301)*x_ref[1]+(0.61064481817)*x_ref[2]**o+(0.482901826784)*x_ref[2]
        ref[4,0,1]=(0.775613779362)*x_ref[0]**o+(0.614521704508)*x_ref[0]+(-0.0189207080695)*x_ref[1]**o+(0.228118267253)*x_ref[1]+(-0.4140063649)*x_ref[2]**o+(0.944802560539)*x_ref[2]
        ref[4,1,0]=(0.118375936359)*x_ref[0]**o+(-0.889963092371)*x_ref[0]+(-0.511745894234)*x_ref[1]**o+(0.545576927355)*x_ref[1]+(-0.11589270826)*x_ref[2]**o+(0.559538534484)*x_ref[2]
        ref[4,1,1]=(-0.933623754993)*x_ref[0]**o+(-0.174346861161)*x_ref[0]+(-0.101529487111)*x_ref[1]**o+(-0.943622750193)*x_ref[1]+(-0.625527859586)*x_ref[2]**o+(-0.463443945073)*x_ref[2]
        ref[5,0,0]=(-0.60504471632)*x_ref[0]**o+(-0.783562086238)*x_ref[0]+(0.614590790432)*x_ref[1]**o+(0.421598441357)*x_ref[1]+(0.433099495473)*x_ref[2]**o+(0.658822060627)*x_ref[2]
        ref[5,0,1]=(0.951028673162)*x_ref[0]**o+(0.757383296595)*x_ref[0]+(-0.671328274248)*x_ref[1]**o+(0.30758815695)*x_ref[1]+(0.429507454083)*x_ref[2]**o+(-0.0784898035186)*x_ref[2]
        ref[5,1,0]=(0.357359564527)*x_ref[0]**o+(0.402138711471)*x_ref[0]+(0.397488404381)*x_ref[1]**o+(0.532261812787)*x_ref[1]+(-0.379695545138)*x_ref[2]**o+(-0.351422772499)*x_ref[2]
        ref[5,1,1]=(0.776246461598)*x_ref[0]**o+(-0.552578551)*x_ref[0]+(0.770179118746)*x_ref[1]**o+(0.51956785933)*x_ref[1]+(-0.182066259842)*x_ref[2]**o+(-0.915270193133)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromSymbol_ReducedFunctionOnContactZero_rank4(self):
      """
      tests interpolate for rank 4 Symbol onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactZero onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactZero(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5, 3, 2),dim=dim)
      s=Data(0,(4, 5, 3, 2),w)
      ref=Data(0,(4, 5, 3, 2),w_ref)
      if dim==2:
        s[0,0,0,0]=(-0.682677446173)*x[0]**o+(0.504486605276)*x[0]+(-0.188432273127)*x[1]**o+(-0.115942662064)*x[1]
        s[0,0,0,1]=(0.0136192617812)*x[0]**o+(-0.538389464927)*x[0]+(0.124702227013)*x[1]**o+(0.501919436502)*x[1]
        s[0,0,1,0]=(0.156652087198)*x[0]**o+(0.337393016047)*x[0]+(-0.448397431209)*x[1]**o+(0.117709269726)*x[1]
        s[0,0,1,1]=(-0.547896793651)*x[0]**o+(-0.747571514787)*x[0]+(-0.340601123893)*x[1]**o+(-0.710572632704)*x[1]
        s[0,0,2,0]=(-0.780567482126)*x[0]**o+(0.245095469386)*x[0]+(0.114230637835)*x[1]**o+(-0.608414329196)*x[1]
        s[0,0,2,1]=(-0.492428786813)*x[0]**o+(0.931675985507)*x[0]+(-0.748267062348)*x[1]**o+(-0.856380494895)*x[1]
        s[0,1,0,0]=(-0.596856834994)*x[0]**o+(0.147456639538)*x[0]+(0.189964473187)*x[1]**o+(0.618844379651)*x[1]
        s[0,1,0,1]=(0.301379428437)*x[0]**o+(-0.970231773668)*x[0]+(-0.298459364955)*x[1]**o+(-0.823167810649)*x[1]
        s[0,1,1,0]=(0.815054294305)*x[0]**o+(-0.79025331211)*x[0]+(-0.348789406252)*x[1]**o+(0.69147436715)*x[1]
        s[0,1,1,1]=(-0.994338848091)*x[0]**o+(-0.352485315079)*x[0]+(-0.825830724905)*x[1]**o+(0.568756966202)*x[1]
        s[0,1,2,0]=(-0.262404141899)*x[0]**o+(0.695058140486)*x[0]+(-0.562413865895)*x[1]**o+(0.744853779046)*x[1]
        s[0,1,2,1]=(-0.526480240707)*x[0]**o+(-0.246633022689)*x[0]+(-0.677508152973)*x[1]**o+(0.263085459331)*x[1]
        s[0,2,0,0]=(-0.440455373866)*x[0]**o+(0.982673077522)*x[0]+(0.267643265744)*x[1]**o+(-0.248708912008)*x[1]
        s[0,2,0,1]=(-0.629497400912)*x[0]**o+(-0.222971392637)*x[0]+(0.476895454043)*x[1]**o+(-0.560588300663)*x[1]
        s[0,2,1,0]=(-0.142675958641)*x[0]**o+(0.858528587711)*x[0]+(-0.848579730169)*x[1]**o+(-0.0813066455284)*x[1]
        s[0,2,1,1]=(0.309891337726)*x[0]**o+(-0.966050939539)*x[0]+(-0.214997020692)*x[1]**o+(0.823855020343)*x[1]
        s[0,2,2,0]=(0.61116679522)*x[0]**o+(0.663744432393)*x[0]+(0.414661416033)*x[1]**o+(-0.996447270631)*x[1]
        s[0,2,2,1]=(-0.634958711796)*x[0]**o+(0.27654613916)*x[0]+(0.157322235818)*x[1]**o+(-0.449130441789)*x[1]
        s[0,3,0,0]=(-0.949513930962)*x[0]**o+(0.43714393009)*x[0]+(0.44401680011)*x[1]**o+(0.775550283269)*x[1]
        s[0,3,0,1]=(-0.560623279771)*x[0]**o+(-0.261230828373)*x[0]+(0.809046529697)*x[1]**o+(-0.00091638239609)*x[1]
        s[0,3,1,0]=(-0.897951425228)*x[0]**o+(-0.854681659889)*x[0]+(-0.600467143561)*x[1]**o+(-0.543812193981)*x[1]
        s[0,3,1,1]=(-0.252640196006)*x[0]**o+(-0.0147679084824)*x[0]+(0.474400135578)*x[1]**o+(-0.649969341279)*x[1]
        s[0,3,2,0]=(0.148265101594)*x[0]**o+(0.865387108497)*x[0]+(-0.853468586257)*x[1]**o+(-0.728220923395)*x[1]
        s[0,3,2,1]=(-0.998415894528)*x[0]**o+(0.385240420153)*x[0]+(-0.970588182269)*x[1]**o+(-0.715015353507)*x[1]
        s[0,4,0,0]=(0.47018041111)*x[0]**o+(-0.709194247739)*x[0]+(-0.219962382546)*x[1]**o+(0.354322367748)*x[1]
        s[0,4,0,1]=(-0.322814742992)*x[0]**o+(0.549832283705)*x[0]+(0.174383028533)*x[1]**o+(0.582821277257)*x[1]
        s[0,4,1,0]=(0.263785420773)*x[0]**o+(-0.072514835468)*x[0]+(-0.857389703063)*x[1]**o+(-0.775322452374)*x[1]
        s[0,4,1,1]=(0.938271810888)*x[0]**o+(-0.823646250497)*x[0]+(-0.685525888071)*x[1]**o+(-0.603286444077)*x[1]
        s[0,4,2,0]=(-0.609415096341)*x[0]**o+(-0.720923292264)*x[0]+(-0.965452230391)*x[1]**o+(0.285193163647)*x[1]
        s[0,4,2,1]=(0.421976649551)*x[0]**o+(0.990010463736)*x[0]+(-0.860923000529)*x[1]**o+(0.726093525288)*x[1]
        s[1,0,0,0]=(-0.996811081802)*x[0]**o+(-0.97863301939)*x[0]+(0.202626167636)*x[1]**o+(0.0905062876049)*x[1]
        s[1,0,0,1]=(-0.364721401486)*x[0]**o+(0.935308652161)*x[0]+(-0.405836642917)*x[1]**o+(0.550390830913)*x[1]
        s[1,0,1,0]=(-0.701902045301)*x[0]**o+(0.531854328517)*x[0]+(-0.732386759539)*x[1]**o+(0.425326883394)*x[1]
        s[1,0,1,1]=(0.447994101591)*x[0]**o+(-0.813809278167)*x[0]+(-0.539345843485)*x[1]**o+(-0.0354180208603)*x[1]
        s[1,0,2,0]=(-0.603871847208)*x[0]**o+(0.842448438568)*x[0]+(-0.7438939901)*x[1]**o+(-0.587219172421)*x[1]
        s[1,0,2,1]=(-0.971267843684)*x[0]**o+(0.259537498927)*x[0]+(-0.698510853299)*x[1]**o+(0.722962896543)*x[1]
        s[1,1,0,0]=(0.594944086583)*x[0]**o+(0.296356236144)*x[0]+(0.0970882705472)*x[1]**o+(0.849524363397)*x[1]
        s[1,1,0,1]=(0.837454132615)*x[0]**o+(0.0858001627187)*x[0]+(0.019269936186)*x[1]**o+(-0.86311221183)*x[1]
        s[1,1,1,0]=(0.947339105706)*x[0]**o+(0.424118233136)*x[0]+(-0.314979294332)*x[1]**o+(0.507664692875)*x[1]
        s[1,1,1,1]=(0.805027334702)*x[0]**o+(0.682733602548)*x[0]+(-0.162752989543)*x[1]**o+(-0.438803102705)*x[1]
        s[1,1,2,0]=(0.24090406784)*x[0]**o+(-0.564828151023)*x[0]+(-0.446384646081)*x[1]**o+(0.06921350334)*x[1]
        s[1,1,2,1]=(0.535823223755)*x[0]**o+(0.0248971256814)*x[0]+(-0.508444148354)*x[1]**o+(0.840622721627)*x[1]
        s[1,2,0,0]=(0.0525491067608)*x[0]**o+(-0.384851762445)*x[0]+(0.305218692316)*x[1]**o+(-0.319267902416)*x[1]
        s[1,2,0,1]=(0.514545608836)*x[0]**o+(0.157544217219)*x[0]+(0.0378829715039)*x[1]**o+(0.778857744661)*x[1]
        s[1,2,1,0]=(0.702907133813)*x[0]**o+(-0.773975432754)*x[0]+(0.489924281654)*x[1]**o+(-0.771122137868)*x[1]
        s[1,2,1,1]=(0.305517959872)*x[0]**o+(-0.229609729031)*x[0]+(-0.999272871312)*x[1]**o+(0.283774230586)*x[1]
        s[1,2,2,0]=(0.943767954042)*x[0]**o+(0.214819458913)*x[0]+(0.883796685301)*x[1]**o+(0.88624077323)*x[1]
        s[1,2,2,1]=(0.904685466034)*x[0]**o+(0.238007243352)*x[0]+(-0.496694568656)*x[1]**o+(0.90503921706)*x[1]
        s[1,3,0,0]=(-0.323719826916)*x[0]**o+(-0.679887105617)*x[0]+(-0.204658991412)*x[1]**o+(-0.782808301573)*x[1]
        s[1,3,0,1]=(-0.915796887204)*x[0]**o+(-0.210816609805)*x[0]+(-0.229804577299)*x[1]**o+(0.224669921335)*x[1]
        s[1,3,1,0]=(0.0502687121641)*x[0]**o+(-0.873227856206)*x[0]+(-0.159031793692)*x[1]**o+(-0.891685753753)*x[1]
        s[1,3,1,1]=(0.492846348595)*x[0]**o+(0.787085648664)*x[0]+(0.65805763747)*x[1]**o+(0.79345020859)*x[1]
        s[1,3,2,0]=(0.186889967831)*x[0]**o+(-0.759385582402)*x[0]+(0.526210288461)*x[1]**o+(0.408440324329)*x[1]
        s[1,3,2,1]=(0.49057760628)*x[0]**o+(0.583558905672)*x[0]+(0.200515771351)*x[1]**o+(-0.0457600002227)*x[1]
        s[1,4,0,0]=(0.986278922497)*x[0]**o+(-0.724078510554)*x[0]+(0.918959062512)*x[1]**o+(-0.696724335474)*x[1]
        s[1,4,0,1]=(-0.624189125377)*x[0]**o+(0.849396531428)*x[0]+(-0.793610811078)*x[1]**o+(0.473468667211)*x[1]
        s[1,4,1,0]=(0.850059248076)*x[0]**o+(-0.167497586421)*x[0]+(-0.401914828635)*x[1]**o+(-0.446166310885)*x[1]
        s[1,4,1,1]=(0.735056540402)*x[0]**o+(0.956860138776)*x[0]+(-0.657753509921)*x[1]**o+(0.104941138398)*x[1]
        s[1,4,2,0]=(0.178383558524)*x[0]**o+(-0.695410327712)*x[0]+(-0.896595978256)*x[1]**o+(0.500721122887)*x[1]
        s[1,4,2,1]=(0.00266985547612)*x[0]**o+(0.569844332841)*x[0]+(0.30469482092)*x[1]**o+(0.18820848487)*x[1]
        s[2,0,0,0]=(0.0726883565313)*x[0]**o+(-0.151895019503)*x[0]+(-0.566412947745)*x[1]**o+(0.605833954559)*x[1]
        s[2,0,0,1]=(-0.759348816055)*x[0]**o+(0.203730579434)*x[0]+(-0.306267994)*x[1]**o+(-0.121655396311)*x[1]
        s[2,0,1,0]=(-0.529988368349)*x[0]**o+(0.265126908621)*x[0]+(-0.127232300402)*x[1]**o+(-0.711018521148)*x[1]
        s[2,0,1,1]=(0.367616519674)*x[0]**o+(-0.856828909791)*x[0]+(-0.988581037916)*x[1]**o+(0.236273236567)*x[1]
        s[2,0,2,0]=(0.980896603038)*x[0]**o+(0.0101391252514)*x[0]+(-0.734647631536)*x[1]**o+(-0.298628688073)*x[1]
        s[2,0,2,1]=(-0.876812266906)*x[0]**o+(-0.0545067479612)*x[0]+(-0.752772584824)*x[1]**o+(-0.797225970025)*x[1]
        s[2,1,0,0]=(0.9776767796)*x[0]**o+(-0.381948430876)*x[0]+(0.705224317124)*x[1]**o+(0.431609349218)*x[1]
        s[2,1,0,1]=(0.0282522834733)*x[0]**o+(-0.519316841837)*x[0]+(-0.898831067576)*x[1]**o+(0.491570698382)*x[1]
        s[2,1,1,0]=(-0.874077440763)*x[0]**o+(0.0050283385939)*x[0]+(-0.45400999477)*x[1]**o+(-0.823942392997)*x[1]
        s[2,1,1,1]=(0.705592988895)*x[0]**o+(0.545242748068)*x[0]+(-0.0240067342318)*x[1]**o+(-0.764691278707)*x[1]
        s[2,1,2,0]=(0.214087039013)*x[0]**o+(0.198346865884)*x[0]+(0.195275868007)*x[1]**o+(-0.615146376705)*x[1]
        s[2,1,2,1]=(0.819578786211)*x[0]**o+(0.238162773292)*x[0]+(-0.488387775705)*x[1]**o+(0.143425392477)*x[1]
        s[2,2,0,0]=(0.41469049634)*x[0]**o+(-0.870291745289)*x[0]+(-0.978682320238)*x[1]**o+(0.315825060763)*x[1]
        s[2,2,0,1]=(-0.725231531597)*x[0]**o+(-0.583383717934)*x[0]+(0.677865725085)*x[1]**o+(0.590055042471)*x[1]
        s[2,2,1,0]=(-0.827316408906)*x[0]**o+(0.152995858286)*x[0]+(-0.864980658862)*x[1]**o+(-0.267986836659)*x[1]
        s[2,2,1,1]=(-0.333272933954)*x[0]**o+(0.339541494142)*x[0]+(0.982274574789)*x[1]**o+(0.488564854285)*x[1]
        s[2,2,2,0]=(0.433303448225)*x[0]**o+(0.325824460514)*x[0]+(-0.418818415293)*x[1]**o+(0.884475442413)*x[1]
        s[2,2,2,1]=(-0.692651589533)*x[0]**o+(0.85015849795)*x[0]+(-0.683352003673)*x[1]**o+(0.458991254181)*x[1]
        s[2,3,0,0]=(-0.754234631384)*x[0]**o+(0.240243331561)*x[0]+(0.545360646104)*x[1]**o+(-0.960857789697)*x[1]
        s[2,3,0,1]=(0.911046185694)*x[0]**o+(-0.909178154095)*x[0]+(-0.330132665955)*x[1]**o+(-0.978660867939)*x[1]
        s[2,3,1,0]=(0.88161349743)*x[0]**o+(0.0812270265162)*x[0]+(-0.584074638899)*x[1]**o+(-0.309263951959)*x[1]
        s[2,3,1,1]=(-0.59736682275)*x[0]**o+(-0.176681221389)*x[0]+(0.750820855296)*x[1]**o+(0.977121067295)*x[1]
        s[2,3,2,0]=(-0.926460632737)*x[0]**o+(-0.465577825838)*x[0]+(0.577728110369)*x[1]**o+(-0.973296874179)*x[1]
        s[2,3,2,1]=(-0.959292420516)*x[0]**o+(-0.173671029796)*x[0]+(0.644855220567)*x[1]**o+(-0.659401238679)*x[1]
        s[2,4,0,0]=(-0.185887734162)*x[0]**o+(-0.184924703674)*x[0]+(0.12829518222)*x[1]**o+(-0.95529948598)*x[1]
        s[2,4,0,1]=(-0.222082370982)*x[0]**o+(0.480067405868)*x[0]+(0.737491570657)*x[1]**o+(-0.0505388125547)*x[1]
        s[2,4,1,0]=(-0.759488182696)*x[0]**o+(-0.209420047186)*x[0]+(-0.257523848658)*x[1]**o+(-0.88701477207)*x[1]
        s[2,4,1,1]=(0.549919898662)*x[0]**o+(-0.204942654253)*x[0]+(0.237781682077)*x[1]**o+(-0.398029223748)*x[1]
        s[2,4,2,0]=(-0.429624077152)*x[0]**o+(0.477737955149)*x[0]+(0.150704189177)*x[1]**o+(-0.651247641141)*x[1]
        s[2,4,2,1]=(-0.894705558322)*x[0]**o+(-0.437755159653)*x[0]+(-0.984539083926)*x[1]**o+(0.771869802974)*x[1]
        s[3,0,0,0]=(0.613531927806)*x[0]**o+(-0.262670681917)*x[0]+(0.272118391563)*x[1]**o+(-0.911450635138)*x[1]
        s[3,0,0,1]=(0.0749962421787)*x[0]**o+(-0.280125180177)*x[0]+(0.535812839231)*x[1]**o+(-0.898511642603)*x[1]
        s[3,0,1,0]=(-0.493682703273)*x[0]**o+(0.564806921005)*x[0]+(-0.424211826783)*x[1]**o+(0.955398127065)*x[1]
        s[3,0,1,1]=(-0.208922295917)*x[0]**o+(0.552971608191)*x[0]+(-0.482834655473)*x[1]**o+(0.916990053757)*x[1]
        s[3,0,2,0]=(0.225448292089)*x[0]**o+(-0.508992793538)*x[0]+(-0.301030316256)*x[1]**o+(0.234012186162)*x[1]
        s[3,0,2,1]=(0.173694397805)*x[0]**o+(-0.0436667050758)*x[0]+(-0.391980496474)*x[1]**o+(0.183288325597)*x[1]
        s[3,1,0,0]=(-0.078216910574)*x[0]**o+(-0.993944069614)*x[0]+(-0.250488879366)*x[1]**o+(0.648115982978)*x[1]
        s[3,1,0,1]=(-0.346993095448)*x[0]**o+(0.630491595383)*x[0]+(0.128199747925)*x[1]**o+(-0.681982197159)*x[1]
        s[3,1,1,0]=(0.0360684354752)*x[0]**o+(-0.8297568105)*x[0]+(0.122927048201)*x[1]**o+(0.0448718398101)*x[1]
        s[3,1,1,1]=(-0.603474854473)*x[0]**o+(-0.435029027744)*x[0]+(0.632649021375)*x[1]**o+(0.414362202135)*x[1]
        s[3,1,2,0]=(-0.912833626509)*x[0]**o+(-0.828668448463)*x[0]+(-0.754484607948)*x[1]**o+(0.503101904128)*x[1]
        s[3,1,2,1]=(0.00649229353309)*x[0]**o+(-0.96124954377)*x[0]+(0.648294131558)*x[1]**o+(-0.957122691907)*x[1]
        s[3,2,0,0]=(0.476913272421)*x[0]**o+(0.0878038200542)*x[0]+(-0.516953661957)*x[1]**o+(0.0134540618391)*x[1]
        s[3,2,0,1]=(0.3748519133)*x[0]**o+(-0.970356842975)*x[0]+(-0.874984768114)*x[1]**o+(-0.0950957723387)*x[1]
        s[3,2,1,0]=(-0.711688301522)*x[0]**o+(-0.20444931953)*x[0]+(0.601925297065)*x[1]**o+(0.939713142428)*x[1]
        s[3,2,1,1]=(-0.869179475562)*x[0]**o+(-0.771823270124)*x[0]+(0.0447051413494)*x[1]**o+(0.769181827448)*x[1]
        s[3,2,2,0]=(0.341502469812)*x[0]**o+(-0.892625503786)*x[0]+(-0.683764736135)*x[1]**o+(-0.208342034139)*x[1]
        s[3,2,2,1]=(-0.876208883831)*x[0]**o+(-0.705748793184)*x[0]+(0.92260186528)*x[1]**o+(-0.437547481021)*x[1]
        s[3,3,0,0]=(-0.387117059014)*x[0]**o+(-0.30316333118)*x[0]+(-0.107906971687)*x[1]**o+(0.27027380043)*x[1]
        s[3,3,0,1]=(0.760401124102)*x[0]**o+(0.0392442137961)*x[0]+(0.527403460784)*x[1]**o+(0.49506478844)*x[1]
        s[3,3,1,0]=(-0.797802219992)*x[0]**o+(-0.718681383351)*x[0]+(0.723820891624)*x[1]**o+(0.851058726763)*x[1]
        s[3,3,1,1]=(-0.477124467307)*x[0]**o+(-0.912639051937)*x[0]+(-0.653399611549)*x[1]**o+(-0.527816963598)*x[1]
        s[3,3,2,0]=(-0.375620015065)*x[0]**o+(0.713563556042)*x[0]+(0.592214060154)*x[1]**o+(-0.663713190063)*x[1]
        s[3,3,2,1]=(0.13839650904)*x[0]**o+(-0.894351932455)*x[0]+(0.130542033693)*x[1]**o+(-0.420895602544)*x[1]
        s[3,4,0,0]=(-0.438194955853)*x[0]**o+(-0.119296803116)*x[0]+(0.683500326777)*x[1]**o+(-0.226368973557)*x[1]
        s[3,4,0,1]=(-0.232920353049)*x[0]**o+(-0.35856926089)*x[0]+(0.882231830184)*x[1]**o+(-0.000957747821531)*x[1]
        s[3,4,1,0]=(-0.0939581521731)*x[0]**o+(0.831315092139)*x[0]+(0.042454046079)*x[1]**o+(0.791725586274)*x[1]
        s[3,4,1,1]=(-0.328628704689)*x[0]**o+(0.434271457185)*x[0]+(0.632929764639)*x[1]**o+(-0.526043410013)*x[1]
        s[3,4,2,0]=(0.429887737881)*x[0]**o+(0.828275504726)*x[0]+(-0.199204123672)*x[1]**o+(-0.363782496284)*x[1]
        s[3,4,2,1]=(-0.446866424171)*x[0]**o+(-0.204025885117)*x[0]+(-0.899560130794)*x[1]**o+(0.184518622033)*x[1]
        ref[0,0,0,0]=(-0.682677446173)*x_ref[0]**o+(0.504486605276)*x_ref[0]+(-0.188432273127)*x_ref[1]**o+(-0.115942662064)*x_ref[1]
        ref[0,0,0,1]=(0.0136192617812)*x_ref[0]**o+(-0.538389464927)*x_ref[0]+(0.124702227013)*x_ref[1]**o+(0.501919436502)*x_ref[1]
        ref[0,0,1,0]=(0.156652087198)*x_ref[0]**o+(0.337393016047)*x_ref[0]+(-0.448397431209)*x_ref[1]**o+(0.117709269726)*x_ref[1]
        ref[0,0,1,1]=(-0.547896793651)*x_ref[0]**o+(-0.747571514787)*x_ref[0]+(-0.340601123893)*x_ref[1]**o+(-0.710572632704)*x_ref[1]
        ref[0,0,2,0]=(-0.780567482126)*x_ref[0]**o+(0.245095469386)*x_ref[0]+(0.114230637835)*x_ref[1]**o+(-0.608414329196)*x_ref[1]
        ref[0,0,2,1]=(-0.492428786813)*x_ref[0]**o+(0.931675985507)*x_ref[0]+(-0.748267062348)*x_ref[1]**o+(-0.856380494895)*x_ref[1]
        ref[0,1,0,0]=(-0.596856834994)*x_ref[0]**o+(0.147456639538)*x_ref[0]+(0.189964473187)*x_ref[1]**o+(0.618844379651)*x_ref[1]
        ref[0,1,0,1]=(0.301379428437)*x_ref[0]**o+(-0.970231773668)*x_ref[0]+(-0.298459364955)*x_ref[1]**o+(-0.823167810649)*x_ref[1]
        ref[0,1,1,0]=(0.815054294305)*x_ref[0]**o+(-0.79025331211)*x_ref[0]+(-0.348789406252)*x_ref[1]**o+(0.69147436715)*x_ref[1]
        ref[0,1,1,1]=(-0.994338848091)*x_ref[0]**o+(-0.352485315079)*x_ref[0]+(-0.825830724905)*x_ref[1]**o+(0.568756966202)*x_ref[1]
        ref[0,1,2,0]=(-0.262404141899)*x_ref[0]**o+(0.695058140486)*x_ref[0]+(-0.562413865895)*x_ref[1]**o+(0.744853779046)*x_ref[1]
        ref[0,1,2,1]=(-0.526480240707)*x_ref[0]**o+(-0.246633022689)*x_ref[0]+(-0.677508152973)*x_ref[1]**o+(0.263085459331)*x_ref[1]
        ref[0,2,0,0]=(-0.440455373866)*x_ref[0]**o+(0.982673077522)*x_ref[0]+(0.267643265744)*x_ref[1]**o+(-0.248708912008)*x_ref[1]
        ref[0,2,0,1]=(-0.629497400912)*x_ref[0]**o+(-0.222971392637)*x_ref[0]+(0.476895454043)*x_ref[1]**o+(-0.560588300663)*x_ref[1]
        ref[0,2,1,0]=(-0.142675958641)*x_ref[0]**o+(0.858528587711)*x_ref[0]+(-0.848579730169)*x_ref[1]**o+(-0.0813066455284)*x_ref[1]
        ref[0,2,1,1]=(0.309891337726)*x_ref[0]**o+(-0.966050939539)*x_ref[0]+(-0.214997020692)*x_ref[1]**o+(0.823855020343)*x_ref[1]
        ref[0,2,2,0]=(0.61116679522)*x_ref[0]**o+(0.663744432393)*x_ref[0]+(0.414661416033)*x_ref[1]**o+(-0.996447270631)*x_ref[1]
        ref[0,2,2,1]=(-0.634958711796)*x_ref[0]**o+(0.27654613916)*x_ref[0]+(0.157322235818)*x_ref[1]**o+(-0.449130441789)*x_ref[1]
        ref[0,3,0,0]=(-0.949513930962)*x_ref[0]**o+(0.43714393009)*x_ref[0]+(0.44401680011)*x_ref[1]**o+(0.775550283269)*x_ref[1]
        ref[0,3,0,1]=(-0.560623279771)*x_ref[0]**o+(-0.261230828373)*x_ref[0]+(0.809046529697)*x_ref[1]**o+(-0.00091638239609)*x_ref[1]
        ref[0,3,1,0]=(-0.897951425228)*x_ref[0]**o+(-0.854681659889)*x_ref[0]+(-0.600467143561)*x_ref[1]**o+(-0.543812193981)*x_ref[1]
        ref[0,3,1,1]=(-0.252640196006)*x_ref[0]**o+(-0.0147679084824)*x_ref[0]+(0.474400135578)*x_ref[1]**o+(-0.649969341279)*x_ref[1]
        ref[0,3,2,0]=(0.148265101594)*x_ref[0]**o+(0.865387108497)*x_ref[0]+(-0.853468586257)*x_ref[1]**o+(-0.728220923395)*x_ref[1]
        ref[0,3,2,1]=(-0.998415894528)*x_ref[0]**o+(0.385240420153)*x_ref[0]+(-0.970588182269)*x_ref[1]**o+(-0.715015353507)*x_ref[1]
        ref[0,4,0,0]=(0.47018041111)*x_ref[0]**o+(-0.709194247739)*x_ref[0]+(-0.219962382546)*x_ref[1]**o+(0.354322367748)*x_ref[1]
        ref[0,4,0,1]=(-0.322814742992)*x_ref[0]**o+(0.549832283705)*x_ref[0]+(0.174383028533)*x_ref[1]**o+(0.582821277257)*x_ref[1]
        ref[0,4,1,0]=(0.263785420773)*x_ref[0]**o+(-0.072514835468)*x_ref[0]+(-0.857389703063)*x_ref[1]**o+(-0.775322452374)*x_ref[1]
        ref[0,4,1,1]=(0.938271810888)*x_ref[0]**o+(-0.823646250497)*x_ref[0]+(-0.685525888071)*x_ref[1]**o+(-0.603286444077)*x_ref[1]
        ref[0,4,2,0]=(-0.609415096341)*x_ref[0]**o+(-0.720923292264)*x_ref[0]+(-0.965452230391)*x_ref[1]**o+(0.285193163647)*x_ref[1]
        ref[0,4,2,1]=(0.421976649551)*x_ref[0]**o+(0.990010463736)*x_ref[0]+(-0.860923000529)*x_ref[1]**o+(0.726093525288)*x_ref[1]
        ref[1,0,0,0]=(-0.996811081802)*x_ref[0]**o+(-0.97863301939)*x_ref[0]+(0.202626167636)*x_ref[1]**o+(0.0905062876049)*x_ref[1]
        ref[1,0,0,1]=(-0.364721401486)*x_ref[0]**o+(0.935308652161)*x_ref[0]+(-0.405836642917)*x_ref[1]**o+(0.550390830913)*x_ref[1]
        ref[1,0,1,0]=(-0.701902045301)*x_ref[0]**o+(0.531854328517)*x_ref[0]+(-0.732386759539)*x_ref[1]**o+(0.425326883394)*x_ref[1]
        ref[1,0,1,1]=(0.447994101591)*x_ref[0]**o+(-0.813809278167)*x_ref[0]+(-0.539345843485)*x_ref[1]**o+(-0.0354180208603)*x_ref[1]
        ref[1,0,2,0]=(-0.603871847208)*x_ref[0]**o+(0.842448438568)*x_ref[0]+(-0.7438939901)*x_ref[1]**o+(-0.587219172421)*x_ref[1]
        ref[1,0,2,1]=(-0.971267843684)*x_ref[0]**o+(0.259537498927)*x_ref[0]+(-0.698510853299)*x_ref[1]**o+(0.722962896543)*x_ref[1]
        ref[1,1,0,0]=(0.594944086583)*x_ref[0]**o+(0.296356236144)*x_ref[0]+(0.0970882705472)*x_ref[1]**o+(0.849524363397)*x_ref[1]
        ref[1,1,0,1]=(0.837454132615)*x_ref[0]**o+(0.0858001627187)*x_ref[0]+(0.019269936186)*x_ref[1]**o+(-0.86311221183)*x_ref[1]
        ref[1,1,1,0]=(0.947339105706)*x_ref[0]**o+(0.424118233136)*x_ref[0]+(-0.314979294332)*x_ref[1]**o+(0.507664692875)*x_ref[1]
        ref[1,1,1,1]=(0.805027334702)*x_ref[0]**o+(0.682733602548)*x_ref[0]+(-0.162752989543)*x_ref[1]**o+(-0.438803102705)*x_ref[1]
        ref[1,1,2,0]=(0.24090406784)*x_ref[0]**o+(-0.564828151023)*x_ref[0]+(-0.446384646081)*x_ref[1]**o+(0.06921350334)*x_ref[1]
        ref[1,1,2,1]=(0.535823223755)*x_ref[0]**o+(0.0248971256814)*x_ref[0]+(-0.508444148354)*x_ref[1]**o+(0.840622721627)*x_ref[1]
        ref[1,2,0,0]=(0.0525491067608)*x_ref[0]**o+(-0.384851762445)*x_ref[0]+(0.305218692316)*x_ref[1]**o+(-0.319267902416)*x_ref[1]
        ref[1,2,0,1]=(0.514545608836)*x_ref[0]**o+(0.157544217219)*x_ref[0]+(0.0378829715039)*x_ref[1]**o+(0.778857744661)*x_ref[1]
        ref[1,2,1,0]=(0.702907133813)*x_ref[0]**o+(-0.773975432754)*x_ref[0]+(0.489924281654)*x_ref[1]**o+(-0.771122137868)*x_ref[1]
        ref[1,2,1,1]=(0.305517959872)*x_ref[0]**o+(-0.229609729031)*x_ref[0]+(-0.999272871312)*x_ref[1]**o+(0.283774230586)*x_ref[1]
        ref[1,2,2,0]=(0.943767954042)*x_ref[0]**o+(0.214819458913)*x_ref[0]+(0.883796685301)*x_ref[1]**o+(0.88624077323)*x_ref[1]
        ref[1,2,2,1]=(0.904685466034)*x_ref[0]**o+(0.238007243352)*x_ref[0]+(-0.496694568656)*x_ref[1]**o+(0.90503921706)*x_ref[1]
        ref[1,3,0,0]=(-0.323719826916)*x_ref[0]**o+(-0.679887105617)*x_ref[0]+(-0.204658991412)*x_ref[1]**o+(-0.782808301573)*x_ref[1]
        ref[1,3,0,1]=(-0.915796887204)*x_ref[0]**o+(-0.210816609805)*x_ref[0]+(-0.229804577299)*x_ref[1]**o+(0.224669921335)*x_ref[1]
        ref[1,3,1,0]=(0.0502687121641)*x_ref[0]**o+(-0.873227856206)*x_ref[0]+(-0.159031793692)*x_ref[1]**o+(-0.891685753753)*x_ref[1]
        ref[1,3,1,1]=(0.492846348595)*x_ref[0]**o+(0.787085648664)*x_ref[0]+(0.65805763747)*x_ref[1]**o+(0.79345020859)*x_ref[1]
        ref[1,3,2,0]=(0.186889967831)*x_ref[0]**o+(-0.759385582402)*x_ref[0]+(0.526210288461)*x_ref[1]**o+(0.408440324329)*x_ref[1]
        ref[1,3,2,1]=(0.49057760628)*x_ref[0]**o+(0.583558905672)*x_ref[0]+(0.200515771351)*x_ref[1]**o+(-0.0457600002227)*x_ref[1]
        ref[1,4,0,0]=(0.986278922497)*x_ref[0]**o+(-0.724078510554)*x_ref[0]+(0.918959062512)*x_ref[1]**o+(-0.696724335474)*x_ref[1]
        ref[1,4,0,1]=(-0.624189125377)*x_ref[0]**o+(0.849396531428)*x_ref[0]+(-0.793610811078)*x_ref[1]**o+(0.473468667211)*x_ref[1]
        ref[1,4,1,0]=(0.850059248076)*x_ref[0]**o+(-0.167497586421)*x_ref[0]+(-0.401914828635)*x_ref[1]**o+(-0.446166310885)*x_ref[1]
        ref[1,4,1,1]=(0.735056540402)*x_ref[0]**o+(0.956860138776)*x_ref[0]+(-0.657753509921)*x_ref[1]**o+(0.104941138398)*x_ref[1]
        ref[1,4,2,0]=(0.178383558524)*x_ref[0]**o+(-0.695410327712)*x_ref[0]+(-0.896595978256)*x_ref[1]**o+(0.500721122887)*x_ref[1]
        ref[1,4,2,1]=(0.00266985547612)*x_ref[0]**o+(0.569844332841)*x_ref[0]+(0.30469482092)*x_ref[1]**o+(0.18820848487)*x_ref[1]
        ref[2,0,0,0]=(0.0726883565313)*x_ref[0]**o+(-0.151895019503)*x_ref[0]+(-0.566412947745)*x_ref[1]**o+(0.605833954559)*x_ref[1]
        ref[2,0,0,1]=(-0.759348816055)*x_ref[0]**o+(0.203730579434)*x_ref[0]+(-0.306267994)*x_ref[1]**o+(-0.121655396311)*x_ref[1]
        ref[2,0,1,0]=(-0.529988368349)*x_ref[0]**o+(0.265126908621)*x_ref[0]+(-0.127232300402)*x_ref[1]**o+(-0.711018521148)*x_ref[1]
        ref[2,0,1,1]=(0.367616519674)*x_ref[0]**o+(-0.856828909791)*x_ref[0]+(-0.988581037916)*x_ref[1]**o+(0.236273236567)*x_ref[1]
        ref[2,0,2,0]=(0.980896603038)*x_ref[0]**o+(0.0101391252514)*x_ref[0]+(-0.734647631536)*x_ref[1]**o+(-0.298628688073)*x_ref[1]
        ref[2,0,2,1]=(-0.876812266906)*x_ref[0]**o+(-0.0545067479612)*x_ref[0]+(-0.752772584824)*x_ref[1]**o+(-0.797225970025)*x_ref[1]
        ref[2,1,0,0]=(0.9776767796)*x_ref[0]**o+(-0.381948430876)*x_ref[0]+(0.705224317124)*x_ref[1]**o+(0.431609349218)*x_ref[1]
        ref[2,1,0,1]=(0.0282522834733)*x_ref[0]**o+(-0.519316841837)*x_ref[0]+(-0.898831067576)*x_ref[1]**o+(0.491570698382)*x_ref[1]
        ref[2,1,1,0]=(-0.874077440763)*x_ref[0]**o+(0.0050283385939)*x_ref[0]+(-0.45400999477)*x_ref[1]**o+(-0.823942392997)*x_ref[1]
        ref[2,1,1,1]=(0.705592988895)*x_ref[0]**o+(0.545242748068)*x_ref[0]+(-0.0240067342318)*x_ref[1]**o+(-0.764691278707)*x_ref[1]
        ref[2,1,2,0]=(0.214087039013)*x_ref[0]**o+(0.198346865884)*x_ref[0]+(0.195275868007)*x_ref[1]**o+(-0.615146376705)*x_ref[1]
        ref[2,1,2,1]=(0.819578786211)*x_ref[0]**o+(0.238162773292)*x_ref[0]+(-0.488387775705)*x_ref[1]**o+(0.143425392477)*x_ref[1]
        ref[2,2,0,0]=(0.41469049634)*x_ref[0]**o+(-0.870291745289)*x_ref[0]+(-0.978682320238)*x_ref[1]**o+(0.315825060763)*x_ref[1]
        ref[2,2,0,1]=(-0.725231531597)*x_ref[0]**o+(-0.583383717934)*x_ref[0]+(0.677865725085)*x_ref[1]**o+(0.590055042471)*x_ref[1]
        ref[2,2,1,0]=(-0.827316408906)*x_ref[0]**o+(0.152995858286)*x_ref[0]+(-0.864980658862)*x_ref[1]**o+(-0.267986836659)*x_ref[1]
        ref[2,2,1,1]=(-0.333272933954)*x_ref[0]**o+(0.339541494142)*x_ref[0]+(0.982274574789)*x_ref[1]**o+(0.488564854285)*x_ref[1]
        ref[2,2,2,0]=(0.433303448225)*x_ref[0]**o+(0.325824460514)*x_ref[0]+(-0.418818415293)*x_ref[1]**o+(0.884475442413)*x_ref[1]
        ref[2,2,2,1]=(-0.692651589533)*x_ref[0]**o+(0.85015849795)*x_ref[0]+(-0.683352003673)*x_ref[1]**o+(0.458991254181)*x_ref[1]
        ref[2,3,0,0]=(-0.754234631384)*x_ref[0]**o+(0.240243331561)*x_ref[0]+(0.545360646104)*x_ref[1]**o+(-0.960857789697)*x_ref[1]
        ref[2,3,0,1]=(0.911046185694)*x_ref[0]**o+(-0.909178154095)*x_ref[0]+(-0.330132665955)*x_ref[1]**o+(-0.978660867939)*x_ref[1]
        ref[2,3,1,0]=(0.88161349743)*x_ref[0]**o+(0.0812270265162)*x_ref[0]+(-0.584074638899)*x_ref[1]**o+(-0.309263951959)*x_ref[1]
        ref[2,3,1,1]=(-0.59736682275)*x_ref[0]**o+(-0.176681221389)*x_ref[0]+(0.750820855296)*x_ref[1]**o+(0.977121067295)*x_ref[1]
        ref[2,3,2,0]=(-0.926460632737)*x_ref[0]**o+(-0.465577825838)*x_ref[0]+(0.577728110369)*x_ref[1]**o+(-0.973296874179)*x_ref[1]
        ref[2,3,2,1]=(-0.959292420516)*x_ref[0]**o+(-0.173671029796)*x_ref[0]+(0.644855220567)*x_ref[1]**o+(-0.659401238679)*x_ref[1]
        ref[2,4,0,0]=(-0.185887734162)*x_ref[0]**o+(-0.184924703674)*x_ref[0]+(0.12829518222)*x_ref[1]**o+(-0.95529948598)*x_ref[1]
        ref[2,4,0,1]=(-0.222082370982)*x_ref[0]**o+(0.480067405868)*x_ref[0]+(0.737491570657)*x_ref[1]**o+(-0.0505388125547)*x_ref[1]
        ref[2,4,1,0]=(-0.759488182696)*x_ref[0]**o+(-0.209420047186)*x_ref[0]+(-0.257523848658)*x_ref[1]**o+(-0.88701477207)*x_ref[1]
        ref[2,4,1,1]=(0.549919898662)*x_ref[0]**o+(-0.204942654253)*x_ref[0]+(0.237781682077)*x_ref[1]**o+(-0.398029223748)*x_ref[1]
        ref[2,4,2,0]=(-0.429624077152)*x_ref[0]**o+(0.477737955149)*x_ref[0]+(0.150704189177)*x_ref[1]**o+(-0.651247641141)*x_ref[1]
        ref[2,4,2,1]=(-0.894705558322)*x_ref[0]**o+(-0.437755159653)*x_ref[0]+(-0.984539083926)*x_ref[1]**o+(0.771869802974)*x_ref[1]
        ref[3,0,0,0]=(0.613531927806)*x_ref[0]**o+(-0.262670681917)*x_ref[0]+(0.272118391563)*x_ref[1]**o+(-0.911450635138)*x_ref[1]
        ref[3,0,0,1]=(0.0749962421787)*x_ref[0]**o+(-0.280125180177)*x_ref[0]+(0.535812839231)*x_ref[1]**o+(-0.898511642603)*x_ref[1]
        ref[3,0,1,0]=(-0.493682703273)*x_ref[0]**o+(0.564806921005)*x_ref[0]+(-0.424211826783)*x_ref[1]**o+(0.955398127065)*x_ref[1]
        ref[3,0,1,1]=(-0.208922295917)*x_ref[0]**o+(0.552971608191)*x_ref[0]+(-0.482834655473)*x_ref[1]**o+(0.916990053757)*x_ref[1]
        ref[3,0,2,0]=(0.225448292089)*x_ref[0]**o+(-0.508992793538)*x_ref[0]+(-0.301030316256)*x_ref[1]**o+(0.234012186162)*x_ref[1]
        ref[3,0,2,1]=(0.173694397805)*x_ref[0]**o+(-0.0436667050758)*x_ref[0]+(-0.391980496474)*x_ref[1]**o+(0.183288325597)*x_ref[1]
        ref[3,1,0,0]=(-0.078216910574)*x_ref[0]**o+(-0.993944069614)*x_ref[0]+(-0.250488879366)*x_ref[1]**o+(0.648115982978)*x_ref[1]
        ref[3,1,0,1]=(-0.346993095448)*x_ref[0]**o+(0.630491595383)*x_ref[0]+(0.128199747925)*x_ref[1]**o+(-0.681982197159)*x_ref[1]
        ref[3,1,1,0]=(0.0360684354752)*x_ref[0]**o+(-0.8297568105)*x_ref[0]+(0.122927048201)*x_ref[1]**o+(0.0448718398101)*x_ref[1]
        ref[3,1,1,1]=(-0.603474854473)*x_ref[0]**o+(-0.435029027744)*x_ref[0]+(0.632649021375)*x_ref[1]**o+(0.414362202135)*x_ref[1]
        ref[3,1,2,0]=(-0.912833626509)*x_ref[0]**o+(-0.828668448463)*x_ref[0]+(-0.754484607948)*x_ref[1]**o+(0.503101904128)*x_ref[1]
        ref[3,1,2,1]=(0.00649229353309)*x_ref[0]**o+(-0.96124954377)*x_ref[0]+(0.648294131558)*x_ref[1]**o+(-0.957122691907)*x_ref[1]
        ref[3,2,0,0]=(0.476913272421)*x_ref[0]**o+(0.0878038200542)*x_ref[0]+(-0.516953661957)*x_ref[1]**o+(0.0134540618391)*x_ref[1]
        ref[3,2,0,1]=(0.3748519133)*x_ref[0]**o+(-0.970356842975)*x_ref[0]+(-0.874984768114)*x_ref[1]**o+(-0.0950957723387)*x_ref[1]
        ref[3,2,1,0]=(-0.711688301522)*x_ref[0]**o+(-0.20444931953)*x_ref[0]+(0.601925297065)*x_ref[1]**o+(0.939713142428)*x_ref[1]
        ref[3,2,1,1]=(-0.869179475562)*x_ref[0]**o+(-0.771823270124)*x_ref[0]+(0.0447051413494)*x_ref[1]**o+(0.769181827448)*x_ref[1]
        ref[3,2,2,0]=(0.341502469812)*x_ref[0]**o+(-0.892625503786)*x_ref[0]+(-0.683764736135)*x_ref[1]**o+(-0.208342034139)*x_ref[1]
        ref[3,2,2,1]=(-0.876208883831)*x_ref[0]**o+(-0.705748793184)*x_ref[0]+(0.92260186528)*x_ref[1]**o+(-0.437547481021)*x_ref[1]
        ref[3,3,0,0]=(-0.387117059014)*x_ref[0]**o+(-0.30316333118)*x_ref[0]+(-0.107906971687)*x_ref[1]**o+(0.27027380043)*x_ref[1]
        ref[3,3,0,1]=(0.760401124102)*x_ref[0]**o+(0.0392442137961)*x_ref[0]+(0.527403460784)*x_ref[1]**o+(0.49506478844)*x_ref[1]
        ref[3,3,1,0]=(-0.797802219992)*x_ref[0]**o+(-0.718681383351)*x_ref[0]+(0.723820891624)*x_ref[1]**o+(0.851058726763)*x_ref[1]
        ref[3,3,1,1]=(-0.477124467307)*x_ref[0]**o+(-0.912639051937)*x_ref[0]+(-0.653399611549)*x_ref[1]**o+(-0.527816963598)*x_ref[1]
        ref[3,3,2,0]=(-0.375620015065)*x_ref[0]**o+(0.713563556042)*x_ref[0]+(0.592214060154)*x_ref[1]**o+(-0.663713190063)*x_ref[1]
        ref[3,3,2,1]=(0.13839650904)*x_ref[0]**o+(-0.894351932455)*x_ref[0]+(0.130542033693)*x_ref[1]**o+(-0.420895602544)*x_ref[1]
        ref[3,4,0,0]=(-0.438194955853)*x_ref[0]**o+(-0.119296803116)*x_ref[0]+(0.683500326777)*x_ref[1]**o+(-0.226368973557)*x_ref[1]
        ref[3,4,0,1]=(-0.232920353049)*x_ref[0]**o+(-0.35856926089)*x_ref[0]+(0.882231830184)*x_ref[1]**o+(-0.000957747821531)*x_ref[1]
        ref[3,4,1,0]=(-0.0939581521731)*x_ref[0]**o+(0.831315092139)*x_ref[0]+(0.042454046079)*x_ref[1]**o+(0.791725586274)*x_ref[1]
        ref[3,4,1,1]=(-0.328628704689)*x_ref[0]**o+(0.434271457185)*x_ref[0]+(0.632929764639)*x_ref[1]**o+(-0.526043410013)*x_ref[1]
        ref[3,4,2,0]=(0.429887737881)*x_ref[0]**o+(0.828275504726)*x_ref[0]+(-0.199204123672)*x_ref[1]**o+(-0.363782496284)*x_ref[1]
        ref[3,4,2,1]=(-0.446866424171)*x_ref[0]**o+(-0.204025885117)*x_ref[0]+(-0.899560130794)*x_ref[1]**o+(0.184518622033)*x_ref[1]
      else:
        s[0,0,0,0]=(0.651971717604)*x[0]**o+(-0.781641304094)*x[0]+(0.535792272807)*x[1]**o+(-0.661083912872)*x[1]+(-0.311952439342)*x[2]**o+(-0.952641714536)*x[2]
        s[0,0,0,1]=(-0.480185392802)*x[0]**o+(-0.957422590142)*x[0]+(-0.25301116344)*x[1]**o+(0.865442078084)*x[1]+(0.836322905432)*x[2]**o+(-0.835302193996)*x[2]
        s[0,0,1,0]=(0.163795654451)*x[0]**o+(0.268353643157)*x[0]+(0.298652899832)*x[1]**o+(0.567324820252)*x[1]+(-0.265398363574)*x[2]**o+(0.204860398351)*x[2]
        s[0,0,1,1]=(0.538423766786)*x[0]**o+(-0.01940169351)*x[0]+(0.294103797101)*x[1]**o+(0.132862986285)*x[1]+(0.995163905667)*x[2]**o+(-0.119153668927)*x[2]
        s[0,0,2,0]=(-0.892916012976)*x[0]**o+(-0.557129189047)*x[0]+(0.0705777480285)*x[1]**o+(0.518661097094)*x[1]+(-0.454082645299)*x[2]**o+(0.632837820367)*x[2]
        s[0,0,2,1]=(0.71834426356)*x[0]**o+(-0.688104502488)*x[0]+(-0.285809102801)*x[1]**o+(0.552799075425)*x[1]+(0.116702254748)*x[2]**o+(-0.34761826549)*x[2]
        s[0,1,0,0]=(0.293581495968)*x[0]**o+(0.201727375498)*x[0]+(-0.416610275343)*x[1]**o+(0.607460451032)*x[1]+(0.0343669057747)*x[2]**o+(-0.530601755969)*x[2]
        s[0,1,0,1]=(0.880681535309)*x[0]**o+(-0.953246533735)*x[0]+(-0.424031774258)*x[1]**o+(-0.52019867795)*x[1]+(-0.250435071705)*x[2]**o+(-0.168661740641)*x[2]
        s[0,1,1,0]=(-0.495577302252)*x[0]**o+(0.792099185033)*x[0]+(0.294589995512)*x[1]**o+(-0.734902995766)*x[1]+(-0.359137354522)*x[2]**o+(-0.0317334336381)*x[2]
        s[0,1,1,1]=(-0.843957318456)*x[0]**o+(0.29759734816)*x[0]+(-0.954015934149)*x[1]**o+(-0.270649016792)*x[1]+(-0.364089303705)*x[2]**o+(-0.396696196833)*x[2]
        s[0,1,2,0]=(0.411843103656)*x[0]**o+(-0.806857649219)*x[0]+(0.0998406396629)*x[1]**o+(0.426060184907)*x[1]+(0.839569241907)*x[2]**o+(-0.22764839333)*x[2]
        s[0,1,2,1]=(-0.970273259116)*x[0]**o+(-0.983160602162)*x[0]+(0.418163962338)*x[1]**o+(-0.110768668492)*x[1]+(0.328089646862)*x[2]**o+(-0.270172456473)*x[2]
        s[0,2,0,0]=(0.683233964281)*x[0]**o+(0.188111552757)*x[0]+(-0.951071798504)*x[1]**o+(-0.0330921159172)*x[1]+(-0.64270161977)*x[2]**o+(-0.499615377327)*x[2]
        s[0,2,0,1]=(0.36794958012)*x[0]**o+(0.398010040747)*x[0]+(-0.309445952639)*x[1]**o+(-0.322623662412)*x[1]+(-0.439692584231)*x[2]**o+(0.00620886333611)*x[2]
        s[0,2,1,0]=(0.588715821634)*x[0]**o+(-0.163253786484)*x[0]+(-0.619853219098)*x[1]**o+(-0.163020284476)*x[1]+(-0.781500850817)*x[2]**o+(-0.13622778235)*x[2]
        s[0,2,1,1]=(0.610025540474)*x[0]**o+(-0.262832084419)*x[0]+(0.109453975557)*x[1]**o+(-0.0910626806898)*x[1]+(0.720567359491)*x[2]**o+(0.221728266865)*x[2]
        s[0,2,2,0]=(-0.967888175709)*x[0]**o+(0.841198387559)*x[0]+(0.0942188974625)*x[1]**o+(0.056715320399)*x[1]+(0.220387799569)*x[2]**o+(-0.991352339366)*x[2]
        s[0,2,2,1]=(-0.243144875987)*x[0]**o+(-0.101607712773)*x[0]+(0.0890556148157)*x[1]**o+(0.968975809324)*x[1]+(-0.680428099066)*x[2]**o+(-0.203127186894)*x[2]
        s[0,3,0,0]=(-0.97698323555)*x[0]**o+(0.723899953965)*x[0]+(0.168932911211)*x[1]**o+(0.210562969565)*x[1]+(0.0241740170862)*x[2]**o+(-0.443543352786)*x[2]
        s[0,3,0,1]=(0.273706631775)*x[0]**o+(-0.064335272978)*x[0]+(-0.289390463262)*x[1]**o+(-0.0911403229125)*x[1]+(0.380520540499)*x[2]**o+(0.930362632813)*x[2]
        s[0,3,1,0]=(0.108247452775)*x[0]**o+(0.670880216846)*x[0]+(0.743592674018)*x[1]**o+(0.110450181904)*x[1]+(0.633556946694)*x[2]**o+(-0.526137817259)*x[2]
        s[0,3,1,1]=(0.770308192835)*x[0]**o+(0.304435220496)*x[0]+(-0.851453559716)*x[1]**o+(-0.549151404489)*x[1]+(-0.512295072491)*x[2]**o+(-0.896216869911)*x[2]
        s[0,3,2,0]=(-0.879389228597)*x[0]**o+(-0.98421803535)*x[0]+(0.310411565504)*x[1]**o+(-0.833601944827)*x[1]+(0.520140917807)*x[2]**o+(0.409319548582)*x[2]
        s[0,3,2,1]=(0.673800312082)*x[0]**o+(0.307843370867)*x[0]+(0.0435576949618)*x[1]**o+(-0.362422816889)*x[1]+(0.96314375221)*x[2]**o+(-0.896763989426)*x[2]
        s[0,4,0,0]=(-0.569321969384)*x[0]**o+(-0.545868340066)*x[0]+(0.951980474226)*x[1]**o+(-0.482241368075)*x[1]+(-0.255170213259)*x[2]**o+(0.841268809686)*x[2]
        s[0,4,0,1]=(-0.837681312588)*x[0]**o+(-0.458077368372)*x[0]+(0.997003677283)*x[1]**o+(0.762129779023)*x[1]+(0.0952433046108)*x[2]**o+(-0.781885974532)*x[2]
        s[0,4,1,0]=(-0.856181540078)*x[0]**o+(-0.841612660714)*x[0]+(0.544038400285)*x[1]**o+(-0.284909634097)*x[1]+(0.11260106121)*x[2]**o+(-0.39480742262)*x[2]
        s[0,4,1,1]=(0.356082675669)*x[0]**o+(0.917688541228)*x[0]+(-0.176959383957)*x[1]**o+(0.416653474575)*x[1]+(-0.662930023412)*x[2]**o+(-0.73010168013)*x[2]
        s[0,4,2,0]=(-0.763454917764)*x[0]**o+(0.302162768082)*x[0]+(0.143386232133)*x[1]**o+(0.91616734981)*x[1]+(-0.813201390394)*x[2]**o+(-0.857982436099)*x[2]
        s[0,4,2,1]=(0.835769822382)*x[0]**o+(0.167610186558)*x[0]+(0.2331699547)*x[1]**o+(0.0015459879901)*x[1]+(0.167167023037)*x[2]**o+(0.550280560723)*x[2]
        s[1,0,0,0]=(-0.217157755228)*x[0]**o+(0.534373560701)*x[0]+(-0.883064731078)*x[1]**o+(0.731486045177)*x[1]+(0.819268782653)*x[2]**o+(0.787662641797)*x[2]
        s[1,0,0,1]=(-0.487819762644)*x[0]**o+(-0.430212238027)*x[0]+(-0.56911636092)*x[1]**o+(-0.728659093202)*x[1]+(0.455173919727)*x[2]**o+(-0.636506423308)*x[2]
        s[1,0,1,0]=(0.900337000667)*x[0]**o+(0.299528369342)*x[0]+(0.681918775341)*x[1]**o+(0.370100893068)*x[1]+(-0.587580308922)*x[2]**o+(0.485274084829)*x[2]
        s[1,0,1,1]=(0.442748593533)*x[0]**o+(-0.0292501463909)*x[0]+(0.551554968843)*x[1]**o+(-0.318313953949)*x[1]+(-0.458151041817)*x[2]**o+(-0.625611121823)*x[2]
        s[1,0,2,0]=(-0.723203702271)*x[0]**o+(0.101313366226)*x[0]+(-0.548133659891)*x[1]**o+(-0.392729656892)*x[1]+(0.46317338606)*x[2]**o+(0.235572773174)*x[2]
        s[1,0,2,1]=(-0.445472509581)*x[0]**o+(-0.552381494615)*x[0]+(0.0815798929496)*x[1]**o+(-0.595946577234)*x[1]+(0.878280129666)*x[2]**o+(-0.274608443481)*x[2]
        s[1,1,0,0]=(0.543392744962)*x[0]**o+(0.79638586218)*x[0]+(0.435895934157)*x[1]**o+(-0.0134442947039)*x[1]+(-0.23586652076)*x[2]**o+(0.120783361228)*x[2]
        s[1,1,0,1]=(-0.50551104509)*x[0]**o+(-0.578612752272)*x[0]+(-0.595688781895)*x[1]**o+(-0.411446622857)*x[1]+(-0.729076372085)*x[2]**o+(0.61997008872)*x[2]
        s[1,1,1,0]=(-0.25176939418)*x[0]**o+(0.0192820953571)*x[0]+(-0.365827108002)*x[1]**o+(-0.58048375365)*x[1]+(-0.109602718093)*x[2]**o+(-0.764760324931)*x[2]
        s[1,1,1,1]=(0.333747531705)*x[0]**o+(-0.975460281758)*x[0]+(-0.263662359116)*x[1]**o+(0.833048103253)*x[1]+(-0.403378941533)*x[2]**o+(0.99255510422)*x[2]
        s[1,1,2,0]=(-0.643575493982)*x[0]**o+(0.408164900892)*x[0]+(0.330669259534)*x[1]**o+(0.696238788141)*x[1]+(0.8187478709)*x[2]**o+(0.269447281859)*x[2]
        s[1,1,2,1]=(0.0975010793595)*x[0]**o+(-0.266707476957)*x[0]+(-0.627484959323)*x[1]**o+(0.189636667701)*x[1]+(0.0634608353528)*x[2]**o+(0.425595191628)*x[2]
        s[1,2,0,0]=(0.144863702314)*x[0]**o+(0.62877495084)*x[0]+(-0.937663622583)*x[1]**o+(0.643925666447)*x[1]+(0.0752876047081)*x[2]**o+(-0.0278560665987)*x[2]
        s[1,2,0,1]=(0.825937768085)*x[0]**o+(-0.350938240562)*x[0]+(-0.316596763726)*x[1]**o+(0.248951654728)*x[1]+(-0.4533348882)*x[2]**o+(-0.840614727297)*x[2]
        s[1,2,1,0]=(0.235071855147)*x[0]**o+(0.818758983424)*x[0]+(-0.962091319923)*x[1]**o+(-0.0687829219781)*x[1]+(0.256236142265)*x[2]**o+(-0.236383412975)*x[2]
        s[1,2,1,1]=(-0.674935027618)*x[0]**o+(0.506392204682)*x[0]+(-0.891135534099)*x[1]**o+(0.200562756224)*x[1]+(-0.509385776169)*x[2]**o+(-0.709115810434)*x[2]
        s[1,2,2,0]=(0.476890911154)*x[0]**o+(-0.716988939897)*x[0]+(-0.708930094063)*x[1]**o+(-0.221417730773)*x[1]+(-0.0530682866975)*x[2]**o+(-0.933479841665)*x[2]
        s[1,2,2,1]=(0.912189530094)*x[0]**o+(-0.287403816104)*x[0]+(-0.755698083102)*x[1]**o+(-0.996716444257)*x[1]+(-0.836456434626)*x[2]**o+(-0.463710726041)*x[2]
        s[1,3,0,0]=(0.253691639919)*x[0]**o+(-0.279433210688)*x[0]+(-0.117635686336)*x[1]**o+(0.790073676599)*x[1]+(-0.559215280716)*x[2]**o+(-0.103190062272)*x[2]
        s[1,3,0,1]=(0.0970821498373)*x[0]**o+(0.814008398347)*x[0]+(0.583184329882)*x[1]**o+(-0.598484860263)*x[1]+(-0.00216176134509)*x[2]**o+(0.14889662547)*x[2]
        s[1,3,1,0]=(-0.730045080625)*x[0]**o+(-0.746336558552)*x[0]+(-0.995897359987)*x[1]**o+(-0.70582565406)*x[1]+(0.36600954398)*x[2]**o+(-0.0843877537778)*x[2]
        s[1,3,1,1]=(-0.714997081413)*x[0]**o+(-0.138823664507)*x[0]+(0.969108046142)*x[1]**o+(-0.201514397247)*x[1]+(-0.438601765508)*x[2]**o+(-0.984456457681)*x[2]
        s[1,3,2,0]=(-0.577419600949)*x[0]**o+(-0.821514372871)*x[0]+(-0.179395810292)*x[1]**o+(-0.904303143215)*x[1]+(0.27616964799)*x[2]**o+(-0.272088845238)*x[2]
        s[1,3,2,1]=(0.31325269807)*x[0]**o+(-0.0134619122423)*x[0]+(-0.641786875798)*x[1]**o+(-0.568855591797)*x[1]+(0.600302453067)*x[2]**o+(0.830044865545)*x[2]
        s[1,4,0,0]=(-0.791605536671)*x[0]**o+(-0.578889866035)*x[0]+(-0.22659053834)*x[1]**o+(0.837093316131)*x[1]+(0.499953644961)*x[2]**o+(-0.289930435498)*x[2]
        s[1,4,0,1]=(0.819284096253)*x[0]**o+(0.272518944584)*x[0]+(0.199276568134)*x[1]**o+(0.132962435684)*x[1]+(-0.896986377734)*x[2]**o+(0.775604010154)*x[2]
        s[1,4,1,0]=(-0.450392824249)*x[0]**o+(0.376154164409)*x[0]+(0.199853639816)*x[1]**o+(-0.679551176507)*x[1]+(0.565506686048)*x[2]**o+(0.391489667485)*x[2]
        s[1,4,1,1]=(-0.0905057426998)*x[0]**o+(-0.749252305812)*x[0]+(-0.21131989398)*x[1]**o+(-0.671560452908)*x[1]+(-0.760505475283)*x[2]**o+(0.763692313655)*x[2]
        s[1,4,2,0]=(-0.219636883594)*x[0]**o+(-0.380991913771)*x[0]+(0.831879211521)*x[1]**o+(-0.0686682700553)*x[1]+(0.843832009742)*x[2]**o+(-0.86627759217)*x[2]
        s[1,4,2,1]=(-0.332685504588)*x[0]**o+(0.589527397306)*x[0]+(0.624226689395)*x[1]**o+(0.0515479774619)*x[1]+(0.156657102499)*x[2]**o+(0.465889992773)*x[2]
        s[2,0,0,0]=(0.250467292043)*x[0]**o+(-0.0658874324582)*x[0]+(-0.0709052912862)*x[1]**o+(0.700837318576)*x[1]+(0.650887276648)*x[2]**o+(0.816187438816)*x[2]
        s[2,0,0,1]=(-0.697069910038)*x[0]**o+(-0.287284875865)*x[0]+(-0.972646385975)*x[1]**o+(-0.430570334599)*x[1]+(0.400826092127)*x[2]**o+(-0.484530830951)*x[2]
        s[2,0,1,0]=(0.58433575578)*x[0]**o+(-0.114741976876)*x[0]+(-0.205615141565)*x[1]**o+(-0.476488876624)*x[1]+(0.323174445651)*x[2]**o+(-0.152550510154)*x[2]
        s[2,0,1,1]=(0.928306678393)*x[0]**o+(0.104020335724)*x[0]+(-0.0416879184197)*x[1]**o+(0.865779079734)*x[1]+(-0.169672240264)*x[2]**o+(0.118778471217)*x[2]
        s[2,0,2,0]=(0.652779555688)*x[0]**o+(-0.0209488281208)*x[0]+(0.809628464928)*x[1]**o+(-0.85121566767)*x[1]+(-0.845109631338)*x[2]**o+(-0.970252635974)*x[2]
        s[2,0,2,1]=(-0.740845790747)*x[0]**o+(-0.705835368678)*x[0]+(0.26040103782)*x[1]**o+(0.814854175049)*x[1]+(-0.00842496430724)*x[2]**o+(0.192519753618)*x[2]
        s[2,1,0,0]=(0.629347590715)*x[0]**o+(0.922723375982)*x[0]+(-0.430492700295)*x[1]**o+(-0.895795670345)*x[1]+(0.708440378414)*x[2]**o+(-0.155782723618)*x[2]
        s[2,1,0,1]=(-0.478612307362)*x[0]**o+(0.645774615123)*x[0]+(-0.461136871779)*x[1]**o+(0.297927339931)*x[1]+(-0.452011143468)*x[2]**o+(0.267002423863)*x[2]
        s[2,1,1,0]=(-0.0591350600103)*x[0]**o+(-0.252358408576)*x[0]+(-0.0412752075681)*x[1]**o+(0.942736455517)*x[1]+(-0.223363982173)*x[2]**o+(-0.752007969767)*x[2]
        s[2,1,1,1]=(0.197290290824)*x[0]**o+(-0.170028529393)*x[0]+(-0.143011200679)*x[1]**o+(-0.694395961945)*x[1]+(0.430761130021)*x[2]**o+(0.298239563293)*x[2]
        s[2,1,2,0]=(0.258474077793)*x[0]**o+(-0.956041015018)*x[0]+(0.360444161221)*x[1]**o+(-0.969922460117)*x[1]+(0.856255540521)*x[2]**o+(0.108872710157)*x[2]
        s[2,1,2,1]=(-0.694400609963)*x[0]**o+(-0.864737376468)*x[0]+(-0.898839555362)*x[1]**o+(-0.655053934442)*x[1]+(-0.511293723446)*x[2]**o+(-0.204242252526)*x[2]
        s[2,2,0,0]=(-0.427946109181)*x[0]**o+(0.435798407553)*x[0]+(-0.567923694982)*x[1]**o+(0.870896124804)*x[1]+(-0.996434693707)*x[2]**o+(-0.683367112977)*x[2]
        s[2,2,0,1]=(-0.219431769721)*x[0]**o+(-0.302922785317)*x[0]+(-0.0998576564441)*x[1]**o+(0.64263522193)*x[1]+(0.53256637446)*x[2]**o+(-0.00387183821982)*x[2]
        s[2,2,1,0]=(-0.174988349945)*x[0]**o+(0.408056661499)*x[0]+(0.55925945151)*x[1]**o+(-0.647959566858)*x[1]+(0.905755316979)*x[2]**o+(0.691774684766)*x[2]
        s[2,2,1,1]=(-0.882995226725)*x[0]**o+(-0.948874015343)*x[0]+(-0.652506601975)*x[1]**o+(-0.137989382315)*x[1]+(0.665695021939)*x[2]**o+(0.163493199569)*x[2]
        s[2,2,2,0]=(0.666421535029)*x[0]**o+(-0.984765122669)*x[0]+(0.399488949215)*x[1]**o+(-0.99963893587)*x[1]+(0.225552018439)*x[2]**o+(0.430770653325)*x[2]
        s[2,2,2,1]=(-0.0284528199834)*x[0]**o+(0.0421195684748)*x[0]+(-0.399757974604)*x[1]**o+(0.305895209328)*x[1]+(0.190920234352)*x[2]**o+(-0.0990327186702)*x[2]
        s[2,3,0,0]=(-0.565899348734)*x[0]**o+(0.636275927936)*x[0]+(0.682710455722)*x[1]**o+(-0.434837962369)*x[1]+(0.252457745239)*x[2]**o+(-0.709529514057)*x[2]
        s[2,3,0,1]=(0.0113021999515)*x[0]**o+(0.451528226044)*x[0]+(-0.705424064371)*x[1]**o+(0.500516985463)*x[1]+(-0.528750586385)*x[2]**o+(0.998214035471)*x[2]
        s[2,3,1,0]=(-0.879296343578)*x[0]**o+(-0.724012783173)*x[0]+(0.51105902172)*x[1]**o+(-0.840460578985)*x[1]+(0.222130858003)*x[2]**o+(0.998500581881)*x[2]
        s[2,3,1,1]=(0.478703623115)*x[0]**o+(-0.94935681973)*x[0]+(-0.609714878079)*x[1]**o+(0.0318988430394)*x[1]+(-0.858395334121)*x[2]**o+(-0.171229770192)*x[2]
        s[2,3,2,0]=(0.0791777006911)*x[0]**o+(-0.306954070098)*x[0]+(-0.182890473769)*x[1]**o+(-0.26384266322)*x[1]+(0.663652835142)*x[2]**o+(0.188229986922)*x[2]
        s[2,3,2,1]=(-0.101910553498)*x[0]**o+(-0.34187147162)*x[0]+(-0.52746320646)*x[1]**o+(-0.891142643574)*x[1]+(-0.84135671258)*x[2]**o+(0.448615749816)*x[2]
        s[2,4,0,0]=(0.928980482137)*x[0]**o+(0.00486879685273)*x[0]+(-0.068529973495)*x[1]**o+(-0.16116665925)*x[1]+(0.247146438634)*x[2]**o+(-0.341715145751)*x[2]
        s[2,4,0,1]=(0.486412050624)*x[0]**o+(-0.929254100014)*x[0]+(0.439267744247)*x[1]**o+(-0.430515713902)*x[1]+(-0.317438643233)*x[2]**o+(-0.0843825737593)*x[2]
        s[2,4,1,0]=(0.0130446306687)*x[0]**o+(0.324310412551)*x[0]+(-0.38112172561)*x[1]**o+(-0.92878637625)*x[1]+(0.32867951654)*x[2]**o+(-0.0315638177916)*x[2]
        s[2,4,1,1]=(-0.254225591431)*x[0]**o+(0.321872838348)*x[0]+(0.179619901508)*x[1]**o+(-0.858713537407)*x[1]+(0.151254493236)*x[2]**o+(-0.340368823543)*x[2]
        s[2,4,2,0]=(-0.0844677926666)*x[0]**o+(0.398017491087)*x[0]+(0.0313455976792)*x[1]**o+(-0.872666391632)*x[1]+(0.301765055045)*x[2]**o+(0.563923225852)*x[2]
        s[2,4,2,1]=(0.814244097177)*x[0]**o+(-0.223957875084)*x[0]+(0.447995085387)*x[1]**o+(0.350953464712)*x[1]+(0.333269027952)*x[2]**o+(-0.941385895584)*x[2]
        s[3,0,0,0]=(0.0789492878053)*x[0]**o+(0.525756966035)*x[0]+(0.87125043555)*x[1]**o+(0.881346392374)*x[1]+(-0.855477786784)*x[2]**o+(-0.991712685096)*x[2]
        s[3,0,0,1]=(-0.0359771254585)*x[0]**o+(0.603218005193)*x[0]+(-0.580729043759)*x[1]**o+(-0.20390195049)*x[1]+(-0.862263281233)*x[2]**o+(-0.196587155144)*x[2]
        s[3,0,1,0]=(0.879750375964)*x[0]**o+(-0.251177194936)*x[0]+(0.191264882539)*x[1]**o+(-0.95611617781)*x[1]+(0.71738769264)*x[2]**o+(-0.691767368606)*x[2]
        s[3,0,1,1]=(-0.761557468157)*x[0]**o+(-0.381655367519)*x[0]+(0.587969140055)*x[1]**o+(0.622246650476)*x[1]+(0.117544678721)*x[2]**o+(-0.576059347687)*x[2]
        s[3,0,2,0]=(-0.587802439618)*x[0]**o+(-0.981432433894)*x[0]+(-0.758496301974)*x[1]**o+(0.656105955022)*x[1]+(-0.689107562745)*x[2]**o+(0.267295196672)*x[2]
        s[3,0,2,1]=(-0.0524224100533)*x[0]**o+(-0.925763321074)*x[0]+(0.218186590694)*x[1]**o+(-0.598934300523)*x[1]+(0.22756773)*x[2]**o+(-0.770210327792)*x[2]
        s[3,1,0,0]=(0.922635777646)*x[0]**o+(-0.278736876358)*x[0]+(0.0631653424614)*x[1]**o+(-0.356448496856)*x[1]+(-0.609671239719)*x[2]**o+(0.189678422301)*x[2]
        s[3,1,0,1]=(-0.509070180487)*x[0]**o+(-0.250043376376)*x[0]+(-0.0697248208177)*x[1]**o+(0.531153766941)*x[1]+(0.240916012233)*x[2]**o+(0.391353341194)*x[2]
        s[3,1,1,0]=(-0.514764390151)*x[0]**o+(-0.347397972881)*x[0]+(0.138729357858)*x[1]**o+(-0.30135683112)*x[1]+(0.541893745985)*x[2]**o+(0.414478801003)*x[2]
        s[3,1,1,1]=(0.866088916369)*x[0]**o+(-0.96729572537)*x[0]+(0.824735563587)*x[1]**o+(-0.479859785014)*x[1]+(-0.175784746993)*x[2]**o+(-0.998407148967)*x[2]
        s[3,1,2,0]=(0.327087092534)*x[0]**o+(0.171000747621)*x[0]+(-0.33910895269)*x[1]**o+(0.349474180167)*x[1]+(0.561935192081)*x[2]**o+(0.358380998288)*x[2]
        s[3,1,2,1]=(0.35646522102)*x[0]**o+(-0.912130691808)*x[0]+(0.796756777424)*x[1]**o+(-0.91884834291)*x[1]+(0.956454722923)*x[2]**o+(0.462859602491)*x[2]
        s[3,2,0,0]=(-0.849637152897)*x[0]**o+(0.353228393186)*x[0]+(-0.725757036845)*x[1]**o+(0.590369089609)*x[1]+(-0.280432657156)*x[2]**o+(0.142413703594)*x[2]
        s[3,2,0,1]=(-0.640766746158)*x[0]**o+(-0.454583129018)*x[0]+(-0.616394565949)*x[1]**o+(0.724166900329)*x[1]+(-0.748682704671)*x[2]**o+(0.934076771628)*x[2]
        s[3,2,1,0]=(-0.824548065846)*x[0]**o+(0.40250159164)*x[0]+(0.763315748823)*x[1]**o+(0.255878553659)*x[1]+(0.872676778336)*x[2]**o+(-0.100430227782)*x[2]
        s[3,2,1,1]=(-0.610248210467)*x[0]**o+(0.160976829271)*x[0]+(-0.384236718764)*x[1]**o+(0.657119635861)*x[1]+(0.161695416084)*x[2]**o+(0.505997006976)*x[2]
        s[3,2,2,0]=(0.220409736038)*x[0]**o+(0.398197530033)*x[0]+(-0.288242008883)*x[1]**o+(0.0832866402504)*x[1]+(0.761030964351)*x[2]**o+(0.751986795183)*x[2]
        s[3,2,2,1]=(-0.396845385157)*x[0]**o+(-0.0792538647037)*x[0]+(-0.327181444898)*x[1]**o+(-0.810682039031)*x[1]+(0.260671755637)*x[2]**o+(-0.242407958043)*x[2]
        s[3,3,0,0]=(-0.663937889328)*x[0]**o+(0.587545393032)*x[0]+(0.210910229094)*x[1]**o+(0.500419329566)*x[1]+(0.202539078075)*x[2]**o+(-0.623007416272)*x[2]
        s[3,3,0,1]=(0.0375980252743)*x[0]**o+(-0.435705054083)*x[0]+(-0.660885021014)*x[1]**o+(-0.0332748037874)*x[1]+(-0.832274220584)*x[2]**o+(0.805002616286)*x[2]
        s[3,3,1,0]=(0.395522385265)*x[0]**o+(0.901865335207)*x[0]+(0.016322521344)*x[1]**o+(-0.532212956041)*x[1]+(0.190751397835)*x[2]**o+(0.234716440942)*x[2]
        s[3,3,1,1]=(0.257090076986)*x[0]**o+(-0.169468215187)*x[0]+(-0.0641337112558)*x[1]**o+(-0.674363367639)*x[1]+(0.13633053895)*x[2]**o+(-0.741438377514)*x[2]
        s[3,3,2,0]=(0.984088274745)*x[0]**o+(-0.669313378065)*x[0]+(0.899763931565)*x[1]**o+(-0.874652895499)*x[1]+(-0.172620100734)*x[2]**o+(-0.635666769897)*x[2]
        s[3,3,2,1]=(0.693112828433)*x[0]**o+(-0.99231490163)*x[0]+(0.525842419539)*x[1]**o+(0.268799565406)*x[1]+(-0.473532039244)*x[2]**o+(0.50555338295)*x[2]
        s[3,4,0,0]=(0.780678215778)*x[0]**o+(0.56287435306)*x[0]+(0.322651378857)*x[1]**o+(0.268655956157)*x[1]+(0.438312596985)*x[2]**o+(0.175512199841)*x[2]
        s[3,4,0,1]=(-0.510399313775)*x[0]**o+(-0.63898491047)*x[0]+(-0.765120717001)*x[1]**o+(0.45570492579)*x[1]+(-0.148930444588)*x[2]**o+(0.458400743585)*x[2]
        s[3,4,1,0]=(-0.629456511378)*x[0]**o+(-0.0956526447002)*x[0]+(0.223599370201)*x[1]**o+(-0.689787947305)*x[1]+(0.834792179813)*x[2]**o+(0.413903455599)*x[2]
        s[3,4,1,1]=(0.89648905371)*x[0]**o+(0.420890001861)*x[0]+(-0.890077307972)*x[1]**o+(0.424785046869)*x[1]+(0.84365123356)*x[2]**o+(0.56417731826)*x[2]
        s[3,4,2,0]=(0.853620031631)*x[0]**o+(-0.719293879422)*x[0]+(-0.811007794139)*x[1]**o+(-0.236043157925)*x[1]+(0.130830104273)*x[2]**o+(-0.554431879475)*x[2]
        s[3,4,2,1]=(-0.555616431129)*x[0]**o+(-0.900929040274)*x[0]+(0.773492786481)*x[1]**o+(-0.445322579241)*x[1]+(0.270832172179)*x[2]**o+(-0.716484089692)*x[2]
        ref[0,0,0,0]=(0.651971717604)*x_ref[0]**o+(-0.781641304094)*x_ref[0]+(0.535792272807)*x_ref[1]**o+(-0.661083912872)*x_ref[1]+(-0.311952439342)*x_ref[2]**o+(-0.952641714536)*x_ref[2]
        ref[0,0,0,1]=(-0.480185392802)*x_ref[0]**o+(-0.957422590142)*x_ref[0]+(-0.25301116344)*x_ref[1]**o+(0.865442078084)*x_ref[1]+(0.836322905432)*x_ref[2]**o+(-0.835302193996)*x_ref[2]
        ref[0,0,1,0]=(0.163795654451)*x_ref[0]**o+(0.268353643157)*x_ref[0]+(0.298652899832)*x_ref[1]**o+(0.567324820252)*x_ref[1]+(-0.265398363574)*x_ref[2]**o+(0.204860398351)*x_ref[2]
        ref[0,0,1,1]=(0.538423766786)*x_ref[0]**o+(-0.01940169351)*x_ref[0]+(0.294103797101)*x_ref[1]**o+(0.132862986285)*x_ref[1]+(0.995163905667)*x_ref[2]**o+(-0.119153668927)*x_ref[2]
        ref[0,0,2,0]=(-0.892916012976)*x_ref[0]**o+(-0.557129189047)*x_ref[0]+(0.0705777480285)*x_ref[1]**o+(0.518661097094)*x_ref[1]+(-0.454082645299)*x_ref[2]**o+(0.632837820367)*x_ref[2]
        ref[0,0,2,1]=(0.71834426356)*x_ref[0]**o+(-0.688104502488)*x_ref[0]+(-0.285809102801)*x_ref[1]**o+(0.552799075425)*x_ref[1]+(0.116702254748)*x_ref[2]**o+(-0.34761826549)*x_ref[2]
        ref[0,1,0,0]=(0.293581495968)*x_ref[0]**o+(0.201727375498)*x_ref[0]+(-0.416610275343)*x_ref[1]**o+(0.607460451032)*x_ref[1]+(0.0343669057747)*x_ref[2]**o+(-0.530601755969)*x_ref[2]
        ref[0,1,0,1]=(0.880681535309)*x_ref[0]**o+(-0.953246533735)*x_ref[0]+(-0.424031774258)*x_ref[1]**o+(-0.52019867795)*x_ref[1]+(-0.250435071705)*x_ref[2]**o+(-0.168661740641)*x_ref[2]
        ref[0,1,1,0]=(-0.495577302252)*x_ref[0]**o+(0.792099185033)*x_ref[0]+(0.294589995512)*x_ref[1]**o+(-0.734902995766)*x_ref[1]+(-0.359137354522)*x_ref[2]**o+(-0.0317334336381)*x_ref[2]
        ref[0,1,1,1]=(-0.843957318456)*x_ref[0]**o+(0.29759734816)*x_ref[0]+(-0.954015934149)*x_ref[1]**o+(-0.270649016792)*x_ref[1]+(-0.364089303705)*x_ref[2]**o+(-0.396696196833)*x_ref[2]
        ref[0,1,2,0]=(0.411843103656)*x_ref[0]**o+(-0.806857649219)*x_ref[0]+(0.0998406396629)*x_ref[1]**o+(0.426060184907)*x_ref[1]+(0.839569241907)*x_ref[2]**o+(-0.22764839333)*x_ref[2]
        ref[0,1,2,1]=(-0.970273259116)*x_ref[0]**o+(-0.983160602162)*x_ref[0]+(0.418163962338)*x_ref[1]**o+(-0.110768668492)*x_ref[1]+(0.328089646862)*x_ref[2]**o+(-0.270172456473)*x_ref[2]
        ref[0,2,0,0]=(0.683233964281)*x_ref[0]**o+(0.188111552757)*x_ref[0]+(-0.951071798504)*x_ref[1]**o+(-0.0330921159172)*x_ref[1]+(-0.64270161977)*x_ref[2]**o+(-0.499615377327)*x_ref[2]
        ref[0,2,0,1]=(0.36794958012)*x_ref[0]**o+(0.398010040747)*x_ref[0]+(-0.309445952639)*x_ref[1]**o+(-0.322623662412)*x_ref[1]+(-0.439692584231)*x_ref[2]**o+(0.00620886333611)*x_ref[2]
        ref[0,2,1,0]=(0.588715821634)*x_ref[0]**o+(-0.163253786484)*x_ref[0]+(-0.619853219098)*x_ref[1]**o+(-0.163020284476)*x_ref[1]+(-0.781500850817)*x_ref[2]**o+(-0.13622778235)*x_ref[2]
        ref[0,2,1,1]=(0.610025540474)*x_ref[0]**o+(-0.262832084419)*x_ref[0]+(0.109453975557)*x_ref[1]**o+(-0.0910626806898)*x_ref[1]+(0.720567359491)*x_ref[2]**o+(0.221728266865)*x_ref[2]
        ref[0,2,2,0]=(-0.967888175709)*x_ref[0]**o+(0.841198387559)*x_ref[0]+(0.0942188974625)*x_ref[1]**o+(0.056715320399)*x_ref[1]+(0.220387799569)*x_ref[2]**o+(-0.991352339366)*x_ref[2]
        ref[0,2,2,1]=(-0.243144875987)*x_ref[0]**o+(-0.101607712773)*x_ref[0]+(0.0890556148157)*x_ref[1]**o+(0.968975809324)*x_ref[1]+(-0.680428099066)*x_ref[2]**o+(-0.203127186894)*x_ref[2]
        ref[0,3,0,0]=(-0.97698323555)*x_ref[0]**o+(0.723899953965)*x_ref[0]+(0.168932911211)*x_ref[1]**o+(0.210562969565)*x_ref[1]+(0.0241740170862)*x_ref[2]**o+(-0.443543352786)*x_ref[2]
        ref[0,3,0,1]=(0.273706631775)*x_ref[0]**o+(-0.064335272978)*x_ref[0]+(-0.289390463262)*x_ref[1]**o+(-0.0911403229125)*x_ref[1]+(0.380520540499)*x_ref[2]**o+(0.930362632813)*x_ref[2]
        ref[0,3,1,0]=(0.108247452775)*x_ref[0]**o+(0.670880216846)*x_ref[0]+(0.743592674018)*x_ref[1]**o+(0.110450181904)*x_ref[1]+(0.633556946694)*x_ref[2]**o+(-0.526137817259)*x_ref[2]
        ref[0,3,1,1]=(0.770308192835)*x_ref[0]**o+(0.304435220496)*x_ref[0]+(-0.851453559716)*x_ref[1]**o+(-0.549151404489)*x_ref[1]+(-0.512295072491)*x_ref[2]**o+(-0.896216869911)*x_ref[2]
        ref[0,3,2,0]=(-0.879389228597)*x_ref[0]**o+(-0.98421803535)*x_ref[0]+(0.310411565504)*x_ref[1]**o+(-0.833601944827)*x_ref[1]+(0.520140917807)*x_ref[2]**o+(0.409319548582)*x_ref[2]
        ref[0,3,2,1]=(0.673800312082)*x_ref[0]**o+(0.307843370867)*x_ref[0]+(0.0435576949618)*x_ref[1]**o+(-0.362422816889)*x_ref[1]+(0.96314375221)*x_ref[2]**o+(-0.896763989426)*x_ref[2]
        ref[0,4,0,0]=(-0.569321969384)*x_ref[0]**o+(-0.545868340066)*x_ref[0]+(0.951980474226)*x_ref[1]**o+(-0.482241368075)*x_ref[1]+(-0.255170213259)*x_ref[2]**o+(0.841268809686)*x_ref[2]
        ref[0,4,0,1]=(-0.837681312588)*x_ref[0]**o+(-0.458077368372)*x_ref[0]+(0.997003677283)*x_ref[1]**o+(0.762129779023)*x_ref[1]+(0.0952433046108)*x_ref[2]**o+(-0.781885974532)*x_ref[2]
        ref[0,4,1,0]=(-0.856181540078)*x_ref[0]**o+(-0.841612660714)*x_ref[0]+(0.544038400285)*x_ref[1]**o+(-0.284909634097)*x_ref[1]+(0.11260106121)*x_ref[2]**o+(-0.39480742262)*x_ref[2]
        ref[0,4,1,1]=(0.356082675669)*x_ref[0]**o+(0.917688541228)*x_ref[0]+(-0.176959383957)*x_ref[1]**o+(0.416653474575)*x_ref[1]+(-0.662930023412)*x_ref[2]**o+(-0.73010168013)*x_ref[2]
        ref[0,4,2,0]=(-0.763454917764)*x_ref[0]**o+(0.302162768082)*x_ref[0]+(0.143386232133)*x_ref[1]**o+(0.91616734981)*x_ref[1]+(-0.813201390394)*x_ref[2]**o+(-0.857982436099)*x_ref[2]
        ref[0,4,2,1]=(0.835769822382)*x_ref[0]**o+(0.167610186558)*x_ref[0]+(0.2331699547)*x_ref[1]**o+(0.0015459879901)*x_ref[1]+(0.167167023037)*x_ref[2]**o+(0.550280560723)*x_ref[2]
        ref[1,0,0,0]=(-0.217157755228)*x_ref[0]**o+(0.534373560701)*x_ref[0]+(-0.883064731078)*x_ref[1]**o+(0.731486045177)*x_ref[1]+(0.819268782653)*x_ref[2]**o+(0.787662641797)*x_ref[2]
        ref[1,0,0,1]=(-0.487819762644)*x_ref[0]**o+(-0.430212238027)*x_ref[0]+(-0.56911636092)*x_ref[1]**o+(-0.728659093202)*x_ref[1]+(0.455173919727)*x_ref[2]**o+(-0.636506423308)*x_ref[2]
        ref[1,0,1,0]=(0.900337000667)*x_ref[0]**o+(0.299528369342)*x_ref[0]+(0.681918775341)*x_ref[1]**o+(0.370100893068)*x_ref[1]+(-0.587580308922)*x_ref[2]**o+(0.485274084829)*x_ref[2]
        ref[1,0,1,1]=(0.442748593533)*x_ref[0]**o+(-0.0292501463909)*x_ref[0]+(0.551554968843)*x_ref[1]**o+(-0.318313953949)*x_ref[1]+(-0.458151041817)*x_ref[2]**o+(-0.625611121823)*x_ref[2]
        ref[1,0,2,0]=(-0.723203702271)*x_ref[0]**o+(0.101313366226)*x_ref[0]+(-0.548133659891)*x_ref[1]**o+(-0.392729656892)*x_ref[1]+(0.46317338606)*x_ref[2]**o+(0.235572773174)*x_ref[2]
        ref[1,0,2,1]=(-0.445472509581)*x_ref[0]**o+(-0.552381494615)*x_ref[0]+(0.0815798929496)*x_ref[1]**o+(-0.595946577234)*x_ref[1]+(0.878280129666)*x_ref[2]**o+(-0.274608443481)*x_ref[2]
        ref[1,1,0,0]=(0.543392744962)*x_ref[0]**o+(0.79638586218)*x_ref[0]+(0.435895934157)*x_ref[1]**o+(-0.0134442947039)*x_ref[1]+(-0.23586652076)*x_ref[2]**o+(0.120783361228)*x_ref[2]
        ref[1,1,0,1]=(-0.50551104509)*x_ref[0]**o+(-0.578612752272)*x_ref[0]+(-0.595688781895)*x_ref[1]**o+(-0.411446622857)*x_ref[1]+(-0.729076372085)*x_ref[2]**o+(0.61997008872)*x_ref[2]
        ref[1,1,1,0]=(-0.25176939418)*x_ref[0]**o+(0.0192820953571)*x_ref[0]+(-0.365827108002)*x_ref[1]**o+(-0.58048375365)*x_ref[1]+(-0.109602718093)*x_ref[2]**o+(-0.764760324931)*x_ref[2]
        ref[1,1,1,1]=(0.333747531705)*x_ref[0]**o+(-0.975460281758)*x_ref[0]+(-0.263662359116)*x_ref[1]**o+(0.833048103253)*x_ref[1]+(-0.403378941533)*x_ref[2]**o+(0.99255510422)*x_ref[2]
        ref[1,1,2,0]=(-0.643575493982)*x_ref[0]**o+(0.408164900892)*x_ref[0]+(0.330669259534)*x_ref[1]**o+(0.696238788141)*x_ref[1]+(0.8187478709)*x_ref[2]**o+(0.269447281859)*x_ref[2]
        ref[1,1,2,1]=(0.0975010793595)*x_ref[0]**o+(-0.266707476957)*x_ref[0]+(-0.627484959323)*x_ref[1]**o+(0.189636667701)*x_ref[1]+(0.0634608353528)*x_ref[2]**o+(0.425595191628)*x_ref[2]
        ref[1,2,0,0]=(0.144863702314)*x_ref[0]**o+(0.62877495084)*x_ref[0]+(-0.937663622583)*x_ref[1]**o+(0.643925666447)*x_ref[1]+(0.0752876047081)*x_ref[2]**o+(-0.0278560665987)*x_ref[2]
        ref[1,2,0,1]=(0.825937768085)*x_ref[0]**o+(-0.350938240562)*x_ref[0]+(-0.316596763726)*x_ref[1]**o+(0.248951654728)*x_ref[1]+(-0.4533348882)*x_ref[2]**o+(-0.840614727297)*x_ref[2]
        ref[1,2,1,0]=(0.235071855147)*x_ref[0]**o+(0.818758983424)*x_ref[0]+(-0.962091319923)*x_ref[1]**o+(-0.0687829219781)*x_ref[1]+(0.256236142265)*x_ref[2]**o+(-0.236383412975)*x_ref[2]
        ref[1,2,1,1]=(-0.674935027618)*x_ref[0]**o+(0.506392204682)*x_ref[0]+(-0.891135534099)*x_ref[1]**o+(0.200562756224)*x_ref[1]+(-0.509385776169)*x_ref[2]**o+(-0.709115810434)*x_ref[2]
        ref[1,2,2,0]=(0.476890911154)*x_ref[0]**o+(-0.716988939897)*x_ref[0]+(-0.708930094063)*x_ref[1]**o+(-0.221417730773)*x_ref[1]+(-0.0530682866975)*x_ref[2]**o+(-0.933479841665)*x_ref[2]
        ref[1,2,2,1]=(0.912189530094)*x_ref[0]**o+(-0.287403816104)*x_ref[0]+(-0.755698083102)*x_ref[1]**o+(-0.996716444257)*x_ref[1]+(-0.836456434626)*x_ref[2]**o+(-0.463710726041)*x_ref[2]
        ref[1,3,0,0]=(0.253691639919)*x_ref[0]**o+(-0.279433210688)*x_ref[0]+(-0.117635686336)*x_ref[1]**o+(0.790073676599)*x_ref[1]+(-0.559215280716)*x_ref[2]**o+(-0.103190062272)*x_ref[2]
        ref[1,3,0,1]=(0.0970821498373)*x_ref[0]**o+(0.814008398347)*x_ref[0]+(0.583184329882)*x_ref[1]**o+(-0.598484860263)*x_ref[1]+(-0.00216176134509)*x_ref[2]**o+(0.14889662547)*x_ref[2]
        ref[1,3,1,0]=(-0.730045080625)*x_ref[0]**o+(-0.746336558552)*x_ref[0]+(-0.995897359987)*x_ref[1]**o+(-0.70582565406)*x_ref[1]+(0.36600954398)*x_ref[2]**o+(-0.0843877537778)*x_ref[2]
        ref[1,3,1,1]=(-0.714997081413)*x_ref[0]**o+(-0.138823664507)*x_ref[0]+(0.969108046142)*x_ref[1]**o+(-0.201514397247)*x_ref[1]+(-0.438601765508)*x_ref[2]**o+(-0.984456457681)*x_ref[2]
        ref[1,3,2,0]=(-0.577419600949)*x_ref[0]**o+(-0.821514372871)*x_ref[0]+(-0.179395810292)*x_ref[1]**o+(-0.904303143215)*x_ref[1]+(0.27616964799)*x_ref[2]**o+(-0.272088845238)*x_ref[2]
        ref[1,3,2,1]=(0.31325269807)*x_ref[0]**o+(-0.0134619122423)*x_ref[0]+(-0.641786875798)*x_ref[1]**o+(-0.568855591797)*x_ref[1]+(0.600302453067)*x_ref[2]**o+(0.830044865545)*x_ref[2]
        ref[1,4,0,0]=(-0.791605536671)*x_ref[0]**o+(-0.578889866035)*x_ref[0]+(-0.22659053834)*x_ref[1]**o+(0.837093316131)*x_ref[1]+(0.499953644961)*x_ref[2]**o+(-0.289930435498)*x_ref[2]
        ref[1,4,0,1]=(0.819284096253)*x_ref[0]**o+(0.272518944584)*x_ref[0]+(0.199276568134)*x_ref[1]**o+(0.132962435684)*x_ref[1]+(-0.896986377734)*x_ref[2]**o+(0.775604010154)*x_ref[2]
        ref[1,4,1,0]=(-0.450392824249)*x_ref[0]**o+(0.376154164409)*x_ref[0]+(0.199853639816)*x_ref[1]**o+(-0.679551176507)*x_ref[1]+(0.565506686048)*x_ref[2]**o+(0.391489667485)*x_ref[2]
        ref[1,4,1,1]=(-0.0905057426998)*x_ref[0]**o+(-0.749252305812)*x_ref[0]+(-0.21131989398)*x_ref[1]**o+(-0.671560452908)*x_ref[1]+(-0.760505475283)*x_ref[2]**o+(0.763692313655)*x_ref[2]
        ref[1,4,2,0]=(-0.219636883594)*x_ref[0]**o+(-0.380991913771)*x_ref[0]+(0.831879211521)*x_ref[1]**o+(-0.0686682700553)*x_ref[1]+(0.843832009742)*x_ref[2]**o+(-0.86627759217)*x_ref[2]
        ref[1,4,2,1]=(-0.332685504588)*x_ref[0]**o+(0.589527397306)*x_ref[0]+(0.624226689395)*x_ref[1]**o+(0.0515479774619)*x_ref[1]+(0.156657102499)*x_ref[2]**o+(0.465889992773)*x_ref[2]
        ref[2,0,0,0]=(0.250467292043)*x_ref[0]**o+(-0.0658874324582)*x_ref[0]+(-0.0709052912862)*x_ref[1]**o+(0.700837318576)*x_ref[1]+(0.650887276648)*x_ref[2]**o+(0.816187438816)*x_ref[2]
        ref[2,0,0,1]=(-0.697069910038)*x_ref[0]**o+(-0.287284875865)*x_ref[0]+(-0.972646385975)*x_ref[1]**o+(-0.430570334599)*x_ref[1]+(0.400826092127)*x_ref[2]**o+(-0.484530830951)*x_ref[2]
        ref[2,0,1,0]=(0.58433575578)*x_ref[0]**o+(-0.114741976876)*x_ref[0]+(-0.205615141565)*x_ref[1]**o+(-0.476488876624)*x_ref[1]+(0.323174445651)*x_ref[2]**o+(-0.152550510154)*x_ref[2]
        ref[2,0,1,1]=(0.928306678393)*x_ref[0]**o+(0.104020335724)*x_ref[0]+(-0.0416879184197)*x_ref[1]**o+(0.865779079734)*x_ref[1]+(-0.169672240264)*x_ref[2]**o+(0.118778471217)*x_ref[2]
        ref[2,0,2,0]=(0.652779555688)*x_ref[0]**o+(-0.0209488281208)*x_ref[0]+(0.809628464928)*x_ref[1]**o+(-0.85121566767)*x_ref[1]+(-0.845109631338)*x_ref[2]**o+(-0.970252635974)*x_ref[2]
        ref[2,0,2,1]=(-0.740845790747)*x_ref[0]**o+(-0.705835368678)*x_ref[0]+(0.26040103782)*x_ref[1]**o+(0.814854175049)*x_ref[1]+(-0.00842496430724)*x_ref[2]**o+(0.192519753618)*x_ref[2]
        ref[2,1,0,0]=(0.629347590715)*x_ref[0]**o+(0.922723375982)*x_ref[0]+(-0.430492700295)*x_ref[1]**o+(-0.895795670345)*x_ref[1]+(0.708440378414)*x_ref[2]**o+(-0.155782723618)*x_ref[2]
        ref[2,1,0,1]=(-0.478612307362)*x_ref[0]**o+(0.645774615123)*x_ref[0]+(-0.461136871779)*x_ref[1]**o+(0.297927339931)*x_ref[1]+(-0.452011143468)*x_ref[2]**o+(0.267002423863)*x_ref[2]
        ref[2,1,1,0]=(-0.0591350600103)*x_ref[0]**o+(-0.252358408576)*x_ref[0]+(-0.0412752075681)*x_ref[1]**o+(0.942736455517)*x_ref[1]+(-0.223363982173)*x_ref[2]**o+(-0.752007969767)*x_ref[2]
        ref[2,1,1,1]=(0.197290290824)*x_ref[0]**o+(-0.170028529393)*x_ref[0]+(-0.143011200679)*x_ref[1]**o+(-0.694395961945)*x_ref[1]+(0.430761130021)*x_ref[2]**o+(0.298239563293)*x_ref[2]
        ref[2,1,2,0]=(0.258474077793)*x_ref[0]**o+(-0.956041015018)*x_ref[0]+(0.360444161221)*x_ref[1]**o+(-0.969922460117)*x_ref[1]+(0.856255540521)*x_ref[2]**o+(0.108872710157)*x_ref[2]
        ref[2,1,2,1]=(-0.694400609963)*x_ref[0]**o+(-0.864737376468)*x_ref[0]+(-0.898839555362)*x_ref[1]**o+(-0.655053934442)*x_ref[1]+(-0.511293723446)*x_ref[2]**o+(-0.204242252526)*x_ref[2]
        ref[2,2,0,0]=(-0.427946109181)*x_ref[0]**o+(0.435798407553)*x_ref[0]+(-0.567923694982)*x_ref[1]**o+(0.870896124804)*x_ref[1]+(-0.996434693707)*x_ref[2]**o+(-0.683367112977)*x_ref[2]
        ref[2,2,0,1]=(-0.219431769721)*x_ref[0]**o+(-0.302922785317)*x_ref[0]+(-0.0998576564441)*x_ref[1]**o+(0.64263522193)*x_ref[1]+(0.53256637446)*x_ref[2]**o+(-0.00387183821982)*x_ref[2]
        ref[2,2,1,0]=(-0.174988349945)*x_ref[0]**o+(0.408056661499)*x_ref[0]+(0.55925945151)*x_ref[1]**o+(-0.647959566858)*x_ref[1]+(0.905755316979)*x_ref[2]**o+(0.691774684766)*x_ref[2]
        ref[2,2,1,1]=(-0.882995226725)*x_ref[0]**o+(-0.948874015343)*x_ref[0]+(-0.652506601975)*x_ref[1]**o+(-0.137989382315)*x_ref[1]+(0.665695021939)*x_ref[2]**o+(0.163493199569)*x_ref[2]
        ref[2,2,2,0]=(0.666421535029)*x_ref[0]**o+(-0.984765122669)*x_ref[0]+(0.399488949215)*x_ref[1]**o+(-0.99963893587)*x_ref[1]+(0.225552018439)*x_ref[2]**o+(0.430770653325)*x_ref[2]
        ref[2,2,2,1]=(-0.0284528199834)*x_ref[0]**o+(0.0421195684748)*x_ref[0]+(-0.399757974604)*x_ref[1]**o+(0.305895209328)*x_ref[1]+(0.190920234352)*x_ref[2]**o+(-0.0990327186702)*x_ref[2]
        ref[2,3,0,0]=(-0.565899348734)*x_ref[0]**o+(0.636275927936)*x_ref[0]+(0.682710455722)*x_ref[1]**o+(-0.434837962369)*x_ref[1]+(0.252457745239)*x_ref[2]**o+(-0.709529514057)*x_ref[2]
        ref[2,3,0,1]=(0.0113021999515)*x_ref[0]**o+(0.451528226044)*x_ref[0]+(-0.705424064371)*x_ref[1]**o+(0.500516985463)*x_ref[1]+(-0.528750586385)*x_ref[2]**o+(0.998214035471)*x_ref[2]
        ref[2,3,1,0]=(-0.879296343578)*x_ref[0]**o+(-0.724012783173)*x_ref[0]+(0.51105902172)*x_ref[1]**o+(-0.840460578985)*x_ref[1]+(0.222130858003)*x_ref[2]**o+(0.998500581881)*x_ref[2]
        ref[2,3,1,1]=(0.478703623115)*x_ref[0]**o+(-0.94935681973)*x_ref[0]+(-0.609714878079)*x_ref[1]**o+(0.0318988430394)*x_ref[1]+(-0.858395334121)*x_ref[2]**o+(-0.171229770192)*x_ref[2]
        ref[2,3,2,0]=(0.0791777006911)*x_ref[0]**o+(-0.306954070098)*x_ref[0]+(-0.182890473769)*x_ref[1]**o+(-0.26384266322)*x_ref[1]+(0.663652835142)*x_ref[2]**o+(0.188229986922)*x_ref[2]
        ref[2,3,2,1]=(-0.101910553498)*x_ref[0]**o+(-0.34187147162)*x_ref[0]+(-0.52746320646)*x_ref[1]**o+(-0.891142643574)*x_ref[1]+(-0.84135671258)*x_ref[2]**o+(0.448615749816)*x_ref[2]
        ref[2,4,0,0]=(0.928980482137)*x_ref[0]**o+(0.00486879685273)*x_ref[0]+(-0.068529973495)*x_ref[1]**o+(-0.16116665925)*x_ref[1]+(0.247146438634)*x_ref[2]**o+(-0.341715145751)*x_ref[2]
        ref[2,4,0,1]=(0.486412050624)*x_ref[0]**o+(-0.929254100014)*x_ref[0]+(0.439267744247)*x_ref[1]**o+(-0.430515713902)*x_ref[1]+(-0.317438643233)*x_ref[2]**o+(-0.0843825737593)*x_ref[2]
        ref[2,4,1,0]=(0.0130446306687)*x_ref[0]**o+(0.324310412551)*x_ref[0]+(-0.38112172561)*x_ref[1]**o+(-0.92878637625)*x_ref[1]+(0.32867951654)*x_ref[2]**o+(-0.0315638177916)*x_ref[2]
        ref[2,4,1,1]=(-0.254225591431)*x_ref[0]**o+(0.321872838348)*x_ref[0]+(0.179619901508)*x_ref[1]**o+(-0.858713537407)*x_ref[1]+(0.151254493236)*x_ref[2]**o+(-0.340368823543)*x_ref[2]
        ref[2,4,2,0]=(-0.0844677926666)*x_ref[0]**o+(0.398017491087)*x_ref[0]+(0.0313455976792)*x_ref[1]**o+(-0.872666391632)*x_ref[1]+(0.301765055045)*x_ref[2]**o+(0.563923225852)*x_ref[2]
        ref[2,4,2,1]=(0.814244097177)*x_ref[0]**o+(-0.223957875084)*x_ref[0]+(0.447995085387)*x_ref[1]**o+(0.350953464712)*x_ref[1]+(0.333269027952)*x_ref[2]**o+(-0.941385895584)*x_ref[2]
        ref[3,0,0,0]=(0.0789492878053)*x_ref[0]**o+(0.525756966035)*x_ref[0]+(0.87125043555)*x_ref[1]**o+(0.881346392374)*x_ref[1]+(-0.855477786784)*x_ref[2]**o+(-0.991712685096)*x_ref[2]
        ref[3,0,0,1]=(-0.0359771254585)*x_ref[0]**o+(0.603218005193)*x_ref[0]+(-0.580729043759)*x_ref[1]**o+(-0.20390195049)*x_ref[1]+(-0.862263281233)*x_ref[2]**o+(-0.196587155144)*x_ref[2]
        ref[3,0,1,0]=(0.879750375964)*x_ref[0]**o+(-0.251177194936)*x_ref[0]+(0.191264882539)*x_ref[1]**o+(-0.95611617781)*x_ref[1]+(0.71738769264)*x_ref[2]**o+(-0.691767368606)*x_ref[2]
        ref[3,0,1,1]=(-0.761557468157)*x_ref[0]**o+(-0.381655367519)*x_ref[0]+(0.587969140055)*x_ref[1]**o+(0.622246650476)*x_ref[1]+(0.117544678721)*x_ref[2]**o+(-0.576059347687)*x_ref[2]
        ref[3,0,2,0]=(-0.587802439618)*x_ref[0]**o+(-0.981432433894)*x_ref[0]+(-0.758496301974)*x_ref[1]**o+(0.656105955022)*x_ref[1]+(-0.689107562745)*x_ref[2]**o+(0.267295196672)*x_ref[2]
        ref[3,0,2,1]=(-0.0524224100533)*x_ref[0]**o+(-0.925763321074)*x_ref[0]+(0.218186590694)*x_ref[1]**o+(-0.598934300523)*x_ref[1]+(0.22756773)*x_ref[2]**o+(-0.770210327792)*x_ref[2]
        ref[3,1,0,0]=(0.922635777646)*x_ref[0]**o+(-0.278736876358)*x_ref[0]+(0.0631653424614)*x_ref[1]**o+(-0.356448496856)*x_ref[1]+(-0.609671239719)*x_ref[2]**o+(0.189678422301)*x_ref[2]
        ref[3,1,0,1]=(-0.509070180487)*x_ref[0]**o+(-0.250043376376)*x_ref[0]+(-0.0697248208177)*x_ref[1]**o+(0.531153766941)*x_ref[1]+(0.240916012233)*x_ref[2]**o+(0.391353341194)*x_ref[2]
        ref[3,1,1,0]=(-0.514764390151)*x_ref[0]**o+(-0.347397972881)*x_ref[0]+(0.138729357858)*x_ref[1]**o+(-0.30135683112)*x_ref[1]+(0.541893745985)*x_ref[2]**o+(0.414478801003)*x_ref[2]
        ref[3,1,1,1]=(0.866088916369)*x_ref[0]**o+(-0.96729572537)*x_ref[0]+(0.824735563587)*x_ref[1]**o+(-0.479859785014)*x_ref[1]+(-0.175784746993)*x_ref[2]**o+(-0.998407148967)*x_ref[2]
        ref[3,1,2,0]=(0.327087092534)*x_ref[0]**o+(0.171000747621)*x_ref[0]+(-0.33910895269)*x_ref[1]**o+(0.349474180167)*x_ref[1]+(0.561935192081)*x_ref[2]**o+(0.358380998288)*x_ref[2]
        ref[3,1,2,1]=(0.35646522102)*x_ref[0]**o+(-0.912130691808)*x_ref[0]+(0.796756777424)*x_ref[1]**o+(-0.91884834291)*x_ref[1]+(0.956454722923)*x_ref[2]**o+(0.462859602491)*x_ref[2]
        ref[3,2,0,0]=(-0.849637152897)*x_ref[0]**o+(0.353228393186)*x_ref[0]+(-0.725757036845)*x_ref[1]**o+(0.590369089609)*x_ref[1]+(-0.280432657156)*x_ref[2]**o+(0.142413703594)*x_ref[2]
        ref[3,2,0,1]=(-0.640766746158)*x_ref[0]**o+(-0.454583129018)*x_ref[0]+(-0.616394565949)*x_ref[1]**o+(0.724166900329)*x_ref[1]+(-0.748682704671)*x_ref[2]**o+(0.934076771628)*x_ref[2]
        ref[3,2,1,0]=(-0.824548065846)*x_ref[0]**o+(0.40250159164)*x_ref[0]+(0.763315748823)*x_ref[1]**o+(0.255878553659)*x_ref[1]+(0.872676778336)*x_ref[2]**o+(-0.100430227782)*x_ref[2]
        ref[3,2,1,1]=(-0.610248210467)*x_ref[0]**o+(0.160976829271)*x_ref[0]+(-0.384236718764)*x_ref[1]**o+(0.657119635861)*x_ref[1]+(0.161695416084)*x_ref[2]**o+(0.505997006976)*x_ref[2]
        ref[3,2,2,0]=(0.220409736038)*x_ref[0]**o+(0.398197530033)*x_ref[0]+(-0.288242008883)*x_ref[1]**o+(0.0832866402504)*x_ref[1]+(0.761030964351)*x_ref[2]**o+(0.751986795183)*x_ref[2]
        ref[3,2,2,1]=(-0.396845385157)*x_ref[0]**o+(-0.0792538647037)*x_ref[0]+(-0.327181444898)*x_ref[1]**o+(-0.810682039031)*x_ref[1]+(0.260671755637)*x_ref[2]**o+(-0.242407958043)*x_ref[2]
        ref[3,3,0,0]=(-0.663937889328)*x_ref[0]**o+(0.587545393032)*x_ref[0]+(0.210910229094)*x_ref[1]**o+(0.500419329566)*x_ref[1]+(0.202539078075)*x_ref[2]**o+(-0.623007416272)*x_ref[2]
        ref[3,3,0,1]=(0.0375980252743)*x_ref[0]**o+(-0.435705054083)*x_ref[0]+(-0.660885021014)*x_ref[1]**o+(-0.0332748037874)*x_ref[1]+(-0.832274220584)*x_ref[2]**o+(0.805002616286)*x_ref[2]
        ref[3,3,1,0]=(0.395522385265)*x_ref[0]**o+(0.901865335207)*x_ref[0]+(0.016322521344)*x_ref[1]**o+(-0.532212956041)*x_ref[1]+(0.190751397835)*x_ref[2]**o+(0.234716440942)*x_ref[2]
        ref[3,3,1,1]=(0.257090076986)*x_ref[0]**o+(-0.169468215187)*x_ref[0]+(-0.0641337112558)*x_ref[1]**o+(-0.674363367639)*x_ref[1]+(0.13633053895)*x_ref[2]**o+(-0.741438377514)*x_ref[2]
        ref[3,3,2,0]=(0.984088274745)*x_ref[0]**o+(-0.669313378065)*x_ref[0]+(0.899763931565)*x_ref[1]**o+(-0.874652895499)*x_ref[1]+(-0.172620100734)*x_ref[2]**o+(-0.635666769897)*x_ref[2]
        ref[3,3,2,1]=(0.693112828433)*x_ref[0]**o+(-0.99231490163)*x_ref[0]+(0.525842419539)*x_ref[1]**o+(0.268799565406)*x_ref[1]+(-0.473532039244)*x_ref[2]**o+(0.50555338295)*x_ref[2]
        ref[3,4,0,0]=(0.780678215778)*x_ref[0]**o+(0.56287435306)*x_ref[0]+(0.322651378857)*x_ref[1]**o+(0.268655956157)*x_ref[1]+(0.438312596985)*x_ref[2]**o+(0.175512199841)*x_ref[2]
        ref[3,4,0,1]=(-0.510399313775)*x_ref[0]**o+(-0.63898491047)*x_ref[0]+(-0.765120717001)*x_ref[1]**o+(0.45570492579)*x_ref[1]+(-0.148930444588)*x_ref[2]**o+(0.458400743585)*x_ref[2]
        ref[3,4,1,0]=(-0.629456511378)*x_ref[0]**o+(-0.0956526447002)*x_ref[0]+(0.223599370201)*x_ref[1]**o+(-0.689787947305)*x_ref[1]+(0.834792179813)*x_ref[2]**o+(0.413903455599)*x_ref[2]
        ref[3,4,1,1]=(0.89648905371)*x_ref[0]**o+(0.420890001861)*x_ref[0]+(-0.890077307972)*x_ref[1]**o+(0.424785046869)*x_ref[1]+(0.84365123356)*x_ref[2]**o+(0.56417731826)*x_ref[2]
        ref[3,4,2,0]=(0.853620031631)*x_ref[0]**o+(-0.719293879422)*x_ref[0]+(-0.811007794139)*x_ref[1]**o+(-0.236043157925)*x_ref[1]+(0.130830104273)*x_ref[2]**o+(-0.554431879475)*x_ref[2]
        ref[3,4,2,1]=(-0.555616431129)*x_ref[0]**o+(-0.900929040274)*x_ref[0]+(0.773492786481)*x_ref[1]**o+(-0.445322579241)*x_ref[1]+(0.270832172179)*x_ref[2]**o+(-0.716484089692)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromSymbol_ReducedFunctionOnContactOne_rank0(self):
      """
      tests interpolate for rank 0 Symbol onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactOne onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactOne(self.domain)
      x=w.getX()
      arg=Symbol(shape=(),dim=dim)
      s=Data(0,(),w)
      ref=Data(0,(),w_ref)
      if dim==2:
        s=(-0.806530631577)*x[0]**o+(-0.341707069796)*x[0]+(0.624950122202)*x[1]**o+(-0.547410220031)*x[1]
        ref=(-0.806530631577)*x_ref[0]**o+(-0.341707069796)*x_ref[0]+(0.624950122202)*x_ref[1]**o+(-0.547410220031)*x_ref[1]
      else:
        s=(-0.227792789776)*x[0]**o+(-0.458918821401)*x[0]+(0.333705314078)*x[1]**o+(0.513071453636)*x[1]+(-0.251497218549)*x[2]**o+(0.447781682612)*x[2]
        ref=(-0.227792789776)*x_ref[0]**o+(-0.458918821401)*x_ref[0]+(0.333705314078)*x_ref[1]**o+(0.513071453636)*x_ref[1]+(-0.251497218549)*x_ref[2]**o+(0.447781682612)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromSymbol_ReducedFunctionOnContactOne_rank1(self):
      """
      tests interpolate for rank 1 Symbol onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactOne onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactOne(self.domain)
      x=w.getX()
      arg=Symbol(shape=(2,),dim=dim)
      s=Data(0,(2,),w)
      ref=Data(0,(2,),w_ref)
      if dim==2:
        s[0]=(-0.432328873396)*x[0]**o+(0.677542875504)*x[0]+(-0.834641628818)*x[1]**o+(0.628835966611)*x[1]
        s[1]=(-0.902852113303)*x[0]**o+(-0.593877706793)*x[0]+(0.395220406099)*x[1]**o+(-0.523411084099)*x[1]
        ref[0]=(-0.432328873396)*x_ref[0]**o+(0.677542875504)*x_ref[0]+(-0.834641628818)*x_ref[1]**o+(0.628835966611)*x_ref[1]
        ref[1]=(-0.902852113303)*x_ref[0]**o+(-0.593877706793)*x_ref[0]+(0.395220406099)*x_ref[1]**o+(-0.523411084099)*x_ref[1]
      else:
        s[0]=(0.297175304924)*x[0]**o+(-0.43517991218)*x[0]+(-0.0313129470041)*x[1]**o+(0.269658750043)*x[1]+(-0.407781379485)*x[2]**o+(-0.605849978158)*x[2]
        s[1]=(-0.525947091801)*x[0]**o+(-0.39352324679)*x[0]+(0.683408550141)*x[1]**o+(-0.623028099041)*x[1]+(0.0817707351822)*x[2]**o+(0.36388186042)*x[2]
        ref[0]=(0.297175304924)*x_ref[0]**o+(-0.43517991218)*x_ref[0]+(-0.0313129470041)*x_ref[1]**o+(0.269658750043)*x_ref[1]+(-0.407781379485)*x_ref[2]**o+(-0.605849978158)*x_ref[2]
        ref[1]=(-0.525947091801)*x_ref[0]**o+(-0.39352324679)*x_ref[0]+(0.683408550141)*x_ref[1]**o+(-0.623028099041)*x_ref[1]+(0.0817707351822)*x_ref[2]**o+(0.36388186042)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromSymbol_ReducedFunctionOnContactOne_rank2(self):
      """
      tests interpolate for rank 2 Symbol onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactOne onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactOne(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5),dim=dim)
      s=Data(0,(4, 5),w)
      ref=Data(0,(4, 5),w_ref)
      if dim==2:
        s[0,0]=(0.406232678425)*x[0]**o+(-0.930963378238)*x[0]+(0.845174140625)*x[1]**o+(0.981192355961)*x[1]
        s[0,1]=(-0.635413173141)*x[0]**o+(-0.497296028209)*x[0]+(-0.994669338217)*x[1]**o+(-0.931999198677)*x[1]
        s[0,2]=(0.0780179414399)*x[0]**o+(0.478842279811)*x[0]+(-0.0352846591054)*x[1]**o+(0.111952755346)*x[1]
        s[0,3]=(-0.934565575643)*x[0]**o+(-0.257127097452)*x[0]+(-0.375100482254)*x[1]**o+(0.26830947979)*x[1]
        s[0,4]=(0.00406708346809)*x[0]**o+(0.428953823715)*x[0]+(0.582342187327)*x[1]**o+(0.915249586352)*x[1]
        s[1,0]=(-0.667807592712)*x[0]**o+(0.16108128518)*x[0]+(0.143438381665)*x[1]**o+(-0.96131677347)*x[1]
        s[1,1]=(0.939048416505)*x[0]**o+(0.953402057565)*x[0]+(-0.541334930887)*x[1]**o+(-0.0575833896416)*x[1]
        s[1,2]=(0.0892590765861)*x[0]**o+(-0.599905917167)*x[0]+(-0.0350800081223)*x[1]**o+(-0.0380326906586)*x[1]
        s[1,3]=(-0.620396486232)*x[0]**o+(-0.147159989485)*x[0]+(-0.975288447854)*x[1]**o+(-0.299830553213)*x[1]
        s[1,4]=(-0.500651954078)*x[0]**o+(0.463784646532)*x[0]+(0.494626131733)*x[1]**o+(0.714313271607)*x[1]
        s[2,0]=(0.999573391428)*x[0]**o+(0.740845556331)*x[0]+(-0.254541944755)*x[1]**o+(0.90861174582)*x[1]
        s[2,1]=(0.106308044175)*x[0]**o+(0.737496999865)*x[0]+(0.345388021114)*x[1]**o+(0.732857975859)*x[1]
        s[2,2]=(0.0488435293882)*x[0]**o+(-0.580187506515)*x[0]+(-0.146074818428)*x[1]**o+(0.0903174437642)*x[1]
        s[2,3]=(0.105892490012)*x[0]**o+(0.0233855588118)*x[0]+(-0.0688234002192)*x[1]**o+(-0.608212758131)*x[1]
        s[2,4]=(0.882922466867)*x[0]**o+(0.346911804995)*x[0]+(-0.247283497286)*x[1]**o+(0.411870299278)*x[1]
        s[3,0]=(0.139595043935)*x[0]**o+(-0.228322879218)*x[0]+(-0.124338355975)*x[1]**o+(0.633626671983)*x[1]
        s[3,1]=(0.0525495199049)*x[0]**o+(0.446874982783)*x[0]+(-0.597381789826)*x[1]**o+(0.232913952793)*x[1]
        s[3,2]=(0.350084550681)*x[0]**o+(-0.329493065869)*x[0]+(0.403975623285)*x[1]**o+(0.560476340931)*x[1]
        s[3,3]=(0.918601061431)*x[0]**o+(0.5842950108)*x[0]+(-0.909414458447)*x[1]**o+(0.208889362019)*x[1]
        s[3,4]=(0.0624223531897)*x[0]**o+(-0.364330980767)*x[0]+(-0.388102867728)*x[1]**o+(0.547027313956)*x[1]
        ref[0,0]=(0.406232678425)*x_ref[0]**o+(-0.930963378238)*x_ref[0]+(0.845174140625)*x_ref[1]**o+(0.981192355961)*x_ref[1]
        ref[0,1]=(-0.635413173141)*x_ref[0]**o+(-0.497296028209)*x_ref[0]+(-0.994669338217)*x_ref[1]**o+(-0.931999198677)*x_ref[1]
        ref[0,2]=(0.0780179414399)*x_ref[0]**o+(0.478842279811)*x_ref[0]+(-0.0352846591054)*x_ref[1]**o+(0.111952755346)*x_ref[1]
        ref[0,3]=(-0.934565575643)*x_ref[0]**o+(-0.257127097452)*x_ref[0]+(-0.375100482254)*x_ref[1]**o+(0.26830947979)*x_ref[1]
        ref[0,4]=(0.00406708346809)*x_ref[0]**o+(0.428953823715)*x_ref[0]+(0.582342187327)*x_ref[1]**o+(0.915249586352)*x_ref[1]
        ref[1,0]=(-0.667807592712)*x_ref[0]**o+(0.16108128518)*x_ref[0]+(0.143438381665)*x_ref[1]**o+(-0.96131677347)*x_ref[1]
        ref[1,1]=(0.939048416505)*x_ref[0]**o+(0.953402057565)*x_ref[0]+(-0.541334930887)*x_ref[1]**o+(-0.0575833896416)*x_ref[1]
        ref[1,2]=(0.0892590765861)*x_ref[0]**o+(-0.599905917167)*x_ref[0]+(-0.0350800081223)*x_ref[1]**o+(-0.0380326906586)*x_ref[1]
        ref[1,3]=(-0.620396486232)*x_ref[0]**o+(-0.147159989485)*x_ref[0]+(-0.975288447854)*x_ref[1]**o+(-0.299830553213)*x_ref[1]
        ref[1,4]=(-0.500651954078)*x_ref[0]**o+(0.463784646532)*x_ref[0]+(0.494626131733)*x_ref[1]**o+(0.714313271607)*x_ref[1]
        ref[2,0]=(0.999573391428)*x_ref[0]**o+(0.740845556331)*x_ref[0]+(-0.254541944755)*x_ref[1]**o+(0.90861174582)*x_ref[1]
        ref[2,1]=(0.106308044175)*x_ref[0]**o+(0.737496999865)*x_ref[0]+(0.345388021114)*x_ref[1]**o+(0.732857975859)*x_ref[1]
        ref[2,2]=(0.0488435293882)*x_ref[0]**o+(-0.580187506515)*x_ref[0]+(-0.146074818428)*x_ref[1]**o+(0.0903174437642)*x_ref[1]
        ref[2,3]=(0.105892490012)*x_ref[0]**o+(0.0233855588118)*x_ref[0]+(-0.0688234002192)*x_ref[1]**o+(-0.608212758131)*x_ref[1]
        ref[2,4]=(0.882922466867)*x_ref[0]**o+(0.346911804995)*x_ref[0]+(-0.247283497286)*x_ref[1]**o+(0.411870299278)*x_ref[1]
        ref[3,0]=(0.139595043935)*x_ref[0]**o+(-0.228322879218)*x_ref[0]+(-0.124338355975)*x_ref[1]**o+(0.633626671983)*x_ref[1]
        ref[3,1]=(0.0525495199049)*x_ref[0]**o+(0.446874982783)*x_ref[0]+(-0.597381789826)*x_ref[1]**o+(0.232913952793)*x_ref[1]
        ref[3,2]=(0.350084550681)*x_ref[0]**o+(-0.329493065869)*x_ref[0]+(0.403975623285)*x_ref[1]**o+(0.560476340931)*x_ref[1]
        ref[3,3]=(0.918601061431)*x_ref[0]**o+(0.5842950108)*x_ref[0]+(-0.909414458447)*x_ref[1]**o+(0.208889362019)*x_ref[1]
        ref[3,4]=(0.0624223531897)*x_ref[0]**o+(-0.364330980767)*x_ref[0]+(-0.388102867728)*x_ref[1]**o+(0.547027313956)*x_ref[1]
      else:
        s[0,0]=(0.127471865714)*x[0]**o+(0.75668808568)*x[0]+(0.0197793355564)*x[1]**o+(0.342611017161)*x[1]+(0.256637655934)*x[2]**o+(0.144098222666)*x[2]
        s[0,1]=(0.460982544361)*x[0]**o+(0.0758913155194)*x[0]+(-0.738649205082)*x[1]**o+(-0.729674519552)*x[1]+(0.134554511124)*x[2]**o+(-0.966054425176)*x[2]
        s[0,2]=(-0.515765669688)*x[0]**o+(0.223475540597)*x[0]+(0.710015035386)*x[1]**o+(-0.810406212241)*x[1]+(-0.706682492015)*x[2]**o+(-0.176836116541)*x[2]
        s[0,3]=(0.849940074716)*x[0]**o+(0.276008832632)*x[0]+(0.753715668597)*x[1]**o+(0.327023435021)*x[1]+(-0.432843830378)*x[2]**o+(-0.89188163433)*x[2]
        s[0,4]=(0.306934523362)*x[0]**o+(0.400359334134)*x[0]+(-0.994806829246)*x[1]**o+(0.0349597549696)*x[1]+(-0.978591943815)*x[2]**o+(0.612187266226)*x[2]
        s[1,0]=(0.0461258495436)*x[0]**o+(-0.959702481081)*x[0]+(-0.3018982478)*x[1]**o+(-0.462251712831)*x[1]+(-0.94462432144)*x[2]**o+(-0.506662578784)*x[2]
        s[1,1]=(0.233144525806)*x[0]**o+(-0.414379147564)*x[0]+(0.38512336385)*x[1]**o+(-0.310763668907)*x[1]+(0.48918840723)*x[2]**o+(0.801469913207)*x[2]
        s[1,2]=(-0.132344623984)*x[0]**o+(-0.242418158654)*x[0]+(-0.618349833846)*x[1]**o+(0.249538312346)*x[1]+(0.466255623863)*x[2]**o+(0.619476167885)*x[2]
        s[1,3]=(-0.600411057229)*x[0]**o+(-0.65398469587)*x[0]+(-0.086416622227)*x[1]**o+(0.497186435444)*x[1]+(0.490006868481)*x[2]**o+(-0.281505238163)*x[2]
        s[1,4]=(-0.833074114751)*x[0]**o+(0.179085322144)*x[0]+(-0.669109910245)*x[1]**o+(0.853858313308)*x[1]+(-0.474984175674)*x[2]**o+(0.0145591425038)*x[2]
        s[2,0]=(-0.463438302204)*x[0]**o+(-0.815969915511)*x[0]+(0.958428072331)*x[1]**o+(-0.760925159788)*x[1]+(0.419348775332)*x[2]**o+(-0.146777978724)*x[2]
        s[2,1]=(0.64963543058)*x[0]**o+(0.713874315445)*x[0]+(0.000795833519698)*x[1]**o+(-0.216757220289)*x[1]+(-0.496572329247)*x[2]**o+(0.535744506796)*x[2]
        s[2,2]=(0.848552125819)*x[0]**o+(0.61034402517)*x[0]+(0.254155499518)*x[1]**o+(0.584452254449)*x[1]+(-0.739215490349)*x[2]**o+(0.553754902653)*x[2]
        s[2,3]=(0.45662694476)*x[0]**o+(0.63839923138)*x[0]+(0.05783248842)*x[1]**o+(0.326555670737)*x[1]+(-0.231728870265)*x[2]**o+(-0.773309888086)*x[2]
        s[2,4]=(-0.848527683977)*x[0]**o+(0.445666786862)*x[0]+(0.96357445665)*x[1]**o+(0.805189746125)*x[1]+(-0.99455326053)*x[2]**o+(-0.149519744556)*x[2]
        s[3,0]=(0.537998019205)*x[0]**o+(-0.160515409613)*x[0]+(-0.518616417074)*x[1]**o+(-0.78931847859)*x[1]+(0.533911685732)*x[2]**o+(0.249570769302)*x[2]
        s[3,1]=(0.127551558633)*x[0]**o+(-0.152792201104)*x[0]+(-0.177992097046)*x[1]**o+(-0.582313615383)*x[1]+(0.180383484767)*x[2]**o+(0.457671553174)*x[2]
        s[3,2]=(0.974312579539)*x[0]**o+(0.0374604823328)*x[0]+(-0.737361607733)*x[1]**o+(0.741091246445)*x[1]+(0.679379795149)*x[2]**o+(-0.904426984523)*x[2]
        s[3,3]=(-0.955715962012)*x[0]**o+(0.136457825359)*x[0]+(0.937865545819)*x[1]**o+(0.39839155392)*x[1]+(0.341983124988)*x[2]**o+(0.521155465952)*x[2]
        s[3,4]=(-0.89730837616)*x[0]**o+(0.615140171681)*x[0]+(-0.71054999877)*x[1]**o+(0.66258279405)*x[1]+(0.241482810749)*x[2]**o+(0.926085913167)*x[2]
        ref[0,0]=(0.127471865714)*x_ref[0]**o+(0.75668808568)*x_ref[0]+(0.0197793355564)*x_ref[1]**o+(0.342611017161)*x_ref[1]+(0.256637655934)*x_ref[2]**o+(0.144098222666)*x_ref[2]
        ref[0,1]=(0.460982544361)*x_ref[0]**o+(0.0758913155194)*x_ref[0]+(-0.738649205082)*x_ref[1]**o+(-0.729674519552)*x_ref[1]+(0.134554511124)*x_ref[2]**o+(-0.966054425176)*x_ref[2]
        ref[0,2]=(-0.515765669688)*x_ref[0]**o+(0.223475540597)*x_ref[0]+(0.710015035386)*x_ref[1]**o+(-0.810406212241)*x_ref[1]+(-0.706682492015)*x_ref[2]**o+(-0.176836116541)*x_ref[2]
        ref[0,3]=(0.849940074716)*x_ref[0]**o+(0.276008832632)*x_ref[0]+(0.753715668597)*x_ref[1]**o+(0.327023435021)*x_ref[1]+(-0.432843830378)*x_ref[2]**o+(-0.89188163433)*x_ref[2]
        ref[0,4]=(0.306934523362)*x_ref[0]**o+(0.400359334134)*x_ref[0]+(-0.994806829246)*x_ref[1]**o+(0.0349597549696)*x_ref[1]+(-0.978591943815)*x_ref[2]**o+(0.612187266226)*x_ref[2]
        ref[1,0]=(0.0461258495436)*x_ref[0]**o+(-0.959702481081)*x_ref[0]+(-0.3018982478)*x_ref[1]**o+(-0.462251712831)*x_ref[1]+(-0.94462432144)*x_ref[2]**o+(-0.506662578784)*x_ref[2]
        ref[1,1]=(0.233144525806)*x_ref[0]**o+(-0.414379147564)*x_ref[0]+(0.38512336385)*x_ref[1]**o+(-0.310763668907)*x_ref[1]+(0.48918840723)*x_ref[2]**o+(0.801469913207)*x_ref[2]
        ref[1,2]=(-0.132344623984)*x_ref[0]**o+(-0.242418158654)*x_ref[0]+(-0.618349833846)*x_ref[1]**o+(0.249538312346)*x_ref[1]+(0.466255623863)*x_ref[2]**o+(0.619476167885)*x_ref[2]
        ref[1,3]=(-0.600411057229)*x_ref[0]**o+(-0.65398469587)*x_ref[0]+(-0.086416622227)*x_ref[1]**o+(0.497186435444)*x_ref[1]+(0.490006868481)*x_ref[2]**o+(-0.281505238163)*x_ref[2]
        ref[1,4]=(-0.833074114751)*x_ref[0]**o+(0.179085322144)*x_ref[0]+(-0.669109910245)*x_ref[1]**o+(0.853858313308)*x_ref[1]+(-0.474984175674)*x_ref[2]**o+(0.0145591425038)*x_ref[2]
        ref[2,0]=(-0.463438302204)*x_ref[0]**o+(-0.815969915511)*x_ref[0]+(0.958428072331)*x_ref[1]**o+(-0.760925159788)*x_ref[1]+(0.419348775332)*x_ref[2]**o+(-0.146777978724)*x_ref[2]
        ref[2,1]=(0.64963543058)*x_ref[0]**o+(0.713874315445)*x_ref[0]+(0.000795833519698)*x_ref[1]**o+(-0.216757220289)*x_ref[1]+(-0.496572329247)*x_ref[2]**o+(0.535744506796)*x_ref[2]
        ref[2,2]=(0.848552125819)*x_ref[0]**o+(0.61034402517)*x_ref[0]+(0.254155499518)*x_ref[1]**o+(0.584452254449)*x_ref[1]+(-0.739215490349)*x_ref[2]**o+(0.553754902653)*x_ref[2]
        ref[2,3]=(0.45662694476)*x_ref[0]**o+(0.63839923138)*x_ref[0]+(0.05783248842)*x_ref[1]**o+(0.326555670737)*x_ref[1]+(-0.231728870265)*x_ref[2]**o+(-0.773309888086)*x_ref[2]
        ref[2,4]=(-0.848527683977)*x_ref[0]**o+(0.445666786862)*x_ref[0]+(0.96357445665)*x_ref[1]**o+(0.805189746125)*x_ref[1]+(-0.99455326053)*x_ref[2]**o+(-0.149519744556)*x_ref[2]
        ref[3,0]=(0.537998019205)*x_ref[0]**o+(-0.160515409613)*x_ref[0]+(-0.518616417074)*x_ref[1]**o+(-0.78931847859)*x_ref[1]+(0.533911685732)*x_ref[2]**o+(0.249570769302)*x_ref[2]
        ref[3,1]=(0.127551558633)*x_ref[0]**o+(-0.152792201104)*x_ref[0]+(-0.177992097046)*x_ref[1]**o+(-0.582313615383)*x_ref[1]+(0.180383484767)*x_ref[2]**o+(0.457671553174)*x_ref[2]
        ref[3,2]=(0.974312579539)*x_ref[0]**o+(0.0374604823328)*x_ref[0]+(-0.737361607733)*x_ref[1]**o+(0.741091246445)*x_ref[1]+(0.679379795149)*x_ref[2]**o+(-0.904426984523)*x_ref[2]
        ref[3,3]=(-0.955715962012)*x_ref[0]**o+(0.136457825359)*x_ref[0]+(0.937865545819)*x_ref[1]**o+(0.39839155392)*x_ref[1]+(0.341983124988)*x_ref[2]**o+(0.521155465952)*x_ref[2]
        ref[3,4]=(-0.89730837616)*x_ref[0]**o+(0.615140171681)*x_ref[0]+(-0.71054999877)*x_ref[1]**o+(0.66258279405)*x_ref[1]+(0.241482810749)*x_ref[2]**o+(0.926085913167)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromSymbol_ReducedFunctionOnContactOne_rank3(self):
      """
      tests interpolate for rank 3 Symbol onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactOne onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactOne(self.domain)
      x=w.getX()
      arg=Symbol(shape=(6, 2, 2),dim=dim)
      s=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2),w_ref)
      if dim==2:
        s[0,0,0]=(0.525970841909)*x[0]**o+(-0.724518605704)*x[0]+(0.0391713787427)*x[1]**o+(0.64004594365)*x[1]
        s[0,0,1]=(-0.601113924997)*x[0]**o+(0.654303363817)*x[0]+(0.830774705813)*x[1]**o+(-0.28075712655)*x[1]
        s[0,1,0]=(0.623015609082)*x[0]**o+(-0.336106614274)*x[0]+(0.184004307562)*x[1]**o+(0.34459670048)*x[1]
        s[0,1,1]=(0.369244348185)*x[0]**o+(-0.340034156943)*x[0]+(-0.263728841525)*x[1]**o+(0.395362860504)*x[1]
        s[1,0,0]=(-0.128245343573)*x[0]**o+(0.321050080112)*x[0]+(-0.759828911874)*x[1]**o+(-0.302784065212)*x[1]
        s[1,0,1]=(0.831781567583)*x[0]**o+(-0.695072817532)*x[0]+(0.765831248069)*x[1]**o+(-0.769559204064)*x[1]
        s[1,1,0]=(0.0231358105158)*x[0]**o+(-0.954466637933)*x[0]+(0.662613084093)*x[1]**o+(0.0859686703063)*x[1]
        s[1,1,1]=(-0.521154903312)*x[0]**o+(-0.556433610043)*x[0]+(0.185456230585)*x[1]**o+(0.033977276066)*x[1]
        s[2,0,0]=(0.629879646753)*x[0]**o+(-0.65521892079)*x[0]+(-0.208826471274)*x[1]**o+(0.737149257339)*x[1]
        s[2,0,1]=(-0.736126565138)*x[0]**o+(0.804035681207)*x[0]+(0.498224979538)*x[1]**o+(-0.959078001162)*x[1]
        s[2,1,0]=(0.286852738029)*x[0]**o+(0.353266502016)*x[0]+(0.940928485256)*x[1]**o+(0.28469674407)*x[1]
        s[2,1,1]=(0.275719311131)*x[0]**o+(0.696627217856)*x[0]+(-0.502165548001)*x[1]**o+(-0.520059660264)*x[1]
        s[3,0,0]=(0.658599467166)*x[0]**o+(-0.0350827779091)*x[0]+(-0.523903710086)*x[1]**o+(-0.844505620942)*x[1]
        s[3,0,1]=(0.480789616648)*x[0]**o+(0.276061116127)*x[0]+(-0.859175724597)*x[1]**o+(0.244549318718)*x[1]
        s[3,1,0]=(-0.542810121335)*x[0]**o+(-0.390621513538)*x[0]+(-0.229917193592)*x[1]**o+(-0.431941808228)*x[1]
        s[3,1,1]=(-0.739568286087)*x[0]**o+(-0.225719095703)*x[0]+(-0.366632490383)*x[1]**o+(-0.342423278749)*x[1]
        s[4,0,0]=(-0.348948968234)*x[0]**o+(0.667848620388)*x[0]+(-0.796730440327)*x[1]**o+(-0.391964865505)*x[1]
        s[4,0,1]=(0.86356141782)*x[0]**o+(0.479677832086)*x[0]+(-0.847297501426)*x[1]**o+(0.184369750148)*x[1]
        s[4,1,0]=(0.239661221119)*x[0]**o+(-0.080303273937)*x[0]+(0.748873062459)*x[1]**o+(-0.330109046822)*x[1]
        s[4,1,1]=(0.24720891844)*x[0]**o+(0.955624455873)*x[0]+(0.916553579019)*x[1]**o+(-0.636220767461)*x[1]
        s[5,0,0]=(0.23234510739)*x[0]**o+(-0.164278159674)*x[0]+(0.0850813643445)*x[1]**o+(0.603719147569)*x[1]
        s[5,0,1]=(-0.678009784208)*x[0]**o+(-0.439176295358)*x[0]+(-0.766581373483)*x[1]**o+(-0.754215528635)*x[1]
        s[5,1,0]=(0.587134653445)*x[0]**o+(-0.0529124327833)*x[0]+(-0.106376381667)*x[1]**o+(-0.606613833859)*x[1]
        s[5,1,1]=(0.0165841802044)*x[0]**o+(0.229033641436)*x[0]+(-0.00243791930366)*x[1]**o+(0.405659544043)*x[1]
        ref[0,0,0]=(0.525970841909)*x_ref[0]**o+(-0.724518605704)*x_ref[0]+(0.0391713787427)*x_ref[1]**o+(0.64004594365)*x_ref[1]
        ref[0,0,1]=(-0.601113924997)*x_ref[0]**o+(0.654303363817)*x_ref[0]+(0.830774705813)*x_ref[1]**o+(-0.28075712655)*x_ref[1]
        ref[0,1,0]=(0.623015609082)*x_ref[0]**o+(-0.336106614274)*x_ref[0]+(0.184004307562)*x_ref[1]**o+(0.34459670048)*x_ref[1]
        ref[0,1,1]=(0.369244348185)*x_ref[0]**o+(-0.340034156943)*x_ref[0]+(-0.263728841525)*x_ref[1]**o+(0.395362860504)*x_ref[1]
        ref[1,0,0]=(-0.128245343573)*x_ref[0]**o+(0.321050080112)*x_ref[0]+(-0.759828911874)*x_ref[1]**o+(-0.302784065212)*x_ref[1]
        ref[1,0,1]=(0.831781567583)*x_ref[0]**o+(-0.695072817532)*x_ref[0]+(0.765831248069)*x_ref[1]**o+(-0.769559204064)*x_ref[1]
        ref[1,1,0]=(0.0231358105158)*x_ref[0]**o+(-0.954466637933)*x_ref[0]+(0.662613084093)*x_ref[1]**o+(0.0859686703063)*x_ref[1]
        ref[1,1,1]=(-0.521154903312)*x_ref[0]**o+(-0.556433610043)*x_ref[0]+(0.185456230585)*x_ref[1]**o+(0.033977276066)*x_ref[1]
        ref[2,0,0]=(0.629879646753)*x_ref[0]**o+(-0.65521892079)*x_ref[0]+(-0.208826471274)*x_ref[1]**o+(0.737149257339)*x_ref[1]
        ref[2,0,1]=(-0.736126565138)*x_ref[0]**o+(0.804035681207)*x_ref[0]+(0.498224979538)*x_ref[1]**o+(-0.959078001162)*x_ref[1]
        ref[2,1,0]=(0.286852738029)*x_ref[0]**o+(0.353266502016)*x_ref[0]+(0.940928485256)*x_ref[1]**o+(0.28469674407)*x_ref[1]
        ref[2,1,1]=(0.275719311131)*x_ref[0]**o+(0.696627217856)*x_ref[0]+(-0.502165548001)*x_ref[1]**o+(-0.520059660264)*x_ref[1]
        ref[3,0,0]=(0.658599467166)*x_ref[0]**o+(-0.0350827779091)*x_ref[0]+(-0.523903710086)*x_ref[1]**o+(-0.844505620942)*x_ref[1]
        ref[3,0,1]=(0.480789616648)*x_ref[0]**o+(0.276061116127)*x_ref[0]+(-0.859175724597)*x_ref[1]**o+(0.244549318718)*x_ref[1]
        ref[3,1,0]=(-0.542810121335)*x_ref[0]**o+(-0.390621513538)*x_ref[0]+(-0.229917193592)*x_ref[1]**o+(-0.431941808228)*x_ref[1]
        ref[3,1,1]=(-0.739568286087)*x_ref[0]**o+(-0.225719095703)*x_ref[0]+(-0.366632490383)*x_ref[1]**o+(-0.342423278749)*x_ref[1]
        ref[4,0,0]=(-0.348948968234)*x_ref[0]**o+(0.667848620388)*x_ref[0]+(-0.796730440327)*x_ref[1]**o+(-0.391964865505)*x_ref[1]
        ref[4,0,1]=(0.86356141782)*x_ref[0]**o+(0.479677832086)*x_ref[0]+(-0.847297501426)*x_ref[1]**o+(0.184369750148)*x_ref[1]
        ref[4,1,0]=(0.239661221119)*x_ref[0]**o+(-0.080303273937)*x_ref[0]+(0.748873062459)*x_ref[1]**o+(-0.330109046822)*x_ref[1]
        ref[4,1,1]=(0.24720891844)*x_ref[0]**o+(0.955624455873)*x_ref[0]+(0.916553579019)*x_ref[1]**o+(-0.636220767461)*x_ref[1]
        ref[5,0,0]=(0.23234510739)*x_ref[0]**o+(-0.164278159674)*x_ref[0]+(0.0850813643445)*x_ref[1]**o+(0.603719147569)*x_ref[1]
        ref[5,0,1]=(-0.678009784208)*x_ref[0]**o+(-0.439176295358)*x_ref[0]+(-0.766581373483)*x_ref[1]**o+(-0.754215528635)*x_ref[1]
        ref[5,1,0]=(0.587134653445)*x_ref[0]**o+(-0.0529124327833)*x_ref[0]+(-0.106376381667)*x_ref[1]**o+(-0.606613833859)*x_ref[1]
        ref[5,1,1]=(0.0165841802044)*x_ref[0]**o+(0.229033641436)*x_ref[0]+(-0.00243791930366)*x_ref[1]**o+(0.405659544043)*x_ref[1]
      else:
        s[0,0,0]=(0.320018603048)*x[0]**o+(-0.695020280658)*x[0]+(-0.255721430929)*x[1]**o+(0.295594488722)*x[1]+(-0.0298772897116)*x[2]**o+(-0.607777786149)*x[2]
        s[0,0,1]=(-0.0539156052122)*x[0]**o+(0.538262769125)*x[0]+(-0.111309490507)*x[1]**o+(-0.793327760235)*x[1]+(-0.789135505737)*x[2]**o+(0.200901242923)*x[2]
        s[0,1,0]=(-0.336577387203)*x[0]**o+(0.210285897539)*x[0]+(0.699559016034)*x[1]**o+(-0.916159806945)*x[1]+(-0.499612048693)*x[2]**o+(0.358485383896)*x[2]
        s[0,1,1]=(-0.47730332075)*x[0]**o+(-0.689133971745)*x[0]+(0.765673037239)*x[1]**o+(0.792799541442)*x[1]+(0.230271541384)*x[2]**o+(0.984973655354)*x[2]
        s[1,0,0]=(-0.338473674404)*x[0]**o+(-0.935684983364)*x[0]+(-0.138901982164)*x[1]**o+(0.699572795452)*x[1]+(-0.199407330193)*x[2]**o+(-0.520709372829)*x[2]
        s[1,0,1]=(0.317819732778)*x[0]**o+(-0.845493146327)*x[0]+(0.905916805401)*x[1]**o+(0.432912349711)*x[1]+(-0.0895289935052)*x[2]**o+(0.585373526078)*x[2]
        s[1,1,0]=(0.946256506041)*x[0]**o+(0.579530928092)*x[0]+(-0.129820000715)*x[1]**o+(-0.80018614234)*x[1]+(-0.672241403531)*x[2]**o+(-0.429302115886)*x[2]
        s[1,1,1]=(0.841539001381)*x[0]**o+(0.91335238179)*x[0]+(-0.886657983812)*x[1]**o+(-0.714058529158)*x[1]+(-0.478059374968)*x[2]**o+(0.289902349534)*x[2]
        s[2,0,0]=(0.716479906976)*x[0]**o+(0.879172628545)*x[0]+(0.702298309498)*x[1]**o+(-0.111784591727)*x[1]+(0.40144635898)*x[2]**o+(-0.234783818652)*x[2]
        s[2,0,1]=(0.122997020314)*x[0]**o+(0.241190251335)*x[0]+(0.814657166415)*x[1]**o+(0.282560032124)*x[1]+(-0.0356499295276)*x[2]**o+(0.166315448017)*x[2]
        s[2,1,0]=(-0.881008094539)*x[0]**o+(0.542764181702)*x[0]+(0.242598400741)*x[1]**o+(0.993781493144)*x[1]+(-0.351534490163)*x[2]**o+(-0.104396569846)*x[2]
        s[2,1,1]=(-0.510128022031)*x[0]**o+(-0.215966730947)*x[0]+(0.393015726048)*x[1]**o+(-0.397942881289)*x[1]+(-0.195641507854)*x[2]**o+(0.266488215631)*x[2]
        s[3,0,0]=(-0.998764822575)*x[0]**o+(0.375416908499)*x[0]+(-0.874575087088)*x[1]**o+(0.0988447992659)*x[1]+(0.424141721707)*x[2]**o+(0.628127032413)*x[2]
        s[3,0,1]=(0.599442581541)*x[0]**o+(-0.618344049198)*x[0]+(-0.496506175349)*x[1]**o+(-0.481350180732)*x[1]+(-0.200782615612)*x[2]**o+(0.0201867652613)*x[2]
        s[3,1,0]=(-0.148594055575)*x[0]**o+(-0.806224651257)*x[0]+(-0.204482720049)*x[1]**o+(-0.49019313701)*x[1]+(0.335531499413)*x[2]**o+(-0.834246885478)*x[2]
        s[3,1,1]=(0.641941231569)*x[0]**o+(-0.90604061382)*x[0]+(0.540142802942)*x[1]**o+(0.72231278143)*x[1]+(-0.859060377851)*x[2]**o+(0.324573297095)*x[2]
        s[4,0,0]=(0.720107219881)*x[0]**o+(-0.62138573659)*x[0]+(-0.226330947503)*x[1]**o+(-0.624699498209)*x[1]+(0.539194640961)*x[2]**o+(0.852063240152)*x[2]
        s[4,0,1]=(0.0942892316041)*x[0]**o+(0.993373220728)*x[0]+(-0.0966784587501)*x[1]**o+(-0.683786013291)*x[1]+(-0.0361512012678)*x[2]**o+(-0.393598599894)*x[2]
        s[4,1,0]=(0.601453506151)*x[0]**o+(-0.307251826677)*x[0]+(0.966431194544)*x[1]**o+(0.244859574503)*x[1]+(-0.965624233728)*x[2]**o+(-0.354176955651)*x[2]
        s[4,1,1]=(0.30845293409)*x[0]**o+(0.485784959358)*x[0]+(-0.339346667554)*x[1]**o+(0.882000066104)*x[1]+(-0.548144598203)*x[2]**o+(-0.872275137718)*x[2]
        s[5,0,0]=(0.0264929806889)*x[0]**o+(-0.420817540591)*x[0]+(0.542643032455)*x[1]**o+(-0.509400295496)*x[1]+(-0.303533536712)*x[2]**o+(0.909439372034)*x[2]
        s[5,0,1]=(0.302318680709)*x[0]**o+(0.610414720074)*x[0]+(-0.253744196335)*x[1]**o+(0.0864740115897)*x[1]+(0.774183311916)*x[2]**o+(0.188320998163)*x[2]
        s[5,1,0]=(-0.341811432391)*x[0]**o+(-0.0568308040717)*x[0]+(0.807309619252)*x[1]**o+(0.947557389711)*x[1]+(-0.142294557146)*x[2]**o+(0.345071619119)*x[2]
        s[5,1,1]=(-0.059329629623)*x[0]**o+(-0.767819910895)*x[0]+(-0.0992852685518)*x[1]**o+(0.118514635891)*x[1]+(0.584899765205)*x[2]**o+(-0.924652287665)*x[2]
        ref[0,0,0]=(0.320018603048)*x_ref[0]**o+(-0.695020280658)*x_ref[0]+(-0.255721430929)*x_ref[1]**o+(0.295594488722)*x_ref[1]+(-0.0298772897116)*x_ref[2]**o+(-0.607777786149)*x_ref[2]
        ref[0,0,1]=(-0.0539156052122)*x_ref[0]**o+(0.538262769125)*x_ref[0]+(-0.111309490507)*x_ref[1]**o+(-0.793327760235)*x_ref[1]+(-0.789135505737)*x_ref[2]**o+(0.200901242923)*x_ref[2]
        ref[0,1,0]=(-0.336577387203)*x_ref[0]**o+(0.210285897539)*x_ref[0]+(0.699559016034)*x_ref[1]**o+(-0.916159806945)*x_ref[1]+(-0.499612048693)*x_ref[2]**o+(0.358485383896)*x_ref[2]
        ref[0,1,1]=(-0.47730332075)*x_ref[0]**o+(-0.689133971745)*x_ref[0]+(0.765673037239)*x_ref[1]**o+(0.792799541442)*x_ref[1]+(0.230271541384)*x_ref[2]**o+(0.984973655354)*x_ref[2]
        ref[1,0,0]=(-0.338473674404)*x_ref[0]**o+(-0.935684983364)*x_ref[0]+(-0.138901982164)*x_ref[1]**o+(0.699572795452)*x_ref[1]+(-0.199407330193)*x_ref[2]**o+(-0.520709372829)*x_ref[2]
        ref[1,0,1]=(0.317819732778)*x_ref[0]**o+(-0.845493146327)*x_ref[0]+(0.905916805401)*x_ref[1]**o+(0.432912349711)*x_ref[1]+(-0.0895289935052)*x_ref[2]**o+(0.585373526078)*x_ref[2]
        ref[1,1,0]=(0.946256506041)*x_ref[0]**o+(0.579530928092)*x_ref[0]+(-0.129820000715)*x_ref[1]**o+(-0.80018614234)*x_ref[1]+(-0.672241403531)*x_ref[2]**o+(-0.429302115886)*x_ref[2]
        ref[1,1,1]=(0.841539001381)*x_ref[0]**o+(0.91335238179)*x_ref[0]+(-0.886657983812)*x_ref[1]**o+(-0.714058529158)*x_ref[1]+(-0.478059374968)*x_ref[2]**o+(0.289902349534)*x_ref[2]
        ref[2,0,0]=(0.716479906976)*x_ref[0]**o+(0.879172628545)*x_ref[0]+(0.702298309498)*x_ref[1]**o+(-0.111784591727)*x_ref[1]+(0.40144635898)*x_ref[2]**o+(-0.234783818652)*x_ref[2]
        ref[2,0,1]=(0.122997020314)*x_ref[0]**o+(0.241190251335)*x_ref[0]+(0.814657166415)*x_ref[1]**o+(0.282560032124)*x_ref[1]+(-0.0356499295276)*x_ref[2]**o+(0.166315448017)*x_ref[2]
        ref[2,1,0]=(-0.881008094539)*x_ref[0]**o+(0.542764181702)*x_ref[0]+(0.242598400741)*x_ref[1]**o+(0.993781493144)*x_ref[1]+(-0.351534490163)*x_ref[2]**o+(-0.104396569846)*x_ref[2]
        ref[2,1,1]=(-0.510128022031)*x_ref[0]**o+(-0.215966730947)*x_ref[0]+(0.393015726048)*x_ref[1]**o+(-0.397942881289)*x_ref[1]+(-0.195641507854)*x_ref[2]**o+(0.266488215631)*x_ref[2]
        ref[3,0,0]=(-0.998764822575)*x_ref[0]**o+(0.375416908499)*x_ref[0]+(-0.874575087088)*x_ref[1]**o+(0.0988447992659)*x_ref[1]+(0.424141721707)*x_ref[2]**o+(0.628127032413)*x_ref[2]
        ref[3,0,1]=(0.599442581541)*x_ref[0]**o+(-0.618344049198)*x_ref[0]+(-0.496506175349)*x_ref[1]**o+(-0.481350180732)*x_ref[1]+(-0.200782615612)*x_ref[2]**o+(0.0201867652613)*x_ref[2]
        ref[3,1,0]=(-0.148594055575)*x_ref[0]**o+(-0.806224651257)*x_ref[0]+(-0.204482720049)*x_ref[1]**o+(-0.49019313701)*x_ref[1]+(0.335531499413)*x_ref[2]**o+(-0.834246885478)*x_ref[2]
        ref[3,1,1]=(0.641941231569)*x_ref[0]**o+(-0.90604061382)*x_ref[0]+(0.540142802942)*x_ref[1]**o+(0.72231278143)*x_ref[1]+(-0.859060377851)*x_ref[2]**o+(0.324573297095)*x_ref[2]
        ref[4,0,0]=(0.720107219881)*x_ref[0]**o+(-0.62138573659)*x_ref[0]+(-0.226330947503)*x_ref[1]**o+(-0.624699498209)*x_ref[1]+(0.539194640961)*x_ref[2]**o+(0.852063240152)*x_ref[2]
        ref[4,0,1]=(0.0942892316041)*x_ref[0]**o+(0.993373220728)*x_ref[0]+(-0.0966784587501)*x_ref[1]**o+(-0.683786013291)*x_ref[1]+(-0.0361512012678)*x_ref[2]**o+(-0.393598599894)*x_ref[2]
        ref[4,1,0]=(0.601453506151)*x_ref[0]**o+(-0.307251826677)*x_ref[0]+(0.966431194544)*x_ref[1]**o+(0.244859574503)*x_ref[1]+(-0.965624233728)*x_ref[2]**o+(-0.354176955651)*x_ref[2]
        ref[4,1,1]=(0.30845293409)*x_ref[0]**o+(0.485784959358)*x_ref[0]+(-0.339346667554)*x_ref[1]**o+(0.882000066104)*x_ref[1]+(-0.548144598203)*x_ref[2]**o+(-0.872275137718)*x_ref[2]
        ref[5,0,0]=(0.0264929806889)*x_ref[0]**o+(-0.420817540591)*x_ref[0]+(0.542643032455)*x_ref[1]**o+(-0.509400295496)*x_ref[1]+(-0.303533536712)*x_ref[2]**o+(0.909439372034)*x_ref[2]
        ref[5,0,1]=(0.302318680709)*x_ref[0]**o+(0.610414720074)*x_ref[0]+(-0.253744196335)*x_ref[1]**o+(0.0864740115897)*x_ref[1]+(0.774183311916)*x_ref[2]**o+(0.188320998163)*x_ref[2]
        ref[5,1,0]=(-0.341811432391)*x_ref[0]**o+(-0.0568308040717)*x_ref[0]+(0.807309619252)*x_ref[1]**o+(0.947557389711)*x_ref[1]+(-0.142294557146)*x_ref[2]**o+(0.345071619119)*x_ref[2]
        ref[5,1,1]=(-0.059329629623)*x_ref[0]**o+(-0.767819910895)*x_ref[0]+(-0.0992852685518)*x_ref[1]**o+(0.118514635891)*x_ref[1]+(0.584899765205)*x_ref[2]**o+(-0.924652287665)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromSymbol_ReducedFunctionOnContactOne_rank4(self):
      """
      tests interpolate for rank 4 Symbol onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactOne onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactOne(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5, 3, 2),dim=dim)
      s=Data(0,(4, 5, 3, 2),w)
      ref=Data(0,(4, 5, 3, 2),w_ref)
      if dim==2:
        s[0,0,0,0]=(0.286414445959)*x[0]**o+(-0.513602739404)*x[0]+(0.415115217751)*x[1]**o+(0.302847208656)*x[1]
        s[0,0,0,1]=(0.259932949392)*x[0]**o+(0.35051585712)*x[0]+(-0.199459717408)*x[1]**o+(0.243907820471)*x[1]
        s[0,0,1,0]=(-0.118977243299)*x[0]**o+(-0.42706283684)*x[0]+(0.538165751605)*x[1]**o+(0.393481400689)*x[1]
        s[0,0,1,1]=(-0.0397069035535)*x[0]**o+(0.861872317511)*x[0]+(-0.15731307612)*x[1]**o+(0.693002169125)*x[1]
        s[0,0,2,0]=(0.815645799181)*x[0]**o+(-0.0776314502273)*x[0]+(-0.593034585971)*x[1]**o+(-0.970680448911)*x[1]
        s[0,0,2,1]=(-0.952302578078)*x[0]**o+(0.651928073409)*x[0]+(-0.239714652084)*x[1]**o+(-0.952274708749)*x[1]
        s[0,1,0,0]=(-0.316151000005)*x[0]**o+(0.216061626502)*x[0]+(-0.882718070647)*x[1]**o+(-0.671547751668)*x[1]
        s[0,1,0,1]=(-0.399909471112)*x[0]**o+(-0.318040874639)*x[0]+(-0.907464404526)*x[1]**o+(0.233304604462)*x[1]
        s[0,1,1,0]=(0.774423529257)*x[0]**o+(0.364389715996)*x[0]+(-0.842201054073)*x[1]**o+(0.404586304258)*x[1]
        s[0,1,1,1]=(0.504496704492)*x[0]**o+(0.622221083753)*x[0]+(0.12478240068)*x[1]**o+(0.353949438798)*x[1]
        s[0,1,2,0]=(-0.845828936398)*x[0]**o+(-0.165296386039)*x[0]+(0.486358102378)*x[1]**o+(-0.761128414131)*x[1]
        s[0,1,2,1]=(-0.587533689601)*x[0]**o+(0.889232467387)*x[0]+(-0.17030097273)*x[1]**o+(-0.939366655195)*x[1]
        s[0,2,0,0]=(-0.583544977678)*x[0]**o+(0.49701340766)*x[0]+(-0.59251363345)*x[1]**o+(0.632063421618)*x[1]
        s[0,2,0,1]=(0.159662476533)*x[0]**o+(-0.926024134426)*x[0]+(-0.359267295402)*x[1]**o+(0.575272568112)*x[1]
        s[0,2,1,0]=(-0.737677964249)*x[0]**o+(0.828179798925)*x[0]+(0.487366022336)*x[1]**o+(-0.80790431335)*x[1]
        s[0,2,1,1]=(-0.633849216356)*x[0]**o+(0.127289254283)*x[0]+(0.282991845718)*x[1]**o+(0.821227139666)*x[1]
        s[0,2,2,0]=(-0.821316223164)*x[0]**o+(0.979879665311)*x[0]+(-0.137787756007)*x[1]**o+(-0.277486407169)*x[1]
        s[0,2,2,1]=(-0.622980962419)*x[0]**o+(-0.696479459661)*x[0]+(-0.410742216866)*x[1]**o+(0.576030773935)*x[1]
        s[0,3,0,0]=(-0.207619411473)*x[0]**o+(-0.469849947705)*x[0]+(-0.671957494397)*x[1]**o+(0.254338352433)*x[1]
        s[0,3,0,1]=(0.243280252346)*x[0]**o+(0.133824860089)*x[0]+(0.0696460542307)*x[1]**o+(0.455743386064)*x[1]
        s[0,3,1,0]=(-0.906691692651)*x[0]**o+(0.787810130121)*x[0]+(-0.219233151952)*x[1]**o+(0.743750472953)*x[1]
        s[0,3,1,1]=(-0.250516865682)*x[0]**o+(-0.780254096438)*x[0]+(-0.739910422511)*x[1]**o+(-0.792068328341)*x[1]
        s[0,3,2,0]=(0.0232686870041)*x[0]**o+(0.0106534323439)*x[0]+(-0.0361419376311)*x[1]**o+(0.566265323633)*x[1]
        s[0,3,2,1]=(-0.638354270703)*x[0]**o+(-0.255988140752)*x[0]+(-0.261309974545)*x[1]**o+(-0.697773476841)*x[1]
        s[0,4,0,0]=(-0.381395765807)*x[0]**o+(-0.365270978705)*x[0]+(0.812150815593)*x[1]**o+(0.719382395957)*x[1]
        s[0,4,0,1]=(0.69594272856)*x[0]**o+(0.278887033989)*x[0]+(0.754272027542)*x[1]**o+(0.793061109177)*x[1]
        s[0,4,1,0]=(0.967293155949)*x[0]**o+(-0.840326330307)*x[0]+(0.299320013494)*x[1]**o+(0.319888429035)*x[1]
        s[0,4,1,1]=(-0.0509734884321)*x[0]**o+(-0.936136061318)*x[0]+(-0.98628193626)*x[1]**o+(0.661193529996)*x[1]
        s[0,4,2,0]=(0.562813543077)*x[0]**o+(0.907184253714)*x[0]+(-0.136123729129)*x[1]**o+(-0.986466864564)*x[1]
        s[0,4,2,1]=(-0.434457154258)*x[0]**o+(-0.574859203931)*x[0]+(-0.383505033613)*x[1]**o+(0.331906199764)*x[1]
        s[1,0,0,0]=(0.0611272547169)*x[0]**o+(-0.136522073182)*x[0]+(0.417326897823)*x[1]**o+(-0.890085105585)*x[1]
        s[1,0,0,1]=(-0.869426624414)*x[0]**o+(-0.186222609222)*x[0]+(0.339956750726)*x[1]**o+(-0.226819210334)*x[1]
        s[1,0,1,0]=(0.399952324098)*x[0]**o+(0.798209020313)*x[0]+(-0.521148352596)*x[1]**o+(0.458003374007)*x[1]
        s[1,0,1,1]=(0.501460721237)*x[0]**o+(0.231248731021)*x[0]+(0.332539535549)*x[1]**o+(0.185347952715)*x[1]
        s[1,0,2,0]=(-0.230801196498)*x[0]**o+(0.823765804775)*x[0]+(-0.258377195949)*x[1]**o+(0.224645264709)*x[1]
        s[1,0,2,1]=(-0.241372395526)*x[0]**o+(0.905482005475)*x[0]+(0.0493139366957)*x[1]**o+(0.541602080684)*x[1]
        s[1,1,0,0]=(0.449706009608)*x[0]**o+(0.359879546784)*x[0]+(-0.493158517575)*x[1]**o+(0.191770207113)*x[1]
        s[1,1,0,1]=(0.957438236069)*x[0]**o+(-0.59909754527)*x[0]+(-0.614098873549)*x[1]**o+(0.0834466945634)*x[1]
        s[1,1,1,0]=(0.146023068688)*x[0]**o+(-0.185385831893)*x[0]+(-0.805313629764)*x[1]**o+(0.230766437006)*x[1]
        s[1,1,1,1]=(0.306071651757)*x[0]**o+(0.921518467891)*x[0]+(0.716057571126)*x[1]**o+(-0.522119477993)*x[1]
        s[1,1,2,0]=(0.684162336448)*x[0]**o+(-0.388731324307)*x[0]+(0.600372268011)*x[1]**o+(-0.771944030767)*x[1]
        s[1,1,2,1]=(0.286204844749)*x[0]**o+(0.400092667734)*x[0]+(0.506091236432)*x[1]**o+(0.858615249035)*x[1]
        s[1,2,0,0]=(-0.786850947851)*x[0]**o+(0.0317031686489)*x[0]+(-0.0562719978878)*x[1]**o+(-0.368745220216)*x[1]
        s[1,2,0,1]=(-0.062898974467)*x[0]**o+(-0.397876556176)*x[0]+(-0.117046688506)*x[1]**o+(-0.548475481806)*x[1]
        s[1,2,1,0]=(-0.478254619878)*x[0]**o+(0.866928591218)*x[0]+(0.4475948677)*x[1]**o+(0.393318603064)*x[1]
        s[1,2,1,1]=(-0.351975278856)*x[0]**o+(-0.377161089123)*x[0]+(0.492009299907)*x[1]**o+(0.569876619926)*x[1]
        s[1,2,2,0]=(-0.556131924542)*x[0]**o+(0.603608830074)*x[0]+(0.409486637292)*x[1]**o+(0.796280414552)*x[1]
        s[1,2,2,1]=(0.52135916937)*x[0]**o+(0.760618210549)*x[0]+(-0.145180244853)*x[1]**o+(0.549313491191)*x[1]
        s[1,3,0,0]=(-0.577089151087)*x[0]**o+(0.821891072229)*x[0]+(0.332425052183)*x[1]**o+(-0.151504483932)*x[1]
        s[1,3,0,1]=(-0.966703491954)*x[0]**o+(0.13997287848)*x[0]+(-0.682812558867)*x[1]**o+(-0.465013684448)*x[1]
        s[1,3,1,0]=(0.908453154364)*x[0]**o+(0.267997575637)*x[0]+(0.587549759517)*x[1]**o+(-0.487274402304)*x[1]
        s[1,3,1,1]=(-0.38747875298)*x[0]**o+(-0.685908980542)*x[0]+(-0.350387154367)*x[1]**o+(0.621711654152)*x[1]
        s[1,3,2,0]=(-0.148266632644)*x[0]**o+(-0.580283160892)*x[0]+(-0.261751144414)*x[1]**o+(0.502757383658)*x[1]
        s[1,3,2,1]=(0.610418736064)*x[0]**o+(0.591832308717)*x[0]+(0.869463161016)*x[1]**o+(0.252614326127)*x[1]
        s[1,4,0,0]=(0.683334561136)*x[0]**o+(0.484163667966)*x[0]+(0.254191241985)*x[1]**o+(0.039976829904)*x[1]
        s[1,4,0,1]=(-0.79220316614)*x[0]**o+(0.749008242295)*x[0]+(-0.539372369578)*x[1]**o+(-0.900518079782)*x[1]
        s[1,4,1,0]=(0.576352091657)*x[0]**o+(0.237846036342)*x[0]+(0.000667246779082)*x[1]**o+(-0.709742732387)*x[1]
        s[1,4,1,1]=(-0.676810333712)*x[0]**o+(0.847889875816)*x[0]+(-0.688138508533)*x[1]**o+(0.133158683641)*x[1]
        s[1,4,2,0]=(0.99415007686)*x[0]**o+(-0.141452245889)*x[0]+(0.775573627086)*x[1]**o+(0.452458721895)*x[1]
        s[1,4,2,1]=(0.994145376118)*x[0]**o+(0.395589592556)*x[0]+(-0.669488954253)*x[1]**o+(0.067046815669)*x[1]
        s[2,0,0,0]=(-0.750281322814)*x[0]**o+(-0.309320623684)*x[0]+(0.626517958661)*x[1]**o+(0.597498890162)*x[1]
        s[2,0,0,1]=(0.271076040325)*x[0]**o+(0.667103080861)*x[0]+(0.422066039225)*x[1]**o+(0.363852331279)*x[1]
        s[2,0,1,0]=(-0.273421011656)*x[0]**o+(-0.0619409465417)*x[0]+(0.0679232143441)*x[1]**o+(-0.720428826448)*x[1]
        s[2,0,1,1]=(-0.647493736809)*x[0]**o+(0.927792745023)*x[0]+(0.888263709553)*x[1]**o+(-0.613668040672)*x[1]
        s[2,0,2,0]=(0.576488068023)*x[0]**o+(0.389605065338)*x[0]+(0.273385863145)*x[1]**o+(0.528205453441)*x[1]
        s[2,0,2,1]=(-0.906661332702)*x[0]**o+(0.0725964985998)*x[0]+(0.184026573733)*x[1]**o+(0.72102096328)*x[1]
        s[2,1,0,0]=(0.0561237068662)*x[0]**o+(0.705088464117)*x[0]+(-0.92649218582)*x[1]**o+(-0.392584665998)*x[1]
        s[2,1,0,1]=(0.590410488233)*x[0]**o+(-0.599541782377)*x[0]+(0.739195238446)*x[1]**o+(-0.388992225425)*x[1]
        s[2,1,1,0]=(0.382347893509)*x[0]**o+(0.65529075645)*x[0]+(0.6281938054)*x[1]**o+(0.122434736283)*x[1]
        s[2,1,1,1]=(-0.286086668538)*x[0]**o+(-0.104121795484)*x[0]+(-0.282055217155)*x[1]**o+(0.173105945475)*x[1]
        s[2,1,2,0]=(-0.287280683011)*x[0]**o+(-0.963651344223)*x[0]+(-0.940112522168)*x[1]**o+(-0.788180434371)*x[1]
        s[2,1,2,1]=(0.61738130828)*x[0]**o+(-0.427708968146)*x[0]+(0.319738892701)*x[1]**o+(0.419063299626)*x[1]
        s[2,2,0,0]=(-0.0499926543896)*x[0]**o+(0.01377519232)*x[0]+(-0.325826729461)*x[1]**o+(-0.392085884375)*x[1]
        s[2,2,0,1]=(-0.0581765276804)*x[0]**o+(-0.208263386025)*x[0]+(0.75198359812)*x[1]**o+(0.97252999759)*x[1]
        s[2,2,1,0]=(-0.726087111179)*x[0]**o+(-0.0573158936919)*x[0]+(-0.742825725839)*x[1]**o+(0.415476090214)*x[1]
        s[2,2,1,1]=(0.917838486533)*x[0]**o+(-0.091211527759)*x[0]+(0.507645127218)*x[1]**o+(-0.742224440501)*x[1]
        s[2,2,2,0]=(-0.610881095534)*x[0]**o+(-0.864358502475)*x[0]+(0.0808508216725)*x[1]**o+(0.199006479357)*x[1]
        s[2,2,2,1]=(-0.946103419369)*x[0]**o+(0.491509709702)*x[0]+(0.612215069813)*x[1]**o+(0.418509110447)*x[1]
        s[2,3,0,0]=(0.0741360472802)*x[0]**o+(-0.994432396766)*x[0]+(-0.332924771563)*x[1]**o+(-0.753108051656)*x[1]
        s[2,3,0,1]=(-0.381576097006)*x[0]**o+(-0.377802667887)*x[0]+(-0.0303870681199)*x[1]**o+(-0.345561266945)*x[1]
        s[2,3,1,0]=(0.642729104556)*x[0]**o+(0.570036794833)*x[0]+(-0.868604403383)*x[1]**o+(-0.671162161412)*x[1]
        s[2,3,1,1]=(0.335963731776)*x[0]**o+(-0.504092976502)*x[0]+(-0.980540079107)*x[1]**o+(0.354777629497)*x[1]
        s[2,3,2,0]=(-0.764056326006)*x[0]**o+(-0.416057085413)*x[0]+(-0.690778483271)*x[1]**o+(-0.760474373055)*x[1]
        s[2,3,2,1]=(0.901089622394)*x[0]**o+(0.13387449325)*x[0]+(0.0138079678955)*x[1]**o+(0.839892772097)*x[1]
        s[2,4,0,0]=(-0.650061300028)*x[0]**o+(-0.349615966399)*x[0]+(0.790768704094)*x[1]**o+(-0.42814753068)*x[1]
        s[2,4,0,1]=(-0.0418088165021)*x[0]**o+(0.150269895143)*x[0]+(-0.529846432011)*x[1]**o+(-0.224508824625)*x[1]
        s[2,4,1,0]=(-0.872715524736)*x[0]**o+(-0.968213567754)*x[0]+(-0.485442904703)*x[1]**o+(-0.201353147903)*x[1]
        s[2,4,1,1]=(0.807375853776)*x[0]**o+(-0.555765909033)*x[0]+(0.509490325151)*x[1]**o+(-0.0402998342759)*x[1]
        s[2,4,2,0]=(0.976103420432)*x[0]**o+(-0.31420590756)*x[0]+(0.965906601649)*x[1]**o+(0.996838317987)*x[1]
        s[2,4,2,1]=(0.927750446685)*x[0]**o+(-0.0669100154417)*x[0]+(0.329400978068)*x[1]**o+(0.673620795081)*x[1]
        s[3,0,0,0]=(0.0339658367621)*x[0]**o+(-0.69057366167)*x[0]+(-0.84841361493)*x[1]**o+(-0.793870889269)*x[1]
        s[3,0,0,1]=(0.580299117541)*x[0]**o+(0.711568306729)*x[0]+(0.68652281169)*x[1]**o+(-0.893997742165)*x[1]
        s[3,0,1,0]=(0.472119483925)*x[0]**o+(0.811060953315)*x[0]+(0.131977099466)*x[1]**o+(0.252305475595)*x[1]
        s[3,0,1,1]=(0.825801074332)*x[0]**o+(-0.494721582584)*x[0]+(0.926019323719)*x[1]**o+(-0.398271744425)*x[1]
        s[3,0,2,0]=(0.456913500128)*x[0]**o+(-0.17032765188)*x[0]+(0.758934382849)*x[1]**o+(-0.260113957998)*x[1]
        s[3,0,2,1]=(-0.903049904081)*x[0]**o+(0.0173063807709)*x[0]+(-0.374982617974)*x[1]**o+(0.334268331811)*x[1]
        s[3,1,0,0]=(-0.22949882827)*x[0]**o+(-0.373561413774)*x[0]+(0.790631265088)*x[1]**o+(-0.75812217101)*x[1]
        s[3,1,0,1]=(0.564879113215)*x[0]**o+(0.33779824151)*x[0]+(0.0367512619169)*x[1]**o+(0.441450704065)*x[1]
        s[3,1,1,0]=(-0.428927604938)*x[0]**o+(-0.523613773862)*x[0]+(0.29478120955)*x[1]**o+(-0.863628205314)*x[1]
        s[3,1,1,1]=(0.459167152825)*x[0]**o+(-0.641899803541)*x[0]+(-0.109951392233)*x[1]**o+(0.399265290858)*x[1]
        s[3,1,2,0]=(0.682558303558)*x[0]**o+(-0.137784370501)*x[0]+(-0.393326255016)*x[1]**o+(-0.678503687693)*x[1]
        s[3,1,2,1]=(-0.0809452179949)*x[0]**o+(0.235685309616)*x[0]+(0.974274730596)*x[1]**o+(-0.894536588546)*x[1]
        s[3,2,0,0]=(-0.0490472374331)*x[0]**o+(-0.473644556364)*x[0]+(-0.382189831802)*x[1]**o+(-0.22504837111)*x[1]
        s[3,2,0,1]=(0.997893391083)*x[0]**o+(0.749295222878)*x[0]+(-0.799214431894)*x[1]**o+(0.57167392758)*x[1]
        s[3,2,1,0]=(-0.064907181579)*x[0]**o+(0.766055162408)*x[0]+(-0.524105307308)*x[1]**o+(0.361103034438)*x[1]
        s[3,2,1,1]=(0.690748511868)*x[0]**o+(0.412961119969)*x[0]+(-0.795479866575)*x[1]**o+(-0.450712265218)*x[1]
        s[3,2,2,0]=(0.34363262378)*x[0]**o+(-0.244910370036)*x[0]+(-0.634877248387)*x[1]**o+(-0.472063405174)*x[1]
        s[3,2,2,1]=(0.246638832327)*x[0]**o+(0.914501562312)*x[0]+(0.751830459279)*x[1]**o+(-0.497325969345)*x[1]
        s[3,3,0,0]=(0.541075396481)*x[0]**o+(-0.634968284805)*x[0]+(-0.231672798016)*x[1]**o+(-0.34422126411)*x[1]
        s[3,3,0,1]=(-0.935640589712)*x[0]**o+(-0.375887818373)*x[0]+(-0.243360729986)*x[1]**o+(-0.0544717388717)*x[1]
        s[3,3,1,0]=(0.154524952927)*x[0]**o+(-0.0863136783351)*x[0]+(-0.653676660981)*x[1]**o+(-0.7454863091)*x[1]
        s[3,3,1,1]=(0.277106623801)*x[0]**o+(-0.913188289955)*x[0]+(-0.79285009308)*x[1]**o+(-0.18601350351)*x[1]
        s[3,3,2,0]=(-0.3323524862)*x[0]**o+(0.94012804832)*x[0]+(0.397258718522)*x[1]**o+(-0.933536176616)*x[1]
        s[3,3,2,1]=(0.969750554915)*x[0]**o+(-0.851907090663)*x[0]+(0.950203728487)*x[1]**o+(-0.712715265376)*x[1]
        s[3,4,0,0]=(0.980063789653)*x[0]**o+(0.935723410963)*x[0]+(0.386913924778)*x[1]**o+(0.180557664607)*x[1]
        s[3,4,0,1]=(0.931878041992)*x[0]**o+(0.88853807113)*x[0]+(-0.187838817545)*x[1]**o+(-0.0251337491704)*x[1]
        s[3,4,1,0]=(-0.479847354936)*x[0]**o+(-0.030224862129)*x[0]+(-0.541789127999)*x[1]**o+(0.240762955814)*x[1]
        s[3,4,1,1]=(-0.917202541548)*x[0]**o+(0.217703883458)*x[0]+(0.248901567101)*x[1]**o+(-0.130053826789)*x[1]
        s[3,4,2,0]=(0.719712207452)*x[0]**o+(-0.606674954161)*x[0]+(0.508851160098)*x[1]**o+(-0.160706213281)*x[1]
        s[3,4,2,1]=(-0.5855419209)*x[0]**o+(-0.689958074797)*x[0]+(-0.782999784933)*x[1]**o+(0.800207229965)*x[1]
        ref[0,0,0,0]=(0.286414445959)*x_ref[0]**o+(-0.513602739404)*x_ref[0]+(0.415115217751)*x_ref[1]**o+(0.302847208656)*x_ref[1]
        ref[0,0,0,1]=(0.259932949392)*x_ref[0]**o+(0.35051585712)*x_ref[0]+(-0.199459717408)*x_ref[1]**o+(0.243907820471)*x_ref[1]
        ref[0,0,1,0]=(-0.118977243299)*x_ref[0]**o+(-0.42706283684)*x_ref[0]+(0.538165751605)*x_ref[1]**o+(0.393481400689)*x_ref[1]
        ref[0,0,1,1]=(-0.0397069035535)*x_ref[0]**o+(0.861872317511)*x_ref[0]+(-0.15731307612)*x_ref[1]**o+(0.693002169125)*x_ref[1]
        ref[0,0,2,0]=(0.815645799181)*x_ref[0]**o+(-0.0776314502273)*x_ref[0]+(-0.593034585971)*x_ref[1]**o+(-0.970680448911)*x_ref[1]
        ref[0,0,2,1]=(-0.952302578078)*x_ref[0]**o+(0.651928073409)*x_ref[0]+(-0.239714652084)*x_ref[1]**o+(-0.952274708749)*x_ref[1]
        ref[0,1,0,0]=(-0.316151000005)*x_ref[0]**o+(0.216061626502)*x_ref[0]+(-0.882718070647)*x_ref[1]**o+(-0.671547751668)*x_ref[1]
        ref[0,1,0,1]=(-0.399909471112)*x_ref[0]**o+(-0.318040874639)*x_ref[0]+(-0.907464404526)*x_ref[1]**o+(0.233304604462)*x_ref[1]
        ref[0,1,1,0]=(0.774423529257)*x_ref[0]**o+(0.364389715996)*x_ref[0]+(-0.842201054073)*x_ref[1]**o+(0.404586304258)*x_ref[1]
        ref[0,1,1,1]=(0.504496704492)*x_ref[0]**o+(0.622221083753)*x_ref[0]+(0.12478240068)*x_ref[1]**o+(0.353949438798)*x_ref[1]
        ref[0,1,2,0]=(-0.845828936398)*x_ref[0]**o+(-0.165296386039)*x_ref[0]+(0.486358102378)*x_ref[1]**o+(-0.761128414131)*x_ref[1]
        ref[0,1,2,1]=(-0.587533689601)*x_ref[0]**o+(0.889232467387)*x_ref[0]+(-0.17030097273)*x_ref[1]**o+(-0.939366655195)*x_ref[1]
        ref[0,2,0,0]=(-0.583544977678)*x_ref[0]**o+(0.49701340766)*x_ref[0]+(-0.59251363345)*x_ref[1]**o+(0.632063421618)*x_ref[1]
        ref[0,2,0,1]=(0.159662476533)*x_ref[0]**o+(-0.926024134426)*x_ref[0]+(-0.359267295402)*x_ref[1]**o+(0.575272568112)*x_ref[1]
        ref[0,2,1,0]=(-0.737677964249)*x_ref[0]**o+(0.828179798925)*x_ref[0]+(0.487366022336)*x_ref[1]**o+(-0.80790431335)*x_ref[1]
        ref[0,2,1,1]=(-0.633849216356)*x_ref[0]**o+(0.127289254283)*x_ref[0]+(0.282991845718)*x_ref[1]**o+(0.821227139666)*x_ref[1]
        ref[0,2,2,0]=(-0.821316223164)*x_ref[0]**o+(0.979879665311)*x_ref[0]+(-0.137787756007)*x_ref[1]**o+(-0.277486407169)*x_ref[1]
        ref[0,2,2,1]=(-0.622980962419)*x_ref[0]**o+(-0.696479459661)*x_ref[0]+(-0.410742216866)*x_ref[1]**o+(0.576030773935)*x_ref[1]
        ref[0,3,0,0]=(-0.207619411473)*x_ref[0]**o+(-0.469849947705)*x_ref[0]+(-0.671957494397)*x_ref[1]**o+(0.254338352433)*x_ref[1]
        ref[0,3,0,1]=(0.243280252346)*x_ref[0]**o+(0.133824860089)*x_ref[0]+(0.0696460542307)*x_ref[1]**o+(0.455743386064)*x_ref[1]
        ref[0,3,1,0]=(-0.906691692651)*x_ref[0]**o+(0.787810130121)*x_ref[0]+(-0.219233151952)*x_ref[1]**o+(0.743750472953)*x_ref[1]
        ref[0,3,1,1]=(-0.250516865682)*x_ref[0]**o+(-0.780254096438)*x_ref[0]+(-0.739910422511)*x_ref[1]**o+(-0.792068328341)*x_ref[1]
        ref[0,3,2,0]=(0.0232686870041)*x_ref[0]**o+(0.0106534323439)*x_ref[0]+(-0.0361419376311)*x_ref[1]**o+(0.566265323633)*x_ref[1]
        ref[0,3,2,1]=(-0.638354270703)*x_ref[0]**o+(-0.255988140752)*x_ref[0]+(-0.261309974545)*x_ref[1]**o+(-0.697773476841)*x_ref[1]
        ref[0,4,0,0]=(-0.381395765807)*x_ref[0]**o+(-0.365270978705)*x_ref[0]+(0.812150815593)*x_ref[1]**o+(0.719382395957)*x_ref[1]
        ref[0,4,0,1]=(0.69594272856)*x_ref[0]**o+(0.278887033989)*x_ref[0]+(0.754272027542)*x_ref[1]**o+(0.793061109177)*x_ref[1]
        ref[0,4,1,0]=(0.967293155949)*x_ref[0]**o+(-0.840326330307)*x_ref[0]+(0.299320013494)*x_ref[1]**o+(0.319888429035)*x_ref[1]
        ref[0,4,1,1]=(-0.0509734884321)*x_ref[0]**o+(-0.936136061318)*x_ref[0]+(-0.98628193626)*x_ref[1]**o+(0.661193529996)*x_ref[1]
        ref[0,4,2,0]=(0.562813543077)*x_ref[0]**o+(0.907184253714)*x_ref[0]+(-0.136123729129)*x_ref[1]**o+(-0.986466864564)*x_ref[1]
        ref[0,4,2,1]=(-0.434457154258)*x_ref[0]**o+(-0.574859203931)*x_ref[0]+(-0.383505033613)*x_ref[1]**o+(0.331906199764)*x_ref[1]
        ref[1,0,0,0]=(0.0611272547169)*x_ref[0]**o+(-0.136522073182)*x_ref[0]+(0.417326897823)*x_ref[1]**o+(-0.890085105585)*x_ref[1]
        ref[1,0,0,1]=(-0.869426624414)*x_ref[0]**o+(-0.186222609222)*x_ref[0]+(0.339956750726)*x_ref[1]**o+(-0.226819210334)*x_ref[1]
        ref[1,0,1,0]=(0.399952324098)*x_ref[0]**o+(0.798209020313)*x_ref[0]+(-0.521148352596)*x_ref[1]**o+(0.458003374007)*x_ref[1]
        ref[1,0,1,1]=(0.501460721237)*x_ref[0]**o+(0.231248731021)*x_ref[0]+(0.332539535549)*x_ref[1]**o+(0.185347952715)*x_ref[1]
        ref[1,0,2,0]=(-0.230801196498)*x_ref[0]**o+(0.823765804775)*x_ref[0]+(-0.258377195949)*x_ref[1]**o+(0.224645264709)*x_ref[1]
        ref[1,0,2,1]=(-0.241372395526)*x_ref[0]**o+(0.905482005475)*x_ref[0]+(0.0493139366957)*x_ref[1]**o+(0.541602080684)*x_ref[1]
        ref[1,1,0,0]=(0.449706009608)*x_ref[0]**o+(0.359879546784)*x_ref[0]+(-0.493158517575)*x_ref[1]**o+(0.191770207113)*x_ref[1]
        ref[1,1,0,1]=(0.957438236069)*x_ref[0]**o+(-0.59909754527)*x_ref[0]+(-0.614098873549)*x_ref[1]**o+(0.0834466945634)*x_ref[1]
        ref[1,1,1,0]=(0.146023068688)*x_ref[0]**o+(-0.185385831893)*x_ref[0]+(-0.805313629764)*x_ref[1]**o+(0.230766437006)*x_ref[1]
        ref[1,1,1,1]=(0.306071651757)*x_ref[0]**o+(0.921518467891)*x_ref[0]+(0.716057571126)*x_ref[1]**o+(-0.522119477993)*x_ref[1]
        ref[1,1,2,0]=(0.684162336448)*x_ref[0]**o+(-0.388731324307)*x_ref[0]+(0.600372268011)*x_ref[1]**o+(-0.771944030767)*x_ref[1]
        ref[1,1,2,1]=(0.286204844749)*x_ref[0]**o+(0.400092667734)*x_ref[0]+(0.506091236432)*x_ref[1]**o+(0.858615249035)*x_ref[1]
        ref[1,2,0,0]=(-0.786850947851)*x_ref[0]**o+(0.0317031686489)*x_ref[0]+(-0.0562719978878)*x_ref[1]**o+(-0.368745220216)*x_ref[1]
        ref[1,2,0,1]=(-0.062898974467)*x_ref[0]**o+(-0.397876556176)*x_ref[0]+(-0.117046688506)*x_ref[1]**o+(-0.548475481806)*x_ref[1]
        ref[1,2,1,0]=(-0.478254619878)*x_ref[0]**o+(0.866928591218)*x_ref[0]+(0.4475948677)*x_ref[1]**o+(0.393318603064)*x_ref[1]
        ref[1,2,1,1]=(-0.351975278856)*x_ref[0]**o+(-0.377161089123)*x_ref[0]+(0.492009299907)*x_ref[1]**o+(0.569876619926)*x_ref[1]
        ref[1,2,2,0]=(-0.556131924542)*x_ref[0]**o+(0.603608830074)*x_ref[0]+(0.409486637292)*x_ref[1]**o+(0.796280414552)*x_ref[1]
        ref[1,2,2,1]=(0.52135916937)*x_ref[0]**o+(0.760618210549)*x_ref[0]+(-0.145180244853)*x_ref[1]**o+(0.549313491191)*x_ref[1]
        ref[1,3,0,0]=(-0.577089151087)*x_ref[0]**o+(0.821891072229)*x_ref[0]+(0.332425052183)*x_ref[1]**o+(-0.151504483932)*x_ref[1]
        ref[1,3,0,1]=(-0.966703491954)*x_ref[0]**o+(0.13997287848)*x_ref[0]+(-0.682812558867)*x_ref[1]**o+(-0.465013684448)*x_ref[1]
        ref[1,3,1,0]=(0.908453154364)*x_ref[0]**o+(0.267997575637)*x_ref[0]+(0.587549759517)*x_ref[1]**o+(-0.487274402304)*x_ref[1]
        ref[1,3,1,1]=(-0.38747875298)*x_ref[0]**o+(-0.685908980542)*x_ref[0]+(-0.350387154367)*x_ref[1]**o+(0.621711654152)*x_ref[1]
        ref[1,3,2,0]=(-0.148266632644)*x_ref[0]**o+(-0.580283160892)*x_ref[0]+(-0.261751144414)*x_ref[1]**o+(0.502757383658)*x_ref[1]
        ref[1,3,2,1]=(0.610418736064)*x_ref[0]**o+(0.591832308717)*x_ref[0]+(0.869463161016)*x_ref[1]**o+(0.252614326127)*x_ref[1]
        ref[1,4,0,0]=(0.683334561136)*x_ref[0]**o+(0.484163667966)*x_ref[0]+(0.254191241985)*x_ref[1]**o+(0.039976829904)*x_ref[1]
        ref[1,4,0,1]=(-0.79220316614)*x_ref[0]**o+(0.749008242295)*x_ref[0]+(-0.539372369578)*x_ref[1]**o+(-0.900518079782)*x_ref[1]
        ref[1,4,1,0]=(0.576352091657)*x_ref[0]**o+(0.237846036342)*x_ref[0]+(0.000667246779082)*x_ref[1]**o+(-0.709742732387)*x_ref[1]
        ref[1,4,1,1]=(-0.676810333712)*x_ref[0]**o+(0.847889875816)*x_ref[0]+(-0.688138508533)*x_ref[1]**o+(0.133158683641)*x_ref[1]
        ref[1,4,2,0]=(0.99415007686)*x_ref[0]**o+(-0.141452245889)*x_ref[0]+(0.775573627086)*x_ref[1]**o+(0.452458721895)*x_ref[1]
        ref[1,4,2,1]=(0.994145376118)*x_ref[0]**o+(0.395589592556)*x_ref[0]+(-0.669488954253)*x_ref[1]**o+(0.067046815669)*x_ref[1]
        ref[2,0,0,0]=(-0.750281322814)*x_ref[0]**o+(-0.309320623684)*x_ref[0]+(0.626517958661)*x_ref[1]**o+(0.597498890162)*x_ref[1]
        ref[2,0,0,1]=(0.271076040325)*x_ref[0]**o+(0.667103080861)*x_ref[0]+(0.422066039225)*x_ref[1]**o+(0.363852331279)*x_ref[1]
        ref[2,0,1,0]=(-0.273421011656)*x_ref[0]**o+(-0.0619409465417)*x_ref[0]+(0.0679232143441)*x_ref[1]**o+(-0.720428826448)*x_ref[1]
        ref[2,0,1,1]=(-0.647493736809)*x_ref[0]**o+(0.927792745023)*x_ref[0]+(0.888263709553)*x_ref[1]**o+(-0.613668040672)*x_ref[1]
        ref[2,0,2,0]=(0.576488068023)*x_ref[0]**o+(0.389605065338)*x_ref[0]+(0.273385863145)*x_ref[1]**o+(0.528205453441)*x_ref[1]
        ref[2,0,2,1]=(-0.906661332702)*x_ref[0]**o+(0.0725964985998)*x_ref[0]+(0.184026573733)*x_ref[1]**o+(0.72102096328)*x_ref[1]
        ref[2,1,0,0]=(0.0561237068662)*x_ref[0]**o+(0.705088464117)*x_ref[0]+(-0.92649218582)*x_ref[1]**o+(-0.392584665998)*x_ref[1]
        ref[2,1,0,1]=(0.590410488233)*x_ref[0]**o+(-0.599541782377)*x_ref[0]+(0.739195238446)*x_ref[1]**o+(-0.388992225425)*x_ref[1]
        ref[2,1,1,0]=(0.382347893509)*x_ref[0]**o+(0.65529075645)*x_ref[0]+(0.6281938054)*x_ref[1]**o+(0.122434736283)*x_ref[1]
        ref[2,1,1,1]=(-0.286086668538)*x_ref[0]**o+(-0.104121795484)*x_ref[0]+(-0.282055217155)*x_ref[1]**o+(0.173105945475)*x_ref[1]
        ref[2,1,2,0]=(-0.287280683011)*x_ref[0]**o+(-0.963651344223)*x_ref[0]+(-0.940112522168)*x_ref[1]**o+(-0.788180434371)*x_ref[1]
        ref[2,1,2,1]=(0.61738130828)*x_ref[0]**o+(-0.427708968146)*x_ref[0]+(0.319738892701)*x_ref[1]**o+(0.419063299626)*x_ref[1]
        ref[2,2,0,0]=(-0.0499926543896)*x_ref[0]**o+(0.01377519232)*x_ref[0]+(-0.325826729461)*x_ref[1]**o+(-0.392085884375)*x_ref[1]
        ref[2,2,0,1]=(-0.0581765276804)*x_ref[0]**o+(-0.208263386025)*x_ref[0]+(0.75198359812)*x_ref[1]**o+(0.97252999759)*x_ref[1]
        ref[2,2,1,0]=(-0.726087111179)*x_ref[0]**o+(-0.0573158936919)*x_ref[0]+(-0.742825725839)*x_ref[1]**o+(0.415476090214)*x_ref[1]
        ref[2,2,1,1]=(0.917838486533)*x_ref[0]**o+(-0.091211527759)*x_ref[0]+(0.507645127218)*x_ref[1]**o+(-0.742224440501)*x_ref[1]
        ref[2,2,2,0]=(-0.610881095534)*x_ref[0]**o+(-0.864358502475)*x_ref[0]+(0.0808508216725)*x_ref[1]**o+(0.199006479357)*x_ref[1]
        ref[2,2,2,1]=(-0.946103419369)*x_ref[0]**o+(0.491509709702)*x_ref[0]+(0.612215069813)*x_ref[1]**o+(0.418509110447)*x_ref[1]
        ref[2,3,0,0]=(0.0741360472802)*x_ref[0]**o+(-0.994432396766)*x_ref[0]+(-0.332924771563)*x_ref[1]**o+(-0.753108051656)*x_ref[1]
        ref[2,3,0,1]=(-0.381576097006)*x_ref[0]**o+(-0.377802667887)*x_ref[0]+(-0.0303870681199)*x_ref[1]**o+(-0.345561266945)*x_ref[1]
        ref[2,3,1,0]=(0.642729104556)*x_ref[0]**o+(0.570036794833)*x_ref[0]+(-0.868604403383)*x_ref[1]**o+(-0.671162161412)*x_ref[1]
        ref[2,3,1,1]=(0.335963731776)*x_ref[0]**o+(-0.504092976502)*x_ref[0]+(-0.980540079107)*x_ref[1]**o+(0.354777629497)*x_ref[1]
        ref[2,3,2,0]=(-0.764056326006)*x_ref[0]**o+(-0.416057085413)*x_ref[0]+(-0.690778483271)*x_ref[1]**o+(-0.760474373055)*x_ref[1]
        ref[2,3,2,1]=(0.901089622394)*x_ref[0]**o+(0.13387449325)*x_ref[0]+(0.0138079678955)*x_ref[1]**o+(0.839892772097)*x_ref[1]
        ref[2,4,0,0]=(-0.650061300028)*x_ref[0]**o+(-0.349615966399)*x_ref[0]+(0.790768704094)*x_ref[1]**o+(-0.42814753068)*x_ref[1]
        ref[2,4,0,1]=(-0.0418088165021)*x_ref[0]**o+(0.150269895143)*x_ref[0]+(-0.529846432011)*x_ref[1]**o+(-0.224508824625)*x_ref[1]
        ref[2,4,1,0]=(-0.872715524736)*x_ref[0]**o+(-0.968213567754)*x_ref[0]+(-0.485442904703)*x_ref[1]**o+(-0.201353147903)*x_ref[1]
        ref[2,4,1,1]=(0.807375853776)*x_ref[0]**o+(-0.555765909033)*x_ref[0]+(0.509490325151)*x_ref[1]**o+(-0.0402998342759)*x_ref[1]
        ref[2,4,2,0]=(0.976103420432)*x_ref[0]**o+(-0.31420590756)*x_ref[0]+(0.965906601649)*x_ref[1]**o+(0.996838317987)*x_ref[1]
        ref[2,4,2,1]=(0.927750446685)*x_ref[0]**o+(-0.0669100154417)*x_ref[0]+(0.329400978068)*x_ref[1]**o+(0.673620795081)*x_ref[1]
        ref[3,0,0,0]=(0.0339658367621)*x_ref[0]**o+(-0.69057366167)*x_ref[0]+(-0.84841361493)*x_ref[1]**o+(-0.793870889269)*x_ref[1]
        ref[3,0,0,1]=(0.580299117541)*x_ref[0]**o+(0.711568306729)*x_ref[0]+(0.68652281169)*x_ref[1]**o+(-0.893997742165)*x_ref[1]
        ref[3,0,1,0]=(0.472119483925)*x_ref[0]**o+(0.811060953315)*x_ref[0]+(0.131977099466)*x_ref[1]**o+(0.252305475595)*x_ref[1]
        ref[3,0,1,1]=(0.825801074332)*x_ref[0]**o+(-0.494721582584)*x_ref[0]+(0.926019323719)*x_ref[1]**o+(-0.398271744425)*x_ref[1]
        ref[3,0,2,0]=(0.456913500128)*x_ref[0]**o+(-0.17032765188)*x_ref[0]+(0.758934382849)*x_ref[1]**o+(-0.260113957998)*x_ref[1]
        ref[3,0,2,1]=(-0.903049904081)*x_ref[0]**o+(0.0173063807709)*x_ref[0]+(-0.374982617974)*x_ref[1]**o+(0.334268331811)*x_ref[1]
        ref[3,1,0,0]=(-0.22949882827)*x_ref[0]**o+(-0.373561413774)*x_ref[0]+(0.790631265088)*x_ref[1]**o+(-0.75812217101)*x_ref[1]
        ref[3,1,0,1]=(0.564879113215)*x_ref[0]**o+(0.33779824151)*x_ref[0]+(0.0367512619169)*x_ref[1]**o+(0.441450704065)*x_ref[1]
        ref[3,1,1,0]=(-0.428927604938)*x_ref[0]**o+(-0.523613773862)*x_ref[0]+(0.29478120955)*x_ref[1]**o+(-0.863628205314)*x_ref[1]
        ref[3,1,1,1]=(0.459167152825)*x_ref[0]**o+(-0.641899803541)*x_ref[0]+(-0.109951392233)*x_ref[1]**o+(0.399265290858)*x_ref[1]
        ref[3,1,2,0]=(0.682558303558)*x_ref[0]**o+(-0.137784370501)*x_ref[0]+(-0.393326255016)*x_ref[1]**o+(-0.678503687693)*x_ref[1]
        ref[3,1,2,1]=(-0.0809452179949)*x_ref[0]**o+(0.235685309616)*x_ref[0]+(0.974274730596)*x_ref[1]**o+(-0.894536588546)*x_ref[1]
        ref[3,2,0,0]=(-0.0490472374331)*x_ref[0]**o+(-0.473644556364)*x_ref[0]+(-0.382189831802)*x_ref[1]**o+(-0.22504837111)*x_ref[1]
        ref[3,2,0,1]=(0.997893391083)*x_ref[0]**o+(0.749295222878)*x_ref[0]+(-0.799214431894)*x_ref[1]**o+(0.57167392758)*x_ref[1]
        ref[3,2,1,0]=(-0.064907181579)*x_ref[0]**o+(0.766055162408)*x_ref[0]+(-0.524105307308)*x_ref[1]**o+(0.361103034438)*x_ref[1]
        ref[3,2,1,1]=(0.690748511868)*x_ref[0]**o+(0.412961119969)*x_ref[0]+(-0.795479866575)*x_ref[1]**o+(-0.450712265218)*x_ref[1]
        ref[3,2,2,0]=(0.34363262378)*x_ref[0]**o+(-0.244910370036)*x_ref[0]+(-0.634877248387)*x_ref[1]**o+(-0.472063405174)*x_ref[1]
        ref[3,2,2,1]=(0.246638832327)*x_ref[0]**o+(0.914501562312)*x_ref[0]+(0.751830459279)*x_ref[1]**o+(-0.497325969345)*x_ref[1]
        ref[3,3,0,0]=(0.541075396481)*x_ref[0]**o+(-0.634968284805)*x_ref[0]+(-0.231672798016)*x_ref[1]**o+(-0.34422126411)*x_ref[1]
        ref[3,3,0,1]=(-0.935640589712)*x_ref[0]**o+(-0.375887818373)*x_ref[0]+(-0.243360729986)*x_ref[1]**o+(-0.0544717388717)*x_ref[1]
        ref[3,3,1,0]=(0.154524952927)*x_ref[0]**o+(-0.0863136783351)*x_ref[0]+(-0.653676660981)*x_ref[1]**o+(-0.7454863091)*x_ref[1]
        ref[3,3,1,1]=(0.277106623801)*x_ref[0]**o+(-0.913188289955)*x_ref[0]+(-0.79285009308)*x_ref[1]**o+(-0.18601350351)*x_ref[1]
        ref[3,3,2,0]=(-0.3323524862)*x_ref[0]**o+(0.94012804832)*x_ref[0]+(0.397258718522)*x_ref[1]**o+(-0.933536176616)*x_ref[1]
        ref[3,3,2,1]=(0.969750554915)*x_ref[0]**o+(-0.851907090663)*x_ref[0]+(0.950203728487)*x_ref[1]**o+(-0.712715265376)*x_ref[1]
        ref[3,4,0,0]=(0.980063789653)*x_ref[0]**o+(0.935723410963)*x_ref[0]+(0.386913924778)*x_ref[1]**o+(0.180557664607)*x_ref[1]
        ref[3,4,0,1]=(0.931878041992)*x_ref[0]**o+(0.88853807113)*x_ref[0]+(-0.187838817545)*x_ref[1]**o+(-0.0251337491704)*x_ref[1]
        ref[3,4,1,0]=(-0.479847354936)*x_ref[0]**o+(-0.030224862129)*x_ref[0]+(-0.541789127999)*x_ref[1]**o+(0.240762955814)*x_ref[1]
        ref[3,4,1,1]=(-0.917202541548)*x_ref[0]**o+(0.217703883458)*x_ref[0]+(0.248901567101)*x_ref[1]**o+(-0.130053826789)*x_ref[1]
        ref[3,4,2,0]=(0.719712207452)*x_ref[0]**o+(-0.606674954161)*x_ref[0]+(0.508851160098)*x_ref[1]**o+(-0.160706213281)*x_ref[1]
        ref[3,4,2,1]=(-0.5855419209)*x_ref[0]**o+(-0.689958074797)*x_ref[0]+(-0.782999784933)*x_ref[1]**o+(0.800207229965)*x_ref[1]
      else:
        s[0,0,0,0]=(-0.0939552237867)*x[0]**o+(-0.233782296435)*x[0]+(0.272117493235)*x[1]**o+(0.938376635468)*x[1]+(0.661832811129)*x[2]**o+(0.282399882837)*x[2]
        s[0,0,0,1]=(0.236203034262)*x[0]**o+(-0.583700442678)*x[0]+(-0.870895994748)*x[1]**o+(0.192686915202)*x[1]+(-0.901022579657)*x[2]**o+(-0.329112379375)*x[2]
        s[0,0,1,0]=(-0.583068685335)*x[0]**o+(0.981024224765)*x[0]+(0.869746509265)*x[1]**o+(0.713473797745)*x[1]+(0.647842443595)*x[2]**o+(0.128105689437)*x[2]
        s[0,0,1,1]=(0.231954364537)*x[0]**o+(0.301876895567)*x[0]+(0.902392392762)*x[1]**o+(0.77394994345)*x[1]+(-0.617945530904)*x[2]**o+(0.596296526597)*x[2]
        s[0,0,2,0]=(-0.848859481595)*x[0]**o+(0.489664239901)*x[0]+(0.101756068383)*x[1]**o+(0.372022728544)*x[1]+(-0.0480662015594)*x[2]**o+(-0.140663588571)*x[2]
        s[0,0,2,1]=(-0.151813501072)*x[0]**o+(-0.0619139652224)*x[0]+(-0.2404019023)*x[1]**o+(0.186875111425)*x[1]+(0.870121943947)*x[2]**o+(-0.401842672093)*x[2]
        s[0,1,0,0]=(-0.862371891518)*x[0]**o+(-0.141519691948)*x[0]+(0.0218465584484)*x[1]**o+(0.474483154145)*x[1]+(0.00642588343357)*x[2]**o+(0.0937465696159)*x[2]
        s[0,1,0,1]=(0.957271234184)*x[0]**o+(0.181371031824)*x[0]+(0.325782983064)*x[1]**o+(-0.758554608166)*x[1]+(0.888493514738)*x[2]**o+(0.360604795131)*x[2]
        s[0,1,1,0]=(0.8810614634)*x[0]**o+(-0.891947928245)*x[0]+(-0.81840638286)*x[1]**o+(-0.338084999054)*x[1]+(-0.408571326721)*x[2]**o+(-0.675124681083)*x[2]
        s[0,1,1,1]=(-0.00418963259364)*x[0]**o+(0.0872512414907)*x[0]+(0.307397770057)*x[1]**o+(-0.784741288514)*x[1]+(-0.288826977888)*x[2]**o+(-0.079246885912)*x[2]
        s[0,1,2,0]=(-0.933072733018)*x[0]**o+(0.141983186826)*x[0]+(-0.512297440539)*x[1]**o+(-0.232926253673)*x[1]+(0.884581006836)*x[2]**o+(-0.196597583936)*x[2]
        s[0,1,2,1]=(-0.264256230447)*x[0]**o+(0.0633721777798)*x[0]+(0.798360353199)*x[1]**o+(-0.323014270481)*x[1]+(0.99252859322)*x[2]**o+(0.933710902797)*x[2]
        s[0,2,0,0]=(0.402384727981)*x[0]**o+(-0.600697619335)*x[0]+(0.0672711555798)*x[1]**o+(-0.905342068352)*x[1]+(0.576003188043)*x[2]**o+(0.136818654256)*x[2]
        s[0,2,0,1]=(0.335215866093)*x[0]**o+(-0.128245834791)*x[0]+(0.214262493314)*x[1]**o+(-0.278381637725)*x[1]+(0.506994464335)*x[2]**o+(-0.426267615741)*x[2]
        s[0,2,1,0]=(0.463302776547)*x[0]**o+(0.836336860793)*x[0]+(0.978118223846)*x[1]**o+(-0.299895814699)*x[1]+(0.816774597805)*x[2]**o+(0.561662450214)*x[2]
        s[0,2,1,1]=(-0.355099999892)*x[0]**o+(-0.867893817303)*x[0]+(0.318137264146)*x[1]**o+(0.996417293091)*x[1]+(0.190720206337)*x[2]**o+(-0.956547404476)*x[2]
        s[0,2,2,0]=(0.636419754289)*x[0]**o+(-0.490121483483)*x[0]+(0.0724785970005)*x[1]**o+(0.654051448577)*x[1]+(-0.682352816989)*x[2]**o+(-0.969295042468)*x[2]
        s[0,2,2,1]=(0.523182385281)*x[0]**o+(0.882794657228)*x[0]+(-0.258639599201)*x[1]**o+(0.870016686554)*x[1]+(0.810959873296)*x[2]**o+(-0.120051000895)*x[2]
        s[0,3,0,0]=(-0.142691620177)*x[0]**o+(-0.26145359895)*x[0]+(0.979353443033)*x[1]**o+(0.812390675638)*x[1]+(0.659244834954)*x[2]**o+(0.951080038113)*x[2]
        s[0,3,0,1]=(-0.969621517275)*x[0]**o+(-0.0841833729288)*x[0]+(0.183088818824)*x[1]**o+(-0.0637411180744)*x[1]+(0.70498987804)*x[2]**o+(0.464729030647)*x[2]
        s[0,3,1,0]=(-0.908811399013)*x[0]**o+(0.469705146462)*x[0]+(0.623185114304)*x[1]**o+(0.156348468944)*x[1]+(-0.0352444815103)*x[2]**o+(-0.0924055149797)*x[2]
        s[0,3,1,1]=(-0.804539460051)*x[0]**o+(-0.64073007907)*x[0]+(0.871919457046)*x[1]**o+(-0.446001955346)*x[1]+(-0.758572789631)*x[2]**o+(-0.943023339532)*x[2]
        s[0,3,2,0]=(0.76012858373)*x[0]**o+(-0.413461345613)*x[0]+(-0.636494843218)*x[1]**o+(0.0576409596138)*x[1]+(0.32356830303)*x[2]**o+(0.66336356169)*x[2]
        s[0,3,2,1]=(-0.682969408206)*x[0]**o+(-0.00378599179446)*x[0]+(0.139592931649)*x[1]**o+(0.230923755162)*x[1]+(-0.359747563013)*x[2]**o+(0.490767562994)*x[2]
        s[0,4,0,0]=(0.40294705204)*x[0]**o+(-0.725991341301)*x[0]+(-0.123299724766)*x[1]**o+(0.783062653249)*x[1]+(0.284875756512)*x[2]**o+(-0.479856616542)*x[2]
        s[0,4,0,1]=(0.671083818646)*x[0]**o+(0.475388684118)*x[0]+(-0.0415590406628)*x[1]**o+(0.523350160591)*x[1]+(-0.869125066985)*x[2]**o+(0.539765124009)*x[2]
        s[0,4,1,0]=(0.517796715592)*x[0]**o+(0.273319492606)*x[0]+(0.131964130241)*x[1]**o+(0.522704060425)*x[1]+(0.945735742195)*x[2]**o+(-0.927060768569)*x[2]
        s[0,4,1,1]=(-0.0877749093432)*x[0]**o+(-0.228856801496)*x[0]+(-0.918861499662)*x[1]**o+(0.783335087368)*x[1]+(0.731374866941)*x[2]**o+(-0.0169316582485)*x[2]
        s[0,4,2,0]=(-0.521914417723)*x[0]**o+(0.100272381296)*x[0]+(-0.37855972365)*x[1]**o+(-0.748174737428)*x[1]+(0.08343326291)*x[2]**o+(-0.284290813001)*x[2]
        s[0,4,2,1]=(-0.622347209837)*x[0]**o+(-0.737347404815)*x[0]+(0.973210808148)*x[1]**o+(0.34940344181)*x[1]+(0.917063169506)*x[2]**o+(0.40443560458)*x[2]
        s[1,0,0,0]=(0.906350070311)*x[0]**o+(-0.514349936373)*x[0]+(-0.0377819340766)*x[1]**o+(-0.868663863385)*x[1]+(-0.910618099331)*x[2]**o+(0.161682748869)*x[2]
        s[1,0,0,1]=(-0.525696726097)*x[0]**o+(-0.564534313649)*x[0]+(0.294413275375)*x[1]**o+(-0.032889914662)*x[1]+(-0.655188068906)*x[2]**o+(-0.544604687619)*x[2]
        s[1,0,1,0]=(0.982917203627)*x[0]**o+(0.23003920025)*x[0]+(0.53970317464)*x[1]**o+(0.488996625882)*x[1]+(0.155567762567)*x[2]**o+(-0.0769333810796)*x[2]
        s[1,0,1,1]=(-0.695407739098)*x[0]**o+(-0.528856183813)*x[0]+(-0.765981763723)*x[1]**o+(-0.747233410366)*x[1]+(-0.217961552142)*x[2]**o+(0.363922845031)*x[2]
        s[1,0,2,0]=(-0.508333136514)*x[0]**o+(-0.0531371629837)*x[0]+(0.147835213524)*x[1]**o+(0.811513270701)*x[1]+(-0.51987266682)*x[2]**o+(-0.840524592697)*x[2]
        s[1,0,2,1]=(0.522917930226)*x[0]**o+(-0.387982699235)*x[0]+(-0.716946321359)*x[1]**o+(0.440470433449)*x[1]+(0.841627656575)*x[2]**o+(0.635104516674)*x[2]
        s[1,1,0,0]=(0.484898204665)*x[0]**o+(-0.753933429744)*x[0]+(-0.275664031761)*x[1]**o+(-0.566814900411)*x[1]+(0.0612476985048)*x[2]**o+(0.437090771267)*x[2]
        s[1,1,0,1]=(-0.182638197905)*x[0]**o+(-0.778469897629)*x[0]+(-0.626963871394)*x[1]**o+(-0.400611638351)*x[1]+(0.895655277012)*x[2]**o+(0.0326121043684)*x[2]
        s[1,1,1,0]=(-0.916569622941)*x[0]**o+(-0.658275271075)*x[0]+(0.250794036795)*x[1]**o+(-0.233593794029)*x[1]+(-0.96809736282)*x[2]**o+(0.656072783817)*x[2]
        s[1,1,1,1]=(0.0171169428141)*x[0]**o+(0.860556263522)*x[0]+(0.345606647004)*x[1]**o+(0.564186697526)*x[1]+(0.588489057339)*x[2]**o+(-0.87110086452)*x[2]
        s[1,1,2,0]=(0.275896407623)*x[0]**o+(0.0122423763916)*x[0]+(0.107452743797)*x[1]**o+(-0.367492719113)*x[1]+(0.884317274048)*x[2]**o+(0.0908799334316)*x[2]
        s[1,1,2,1]=(0.642219111519)*x[0]**o+(-0.183742201216)*x[0]+(-0.327197867043)*x[1]**o+(0.322321062553)*x[1]+(-0.183074011204)*x[2]**o+(0.720021602264)*x[2]
        s[1,2,0,0]=(0.532332942294)*x[0]**o+(0.950878496192)*x[0]+(0.819174829388)*x[1]**o+(0.43412262514)*x[1]+(-0.166925240319)*x[2]**o+(-0.698173204203)*x[2]
        s[1,2,0,1]=(0.259909649155)*x[0]**o+(-0.264967838281)*x[0]+(-0.774874307838)*x[1]**o+(-0.275886773315)*x[1]+(0.801233942875)*x[2]**o+(0.27496052417)*x[2]
        s[1,2,1,0]=(-0.746564397549)*x[0]**o+(-0.690480530678)*x[0]+(0.114986469362)*x[1]**o+(-0.751415850669)*x[1]+(0.119951306293)*x[2]**o+(-0.665012834766)*x[2]
        s[1,2,1,1]=(-0.851672541025)*x[0]**o+(0.235181175896)*x[0]+(-0.62908061934)*x[1]**o+(-0.438672467151)*x[1]+(-0.761945783738)*x[2]**o+(0.580730895385)*x[2]
        s[1,2,2,0]=(0.676776403141)*x[0]**o+(-0.707505868325)*x[0]+(0.724421249054)*x[1]**o+(0.120766197116)*x[1]+(-0.760834341632)*x[2]**o+(-0.689464446912)*x[2]
        s[1,2,2,1]=(0.147028945539)*x[0]**o+(0.199909936204)*x[0]+(0.903168165102)*x[1]**o+(-0.46293792368)*x[1]+(-0.282517651615)*x[2]**o+(0.740619136229)*x[2]
        s[1,3,0,0]=(-0.87747323136)*x[0]**o+(-0.361656418039)*x[0]+(-0.14856291522)*x[1]**o+(0.474044532693)*x[1]+(0.138460936937)*x[2]**o+(-0.145182012384)*x[2]
        s[1,3,0,1]=(0.748334497747)*x[0]**o+(0.892498113089)*x[0]+(0.239113720175)*x[1]**o+(-0.762524411251)*x[1]+(-0.488407873495)*x[2]**o+(0.158979901061)*x[2]
        s[1,3,1,0]=(0.465350922376)*x[0]**o+(-0.654156421791)*x[0]+(0.878703330328)*x[1]**o+(0.537076882157)*x[1]+(0.389722314724)*x[2]**o+(-0.120902843496)*x[2]
        s[1,3,1,1]=(-0.153324789456)*x[0]**o+(-0.842917613229)*x[0]+(0.984971864624)*x[1]**o+(0.349269583591)*x[1]+(0.484775635349)*x[2]**o+(0.0123674100644)*x[2]
        s[1,3,2,0]=(-0.175935804533)*x[0]**o+(0.0654493735159)*x[0]+(0.726667315683)*x[1]**o+(0.979882137896)*x[1]+(0.543445934589)*x[2]**o+(-0.149142988507)*x[2]
        s[1,3,2,1]=(0.235188036633)*x[0]**o+(-0.383249536636)*x[0]+(-0.445575566142)*x[1]**o+(-0.819686823355)*x[1]+(0.170138244522)*x[2]**o+(-0.80311924596)*x[2]
        s[1,4,0,0]=(0.471387483772)*x[0]**o+(-0.591448975176)*x[0]+(-0.833833649412)*x[1]**o+(0.770775874052)*x[1]+(0.537769198907)*x[2]**o+(-0.646003815898)*x[2]
        s[1,4,0,1]=(-0.797500020228)*x[0]**o+(-0.136543576325)*x[0]+(0.11979987113)*x[1]**o+(-0.495999866876)*x[1]+(-0.611958471401)*x[2]**o+(-0.588202607164)*x[2]
        s[1,4,1,0]=(-0.550038387555)*x[0]**o+(0.0844321262213)*x[0]+(0.131005384757)*x[1]**o+(-0.515674980981)*x[1]+(-0.637043856082)*x[2]**o+(0.333864121142)*x[2]
        s[1,4,1,1]=(0.71740532023)*x[0]**o+(-0.868269127293)*x[0]+(0.323641764571)*x[1]**o+(0.820948811255)*x[1]+(0.799638086908)*x[2]**o+(-0.558360429849)*x[2]
        s[1,4,2,0]=(-0.366976697335)*x[0]**o+(-0.121083094264)*x[0]+(0.650559587113)*x[1]**o+(-0.400220219011)*x[1]+(-0.0785928535861)*x[2]**o+(-0.50420364324)*x[2]
        s[1,4,2,1]=(0.872011301914)*x[0]**o+(0.0300770125292)*x[0]+(-0.965040697782)*x[1]**o+(-0.373825300018)*x[1]+(-0.988463549572)*x[2]**o+(0.906686888358)*x[2]
        s[2,0,0,0]=(-0.644460598597)*x[0]**o+(0.69918562566)*x[0]+(0.0995801100352)*x[1]**o+(0.257114765725)*x[1]+(-0.172788858436)*x[2]**o+(-0.625310450189)*x[2]
        s[2,0,0,1]=(0.331738659918)*x[0]**o+(0.34134401217)*x[0]+(0.667229795214)*x[1]**o+(0.248779696936)*x[1]+(0.782703155201)*x[2]**o+(-0.0117889862376)*x[2]
        s[2,0,1,0]=(0.456825869656)*x[0]**o+(0.618685513764)*x[0]+(0.00330761288087)*x[1]**o+(-0.839303906537)*x[1]+(0.16840639247)*x[2]**o+(-0.177323734594)*x[2]
        s[2,0,1,1]=(0.543967300033)*x[0]**o+(0.537337400025)*x[0]+(-0.232374591177)*x[1]**o+(-0.163640094925)*x[1]+(0.146221483058)*x[2]**o+(0.833185534106)*x[2]
        s[2,0,2,0]=(-0.283419495517)*x[0]**o+(-0.0641847006529)*x[0]+(0.933228590846)*x[1]**o+(0.575529900563)*x[1]+(-0.634849100928)*x[2]**o+(-0.898678364957)*x[2]
        s[2,0,2,1]=(0.248202597053)*x[0]**o+(-0.395428705005)*x[0]+(-0.921778142592)*x[1]**o+(0.752699910606)*x[1]+(-0.208939925012)*x[2]**o+(0.252031969391)*x[2]
        s[2,1,0,0]=(0.543045279112)*x[0]**o+(-0.200796280468)*x[0]+(-0.850234504692)*x[1]**o+(-0.608668852127)*x[1]+(-0.773553907964)*x[2]**o+(-0.727318461167)*x[2]
        s[2,1,0,1]=(0.670138313505)*x[0]**o+(0.763313503204)*x[0]+(-0.712024856615)*x[1]**o+(-0.0810633888547)*x[1]+(-0.543339707954)*x[2]**o+(-0.0944422473469)*x[2]
        s[2,1,1,0]=(0.66812399426)*x[0]**o+(-0.00267070537766)*x[0]+(0.704373662345)*x[1]**o+(-0.778778632343)*x[1]+(-0.195017717355)*x[2]**o+(-0.718710830736)*x[2]
        s[2,1,1,1]=(-0.905514219789)*x[0]**o+(-0.899693862646)*x[0]+(-0.791063116492)*x[1]**o+(-0.470046803205)*x[1]+(0.954924769408)*x[2]**o+(0.36799445545)*x[2]
        s[2,1,2,0]=(0.363438950523)*x[0]**o+(-0.788582160746)*x[0]+(0.316623350464)*x[1]**o+(-0.621859877627)*x[1]+(-0.899933912274)*x[2]**o+(-0.663582380926)*x[2]
        s[2,1,2,1]=(0.70767884819)*x[0]**o+(-0.337197298469)*x[0]+(0.329833087038)*x[1]**o+(-0.767476122169)*x[1]+(0.813567450674)*x[2]**o+(-0.230739080738)*x[2]
        s[2,2,0,0]=(0.425629461437)*x[0]**o+(0.736062549141)*x[0]+(0.0766779443487)*x[1]**o+(-0.645685464512)*x[1]+(-0.563156862696)*x[2]**o+(-0.200669234455)*x[2]
        s[2,2,0,1]=(-0.436511105169)*x[0]**o+(-0.784209106822)*x[0]+(0.476448752447)*x[1]**o+(-0.976253075887)*x[1]+(0.744207251461)*x[2]**o+(-0.480585834629)*x[2]
        s[2,2,1,0]=(-0.160519965814)*x[0]**o+(-0.740586061373)*x[0]+(-0.291540051837)*x[1]**o+(0.0186903802094)*x[1]+(-0.456120795038)*x[2]**o+(0.930603860664)*x[2]
        s[2,2,1,1]=(0.835200574398)*x[0]**o+(-0.111618184052)*x[0]+(0.394793418949)*x[1]**o+(0.66759987425)*x[1]+(0.902913963474)*x[2]**o+(-0.840871310598)*x[2]
        s[2,2,2,0]=(-0.745841363439)*x[0]**o+(0.859714040441)*x[0]+(-0.133922374335)*x[1]**o+(-0.539065635521)*x[1]+(-0.0496309374348)*x[2]**o+(-0.3150513125)*x[2]
        s[2,2,2,1]=(-0.121952493914)*x[0]**o+(-0.145565953724)*x[0]+(-0.46956691252)*x[1]**o+(0.594587113542)*x[1]+(0.801503774207)*x[2]**o+(-0.559107902481)*x[2]
        s[2,3,0,0]=(0.107705603601)*x[0]**o+(0.83167538493)*x[0]+(0.719052082882)*x[1]**o+(-0.0418541572577)*x[1]+(-0.760679778738)*x[2]**o+(-0.269573603299)*x[2]
        s[2,3,0,1]=(-0.610961305034)*x[0]**o+(-0.48822367684)*x[0]+(-0.533146614698)*x[1]**o+(-0.0634033507577)*x[1]+(-0.525819529758)*x[2]**o+(0.70089075883)*x[2]
        s[2,3,1,0]=(0.503925514982)*x[0]**o+(0.65957769246)*x[0]+(-0.624626366929)*x[1]**o+(-0.374542181845)*x[1]+(-0.363387031742)*x[2]**o+(-0.00563190329728)*x[2]
        s[2,3,1,1]=(0.337750136392)*x[0]**o+(-0.364300637249)*x[0]+(0.0892494275431)*x[1]**o+(0.958228736151)*x[1]+(0.203359176466)*x[2]**o+(-0.454161162092)*x[2]
        s[2,3,2,0]=(0.171894913416)*x[0]**o+(0.930342123436)*x[0]+(0.888973400724)*x[1]**o+(0.732732070997)*x[1]+(-0.365536399448)*x[2]**o+(0.719027934816)*x[2]
        s[2,3,2,1]=(0.0718345473684)*x[0]**o+(-0.351767950331)*x[0]+(-0.427880627822)*x[1]**o+(0.906554327062)*x[1]+(0.209905901579)*x[2]**o+(0.343983972848)*x[2]
        s[2,4,0,0]=(0.20082437951)*x[0]**o+(0.411679233479)*x[0]+(-0.146683847376)*x[1]**o+(0.619164773872)*x[1]+(0.283942602042)*x[2]**o+(0.328844649922)*x[2]
        s[2,4,0,1]=(0.512237008877)*x[0]**o+(-0.0799414456365)*x[0]+(-0.444485238118)*x[1]**o+(-0.617601180088)*x[1]+(-0.522016560329)*x[2]**o+(0.764966540338)*x[2]
        s[2,4,1,0]=(-0.906295486351)*x[0]**o+(0.213043439406)*x[0]+(-0.841541923905)*x[1]**o+(0.613189806332)*x[1]+(0.98016601466)*x[2]**o+(0.373100080758)*x[2]
        s[2,4,1,1]=(0.421256899132)*x[0]**o+(-0.148027161979)*x[0]+(-0.436551670748)*x[1]**o+(0.15389527845)*x[1]+(-0.202772011213)*x[2]**o+(0.526456981868)*x[2]
        s[2,4,2,0]=(0.326695825605)*x[0]**o+(-0.658196467156)*x[0]+(-0.0229897453445)*x[1]**o+(0.640660125725)*x[1]+(0.604325801088)*x[2]**o+(0.415440861181)*x[2]
        s[2,4,2,1]=(-0.353746711731)*x[0]**o+(0.707902797414)*x[0]+(-0.117816706586)*x[1]**o+(0.767979696436)*x[1]+(0.0361750168523)*x[2]**o+(-0.904298176296)*x[2]
        s[3,0,0,0]=(0.274392483594)*x[0]**o+(-0.142144450689)*x[0]+(-0.308306223778)*x[1]**o+(0.674122485073)*x[1]+(0.259741082605)*x[2]**o+(-0.249672127213)*x[2]
        s[3,0,0,1]=(-0.299212636089)*x[0]**o+(0.670140835575)*x[0]+(0.0513019322721)*x[1]**o+(0.460049892726)*x[1]+(0.167967285754)*x[2]**o+(-0.463520967247)*x[2]
        s[3,0,1,0]=(0.850761515544)*x[0]**o+(0.945654906297)*x[0]+(-0.841039831472)*x[1]**o+(-0.051572515458)*x[1]+(0.750689246331)*x[2]**o+(0.0651065819687)*x[2]
        s[3,0,1,1]=(-0.387082102682)*x[0]**o+(-0.89541016885)*x[0]+(-0.903675915508)*x[1]**o+(-0.268304570988)*x[1]+(-0.259828889903)*x[2]**o+(-0.983650155837)*x[2]
        s[3,0,2,0]=(0.497841641089)*x[0]**o+(0.80522306204)*x[0]+(0.532637885608)*x[1]**o+(-0.345627398376)*x[1]+(-0.768962683025)*x[2]**o+(-0.608176054775)*x[2]
        s[3,0,2,1]=(0.00437007549865)*x[0]**o+(0.847511281463)*x[0]+(0.348033100721)*x[1]**o+(0.348640167731)*x[1]+(-0.482286157201)*x[2]**o+(0.23184848892)*x[2]
        s[3,1,0,0]=(0.79458009346)*x[0]**o+(0.346244735823)*x[0]+(0.274964941045)*x[1]**o+(0.306665745695)*x[1]+(0.752939751818)*x[2]**o+(-0.0212951945504)*x[2]
        s[3,1,0,1]=(0.279860784064)*x[0]**o+(0.728689070796)*x[0]+(0.112157508671)*x[1]**o+(-0.885233951967)*x[1]+(-0.469780984101)*x[2]**o+(0.177020475912)*x[2]
        s[3,1,1,0]=(0.433534890526)*x[0]**o+(0.485733515889)*x[0]+(0.830706912979)*x[1]**o+(0.118568290881)*x[1]+(-0.863800517187)*x[2]**o+(0.737792919447)*x[2]
        s[3,1,1,1]=(0.791784006945)*x[0]**o+(-0.112270136239)*x[0]+(0.874488179673)*x[1]**o+(-0.678555278645)*x[1]+(0.985727158563)*x[2]**o+(0.920137638515)*x[2]
        s[3,1,2,0]=(-0.961254804108)*x[0]**o+(0.243551301383)*x[0]+(-0.311248382242)*x[1]**o+(0.41289285623)*x[1]+(-0.371475600554)*x[2]**o+(-0.0582608041921)*x[2]
        s[3,1,2,1]=(-0.144445942195)*x[0]**o+(-0.418750582693)*x[0]+(-0.468333324126)*x[1]**o+(-0.658823898386)*x[1]+(-0.909121304085)*x[2]**o+(-0.600433830184)*x[2]
        s[3,2,0,0]=(-0.286554311888)*x[0]**o+(-0.737505598582)*x[0]+(0.566521914245)*x[1]**o+(0.498091322078)*x[1]+(0.0202811708798)*x[2]**o+(-0.792584826215)*x[2]
        s[3,2,0,1]=(-0.52573040544)*x[0]**o+(0.256454412031)*x[0]+(-0.771513393908)*x[1]**o+(0.0769633391923)*x[1]+(0.447325513976)*x[2]**o+(0.860889667608)*x[2]
        s[3,2,1,0]=(-0.776650147122)*x[0]**o+(-0.597663916146)*x[0]+(-0.56596948577)*x[1]**o+(0.606515796008)*x[1]+(-0.0569814692195)*x[2]**o+(0.0832162598646)*x[2]
        s[3,2,1,1]=(-0.702983139798)*x[0]**o+(-0.0721500323076)*x[0]+(-0.696083887377)*x[1]**o+(-0.00975653495795)*x[1]+(0.972659180196)*x[2]**o+(0.968527616515)*x[2]
        s[3,2,2,0]=(-0.861690620984)*x[0]**o+(0.169760353347)*x[0]+(-0.774337793749)*x[1]**o+(-0.799362859305)*x[1]+(-0.352790827791)*x[2]**o+(-0.31255981171)*x[2]
        s[3,2,2,1]=(0.264209810017)*x[0]**o+(-0.86560795678)*x[0]+(0.306794805255)*x[1]**o+(0.795495753342)*x[1]+(0.5631729589)*x[2]**o+(-0.901732966437)*x[2]
        s[3,3,0,0]=(0.151220066195)*x[0]**o+(0.294238555124)*x[0]+(0.725179461723)*x[1]**o+(0.823417422915)*x[1]+(0.826457989801)*x[2]**o+(-0.068783769033)*x[2]
        s[3,3,0,1]=(0.965148541899)*x[0]**o+(0.054903011166)*x[0]+(-0.564782216515)*x[1]**o+(-0.825732725252)*x[1]+(-0.172033069024)*x[2]**o+(0.657198365579)*x[2]
        s[3,3,1,0]=(0.695530033492)*x[0]**o+(-0.251400308058)*x[0]+(-0.379416854517)*x[1]**o+(-0.491666460242)*x[1]+(0.215166477617)*x[2]**o+(0.690685608974)*x[2]
        s[3,3,1,1]=(0.377513737173)*x[0]**o+(-0.0172594331468)*x[0]+(0.42799383164)*x[1]**o+(0.0607216253163)*x[1]+(-0.153084202648)*x[2]**o+(0.411927363219)*x[2]
        s[3,3,2,0]=(-0.125731371056)*x[0]**o+(0.33378895482)*x[0]+(-0.448245878593)*x[1]**o+(-0.764746615313)*x[1]+(0.181875287538)*x[2]**o+(-0.581992845032)*x[2]
        s[3,3,2,1]=(-0.584517106374)*x[0]**o+(0.793927805805)*x[0]+(0.300548289158)*x[1]**o+(0.0449525116422)*x[1]+(0.883866474871)*x[2]**o+(0.518036509563)*x[2]
        s[3,4,0,0]=(-0.685669036611)*x[0]**o+(-0.406759680831)*x[0]+(0.020288611842)*x[1]**o+(0.524437172569)*x[1]+(-0.377692301032)*x[2]**o+(-0.969562114494)*x[2]
        s[3,4,0,1]=(0.541982416337)*x[0]**o+(0.839468986455)*x[0]+(-0.542504618993)*x[1]**o+(-0.458056288842)*x[1]+(0.982937308016)*x[2]**o+(0.751237990644)*x[2]
        s[3,4,1,0]=(-0.623461568258)*x[0]**o+(0.469859301643)*x[0]+(0.841657608811)*x[1]**o+(0.819674909109)*x[1]+(-0.655140622922)*x[2]**o+(0.0824474652875)*x[2]
        s[3,4,1,1]=(0.906724250555)*x[0]**o+(0.14855445564)*x[0]+(-0.862379845699)*x[1]**o+(-0.853300535973)*x[1]+(-0.714581251705)*x[2]**o+(-0.956268149641)*x[2]
        s[3,4,2,0]=(-0.721603766365)*x[0]**o+(0.132835246572)*x[0]+(0.0527745535366)*x[1]**o+(-0.161464199846)*x[1]+(-0.551405664077)*x[2]**o+(-0.696496777295)*x[2]
        s[3,4,2,1]=(-0.215666532882)*x[0]**o+(0.145798314909)*x[0]+(0.547835453007)*x[1]**o+(0.6070128325)*x[1]+(0.179306259279)*x[2]**o+(-0.0234805628101)*x[2]
        ref[0,0,0,0]=(-0.0939552237867)*x_ref[0]**o+(-0.233782296435)*x_ref[0]+(0.272117493235)*x_ref[1]**o+(0.938376635468)*x_ref[1]+(0.661832811129)*x_ref[2]**o+(0.282399882837)*x_ref[2]
        ref[0,0,0,1]=(0.236203034262)*x_ref[0]**o+(-0.583700442678)*x_ref[0]+(-0.870895994748)*x_ref[1]**o+(0.192686915202)*x_ref[1]+(-0.901022579657)*x_ref[2]**o+(-0.329112379375)*x_ref[2]
        ref[0,0,1,0]=(-0.583068685335)*x_ref[0]**o+(0.981024224765)*x_ref[0]+(0.869746509265)*x_ref[1]**o+(0.713473797745)*x_ref[1]+(0.647842443595)*x_ref[2]**o+(0.128105689437)*x_ref[2]
        ref[0,0,1,1]=(0.231954364537)*x_ref[0]**o+(0.301876895567)*x_ref[0]+(0.902392392762)*x_ref[1]**o+(0.77394994345)*x_ref[1]+(-0.617945530904)*x_ref[2]**o+(0.596296526597)*x_ref[2]
        ref[0,0,2,0]=(-0.848859481595)*x_ref[0]**o+(0.489664239901)*x_ref[0]+(0.101756068383)*x_ref[1]**o+(0.372022728544)*x_ref[1]+(-0.0480662015594)*x_ref[2]**o+(-0.140663588571)*x_ref[2]
        ref[0,0,2,1]=(-0.151813501072)*x_ref[0]**o+(-0.0619139652224)*x_ref[0]+(-0.2404019023)*x_ref[1]**o+(0.186875111425)*x_ref[1]+(0.870121943947)*x_ref[2]**o+(-0.401842672093)*x_ref[2]
        ref[0,1,0,0]=(-0.862371891518)*x_ref[0]**o+(-0.141519691948)*x_ref[0]+(0.0218465584484)*x_ref[1]**o+(0.474483154145)*x_ref[1]+(0.00642588343357)*x_ref[2]**o+(0.0937465696159)*x_ref[2]
        ref[0,1,0,1]=(0.957271234184)*x_ref[0]**o+(0.181371031824)*x_ref[0]+(0.325782983064)*x_ref[1]**o+(-0.758554608166)*x_ref[1]+(0.888493514738)*x_ref[2]**o+(0.360604795131)*x_ref[2]
        ref[0,1,1,0]=(0.8810614634)*x_ref[0]**o+(-0.891947928245)*x_ref[0]+(-0.81840638286)*x_ref[1]**o+(-0.338084999054)*x_ref[1]+(-0.408571326721)*x_ref[2]**o+(-0.675124681083)*x_ref[2]
        ref[0,1,1,1]=(-0.00418963259364)*x_ref[0]**o+(0.0872512414907)*x_ref[0]+(0.307397770057)*x_ref[1]**o+(-0.784741288514)*x_ref[1]+(-0.288826977888)*x_ref[2]**o+(-0.079246885912)*x_ref[2]
        ref[0,1,2,0]=(-0.933072733018)*x_ref[0]**o+(0.141983186826)*x_ref[0]+(-0.512297440539)*x_ref[1]**o+(-0.232926253673)*x_ref[1]+(0.884581006836)*x_ref[2]**o+(-0.196597583936)*x_ref[2]
        ref[0,1,2,1]=(-0.264256230447)*x_ref[0]**o+(0.0633721777798)*x_ref[0]+(0.798360353199)*x_ref[1]**o+(-0.323014270481)*x_ref[1]+(0.99252859322)*x_ref[2]**o+(0.933710902797)*x_ref[2]
        ref[0,2,0,0]=(0.402384727981)*x_ref[0]**o+(-0.600697619335)*x_ref[0]+(0.0672711555798)*x_ref[1]**o+(-0.905342068352)*x_ref[1]+(0.576003188043)*x_ref[2]**o+(0.136818654256)*x_ref[2]
        ref[0,2,0,1]=(0.335215866093)*x_ref[0]**o+(-0.128245834791)*x_ref[0]+(0.214262493314)*x_ref[1]**o+(-0.278381637725)*x_ref[1]+(0.506994464335)*x_ref[2]**o+(-0.426267615741)*x_ref[2]
        ref[0,2,1,0]=(0.463302776547)*x_ref[0]**o+(0.836336860793)*x_ref[0]+(0.978118223846)*x_ref[1]**o+(-0.299895814699)*x_ref[1]+(0.816774597805)*x_ref[2]**o+(0.561662450214)*x_ref[2]
        ref[0,2,1,1]=(-0.355099999892)*x_ref[0]**o+(-0.867893817303)*x_ref[0]+(0.318137264146)*x_ref[1]**o+(0.996417293091)*x_ref[1]+(0.190720206337)*x_ref[2]**o+(-0.956547404476)*x_ref[2]
        ref[0,2,2,0]=(0.636419754289)*x_ref[0]**o+(-0.490121483483)*x_ref[0]+(0.0724785970005)*x_ref[1]**o+(0.654051448577)*x_ref[1]+(-0.682352816989)*x_ref[2]**o+(-0.969295042468)*x_ref[2]
        ref[0,2,2,1]=(0.523182385281)*x_ref[0]**o+(0.882794657228)*x_ref[0]+(-0.258639599201)*x_ref[1]**o+(0.870016686554)*x_ref[1]+(0.810959873296)*x_ref[2]**o+(-0.120051000895)*x_ref[2]
        ref[0,3,0,0]=(-0.142691620177)*x_ref[0]**o+(-0.26145359895)*x_ref[0]+(0.979353443033)*x_ref[1]**o+(0.812390675638)*x_ref[1]+(0.659244834954)*x_ref[2]**o+(0.951080038113)*x_ref[2]
        ref[0,3,0,1]=(-0.969621517275)*x_ref[0]**o+(-0.0841833729288)*x_ref[0]+(0.183088818824)*x_ref[1]**o+(-0.0637411180744)*x_ref[1]+(0.70498987804)*x_ref[2]**o+(0.464729030647)*x_ref[2]
        ref[0,3,1,0]=(-0.908811399013)*x_ref[0]**o+(0.469705146462)*x_ref[0]+(0.623185114304)*x_ref[1]**o+(0.156348468944)*x_ref[1]+(-0.0352444815103)*x_ref[2]**o+(-0.0924055149797)*x_ref[2]
        ref[0,3,1,1]=(-0.804539460051)*x_ref[0]**o+(-0.64073007907)*x_ref[0]+(0.871919457046)*x_ref[1]**o+(-0.446001955346)*x_ref[1]+(-0.758572789631)*x_ref[2]**o+(-0.943023339532)*x_ref[2]
        ref[0,3,2,0]=(0.76012858373)*x_ref[0]**o+(-0.413461345613)*x_ref[0]+(-0.636494843218)*x_ref[1]**o+(0.0576409596138)*x_ref[1]+(0.32356830303)*x_ref[2]**o+(0.66336356169)*x_ref[2]
        ref[0,3,2,1]=(-0.682969408206)*x_ref[0]**o+(-0.00378599179446)*x_ref[0]+(0.139592931649)*x_ref[1]**o+(0.230923755162)*x_ref[1]+(-0.359747563013)*x_ref[2]**o+(0.490767562994)*x_ref[2]
        ref[0,4,0,0]=(0.40294705204)*x_ref[0]**o+(-0.725991341301)*x_ref[0]+(-0.123299724766)*x_ref[1]**o+(0.783062653249)*x_ref[1]+(0.284875756512)*x_ref[2]**o+(-0.479856616542)*x_ref[2]
        ref[0,4,0,1]=(0.671083818646)*x_ref[0]**o+(0.475388684118)*x_ref[0]+(-0.0415590406628)*x_ref[1]**o+(0.523350160591)*x_ref[1]+(-0.869125066985)*x_ref[2]**o+(0.539765124009)*x_ref[2]
        ref[0,4,1,0]=(0.517796715592)*x_ref[0]**o+(0.273319492606)*x_ref[0]+(0.131964130241)*x_ref[1]**o+(0.522704060425)*x_ref[1]+(0.945735742195)*x_ref[2]**o+(-0.927060768569)*x_ref[2]
        ref[0,4,1,1]=(-0.0877749093432)*x_ref[0]**o+(-0.228856801496)*x_ref[0]+(-0.918861499662)*x_ref[1]**o+(0.783335087368)*x_ref[1]+(0.731374866941)*x_ref[2]**o+(-0.0169316582485)*x_ref[2]
        ref[0,4,2,0]=(-0.521914417723)*x_ref[0]**o+(0.100272381296)*x_ref[0]+(-0.37855972365)*x_ref[1]**o+(-0.748174737428)*x_ref[1]+(0.08343326291)*x_ref[2]**o+(-0.284290813001)*x_ref[2]
        ref[0,4,2,1]=(-0.622347209837)*x_ref[0]**o+(-0.737347404815)*x_ref[0]+(0.973210808148)*x_ref[1]**o+(0.34940344181)*x_ref[1]+(0.917063169506)*x_ref[2]**o+(0.40443560458)*x_ref[2]
        ref[1,0,0,0]=(0.906350070311)*x_ref[0]**o+(-0.514349936373)*x_ref[0]+(-0.0377819340766)*x_ref[1]**o+(-0.868663863385)*x_ref[1]+(-0.910618099331)*x_ref[2]**o+(0.161682748869)*x_ref[2]
        ref[1,0,0,1]=(-0.525696726097)*x_ref[0]**o+(-0.564534313649)*x_ref[0]+(0.294413275375)*x_ref[1]**o+(-0.032889914662)*x_ref[1]+(-0.655188068906)*x_ref[2]**o+(-0.544604687619)*x_ref[2]
        ref[1,0,1,0]=(0.982917203627)*x_ref[0]**o+(0.23003920025)*x_ref[0]+(0.53970317464)*x_ref[1]**o+(0.488996625882)*x_ref[1]+(0.155567762567)*x_ref[2]**o+(-0.0769333810796)*x_ref[2]
        ref[1,0,1,1]=(-0.695407739098)*x_ref[0]**o+(-0.528856183813)*x_ref[0]+(-0.765981763723)*x_ref[1]**o+(-0.747233410366)*x_ref[1]+(-0.217961552142)*x_ref[2]**o+(0.363922845031)*x_ref[2]
        ref[1,0,2,0]=(-0.508333136514)*x_ref[0]**o+(-0.0531371629837)*x_ref[0]+(0.147835213524)*x_ref[1]**o+(0.811513270701)*x_ref[1]+(-0.51987266682)*x_ref[2]**o+(-0.840524592697)*x_ref[2]
        ref[1,0,2,1]=(0.522917930226)*x_ref[0]**o+(-0.387982699235)*x_ref[0]+(-0.716946321359)*x_ref[1]**o+(0.440470433449)*x_ref[1]+(0.841627656575)*x_ref[2]**o+(0.635104516674)*x_ref[2]
        ref[1,1,0,0]=(0.484898204665)*x_ref[0]**o+(-0.753933429744)*x_ref[0]+(-0.275664031761)*x_ref[1]**o+(-0.566814900411)*x_ref[1]+(0.0612476985048)*x_ref[2]**o+(0.437090771267)*x_ref[2]
        ref[1,1,0,1]=(-0.182638197905)*x_ref[0]**o+(-0.778469897629)*x_ref[0]+(-0.626963871394)*x_ref[1]**o+(-0.400611638351)*x_ref[1]+(0.895655277012)*x_ref[2]**o+(0.0326121043684)*x_ref[2]
        ref[1,1,1,0]=(-0.916569622941)*x_ref[0]**o+(-0.658275271075)*x_ref[0]+(0.250794036795)*x_ref[1]**o+(-0.233593794029)*x_ref[1]+(-0.96809736282)*x_ref[2]**o+(0.656072783817)*x_ref[2]
        ref[1,1,1,1]=(0.0171169428141)*x_ref[0]**o+(0.860556263522)*x_ref[0]+(0.345606647004)*x_ref[1]**o+(0.564186697526)*x_ref[1]+(0.588489057339)*x_ref[2]**o+(-0.87110086452)*x_ref[2]
        ref[1,1,2,0]=(0.275896407623)*x_ref[0]**o+(0.0122423763916)*x_ref[0]+(0.107452743797)*x_ref[1]**o+(-0.367492719113)*x_ref[1]+(0.884317274048)*x_ref[2]**o+(0.0908799334316)*x_ref[2]
        ref[1,1,2,1]=(0.642219111519)*x_ref[0]**o+(-0.183742201216)*x_ref[0]+(-0.327197867043)*x_ref[1]**o+(0.322321062553)*x_ref[1]+(-0.183074011204)*x_ref[2]**o+(0.720021602264)*x_ref[2]
        ref[1,2,0,0]=(0.532332942294)*x_ref[0]**o+(0.950878496192)*x_ref[0]+(0.819174829388)*x_ref[1]**o+(0.43412262514)*x_ref[1]+(-0.166925240319)*x_ref[2]**o+(-0.698173204203)*x_ref[2]
        ref[1,2,0,1]=(0.259909649155)*x_ref[0]**o+(-0.264967838281)*x_ref[0]+(-0.774874307838)*x_ref[1]**o+(-0.275886773315)*x_ref[1]+(0.801233942875)*x_ref[2]**o+(0.27496052417)*x_ref[2]
        ref[1,2,1,0]=(-0.746564397549)*x_ref[0]**o+(-0.690480530678)*x_ref[0]+(0.114986469362)*x_ref[1]**o+(-0.751415850669)*x_ref[1]+(0.119951306293)*x_ref[2]**o+(-0.665012834766)*x_ref[2]
        ref[1,2,1,1]=(-0.851672541025)*x_ref[0]**o+(0.235181175896)*x_ref[0]+(-0.62908061934)*x_ref[1]**o+(-0.438672467151)*x_ref[1]+(-0.761945783738)*x_ref[2]**o+(0.580730895385)*x_ref[2]
        ref[1,2,2,0]=(0.676776403141)*x_ref[0]**o+(-0.707505868325)*x_ref[0]+(0.724421249054)*x_ref[1]**o+(0.120766197116)*x_ref[1]+(-0.760834341632)*x_ref[2]**o+(-0.689464446912)*x_ref[2]
        ref[1,2,2,1]=(0.147028945539)*x_ref[0]**o+(0.199909936204)*x_ref[0]+(0.903168165102)*x_ref[1]**o+(-0.46293792368)*x_ref[1]+(-0.282517651615)*x_ref[2]**o+(0.740619136229)*x_ref[2]
        ref[1,3,0,0]=(-0.87747323136)*x_ref[0]**o+(-0.361656418039)*x_ref[0]+(-0.14856291522)*x_ref[1]**o+(0.474044532693)*x_ref[1]+(0.138460936937)*x_ref[2]**o+(-0.145182012384)*x_ref[2]
        ref[1,3,0,1]=(0.748334497747)*x_ref[0]**o+(0.892498113089)*x_ref[0]+(0.239113720175)*x_ref[1]**o+(-0.762524411251)*x_ref[1]+(-0.488407873495)*x_ref[2]**o+(0.158979901061)*x_ref[2]
        ref[1,3,1,0]=(0.465350922376)*x_ref[0]**o+(-0.654156421791)*x_ref[0]+(0.878703330328)*x_ref[1]**o+(0.537076882157)*x_ref[1]+(0.389722314724)*x_ref[2]**o+(-0.120902843496)*x_ref[2]
        ref[1,3,1,1]=(-0.153324789456)*x_ref[0]**o+(-0.842917613229)*x_ref[0]+(0.984971864624)*x_ref[1]**o+(0.349269583591)*x_ref[1]+(0.484775635349)*x_ref[2]**o+(0.0123674100644)*x_ref[2]
        ref[1,3,2,0]=(-0.175935804533)*x_ref[0]**o+(0.0654493735159)*x_ref[0]+(0.726667315683)*x_ref[1]**o+(0.979882137896)*x_ref[1]+(0.543445934589)*x_ref[2]**o+(-0.149142988507)*x_ref[2]
        ref[1,3,2,1]=(0.235188036633)*x_ref[0]**o+(-0.383249536636)*x_ref[0]+(-0.445575566142)*x_ref[1]**o+(-0.819686823355)*x_ref[1]+(0.170138244522)*x_ref[2]**o+(-0.80311924596)*x_ref[2]
        ref[1,4,0,0]=(0.471387483772)*x_ref[0]**o+(-0.591448975176)*x_ref[0]+(-0.833833649412)*x_ref[1]**o+(0.770775874052)*x_ref[1]+(0.537769198907)*x_ref[2]**o+(-0.646003815898)*x_ref[2]
        ref[1,4,0,1]=(-0.797500020228)*x_ref[0]**o+(-0.136543576325)*x_ref[0]+(0.11979987113)*x_ref[1]**o+(-0.495999866876)*x_ref[1]+(-0.611958471401)*x_ref[2]**o+(-0.588202607164)*x_ref[2]
        ref[1,4,1,0]=(-0.550038387555)*x_ref[0]**o+(0.0844321262213)*x_ref[0]+(0.131005384757)*x_ref[1]**o+(-0.515674980981)*x_ref[1]+(-0.637043856082)*x_ref[2]**o+(0.333864121142)*x_ref[2]
        ref[1,4,1,1]=(0.71740532023)*x_ref[0]**o+(-0.868269127293)*x_ref[0]+(0.323641764571)*x_ref[1]**o+(0.820948811255)*x_ref[1]+(0.799638086908)*x_ref[2]**o+(-0.558360429849)*x_ref[2]
        ref[1,4,2,0]=(-0.366976697335)*x_ref[0]**o+(-0.121083094264)*x_ref[0]+(0.650559587113)*x_ref[1]**o+(-0.400220219011)*x_ref[1]+(-0.0785928535861)*x_ref[2]**o+(-0.50420364324)*x_ref[2]
        ref[1,4,2,1]=(0.872011301914)*x_ref[0]**o+(0.0300770125292)*x_ref[0]+(-0.965040697782)*x_ref[1]**o+(-0.373825300018)*x_ref[1]+(-0.988463549572)*x_ref[2]**o+(0.906686888358)*x_ref[2]
        ref[2,0,0,0]=(-0.644460598597)*x_ref[0]**o+(0.69918562566)*x_ref[0]+(0.0995801100352)*x_ref[1]**o+(0.257114765725)*x_ref[1]+(-0.172788858436)*x_ref[2]**o+(-0.625310450189)*x_ref[2]
        ref[2,0,0,1]=(0.331738659918)*x_ref[0]**o+(0.34134401217)*x_ref[0]+(0.667229795214)*x_ref[1]**o+(0.248779696936)*x_ref[1]+(0.782703155201)*x_ref[2]**o+(-0.0117889862376)*x_ref[2]
        ref[2,0,1,0]=(0.456825869656)*x_ref[0]**o+(0.618685513764)*x_ref[0]+(0.00330761288087)*x_ref[1]**o+(-0.839303906537)*x_ref[1]+(0.16840639247)*x_ref[2]**o+(-0.177323734594)*x_ref[2]
        ref[2,0,1,1]=(0.543967300033)*x_ref[0]**o+(0.537337400025)*x_ref[0]+(-0.232374591177)*x_ref[1]**o+(-0.163640094925)*x_ref[1]+(0.146221483058)*x_ref[2]**o+(0.833185534106)*x_ref[2]
        ref[2,0,2,0]=(-0.283419495517)*x_ref[0]**o+(-0.0641847006529)*x_ref[0]+(0.933228590846)*x_ref[1]**o+(0.575529900563)*x_ref[1]+(-0.634849100928)*x_ref[2]**o+(-0.898678364957)*x_ref[2]
        ref[2,0,2,1]=(0.248202597053)*x_ref[0]**o+(-0.395428705005)*x_ref[0]+(-0.921778142592)*x_ref[1]**o+(0.752699910606)*x_ref[1]+(-0.208939925012)*x_ref[2]**o+(0.252031969391)*x_ref[2]
        ref[2,1,0,0]=(0.543045279112)*x_ref[0]**o+(-0.200796280468)*x_ref[0]+(-0.850234504692)*x_ref[1]**o+(-0.608668852127)*x_ref[1]+(-0.773553907964)*x_ref[2]**o+(-0.727318461167)*x_ref[2]
        ref[2,1,0,1]=(0.670138313505)*x_ref[0]**o+(0.763313503204)*x_ref[0]+(-0.712024856615)*x_ref[1]**o+(-0.0810633888547)*x_ref[1]+(-0.543339707954)*x_ref[2]**o+(-0.0944422473469)*x_ref[2]
        ref[2,1,1,0]=(0.66812399426)*x_ref[0]**o+(-0.00267070537766)*x_ref[0]+(0.704373662345)*x_ref[1]**o+(-0.778778632343)*x_ref[1]+(-0.195017717355)*x_ref[2]**o+(-0.718710830736)*x_ref[2]
        ref[2,1,1,1]=(-0.905514219789)*x_ref[0]**o+(-0.899693862646)*x_ref[0]+(-0.791063116492)*x_ref[1]**o+(-0.470046803205)*x_ref[1]+(0.954924769408)*x_ref[2]**o+(0.36799445545)*x_ref[2]
        ref[2,1,2,0]=(0.363438950523)*x_ref[0]**o+(-0.788582160746)*x_ref[0]+(0.316623350464)*x_ref[1]**o+(-0.621859877627)*x_ref[1]+(-0.899933912274)*x_ref[2]**o+(-0.663582380926)*x_ref[2]
        ref[2,1,2,1]=(0.70767884819)*x_ref[0]**o+(-0.337197298469)*x_ref[0]+(0.329833087038)*x_ref[1]**o+(-0.767476122169)*x_ref[1]+(0.813567450674)*x_ref[2]**o+(-0.230739080738)*x_ref[2]
        ref[2,2,0,0]=(0.425629461437)*x_ref[0]**o+(0.736062549141)*x_ref[0]+(0.0766779443487)*x_ref[1]**o+(-0.645685464512)*x_ref[1]+(-0.563156862696)*x_ref[2]**o+(-0.200669234455)*x_ref[2]
        ref[2,2,0,1]=(-0.436511105169)*x_ref[0]**o+(-0.784209106822)*x_ref[0]+(0.476448752447)*x_ref[1]**o+(-0.976253075887)*x_ref[1]+(0.744207251461)*x_ref[2]**o+(-0.480585834629)*x_ref[2]
        ref[2,2,1,0]=(-0.160519965814)*x_ref[0]**o+(-0.740586061373)*x_ref[0]+(-0.291540051837)*x_ref[1]**o+(0.0186903802094)*x_ref[1]+(-0.456120795038)*x_ref[2]**o+(0.930603860664)*x_ref[2]
        ref[2,2,1,1]=(0.835200574398)*x_ref[0]**o+(-0.111618184052)*x_ref[0]+(0.394793418949)*x_ref[1]**o+(0.66759987425)*x_ref[1]+(0.902913963474)*x_ref[2]**o+(-0.840871310598)*x_ref[2]
        ref[2,2,2,0]=(-0.745841363439)*x_ref[0]**o+(0.859714040441)*x_ref[0]+(-0.133922374335)*x_ref[1]**o+(-0.539065635521)*x_ref[1]+(-0.0496309374348)*x_ref[2]**o+(-0.3150513125)*x_ref[2]
        ref[2,2,2,1]=(-0.121952493914)*x_ref[0]**o+(-0.145565953724)*x_ref[0]+(-0.46956691252)*x_ref[1]**o+(0.594587113542)*x_ref[1]+(0.801503774207)*x_ref[2]**o+(-0.559107902481)*x_ref[2]
        ref[2,3,0,0]=(0.107705603601)*x_ref[0]**o+(0.83167538493)*x_ref[0]+(0.719052082882)*x_ref[1]**o+(-0.0418541572577)*x_ref[1]+(-0.760679778738)*x_ref[2]**o+(-0.269573603299)*x_ref[2]
        ref[2,3,0,1]=(-0.610961305034)*x_ref[0]**o+(-0.48822367684)*x_ref[0]+(-0.533146614698)*x_ref[1]**o+(-0.0634033507577)*x_ref[1]+(-0.525819529758)*x_ref[2]**o+(0.70089075883)*x_ref[2]
        ref[2,3,1,0]=(0.503925514982)*x_ref[0]**o+(0.65957769246)*x_ref[0]+(-0.624626366929)*x_ref[1]**o+(-0.374542181845)*x_ref[1]+(-0.363387031742)*x_ref[2]**o+(-0.00563190329728)*x_ref[2]
        ref[2,3,1,1]=(0.337750136392)*x_ref[0]**o+(-0.364300637249)*x_ref[0]+(0.0892494275431)*x_ref[1]**o+(0.958228736151)*x_ref[1]+(0.203359176466)*x_ref[2]**o+(-0.454161162092)*x_ref[2]
        ref[2,3,2,0]=(0.171894913416)*x_ref[0]**o+(0.930342123436)*x_ref[0]+(0.888973400724)*x_ref[1]**o+(0.732732070997)*x_ref[1]+(-0.365536399448)*x_ref[2]**o+(0.719027934816)*x_ref[2]
        ref[2,3,2,1]=(0.0718345473684)*x_ref[0]**o+(-0.351767950331)*x_ref[0]+(-0.427880627822)*x_ref[1]**o+(0.906554327062)*x_ref[1]+(0.209905901579)*x_ref[2]**o+(0.343983972848)*x_ref[2]
        ref[2,4,0,0]=(0.20082437951)*x_ref[0]**o+(0.411679233479)*x_ref[0]+(-0.146683847376)*x_ref[1]**o+(0.619164773872)*x_ref[1]+(0.283942602042)*x_ref[2]**o+(0.328844649922)*x_ref[2]
        ref[2,4,0,1]=(0.512237008877)*x_ref[0]**o+(-0.0799414456365)*x_ref[0]+(-0.444485238118)*x_ref[1]**o+(-0.617601180088)*x_ref[1]+(-0.522016560329)*x_ref[2]**o+(0.764966540338)*x_ref[2]
        ref[2,4,1,0]=(-0.906295486351)*x_ref[0]**o+(0.213043439406)*x_ref[0]+(-0.841541923905)*x_ref[1]**o+(0.613189806332)*x_ref[1]+(0.98016601466)*x_ref[2]**o+(0.373100080758)*x_ref[2]
        ref[2,4,1,1]=(0.421256899132)*x_ref[0]**o+(-0.148027161979)*x_ref[0]+(-0.436551670748)*x_ref[1]**o+(0.15389527845)*x_ref[1]+(-0.202772011213)*x_ref[2]**o+(0.526456981868)*x_ref[2]
        ref[2,4,2,0]=(0.326695825605)*x_ref[0]**o+(-0.658196467156)*x_ref[0]+(-0.0229897453445)*x_ref[1]**o+(0.640660125725)*x_ref[1]+(0.604325801088)*x_ref[2]**o+(0.415440861181)*x_ref[2]
        ref[2,4,2,1]=(-0.353746711731)*x_ref[0]**o+(0.707902797414)*x_ref[0]+(-0.117816706586)*x_ref[1]**o+(0.767979696436)*x_ref[1]+(0.0361750168523)*x_ref[2]**o+(-0.904298176296)*x_ref[2]
        ref[3,0,0,0]=(0.274392483594)*x_ref[0]**o+(-0.142144450689)*x_ref[0]+(-0.308306223778)*x_ref[1]**o+(0.674122485073)*x_ref[1]+(0.259741082605)*x_ref[2]**o+(-0.249672127213)*x_ref[2]
        ref[3,0,0,1]=(-0.299212636089)*x_ref[0]**o+(0.670140835575)*x_ref[0]+(0.0513019322721)*x_ref[1]**o+(0.460049892726)*x_ref[1]+(0.167967285754)*x_ref[2]**o+(-0.463520967247)*x_ref[2]
        ref[3,0,1,0]=(0.850761515544)*x_ref[0]**o+(0.945654906297)*x_ref[0]+(-0.841039831472)*x_ref[1]**o+(-0.051572515458)*x_ref[1]+(0.750689246331)*x_ref[2]**o+(0.0651065819687)*x_ref[2]
        ref[3,0,1,1]=(-0.387082102682)*x_ref[0]**o+(-0.89541016885)*x_ref[0]+(-0.903675915508)*x_ref[1]**o+(-0.268304570988)*x_ref[1]+(-0.259828889903)*x_ref[2]**o+(-0.983650155837)*x_ref[2]
        ref[3,0,2,0]=(0.497841641089)*x_ref[0]**o+(0.80522306204)*x_ref[0]+(0.532637885608)*x_ref[1]**o+(-0.345627398376)*x_ref[1]+(-0.768962683025)*x_ref[2]**o+(-0.608176054775)*x_ref[2]
        ref[3,0,2,1]=(0.00437007549865)*x_ref[0]**o+(0.847511281463)*x_ref[0]+(0.348033100721)*x_ref[1]**o+(0.348640167731)*x_ref[1]+(-0.482286157201)*x_ref[2]**o+(0.23184848892)*x_ref[2]
        ref[3,1,0,0]=(0.79458009346)*x_ref[0]**o+(0.346244735823)*x_ref[0]+(0.274964941045)*x_ref[1]**o+(0.306665745695)*x_ref[1]+(0.752939751818)*x_ref[2]**o+(-0.0212951945504)*x_ref[2]
        ref[3,1,0,1]=(0.279860784064)*x_ref[0]**o+(0.728689070796)*x_ref[0]+(0.112157508671)*x_ref[1]**o+(-0.885233951967)*x_ref[1]+(-0.469780984101)*x_ref[2]**o+(0.177020475912)*x_ref[2]
        ref[3,1,1,0]=(0.433534890526)*x_ref[0]**o+(0.485733515889)*x_ref[0]+(0.830706912979)*x_ref[1]**o+(0.118568290881)*x_ref[1]+(-0.863800517187)*x_ref[2]**o+(0.737792919447)*x_ref[2]
        ref[3,1,1,1]=(0.791784006945)*x_ref[0]**o+(-0.112270136239)*x_ref[0]+(0.874488179673)*x_ref[1]**o+(-0.678555278645)*x_ref[1]+(0.985727158563)*x_ref[2]**o+(0.920137638515)*x_ref[2]
        ref[3,1,2,0]=(-0.961254804108)*x_ref[0]**o+(0.243551301383)*x_ref[0]+(-0.311248382242)*x_ref[1]**o+(0.41289285623)*x_ref[1]+(-0.371475600554)*x_ref[2]**o+(-0.0582608041921)*x_ref[2]
        ref[3,1,2,1]=(-0.144445942195)*x_ref[0]**o+(-0.418750582693)*x_ref[0]+(-0.468333324126)*x_ref[1]**o+(-0.658823898386)*x_ref[1]+(-0.909121304085)*x_ref[2]**o+(-0.600433830184)*x_ref[2]
        ref[3,2,0,0]=(-0.286554311888)*x_ref[0]**o+(-0.737505598582)*x_ref[0]+(0.566521914245)*x_ref[1]**o+(0.498091322078)*x_ref[1]+(0.0202811708798)*x_ref[2]**o+(-0.792584826215)*x_ref[2]
        ref[3,2,0,1]=(-0.52573040544)*x_ref[0]**o+(0.256454412031)*x_ref[0]+(-0.771513393908)*x_ref[1]**o+(0.0769633391923)*x_ref[1]+(0.447325513976)*x_ref[2]**o+(0.860889667608)*x_ref[2]
        ref[3,2,1,0]=(-0.776650147122)*x_ref[0]**o+(-0.597663916146)*x_ref[0]+(-0.56596948577)*x_ref[1]**o+(0.606515796008)*x_ref[1]+(-0.0569814692195)*x_ref[2]**o+(0.0832162598646)*x_ref[2]
        ref[3,2,1,1]=(-0.702983139798)*x_ref[0]**o+(-0.0721500323076)*x_ref[0]+(-0.696083887377)*x_ref[1]**o+(-0.00975653495795)*x_ref[1]+(0.972659180196)*x_ref[2]**o+(0.968527616515)*x_ref[2]
        ref[3,2,2,0]=(-0.861690620984)*x_ref[0]**o+(0.169760353347)*x_ref[0]+(-0.774337793749)*x_ref[1]**o+(-0.799362859305)*x_ref[1]+(-0.352790827791)*x_ref[2]**o+(-0.31255981171)*x_ref[2]
        ref[3,2,2,1]=(0.264209810017)*x_ref[0]**o+(-0.86560795678)*x_ref[0]+(0.306794805255)*x_ref[1]**o+(0.795495753342)*x_ref[1]+(0.5631729589)*x_ref[2]**o+(-0.901732966437)*x_ref[2]
        ref[3,3,0,0]=(0.151220066195)*x_ref[0]**o+(0.294238555124)*x_ref[0]+(0.725179461723)*x_ref[1]**o+(0.823417422915)*x_ref[1]+(0.826457989801)*x_ref[2]**o+(-0.068783769033)*x_ref[2]
        ref[3,3,0,1]=(0.965148541899)*x_ref[0]**o+(0.054903011166)*x_ref[0]+(-0.564782216515)*x_ref[1]**o+(-0.825732725252)*x_ref[1]+(-0.172033069024)*x_ref[2]**o+(0.657198365579)*x_ref[2]
        ref[3,3,1,0]=(0.695530033492)*x_ref[0]**o+(-0.251400308058)*x_ref[0]+(-0.379416854517)*x_ref[1]**o+(-0.491666460242)*x_ref[1]+(0.215166477617)*x_ref[2]**o+(0.690685608974)*x_ref[2]
        ref[3,3,1,1]=(0.377513737173)*x_ref[0]**o+(-0.0172594331468)*x_ref[0]+(0.42799383164)*x_ref[1]**o+(0.0607216253163)*x_ref[1]+(-0.153084202648)*x_ref[2]**o+(0.411927363219)*x_ref[2]
        ref[3,3,2,0]=(-0.125731371056)*x_ref[0]**o+(0.33378895482)*x_ref[0]+(-0.448245878593)*x_ref[1]**o+(-0.764746615313)*x_ref[1]+(0.181875287538)*x_ref[2]**o+(-0.581992845032)*x_ref[2]
        ref[3,3,2,1]=(-0.584517106374)*x_ref[0]**o+(0.793927805805)*x_ref[0]+(0.300548289158)*x_ref[1]**o+(0.0449525116422)*x_ref[1]+(0.883866474871)*x_ref[2]**o+(0.518036509563)*x_ref[2]
        ref[3,4,0,0]=(-0.685669036611)*x_ref[0]**o+(-0.406759680831)*x_ref[0]+(0.020288611842)*x_ref[1]**o+(0.524437172569)*x_ref[1]+(-0.377692301032)*x_ref[2]**o+(-0.969562114494)*x_ref[2]
        ref[3,4,0,1]=(0.541982416337)*x_ref[0]**o+(0.839468986455)*x_ref[0]+(-0.542504618993)*x_ref[1]**o+(-0.458056288842)*x_ref[1]+(0.982937308016)*x_ref[2]**o+(0.751237990644)*x_ref[2]
        ref[3,4,1,0]=(-0.623461568258)*x_ref[0]**o+(0.469859301643)*x_ref[0]+(0.841657608811)*x_ref[1]**o+(0.819674909109)*x_ref[1]+(-0.655140622922)*x_ref[2]**o+(0.0824474652875)*x_ref[2]
        ref[3,4,1,1]=(0.906724250555)*x_ref[0]**o+(0.14855445564)*x_ref[0]+(-0.862379845699)*x_ref[1]**o+(-0.853300535973)*x_ref[1]+(-0.714581251705)*x_ref[2]**o+(-0.956268149641)*x_ref[2]
        ref[3,4,2,0]=(-0.721603766365)*x_ref[0]**o+(0.132835246572)*x_ref[0]+(0.0527745535366)*x_ref[1]**o+(-0.161464199846)*x_ref[1]+(-0.551405664077)*x_ref[2]**o+(-0.696496777295)*x_ref[2]
        ref[3,4,2,1]=(-0.215666532882)*x_ref[0]**o+(0.145798314909)*x_ref[0]+(0.547835453007)*x_ref[1]**o+(0.6070128325)*x_ref[1]+(0.179306259279)*x_ref[2]**o+(-0.0234805628101)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_ContinuousFunction_rank0(self):
      """
      tests interpolate for rank 0 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ContinuousFunction onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(),w)
      ref=Data(0,(),w_ref)
      if dim==2:
        arg=(0.492060381705)*x[0]**o+(0.578791868713)*x[0]+(-0.439849757114)*x[1]**o+(-0.52629328967)*x[1]
        ref=(0.492060381705)*x_ref[0]**o+(0.578791868713)*x_ref[0]+(-0.439849757114)*x_ref[1]**o+(-0.52629328967)*x_ref[1]
      else:
        arg=(0.954737963273)*x[0]**o+(0.452578946878)*x[0]+(0.0471085060698)*x[1]**o+(-0.640169119832)*x[1]+(0.710401490627)*x[2]**o+(-0.382589884739)*x[2]
        ref=(0.954737963273)*x_ref[0]**o+(0.452578946878)*x_ref[0]+(0.0471085060698)*x_ref[1]**o+(-0.640169119832)*x_ref[1]+(0.710401490627)*x_ref[2]**o+(-0.382589884739)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_ContinuousFunction_rank1(self):
      """
      tests interpolate for rank 1 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ContinuousFunction onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(2,),w)
      ref=Data(0,(2,),w_ref)
      if dim==2:
        arg[0]=(-0.59604585527)*x[0]**o+(0.826981169401)*x[0]+(0.227617349484)*x[1]**o+(0.184907819029)*x[1]
        arg[1]=(-0.424684036731)*x[0]**o+(0.346537997905)*x[0]+(0.891921272597)*x[1]**o+(-0.145924182872)*x[1]
        ref[0]=(-0.59604585527)*x_ref[0]**o+(0.826981169401)*x_ref[0]+(0.227617349484)*x_ref[1]**o+(0.184907819029)*x_ref[1]
        ref[1]=(-0.424684036731)*x_ref[0]**o+(0.346537997905)*x_ref[0]+(0.891921272597)*x_ref[1]**o+(-0.145924182872)*x_ref[1]
      else:
        arg[0]=(0.777228173978)*x[0]**o+(-0.812215827155)*x[0]+(-0.618922680968)*x[1]**o+(0.0660312103206)*x[1]+(-0.917545504466)*x[2]**o+(0.988524619414)*x[2]
        arg[1]=(0.110219469565)*x[0]**o+(-0.441866866062)*x[0]+(-0.654447764335)*x[1]**o+(0.230716133289)*x[1]+(0.347487317165)*x[2]**o+(-0.948877903043)*x[2]
        ref[0]=(0.777228173978)*x_ref[0]**o+(-0.812215827155)*x_ref[0]+(-0.618922680968)*x_ref[1]**o+(0.0660312103206)*x_ref[1]+(-0.917545504466)*x_ref[2]**o+(0.988524619414)*x_ref[2]
        ref[1]=(0.110219469565)*x_ref[0]**o+(-0.441866866062)*x_ref[0]+(-0.654447764335)*x_ref[1]**o+(0.230716133289)*x_ref[1]+(0.347487317165)*x_ref[2]**o+(-0.948877903043)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_ContinuousFunction_rank2(self):
      """
      tests interpolate for rank 2 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ContinuousFunction onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5),w)
      ref=Data(0,(4, 5),w_ref)
      if dim==2:
        arg[0,0]=(-0.958809754913)*x[0]**o+(-0.844717049041)*x[0]+(0.462564514427)*x[1]**o+(-0.247678335594)*x[1]
        arg[0,1]=(0.700700054189)*x[0]**o+(-0.653724611457)*x[0]+(0.52659177803)*x[1]**o+(-0.387559446418)*x[1]
        arg[0,2]=(-0.326124300134)*x[0]**o+(0.115197897152)*x[0]+(-0.876340641846)*x[1]**o+(0.433060472581)*x[1]
        arg[0,3]=(-0.101215736407)*x[0]**o+(-0.483684510661)*x[0]+(0.0418770907036)*x[1]**o+(-0.845564414421)*x[1]
        arg[0,4]=(-0.293094855718)*x[0]**o+(0.710954930773)*x[0]+(0.238671713869)*x[1]**o+(0.576638343412)*x[1]
        arg[1,0]=(0.529861280122)*x[0]**o+(-0.259437904048)*x[0]+(-0.620725288158)*x[1]**o+(0.326787999382)*x[1]
        arg[1,1]=(0.421245543012)*x[0]**o+(-0.954139636698)*x[0]+(0.0498199734926)*x[1]**o+(0.24010238499)*x[1]
        arg[1,2]=(0.333362362373)*x[0]**o+(-0.999953175952)*x[0]+(0.866193849246)*x[1]**o+(0.764287410177)*x[1]
        arg[1,3]=(0.470415957539)*x[0]**o+(-0.133477966871)*x[0]+(-0.0416970809832)*x[1]**o+(-0.416352740444)*x[1]
        arg[1,4]=(-0.178613018492)*x[0]**o+(-0.967264985978)*x[0]+(-0.297102423687)*x[1]**o+(0.884184973413)*x[1]
        arg[2,0]=(-0.29363453294)*x[0]**o+(-0.0237360594862)*x[0]+(-0.300776450747)*x[1]**o+(-0.493932721521)*x[1]
        arg[2,1]=(0.795045745699)*x[0]**o+(0.984956851154)*x[0]+(-0.724208405235)*x[1]**o+(0.59029067239)*x[1]
        arg[2,2]=(-0.803012112531)*x[0]**o+(-0.831727785428)*x[0]+(0.330295112332)*x[1]**o+(0.0966317908147)*x[1]
        arg[2,3]=(-0.652332429144)*x[0]**o+(-0.38689457016)*x[0]+(-0.391780825295)*x[1]**o+(-0.444284790848)*x[1]
        arg[2,4]=(0.416775161398)*x[0]**o+(-0.16281006536)*x[0]+(-0.742938932867)*x[1]**o+(0.456486183714)*x[1]
        arg[3,0]=(0.506731632086)*x[0]**o+(-0.222468320035)*x[0]+(-0.906398223556)*x[1]**o+(-0.0250630448611)*x[1]
        arg[3,1]=(-0.39207398783)*x[0]**o+(-0.574739374252)*x[0]+(0.262101584569)*x[1]**o+(0.249587535087)*x[1]
        arg[3,2]=(-0.252593913503)*x[0]**o+(-0.864294052989)*x[0]+(0.0110434296336)*x[1]**o+(-0.224585040999)*x[1]
        arg[3,3]=(-0.631704469212)*x[0]**o+(-0.495153785048)*x[0]+(-0.304420746672)*x[1]**o+(-0.254005222021)*x[1]
        arg[3,4]=(0.775631055421)*x[0]**o+(0.370095409961)*x[0]+(0.894627612994)*x[1]**o+(-0.329716708629)*x[1]
        ref[0,0]=(-0.958809754913)*x_ref[0]**o+(-0.844717049041)*x_ref[0]+(0.462564514427)*x_ref[1]**o+(-0.247678335594)*x_ref[1]
        ref[0,1]=(0.700700054189)*x_ref[0]**o+(-0.653724611457)*x_ref[0]+(0.52659177803)*x_ref[1]**o+(-0.387559446418)*x_ref[1]
        ref[0,2]=(-0.326124300134)*x_ref[0]**o+(0.115197897152)*x_ref[0]+(-0.876340641846)*x_ref[1]**o+(0.433060472581)*x_ref[1]
        ref[0,3]=(-0.101215736407)*x_ref[0]**o+(-0.483684510661)*x_ref[0]+(0.0418770907036)*x_ref[1]**o+(-0.845564414421)*x_ref[1]
        ref[0,4]=(-0.293094855718)*x_ref[0]**o+(0.710954930773)*x_ref[0]+(0.238671713869)*x_ref[1]**o+(0.576638343412)*x_ref[1]
        ref[1,0]=(0.529861280122)*x_ref[0]**o+(-0.259437904048)*x_ref[0]+(-0.620725288158)*x_ref[1]**o+(0.326787999382)*x_ref[1]
        ref[1,1]=(0.421245543012)*x_ref[0]**o+(-0.954139636698)*x_ref[0]+(0.0498199734926)*x_ref[1]**o+(0.24010238499)*x_ref[1]
        ref[1,2]=(0.333362362373)*x_ref[0]**o+(-0.999953175952)*x_ref[0]+(0.866193849246)*x_ref[1]**o+(0.764287410177)*x_ref[1]
        ref[1,3]=(0.470415957539)*x_ref[0]**o+(-0.133477966871)*x_ref[0]+(-0.0416970809832)*x_ref[1]**o+(-0.416352740444)*x_ref[1]
        ref[1,4]=(-0.178613018492)*x_ref[0]**o+(-0.967264985978)*x_ref[0]+(-0.297102423687)*x_ref[1]**o+(0.884184973413)*x_ref[1]
        ref[2,0]=(-0.29363453294)*x_ref[0]**o+(-0.0237360594862)*x_ref[0]+(-0.300776450747)*x_ref[1]**o+(-0.493932721521)*x_ref[1]
        ref[2,1]=(0.795045745699)*x_ref[0]**o+(0.984956851154)*x_ref[0]+(-0.724208405235)*x_ref[1]**o+(0.59029067239)*x_ref[1]
        ref[2,2]=(-0.803012112531)*x_ref[0]**o+(-0.831727785428)*x_ref[0]+(0.330295112332)*x_ref[1]**o+(0.0966317908147)*x_ref[1]
        ref[2,3]=(-0.652332429144)*x_ref[0]**o+(-0.38689457016)*x_ref[0]+(-0.391780825295)*x_ref[1]**o+(-0.444284790848)*x_ref[1]
        ref[2,4]=(0.416775161398)*x_ref[0]**o+(-0.16281006536)*x_ref[0]+(-0.742938932867)*x_ref[1]**o+(0.456486183714)*x_ref[1]
        ref[3,0]=(0.506731632086)*x_ref[0]**o+(-0.222468320035)*x_ref[0]+(-0.906398223556)*x_ref[1]**o+(-0.0250630448611)*x_ref[1]
        ref[3,1]=(-0.39207398783)*x_ref[0]**o+(-0.574739374252)*x_ref[0]+(0.262101584569)*x_ref[1]**o+(0.249587535087)*x_ref[1]
        ref[3,2]=(-0.252593913503)*x_ref[0]**o+(-0.864294052989)*x_ref[0]+(0.0110434296336)*x_ref[1]**o+(-0.224585040999)*x_ref[1]
        ref[3,3]=(-0.631704469212)*x_ref[0]**o+(-0.495153785048)*x_ref[0]+(-0.304420746672)*x_ref[1]**o+(-0.254005222021)*x_ref[1]
        ref[3,4]=(0.775631055421)*x_ref[0]**o+(0.370095409961)*x_ref[0]+(0.894627612994)*x_ref[1]**o+(-0.329716708629)*x_ref[1]
      else:
        arg[0,0]=(0.392450639899)*x[0]**o+(-0.685488230002)*x[0]+(-0.41545928428)*x[1]**o+(0.244144392379)*x[1]+(0.602642156528)*x[2]**o+(-0.348717378461)*x[2]
        arg[0,1]=(-0.341243165541)*x[0]**o+(-0.616870164204)*x[0]+(-0.510613066362)*x[1]**o+(-0.81387010212)*x[1]+(0.856239691119)*x[2]**o+(0.0744651854239)*x[2]
        arg[0,2]=(-0.765983413604)*x[0]**o+(0.474551394174)*x[0]+(-0.939524929765)*x[1]**o+(0.418297898065)*x[1]+(0.00589909539531)*x[2]**o+(0.0889012994897)*x[2]
        arg[0,3]=(-0.521217563352)*x[0]**o+(-0.256316459647)*x[0]+(-0.0265965098379)*x[1]**o+(-0.597562278966)*x[1]+(0.0389151661339)*x[2]**o+(-0.572109451352)*x[2]
        arg[0,4]=(-0.853801268491)*x[0]**o+(-0.691274812057)*x[0]+(-0.682435618581)*x[1]**o+(-0.974334375359)*x[1]+(-0.642333316794)*x[2]**o+(0.353816751252)*x[2]
        arg[1,0]=(-0.657545394322)*x[0]**o+(-0.334172338096)*x[0]+(0.016748279459)*x[1]**o+(-0.294858621125)*x[1]+(-0.49403880046)*x[2]**o+(-0.152416774822)*x[2]
        arg[1,1]=(-0.602731976042)*x[0]**o+(0.685046816545)*x[0]+(0.750001644716)*x[1]**o+(-0.202076972289)*x[1]+(-0.0232888078434)*x[2]**o+(-0.0774477649878)*x[2]
        arg[1,2]=(-0.707321785527)*x[0]**o+(0.618526464205)*x[0]+(-0.578548442468)*x[1]**o+(-0.411374744442)*x[1]+(-0.944277708787)*x[2]**o+(0.647856975707)*x[2]
        arg[1,3]=(-0.222503271022)*x[0]**o+(-0.929428092728)*x[0]+(-0.976553212424)*x[1]**o+(0.0376439617084)*x[1]+(0.0428078568512)*x[2]**o+(-0.441850944272)*x[2]
        arg[1,4]=(-0.862164898455)*x[0]**o+(-0.989900221991)*x[0]+(0.110610844567)*x[1]**o+(0.419384612236)*x[1]+(-0.00271948841262)*x[2]**o+(0.301635101629)*x[2]
        arg[2,0]=(-0.0950421881942)*x[0]**o+(-0.741293053909)*x[0]+(-0.461538879874)*x[1]**o+(-0.8199842553)*x[1]+(0.0590344916829)*x[2]**o+(0.352541518434)*x[2]
        arg[2,1]=(-0.734407995647)*x[0]**o+(-0.726167721992)*x[0]+(0.700521229916)*x[1]**o+(0.753555863428)*x[1]+(-0.891638139696)*x[2]**o+(-0.996550711213)*x[2]
        arg[2,2]=(0.777110352629)*x[0]**o+(0.309719201157)*x[0]+(0.026816547522)*x[1]**o+(0.33161010287)*x[1]+(0.257929504352)*x[2]**o+(0.71871314654)*x[2]
        arg[2,3]=(-0.0982522143825)*x[0]**o+(0.0532144836415)*x[0]+(0.155895442381)*x[1]**o+(-0.872222127033)*x[1]+(0.810538165596)*x[2]**o+(-0.917071363009)*x[2]
        arg[2,4]=(0.882674783877)*x[0]**o+(-0.246439463648)*x[0]+(-0.478821457401)*x[1]**o+(-0.629183697296)*x[1]+(0.833283185393)*x[2]**o+(-0.895796651184)*x[2]
        arg[3,0]=(0.949019461395)*x[0]**o+(0.437511668107)*x[0]+(0.46679355983)*x[1]**o+(0.375391913775)*x[1]+(0.992336073925)*x[2]**o+(0.784847856608)*x[2]
        arg[3,1]=(0.10730061928)*x[0]**o+(0.624678931808)*x[0]+(-0.105084917663)*x[1]**o+(-0.0650034223387)*x[1]+(-0.271922353868)*x[2]**o+(-0.182967296038)*x[2]
        arg[3,2]=(0.115856234055)*x[0]**o+(0.711912354922)*x[0]+(-0.317962972122)*x[1]**o+(0.577856092951)*x[1]+(-0.246833922867)*x[2]**o+(0.354116462218)*x[2]
        arg[3,3]=(0.40937236699)*x[0]**o+(0.639252679403)*x[0]+(0.540206943219)*x[1]**o+(0.287006139758)*x[1]+(0.123311433598)*x[2]**o+(-0.0300672780213)*x[2]
        arg[3,4]=(-0.190299257713)*x[0]**o+(-0.022055675261)*x[0]+(-0.640751994447)*x[1]**o+(-0.411801947317)*x[1]+(0.0675008125057)*x[2]**o+(-0.659424255203)*x[2]
        ref[0,0]=(0.392450639899)*x_ref[0]**o+(-0.685488230002)*x_ref[0]+(-0.41545928428)*x_ref[1]**o+(0.244144392379)*x_ref[1]+(0.602642156528)*x_ref[2]**o+(-0.348717378461)*x_ref[2]
        ref[0,1]=(-0.341243165541)*x_ref[0]**o+(-0.616870164204)*x_ref[0]+(-0.510613066362)*x_ref[1]**o+(-0.81387010212)*x_ref[1]+(0.856239691119)*x_ref[2]**o+(0.0744651854239)*x_ref[2]
        ref[0,2]=(-0.765983413604)*x_ref[0]**o+(0.474551394174)*x_ref[0]+(-0.939524929765)*x_ref[1]**o+(0.418297898065)*x_ref[1]+(0.00589909539531)*x_ref[2]**o+(0.0889012994897)*x_ref[2]
        ref[0,3]=(-0.521217563352)*x_ref[0]**o+(-0.256316459647)*x_ref[0]+(-0.0265965098379)*x_ref[1]**o+(-0.597562278966)*x_ref[1]+(0.0389151661339)*x_ref[2]**o+(-0.572109451352)*x_ref[2]
        ref[0,4]=(-0.853801268491)*x_ref[0]**o+(-0.691274812057)*x_ref[0]+(-0.682435618581)*x_ref[1]**o+(-0.974334375359)*x_ref[1]+(-0.642333316794)*x_ref[2]**o+(0.353816751252)*x_ref[2]
        ref[1,0]=(-0.657545394322)*x_ref[0]**o+(-0.334172338096)*x_ref[0]+(0.016748279459)*x_ref[1]**o+(-0.294858621125)*x_ref[1]+(-0.49403880046)*x_ref[2]**o+(-0.152416774822)*x_ref[2]
        ref[1,1]=(-0.602731976042)*x_ref[0]**o+(0.685046816545)*x_ref[0]+(0.750001644716)*x_ref[1]**o+(-0.202076972289)*x_ref[1]+(-0.0232888078434)*x_ref[2]**o+(-0.0774477649878)*x_ref[2]
        ref[1,2]=(-0.707321785527)*x_ref[0]**o+(0.618526464205)*x_ref[0]+(-0.578548442468)*x_ref[1]**o+(-0.411374744442)*x_ref[1]+(-0.944277708787)*x_ref[2]**o+(0.647856975707)*x_ref[2]
        ref[1,3]=(-0.222503271022)*x_ref[0]**o+(-0.929428092728)*x_ref[0]+(-0.976553212424)*x_ref[1]**o+(0.0376439617084)*x_ref[1]+(0.0428078568512)*x_ref[2]**o+(-0.441850944272)*x_ref[2]
        ref[1,4]=(-0.862164898455)*x_ref[0]**o+(-0.989900221991)*x_ref[0]+(0.110610844567)*x_ref[1]**o+(0.419384612236)*x_ref[1]+(-0.00271948841262)*x_ref[2]**o+(0.301635101629)*x_ref[2]
        ref[2,0]=(-0.0950421881942)*x_ref[0]**o+(-0.741293053909)*x_ref[0]+(-0.461538879874)*x_ref[1]**o+(-0.8199842553)*x_ref[1]+(0.0590344916829)*x_ref[2]**o+(0.352541518434)*x_ref[2]
        ref[2,1]=(-0.734407995647)*x_ref[0]**o+(-0.726167721992)*x_ref[0]+(0.700521229916)*x_ref[1]**o+(0.753555863428)*x_ref[1]+(-0.891638139696)*x_ref[2]**o+(-0.996550711213)*x_ref[2]
        ref[2,2]=(0.777110352629)*x_ref[0]**o+(0.309719201157)*x_ref[0]+(0.026816547522)*x_ref[1]**o+(0.33161010287)*x_ref[1]+(0.257929504352)*x_ref[2]**o+(0.71871314654)*x_ref[2]
        ref[2,3]=(-0.0982522143825)*x_ref[0]**o+(0.0532144836415)*x_ref[0]+(0.155895442381)*x_ref[1]**o+(-0.872222127033)*x_ref[1]+(0.810538165596)*x_ref[2]**o+(-0.917071363009)*x_ref[2]
        ref[2,4]=(0.882674783877)*x_ref[0]**o+(-0.246439463648)*x_ref[0]+(-0.478821457401)*x_ref[1]**o+(-0.629183697296)*x_ref[1]+(0.833283185393)*x_ref[2]**o+(-0.895796651184)*x_ref[2]
        ref[3,0]=(0.949019461395)*x_ref[0]**o+(0.437511668107)*x_ref[0]+(0.46679355983)*x_ref[1]**o+(0.375391913775)*x_ref[1]+(0.992336073925)*x_ref[2]**o+(0.784847856608)*x_ref[2]
        ref[3,1]=(0.10730061928)*x_ref[0]**o+(0.624678931808)*x_ref[0]+(-0.105084917663)*x_ref[1]**o+(-0.0650034223387)*x_ref[1]+(-0.271922353868)*x_ref[2]**o+(-0.182967296038)*x_ref[2]
        ref[3,2]=(0.115856234055)*x_ref[0]**o+(0.711912354922)*x_ref[0]+(-0.317962972122)*x_ref[1]**o+(0.577856092951)*x_ref[1]+(-0.246833922867)*x_ref[2]**o+(0.354116462218)*x_ref[2]
        ref[3,3]=(0.40937236699)*x_ref[0]**o+(0.639252679403)*x_ref[0]+(0.540206943219)*x_ref[1]**o+(0.287006139758)*x_ref[1]+(0.123311433598)*x_ref[2]**o+(-0.0300672780213)*x_ref[2]
        ref[3,4]=(-0.190299257713)*x_ref[0]**o+(-0.022055675261)*x_ref[0]+(-0.640751994447)*x_ref[1]**o+(-0.411801947317)*x_ref[1]+(0.0675008125057)*x_ref[2]**o+(-0.659424255203)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_ContinuousFunction_rank3(self):
      """
      tests interpolate for rank 3 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ContinuousFunction onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2),w_ref)
      if dim==2:
        arg[0,0,0]=(-0.697323110242)*x[0]**o+(-0.806494185218)*x[0]+(0.387914602484)*x[1]**o+(0.841835487965)*x[1]
        arg[0,0,1]=(-0.77670653908)*x[0]**o+(0.311710212706)*x[0]+(-0.0951864020181)*x[1]**o+(-0.14909139946)*x[1]
        arg[0,1,0]=(0.80933791543)*x[0]**o+(0.72441968276)*x[0]+(-0.446054850085)*x[1]**o+(0.419303534988)*x[1]
        arg[0,1,1]=(0.174464190017)*x[0]**o+(0.330609661878)*x[0]+(0.798365390058)*x[1]**o+(0.354899223817)*x[1]
        arg[1,0,0]=(0.36073672839)*x[0]**o+(0.831900846214)*x[0]+(-0.656354645044)*x[1]**o+(0.200645044646)*x[1]
        arg[1,0,1]=(0.817889801065)*x[0]**o+(0.34052889294)*x[0]+(-0.2416944924)*x[1]**o+(0.0922342148934)*x[1]
        arg[1,1,0]=(-0.962116509558)*x[0]**o+(-0.0785232077542)*x[0]+(-0.981399563356)*x[1]**o+(0.785509095614)*x[1]
        arg[1,1,1]=(-0.688444030508)*x[0]**o+(0.651617638972)*x[0]+(0.160584046401)*x[1]**o+(0.253258991923)*x[1]
        arg[2,0,0]=(0.0728970412574)*x[0]**o+(-0.647328240217)*x[0]+(0.517665494535)*x[1]**o+(-0.416383753573)*x[1]
        arg[2,0,1]=(0.148104739733)*x[0]**o+(-0.378371639837)*x[0]+(0.848608631336)*x[1]**o+(-0.881516167852)*x[1]
        arg[2,1,0]=(-0.506376676256)*x[0]**o+(0.234090340446)*x[0]+(-0.737828959566)*x[1]**o+(-0.74093763135)*x[1]
        arg[2,1,1]=(0.259605354135)*x[0]**o+(0.537424356513)*x[0]+(-0.405718119693)*x[1]**o+(0.120663712044)*x[1]
        arg[3,0,0]=(-0.609659975269)*x[0]**o+(-0.455923293874)*x[0]+(-0.00765676965971)*x[1]**o+(-0.710899349259)*x[1]
        arg[3,0,1]=(-0.731011612904)*x[0]**o+(-0.355060765358)*x[0]+(-0.127369372538)*x[1]**o+(0.694265371908)*x[1]
        arg[3,1,0]=(0.188766337132)*x[0]**o+(-0.194878304215)*x[0]+(-0.963356218109)*x[1]**o+(0.327053372293)*x[1]
        arg[3,1,1]=(-0.566712779324)*x[0]**o+(-0.251473305071)*x[0]+(0.0181297146138)*x[1]**o+(0.257664697754)*x[1]
        arg[4,0,0]=(0.574100764569)*x[0]**o+(-0.717776999574)*x[0]+(0.880998124155)*x[1]**o+(-0.810615928848)*x[1]
        arg[4,0,1]=(0.42552502518)*x[0]**o+(0.456751341147)*x[0]+(-0.484998619005)*x[1]**o+(0.932151187696)*x[1]
        arg[4,1,0]=(0.56865980485)*x[0]**o+(0.593928872734)*x[0]+(-0.475639894836)*x[1]**o+(-0.990718402693)*x[1]
        arg[4,1,1]=(0.129586342261)*x[0]**o+(-0.408951830619)*x[0]+(0.132518475289)*x[1]**o+(0.708612820692)*x[1]
        arg[5,0,0]=(0.0708038739082)*x[0]**o+(0.524270558649)*x[0]+(0.368504533482)*x[1]**o+(0.935828810393)*x[1]
        arg[5,0,1]=(-0.383650306829)*x[0]**o+(-0.0439362943961)*x[0]+(0.80939275198)*x[1]**o+(-0.61335508023)*x[1]
        arg[5,1,0]=(-0.621571339247)*x[0]**o+(-0.185069293199)*x[0]+(0.368472011431)*x[1]**o+(-0.388752495932)*x[1]
        arg[5,1,1]=(-0.263310763174)*x[0]**o+(-0.694616630015)*x[0]+(0.271238274203)*x[1]**o+(-0.477861402347)*x[1]
        ref[0,0,0]=(-0.697323110242)*x_ref[0]**o+(-0.806494185218)*x_ref[0]+(0.387914602484)*x_ref[1]**o+(0.841835487965)*x_ref[1]
        ref[0,0,1]=(-0.77670653908)*x_ref[0]**o+(0.311710212706)*x_ref[0]+(-0.0951864020181)*x_ref[1]**o+(-0.14909139946)*x_ref[1]
        ref[0,1,0]=(0.80933791543)*x_ref[0]**o+(0.72441968276)*x_ref[0]+(-0.446054850085)*x_ref[1]**o+(0.419303534988)*x_ref[1]
        ref[0,1,1]=(0.174464190017)*x_ref[0]**o+(0.330609661878)*x_ref[0]+(0.798365390058)*x_ref[1]**o+(0.354899223817)*x_ref[1]
        ref[1,0,0]=(0.36073672839)*x_ref[0]**o+(0.831900846214)*x_ref[0]+(-0.656354645044)*x_ref[1]**o+(0.200645044646)*x_ref[1]
        ref[1,0,1]=(0.817889801065)*x_ref[0]**o+(0.34052889294)*x_ref[0]+(-0.2416944924)*x_ref[1]**o+(0.0922342148934)*x_ref[1]
        ref[1,1,0]=(-0.962116509558)*x_ref[0]**o+(-0.0785232077542)*x_ref[0]+(-0.981399563356)*x_ref[1]**o+(0.785509095614)*x_ref[1]
        ref[1,1,1]=(-0.688444030508)*x_ref[0]**o+(0.651617638972)*x_ref[0]+(0.160584046401)*x_ref[1]**o+(0.253258991923)*x_ref[1]
        ref[2,0,0]=(0.0728970412574)*x_ref[0]**o+(-0.647328240217)*x_ref[0]+(0.517665494535)*x_ref[1]**o+(-0.416383753573)*x_ref[1]
        ref[2,0,1]=(0.148104739733)*x_ref[0]**o+(-0.378371639837)*x_ref[0]+(0.848608631336)*x_ref[1]**o+(-0.881516167852)*x_ref[1]
        ref[2,1,0]=(-0.506376676256)*x_ref[0]**o+(0.234090340446)*x_ref[0]+(-0.737828959566)*x_ref[1]**o+(-0.74093763135)*x_ref[1]
        ref[2,1,1]=(0.259605354135)*x_ref[0]**o+(0.537424356513)*x_ref[0]+(-0.405718119693)*x_ref[1]**o+(0.120663712044)*x_ref[1]
        ref[3,0,0]=(-0.609659975269)*x_ref[0]**o+(-0.455923293874)*x_ref[0]+(-0.00765676965971)*x_ref[1]**o+(-0.710899349259)*x_ref[1]
        ref[3,0,1]=(-0.731011612904)*x_ref[0]**o+(-0.355060765358)*x_ref[0]+(-0.127369372538)*x_ref[1]**o+(0.694265371908)*x_ref[1]
        ref[3,1,0]=(0.188766337132)*x_ref[0]**o+(-0.194878304215)*x_ref[0]+(-0.963356218109)*x_ref[1]**o+(0.327053372293)*x_ref[1]
        ref[3,1,1]=(-0.566712779324)*x_ref[0]**o+(-0.251473305071)*x_ref[0]+(0.0181297146138)*x_ref[1]**o+(0.257664697754)*x_ref[1]
        ref[4,0,0]=(0.574100764569)*x_ref[0]**o+(-0.717776999574)*x_ref[0]+(0.880998124155)*x_ref[1]**o+(-0.810615928848)*x_ref[1]
        ref[4,0,1]=(0.42552502518)*x_ref[0]**o+(0.456751341147)*x_ref[0]+(-0.484998619005)*x_ref[1]**o+(0.932151187696)*x_ref[1]
        ref[4,1,0]=(0.56865980485)*x_ref[0]**o+(0.593928872734)*x_ref[0]+(-0.475639894836)*x_ref[1]**o+(-0.990718402693)*x_ref[1]
        ref[4,1,1]=(0.129586342261)*x_ref[0]**o+(-0.408951830619)*x_ref[0]+(0.132518475289)*x_ref[1]**o+(0.708612820692)*x_ref[1]
        ref[5,0,0]=(0.0708038739082)*x_ref[0]**o+(0.524270558649)*x_ref[0]+(0.368504533482)*x_ref[1]**o+(0.935828810393)*x_ref[1]
        ref[5,0,1]=(-0.383650306829)*x_ref[0]**o+(-0.0439362943961)*x_ref[0]+(0.80939275198)*x_ref[1]**o+(-0.61335508023)*x_ref[1]
        ref[5,1,0]=(-0.621571339247)*x_ref[0]**o+(-0.185069293199)*x_ref[0]+(0.368472011431)*x_ref[1]**o+(-0.388752495932)*x_ref[1]
        ref[5,1,1]=(-0.263310763174)*x_ref[0]**o+(-0.694616630015)*x_ref[0]+(0.271238274203)*x_ref[1]**o+(-0.477861402347)*x_ref[1]
      else:
        arg[0,0,0]=(0.287534645778)*x[0]**o+(0.94865229379)*x[0]+(0.655299232864)*x[1]**o+(0.350034924637)*x[1]+(0.556341391183)*x[2]**o+(0.840643265765)*x[2]
        arg[0,0,1]=(-0.207213781549)*x[0]**o+(-0.0394497047797)*x[0]+(-0.0236016308575)*x[1]**o+(0.344122299199)*x[1]+(0.389588325996)*x[2]**o+(0.941980468657)*x[2]
        arg[0,1,0]=(0.00980412235862)*x[0]**o+(0.74862549447)*x[0]+(0.630165284383)*x[1]**o+(0.234711268989)*x[1]+(-0.22509981272)*x[2]**o+(-0.0153888168649)*x[2]
        arg[0,1,1]=(-0.87309186262)*x[0]**o+(0.833526923917)*x[0]+(0.78785479415)*x[1]**o+(0.823967259171)*x[1]+(0.320244316109)*x[2]**o+(-0.735225854417)*x[2]
        arg[1,0,0]=(-0.0892232551051)*x[0]**o+(-0.346728833563)*x[0]+(0.422774634671)*x[1]**o+(0.793222381957)*x[1]+(-0.670476677006)*x[2]**o+(0.896884540116)*x[2]
        arg[1,0,1]=(-0.0023382136345)*x[0]**o+(0.73058281545)*x[0]+(0.11828880374)*x[1]**o+(-0.691839091619)*x[1]+(-0.39086463392)*x[2]**o+(0.66200209782)*x[2]
        arg[1,1,0]=(-0.924238055558)*x[0]**o+(-0.334208271705)*x[0]+(-0.57106045555)*x[1]**o+(-0.765650999193)*x[1]+(0.0404533468054)*x[2]**o+(-0.225975260777)*x[2]
        arg[1,1,1]=(-0.351110999108)*x[0]**o+(0.626166346251)*x[0]+(0.385737198467)*x[1]**o+(0.577561466908)*x[1]+(0.708325217242)*x[2]**o+(0.193727923904)*x[2]
        arg[2,0,0]=(0.278942080997)*x[0]**o+(-0.569761653572)*x[0]+(-0.624879270918)*x[1]**o+(-0.0151754112896)*x[1]+(0.411399083825)*x[2]**o+(-0.846669568641)*x[2]
        arg[2,0,1]=(-0.543586963879)*x[0]**o+(-0.383959956625)*x[0]+(0.184375944938)*x[1]**o+(-0.104178233562)*x[1]+(-0.223101894662)*x[2]**o+(-0.17221835595)*x[2]
        arg[2,1,0]=(0.317103921207)*x[0]**o+(-0.919542461354)*x[0]+(0.56666554454)*x[1]**o+(0.895772298847)*x[1]+(0.714550707822)*x[2]**o+(0.634702089556)*x[2]
        arg[2,1,1]=(0.986576254164)*x[0]**o+(0.544493787633)*x[0]+(0.964620027203)*x[1]**o+(0.259991384678)*x[1]+(-0.577361835739)*x[2]**o+(0.651593456822)*x[2]
        arg[3,0,0]=(0.362841472614)*x[0]**o+(-0.521771228041)*x[0]+(-0.594530755517)*x[1]**o+(0.700647681241)*x[1]+(-0.131683781311)*x[2]**o+(0.636310189834)*x[2]
        arg[3,0,1]=(0.885891862321)*x[0]**o+(0.397438320764)*x[0]+(0.711380452937)*x[1]**o+(0.962571325144)*x[1]+(-0.300114113339)*x[2]**o+(0.34700388868)*x[2]
        arg[3,1,0]=(0.52140726408)*x[0]**o+(-0.0921295295522)*x[0]+(-0.823076091446)*x[1]**o+(0.477401876125)*x[1]+(-0.973057905067)*x[2]**o+(-0.975526794128)*x[2]
        arg[3,1,1]=(-0.241503887113)*x[0]**o+(0.133988088285)*x[0]+(0.243026393121)*x[1]**o+(0.0848716779473)*x[1]+(0.140349614791)*x[2]**o+(-0.217182025695)*x[2]
        arg[4,0,0]=(0.680080194053)*x[0]**o+(0.512009633844)*x[0]+(-0.640214066363)*x[1]**o+(0.571800633214)*x[1]+(-0.749395617545)*x[2]**o+(0.448487668786)*x[2]
        arg[4,0,1]=(-0.792959916349)*x[0]**o+(-0.433770516776)*x[0]+(-0.380980758215)*x[1]**o+(-0.831154769101)*x[1]+(0.277551460651)*x[2]**o+(0.626476984233)*x[2]
        arg[4,1,0]=(-0.285102566546)*x[0]**o+(0.745636275253)*x[0]+(0.4999427836)*x[1]**o+(0.561343302884)*x[1]+(0.796220302361)*x[2]**o+(-0.830020520143)*x[2]
        arg[4,1,1]=(0.237420507314)*x[0]**o+(0.327181858092)*x[0]+(-0.895743235593)*x[1]**o+(-0.496964477443)*x[1]+(0.57336428204)*x[2]**o+(-0.154267071287)*x[2]
        arg[5,0,0]=(0.0401925997607)*x[0]**o+(-0.0155957424748)*x[0]+(-0.658363451239)*x[1]**o+(-0.175223211879)*x[1]+(-0.743729768327)*x[2]**o+(0.391531327128)*x[2]
        arg[5,0,1]=(0.836239801824)*x[0]**o+(-0.72288085675)*x[0]+(0.177981042887)*x[1]**o+(0.354505260098)*x[1]+(-0.180520504267)*x[2]**o+(-0.380306239974)*x[2]
        arg[5,1,0]=(-0.65536853024)*x[0]**o+(-0.00573854242476)*x[0]+(-0.261460514314)*x[1]**o+(-0.289771405693)*x[1]+(0.750775397523)*x[2]**o+(-0.333215608562)*x[2]
        arg[5,1,1]=(-0.579532884024)*x[0]**o+(-0.208677373724)*x[0]+(-0.379612024024)*x[1]**o+(-0.0565813995114)*x[1]+(-0.867913562592)*x[2]**o+(-0.254654298293)*x[2]
        ref[0,0,0]=(0.287534645778)*x_ref[0]**o+(0.94865229379)*x_ref[0]+(0.655299232864)*x_ref[1]**o+(0.350034924637)*x_ref[1]+(0.556341391183)*x_ref[2]**o+(0.840643265765)*x_ref[2]
        ref[0,0,1]=(-0.207213781549)*x_ref[0]**o+(-0.0394497047797)*x_ref[0]+(-0.0236016308575)*x_ref[1]**o+(0.344122299199)*x_ref[1]+(0.389588325996)*x_ref[2]**o+(0.941980468657)*x_ref[2]
        ref[0,1,0]=(0.00980412235862)*x_ref[0]**o+(0.74862549447)*x_ref[0]+(0.630165284383)*x_ref[1]**o+(0.234711268989)*x_ref[1]+(-0.22509981272)*x_ref[2]**o+(-0.0153888168649)*x_ref[2]
        ref[0,1,1]=(-0.87309186262)*x_ref[0]**o+(0.833526923917)*x_ref[0]+(0.78785479415)*x_ref[1]**o+(0.823967259171)*x_ref[1]+(0.320244316109)*x_ref[2]**o+(-0.735225854417)*x_ref[2]
        ref[1,0,0]=(-0.0892232551051)*x_ref[0]**o+(-0.346728833563)*x_ref[0]+(0.422774634671)*x_ref[1]**o+(0.793222381957)*x_ref[1]+(-0.670476677006)*x_ref[2]**o+(0.896884540116)*x_ref[2]
        ref[1,0,1]=(-0.0023382136345)*x_ref[0]**o+(0.73058281545)*x_ref[0]+(0.11828880374)*x_ref[1]**o+(-0.691839091619)*x_ref[1]+(-0.39086463392)*x_ref[2]**o+(0.66200209782)*x_ref[2]
        ref[1,1,0]=(-0.924238055558)*x_ref[0]**o+(-0.334208271705)*x_ref[0]+(-0.57106045555)*x_ref[1]**o+(-0.765650999193)*x_ref[1]+(0.0404533468054)*x_ref[2]**o+(-0.225975260777)*x_ref[2]
        ref[1,1,1]=(-0.351110999108)*x_ref[0]**o+(0.626166346251)*x_ref[0]+(0.385737198467)*x_ref[1]**o+(0.577561466908)*x_ref[1]+(0.708325217242)*x_ref[2]**o+(0.193727923904)*x_ref[2]
        ref[2,0,0]=(0.278942080997)*x_ref[0]**o+(-0.569761653572)*x_ref[0]+(-0.624879270918)*x_ref[1]**o+(-0.0151754112896)*x_ref[1]+(0.411399083825)*x_ref[2]**o+(-0.846669568641)*x_ref[2]
        ref[2,0,1]=(-0.543586963879)*x_ref[0]**o+(-0.383959956625)*x_ref[0]+(0.184375944938)*x_ref[1]**o+(-0.104178233562)*x_ref[1]+(-0.223101894662)*x_ref[2]**o+(-0.17221835595)*x_ref[2]
        ref[2,1,0]=(0.317103921207)*x_ref[0]**o+(-0.919542461354)*x_ref[0]+(0.56666554454)*x_ref[1]**o+(0.895772298847)*x_ref[1]+(0.714550707822)*x_ref[2]**o+(0.634702089556)*x_ref[2]
        ref[2,1,1]=(0.986576254164)*x_ref[0]**o+(0.544493787633)*x_ref[0]+(0.964620027203)*x_ref[1]**o+(0.259991384678)*x_ref[1]+(-0.577361835739)*x_ref[2]**o+(0.651593456822)*x_ref[2]
        ref[3,0,0]=(0.362841472614)*x_ref[0]**o+(-0.521771228041)*x_ref[0]+(-0.594530755517)*x_ref[1]**o+(0.700647681241)*x_ref[1]+(-0.131683781311)*x_ref[2]**o+(0.636310189834)*x_ref[2]
        ref[3,0,1]=(0.885891862321)*x_ref[0]**o+(0.397438320764)*x_ref[0]+(0.711380452937)*x_ref[1]**o+(0.962571325144)*x_ref[1]+(-0.300114113339)*x_ref[2]**o+(0.34700388868)*x_ref[2]
        ref[3,1,0]=(0.52140726408)*x_ref[0]**o+(-0.0921295295522)*x_ref[0]+(-0.823076091446)*x_ref[1]**o+(0.477401876125)*x_ref[1]+(-0.973057905067)*x_ref[2]**o+(-0.975526794128)*x_ref[2]
        ref[3,1,1]=(-0.241503887113)*x_ref[0]**o+(0.133988088285)*x_ref[0]+(0.243026393121)*x_ref[1]**o+(0.0848716779473)*x_ref[1]+(0.140349614791)*x_ref[2]**o+(-0.217182025695)*x_ref[2]
        ref[4,0,0]=(0.680080194053)*x_ref[0]**o+(0.512009633844)*x_ref[0]+(-0.640214066363)*x_ref[1]**o+(0.571800633214)*x_ref[1]+(-0.749395617545)*x_ref[2]**o+(0.448487668786)*x_ref[2]
        ref[4,0,1]=(-0.792959916349)*x_ref[0]**o+(-0.433770516776)*x_ref[0]+(-0.380980758215)*x_ref[1]**o+(-0.831154769101)*x_ref[1]+(0.277551460651)*x_ref[2]**o+(0.626476984233)*x_ref[2]
        ref[4,1,0]=(-0.285102566546)*x_ref[0]**o+(0.745636275253)*x_ref[0]+(0.4999427836)*x_ref[1]**o+(0.561343302884)*x_ref[1]+(0.796220302361)*x_ref[2]**o+(-0.830020520143)*x_ref[2]
        ref[4,1,1]=(0.237420507314)*x_ref[0]**o+(0.327181858092)*x_ref[0]+(-0.895743235593)*x_ref[1]**o+(-0.496964477443)*x_ref[1]+(0.57336428204)*x_ref[2]**o+(-0.154267071287)*x_ref[2]
        ref[5,0,0]=(0.0401925997607)*x_ref[0]**o+(-0.0155957424748)*x_ref[0]+(-0.658363451239)*x_ref[1]**o+(-0.175223211879)*x_ref[1]+(-0.743729768327)*x_ref[2]**o+(0.391531327128)*x_ref[2]
        ref[5,0,1]=(0.836239801824)*x_ref[0]**o+(-0.72288085675)*x_ref[0]+(0.177981042887)*x_ref[1]**o+(0.354505260098)*x_ref[1]+(-0.180520504267)*x_ref[2]**o+(-0.380306239974)*x_ref[2]
        ref[5,1,0]=(-0.65536853024)*x_ref[0]**o+(-0.00573854242476)*x_ref[0]+(-0.261460514314)*x_ref[1]**o+(-0.289771405693)*x_ref[1]+(0.750775397523)*x_ref[2]**o+(-0.333215608562)*x_ref[2]
        ref[5,1,1]=(-0.579532884024)*x_ref[0]**o+(-0.208677373724)*x_ref[0]+(-0.379612024024)*x_ref[1]**o+(-0.0565813995114)*x_ref[1]+(-0.867913562592)*x_ref[2]**o+(-0.254654298293)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_ContinuousFunction_rank4(self):
      """
      tests interpolate for rank 4 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ContinuousFunction onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5, 3, 2),w)
      ref=Data(0,(4, 5, 3, 2),w_ref)
      if dim==2:
        arg[0,0,0,0]=(-0.274472293956)*x[0]**o+(-0.293347130929)*x[0]+(0.392586448254)*x[1]**o+(-0.403066854054)*x[1]
        arg[0,0,0,1]=(0.385933192907)*x[0]**o+(0.484240065867)*x[0]+(0.144565631551)*x[1]**o+(0.882508492739)*x[1]
        arg[0,0,1,0]=(0.993368905137)*x[0]**o+(0.340349919343)*x[0]+(-0.956806746524)*x[1]**o+(0.990607958001)*x[1]
        arg[0,0,1,1]=(-0.88610250102)*x[0]**o+(-0.543705587259)*x[0]+(0.170798164924)*x[1]**o+(0.685331195754)*x[1]
        arg[0,0,2,0]=(-0.922211611143)*x[0]**o+(-0.103640037893)*x[0]+(0.401354703434)*x[1]**o+(-0.117826261635)*x[1]
        arg[0,0,2,1]=(-0.391601387132)*x[0]**o+(-0.472753344804)*x[0]+(-0.473433838993)*x[1]**o+(-0.173501635989)*x[1]
        arg[0,1,0,0]=(0.747353274916)*x[0]**o+(-0.960856612086)*x[0]+(-0.54809012978)*x[1]**o+(0.862023893332)*x[1]
        arg[0,1,0,1]=(-0.256359001149)*x[0]**o+(-0.868975044191)*x[0]+(0.494314269744)*x[1]**o+(0.949957481172)*x[1]
        arg[0,1,1,0]=(-0.0256658750601)*x[0]**o+(0.145499104203)*x[0]+(-0.68669791629)*x[1]**o+(0.447326207295)*x[1]
        arg[0,1,1,1]=(0.150617284467)*x[0]**o+(-0.350046088196)*x[0]+(-0.367671302053)*x[1]**o+(0.633220122068)*x[1]
        arg[0,1,2,0]=(-0.714145362271)*x[0]**o+(-0.957661490758)*x[0]+(0.386253770747)*x[1]**o+(0.982634262634)*x[1]
        arg[0,1,2,1]=(0.707565096824)*x[0]**o+(0.281758058573)*x[0]+(-0.407791315587)*x[1]**o+(0.86069036983)*x[1]
        arg[0,2,0,0]=(-0.453860220598)*x[0]**o+(0.944457388061)*x[0]+(-0.903818960162)*x[1]**o+(-0.799423205297)*x[1]
        arg[0,2,0,1]=(-0.621257500941)*x[0]**o+(0.748963737937)*x[0]+(-0.12660934458)*x[1]**o+(-0.362830639592)*x[1]
        arg[0,2,1,0]=(-0.702485820037)*x[0]**o+(0.355584898593)*x[0]+(0.657127357751)*x[1]**o+(0.535533390762)*x[1]
        arg[0,2,1,1]=(0.795733125008)*x[0]**o+(0.983607327249)*x[0]+(-0.463818780644)*x[1]**o+(-0.681831588244)*x[1]
        arg[0,2,2,0]=(0.335783204138)*x[0]**o+(-0.712225782679)*x[0]+(0.853426644096)*x[1]**o+(-0.143845263686)*x[1]
        arg[0,2,2,1]=(0.389724375144)*x[0]**o+(0.715176227705)*x[0]+(-0.660690893888)*x[1]**o+(0.428289724709)*x[1]
        arg[0,3,0,0]=(-0.363836203724)*x[0]**o+(-0.257775427457)*x[0]+(0.407904889098)*x[1]**o+(-0.731273814284)*x[1]
        arg[0,3,0,1]=(0.402038262494)*x[0]**o+(0.33289103638)*x[0]+(0.444957562329)*x[1]**o+(0.0657971935664)*x[1]
        arg[0,3,1,0]=(0.979673814602)*x[0]**o+(-0.133405332848)*x[0]+(0.23884689304)*x[1]**o+(0.728464997681)*x[1]
        arg[0,3,1,1]=(0.724603429246)*x[0]**o+(-0.391040898144)*x[0]+(-0.966816499003)*x[1]**o+(0.0984302578905)*x[1]
        arg[0,3,2,0]=(-0.986139645754)*x[0]**o+(0.60807351165)*x[0]+(0.799807413231)*x[1]**o+(0.12091479711)*x[1]
        arg[0,3,2,1]=(0.387453999377)*x[0]**o+(0.212725006628)*x[0]+(-0.120374039139)*x[1]**o+(-0.262304757951)*x[1]
        arg[0,4,0,0]=(-0.260545986276)*x[0]**o+(0.251085312932)*x[0]+(0.738082708518)*x[1]**o+(-0.0804883797938)*x[1]
        arg[0,4,0,1]=(-0.190701356138)*x[0]**o+(-0.0205295935522)*x[0]+(0.904261681819)*x[1]**o+(-0.859647709957)*x[1]
        arg[0,4,1,0]=(-0.945357492849)*x[0]**o+(-0.85298463389)*x[0]+(0.688964336673)*x[1]**o+(-0.672881696932)*x[1]
        arg[0,4,1,1]=(0.877720258045)*x[0]**o+(-0.361228026805)*x[0]+(-0.326626527006)*x[1]**o+(-0.274100421232)*x[1]
        arg[0,4,2,0]=(0.202358729084)*x[0]**o+(-0.997571481837)*x[0]+(0.431193711862)*x[1]**o+(0.513561123915)*x[1]
        arg[0,4,2,1]=(-0.136602954592)*x[0]**o+(-0.40864934437)*x[0]+(0.675945574351)*x[1]**o+(0.728757793536)*x[1]
        arg[1,0,0,0]=(0.835464261851)*x[0]**o+(-0.131447020601)*x[0]+(0.0810352521446)*x[1]**o+(0.664744750899)*x[1]
        arg[1,0,0,1]=(0.239716928689)*x[0]**o+(-0.192258999531)*x[0]+(0.615877054036)*x[1]**o+(0.835274774437)*x[1]
        arg[1,0,1,0]=(0.509342803767)*x[0]**o+(-0.633944481882)*x[0]+(0.262098006086)*x[1]**o+(0.159902070799)*x[1]
        arg[1,0,1,1]=(0.894432440588)*x[0]**o+(-0.167778431319)*x[0]+(-0.984183148477)*x[1]**o+(-0.554981658632)*x[1]
        arg[1,0,2,0]=(-0.762472607231)*x[0]**o+(0.405550139256)*x[0]+(0.4051290641)*x[1]**o+(-0.27180186672)*x[1]
        arg[1,0,2,1]=(0.663869273816)*x[0]**o+(0.867453974628)*x[0]+(-0.279659969132)*x[1]**o+(-0.370539101204)*x[1]
        arg[1,1,0,0]=(-0.969326778164)*x[0]**o+(0.965471349497)*x[0]+(-0.529983232946)*x[1]**o+(0.714091610977)*x[1]
        arg[1,1,0,1]=(-0.840360667093)*x[0]**o+(-0.399319808256)*x[0]+(0.568769915582)*x[1]**o+(0.872427494442)*x[1]
        arg[1,1,1,0]=(0.46099116941)*x[0]**o+(-0.986008874142)*x[0]+(-0.53999023556)*x[1]**o+(-0.549695567855)*x[1]
        arg[1,1,1,1]=(0.485944918559)*x[0]**o+(0.189773828651)*x[0]+(-0.611654980493)*x[1]**o+(-0.337071911657)*x[1]
        arg[1,1,2,0]=(0.191663603248)*x[0]**o+(0.0906374965004)*x[0]+(-0.271051598251)*x[1]**o+(0.880507788825)*x[1]
        arg[1,1,2,1]=(-0.439514997001)*x[0]**o+(0.701230426224)*x[0]+(-0.193231641029)*x[1]**o+(0.950584054332)*x[1]
        arg[1,2,0,0]=(0.439456565259)*x[0]**o+(-0.25664171484)*x[0]+(-0.653059819329)*x[1]**o+(-0.680544747777)*x[1]
        arg[1,2,0,1]=(0.34890454443)*x[0]**o+(-0.69851200963)*x[0]+(-0.00159381688549)*x[1]**o+(-0.612359675499)*x[1]
        arg[1,2,1,0]=(0.132759671375)*x[0]**o+(-0.235126716401)*x[0]+(0.652068885826)*x[1]**o+(0.749698645371)*x[1]
        arg[1,2,1,1]=(0.934980403627)*x[0]**o+(0.321409273437)*x[0]+(-0.605909929553)*x[1]**o+(0.135554927418)*x[1]
        arg[1,2,2,0]=(-0.68327709329)*x[0]**o+(-0.0381292756005)*x[0]+(0.710953191114)*x[1]**o+(-0.344522256689)*x[1]
        arg[1,2,2,1]=(-0.813976235652)*x[0]**o+(0.613879708123)*x[0]+(0.451599277293)*x[1]**o+(-0.6134026375)*x[1]
        arg[1,3,0,0]=(0.595310128627)*x[0]**o+(-0.247621573131)*x[0]+(0.165607783104)*x[1]**o+(0.578132180805)*x[1]
        arg[1,3,0,1]=(0.848827238882)*x[0]**o+(0.173154067999)*x[0]+(0.340662419437)*x[1]**o+(-0.389656447251)*x[1]
        arg[1,3,1,0]=(0.250888415228)*x[0]**o+(-0.278203136198)*x[0]+(-0.693211186734)*x[1]**o+(0.613812194735)*x[1]
        arg[1,3,1,1]=(0.634533003162)*x[0]**o+(-0.0924665762471)*x[0]+(0.749575894384)*x[1]**o+(-0.737205017415)*x[1]
        arg[1,3,2,0]=(0.607750966973)*x[0]**o+(0.905016928404)*x[0]+(-0.171049542163)*x[1]**o+(-0.20526244601)*x[1]
        arg[1,3,2,1]=(0.723946025405)*x[0]**o+(0.0157576743987)*x[0]+(0.190875428312)*x[1]**o+(0.407925899509)*x[1]
        arg[1,4,0,0]=(-0.561582695803)*x[0]**o+(-0.350262461371)*x[0]+(0.57610409872)*x[1]**o+(0.881780963877)*x[1]
        arg[1,4,0,1]=(0.077066498695)*x[0]**o+(0.515597802516)*x[0]+(0.332011413016)*x[1]**o+(0.215491334787)*x[1]
        arg[1,4,1,0]=(0.33858727737)*x[0]**o+(-0.573231800768)*x[0]+(-0.559915809132)*x[1]**o+(0.0240869142176)*x[1]
        arg[1,4,1,1]=(-0.684581727952)*x[0]**o+(0.365541904201)*x[0]+(0.204252397376)*x[1]**o+(0.149300766054)*x[1]
        arg[1,4,2,0]=(-0.574225088383)*x[0]**o+(0.0286339776067)*x[0]+(-0.358732122607)*x[1]**o+(-0.416932738957)*x[1]
        arg[1,4,2,1]=(-0.308460184932)*x[0]**o+(0.389214987836)*x[0]+(-0.358303257031)*x[1]**o+(0.348109483688)*x[1]
        arg[2,0,0,0]=(-0.811318668804)*x[0]**o+(-0.298028116982)*x[0]+(0.679871523483)*x[1]**o+(0.958042925287)*x[1]
        arg[2,0,0,1]=(-0.616521818338)*x[0]**o+(-0.0817124854496)*x[0]+(0.403257366893)*x[1]**o+(0.740689710435)*x[1]
        arg[2,0,1,0]=(0.271046179127)*x[0]**o+(0.814928093947)*x[0]+(0.423722781579)*x[1]**o+(0.925333209811)*x[1]
        arg[2,0,1,1]=(0.0596651806816)*x[0]**o+(0.493854605359)*x[0]+(0.539998537439)*x[1]**o+(-0.910865639375)*x[1]
        arg[2,0,2,0]=(0.503267602327)*x[0]**o+(-0.270966471743)*x[0]+(-0.342063366676)*x[1]**o+(-0.975700224705)*x[1]
        arg[2,0,2,1]=(-0.0362358939195)*x[0]**o+(-0.766033869339)*x[0]+(0.891365100088)*x[1]**o+(0.815025898929)*x[1]
        arg[2,1,0,0]=(-0.667240459984)*x[0]**o+(0.8659076701)*x[0]+(0.215089397562)*x[1]**o+(0.464769182307)*x[1]
        arg[2,1,0,1]=(-0.298625221931)*x[0]**o+(-0.504329743472)*x[0]+(0.412472510208)*x[1]**o+(0.241693152889)*x[1]
        arg[2,1,1,0]=(-0.670730179133)*x[0]**o+(0.0201070113832)*x[0]+(-0.572725898919)*x[1]**o+(0.665609259014)*x[1]
        arg[2,1,1,1]=(-0.319080984089)*x[0]**o+(0.0299423045224)*x[0]+(0.0152311810613)*x[1]**o+(-0.784646189026)*x[1]
        arg[2,1,2,0]=(0.153308789716)*x[0]**o+(0.725808150613)*x[0]+(0.62876493338)*x[1]**o+(0.80275284252)*x[1]
        arg[2,1,2,1]=(0.804698870013)*x[0]**o+(-0.631078773061)*x[0]+(-0.015221975698)*x[1]**o+(-0.954169686746)*x[1]
        arg[2,2,0,0]=(0.169022797498)*x[0]**o+(0.733025791099)*x[0]+(0.321554996633)*x[1]**o+(0.292173365364)*x[1]
        arg[2,2,0,1]=(0.327345955109)*x[0]**o+(0.21757823896)*x[0]+(0.873027160446)*x[1]**o+(-0.250192861579)*x[1]
        arg[2,2,1,0]=(0.394387190832)*x[0]**o+(-0.613661883482)*x[0]+(0.806965291386)*x[1]**o+(0.244513244016)*x[1]
        arg[2,2,1,1]=(0.043913907536)*x[0]**o+(-0.303567806424)*x[0]+(0.547037093323)*x[1]**o+(-0.598687385328)*x[1]
        arg[2,2,2,0]=(0.875663154787)*x[0]**o+(-0.34984832772)*x[0]+(0.879143174396)*x[1]**o+(-0.193154051109)*x[1]
        arg[2,2,2,1]=(0.940311132918)*x[0]**o+(-0.358708743382)*x[0]+(0.117370945146)*x[1]**o+(-0.664208034757)*x[1]
        arg[2,3,0,0]=(0.0334525660588)*x[0]**o+(0.592805915255)*x[0]+(-0.278216157971)*x[1]**o+(0.894369967671)*x[1]
        arg[2,3,0,1]=(-0.966591106326)*x[0]**o+(-0.882160390768)*x[0]+(0.843982702732)*x[1]**o+(0.363868921656)*x[1]
        arg[2,3,1,0]=(-0.246392342771)*x[0]**o+(-0.934459096242)*x[0]+(-0.472771368537)*x[1]**o+(0.148854304706)*x[1]
        arg[2,3,1,1]=(0.248218424051)*x[0]**o+(0.0980396576111)*x[0]+(0.413322166489)*x[1]**o+(0.429747261346)*x[1]
        arg[2,3,2,0]=(0.742987565733)*x[0]**o+(-0.213736450136)*x[0]+(0.996975741314)*x[1]**o+(0.960709958665)*x[1]
        arg[2,3,2,1]=(0.436149490977)*x[0]**o+(0.87632181197)*x[0]+(-0.881934669912)*x[1]**o+(-0.457520504175)*x[1]
        arg[2,4,0,0]=(-0.519014330693)*x[0]**o+(0.938619608407)*x[0]+(-0.847954563609)*x[1]**o+(0.791774379621)*x[1]
        arg[2,4,0,1]=(-0.599487299453)*x[0]**o+(0.977867555132)*x[0]+(0.693105630579)*x[1]**o+(-0.688827812942)*x[1]
        arg[2,4,1,0]=(0.955907964163)*x[0]**o+(0.0724056986172)*x[0]+(0.254773619772)*x[1]**o+(0.0428861146964)*x[1]
        arg[2,4,1,1]=(-0.422202101998)*x[0]**o+(0.803263223544)*x[0]+(0.456048140785)*x[1]**o+(0.0419133126183)*x[1]
        arg[2,4,2,0]=(-0.182308704781)*x[0]**o+(0.862282443488)*x[0]+(-0.844027007955)*x[1]**o+(0.586220526376)*x[1]
        arg[2,4,2,1]=(-0.801542666229)*x[0]**o+(0.903685299455)*x[0]+(0.428926544377)*x[1]**o+(-0.891220471549)*x[1]
        arg[3,0,0,0]=(0.602917787713)*x[0]**o+(-0.592239549743)*x[0]+(0.761830006718)*x[1]**o+(-0.784264167777)*x[1]
        arg[3,0,0,1]=(0.819962957732)*x[0]**o+(-0.455126790372)*x[0]+(-0.144898728525)*x[1]**o+(0.121566345339)*x[1]
        arg[3,0,1,0]=(0.570694941559)*x[0]**o+(-0.505678656386)*x[0]+(0.295536587355)*x[1]**o+(-0.852779737814)*x[1]
        arg[3,0,1,1]=(-0.226246858753)*x[0]**o+(-0.513900306228)*x[0]+(-0.389483591407)*x[1]**o+(-0.735105335539)*x[1]
        arg[3,0,2,0]=(0.941877610938)*x[0]**o+(0.93808330298)*x[0]+(0.189785162182)*x[1]**o+(0.546525366304)*x[1]
        arg[3,0,2,1]=(0.10639100485)*x[0]**o+(-0.773361007972)*x[0]+(-0.470531427579)*x[1]**o+(0.467359342808)*x[1]
        arg[3,1,0,0]=(0.857725599589)*x[0]**o+(0.0881746413132)*x[0]+(0.040713151152)*x[1]**o+(-0.52070729883)*x[1]
        arg[3,1,0,1]=(0.266807652146)*x[0]**o+(0.337496673892)*x[0]+(-0.562266297445)*x[1]**o+(0.326015195113)*x[1]
        arg[3,1,1,0]=(0.177275509465)*x[0]**o+(-0.647855761301)*x[0]+(-0.0963373631995)*x[1]**o+(-0.535256252994)*x[1]
        arg[3,1,1,1]=(0.430309295606)*x[0]**o+(-0.0894749557428)*x[0]+(0.503470724483)*x[1]**o+(-0.202361122248)*x[1]
        arg[3,1,2,0]=(0.283995014194)*x[0]**o+(-0.544419907194)*x[0]+(0.78478571712)*x[1]**o+(0.341540664147)*x[1]
        arg[3,1,2,1]=(-0.281042869936)*x[0]**o+(0.728771766492)*x[0]+(-0.298974388557)*x[1]**o+(-0.836789906985)*x[1]
        arg[3,2,0,0]=(-0.415323371561)*x[0]**o+(0.583971137371)*x[0]+(-0.265658679489)*x[1]**o+(-0.432447373809)*x[1]
        arg[3,2,0,1]=(0.690486298855)*x[0]**o+(0.552521632226)*x[0]+(0.237622739042)*x[1]**o+(-0.254513693924)*x[1]
        arg[3,2,1,0]=(0.453985687207)*x[0]**o+(-0.785385580962)*x[0]+(0.0190987841634)*x[1]**o+(-0.7923288861)*x[1]
        arg[3,2,1,1]=(-0.964082656379)*x[0]**o+(-0.806315937371)*x[0]+(0.271145420366)*x[1]**o+(-0.839014535945)*x[1]
        arg[3,2,2,0]=(-0.372062773997)*x[0]**o+(-0.069749560567)*x[0]+(-0.126310395383)*x[1]**o+(0.972025865285)*x[1]
        arg[3,2,2,1]=(-0.580353020086)*x[0]**o+(-0.152746821477)*x[0]+(0.297553660535)*x[1]**o+(0.444791568145)*x[1]
        arg[3,3,0,0]=(-0.361520029439)*x[0]**o+(-0.810391070499)*x[0]+(0.462123882642)*x[1]**o+(-0.974534227742)*x[1]
        arg[3,3,0,1]=(0.113370617837)*x[0]**o+(0.428257355435)*x[0]+(-0.18656623758)*x[1]**o+(0.206503327394)*x[1]
        arg[3,3,1,0]=(0.341328445689)*x[0]**o+(0.34002947054)*x[0]+(0.997677286362)*x[1]**o+(0.43300073225)*x[1]
        arg[3,3,1,1]=(-0.513912982823)*x[0]**o+(0.826573827064)*x[0]+(-0.701417864085)*x[1]**o+(0.567078883219)*x[1]
        arg[3,3,2,0]=(-0.325536866965)*x[0]**o+(0.924405767451)*x[0]+(-0.0481381954346)*x[1]**o+(0.912210617073)*x[1]
        arg[3,3,2,1]=(0.491532434302)*x[0]**o+(-0.584980848036)*x[0]+(-0.788317530858)*x[1]**o+(-0.900681893172)*x[1]
        arg[3,4,0,0]=(0.432249855148)*x[0]**o+(-0.673369312552)*x[0]+(0.244704845849)*x[1]**o+(-0.300995595367)*x[1]
        arg[3,4,0,1]=(0.383037211984)*x[0]**o+(0.299711400488)*x[0]+(0.870584255454)*x[1]**o+(-0.33571823329)*x[1]
        arg[3,4,1,0]=(0.737134180177)*x[0]**o+(-0.101307090628)*x[0]+(-0.970632061143)*x[1]**o+(-0.449042684137)*x[1]
        arg[3,4,1,1]=(0.00684250643632)*x[0]**o+(0.676964654022)*x[0]+(0.789069834858)*x[1]**o+(-0.829739508197)*x[1]
        arg[3,4,2,0]=(0.7279044944)*x[0]**o+(0.215711122038)*x[0]+(0.385631252632)*x[1]**o+(0.629390433529)*x[1]
        arg[3,4,2,1]=(-0.380750716629)*x[0]**o+(-0.413040692437)*x[0]+(0.26795244639)*x[1]**o+(0.492475504659)*x[1]
        ref[0,0,0,0]=(-0.274472293956)*x_ref[0]**o+(-0.293347130929)*x_ref[0]+(0.392586448254)*x_ref[1]**o+(-0.403066854054)*x_ref[1]
        ref[0,0,0,1]=(0.385933192907)*x_ref[0]**o+(0.484240065867)*x_ref[0]+(0.144565631551)*x_ref[1]**o+(0.882508492739)*x_ref[1]
        ref[0,0,1,0]=(0.993368905137)*x_ref[0]**o+(0.340349919343)*x_ref[0]+(-0.956806746524)*x_ref[1]**o+(0.990607958001)*x_ref[1]
        ref[0,0,1,1]=(-0.88610250102)*x_ref[0]**o+(-0.543705587259)*x_ref[0]+(0.170798164924)*x_ref[1]**o+(0.685331195754)*x_ref[1]
        ref[0,0,2,0]=(-0.922211611143)*x_ref[0]**o+(-0.103640037893)*x_ref[0]+(0.401354703434)*x_ref[1]**o+(-0.117826261635)*x_ref[1]
        ref[0,0,2,1]=(-0.391601387132)*x_ref[0]**o+(-0.472753344804)*x_ref[0]+(-0.473433838993)*x_ref[1]**o+(-0.173501635989)*x_ref[1]
        ref[0,1,0,0]=(0.747353274916)*x_ref[0]**o+(-0.960856612086)*x_ref[0]+(-0.54809012978)*x_ref[1]**o+(0.862023893332)*x_ref[1]
        ref[0,1,0,1]=(-0.256359001149)*x_ref[0]**o+(-0.868975044191)*x_ref[0]+(0.494314269744)*x_ref[1]**o+(0.949957481172)*x_ref[1]
        ref[0,1,1,0]=(-0.0256658750601)*x_ref[0]**o+(0.145499104203)*x_ref[0]+(-0.68669791629)*x_ref[1]**o+(0.447326207295)*x_ref[1]
        ref[0,1,1,1]=(0.150617284467)*x_ref[0]**o+(-0.350046088196)*x_ref[0]+(-0.367671302053)*x_ref[1]**o+(0.633220122068)*x_ref[1]
        ref[0,1,2,0]=(-0.714145362271)*x_ref[0]**o+(-0.957661490758)*x_ref[0]+(0.386253770747)*x_ref[1]**o+(0.982634262634)*x_ref[1]
        ref[0,1,2,1]=(0.707565096824)*x_ref[0]**o+(0.281758058573)*x_ref[0]+(-0.407791315587)*x_ref[1]**o+(0.86069036983)*x_ref[1]
        ref[0,2,0,0]=(-0.453860220598)*x_ref[0]**o+(0.944457388061)*x_ref[0]+(-0.903818960162)*x_ref[1]**o+(-0.799423205297)*x_ref[1]
        ref[0,2,0,1]=(-0.621257500941)*x_ref[0]**o+(0.748963737937)*x_ref[0]+(-0.12660934458)*x_ref[1]**o+(-0.362830639592)*x_ref[1]
        ref[0,2,1,0]=(-0.702485820037)*x_ref[0]**o+(0.355584898593)*x_ref[0]+(0.657127357751)*x_ref[1]**o+(0.535533390762)*x_ref[1]
        ref[0,2,1,1]=(0.795733125008)*x_ref[0]**o+(0.983607327249)*x_ref[0]+(-0.463818780644)*x_ref[1]**o+(-0.681831588244)*x_ref[1]
        ref[0,2,2,0]=(0.335783204138)*x_ref[0]**o+(-0.712225782679)*x_ref[0]+(0.853426644096)*x_ref[1]**o+(-0.143845263686)*x_ref[1]
        ref[0,2,2,1]=(0.389724375144)*x_ref[0]**o+(0.715176227705)*x_ref[0]+(-0.660690893888)*x_ref[1]**o+(0.428289724709)*x_ref[1]
        ref[0,3,0,0]=(-0.363836203724)*x_ref[0]**o+(-0.257775427457)*x_ref[0]+(0.407904889098)*x_ref[1]**o+(-0.731273814284)*x_ref[1]
        ref[0,3,0,1]=(0.402038262494)*x_ref[0]**o+(0.33289103638)*x_ref[0]+(0.444957562329)*x_ref[1]**o+(0.0657971935664)*x_ref[1]
        ref[0,3,1,0]=(0.979673814602)*x_ref[0]**o+(-0.133405332848)*x_ref[0]+(0.23884689304)*x_ref[1]**o+(0.728464997681)*x_ref[1]
        ref[0,3,1,1]=(0.724603429246)*x_ref[0]**o+(-0.391040898144)*x_ref[0]+(-0.966816499003)*x_ref[1]**o+(0.0984302578905)*x_ref[1]
        ref[0,3,2,0]=(-0.986139645754)*x_ref[0]**o+(0.60807351165)*x_ref[0]+(0.799807413231)*x_ref[1]**o+(0.12091479711)*x_ref[1]
        ref[0,3,2,1]=(0.387453999377)*x_ref[0]**o+(0.212725006628)*x_ref[0]+(-0.120374039139)*x_ref[1]**o+(-0.262304757951)*x_ref[1]
        ref[0,4,0,0]=(-0.260545986276)*x_ref[0]**o+(0.251085312932)*x_ref[0]+(0.738082708518)*x_ref[1]**o+(-0.0804883797938)*x_ref[1]
        ref[0,4,0,1]=(-0.190701356138)*x_ref[0]**o+(-0.0205295935522)*x_ref[0]+(0.904261681819)*x_ref[1]**o+(-0.859647709957)*x_ref[1]
        ref[0,4,1,0]=(-0.945357492849)*x_ref[0]**o+(-0.85298463389)*x_ref[0]+(0.688964336673)*x_ref[1]**o+(-0.672881696932)*x_ref[1]
        ref[0,4,1,1]=(0.877720258045)*x_ref[0]**o+(-0.361228026805)*x_ref[0]+(-0.326626527006)*x_ref[1]**o+(-0.274100421232)*x_ref[1]
        ref[0,4,2,0]=(0.202358729084)*x_ref[0]**o+(-0.997571481837)*x_ref[0]+(0.431193711862)*x_ref[1]**o+(0.513561123915)*x_ref[1]
        ref[0,4,2,1]=(-0.136602954592)*x_ref[0]**o+(-0.40864934437)*x_ref[0]+(0.675945574351)*x_ref[1]**o+(0.728757793536)*x_ref[1]
        ref[1,0,0,0]=(0.835464261851)*x_ref[0]**o+(-0.131447020601)*x_ref[0]+(0.0810352521446)*x_ref[1]**o+(0.664744750899)*x_ref[1]
        ref[1,0,0,1]=(0.239716928689)*x_ref[0]**o+(-0.192258999531)*x_ref[0]+(0.615877054036)*x_ref[1]**o+(0.835274774437)*x_ref[1]
        ref[1,0,1,0]=(0.509342803767)*x_ref[0]**o+(-0.633944481882)*x_ref[0]+(0.262098006086)*x_ref[1]**o+(0.159902070799)*x_ref[1]
        ref[1,0,1,1]=(0.894432440588)*x_ref[0]**o+(-0.167778431319)*x_ref[0]+(-0.984183148477)*x_ref[1]**o+(-0.554981658632)*x_ref[1]
        ref[1,0,2,0]=(-0.762472607231)*x_ref[0]**o+(0.405550139256)*x_ref[0]+(0.4051290641)*x_ref[1]**o+(-0.27180186672)*x_ref[1]
        ref[1,0,2,1]=(0.663869273816)*x_ref[0]**o+(0.867453974628)*x_ref[0]+(-0.279659969132)*x_ref[1]**o+(-0.370539101204)*x_ref[1]
        ref[1,1,0,0]=(-0.969326778164)*x_ref[0]**o+(0.965471349497)*x_ref[0]+(-0.529983232946)*x_ref[1]**o+(0.714091610977)*x_ref[1]
        ref[1,1,0,1]=(-0.840360667093)*x_ref[0]**o+(-0.399319808256)*x_ref[0]+(0.568769915582)*x_ref[1]**o+(0.872427494442)*x_ref[1]
        ref[1,1,1,0]=(0.46099116941)*x_ref[0]**o+(-0.986008874142)*x_ref[0]+(-0.53999023556)*x_ref[1]**o+(-0.549695567855)*x_ref[1]
        ref[1,1,1,1]=(0.485944918559)*x_ref[0]**o+(0.189773828651)*x_ref[0]+(-0.611654980493)*x_ref[1]**o+(-0.337071911657)*x_ref[1]
        ref[1,1,2,0]=(0.191663603248)*x_ref[0]**o+(0.0906374965004)*x_ref[0]+(-0.271051598251)*x_ref[1]**o+(0.880507788825)*x_ref[1]
        ref[1,1,2,1]=(-0.439514997001)*x_ref[0]**o+(0.701230426224)*x_ref[0]+(-0.193231641029)*x_ref[1]**o+(0.950584054332)*x_ref[1]
        ref[1,2,0,0]=(0.439456565259)*x_ref[0]**o+(-0.25664171484)*x_ref[0]+(-0.653059819329)*x_ref[1]**o+(-0.680544747777)*x_ref[1]
        ref[1,2,0,1]=(0.34890454443)*x_ref[0]**o+(-0.69851200963)*x_ref[0]+(-0.00159381688549)*x_ref[1]**o+(-0.612359675499)*x_ref[1]
        ref[1,2,1,0]=(0.132759671375)*x_ref[0]**o+(-0.235126716401)*x_ref[0]+(0.652068885826)*x_ref[1]**o+(0.749698645371)*x_ref[1]
        ref[1,2,1,1]=(0.934980403627)*x_ref[0]**o+(0.321409273437)*x_ref[0]+(-0.605909929553)*x_ref[1]**o+(0.135554927418)*x_ref[1]
        ref[1,2,2,0]=(-0.68327709329)*x_ref[0]**o+(-0.0381292756005)*x_ref[0]+(0.710953191114)*x_ref[1]**o+(-0.344522256689)*x_ref[1]
        ref[1,2,2,1]=(-0.813976235652)*x_ref[0]**o+(0.613879708123)*x_ref[0]+(0.451599277293)*x_ref[1]**o+(-0.6134026375)*x_ref[1]
        ref[1,3,0,0]=(0.595310128627)*x_ref[0]**o+(-0.247621573131)*x_ref[0]+(0.165607783104)*x_ref[1]**o+(0.578132180805)*x_ref[1]
        ref[1,3,0,1]=(0.848827238882)*x_ref[0]**o+(0.173154067999)*x_ref[0]+(0.340662419437)*x_ref[1]**o+(-0.389656447251)*x_ref[1]
        ref[1,3,1,0]=(0.250888415228)*x_ref[0]**o+(-0.278203136198)*x_ref[0]+(-0.693211186734)*x_ref[1]**o+(0.613812194735)*x_ref[1]
        ref[1,3,1,1]=(0.634533003162)*x_ref[0]**o+(-0.0924665762471)*x_ref[0]+(0.749575894384)*x_ref[1]**o+(-0.737205017415)*x_ref[1]
        ref[1,3,2,0]=(0.607750966973)*x_ref[0]**o+(0.905016928404)*x_ref[0]+(-0.171049542163)*x_ref[1]**o+(-0.20526244601)*x_ref[1]
        ref[1,3,2,1]=(0.723946025405)*x_ref[0]**o+(0.0157576743987)*x_ref[0]+(0.190875428312)*x_ref[1]**o+(0.407925899509)*x_ref[1]
        ref[1,4,0,0]=(-0.561582695803)*x_ref[0]**o+(-0.350262461371)*x_ref[0]+(0.57610409872)*x_ref[1]**o+(0.881780963877)*x_ref[1]
        ref[1,4,0,1]=(0.077066498695)*x_ref[0]**o+(0.515597802516)*x_ref[0]+(0.332011413016)*x_ref[1]**o+(0.215491334787)*x_ref[1]
        ref[1,4,1,0]=(0.33858727737)*x_ref[0]**o+(-0.573231800768)*x_ref[0]+(-0.559915809132)*x_ref[1]**o+(0.0240869142176)*x_ref[1]
        ref[1,4,1,1]=(-0.684581727952)*x_ref[0]**o+(0.365541904201)*x_ref[0]+(0.204252397376)*x_ref[1]**o+(0.149300766054)*x_ref[1]
        ref[1,4,2,0]=(-0.574225088383)*x_ref[0]**o+(0.0286339776067)*x_ref[0]+(-0.358732122607)*x_ref[1]**o+(-0.416932738957)*x_ref[1]
        ref[1,4,2,1]=(-0.308460184932)*x_ref[0]**o+(0.389214987836)*x_ref[0]+(-0.358303257031)*x_ref[1]**o+(0.348109483688)*x_ref[1]
        ref[2,0,0,0]=(-0.811318668804)*x_ref[0]**o+(-0.298028116982)*x_ref[0]+(0.679871523483)*x_ref[1]**o+(0.958042925287)*x_ref[1]
        ref[2,0,0,1]=(-0.616521818338)*x_ref[0]**o+(-0.0817124854496)*x_ref[0]+(0.403257366893)*x_ref[1]**o+(0.740689710435)*x_ref[1]
        ref[2,0,1,0]=(0.271046179127)*x_ref[0]**o+(0.814928093947)*x_ref[0]+(0.423722781579)*x_ref[1]**o+(0.925333209811)*x_ref[1]
        ref[2,0,1,1]=(0.0596651806816)*x_ref[0]**o+(0.493854605359)*x_ref[0]+(0.539998537439)*x_ref[1]**o+(-0.910865639375)*x_ref[1]
        ref[2,0,2,0]=(0.503267602327)*x_ref[0]**o+(-0.270966471743)*x_ref[0]+(-0.342063366676)*x_ref[1]**o+(-0.975700224705)*x_ref[1]
        ref[2,0,2,1]=(-0.0362358939195)*x_ref[0]**o+(-0.766033869339)*x_ref[0]+(0.891365100088)*x_ref[1]**o+(0.815025898929)*x_ref[1]
        ref[2,1,0,0]=(-0.667240459984)*x_ref[0]**o+(0.8659076701)*x_ref[0]+(0.215089397562)*x_ref[1]**o+(0.464769182307)*x_ref[1]
        ref[2,1,0,1]=(-0.298625221931)*x_ref[0]**o+(-0.504329743472)*x_ref[0]+(0.412472510208)*x_ref[1]**o+(0.241693152889)*x_ref[1]
        ref[2,1,1,0]=(-0.670730179133)*x_ref[0]**o+(0.0201070113832)*x_ref[0]+(-0.572725898919)*x_ref[1]**o+(0.665609259014)*x_ref[1]
        ref[2,1,1,1]=(-0.319080984089)*x_ref[0]**o+(0.0299423045224)*x_ref[0]+(0.0152311810613)*x_ref[1]**o+(-0.784646189026)*x_ref[1]
        ref[2,1,2,0]=(0.153308789716)*x_ref[0]**o+(0.725808150613)*x_ref[0]+(0.62876493338)*x_ref[1]**o+(0.80275284252)*x_ref[1]
        ref[2,1,2,1]=(0.804698870013)*x_ref[0]**o+(-0.631078773061)*x_ref[0]+(-0.015221975698)*x_ref[1]**o+(-0.954169686746)*x_ref[1]
        ref[2,2,0,0]=(0.169022797498)*x_ref[0]**o+(0.733025791099)*x_ref[0]+(0.321554996633)*x_ref[1]**o+(0.292173365364)*x_ref[1]
        ref[2,2,0,1]=(0.327345955109)*x_ref[0]**o+(0.21757823896)*x_ref[0]+(0.873027160446)*x_ref[1]**o+(-0.250192861579)*x_ref[1]
        ref[2,2,1,0]=(0.394387190832)*x_ref[0]**o+(-0.613661883482)*x_ref[0]+(0.806965291386)*x_ref[1]**o+(0.244513244016)*x_ref[1]
        ref[2,2,1,1]=(0.043913907536)*x_ref[0]**o+(-0.303567806424)*x_ref[0]+(0.547037093323)*x_ref[1]**o+(-0.598687385328)*x_ref[1]
        ref[2,2,2,0]=(0.875663154787)*x_ref[0]**o+(-0.34984832772)*x_ref[0]+(0.879143174396)*x_ref[1]**o+(-0.193154051109)*x_ref[1]
        ref[2,2,2,1]=(0.940311132918)*x_ref[0]**o+(-0.358708743382)*x_ref[0]+(0.117370945146)*x_ref[1]**o+(-0.664208034757)*x_ref[1]
        ref[2,3,0,0]=(0.0334525660588)*x_ref[0]**o+(0.592805915255)*x_ref[0]+(-0.278216157971)*x_ref[1]**o+(0.894369967671)*x_ref[1]
        ref[2,3,0,1]=(-0.966591106326)*x_ref[0]**o+(-0.882160390768)*x_ref[0]+(0.843982702732)*x_ref[1]**o+(0.363868921656)*x_ref[1]
        ref[2,3,1,0]=(-0.246392342771)*x_ref[0]**o+(-0.934459096242)*x_ref[0]+(-0.472771368537)*x_ref[1]**o+(0.148854304706)*x_ref[1]
        ref[2,3,1,1]=(0.248218424051)*x_ref[0]**o+(0.0980396576111)*x_ref[0]+(0.413322166489)*x_ref[1]**o+(0.429747261346)*x_ref[1]
        ref[2,3,2,0]=(0.742987565733)*x_ref[0]**o+(-0.213736450136)*x_ref[0]+(0.996975741314)*x_ref[1]**o+(0.960709958665)*x_ref[1]
        ref[2,3,2,1]=(0.436149490977)*x_ref[0]**o+(0.87632181197)*x_ref[0]+(-0.881934669912)*x_ref[1]**o+(-0.457520504175)*x_ref[1]
        ref[2,4,0,0]=(-0.519014330693)*x_ref[0]**o+(0.938619608407)*x_ref[0]+(-0.847954563609)*x_ref[1]**o+(0.791774379621)*x_ref[1]
        ref[2,4,0,1]=(-0.599487299453)*x_ref[0]**o+(0.977867555132)*x_ref[0]+(0.693105630579)*x_ref[1]**o+(-0.688827812942)*x_ref[1]
        ref[2,4,1,0]=(0.955907964163)*x_ref[0]**o+(0.0724056986172)*x_ref[0]+(0.254773619772)*x_ref[1]**o+(0.0428861146964)*x_ref[1]
        ref[2,4,1,1]=(-0.422202101998)*x_ref[0]**o+(0.803263223544)*x_ref[0]+(0.456048140785)*x_ref[1]**o+(0.0419133126183)*x_ref[1]
        ref[2,4,2,0]=(-0.182308704781)*x_ref[0]**o+(0.862282443488)*x_ref[0]+(-0.844027007955)*x_ref[1]**o+(0.586220526376)*x_ref[1]
        ref[2,4,2,1]=(-0.801542666229)*x_ref[0]**o+(0.903685299455)*x_ref[0]+(0.428926544377)*x_ref[1]**o+(-0.891220471549)*x_ref[1]
        ref[3,0,0,0]=(0.602917787713)*x_ref[0]**o+(-0.592239549743)*x_ref[0]+(0.761830006718)*x_ref[1]**o+(-0.784264167777)*x_ref[1]
        ref[3,0,0,1]=(0.819962957732)*x_ref[0]**o+(-0.455126790372)*x_ref[0]+(-0.144898728525)*x_ref[1]**o+(0.121566345339)*x_ref[1]
        ref[3,0,1,0]=(0.570694941559)*x_ref[0]**o+(-0.505678656386)*x_ref[0]+(0.295536587355)*x_ref[1]**o+(-0.852779737814)*x_ref[1]
        ref[3,0,1,1]=(-0.226246858753)*x_ref[0]**o+(-0.513900306228)*x_ref[0]+(-0.389483591407)*x_ref[1]**o+(-0.735105335539)*x_ref[1]
        ref[3,0,2,0]=(0.941877610938)*x_ref[0]**o+(0.93808330298)*x_ref[0]+(0.189785162182)*x_ref[1]**o+(0.546525366304)*x_ref[1]
        ref[3,0,2,1]=(0.10639100485)*x_ref[0]**o+(-0.773361007972)*x_ref[0]+(-0.470531427579)*x_ref[1]**o+(0.467359342808)*x_ref[1]
        ref[3,1,0,0]=(0.857725599589)*x_ref[0]**o+(0.0881746413132)*x_ref[0]+(0.040713151152)*x_ref[1]**o+(-0.52070729883)*x_ref[1]
        ref[3,1,0,1]=(0.266807652146)*x_ref[0]**o+(0.337496673892)*x_ref[0]+(-0.562266297445)*x_ref[1]**o+(0.326015195113)*x_ref[1]
        ref[3,1,1,0]=(0.177275509465)*x_ref[0]**o+(-0.647855761301)*x_ref[0]+(-0.0963373631995)*x_ref[1]**o+(-0.535256252994)*x_ref[1]
        ref[3,1,1,1]=(0.430309295606)*x_ref[0]**o+(-0.0894749557428)*x_ref[0]+(0.503470724483)*x_ref[1]**o+(-0.202361122248)*x_ref[1]
        ref[3,1,2,0]=(0.283995014194)*x_ref[0]**o+(-0.544419907194)*x_ref[0]+(0.78478571712)*x_ref[1]**o+(0.341540664147)*x_ref[1]
        ref[3,1,2,1]=(-0.281042869936)*x_ref[0]**o+(0.728771766492)*x_ref[0]+(-0.298974388557)*x_ref[1]**o+(-0.836789906985)*x_ref[1]
        ref[3,2,0,0]=(-0.415323371561)*x_ref[0]**o+(0.583971137371)*x_ref[0]+(-0.265658679489)*x_ref[1]**o+(-0.432447373809)*x_ref[1]
        ref[3,2,0,1]=(0.690486298855)*x_ref[0]**o+(0.552521632226)*x_ref[0]+(0.237622739042)*x_ref[1]**o+(-0.254513693924)*x_ref[1]
        ref[3,2,1,0]=(0.453985687207)*x_ref[0]**o+(-0.785385580962)*x_ref[0]+(0.0190987841634)*x_ref[1]**o+(-0.7923288861)*x_ref[1]
        ref[3,2,1,1]=(-0.964082656379)*x_ref[0]**o+(-0.806315937371)*x_ref[0]+(0.271145420366)*x_ref[1]**o+(-0.839014535945)*x_ref[1]
        ref[3,2,2,0]=(-0.372062773997)*x_ref[0]**o+(-0.069749560567)*x_ref[0]+(-0.126310395383)*x_ref[1]**o+(0.972025865285)*x_ref[1]
        ref[3,2,2,1]=(-0.580353020086)*x_ref[0]**o+(-0.152746821477)*x_ref[0]+(0.297553660535)*x_ref[1]**o+(0.444791568145)*x_ref[1]
        ref[3,3,0,0]=(-0.361520029439)*x_ref[0]**o+(-0.810391070499)*x_ref[0]+(0.462123882642)*x_ref[1]**o+(-0.974534227742)*x_ref[1]
        ref[3,3,0,1]=(0.113370617837)*x_ref[0]**o+(0.428257355435)*x_ref[0]+(-0.18656623758)*x_ref[1]**o+(0.206503327394)*x_ref[1]
        ref[3,3,1,0]=(0.341328445689)*x_ref[0]**o+(0.34002947054)*x_ref[0]+(0.997677286362)*x_ref[1]**o+(0.43300073225)*x_ref[1]
        ref[3,3,1,1]=(-0.513912982823)*x_ref[0]**o+(0.826573827064)*x_ref[0]+(-0.701417864085)*x_ref[1]**o+(0.567078883219)*x_ref[1]
        ref[3,3,2,0]=(-0.325536866965)*x_ref[0]**o+(0.924405767451)*x_ref[0]+(-0.0481381954346)*x_ref[1]**o+(0.912210617073)*x_ref[1]
        ref[3,3,2,1]=(0.491532434302)*x_ref[0]**o+(-0.584980848036)*x_ref[0]+(-0.788317530858)*x_ref[1]**o+(-0.900681893172)*x_ref[1]
        ref[3,4,0,0]=(0.432249855148)*x_ref[0]**o+(-0.673369312552)*x_ref[0]+(0.244704845849)*x_ref[1]**o+(-0.300995595367)*x_ref[1]
        ref[3,4,0,1]=(0.383037211984)*x_ref[0]**o+(0.299711400488)*x_ref[0]+(0.870584255454)*x_ref[1]**o+(-0.33571823329)*x_ref[1]
        ref[3,4,1,0]=(0.737134180177)*x_ref[0]**o+(-0.101307090628)*x_ref[0]+(-0.970632061143)*x_ref[1]**o+(-0.449042684137)*x_ref[1]
        ref[3,4,1,1]=(0.00684250643632)*x_ref[0]**o+(0.676964654022)*x_ref[0]+(0.789069834858)*x_ref[1]**o+(-0.829739508197)*x_ref[1]
        ref[3,4,2,0]=(0.7279044944)*x_ref[0]**o+(0.215711122038)*x_ref[0]+(0.385631252632)*x_ref[1]**o+(0.629390433529)*x_ref[1]
        ref[3,4,2,1]=(-0.380750716629)*x_ref[0]**o+(-0.413040692437)*x_ref[0]+(0.26795244639)*x_ref[1]**o+(0.492475504659)*x_ref[1]
      else:
        arg[0,0,0,0]=(0.287160004495)*x[0]**o+(0.35112250158)*x[0]+(0.061693427026)*x[1]**o+(-0.752505628029)*x[1]+(-0.57137756028)*x[2]**o+(-0.0397598892504)*x[2]
        arg[0,0,0,1]=(0.272799584607)*x[0]**o+(-0.569947956933)*x[0]+(0.322189261657)*x[1]**o+(-0.440552272224)*x[1]+(-0.10460241481)*x[2]**o+(-0.884848984752)*x[2]
        arg[0,0,1,0]=(0.147766301604)*x[0]**o+(0.405651654428)*x[0]+(-0.440827209529)*x[1]**o+(-0.399952864271)*x[1]+(-0.770191009459)*x[2]**o+(-0.789853968049)*x[2]
        arg[0,0,1,1]=(0.688053165375)*x[0]**o+(-0.154890421924)*x[0]+(0.583684164319)*x[1]**o+(-0.398082068217)*x[1]+(-0.573142923099)*x[2]**o+(-0.205271877337)*x[2]
        arg[0,0,2,0]=(0.807043275877)*x[0]**o+(0.843046383231)*x[0]+(-0.588146471402)*x[1]**o+(0.157787304557)*x[1]+(-0.771009748568)*x[2]**o+(0.162090791609)*x[2]
        arg[0,0,2,1]=(-0.0706800489392)*x[0]**o+(0.499961344665)*x[0]+(0.208926178797)*x[1]**o+(-0.7462472769)*x[1]+(-0.105475405648)*x[2]**o+(-0.18670229948)*x[2]
        arg[0,1,0,0]=(-0.696657281943)*x[0]**o+(-0.0251785345759)*x[0]+(-0.579575530451)*x[1]**o+(-0.0628378665725)*x[1]+(0.668395441302)*x[2]**o+(0.813478105092)*x[2]
        arg[0,1,0,1]=(-0.574784314876)*x[0]**o+(-0.321268045204)*x[0]+(0.160704464213)*x[1]**o+(-0.676016748103)*x[1]+(0.918971102865)*x[2]**o+(0.252082410081)*x[2]
        arg[0,1,1,0]=(-0.905062846817)*x[0]**o+(0.788906658477)*x[0]+(-0.408807611936)*x[1]**o+(0.114686186437)*x[1]+(-0.654685568493)*x[2]**o+(0.0879202139254)*x[2]
        arg[0,1,1,1]=(-0.68020231443)*x[0]**o+(-0.82049202902)*x[0]+(-0.958352740187)*x[1]**o+(-0.33201745802)*x[1]+(0.539810363682)*x[2]**o+(0.744988573087)*x[2]
        arg[0,1,2,0]=(-0.960884744997)*x[0]**o+(-0.704157132503)*x[0]+(0.282236905414)*x[1]**o+(0.756363456433)*x[1]+(-0.365400101792)*x[2]**o+(0.975735846554)*x[2]
        arg[0,1,2,1]=(-0.691780076951)*x[0]**o+(-0.952575440059)*x[0]+(-0.921247808098)*x[1]**o+(-0.88872834095)*x[1]+(0.534103013833)*x[2]**o+(-0.602295186104)*x[2]
        arg[0,2,0,0]=(0.460698415704)*x[0]**o+(0.770243814897)*x[0]+(-0.380381727574)*x[1]**o+(0.476360845668)*x[1]+(-0.681040425102)*x[2]**o+(-0.244369435116)*x[2]
        arg[0,2,0,1]=(0.251591652281)*x[0]**o+(0.748234919686)*x[0]+(-0.698189037307)*x[1]**o+(-0.292400412508)*x[1]+(0.613850696577)*x[2]**o+(-0.942243226834)*x[2]
        arg[0,2,1,0]=(-0.718090066328)*x[0]**o+(-0.322499511794)*x[0]+(-0.75105623397)*x[1]**o+(0.526680271832)*x[1]+(-0.186671531793)*x[2]**o+(-0.449273875991)*x[2]
        arg[0,2,1,1]=(-0.52165552452)*x[0]**o+(0.902446006085)*x[0]+(-0.915575172702)*x[1]**o+(-0.150816194662)*x[1]+(0.596782785689)*x[2]**o+(0.546880865642)*x[2]
        arg[0,2,2,0]=(-0.104486704816)*x[0]**o+(0.723863699457)*x[0]+(0.758622117279)*x[1]**o+(0.372271580145)*x[1]+(-0.0820570383792)*x[2]**o+(-0.63539781104)*x[2]
        arg[0,2,2,1]=(0.0355930823603)*x[0]**o+(0.0931942600432)*x[0]+(-0.118096985978)*x[1]**o+(0.296983782212)*x[1]+(-0.864491763712)*x[2]**o+(0.942230663978)*x[2]
        arg[0,3,0,0]=(0.371153709815)*x[0]**o+(0.0427309083893)*x[0]+(-0.886563919084)*x[1]**o+(-0.14946513404)*x[1]+(0.80903804519)*x[2]**o+(0.69728533073)*x[2]
        arg[0,3,0,1]=(0.0593930396142)*x[0]**o+(-0.370327974089)*x[0]+(-0.388354464683)*x[1]**o+(-0.367648771812)*x[1]+(-0.573754821458)*x[2]**o+(-0.947499155479)*x[2]
        arg[0,3,1,0]=(-0.869817317)*x[0]**o+(0.874317772678)*x[0]+(-0.870060178375)*x[1]**o+(0.66571074524)*x[1]+(-0.669052377378)*x[2]**o+(0.940212680575)*x[2]
        arg[0,3,1,1]=(-0.417445606772)*x[0]**o+(0.942606057619)*x[0]+(-0.133936034409)*x[1]**o+(-0.363274280866)*x[1]+(0.83487439935)*x[2]**o+(-0.108398981461)*x[2]
        arg[0,3,2,0]=(0.363623383293)*x[0]**o+(0.624267309217)*x[0]+(0.801719984899)*x[1]**o+(0.189212370122)*x[1]+(0.464633630928)*x[2]**o+(-0.979808318438)*x[2]
        arg[0,3,2,1]=(0.85384408805)*x[0]**o+(-0.889214754116)*x[0]+(0.164915391224)*x[1]**o+(0.0168991405201)*x[1]+(0.097171994236)*x[2]**o+(-0.146094197339)*x[2]
        arg[0,4,0,0]=(0.778356158972)*x[0]**o+(0.910241265149)*x[0]+(0.123690863272)*x[1]**o+(0.87802121779)*x[1]+(-0.714615668595)*x[2]**o+(-0.437472831583)*x[2]
        arg[0,4,0,1]=(-0.157185481783)*x[0]**o+(0.352802163968)*x[0]+(0.390184123612)*x[1]**o+(-0.680415093098)*x[1]+(-0.121195042454)*x[2]**o+(-0.713693712117)*x[2]
        arg[0,4,1,0]=(0.778115668646)*x[0]**o+(-0.111729042892)*x[0]+(0.451518002735)*x[1]**o+(0.232731183374)*x[1]+(-0.764246158086)*x[2]**o+(-0.795525155328)*x[2]
        arg[0,4,1,1]=(-0.880652367027)*x[0]**o+(-0.314966079126)*x[0]+(-0.640720373065)*x[1]**o+(0.336230541901)*x[1]+(0.566956999526)*x[2]**o+(0.74098258811)*x[2]
        arg[0,4,2,0]=(0.908657274301)*x[0]**o+(0.636427823614)*x[0]+(0.650137414386)*x[1]**o+(0.695558379005)*x[1]+(-0.489852401087)*x[2]**o+(-0.479227108044)*x[2]
        arg[0,4,2,1]=(0.015734241525)*x[0]**o+(0.0719884727088)*x[0]+(0.811864895261)*x[1]**o+(0.959127986784)*x[1]+(-0.238146663368)*x[2]**o+(-0.454939322571)*x[2]
        arg[1,0,0,0]=(-0.918218246559)*x[0]**o+(-0.662787169269)*x[0]+(-0.954035144111)*x[1]**o+(-0.647264685228)*x[1]+(-0.834085960213)*x[2]**o+(-0.167541951024)*x[2]
        arg[1,0,0,1]=(-0.670950869382)*x[0]**o+(0.826472159989)*x[0]+(0.180410872786)*x[1]**o+(-0.715162871594)*x[1]+(0.653287853169)*x[2]**o+(0.892060529625)*x[2]
        arg[1,0,1,0]=(0.398789786407)*x[0]**o+(0.689408033554)*x[0]+(0.124338245128)*x[1]**o+(0.748280808587)*x[1]+(0.418357122274)*x[2]**o+(0.230707440075)*x[2]
        arg[1,0,1,1]=(-0.912778964055)*x[0]**o+(0.904288708694)*x[0]+(-0.874744856901)*x[1]**o+(0.550162195719)*x[1]+(-0.7526423373)*x[2]**o+(0.820993231654)*x[2]
        arg[1,0,2,0]=(-0.843459684648)*x[0]**o+(-0.579708764091)*x[0]+(-0.876072131849)*x[1]**o+(0.933694599696)*x[1]+(0.507781209021)*x[2]**o+(-0.719729482133)*x[2]
        arg[1,0,2,1]=(0.594566925796)*x[0]**o+(-0.994301292358)*x[0]+(0.808114389564)*x[1]**o+(-0.970803712864)*x[1]+(0.433544910389)*x[2]**o+(0.260659440299)*x[2]
        arg[1,1,0,0]=(0.997377553172)*x[0]**o+(0.458428616383)*x[0]+(0.357280778493)*x[1]**o+(0.349837110741)*x[1]+(-0.709592214445)*x[2]**o+(-0.630000739033)*x[2]
        arg[1,1,0,1]=(0.121595650501)*x[0]**o+(0.40870120924)*x[0]+(0.576917123807)*x[1]**o+(-0.932268308325)*x[1]+(0.710043472116)*x[2]**o+(-0.716528412077)*x[2]
        arg[1,1,1,0]=(-0.151416035033)*x[0]**o+(0.0255347649787)*x[0]+(-0.752987701889)*x[1]**o+(-0.480969875813)*x[1]+(-0.610514504179)*x[2]**o+(0.959274757402)*x[2]
        arg[1,1,1,1]=(0.76695706106)*x[0]**o+(-0.0750965975512)*x[0]+(0.846833900735)*x[1]**o+(-0.105838465439)*x[1]+(0.541027486457)*x[2]**o+(0.152543017589)*x[2]
        arg[1,1,2,0]=(0.0856977181865)*x[0]**o+(-0.188139369706)*x[0]+(0.936071565289)*x[1]**o+(-0.3958713036)*x[1]+(-0.350945878893)*x[2]**o+(0.382302674883)*x[2]
        arg[1,1,2,1]=(0.873054234789)*x[0]**o+(-0.490869716146)*x[0]+(0.46720526212)*x[1]**o+(-0.334203412334)*x[1]+(-0.81140895636)*x[2]**o+(-0.973858971749)*x[2]
        arg[1,2,0,0]=(-0.716676352443)*x[0]**o+(0.659821396277)*x[0]+(0.335964292954)*x[1]**o+(0.11609664773)*x[1]+(-0.889186749466)*x[2]**o+(0.454231973946)*x[2]
        arg[1,2,0,1]=(0.367048872648)*x[0]**o+(0.276083713126)*x[0]+(0.686639128043)*x[1]**o+(-0.991414415293)*x[1]+(0.25360515964)*x[2]**o+(-0.377623315093)*x[2]
        arg[1,2,1,0]=(-0.723089358777)*x[0]**o+(-0.516690112406)*x[0]+(0.262334273979)*x[1]**o+(0.140773502537)*x[1]+(0.447333972396)*x[2]**o+(-0.551395692966)*x[2]
        arg[1,2,1,1]=(0.738791182491)*x[0]**o+(-0.978120626668)*x[0]+(-0.217970433713)*x[1]**o+(0.0181011067492)*x[1]+(0.756322867523)*x[2]**o+(0.495269195978)*x[2]
        arg[1,2,2,0]=(-0.630506449854)*x[0]**o+(-0.0865482364231)*x[0]+(-0.415896403157)*x[1]**o+(-0.712833149895)*x[1]+(-0.180482789777)*x[2]**o+(0.00788290661235)*x[2]
        arg[1,2,2,1]=(0.0946913865377)*x[0]**o+(0.752854827716)*x[0]+(0.257846220843)*x[1]**o+(-0.953926983578)*x[1]+(-0.388505920682)*x[2]**o+(-0.10751401251)*x[2]
        arg[1,3,0,0]=(0.694014030611)*x[0]**o+(0.0433103547797)*x[0]+(-0.0805025225569)*x[1]**o+(0.389052254754)*x[1]+(-0.813210939614)*x[2]**o+(-0.593788787896)*x[2]
        arg[1,3,0,1]=(0.0101606353406)*x[0]**o+(-0.849874258855)*x[0]+(-0.0578973959381)*x[1]**o+(0.942113140424)*x[1]+(0.613716459216)*x[2]**o+(0.72631176833)*x[2]
        arg[1,3,1,0]=(0.264703608811)*x[0]**o+(0.457962548326)*x[0]+(-0.433795242673)*x[1]**o+(0.650995712848)*x[1]+(0.372471807967)*x[2]**o+(0.00416636841047)*x[2]
        arg[1,3,1,1]=(-0.73686451669)*x[0]**o+(-0.0425614910776)*x[0]+(-0.587820431311)*x[1]**o+(-0.377628617833)*x[1]+(-0.463964014406)*x[2]**o+(0.283273874989)*x[2]
        arg[1,3,2,0]=(-0.689341439143)*x[0]**o+(-0.675861918889)*x[0]+(-0.918622662352)*x[1]**o+(0.740990261888)*x[1]+(0.0454007677041)*x[2]**o+(0.513623361074)*x[2]
        arg[1,3,2,1]=(0.13229101848)*x[0]**o+(0.171873799817)*x[0]+(-0.154491174434)*x[1]**o+(0.897976884089)*x[1]+(-0.598886311821)*x[2]**o+(0.738696710918)*x[2]
        arg[1,4,0,0]=(0.59403636966)*x[0]**o+(0.93856848969)*x[0]+(0.80164849167)*x[1]**o+(-0.753322979568)*x[1]+(-0.107098073916)*x[2]**o+(-0.623047487973)*x[2]
        arg[1,4,0,1]=(0.291046099082)*x[0]**o+(-0.437982471049)*x[0]+(0.017395762947)*x[1]**o+(0.852440593923)*x[1]+(0.630825424359)*x[2]**o+(0.419347491727)*x[2]
        arg[1,4,1,0]=(-0.747962012701)*x[0]**o+(-0.11622032094)*x[0]+(0.324950549781)*x[1]**o+(0.950911398327)*x[1]+(-0.18098990205)*x[2]**o+(-0.676060633522)*x[2]
        arg[1,4,1,1]=(0.545224363128)*x[0]**o+(0.753376367121)*x[0]+(0.271426695753)*x[1]**o+(-0.139309844675)*x[1]+(-0.923434716375)*x[2]**o+(-0.985988324389)*x[2]
        arg[1,4,2,0]=(-0.532260822047)*x[0]**o+(0.901407135724)*x[0]+(0.937451330141)*x[1]**o+(0.285217666559)*x[1]+(-0.133028984824)*x[2]**o+(0.993965133696)*x[2]
        arg[1,4,2,1]=(0.2231815607)*x[0]**o+(0.994845537582)*x[0]+(0.0368727378195)*x[1]**o+(0.511772933777)*x[1]+(-0.871580215976)*x[2]**o+(-0.587010057329)*x[2]
        arg[2,0,0,0]=(-0.55149411913)*x[0]**o+(-0.905541278559)*x[0]+(0.461217373096)*x[1]**o+(-0.53359853241)*x[1]+(-0.666643290883)*x[2]**o+(0.151495826926)*x[2]
        arg[2,0,0,1]=(-0.980485062148)*x[0]**o+(0.150165616162)*x[0]+(0.59165725487)*x[1]**o+(-0.243853536017)*x[1]+(0.66816261686)*x[2]**o+(0.2425696131)*x[2]
        arg[2,0,1,0]=(-0.509337213758)*x[0]**o+(0.652936706687)*x[0]+(-0.849214338091)*x[1]**o+(-0.523354123549)*x[1]+(0.371515689887)*x[2]**o+(0.799989815414)*x[2]
        arg[2,0,1,1]=(0.461584544541)*x[0]**o+(0.662931355954)*x[0]+(0.744415634786)*x[1]**o+(-0.016743801256)*x[1]+(0.172945846325)*x[2]**o+(-0.0889827089341)*x[2]
        arg[2,0,2,0]=(0.506514791562)*x[0]**o+(0.974477160694)*x[0]+(-0.968485343167)*x[1]**o+(0.427480013599)*x[1]+(0.0460755879358)*x[2]**o+(0.186500994948)*x[2]
        arg[2,0,2,1]=(0.259831607993)*x[0]**o+(-0.0141290444125)*x[0]+(-0.968452177682)*x[1]**o+(0.635619125009)*x[1]+(-0.958848431827)*x[2]**o+(-0.0977241523308)*x[2]
        arg[2,1,0,0]=(0.204233222933)*x[0]**o+(0.834739381877)*x[0]+(0.19961185048)*x[1]**o+(-0.238470380578)*x[1]+(-0.749979713773)*x[2]**o+(-0.268168886251)*x[2]
        arg[2,1,0,1]=(0.998123643088)*x[0]**o+(0.635823733041)*x[0]+(-0.265447124419)*x[1]**o+(-0.225582751946)*x[1]+(0.91705937439)*x[2]**o+(-0.739461510792)*x[2]
        arg[2,1,1,0]=(0.0800715288646)*x[0]**o+(-0.37677196659)*x[0]+(-0.0737286350219)*x[1]**o+(0.032076062933)*x[1]+(0.245421912709)*x[2]**o+(-0.8378023935)*x[2]
        arg[2,1,1,1]=(-0.572376055603)*x[0]**o+(0.8675468469)*x[0]+(-0.533012889867)*x[1]**o+(-0.247215658297)*x[1]+(0.673144564693)*x[2]**o+(-0.749136108985)*x[2]
        arg[2,1,2,0]=(-0.661188192488)*x[0]**o+(-0.924435593008)*x[0]+(-0.791845808916)*x[1]**o+(0.00133765351092)*x[1]+(0.233676862873)*x[2]**o+(0.981812737962)*x[2]
        arg[2,1,2,1]=(0.332072445986)*x[0]**o+(-0.909373798588)*x[0]+(0.927025647819)*x[1]**o+(-0.936348942617)*x[1]+(0.251508592152)*x[2]**o+(-0.468147973987)*x[2]
        arg[2,2,0,0]=(0.892377893841)*x[0]**o+(-0.147933293091)*x[0]+(0.950491102076)*x[1]**o+(0.595250355926)*x[1]+(0.0169764649992)*x[2]**o+(-0.683137752586)*x[2]
        arg[2,2,0,1]=(0.00579397005645)*x[0]**o+(0.627197373917)*x[0]+(0.663574756196)*x[1]**o+(0.934106767513)*x[1]+(0.105844878022)*x[2]**o+(0.513322917688)*x[2]
        arg[2,2,1,0]=(-0.168776016133)*x[0]**o+(-0.940071612944)*x[0]+(0.0366053554566)*x[1]**o+(-0.628010909771)*x[1]+(0.250023045906)*x[2]**o+(-0.00327322332166)*x[2]
        arg[2,2,1,1]=(0.904155764376)*x[0]**o+(-0.258018307306)*x[0]+(0.65862820521)*x[1]**o+(0.434480798515)*x[1]+(0.4089419173)*x[2]**o+(-0.518032695598)*x[2]
        arg[2,2,2,0]=(-0.41470070958)*x[0]**o+(-0.783186751952)*x[0]+(0.548309561942)*x[1]**o+(0.837130592803)*x[1]+(-0.346808491848)*x[2]**o+(0.165416040985)*x[2]
        arg[2,2,2,1]=(0.997845909195)*x[0]**o+(0.448428042158)*x[0]+(-0.657050004311)*x[1]**o+(0.493104851747)*x[1]+(0.558315071197)*x[2]**o+(0.206659432923)*x[2]
        arg[2,3,0,0]=(0.433874360232)*x[0]**o+(-0.0635646197131)*x[0]+(-0.521690521523)*x[1]**o+(-0.298130684888)*x[1]+(-0.131905313644)*x[2]**o+(0.64389451112)*x[2]
        arg[2,3,0,1]=(0.457729315216)*x[0]**o+(-0.419323242135)*x[0]+(0.614544051135)*x[1]**o+(-0.701671999275)*x[1]+(0.905494300727)*x[2]**o+(-0.565716985337)*x[2]
        arg[2,3,1,0]=(0.256570728097)*x[0]**o+(0.866349285103)*x[0]+(-0.115768354896)*x[1]**o+(-0.613044456549)*x[1]+(-0.20074185156)*x[2]**o+(-0.188403571776)*x[2]
        arg[2,3,1,1]=(0.767479943414)*x[0]**o+(0.884066215007)*x[0]+(0.363855420056)*x[1]**o+(0.919698981104)*x[1]+(0.225747762317)*x[2]**o+(-0.220362271104)*x[2]
        arg[2,3,2,0]=(-0.746469501382)*x[0]**o+(0.680520398001)*x[0]+(0.926950996325)*x[1]**o+(-0.122831303044)*x[1]+(-0.27226561675)*x[2]**o+(0.933304087951)*x[2]
        arg[2,3,2,1]=(-0.0961434747216)*x[0]**o+(0.987660516767)*x[0]+(0.288175205045)*x[1]**o+(0.683720820403)*x[1]+(0.354403143642)*x[2]**o+(-0.745694089132)*x[2]
        arg[2,4,0,0]=(0.921750835986)*x[0]**o+(-0.0358861598394)*x[0]+(0.190129645259)*x[1]**o+(0.377067803331)*x[1]+(-0.0188615236582)*x[2]**o+(0.804533888824)*x[2]
        arg[2,4,0,1]=(-0.256132234484)*x[0]**o+(-0.102661411017)*x[0]+(-0.903465157009)*x[1]**o+(0.50133310168)*x[1]+(-0.0889102598619)*x[2]**o+(-0.0310174238365)*x[2]
        arg[2,4,1,0]=(0.58483798855)*x[0]**o+(0.967943344305)*x[0]+(-0.0701080005438)*x[1]**o+(0.489476104406)*x[1]+(0.615308427038)*x[2]**o+(-0.38065505709)*x[2]
        arg[2,4,1,1]=(0.647734130724)*x[0]**o+(-0.930934392719)*x[0]+(0.267370540491)*x[1]**o+(-0.0175019424556)*x[1]+(0.38593657129)*x[2]**o+(-0.985922292359)*x[2]
        arg[2,4,2,0]=(-0.289971479933)*x[0]**o+(0.882014313157)*x[0]+(0.147321067579)*x[1]**o+(-0.176771838489)*x[1]+(0.710916531544)*x[2]**o+(-0.0856290170401)*x[2]
        arg[2,4,2,1]=(-0.465930997733)*x[0]**o+(-0.305563942094)*x[0]+(-0.824535262061)*x[1]**o+(-0.917892226972)*x[1]+(0.624462871955)*x[2]**o+(0.544736500065)*x[2]
        arg[3,0,0,0]=(-0.673230815097)*x[0]**o+(-0.726292712917)*x[0]+(0.0531037021688)*x[1]**o+(-0.467219030159)*x[1]+(-0.0729861369718)*x[2]**o+(-0.440394335015)*x[2]
        arg[3,0,0,1]=(-0.358235295558)*x[0]**o+(-0.301184713671)*x[0]+(-0.496030649086)*x[1]**o+(-0.125922184777)*x[1]+(0.946715036973)*x[2]**o+(0.846370337893)*x[2]
        arg[3,0,1,0]=(-0.394175109612)*x[0]**o+(-0.85137582307)*x[0]+(-0.667804785044)*x[1]**o+(0.156762109891)*x[1]+(-0.616444700525)*x[2]**o+(0.0738729399356)*x[2]
        arg[3,0,1,1]=(-0.652875462534)*x[0]**o+(-0.0605650192541)*x[0]+(0.840539933823)*x[1]**o+(-0.422819423301)*x[1]+(0.113440226819)*x[2]**o+(-0.915154532306)*x[2]
        arg[3,0,2,0]=(-0.162863352328)*x[0]**o+(-0.364461332292)*x[0]+(-0.222181741398)*x[1]**o+(-0.563668008365)*x[1]+(0.118491770105)*x[2]**o+(0.0809155053022)*x[2]
        arg[3,0,2,1]=(-0.985278929437)*x[0]**o+(-0.919440760157)*x[0]+(-0.776910686)*x[1]**o+(-0.884270488943)*x[1]+(0.984279010363)*x[2]**o+(-0.489503278582)*x[2]
        arg[3,1,0,0]=(-0.806565449823)*x[0]**o+(-0.0851385375281)*x[0]+(-0.88851381773)*x[1]**o+(-0.666539867528)*x[1]+(0.154825531411)*x[2]**o+(0.764069849144)*x[2]
        arg[3,1,0,1]=(-0.222125783918)*x[0]**o+(0.568035675467)*x[0]+(0.470171778932)*x[1]**o+(-0.307608710881)*x[1]+(-0.836083742471)*x[2]**o+(0.553597553381)*x[2]
        arg[3,1,1,0]=(0.431237389847)*x[0]**o+(-0.141391012471)*x[0]+(0.726895025519)*x[1]**o+(-0.527868751679)*x[1]+(0.711761083137)*x[2]**o+(0.996992864103)*x[2]
        arg[3,1,1,1]=(-0.123885959924)*x[0]**o+(0.877034475529)*x[0]+(-0.0432492682928)*x[1]**o+(-0.370315494618)*x[1]+(0.243139719071)*x[2]**o+(-0.684318634902)*x[2]
        arg[3,1,2,0]=(-0.477214489355)*x[0]**o+(0.340808576619)*x[0]+(-0.0991802306558)*x[1]**o+(0.462599821908)*x[1]+(0.091768281564)*x[2]**o+(0.0315552838839)*x[2]
        arg[3,1,2,1]=(0.676189142894)*x[0]**o+(0.510136386913)*x[0]+(-0.558579041952)*x[1]**o+(0.961615047877)*x[1]+(0.510717386124)*x[2]**o+(0.774430183432)*x[2]
        arg[3,2,0,0]=(0.158370232444)*x[0]**o+(0.209424855992)*x[0]+(0.691438753013)*x[1]**o+(-0.781594298738)*x[1]+(-0.0188536627517)*x[2]**o+(0.136400798995)*x[2]
        arg[3,2,0,1]=(-0.440674414328)*x[0]**o+(0.118228612687)*x[0]+(-0.700026492613)*x[1]**o+(-0.981341215714)*x[1]+(-0.172847519578)*x[2]**o+(-0.423095202172)*x[2]
        arg[3,2,1,0]=(-0.0694960391695)*x[0]**o+(-0.66448610135)*x[0]+(-0.177572703748)*x[1]**o+(-0.060488161727)*x[1]+(0.306021853885)*x[2]**o+(0.0730338716187)*x[2]
        arg[3,2,1,1]=(-0.360662154547)*x[0]**o+(0.577679359648)*x[0]+(-0.515313534966)*x[1]**o+(0.217352687358)*x[1]+(0.728367577882)*x[2]**o+(0.38505849834)*x[2]
        arg[3,2,2,0]=(0.568671861232)*x[0]**o+(-0.0792490917882)*x[0]+(0.0188002885229)*x[1]**o+(-0.956862575759)*x[1]+(-0.844332442983)*x[2]**o+(0.670792928042)*x[2]
        arg[3,2,2,1]=(0.833099204713)*x[0]**o+(0.368482169933)*x[0]+(-0.926002498892)*x[1]**o+(-0.130646446568)*x[1]+(0.81164928914)*x[2]**o+(-0.641079919927)*x[2]
        arg[3,3,0,0]=(-0.277769350989)*x[0]**o+(0.753452018649)*x[0]+(-0.766322344505)*x[1]**o+(-0.679108224612)*x[1]+(0.0799499795424)*x[2]**o+(0.255751308975)*x[2]
        arg[3,3,0,1]=(0.429176753733)*x[0]**o+(0.123106050259)*x[0]+(-0.169037728992)*x[1]**o+(0.749245933687)*x[1]+(0.560140859355)*x[2]**o+(-0.937091840021)*x[2]
        arg[3,3,1,0]=(0.648792557593)*x[0]**o+(-0.057555187513)*x[0]+(0.883290331726)*x[1]**o+(0.539276905995)*x[1]+(0.473614408583)*x[2]**o+(0.723258340333)*x[2]
        arg[3,3,1,1]=(0.527298833572)*x[0]**o+(0.0753824058279)*x[0]+(0.719196099257)*x[1]**o+(-0.0124603350035)*x[1]+(-0.964484650253)*x[2]**o+(0.404444269694)*x[2]
        arg[3,3,2,0]=(-0.403640677462)*x[0]**o+(0.906195964766)*x[0]+(-0.561419078404)*x[1]**o+(0.645408717838)*x[1]+(0.671624799609)*x[2]**o+(-0.573901868621)*x[2]
        arg[3,3,2,1]=(-0.288117182165)*x[0]**o+(-0.391003584945)*x[0]+(-0.555037182038)*x[1]**o+(0.299362209017)*x[1]+(-0.552429724326)*x[2]**o+(-0.416923306774)*x[2]
        arg[3,4,0,0]=(-0.19594719192)*x[0]**o+(0.150898934692)*x[0]+(0.210935232044)*x[1]**o+(-0.319764293347)*x[1]+(-0.642450751195)*x[2]**o+(0.22231085168)*x[2]
        arg[3,4,0,1]=(-0.0514589460225)*x[0]**o+(0.0366025646754)*x[0]+(0.416280576691)*x[1]**o+(0.609178063951)*x[1]+(0.835611677062)*x[2]**o+(-0.131630571478)*x[2]
        arg[3,4,1,0]=(0.171509152214)*x[0]**o+(-0.761188271093)*x[0]+(-0.915951802896)*x[1]**o+(-0.451796709915)*x[1]+(-0.979847828162)*x[2]**o+(0.0566453393315)*x[2]
        arg[3,4,1,1]=(0.690427998102)*x[0]**o+(-0.358318625695)*x[0]+(0.330270871613)*x[1]**o+(-0.724392522137)*x[1]+(-0.535474623489)*x[2]**o+(-0.12341743885)*x[2]
        arg[3,4,2,0]=(0.488090753426)*x[0]**o+(-0.352417078372)*x[0]+(0.971988099319)*x[1]**o+(-0.288929516009)*x[1]+(0.612576734094)*x[2]**o+(-0.43799514603)*x[2]
        arg[3,4,2,1]=(-0.098768101356)*x[0]**o+(0.123300661895)*x[0]+(-0.0489724280789)*x[1]**o+(-0.422508669137)*x[1]+(-0.861483104139)*x[2]**o+(0.950198316001)*x[2]
        ref[0,0,0,0]=(0.287160004495)*x_ref[0]**o+(0.35112250158)*x_ref[0]+(0.061693427026)*x_ref[1]**o+(-0.752505628029)*x_ref[1]+(-0.57137756028)*x_ref[2]**o+(-0.0397598892504)*x_ref[2]
        ref[0,0,0,1]=(0.272799584607)*x_ref[0]**o+(-0.569947956933)*x_ref[0]+(0.322189261657)*x_ref[1]**o+(-0.440552272224)*x_ref[1]+(-0.10460241481)*x_ref[2]**o+(-0.884848984752)*x_ref[2]
        ref[0,0,1,0]=(0.147766301604)*x_ref[0]**o+(0.405651654428)*x_ref[0]+(-0.440827209529)*x_ref[1]**o+(-0.399952864271)*x_ref[1]+(-0.770191009459)*x_ref[2]**o+(-0.789853968049)*x_ref[2]
        ref[0,0,1,1]=(0.688053165375)*x_ref[0]**o+(-0.154890421924)*x_ref[0]+(0.583684164319)*x_ref[1]**o+(-0.398082068217)*x_ref[1]+(-0.573142923099)*x_ref[2]**o+(-0.205271877337)*x_ref[2]
        ref[0,0,2,0]=(0.807043275877)*x_ref[0]**o+(0.843046383231)*x_ref[0]+(-0.588146471402)*x_ref[1]**o+(0.157787304557)*x_ref[1]+(-0.771009748568)*x_ref[2]**o+(0.162090791609)*x_ref[2]
        ref[0,0,2,1]=(-0.0706800489392)*x_ref[0]**o+(0.499961344665)*x_ref[0]+(0.208926178797)*x_ref[1]**o+(-0.7462472769)*x_ref[1]+(-0.105475405648)*x_ref[2]**o+(-0.18670229948)*x_ref[2]
        ref[0,1,0,0]=(-0.696657281943)*x_ref[0]**o+(-0.0251785345759)*x_ref[0]+(-0.579575530451)*x_ref[1]**o+(-0.0628378665725)*x_ref[1]+(0.668395441302)*x_ref[2]**o+(0.813478105092)*x_ref[2]
        ref[0,1,0,1]=(-0.574784314876)*x_ref[0]**o+(-0.321268045204)*x_ref[0]+(0.160704464213)*x_ref[1]**o+(-0.676016748103)*x_ref[1]+(0.918971102865)*x_ref[2]**o+(0.252082410081)*x_ref[2]
        ref[0,1,1,0]=(-0.905062846817)*x_ref[0]**o+(0.788906658477)*x_ref[0]+(-0.408807611936)*x_ref[1]**o+(0.114686186437)*x_ref[1]+(-0.654685568493)*x_ref[2]**o+(0.0879202139254)*x_ref[2]
        ref[0,1,1,1]=(-0.68020231443)*x_ref[0]**o+(-0.82049202902)*x_ref[0]+(-0.958352740187)*x_ref[1]**o+(-0.33201745802)*x_ref[1]+(0.539810363682)*x_ref[2]**o+(0.744988573087)*x_ref[2]
        ref[0,1,2,0]=(-0.960884744997)*x_ref[0]**o+(-0.704157132503)*x_ref[0]+(0.282236905414)*x_ref[1]**o+(0.756363456433)*x_ref[1]+(-0.365400101792)*x_ref[2]**o+(0.975735846554)*x_ref[2]
        ref[0,1,2,1]=(-0.691780076951)*x_ref[0]**o+(-0.952575440059)*x_ref[0]+(-0.921247808098)*x_ref[1]**o+(-0.88872834095)*x_ref[1]+(0.534103013833)*x_ref[2]**o+(-0.602295186104)*x_ref[2]
        ref[0,2,0,0]=(0.460698415704)*x_ref[0]**o+(0.770243814897)*x_ref[0]+(-0.380381727574)*x_ref[1]**o+(0.476360845668)*x_ref[1]+(-0.681040425102)*x_ref[2]**o+(-0.244369435116)*x_ref[2]
        ref[0,2,0,1]=(0.251591652281)*x_ref[0]**o+(0.748234919686)*x_ref[0]+(-0.698189037307)*x_ref[1]**o+(-0.292400412508)*x_ref[1]+(0.613850696577)*x_ref[2]**o+(-0.942243226834)*x_ref[2]
        ref[0,2,1,0]=(-0.718090066328)*x_ref[0]**o+(-0.322499511794)*x_ref[0]+(-0.75105623397)*x_ref[1]**o+(0.526680271832)*x_ref[1]+(-0.186671531793)*x_ref[2]**o+(-0.449273875991)*x_ref[2]
        ref[0,2,1,1]=(-0.52165552452)*x_ref[0]**o+(0.902446006085)*x_ref[0]+(-0.915575172702)*x_ref[1]**o+(-0.150816194662)*x_ref[1]+(0.596782785689)*x_ref[2]**o+(0.546880865642)*x_ref[2]
        ref[0,2,2,0]=(-0.104486704816)*x_ref[0]**o+(0.723863699457)*x_ref[0]+(0.758622117279)*x_ref[1]**o+(0.372271580145)*x_ref[1]+(-0.0820570383792)*x_ref[2]**o+(-0.63539781104)*x_ref[2]
        ref[0,2,2,1]=(0.0355930823603)*x_ref[0]**o+(0.0931942600432)*x_ref[0]+(-0.118096985978)*x_ref[1]**o+(0.296983782212)*x_ref[1]+(-0.864491763712)*x_ref[2]**o+(0.942230663978)*x_ref[2]
        ref[0,3,0,0]=(0.371153709815)*x_ref[0]**o+(0.0427309083893)*x_ref[0]+(-0.886563919084)*x_ref[1]**o+(-0.14946513404)*x_ref[1]+(0.80903804519)*x_ref[2]**o+(0.69728533073)*x_ref[2]
        ref[0,3,0,1]=(0.0593930396142)*x_ref[0]**o+(-0.370327974089)*x_ref[0]+(-0.388354464683)*x_ref[1]**o+(-0.367648771812)*x_ref[1]+(-0.573754821458)*x_ref[2]**o+(-0.947499155479)*x_ref[2]
        ref[0,3,1,0]=(-0.869817317)*x_ref[0]**o+(0.874317772678)*x_ref[0]+(-0.870060178375)*x_ref[1]**o+(0.66571074524)*x_ref[1]+(-0.669052377378)*x_ref[2]**o+(0.940212680575)*x_ref[2]
        ref[0,3,1,1]=(-0.417445606772)*x_ref[0]**o+(0.942606057619)*x_ref[0]+(-0.133936034409)*x_ref[1]**o+(-0.363274280866)*x_ref[1]+(0.83487439935)*x_ref[2]**o+(-0.108398981461)*x_ref[2]
        ref[0,3,2,0]=(0.363623383293)*x_ref[0]**o+(0.624267309217)*x_ref[0]+(0.801719984899)*x_ref[1]**o+(0.189212370122)*x_ref[1]+(0.464633630928)*x_ref[2]**o+(-0.979808318438)*x_ref[2]
        ref[0,3,2,1]=(0.85384408805)*x_ref[0]**o+(-0.889214754116)*x_ref[0]+(0.164915391224)*x_ref[1]**o+(0.0168991405201)*x_ref[1]+(0.097171994236)*x_ref[2]**o+(-0.146094197339)*x_ref[2]
        ref[0,4,0,0]=(0.778356158972)*x_ref[0]**o+(0.910241265149)*x_ref[0]+(0.123690863272)*x_ref[1]**o+(0.87802121779)*x_ref[1]+(-0.714615668595)*x_ref[2]**o+(-0.437472831583)*x_ref[2]
        ref[0,4,0,1]=(-0.157185481783)*x_ref[0]**o+(0.352802163968)*x_ref[0]+(0.390184123612)*x_ref[1]**o+(-0.680415093098)*x_ref[1]+(-0.121195042454)*x_ref[2]**o+(-0.713693712117)*x_ref[2]
        ref[0,4,1,0]=(0.778115668646)*x_ref[0]**o+(-0.111729042892)*x_ref[0]+(0.451518002735)*x_ref[1]**o+(0.232731183374)*x_ref[1]+(-0.764246158086)*x_ref[2]**o+(-0.795525155328)*x_ref[2]
        ref[0,4,1,1]=(-0.880652367027)*x_ref[0]**o+(-0.314966079126)*x_ref[0]+(-0.640720373065)*x_ref[1]**o+(0.336230541901)*x_ref[1]+(0.566956999526)*x_ref[2]**o+(0.74098258811)*x_ref[2]
        ref[0,4,2,0]=(0.908657274301)*x_ref[0]**o+(0.636427823614)*x_ref[0]+(0.650137414386)*x_ref[1]**o+(0.695558379005)*x_ref[1]+(-0.489852401087)*x_ref[2]**o+(-0.479227108044)*x_ref[2]
        ref[0,4,2,1]=(0.015734241525)*x_ref[0]**o+(0.0719884727088)*x_ref[0]+(0.811864895261)*x_ref[1]**o+(0.959127986784)*x_ref[1]+(-0.238146663368)*x_ref[2]**o+(-0.454939322571)*x_ref[2]
        ref[1,0,0,0]=(-0.918218246559)*x_ref[0]**o+(-0.662787169269)*x_ref[0]+(-0.954035144111)*x_ref[1]**o+(-0.647264685228)*x_ref[1]+(-0.834085960213)*x_ref[2]**o+(-0.167541951024)*x_ref[2]
        ref[1,0,0,1]=(-0.670950869382)*x_ref[0]**o+(0.826472159989)*x_ref[0]+(0.180410872786)*x_ref[1]**o+(-0.715162871594)*x_ref[1]+(0.653287853169)*x_ref[2]**o+(0.892060529625)*x_ref[2]
        ref[1,0,1,0]=(0.398789786407)*x_ref[0]**o+(0.689408033554)*x_ref[0]+(0.124338245128)*x_ref[1]**o+(0.748280808587)*x_ref[1]+(0.418357122274)*x_ref[2]**o+(0.230707440075)*x_ref[2]
        ref[1,0,1,1]=(-0.912778964055)*x_ref[0]**o+(0.904288708694)*x_ref[0]+(-0.874744856901)*x_ref[1]**o+(0.550162195719)*x_ref[1]+(-0.7526423373)*x_ref[2]**o+(0.820993231654)*x_ref[2]
        ref[1,0,2,0]=(-0.843459684648)*x_ref[0]**o+(-0.579708764091)*x_ref[0]+(-0.876072131849)*x_ref[1]**o+(0.933694599696)*x_ref[1]+(0.507781209021)*x_ref[2]**o+(-0.719729482133)*x_ref[2]
        ref[1,0,2,1]=(0.594566925796)*x_ref[0]**o+(-0.994301292358)*x_ref[0]+(0.808114389564)*x_ref[1]**o+(-0.970803712864)*x_ref[1]+(0.433544910389)*x_ref[2]**o+(0.260659440299)*x_ref[2]
        ref[1,1,0,0]=(0.997377553172)*x_ref[0]**o+(0.458428616383)*x_ref[0]+(0.357280778493)*x_ref[1]**o+(0.349837110741)*x_ref[1]+(-0.709592214445)*x_ref[2]**o+(-0.630000739033)*x_ref[2]
        ref[1,1,0,1]=(0.121595650501)*x_ref[0]**o+(0.40870120924)*x_ref[0]+(0.576917123807)*x_ref[1]**o+(-0.932268308325)*x_ref[1]+(0.710043472116)*x_ref[2]**o+(-0.716528412077)*x_ref[2]
        ref[1,1,1,0]=(-0.151416035033)*x_ref[0]**o+(0.0255347649787)*x_ref[0]+(-0.752987701889)*x_ref[1]**o+(-0.480969875813)*x_ref[1]+(-0.610514504179)*x_ref[2]**o+(0.959274757402)*x_ref[2]
        ref[1,1,1,1]=(0.76695706106)*x_ref[0]**o+(-0.0750965975512)*x_ref[0]+(0.846833900735)*x_ref[1]**o+(-0.105838465439)*x_ref[1]+(0.541027486457)*x_ref[2]**o+(0.152543017589)*x_ref[2]
        ref[1,1,2,0]=(0.0856977181865)*x_ref[0]**o+(-0.188139369706)*x_ref[0]+(0.936071565289)*x_ref[1]**o+(-0.3958713036)*x_ref[1]+(-0.350945878893)*x_ref[2]**o+(0.382302674883)*x_ref[2]
        ref[1,1,2,1]=(0.873054234789)*x_ref[0]**o+(-0.490869716146)*x_ref[0]+(0.46720526212)*x_ref[1]**o+(-0.334203412334)*x_ref[1]+(-0.81140895636)*x_ref[2]**o+(-0.973858971749)*x_ref[2]
        ref[1,2,0,0]=(-0.716676352443)*x_ref[0]**o+(0.659821396277)*x_ref[0]+(0.335964292954)*x_ref[1]**o+(0.11609664773)*x_ref[1]+(-0.889186749466)*x_ref[2]**o+(0.454231973946)*x_ref[2]
        ref[1,2,0,1]=(0.367048872648)*x_ref[0]**o+(0.276083713126)*x_ref[0]+(0.686639128043)*x_ref[1]**o+(-0.991414415293)*x_ref[1]+(0.25360515964)*x_ref[2]**o+(-0.377623315093)*x_ref[2]
        ref[1,2,1,0]=(-0.723089358777)*x_ref[0]**o+(-0.516690112406)*x_ref[0]+(0.262334273979)*x_ref[1]**o+(0.140773502537)*x_ref[1]+(0.447333972396)*x_ref[2]**o+(-0.551395692966)*x_ref[2]
        ref[1,2,1,1]=(0.738791182491)*x_ref[0]**o+(-0.978120626668)*x_ref[0]+(-0.217970433713)*x_ref[1]**o+(0.0181011067492)*x_ref[1]+(0.756322867523)*x_ref[2]**o+(0.495269195978)*x_ref[2]
        ref[1,2,2,0]=(-0.630506449854)*x_ref[0]**o+(-0.0865482364231)*x_ref[0]+(-0.415896403157)*x_ref[1]**o+(-0.712833149895)*x_ref[1]+(-0.180482789777)*x_ref[2]**o+(0.00788290661235)*x_ref[2]
        ref[1,2,2,1]=(0.0946913865377)*x_ref[0]**o+(0.752854827716)*x_ref[0]+(0.257846220843)*x_ref[1]**o+(-0.953926983578)*x_ref[1]+(-0.388505920682)*x_ref[2]**o+(-0.10751401251)*x_ref[2]
        ref[1,3,0,0]=(0.694014030611)*x_ref[0]**o+(0.0433103547797)*x_ref[0]+(-0.0805025225569)*x_ref[1]**o+(0.389052254754)*x_ref[1]+(-0.813210939614)*x_ref[2]**o+(-0.593788787896)*x_ref[2]
        ref[1,3,0,1]=(0.0101606353406)*x_ref[0]**o+(-0.849874258855)*x_ref[0]+(-0.0578973959381)*x_ref[1]**o+(0.942113140424)*x_ref[1]+(0.613716459216)*x_ref[2]**o+(0.72631176833)*x_ref[2]
        ref[1,3,1,0]=(0.264703608811)*x_ref[0]**o+(0.457962548326)*x_ref[0]+(-0.433795242673)*x_ref[1]**o+(0.650995712848)*x_ref[1]+(0.372471807967)*x_ref[2]**o+(0.00416636841047)*x_ref[2]
        ref[1,3,1,1]=(-0.73686451669)*x_ref[0]**o+(-0.0425614910776)*x_ref[0]+(-0.587820431311)*x_ref[1]**o+(-0.377628617833)*x_ref[1]+(-0.463964014406)*x_ref[2]**o+(0.283273874989)*x_ref[2]
        ref[1,3,2,0]=(-0.689341439143)*x_ref[0]**o+(-0.675861918889)*x_ref[0]+(-0.918622662352)*x_ref[1]**o+(0.740990261888)*x_ref[1]+(0.0454007677041)*x_ref[2]**o+(0.513623361074)*x_ref[2]
        ref[1,3,2,1]=(0.13229101848)*x_ref[0]**o+(0.171873799817)*x_ref[0]+(-0.154491174434)*x_ref[1]**o+(0.897976884089)*x_ref[1]+(-0.598886311821)*x_ref[2]**o+(0.738696710918)*x_ref[2]
        ref[1,4,0,0]=(0.59403636966)*x_ref[0]**o+(0.93856848969)*x_ref[0]+(0.80164849167)*x_ref[1]**o+(-0.753322979568)*x_ref[1]+(-0.107098073916)*x_ref[2]**o+(-0.623047487973)*x_ref[2]
        ref[1,4,0,1]=(0.291046099082)*x_ref[0]**o+(-0.437982471049)*x_ref[0]+(0.017395762947)*x_ref[1]**o+(0.852440593923)*x_ref[1]+(0.630825424359)*x_ref[2]**o+(0.419347491727)*x_ref[2]
        ref[1,4,1,0]=(-0.747962012701)*x_ref[0]**o+(-0.11622032094)*x_ref[0]+(0.324950549781)*x_ref[1]**o+(0.950911398327)*x_ref[1]+(-0.18098990205)*x_ref[2]**o+(-0.676060633522)*x_ref[2]
        ref[1,4,1,1]=(0.545224363128)*x_ref[0]**o+(0.753376367121)*x_ref[0]+(0.271426695753)*x_ref[1]**o+(-0.139309844675)*x_ref[1]+(-0.923434716375)*x_ref[2]**o+(-0.985988324389)*x_ref[2]
        ref[1,4,2,0]=(-0.532260822047)*x_ref[0]**o+(0.901407135724)*x_ref[0]+(0.937451330141)*x_ref[1]**o+(0.285217666559)*x_ref[1]+(-0.133028984824)*x_ref[2]**o+(0.993965133696)*x_ref[2]
        ref[1,4,2,1]=(0.2231815607)*x_ref[0]**o+(0.994845537582)*x_ref[0]+(0.0368727378195)*x_ref[1]**o+(0.511772933777)*x_ref[1]+(-0.871580215976)*x_ref[2]**o+(-0.587010057329)*x_ref[2]
        ref[2,0,0,0]=(-0.55149411913)*x_ref[0]**o+(-0.905541278559)*x_ref[0]+(0.461217373096)*x_ref[1]**o+(-0.53359853241)*x_ref[1]+(-0.666643290883)*x_ref[2]**o+(0.151495826926)*x_ref[2]
        ref[2,0,0,1]=(-0.980485062148)*x_ref[0]**o+(0.150165616162)*x_ref[0]+(0.59165725487)*x_ref[1]**o+(-0.243853536017)*x_ref[1]+(0.66816261686)*x_ref[2]**o+(0.2425696131)*x_ref[2]
        ref[2,0,1,0]=(-0.509337213758)*x_ref[0]**o+(0.652936706687)*x_ref[0]+(-0.849214338091)*x_ref[1]**o+(-0.523354123549)*x_ref[1]+(0.371515689887)*x_ref[2]**o+(0.799989815414)*x_ref[2]
        ref[2,0,1,1]=(0.461584544541)*x_ref[0]**o+(0.662931355954)*x_ref[0]+(0.744415634786)*x_ref[1]**o+(-0.016743801256)*x_ref[1]+(0.172945846325)*x_ref[2]**o+(-0.0889827089341)*x_ref[2]
        ref[2,0,2,0]=(0.506514791562)*x_ref[0]**o+(0.974477160694)*x_ref[0]+(-0.968485343167)*x_ref[1]**o+(0.427480013599)*x_ref[1]+(0.0460755879358)*x_ref[2]**o+(0.186500994948)*x_ref[2]
        ref[2,0,2,1]=(0.259831607993)*x_ref[0]**o+(-0.0141290444125)*x_ref[0]+(-0.968452177682)*x_ref[1]**o+(0.635619125009)*x_ref[1]+(-0.958848431827)*x_ref[2]**o+(-0.0977241523308)*x_ref[2]
        ref[2,1,0,0]=(0.204233222933)*x_ref[0]**o+(0.834739381877)*x_ref[0]+(0.19961185048)*x_ref[1]**o+(-0.238470380578)*x_ref[1]+(-0.749979713773)*x_ref[2]**o+(-0.268168886251)*x_ref[2]
        ref[2,1,0,1]=(0.998123643088)*x_ref[0]**o+(0.635823733041)*x_ref[0]+(-0.265447124419)*x_ref[1]**o+(-0.225582751946)*x_ref[1]+(0.91705937439)*x_ref[2]**o+(-0.739461510792)*x_ref[2]
        ref[2,1,1,0]=(0.0800715288646)*x_ref[0]**o+(-0.37677196659)*x_ref[0]+(-0.0737286350219)*x_ref[1]**o+(0.032076062933)*x_ref[1]+(0.245421912709)*x_ref[2]**o+(-0.8378023935)*x_ref[2]
        ref[2,1,1,1]=(-0.572376055603)*x_ref[0]**o+(0.8675468469)*x_ref[0]+(-0.533012889867)*x_ref[1]**o+(-0.247215658297)*x_ref[1]+(0.673144564693)*x_ref[2]**o+(-0.749136108985)*x_ref[2]
        ref[2,1,2,0]=(-0.661188192488)*x_ref[0]**o+(-0.924435593008)*x_ref[0]+(-0.791845808916)*x_ref[1]**o+(0.00133765351092)*x_ref[1]+(0.233676862873)*x_ref[2]**o+(0.981812737962)*x_ref[2]
        ref[2,1,2,1]=(0.332072445986)*x_ref[0]**o+(-0.909373798588)*x_ref[0]+(0.927025647819)*x_ref[1]**o+(-0.936348942617)*x_ref[1]+(0.251508592152)*x_ref[2]**o+(-0.468147973987)*x_ref[2]
        ref[2,2,0,0]=(0.892377893841)*x_ref[0]**o+(-0.147933293091)*x_ref[0]+(0.950491102076)*x_ref[1]**o+(0.595250355926)*x_ref[1]+(0.0169764649992)*x_ref[2]**o+(-0.683137752586)*x_ref[2]
        ref[2,2,0,1]=(0.00579397005645)*x_ref[0]**o+(0.627197373917)*x_ref[0]+(0.663574756196)*x_ref[1]**o+(0.934106767513)*x_ref[1]+(0.105844878022)*x_ref[2]**o+(0.513322917688)*x_ref[2]
        ref[2,2,1,0]=(-0.168776016133)*x_ref[0]**o+(-0.940071612944)*x_ref[0]+(0.0366053554566)*x_ref[1]**o+(-0.628010909771)*x_ref[1]+(0.250023045906)*x_ref[2]**o+(-0.00327322332166)*x_ref[2]
        ref[2,2,1,1]=(0.904155764376)*x_ref[0]**o+(-0.258018307306)*x_ref[0]+(0.65862820521)*x_ref[1]**o+(0.434480798515)*x_ref[1]+(0.4089419173)*x_ref[2]**o+(-0.518032695598)*x_ref[2]
        ref[2,2,2,0]=(-0.41470070958)*x_ref[0]**o+(-0.783186751952)*x_ref[0]+(0.548309561942)*x_ref[1]**o+(0.837130592803)*x_ref[1]+(-0.346808491848)*x_ref[2]**o+(0.165416040985)*x_ref[2]
        ref[2,2,2,1]=(0.997845909195)*x_ref[0]**o+(0.448428042158)*x_ref[0]+(-0.657050004311)*x_ref[1]**o+(0.493104851747)*x_ref[1]+(0.558315071197)*x_ref[2]**o+(0.206659432923)*x_ref[2]
        ref[2,3,0,0]=(0.433874360232)*x_ref[0]**o+(-0.0635646197131)*x_ref[0]+(-0.521690521523)*x_ref[1]**o+(-0.298130684888)*x_ref[1]+(-0.131905313644)*x_ref[2]**o+(0.64389451112)*x_ref[2]
        ref[2,3,0,1]=(0.457729315216)*x_ref[0]**o+(-0.419323242135)*x_ref[0]+(0.614544051135)*x_ref[1]**o+(-0.701671999275)*x_ref[1]+(0.905494300727)*x_ref[2]**o+(-0.565716985337)*x_ref[2]
        ref[2,3,1,0]=(0.256570728097)*x_ref[0]**o+(0.866349285103)*x_ref[0]+(-0.115768354896)*x_ref[1]**o+(-0.613044456549)*x_ref[1]+(-0.20074185156)*x_ref[2]**o+(-0.188403571776)*x_ref[2]
        ref[2,3,1,1]=(0.767479943414)*x_ref[0]**o+(0.884066215007)*x_ref[0]+(0.363855420056)*x_ref[1]**o+(0.919698981104)*x_ref[1]+(0.225747762317)*x_ref[2]**o+(-0.220362271104)*x_ref[2]
        ref[2,3,2,0]=(-0.746469501382)*x_ref[0]**o+(0.680520398001)*x_ref[0]+(0.926950996325)*x_ref[1]**o+(-0.122831303044)*x_ref[1]+(-0.27226561675)*x_ref[2]**o+(0.933304087951)*x_ref[2]
        ref[2,3,2,1]=(-0.0961434747216)*x_ref[0]**o+(0.987660516767)*x_ref[0]+(0.288175205045)*x_ref[1]**o+(0.683720820403)*x_ref[1]+(0.354403143642)*x_ref[2]**o+(-0.745694089132)*x_ref[2]
        ref[2,4,0,0]=(0.921750835986)*x_ref[0]**o+(-0.0358861598394)*x_ref[0]+(0.190129645259)*x_ref[1]**o+(0.377067803331)*x_ref[1]+(-0.0188615236582)*x_ref[2]**o+(0.804533888824)*x_ref[2]
        ref[2,4,0,1]=(-0.256132234484)*x_ref[0]**o+(-0.102661411017)*x_ref[0]+(-0.903465157009)*x_ref[1]**o+(0.50133310168)*x_ref[1]+(-0.0889102598619)*x_ref[2]**o+(-0.0310174238365)*x_ref[2]
        ref[2,4,1,0]=(0.58483798855)*x_ref[0]**o+(0.967943344305)*x_ref[0]+(-0.0701080005438)*x_ref[1]**o+(0.489476104406)*x_ref[1]+(0.615308427038)*x_ref[2]**o+(-0.38065505709)*x_ref[2]
        ref[2,4,1,1]=(0.647734130724)*x_ref[0]**o+(-0.930934392719)*x_ref[0]+(0.267370540491)*x_ref[1]**o+(-0.0175019424556)*x_ref[1]+(0.38593657129)*x_ref[2]**o+(-0.985922292359)*x_ref[2]
        ref[2,4,2,0]=(-0.289971479933)*x_ref[0]**o+(0.882014313157)*x_ref[0]+(0.147321067579)*x_ref[1]**o+(-0.176771838489)*x_ref[1]+(0.710916531544)*x_ref[2]**o+(-0.0856290170401)*x_ref[2]
        ref[2,4,2,1]=(-0.465930997733)*x_ref[0]**o+(-0.305563942094)*x_ref[0]+(-0.824535262061)*x_ref[1]**o+(-0.917892226972)*x_ref[1]+(0.624462871955)*x_ref[2]**o+(0.544736500065)*x_ref[2]
        ref[3,0,0,0]=(-0.673230815097)*x_ref[0]**o+(-0.726292712917)*x_ref[0]+(0.0531037021688)*x_ref[1]**o+(-0.467219030159)*x_ref[1]+(-0.0729861369718)*x_ref[2]**o+(-0.440394335015)*x_ref[2]
        ref[3,0,0,1]=(-0.358235295558)*x_ref[0]**o+(-0.301184713671)*x_ref[0]+(-0.496030649086)*x_ref[1]**o+(-0.125922184777)*x_ref[1]+(0.946715036973)*x_ref[2]**o+(0.846370337893)*x_ref[2]
        ref[3,0,1,0]=(-0.394175109612)*x_ref[0]**o+(-0.85137582307)*x_ref[0]+(-0.667804785044)*x_ref[1]**o+(0.156762109891)*x_ref[1]+(-0.616444700525)*x_ref[2]**o+(0.0738729399356)*x_ref[2]
        ref[3,0,1,1]=(-0.652875462534)*x_ref[0]**o+(-0.0605650192541)*x_ref[0]+(0.840539933823)*x_ref[1]**o+(-0.422819423301)*x_ref[1]+(0.113440226819)*x_ref[2]**o+(-0.915154532306)*x_ref[2]
        ref[3,0,2,0]=(-0.162863352328)*x_ref[0]**o+(-0.364461332292)*x_ref[0]+(-0.222181741398)*x_ref[1]**o+(-0.563668008365)*x_ref[1]+(0.118491770105)*x_ref[2]**o+(0.0809155053022)*x_ref[2]
        ref[3,0,2,1]=(-0.985278929437)*x_ref[0]**o+(-0.919440760157)*x_ref[0]+(-0.776910686)*x_ref[1]**o+(-0.884270488943)*x_ref[1]+(0.984279010363)*x_ref[2]**o+(-0.489503278582)*x_ref[2]
        ref[3,1,0,0]=(-0.806565449823)*x_ref[0]**o+(-0.0851385375281)*x_ref[0]+(-0.88851381773)*x_ref[1]**o+(-0.666539867528)*x_ref[1]+(0.154825531411)*x_ref[2]**o+(0.764069849144)*x_ref[2]
        ref[3,1,0,1]=(-0.222125783918)*x_ref[0]**o+(0.568035675467)*x_ref[0]+(0.470171778932)*x_ref[1]**o+(-0.307608710881)*x_ref[1]+(-0.836083742471)*x_ref[2]**o+(0.553597553381)*x_ref[2]
        ref[3,1,1,0]=(0.431237389847)*x_ref[0]**o+(-0.141391012471)*x_ref[0]+(0.726895025519)*x_ref[1]**o+(-0.527868751679)*x_ref[1]+(0.711761083137)*x_ref[2]**o+(0.996992864103)*x_ref[2]
        ref[3,1,1,1]=(-0.123885959924)*x_ref[0]**o+(0.877034475529)*x_ref[0]+(-0.0432492682928)*x_ref[1]**o+(-0.370315494618)*x_ref[1]+(0.243139719071)*x_ref[2]**o+(-0.684318634902)*x_ref[2]
        ref[3,1,2,0]=(-0.477214489355)*x_ref[0]**o+(0.340808576619)*x_ref[0]+(-0.0991802306558)*x_ref[1]**o+(0.462599821908)*x_ref[1]+(0.091768281564)*x_ref[2]**o+(0.0315552838839)*x_ref[2]
        ref[3,1,2,1]=(0.676189142894)*x_ref[0]**o+(0.510136386913)*x_ref[0]+(-0.558579041952)*x_ref[1]**o+(0.961615047877)*x_ref[1]+(0.510717386124)*x_ref[2]**o+(0.774430183432)*x_ref[2]
        ref[3,2,0,0]=(0.158370232444)*x_ref[0]**o+(0.209424855992)*x_ref[0]+(0.691438753013)*x_ref[1]**o+(-0.781594298738)*x_ref[1]+(-0.0188536627517)*x_ref[2]**o+(0.136400798995)*x_ref[2]
        ref[3,2,0,1]=(-0.440674414328)*x_ref[0]**o+(0.118228612687)*x_ref[0]+(-0.700026492613)*x_ref[1]**o+(-0.981341215714)*x_ref[1]+(-0.172847519578)*x_ref[2]**o+(-0.423095202172)*x_ref[2]
        ref[3,2,1,0]=(-0.0694960391695)*x_ref[0]**o+(-0.66448610135)*x_ref[0]+(-0.177572703748)*x_ref[1]**o+(-0.060488161727)*x_ref[1]+(0.306021853885)*x_ref[2]**o+(0.0730338716187)*x_ref[2]
        ref[3,2,1,1]=(-0.360662154547)*x_ref[0]**o+(0.577679359648)*x_ref[0]+(-0.515313534966)*x_ref[1]**o+(0.217352687358)*x_ref[1]+(0.728367577882)*x_ref[2]**o+(0.38505849834)*x_ref[2]
        ref[3,2,2,0]=(0.568671861232)*x_ref[0]**o+(-0.0792490917882)*x_ref[0]+(0.0188002885229)*x_ref[1]**o+(-0.956862575759)*x_ref[1]+(-0.844332442983)*x_ref[2]**o+(0.670792928042)*x_ref[2]
        ref[3,2,2,1]=(0.833099204713)*x_ref[0]**o+(0.368482169933)*x_ref[0]+(-0.926002498892)*x_ref[1]**o+(-0.130646446568)*x_ref[1]+(0.81164928914)*x_ref[2]**o+(-0.641079919927)*x_ref[2]
        ref[3,3,0,0]=(-0.277769350989)*x_ref[0]**o+(0.753452018649)*x_ref[0]+(-0.766322344505)*x_ref[1]**o+(-0.679108224612)*x_ref[1]+(0.0799499795424)*x_ref[2]**o+(0.255751308975)*x_ref[2]
        ref[3,3,0,1]=(0.429176753733)*x_ref[0]**o+(0.123106050259)*x_ref[0]+(-0.169037728992)*x_ref[1]**o+(0.749245933687)*x_ref[1]+(0.560140859355)*x_ref[2]**o+(-0.937091840021)*x_ref[2]
        ref[3,3,1,0]=(0.648792557593)*x_ref[0]**o+(-0.057555187513)*x_ref[0]+(0.883290331726)*x_ref[1]**o+(0.539276905995)*x_ref[1]+(0.473614408583)*x_ref[2]**o+(0.723258340333)*x_ref[2]
        ref[3,3,1,1]=(0.527298833572)*x_ref[0]**o+(0.0753824058279)*x_ref[0]+(0.719196099257)*x_ref[1]**o+(-0.0124603350035)*x_ref[1]+(-0.964484650253)*x_ref[2]**o+(0.404444269694)*x_ref[2]
        ref[3,3,2,0]=(-0.403640677462)*x_ref[0]**o+(0.906195964766)*x_ref[0]+(-0.561419078404)*x_ref[1]**o+(0.645408717838)*x_ref[1]+(0.671624799609)*x_ref[2]**o+(-0.573901868621)*x_ref[2]
        ref[3,3,2,1]=(-0.288117182165)*x_ref[0]**o+(-0.391003584945)*x_ref[0]+(-0.555037182038)*x_ref[1]**o+(0.299362209017)*x_ref[1]+(-0.552429724326)*x_ref[2]**o+(-0.416923306774)*x_ref[2]
        ref[3,4,0,0]=(-0.19594719192)*x_ref[0]**o+(0.150898934692)*x_ref[0]+(0.210935232044)*x_ref[1]**o+(-0.319764293347)*x_ref[1]+(-0.642450751195)*x_ref[2]**o+(0.22231085168)*x_ref[2]
        ref[3,4,0,1]=(-0.0514589460225)*x_ref[0]**o+(0.0366025646754)*x_ref[0]+(0.416280576691)*x_ref[1]**o+(0.609178063951)*x_ref[1]+(0.835611677062)*x_ref[2]**o+(-0.131630571478)*x_ref[2]
        ref[3,4,1,0]=(0.171509152214)*x_ref[0]**o+(-0.761188271093)*x_ref[0]+(-0.915951802896)*x_ref[1]**o+(-0.451796709915)*x_ref[1]+(-0.979847828162)*x_ref[2]**o+(0.0566453393315)*x_ref[2]
        ref[3,4,1,1]=(0.690427998102)*x_ref[0]**o+(-0.358318625695)*x_ref[0]+(0.330270871613)*x_ref[1]**o+(-0.724392522137)*x_ref[1]+(-0.535474623489)*x_ref[2]**o+(-0.12341743885)*x_ref[2]
        ref[3,4,2,0]=(0.488090753426)*x_ref[0]**o+(-0.352417078372)*x_ref[0]+(0.971988099319)*x_ref[1]**o+(-0.288929516009)*x_ref[1]+(0.612576734094)*x_ref[2]**o+(-0.43799514603)*x_ref[2]
        ref[3,4,2,1]=(-0.098768101356)*x_ref[0]**o+(0.123300661895)*x_ref[0]+(-0.0489724280789)*x_ref[1]**o+(-0.422508669137)*x_ref[1]+(-0.861483104139)*x_ref[2]**o+(0.950198316001)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_Solution_rank0(self):
      """
      tests interpolate for rank 0 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from Solution onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Data(0,(),w)
      ref=Data(0,(),w_ref)
      if dim==2:
        arg=(0.724373964817)*x[0]**o+(-0.0560850774901)*x[0]+(0.201815078281)*x[1]**o+(-0.721683697048)*x[1]
        ref=(0.724373964817)*x_ref[0]**o+(-0.0560850774901)*x_ref[0]+(0.201815078281)*x_ref[1]**o+(-0.721683697048)*x_ref[1]
      else:
        arg=(0.390296010052)*x[0]**o+(-0.279464234123)*x[0]+(-0.791334947488)*x[1]**o+(-0.0210629346704)*x[1]+(0.0816556526012)*x[2]**o+(0.77326269137)*x[2]
        ref=(0.390296010052)*x_ref[0]**o+(-0.279464234123)*x_ref[0]+(-0.791334947488)*x_ref[1]**o+(-0.0210629346704)*x_ref[1]+(0.0816556526012)*x_ref[2]**o+(0.77326269137)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_Solution_rank1(self):
      """
      tests interpolate for rank 1 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from Solution onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Data(0,(2,),w)
      ref=Data(0,(2,),w_ref)
      if dim==2:
        arg[0]=(0.645585823917)*x[0]**o+(-0.203443728436)*x[0]+(0.278126891692)*x[1]**o+(0.922093564911)*x[1]
        arg[1]=(0.23370137029)*x[0]**o+(0.84371173541)*x[0]+(0.99297574548)*x[1]**o+(0.72481785931)*x[1]
        ref[0]=(0.645585823917)*x_ref[0]**o+(-0.203443728436)*x_ref[0]+(0.278126891692)*x_ref[1]**o+(0.922093564911)*x_ref[1]
        ref[1]=(0.23370137029)*x_ref[0]**o+(0.84371173541)*x_ref[0]+(0.99297574548)*x_ref[1]**o+(0.72481785931)*x_ref[1]
      else:
        arg[0]=(0.159998049951)*x[0]**o+(-0.863367327023)*x[0]+(-0.0628498246638)*x[1]**o+(0.685139358273)*x[1]+(0.783213360641)*x[2]**o+(0.397526421338)*x[2]
        arg[1]=(0.165519279053)*x[0]**o+(-0.0936018967661)*x[0]+(0.863047696619)*x[1]**o+(0.653118186434)*x[1]+(-0.408224131944)*x[2]**o+(0.82400132854)*x[2]
        ref[0]=(0.159998049951)*x_ref[0]**o+(-0.863367327023)*x_ref[0]+(-0.0628498246638)*x_ref[1]**o+(0.685139358273)*x_ref[1]+(0.783213360641)*x_ref[2]**o+(0.397526421338)*x_ref[2]
        ref[1]=(0.165519279053)*x_ref[0]**o+(-0.0936018967661)*x_ref[0]+(0.863047696619)*x_ref[1]**o+(0.653118186434)*x_ref[1]+(-0.408224131944)*x_ref[2]**o+(0.82400132854)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_Solution_rank2(self):
      """
      tests interpolate for rank 2 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from Solution onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5),w)
      ref=Data(0,(4, 5),w_ref)
      if dim==2:
        arg[0,0]=(0.490709482882)*x[0]**o+(0.518382584887)*x[0]+(-0.940707425885)*x[1]**o+(0.224578726673)*x[1]
        arg[0,1]=(-0.747621141516)*x[0]**o+(-0.217452489097)*x[0]+(0.929120128008)*x[1]**o+(0.55263700209)*x[1]
        arg[0,2]=(-0.183545119672)*x[0]**o+(0.148192386895)*x[0]+(-0.22046309708)*x[1]**o+(0.433620446668)*x[1]
        arg[0,3]=(-0.0724840266959)*x[0]**o+(0.835144620602)*x[0]+(-0.962971534144)*x[1]**o+(-0.149681811534)*x[1]
        arg[0,4]=(-0.380150192228)*x[0]**o+(-0.309494603814)*x[0]+(-0.217425635182)*x[1]**o+(0.93567286482)*x[1]
        arg[1,0]=(0.0367921563947)*x[0]**o+(0.220217625576)*x[0]+(-0.120595819238)*x[1]**o+(0.308108793279)*x[1]
        arg[1,1]=(-0.142860237109)*x[0]**o+(-0.789758888253)*x[0]+(0.748085258229)*x[1]**o+(0.784767485883)*x[1]
        arg[1,2]=(-0.154690818845)*x[0]**o+(-0.678458370784)*x[0]+(0.750742996193)*x[1]**o+(-0.350199095531)*x[1]
        arg[1,3]=(-0.893295215013)*x[0]**o+(0.0295805814985)*x[0]+(0.265893730887)*x[1]**o+(0.435487860418)*x[1]
        arg[1,4]=(-0.302701704727)*x[0]**o+(0.618274652266)*x[0]+(-0.688386407308)*x[1]**o+(-0.832515064466)*x[1]
        arg[2,0]=(-0.926658745469)*x[0]**o+(0.548174350496)*x[0]+(-0.350700366641)*x[1]**o+(0.530188602011)*x[1]
        arg[2,1]=(0.900965385167)*x[0]**o+(0.0234051845124)*x[0]+(-0.00783834941699)*x[1]**o+(-0.766352156598)*x[1]
        arg[2,2]=(0.694733282444)*x[0]**o+(-0.0644519234093)*x[0]+(0.0493475528391)*x[1]**o+(-0.589101546403)*x[1]
        arg[2,3]=(-0.100333102737)*x[0]**o+(-0.633740106784)*x[0]+(0.408697826034)*x[1]**o+(-0.429059439472)*x[1]
        arg[2,4]=(0.635563324404)*x[0]**o+(0.955881346028)*x[0]+(0.0755013269452)*x[1]**o+(-0.743681282337)*x[1]
        arg[3,0]=(0.604891134834)*x[0]**o+(0.942047409702)*x[0]+(-0.00544373299112)*x[1]**o+(0.447523533166)*x[1]
        arg[3,1]=(-0.980967446837)*x[0]**o+(0.019824933239)*x[0]+(-0.826876930392)*x[1]**o+(0.113548968693)*x[1]
        arg[3,2]=(-0.731417395373)*x[0]**o+(-0.275072472108)*x[0]+(0.933867711492)*x[1]**o+(-0.444847972819)*x[1]
        arg[3,3]=(0.00121172413626)*x[0]**o+(-0.171073908218)*x[0]+(0.913182873344)*x[1]**o+(0.960256570926)*x[1]
        arg[3,4]=(-0.395134372727)*x[0]**o+(0.646385082409)*x[0]+(0.17387019219)*x[1]**o+(-0.287550462397)*x[1]
        ref[0,0]=(0.490709482882)*x_ref[0]**o+(0.518382584887)*x_ref[0]+(-0.940707425885)*x_ref[1]**o+(0.224578726673)*x_ref[1]
        ref[0,1]=(-0.747621141516)*x_ref[0]**o+(-0.217452489097)*x_ref[0]+(0.929120128008)*x_ref[1]**o+(0.55263700209)*x_ref[1]
        ref[0,2]=(-0.183545119672)*x_ref[0]**o+(0.148192386895)*x_ref[0]+(-0.22046309708)*x_ref[1]**o+(0.433620446668)*x_ref[1]
        ref[0,3]=(-0.0724840266959)*x_ref[0]**o+(0.835144620602)*x_ref[0]+(-0.962971534144)*x_ref[1]**o+(-0.149681811534)*x_ref[1]
        ref[0,4]=(-0.380150192228)*x_ref[0]**o+(-0.309494603814)*x_ref[0]+(-0.217425635182)*x_ref[1]**o+(0.93567286482)*x_ref[1]
        ref[1,0]=(0.0367921563947)*x_ref[0]**o+(0.220217625576)*x_ref[0]+(-0.120595819238)*x_ref[1]**o+(0.308108793279)*x_ref[1]
        ref[1,1]=(-0.142860237109)*x_ref[0]**o+(-0.789758888253)*x_ref[0]+(0.748085258229)*x_ref[1]**o+(0.784767485883)*x_ref[1]
        ref[1,2]=(-0.154690818845)*x_ref[0]**o+(-0.678458370784)*x_ref[0]+(0.750742996193)*x_ref[1]**o+(-0.350199095531)*x_ref[1]
        ref[1,3]=(-0.893295215013)*x_ref[0]**o+(0.0295805814985)*x_ref[0]+(0.265893730887)*x_ref[1]**o+(0.435487860418)*x_ref[1]
        ref[1,4]=(-0.302701704727)*x_ref[0]**o+(0.618274652266)*x_ref[0]+(-0.688386407308)*x_ref[1]**o+(-0.832515064466)*x_ref[1]
        ref[2,0]=(-0.926658745469)*x_ref[0]**o+(0.548174350496)*x_ref[0]+(-0.350700366641)*x_ref[1]**o+(0.530188602011)*x_ref[1]
        ref[2,1]=(0.900965385167)*x_ref[0]**o+(0.0234051845124)*x_ref[0]+(-0.00783834941699)*x_ref[1]**o+(-0.766352156598)*x_ref[1]
        ref[2,2]=(0.694733282444)*x_ref[0]**o+(-0.0644519234093)*x_ref[0]+(0.0493475528391)*x_ref[1]**o+(-0.589101546403)*x_ref[1]
        ref[2,3]=(-0.100333102737)*x_ref[0]**o+(-0.633740106784)*x_ref[0]+(0.408697826034)*x_ref[1]**o+(-0.429059439472)*x_ref[1]
        ref[2,4]=(0.635563324404)*x_ref[0]**o+(0.955881346028)*x_ref[0]+(0.0755013269452)*x_ref[1]**o+(-0.743681282337)*x_ref[1]
        ref[3,0]=(0.604891134834)*x_ref[0]**o+(0.942047409702)*x_ref[0]+(-0.00544373299112)*x_ref[1]**o+(0.447523533166)*x_ref[1]
        ref[3,1]=(-0.980967446837)*x_ref[0]**o+(0.019824933239)*x_ref[0]+(-0.826876930392)*x_ref[1]**o+(0.113548968693)*x_ref[1]
        ref[3,2]=(-0.731417395373)*x_ref[0]**o+(-0.275072472108)*x_ref[0]+(0.933867711492)*x_ref[1]**o+(-0.444847972819)*x_ref[1]
        ref[3,3]=(0.00121172413626)*x_ref[0]**o+(-0.171073908218)*x_ref[0]+(0.913182873344)*x_ref[1]**o+(0.960256570926)*x_ref[1]
        ref[3,4]=(-0.395134372727)*x_ref[0]**o+(0.646385082409)*x_ref[0]+(0.17387019219)*x_ref[1]**o+(-0.287550462397)*x_ref[1]
      else:
        arg[0,0]=(0.386545340192)*x[0]**o+(-0.397636296285)*x[0]+(-0.849437816516)*x[1]**o+(-0.83083406493)*x[1]+(0.479216612031)*x[2]**o+(0.0903787541308)*x[2]
        arg[0,1]=(-0.331651855726)*x[0]**o+(0.849421626918)*x[0]+(0.528368477362)*x[1]**o+(-0.015122214824)*x[1]+(0.494310578452)*x[2]**o+(0.190239928161)*x[2]
        arg[0,2]=(-0.662389759646)*x[0]**o+(0.204378343962)*x[0]+(0.486802744897)*x[1]**o+(0.822174784922)*x[1]+(-0.309203812647)*x[2]**o+(-0.101665040799)*x[2]
        arg[0,3]=(0.609106341828)*x[0]**o+(0.658404088333)*x[0]+(-0.0092991120732)*x[1]**o+(-0.73794934348)*x[1]+(0.270245135914)*x[2]**o+(-0.594067838407)*x[2]
        arg[0,4]=(-0.291752213357)*x[0]**o+(-0.596751343066)*x[0]+(-0.109196903078)*x[1]**o+(-0.868610307708)*x[1]+(0.622541308544)*x[2]**o+(0.482202762224)*x[2]
        arg[1,0]=(-0.113988933227)*x[0]**o+(-0.846328804695)*x[0]+(0.691769553626)*x[1]**o+(-0.377441645148)*x[1]+(-0.769857970682)*x[2]**o+(-0.583083729037)*x[2]
        arg[1,1]=(-0.635333757145)*x[0]**o+(0.194761117849)*x[0]+(-0.268287296731)*x[1]**o+(0.463906930647)*x[1]+(0.768293821029)*x[2]**o+(-0.532824906687)*x[2]
        arg[1,2]=(0.355380090138)*x[0]**o+(-0.181191575848)*x[0]+(-0.558667766467)*x[1]**o+(0.561707671872)*x[1]+(-0.427004232725)*x[2]**o+(-0.996564299989)*x[2]
        arg[1,3]=(-0.891638892893)*x[0]**o+(0.0867350850512)*x[0]+(0.880258390584)*x[1]**o+(-0.798123407525)*x[1]+(-0.571505202201)*x[2]**o+(0.35374245063)*x[2]
        arg[1,4]=(0.543632496941)*x[0]**o+(-0.630363803582)*x[0]+(0.38107312058)*x[1]**o+(0.222401831883)*x[1]+(-0.932212080175)*x[2]**o+(-0.0482899826174)*x[2]
        arg[2,0]=(0.46514308325)*x[0]**o+(-0.907464652384)*x[0]+(-0.706105595148)*x[1]**o+(0.486989958675)*x[1]+(0.422702683485)*x[2]**o+(0.640916925932)*x[2]
        arg[2,1]=(0.912808317479)*x[0]**o+(-0.860779312763)*x[0]+(-0.915364956133)*x[1]**o+(-0.314186117224)*x[1]+(-0.860814410035)*x[2]**o+(0.837570442722)*x[2]
        arg[2,2]=(0.00932917919464)*x[0]**o+(0.393773283049)*x[0]+(0.699998572668)*x[1]**o+(0.153407885795)*x[1]+(-0.166245189281)*x[2]**o+(-0.430541214236)*x[2]
        arg[2,3]=(-0.00127511229816)*x[0]**o+(-0.327443174924)*x[0]+(0.61881322537)*x[1]**o+(-0.725854102962)*x[1]+(0.650649062427)*x[2]**o+(0.246373184041)*x[2]
        arg[2,4]=(-0.530580898049)*x[0]**o+(-0.208506132901)*x[0]+(-0.102049332015)*x[1]**o+(-0.537037644189)*x[1]+(0.740113470729)*x[2]**o+(0.12094405264)*x[2]
        arg[3,0]=(-0.103423602042)*x[0]**o+(-0.535742323666)*x[0]+(0.678663019405)*x[1]**o+(0.579375950057)*x[1]+(0.323627490052)*x[2]**o+(-0.681323301356)*x[2]
        arg[3,1]=(0.670239087519)*x[0]**o+(0.209657968128)*x[0]+(-0.357490300727)*x[1]**o+(-0.441776782321)*x[1]+(0.00927389037562)*x[2]**o+(0.633527599723)*x[2]
        arg[3,2]=(0.415569606138)*x[0]**o+(-0.422211616068)*x[0]+(-0.695086235645)*x[1]**o+(0.709158289848)*x[1]+(-0.523450845183)*x[2]**o+(0.555778994398)*x[2]
        arg[3,3]=(0.0188993892075)*x[0]**o+(0.611088737828)*x[0]+(0.0313795167921)*x[1]**o+(0.0842651755016)*x[1]+(0.288414199415)*x[2]**o+(0.695910780235)*x[2]
        arg[3,4]=(-0.500338800879)*x[0]**o+(0.0984192369328)*x[0]+(0.71163172992)*x[1]**o+(-0.102123116139)*x[1]+(0.954613955678)*x[2]**o+(0.384872409422)*x[2]
        ref[0,0]=(0.386545340192)*x_ref[0]**o+(-0.397636296285)*x_ref[0]+(-0.849437816516)*x_ref[1]**o+(-0.83083406493)*x_ref[1]+(0.479216612031)*x_ref[2]**o+(0.0903787541308)*x_ref[2]
        ref[0,1]=(-0.331651855726)*x_ref[0]**o+(0.849421626918)*x_ref[0]+(0.528368477362)*x_ref[1]**o+(-0.015122214824)*x_ref[1]+(0.494310578452)*x_ref[2]**o+(0.190239928161)*x_ref[2]
        ref[0,2]=(-0.662389759646)*x_ref[0]**o+(0.204378343962)*x_ref[0]+(0.486802744897)*x_ref[1]**o+(0.822174784922)*x_ref[1]+(-0.309203812647)*x_ref[2]**o+(-0.101665040799)*x_ref[2]
        ref[0,3]=(0.609106341828)*x_ref[0]**o+(0.658404088333)*x_ref[0]+(-0.0092991120732)*x_ref[1]**o+(-0.73794934348)*x_ref[1]+(0.270245135914)*x_ref[2]**o+(-0.594067838407)*x_ref[2]
        ref[0,4]=(-0.291752213357)*x_ref[0]**o+(-0.596751343066)*x_ref[0]+(-0.109196903078)*x_ref[1]**o+(-0.868610307708)*x_ref[1]+(0.622541308544)*x_ref[2]**o+(0.482202762224)*x_ref[2]
        ref[1,0]=(-0.113988933227)*x_ref[0]**o+(-0.846328804695)*x_ref[0]+(0.691769553626)*x_ref[1]**o+(-0.377441645148)*x_ref[1]+(-0.769857970682)*x_ref[2]**o+(-0.583083729037)*x_ref[2]
        ref[1,1]=(-0.635333757145)*x_ref[0]**o+(0.194761117849)*x_ref[0]+(-0.268287296731)*x_ref[1]**o+(0.463906930647)*x_ref[1]+(0.768293821029)*x_ref[2]**o+(-0.532824906687)*x_ref[2]
        ref[1,2]=(0.355380090138)*x_ref[0]**o+(-0.181191575848)*x_ref[0]+(-0.558667766467)*x_ref[1]**o+(0.561707671872)*x_ref[1]+(-0.427004232725)*x_ref[2]**o+(-0.996564299989)*x_ref[2]
        ref[1,3]=(-0.891638892893)*x_ref[0]**o+(0.0867350850512)*x_ref[0]+(0.880258390584)*x_ref[1]**o+(-0.798123407525)*x_ref[1]+(-0.571505202201)*x_ref[2]**o+(0.35374245063)*x_ref[2]
        ref[1,4]=(0.543632496941)*x_ref[0]**o+(-0.630363803582)*x_ref[0]+(0.38107312058)*x_ref[1]**o+(0.222401831883)*x_ref[1]+(-0.932212080175)*x_ref[2]**o+(-0.0482899826174)*x_ref[2]
        ref[2,0]=(0.46514308325)*x_ref[0]**o+(-0.907464652384)*x_ref[0]+(-0.706105595148)*x_ref[1]**o+(0.486989958675)*x_ref[1]+(0.422702683485)*x_ref[2]**o+(0.640916925932)*x_ref[2]
        ref[2,1]=(0.912808317479)*x_ref[0]**o+(-0.860779312763)*x_ref[0]+(-0.915364956133)*x_ref[1]**o+(-0.314186117224)*x_ref[1]+(-0.860814410035)*x_ref[2]**o+(0.837570442722)*x_ref[2]
        ref[2,2]=(0.00932917919464)*x_ref[0]**o+(0.393773283049)*x_ref[0]+(0.699998572668)*x_ref[1]**o+(0.153407885795)*x_ref[1]+(-0.166245189281)*x_ref[2]**o+(-0.430541214236)*x_ref[2]
        ref[2,3]=(-0.00127511229816)*x_ref[0]**o+(-0.327443174924)*x_ref[0]+(0.61881322537)*x_ref[1]**o+(-0.725854102962)*x_ref[1]+(0.650649062427)*x_ref[2]**o+(0.246373184041)*x_ref[2]
        ref[2,4]=(-0.530580898049)*x_ref[0]**o+(-0.208506132901)*x_ref[0]+(-0.102049332015)*x_ref[1]**o+(-0.537037644189)*x_ref[1]+(0.740113470729)*x_ref[2]**o+(0.12094405264)*x_ref[2]
        ref[3,0]=(-0.103423602042)*x_ref[0]**o+(-0.535742323666)*x_ref[0]+(0.678663019405)*x_ref[1]**o+(0.579375950057)*x_ref[1]+(0.323627490052)*x_ref[2]**o+(-0.681323301356)*x_ref[2]
        ref[3,1]=(0.670239087519)*x_ref[0]**o+(0.209657968128)*x_ref[0]+(-0.357490300727)*x_ref[1]**o+(-0.441776782321)*x_ref[1]+(0.00927389037562)*x_ref[2]**o+(0.633527599723)*x_ref[2]
        ref[3,2]=(0.415569606138)*x_ref[0]**o+(-0.422211616068)*x_ref[0]+(-0.695086235645)*x_ref[1]**o+(0.709158289848)*x_ref[1]+(-0.523450845183)*x_ref[2]**o+(0.555778994398)*x_ref[2]
        ref[3,3]=(0.0188993892075)*x_ref[0]**o+(0.611088737828)*x_ref[0]+(0.0313795167921)*x_ref[1]**o+(0.0842651755016)*x_ref[1]+(0.288414199415)*x_ref[2]**o+(0.695910780235)*x_ref[2]
        ref[3,4]=(-0.500338800879)*x_ref[0]**o+(0.0984192369328)*x_ref[0]+(0.71163172992)*x_ref[1]**o+(-0.102123116139)*x_ref[1]+(0.954613955678)*x_ref[2]**o+(0.384872409422)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_Solution_rank3(self):
      """
      tests interpolate for rank 3 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from Solution onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2),w_ref)
      if dim==2:
        arg[0,0,0]=(0.317274364174)*x[0]**o+(-0.565307843766)*x[0]+(-0.367874638208)*x[1]**o+(0.0029581033546)*x[1]
        arg[0,0,1]=(0.832728755858)*x[0]**o+(0.245271193641)*x[0]+(-0.727130029377)*x[1]**o+(0.137912975829)*x[1]
        arg[0,1,0]=(-0.634508968932)*x[0]**o+(-0.864200861637)*x[0]+(-0.566985971867)*x[1]**o+(0.458187145519)*x[1]
        arg[0,1,1]=(0.133772683885)*x[0]**o+(-0.630800341215)*x[0]+(0.958934971335)*x[1]**o+(-0.15839182257)*x[1]
        arg[1,0,0]=(-0.0948724752087)*x[0]**o+(-0.25589337469)*x[0]+(-0.315713312867)*x[1]**o+(0.223487524432)*x[1]
        arg[1,0,1]=(0.794634271698)*x[0]**o+(-0.505797079712)*x[0]+(0.54367349338)*x[1]**o+(0.610149581933)*x[1]
        arg[1,1,0]=(-0.447039667341)*x[0]**o+(-0.917009838715)*x[0]+(0.525331113522)*x[1]**o+(-0.626176620973)*x[1]
        arg[1,1,1]=(0.0749028334839)*x[0]**o+(0.106950341725)*x[0]+(-0.375321288987)*x[1]**o+(0.510260762626)*x[1]
        arg[2,0,0]=(-0.286661970669)*x[0]**o+(0.584169107201)*x[0]+(0.189852415568)*x[1]**o+(-0.685695910848)*x[1]
        arg[2,0,1]=(-0.689509796724)*x[0]**o+(-0.316067007053)*x[0]+(0.93758916753)*x[1]**o+(0.987368705364)*x[1]
        arg[2,1,0]=(-0.411695937554)*x[0]**o+(0.858528628791)*x[0]+(-0.579361651418)*x[1]**o+(-0.179617869494)*x[1]
        arg[2,1,1]=(0.871441175049)*x[0]**o+(0.445176827889)*x[0]+(0.282613647019)*x[1]**o+(-0.0414230375843)*x[1]
        arg[3,0,0]=(-0.242136375691)*x[0]**o+(-0.91917484201)*x[0]+(-0.88026838607)*x[1]**o+(0.363309786006)*x[1]
        arg[3,0,1]=(-0.991221762386)*x[0]**o+(-0.0149280738883)*x[0]+(0.420322977563)*x[1]**o+(-0.0533076687981)*x[1]
        arg[3,1,0]=(-0.889003524303)*x[0]**o+(-0.023519502168)*x[0]+(0.826015855889)*x[1]**o+(0.0568467842617)*x[1]
        arg[3,1,1]=(0.608567002824)*x[0]**o+(-0.531288593441)*x[0]+(0.0886678034719)*x[1]**o+(-0.732353220319)*x[1]
        arg[4,0,0]=(-0.0537755786538)*x[0]**o+(-0.475886396481)*x[0]+(-0.153787571145)*x[1]**o+(0.689850737569)*x[1]
        arg[4,0,1]=(0.413017674549)*x[0]**o+(-0.993353261494)*x[0]+(-0.944597571268)*x[1]**o+(0.566423551758)*x[1]
        arg[4,1,0]=(-0.716864417268)*x[0]**o+(-0.871574272865)*x[0]+(0.394071572628)*x[1]**o+(0.452903048597)*x[1]
        arg[4,1,1]=(0.861790785883)*x[0]**o+(-0.511789401943)*x[0]+(-0.272452527007)*x[1]**o+(-0.379021521619)*x[1]
        arg[5,0,0]=(0.509668147054)*x[0]**o+(0.919355584411)*x[0]+(-0.673215166917)*x[1]**o+(0.766459631931)*x[1]
        arg[5,0,1]=(0.177006227916)*x[0]**o+(-0.275118477)*x[0]+(0.0302367843701)*x[1]**o+(-0.761226360328)*x[1]
        arg[5,1,0]=(0.013432534638)*x[0]**o+(-0.911626567027)*x[0]+(-0.710520442529)*x[1]**o+(-0.882317448423)*x[1]
        arg[5,1,1]=(0.595583093615)*x[0]**o+(0.163527905156)*x[0]+(-0.122165051333)*x[1]**o+(-0.526359288181)*x[1]
        ref[0,0,0]=(0.317274364174)*x_ref[0]**o+(-0.565307843766)*x_ref[0]+(-0.367874638208)*x_ref[1]**o+(0.0029581033546)*x_ref[1]
        ref[0,0,1]=(0.832728755858)*x_ref[0]**o+(0.245271193641)*x_ref[0]+(-0.727130029377)*x_ref[1]**o+(0.137912975829)*x_ref[1]
        ref[0,1,0]=(-0.634508968932)*x_ref[0]**o+(-0.864200861637)*x_ref[0]+(-0.566985971867)*x_ref[1]**o+(0.458187145519)*x_ref[1]
        ref[0,1,1]=(0.133772683885)*x_ref[0]**o+(-0.630800341215)*x_ref[0]+(0.958934971335)*x_ref[1]**o+(-0.15839182257)*x_ref[1]
        ref[1,0,0]=(-0.0948724752087)*x_ref[0]**o+(-0.25589337469)*x_ref[0]+(-0.315713312867)*x_ref[1]**o+(0.223487524432)*x_ref[1]
        ref[1,0,1]=(0.794634271698)*x_ref[0]**o+(-0.505797079712)*x_ref[0]+(0.54367349338)*x_ref[1]**o+(0.610149581933)*x_ref[1]
        ref[1,1,0]=(-0.447039667341)*x_ref[0]**o+(-0.917009838715)*x_ref[0]+(0.525331113522)*x_ref[1]**o+(-0.626176620973)*x_ref[1]
        ref[1,1,1]=(0.0749028334839)*x_ref[0]**o+(0.106950341725)*x_ref[0]+(-0.375321288987)*x_ref[1]**o+(0.510260762626)*x_ref[1]
        ref[2,0,0]=(-0.286661970669)*x_ref[0]**o+(0.584169107201)*x_ref[0]+(0.189852415568)*x_ref[1]**o+(-0.685695910848)*x_ref[1]
        ref[2,0,1]=(-0.689509796724)*x_ref[0]**o+(-0.316067007053)*x_ref[0]+(0.93758916753)*x_ref[1]**o+(0.987368705364)*x_ref[1]
        ref[2,1,0]=(-0.411695937554)*x_ref[0]**o+(0.858528628791)*x_ref[0]+(-0.579361651418)*x_ref[1]**o+(-0.179617869494)*x_ref[1]
        ref[2,1,1]=(0.871441175049)*x_ref[0]**o+(0.445176827889)*x_ref[0]+(0.282613647019)*x_ref[1]**o+(-0.0414230375843)*x_ref[1]
        ref[3,0,0]=(-0.242136375691)*x_ref[0]**o+(-0.91917484201)*x_ref[0]+(-0.88026838607)*x_ref[1]**o+(0.363309786006)*x_ref[1]
        ref[3,0,1]=(-0.991221762386)*x_ref[0]**o+(-0.0149280738883)*x_ref[0]+(0.420322977563)*x_ref[1]**o+(-0.0533076687981)*x_ref[1]
        ref[3,1,0]=(-0.889003524303)*x_ref[0]**o+(-0.023519502168)*x_ref[0]+(0.826015855889)*x_ref[1]**o+(0.0568467842617)*x_ref[1]
        ref[3,1,1]=(0.608567002824)*x_ref[0]**o+(-0.531288593441)*x_ref[0]+(0.0886678034719)*x_ref[1]**o+(-0.732353220319)*x_ref[1]
        ref[4,0,0]=(-0.0537755786538)*x_ref[0]**o+(-0.475886396481)*x_ref[0]+(-0.153787571145)*x_ref[1]**o+(0.689850737569)*x_ref[1]
        ref[4,0,1]=(0.413017674549)*x_ref[0]**o+(-0.993353261494)*x_ref[0]+(-0.944597571268)*x_ref[1]**o+(0.566423551758)*x_ref[1]
        ref[4,1,0]=(-0.716864417268)*x_ref[0]**o+(-0.871574272865)*x_ref[0]+(0.394071572628)*x_ref[1]**o+(0.452903048597)*x_ref[1]
        ref[4,1,1]=(0.861790785883)*x_ref[0]**o+(-0.511789401943)*x_ref[0]+(-0.272452527007)*x_ref[1]**o+(-0.379021521619)*x_ref[1]
        ref[5,0,0]=(0.509668147054)*x_ref[0]**o+(0.919355584411)*x_ref[0]+(-0.673215166917)*x_ref[1]**o+(0.766459631931)*x_ref[1]
        ref[5,0,1]=(0.177006227916)*x_ref[0]**o+(-0.275118477)*x_ref[0]+(0.0302367843701)*x_ref[1]**o+(-0.761226360328)*x_ref[1]
        ref[5,1,0]=(0.013432534638)*x_ref[0]**o+(-0.911626567027)*x_ref[0]+(-0.710520442529)*x_ref[1]**o+(-0.882317448423)*x_ref[1]
        ref[5,1,1]=(0.595583093615)*x_ref[0]**o+(0.163527905156)*x_ref[0]+(-0.122165051333)*x_ref[1]**o+(-0.526359288181)*x_ref[1]
      else:
        arg[0,0,0]=(0.230695674262)*x[0]**o+(-0.987212801716)*x[0]+(-0.0599001434987)*x[1]**o+(0.719591259555)*x[1]+(-0.665186369759)*x[2]**o+(-0.563552602515)*x[2]
        arg[0,0,1]=(-0.775441966131)*x[0]**o+(-0.152166849702)*x[0]+(0.626466789363)*x[1]**o+(0.179356346581)*x[1]+(-0.687406927434)*x[2]**o+(-0.563492313114)*x[2]
        arg[0,1,0]=(0.749522748133)*x[0]**o+(-0.582456308337)*x[0]+(0.676882643058)*x[1]**o+(0.365363182598)*x[1]+(-0.550867708812)*x[2]**o+(0.777938151183)*x[2]
        arg[0,1,1]=(0.389788552775)*x[0]**o+(-0.298877489997)*x[0]+(0.560326209269)*x[1]**o+(-0.113249832155)*x[1]+(0.588087510254)*x[2]**o+(-0.593503776461)*x[2]
        arg[1,0,0]=(-0.354661103909)*x[0]**o+(-0.195375369549)*x[0]+(-0.798795069895)*x[1]**o+(0.0968631048579)*x[1]+(-0.016118190391)*x[2]**o+(-0.903745407705)*x[2]
        arg[1,0,1]=(-0.383688834784)*x[0]**o+(0.861256231933)*x[0]+(-0.767204834667)*x[1]**o+(0.338875482372)*x[1]+(0.117146310891)*x[2]**o+(-0.302651846083)*x[2]
        arg[1,1,0]=(0.646263670969)*x[0]**o+(-0.447984008871)*x[0]+(-0.169322492785)*x[1]**o+(-0.570614724481)*x[1]+(-0.848518457618)*x[2]**o+(-0.0750005539216)*x[2]
        arg[1,1,1]=(-0.232496745304)*x[0]**o+(0.182581173413)*x[0]+(0.8032436357)*x[1]**o+(-0.391266449476)*x[1]+(0.491677629766)*x[2]**o+(-0.188361708708)*x[2]
        arg[2,0,0]=(-0.571145099091)*x[0]**o+(-0.486855242188)*x[0]+(0.52833582073)*x[1]**o+(0.326716348242)*x[1]+(0.528475192869)*x[2]**o+(0.163280575137)*x[2]
        arg[2,0,1]=(-0.368889674601)*x[0]**o+(0.601775816775)*x[0]+(0.0281742455403)*x[1]**o+(-0.198869716243)*x[1]+(0.193343065506)*x[2]**o+(-0.0614451571108)*x[2]
        arg[2,1,0]=(0.724595792492)*x[0]**o+(-0.206983407651)*x[0]+(-0.33966393719)*x[1]**o+(0.929465830796)*x[1]+(-0.938517195863)*x[2]**o+(0.70874077222)*x[2]
        arg[2,1,1]=(-0.536303362606)*x[0]**o+(-0.430663025812)*x[0]+(-0.0845923469062)*x[1]**o+(0.866655631144)*x[1]+(-0.2378102403)*x[2]**o+(0.969070797268)*x[2]
        arg[3,0,0]=(0.345137352559)*x[0]**o+(-0.696652518382)*x[0]+(0.914998974008)*x[1]**o+(-0.874935163156)*x[1]+(0.947812661283)*x[2]**o+(0.868611662789)*x[2]
        arg[3,0,1]=(-0.421640746358)*x[0]**o+(-0.37925991208)*x[0]+(0.468281241452)*x[1]**o+(0.439086689805)*x[1]+(-0.607986317001)*x[2]**o+(0.269488168937)*x[2]
        arg[3,1,0]=(-0.550485456473)*x[0]**o+(-0.520018714495)*x[0]+(0.439679309654)*x[1]**o+(-0.0652434084668)*x[1]+(-0.584838668708)*x[2]**o+(-0.46047281034)*x[2]
        arg[3,1,1]=(-0.421746456038)*x[0]**o+(0.370489549278)*x[0]+(-0.683682295105)*x[1]**o+(0.324051718035)*x[1]+(-0.579046430636)*x[2]**o+(0.587623604941)*x[2]
        arg[4,0,0]=(-0.167582584882)*x[0]**o+(0.524904122243)*x[0]+(0.665109348362)*x[1]**o+(-0.093655366509)*x[1]+(0.704404656254)*x[2]**o+(-0.402602441741)*x[2]
        arg[4,0,1]=(0.633854964824)*x[0]**o+(-0.836385840194)*x[0]+(-0.0995437996628)*x[1]**o+(-0.200285338869)*x[1]+(0.535023783881)*x[2]**o+(0.884702543593)*x[2]
        arg[4,1,0]=(-0.953619378208)*x[0]**o+(-0.390518815669)*x[0]+(-0.0590889178236)*x[1]**o+(-0.997225825019)*x[1]+(0.0520418736402)*x[2]**o+(0.484749465262)*x[2]
        arg[4,1,1]=(-0.440389854756)*x[0]**o+(0.15520431778)*x[0]+(0.346839174513)*x[1]**o+(0.0172676873159)*x[1]+(0.86275919224)*x[2]**o+(0.568069148992)*x[2]
        arg[5,0,0]=(-0.303687941217)*x[0]**o+(0.516115915687)*x[0]+(-0.750276597519)*x[1]**o+(0.734039058778)*x[1]+(-0.624499210252)*x[2]**o+(-0.926338135421)*x[2]
        arg[5,0,1]=(-0.472230777895)*x[0]**o+(-0.557827728373)*x[0]+(-0.403783023157)*x[1]**o+(0.70433336955)*x[1]+(-0.0211737454531)*x[2]**o+(0.0774406125836)*x[2]
        arg[5,1,0]=(0.718149875204)*x[0]**o+(-0.183000739787)*x[0]+(-0.655724252373)*x[1]**o+(-0.634941710776)*x[1]+(-0.26300163323)*x[2]**o+(-0.25105424715)*x[2]
        arg[5,1,1]=(0.109601904441)*x[0]**o+(-0.00640764631554)*x[0]+(-0.488035610009)*x[1]**o+(0.909668323769)*x[1]+(0.229692168501)*x[2]**o+(-0.739241309497)*x[2]
        ref[0,0,0]=(0.230695674262)*x_ref[0]**o+(-0.987212801716)*x_ref[0]+(-0.0599001434987)*x_ref[1]**o+(0.719591259555)*x_ref[1]+(-0.665186369759)*x_ref[2]**o+(-0.563552602515)*x_ref[2]
        ref[0,0,1]=(-0.775441966131)*x_ref[0]**o+(-0.152166849702)*x_ref[0]+(0.626466789363)*x_ref[1]**o+(0.179356346581)*x_ref[1]+(-0.687406927434)*x_ref[2]**o+(-0.563492313114)*x_ref[2]
        ref[0,1,0]=(0.749522748133)*x_ref[0]**o+(-0.582456308337)*x_ref[0]+(0.676882643058)*x_ref[1]**o+(0.365363182598)*x_ref[1]+(-0.550867708812)*x_ref[2]**o+(0.777938151183)*x_ref[2]
        ref[0,1,1]=(0.389788552775)*x_ref[0]**o+(-0.298877489997)*x_ref[0]+(0.560326209269)*x_ref[1]**o+(-0.113249832155)*x_ref[1]+(0.588087510254)*x_ref[2]**o+(-0.593503776461)*x_ref[2]
        ref[1,0,0]=(-0.354661103909)*x_ref[0]**o+(-0.195375369549)*x_ref[0]+(-0.798795069895)*x_ref[1]**o+(0.0968631048579)*x_ref[1]+(-0.016118190391)*x_ref[2]**o+(-0.903745407705)*x_ref[2]
        ref[1,0,1]=(-0.383688834784)*x_ref[0]**o+(0.861256231933)*x_ref[0]+(-0.767204834667)*x_ref[1]**o+(0.338875482372)*x_ref[1]+(0.117146310891)*x_ref[2]**o+(-0.302651846083)*x_ref[2]
        ref[1,1,0]=(0.646263670969)*x_ref[0]**o+(-0.447984008871)*x_ref[0]+(-0.169322492785)*x_ref[1]**o+(-0.570614724481)*x_ref[1]+(-0.848518457618)*x_ref[2]**o+(-0.0750005539216)*x_ref[2]
        ref[1,1,1]=(-0.232496745304)*x_ref[0]**o+(0.182581173413)*x_ref[0]+(0.8032436357)*x_ref[1]**o+(-0.391266449476)*x_ref[1]+(0.491677629766)*x_ref[2]**o+(-0.188361708708)*x_ref[2]
        ref[2,0,0]=(-0.571145099091)*x_ref[0]**o+(-0.486855242188)*x_ref[0]+(0.52833582073)*x_ref[1]**o+(0.326716348242)*x_ref[1]+(0.528475192869)*x_ref[2]**o+(0.163280575137)*x_ref[2]
        ref[2,0,1]=(-0.368889674601)*x_ref[0]**o+(0.601775816775)*x_ref[0]+(0.0281742455403)*x_ref[1]**o+(-0.198869716243)*x_ref[1]+(0.193343065506)*x_ref[2]**o+(-0.0614451571108)*x_ref[2]
        ref[2,1,0]=(0.724595792492)*x_ref[0]**o+(-0.206983407651)*x_ref[0]+(-0.33966393719)*x_ref[1]**o+(0.929465830796)*x_ref[1]+(-0.938517195863)*x_ref[2]**o+(0.70874077222)*x_ref[2]
        ref[2,1,1]=(-0.536303362606)*x_ref[0]**o+(-0.430663025812)*x_ref[0]+(-0.0845923469062)*x_ref[1]**o+(0.866655631144)*x_ref[1]+(-0.2378102403)*x_ref[2]**o+(0.969070797268)*x_ref[2]
        ref[3,0,0]=(0.345137352559)*x_ref[0]**o+(-0.696652518382)*x_ref[0]+(0.914998974008)*x_ref[1]**o+(-0.874935163156)*x_ref[1]+(0.947812661283)*x_ref[2]**o+(0.868611662789)*x_ref[2]
        ref[3,0,1]=(-0.421640746358)*x_ref[0]**o+(-0.37925991208)*x_ref[0]+(0.468281241452)*x_ref[1]**o+(0.439086689805)*x_ref[1]+(-0.607986317001)*x_ref[2]**o+(0.269488168937)*x_ref[2]
        ref[3,1,0]=(-0.550485456473)*x_ref[0]**o+(-0.520018714495)*x_ref[0]+(0.439679309654)*x_ref[1]**o+(-0.0652434084668)*x_ref[1]+(-0.584838668708)*x_ref[2]**o+(-0.46047281034)*x_ref[2]
        ref[3,1,1]=(-0.421746456038)*x_ref[0]**o+(0.370489549278)*x_ref[0]+(-0.683682295105)*x_ref[1]**o+(0.324051718035)*x_ref[1]+(-0.579046430636)*x_ref[2]**o+(0.587623604941)*x_ref[2]
        ref[4,0,0]=(-0.167582584882)*x_ref[0]**o+(0.524904122243)*x_ref[0]+(0.665109348362)*x_ref[1]**o+(-0.093655366509)*x_ref[1]+(0.704404656254)*x_ref[2]**o+(-0.402602441741)*x_ref[2]
        ref[4,0,1]=(0.633854964824)*x_ref[0]**o+(-0.836385840194)*x_ref[0]+(-0.0995437996628)*x_ref[1]**o+(-0.200285338869)*x_ref[1]+(0.535023783881)*x_ref[2]**o+(0.884702543593)*x_ref[2]
        ref[4,1,0]=(-0.953619378208)*x_ref[0]**o+(-0.390518815669)*x_ref[0]+(-0.0590889178236)*x_ref[1]**o+(-0.997225825019)*x_ref[1]+(0.0520418736402)*x_ref[2]**o+(0.484749465262)*x_ref[2]
        ref[4,1,1]=(-0.440389854756)*x_ref[0]**o+(0.15520431778)*x_ref[0]+(0.346839174513)*x_ref[1]**o+(0.0172676873159)*x_ref[1]+(0.86275919224)*x_ref[2]**o+(0.568069148992)*x_ref[2]
        ref[5,0,0]=(-0.303687941217)*x_ref[0]**o+(0.516115915687)*x_ref[0]+(-0.750276597519)*x_ref[1]**o+(0.734039058778)*x_ref[1]+(-0.624499210252)*x_ref[2]**o+(-0.926338135421)*x_ref[2]
        ref[5,0,1]=(-0.472230777895)*x_ref[0]**o+(-0.557827728373)*x_ref[0]+(-0.403783023157)*x_ref[1]**o+(0.70433336955)*x_ref[1]+(-0.0211737454531)*x_ref[2]**o+(0.0774406125836)*x_ref[2]
        ref[5,1,0]=(0.718149875204)*x_ref[0]**o+(-0.183000739787)*x_ref[0]+(-0.655724252373)*x_ref[1]**o+(-0.634941710776)*x_ref[1]+(-0.26300163323)*x_ref[2]**o+(-0.25105424715)*x_ref[2]
        ref[5,1,1]=(0.109601904441)*x_ref[0]**o+(-0.00640764631554)*x_ref[0]+(-0.488035610009)*x_ref[1]**o+(0.909668323769)*x_ref[1]+(0.229692168501)*x_ref[2]**o+(-0.739241309497)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_Solution_rank4(self):
      """
      tests interpolate for rank 4 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from Solution onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5, 3, 2),w)
      ref=Data(0,(4, 5, 3, 2),w_ref)
      if dim==2:
        arg[0,0,0,0]=(0.912518456494)*x[0]**o+(-0.828176151903)*x[0]+(-0.313940298121)*x[1]**o+(-0.850120336531)*x[1]
        arg[0,0,0,1]=(0.14415852783)*x[0]**o+(-0.294155246668)*x[0]+(0.562517248232)*x[1]**o+(-0.204505938286)*x[1]
        arg[0,0,1,0]=(0.555455404368)*x[0]**o+(0.916425343576)*x[0]+(0.547252315693)*x[1]**o+(-0.0992842367479)*x[1]
        arg[0,0,1,1]=(0.603632473928)*x[0]**o+(0.505571549062)*x[0]+(-0.490402921542)*x[1]**o+(0.908624427316)*x[1]
        arg[0,0,2,0]=(0.729542308985)*x[0]**o+(-0.654310868638)*x[0]+(0.0845981276245)*x[1]**o+(-0.95231604686)*x[1]
        arg[0,0,2,1]=(0.0511615606745)*x[0]**o+(-0.939469887975)*x[0]+(-0.206229418155)*x[1]**o+(0.254879844701)*x[1]
        arg[0,1,0,0]=(0.257917344507)*x[0]**o+(0.860729621288)*x[0]+(0.489966193348)*x[1]**o+(0.174897233858)*x[1]
        arg[0,1,0,1]=(0.336588362051)*x[0]**o+(-0.690598635114)*x[0]+(-0.0913735300483)*x[1]**o+(-0.932914439004)*x[1]
        arg[0,1,1,0]=(0.244583093108)*x[0]**o+(-0.827768167758)*x[0]+(0.573065359412)*x[1]**o+(0.176685515596)*x[1]
        arg[0,1,1,1]=(-0.520550726127)*x[0]**o+(-0.605243177286)*x[0]+(0.130549556226)*x[1]**o+(0.750019884067)*x[1]
        arg[0,1,2,0]=(-0.680201627614)*x[0]**o+(0.679925324882)*x[0]+(-0.060408604263)*x[1]**o+(0.568125689359)*x[1]
        arg[0,1,2,1]=(-0.111908859432)*x[0]**o+(0.256521037018)*x[0]+(0.332050616611)*x[1]**o+(-0.0500016397683)*x[1]
        arg[0,2,0,0]=(-0.484953933663)*x[0]**o+(0.628032211831)*x[0]+(-0.63297141454)*x[1]**o+(0.141016662755)*x[1]
        arg[0,2,0,1]=(-0.463920680465)*x[0]**o+(-0.868825520511)*x[0]+(-0.36594425006)*x[1]**o+(-0.823052216578)*x[1]
        arg[0,2,1,0]=(0.972337802357)*x[0]**o+(0.943605157343)*x[0]+(-0.113950717844)*x[1]**o+(-0.839596452897)*x[1]
        arg[0,2,1,1]=(-0.295562401302)*x[0]**o+(0.735770621017)*x[0]+(-0.183838912348)*x[1]**o+(-0.929365380252)*x[1]
        arg[0,2,2,0]=(0.912886298388)*x[0]**o+(0.847507982065)*x[0]+(-0.341306036323)*x[1]**o+(0.393627025978)*x[1]
        arg[0,2,2,1]=(-0.769650638386)*x[0]**o+(-0.619672356488)*x[0]+(-0.126894288906)*x[1]**o+(0.231547632139)*x[1]
        arg[0,3,0,0]=(-0.884442976834)*x[0]**o+(-0.0194505760676)*x[0]+(-0.338764675072)*x[1]**o+(0.723481600171)*x[1]
        arg[0,3,0,1]=(0.655973555091)*x[0]**o+(-0.556230649412)*x[0]+(-0.850356832441)*x[1]**o+(0.115267314853)*x[1]
        arg[0,3,1,0]=(0.659628443559)*x[0]**o+(-0.586677822491)*x[0]+(0.135930588518)*x[1]**o+(0.206866895948)*x[1]
        arg[0,3,1,1]=(-0.700258957557)*x[0]**o+(-0.699774198764)*x[0]+(-0.294009479186)*x[1]**o+(0.848159666503)*x[1]
        arg[0,3,2,0]=(-0.927501754488)*x[0]**o+(-0.981178071331)*x[0]+(0.614074194451)*x[1]**o+(0.711549573086)*x[1]
        arg[0,3,2,1]=(0.312434473198)*x[0]**o+(-0.241506125991)*x[0]+(-0.0840979088912)*x[1]**o+(0.902948463117)*x[1]
        arg[0,4,0,0]=(-0.892269860242)*x[0]**o+(0.823434435855)*x[0]+(0.50499383365)*x[1]**o+(0.763736541034)*x[1]
        arg[0,4,0,1]=(-0.606959691191)*x[0]**o+(0.945562536791)*x[0]+(-0.46271929481)*x[1]**o+(0.929514432096)*x[1]
        arg[0,4,1,0]=(0.953382685939)*x[0]**o+(0.553766359805)*x[0]+(-0.191004010293)*x[1]**o+(0.556369491698)*x[1]
        arg[0,4,1,1]=(-0.269402649165)*x[0]**o+(-0.158926232076)*x[0]+(-0.390038749681)*x[1]**o+(0.553655192666)*x[1]
        arg[0,4,2,0]=(0.0785586372006)*x[0]**o+(-0.778452221862)*x[0]+(-0.927627273292)*x[1]**o+(-0.187742405246)*x[1]
        arg[0,4,2,1]=(-0.965329945447)*x[0]**o+(0.304539006089)*x[0]+(0.213031689568)*x[1]**o+(-0.2807790149)*x[1]
        arg[1,0,0,0]=(0.981534698569)*x[0]**o+(0.466451685806)*x[0]+(0.845322931856)*x[1]**o+(0.582422032334)*x[1]
        arg[1,0,0,1]=(-0.0645798740394)*x[0]**o+(0.0475214206619)*x[0]+(-0.687759433076)*x[1]**o+(0.403440938285)*x[1]
        arg[1,0,1,0]=(-0.317887773392)*x[0]**o+(0.59101163416)*x[0]+(0.658626263291)*x[1]**o+(-0.618167594137)*x[1]
        arg[1,0,1,1]=(-0.314093092813)*x[0]**o+(0.699933899942)*x[0]+(0.824195260822)*x[1]**o+(-0.42875327345)*x[1]
        arg[1,0,2,0]=(-0.254254956887)*x[0]**o+(-0.912137080163)*x[0]+(0.360457121452)*x[1]**o+(-0.902596912588)*x[1]
        arg[1,0,2,1]=(-0.577214593683)*x[0]**o+(-0.419495663008)*x[0]+(-0.803459783331)*x[1]**o+(0.490885513568)*x[1]
        arg[1,1,0,0]=(0.477947766892)*x[0]**o+(-0.853046971581)*x[0]+(0.576044041601)*x[1]**o+(-0.447007701252)*x[1]
        arg[1,1,0,1]=(0.5886933409)*x[0]**o+(0.253650778351)*x[0]+(-0.198070684431)*x[1]**o+(0.189479364528)*x[1]
        arg[1,1,1,0]=(-0.528744478872)*x[0]**o+(-0.504072059557)*x[0]+(-0.0161832474015)*x[1]**o+(0.639592055966)*x[1]
        arg[1,1,1,1]=(-0.581844827169)*x[0]**o+(0.997883224538)*x[0]+(0.9354364489)*x[1]**o+(0.884060785136)*x[1]
        arg[1,1,2,0]=(0.858226955817)*x[0]**o+(0.707261698312)*x[0]+(-0.738309111076)*x[1]**o+(-0.644655976448)*x[1]
        arg[1,1,2,1]=(-0.254976445964)*x[0]**o+(-0.624982185158)*x[0]+(0.304854486996)*x[1]**o+(0.597089811682)*x[1]
        arg[1,2,0,0]=(-0.625190759215)*x[0]**o+(-0.267820197905)*x[0]+(-0.721452024377)*x[1]**o+(-0.830984245819)*x[1]
        arg[1,2,0,1]=(-0.972257608766)*x[0]**o+(0.981368890544)*x[0]+(-0.925886510064)*x[1]**o+(-0.337107356862)*x[1]
        arg[1,2,1,0]=(-0.178932954207)*x[0]**o+(-0.941789507374)*x[0]+(0.360726744794)*x[1]**o+(0.347621863123)*x[1]
        arg[1,2,1,1]=(-0.59790695611)*x[0]**o+(0.552509013166)*x[0]+(0.592412443998)*x[1]**o+(0.105369637971)*x[1]
        arg[1,2,2,0]=(0.300713195225)*x[0]**o+(0.814257503574)*x[0]+(-0.281407483492)*x[1]**o+(-0.803431681095)*x[1]
        arg[1,2,2,1]=(0.80113022942)*x[0]**o+(0.749336171665)*x[0]+(-0.0842928982283)*x[1]**o+(-0.974041792058)*x[1]
        arg[1,3,0,0]=(0.533118876666)*x[0]**o+(-0.00888178987567)*x[0]+(0.372869213762)*x[1]**o+(-0.652513683144)*x[1]
        arg[1,3,0,1]=(-0.295090222537)*x[0]**o+(0.772749124303)*x[0]+(-0.00857029468729)*x[1]**o+(0.927794431125)*x[1]
        arg[1,3,1,0]=(-0.152537465441)*x[0]**o+(0.154900534979)*x[0]+(-0.488743959543)*x[1]**o+(-0.0318497199232)*x[1]
        arg[1,3,1,1]=(-0.754333787921)*x[0]**o+(-0.0583007358701)*x[0]+(-0.050921106891)*x[1]**o+(0.161426452156)*x[1]
        arg[1,3,2,0]=(0.387357469451)*x[0]**o+(-0.213640305885)*x[0]+(-0.844609663023)*x[1]**o+(0.0761914594192)*x[1]
        arg[1,3,2,1]=(0.36964862459)*x[0]**o+(0.887888401952)*x[0]+(-0.933930511093)*x[1]**o+(0.769046614932)*x[1]
        arg[1,4,0,0]=(0.680878887266)*x[0]**o+(0.62930033679)*x[0]+(0.894793542585)*x[1]**o+(-0.636881609067)*x[1]
        arg[1,4,0,1]=(-0.346053756598)*x[0]**o+(0.225254433923)*x[0]+(-0.861052985621)*x[1]**o+(0.415123108136)*x[1]
        arg[1,4,1,0]=(-0.997758408183)*x[0]**o+(-0.265055614341)*x[0]+(-0.7568881617)*x[1]**o+(0.95144584777)*x[1]
        arg[1,4,1,1]=(0.227777912008)*x[0]**o+(-0.83647660817)*x[0]+(-0.275733410165)*x[1]**o+(-0.971560248343)*x[1]
        arg[1,4,2,0]=(0.718803859114)*x[0]**o+(-0.978966510682)*x[0]+(0.463102438649)*x[1]**o+(0.987805610534)*x[1]
        arg[1,4,2,1]=(-0.107543838917)*x[0]**o+(-0.362684018694)*x[0]+(-0.310926939373)*x[1]**o+(-0.571571044712)*x[1]
        arg[2,0,0,0]=(-0.475977416182)*x[0]**o+(-0.627173002609)*x[0]+(-0.369962970984)*x[1]**o+(-0.245077302537)*x[1]
        arg[2,0,0,1]=(0.913949611751)*x[0]**o+(0.645611424617)*x[0]+(0.639304734076)*x[1]**o+(-0.12171441998)*x[1]
        arg[2,0,1,0]=(0.251434098263)*x[0]**o+(-0.119898861407)*x[0]+(-0.365614523709)*x[1]**o+(-0.802928738296)*x[1]
        arg[2,0,1,1]=(0.756390927006)*x[0]**o+(0.246578936199)*x[0]+(0.714968085461)*x[1]**o+(-0.929428808096)*x[1]
        arg[2,0,2,0]=(-0.516888361447)*x[0]**o+(0.0344339250886)*x[0]+(-0.383709788774)*x[1]**o+(-0.437344417623)*x[1]
        arg[2,0,2,1]=(-0.219307162317)*x[0]**o+(0.704909718842)*x[0]+(-0.340454491855)*x[1]**o+(-0.933866231773)*x[1]
        arg[2,1,0,0]=(-0.0618967891627)*x[0]**o+(-0.0590712030789)*x[0]+(0.116776433077)*x[1]**o+(0.0241239764104)*x[1]
        arg[2,1,0,1]=(-0.802207645638)*x[0]**o+(0.590732740531)*x[0]+(-0.25990272933)*x[1]**o+(-0.257633805416)*x[1]
        arg[2,1,1,0]=(0.827623274379)*x[0]**o+(-0.450310573599)*x[0]+(0.213809775826)*x[1]**o+(-0.905410777787)*x[1]
        arg[2,1,1,1]=(-0.624510830356)*x[0]**o+(-0.705682607787)*x[0]+(0.430105716993)*x[1]**o+(0.975510832521)*x[1]
        arg[2,1,2,0]=(-0.297562223309)*x[0]**o+(-0.99090348994)*x[0]+(0.548589416291)*x[1]**o+(0.591718478583)*x[1]
        arg[2,1,2,1]=(-0.531899919376)*x[0]**o+(-0.227453719581)*x[0]+(-0.131664806128)*x[1]**o+(0.0521895817509)*x[1]
        arg[2,2,0,0]=(-0.592291745786)*x[0]**o+(-0.586879961972)*x[0]+(0.563196552327)*x[1]**o+(0.876989321187)*x[1]
        arg[2,2,0,1]=(-0.273360445369)*x[0]**o+(-0.714740527775)*x[0]+(-0.647699497837)*x[1]**o+(-0.0405089612762)*x[1]
        arg[2,2,1,0]=(-0.536629732224)*x[0]**o+(0.72656373966)*x[0]+(0.986630594165)*x[1]**o+(0.0741630711667)*x[1]
        arg[2,2,1,1]=(0.770746088838)*x[0]**o+(-0.219279833426)*x[0]+(0.875445174779)*x[1]**o+(-0.947889926444)*x[1]
        arg[2,2,2,0]=(-0.41428815119)*x[0]**o+(-0.696166268903)*x[0]+(-0.96838062553)*x[1]**o+(0.781394205899)*x[1]
        arg[2,2,2,1]=(-0.810267960752)*x[0]**o+(-0.873632687702)*x[0]+(0.0131971321611)*x[1]**o+(-0.552767192193)*x[1]
        arg[2,3,0,0]=(-0.216091366291)*x[0]**o+(-0.17824047472)*x[0]+(-0.4644515725)*x[1]**o+(0.135819458957)*x[1]
        arg[2,3,0,1]=(-0.0823616523105)*x[0]**o+(0.33121362151)*x[0]+(0.926719919839)*x[1]**o+(0.663475641636)*x[1]
        arg[2,3,1,0]=(-0.278303066047)*x[0]**o+(-0.457286378314)*x[0]+(0.186792859092)*x[1]**o+(-0.734245612464)*x[1]
        arg[2,3,1,1]=(0.810871459779)*x[0]**o+(-0.706641995324)*x[0]+(-0.287827176091)*x[1]**o+(0.453216090282)*x[1]
        arg[2,3,2,0]=(0.83826221049)*x[0]**o+(0.556887792111)*x[0]+(-0.000231102245991)*x[1]**o+(-0.440730958954)*x[1]
        arg[2,3,2,1]=(-0.455087877907)*x[0]**o+(-0.132259634036)*x[0]+(-0.492728335525)*x[1]**o+(-0.880311317777)*x[1]
        arg[2,4,0,0]=(-0.303141352388)*x[0]**o+(0.426137046804)*x[0]+(0.788934776738)*x[1]**o+(0.157477785179)*x[1]
        arg[2,4,0,1]=(0.603411288614)*x[0]**o+(-0.533561451987)*x[0]+(-0.901259210942)*x[1]**o+(0.547372624746)*x[1]
        arg[2,4,1,0]=(0.69517961739)*x[0]**o+(-0.106282888246)*x[0]+(0.257484565681)*x[1]**o+(0.655749633294)*x[1]
        arg[2,4,1,1]=(-0.072748573118)*x[0]**o+(0.905966560113)*x[0]+(0.433145718123)*x[1]**o+(0.187620401615)*x[1]
        arg[2,4,2,0]=(-0.0280176325954)*x[0]**o+(0.489610131347)*x[0]+(-0.740554922134)*x[1]**o+(-0.85045302096)*x[1]
        arg[2,4,2,1]=(-0.158503180709)*x[0]**o+(0.525929643075)*x[0]+(-0.352022652515)*x[1]**o+(-0.706553104932)*x[1]
        arg[3,0,0,0]=(-0.837316508127)*x[0]**o+(-0.89490181779)*x[0]+(0.510084075291)*x[1]**o+(0.321206310024)*x[1]
        arg[3,0,0,1]=(-0.325038591079)*x[0]**o+(0.140342922753)*x[0]+(-0.937466403004)*x[1]**o+(0.580557383497)*x[1]
        arg[3,0,1,0]=(-0.939713654062)*x[0]**o+(-0.394660244166)*x[0]+(0.762376614557)*x[1]**o+(-0.135565547532)*x[1]
        arg[3,0,1,1]=(0.0275332524464)*x[0]**o+(0.368737701882)*x[0]+(0.983641372218)*x[1]**o+(0.573815516366)*x[1]
        arg[3,0,2,0]=(0.816957570314)*x[0]**o+(-0.357626892562)*x[0]+(-0.368933149642)*x[1]**o+(-0.692502486914)*x[1]
        arg[3,0,2,1]=(-0.372408810926)*x[0]**o+(-0.113036390416)*x[0]+(-0.355880297003)*x[1]**o+(0.755478191481)*x[1]
        arg[3,1,0,0]=(0.0399012900832)*x[0]**o+(-0.879576900086)*x[0]+(-0.660858776134)*x[1]**o+(-0.59837040429)*x[1]
        arg[3,1,0,1]=(0.569397128496)*x[0]**o+(-0.138333417746)*x[0]+(0.257769147727)*x[1]**o+(0.174737141027)*x[1]
        arg[3,1,1,0]=(-0.67938408823)*x[0]**o+(0.862731161981)*x[0]+(-0.792538485264)*x[1]**o+(-0.0805197796157)*x[1]
        arg[3,1,1,1]=(0.0113523229279)*x[0]**o+(0.475393597561)*x[0]+(-0.705748021886)*x[1]**o+(0.00836460135285)*x[1]
        arg[3,1,2,0]=(0.463839509799)*x[0]**o+(-0.12663544643)*x[0]+(0.353158399738)*x[1]**o+(0.674929537597)*x[1]
        arg[3,1,2,1]=(-0.6019896019)*x[0]**o+(-0.638280685565)*x[0]+(0.280047072988)*x[1]**o+(0.569239958138)*x[1]
        arg[3,2,0,0]=(-0.113906500346)*x[0]**o+(-0.493494498501)*x[0]+(-0.686419229162)*x[1]**o+(-0.0727046763516)*x[1]
        arg[3,2,0,1]=(-0.302092112795)*x[0]**o+(-0.74819167)*x[0]+(-0.478908975282)*x[1]**o+(0.430024789627)*x[1]
        arg[3,2,1,0]=(0.353535608984)*x[0]**o+(-0.743408549264)*x[0]+(-0.069122881538)*x[1]**o+(-0.0767595089529)*x[1]
        arg[3,2,1,1]=(-0.629946386234)*x[0]**o+(0.851779590894)*x[0]+(0.728577108695)*x[1]**o+(-0.366983040147)*x[1]
        arg[3,2,2,0]=(0.1685167508)*x[0]**o+(0.149999747637)*x[0]+(0.709320430374)*x[1]**o+(-0.280950677222)*x[1]
        arg[3,2,2,1]=(-0.956164974578)*x[0]**o+(0.240460380901)*x[0]+(-0.2820176577)*x[1]**o+(-0.595667156955)*x[1]
        arg[3,3,0,0]=(-0.0711295055938)*x[0]**o+(-0.314513251791)*x[0]+(-0.41924591293)*x[1]**o+(0.0813197780959)*x[1]
        arg[3,3,0,1]=(-0.230592584877)*x[0]**o+(-0.488005144715)*x[0]+(-0.450611759976)*x[1]**o+(-0.0571152769768)*x[1]
        arg[3,3,1,0]=(-0.608255699014)*x[0]**o+(-0.697324108575)*x[0]+(-0.885935900746)*x[1]**o+(-0.884497550016)*x[1]
        arg[3,3,1,1]=(0.857286225292)*x[0]**o+(0.509954550626)*x[0]+(0.182030821433)*x[1]**o+(-0.268076060316)*x[1]
        arg[3,3,2,0]=(-0.77322653479)*x[0]**o+(-0.583013210018)*x[0]+(-0.683867300277)*x[1]**o+(0.754530541251)*x[1]
        arg[3,3,2,1]=(-0.744667985181)*x[0]**o+(0.164165128978)*x[0]+(0.412340155958)*x[1]**o+(-0.38451107118)*x[1]
        arg[3,4,0,0]=(0.216720755049)*x[0]**o+(-0.347803433722)*x[0]+(0.417376268797)*x[1]**o+(0.475787304652)*x[1]
        arg[3,4,0,1]=(-0.737038295974)*x[0]**o+(0.579172519178)*x[0]+(-0.900272993672)*x[1]**o+(0.407018603625)*x[1]
        arg[3,4,1,0]=(-0.898959236474)*x[0]**o+(-0.886636008121)*x[0]+(-0.901972610044)*x[1]**o+(-0.518393782318)*x[1]
        arg[3,4,1,1]=(0.976342884921)*x[0]**o+(-0.273568319493)*x[0]+(-0.332424980846)*x[1]**o+(-0.0146661731139)*x[1]
        arg[3,4,2,0]=(0.794792214027)*x[0]**o+(0.846927984413)*x[0]+(0.535818809709)*x[1]**o+(-0.49324400033)*x[1]
        arg[3,4,2,1]=(0.695766308812)*x[0]**o+(-0.37923153689)*x[0]+(0.0633513026782)*x[1]**o+(0.12334892105)*x[1]
        ref[0,0,0,0]=(0.912518456494)*x_ref[0]**o+(-0.828176151903)*x_ref[0]+(-0.313940298121)*x_ref[1]**o+(-0.850120336531)*x_ref[1]
        ref[0,0,0,1]=(0.14415852783)*x_ref[0]**o+(-0.294155246668)*x_ref[0]+(0.562517248232)*x_ref[1]**o+(-0.204505938286)*x_ref[1]
        ref[0,0,1,0]=(0.555455404368)*x_ref[0]**o+(0.916425343576)*x_ref[0]+(0.547252315693)*x_ref[1]**o+(-0.0992842367479)*x_ref[1]
        ref[0,0,1,1]=(0.603632473928)*x_ref[0]**o+(0.505571549062)*x_ref[0]+(-0.490402921542)*x_ref[1]**o+(0.908624427316)*x_ref[1]
        ref[0,0,2,0]=(0.729542308985)*x_ref[0]**o+(-0.654310868638)*x_ref[0]+(0.0845981276245)*x_ref[1]**o+(-0.95231604686)*x_ref[1]
        ref[0,0,2,1]=(0.0511615606745)*x_ref[0]**o+(-0.939469887975)*x_ref[0]+(-0.206229418155)*x_ref[1]**o+(0.254879844701)*x_ref[1]
        ref[0,1,0,0]=(0.257917344507)*x_ref[0]**o+(0.860729621288)*x_ref[0]+(0.489966193348)*x_ref[1]**o+(0.174897233858)*x_ref[1]
        ref[0,1,0,1]=(0.336588362051)*x_ref[0]**o+(-0.690598635114)*x_ref[0]+(-0.0913735300483)*x_ref[1]**o+(-0.932914439004)*x_ref[1]
        ref[0,1,1,0]=(0.244583093108)*x_ref[0]**o+(-0.827768167758)*x_ref[0]+(0.573065359412)*x_ref[1]**o+(0.176685515596)*x_ref[1]
        ref[0,1,1,1]=(-0.520550726127)*x_ref[0]**o+(-0.605243177286)*x_ref[0]+(0.130549556226)*x_ref[1]**o+(0.750019884067)*x_ref[1]
        ref[0,1,2,0]=(-0.680201627614)*x_ref[0]**o+(0.679925324882)*x_ref[0]+(-0.060408604263)*x_ref[1]**o+(0.568125689359)*x_ref[1]
        ref[0,1,2,1]=(-0.111908859432)*x_ref[0]**o+(0.256521037018)*x_ref[0]+(0.332050616611)*x_ref[1]**o+(-0.0500016397683)*x_ref[1]
        ref[0,2,0,0]=(-0.484953933663)*x_ref[0]**o+(0.628032211831)*x_ref[0]+(-0.63297141454)*x_ref[1]**o+(0.141016662755)*x_ref[1]
        ref[0,2,0,1]=(-0.463920680465)*x_ref[0]**o+(-0.868825520511)*x_ref[0]+(-0.36594425006)*x_ref[1]**o+(-0.823052216578)*x_ref[1]
        ref[0,2,1,0]=(0.972337802357)*x_ref[0]**o+(0.943605157343)*x_ref[0]+(-0.113950717844)*x_ref[1]**o+(-0.839596452897)*x_ref[1]
        ref[0,2,1,1]=(-0.295562401302)*x_ref[0]**o+(0.735770621017)*x_ref[0]+(-0.183838912348)*x_ref[1]**o+(-0.929365380252)*x_ref[1]
        ref[0,2,2,0]=(0.912886298388)*x_ref[0]**o+(0.847507982065)*x_ref[0]+(-0.341306036323)*x_ref[1]**o+(0.393627025978)*x_ref[1]
        ref[0,2,2,1]=(-0.769650638386)*x_ref[0]**o+(-0.619672356488)*x_ref[0]+(-0.126894288906)*x_ref[1]**o+(0.231547632139)*x_ref[1]
        ref[0,3,0,0]=(-0.884442976834)*x_ref[0]**o+(-0.0194505760676)*x_ref[0]+(-0.338764675072)*x_ref[1]**o+(0.723481600171)*x_ref[1]
        ref[0,3,0,1]=(0.655973555091)*x_ref[0]**o+(-0.556230649412)*x_ref[0]+(-0.850356832441)*x_ref[1]**o+(0.115267314853)*x_ref[1]
        ref[0,3,1,0]=(0.659628443559)*x_ref[0]**o+(-0.586677822491)*x_ref[0]+(0.135930588518)*x_ref[1]**o+(0.206866895948)*x_ref[1]
        ref[0,3,1,1]=(-0.700258957557)*x_ref[0]**o+(-0.699774198764)*x_ref[0]+(-0.294009479186)*x_ref[1]**o+(0.848159666503)*x_ref[1]
        ref[0,3,2,0]=(-0.927501754488)*x_ref[0]**o+(-0.981178071331)*x_ref[0]+(0.614074194451)*x_ref[1]**o+(0.711549573086)*x_ref[1]
        ref[0,3,2,1]=(0.312434473198)*x_ref[0]**o+(-0.241506125991)*x_ref[0]+(-0.0840979088912)*x_ref[1]**o+(0.902948463117)*x_ref[1]
        ref[0,4,0,0]=(-0.892269860242)*x_ref[0]**o+(0.823434435855)*x_ref[0]+(0.50499383365)*x_ref[1]**o+(0.763736541034)*x_ref[1]
        ref[0,4,0,1]=(-0.606959691191)*x_ref[0]**o+(0.945562536791)*x_ref[0]+(-0.46271929481)*x_ref[1]**o+(0.929514432096)*x_ref[1]
        ref[0,4,1,0]=(0.953382685939)*x_ref[0]**o+(0.553766359805)*x_ref[0]+(-0.191004010293)*x_ref[1]**o+(0.556369491698)*x_ref[1]
        ref[0,4,1,1]=(-0.269402649165)*x_ref[0]**o+(-0.158926232076)*x_ref[0]+(-0.390038749681)*x_ref[1]**o+(0.553655192666)*x_ref[1]
        ref[0,4,2,0]=(0.0785586372006)*x_ref[0]**o+(-0.778452221862)*x_ref[0]+(-0.927627273292)*x_ref[1]**o+(-0.187742405246)*x_ref[1]
        ref[0,4,2,1]=(-0.965329945447)*x_ref[0]**o+(0.304539006089)*x_ref[0]+(0.213031689568)*x_ref[1]**o+(-0.2807790149)*x_ref[1]
        ref[1,0,0,0]=(0.981534698569)*x_ref[0]**o+(0.466451685806)*x_ref[0]+(0.845322931856)*x_ref[1]**o+(0.582422032334)*x_ref[1]
        ref[1,0,0,1]=(-0.0645798740394)*x_ref[0]**o+(0.0475214206619)*x_ref[0]+(-0.687759433076)*x_ref[1]**o+(0.403440938285)*x_ref[1]
        ref[1,0,1,0]=(-0.317887773392)*x_ref[0]**o+(0.59101163416)*x_ref[0]+(0.658626263291)*x_ref[1]**o+(-0.618167594137)*x_ref[1]
        ref[1,0,1,1]=(-0.314093092813)*x_ref[0]**o+(0.699933899942)*x_ref[0]+(0.824195260822)*x_ref[1]**o+(-0.42875327345)*x_ref[1]
        ref[1,0,2,0]=(-0.254254956887)*x_ref[0]**o+(-0.912137080163)*x_ref[0]+(0.360457121452)*x_ref[1]**o+(-0.902596912588)*x_ref[1]
        ref[1,0,2,1]=(-0.577214593683)*x_ref[0]**o+(-0.419495663008)*x_ref[0]+(-0.803459783331)*x_ref[1]**o+(0.490885513568)*x_ref[1]
        ref[1,1,0,0]=(0.477947766892)*x_ref[0]**o+(-0.853046971581)*x_ref[0]+(0.576044041601)*x_ref[1]**o+(-0.447007701252)*x_ref[1]
        ref[1,1,0,1]=(0.5886933409)*x_ref[0]**o+(0.253650778351)*x_ref[0]+(-0.198070684431)*x_ref[1]**o+(0.189479364528)*x_ref[1]
        ref[1,1,1,0]=(-0.528744478872)*x_ref[0]**o+(-0.504072059557)*x_ref[0]+(-0.0161832474015)*x_ref[1]**o+(0.639592055966)*x_ref[1]
        ref[1,1,1,1]=(-0.581844827169)*x_ref[0]**o+(0.997883224538)*x_ref[0]+(0.9354364489)*x_ref[1]**o+(0.884060785136)*x_ref[1]
        ref[1,1,2,0]=(0.858226955817)*x_ref[0]**o+(0.707261698312)*x_ref[0]+(-0.738309111076)*x_ref[1]**o+(-0.644655976448)*x_ref[1]
        ref[1,1,2,1]=(-0.254976445964)*x_ref[0]**o+(-0.624982185158)*x_ref[0]+(0.304854486996)*x_ref[1]**o+(0.597089811682)*x_ref[1]
        ref[1,2,0,0]=(-0.625190759215)*x_ref[0]**o+(-0.267820197905)*x_ref[0]+(-0.721452024377)*x_ref[1]**o+(-0.830984245819)*x_ref[1]
        ref[1,2,0,1]=(-0.972257608766)*x_ref[0]**o+(0.981368890544)*x_ref[0]+(-0.925886510064)*x_ref[1]**o+(-0.337107356862)*x_ref[1]
        ref[1,2,1,0]=(-0.178932954207)*x_ref[0]**o+(-0.941789507374)*x_ref[0]+(0.360726744794)*x_ref[1]**o+(0.347621863123)*x_ref[1]
        ref[1,2,1,1]=(-0.59790695611)*x_ref[0]**o+(0.552509013166)*x_ref[0]+(0.592412443998)*x_ref[1]**o+(0.105369637971)*x_ref[1]
        ref[1,2,2,0]=(0.300713195225)*x_ref[0]**o+(0.814257503574)*x_ref[0]+(-0.281407483492)*x_ref[1]**o+(-0.803431681095)*x_ref[1]
        ref[1,2,2,1]=(0.80113022942)*x_ref[0]**o+(0.749336171665)*x_ref[0]+(-0.0842928982283)*x_ref[1]**o+(-0.974041792058)*x_ref[1]
        ref[1,3,0,0]=(0.533118876666)*x_ref[0]**o+(-0.00888178987567)*x_ref[0]+(0.372869213762)*x_ref[1]**o+(-0.652513683144)*x_ref[1]
        ref[1,3,0,1]=(-0.295090222537)*x_ref[0]**o+(0.772749124303)*x_ref[0]+(-0.00857029468729)*x_ref[1]**o+(0.927794431125)*x_ref[1]
        ref[1,3,1,0]=(-0.152537465441)*x_ref[0]**o+(0.154900534979)*x_ref[0]+(-0.488743959543)*x_ref[1]**o+(-0.0318497199232)*x_ref[1]
        ref[1,3,1,1]=(-0.754333787921)*x_ref[0]**o+(-0.0583007358701)*x_ref[0]+(-0.050921106891)*x_ref[1]**o+(0.161426452156)*x_ref[1]
        ref[1,3,2,0]=(0.387357469451)*x_ref[0]**o+(-0.213640305885)*x_ref[0]+(-0.844609663023)*x_ref[1]**o+(0.0761914594192)*x_ref[1]
        ref[1,3,2,1]=(0.36964862459)*x_ref[0]**o+(0.887888401952)*x_ref[0]+(-0.933930511093)*x_ref[1]**o+(0.769046614932)*x_ref[1]
        ref[1,4,0,0]=(0.680878887266)*x_ref[0]**o+(0.62930033679)*x_ref[0]+(0.894793542585)*x_ref[1]**o+(-0.636881609067)*x_ref[1]
        ref[1,4,0,1]=(-0.346053756598)*x_ref[0]**o+(0.225254433923)*x_ref[0]+(-0.861052985621)*x_ref[1]**o+(0.415123108136)*x_ref[1]
        ref[1,4,1,0]=(-0.997758408183)*x_ref[0]**o+(-0.265055614341)*x_ref[0]+(-0.7568881617)*x_ref[1]**o+(0.95144584777)*x_ref[1]
        ref[1,4,1,1]=(0.227777912008)*x_ref[0]**o+(-0.83647660817)*x_ref[0]+(-0.275733410165)*x_ref[1]**o+(-0.971560248343)*x_ref[1]
        ref[1,4,2,0]=(0.718803859114)*x_ref[0]**o+(-0.978966510682)*x_ref[0]+(0.463102438649)*x_ref[1]**o+(0.987805610534)*x_ref[1]
        ref[1,4,2,1]=(-0.107543838917)*x_ref[0]**o+(-0.362684018694)*x_ref[0]+(-0.310926939373)*x_ref[1]**o+(-0.571571044712)*x_ref[1]
        ref[2,0,0,0]=(-0.475977416182)*x_ref[0]**o+(-0.627173002609)*x_ref[0]+(-0.369962970984)*x_ref[1]**o+(-0.245077302537)*x_ref[1]
        ref[2,0,0,1]=(0.913949611751)*x_ref[0]**o+(0.645611424617)*x_ref[0]+(0.639304734076)*x_ref[1]**o+(-0.12171441998)*x_ref[1]
        ref[2,0,1,0]=(0.251434098263)*x_ref[0]**o+(-0.119898861407)*x_ref[0]+(-0.365614523709)*x_ref[1]**o+(-0.802928738296)*x_ref[1]
        ref[2,0,1,1]=(0.756390927006)*x_ref[0]**o+(0.246578936199)*x_ref[0]+(0.714968085461)*x_ref[1]**o+(-0.929428808096)*x_ref[1]
        ref[2,0,2,0]=(-0.516888361447)*x_ref[0]**o+(0.0344339250886)*x_ref[0]+(-0.383709788774)*x_ref[1]**o+(-0.437344417623)*x_ref[1]
        ref[2,0,2,1]=(-0.219307162317)*x_ref[0]**o+(0.704909718842)*x_ref[0]+(-0.340454491855)*x_ref[1]**o+(-0.933866231773)*x_ref[1]
        ref[2,1,0,0]=(-0.0618967891627)*x_ref[0]**o+(-0.0590712030789)*x_ref[0]+(0.116776433077)*x_ref[1]**o+(0.0241239764104)*x_ref[1]
        ref[2,1,0,1]=(-0.802207645638)*x_ref[0]**o+(0.590732740531)*x_ref[0]+(-0.25990272933)*x_ref[1]**o+(-0.257633805416)*x_ref[1]
        ref[2,1,1,0]=(0.827623274379)*x_ref[0]**o+(-0.450310573599)*x_ref[0]+(0.213809775826)*x_ref[1]**o+(-0.905410777787)*x_ref[1]
        ref[2,1,1,1]=(-0.624510830356)*x_ref[0]**o+(-0.705682607787)*x_ref[0]+(0.430105716993)*x_ref[1]**o+(0.975510832521)*x_ref[1]
        ref[2,1,2,0]=(-0.297562223309)*x_ref[0]**o+(-0.99090348994)*x_ref[0]+(0.548589416291)*x_ref[1]**o+(0.591718478583)*x_ref[1]
        ref[2,1,2,1]=(-0.531899919376)*x_ref[0]**o+(-0.227453719581)*x_ref[0]+(-0.131664806128)*x_ref[1]**o+(0.0521895817509)*x_ref[1]
        ref[2,2,0,0]=(-0.592291745786)*x_ref[0]**o+(-0.586879961972)*x_ref[0]+(0.563196552327)*x_ref[1]**o+(0.876989321187)*x_ref[1]
        ref[2,2,0,1]=(-0.273360445369)*x_ref[0]**o+(-0.714740527775)*x_ref[0]+(-0.647699497837)*x_ref[1]**o+(-0.0405089612762)*x_ref[1]
        ref[2,2,1,0]=(-0.536629732224)*x_ref[0]**o+(0.72656373966)*x_ref[0]+(0.986630594165)*x_ref[1]**o+(0.0741630711667)*x_ref[1]
        ref[2,2,1,1]=(0.770746088838)*x_ref[0]**o+(-0.219279833426)*x_ref[0]+(0.875445174779)*x_ref[1]**o+(-0.947889926444)*x_ref[1]
        ref[2,2,2,0]=(-0.41428815119)*x_ref[0]**o+(-0.696166268903)*x_ref[0]+(-0.96838062553)*x_ref[1]**o+(0.781394205899)*x_ref[1]
        ref[2,2,2,1]=(-0.810267960752)*x_ref[0]**o+(-0.873632687702)*x_ref[0]+(0.0131971321611)*x_ref[1]**o+(-0.552767192193)*x_ref[1]
        ref[2,3,0,0]=(-0.216091366291)*x_ref[0]**o+(-0.17824047472)*x_ref[0]+(-0.4644515725)*x_ref[1]**o+(0.135819458957)*x_ref[1]
        ref[2,3,0,1]=(-0.0823616523105)*x_ref[0]**o+(0.33121362151)*x_ref[0]+(0.926719919839)*x_ref[1]**o+(0.663475641636)*x_ref[1]
        ref[2,3,1,0]=(-0.278303066047)*x_ref[0]**o+(-0.457286378314)*x_ref[0]+(0.186792859092)*x_ref[1]**o+(-0.734245612464)*x_ref[1]
        ref[2,3,1,1]=(0.810871459779)*x_ref[0]**o+(-0.706641995324)*x_ref[0]+(-0.287827176091)*x_ref[1]**o+(0.453216090282)*x_ref[1]
        ref[2,3,2,0]=(0.83826221049)*x_ref[0]**o+(0.556887792111)*x_ref[0]+(-0.000231102245991)*x_ref[1]**o+(-0.440730958954)*x_ref[1]
        ref[2,3,2,1]=(-0.455087877907)*x_ref[0]**o+(-0.132259634036)*x_ref[0]+(-0.492728335525)*x_ref[1]**o+(-0.880311317777)*x_ref[1]
        ref[2,4,0,0]=(-0.303141352388)*x_ref[0]**o+(0.426137046804)*x_ref[0]+(0.788934776738)*x_ref[1]**o+(0.157477785179)*x_ref[1]
        ref[2,4,0,1]=(0.603411288614)*x_ref[0]**o+(-0.533561451987)*x_ref[0]+(-0.901259210942)*x_ref[1]**o+(0.547372624746)*x_ref[1]
        ref[2,4,1,0]=(0.69517961739)*x_ref[0]**o+(-0.106282888246)*x_ref[0]+(0.257484565681)*x_ref[1]**o+(0.655749633294)*x_ref[1]
        ref[2,4,1,1]=(-0.072748573118)*x_ref[0]**o+(0.905966560113)*x_ref[0]+(0.433145718123)*x_ref[1]**o+(0.187620401615)*x_ref[1]
        ref[2,4,2,0]=(-0.0280176325954)*x_ref[0]**o+(0.489610131347)*x_ref[0]+(-0.740554922134)*x_ref[1]**o+(-0.85045302096)*x_ref[1]
        ref[2,4,2,1]=(-0.158503180709)*x_ref[0]**o+(0.525929643075)*x_ref[0]+(-0.352022652515)*x_ref[1]**o+(-0.706553104932)*x_ref[1]
        ref[3,0,0,0]=(-0.837316508127)*x_ref[0]**o+(-0.89490181779)*x_ref[0]+(0.510084075291)*x_ref[1]**o+(0.321206310024)*x_ref[1]
        ref[3,0,0,1]=(-0.325038591079)*x_ref[0]**o+(0.140342922753)*x_ref[0]+(-0.937466403004)*x_ref[1]**o+(0.580557383497)*x_ref[1]
        ref[3,0,1,0]=(-0.939713654062)*x_ref[0]**o+(-0.394660244166)*x_ref[0]+(0.762376614557)*x_ref[1]**o+(-0.135565547532)*x_ref[1]
        ref[3,0,1,1]=(0.0275332524464)*x_ref[0]**o+(0.368737701882)*x_ref[0]+(0.983641372218)*x_ref[1]**o+(0.573815516366)*x_ref[1]
        ref[3,0,2,0]=(0.816957570314)*x_ref[0]**o+(-0.357626892562)*x_ref[0]+(-0.368933149642)*x_ref[1]**o+(-0.692502486914)*x_ref[1]
        ref[3,0,2,1]=(-0.372408810926)*x_ref[0]**o+(-0.113036390416)*x_ref[0]+(-0.355880297003)*x_ref[1]**o+(0.755478191481)*x_ref[1]
        ref[3,1,0,0]=(0.0399012900832)*x_ref[0]**o+(-0.879576900086)*x_ref[0]+(-0.660858776134)*x_ref[1]**o+(-0.59837040429)*x_ref[1]
        ref[3,1,0,1]=(0.569397128496)*x_ref[0]**o+(-0.138333417746)*x_ref[0]+(0.257769147727)*x_ref[1]**o+(0.174737141027)*x_ref[1]
        ref[3,1,1,0]=(-0.67938408823)*x_ref[0]**o+(0.862731161981)*x_ref[0]+(-0.792538485264)*x_ref[1]**o+(-0.0805197796157)*x_ref[1]
        ref[3,1,1,1]=(0.0113523229279)*x_ref[0]**o+(0.475393597561)*x_ref[0]+(-0.705748021886)*x_ref[1]**o+(0.00836460135285)*x_ref[1]
        ref[3,1,2,0]=(0.463839509799)*x_ref[0]**o+(-0.12663544643)*x_ref[0]+(0.353158399738)*x_ref[1]**o+(0.674929537597)*x_ref[1]
        ref[3,1,2,1]=(-0.6019896019)*x_ref[0]**o+(-0.638280685565)*x_ref[0]+(0.280047072988)*x_ref[1]**o+(0.569239958138)*x_ref[1]
        ref[3,2,0,0]=(-0.113906500346)*x_ref[0]**o+(-0.493494498501)*x_ref[0]+(-0.686419229162)*x_ref[1]**o+(-0.0727046763516)*x_ref[1]
        ref[3,2,0,1]=(-0.302092112795)*x_ref[0]**o+(-0.74819167)*x_ref[0]+(-0.478908975282)*x_ref[1]**o+(0.430024789627)*x_ref[1]
        ref[3,2,1,0]=(0.353535608984)*x_ref[0]**o+(-0.743408549264)*x_ref[0]+(-0.069122881538)*x_ref[1]**o+(-0.0767595089529)*x_ref[1]
        ref[3,2,1,1]=(-0.629946386234)*x_ref[0]**o+(0.851779590894)*x_ref[0]+(0.728577108695)*x_ref[1]**o+(-0.366983040147)*x_ref[1]
        ref[3,2,2,0]=(0.1685167508)*x_ref[0]**o+(0.149999747637)*x_ref[0]+(0.709320430374)*x_ref[1]**o+(-0.280950677222)*x_ref[1]
        ref[3,2,2,1]=(-0.956164974578)*x_ref[0]**o+(0.240460380901)*x_ref[0]+(-0.2820176577)*x_ref[1]**o+(-0.595667156955)*x_ref[1]
        ref[3,3,0,0]=(-0.0711295055938)*x_ref[0]**o+(-0.314513251791)*x_ref[0]+(-0.41924591293)*x_ref[1]**o+(0.0813197780959)*x_ref[1]
        ref[3,3,0,1]=(-0.230592584877)*x_ref[0]**o+(-0.488005144715)*x_ref[0]+(-0.450611759976)*x_ref[1]**o+(-0.0571152769768)*x_ref[1]
        ref[3,3,1,0]=(-0.608255699014)*x_ref[0]**o+(-0.697324108575)*x_ref[0]+(-0.885935900746)*x_ref[1]**o+(-0.884497550016)*x_ref[1]
        ref[3,3,1,1]=(0.857286225292)*x_ref[0]**o+(0.509954550626)*x_ref[0]+(0.182030821433)*x_ref[1]**o+(-0.268076060316)*x_ref[1]
        ref[3,3,2,0]=(-0.77322653479)*x_ref[0]**o+(-0.583013210018)*x_ref[0]+(-0.683867300277)*x_ref[1]**o+(0.754530541251)*x_ref[1]
        ref[3,3,2,1]=(-0.744667985181)*x_ref[0]**o+(0.164165128978)*x_ref[0]+(0.412340155958)*x_ref[1]**o+(-0.38451107118)*x_ref[1]
        ref[3,4,0,0]=(0.216720755049)*x_ref[0]**o+(-0.347803433722)*x_ref[0]+(0.417376268797)*x_ref[1]**o+(0.475787304652)*x_ref[1]
        ref[3,4,0,1]=(-0.737038295974)*x_ref[0]**o+(0.579172519178)*x_ref[0]+(-0.900272993672)*x_ref[1]**o+(0.407018603625)*x_ref[1]
        ref[3,4,1,0]=(-0.898959236474)*x_ref[0]**o+(-0.886636008121)*x_ref[0]+(-0.901972610044)*x_ref[1]**o+(-0.518393782318)*x_ref[1]
        ref[3,4,1,1]=(0.976342884921)*x_ref[0]**o+(-0.273568319493)*x_ref[0]+(-0.332424980846)*x_ref[1]**o+(-0.0146661731139)*x_ref[1]
        ref[3,4,2,0]=(0.794792214027)*x_ref[0]**o+(0.846927984413)*x_ref[0]+(0.535818809709)*x_ref[1]**o+(-0.49324400033)*x_ref[1]
        ref[3,4,2,1]=(0.695766308812)*x_ref[0]**o+(-0.37923153689)*x_ref[0]+(0.0633513026782)*x_ref[1]**o+(0.12334892105)*x_ref[1]
      else:
        arg[0,0,0,0]=(-0.401901998082)*x[0]**o+(-0.805091116808)*x[0]+(0.997659882792)*x[1]**o+(-0.953957389492)*x[1]+(-0.450185408289)*x[2]**o+(0.636472207773)*x[2]
        arg[0,0,0,1]=(-0.786386817128)*x[0]**o+(-0.882559775393)*x[0]+(0.117871400819)*x[1]**o+(-0.625922274774)*x[1]+(0.410431886931)*x[2]**o+(-0.979356501067)*x[2]
        arg[0,0,1,0]=(0.686025409275)*x[0]**o+(-0.19806907659)*x[0]+(-0.710420224449)*x[1]**o+(-0.839236953983)*x[1]+(-0.79971896093)*x[2]**o+(0.378944111319)*x[2]
        arg[0,0,1,1]=(0.771205112484)*x[0]**o+(-0.007083192859)*x[0]+(0.672918588001)*x[1]**o+(-0.351652495793)*x[1]+(-0.841767161187)*x[2]**o+(-0.345642373014)*x[2]
        arg[0,0,2,0]=(0.266930729015)*x[0]**o+(-0.873808152307)*x[0]+(0.206466759135)*x[1]**o+(0.539888649948)*x[1]+(0.17985862191)*x[2]**o+(0.240651460153)*x[2]
        arg[0,0,2,1]=(-0.232569768041)*x[0]**o+(-0.579841454072)*x[0]+(-0.706956104891)*x[1]**o+(0.822863815698)*x[1]+(0.345774991358)*x[2]**o+(-0.0848567668311)*x[2]
        arg[0,1,0,0]=(0.51593851507)*x[0]**o+(-0.233342434349)*x[0]+(-0.92498298026)*x[1]**o+(0.164540234829)*x[1]+(0.356559744499)*x[2]**o+(0.483902511717)*x[2]
        arg[0,1,0,1]=(-0.811509654443)*x[0]**o+(0.73929453189)*x[0]+(0.256205706418)*x[1]**o+(-0.667924913793)*x[1]+(0.266065641257)*x[2]**o+(0.856490258029)*x[2]
        arg[0,1,1,0]=(0.502051590284)*x[0]**o+(0.927283797128)*x[0]+(0.500854521411)*x[1]**o+(-0.486326996369)*x[1]+(-0.327578523685)*x[2]**o+(-0.28106070397)*x[2]
        arg[0,1,1,1]=(0.201740014513)*x[0]**o+(-0.166988241759)*x[0]+(-0.513072955566)*x[1]**o+(0.67179532417)*x[1]+(-0.72135980627)*x[2]**o+(0.763548886467)*x[2]
        arg[0,1,2,0]=(0.3995590338)*x[0]**o+(0.433286929621)*x[0]+(-0.287309453792)*x[1]**o+(-0.0671679443074)*x[1]+(-0.80815399993)*x[2]**o+(-0.398740362114)*x[2]
        arg[0,1,2,1]=(-0.16314640684)*x[0]**o+(-0.698681697367)*x[0]+(-0.11609738418)*x[1]**o+(-0.798465154739)*x[1]+(-0.169052628971)*x[2]**o+(-0.880316962539)*x[2]
        arg[0,2,0,0]=(-0.219731279442)*x[0]**o+(0.731464233112)*x[0]+(0.637811533993)*x[1]**o+(0.381218512692)*x[1]+(-0.181357790048)*x[2]**o+(-0.296793024723)*x[2]
        arg[0,2,0,1]=(0.566451624572)*x[0]**o+(0.302904935559)*x[0]+(-0.771526665232)*x[1]**o+(0.460577683302)*x[1]+(-0.921312455507)*x[2]**o+(0.604299765394)*x[2]
        arg[0,2,1,0]=(-0.0887516895197)*x[0]**o+(0.872208558285)*x[0]+(-0.306564750999)*x[1]**o+(0.841982500263)*x[1]+(-0.550398737046)*x[2]**o+(-0.869697162427)*x[2]
        arg[0,2,1,1]=(-0.652471383912)*x[0]**o+(0.634592304393)*x[0]+(0.0709212473167)*x[1]**o+(-0.671330082579)*x[1]+(-0.232895004478)*x[2]**o+(0.479008351165)*x[2]
        arg[0,2,2,0]=(-0.0764115046136)*x[0]**o+(0.649230346025)*x[0]+(0.544503480999)*x[1]**o+(0.788838981797)*x[1]+(0.416236148267)*x[2]**o+(0.722979541918)*x[2]
        arg[0,2,2,1]=(-0.0776368628837)*x[0]**o+(0.801984674386)*x[0]+(0.443670313486)*x[1]**o+(0.424676792255)*x[1]+(-0.895989937982)*x[2]**o+(0.0774648143589)*x[2]
        arg[0,3,0,0]=(-0.39041318438)*x[0]**o+(-0.174335409262)*x[0]+(0.658666482337)*x[1]**o+(0.23060605921)*x[1]+(-0.336603364419)*x[2]**o+(0.871530192198)*x[2]
        arg[0,3,0,1]=(-0.493104661363)*x[0]**o+(0.590779697499)*x[0]+(-0.903897457443)*x[1]**o+(-0.377640520704)*x[1]+(0.227174544833)*x[2]**o+(-0.860150483539)*x[2]
        arg[0,3,1,0]=(-0.093367253954)*x[0]**o+(0.159230499811)*x[0]+(-0.731396509936)*x[1]**o+(0.600088148838)*x[1]+(0.829175660147)*x[2]**o+(-0.435878280919)*x[2]
        arg[0,3,1,1]=(0.665125540711)*x[0]**o+(-0.92128737118)*x[0]+(-0.33260297543)*x[1]**o+(0.682569476816)*x[1]+(0.999761405058)*x[2]**o+(-0.748303702618)*x[2]
        arg[0,3,2,0]=(-0.430442277525)*x[0]**o+(-0.841437664378)*x[0]+(-0.374267371055)*x[1]**o+(0.667860327373)*x[1]+(0.694185699238)*x[2]**o+(-0.673629629622)*x[2]
        arg[0,3,2,1]=(0.304808617243)*x[0]**o+(-0.894407776037)*x[0]+(-0.758585281699)*x[1]**o+(0.20519963241)*x[1]+(0.538222015181)*x[2]**o+(-0.792962435573)*x[2]
        arg[0,4,0,0]=(0.568300300269)*x[0]**o+(0.407455936625)*x[0]+(0.980898344535)*x[1]**o+(0.988220474461)*x[1]+(0.0598892620089)*x[2]**o+(-0.279555207723)*x[2]
        arg[0,4,0,1]=(-0.208391854989)*x[0]**o+(0.9428019078)*x[0]+(-0.540916877275)*x[1]**o+(-0.427185538555)*x[1]+(-0.879268085857)*x[2]**o+(0.025102321099)*x[2]
        arg[0,4,1,0]=(0.412690112527)*x[0]**o+(-0.957376863734)*x[0]+(-0.913623139719)*x[1]**o+(-0.462418849979)*x[1]+(0.463336747019)*x[2]**o+(0.330747531293)*x[2]
        arg[0,4,1,1]=(-0.632140552118)*x[0]**o+(0.830769577671)*x[0]+(-0.888012251479)*x[1]**o+(-0.0560181569699)*x[1]+(-0.70068099248)*x[2]**o+(0.834483253039)*x[2]
        arg[0,4,2,0]=(0.0429717567385)*x[0]**o+(-0.691784042595)*x[0]+(-0.264988764713)*x[1]**o+(0.359873170931)*x[1]+(-0.116275128437)*x[2]**o+(0.0348873917345)*x[2]
        arg[0,4,2,1]=(-0.203754358311)*x[0]**o+(0.177896174646)*x[0]+(-0.0785920580727)*x[1]**o+(-0.124905949184)*x[1]+(0.968871355367)*x[2]**o+(-0.0369722247589)*x[2]
        arg[1,0,0,0]=(0.594851756786)*x[0]**o+(-0.594400822936)*x[0]+(0.868401350949)*x[1]**o+(0.186739616297)*x[1]+(-0.937219946754)*x[2]**o+(-0.248916422131)*x[2]
        arg[1,0,0,1]=(-0.639865978203)*x[0]**o+(-0.00990375501714)*x[0]+(-0.038266357411)*x[1]**o+(-0.741522380136)*x[1]+(0.460572922772)*x[2]**o+(0.4314205896)*x[2]
        arg[1,0,1,0]=(-0.460110413486)*x[0]**o+(-0.0964934735182)*x[0]+(0.474209024352)*x[1]**o+(-0.928476110805)*x[1]+(0.598621411487)*x[2]**o+(-0.00858812754345)*x[2]
        arg[1,0,1,1]=(0.936466172807)*x[0]**o+(-0.363937334197)*x[0]+(-0.540224651799)*x[1]**o+(0.484530722263)*x[1]+(0.675416993296)*x[2]**o+(0.89657344343)*x[2]
        arg[1,0,2,0]=(-0.713050692555)*x[0]**o+(-0.00851892179229)*x[0]+(0.689494113809)*x[1]**o+(0.633341028044)*x[1]+(0.272726911113)*x[2]**o+(-0.122017566819)*x[2]
        arg[1,0,2,1]=(0.793152080981)*x[0]**o+(0.287379739639)*x[0]+(0.304047372771)*x[1]**o+(-0.509079361053)*x[1]+(0.888302006985)*x[2]**o+(-0.297023045775)*x[2]
        arg[1,1,0,0]=(-0.69695118901)*x[0]**o+(0.505182668512)*x[0]+(0.827555590952)*x[1]**o+(-0.777232447056)*x[1]+(-0.460918138036)*x[2]**o+(0.782090206922)*x[2]
        arg[1,1,0,1]=(-0.435398535566)*x[0]**o+(-0.0997593681984)*x[0]+(0.140250252691)*x[1]**o+(-0.0204469256488)*x[1]+(-0.445836771028)*x[2]**o+(-0.775366037861)*x[2]
        arg[1,1,1,0]=(0.918015347957)*x[0]**o+(-0.491940111489)*x[0]+(0.793071617663)*x[1]**o+(-0.607035294859)*x[1]+(0.610076618566)*x[2]**o+(0.148116733513)*x[2]
        arg[1,1,1,1]=(0.224062518811)*x[0]**o+(-0.145897156735)*x[0]+(0.165380866901)*x[1]**o+(0.910562981814)*x[1]+(-0.406620561548)*x[2]**o+(0.528862812468)*x[2]
        arg[1,1,2,0]=(0.85148444093)*x[0]**o+(0.13210280653)*x[0]+(0.364136256712)*x[1]**o+(0.520388575386)*x[1]+(0.0145641287346)*x[2]**o+(-0.7849036351)*x[2]
        arg[1,1,2,1]=(0.493837193832)*x[0]**o+(-0.193326176569)*x[0]+(-0.665920972154)*x[1]**o+(-0.321645070095)*x[1]+(0.0209354853869)*x[2]**o+(-0.328315981752)*x[2]
        arg[1,2,0,0]=(-0.349281164766)*x[0]**o+(-0.318415198113)*x[0]+(-0.687972781234)*x[1]**o+(0.91825998605)*x[1]+(0.396868773798)*x[2]**o+(0.998204458027)*x[2]
        arg[1,2,0,1]=(-0.967493222935)*x[0]**o+(-0.721128465653)*x[0]+(-0.672178802974)*x[1]**o+(0.0924145224431)*x[1]+(0.114040587594)*x[2]**o+(0.707623895537)*x[2]
        arg[1,2,1,0]=(-0.253327088773)*x[0]**o+(0.206063827625)*x[0]+(0.355152654895)*x[1]**o+(-0.27339285621)*x[1]+(-0.0135567122291)*x[2]**o+(-0.983990446362)*x[2]
        arg[1,2,1,1]=(0.98471687132)*x[0]**o+(0.262902745655)*x[0]+(-0.589359138227)*x[1]**o+(-0.0982548958556)*x[1]+(-0.526161200133)*x[2]**o+(-0.850939484752)*x[2]
        arg[1,2,2,0]=(0.37583596705)*x[0]**o+(0.274845981222)*x[0]+(0.601331110943)*x[1]**o+(0.788294448218)*x[1]+(0.366445166459)*x[2]**o+(-0.554935072394)*x[2]
        arg[1,2,2,1]=(-0.986387850237)*x[0]**o+(0.68772989781)*x[0]+(0.345166604933)*x[1]**o+(0.930376726384)*x[1]+(0.760974421701)*x[2]**o+(0.261797396808)*x[2]
        arg[1,3,0,0]=(-0.171005013523)*x[0]**o+(-0.339229596812)*x[0]+(-0.150317419556)*x[1]**o+(-0.0146533427463)*x[1]+(0.957020663875)*x[2]**o+(0.616513940347)*x[2]
        arg[1,3,0,1]=(-0.0727328269499)*x[0]**o+(-0.635591748073)*x[0]+(-0.502426989774)*x[1]**o+(0.772595119619)*x[1]+(-0.76006511271)*x[2]**o+(-0.665920288084)*x[2]
        arg[1,3,1,0]=(-0.394284205622)*x[0]**o+(-0.105959165811)*x[0]+(0.72649045701)*x[1]**o+(-0.970276379473)*x[1]+(-0.236520349415)*x[2]**o+(0.994973267783)*x[2]
        arg[1,3,1,1]=(-0.444149867906)*x[0]**o+(0.748449201475)*x[0]+(0.419604170756)*x[1]**o+(-0.0291702705178)*x[1]+(-0.109580142628)*x[2]**o+(-0.0791870699838)*x[2]
        arg[1,3,2,0]=(-0.629657820523)*x[0]**o+(-0.995576552624)*x[0]+(0.530135462206)*x[1]**o+(0.197336335845)*x[1]+(0.0224815558807)*x[2]**o+(0.240498463912)*x[2]
        arg[1,3,2,1]=(0.52933948346)*x[0]**o+(0.98353913011)*x[0]+(0.0217614943138)*x[1]**o+(0.0157778453682)*x[1]+(0.177956932986)*x[2]**o+(0.234629778706)*x[2]
        arg[1,4,0,0]=(0.593028367137)*x[0]**o+(0.0340892538994)*x[0]+(-0.0295133170051)*x[1]**o+(0.0865921041963)*x[1]+(0.641131718656)*x[2]**o+(0.788858018609)*x[2]
        arg[1,4,0,1]=(-0.283333820634)*x[0]**o+(0.864482725099)*x[0]+(-0.560218794286)*x[1]**o+(-0.0083115362546)*x[1]+(0.726423285129)*x[2]**o+(0.031531339166)*x[2]
        arg[1,4,1,0]=(0.801452411554)*x[0]**o+(-0.59916393038)*x[0]+(-0.106568863591)*x[1]**o+(0.357442125133)*x[1]+(-0.203380280132)*x[2]**o+(-0.129031606441)*x[2]
        arg[1,4,1,1]=(-0.589220928637)*x[0]**o+(-0.308949389428)*x[0]+(0.785549858338)*x[1]**o+(-0.131153853015)*x[1]+(-0.30571855555)*x[2]**o+(-0.329634802588)*x[2]
        arg[1,4,2,0]=(-0.479686944)*x[0]**o+(-0.707702026873)*x[0]+(0.635894172352)*x[1]**o+(0.370673635069)*x[1]+(0.942862875991)*x[2]**o+(-0.846943456738)*x[2]
        arg[1,4,2,1]=(-0.403737324968)*x[0]**o+(0.101664426734)*x[0]+(-0.677093482354)*x[1]**o+(-0.36526066745)*x[1]+(-0.0889619136182)*x[2]**o+(-0.772491940844)*x[2]
        arg[2,0,0,0]=(0.753148325987)*x[0]**o+(-0.342843083128)*x[0]+(0.812661029553)*x[1]**o+(-0.112683035459)*x[1]+(0.900976470275)*x[2]**o+(0.00934086955102)*x[2]
        arg[2,0,0,1]=(-0.682315007143)*x[0]**o+(-0.694069900861)*x[0]+(-0.469082247747)*x[1]**o+(-0.522165903301)*x[1]+(-0.0997886221924)*x[2]**o+(-0.954161552419)*x[2]
        arg[2,0,1,0]=(0.780125539705)*x[0]**o+(-0.445331450225)*x[0]+(-0.282765510424)*x[1]**o+(-0.0131898361592)*x[1]+(-0.200186433923)*x[2]**o+(0.715139435696)*x[2]
        arg[2,0,1,1]=(-0.132310660786)*x[0]**o+(-0.0981344234369)*x[0]+(0.959297529475)*x[1]**o+(-0.85944894515)*x[1]+(-0.396258659619)*x[2]**o+(-0.09672066185)*x[2]
        arg[2,0,2,0]=(-0.344134014191)*x[0]**o+(0.534345864287)*x[0]+(0.797209509484)*x[1]**o+(0.0479050101499)*x[1]+(0.13620476987)*x[2]**o+(0.82708115383)*x[2]
        arg[2,0,2,1]=(-0.342509328238)*x[0]**o+(-0.833164493006)*x[0]+(0.472047721532)*x[1]**o+(0.301613158637)*x[1]+(0.858691679638)*x[2]**o+(-0.691744164006)*x[2]
        arg[2,1,0,0]=(0.498979390299)*x[0]**o+(0.371513881824)*x[0]+(-0.243506167971)*x[1]**o+(-0.245423890439)*x[1]+(-0.0494707840274)*x[2]**o+(0.0965573709928)*x[2]
        arg[2,1,0,1]=(0.665982455862)*x[0]**o+(-0.409710866336)*x[0]+(0.669257568046)*x[1]**o+(0.711947704788)*x[1]+(-0.0362568635785)*x[2]**o+(-0.30394278188)*x[2]
        arg[2,1,1,0]=(0.296245249928)*x[0]**o+(0.132631019281)*x[0]+(-0.376880152158)*x[1]**o+(0.303184156241)*x[1]+(0.836046560882)*x[2]**o+(0.195205645106)*x[2]
        arg[2,1,1,1]=(-0.971045820953)*x[0]**o+(0.154708896467)*x[0]+(-0.663844656602)*x[1]**o+(-0.0873797588245)*x[1]+(0.985123372542)*x[2]**o+(0.349554922131)*x[2]
        arg[2,1,2,0]=(-0.0117443431348)*x[0]**o+(-0.71980503985)*x[0]+(0.175348078994)*x[1]**o+(-0.883987892161)*x[1]+(0.0587275450458)*x[2]**o+(0.464811715548)*x[2]
        arg[2,1,2,1]=(-0.309202575765)*x[0]**o+(-0.528513248453)*x[0]+(-0.509298303972)*x[1]**o+(-0.228278863496)*x[1]+(0.933917053676)*x[2]**o+(-0.823678942264)*x[2]
        arg[2,2,0,0]=(0.374387386929)*x[0]**o+(0.64969993847)*x[0]+(-0.939822635641)*x[1]**o+(0.578249430422)*x[1]+(-0.973412660635)*x[2]**o+(0.484052718386)*x[2]
        arg[2,2,0,1]=(-0.972635162618)*x[0]**o+(-0.255007433802)*x[0]+(0.134165187097)*x[1]**o+(0.313213615976)*x[1]+(0.143298148686)*x[2]**o+(0.901819440229)*x[2]
        arg[2,2,1,0]=(0.868173256315)*x[0]**o+(-0.532325343453)*x[0]+(-0.712500880845)*x[1]**o+(0.216943462308)*x[1]+(0.733920091212)*x[2]**o+(0.0992400603383)*x[2]
        arg[2,2,1,1]=(0.642839350527)*x[0]**o+(0.127721543559)*x[0]+(-0.152686523032)*x[1]**o+(-0.399032361234)*x[1]+(0.317014518842)*x[2]**o+(0.905538790115)*x[2]
        arg[2,2,2,0]=(0.685267932413)*x[0]**o+(-0.167571547782)*x[0]+(-0.213446144405)*x[1]**o+(0.420270456377)*x[1]+(0.827725350621)*x[2]**o+(-0.920242274399)*x[2]
        arg[2,2,2,1]=(-0.507424593437)*x[0]**o+(-0.694222840019)*x[0]+(-0.904108970188)*x[1]**o+(-0.750899614966)*x[1]+(0.783997553302)*x[2]**o+(0.196327513447)*x[2]
        arg[2,3,0,0]=(-0.473900600479)*x[0]**o+(-0.464347097789)*x[0]+(-0.472423358671)*x[1]**o+(0.373786456364)*x[1]+(-0.989371300414)*x[2]**o+(-0.959508065239)*x[2]
        arg[2,3,0,1]=(0.174375189399)*x[0]**o+(0.925503059886)*x[0]+(-0.647274384723)*x[1]**o+(-0.577149479679)*x[1]+(0.171298877768)*x[2]**o+(-0.936612100882)*x[2]
        arg[2,3,1,0]=(0.482959343396)*x[0]**o+(0.842243554011)*x[0]+(0.814423633314)*x[1]**o+(-0.989445050315)*x[1]+(0.817933433065)*x[2]**o+(0.552396881404)*x[2]
        arg[2,3,1,1]=(0.55641736229)*x[0]**o+(0.498094640058)*x[0]+(0.0890113606037)*x[1]**o+(0.36592723553)*x[1]+(0.458607593386)*x[2]**o+(0.603047420063)*x[2]
        arg[2,3,2,0]=(0.884515080124)*x[0]**o+(-0.561898381022)*x[0]+(-0.292475503834)*x[1]**o+(0.425634423478)*x[1]+(0.65813323252)*x[2]**o+(-0.548850399842)*x[2]
        arg[2,3,2,1]=(-0.72895566982)*x[0]**o+(-0.569405588005)*x[0]+(0.117770875594)*x[1]**o+(-0.33515735969)*x[1]+(0.102062736382)*x[2]**o+(0.611532417683)*x[2]
        arg[2,4,0,0]=(-0.669208630424)*x[0]**o+(0.944393014177)*x[0]+(-0.00345511889683)*x[1]**o+(0.0695279595783)*x[1]+(0.496948250833)*x[2]**o+(-0.899750463971)*x[2]
        arg[2,4,0,1]=(-0.387828717292)*x[0]**o+(0.621172267004)*x[0]+(-0.280632674111)*x[1]**o+(0.401702567575)*x[1]+(-0.65554489378)*x[2]**o+(0.403018047007)*x[2]
        arg[2,4,1,0]=(-0.0770246925803)*x[0]**o+(0.52913302745)*x[0]+(0.922468724357)*x[1]**o+(-0.304442750439)*x[1]+(-0.556575079059)*x[2]**o+(-0.725790672251)*x[2]
        arg[2,4,1,1]=(-0.828378342911)*x[0]**o+(-0.0696370821776)*x[0]+(0.394018581667)*x[1]**o+(-0.251099848269)*x[1]+(-0.29878985656)*x[2]**o+(0.925753371056)*x[2]
        arg[2,4,2,0]=(-0.818277786029)*x[0]**o+(0.225862354706)*x[0]+(-0.629050212595)*x[1]**o+(0.139186107062)*x[1]+(-0.0134835519238)*x[2]**o+(0.301899528641)*x[2]
        arg[2,4,2,1]=(-0.783053684767)*x[0]**o+(-0.0926938783599)*x[0]+(0.519034210352)*x[1]**o+(-0.494564657126)*x[1]+(0.536117310622)*x[2]**o+(0.6278756634)*x[2]
        arg[3,0,0,0]=(-0.694353957685)*x[0]**o+(-0.361037363965)*x[0]+(-0.327004091885)*x[1]**o+(-0.765735142884)*x[1]+(-0.810915729713)*x[2]**o+(0.0640539311609)*x[2]
        arg[3,0,0,1]=(-0.131601548312)*x[0]**o+(0.271587756708)*x[0]+(0.322486168005)*x[1]**o+(0.238086581801)*x[1]+(-0.488081720555)*x[2]**o+(0.607278995635)*x[2]
        arg[3,0,1,0]=(0.903075418102)*x[0]**o+(-0.738609834848)*x[0]+(-0.165097989126)*x[1]**o+(-0.25967140914)*x[1]+(0.271649887428)*x[2]**o+(-0.786131648727)*x[2]
        arg[3,0,1,1]=(-0.963442989157)*x[0]**o+(-0.240205602334)*x[0]+(0.102153389731)*x[1]**o+(-0.106402434995)*x[1]+(-0.383097724211)*x[2]**o+(0.680290963195)*x[2]
        arg[3,0,2,0]=(-0.373162835796)*x[0]**o+(-0.186323761814)*x[0]+(0.661494315705)*x[1]**o+(0.320072189494)*x[1]+(0.0964443431067)*x[2]**o+(0.829015838241)*x[2]
        arg[3,0,2,1]=(0.527719514862)*x[0]**o+(0.0819948261665)*x[0]+(0.826598712178)*x[1]**o+(-0.588102975125)*x[1]+(-0.632584402292)*x[2]**o+(0.0826992233702)*x[2]
        arg[3,1,0,0]=(-0.102243686287)*x[0]**o+(-0.497336112929)*x[0]+(0.941827592628)*x[1]**o+(0.68678991944)*x[1]+(-0.508684841667)*x[2]**o+(-0.752939965161)*x[2]
        arg[3,1,0,1]=(-0.896417523183)*x[0]**o+(0.759224781685)*x[0]+(0.138537695823)*x[1]**o+(0.466727480471)*x[1]+(-0.838380824412)*x[2]**o+(0.285761157398)*x[2]
        arg[3,1,1,0]=(-0.547853182919)*x[0]**o+(-0.41595413414)*x[0]+(0.76894221584)*x[1]**o+(-0.142394275192)*x[1]+(-0.96349294581)*x[2]**o+(-0.873639964693)*x[2]
        arg[3,1,1,1]=(-0.151299403888)*x[0]**o+(0.789406063878)*x[0]+(0.163280670253)*x[1]**o+(-0.287315494745)*x[1]+(-0.627764670218)*x[2]**o+(0.842131632598)*x[2]
        arg[3,1,2,0]=(-0.957096489189)*x[0]**o+(0.0616037805677)*x[0]+(0.255858200166)*x[1]**o+(-0.430555450798)*x[1]+(0.730436930737)*x[2]**o+(0.405094271653)*x[2]
        arg[3,1,2,1]=(-0.924756832665)*x[0]**o+(0.525754162391)*x[0]+(0.281303228038)*x[1]**o+(-0.479500161878)*x[1]+(0.658416209181)*x[2]**o+(-0.0413881595557)*x[2]
        arg[3,2,0,0]=(0.516429381997)*x[0]**o+(-0.8387318467)*x[0]+(0.329773978227)*x[1]**o+(0.599641753085)*x[1]+(0.939937743271)*x[2]**o+(-0.247331169626)*x[2]
        arg[3,2,0,1]=(-0.644982044558)*x[0]**o+(-0.242033352774)*x[0]+(0.531835185779)*x[1]**o+(-0.851565064183)*x[1]+(-0.79213624915)*x[2]**o+(0.0607365690732)*x[2]
        arg[3,2,1,0]=(0.359781383537)*x[0]**o+(-0.0529212842074)*x[0]+(0.845915169452)*x[1]**o+(-0.323979570458)*x[1]+(0.828909997631)*x[2]**o+(-0.839886339138)*x[2]
        arg[3,2,1,1]=(0.559476630421)*x[0]**o+(-0.189725214615)*x[0]+(-0.00927446700413)*x[1]**o+(-0.234703720782)*x[1]+(-0.608362806755)*x[2]**o+(0.968571765534)*x[2]
        arg[3,2,2,0]=(0.924837507913)*x[0]**o+(0.846397611121)*x[0]+(-0.837765472403)*x[1]**o+(0.0225105985057)*x[1]+(-0.834187513844)*x[2]**o+(0.194311845578)*x[2]
        arg[3,2,2,1]=(-0.183561089961)*x[0]**o+(0.0745071729471)*x[0]+(-0.660415615202)*x[1]**o+(-0.528184997345)*x[1]+(0.205554094557)*x[2]**o+(-0.0309246318944)*x[2]
        arg[3,3,0,0]=(-0.67124095974)*x[0]**o+(-0.48501497083)*x[0]+(-0.0394345869094)*x[1]**o+(0.586692270429)*x[1]+(-0.593760749687)*x[2]**o+(-0.124979751327)*x[2]
        arg[3,3,0,1]=(0.778544173855)*x[0]**o+(-0.931466478494)*x[0]+(-0.746894777998)*x[1]**o+(0.806417936297)*x[1]+(0.989460929848)*x[2]**o+(0.666534366369)*x[2]
        arg[3,3,1,0]=(-0.798870838602)*x[0]**o+(-0.294618868937)*x[0]+(0.275029092968)*x[1]**o+(0.45390580364)*x[1]+(-0.311797818508)*x[2]**o+(0.751606624393)*x[2]
        arg[3,3,1,1]=(-0.924260987477)*x[0]**o+(0.771970726902)*x[0]+(-0.909099224213)*x[1]**o+(-0.45204206766)*x[1]+(0.630042614017)*x[2]**o+(-0.990915130464)*x[2]
        arg[3,3,2,0]=(0.734962154852)*x[0]**o+(-0.00621898011475)*x[0]+(-0.786362527189)*x[1]**o+(0.623507947406)*x[1]+(0.867175567104)*x[2]**o+(-0.383283255448)*x[2]
        arg[3,3,2,1]=(-0.955613156717)*x[0]**o+(0.688013754841)*x[0]+(-0.600559375208)*x[1]**o+(0.453342626645)*x[1]+(-0.864890477961)*x[2]**o+(-0.458029873063)*x[2]
        arg[3,4,0,0]=(-0.897010884834)*x[0]**o+(-0.251223576458)*x[0]+(-0.0338052047964)*x[1]**o+(-0.903421466132)*x[1]+(0.836864217874)*x[2]**o+(0.660805905175)*x[2]
        arg[3,4,0,1]=(-0.286025690215)*x[0]**o+(0.746732975604)*x[0]+(0.375810853218)*x[1]**o+(-0.985192974869)*x[1]+(-0.731463214455)*x[2]**o+(-0.465450352444)*x[2]
        arg[3,4,1,0]=(-0.500910000652)*x[0]**o+(0.387502419727)*x[0]+(0.962369164802)*x[1]**o+(-0.10933824251)*x[1]+(-0.33642703989)*x[2]**o+(0.546851133256)*x[2]
        arg[3,4,1,1]=(-0.117959459645)*x[0]**o+(0.867699323392)*x[0]+(0.950506403861)*x[1]**o+(0.613806207745)*x[1]+(0.425723173807)*x[2]**o+(-0.906937548619)*x[2]
        arg[3,4,2,0]=(0.331114892194)*x[0]**o+(-0.168829643424)*x[0]+(-0.855962311982)*x[1]**o+(-0.144202406673)*x[1]+(0.0769985713088)*x[2]**o+(0.856003746423)*x[2]
        arg[3,4,2,1]=(-0.439397439364)*x[0]**o+(0.686116343481)*x[0]+(0.519883854523)*x[1]**o+(0.716231469289)*x[1]+(0.270098231149)*x[2]**o+(-0.448525436427)*x[2]
        ref[0,0,0,0]=(-0.401901998082)*x_ref[0]**o+(-0.805091116808)*x_ref[0]+(0.997659882792)*x_ref[1]**o+(-0.953957389492)*x_ref[1]+(-0.450185408289)*x_ref[2]**o+(0.636472207773)*x_ref[2]
        ref[0,0,0,1]=(-0.786386817128)*x_ref[0]**o+(-0.882559775393)*x_ref[0]+(0.117871400819)*x_ref[1]**o+(-0.625922274774)*x_ref[1]+(0.410431886931)*x_ref[2]**o+(-0.979356501067)*x_ref[2]
        ref[0,0,1,0]=(0.686025409275)*x_ref[0]**o+(-0.19806907659)*x_ref[0]+(-0.710420224449)*x_ref[1]**o+(-0.839236953983)*x_ref[1]+(-0.79971896093)*x_ref[2]**o+(0.378944111319)*x_ref[2]
        ref[0,0,1,1]=(0.771205112484)*x_ref[0]**o+(-0.007083192859)*x_ref[0]+(0.672918588001)*x_ref[1]**o+(-0.351652495793)*x_ref[1]+(-0.841767161187)*x_ref[2]**o+(-0.345642373014)*x_ref[2]
        ref[0,0,2,0]=(0.266930729015)*x_ref[0]**o+(-0.873808152307)*x_ref[0]+(0.206466759135)*x_ref[1]**o+(0.539888649948)*x_ref[1]+(0.17985862191)*x_ref[2]**o+(0.240651460153)*x_ref[2]
        ref[0,0,2,1]=(-0.232569768041)*x_ref[0]**o+(-0.579841454072)*x_ref[0]+(-0.706956104891)*x_ref[1]**o+(0.822863815698)*x_ref[1]+(0.345774991358)*x_ref[2]**o+(-0.0848567668311)*x_ref[2]
        ref[0,1,0,0]=(0.51593851507)*x_ref[0]**o+(-0.233342434349)*x_ref[0]+(-0.92498298026)*x_ref[1]**o+(0.164540234829)*x_ref[1]+(0.356559744499)*x_ref[2]**o+(0.483902511717)*x_ref[2]
        ref[0,1,0,1]=(-0.811509654443)*x_ref[0]**o+(0.73929453189)*x_ref[0]+(0.256205706418)*x_ref[1]**o+(-0.667924913793)*x_ref[1]+(0.266065641257)*x_ref[2]**o+(0.856490258029)*x_ref[2]
        ref[0,1,1,0]=(0.502051590284)*x_ref[0]**o+(0.927283797128)*x_ref[0]+(0.500854521411)*x_ref[1]**o+(-0.486326996369)*x_ref[1]+(-0.327578523685)*x_ref[2]**o+(-0.28106070397)*x_ref[2]
        ref[0,1,1,1]=(0.201740014513)*x_ref[0]**o+(-0.166988241759)*x_ref[0]+(-0.513072955566)*x_ref[1]**o+(0.67179532417)*x_ref[1]+(-0.72135980627)*x_ref[2]**o+(0.763548886467)*x_ref[2]
        ref[0,1,2,0]=(0.3995590338)*x_ref[0]**o+(0.433286929621)*x_ref[0]+(-0.287309453792)*x_ref[1]**o+(-0.0671679443074)*x_ref[1]+(-0.80815399993)*x_ref[2]**o+(-0.398740362114)*x_ref[2]
        ref[0,1,2,1]=(-0.16314640684)*x_ref[0]**o+(-0.698681697367)*x_ref[0]+(-0.11609738418)*x_ref[1]**o+(-0.798465154739)*x_ref[1]+(-0.169052628971)*x_ref[2]**o+(-0.880316962539)*x_ref[2]
        ref[0,2,0,0]=(-0.219731279442)*x_ref[0]**o+(0.731464233112)*x_ref[0]+(0.637811533993)*x_ref[1]**o+(0.381218512692)*x_ref[1]+(-0.181357790048)*x_ref[2]**o+(-0.296793024723)*x_ref[2]
        ref[0,2,0,1]=(0.566451624572)*x_ref[0]**o+(0.302904935559)*x_ref[0]+(-0.771526665232)*x_ref[1]**o+(0.460577683302)*x_ref[1]+(-0.921312455507)*x_ref[2]**o+(0.604299765394)*x_ref[2]
        ref[0,2,1,0]=(-0.0887516895197)*x_ref[0]**o+(0.872208558285)*x_ref[0]+(-0.306564750999)*x_ref[1]**o+(0.841982500263)*x_ref[1]+(-0.550398737046)*x_ref[2]**o+(-0.869697162427)*x_ref[2]
        ref[0,2,1,1]=(-0.652471383912)*x_ref[0]**o+(0.634592304393)*x_ref[0]+(0.0709212473167)*x_ref[1]**o+(-0.671330082579)*x_ref[1]+(-0.232895004478)*x_ref[2]**o+(0.479008351165)*x_ref[2]
        ref[0,2,2,0]=(-0.0764115046136)*x_ref[0]**o+(0.649230346025)*x_ref[0]+(0.544503480999)*x_ref[1]**o+(0.788838981797)*x_ref[1]+(0.416236148267)*x_ref[2]**o+(0.722979541918)*x_ref[2]
        ref[0,2,2,1]=(-0.0776368628837)*x_ref[0]**o+(0.801984674386)*x_ref[0]+(0.443670313486)*x_ref[1]**o+(0.424676792255)*x_ref[1]+(-0.895989937982)*x_ref[2]**o+(0.0774648143589)*x_ref[2]
        ref[0,3,0,0]=(-0.39041318438)*x_ref[0]**o+(-0.174335409262)*x_ref[0]+(0.658666482337)*x_ref[1]**o+(0.23060605921)*x_ref[1]+(-0.336603364419)*x_ref[2]**o+(0.871530192198)*x_ref[2]
        ref[0,3,0,1]=(-0.493104661363)*x_ref[0]**o+(0.590779697499)*x_ref[0]+(-0.903897457443)*x_ref[1]**o+(-0.377640520704)*x_ref[1]+(0.227174544833)*x_ref[2]**o+(-0.860150483539)*x_ref[2]
        ref[0,3,1,0]=(-0.093367253954)*x_ref[0]**o+(0.159230499811)*x_ref[0]+(-0.731396509936)*x_ref[1]**o+(0.600088148838)*x_ref[1]+(0.829175660147)*x_ref[2]**o+(-0.435878280919)*x_ref[2]
        ref[0,3,1,1]=(0.665125540711)*x_ref[0]**o+(-0.92128737118)*x_ref[0]+(-0.33260297543)*x_ref[1]**o+(0.682569476816)*x_ref[1]+(0.999761405058)*x_ref[2]**o+(-0.748303702618)*x_ref[2]
        ref[0,3,2,0]=(-0.430442277525)*x_ref[0]**o+(-0.841437664378)*x_ref[0]+(-0.374267371055)*x_ref[1]**o+(0.667860327373)*x_ref[1]+(0.694185699238)*x_ref[2]**o+(-0.673629629622)*x_ref[2]
        ref[0,3,2,1]=(0.304808617243)*x_ref[0]**o+(-0.894407776037)*x_ref[0]+(-0.758585281699)*x_ref[1]**o+(0.20519963241)*x_ref[1]+(0.538222015181)*x_ref[2]**o+(-0.792962435573)*x_ref[2]
        ref[0,4,0,0]=(0.568300300269)*x_ref[0]**o+(0.407455936625)*x_ref[0]+(0.980898344535)*x_ref[1]**o+(0.988220474461)*x_ref[1]+(0.0598892620089)*x_ref[2]**o+(-0.279555207723)*x_ref[2]
        ref[0,4,0,1]=(-0.208391854989)*x_ref[0]**o+(0.9428019078)*x_ref[0]+(-0.540916877275)*x_ref[1]**o+(-0.427185538555)*x_ref[1]+(-0.879268085857)*x_ref[2]**o+(0.025102321099)*x_ref[2]
        ref[0,4,1,0]=(0.412690112527)*x_ref[0]**o+(-0.957376863734)*x_ref[0]+(-0.913623139719)*x_ref[1]**o+(-0.462418849979)*x_ref[1]+(0.463336747019)*x_ref[2]**o+(0.330747531293)*x_ref[2]
        ref[0,4,1,1]=(-0.632140552118)*x_ref[0]**o+(0.830769577671)*x_ref[0]+(-0.888012251479)*x_ref[1]**o+(-0.0560181569699)*x_ref[1]+(-0.70068099248)*x_ref[2]**o+(0.834483253039)*x_ref[2]
        ref[0,4,2,0]=(0.0429717567385)*x_ref[0]**o+(-0.691784042595)*x_ref[0]+(-0.264988764713)*x_ref[1]**o+(0.359873170931)*x_ref[1]+(-0.116275128437)*x_ref[2]**o+(0.0348873917345)*x_ref[2]
        ref[0,4,2,1]=(-0.203754358311)*x_ref[0]**o+(0.177896174646)*x_ref[0]+(-0.0785920580727)*x_ref[1]**o+(-0.124905949184)*x_ref[1]+(0.968871355367)*x_ref[2]**o+(-0.0369722247589)*x_ref[2]
        ref[1,0,0,0]=(0.594851756786)*x_ref[0]**o+(-0.594400822936)*x_ref[0]+(0.868401350949)*x_ref[1]**o+(0.186739616297)*x_ref[1]+(-0.937219946754)*x_ref[2]**o+(-0.248916422131)*x_ref[2]
        ref[1,0,0,1]=(-0.639865978203)*x_ref[0]**o+(-0.00990375501714)*x_ref[0]+(-0.038266357411)*x_ref[1]**o+(-0.741522380136)*x_ref[1]+(0.460572922772)*x_ref[2]**o+(0.4314205896)*x_ref[2]
        ref[1,0,1,0]=(-0.460110413486)*x_ref[0]**o+(-0.0964934735182)*x_ref[0]+(0.474209024352)*x_ref[1]**o+(-0.928476110805)*x_ref[1]+(0.598621411487)*x_ref[2]**o+(-0.00858812754345)*x_ref[2]
        ref[1,0,1,1]=(0.936466172807)*x_ref[0]**o+(-0.363937334197)*x_ref[0]+(-0.540224651799)*x_ref[1]**o+(0.484530722263)*x_ref[1]+(0.675416993296)*x_ref[2]**o+(0.89657344343)*x_ref[2]
        ref[1,0,2,0]=(-0.713050692555)*x_ref[0]**o+(-0.00851892179229)*x_ref[0]+(0.689494113809)*x_ref[1]**o+(0.633341028044)*x_ref[1]+(0.272726911113)*x_ref[2]**o+(-0.122017566819)*x_ref[2]
        ref[1,0,2,1]=(0.793152080981)*x_ref[0]**o+(0.287379739639)*x_ref[0]+(0.304047372771)*x_ref[1]**o+(-0.509079361053)*x_ref[1]+(0.888302006985)*x_ref[2]**o+(-0.297023045775)*x_ref[2]
        ref[1,1,0,0]=(-0.69695118901)*x_ref[0]**o+(0.505182668512)*x_ref[0]+(0.827555590952)*x_ref[1]**o+(-0.777232447056)*x_ref[1]+(-0.460918138036)*x_ref[2]**o+(0.782090206922)*x_ref[2]
        ref[1,1,0,1]=(-0.435398535566)*x_ref[0]**o+(-0.0997593681984)*x_ref[0]+(0.140250252691)*x_ref[1]**o+(-0.0204469256488)*x_ref[1]+(-0.445836771028)*x_ref[2]**o+(-0.775366037861)*x_ref[2]
        ref[1,1,1,0]=(0.918015347957)*x_ref[0]**o+(-0.491940111489)*x_ref[0]+(0.793071617663)*x_ref[1]**o+(-0.607035294859)*x_ref[1]+(0.610076618566)*x_ref[2]**o+(0.148116733513)*x_ref[2]
        ref[1,1,1,1]=(0.224062518811)*x_ref[0]**o+(-0.145897156735)*x_ref[0]+(0.165380866901)*x_ref[1]**o+(0.910562981814)*x_ref[1]+(-0.406620561548)*x_ref[2]**o+(0.528862812468)*x_ref[2]
        ref[1,1,2,0]=(0.85148444093)*x_ref[0]**o+(0.13210280653)*x_ref[0]+(0.364136256712)*x_ref[1]**o+(0.520388575386)*x_ref[1]+(0.0145641287346)*x_ref[2]**o+(-0.7849036351)*x_ref[2]
        ref[1,1,2,1]=(0.493837193832)*x_ref[0]**o+(-0.193326176569)*x_ref[0]+(-0.665920972154)*x_ref[1]**o+(-0.321645070095)*x_ref[1]+(0.0209354853869)*x_ref[2]**o+(-0.328315981752)*x_ref[2]
        ref[1,2,0,0]=(-0.349281164766)*x_ref[0]**o+(-0.318415198113)*x_ref[0]+(-0.687972781234)*x_ref[1]**o+(0.91825998605)*x_ref[1]+(0.396868773798)*x_ref[2]**o+(0.998204458027)*x_ref[2]
        ref[1,2,0,1]=(-0.967493222935)*x_ref[0]**o+(-0.721128465653)*x_ref[0]+(-0.672178802974)*x_ref[1]**o+(0.0924145224431)*x_ref[1]+(0.114040587594)*x_ref[2]**o+(0.707623895537)*x_ref[2]
        ref[1,2,1,0]=(-0.253327088773)*x_ref[0]**o+(0.206063827625)*x_ref[0]+(0.355152654895)*x_ref[1]**o+(-0.27339285621)*x_ref[1]+(-0.0135567122291)*x_ref[2]**o+(-0.983990446362)*x_ref[2]
        ref[1,2,1,1]=(0.98471687132)*x_ref[0]**o+(0.262902745655)*x_ref[0]+(-0.589359138227)*x_ref[1]**o+(-0.0982548958556)*x_ref[1]+(-0.526161200133)*x_ref[2]**o+(-0.850939484752)*x_ref[2]
        ref[1,2,2,0]=(0.37583596705)*x_ref[0]**o+(0.274845981222)*x_ref[0]+(0.601331110943)*x_ref[1]**o+(0.788294448218)*x_ref[1]+(0.366445166459)*x_ref[2]**o+(-0.554935072394)*x_ref[2]
        ref[1,2,2,1]=(-0.986387850237)*x_ref[0]**o+(0.68772989781)*x_ref[0]+(0.345166604933)*x_ref[1]**o+(0.930376726384)*x_ref[1]+(0.760974421701)*x_ref[2]**o+(0.261797396808)*x_ref[2]
        ref[1,3,0,0]=(-0.171005013523)*x_ref[0]**o+(-0.339229596812)*x_ref[0]+(-0.150317419556)*x_ref[1]**o+(-0.0146533427463)*x_ref[1]+(0.957020663875)*x_ref[2]**o+(0.616513940347)*x_ref[2]
        ref[1,3,0,1]=(-0.0727328269499)*x_ref[0]**o+(-0.635591748073)*x_ref[0]+(-0.502426989774)*x_ref[1]**o+(0.772595119619)*x_ref[1]+(-0.76006511271)*x_ref[2]**o+(-0.665920288084)*x_ref[2]
        ref[1,3,1,0]=(-0.394284205622)*x_ref[0]**o+(-0.105959165811)*x_ref[0]+(0.72649045701)*x_ref[1]**o+(-0.970276379473)*x_ref[1]+(-0.236520349415)*x_ref[2]**o+(0.994973267783)*x_ref[2]
        ref[1,3,1,1]=(-0.444149867906)*x_ref[0]**o+(0.748449201475)*x_ref[0]+(0.419604170756)*x_ref[1]**o+(-0.0291702705178)*x_ref[1]+(-0.109580142628)*x_ref[2]**o+(-0.0791870699838)*x_ref[2]
        ref[1,3,2,0]=(-0.629657820523)*x_ref[0]**o+(-0.995576552624)*x_ref[0]+(0.530135462206)*x_ref[1]**o+(0.197336335845)*x_ref[1]+(0.0224815558807)*x_ref[2]**o+(0.240498463912)*x_ref[2]
        ref[1,3,2,1]=(0.52933948346)*x_ref[0]**o+(0.98353913011)*x_ref[0]+(0.0217614943138)*x_ref[1]**o+(0.0157778453682)*x_ref[1]+(0.177956932986)*x_ref[2]**o+(0.234629778706)*x_ref[2]
        ref[1,4,0,0]=(0.593028367137)*x_ref[0]**o+(0.0340892538994)*x_ref[0]+(-0.0295133170051)*x_ref[1]**o+(0.0865921041963)*x_ref[1]+(0.641131718656)*x_ref[2]**o+(0.788858018609)*x_ref[2]
        ref[1,4,0,1]=(-0.283333820634)*x_ref[0]**o+(0.864482725099)*x_ref[0]+(-0.560218794286)*x_ref[1]**o+(-0.0083115362546)*x_ref[1]+(0.726423285129)*x_ref[2]**o+(0.031531339166)*x_ref[2]
        ref[1,4,1,0]=(0.801452411554)*x_ref[0]**o+(-0.59916393038)*x_ref[0]+(-0.106568863591)*x_ref[1]**o+(0.357442125133)*x_ref[1]+(-0.203380280132)*x_ref[2]**o+(-0.129031606441)*x_ref[2]
        ref[1,4,1,1]=(-0.589220928637)*x_ref[0]**o+(-0.308949389428)*x_ref[0]+(0.785549858338)*x_ref[1]**o+(-0.131153853015)*x_ref[1]+(-0.30571855555)*x_ref[2]**o+(-0.329634802588)*x_ref[2]
        ref[1,4,2,0]=(-0.479686944)*x_ref[0]**o+(-0.707702026873)*x_ref[0]+(0.635894172352)*x_ref[1]**o+(0.370673635069)*x_ref[1]+(0.942862875991)*x_ref[2]**o+(-0.846943456738)*x_ref[2]
        ref[1,4,2,1]=(-0.403737324968)*x_ref[0]**o+(0.101664426734)*x_ref[0]+(-0.677093482354)*x_ref[1]**o+(-0.36526066745)*x_ref[1]+(-0.0889619136182)*x_ref[2]**o+(-0.772491940844)*x_ref[2]
        ref[2,0,0,0]=(0.753148325987)*x_ref[0]**o+(-0.342843083128)*x_ref[0]+(0.812661029553)*x_ref[1]**o+(-0.112683035459)*x_ref[1]+(0.900976470275)*x_ref[2]**o+(0.00934086955102)*x_ref[2]
        ref[2,0,0,1]=(-0.682315007143)*x_ref[0]**o+(-0.694069900861)*x_ref[0]+(-0.469082247747)*x_ref[1]**o+(-0.522165903301)*x_ref[1]+(-0.0997886221924)*x_ref[2]**o+(-0.954161552419)*x_ref[2]
        ref[2,0,1,0]=(0.780125539705)*x_ref[0]**o+(-0.445331450225)*x_ref[0]+(-0.282765510424)*x_ref[1]**o+(-0.0131898361592)*x_ref[1]+(-0.200186433923)*x_ref[2]**o+(0.715139435696)*x_ref[2]
        ref[2,0,1,1]=(-0.132310660786)*x_ref[0]**o+(-0.0981344234369)*x_ref[0]+(0.959297529475)*x_ref[1]**o+(-0.85944894515)*x_ref[1]+(-0.396258659619)*x_ref[2]**o+(-0.09672066185)*x_ref[2]
        ref[2,0,2,0]=(-0.344134014191)*x_ref[0]**o+(0.534345864287)*x_ref[0]+(0.797209509484)*x_ref[1]**o+(0.0479050101499)*x_ref[1]+(0.13620476987)*x_ref[2]**o+(0.82708115383)*x_ref[2]
        ref[2,0,2,1]=(-0.342509328238)*x_ref[0]**o+(-0.833164493006)*x_ref[0]+(0.472047721532)*x_ref[1]**o+(0.301613158637)*x_ref[1]+(0.858691679638)*x_ref[2]**o+(-0.691744164006)*x_ref[2]
        ref[2,1,0,0]=(0.498979390299)*x_ref[0]**o+(0.371513881824)*x_ref[0]+(-0.243506167971)*x_ref[1]**o+(-0.245423890439)*x_ref[1]+(-0.0494707840274)*x_ref[2]**o+(0.0965573709928)*x_ref[2]
        ref[2,1,0,1]=(0.665982455862)*x_ref[0]**o+(-0.409710866336)*x_ref[0]+(0.669257568046)*x_ref[1]**o+(0.711947704788)*x_ref[1]+(-0.0362568635785)*x_ref[2]**o+(-0.30394278188)*x_ref[2]
        ref[2,1,1,0]=(0.296245249928)*x_ref[0]**o+(0.132631019281)*x_ref[0]+(-0.376880152158)*x_ref[1]**o+(0.303184156241)*x_ref[1]+(0.836046560882)*x_ref[2]**o+(0.195205645106)*x_ref[2]
        ref[2,1,1,1]=(-0.971045820953)*x_ref[0]**o+(0.154708896467)*x_ref[0]+(-0.663844656602)*x_ref[1]**o+(-0.0873797588245)*x_ref[1]+(0.985123372542)*x_ref[2]**o+(0.349554922131)*x_ref[2]
        ref[2,1,2,0]=(-0.0117443431348)*x_ref[0]**o+(-0.71980503985)*x_ref[0]+(0.175348078994)*x_ref[1]**o+(-0.883987892161)*x_ref[1]+(0.0587275450458)*x_ref[2]**o+(0.464811715548)*x_ref[2]
        ref[2,1,2,1]=(-0.309202575765)*x_ref[0]**o+(-0.528513248453)*x_ref[0]+(-0.509298303972)*x_ref[1]**o+(-0.228278863496)*x_ref[1]+(0.933917053676)*x_ref[2]**o+(-0.823678942264)*x_ref[2]
        ref[2,2,0,0]=(0.374387386929)*x_ref[0]**o+(0.64969993847)*x_ref[0]+(-0.939822635641)*x_ref[1]**o+(0.578249430422)*x_ref[1]+(-0.973412660635)*x_ref[2]**o+(0.484052718386)*x_ref[2]
        ref[2,2,0,1]=(-0.972635162618)*x_ref[0]**o+(-0.255007433802)*x_ref[0]+(0.134165187097)*x_ref[1]**o+(0.313213615976)*x_ref[1]+(0.143298148686)*x_ref[2]**o+(0.901819440229)*x_ref[2]
        ref[2,2,1,0]=(0.868173256315)*x_ref[0]**o+(-0.532325343453)*x_ref[0]+(-0.712500880845)*x_ref[1]**o+(0.216943462308)*x_ref[1]+(0.733920091212)*x_ref[2]**o+(0.0992400603383)*x_ref[2]
        ref[2,2,1,1]=(0.642839350527)*x_ref[0]**o+(0.127721543559)*x_ref[0]+(-0.152686523032)*x_ref[1]**o+(-0.399032361234)*x_ref[1]+(0.317014518842)*x_ref[2]**o+(0.905538790115)*x_ref[2]
        ref[2,2,2,0]=(0.685267932413)*x_ref[0]**o+(-0.167571547782)*x_ref[0]+(-0.213446144405)*x_ref[1]**o+(0.420270456377)*x_ref[1]+(0.827725350621)*x_ref[2]**o+(-0.920242274399)*x_ref[2]
        ref[2,2,2,1]=(-0.507424593437)*x_ref[0]**o+(-0.694222840019)*x_ref[0]+(-0.904108970188)*x_ref[1]**o+(-0.750899614966)*x_ref[1]+(0.783997553302)*x_ref[2]**o+(0.196327513447)*x_ref[2]
        ref[2,3,0,0]=(-0.473900600479)*x_ref[0]**o+(-0.464347097789)*x_ref[0]+(-0.472423358671)*x_ref[1]**o+(0.373786456364)*x_ref[1]+(-0.989371300414)*x_ref[2]**o+(-0.959508065239)*x_ref[2]
        ref[2,3,0,1]=(0.174375189399)*x_ref[0]**o+(0.925503059886)*x_ref[0]+(-0.647274384723)*x_ref[1]**o+(-0.577149479679)*x_ref[1]+(0.171298877768)*x_ref[2]**o+(-0.936612100882)*x_ref[2]
        ref[2,3,1,0]=(0.482959343396)*x_ref[0]**o+(0.842243554011)*x_ref[0]+(0.814423633314)*x_ref[1]**o+(-0.989445050315)*x_ref[1]+(0.817933433065)*x_ref[2]**o+(0.552396881404)*x_ref[2]
        ref[2,3,1,1]=(0.55641736229)*x_ref[0]**o+(0.498094640058)*x_ref[0]+(0.0890113606037)*x_ref[1]**o+(0.36592723553)*x_ref[1]+(0.458607593386)*x_ref[2]**o+(0.603047420063)*x_ref[2]
        ref[2,3,2,0]=(0.884515080124)*x_ref[0]**o+(-0.561898381022)*x_ref[0]+(-0.292475503834)*x_ref[1]**o+(0.425634423478)*x_ref[1]+(0.65813323252)*x_ref[2]**o+(-0.548850399842)*x_ref[2]
        ref[2,3,2,1]=(-0.72895566982)*x_ref[0]**o+(-0.569405588005)*x_ref[0]+(0.117770875594)*x_ref[1]**o+(-0.33515735969)*x_ref[1]+(0.102062736382)*x_ref[2]**o+(0.611532417683)*x_ref[2]
        ref[2,4,0,0]=(-0.669208630424)*x_ref[0]**o+(0.944393014177)*x_ref[0]+(-0.00345511889683)*x_ref[1]**o+(0.0695279595783)*x_ref[1]+(0.496948250833)*x_ref[2]**o+(-0.899750463971)*x_ref[2]
        ref[2,4,0,1]=(-0.387828717292)*x_ref[0]**o+(0.621172267004)*x_ref[0]+(-0.280632674111)*x_ref[1]**o+(0.401702567575)*x_ref[1]+(-0.65554489378)*x_ref[2]**o+(0.403018047007)*x_ref[2]
        ref[2,4,1,0]=(-0.0770246925803)*x_ref[0]**o+(0.52913302745)*x_ref[0]+(0.922468724357)*x_ref[1]**o+(-0.304442750439)*x_ref[1]+(-0.556575079059)*x_ref[2]**o+(-0.725790672251)*x_ref[2]
        ref[2,4,1,1]=(-0.828378342911)*x_ref[0]**o+(-0.0696370821776)*x_ref[0]+(0.394018581667)*x_ref[1]**o+(-0.251099848269)*x_ref[1]+(-0.29878985656)*x_ref[2]**o+(0.925753371056)*x_ref[2]
        ref[2,4,2,0]=(-0.818277786029)*x_ref[0]**o+(0.225862354706)*x_ref[0]+(-0.629050212595)*x_ref[1]**o+(0.139186107062)*x_ref[1]+(-0.0134835519238)*x_ref[2]**o+(0.301899528641)*x_ref[2]
        ref[2,4,2,1]=(-0.783053684767)*x_ref[0]**o+(-0.0926938783599)*x_ref[0]+(0.519034210352)*x_ref[1]**o+(-0.494564657126)*x_ref[1]+(0.536117310622)*x_ref[2]**o+(0.6278756634)*x_ref[2]
        ref[3,0,0,0]=(-0.694353957685)*x_ref[0]**o+(-0.361037363965)*x_ref[0]+(-0.327004091885)*x_ref[1]**o+(-0.765735142884)*x_ref[1]+(-0.810915729713)*x_ref[2]**o+(0.0640539311609)*x_ref[2]
        ref[3,0,0,1]=(-0.131601548312)*x_ref[0]**o+(0.271587756708)*x_ref[0]+(0.322486168005)*x_ref[1]**o+(0.238086581801)*x_ref[1]+(-0.488081720555)*x_ref[2]**o+(0.607278995635)*x_ref[2]
        ref[3,0,1,0]=(0.903075418102)*x_ref[0]**o+(-0.738609834848)*x_ref[0]+(-0.165097989126)*x_ref[1]**o+(-0.25967140914)*x_ref[1]+(0.271649887428)*x_ref[2]**o+(-0.786131648727)*x_ref[2]
        ref[3,0,1,1]=(-0.963442989157)*x_ref[0]**o+(-0.240205602334)*x_ref[0]+(0.102153389731)*x_ref[1]**o+(-0.106402434995)*x_ref[1]+(-0.383097724211)*x_ref[2]**o+(0.680290963195)*x_ref[2]
        ref[3,0,2,0]=(-0.373162835796)*x_ref[0]**o+(-0.186323761814)*x_ref[0]+(0.661494315705)*x_ref[1]**o+(0.320072189494)*x_ref[1]+(0.0964443431067)*x_ref[2]**o+(0.829015838241)*x_ref[2]
        ref[3,0,2,1]=(0.527719514862)*x_ref[0]**o+(0.0819948261665)*x_ref[0]+(0.826598712178)*x_ref[1]**o+(-0.588102975125)*x_ref[1]+(-0.632584402292)*x_ref[2]**o+(0.0826992233702)*x_ref[2]
        ref[3,1,0,0]=(-0.102243686287)*x_ref[0]**o+(-0.497336112929)*x_ref[0]+(0.941827592628)*x_ref[1]**o+(0.68678991944)*x_ref[1]+(-0.508684841667)*x_ref[2]**o+(-0.752939965161)*x_ref[2]
        ref[3,1,0,1]=(-0.896417523183)*x_ref[0]**o+(0.759224781685)*x_ref[0]+(0.138537695823)*x_ref[1]**o+(0.466727480471)*x_ref[1]+(-0.838380824412)*x_ref[2]**o+(0.285761157398)*x_ref[2]
        ref[3,1,1,0]=(-0.547853182919)*x_ref[0]**o+(-0.41595413414)*x_ref[0]+(0.76894221584)*x_ref[1]**o+(-0.142394275192)*x_ref[1]+(-0.96349294581)*x_ref[2]**o+(-0.873639964693)*x_ref[2]
        ref[3,1,1,1]=(-0.151299403888)*x_ref[0]**o+(0.789406063878)*x_ref[0]+(0.163280670253)*x_ref[1]**o+(-0.287315494745)*x_ref[1]+(-0.627764670218)*x_ref[2]**o+(0.842131632598)*x_ref[2]
        ref[3,1,2,0]=(-0.957096489189)*x_ref[0]**o+(0.0616037805677)*x_ref[0]+(0.255858200166)*x_ref[1]**o+(-0.430555450798)*x_ref[1]+(0.730436930737)*x_ref[2]**o+(0.405094271653)*x_ref[2]
        ref[3,1,2,1]=(-0.924756832665)*x_ref[0]**o+(0.525754162391)*x_ref[0]+(0.281303228038)*x_ref[1]**o+(-0.479500161878)*x_ref[1]+(0.658416209181)*x_ref[2]**o+(-0.0413881595557)*x_ref[2]
        ref[3,2,0,0]=(0.516429381997)*x_ref[0]**o+(-0.8387318467)*x_ref[0]+(0.329773978227)*x_ref[1]**o+(0.599641753085)*x_ref[1]+(0.939937743271)*x_ref[2]**o+(-0.247331169626)*x_ref[2]
        ref[3,2,0,1]=(-0.644982044558)*x_ref[0]**o+(-0.242033352774)*x_ref[0]+(0.531835185779)*x_ref[1]**o+(-0.851565064183)*x_ref[1]+(-0.79213624915)*x_ref[2]**o+(0.0607365690732)*x_ref[2]
        ref[3,2,1,0]=(0.359781383537)*x_ref[0]**o+(-0.0529212842074)*x_ref[0]+(0.845915169452)*x_ref[1]**o+(-0.323979570458)*x_ref[1]+(0.828909997631)*x_ref[2]**o+(-0.839886339138)*x_ref[2]
        ref[3,2,1,1]=(0.559476630421)*x_ref[0]**o+(-0.189725214615)*x_ref[0]+(-0.00927446700413)*x_ref[1]**o+(-0.234703720782)*x_ref[1]+(-0.608362806755)*x_ref[2]**o+(0.968571765534)*x_ref[2]
        ref[3,2,2,0]=(0.924837507913)*x_ref[0]**o+(0.846397611121)*x_ref[0]+(-0.837765472403)*x_ref[1]**o+(0.0225105985057)*x_ref[1]+(-0.834187513844)*x_ref[2]**o+(0.194311845578)*x_ref[2]
        ref[3,2,2,1]=(-0.183561089961)*x_ref[0]**o+(0.0745071729471)*x_ref[0]+(-0.660415615202)*x_ref[1]**o+(-0.528184997345)*x_ref[1]+(0.205554094557)*x_ref[2]**o+(-0.0309246318944)*x_ref[2]
        ref[3,3,0,0]=(-0.67124095974)*x_ref[0]**o+(-0.48501497083)*x_ref[0]+(-0.0394345869094)*x_ref[1]**o+(0.586692270429)*x_ref[1]+(-0.593760749687)*x_ref[2]**o+(-0.124979751327)*x_ref[2]
        ref[3,3,0,1]=(0.778544173855)*x_ref[0]**o+(-0.931466478494)*x_ref[0]+(-0.746894777998)*x_ref[1]**o+(0.806417936297)*x_ref[1]+(0.989460929848)*x_ref[2]**o+(0.666534366369)*x_ref[2]
        ref[3,3,1,0]=(-0.798870838602)*x_ref[0]**o+(-0.294618868937)*x_ref[0]+(0.275029092968)*x_ref[1]**o+(0.45390580364)*x_ref[1]+(-0.311797818508)*x_ref[2]**o+(0.751606624393)*x_ref[2]
        ref[3,3,1,1]=(-0.924260987477)*x_ref[0]**o+(0.771970726902)*x_ref[0]+(-0.909099224213)*x_ref[1]**o+(-0.45204206766)*x_ref[1]+(0.630042614017)*x_ref[2]**o+(-0.990915130464)*x_ref[2]
        ref[3,3,2,0]=(0.734962154852)*x_ref[0]**o+(-0.00621898011475)*x_ref[0]+(-0.786362527189)*x_ref[1]**o+(0.623507947406)*x_ref[1]+(0.867175567104)*x_ref[2]**o+(-0.383283255448)*x_ref[2]
        ref[3,3,2,1]=(-0.955613156717)*x_ref[0]**o+(0.688013754841)*x_ref[0]+(-0.600559375208)*x_ref[1]**o+(0.453342626645)*x_ref[1]+(-0.864890477961)*x_ref[2]**o+(-0.458029873063)*x_ref[2]
        ref[3,4,0,0]=(-0.897010884834)*x_ref[0]**o+(-0.251223576458)*x_ref[0]+(-0.0338052047964)*x_ref[1]**o+(-0.903421466132)*x_ref[1]+(0.836864217874)*x_ref[2]**o+(0.660805905175)*x_ref[2]
        ref[3,4,0,1]=(-0.286025690215)*x_ref[0]**o+(0.746732975604)*x_ref[0]+(0.375810853218)*x_ref[1]**o+(-0.985192974869)*x_ref[1]+(-0.731463214455)*x_ref[2]**o+(-0.465450352444)*x_ref[2]
        ref[3,4,1,0]=(-0.500910000652)*x_ref[0]**o+(0.387502419727)*x_ref[0]+(0.962369164802)*x_ref[1]**o+(-0.10933824251)*x_ref[1]+(-0.33642703989)*x_ref[2]**o+(0.546851133256)*x_ref[2]
        ref[3,4,1,1]=(-0.117959459645)*x_ref[0]**o+(0.867699323392)*x_ref[0]+(0.950506403861)*x_ref[1]**o+(0.613806207745)*x_ref[1]+(0.425723173807)*x_ref[2]**o+(-0.906937548619)*x_ref[2]
        ref[3,4,2,0]=(0.331114892194)*x_ref[0]**o+(-0.168829643424)*x_ref[0]+(-0.855962311982)*x_ref[1]**o+(-0.144202406673)*x_ref[1]+(0.0769985713088)*x_ref[2]**o+(0.856003746423)*x_ref[2]
        ref[3,4,2,1]=(-0.439397439364)*x_ref[0]**o+(0.686116343481)*x_ref[0]+(0.519883854523)*x_ref[1]**o+(0.716231469289)*x_ref[1]+(0.270098231149)*x_ref[2]**o+(-0.448525436427)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_ReducedSolution_rank0(self):
      """
      tests interpolate for rank 0 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedSolution onto ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Data(0,(),w)
      ref=Data(0,(),w_ref)
      if dim==2:
        arg=(-0.787646538574)*x[0]+(0.912422904218)*x[1]
        ref=(-0.787646538574)*x_ref[0]+(0.912422904218)*x_ref[1]
      else:
        arg=(0.934044939024)*x[0]+(0.628477728603)*x[1]+(-0.786339405837)*x[2]
        ref=(0.934044939024)*x_ref[0]+(0.628477728603)*x_ref[1]+(-0.786339405837)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_ReducedSolution_rank1(self):
      """
      tests interpolate for rank 1 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedSolution onto ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Data(0,(2,),w)
      ref=Data(0,(2,),w_ref)
      if dim==2:
        arg[0]=(-0.588326094839)*x[0]+(1.56296941343)*x[1]
        arg[1]=(-0.802191222185)*x[0]+(-1.13471016062)*x[1]
        ref[0]=(-0.588326094839)*x[0]+(1.56296941343)*x[1]
        ref[1]=(-0.802191222185)*x[0]+(-1.13471016062)*x[1]
      else:
        arg[0]=(0.160702771321)*x[0]+(-0.0784425374814)*x[1]+(1.65226639722)*x[2]
        arg[1]=(0.745050820919)*x[0]+(-0.183859360073)*x[1]+(0.399813139488)*x[2]
        ref[0]=(0.160702771321)*x[0]+(-0.0784425374814)*x[1]+(1.65226639722)*x[2]
        ref[1]=(0.745050820919)*x[0]+(-0.183859360073)*x[1]+(0.399813139488)*x[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_ReducedSolution_rank2(self):
      """
      tests interpolate for rank 2 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedSolution onto ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5),w)
      ref=Data(0,(4, 5),w_ref)
      if dim==2:
        arg[0,0]=(-0.797066914479)*x[0]+(-0.0269853164939)*x[1]
        arg[0,1]=(-1.07443661798)*x[0]+(0.818510216043)*x[1]
        arg[0,2]=(0.398735262179)*x[0]+(-0.580387872083)*x[1]
        arg[0,3]=(-0.003185613959)*x[0]+(-0.670506330615)*x[1]
        arg[0,4]=(0.385009570186)*x[0]+(0.858570512936)*x[1]
        arg[1,0]=(1.60917176753)*x[0]+(-0.0866362449992)*x[1]
        arg[1,1]=(-0.883782122452)*x[0]+(-0.402187705578)*x[1]
        arg[1,2]=(0.153893095122)*x[0]+(-1.19636362644)*x[1]
        arg[1,3]=(1.92041961031)*x[0]+(0.325857877636)*x[1]
        arg[1,4]=(-0.0947698843729)*x[0]+(-0.0663785758627)*x[1]
        arg[2,0]=(0.0232949467051)*x[0]+(1.60134210326)*x[1]
        arg[2,1]=(-0.521182882924)*x[0]+(-0.507401571343)*x[1]
        arg[2,2]=(0.833797117616)*x[0]+(-0.268886477194)*x[1]
        arg[2,3]=(-0.867762611227)*x[0]+(1.03727088566)*x[1]
        arg[2,4]=(0.380510651918)*x[0]+(0.350246888456)*x[1]
        arg[3,0]=(-1.10155500839)*x[0]+(0.414683749118)*x[1]
        arg[3,1]=(-0.0306733233552)*x[0]+(-0.351251014237)*x[1]
        arg[3,2]=(-0.0708773001498)*x[0]+(-0.326636587948)*x[1]
        arg[3,3]=(-1.32553119895)*x[0]+(-1.5852157477)*x[1]
        arg[3,4]=(1.40177154802)*x[0]+(-1.24804787994)*x[1]
        ref[0,0]=(-0.797066914479)*x_ref[0]+(-0.0269853164939)*x_ref[1]
        ref[0,1]=(-1.07443661798)*x_ref[0]+(0.818510216043)*x_ref[1]
        ref[0,2]=(0.398735262179)*x_ref[0]+(-0.580387872083)*x_ref[1]
        ref[0,3]=(-0.003185613959)*x_ref[0]+(-0.670506330615)*x_ref[1]
        ref[0,4]=(0.385009570186)*x_ref[0]+(0.858570512936)*x_ref[1]
        ref[1,0]=(1.60917176753)*x_ref[0]+(-0.0866362449992)*x_ref[1]
        ref[1,1]=(-0.883782122452)*x_ref[0]+(-0.402187705578)*x_ref[1]
        ref[1,2]=(0.153893095122)*x_ref[0]+(-1.19636362644)*x_ref[1]
        ref[1,3]=(1.92041961031)*x_ref[0]+(0.325857877636)*x_ref[1]
        ref[1,4]=(-0.0947698843729)*x_ref[0]+(-0.0663785758627)*x_ref[1]
        ref[2,0]=(0.0232949467051)*x_ref[0]+(1.60134210326)*x_ref[1]
        ref[2,1]=(-0.521182882924)*x_ref[0]+(-0.507401571343)*x_ref[1]
        ref[2,2]=(0.833797117616)*x_ref[0]+(-0.268886477194)*x_ref[1]
        ref[2,3]=(-0.867762611227)*x_ref[0]+(1.03727088566)*x_ref[1]
        ref[2,4]=(0.380510651918)*x_ref[0]+(0.350246888456)*x_ref[1]
        ref[3,0]=(-1.10155500839)*x_ref[0]+(0.414683749118)*x_ref[1]
        ref[3,1]=(-0.0306733233552)*x_ref[0]+(-0.351251014237)*x_ref[1]
        ref[3,2]=(-0.0708773001498)*x_ref[0]+(-0.326636587948)*x_ref[1]
        ref[3,3]=(-1.32553119895)*x_ref[0]+(-1.5852157477)*x_ref[1]
        ref[3,4]=(1.40177154802)*x_ref[0]+(-1.24804787994)*x_ref[1]
      else:
        arg[0,0]=(0.000362194678523)*x[0]+(-1.63949220671)*x[1]+(-0.176997156562)*x[2]
        arg[0,1]=(-0.357589049408)*x[0]+(0.828191515325)*x[1]+(-0.269706780136)*x[2]
        arg[0,2]=(0.886395070544)*x[0]+(0.815688451839)*x[1]+(-0.658412270558)*x[2]
        arg[0,3]=(-0.222573318026)*x[0]+(0.137763803092)*x[1]+(0.171902150187)*x[2]
        arg[0,4]=(1.05159741251)*x[0]+(0.923836841038)*x[1]+(-0.426590801731)*x[2]
        arg[1,0]=(-0.00718419057788)*x[0]+(1.55809964998)*x[1]+(-0.912766657496)*x[2]
        arg[1,1]=(1.30769045785)*x[0]+(-0.0356609338389)*x[1]+(0.214543084134)*x[2]
        arg[1,2]=(-1.56763014241)*x[0]+(-0.582970995722)*x[1]+(-0.513972924752)*x[2]
        arg[1,3]=(0.118027736928)*x[0]+(-0.28795014876)*x[1]+(0.620718623034)*x[2]
        arg[1,4]=(-0.784496170193)*x[0]+(0.233037866924)*x[1]+(-0.0500718134572)*x[2]
        arg[2,0]=(0.800851281806)*x[0]+(0.219346484846)*x[1]+(1.19125787701)*x[2]
        arg[2,1]=(-1.45025879429)*x[0]+(-1.32011538154)*x[1]+(0.47946669308)*x[2]
        arg[2,2]=(-0.231377110469)*x[0]+(-0.921551695999)*x[1]+(-0.298964635841)*x[2]
        arg[2,3]=(0.965766335546)*x[0]+(0.424791610089)*x[1]+(1.01860652092)*x[2]
        arg[2,4]=(0.676247093636)*x[0]+(-1.50802741511)*x[1]+(0.0385960918391)*x[2]
        arg[3,0]=(0.322378151756)*x[0]+(-0.483633005509)*x[1]+(-0.852371707865)*x[2]
        arg[3,1]=(-0.241998307153)*x[0]+(-1.08134340082)*x[1]+(-0.849574532537)*x[2]
        arg[3,2]=(-0.0185751405467)*x[0]+(0.179558416001)*x[1]+(-0.488727308548)*x[2]
        arg[3,3]=(0.0984555057845)*x[0]+(1.03085967788)*x[1]+(-0.357009094371)*x[2]
        arg[3,4]=(0.430593980121)*x[0]+(0.689565614489)*x[1]+(-0.0916541808491)*x[2]
        ref[0,0]=(0.000362194678523)*x_ref[0]+(-1.63949220671)*x_ref[1]+(-0.176997156562)*x_ref[2]
        ref[0,1]=(-0.357589049408)*x_ref[0]+(0.828191515325)*x_ref[1]+(-0.269706780136)*x_ref[2]
        ref[0,2]=(0.886395070544)*x_ref[0]+(0.815688451839)*x_ref[1]+(-0.658412270558)*x_ref[2]
        ref[0,3]=(-0.222573318026)*x_ref[0]+(0.137763803092)*x_ref[1]+(0.171902150187)*x_ref[2]
        ref[0,4]=(1.05159741251)*x_ref[0]+(0.923836841038)*x_ref[1]+(-0.426590801731)*x_ref[2]
        ref[1,0]=(-0.00718419057788)*x_ref[0]+(1.55809964998)*x_ref[1]+(-0.912766657496)*x_ref[2]
        ref[1,1]=(1.30769045785)*x_ref[0]+(-0.0356609338389)*x_ref[1]+(0.214543084134)*x_ref[2]
        ref[1,2]=(-1.56763014241)*x_ref[0]+(-0.582970995722)*x_ref[1]+(-0.513972924752)*x_ref[2]
        ref[1,3]=(0.118027736928)*x_ref[0]+(-0.28795014876)*x_ref[1]+(0.620718623034)*x_ref[2]
        ref[1,4]=(-0.784496170193)*x_ref[0]+(0.233037866924)*x_ref[1]+(-0.0500718134572)*x_ref[2]
        ref[2,0]=(0.800851281806)*x_ref[0]+(0.219346484846)*x_ref[1]+(1.19125787701)*x_ref[2]
        ref[2,1]=(-1.45025879429)*x_ref[0]+(-1.32011538154)*x_ref[1]+(0.47946669308)*x_ref[2]
        ref[2,2]=(-0.231377110469)*x_ref[0]+(-0.921551695999)*x_ref[1]+(-0.298964635841)*x_ref[2]
        ref[2,3]=(0.965766335546)*x_ref[0]+(0.424791610089)*x_ref[1]+(1.01860652092)*x_ref[2]
        ref[2,4]=(0.676247093636)*x_ref[0]+(-1.50802741511)*x_ref[1]+(0.0385960918391)*x_ref[2]
        ref[3,0]=(0.322378151756)*x_ref[0]+(-0.483633005509)*x_ref[1]+(-0.852371707865)*x_ref[2]
        ref[3,1]=(-0.241998307153)*x_ref[0]+(-1.08134340082)*x_ref[1]+(-0.849574532537)*x_ref[2]
        ref[3,2]=(-0.0185751405467)*x_ref[0]+(0.179558416001)*x_ref[1]+(-0.488727308548)*x_ref[2]
        ref[3,3]=(0.0984555057845)*x_ref[0]+(1.03085967788)*x_ref[1]+(-0.357009094371)*x_ref[2]
        ref[3,4]=(0.430593980121)*x_ref[0]+(0.689565614489)*x_ref[1]+(-0.0916541808491)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_ReducedSolution_rank3(self):
      """
      tests interpolate for rank 3 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedSolution onto ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2),w_ref)
      if dim==2:
        arg[0,0,0]=(-0.0641544023301)*x[0]+(0.364286357763)*x[1]
        arg[0,0,1]=(-0.858288240411)*x[0]+(-0.847764393559)*x[1]
        arg[0,1,0]=(-0.656948775029)*x[0]+(1.69748608992)*x[1]
        arg[0,1,1]=(0.579399191549)*x[0]+(1.48407368783)*x[1]
        arg[1,0,0]=(-0.35948657403)*x[0]+(0.795436090765)*x[1]
        arg[1,0,1]=(-1.32807156246)*x[0]+(0.921960610294)*x[1]
        arg[1,1,0]=(1.0904085219)*x[0]+(-1.41870997296)*x[1]
        arg[1,1,1]=(-0.677691596793)*x[0]+(-0.395373706333)*x[1]
        arg[2,0,0]=(0.295591917216)*x[0]+(-1.75550830457)*x[1]
        arg[2,0,1]=(0.506665326626)*x[0]+(0.273265713965)*x[1]
        arg[2,1,0]=(0.543899865352)*x[0]+(-0.0678457091657)*x[1]
        arg[2,1,1]=(-0.887824769535)*x[0]+(0.527475538055)*x[1]
        arg[3,0,0]=(-0.788511557365)*x[0]+(-0.635972100675)*x[1]
        arg[3,0,1]=(-1.02799066459)*x[0]+(-0.416241247725)*x[1]
        arg[3,1,0]=(-0.310072483091)*x[0]+(1.2445160445)*x[1]
        arg[3,1,1]=(0.912462196567)*x[0]+(0.631855269783)*x[1]
        arg[4,0,0]=(0.700426861807)*x[0]+(-0.491365913713)*x[1]
        arg[4,0,1]=(0.628791538338)*x[0]+(0.640767645586)*x[1]
        arg[4,1,0]=(0.370886720329)*x[0]+(0.823663448834)*x[1]
        arg[4,1,1]=(1.75857310774)*x[0]+(0.445439424641)*x[1]
        arg[5,0,0]=(1.39637385066)*x[0]+(-0.812211860169)*x[1]
        arg[5,0,1]=(0.865050350195)*x[0]+(-1.25983380375)*x[1]
        arg[5,1,0]=(-0.0429057645445)*x[0]+(-1.08490578934)*x[1]
        arg[5,1,1]=(1.42851264594)*x[0]+(1.52192971749)*x[1]
        ref[0,0,0]=(-0.0641544023301)*x_ref[0]+(0.364286357763)*x_ref[1]
        ref[0,0,1]=(-0.858288240411)*x_ref[0]+(-0.847764393559)*x_ref[1]
        ref[0,1,0]=(-0.656948775029)*x_ref[0]+(1.69748608992)*x_ref[1]
        ref[0,1,1]=(0.579399191549)*x_ref[0]+(1.48407368783)*x_ref[1]
        ref[1,0,0]=(-0.35948657403)*x_ref[0]+(0.795436090765)*x_ref[1]
        ref[1,0,1]=(-1.32807156246)*x_ref[0]+(0.921960610294)*x_ref[1]
        ref[1,1,0]=(1.0904085219)*x_ref[0]+(-1.41870997296)*x_ref[1]
        ref[1,1,1]=(-0.677691596793)*x_ref[0]+(-0.395373706333)*x_ref[1]
        ref[2,0,0]=(0.295591917216)*x_ref[0]+(-1.75550830457)*x_ref[1]
        ref[2,0,1]=(0.506665326626)*x_ref[0]+(0.273265713965)*x_ref[1]
        ref[2,1,0]=(0.543899865352)*x_ref[0]+(-0.0678457091657)*x_ref[1]
        ref[2,1,1]=(-0.887824769535)*x_ref[0]+(0.527475538055)*x_ref[1]
        ref[3,0,0]=(-0.788511557365)*x_ref[0]+(-0.635972100675)*x_ref[1]
        ref[3,0,1]=(-1.02799066459)*x_ref[0]+(-0.416241247725)*x_ref[1]
        ref[3,1,0]=(-0.310072483091)*x_ref[0]+(1.2445160445)*x_ref[1]
        ref[3,1,1]=(0.912462196567)*x_ref[0]+(0.631855269783)*x_ref[1]
        ref[4,0,0]=(0.700426861807)*x_ref[0]+(-0.491365913713)*x_ref[1]
        ref[4,0,1]=(0.628791538338)*x_ref[0]+(0.640767645586)*x_ref[1]
        ref[4,1,0]=(0.370886720329)*x_ref[0]+(0.823663448834)*x_ref[1]
        ref[4,1,1]=(1.75857310774)*x_ref[0]+(0.445439424641)*x_ref[1]
        ref[5,0,0]=(1.39637385066)*x_ref[0]+(-0.812211860169)*x_ref[1]
        ref[5,0,1]=(0.865050350195)*x_ref[0]+(-1.25983380375)*x_ref[1]
        ref[5,1,0]=(-0.0429057645445)*x_ref[0]+(-1.08490578934)*x_ref[1]
        ref[5,1,1]=(1.42851264594)*x_ref[0]+(1.52192971749)*x_ref[1]
      else:
        arg[0,0,0]=(0.731194732468)*x[0]+(-0.0822115660022)*x[1]+(-0.403175091289)*x[2]
        arg[0,0,1]=(0.39292230307)*x[0]+(1.48598681615)*x[1]+(1.05545374107)*x[2]
        arg[0,1,0]=(0.306844260706)*x[0]+(0.836496026763)*x[1]+(-0.121120983155)*x[2]
        arg[0,1,1]=(-0.155245802062)*x[0]+(0.800232934864)*x[1]+(0.373397160636)*x[2]
        arg[1,0,0]=(0.155941790359)*x[0]+(0.826821969281)*x[1]+(0.849596807425)*x[2]
        arg[1,0,1]=(-0.0988854656187)*x[0]+(-0.855032499842)*x[1]+(0.141988288566)*x[2]
        arg[1,1,0]=(-0.761032539296)*x[0]+(-0.0892979073234)*x[1]+(1.93034385132)*x[2]
        arg[1,1,1]=(-1.08303603558)*x[0]+(-0.487250117659)*x[1]+(0.0358501036173)*x[2]
        arg[2,0,0]=(0.148391465413)*x[0]+(-0.731963025625)*x[1]+(0.484236965689)*x[2]
        arg[2,0,1]=(1.06691217722)*x[0]+(-0.638095266244)*x[1]+(-0.971838410779)*x[2]
        arg[2,1,0]=(-0.377750829061)*x[0]+(-0.101467967263)*x[1]+(-1.54241387349)*x[2]
        arg[2,1,1]=(0.62210350231)*x[0]+(0.0435334873911)*x[1]+(-0.299303646623)*x[2]
        arg[3,0,0]=(0.523811147108)*x[0]+(-1.09709839108)*x[1]+(1.83592280935)*x[2]
        arg[3,0,1]=(0.957738143475)*x[0]+(0.282158440867)*x[1]+(0.621694180522)*x[2]
        arg[3,1,0]=(0.366557891822)*x[0]+(0.812194047631)*x[1]+(-0.245067876349)*x[2]
        arg[3,1,1]=(0.604859464982)*x[0]+(1.08415514854)*x[1]+(-1.470128848)*x[2]
        arg[4,0,0]=(-0.13031502499)*x[0]+(-1.20707238733)*x[1]+(0.108382497552)*x[2]
        arg[4,0,1]=(1.30453246317)*x[0]+(1.59696612588)*x[1]+(-0.234259043697)*x[2]
        arg[4,1,0]=(-0.233494640836)*x[0]+(1.10701061816)*x[1]+(-0.409927013674)*x[2]
        arg[4,1,1]=(0.102708240266)*x[0]+(-0.381598345002)*x[1]+(-1.39077748656)*x[2]
        arg[5,0,0]=(1.06217404493)*x[0]+(1.1600791444)*x[1]+(-1.45242735389)*x[2]
        arg[5,0,1]=(-0.423024189506)*x[0]+(-0.0868948651641)*x[1]+(0.0894539043522)*x[2]
        arg[5,1,0]=(1.17650613067)*x[0]+(0.993832919965)*x[1]+(1.16546768569)*x[2]
        arg[5,1,1]=(-0.61549132897)*x[0]+(-0.499619662734)*x[1]+(-0.643422281669)*x[2]
        ref[0,0,0]=(0.731194732468)*x_ref[0]+(-0.0822115660022)*x_ref[1]+(-0.403175091289)*x_ref[2]
        ref[0,0,1]=(0.39292230307)*x_ref[0]+(1.48598681615)*x_ref[1]+(1.05545374107)*x_ref[2]
        ref[0,1,0]=(0.306844260706)*x_ref[0]+(0.836496026763)*x_ref[1]+(-0.121120983155)*x_ref[2]
        ref[0,1,1]=(-0.155245802062)*x_ref[0]+(0.800232934864)*x_ref[1]+(0.373397160636)*x_ref[2]
        ref[1,0,0]=(0.155941790359)*x_ref[0]+(0.826821969281)*x_ref[1]+(0.849596807425)*x_ref[2]
        ref[1,0,1]=(-0.0988854656187)*x_ref[0]+(-0.855032499842)*x_ref[1]+(0.141988288566)*x_ref[2]
        ref[1,1,0]=(-0.761032539296)*x_ref[0]+(-0.0892979073234)*x_ref[1]+(1.93034385132)*x_ref[2]
        ref[1,1,1]=(-1.08303603558)*x_ref[0]+(-0.487250117659)*x_ref[1]+(0.0358501036173)*x_ref[2]
        ref[2,0,0]=(0.148391465413)*x_ref[0]+(-0.731963025625)*x_ref[1]+(0.484236965689)*x_ref[2]
        ref[2,0,1]=(1.06691217722)*x_ref[0]+(-0.638095266244)*x_ref[1]+(-0.971838410779)*x_ref[2]
        ref[2,1,0]=(-0.377750829061)*x_ref[0]+(-0.101467967263)*x_ref[1]+(-1.54241387349)*x_ref[2]
        ref[2,1,1]=(0.62210350231)*x_ref[0]+(0.0435334873911)*x_ref[1]+(-0.299303646623)*x_ref[2]
        ref[3,0,0]=(0.523811147108)*x_ref[0]+(-1.09709839108)*x_ref[1]+(1.83592280935)*x_ref[2]
        ref[3,0,1]=(0.957738143475)*x_ref[0]+(0.282158440867)*x_ref[1]+(0.621694180522)*x_ref[2]
        ref[3,1,0]=(0.366557891822)*x_ref[0]+(0.812194047631)*x_ref[1]+(-0.245067876349)*x_ref[2]
        ref[3,1,1]=(0.604859464982)*x_ref[0]+(1.08415514854)*x_ref[1]+(-1.470128848)*x_ref[2]
        ref[4,0,0]=(-0.13031502499)*x_ref[0]+(-1.20707238733)*x_ref[1]+(0.108382497552)*x_ref[2]
        ref[4,0,1]=(1.30453246317)*x_ref[0]+(1.59696612588)*x_ref[1]+(-0.234259043697)*x_ref[2]
        ref[4,1,0]=(-0.233494640836)*x_ref[0]+(1.10701061816)*x_ref[1]+(-0.409927013674)*x_ref[2]
        ref[4,1,1]=(0.102708240266)*x_ref[0]+(-0.381598345002)*x_ref[1]+(-1.39077748656)*x_ref[2]
        ref[5,0,0]=(1.06217404493)*x_ref[0]+(1.1600791444)*x_ref[1]+(-1.45242735389)*x_ref[2]
        ref[5,0,1]=(-0.423024189506)*x_ref[0]+(-0.0868948651641)*x_ref[1]+(0.0894539043522)*x_ref[2]
        ref[5,1,0]=(1.17650613067)*x_ref[0]+(0.993832919965)*x_ref[1]+(1.16546768569)*x_ref[2]
        ref[5,1,1]=(-0.61549132897)*x_ref[0]+(-0.499619662734)*x_ref[1]+(-0.643422281669)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_ReducedSolution_rank4(self):
      """
      tests interpolate for rank 4 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedSolution onto ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5, 3, 2),w)
      ref=Data(0,(4, 5, 3, 2),w_ref)
      if dim==2:
        arg[0,0,0,0]=(0.227969627252)*x[0]+(0.778177350248)*x[1]
        arg[0,0,0,1]=(-0.684883711494)*x[0]+(-0.574285957132)*x[1]
        arg[0,0,1,0]=(0.483090608974)*x[0]+(0.0779929087959)*x[1]
        arg[0,0,1,1]=(-0.293150814932)*x[0]+(1.35368931993)*x[1]
        arg[0,0,2,0]=(-0.190664734662)*x[0]+(0.910883386754)*x[1]
        arg[0,0,2,1]=(-1.51255575457)*x[0]+(0.421560181501)*x[1]
        arg[0,1,0,0]=(-0.833402828032)*x[0]+(-0.400466806781)*x[1]
        arg[0,1,0,1]=(-0.212099323605)*x[0]+(-0.112478697221)*x[1]
        arg[0,1,1,0]=(0.57687662926)*x[0]+(0.869758954551)*x[1]
        arg[0,1,1,1]=(1.37261769917)*x[0]+(1.37855789907)*x[1]
        arg[0,1,2,0]=(-1.02838589722)*x[0]+(-0.280409246204)*x[1]
        arg[0,1,2,1]=(1.2333373468)*x[0]+(0.337631183798)*x[1]
        arg[0,2,0,0]=(-0.249065448772)*x[0]+(0.834483646317)*x[1]
        arg[0,2,0,1]=(-1.30860297683)*x[0]+(-0.653929967017)*x[1]
        arg[0,2,1,0]=(-0.0155994484453)*x[0]+(-0.274023166937)*x[1]
        arg[0,2,1,1]=(0.31458360335)*x[0]+(1.59507700673)*x[1]
        arg[0,2,2,0]=(-0.11442619553)*x[0]+(1.6577028576)*x[1]
        arg[0,2,2,1]=(0.274261132032)*x[0]+(1.15236962479)*x[1]
        arg[0,3,0,0]=(1.91433860239)*x[0]+(-0.316349747811)*x[1]
        arg[0,3,0,1]=(-1.42457591054)*x[0]+(-0.362812690805)*x[1]
        arg[0,3,1,0]=(0.572734883096)*x[0]+(0.424724787283)*x[1]
        arg[0,3,1,1]=(-0.118538376813)*x[0]+(0.25190312527)*x[1]
        arg[0,3,2,0]=(-0.533409510081)*x[0]+(-0.804349107915)*x[1]
        arg[0,3,2,1]=(-0.290555363666)*x[0]+(-0.912641477145)*x[1]
        arg[0,4,0,0]=(-0.469612775918)*x[0]+(-0.0447411310793)*x[1]
        arg[0,4,0,1]=(0.108538258831)*x[0]+(0.0453962016662)*x[1]
        arg[0,4,1,0]=(-0.130785944028)*x[0]+(0.0887280914032)*x[1]
        arg[0,4,1,1]=(0.166412309585)*x[0]+(-0.75130797706)*x[1]
        arg[0,4,2,0]=(1.22538232455)*x[0]+(0.112925588343)*x[1]
        arg[0,4,2,1]=(-1.02455171854)*x[0]+(-0.725754942749)*x[1]
        arg[1,0,0,0]=(-0.869171086952)*x[0]+(-0.948876783682)*x[1]
        arg[1,0,0,1]=(0.17730673689)*x[0]+(-0.00166180855579)*x[1]
        arg[1,0,1,0]=(0.482072704433)*x[0]+(0.895847096783)*x[1]
        arg[1,0,1,1]=(0.695736834617)*x[0]+(0.358082704558)*x[1]
        arg[1,0,2,0]=(-0.501799636565)*x[0]+(0.363612558025)*x[1]
        arg[1,0,2,1]=(-1.47204447226)*x[0]+(1.91281426121)*x[1]
        arg[1,1,0,0]=(0.241893182978)*x[0]+(0.95199758669)*x[1]
        arg[1,1,0,1]=(-0.68112361827)*x[0]+(-0.241935141301)*x[1]
        arg[1,1,1,0]=(-0.553030972306)*x[0]+(0.0244503690439)*x[1]
        arg[1,1,1,1]=(0.278177176896)*x[0]+(1.58693846427)*x[1]
        arg[1,1,2,0]=(-0.87980015517)*x[0]+(0.315618837123)*x[1]
        arg[1,1,2,1]=(-0.335783961379)*x[0]+(0.368650136138)*x[1]
        arg[1,2,0,0]=(-0.903535146802)*x[0]+(1.68182923194)*x[1]
        arg[1,2,0,1]=(1.06511511382)*x[0]+(-1.08524762302)*x[1]
        arg[1,2,1,0]=(-0.0455274784681)*x[0]+(-0.519576710661)*x[1]
        arg[1,2,1,1]=(0.735100217062)*x[0]+(-0.986268682671)*x[1]
        arg[1,2,2,0]=(-0.544610797462)*x[0]+(0.447237114882)*x[1]
        arg[1,2,2,1]=(0.385242031153)*x[0]+(-0.960538844247)*x[1]
        arg[1,3,0,0]=(-0.354927072897)*x[0]+(0.842574324062)*x[1]
        arg[1,3,0,1]=(0.319036427919)*x[0]+(-0.156282262536)*x[1]
        arg[1,3,1,0]=(0.587123807717)*x[0]+(1.65277990726)*x[1]
        arg[1,3,1,1]=(0.791697370298)*x[0]+(-1.5681680344)*x[1]
        arg[1,3,2,0]=(-1.41931242045)*x[0]+(0.351285136557)*x[1]
        arg[1,3,2,1]=(0.53175704402)*x[0]+(0.987523241324)*x[1]
        arg[1,4,0,0]=(0.00533504818465)*x[0]+(-0.158886930817)*x[1]
        arg[1,4,0,1]=(1.23725297303)*x[0]+(-0.302896452172)*x[1]
        arg[1,4,1,0]=(0.433953146329)*x[0]+(-0.0721602234887)*x[1]
        arg[1,4,1,1]=(-0.383499726865)*x[0]+(-0.253834671235)*x[1]
        arg[1,4,2,0]=(-0.480776695247)*x[0]+(-0.302145524759)*x[1]
        arg[1,4,2,1]=(1.14113180903)*x[0]+(-1.24304726003)*x[1]
        arg[2,0,0,0]=(0.513776013893)*x[0]+(-0.16660259885)*x[1]
        arg[2,0,0,1]=(1.55739209698)*x[0]+(0.0524084654531)*x[1]
        arg[2,0,1,0]=(0.222567881446)*x[0]+(-0.357158470388)*x[1]
        arg[2,0,1,1]=(0.0179816596229)*x[0]+(0.231534460784)*x[1]
        arg[2,0,2,0]=(1.58460345297)*x[0]+(-0.383588433888)*x[1]
        arg[2,0,2,1]=(-1.73386465536)*x[0]+(-0.743326657544)*x[1]
        arg[2,1,0,0]=(-0.737976950246)*x[0]+(-1.01053396224)*x[1]
        arg[2,1,0,1]=(0.926911190393)*x[0]+(0.620251783925)*x[1]
        arg[2,1,1,0]=(-1.34289630129)*x[0]+(-0.429933681523)*x[1]
        arg[2,1,1,1]=(1.79530806415)*x[0]+(-0.539783957021)*x[1]
        arg[2,1,2,0]=(0.237950974938)*x[0]+(1.93783427703)*x[1]
        arg[2,1,2,1]=(-0.439640741973)*x[0]+(-0.61543984673)*x[1]
        arg[2,2,0,0]=(-1.66403612438)*x[0]+(1.00839566441)*x[1]
        arg[2,2,0,1]=(0.249635983351)*x[0]+(-0.722261363675)*x[1]
        arg[2,2,1,0]=(1.34179674459)*x[0]+(0.502542144188)*x[1]
        arg[2,2,1,1]=(-0.815862586074)*x[0]+(-0.110688273707)*x[1]
        arg[2,2,2,0]=(-0.686810719526)*x[0]+(0.500406563421)*x[1]
        arg[2,2,2,1]=(0.681231089598)*x[0]+(0.605219505613)*x[1]
        arg[2,3,0,0]=(-0.953299029956)*x[0]+(-0.223725858372)*x[1]
        arg[2,3,0,1]=(0.831380756684)*x[0]+(-0.0379777076335)*x[1]
        arg[2,3,1,0]=(-1.63804393157)*x[0]+(0.148235731301)*x[1]
        arg[2,3,1,1]=(-0.17786458643)*x[0]+(-1.77342152699)*x[1]
        arg[2,3,2,0]=(0.0210446244192)*x[0]+(1.59778781524)*x[1]
        arg[2,3,2,1]=(0.877162721214)*x[0]+(0.262990448614)*x[1]
        arg[2,4,0,0]=(-0.650265970331)*x[0]+(-1.07003604369)*x[1]
        arg[2,4,0,1]=(1.12940101029)*x[0]+(0.860589290891)*x[1]
        arg[2,4,1,0]=(0.520056210056)*x[0]+(-1.47691193979)*x[1]
        arg[2,4,1,1]=(1.26823535235)*x[0]+(0.496811720647)*x[1]
        arg[2,4,2,0]=(-1.388913443)*x[0]+(0.585343197943)*x[1]
        arg[2,4,2,1]=(-0.762904547594)*x[0]+(0.503438815917)*x[1]
        arg[3,0,0,0]=(0.382862686993)*x[0]+(-0.923525513303)*x[1]
        arg[3,0,0,1]=(-0.54305794485)*x[0]+(0.467513531444)*x[1]
        arg[3,0,1,0]=(1.24228759776)*x[0]+(-1.65740117421)*x[1]
        arg[3,0,1,1]=(-0.463045362742)*x[0]+(0.186438587087)*x[1]
        arg[3,0,2,0]=(0.296881980828)*x[0]+(-0.385878814949)*x[1]
        arg[3,0,2,1]=(-0.299705899564)*x[0]+(0.490736038538)*x[1]
        arg[3,1,0,0]=(-0.786851428658)*x[0]+(-0.760593794308)*x[1]
        arg[3,1,0,1]=(0.506851336483)*x[0]+(0.995163951197)*x[1]
        arg[3,1,1,0]=(-0.678883937341)*x[0]+(0.56996778248)*x[1]
        arg[3,1,1,1]=(-0.401483642528)*x[0]+(0.0476797332537)*x[1]
        arg[3,1,2,0]=(-0.585357955375)*x[0]+(0.330103647539)*x[1]
        arg[3,1,2,1]=(-0.355539976825)*x[0]+(0.416287924349)*x[1]
        arg[3,2,0,0]=(-0.662837130286)*x[0]+(-0.26640899674)*x[1]
        arg[3,2,0,1]=(0.397553900588)*x[0]+(-1.21708620294)*x[1]
        arg[3,2,1,0]=(0.464963326429)*x[0]+(-0.774515114142)*x[1]
        arg[3,2,1,1]=(0.834351469791)*x[0]+(0.487629540715)*x[1]
        arg[3,2,2,0]=(0.27622693716)*x[0]+(0.811338337719)*x[1]
        arg[3,2,2,1]=(0.0384997772653)*x[0]+(1.03724996578)*x[1]
        arg[3,3,0,0]=(-0.805491987874)*x[0]+(-0.476384547853)*x[1]
        arg[3,3,0,1]=(0.615551780195)*x[0]+(-0.312805967691)*x[1]
        arg[3,3,1,0]=(0.348666798955)*x[0]+(-1.31271156524)*x[1]
        arg[3,3,1,1]=(-0.423569975374)*x[0]+(0.930713125389)*x[1]
        arg[3,3,2,0]=(0.325632686234)*x[0]+(-0.166401033259)*x[1]
        arg[3,3,2,1]=(0.126924861797)*x[0]+(-0.425380897969)*x[1]
        arg[3,4,0,0]=(0.169532204538)*x[0]+(0.614632525765)*x[1]
        arg[3,4,0,1]=(-0.938098696851)*x[0]+(0.648854147587)*x[1]
        arg[3,4,1,0]=(-0.545143970434)*x[0]+(-1.59787515966)*x[1]
        arg[3,4,1,1]=(-0.711190914978)*x[0]+(0.897331522911)*x[1]
        arg[3,4,2,0]=(-0.386498124525)*x[0]+(0.299740420711)*x[1]
        arg[3,4,2,1]=(0.617683032062)*x[0]+(-0.449587703061)*x[1]
        ref[0,0,0,0]=(0.227969627252)*x_ref[0]+(0.778177350248)*x_ref[1]
        ref[0,0,0,1]=(-0.684883711494)*x_ref[0]+(-0.574285957132)*x_ref[1]
        ref[0,0,1,0]=(0.483090608974)*x_ref[0]+(0.0779929087959)*x_ref[1]
        ref[0,0,1,1]=(-0.293150814932)*x_ref[0]+(1.35368931993)*x_ref[1]
        ref[0,0,2,0]=(-0.190664734662)*x_ref[0]+(0.910883386754)*x_ref[1]
        ref[0,0,2,1]=(-1.51255575457)*x_ref[0]+(0.421560181501)*x_ref[1]
        ref[0,1,0,0]=(-0.833402828032)*x_ref[0]+(-0.400466806781)*x_ref[1]
        ref[0,1,0,1]=(-0.212099323605)*x_ref[0]+(-0.112478697221)*x_ref[1]
        ref[0,1,1,0]=(0.57687662926)*x_ref[0]+(0.869758954551)*x_ref[1]
        ref[0,1,1,1]=(1.37261769917)*x_ref[0]+(1.37855789907)*x_ref[1]
        ref[0,1,2,0]=(-1.02838589722)*x_ref[0]+(-0.280409246204)*x_ref[1]
        ref[0,1,2,1]=(1.2333373468)*x_ref[0]+(0.337631183798)*x_ref[1]
        ref[0,2,0,0]=(-0.249065448772)*x_ref[0]+(0.834483646317)*x_ref[1]
        ref[0,2,0,1]=(-1.30860297683)*x_ref[0]+(-0.653929967017)*x_ref[1]
        ref[0,2,1,0]=(-0.0155994484453)*x_ref[0]+(-0.274023166937)*x_ref[1]
        ref[0,2,1,1]=(0.31458360335)*x_ref[0]+(1.59507700673)*x_ref[1]
        ref[0,2,2,0]=(-0.11442619553)*x_ref[0]+(1.6577028576)*x_ref[1]
        ref[0,2,2,1]=(0.274261132032)*x_ref[0]+(1.15236962479)*x_ref[1]
        ref[0,3,0,0]=(1.91433860239)*x_ref[0]+(-0.316349747811)*x_ref[1]
        ref[0,3,0,1]=(-1.42457591054)*x_ref[0]+(-0.362812690805)*x_ref[1]
        ref[0,3,1,0]=(0.572734883096)*x_ref[0]+(0.424724787283)*x_ref[1]
        ref[0,3,1,1]=(-0.118538376813)*x_ref[0]+(0.25190312527)*x_ref[1]
        ref[0,3,2,0]=(-0.533409510081)*x_ref[0]+(-0.804349107915)*x_ref[1]
        ref[0,3,2,1]=(-0.290555363666)*x_ref[0]+(-0.912641477145)*x_ref[1]
        ref[0,4,0,0]=(-0.469612775918)*x_ref[0]+(-0.0447411310793)*x_ref[1]
        ref[0,4,0,1]=(0.108538258831)*x_ref[0]+(0.0453962016662)*x_ref[1]
        ref[0,4,1,0]=(-0.130785944028)*x_ref[0]+(0.0887280914032)*x_ref[1]
        ref[0,4,1,1]=(0.166412309585)*x_ref[0]+(-0.75130797706)*x_ref[1]
        ref[0,4,2,0]=(1.22538232455)*x_ref[0]+(0.112925588343)*x_ref[1]
        ref[0,4,2,1]=(-1.02455171854)*x_ref[0]+(-0.725754942749)*x_ref[1]
        ref[1,0,0,0]=(-0.869171086952)*x_ref[0]+(-0.948876783682)*x_ref[1]
        ref[1,0,0,1]=(0.17730673689)*x_ref[0]+(-0.00166180855579)*x_ref[1]
        ref[1,0,1,0]=(0.482072704433)*x_ref[0]+(0.895847096783)*x_ref[1]
        ref[1,0,1,1]=(0.695736834617)*x_ref[0]+(0.358082704558)*x_ref[1]
        ref[1,0,2,0]=(-0.501799636565)*x_ref[0]+(0.363612558025)*x_ref[1]
        ref[1,0,2,1]=(-1.47204447226)*x_ref[0]+(1.91281426121)*x_ref[1]
        ref[1,1,0,0]=(0.241893182978)*x_ref[0]+(0.95199758669)*x_ref[1]
        ref[1,1,0,1]=(-0.68112361827)*x_ref[0]+(-0.241935141301)*x_ref[1]
        ref[1,1,1,0]=(-0.553030972306)*x_ref[0]+(0.0244503690439)*x_ref[1]
        ref[1,1,1,1]=(0.278177176896)*x_ref[0]+(1.58693846427)*x_ref[1]
        ref[1,1,2,0]=(-0.87980015517)*x_ref[0]+(0.315618837123)*x_ref[1]
        ref[1,1,2,1]=(-0.335783961379)*x_ref[0]+(0.368650136138)*x_ref[1]
        ref[1,2,0,0]=(-0.903535146802)*x_ref[0]+(1.68182923194)*x_ref[1]
        ref[1,2,0,1]=(1.06511511382)*x_ref[0]+(-1.08524762302)*x_ref[1]
        ref[1,2,1,0]=(-0.0455274784681)*x_ref[0]+(-0.519576710661)*x_ref[1]
        ref[1,2,1,1]=(0.735100217062)*x_ref[0]+(-0.986268682671)*x_ref[1]
        ref[1,2,2,0]=(-0.544610797462)*x_ref[0]+(0.447237114882)*x_ref[1]
        ref[1,2,2,1]=(0.385242031153)*x_ref[0]+(-0.960538844247)*x_ref[1]
        ref[1,3,0,0]=(-0.354927072897)*x_ref[0]+(0.842574324062)*x_ref[1]
        ref[1,3,0,1]=(0.319036427919)*x_ref[0]+(-0.156282262536)*x_ref[1]
        ref[1,3,1,0]=(0.587123807717)*x_ref[0]+(1.65277990726)*x_ref[1]
        ref[1,3,1,1]=(0.791697370298)*x_ref[0]+(-1.5681680344)*x_ref[1]
        ref[1,3,2,0]=(-1.41931242045)*x_ref[0]+(0.351285136557)*x_ref[1]
        ref[1,3,2,1]=(0.53175704402)*x_ref[0]+(0.987523241324)*x_ref[1]
        ref[1,4,0,0]=(0.00533504818465)*x_ref[0]+(-0.158886930817)*x_ref[1]
        ref[1,4,0,1]=(1.23725297303)*x_ref[0]+(-0.302896452172)*x_ref[1]
        ref[1,4,1,0]=(0.433953146329)*x_ref[0]+(-0.0721602234887)*x_ref[1]
        ref[1,4,1,1]=(-0.383499726865)*x_ref[0]+(-0.253834671235)*x_ref[1]
        ref[1,4,2,0]=(-0.480776695247)*x_ref[0]+(-0.302145524759)*x_ref[1]
        ref[1,4,2,1]=(1.14113180903)*x_ref[0]+(-1.24304726003)*x_ref[1]
        ref[2,0,0,0]=(0.513776013893)*x_ref[0]+(-0.16660259885)*x_ref[1]
        ref[2,0,0,1]=(1.55739209698)*x_ref[0]+(0.0524084654531)*x_ref[1]
        ref[2,0,1,0]=(0.222567881446)*x_ref[0]+(-0.357158470388)*x_ref[1]
        ref[2,0,1,1]=(0.0179816596229)*x_ref[0]+(0.231534460784)*x_ref[1]
        ref[2,0,2,0]=(1.58460345297)*x_ref[0]+(-0.383588433888)*x_ref[1]
        ref[2,0,2,1]=(-1.73386465536)*x_ref[0]+(-0.743326657544)*x_ref[1]
        ref[2,1,0,0]=(-0.737976950246)*x_ref[0]+(-1.01053396224)*x_ref[1]
        ref[2,1,0,1]=(0.926911190393)*x_ref[0]+(0.620251783925)*x_ref[1]
        ref[2,1,1,0]=(-1.34289630129)*x_ref[0]+(-0.429933681523)*x_ref[1]
        ref[2,1,1,1]=(1.79530806415)*x_ref[0]+(-0.539783957021)*x_ref[1]
        ref[2,1,2,0]=(0.237950974938)*x_ref[0]+(1.93783427703)*x_ref[1]
        ref[2,1,2,1]=(-0.439640741973)*x_ref[0]+(-0.61543984673)*x_ref[1]
        ref[2,2,0,0]=(-1.66403612438)*x_ref[0]+(1.00839566441)*x_ref[1]
        ref[2,2,0,1]=(0.249635983351)*x_ref[0]+(-0.722261363675)*x_ref[1]
        ref[2,2,1,0]=(1.34179674459)*x_ref[0]+(0.502542144188)*x_ref[1]
        ref[2,2,1,1]=(-0.815862586074)*x_ref[0]+(-0.110688273707)*x_ref[1]
        ref[2,2,2,0]=(-0.686810719526)*x_ref[0]+(0.500406563421)*x_ref[1]
        ref[2,2,2,1]=(0.681231089598)*x_ref[0]+(0.605219505613)*x_ref[1]
        ref[2,3,0,0]=(-0.953299029956)*x_ref[0]+(-0.223725858372)*x_ref[1]
        ref[2,3,0,1]=(0.831380756684)*x_ref[0]+(-0.0379777076335)*x_ref[1]
        ref[2,3,1,0]=(-1.63804393157)*x_ref[0]+(0.148235731301)*x_ref[1]
        ref[2,3,1,1]=(-0.17786458643)*x_ref[0]+(-1.77342152699)*x_ref[1]
        ref[2,3,2,0]=(0.0210446244192)*x_ref[0]+(1.59778781524)*x_ref[1]
        ref[2,3,2,1]=(0.877162721214)*x_ref[0]+(0.262990448614)*x_ref[1]
        ref[2,4,0,0]=(-0.650265970331)*x_ref[0]+(-1.07003604369)*x_ref[1]
        ref[2,4,0,1]=(1.12940101029)*x_ref[0]+(0.860589290891)*x_ref[1]
        ref[2,4,1,0]=(0.520056210056)*x_ref[0]+(-1.47691193979)*x_ref[1]
        ref[2,4,1,1]=(1.26823535235)*x_ref[0]+(0.496811720647)*x_ref[1]
        ref[2,4,2,0]=(-1.388913443)*x_ref[0]+(0.585343197943)*x_ref[1]
        ref[2,4,2,1]=(-0.762904547594)*x_ref[0]+(0.503438815917)*x_ref[1]
        ref[3,0,0,0]=(0.382862686993)*x_ref[0]+(-0.923525513303)*x_ref[1]
        ref[3,0,0,1]=(-0.54305794485)*x_ref[0]+(0.467513531444)*x_ref[1]
        ref[3,0,1,0]=(1.24228759776)*x_ref[0]+(-1.65740117421)*x_ref[1]
        ref[3,0,1,1]=(-0.463045362742)*x_ref[0]+(0.186438587087)*x_ref[1]
        ref[3,0,2,0]=(0.296881980828)*x_ref[0]+(-0.385878814949)*x_ref[1]
        ref[3,0,2,1]=(-0.299705899564)*x_ref[0]+(0.490736038538)*x_ref[1]
        ref[3,1,0,0]=(-0.786851428658)*x_ref[0]+(-0.760593794308)*x_ref[1]
        ref[3,1,0,1]=(0.506851336483)*x_ref[0]+(0.995163951197)*x_ref[1]
        ref[3,1,1,0]=(-0.678883937341)*x_ref[0]+(0.56996778248)*x_ref[1]
        ref[3,1,1,1]=(-0.401483642528)*x_ref[0]+(0.0476797332537)*x_ref[1]
        ref[3,1,2,0]=(-0.585357955375)*x_ref[0]+(0.330103647539)*x_ref[1]
        ref[3,1,2,1]=(-0.355539976825)*x_ref[0]+(0.416287924349)*x_ref[1]
        ref[3,2,0,0]=(-0.662837130286)*x_ref[0]+(-0.26640899674)*x_ref[1]
        ref[3,2,0,1]=(0.397553900588)*x_ref[0]+(-1.21708620294)*x_ref[1]
        ref[3,2,1,0]=(0.464963326429)*x_ref[0]+(-0.774515114142)*x_ref[1]
        ref[3,2,1,1]=(0.834351469791)*x_ref[0]+(0.487629540715)*x_ref[1]
        ref[3,2,2,0]=(0.27622693716)*x_ref[0]+(0.811338337719)*x_ref[1]
        ref[3,2,2,1]=(0.0384997772653)*x_ref[0]+(1.03724996578)*x_ref[1]
        ref[3,3,0,0]=(-0.805491987874)*x_ref[0]+(-0.476384547853)*x_ref[1]
        ref[3,3,0,1]=(0.615551780195)*x_ref[0]+(-0.312805967691)*x_ref[1]
        ref[3,3,1,0]=(0.348666798955)*x_ref[0]+(-1.31271156524)*x_ref[1]
        ref[3,3,1,1]=(-0.423569975374)*x_ref[0]+(0.930713125389)*x_ref[1]
        ref[3,3,2,0]=(0.325632686234)*x_ref[0]+(-0.166401033259)*x_ref[1]
        ref[3,3,2,1]=(0.126924861797)*x_ref[0]+(-0.425380897969)*x_ref[1]
        ref[3,4,0,0]=(0.169532204538)*x_ref[0]+(0.614632525765)*x_ref[1]
        ref[3,4,0,1]=(-0.938098696851)*x_ref[0]+(0.648854147587)*x_ref[1]
        ref[3,4,1,0]=(-0.545143970434)*x_ref[0]+(-1.59787515966)*x_ref[1]
        ref[3,4,1,1]=(-0.711190914978)*x_ref[0]+(0.897331522911)*x_ref[1]
        ref[3,4,2,0]=(-0.386498124525)*x_ref[0]+(0.299740420711)*x_ref[1]
        ref[3,4,2,1]=(0.617683032062)*x_ref[0]+(-0.449587703061)*x_ref[1]
      else:
        arg[0,0,0,0]=(-0.791620170192)*x[0]+(1.03871460301)*x[1]+(-0.225035657687)*x[2]
        arg[0,0,0,1]=(-0.464422319203)*x[0]+(0.769454931053)*x[1]+(-0.783260637771)*x[2]
        arg[0,0,1,0]=(0.72655538215)*x[0]+(-1.65736312015)*x[1]+(1.08402505927)*x[2]
        arg[0,0,1,1]=(-0.207576709573)*x[0]+(-0.55250594761)*x[1]+(-0.923702938288)*x[2]
        arg[0,0,2,0]=(-0.701991085516)*x[0]+(-1.76863643107)*x[1]+(0.831382933104)*x[2]
        arg[0,0,2,1]=(0.0640035527319)*x[0]+(0.425446342976)*x[1]+(1.64626766696)*x[2]
        arg[0,1,0,0]=(1.01022963801)*x[0]+(-0.0215015059716)*x[1]+(0.849862425592)*x[2]
        arg[0,1,0,1]=(-0.45776932943)*x[0]+(0.7621342151)*x[1]+(-1.40123018391)*x[2]
        arg[0,1,1,0]=(0.737485054127)*x[0]+(-0.147513905858)*x[1]+(-0.616928595213)*x[2]
        arg[0,1,1,1]=(-0.0608005391894)*x[0]+(-0.154978210573)*x[1]+(-1.40322011577)*x[2]
        arg[0,1,2,0]=(1.59908177714)*x[0]+(-0.378738573932)*x[1]+(1.45285315247)*x[2]
        arg[0,1,2,1]=(-0.622552157046)*x[0]+(1.19996417516)*x[1]+(0.728842900843)*x[2]
        arg[0,2,0,0]=(-0.46942034691)*x[0]+(0.333309721372)*x[1]+(-0.280846553832)*x[2]
        arg[0,2,0,1]=(1.22208788109)*x[0]+(0.369339594285)*x[1]+(1.31078425589)*x[2]
        arg[0,2,1,0]=(0.0794165004114)*x[0]+(0.44665451846)*x[1]+(-0.814625708916)*x[2]
        arg[0,2,1,1]=(0.594200807165)*x[0]+(-0.00616720333193)*x[1]+(0.382706175172)*x[2]
        arg[0,2,2,0]=(0.0372230170459)*x[0]+(0.440339923291)*x[1]+(0.137641070028)*x[2]
        arg[0,2,2,1]=(-1.29863078274)*x[0]+(0.283142455171)*x[1]+(0.0844540074659)*x[2]
        arg[0,3,0,0]=(1.72811808036)*x[0]+(0.536456781522)*x[1]+(0.0266737704706)*x[2]
        arg[0,3,0,1]=(0.148219555101)*x[0]+(-0.6887500275)*x[1]+(-0.405613136632)*x[2]
        arg[0,3,1,0]=(0.708262537279)*x[0]+(1.16319635773)*x[1]+(-0.030808721674)*x[2]
        arg[0,3,1,1]=(0.80834615863)*x[0]+(-0.176789000925)*x[1]+(-1.72576120963)*x[2]
        arg[0,3,2,0]=(0.21128315299)*x[0]+(0.770037046126)*x[1]+(0.988028934422)*x[2]
        arg[0,3,2,1]=(1.10522136317)*x[0]+(1.6319351743)*x[1]+(-0.784693713387)*x[2]
        arg[0,4,0,0]=(0.121088121064)*x[0]+(-1.79718067791)*x[1]+(-0.907857173318)*x[2]
        arg[0,4,0,1]=(-0.695887294217)*x[0]+(-1.24166787807)*x[1]+(-1.24968023856)*x[2]
        arg[0,4,1,0]=(-0.202510797291)*x[0]+(0.628310156105)*x[1]+(0.605146447822)*x[2]
        arg[0,4,1,1]=(0.149227458424)*x[0]+(-1.5650392775)*x[1]+(-0.763944892663)*x[2]
        arg[0,4,2,0]=(-0.393218625872)*x[0]+(0.479145914948)*x[1]+(-0.313960447492)*x[2]
        arg[0,4,2,1]=(-0.0191807443799)*x[0]+(0.592992659119)*x[1]+(-0.475633115484)*x[2]
        arg[1,0,0,0]=(-0.964829354261)*x[0]+(0.453014802906)*x[1]+(-0.779502566938)*x[2]
        arg[1,0,0,1]=(0.0151205739508)*x[0]+(-0.153706429594)*x[1]+(0.00866460475724)*x[2]
        arg[1,0,1,0]=(0.92592276366)*x[0]+(0.530787400525)*x[1]+(-0.0577912606876)*x[2]
        arg[1,0,1,1]=(0.948404467077)*x[0]+(-0.265905470751)*x[1]+(0.472349192005)*x[2]
        arg[1,0,2,0]=(0.0176615383702)*x[0]+(0.679605821761)*x[1]+(-0.0333590911941)*x[2]
        arg[1,0,2,1]=(-1.15594952629)*x[0]+(0.264868376508)*x[1]+(-0.883690535499)*x[2]
        arg[1,1,0,0]=(-0.738712703994)*x[0]+(0.545839232808)*x[1]+(0.592474501134)*x[2]
        arg[1,1,0,1]=(-1.3468960453)*x[0]+(-0.00848202443614)*x[1]+(-0.0124138286126)*x[2]
        arg[1,1,1,0]=(0.900228409707)*x[0]+(-1.86848082357)*x[1]+(0.421539431142)*x[2]
        arg[1,1,1,1]=(1.42754013963)*x[0]+(-0.861376440449)*x[1]+(0.0965520362291)*x[2]
        arg[1,1,2,0]=(-0.35873674515)*x[0]+(0.0862592617941)*x[1]+(-0.536106207025)*x[2]
        arg[1,1,2,1]=(0.90712910097)*x[0]+(0.974545253055)*x[1]+(0.570708510858)*x[2]
        arg[1,2,0,0]=(1.19310775669)*x[0]+(0.309414331679)*x[1]+(-0.271298061586)*x[2]
        arg[1,2,0,1]=(0.551778129545)*x[0]+(0.539028430134)*x[1]+(0.239406239262)*x[2]
        arg[1,2,1,0]=(-0.810625054344)*x[0]+(-0.241072190233)*x[1]+(-1.58590369097)*x[2]
        arg[1,2,1,1]=(0.529669075734)*x[0]+(-0.890365024163)*x[1]+(-1.69798509084)*x[2]
        arg[1,2,2,0]=(0.0437087707448)*x[0]+(0.88372190198)*x[1]+(1.86547747806)*x[2]
        arg[1,2,2,1]=(1.30739438898)*x[0]+(0.315850112101)*x[1]+(-0.262001133683)*x[2]
        arg[1,3,0,0]=(-0.143426236837)*x[0]+(-1.1703384826)*x[1]+(0.50608348646)*x[2]
        arg[1,3,0,1]=(-0.0444408776922)*x[0]+(1.26952782036)*x[1]+(1.67192602934)*x[2]
        arg[1,3,1,0]=(0.672742824368)*x[0]+(-0.785246676537)*x[1]+(0.716130664056)*x[2]
        arg[1,3,1,1]=(-0.295182508335)*x[0]+(-1.53658860897)*x[1]+(-1.87133995422)*x[2]
        arg[1,3,2,0]=(-0.440859530081)*x[0]+(0.0730120015821)*x[1]+(-1.02465537353)*x[2]
        arg[1,3,2,1]=(0.838320612003)*x[0]+(1.81233688454)*x[1]+(-1.0633876685)*x[2]
        arg[1,4,0,0]=(0.52440453051)*x[0]+(-0.0182588999022)*x[1]+(1.05556310351)*x[2]
        arg[1,4,0,1]=(0.766612727495)*x[0]+(1.58987590746)*x[1]+(-0.703801484569)*x[2]
        arg[1,4,1,0]=(0.373439757535)*x[0]+(-0.0676963704305)*x[1]+(0.0948034094293)*x[2]
        arg[1,4,1,1]=(-0.542283543375)*x[0]+(-0.454408605761)*x[1]+(-0.398904635316)*x[2]
        arg[1,4,2,0]=(1.3374871143)*x[0]+(0.0067849394587)*x[1]+(-0.319452094882)*x[2]
        arg[1,4,2,1]=(0.38530755393)*x[0]+(1.05356462429)*x[1]+(-0.302179447719)*x[2]
        arg[2,0,0,0]=(-0.123109761583)*x[0]+(0.69223848143)*x[1]+(-1.36903487092)*x[2]
        arg[2,0,0,1]=(0.121957328459)*x[0]+(0.102229685387)*x[1]+(0.962499974737)*x[2]
        arg[2,0,1,0]=(0.478614188653)*x[0]+(-1.23058817023)*x[1]+(1.30950997478)*x[2]
        arg[2,0,1,1]=(-0.529033562461)*x[0]+(0.563030509591)*x[1]+(-0.564939226654)*x[2]
        arg[2,0,2,0]=(-0.377740282406)*x[0]+(1.20532110343)*x[1]+(1.55615174947)*x[2]
        arg[2,0,2,1]=(0.539119710703)*x[0]+(1.91771865857)*x[1]+(0.0809882488048)*x[2]
        arg[2,1,0,0]=(0.00548664737796)*x[0]+(-0.157716099068)*x[1]+(-1.40009044235)*x[2]
        arg[2,1,0,1]=(0.0667557890666)*x[0]+(0.742484448131)*x[1]+(-0.325246111346)*x[2]
        arg[2,1,1,0]=(-0.214965474071)*x[0]+(-0.215055151964)*x[1]+(1.15673960537)*x[2]
        arg[2,1,1,1]=(0.605764831789)*x[0]+(-1.0432964832)*x[1]+(0.645482523161)*x[2]
        arg[2,1,2,0]=(-1.13098364088)*x[0]+(-0.932044089041)*x[1]+(-0.308053828241)*x[2]
        arg[2,1,2,1]=(0.481961305459)*x[0]+(-0.101277472029)*x[1]+(-0.17520207974)*x[2]
        arg[2,2,0,0]=(0.579831852303)*x[0]+(0.538731990254)*x[1]+(-0.0897460374205)*x[2]
        arg[2,2,0,1]=(-1.8763181434)*x[0]+(-0.132710928569)*x[1]+(1.03169751651)*x[2]
        arg[2,2,1,0]=(0.56871511431)*x[0]+(0.075333481656)*x[1]+(1.10238818953)*x[2]
        arg[2,2,1,1]=(1.49347755246)*x[0]+(-1.00500941047)*x[1]+(-0.181733196359)*x[2]
        arg[2,2,2,0]=(0.532881509971)*x[0]+(-0.364333964659)*x[1]+(1.09633720883)*x[2]
        arg[2,2,2,1]=(-0.518419048615)*x[0]+(-0.48091100593)*x[1]+(0.724628840153)*x[2]
        arg[2,3,0,0]=(0.353475859331)*x[0]+(-0.0239879269825)*x[1]+(1.32068893718)*x[2]
        arg[2,3,0,1]=(0.553999122102)*x[0]+(-0.128530869239)*x[1]+(0.134663336287)*x[2]
        arg[2,3,1,0]=(-1.59775882569)*x[0]+(-1.07335424785)*x[1]+(1.39223587988)*x[2]
        arg[2,3,1,1]=(-0.921252080095)*x[0]+(1.46226299067)*x[1]+(-1.13412499932)*x[2]
        arg[2,3,2,0]=(0.417572910178)*x[0]+(0.0814889599489)*x[1]+(0.292159249719)*x[2]
        arg[2,3,2,1]=(1.44620228921)*x[0]+(0.767854577367)*x[1]+(-0.0113709806383)*x[2]
        arg[2,4,0,0]=(-0.156987001681)*x[0]+(-0.922524257833)*x[1]+(0.521128360838)*x[2]
        arg[2,4,0,1]=(0.154198133354)*x[0]+(0.462020370175)*x[1]+(0.966768653665)*x[2]
        arg[2,4,1,0]=(0.507836849242)*x[0]+(1.16701911244)*x[1]+(0.264543038211)*x[2]
        arg[2,4,1,1]=(0.452059436879)*x[0]+(0.409662432778)*x[1]+(-0.403800872595)*x[2]
        arg[2,4,2,0]=(0.521175521008)*x[0]+(-0.362551578816)*x[1]+(0.0922727331623)*x[2]
        arg[2,4,2,1]=(-1.0990361823)*x[0]+(1.73655146851)*x[1]+(-0.867917402922)*x[2]
        arg[3,0,0,0]=(-0.0527010882236)*x[0]+(-1.11528215488)*x[1]+(1.3141906393)*x[2]
        arg[3,0,0,1]=(0.151284271917)*x[0]+(-0.0159077450668)*x[1]+(1.1798338931)*x[2]
        arg[3,0,1,0]=(-1.7648963435)*x[0]+(0.229511437402)*x[1]+(0.924415900312)*x[2]
        arg[3,0,1,1]=(-0.498271821094)*x[0]+(1.58363063019)*x[1]+(1.64871750388)*x[2]
        arg[3,0,2,0]=(-1.11245170725)*x[0]+(1.27662612569)*x[1]+(0.145950372369)*x[2]
        arg[3,0,2,1]=(-0.691643924238)*x[0]+(-0.178211874279)*x[1]+(-0.341071050366)*x[2]
        arg[3,1,0,0]=(-0.929804889461)*x[0]+(0.199087067309)*x[1]+(1.00150168883)*x[2]
        arg[3,1,0,1]=(-1.25152265093)*x[0]+(0.517076294911)*x[1]+(0.103798312362)*x[2]
        arg[3,1,1,0]=(0.797316290623)*x[0]+(0.0239285379293)*x[1]+(-0.836519299418)*x[2]
        arg[3,1,1,1]=(-0.470084592176)*x[0]+(0.844364701728)*x[1]+(-0.027373275093)*x[2]
        arg[3,1,2,0]=(1.0340279373)*x[0]+(-0.0133951276315)*x[1]+(1.17710546194)*x[2]
        arg[3,1,2,1]=(0.0325929541496)*x[0]+(1.13784867184)*x[1]+(-0.0529194266769)*x[2]
        arg[3,2,0,0]=(1.92837561772)*x[0]+(-1.63633468831)*x[1]+(1.4212324281)*x[2]
        arg[3,2,0,1]=(1.09556054408)*x[0]+(1.43286059545)*x[1]+(1.20005604593)*x[2]
        arg[3,2,1,0]=(0.308887169169)*x[0]+(-0.149214224208)*x[1]+(-1.13559614877)*x[2]
        arg[3,2,1,1]=(-0.0648352909004)*x[0]+(-0.5645470596)*x[1]+(-0.525131730514)*x[2]
        arg[3,2,2,0]=(0.21826829974)*x[0]+(1.5950239996)*x[1]+(0.333592425438)*x[2]
        arg[3,2,2,1]=(-1.25613956172)*x[0]+(-0.242338679844)*x[1]+(-0.958875577074)*x[2]
        arg[3,3,0,0]=(1.65005362159)*x[0]+(-1.16713867165)*x[1]+(-0.282495293685)*x[2]
        arg[3,3,0,1]=(1.04854406427)*x[0]+(0.876270949531)*x[1]+(0.291036304743)*x[2]
        arg[3,3,1,0]=(-1.66619451175)*x[0]+(0.656275192484)*x[1]+(-1.27104499942)*x[2]
        arg[3,3,1,1]=(1.55407146907)*x[0]+(-0.322950943159)*x[1]+(-0.287565403896)*x[2]
        arg[3,3,2,0]=(-1.54760268583)*x[0]+(-1.45847761887)*x[1]+(1.13939567009)*x[2]
        arg[3,3,2,1]=(1.24632351699)*x[0]+(0.0265500081594)*x[1]+(1.61925095159)*x[2]
        arg[3,4,0,0]=(-0.0430889433549)*x[0]+(-1.46995559463)*x[1]+(-1.14915159823)*x[2]
        arg[3,4,0,1]=(0.0392641403409)*x[0]+(0.0603053049639)*x[1]+(0.647321169934)*x[2]
        arg[3,4,1,0]=(-0.156300985227)*x[0]+(0.79369911928)*x[1]+(-0.413124762507)*x[2]
        arg[3,4,1,1]=(0.258402059015)*x[0]+(0.00227307988585)*x[1]+(0.70067141295)*x[2]
        arg[3,4,2,0]=(-1.15131298872)*x[0]+(-0.627449043157)*x[1]+(-0.402268345861)*x[2]
        arg[3,4,2,1]=(0.455934238179)*x[0]+(-0.951120759108)*x[1]+(-0.280459367131)*x[2]
        ref[0,0,0,0]=(-0.791620170192)*x_ref[0]+(1.03871460301)*x_ref[1]+(-0.225035657687)*x_ref[2]
        ref[0,0,0,1]=(-0.464422319203)*x_ref[0]+(0.769454931053)*x_ref[1]+(-0.783260637771)*x_ref[2]
        ref[0,0,1,0]=(0.72655538215)*x_ref[0]+(-1.65736312015)*x_ref[1]+(1.08402505927)*x_ref[2]
        ref[0,0,1,1]=(-0.207576709573)*x_ref[0]+(-0.55250594761)*x_ref[1]+(-0.923702938288)*x_ref[2]
        ref[0,0,2,0]=(-0.701991085516)*x_ref[0]+(-1.76863643107)*x_ref[1]+(0.831382933104)*x_ref[2]
        ref[0,0,2,1]=(0.0640035527319)*x_ref[0]+(0.425446342976)*x_ref[1]+(1.64626766696)*x_ref[2]
        ref[0,1,0,0]=(1.01022963801)*x_ref[0]+(-0.0215015059716)*x_ref[1]+(0.849862425592)*x_ref[2]
        ref[0,1,0,1]=(-0.45776932943)*x_ref[0]+(0.7621342151)*x_ref[1]+(-1.40123018391)*x_ref[2]
        ref[0,1,1,0]=(0.737485054127)*x_ref[0]+(-0.147513905858)*x_ref[1]+(-0.616928595213)*x_ref[2]
        ref[0,1,1,1]=(-0.0608005391894)*x_ref[0]+(-0.154978210573)*x_ref[1]+(-1.40322011577)*x_ref[2]
        ref[0,1,2,0]=(1.59908177714)*x_ref[0]+(-0.378738573932)*x_ref[1]+(1.45285315247)*x_ref[2]
        ref[0,1,2,1]=(-0.622552157046)*x_ref[0]+(1.19996417516)*x_ref[1]+(0.728842900843)*x_ref[2]
        ref[0,2,0,0]=(-0.46942034691)*x_ref[0]+(0.333309721372)*x_ref[1]+(-0.280846553832)*x_ref[2]
        ref[0,2,0,1]=(1.22208788109)*x_ref[0]+(0.369339594285)*x_ref[1]+(1.31078425589)*x_ref[2]
        ref[0,2,1,0]=(0.0794165004114)*x_ref[0]+(0.44665451846)*x_ref[1]+(-0.814625708916)*x_ref[2]
        ref[0,2,1,1]=(0.594200807165)*x_ref[0]+(-0.00616720333193)*x_ref[1]+(0.382706175172)*x_ref[2]
        ref[0,2,2,0]=(0.0372230170459)*x_ref[0]+(0.440339923291)*x_ref[1]+(0.137641070028)*x_ref[2]
        ref[0,2,2,1]=(-1.29863078274)*x_ref[0]+(0.283142455171)*x_ref[1]+(0.0844540074659)*x_ref[2]
        ref[0,3,0,0]=(1.72811808036)*x_ref[0]+(0.536456781522)*x_ref[1]+(0.0266737704706)*x_ref[2]
        ref[0,3,0,1]=(0.148219555101)*x_ref[0]+(-0.6887500275)*x_ref[1]+(-0.405613136632)*x_ref[2]
        ref[0,3,1,0]=(0.708262537279)*x_ref[0]+(1.16319635773)*x_ref[1]+(-0.030808721674)*x_ref[2]
        ref[0,3,1,1]=(0.80834615863)*x_ref[0]+(-0.176789000925)*x_ref[1]+(-1.72576120963)*x_ref[2]
        ref[0,3,2,0]=(0.21128315299)*x_ref[0]+(0.770037046126)*x_ref[1]+(0.988028934422)*x_ref[2]
        ref[0,3,2,1]=(1.10522136317)*x_ref[0]+(1.6319351743)*x_ref[1]+(-0.784693713387)*x_ref[2]
        ref[0,4,0,0]=(0.121088121064)*x_ref[0]+(-1.79718067791)*x_ref[1]+(-0.907857173318)*x_ref[2]
        ref[0,4,0,1]=(-0.695887294217)*x_ref[0]+(-1.24166787807)*x_ref[1]+(-1.24968023856)*x_ref[2]
        ref[0,4,1,0]=(-0.202510797291)*x_ref[0]+(0.628310156105)*x_ref[1]+(0.605146447822)*x_ref[2]
        ref[0,4,1,1]=(0.149227458424)*x_ref[0]+(-1.5650392775)*x_ref[1]+(-0.763944892663)*x_ref[2]
        ref[0,4,2,0]=(-0.393218625872)*x_ref[0]+(0.479145914948)*x_ref[1]+(-0.313960447492)*x_ref[2]
        ref[0,4,2,1]=(-0.0191807443799)*x_ref[0]+(0.592992659119)*x_ref[1]+(-0.475633115484)*x_ref[2]
        ref[1,0,0,0]=(-0.964829354261)*x_ref[0]+(0.453014802906)*x_ref[1]+(-0.779502566938)*x_ref[2]
        ref[1,0,0,1]=(0.0151205739508)*x_ref[0]+(-0.153706429594)*x_ref[1]+(0.00866460475724)*x_ref[2]
        ref[1,0,1,0]=(0.92592276366)*x_ref[0]+(0.530787400525)*x_ref[1]+(-0.0577912606876)*x_ref[2]
        ref[1,0,1,1]=(0.948404467077)*x_ref[0]+(-0.265905470751)*x_ref[1]+(0.472349192005)*x_ref[2]
        ref[1,0,2,0]=(0.0176615383702)*x_ref[0]+(0.679605821761)*x_ref[1]+(-0.0333590911941)*x_ref[2]
        ref[1,0,2,1]=(-1.15594952629)*x_ref[0]+(0.264868376508)*x_ref[1]+(-0.883690535499)*x_ref[2]
        ref[1,1,0,0]=(-0.738712703994)*x_ref[0]+(0.545839232808)*x_ref[1]+(0.592474501134)*x_ref[2]
        ref[1,1,0,1]=(-1.3468960453)*x_ref[0]+(-0.00848202443614)*x_ref[1]+(-0.0124138286126)*x_ref[2]
        ref[1,1,1,0]=(0.900228409707)*x_ref[0]+(-1.86848082357)*x_ref[1]+(0.421539431142)*x_ref[2]
        ref[1,1,1,1]=(1.42754013963)*x_ref[0]+(-0.861376440449)*x_ref[1]+(0.0965520362291)*x_ref[2]
        ref[1,1,2,0]=(-0.35873674515)*x_ref[0]+(0.0862592617941)*x_ref[1]+(-0.536106207025)*x_ref[2]
        ref[1,1,2,1]=(0.90712910097)*x_ref[0]+(0.974545253055)*x_ref[1]+(0.570708510858)*x_ref[2]
        ref[1,2,0,0]=(1.19310775669)*x_ref[0]+(0.309414331679)*x_ref[1]+(-0.271298061586)*x_ref[2]
        ref[1,2,0,1]=(0.551778129545)*x_ref[0]+(0.539028430134)*x_ref[1]+(0.239406239262)*x_ref[2]
        ref[1,2,1,0]=(-0.810625054344)*x_ref[0]+(-0.241072190233)*x_ref[1]+(-1.58590369097)*x_ref[2]
        ref[1,2,1,1]=(0.529669075734)*x_ref[0]+(-0.890365024163)*x_ref[1]+(-1.69798509084)*x_ref[2]
        ref[1,2,2,0]=(0.0437087707448)*x_ref[0]+(0.88372190198)*x_ref[1]+(1.86547747806)*x_ref[2]
        ref[1,2,2,1]=(1.30739438898)*x_ref[0]+(0.315850112101)*x_ref[1]+(-0.262001133683)*x_ref[2]
        ref[1,3,0,0]=(-0.143426236837)*x_ref[0]+(-1.1703384826)*x_ref[1]+(0.50608348646)*x_ref[2]
        ref[1,3,0,1]=(-0.0444408776922)*x_ref[0]+(1.26952782036)*x_ref[1]+(1.67192602934)*x_ref[2]
        ref[1,3,1,0]=(0.672742824368)*x_ref[0]+(-0.785246676537)*x_ref[1]+(0.716130664056)*x_ref[2]
        ref[1,3,1,1]=(-0.295182508335)*x_ref[0]+(-1.53658860897)*x_ref[1]+(-1.87133995422)*x_ref[2]
        ref[1,3,2,0]=(-0.440859530081)*x_ref[0]+(0.0730120015821)*x_ref[1]+(-1.02465537353)*x_ref[2]
        ref[1,3,2,1]=(0.838320612003)*x_ref[0]+(1.81233688454)*x_ref[1]+(-1.0633876685)*x_ref[2]
        ref[1,4,0,0]=(0.52440453051)*x_ref[0]+(-0.0182588999022)*x_ref[1]+(1.05556310351)*x_ref[2]
        ref[1,4,0,1]=(0.766612727495)*x_ref[0]+(1.58987590746)*x_ref[1]+(-0.703801484569)*x_ref[2]
        ref[1,4,1,0]=(0.373439757535)*x_ref[0]+(-0.0676963704305)*x_ref[1]+(0.0948034094293)*x_ref[2]
        ref[1,4,1,1]=(-0.542283543375)*x_ref[0]+(-0.454408605761)*x_ref[1]+(-0.398904635316)*x_ref[2]
        ref[1,4,2,0]=(1.3374871143)*x_ref[0]+(0.0067849394587)*x_ref[1]+(-0.319452094882)*x_ref[2]
        ref[1,4,2,1]=(0.38530755393)*x_ref[0]+(1.05356462429)*x_ref[1]+(-0.302179447719)*x_ref[2]
        ref[2,0,0,0]=(-0.123109761583)*x_ref[0]+(0.69223848143)*x_ref[1]+(-1.36903487092)*x_ref[2]
        ref[2,0,0,1]=(0.121957328459)*x_ref[0]+(0.102229685387)*x_ref[1]+(0.962499974737)*x_ref[2]
        ref[2,0,1,0]=(0.478614188653)*x_ref[0]+(-1.23058817023)*x_ref[1]+(1.30950997478)*x_ref[2]
        ref[2,0,1,1]=(-0.529033562461)*x_ref[0]+(0.563030509591)*x_ref[1]+(-0.564939226654)*x_ref[2]
        ref[2,0,2,0]=(-0.377740282406)*x_ref[0]+(1.20532110343)*x_ref[1]+(1.55615174947)*x_ref[2]
        ref[2,0,2,1]=(0.539119710703)*x_ref[0]+(1.91771865857)*x_ref[1]+(0.0809882488048)*x_ref[2]
        ref[2,1,0,0]=(0.00548664737796)*x_ref[0]+(-0.157716099068)*x_ref[1]+(-1.40009044235)*x_ref[2]
        ref[2,1,0,1]=(0.0667557890666)*x_ref[0]+(0.742484448131)*x_ref[1]+(-0.325246111346)*x_ref[2]
        ref[2,1,1,0]=(-0.214965474071)*x_ref[0]+(-0.215055151964)*x_ref[1]+(1.15673960537)*x_ref[2]
        ref[2,1,1,1]=(0.605764831789)*x_ref[0]+(-1.0432964832)*x_ref[1]+(0.645482523161)*x_ref[2]
        ref[2,1,2,0]=(-1.13098364088)*x_ref[0]+(-0.932044089041)*x_ref[1]+(-0.308053828241)*x_ref[2]
        ref[2,1,2,1]=(0.481961305459)*x_ref[0]+(-0.101277472029)*x_ref[1]+(-0.17520207974)*x_ref[2]
        ref[2,2,0,0]=(0.579831852303)*x_ref[0]+(0.538731990254)*x_ref[1]+(-0.0897460374205)*x_ref[2]
        ref[2,2,0,1]=(-1.8763181434)*x_ref[0]+(-0.132710928569)*x_ref[1]+(1.03169751651)*x_ref[2]
        ref[2,2,1,0]=(0.56871511431)*x_ref[0]+(0.075333481656)*x_ref[1]+(1.10238818953)*x_ref[2]
        ref[2,2,1,1]=(1.49347755246)*x_ref[0]+(-1.00500941047)*x_ref[1]+(-0.181733196359)*x_ref[2]
        ref[2,2,2,0]=(0.532881509971)*x_ref[0]+(-0.364333964659)*x_ref[1]+(1.09633720883)*x_ref[2]
        ref[2,2,2,1]=(-0.518419048615)*x_ref[0]+(-0.48091100593)*x_ref[1]+(0.724628840153)*x_ref[2]
        ref[2,3,0,0]=(0.353475859331)*x_ref[0]+(-0.0239879269825)*x_ref[1]+(1.32068893718)*x_ref[2]
        ref[2,3,0,1]=(0.553999122102)*x_ref[0]+(-0.128530869239)*x_ref[1]+(0.134663336287)*x_ref[2]
        ref[2,3,1,0]=(-1.59775882569)*x_ref[0]+(-1.07335424785)*x_ref[1]+(1.39223587988)*x_ref[2]
        ref[2,3,1,1]=(-0.921252080095)*x_ref[0]+(1.46226299067)*x_ref[1]+(-1.13412499932)*x_ref[2]
        ref[2,3,2,0]=(0.417572910178)*x_ref[0]+(0.0814889599489)*x_ref[1]+(0.292159249719)*x_ref[2]
        ref[2,3,2,1]=(1.44620228921)*x_ref[0]+(0.767854577367)*x_ref[1]+(-0.0113709806383)*x_ref[2]
        ref[2,4,0,0]=(-0.156987001681)*x_ref[0]+(-0.922524257833)*x_ref[1]+(0.521128360838)*x_ref[2]
        ref[2,4,0,1]=(0.154198133354)*x_ref[0]+(0.462020370175)*x_ref[1]+(0.966768653665)*x_ref[2]
        ref[2,4,1,0]=(0.507836849242)*x_ref[0]+(1.16701911244)*x_ref[1]+(0.264543038211)*x_ref[2]
        ref[2,4,1,1]=(0.452059436879)*x_ref[0]+(0.409662432778)*x_ref[1]+(-0.403800872595)*x_ref[2]
        ref[2,4,2,0]=(0.521175521008)*x_ref[0]+(-0.362551578816)*x_ref[1]+(0.0922727331623)*x_ref[2]
        ref[2,4,2,1]=(-1.0990361823)*x_ref[0]+(1.73655146851)*x_ref[1]+(-0.867917402922)*x_ref[2]
        ref[3,0,0,0]=(-0.0527010882236)*x_ref[0]+(-1.11528215488)*x_ref[1]+(1.3141906393)*x_ref[2]
        ref[3,0,0,1]=(0.151284271917)*x_ref[0]+(-0.0159077450668)*x_ref[1]+(1.1798338931)*x_ref[2]
        ref[3,0,1,0]=(-1.7648963435)*x_ref[0]+(0.229511437402)*x_ref[1]+(0.924415900312)*x_ref[2]
        ref[3,0,1,1]=(-0.498271821094)*x_ref[0]+(1.58363063019)*x_ref[1]+(1.64871750388)*x_ref[2]
        ref[3,0,2,0]=(-1.11245170725)*x_ref[0]+(1.27662612569)*x_ref[1]+(0.145950372369)*x_ref[2]
        ref[3,0,2,1]=(-0.691643924238)*x_ref[0]+(-0.178211874279)*x_ref[1]+(-0.341071050366)*x_ref[2]
        ref[3,1,0,0]=(-0.929804889461)*x_ref[0]+(0.199087067309)*x_ref[1]+(1.00150168883)*x_ref[2]
        ref[3,1,0,1]=(-1.25152265093)*x_ref[0]+(0.517076294911)*x_ref[1]+(0.103798312362)*x_ref[2]
        ref[3,1,1,0]=(0.797316290623)*x_ref[0]+(0.0239285379293)*x_ref[1]+(-0.836519299418)*x_ref[2]
        ref[3,1,1,1]=(-0.470084592176)*x_ref[0]+(0.844364701728)*x_ref[1]+(-0.027373275093)*x_ref[2]
        ref[3,1,2,0]=(1.0340279373)*x_ref[0]+(-0.0133951276315)*x_ref[1]+(1.17710546194)*x_ref[2]
        ref[3,1,2,1]=(0.0325929541496)*x_ref[0]+(1.13784867184)*x_ref[1]+(-0.0529194266769)*x_ref[2]
        ref[3,2,0,0]=(1.92837561772)*x_ref[0]+(-1.63633468831)*x_ref[1]+(1.4212324281)*x_ref[2]
        ref[3,2,0,1]=(1.09556054408)*x_ref[0]+(1.43286059545)*x_ref[1]+(1.20005604593)*x_ref[2]
        ref[3,2,1,0]=(0.308887169169)*x_ref[0]+(-0.149214224208)*x_ref[1]+(-1.13559614877)*x_ref[2]
        ref[3,2,1,1]=(-0.0648352909004)*x_ref[0]+(-0.5645470596)*x_ref[1]+(-0.525131730514)*x_ref[2]
        ref[3,2,2,0]=(0.21826829974)*x_ref[0]+(1.5950239996)*x_ref[1]+(0.333592425438)*x_ref[2]
        ref[3,2,2,1]=(-1.25613956172)*x_ref[0]+(-0.242338679844)*x_ref[1]+(-0.958875577074)*x_ref[2]
        ref[3,3,0,0]=(1.65005362159)*x_ref[0]+(-1.16713867165)*x_ref[1]+(-0.282495293685)*x_ref[2]
        ref[3,3,0,1]=(1.04854406427)*x_ref[0]+(0.876270949531)*x_ref[1]+(0.291036304743)*x_ref[2]
        ref[3,3,1,0]=(-1.66619451175)*x_ref[0]+(0.656275192484)*x_ref[1]+(-1.27104499942)*x_ref[2]
        ref[3,3,1,1]=(1.55407146907)*x_ref[0]+(-0.322950943159)*x_ref[1]+(-0.287565403896)*x_ref[2]
        ref[3,3,2,0]=(-1.54760268583)*x_ref[0]+(-1.45847761887)*x_ref[1]+(1.13939567009)*x_ref[2]
        ref[3,3,2,1]=(1.24632351699)*x_ref[0]+(0.0265500081594)*x_ref[1]+(1.61925095159)*x_ref[2]
        ref[3,4,0,0]=(-0.0430889433549)*x_ref[0]+(-1.46995559463)*x_ref[1]+(-1.14915159823)*x_ref[2]
        ref[3,4,0,1]=(0.0392641403409)*x_ref[0]+(0.0603053049639)*x_ref[1]+(0.647321169934)*x_ref[2]
        ref[3,4,1,0]=(-0.156300985227)*x_ref[0]+(0.79369911928)*x_ref[1]+(-0.413124762507)*x_ref[2]
        ref[3,4,1,1]=(0.258402059015)*x_ref[0]+(0.00227307988585)*x_ref[1]+(0.70067141295)*x_ref[2]
        ref[3,4,2,0]=(-1.15131298872)*x_ref[0]+(-0.627449043157)*x_ref[1]+(-0.402268345861)*x_ref[2]
        ref[3,4,2,1]=(0.455934238179)*x_ref[0]+(-0.951120759108)*x_ref[1]+(-0.280459367131)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_ReducedContinuousFunction_rank0(self):
      """
      tests interpolate for rank 0 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedContinuousFunction onto ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(),w)
      ref=Data(0,(),w_ref)
      if dim==2:
        arg=(-0.787646538574)*x[0]+(0.912422904218)*x[1]
        ref=(-0.787646538574)*x_ref[0]+(0.912422904218)*x_ref[1]
      else:
        arg=(0.934044939024)*x[0]+(0.628477728603)*x[1]+(-0.786339405837)*x[2]
        ref=(0.934044939024)*x_ref[0]+(0.628477728603)*x_ref[1]+(-0.786339405837)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_ReducedContinuousFunction_rank1(self):
      """
      tests interpolate for rank 1 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedContinuousFunction onto ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(2,),w)
      ref=Data(0,(2,),w_ref)
      if dim==2:
        arg[0]=(-0.588326094839)*x[0]+(1.56296941343)*x[1]
        arg[1]=(-0.802191222185)*x[0]+(-1.13471016062)*x[1]
        ref[0]=(-0.588326094839)*x[0]+(1.56296941343)*x[1]
        ref[1]=(-0.802191222185)*x[0]+(-1.13471016062)*x[1]
      else:
        arg[0]=(0.160702771321)*x[0]+(-0.0784425374814)*x[1]+(1.65226639722)*x[2]
        arg[1]=(0.745050820919)*x[0]+(-0.183859360073)*x[1]+(0.399813139488)*x[2]
        ref[0]=(0.160702771321)*x[0]+(-0.0784425374814)*x[1]+(1.65226639722)*x[2]
        ref[1]=(0.745050820919)*x[0]+(-0.183859360073)*x[1]+(0.399813139488)*x[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_ReducedContinuousFunction_rank2(self):
      """
      tests interpolate for rank 2 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedContinuousFunction onto ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5),w)
      ref=Data(0,(4, 5),w_ref)
      if dim==2:
        arg[0,0]=(-0.797066914479)*x[0]+(-0.0269853164939)*x[1]
        arg[0,1]=(-1.07443661798)*x[0]+(0.818510216043)*x[1]
        arg[0,2]=(0.398735262179)*x[0]+(-0.580387872083)*x[1]
        arg[0,3]=(-0.003185613959)*x[0]+(-0.670506330615)*x[1]
        arg[0,4]=(0.385009570186)*x[0]+(0.858570512936)*x[1]
        arg[1,0]=(1.60917176753)*x[0]+(-0.0866362449992)*x[1]
        arg[1,1]=(-0.883782122452)*x[0]+(-0.402187705578)*x[1]
        arg[1,2]=(0.153893095122)*x[0]+(-1.19636362644)*x[1]
        arg[1,3]=(1.92041961031)*x[0]+(0.325857877636)*x[1]
        arg[1,4]=(-0.0947698843729)*x[0]+(-0.0663785758627)*x[1]
        arg[2,0]=(0.0232949467051)*x[0]+(1.60134210326)*x[1]
        arg[2,1]=(-0.521182882924)*x[0]+(-0.507401571343)*x[1]
        arg[2,2]=(0.833797117616)*x[0]+(-0.268886477194)*x[1]
        arg[2,3]=(-0.867762611227)*x[0]+(1.03727088566)*x[1]
        arg[2,4]=(0.380510651918)*x[0]+(0.350246888456)*x[1]
        arg[3,0]=(-1.10155500839)*x[0]+(0.414683749118)*x[1]
        arg[3,1]=(-0.0306733233552)*x[0]+(-0.351251014237)*x[1]
        arg[3,2]=(-0.0708773001498)*x[0]+(-0.326636587948)*x[1]
        arg[3,3]=(-1.32553119895)*x[0]+(-1.5852157477)*x[1]
        arg[3,4]=(1.40177154802)*x[0]+(-1.24804787994)*x[1]
        ref[0,0]=(-0.797066914479)*x_ref[0]+(-0.0269853164939)*x_ref[1]
        ref[0,1]=(-1.07443661798)*x_ref[0]+(0.818510216043)*x_ref[1]
        ref[0,2]=(0.398735262179)*x_ref[0]+(-0.580387872083)*x_ref[1]
        ref[0,3]=(-0.003185613959)*x_ref[0]+(-0.670506330615)*x_ref[1]
        ref[0,4]=(0.385009570186)*x_ref[0]+(0.858570512936)*x_ref[1]
        ref[1,0]=(1.60917176753)*x_ref[0]+(-0.0866362449992)*x_ref[1]
        ref[1,1]=(-0.883782122452)*x_ref[0]+(-0.402187705578)*x_ref[1]
        ref[1,2]=(0.153893095122)*x_ref[0]+(-1.19636362644)*x_ref[1]
        ref[1,3]=(1.92041961031)*x_ref[0]+(0.325857877636)*x_ref[1]
        ref[1,4]=(-0.0947698843729)*x_ref[0]+(-0.0663785758627)*x_ref[1]
        ref[2,0]=(0.0232949467051)*x_ref[0]+(1.60134210326)*x_ref[1]
        ref[2,1]=(-0.521182882924)*x_ref[0]+(-0.507401571343)*x_ref[1]
        ref[2,2]=(0.833797117616)*x_ref[0]+(-0.268886477194)*x_ref[1]
        ref[2,3]=(-0.867762611227)*x_ref[0]+(1.03727088566)*x_ref[1]
        ref[2,4]=(0.380510651918)*x_ref[0]+(0.350246888456)*x_ref[1]
        ref[3,0]=(-1.10155500839)*x_ref[0]+(0.414683749118)*x_ref[1]
        ref[3,1]=(-0.0306733233552)*x_ref[0]+(-0.351251014237)*x_ref[1]
        ref[3,2]=(-0.0708773001498)*x_ref[0]+(-0.326636587948)*x_ref[1]
        ref[3,3]=(-1.32553119895)*x_ref[0]+(-1.5852157477)*x_ref[1]
        ref[3,4]=(1.40177154802)*x_ref[0]+(-1.24804787994)*x_ref[1]
      else:
        arg[0,0]=(0.000362194678523)*x[0]+(-1.63949220671)*x[1]+(-0.176997156562)*x[2]
        arg[0,1]=(-0.357589049408)*x[0]+(0.828191515325)*x[1]+(-0.269706780136)*x[2]
        arg[0,2]=(0.886395070544)*x[0]+(0.815688451839)*x[1]+(-0.658412270558)*x[2]
        arg[0,3]=(-0.222573318026)*x[0]+(0.137763803092)*x[1]+(0.171902150187)*x[2]
        arg[0,4]=(1.05159741251)*x[0]+(0.923836841038)*x[1]+(-0.426590801731)*x[2]
        arg[1,0]=(-0.00718419057788)*x[0]+(1.55809964998)*x[1]+(-0.912766657496)*x[2]
        arg[1,1]=(1.30769045785)*x[0]+(-0.0356609338389)*x[1]+(0.214543084134)*x[2]
        arg[1,2]=(-1.56763014241)*x[0]+(-0.582970995722)*x[1]+(-0.513972924752)*x[2]
        arg[1,3]=(0.118027736928)*x[0]+(-0.28795014876)*x[1]+(0.620718623034)*x[2]
        arg[1,4]=(-0.784496170193)*x[0]+(0.233037866924)*x[1]+(-0.0500718134572)*x[2]
        arg[2,0]=(0.800851281806)*x[0]+(0.219346484846)*x[1]+(1.19125787701)*x[2]
        arg[2,1]=(-1.45025879429)*x[0]+(-1.32011538154)*x[1]+(0.47946669308)*x[2]
        arg[2,2]=(-0.231377110469)*x[0]+(-0.921551695999)*x[1]+(-0.298964635841)*x[2]
        arg[2,3]=(0.965766335546)*x[0]+(0.424791610089)*x[1]+(1.01860652092)*x[2]
        arg[2,4]=(0.676247093636)*x[0]+(-1.50802741511)*x[1]+(0.0385960918391)*x[2]
        arg[3,0]=(0.322378151756)*x[0]+(-0.483633005509)*x[1]+(-0.852371707865)*x[2]
        arg[3,1]=(-0.241998307153)*x[0]+(-1.08134340082)*x[1]+(-0.849574532537)*x[2]
        arg[3,2]=(-0.0185751405467)*x[0]+(0.179558416001)*x[1]+(-0.488727308548)*x[2]
        arg[3,3]=(0.0984555057845)*x[0]+(1.03085967788)*x[1]+(-0.357009094371)*x[2]
        arg[3,4]=(0.430593980121)*x[0]+(0.689565614489)*x[1]+(-0.0916541808491)*x[2]
        ref[0,0]=(0.000362194678523)*x_ref[0]+(-1.63949220671)*x_ref[1]+(-0.176997156562)*x_ref[2]
        ref[0,1]=(-0.357589049408)*x_ref[0]+(0.828191515325)*x_ref[1]+(-0.269706780136)*x_ref[2]
        ref[0,2]=(0.886395070544)*x_ref[0]+(0.815688451839)*x_ref[1]+(-0.658412270558)*x_ref[2]
        ref[0,3]=(-0.222573318026)*x_ref[0]+(0.137763803092)*x_ref[1]+(0.171902150187)*x_ref[2]
        ref[0,4]=(1.05159741251)*x_ref[0]+(0.923836841038)*x_ref[1]+(-0.426590801731)*x_ref[2]
        ref[1,0]=(-0.00718419057788)*x_ref[0]+(1.55809964998)*x_ref[1]+(-0.912766657496)*x_ref[2]
        ref[1,1]=(1.30769045785)*x_ref[0]+(-0.0356609338389)*x_ref[1]+(0.214543084134)*x_ref[2]
        ref[1,2]=(-1.56763014241)*x_ref[0]+(-0.582970995722)*x_ref[1]+(-0.513972924752)*x_ref[2]
        ref[1,3]=(0.118027736928)*x_ref[0]+(-0.28795014876)*x_ref[1]+(0.620718623034)*x_ref[2]
        ref[1,4]=(-0.784496170193)*x_ref[0]+(0.233037866924)*x_ref[1]+(-0.0500718134572)*x_ref[2]
        ref[2,0]=(0.800851281806)*x_ref[0]+(0.219346484846)*x_ref[1]+(1.19125787701)*x_ref[2]
        ref[2,1]=(-1.45025879429)*x_ref[0]+(-1.32011538154)*x_ref[1]+(0.47946669308)*x_ref[2]
        ref[2,2]=(-0.231377110469)*x_ref[0]+(-0.921551695999)*x_ref[1]+(-0.298964635841)*x_ref[2]
        ref[2,3]=(0.965766335546)*x_ref[0]+(0.424791610089)*x_ref[1]+(1.01860652092)*x_ref[2]
        ref[2,4]=(0.676247093636)*x_ref[0]+(-1.50802741511)*x_ref[1]+(0.0385960918391)*x_ref[2]
        ref[3,0]=(0.322378151756)*x_ref[0]+(-0.483633005509)*x_ref[1]+(-0.852371707865)*x_ref[2]
        ref[3,1]=(-0.241998307153)*x_ref[0]+(-1.08134340082)*x_ref[1]+(-0.849574532537)*x_ref[2]
        ref[3,2]=(-0.0185751405467)*x_ref[0]+(0.179558416001)*x_ref[1]+(-0.488727308548)*x_ref[2]
        ref[3,3]=(0.0984555057845)*x_ref[0]+(1.03085967788)*x_ref[1]+(-0.357009094371)*x_ref[2]
        ref[3,4]=(0.430593980121)*x_ref[0]+(0.689565614489)*x_ref[1]+(-0.0916541808491)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_ReducedContinuousFunction_rank3(self):
      """
      tests interpolate for rank 3 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedContinuousFunction onto ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2),w_ref)
      if dim==2:
        arg[0,0,0]=(-0.0641544023301)*x[0]+(0.364286357763)*x[1]
        arg[0,0,1]=(-0.858288240411)*x[0]+(-0.847764393559)*x[1]
        arg[0,1,0]=(-0.656948775029)*x[0]+(1.69748608992)*x[1]
        arg[0,1,1]=(0.579399191549)*x[0]+(1.48407368783)*x[1]
        arg[1,0,0]=(-0.35948657403)*x[0]+(0.795436090765)*x[1]
        arg[1,0,1]=(-1.32807156246)*x[0]+(0.921960610294)*x[1]
        arg[1,1,0]=(1.0904085219)*x[0]+(-1.41870997296)*x[1]
        arg[1,1,1]=(-0.677691596793)*x[0]+(-0.395373706333)*x[1]
        arg[2,0,0]=(0.295591917216)*x[0]+(-1.75550830457)*x[1]
        arg[2,0,1]=(0.506665326626)*x[0]+(0.273265713965)*x[1]
        arg[2,1,0]=(0.543899865352)*x[0]+(-0.0678457091657)*x[1]
        arg[2,1,1]=(-0.887824769535)*x[0]+(0.527475538055)*x[1]
        arg[3,0,0]=(-0.788511557365)*x[0]+(-0.635972100675)*x[1]
        arg[3,0,1]=(-1.02799066459)*x[0]+(-0.416241247725)*x[1]
        arg[3,1,0]=(-0.310072483091)*x[0]+(1.2445160445)*x[1]
        arg[3,1,1]=(0.912462196567)*x[0]+(0.631855269783)*x[1]
        arg[4,0,0]=(0.700426861807)*x[0]+(-0.491365913713)*x[1]
        arg[4,0,1]=(0.628791538338)*x[0]+(0.640767645586)*x[1]
        arg[4,1,0]=(0.370886720329)*x[0]+(0.823663448834)*x[1]
        arg[4,1,1]=(1.75857310774)*x[0]+(0.445439424641)*x[1]
        arg[5,0,0]=(1.39637385066)*x[0]+(-0.812211860169)*x[1]
        arg[5,0,1]=(0.865050350195)*x[0]+(-1.25983380375)*x[1]
        arg[5,1,0]=(-0.0429057645445)*x[0]+(-1.08490578934)*x[1]
        arg[5,1,1]=(1.42851264594)*x[0]+(1.52192971749)*x[1]
        ref[0,0,0]=(-0.0641544023301)*x_ref[0]+(0.364286357763)*x_ref[1]
        ref[0,0,1]=(-0.858288240411)*x_ref[0]+(-0.847764393559)*x_ref[1]
        ref[0,1,0]=(-0.656948775029)*x_ref[0]+(1.69748608992)*x_ref[1]
        ref[0,1,1]=(0.579399191549)*x_ref[0]+(1.48407368783)*x_ref[1]
        ref[1,0,0]=(-0.35948657403)*x_ref[0]+(0.795436090765)*x_ref[1]
        ref[1,0,1]=(-1.32807156246)*x_ref[0]+(0.921960610294)*x_ref[1]
        ref[1,1,0]=(1.0904085219)*x_ref[0]+(-1.41870997296)*x_ref[1]
        ref[1,1,1]=(-0.677691596793)*x_ref[0]+(-0.395373706333)*x_ref[1]
        ref[2,0,0]=(0.295591917216)*x_ref[0]+(-1.75550830457)*x_ref[1]
        ref[2,0,1]=(0.506665326626)*x_ref[0]+(0.273265713965)*x_ref[1]
        ref[2,1,0]=(0.543899865352)*x_ref[0]+(-0.0678457091657)*x_ref[1]
        ref[2,1,1]=(-0.887824769535)*x_ref[0]+(0.527475538055)*x_ref[1]
        ref[3,0,0]=(-0.788511557365)*x_ref[0]+(-0.635972100675)*x_ref[1]
        ref[3,0,1]=(-1.02799066459)*x_ref[0]+(-0.416241247725)*x_ref[1]
        ref[3,1,0]=(-0.310072483091)*x_ref[0]+(1.2445160445)*x_ref[1]
        ref[3,1,1]=(0.912462196567)*x_ref[0]+(0.631855269783)*x_ref[1]
        ref[4,0,0]=(0.700426861807)*x_ref[0]+(-0.491365913713)*x_ref[1]
        ref[4,0,1]=(0.628791538338)*x_ref[0]+(0.640767645586)*x_ref[1]
        ref[4,1,0]=(0.370886720329)*x_ref[0]+(0.823663448834)*x_ref[1]
        ref[4,1,1]=(1.75857310774)*x_ref[0]+(0.445439424641)*x_ref[1]
        ref[5,0,0]=(1.39637385066)*x_ref[0]+(-0.812211860169)*x_ref[1]
        ref[5,0,1]=(0.865050350195)*x_ref[0]+(-1.25983380375)*x_ref[1]
        ref[5,1,0]=(-0.0429057645445)*x_ref[0]+(-1.08490578934)*x_ref[1]
        ref[5,1,1]=(1.42851264594)*x_ref[0]+(1.52192971749)*x_ref[1]
      else:
        arg[0,0,0]=(0.731194732468)*x[0]+(-0.0822115660022)*x[1]+(-0.403175091289)*x[2]
        arg[0,0,1]=(0.39292230307)*x[0]+(1.48598681615)*x[1]+(1.05545374107)*x[2]
        arg[0,1,0]=(0.306844260706)*x[0]+(0.836496026763)*x[1]+(-0.121120983155)*x[2]
        arg[0,1,1]=(-0.155245802062)*x[0]+(0.800232934864)*x[1]+(0.373397160636)*x[2]
        arg[1,0,0]=(0.155941790359)*x[0]+(0.826821969281)*x[1]+(0.849596807425)*x[2]
        arg[1,0,1]=(-0.0988854656187)*x[0]+(-0.855032499842)*x[1]+(0.141988288566)*x[2]
        arg[1,1,0]=(-0.761032539296)*x[0]+(-0.0892979073234)*x[1]+(1.93034385132)*x[2]
        arg[1,1,1]=(-1.08303603558)*x[0]+(-0.487250117659)*x[1]+(0.0358501036173)*x[2]
        arg[2,0,0]=(0.148391465413)*x[0]+(-0.731963025625)*x[1]+(0.484236965689)*x[2]
        arg[2,0,1]=(1.06691217722)*x[0]+(-0.638095266244)*x[1]+(-0.971838410779)*x[2]
        arg[2,1,0]=(-0.377750829061)*x[0]+(-0.101467967263)*x[1]+(-1.54241387349)*x[2]
        arg[2,1,1]=(0.62210350231)*x[0]+(0.0435334873911)*x[1]+(-0.299303646623)*x[2]
        arg[3,0,0]=(0.523811147108)*x[0]+(-1.09709839108)*x[1]+(1.83592280935)*x[2]
        arg[3,0,1]=(0.957738143475)*x[0]+(0.282158440867)*x[1]+(0.621694180522)*x[2]
        arg[3,1,0]=(0.366557891822)*x[0]+(0.812194047631)*x[1]+(-0.245067876349)*x[2]
        arg[3,1,1]=(0.604859464982)*x[0]+(1.08415514854)*x[1]+(-1.470128848)*x[2]
        arg[4,0,0]=(-0.13031502499)*x[0]+(-1.20707238733)*x[1]+(0.108382497552)*x[2]
        arg[4,0,1]=(1.30453246317)*x[0]+(1.59696612588)*x[1]+(-0.234259043697)*x[2]
        arg[4,1,0]=(-0.233494640836)*x[0]+(1.10701061816)*x[1]+(-0.409927013674)*x[2]
        arg[4,1,1]=(0.102708240266)*x[0]+(-0.381598345002)*x[1]+(-1.39077748656)*x[2]
        arg[5,0,0]=(1.06217404493)*x[0]+(1.1600791444)*x[1]+(-1.45242735389)*x[2]
        arg[5,0,1]=(-0.423024189506)*x[0]+(-0.0868948651641)*x[1]+(0.0894539043522)*x[2]
        arg[5,1,0]=(1.17650613067)*x[0]+(0.993832919965)*x[1]+(1.16546768569)*x[2]
        arg[5,1,1]=(-0.61549132897)*x[0]+(-0.499619662734)*x[1]+(-0.643422281669)*x[2]
        ref[0,0,0]=(0.731194732468)*x_ref[0]+(-0.0822115660022)*x_ref[1]+(-0.403175091289)*x_ref[2]
        ref[0,0,1]=(0.39292230307)*x_ref[0]+(1.48598681615)*x_ref[1]+(1.05545374107)*x_ref[2]
        ref[0,1,0]=(0.306844260706)*x_ref[0]+(0.836496026763)*x_ref[1]+(-0.121120983155)*x_ref[2]
        ref[0,1,1]=(-0.155245802062)*x_ref[0]+(0.800232934864)*x_ref[1]+(0.373397160636)*x_ref[2]
        ref[1,0,0]=(0.155941790359)*x_ref[0]+(0.826821969281)*x_ref[1]+(0.849596807425)*x_ref[2]
        ref[1,0,1]=(-0.0988854656187)*x_ref[0]+(-0.855032499842)*x_ref[1]+(0.141988288566)*x_ref[2]
        ref[1,1,0]=(-0.761032539296)*x_ref[0]+(-0.0892979073234)*x_ref[1]+(1.93034385132)*x_ref[2]
        ref[1,1,1]=(-1.08303603558)*x_ref[0]+(-0.487250117659)*x_ref[1]+(0.0358501036173)*x_ref[2]
        ref[2,0,0]=(0.148391465413)*x_ref[0]+(-0.731963025625)*x_ref[1]+(0.484236965689)*x_ref[2]
        ref[2,0,1]=(1.06691217722)*x_ref[0]+(-0.638095266244)*x_ref[1]+(-0.971838410779)*x_ref[2]
        ref[2,1,0]=(-0.377750829061)*x_ref[0]+(-0.101467967263)*x_ref[1]+(-1.54241387349)*x_ref[2]
        ref[2,1,1]=(0.62210350231)*x_ref[0]+(0.0435334873911)*x_ref[1]+(-0.299303646623)*x_ref[2]
        ref[3,0,0]=(0.523811147108)*x_ref[0]+(-1.09709839108)*x_ref[1]+(1.83592280935)*x_ref[2]
        ref[3,0,1]=(0.957738143475)*x_ref[0]+(0.282158440867)*x_ref[1]+(0.621694180522)*x_ref[2]
        ref[3,1,0]=(0.366557891822)*x_ref[0]+(0.812194047631)*x_ref[1]+(-0.245067876349)*x_ref[2]
        ref[3,1,1]=(0.604859464982)*x_ref[0]+(1.08415514854)*x_ref[1]+(-1.470128848)*x_ref[2]
        ref[4,0,0]=(-0.13031502499)*x_ref[0]+(-1.20707238733)*x_ref[1]+(0.108382497552)*x_ref[2]
        ref[4,0,1]=(1.30453246317)*x_ref[0]+(1.59696612588)*x_ref[1]+(-0.234259043697)*x_ref[2]
        ref[4,1,0]=(-0.233494640836)*x_ref[0]+(1.10701061816)*x_ref[1]+(-0.409927013674)*x_ref[2]
        ref[4,1,1]=(0.102708240266)*x_ref[0]+(-0.381598345002)*x_ref[1]+(-1.39077748656)*x_ref[2]
        ref[5,0,0]=(1.06217404493)*x_ref[0]+(1.1600791444)*x_ref[1]+(-1.45242735389)*x_ref[2]
        ref[5,0,1]=(-0.423024189506)*x_ref[0]+(-0.0868948651641)*x_ref[1]+(0.0894539043522)*x_ref[2]
        ref[5,1,0]=(1.17650613067)*x_ref[0]+(0.993832919965)*x_ref[1]+(1.16546768569)*x_ref[2]
        ref[5,1,1]=(-0.61549132897)*x_ref[0]+(-0.499619662734)*x_ref[1]+(-0.643422281669)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_ReducedContinuousFunction_rank4(self):
      """
      tests interpolate for rank 4 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedContinuousFunction onto ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5, 3, 2),w)
      ref=Data(0,(4, 5, 3, 2),w_ref)
      if dim==2:
        arg[0,0,0,0]=(0.227969627252)*x[0]+(0.778177350248)*x[1]
        arg[0,0,0,1]=(-0.684883711494)*x[0]+(-0.574285957132)*x[1]
        arg[0,0,1,0]=(0.483090608974)*x[0]+(0.0779929087959)*x[1]
        arg[0,0,1,1]=(-0.293150814932)*x[0]+(1.35368931993)*x[1]
        arg[0,0,2,0]=(-0.190664734662)*x[0]+(0.910883386754)*x[1]
        arg[0,0,2,1]=(-1.51255575457)*x[0]+(0.421560181501)*x[1]
        arg[0,1,0,0]=(-0.833402828032)*x[0]+(-0.400466806781)*x[1]
        arg[0,1,0,1]=(-0.212099323605)*x[0]+(-0.112478697221)*x[1]
        arg[0,1,1,0]=(0.57687662926)*x[0]+(0.869758954551)*x[1]
        arg[0,1,1,1]=(1.37261769917)*x[0]+(1.37855789907)*x[1]
        arg[0,1,2,0]=(-1.02838589722)*x[0]+(-0.280409246204)*x[1]
        arg[0,1,2,1]=(1.2333373468)*x[0]+(0.337631183798)*x[1]
        arg[0,2,0,0]=(-0.249065448772)*x[0]+(0.834483646317)*x[1]
        arg[0,2,0,1]=(-1.30860297683)*x[0]+(-0.653929967017)*x[1]
        arg[0,2,1,0]=(-0.0155994484453)*x[0]+(-0.274023166937)*x[1]
        arg[0,2,1,1]=(0.31458360335)*x[0]+(1.59507700673)*x[1]
        arg[0,2,2,0]=(-0.11442619553)*x[0]+(1.6577028576)*x[1]
        arg[0,2,2,1]=(0.274261132032)*x[0]+(1.15236962479)*x[1]
        arg[0,3,0,0]=(1.91433860239)*x[0]+(-0.316349747811)*x[1]
        arg[0,3,0,1]=(-1.42457591054)*x[0]+(-0.362812690805)*x[1]
        arg[0,3,1,0]=(0.572734883096)*x[0]+(0.424724787283)*x[1]
        arg[0,3,1,1]=(-0.118538376813)*x[0]+(0.25190312527)*x[1]
        arg[0,3,2,0]=(-0.533409510081)*x[0]+(-0.804349107915)*x[1]
        arg[0,3,2,1]=(-0.290555363666)*x[0]+(-0.912641477145)*x[1]
        arg[0,4,0,0]=(-0.469612775918)*x[0]+(-0.0447411310793)*x[1]
        arg[0,4,0,1]=(0.108538258831)*x[0]+(0.0453962016662)*x[1]
        arg[0,4,1,0]=(-0.130785944028)*x[0]+(0.0887280914032)*x[1]
        arg[0,4,1,1]=(0.166412309585)*x[0]+(-0.75130797706)*x[1]
        arg[0,4,2,0]=(1.22538232455)*x[0]+(0.112925588343)*x[1]
        arg[0,4,2,1]=(-1.02455171854)*x[0]+(-0.725754942749)*x[1]
        arg[1,0,0,0]=(-0.869171086952)*x[0]+(-0.948876783682)*x[1]
        arg[1,0,0,1]=(0.17730673689)*x[0]+(-0.00166180855579)*x[1]
        arg[1,0,1,0]=(0.482072704433)*x[0]+(0.895847096783)*x[1]
        arg[1,0,1,1]=(0.695736834617)*x[0]+(0.358082704558)*x[1]
        arg[1,0,2,0]=(-0.501799636565)*x[0]+(0.363612558025)*x[1]
        arg[1,0,2,1]=(-1.47204447226)*x[0]+(1.91281426121)*x[1]
        arg[1,1,0,0]=(0.241893182978)*x[0]+(0.95199758669)*x[1]
        arg[1,1,0,1]=(-0.68112361827)*x[0]+(-0.241935141301)*x[1]
        arg[1,1,1,0]=(-0.553030972306)*x[0]+(0.0244503690439)*x[1]
        arg[1,1,1,1]=(0.278177176896)*x[0]+(1.58693846427)*x[1]
        arg[1,1,2,0]=(-0.87980015517)*x[0]+(0.315618837123)*x[1]
        arg[1,1,2,1]=(-0.335783961379)*x[0]+(0.368650136138)*x[1]
        arg[1,2,0,0]=(-0.903535146802)*x[0]+(1.68182923194)*x[1]
        arg[1,2,0,1]=(1.06511511382)*x[0]+(-1.08524762302)*x[1]
        arg[1,2,1,0]=(-0.0455274784681)*x[0]+(-0.519576710661)*x[1]
        arg[1,2,1,1]=(0.735100217062)*x[0]+(-0.986268682671)*x[1]
        arg[1,2,2,0]=(-0.544610797462)*x[0]+(0.447237114882)*x[1]
        arg[1,2,2,1]=(0.385242031153)*x[0]+(-0.960538844247)*x[1]
        arg[1,3,0,0]=(-0.354927072897)*x[0]+(0.842574324062)*x[1]
        arg[1,3,0,1]=(0.319036427919)*x[0]+(-0.156282262536)*x[1]
        arg[1,3,1,0]=(0.587123807717)*x[0]+(1.65277990726)*x[1]
        arg[1,3,1,1]=(0.791697370298)*x[0]+(-1.5681680344)*x[1]
        arg[1,3,2,0]=(-1.41931242045)*x[0]+(0.351285136557)*x[1]
        arg[1,3,2,1]=(0.53175704402)*x[0]+(0.987523241324)*x[1]
        arg[1,4,0,0]=(0.00533504818465)*x[0]+(-0.158886930817)*x[1]
        arg[1,4,0,1]=(1.23725297303)*x[0]+(-0.302896452172)*x[1]
        arg[1,4,1,0]=(0.433953146329)*x[0]+(-0.0721602234887)*x[1]
        arg[1,4,1,1]=(-0.383499726865)*x[0]+(-0.253834671235)*x[1]
        arg[1,4,2,0]=(-0.480776695247)*x[0]+(-0.302145524759)*x[1]
        arg[1,4,2,1]=(1.14113180903)*x[0]+(-1.24304726003)*x[1]
        arg[2,0,0,0]=(0.513776013893)*x[0]+(-0.16660259885)*x[1]
        arg[2,0,0,1]=(1.55739209698)*x[0]+(0.0524084654531)*x[1]
        arg[2,0,1,0]=(0.222567881446)*x[0]+(-0.357158470388)*x[1]
        arg[2,0,1,1]=(0.0179816596229)*x[0]+(0.231534460784)*x[1]
        arg[2,0,2,0]=(1.58460345297)*x[0]+(-0.383588433888)*x[1]
        arg[2,0,2,1]=(-1.73386465536)*x[0]+(-0.743326657544)*x[1]
        arg[2,1,0,0]=(-0.737976950246)*x[0]+(-1.01053396224)*x[1]
        arg[2,1,0,1]=(0.926911190393)*x[0]+(0.620251783925)*x[1]
        arg[2,1,1,0]=(-1.34289630129)*x[0]+(-0.429933681523)*x[1]
        arg[2,1,1,1]=(1.79530806415)*x[0]+(-0.539783957021)*x[1]
        arg[2,1,2,0]=(0.237950974938)*x[0]+(1.93783427703)*x[1]
        arg[2,1,2,1]=(-0.439640741973)*x[0]+(-0.61543984673)*x[1]
        arg[2,2,0,0]=(-1.66403612438)*x[0]+(1.00839566441)*x[1]
        arg[2,2,0,1]=(0.249635983351)*x[0]+(-0.722261363675)*x[1]
        arg[2,2,1,0]=(1.34179674459)*x[0]+(0.502542144188)*x[1]
        arg[2,2,1,1]=(-0.815862586074)*x[0]+(-0.110688273707)*x[1]
        arg[2,2,2,0]=(-0.686810719526)*x[0]+(0.500406563421)*x[1]
        arg[2,2,2,1]=(0.681231089598)*x[0]+(0.605219505613)*x[1]
        arg[2,3,0,0]=(-0.953299029956)*x[0]+(-0.223725858372)*x[1]
        arg[2,3,0,1]=(0.831380756684)*x[0]+(-0.0379777076335)*x[1]
        arg[2,3,1,0]=(-1.63804393157)*x[0]+(0.148235731301)*x[1]
        arg[2,3,1,1]=(-0.17786458643)*x[0]+(-1.77342152699)*x[1]
        arg[2,3,2,0]=(0.0210446244192)*x[0]+(1.59778781524)*x[1]
        arg[2,3,2,1]=(0.877162721214)*x[0]+(0.262990448614)*x[1]
        arg[2,4,0,0]=(-0.650265970331)*x[0]+(-1.07003604369)*x[1]
        arg[2,4,0,1]=(1.12940101029)*x[0]+(0.860589290891)*x[1]
        arg[2,4,1,0]=(0.520056210056)*x[0]+(-1.47691193979)*x[1]
        arg[2,4,1,1]=(1.26823535235)*x[0]+(0.496811720647)*x[1]
        arg[2,4,2,0]=(-1.388913443)*x[0]+(0.585343197943)*x[1]
        arg[2,4,2,1]=(-0.762904547594)*x[0]+(0.503438815917)*x[1]
        arg[3,0,0,0]=(0.382862686993)*x[0]+(-0.923525513303)*x[1]
        arg[3,0,0,1]=(-0.54305794485)*x[0]+(0.467513531444)*x[1]
        arg[3,0,1,0]=(1.24228759776)*x[0]+(-1.65740117421)*x[1]
        arg[3,0,1,1]=(-0.463045362742)*x[0]+(0.186438587087)*x[1]
        arg[3,0,2,0]=(0.296881980828)*x[0]+(-0.385878814949)*x[1]
        arg[3,0,2,1]=(-0.299705899564)*x[0]+(0.490736038538)*x[1]
        arg[3,1,0,0]=(-0.786851428658)*x[0]+(-0.760593794308)*x[1]
        arg[3,1,0,1]=(0.506851336483)*x[0]+(0.995163951197)*x[1]
        arg[3,1,1,0]=(-0.678883937341)*x[0]+(0.56996778248)*x[1]
        arg[3,1,1,1]=(-0.401483642528)*x[0]+(0.0476797332537)*x[1]
        arg[3,1,2,0]=(-0.585357955375)*x[0]+(0.330103647539)*x[1]
        arg[3,1,2,1]=(-0.355539976825)*x[0]+(0.416287924349)*x[1]
        arg[3,2,0,0]=(-0.662837130286)*x[0]+(-0.26640899674)*x[1]
        arg[3,2,0,1]=(0.397553900588)*x[0]+(-1.21708620294)*x[1]
        arg[3,2,1,0]=(0.464963326429)*x[0]+(-0.774515114142)*x[1]
        arg[3,2,1,1]=(0.834351469791)*x[0]+(0.487629540715)*x[1]
        arg[3,2,2,0]=(0.27622693716)*x[0]+(0.811338337719)*x[1]
        arg[3,2,2,1]=(0.0384997772653)*x[0]+(1.03724996578)*x[1]
        arg[3,3,0,0]=(-0.805491987874)*x[0]+(-0.476384547853)*x[1]
        arg[3,3,0,1]=(0.615551780195)*x[0]+(-0.312805967691)*x[1]
        arg[3,3,1,0]=(0.348666798955)*x[0]+(-1.31271156524)*x[1]
        arg[3,3,1,1]=(-0.423569975374)*x[0]+(0.930713125389)*x[1]
        arg[3,3,2,0]=(0.325632686234)*x[0]+(-0.166401033259)*x[1]
        arg[3,3,2,1]=(0.126924861797)*x[0]+(-0.425380897969)*x[1]
        arg[3,4,0,0]=(0.169532204538)*x[0]+(0.614632525765)*x[1]
        arg[3,4,0,1]=(-0.938098696851)*x[0]+(0.648854147587)*x[1]
        arg[3,4,1,0]=(-0.545143970434)*x[0]+(-1.59787515966)*x[1]
        arg[3,4,1,1]=(-0.711190914978)*x[0]+(0.897331522911)*x[1]
        arg[3,4,2,0]=(-0.386498124525)*x[0]+(0.299740420711)*x[1]
        arg[3,4,2,1]=(0.617683032062)*x[0]+(-0.449587703061)*x[1]
        ref[0,0,0,0]=(0.227969627252)*x_ref[0]+(0.778177350248)*x_ref[1]
        ref[0,0,0,1]=(-0.684883711494)*x_ref[0]+(-0.574285957132)*x_ref[1]
        ref[0,0,1,0]=(0.483090608974)*x_ref[0]+(0.0779929087959)*x_ref[1]
        ref[0,0,1,1]=(-0.293150814932)*x_ref[0]+(1.35368931993)*x_ref[1]
        ref[0,0,2,0]=(-0.190664734662)*x_ref[0]+(0.910883386754)*x_ref[1]
        ref[0,0,2,1]=(-1.51255575457)*x_ref[0]+(0.421560181501)*x_ref[1]
        ref[0,1,0,0]=(-0.833402828032)*x_ref[0]+(-0.400466806781)*x_ref[1]
        ref[0,1,0,1]=(-0.212099323605)*x_ref[0]+(-0.112478697221)*x_ref[1]
        ref[0,1,1,0]=(0.57687662926)*x_ref[0]+(0.869758954551)*x_ref[1]
        ref[0,1,1,1]=(1.37261769917)*x_ref[0]+(1.37855789907)*x_ref[1]
        ref[0,1,2,0]=(-1.02838589722)*x_ref[0]+(-0.280409246204)*x_ref[1]
        ref[0,1,2,1]=(1.2333373468)*x_ref[0]+(0.337631183798)*x_ref[1]
        ref[0,2,0,0]=(-0.249065448772)*x_ref[0]+(0.834483646317)*x_ref[1]
        ref[0,2,0,1]=(-1.30860297683)*x_ref[0]+(-0.653929967017)*x_ref[1]
        ref[0,2,1,0]=(-0.0155994484453)*x_ref[0]+(-0.274023166937)*x_ref[1]
        ref[0,2,1,1]=(0.31458360335)*x_ref[0]+(1.59507700673)*x_ref[1]
        ref[0,2,2,0]=(-0.11442619553)*x_ref[0]+(1.6577028576)*x_ref[1]
        ref[0,2,2,1]=(0.274261132032)*x_ref[0]+(1.15236962479)*x_ref[1]
        ref[0,3,0,0]=(1.91433860239)*x_ref[0]+(-0.316349747811)*x_ref[1]
        ref[0,3,0,1]=(-1.42457591054)*x_ref[0]+(-0.362812690805)*x_ref[1]
        ref[0,3,1,0]=(0.572734883096)*x_ref[0]+(0.424724787283)*x_ref[1]
        ref[0,3,1,1]=(-0.118538376813)*x_ref[0]+(0.25190312527)*x_ref[1]
        ref[0,3,2,0]=(-0.533409510081)*x_ref[0]+(-0.804349107915)*x_ref[1]
        ref[0,3,2,1]=(-0.290555363666)*x_ref[0]+(-0.912641477145)*x_ref[1]
        ref[0,4,0,0]=(-0.469612775918)*x_ref[0]+(-0.0447411310793)*x_ref[1]
        ref[0,4,0,1]=(0.108538258831)*x_ref[0]+(0.0453962016662)*x_ref[1]
        ref[0,4,1,0]=(-0.130785944028)*x_ref[0]+(0.0887280914032)*x_ref[1]
        ref[0,4,1,1]=(0.166412309585)*x_ref[0]+(-0.75130797706)*x_ref[1]
        ref[0,4,2,0]=(1.22538232455)*x_ref[0]+(0.112925588343)*x_ref[1]
        ref[0,4,2,1]=(-1.02455171854)*x_ref[0]+(-0.725754942749)*x_ref[1]
        ref[1,0,0,0]=(-0.869171086952)*x_ref[0]+(-0.948876783682)*x_ref[1]
        ref[1,0,0,1]=(0.17730673689)*x_ref[0]+(-0.00166180855579)*x_ref[1]
        ref[1,0,1,0]=(0.482072704433)*x_ref[0]+(0.895847096783)*x_ref[1]
        ref[1,0,1,1]=(0.695736834617)*x_ref[0]+(0.358082704558)*x_ref[1]
        ref[1,0,2,0]=(-0.501799636565)*x_ref[0]+(0.363612558025)*x_ref[1]
        ref[1,0,2,1]=(-1.47204447226)*x_ref[0]+(1.91281426121)*x_ref[1]
        ref[1,1,0,0]=(0.241893182978)*x_ref[0]+(0.95199758669)*x_ref[1]
        ref[1,1,0,1]=(-0.68112361827)*x_ref[0]+(-0.241935141301)*x_ref[1]
        ref[1,1,1,0]=(-0.553030972306)*x_ref[0]+(0.0244503690439)*x_ref[1]
        ref[1,1,1,1]=(0.278177176896)*x_ref[0]+(1.58693846427)*x_ref[1]
        ref[1,1,2,0]=(-0.87980015517)*x_ref[0]+(0.315618837123)*x_ref[1]
        ref[1,1,2,1]=(-0.335783961379)*x_ref[0]+(0.368650136138)*x_ref[1]
        ref[1,2,0,0]=(-0.903535146802)*x_ref[0]+(1.68182923194)*x_ref[1]
        ref[1,2,0,1]=(1.06511511382)*x_ref[0]+(-1.08524762302)*x_ref[1]
        ref[1,2,1,0]=(-0.0455274784681)*x_ref[0]+(-0.519576710661)*x_ref[1]
        ref[1,2,1,1]=(0.735100217062)*x_ref[0]+(-0.986268682671)*x_ref[1]
        ref[1,2,2,0]=(-0.544610797462)*x_ref[0]+(0.447237114882)*x_ref[1]
        ref[1,2,2,1]=(0.385242031153)*x_ref[0]+(-0.960538844247)*x_ref[1]
        ref[1,3,0,0]=(-0.354927072897)*x_ref[0]+(0.842574324062)*x_ref[1]
        ref[1,3,0,1]=(0.319036427919)*x_ref[0]+(-0.156282262536)*x_ref[1]
        ref[1,3,1,0]=(0.587123807717)*x_ref[0]+(1.65277990726)*x_ref[1]
        ref[1,3,1,1]=(0.791697370298)*x_ref[0]+(-1.5681680344)*x_ref[1]
        ref[1,3,2,0]=(-1.41931242045)*x_ref[0]+(0.351285136557)*x_ref[1]
        ref[1,3,2,1]=(0.53175704402)*x_ref[0]+(0.987523241324)*x_ref[1]
        ref[1,4,0,0]=(0.00533504818465)*x_ref[0]+(-0.158886930817)*x_ref[1]
        ref[1,4,0,1]=(1.23725297303)*x_ref[0]+(-0.302896452172)*x_ref[1]
        ref[1,4,1,0]=(0.433953146329)*x_ref[0]+(-0.0721602234887)*x_ref[1]
        ref[1,4,1,1]=(-0.383499726865)*x_ref[0]+(-0.253834671235)*x_ref[1]
        ref[1,4,2,0]=(-0.480776695247)*x_ref[0]+(-0.302145524759)*x_ref[1]
        ref[1,4,2,1]=(1.14113180903)*x_ref[0]+(-1.24304726003)*x_ref[1]
        ref[2,0,0,0]=(0.513776013893)*x_ref[0]+(-0.16660259885)*x_ref[1]
        ref[2,0,0,1]=(1.55739209698)*x_ref[0]+(0.0524084654531)*x_ref[1]
        ref[2,0,1,0]=(0.222567881446)*x_ref[0]+(-0.357158470388)*x_ref[1]
        ref[2,0,1,1]=(0.0179816596229)*x_ref[0]+(0.231534460784)*x_ref[1]
        ref[2,0,2,0]=(1.58460345297)*x_ref[0]+(-0.383588433888)*x_ref[1]
        ref[2,0,2,1]=(-1.73386465536)*x_ref[0]+(-0.743326657544)*x_ref[1]
        ref[2,1,0,0]=(-0.737976950246)*x_ref[0]+(-1.01053396224)*x_ref[1]
        ref[2,1,0,1]=(0.926911190393)*x_ref[0]+(0.620251783925)*x_ref[1]
        ref[2,1,1,0]=(-1.34289630129)*x_ref[0]+(-0.429933681523)*x_ref[1]
        ref[2,1,1,1]=(1.79530806415)*x_ref[0]+(-0.539783957021)*x_ref[1]
        ref[2,1,2,0]=(0.237950974938)*x_ref[0]+(1.93783427703)*x_ref[1]
        ref[2,1,2,1]=(-0.439640741973)*x_ref[0]+(-0.61543984673)*x_ref[1]
        ref[2,2,0,0]=(-1.66403612438)*x_ref[0]+(1.00839566441)*x_ref[1]
        ref[2,2,0,1]=(0.249635983351)*x_ref[0]+(-0.722261363675)*x_ref[1]
        ref[2,2,1,0]=(1.34179674459)*x_ref[0]+(0.502542144188)*x_ref[1]
        ref[2,2,1,1]=(-0.815862586074)*x_ref[0]+(-0.110688273707)*x_ref[1]
        ref[2,2,2,0]=(-0.686810719526)*x_ref[0]+(0.500406563421)*x_ref[1]
        ref[2,2,2,1]=(0.681231089598)*x_ref[0]+(0.605219505613)*x_ref[1]
        ref[2,3,0,0]=(-0.953299029956)*x_ref[0]+(-0.223725858372)*x_ref[1]
        ref[2,3,0,1]=(0.831380756684)*x_ref[0]+(-0.0379777076335)*x_ref[1]
        ref[2,3,1,0]=(-1.63804393157)*x_ref[0]+(0.148235731301)*x_ref[1]
        ref[2,3,1,1]=(-0.17786458643)*x_ref[0]+(-1.77342152699)*x_ref[1]
        ref[2,3,2,0]=(0.0210446244192)*x_ref[0]+(1.59778781524)*x_ref[1]
        ref[2,3,2,1]=(0.877162721214)*x_ref[0]+(0.262990448614)*x_ref[1]
        ref[2,4,0,0]=(-0.650265970331)*x_ref[0]+(-1.07003604369)*x_ref[1]
        ref[2,4,0,1]=(1.12940101029)*x_ref[0]+(0.860589290891)*x_ref[1]
        ref[2,4,1,0]=(0.520056210056)*x_ref[0]+(-1.47691193979)*x_ref[1]
        ref[2,4,1,1]=(1.26823535235)*x_ref[0]+(0.496811720647)*x_ref[1]
        ref[2,4,2,0]=(-1.388913443)*x_ref[0]+(0.585343197943)*x_ref[1]
        ref[2,4,2,1]=(-0.762904547594)*x_ref[0]+(0.503438815917)*x_ref[1]
        ref[3,0,0,0]=(0.382862686993)*x_ref[0]+(-0.923525513303)*x_ref[1]
        ref[3,0,0,1]=(-0.54305794485)*x_ref[0]+(0.467513531444)*x_ref[1]
        ref[3,0,1,0]=(1.24228759776)*x_ref[0]+(-1.65740117421)*x_ref[1]
        ref[3,0,1,1]=(-0.463045362742)*x_ref[0]+(0.186438587087)*x_ref[1]
        ref[3,0,2,0]=(0.296881980828)*x_ref[0]+(-0.385878814949)*x_ref[1]
        ref[3,0,2,1]=(-0.299705899564)*x_ref[0]+(0.490736038538)*x_ref[1]
        ref[3,1,0,0]=(-0.786851428658)*x_ref[0]+(-0.760593794308)*x_ref[1]
        ref[3,1,0,1]=(0.506851336483)*x_ref[0]+(0.995163951197)*x_ref[1]
        ref[3,1,1,0]=(-0.678883937341)*x_ref[0]+(0.56996778248)*x_ref[1]
        ref[3,1,1,1]=(-0.401483642528)*x_ref[0]+(0.0476797332537)*x_ref[1]
        ref[3,1,2,0]=(-0.585357955375)*x_ref[0]+(0.330103647539)*x_ref[1]
        ref[3,1,2,1]=(-0.355539976825)*x_ref[0]+(0.416287924349)*x_ref[1]
        ref[3,2,0,0]=(-0.662837130286)*x_ref[0]+(-0.26640899674)*x_ref[1]
        ref[3,2,0,1]=(0.397553900588)*x_ref[0]+(-1.21708620294)*x_ref[1]
        ref[3,2,1,0]=(0.464963326429)*x_ref[0]+(-0.774515114142)*x_ref[1]
        ref[3,2,1,1]=(0.834351469791)*x_ref[0]+(0.487629540715)*x_ref[1]
        ref[3,2,2,0]=(0.27622693716)*x_ref[0]+(0.811338337719)*x_ref[1]
        ref[3,2,2,1]=(0.0384997772653)*x_ref[0]+(1.03724996578)*x_ref[1]
        ref[3,3,0,0]=(-0.805491987874)*x_ref[0]+(-0.476384547853)*x_ref[1]
        ref[3,3,0,1]=(0.615551780195)*x_ref[0]+(-0.312805967691)*x_ref[1]
        ref[3,3,1,0]=(0.348666798955)*x_ref[0]+(-1.31271156524)*x_ref[1]
        ref[3,3,1,1]=(-0.423569975374)*x_ref[0]+(0.930713125389)*x_ref[1]
        ref[3,3,2,0]=(0.325632686234)*x_ref[0]+(-0.166401033259)*x_ref[1]
        ref[3,3,2,1]=(0.126924861797)*x_ref[0]+(-0.425380897969)*x_ref[1]
        ref[3,4,0,0]=(0.169532204538)*x_ref[0]+(0.614632525765)*x_ref[1]
        ref[3,4,0,1]=(-0.938098696851)*x_ref[0]+(0.648854147587)*x_ref[1]
        ref[3,4,1,0]=(-0.545143970434)*x_ref[0]+(-1.59787515966)*x_ref[1]
        ref[3,4,1,1]=(-0.711190914978)*x_ref[0]+(0.897331522911)*x_ref[1]
        ref[3,4,2,0]=(-0.386498124525)*x_ref[0]+(0.299740420711)*x_ref[1]
        ref[3,4,2,1]=(0.617683032062)*x_ref[0]+(-0.449587703061)*x_ref[1]
      else:
        arg[0,0,0,0]=(-0.791620170192)*x[0]+(1.03871460301)*x[1]+(-0.225035657687)*x[2]
        arg[0,0,0,1]=(-0.464422319203)*x[0]+(0.769454931053)*x[1]+(-0.783260637771)*x[2]
        arg[0,0,1,0]=(0.72655538215)*x[0]+(-1.65736312015)*x[1]+(1.08402505927)*x[2]
        arg[0,0,1,1]=(-0.207576709573)*x[0]+(-0.55250594761)*x[1]+(-0.923702938288)*x[2]
        arg[0,0,2,0]=(-0.701991085516)*x[0]+(-1.76863643107)*x[1]+(0.831382933104)*x[2]
        arg[0,0,2,1]=(0.0640035527319)*x[0]+(0.425446342976)*x[1]+(1.64626766696)*x[2]
        arg[0,1,0,0]=(1.01022963801)*x[0]+(-0.0215015059716)*x[1]+(0.849862425592)*x[2]
        arg[0,1,0,1]=(-0.45776932943)*x[0]+(0.7621342151)*x[1]+(-1.40123018391)*x[2]
        arg[0,1,1,0]=(0.737485054127)*x[0]+(-0.147513905858)*x[1]+(-0.616928595213)*x[2]
        arg[0,1,1,1]=(-0.0608005391894)*x[0]+(-0.154978210573)*x[1]+(-1.40322011577)*x[2]
        arg[0,1,2,0]=(1.59908177714)*x[0]+(-0.378738573932)*x[1]+(1.45285315247)*x[2]
        arg[0,1,2,1]=(-0.622552157046)*x[0]+(1.19996417516)*x[1]+(0.728842900843)*x[2]
        arg[0,2,0,0]=(-0.46942034691)*x[0]+(0.333309721372)*x[1]+(-0.280846553832)*x[2]
        arg[0,2,0,1]=(1.22208788109)*x[0]+(0.369339594285)*x[1]+(1.31078425589)*x[2]
        arg[0,2,1,0]=(0.0794165004114)*x[0]+(0.44665451846)*x[1]+(-0.814625708916)*x[2]
        arg[0,2,1,1]=(0.594200807165)*x[0]+(-0.00616720333193)*x[1]+(0.382706175172)*x[2]
        arg[0,2,2,0]=(0.0372230170459)*x[0]+(0.440339923291)*x[1]+(0.137641070028)*x[2]
        arg[0,2,2,1]=(-1.29863078274)*x[0]+(0.283142455171)*x[1]+(0.0844540074659)*x[2]
        arg[0,3,0,0]=(1.72811808036)*x[0]+(0.536456781522)*x[1]+(0.0266737704706)*x[2]
        arg[0,3,0,1]=(0.148219555101)*x[0]+(-0.6887500275)*x[1]+(-0.405613136632)*x[2]
        arg[0,3,1,0]=(0.708262537279)*x[0]+(1.16319635773)*x[1]+(-0.030808721674)*x[2]
        arg[0,3,1,1]=(0.80834615863)*x[0]+(-0.176789000925)*x[1]+(-1.72576120963)*x[2]
        arg[0,3,2,0]=(0.21128315299)*x[0]+(0.770037046126)*x[1]+(0.988028934422)*x[2]
        arg[0,3,2,1]=(1.10522136317)*x[0]+(1.6319351743)*x[1]+(-0.784693713387)*x[2]
        arg[0,4,0,0]=(0.121088121064)*x[0]+(-1.79718067791)*x[1]+(-0.907857173318)*x[2]
        arg[0,4,0,1]=(-0.695887294217)*x[0]+(-1.24166787807)*x[1]+(-1.24968023856)*x[2]
        arg[0,4,1,0]=(-0.202510797291)*x[0]+(0.628310156105)*x[1]+(0.605146447822)*x[2]
        arg[0,4,1,1]=(0.149227458424)*x[0]+(-1.5650392775)*x[1]+(-0.763944892663)*x[2]
        arg[0,4,2,0]=(-0.393218625872)*x[0]+(0.479145914948)*x[1]+(-0.313960447492)*x[2]
        arg[0,4,2,1]=(-0.0191807443799)*x[0]+(0.592992659119)*x[1]+(-0.475633115484)*x[2]
        arg[1,0,0,0]=(-0.964829354261)*x[0]+(0.453014802906)*x[1]+(-0.779502566938)*x[2]
        arg[1,0,0,1]=(0.0151205739508)*x[0]+(-0.153706429594)*x[1]+(0.00866460475724)*x[2]
        arg[1,0,1,0]=(0.92592276366)*x[0]+(0.530787400525)*x[1]+(-0.0577912606876)*x[2]
        arg[1,0,1,1]=(0.948404467077)*x[0]+(-0.265905470751)*x[1]+(0.472349192005)*x[2]
        arg[1,0,2,0]=(0.0176615383702)*x[0]+(0.679605821761)*x[1]+(-0.0333590911941)*x[2]
        arg[1,0,2,1]=(-1.15594952629)*x[0]+(0.264868376508)*x[1]+(-0.883690535499)*x[2]
        arg[1,1,0,0]=(-0.738712703994)*x[0]+(0.545839232808)*x[1]+(0.592474501134)*x[2]
        arg[1,1,0,1]=(-1.3468960453)*x[0]+(-0.00848202443614)*x[1]+(-0.0124138286126)*x[2]
        arg[1,1,1,0]=(0.900228409707)*x[0]+(-1.86848082357)*x[1]+(0.421539431142)*x[2]
        arg[1,1,1,1]=(1.42754013963)*x[0]+(-0.861376440449)*x[1]+(0.0965520362291)*x[2]
        arg[1,1,2,0]=(-0.35873674515)*x[0]+(0.0862592617941)*x[1]+(-0.536106207025)*x[2]
        arg[1,1,2,1]=(0.90712910097)*x[0]+(0.974545253055)*x[1]+(0.570708510858)*x[2]
        arg[1,2,0,0]=(1.19310775669)*x[0]+(0.309414331679)*x[1]+(-0.271298061586)*x[2]
        arg[1,2,0,1]=(0.551778129545)*x[0]+(0.539028430134)*x[1]+(0.239406239262)*x[2]
        arg[1,2,1,0]=(-0.810625054344)*x[0]+(-0.241072190233)*x[1]+(-1.58590369097)*x[2]
        arg[1,2,1,1]=(0.529669075734)*x[0]+(-0.890365024163)*x[1]+(-1.69798509084)*x[2]
        arg[1,2,2,0]=(0.0437087707448)*x[0]+(0.88372190198)*x[1]+(1.86547747806)*x[2]
        arg[1,2,2,1]=(1.30739438898)*x[0]+(0.315850112101)*x[1]+(-0.262001133683)*x[2]
        arg[1,3,0,0]=(-0.143426236837)*x[0]+(-1.1703384826)*x[1]+(0.50608348646)*x[2]
        arg[1,3,0,1]=(-0.0444408776922)*x[0]+(1.26952782036)*x[1]+(1.67192602934)*x[2]
        arg[1,3,1,0]=(0.672742824368)*x[0]+(-0.785246676537)*x[1]+(0.716130664056)*x[2]
        arg[1,3,1,1]=(-0.295182508335)*x[0]+(-1.53658860897)*x[1]+(-1.87133995422)*x[2]
        arg[1,3,2,0]=(-0.440859530081)*x[0]+(0.0730120015821)*x[1]+(-1.02465537353)*x[2]
        arg[1,3,2,1]=(0.838320612003)*x[0]+(1.81233688454)*x[1]+(-1.0633876685)*x[2]
        arg[1,4,0,0]=(0.52440453051)*x[0]+(-0.0182588999022)*x[1]+(1.05556310351)*x[2]
        arg[1,4,0,1]=(0.766612727495)*x[0]+(1.58987590746)*x[1]+(-0.703801484569)*x[2]
        arg[1,4,1,0]=(0.373439757535)*x[0]+(-0.0676963704305)*x[1]+(0.0948034094293)*x[2]
        arg[1,4,1,1]=(-0.542283543375)*x[0]+(-0.454408605761)*x[1]+(-0.398904635316)*x[2]
        arg[1,4,2,0]=(1.3374871143)*x[0]+(0.0067849394587)*x[1]+(-0.319452094882)*x[2]
        arg[1,4,2,1]=(0.38530755393)*x[0]+(1.05356462429)*x[1]+(-0.302179447719)*x[2]
        arg[2,0,0,0]=(-0.123109761583)*x[0]+(0.69223848143)*x[1]+(-1.36903487092)*x[2]
        arg[2,0,0,1]=(0.121957328459)*x[0]+(0.102229685387)*x[1]+(0.962499974737)*x[2]
        arg[2,0,1,0]=(0.478614188653)*x[0]+(-1.23058817023)*x[1]+(1.30950997478)*x[2]
        arg[2,0,1,1]=(-0.529033562461)*x[0]+(0.563030509591)*x[1]+(-0.564939226654)*x[2]
        arg[2,0,2,0]=(-0.377740282406)*x[0]+(1.20532110343)*x[1]+(1.55615174947)*x[2]
        arg[2,0,2,1]=(0.539119710703)*x[0]+(1.91771865857)*x[1]+(0.0809882488048)*x[2]
        arg[2,1,0,0]=(0.00548664737796)*x[0]+(-0.157716099068)*x[1]+(-1.40009044235)*x[2]
        arg[2,1,0,1]=(0.0667557890666)*x[0]+(0.742484448131)*x[1]+(-0.325246111346)*x[2]
        arg[2,1,1,0]=(-0.214965474071)*x[0]+(-0.215055151964)*x[1]+(1.15673960537)*x[2]
        arg[2,1,1,1]=(0.605764831789)*x[0]+(-1.0432964832)*x[1]+(0.645482523161)*x[2]
        arg[2,1,2,0]=(-1.13098364088)*x[0]+(-0.932044089041)*x[1]+(-0.308053828241)*x[2]
        arg[2,1,2,1]=(0.481961305459)*x[0]+(-0.101277472029)*x[1]+(-0.17520207974)*x[2]
        arg[2,2,0,0]=(0.579831852303)*x[0]+(0.538731990254)*x[1]+(-0.0897460374205)*x[2]
        arg[2,2,0,1]=(-1.8763181434)*x[0]+(-0.132710928569)*x[1]+(1.03169751651)*x[2]
        arg[2,2,1,0]=(0.56871511431)*x[0]+(0.075333481656)*x[1]+(1.10238818953)*x[2]
        arg[2,2,1,1]=(1.49347755246)*x[0]+(-1.00500941047)*x[1]+(-0.181733196359)*x[2]
        arg[2,2,2,0]=(0.532881509971)*x[0]+(-0.364333964659)*x[1]+(1.09633720883)*x[2]
        arg[2,2,2,1]=(-0.518419048615)*x[0]+(-0.48091100593)*x[1]+(0.724628840153)*x[2]
        arg[2,3,0,0]=(0.353475859331)*x[0]+(-0.0239879269825)*x[1]+(1.32068893718)*x[2]
        arg[2,3,0,1]=(0.553999122102)*x[0]+(-0.128530869239)*x[1]+(0.134663336287)*x[2]
        arg[2,3,1,0]=(-1.59775882569)*x[0]+(-1.07335424785)*x[1]+(1.39223587988)*x[2]
        arg[2,3,1,1]=(-0.921252080095)*x[0]+(1.46226299067)*x[1]+(-1.13412499932)*x[2]
        arg[2,3,2,0]=(0.417572910178)*x[0]+(0.0814889599489)*x[1]+(0.292159249719)*x[2]
        arg[2,3,2,1]=(1.44620228921)*x[0]+(0.767854577367)*x[1]+(-0.0113709806383)*x[2]
        arg[2,4,0,0]=(-0.156987001681)*x[0]+(-0.922524257833)*x[1]+(0.521128360838)*x[2]
        arg[2,4,0,1]=(0.154198133354)*x[0]+(0.462020370175)*x[1]+(0.966768653665)*x[2]
        arg[2,4,1,0]=(0.507836849242)*x[0]+(1.16701911244)*x[1]+(0.264543038211)*x[2]
        arg[2,4,1,1]=(0.452059436879)*x[0]+(0.409662432778)*x[1]+(-0.403800872595)*x[2]
        arg[2,4,2,0]=(0.521175521008)*x[0]+(-0.362551578816)*x[1]+(0.0922727331623)*x[2]
        arg[2,4,2,1]=(-1.0990361823)*x[0]+(1.73655146851)*x[1]+(-0.867917402922)*x[2]
        arg[3,0,0,0]=(-0.0527010882236)*x[0]+(-1.11528215488)*x[1]+(1.3141906393)*x[2]
        arg[3,0,0,1]=(0.151284271917)*x[0]+(-0.0159077450668)*x[1]+(1.1798338931)*x[2]
        arg[3,0,1,0]=(-1.7648963435)*x[0]+(0.229511437402)*x[1]+(0.924415900312)*x[2]
        arg[3,0,1,1]=(-0.498271821094)*x[0]+(1.58363063019)*x[1]+(1.64871750388)*x[2]
        arg[3,0,2,0]=(-1.11245170725)*x[0]+(1.27662612569)*x[1]+(0.145950372369)*x[2]
        arg[3,0,2,1]=(-0.691643924238)*x[0]+(-0.178211874279)*x[1]+(-0.341071050366)*x[2]
        arg[3,1,0,0]=(-0.929804889461)*x[0]+(0.199087067309)*x[1]+(1.00150168883)*x[2]
        arg[3,1,0,1]=(-1.25152265093)*x[0]+(0.517076294911)*x[1]+(0.103798312362)*x[2]
        arg[3,1,1,0]=(0.797316290623)*x[0]+(0.0239285379293)*x[1]+(-0.836519299418)*x[2]
        arg[3,1,1,1]=(-0.470084592176)*x[0]+(0.844364701728)*x[1]+(-0.027373275093)*x[2]
        arg[3,1,2,0]=(1.0340279373)*x[0]+(-0.0133951276315)*x[1]+(1.17710546194)*x[2]
        arg[3,1,2,1]=(0.0325929541496)*x[0]+(1.13784867184)*x[1]+(-0.0529194266769)*x[2]
        arg[3,2,0,0]=(1.92837561772)*x[0]+(-1.63633468831)*x[1]+(1.4212324281)*x[2]
        arg[3,2,0,1]=(1.09556054408)*x[0]+(1.43286059545)*x[1]+(1.20005604593)*x[2]
        arg[3,2,1,0]=(0.308887169169)*x[0]+(-0.149214224208)*x[1]+(-1.13559614877)*x[2]
        arg[3,2,1,1]=(-0.0648352909004)*x[0]+(-0.5645470596)*x[1]+(-0.525131730514)*x[2]
        arg[3,2,2,0]=(0.21826829974)*x[0]+(1.5950239996)*x[1]+(0.333592425438)*x[2]
        arg[3,2,2,1]=(-1.25613956172)*x[0]+(-0.242338679844)*x[1]+(-0.958875577074)*x[2]
        arg[3,3,0,0]=(1.65005362159)*x[0]+(-1.16713867165)*x[1]+(-0.282495293685)*x[2]
        arg[3,3,0,1]=(1.04854406427)*x[0]+(0.876270949531)*x[1]+(0.291036304743)*x[2]
        arg[3,3,1,0]=(-1.66619451175)*x[0]+(0.656275192484)*x[1]+(-1.27104499942)*x[2]
        arg[3,3,1,1]=(1.55407146907)*x[0]+(-0.322950943159)*x[1]+(-0.287565403896)*x[2]
        arg[3,3,2,0]=(-1.54760268583)*x[0]+(-1.45847761887)*x[1]+(1.13939567009)*x[2]
        arg[3,3,2,1]=(1.24632351699)*x[0]+(0.0265500081594)*x[1]+(1.61925095159)*x[2]
        arg[3,4,0,0]=(-0.0430889433549)*x[0]+(-1.46995559463)*x[1]+(-1.14915159823)*x[2]
        arg[3,4,0,1]=(0.0392641403409)*x[0]+(0.0603053049639)*x[1]+(0.647321169934)*x[2]
        arg[3,4,1,0]=(-0.156300985227)*x[0]+(0.79369911928)*x[1]+(-0.413124762507)*x[2]
        arg[3,4,1,1]=(0.258402059015)*x[0]+(0.00227307988585)*x[1]+(0.70067141295)*x[2]
        arg[3,4,2,0]=(-1.15131298872)*x[0]+(-0.627449043157)*x[1]+(-0.402268345861)*x[2]
        arg[3,4,2,1]=(0.455934238179)*x[0]+(-0.951120759108)*x[1]+(-0.280459367131)*x[2]
        ref[0,0,0,0]=(-0.791620170192)*x_ref[0]+(1.03871460301)*x_ref[1]+(-0.225035657687)*x_ref[2]
        ref[0,0,0,1]=(-0.464422319203)*x_ref[0]+(0.769454931053)*x_ref[1]+(-0.783260637771)*x_ref[2]
        ref[0,0,1,0]=(0.72655538215)*x_ref[0]+(-1.65736312015)*x_ref[1]+(1.08402505927)*x_ref[2]
        ref[0,0,1,1]=(-0.207576709573)*x_ref[0]+(-0.55250594761)*x_ref[1]+(-0.923702938288)*x_ref[2]
        ref[0,0,2,0]=(-0.701991085516)*x_ref[0]+(-1.76863643107)*x_ref[1]+(0.831382933104)*x_ref[2]
        ref[0,0,2,1]=(0.0640035527319)*x_ref[0]+(0.425446342976)*x_ref[1]+(1.64626766696)*x_ref[2]
        ref[0,1,0,0]=(1.01022963801)*x_ref[0]+(-0.0215015059716)*x_ref[1]+(0.849862425592)*x_ref[2]
        ref[0,1,0,1]=(-0.45776932943)*x_ref[0]+(0.7621342151)*x_ref[1]+(-1.40123018391)*x_ref[2]
        ref[0,1,1,0]=(0.737485054127)*x_ref[0]+(-0.147513905858)*x_ref[1]+(-0.616928595213)*x_ref[2]
        ref[0,1,1,1]=(-0.0608005391894)*x_ref[0]+(-0.154978210573)*x_ref[1]+(-1.40322011577)*x_ref[2]
        ref[0,1,2,0]=(1.59908177714)*x_ref[0]+(-0.378738573932)*x_ref[1]+(1.45285315247)*x_ref[2]
        ref[0,1,2,1]=(-0.622552157046)*x_ref[0]+(1.19996417516)*x_ref[1]+(0.728842900843)*x_ref[2]
        ref[0,2,0,0]=(-0.46942034691)*x_ref[0]+(0.333309721372)*x_ref[1]+(-0.280846553832)*x_ref[2]
        ref[0,2,0,1]=(1.22208788109)*x_ref[0]+(0.369339594285)*x_ref[1]+(1.31078425589)*x_ref[2]
        ref[0,2,1,0]=(0.0794165004114)*x_ref[0]+(0.44665451846)*x_ref[1]+(-0.814625708916)*x_ref[2]
        ref[0,2,1,1]=(0.594200807165)*x_ref[0]+(-0.00616720333193)*x_ref[1]+(0.382706175172)*x_ref[2]
        ref[0,2,2,0]=(0.0372230170459)*x_ref[0]+(0.440339923291)*x_ref[1]+(0.137641070028)*x_ref[2]
        ref[0,2,2,1]=(-1.29863078274)*x_ref[0]+(0.283142455171)*x_ref[1]+(0.0844540074659)*x_ref[2]
        ref[0,3,0,0]=(1.72811808036)*x_ref[0]+(0.536456781522)*x_ref[1]+(0.0266737704706)*x_ref[2]
        ref[0,3,0,1]=(0.148219555101)*x_ref[0]+(-0.6887500275)*x_ref[1]+(-0.405613136632)*x_ref[2]
        ref[0,3,1,0]=(0.708262537279)*x_ref[0]+(1.16319635773)*x_ref[1]+(-0.030808721674)*x_ref[2]
        ref[0,3,1,1]=(0.80834615863)*x_ref[0]+(-0.176789000925)*x_ref[1]+(-1.72576120963)*x_ref[2]
        ref[0,3,2,0]=(0.21128315299)*x_ref[0]+(0.770037046126)*x_ref[1]+(0.988028934422)*x_ref[2]
        ref[0,3,2,1]=(1.10522136317)*x_ref[0]+(1.6319351743)*x_ref[1]+(-0.784693713387)*x_ref[2]
        ref[0,4,0,0]=(0.121088121064)*x_ref[0]+(-1.79718067791)*x_ref[1]+(-0.907857173318)*x_ref[2]
        ref[0,4,0,1]=(-0.695887294217)*x_ref[0]+(-1.24166787807)*x_ref[1]+(-1.24968023856)*x_ref[2]
        ref[0,4,1,0]=(-0.202510797291)*x_ref[0]+(0.628310156105)*x_ref[1]+(0.605146447822)*x_ref[2]
        ref[0,4,1,1]=(0.149227458424)*x_ref[0]+(-1.5650392775)*x_ref[1]+(-0.763944892663)*x_ref[2]
        ref[0,4,2,0]=(-0.393218625872)*x_ref[0]+(0.479145914948)*x_ref[1]+(-0.313960447492)*x_ref[2]
        ref[0,4,2,1]=(-0.0191807443799)*x_ref[0]+(0.592992659119)*x_ref[1]+(-0.475633115484)*x_ref[2]
        ref[1,0,0,0]=(-0.964829354261)*x_ref[0]+(0.453014802906)*x_ref[1]+(-0.779502566938)*x_ref[2]
        ref[1,0,0,1]=(0.0151205739508)*x_ref[0]+(-0.153706429594)*x_ref[1]+(0.00866460475724)*x_ref[2]
        ref[1,0,1,0]=(0.92592276366)*x_ref[0]+(0.530787400525)*x_ref[1]+(-0.0577912606876)*x_ref[2]
        ref[1,0,1,1]=(0.948404467077)*x_ref[0]+(-0.265905470751)*x_ref[1]+(0.472349192005)*x_ref[2]
        ref[1,0,2,0]=(0.0176615383702)*x_ref[0]+(0.679605821761)*x_ref[1]+(-0.0333590911941)*x_ref[2]
        ref[1,0,2,1]=(-1.15594952629)*x_ref[0]+(0.264868376508)*x_ref[1]+(-0.883690535499)*x_ref[2]
        ref[1,1,0,0]=(-0.738712703994)*x_ref[0]+(0.545839232808)*x_ref[1]+(0.592474501134)*x_ref[2]
        ref[1,1,0,1]=(-1.3468960453)*x_ref[0]+(-0.00848202443614)*x_ref[1]+(-0.0124138286126)*x_ref[2]
        ref[1,1,1,0]=(0.900228409707)*x_ref[0]+(-1.86848082357)*x_ref[1]+(0.421539431142)*x_ref[2]
        ref[1,1,1,1]=(1.42754013963)*x_ref[0]+(-0.861376440449)*x_ref[1]+(0.0965520362291)*x_ref[2]
        ref[1,1,2,0]=(-0.35873674515)*x_ref[0]+(0.0862592617941)*x_ref[1]+(-0.536106207025)*x_ref[2]
        ref[1,1,2,1]=(0.90712910097)*x_ref[0]+(0.974545253055)*x_ref[1]+(0.570708510858)*x_ref[2]
        ref[1,2,0,0]=(1.19310775669)*x_ref[0]+(0.309414331679)*x_ref[1]+(-0.271298061586)*x_ref[2]
        ref[1,2,0,1]=(0.551778129545)*x_ref[0]+(0.539028430134)*x_ref[1]+(0.239406239262)*x_ref[2]
        ref[1,2,1,0]=(-0.810625054344)*x_ref[0]+(-0.241072190233)*x_ref[1]+(-1.58590369097)*x_ref[2]
        ref[1,2,1,1]=(0.529669075734)*x_ref[0]+(-0.890365024163)*x_ref[1]+(-1.69798509084)*x_ref[2]
        ref[1,2,2,0]=(0.0437087707448)*x_ref[0]+(0.88372190198)*x_ref[1]+(1.86547747806)*x_ref[2]
        ref[1,2,2,1]=(1.30739438898)*x_ref[0]+(0.315850112101)*x_ref[1]+(-0.262001133683)*x_ref[2]
        ref[1,3,0,0]=(-0.143426236837)*x_ref[0]+(-1.1703384826)*x_ref[1]+(0.50608348646)*x_ref[2]
        ref[1,3,0,1]=(-0.0444408776922)*x_ref[0]+(1.26952782036)*x_ref[1]+(1.67192602934)*x_ref[2]
        ref[1,3,1,0]=(0.672742824368)*x_ref[0]+(-0.785246676537)*x_ref[1]+(0.716130664056)*x_ref[2]
        ref[1,3,1,1]=(-0.295182508335)*x_ref[0]+(-1.53658860897)*x_ref[1]+(-1.87133995422)*x_ref[2]
        ref[1,3,2,0]=(-0.440859530081)*x_ref[0]+(0.0730120015821)*x_ref[1]+(-1.02465537353)*x_ref[2]
        ref[1,3,2,1]=(0.838320612003)*x_ref[0]+(1.81233688454)*x_ref[1]+(-1.0633876685)*x_ref[2]
        ref[1,4,0,0]=(0.52440453051)*x_ref[0]+(-0.0182588999022)*x_ref[1]+(1.05556310351)*x_ref[2]
        ref[1,4,0,1]=(0.766612727495)*x_ref[0]+(1.58987590746)*x_ref[1]+(-0.703801484569)*x_ref[2]
        ref[1,4,1,0]=(0.373439757535)*x_ref[0]+(-0.0676963704305)*x_ref[1]+(0.0948034094293)*x_ref[2]
        ref[1,4,1,1]=(-0.542283543375)*x_ref[0]+(-0.454408605761)*x_ref[1]+(-0.398904635316)*x_ref[2]
        ref[1,4,2,0]=(1.3374871143)*x_ref[0]+(0.0067849394587)*x_ref[1]+(-0.319452094882)*x_ref[2]
        ref[1,4,2,1]=(0.38530755393)*x_ref[0]+(1.05356462429)*x_ref[1]+(-0.302179447719)*x_ref[2]
        ref[2,0,0,0]=(-0.123109761583)*x_ref[0]+(0.69223848143)*x_ref[1]+(-1.36903487092)*x_ref[2]
        ref[2,0,0,1]=(0.121957328459)*x_ref[0]+(0.102229685387)*x_ref[1]+(0.962499974737)*x_ref[2]
        ref[2,0,1,0]=(0.478614188653)*x_ref[0]+(-1.23058817023)*x_ref[1]+(1.30950997478)*x_ref[2]
        ref[2,0,1,1]=(-0.529033562461)*x_ref[0]+(0.563030509591)*x_ref[1]+(-0.564939226654)*x_ref[2]
        ref[2,0,2,0]=(-0.377740282406)*x_ref[0]+(1.20532110343)*x_ref[1]+(1.55615174947)*x_ref[2]
        ref[2,0,2,1]=(0.539119710703)*x_ref[0]+(1.91771865857)*x_ref[1]+(0.0809882488048)*x_ref[2]
        ref[2,1,0,0]=(0.00548664737796)*x_ref[0]+(-0.157716099068)*x_ref[1]+(-1.40009044235)*x_ref[2]
        ref[2,1,0,1]=(0.0667557890666)*x_ref[0]+(0.742484448131)*x_ref[1]+(-0.325246111346)*x_ref[2]
        ref[2,1,1,0]=(-0.214965474071)*x_ref[0]+(-0.215055151964)*x_ref[1]+(1.15673960537)*x_ref[2]
        ref[2,1,1,1]=(0.605764831789)*x_ref[0]+(-1.0432964832)*x_ref[1]+(0.645482523161)*x_ref[2]
        ref[2,1,2,0]=(-1.13098364088)*x_ref[0]+(-0.932044089041)*x_ref[1]+(-0.308053828241)*x_ref[2]
        ref[2,1,2,1]=(0.481961305459)*x_ref[0]+(-0.101277472029)*x_ref[1]+(-0.17520207974)*x_ref[2]
        ref[2,2,0,0]=(0.579831852303)*x_ref[0]+(0.538731990254)*x_ref[1]+(-0.0897460374205)*x_ref[2]
        ref[2,2,0,1]=(-1.8763181434)*x_ref[0]+(-0.132710928569)*x_ref[1]+(1.03169751651)*x_ref[2]
        ref[2,2,1,0]=(0.56871511431)*x_ref[0]+(0.075333481656)*x_ref[1]+(1.10238818953)*x_ref[2]
        ref[2,2,1,1]=(1.49347755246)*x_ref[0]+(-1.00500941047)*x_ref[1]+(-0.181733196359)*x_ref[2]
        ref[2,2,2,0]=(0.532881509971)*x_ref[0]+(-0.364333964659)*x_ref[1]+(1.09633720883)*x_ref[2]
        ref[2,2,2,1]=(-0.518419048615)*x_ref[0]+(-0.48091100593)*x_ref[1]+(0.724628840153)*x_ref[2]
        ref[2,3,0,0]=(0.353475859331)*x_ref[0]+(-0.0239879269825)*x_ref[1]+(1.32068893718)*x_ref[2]
        ref[2,3,0,1]=(0.553999122102)*x_ref[0]+(-0.128530869239)*x_ref[1]+(0.134663336287)*x_ref[2]
        ref[2,3,1,0]=(-1.59775882569)*x_ref[0]+(-1.07335424785)*x_ref[1]+(1.39223587988)*x_ref[2]
        ref[2,3,1,1]=(-0.921252080095)*x_ref[0]+(1.46226299067)*x_ref[1]+(-1.13412499932)*x_ref[2]
        ref[2,3,2,0]=(0.417572910178)*x_ref[0]+(0.0814889599489)*x_ref[1]+(0.292159249719)*x_ref[2]
        ref[2,3,2,1]=(1.44620228921)*x_ref[0]+(0.767854577367)*x_ref[1]+(-0.0113709806383)*x_ref[2]
        ref[2,4,0,0]=(-0.156987001681)*x_ref[0]+(-0.922524257833)*x_ref[1]+(0.521128360838)*x_ref[2]
        ref[2,4,0,1]=(0.154198133354)*x_ref[0]+(0.462020370175)*x_ref[1]+(0.966768653665)*x_ref[2]
        ref[2,4,1,0]=(0.507836849242)*x_ref[0]+(1.16701911244)*x_ref[1]+(0.264543038211)*x_ref[2]
        ref[2,4,1,1]=(0.452059436879)*x_ref[0]+(0.409662432778)*x_ref[1]+(-0.403800872595)*x_ref[2]
        ref[2,4,2,0]=(0.521175521008)*x_ref[0]+(-0.362551578816)*x_ref[1]+(0.0922727331623)*x_ref[2]
        ref[2,4,2,1]=(-1.0990361823)*x_ref[0]+(1.73655146851)*x_ref[1]+(-0.867917402922)*x_ref[2]
        ref[3,0,0,0]=(-0.0527010882236)*x_ref[0]+(-1.11528215488)*x_ref[1]+(1.3141906393)*x_ref[2]
        ref[3,0,0,1]=(0.151284271917)*x_ref[0]+(-0.0159077450668)*x_ref[1]+(1.1798338931)*x_ref[2]
        ref[3,0,1,0]=(-1.7648963435)*x_ref[0]+(0.229511437402)*x_ref[1]+(0.924415900312)*x_ref[2]
        ref[3,0,1,1]=(-0.498271821094)*x_ref[0]+(1.58363063019)*x_ref[1]+(1.64871750388)*x_ref[2]
        ref[3,0,2,0]=(-1.11245170725)*x_ref[0]+(1.27662612569)*x_ref[1]+(0.145950372369)*x_ref[2]
        ref[3,0,2,1]=(-0.691643924238)*x_ref[0]+(-0.178211874279)*x_ref[1]+(-0.341071050366)*x_ref[2]
        ref[3,1,0,0]=(-0.929804889461)*x_ref[0]+(0.199087067309)*x_ref[1]+(1.00150168883)*x_ref[2]
        ref[3,1,0,1]=(-1.25152265093)*x_ref[0]+(0.517076294911)*x_ref[1]+(0.103798312362)*x_ref[2]
        ref[3,1,1,0]=(0.797316290623)*x_ref[0]+(0.0239285379293)*x_ref[1]+(-0.836519299418)*x_ref[2]
        ref[3,1,1,1]=(-0.470084592176)*x_ref[0]+(0.844364701728)*x_ref[1]+(-0.027373275093)*x_ref[2]
        ref[3,1,2,0]=(1.0340279373)*x_ref[0]+(-0.0133951276315)*x_ref[1]+(1.17710546194)*x_ref[2]
        ref[3,1,2,1]=(0.0325929541496)*x_ref[0]+(1.13784867184)*x_ref[1]+(-0.0529194266769)*x_ref[2]
        ref[3,2,0,0]=(1.92837561772)*x_ref[0]+(-1.63633468831)*x_ref[1]+(1.4212324281)*x_ref[2]
        ref[3,2,0,1]=(1.09556054408)*x_ref[0]+(1.43286059545)*x_ref[1]+(1.20005604593)*x_ref[2]
        ref[3,2,1,0]=(0.308887169169)*x_ref[0]+(-0.149214224208)*x_ref[1]+(-1.13559614877)*x_ref[2]
        ref[3,2,1,1]=(-0.0648352909004)*x_ref[0]+(-0.5645470596)*x_ref[1]+(-0.525131730514)*x_ref[2]
        ref[3,2,2,0]=(0.21826829974)*x_ref[0]+(1.5950239996)*x_ref[1]+(0.333592425438)*x_ref[2]
        ref[3,2,2,1]=(-1.25613956172)*x_ref[0]+(-0.242338679844)*x_ref[1]+(-0.958875577074)*x_ref[2]
        ref[3,3,0,0]=(1.65005362159)*x_ref[0]+(-1.16713867165)*x_ref[1]+(-0.282495293685)*x_ref[2]
        ref[3,3,0,1]=(1.04854406427)*x_ref[0]+(0.876270949531)*x_ref[1]+(0.291036304743)*x_ref[2]
        ref[3,3,1,0]=(-1.66619451175)*x_ref[0]+(0.656275192484)*x_ref[1]+(-1.27104499942)*x_ref[2]
        ref[3,3,1,1]=(1.55407146907)*x_ref[0]+(-0.322950943159)*x_ref[1]+(-0.287565403896)*x_ref[2]
        ref[3,3,2,0]=(-1.54760268583)*x_ref[0]+(-1.45847761887)*x_ref[1]+(1.13939567009)*x_ref[2]
        ref[3,3,2,1]=(1.24632351699)*x_ref[0]+(0.0265500081594)*x_ref[1]+(1.61925095159)*x_ref[2]
        ref[3,4,0,0]=(-0.0430889433549)*x_ref[0]+(-1.46995559463)*x_ref[1]+(-1.14915159823)*x_ref[2]
        ref[3,4,0,1]=(0.0392641403409)*x_ref[0]+(0.0603053049639)*x_ref[1]+(0.647321169934)*x_ref[2]
        ref[3,4,1,0]=(-0.156300985227)*x_ref[0]+(0.79369911928)*x_ref[1]+(-0.413124762507)*x_ref[2]
        ref[3,4,1,1]=(0.258402059015)*x_ref[0]+(0.00227307988585)*x_ref[1]+(0.70067141295)*x_ref[2]
        ref[3,4,2,0]=(-1.15131298872)*x_ref[0]+(-0.627449043157)*x_ref[1]+(-0.402268345861)*x_ref[2]
        ref[3,4,2,1]=(0.455934238179)*x_ref[0]+(-0.951120759108)*x_ref[1]+(-0.280459367131)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_ReducedFunctionOnContactZero_rank0(self):
      """
      tests interpolate for rank 0 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactZero onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactZero(self.domain)
      x=w.getX()
      arg=Data(0,(),w)
      ref=Data(0,(),w_ref)
      if dim==2:
        arg=(0.203238386406)*x[0]**o+(0.734542238717)*x[0]+(-0.627248060056)*x[1]**o+(0.347224669854)*x[1]
        ref=(0.203238386406)*x_ref[0]**o+(0.734542238717)*x_ref[0]+(-0.627248060056)*x_ref[1]**o+(0.347224669854)*x_ref[1]
      else:
        arg=(-0.387501489724)*x[0]**o+(0.752554332333)*x[0]+(-0.0831588271355)*x[1]**o+(0.615191870812)*x[1]+(-0.264702923622)*x[2]**o+(0.404594629993)*x[2]
        ref=(-0.387501489724)*x_ref[0]**o+(0.752554332333)*x_ref[0]+(-0.0831588271355)*x_ref[1]**o+(0.615191870812)*x_ref[1]+(-0.264702923622)*x_ref[2]**o+(0.404594629993)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_ReducedFunctionOnContactZero_rank1(self):
      """
      tests interpolate for rank 1 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactZero onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactZero(self.domain)
      x=w.getX()
      arg=Data(0,(2,),w)
      ref=Data(0,(2,),w_ref)
      if dim==2:
        arg[0]=(0.938444792136)*x[0]**o+(-0.809407040724)*x[0]+(-0.963148615762)*x[1]**o+(-0.257000244129)*x[1]
        arg[1]=(-0.791280640039)*x[0]**o+(0.293858623095)*x[0]+(0.964942589784)*x[1]**o+(0.522071353081)*x[1]
        ref[0]=(0.938444792136)*x_ref[0]**o+(-0.809407040724)*x_ref[0]+(-0.963148615762)*x_ref[1]**o+(-0.257000244129)*x_ref[1]
        ref[1]=(-0.791280640039)*x_ref[0]**o+(0.293858623095)*x_ref[0]+(0.964942589784)*x_ref[1]**o+(0.522071353081)*x_ref[1]
      else:
        arg[0]=(0.566110580666)*x[0]**o+(-0.89723913213)*x[0]+(-0.54781629706)*x[1]**o+(-0.0405507445628)*x[1]+(0.435477036501)*x[2]**o+(-0.296404435183)*x[2]
        arg[1]=(-0.436126359475)*x[0]**o+(0.863706244241)*x[0]+(-0.149818409243)*x[1]**o+(-0.654801920098)*x[1]+(-0.281005092809)*x[2]**o+(-0.0257179420057)*x[2]
        ref[0]=(0.566110580666)*x_ref[0]**o+(-0.89723913213)*x_ref[0]+(-0.54781629706)*x_ref[1]**o+(-0.0405507445628)*x_ref[1]+(0.435477036501)*x_ref[2]**o+(-0.296404435183)*x_ref[2]
        ref[1]=(-0.436126359475)*x_ref[0]**o+(0.863706244241)*x_ref[0]+(-0.149818409243)*x_ref[1]**o+(-0.654801920098)*x_ref[1]+(-0.281005092809)*x_ref[2]**o+(-0.0257179420057)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_ReducedFunctionOnContactZero_rank2(self):
      """
      tests interpolate for rank 2 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactZero onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactZero(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5),w)
      ref=Data(0,(4, 5),w_ref)
      if dim==2:
        arg[0,0]=(0.0615713819144)*x[0]**o+(0.325133297443)*x[0]+(-0.529769594257)*x[1]**o+(-0.206126939263)*x[1]
        arg[0,1]=(0.090956391947)*x[0]**o+(0.105841035331)*x[0]+(-0.537033610998)*x[1]**o+(0.429033781495)*x[1]
        arg[0,2]=(-0.568216263139)*x[0]**o+(0.698057796806)*x[0]+(-0.0981614063428)*x[1]**o+(-0.952212332523)*x[1]
        arg[0,3]=(0.128505813477)*x[0]**o+(0.206828305852)*x[0]+(0.938851873532)*x[1]**o+(-0.339234589281)*x[1]
        arg[0,4]=(0.581805477855)*x[0]**o+(-0.621122347153)*x[0]+(-0.601029625565)*x[1]**o+(0.243084366725)*x[1]
        arg[1,0]=(-0.342411991491)*x[0]**o+(0.642434320731)*x[0]+(-0.452657094515)*x[1]**o+(-0.899779109444)*x[1]
        arg[1,1]=(-0.423230603077)*x[0]**o+(0.739600521332)*x[0]+(0.916226480419)*x[1]**o+(0.755264173162)*x[1]
        arg[1,2]=(0.381864221314)*x[0]**o+(0.806402187902)*x[0]+(0.410055116458)*x[1]**o+(-0.264405213472)*x[1]
        arg[1,3]=(0.293401427429)*x[0]**o+(-0.439209675154)*x[0]+(0.299588226145)*x[1]**o+(-0.289652201976)*x[1]
        arg[1,4]=(0.157626502865)*x[0]**o+(0.32264788209)*x[0]+(0.284522039757)*x[1]**o+(-0.460413671191)*x[1]
        arg[2,0]=(0.486148207)*x[0]**o+(0.0366950072658)*x[0]+(0.315299830901)*x[1]**o+(-0.320405042262)*x[1]
        arg[2,1]=(0.207440897542)*x[0]**o+(-0.604659890692)*x[0]+(-0.64841431916)*x[1]**o+(0.0881921209641)*x[1]
        arg[2,2]=(-0.61376840347)*x[0]**o+(0.421837679185)*x[0]+(-0.0445850531916)*x[1]**o+(0.771892018903)*x[1]
        arg[2,3]=(0.632443182263)*x[0]**o+(-0.965297013012)*x[0]+(-0.029677959681)*x[1]**o+(0.598431910551)*x[1]
        arg[2,4]=(-0.457656411094)*x[0]**o+(0.122435925199)*x[0]+(0.772690251937)*x[1]**o+(0.954829922926)*x[1]
        arg[3,0]=(0.929112925461)*x[0]**o+(-0.35689901655)*x[0]+(-0.139192866782)*x[1]**o+(-0.677883436952)*x[1]
        arg[3,1]=(-0.410318721103)*x[0]**o+(0.370830312699)*x[0]+(0.429011619434)*x[1]**o+(0.81780420854)*x[1]
        arg[3,2]=(0.312456682985)*x[0]**o+(0.526447664999)*x[0]+(0.763610570932)*x[1]**o+(-0.631887901017)*x[1]
        arg[3,3]=(0.749121567955)*x[0]**o+(0.168525340464)*x[0]+(-0.838409619895)*x[1]**o+(0.912745137098)*x[1]
        arg[3,4]=(-0.665336604643)*x[0]**o+(0.638293226479)*x[0]+(0.515957084712)*x[1]**o+(-0.261426720977)*x[1]
        ref[0,0]=(0.0615713819144)*x_ref[0]**o+(0.325133297443)*x_ref[0]+(-0.529769594257)*x_ref[1]**o+(-0.206126939263)*x_ref[1]
        ref[0,1]=(0.090956391947)*x_ref[0]**o+(0.105841035331)*x_ref[0]+(-0.537033610998)*x_ref[1]**o+(0.429033781495)*x_ref[1]
        ref[0,2]=(-0.568216263139)*x_ref[0]**o+(0.698057796806)*x_ref[0]+(-0.0981614063428)*x_ref[1]**o+(-0.952212332523)*x_ref[1]
        ref[0,3]=(0.128505813477)*x_ref[0]**o+(0.206828305852)*x_ref[0]+(0.938851873532)*x_ref[1]**o+(-0.339234589281)*x_ref[1]
        ref[0,4]=(0.581805477855)*x_ref[0]**o+(-0.621122347153)*x_ref[0]+(-0.601029625565)*x_ref[1]**o+(0.243084366725)*x_ref[1]
        ref[1,0]=(-0.342411991491)*x_ref[0]**o+(0.642434320731)*x_ref[0]+(-0.452657094515)*x_ref[1]**o+(-0.899779109444)*x_ref[1]
        ref[1,1]=(-0.423230603077)*x_ref[0]**o+(0.739600521332)*x_ref[0]+(0.916226480419)*x_ref[1]**o+(0.755264173162)*x_ref[1]
        ref[1,2]=(0.381864221314)*x_ref[0]**o+(0.806402187902)*x_ref[0]+(0.410055116458)*x_ref[1]**o+(-0.264405213472)*x_ref[1]
        ref[1,3]=(0.293401427429)*x_ref[0]**o+(-0.439209675154)*x_ref[0]+(0.299588226145)*x_ref[1]**o+(-0.289652201976)*x_ref[1]
        ref[1,4]=(0.157626502865)*x_ref[0]**o+(0.32264788209)*x_ref[0]+(0.284522039757)*x_ref[1]**o+(-0.460413671191)*x_ref[1]
        ref[2,0]=(0.486148207)*x_ref[0]**o+(0.0366950072658)*x_ref[0]+(0.315299830901)*x_ref[1]**o+(-0.320405042262)*x_ref[1]
        ref[2,1]=(0.207440897542)*x_ref[0]**o+(-0.604659890692)*x_ref[0]+(-0.64841431916)*x_ref[1]**o+(0.0881921209641)*x_ref[1]
        ref[2,2]=(-0.61376840347)*x_ref[0]**o+(0.421837679185)*x_ref[0]+(-0.0445850531916)*x_ref[1]**o+(0.771892018903)*x_ref[1]
        ref[2,3]=(0.632443182263)*x_ref[0]**o+(-0.965297013012)*x_ref[0]+(-0.029677959681)*x_ref[1]**o+(0.598431910551)*x_ref[1]
        ref[2,4]=(-0.457656411094)*x_ref[0]**o+(0.122435925199)*x_ref[0]+(0.772690251937)*x_ref[1]**o+(0.954829922926)*x_ref[1]
        ref[3,0]=(0.929112925461)*x_ref[0]**o+(-0.35689901655)*x_ref[0]+(-0.139192866782)*x_ref[1]**o+(-0.677883436952)*x_ref[1]
        ref[3,1]=(-0.410318721103)*x_ref[0]**o+(0.370830312699)*x_ref[0]+(0.429011619434)*x_ref[1]**o+(0.81780420854)*x_ref[1]
        ref[3,2]=(0.312456682985)*x_ref[0]**o+(0.526447664999)*x_ref[0]+(0.763610570932)*x_ref[1]**o+(-0.631887901017)*x_ref[1]
        ref[3,3]=(0.749121567955)*x_ref[0]**o+(0.168525340464)*x_ref[0]+(-0.838409619895)*x_ref[1]**o+(0.912745137098)*x_ref[1]
        ref[3,4]=(-0.665336604643)*x_ref[0]**o+(0.638293226479)*x_ref[0]+(0.515957084712)*x_ref[1]**o+(-0.261426720977)*x_ref[1]
      else:
        arg[0,0]=(-0.330165086386)*x[0]**o+(0.60381119065)*x[0]+(0.981018763011)*x[1]**o+(0.0487965999608)*x[1]+(-0.389835388349)*x[2]**o+(0.111199857376)*x[2]
        arg[0,1]=(0.518239033178)*x[0]**o+(0.60358784684)*x[0]+(-0.476990159655)*x[1]**o+(-0.93871565843)*x[1]+(-0.869845295741)*x[2]**o+(0.391409745802)*x[2]
        arg[0,2]=(0.0927396115951)*x[0]**o+(0.546122822928)*x[0]+(-0.557424056733)*x[1]**o+(0.0623374966436)*x[1]+(-0.414595153536)*x[2]**o+(-0.196069073366)*x[2]
        arg[0,3]=(-0.678432580882)*x[0]**o+(0.281850265141)*x[0]+(-0.323978960602)*x[1]**o+(0.655558769921)*x[1]+(-0.295747032089)*x[2]**o+(-0.152719838982)*x[2]
        arg[0,4]=(-0.904177669203)*x[0]**o+(0.750395234205)*x[0]+(0.267431783461)*x[1]**o+(-0.781406235306)*x[1]+(0.856906655757)*x[2]**o+(-0.875554940224)*x[2]
        arg[1,0]=(-0.243144846569)*x[0]**o+(0.970584922467)*x[0]+(-0.977250652473)*x[1]**o+(0.0239563255869)*x[1]+(-0.769423602851)*x[2]**o+(-0.433896361294)*x[2]
        arg[1,1]=(0.31104031046)*x[0]**o+(0.899859709831)*x[0]+(0.692351561919)*x[1]**o+(-0.0838870550002)*x[1]+(-0.276105616798)*x[2]**o+(0.115501348248)*x[2]
        arg[1,2]=(0.56702114595)*x[0]**o+(0.00240284646131)*x[0]+(-0.676314888884)*x[1]**o+(-0.678204346328)*x[1]+(0.472697088984)*x[2]**o+(-0.493949191712)*x[2]
        arg[1,3]=(-0.242548554067)*x[0]**o+(0.980927507053)*x[0]+(-0.955706390436)*x[1]**o+(0.347031550126)*x[1]+(0.234352268238)*x[2]**o+(0.321126345333)*x[2]
        arg[1,4]=(-0.390890566228)*x[0]**o+(-0.958497753144)*x[0]+(0.316397716043)*x[1]**o+(0.952289821769)*x[1]+(-0.027182029514)*x[2]**o+(-0.454422377872)*x[2]
        arg[2,0]=(0.295865166318)*x[0]**o+(-0.301129718468)*x[0]+(-0.748135755269)*x[1]**o+(-0.762413814085)*x[1]+(0.572270067591)*x[2]**o+(-0.393847032684)*x[2]
        arg[2,1]=(0.20229063622)*x[0]**o+(-0.768444028163)*x[0]+(0.378669775381)*x[1]**o+(-0.703384410944)*x[1]+(0.355653859396)*x[2]**o+(-0.228168949639)*x[2]
        arg[2,2]=(-0.31725134742)*x[0]**o+(0.525802867508)*x[0]+(0.13375126758)*x[1]**o+(-0.673551491186)*x[1]+(0.498905883728)*x[2]**o+(-0.102794461221)*x[2]
        arg[2,3]=(-0.396813838168)*x[0]**o+(0.0892605807843)*x[0]+(0.244133298111)*x[1]**o+(0.310900636168)*x[1]+(-0.256807063422)*x[2]**o+(0.565370697688)*x[2]
        arg[2,4]=(0.42011835881)*x[0]**o+(0.439847594375)*x[0]+(0.0731897272006)*x[1]**o+(-0.373911054753)*x[1]+(-0.50313315963)*x[2]**o+(0.356362677601)*x[2]
        arg[3,0]=(-0.444672845607)*x[0]**o+(-0.978424347535)*x[0]+(0.421699748442)*x[1]**o+(-0.0111798084161)*x[1]+(0.206042685881)*x[2]**o+(0.990523831215)*x[2]
        arg[3,1]=(0.942429627068)*x[0]**o+(-0.769989484282)*x[0]+(-0.879916738075)*x[1]**o+(-0.746448112871)*x[1]+(0.280824650861)*x[2]**o+(0.0632084823022)*x[2]
        arg[3,2]=(0.281070481177)*x[0]**o+(0.628020001876)*x[0]+(0.347767303962)*x[1]**o+(-0.276761252142)*x[1]+(0.950345260734)*x[2]**o+(-0.00148186335195)*x[2]
        arg[3,3]=(-0.37590210465)*x[0]**o+(0.034147350068)*x[0]+(0.245173418336)*x[1]**o+(0.0564051241759)*x[1]+(-0.157928106127)*x[2]**o+(-0.611677441847)*x[2]
        arg[3,4]=(-0.279066532276)*x[0]**o+(0.316033119063)*x[0]+(0.655948999472)*x[1]**o+(-0.520504180943)*x[1]+(0.531738158819)*x[2]**o+(0.899490827443)*x[2]
        ref[0,0]=(-0.330165086386)*x_ref[0]**o+(0.60381119065)*x_ref[0]+(0.981018763011)*x_ref[1]**o+(0.0487965999608)*x_ref[1]+(-0.389835388349)*x_ref[2]**o+(0.111199857376)*x_ref[2]
        ref[0,1]=(0.518239033178)*x_ref[0]**o+(0.60358784684)*x_ref[0]+(-0.476990159655)*x_ref[1]**o+(-0.93871565843)*x_ref[1]+(-0.869845295741)*x_ref[2]**o+(0.391409745802)*x_ref[2]
        ref[0,2]=(0.0927396115951)*x_ref[0]**o+(0.546122822928)*x_ref[0]+(-0.557424056733)*x_ref[1]**o+(0.0623374966436)*x_ref[1]+(-0.414595153536)*x_ref[2]**o+(-0.196069073366)*x_ref[2]
        ref[0,3]=(-0.678432580882)*x_ref[0]**o+(0.281850265141)*x_ref[0]+(-0.323978960602)*x_ref[1]**o+(0.655558769921)*x_ref[1]+(-0.295747032089)*x_ref[2]**o+(-0.152719838982)*x_ref[2]
        ref[0,4]=(-0.904177669203)*x_ref[0]**o+(0.750395234205)*x_ref[0]+(0.267431783461)*x_ref[1]**o+(-0.781406235306)*x_ref[1]+(0.856906655757)*x_ref[2]**o+(-0.875554940224)*x_ref[2]
        ref[1,0]=(-0.243144846569)*x_ref[0]**o+(0.970584922467)*x_ref[0]+(-0.977250652473)*x_ref[1]**o+(0.0239563255869)*x_ref[1]+(-0.769423602851)*x_ref[2]**o+(-0.433896361294)*x_ref[2]
        ref[1,1]=(0.31104031046)*x_ref[0]**o+(0.899859709831)*x_ref[0]+(0.692351561919)*x_ref[1]**o+(-0.0838870550002)*x_ref[1]+(-0.276105616798)*x_ref[2]**o+(0.115501348248)*x_ref[2]
        ref[1,2]=(0.56702114595)*x_ref[0]**o+(0.00240284646131)*x_ref[0]+(-0.676314888884)*x_ref[1]**o+(-0.678204346328)*x_ref[1]+(0.472697088984)*x_ref[2]**o+(-0.493949191712)*x_ref[2]
        ref[1,3]=(-0.242548554067)*x_ref[0]**o+(0.980927507053)*x_ref[0]+(-0.955706390436)*x_ref[1]**o+(0.347031550126)*x_ref[1]+(0.234352268238)*x_ref[2]**o+(0.321126345333)*x_ref[2]
        ref[1,4]=(-0.390890566228)*x_ref[0]**o+(-0.958497753144)*x_ref[0]+(0.316397716043)*x_ref[1]**o+(0.952289821769)*x_ref[1]+(-0.027182029514)*x_ref[2]**o+(-0.454422377872)*x_ref[2]
        ref[2,0]=(0.295865166318)*x_ref[0]**o+(-0.301129718468)*x_ref[0]+(-0.748135755269)*x_ref[1]**o+(-0.762413814085)*x_ref[1]+(0.572270067591)*x_ref[2]**o+(-0.393847032684)*x_ref[2]
        ref[2,1]=(0.20229063622)*x_ref[0]**o+(-0.768444028163)*x_ref[0]+(0.378669775381)*x_ref[1]**o+(-0.703384410944)*x_ref[1]+(0.355653859396)*x_ref[2]**o+(-0.228168949639)*x_ref[2]
        ref[2,2]=(-0.31725134742)*x_ref[0]**o+(0.525802867508)*x_ref[0]+(0.13375126758)*x_ref[1]**o+(-0.673551491186)*x_ref[1]+(0.498905883728)*x_ref[2]**o+(-0.102794461221)*x_ref[2]
        ref[2,3]=(-0.396813838168)*x_ref[0]**o+(0.0892605807843)*x_ref[0]+(0.244133298111)*x_ref[1]**o+(0.310900636168)*x_ref[1]+(-0.256807063422)*x_ref[2]**o+(0.565370697688)*x_ref[2]
        ref[2,4]=(0.42011835881)*x_ref[0]**o+(0.439847594375)*x_ref[0]+(0.0731897272006)*x_ref[1]**o+(-0.373911054753)*x_ref[1]+(-0.50313315963)*x_ref[2]**o+(0.356362677601)*x_ref[2]
        ref[3,0]=(-0.444672845607)*x_ref[0]**o+(-0.978424347535)*x_ref[0]+(0.421699748442)*x_ref[1]**o+(-0.0111798084161)*x_ref[1]+(0.206042685881)*x_ref[2]**o+(0.990523831215)*x_ref[2]
        ref[3,1]=(0.942429627068)*x_ref[0]**o+(-0.769989484282)*x_ref[0]+(-0.879916738075)*x_ref[1]**o+(-0.746448112871)*x_ref[1]+(0.280824650861)*x_ref[2]**o+(0.0632084823022)*x_ref[2]
        ref[3,2]=(0.281070481177)*x_ref[0]**o+(0.628020001876)*x_ref[0]+(0.347767303962)*x_ref[1]**o+(-0.276761252142)*x_ref[1]+(0.950345260734)*x_ref[2]**o+(-0.00148186335195)*x_ref[2]
        ref[3,3]=(-0.37590210465)*x_ref[0]**o+(0.034147350068)*x_ref[0]+(0.245173418336)*x_ref[1]**o+(0.0564051241759)*x_ref[1]+(-0.157928106127)*x_ref[2]**o+(-0.611677441847)*x_ref[2]
        ref[3,4]=(-0.279066532276)*x_ref[0]**o+(0.316033119063)*x_ref[0]+(0.655948999472)*x_ref[1]**o+(-0.520504180943)*x_ref[1]+(0.531738158819)*x_ref[2]**o+(0.899490827443)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_ReducedFunctionOnContactZero_rank3(self):
      """
      tests interpolate for rank 3 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactZero onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactZero(self.domain)
      x=w.getX()
      arg=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2),w_ref)
      if dim==2:
        arg[0,0,0]=(0.415438516191)*x[0]**o+(0.374659323173)*x[0]+(0.688470746611)*x[1]**o+(0.498183692176)*x[1]
        arg[0,0,1]=(-0.846725329522)*x[0]**o+(0.414376816326)*x[0]+(0.651609661615)*x[1]**o+(0.357606396667)*x[1]
        arg[0,1,0]=(0.728029172023)*x[0]**o+(0.975728459112)*x[0]+(-0.658211371639)*x[1]**o+(0.411894140351)*x[1]
        arg[0,1,1]=(0.807350281817)*x[0]**o+(-0.369755733522)*x[0]+(-0.519483139314)*x[1]**o+(-0.709353537648)*x[1]
        arg[1,0,0]=(0.0588259329479)*x[0]**o+(0.218362870441)*x[0]+(-0.0279239318934)*x[1]**o+(-0.75783635329)*x[1]
        arg[1,0,1]=(-0.429573558505)*x[0]**o+(-0.339046134618)*x[0]+(0.412777825743)*x[1]**o+(-0.306607333437)*x[1]
        arg[1,1,0]=(0.0222661407543)*x[0]**o+(0.927923674781)*x[0]+(-0.767468570033)*x[1]**o+(-0.0659150590367)*x[1]
        arg[1,1,1]=(0.372811733787)*x[0]**o+(-0.286500260061)*x[0]+(-0.807504375569)*x[1]**o+(-0.992850148234)*x[1]
        arg[2,0,0]=(0.368229828596)*x[0]**o+(0.672540290379)*x[0]+(-0.760462595095)*x[1]**o+(-0.761242492425)*x[1]
        arg[2,0,1]=(0.306249523133)*x[0]**o+(0.769928924717)*x[0]+(0.610293800742)*x[1]**o+(-0.221996271153)*x[1]
        arg[2,1,0]=(-0.909148832991)*x[0]**o+(-0.0293038712703)*x[0]+(-0.161707473269)*x[1]**o+(-0.534345319122)*x[1]
        arg[2,1,1]=(-0.146432487737)*x[0]**o+(0.93729266958)*x[0]+(-0.561919238789)*x[1]**o+(-0.449440501171)*x[1]
        arg[3,0,0]=(0.411711661035)*x[0]**o+(-0.0699878047274)*x[0]+(-0.0227688826645)*x[1]**o+(-0.939787633547)*x[1]
        arg[3,0,1]=(-0.82108820245)*x[0]**o+(0.225501168847)*x[0]+(0.622921674682)*x[1]**o+(0.61092413094)*x[1]
        arg[3,1,0]=(0.758116155912)*x[0]**o+(-0.117316615447)*x[0]+(0.0702245119124)*x[1]**o+(-0.275114508561)*x[1]
        arg[3,1,1]=(-0.32568550821)*x[0]**o+(-0.280144846326)*x[0]+(0.194971187876)*x[1]**o+(-0.728447244746)*x[1]
        arg[4,0,0]=(0.880042622981)*x[0]**o+(-0.395684607563)*x[0]+(-0.563304061265)*x[1]**o+(0.968663606772)*x[1]
        arg[4,0,1]=(-0.515392712458)*x[0]**o+(0.820857647342)*x[0]+(-0.476289639495)*x[1]**o+(-0.478062589609)*x[1]
        arg[4,1,0]=(0.534322425269)*x[0]**o+(0.975618695265)*x[0]+(-0.195764005045)*x[1]**o+(0.561996200643)*x[1]
        arg[4,1,1]=(-0.906243138672)*x[0]**o+(-0.580985047294)*x[0]+(-0.094609405051)*x[1]**o+(0.734326711649)*x[1]
        arg[5,0,0]=(-0.0655516121632)*x[0]**o+(0.785493123174)*x[0]+(-0.760068333675)*x[1]**o+(0.693869247554)*x[1]
        arg[5,0,1]=(-0.931019838892)*x[0]**o+(0.80958291621)*x[0]+(-0.590591206111)*x[1]**o+(-0.800900346601)*x[1]
        arg[5,1,0]=(0.448779048952)*x[0]**o+(-0.793450503268)*x[0]+(0.161067403301)*x[1]**o+(0.432054043544)*x[1]
        arg[5,1,1]=(0.0423607361972)*x[0]**o+(-0.664958045702)*x[0]+(-0.174728816918)*x[1]**o+(-0.394390068699)*x[1]
        ref[0,0,0]=(0.415438516191)*x_ref[0]**o+(0.374659323173)*x_ref[0]+(0.688470746611)*x_ref[1]**o+(0.498183692176)*x_ref[1]
        ref[0,0,1]=(-0.846725329522)*x_ref[0]**o+(0.414376816326)*x_ref[0]+(0.651609661615)*x_ref[1]**o+(0.357606396667)*x_ref[1]
        ref[0,1,0]=(0.728029172023)*x_ref[0]**o+(0.975728459112)*x_ref[0]+(-0.658211371639)*x_ref[1]**o+(0.411894140351)*x_ref[1]
        ref[0,1,1]=(0.807350281817)*x_ref[0]**o+(-0.369755733522)*x_ref[0]+(-0.519483139314)*x_ref[1]**o+(-0.709353537648)*x_ref[1]
        ref[1,0,0]=(0.0588259329479)*x_ref[0]**o+(0.218362870441)*x_ref[0]+(-0.0279239318934)*x_ref[1]**o+(-0.75783635329)*x_ref[1]
        ref[1,0,1]=(-0.429573558505)*x_ref[0]**o+(-0.339046134618)*x_ref[0]+(0.412777825743)*x_ref[1]**o+(-0.306607333437)*x_ref[1]
        ref[1,1,0]=(0.0222661407543)*x_ref[0]**o+(0.927923674781)*x_ref[0]+(-0.767468570033)*x_ref[1]**o+(-0.0659150590367)*x_ref[1]
        ref[1,1,1]=(0.372811733787)*x_ref[0]**o+(-0.286500260061)*x_ref[0]+(-0.807504375569)*x_ref[1]**o+(-0.992850148234)*x_ref[1]
        ref[2,0,0]=(0.368229828596)*x_ref[0]**o+(0.672540290379)*x_ref[0]+(-0.760462595095)*x_ref[1]**o+(-0.761242492425)*x_ref[1]
        ref[2,0,1]=(0.306249523133)*x_ref[0]**o+(0.769928924717)*x_ref[0]+(0.610293800742)*x_ref[1]**o+(-0.221996271153)*x_ref[1]
        ref[2,1,0]=(-0.909148832991)*x_ref[0]**o+(-0.0293038712703)*x_ref[0]+(-0.161707473269)*x_ref[1]**o+(-0.534345319122)*x_ref[1]
        ref[2,1,1]=(-0.146432487737)*x_ref[0]**o+(0.93729266958)*x_ref[0]+(-0.561919238789)*x_ref[1]**o+(-0.449440501171)*x_ref[1]
        ref[3,0,0]=(0.411711661035)*x_ref[0]**o+(-0.0699878047274)*x_ref[0]+(-0.0227688826645)*x_ref[1]**o+(-0.939787633547)*x_ref[1]
        ref[3,0,1]=(-0.82108820245)*x_ref[0]**o+(0.225501168847)*x_ref[0]+(0.622921674682)*x_ref[1]**o+(0.61092413094)*x_ref[1]
        ref[3,1,0]=(0.758116155912)*x_ref[0]**o+(-0.117316615447)*x_ref[0]+(0.0702245119124)*x_ref[1]**o+(-0.275114508561)*x_ref[1]
        ref[3,1,1]=(-0.32568550821)*x_ref[0]**o+(-0.280144846326)*x_ref[0]+(0.194971187876)*x_ref[1]**o+(-0.728447244746)*x_ref[1]
        ref[4,0,0]=(0.880042622981)*x_ref[0]**o+(-0.395684607563)*x_ref[0]+(-0.563304061265)*x_ref[1]**o+(0.968663606772)*x_ref[1]
        ref[4,0,1]=(-0.515392712458)*x_ref[0]**o+(0.820857647342)*x_ref[0]+(-0.476289639495)*x_ref[1]**o+(-0.478062589609)*x_ref[1]
        ref[4,1,0]=(0.534322425269)*x_ref[0]**o+(0.975618695265)*x_ref[0]+(-0.195764005045)*x_ref[1]**o+(0.561996200643)*x_ref[1]
        ref[4,1,1]=(-0.906243138672)*x_ref[0]**o+(-0.580985047294)*x_ref[0]+(-0.094609405051)*x_ref[1]**o+(0.734326711649)*x_ref[1]
        ref[5,0,0]=(-0.0655516121632)*x_ref[0]**o+(0.785493123174)*x_ref[0]+(-0.760068333675)*x_ref[1]**o+(0.693869247554)*x_ref[1]
        ref[5,0,1]=(-0.931019838892)*x_ref[0]**o+(0.80958291621)*x_ref[0]+(-0.590591206111)*x_ref[1]**o+(-0.800900346601)*x_ref[1]
        ref[5,1,0]=(0.448779048952)*x_ref[0]**o+(-0.793450503268)*x_ref[0]+(0.161067403301)*x_ref[1]**o+(0.432054043544)*x_ref[1]
        ref[5,1,1]=(0.0423607361972)*x_ref[0]**o+(-0.664958045702)*x_ref[0]+(-0.174728816918)*x_ref[1]**o+(-0.394390068699)*x_ref[1]
      else:
        arg[0,0,0]=(0.34355250678)*x[0]**o+(0.554590991175)*x[0]+(0.00673862595967)*x[1]**o+(-0.405764224977)*x[1]+(0.0865564883661)*x[2]**o+(0.102294137607)*x[2]
        arg[0,0,1]=(0.855234460708)*x[0]**o+(-0.710901486015)*x[0]+(0.546393138087)*x[1]**o+(0.847113479239)*x[1]+(-0.650701604407)*x[2]**o+(-0.914755510262)*x[2]
        arg[0,1,0]=(0.252544568503)*x[0]**o+(-0.473471976093)*x[0]+(-0.587315274764)*x[1]**o+(-0.269940894375)*x[1]+(0.360340735507)*x[2]**o+(-0.745440004827)*x[2]
        arg[0,1,1]=(0.11967506555)*x[0]**o+(0.696836453942)*x[0]+(-0.871162187396)*x[1]**o+(-0.315170111067)*x[1]+(0.232066109266)*x[2]**o+(-0.464730524838)*x[2]
        arg[1,0,0]=(-0.790535662997)*x[0]**o+(-0.964779159884)*x[0]+(-0.689937881374)*x[1]**o+(0.422799231262)*x[1]+(-0.97315874804)*x[2]**o+(-0.532103115205)*x[2]
        arg[1,0,1]=(-0.0487589554945)*x[0]**o+(-0.622735713658)*x[0]+(-0.520842613265)*x[1]**o+(0.803711422599)*x[1]+(0.440648875556)*x[2]**o+(0.98786672574)*x[2]
        arg[1,1,0]=(0.811189977538)*x[0]**o+(0.0175150486788)*x[0]+(-0.124328899976)*x[1]**o+(-0.942398680503)*x[1]+(-0.149270209895)*x[2]**o+(-0.853084407696)*x[2]
        arg[1,1,1]=(0.688378159744)*x[0]**o+(0.408766927878)*x[0]+(0.834128919609)*x[1]**o+(0.253605923886)*x[1]+(0.424992121054)*x[2]**o+(0.21016117963)*x[2]
        arg[2,0,0]=(0.277469067087)*x[0]**o+(0.539821557611)*x[0]+(0.348679402521)*x[1]**o+(-0.469772506537)*x[1]+(0.454583229137)*x[2]**o+(0.653681481133)*x[2]
        arg[2,0,1]=(-0.934067178483)*x[0]**o+(-0.172735098497)*x[0]+(-0.0347256850948)*x[1]**o+(-0.459971424944)*x[1]+(0.932661853019)*x[2]**o+(-0.186822590955)*x[2]
        arg[2,1,0]=(0.71698574321)*x[0]**o+(-0.197244831261)*x[0]+(0.183419576182)*x[1]**o+(-0.871451480853)*x[1]+(-0.349396269955)*x[2]**o+(0.095909246936)*x[2]
        arg[2,1,1]=(-0.00972985962106)*x[0]**o+(-0.919204152811)*x[0]+(-0.751744404365)*x[1]**o+(0.377096436852)*x[1]+(-0.924228885362)*x[2]**o+(0.451053813959)*x[2]
        arg[3,0,0]=(0.951119968373)*x[0]**o+(-0.0680027504528)*x[0]+(-0.113660990287)*x[1]**o+(-0.428999453003)*x[1]+(0.171804544103)*x[2]**o+(0.677635036297)*x[2]
        arg[3,0,1]=(0.62534864581)*x[0]**o+(0.389496246771)*x[0]+(0.621295327948)*x[1]**o+(-0.684165739023)*x[1]+(0.258637161492)*x[2]**o+(0.0441460764076)*x[2]
        arg[3,1,0]=(0.0348922721145)*x[0]**o+(0.091201286767)*x[0]+(0.421079169132)*x[1]**o+(-0.616504862591)*x[1]+(0.563927673176)*x[2]**o+(-0.183993394822)*x[2]
        arg[3,1,1]=(0.657959811783)*x[0]**o+(0.229729671382)*x[0]+(0.549853155207)*x[1]**o+(-0.847412930565)*x[1]+(-0.0780682500337)*x[2]**o+(0.574435563406)*x[2]
        arg[4,0,0]=(-0.679988515344)*x[0]**o+(0.0517331630343)*x[0]+(-0.160281064879)*x[1]**o+(-0.0672405944261)*x[1]+(-0.118968444607)*x[2]**o+(0.00980065834148)*x[2]
        arg[4,0,1]=(-0.150457972951)*x[0]**o+(0.671586511406)*x[0]+(-0.844112675259)*x[1]**o+(0.42766255101)*x[1]+(-0.22449721018)*x[2]**o+(-0.150416151059)*x[2]
        arg[4,1,0]=(0.0768992597753)*x[0]**o+(0.0139605476692)*x[0]+(-0.831269554863)*x[1]**o+(-0.279339344511)*x[1]+(0.387318821185)*x[2]**o+(0.967687121678)*x[2]
        arg[4,1,1]=(-0.670585948598)*x[0]**o+(0.422715818654)*x[0]+(0.623702390878)*x[1]**o+(0.922646081646)*x[1]+(-0.714191287039)*x[2]**o+(-0.99018084869)*x[2]
        arg[5,0,0]=(0.100273819257)*x[0]**o+(-0.769422291863)*x[0]+(-0.152611272068)*x[1]**o+(-0.396798631305)*x[1]+(0.502251047462)*x[2]**o+(0.993791216449)*x[2]
        arg[5,0,1]=(-0.0222540900657)*x[0]**o+(-0.339810353987)*x[0]+(0.241665078696)*x[1]**o+(0.111020568564)*x[1]+(0.298147316931)*x[2]**o+(0.391278660594)*x[2]
        arg[5,1,0]=(0.828297599555)*x[0]**o+(0.237678875371)*x[0]+(0.924551126274)*x[1]**o+(-0.663695049485)*x[1]+(-0.547123493132)*x[2]**o+(-0.788635745118)*x[2]
        arg[5,1,1]=(0.0726334370982)*x[0]**o+(-0.875688492957)*x[0]+(0.473726235108)*x[1]**o+(0.023025596151)*x[1]+(0.827449396402)*x[2]**o+(-0.154698941604)*x[2]
        ref[0,0,0]=(0.34355250678)*x_ref[0]**o+(0.554590991175)*x_ref[0]+(0.00673862595967)*x_ref[1]**o+(-0.405764224977)*x_ref[1]+(0.0865564883661)*x_ref[2]**o+(0.102294137607)*x_ref[2]
        ref[0,0,1]=(0.855234460708)*x_ref[0]**o+(-0.710901486015)*x_ref[0]+(0.546393138087)*x_ref[1]**o+(0.847113479239)*x_ref[1]+(-0.650701604407)*x_ref[2]**o+(-0.914755510262)*x_ref[2]
        ref[0,1,0]=(0.252544568503)*x_ref[0]**o+(-0.473471976093)*x_ref[0]+(-0.587315274764)*x_ref[1]**o+(-0.269940894375)*x_ref[1]+(0.360340735507)*x_ref[2]**o+(-0.745440004827)*x_ref[2]
        ref[0,1,1]=(0.11967506555)*x_ref[0]**o+(0.696836453942)*x_ref[0]+(-0.871162187396)*x_ref[1]**o+(-0.315170111067)*x_ref[1]+(0.232066109266)*x_ref[2]**o+(-0.464730524838)*x_ref[2]
        ref[1,0,0]=(-0.790535662997)*x_ref[0]**o+(-0.964779159884)*x_ref[0]+(-0.689937881374)*x_ref[1]**o+(0.422799231262)*x_ref[1]+(-0.97315874804)*x_ref[2]**o+(-0.532103115205)*x_ref[2]
        ref[1,0,1]=(-0.0487589554945)*x_ref[0]**o+(-0.622735713658)*x_ref[0]+(-0.520842613265)*x_ref[1]**o+(0.803711422599)*x_ref[1]+(0.440648875556)*x_ref[2]**o+(0.98786672574)*x_ref[2]
        ref[1,1,0]=(0.811189977538)*x_ref[0]**o+(0.0175150486788)*x_ref[0]+(-0.124328899976)*x_ref[1]**o+(-0.942398680503)*x_ref[1]+(-0.149270209895)*x_ref[2]**o+(-0.853084407696)*x_ref[2]
        ref[1,1,1]=(0.688378159744)*x_ref[0]**o+(0.408766927878)*x_ref[0]+(0.834128919609)*x_ref[1]**o+(0.253605923886)*x_ref[1]+(0.424992121054)*x_ref[2]**o+(0.21016117963)*x_ref[2]
        ref[2,0,0]=(0.277469067087)*x_ref[0]**o+(0.539821557611)*x_ref[0]+(0.348679402521)*x_ref[1]**o+(-0.469772506537)*x_ref[1]+(0.454583229137)*x_ref[2]**o+(0.653681481133)*x_ref[2]
        ref[2,0,1]=(-0.934067178483)*x_ref[0]**o+(-0.172735098497)*x_ref[0]+(-0.0347256850948)*x_ref[1]**o+(-0.459971424944)*x_ref[1]+(0.932661853019)*x_ref[2]**o+(-0.186822590955)*x_ref[2]
        ref[2,1,0]=(0.71698574321)*x_ref[0]**o+(-0.197244831261)*x_ref[0]+(0.183419576182)*x_ref[1]**o+(-0.871451480853)*x_ref[1]+(-0.349396269955)*x_ref[2]**o+(0.095909246936)*x_ref[2]
        ref[2,1,1]=(-0.00972985962106)*x_ref[0]**o+(-0.919204152811)*x_ref[0]+(-0.751744404365)*x_ref[1]**o+(0.377096436852)*x_ref[1]+(-0.924228885362)*x_ref[2]**o+(0.451053813959)*x_ref[2]
        ref[3,0,0]=(0.951119968373)*x_ref[0]**o+(-0.0680027504528)*x_ref[0]+(-0.113660990287)*x_ref[1]**o+(-0.428999453003)*x_ref[1]+(0.171804544103)*x_ref[2]**o+(0.677635036297)*x_ref[2]
        ref[3,0,1]=(0.62534864581)*x_ref[0]**o+(0.389496246771)*x_ref[0]+(0.621295327948)*x_ref[1]**o+(-0.684165739023)*x_ref[1]+(0.258637161492)*x_ref[2]**o+(0.0441460764076)*x_ref[2]
        ref[3,1,0]=(0.0348922721145)*x_ref[0]**o+(0.091201286767)*x_ref[0]+(0.421079169132)*x_ref[1]**o+(-0.616504862591)*x_ref[1]+(0.563927673176)*x_ref[2]**o+(-0.183993394822)*x_ref[2]
        ref[3,1,1]=(0.657959811783)*x_ref[0]**o+(0.229729671382)*x_ref[0]+(0.549853155207)*x_ref[1]**o+(-0.847412930565)*x_ref[1]+(-0.0780682500337)*x_ref[2]**o+(0.574435563406)*x_ref[2]
        ref[4,0,0]=(-0.679988515344)*x_ref[0]**o+(0.0517331630343)*x_ref[0]+(-0.160281064879)*x_ref[1]**o+(-0.0672405944261)*x_ref[1]+(-0.118968444607)*x_ref[2]**o+(0.00980065834148)*x_ref[2]
        ref[4,0,1]=(-0.150457972951)*x_ref[0]**o+(0.671586511406)*x_ref[0]+(-0.844112675259)*x_ref[1]**o+(0.42766255101)*x_ref[1]+(-0.22449721018)*x_ref[2]**o+(-0.150416151059)*x_ref[2]
        ref[4,1,0]=(0.0768992597753)*x_ref[0]**o+(0.0139605476692)*x_ref[0]+(-0.831269554863)*x_ref[1]**o+(-0.279339344511)*x_ref[1]+(0.387318821185)*x_ref[2]**o+(0.967687121678)*x_ref[2]
        ref[4,1,1]=(-0.670585948598)*x_ref[0]**o+(0.422715818654)*x_ref[0]+(0.623702390878)*x_ref[1]**o+(0.922646081646)*x_ref[1]+(-0.714191287039)*x_ref[2]**o+(-0.99018084869)*x_ref[2]
        ref[5,0,0]=(0.100273819257)*x_ref[0]**o+(-0.769422291863)*x_ref[0]+(-0.152611272068)*x_ref[1]**o+(-0.396798631305)*x_ref[1]+(0.502251047462)*x_ref[2]**o+(0.993791216449)*x_ref[2]
        ref[5,0,1]=(-0.0222540900657)*x_ref[0]**o+(-0.339810353987)*x_ref[0]+(0.241665078696)*x_ref[1]**o+(0.111020568564)*x_ref[1]+(0.298147316931)*x_ref[2]**o+(0.391278660594)*x_ref[2]
        ref[5,1,0]=(0.828297599555)*x_ref[0]**o+(0.237678875371)*x_ref[0]+(0.924551126274)*x_ref[1]**o+(-0.663695049485)*x_ref[1]+(-0.547123493132)*x_ref[2]**o+(-0.788635745118)*x_ref[2]
        ref[5,1,1]=(0.0726334370982)*x_ref[0]**o+(-0.875688492957)*x_ref[0]+(0.473726235108)*x_ref[1]**o+(0.023025596151)*x_ref[1]+(0.827449396402)*x_ref[2]**o+(-0.154698941604)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_ReducedFunctionOnContactZero_rank4(self):
      """
      tests interpolate for rank 4 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactZero onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactZero(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5, 3, 2),w)
      ref=Data(0,(4, 5, 3, 2),w_ref)
      if dim==2:
        arg[0,0,0,0]=(-0.0338299192888)*x[0]**o+(-0.114906274359)*x[0]+(0.15010339398)*x[1]**o+(0.625295991328)*x[1]
        arg[0,0,0,1]=(-0.1311901039)*x[0]**o+(-0.551712861217)*x[0]+(-0.287411404506)*x[1]**o+(0.0242820629572)*x[1]
        arg[0,0,1,0]=(0.121962094111)*x[0]**o+(0.203169622345)*x[0]+(0.241504218053)*x[1]**o+(-0.0696356239498)*x[1]
        arg[0,0,1,1]=(0.0461537352329)*x[0]**o+(-0.275908938567)*x[0]+(-0.271184121027)*x[1]**o+(-0.364024729567)*x[1]
        arg[0,0,2,0]=(0.199272234315)*x[0]**o+(-0.906364258855)*x[0]+(-0.92102930037)*x[1]**o+(0.0718981903454)*x[1]
        arg[0,0,2,1]=(-0.742950958383)*x[0]**o+(0.0301464847722)*x[0]+(0.466993477687)*x[1]**o+(-0.402131750698)*x[1]
        arg[0,1,0,0]=(-0.0653776546375)*x[0]**o+(-0.679360612372)*x[0]+(-0.32783837768)*x[1]**o+(-0.113753016428)*x[1]
        arg[0,1,0,1]=(0.58610119414)*x[0]**o+(0.142613166231)*x[0]+(0.0409105619986)*x[1]**o+(-0.660641663558)*x[1]
        arg[0,1,1,0]=(0.212172308585)*x[0]**o+(-0.452767768344)*x[0]+(0.963053969889)*x[1]**o+(0.752691773953)*x[1]
        arg[0,1,1,1]=(-0.514998377887)*x[0]**o+(0.0872464329801)*x[0]+(0.265590295505)*x[1]**o+(0.557238826549)*x[1]
        arg[0,1,2,0]=(0.360523954724)*x[0]**o+(0.550238402994)*x[0]+(-0.206349415539)*x[1]**o+(0.357304730511)*x[1]
        arg[0,1,2,1]=(-0.872755217435)*x[0]**o+(0.190943683664)*x[0]+(-0.245454292354)*x[1]**o+(0.507229578506)*x[1]
        arg[0,2,0,0]=(0.994619505257)*x[0]**o+(0.33482820568)*x[0]+(-0.12721791236)*x[1]**o+(-0.505873171195)*x[1]
        arg[0,2,0,1]=(0.224805131522)*x[0]**o+(0.332022434603)*x[0]+(0.0583494507268)*x[1]**o+(0.857185410994)*x[1]
        arg[0,2,1,0]=(-0.425367077516)*x[0]**o+(-0.25207101061)*x[0]+(-0.11164491753)*x[1]**o+(0.44752084285)*x[1]
        arg[0,2,1,1]=(-0.158725210751)*x[0]**o+(-0.0692363138932)*x[0]+(0.214803128256)*x[1]**o+(0.886699106748)*x[1]
        arg[0,2,2,0]=(-0.332146331257)*x[0]**o+(0.114845050128)*x[0]+(-0.0781411611124)*x[1]**o+(0.474972057387)*x[1]
        arg[0,2,2,1]=(-0.481949630694)*x[0]**o+(0.644984972151)*x[0]+(0.171146519527)*x[1]**o+(0.57534803966)*x[1]
        arg[0,3,0,0]=(0.7580633024)*x[0]**o+(0.0280511776577)*x[0]+(0.375583652078)*x[1]**o+(-0.809112787284)*x[1]
        arg[0,3,0,1]=(-0.466737159934)*x[0]**o+(0.410032775686)*x[0]+(0.176141189584)*x[1]**o+(-0.228288309456)*x[1]
        arg[0,3,1,0]=(0.427129582196)*x[0]**o+(-0.718385859483)*x[0]+(-0.172261212989)*x[1]**o+(0.913157476188)*x[1]
        arg[0,3,1,1]=(0.952506706487)*x[0]**o+(-0.497478163899)*x[0]+(-0.930273038716)*x[1]**o+(0.673573149025)*x[1]
        arg[0,3,2,0]=(-0.0074302930747)*x[0]**o+(0.226424842035)*x[0]+(0.871190620671)*x[1]**o+(0.277695333679)*x[1]
        arg[0,3,2,1]=(-0.540820239946)*x[0]**o+(0.33044763211)*x[0]+(-0.772648820924)*x[1]**o+(-0.362410088021)*x[1]
        arg[0,4,0,0]=(-0.748396039111)*x[0]**o+(0.215131518212)*x[0]+(0.514087824403)*x[1]**o+(0.335563854205)*x[1]
        arg[0,4,0,1]=(-0.443263590391)*x[0]**o+(-0.523167282542)*x[0]+(0.122612452799)*x[1]**o+(0.216527857615)*x[1]
        arg[0,4,1,0]=(0.448176296777)*x[0]**o+(0.478292146131)*x[0]+(-0.558921083502)*x[1]**o+(-0.17018258654)*x[1]
        arg[0,4,1,1]=(-0.912765673953)*x[0]**o+(0.751746623223)*x[0]+(-0.104811579057)*x[1]**o+(0.456436870323)*x[1]
        arg[0,4,2,0]=(0.157946450932)*x[0]**o+(0.147858587857)*x[0]+(0.775747362874)*x[1]**o+(0.478478862463)*x[1]
        arg[0,4,2,1]=(-0.819625058185)*x[0]**o+(-0.0231980393919)*x[0]+(-0.186647585522)*x[1]**o+(0.513350371371)*x[1]
        arg[1,0,0,0]=(-0.942373946995)*x[0]**o+(-0.761326160931)*x[0]+(0.192511867167)*x[1]**o+(-0.50895969579)*x[1]
        arg[1,0,0,1]=(0.0670813349951)*x[0]**o+(0.55602953512)*x[0]+(0.757621178635)*x[1]**o+(0.741550201759)*x[1]
        arg[1,0,1,0]=(-0.114642609493)*x[0]**o+(-0.639500048447)*x[0]+(-0.861813039243)*x[1]**o+(0.36413406744)*x[1]
        arg[1,0,1,1]=(0.126555451594)*x[0]**o+(-0.885172557082)*x[0]+(-0.376073252921)*x[1]**o+(0.166102850653)*x[1]
        arg[1,0,2,0]=(-0.381517068219)*x[0]**o+(0.94870328267)*x[0]+(-0.129934777755)*x[1]**o+(0.78090715758)*x[1]
        arg[1,0,2,1]=(-0.535852302247)*x[0]**o+(0.63751039318)*x[0]+(-0.537556159925)*x[1]**o+(0.147766714221)*x[1]
        arg[1,1,0,0]=(-0.605543047533)*x[0]**o+(-0.366612785372)*x[0]+(0.544900149212)*x[1]**o+(0.722108900447)*x[1]
        arg[1,1,0,1]=(0.133378561142)*x[0]**o+(-0.227808772494)*x[0]+(-0.729399624175)*x[1]**o+(0.307571743455)*x[1]
        arg[1,1,1,0]=(0.0458830538139)*x[0]**o+(0.0585044493026)*x[0]+(-0.136630547605)*x[1]**o+(-0.0823288389309)*x[1]
        arg[1,1,1,1]=(0.339263808784)*x[0]**o+(0.00988917109076)*x[0]+(-0.786591838083)*x[1]**o+(-0.361686799179)*x[1]
        arg[1,1,2,0]=(0.789942037592)*x[0]**o+(-0.266689758676)*x[0]+(0.464856327495)*x[1]**o+(0.138022774293)*x[1]
        arg[1,1,2,1]=(-0.723544867217)*x[0]**o+(-0.523023674205)*x[0]+(0.629726026331)*x[1]**o+(-0.867808120997)*x[1]
        arg[1,2,0,0]=(-0.495589518787)*x[0]**o+(-0.561664880262)*x[0]+(0.0319320417078)*x[1]**o+(-0.153720661498)*x[1]
        arg[1,2,0,1]=(0.293168114024)*x[0]**o+(-0.344107163981)*x[0]+(-0.275629670068)*x[1]**o+(0.06103653487)*x[1]
        arg[1,2,1,0]=(0.495640927339)*x[0]**o+(0.596540095043)*x[0]+(0.208792567756)*x[1]**o+(0.886040724145)*x[1]
        arg[1,2,1,1]=(0.331476398093)*x[0]**o+(-0.472588572455)*x[0]+(-0.455670457285)*x[1]**o+(-0.731223615642)*x[1]
        arg[1,2,2,0]=(-0.127107829674)*x[0]**o+(-0.21113128456)*x[0]+(-0.733928194168)*x[1]**o+(-0.174127696853)*x[1]
        arg[1,2,2,1]=(0.521879316151)*x[0]**o+(0.983424557865)*x[0]+(-0.288659660182)*x[1]**o+(-0.431045203113)*x[1]
        arg[1,3,0,0]=(0.292348765746)*x[0]**o+(0.0730741240915)*x[0]+(0.079624022683)*x[1]**o+(0.237483071633)*x[1]
        arg[1,3,0,1]=(0.291035771848)*x[0]**o+(-0.482909477855)*x[0]+(-0.999142129615)*x[1]**o+(-0.559723367197)*x[1]
        arg[1,3,1,0]=(0.747096275653)*x[0]**o+(0.762757649624)*x[0]+(-0.821650889507)*x[1]**o+(0.301128670189)*x[1]
        arg[1,3,1,1]=(0.82703681414)*x[0]**o+(-0.336570070436)*x[0]+(0.798131952979)*x[1]**o+(0.603562737956)*x[1]
        arg[1,3,2,0]=(-0.164875914126)*x[0]**o+(0.219409873902)*x[0]+(0.784253160678)*x[1]**o+(0.0219597460707)*x[1]
        arg[1,3,2,1]=(0.124778899794)*x[0]**o+(0.785367842145)*x[0]+(0.948111125908)*x[1]**o+(-0.887427847089)*x[1]
        arg[1,4,0,0]=(0.0911816002883)*x[0]**o+(0.645410022441)*x[0]+(0.720551531965)*x[1]**o+(-0.388407683413)*x[1]
        arg[1,4,0,1]=(-0.0416762613062)*x[0]**o+(-0.548874829834)*x[0]+(-0.377952555557)*x[1]**o+(0.742795234361)*x[1]
        arg[1,4,1,0]=(-0.896929017971)*x[0]**o+(0.678343686516)*x[0]+(0.711572763082)*x[1]**o+(-0.43397331651)*x[1]
        arg[1,4,1,1]=(-0.190804421566)*x[0]**o+(-0.0848405124789)*x[0]+(0.36246957905)*x[1]**o+(0.963599086227)*x[1]
        arg[1,4,2,0]=(-0.0210248567982)*x[0]**o+(0.545977814581)*x[0]+(0.858962864401)*x[1]**o+(0.113792851997)*x[1]
        arg[1,4,2,1]=(0.229641892622)*x[0]**o+(-0.739561617198)*x[0]+(-0.164372740617)*x[1]**o+(0.537097868843)*x[1]
        arg[2,0,0,0]=(-0.13321389343)*x[0]**o+(-0.668403350503)*x[0]+(-0.924879105983)*x[1]**o+(-0.5161790717)*x[1]
        arg[2,0,0,1]=(0.26582844685)*x[0]**o+(0.89048660684)*x[0]+(-0.377225981719)*x[1]**o+(-0.303790710194)*x[1]
        arg[2,0,1,0]=(0.0997470860283)*x[0]**o+(0.0385427264217)*x[0]+(-0.71387347273)*x[1]**o+(-0.621376499192)*x[1]
        arg[2,0,1,1]=(0.595544616326)*x[0]**o+(-0.850529503975)*x[0]+(0.602203957938)*x[1]**o+(-0.329484843907)*x[1]
        arg[2,0,2,0]=(-0.106031967029)*x[0]**o+(-0.165122821622)*x[0]+(0.647473815919)*x[1]**o+(0.28942226062)*x[1]
        arg[2,0,2,1]=(0.585955875472)*x[0]**o+(0.922667710403)*x[0]+(-0.0710905957841)*x[1]**o+(-0.669138163697)*x[1]
        arg[2,1,0,0]=(0.461941211081)*x[0]**o+(0.723022466263)*x[0]+(-0.118551893472)*x[1]**o+(0.00445187353637)*x[1]
        arg[2,1,0,1]=(-0.253168248108)*x[0]**o+(0.476938842605)*x[0]+(0.476641658034)*x[1]**o+(-0.916886387595)*x[1]
        arg[2,1,1,0]=(0.722029309503)*x[0]**o+(0.0588848593425)*x[0]+(0.84737370043)*x[1]**o+(-0.674604415213)*x[1]
        arg[2,1,1,1]=(-0.644736759298)*x[0]**o+(-0.212847303893)*x[0]+(0.25394587552)*x[1]**o+(-0.366955175081)*x[1]
        arg[2,1,2,0]=(-0.657395296801)*x[0]**o+(-0.967570027599)*x[0]+(0.180926628541)*x[1]**o+(-0.940599897942)*x[1]
        arg[2,1,2,1]=(0.849897328801)*x[0]**o+(-0.299199474051)*x[0]+(0.427221207827)*x[1]**o+(0.212027985441)*x[1]
        arg[2,2,0,0]=(-0.561573507381)*x[0]**o+(-0.446925224908)*x[0]+(0.132091224948)*x[1]**o+(-0.673375661849)*x[1]
        arg[2,2,0,1]=(-0.386657291574)*x[0]**o+(0.356784637112)*x[0]+(0.644292006233)*x[1]**o+(0.922937167842)*x[1]
        arg[2,2,1,0]=(0.484446757285)*x[0]**o+(-0.822469870455)*x[0]+(-0.89605890123)*x[1]**o+(-0.895891500031)*x[1]
        arg[2,2,1,1]=(-0.0930740520192)*x[0]**o+(0.953356473678)*x[0]+(-0.714418458806)*x[1]**o+(-0.100584950946)*x[1]
        arg[2,2,2,0]=(0.945572242667)*x[0]**o+(-0.259280849605)*x[0]+(-0.925459846341)*x[1]**o+(0.906002758873)*x[1]
        arg[2,2,2,1]=(-0.638993332975)*x[0]**o+(-0.668594641703)*x[0]+(0.012666231369)*x[1]**o+(0.89732685856)*x[1]
        arg[2,3,0,0]=(-0.885061116144)*x[0]**o+(-0.576097536483)*x[0]+(-0.43166075788)*x[1]**o+(0.538821620917)*x[1]
        arg[2,3,0,1]=(-0.0743638381797)*x[0]**o+(0.251037500035)*x[0]+(-0.905989928076)*x[1]**o+(0.243771000867)*x[1]
        arg[2,3,1,0]=(-0.016045622351)*x[0]**o+(-0.632944072174)*x[0]+(-0.296175918793)*x[1]**o+(-0.00120123641365)*x[1]
        arg[2,3,1,1]=(-0.896354921812)*x[0]**o+(-0.331348989027)*x[0]+(-0.450865285201)*x[1]**o+(-0.243268582009)*x[1]
        arg[2,3,2,0]=(0.609332927431)*x[0]**o+(-0.0815856802115)*x[0]+(0.201924792273)*x[1]**o+(-0.890424451348)*x[1]
        arg[2,3,2,1]=(0.395090137264)*x[0]**o+(0.294230442573)*x[0]+(-0.345371510364)*x[1]**o+(0.181229663241)*x[1]
        arg[2,4,0,0]=(-0.823684509034)*x[0]**o+(0.431019301942)*x[0]+(0.554707015054)*x[1]**o+(-0.846322903003)*x[1]
        arg[2,4,0,1]=(0.862984476658)*x[0]**o+(0.20512500102)*x[0]+(0.0247756159287)*x[1]**o+(-0.374795100946)*x[1]
        arg[2,4,1,0]=(-0.0849914269517)*x[0]**o+(0.754908200339)*x[0]+(0.593363754807)*x[1]**o+(0.124923961126)*x[1]
        arg[2,4,1,1]=(-0.691440264234)*x[0]**o+(0.825739298015)*x[0]+(0.426452231478)*x[1]**o+(0.0995610721341)*x[1]
        arg[2,4,2,0]=(-0.175322750567)*x[0]**o+(0.419750865569)*x[0]+(0.208448987712)*x[1]**o+(0.150584798952)*x[1]
        arg[2,4,2,1]=(0.931680152344)*x[0]**o+(0.390947206307)*x[0]+(0.388437348378)*x[1]**o+(-0.808413079032)*x[1]
        arg[3,0,0,0]=(0.58855431402)*x[0]**o+(0.383865913364)*x[0]+(-0.691445323219)*x[1]**o+(-0.443370658523)*x[1]
        arg[3,0,0,1]=(-0.657371481478)*x[0]**o+(0.746891082673)*x[0]+(-0.829229947773)*x[1]**o+(0.761915670404)*x[1]
        arg[3,0,1,0]=(-0.431502932714)*x[0]**o+(-0.961472285716)*x[0]+(0.0423384388031)*x[1]**o+(0.339373458118)*x[1]
        arg[3,0,1,1]=(-0.272831163259)*x[0]**o+(-0.533438904494)*x[0]+(-0.595625384875)*x[1]**o+(0.0366857022323)*x[1]
        arg[3,0,2,0]=(-0.319684662306)*x[0]**o+(0.672185828912)*x[0]+(0.483126548994)*x[1]**o+(0.745872629604)*x[1]
        arg[3,0,2,1]=(0.437895277404)*x[0]**o+(-0.337209416963)*x[0]+(0.101080522788)*x[1]**o+(0.892022488644)*x[1]
        arg[3,1,0,0]=(-0.582325078455)*x[0]**o+(-0.854689347157)*x[0]+(0.0308638312259)*x[1]**o+(0.633382365704)*x[1]
        arg[3,1,0,1]=(0.319735178376)*x[0]**o+(-0.782764593952)*x[0]+(0.082230132244)*x[1]**o+(-0.946010937852)*x[1]
        arg[3,1,1,0]=(-0.698490410666)*x[0]**o+(-0.801358802689)*x[0]+(0.20695648235)*x[1]**o+(0.75257523471)*x[1]
        arg[3,1,1,1]=(0.66321696527)*x[0]**o+(0.704572918868)*x[0]+(0.528208879002)*x[1]**o+(-0.408378325513)*x[1]
        arg[3,1,2,0]=(-0.925177198417)*x[0]**o+(-0.152427811549)*x[0]+(0.341781993583)*x[1]**o+(-0.589976456377)*x[1]
        arg[3,1,2,1]=(0.206706628353)*x[0]**o+(0.673777173329)*x[0]+(-0.994138314254)*x[1]**o+(-0.843696565863)*x[1]
        arg[3,2,0,0]=(-0.588881067333)*x[0]**o+(-0.537572388341)*x[0]+(-0.206570371675)*x[1]**o+(-0.127468554294)*x[1]
        arg[3,2,0,1]=(0.498638284096)*x[0]**o+(-0.488278831086)*x[0]+(-0.28320874594)*x[1]**o+(-0.638241576615)*x[1]
        arg[3,2,1,0]=(-0.445780790822)*x[0]**o+(-0.347493953089)*x[0]+(-0.558010086232)*x[1]**o+(-0.0287339307689)*x[1]
        arg[3,2,1,1]=(-0.361727811272)*x[0]**o+(-0.559565378007)*x[0]+(-0.211685558972)*x[1]**o+(0.330088396811)*x[1]
        arg[3,2,2,0]=(-0.42600117831)*x[0]**o+(-0.488997111988)*x[0]+(0.127421278051)*x[1]**o+(-0.48233167806)*x[1]
        arg[3,2,2,1]=(0.120468829686)*x[0]**o+(0.0675603906118)*x[0]+(-0.60674361864)*x[1]**o+(0.639115708258)*x[1]
        arg[3,3,0,0]=(0.369343033015)*x[0]**o+(0.707735743593)*x[0]+(-0.426275591836)*x[1]**o+(-0.733377765625)*x[1]
        arg[3,3,0,1]=(-0.931245338156)*x[0]**o+(-0.465861794185)*x[0]+(-0.650664500687)*x[1]**o+(0.103077662402)*x[1]
        arg[3,3,1,0]=(0.731834744512)*x[0]**o+(0.267840571071)*x[0]+(0.952106457495)*x[1]**o+(-0.943845364366)*x[1]
        arg[3,3,1,1]=(-0.21692720916)*x[0]**o+(0.580841374997)*x[0]+(-0.412129341223)*x[1]**o+(0.40414445304)*x[1]
        arg[3,3,2,0]=(-0.447758598726)*x[0]**o+(0.0616926352912)*x[0]+(-0.824624226122)*x[1]**o+(0.19099833104)*x[1]
        arg[3,3,2,1]=(-0.615519138165)*x[0]**o+(0.233926822164)*x[0]+(-0.969014060236)*x[1]**o+(0.835103627893)*x[1]
        arg[3,4,0,0]=(0.550778983301)*x[0]**o+(0.0198253530882)*x[0]+(-0.39383849187)*x[1]**o+(0.367721826304)*x[1]
        arg[3,4,0,1]=(0.444877923149)*x[0]**o+(-0.899159835223)*x[0]+(0.923392540452)*x[1]**o+(0.512905141718)*x[1]
        arg[3,4,1,0]=(-0.563807675145)*x[0]**o+(0.994405383899)*x[0]+(-0.777352948218)*x[1]**o+(0.865994906227)*x[1]
        arg[3,4,1,1]=(-0.593800452139)*x[0]**o+(0.613778803237)*x[0]+(0.582200979718)*x[1]**o+(0.448026243271)*x[1]
        arg[3,4,2,0]=(-0.900331125416)*x[0]**o+(0.234613965424)*x[0]+(0.510400273805)*x[1]**o+(0.0836773641661)*x[1]
        arg[3,4,2,1]=(0.710999097307)*x[0]**o+(0.886603988265)*x[0]+(-0.405644887679)*x[1]**o+(0.362257258809)*x[1]
        ref[0,0,0,0]=(-0.0338299192888)*x_ref[0]**o+(-0.114906274359)*x_ref[0]+(0.15010339398)*x_ref[1]**o+(0.625295991328)*x_ref[1]
        ref[0,0,0,1]=(-0.1311901039)*x_ref[0]**o+(-0.551712861217)*x_ref[0]+(-0.287411404506)*x_ref[1]**o+(0.0242820629572)*x_ref[1]
        ref[0,0,1,0]=(0.121962094111)*x_ref[0]**o+(0.203169622345)*x_ref[0]+(0.241504218053)*x_ref[1]**o+(-0.0696356239498)*x_ref[1]
        ref[0,0,1,1]=(0.0461537352329)*x_ref[0]**o+(-0.275908938567)*x_ref[0]+(-0.271184121027)*x_ref[1]**o+(-0.364024729567)*x_ref[1]
        ref[0,0,2,0]=(0.199272234315)*x_ref[0]**o+(-0.906364258855)*x_ref[0]+(-0.92102930037)*x_ref[1]**o+(0.0718981903454)*x_ref[1]
        ref[0,0,2,1]=(-0.742950958383)*x_ref[0]**o+(0.0301464847722)*x_ref[0]+(0.466993477687)*x_ref[1]**o+(-0.402131750698)*x_ref[1]
        ref[0,1,0,0]=(-0.0653776546375)*x_ref[0]**o+(-0.679360612372)*x_ref[0]+(-0.32783837768)*x_ref[1]**o+(-0.113753016428)*x_ref[1]
        ref[0,1,0,1]=(0.58610119414)*x_ref[0]**o+(0.142613166231)*x_ref[0]+(0.0409105619986)*x_ref[1]**o+(-0.660641663558)*x_ref[1]
        ref[0,1,1,0]=(0.212172308585)*x_ref[0]**o+(-0.452767768344)*x_ref[0]+(0.963053969889)*x_ref[1]**o+(0.752691773953)*x_ref[1]
        ref[0,1,1,1]=(-0.514998377887)*x_ref[0]**o+(0.0872464329801)*x_ref[0]+(0.265590295505)*x_ref[1]**o+(0.557238826549)*x_ref[1]
        ref[0,1,2,0]=(0.360523954724)*x_ref[0]**o+(0.550238402994)*x_ref[0]+(-0.206349415539)*x_ref[1]**o+(0.357304730511)*x_ref[1]
        ref[0,1,2,1]=(-0.872755217435)*x_ref[0]**o+(0.190943683664)*x_ref[0]+(-0.245454292354)*x_ref[1]**o+(0.507229578506)*x_ref[1]
        ref[0,2,0,0]=(0.994619505257)*x_ref[0]**o+(0.33482820568)*x_ref[0]+(-0.12721791236)*x_ref[1]**o+(-0.505873171195)*x_ref[1]
        ref[0,2,0,1]=(0.224805131522)*x_ref[0]**o+(0.332022434603)*x_ref[0]+(0.0583494507268)*x_ref[1]**o+(0.857185410994)*x_ref[1]
        ref[0,2,1,0]=(-0.425367077516)*x_ref[0]**o+(-0.25207101061)*x_ref[0]+(-0.11164491753)*x_ref[1]**o+(0.44752084285)*x_ref[1]
        ref[0,2,1,1]=(-0.158725210751)*x_ref[0]**o+(-0.0692363138932)*x_ref[0]+(0.214803128256)*x_ref[1]**o+(0.886699106748)*x_ref[1]
        ref[0,2,2,0]=(-0.332146331257)*x_ref[0]**o+(0.114845050128)*x_ref[0]+(-0.0781411611124)*x_ref[1]**o+(0.474972057387)*x_ref[1]
        ref[0,2,2,1]=(-0.481949630694)*x_ref[0]**o+(0.644984972151)*x_ref[0]+(0.171146519527)*x_ref[1]**o+(0.57534803966)*x_ref[1]
        ref[0,3,0,0]=(0.7580633024)*x_ref[0]**o+(0.0280511776577)*x_ref[0]+(0.375583652078)*x_ref[1]**o+(-0.809112787284)*x_ref[1]
        ref[0,3,0,1]=(-0.466737159934)*x_ref[0]**o+(0.410032775686)*x_ref[0]+(0.176141189584)*x_ref[1]**o+(-0.228288309456)*x_ref[1]
        ref[0,3,1,0]=(0.427129582196)*x_ref[0]**o+(-0.718385859483)*x_ref[0]+(-0.172261212989)*x_ref[1]**o+(0.913157476188)*x_ref[1]
        ref[0,3,1,1]=(0.952506706487)*x_ref[0]**o+(-0.497478163899)*x_ref[0]+(-0.930273038716)*x_ref[1]**o+(0.673573149025)*x_ref[1]
        ref[0,3,2,0]=(-0.0074302930747)*x_ref[0]**o+(0.226424842035)*x_ref[0]+(0.871190620671)*x_ref[1]**o+(0.277695333679)*x_ref[1]
        ref[0,3,2,1]=(-0.540820239946)*x_ref[0]**o+(0.33044763211)*x_ref[0]+(-0.772648820924)*x_ref[1]**o+(-0.362410088021)*x_ref[1]
        ref[0,4,0,0]=(-0.748396039111)*x_ref[0]**o+(0.215131518212)*x_ref[0]+(0.514087824403)*x_ref[1]**o+(0.335563854205)*x_ref[1]
        ref[0,4,0,1]=(-0.443263590391)*x_ref[0]**o+(-0.523167282542)*x_ref[0]+(0.122612452799)*x_ref[1]**o+(0.216527857615)*x_ref[1]
        ref[0,4,1,0]=(0.448176296777)*x_ref[0]**o+(0.478292146131)*x_ref[0]+(-0.558921083502)*x_ref[1]**o+(-0.17018258654)*x_ref[1]
        ref[0,4,1,1]=(-0.912765673953)*x_ref[0]**o+(0.751746623223)*x_ref[0]+(-0.104811579057)*x_ref[1]**o+(0.456436870323)*x_ref[1]
        ref[0,4,2,0]=(0.157946450932)*x_ref[0]**o+(0.147858587857)*x_ref[0]+(0.775747362874)*x_ref[1]**o+(0.478478862463)*x_ref[1]
        ref[0,4,2,1]=(-0.819625058185)*x_ref[0]**o+(-0.0231980393919)*x_ref[0]+(-0.186647585522)*x_ref[1]**o+(0.513350371371)*x_ref[1]
        ref[1,0,0,0]=(-0.942373946995)*x_ref[0]**o+(-0.761326160931)*x_ref[0]+(0.192511867167)*x_ref[1]**o+(-0.50895969579)*x_ref[1]
        ref[1,0,0,1]=(0.0670813349951)*x_ref[0]**o+(0.55602953512)*x_ref[0]+(0.757621178635)*x_ref[1]**o+(0.741550201759)*x_ref[1]
        ref[1,0,1,0]=(-0.114642609493)*x_ref[0]**o+(-0.639500048447)*x_ref[0]+(-0.861813039243)*x_ref[1]**o+(0.36413406744)*x_ref[1]
        ref[1,0,1,1]=(0.126555451594)*x_ref[0]**o+(-0.885172557082)*x_ref[0]+(-0.376073252921)*x_ref[1]**o+(0.166102850653)*x_ref[1]
        ref[1,0,2,0]=(-0.381517068219)*x_ref[0]**o+(0.94870328267)*x_ref[0]+(-0.129934777755)*x_ref[1]**o+(0.78090715758)*x_ref[1]
        ref[1,0,2,1]=(-0.535852302247)*x_ref[0]**o+(0.63751039318)*x_ref[0]+(-0.537556159925)*x_ref[1]**o+(0.147766714221)*x_ref[1]
        ref[1,1,0,0]=(-0.605543047533)*x_ref[0]**o+(-0.366612785372)*x_ref[0]+(0.544900149212)*x_ref[1]**o+(0.722108900447)*x_ref[1]
        ref[1,1,0,1]=(0.133378561142)*x_ref[0]**o+(-0.227808772494)*x_ref[0]+(-0.729399624175)*x_ref[1]**o+(0.307571743455)*x_ref[1]
        ref[1,1,1,0]=(0.0458830538139)*x_ref[0]**o+(0.0585044493026)*x_ref[0]+(-0.136630547605)*x_ref[1]**o+(-0.0823288389309)*x_ref[1]
        ref[1,1,1,1]=(0.339263808784)*x_ref[0]**o+(0.00988917109076)*x_ref[0]+(-0.786591838083)*x_ref[1]**o+(-0.361686799179)*x_ref[1]
        ref[1,1,2,0]=(0.789942037592)*x_ref[0]**o+(-0.266689758676)*x_ref[0]+(0.464856327495)*x_ref[1]**o+(0.138022774293)*x_ref[1]
        ref[1,1,2,1]=(-0.723544867217)*x_ref[0]**o+(-0.523023674205)*x_ref[0]+(0.629726026331)*x_ref[1]**o+(-0.867808120997)*x_ref[1]
        ref[1,2,0,0]=(-0.495589518787)*x_ref[0]**o+(-0.561664880262)*x_ref[0]+(0.0319320417078)*x_ref[1]**o+(-0.153720661498)*x_ref[1]
        ref[1,2,0,1]=(0.293168114024)*x_ref[0]**o+(-0.344107163981)*x_ref[0]+(-0.275629670068)*x_ref[1]**o+(0.06103653487)*x_ref[1]
        ref[1,2,1,0]=(0.495640927339)*x_ref[0]**o+(0.596540095043)*x_ref[0]+(0.208792567756)*x_ref[1]**o+(0.886040724145)*x_ref[1]
        ref[1,2,1,1]=(0.331476398093)*x_ref[0]**o+(-0.472588572455)*x_ref[0]+(-0.455670457285)*x_ref[1]**o+(-0.731223615642)*x_ref[1]
        ref[1,2,2,0]=(-0.127107829674)*x_ref[0]**o+(-0.21113128456)*x_ref[0]+(-0.733928194168)*x_ref[1]**o+(-0.174127696853)*x_ref[1]
        ref[1,2,2,1]=(0.521879316151)*x_ref[0]**o+(0.983424557865)*x_ref[0]+(-0.288659660182)*x_ref[1]**o+(-0.431045203113)*x_ref[1]
        ref[1,3,0,0]=(0.292348765746)*x_ref[0]**o+(0.0730741240915)*x_ref[0]+(0.079624022683)*x_ref[1]**o+(0.237483071633)*x_ref[1]
        ref[1,3,0,1]=(0.291035771848)*x_ref[0]**o+(-0.482909477855)*x_ref[0]+(-0.999142129615)*x_ref[1]**o+(-0.559723367197)*x_ref[1]
        ref[1,3,1,0]=(0.747096275653)*x_ref[0]**o+(0.762757649624)*x_ref[0]+(-0.821650889507)*x_ref[1]**o+(0.301128670189)*x_ref[1]
        ref[1,3,1,1]=(0.82703681414)*x_ref[0]**o+(-0.336570070436)*x_ref[0]+(0.798131952979)*x_ref[1]**o+(0.603562737956)*x_ref[1]
        ref[1,3,2,0]=(-0.164875914126)*x_ref[0]**o+(0.219409873902)*x_ref[0]+(0.784253160678)*x_ref[1]**o+(0.0219597460707)*x_ref[1]
        ref[1,3,2,1]=(0.124778899794)*x_ref[0]**o+(0.785367842145)*x_ref[0]+(0.948111125908)*x_ref[1]**o+(-0.887427847089)*x_ref[1]
        ref[1,4,0,0]=(0.0911816002883)*x_ref[0]**o+(0.645410022441)*x_ref[0]+(0.720551531965)*x_ref[1]**o+(-0.388407683413)*x_ref[1]
        ref[1,4,0,1]=(-0.0416762613062)*x_ref[0]**o+(-0.548874829834)*x_ref[0]+(-0.377952555557)*x_ref[1]**o+(0.742795234361)*x_ref[1]
        ref[1,4,1,0]=(-0.896929017971)*x_ref[0]**o+(0.678343686516)*x_ref[0]+(0.711572763082)*x_ref[1]**o+(-0.43397331651)*x_ref[1]
        ref[1,4,1,1]=(-0.190804421566)*x_ref[0]**o+(-0.0848405124789)*x_ref[0]+(0.36246957905)*x_ref[1]**o+(0.963599086227)*x_ref[1]
        ref[1,4,2,0]=(-0.0210248567982)*x_ref[0]**o+(0.545977814581)*x_ref[0]+(0.858962864401)*x_ref[1]**o+(0.113792851997)*x_ref[1]
        ref[1,4,2,1]=(0.229641892622)*x_ref[0]**o+(-0.739561617198)*x_ref[0]+(-0.164372740617)*x_ref[1]**o+(0.537097868843)*x_ref[1]
        ref[2,0,0,0]=(-0.13321389343)*x_ref[0]**o+(-0.668403350503)*x_ref[0]+(-0.924879105983)*x_ref[1]**o+(-0.5161790717)*x_ref[1]
        ref[2,0,0,1]=(0.26582844685)*x_ref[0]**o+(0.89048660684)*x_ref[0]+(-0.377225981719)*x_ref[1]**o+(-0.303790710194)*x_ref[1]
        ref[2,0,1,0]=(0.0997470860283)*x_ref[0]**o+(0.0385427264217)*x_ref[0]+(-0.71387347273)*x_ref[1]**o+(-0.621376499192)*x_ref[1]
        ref[2,0,1,1]=(0.595544616326)*x_ref[0]**o+(-0.850529503975)*x_ref[0]+(0.602203957938)*x_ref[1]**o+(-0.329484843907)*x_ref[1]
        ref[2,0,2,0]=(-0.106031967029)*x_ref[0]**o+(-0.165122821622)*x_ref[0]+(0.647473815919)*x_ref[1]**o+(0.28942226062)*x_ref[1]
        ref[2,0,2,1]=(0.585955875472)*x_ref[0]**o+(0.922667710403)*x_ref[0]+(-0.0710905957841)*x_ref[1]**o+(-0.669138163697)*x_ref[1]
        ref[2,1,0,0]=(0.461941211081)*x_ref[0]**o+(0.723022466263)*x_ref[0]+(-0.118551893472)*x_ref[1]**o+(0.00445187353637)*x_ref[1]
        ref[2,1,0,1]=(-0.253168248108)*x_ref[0]**o+(0.476938842605)*x_ref[0]+(0.476641658034)*x_ref[1]**o+(-0.916886387595)*x_ref[1]
        ref[2,1,1,0]=(0.722029309503)*x_ref[0]**o+(0.0588848593425)*x_ref[0]+(0.84737370043)*x_ref[1]**o+(-0.674604415213)*x_ref[1]
        ref[2,1,1,1]=(-0.644736759298)*x_ref[0]**o+(-0.212847303893)*x_ref[0]+(0.25394587552)*x_ref[1]**o+(-0.366955175081)*x_ref[1]
        ref[2,1,2,0]=(-0.657395296801)*x_ref[0]**o+(-0.967570027599)*x_ref[0]+(0.180926628541)*x_ref[1]**o+(-0.940599897942)*x_ref[1]
        ref[2,1,2,1]=(0.849897328801)*x_ref[0]**o+(-0.299199474051)*x_ref[0]+(0.427221207827)*x_ref[1]**o+(0.212027985441)*x_ref[1]
        ref[2,2,0,0]=(-0.561573507381)*x_ref[0]**o+(-0.446925224908)*x_ref[0]+(0.132091224948)*x_ref[1]**o+(-0.673375661849)*x_ref[1]
        ref[2,2,0,1]=(-0.386657291574)*x_ref[0]**o+(0.356784637112)*x_ref[0]+(0.644292006233)*x_ref[1]**o+(0.922937167842)*x_ref[1]
        ref[2,2,1,0]=(0.484446757285)*x_ref[0]**o+(-0.822469870455)*x_ref[0]+(-0.89605890123)*x_ref[1]**o+(-0.895891500031)*x_ref[1]
        ref[2,2,1,1]=(-0.0930740520192)*x_ref[0]**o+(0.953356473678)*x_ref[0]+(-0.714418458806)*x_ref[1]**o+(-0.100584950946)*x_ref[1]
        ref[2,2,2,0]=(0.945572242667)*x_ref[0]**o+(-0.259280849605)*x_ref[0]+(-0.925459846341)*x_ref[1]**o+(0.906002758873)*x_ref[1]
        ref[2,2,2,1]=(-0.638993332975)*x_ref[0]**o+(-0.668594641703)*x_ref[0]+(0.012666231369)*x_ref[1]**o+(0.89732685856)*x_ref[1]
        ref[2,3,0,0]=(-0.885061116144)*x_ref[0]**o+(-0.576097536483)*x_ref[0]+(-0.43166075788)*x_ref[1]**o+(0.538821620917)*x_ref[1]
        ref[2,3,0,1]=(-0.0743638381797)*x_ref[0]**o+(0.251037500035)*x_ref[0]+(-0.905989928076)*x_ref[1]**o+(0.243771000867)*x_ref[1]
        ref[2,3,1,0]=(-0.016045622351)*x_ref[0]**o+(-0.632944072174)*x_ref[0]+(-0.296175918793)*x_ref[1]**o+(-0.00120123641365)*x_ref[1]
        ref[2,3,1,1]=(-0.896354921812)*x_ref[0]**o+(-0.331348989027)*x_ref[0]+(-0.450865285201)*x_ref[1]**o+(-0.243268582009)*x_ref[1]
        ref[2,3,2,0]=(0.609332927431)*x_ref[0]**o+(-0.0815856802115)*x_ref[0]+(0.201924792273)*x_ref[1]**o+(-0.890424451348)*x_ref[1]
        ref[2,3,2,1]=(0.395090137264)*x_ref[0]**o+(0.294230442573)*x_ref[0]+(-0.345371510364)*x_ref[1]**o+(0.181229663241)*x_ref[1]
        ref[2,4,0,0]=(-0.823684509034)*x_ref[0]**o+(0.431019301942)*x_ref[0]+(0.554707015054)*x_ref[1]**o+(-0.846322903003)*x_ref[1]
        ref[2,4,0,1]=(0.862984476658)*x_ref[0]**o+(0.20512500102)*x_ref[0]+(0.0247756159287)*x_ref[1]**o+(-0.374795100946)*x_ref[1]
        ref[2,4,1,0]=(-0.0849914269517)*x_ref[0]**o+(0.754908200339)*x_ref[0]+(0.593363754807)*x_ref[1]**o+(0.124923961126)*x_ref[1]
        ref[2,4,1,1]=(-0.691440264234)*x_ref[0]**o+(0.825739298015)*x_ref[0]+(0.426452231478)*x_ref[1]**o+(0.0995610721341)*x_ref[1]
        ref[2,4,2,0]=(-0.175322750567)*x_ref[0]**o+(0.419750865569)*x_ref[0]+(0.208448987712)*x_ref[1]**o+(0.150584798952)*x_ref[1]
        ref[2,4,2,1]=(0.931680152344)*x_ref[0]**o+(0.390947206307)*x_ref[0]+(0.388437348378)*x_ref[1]**o+(-0.808413079032)*x_ref[1]
        ref[3,0,0,0]=(0.58855431402)*x_ref[0]**o+(0.383865913364)*x_ref[0]+(-0.691445323219)*x_ref[1]**o+(-0.443370658523)*x_ref[1]
        ref[3,0,0,1]=(-0.657371481478)*x_ref[0]**o+(0.746891082673)*x_ref[0]+(-0.829229947773)*x_ref[1]**o+(0.761915670404)*x_ref[1]
        ref[3,0,1,0]=(-0.431502932714)*x_ref[0]**o+(-0.961472285716)*x_ref[0]+(0.0423384388031)*x_ref[1]**o+(0.339373458118)*x_ref[1]
        ref[3,0,1,1]=(-0.272831163259)*x_ref[0]**o+(-0.533438904494)*x_ref[0]+(-0.595625384875)*x_ref[1]**o+(0.0366857022323)*x_ref[1]
        ref[3,0,2,0]=(-0.319684662306)*x_ref[0]**o+(0.672185828912)*x_ref[0]+(0.483126548994)*x_ref[1]**o+(0.745872629604)*x_ref[1]
        ref[3,0,2,1]=(0.437895277404)*x_ref[0]**o+(-0.337209416963)*x_ref[0]+(0.101080522788)*x_ref[1]**o+(0.892022488644)*x_ref[1]
        ref[3,1,0,0]=(-0.582325078455)*x_ref[0]**o+(-0.854689347157)*x_ref[0]+(0.0308638312259)*x_ref[1]**o+(0.633382365704)*x_ref[1]
        ref[3,1,0,1]=(0.319735178376)*x_ref[0]**o+(-0.782764593952)*x_ref[0]+(0.082230132244)*x_ref[1]**o+(-0.946010937852)*x_ref[1]
        ref[3,1,1,0]=(-0.698490410666)*x_ref[0]**o+(-0.801358802689)*x_ref[0]+(0.20695648235)*x_ref[1]**o+(0.75257523471)*x_ref[1]
        ref[3,1,1,1]=(0.66321696527)*x_ref[0]**o+(0.704572918868)*x_ref[0]+(0.528208879002)*x_ref[1]**o+(-0.408378325513)*x_ref[1]
        ref[3,1,2,0]=(-0.925177198417)*x_ref[0]**o+(-0.152427811549)*x_ref[0]+(0.341781993583)*x_ref[1]**o+(-0.589976456377)*x_ref[1]
        ref[3,1,2,1]=(0.206706628353)*x_ref[0]**o+(0.673777173329)*x_ref[0]+(-0.994138314254)*x_ref[1]**o+(-0.843696565863)*x_ref[1]
        ref[3,2,0,0]=(-0.588881067333)*x_ref[0]**o+(-0.537572388341)*x_ref[0]+(-0.206570371675)*x_ref[1]**o+(-0.127468554294)*x_ref[1]
        ref[3,2,0,1]=(0.498638284096)*x_ref[0]**o+(-0.488278831086)*x_ref[0]+(-0.28320874594)*x_ref[1]**o+(-0.638241576615)*x_ref[1]
        ref[3,2,1,0]=(-0.445780790822)*x_ref[0]**o+(-0.347493953089)*x_ref[0]+(-0.558010086232)*x_ref[1]**o+(-0.0287339307689)*x_ref[1]
        ref[3,2,1,1]=(-0.361727811272)*x_ref[0]**o+(-0.559565378007)*x_ref[0]+(-0.211685558972)*x_ref[1]**o+(0.330088396811)*x_ref[1]
        ref[3,2,2,0]=(-0.42600117831)*x_ref[0]**o+(-0.488997111988)*x_ref[0]+(0.127421278051)*x_ref[1]**o+(-0.48233167806)*x_ref[1]
        ref[3,2,2,1]=(0.120468829686)*x_ref[0]**o+(0.0675603906118)*x_ref[0]+(-0.60674361864)*x_ref[1]**o+(0.639115708258)*x_ref[1]
        ref[3,3,0,0]=(0.369343033015)*x_ref[0]**o+(0.707735743593)*x_ref[0]+(-0.426275591836)*x_ref[1]**o+(-0.733377765625)*x_ref[1]
        ref[3,3,0,1]=(-0.931245338156)*x_ref[0]**o+(-0.465861794185)*x_ref[0]+(-0.650664500687)*x_ref[1]**o+(0.103077662402)*x_ref[1]
        ref[3,3,1,0]=(0.731834744512)*x_ref[0]**o+(0.267840571071)*x_ref[0]+(0.952106457495)*x_ref[1]**o+(-0.943845364366)*x_ref[1]
        ref[3,3,1,1]=(-0.21692720916)*x_ref[0]**o+(0.580841374997)*x_ref[0]+(-0.412129341223)*x_ref[1]**o+(0.40414445304)*x_ref[1]
        ref[3,3,2,0]=(-0.447758598726)*x_ref[0]**o+(0.0616926352912)*x_ref[0]+(-0.824624226122)*x_ref[1]**o+(0.19099833104)*x_ref[1]
        ref[3,3,2,1]=(-0.615519138165)*x_ref[0]**o+(0.233926822164)*x_ref[0]+(-0.969014060236)*x_ref[1]**o+(0.835103627893)*x_ref[1]
        ref[3,4,0,0]=(0.550778983301)*x_ref[0]**o+(0.0198253530882)*x_ref[0]+(-0.39383849187)*x_ref[1]**o+(0.367721826304)*x_ref[1]
        ref[3,4,0,1]=(0.444877923149)*x_ref[0]**o+(-0.899159835223)*x_ref[0]+(0.923392540452)*x_ref[1]**o+(0.512905141718)*x_ref[1]
        ref[3,4,1,0]=(-0.563807675145)*x_ref[0]**o+(0.994405383899)*x_ref[0]+(-0.777352948218)*x_ref[1]**o+(0.865994906227)*x_ref[1]
        ref[3,4,1,1]=(-0.593800452139)*x_ref[0]**o+(0.613778803237)*x_ref[0]+(0.582200979718)*x_ref[1]**o+(0.448026243271)*x_ref[1]
        ref[3,4,2,0]=(-0.900331125416)*x_ref[0]**o+(0.234613965424)*x_ref[0]+(0.510400273805)*x_ref[1]**o+(0.0836773641661)*x_ref[1]
        ref[3,4,2,1]=(0.710999097307)*x_ref[0]**o+(0.886603988265)*x_ref[0]+(-0.405644887679)*x_ref[1]**o+(0.362257258809)*x_ref[1]
      else:
        arg[0,0,0,0]=(0.310272924207)*x[0]**o+(-0.664497806351)*x[0]+(0.741942132609)*x[1]**o+(0.636051298779)*x[1]+(0.76627159146)*x[2]**o+(0.492960732588)*x[2]
        arg[0,0,0,1]=(-0.513740801773)*x[0]**o+(-0.633330750909)*x[0]+(0.177934618983)*x[1]**o+(-0.323745723166)*x[1]+(0.694167180904)*x[2]**o+(0.441239251435)*x[2]
        arg[0,0,1,0]=(0.417477695848)*x[0]**o+(-0.923415976002)*x[0]+(0.0765555276662)*x[1]**o+(-0.277290890834)*x[1]+(0.0177191326614)*x[2]**o+(-0.0842520100387)*x[2]
        arg[0,0,1,1]=(-0.761916249894)*x[0]**o+(0.169673139552)*x[0]+(0.0306290828795)*x[1]**o+(0.842234592613)*x[1]+(-0.350342692669)*x[2]**o+(0.321292270658)*x[2]
        arg[0,0,2,0]=(-0.707494137624)*x[0]**o+(0.346873490395)*x[0]+(-0.68281925138)*x[1]**o+(-0.830538083274)*x[1]+(0.329372141613)*x[2]**o+(-0.121366157163)*x[2]
        arg[0,0,2,1]=(-0.708018186507)*x[0]**o+(-0.778051239314)*x[0]+(-0.43106291563)*x[1]**o+(-0.0115914251143)*x[1]+(0.487334215892)*x[2]**o+(-0.289232284658)*x[2]
        arg[0,1,0,0]=(0.530614704199)*x[0]**o+(0.518622028317)*x[0]+(-0.0143129589551)*x[1]**o+(0.808469352257)*x[1]+(-0.156787373225)*x[2]**o+(0.668887813143)*x[2]
        arg[0,1,0,1]=(0.953223233706)*x[0]**o+(-0.164323838353)*x[0]+(0.336717320858)*x[1]**o+(-0.520230488172)*x[1]+(0.689960738002)*x[2]**o+(0.487630947578)*x[2]
        arg[0,1,1,0]=(-0.418095373429)*x[0]**o+(-0.941817844939)*x[0]+(0.88105115607)*x[1]**o+(-0.820502501439)*x[1]+(-0.693858267667)*x[2]**o+(0.530567713127)*x[2]
        arg[0,1,1,1]=(-0.716861320218)*x[0]**o+(-0.736619859467)*x[0]+(-0.885478605601)*x[1]**o+(-0.129375578967)*x[1]+(-0.0181433907318)*x[2]**o+(0.373255537959)*x[2]
        arg[0,1,2,0]=(-0.174742583167)*x[0]**o+(-0.0887967480112)*x[0]+(-0.291168698662)*x[1]**o+(-0.188608506769)*x[1]+(0.22479613454)*x[2]**o+(0.34173928582)*x[2]
        arg[0,1,2,1]=(0.117208929504)*x[0]**o+(-0.420969282079)*x[0]+(0.585298109836)*x[1]**o+(0.771355162745)*x[1]+(-0.51374493747)*x[2]**o+(0.785016548855)*x[2]
        arg[0,2,0,0]=(0.737394887886)*x[0]**o+(0.966080271342)*x[0]+(-0.373178384492)*x[1]**o+(-0.845505386142)*x[1]+(-0.274654519371)*x[2]**o+(0.709356557887)*x[2]
        arg[0,2,0,1]=(0.0365864090636)*x[0]**o+(-0.393382479787)*x[0]+(0.018273360977)*x[1]**o+(-0.509201909328)*x[1]+(-0.777852392534)*x[2]**o+(0.0417280247507)*x[2]
        arg[0,2,1,0]=(0.637829302751)*x[0]**o+(-0.767824790186)*x[0]+(0.843382009511)*x[1]**o+(-0.599633234403)*x[1]+(0.14283138941)*x[2]**o+(-0.354404633991)*x[2]
        arg[0,2,1,1]=(-0.465258613599)*x[0]**o+(0.09341154383)*x[0]+(-0.629114782988)*x[1]**o+(0.91608655383)*x[1]+(-0.692901135848)*x[2]**o+(-0.707225310667)*x[2]
        arg[0,2,2,0]=(0.9935958759)*x[0]**o+(-0.385008742466)*x[0]+(0.153885356854)*x[1]**o+(0.146270680656)*x[1]+(0.305838410343)*x[2]**o+(0.141193552717)*x[2]
        arg[0,2,2,1]=(-0.487538023894)*x[0]**o+(-0.814283442647)*x[0]+(0.166008520199)*x[1]**o+(0.313655774416)*x[1]+(0.282308782973)*x[2]**o+(0.309176062484)*x[2]
        arg[0,3,0,0]=(0.00203507798913)*x[0]**o+(-0.19902542604)*x[0]+(0.88110918624)*x[1]**o+(0.223549219288)*x[1]+(0.135758952369)*x[2]**o+(0.59116783639)*x[2]
        arg[0,3,0,1]=(-0.185063700767)*x[0]**o+(0.833862155173)*x[0]+(-0.0939117509645)*x[1]**o+(0.288003692231)*x[1]+(0.580196157839)*x[2]**o+(0.299320177754)*x[2]
        arg[0,3,1,0]=(-0.039100718826)*x[0]**o+(0.423179869395)*x[0]+(-0.013669578975)*x[1]**o+(-0.355053343085)*x[1]+(0.89285158975)*x[2]**o+(0.392984367877)*x[2]
        arg[0,3,1,1]=(0.0988989416429)*x[0]**o+(-0.446243235174)*x[0]+(-0.691253313617)*x[1]**o+(0.338823693086)*x[1]+(0.707054126334)*x[2]**o+(0.601121388278)*x[2]
        arg[0,3,2,0]=(-0.191366770484)*x[0]**o+(0.961791275903)*x[0]+(-0.103547475035)*x[1]**o+(-0.745236510079)*x[1]+(-0.157506981712)*x[2]**o+(0.986229506787)*x[2]
        arg[0,3,2,1]=(-0.886091883647)*x[0]**o+(0.6979804248)*x[0]+(-0.366575419428)*x[1]**o+(-0.958601153259)*x[1]+(-0.0119724511494)*x[2]**o+(-0.76101132076)*x[2]
        arg[0,4,0,0]=(0.923290554434)*x[0]**o+(0.411359492198)*x[0]+(-0.977999562875)*x[1]**o+(0.352085161527)*x[1]+(0.955745235349)*x[2]**o+(-0.978885609257)*x[2]
        arg[0,4,0,1]=(0.226973479105)*x[0]**o+(-0.423881897251)*x[0]+(0.889101969481)*x[1]**o+(-0.994236389028)*x[1]+(0.0159169033361)*x[2]**o+(-0.559935154203)*x[2]
        arg[0,4,1,0]=(-0.225427291797)*x[0]**o+(-0.557334621848)*x[0]+(-0.486252586142)*x[1]**o+(0.0497120905216)*x[1]+(0.84165736691)*x[2]**o+(-0.185010156797)*x[2]
        arg[0,4,1,1]=(0.762694602609)*x[0]**o+(-0.447367170727)*x[0]+(-0.61103181818)*x[1]**o+(0.689761057656)*x[1]+(-0.631855120122)*x[2]**o+(0.33274655602)*x[2]
        arg[0,4,2,0]=(-0.198884868914)*x[0]**o+(0.716371959278)*x[0]+(0.260871768919)*x[1]**o+(0.622241740082)*x[1]+(0.774103524035)*x[2]**o+(0.186132703676)*x[2]
        arg[0,4,2,1]=(-0.309450645317)*x[0]**o+(0.204754571336)*x[0]+(0.483402810171)*x[1]**o+(0.404620744188)*x[1]+(-0.699137246959)*x[2]**o+(0.467428706196)*x[2]
        arg[1,0,0,0]=(0.642289506448)*x[0]**o+(0.383990304394)*x[0]+(0.556940973318)*x[1]**o+(0.135219729041)*x[1]+(0.577195836358)*x[2]**o+(-0.338924419674)*x[2]
        arg[1,0,0,1]=(-0.920873911295)*x[0]**o+(-0.330610192304)*x[0]+(-0.117226803856)*x[1]**o+(0.172482867684)*x[1]+(0.909140902709)*x[2]**o+(-0.590494167913)*x[2]
        arg[1,0,1,0]=(0.210161538861)*x[0]**o+(-0.83860241761)*x[0]+(0.0257180382661)*x[1]**o+(0.877633854348)*x[1]+(0.26045311754)*x[2]**o+(-0.971349902714)*x[2]
        arg[1,0,1,1]=(0.70736843125)*x[0]**o+(-0.765121985431)*x[0]+(-0.196525225943)*x[1]**o+(-0.175928729346)*x[1]+(0.945523028748)*x[2]**o+(0.877830322823)*x[2]
        arg[1,0,2,0]=(-0.963513016065)*x[0]**o+(0.0820537856351)*x[0]+(0.0122070196118)*x[1]**o+(-0.154147449592)*x[1]+(0.323844633268)*x[2]**o+(-0.124897698214)*x[2]
        arg[1,0,2,1]=(0.656267566405)*x[0]**o+(0.0174786620768)*x[0]+(0.763902435618)*x[1]**o+(-0.986021506043)*x[1]+(0.632625555994)*x[2]**o+(-0.110657743523)*x[2]
        arg[1,1,0,0]=(0.00354144688732)*x[0]**o+(-0.813515808171)*x[0]+(-0.100717717446)*x[1]**o+(0.0543708163629)*x[1]+(0.785927880762)*x[2]**o+(0.0109166652227)*x[2]
        arg[1,1,0,1]=(-0.239705688631)*x[0]**o+(0.402915642509)*x[0]+(0.463371068163)*x[1]**o+(-0.507302462928)*x[1]+(-0.0789844128295)*x[2]**o+(0.669144891499)*x[2]
        arg[1,1,1,0]=(-0.491891367001)*x[0]**o+(-0.524377636928)*x[0]+(0.903091863917)*x[1]**o+(-0.554368807183)*x[1]+(0.332899195339)*x[2]**o+(-0.645216147049)*x[2]
        arg[1,1,1,1]=(-0.915843941243)*x[0]**o+(-0.233175654748)*x[0]+(0.467730425037)*x[1]**o+(-0.765957936683)*x[1]+(0.726678877276)*x[2]**o+(0.770533947942)*x[2]
        arg[1,1,2,0]=(0.678071852032)*x[0]**o+(0.266537844237)*x[0]+(0.324335300151)*x[1]**o+(-0.286713971373)*x[1]+(-0.702384414014)*x[2]**o+(0.887198276519)*x[2]
        arg[1,1,2,1]=(0.0466444959648)*x[0]**o+(0.689162912677)*x[0]+(-0.35033923676)*x[1]**o+(-0.0536380766876)*x[1]+(-0.511833442517)*x[2]**o+(0.125566299829)*x[2]
        arg[1,2,0,0]=(-0.803170431622)*x[0]**o+(0.476211169752)*x[0]+(-0.170510267953)*x[1]**o+(-0.765945735317)*x[1]+(-0.169966329291)*x[2]**o+(-0.988525918724)*x[2]
        arg[1,2,0,1]=(0.289618751212)*x[0]**o+(0.294379040976)*x[0]+(-0.922478416838)*x[1]**o+(0.210546495537)*x[1]+(-0.87266381496)*x[2]**o+(-0.865571669577)*x[2]
        arg[1,2,1,0]=(-0.292811054005)*x[0]**o+(0.651280833746)*x[0]+(-0.625004203736)*x[1]**o+(0.372078169172)*x[1]+(-0.0539099055169)*x[2]**o+(0.0911869270008)*x[2]
        arg[1,2,1,1]=(0.496217698294)*x[0]**o+(-0.986021787133)*x[0]+(0.246417059532)*x[1]**o+(0.610226802401)*x[1]+(0.991534665298)*x[2]**o+(-0.507441185982)*x[2]
        arg[1,2,2,0]=(-0.839902565947)*x[0]**o+(-0.279607147278)*x[0]+(0.278212307963)*x[1]**o+(0.609793640036)*x[1]+(0.940471913774)*x[2]**o+(0.21860828705)*x[2]
        arg[1,2,2,1]=(0.789578112278)*x[0]**o+(-0.0226697818608)*x[0]+(0.216321340291)*x[1]**o+(-0.701818463454)*x[1]+(-0.166689030646)*x[2]**o+(0.966131542292)*x[2]
        arg[1,3,0,0]=(0.149337042551)*x[0]**o+(-0.785050579674)*x[0]+(0.530219924721)*x[1]**o+(-0.971927239398)*x[1]+(0.950917908603)*x[2]**o+(0.132668077447)*x[2]
        arg[1,3,0,1]=(0.983081559724)*x[0]**o+(-0.358694498455)*x[0]+(0.181080099281)*x[1]**o+(-0.699756291643)*x[1]+(-0.744335953304)*x[2]**o+(0.00262043221382)*x[2]
        arg[1,3,1,0]=(0.31776032045)*x[0]**o+(-0.0726184220029)*x[0]+(-0.759883735971)*x[1]**o+(-0.6389710873)*x[1]+(-0.607858499502)*x[2]**o+(-0.397169895784)*x[2]
        arg[1,3,1,1]=(-0.950468389728)*x[0]**o+(0.831121915521)*x[0]+(0.552857011083)*x[1]**o+(0.848599672594)*x[1]+(0.432183564231)*x[2]**o+(0.57537721362)*x[2]
        arg[1,3,2,0]=(0.736080143601)*x[0]**o+(-0.804140307747)*x[0]+(-0.163864867884)*x[1]**o+(0.950874042315)*x[1]+(-0.99676139412)*x[2]**o+(0.332320294246)*x[2]
        arg[1,3,2,1]=(0.119082582041)*x[0]**o+(-0.969740778852)*x[0]+(0.828432583532)*x[1]**o+(0.190324050395)*x[1]+(-0.42639142674)*x[2]**o+(0.315519578192)*x[2]
        arg[1,4,0,0]=(-0.798465141775)*x[0]**o+(-0.717282977908)*x[0]+(0.935655148713)*x[1]**o+(-0.655752964484)*x[1]+(0.159990149099)*x[2]**o+(-0.537115062687)*x[2]
        arg[1,4,0,1]=(0.445151775007)*x[0]**o+(-0.867956302051)*x[0]+(-0.809856879825)*x[1]**o+(-0.326897750981)*x[1]+(0.756352311987)*x[2]**o+(0.577585513459)*x[2]
        arg[1,4,1,0]=(0.294243347591)*x[0]**o+(0.774741656931)*x[0]+(-0.0799985737984)*x[1]**o+(0.179789107412)*x[1]+(-0.612198025189)*x[2]**o+(-0.07086009611)*x[2]
        arg[1,4,1,1]=(0.0692586287045)*x[0]**o+(-0.671956300122)*x[0]+(-0.948667104683)*x[1]**o+(0.68656246801)*x[1]+(-0.352975297696)*x[2]**o+(0.277952113716)*x[2]
        arg[1,4,2,0]=(-0.491316095517)*x[0]**o+(0.725899087239)*x[0]+(0.634113963431)*x[1]**o+(0.235694393453)*x[1]+(-0.15600720713)*x[2]**o+(0.954814976539)*x[2]
        arg[1,4,2,1]=(0.120206996339)*x[0]**o+(-0.86151411823)*x[0]+(-0.137093650097)*x[1]**o+(0.633402978246)*x[1]+(-0.42515364728)*x[2]**o+(-0.987634210617)*x[2]
        arg[2,0,0,0]=(0.539463971222)*x[0]**o+(-0.03435650579)*x[0]+(-0.884395295544)*x[1]**o+(0.215454879022)*x[1]+(0.842377328246)*x[2]**o+(0.370920136871)*x[2]
        arg[2,0,0,1]=(-0.411505463039)*x[0]**o+(0.446494039439)*x[0]+(0.0313771969556)*x[1]**o+(0.602893223061)*x[1]+(-0.30383490021)*x[2]**o+(-0.788028756758)*x[2]
        arg[2,0,1,0]=(0.489009615999)*x[0]**o+(-0.95508248016)*x[0]+(-0.21489475022)*x[1]**o+(-0.857263478431)*x[1]+(-0.123703409045)*x[2]**o+(0.663389562431)*x[2]
        arg[2,0,1,1]=(-0.0224763310929)*x[0]**o+(-0.315886249237)*x[0]+(-0.318201849695)*x[1]**o+(0.423732682462)*x[1]+(0.396065636381)*x[2]**o+(0.0816717878865)*x[2]
        arg[2,0,2,0]=(-0.191432618224)*x[0]**o+(0.688389390795)*x[0]+(-0.382948112159)*x[1]**o+(0.562905173818)*x[1]+(-0.118396714029)*x[2]**o+(0.234478642577)*x[2]
        arg[2,0,2,1]=(-0.348008173171)*x[0]**o+(0.191994166636)*x[0]+(0.450556057147)*x[1]**o+(-0.710821693343)*x[1]+(-0.245119333857)*x[2]**o+(0.262869525239)*x[2]
        arg[2,1,0,0]=(-0.160444506203)*x[0]**o+(-0.816142837432)*x[0]+(-0.619722480492)*x[1]**o+(-0.989077039675)*x[1]+(-0.0282349103977)*x[2]**o+(-0.629849819691)*x[2]
        arg[2,1,0,1]=(0.379513765297)*x[0]**o+(0.934652105586)*x[0]+(-0.632636427868)*x[1]**o+(-0.0492550496553)*x[1]+(0.137733572307)*x[2]**o+(0.779782637942)*x[2]
        arg[2,1,1,0]=(0.469499320417)*x[0]**o+(0.570880924125)*x[0]+(-0.0329833251419)*x[1]**o+(-0.0388751034604)*x[1]+(-0.878520094389)*x[2]**o+(0.992656649048)*x[2]
        arg[2,1,1,1]=(0.592005181995)*x[0]**o+(-0.553980493526)*x[0]+(0.359837665019)*x[1]**o+(0.811107941179)*x[1]+(-0.441929650912)*x[2]**o+(-0.768618378636)*x[2]
        arg[2,1,2,0]=(0.741376258816)*x[0]**o+(-0.238909756717)*x[0]+(0.204631082449)*x[1]**o+(0.970501662949)*x[1]+(-0.615638092239)*x[2]**o+(-0.575948750844)*x[2]
        arg[2,1,2,1]=(0.774050933112)*x[0]**o+(-0.872234076721)*x[0]+(-0.0936189240061)*x[1]**o+(-0.773608756961)*x[1]+(-0.119171907808)*x[2]**o+(-0.256124796845)*x[2]
        arg[2,2,0,0]=(0.459481754425)*x[0]**o+(0.478364929493)*x[0]+(0.460199733287)*x[1]**o+(0.66880294659)*x[1]+(-0.859525058389)*x[2]**o+(0.77960284414)*x[2]
        arg[2,2,0,1]=(0.690557835207)*x[0]**o+(-0.812305038592)*x[0]+(0.837924124791)*x[1]**o+(-0.785979206364)*x[1]+(-0.295553949819)*x[2]**o+(0.575096295897)*x[2]
        arg[2,2,1,0]=(0.443193683205)*x[0]**o+(0.0355033222253)*x[0]+(0.497641002766)*x[1]**o+(-0.636955142451)*x[1]+(-0.265324357454)*x[2]**o+(0.734148569078)*x[2]
        arg[2,2,1,1]=(-0.088707242599)*x[0]**o+(0.513403518669)*x[0]+(0.337741737651)*x[1]**o+(-0.174843924478)*x[1]+(0.687291255986)*x[2]**o+(0.473850502258)*x[2]
        arg[2,2,2,0]=(0.5305624642)*x[0]**o+(-0.972997835953)*x[0]+(0.00197107287888)*x[1]**o+(-0.75490872634)*x[1]+(-0.0569624569435)*x[2]**o+(-0.0780532868631)*x[2]
        arg[2,2,2,1]=(-0.744584471866)*x[0]**o+(0.915494909272)*x[0]+(0.239235050038)*x[1]**o+(-0.192466753187)*x[1]+(0.933556550824)*x[2]**o+(-0.786420251487)*x[2]
        arg[2,3,0,0]=(0.472114530627)*x[0]**o+(0.0693211807557)*x[0]+(-0.274273425713)*x[1]**o+(0.136242554894)*x[1]+(-0.329240675785)*x[2]**o+(0.0212539247343)*x[2]
        arg[2,3,0,1]=(-0.780426108099)*x[0]**o+(-0.632382551025)*x[0]+(0.554926949239)*x[1]**o+(0.264204009059)*x[1]+(-0.868007525824)*x[2]**o+(-0.937631120201)*x[2]
        arg[2,3,1,0]=(-0.826692080947)*x[0]**o+(0.915241858538)*x[0]+(-0.49986160859)*x[1]**o+(0.921686464658)*x[1]+(0.71996666304)*x[2]**o+(0.297609501096)*x[2]
        arg[2,3,1,1]=(0.991896994614)*x[0]**o+(-0.543310008103)*x[0]+(0.0791899631111)*x[1]**o+(-0.914088200145)*x[1]+(-0.496337624951)*x[2]**o+(0.708950898614)*x[2]
        arg[2,3,2,0]=(-0.235116966432)*x[0]**o+(-0.492848730207)*x[0]+(0.428728646789)*x[1]**o+(-0.410747634691)*x[1]+(0.705059815458)*x[2]**o+(-0.185604633831)*x[2]
        arg[2,3,2,1]=(0.838829037117)*x[0]**o+(0.670561147563)*x[0]+(0.621407965075)*x[1]**o+(-0.10624946305)*x[1]+(0.893703402687)*x[2]**o+(0.985504995821)*x[2]
        arg[2,4,0,0]=(0.635072504682)*x[0]**o+(-0.681962427555)*x[0]+(-0.00774518011081)*x[1]**o+(-0.264996228668)*x[1]+(0.690668403198)*x[2]**o+(0.883573761062)*x[2]
        arg[2,4,0,1]=(0.545632268651)*x[0]**o+(0.439668519394)*x[0]+(-0.482479808491)*x[1]**o+(0.752028962391)*x[1]+(0.0596773623939)*x[2]**o+(-0.369987181127)*x[2]
        arg[2,4,1,0]=(0.659005322796)*x[0]**o+(0.503940833428)*x[0]+(0.643633321586)*x[1]**o+(0.927082323252)*x[1]+(0.354072146184)*x[2]**o+(-0.638737560608)*x[2]
        arg[2,4,1,1]=(-0.443548860913)*x[0]**o+(0.707607048458)*x[0]+(-0.955225746402)*x[1]**o+(0.515995563585)*x[1]+(-0.288406637165)*x[2]**o+(-0.760864937318)*x[2]
        arg[2,4,2,0]=(0.898146580355)*x[0]**o+(-0.590650787106)*x[0]+(0.036095789753)*x[1]**o+(0.326040260581)*x[1]+(0.680069361306)*x[2]**o+(0.333839099341)*x[2]
        arg[2,4,2,1]=(-0.151364260271)*x[0]**o+(-0.675198183832)*x[0]+(-0.183626298978)*x[1]**o+(-0.182565217146)*x[1]+(0.0345632129814)*x[2]**o+(-0.581534100763)*x[2]
        arg[3,0,0,0]=(-0.942351703246)*x[0]**o+(-0.699567649453)*x[0]+(-0.832068665702)*x[1]**o+(0.163439416504)*x[1]+(0.742500339519)*x[2]**o+(-0.492882887489)*x[2]
        arg[3,0,0,1]=(0.990078296484)*x[0]**o+(-0.999837336587)*x[0]+(0.197589456459)*x[1]**o+(0.431403059019)*x[1]+(0.859864015871)*x[2]**o+(-0.267474829282)*x[2]
        arg[3,0,1,0]=(-0.776773660723)*x[0]**o+(-0.189048370486)*x[0]+(0.926953856802)*x[1]**o+(-0.72314744652)*x[1]+(0.330064363809)*x[2]**o+(0.188533334359)*x[2]
        arg[3,0,1,1]=(-0.854830058762)*x[0]**o+(0.832073883276)*x[0]+(-0.425691165124)*x[1]**o+(0.162351500316)*x[1]+(0.370604864512)*x[2]**o+(-0.581907071484)*x[2]
        arg[3,0,2,0]=(-0.316716909548)*x[0]**o+(0.335417284383)*x[0]+(-0.783061160352)*x[1]**o+(-0.253972519239)*x[1]+(-0.826432933721)*x[2]**o+(0.267368378363)*x[2]
        arg[3,0,2,1]=(-0.765899705462)*x[0]**o+(-0.0993875861476)*x[0]+(-0.211785004352)*x[1]**o+(-0.635330151497)*x[1]+(0.414575412599)*x[2]**o+(0.586882274623)*x[2]
        arg[3,1,0,0]=(0.987776377237)*x[0]**o+(0.796660669191)*x[0]+(0.580724891862)*x[1]**o+(-0.24149253995)*x[1]+(-0.633421016909)*x[2]**o+(-0.220110913711)*x[2]
        arg[3,1,0,1]=(-0.182240918081)*x[0]**o+(-0.290526050572)*x[0]+(0.00283230959976)*x[1]**o+(-0.933282166805)*x[1]+(-0.857893235883)*x[2]**o+(0.551414074127)*x[2]
        arg[3,1,1,0]=(-0.854435997816)*x[0]**o+(0.706455144263)*x[0]+(0.406237965254)*x[1]**o+(0.192071142415)*x[1]+(0.357966796834)*x[2]**o+(0.887011087987)*x[2]
        arg[3,1,1,1]=(0.832013396996)*x[0]**o+(0.696133072321)*x[0]+(0.682726968712)*x[1]**o+(0.613316240514)*x[1]+(0.12600339887)*x[2]**o+(-0.620088231382)*x[2]
        arg[3,1,2,0]=(-0.178681615221)*x[0]**o+(-0.952835986168)*x[0]+(0.299692693746)*x[1]**o+(0.892451908416)*x[1]+(0.202403273634)*x[2]**o+(0.677212897943)*x[2]
        arg[3,1,2,1]=(0.0592594053342)*x[0]**o+(0.56391153959)*x[0]+(-0.874866627667)*x[1]**o+(0.432102900769)*x[1]+(-0.49073126285)*x[2]**o+(-0.010495860028)*x[2]
        arg[3,2,0,0]=(0.548006445814)*x[0]**o+(0.667313392202)*x[0]+(0.397805355499)*x[1]**o+(-0.788100267808)*x[1]+(0.869234055258)*x[2]**o+(-0.830241520401)*x[2]
        arg[3,2,0,1]=(0.72586139216)*x[0]**o+(-0.931878957878)*x[0]+(-0.94119555858)*x[1]**o+(-0.133544806169)*x[1]+(0.754263323382)*x[2]**o+(-0.446604740157)*x[2]
        arg[3,2,1,0]=(-0.28135830707)*x[0]**o+(-0.478744058518)*x[0]+(0.88219033731)*x[1]**o+(0.460587286945)*x[1]+(-0.849692146096)*x[2]**o+(0.0927701345107)*x[2]
        arg[3,2,1,1]=(-0.681859641611)*x[0]**o+(-0.670164293088)*x[0]+(0.796366374435)*x[1]**o+(0.837230317054)*x[1]+(-0.413283915904)*x[2]**o+(0.779624227187)*x[2]
        arg[3,2,2,0]=(-0.803242771255)*x[0]**o+(-0.0687616271776)*x[0]+(-0.948856027999)*x[1]**o+(0.762568790931)*x[1]+(0.937588292604)*x[2]**o+(-0.0224232611283)*x[2]
        arg[3,2,2,1]=(-0.215752944937)*x[0]**o+(-0.807114296075)*x[0]+(0.970625375964)*x[1]**o+(0.68578124692)*x[1]+(-0.165007047419)*x[2]**o+(0.942718439981)*x[2]
        arg[3,3,0,0]=(0.152001104716)*x[0]**o+(-0.657242342357)*x[0]+(0.559608661705)*x[1]**o+(0.0764403273251)*x[1]+(0.633414754695)*x[2]**o+(0.430777868344)*x[2]
        arg[3,3,0,1]=(-0.898109497759)*x[0]**o+(0.736574249406)*x[0]+(-0.569584184574)*x[1]**o+(-0.145501395506)*x[1]+(-0.412424116475)*x[2]**o+(-0.416350923096)*x[2]
        arg[3,3,1,0]=(0.602022795614)*x[0]**o+(-0.848525569825)*x[0]+(-0.713426287505)*x[1]**o+(-0.829278452687)*x[1]+(-0.491257659316)*x[2]**o+(0.90219908999)*x[2]
        arg[3,3,1,1]=(-0.769110216066)*x[0]**o+(-0.789464037338)*x[0]+(-0.363686172584)*x[1]**o+(-0.86190787068)*x[1]+(0.275438119232)*x[2]**o+(0.285209834248)*x[2]
        arg[3,3,2,0]=(0.351662405853)*x[0]**o+(0.8403302307)*x[0]+(-0.449869332519)*x[1]**o+(0.936043614838)*x[1]+(-0.656908777335)*x[2]**o+(-0.231923872531)*x[2]
        arg[3,3,2,1]=(0.873848945653)*x[0]**o+(0.551429883959)*x[0]+(-0.577942548701)*x[1]**o+(-0.254317163049)*x[1]+(0.64878089878)*x[2]**o+(0.7135339515)*x[2]
        arg[3,4,0,0]=(0.891435610494)*x[0]**o+(0.598893704498)*x[0]+(0.24682094756)*x[1]**o+(0.0895612710338)*x[1]+(-0.513686244953)*x[2]**o+(0.307205505502)*x[2]
        arg[3,4,0,1]=(-0.577752944196)*x[0]**o+(-0.458383751623)*x[0]+(-0.913956988143)*x[1]**o+(0.470857602235)*x[1]+(0.631726850665)*x[2]**o+(-0.112962289904)*x[2]
        arg[3,4,1,0]=(-0.65648177489)*x[0]**o+(-0.843530972747)*x[0]+(-0.0332153723809)*x[1]**o+(0.230484263355)*x[1]+(-0.865687508592)*x[2]**o+(0.122751735082)*x[2]
        arg[3,4,1,1]=(-0.353502640734)*x[0]**o+(-0.816020900282)*x[0]+(0.943688796868)*x[1]**o+(-0.526707881958)*x[1]+(-0.790258298117)*x[2]**o+(-0.748706809042)*x[2]
        arg[3,4,2,0]=(-0.596345912175)*x[0]**o+(0.872771121837)*x[0]+(0.100107395575)*x[1]**o+(0.950482249217)*x[1]+(0.0487634582783)*x[2]**o+(-0.0285218097887)*x[2]
        arg[3,4,2,1]=(-0.827443553933)*x[0]**o+(-0.922564800149)*x[0]+(-0.810354385287)*x[1]**o+(0.630079217293)*x[1]+(-0.316280559723)*x[2]**o+(0.977830435871)*x[2]
        ref[0,0,0,0]=(0.310272924207)*x_ref[0]**o+(-0.664497806351)*x_ref[0]+(0.741942132609)*x_ref[1]**o+(0.636051298779)*x_ref[1]+(0.76627159146)*x_ref[2]**o+(0.492960732588)*x_ref[2]
        ref[0,0,0,1]=(-0.513740801773)*x_ref[0]**o+(-0.633330750909)*x_ref[0]+(0.177934618983)*x_ref[1]**o+(-0.323745723166)*x_ref[1]+(0.694167180904)*x_ref[2]**o+(0.441239251435)*x_ref[2]
        ref[0,0,1,0]=(0.417477695848)*x_ref[0]**o+(-0.923415976002)*x_ref[0]+(0.0765555276662)*x_ref[1]**o+(-0.277290890834)*x_ref[1]+(0.0177191326614)*x_ref[2]**o+(-0.0842520100387)*x_ref[2]
        ref[0,0,1,1]=(-0.761916249894)*x_ref[0]**o+(0.169673139552)*x_ref[0]+(0.0306290828795)*x_ref[1]**o+(0.842234592613)*x_ref[1]+(-0.350342692669)*x_ref[2]**o+(0.321292270658)*x_ref[2]
        ref[0,0,2,0]=(-0.707494137624)*x_ref[0]**o+(0.346873490395)*x_ref[0]+(-0.68281925138)*x_ref[1]**o+(-0.830538083274)*x_ref[1]+(0.329372141613)*x_ref[2]**o+(-0.121366157163)*x_ref[2]
        ref[0,0,2,1]=(-0.708018186507)*x_ref[0]**o+(-0.778051239314)*x_ref[0]+(-0.43106291563)*x_ref[1]**o+(-0.0115914251143)*x_ref[1]+(0.487334215892)*x_ref[2]**o+(-0.289232284658)*x_ref[2]
        ref[0,1,0,0]=(0.530614704199)*x_ref[0]**o+(0.518622028317)*x_ref[0]+(-0.0143129589551)*x_ref[1]**o+(0.808469352257)*x_ref[1]+(-0.156787373225)*x_ref[2]**o+(0.668887813143)*x_ref[2]
        ref[0,1,0,1]=(0.953223233706)*x_ref[0]**o+(-0.164323838353)*x_ref[0]+(0.336717320858)*x_ref[1]**o+(-0.520230488172)*x_ref[1]+(0.689960738002)*x_ref[2]**o+(0.487630947578)*x_ref[2]
        ref[0,1,1,0]=(-0.418095373429)*x_ref[0]**o+(-0.941817844939)*x_ref[0]+(0.88105115607)*x_ref[1]**o+(-0.820502501439)*x_ref[1]+(-0.693858267667)*x_ref[2]**o+(0.530567713127)*x_ref[2]
        ref[0,1,1,1]=(-0.716861320218)*x_ref[0]**o+(-0.736619859467)*x_ref[0]+(-0.885478605601)*x_ref[1]**o+(-0.129375578967)*x_ref[1]+(-0.0181433907318)*x_ref[2]**o+(0.373255537959)*x_ref[2]
        ref[0,1,2,0]=(-0.174742583167)*x_ref[0]**o+(-0.0887967480112)*x_ref[0]+(-0.291168698662)*x_ref[1]**o+(-0.188608506769)*x_ref[1]+(0.22479613454)*x_ref[2]**o+(0.34173928582)*x_ref[2]
        ref[0,1,2,1]=(0.117208929504)*x_ref[0]**o+(-0.420969282079)*x_ref[0]+(0.585298109836)*x_ref[1]**o+(0.771355162745)*x_ref[1]+(-0.51374493747)*x_ref[2]**o+(0.785016548855)*x_ref[2]
        ref[0,2,0,0]=(0.737394887886)*x_ref[0]**o+(0.966080271342)*x_ref[0]+(-0.373178384492)*x_ref[1]**o+(-0.845505386142)*x_ref[1]+(-0.274654519371)*x_ref[2]**o+(0.709356557887)*x_ref[2]
        ref[0,2,0,1]=(0.0365864090636)*x_ref[0]**o+(-0.393382479787)*x_ref[0]+(0.018273360977)*x_ref[1]**o+(-0.509201909328)*x_ref[1]+(-0.777852392534)*x_ref[2]**o+(0.0417280247507)*x_ref[2]
        ref[0,2,1,0]=(0.637829302751)*x_ref[0]**o+(-0.767824790186)*x_ref[0]+(0.843382009511)*x_ref[1]**o+(-0.599633234403)*x_ref[1]+(0.14283138941)*x_ref[2]**o+(-0.354404633991)*x_ref[2]
        ref[0,2,1,1]=(-0.465258613599)*x_ref[0]**o+(0.09341154383)*x_ref[0]+(-0.629114782988)*x_ref[1]**o+(0.91608655383)*x_ref[1]+(-0.692901135848)*x_ref[2]**o+(-0.707225310667)*x_ref[2]
        ref[0,2,2,0]=(0.9935958759)*x_ref[0]**o+(-0.385008742466)*x_ref[0]+(0.153885356854)*x_ref[1]**o+(0.146270680656)*x_ref[1]+(0.305838410343)*x_ref[2]**o+(0.141193552717)*x_ref[2]
        ref[0,2,2,1]=(-0.487538023894)*x_ref[0]**o+(-0.814283442647)*x_ref[0]+(0.166008520199)*x_ref[1]**o+(0.313655774416)*x_ref[1]+(0.282308782973)*x_ref[2]**o+(0.309176062484)*x_ref[2]
        ref[0,3,0,0]=(0.00203507798913)*x_ref[0]**o+(-0.19902542604)*x_ref[0]+(0.88110918624)*x_ref[1]**o+(0.223549219288)*x_ref[1]+(0.135758952369)*x_ref[2]**o+(0.59116783639)*x_ref[2]
        ref[0,3,0,1]=(-0.185063700767)*x_ref[0]**o+(0.833862155173)*x_ref[0]+(-0.0939117509645)*x_ref[1]**o+(0.288003692231)*x_ref[1]+(0.580196157839)*x_ref[2]**o+(0.299320177754)*x_ref[2]
        ref[0,3,1,0]=(-0.039100718826)*x_ref[0]**o+(0.423179869395)*x_ref[0]+(-0.013669578975)*x_ref[1]**o+(-0.355053343085)*x_ref[1]+(0.89285158975)*x_ref[2]**o+(0.392984367877)*x_ref[2]
        ref[0,3,1,1]=(0.0988989416429)*x_ref[0]**o+(-0.446243235174)*x_ref[0]+(-0.691253313617)*x_ref[1]**o+(0.338823693086)*x_ref[1]+(0.707054126334)*x_ref[2]**o+(0.601121388278)*x_ref[2]
        ref[0,3,2,0]=(-0.191366770484)*x_ref[0]**o+(0.961791275903)*x_ref[0]+(-0.103547475035)*x_ref[1]**o+(-0.745236510079)*x_ref[1]+(-0.157506981712)*x_ref[2]**o+(0.986229506787)*x_ref[2]
        ref[0,3,2,1]=(-0.886091883647)*x_ref[0]**o+(0.6979804248)*x_ref[0]+(-0.366575419428)*x_ref[1]**o+(-0.958601153259)*x_ref[1]+(-0.0119724511494)*x_ref[2]**o+(-0.76101132076)*x_ref[2]
        ref[0,4,0,0]=(0.923290554434)*x_ref[0]**o+(0.411359492198)*x_ref[0]+(-0.977999562875)*x_ref[1]**o+(0.352085161527)*x_ref[1]+(0.955745235349)*x_ref[2]**o+(-0.978885609257)*x_ref[2]
        ref[0,4,0,1]=(0.226973479105)*x_ref[0]**o+(-0.423881897251)*x_ref[0]+(0.889101969481)*x_ref[1]**o+(-0.994236389028)*x_ref[1]+(0.0159169033361)*x_ref[2]**o+(-0.559935154203)*x_ref[2]
        ref[0,4,1,0]=(-0.225427291797)*x_ref[0]**o+(-0.557334621848)*x_ref[0]+(-0.486252586142)*x_ref[1]**o+(0.0497120905216)*x_ref[1]+(0.84165736691)*x_ref[2]**o+(-0.185010156797)*x_ref[2]
        ref[0,4,1,1]=(0.762694602609)*x_ref[0]**o+(-0.447367170727)*x_ref[0]+(-0.61103181818)*x_ref[1]**o+(0.689761057656)*x_ref[1]+(-0.631855120122)*x_ref[2]**o+(0.33274655602)*x_ref[2]
        ref[0,4,2,0]=(-0.198884868914)*x_ref[0]**o+(0.716371959278)*x_ref[0]+(0.260871768919)*x_ref[1]**o+(0.622241740082)*x_ref[1]+(0.774103524035)*x_ref[2]**o+(0.186132703676)*x_ref[2]
        ref[0,4,2,1]=(-0.309450645317)*x_ref[0]**o+(0.204754571336)*x_ref[0]+(0.483402810171)*x_ref[1]**o+(0.404620744188)*x_ref[1]+(-0.699137246959)*x_ref[2]**o+(0.467428706196)*x_ref[2]
        ref[1,0,0,0]=(0.642289506448)*x_ref[0]**o+(0.383990304394)*x_ref[0]+(0.556940973318)*x_ref[1]**o+(0.135219729041)*x_ref[1]+(0.577195836358)*x_ref[2]**o+(-0.338924419674)*x_ref[2]
        ref[1,0,0,1]=(-0.920873911295)*x_ref[0]**o+(-0.330610192304)*x_ref[0]+(-0.117226803856)*x_ref[1]**o+(0.172482867684)*x_ref[1]+(0.909140902709)*x_ref[2]**o+(-0.590494167913)*x_ref[2]
        ref[1,0,1,0]=(0.210161538861)*x_ref[0]**o+(-0.83860241761)*x_ref[0]+(0.0257180382661)*x_ref[1]**o+(0.877633854348)*x_ref[1]+(0.26045311754)*x_ref[2]**o+(-0.971349902714)*x_ref[2]
        ref[1,0,1,1]=(0.70736843125)*x_ref[0]**o+(-0.765121985431)*x_ref[0]+(-0.196525225943)*x_ref[1]**o+(-0.175928729346)*x_ref[1]+(0.945523028748)*x_ref[2]**o+(0.877830322823)*x_ref[2]
        ref[1,0,2,0]=(-0.963513016065)*x_ref[0]**o+(0.0820537856351)*x_ref[0]+(0.0122070196118)*x_ref[1]**o+(-0.154147449592)*x_ref[1]+(0.323844633268)*x_ref[2]**o+(-0.124897698214)*x_ref[2]
        ref[1,0,2,1]=(0.656267566405)*x_ref[0]**o+(0.0174786620768)*x_ref[0]+(0.763902435618)*x_ref[1]**o+(-0.986021506043)*x_ref[1]+(0.632625555994)*x_ref[2]**o+(-0.110657743523)*x_ref[2]
        ref[1,1,0,0]=(0.00354144688732)*x_ref[0]**o+(-0.813515808171)*x_ref[0]+(-0.100717717446)*x_ref[1]**o+(0.0543708163629)*x_ref[1]+(0.785927880762)*x_ref[2]**o+(0.0109166652227)*x_ref[2]
        ref[1,1,0,1]=(-0.239705688631)*x_ref[0]**o+(0.402915642509)*x_ref[0]+(0.463371068163)*x_ref[1]**o+(-0.507302462928)*x_ref[1]+(-0.0789844128295)*x_ref[2]**o+(0.669144891499)*x_ref[2]
        ref[1,1,1,0]=(-0.491891367001)*x_ref[0]**o+(-0.524377636928)*x_ref[0]+(0.903091863917)*x_ref[1]**o+(-0.554368807183)*x_ref[1]+(0.332899195339)*x_ref[2]**o+(-0.645216147049)*x_ref[2]
        ref[1,1,1,1]=(-0.915843941243)*x_ref[0]**o+(-0.233175654748)*x_ref[0]+(0.467730425037)*x_ref[1]**o+(-0.765957936683)*x_ref[1]+(0.726678877276)*x_ref[2]**o+(0.770533947942)*x_ref[2]
        ref[1,1,2,0]=(0.678071852032)*x_ref[0]**o+(0.266537844237)*x_ref[0]+(0.324335300151)*x_ref[1]**o+(-0.286713971373)*x_ref[1]+(-0.702384414014)*x_ref[2]**o+(0.887198276519)*x_ref[2]
        ref[1,1,2,1]=(0.0466444959648)*x_ref[0]**o+(0.689162912677)*x_ref[0]+(-0.35033923676)*x_ref[1]**o+(-0.0536380766876)*x_ref[1]+(-0.511833442517)*x_ref[2]**o+(0.125566299829)*x_ref[2]
        ref[1,2,0,0]=(-0.803170431622)*x_ref[0]**o+(0.476211169752)*x_ref[0]+(-0.170510267953)*x_ref[1]**o+(-0.765945735317)*x_ref[1]+(-0.169966329291)*x_ref[2]**o+(-0.988525918724)*x_ref[2]
        ref[1,2,0,1]=(0.289618751212)*x_ref[0]**o+(0.294379040976)*x_ref[0]+(-0.922478416838)*x_ref[1]**o+(0.210546495537)*x_ref[1]+(-0.87266381496)*x_ref[2]**o+(-0.865571669577)*x_ref[2]
        ref[1,2,1,0]=(-0.292811054005)*x_ref[0]**o+(0.651280833746)*x_ref[0]+(-0.625004203736)*x_ref[1]**o+(0.372078169172)*x_ref[1]+(-0.0539099055169)*x_ref[2]**o+(0.0911869270008)*x_ref[2]
        ref[1,2,1,1]=(0.496217698294)*x_ref[0]**o+(-0.986021787133)*x_ref[0]+(0.246417059532)*x_ref[1]**o+(0.610226802401)*x_ref[1]+(0.991534665298)*x_ref[2]**o+(-0.507441185982)*x_ref[2]
        ref[1,2,2,0]=(-0.839902565947)*x_ref[0]**o+(-0.279607147278)*x_ref[0]+(0.278212307963)*x_ref[1]**o+(0.609793640036)*x_ref[1]+(0.940471913774)*x_ref[2]**o+(0.21860828705)*x_ref[2]
        ref[1,2,2,1]=(0.789578112278)*x_ref[0]**o+(-0.0226697818608)*x_ref[0]+(0.216321340291)*x_ref[1]**o+(-0.701818463454)*x_ref[1]+(-0.166689030646)*x_ref[2]**o+(0.966131542292)*x_ref[2]
        ref[1,3,0,0]=(0.149337042551)*x_ref[0]**o+(-0.785050579674)*x_ref[0]+(0.530219924721)*x_ref[1]**o+(-0.971927239398)*x_ref[1]+(0.950917908603)*x_ref[2]**o+(0.132668077447)*x_ref[2]
        ref[1,3,0,1]=(0.983081559724)*x_ref[0]**o+(-0.358694498455)*x_ref[0]+(0.181080099281)*x_ref[1]**o+(-0.699756291643)*x_ref[1]+(-0.744335953304)*x_ref[2]**o+(0.00262043221382)*x_ref[2]
        ref[1,3,1,0]=(0.31776032045)*x_ref[0]**o+(-0.0726184220029)*x_ref[0]+(-0.759883735971)*x_ref[1]**o+(-0.6389710873)*x_ref[1]+(-0.607858499502)*x_ref[2]**o+(-0.397169895784)*x_ref[2]
        ref[1,3,1,1]=(-0.950468389728)*x_ref[0]**o+(0.831121915521)*x_ref[0]+(0.552857011083)*x_ref[1]**o+(0.848599672594)*x_ref[1]+(0.432183564231)*x_ref[2]**o+(0.57537721362)*x_ref[2]
        ref[1,3,2,0]=(0.736080143601)*x_ref[0]**o+(-0.804140307747)*x_ref[0]+(-0.163864867884)*x_ref[1]**o+(0.950874042315)*x_ref[1]+(-0.99676139412)*x_ref[2]**o+(0.332320294246)*x_ref[2]
        ref[1,3,2,1]=(0.119082582041)*x_ref[0]**o+(-0.969740778852)*x_ref[0]+(0.828432583532)*x_ref[1]**o+(0.190324050395)*x_ref[1]+(-0.42639142674)*x_ref[2]**o+(0.315519578192)*x_ref[2]
        ref[1,4,0,0]=(-0.798465141775)*x_ref[0]**o+(-0.717282977908)*x_ref[0]+(0.935655148713)*x_ref[1]**o+(-0.655752964484)*x_ref[1]+(0.159990149099)*x_ref[2]**o+(-0.537115062687)*x_ref[2]
        ref[1,4,0,1]=(0.445151775007)*x_ref[0]**o+(-0.867956302051)*x_ref[0]+(-0.809856879825)*x_ref[1]**o+(-0.326897750981)*x_ref[1]+(0.756352311987)*x_ref[2]**o+(0.577585513459)*x_ref[2]
        ref[1,4,1,0]=(0.294243347591)*x_ref[0]**o+(0.774741656931)*x_ref[0]+(-0.0799985737984)*x_ref[1]**o+(0.179789107412)*x_ref[1]+(-0.612198025189)*x_ref[2]**o+(-0.07086009611)*x_ref[2]
        ref[1,4,1,1]=(0.0692586287045)*x_ref[0]**o+(-0.671956300122)*x_ref[0]+(-0.948667104683)*x_ref[1]**o+(0.68656246801)*x_ref[1]+(-0.352975297696)*x_ref[2]**o+(0.277952113716)*x_ref[2]
        ref[1,4,2,0]=(-0.491316095517)*x_ref[0]**o+(0.725899087239)*x_ref[0]+(0.634113963431)*x_ref[1]**o+(0.235694393453)*x_ref[1]+(-0.15600720713)*x_ref[2]**o+(0.954814976539)*x_ref[2]
        ref[1,4,2,1]=(0.120206996339)*x_ref[0]**o+(-0.86151411823)*x_ref[0]+(-0.137093650097)*x_ref[1]**o+(0.633402978246)*x_ref[1]+(-0.42515364728)*x_ref[2]**o+(-0.987634210617)*x_ref[2]
        ref[2,0,0,0]=(0.539463971222)*x_ref[0]**o+(-0.03435650579)*x_ref[0]+(-0.884395295544)*x_ref[1]**o+(0.215454879022)*x_ref[1]+(0.842377328246)*x_ref[2]**o+(0.370920136871)*x_ref[2]
        ref[2,0,0,1]=(-0.411505463039)*x_ref[0]**o+(0.446494039439)*x_ref[0]+(0.0313771969556)*x_ref[1]**o+(0.602893223061)*x_ref[1]+(-0.30383490021)*x_ref[2]**o+(-0.788028756758)*x_ref[2]
        ref[2,0,1,0]=(0.489009615999)*x_ref[0]**o+(-0.95508248016)*x_ref[0]+(-0.21489475022)*x_ref[1]**o+(-0.857263478431)*x_ref[1]+(-0.123703409045)*x_ref[2]**o+(0.663389562431)*x_ref[2]
        ref[2,0,1,1]=(-0.0224763310929)*x_ref[0]**o+(-0.315886249237)*x_ref[0]+(-0.318201849695)*x_ref[1]**o+(0.423732682462)*x_ref[1]+(0.396065636381)*x_ref[2]**o+(0.0816717878865)*x_ref[2]
        ref[2,0,2,0]=(-0.191432618224)*x_ref[0]**o+(0.688389390795)*x_ref[0]+(-0.382948112159)*x_ref[1]**o+(0.562905173818)*x_ref[1]+(-0.118396714029)*x_ref[2]**o+(0.234478642577)*x_ref[2]
        ref[2,0,2,1]=(-0.348008173171)*x_ref[0]**o+(0.191994166636)*x_ref[0]+(0.450556057147)*x_ref[1]**o+(-0.710821693343)*x_ref[1]+(-0.245119333857)*x_ref[2]**o+(0.262869525239)*x_ref[2]
        ref[2,1,0,0]=(-0.160444506203)*x_ref[0]**o+(-0.816142837432)*x_ref[0]+(-0.619722480492)*x_ref[1]**o+(-0.989077039675)*x_ref[1]+(-0.0282349103977)*x_ref[2]**o+(-0.629849819691)*x_ref[2]
        ref[2,1,0,1]=(0.379513765297)*x_ref[0]**o+(0.934652105586)*x_ref[0]+(-0.632636427868)*x_ref[1]**o+(-0.0492550496553)*x_ref[1]+(0.137733572307)*x_ref[2]**o+(0.779782637942)*x_ref[2]
        ref[2,1,1,0]=(0.469499320417)*x_ref[0]**o+(0.570880924125)*x_ref[0]+(-0.0329833251419)*x_ref[1]**o+(-0.0388751034604)*x_ref[1]+(-0.878520094389)*x_ref[2]**o+(0.992656649048)*x_ref[2]
        ref[2,1,1,1]=(0.592005181995)*x_ref[0]**o+(-0.553980493526)*x_ref[0]+(0.359837665019)*x_ref[1]**o+(0.811107941179)*x_ref[1]+(-0.441929650912)*x_ref[2]**o+(-0.768618378636)*x_ref[2]
        ref[2,1,2,0]=(0.741376258816)*x_ref[0]**o+(-0.238909756717)*x_ref[0]+(0.204631082449)*x_ref[1]**o+(0.970501662949)*x_ref[1]+(-0.615638092239)*x_ref[2]**o+(-0.575948750844)*x_ref[2]
        ref[2,1,2,1]=(0.774050933112)*x_ref[0]**o+(-0.872234076721)*x_ref[0]+(-0.0936189240061)*x_ref[1]**o+(-0.773608756961)*x_ref[1]+(-0.119171907808)*x_ref[2]**o+(-0.256124796845)*x_ref[2]
        ref[2,2,0,0]=(0.459481754425)*x_ref[0]**o+(0.478364929493)*x_ref[0]+(0.460199733287)*x_ref[1]**o+(0.66880294659)*x_ref[1]+(-0.859525058389)*x_ref[2]**o+(0.77960284414)*x_ref[2]
        ref[2,2,0,1]=(0.690557835207)*x_ref[0]**o+(-0.812305038592)*x_ref[0]+(0.837924124791)*x_ref[1]**o+(-0.785979206364)*x_ref[1]+(-0.295553949819)*x_ref[2]**o+(0.575096295897)*x_ref[2]
        ref[2,2,1,0]=(0.443193683205)*x_ref[0]**o+(0.0355033222253)*x_ref[0]+(0.497641002766)*x_ref[1]**o+(-0.636955142451)*x_ref[1]+(-0.265324357454)*x_ref[2]**o+(0.734148569078)*x_ref[2]
        ref[2,2,1,1]=(-0.088707242599)*x_ref[0]**o+(0.513403518669)*x_ref[0]+(0.337741737651)*x_ref[1]**o+(-0.174843924478)*x_ref[1]+(0.687291255986)*x_ref[2]**o+(0.473850502258)*x_ref[2]
        ref[2,2,2,0]=(0.5305624642)*x_ref[0]**o+(-0.972997835953)*x_ref[0]+(0.00197107287888)*x_ref[1]**o+(-0.75490872634)*x_ref[1]+(-0.0569624569435)*x_ref[2]**o+(-0.0780532868631)*x_ref[2]
        ref[2,2,2,1]=(-0.744584471866)*x_ref[0]**o+(0.915494909272)*x_ref[0]+(0.239235050038)*x_ref[1]**o+(-0.192466753187)*x_ref[1]+(0.933556550824)*x_ref[2]**o+(-0.786420251487)*x_ref[2]
        ref[2,3,0,0]=(0.472114530627)*x_ref[0]**o+(0.0693211807557)*x_ref[0]+(-0.274273425713)*x_ref[1]**o+(0.136242554894)*x_ref[1]+(-0.329240675785)*x_ref[2]**o+(0.0212539247343)*x_ref[2]
        ref[2,3,0,1]=(-0.780426108099)*x_ref[0]**o+(-0.632382551025)*x_ref[0]+(0.554926949239)*x_ref[1]**o+(0.264204009059)*x_ref[1]+(-0.868007525824)*x_ref[2]**o+(-0.937631120201)*x_ref[2]
        ref[2,3,1,0]=(-0.826692080947)*x_ref[0]**o+(0.915241858538)*x_ref[0]+(-0.49986160859)*x_ref[1]**o+(0.921686464658)*x_ref[1]+(0.71996666304)*x_ref[2]**o+(0.297609501096)*x_ref[2]
        ref[2,3,1,1]=(0.991896994614)*x_ref[0]**o+(-0.543310008103)*x_ref[0]+(0.0791899631111)*x_ref[1]**o+(-0.914088200145)*x_ref[1]+(-0.496337624951)*x_ref[2]**o+(0.708950898614)*x_ref[2]
        ref[2,3,2,0]=(-0.235116966432)*x_ref[0]**o+(-0.492848730207)*x_ref[0]+(0.428728646789)*x_ref[1]**o+(-0.410747634691)*x_ref[1]+(0.705059815458)*x_ref[2]**o+(-0.185604633831)*x_ref[2]
        ref[2,3,2,1]=(0.838829037117)*x_ref[0]**o+(0.670561147563)*x_ref[0]+(0.621407965075)*x_ref[1]**o+(-0.10624946305)*x_ref[1]+(0.893703402687)*x_ref[2]**o+(0.985504995821)*x_ref[2]
        ref[2,4,0,0]=(0.635072504682)*x_ref[0]**o+(-0.681962427555)*x_ref[0]+(-0.00774518011081)*x_ref[1]**o+(-0.264996228668)*x_ref[1]+(0.690668403198)*x_ref[2]**o+(0.883573761062)*x_ref[2]
        ref[2,4,0,1]=(0.545632268651)*x_ref[0]**o+(0.439668519394)*x_ref[0]+(-0.482479808491)*x_ref[1]**o+(0.752028962391)*x_ref[1]+(0.0596773623939)*x_ref[2]**o+(-0.369987181127)*x_ref[2]
        ref[2,4,1,0]=(0.659005322796)*x_ref[0]**o+(0.503940833428)*x_ref[0]+(0.643633321586)*x_ref[1]**o+(0.927082323252)*x_ref[1]+(0.354072146184)*x_ref[2]**o+(-0.638737560608)*x_ref[2]
        ref[2,4,1,1]=(-0.443548860913)*x_ref[0]**o+(0.707607048458)*x_ref[0]+(-0.955225746402)*x_ref[1]**o+(0.515995563585)*x_ref[1]+(-0.288406637165)*x_ref[2]**o+(-0.760864937318)*x_ref[2]
        ref[2,4,2,0]=(0.898146580355)*x_ref[0]**o+(-0.590650787106)*x_ref[0]+(0.036095789753)*x_ref[1]**o+(0.326040260581)*x_ref[1]+(0.680069361306)*x_ref[2]**o+(0.333839099341)*x_ref[2]
        ref[2,4,2,1]=(-0.151364260271)*x_ref[0]**o+(-0.675198183832)*x_ref[0]+(-0.183626298978)*x_ref[1]**o+(-0.182565217146)*x_ref[1]+(0.0345632129814)*x_ref[2]**o+(-0.581534100763)*x_ref[2]
        ref[3,0,0,0]=(-0.942351703246)*x_ref[0]**o+(-0.699567649453)*x_ref[0]+(-0.832068665702)*x_ref[1]**o+(0.163439416504)*x_ref[1]+(0.742500339519)*x_ref[2]**o+(-0.492882887489)*x_ref[2]
        ref[3,0,0,1]=(0.990078296484)*x_ref[0]**o+(-0.999837336587)*x_ref[0]+(0.197589456459)*x_ref[1]**o+(0.431403059019)*x_ref[1]+(0.859864015871)*x_ref[2]**o+(-0.267474829282)*x_ref[2]
        ref[3,0,1,0]=(-0.776773660723)*x_ref[0]**o+(-0.189048370486)*x_ref[0]+(0.926953856802)*x_ref[1]**o+(-0.72314744652)*x_ref[1]+(0.330064363809)*x_ref[2]**o+(0.188533334359)*x_ref[2]
        ref[3,0,1,1]=(-0.854830058762)*x_ref[0]**o+(0.832073883276)*x_ref[0]+(-0.425691165124)*x_ref[1]**o+(0.162351500316)*x_ref[1]+(0.370604864512)*x_ref[2]**o+(-0.581907071484)*x_ref[2]
        ref[3,0,2,0]=(-0.316716909548)*x_ref[0]**o+(0.335417284383)*x_ref[0]+(-0.783061160352)*x_ref[1]**o+(-0.253972519239)*x_ref[1]+(-0.826432933721)*x_ref[2]**o+(0.267368378363)*x_ref[2]
        ref[3,0,2,1]=(-0.765899705462)*x_ref[0]**o+(-0.0993875861476)*x_ref[0]+(-0.211785004352)*x_ref[1]**o+(-0.635330151497)*x_ref[1]+(0.414575412599)*x_ref[2]**o+(0.586882274623)*x_ref[2]
        ref[3,1,0,0]=(0.987776377237)*x_ref[0]**o+(0.796660669191)*x_ref[0]+(0.580724891862)*x_ref[1]**o+(-0.24149253995)*x_ref[1]+(-0.633421016909)*x_ref[2]**o+(-0.220110913711)*x_ref[2]
        ref[3,1,0,1]=(-0.182240918081)*x_ref[0]**o+(-0.290526050572)*x_ref[0]+(0.00283230959976)*x_ref[1]**o+(-0.933282166805)*x_ref[1]+(-0.857893235883)*x_ref[2]**o+(0.551414074127)*x_ref[2]
        ref[3,1,1,0]=(-0.854435997816)*x_ref[0]**o+(0.706455144263)*x_ref[0]+(0.406237965254)*x_ref[1]**o+(0.192071142415)*x_ref[1]+(0.357966796834)*x_ref[2]**o+(0.887011087987)*x_ref[2]
        ref[3,1,1,1]=(0.832013396996)*x_ref[0]**o+(0.696133072321)*x_ref[0]+(0.682726968712)*x_ref[1]**o+(0.613316240514)*x_ref[1]+(0.12600339887)*x_ref[2]**o+(-0.620088231382)*x_ref[2]
        ref[3,1,2,0]=(-0.178681615221)*x_ref[0]**o+(-0.952835986168)*x_ref[0]+(0.299692693746)*x_ref[1]**o+(0.892451908416)*x_ref[1]+(0.202403273634)*x_ref[2]**o+(0.677212897943)*x_ref[2]
        ref[3,1,2,1]=(0.0592594053342)*x_ref[0]**o+(0.56391153959)*x_ref[0]+(-0.874866627667)*x_ref[1]**o+(0.432102900769)*x_ref[1]+(-0.49073126285)*x_ref[2]**o+(-0.010495860028)*x_ref[2]
        ref[3,2,0,0]=(0.548006445814)*x_ref[0]**o+(0.667313392202)*x_ref[0]+(0.397805355499)*x_ref[1]**o+(-0.788100267808)*x_ref[1]+(0.869234055258)*x_ref[2]**o+(-0.830241520401)*x_ref[2]
        ref[3,2,0,1]=(0.72586139216)*x_ref[0]**o+(-0.931878957878)*x_ref[0]+(-0.94119555858)*x_ref[1]**o+(-0.133544806169)*x_ref[1]+(0.754263323382)*x_ref[2]**o+(-0.446604740157)*x_ref[2]
        ref[3,2,1,0]=(-0.28135830707)*x_ref[0]**o+(-0.478744058518)*x_ref[0]+(0.88219033731)*x_ref[1]**o+(0.460587286945)*x_ref[1]+(-0.849692146096)*x_ref[2]**o+(0.0927701345107)*x_ref[2]
        ref[3,2,1,1]=(-0.681859641611)*x_ref[0]**o+(-0.670164293088)*x_ref[0]+(0.796366374435)*x_ref[1]**o+(0.837230317054)*x_ref[1]+(-0.413283915904)*x_ref[2]**o+(0.779624227187)*x_ref[2]
        ref[3,2,2,0]=(-0.803242771255)*x_ref[0]**o+(-0.0687616271776)*x_ref[0]+(-0.948856027999)*x_ref[1]**o+(0.762568790931)*x_ref[1]+(0.937588292604)*x_ref[2]**o+(-0.0224232611283)*x_ref[2]
        ref[3,2,2,1]=(-0.215752944937)*x_ref[0]**o+(-0.807114296075)*x_ref[0]+(0.970625375964)*x_ref[1]**o+(0.68578124692)*x_ref[1]+(-0.165007047419)*x_ref[2]**o+(0.942718439981)*x_ref[2]
        ref[3,3,0,0]=(0.152001104716)*x_ref[0]**o+(-0.657242342357)*x_ref[0]+(0.559608661705)*x_ref[1]**o+(0.0764403273251)*x_ref[1]+(0.633414754695)*x_ref[2]**o+(0.430777868344)*x_ref[2]
        ref[3,3,0,1]=(-0.898109497759)*x_ref[0]**o+(0.736574249406)*x_ref[0]+(-0.569584184574)*x_ref[1]**o+(-0.145501395506)*x_ref[1]+(-0.412424116475)*x_ref[2]**o+(-0.416350923096)*x_ref[2]
        ref[3,3,1,0]=(0.602022795614)*x_ref[0]**o+(-0.848525569825)*x_ref[0]+(-0.713426287505)*x_ref[1]**o+(-0.829278452687)*x_ref[1]+(-0.491257659316)*x_ref[2]**o+(0.90219908999)*x_ref[2]
        ref[3,3,1,1]=(-0.769110216066)*x_ref[0]**o+(-0.789464037338)*x_ref[0]+(-0.363686172584)*x_ref[1]**o+(-0.86190787068)*x_ref[1]+(0.275438119232)*x_ref[2]**o+(0.285209834248)*x_ref[2]
        ref[3,3,2,0]=(0.351662405853)*x_ref[0]**o+(0.8403302307)*x_ref[0]+(-0.449869332519)*x_ref[1]**o+(0.936043614838)*x_ref[1]+(-0.656908777335)*x_ref[2]**o+(-0.231923872531)*x_ref[2]
        ref[3,3,2,1]=(0.873848945653)*x_ref[0]**o+(0.551429883959)*x_ref[0]+(-0.577942548701)*x_ref[1]**o+(-0.254317163049)*x_ref[1]+(0.64878089878)*x_ref[2]**o+(0.7135339515)*x_ref[2]
        ref[3,4,0,0]=(0.891435610494)*x_ref[0]**o+(0.598893704498)*x_ref[0]+(0.24682094756)*x_ref[1]**o+(0.0895612710338)*x_ref[1]+(-0.513686244953)*x_ref[2]**o+(0.307205505502)*x_ref[2]
        ref[3,4,0,1]=(-0.577752944196)*x_ref[0]**o+(-0.458383751623)*x_ref[0]+(-0.913956988143)*x_ref[1]**o+(0.470857602235)*x_ref[1]+(0.631726850665)*x_ref[2]**o+(-0.112962289904)*x_ref[2]
        ref[3,4,1,0]=(-0.65648177489)*x_ref[0]**o+(-0.843530972747)*x_ref[0]+(-0.0332153723809)*x_ref[1]**o+(0.230484263355)*x_ref[1]+(-0.865687508592)*x_ref[2]**o+(0.122751735082)*x_ref[2]
        ref[3,4,1,1]=(-0.353502640734)*x_ref[0]**o+(-0.816020900282)*x_ref[0]+(0.943688796868)*x_ref[1]**o+(-0.526707881958)*x_ref[1]+(-0.790258298117)*x_ref[2]**o+(-0.748706809042)*x_ref[2]
        ref[3,4,2,0]=(-0.596345912175)*x_ref[0]**o+(0.872771121837)*x_ref[0]+(0.100107395575)*x_ref[1]**o+(0.950482249217)*x_ref[1]+(0.0487634582783)*x_ref[2]**o+(-0.0285218097887)*x_ref[2]
        ref[3,4,2,1]=(-0.827443553933)*x_ref[0]**o+(-0.922564800149)*x_ref[0]+(-0.810354385287)*x_ref[1]**o+(0.630079217293)*x_ref[1]+(-0.316280559723)*x_ref[2]**o+(0.977830435871)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_ReducedFunctionOnContactOne_rank0(self):
      """
      tests interpolate for rank 0 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactOne onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactOne(self.domain)
      x=w.getX()
      arg=Data(0,(),w)
      ref=Data(0,(),w_ref)
      if dim==2:
        arg=(0.142445811229)*x[0]**o+(-0.170111870439)*x[0]+(-0.571150729608)*x[1]**o+(-0.799596368319)*x[1]
        ref=(0.142445811229)*x_ref[0]**o+(-0.170111870439)*x_ref[0]+(-0.571150729608)*x_ref[1]**o+(-0.799596368319)*x_ref[1]
      else:
        arg=(0.131269935442)*x[0]**o+(0.67764849927)*x[0]+(0.199946665)*x[1]**o+(0.775678673404)*x[1]+(-0.818505707274)*x[2]**o+(-0.141160005914)*x[2]
        ref=(0.131269935442)*x_ref[0]**o+(0.67764849927)*x_ref[0]+(0.199946665)*x_ref[1]**o+(0.775678673404)*x_ref[1]+(-0.818505707274)*x_ref[2]**o+(-0.141160005914)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_ReducedFunctionOnContactOne_rank1(self):
      """
      tests interpolate for rank 1 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactOne onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactOne(self.domain)
      x=w.getX()
      arg=Data(0,(2,),w)
      ref=Data(0,(2,),w_ref)
      if dim==2:
        arg[0]=(-0.271030346059)*x[0]**o+(-0.604054986035)*x[0]+(-0.856492962015)*x[1]**o+(0.229582187423)*x[1]
        arg[1]=(-0.152539485929)*x[0]**o+(0.936634973224)*x[0]+(0.805016132937)*x[1]**o+(-0.6832854412)*x[1]
        ref[0]=(-0.271030346059)*x_ref[0]**o+(-0.604054986035)*x_ref[0]+(-0.856492962015)*x_ref[1]**o+(0.229582187423)*x_ref[1]
        ref[1]=(-0.152539485929)*x_ref[0]**o+(0.936634973224)*x_ref[0]+(0.805016132937)*x_ref[1]**o+(-0.6832854412)*x_ref[1]
      else:
        arg[0]=(-0.484580515247)*x[0]**o+(-0.134539997002)*x[0]+(0.589742989514)*x[1]**o+(-0.950803827747)*x[1]+(-0.852976687869)*x[2]**o+(0.0999526415345)*x[2]
        arg[1]=(-0.863620642149)*x[0]**o+(-0.16255544441)*x[0]+(-0.4963232161)*x[1]**o+(0.180988811217)*x[1]+(-0.787186716126)*x[2]**o+(-0.359825851725)*x[2]
        ref[0]=(-0.484580515247)*x_ref[0]**o+(-0.134539997002)*x_ref[0]+(0.589742989514)*x_ref[1]**o+(-0.950803827747)*x_ref[1]+(-0.852976687869)*x_ref[2]**o+(0.0999526415345)*x_ref[2]
        ref[1]=(-0.863620642149)*x_ref[0]**o+(-0.16255544441)*x_ref[0]+(-0.4963232161)*x_ref[1]**o+(0.180988811217)*x_ref[1]+(-0.787186716126)*x_ref[2]**o+(-0.359825851725)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_ReducedFunctionOnContactOne_rank2(self):
      """
      tests interpolate for rank 2 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactOne onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactOne(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5),w)
      ref=Data(0,(4, 5),w_ref)
      if dim==2:
        arg[0,0]=(-0.88064835903)*x[0]**o+(-0.140271048585)*x[0]+(-0.363467917345)*x[1]**o+(0.382530733181)*x[1]
        arg[0,1]=(0.835096280471)*x[0]**o+(0.132077921375)*x[0]+(-0.814392115734)*x[1]**o+(-0.379227781873)*x[1]
        arg[0,2]=(0.257391174596)*x[0]**o+(-0.135017751364)*x[0]+(-0.462760793484)*x[1]**o+(-0.332510860616)*x[1]
        arg[0,3]=(0.303541666957)*x[0]**o+(0.729054207976)*x[0]+(-0.0474210800089)*x[1]**o+(0.213859363809)*x[1]
        arg[0,4]=(0.564466740909)*x[0]**o+(0.684761789227)*x[0]+(0.432934636771)*x[1]**o+(0.261269614075)*x[1]
        arg[1,0]=(0.83080596334)*x[0]**o+(0.968795153198)*x[0]+(0.835115581928)*x[1]**o+(-0.134688885728)*x[1]
        arg[1,1]=(-0.876235093042)*x[0]**o+(-0.43923440557)*x[0]+(-0.641186633503)*x[1]**o+(-0.987129244994)*x[1]
        arg[1,2]=(-0.994007922096)*x[0]**o+(0.534914510762)*x[0]+(0.754420969331)*x[1]**o+(0.241533891038)*x[1]
        arg[1,3]=(-0.196027247194)*x[0]**o+(0.962231147494)*x[0]+(-0.819146467532)*x[1]**o+(0.966024528731)*x[1]
        arg[1,4]=(-0.577014723723)*x[0]**o+(0.350310620855)*x[0]+(0.16266475329)*x[1]**o+(-0.926739533525)*x[1]
        arg[2,0]=(-0.262964780787)*x[0]**o+(-0.824445469277)*x[0]+(-0.982497605841)*x[1]**o+(0.900080022932)*x[1]
        arg[2,1]=(0.54409272334)*x[0]**o+(0.789421020125)*x[0]+(0.672239490644)*x[1]**o+(-0.0306885937363)*x[1]
        arg[2,2]=(-0.130272075526)*x[0]**o+(0.51213564573)*x[0]+(-0.819817699321)*x[1]**o+(-0.259068167835)*x[1]
        arg[2,3]=(-0.61526267092)*x[0]**o+(-0.504362726269)*x[0]+(0.362429515671)*x[1]**o+(-0.0921852465615)*x[1]
        arg[2,4]=(-0.344772989987)*x[0]**o+(0.260747743828)*x[0]+(0.474457474131)*x[1]**o+(0.436008642452)*x[1]
        arg[3,0]=(-0.72740499107)*x[0]**o+(-0.297524430077)*x[0]+(-0.809254720524)*x[1]**o+(-0.230695822532)*x[1]
        arg[3,1]=(-0.0817042383885)*x[0]**o+(-0.324890823023)*x[0]+(0.923635342281)*x[1]**o+(-0.336825317481)*x[1]
        arg[3,2]=(-0.872954045409)*x[0]**o+(0.242165239629)*x[0]+(0.919264824591)*x[1]**o+(0.424247774945)*x[1]
        arg[3,3]=(0.146344350418)*x[0]**o+(-0.705285807962)*x[0]+(-0.0735095471635)*x[1]**o+(-0.464180824267)*x[1]
        arg[3,4]=(0.624085113245)*x[0]**o+(0.719452515121)*x[0]+(0.28331108038)*x[1]**o+(-0.897435702442)*x[1]
        ref[0,0]=(-0.88064835903)*x_ref[0]**o+(-0.140271048585)*x_ref[0]+(-0.363467917345)*x_ref[1]**o+(0.382530733181)*x_ref[1]
        ref[0,1]=(0.835096280471)*x_ref[0]**o+(0.132077921375)*x_ref[0]+(-0.814392115734)*x_ref[1]**o+(-0.379227781873)*x_ref[1]
        ref[0,2]=(0.257391174596)*x_ref[0]**o+(-0.135017751364)*x_ref[0]+(-0.462760793484)*x_ref[1]**o+(-0.332510860616)*x_ref[1]
        ref[0,3]=(0.303541666957)*x_ref[0]**o+(0.729054207976)*x_ref[0]+(-0.0474210800089)*x_ref[1]**o+(0.213859363809)*x_ref[1]
        ref[0,4]=(0.564466740909)*x_ref[0]**o+(0.684761789227)*x_ref[0]+(0.432934636771)*x_ref[1]**o+(0.261269614075)*x_ref[1]
        ref[1,0]=(0.83080596334)*x_ref[0]**o+(0.968795153198)*x_ref[0]+(0.835115581928)*x_ref[1]**o+(-0.134688885728)*x_ref[1]
        ref[1,1]=(-0.876235093042)*x_ref[0]**o+(-0.43923440557)*x_ref[0]+(-0.641186633503)*x_ref[1]**o+(-0.987129244994)*x_ref[1]
        ref[1,2]=(-0.994007922096)*x_ref[0]**o+(0.534914510762)*x_ref[0]+(0.754420969331)*x_ref[1]**o+(0.241533891038)*x_ref[1]
        ref[1,3]=(-0.196027247194)*x_ref[0]**o+(0.962231147494)*x_ref[0]+(-0.819146467532)*x_ref[1]**o+(0.966024528731)*x_ref[1]
        ref[1,4]=(-0.577014723723)*x_ref[0]**o+(0.350310620855)*x_ref[0]+(0.16266475329)*x_ref[1]**o+(-0.926739533525)*x_ref[1]
        ref[2,0]=(-0.262964780787)*x_ref[0]**o+(-0.824445469277)*x_ref[0]+(-0.982497605841)*x_ref[1]**o+(0.900080022932)*x_ref[1]
        ref[2,1]=(0.54409272334)*x_ref[0]**o+(0.789421020125)*x_ref[0]+(0.672239490644)*x_ref[1]**o+(-0.0306885937363)*x_ref[1]
        ref[2,2]=(-0.130272075526)*x_ref[0]**o+(0.51213564573)*x_ref[0]+(-0.819817699321)*x_ref[1]**o+(-0.259068167835)*x_ref[1]
        ref[2,3]=(-0.61526267092)*x_ref[0]**o+(-0.504362726269)*x_ref[0]+(0.362429515671)*x_ref[1]**o+(-0.0921852465615)*x_ref[1]
        ref[2,4]=(-0.344772989987)*x_ref[0]**o+(0.260747743828)*x_ref[0]+(0.474457474131)*x_ref[1]**o+(0.436008642452)*x_ref[1]
        ref[3,0]=(-0.72740499107)*x_ref[0]**o+(-0.297524430077)*x_ref[0]+(-0.809254720524)*x_ref[1]**o+(-0.230695822532)*x_ref[1]
        ref[3,1]=(-0.0817042383885)*x_ref[0]**o+(-0.324890823023)*x_ref[0]+(0.923635342281)*x_ref[1]**o+(-0.336825317481)*x_ref[1]
        ref[3,2]=(-0.872954045409)*x_ref[0]**o+(0.242165239629)*x_ref[0]+(0.919264824591)*x_ref[1]**o+(0.424247774945)*x_ref[1]
        ref[3,3]=(0.146344350418)*x_ref[0]**o+(-0.705285807962)*x_ref[0]+(-0.0735095471635)*x_ref[1]**o+(-0.464180824267)*x_ref[1]
        ref[3,4]=(0.624085113245)*x_ref[0]**o+(0.719452515121)*x_ref[0]+(0.28331108038)*x_ref[1]**o+(-0.897435702442)*x_ref[1]
      else:
        arg[0,0]=(0.47332206873)*x[0]**o+(0.20564453167)*x[0]+(-0.175485469473)*x[1]**o+(-0.25166656853)*x[1]+(-0.242693318919)*x[2]**o+(-0.773417228526)*x[2]
        arg[0,1]=(0.788563488587)*x[0]**o+(0.598162739778)*x[0]+(-0.736463777722)*x[1]**o+(-0.161555683484)*x[1]+(0.480782882175)*x[2]**o+(-0.500552084345)*x[2]
        arg[0,2]=(-0.207893050318)*x[0]**o+(0.206221241577)*x[0]+(0.306329036151)*x[1]**o+(0.585237890094)*x[1]+(0.0276469766188)*x[2]**o+(0.0502503670529)*x[2]
        arg[0,3]=(0.914950534537)*x[0]**o+(0.401315333176)*x[0]+(0.245224334898)*x[1]**o+(0.526049350134)*x[1]+(-0.735617023846)*x[2]**o+(-0.674692140531)*x[2]
        arg[0,4]=(-0.359566330106)*x[0]**o+(-0.279627730979)*x[0]+(-0.08398829656)*x[1]**o+(-0.789678621983)*x[1]+(-0.357374658443)*x[2]**o+(0.795724504883)*x[2]
        arg[1,0]=(-0.0715278193161)*x[0]**o+(-0.562120168502)*x[0]+(0.428614380101)*x[1]**o+(0.115408373591)*x[1]+(-0.154674397872)*x[2]**o+(0.233468972682)*x[2]
        arg[1,1]=(0.707445640039)*x[0]**o+(-0.719649334705)*x[0]+(-0.349743923514)*x[1]**o+(0.369262203159)*x[1]+(0.712786695739)*x[2]**o+(-0.962450154391)*x[2]
        arg[1,2]=(-0.306983094408)*x[0]**o+(-0.933322822964)*x[0]+(-0.525515899443)*x[1]**o+(0.642320930433)*x[1]+(-0.950421802297)*x[2]**o+(-0.392142385683)*x[2]
        arg[1,3]=(-0.765158324235)*x[0]**o+(-0.362010493356)*x[0]+(-0.918673930909)*x[1]**o+(0.72772784184)*x[1]+(-0.392848438408)*x[2]**o+(0.566119231816)*x[2]
        arg[1,4]=(0.429984662315)*x[0]**o+(-0.165476415697)*x[0]+(0.778539911928)*x[1]**o+(0.411518911472)*x[1]+(0.418323432603)*x[2]**o+(-0.846495913126)*x[2]
        arg[2,0]=(-0.836556750858)*x[0]**o+(0.976027316021)*x[0]+(-0.556499096672)*x[1]**o+(0.320038395915)*x[1]+(0.377663884516)*x[2]**o+(0.491202727923)*x[2]
        arg[2,1]=(-0.329597826951)*x[0]**o+(-0.555819984044)*x[0]+(0.110029358927)*x[1]**o+(-0.737070187836)*x[1]+(-0.587142473737)*x[2]**o+(-0.0388834177112)*x[2]
        arg[2,2]=(0.442251030646)*x[0]**o+(0.285010574287)*x[0]+(0.603267537331)*x[1]**o+(-0.0776884302384)*x[1]+(-0.123541484708)*x[2]**o+(0.44717125529)*x[2]
        arg[2,3]=(0.474458930313)*x[0]**o+(0.415947401939)*x[0]+(0.381539850696)*x[1]**o+(-0.307590909422)*x[1]+(0.488348451586)*x[2]**o+(0.336883810306)*x[2]
        arg[2,4]=(0.804466542611)*x[0]**o+(0.891337158669)*x[0]+(-0.599119487307)*x[1]**o+(-0.92047732727)*x[1]+(-0.251489480657)*x[2]**o+(-0.266483991526)*x[2]
        arg[3,0]=(-0.733633520827)*x[0]**o+(0.494805537624)*x[0]+(-0.117702558274)*x[1]**o+(-0.000461693303442)*x[1]+(-0.102033981202)*x[2]**o+(-0.772865985272)*x[2]
        arg[3,1]=(-0.636901276465)*x[0]**o+(-0.436330072041)*x[0]+(0.403048850222)*x[1]**o+(-0.94737100009)*x[1]+(0.781059874101)*x[2]**o+(-0.952781516037)*x[2]
        arg[3,2]=(-0.489372336164)*x[0]**o+(0.221211043152)*x[0]+(0.793975373906)*x[1]**o+(0.424349965288)*x[1]+(0.453332885662)*x[2]**o+(-0.513541753005)*x[2]
        arg[3,3]=(-0.700899677836)*x[0]**o+(-0.984271896306)*x[0]+(-0.663909486898)*x[1]**o+(0.461302174316)*x[1]+(-0.906457707724)*x[2]**o+(-0.0440003164038)*x[2]
        arg[3,4]=(0.838326295366)*x[0]**o+(-0.483407593962)*x[0]+(-0.122458752397)*x[1]**o+(0.607337951095)*x[1]+(0.750481384434)*x[2]**o+(-0.677155633204)*x[2]
        ref[0,0]=(0.47332206873)*x_ref[0]**o+(0.20564453167)*x_ref[0]+(-0.175485469473)*x_ref[1]**o+(-0.25166656853)*x_ref[1]+(-0.242693318919)*x_ref[2]**o+(-0.773417228526)*x_ref[2]
        ref[0,1]=(0.788563488587)*x_ref[0]**o+(0.598162739778)*x_ref[0]+(-0.736463777722)*x_ref[1]**o+(-0.161555683484)*x_ref[1]+(0.480782882175)*x_ref[2]**o+(-0.500552084345)*x_ref[2]
        ref[0,2]=(-0.207893050318)*x_ref[0]**o+(0.206221241577)*x_ref[0]+(0.306329036151)*x_ref[1]**o+(0.585237890094)*x_ref[1]+(0.0276469766188)*x_ref[2]**o+(0.0502503670529)*x_ref[2]
        ref[0,3]=(0.914950534537)*x_ref[0]**o+(0.401315333176)*x_ref[0]+(0.245224334898)*x_ref[1]**o+(0.526049350134)*x_ref[1]+(-0.735617023846)*x_ref[2]**o+(-0.674692140531)*x_ref[2]
        ref[0,4]=(-0.359566330106)*x_ref[0]**o+(-0.279627730979)*x_ref[0]+(-0.08398829656)*x_ref[1]**o+(-0.789678621983)*x_ref[1]+(-0.357374658443)*x_ref[2]**o+(0.795724504883)*x_ref[2]
        ref[1,0]=(-0.0715278193161)*x_ref[0]**o+(-0.562120168502)*x_ref[0]+(0.428614380101)*x_ref[1]**o+(0.115408373591)*x_ref[1]+(-0.154674397872)*x_ref[2]**o+(0.233468972682)*x_ref[2]
        ref[1,1]=(0.707445640039)*x_ref[0]**o+(-0.719649334705)*x_ref[0]+(-0.349743923514)*x_ref[1]**o+(0.369262203159)*x_ref[1]+(0.712786695739)*x_ref[2]**o+(-0.962450154391)*x_ref[2]
        ref[1,2]=(-0.306983094408)*x_ref[0]**o+(-0.933322822964)*x_ref[0]+(-0.525515899443)*x_ref[1]**o+(0.642320930433)*x_ref[1]+(-0.950421802297)*x_ref[2]**o+(-0.392142385683)*x_ref[2]
        ref[1,3]=(-0.765158324235)*x_ref[0]**o+(-0.362010493356)*x_ref[0]+(-0.918673930909)*x_ref[1]**o+(0.72772784184)*x_ref[1]+(-0.392848438408)*x_ref[2]**o+(0.566119231816)*x_ref[2]
        ref[1,4]=(0.429984662315)*x_ref[0]**o+(-0.165476415697)*x_ref[0]+(0.778539911928)*x_ref[1]**o+(0.411518911472)*x_ref[1]+(0.418323432603)*x_ref[2]**o+(-0.846495913126)*x_ref[2]
        ref[2,0]=(-0.836556750858)*x_ref[0]**o+(0.976027316021)*x_ref[0]+(-0.556499096672)*x_ref[1]**o+(0.320038395915)*x_ref[1]+(0.377663884516)*x_ref[2]**o+(0.491202727923)*x_ref[2]
        ref[2,1]=(-0.329597826951)*x_ref[0]**o+(-0.555819984044)*x_ref[0]+(0.110029358927)*x_ref[1]**o+(-0.737070187836)*x_ref[1]+(-0.587142473737)*x_ref[2]**o+(-0.0388834177112)*x_ref[2]
        ref[2,2]=(0.442251030646)*x_ref[0]**o+(0.285010574287)*x_ref[0]+(0.603267537331)*x_ref[1]**o+(-0.0776884302384)*x_ref[1]+(-0.123541484708)*x_ref[2]**o+(0.44717125529)*x_ref[2]
        ref[2,3]=(0.474458930313)*x_ref[0]**o+(0.415947401939)*x_ref[0]+(0.381539850696)*x_ref[1]**o+(-0.307590909422)*x_ref[1]+(0.488348451586)*x_ref[2]**o+(0.336883810306)*x_ref[2]
        ref[2,4]=(0.804466542611)*x_ref[0]**o+(0.891337158669)*x_ref[0]+(-0.599119487307)*x_ref[1]**o+(-0.92047732727)*x_ref[1]+(-0.251489480657)*x_ref[2]**o+(-0.266483991526)*x_ref[2]
        ref[3,0]=(-0.733633520827)*x_ref[0]**o+(0.494805537624)*x_ref[0]+(-0.117702558274)*x_ref[1]**o+(-0.000461693303442)*x_ref[1]+(-0.102033981202)*x_ref[2]**o+(-0.772865985272)*x_ref[2]
        ref[3,1]=(-0.636901276465)*x_ref[0]**o+(-0.436330072041)*x_ref[0]+(0.403048850222)*x_ref[1]**o+(-0.94737100009)*x_ref[1]+(0.781059874101)*x_ref[2]**o+(-0.952781516037)*x_ref[2]
        ref[3,2]=(-0.489372336164)*x_ref[0]**o+(0.221211043152)*x_ref[0]+(0.793975373906)*x_ref[1]**o+(0.424349965288)*x_ref[1]+(0.453332885662)*x_ref[2]**o+(-0.513541753005)*x_ref[2]
        ref[3,3]=(-0.700899677836)*x_ref[0]**o+(-0.984271896306)*x_ref[0]+(-0.663909486898)*x_ref[1]**o+(0.461302174316)*x_ref[1]+(-0.906457707724)*x_ref[2]**o+(-0.0440003164038)*x_ref[2]
        ref[3,4]=(0.838326295366)*x_ref[0]**o+(-0.483407593962)*x_ref[0]+(-0.122458752397)*x_ref[1]**o+(0.607337951095)*x_ref[1]+(0.750481384434)*x_ref[2]**o+(-0.677155633204)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_ReducedFunctionOnContactOne_rank3(self):
      """
      tests interpolate for rank 3 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactOne onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactOne(self.domain)
      x=w.getX()
      arg=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2),w_ref)
      if dim==2:
        arg[0,0,0]=(0.111473275114)*x[0]**o+(-0.552264448532)*x[0]+(0.67142534913)*x[1]**o+(-0.642305751076)*x[1]
        arg[0,0,1]=(-0.431075800974)*x[0]**o+(0.911382078454)*x[0]+(0.546541644612)*x[1]**o+(-0.67679652536)*x[1]
        arg[0,1,0]=(0.103056292292)*x[0]**o+(-0.640858804416)*x[0]+(0.159596964992)*x[1]**o+(-0.237782007316)*x[1]
        arg[0,1,1]=(0.975898170389)*x[0]**o+(0.255916326383)*x[0]+(-0.692770462322)*x[1]**o+(0.786355114846)*x[1]
        arg[1,0,0]=(-0.669378370759)*x[0]**o+(-0.0153788709146)*x[0]+(0.924383188557)*x[1]**o+(0.885037556829)*x[1]
        arg[1,0,1]=(-0.509891819881)*x[0]**o+(-0.377779514697)*x[0]+(-0.343176473036)*x[1]**o+(-0.39978416725)*x[1]
        arg[1,1,0]=(0.457959467665)*x[0]**o+(0.786511967341)*x[0]+(0.256561665146)*x[1]**o+(-0.261300099643)*x[1]
        arg[1,1,1]=(0.219754515189)*x[0]**o+(-0.131747762867)*x[0]+(0.736017818893)*x[1]**o+(-0.450963700533)*x[1]
        arg[2,0,0]=(-0.182946832789)*x[0]**o+(-0.103064731935)*x[0]+(-0.223584334222)*x[1]**o+(-0.578088201628)*x[1]
        arg[2,0,1]=(-0.830744157425)*x[0]**o+(0.590627167207)*x[0]+(-0.538048378049)*x[1]**o+(-0.0529793352741)*x[1]
        arg[2,1,0]=(-0.307908670388)*x[0]**o+(-0.929987734045)*x[0]+(-0.167190563092)*x[1]**o+(0.126466628227)*x[1]
        arg[2,1,1]=(-0.812045378916)*x[0]**o+(0.676973938072)*x[0]+(0.528243909204)*x[1]**o+(-0.594241387691)*x[1]
        arg[3,0,0]=(-0.0198606164114)*x[0]**o+(-0.00574708685554)*x[0]+(0.908946312745)*x[1]**o+(0.775054099788)*x[1]
        arg[3,0,1]=(-0.810078120518)*x[0]**o+(-0.960585888485)*x[0]+(-0.0747272427198)*x[1]**o+(0.164107641817)*x[1]
        arg[3,1,0]=(-0.535791917139)*x[0]**o+(0.0424226440074)*x[0]+(0.701479263902)*x[1]**o+(0.787745423088)*x[1]
        arg[3,1,1]=(-0.896532152793)*x[0]**o+(-0.0224566920251)*x[0]+(-0.554655626689)*x[1]**o+(0.41012867952)*x[1]
        arg[4,0,0]=(0.972433817496)*x[0]**o+(-0.440662536033)*x[0]+(-0.910456530943)*x[1]**o+(-0.333429535028)*x[1]
        arg[4,0,1]=(-0.415824313942)*x[0]**o+(0.00255557712922)*x[0]+(0.304897639195)*x[1]**o+(-0.866600848052)*x[1]
        arg[4,1,0]=(-0.866277739076)*x[0]**o+(0.282976169776)*x[0]+(0.477528278501)*x[1]**o+(-0.233506497695)*x[1]
        arg[4,1,1]=(-0.12729626381)*x[0]**o+(-0.723204796785)*x[0]+(0.0856495713442)*x[1]**o+(-0.214819437731)*x[1]
        arg[5,0,0]=(0.208133503644)*x[0]**o+(-0.955529202895)*x[0]+(0.0291569127831)*x[1]**o+(0.489913918305)*x[1]
        arg[5,0,1]=(0.96923667477)*x[0]**o+(0.543776525315)*x[0]+(0.38666450029)*x[1]**o+(-0.193493550272)*x[1]
        arg[5,1,0]=(-0.0912940016888)*x[0]**o+(-0.477561072681)*x[0]+(0.474816021613)*x[1]**o+(-0.252664229253)*x[1]
        arg[5,1,1]=(0.748337265159)*x[0]**o+(-0.706194454618)*x[0]+(-0.504466281507)*x[1]**o+(0.582500817679)*x[1]
        ref[0,0,0]=(0.111473275114)*x_ref[0]**o+(-0.552264448532)*x_ref[0]+(0.67142534913)*x_ref[1]**o+(-0.642305751076)*x_ref[1]
        ref[0,0,1]=(-0.431075800974)*x_ref[0]**o+(0.911382078454)*x_ref[0]+(0.546541644612)*x_ref[1]**o+(-0.67679652536)*x_ref[1]
        ref[0,1,0]=(0.103056292292)*x_ref[0]**o+(-0.640858804416)*x_ref[0]+(0.159596964992)*x_ref[1]**o+(-0.237782007316)*x_ref[1]
        ref[0,1,1]=(0.975898170389)*x_ref[0]**o+(0.255916326383)*x_ref[0]+(-0.692770462322)*x_ref[1]**o+(0.786355114846)*x_ref[1]
        ref[1,0,0]=(-0.669378370759)*x_ref[0]**o+(-0.0153788709146)*x_ref[0]+(0.924383188557)*x_ref[1]**o+(0.885037556829)*x_ref[1]
        ref[1,0,1]=(-0.509891819881)*x_ref[0]**o+(-0.377779514697)*x_ref[0]+(-0.343176473036)*x_ref[1]**o+(-0.39978416725)*x_ref[1]
        ref[1,1,0]=(0.457959467665)*x_ref[0]**o+(0.786511967341)*x_ref[0]+(0.256561665146)*x_ref[1]**o+(-0.261300099643)*x_ref[1]
        ref[1,1,1]=(0.219754515189)*x_ref[0]**o+(-0.131747762867)*x_ref[0]+(0.736017818893)*x_ref[1]**o+(-0.450963700533)*x_ref[1]
        ref[2,0,0]=(-0.182946832789)*x_ref[0]**o+(-0.103064731935)*x_ref[0]+(-0.223584334222)*x_ref[1]**o+(-0.578088201628)*x_ref[1]
        ref[2,0,1]=(-0.830744157425)*x_ref[0]**o+(0.590627167207)*x_ref[0]+(-0.538048378049)*x_ref[1]**o+(-0.0529793352741)*x_ref[1]
        ref[2,1,0]=(-0.307908670388)*x_ref[0]**o+(-0.929987734045)*x_ref[0]+(-0.167190563092)*x_ref[1]**o+(0.126466628227)*x_ref[1]
        ref[2,1,1]=(-0.812045378916)*x_ref[0]**o+(0.676973938072)*x_ref[0]+(0.528243909204)*x_ref[1]**o+(-0.594241387691)*x_ref[1]
        ref[3,0,0]=(-0.0198606164114)*x_ref[0]**o+(-0.00574708685554)*x_ref[0]+(0.908946312745)*x_ref[1]**o+(0.775054099788)*x_ref[1]
        ref[3,0,1]=(-0.810078120518)*x_ref[0]**o+(-0.960585888485)*x_ref[0]+(-0.0747272427198)*x_ref[1]**o+(0.164107641817)*x_ref[1]
        ref[3,1,0]=(-0.535791917139)*x_ref[0]**o+(0.0424226440074)*x_ref[0]+(0.701479263902)*x_ref[1]**o+(0.787745423088)*x_ref[1]
        ref[3,1,1]=(-0.896532152793)*x_ref[0]**o+(-0.0224566920251)*x_ref[0]+(-0.554655626689)*x_ref[1]**o+(0.41012867952)*x_ref[1]
        ref[4,0,0]=(0.972433817496)*x_ref[0]**o+(-0.440662536033)*x_ref[0]+(-0.910456530943)*x_ref[1]**o+(-0.333429535028)*x_ref[1]
        ref[4,0,1]=(-0.415824313942)*x_ref[0]**o+(0.00255557712922)*x_ref[0]+(0.304897639195)*x_ref[1]**o+(-0.866600848052)*x_ref[1]
        ref[4,1,0]=(-0.866277739076)*x_ref[0]**o+(0.282976169776)*x_ref[0]+(0.477528278501)*x_ref[1]**o+(-0.233506497695)*x_ref[1]
        ref[4,1,1]=(-0.12729626381)*x_ref[0]**o+(-0.723204796785)*x_ref[0]+(0.0856495713442)*x_ref[1]**o+(-0.214819437731)*x_ref[1]
        ref[5,0,0]=(0.208133503644)*x_ref[0]**o+(-0.955529202895)*x_ref[0]+(0.0291569127831)*x_ref[1]**o+(0.489913918305)*x_ref[1]
        ref[5,0,1]=(0.96923667477)*x_ref[0]**o+(0.543776525315)*x_ref[0]+(0.38666450029)*x_ref[1]**o+(-0.193493550272)*x_ref[1]
        ref[5,1,0]=(-0.0912940016888)*x_ref[0]**o+(-0.477561072681)*x_ref[0]+(0.474816021613)*x_ref[1]**o+(-0.252664229253)*x_ref[1]
        ref[5,1,1]=(0.748337265159)*x_ref[0]**o+(-0.706194454618)*x_ref[0]+(-0.504466281507)*x_ref[1]**o+(0.582500817679)*x_ref[1]
      else:
        arg[0,0,0]=(-0.179972720095)*x[0]**o+(-0.546715268523)*x[0]+(0.835867431319)*x[1]**o+(0.626798499259)*x[1]+(0.397356013278)*x[2]**o+(0.0771586784158)*x[2]
        arg[0,0,1]=(0.625676031039)*x[0]**o+(0.65843849475)*x[0]+(0.456469199307)*x[1]**o+(-0.970709759479)*x[1]+(-0.699606143992)*x[2]**o+(0.76429400724)*x[2]
        arg[0,1,0]=(0.613916424165)*x[0]**o+(0.556778588303)*x[0]+(-0.509658743529)*x[1]**o+(-0.978718968218)*x[1]+(-0.603928369428)*x[2]**o+(-0.550813953692)*x[2]
        arg[0,1,1]=(-0.935214755436)*x[0]**o+(0.44745901813)*x[0]+(-0.582828846453)*x[1]**o+(-0.181547183233)*x[1]+(-0.183912053485)*x[2]**o+(0.446152494296)*x[2]
        arg[1,0,0]=(-0.486345219779)*x[0]**o+(0.753529436833)*x[0]+(0.138157856552)*x[1]**o+(-0.874849020539)*x[1]+(0.374396543792)*x[2]**o+(0.26548456046)*x[2]
        arg[1,0,1]=(0.492908753036)*x[0]**o+(-0.418118685105)*x[0]+(0.880817569823)*x[1]**o+(-0.827691145524)*x[1]+(0.238633156133)*x[2]**o+(-0.463687731341)*x[2]
        arg[1,1,0]=(-0.397373578442)*x[0]**o+(-0.316321105607)*x[0]+(0.647511225459)*x[1]**o+(-0.82553752195)*x[1]+(-0.715549303171)*x[2]**o+(0.804623957459)*x[2]
        arg[1,1,1]=(0.414743130234)*x[0]**o+(0.0201794413683)*x[0]+(-0.636337599584)*x[1]**o+(-0.230053616026)*x[1]+(-0.4302153311)*x[2]**o+(-0.387468029626)*x[2]
        arg[2,0,0]=(0.715100517336)*x[0]**o+(-0.4286120178)*x[0]+(0.0987858542117)*x[1]**o+(0.815093470301)*x[1]+(0.293455353257)*x[2]**o+(0.200576090379)*x[2]
        arg[2,0,1]=(-0.158330015822)*x[0]**o+(0.363339023457)*x[0]+(-0.796055594971)*x[1]**o+(-0.213299205306)*x[1]+(-0.136620988812)*x[2]**o+(0.10733886947)*x[2]
        arg[2,1,0]=(-0.258729405118)*x[0]**o+(0.0669553480038)*x[0]+(0.997663166613)*x[1]**o+(0.00398861063931)*x[1]+(-0.989459856803)*x[2]**o+(0.932687384325)*x[2]
        arg[2,1,1]=(-0.974431322319)*x[0]**o+(-0.486447431804)*x[0]+(0.871784873347)*x[1]**o+(-0.978904038765)*x[1]+(0.647341201506)*x[2]**o+(-0.269683556528)*x[2]
        arg[3,0,0]=(-0.369507398073)*x[0]**o+(-0.772085607074)*x[0]+(0.417550881937)*x[1]**o+(0.271743719652)*x[1]+(0.881807154737)*x[2]**o+(0.54725428039)*x[2]
        arg[3,0,1]=(0.440089335941)*x[0]**o+(0.0844171421294)*x[0]+(0.926389704362)*x[1]**o+(-0.0709182998297)*x[1]+(0.903978660317)*x[2]**o+(-0.160556206716)*x[2]
        arg[3,1,0]=(-0.49936391282)*x[0]**o+(-0.632753931881)*x[0]+(-0.604686084554)*x[1]**o+(-0.954205199531)*x[1]+(-0.535102184845)*x[2]**o+(0.621683814123)*x[2]
        arg[3,1,1]=(-0.117790973132)*x[0]**o+(-0.649091727896)*x[0]+(0.925682916785)*x[1]**o+(0.852053515469)*x[1]+(-0.479218429897)*x[2]**o+(0.426823478575)*x[2]
        arg[4,0,0]=(0.454174564319)*x[0]**o+(-0.855975647406)*x[0]+(0.775972969288)*x[1]**o+(0.267900999893)*x[1]+(0.221387501995)*x[2]**o+(-0.26413791201)*x[2]
        arg[4,0,1]=(0.875191302431)*x[0]**o+(-0.362782502152)*x[0]+(0.24600070339)*x[1]**o+(-0.566709533298)*x[1]+(-0.82187455531)*x[2]**o+(-0.943929640301)*x[2]
        arg[4,1,0]=(-0.409313259973)*x[0]**o+(-0.784833509001)*x[0]+(-0.939690505056)*x[1]**o+(-0.0736280226974)*x[1]+(0.939072745486)*x[2]**o+(0.611697159959)*x[2]
        arg[4,1,1]=(0.893374246808)*x[0]**o+(0.36113513561)*x[0]+(0.227899271271)*x[1]**o+(-0.847221657012)*x[1]+(-0.0106368947417)*x[2]**o+(0.0358896817595)*x[2]
        arg[5,0,0]=(0.0329230524924)*x[0]**o+(0.0565192084014)*x[0]+(-0.68398448658)*x[1]**o+(0.909035618076)*x[1]+(-0.900060440872)*x[2]**o+(-0.481014787833)*x[2]
        arg[5,0,1]=(0.404966692391)*x[0]**o+(0.647468764373)*x[0]+(-0.723847499346)*x[1]**o+(-0.0740896957606)*x[1]+(-0.00628515079923)*x[2]**o+(0.906667863539)*x[2]
        arg[5,1,0]=(0.983181781531)*x[0]**o+(-0.297325990236)*x[0]+(0.00489064543352)*x[1]**o+(0.667560767038)*x[1]+(-0.665768141301)*x[2]**o+(0.772761393608)*x[2]
        arg[5,1,1]=(-0.496581691354)*x[0]**o+(0.0121337979505)*x[0]+(0.492281382726)*x[1]**o+(0.545232714025)*x[1]+(0.418321481532)*x[2]**o+(0.489635969764)*x[2]
        ref[0,0,0]=(-0.179972720095)*x_ref[0]**o+(-0.546715268523)*x_ref[0]+(0.835867431319)*x_ref[1]**o+(0.626798499259)*x_ref[1]+(0.397356013278)*x_ref[2]**o+(0.0771586784158)*x_ref[2]
        ref[0,0,1]=(0.625676031039)*x_ref[0]**o+(0.65843849475)*x_ref[0]+(0.456469199307)*x_ref[1]**o+(-0.970709759479)*x_ref[1]+(-0.699606143992)*x_ref[2]**o+(0.76429400724)*x_ref[2]
        ref[0,1,0]=(0.613916424165)*x_ref[0]**o+(0.556778588303)*x_ref[0]+(-0.509658743529)*x_ref[1]**o+(-0.978718968218)*x_ref[1]+(-0.603928369428)*x_ref[2]**o+(-0.550813953692)*x_ref[2]
        ref[0,1,1]=(-0.935214755436)*x_ref[0]**o+(0.44745901813)*x_ref[0]+(-0.582828846453)*x_ref[1]**o+(-0.181547183233)*x_ref[1]+(-0.183912053485)*x_ref[2]**o+(0.446152494296)*x_ref[2]
        ref[1,0,0]=(-0.486345219779)*x_ref[0]**o+(0.753529436833)*x_ref[0]+(0.138157856552)*x_ref[1]**o+(-0.874849020539)*x_ref[1]+(0.374396543792)*x_ref[2]**o+(0.26548456046)*x_ref[2]
        ref[1,0,1]=(0.492908753036)*x_ref[0]**o+(-0.418118685105)*x_ref[0]+(0.880817569823)*x_ref[1]**o+(-0.827691145524)*x_ref[1]+(0.238633156133)*x_ref[2]**o+(-0.463687731341)*x_ref[2]
        ref[1,1,0]=(-0.397373578442)*x_ref[0]**o+(-0.316321105607)*x_ref[0]+(0.647511225459)*x_ref[1]**o+(-0.82553752195)*x_ref[1]+(-0.715549303171)*x_ref[2]**o+(0.804623957459)*x_ref[2]
        ref[1,1,1]=(0.414743130234)*x_ref[0]**o+(0.0201794413683)*x_ref[0]+(-0.636337599584)*x_ref[1]**o+(-0.230053616026)*x_ref[1]+(-0.4302153311)*x_ref[2]**o+(-0.387468029626)*x_ref[2]
        ref[2,0,0]=(0.715100517336)*x_ref[0]**o+(-0.4286120178)*x_ref[0]+(0.0987858542117)*x_ref[1]**o+(0.815093470301)*x_ref[1]+(0.293455353257)*x_ref[2]**o+(0.200576090379)*x_ref[2]
        ref[2,0,1]=(-0.158330015822)*x_ref[0]**o+(0.363339023457)*x_ref[0]+(-0.796055594971)*x_ref[1]**o+(-0.213299205306)*x_ref[1]+(-0.136620988812)*x_ref[2]**o+(0.10733886947)*x_ref[2]
        ref[2,1,0]=(-0.258729405118)*x_ref[0]**o+(0.0669553480038)*x_ref[0]+(0.997663166613)*x_ref[1]**o+(0.00398861063931)*x_ref[1]+(-0.989459856803)*x_ref[2]**o+(0.932687384325)*x_ref[2]
        ref[2,1,1]=(-0.974431322319)*x_ref[0]**o+(-0.486447431804)*x_ref[0]+(0.871784873347)*x_ref[1]**o+(-0.978904038765)*x_ref[1]+(0.647341201506)*x_ref[2]**o+(-0.269683556528)*x_ref[2]
        ref[3,0,0]=(-0.369507398073)*x_ref[0]**o+(-0.772085607074)*x_ref[0]+(0.417550881937)*x_ref[1]**o+(0.271743719652)*x_ref[1]+(0.881807154737)*x_ref[2]**o+(0.54725428039)*x_ref[2]
        ref[3,0,1]=(0.440089335941)*x_ref[0]**o+(0.0844171421294)*x_ref[0]+(0.926389704362)*x_ref[1]**o+(-0.0709182998297)*x_ref[1]+(0.903978660317)*x_ref[2]**o+(-0.160556206716)*x_ref[2]
        ref[3,1,0]=(-0.49936391282)*x_ref[0]**o+(-0.632753931881)*x_ref[0]+(-0.604686084554)*x_ref[1]**o+(-0.954205199531)*x_ref[1]+(-0.535102184845)*x_ref[2]**o+(0.621683814123)*x_ref[2]
        ref[3,1,1]=(-0.117790973132)*x_ref[0]**o+(-0.649091727896)*x_ref[0]+(0.925682916785)*x_ref[1]**o+(0.852053515469)*x_ref[1]+(-0.479218429897)*x_ref[2]**o+(0.426823478575)*x_ref[2]
        ref[4,0,0]=(0.454174564319)*x_ref[0]**o+(-0.855975647406)*x_ref[0]+(0.775972969288)*x_ref[1]**o+(0.267900999893)*x_ref[1]+(0.221387501995)*x_ref[2]**o+(-0.26413791201)*x_ref[2]
        ref[4,0,1]=(0.875191302431)*x_ref[0]**o+(-0.362782502152)*x_ref[0]+(0.24600070339)*x_ref[1]**o+(-0.566709533298)*x_ref[1]+(-0.82187455531)*x_ref[2]**o+(-0.943929640301)*x_ref[2]
        ref[4,1,0]=(-0.409313259973)*x_ref[0]**o+(-0.784833509001)*x_ref[0]+(-0.939690505056)*x_ref[1]**o+(-0.0736280226974)*x_ref[1]+(0.939072745486)*x_ref[2]**o+(0.611697159959)*x_ref[2]
        ref[4,1,1]=(0.893374246808)*x_ref[0]**o+(0.36113513561)*x_ref[0]+(0.227899271271)*x_ref[1]**o+(-0.847221657012)*x_ref[1]+(-0.0106368947417)*x_ref[2]**o+(0.0358896817595)*x_ref[2]
        ref[5,0,0]=(0.0329230524924)*x_ref[0]**o+(0.0565192084014)*x_ref[0]+(-0.68398448658)*x_ref[1]**o+(0.909035618076)*x_ref[1]+(-0.900060440872)*x_ref[2]**o+(-0.481014787833)*x_ref[2]
        ref[5,0,1]=(0.404966692391)*x_ref[0]**o+(0.647468764373)*x_ref[0]+(-0.723847499346)*x_ref[1]**o+(-0.0740896957606)*x_ref[1]+(-0.00628515079923)*x_ref[2]**o+(0.906667863539)*x_ref[2]
        ref[5,1,0]=(0.983181781531)*x_ref[0]**o+(-0.297325990236)*x_ref[0]+(0.00489064543352)*x_ref[1]**o+(0.667560767038)*x_ref[1]+(-0.665768141301)*x_ref[2]**o+(0.772761393608)*x_ref[2]
        ref[5,1,1]=(-0.496581691354)*x_ref[0]**o+(0.0121337979505)*x_ref[0]+(0.492281382726)*x_ref[1]**o+(0.545232714025)*x_ref[1]+(0.418321481532)*x_ref[2]**o+(0.489635969764)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromData_ReducedFunctionOnContactOne_rank4(self):
      """
      tests interpolate for rank 4 Data onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactOne onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactOne(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5, 3, 2),w)
      ref=Data(0,(4, 5, 3, 2),w_ref)
      if dim==2:
        arg[0,0,0,0]=(-0.282394950702)*x[0]**o+(0.836580157584)*x[0]+(0.337652262648)*x[1]**o+(-0.442694263325)*x[1]
        arg[0,0,0,1]=(0.85150774028)*x[0]**o+(0.998297296593)*x[0]+(-0.710029954342)*x[1]**o+(0.0816168287837)*x[1]
        arg[0,0,1,0]=(-0.636435102838)*x[0]**o+(-0.613479863553)*x[0]+(-0.861506513168)*x[1]**o+(-0.242998961478)*x[1]
        arg[0,0,1,1]=(-0.00671863978521)*x[0]**o+(0.909404084068)*x[0]+(-0.734956067026)*x[1]**o+(0.256530538264)*x[1]
        arg[0,0,2,0]=(-0.646631608999)*x[0]**o+(-0.521130455843)*x[0]+(-0.44328404557)*x[1]**o+(-0.156860325817)*x[1]
        arg[0,0,2,1]=(-0.817550008949)*x[0]**o+(0.702414497795)*x[0]+(0.946449680763)*x[1]**o+(-0.61576361925)*x[1]
        arg[0,1,0,0]=(-0.527206691595)*x[0]**o+(-0.43469216984)*x[0]+(-0.108455901154)*x[1]**o+(0.435646962156)*x[1]
        arg[0,1,0,1]=(-0.148522219535)*x[0]**o+(0.146912249755)*x[0]+(-0.414934945665)*x[1]**o+(-0.422754988714)*x[1]
        arg[0,1,1,0]=(0.177760972947)*x[0]**o+(-0.938522530862)*x[0]+(0.566229652264)*x[1]**o+(0.255269743628)*x[1]
        arg[0,1,1,1]=(-0.317807933792)*x[0]**o+(0.432181451246)*x[0]+(-0.358821513362)*x[1]**o+(-0.470517232585)*x[1]
        arg[0,1,2,0]=(0.486536430227)*x[0]**o+(-0.533653809349)*x[0]+(-0.126612514045)*x[1]**o+(0.103088115487)*x[1]
        arg[0,1,2,1]=(-0.995976559589)*x[0]**o+(0.965537167421)*x[0]+(-0.351307699531)*x[1]**o+(-0.684367434666)*x[1]
        arg[0,2,0,0]=(0.976008613768)*x[0]**o+(-0.890573007002)*x[0]+(-0.440709423944)*x[1]**o+(0.315724459367)*x[1]
        arg[0,2,0,1]=(-0.595275811962)*x[0]**o+(-0.599634144509)*x[0]+(0.646466002346)*x[1]**o+(-0.94529568651)*x[1]
        arg[0,2,1,0]=(0.977691521821)*x[0]**o+(-0.0730997643682)*x[0]+(-0.338897498737)*x[1]**o+(0.246308678824)*x[1]
        arg[0,2,1,1]=(-0.111317410732)*x[0]**o+(-0.574381793398)*x[0]+(-0.0314431899843)*x[1]**o+(0.393105047096)*x[1]
        arg[0,2,2,0]=(0.886320020757)*x[0]**o+(-0.785042003544)*x[0]+(0.82609269979)*x[1]**o+(-0.677169484337)*x[1]
        arg[0,2,2,1]=(0.811027138079)*x[0]**o+(-0.375108470948)*x[0]+(0.781472464253)*x[1]**o+(0.217640794143)*x[1]
        arg[0,3,0,0]=(0.264884117091)*x[0]**o+(0.116591319557)*x[0]+(0.282667679763)*x[1]**o+(0.111123454337)*x[1]
        arg[0,3,0,1]=(0.0451626985922)*x[0]**o+(-0.93771279728)*x[0]+(0.129171764072)*x[1]**o+(0.598367954162)*x[1]
        arg[0,3,1,0]=(0.905243511086)*x[0]**o+(0.0837062573586)*x[0]+(0.0657223248433)*x[1]**o+(-0.746644690072)*x[1]
        arg[0,3,1,1]=(-0.0547856755485)*x[0]**o+(-0.22157806755)*x[0]+(0.124623625108)*x[1]**o+(0.0436590157081)*x[1]
        arg[0,3,2,0]=(-0.558323744577)*x[0]**o+(-0.646043519007)*x[0]+(0.991885458004)*x[1]**o+(-0.634140808163)*x[1]
        arg[0,3,2,1]=(-0.27247983028)*x[0]**o+(0.803906404485)*x[0]+(-0.816826609924)*x[1]**o+(-0.969532783336)*x[1]
        arg[0,4,0,0]=(0.638733842891)*x[0]**o+(-0.972793270208)*x[0]+(0.714611008099)*x[1]**o+(0.586988595847)*x[1]
        arg[0,4,0,1]=(0.63584760734)*x[0]**o+(-0.653740236747)*x[0]+(-0.596304731446)*x[1]**o+(0.42519786429)*x[1]
        arg[0,4,1,0]=(0.0665562973529)*x[0]**o+(0.362152092093)*x[0]+(0.732530335739)*x[1]**o+(0.618536326941)*x[1]
        arg[0,4,1,1]=(0.486060667608)*x[0]**o+(0.59489013279)*x[0]+(-0.547237017498)*x[1]**o+(0.0371894705857)*x[1]
        arg[0,4,2,0]=(0.395505844397)*x[0]**o+(-0.869090434535)*x[0]+(-0.606640827437)*x[1]**o+(0.269995459083)*x[1]
        arg[0,4,2,1]=(0.304671547176)*x[0]**o+(-0.515772523849)*x[0]+(0.546114300876)*x[1]**o+(0.530247404341)*x[1]
        arg[1,0,0,0]=(0.408332677433)*x[0]**o+(-0.0778807359918)*x[0]+(-0.803337489297)*x[1]**o+(-0.123835718422)*x[1]
        arg[1,0,0,1]=(-0.00441620254944)*x[0]**o+(0.0321259598831)*x[0]+(-0.998381508472)*x[1]**o+(0.800767747511)*x[1]
        arg[1,0,1,0]=(-0.923683057418)*x[0]**o+(0.579405114902)*x[0]+(-0.939639823585)*x[1]**o+(-0.138423358134)*x[1]
        arg[1,0,1,1]=(-0.558420234848)*x[0]**o+(0.133316569813)*x[0]+(-0.294134717329)*x[1]**o+(-0.750897749044)*x[1]
        arg[1,0,2,0]=(0.478951298545)*x[0]**o+(-0.807173978799)*x[0]+(0.852438146763)*x[1]**o+(-0.103814229563)*x[1]
        arg[1,0,2,1]=(-0.875250726603)*x[0]**o+(-0.934803428491)*x[0]+(0.143352873672)*x[1]**o+(0.600934881206)*x[1]
        arg[1,1,0,0]=(-0.0539476321018)*x[0]**o+(0.194177663417)*x[0]+(-0.862877991009)*x[1]**o+(-0.438337993664)*x[1]
        arg[1,1,0,1]=(-0.623379355474)*x[0]**o+(-0.0746994739748)*x[0]+(0.365005438956)*x[1]**o+(-0.431737688205)*x[1]
        arg[1,1,1,0]=(0.0936054461735)*x[0]**o+(-0.974594269446)*x[0]+(-0.121917598428)*x[1]**o+(-0.84429083727)*x[1]
        arg[1,1,1,1]=(0.967763348672)*x[0]**o+(0.331698826475)*x[0]+(0.66448515877)*x[1]**o+(0.00214985930661)*x[1]
        arg[1,1,2,0]=(-0.312146580762)*x[0]**o+(0.998152287706)*x[0]+(-0.474242942777)*x[1]**o+(0.637827347846)*x[1]
        arg[1,1,2,1]=(-0.556765443436)*x[0]**o+(0.227327929774)*x[0]+(0.739457845569)*x[1]**o+(-0.615070017663)*x[1]
        arg[1,2,0,0]=(-0.23030660552)*x[0]**o+(0.750479465953)*x[0]+(0.278215408392)*x[1]**o+(-0.640618698224)*x[1]
        arg[1,2,0,1]=(-0.0713217101129)*x[0]**o+(0.500737477887)*x[0]+(-0.972834770005)*x[1]**o+(-0.420386056033)*x[1]
        arg[1,2,1,0]=(-0.497322989847)*x[0]**o+(-0.528685124841)*x[0]+(-0.934386348496)*x[1]**o+(0.191930553839)*x[1]
        arg[1,2,1,1]=(0.124362419781)*x[0]**o+(-0.0457849787456)*x[0]+(-0.360596154712)*x[1]**o+(0.799736253715)*x[1]
        arg[1,2,2,0]=(-0.346294878893)*x[0]**o+(0.427772309361)*x[0]+(-0.0642666791012)*x[1]**o+(-0.214442238311)*x[1]
        arg[1,2,2,1]=(0.24434837411)*x[0]**o+(-0.913373159786)*x[0]+(0.158750862391)*x[1]**o+(-0.544328807576)*x[1]
        arg[1,3,0,0]=(0.870847002687)*x[0]**o+(-0.466884462301)*x[0]+(-0.343358829445)*x[1]**o+(-0.180956419766)*x[1]
        arg[1,3,0,1]=(-0.813518850863)*x[0]**o+(0.960810467501)*x[0]+(0.35091416904)*x[1]**o+(0.84534544948)*x[1]
        arg[1,3,1,0]=(-0.284687751218)*x[0]**o+(-0.274583519355)*x[0]+(0.338892764102)*x[1]**o+(-0.145142443753)*x[1]
        arg[1,3,1,1]=(0.921064262137)*x[0]**o+(0.152552421678)*x[0]+(0.827247693375)*x[1]**o+(-0.569529848619)*x[1]
        arg[1,3,2,0]=(0.133567630356)*x[0]**o+(-0.0429397358364)*x[0]+(-0.175383170663)*x[1]**o+(0.584407631334)*x[1]
        arg[1,3,2,1]=(0.203880167371)*x[0]**o+(-0.422978073825)*x[0]+(-0.210109267588)*x[1]**o+(-0.902327373219)*x[1]
        arg[1,4,0,0]=(-0.566386864876)*x[0]**o+(-0.139795085483)*x[0]+(-0.00852913464238)*x[1]**o+(0.569151381884)*x[1]
        arg[1,4,0,1]=(-0.937256625212)*x[0]**o+(0.563438253128)*x[0]+(-0.586182238715)*x[1]**o+(-0.566169678723)*x[1]
        arg[1,4,1,0]=(-0.73296890712)*x[0]**o+(0.359249594406)*x[0]+(0.66050790616)*x[1]**o+(-0.367919797426)*x[1]
        arg[1,4,1,1]=(-0.81165734248)*x[0]**o+(0.460592817838)*x[0]+(0.953573909492)*x[1]**o+(0.329659021274)*x[1]
        arg[1,4,2,0]=(0.345994483595)*x[0]**o+(-0.18547324343)*x[0]+(0.669452926238)*x[1]**o+(0.42513249272)*x[1]
        arg[1,4,2,1]=(0.250677642616)*x[0]**o+(0.466118496803)*x[0]+(-0.776608543595)*x[1]**o+(0.449144243682)*x[1]
        arg[2,0,0,0]=(0.244172909021)*x[0]**o+(0.0311068322898)*x[0]+(-0.909348039508)*x[1]**o+(0.376500719414)*x[1]
        arg[2,0,0,1]=(-0.033516640667)*x[0]**o+(0.698424718328)*x[0]+(-0.282814785542)*x[1]**o+(0.61118022947)*x[1]
        arg[2,0,1,0]=(-0.946672649538)*x[0]**o+(0.814325244591)*x[0]+(-0.353215562825)*x[1]**o+(-0.138150257429)*x[1]
        arg[2,0,1,1]=(-0.886509051692)*x[0]**o+(-0.0468334385819)*x[0]+(0.71984719189)*x[1]**o+(-0.324744971493)*x[1]
        arg[2,0,2,0]=(-0.780921647053)*x[0]**o+(-0.401418496932)*x[0]+(0.647847274332)*x[1]**o+(-0.831834322278)*x[1]
        arg[2,0,2,1]=(0.835155004862)*x[0]**o+(0.684459986902)*x[0]+(-0.301244240677)*x[1]**o+(-0.161292780187)*x[1]
        arg[2,1,0,0]=(-0.652199398394)*x[0]**o+(0.62480722178)*x[0]+(-0.474535712914)*x[1]**o+(0.926061797766)*x[1]
        arg[2,1,0,1]=(-0.813365478568)*x[0]**o+(-0.38795792417)*x[0]+(0.378161353697)*x[1]**o+(0.70581806211)*x[1]
        arg[2,1,1,0]=(0.729265594934)*x[0]**o+(-0.469397842359)*x[0]+(0.642472199926)*x[1]**o+(-0.779527283011)*x[1]
        arg[2,1,1,1]=(0.0517342917574)*x[0]**o+(-0.414111007917)*x[0]+(-0.89437375241)*x[1]**o+(-0.830119585842)*x[1]
        arg[2,1,2,0]=(0.769898486257)*x[0]**o+(0.791725624832)*x[0]+(0.297519329396)*x[1]**o+(-0.959462006611)*x[1]
        arg[2,1,2,1]=(0.817802144251)*x[0]**o+(0.69851258911)*x[0]+(-0.174401735228)*x[1]**o+(0.13449696498)*x[1]
        arg[2,2,0,0]=(0.547972662256)*x[0]**o+(-0.66491929979)*x[0]+(0.330783041672)*x[1]**o+(0.816196730606)*x[1]
        arg[2,2,0,1]=(0.245971097763)*x[0]**o+(0.213087481481)*x[0]+(-0.840766291392)*x[1]**o+(0.388414557376)*x[1]
        arg[2,2,1,0]=(0.523151676444)*x[0]**o+(0.854817486027)*x[0]+(-0.53570188528)*x[1]**o+(-0.68149362242)*x[1]
        arg[2,2,1,1]=(-0.984200247787)*x[0]**o+(0.83324365092)*x[0]+(-0.896184408816)*x[1]**o+(0.102276243487)*x[1]
        arg[2,2,2,0]=(-0.216357164308)*x[0]**o+(-0.597984882107)*x[0]+(-0.205435921511)*x[1]**o+(0.484947652759)*x[1]
        arg[2,2,2,1]=(-0.650046253868)*x[0]**o+(-0.624878186105)*x[0]+(0.786519914406)*x[1]**o+(0.28027273095)*x[1]
        arg[2,3,0,0]=(-0.0821718346579)*x[0]**o+(-0.687049917467)*x[0]+(0.380792333736)*x[1]**o+(0.151185837715)*x[1]
        arg[2,3,0,1]=(0.61796113854)*x[0]**o+(0.844624413065)*x[0]+(-0.980619401098)*x[1]**o+(-0.854924862641)*x[1]
        arg[2,3,1,0]=(0.812293906883)*x[0]**o+(-0.225593208768)*x[0]+(0.964175374163)*x[1]**o+(0.848850064926)*x[1]
        arg[2,3,1,1]=(0.376453572937)*x[0]**o+(0.994818812059)*x[0]+(0.58872018303)*x[1]**o+(0.68332473361)*x[1]
        arg[2,3,2,0]=(-0.00948421972234)*x[0]**o+(-0.680685462738)*x[0]+(-0.767076278167)*x[1]**o+(0.210165437284)*x[1]
        arg[2,3,2,1]=(-0.455199698263)*x[0]**o+(-0.763430401419)*x[0]+(0.183773350212)*x[1]**o+(0.43105940923)*x[1]
        arg[2,4,0,0]=(-0.673962025988)*x[0]**o+(-0.679073069725)*x[0]+(0.11018431715)*x[1]**o+(0.349020527482)*x[1]
        arg[2,4,0,1]=(-0.876691282076)*x[0]**o+(0.920902226631)*x[0]+(-0.477417470246)*x[1]**o+(0.836417982216)*x[1]
        arg[2,4,1,0]=(0.567966851024)*x[0]**o+(0.583241431716)*x[0]+(-0.271851271008)*x[1]**o+(0.678727902863)*x[1]
        arg[2,4,1,1]=(0.707400094)*x[0]**o+(0.802820856444)*x[0]+(0.84518937616)*x[1]**o+(-0.0267193533403)*x[1]
        arg[2,4,2,0]=(0.213268920036)*x[0]**o+(0.780721415328)*x[0]+(0.839292948154)*x[1]**o+(-0.0376334744443)*x[1]
        arg[2,4,2,1]=(0.136255687031)*x[0]**o+(-0.582766278693)*x[0]+(0.257662521321)*x[1]**o+(0.808337521979)*x[1]
        arg[3,0,0,0]=(-0.524068770822)*x[0]**o+(0.942236878842)*x[0]+(0.889496255793)*x[1]**o+(-0.904810742393)*x[1]
        arg[3,0,0,1]=(0.780157465475)*x[0]**o+(-0.763811668832)*x[0]+(-0.517482070639)*x[1]**o+(0.662770780676)*x[1]
        arg[3,0,1,0]=(-0.693227564812)*x[0]**o+(-0.763506211841)*x[0]+(-0.427709881126)*x[1]**o+(0.159109827204)*x[1]
        arg[3,0,1,1]=(0.644975629084)*x[0]**o+(-0.928112761899)*x[0]+(-0.530772740083)*x[1]**o+(-0.723174790087)*x[1]
        arg[3,0,2,0]=(-0.923467274097)*x[0]**o+(0.806128117288)*x[0]+(0.177779363207)*x[1]**o+(0.862243885542)*x[1]
        arg[3,0,2,1]=(-0.0823252901358)*x[0]**o+(0.289022916284)*x[0]+(-0.518508771021)*x[1]**o+(0.174564769845)*x[1]
        arg[3,1,0,0]=(0.608837578444)*x[0]**o+(0.898216758361)*x[0]+(-0.844730109761)*x[1]**o+(0.745974173207)*x[1]
        arg[3,1,0,1]=(-0.11776411078)*x[0]**o+(0.186256502356)*x[0]+(0.565289400333)*x[1]**o+(-0.776173716206)*x[1]
        arg[3,1,1,0]=(-0.170736580152)*x[0]**o+(0.829581661683)*x[0]+(0.310135833859)*x[1]**o+(-0.509629545687)*x[1]
        arg[3,1,1,1]=(-0.225246122751)*x[0]**o+(0.586754377294)*x[0]+(-0.726064725635)*x[1]**o+(0.136952605704)*x[1]
        arg[3,1,2,0]=(-0.287932115478)*x[0]**o+(0.858031396153)*x[0]+(-0.493441745353)*x[1]**o+(0.601827664994)*x[1]
        arg[3,1,2,1]=(0.665503842862)*x[0]**o+(-0.515998970452)*x[0]+(-0.890055558624)*x[1]**o+(-0.0446371451457)*x[1]
        arg[3,2,0,0]=(-0.33741557585)*x[0]**o+(-0.416183752333)*x[0]+(-0.92430073538)*x[1]**o+(-0.486265090904)*x[1]
        arg[3,2,0,1]=(0.910860925994)*x[0]**o+(-0.896869755389)*x[0]+(0.0634658185324)*x[1]**o+(-0.383136455739)*x[1]
        arg[3,2,1,0]=(0.386839297976)*x[0]**o+(-0.190659927071)*x[0]+(-0.304846978185)*x[1]**o+(-0.92705511899)*x[1]
        arg[3,2,1,1]=(-0.0197812018683)*x[0]**o+(-0.479083095775)*x[0]+(0.502489976089)*x[1]**o+(-0.859440665475)*x[1]
        arg[3,2,2,0]=(-0.507672045299)*x[0]**o+(0.498689676072)*x[0]+(-0.719735804645)*x[1]**o+(-0.522053306674)*x[1]
        arg[3,2,2,1]=(0.46207629211)*x[0]**o+(-0.496697066039)*x[0]+(0.994069339904)*x[1]**o+(0.873222754449)*x[1]
        arg[3,3,0,0]=(0.126410029692)*x[0]**o+(0.51067050196)*x[0]+(-0.978384824088)*x[1]**o+(-0.68606930503)*x[1]
        arg[3,3,0,1]=(0.376888157008)*x[0]**o+(-0.14015764025)*x[0]+(0.523667700758)*x[1]**o+(0.0277726855258)*x[1]
        arg[3,3,1,0]=(-0.105865233294)*x[0]**o+(-0.0515230741203)*x[0]+(0.203766525036)*x[1]**o+(-0.0431893014387)*x[1]
        arg[3,3,1,1]=(-0.404351448486)*x[0]**o+(-0.639161429157)*x[0]+(-0.957341091708)*x[1]**o+(-0.103346466984)*x[1]
        arg[3,3,2,0]=(-0.0523508780623)*x[0]**o+(-0.563589575544)*x[0]+(0.158487727325)*x[1]**o+(-0.519455492178)*x[1]
        arg[3,3,2,1]=(-0.823286342218)*x[0]**o+(-0.493663499535)*x[0]+(-0.409825574064)*x[1]**o+(-0.0406101279806)*x[1]
        arg[3,4,0,0]=(0.926212910387)*x[0]**o+(-0.0944507963)*x[0]+(0.640740499391)*x[1]**o+(-0.106969076047)*x[1]
        arg[3,4,0,1]=(-0.912044993941)*x[0]**o+(-0.495323777918)*x[0]+(-0.595831845108)*x[1]**o+(-0.313723426318)*x[1]
        arg[3,4,1,0]=(-0.0730898742548)*x[0]**o+(0.0524008484492)*x[0]+(-0.948232935497)*x[1]**o+(0.952670824572)*x[1]
        arg[3,4,1,1]=(-0.81869438138)*x[0]**o+(-0.894654266952)*x[0]+(0.414078621872)*x[1]**o+(-0.876690817847)*x[1]
        arg[3,4,2,0]=(0.00688155078441)*x[0]**o+(0.845964586551)*x[0]+(0.0183149371892)*x[1]**o+(-0.882931394537)*x[1]
        arg[3,4,2,1]=(-0.734830097149)*x[0]**o+(-0.37824588859)*x[0]+(-0.815402620311)*x[1]**o+(-0.357905685517)*x[1]
        ref[0,0,0,0]=(-0.282394950702)*x_ref[0]**o+(0.836580157584)*x_ref[0]+(0.337652262648)*x_ref[1]**o+(-0.442694263325)*x_ref[1]
        ref[0,0,0,1]=(0.85150774028)*x_ref[0]**o+(0.998297296593)*x_ref[0]+(-0.710029954342)*x_ref[1]**o+(0.0816168287837)*x_ref[1]
        ref[0,0,1,0]=(-0.636435102838)*x_ref[0]**o+(-0.613479863553)*x_ref[0]+(-0.861506513168)*x_ref[1]**o+(-0.242998961478)*x_ref[1]
        ref[0,0,1,1]=(-0.00671863978521)*x_ref[0]**o+(0.909404084068)*x_ref[0]+(-0.734956067026)*x_ref[1]**o+(0.256530538264)*x_ref[1]
        ref[0,0,2,0]=(-0.646631608999)*x_ref[0]**o+(-0.521130455843)*x_ref[0]+(-0.44328404557)*x_ref[1]**o+(-0.156860325817)*x_ref[1]
        ref[0,0,2,1]=(-0.817550008949)*x_ref[0]**o+(0.702414497795)*x_ref[0]+(0.946449680763)*x_ref[1]**o+(-0.61576361925)*x_ref[1]
        ref[0,1,0,0]=(-0.527206691595)*x_ref[0]**o+(-0.43469216984)*x_ref[0]+(-0.108455901154)*x_ref[1]**o+(0.435646962156)*x_ref[1]
        ref[0,1,0,1]=(-0.148522219535)*x_ref[0]**o+(0.146912249755)*x_ref[0]+(-0.414934945665)*x_ref[1]**o+(-0.422754988714)*x_ref[1]
        ref[0,1,1,0]=(0.177760972947)*x_ref[0]**o+(-0.938522530862)*x_ref[0]+(0.566229652264)*x_ref[1]**o+(0.255269743628)*x_ref[1]
        ref[0,1,1,1]=(-0.317807933792)*x_ref[0]**o+(0.432181451246)*x_ref[0]+(-0.358821513362)*x_ref[1]**o+(-0.470517232585)*x_ref[1]
        ref[0,1,2,0]=(0.486536430227)*x_ref[0]**o+(-0.533653809349)*x_ref[0]+(-0.126612514045)*x_ref[1]**o+(0.103088115487)*x_ref[1]
        ref[0,1,2,1]=(-0.995976559589)*x_ref[0]**o+(0.965537167421)*x_ref[0]+(-0.351307699531)*x_ref[1]**o+(-0.684367434666)*x_ref[1]
        ref[0,2,0,0]=(0.976008613768)*x_ref[0]**o+(-0.890573007002)*x_ref[0]+(-0.440709423944)*x_ref[1]**o+(0.315724459367)*x_ref[1]
        ref[0,2,0,1]=(-0.595275811962)*x_ref[0]**o+(-0.599634144509)*x_ref[0]+(0.646466002346)*x_ref[1]**o+(-0.94529568651)*x_ref[1]
        ref[0,2,1,0]=(0.977691521821)*x_ref[0]**o+(-0.0730997643682)*x_ref[0]+(-0.338897498737)*x_ref[1]**o+(0.246308678824)*x_ref[1]
        ref[0,2,1,1]=(-0.111317410732)*x_ref[0]**o+(-0.574381793398)*x_ref[0]+(-0.0314431899843)*x_ref[1]**o+(0.393105047096)*x_ref[1]
        ref[0,2,2,0]=(0.886320020757)*x_ref[0]**o+(-0.785042003544)*x_ref[0]+(0.82609269979)*x_ref[1]**o+(-0.677169484337)*x_ref[1]
        ref[0,2,2,1]=(0.811027138079)*x_ref[0]**o+(-0.375108470948)*x_ref[0]+(0.781472464253)*x_ref[1]**o+(0.217640794143)*x_ref[1]
        ref[0,3,0,0]=(0.264884117091)*x_ref[0]**o+(0.116591319557)*x_ref[0]+(0.282667679763)*x_ref[1]**o+(0.111123454337)*x_ref[1]
        ref[0,3,0,1]=(0.0451626985922)*x_ref[0]**o+(-0.93771279728)*x_ref[0]+(0.129171764072)*x_ref[1]**o+(0.598367954162)*x_ref[1]
        ref[0,3,1,0]=(0.905243511086)*x_ref[0]**o+(0.0837062573586)*x_ref[0]+(0.0657223248433)*x_ref[1]**o+(-0.746644690072)*x_ref[1]
        ref[0,3,1,1]=(-0.0547856755485)*x_ref[0]**o+(-0.22157806755)*x_ref[0]+(0.124623625108)*x_ref[1]**o+(0.0436590157081)*x_ref[1]
        ref[0,3,2,0]=(-0.558323744577)*x_ref[0]**o+(-0.646043519007)*x_ref[0]+(0.991885458004)*x_ref[1]**o+(-0.634140808163)*x_ref[1]
        ref[0,3,2,1]=(-0.27247983028)*x_ref[0]**o+(0.803906404485)*x_ref[0]+(-0.816826609924)*x_ref[1]**o+(-0.969532783336)*x_ref[1]
        ref[0,4,0,0]=(0.638733842891)*x_ref[0]**o+(-0.972793270208)*x_ref[0]+(0.714611008099)*x_ref[1]**o+(0.586988595847)*x_ref[1]
        ref[0,4,0,1]=(0.63584760734)*x_ref[0]**o+(-0.653740236747)*x_ref[0]+(-0.596304731446)*x_ref[1]**o+(0.42519786429)*x_ref[1]
        ref[0,4,1,0]=(0.0665562973529)*x_ref[0]**o+(0.362152092093)*x_ref[0]+(0.732530335739)*x_ref[1]**o+(0.618536326941)*x_ref[1]
        ref[0,4,1,1]=(0.486060667608)*x_ref[0]**o+(0.59489013279)*x_ref[0]+(-0.547237017498)*x_ref[1]**o+(0.0371894705857)*x_ref[1]
        ref[0,4,2,0]=(0.395505844397)*x_ref[0]**o+(-0.869090434535)*x_ref[0]+(-0.606640827437)*x_ref[1]**o+(0.269995459083)*x_ref[1]
        ref[0,4,2,1]=(0.304671547176)*x_ref[0]**o+(-0.515772523849)*x_ref[0]+(0.546114300876)*x_ref[1]**o+(0.530247404341)*x_ref[1]
        ref[1,0,0,0]=(0.408332677433)*x_ref[0]**o+(-0.0778807359918)*x_ref[0]+(-0.803337489297)*x_ref[1]**o+(-0.123835718422)*x_ref[1]
        ref[1,0,0,1]=(-0.00441620254944)*x_ref[0]**o+(0.0321259598831)*x_ref[0]+(-0.998381508472)*x_ref[1]**o+(0.800767747511)*x_ref[1]
        ref[1,0,1,0]=(-0.923683057418)*x_ref[0]**o+(0.579405114902)*x_ref[0]+(-0.939639823585)*x_ref[1]**o+(-0.138423358134)*x_ref[1]
        ref[1,0,1,1]=(-0.558420234848)*x_ref[0]**o+(0.133316569813)*x_ref[0]+(-0.294134717329)*x_ref[1]**o+(-0.750897749044)*x_ref[1]
        ref[1,0,2,0]=(0.478951298545)*x_ref[0]**o+(-0.807173978799)*x_ref[0]+(0.852438146763)*x_ref[1]**o+(-0.103814229563)*x_ref[1]
        ref[1,0,2,1]=(-0.875250726603)*x_ref[0]**o+(-0.934803428491)*x_ref[0]+(0.143352873672)*x_ref[1]**o+(0.600934881206)*x_ref[1]
        ref[1,1,0,0]=(-0.0539476321018)*x_ref[0]**o+(0.194177663417)*x_ref[0]+(-0.862877991009)*x_ref[1]**o+(-0.438337993664)*x_ref[1]
        ref[1,1,0,1]=(-0.623379355474)*x_ref[0]**o+(-0.0746994739748)*x_ref[0]+(0.365005438956)*x_ref[1]**o+(-0.431737688205)*x_ref[1]
        ref[1,1,1,0]=(0.0936054461735)*x_ref[0]**o+(-0.974594269446)*x_ref[0]+(-0.121917598428)*x_ref[1]**o+(-0.84429083727)*x_ref[1]
        ref[1,1,1,1]=(0.967763348672)*x_ref[0]**o+(0.331698826475)*x_ref[0]+(0.66448515877)*x_ref[1]**o+(0.00214985930661)*x_ref[1]
        ref[1,1,2,0]=(-0.312146580762)*x_ref[0]**o+(0.998152287706)*x_ref[0]+(-0.474242942777)*x_ref[1]**o+(0.637827347846)*x_ref[1]
        ref[1,1,2,1]=(-0.556765443436)*x_ref[0]**o+(0.227327929774)*x_ref[0]+(0.739457845569)*x_ref[1]**o+(-0.615070017663)*x_ref[1]
        ref[1,2,0,0]=(-0.23030660552)*x_ref[0]**o+(0.750479465953)*x_ref[0]+(0.278215408392)*x_ref[1]**o+(-0.640618698224)*x_ref[1]
        ref[1,2,0,1]=(-0.0713217101129)*x_ref[0]**o+(0.500737477887)*x_ref[0]+(-0.972834770005)*x_ref[1]**o+(-0.420386056033)*x_ref[1]
        ref[1,2,1,0]=(-0.497322989847)*x_ref[0]**o+(-0.528685124841)*x_ref[0]+(-0.934386348496)*x_ref[1]**o+(0.191930553839)*x_ref[1]
        ref[1,2,1,1]=(0.124362419781)*x_ref[0]**o+(-0.0457849787456)*x_ref[0]+(-0.360596154712)*x_ref[1]**o+(0.799736253715)*x_ref[1]
        ref[1,2,2,0]=(-0.346294878893)*x_ref[0]**o+(0.427772309361)*x_ref[0]+(-0.0642666791012)*x_ref[1]**o+(-0.214442238311)*x_ref[1]
        ref[1,2,2,1]=(0.24434837411)*x_ref[0]**o+(-0.913373159786)*x_ref[0]+(0.158750862391)*x_ref[1]**o+(-0.544328807576)*x_ref[1]
        ref[1,3,0,0]=(0.870847002687)*x_ref[0]**o+(-0.466884462301)*x_ref[0]+(-0.343358829445)*x_ref[1]**o+(-0.180956419766)*x_ref[1]
        ref[1,3,0,1]=(-0.813518850863)*x_ref[0]**o+(0.960810467501)*x_ref[0]+(0.35091416904)*x_ref[1]**o+(0.84534544948)*x_ref[1]
        ref[1,3,1,0]=(-0.284687751218)*x_ref[0]**o+(-0.274583519355)*x_ref[0]+(0.338892764102)*x_ref[1]**o+(-0.145142443753)*x_ref[1]
        ref[1,3,1,1]=(0.921064262137)*x_ref[0]**o+(0.152552421678)*x_ref[0]+(0.827247693375)*x_ref[1]**o+(-0.569529848619)*x_ref[1]
        ref[1,3,2,0]=(0.133567630356)*x_ref[0]**o+(-0.0429397358364)*x_ref[0]+(-0.175383170663)*x_ref[1]**o+(0.584407631334)*x_ref[1]
        ref[1,3,2,1]=(0.203880167371)*x_ref[0]**o+(-0.422978073825)*x_ref[0]+(-0.210109267588)*x_ref[1]**o+(-0.902327373219)*x_ref[1]
        ref[1,4,0,0]=(-0.566386864876)*x_ref[0]**o+(-0.139795085483)*x_ref[0]+(-0.00852913464238)*x_ref[1]**o+(0.569151381884)*x_ref[1]
        ref[1,4,0,1]=(-0.937256625212)*x_ref[0]**o+(0.563438253128)*x_ref[0]+(-0.586182238715)*x_ref[1]**o+(-0.566169678723)*x_ref[1]
        ref[1,4,1,0]=(-0.73296890712)*x_ref[0]**o+(0.359249594406)*x_ref[0]+(0.66050790616)*x_ref[1]**o+(-0.367919797426)*x_ref[1]
        ref[1,4,1,1]=(-0.81165734248)*x_ref[0]**o+(0.460592817838)*x_ref[0]+(0.953573909492)*x_ref[1]**o+(0.329659021274)*x_ref[1]
        ref[1,4,2,0]=(0.345994483595)*x_ref[0]**o+(-0.18547324343)*x_ref[0]+(0.669452926238)*x_ref[1]**o+(0.42513249272)*x_ref[1]
        ref[1,4,2,1]=(0.250677642616)*x_ref[0]**o+(0.466118496803)*x_ref[0]+(-0.776608543595)*x_ref[1]**o+(0.449144243682)*x_ref[1]
        ref[2,0,0,0]=(0.244172909021)*x_ref[0]**o+(0.0311068322898)*x_ref[0]+(-0.909348039508)*x_ref[1]**o+(0.376500719414)*x_ref[1]
        ref[2,0,0,1]=(-0.033516640667)*x_ref[0]**o+(0.698424718328)*x_ref[0]+(-0.282814785542)*x_ref[1]**o+(0.61118022947)*x_ref[1]
        ref[2,0,1,0]=(-0.946672649538)*x_ref[0]**o+(0.814325244591)*x_ref[0]+(-0.353215562825)*x_ref[1]**o+(-0.138150257429)*x_ref[1]
        ref[2,0,1,1]=(-0.886509051692)*x_ref[0]**o+(-0.0468334385819)*x_ref[0]+(0.71984719189)*x_ref[1]**o+(-0.324744971493)*x_ref[1]
        ref[2,0,2,0]=(-0.780921647053)*x_ref[0]**o+(-0.401418496932)*x_ref[0]+(0.647847274332)*x_ref[1]**o+(-0.831834322278)*x_ref[1]
        ref[2,0,2,1]=(0.835155004862)*x_ref[0]**o+(0.684459986902)*x_ref[0]+(-0.301244240677)*x_ref[1]**o+(-0.161292780187)*x_ref[1]
        ref[2,1,0,0]=(-0.652199398394)*x_ref[0]**o+(0.62480722178)*x_ref[0]+(-0.474535712914)*x_ref[1]**o+(0.926061797766)*x_ref[1]
        ref[2,1,0,1]=(-0.813365478568)*x_ref[0]**o+(-0.38795792417)*x_ref[0]+(0.378161353697)*x_ref[1]**o+(0.70581806211)*x_ref[1]
        ref[2,1,1,0]=(0.729265594934)*x_ref[0]**o+(-0.469397842359)*x_ref[0]+(0.642472199926)*x_ref[1]**o+(-0.779527283011)*x_ref[1]
        ref[2,1,1,1]=(0.0517342917574)*x_ref[0]**o+(-0.414111007917)*x_ref[0]+(-0.89437375241)*x_ref[1]**o+(-0.830119585842)*x_ref[1]
        ref[2,1,2,0]=(0.769898486257)*x_ref[0]**o+(0.791725624832)*x_ref[0]+(0.297519329396)*x_ref[1]**o+(-0.959462006611)*x_ref[1]
        ref[2,1,2,1]=(0.817802144251)*x_ref[0]**o+(0.69851258911)*x_ref[0]+(-0.174401735228)*x_ref[1]**o+(0.13449696498)*x_ref[1]
        ref[2,2,0,0]=(0.547972662256)*x_ref[0]**o+(-0.66491929979)*x_ref[0]+(0.330783041672)*x_ref[1]**o+(0.816196730606)*x_ref[1]
        ref[2,2,0,1]=(0.245971097763)*x_ref[0]**o+(0.213087481481)*x_ref[0]+(-0.840766291392)*x_ref[1]**o+(0.388414557376)*x_ref[1]
        ref[2,2,1,0]=(0.523151676444)*x_ref[0]**o+(0.854817486027)*x_ref[0]+(-0.53570188528)*x_ref[1]**o+(-0.68149362242)*x_ref[1]
        ref[2,2,1,1]=(-0.984200247787)*x_ref[0]**o+(0.83324365092)*x_ref[0]+(-0.896184408816)*x_ref[1]**o+(0.102276243487)*x_ref[1]
        ref[2,2,2,0]=(-0.216357164308)*x_ref[0]**o+(-0.597984882107)*x_ref[0]+(-0.205435921511)*x_ref[1]**o+(0.484947652759)*x_ref[1]
        ref[2,2,2,1]=(-0.650046253868)*x_ref[0]**o+(-0.624878186105)*x_ref[0]+(0.786519914406)*x_ref[1]**o+(0.28027273095)*x_ref[1]
        ref[2,3,0,0]=(-0.0821718346579)*x_ref[0]**o+(-0.687049917467)*x_ref[0]+(0.380792333736)*x_ref[1]**o+(0.151185837715)*x_ref[1]
        ref[2,3,0,1]=(0.61796113854)*x_ref[0]**o+(0.844624413065)*x_ref[0]+(-0.980619401098)*x_ref[1]**o+(-0.854924862641)*x_ref[1]
        ref[2,3,1,0]=(0.812293906883)*x_ref[0]**o+(-0.225593208768)*x_ref[0]+(0.964175374163)*x_ref[1]**o+(0.848850064926)*x_ref[1]
        ref[2,3,1,1]=(0.376453572937)*x_ref[0]**o+(0.994818812059)*x_ref[0]+(0.58872018303)*x_ref[1]**o+(0.68332473361)*x_ref[1]
        ref[2,3,2,0]=(-0.00948421972234)*x_ref[0]**o+(-0.680685462738)*x_ref[0]+(-0.767076278167)*x_ref[1]**o+(0.210165437284)*x_ref[1]
        ref[2,3,2,1]=(-0.455199698263)*x_ref[0]**o+(-0.763430401419)*x_ref[0]+(0.183773350212)*x_ref[1]**o+(0.43105940923)*x_ref[1]
        ref[2,4,0,0]=(-0.673962025988)*x_ref[0]**o+(-0.679073069725)*x_ref[0]+(0.11018431715)*x_ref[1]**o+(0.349020527482)*x_ref[1]
        ref[2,4,0,1]=(-0.876691282076)*x_ref[0]**o+(0.920902226631)*x_ref[0]+(-0.477417470246)*x_ref[1]**o+(0.836417982216)*x_ref[1]
        ref[2,4,1,0]=(0.567966851024)*x_ref[0]**o+(0.583241431716)*x_ref[0]+(-0.271851271008)*x_ref[1]**o+(0.678727902863)*x_ref[1]
        ref[2,4,1,1]=(0.707400094)*x_ref[0]**o+(0.802820856444)*x_ref[0]+(0.84518937616)*x_ref[1]**o+(-0.0267193533403)*x_ref[1]
        ref[2,4,2,0]=(0.213268920036)*x_ref[0]**o+(0.780721415328)*x_ref[0]+(0.839292948154)*x_ref[1]**o+(-0.0376334744443)*x_ref[1]
        ref[2,4,2,1]=(0.136255687031)*x_ref[0]**o+(-0.582766278693)*x_ref[0]+(0.257662521321)*x_ref[1]**o+(0.808337521979)*x_ref[1]
        ref[3,0,0,0]=(-0.524068770822)*x_ref[0]**o+(0.942236878842)*x_ref[0]+(0.889496255793)*x_ref[1]**o+(-0.904810742393)*x_ref[1]
        ref[3,0,0,1]=(0.780157465475)*x_ref[0]**o+(-0.763811668832)*x_ref[0]+(-0.517482070639)*x_ref[1]**o+(0.662770780676)*x_ref[1]
        ref[3,0,1,0]=(-0.693227564812)*x_ref[0]**o+(-0.763506211841)*x_ref[0]+(-0.427709881126)*x_ref[1]**o+(0.159109827204)*x_ref[1]
        ref[3,0,1,1]=(0.644975629084)*x_ref[0]**o+(-0.928112761899)*x_ref[0]+(-0.530772740083)*x_ref[1]**o+(-0.723174790087)*x_ref[1]
        ref[3,0,2,0]=(-0.923467274097)*x_ref[0]**o+(0.806128117288)*x_ref[0]+(0.177779363207)*x_ref[1]**o+(0.862243885542)*x_ref[1]
        ref[3,0,2,1]=(-0.0823252901358)*x_ref[0]**o+(0.289022916284)*x_ref[0]+(-0.518508771021)*x_ref[1]**o+(0.174564769845)*x_ref[1]
        ref[3,1,0,0]=(0.608837578444)*x_ref[0]**o+(0.898216758361)*x_ref[0]+(-0.844730109761)*x_ref[1]**o+(0.745974173207)*x_ref[1]
        ref[3,1,0,1]=(-0.11776411078)*x_ref[0]**o+(0.186256502356)*x_ref[0]+(0.565289400333)*x_ref[1]**o+(-0.776173716206)*x_ref[1]
        ref[3,1,1,0]=(-0.170736580152)*x_ref[0]**o+(0.829581661683)*x_ref[0]+(0.310135833859)*x_ref[1]**o+(-0.509629545687)*x_ref[1]
        ref[3,1,1,1]=(-0.225246122751)*x_ref[0]**o+(0.586754377294)*x_ref[0]+(-0.726064725635)*x_ref[1]**o+(0.136952605704)*x_ref[1]
        ref[3,1,2,0]=(-0.287932115478)*x_ref[0]**o+(0.858031396153)*x_ref[0]+(-0.493441745353)*x_ref[1]**o+(0.601827664994)*x_ref[1]
        ref[3,1,2,1]=(0.665503842862)*x_ref[0]**o+(-0.515998970452)*x_ref[0]+(-0.890055558624)*x_ref[1]**o+(-0.0446371451457)*x_ref[1]
        ref[3,2,0,0]=(-0.33741557585)*x_ref[0]**o+(-0.416183752333)*x_ref[0]+(-0.92430073538)*x_ref[1]**o+(-0.486265090904)*x_ref[1]
        ref[3,2,0,1]=(0.910860925994)*x_ref[0]**o+(-0.896869755389)*x_ref[0]+(0.0634658185324)*x_ref[1]**o+(-0.383136455739)*x_ref[1]
        ref[3,2,1,0]=(0.386839297976)*x_ref[0]**o+(-0.190659927071)*x_ref[0]+(-0.304846978185)*x_ref[1]**o+(-0.92705511899)*x_ref[1]
        ref[3,2,1,1]=(-0.0197812018683)*x_ref[0]**o+(-0.479083095775)*x_ref[0]+(0.502489976089)*x_ref[1]**o+(-0.859440665475)*x_ref[1]
        ref[3,2,2,0]=(-0.507672045299)*x_ref[0]**o+(0.498689676072)*x_ref[0]+(-0.719735804645)*x_ref[1]**o+(-0.522053306674)*x_ref[1]
        ref[3,2,2,1]=(0.46207629211)*x_ref[0]**o+(-0.496697066039)*x_ref[0]+(0.994069339904)*x_ref[1]**o+(0.873222754449)*x_ref[1]
        ref[3,3,0,0]=(0.126410029692)*x_ref[0]**o+(0.51067050196)*x_ref[0]+(-0.978384824088)*x_ref[1]**o+(-0.68606930503)*x_ref[1]
        ref[3,3,0,1]=(0.376888157008)*x_ref[0]**o+(-0.14015764025)*x_ref[0]+(0.523667700758)*x_ref[1]**o+(0.0277726855258)*x_ref[1]
        ref[3,3,1,0]=(-0.105865233294)*x_ref[0]**o+(-0.0515230741203)*x_ref[0]+(0.203766525036)*x_ref[1]**o+(-0.0431893014387)*x_ref[1]
        ref[3,3,1,1]=(-0.404351448486)*x_ref[0]**o+(-0.639161429157)*x_ref[0]+(-0.957341091708)*x_ref[1]**o+(-0.103346466984)*x_ref[1]
        ref[3,3,2,0]=(-0.0523508780623)*x_ref[0]**o+(-0.563589575544)*x_ref[0]+(0.158487727325)*x_ref[1]**o+(-0.519455492178)*x_ref[1]
        ref[3,3,2,1]=(-0.823286342218)*x_ref[0]**o+(-0.493663499535)*x_ref[0]+(-0.409825574064)*x_ref[1]**o+(-0.0406101279806)*x_ref[1]
        ref[3,4,0,0]=(0.926212910387)*x_ref[0]**o+(-0.0944507963)*x_ref[0]+(0.640740499391)*x_ref[1]**o+(-0.106969076047)*x_ref[1]
        ref[3,4,0,1]=(-0.912044993941)*x_ref[0]**o+(-0.495323777918)*x_ref[0]+(-0.595831845108)*x_ref[1]**o+(-0.313723426318)*x_ref[1]
        ref[3,4,1,0]=(-0.0730898742548)*x_ref[0]**o+(0.0524008484492)*x_ref[0]+(-0.948232935497)*x_ref[1]**o+(0.952670824572)*x_ref[1]
        ref[3,4,1,1]=(-0.81869438138)*x_ref[0]**o+(-0.894654266952)*x_ref[0]+(0.414078621872)*x_ref[1]**o+(-0.876690817847)*x_ref[1]
        ref[3,4,2,0]=(0.00688155078441)*x_ref[0]**o+(0.845964586551)*x_ref[0]+(0.0183149371892)*x_ref[1]**o+(-0.882931394537)*x_ref[1]
        ref[3,4,2,1]=(-0.734830097149)*x_ref[0]**o+(-0.37824588859)*x_ref[0]+(-0.815402620311)*x_ref[1]**o+(-0.357905685517)*x_ref[1]
      else:
        arg[0,0,0,0]=(-0.110519315354)*x[0]**o+(0.904227374292)*x[0]+(0.459005710707)*x[1]**o+(-0.980231422256)*x[1]+(-0.819713355475)*x[2]**o+(-0.93459517989)*x[2]
        arg[0,0,0,1]=(-0.326902946413)*x[0]**o+(0.489327766219)*x[0]+(-0.965308741061)*x[1]**o+(0.0351399111471)*x[1]+(0.257760096595)*x[2]**o+(0.125123023244)*x[2]
        arg[0,0,1,0]=(-0.584360710004)*x[0]**o+(0.681892732544)*x[0]+(-0.118257741614)*x[1]**o+(0.839875738892)*x[1]+(0.107934154241)*x[2]**o+(0.279757054665)*x[2]
        arg[0,0,1,1]=(0.90973214794)*x[0]**o+(-0.776804618826)*x[0]+(-0.910633853381)*x[1]**o+(-0.25678788529)*x[1]+(0.783709363646)*x[2]**o+(-0.922660855915)*x[2]
        arg[0,0,2,0]=(-0.802273263394)*x[0]**o+(-0.917749665952)*x[0]+(0.796206817226)*x[1]**o+(0.917284637885)*x[1]+(0.219841508893)*x[2]**o+(0.0655848200083)*x[2]
        arg[0,0,2,1]=(0.550596792343)*x[0]**o+(-0.760207057617)*x[0]+(0.0368469683098)*x[1]**o+(0.509347717331)*x[1]+(0.813780778487)*x[2]**o+(-0.432656298451)*x[2]
        arg[0,1,0,0]=(-0.238502042941)*x[0]**o+(-0.966632221032)*x[0]+(-0.373745652933)*x[1]**o+(-0.436176336389)*x[1]+(0.124280197699)*x[2]**o+(0.552700562803)*x[2]
        arg[0,1,0,1]=(-0.19848053276)*x[0]**o+(0.110167634957)*x[0]+(-0.748553857473)*x[1]**o+(0.355932508959)*x[1]+(-0.226987914753)*x[2]**o+(-0.68870238197)*x[2]
        arg[0,1,1,0]=(0.365822518882)*x[0]**o+(0.327285102897)*x[0]+(0.549212504635)*x[1]**o+(0.918179946504)*x[1]+(0.20644004679)*x[2]**o+(0.821299409756)*x[2]
        arg[0,1,1,1]=(-0.366167489101)*x[0]**o+(0.529933186572)*x[0]+(-0.604965310967)*x[1]**o+(-0.833093092352)*x[1]+(-0.73593913912)*x[2]**o+(0.872325614453)*x[2]
        arg[0,1,2,0]=(0.289123646902)*x[0]**o+(0.705862153789)*x[0]+(-0.319756886822)*x[1]**o+(-0.453220232524)*x[1]+(-0.192336374167)*x[2]**o+(0.363575495006)*x[2]
        arg[0,1,2,1]=(-0.213014833353)*x[0]**o+(-0.0706784230627)*x[0]+(-0.941272982229)*x[1]**o+(-0.96641361036)*x[1]+(0.289919014964)*x[2]**o+(0.57658772066)*x[2]
        arg[0,2,0,0]=(-0.406683991704)*x[0]**o+(-0.0110042213592)*x[0]+(-0.273148083258)*x[1]**o+(0.054257030447)*x[1]+(0.387598147681)*x[2]**o+(0.480398109069)*x[2]
        arg[0,2,0,1]=(0.318713057562)*x[0]**o+(-0.449467749585)*x[0]+(0.669991267829)*x[1]**o+(0.237986860249)*x[1]+(0.894370523471)*x[2]**o+(-0.397832436233)*x[2]
        arg[0,2,1,0]=(-0.858603253642)*x[0]**o+(-0.713122015919)*x[0]+(0.870251906188)*x[1]**o+(-0.781601254578)*x[1]+(-0.504531525822)*x[2]**o+(-0.699819349556)*x[2]
        arg[0,2,1,1]=(-0.825591116361)*x[0]**o+(-0.552452876394)*x[0]+(-0.262627261618)*x[1]**o+(-0.147127256184)*x[1]+(-0.171485665103)*x[2]**o+(0.823279741501)*x[2]
        arg[0,2,2,0]=(-0.0416279689068)*x[0]**o+(-0.0903338000001)*x[0]+(0.911673780635)*x[1]**o+(0.86298673049)*x[1]+(-0.60251179847)*x[2]**o+(0.378303877746)*x[2]
        arg[0,2,2,1]=(0.261384106387)*x[0]**o+(0.0326416199811)*x[0]+(-0.426740102269)*x[1]**o+(-0.445401626921)*x[1]+(-0.388668254526)*x[2]**o+(-0.977942913284)*x[2]
        arg[0,3,0,0]=(-0.0195053813483)*x[0]**o+(-0.294061998664)*x[0]+(0.790461098426)*x[1]**o+(-0.506334268838)*x[1]+(0.394015541605)*x[2]**o+(-0.562248105432)*x[2]
        arg[0,3,0,1]=(0.390726516735)*x[0]**o+(-0.757597437859)*x[0]+(0.861882149861)*x[1]**o+(0.621870451314)*x[1]+(0.900777666162)*x[2]**o+(0.271507131938)*x[2]
        arg[0,3,1,0]=(-0.725635966475)*x[0]**o+(-0.695837548737)*x[0]+(-0.163078176328)*x[1]**o+(-0.0435808418729)*x[1]+(0.0890756544611)*x[2]**o+(-0.270422627949)*x[2]
        arg[0,3,1,1]=(-0.175498720647)*x[0]**o+(-0.213605441632)*x[0]+(-0.358632620362)*x[1]**o+(-0.193273004152)*x[1]+(0.937704823805)*x[2]**o+(0.861707832291)*x[2]
        arg[0,3,2,0]=(0.0976010265611)*x[0]**o+(0.995135925341)*x[0]+(0.755339287387)*x[1]**o+(-0.924722068193)*x[1]+(-0.480212181904)*x[2]**o+(-0.40237760439)*x[2]
        arg[0,3,2,1]=(-0.679215392952)*x[0]**o+(-0.00796096253297)*x[0]+(-0.964972599421)*x[1]**o+(-0.410415435656)*x[1]+(-0.292638067785)*x[2]**o+(-0.950383976561)*x[2]
        arg[0,4,0,0]=(0.245586594928)*x[0]**o+(-0.593420401896)*x[0]+(0.553450486534)*x[1]**o+(0.173814965689)*x[1]+(0.0719392610563)*x[2]**o+(0.676231437486)*x[2]
        arg[0,4,0,1]=(0.154382756425)*x[0]**o+(-0.0655740293432)*x[0]+(0.924689745142)*x[1]**o+(-0.246876531175)*x[1]+(-0.241460713543)*x[2]**o+(-0.489840108568)*x[2]
        arg[0,4,1,0]=(-0.482705840359)*x[0]**o+(-0.392960380114)*x[0]+(-0.503463823972)*x[1]**o+(0.317163204738)*x[1]+(0.775048877356)*x[2]**o+(0.599889732326)*x[2]
        arg[0,4,1,1]=(-0.122555861716)*x[0]**o+(0.930175770449)*x[0]+(0.483503764941)*x[1]**o+(0.340001033217)*x[1]+(0.672924280979)*x[2]**o+(0.746489359683)*x[2]
        arg[0,4,2,0]=(-0.339421141033)*x[0]**o+(-0.604966575169)*x[0]+(0.862413017648)*x[1]**o+(0.936752078454)*x[1]+(-0.0549628353285)*x[2]**o+(-0.0858885835314)*x[2]
        arg[0,4,2,1]=(-0.581767357421)*x[0]**o+(-0.644286027548)*x[0]+(0.76948396865)*x[1]**o+(0.692265904744)*x[1]+(0.955871091531)*x[2]**o+(-0.378888503457)*x[2]
        arg[1,0,0,0]=(-0.0407846595046)*x[0]**o+(-0.293709943251)*x[0]+(-0.310552931582)*x[1]**o+(-0.696244178466)*x[1]+(0.962985284252)*x[2]**o+(-0.0266457621404)*x[2]
        arg[1,0,0,1]=(-0.480942811584)*x[0]**o+(-0.400158759679)*x[0]+(0.311171869807)*x[1]**o+(-0.79518809427)*x[1]+(-0.497004450478)*x[2]**o+(-0.639822005752)*x[2]
        arg[1,0,1,0]=(0.158738648483)*x[0]**o+(0.274859816468)*x[0]+(-0.208677612654)*x[1]**o+(0.911107598366)*x[1]+(-0.400163007778)*x[2]**o+(0.276296890267)*x[2]
        arg[1,0,1,1]=(0.613312276864)*x[0]**o+(-0.644237218776)*x[0]+(0.81927272895)*x[1]**o+(-0.12971860556)*x[1]+(0.214379406799)*x[2]**o+(0.856335675043)*x[2]
        arg[1,0,2,0]=(-0.699925173766)*x[0]**o+(-0.560613547952)*x[0]+(-0.355325107637)*x[1]**o+(-0.598747822041)*x[1]+(0.243087037553)*x[2]**o+(-0.484176993015)*x[2]
        arg[1,0,2,1]=(-0.218912300848)*x[0]**o+(-0.259483312969)*x[0]+(-0.863788186295)*x[1]**o+(0.924423774757)*x[1]+(-0.0823752192204)*x[2]**o+(-0.928377900602)*x[2]
        arg[1,1,0,0]=(-0.368901144425)*x[0]**o+(0.544104655592)*x[0]+(0.321295151176)*x[1]**o+(0.395692754134)*x[1]+(-0.721319111754)*x[2]**o+(-0.698518442137)*x[2]
        arg[1,1,0,1]=(-0.592867336047)*x[0]**o+(0.145120231038)*x[0]+(0.110390771366)*x[1]**o+(0.168333368086)*x[1]+(-0.577866419921)*x[2]**o+(0.705027708469)*x[2]
        arg[1,1,1,0]=(-0.210603428235)*x[0]**o+(-0.250156447116)*x[0]+(-0.397803248976)*x[1]**o+(0.332163380867)*x[1]+(-0.250648216541)*x[2]**o+(-0.0693729055697)*x[2]
        arg[1,1,1,1]=(0.324927558096)*x[0]**o+(-0.262726116223)*x[0]+(-0.555439758488)*x[1]**o+(-0.865019248609)*x[1]+(-0.0971907425857)*x[2]**o+(-0.742359236711)*x[2]
        arg[1,1,2,0]=(0.161058869414)*x[0]**o+(0.349191500791)*x[0]+(0.468336473315)*x[1]**o+(-0.856364775995)*x[1]+(0.491736696689)*x[2]**o+(0.352984104173)*x[2]
        arg[1,1,2,1]=(-0.243275976508)*x[0]**o+(-0.785605388878)*x[0]+(-0.282478919296)*x[1]**o+(-0.728851522341)*x[1]+(0.375134848237)*x[2]**o+(-0.486200561256)*x[2]
        arg[1,2,0,0]=(-0.172241451405)*x[0]**o+(0.422361498196)*x[0]+(0.464862755164)*x[1]**o+(0.466661717105)*x[1]+(-0.886209677512)*x[2]**o+(-0.43529076789)*x[2]
        arg[1,2,0,1]=(0.946582222405)*x[0]**o+(0.513640973894)*x[0]+(-0.923788000633)*x[1]**o+(-0.358163567195)*x[1]+(-0.709999132129)*x[2]**o+(-0.761163299694)*x[2]
        arg[1,2,1,0]=(0.236339981889)*x[0]**o+(0.259874866134)*x[0]+(-0.32460762691)*x[1]**o+(-0.629786950313)*x[1]+(-0.679261685402)*x[2]**o+(0.593069192918)*x[2]
        arg[1,2,1,1]=(0.11006800009)*x[0]**o+(0.970532991787)*x[0]+(-0.467608188361)*x[1]**o+(-0.959281984552)*x[1]+(0.610396622519)*x[2]**o+(0.282468600058)*x[2]
        arg[1,2,2,0]=(-0.656419157742)*x[0]**o+(0.234027308171)*x[0]+(-0.67027206978)*x[1]**o+(-0.746034436817)*x[1]+(-0.771894073652)*x[2]**o+(0.949545830287)*x[2]
        arg[1,2,2,1]=(0.615884236811)*x[0]**o+(-0.670156612555)*x[0]+(0.543165493345)*x[1]**o+(-0.618294277594)*x[1]+(-0.974114255061)*x[2]**o+(-0.762320542298)*x[2]
        arg[1,3,0,0]=(0.096201957648)*x[0]**o+(0.440870816208)*x[0]+(0.124448859607)*x[1]**o+(0.507919407561)*x[1]+(-0.401666263687)*x[2]**o+(-0.437653713723)*x[2]
        arg[1,3,0,1]=(0.165129229294)*x[0]**o+(-0.353217428955)*x[0]+(0.473666065504)*x[1]**o+(0.907667166988)*x[1]+(-0.864298533288)*x[2]**o+(0.641505117281)*x[2]
        arg[1,3,1,0]=(0.381900624425)*x[0]**o+(0.427520178597)*x[0]+(-0.560279540293)*x[1]**o+(0.190814532271)*x[1]+(-0.187947688629)*x[2]**o+(-0.127694148593)*x[2]
        arg[1,3,1,1]=(-0.903921363895)*x[0]**o+(-0.659942040828)*x[0]+(-0.549460213492)*x[1]**o+(0.869909651895)*x[1]+(-0.962273439484)*x[2]**o+(-0.046365857751)*x[2]
        arg[1,3,2,0]=(-0.021688482187)*x[0]**o+(-0.209531000717)*x[0]+(0.668564395133)*x[1]**o+(-0.0347978700975)*x[1]+(0.947677147488)*x[2]**o+(-0.0152385052475)*x[2]
        arg[1,3,2,1]=(0.880839485817)*x[0]**o+(-0.319275929312)*x[0]+(-0.940056657485)*x[1]**o+(0.570855397726)*x[1]+(0.279710812074)*x[2]**o+(0.496622215051)*x[2]
        arg[1,4,0,0]=(-0.139033645615)*x[0]**o+(-0.271453722366)*x[0]+(-0.268516720244)*x[1]**o+(-0.0467555017438)*x[1]+(-0.41618262265)*x[2]**o+(0.315320363813)*x[2]
        arg[1,4,0,1]=(0.296108607204)*x[0]**o+(-0.184582165661)*x[0]+(0.121416543744)*x[1]**o+(-0.922284926751)*x[1]+(-0.437600089668)*x[2]**o+(-0.608800626513)*x[2]
        arg[1,4,1,0]=(0.8942028423)*x[0]**o+(0.673402831188)*x[0]+(-0.622529195562)*x[1]**o+(-0.266052338798)*x[1]+(-0.273167851702)*x[2]**o+(-0.312211982747)*x[2]
        arg[1,4,1,1]=(-0.102315711178)*x[0]**o+(-0.885515719371)*x[0]+(-0.532910101204)*x[1]**o+(0.798525944348)*x[1]+(0.640586802303)*x[2]**o+(0.36308806536)*x[2]
        arg[1,4,2,0]=(0.376793506541)*x[0]**o+(-0.667063564875)*x[0]+(0.609591985816)*x[1]**o+(0.28347745142)*x[1]+(-0.751832936302)*x[2]**o+(-0.683738856965)*x[2]
        arg[1,4,2,1]=(-0.51412252819)*x[0]**o+(0.559560297206)*x[0]+(0.751134603429)*x[1]**o+(-0.0031812310797)*x[1]+(-0.525776122178)*x[2]**o+(0.338471942848)*x[2]
        arg[2,0,0,0]=(0.937572244905)*x[0]**o+(-0.293020870379)*x[0]+(-0.918591520817)*x[1]**o+(0.622194477261)*x[1]+(0.31641482851)*x[2]**o+(0.276912266864)*x[2]
        arg[2,0,0,1]=(0.853041599998)*x[0]**o+(-0.0220623724171)*x[0]+(0.89909044296)*x[1]**o+(-0.838782642732)*x[1]+(0.0133197592042)*x[2]**o+(-0.868977618993)*x[2]
        arg[2,0,1,0]=(-0.325799042976)*x[0]**o+(0.746269971994)*x[0]+(0.882671061757)*x[1]**o+(-0.284133373252)*x[1]+(0.120086549521)*x[2]**o+(-0.397352610969)*x[2]
        arg[2,0,1,1]=(0.282336934951)*x[0]**o+(-0.339893978547)*x[0]+(-0.0861660125386)*x[1]**o+(0.283063579798)*x[1]+(0.714762478569)*x[2]**o+(-0.497021718549)*x[2]
        arg[2,0,2,0]=(0.891782779491)*x[0]**o+(-0.817647577644)*x[0]+(0.364088142085)*x[1]**o+(-0.950577637048)*x[1]+(0.4070663557)*x[2]**o+(0.270753179246)*x[2]
        arg[2,0,2,1]=(0.316594913808)*x[0]**o+(0.310731486851)*x[0]+(0.250421828765)*x[1]**o+(0.772786141458)*x[1]+(0.64843040679)*x[2]**o+(-0.691903332511)*x[2]
        arg[2,1,0,0]=(0.87180715028)*x[0]**o+(-0.953173230375)*x[0]+(-0.823516344548)*x[1]**o+(0.934653756548)*x[1]+(-0.537124962489)*x[2]**o+(-0.233365939659)*x[2]
        arg[2,1,0,1]=(-0.407189744631)*x[0]**o+(-0.754425720288)*x[0]+(0.737526767897)*x[1]**o+(0.745393943502)*x[1]+(0.811744293685)*x[2]**o+(-0.459556659177)*x[2]
        arg[2,1,1,0]=(-0.693529259877)*x[0]**o+(0.580197938048)*x[0]+(0.506055440405)*x[1]**o+(0.122531014759)*x[1]+(0.427736137775)*x[2]**o+(-0.83260140401)*x[2]
        arg[2,1,1,1]=(0.542621975026)*x[0]**o+(-0.568425713106)*x[0]+(0.303002043446)*x[1]**o+(0.363286261651)*x[1]+(0.2729476289)*x[2]**o+(-0.0113629744659)*x[2]
        arg[2,1,2,0]=(-0.266750235694)*x[0]**o+(0.10116400306)*x[0]+(0.315977962897)*x[1]**o+(-0.406236201454)*x[1]+(-0.586845072374)*x[2]**o+(0.513937047282)*x[2]
        arg[2,1,2,1]=(-0.518780811975)*x[0]**o+(0.275053640829)*x[0]+(0.4234970646)*x[1]**o+(0.389585163376)*x[1]+(-0.609771068778)*x[2]**o+(-0.121467061755)*x[2]
        arg[2,2,0,0]=(0.138823207466)*x[0]**o+(-0.779837455014)*x[0]+(0.70167049555)*x[1]**o+(0.766428813369)*x[1]+(0.114023530366)*x[2]**o+(0.650098536187)*x[2]
        arg[2,2,0,1]=(0.974998667633)*x[0]**o+(0.958471050654)*x[0]+(0.193515162503)*x[1]**o+(-0.252148167253)*x[1]+(0.495475460125)*x[2]**o+(0.562693347983)*x[2]
        arg[2,2,1,0]=(-0.552379548988)*x[0]**o+(-0.831399971618)*x[0]+(0.437581216647)*x[1]**o+(-0.775331267696)*x[1]+(0.729041628844)*x[2]**o+(0.633113359893)*x[2]
        arg[2,2,1,1]=(0.379858437049)*x[0]**o+(0.923586504203)*x[0]+(0.707244783969)*x[1]**o+(-0.293157375059)*x[1]+(-0.368717941876)*x[2]**o+(-0.202610288616)*x[2]
        arg[2,2,2,0]=(-0.804826108231)*x[0]**o+(-0.0208712096312)*x[0]+(0.0482547072498)*x[1]**o+(0.852366543609)*x[1]+(0.328962500136)*x[2]**o+(-0.132777835512)*x[2]
        arg[2,2,2,1]=(0.91373752217)*x[0]**o+(-0.792112120729)*x[0]+(-0.0457564087072)*x[1]**o+(0.309694852949)*x[1]+(0.089219285337)*x[2]**o+(-0.516000514614)*x[2]
        arg[2,3,0,0]=(0.343215129115)*x[0]**o+(0.40804099837)*x[0]+(-0.865942442914)*x[1]**o+(0.842241683043)*x[1]+(0.0727803297601)*x[2]**o+(-0.395225682949)*x[2]
        arg[2,3,0,1]=(-0.965340522771)*x[0]**o+(-0.897056478677)*x[0]+(-0.305205944062)*x[1]**o+(0.393941789685)*x[1]+(-0.253328379131)*x[2]**o+(0.0951510469271)*x[2]
        arg[2,3,1,0]=(0.536251514752)*x[0]**o+(0.0629776386779)*x[0]+(-0.706311222379)*x[1]**o+(-0.84362242029)*x[1]+(0.00110678760539)*x[2]**o+(-0.30286059941)*x[2]
        arg[2,3,1,1]=(0.114584737295)*x[0]**o+(0.0708763738067)*x[0]+(-0.454760062397)*x[1]**o+(0.228742539922)*x[1]+(0.514960810352)*x[2]**o+(-0.815749211085)*x[2]
        arg[2,3,2,0]=(0.246335688008)*x[0]**o+(-0.239199299885)*x[0]+(-0.447390151659)*x[1]**o+(0.918949356391)*x[1]+(-0.307864840175)*x[2]**o+(0.560139080481)*x[2]
        arg[2,3,2,1]=(0.118766035723)*x[0]**o+(0.610711164899)*x[0]+(0.336628771776)*x[1]**o+(0.833699268751)*x[1]+(-0.424694647429)*x[2]**o+(-0.639929688104)*x[2]
        arg[2,4,0,0]=(0.814543506987)*x[0]**o+(-0.797684321368)*x[0]+(0.337242635783)*x[1]**o+(0.278139837072)*x[1]+(0.633511526441)*x[2]**o+(0.372175615028)*x[2]
        arg[2,4,0,1]=(-0.0590668694094)*x[0]**o+(0.256732067267)*x[0]+(0.519737187577)*x[1]**o+(-0.376391855948)*x[1]+(-0.810934658651)*x[2]**o+(0.826899476928)*x[2]
        arg[2,4,1,0]=(-0.428976843661)*x[0]**o+(0.672667833313)*x[0]+(0.263194890946)*x[1]**o+(-0.894790773661)*x[1]+(-0.550298557295)*x[2]**o+(-0.00560384035502)*x[2]
        arg[2,4,1,1]=(-0.818372430144)*x[0]**o+(-0.829920833073)*x[0]+(-0.359560636119)*x[1]**o+(-0.113484563205)*x[1]+(0.586796110365)*x[2]**o+(0.296820898426)*x[2]
        arg[2,4,2,0]=(-0.691597969566)*x[0]**o+(0.715813153538)*x[0]+(0.954093612503)*x[1]**o+(0.88803420754)*x[1]+(0.697447667765)*x[2]**o+(0.793730496625)*x[2]
        arg[2,4,2,1]=(0.351099833336)*x[0]**o+(-0.716419272807)*x[0]+(0.318726674998)*x[1]**o+(0.246151914414)*x[1]+(-0.945520673884)*x[2]**o+(-0.489026233954)*x[2]
        arg[3,0,0,0]=(0.511598022804)*x[0]**o+(0.567617269582)*x[0]+(-0.526293319137)*x[1]**o+(0.333750098146)*x[1]+(-0.0869228849144)*x[2]**o+(-0.204072834867)*x[2]
        arg[3,0,0,1]=(0.905384007172)*x[0]**o+(-0.661067390314)*x[0]+(-0.470790632295)*x[1]**o+(-0.149608017616)*x[1]+(0.427879278267)*x[2]**o+(0.987040179184)*x[2]
        arg[3,0,1,0]=(-0.946562006809)*x[0]**o+(0.64141909633)*x[0]+(-0.762076826276)*x[1]**o+(-0.642912850794)*x[1]+(0.712567898559)*x[2]**o+(-0.292128944205)*x[2]
        arg[3,0,1,1]=(-0.955087949361)*x[0]**o+(0.426588041133)*x[0]+(0.339313104127)*x[1]**o+(-0.375147344045)*x[1]+(0.967259014679)*x[2]**o+(0.504521081858)*x[2]
        arg[3,0,2,0]=(0.141876598344)*x[0]**o+(-0.335579252825)*x[0]+(0.514552993602)*x[1]**o+(-0.531116948499)*x[1]+(-0.369319413141)*x[2]**o+(0.0374413262362)*x[2]
        arg[3,0,2,1]=(0.678360463658)*x[0]**o+(-0.357900119646)*x[0]+(0.300750717059)*x[1]**o+(0.516196189448)*x[1]+(0.0535466018203)*x[2]**o+(-0.149961940786)*x[2]
        arg[3,1,0,0]=(0.0975994351079)*x[0]**o+(0.634544068499)*x[0]+(-0.963217046352)*x[1]**o+(-0.804658091456)*x[1]+(-0.620132720823)*x[2]**o+(0.862250619369)*x[2]
        arg[3,1,0,1]=(0.210306328879)*x[0]**o+(-0.457341187966)*x[0]+(0.957556084801)*x[1]**o+(0.704073170928)*x[1]+(0.423134492133)*x[2]**o+(-0.167268375412)*x[2]
        arg[3,1,1,0]=(-0.909228817744)*x[0]**o+(0.110665018921)*x[0]+(0.36275630147)*x[1]**o+(0.346866824763)*x[1]+(-0.535600547489)*x[2]**o+(0.627638814375)*x[2]
        arg[3,1,1,1]=(0.328780933823)*x[0]**o+(-0.249563994161)*x[0]+(-0.555789062892)*x[1]**o+(-0.917287525036)*x[1]+(0.386191175017)*x[2]**o+(0.123904319061)*x[2]
        arg[3,1,2,0]=(-0.838409901623)*x[0]**o+(0.673479705393)*x[0]+(0.482921104917)*x[1]**o+(0.136411388588)*x[1]+(0.583674116225)*x[2]**o+(0.305100836877)*x[2]
        arg[3,1,2,1]=(0.952961008312)*x[0]**o+(-0.0501981079532)*x[0]+(0.661434464642)*x[1]**o+(0.65030469399)*x[1]+(-0.157562001295)*x[2]**o+(-0.249551892917)*x[2]
        arg[3,2,0,0]=(-0.0232804500286)*x[0]**o+(-0.559229392203)*x[0]+(0.127967930273)*x[1]**o+(-0.635827181645)*x[1]+(-0.0426703224383)*x[2]**o+(0.47720230258)*x[2]
        arg[3,2,0,1]=(-0.421658996106)*x[0]**o+(-0.468260038077)*x[0]+(0.479083572554)*x[1]**o+(-0.214417881841)*x[1]+(0.372430534261)*x[2]**o+(0.133063789546)*x[2]
        arg[3,2,1,0]=(0.905811586567)*x[0]**o+(0.245447340384)*x[0]+(-0.337650999146)*x[1]**o+(0.908188235976)*x[1]+(-0.804029751277)*x[2]**o+(0.0512676456643)*x[2]
        arg[3,2,1,1]=(-0.891486839409)*x[0]**o+(0.20753794329)*x[0]+(0.879183464252)*x[1]**o+(0.902028906992)*x[1]+(0.416776813598)*x[2]**o+(0.491964374373)*x[2]
        arg[3,2,2,0]=(0.653067027119)*x[0]**o+(0.765293890745)*x[0]+(-0.348902169508)*x[1]**o+(0.324429035849)*x[1]+(-0.363026365065)*x[2]**o+(0.0816186405511)*x[2]
        arg[3,2,2,1]=(-0.0415375052325)*x[0]**o+(-0.480779075784)*x[0]+(-0.137223552938)*x[1]**o+(-0.915733625512)*x[1]+(0.750199445159)*x[2]**o+(-0.789818945171)*x[2]
        arg[3,3,0,0]=(0.0744921239083)*x[0]**o+(0.0700589144159)*x[0]+(-0.327105425297)*x[1]**o+(0.469843257765)*x[1]+(-0.170191600844)*x[2]**o+(0.732071534155)*x[2]
        arg[3,3,0,1]=(0.172643927441)*x[0]**o+(0.848924114383)*x[0]+(-0.79125762228)*x[1]**o+(-0.694725775976)*x[1]+(-0.309710503985)*x[2]**o+(-0.405310234927)*x[2]
        arg[3,3,1,0]=(-0.465911787595)*x[0]**o+(-0.71080474709)*x[0]+(0.959925934203)*x[1]**o+(-0.111110644825)*x[1]+(-0.145349925218)*x[2]**o+(0.122955165811)*x[2]
        arg[3,3,1,1]=(-0.769111863754)*x[0]**o+(0.348036938532)*x[0]+(0.485890856474)*x[1]**o+(0.382595218985)*x[1]+(-0.897488692981)*x[2]**o+(0.830822333819)*x[2]
        arg[3,3,2,0]=(0.341890793681)*x[0]**o+(-0.667145936571)*x[0]+(-0.145094108268)*x[1]**o+(0.771433911212)*x[1]+(-0.299752164325)*x[2]**o+(-0.357856549677)*x[2]
        arg[3,3,2,1]=(0.26381838796)*x[0]**o+(-0.44474724998)*x[0]+(0.310283988195)*x[1]**o+(0.698675181314)*x[1]+(0.437624133238)*x[2]**o+(0.394916054445)*x[2]
        arg[3,4,0,0]=(-0.576499406674)*x[0]**o+(0.770936634959)*x[0]+(0.707441735783)*x[1]**o+(-0.794953257043)*x[1]+(-0.535342943209)*x[2]**o+(-0.509446811787)*x[2]
        arg[3,4,0,1]=(0.161115012113)*x[0]**o+(0.11079092252)*x[0]+(0.0633804181164)*x[1]**o+(-0.406049293019)*x[1]+(-0.824283994336)*x[2]**o+(-0.481107560499)*x[2]
        arg[3,4,1,0]=(-0.411133810295)*x[0]**o+(0.988486625099)*x[0]+(-0.36050191367)*x[1]**o+(0.469479821965)*x[1]+(-0.431175357325)*x[2]**o+(-0.966295015263)*x[2]
        arg[3,4,1,1]=(0.233731410367)*x[0]**o+(-0.821900740766)*x[0]+(-0.823123461385)*x[1]**o+(-0.380537914421)*x[1]+(-0.896146300355)*x[2]**o+(-0.775172990257)*x[2]
        arg[3,4,2,0]=(-0.96323363586)*x[0]**o+(0.0935412405245)*x[0]+(0.929333026259)*x[1]**o+(-0.322236627348)*x[1]+(-0.390898135964)*x[2]**o+(-0.766178290162)*x[2]
        arg[3,4,2,1]=(-0.773670533268)*x[0]**o+(-0.156975157098)*x[0]+(-0.91512996795)*x[1]**o+(0.0247803682126)*x[1]+(0.995159733782)*x[2]**o+(-0.669224144225)*x[2]
        ref[0,0,0,0]=(-0.110519315354)*x_ref[0]**o+(0.904227374292)*x_ref[0]+(0.459005710707)*x_ref[1]**o+(-0.980231422256)*x_ref[1]+(-0.819713355475)*x_ref[2]**o+(-0.93459517989)*x_ref[2]
        ref[0,0,0,1]=(-0.326902946413)*x_ref[0]**o+(0.489327766219)*x_ref[0]+(-0.965308741061)*x_ref[1]**o+(0.0351399111471)*x_ref[1]+(0.257760096595)*x_ref[2]**o+(0.125123023244)*x_ref[2]
        ref[0,0,1,0]=(-0.584360710004)*x_ref[0]**o+(0.681892732544)*x_ref[0]+(-0.118257741614)*x_ref[1]**o+(0.839875738892)*x_ref[1]+(0.107934154241)*x_ref[2]**o+(0.279757054665)*x_ref[2]
        ref[0,0,1,1]=(0.90973214794)*x_ref[0]**o+(-0.776804618826)*x_ref[0]+(-0.910633853381)*x_ref[1]**o+(-0.25678788529)*x_ref[1]+(0.783709363646)*x_ref[2]**o+(-0.922660855915)*x_ref[2]
        ref[0,0,2,0]=(-0.802273263394)*x_ref[0]**o+(-0.917749665952)*x_ref[0]+(0.796206817226)*x_ref[1]**o+(0.917284637885)*x_ref[1]+(0.219841508893)*x_ref[2]**o+(0.0655848200083)*x_ref[2]
        ref[0,0,2,1]=(0.550596792343)*x_ref[0]**o+(-0.760207057617)*x_ref[0]+(0.0368469683098)*x_ref[1]**o+(0.509347717331)*x_ref[1]+(0.813780778487)*x_ref[2]**o+(-0.432656298451)*x_ref[2]
        ref[0,1,0,0]=(-0.238502042941)*x_ref[0]**o+(-0.966632221032)*x_ref[0]+(-0.373745652933)*x_ref[1]**o+(-0.436176336389)*x_ref[1]+(0.124280197699)*x_ref[2]**o+(0.552700562803)*x_ref[2]
        ref[0,1,0,1]=(-0.19848053276)*x_ref[0]**o+(0.110167634957)*x_ref[0]+(-0.748553857473)*x_ref[1]**o+(0.355932508959)*x_ref[1]+(-0.226987914753)*x_ref[2]**o+(-0.68870238197)*x_ref[2]
        ref[0,1,1,0]=(0.365822518882)*x_ref[0]**o+(0.327285102897)*x_ref[0]+(0.549212504635)*x_ref[1]**o+(0.918179946504)*x_ref[1]+(0.20644004679)*x_ref[2]**o+(0.821299409756)*x_ref[2]
        ref[0,1,1,1]=(-0.366167489101)*x_ref[0]**o+(0.529933186572)*x_ref[0]+(-0.604965310967)*x_ref[1]**o+(-0.833093092352)*x_ref[1]+(-0.73593913912)*x_ref[2]**o+(0.872325614453)*x_ref[2]
        ref[0,1,2,0]=(0.289123646902)*x_ref[0]**o+(0.705862153789)*x_ref[0]+(-0.319756886822)*x_ref[1]**o+(-0.453220232524)*x_ref[1]+(-0.192336374167)*x_ref[2]**o+(0.363575495006)*x_ref[2]
        ref[0,1,2,1]=(-0.213014833353)*x_ref[0]**o+(-0.0706784230627)*x_ref[0]+(-0.941272982229)*x_ref[1]**o+(-0.96641361036)*x_ref[1]+(0.289919014964)*x_ref[2]**o+(0.57658772066)*x_ref[2]
        ref[0,2,0,0]=(-0.406683991704)*x_ref[0]**o+(-0.0110042213592)*x_ref[0]+(-0.273148083258)*x_ref[1]**o+(0.054257030447)*x_ref[1]+(0.387598147681)*x_ref[2]**o+(0.480398109069)*x_ref[2]
        ref[0,2,0,1]=(0.318713057562)*x_ref[0]**o+(-0.449467749585)*x_ref[0]+(0.669991267829)*x_ref[1]**o+(0.237986860249)*x_ref[1]+(0.894370523471)*x_ref[2]**o+(-0.397832436233)*x_ref[2]
        ref[0,2,1,0]=(-0.858603253642)*x_ref[0]**o+(-0.713122015919)*x_ref[0]+(0.870251906188)*x_ref[1]**o+(-0.781601254578)*x_ref[1]+(-0.504531525822)*x_ref[2]**o+(-0.699819349556)*x_ref[2]
        ref[0,2,1,1]=(-0.825591116361)*x_ref[0]**o+(-0.552452876394)*x_ref[0]+(-0.262627261618)*x_ref[1]**o+(-0.147127256184)*x_ref[1]+(-0.171485665103)*x_ref[2]**o+(0.823279741501)*x_ref[2]
        ref[0,2,2,0]=(-0.0416279689068)*x_ref[0]**o+(-0.0903338000001)*x_ref[0]+(0.911673780635)*x_ref[1]**o+(0.86298673049)*x_ref[1]+(-0.60251179847)*x_ref[2]**o+(0.378303877746)*x_ref[2]
        ref[0,2,2,1]=(0.261384106387)*x_ref[0]**o+(0.0326416199811)*x_ref[0]+(-0.426740102269)*x_ref[1]**o+(-0.445401626921)*x_ref[1]+(-0.388668254526)*x_ref[2]**o+(-0.977942913284)*x_ref[2]
        ref[0,3,0,0]=(-0.0195053813483)*x_ref[0]**o+(-0.294061998664)*x_ref[0]+(0.790461098426)*x_ref[1]**o+(-0.506334268838)*x_ref[1]+(0.394015541605)*x_ref[2]**o+(-0.562248105432)*x_ref[2]
        ref[0,3,0,1]=(0.390726516735)*x_ref[0]**o+(-0.757597437859)*x_ref[0]+(0.861882149861)*x_ref[1]**o+(0.621870451314)*x_ref[1]+(0.900777666162)*x_ref[2]**o+(0.271507131938)*x_ref[2]
        ref[0,3,1,0]=(-0.725635966475)*x_ref[0]**o+(-0.695837548737)*x_ref[0]+(-0.163078176328)*x_ref[1]**o+(-0.0435808418729)*x_ref[1]+(0.0890756544611)*x_ref[2]**o+(-0.270422627949)*x_ref[2]
        ref[0,3,1,1]=(-0.175498720647)*x_ref[0]**o+(-0.213605441632)*x_ref[0]+(-0.358632620362)*x_ref[1]**o+(-0.193273004152)*x_ref[1]+(0.937704823805)*x_ref[2]**o+(0.861707832291)*x_ref[2]
        ref[0,3,2,0]=(0.0976010265611)*x_ref[0]**o+(0.995135925341)*x_ref[0]+(0.755339287387)*x_ref[1]**o+(-0.924722068193)*x_ref[1]+(-0.480212181904)*x_ref[2]**o+(-0.40237760439)*x_ref[2]
        ref[0,3,2,1]=(-0.679215392952)*x_ref[0]**o+(-0.00796096253297)*x_ref[0]+(-0.964972599421)*x_ref[1]**o+(-0.410415435656)*x_ref[1]+(-0.292638067785)*x_ref[2]**o+(-0.950383976561)*x_ref[2]
        ref[0,4,0,0]=(0.245586594928)*x_ref[0]**o+(-0.593420401896)*x_ref[0]+(0.553450486534)*x_ref[1]**o+(0.173814965689)*x_ref[1]+(0.0719392610563)*x_ref[2]**o+(0.676231437486)*x_ref[2]
        ref[0,4,0,1]=(0.154382756425)*x_ref[0]**o+(-0.0655740293432)*x_ref[0]+(0.924689745142)*x_ref[1]**o+(-0.246876531175)*x_ref[1]+(-0.241460713543)*x_ref[2]**o+(-0.489840108568)*x_ref[2]
        ref[0,4,1,0]=(-0.482705840359)*x_ref[0]**o+(-0.392960380114)*x_ref[0]+(-0.503463823972)*x_ref[1]**o+(0.317163204738)*x_ref[1]+(0.775048877356)*x_ref[2]**o+(0.599889732326)*x_ref[2]
        ref[0,4,1,1]=(-0.122555861716)*x_ref[0]**o+(0.930175770449)*x_ref[0]+(0.483503764941)*x_ref[1]**o+(0.340001033217)*x_ref[1]+(0.672924280979)*x_ref[2]**o+(0.746489359683)*x_ref[2]
        ref[0,4,2,0]=(-0.339421141033)*x_ref[0]**o+(-0.604966575169)*x_ref[0]+(0.862413017648)*x_ref[1]**o+(0.936752078454)*x_ref[1]+(-0.0549628353285)*x_ref[2]**o+(-0.0858885835314)*x_ref[2]
        ref[0,4,2,1]=(-0.581767357421)*x_ref[0]**o+(-0.644286027548)*x_ref[0]+(0.76948396865)*x_ref[1]**o+(0.692265904744)*x_ref[1]+(0.955871091531)*x_ref[2]**o+(-0.378888503457)*x_ref[2]
        ref[1,0,0,0]=(-0.0407846595046)*x_ref[0]**o+(-0.293709943251)*x_ref[0]+(-0.310552931582)*x_ref[1]**o+(-0.696244178466)*x_ref[1]+(0.962985284252)*x_ref[2]**o+(-0.0266457621404)*x_ref[2]
        ref[1,0,0,1]=(-0.480942811584)*x_ref[0]**o+(-0.400158759679)*x_ref[0]+(0.311171869807)*x_ref[1]**o+(-0.79518809427)*x_ref[1]+(-0.497004450478)*x_ref[2]**o+(-0.639822005752)*x_ref[2]
        ref[1,0,1,0]=(0.158738648483)*x_ref[0]**o+(0.274859816468)*x_ref[0]+(-0.208677612654)*x_ref[1]**o+(0.911107598366)*x_ref[1]+(-0.400163007778)*x_ref[2]**o+(0.276296890267)*x_ref[2]
        ref[1,0,1,1]=(0.613312276864)*x_ref[0]**o+(-0.644237218776)*x_ref[0]+(0.81927272895)*x_ref[1]**o+(-0.12971860556)*x_ref[1]+(0.214379406799)*x_ref[2]**o+(0.856335675043)*x_ref[2]
        ref[1,0,2,0]=(-0.699925173766)*x_ref[0]**o+(-0.560613547952)*x_ref[0]+(-0.355325107637)*x_ref[1]**o+(-0.598747822041)*x_ref[1]+(0.243087037553)*x_ref[2]**o+(-0.484176993015)*x_ref[2]
        ref[1,0,2,1]=(-0.218912300848)*x_ref[0]**o+(-0.259483312969)*x_ref[0]+(-0.863788186295)*x_ref[1]**o+(0.924423774757)*x_ref[1]+(-0.0823752192204)*x_ref[2]**o+(-0.928377900602)*x_ref[2]
        ref[1,1,0,0]=(-0.368901144425)*x_ref[0]**o+(0.544104655592)*x_ref[0]+(0.321295151176)*x_ref[1]**o+(0.395692754134)*x_ref[1]+(-0.721319111754)*x_ref[2]**o+(-0.698518442137)*x_ref[2]
        ref[1,1,0,1]=(-0.592867336047)*x_ref[0]**o+(0.145120231038)*x_ref[0]+(0.110390771366)*x_ref[1]**o+(0.168333368086)*x_ref[1]+(-0.577866419921)*x_ref[2]**o+(0.705027708469)*x_ref[2]
        ref[1,1,1,0]=(-0.210603428235)*x_ref[0]**o+(-0.250156447116)*x_ref[0]+(-0.397803248976)*x_ref[1]**o+(0.332163380867)*x_ref[1]+(-0.250648216541)*x_ref[2]**o+(-0.0693729055697)*x_ref[2]
        ref[1,1,1,1]=(0.324927558096)*x_ref[0]**o+(-0.262726116223)*x_ref[0]+(-0.555439758488)*x_ref[1]**o+(-0.865019248609)*x_ref[1]+(-0.0971907425857)*x_ref[2]**o+(-0.742359236711)*x_ref[2]
        ref[1,1,2,0]=(0.161058869414)*x_ref[0]**o+(0.349191500791)*x_ref[0]+(0.468336473315)*x_ref[1]**o+(-0.856364775995)*x_ref[1]+(0.491736696689)*x_ref[2]**o+(0.352984104173)*x_ref[2]
        ref[1,1,2,1]=(-0.243275976508)*x_ref[0]**o+(-0.785605388878)*x_ref[0]+(-0.282478919296)*x_ref[1]**o+(-0.728851522341)*x_ref[1]+(0.375134848237)*x_ref[2]**o+(-0.486200561256)*x_ref[2]
        ref[1,2,0,0]=(-0.172241451405)*x_ref[0]**o+(0.422361498196)*x_ref[0]+(0.464862755164)*x_ref[1]**o+(0.466661717105)*x_ref[1]+(-0.886209677512)*x_ref[2]**o+(-0.43529076789)*x_ref[2]
        ref[1,2,0,1]=(0.946582222405)*x_ref[0]**o+(0.513640973894)*x_ref[0]+(-0.923788000633)*x_ref[1]**o+(-0.358163567195)*x_ref[1]+(-0.709999132129)*x_ref[2]**o+(-0.761163299694)*x_ref[2]
        ref[1,2,1,0]=(0.236339981889)*x_ref[0]**o+(0.259874866134)*x_ref[0]+(-0.32460762691)*x_ref[1]**o+(-0.629786950313)*x_ref[1]+(-0.679261685402)*x_ref[2]**o+(0.593069192918)*x_ref[2]
        ref[1,2,1,1]=(0.11006800009)*x_ref[0]**o+(0.970532991787)*x_ref[0]+(-0.467608188361)*x_ref[1]**o+(-0.959281984552)*x_ref[1]+(0.610396622519)*x_ref[2]**o+(0.282468600058)*x_ref[2]
        ref[1,2,2,0]=(-0.656419157742)*x_ref[0]**o+(0.234027308171)*x_ref[0]+(-0.67027206978)*x_ref[1]**o+(-0.746034436817)*x_ref[1]+(-0.771894073652)*x_ref[2]**o+(0.949545830287)*x_ref[2]
        ref[1,2,2,1]=(0.615884236811)*x_ref[0]**o+(-0.670156612555)*x_ref[0]+(0.543165493345)*x_ref[1]**o+(-0.618294277594)*x_ref[1]+(-0.974114255061)*x_ref[2]**o+(-0.762320542298)*x_ref[2]
        ref[1,3,0,0]=(0.096201957648)*x_ref[0]**o+(0.440870816208)*x_ref[0]+(0.124448859607)*x_ref[1]**o+(0.507919407561)*x_ref[1]+(-0.401666263687)*x_ref[2]**o+(-0.437653713723)*x_ref[2]
        ref[1,3,0,1]=(0.165129229294)*x_ref[0]**o+(-0.353217428955)*x_ref[0]+(0.473666065504)*x_ref[1]**o+(0.907667166988)*x_ref[1]+(-0.864298533288)*x_ref[2]**o+(0.641505117281)*x_ref[2]
        ref[1,3,1,0]=(0.381900624425)*x_ref[0]**o+(0.427520178597)*x_ref[0]+(-0.560279540293)*x_ref[1]**o+(0.190814532271)*x_ref[1]+(-0.187947688629)*x_ref[2]**o+(-0.127694148593)*x_ref[2]
        ref[1,3,1,1]=(-0.903921363895)*x_ref[0]**o+(-0.659942040828)*x_ref[0]+(-0.549460213492)*x_ref[1]**o+(0.869909651895)*x_ref[1]+(-0.962273439484)*x_ref[2]**o+(-0.046365857751)*x_ref[2]
        ref[1,3,2,0]=(-0.021688482187)*x_ref[0]**o+(-0.209531000717)*x_ref[0]+(0.668564395133)*x_ref[1]**o+(-0.0347978700975)*x_ref[1]+(0.947677147488)*x_ref[2]**o+(-0.0152385052475)*x_ref[2]
        ref[1,3,2,1]=(0.880839485817)*x_ref[0]**o+(-0.319275929312)*x_ref[0]+(-0.940056657485)*x_ref[1]**o+(0.570855397726)*x_ref[1]+(0.279710812074)*x_ref[2]**o+(0.496622215051)*x_ref[2]
        ref[1,4,0,0]=(-0.139033645615)*x_ref[0]**o+(-0.271453722366)*x_ref[0]+(-0.268516720244)*x_ref[1]**o+(-0.0467555017438)*x_ref[1]+(-0.41618262265)*x_ref[2]**o+(0.315320363813)*x_ref[2]
        ref[1,4,0,1]=(0.296108607204)*x_ref[0]**o+(-0.184582165661)*x_ref[0]+(0.121416543744)*x_ref[1]**o+(-0.922284926751)*x_ref[1]+(-0.437600089668)*x_ref[2]**o+(-0.608800626513)*x_ref[2]
        ref[1,4,1,0]=(0.8942028423)*x_ref[0]**o+(0.673402831188)*x_ref[0]+(-0.622529195562)*x_ref[1]**o+(-0.266052338798)*x_ref[1]+(-0.273167851702)*x_ref[2]**o+(-0.312211982747)*x_ref[2]
        ref[1,4,1,1]=(-0.102315711178)*x_ref[0]**o+(-0.885515719371)*x_ref[0]+(-0.532910101204)*x_ref[1]**o+(0.798525944348)*x_ref[1]+(0.640586802303)*x_ref[2]**o+(0.36308806536)*x_ref[2]
        ref[1,4,2,0]=(0.376793506541)*x_ref[0]**o+(-0.667063564875)*x_ref[0]+(0.609591985816)*x_ref[1]**o+(0.28347745142)*x_ref[1]+(-0.751832936302)*x_ref[2]**o+(-0.683738856965)*x_ref[2]
        ref[1,4,2,1]=(-0.51412252819)*x_ref[0]**o+(0.559560297206)*x_ref[0]+(0.751134603429)*x_ref[1]**o+(-0.0031812310797)*x_ref[1]+(-0.525776122178)*x_ref[2]**o+(0.338471942848)*x_ref[2]
        ref[2,0,0,0]=(0.937572244905)*x_ref[0]**o+(-0.293020870379)*x_ref[0]+(-0.918591520817)*x_ref[1]**o+(0.622194477261)*x_ref[1]+(0.31641482851)*x_ref[2]**o+(0.276912266864)*x_ref[2]
        ref[2,0,0,1]=(0.853041599998)*x_ref[0]**o+(-0.0220623724171)*x_ref[0]+(0.89909044296)*x_ref[1]**o+(-0.838782642732)*x_ref[1]+(0.0133197592042)*x_ref[2]**o+(-0.868977618993)*x_ref[2]
        ref[2,0,1,0]=(-0.325799042976)*x_ref[0]**o+(0.746269971994)*x_ref[0]+(0.882671061757)*x_ref[1]**o+(-0.284133373252)*x_ref[1]+(0.120086549521)*x_ref[2]**o+(-0.397352610969)*x_ref[2]
        ref[2,0,1,1]=(0.282336934951)*x_ref[0]**o+(-0.339893978547)*x_ref[0]+(-0.0861660125386)*x_ref[1]**o+(0.283063579798)*x_ref[1]+(0.714762478569)*x_ref[2]**o+(-0.497021718549)*x_ref[2]
        ref[2,0,2,0]=(0.891782779491)*x_ref[0]**o+(-0.817647577644)*x_ref[0]+(0.364088142085)*x_ref[1]**o+(-0.950577637048)*x_ref[1]+(0.4070663557)*x_ref[2]**o+(0.270753179246)*x_ref[2]
        ref[2,0,2,1]=(0.316594913808)*x_ref[0]**o+(0.310731486851)*x_ref[0]+(0.250421828765)*x_ref[1]**o+(0.772786141458)*x_ref[1]+(0.64843040679)*x_ref[2]**o+(-0.691903332511)*x_ref[2]
        ref[2,1,0,0]=(0.87180715028)*x_ref[0]**o+(-0.953173230375)*x_ref[0]+(-0.823516344548)*x_ref[1]**o+(0.934653756548)*x_ref[1]+(-0.537124962489)*x_ref[2]**o+(-0.233365939659)*x_ref[2]
        ref[2,1,0,1]=(-0.407189744631)*x_ref[0]**o+(-0.754425720288)*x_ref[0]+(0.737526767897)*x_ref[1]**o+(0.745393943502)*x_ref[1]+(0.811744293685)*x_ref[2]**o+(-0.459556659177)*x_ref[2]
        ref[2,1,1,0]=(-0.693529259877)*x_ref[0]**o+(0.580197938048)*x_ref[0]+(0.506055440405)*x_ref[1]**o+(0.122531014759)*x_ref[1]+(0.427736137775)*x_ref[2]**o+(-0.83260140401)*x_ref[2]
        ref[2,1,1,1]=(0.542621975026)*x_ref[0]**o+(-0.568425713106)*x_ref[0]+(0.303002043446)*x_ref[1]**o+(0.363286261651)*x_ref[1]+(0.2729476289)*x_ref[2]**o+(-0.0113629744659)*x_ref[2]
        ref[2,1,2,0]=(-0.266750235694)*x_ref[0]**o+(0.10116400306)*x_ref[0]+(0.315977962897)*x_ref[1]**o+(-0.406236201454)*x_ref[1]+(-0.586845072374)*x_ref[2]**o+(0.513937047282)*x_ref[2]
        ref[2,1,2,1]=(-0.518780811975)*x_ref[0]**o+(0.275053640829)*x_ref[0]+(0.4234970646)*x_ref[1]**o+(0.389585163376)*x_ref[1]+(-0.609771068778)*x_ref[2]**o+(-0.121467061755)*x_ref[2]
        ref[2,2,0,0]=(0.138823207466)*x_ref[0]**o+(-0.779837455014)*x_ref[0]+(0.70167049555)*x_ref[1]**o+(0.766428813369)*x_ref[1]+(0.114023530366)*x_ref[2]**o+(0.650098536187)*x_ref[2]
        ref[2,2,0,1]=(0.974998667633)*x_ref[0]**o+(0.958471050654)*x_ref[0]+(0.193515162503)*x_ref[1]**o+(-0.252148167253)*x_ref[1]+(0.495475460125)*x_ref[2]**o+(0.562693347983)*x_ref[2]
        ref[2,2,1,0]=(-0.552379548988)*x_ref[0]**o+(-0.831399971618)*x_ref[0]+(0.437581216647)*x_ref[1]**o+(-0.775331267696)*x_ref[1]+(0.729041628844)*x_ref[2]**o+(0.633113359893)*x_ref[2]
        ref[2,2,1,1]=(0.379858437049)*x_ref[0]**o+(0.923586504203)*x_ref[0]+(0.707244783969)*x_ref[1]**o+(-0.293157375059)*x_ref[1]+(-0.368717941876)*x_ref[2]**o+(-0.202610288616)*x_ref[2]
        ref[2,2,2,0]=(-0.804826108231)*x_ref[0]**o+(-0.0208712096312)*x_ref[0]+(0.0482547072498)*x_ref[1]**o+(0.852366543609)*x_ref[1]+(0.328962500136)*x_ref[2]**o+(-0.132777835512)*x_ref[2]
        ref[2,2,2,1]=(0.91373752217)*x_ref[0]**o+(-0.792112120729)*x_ref[0]+(-0.0457564087072)*x_ref[1]**o+(0.309694852949)*x_ref[1]+(0.089219285337)*x_ref[2]**o+(-0.516000514614)*x_ref[2]
        ref[2,3,0,0]=(0.343215129115)*x_ref[0]**o+(0.40804099837)*x_ref[0]+(-0.865942442914)*x_ref[1]**o+(0.842241683043)*x_ref[1]+(0.0727803297601)*x_ref[2]**o+(-0.395225682949)*x_ref[2]
        ref[2,3,0,1]=(-0.965340522771)*x_ref[0]**o+(-0.897056478677)*x_ref[0]+(-0.305205944062)*x_ref[1]**o+(0.393941789685)*x_ref[1]+(-0.253328379131)*x_ref[2]**o+(0.0951510469271)*x_ref[2]
        ref[2,3,1,0]=(0.536251514752)*x_ref[0]**o+(0.0629776386779)*x_ref[0]+(-0.706311222379)*x_ref[1]**o+(-0.84362242029)*x_ref[1]+(0.00110678760539)*x_ref[2]**o+(-0.30286059941)*x_ref[2]
        ref[2,3,1,1]=(0.114584737295)*x_ref[0]**o+(0.0708763738067)*x_ref[0]+(-0.454760062397)*x_ref[1]**o+(0.228742539922)*x_ref[1]+(0.514960810352)*x_ref[2]**o+(-0.815749211085)*x_ref[2]
        ref[2,3,2,0]=(0.246335688008)*x_ref[0]**o+(-0.239199299885)*x_ref[0]+(-0.447390151659)*x_ref[1]**o+(0.918949356391)*x_ref[1]+(-0.307864840175)*x_ref[2]**o+(0.560139080481)*x_ref[2]
        ref[2,3,2,1]=(0.118766035723)*x_ref[0]**o+(0.610711164899)*x_ref[0]+(0.336628771776)*x_ref[1]**o+(0.833699268751)*x_ref[1]+(-0.424694647429)*x_ref[2]**o+(-0.639929688104)*x_ref[2]
        ref[2,4,0,0]=(0.814543506987)*x_ref[0]**o+(-0.797684321368)*x_ref[0]+(0.337242635783)*x_ref[1]**o+(0.278139837072)*x_ref[1]+(0.633511526441)*x_ref[2]**o+(0.372175615028)*x_ref[2]
        ref[2,4,0,1]=(-0.0590668694094)*x_ref[0]**o+(0.256732067267)*x_ref[0]+(0.519737187577)*x_ref[1]**o+(-0.376391855948)*x_ref[1]+(-0.810934658651)*x_ref[2]**o+(0.826899476928)*x_ref[2]
        ref[2,4,1,0]=(-0.428976843661)*x_ref[0]**o+(0.672667833313)*x_ref[0]+(0.263194890946)*x_ref[1]**o+(-0.894790773661)*x_ref[1]+(-0.550298557295)*x_ref[2]**o+(-0.00560384035502)*x_ref[2]
        ref[2,4,1,1]=(-0.818372430144)*x_ref[0]**o+(-0.829920833073)*x_ref[0]+(-0.359560636119)*x_ref[1]**o+(-0.113484563205)*x_ref[1]+(0.586796110365)*x_ref[2]**o+(0.296820898426)*x_ref[2]
        ref[2,4,2,0]=(-0.691597969566)*x_ref[0]**o+(0.715813153538)*x_ref[0]+(0.954093612503)*x_ref[1]**o+(0.88803420754)*x_ref[1]+(0.697447667765)*x_ref[2]**o+(0.793730496625)*x_ref[2]
        ref[2,4,2,1]=(0.351099833336)*x_ref[0]**o+(-0.716419272807)*x_ref[0]+(0.318726674998)*x_ref[1]**o+(0.246151914414)*x_ref[1]+(-0.945520673884)*x_ref[2]**o+(-0.489026233954)*x_ref[2]
        ref[3,0,0,0]=(0.511598022804)*x_ref[0]**o+(0.567617269582)*x_ref[0]+(-0.526293319137)*x_ref[1]**o+(0.333750098146)*x_ref[1]+(-0.0869228849144)*x_ref[2]**o+(-0.204072834867)*x_ref[2]
        ref[3,0,0,1]=(0.905384007172)*x_ref[0]**o+(-0.661067390314)*x_ref[0]+(-0.470790632295)*x_ref[1]**o+(-0.149608017616)*x_ref[1]+(0.427879278267)*x_ref[2]**o+(0.987040179184)*x_ref[2]
        ref[3,0,1,0]=(-0.946562006809)*x_ref[0]**o+(0.64141909633)*x_ref[0]+(-0.762076826276)*x_ref[1]**o+(-0.642912850794)*x_ref[1]+(0.712567898559)*x_ref[2]**o+(-0.292128944205)*x_ref[2]
        ref[3,0,1,1]=(-0.955087949361)*x_ref[0]**o+(0.426588041133)*x_ref[0]+(0.339313104127)*x_ref[1]**o+(-0.375147344045)*x_ref[1]+(0.967259014679)*x_ref[2]**o+(0.504521081858)*x_ref[2]
        ref[3,0,2,0]=(0.141876598344)*x_ref[0]**o+(-0.335579252825)*x_ref[0]+(0.514552993602)*x_ref[1]**o+(-0.531116948499)*x_ref[1]+(-0.369319413141)*x_ref[2]**o+(0.0374413262362)*x_ref[2]
        ref[3,0,2,1]=(0.678360463658)*x_ref[0]**o+(-0.357900119646)*x_ref[0]+(0.300750717059)*x_ref[1]**o+(0.516196189448)*x_ref[1]+(0.0535466018203)*x_ref[2]**o+(-0.149961940786)*x_ref[2]
        ref[3,1,0,0]=(0.0975994351079)*x_ref[0]**o+(0.634544068499)*x_ref[0]+(-0.963217046352)*x_ref[1]**o+(-0.804658091456)*x_ref[1]+(-0.620132720823)*x_ref[2]**o+(0.862250619369)*x_ref[2]
        ref[3,1,0,1]=(0.210306328879)*x_ref[0]**o+(-0.457341187966)*x_ref[0]+(0.957556084801)*x_ref[1]**o+(0.704073170928)*x_ref[1]+(0.423134492133)*x_ref[2]**o+(-0.167268375412)*x_ref[2]
        ref[3,1,1,0]=(-0.909228817744)*x_ref[0]**o+(0.110665018921)*x_ref[0]+(0.36275630147)*x_ref[1]**o+(0.346866824763)*x_ref[1]+(-0.535600547489)*x_ref[2]**o+(0.627638814375)*x_ref[2]
        ref[3,1,1,1]=(0.328780933823)*x_ref[0]**o+(-0.249563994161)*x_ref[0]+(-0.555789062892)*x_ref[1]**o+(-0.917287525036)*x_ref[1]+(0.386191175017)*x_ref[2]**o+(0.123904319061)*x_ref[2]
        ref[3,1,2,0]=(-0.838409901623)*x_ref[0]**o+(0.673479705393)*x_ref[0]+(0.482921104917)*x_ref[1]**o+(0.136411388588)*x_ref[1]+(0.583674116225)*x_ref[2]**o+(0.305100836877)*x_ref[2]
        ref[3,1,2,1]=(0.952961008312)*x_ref[0]**o+(-0.0501981079532)*x_ref[0]+(0.661434464642)*x_ref[1]**o+(0.65030469399)*x_ref[1]+(-0.157562001295)*x_ref[2]**o+(-0.249551892917)*x_ref[2]
        ref[3,2,0,0]=(-0.0232804500286)*x_ref[0]**o+(-0.559229392203)*x_ref[0]+(0.127967930273)*x_ref[1]**o+(-0.635827181645)*x_ref[1]+(-0.0426703224383)*x_ref[2]**o+(0.47720230258)*x_ref[2]
        ref[3,2,0,1]=(-0.421658996106)*x_ref[0]**o+(-0.468260038077)*x_ref[0]+(0.479083572554)*x_ref[1]**o+(-0.214417881841)*x_ref[1]+(0.372430534261)*x_ref[2]**o+(0.133063789546)*x_ref[2]
        ref[3,2,1,0]=(0.905811586567)*x_ref[0]**o+(0.245447340384)*x_ref[0]+(-0.337650999146)*x_ref[1]**o+(0.908188235976)*x_ref[1]+(-0.804029751277)*x_ref[2]**o+(0.0512676456643)*x_ref[2]
        ref[3,2,1,1]=(-0.891486839409)*x_ref[0]**o+(0.20753794329)*x_ref[0]+(0.879183464252)*x_ref[1]**o+(0.902028906992)*x_ref[1]+(0.416776813598)*x_ref[2]**o+(0.491964374373)*x_ref[2]
        ref[3,2,2,0]=(0.653067027119)*x_ref[0]**o+(0.765293890745)*x_ref[0]+(-0.348902169508)*x_ref[1]**o+(0.324429035849)*x_ref[1]+(-0.363026365065)*x_ref[2]**o+(0.0816186405511)*x_ref[2]
        ref[3,2,2,1]=(-0.0415375052325)*x_ref[0]**o+(-0.480779075784)*x_ref[0]+(-0.137223552938)*x_ref[1]**o+(-0.915733625512)*x_ref[1]+(0.750199445159)*x_ref[2]**o+(-0.789818945171)*x_ref[2]
        ref[3,3,0,0]=(0.0744921239083)*x_ref[0]**o+(0.0700589144159)*x_ref[0]+(-0.327105425297)*x_ref[1]**o+(0.469843257765)*x_ref[1]+(-0.170191600844)*x_ref[2]**o+(0.732071534155)*x_ref[2]
        ref[3,3,0,1]=(0.172643927441)*x_ref[0]**o+(0.848924114383)*x_ref[0]+(-0.79125762228)*x_ref[1]**o+(-0.694725775976)*x_ref[1]+(-0.309710503985)*x_ref[2]**o+(-0.405310234927)*x_ref[2]
        ref[3,3,1,0]=(-0.465911787595)*x_ref[0]**o+(-0.71080474709)*x_ref[0]+(0.959925934203)*x_ref[1]**o+(-0.111110644825)*x_ref[1]+(-0.145349925218)*x_ref[2]**o+(0.122955165811)*x_ref[2]
        ref[3,3,1,1]=(-0.769111863754)*x_ref[0]**o+(0.348036938532)*x_ref[0]+(0.485890856474)*x_ref[1]**o+(0.382595218985)*x_ref[1]+(-0.897488692981)*x_ref[2]**o+(0.830822333819)*x_ref[2]
        ref[3,3,2,0]=(0.341890793681)*x_ref[0]**o+(-0.667145936571)*x_ref[0]+(-0.145094108268)*x_ref[1]**o+(0.771433911212)*x_ref[1]+(-0.299752164325)*x_ref[2]**o+(-0.357856549677)*x_ref[2]
        ref[3,3,2,1]=(0.26381838796)*x_ref[0]**o+(-0.44474724998)*x_ref[0]+(0.310283988195)*x_ref[1]**o+(0.698675181314)*x_ref[1]+(0.437624133238)*x_ref[2]**o+(0.394916054445)*x_ref[2]
        ref[3,4,0,0]=(-0.576499406674)*x_ref[0]**o+(0.770936634959)*x_ref[0]+(0.707441735783)*x_ref[1]**o+(-0.794953257043)*x_ref[1]+(-0.535342943209)*x_ref[2]**o+(-0.509446811787)*x_ref[2]
        ref[3,4,0,1]=(0.161115012113)*x_ref[0]**o+(0.11079092252)*x_ref[0]+(0.0633804181164)*x_ref[1]**o+(-0.406049293019)*x_ref[1]+(-0.824283994336)*x_ref[2]**o+(-0.481107560499)*x_ref[2]
        ref[3,4,1,0]=(-0.411133810295)*x_ref[0]**o+(0.988486625099)*x_ref[0]+(-0.36050191367)*x_ref[1]**o+(0.469479821965)*x_ref[1]+(-0.431175357325)*x_ref[2]**o+(-0.966295015263)*x_ref[2]
        ref[3,4,1,1]=(0.233731410367)*x_ref[0]**o+(-0.821900740766)*x_ref[0]+(-0.823123461385)*x_ref[1]**o+(-0.380537914421)*x_ref[1]+(-0.896146300355)*x_ref[2]**o+(-0.775172990257)*x_ref[2]
        ref[3,4,2,0]=(-0.96323363586)*x_ref[0]**o+(0.0935412405245)*x_ref[0]+(0.929333026259)*x_ref[1]**o+(-0.322236627348)*x_ref[1]+(-0.390898135964)*x_ref[2]**o+(-0.766178290162)*x_ref[2]
        ref[3,4,2,1]=(-0.773670533268)*x_ref[0]**o+(-0.156975157098)*x_ref[0]+(-0.91512996795)*x_ref[1]**o+(0.0247803682126)*x_ref[1]+(0.995159733782)*x_ref[2]**o+(-0.669224144225)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_ContinuousFunction_rank0(self):
      """
      tests interpolate for rank 0 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ContinuousFunction onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(),dim=dim)
      s=Data(0,(),w)
      ref=Data(0,(),w_ref)
      if dim==2:
        s=(-0.0411847356752)*x[0]**o+(-0.978252807465)*x[0]+(-0.765516202331)*x[1]**o+(-0.60628403943)*x[1]
        ref=(-0.0411847356752)*x_ref[0]**o+(-0.978252807465)*x_ref[0]+(-0.765516202331)*x_ref[1]**o+(-0.60628403943)*x_ref[1]
      else:
        s=(-0.520571545539)*x[0]**o+(0.516384538651)*x[0]+(-0.865529136972)*x[1]**o+(0.80164386579)*x[1]+(-0.330695181072)*x[2]**o+(-0.758708438741)*x[2]
        ref=(-0.520571545539)*x_ref[0]**o+(0.516384538651)*x_ref[0]+(-0.865529136972)*x_ref[1]**o+(0.80164386579)*x_ref[1]+(-0.330695181072)*x_ref[2]**o+(-0.758708438741)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_ContinuousFunction_rank1(self):
      """
      tests interpolate for rank 1 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ContinuousFunction onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(2,),dim=dim)
      s=Data(0,(2,),w)
      ref=Data(0,(2,),w_ref)
      if dim==2:
        s[0]=(0.505196018365)*x[0]**o+(0.731510032507)*x[0]+(-0.0776298756069)*x[1]**o+(0.223327557462)*x[1]
        s[1]=(-0.239558790149)*x[0]**o+(0.608302150075)*x[0]+(-0.840461486286)*x[1]**o+(-0.0797379857546)*x[1]
        ref[0]=(0.505196018365)*x_ref[0]**o+(0.731510032507)*x_ref[0]+(-0.0776298756069)*x_ref[1]**o+(0.223327557462)*x_ref[1]
        ref[1]=(-0.239558790149)*x_ref[0]**o+(0.608302150075)*x_ref[0]+(-0.840461486286)*x_ref[1]**o+(-0.0797379857546)*x_ref[1]
      else:
        s[0]=(-0.853964974211)*x[0]**o+(-0.436921639846)*x[0]+(-0.356712309058)*x[1]**o+(0.324151975277)*x[1]+(0.338538264809)*x[2]**o+(0.254654297562)*x[2]
        s[1]=(0.46616823299)*x[0]**o+(0.64792464198)*x[0]+(-0.655892247116)*x[1]**o+(-0.9837201382)*x[1]+(0.36604513951)*x[2]**o+(0.343454378544)*x[2]
        ref[0]=(-0.853964974211)*x_ref[0]**o+(-0.436921639846)*x_ref[0]+(-0.356712309058)*x_ref[1]**o+(0.324151975277)*x_ref[1]+(0.338538264809)*x_ref[2]**o+(0.254654297562)*x_ref[2]
        ref[1]=(0.46616823299)*x_ref[0]**o+(0.64792464198)*x_ref[0]+(-0.655892247116)*x_ref[1]**o+(-0.9837201382)*x_ref[1]+(0.36604513951)*x_ref[2]**o+(0.343454378544)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_ContinuousFunction_rank2(self):
      """
      tests interpolate for rank 2 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ContinuousFunction onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5),dim=dim)
      s=Data(0,(4, 5),w)
      ref=Data(0,(4, 5),w_ref)
      if dim==2:
        s[0,0]=(0.403739435229)*x[0]**o+(-0.976543623588)*x[0]+(0.408638664295)*x[1]**o+(0.290441090188)*x[1]
        s[0,1]=(-0.186112372212)*x[0]**o+(-0.885114653425)*x[0]+(0.79020447405)*x[1]**o+(-0.988138900337)*x[1]
        s[0,2]=(-0.749718519557)*x[0]**o+(-0.556662006553)*x[0]+(-0.388628750048)*x[1]**o+(0.488811308506)*x[1]
        s[0,3]=(0.55958060511)*x[0]**o+(-0.692678975478)*x[0]+(-0.507024092372)*x[1]**o+(0.547618022755)*x[1]
        s[0,4]=(0.393523537188)*x[0]**o+(-0.278215081058)*x[0]+(0.524241376975)*x[1]**o+(-0.18209236848)*x[1]
        s[1,0]=(0.277364605261)*x[0]**o+(0.867787382941)*x[0]+(-0.468812178843)*x[1]**o+(-0.915273555727)*x[1]
        s[1,1]=(-0.883469445503)*x[0]**o+(0.949713147084)*x[0]+(-0.195972546782)*x[1]**o+(-0.550840574214)*x[1]
        s[1,2]=(-0.0612993770053)*x[0]**o+(0.349066961111)*x[0]+(0.117833853269)*x[1]**o+(0.915145781122)*x[1]
        s[1,3]=(-0.221469359751)*x[0]**o+(0.860157157768)*x[0]+(0.334660915861)*x[1]**o+(0.0202241427453)*x[1]
        s[1,4]=(-0.859671577801)*x[0]**o+(-0.751878816346)*x[0]+(0.970178380651)*x[1]**o+(-0.852998297383)*x[1]
        s[2,0]=(-0.273019645199)*x[0]**o+(0.77546843291)*x[0]+(0.40198451057)*x[1]**o+(0.719427949335)*x[1]
        s[2,1]=(-0.615814919857)*x[0]**o+(-0.804830902046)*x[0]+(-0.55666668286)*x[1]**o+(-0.436274362037)*x[1]
        s[2,2]=(-0.257042626769)*x[0]**o+(-0.0674177291518)*x[0]+(-0.487890885639)*x[1]**o+(-0.276346354771)*x[1]
        s[2,3]=(-0.0863705553448)*x[0]**o+(-0.456256145846)*x[0]+(0.279365337349)*x[1]**o+(-0.788366887305)*x[1]
        s[2,4]=(0.00696265740333)*x[0]**o+(-0.903180556586)*x[0]+(0.299924222843)*x[1]**o+(0.432554144375)*x[1]
        s[3,0]=(0.928471341856)*x[0]**o+(-0.58681191854)*x[0]+(0.592280256168)*x[1]**o+(-0.486182643564)*x[1]
        s[3,1]=(-0.949679045727)*x[0]**o+(0.28219904154)*x[0]+(0.742879063098)*x[1]**o+(-0.0189753136959)*x[1]
        s[3,2]=(-0.386608884833)*x[0]**o+(0.771169732795)*x[0]+(-0.864253528029)*x[1]**o+(-0.0196378783985)*x[1]
        s[3,3]=(0.744817366451)*x[0]**o+(0.973334266028)*x[0]+(0.879085180711)*x[1]**o+(-0.682866003591)*x[1]
        s[3,4]=(-0.453687552098)*x[0]**o+(-0.421237560962)*x[0]+(0.55363269981)*x[1]**o+(-0.446292920116)*x[1]
        ref[0,0]=(0.403739435229)*x_ref[0]**o+(-0.976543623588)*x_ref[0]+(0.408638664295)*x_ref[1]**o+(0.290441090188)*x_ref[1]
        ref[0,1]=(-0.186112372212)*x_ref[0]**o+(-0.885114653425)*x_ref[0]+(0.79020447405)*x_ref[1]**o+(-0.988138900337)*x_ref[1]
        ref[0,2]=(-0.749718519557)*x_ref[0]**o+(-0.556662006553)*x_ref[0]+(-0.388628750048)*x_ref[1]**o+(0.488811308506)*x_ref[1]
        ref[0,3]=(0.55958060511)*x_ref[0]**o+(-0.692678975478)*x_ref[0]+(-0.507024092372)*x_ref[1]**o+(0.547618022755)*x_ref[1]
        ref[0,4]=(0.393523537188)*x_ref[0]**o+(-0.278215081058)*x_ref[0]+(0.524241376975)*x_ref[1]**o+(-0.18209236848)*x_ref[1]
        ref[1,0]=(0.277364605261)*x_ref[0]**o+(0.867787382941)*x_ref[0]+(-0.468812178843)*x_ref[1]**o+(-0.915273555727)*x_ref[1]
        ref[1,1]=(-0.883469445503)*x_ref[0]**o+(0.949713147084)*x_ref[0]+(-0.195972546782)*x_ref[1]**o+(-0.550840574214)*x_ref[1]
        ref[1,2]=(-0.0612993770053)*x_ref[0]**o+(0.349066961111)*x_ref[0]+(0.117833853269)*x_ref[1]**o+(0.915145781122)*x_ref[1]
        ref[1,3]=(-0.221469359751)*x_ref[0]**o+(0.860157157768)*x_ref[0]+(0.334660915861)*x_ref[1]**o+(0.0202241427453)*x_ref[1]
        ref[1,4]=(-0.859671577801)*x_ref[0]**o+(-0.751878816346)*x_ref[0]+(0.970178380651)*x_ref[1]**o+(-0.852998297383)*x_ref[1]
        ref[2,0]=(-0.273019645199)*x_ref[0]**o+(0.77546843291)*x_ref[0]+(0.40198451057)*x_ref[1]**o+(0.719427949335)*x_ref[1]
        ref[2,1]=(-0.615814919857)*x_ref[0]**o+(-0.804830902046)*x_ref[0]+(-0.55666668286)*x_ref[1]**o+(-0.436274362037)*x_ref[1]
        ref[2,2]=(-0.257042626769)*x_ref[0]**o+(-0.0674177291518)*x_ref[0]+(-0.487890885639)*x_ref[1]**o+(-0.276346354771)*x_ref[1]
        ref[2,3]=(-0.0863705553448)*x_ref[0]**o+(-0.456256145846)*x_ref[0]+(0.279365337349)*x_ref[1]**o+(-0.788366887305)*x_ref[1]
        ref[2,4]=(0.00696265740333)*x_ref[0]**o+(-0.903180556586)*x_ref[0]+(0.299924222843)*x_ref[1]**o+(0.432554144375)*x_ref[1]
        ref[3,0]=(0.928471341856)*x_ref[0]**o+(-0.58681191854)*x_ref[0]+(0.592280256168)*x_ref[1]**o+(-0.486182643564)*x_ref[1]
        ref[3,1]=(-0.949679045727)*x_ref[0]**o+(0.28219904154)*x_ref[0]+(0.742879063098)*x_ref[1]**o+(-0.0189753136959)*x_ref[1]
        ref[3,2]=(-0.386608884833)*x_ref[0]**o+(0.771169732795)*x_ref[0]+(-0.864253528029)*x_ref[1]**o+(-0.0196378783985)*x_ref[1]
        ref[3,3]=(0.744817366451)*x_ref[0]**o+(0.973334266028)*x_ref[0]+(0.879085180711)*x_ref[1]**o+(-0.682866003591)*x_ref[1]
        ref[3,4]=(-0.453687552098)*x_ref[0]**o+(-0.421237560962)*x_ref[0]+(0.55363269981)*x_ref[1]**o+(-0.446292920116)*x_ref[1]
      else:
        s[0,0]=(-0.200149935444)*x[0]**o+(0.507524859849)*x[0]+(0.619745083854)*x[1]**o+(0.627105339142)*x[1]+(0.563386979803)*x[2]**o+(-0.961618026889)*x[2]
        s[0,1]=(0.363899424005)*x[0]**o+(0.924425901309)*x[0]+(-0.658883398191)*x[1]**o+(0.481606823837)*x[1]+(-0.159370395541)*x[2]**o+(-0.311388746226)*x[2]
        s[0,2]=(-0.0293410199192)*x[0]**o+(0.256908390845)*x[0]+(-0.909877958762)*x[1]**o+(0.105112556474)*x[1]+(0.252859198357)*x[2]**o+(0.92186026337)*x[2]
        s[0,3]=(-0.00170582078327)*x[0]**o+(0.730720627097)*x[0]+(-0.801575927568)*x[1]**o+(0.455500499782)*x[1]+(0.380759901094)*x[2]**o+(-0.823963841034)*x[2]
        s[0,4]=(0.259808988031)*x[0]**o+(0.863791713598)*x[0]+(0.553188235768)*x[1]**o+(-0.464509312659)*x[1]+(0.42570171)*x[2]**o+(0.192687787615)*x[2]
        s[1,0]=(0.994753432059)*x[0]**o+(-0.68864217861)*x[0]+(-0.0204526591618)*x[1]**o+(-0.306443480795)*x[1]+(0.148406858899)*x[2]**o+(0.854009274563)*x[2]
        s[1,1]=(0.173721730295)*x[0]**o+(-0.987657366279)*x[0]+(0.253868939396)*x[1]**o+(-0.771588800243)*x[1]+(0.53766861285)*x[2]**o+(0.719877605356)*x[2]
        s[1,2]=(0.430924077377)*x[0]**o+(-0.746769688215)*x[0]+(0.828996359633)*x[1]**o+(-0.192590851955)*x[1]+(0.669402098684)*x[2]**o+(-0.962105583491)*x[2]
        s[1,3]=(-0.410727896031)*x[0]**o+(-0.833008011026)*x[0]+(-0.927201509946)*x[1]**o+(-0.876016513532)*x[1]+(0.0914553334551)*x[2]**o+(-0.348128799709)*x[2]
        s[1,4]=(-0.73394473562)*x[0]**o+(0.00706331438319)*x[0]+(-0.205588695357)*x[1]**o+(0.714894523918)*x[1]+(-0.713385984004)*x[2]**o+(-0.385550060437)*x[2]
        s[2,0]=(-0.0853783479307)*x[0]**o+(0.596547791334)*x[0]+(-0.0925317449093)*x[1]**o+(0.751414290883)*x[1]+(0.838736367666)*x[2]**o+(0.735536663934)*x[2]
        s[2,1]=(-0.159026214794)*x[0]**o+(0.949032823761)*x[0]+(-0.422829286588)*x[1]**o+(0.985896164837)*x[1]+(-0.758176201469)*x[2]**o+(0.283868871122)*x[2]
        s[2,2]=(-0.14639325109)*x[0]**o+(0.346468853582)*x[0]+(0.563105800299)*x[1]**o+(-0.531036765601)*x[1]+(-0.0990976207418)*x[2]**o+(0.750636702702)*x[2]
        s[2,3]=(0.849599418085)*x[0]**o+(-0.779136748258)*x[0]+(0.730951130588)*x[1]**o+(0.53628857622)*x[1]+(-0.947230186497)*x[2]**o+(-0.72450102135)*x[2]
        s[2,4]=(-0.700617249374)*x[0]**o+(-0.10080480368)*x[0]+(-0.507852228465)*x[1]**o+(-0.857417192918)*x[1]+(0.640292095556)*x[2]**o+(-0.929256110195)*x[2]
        s[3,0]=(0.653772333481)*x[0]**o+(0.000343679690948)*x[0]+(0.668997254669)*x[1]**o+(0.362901119538)*x[1]+(-0.830987535544)*x[2]**o+(-0.423720904628)*x[2]
        s[3,1]=(-0.0420273808908)*x[0]**o+(0.323011823157)*x[0]+(-0.24882314712)*x[1]**o+(-0.423985714012)*x[1]+(0.752345053106)*x[2]**o+(-0.160413317211)*x[2]
        s[3,2]=(0.890620617817)*x[0]**o+(0.783464544239)*x[0]+(0.790371129121)*x[1]**o+(0.237259212081)*x[1]+(0.610840739105)*x[2]**o+(0.973783326469)*x[2]
        s[3,3]=(-0.379379310577)*x[0]**o+(0.690137408246)*x[0]+(-0.779634423108)*x[1]**o+(-0.974077688357)*x[1]+(0.788216664671)*x[2]**o+(0.101022285352)*x[2]
        s[3,4]=(-0.505116742845)*x[0]**o+(0.923132757962)*x[0]+(-0.0146493711199)*x[1]**o+(0.994421781292)*x[1]+(-0.20080218328)*x[2]**o+(0.555271515554)*x[2]
        ref[0,0]=(-0.200149935444)*x_ref[0]**o+(0.507524859849)*x_ref[0]+(0.619745083854)*x_ref[1]**o+(0.627105339142)*x_ref[1]+(0.563386979803)*x_ref[2]**o+(-0.961618026889)*x_ref[2]
        ref[0,1]=(0.363899424005)*x_ref[0]**o+(0.924425901309)*x_ref[0]+(-0.658883398191)*x_ref[1]**o+(0.481606823837)*x_ref[1]+(-0.159370395541)*x_ref[2]**o+(-0.311388746226)*x_ref[2]
        ref[0,2]=(-0.0293410199192)*x_ref[0]**o+(0.256908390845)*x_ref[0]+(-0.909877958762)*x_ref[1]**o+(0.105112556474)*x_ref[1]+(0.252859198357)*x_ref[2]**o+(0.92186026337)*x_ref[2]
        ref[0,3]=(-0.00170582078327)*x_ref[0]**o+(0.730720627097)*x_ref[0]+(-0.801575927568)*x_ref[1]**o+(0.455500499782)*x_ref[1]+(0.380759901094)*x_ref[2]**o+(-0.823963841034)*x_ref[2]
        ref[0,4]=(0.259808988031)*x_ref[0]**o+(0.863791713598)*x_ref[0]+(0.553188235768)*x_ref[1]**o+(-0.464509312659)*x_ref[1]+(0.42570171)*x_ref[2]**o+(0.192687787615)*x_ref[2]
        ref[1,0]=(0.994753432059)*x_ref[0]**o+(-0.68864217861)*x_ref[0]+(-0.0204526591618)*x_ref[1]**o+(-0.306443480795)*x_ref[1]+(0.148406858899)*x_ref[2]**o+(0.854009274563)*x_ref[2]
        ref[1,1]=(0.173721730295)*x_ref[0]**o+(-0.987657366279)*x_ref[0]+(0.253868939396)*x_ref[1]**o+(-0.771588800243)*x_ref[1]+(0.53766861285)*x_ref[2]**o+(0.719877605356)*x_ref[2]
        ref[1,2]=(0.430924077377)*x_ref[0]**o+(-0.746769688215)*x_ref[0]+(0.828996359633)*x_ref[1]**o+(-0.192590851955)*x_ref[1]+(0.669402098684)*x_ref[2]**o+(-0.962105583491)*x_ref[2]
        ref[1,3]=(-0.410727896031)*x_ref[0]**o+(-0.833008011026)*x_ref[0]+(-0.927201509946)*x_ref[1]**o+(-0.876016513532)*x_ref[1]+(0.0914553334551)*x_ref[2]**o+(-0.348128799709)*x_ref[2]
        ref[1,4]=(-0.73394473562)*x_ref[0]**o+(0.00706331438319)*x_ref[0]+(-0.205588695357)*x_ref[1]**o+(0.714894523918)*x_ref[1]+(-0.713385984004)*x_ref[2]**o+(-0.385550060437)*x_ref[2]
        ref[2,0]=(-0.0853783479307)*x_ref[0]**o+(0.596547791334)*x_ref[0]+(-0.0925317449093)*x_ref[1]**o+(0.751414290883)*x_ref[1]+(0.838736367666)*x_ref[2]**o+(0.735536663934)*x_ref[2]
        ref[2,1]=(-0.159026214794)*x_ref[0]**o+(0.949032823761)*x_ref[0]+(-0.422829286588)*x_ref[1]**o+(0.985896164837)*x_ref[1]+(-0.758176201469)*x_ref[2]**o+(0.283868871122)*x_ref[2]
        ref[2,2]=(-0.14639325109)*x_ref[0]**o+(0.346468853582)*x_ref[0]+(0.563105800299)*x_ref[1]**o+(-0.531036765601)*x_ref[1]+(-0.0990976207418)*x_ref[2]**o+(0.750636702702)*x_ref[2]
        ref[2,3]=(0.849599418085)*x_ref[0]**o+(-0.779136748258)*x_ref[0]+(0.730951130588)*x_ref[1]**o+(0.53628857622)*x_ref[1]+(-0.947230186497)*x_ref[2]**o+(-0.72450102135)*x_ref[2]
        ref[2,4]=(-0.700617249374)*x_ref[0]**o+(-0.10080480368)*x_ref[0]+(-0.507852228465)*x_ref[1]**o+(-0.857417192918)*x_ref[1]+(0.640292095556)*x_ref[2]**o+(-0.929256110195)*x_ref[2]
        ref[3,0]=(0.653772333481)*x_ref[0]**o+(0.000343679690948)*x_ref[0]+(0.668997254669)*x_ref[1]**o+(0.362901119538)*x_ref[1]+(-0.830987535544)*x_ref[2]**o+(-0.423720904628)*x_ref[2]
        ref[3,1]=(-0.0420273808908)*x_ref[0]**o+(0.323011823157)*x_ref[0]+(-0.24882314712)*x_ref[1]**o+(-0.423985714012)*x_ref[1]+(0.752345053106)*x_ref[2]**o+(-0.160413317211)*x_ref[2]
        ref[3,2]=(0.890620617817)*x_ref[0]**o+(0.783464544239)*x_ref[0]+(0.790371129121)*x_ref[1]**o+(0.237259212081)*x_ref[1]+(0.610840739105)*x_ref[2]**o+(0.973783326469)*x_ref[2]
        ref[3,3]=(-0.379379310577)*x_ref[0]**o+(0.690137408246)*x_ref[0]+(-0.779634423108)*x_ref[1]**o+(-0.974077688357)*x_ref[1]+(0.788216664671)*x_ref[2]**o+(0.101022285352)*x_ref[2]
        ref[3,4]=(-0.505116742845)*x_ref[0]**o+(0.923132757962)*x_ref[0]+(-0.0146493711199)*x_ref[1]**o+(0.994421781292)*x_ref[1]+(-0.20080218328)*x_ref[2]**o+(0.555271515554)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_ContinuousFunction_rank3(self):
      """
      tests interpolate for rank 3 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ContinuousFunction onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(6, 2, 2),dim=dim)
      s=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2),w_ref)
      if dim==2:
        s[0,0,0]=(-0.564155350537)*x[0]**o+(-0.800380058082)*x[0]+(0.689149742912)*x[1]**o+(-0.730209879145)*x[1]
        s[0,0,1]=(-0.875752239419)*x[0]**o+(0.620770279709)*x[0]+(-0.405669056556)*x[1]**o+(-0.650461825052)*x[1]
        s[0,1,0]=(-0.986148210247)*x[0]**o+(-0.915312621763)*x[0]+(0.96862065918)*x[1]**o+(-0.789867091704)*x[1]
        s[0,1,1]=(0.937339541643)*x[0]**o+(0.707719614445)*x[0]+(-0.0185139390834)*x[1]**o+(0.370474130738)*x[1]
        s[1,0,0]=(0.351845644061)*x[0]**o+(-0.790772449987)*x[0]+(0.446801930114)*x[1]**o+(-0.79526180504)*x[1]
        s[1,0,1]=(0.807251261875)*x[0]**o+(-0.955059117804)*x[0]+(-0.797368998645)*x[1]**o+(-0.334719758479)*x[1]
        s[1,1,0]=(0.896257416662)*x[0]**o+(-0.630142892036)*x[0]+(0.697410593951)*x[1]**o+(-0.719230278941)*x[1]
        s[1,1,1]=(-0.628604182238)*x[0]**o+(-0.98443009117)*x[0]+(0.213668228562)*x[1]**o+(0.67702641015)*x[1]
        s[2,0,0]=(0.14317864629)*x[0]**o+(-0.668340038203)*x[0]+(-0.624330217214)*x[1]**o+(-0.18225231407)*x[1]
        s[2,0,1]=(-0.120451964275)*x[0]**o+(0.476453331533)*x[0]+(0.428548558072)*x[1]**o+(0.487245871651)*x[1]
        s[2,1,0]=(0.957821516578)*x[0]**o+(0.245067538044)*x[0]+(0.59057574623)*x[1]**o+(0.799481912975)*x[1]
        s[2,1,1]=(0.157267505975)*x[0]**o+(0.486829437266)*x[0]+(0.72573406263)*x[1]**o+(-0.854913668061)*x[1]
        s[3,0,0]=(0.981080241547)*x[0]**o+(-0.475318323052)*x[0]+(-0.309438112923)*x[1]**o+(0.432416200997)*x[1]
        s[3,0,1]=(0.598520022432)*x[0]**o+(-0.384489046077)*x[0]+(-0.563263372262)*x[1]**o+(-0.787292999177)*x[1]
        s[3,1,0]=(-0.581355275119)*x[0]**o+(-0.404673888552)*x[0]+(-0.481631833004)*x[1]**o+(-0.873167338173)*x[1]
        s[3,1,1]=(-0.809669221808)*x[0]**o+(0.271025961643)*x[0]+(0.532318218814)*x[1]**o+(0.892267754107)*x[1]
        s[4,0,0]=(-0.654686292022)*x[0]**o+(-0.21708498353)*x[0]+(-0.277205071363)*x[1]**o+(-0.427753053795)*x[1]
        s[4,0,1]=(-0.540515032413)*x[0]**o+(-0.902343725594)*x[0]+(-0.880588922705)*x[1]**o+(0.449347508903)*x[1]
        s[4,1,0]=(-0.429101982872)*x[0]**o+(0.537796527897)*x[0]+(0.417004167989)*x[1]**o+(-0.0986410894762)*x[1]
        s[4,1,1]=(0.760414524096)*x[0]**o+(0.587360702148)*x[0]+(-0.977847482262)*x[1]**o+(0.233948889011)*x[1]
        s[5,0,0]=(0.489553684109)*x[0]**o+(-0.576529833035)*x[0]+(0.534315888534)*x[1]**o+(-0.27340402533)*x[1]
        s[5,0,1]=(-0.399620876418)*x[0]**o+(-0.313714298147)*x[0]+(0.180876722218)*x[1]**o+(-0.783972277649)*x[1]
        s[5,1,0]=(-0.35422230106)*x[0]**o+(0.015751912455)*x[0]+(0.10875420172)*x[1]**o+(-0.264805104189)*x[1]
        s[5,1,1]=(0.275494542521)*x[0]**o+(-0.758423085522)*x[0]+(0.223846690671)*x[1]**o+(0.717226976294)*x[1]
        ref[0,0,0]=(-0.564155350537)*x_ref[0]**o+(-0.800380058082)*x_ref[0]+(0.689149742912)*x_ref[1]**o+(-0.730209879145)*x_ref[1]
        ref[0,0,1]=(-0.875752239419)*x_ref[0]**o+(0.620770279709)*x_ref[0]+(-0.405669056556)*x_ref[1]**o+(-0.650461825052)*x_ref[1]
        ref[0,1,0]=(-0.986148210247)*x_ref[0]**o+(-0.915312621763)*x_ref[0]+(0.96862065918)*x_ref[1]**o+(-0.789867091704)*x_ref[1]
        ref[0,1,1]=(0.937339541643)*x_ref[0]**o+(0.707719614445)*x_ref[0]+(-0.0185139390834)*x_ref[1]**o+(0.370474130738)*x_ref[1]
        ref[1,0,0]=(0.351845644061)*x_ref[0]**o+(-0.790772449987)*x_ref[0]+(0.446801930114)*x_ref[1]**o+(-0.79526180504)*x_ref[1]
        ref[1,0,1]=(0.807251261875)*x_ref[0]**o+(-0.955059117804)*x_ref[0]+(-0.797368998645)*x_ref[1]**o+(-0.334719758479)*x_ref[1]
        ref[1,1,0]=(0.896257416662)*x_ref[0]**o+(-0.630142892036)*x_ref[0]+(0.697410593951)*x_ref[1]**o+(-0.719230278941)*x_ref[1]
        ref[1,1,1]=(-0.628604182238)*x_ref[0]**o+(-0.98443009117)*x_ref[0]+(0.213668228562)*x_ref[1]**o+(0.67702641015)*x_ref[1]
        ref[2,0,0]=(0.14317864629)*x_ref[0]**o+(-0.668340038203)*x_ref[0]+(-0.624330217214)*x_ref[1]**o+(-0.18225231407)*x_ref[1]
        ref[2,0,1]=(-0.120451964275)*x_ref[0]**o+(0.476453331533)*x_ref[0]+(0.428548558072)*x_ref[1]**o+(0.487245871651)*x_ref[1]
        ref[2,1,0]=(0.957821516578)*x_ref[0]**o+(0.245067538044)*x_ref[0]+(0.59057574623)*x_ref[1]**o+(0.799481912975)*x_ref[1]
        ref[2,1,1]=(0.157267505975)*x_ref[0]**o+(0.486829437266)*x_ref[0]+(0.72573406263)*x_ref[1]**o+(-0.854913668061)*x_ref[1]
        ref[3,0,0]=(0.981080241547)*x_ref[0]**o+(-0.475318323052)*x_ref[0]+(-0.309438112923)*x_ref[1]**o+(0.432416200997)*x_ref[1]
        ref[3,0,1]=(0.598520022432)*x_ref[0]**o+(-0.384489046077)*x_ref[0]+(-0.563263372262)*x_ref[1]**o+(-0.787292999177)*x_ref[1]
        ref[3,1,0]=(-0.581355275119)*x_ref[0]**o+(-0.404673888552)*x_ref[0]+(-0.481631833004)*x_ref[1]**o+(-0.873167338173)*x_ref[1]
        ref[3,1,1]=(-0.809669221808)*x_ref[0]**o+(0.271025961643)*x_ref[0]+(0.532318218814)*x_ref[1]**o+(0.892267754107)*x_ref[1]
        ref[4,0,0]=(-0.654686292022)*x_ref[0]**o+(-0.21708498353)*x_ref[0]+(-0.277205071363)*x_ref[1]**o+(-0.427753053795)*x_ref[1]
        ref[4,0,1]=(-0.540515032413)*x_ref[0]**o+(-0.902343725594)*x_ref[0]+(-0.880588922705)*x_ref[1]**o+(0.449347508903)*x_ref[1]
        ref[4,1,0]=(-0.429101982872)*x_ref[0]**o+(0.537796527897)*x_ref[0]+(0.417004167989)*x_ref[1]**o+(-0.0986410894762)*x_ref[1]
        ref[4,1,1]=(0.760414524096)*x_ref[0]**o+(0.587360702148)*x_ref[0]+(-0.977847482262)*x_ref[1]**o+(0.233948889011)*x_ref[1]
        ref[5,0,0]=(0.489553684109)*x_ref[0]**o+(-0.576529833035)*x_ref[0]+(0.534315888534)*x_ref[1]**o+(-0.27340402533)*x_ref[1]
        ref[5,0,1]=(-0.399620876418)*x_ref[0]**o+(-0.313714298147)*x_ref[0]+(0.180876722218)*x_ref[1]**o+(-0.783972277649)*x_ref[1]
        ref[5,1,0]=(-0.35422230106)*x_ref[0]**o+(0.015751912455)*x_ref[0]+(0.10875420172)*x_ref[1]**o+(-0.264805104189)*x_ref[1]
        ref[5,1,1]=(0.275494542521)*x_ref[0]**o+(-0.758423085522)*x_ref[0]+(0.223846690671)*x_ref[1]**o+(0.717226976294)*x_ref[1]
      else:
        s[0,0,0]=(-0.774622440694)*x[0]**o+(-0.871495007805)*x[0]+(0.462893985101)*x[1]**o+(0.97809884411)*x[1]+(-0.294782632822)*x[2]**o+(0.758086059139)*x[2]
        s[0,0,1]=(0.593542531653)*x[0]**o+(-0.623508645258)*x[0]+(-0.869924719996)*x[1]**o+(0.683987028818)*x[1]+(0.214983363659)*x[2]**o+(0.937703017041)*x[2]
        s[0,1,0]=(0.683415745305)*x[0]**o+(0.514284128555)*x[0]+(-0.495949103899)*x[1]**o+(0.0846543741985)*x[1]+(0.713593706483)*x[2]**o+(0.68727976891)*x[2]
        s[0,1,1]=(0.890309435361)*x[0]**o+(-0.971477359218)*x[0]+(0.832140831851)*x[1]**o+(-0.208499413601)*x[1]+(0.632915577047)*x[2]**o+(0.255341853742)*x[2]
        s[1,0,0]=(0.180392491602)*x[0]**o+(-0.535445127886)*x[0]+(-0.785876938514)*x[1]**o+(0.0816315522405)*x[1]+(0.648551332906)*x[2]**o+(0.0429567512952)*x[2]
        s[1,0,1]=(-0.662744890855)*x[0]**o+(0.961507967025)*x[0]+(-0.535104990223)*x[1]**o+(-0.100555677207)*x[1]+(0.254286176467)*x[2]**o+(0.23822457275)*x[2]
        s[1,1,0]=(0.0618685350699)*x[0]**o+(0.801588371288)*x[0]+(0.775832395529)*x[1]**o+(0.76315781257)*x[1]+(0.201501889875)*x[2]**o+(-0.594080987915)*x[2]
        s[1,1,1]=(-0.472648941872)*x[0]**o+(-0.268595087901)*x[0]+(-0.634032640513)*x[1]**o+(0.895658995237)*x[1]+(-0.311243350516)*x[2]**o+(-0.941899709416)*x[2]
        s[2,0,0]=(-0.937843815317)*x[0]**o+(-0.732052451935)*x[0]+(-0.195729427352)*x[1]**o+(-0.686479979743)*x[1]+(0.233916070094)*x[2]**o+(-0.0840356688737)*x[2]
        s[2,0,1]=(0.010530087739)*x[0]**o+(-0.790921976555)*x[0]+(0.0748807338793)*x[1]**o+(-0.903727762342)*x[1]+(0.627134883108)*x[2]**o+(-0.0387413340061)*x[2]
        s[2,1,0]=(0.546735016177)*x[0]**o+(0.315849018818)*x[0]+(-0.95747551189)*x[1]**o+(0.0565452708853)*x[1]+(0.305498701829)*x[2]**o+(0.122314734589)*x[2]
        s[2,1,1]=(-0.838026718661)*x[0]**o+(0.128953039794)*x[0]+(0.624067160575)*x[1]**o+(0.444434080889)*x[1]+(-0.835923583772)*x[2]**o+(-0.286578073481)*x[2]
        s[3,0,0]=(0.00360603498589)*x[0]**o+(0.842043331421)*x[0]+(0.362497876527)*x[1]**o+(0.0681458369022)*x[1]+(0.75852693193)*x[2]**o+(-0.53716235)*x[2]
        s[3,0,1]=(-0.159678757928)*x[0]**o+(0.883308054535)*x[0]+(0.21591905568)*x[1]**o+(-0.898265348642)*x[1]+(-0.951423349936)*x[2]**o+(-0.240034084504)*x[2]
        s[3,1,0]=(0.692230687386)*x[0]**o+(0.0335587781762)*x[0]+(-0.766571197965)*x[1]**o+(0.128927596751)*x[1]+(0.986505471196)*x[2]**o+(0.149177674029)*x[2]
        s[3,1,1]=(0.135591222661)*x[0]**o+(0.931657866742)*x[0]+(-0.4633142936)*x[1]**o+(0.868618051626)*x[1]+(-0.563215256065)*x[2]**o+(0.937159665351)*x[2]
        s[4,0,0]=(-0.00496897513325)*x[0]**o+(0.64627613088)*x[0]+(-0.63362415209)*x[1]**o+(0.875299573404)*x[1]+(0.00915422996533)*x[2]**o+(-0.898719885994)*x[2]
        s[4,0,1]=(0.531986621993)*x[0]**o+(-0.592503622446)*x[0]+(0.168725128639)*x[1]**o+(-0.415718155368)*x[1]+(0.153807804669)*x[2]**o+(0.0517662389301)*x[2]
        s[4,1,0]=(-0.40167686609)*x[0]**o+(0.765205907898)*x[0]+(-0.330400749167)*x[1]**o+(-0.227287643007)*x[1]+(0.86248041007)*x[2]**o+(0.475472570688)*x[2]
        s[4,1,1]=(-0.379429448909)*x[0]**o+(-0.746283001238)*x[0]+(0.451902471866)*x[1]**o+(0.975850402176)*x[1]+(-0.752666040136)*x[2]**o+(0.310437725938)*x[2]
        s[5,0,0]=(-0.790243943683)*x[0]**o+(-0.483698083361)*x[0]+(0.298354468428)*x[1]**o+(-0.430878231617)*x[1]+(-0.970790186654)*x[2]**o+(0.295573821695)*x[2]
        s[5,0,1]=(0.342850991179)*x[0]**o+(0.716485409508)*x[0]+(0.0929941902588)*x[1]**o+(-0.804594470152)*x[1]+(-0.6262904765)*x[2]**o+(0.1281559082)*x[2]
        s[5,1,0]=(0.6989819417)*x[0]**o+(0.513436682661)*x[0]+(-0.398488542709)*x[1]**o+(0.429421472572)*x[1]+(0.0491769595898)*x[2]**o+(0.402704297546)*x[2]
        s[5,1,1]=(-0.638592369916)*x[0]**o+(0.164631869734)*x[0]+(0.306302338543)*x[1]**o+(0.293774443655)*x[1]+(-0.536801505818)*x[2]**o+(0.758326129903)*x[2]
        ref[0,0,0]=(-0.774622440694)*x_ref[0]**o+(-0.871495007805)*x_ref[0]+(0.462893985101)*x_ref[1]**o+(0.97809884411)*x_ref[1]+(-0.294782632822)*x_ref[2]**o+(0.758086059139)*x_ref[2]
        ref[0,0,1]=(0.593542531653)*x_ref[0]**o+(-0.623508645258)*x_ref[0]+(-0.869924719996)*x_ref[1]**o+(0.683987028818)*x_ref[1]+(0.214983363659)*x_ref[2]**o+(0.937703017041)*x_ref[2]
        ref[0,1,0]=(0.683415745305)*x_ref[0]**o+(0.514284128555)*x_ref[0]+(-0.495949103899)*x_ref[1]**o+(0.0846543741985)*x_ref[1]+(0.713593706483)*x_ref[2]**o+(0.68727976891)*x_ref[2]
        ref[0,1,1]=(0.890309435361)*x_ref[0]**o+(-0.971477359218)*x_ref[0]+(0.832140831851)*x_ref[1]**o+(-0.208499413601)*x_ref[1]+(0.632915577047)*x_ref[2]**o+(0.255341853742)*x_ref[2]
        ref[1,0,0]=(0.180392491602)*x_ref[0]**o+(-0.535445127886)*x_ref[0]+(-0.785876938514)*x_ref[1]**o+(0.0816315522405)*x_ref[1]+(0.648551332906)*x_ref[2]**o+(0.0429567512952)*x_ref[2]
        ref[1,0,1]=(-0.662744890855)*x_ref[0]**o+(0.961507967025)*x_ref[0]+(-0.535104990223)*x_ref[1]**o+(-0.100555677207)*x_ref[1]+(0.254286176467)*x_ref[2]**o+(0.23822457275)*x_ref[2]
        ref[1,1,0]=(0.0618685350699)*x_ref[0]**o+(0.801588371288)*x_ref[0]+(0.775832395529)*x_ref[1]**o+(0.76315781257)*x_ref[1]+(0.201501889875)*x_ref[2]**o+(-0.594080987915)*x_ref[2]
        ref[1,1,1]=(-0.472648941872)*x_ref[0]**o+(-0.268595087901)*x_ref[0]+(-0.634032640513)*x_ref[1]**o+(0.895658995237)*x_ref[1]+(-0.311243350516)*x_ref[2]**o+(-0.941899709416)*x_ref[2]
        ref[2,0,0]=(-0.937843815317)*x_ref[0]**o+(-0.732052451935)*x_ref[0]+(-0.195729427352)*x_ref[1]**o+(-0.686479979743)*x_ref[1]+(0.233916070094)*x_ref[2]**o+(-0.0840356688737)*x_ref[2]
        ref[2,0,1]=(0.010530087739)*x_ref[0]**o+(-0.790921976555)*x_ref[0]+(0.0748807338793)*x_ref[1]**o+(-0.903727762342)*x_ref[1]+(0.627134883108)*x_ref[2]**o+(-0.0387413340061)*x_ref[2]
        ref[2,1,0]=(0.546735016177)*x_ref[0]**o+(0.315849018818)*x_ref[0]+(-0.95747551189)*x_ref[1]**o+(0.0565452708853)*x_ref[1]+(0.305498701829)*x_ref[2]**o+(0.122314734589)*x_ref[2]
        ref[2,1,1]=(-0.838026718661)*x_ref[0]**o+(0.128953039794)*x_ref[0]+(0.624067160575)*x_ref[1]**o+(0.444434080889)*x_ref[1]+(-0.835923583772)*x_ref[2]**o+(-0.286578073481)*x_ref[2]
        ref[3,0,0]=(0.00360603498589)*x_ref[0]**o+(0.842043331421)*x_ref[0]+(0.362497876527)*x_ref[1]**o+(0.0681458369022)*x_ref[1]+(0.75852693193)*x_ref[2]**o+(-0.53716235)*x_ref[2]
        ref[3,0,1]=(-0.159678757928)*x_ref[0]**o+(0.883308054535)*x_ref[0]+(0.21591905568)*x_ref[1]**o+(-0.898265348642)*x_ref[1]+(-0.951423349936)*x_ref[2]**o+(-0.240034084504)*x_ref[2]
        ref[3,1,0]=(0.692230687386)*x_ref[0]**o+(0.0335587781762)*x_ref[0]+(-0.766571197965)*x_ref[1]**o+(0.128927596751)*x_ref[1]+(0.986505471196)*x_ref[2]**o+(0.149177674029)*x_ref[2]
        ref[3,1,1]=(0.135591222661)*x_ref[0]**o+(0.931657866742)*x_ref[0]+(-0.4633142936)*x_ref[1]**o+(0.868618051626)*x_ref[1]+(-0.563215256065)*x_ref[2]**o+(0.937159665351)*x_ref[2]
        ref[4,0,0]=(-0.00496897513325)*x_ref[0]**o+(0.64627613088)*x_ref[0]+(-0.63362415209)*x_ref[1]**o+(0.875299573404)*x_ref[1]+(0.00915422996533)*x_ref[2]**o+(-0.898719885994)*x_ref[2]
        ref[4,0,1]=(0.531986621993)*x_ref[0]**o+(-0.592503622446)*x_ref[0]+(0.168725128639)*x_ref[1]**o+(-0.415718155368)*x_ref[1]+(0.153807804669)*x_ref[2]**o+(0.0517662389301)*x_ref[2]
        ref[4,1,0]=(-0.40167686609)*x_ref[0]**o+(0.765205907898)*x_ref[0]+(-0.330400749167)*x_ref[1]**o+(-0.227287643007)*x_ref[1]+(0.86248041007)*x_ref[2]**o+(0.475472570688)*x_ref[2]
        ref[4,1,1]=(-0.379429448909)*x_ref[0]**o+(-0.746283001238)*x_ref[0]+(0.451902471866)*x_ref[1]**o+(0.975850402176)*x_ref[1]+(-0.752666040136)*x_ref[2]**o+(0.310437725938)*x_ref[2]
        ref[5,0,0]=(-0.790243943683)*x_ref[0]**o+(-0.483698083361)*x_ref[0]+(0.298354468428)*x_ref[1]**o+(-0.430878231617)*x_ref[1]+(-0.970790186654)*x_ref[2]**o+(0.295573821695)*x_ref[2]
        ref[5,0,1]=(0.342850991179)*x_ref[0]**o+(0.716485409508)*x_ref[0]+(0.0929941902588)*x_ref[1]**o+(-0.804594470152)*x_ref[1]+(-0.6262904765)*x_ref[2]**o+(0.1281559082)*x_ref[2]
        ref[5,1,0]=(0.6989819417)*x_ref[0]**o+(0.513436682661)*x_ref[0]+(-0.398488542709)*x_ref[1]**o+(0.429421472572)*x_ref[1]+(0.0491769595898)*x_ref[2]**o+(0.402704297546)*x_ref[2]
        ref[5,1,1]=(-0.638592369916)*x_ref[0]**o+(0.164631869734)*x_ref[0]+(0.306302338543)*x_ref[1]**o+(0.293774443655)*x_ref[1]+(-0.536801505818)*x_ref[2]**o+(0.758326129903)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_ContinuousFunction_rank4(self):
      """
      tests interpolate for rank 4 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ContinuousFunction onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5, 3, 2),dim=dim)
      s=Data(0,(4, 5, 3, 2),w)
      ref=Data(0,(4, 5, 3, 2),w_ref)
      if dim==2:
        s[0,0,0,0]=(-0.474386706114)*x[0]**o+(-0.0739442523179)*x[0]+(-0.934839682165)*x[1]**o+(-0.874748715361)*x[1]
        s[0,0,0,1]=(0.970835666471)*x[0]**o+(-0.210304396563)*x[0]+(-0.245577454075)*x[1]**o+(0.706874293911)*x[1]
        s[0,0,1,0]=(-0.886508211386)*x[0]**o+(0.40749471331)*x[0]+(-0.575539262859)*x[1]**o+(-0.714242182567)*x[1]
        s[0,0,1,1]=(0.0925249369772)*x[0]**o+(-0.326000469119)*x[0]+(0.427385578633)*x[1]**o+(0.723913449012)*x[1]
        s[0,0,2,0]=(0.178301174425)*x[0]**o+(-0.935408769107)*x[0]+(0.185756314077)*x[1]**o+(0.779639226265)*x[1]
        s[0,0,2,1]=(0.517627638569)*x[0]**o+(0.833228507097)*x[0]+(0.509792014643)*x[1]**o+(0.512759085233)*x[1]
        s[0,1,0,0]=(0.943220804246)*x[0]**o+(0.358695377859)*x[0]+(0.483282522344)*x[1]**o+(-0.780850766978)*x[1]
        s[0,1,0,1]=(0.228281969805)*x[0]**o+(0.378112450475)*x[0]+(-0.265569820367)*x[1]**o+(0.793063241358)*x[1]
        s[0,1,1,0]=(-0.991239767457)*x[0]**o+(0.584083573937)*x[0]+(0.00907947673395)*x[1]**o+(-0.0949125419342)*x[1]
        s[0,1,1,1]=(0.312976776903)*x[0]**o+(-0.572058365439)*x[0]+(0.337493203667)*x[1]**o+(-0.559749276661)*x[1]
        s[0,1,2,0]=(-0.154473081077)*x[0]**o+(0.946407875413)*x[0]+(0.721108882799)*x[1]**o+(0.409146005045)*x[1]
        s[0,1,2,1]=(0.42027715889)*x[0]**o+(-0.892090996743)*x[0]+(-0.718807145454)*x[1]**o+(0.229975539898)*x[1]
        s[0,2,0,0]=(0.378117082397)*x[0]**o+(-0.613916810385)*x[0]+(-0.564348246694)*x[1]**o+(0.101261637507)*x[1]
        s[0,2,0,1]=(0.839696159524)*x[0]**o+(0.24184619932)*x[0]+(-0.095338864129)*x[1]**o+(0.903106275587)*x[1]
        s[0,2,1,0]=(0.620479230354)*x[0]**o+(-0.34112105179)*x[0]+(0.624902166524)*x[1]**o+(-0.693289910121)*x[1]
        s[0,2,1,1]=(-0.39760464469)*x[0]**o+(-0.921383269336)*x[0]+(0.753657132321)*x[1]**o+(0.324768021371)*x[1]
        s[0,2,2,0]=(0.0243534816004)*x[0]**o+(0.290663121221)*x[0]+(0.0305666628617)*x[1]**o+(0.0446693318891)*x[1]
        s[0,2,2,1]=(-0.14154275886)*x[0]**o+(0.881207887999)*x[0]+(0.399029780289)*x[1]**o+(0.168497381665)*x[1]
        s[0,3,0,0]=(-0.233073561791)*x[0]**o+(-0.513212321666)*x[0]+(-0.651480028725)*x[1]**o+(0.147498573261)*x[1]
        s[0,3,0,1]=(-0.918136406734)*x[0]**o+(-0.279161242114)*x[0]+(0.983432543886)*x[1]**o+(-0.723257302563)*x[1]
        s[0,3,1,0]=(0.290070399137)*x[0]**o+(0.485832779323)*x[0]+(0.955571801349)*x[1]**o+(0.552993034163)*x[1]
        s[0,3,1,1]=(0.688570077213)*x[0]**o+(0.578716466405)*x[0]+(-0.40829418423)*x[1]**o+(-0.918196317732)*x[1]
        s[0,3,2,0]=(0.403069347997)*x[0]**o+(0.119390454784)*x[0]+(-0.322018782913)*x[1]**o+(0.224814772968)*x[1]
        s[0,3,2,1]=(-0.23084264112)*x[0]**o+(-0.398235674194)*x[0]+(-0.730556055687)*x[1]**o+(0.293981158644)*x[1]
        s[0,4,0,0]=(-0.0984201873925)*x[0]**o+(0.018359257988)*x[0]+(-0.0781876875229)*x[1]**o+(0.253344618889)*x[1]
        s[0,4,0,1]=(-0.20899724864)*x[0]**o+(-0.722019204308)*x[0]+(0.555162582715)*x[1]**o+(-0.958068826429)*x[1]
        s[0,4,1,0]=(0.477725976915)*x[0]**o+(0.295872229679)*x[0]+(-0.828208555398)*x[1]**o+(-0.931582818519)*x[1]
        s[0,4,1,1]=(0.721285453077)*x[0]**o+(-0.042927949943)*x[0]+(0.680231405034)*x[1]**o+(-0.16043805248)*x[1]
        s[0,4,2,0]=(0.985988751136)*x[0]**o+(0.885785903078)*x[0]+(-0.0410955856477)*x[1]**o+(-0.552089008201)*x[1]
        s[0,4,2,1]=(0.847466818225)*x[0]**o+(-0.51057282448)*x[0]+(0.163775791256)*x[1]**o+(0.0152553094575)*x[1]
        s[1,0,0,0]=(0.0343855486006)*x[0]**o+(-0.214334464898)*x[0]+(-0.573344070879)*x[1]**o+(0.24905956136)*x[1]
        s[1,0,0,1]=(0.783338978316)*x[0]**o+(0.781324842059)*x[0]+(-0.847509450777)*x[1]**o+(-0.858994488444)*x[1]
        s[1,0,1,0]=(0.59719044885)*x[0]**o+(0.347038111035)*x[0]+(-0.385761256521)*x[1]**o+(-0.571166989318)*x[1]
        s[1,0,1,1]=(0.636911008507)*x[0]**o+(-0.825376916398)*x[0]+(0.32394534216)*x[1]**o+(0.653119502004)*x[1]
        s[1,0,2,0]=(-0.669732532783)*x[0]**o+(-0.145427031253)*x[0]+(-0.303693857831)*x[1]**o+(-0.727878524833)*x[1]
        s[1,0,2,1]=(0.621061585764)*x[0]**o+(0.243432217146)*x[0]+(-0.888610099037)*x[1]**o+(0.298051589144)*x[1]
        s[1,1,0,0]=(0.357341721813)*x[0]**o+(0.346506498237)*x[0]+(-0.555625437961)*x[1]**o+(-0.0662579456147)*x[1]
        s[1,1,0,1]=(0.96955420495)*x[0]**o+(-0.190486134786)*x[0]+(-0.900124677941)*x[1]**o+(0.659993963277)*x[1]
        s[1,1,1,0]=(-0.517837108406)*x[0]**o+(0.00779665389526)*x[0]+(-0.0544022169324)*x[1]**o+(0.888217855388)*x[1]
        s[1,1,1,1]=(0.252515646862)*x[0]**o+(0.328948655235)*x[0]+(0.265916563145)*x[1]**o+(-0.491865215095)*x[1]
        s[1,1,2,0]=(0.922520219928)*x[0]**o+(0.463811222486)*x[0]+(0.651244201962)*x[1]**o+(0.982565677231)*x[1]
        s[1,1,2,1]=(0.257711278265)*x[0]**o+(0.23971381197)*x[0]+(-0.168559977915)*x[1]**o+(0.453909616568)*x[1]
        s[1,2,0,0]=(0.0426933948917)*x[0]**o+(-0.477381944712)*x[0]+(0.503797983762)*x[1]**o+(-0.943092701839)*x[1]
        s[1,2,0,1]=(0.913591544613)*x[0]**o+(-0.895063818412)*x[0]+(0.252264576564)*x[1]**o+(0.424846745491)*x[1]
        s[1,2,1,0]=(0.4199934247)*x[0]**o+(-0.128668379749)*x[0]+(-0.157997092493)*x[1]**o+(-0.174476338072)*x[1]
        s[1,2,1,1]=(0.394310091468)*x[0]**o+(0.599539798636)*x[0]+(0.133087615453)*x[1]**o+(-0.22271509754)*x[1]
        s[1,2,2,0]=(0.559521330007)*x[0]**o+(-0.411300693025)*x[0]+(-0.543921885179)*x[1]**o+(-0.0506834937614)*x[1]
        s[1,2,2,1]=(-0.423996886775)*x[0]**o+(0.14409349326)*x[0]+(-0.970632608145)*x[1]**o+(0.582937355294)*x[1]
        s[1,3,0,0]=(-0.884972026022)*x[0]**o+(-0.872260393893)*x[0]+(0.224463013859)*x[1]**o+(-0.245013144519)*x[1]
        s[1,3,0,1]=(-0.834873836296)*x[0]**o+(-0.778611513205)*x[0]+(0.989113281842)*x[1]**o+(0.847047912646)*x[1]
        s[1,3,1,0]=(-0.0900703847239)*x[0]**o+(-0.0295919916877)*x[0]+(0.510380741505)*x[1]**o+(-0.00354494905936)*x[1]
        s[1,3,1,1]=(0.0422103052661)*x[0]**o+(0.31376324435)*x[0]+(0.831890412519)*x[1]**o+(0.764406560706)*x[1]
        s[1,3,2,0]=(0.496699862489)*x[0]**o+(0.477514051712)*x[0]+(0.156435695163)*x[1]**o+(0.339443828388)*x[1]
        s[1,3,2,1]=(0.396689832399)*x[0]**o+(0.243605505552)*x[0]+(-0.0357728699149)*x[1]**o+(0.930734881576)*x[1]
        s[1,4,0,0]=(-0.90788627878)*x[0]**o+(0.82597715373)*x[0]+(0.287988132693)*x[1]**o+(-0.834307302134)*x[1]
        s[1,4,0,1]=(-0.942882799669)*x[0]**o+(-0.149609905045)*x[0]+(-0.340025012164)*x[1]**o+(0.539546162654)*x[1]
        s[1,4,1,0]=(-0.244377326727)*x[0]**o+(-0.586439384775)*x[0]+(0.89506494604)*x[1]**o+(0.233226803834)*x[1]
        s[1,4,1,1]=(0.864582224514)*x[0]**o+(0.0687754700765)*x[0]+(-0.641556536115)*x[1]**o+(0.22036806322)*x[1]
        s[1,4,2,0]=(-0.552589117461)*x[0]**o+(-0.705244093999)*x[0]+(0.378837921567)*x[1]**o+(-0.173968977986)*x[1]
        s[1,4,2,1]=(0.846900000181)*x[0]**o+(-0.52589472084)*x[0]+(0.872400573242)*x[1]**o+(0.182683546911)*x[1]
        s[2,0,0,0]=(0.305006901177)*x[0]**o+(-0.732858762152)*x[0]+(-0.882239786812)*x[1]**o+(0.481712295538)*x[1]
        s[2,0,0,1]=(0.836332373747)*x[0]**o+(0.63905574965)*x[0]+(0.039226709213)*x[1]**o+(0.704758586936)*x[1]
        s[2,0,1,0]=(0.51805564252)*x[0]**o+(0.641883965761)*x[0]+(0.901663292208)*x[1]**o+(0.375272715842)*x[1]
        s[2,0,1,1]=(-0.980058913052)*x[0]**o+(0.385619663167)*x[0]+(0.0722192483217)*x[1]**o+(0.00650731529315)*x[1]
        s[2,0,2,0]=(-0.25791454536)*x[0]**o+(0.460015385804)*x[0]+(0.234970544559)*x[1]**o+(-0.11962323176)*x[1]
        s[2,0,2,1]=(0.023390322577)*x[0]**o+(-0.104449150638)*x[0]+(0.149724956284)*x[1]**o+(-0.309848596985)*x[1]
        s[2,1,0,0]=(0.603565629307)*x[0]**o+(-0.143206239367)*x[0]+(-0.0430666954645)*x[1]**o+(-0.154546675756)*x[1]
        s[2,1,0,1]=(-0.980112660961)*x[0]**o+(0.419145835585)*x[0]+(-0.76202669209)*x[1]**o+(-0.648615698697)*x[1]
        s[2,1,1,0]=(0.805964945679)*x[0]**o+(0.650144650504)*x[0]+(0.308110033125)*x[1]**o+(0.0666476104838)*x[1]
        s[2,1,1,1]=(0.42561398203)*x[0]**o+(0.259276259654)*x[0]+(-0.215194398301)*x[1]**o+(0.880282506885)*x[1]
        s[2,1,2,0]=(-0.413779899229)*x[0]**o+(-0.796859182258)*x[0]+(0.606865226329)*x[1]**o+(-0.783313517188)*x[1]
        s[2,1,2,1]=(0.402767439742)*x[0]**o+(-0.692468937875)*x[0]+(0.27866251803)*x[1]**o+(0.95092559773)*x[1]
        s[2,2,0,0]=(0.749107532378)*x[0]**o+(-0.364633672479)*x[0]+(0.970710337513)*x[1]**o+(0.1986749977)*x[1]
        s[2,2,0,1]=(-0.722832876805)*x[0]**o+(-0.238825576333)*x[0]+(-0.101752751414)*x[1]**o+(-0.960067435044)*x[1]
        s[2,2,1,0]=(-0.704825996403)*x[0]**o+(0.74244530617)*x[0]+(-0.586571188279)*x[1]**o+(0.841297962355)*x[1]
        s[2,2,1,1]=(0.586127464995)*x[0]**o+(0.454050196541)*x[0]+(0.225294694159)*x[1]**o+(0.705925824031)*x[1]
        s[2,2,2,0]=(0.535446366333)*x[0]**o+(-0.787565531495)*x[0]+(-0.566053725536)*x[1]**o+(0.468968266813)*x[1]
        s[2,2,2,1]=(0.449236149776)*x[0]**o+(0.589799099557)*x[0]+(0.189540632578)*x[1]**o+(-0.775036139482)*x[1]
        s[2,3,0,0]=(0.979572074948)*x[0]**o+(-0.806420343846)*x[0]+(0.661682487968)*x[1]**o+(-0.615846811222)*x[1]
        s[2,3,0,1]=(0.021072244205)*x[0]**o+(0.878344628134)*x[0]+(-0.381067816502)*x[1]**o+(0.229027446937)*x[1]
        s[2,3,1,0]=(0.0182291979593)*x[0]**o+(0.625595584968)*x[0]+(-0.824161018488)*x[1]**o+(-0.316191235785)*x[1]
        s[2,3,1,1]=(-0.0168799900253)*x[0]**o+(-0.137230078645)*x[0]+(0.0240280311645)*x[1]**o+(-0.364099905801)*x[1]
        s[2,3,2,0]=(0.374193672582)*x[0]**o+(0.429814892173)*x[0]+(0.0194417431204)*x[1]**o+(0.830453174612)*x[1]
        s[2,3,2,1]=(-0.683758441576)*x[0]**o+(-0.759635036652)*x[0]+(-0.50102074202)*x[1]**o+(0.123764523628)*x[1]
        s[2,4,0,0]=(-0.0817296101008)*x[0]**o+(0.730474427452)*x[0]+(-0.0733544873209)*x[1]**o+(0.416303527556)*x[1]
        s[2,4,0,1]=(0.258598296267)*x[0]**o+(0.686740404407)*x[0]+(0.543414259362)*x[1]**o+(0.518071987904)*x[1]
        s[2,4,1,0]=(-0.32475099592)*x[0]**o+(-0.0460901003729)*x[0]+(0.275611466137)*x[1]**o+(0.8415303665)*x[1]
        s[2,4,1,1]=(0.5015310862)*x[0]**o+(-0.71451126551)*x[0]+(0.897703909295)*x[1]**o+(-0.0476780709524)*x[1]
        s[2,4,2,0]=(0.612722779576)*x[0]**o+(0.424485459817)*x[0]+(-0.136026922168)*x[1]**o+(0.16727850944)*x[1]
        s[2,4,2,1]=(-0.841159017194)*x[0]**o+(0.856949657441)*x[0]+(-0.975786438976)*x[1]**o+(-0.407229683889)*x[1]
        s[3,0,0,0]=(0.833459943775)*x[0]**o+(-0.230178834842)*x[0]+(-0.660801597291)*x[1]**o+(-0.738869823127)*x[1]
        s[3,0,0,1]=(0.30987970183)*x[0]**o+(-0.610707485471)*x[0]+(0.628798231358)*x[1]**o+(-0.0491116840572)*x[1]
        s[3,0,1,0]=(0.649739974341)*x[0]**o+(-0.450666468651)*x[0]+(0.901638992166)*x[1]**o+(-0.787549067401)*x[1]
        s[3,0,1,1]=(0.934583955853)*x[0]**o+(-0.859086605977)*x[0]+(0.0975143570756)*x[1]**o+(0.0350097808797)*x[1]
        s[3,0,2,0]=(0.837723849965)*x[0]**o+(-0.665696123522)*x[0]+(-0.70629678851)*x[1]**o+(-0.468945853667)*x[1]
        s[3,0,2,1]=(0.526302645633)*x[0]**o+(-0.0979568077223)*x[0]+(-0.767623484018)*x[1]**o+(-0.21912588217)*x[1]
        s[3,1,0,0]=(0.18832505785)*x[0]**o+(0.45004009293)*x[0]+(-0.823702397711)*x[1]**o+(-0.912642057295)*x[1]
        s[3,1,0,1]=(0.554483721388)*x[0]**o+(-0.607925140409)*x[0]+(-0.998308584629)*x[1]**o+(-0.538184347898)*x[1]
        s[3,1,1,0]=(-0.682363035722)*x[0]**o+(0.55318030401)*x[0]+(0.562408350319)*x[1]**o+(0.220145757421)*x[1]
        s[3,1,1,1]=(0.924816884714)*x[0]**o+(-0.295767203325)*x[0]+(0.230343937118)*x[1]**o+(0.957747143338)*x[1]
        s[3,1,2,0]=(0.235422171194)*x[0]**o+(0.264750603603)*x[0]+(0.13053017764)*x[1]**o+(0.628805980195)*x[1]
        s[3,1,2,1]=(-0.603111233811)*x[0]**o+(-0.325257245906)*x[0]+(-0.879402225173)*x[1]**o+(0.977736365104)*x[1]
        s[3,2,0,0]=(0.463242222753)*x[0]**o+(-0.802230749276)*x[0]+(0.085387156258)*x[1]**o+(0.18309223952)*x[1]
        s[3,2,0,1]=(0.778060805693)*x[0]**o+(0.32136550684)*x[0]+(0.775284717696)*x[1]**o+(0.151449430741)*x[1]
        s[3,2,1,0]=(0.288473735099)*x[0]**o+(0.675142038399)*x[0]+(-0.313098988069)*x[1]**o+(-0.771883975974)*x[1]
        s[3,2,1,1]=(0.715304130316)*x[0]**o+(0.994375393616)*x[0]+(-0.874667784061)*x[1]**o+(-0.29510377129)*x[1]
        s[3,2,2,0]=(-0.211496384269)*x[0]**o+(0.146746732763)*x[0]+(0.234317983556)*x[1]**o+(0.844710015939)*x[1]
        s[3,2,2,1]=(0.557426563891)*x[0]**o+(0.181380049852)*x[0]+(-0.068830884747)*x[1]**o+(0.424778582562)*x[1]
        s[3,3,0,0]=(-0.784389821334)*x[0]**o+(0.533073578323)*x[0]+(-0.172622226211)*x[1]**o+(-0.0882056556263)*x[1]
        s[3,3,0,1]=(-0.369034971239)*x[0]**o+(-0.323409858793)*x[0]+(0.408766889438)*x[1]**o+(-0.653470515807)*x[1]
        s[3,3,1,0]=(0.239411265848)*x[0]**o+(-0.672719205684)*x[0]+(-0.751715345243)*x[1]**o+(-0.4586462848)*x[1]
        s[3,3,1,1]=(-0.937724235502)*x[0]**o+(-0.9284117207)*x[0]+(-0.584507468424)*x[1]**o+(0.779562243368)*x[1]
        s[3,3,2,0]=(-0.563406066207)*x[0]**o+(0.115504525413)*x[0]+(0.0230187346201)*x[1]**o+(0.960113641875)*x[1]
        s[3,3,2,1]=(0.792686350755)*x[0]**o+(0.560920910518)*x[0]+(0.0766597516323)*x[1]**o+(-0.728345800422)*x[1]
        s[3,4,0,0]=(-0.182383283107)*x[0]**o+(0.604818623768)*x[0]+(0.756868334363)*x[1]**o+(0.0259669270239)*x[1]
        s[3,4,0,1]=(0.547592534695)*x[0]**o+(0.666130932562)*x[0]+(-0.922839241546)*x[1]**o+(-0.143547846716)*x[1]
        s[3,4,1,0]=(-0.43295353947)*x[0]**o+(0.342981294189)*x[0]+(0.785780092257)*x[1]**o+(0.480176709076)*x[1]
        s[3,4,1,1]=(-0.859691385907)*x[0]**o+(0.124338939229)*x[0]+(-0.883988405006)*x[1]**o+(0.0683789905855)*x[1]
        s[3,4,2,0]=(-0.106494101633)*x[0]**o+(0.276437588849)*x[0]+(0.408900924262)*x[1]**o+(-0.479798281555)*x[1]
        s[3,4,2,1]=(-0.226633020661)*x[0]**o+(0.487277344856)*x[0]+(-0.0241726779927)*x[1]**o+(-0.778823212273)*x[1]
        ref[0,0,0,0]=(-0.474386706114)*x_ref[0]**o+(-0.0739442523179)*x_ref[0]+(-0.934839682165)*x_ref[1]**o+(-0.874748715361)*x_ref[1]
        ref[0,0,0,1]=(0.970835666471)*x_ref[0]**o+(-0.210304396563)*x_ref[0]+(-0.245577454075)*x_ref[1]**o+(0.706874293911)*x_ref[1]
        ref[0,0,1,0]=(-0.886508211386)*x_ref[0]**o+(0.40749471331)*x_ref[0]+(-0.575539262859)*x_ref[1]**o+(-0.714242182567)*x_ref[1]
        ref[0,0,1,1]=(0.0925249369772)*x_ref[0]**o+(-0.326000469119)*x_ref[0]+(0.427385578633)*x_ref[1]**o+(0.723913449012)*x_ref[1]
        ref[0,0,2,0]=(0.178301174425)*x_ref[0]**o+(-0.935408769107)*x_ref[0]+(0.185756314077)*x_ref[1]**o+(0.779639226265)*x_ref[1]
        ref[0,0,2,1]=(0.517627638569)*x_ref[0]**o+(0.833228507097)*x_ref[0]+(0.509792014643)*x_ref[1]**o+(0.512759085233)*x_ref[1]
        ref[0,1,0,0]=(0.943220804246)*x_ref[0]**o+(0.358695377859)*x_ref[0]+(0.483282522344)*x_ref[1]**o+(-0.780850766978)*x_ref[1]
        ref[0,1,0,1]=(0.228281969805)*x_ref[0]**o+(0.378112450475)*x_ref[0]+(-0.265569820367)*x_ref[1]**o+(0.793063241358)*x_ref[1]
        ref[0,1,1,0]=(-0.991239767457)*x_ref[0]**o+(0.584083573937)*x_ref[0]+(0.00907947673395)*x_ref[1]**o+(-0.0949125419342)*x_ref[1]
        ref[0,1,1,1]=(0.312976776903)*x_ref[0]**o+(-0.572058365439)*x_ref[0]+(0.337493203667)*x_ref[1]**o+(-0.559749276661)*x_ref[1]
        ref[0,1,2,0]=(-0.154473081077)*x_ref[0]**o+(0.946407875413)*x_ref[0]+(0.721108882799)*x_ref[1]**o+(0.409146005045)*x_ref[1]
        ref[0,1,2,1]=(0.42027715889)*x_ref[0]**o+(-0.892090996743)*x_ref[0]+(-0.718807145454)*x_ref[1]**o+(0.229975539898)*x_ref[1]
        ref[0,2,0,0]=(0.378117082397)*x_ref[0]**o+(-0.613916810385)*x_ref[0]+(-0.564348246694)*x_ref[1]**o+(0.101261637507)*x_ref[1]
        ref[0,2,0,1]=(0.839696159524)*x_ref[0]**o+(0.24184619932)*x_ref[0]+(-0.095338864129)*x_ref[1]**o+(0.903106275587)*x_ref[1]
        ref[0,2,1,0]=(0.620479230354)*x_ref[0]**o+(-0.34112105179)*x_ref[0]+(0.624902166524)*x_ref[1]**o+(-0.693289910121)*x_ref[1]
        ref[0,2,1,1]=(-0.39760464469)*x_ref[0]**o+(-0.921383269336)*x_ref[0]+(0.753657132321)*x_ref[1]**o+(0.324768021371)*x_ref[1]
        ref[0,2,2,0]=(0.0243534816004)*x_ref[0]**o+(0.290663121221)*x_ref[0]+(0.0305666628617)*x_ref[1]**o+(0.0446693318891)*x_ref[1]
        ref[0,2,2,1]=(-0.14154275886)*x_ref[0]**o+(0.881207887999)*x_ref[0]+(0.399029780289)*x_ref[1]**o+(0.168497381665)*x_ref[1]
        ref[0,3,0,0]=(-0.233073561791)*x_ref[0]**o+(-0.513212321666)*x_ref[0]+(-0.651480028725)*x_ref[1]**o+(0.147498573261)*x_ref[1]
        ref[0,3,0,1]=(-0.918136406734)*x_ref[0]**o+(-0.279161242114)*x_ref[0]+(0.983432543886)*x_ref[1]**o+(-0.723257302563)*x_ref[1]
        ref[0,3,1,0]=(0.290070399137)*x_ref[0]**o+(0.485832779323)*x_ref[0]+(0.955571801349)*x_ref[1]**o+(0.552993034163)*x_ref[1]
        ref[0,3,1,1]=(0.688570077213)*x_ref[0]**o+(0.578716466405)*x_ref[0]+(-0.40829418423)*x_ref[1]**o+(-0.918196317732)*x_ref[1]
        ref[0,3,2,0]=(0.403069347997)*x_ref[0]**o+(0.119390454784)*x_ref[0]+(-0.322018782913)*x_ref[1]**o+(0.224814772968)*x_ref[1]
        ref[0,3,2,1]=(-0.23084264112)*x_ref[0]**o+(-0.398235674194)*x_ref[0]+(-0.730556055687)*x_ref[1]**o+(0.293981158644)*x_ref[1]
        ref[0,4,0,0]=(-0.0984201873925)*x_ref[0]**o+(0.018359257988)*x_ref[0]+(-0.0781876875229)*x_ref[1]**o+(0.253344618889)*x_ref[1]
        ref[0,4,0,1]=(-0.20899724864)*x_ref[0]**o+(-0.722019204308)*x_ref[0]+(0.555162582715)*x_ref[1]**o+(-0.958068826429)*x_ref[1]
        ref[0,4,1,0]=(0.477725976915)*x_ref[0]**o+(0.295872229679)*x_ref[0]+(-0.828208555398)*x_ref[1]**o+(-0.931582818519)*x_ref[1]
        ref[0,4,1,1]=(0.721285453077)*x_ref[0]**o+(-0.042927949943)*x_ref[0]+(0.680231405034)*x_ref[1]**o+(-0.16043805248)*x_ref[1]
        ref[0,4,2,0]=(0.985988751136)*x_ref[0]**o+(0.885785903078)*x_ref[0]+(-0.0410955856477)*x_ref[1]**o+(-0.552089008201)*x_ref[1]
        ref[0,4,2,1]=(0.847466818225)*x_ref[0]**o+(-0.51057282448)*x_ref[0]+(0.163775791256)*x_ref[1]**o+(0.0152553094575)*x_ref[1]
        ref[1,0,0,0]=(0.0343855486006)*x_ref[0]**o+(-0.214334464898)*x_ref[0]+(-0.573344070879)*x_ref[1]**o+(0.24905956136)*x_ref[1]
        ref[1,0,0,1]=(0.783338978316)*x_ref[0]**o+(0.781324842059)*x_ref[0]+(-0.847509450777)*x_ref[1]**o+(-0.858994488444)*x_ref[1]
        ref[1,0,1,0]=(0.59719044885)*x_ref[0]**o+(0.347038111035)*x_ref[0]+(-0.385761256521)*x_ref[1]**o+(-0.571166989318)*x_ref[1]
        ref[1,0,1,1]=(0.636911008507)*x_ref[0]**o+(-0.825376916398)*x_ref[0]+(0.32394534216)*x_ref[1]**o+(0.653119502004)*x_ref[1]
        ref[1,0,2,0]=(-0.669732532783)*x_ref[0]**o+(-0.145427031253)*x_ref[0]+(-0.303693857831)*x_ref[1]**o+(-0.727878524833)*x_ref[1]
        ref[1,0,2,1]=(0.621061585764)*x_ref[0]**o+(0.243432217146)*x_ref[0]+(-0.888610099037)*x_ref[1]**o+(0.298051589144)*x_ref[1]
        ref[1,1,0,0]=(0.357341721813)*x_ref[0]**o+(0.346506498237)*x_ref[0]+(-0.555625437961)*x_ref[1]**o+(-0.0662579456147)*x_ref[1]
        ref[1,1,0,1]=(0.96955420495)*x_ref[0]**o+(-0.190486134786)*x_ref[0]+(-0.900124677941)*x_ref[1]**o+(0.659993963277)*x_ref[1]
        ref[1,1,1,0]=(-0.517837108406)*x_ref[0]**o+(0.00779665389526)*x_ref[0]+(-0.0544022169324)*x_ref[1]**o+(0.888217855388)*x_ref[1]
        ref[1,1,1,1]=(0.252515646862)*x_ref[0]**o+(0.328948655235)*x_ref[0]+(0.265916563145)*x_ref[1]**o+(-0.491865215095)*x_ref[1]
        ref[1,1,2,0]=(0.922520219928)*x_ref[0]**o+(0.463811222486)*x_ref[0]+(0.651244201962)*x_ref[1]**o+(0.982565677231)*x_ref[1]
        ref[1,1,2,1]=(0.257711278265)*x_ref[0]**o+(0.23971381197)*x_ref[0]+(-0.168559977915)*x_ref[1]**o+(0.453909616568)*x_ref[1]
        ref[1,2,0,0]=(0.0426933948917)*x_ref[0]**o+(-0.477381944712)*x_ref[0]+(0.503797983762)*x_ref[1]**o+(-0.943092701839)*x_ref[1]
        ref[1,2,0,1]=(0.913591544613)*x_ref[0]**o+(-0.895063818412)*x_ref[0]+(0.252264576564)*x_ref[1]**o+(0.424846745491)*x_ref[1]
        ref[1,2,1,0]=(0.4199934247)*x_ref[0]**o+(-0.128668379749)*x_ref[0]+(-0.157997092493)*x_ref[1]**o+(-0.174476338072)*x_ref[1]
        ref[1,2,1,1]=(0.394310091468)*x_ref[0]**o+(0.599539798636)*x_ref[0]+(0.133087615453)*x_ref[1]**o+(-0.22271509754)*x_ref[1]
        ref[1,2,2,0]=(0.559521330007)*x_ref[0]**o+(-0.411300693025)*x_ref[0]+(-0.543921885179)*x_ref[1]**o+(-0.0506834937614)*x_ref[1]
        ref[1,2,2,1]=(-0.423996886775)*x_ref[0]**o+(0.14409349326)*x_ref[0]+(-0.970632608145)*x_ref[1]**o+(0.582937355294)*x_ref[1]
        ref[1,3,0,0]=(-0.884972026022)*x_ref[0]**o+(-0.872260393893)*x_ref[0]+(0.224463013859)*x_ref[1]**o+(-0.245013144519)*x_ref[1]
        ref[1,3,0,1]=(-0.834873836296)*x_ref[0]**o+(-0.778611513205)*x_ref[0]+(0.989113281842)*x_ref[1]**o+(0.847047912646)*x_ref[1]
        ref[1,3,1,0]=(-0.0900703847239)*x_ref[0]**o+(-0.0295919916877)*x_ref[0]+(0.510380741505)*x_ref[1]**o+(-0.00354494905936)*x_ref[1]
        ref[1,3,1,1]=(0.0422103052661)*x_ref[0]**o+(0.31376324435)*x_ref[0]+(0.831890412519)*x_ref[1]**o+(0.764406560706)*x_ref[1]
        ref[1,3,2,0]=(0.496699862489)*x_ref[0]**o+(0.477514051712)*x_ref[0]+(0.156435695163)*x_ref[1]**o+(0.339443828388)*x_ref[1]
        ref[1,3,2,1]=(0.396689832399)*x_ref[0]**o+(0.243605505552)*x_ref[0]+(-0.0357728699149)*x_ref[1]**o+(0.930734881576)*x_ref[1]
        ref[1,4,0,0]=(-0.90788627878)*x_ref[0]**o+(0.82597715373)*x_ref[0]+(0.287988132693)*x_ref[1]**o+(-0.834307302134)*x_ref[1]
        ref[1,4,0,1]=(-0.942882799669)*x_ref[0]**o+(-0.149609905045)*x_ref[0]+(-0.340025012164)*x_ref[1]**o+(0.539546162654)*x_ref[1]
        ref[1,4,1,0]=(-0.244377326727)*x_ref[0]**o+(-0.586439384775)*x_ref[0]+(0.89506494604)*x_ref[1]**o+(0.233226803834)*x_ref[1]
        ref[1,4,1,1]=(0.864582224514)*x_ref[0]**o+(0.0687754700765)*x_ref[0]+(-0.641556536115)*x_ref[1]**o+(0.22036806322)*x_ref[1]
        ref[1,4,2,0]=(-0.552589117461)*x_ref[0]**o+(-0.705244093999)*x_ref[0]+(0.378837921567)*x_ref[1]**o+(-0.173968977986)*x_ref[1]
        ref[1,4,2,1]=(0.846900000181)*x_ref[0]**o+(-0.52589472084)*x_ref[0]+(0.872400573242)*x_ref[1]**o+(0.182683546911)*x_ref[1]
        ref[2,0,0,0]=(0.305006901177)*x_ref[0]**o+(-0.732858762152)*x_ref[0]+(-0.882239786812)*x_ref[1]**o+(0.481712295538)*x_ref[1]
        ref[2,0,0,1]=(0.836332373747)*x_ref[0]**o+(0.63905574965)*x_ref[0]+(0.039226709213)*x_ref[1]**o+(0.704758586936)*x_ref[1]
        ref[2,0,1,0]=(0.51805564252)*x_ref[0]**o+(0.641883965761)*x_ref[0]+(0.901663292208)*x_ref[1]**o+(0.375272715842)*x_ref[1]
        ref[2,0,1,1]=(-0.980058913052)*x_ref[0]**o+(0.385619663167)*x_ref[0]+(0.0722192483217)*x_ref[1]**o+(0.00650731529315)*x_ref[1]
        ref[2,0,2,0]=(-0.25791454536)*x_ref[0]**o+(0.460015385804)*x_ref[0]+(0.234970544559)*x_ref[1]**o+(-0.11962323176)*x_ref[1]
        ref[2,0,2,1]=(0.023390322577)*x_ref[0]**o+(-0.104449150638)*x_ref[0]+(0.149724956284)*x_ref[1]**o+(-0.309848596985)*x_ref[1]
        ref[2,1,0,0]=(0.603565629307)*x_ref[0]**o+(-0.143206239367)*x_ref[0]+(-0.0430666954645)*x_ref[1]**o+(-0.154546675756)*x_ref[1]
        ref[2,1,0,1]=(-0.980112660961)*x_ref[0]**o+(0.419145835585)*x_ref[0]+(-0.76202669209)*x_ref[1]**o+(-0.648615698697)*x_ref[1]
        ref[2,1,1,0]=(0.805964945679)*x_ref[0]**o+(0.650144650504)*x_ref[0]+(0.308110033125)*x_ref[1]**o+(0.0666476104838)*x_ref[1]
        ref[2,1,1,1]=(0.42561398203)*x_ref[0]**o+(0.259276259654)*x_ref[0]+(-0.215194398301)*x_ref[1]**o+(0.880282506885)*x_ref[1]
        ref[2,1,2,0]=(-0.413779899229)*x_ref[0]**o+(-0.796859182258)*x_ref[0]+(0.606865226329)*x_ref[1]**o+(-0.783313517188)*x_ref[1]
        ref[2,1,2,1]=(0.402767439742)*x_ref[0]**o+(-0.692468937875)*x_ref[0]+(0.27866251803)*x_ref[1]**o+(0.95092559773)*x_ref[1]
        ref[2,2,0,0]=(0.749107532378)*x_ref[0]**o+(-0.364633672479)*x_ref[0]+(0.970710337513)*x_ref[1]**o+(0.1986749977)*x_ref[1]
        ref[2,2,0,1]=(-0.722832876805)*x_ref[0]**o+(-0.238825576333)*x_ref[0]+(-0.101752751414)*x_ref[1]**o+(-0.960067435044)*x_ref[1]
        ref[2,2,1,0]=(-0.704825996403)*x_ref[0]**o+(0.74244530617)*x_ref[0]+(-0.586571188279)*x_ref[1]**o+(0.841297962355)*x_ref[1]
        ref[2,2,1,1]=(0.586127464995)*x_ref[0]**o+(0.454050196541)*x_ref[0]+(0.225294694159)*x_ref[1]**o+(0.705925824031)*x_ref[1]
        ref[2,2,2,0]=(0.535446366333)*x_ref[0]**o+(-0.787565531495)*x_ref[0]+(-0.566053725536)*x_ref[1]**o+(0.468968266813)*x_ref[1]
        ref[2,2,2,1]=(0.449236149776)*x_ref[0]**o+(0.589799099557)*x_ref[0]+(0.189540632578)*x_ref[1]**o+(-0.775036139482)*x_ref[1]
        ref[2,3,0,0]=(0.979572074948)*x_ref[0]**o+(-0.806420343846)*x_ref[0]+(0.661682487968)*x_ref[1]**o+(-0.615846811222)*x_ref[1]
        ref[2,3,0,1]=(0.021072244205)*x_ref[0]**o+(0.878344628134)*x_ref[0]+(-0.381067816502)*x_ref[1]**o+(0.229027446937)*x_ref[1]
        ref[2,3,1,0]=(0.0182291979593)*x_ref[0]**o+(0.625595584968)*x_ref[0]+(-0.824161018488)*x_ref[1]**o+(-0.316191235785)*x_ref[1]
        ref[2,3,1,1]=(-0.0168799900253)*x_ref[0]**o+(-0.137230078645)*x_ref[0]+(0.0240280311645)*x_ref[1]**o+(-0.364099905801)*x_ref[1]
        ref[2,3,2,0]=(0.374193672582)*x_ref[0]**o+(0.429814892173)*x_ref[0]+(0.0194417431204)*x_ref[1]**o+(0.830453174612)*x_ref[1]
        ref[2,3,2,1]=(-0.683758441576)*x_ref[0]**o+(-0.759635036652)*x_ref[0]+(-0.50102074202)*x_ref[1]**o+(0.123764523628)*x_ref[1]
        ref[2,4,0,0]=(-0.0817296101008)*x_ref[0]**o+(0.730474427452)*x_ref[0]+(-0.0733544873209)*x_ref[1]**o+(0.416303527556)*x_ref[1]
        ref[2,4,0,1]=(0.258598296267)*x_ref[0]**o+(0.686740404407)*x_ref[0]+(0.543414259362)*x_ref[1]**o+(0.518071987904)*x_ref[1]
        ref[2,4,1,0]=(-0.32475099592)*x_ref[0]**o+(-0.0460901003729)*x_ref[0]+(0.275611466137)*x_ref[1]**o+(0.8415303665)*x_ref[1]
        ref[2,4,1,1]=(0.5015310862)*x_ref[0]**o+(-0.71451126551)*x_ref[0]+(0.897703909295)*x_ref[1]**o+(-0.0476780709524)*x_ref[1]
        ref[2,4,2,0]=(0.612722779576)*x_ref[0]**o+(0.424485459817)*x_ref[0]+(-0.136026922168)*x_ref[1]**o+(0.16727850944)*x_ref[1]
        ref[2,4,2,1]=(-0.841159017194)*x_ref[0]**o+(0.856949657441)*x_ref[0]+(-0.975786438976)*x_ref[1]**o+(-0.407229683889)*x_ref[1]
        ref[3,0,0,0]=(0.833459943775)*x_ref[0]**o+(-0.230178834842)*x_ref[0]+(-0.660801597291)*x_ref[1]**o+(-0.738869823127)*x_ref[1]
        ref[3,0,0,1]=(0.30987970183)*x_ref[0]**o+(-0.610707485471)*x_ref[0]+(0.628798231358)*x_ref[1]**o+(-0.0491116840572)*x_ref[1]
        ref[3,0,1,0]=(0.649739974341)*x_ref[0]**o+(-0.450666468651)*x_ref[0]+(0.901638992166)*x_ref[1]**o+(-0.787549067401)*x_ref[1]
        ref[3,0,1,1]=(0.934583955853)*x_ref[0]**o+(-0.859086605977)*x_ref[0]+(0.0975143570756)*x_ref[1]**o+(0.0350097808797)*x_ref[1]
        ref[3,0,2,0]=(0.837723849965)*x_ref[0]**o+(-0.665696123522)*x_ref[0]+(-0.70629678851)*x_ref[1]**o+(-0.468945853667)*x_ref[1]
        ref[3,0,2,1]=(0.526302645633)*x_ref[0]**o+(-0.0979568077223)*x_ref[0]+(-0.767623484018)*x_ref[1]**o+(-0.21912588217)*x_ref[1]
        ref[3,1,0,0]=(0.18832505785)*x_ref[0]**o+(0.45004009293)*x_ref[0]+(-0.823702397711)*x_ref[1]**o+(-0.912642057295)*x_ref[1]
        ref[3,1,0,1]=(0.554483721388)*x_ref[0]**o+(-0.607925140409)*x_ref[0]+(-0.998308584629)*x_ref[1]**o+(-0.538184347898)*x_ref[1]
        ref[3,1,1,0]=(-0.682363035722)*x_ref[0]**o+(0.55318030401)*x_ref[0]+(0.562408350319)*x_ref[1]**o+(0.220145757421)*x_ref[1]
        ref[3,1,1,1]=(0.924816884714)*x_ref[0]**o+(-0.295767203325)*x_ref[0]+(0.230343937118)*x_ref[1]**o+(0.957747143338)*x_ref[1]
        ref[3,1,2,0]=(0.235422171194)*x_ref[0]**o+(0.264750603603)*x_ref[0]+(0.13053017764)*x_ref[1]**o+(0.628805980195)*x_ref[1]
        ref[3,1,2,1]=(-0.603111233811)*x_ref[0]**o+(-0.325257245906)*x_ref[0]+(-0.879402225173)*x_ref[1]**o+(0.977736365104)*x_ref[1]
        ref[3,2,0,0]=(0.463242222753)*x_ref[0]**o+(-0.802230749276)*x_ref[0]+(0.085387156258)*x_ref[1]**o+(0.18309223952)*x_ref[1]
        ref[3,2,0,1]=(0.778060805693)*x_ref[0]**o+(0.32136550684)*x_ref[0]+(0.775284717696)*x_ref[1]**o+(0.151449430741)*x_ref[1]
        ref[3,2,1,0]=(0.288473735099)*x_ref[0]**o+(0.675142038399)*x_ref[0]+(-0.313098988069)*x_ref[1]**o+(-0.771883975974)*x_ref[1]
        ref[3,2,1,1]=(0.715304130316)*x_ref[0]**o+(0.994375393616)*x_ref[0]+(-0.874667784061)*x_ref[1]**o+(-0.29510377129)*x_ref[1]
        ref[3,2,2,0]=(-0.211496384269)*x_ref[0]**o+(0.146746732763)*x_ref[0]+(0.234317983556)*x_ref[1]**o+(0.844710015939)*x_ref[1]
        ref[3,2,2,1]=(0.557426563891)*x_ref[0]**o+(0.181380049852)*x_ref[0]+(-0.068830884747)*x_ref[1]**o+(0.424778582562)*x_ref[1]
        ref[3,3,0,0]=(-0.784389821334)*x_ref[0]**o+(0.533073578323)*x_ref[0]+(-0.172622226211)*x_ref[1]**o+(-0.0882056556263)*x_ref[1]
        ref[3,3,0,1]=(-0.369034971239)*x_ref[0]**o+(-0.323409858793)*x_ref[0]+(0.408766889438)*x_ref[1]**o+(-0.653470515807)*x_ref[1]
        ref[3,3,1,0]=(0.239411265848)*x_ref[0]**o+(-0.672719205684)*x_ref[0]+(-0.751715345243)*x_ref[1]**o+(-0.4586462848)*x_ref[1]
        ref[3,3,1,1]=(-0.937724235502)*x_ref[0]**o+(-0.9284117207)*x_ref[0]+(-0.584507468424)*x_ref[1]**o+(0.779562243368)*x_ref[1]
        ref[3,3,2,0]=(-0.563406066207)*x_ref[0]**o+(0.115504525413)*x_ref[0]+(0.0230187346201)*x_ref[1]**o+(0.960113641875)*x_ref[1]
        ref[3,3,2,1]=(0.792686350755)*x_ref[0]**o+(0.560920910518)*x_ref[0]+(0.0766597516323)*x_ref[1]**o+(-0.728345800422)*x_ref[1]
        ref[3,4,0,0]=(-0.182383283107)*x_ref[0]**o+(0.604818623768)*x_ref[0]+(0.756868334363)*x_ref[1]**o+(0.0259669270239)*x_ref[1]
        ref[3,4,0,1]=(0.547592534695)*x_ref[0]**o+(0.666130932562)*x_ref[0]+(-0.922839241546)*x_ref[1]**o+(-0.143547846716)*x_ref[1]
        ref[3,4,1,0]=(-0.43295353947)*x_ref[0]**o+(0.342981294189)*x_ref[0]+(0.785780092257)*x_ref[1]**o+(0.480176709076)*x_ref[1]
        ref[3,4,1,1]=(-0.859691385907)*x_ref[0]**o+(0.124338939229)*x_ref[0]+(-0.883988405006)*x_ref[1]**o+(0.0683789905855)*x_ref[1]
        ref[3,4,2,0]=(-0.106494101633)*x_ref[0]**o+(0.276437588849)*x_ref[0]+(0.408900924262)*x_ref[1]**o+(-0.479798281555)*x_ref[1]
        ref[3,4,2,1]=(-0.226633020661)*x_ref[0]**o+(0.487277344856)*x_ref[0]+(-0.0241726779927)*x_ref[1]**o+(-0.778823212273)*x_ref[1]
      else:
        s[0,0,0,0]=(-0.400665270553)*x[0]**o+(-0.465656794893)*x[0]+(-0.103120094839)*x[1]**o+(0.0464026120794)*x[1]+(0.030596268742)*x[2]**o+(0.00334628754684)*x[2]
        s[0,0,0,1]=(0.937480902128)*x[0]**o+(0.0466658012652)*x[0]+(0.817801316276)*x[1]**o+(-0.142948501497)*x[1]+(-0.946943305156)*x[2]**o+(-0.633505380306)*x[2]
        s[0,0,1,0]=(-0.441314521722)*x[0]**o+(-0.42022483938)*x[0]+(-0.833228022905)*x[1]**o+(-0.225426650714)*x[1]+(0.332718715913)*x[2]**o+(0.123261640846)*x[2]
        s[0,0,1,1]=(0.335875498268)*x[0]**o+(0.983700594491)*x[0]+(-0.214325088125)*x[1]**o+(-0.893543563881)*x[1]+(0.0631361789468)*x[2]**o+(0.4467209081)*x[2]
        s[0,0,2,0]=(0.658215366848)*x[0]**o+(-0.672295829784)*x[0]+(-0.0137993447332)*x[1]**o+(0.698632148806)*x[1]+(-0.221572355182)*x[2]**o+(0.956096660679)*x[2]
        s[0,0,2,1]=(-0.578795938537)*x[0]**o+(0.750684927067)*x[0]+(0.333363761953)*x[1]**o+(-0.617804591638)*x[1]+(-0.0469494394831)*x[2]**o+(0.16187318118)*x[2]
        s[0,1,0,0]=(-0.0638097634115)*x[0]**o+(-0.38507312448)*x[0]+(-0.663736729005)*x[1]**o+(-0.989965410511)*x[1]+(0.230748674134)*x[2]**o+(-0.830580398633)*x[2]
        s[0,1,0,1]=(-0.658956286059)*x[0]**o+(0.345180478933)*x[0]+(-0.544808816473)*x[1]**o+(0.799798879144)*x[1]+(-0.862695252289)*x[2]**o+(0.757134401664)*x[2]
        s[0,1,1,0]=(0.456144819828)*x[0]**o+(0.969408065204)*x[0]+(0.488759322896)*x[1]**o+(0.191259701658)*x[1]+(-0.211530905801)*x[2]**o+(0.0976530925065)*x[2]
        s[0,1,1,1]=(-0.160993701908)*x[0]**o+(-0.768084723399)*x[0]+(0.200776153747)*x[1]**o+(0.380923444552)*x[1]+(0.152388183214)*x[2]**o+(0.0345495856869)*x[2]
        s[0,1,2,0]=(-0.249731094314)*x[0]**o+(0.420921348364)*x[0]+(-0.745813017956)*x[1]**o+(-0.128319964472)*x[1]+(-0.134590180255)*x[2]**o+(-0.598364990007)*x[2]
        s[0,1,2,1]=(0.608497167751)*x[0]**o+(0.134066300157)*x[0]+(-0.219556933024)*x[1]**o+(0.845095947325)*x[1]+(-0.0910112208065)*x[2]**o+(-0.919387374125)*x[2]
        s[0,2,0,0]=(-0.603722566149)*x[0]**o+(-0.141049572403)*x[0]+(-0.433072627843)*x[1]**o+(-0.266640930301)*x[1]+(-0.0520863163599)*x[2]**o+(0.113826044084)*x[2]
        s[0,2,0,1]=(0.735600439472)*x[0]**o+(-0.683666203432)*x[0]+(0.439527311107)*x[1]**o+(0.167685906293)*x[1]+(-0.526256697628)*x[2]**o+(-0.152028404784)*x[2]
        s[0,2,1,0]=(-0.484268475407)*x[0]**o+(-0.1302369788)*x[0]+(0.0996963541291)*x[1]**o+(0.525432686771)*x[1]+(0.915056396412)*x[2]**o+(-0.93142015197)*x[2]
        s[0,2,1,1]=(0.103016217069)*x[0]**o+(0.00517836018812)*x[0]+(-0.28331822161)*x[1]**o+(0.413800832478)*x[1]+(0.94613674069)*x[2]**o+(-0.287932854599)*x[2]
        s[0,2,2,0]=(0.512958587386)*x[0]**o+(-0.851145503923)*x[0]+(-0.535377708818)*x[1]**o+(-0.379448444669)*x[1]+(-0.238808818844)*x[2]**o+(0.567146071008)*x[2]
        s[0,2,2,1]=(-0.141490825422)*x[0]**o+(0.133948738844)*x[0]+(0.336733111602)*x[1]**o+(-0.223425201179)*x[1]+(-0.646454138519)*x[2]**o+(0.823936105317)*x[2]
        s[0,3,0,0]=(-0.938725695354)*x[0]**o+(0.988131864396)*x[0]+(-0.82501934096)*x[1]**o+(-0.257156351526)*x[1]+(-0.00349993655072)*x[2]**o+(0.580041226419)*x[2]
        s[0,3,0,1]=(0.814169577155)*x[0]**o+(-0.686109074376)*x[0]+(-0.965459022554)*x[1]**o+(0.10941355421)*x[1]+(0.609050101353)*x[2]**o+(-0.965588393623)*x[2]
        s[0,3,1,0]=(-0.820571213003)*x[0]**o+(0.746749194774)*x[0]+(0.710345174116)*x[1]**o+(-0.892921665205)*x[1]+(0.4004855541)*x[2]**o+(-0.695117880583)*x[2]
        s[0,3,1,1]=(-0.122273772977)*x[0]**o+(0.570499591543)*x[0]+(0.795058034707)*x[1]**o+(0.925412519293)*x[1]+(-0.0932686873001)*x[2]**o+(-0.686679792186)*x[2]
        s[0,3,2,0]=(-0.0042793185541)*x[0]**o+(-0.821480764508)*x[0]+(0.875015113726)*x[1]**o+(0.649383518184)*x[1]+(-0.288257569005)*x[2]**o+(-0.743941827161)*x[2]
        s[0,3,2,1]=(0.974182567966)*x[0]**o+(0.733657486071)*x[0]+(0.780035494556)*x[1]**o+(0.749708949611)*x[1]+(-0.521673379431)*x[2]**o+(0.673844867694)*x[2]
        s[0,4,0,0]=(-0.150057729107)*x[0]**o+(0.102611114471)*x[0]+(-0.0633864241155)*x[1]**o+(-0.329289187178)*x[1]+(0.315706711936)*x[2]**o+(-0.554443679394)*x[2]
        s[0,4,0,1]=(0.564883543338)*x[0]**o+(0.535008108501)*x[0]+(0.815233609718)*x[1]**o+(-0.766536697052)*x[1]+(0.997595942191)*x[2]**o+(0.991881374395)*x[2]
        s[0,4,1,0]=(-0.725748156838)*x[0]**o+(-0.817693342658)*x[0]+(0.501105656184)*x[1]**o+(-0.331771590595)*x[1]+(-0.757619653898)*x[2]**o+(-0.885657271086)*x[2]
        s[0,4,1,1]=(-0.559814749411)*x[0]**o+(0.665634435712)*x[0]+(0.787002371365)*x[1]**o+(-0.762034037705)*x[1]+(0.508112385178)*x[2]**o+(-0.985681857439)*x[2]
        s[0,4,2,0]=(0.504217223699)*x[0]**o+(-0.361750429828)*x[0]+(0.19299099109)*x[1]**o+(0.338521927796)*x[1]+(0.549199682759)*x[2]**o+(0.891662590379)*x[2]
        s[0,4,2,1]=(-0.251870804657)*x[0]**o+(0.776378068034)*x[0]+(-0.769493074808)*x[1]**o+(0.929429944731)*x[1]+(0.679845951817)*x[2]**o+(0.261179050197)*x[2]
        s[1,0,0,0]=(0.339823121204)*x[0]**o+(0.212075564821)*x[0]+(0.103901436655)*x[1]**o+(0.887698805047)*x[1]+(-0.467697404092)*x[2]**o+(0.71153584324)*x[2]
        s[1,0,0,1]=(0.610147759489)*x[0]**o+(-0.820866072636)*x[0]+(0.505294661936)*x[1]**o+(-0.206382410595)*x[1]+(0.343889298437)*x[2]**o+(0.165680393811)*x[2]
        s[1,0,1,0]=(-0.155342751495)*x[0]**o+(-0.638727698114)*x[0]+(-0.245328510129)*x[1]**o+(0.816689513534)*x[1]+(0.138847830114)*x[2]**o+(0.286577642911)*x[2]
        s[1,0,1,1]=(0.804058022285)*x[0]**o+(-0.922679564915)*x[0]+(-0.755679292664)*x[1]**o+(-0.409251039141)*x[1]+(-0.697869565514)*x[2]**o+(-0.0749517480557)*x[2]
        s[1,0,2,0]=(-0.345641262813)*x[0]**o+(0.31066537094)*x[0]+(0.165733093512)*x[1]**o+(-0.733996040753)*x[1]+(0.240017268146)*x[2]**o+(0.981970338884)*x[2]
        s[1,0,2,1]=(-0.0394605078977)*x[0]**o+(-0.671255389736)*x[0]+(-0.58457671038)*x[1]**o+(-0.308411795131)*x[1]+(0.944299000242)*x[2]**o+(0.0274884386004)*x[2]
        s[1,1,0,0]=(0.285078336337)*x[0]**o+(-0.29609479737)*x[0]+(0.851762966843)*x[1]**o+(-0.56978612625)*x[1]+(-0.416902609766)*x[2]**o+(-0.209352333275)*x[2]
        s[1,1,0,1]=(0.165305243827)*x[0]**o+(0.569657682766)*x[0]+(-0.158936758165)*x[1]**o+(-0.826897725956)*x[1]+(-0.906153311902)*x[2]**o+(-0.508208690461)*x[2]
        s[1,1,1,0]=(-0.168856536186)*x[0]**o+(0.600931452022)*x[0]+(0.515269032144)*x[1]**o+(-0.615629628046)*x[1]+(-0.248016004788)*x[2]**o+(-0.236340748995)*x[2]
        s[1,1,1,1]=(-0.265342410313)*x[0]**o+(0.546304098539)*x[0]+(0.44451795278)*x[1]**o+(0.53863749512)*x[1]+(-0.400353954694)*x[2]**o+(0.762091735577)*x[2]
        s[1,1,2,0]=(-0.828507588085)*x[0]**o+(0.310151266525)*x[0]+(0.767699919156)*x[1]**o+(-0.802526284756)*x[1]+(-0.12986443685)*x[2]**o+(0.555810425336)*x[2]
        s[1,1,2,1]=(-0.0843531370201)*x[0]**o+(0.138067935996)*x[0]+(-0.819663672158)*x[1]**o+(0.227177408663)*x[1]+(-0.481488807321)*x[2]**o+(0.277780608851)*x[2]
        s[1,2,0,0]=(0.843912054965)*x[0]**o+(-0.532942686099)*x[0]+(-0.12263392247)*x[1]**o+(-0.918649820864)*x[1]+(-0.898969788765)*x[2]**o+(0.401718907796)*x[2]
        s[1,2,0,1]=(0.500181344385)*x[0]**o+(-0.695081419586)*x[0]+(0.49127001192)*x[1]**o+(0.424707942699)*x[1]+(-0.170525943009)*x[2]**o+(-0.908857558417)*x[2]
        s[1,2,1,0]=(0.374108805296)*x[0]**o+(-0.778311131014)*x[0]+(-0.619173488016)*x[1]**o+(-0.0906082150795)*x[1]+(0.312783760308)*x[2]**o+(0.50073883273)*x[2]
        s[1,2,1,1]=(0.474622980263)*x[0]**o+(0.00968769992516)*x[0]+(-0.246566756628)*x[1]**o+(0.868983907653)*x[1]+(-0.318924531295)*x[2]**o+(-0.340314263979)*x[2]
        s[1,2,2,0]=(0.621111695462)*x[0]**o+(0.655354521723)*x[0]+(-0.992404590253)*x[1]**o+(-0.497860913536)*x[1]+(-0.963211421906)*x[2]**o+(0.493347857543)*x[2]
        s[1,2,2,1]=(-0.32035342079)*x[0]**o+(0.351692950156)*x[0]+(-0.715457409869)*x[1]**o+(0.317941180953)*x[1]+(0.831453341583)*x[2]**o+(0.330720124015)*x[2]
        s[1,3,0,0]=(-0.775605593731)*x[0]**o+(-0.985544306967)*x[0]+(0.838313790256)*x[1]**o+(-0.0515692297878)*x[1]+(0.6757488996)*x[2]**o+(-0.398401163936)*x[2]
        s[1,3,0,1]=(0.129462614525)*x[0]**o+(-0.225290874002)*x[0]+(0.1627137973)*x[1]**o+(0.96490154212)*x[1]+(-0.0348196505137)*x[2]**o+(0.00501682452837)*x[2]
        s[1,3,1,0]=(-0.774906442617)*x[0]**o+(0.554579514554)*x[0]+(0.68683898073)*x[1]**o+(0.687422147497)*x[1]+(0.812740760401)*x[2]**o+(0.650574541761)*x[2]
        s[1,3,1,1]=(0.371414654481)*x[0]**o+(-0.460895148602)*x[0]+(-0.60035497386)*x[1]**o+(0.382490052311)*x[1]+(0.348365164241)*x[2]**o+(0.700564991456)*x[2]
        s[1,3,2,0]=(0.0627821130997)*x[0]**o+(-0.108341459129)*x[0]+(-0.459931858236)*x[1]**o+(-0.84999694423)*x[1]+(-0.369143500965)*x[2]**o+(-0.204962250898)*x[2]
        s[1,3,2,1]=(0.940231653623)*x[0]**o+(0.432366518466)*x[0]+(0.965573462127)*x[1]**o+(-0.400536850676)*x[1]+(0.627651239821)*x[2]**o+(0.541970384011)*x[2]
        s[1,4,0,0]=(-0.905529267496)*x[0]**o+(-0.728637228667)*x[0]+(0.528565304374)*x[1]**o+(-0.0791953805449)*x[1]+(0.278340538308)*x[2]**o+(0.0829173042009)*x[2]
        s[1,4,0,1]=(0.196476386515)*x[0]**o+(0.960964133173)*x[0]+(-0.975854821639)*x[1]**o+(-0.231600097119)*x[1]+(0.462289007605)*x[2]**o+(-0.41548631733)*x[2]
        s[1,4,1,0]=(0.133593658437)*x[0]**o+(0.27713134658)*x[0]+(-0.626190133661)*x[1]**o+(0.22128472818)*x[1]+(-0.0310045745464)*x[2]**o+(-0.308952151364)*x[2]
        s[1,4,1,1]=(-0.540782388402)*x[0]**o+(0.782610613508)*x[0]+(-0.121818568845)*x[1]**o+(-0.481514588156)*x[1]+(0.92789834048)*x[2]**o+(-0.053669614999)*x[2]
        s[1,4,2,0]=(0.228180064246)*x[0]**o+(-0.387791183012)*x[0]+(-0.809517349857)*x[1]**o+(0.104233744095)*x[1]+(-0.010561196572)*x[2]**o+(0.812403786777)*x[2]
        s[1,4,2,1]=(-0.213527688719)*x[0]**o+(0.444377334059)*x[0]+(0.250474600963)*x[1]**o+(-0.628454953053)*x[1]+(-0.149400612708)*x[2]**o+(-0.362500181895)*x[2]
        s[2,0,0,0]=(0.424970215816)*x[0]**o+(-0.409634055175)*x[0]+(-0.209831967682)*x[1]**o+(0.777759135142)*x[1]+(0.904286801356)*x[2]**o+(-0.533118859622)*x[2]
        s[2,0,0,1]=(-0.0634924164631)*x[0]**o+(0.61665557894)*x[0]+(0.886000765027)*x[1]**o+(-0.283672331088)*x[1]+(-0.104923941729)*x[2]**o+(0.647966459575)*x[2]
        s[2,0,1,0]=(-0.601180947882)*x[0]**o+(-0.853176409106)*x[0]+(0.589049472252)*x[1]**o+(0.947188905512)*x[1]+(0.937016925257)*x[2]**o+(-0.528933622785)*x[2]
        s[2,0,1,1]=(-0.493108989214)*x[0]**o+(0.123452031172)*x[0]+(0.579015108934)*x[1]**o+(-0.224656610786)*x[1]+(-0.351966996224)*x[2]**o+(-0.58667858467)*x[2]
        s[2,0,2,0]=(0.342923581511)*x[0]**o+(-0.787209375031)*x[0]+(0.0689438410554)*x[1]**o+(0.624797184447)*x[1]+(0.508789531503)*x[2]**o+(0.17151319396)*x[2]
        s[2,0,2,1]=(-0.875568056219)*x[0]**o+(-0.113031987088)*x[0]+(-0.0609746766081)*x[1]**o+(0.196111602161)*x[1]+(-0.00186601407674)*x[2]**o+(-0.695244625545)*x[2]
        s[2,1,0,0]=(0.0760124938281)*x[0]**o+(0.878269236128)*x[0]+(0.424404663038)*x[1]**o+(0.382191251895)*x[1]+(-0.799519816473)*x[2]**o+(0.756150518781)*x[2]
        s[2,1,0,1]=(-0.620026715848)*x[0]**o+(0.120584354877)*x[0]+(0.794571749541)*x[1]**o+(0.775886892396)*x[1]+(-0.455364446354)*x[2]**o+(-0.25890753368)*x[2]
        s[2,1,1,0]=(-0.434917779388)*x[0]**o+(0.893805701213)*x[0]+(-0.359966318659)*x[1]**o+(-0.92549769048)*x[1]+(0.632194530492)*x[2]**o+(0.783055504124)*x[2]
        s[2,1,1,1]=(-0.30435434794)*x[0]**o+(-0.132800060169)*x[0]+(0.200039404417)*x[1]**o+(-0.690955124262)*x[1]+(-0.984472889901)*x[2]**o+(0.0242811209734)*x[2]
        s[2,1,2,0]=(-0.223815583384)*x[0]**o+(0.592762849244)*x[0]+(0.281436806248)*x[1]**o+(0.822974431302)*x[1]+(-0.0648638855118)*x[2]**o+(-0.798035298439)*x[2]
        s[2,1,2,1]=(-0.847339863019)*x[0]**o+(0.603483610026)*x[0]+(0.865918681346)*x[1]**o+(-0.0625995244022)*x[1]+(0.767004105382)*x[2]**o+(0.536886296262)*x[2]
        s[2,2,0,0]=(0.0907868991281)*x[0]**o+(0.0361485757336)*x[0]+(0.725787806874)*x[1]**o+(-0.955232659581)*x[1]+(0.015387691978)*x[2]**o+(-0.969939112182)*x[2]
        s[2,2,0,1]=(-0.702771278477)*x[0]**o+(-0.376611311235)*x[0]+(0.321676098666)*x[1]**o+(0.2741544323)*x[1]+(-0.490015464554)*x[2]**o+(-0.287278266636)*x[2]
        s[2,2,1,0]=(-0.239838875645)*x[0]**o+(-0.46816615613)*x[0]+(0.208010055752)*x[1]**o+(0.343192409907)*x[1]+(-0.752447682834)*x[2]**o+(-0.450307042264)*x[2]
        s[2,2,1,1]=(-0.377645472017)*x[0]**o+(0.979388261808)*x[0]+(0.899484011996)*x[1]**o+(0.278589875985)*x[1]+(-0.0566572805419)*x[2]**o+(-0.329814353957)*x[2]
        s[2,2,2,0]=(0.329972082761)*x[0]**o+(-0.40893298634)*x[0]+(0.14025500949)*x[1]**o+(-0.915605553253)*x[1]+(-0.743877103988)*x[2]**o+(0.0625993015596)*x[2]
        s[2,2,2,1]=(-0.426922382741)*x[0]**o+(0.638893012106)*x[0]+(0.775217826454)*x[1]**o+(0.64924098439)*x[1]+(-0.157863119278)*x[2]**o+(-0.785891322569)*x[2]
        s[2,3,0,0]=(-0.573635249481)*x[0]**o+(0.466378353545)*x[0]+(-0.968280361551)*x[1]**o+(-0.521040950167)*x[1]+(-0.979861738542)*x[2]**o+(0.293316009588)*x[2]
        s[2,3,0,1]=(-0.987445004264)*x[0]**o+(-0.855558575123)*x[0]+(0.703881586225)*x[1]**o+(0.393607926606)*x[1]+(0.615218365579)*x[2]**o+(0.865070564586)*x[2]
        s[2,3,1,0]=(0.243032241627)*x[0]**o+(0.447193793692)*x[0]+(0.121835082448)*x[1]**o+(0.253731889763)*x[1]+(0.482861064484)*x[2]**o+(0.0857661253915)*x[2]
        s[2,3,1,1]=(0.630205092814)*x[0]**o+(-0.230885016198)*x[0]+(-0.00423203911773)*x[1]**o+(0.560066624606)*x[1]+(-0.850065237172)*x[2]**o+(-0.0608202441365)*x[2]
        s[2,3,2,0]=(0.956674282321)*x[0]**o+(0.478516276613)*x[0]+(-0.142801297861)*x[1]**o+(-0.470299296114)*x[1]+(0.947735333619)*x[2]**o+(0.343691324696)*x[2]
        s[2,3,2,1]=(0.366910635243)*x[0]**o+(-0.0831991185812)*x[0]+(-0.828901851379)*x[1]**o+(-0.537917965468)*x[1]+(-0.581342127867)*x[2]**o+(-0.307660153071)*x[2]
        s[2,4,0,0]=(0.340464857467)*x[0]**o+(0.881022180176)*x[0]+(-0.223178224315)*x[1]**o+(0.0108952156846)*x[1]+(-0.130766988014)*x[2]**o+(-0.655755053638)*x[2]
        s[2,4,0,1]=(-0.142352024958)*x[0]**o+(-0.279134821704)*x[0]+(0.696577046583)*x[1]**o+(-0.913765778235)*x[1]+(0.674571789555)*x[2]**o+(-0.236086993066)*x[2]
        s[2,4,1,0]=(0.993240530889)*x[0]**o+(0.152584699472)*x[0]+(0.364843511953)*x[1]**o+(0.900205378533)*x[1]+(0.397560553857)*x[2]**o+(0.334533062647)*x[2]
        s[2,4,1,1]=(-0.261396791645)*x[0]**o+(0.96949114295)*x[0]+(0.604187009028)*x[1]**o+(-0.667369171782)*x[1]+(-0.313439370957)*x[2]**o+(0.448859840994)*x[2]
        s[2,4,2,0]=(-0.508273068959)*x[0]**o+(0.473127945457)*x[0]+(0.512686326084)*x[1]**o+(-0.872890815913)*x[1]+(-0.515333454444)*x[2]**o+(-0.578402077636)*x[2]
        s[2,4,2,1]=(-0.78684369593)*x[0]**o+(0.719254214286)*x[0]+(-0.857414812359)*x[1]**o+(-0.260538452293)*x[1]+(0.940982479653)*x[2]**o+(-0.94620100965)*x[2]
        s[3,0,0,0]=(0.741419615809)*x[0]**o+(0.535310119259)*x[0]+(-0.396521376661)*x[1]**o+(0.238209214692)*x[1]+(0.550440543811)*x[2]**o+(0.592067475252)*x[2]
        s[3,0,0,1]=(0.423030333878)*x[0]**o+(0.434870473967)*x[0]+(-0.599556117755)*x[1]**o+(0.299434424134)*x[1]+(0.120835992748)*x[2]**o+(-0.314011916578)*x[2]
        s[3,0,1,0]=(-0.326589197488)*x[0]**o+(-0.5670140013)*x[0]+(0.0548382156857)*x[1]**o+(0.689609176876)*x[1]+(0.257405458159)*x[2]**o+(-0.896175631279)*x[2]
        s[3,0,1,1]=(0.221120334733)*x[0]**o+(-0.273395088002)*x[0]+(0.678993292365)*x[1]**o+(-0.261592406628)*x[1]+(0.0129202430825)*x[2]**o+(0.148164299785)*x[2]
        s[3,0,2,0]=(0.894503166055)*x[0]**o+(-0.468004653303)*x[0]+(-0.103574702833)*x[1]**o+(-0.390725885766)*x[1]+(0.081712057468)*x[2]**o+(-0.507015846204)*x[2]
        s[3,0,2,1]=(0.712909263678)*x[0]**o+(-0.412420758959)*x[0]+(-0.620377453769)*x[1]**o+(-0.166241669671)*x[1]+(-0.738733737609)*x[2]**o+(-0.757426176741)*x[2]
        s[3,1,0,0]=(-0.166116194562)*x[0]**o+(0.988666848357)*x[0]+(-0.483399298528)*x[1]**o+(0.777960698485)*x[1]+(-0.374839973139)*x[2]**o+(0.914310220752)*x[2]
        s[3,1,0,1]=(-0.578374538082)*x[0]**o+(0.228605767869)*x[0]+(0.120418953818)*x[1]**o+(-0.693317296354)*x[1]+(-0.691997757818)*x[2]**o+(0.551252364781)*x[2]
        s[3,1,1,0]=(-0.384863493904)*x[0]**o+(0.300215711827)*x[0]+(0.948187183083)*x[1]**o+(-0.184464558346)*x[1]+(-0.7671733816)*x[2]**o+(0.307217058571)*x[2]
        s[3,1,1,1]=(0.578689722929)*x[0]**o+(0.885714287205)*x[0]+(0.722247905148)*x[1]**o+(0.933475647861)*x[1]+(0.211727836913)*x[2]**o+(0.711589357201)*x[2]
        s[3,1,2,0]=(-0.116046568217)*x[0]**o+(-0.735782802049)*x[0]+(-0.968041163959)*x[1]**o+(0.314957910843)*x[1]+(0.639581098814)*x[2]**o+(-0.566524915874)*x[2]
        s[3,1,2,1]=(0.0690920573575)*x[0]**o+(-0.0344562175408)*x[0]+(-0.729181462751)*x[1]**o+(-0.352992891115)*x[1]+(-0.230186276771)*x[2]**o+(-0.392393764843)*x[2]
        s[3,2,0,0]=(-0.897572361738)*x[0]**o+(0.42292848631)*x[0]+(0.0402102445405)*x[1]**o+(-0.695803229507)*x[1]+(0.657341764505)*x[2]**o+(0.534681573441)*x[2]
        s[3,2,0,1]=(-0.883898106985)*x[0]**o+(0.563240147064)*x[0]+(0.596180998828)*x[1]**o+(0.278590998454)*x[1]+(-0.686188881246)*x[2]**o+(0.827407211045)*x[2]
        s[3,2,1,0]=(-0.896992911933)*x[0]**o+(0.349871189445)*x[0]+(0.959261344864)*x[1]**o+(0.567834800455)*x[1]+(0.624336079204)*x[2]**o+(-0.853743864993)*x[2]
        s[3,2,1,1]=(0.947681143718)*x[0]**o+(0.563362524754)*x[0]+(0.398321082655)*x[1]**o+(0.217952803297)*x[1]+(0.243654787599)*x[2]**o+(-0.927517802601)*x[2]
        s[3,2,2,0]=(0.330842075788)*x[0]**o+(0.630950656819)*x[0]+(-0.469619054343)*x[1]**o+(-0.354739266808)*x[1]+(0.253515569858)*x[2]**o+(0.635440000647)*x[2]
        s[3,2,2,1]=(0.729280956878)*x[0]**o+(0.734399392463)*x[0]+(0.800616353182)*x[1]**o+(-0.764367237486)*x[1]+(0.90307570667)*x[2]**o+(-0.957618675411)*x[2]
        s[3,3,0,0]=(-0.783664290925)*x[0]**o+(-0.609524665456)*x[0]+(0.0126829605834)*x[1]**o+(0.396479443362)*x[1]+(0.262779944509)*x[2]**o+(-0.666191777094)*x[2]
        s[3,3,0,1]=(0.412930217814)*x[0]**o+(0.993234310311)*x[0]+(-0.123107776537)*x[1]**o+(-0.156132010898)*x[1]+(-0.72881870001)*x[2]**o+(0.757371069942)*x[2]
        s[3,3,1,0]=(-0.829528778744)*x[0]**o+(-0.676499470344)*x[0]+(-0.65512195457)*x[1]**o+(0.758109011229)*x[1]+(-0.5027027576)*x[2]**o+(-0.139493760854)*x[2]
        s[3,3,1,1]=(-0.489794277367)*x[0]**o+(-0.58772339266)*x[0]+(-0.920341394628)*x[1]**o+(-0.0302202079576)*x[1]+(-0.874433835334)*x[2]**o+(-0.382644983451)*x[2]
        s[3,3,2,0]=(-0.729507420376)*x[0]**o+(0.785351449428)*x[0]+(0.134356586244)*x[1]**o+(-0.786187017995)*x[1]+(-0.460746337928)*x[2]**o+(0.24750031597)*x[2]
        s[3,3,2,1]=(0.422343530499)*x[0]**o+(0.951298731532)*x[0]+(0.746519769375)*x[1]**o+(-0.787687730811)*x[1]+(-0.189033858381)*x[2]**o+(0.875049681221)*x[2]
        s[3,4,0,0]=(-0.784862616375)*x[0]**o+(0.605595736907)*x[0]+(-0.758269958875)*x[1]**o+(0.983503797662)*x[1]+(-0.897215719033)*x[2]**o+(0.434216381093)*x[2]
        s[3,4,0,1]=(0.959540582812)*x[0]**o+(0.74585782554)*x[0]+(0.00254441839877)*x[1]**o+(0.026781582564)*x[1]+(-0.0121835909437)*x[2]**o+(-0.166652448502)*x[2]
        s[3,4,1,0]=(-0.875170954413)*x[0]**o+(0.963228995313)*x[0]+(-0.198757196024)*x[1]**o+(-0.16115897441)*x[1]+(0.668387545829)*x[2]**o+(0.784441651668)*x[2]
        s[3,4,1,1]=(-0.0966644403845)*x[0]**o+(-0.0128070214998)*x[0]+(0.471059883662)*x[1]**o+(0.792978130351)*x[1]+(-0.709010757124)*x[2]**o+(-0.535329042416)*x[2]
        s[3,4,2,0]=(0.390795392416)*x[0]**o+(0.0465531497232)*x[0]+(-0.665065120082)*x[1]**o+(-0.637116189764)*x[1]+(0.671385406225)*x[2]**o+(-0.663428041906)*x[2]
        s[3,4,2,1]=(-0.962153478958)*x[0]**o+(0.255601758555)*x[0]+(-0.843511999734)*x[1]**o+(0.677818749903)*x[1]+(-0.273341883139)*x[2]**o+(0.846596068215)*x[2]
        ref[0,0,0,0]=(-0.400665270553)*x_ref[0]**o+(-0.465656794893)*x_ref[0]+(-0.103120094839)*x_ref[1]**o+(0.0464026120794)*x_ref[1]+(0.030596268742)*x_ref[2]**o+(0.00334628754684)*x_ref[2]
        ref[0,0,0,1]=(0.937480902128)*x_ref[0]**o+(0.0466658012652)*x_ref[0]+(0.817801316276)*x_ref[1]**o+(-0.142948501497)*x_ref[1]+(-0.946943305156)*x_ref[2]**o+(-0.633505380306)*x_ref[2]
        ref[0,0,1,0]=(-0.441314521722)*x_ref[0]**o+(-0.42022483938)*x_ref[0]+(-0.833228022905)*x_ref[1]**o+(-0.225426650714)*x_ref[1]+(0.332718715913)*x_ref[2]**o+(0.123261640846)*x_ref[2]
        ref[0,0,1,1]=(0.335875498268)*x_ref[0]**o+(0.983700594491)*x_ref[0]+(-0.214325088125)*x_ref[1]**o+(-0.893543563881)*x_ref[1]+(0.0631361789468)*x_ref[2]**o+(0.4467209081)*x_ref[2]
        ref[0,0,2,0]=(0.658215366848)*x_ref[0]**o+(-0.672295829784)*x_ref[0]+(-0.0137993447332)*x_ref[1]**o+(0.698632148806)*x_ref[1]+(-0.221572355182)*x_ref[2]**o+(0.956096660679)*x_ref[2]
        ref[0,0,2,1]=(-0.578795938537)*x_ref[0]**o+(0.750684927067)*x_ref[0]+(0.333363761953)*x_ref[1]**o+(-0.617804591638)*x_ref[1]+(-0.0469494394831)*x_ref[2]**o+(0.16187318118)*x_ref[2]
        ref[0,1,0,0]=(-0.0638097634115)*x_ref[0]**o+(-0.38507312448)*x_ref[0]+(-0.663736729005)*x_ref[1]**o+(-0.989965410511)*x_ref[1]+(0.230748674134)*x_ref[2]**o+(-0.830580398633)*x_ref[2]
        ref[0,1,0,1]=(-0.658956286059)*x_ref[0]**o+(0.345180478933)*x_ref[0]+(-0.544808816473)*x_ref[1]**o+(0.799798879144)*x_ref[1]+(-0.862695252289)*x_ref[2]**o+(0.757134401664)*x_ref[2]
        ref[0,1,1,0]=(0.456144819828)*x_ref[0]**o+(0.969408065204)*x_ref[0]+(0.488759322896)*x_ref[1]**o+(0.191259701658)*x_ref[1]+(-0.211530905801)*x_ref[2]**o+(0.0976530925065)*x_ref[2]
        ref[0,1,1,1]=(-0.160993701908)*x_ref[0]**o+(-0.768084723399)*x_ref[0]+(0.200776153747)*x_ref[1]**o+(0.380923444552)*x_ref[1]+(0.152388183214)*x_ref[2]**o+(0.0345495856869)*x_ref[2]
        ref[0,1,2,0]=(-0.249731094314)*x_ref[0]**o+(0.420921348364)*x_ref[0]+(-0.745813017956)*x_ref[1]**o+(-0.128319964472)*x_ref[1]+(-0.134590180255)*x_ref[2]**o+(-0.598364990007)*x_ref[2]
        ref[0,1,2,1]=(0.608497167751)*x_ref[0]**o+(0.134066300157)*x_ref[0]+(-0.219556933024)*x_ref[1]**o+(0.845095947325)*x_ref[1]+(-0.0910112208065)*x_ref[2]**o+(-0.919387374125)*x_ref[2]
        ref[0,2,0,0]=(-0.603722566149)*x_ref[0]**o+(-0.141049572403)*x_ref[0]+(-0.433072627843)*x_ref[1]**o+(-0.266640930301)*x_ref[1]+(-0.0520863163599)*x_ref[2]**o+(0.113826044084)*x_ref[2]
        ref[0,2,0,1]=(0.735600439472)*x_ref[0]**o+(-0.683666203432)*x_ref[0]+(0.439527311107)*x_ref[1]**o+(0.167685906293)*x_ref[1]+(-0.526256697628)*x_ref[2]**o+(-0.152028404784)*x_ref[2]
        ref[0,2,1,0]=(-0.484268475407)*x_ref[0]**o+(-0.1302369788)*x_ref[0]+(0.0996963541291)*x_ref[1]**o+(0.525432686771)*x_ref[1]+(0.915056396412)*x_ref[2]**o+(-0.93142015197)*x_ref[2]
        ref[0,2,1,1]=(0.103016217069)*x_ref[0]**o+(0.00517836018812)*x_ref[0]+(-0.28331822161)*x_ref[1]**o+(0.413800832478)*x_ref[1]+(0.94613674069)*x_ref[2]**o+(-0.287932854599)*x_ref[2]
        ref[0,2,2,0]=(0.512958587386)*x_ref[0]**o+(-0.851145503923)*x_ref[0]+(-0.535377708818)*x_ref[1]**o+(-0.379448444669)*x_ref[1]+(-0.238808818844)*x_ref[2]**o+(0.567146071008)*x_ref[2]
        ref[0,2,2,1]=(-0.141490825422)*x_ref[0]**o+(0.133948738844)*x_ref[0]+(0.336733111602)*x_ref[1]**o+(-0.223425201179)*x_ref[1]+(-0.646454138519)*x_ref[2]**o+(0.823936105317)*x_ref[2]
        ref[0,3,0,0]=(-0.938725695354)*x_ref[0]**o+(0.988131864396)*x_ref[0]+(-0.82501934096)*x_ref[1]**o+(-0.257156351526)*x_ref[1]+(-0.00349993655072)*x_ref[2]**o+(0.580041226419)*x_ref[2]
        ref[0,3,0,1]=(0.814169577155)*x_ref[0]**o+(-0.686109074376)*x_ref[0]+(-0.965459022554)*x_ref[1]**o+(0.10941355421)*x_ref[1]+(0.609050101353)*x_ref[2]**o+(-0.965588393623)*x_ref[2]
        ref[0,3,1,0]=(-0.820571213003)*x_ref[0]**o+(0.746749194774)*x_ref[0]+(0.710345174116)*x_ref[1]**o+(-0.892921665205)*x_ref[1]+(0.4004855541)*x_ref[2]**o+(-0.695117880583)*x_ref[2]
        ref[0,3,1,1]=(-0.122273772977)*x_ref[0]**o+(0.570499591543)*x_ref[0]+(0.795058034707)*x_ref[1]**o+(0.925412519293)*x_ref[1]+(-0.0932686873001)*x_ref[2]**o+(-0.686679792186)*x_ref[2]
        ref[0,3,2,0]=(-0.0042793185541)*x_ref[0]**o+(-0.821480764508)*x_ref[0]+(0.875015113726)*x_ref[1]**o+(0.649383518184)*x_ref[1]+(-0.288257569005)*x_ref[2]**o+(-0.743941827161)*x_ref[2]
        ref[0,3,2,1]=(0.974182567966)*x_ref[0]**o+(0.733657486071)*x_ref[0]+(0.780035494556)*x_ref[1]**o+(0.749708949611)*x_ref[1]+(-0.521673379431)*x_ref[2]**o+(0.673844867694)*x_ref[2]
        ref[0,4,0,0]=(-0.150057729107)*x_ref[0]**o+(0.102611114471)*x_ref[0]+(-0.0633864241155)*x_ref[1]**o+(-0.329289187178)*x_ref[1]+(0.315706711936)*x_ref[2]**o+(-0.554443679394)*x_ref[2]
        ref[0,4,0,1]=(0.564883543338)*x_ref[0]**o+(0.535008108501)*x_ref[0]+(0.815233609718)*x_ref[1]**o+(-0.766536697052)*x_ref[1]+(0.997595942191)*x_ref[2]**o+(0.991881374395)*x_ref[2]
        ref[0,4,1,0]=(-0.725748156838)*x_ref[0]**o+(-0.817693342658)*x_ref[0]+(0.501105656184)*x_ref[1]**o+(-0.331771590595)*x_ref[1]+(-0.757619653898)*x_ref[2]**o+(-0.885657271086)*x_ref[2]
        ref[0,4,1,1]=(-0.559814749411)*x_ref[0]**o+(0.665634435712)*x_ref[0]+(0.787002371365)*x_ref[1]**o+(-0.762034037705)*x_ref[1]+(0.508112385178)*x_ref[2]**o+(-0.985681857439)*x_ref[2]
        ref[0,4,2,0]=(0.504217223699)*x_ref[0]**o+(-0.361750429828)*x_ref[0]+(0.19299099109)*x_ref[1]**o+(0.338521927796)*x_ref[1]+(0.549199682759)*x_ref[2]**o+(0.891662590379)*x_ref[2]
        ref[0,4,2,1]=(-0.251870804657)*x_ref[0]**o+(0.776378068034)*x_ref[0]+(-0.769493074808)*x_ref[1]**o+(0.929429944731)*x_ref[1]+(0.679845951817)*x_ref[2]**o+(0.261179050197)*x_ref[2]
        ref[1,0,0,0]=(0.339823121204)*x_ref[0]**o+(0.212075564821)*x_ref[0]+(0.103901436655)*x_ref[1]**o+(0.887698805047)*x_ref[1]+(-0.467697404092)*x_ref[2]**o+(0.71153584324)*x_ref[2]
        ref[1,0,0,1]=(0.610147759489)*x_ref[0]**o+(-0.820866072636)*x_ref[0]+(0.505294661936)*x_ref[1]**o+(-0.206382410595)*x_ref[1]+(0.343889298437)*x_ref[2]**o+(0.165680393811)*x_ref[2]
        ref[1,0,1,0]=(-0.155342751495)*x_ref[0]**o+(-0.638727698114)*x_ref[0]+(-0.245328510129)*x_ref[1]**o+(0.816689513534)*x_ref[1]+(0.138847830114)*x_ref[2]**o+(0.286577642911)*x_ref[2]
        ref[1,0,1,1]=(0.804058022285)*x_ref[0]**o+(-0.922679564915)*x_ref[0]+(-0.755679292664)*x_ref[1]**o+(-0.409251039141)*x_ref[1]+(-0.697869565514)*x_ref[2]**o+(-0.0749517480557)*x_ref[2]
        ref[1,0,2,0]=(-0.345641262813)*x_ref[0]**o+(0.31066537094)*x_ref[0]+(0.165733093512)*x_ref[1]**o+(-0.733996040753)*x_ref[1]+(0.240017268146)*x_ref[2]**o+(0.981970338884)*x_ref[2]
        ref[1,0,2,1]=(-0.0394605078977)*x_ref[0]**o+(-0.671255389736)*x_ref[0]+(-0.58457671038)*x_ref[1]**o+(-0.308411795131)*x_ref[1]+(0.944299000242)*x_ref[2]**o+(0.0274884386004)*x_ref[2]
        ref[1,1,0,0]=(0.285078336337)*x_ref[0]**o+(-0.29609479737)*x_ref[0]+(0.851762966843)*x_ref[1]**o+(-0.56978612625)*x_ref[1]+(-0.416902609766)*x_ref[2]**o+(-0.209352333275)*x_ref[2]
        ref[1,1,0,1]=(0.165305243827)*x_ref[0]**o+(0.569657682766)*x_ref[0]+(-0.158936758165)*x_ref[1]**o+(-0.826897725956)*x_ref[1]+(-0.906153311902)*x_ref[2]**o+(-0.508208690461)*x_ref[2]
        ref[1,1,1,0]=(-0.168856536186)*x_ref[0]**o+(0.600931452022)*x_ref[0]+(0.515269032144)*x_ref[1]**o+(-0.615629628046)*x_ref[1]+(-0.248016004788)*x_ref[2]**o+(-0.236340748995)*x_ref[2]
        ref[1,1,1,1]=(-0.265342410313)*x_ref[0]**o+(0.546304098539)*x_ref[0]+(0.44451795278)*x_ref[1]**o+(0.53863749512)*x_ref[1]+(-0.400353954694)*x_ref[2]**o+(0.762091735577)*x_ref[2]
        ref[1,1,2,0]=(-0.828507588085)*x_ref[0]**o+(0.310151266525)*x_ref[0]+(0.767699919156)*x_ref[1]**o+(-0.802526284756)*x_ref[1]+(-0.12986443685)*x_ref[2]**o+(0.555810425336)*x_ref[2]
        ref[1,1,2,1]=(-0.0843531370201)*x_ref[0]**o+(0.138067935996)*x_ref[0]+(-0.819663672158)*x_ref[1]**o+(0.227177408663)*x_ref[1]+(-0.481488807321)*x_ref[2]**o+(0.277780608851)*x_ref[2]
        ref[1,2,0,0]=(0.843912054965)*x_ref[0]**o+(-0.532942686099)*x_ref[0]+(-0.12263392247)*x_ref[1]**o+(-0.918649820864)*x_ref[1]+(-0.898969788765)*x_ref[2]**o+(0.401718907796)*x_ref[2]
        ref[1,2,0,1]=(0.500181344385)*x_ref[0]**o+(-0.695081419586)*x_ref[0]+(0.49127001192)*x_ref[1]**o+(0.424707942699)*x_ref[1]+(-0.170525943009)*x_ref[2]**o+(-0.908857558417)*x_ref[2]
        ref[1,2,1,0]=(0.374108805296)*x_ref[0]**o+(-0.778311131014)*x_ref[0]+(-0.619173488016)*x_ref[1]**o+(-0.0906082150795)*x_ref[1]+(0.312783760308)*x_ref[2]**o+(0.50073883273)*x_ref[2]
        ref[1,2,1,1]=(0.474622980263)*x_ref[0]**o+(0.00968769992516)*x_ref[0]+(-0.246566756628)*x_ref[1]**o+(0.868983907653)*x_ref[1]+(-0.318924531295)*x_ref[2]**o+(-0.340314263979)*x_ref[2]
        ref[1,2,2,0]=(0.621111695462)*x_ref[0]**o+(0.655354521723)*x_ref[0]+(-0.992404590253)*x_ref[1]**o+(-0.497860913536)*x_ref[1]+(-0.963211421906)*x_ref[2]**o+(0.493347857543)*x_ref[2]
        ref[1,2,2,1]=(-0.32035342079)*x_ref[0]**o+(0.351692950156)*x_ref[0]+(-0.715457409869)*x_ref[1]**o+(0.317941180953)*x_ref[1]+(0.831453341583)*x_ref[2]**o+(0.330720124015)*x_ref[2]
        ref[1,3,0,0]=(-0.775605593731)*x_ref[0]**o+(-0.985544306967)*x_ref[0]+(0.838313790256)*x_ref[1]**o+(-0.0515692297878)*x_ref[1]+(0.6757488996)*x_ref[2]**o+(-0.398401163936)*x_ref[2]
        ref[1,3,0,1]=(0.129462614525)*x_ref[0]**o+(-0.225290874002)*x_ref[0]+(0.1627137973)*x_ref[1]**o+(0.96490154212)*x_ref[1]+(-0.0348196505137)*x_ref[2]**o+(0.00501682452837)*x_ref[2]
        ref[1,3,1,0]=(-0.774906442617)*x_ref[0]**o+(0.554579514554)*x_ref[0]+(0.68683898073)*x_ref[1]**o+(0.687422147497)*x_ref[1]+(0.812740760401)*x_ref[2]**o+(0.650574541761)*x_ref[2]
        ref[1,3,1,1]=(0.371414654481)*x_ref[0]**o+(-0.460895148602)*x_ref[0]+(-0.60035497386)*x_ref[1]**o+(0.382490052311)*x_ref[1]+(0.348365164241)*x_ref[2]**o+(0.700564991456)*x_ref[2]
        ref[1,3,2,0]=(0.0627821130997)*x_ref[0]**o+(-0.108341459129)*x_ref[0]+(-0.459931858236)*x_ref[1]**o+(-0.84999694423)*x_ref[1]+(-0.369143500965)*x_ref[2]**o+(-0.204962250898)*x_ref[2]
        ref[1,3,2,1]=(0.940231653623)*x_ref[0]**o+(0.432366518466)*x_ref[0]+(0.965573462127)*x_ref[1]**o+(-0.400536850676)*x_ref[1]+(0.627651239821)*x_ref[2]**o+(0.541970384011)*x_ref[2]
        ref[1,4,0,0]=(-0.905529267496)*x_ref[0]**o+(-0.728637228667)*x_ref[0]+(0.528565304374)*x_ref[1]**o+(-0.0791953805449)*x_ref[1]+(0.278340538308)*x_ref[2]**o+(0.0829173042009)*x_ref[2]
        ref[1,4,0,1]=(0.196476386515)*x_ref[0]**o+(0.960964133173)*x_ref[0]+(-0.975854821639)*x_ref[1]**o+(-0.231600097119)*x_ref[1]+(0.462289007605)*x_ref[2]**o+(-0.41548631733)*x_ref[2]
        ref[1,4,1,0]=(0.133593658437)*x_ref[0]**o+(0.27713134658)*x_ref[0]+(-0.626190133661)*x_ref[1]**o+(0.22128472818)*x_ref[1]+(-0.0310045745464)*x_ref[2]**o+(-0.308952151364)*x_ref[2]
        ref[1,4,1,1]=(-0.540782388402)*x_ref[0]**o+(0.782610613508)*x_ref[0]+(-0.121818568845)*x_ref[1]**o+(-0.481514588156)*x_ref[1]+(0.92789834048)*x_ref[2]**o+(-0.053669614999)*x_ref[2]
        ref[1,4,2,0]=(0.228180064246)*x_ref[0]**o+(-0.387791183012)*x_ref[0]+(-0.809517349857)*x_ref[1]**o+(0.104233744095)*x_ref[1]+(-0.010561196572)*x_ref[2]**o+(0.812403786777)*x_ref[2]
        ref[1,4,2,1]=(-0.213527688719)*x_ref[0]**o+(0.444377334059)*x_ref[0]+(0.250474600963)*x_ref[1]**o+(-0.628454953053)*x_ref[1]+(-0.149400612708)*x_ref[2]**o+(-0.362500181895)*x_ref[2]
        ref[2,0,0,0]=(0.424970215816)*x_ref[0]**o+(-0.409634055175)*x_ref[0]+(-0.209831967682)*x_ref[1]**o+(0.777759135142)*x_ref[1]+(0.904286801356)*x_ref[2]**o+(-0.533118859622)*x_ref[2]
        ref[2,0,0,1]=(-0.0634924164631)*x_ref[0]**o+(0.61665557894)*x_ref[0]+(0.886000765027)*x_ref[1]**o+(-0.283672331088)*x_ref[1]+(-0.104923941729)*x_ref[2]**o+(0.647966459575)*x_ref[2]
        ref[2,0,1,0]=(-0.601180947882)*x_ref[0]**o+(-0.853176409106)*x_ref[0]+(0.589049472252)*x_ref[1]**o+(0.947188905512)*x_ref[1]+(0.937016925257)*x_ref[2]**o+(-0.528933622785)*x_ref[2]
        ref[2,0,1,1]=(-0.493108989214)*x_ref[0]**o+(0.123452031172)*x_ref[0]+(0.579015108934)*x_ref[1]**o+(-0.224656610786)*x_ref[1]+(-0.351966996224)*x_ref[2]**o+(-0.58667858467)*x_ref[2]
        ref[2,0,2,0]=(0.342923581511)*x_ref[0]**o+(-0.787209375031)*x_ref[0]+(0.0689438410554)*x_ref[1]**o+(0.624797184447)*x_ref[1]+(0.508789531503)*x_ref[2]**o+(0.17151319396)*x_ref[2]
        ref[2,0,2,1]=(-0.875568056219)*x_ref[0]**o+(-0.113031987088)*x_ref[0]+(-0.0609746766081)*x_ref[1]**o+(0.196111602161)*x_ref[1]+(-0.00186601407674)*x_ref[2]**o+(-0.695244625545)*x_ref[2]
        ref[2,1,0,0]=(0.0760124938281)*x_ref[0]**o+(0.878269236128)*x_ref[0]+(0.424404663038)*x_ref[1]**o+(0.382191251895)*x_ref[1]+(-0.799519816473)*x_ref[2]**o+(0.756150518781)*x_ref[2]
        ref[2,1,0,1]=(-0.620026715848)*x_ref[0]**o+(0.120584354877)*x_ref[0]+(0.794571749541)*x_ref[1]**o+(0.775886892396)*x_ref[1]+(-0.455364446354)*x_ref[2]**o+(-0.25890753368)*x_ref[2]
        ref[2,1,1,0]=(-0.434917779388)*x_ref[0]**o+(0.893805701213)*x_ref[0]+(-0.359966318659)*x_ref[1]**o+(-0.92549769048)*x_ref[1]+(0.632194530492)*x_ref[2]**o+(0.783055504124)*x_ref[2]
        ref[2,1,1,1]=(-0.30435434794)*x_ref[0]**o+(-0.132800060169)*x_ref[0]+(0.200039404417)*x_ref[1]**o+(-0.690955124262)*x_ref[1]+(-0.984472889901)*x_ref[2]**o+(0.0242811209734)*x_ref[2]
        ref[2,1,2,0]=(-0.223815583384)*x_ref[0]**o+(0.592762849244)*x_ref[0]+(0.281436806248)*x_ref[1]**o+(0.822974431302)*x_ref[1]+(-0.0648638855118)*x_ref[2]**o+(-0.798035298439)*x_ref[2]
        ref[2,1,2,1]=(-0.847339863019)*x_ref[0]**o+(0.603483610026)*x_ref[0]+(0.865918681346)*x_ref[1]**o+(-0.0625995244022)*x_ref[1]+(0.767004105382)*x_ref[2]**o+(0.536886296262)*x_ref[2]
        ref[2,2,0,0]=(0.0907868991281)*x_ref[0]**o+(0.0361485757336)*x_ref[0]+(0.725787806874)*x_ref[1]**o+(-0.955232659581)*x_ref[1]+(0.015387691978)*x_ref[2]**o+(-0.969939112182)*x_ref[2]
        ref[2,2,0,1]=(-0.702771278477)*x_ref[0]**o+(-0.376611311235)*x_ref[0]+(0.321676098666)*x_ref[1]**o+(0.2741544323)*x_ref[1]+(-0.490015464554)*x_ref[2]**o+(-0.287278266636)*x_ref[2]
        ref[2,2,1,0]=(-0.239838875645)*x_ref[0]**o+(-0.46816615613)*x_ref[0]+(0.208010055752)*x_ref[1]**o+(0.343192409907)*x_ref[1]+(-0.752447682834)*x_ref[2]**o+(-0.450307042264)*x_ref[2]
        ref[2,2,1,1]=(-0.377645472017)*x_ref[0]**o+(0.979388261808)*x_ref[0]+(0.899484011996)*x_ref[1]**o+(0.278589875985)*x_ref[1]+(-0.0566572805419)*x_ref[2]**o+(-0.329814353957)*x_ref[2]
        ref[2,2,2,0]=(0.329972082761)*x_ref[0]**o+(-0.40893298634)*x_ref[0]+(0.14025500949)*x_ref[1]**o+(-0.915605553253)*x_ref[1]+(-0.743877103988)*x_ref[2]**o+(0.0625993015596)*x_ref[2]
        ref[2,2,2,1]=(-0.426922382741)*x_ref[0]**o+(0.638893012106)*x_ref[0]+(0.775217826454)*x_ref[1]**o+(0.64924098439)*x_ref[1]+(-0.157863119278)*x_ref[2]**o+(-0.785891322569)*x_ref[2]
        ref[2,3,0,0]=(-0.573635249481)*x_ref[0]**o+(0.466378353545)*x_ref[0]+(-0.968280361551)*x_ref[1]**o+(-0.521040950167)*x_ref[1]+(-0.979861738542)*x_ref[2]**o+(0.293316009588)*x_ref[2]
        ref[2,3,0,1]=(-0.987445004264)*x_ref[0]**o+(-0.855558575123)*x_ref[0]+(0.703881586225)*x_ref[1]**o+(0.393607926606)*x_ref[1]+(0.615218365579)*x_ref[2]**o+(0.865070564586)*x_ref[2]
        ref[2,3,1,0]=(0.243032241627)*x_ref[0]**o+(0.447193793692)*x_ref[0]+(0.121835082448)*x_ref[1]**o+(0.253731889763)*x_ref[1]+(0.482861064484)*x_ref[2]**o+(0.0857661253915)*x_ref[2]
        ref[2,3,1,1]=(0.630205092814)*x_ref[0]**o+(-0.230885016198)*x_ref[0]+(-0.00423203911773)*x_ref[1]**o+(0.560066624606)*x_ref[1]+(-0.850065237172)*x_ref[2]**o+(-0.0608202441365)*x_ref[2]
        ref[2,3,2,0]=(0.956674282321)*x_ref[0]**o+(0.478516276613)*x_ref[0]+(-0.142801297861)*x_ref[1]**o+(-0.470299296114)*x_ref[1]+(0.947735333619)*x_ref[2]**o+(0.343691324696)*x_ref[2]
        ref[2,3,2,1]=(0.366910635243)*x_ref[0]**o+(-0.0831991185812)*x_ref[0]+(-0.828901851379)*x_ref[1]**o+(-0.537917965468)*x_ref[1]+(-0.581342127867)*x_ref[2]**o+(-0.307660153071)*x_ref[2]
        ref[2,4,0,0]=(0.340464857467)*x_ref[0]**o+(0.881022180176)*x_ref[0]+(-0.223178224315)*x_ref[1]**o+(0.0108952156846)*x_ref[1]+(-0.130766988014)*x_ref[2]**o+(-0.655755053638)*x_ref[2]
        ref[2,4,0,1]=(-0.142352024958)*x_ref[0]**o+(-0.279134821704)*x_ref[0]+(0.696577046583)*x_ref[1]**o+(-0.913765778235)*x_ref[1]+(0.674571789555)*x_ref[2]**o+(-0.236086993066)*x_ref[2]
        ref[2,4,1,0]=(0.993240530889)*x_ref[0]**o+(0.152584699472)*x_ref[0]+(0.364843511953)*x_ref[1]**o+(0.900205378533)*x_ref[1]+(0.397560553857)*x_ref[2]**o+(0.334533062647)*x_ref[2]
        ref[2,4,1,1]=(-0.261396791645)*x_ref[0]**o+(0.96949114295)*x_ref[0]+(0.604187009028)*x_ref[1]**o+(-0.667369171782)*x_ref[1]+(-0.313439370957)*x_ref[2]**o+(0.448859840994)*x_ref[2]
        ref[2,4,2,0]=(-0.508273068959)*x_ref[0]**o+(0.473127945457)*x_ref[0]+(0.512686326084)*x_ref[1]**o+(-0.872890815913)*x_ref[1]+(-0.515333454444)*x_ref[2]**o+(-0.578402077636)*x_ref[2]
        ref[2,4,2,1]=(-0.78684369593)*x_ref[0]**o+(0.719254214286)*x_ref[0]+(-0.857414812359)*x_ref[1]**o+(-0.260538452293)*x_ref[1]+(0.940982479653)*x_ref[2]**o+(-0.94620100965)*x_ref[2]
        ref[3,0,0,0]=(0.741419615809)*x_ref[0]**o+(0.535310119259)*x_ref[0]+(-0.396521376661)*x_ref[1]**o+(0.238209214692)*x_ref[1]+(0.550440543811)*x_ref[2]**o+(0.592067475252)*x_ref[2]
        ref[3,0,0,1]=(0.423030333878)*x_ref[0]**o+(0.434870473967)*x_ref[0]+(-0.599556117755)*x_ref[1]**o+(0.299434424134)*x_ref[1]+(0.120835992748)*x_ref[2]**o+(-0.314011916578)*x_ref[2]
        ref[3,0,1,0]=(-0.326589197488)*x_ref[0]**o+(-0.5670140013)*x_ref[0]+(0.0548382156857)*x_ref[1]**o+(0.689609176876)*x_ref[1]+(0.257405458159)*x_ref[2]**o+(-0.896175631279)*x_ref[2]
        ref[3,0,1,1]=(0.221120334733)*x_ref[0]**o+(-0.273395088002)*x_ref[0]+(0.678993292365)*x_ref[1]**o+(-0.261592406628)*x_ref[1]+(0.0129202430825)*x_ref[2]**o+(0.148164299785)*x_ref[2]
        ref[3,0,2,0]=(0.894503166055)*x_ref[0]**o+(-0.468004653303)*x_ref[0]+(-0.103574702833)*x_ref[1]**o+(-0.390725885766)*x_ref[1]+(0.081712057468)*x_ref[2]**o+(-0.507015846204)*x_ref[2]
        ref[3,0,2,1]=(0.712909263678)*x_ref[0]**o+(-0.412420758959)*x_ref[0]+(-0.620377453769)*x_ref[1]**o+(-0.166241669671)*x_ref[1]+(-0.738733737609)*x_ref[2]**o+(-0.757426176741)*x_ref[2]
        ref[3,1,0,0]=(-0.166116194562)*x_ref[0]**o+(0.988666848357)*x_ref[0]+(-0.483399298528)*x_ref[1]**o+(0.777960698485)*x_ref[1]+(-0.374839973139)*x_ref[2]**o+(0.914310220752)*x_ref[2]
        ref[3,1,0,1]=(-0.578374538082)*x_ref[0]**o+(0.228605767869)*x_ref[0]+(0.120418953818)*x_ref[1]**o+(-0.693317296354)*x_ref[1]+(-0.691997757818)*x_ref[2]**o+(0.551252364781)*x_ref[2]
        ref[3,1,1,0]=(-0.384863493904)*x_ref[0]**o+(0.300215711827)*x_ref[0]+(0.948187183083)*x_ref[1]**o+(-0.184464558346)*x_ref[1]+(-0.7671733816)*x_ref[2]**o+(0.307217058571)*x_ref[2]
        ref[3,1,1,1]=(0.578689722929)*x_ref[0]**o+(0.885714287205)*x_ref[0]+(0.722247905148)*x_ref[1]**o+(0.933475647861)*x_ref[1]+(0.211727836913)*x_ref[2]**o+(0.711589357201)*x_ref[2]
        ref[3,1,2,0]=(-0.116046568217)*x_ref[0]**o+(-0.735782802049)*x_ref[0]+(-0.968041163959)*x_ref[1]**o+(0.314957910843)*x_ref[1]+(0.639581098814)*x_ref[2]**o+(-0.566524915874)*x_ref[2]
        ref[3,1,2,1]=(0.0690920573575)*x_ref[0]**o+(-0.0344562175408)*x_ref[0]+(-0.729181462751)*x_ref[1]**o+(-0.352992891115)*x_ref[1]+(-0.230186276771)*x_ref[2]**o+(-0.392393764843)*x_ref[2]
        ref[3,2,0,0]=(-0.897572361738)*x_ref[0]**o+(0.42292848631)*x_ref[0]+(0.0402102445405)*x_ref[1]**o+(-0.695803229507)*x_ref[1]+(0.657341764505)*x_ref[2]**o+(0.534681573441)*x_ref[2]
        ref[3,2,0,1]=(-0.883898106985)*x_ref[0]**o+(0.563240147064)*x_ref[0]+(0.596180998828)*x_ref[1]**o+(0.278590998454)*x_ref[1]+(-0.686188881246)*x_ref[2]**o+(0.827407211045)*x_ref[2]
        ref[3,2,1,0]=(-0.896992911933)*x_ref[0]**o+(0.349871189445)*x_ref[0]+(0.959261344864)*x_ref[1]**o+(0.567834800455)*x_ref[1]+(0.624336079204)*x_ref[2]**o+(-0.853743864993)*x_ref[2]
        ref[3,2,1,1]=(0.947681143718)*x_ref[0]**o+(0.563362524754)*x_ref[0]+(0.398321082655)*x_ref[1]**o+(0.217952803297)*x_ref[1]+(0.243654787599)*x_ref[2]**o+(-0.927517802601)*x_ref[2]
        ref[3,2,2,0]=(0.330842075788)*x_ref[0]**o+(0.630950656819)*x_ref[0]+(-0.469619054343)*x_ref[1]**o+(-0.354739266808)*x_ref[1]+(0.253515569858)*x_ref[2]**o+(0.635440000647)*x_ref[2]
        ref[3,2,2,1]=(0.729280956878)*x_ref[0]**o+(0.734399392463)*x_ref[0]+(0.800616353182)*x_ref[1]**o+(-0.764367237486)*x_ref[1]+(0.90307570667)*x_ref[2]**o+(-0.957618675411)*x_ref[2]
        ref[3,3,0,0]=(-0.783664290925)*x_ref[0]**o+(-0.609524665456)*x_ref[0]+(0.0126829605834)*x_ref[1]**o+(0.396479443362)*x_ref[1]+(0.262779944509)*x_ref[2]**o+(-0.666191777094)*x_ref[2]
        ref[3,3,0,1]=(0.412930217814)*x_ref[0]**o+(0.993234310311)*x_ref[0]+(-0.123107776537)*x_ref[1]**o+(-0.156132010898)*x_ref[1]+(-0.72881870001)*x_ref[2]**o+(0.757371069942)*x_ref[2]
        ref[3,3,1,0]=(-0.829528778744)*x_ref[0]**o+(-0.676499470344)*x_ref[0]+(-0.65512195457)*x_ref[1]**o+(0.758109011229)*x_ref[1]+(-0.5027027576)*x_ref[2]**o+(-0.139493760854)*x_ref[2]
        ref[3,3,1,1]=(-0.489794277367)*x_ref[0]**o+(-0.58772339266)*x_ref[0]+(-0.920341394628)*x_ref[1]**o+(-0.0302202079576)*x_ref[1]+(-0.874433835334)*x_ref[2]**o+(-0.382644983451)*x_ref[2]
        ref[3,3,2,0]=(-0.729507420376)*x_ref[0]**o+(0.785351449428)*x_ref[0]+(0.134356586244)*x_ref[1]**o+(-0.786187017995)*x_ref[1]+(-0.460746337928)*x_ref[2]**o+(0.24750031597)*x_ref[2]
        ref[3,3,2,1]=(0.422343530499)*x_ref[0]**o+(0.951298731532)*x_ref[0]+(0.746519769375)*x_ref[1]**o+(-0.787687730811)*x_ref[1]+(-0.189033858381)*x_ref[2]**o+(0.875049681221)*x_ref[2]
        ref[3,4,0,0]=(-0.784862616375)*x_ref[0]**o+(0.605595736907)*x_ref[0]+(-0.758269958875)*x_ref[1]**o+(0.983503797662)*x_ref[1]+(-0.897215719033)*x_ref[2]**o+(0.434216381093)*x_ref[2]
        ref[3,4,0,1]=(0.959540582812)*x_ref[0]**o+(0.74585782554)*x_ref[0]+(0.00254441839877)*x_ref[1]**o+(0.026781582564)*x_ref[1]+(-0.0121835909437)*x_ref[2]**o+(-0.166652448502)*x_ref[2]
        ref[3,4,1,0]=(-0.875170954413)*x_ref[0]**o+(0.963228995313)*x_ref[0]+(-0.198757196024)*x_ref[1]**o+(-0.16115897441)*x_ref[1]+(0.668387545829)*x_ref[2]**o+(0.784441651668)*x_ref[2]
        ref[3,4,1,1]=(-0.0966644403845)*x_ref[0]**o+(-0.0128070214998)*x_ref[0]+(0.471059883662)*x_ref[1]**o+(0.792978130351)*x_ref[1]+(-0.709010757124)*x_ref[2]**o+(-0.535329042416)*x_ref[2]
        ref[3,4,2,0]=(0.390795392416)*x_ref[0]**o+(0.0465531497232)*x_ref[0]+(-0.665065120082)*x_ref[1]**o+(-0.637116189764)*x_ref[1]+(0.671385406225)*x_ref[2]**o+(-0.663428041906)*x_ref[2]
        ref[3,4,2,1]=(-0.962153478958)*x_ref[0]**o+(0.255601758555)*x_ref[0]+(-0.843511999734)*x_ref[1]**o+(0.677818749903)*x_ref[1]+(-0.273341883139)*x_ref[2]**o+(0.846596068215)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_Solution_rank0(self):
      """
      tests interpolate for rank 0 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from Solution onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(),dim=dim)
      s=Data(0,(),w)
      ref=Data(0,(),w_ref)
      if dim==2:
        s=(0.887413974092)*x[0]**o+(-0.7148023493)*x[0]+(0.39224588792)*x[1]**o+(0.712669303726)*x[1]
        ref=(0.887413974092)*x_ref[0]**o+(-0.7148023493)*x_ref[0]+(0.39224588792)*x_ref[1]**o+(0.712669303726)*x_ref[1]
      else:
        s=(0.435927213379)*x[0]**o+(-0.406836076828)*x[0]+(0.240829091966)*x[1]**o+(-0.782431907383)*x[1]+(0.968499178848)*x[2]**o+(0.11248632948)*x[2]
        ref=(0.435927213379)*x_ref[0]**o+(-0.406836076828)*x_ref[0]+(0.240829091966)*x_ref[1]**o+(-0.782431907383)*x_ref[1]+(0.968499178848)*x_ref[2]**o+(0.11248632948)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_Solution_rank1(self):
      """
      tests interpolate for rank 1 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from Solution onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(2,),dim=dim)
      s=Data(0,(2,),w)
      ref=Data(0,(2,),w_ref)
      if dim==2:
        s[0]=(0.497549313651)*x[0]**o+(0.342704471642)*x[0]+(0.576164780397)*x[1]**o+(0.331889751836)*x[1]
        s[1]=(0.245818716931)*x[0]**o+(0.275654781295)*x[0]+(0.444526351413)*x[1]**o+(-0.783404039317)*x[1]
        ref[0]=(0.497549313651)*x_ref[0]**o+(0.342704471642)*x_ref[0]+(0.576164780397)*x_ref[1]**o+(0.331889751836)*x_ref[1]
        ref[1]=(0.245818716931)*x_ref[0]**o+(0.275654781295)*x_ref[0]+(0.444526351413)*x_ref[1]**o+(-0.783404039317)*x_ref[1]
      else:
        s[0]=(0.234557077894)*x[0]**o+(-0.201831350239)*x[0]+(-0.869318965642)*x[1]**o+(-0.715212786062)*x[1]+(0.45304187029)*x[2]**o+(0.744681443133)*x[2]
        s[1]=(0.00903347119997)*x[0]**o+(0.746691772493)*x[0]+(-0.538042235565)*x[1]**o+(0.932594018318)*x[1]+(0.58848513662)*x[2]**o+(-0.977139295615)*x[2]
        ref[0]=(0.234557077894)*x_ref[0]**o+(-0.201831350239)*x_ref[0]+(-0.869318965642)*x_ref[1]**o+(-0.715212786062)*x_ref[1]+(0.45304187029)*x_ref[2]**o+(0.744681443133)*x_ref[2]
        ref[1]=(0.00903347119997)*x_ref[0]**o+(0.746691772493)*x_ref[0]+(-0.538042235565)*x_ref[1]**o+(0.932594018318)*x_ref[1]+(0.58848513662)*x_ref[2]**o+(-0.977139295615)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_Solution_rank2(self):
      """
      tests interpolate for rank 2 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from Solution onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5),dim=dim)
      s=Data(0,(4, 5),w)
      ref=Data(0,(4, 5),w_ref)
      if dim==2:
        s[0,0]=(-0.662397963931)*x[0]**o+(-0.398404018087)*x[0]+(-0.725256273358)*x[1]**o+(0.423211349101)*x[1]
        s[0,1]=(-0.167952506192)*x[0]**o+(-0.718540804013)*x[0]+(-0.781553467855)*x[1]**o+(0.0228480720358)*x[1]
        s[0,2]=(-0.13358806923)*x[0]**o+(0.398616852411)*x[0]+(-0.311696313693)*x[1]**o+(-0.631789739932)*x[1]
        s[0,3]=(0.855686693303)*x[0]**o+(0.630881886632)*x[0]+(-0.814763888676)*x[1]**o+(-0.614872151046)*x[1]
        s[0,4]=(0.937592800976)*x[0]**o+(-0.348259700685)*x[0]+(0.506034550337)*x[1]**o+(0.0144403536426)*x[1]
        s[1,0]=(-0.994330568705)*x[0]**o+(0.0966914505526)*x[0]+(0.353136362033)*x[1]**o+(0.241811294194)*x[1]
        s[1,1]=(-0.88991407014)*x[0]**o+(0.0439813622179)*x[0]+(0.922533826895)*x[1]**o+(0.0792543536698)*x[1]
        s[1,2]=(-0.57254546238)*x[0]**o+(0.132011764714)*x[0]+(0.919045869867)*x[1]**o+(-0.781631515229)*x[1]
        s[1,3]=(0.125157999689)*x[0]**o+(-0.239227948738)*x[0]+(-0.904449723359)*x[1]**o+(-0.726384322833)*x[1]
        s[1,4]=(0.157119099313)*x[0]**o+(0.149352102671)*x[0]+(-0.715944585517)*x[1]**o+(0.559674967195)*x[1]
        s[2,0]=(0.655983809801)*x[0]**o+(-0.0230135667792)*x[0]+(-0.963391677483)*x[1]**o+(-0.705626017783)*x[1]
        s[2,1]=(-0.405526943481)*x[0]**o+(0.528349592338)*x[0]+(0.881082721559)*x[1]**o+(0.683763434255)*x[1]
        s[2,2]=(0.747664855298)*x[0]**o+(-0.336598258345)*x[0]+(-0.15756896687)*x[1]**o+(0.657585259369)*x[1]
        s[2,3]=(-0.206021660879)*x[0]**o+(0.496139719147)*x[0]+(-0.0609353622346)*x[1]**o+(0.532090076594)*x[1]
        s[2,4]=(-0.870347013263)*x[0]**o+(0.540121560488)*x[0]+(0.373986056701)*x[1]**o+(0.586491974916)*x[1]
        s[3,0]=(-0.981237374017)*x[0]**o+(0.22435680807)*x[0]+(0.988571281429)*x[1]**o+(-0.66346591076)*x[1]
        s[3,1]=(-0.939588996339)*x[0]**o+(-0.350214307841)*x[0]+(-0.269044584808)*x[1]**o+(0.115043285059)*x[1]
        s[3,2]=(-0.401479061703)*x[0]**o+(-0.922855198541)*x[0]+(0.148477007737)*x[1]**o+(-0.64051890925)*x[1]
        s[3,3]=(0.324132581573)*x[0]**o+(0.328081603393)*x[0]+(0.938536399063)*x[1]**o+(0.742951802582)*x[1]
        s[3,4]=(-0.476640426723)*x[0]**o+(0.485808406776)*x[0]+(-0.649177668213)*x[1]**o+(0.925209215964)*x[1]
        ref[0,0]=(-0.662397963931)*x_ref[0]**o+(-0.398404018087)*x_ref[0]+(-0.725256273358)*x_ref[1]**o+(0.423211349101)*x_ref[1]
        ref[0,1]=(-0.167952506192)*x_ref[0]**o+(-0.718540804013)*x_ref[0]+(-0.781553467855)*x_ref[1]**o+(0.0228480720358)*x_ref[1]
        ref[0,2]=(-0.13358806923)*x_ref[0]**o+(0.398616852411)*x_ref[0]+(-0.311696313693)*x_ref[1]**o+(-0.631789739932)*x_ref[1]
        ref[0,3]=(0.855686693303)*x_ref[0]**o+(0.630881886632)*x_ref[0]+(-0.814763888676)*x_ref[1]**o+(-0.614872151046)*x_ref[1]
        ref[0,4]=(0.937592800976)*x_ref[0]**o+(-0.348259700685)*x_ref[0]+(0.506034550337)*x_ref[1]**o+(0.0144403536426)*x_ref[1]
        ref[1,0]=(-0.994330568705)*x_ref[0]**o+(0.0966914505526)*x_ref[0]+(0.353136362033)*x_ref[1]**o+(0.241811294194)*x_ref[1]
        ref[1,1]=(-0.88991407014)*x_ref[0]**o+(0.0439813622179)*x_ref[0]+(0.922533826895)*x_ref[1]**o+(0.0792543536698)*x_ref[1]
        ref[1,2]=(-0.57254546238)*x_ref[0]**o+(0.132011764714)*x_ref[0]+(0.919045869867)*x_ref[1]**o+(-0.781631515229)*x_ref[1]
        ref[1,3]=(0.125157999689)*x_ref[0]**o+(-0.239227948738)*x_ref[0]+(-0.904449723359)*x_ref[1]**o+(-0.726384322833)*x_ref[1]
        ref[1,4]=(0.157119099313)*x_ref[0]**o+(0.149352102671)*x_ref[0]+(-0.715944585517)*x_ref[1]**o+(0.559674967195)*x_ref[1]
        ref[2,0]=(0.655983809801)*x_ref[0]**o+(-0.0230135667792)*x_ref[0]+(-0.963391677483)*x_ref[1]**o+(-0.705626017783)*x_ref[1]
        ref[2,1]=(-0.405526943481)*x_ref[0]**o+(0.528349592338)*x_ref[0]+(0.881082721559)*x_ref[1]**o+(0.683763434255)*x_ref[1]
        ref[2,2]=(0.747664855298)*x_ref[0]**o+(-0.336598258345)*x_ref[0]+(-0.15756896687)*x_ref[1]**o+(0.657585259369)*x_ref[1]
        ref[2,3]=(-0.206021660879)*x_ref[0]**o+(0.496139719147)*x_ref[0]+(-0.0609353622346)*x_ref[1]**o+(0.532090076594)*x_ref[1]
        ref[2,4]=(-0.870347013263)*x_ref[0]**o+(0.540121560488)*x_ref[0]+(0.373986056701)*x_ref[1]**o+(0.586491974916)*x_ref[1]
        ref[3,0]=(-0.981237374017)*x_ref[0]**o+(0.22435680807)*x_ref[0]+(0.988571281429)*x_ref[1]**o+(-0.66346591076)*x_ref[1]
        ref[3,1]=(-0.939588996339)*x_ref[0]**o+(-0.350214307841)*x_ref[0]+(-0.269044584808)*x_ref[1]**o+(0.115043285059)*x_ref[1]
        ref[3,2]=(-0.401479061703)*x_ref[0]**o+(-0.922855198541)*x_ref[0]+(0.148477007737)*x_ref[1]**o+(-0.64051890925)*x_ref[1]
        ref[3,3]=(0.324132581573)*x_ref[0]**o+(0.328081603393)*x_ref[0]+(0.938536399063)*x_ref[1]**o+(0.742951802582)*x_ref[1]
        ref[3,4]=(-0.476640426723)*x_ref[0]**o+(0.485808406776)*x_ref[0]+(-0.649177668213)*x_ref[1]**o+(0.925209215964)*x_ref[1]
      else:
        s[0,0]=(-0.564914581723)*x[0]**o+(0.258795214127)*x[0]+(-0.220020909772)*x[1]**o+(0.810531687745)*x[1]+(0.454950462482)*x[2]**o+(0.601383225786)*x[2]
        s[0,1]=(-0.11600536761)*x[0]**o+(-0.690336577243)*x[0]+(-0.835351354712)*x[1]**o+(-0.158331093203)*x[1]+(0.645069697527)*x[2]**o+(0.126534640518)*x[2]
        s[0,2]=(-0.350053630252)*x[0]**o+(-0.890479764661)*x[0]+(-0.280608934641)*x[1]**o+(-0.855574890474)*x[1]+(-0.988708278729)*x[2]**o+(0.0791783723801)*x[2]
        s[0,3]=(0.987927769717)*x[0]**o+(-0.347061742182)*x[0]+(0.501082056996)*x[1]**o+(-0.708442600496)*x[1]+(0.948114708446)*x[2]**o+(0.211831323577)*x[2]
        s[0,4]=(-0.930773949241)*x[0]**o+(-0.545284390651)*x[0]+(0.198879874511)*x[1]**o+(-0.221794025714)*x[1]+(-0.447838984339)*x[2]**o+(-0.610640502396)*x[2]
        s[1,0]=(-0.156742047015)*x[0]**o+(0.843077944384)*x[0]+(-0.28777246252)*x[1]**o+(0.638858613604)*x[1]+(-0.503228005834)*x[2]**o+(-0.960025936087)*x[2]
        s[1,1]=(-0.270691254249)*x[0]**o+(0.0299500422959)*x[0]+(-0.894145166738)*x[1]**o+(0.85820171409)*x[1]+(0.61650626344)*x[2]**o+(-0.480895585635)*x[2]
        s[1,2]=(0.896861224301)*x[0]**o+(0.00816715231085)*x[0]+(-0.203903225872)*x[1]**o+(-0.126928738689)*x[1]+(-0.805120272595)*x[2]**o+(0.98588145789)*x[2]
        s[1,3]=(-0.21275125769)*x[0]**o+(0.458722054632)*x[0]+(0.153520900666)*x[1]**o+(0.512505902794)*x[1]+(0.576294451799)*x[2]**o+(0.604008619969)*x[2]
        s[1,4]=(-0.882533391878)*x[0]**o+(0.616362261559)*x[0]+(0.155750337317)*x[1]**o+(-0.0162797277104)*x[1]+(0.226933640416)*x[2]**o+(-0.118618269284)*x[2]
        s[2,0]=(-0.149844951974)*x[0]**o+(0.601452947295)*x[0]+(-0.447624109479)*x[1]**o+(-0.854580861366)*x[1]+(0.392948004882)*x[2]**o+(0.0820844858757)*x[2]
        s[2,1]=(-0.256158991676)*x[0]**o+(0.161885127371)*x[0]+(0.702594613311)*x[1]**o+(0.598617952297)*x[1]+(-0.338391564856)*x[2]**o+(0.669697240278)*x[2]
        s[2,2]=(0.441512025449)*x[0]**o+(-0.761281221569)*x[0]+(-0.445161879502)*x[1]**o+(0.268033115176)*x[1]+(-0.968003905235)*x[2]**o+(0.597022138075)*x[2]
        s[2,3]=(-0.214740575251)*x[0]**o+(0.318615769596)*x[0]+(0.554289329072)*x[1]**o+(-0.785944656086)*x[1]+(0.599828055452)*x[2]**o+(-0.221979522444)*x[2]
        s[2,4]=(-0.195666204125)*x[0]**o+(-0.0924232910586)*x[0]+(-0.442263606773)*x[1]**o+(0.114036215441)*x[1]+(-0.92582505345)*x[2]**o+(-0.138685852496)*x[2]
        s[3,0]=(0.376151237664)*x[0]**o+(0.352792436289)*x[0]+(0.755168859218)*x[1]**o+(-0.888367652073)*x[1]+(-0.0969244305571)*x[2]**o+(-0.210556734576)*x[2]
        s[3,1]=(-0.802814538265)*x[0]**o+(-0.512660557023)*x[0]+(-0.574936227852)*x[1]**o+(0.587382818046)*x[1]+(-0.500271058925)*x[2]**o+(-0.385863400163)*x[2]
        s[3,2]=(-0.487420035559)*x[0]**o+(-0.18302859198)*x[0]+(-0.477510753173)*x[1]**o+(-0.438221120428)*x[1]+(0.551062942324)*x[2]**o+(-0.0447436082813)*x[2]
        s[3,3]=(-0.934219151253)*x[0]**o+(0.398356052198)*x[0]+(0.317803529004)*x[1]**o+(0.996396715585)*x[1]+(-0.0662920794517)*x[2]**o+(-0.514553218432)*x[2]
        s[3,4]=(-0.775751006525)*x[0]**o+(-0.846022290911)*x[0]+(0.394673920051)*x[1]**o+(0.332106638437)*x[1]+(-0.925799206735)*x[2]**o+(0.908724437176)*x[2]
        ref[0,0]=(-0.564914581723)*x_ref[0]**o+(0.258795214127)*x_ref[0]+(-0.220020909772)*x_ref[1]**o+(0.810531687745)*x_ref[1]+(0.454950462482)*x_ref[2]**o+(0.601383225786)*x_ref[2]
        ref[0,1]=(-0.11600536761)*x_ref[0]**o+(-0.690336577243)*x_ref[0]+(-0.835351354712)*x_ref[1]**o+(-0.158331093203)*x_ref[1]+(0.645069697527)*x_ref[2]**o+(0.126534640518)*x_ref[2]
        ref[0,2]=(-0.350053630252)*x_ref[0]**o+(-0.890479764661)*x_ref[0]+(-0.280608934641)*x_ref[1]**o+(-0.855574890474)*x_ref[1]+(-0.988708278729)*x_ref[2]**o+(0.0791783723801)*x_ref[2]
        ref[0,3]=(0.987927769717)*x_ref[0]**o+(-0.347061742182)*x_ref[0]+(0.501082056996)*x_ref[1]**o+(-0.708442600496)*x_ref[1]+(0.948114708446)*x_ref[2]**o+(0.211831323577)*x_ref[2]
        ref[0,4]=(-0.930773949241)*x_ref[0]**o+(-0.545284390651)*x_ref[0]+(0.198879874511)*x_ref[1]**o+(-0.221794025714)*x_ref[1]+(-0.447838984339)*x_ref[2]**o+(-0.610640502396)*x_ref[2]
        ref[1,0]=(-0.156742047015)*x_ref[0]**o+(0.843077944384)*x_ref[0]+(-0.28777246252)*x_ref[1]**o+(0.638858613604)*x_ref[1]+(-0.503228005834)*x_ref[2]**o+(-0.960025936087)*x_ref[2]
        ref[1,1]=(-0.270691254249)*x_ref[0]**o+(0.0299500422959)*x_ref[0]+(-0.894145166738)*x_ref[1]**o+(0.85820171409)*x_ref[1]+(0.61650626344)*x_ref[2]**o+(-0.480895585635)*x_ref[2]
        ref[1,2]=(0.896861224301)*x_ref[0]**o+(0.00816715231085)*x_ref[0]+(-0.203903225872)*x_ref[1]**o+(-0.126928738689)*x_ref[1]+(-0.805120272595)*x_ref[2]**o+(0.98588145789)*x_ref[2]
        ref[1,3]=(-0.21275125769)*x_ref[0]**o+(0.458722054632)*x_ref[0]+(0.153520900666)*x_ref[1]**o+(0.512505902794)*x_ref[1]+(0.576294451799)*x_ref[2]**o+(0.604008619969)*x_ref[2]
        ref[1,4]=(-0.882533391878)*x_ref[0]**o+(0.616362261559)*x_ref[0]+(0.155750337317)*x_ref[1]**o+(-0.0162797277104)*x_ref[1]+(0.226933640416)*x_ref[2]**o+(-0.118618269284)*x_ref[2]
        ref[2,0]=(-0.149844951974)*x_ref[0]**o+(0.601452947295)*x_ref[0]+(-0.447624109479)*x_ref[1]**o+(-0.854580861366)*x_ref[1]+(0.392948004882)*x_ref[2]**o+(0.0820844858757)*x_ref[2]
        ref[2,1]=(-0.256158991676)*x_ref[0]**o+(0.161885127371)*x_ref[0]+(0.702594613311)*x_ref[1]**o+(0.598617952297)*x_ref[1]+(-0.338391564856)*x_ref[2]**o+(0.669697240278)*x_ref[2]
        ref[2,2]=(0.441512025449)*x_ref[0]**o+(-0.761281221569)*x_ref[0]+(-0.445161879502)*x_ref[1]**o+(0.268033115176)*x_ref[1]+(-0.968003905235)*x_ref[2]**o+(0.597022138075)*x_ref[2]
        ref[2,3]=(-0.214740575251)*x_ref[0]**o+(0.318615769596)*x_ref[0]+(0.554289329072)*x_ref[1]**o+(-0.785944656086)*x_ref[1]+(0.599828055452)*x_ref[2]**o+(-0.221979522444)*x_ref[2]
        ref[2,4]=(-0.195666204125)*x_ref[0]**o+(-0.0924232910586)*x_ref[0]+(-0.442263606773)*x_ref[1]**o+(0.114036215441)*x_ref[1]+(-0.92582505345)*x_ref[2]**o+(-0.138685852496)*x_ref[2]
        ref[3,0]=(0.376151237664)*x_ref[0]**o+(0.352792436289)*x_ref[0]+(0.755168859218)*x_ref[1]**o+(-0.888367652073)*x_ref[1]+(-0.0969244305571)*x_ref[2]**o+(-0.210556734576)*x_ref[2]
        ref[3,1]=(-0.802814538265)*x_ref[0]**o+(-0.512660557023)*x_ref[0]+(-0.574936227852)*x_ref[1]**o+(0.587382818046)*x_ref[1]+(-0.500271058925)*x_ref[2]**o+(-0.385863400163)*x_ref[2]
        ref[3,2]=(-0.487420035559)*x_ref[0]**o+(-0.18302859198)*x_ref[0]+(-0.477510753173)*x_ref[1]**o+(-0.438221120428)*x_ref[1]+(0.551062942324)*x_ref[2]**o+(-0.0447436082813)*x_ref[2]
        ref[3,3]=(-0.934219151253)*x_ref[0]**o+(0.398356052198)*x_ref[0]+(0.317803529004)*x_ref[1]**o+(0.996396715585)*x_ref[1]+(-0.0662920794517)*x_ref[2]**o+(-0.514553218432)*x_ref[2]
        ref[3,4]=(-0.775751006525)*x_ref[0]**o+(-0.846022290911)*x_ref[0]+(0.394673920051)*x_ref[1]**o+(0.332106638437)*x_ref[1]+(-0.925799206735)*x_ref[2]**o+(0.908724437176)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_Solution_rank3(self):
      """
      tests interpolate for rank 3 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from Solution onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(6, 2, 2),dim=dim)
      s=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2),w_ref)
      if dim==2:
        s[0,0,0]=(0.166904084074)*x[0]**o+(0.528859695436)*x[0]+(0.767492755038)*x[1]**o+(0.219756088661)*x[1]
        s[0,0,1]=(0.116354249122)*x[0]**o+(-0.489054596883)*x[0]+(0.843432052987)*x[1]**o+(0.792328551842)*x[1]
        s[0,1,0]=(0.316210380258)*x[0]**o+(0.567201270085)*x[0]+(-0.998412810858)*x[1]**o+(-0.523766343794)*x[1]
        s[0,1,1]=(-0.131361133794)*x[0]**o+(-0.415781217922)*x[0]+(-0.743444932389)*x[1]**o+(0.0440356388227)*x[1]
        s[1,0,0]=(-0.13886008398)*x[0]**o+(0.94188356362)*x[0]+(-0.310067426785)*x[1]**o+(-0.0123016016596)*x[1]
        s[1,0,1]=(-0.451586142333)*x[0]**o+(-0.00942276847807)*x[0]+(0.141967698659)*x[1]**o+(0.0763178193995)*x[1]
        s[1,1,0]=(-0.959190563677)*x[0]**o+(0.954437590409)*x[0]+(0.862816899262)*x[1]**o+(-0.849746515492)*x[1]
        s[1,1,1]=(0.277945454335)*x[0]**o+(-0.269801922972)*x[0]+(0.991405174196)*x[1]**o+(-0.481494765175)*x[1]
        s[2,0,0]=(-0.449455660138)*x[0]**o+(0.903463595246)*x[0]+(-0.796159239153)*x[1]**o+(0.178892715811)*x[1]
        s[2,0,1]=(-0.361748699382)*x[0]**o+(-0.380839684794)*x[0]+(0.167962669517)*x[1]**o+(-0.657300482681)*x[1]
        s[2,1,0]=(-0.00835791778047)*x[0]**o+(-0.964628782216)*x[0]+(-0.242890776847)*x[1]**o+(0.580549752444)*x[1]
        s[2,1,1]=(0.31424829894)*x[0]**o+(-0.902559989702)*x[0]+(0.11028478987)*x[1]**o+(-0.408366814277)*x[1]
        s[3,0,0]=(0.0269640344016)*x[0]**o+(0.742724075544)*x[0]+(-0.275698627658)*x[1]**o+(-0.666351552089)*x[1]
        s[3,0,1]=(0.634533702145)*x[0]**o+(-0.962956780064)*x[0]+(-0.117282164464)*x[1]**o+(0.116533409993)*x[1]
        s[3,1,0]=(0.0889039574007)*x[0]**o+(-0.302373214302)*x[0]+(0.0840411077509)*x[1]**o+(0.938857669151)*x[1]
        s[3,1,1]=(-0.814069429474)*x[0]**o+(-0.157632753675)*x[0]+(0.0908314139661)*x[1]**o+(0.918730301934)*x[1]
        s[4,0,0]=(0.535565487565)*x[0]**o+(-0.136652909062)*x[0]+(0.805198026614)*x[1]**o+(0.643003183224)*x[1]
        s[4,0,1]=(-0.80530904582)*x[0]**o+(0.966761926917)*x[0]+(0.427549028006)*x[1]**o+(0.262827083069)*x[1]
        s[4,1,0]=(-0.71456682036)*x[0]**o+(-0.0739127178784)*x[0]+(-0.438432370229)*x[1]**o+(-0.550981212497)*x[1]
        s[4,1,1]=(-0.536067456171)*x[0]**o+(0.343971454511)*x[0]+(-0.738912708731)*x[1]**o+(-0.721452063307)*x[1]
        s[5,0,0]=(-0.474192484949)*x[0]**o+(0.202037328764)*x[0]+(0.899258010081)*x[1]**o+(0.409255983049)*x[1]
        s[5,0,1]=(-0.0106650866193)*x[0]**o+(0.16386057287)*x[0]+(-0.027569820513)*x[1]**o+(0.49082096942)*x[1]
        s[5,1,0]=(0.888476355004)*x[0]**o+(-0.605284684678)*x[0]+(-0.767736685512)*x[1]**o+(0.443614918182)*x[1]
        s[5,1,1]=(0.538075122922)*x[0]**o+(0.0902918352337)*x[0]+(-0.848299366243)*x[1]**o+(-0.222800083729)*x[1]
        ref[0,0,0]=(0.166904084074)*x_ref[0]**o+(0.528859695436)*x_ref[0]+(0.767492755038)*x_ref[1]**o+(0.219756088661)*x_ref[1]
        ref[0,0,1]=(0.116354249122)*x_ref[0]**o+(-0.489054596883)*x_ref[0]+(0.843432052987)*x_ref[1]**o+(0.792328551842)*x_ref[1]
        ref[0,1,0]=(0.316210380258)*x_ref[0]**o+(0.567201270085)*x_ref[0]+(-0.998412810858)*x_ref[1]**o+(-0.523766343794)*x_ref[1]
        ref[0,1,1]=(-0.131361133794)*x_ref[0]**o+(-0.415781217922)*x_ref[0]+(-0.743444932389)*x_ref[1]**o+(0.0440356388227)*x_ref[1]
        ref[1,0,0]=(-0.13886008398)*x_ref[0]**o+(0.94188356362)*x_ref[0]+(-0.310067426785)*x_ref[1]**o+(-0.0123016016596)*x_ref[1]
        ref[1,0,1]=(-0.451586142333)*x_ref[0]**o+(-0.00942276847807)*x_ref[0]+(0.141967698659)*x_ref[1]**o+(0.0763178193995)*x_ref[1]
        ref[1,1,0]=(-0.959190563677)*x_ref[0]**o+(0.954437590409)*x_ref[0]+(0.862816899262)*x_ref[1]**o+(-0.849746515492)*x_ref[1]
        ref[1,1,1]=(0.277945454335)*x_ref[0]**o+(-0.269801922972)*x_ref[0]+(0.991405174196)*x_ref[1]**o+(-0.481494765175)*x_ref[1]
        ref[2,0,0]=(-0.449455660138)*x_ref[0]**o+(0.903463595246)*x_ref[0]+(-0.796159239153)*x_ref[1]**o+(0.178892715811)*x_ref[1]
        ref[2,0,1]=(-0.361748699382)*x_ref[0]**o+(-0.380839684794)*x_ref[0]+(0.167962669517)*x_ref[1]**o+(-0.657300482681)*x_ref[1]
        ref[2,1,0]=(-0.00835791778047)*x_ref[0]**o+(-0.964628782216)*x_ref[0]+(-0.242890776847)*x_ref[1]**o+(0.580549752444)*x_ref[1]
        ref[2,1,1]=(0.31424829894)*x_ref[0]**o+(-0.902559989702)*x_ref[0]+(0.11028478987)*x_ref[1]**o+(-0.408366814277)*x_ref[1]
        ref[3,0,0]=(0.0269640344016)*x_ref[0]**o+(0.742724075544)*x_ref[0]+(-0.275698627658)*x_ref[1]**o+(-0.666351552089)*x_ref[1]
        ref[3,0,1]=(0.634533702145)*x_ref[0]**o+(-0.962956780064)*x_ref[0]+(-0.117282164464)*x_ref[1]**o+(0.116533409993)*x_ref[1]
        ref[3,1,0]=(0.0889039574007)*x_ref[0]**o+(-0.302373214302)*x_ref[0]+(0.0840411077509)*x_ref[1]**o+(0.938857669151)*x_ref[1]
        ref[3,1,1]=(-0.814069429474)*x_ref[0]**o+(-0.157632753675)*x_ref[0]+(0.0908314139661)*x_ref[1]**o+(0.918730301934)*x_ref[1]
        ref[4,0,0]=(0.535565487565)*x_ref[0]**o+(-0.136652909062)*x_ref[0]+(0.805198026614)*x_ref[1]**o+(0.643003183224)*x_ref[1]
        ref[4,0,1]=(-0.80530904582)*x_ref[0]**o+(0.966761926917)*x_ref[0]+(0.427549028006)*x_ref[1]**o+(0.262827083069)*x_ref[1]
        ref[4,1,0]=(-0.71456682036)*x_ref[0]**o+(-0.0739127178784)*x_ref[0]+(-0.438432370229)*x_ref[1]**o+(-0.550981212497)*x_ref[1]
        ref[4,1,1]=(-0.536067456171)*x_ref[0]**o+(0.343971454511)*x_ref[0]+(-0.738912708731)*x_ref[1]**o+(-0.721452063307)*x_ref[1]
        ref[5,0,0]=(-0.474192484949)*x_ref[0]**o+(0.202037328764)*x_ref[0]+(0.899258010081)*x_ref[1]**o+(0.409255983049)*x_ref[1]
        ref[5,0,1]=(-0.0106650866193)*x_ref[0]**o+(0.16386057287)*x_ref[0]+(-0.027569820513)*x_ref[1]**o+(0.49082096942)*x_ref[1]
        ref[5,1,0]=(0.888476355004)*x_ref[0]**o+(-0.605284684678)*x_ref[0]+(-0.767736685512)*x_ref[1]**o+(0.443614918182)*x_ref[1]
        ref[5,1,1]=(0.538075122922)*x_ref[0]**o+(0.0902918352337)*x_ref[0]+(-0.848299366243)*x_ref[1]**o+(-0.222800083729)*x_ref[1]
      else:
        s[0,0,0]=(0.100353793751)*x[0]**o+(-0.00847651366844)*x[0]+(-0.916659095312)*x[1]**o+(0.43123854291)*x[1]+(-0.325548712866)*x[2]**o+(0.436984953031)*x[2]
        s[0,0,1]=(-0.450705520207)*x[0]**o+(-0.0420094946238)*x[0]+(0.0471679698967)*x[1]**o+(-0.666536961424)*x[1]+(0.629675854589)*x[2]**o+(0.789343254309)*x[2]
        s[0,1,0]=(-0.753618765099)*x[0]**o+(0.327794192265)*x[0]+(-0.230331681868)*x[1]**o+(0.414256616776)*x[1]+(0.861119329276)*x[2]**o+(-0.663194921545)*x[2]
        s[0,1,1]=(-0.897978378096)*x[0]**o+(-0.758509805972)*x[0]+(0.527577710213)*x[1]**o+(-0.668678399218)*x[1]+(0.274556839547)*x[2]**o+(0.838582981011)*x[2]
        s[1,0,0]=(-0.983732811569)*x[0]**o+(0.681831497881)*x[0]+(-0.834619144249)*x[1]**o+(-0.585649970853)*x[1]+(-0.443723853248)*x[2]**o+(-0.24230119834)*x[2]
        s[1,0,1]=(0.223700452689)*x[0]**o+(-0.976082665433)*x[0]+(0.682274036244)*x[1]**o+(0.513516465519)*x[1]+(0.604542635213)*x[2]**o+(0.461982295722)*x[2]
        s[1,1,0]=(-0.124528740755)*x[0]**o+(0.982942403581)*x[0]+(0.168657059059)*x[1]**o+(0.468525342776)*x[1]+(-0.628181131076)*x[2]**o+(-0.351466961112)*x[2]
        s[1,1,1]=(-0.147571648722)*x[0]**o+(0.823854564267)*x[0]+(0.808529030428)*x[1]**o+(0.809937338033)*x[1]+(0.86059984675)*x[2]**o+(-0.245256089602)*x[2]
        s[2,0,0]=(-0.421589109694)*x[0]**o+(0.739843964299)*x[0]+(-0.807211114691)*x[1]**o+(0.926253044589)*x[1]+(0.0247961654116)*x[2]**o+(0.372268514736)*x[2]
        s[2,0,1]=(-0.271986963767)*x[0]**o+(0.653599242745)*x[0]+(-0.950340669539)*x[1]**o+(0.486306280058)*x[1]+(-0.289166423358)*x[2]**o+(-0.05193840419)*x[2]
        s[2,1,0]=(-0.439847329001)*x[0]**o+(-0.0414982844889)*x[0]+(0.724606145337)*x[1]**o+(0.800164759919)*x[1]+(-0.948978159606)*x[2]**o+(0.330443199192)*x[2]
        s[2,1,1]=(0.145027947835)*x[0]**o+(-0.227814589191)*x[0]+(0.0969921713223)*x[1]**o+(0.70541078023)*x[1]+(0.536631305638)*x[2]**o+(0.761601818309)*x[2]
        s[3,0,0]=(-0.382247778154)*x[0]**o+(0.0243544380509)*x[0]+(0.515439251148)*x[1]**o+(0.531106239532)*x[1]+(-0.303961766943)*x[2]**o+(0.463673819842)*x[2]
        s[3,0,1]=(0.375551089648)*x[0]**o+(-0.0674026512267)*x[0]+(0.0797417786103)*x[1]**o+(0.90997116985)*x[1]+(-0.613366372871)*x[2]**o+(-0.0981062035417)*x[2]
        s[3,1,0]=(-0.467358089029)*x[0]**o+(-0.656202312608)*x[0]+(-0.00790864600562)*x[1]**o+(0.501618787933)*x[1]+(0.118835067496)*x[2]**o+(0.253600511775)*x[2]
        s[3,1,1]=(-0.0613387848811)*x[0]**o+(0.0555375789746)*x[0]+(0.467925348711)*x[1]**o+(-0.521361813902)*x[1]+(-0.890969702071)*x[2]**o+(-0.0588424357829)*x[2]
        s[4,0,0]=(-0.585761522751)*x[0]**o+(0.362766428544)*x[0]+(0.405833998664)*x[1]**o+(0.927768276759)*x[1]+(-0.515236556184)*x[2]**o+(0.903733254638)*x[2]
        s[4,0,1]=(-0.443373751826)*x[0]**o+(0.589695272429)*x[0]+(-0.547332598122)*x[1]**o+(0.547140256879)*x[1]+(0.0276104869591)*x[2]**o+(-0.00901656145103)*x[2]
        s[4,1,0]=(0.327714447478)*x[0]**o+(0.333181860896)*x[0]+(0.00402719775545)*x[1]**o+(0.119030826449)*x[1]+(0.121265643664)*x[2]**o+(0.3209309716)*x[2]
        s[4,1,1]=(-0.428606778799)*x[0]**o+(-0.0240488914911)*x[0]+(-0.617222089407)*x[1]**o+(0.326500165958)*x[1]+(0.695161972341)*x[2]**o+(-0.674385407299)*x[2]
        s[5,0,0]=(0.96840017886)*x[0]**o+(0.977460834915)*x[0]+(-0.0847928163657)*x[1]**o+(-0.289814405309)*x[1]+(-0.556056635259)*x[2]**o+(-0.127899784078)*x[2]
        s[5,0,1]=(-0.535056372252)*x[0]**o+(-0.557336923293)*x[0]+(-0.982050329504)*x[1]**o+(-0.186847262795)*x[1]+(0.859894693504)*x[2]**o+(-0.993989085549)*x[2]
        s[5,1,0]=(0.846310277081)*x[0]**o+(-0.996136776377)*x[0]+(0.221846701921)*x[1]**o+(0.947752560707)*x[1]+(-0.674009316486)*x[2]**o+(0.850085324498)*x[2]
        s[5,1,1]=(-0.0244426977213)*x[0]**o+(-0.768717160785)*x[0]+(0.42356601994)*x[1]**o+(-0.359166983456)*x[1]+(-0.671384564312)*x[2]**o+(0.850466416962)*x[2]
        ref[0,0,0]=(0.100353793751)*x_ref[0]**o+(-0.00847651366844)*x_ref[0]+(-0.916659095312)*x_ref[1]**o+(0.43123854291)*x_ref[1]+(-0.325548712866)*x_ref[2]**o+(0.436984953031)*x_ref[2]
        ref[0,0,1]=(-0.450705520207)*x_ref[0]**o+(-0.0420094946238)*x_ref[0]+(0.0471679698967)*x_ref[1]**o+(-0.666536961424)*x_ref[1]+(0.629675854589)*x_ref[2]**o+(0.789343254309)*x_ref[2]
        ref[0,1,0]=(-0.753618765099)*x_ref[0]**o+(0.327794192265)*x_ref[0]+(-0.230331681868)*x_ref[1]**o+(0.414256616776)*x_ref[1]+(0.861119329276)*x_ref[2]**o+(-0.663194921545)*x_ref[2]
        ref[0,1,1]=(-0.897978378096)*x_ref[0]**o+(-0.758509805972)*x_ref[0]+(0.527577710213)*x_ref[1]**o+(-0.668678399218)*x_ref[1]+(0.274556839547)*x_ref[2]**o+(0.838582981011)*x_ref[2]
        ref[1,0,0]=(-0.983732811569)*x_ref[0]**o+(0.681831497881)*x_ref[0]+(-0.834619144249)*x_ref[1]**o+(-0.585649970853)*x_ref[1]+(-0.443723853248)*x_ref[2]**o+(-0.24230119834)*x_ref[2]
        ref[1,0,1]=(0.223700452689)*x_ref[0]**o+(-0.976082665433)*x_ref[0]+(0.682274036244)*x_ref[1]**o+(0.513516465519)*x_ref[1]+(0.604542635213)*x_ref[2]**o+(0.461982295722)*x_ref[2]
        ref[1,1,0]=(-0.124528740755)*x_ref[0]**o+(0.982942403581)*x_ref[0]+(0.168657059059)*x_ref[1]**o+(0.468525342776)*x_ref[1]+(-0.628181131076)*x_ref[2]**o+(-0.351466961112)*x_ref[2]
        ref[1,1,1]=(-0.147571648722)*x_ref[0]**o+(0.823854564267)*x_ref[0]+(0.808529030428)*x_ref[1]**o+(0.809937338033)*x_ref[1]+(0.86059984675)*x_ref[2]**o+(-0.245256089602)*x_ref[2]
        ref[2,0,0]=(-0.421589109694)*x_ref[0]**o+(0.739843964299)*x_ref[0]+(-0.807211114691)*x_ref[1]**o+(0.926253044589)*x_ref[1]+(0.0247961654116)*x_ref[2]**o+(0.372268514736)*x_ref[2]
        ref[2,0,1]=(-0.271986963767)*x_ref[0]**o+(0.653599242745)*x_ref[0]+(-0.950340669539)*x_ref[1]**o+(0.486306280058)*x_ref[1]+(-0.289166423358)*x_ref[2]**o+(-0.05193840419)*x_ref[2]
        ref[2,1,0]=(-0.439847329001)*x_ref[0]**o+(-0.0414982844889)*x_ref[0]+(0.724606145337)*x_ref[1]**o+(0.800164759919)*x_ref[1]+(-0.948978159606)*x_ref[2]**o+(0.330443199192)*x_ref[2]
        ref[2,1,1]=(0.145027947835)*x_ref[0]**o+(-0.227814589191)*x_ref[0]+(0.0969921713223)*x_ref[1]**o+(0.70541078023)*x_ref[1]+(0.536631305638)*x_ref[2]**o+(0.761601818309)*x_ref[2]
        ref[3,0,0]=(-0.382247778154)*x_ref[0]**o+(0.0243544380509)*x_ref[0]+(0.515439251148)*x_ref[1]**o+(0.531106239532)*x_ref[1]+(-0.303961766943)*x_ref[2]**o+(0.463673819842)*x_ref[2]
        ref[3,0,1]=(0.375551089648)*x_ref[0]**o+(-0.0674026512267)*x_ref[0]+(0.0797417786103)*x_ref[1]**o+(0.90997116985)*x_ref[1]+(-0.613366372871)*x_ref[2]**o+(-0.0981062035417)*x_ref[2]
        ref[3,1,0]=(-0.467358089029)*x_ref[0]**o+(-0.656202312608)*x_ref[0]+(-0.00790864600562)*x_ref[1]**o+(0.501618787933)*x_ref[1]+(0.118835067496)*x_ref[2]**o+(0.253600511775)*x_ref[2]
        ref[3,1,1]=(-0.0613387848811)*x_ref[0]**o+(0.0555375789746)*x_ref[0]+(0.467925348711)*x_ref[1]**o+(-0.521361813902)*x_ref[1]+(-0.890969702071)*x_ref[2]**o+(-0.0588424357829)*x_ref[2]
        ref[4,0,0]=(-0.585761522751)*x_ref[0]**o+(0.362766428544)*x_ref[0]+(0.405833998664)*x_ref[1]**o+(0.927768276759)*x_ref[1]+(-0.515236556184)*x_ref[2]**o+(0.903733254638)*x_ref[2]
        ref[4,0,1]=(-0.443373751826)*x_ref[0]**o+(0.589695272429)*x_ref[0]+(-0.547332598122)*x_ref[1]**o+(0.547140256879)*x_ref[1]+(0.0276104869591)*x_ref[2]**o+(-0.00901656145103)*x_ref[2]
        ref[4,1,0]=(0.327714447478)*x_ref[0]**o+(0.333181860896)*x_ref[0]+(0.00402719775545)*x_ref[1]**o+(0.119030826449)*x_ref[1]+(0.121265643664)*x_ref[2]**o+(0.3209309716)*x_ref[2]
        ref[4,1,1]=(-0.428606778799)*x_ref[0]**o+(-0.0240488914911)*x_ref[0]+(-0.617222089407)*x_ref[1]**o+(0.326500165958)*x_ref[1]+(0.695161972341)*x_ref[2]**o+(-0.674385407299)*x_ref[2]
        ref[5,0,0]=(0.96840017886)*x_ref[0]**o+(0.977460834915)*x_ref[0]+(-0.0847928163657)*x_ref[1]**o+(-0.289814405309)*x_ref[1]+(-0.556056635259)*x_ref[2]**o+(-0.127899784078)*x_ref[2]
        ref[5,0,1]=(-0.535056372252)*x_ref[0]**o+(-0.557336923293)*x_ref[0]+(-0.982050329504)*x_ref[1]**o+(-0.186847262795)*x_ref[1]+(0.859894693504)*x_ref[2]**o+(-0.993989085549)*x_ref[2]
        ref[5,1,0]=(0.846310277081)*x_ref[0]**o+(-0.996136776377)*x_ref[0]+(0.221846701921)*x_ref[1]**o+(0.947752560707)*x_ref[1]+(-0.674009316486)*x_ref[2]**o+(0.850085324498)*x_ref[2]
        ref[5,1,1]=(-0.0244426977213)*x_ref[0]**o+(-0.768717160785)*x_ref[0]+(0.42356601994)*x_ref[1]**o+(-0.359166983456)*x_ref[1]+(-0.671384564312)*x_ref[2]**o+(0.850466416962)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_Solution_rank4(self):
      """
      tests interpolate for rank 4 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from Solution onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5, 3, 2),dim=dim)
      s=Data(0,(4, 5, 3, 2),w)
      ref=Data(0,(4, 5, 3, 2),w_ref)
      if dim==2:
        s[0,0,0,0]=(-0.099815403496)*x[0]**o+(-0.918641975262)*x[0]+(-0.643800270537)*x[1]**o+(-0.188947131246)*x[1]
        s[0,0,0,1]=(-0.654158499426)*x[0]**o+(-0.158917070952)*x[0]+(0.482580361439)*x[1]**o+(0.654085120986)*x[1]
        s[0,0,1,0]=(-0.82639526996)*x[0]**o+(-0.852788225935)*x[0]+(-0.883559061247)*x[1]**o+(-0.762093398747)*x[1]
        s[0,0,1,1]=(-0.410064006917)*x[0]**o+(-0.358595400023)*x[0]+(-0.744803691415)*x[1]**o+(-0.418642435874)*x[1]
        s[0,0,2,0]=(0.627425019462)*x[0]**o+(-0.759635228096)*x[0]+(-0.225478401461)*x[1]**o+(-0.394678152175)*x[1]
        s[0,0,2,1]=(0.282477965976)*x[0]**o+(0.591668341745)*x[0]+(0.997081550686)*x[1]**o+(-0.16137686071)*x[1]
        s[0,1,0,0]=(0.503328443933)*x[0]**o+(-0.349809904995)*x[0]+(0.675840382965)*x[1]**o+(0.122847940652)*x[1]
        s[0,1,0,1]=(-0.693842907429)*x[0]**o+(0.242153763819)*x[0]+(0.669516208483)*x[1]**o+(-0.966639099473)*x[1]
        s[0,1,1,0]=(-0.143996595411)*x[0]**o+(-0.0789367221661)*x[0]+(0.44978165828)*x[1]**o+(0.982272605823)*x[1]
        s[0,1,1,1]=(0.217504837315)*x[0]**o+(0.62345834313)*x[0]+(0.421663416031)*x[1]**o+(0.298214724947)*x[1]
        s[0,1,2,0]=(-0.192788493597)*x[0]**o+(0.59360099119)*x[0]+(0.798640786481)*x[1]**o+(-0.0618029984487)*x[1]
        s[0,1,2,1]=(-0.448474976435)*x[0]**o+(-0.136832009015)*x[0]+(-0.597189414931)*x[1]**o+(0.502732362863)*x[1]
        s[0,2,0,0]=(0.252384046343)*x[0]**o+(-0.400123690831)*x[0]+(-0.545481308678)*x[1]**o+(-0.388790715614)*x[1]
        s[0,2,0,1]=(-0.66240802679)*x[0]**o+(0.77064500558)*x[0]+(0.5239431782)*x[1]**o+(0.467410945489)*x[1]
        s[0,2,1,0]=(-0.85322788336)*x[0]**o+(-0.690030962118)*x[0]+(0.699888315472)*x[1]**o+(-0.89805156241)*x[1]
        s[0,2,1,1]=(-0.255889046261)*x[0]**o+(0.0541259188422)*x[0]+(-0.749587408291)*x[1]**o+(0.791965754891)*x[1]
        s[0,2,2,0]=(0.496462410445)*x[0]**o+(0.772074605719)*x[0]+(0.180850727108)*x[1]**o+(-0.487496085214)*x[1]
        s[0,2,2,1]=(0.706055709638)*x[0]**o+(-0.598906910405)*x[0]+(-0.152966385845)*x[1]**o+(0.871383558924)*x[1]
        s[0,3,0,0]=(-0.951858396669)*x[0]**o+(-0.794420515046)*x[0]+(0.834659431502)*x[1]**o+(0.481994252391)*x[1]
        s[0,3,0,1]=(-0.578325718357)*x[0]**o+(-0.843177109188)*x[0]+(0.38052630466)*x[1]**o+(0.0999564588392)*x[1]
        s[0,3,1,0]=(0.251016293115)*x[0]**o+(-0.846784412909)*x[0]+(0.615130638894)*x[1]**o+(-0.545472677499)*x[1]
        s[0,3,1,1]=(0.0242041003245)*x[0]**o+(-0.558470673629)*x[0]+(0.0355389532695)*x[1]**o+(-0.82512767128)*x[1]
        s[0,3,2,0]=(0.104901847054)*x[0]**o+(0.176002645436)*x[0]+(-0.0529040105457)*x[1]**o+(-0.497309382471)*x[1]
        s[0,3,2,1]=(-0.19187265394)*x[0]**o+(-0.459102146282)*x[0]+(0.263116634482)*x[1]**o+(0.573723666261)*x[1]
        s[0,4,0,0]=(-0.605664936504)*x[0]**o+(0.236623083899)*x[0]+(-0.573965234185)*x[1]**o+(-0.191541238111)*x[1]
        s[0,4,0,1]=(0.735297405377)*x[0]**o+(-0.863752806793)*x[0]+(0.130990772115)*x[1]**o+(0.153051776075)*x[1]
        s[0,4,1,0]=(0.83806722488)*x[0]**o+(0.952102086801)*x[0]+(0.921278259866)*x[1]**o+(0.100889262792)*x[1]
        s[0,4,1,1]=(0.422551160597)*x[0]**o+(-0.68105266886)*x[0]+(-0.0230668740384)*x[1]**o+(0.608853763751)*x[1]
        s[0,4,2,0]=(-0.509106957279)*x[0]**o+(0.535714529222)*x[0]+(0.407059274864)*x[1]**o+(0.0304780664648)*x[1]
        s[0,4,2,1]=(-0.899880077882)*x[0]**o+(-0.150489820435)*x[0]+(-0.329228777137)*x[1]**o+(-0.325670285955)*x[1]
        s[1,0,0,0]=(0.723148414781)*x[0]**o+(0.276861527427)*x[0]+(-0.457164017461)*x[1]**o+(0.587623869716)*x[1]
        s[1,0,0,1]=(-0.656042247607)*x[0]**o+(0.599876456972)*x[0]+(-0.487723393567)*x[1]**o+(-0.293330841893)*x[1]
        s[1,0,1,0]=(0.498385487237)*x[0]**o+(0.823826806076)*x[0]+(-0.260001108163)*x[1]**o+(-0.520083127954)*x[1]
        s[1,0,1,1]=(0.0511021547178)*x[0]**o+(0.0752387451382)*x[0]+(-0.295018309613)*x[1]**o+(0.410655676791)*x[1]
        s[1,0,2,0]=(-0.926038025877)*x[0]**o+(-0.101726746627)*x[0]+(-0.714060837737)*x[1]**o+(0.291653139474)*x[1]
        s[1,0,2,1]=(0.416231421476)*x[0]**o+(0.968331479531)*x[0]+(0.13575381957)*x[1]**o+(0.0581635556746)*x[1]
        s[1,1,0,0]=(0.483218423871)*x[0]**o+(0.837199391237)*x[0]+(-0.332232511767)*x[1]**o+(0.861714372392)*x[1]
        s[1,1,0,1]=(-0.861985810928)*x[0]**o+(0.704494207363)*x[0]+(0.0120560011404)*x[1]**o+(-0.922350499146)*x[1]
        s[1,1,1,0]=(0.710471809896)*x[0]**o+(0.10838308437)*x[0]+(-0.936553657991)*x[1]**o+(-0.157956217106)*x[1]
        s[1,1,1,1]=(0.61604545132)*x[0]**o+(0.161202252636)*x[0]+(-0.208993778286)*x[1]**o+(0.150797102116)*x[1]
        s[1,1,2,0]=(0.237396232488)*x[0]**o+(-0.260415056277)*x[0]+(-0.760219527005)*x[1]**o+(0.203694444805)*x[1]
        s[1,1,2,1]=(-0.656645187501)*x[0]**o+(-0.84629755936)*x[0]+(-0.465896055143)*x[1]**o+(0.335524534826)*x[1]
        s[1,2,0,0]=(-0.546393675559)*x[0]**o+(0.0344401682615)*x[0]+(-0.267054613816)*x[1]**o+(-0.345211291755)*x[1]
        s[1,2,0,1]=(0.807935620456)*x[0]**o+(0.89820026562)*x[0]+(-0.471804773843)*x[1]**o+(0.0183931405417)*x[1]
        s[1,2,1,0]=(0.347457777403)*x[0]**o+(0.894578611023)*x[0]+(0.506086327041)*x[1]**o+(-0.0552731455772)*x[1]
        s[1,2,1,1]=(0.898402940142)*x[0]**o+(-0.850905749447)*x[0]+(0.421790306199)*x[1]**o+(0.560435305462)*x[1]
        s[1,2,2,0]=(0.417854170807)*x[0]**o+(0.0808123920631)*x[0]+(0.685114818766)*x[1]**o+(0.627248749954)*x[1]
        s[1,2,2,1]=(0.608689720682)*x[0]**o+(-0.215111751974)*x[0]+(-0.157391477249)*x[1]**o+(-0.157398505688)*x[1]
        s[1,3,0,0]=(0.453375349378)*x[0]**o+(-0.232490378969)*x[0]+(0.370581316376)*x[1]**o+(0.2321645519)*x[1]
        s[1,3,0,1]=(-0.921453677654)*x[0]**o+(-0.272608399386)*x[0]+(0.980108178061)*x[1]**o+(0.0304202383868)*x[1]
        s[1,3,1,0]=(-0.703951869845)*x[0]**o+(0.232888375306)*x[0]+(0.737108419956)*x[1]**o+(-0.815317463111)*x[1]
        s[1,3,1,1]=(-0.849808179245)*x[0]**o+(-0.848051887981)*x[0]+(0.452157961656)*x[1]**o+(0.942533763098)*x[1]
        s[1,3,2,0]=(0.276310583942)*x[0]**o+(-0.426217969193)*x[0]+(0.383056703597)*x[1]**o+(-0.100608872707)*x[1]
        s[1,3,2,1]=(0.792743379061)*x[0]**o+(-0.638898354)*x[0]+(0.0900755365083)*x[1]**o+(0.346700761651)*x[1]
        s[1,4,0,0]=(0.227940749448)*x[0]**o+(-0.723141488706)*x[0]+(-0.597539261807)*x[1]**o+(-0.0547634570298)*x[1]
        s[1,4,0,1]=(-0.00150193441347)*x[0]**o+(-0.875771867697)*x[0]+(0.554295315299)*x[1]**o+(0.179291021879)*x[1]
        s[1,4,1,0]=(0.665263753178)*x[0]**o+(-0.261999229902)*x[0]+(0.177708351596)*x[1]**o+(-0.71011990376)*x[1]
        s[1,4,1,1]=(0.416094331837)*x[0]**o+(0.809191029905)*x[0]+(-0.631251155279)*x[1]**o+(0.758593347696)*x[1]
        s[1,4,2,0]=(0.50941363283)*x[0]**o+(-0.652536088555)*x[0]+(-0.0994076235125)*x[1]**o+(0.186614952474)*x[1]
        s[1,4,2,1]=(0.129263234225)*x[0]**o+(0.115870119224)*x[0]+(-0.25581908442)*x[1]**o+(0.295990548791)*x[1]
        s[2,0,0,0]=(-0.432850417862)*x[0]**o+(-0.491964804361)*x[0]+(0.0106143845453)*x[1]**o+(0.787362504175)*x[1]
        s[2,0,0,1]=(0.31617556259)*x[0]**o+(0.255445781486)*x[0]+(-0.302678692069)*x[1]**o+(0.840840019713)*x[1]
        s[2,0,1,0]=(-0.855359978332)*x[0]**o+(0.410554550223)*x[0]+(-0.703253338388)*x[1]**o+(0.59010970561)*x[1]
        s[2,0,1,1]=(-0.573270259685)*x[0]**o+(0.928396842624)*x[0]+(0.94155331127)*x[1]**o+(0.160140257961)*x[1]
        s[2,0,2,0]=(0.0524505638003)*x[0]**o+(-0.736508989499)*x[0]+(-0.565060891384)*x[1]**o+(-0.613736274563)*x[1]
        s[2,0,2,1]=(-0.776619329318)*x[0]**o+(0.485104655735)*x[0]+(-0.135863414015)*x[1]**o+(-0.14173677679)*x[1]
        s[2,1,0,0]=(0.662345907479)*x[0]**o+(-0.900993470049)*x[0]+(-0.882752065723)*x[1]**o+(0.642775104186)*x[1]
        s[2,1,0,1]=(-0.189295597666)*x[0]**o+(0.0165605763413)*x[0]+(0.989119296804)*x[1]**o+(0.262678778013)*x[1]
        s[2,1,1,0]=(0.927673133577)*x[0]**o+(-0.0248539427539)*x[0]+(0.504610255651)*x[1]**o+(-0.593911741472)*x[1]
        s[2,1,1,1]=(0.899525029448)*x[0]**o+(-0.97195963201)*x[0]+(-0.461931155897)*x[1]**o+(0.91354314585)*x[1]
        s[2,1,2,0]=(0.398654077456)*x[0]**o+(-0.429950726187)*x[0]+(0.378910374088)*x[1]**o+(-0.600308536915)*x[1]
        s[2,1,2,1]=(0.684159561112)*x[0]**o+(-0.408717922991)*x[0]+(0.0986838286044)*x[1]**o+(0.446743772828)*x[1]
        s[2,2,0,0]=(-0.657789003001)*x[0]**o+(-0.696243850831)*x[0]+(0.143074391914)*x[1]**o+(-0.784498727037)*x[1]
        s[2,2,0,1]=(-0.035450676833)*x[0]**o+(-0.453531837262)*x[0]+(0.59955708994)*x[1]**o+(0.35879284257)*x[1]
        s[2,2,1,0]=(0.302411531858)*x[0]**o+(-0.120739744552)*x[0]+(0.23248426438)*x[1]**o+(-0.39024395986)*x[1]
        s[2,2,1,1]=(0.957811592451)*x[0]**o+(-0.444133483391)*x[0]+(-0.917299241507)*x[1]**o+(0.66053932766)*x[1]
        s[2,2,2,0]=(0.361654572167)*x[0]**o+(-0.860498450204)*x[0]+(0.839403953375)*x[1]**o+(-0.386466411338)*x[1]
        s[2,2,2,1]=(-0.0143243836126)*x[0]**o+(0.444553204161)*x[0]+(0.0214936163191)*x[1]**o+(-0.257450046824)*x[1]
        s[2,3,0,0]=(0.841171215625)*x[0]**o+(-0.342091657341)*x[0]+(0.429215544167)*x[1]**o+(0.291784964305)*x[1]
        s[2,3,0,1]=(0.650468319582)*x[0]**o+(-0.121782834777)*x[0]+(0.887022664242)*x[1]**o+(0.957311577092)*x[1]
        s[2,3,1,0]=(0.156495854578)*x[0]**o+(0.855814547392)*x[0]+(0.320209813846)*x[1]**o+(0.458769463001)*x[1]
        s[2,3,1,1]=(0.34344265441)*x[0]**o+(-0.992393068373)*x[0]+(-0.0554086726336)*x[1]**o+(-0.985825317842)*x[1]
        s[2,3,2,0]=(-0.866276931509)*x[0]**o+(-0.343662754778)*x[0]+(0.0302242797749)*x[1]**o+(-0.652051073003)*x[1]
        s[2,3,2,1]=(-0.68451689603)*x[0]**o+(0.564121866438)*x[0]+(-0.136564076086)*x[1]**o+(0.648701900333)*x[1]
        s[2,4,0,0]=(-0.430429138775)*x[0]**o+(0.721222678082)*x[0]+(-0.0375465605006)*x[1]**o+(-0.462643638758)*x[1]
        s[2,4,0,1]=(-0.321870157728)*x[0]**o+(0.884979965704)*x[0]+(0.210984337677)*x[1]**o+(-0.796486469869)*x[1]
        s[2,4,1,0]=(0.365646287696)*x[0]**o+(-0.600693071289)*x[0]+(-0.998407218964)*x[1]**o+(-0.119949305545)*x[1]
        s[2,4,1,1]=(-0.763923462892)*x[0]**o+(-0.831011577325)*x[0]+(0.547670679354)*x[1]**o+(-0.856456618287)*x[1]
        s[2,4,2,0]=(0.107211541473)*x[0]**o+(-0.787737471224)*x[0]+(0.00381729482611)*x[1]**o+(-0.737759009165)*x[1]
        s[2,4,2,1]=(0.233196464862)*x[0]**o+(-0.464864449316)*x[0]+(0.229081432725)*x[1]**o+(-0.316321259146)*x[1]
        s[3,0,0,0]=(0.84633152323)*x[0]**o+(0.107225324454)*x[0]+(-0.768659465295)*x[1]**o+(0.441758939429)*x[1]
        s[3,0,0,1]=(-0.317081751011)*x[0]**o+(-0.557807443325)*x[0]+(-0.513518142998)*x[1]**o+(0.969406024047)*x[1]
        s[3,0,1,0]=(0.470133545453)*x[0]**o+(0.689603661125)*x[0]+(-0.358563691038)*x[1]**o+(0.0930824872861)*x[1]
        s[3,0,1,1]=(0.31919974075)*x[0]**o+(-0.555789020025)*x[0]+(0.0108880065973)*x[1]**o+(0.519548687393)*x[1]
        s[3,0,2,0]=(0.663881497435)*x[0]**o+(-0.668884360184)*x[0]+(-0.249669522726)*x[1]**o+(0.978543111491)*x[1]
        s[3,0,2,1]=(0.119167252087)*x[0]**o+(0.80927599582)*x[0]+(0.526489556971)*x[1]**o+(-0.440104597696)*x[1]
        s[3,1,0,0]=(-0.367446660377)*x[0]**o+(-0.326052348072)*x[0]+(0.841697795111)*x[1]**o+(0.403206833675)*x[1]
        s[3,1,0,1]=(-0.647927382559)*x[0]**o+(0.096852636085)*x[0]+(-0.126798636274)*x[1]**o+(0.117295901868)*x[1]
        s[3,1,1,0]=(0.827676755777)*x[0]**o+(-0.922576416262)*x[0]+(-0.3927169369)*x[1]**o+(0.828141309451)*x[1]
        s[3,1,1,1]=(-0.0442514478276)*x[0]**o+(-0.17904563214)*x[0]+(0.0870253733863)*x[1]**o+(-0.521249847127)*x[1]
        s[3,1,2,0]=(0.295531599451)*x[0]**o+(-0.125313746461)*x[0]+(0.982602753269)*x[1]**o+(-0.469029795983)*x[1]
        s[3,1,2,1]=(0.950273049304)*x[0]**o+(0.859212132249)*x[0]+(0.728565507333)*x[1]**o+(-0.274531603828)*x[1]
        s[3,2,0,0]=(-0.885845551336)*x[0]**o+(-0.704449229328)*x[0]+(-0.571836258871)*x[1]**o+(-0.0476396264682)*x[1]
        s[3,2,0,1]=(0.0713925637733)*x[0]**o+(0.294650464749)*x[0]+(-0.917378997351)*x[1]**o+(-0.715770468859)*x[1]
        s[3,2,1,0]=(-0.715304376467)*x[0]**o+(0.252625168227)*x[0]+(0.942098162684)*x[1]**o+(-0.0579654780836)*x[1]
        s[3,2,1,1]=(-0.937718965835)*x[0]**o+(0.649603348472)*x[0]+(-0.293284658463)*x[1]**o+(-0.347005762941)*x[1]
        s[3,2,2,0]=(-0.042358031525)*x[0]**o+(0.036891446424)*x[0]+(-0.229597055696)*x[1]**o+(0.0283282394156)*x[1]
        s[3,2,2,1]=(-0.0186366313091)*x[0]**o+(-0.0127649188124)*x[0]+(0.803780828202)*x[1]**o+(0.961045979755)*x[1]
        s[3,3,0,0]=(-0.27020299765)*x[0]**o+(-0.558034547573)*x[0]+(-0.524049515519)*x[1]**o+(-0.276595466979)*x[1]
        s[3,3,0,1]=(-0.510463688955)*x[0]**o+(-0.267341166705)*x[0]+(0.547679624943)*x[1]**o+(0.521119242662)*x[1]
        s[3,3,1,0]=(-0.155255975174)*x[0]**o+(0.365872110897)*x[0]+(-0.725704463467)*x[1]**o+(-0.790632654371)*x[1]
        s[3,3,1,1]=(-0.5111339808)*x[0]**o+(-0.0339146931841)*x[0]+(-0.696520447897)*x[1]**o+(-0.229679747746)*x[1]
        s[3,3,2,0]=(-0.552425399042)*x[0]**o+(-0.971205157087)*x[0]+(0.394260185879)*x[1]**o+(-0.164097031641)*x[1]
        s[3,3,2,1]=(-0.386876641825)*x[0]**o+(0.215459781811)*x[0]+(-0.332800684344)*x[1]**o+(0.419984531651)*x[1]
        s[3,4,0,0]=(-0.430256150077)*x[0]**o+(-0.110854674619)*x[0]+(0.687593230685)*x[1]**o+(-0.686895166454)*x[1]
        s[3,4,0,1]=(0.240753477602)*x[0]**o+(-0.815224522564)*x[0]+(0.499109852429)*x[1]**o+(-0.106870206363)*x[1]
        s[3,4,1,0]=(0.168547621411)*x[0]**o+(-0.509465054282)*x[0]+(-0.806347768707)*x[1]**o+(-0.799422054705)*x[1]
        s[3,4,1,1]=(-0.634806382514)*x[0]**o+(-0.125122793997)*x[0]+(-0.918019563645)*x[1]**o+(-0.341625694299)*x[1]
        s[3,4,2,0]=(-0.136108140149)*x[0]**o+(-0.780175946944)*x[0]+(0.0441301051696)*x[1]**o+(-0.326789608501)*x[1]
        s[3,4,2,1]=(0.181450357705)*x[0]**o+(0.467047306052)*x[0]+(0.182531831827)*x[1]**o+(-0.598701482526)*x[1]
        ref[0,0,0,0]=(-0.099815403496)*x_ref[0]**o+(-0.918641975262)*x_ref[0]+(-0.643800270537)*x_ref[1]**o+(-0.188947131246)*x_ref[1]
        ref[0,0,0,1]=(-0.654158499426)*x_ref[0]**o+(-0.158917070952)*x_ref[0]+(0.482580361439)*x_ref[1]**o+(0.654085120986)*x_ref[1]
        ref[0,0,1,0]=(-0.82639526996)*x_ref[0]**o+(-0.852788225935)*x_ref[0]+(-0.883559061247)*x_ref[1]**o+(-0.762093398747)*x_ref[1]
        ref[0,0,1,1]=(-0.410064006917)*x_ref[0]**o+(-0.358595400023)*x_ref[0]+(-0.744803691415)*x_ref[1]**o+(-0.418642435874)*x_ref[1]
        ref[0,0,2,0]=(0.627425019462)*x_ref[0]**o+(-0.759635228096)*x_ref[0]+(-0.225478401461)*x_ref[1]**o+(-0.394678152175)*x_ref[1]
        ref[0,0,2,1]=(0.282477965976)*x_ref[0]**o+(0.591668341745)*x_ref[0]+(0.997081550686)*x_ref[1]**o+(-0.16137686071)*x_ref[1]
        ref[0,1,0,0]=(0.503328443933)*x_ref[0]**o+(-0.349809904995)*x_ref[0]+(0.675840382965)*x_ref[1]**o+(0.122847940652)*x_ref[1]
        ref[0,1,0,1]=(-0.693842907429)*x_ref[0]**o+(0.242153763819)*x_ref[0]+(0.669516208483)*x_ref[1]**o+(-0.966639099473)*x_ref[1]
        ref[0,1,1,0]=(-0.143996595411)*x_ref[0]**o+(-0.0789367221661)*x_ref[0]+(0.44978165828)*x_ref[1]**o+(0.982272605823)*x_ref[1]
        ref[0,1,1,1]=(0.217504837315)*x_ref[0]**o+(0.62345834313)*x_ref[0]+(0.421663416031)*x_ref[1]**o+(0.298214724947)*x_ref[1]
        ref[0,1,2,0]=(-0.192788493597)*x_ref[0]**o+(0.59360099119)*x_ref[0]+(0.798640786481)*x_ref[1]**o+(-0.0618029984487)*x_ref[1]
        ref[0,1,2,1]=(-0.448474976435)*x_ref[0]**o+(-0.136832009015)*x_ref[0]+(-0.597189414931)*x_ref[1]**o+(0.502732362863)*x_ref[1]
        ref[0,2,0,0]=(0.252384046343)*x_ref[0]**o+(-0.400123690831)*x_ref[0]+(-0.545481308678)*x_ref[1]**o+(-0.388790715614)*x_ref[1]
        ref[0,2,0,1]=(-0.66240802679)*x_ref[0]**o+(0.77064500558)*x_ref[0]+(0.5239431782)*x_ref[1]**o+(0.467410945489)*x_ref[1]
        ref[0,2,1,0]=(-0.85322788336)*x_ref[0]**o+(-0.690030962118)*x_ref[0]+(0.699888315472)*x_ref[1]**o+(-0.89805156241)*x_ref[1]
        ref[0,2,1,1]=(-0.255889046261)*x_ref[0]**o+(0.0541259188422)*x_ref[0]+(-0.749587408291)*x_ref[1]**o+(0.791965754891)*x_ref[1]
        ref[0,2,2,0]=(0.496462410445)*x_ref[0]**o+(0.772074605719)*x_ref[0]+(0.180850727108)*x_ref[1]**o+(-0.487496085214)*x_ref[1]
        ref[0,2,2,1]=(0.706055709638)*x_ref[0]**o+(-0.598906910405)*x_ref[0]+(-0.152966385845)*x_ref[1]**o+(0.871383558924)*x_ref[1]
        ref[0,3,0,0]=(-0.951858396669)*x_ref[0]**o+(-0.794420515046)*x_ref[0]+(0.834659431502)*x_ref[1]**o+(0.481994252391)*x_ref[1]
        ref[0,3,0,1]=(-0.578325718357)*x_ref[0]**o+(-0.843177109188)*x_ref[0]+(0.38052630466)*x_ref[1]**o+(0.0999564588392)*x_ref[1]
        ref[0,3,1,0]=(0.251016293115)*x_ref[0]**o+(-0.846784412909)*x_ref[0]+(0.615130638894)*x_ref[1]**o+(-0.545472677499)*x_ref[1]
        ref[0,3,1,1]=(0.0242041003245)*x_ref[0]**o+(-0.558470673629)*x_ref[0]+(0.0355389532695)*x_ref[1]**o+(-0.82512767128)*x_ref[1]
        ref[0,3,2,0]=(0.104901847054)*x_ref[0]**o+(0.176002645436)*x_ref[0]+(-0.0529040105457)*x_ref[1]**o+(-0.497309382471)*x_ref[1]
        ref[0,3,2,1]=(-0.19187265394)*x_ref[0]**o+(-0.459102146282)*x_ref[0]+(0.263116634482)*x_ref[1]**o+(0.573723666261)*x_ref[1]
        ref[0,4,0,0]=(-0.605664936504)*x_ref[0]**o+(0.236623083899)*x_ref[0]+(-0.573965234185)*x_ref[1]**o+(-0.191541238111)*x_ref[1]
        ref[0,4,0,1]=(0.735297405377)*x_ref[0]**o+(-0.863752806793)*x_ref[0]+(0.130990772115)*x_ref[1]**o+(0.153051776075)*x_ref[1]
        ref[0,4,1,0]=(0.83806722488)*x_ref[0]**o+(0.952102086801)*x_ref[0]+(0.921278259866)*x_ref[1]**o+(0.100889262792)*x_ref[1]
        ref[0,4,1,1]=(0.422551160597)*x_ref[0]**o+(-0.68105266886)*x_ref[0]+(-0.0230668740384)*x_ref[1]**o+(0.608853763751)*x_ref[1]
        ref[0,4,2,0]=(-0.509106957279)*x_ref[0]**o+(0.535714529222)*x_ref[0]+(0.407059274864)*x_ref[1]**o+(0.0304780664648)*x_ref[1]
        ref[0,4,2,1]=(-0.899880077882)*x_ref[0]**o+(-0.150489820435)*x_ref[0]+(-0.329228777137)*x_ref[1]**o+(-0.325670285955)*x_ref[1]
        ref[1,0,0,0]=(0.723148414781)*x_ref[0]**o+(0.276861527427)*x_ref[0]+(-0.457164017461)*x_ref[1]**o+(0.587623869716)*x_ref[1]
        ref[1,0,0,1]=(-0.656042247607)*x_ref[0]**o+(0.599876456972)*x_ref[0]+(-0.487723393567)*x_ref[1]**o+(-0.293330841893)*x_ref[1]
        ref[1,0,1,0]=(0.498385487237)*x_ref[0]**o+(0.823826806076)*x_ref[0]+(-0.260001108163)*x_ref[1]**o+(-0.520083127954)*x_ref[1]
        ref[1,0,1,1]=(0.0511021547178)*x_ref[0]**o+(0.0752387451382)*x_ref[0]+(-0.295018309613)*x_ref[1]**o+(0.410655676791)*x_ref[1]
        ref[1,0,2,0]=(-0.926038025877)*x_ref[0]**o+(-0.101726746627)*x_ref[0]+(-0.714060837737)*x_ref[1]**o+(0.291653139474)*x_ref[1]
        ref[1,0,2,1]=(0.416231421476)*x_ref[0]**o+(0.968331479531)*x_ref[0]+(0.13575381957)*x_ref[1]**o+(0.0581635556746)*x_ref[1]
        ref[1,1,0,0]=(0.483218423871)*x_ref[0]**o+(0.837199391237)*x_ref[0]+(-0.332232511767)*x_ref[1]**o+(0.861714372392)*x_ref[1]
        ref[1,1,0,1]=(-0.861985810928)*x_ref[0]**o+(0.704494207363)*x_ref[0]+(0.0120560011404)*x_ref[1]**o+(-0.922350499146)*x_ref[1]
        ref[1,1,1,0]=(0.710471809896)*x_ref[0]**o+(0.10838308437)*x_ref[0]+(-0.936553657991)*x_ref[1]**o+(-0.157956217106)*x_ref[1]
        ref[1,1,1,1]=(0.61604545132)*x_ref[0]**o+(0.161202252636)*x_ref[0]+(-0.208993778286)*x_ref[1]**o+(0.150797102116)*x_ref[1]
        ref[1,1,2,0]=(0.237396232488)*x_ref[0]**o+(-0.260415056277)*x_ref[0]+(-0.760219527005)*x_ref[1]**o+(0.203694444805)*x_ref[1]
        ref[1,1,2,1]=(-0.656645187501)*x_ref[0]**o+(-0.84629755936)*x_ref[0]+(-0.465896055143)*x_ref[1]**o+(0.335524534826)*x_ref[1]
        ref[1,2,0,0]=(-0.546393675559)*x_ref[0]**o+(0.0344401682615)*x_ref[0]+(-0.267054613816)*x_ref[1]**o+(-0.345211291755)*x_ref[1]
        ref[1,2,0,1]=(0.807935620456)*x_ref[0]**o+(0.89820026562)*x_ref[0]+(-0.471804773843)*x_ref[1]**o+(0.0183931405417)*x_ref[1]
        ref[1,2,1,0]=(0.347457777403)*x_ref[0]**o+(0.894578611023)*x_ref[0]+(0.506086327041)*x_ref[1]**o+(-0.0552731455772)*x_ref[1]
        ref[1,2,1,1]=(0.898402940142)*x_ref[0]**o+(-0.850905749447)*x_ref[0]+(0.421790306199)*x_ref[1]**o+(0.560435305462)*x_ref[1]
        ref[1,2,2,0]=(0.417854170807)*x_ref[0]**o+(0.0808123920631)*x_ref[0]+(0.685114818766)*x_ref[1]**o+(0.627248749954)*x_ref[1]
        ref[1,2,2,1]=(0.608689720682)*x_ref[0]**o+(-0.215111751974)*x_ref[0]+(-0.157391477249)*x_ref[1]**o+(-0.157398505688)*x_ref[1]
        ref[1,3,0,0]=(0.453375349378)*x_ref[0]**o+(-0.232490378969)*x_ref[0]+(0.370581316376)*x_ref[1]**o+(0.2321645519)*x_ref[1]
        ref[1,3,0,1]=(-0.921453677654)*x_ref[0]**o+(-0.272608399386)*x_ref[0]+(0.980108178061)*x_ref[1]**o+(0.0304202383868)*x_ref[1]
        ref[1,3,1,0]=(-0.703951869845)*x_ref[0]**o+(0.232888375306)*x_ref[0]+(0.737108419956)*x_ref[1]**o+(-0.815317463111)*x_ref[1]
        ref[1,3,1,1]=(-0.849808179245)*x_ref[0]**o+(-0.848051887981)*x_ref[0]+(0.452157961656)*x_ref[1]**o+(0.942533763098)*x_ref[1]
        ref[1,3,2,0]=(0.276310583942)*x_ref[0]**o+(-0.426217969193)*x_ref[0]+(0.383056703597)*x_ref[1]**o+(-0.100608872707)*x_ref[1]
        ref[1,3,2,1]=(0.792743379061)*x_ref[0]**o+(-0.638898354)*x_ref[0]+(0.0900755365083)*x_ref[1]**o+(0.346700761651)*x_ref[1]
        ref[1,4,0,0]=(0.227940749448)*x_ref[0]**o+(-0.723141488706)*x_ref[0]+(-0.597539261807)*x_ref[1]**o+(-0.0547634570298)*x_ref[1]
        ref[1,4,0,1]=(-0.00150193441347)*x_ref[0]**o+(-0.875771867697)*x_ref[0]+(0.554295315299)*x_ref[1]**o+(0.179291021879)*x_ref[1]
        ref[1,4,1,0]=(0.665263753178)*x_ref[0]**o+(-0.261999229902)*x_ref[0]+(0.177708351596)*x_ref[1]**o+(-0.71011990376)*x_ref[1]
        ref[1,4,1,1]=(0.416094331837)*x_ref[0]**o+(0.809191029905)*x_ref[0]+(-0.631251155279)*x_ref[1]**o+(0.758593347696)*x_ref[1]
        ref[1,4,2,0]=(0.50941363283)*x_ref[0]**o+(-0.652536088555)*x_ref[0]+(-0.0994076235125)*x_ref[1]**o+(0.186614952474)*x_ref[1]
        ref[1,4,2,1]=(0.129263234225)*x_ref[0]**o+(0.115870119224)*x_ref[0]+(-0.25581908442)*x_ref[1]**o+(0.295990548791)*x_ref[1]
        ref[2,0,0,0]=(-0.432850417862)*x_ref[0]**o+(-0.491964804361)*x_ref[0]+(0.0106143845453)*x_ref[1]**o+(0.787362504175)*x_ref[1]
        ref[2,0,0,1]=(0.31617556259)*x_ref[0]**o+(0.255445781486)*x_ref[0]+(-0.302678692069)*x_ref[1]**o+(0.840840019713)*x_ref[1]
        ref[2,0,1,0]=(-0.855359978332)*x_ref[0]**o+(0.410554550223)*x_ref[0]+(-0.703253338388)*x_ref[1]**o+(0.59010970561)*x_ref[1]
        ref[2,0,1,1]=(-0.573270259685)*x_ref[0]**o+(0.928396842624)*x_ref[0]+(0.94155331127)*x_ref[1]**o+(0.160140257961)*x_ref[1]
        ref[2,0,2,0]=(0.0524505638003)*x_ref[0]**o+(-0.736508989499)*x_ref[0]+(-0.565060891384)*x_ref[1]**o+(-0.613736274563)*x_ref[1]
        ref[2,0,2,1]=(-0.776619329318)*x_ref[0]**o+(0.485104655735)*x_ref[0]+(-0.135863414015)*x_ref[1]**o+(-0.14173677679)*x_ref[1]
        ref[2,1,0,0]=(0.662345907479)*x_ref[0]**o+(-0.900993470049)*x_ref[0]+(-0.882752065723)*x_ref[1]**o+(0.642775104186)*x_ref[1]
        ref[2,1,0,1]=(-0.189295597666)*x_ref[0]**o+(0.0165605763413)*x_ref[0]+(0.989119296804)*x_ref[1]**o+(0.262678778013)*x_ref[1]
        ref[2,1,1,0]=(0.927673133577)*x_ref[0]**o+(-0.0248539427539)*x_ref[0]+(0.504610255651)*x_ref[1]**o+(-0.593911741472)*x_ref[1]
        ref[2,1,1,1]=(0.899525029448)*x_ref[0]**o+(-0.97195963201)*x_ref[0]+(-0.461931155897)*x_ref[1]**o+(0.91354314585)*x_ref[1]
        ref[2,1,2,0]=(0.398654077456)*x_ref[0]**o+(-0.429950726187)*x_ref[0]+(0.378910374088)*x_ref[1]**o+(-0.600308536915)*x_ref[1]
        ref[2,1,2,1]=(0.684159561112)*x_ref[0]**o+(-0.408717922991)*x_ref[0]+(0.0986838286044)*x_ref[1]**o+(0.446743772828)*x_ref[1]
        ref[2,2,0,0]=(-0.657789003001)*x_ref[0]**o+(-0.696243850831)*x_ref[0]+(0.143074391914)*x_ref[1]**o+(-0.784498727037)*x_ref[1]
        ref[2,2,0,1]=(-0.035450676833)*x_ref[0]**o+(-0.453531837262)*x_ref[0]+(0.59955708994)*x_ref[1]**o+(0.35879284257)*x_ref[1]
        ref[2,2,1,0]=(0.302411531858)*x_ref[0]**o+(-0.120739744552)*x_ref[0]+(0.23248426438)*x_ref[1]**o+(-0.39024395986)*x_ref[1]
        ref[2,2,1,1]=(0.957811592451)*x_ref[0]**o+(-0.444133483391)*x_ref[0]+(-0.917299241507)*x_ref[1]**o+(0.66053932766)*x_ref[1]
        ref[2,2,2,0]=(0.361654572167)*x_ref[0]**o+(-0.860498450204)*x_ref[0]+(0.839403953375)*x_ref[1]**o+(-0.386466411338)*x_ref[1]
        ref[2,2,2,1]=(-0.0143243836126)*x_ref[0]**o+(0.444553204161)*x_ref[0]+(0.0214936163191)*x_ref[1]**o+(-0.257450046824)*x_ref[1]
        ref[2,3,0,0]=(0.841171215625)*x_ref[0]**o+(-0.342091657341)*x_ref[0]+(0.429215544167)*x_ref[1]**o+(0.291784964305)*x_ref[1]
        ref[2,3,0,1]=(0.650468319582)*x_ref[0]**o+(-0.121782834777)*x_ref[0]+(0.887022664242)*x_ref[1]**o+(0.957311577092)*x_ref[1]
        ref[2,3,1,0]=(0.156495854578)*x_ref[0]**o+(0.855814547392)*x_ref[0]+(0.320209813846)*x_ref[1]**o+(0.458769463001)*x_ref[1]
        ref[2,3,1,1]=(0.34344265441)*x_ref[0]**o+(-0.992393068373)*x_ref[0]+(-0.0554086726336)*x_ref[1]**o+(-0.985825317842)*x_ref[1]
        ref[2,3,2,0]=(-0.866276931509)*x_ref[0]**o+(-0.343662754778)*x_ref[0]+(0.0302242797749)*x_ref[1]**o+(-0.652051073003)*x_ref[1]
        ref[2,3,2,1]=(-0.68451689603)*x_ref[0]**o+(0.564121866438)*x_ref[0]+(-0.136564076086)*x_ref[1]**o+(0.648701900333)*x_ref[1]
        ref[2,4,0,0]=(-0.430429138775)*x_ref[0]**o+(0.721222678082)*x_ref[0]+(-0.0375465605006)*x_ref[1]**o+(-0.462643638758)*x_ref[1]
        ref[2,4,0,1]=(-0.321870157728)*x_ref[0]**o+(0.884979965704)*x_ref[0]+(0.210984337677)*x_ref[1]**o+(-0.796486469869)*x_ref[1]
        ref[2,4,1,0]=(0.365646287696)*x_ref[0]**o+(-0.600693071289)*x_ref[0]+(-0.998407218964)*x_ref[1]**o+(-0.119949305545)*x_ref[1]
        ref[2,4,1,1]=(-0.763923462892)*x_ref[0]**o+(-0.831011577325)*x_ref[0]+(0.547670679354)*x_ref[1]**o+(-0.856456618287)*x_ref[1]
        ref[2,4,2,0]=(0.107211541473)*x_ref[0]**o+(-0.787737471224)*x_ref[0]+(0.00381729482611)*x_ref[1]**o+(-0.737759009165)*x_ref[1]
        ref[2,4,2,1]=(0.233196464862)*x_ref[0]**o+(-0.464864449316)*x_ref[0]+(0.229081432725)*x_ref[1]**o+(-0.316321259146)*x_ref[1]
        ref[3,0,0,0]=(0.84633152323)*x_ref[0]**o+(0.107225324454)*x_ref[0]+(-0.768659465295)*x_ref[1]**o+(0.441758939429)*x_ref[1]
        ref[3,0,0,1]=(-0.317081751011)*x_ref[0]**o+(-0.557807443325)*x_ref[0]+(-0.513518142998)*x_ref[1]**o+(0.969406024047)*x_ref[1]
        ref[3,0,1,0]=(0.470133545453)*x_ref[0]**o+(0.689603661125)*x_ref[0]+(-0.358563691038)*x_ref[1]**o+(0.0930824872861)*x_ref[1]
        ref[3,0,1,1]=(0.31919974075)*x_ref[0]**o+(-0.555789020025)*x_ref[0]+(0.0108880065973)*x_ref[1]**o+(0.519548687393)*x_ref[1]
        ref[3,0,2,0]=(0.663881497435)*x_ref[0]**o+(-0.668884360184)*x_ref[0]+(-0.249669522726)*x_ref[1]**o+(0.978543111491)*x_ref[1]
        ref[3,0,2,1]=(0.119167252087)*x_ref[0]**o+(0.80927599582)*x_ref[0]+(0.526489556971)*x_ref[1]**o+(-0.440104597696)*x_ref[1]
        ref[3,1,0,0]=(-0.367446660377)*x_ref[0]**o+(-0.326052348072)*x_ref[0]+(0.841697795111)*x_ref[1]**o+(0.403206833675)*x_ref[1]
        ref[3,1,0,1]=(-0.647927382559)*x_ref[0]**o+(0.096852636085)*x_ref[0]+(-0.126798636274)*x_ref[1]**o+(0.117295901868)*x_ref[1]
        ref[3,1,1,0]=(0.827676755777)*x_ref[0]**o+(-0.922576416262)*x_ref[0]+(-0.3927169369)*x_ref[1]**o+(0.828141309451)*x_ref[1]
        ref[3,1,1,1]=(-0.0442514478276)*x_ref[0]**o+(-0.17904563214)*x_ref[0]+(0.0870253733863)*x_ref[1]**o+(-0.521249847127)*x_ref[1]
        ref[3,1,2,0]=(0.295531599451)*x_ref[0]**o+(-0.125313746461)*x_ref[0]+(0.982602753269)*x_ref[1]**o+(-0.469029795983)*x_ref[1]
        ref[3,1,2,1]=(0.950273049304)*x_ref[0]**o+(0.859212132249)*x_ref[0]+(0.728565507333)*x_ref[1]**o+(-0.274531603828)*x_ref[1]
        ref[3,2,0,0]=(-0.885845551336)*x_ref[0]**o+(-0.704449229328)*x_ref[0]+(-0.571836258871)*x_ref[1]**o+(-0.0476396264682)*x_ref[1]
        ref[3,2,0,1]=(0.0713925637733)*x_ref[0]**o+(0.294650464749)*x_ref[0]+(-0.917378997351)*x_ref[1]**o+(-0.715770468859)*x_ref[1]
        ref[3,2,1,0]=(-0.715304376467)*x_ref[0]**o+(0.252625168227)*x_ref[0]+(0.942098162684)*x_ref[1]**o+(-0.0579654780836)*x_ref[1]
        ref[3,2,1,1]=(-0.937718965835)*x_ref[0]**o+(0.649603348472)*x_ref[0]+(-0.293284658463)*x_ref[1]**o+(-0.347005762941)*x_ref[1]
        ref[3,2,2,0]=(-0.042358031525)*x_ref[0]**o+(0.036891446424)*x_ref[0]+(-0.229597055696)*x_ref[1]**o+(0.0283282394156)*x_ref[1]
        ref[3,2,2,1]=(-0.0186366313091)*x_ref[0]**o+(-0.0127649188124)*x_ref[0]+(0.803780828202)*x_ref[1]**o+(0.961045979755)*x_ref[1]
        ref[3,3,0,0]=(-0.27020299765)*x_ref[0]**o+(-0.558034547573)*x_ref[0]+(-0.524049515519)*x_ref[1]**o+(-0.276595466979)*x_ref[1]
        ref[3,3,0,1]=(-0.510463688955)*x_ref[0]**o+(-0.267341166705)*x_ref[0]+(0.547679624943)*x_ref[1]**o+(0.521119242662)*x_ref[1]
        ref[3,3,1,0]=(-0.155255975174)*x_ref[0]**o+(0.365872110897)*x_ref[0]+(-0.725704463467)*x_ref[1]**o+(-0.790632654371)*x_ref[1]
        ref[3,3,1,1]=(-0.5111339808)*x_ref[0]**o+(-0.0339146931841)*x_ref[0]+(-0.696520447897)*x_ref[1]**o+(-0.229679747746)*x_ref[1]
        ref[3,3,2,0]=(-0.552425399042)*x_ref[0]**o+(-0.971205157087)*x_ref[0]+(0.394260185879)*x_ref[1]**o+(-0.164097031641)*x_ref[1]
        ref[3,3,2,1]=(-0.386876641825)*x_ref[0]**o+(0.215459781811)*x_ref[0]+(-0.332800684344)*x_ref[1]**o+(0.419984531651)*x_ref[1]
        ref[3,4,0,0]=(-0.430256150077)*x_ref[0]**o+(-0.110854674619)*x_ref[0]+(0.687593230685)*x_ref[1]**o+(-0.686895166454)*x_ref[1]
        ref[3,4,0,1]=(0.240753477602)*x_ref[0]**o+(-0.815224522564)*x_ref[0]+(0.499109852429)*x_ref[1]**o+(-0.106870206363)*x_ref[1]
        ref[3,4,1,0]=(0.168547621411)*x_ref[0]**o+(-0.509465054282)*x_ref[0]+(-0.806347768707)*x_ref[1]**o+(-0.799422054705)*x_ref[1]
        ref[3,4,1,1]=(-0.634806382514)*x_ref[0]**o+(-0.125122793997)*x_ref[0]+(-0.918019563645)*x_ref[1]**o+(-0.341625694299)*x_ref[1]
        ref[3,4,2,0]=(-0.136108140149)*x_ref[0]**o+(-0.780175946944)*x_ref[0]+(0.0441301051696)*x_ref[1]**o+(-0.326789608501)*x_ref[1]
        ref[3,4,2,1]=(0.181450357705)*x_ref[0]**o+(0.467047306052)*x_ref[0]+(0.182531831827)*x_ref[1]**o+(-0.598701482526)*x_ref[1]
      else:
        s[0,0,0,0]=(-0.0362500650077)*x[0]**o+(0.692407811012)*x[0]+(-0.855107417704)*x[1]**o+(-0.0397854200762)*x[1]+(0.488177999316)*x[2]**o+(0.946478266036)*x[2]
        s[0,0,0,1]=(-0.533284434038)*x[0]**o+(-0.599290705017)*x[0]+(0.198922961617)*x[1]**o+(0.652839426276)*x[1]+(0.560109621832)*x[2]**o+(0.167310997616)*x[2]
        s[0,0,1,0]=(-0.21016675446)*x[0]**o+(0.812498932794)*x[0]+(-0.611566184558)*x[1]**o+(0.626231122402)*x[1]+(0.536374628942)*x[2]**o+(0.8520547245)*x[2]
        s[0,0,1,1]=(0.26308571414)*x[0]**o+(-0.359237396089)*x[0]+(0.146879292748)*x[1]**o+(-0.241831631389)*x[1]+(0.812820522163)*x[2]**o+(0.824191692943)*x[2]
        s[0,0,2,0]=(-0.585682391332)*x[0]**o+(0.82273803111)*x[0]+(-0.715688315155)*x[1]**o+(-0.997893564593)*x[1]+(0.39588322481)*x[2]**o+(0.410447772982)*x[2]
        s[0,0,2,1]=(0.491817354047)*x[0]**o+(0.105289616977)*x[0]+(0.0581932965006)*x[1]**o+(-0.584153248946)*x[1]+(-0.837305609444)*x[2]**o+(-0.140188544108)*x[2]
        s[0,1,0,0]=(-0.158174387659)*x[0]**o+(0.774512978962)*x[0]+(0.953772629099)*x[1]**o+(-0.0474582135671)*x[1]+(-0.691917863221)*x[2]**o+(0.978550525536)*x[2]
        s[0,1,0,1]=(-0.696002534433)*x[0]**o+(-0.904833590237)*x[0]+(-0.991879336542)*x[1]**o+(-0.00301423741703)*x[1]+(-0.495919957691)*x[2]**o+(0.923872653662)*x[2]
        s[0,1,1,0]=(-0.617187346815)*x[0]**o+(-0.0283413268672)*x[0]+(0.439716253425)*x[1]**o+(0.679148169861)*x[1]+(-0.5494839537)*x[2]**o+(0.780531698733)*x[2]
        s[0,1,1,1]=(0.0763086894431)*x[0]**o+(0.149024988807)*x[0]+(0.660061724201)*x[1]**o+(0.227260856773)*x[1]+(0.604941557505)*x[2]**o+(-0.32106743372)*x[2]
        s[0,1,2,0]=(-0.710651974651)*x[0]**o+(0.729233023268)*x[0]+(0.830588938931)*x[1]**o+(0.930100931807)*x[1]+(0.576259601615)*x[2]**o+(0.995388406293)*x[2]
        s[0,1,2,1]=(0.635415205147)*x[0]**o+(-0.0475242382825)*x[0]+(0.779184290843)*x[1]**o+(0.161316071106)*x[1]+(-0.321426515539)*x[2]**o+(-0.188018565147)*x[2]
        s[0,2,0,0]=(-0.234573769896)*x[0]**o+(-0.35461213258)*x[0]+(0.322664245412)*x[1]**o+(-0.36584459555)*x[1]+(0.656560085187)*x[2]**o+(-0.380544830082)*x[2]
        s[0,2,0,1]=(0.314171153213)*x[0]**o+(-0.871055875712)*x[0]+(0.600399452124)*x[1]**o+(-0.710939791885)*x[1]+(-0.189029597187)*x[2]**o+(-0.29466779727)*x[2]
        s[0,2,1,0]=(0.911500533914)*x[0]**o+(-0.217077039937)*x[0]+(0.488325722425)*x[1]**o+(0.781071969291)*x[1]+(-0.883567213205)*x[2]**o+(0.938457876454)*x[2]
        s[0,2,1,1]=(0.947554813988)*x[0]**o+(0.108161094029)*x[0]+(0.0814722282926)*x[1]**o+(0.394738648377)*x[1]+(-0.585604351258)*x[2]**o+(0.424262882559)*x[2]
        s[0,2,2,0]=(0.130793077586)*x[0]**o+(-0.0759869806951)*x[0]+(0.524572237994)*x[1]**o+(-0.0494098932334)*x[1]+(-0.162573831498)*x[2]**o+(0.950665975392)*x[2]
        s[0,2,2,1]=(0.0636110845427)*x[0]**o+(-0.152586932248)*x[0]+(-0.428472289058)*x[1]**o+(0.943093556156)*x[1]+(-0.925181631256)*x[2]**o+(-0.850747416138)*x[2]
        s[0,3,0,0]=(0.803654693645)*x[0]**o+(-0.852793510169)*x[0]+(0.372325502382)*x[1]**o+(0.437191610865)*x[1]+(-0.626819779481)*x[2]**o+(-0.969085192013)*x[2]
        s[0,3,0,1]=(-0.498449247588)*x[0]**o+(-0.174254842954)*x[0]+(-0.751570511608)*x[1]**o+(0.0278714978689)*x[1]+(0.732061097781)*x[2]**o+(0.434922200273)*x[2]
        s[0,3,1,0]=(0.356966448806)*x[0]**o+(-0.754346917896)*x[0]+(-0.200096880735)*x[1]**o+(0.726202986575)*x[1]+(-0.273975127289)*x[2]**o+(0.922054852117)*x[2]
        s[0,3,1,1]=(0.294033872459)*x[0]**o+(0.435258372604)*x[0]+(-0.606222342027)*x[1]**o+(-0.418222134998)*x[1]+(-0.617263336049)*x[2]**o+(-0.655912764464)*x[2]
        s[0,3,2,0]=(-0.552805093123)*x[0]**o+(0.814985531695)*x[0]+(0.201701287999)*x[1]**o+(0.449378802496)*x[1]+(0.106877031271)*x[2]**o+(0.972523159602)*x[2]
        s[0,3,2,1]=(-0.123112956117)*x[0]**o+(-0.9669799128)*x[0]+(-0.365437347923)*x[1]**o+(0.95077682916)*x[1]+(-0.68795960948)*x[2]**o+(0.282323391952)*x[2]
        s[0,4,0,0]=(0.762949486092)*x[0]**o+(0.275310784251)*x[0]+(0.90824072581)*x[1]**o+(0.267277710972)*x[1]+(-0.387183781105)*x[2]**o+(-0.919714964098)*x[2]
        s[0,4,0,1]=(-0.219860124563)*x[0]**o+(0.458617314398)*x[0]+(-0.253950962469)*x[1]**o+(-0.213326656667)*x[1]+(-0.914189191308)*x[2]**o+(-0.88808493928)*x[2]
        s[0,4,1,0]=(0.0111879656405)*x[0]**o+(0.857083093627)*x[0]+(0.397087247889)*x[1]**o+(-0.411624885427)*x[1]+(-0.533535312833)*x[2]**o+(-0.473765290954)*x[2]
        s[0,4,1,1]=(0.408477302242)*x[0]**o+(-0.336255491206)*x[0]+(0.535004316057)*x[1]**o+(0.646300732798)*x[1]+(-0.683570698066)*x[2]**o+(-0.691723219637)*x[2]
        s[0,4,2,0]=(0.229517234528)*x[0]**o+(-0.327638251614)*x[0]+(0.752536339629)*x[1]**o+(0.440600000033)*x[1]+(-0.166203457183)*x[2]**o+(-0.632615828058)*x[2]
        s[0,4,2,1]=(0.128453646267)*x[0]**o+(-0.152501701527)*x[0]+(0.131616948331)*x[1]**o+(-0.560356219781)*x[1]+(0.369923630213)*x[2]**o+(-0.405937225725)*x[2]
        s[1,0,0,0]=(-0.498802287823)*x[0]**o+(0.974250100105)*x[0]+(-0.752335049339)*x[1]**o+(0.460106742976)*x[1]+(0.878914200699)*x[2]**o+(-0.85963539732)*x[2]
        s[1,0,0,1]=(0.937345235384)*x[0]**o+(0.0616042350221)*x[0]+(-0.63597655214)*x[1]**o+(0.200363730821)*x[1]+(0.677520498976)*x[2]**o+(0.310367850218)*x[2]
        s[1,0,1,0]=(-0.328289664191)*x[0]**o+(-0.439002316683)*x[0]+(-0.753405240499)*x[1]**o+(-0.749020227582)*x[1]+(0.200333899797)*x[2]**o+(0.63068525957)*x[2]
        s[1,0,1,1]=(-0.522381373851)*x[0]**o+(-0.877707833729)*x[0]+(-0.0334841793702)*x[1]**o+(0.00769816693416)*x[1]+(-0.81697893166)*x[2]**o+(-0.47349699259)*x[2]
        s[1,0,2,0]=(0.0786863277722)*x[0]**o+(-0.47462306827)*x[0]+(-0.850065158625)*x[1]**o+(0.0481804762762)*x[1]+(0.239465148926)*x[2]**o+(0.484166919183)*x[2]
        s[1,0,2,1]=(-0.0411433282485)*x[0]**o+(-0.851343034379)*x[0]+(-0.518404507699)*x[1]**o+(-0.357342937948)*x[1]+(-0.57398214485)*x[2]**o+(-0.0989103206187)*x[2]
        s[1,1,0,0]=(-0.060737698278)*x[0]**o+(-0.103483640965)*x[0]+(-0.206004957088)*x[1]**o+(0.556901021479)*x[1]+(0.951054830915)*x[2]**o+(0.668118573076)*x[2]
        s[1,1,0,1]=(0.0668860744649)*x[0]**o+(0.0490910382588)*x[0]+(0.817265796062)*x[1]**o+(-0.939419683478)*x[1]+(-0.800905110922)*x[2]**o+(-0.546647838589)*x[2]
        s[1,1,1,0]=(-0.437101637549)*x[0]**o+(0.269260162519)*x[0]+(-0.026802186298)*x[1]**o+(-0.932589891991)*x[1]+(-0.313814820129)*x[2]**o+(-0.436056943196)*x[2]
        s[1,1,1,1]=(-0.893118860321)*x[0]**o+(-0.837193118481)*x[0]+(-0.58313090725)*x[1]**o+(-0.657017739851)*x[1]+(-0.0158333033033)*x[2]**o+(0.286927848108)*x[2]
        s[1,1,2,0]=(-0.414914106956)*x[0]**o+(-0.295021056636)*x[0]+(-0.100513024651)*x[1]**o+(0.119321919323)*x[1]+(-0.911626057698)*x[2]**o+(0.0715733685275)*x[2]
        s[1,1,2,1]=(0.415073119201)*x[0]**o+(-0.79060072691)*x[0]+(-0.280560285152)*x[1]**o+(-0.651271923585)*x[1]+(-0.783523725291)*x[2]**o+(-0.165699899479)*x[2]
        s[1,2,0,0]=(0.204696440606)*x[0]**o+(0.58070882041)*x[0]+(0.498396642996)*x[1]**o+(0.270570176185)*x[1]+(-0.261604910878)*x[2]**o+(0.788402570665)*x[2]
        s[1,2,0,1]=(-0.163227090863)*x[0]**o+(0.17193387282)*x[0]+(0.559640507601)*x[1]**o+(0.39180422655)*x[1]+(0.182426715866)*x[2]**o+(0.52544860689)*x[2]
        s[1,2,1,0]=(-0.0826612289833)*x[0]**o+(-0.244737952715)*x[0]+(0.887738370973)*x[1]**o+(0.593438512127)*x[1]+(0.73128277327)*x[2]**o+(0.229067764431)*x[2]
        s[1,2,1,1]=(-0.117634040351)*x[0]**o+(-0.490365253501)*x[0]+(0.654067544802)*x[1]**o+(-0.862866217687)*x[1]+(-0.563450922747)*x[2]**o+(-0.785940469253)*x[2]
        s[1,2,2,0]=(0.486381246904)*x[0]**o+(-0.119137386359)*x[0]+(0.251388315083)*x[1]**o+(-0.142435376157)*x[1]+(-0.882418773242)*x[2]**o+(0.509436056026)*x[2]
        s[1,2,2,1]=(-0.69532427028)*x[0]**o+(-0.587331382396)*x[0]+(0.750092712907)*x[1]**o+(0.864188183197)*x[1]+(-0.0336735158745)*x[2]**o+(-0.118692941484)*x[2]
        s[1,3,0,0]=(0.287194598069)*x[0]**o+(0.775480786321)*x[0]+(0.24820537283)*x[1]**o+(0.579330345046)*x[1]+(0.641058615756)*x[2]**o+(-0.358716444584)*x[2]
        s[1,3,0,1]=(-0.218876040408)*x[0]**o+(-0.581134176647)*x[0]+(0.47937196949)*x[1]**o+(0.0529063680602)*x[1]+(-0.42659429504)*x[2]**o+(-0.445845798304)*x[2]
        s[1,3,1,0]=(0.500331134532)*x[0]**o+(0.764920932873)*x[0]+(0.256972231189)*x[1]**o+(0.279967260143)*x[1]+(0.90143027778)*x[2]**o+(-0.753075337423)*x[2]
        s[1,3,1,1]=(0.0478484319999)*x[0]**o+(0.239476880568)*x[0]+(-0.286765457909)*x[1]**o+(-0.952926972714)*x[1]+(-0.117698931152)*x[2]**o+(-0.950120391675)*x[2]
        s[1,3,2,0]=(-0.404765291554)*x[0]**o+(0.0796360913293)*x[0]+(0.743564239837)*x[1]**o+(-0.0431456147708)*x[1]+(-0.82536795966)*x[2]**o+(-0.770711063789)*x[2]
        s[1,3,2,1]=(-0.6113113129)*x[0]**o+(0.330568908804)*x[0]+(-0.391402722722)*x[1]**o+(0.415093002807)*x[1]+(-0.893617348746)*x[2]**o+(0.520968946942)*x[2]
        s[1,4,0,0]=(-0.209574438366)*x[0]**o+(0.644246054163)*x[0]+(-0.0664046919847)*x[1]**o+(0.214820911697)*x[1]+(-0.10545535704)*x[2]**o+(0.655063184877)*x[2]
        s[1,4,0,1]=(-0.38446029244)*x[0]**o+(-0.802691253671)*x[0]+(-0.306211901233)*x[1]**o+(-0.755184562233)*x[1]+(-0.704583961281)*x[2]**o+(0.0410695954026)*x[2]
        s[1,4,1,0]=(-0.512877649374)*x[0]**o+(-0.0190184293267)*x[0]+(0.588904418509)*x[1]**o+(-0.343320898588)*x[1]+(-0.16543066117)*x[2]**o+(-0.118363009949)*x[2]
        s[1,4,1,1]=(-0.410758135711)*x[0]**o+(0.302846037727)*x[0]+(0.674701250578)*x[1]**o+(-0.96295102282)*x[1]+(0.787271275616)*x[2]**o+(0.925818905258)*x[2]
        s[1,4,2,0]=(0.629642717166)*x[0]**o+(-0.696019558128)*x[0]+(0.465609587281)*x[1]**o+(0.191961667012)*x[1]+(-0.0182853530912)*x[2]**o+(0.504352699288)*x[2]
        s[1,4,2,1]=(-0.561295424723)*x[0]**o+(0.830136434947)*x[0]+(0.981595445935)*x[1]**o+(-0.0881426921172)*x[1]+(-0.581790941676)*x[2]**o+(-0.489101727188)*x[2]
        s[2,0,0,0]=(0.173249525275)*x[0]**o+(0.376871332154)*x[0]+(-0.379974081425)*x[1]**o+(-0.118151578515)*x[1]+(0.371715379047)*x[2]**o+(0.115273894116)*x[2]
        s[2,0,0,1]=(0.656339759814)*x[0]**o+(-0.0783529441613)*x[0]+(0.988608900721)*x[1]**o+(-0.498370704737)*x[1]+(0.886731679317)*x[2]**o+(0.251026122418)*x[2]
        s[2,0,1,0]=(-0.350592561402)*x[0]**o+(-0.592294993708)*x[0]+(0.860912193414)*x[1]**o+(0.678591565883)*x[1]+(-0.823185915223)*x[2]**o+(-0.882500202366)*x[2]
        s[2,0,1,1]=(0.84096537128)*x[0]**o+(-0.78582850022)*x[0]+(-0.872411932976)*x[1]**o+(0.208867262377)*x[1]+(0.795227125238)*x[2]**o+(0.434019201479)*x[2]
        s[2,0,2,0]=(0.670038316075)*x[0]**o+(0.221065948244)*x[0]+(-0.0801945207329)*x[1]**o+(0.35963157451)*x[1]+(-0.523905944746)*x[2]**o+(0.113327871154)*x[2]
        s[2,0,2,1]=(-0.790165918677)*x[0]**o+(0.826330759592)*x[0]+(0.816532564314)*x[1]**o+(-0.947088135238)*x[1]+(0.498519891556)*x[2]**o+(-0.221796346503)*x[2]
        s[2,1,0,0]=(0.679814393957)*x[0]**o+(-0.287910028702)*x[0]+(-0.179717372152)*x[1]**o+(-0.895091171351)*x[1]+(0.453459163192)*x[2]**o+(-0.732993783456)*x[2]
        s[2,1,0,1]=(-0.917014070487)*x[0]**o+(-0.28427666054)*x[0]+(0.520172057178)*x[1]**o+(0.973381422092)*x[1]+(-0.785604082848)*x[2]**o+(-0.280246362235)*x[2]
        s[2,1,1,0]=(-0.975543038311)*x[0]**o+(0.0471394295003)*x[0]+(-0.744014701)*x[1]**o+(-0.309281686644)*x[1]+(0.970209409582)*x[2]**o+(0.182638690597)*x[2]
        s[2,1,1,1]=(0.754879118472)*x[0]**o+(-0.345631769168)*x[0]+(0.864272677704)*x[1]**o+(-0.634026748154)*x[1]+(-0.25656646391)*x[2]**o+(-0.976807129917)*x[2]
        s[2,1,2,0]=(0.0378913818133)*x[0]**o+(-0.589418434757)*x[0]+(0.500291612315)*x[1]**o+(0.831173369231)*x[1]+(0.0773865261454)*x[2]**o+(-0.170449265388)*x[2]
        s[2,1,2,1]=(0.363668102897)*x[0]**o+(0.0152347755192)*x[0]+(0.295985339318)*x[1]**o+(-0.695348667793)*x[1]+(0.654035675777)*x[2]**o+(-0.998480773881)*x[2]
        s[2,2,0,0]=(-0.248968924843)*x[0]**o+(-0.454040417829)*x[0]+(0.618989286259)*x[1]**o+(-0.284627664137)*x[1]+(0.597922789444)*x[2]**o+(-0.0544707719136)*x[2]
        s[2,2,0,1]=(-0.564540493714)*x[0]**o+(0.948462945081)*x[0]+(-0.554305925084)*x[1]**o+(-0.683216336448)*x[1]+(-0.393720863511)*x[2]**o+(-0.454706611434)*x[2]
        s[2,2,1,0]=(0.00267926298461)*x[0]**o+(0.69057345924)*x[0]+(0.972990086053)*x[1]**o+(0.334585200812)*x[1]+(0.458913228071)*x[2]**o+(-0.927443691914)*x[2]
        s[2,2,1,1]=(-0.869817596024)*x[0]**o+(0.183295661514)*x[0]+(-0.215314084509)*x[1]**o+(0.274879624616)*x[1]+(-0.872459015851)*x[2]**o+(-0.43371203901)*x[2]
        s[2,2,2,0]=(-0.59916469262)*x[0]**o+(-0.635232312962)*x[0]+(-0.730547369574)*x[1]**o+(-0.199910106304)*x[1]+(-0.81824698793)*x[2]**o+(-0.351742356136)*x[2]
        s[2,2,2,1]=(0.52115654452)*x[0]**o+(-0.648726400371)*x[0]+(0.0705029035387)*x[1]**o+(0.82069581623)*x[1]+(0.262897099948)*x[2]**o+(0.302543200334)*x[2]
        s[2,3,0,0]=(-0.887996543825)*x[0]**o+(-0.141917392835)*x[0]+(0.458379292046)*x[1]**o+(0.279749722412)*x[1]+(-0.185582606045)*x[2]**o+(-0.408626717355)*x[2]
        s[2,3,0,1]=(-0.0708646458785)*x[0]**o+(-0.723087450876)*x[0]+(0.801053779885)*x[1]**o+(-0.316692437663)*x[1]+(0.313620016387)*x[2]**o+(-0.802237492322)*x[2]
        s[2,3,1,0]=(-0.413969268245)*x[0]**o+(-0.419189503807)*x[0]+(-0.150655589102)*x[1]**o+(-0.451064114786)*x[1]+(-0.11752330372)*x[2]**o+(-0.564979364641)*x[2]
        s[2,3,1,1]=(-0.127258808057)*x[0]**o+(-0.166240866424)*x[0]+(0.482435262134)*x[1]**o+(0.210083811762)*x[1]+(0.496523704735)*x[2]**o+(-0.528305427062)*x[2]
        s[2,3,2,0]=(-0.619368230074)*x[0]**o+(0.196929842772)*x[0]+(0.45226305027)*x[1]**o+(-0.949509654146)*x[1]+(-0.200199092979)*x[2]**o+(0.879368741494)*x[2]
        s[2,3,2,1]=(0.727974979035)*x[0]**o+(-0.486118068882)*x[0]+(0.330990540492)*x[1]**o+(0.627547710515)*x[1]+(-0.00312667743686)*x[2]**o+(0.34180677866)*x[2]
        s[2,4,0,0]=(-0.849340738634)*x[0]**o+(0.105575131464)*x[0]+(0.531365100651)*x[1]**o+(-0.885829562648)*x[1]+(-0.959105112582)*x[2]**o+(0.6210710685)*x[2]
        s[2,4,0,1]=(-0.525287919972)*x[0]**o+(0.685382709266)*x[0]+(0.706077262452)*x[1]**o+(-0.577859356102)*x[1]+(0.494281341217)*x[2]**o+(0.936337201867)*x[2]
        s[2,4,1,0]=(-0.0263664228096)*x[0]**o+(0.609335710155)*x[0]+(-0.417542795725)*x[1]**o+(0.127890280292)*x[1]+(0.0530446261715)*x[2]**o+(-0.203436845419)*x[2]
        s[2,4,1,1]=(0.58910765961)*x[0]**o+(-0.233711911277)*x[0]+(-0.68506108197)*x[1]**o+(-0.796608219305)*x[1]+(-0.55051026522)*x[2]**o+(-0.663284299983)*x[2]
        s[2,4,2,0]=(0.407867058666)*x[0]**o+(-0.155472572874)*x[0]+(0.0574795179339)*x[1]**o+(0.534562635965)*x[1]+(-0.35303239373)*x[2]**o+(0.690463242713)*x[2]
        s[2,4,2,1]=(0.458514172477)*x[0]**o+(0.493538792362)*x[0]+(-0.534087073406)*x[1]**o+(0.140190942794)*x[1]+(0.53904059561)*x[2]**o+(0.511924245166)*x[2]
        s[3,0,0,0]=(0.80688664483)*x[0]**o+(0.641990457435)*x[0]+(0.258439350653)*x[1]**o+(0.360949512497)*x[1]+(0.71055262439)*x[2]**o+(-0.14812833383)*x[2]
        s[3,0,0,1]=(0.349076196526)*x[0]**o+(-0.250426445203)*x[0]+(0.363350736371)*x[1]**o+(0.62331683755)*x[1]+(-0.799481730613)*x[2]**o+(0.86946545402)*x[2]
        s[3,0,1,0]=(-0.461900551752)*x[0]**o+(-0.530275233976)*x[0]+(0.778112655963)*x[1]**o+(-0.548160947996)*x[1]+(0.27643541045)*x[2]**o+(0.558519627971)*x[2]
        s[3,0,1,1]=(-0.870845434837)*x[0]**o+(0.90473098031)*x[0]+(0.356324338565)*x[1]**o+(-0.173579775601)*x[1]+(0.866960109024)*x[2]**o+(0.22810880895)*x[2]
        s[3,0,2,0]=(0.449562471445)*x[0]**o+(-0.133977676845)*x[0]+(-0.945220890263)*x[1]**o+(0.999013496171)*x[1]+(-0.612837266133)*x[2]**o+(-0.695797874487)*x[2]
        s[3,0,2,1]=(-0.588105630499)*x[0]**o+(0.176592342414)*x[0]+(-0.820991814796)*x[1]**o+(0.0010811086852)*x[1]+(0.498174763907)*x[2]**o+(-0.0223969132485)*x[2]
        s[3,1,0,0]=(0.653483773188)*x[0]**o+(0.460215104331)*x[0]+(-0.287173677147)*x[1]**o+(-0.837222104854)*x[1]+(0.457524748826)*x[2]**o+(-0.638268772518)*x[2]
        s[3,1,0,1]=(-0.0999951386915)*x[0]**o+(0.294288175142)*x[0]+(0.159178572562)*x[1]**o+(0.13437803751)*x[1]+(-0.411939992038)*x[2]**o+(0.283003770703)*x[2]
        s[3,1,1,0]=(-0.862929516639)*x[0]**o+(-0.455798937374)*x[0]+(0.972383754049)*x[1]**o+(0.251968652813)*x[1]+(-0.631965845999)*x[2]**o+(-0.418936218512)*x[2]
        s[3,1,1,1]=(-0.0265533722079)*x[0]**o+(-0.720054910456)*x[0]+(-0.134379948062)*x[1]**o+(-0.535223772178)*x[1]+(-0.711365379454)*x[2]**o+(0.39963284442)*x[2]
        s[3,1,2,0]=(-0.9691936764)*x[0]**o+(-0.220678275504)*x[0]+(0.423830480119)*x[1]**o+(-0.286816976029)*x[1]+(-0.637835943762)*x[2]**o+(0.389581647726)*x[2]
        s[3,1,2,1]=(-0.375225433223)*x[0]**o+(0.339950113552)*x[0]+(0.272089140767)*x[1]**o+(-0.671811329081)*x[1]+(-0.341834415939)*x[2]**o+(-0.974852723909)*x[2]
        s[3,2,0,0]=(-0.13212102165)*x[0]**o+(-0.529435682355)*x[0]+(0.57321409695)*x[1]**o+(0.123050260287)*x[1]+(-0.50608599056)*x[2]**o+(-0.258999018643)*x[2]
        s[3,2,0,1]=(-0.594320198162)*x[0]**o+(0.59617231387)*x[0]+(-0.132055038377)*x[1]**o+(0.0267055057967)*x[1]+(-0.0909023410879)*x[2]**o+(-0.327689716933)*x[2]
        s[3,2,1,0]=(0.352248348539)*x[0]**o+(-0.937172168074)*x[0]+(0.492884278868)*x[1]**o+(0.599164015797)*x[1]+(-0.306391881602)*x[2]**o+(-0.124914738757)*x[2]
        s[3,2,1,1]=(-0.124766143355)*x[0]**o+(-0.201629423485)*x[0]+(0.69199028272)*x[1]**o+(0.0909253140313)*x[1]+(-0.0807463990698)*x[2]**o+(0.274206212499)*x[2]
        s[3,2,2,0]=(-0.939257892238)*x[0]**o+(-0.0905023289297)*x[0]+(-0.101189471915)*x[1]**o+(-0.850452366024)*x[1]+(0.706374760942)*x[2]**o+(-0.0741989595692)*x[2]
        s[3,2,2,1]=(-0.956776001745)*x[0]**o+(0.313587985523)*x[0]+(0.501857466469)*x[1]**o+(0.866521730616)*x[1]+(-0.113777691526)*x[2]**o+(-0.43724474427)*x[2]
        s[3,3,0,0]=(-0.794240122389)*x[0]**o+(0.117246192404)*x[0]+(0.730901073854)*x[1]**o+(0.406753741761)*x[1]+(-0.987815418361)*x[2]**o+(0.543031203217)*x[2]
        s[3,3,0,1]=(-0.539899807208)*x[0]**o+(0.425096558373)*x[0]+(-0.261756005984)*x[1]**o+(0.219196945781)*x[1]+(-0.458238548056)*x[2]**o+(-0.396891207522)*x[2]
        s[3,3,1,0]=(-0.512322191935)*x[0]**o+(0.158356684082)*x[0]+(-0.702591537714)*x[1]**o+(0.202771352983)*x[1]+(-0.346772902785)*x[2]**o+(0.260366433089)*x[2]
        s[3,3,1,1]=(0.839858482584)*x[0]**o+(0.132933571004)*x[0]+(0.840682050408)*x[1]**o+(0.490904029834)*x[1]+(-0.298290729461)*x[2]**o+(-0.467229890802)*x[2]
        s[3,3,2,0]=(-0.691489850326)*x[0]**o+(0.852014173253)*x[0]+(0.484568640775)*x[1]**o+(0.0788611533234)*x[1]+(0.335927774531)*x[2]**o+(-0.507648906088)*x[2]
        s[3,3,2,1]=(0.140576758316)*x[0]**o+(-0.783440190349)*x[0]+(-0.408503163319)*x[1]**o+(0.0871935010894)*x[1]+(0.478151373206)*x[2]**o+(0.837020241411)*x[2]
        s[3,4,0,0]=(-0.956822615698)*x[0]**o+(0.594397877716)*x[0]+(0.844357571956)*x[1]**o+(0.173315874424)*x[1]+(0.96181728365)*x[2]**o+(-0.900445532048)*x[2]
        s[3,4,0,1]=(0.887824966747)*x[0]**o+(-0.579258071661)*x[0]+(0.983311456722)*x[1]**o+(0.996052998934)*x[1]+(0.215867343112)*x[2]**o+(0.614855622355)*x[2]
        s[3,4,1,0]=(0.482666172028)*x[0]**o+(0.0258626246719)*x[0]+(-0.394140399663)*x[1]**o+(0.192974809281)*x[1]+(-0.265879383852)*x[2]**o+(-0.0409441022179)*x[2]
        s[3,4,1,1]=(0.0600752930856)*x[0]**o+(-0.885644457634)*x[0]+(-0.54869761625)*x[1]**o+(0.4745411417)*x[1]+(-0.554764519536)*x[2]**o+(-0.159514159559)*x[2]
        s[3,4,2,0]=(-0.804675220841)*x[0]**o+(0.586839632342)*x[0]+(-0.329997060623)*x[1]**o+(0.242316008423)*x[1]+(0.272064229179)*x[2]**o+(0.24348627845)*x[2]
        s[3,4,2,1]=(0.847664079563)*x[0]**o+(0.927723091901)*x[0]+(-0.861048553467)*x[1]**o+(-0.413216499555)*x[1]+(0.546938488463)*x[2]**o+(0.0558689143145)*x[2]
        ref[0,0,0,0]=(-0.0362500650077)*x_ref[0]**o+(0.692407811012)*x_ref[0]+(-0.855107417704)*x_ref[1]**o+(-0.0397854200762)*x_ref[1]+(0.488177999316)*x_ref[2]**o+(0.946478266036)*x_ref[2]
        ref[0,0,0,1]=(-0.533284434038)*x_ref[0]**o+(-0.599290705017)*x_ref[0]+(0.198922961617)*x_ref[1]**o+(0.652839426276)*x_ref[1]+(0.560109621832)*x_ref[2]**o+(0.167310997616)*x_ref[2]
        ref[0,0,1,0]=(-0.21016675446)*x_ref[0]**o+(0.812498932794)*x_ref[0]+(-0.611566184558)*x_ref[1]**o+(0.626231122402)*x_ref[1]+(0.536374628942)*x_ref[2]**o+(0.8520547245)*x_ref[2]
        ref[0,0,1,1]=(0.26308571414)*x_ref[0]**o+(-0.359237396089)*x_ref[0]+(0.146879292748)*x_ref[1]**o+(-0.241831631389)*x_ref[1]+(0.812820522163)*x_ref[2]**o+(0.824191692943)*x_ref[2]
        ref[0,0,2,0]=(-0.585682391332)*x_ref[0]**o+(0.82273803111)*x_ref[0]+(-0.715688315155)*x_ref[1]**o+(-0.997893564593)*x_ref[1]+(0.39588322481)*x_ref[2]**o+(0.410447772982)*x_ref[2]
        ref[0,0,2,1]=(0.491817354047)*x_ref[0]**o+(0.105289616977)*x_ref[0]+(0.0581932965006)*x_ref[1]**o+(-0.584153248946)*x_ref[1]+(-0.837305609444)*x_ref[2]**o+(-0.140188544108)*x_ref[2]
        ref[0,1,0,0]=(-0.158174387659)*x_ref[0]**o+(0.774512978962)*x_ref[0]+(0.953772629099)*x_ref[1]**o+(-0.0474582135671)*x_ref[1]+(-0.691917863221)*x_ref[2]**o+(0.978550525536)*x_ref[2]
        ref[0,1,0,1]=(-0.696002534433)*x_ref[0]**o+(-0.904833590237)*x_ref[0]+(-0.991879336542)*x_ref[1]**o+(-0.00301423741703)*x_ref[1]+(-0.495919957691)*x_ref[2]**o+(0.923872653662)*x_ref[2]
        ref[0,1,1,0]=(-0.617187346815)*x_ref[0]**o+(-0.0283413268672)*x_ref[0]+(0.439716253425)*x_ref[1]**o+(0.679148169861)*x_ref[1]+(-0.5494839537)*x_ref[2]**o+(0.780531698733)*x_ref[2]
        ref[0,1,1,1]=(0.0763086894431)*x_ref[0]**o+(0.149024988807)*x_ref[0]+(0.660061724201)*x_ref[1]**o+(0.227260856773)*x_ref[1]+(0.604941557505)*x_ref[2]**o+(-0.32106743372)*x_ref[2]
        ref[0,1,2,0]=(-0.710651974651)*x_ref[0]**o+(0.729233023268)*x_ref[0]+(0.830588938931)*x_ref[1]**o+(0.930100931807)*x_ref[1]+(0.576259601615)*x_ref[2]**o+(0.995388406293)*x_ref[2]
        ref[0,1,2,1]=(0.635415205147)*x_ref[0]**o+(-0.0475242382825)*x_ref[0]+(0.779184290843)*x_ref[1]**o+(0.161316071106)*x_ref[1]+(-0.321426515539)*x_ref[2]**o+(-0.188018565147)*x_ref[2]
        ref[0,2,0,0]=(-0.234573769896)*x_ref[0]**o+(-0.35461213258)*x_ref[0]+(0.322664245412)*x_ref[1]**o+(-0.36584459555)*x_ref[1]+(0.656560085187)*x_ref[2]**o+(-0.380544830082)*x_ref[2]
        ref[0,2,0,1]=(0.314171153213)*x_ref[0]**o+(-0.871055875712)*x_ref[0]+(0.600399452124)*x_ref[1]**o+(-0.710939791885)*x_ref[1]+(-0.189029597187)*x_ref[2]**o+(-0.29466779727)*x_ref[2]
        ref[0,2,1,0]=(0.911500533914)*x_ref[0]**o+(-0.217077039937)*x_ref[0]+(0.488325722425)*x_ref[1]**o+(0.781071969291)*x_ref[1]+(-0.883567213205)*x_ref[2]**o+(0.938457876454)*x_ref[2]
        ref[0,2,1,1]=(0.947554813988)*x_ref[0]**o+(0.108161094029)*x_ref[0]+(0.0814722282926)*x_ref[1]**o+(0.394738648377)*x_ref[1]+(-0.585604351258)*x_ref[2]**o+(0.424262882559)*x_ref[2]
        ref[0,2,2,0]=(0.130793077586)*x_ref[0]**o+(-0.0759869806951)*x_ref[0]+(0.524572237994)*x_ref[1]**o+(-0.0494098932334)*x_ref[1]+(-0.162573831498)*x_ref[2]**o+(0.950665975392)*x_ref[2]
        ref[0,2,2,1]=(0.0636110845427)*x_ref[0]**o+(-0.152586932248)*x_ref[0]+(-0.428472289058)*x_ref[1]**o+(0.943093556156)*x_ref[1]+(-0.925181631256)*x_ref[2]**o+(-0.850747416138)*x_ref[2]
        ref[0,3,0,0]=(0.803654693645)*x_ref[0]**o+(-0.852793510169)*x_ref[0]+(0.372325502382)*x_ref[1]**o+(0.437191610865)*x_ref[1]+(-0.626819779481)*x_ref[2]**o+(-0.969085192013)*x_ref[2]
        ref[0,3,0,1]=(-0.498449247588)*x_ref[0]**o+(-0.174254842954)*x_ref[0]+(-0.751570511608)*x_ref[1]**o+(0.0278714978689)*x_ref[1]+(0.732061097781)*x_ref[2]**o+(0.434922200273)*x_ref[2]
        ref[0,3,1,0]=(0.356966448806)*x_ref[0]**o+(-0.754346917896)*x_ref[0]+(-0.200096880735)*x_ref[1]**o+(0.726202986575)*x_ref[1]+(-0.273975127289)*x_ref[2]**o+(0.922054852117)*x_ref[2]
        ref[0,3,1,1]=(0.294033872459)*x_ref[0]**o+(0.435258372604)*x_ref[0]+(-0.606222342027)*x_ref[1]**o+(-0.418222134998)*x_ref[1]+(-0.617263336049)*x_ref[2]**o+(-0.655912764464)*x_ref[2]
        ref[0,3,2,0]=(-0.552805093123)*x_ref[0]**o+(0.814985531695)*x_ref[0]+(0.201701287999)*x_ref[1]**o+(0.449378802496)*x_ref[1]+(0.106877031271)*x_ref[2]**o+(0.972523159602)*x_ref[2]
        ref[0,3,2,1]=(-0.123112956117)*x_ref[0]**o+(-0.9669799128)*x_ref[0]+(-0.365437347923)*x_ref[1]**o+(0.95077682916)*x_ref[1]+(-0.68795960948)*x_ref[2]**o+(0.282323391952)*x_ref[2]
        ref[0,4,0,0]=(0.762949486092)*x_ref[0]**o+(0.275310784251)*x_ref[0]+(0.90824072581)*x_ref[1]**o+(0.267277710972)*x_ref[1]+(-0.387183781105)*x_ref[2]**o+(-0.919714964098)*x_ref[2]
        ref[0,4,0,1]=(-0.219860124563)*x_ref[0]**o+(0.458617314398)*x_ref[0]+(-0.253950962469)*x_ref[1]**o+(-0.213326656667)*x_ref[1]+(-0.914189191308)*x_ref[2]**o+(-0.88808493928)*x_ref[2]
        ref[0,4,1,0]=(0.0111879656405)*x_ref[0]**o+(0.857083093627)*x_ref[0]+(0.397087247889)*x_ref[1]**o+(-0.411624885427)*x_ref[1]+(-0.533535312833)*x_ref[2]**o+(-0.473765290954)*x_ref[2]
        ref[0,4,1,1]=(0.408477302242)*x_ref[0]**o+(-0.336255491206)*x_ref[0]+(0.535004316057)*x_ref[1]**o+(0.646300732798)*x_ref[1]+(-0.683570698066)*x_ref[2]**o+(-0.691723219637)*x_ref[2]
        ref[0,4,2,0]=(0.229517234528)*x_ref[0]**o+(-0.327638251614)*x_ref[0]+(0.752536339629)*x_ref[1]**o+(0.440600000033)*x_ref[1]+(-0.166203457183)*x_ref[2]**o+(-0.632615828058)*x_ref[2]
        ref[0,4,2,1]=(0.128453646267)*x_ref[0]**o+(-0.152501701527)*x_ref[0]+(0.131616948331)*x_ref[1]**o+(-0.560356219781)*x_ref[1]+(0.369923630213)*x_ref[2]**o+(-0.405937225725)*x_ref[2]
        ref[1,0,0,0]=(-0.498802287823)*x_ref[0]**o+(0.974250100105)*x_ref[0]+(-0.752335049339)*x_ref[1]**o+(0.460106742976)*x_ref[1]+(0.878914200699)*x_ref[2]**o+(-0.85963539732)*x_ref[2]
        ref[1,0,0,1]=(0.937345235384)*x_ref[0]**o+(0.0616042350221)*x_ref[0]+(-0.63597655214)*x_ref[1]**o+(0.200363730821)*x_ref[1]+(0.677520498976)*x_ref[2]**o+(0.310367850218)*x_ref[2]
        ref[1,0,1,0]=(-0.328289664191)*x_ref[0]**o+(-0.439002316683)*x_ref[0]+(-0.753405240499)*x_ref[1]**o+(-0.749020227582)*x_ref[1]+(0.200333899797)*x_ref[2]**o+(0.63068525957)*x_ref[2]
        ref[1,0,1,1]=(-0.522381373851)*x_ref[0]**o+(-0.877707833729)*x_ref[0]+(-0.0334841793702)*x_ref[1]**o+(0.00769816693416)*x_ref[1]+(-0.81697893166)*x_ref[2]**o+(-0.47349699259)*x_ref[2]
        ref[1,0,2,0]=(0.0786863277722)*x_ref[0]**o+(-0.47462306827)*x_ref[0]+(-0.850065158625)*x_ref[1]**o+(0.0481804762762)*x_ref[1]+(0.239465148926)*x_ref[2]**o+(0.484166919183)*x_ref[2]
        ref[1,0,2,1]=(-0.0411433282485)*x_ref[0]**o+(-0.851343034379)*x_ref[0]+(-0.518404507699)*x_ref[1]**o+(-0.357342937948)*x_ref[1]+(-0.57398214485)*x_ref[2]**o+(-0.0989103206187)*x_ref[2]
        ref[1,1,0,0]=(-0.060737698278)*x_ref[0]**o+(-0.103483640965)*x_ref[0]+(-0.206004957088)*x_ref[1]**o+(0.556901021479)*x_ref[1]+(0.951054830915)*x_ref[2]**o+(0.668118573076)*x_ref[2]
        ref[1,1,0,1]=(0.0668860744649)*x_ref[0]**o+(0.0490910382588)*x_ref[0]+(0.817265796062)*x_ref[1]**o+(-0.939419683478)*x_ref[1]+(-0.800905110922)*x_ref[2]**o+(-0.546647838589)*x_ref[2]
        ref[1,1,1,0]=(-0.437101637549)*x_ref[0]**o+(0.269260162519)*x_ref[0]+(-0.026802186298)*x_ref[1]**o+(-0.932589891991)*x_ref[1]+(-0.313814820129)*x_ref[2]**o+(-0.436056943196)*x_ref[2]
        ref[1,1,1,1]=(-0.893118860321)*x_ref[0]**o+(-0.837193118481)*x_ref[0]+(-0.58313090725)*x_ref[1]**o+(-0.657017739851)*x_ref[1]+(-0.0158333033033)*x_ref[2]**o+(0.286927848108)*x_ref[2]
        ref[1,1,2,0]=(-0.414914106956)*x_ref[0]**o+(-0.295021056636)*x_ref[0]+(-0.100513024651)*x_ref[1]**o+(0.119321919323)*x_ref[1]+(-0.911626057698)*x_ref[2]**o+(0.0715733685275)*x_ref[2]
        ref[1,1,2,1]=(0.415073119201)*x_ref[0]**o+(-0.79060072691)*x_ref[0]+(-0.280560285152)*x_ref[1]**o+(-0.651271923585)*x_ref[1]+(-0.783523725291)*x_ref[2]**o+(-0.165699899479)*x_ref[2]
        ref[1,2,0,0]=(0.204696440606)*x_ref[0]**o+(0.58070882041)*x_ref[0]+(0.498396642996)*x_ref[1]**o+(0.270570176185)*x_ref[1]+(-0.261604910878)*x_ref[2]**o+(0.788402570665)*x_ref[2]
        ref[1,2,0,1]=(-0.163227090863)*x_ref[0]**o+(0.17193387282)*x_ref[0]+(0.559640507601)*x_ref[1]**o+(0.39180422655)*x_ref[1]+(0.182426715866)*x_ref[2]**o+(0.52544860689)*x_ref[2]
        ref[1,2,1,0]=(-0.0826612289833)*x_ref[0]**o+(-0.244737952715)*x_ref[0]+(0.887738370973)*x_ref[1]**o+(0.593438512127)*x_ref[1]+(0.73128277327)*x_ref[2]**o+(0.229067764431)*x_ref[2]
        ref[1,2,1,1]=(-0.117634040351)*x_ref[0]**o+(-0.490365253501)*x_ref[0]+(0.654067544802)*x_ref[1]**o+(-0.862866217687)*x_ref[1]+(-0.563450922747)*x_ref[2]**o+(-0.785940469253)*x_ref[2]
        ref[1,2,2,0]=(0.486381246904)*x_ref[0]**o+(-0.119137386359)*x_ref[0]+(0.251388315083)*x_ref[1]**o+(-0.142435376157)*x_ref[1]+(-0.882418773242)*x_ref[2]**o+(0.509436056026)*x_ref[2]
        ref[1,2,2,1]=(-0.69532427028)*x_ref[0]**o+(-0.587331382396)*x_ref[0]+(0.750092712907)*x_ref[1]**o+(0.864188183197)*x_ref[1]+(-0.0336735158745)*x_ref[2]**o+(-0.118692941484)*x_ref[2]
        ref[1,3,0,0]=(0.287194598069)*x_ref[0]**o+(0.775480786321)*x_ref[0]+(0.24820537283)*x_ref[1]**o+(0.579330345046)*x_ref[1]+(0.641058615756)*x_ref[2]**o+(-0.358716444584)*x_ref[2]
        ref[1,3,0,1]=(-0.218876040408)*x_ref[0]**o+(-0.581134176647)*x_ref[0]+(0.47937196949)*x_ref[1]**o+(0.0529063680602)*x_ref[1]+(-0.42659429504)*x_ref[2]**o+(-0.445845798304)*x_ref[2]
        ref[1,3,1,0]=(0.500331134532)*x_ref[0]**o+(0.764920932873)*x_ref[0]+(0.256972231189)*x_ref[1]**o+(0.279967260143)*x_ref[1]+(0.90143027778)*x_ref[2]**o+(-0.753075337423)*x_ref[2]
        ref[1,3,1,1]=(0.0478484319999)*x_ref[0]**o+(0.239476880568)*x_ref[0]+(-0.286765457909)*x_ref[1]**o+(-0.952926972714)*x_ref[1]+(-0.117698931152)*x_ref[2]**o+(-0.950120391675)*x_ref[2]
        ref[1,3,2,0]=(-0.404765291554)*x_ref[0]**o+(0.0796360913293)*x_ref[0]+(0.743564239837)*x_ref[1]**o+(-0.0431456147708)*x_ref[1]+(-0.82536795966)*x_ref[2]**o+(-0.770711063789)*x_ref[2]
        ref[1,3,2,1]=(-0.6113113129)*x_ref[0]**o+(0.330568908804)*x_ref[0]+(-0.391402722722)*x_ref[1]**o+(0.415093002807)*x_ref[1]+(-0.893617348746)*x_ref[2]**o+(0.520968946942)*x_ref[2]
        ref[1,4,0,0]=(-0.209574438366)*x_ref[0]**o+(0.644246054163)*x_ref[0]+(-0.0664046919847)*x_ref[1]**o+(0.214820911697)*x_ref[1]+(-0.10545535704)*x_ref[2]**o+(0.655063184877)*x_ref[2]
        ref[1,4,0,1]=(-0.38446029244)*x_ref[0]**o+(-0.802691253671)*x_ref[0]+(-0.306211901233)*x_ref[1]**o+(-0.755184562233)*x_ref[1]+(-0.704583961281)*x_ref[2]**o+(0.0410695954026)*x_ref[2]
        ref[1,4,1,0]=(-0.512877649374)*x_ref[0]**o+(-0.0190184293267)*x_ref[0]+(0.588904418509)*x_ref[1]**o+(-0.343320898588)*x_ref[1]+(-0.16543066117)*x_ref[2]**o+(-0.118363009949)*x_ref[2]
        ref[1,4,1,1]=(-0.410758135711)*x_ref[0]**o+(0.302846037727)*x_ref[0]+(0.674701250578)*x_ref[1]**o+(-0.96295102282)*x_ref[1]+(0.787271275616)*x_ref[2]**o+(0.925818905258)*x_ref[2]
        ref[1,4,2,0]=(0.629642717166)*x_ref[0]**o+(-0.696019558128)*x_ref[0]+(0.465609587281)*x_ref[1]**o+(0.191961667012)*x_ref[1]+(-0.0182853530912)*x_ref[2]**o+(0.504352699288)*x_ref[2]
        ref[1,4,2,1]=(-0.561295424723)*x_ref[0]**o+(0.830136434947)*x_ref[0]+(0.981595445935)*x_ref[1]**o+(-0.0881426921172)*x_ref[1]+(-0.581790941676)*x_ref[2]**o+(-0.489101727188)*x_ref[2]
        ref[2,0,0,0]=(0.173249525275)*x_ref[0]**o+(0.376871332154)*x_ref[0]+(-0.379974081425)*x_ref[1]**o+(-0.118151578515)*x_ref[1]+(0.371715379047)*x_ref[2]**o+(0.115273894116)*x_ref[2]
        ref[2,0,0,1]=(0.656339759814)*x_ref[0]**o+(-0.0783529441613)*x_ref[0]+(0.988608900721)*x_ref[1]**o+(-0.498370704737)*x_ref[1]+(0.886731679317)*x_ref[2]**o+(0.251026122418)*x_ref[2]
        ref[2,0,1,0]=(-0.350592561402)*x_ref[0]**o+(-0.592294993708)*x_ref[0]+(0.860912193414)*x_ref[1]**o+(0.678591565883)*x_ref[1]+(-0.823185915223)*x_ref[2]**o+(-0.882500202366)*x_ref[2]
        ref[2,0,1,1]=(0.84096537128)*x_ref[0]**o+(-0.78582850022)*x_ref[0]+(-0.872411932976)*x_ref[1]**o+(0.208867262377)*x_ref[1]+(0.795227125238)*x_ref[2]**o+(0.434019201479)*x_ref[2]
        ref[2,0,2,0]=(0.670038316075)*x_ref[0]**o+(0.221065948244)*x_ref[0]+(-0.0801945207329)*x_ref[1]**o+(0.35963157451)*x_ref[1]+(-0.523905944746)*x_ref[2]**o+(0.113327871154)*x_ref[2]
        ref[2,0,2,1]=(-0.790165918677)*x_ref[0]**o+(0.826330759592)*x_ref[0]+(0.816532564314)*x_ref[1]**o+(-0.947088135238)*x_ref[1]+(0.498519891556)*x_ref[2]**o+(-0.221796346503)*x_ref[2]
        ref[2,1,0,0]=(0.679814393957)*x_ref[0]**o+(-0.287910028702)*x_ref[0]+(-0.179717372152)*x_ref[1]**o+(-0.895091171351)*x_ref[1]+(0.453459163192)*x_ref[2]**o+(-0.732993783456)*x_ref[2]
        ref[2,1,0,1]=(-0.917014070487)*x_ref[0]**o+(-0.28427666054)*x_ref[0]+(0.520172057178)*x_ref[1]**o+(0.973381422092)*x_ref[1]+(-0.785604082848)*x_ref[2]**o+(-0.280246362235)*x_ref[2]
        ref[2,1,1,0]=(-0.975543038311)*x_ref[0]**o+(0.0471394295003)*x_ref[0]+(-0.744014701)*x_ref[1]**o+(-0.309281686644)*x_ref[1]+(0.970209409582)*x_ref[2]**o+(0.182638690597)*x_ref[2]
        ref[2,1,1,1]=(0.754879118472)*x_ref[0]**o+(-0.345631769168)*x_ref[0]+(0.864272677704)*x_ref[1]**o+(-0.634026748154)*x_ref[1]+(-0.25656646391)*x_ref[2]**o+(-0.976807129917)*x_ref[2]
        ref[2,1,2,0]=(0.0378913818133)*x_ref[0]**o+(-0.589418434757)*x_ref[0]+(0.500291612315)*x_ref[1]**o+(0.831173369231)*x_ref[1]+(0.0773865261454)*x_ref[2]**o+(-0.170449265388)*x_ref[2]
        ref[2,1,2,1]=(0.363668102897)*x_ref[0]**o+(0.0152347755192)*x_ref[0]+(0.295985339318)*x_ref[1]**o+(-0.695348667793)*x_ref[1]+(0.654035675777)*x_ref[2]**o+(-0.998480773881)*x_ref[2]
        ref[2,2,0,0]=(-0.248968924843)*x_ref[0]**o+(-0.454040417829)*x_ref[0]+(0.618989286259)*x_ref[1]**o+(-0.284627664137)*x_ref[1]+(0.597922789444)*x_ref[2]**o+(-0.0544707719136)*x_ref[2]
        ref[2,2,0,1]=(-0.564540493714)*x_ref[0]**o+(0.948462945081)*x_ref[0]+(-0.554305925084)*x_ref[1]**o+(-0.683216336448)*x_ref[1]+(-0.393720863511)*x_ref[2]**o+(-0.454706611434)*x_ref[2]
        ref[2,2,1,0]=(0.00267926298461)*x_ref[0]**o+(0.69057345924)*x_ref[0]+(0.972990086053)*x_ref[1]**o+(0.334585200812)*x_ref[1]+(0.458913228071)*x_ref[2]**o+(-0.927443691914)*x_ref[2]
        ref[2,2,1,1]=(-0.869817596024)*x_ref[0]**o+(0.183295661514)*x_ref[0]+(-0.215314084509)*x_ref[1]**o+(0.274879624616)*x_ref[1]+(-0.872459015851)*x_ref[2]**o+(-0.43371203901)*x_ref[2]
        ref[2,2,2,0]=(-0.59916469262)*x_ref[0]**o+(-0.635232312962)*x_ref[0]+(-0.730547369574)*x_ref[1]**o+(-0.199910106304)*x_ref[1]+(-0.81824698793)*x_ref[2]**o+(-0.351742356136)*x_ref[2]
        ref[2,2,2,1]=(0.52115654452)*x_ref[0]**o+(-0.648726400371)*x_ref[0]+(0.0705029035387)*x_ref[1]**o+(0.82069581623)*x_ref[1]+(0.262897099948)*x_ref[2]**o+(0.302543200334)*x_ref[2]
        ref[2,3,0,0]=(-0.887996543825)*x_ref[0]**o+(-0.141917392835)*x_ref[0]+(0.458379292046)*x_ref[1]**o+(0.279749722412)*x_ref[1]+(-0.185582606045)*x_ref[2]**o+(-0.408626717355)*x_ref[2]
        ref[2,3,0,1]=(-0.0708646458785)*x_ref[0]**o+(-0.723087450876)*x_ref[0]+(0.801053779885)*x_ref[1]**o+(-0.316692437663)*x_ref[1]+(0.313620016387)*x_ref[2]**o+(-0.802237492322)*x_ref[2]
        ref[2,3,1,0]=(-0.413969268245)*x_ref[0]**o+(-0.419189503807)*x_ref[0]+(-0.150655589102)*x_ref[1]**o+(-0.451064114786)*x_ref[1]+(-0.11752330372)*x_ref[2]**o+(-0.564979364641)*x_ref[2]
        ref[2,3,1,1]=(-0.127258808057)*x_ref[0]**o+(-0.166240866424)*x_ref[0]+(0.482435262134)*x_ref[1]**o+(0.210083811762)*x_ref[1]+(0.496523704735)*x_ref[2]**o+(-0.528305427062)*x_ref[2]
        ref[2,3,2,0]=(-0.619368230074)*x_ref[0]**o+(0.196929842772)*x_ref[0]+(0.45226305027)*x_ref[1]**o+(-0.949509654146)*x_ref[1]+(-0.200199092979)*x_ref[2]**o+(0.879368741494)*x_ref[2]
        ref[2,3,2,1]=(0.727974979035)*x_ref[0]**o+(-0.486118068882)*x_ref[0]+(0.330990540492)*x_ref[1]**o+(0.627547710515)*x_ref[1]+(-0.00312667743686)*x_ref[2]**o+(0.34180677866)*x_ref[2]
        ref[2,4,0,0]=(-0.849340738634)*x_ref[0]**o+(0.105575131464)*x_ref[0]+(0.531365100651)*x_ref[1]**o+(-0.885829562648)*x_ref[1]+(-0.959105112582)*x_ref[2]**o+(0.6210710685)*x_ref[2]
        ref[2,4,0,1]=(-0.525287919972)*x_ref[0]**o+(0.685382709266)*x_ref[0]+(0.706077262452)*x_ref[1]**o+(-0.577859356102)*x_ref[1]+(0.494281341217)*x_ref[2]**o+(0.936337201867)*x_ref[2]
        ref[2,4,1,0]=(-0.0263664228096)*x_ref[0]**o+(0.609335710155)*x_ref[0]+(-0.417542795725)*x_ref[1]**o+(0.127890280292)*x_ref[1]+(0.0530446261715)*x_ref[2]**o+(-0.203436845419)*x_ref[2]
        ref[2,4,1,1]=(0.58910765961)*x_ref[0]**o+(-0.233711911277)*x_ref[0]+(-0.68506108197)*x_ref[1]**o+(-0.796608219305)*x_ref[1]+(-0.55051026522)*x_ref[2]**o+(-0.663284299983)*x_ref[2]
        ref[2,4,2,0]=(0.407867058666)*x_ref[0]**o+(-0.155472572874)*x_ref[0]+(0.0574795179339)*x_ref[1]**o+(0.534562635965)*x_ref[1]+(-0.35303239373)*x_ref[2]**o+(0.690463242713)*x_ref[2]
        ref[2,4,2,1]=(0.458514172477)*x_ref[0]**o+(0.493538792362)*x_ref[0]+(-0.534087073406)*x_ref[1]**o+(0.140190942794)*x_ref[1]+(0.53904059561)*x_ref[2]**o+(0.511924245166)*x_ref[2]
        ref[3,0,0,0]=(0.80688664483)*x_ref[0]**o+(0.641990457435)*x_ref[0]+(0.258439350653)*x_ref[1]**o+(0.360949512497)*x_ref[1]+(0.71055262439)*x_ref[2]**o+(-0.14812833383)*x_ref[2]
        ref[3,0,0,1]=(0.349076196526)*x_ref[0]**o+(-0.250426445203)*x_ref[0]+(0.363350736371)*x_ref[1]**o+(0.62331683755)*x_ref[1]+(-0.799481730613)*x_ref[2]**o+(0.86946545402)*x_ref[2]
        ref[3,0,1,0]=(-0.461900551752)*x_ref[0]**o+(-0.530275233976)*x_ref[0]+(0.778112655963)*x_ref[1]**o+(-0.548160947996)*x_ref[1]+(0.27643541045)*x_ref[2]**o+(0.558519627971)*x_ref[2]
        ref[3,0,1,1]=(-0.870845434837)*x_ref[0]**o+(0.90473098031)*x_ref[0]+(0.356324338565)*x_ref[1]**o+(-0.173579775601)*x_ref[1]+(0.866960109024)*x_ref[2]**o+(0.22810880895)*x_ref[2]
        ref[3,0,2,0]=(0.449562471445)*x_ref[0]**o+(-0.133977676845)*x_ref[0]+(-0.945220890263)*x_ref[1]**o+(0.999013496171)*x_ref[1]+(-0.612837266133)*x_ref[2]**o+(-0.695797874487)*x_ref[2]
        ref[3,0,2,1]=(-0.588105630499)*x_ref[0]**o+(0.176592342414)*x_ref[0]+(-0.820991814796)*x_ref[1]**o+(0.0010811086852)*x_ref[1]+(0.498174763907)*x_ref[2]**o+(-0.0223969132485)*x_ref[2]
        ref[3,1,0,0]=(0.653483773188)*x_ref[0]**o+(0.460215104331)*x_ref[0]+(-0.287173677147)*x_ref[1]**o+(-0.837222104854)*x_ref[1]+(0.457524748826)*x_ref[2]**o+(-0.638268772518)*x_ref[2]
        ref[3,1,0,1]=(-0.0999951386915)*x_ref[0]**o+(0.294288175142)*x_ref[0]+(0.159178572562)*x_ref[1]**o+(0.13437803751)*x_ref[1]+(-0.411939992038)*x_ref[2]**o+(0.283003770703)*x_ref[2]
        ref[3,1,1,0]=(-0.862929516639)*x_ref[0]**o+(-0.455798937374)*x_ref[0]+(0.972383754049)*x_ref[1]**o+(0.251968652813)*x_ref[1]+(-0.631965845999)*x_ref[2]**o+(-0.418936218512)*x_ref[2]
        ref[3,1,1,1]=(-0.0265533722079)*x_ref[0]**o+(-0.720054910456)*x_ref[0]+(-0.134379948062)*x_ref[1]**o+(-0.535223772178)*x_ref[1]+(-0.711365379454)*x_ref[2]**o+(0.39963284442)*x_ref[2]
        ref[3,1,2,0]=(-0.9691936764)*x_ref[0]**o+(-0.220678275504)*x_ref[0]+(0.423830480119)*x_ref[1]**o+(-0.286816976029)*x_ref[1]+(-0.637835943762)*x_ref[2]**o+(0.389581647726)*x_ref[2]
        ref[3,1,2,1]=(-0.375225433223)*x_ref[0]**o+(0.339950113552)*x_ref[0]+(0.272089140767)*x_ref[1]**o+(-0.671811329081)*x_ref[1]+(-0.341834415939)*x_ref[2]**o+(-0.974852723909)*x_ref[2]
        ref[3,2,0,0]=(-0.13212102165)*x_ref[0]**o+(-0.529435682355)*x_ref[0]+(0.57321409695)*x_ref[1]**o+(0.123050260287)*x_ref[1]+(-0.50608599056)*x_ref[2]**o+(-0.258999018643)*x_ref[2]
        ref[3,2,0,1]=(-0.594320198162)*x_ref[0]**o+(0.59617231387)*x_ref[0]+(-0.132055038377)*x_ref[1]**o+(0.0267055057967)*x_ref[1]+(-0.0909023410879)*x_ref[2]**o+(-0.327689716933)*x_ref[2]
        ref[3,2,1,0]=(0.352248348539)*x_ref[0]**o+(-0.937172168074)*x_ref[0]+(0.492884278868)*x_ref[1]**o+(0.599164015797)*x_ref[1]+(-0.306391881602)*x_ref[2]**o+(-0.124914738757)*x_ref[2]
        ref[3,2,1,1]=(-0.124766143355)*x_ref[0]**o+(-0.201629423485)*x_ref[0]+(0.69199028272)*x_ref[1]**o+(0.0909253140313)*x_ref[1]+(-0.0807463990698)*x_ref[2]**o+(0.274206212499)*x_ref[2]
        ref[3,2,2,0]=(-0.939257892238)*x_ref[0]**o+(-0.0905023289297)*x_ref[0]+(-0.101189471915)*x_ref[1]**o+(-0.850452366024)*x_ref[1]+(0.706374760942)*x_ref[2]**o+(-0.0741989595692)*x_ref[2]
        ref[3,2,2,1]=(-0.956776001745)*x_ref[0]**o+(0.313587985523)*x_ref[0]+(0.501857466469)*x_ref[1]**o+(0.866521730616)*x_ref[1]+(-0.113777691526)*x_ref[2]**o+(-0.43724474427)*x_ref[2]
        ref[3,3,0,0]=(-0.794240122389)*x_ref[0]**o+(0.117246192404)*x_ref[0]+(0.730901073854)*x_ref[1]**o+(0.406753741761)*x_ref[1]+(-0.987815418361)*x_ref[2]**o+(0.543031203217)*x_ref[2]
        ref[3,3,0,1]=(-0.539899807208)*x_ref[0]**o+(0.425096558373)*x_ref[0]+(-0.261756005984)*x_ref[1]**o+(0.219196945781)*x_ref[1]+(-0.458238548056)*x_ref[2]**o+(-0.396891207522)*x_ref[2]
        ref[3,3,1,0]=(-0.512322191935)*x_ref[0]**o+(0.158356684082)*x_ref[0]+(-0.702591537714)*x_ref[1]**o+(0.202771352983)*x_ref[1]+(-0.346772902785)*x_ref[2]**o+(0.260366433089)*x_ref[2]
        ref[3,3,1,1]=(0.839858482584)*x_ref[0]**o+(0.132933571004)*x_ref[0]+(0.840682050408)*x_ref[1]**o+(0.490904029834)*x_ref[1]+(-0.298290729461)*x_ref[2]**o+(-0.467229890802)*x_ref[2]
        ref[3,3,2,0]=(-0.691489850326)*x_ref[0]**o+(0.852014173253)*x_ref[0]+(0.484568640775)*x_ref[1]**o+(0.0788611533234)*x_ref[1]+(0.335927774531)*x_ref[2]**o+(-0.507648906088)*x_ref[2]
        ref[3,3,2,1]=(0.140576758316)*x_ref[0]**o+(-0.783440190349)*x_ref[0]+(-0.408503163319)*x_ref[1]**o+(0.0871935010894)*x_ref[1]+(0.478151373206)*x_ref[2]**o+(0.837020241411)*x_ref[2]
        ref[3,4,0,0]=(-0.956822615698)*x_ref[0]**o+(0.594397877716)*x_ref[0]+(0.844357571956)*x_ref[1]**o+(0.173315874424)*x_ref[1]+(0.96181728365)*x_ref[2]**o+(-0.900445532048)*x_ref[2]
        ref[3,4,0,1]=(0.887824966747)*x_ref[0]**o+(-0.579258071661)*x_ref[0]+(0.983311456722)*x_ref[1]**o+(0.996052998934)*x_ref[1]+(0.215867343112)*x_ref[2]**o+(0.614855622355)*x_ref[2]
        ref[3,4,1,0]=(0.482666172028)*x_ref[0]**o+(0.0258626246719)*x_ref[0]+(-0.394140399663)*x_ref[1]**o+(0.192974809281)*x_ref[1]+(-0.265879383852)*x_ref[2]**o+(-0.0409441022179)*x_ref[2]
        ref[3,4,1,1]=(0.0600752930856)*x_ref[0]**o+(-0.885644457634)*x_ref[0]+(-0.54869761625)*x_ref[1]**o+(0.4745411417)*x_ref[1]+(-0.554764519536)*x_ref[2]**o+(-0.159514159559)*x_ref[2]
        ref[3,4,2,0]=(-0.804675220841)*x_ref[0]**o+(0.586839632342)*x_ref[0]+(-0.329997060623)*x_ref[1]**o+(0.242316008423)*x_ref[1]+(0.272064229179)*x_ref[2]**o+(0.24348627845)*x_ref[2]
        ref[3,4,2,1]=(0.847664079563)*x_ref[0]**o+(0.927723091901)*x_ref[0]+(-0.861048553467)*x_ref[1]**o+(-0.413216499555)*x_ref[1]+(0.546938488463)*x_ref[2]**o+(0.0558689143145)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_ReducedSolution_rank0(self):
      """
      tests interpolate for rank 0 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedSolution onto ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(),dim=dim)
      s=Data(0,(),w)
      ref=Data(0,(),w_ref)
      if dim==2:
        s=(-1.64974193328)*x[0]+(1.68675911612)*x[1]
        ref=(-1.64974193328)*x_ref[0]+(1.68675911612)*x_ref[1]
      else:
        s=(-0.517718083008)*x[0]+(0.479679773182)*x[1]+(0.72459612383)*x[2]
        ref=(-0.517718083008)*x_ref[0]+(0.479679773182)*x_ref[1]+(0.72459612383)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_ReducedSolution_rank1(self):
      """
      tests interpolate for rank 1 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedSolution onto ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(2,),dim=dim)
      s=Data(0,(2,),w)
      ref=Data(0,(2,),w_ref)
      if dim==2:
        s[0]=(-0.415290633997)*x[0]+(0.118775036022)*x[1]
        s[1]=(-0.833749289762)*x[0]+(0.121297221921)*x[1]
        ref[0]=(-0.415290633997)*x[0]+(0.118775036022)*x[1]
        ref[1]=(-0.833749289762)*x[0]+(0.121297221921)*x[1]
      else:
        s[0]=(-0.336174941008)*x[0]+(-0.844918885743)*x[1]+(-0.410649979186)*x[2]
        s[1]=(0.112527237498)*x[0]+(-0.867632388528)*x[1]+(1.42359104479)*x[2]
        ref[0]=(-0.336174941008)*x[0]+(-0.844918885743)*x[1]+(-0.410649979186)*x[2]
        ref[1]=(0.112527237498)*x[0]+(-0.867632388528)*x[1]+(1.42359104479)*x[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_ReducedSolution_rank2(self):
      """
      tests interpolate for rank 2 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedSolution onto ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5),dim=dim)
      s=Data(0,(4, 5),w)
      ref=Data(0,(4, 5),w_ref)
      if dim==2:
        s[0,0]=(-0.833920166667)*x[0]+(-0.105734397941)*x[1]
        s[0,1]=(-0.0535194301789)*x[0]+(1.2869199411)*x[1]
        s[0,2]=(-1.81783608134)*x[0]+(-0.0858549172679)*x[1]
        s[0,3]=(0.133570573696)*x[0]+(0.398281712973)*x[1]
        s[0,4]=(-1.2514060292)*x[0]+(-0.24743946144)*x[1]
        s[1,0]=(0.917760776668)*x[0]+(0.179758313228)*x[1]
        s[1,1]=(0.677772149724)*x[0]+(0.762705763999)*x[1]
        s[1,2]=(-0.505172381789)*x[0]+(0.412365083581)*x[1]
        s[1,3]=(0.0481873177823)*x[0]+(-0.449871653505)*x[1]
        s[1,4]=(-1.43052283016)*x[0]+(0.553426132082)*x[1]
        s[2,0]=(0.337172157552)*x[0]+(1.56154138911)*x[1]
        s[2,1]=(-0.338189138128)*x[0]+(-0.62614063612)*x[1]
        s[2,2]=(-1.51606676783)*x[0]+(-0.13283454533)*x[1]
        s[2,3]=(0.727068106091)*x[0]+(1.00949960235)*x[1]
        s[2,4]=(0.297941246402)*x[0]+(0.952119168452)*x[1]
        s[3,0]=(0.724157315025)*x[0]+(0.962405533793)*x[1]
        s[3,1]=(-0.239168209123)*x[0]+(1.28280794616)*x[1]
        s[3,2]=(-0.0336024791682)*x[0]+(-0.775860593407)*x[1]
        s[3,3]=(-0.118809020482)*x[0]+(0.51890714417)*x[1]
        s[3,4]=(0.374315302545)*x[0]+(-0.258942994)*x[1]
        ref[0,0]=(-0.833920166667)*x_ref[0]+(-0.105734397941)*x_ref[1]
        ref[0,1]=(-0.0535194301789)*x_ref[0]+(1.2869199411)*x_ref[1]
        ref[0,2]=(-1.81783608134)*x_ref[0]+(-0.0858549172679)*x_ref[1]
        ref[0,3]=(0.133570573696)*x_ref[0]+(0.398281712973)*x_ref[1]
        ref[0,4]=(-1.2514060292)*x_ref[0]+(-0.24743946144)*x_ref[1]
        ref[1,0]=(0.917760776668)*x_ref[0]+(0.179758313228)*x_ref[1]
        ref[1,1]=(0.677772149724)*x_ref[0]+(0.762705763999)*x_ref[1]
        ref[1,2]=(-0.505172381789)*x_ref[0]+(0.412365083581)*x_ref[1]
        ref[1,3]=(0.0481873177823)*x_ref[0]+(-0.449871653505)*x_ref[1]
        ref[1,4]=(-1.43052283016)*x_ref[0]+(0.553426132082)*x_ref[1]
        ref[2,0]=(0.337172157552)*x_ref[0]+(1.56154138911)*x_ref[1]
        ref[2,1]=(-0.338189138128)*x_ref[0]+(-0.62614063612)*x_ref[1]
        ref[2,2]=(-1.51606676783)*x_ref[0]+(-0.13283454533)*x_ref[1]
        ref[2,3]=(0.727068106091)*x_ref[0]+(1.00949960235)*x_ref[1]
        ref[2,4]=(0.297941246402)*x_ref[0]+(0.952119168452)*x_ref[1]
        ref[3,0]=(0.724157315025)*x_ref[0]+(0.962405533793)*x_ref[1]
        ref[3,1]=(-0.239168209123)*x_ref[0]+(1.28280794616)*x_ref[1]
        ref[3,2]=(-0.0336024791682)*x_ref[0]+(-0.775860593407)*x_ref[1]
        ref[3,3]=(-0.118809020482)*x_ref[0]+(0.51890714417)*x_ref[1]
        ref[3,4]=(0.374315302545)*x_ref[0]+(-0.258942994)*x_ref[1]
      else:
        s[0,0]=(1.18033303246)*x[0]+(0.59372740835)*x[1]+(-0.684959540977)*x[2]
        s[0,1]=(0.65301033503)*x[0]+(-1.17147722359)*x[1]+(0.455404166733)*x[2]
        s[0,2]=(-0.412048337979)*x[0]+(-1.24933561194)*x[1]+(-0.636287202752)*x[2]
        s[0,3]=(0.122156598551)*x[0]+(0.103952485283)*x[1]+(-0.564471099255)*x[2]
        s[0,4]=(0.737873603859)*x[0]+(0.139615778712)*x[1]+(0.523544473477)*x[2]
        s[1,0]=(0.0198124265514)*x[0]+(-0.335073882197)*x[1]+(-0.805965691618)*x[2]
        s[1,1]=(-1.28485879121)*x[0]+(-0.618847596536)*x[1]+(1.32879899911)*x[2]
        s[1,2]=(1.09319515176)*x[0]+(0.543230471078)*x[1]+(1.56430808151)*x[2]
        s[1,3]=(0.541425658238)*x[0]+(0.0140217060596)*x[1]+(0.603558416867)*x[2]
        s[1,4]=(0.303110106189)*x[0]+(-0.103009794147)*x[1]+(-1.68324105571)*x[2]
        s[2,0]=(-1.65609235169)*x[0]+(-0.290040263496)*x[1]+(0.477073850071)*x[2]
        s[2,1]=(-0.897378807931)*x[0]+(0.191583115578)*x[1]+(0.213226828436)*x[2]
        s[2,2]=(0.523724313062)*x[0]+(0.214413673529)*x[1]+(0.533345615094)*x[2]
        s[2,3]=(-0.994043729754)*x[0]+(-0.307957055273)*x[1]+(0.0422995421427)*x[2]
        s[2,4]=(1.22590973976)*x[0]+(-1.65014223339)*x[1]+(-1.59726996804)*x[2]
        s[3,0]=(-0.0870943824713)*x[0]+(-0.45766718732)*x[1]+(-0.878874753774)*x[2]
        s[3,1]=(0.244470697307)*x[0]+(-0.0429568231364)*x[1]+(-1.57098356615)*x[2]
        s[3,2]=(1.35091699523)*x[0]+(-0.127297590922)*x[1]+(-0.556702799475)*x[2]
        s[3,3]=(0.189117166393)*x[0]+(0.317562328009)*x[1]+(-0.59788285723)*x[2]
        s[3,4]=(-0.155355776878)*x[0]+(-0.616350184089)*x[1]+(0.296083681054)*x[2]
        ref[0,0]=(1.18033303246)*x_ref[0]+(0.59372740835)*x_ref[1]+(-0.684959540977)*x_ref[2]
        ref[0,1]=(0.65301033503)*x_ref[0]+(-1.17147722359)*x_ref[1]+(0.455404166733)*x_ref[2]
        ref[0,2]=(-0.412048337979)*x_ref[0]+(-1.24933561194)*x_ref[1]+(-0.636287202752)*x_ref[2]
        ref[0,3]=(0.122156598551)*x_ref[0]+(0.103952485283)*x_ref[1]+(-0.564471099255)*x_ref[2]
        ref[0,4]=(0.737873603859)*x_ref[0]+(0.139615778712)*x_ref[1]+(0.523544473477)*x_ref[2]
        ref[1,0]=(0.0198124265514)*x_ref[0]+(-0.335073882197)*x_ref[1]+(-0.805965691618)*x_ref[2]
        ref[1,1]=(-1.28485879121)*x_ref[0]+(-0.618847596536)*x_ref[1]+(1.32879899911)*x_ref[2]
        ref[1,2]=(1.09319515176)*x_ref[0]+(0.543230471078)*x_ref[1]+(1.56430808151)*x_ref[2]
        ref[1,3]=(0.541425658238)*x_ref[0]+(0.0140217060596)*x_ref[1]+(0.603558416867)*x_ref[2]
        ref[1,4]=(0.303110106189)*x_ref[0]+(-0.103009794147)*x_ref[1]+(-1.68324105571)*x_ref[2]
        ref[2,0]=(-1.65609235169)*x_ref[0]+(-0.290040263496)*x_ref[1]+(0.477073850071)*x_ref[2]
        ref[2,1]=(-0.897378807931)*x_ref[0]+(0.191583115578)*x_ref[1]+(0.213226828436)*x_ref[2]
        ref[2,2]=(0.523724313062)*x_ref[0]+(0.214413673529)*x_ref[1]+(0.533345615094)*x_ref[2]
        ref[2,3]=(-0.994043729754)*x_ref[0]+(-0.307957055273)*x_ref[1]+(0.0422995421427)*x_ref[2]
        ref[2,4]=(1.22590973976)*x_ref[0]+(-1.65014223339)*x_ref[1]+(-1.59726996804)*x_ref[2]
        ref[3,0]=(-0.0870943824713)*x_ref[0]+(-0.45766718732)*x_ref[1]+(-0.878874753774)*x_ref[2]
        ref[3,1]=(0.244470697307)*x_ref[0]+(-0.0429568231364)*x_ref[1]+(-1.57098356615)*x_ref[2]
        ref[3,2]=(1.35091699523)*x_ref[0]+(-0.127297590922)*x_ref[1]+(-0.556702799475)*x_ref[2]
        ref[3,3]=(0.189117166393)*x_ref[0]+(0.317562328009)*x_ref[1]+(-0.59788285723)*x_ref[2]
        ref[3,4]=(-0.155355776878)*x_ref[0]+(-0.616350184089)*x_ref[1]+(0.296083681054)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_ReducedSolution_rank3(self):
      """
      tests interpolate for rank 3 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedSolution onto ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(6, 2, 2),dim=dim)
      s=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2),w_ref)
      if dim==2:
        s[0,0,0]=(0.0730549738553)*x[0]+(0.585649475666)*x[1]
        s[0,0,1]=(-0.449075310223)*x[0]+(1.39116964942)*x[1]
        s[0,1,0]=(0.0210032238157)*x[0]+(1.33904896697)*x[1]
        s[0,1,1]=(0.556125399236)*x[0]+(-0.00311484949343)*x[1]
        s[1,0,0]=(0.938296971476)*x[0]+(1.13223213262)*x[1]
        s[1,0,1]=(0.744133292867)*x[0]+(0.0271311625871)*x[1]
        s[1,1,0]=(-0.0478922372672)*x[0]+(-1.08882574207)*x[1]
        s[1,1,1]=(0.738328994373)*x[0]+(-0.730449948444)*x[1]
        s[2,0,0]=(0.66593099915)*x[0]+(-0.185918605751)*x[1]
        s[2,0,1]=(0.450972410432)*x[0]+(1.3393675839)*x[1]
        s[2,1,0]=(0.897370528228)*x[0]+(0.765127571621)*x[1]
        s[2,1,1]=(-0.349658725406)*x[0]+(-0.536332219959)*x[1]
        s[3,0,0]=(1.11112898963)*x[0]+(1.50253301112)*x[1]
        s[3,0,1]=(0.238957188525)*x[0]+(0.775368032877)*x[1]
        s[3,1,0]=(0.612991596596)*x[0]+(0.0718037077356)*x[1]
        s[3,1,1]=(-0.40039392925)*x[0]+(-1.58712410632)*x[1]
        s[4,0,0]=(0.23241254342)*x[0]+(-0.547054930927)*x[1]
        s[4,0,1]=(-0.976995206698)*x[0]+(1.251791534)*x[1]
        s[4,1,0]=(-1.13243662763)*x[0]+(0.56739947752)*x[1]
        s[4,1,1]=(-0.543613180121)*x[0]+(0.649805921416)*x[1]
        s[5,0,0]=(-0.0690689383083)*x[0]+(0.818563064557)*x[1]
        s[5,0,1]=(0.362899720121)*x[0]+(-0.112604112729)*x[1]
        s[5,1,0]=(-0.291756903179)*x[0]+(-0.538221294595)*x[1]
        s[5,1,1]=(0.60406331286)*x[0]+(0.228744896658)*x[1]
        ref[0,0,0]=(0.0730549738553)*x_ref[0]+(0.585649475666)*x_ref[1]
        ref[0,0,1]=(-0.449075310223)*x_ref[0]+(1.39116964942)*x_ref[1]
        ref[0,1,0]=(0.0210032238157)*x_ref[0]+(1.33904896697)*x_ref[1]
        ref[0,1,1]=(0.556125399236)*x_ref[0]+(-0.00311484949343)*x_ref[1]
        ref[1,0,0]=(0.938296971476)*x_ref[0]+(1.13223213262)*x_ref[1]
        ref[1,0,1]=(0.744133292867)*x_ref[0]+(0.0271311625871)*x_ref[1]
        ref[1,1,0]=(-0.0478922372672)*x_ref[0]+(-1.08882574207)*x_ref[1]
        ref[1,1,1]=(0.738328994373)*x_ref[0]+(-0.730449948444)*x_ref[1]
        ref[2,0,0]=(0.66593099915)*x_ref[0]+(-0.185918605751)*x_ref[1]
        ref[2,0,1]=(0.450972410432)*x_ref[0]+(1.3393675839)*x_ref[1]
        ref[2,1,0]=(0.897370528228)*x_ref[0]+(0.765127571621)*x_ref[1]
        ref[2,1,1]=(-0.349658725406)*x_ref[0]+(-0.536332219959)*x_ref[1]
        ref[3,0,0]=(1.11112898963)*x_ref[0]+(1.50253301112)*x_ref[1]
        ref[3,0,1]=(0.238957188525)*x_ref[0]+(0.775368032877)*x_ref[1]
        ref[3,1,0]=(0.612991596596)*x_ref[0]+(0.0718037077356)*x_ref[1]
        ref[3,1,1]=(-0.40039392925)*x_ref[0]+(-1.58712410632)*x_ref[1]
        ref[4,0,0]=(0.23241254342)*x_ref[0]+(-0.547054930927)*x_ref[1]
        ref[4,0,1]=(-0.976995206698)*x_ref[0]+(1.251791534)*x_ref[1]
        ref[4,1,0]=(-1.13243662763)*x_ref[0]+(0.56739947752)*x_ref[1]
        ref[4,1,1]=(-0.543613180121)*x_ref[0]+(0.649805921416)*x_ref[1]
        ref[5,0,0]=(-0.0690689383083)*x_ref[0]+(0.818563064557)*x_ref[1]
        ref[5,0,1]=(0.362899720121)*x_ref[0]+(-0.112604112729)*x_ref[1]
        ref[5,1,0]=(-0.291756903179)*x_ref[0]+(-0.538221294595)*x_ref[1]
        ref[5,1,1]=(0.60406331286)*x_ref[0]+(0.228744896658)*x_ref[1]
      else:
        s[0,0,0]=(0.811538724853)*x[0]+(-0.347250364239)*x[1]+(-0.116011094391)*x[2]
        s[0,0,1]=(1.69112205586)*x[0]+(-1.04197300041)*x[1]+(0.849118123462)*x[2]
        s[0,1,0]=(1.30816943778)*x[0]+(1.01636941747)*x[1]+(-0.849746116393)*x[2]
        s[0,1,1]=(0.888683598925)*x[0]+(-1.60930356378)*x[1]+(0.522151570265)*x[2]
        s[1,0,0]=(1.03821316703)*x[0]+(-1.19647941504)*x[1]+(-1.08543246224)*x[2]
        s[1,0,1]=(-0.314182278588)*x[0]+(0.144825414517)*x[1]+(0.162613879231)*x[2]
        s[1,1,0]=(0.0416209427382)*x[0]+(0.0390092175082)*x[1]+(1.55616366687)*x[2]
        s[1,1,1]=(-0.577251716148)*x[0]+(0.717529140134)*x[1]+(-0.567199280535)*x[2]
        s[2,0,0]=(1.26716303575)*x[0]+(0.119576983216)*x[1]+(-0.602181033831)*x[2]
        s[2,0,1]=(0.533935330879)*x[0]+(0.292003869006)*x[1]+(-0.0917835680855)*x[2]
        s[2,1,0]=(-1.50218600857)*x[0]+(1.23418550746)*x[1]+(-0.600240530145)*x[2]
        s[2,1,1]=(0.689151741161)*x[0]+(0.909077564156)*x[1]+(0.924638961344)*x[2]
        s[3,0,0]=(-0.763411711557)*x[0]+(-0.862413521307)*x[1]+(1.49095932554)*x[2]
        s[3,0,1]=(-0.215589144977)*x[0]+(0.305301450314)*x[1]+(-0.468853406603)*x[2]
        s[3,1,0]=(-0.433045929433)*x[0]+(-0.907669929171)*x[1]+(0.917462300039)*x[2]
        s[3,1,1]=(0.229593212783)*x[0]+(-1.79371444035)*x[1]+(-0.848409349534)*x[2]
        s[4,0,0]=(1.40249316312)*x[0]+(-0.766609553683)*x[1]+(0.337911083882)*x[2]
        s[4,0,1]=(0.728650674867)*x[0]+(0.631222570369)*x[1]+(1.2491315131)*x[2]
        s[4,1,0]=(1.03998601102)*x[0]+(-1.81204917532)*x[1]+(0.781499164268)*x[2]
        s[4,1,1]=(-0.151999545303)*x[0]+(-0.767585861167)*x[1]+(1.17261654125)*x[2]
        s[5,0,0]=(0.0632249575346)*x[0]+(0.333565970302)*x[1]+(-1.66679001354)*x[2]
        s[5,0,1]=(0.945677231929)*x[0]+(-1.15936834775)*x[1]+(-0.644891644457)*x[2]
        s[5,1,0]=(1.08939087839)*x[0]+(0.687932839014)*x[1]+(-0.473426758917)*x[2]
        s[5,1,1]=(0.0349915672648)*x[0]+(-1.48593425129)*x[1]+(0.145421756236)*x[2]
        ref[0,0,0]=(0.811538724853)*x_ref[0]+(-0.347250364239)*x_ref[1]+(-0.116011094391)*x_ref[2]
        ref[0,0,1]=(1.69112205586)*x_ref[0]+(-1.04197300041)*x_ref[1]+(0.849118123462)*x_ref[2]
        ref[0,1,0]=(1.30816943778)*x_ref[0]+(1.01636941747)*x_ref[1]+(-0.849746116393)*x_ref[2]
        ref[0,1,1]=(0.888683598925)*x_ref[0]+(-1.60930356378)*x_ref[1]+(0.522151570265)*x_ref[2]
        ref[1,0,0]=(1.03821316703)*x_ref[0]+(-1.19647941504)*x_ref[1]+(-1.08543246224)*x_ref[2]
        ref[1,0,1]=(-0.314182278588)*x_ref[0]+(0.144825414517)*x_ref[1]+(0.162613879231)*x_ref[2]
        ref[1,1,0]=(0.0416209427382)*x_ref[0]+(0.0390092175082)*x_ref[1]+(1.55616366687)*x_ref[2]
        ref[1,1,1]=(-0.577251716148)*x_ref[0]+(0.717529140134)*x_ref[1]+(-0.567199280535)*x_ref[2]
        ref[2,0,0]=(1.26716303575)*x_ref[0]+(0.119576983216)*x_ref[1]+(-0.602181033831)*x_ref[2]
        ref[2,0,1]=(0.533935330879)*x_ref[0]+(0.292003869006)*x_ref[1]+(-0.0917835680855)*x_ref[2]
        ref[2,1,0]=(-1.50218600857)*x_ref[0]+(1.23418550746)*x_ref[1]+(-0.600240530145)*x_ref[2]
        ref[2,1,1]=(0.689151741161)*x_ref[0]+(0.909077564156)*x_ref[1]+(0.924638961344)*x_ref[2]
        ref[3,0,0]=(-0.763411711557)*x_ref[0]+(-0.862413521307)*x_ref[1]+(1.49095932554)*x_ref[2]
        ref[3,0,1]=(-0.215589144977)*x_ref[0]+(0.305301450314)*x_ref[1]+(-0.468853406603)*x_ref[2]
        ref[3,1,0]=(-0.433045929433)*x_ref[0]+(-0.907669929171)*x_ref[1]+(0.917462300039)*x_ref[2]
        ref[3,1,1]=(0.229593212783)*x_ref[0]+(-1.79371444035)*x_ref[1]+(-0.848409349534)*x_ref[2]
        ref[4,0,0]=(1.40249316312)*x_ref[0]+(-0.766609553683)*x_ref[1]+(0.337911083882)*x_ref[2]
        ref[4,0,1]=(0.728650674867)*x_ref[0]+(0.631222570369)*x_ref[1]+(1.2491315131)*x_ref[2]
        ref[4,1,0]=(1.03998601102)*x_ref[0]+(-1.81204917532)*x_ref[1]+(0.781499164268)*x_ref[2]
        ref[4,1,1]=(-0.151999545303)*x_ref[0]+(-0.767585861167)*x_ref[1]+(1.17261654125)*x_ref[2]
        ref[5,0,0]=(0.0632249575346)*x_ref[0]+(0.333565970302)*x_ref[1]+(-1.66679001354)*x_ref[2]
        ref[5,0,1]=(0.945677231929)*x_ref[0]+(-1.15936834775)*x_ref[1]+(-0.644891644457)*x_ref[2]
        ref[5,1,0]=(1.08939087839)*x_ref[0]+(0.687932839014)*x_ref[1]+(-0.473426758917)*x_ref[2]
        ref[5,1,1]=(0.0349915672648)*x_ref[0]+(-1.48593425129)*x_ref[1]+(0.145421756236)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_ReducedSolution_rank4(self):
      """
      tests interpolate for rank 4 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedSolution onto ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5, 3, 2),dim=dim)
      s=Data(0,(4, 5, 3, 2),w)
      ref=Data(0,(4, 5, 3, 2),w_ref)
      if dim==2:
        s[0,0,0,0]=(0.609727033659)*x[0]+(-0.901858940633)*x[1]
        s[0,0,0,1]=(-0.221259461243)*x[0]+(0.351136340908)*x[1]
        s[0,0,1,0]=(0.497173734282)*x[0]+(0.899142906899)*x[1]
        s[0,0,1,1]=(-1.55206649551)*x[0]+(-0.680257384044)*x[1]
        s[0,0,2,0]=(-0.932033638033)*x[0]+(-0.46503758126)*x[1]
        s[0,0,2,1]=(-0.0372650123525)*x[0]+(0.0173714029098)*x[1]
        s[0,1,0,0]=(0.855963098465)*x[0]+(0.760176320898)*x[1]
        s[0,1,0,1]=(1.37546120327)*x[0]+(1.72916158331)*x[1]
        s[0,1,1,0]=(0.25469861613)*x[0]+(0.71445716033)*x[1]
        s[0,1,1,1]=(0.198455659765)*x[0]+(0.0410276829495)*x[1]
        s[0,1,2,0]=(-0.628080427667)*x[0]+(-0.0197206025984)*x[1]
        s[0,1,2,1]=(1.26759029895)*x[0]+(1.15914953253)*x[1]
        s[0,2,0,0]=(-1.43203053441)*x[0]+(0.366301170069)*x[1]
        s[0,2,0,1]=(0.420527802229)*x[0]+(-0.709209747415)*x[1]
        s[0,2,1,0]=(0.399042264829)*x[0]+(0.0453132754731)*x[1]
        s[0,2,1,1]=(1.2074346062)*x[0]+(-1.14163410992)*x[1]
        s[0,2,2,0]=(-0.00270984158569)*x[0]+(-1.63216200182)*x[1]
        s[0,2,2,1]=(0.174276088847)*x[0]+(-1.04789472231)*x[1]
        s[0,3,0,0]=(-0.47704729036)*x[0]+(-1.2550637204)*x[1]
        s[0,3,0,1]=(0.348356873191)*x[0]+(-0.796668578265)*x[1]
        s[0,3,1,0]=(-0.52724725097)*x[0]+(1.26163123592)*x[1]
        s[0,3,1,1]=(-1.52681799261)*x[0]+(0.936041931735)*x[1]
        s[0,3,2,0]=(1.33689673219)*x[0]+(-1.34901214018)*x[1]
        s[0,3,2,1]=(1.57651286047)*x[0]+(0.245413762864)*x[1]
        s[0,4,0,0]=(0.194500181498)*x[0]+(-0.0231951844441)*x[1]
        s[0,4,0,1]=(-1.49372076697)*x[0]+(-0.111369678181)*x[1]
        s[0,4,1,0]=(-0.74121646836)*x[0]+(-0.817740156307)*x[1]
        s[0,4,1,1]=(-0.233643397123)*x[0]+(0.672715068796)*x[1]
        s[0,4,2,0]=(1.05850947579)*x[0]+(0.386484386191)*x[1]
        s[0,4,2,1]=(1.45105922963)*x[0]+(-0.367458625996)*x[1]
        s[1,0,0,0]=(-0.391803788669)*x[0]+(0.797592300662)*x[1]
        s[1,0,0,1]=(-0.205299072883)*x[0]+(0.677427935558)*x[1]
        s[1,0,1,0]=(1.56225329031)*x[0]+(1.17468092515)*x[1]
        s[1,0,1,1]=(1.4586005658)*x[0]+(1.36481062943)*x[1]
        s[1,0,2,0]=(-1.53281881413)*x[0]+(0.207028833974)*x[1]
        s[1,0,2,1]=(0.0469239691648)*x[0]+(1.03547812705)*x[1]
        s[1,1,0,0]=(1.39614335897)*x[0]+(0.82046489695)*x[1]
        s[1,1,0,1]=(-0.775462741637)*x[0]+(-0.275081761646)*x[1]
        s[1,1,1,0]=(0.173353028608)*x[0]+(-1.05620431816)*x[1]
        s[1,1,1,1]=(-0.464192870247)*x[0]+(0.212204784206)*x[1]
        s[1,1,2,0]=(-0.383813531866)*x[0]+(-0.0773759374234)*x[1]
        s[1,1,2,1]=(-1.07811385759)*x[0]+(-0.457232330819)*x[1]
        s[1,2,0,0]=(-0.295869147576)*x[0]+(0.166843915631)*x[1]
        s[1,2,0,1]=(0.571421034325)*x[0]+(1.46722583245)*x[1]
        s[1,2,1,0]=(-0.34494944257)*x[0]+(-0.286867052491)*x[1]
        s[1,2,1,1]=(1.43128885761)*x[0]+(0.21277887035)*x[1]
        s[1,2,2,0]=(-1.19397187873)*x[0]+(0.126397665227)*x[1]
        s[1,2,2,1]=(0.171368751954)*x[0]+(-1.12100254625)*x[1]
        s[1,3,0,0]=(0.0975333077604)*x[0]+(0.583524660188)*x[1]
        s[1,3,0,1]=(0.0271450955458)*x[0]+(0.368978332867)*x[1]
        s[1,3,1,0]=(1.22806968486)*x[0]+(-0.569932748468)*x[1]
        s[1,3,1,1]=(-0.691989839521)*x[0]+(-0.829822182922)*x[1]
        s[1,3,2,0]=(0.743776534098)*x[0]+(1.3668937127)*x[1]
        s[1,3,2,1]=(0.192206575204)*x[0]+(0.96576702824)*x[1]
        s[1,4,0,0]=(0.149381460079)*x[0]+(-0.987875538036)*x[1]
        s[1,4,0,1]=(-0.706855836683)*x[0]+(0.0410884211631)*x[1]
        s[1,4,1,0]=(-0.222650274)*x[0]+(0.59589242048)*x[1]
        s[1,4,1,1]=(0.541489186743)*x[0]+(-1.22543996074)*x[1]
        s[1,4,2,0]=(0.630234824652)*x[0]+(0.508436575179)*x[1]
        s[1,4,2,1]=(0.990062359294)*x[0]+(-0.639884693407)*x[1]
        s[2,0,0,0]=(0.378887461733)*x[0]+(1.71999809344)*x[1]
        s[2,0,0,1]=(0.182412683544)*x[0]+(-0.401183243877)*x[1]
        s[2,0,1,0]=(0.325775290396)*x[0]+(-0.656088786126)*x[1]
        s[2,0,1,1]=(0.316355229742)*x[0]+(1.54087545195)*x[1]
        s[2,0,2,0]=(1.14203881183)*x[0]+(0.703207191364)*x[1]
        s[2,0,2,1]=(-1.05211790226)*x[0]+(1.68596483036)*x[1]
        s[2,1,0,0]=(0.704109387934)*x[0]+(-0.589003921574)*x[1]
        s[2,1,0,1]=(-0.382489922028)*x[0]+(0.387647642942)*x[1]
        s[2,1,1,0]=(-0.0343434958891)*x[0]+(-0.119176404279)*x[1]
        s[2,1,1,1]=(0.567853527929)*x[0]+(1.32523261969)*x[1]
        s[2,1,2,0]=(1.03496000653)*x[0]+(0.110203591459)*x[1]
        s[2,1,2,1]=(0.560149997806)*x[0]+(-1.62893742873)*x[1]
        s[2,2,0,0]=(1.1801792312)*x[0]+(1.1912385935)*x[1]
        s[2,2,0,1]=(0.936932817634)*x[0]+(-1.49082021974)*x[1]
        s[2,2,1,0]=(-0.305582892069)*x[0]+(-0.267235698563)*x[1]
        s[2,2,1,1]=(0.107449730184)*x[0]+(-0.325267993615)*x[1]
        s[2,2,2,0]=(-1.08118986347)*x[0]+(0.144190727334)*x[1]
        s[2,2,2,1]=(1.44755060146)*x[0]+(-0.127612338817)*x[1]
        s[2,3,0,0]=(-0.666943112999)*x[0]+(-0.780402371989)*x[1]
        s[2,3,0,1]=(0.724206717771)*x[0]+(-0.523593908278)*x[1]
        s[2,3,1,0]=(1.86101197199)*x[0]+(-0.710894911302)*x[1]
        s[2,3,1,1]=(-1.74409232023)*x[0]+(0.407821816271)*x[1]
        s[2,3,2,0]=(0.646304172575)*x[0]+(0.0132628778543)*x[1]
        s[2,3,2,1]=(-0.746041438673)*x[0]+(0.516304872741)*x[1]
        s[2,4,0,0]=(0.0326900201283)*x[0]+(-0.533021156401)*x[1]
        s[2,4,0,1]=(0.088598956981)*x[0]+(0.764307970269)*x[1]
        s[2,4,1,0]=(-0.749553477756)*x[0]+(0.379646792939)*x[1]
        s[2,4,1,1]=(-0.243635321378)*x[0]+(-0.470022600261)*x[1]
        s[2,4,2,0]=(0.586398757247)*x[0]+(-0.116982897683)*x[1]
        s[2,4,2,1]=(-0.989012121879)*x[0]+(-1.25006535257)*x[1]
        s[3,0,0,0]=(1.25228306081)*x[0]+(-0.0932717035896)*x[1]
        s[3,0,0,1]=(1.5975516771)*x[0]+(-0.177733286111)*x[1]
        s[3,0,1,0]=(-0.659910070911)*x[0]+(-0.0999498944812)*x[1]
        s[3,0,1,1]=(-0.299870273662)*x[0]+(-1.22977656744)*x[1]
        s[3,0,2,0]=(-0.158211581143)*x[0]+(-0.141421380503)*x[1]
        s[3,0,2,1]=(-1.48226452788)*x[0]+(-0.731264943309)*x[1]
        s[3,1,0,0]=(-1.64589827853)*x[0]+(1.19692573998)*x[1]
        s[3,1,0,1]=(0.361329655357)*x[0]+(0.234310482171)*x[1]
        s[3,1,1,0]=(-1.14077649516)*x[0]+(0.132657084065)*x[1]
        s[3,1,1,1]=(1.41067975663)*x[0]+(0.818243117884)*x[1]
        s[3,1,2,0]=(-1.53137884342)*x[0]+(-0.0127805333093)*x[1]
        s[3,1,2,1]=(0.240455755486)*x[0]+(-1.02361753893)*x[1]
        s[3,2,0,0]=(-0.334803473072)*x[0]+(-0.807452510423)*x[1]
        s[3,2,0,1]=(0.617324536974)*x[0]+(-0.733163471774)*x[1]
        s[3,2,1,0]=(-0.634580050897)*x[0]+(0.362106278168)*x[1]
        s[3,2,1,1]=(-0.21525531818)*x[0]+(0.982902540898)*x[1]
        s[3,2,2,0]=(-0.590682803195)*x[0]+(-1.27537880632)*x[1]
        s[3,2,2,1]=(0.142578313438)*x[0]+(0.212341687291)*x[1]
        s[3,3,0,0]=(-0.758345317216)*x[0]+(-0.971914631359)*x[1]
        s[3,3,0,1]=(0.569463909234)*x[0]+(0.498848917338)*x[1]
        s[3,3,1,0]=(-0.771456575632)*x[0]+(-0.130456072944)*x[1]
        s[3,3,1,1]=(0.811799269245)*x[0]+(-0.829647203396)*x[1]
        s[3,3,2,0]=(-0.814979427693)*x[0]+(-0.0416092796813)*x[1]
        s[3,3,2,1]=(0.933434189127)*x[0]+(0.60794704936)*x[1]
        s[3,4,0,0]=(0.160280566478)*x[0]+(-0.0141616040181)*x[1]
        s[3,4,0,1]=(-1.34835766268)*x[0]+(0.178951225861)*x[1]
        s[3,4,1,0]=(1.67632814658)*x[0]+(1.31316096483)*x[1]
        s[3,4,1,1]=(-0.603403275376)*x[0]+(0.0607753526431)*x[1]
        s[3,4,2,0]=(-0.178440295473)*x[0]+(0.318102694371)*x[1]
        s[3,4,2,1]=(0.781285160749)*x[0]+(0.625651915981)*x[1]
        ref[0,0,0,0]=(0.609727033659)*x_ref[0]+(-0.901858940633)*x_ref[1]
        ref[0,0,0,1]=(-0.221259461243)*x_ref[0]+(0.351136340908)*x_ref[1]
        ref[0,0,1,0]=(0.497173734282)*x_ref[0]+(0.899142906899)*x_ref[1]
        ref[0,0,1,1]=(-1.55206649551)*x_ref[0]+(-0.680257384044)*x_ref[1]
        ref[0,0,2,0]=(-0.932033638033)*x_ref[0]+(-0.46503758126)*x_ref[1]
        ref[0,0,2,1]=(-0.0372650123525)*x_ref[0]+(0.0173714029098)*x_ref[1]
        ref[0,1,0,0]=(0.855963098465)*x_ref[0]+(0.760176320898)*x_ref[1]
        ref[0,1,0,1]=(1.37546120327)*x_ref[0]+(1.72916158331)*x_ref[1]
        ref[0,1,1,0]=(0.25469861613)*x_ref[0]+(0.71445716033)*x_ref[1]
        ref[0,1,1,1]=(0.198455659765)*x_ref[0]+(0.0410276829495)*x_ref[1]
        ref[0,1,2,0]=(-0.628080427667)*x_ref[0]+(-0.0197206025984)*x_ref[1]
        ref[0,1,2,1]=(1.26759029895)*x_ref[0]+(1.15914953253)*x_ref[1]
        ref[0,2,0,0]=(-1.43203053441)*x_ref[0]+(0.366301170069)*x_ref[1]
        ref[0,2,0,1]=(0.420527802229)*x_ref[0]+(-0.709209747415)*x_ref[1]
        ref[0,2,1,0]=(0.399042264829)*x_ref[0]+(0.0453132754731)*x_ref[1]
        ref[0,2,1,1]=(1.2074346062)*x_ref[0]+(-1.14163410992)*x_ref[1]
        ref[0,2,2,0]=(-0.00270984158569)*x_ref[0]+(-1.63216200182)*x_ref[1]
        ref[0,2,2,1]=(0.174276088847)*x_ref[0]+(-1.04789472231)*x_ref[1]
        ref[0,3,0,0]=(-0.47704729036)*x_ref[0]+(-1.2550637204)*x_ref[1]
        ref[0,3,0,1]=(0.348356873191)*x_ref[0]+(-0.796668578265)*x_ref[1]
        ref[0,3,1,0]=(-0.52724725097)*x_ref[0]+(1.26163123592)*x_ref[1]
        ref[0,3,1,1]=(-1.52681799261)*x_ref[0]+(0.936041931735)*x_ref[1]
        ref[0,3,2,0]=(1.33689673219)*x_ref[0]+(-1.34901214018)*x_ref[1]
        ref[0,3,2,1]=(1.57651286047)*x_ref[0]+(0.245413762864)*x_ref[1]
        ref[0,4,0,0]=(0.194500181498)*x_ref[0]+(-0.0231951844441)*x_ref[1]
        ref[0,4,0,1]=(-1.49372076697)*x_ref[0]+(-0.111369678181)*x_ref[1]
        ref[0,4,1,0]=(-0.74121646836)*x_ref[0]+(-0.817740156307)*x_ref[1]
        ref[0,4,1,1]=(-0.233643397123)*x_ref[0]+(0.672715068796)*x_ref[1]
        ref[0,4,2,0]=(1.05850947579)*x_ref[0]+(0.386484386191)*x_ref[1]
        ref[0,4,2,1]=(1.45105922963)*x_ref[0]+(-0.367458625996)*x_ref[1]
        ref[1,0,0,0]=(-0.391803788669)*x_ref[0]+(0.797592300662)*x_ref[1]
        ref[1,0,0,1]=(-0.205299072883)*x_ref[0]+(0.677427935558)*x_ref[1]
        ref[1,0,1,0]=(1.56225329031)*x_ref[0]+(1.17468092515)*x_ref[1]
        ref[1,0,1,1]=(1.4586005658)*x_ref[0]+(1.36481062943)*x_ref[1]
        ref[1,0,2,0]=(-1.53281881413)*x_ref[0]+(0.207028833974)*x_ref[1]
        ref[1,0,2,1]=(0.0469239691648)*x_ref[0]+(1.03547812705)*x_ref[1]
        ref[1,1,0,0]=(1.39614335897)*x_ref[0]+(0.82046489695)*x_ref[1]
        ref[1,1,0,1]=(-0.775462741637)*x_ref[0]+(-0.275081761646)*x_ref[1]
        ref[1,1,1,0]=(0.173353028608)*x_ref[0]+(-1.05620431816)*x_ref[1]
        ref[1,1,1,1]=(-0.464192870247)*x_ref[0]+(0.212204784206)*x_ref[1]
        ref[1,1,2,0]=(-0.383813531866)*x_ref[0]+(-0.0773759374234)*x_ref[1]
        ref[1,1,2,1]=(-1.07811385759)*x_ref[0]+(-0.457232330819)*x_ref[1]
        ref[1,2,0,0]=(-0.295869147576)*x_ref[0]+(0.166843915631)*x_ref[1]
        ref[1,2,0,1]=(0.571421034325)*x_ref[0]+(1.46722583245)*x_ref[1]
        ref[1,2,1,0]=(-0.34494944257)*x_ref[0]+(-0.286867052491)*x_ref[1]
        ref[1,2,1,1]=(1.43128885761)*x_ref[0]+(0.21277887035)*x_ref[1]
        ref[1,2,2,0]=(-1.19397187873)*x_ref[0]+(0.126397665227)*x_ref[1]
        ref[1,2,2,1]=(0.171368751954)*x_ref[0]+(-1.12100254625)*x_ref[1]
        ref[1,3,0,0]=(0.0975333077604)*x_ref[0]+(0.583524660188)*x_ref[1]
        ref[1,3,0,1]=(0.0271450955458)*x_ref[0]+(0.368978332867)*x_ref[1]
        ref[1,3,1,0]=(1.22806968486)*x_ref[0]+(-0.569932748468)*x_ref[1]
        ref[1,3,1,1]=(-0.691989839521)*x_ref[0]+(-0.829822182922)*x_ref[1]
        ref[1,3,2,0]=(0.743776534098)*x_ref[0]+(1.3668937127)*x_ref[1]
        ref[1,3,2,1]=(0.192206575204)*x_ref[0]+(0.96576702824)*x_ref[1]
        ref[1,4,0,0]=(0.149381460079)*x_ref[0]+(-0.987875538036)*x_ref[1]
        ref[1,4,0,1]=(-0.706855836683)*x_ref[0]+(0.0410884211631)*x_ref[1]
        ref[1,4,1,0]=(-0.222650274)*x_ref[0]+(0.59589242048)*x_ref[1]
        ref[1,4,1,1]=(0.541489186743)*x_ref[0]+(-1.22543996074)*x_ref[1]
        ref[1,4,2,0]=(0.630234824652)*x_ref[0]+(0.508436575179)*x_ref[1]
        ref[1,4,2,1]=(0.990062359294)*x_ref[0]+(-0.639884693407)*x_ref[1]
        ref[2,0,0,0]=(0.378887461733)*x_ref[0]+(1.71999809344)*x_ref[1]
        ref[2,0,0,1]=(0.182412683544)*x_ref[0]+(-0.401183243877)*x_ref[1]
        ref[2,0,1,0]=(0.325775290396)*x_ref[0]+(-0.656088786126)*x_ref[1]
        ref[2,0,1,1]=(0.316355229742)*x_ref[0]+(1.54087545195)*x_ref[1]
        ref[2,0,2,0]=(1.14203881183)*x_ref[0]+(0.703207191364)*x_ref[1]
        ref[2,0,2,1]=(-1.05211790226)*x_ref[0]+(1.68596483036)*x_ref[1]
        ref[2,1,0,0]=(0.704109387934)*x_ref[0]+(-0.589003921574)*x_ref[1]
        ref[2,1,0,1]=(-0.382489922028)*x_ref[0]+(0.387647642942)*x_ref[1]
        ref[2,1,1,0]=(-0.0343434958891)*x_ref[0]+(-0.119176404279)*x_ref[1]
        ref[2,1,1,1]=(0.567853527929)*x_ref[0]+(1.32523261969)*x_ref[1]
        ref[2,1,2,0]=(1.03496000653)*x_ref[0]+(0.110203591459)*x_ref[1]
        ref[2,1,2,1]=(0.560149997806)*x_ref[0]+(-1.62893742873)*x_ref[1]
        ref[2,2,0,0]=(1.1801792312)*x_ref[0]+(1.1912385935)*x_ref[1]
        ref[2,2,0,1]=(0.936932817634)*x_ref[0]+(-1.49082021974)*x_ref[1]
        ref[2,2,1,0]=(-0.305582892069)*x_ref[0]+(-0.267235698563)*x_ref[1]
        ref[2,2,1,1]=(0.107449730184)*x_ref[0]+(-0.325267993615)*x_ref[1]
        ref[2,2,2,0]=(-1.08118986347)*x_ref[0]+(0.144190727334)*x_ref[1]
        ref[2,2,2,1]=(1.44755060146)*x_ref[0]+(-0.127612338817)*x_ref[1]
        ref[2,3,0,0]=(-0.666943112999)*x_ref[0]+(-0.780402371989)*x_ref[1]
        ref[2,3,0,1]=(0.724206717771)*x_ref[0]+(-0.523593908278)*x_ref[1]
        ref[2,3,1,0]=(1.86101197199)*x_ref[0]+(-0.710894911302)*x_ref[1]
        ref[2,3,1,1]=(-1.74409232023)*x_ref[0]+(0.407821816271)*x_ref[1]
        ref[2,3,2,0]=(0.646304172575)*x_ref[0]+(0.0132628778543)*x_ref[1]
        ref[2,3,2,1]=(-0.746041438673)*x_ref[0]+(0.516304872741)*x_ref[1]
        ref[2,4,0,0]=(0.0326900201283)*x_ref[0]+(-0.533021156401)*x_ref[1]
        ref[2,4,0,1]=(0.088598956981)*x_ref[0]+(0.764307970269)*x_ref[1]
        ref[2,4,1,0]=(-0.749553477756)*x_ref[0]+(0.379646792939)*x_ref[1]
        ref[2,4,1,1]=(-0.243635321378)*x_ref[0]+(-0.470022600261)*x_ref[1]
        ref[2,4,2,0]=(0.586398757247)*x_ref[0]+(-0.116982897683)*x_ref[1]
        ref[2,4,2,1]=(-0.989012121879)*x_ref[0]+(-1.25006535257)*x_ref[1]
        ref[3,0,0,0]=(1.25228306081)*x_ref[0]+(-0.0932717035896)*x_ref[1]
        ref[3,0,0,1]=(1.5975516771)*x_ref[0]+(-0.177733286111)*x_ref[1]
        ref[3,0,1,0]=(-0.659910070911)*x_ref[0]+(-0.0999498944812)*x_ref[1]
        ref[3,0,1,1]=(-0.299870273662)*x_ref[0]+(-1.22977656744)*x_ref[1]
        ref[3,0,2,0]=(-0.158211581143)*x_ref[0]+(-0.141421380503)*x_ref[1]
        ref[3,0,2,1]=(-1.48226452788)*x_ref[0]+(-0.731264943309)*x_ref[1]
        ref[3,1,0,0]=(-1.64589827853)*x_ref[0]+(1.19692573998)*x_ref[1]
        ref[3,1,0,1]=(0.361329655357)*x_ref[0]+(0.234310482171)*x_ref[1]
        ref[3,1,1,0]=(-1.14077649516)*x_ref[0]+(0.132657084065)*x_ref[1]
        ref[3,1,1,1]=(1.41067975663)*x_ref[0]+(0.818243117884)*x_ref[1]
        ref[3,1,2,0]=(-1.53137884342)*x_ref[0]+(-0.0127805333093)*x_ref[1]
        ref[3,1,2,1]=(0.240455755486)*x_ref[0]+(-1.02361753893)*x_ref[1]
        ref[3,2,0,0]=(-0.334803473072)*x_ref[0]+(-0.807452510423)*x_ref[1]
        ref[3,2,0,1]=(0.617324536974)*x_ref[0]+(-0.733163471774)*x_ref[1]
        ref[3,2,1,0]=(-0.634580050897)*x_ref[0]+(0.362106278168)*x_ref[1]
        ref[3,2,1,1]=(-0.21525531818)*x_ref[0]+(0.982902540898)*x_ref[1]
        ref[3,2,2,0]=(-0.590682803195)*x_ref[0]+(-1.27537880632)*x_ref[1]
        ref[3,2,2,1]=(0.142578313438)*x_ref[0]+(0.212341687291)*x_ref[1]
        ref[3,3,0,0]=(-0.758345317216)*x_ref[0]+(-0.971914631359)*x_ref[1]
        ref[3,3,0,1]=(0.569463909234)*x_ref[0]+(0.498848917338)*x_ref[1]
        ref[3,3,1,0]=(-0.771456575632)*x_ref[0]+(-0.130456072944)*x_ref[1]
        ref[3,3,1,1]=(0.811799269245)*x_ref[0]+(-0.829647203396)*x_ref[1]
        ref[3,3,2,0]=(-0.814979427693)*x_ref[0]+(-0.0416092796813)*x_ref[1]
        ref[3,3,2,1]=(0.933434189127)*x_ref[0]+(0.60794704936)*x_ref[1]
        ref[3,4,0,0]=(0.160280566478)*x_ref[0]+(-0.0141616040181)*x_ref[1]
        ref[3,4,0,1]=(-1.34835766268)*x_ref[0]+(0.178951225861)*x_ref[1]
        ref[3,4,1,0]=(1.67632814658)*x_ref[0]+(1.31316096483)*x_ref[1]
        ref[3,4,1,1]=(-0.603403275376)*x_ref[0]+(0.0607753526431)*x_ref[1]
        ref[3,4,2,0]=(-0.178440295473)*x_ref[0]+(0.318102694371)*x_ref[1]
        ref[3,4,2,1]=(0.781285160749)*x_ref[0]+(0.625651915981)*x_ref[1]
      else:
        s[0,0,0,0]=(0.847845815599)*x[0]+(-0.610234211476)*x[1]+(-0.627625510082)*x[2]
        s[0,0,0,1]=(0.0126184651041)*x[0]+(-0.0651294063564)*x[1]+(-0.274615935201)*x[2]
        s[0,0,1,0]=(-0.545925417969)*x[0]+(0.421929302201)*x[1]+(-0.910418625383)*x[2]
        s[0,0,1,1]=(1.58754252937)*x[0]+(0.110111591047)*x[1]+(-1.64780563736)*x[2]
        s[0,0,2,0]=(-0.510331643992)*x[0]+(0.827685326542)*x[1]+(1.31447962917)*x[2]
        s[0,0,2,1]=(-1.29575211142)*x[0]+(-0.318096803401)*x[1]+(-0.274660613777)*x[2]
        s[0,1,0,0]=(0.77444944835)*x[0]+(-1.0893411122)*x[1]+(0.331702744143)*x[2]
        s[0,1,0,1]=(0.624693916432)*x[0]+(-0.306552086758)*x[1]+(-0.845524417807)*x[2]
        s[0,1,1,0]=(0.0219492668609)*x[0]+(-1.13712865204)*x[1]+(-0.193051406533)*x[2]
        s[0,1,1,1]=(-1.32696889246)*x[0]+(1.63857177904)*x[1]+(0.194095476936)*x[2]
        s[0,1,2,0]=(1.19169978642)*x[0]+(-1.88274910505)*x[1]+(0.321438561021)*x[2]
        s[0,1,2,1]=(-0.765748081809)*x[0]+(-0.163422665563)*x[1]+(0.256721885141)*x[2]
        s[0,2,0,0]=(-0.863281319991)*x[0]+(-0.021828308413)*x[1]+(0.868312889989)*x[2]
        s[0,2,0,1]=(-0.57479341478)*x[0]+(0.404861121887)*x[1]+(-0.274037007699)*x[2]
        s[0,2,1,0]=(-0.529171478131)*x[0]+(0.836001872018)*x[1]+(-1.12226189724)*x[2]
        s[0,2,1,1]=(-0.821929160411)*x[0]+(0.0466762602824)*x[1]+(-0.0514331500765)*x[2]
        s[0,2,2,0]=(-0.605940618544)*x[0]+(-1.54379919132)*x[1]+(1.63243021269)*x[2]
        s[0,2,2,1]=(-1.49695860655)*x[0]+(-0.601345463475)*x[1]+(-0.753443592162)*x[2]
        s[0,3,0,0]=(1.33514081849)*x[0]+(0.955849232345)*x[1]+(-0.872221354599)*x[2]
        s[0,3,0,1]=(0.939731890901)*x[0]+(-1.16116012822)*x[1]+(-0.601602002743)*x[2]
        s[0,3,1,0]=(-0.675545889094)*x[0]+(0.736755090077)*x[1]+(-0.437370345974)*x[2]
        s[0,3,1,1]=(-1.17899966213)*x[0]+(0.422125349488)*x[1]+(-0.815649229407)*x[2]
        s[0,3,2,0]=(0.0773790438387)*x[0]+(0.148378160687)*x[1]+(-1.0156040466)*x[2]
        s[0,3,2,1]=(0.519422925349)*x[0]+(0.189993892084)*x[1]+(1.59654399673)*x[2]
        s[0,4,0,0]=(-0.0593352727817)*x[0]+(0.0865384232804)*x[1]+(-0.316122229956)*x[2]
        s[0,4,0,1]=(0.935220504817)*x[0]+(0.868143343952)*x[1]+(-0.470286174039)*x[2]
        s[0,4,1,0]=(0.763506086212)*x[0]+(-0.0756795330833)*x[1]+(1.06812672934)*x[2]
        s[0,4,1,1]=(-0.446580256351)*x[0]+(0.653986880233)*x[1]+(0.634364696873)*x[2]
        s[0,4,2,0]=(-0.72871248403)*x[0]+(0.0839150289011)*x[1]+(0.178899292496)*x[2]
        s[0,4,2,1]=(1.30808288445)*x[0]+(0.548931211195)*x[1]+(-0.971585668771)*x[2]
        s[1,0,0,0]=(-0.737171903787)*x[0]+(-0.137110160891)*x[1]+(1.24817883553)*x[2]
        s[1,0,0,1]=(0.589824248199)*x[0]+(-0.191688810075)*x[1]+(1.33820051734)*x[2]
        s[1,0,1,0]=(-0.598063103136)*x[0]+(0.347705502684)*x[1]+(0.490747627936)*x[2]
        s[1,0,1,1]=(0.287829427303)*x[0]+(-0.853875346674)*x[1]+(-0.227722773909)*x[2]
        s[1,0,2,0]=(-1.22183236343)*x[0]+(-0.162406713796)*x[1]+(-0.436433884201)*x[2]
        s[1,0,2,1]=(0.929372688401)*x[0]+(0.469931776805)*x[1]+(0.739110972137)*x[2]
        s[1,1,0,0]=(0.391824586)*x[0]+(-0.551498198462)*x[1]+(-0.588935046796)*x[2]
        s[1,1,0,1]=(0.849805745827)*x[0]+(0.557121315313)*x[1]+(1.80284061615)*x[2]
        s[1,1,1,0]=(-1.62270432051)*x[0]+(-1.55943636923)*x[1]+(0.629204169009)*x[2]
        s[1,1,1,1]=(-0.191271118514)*x[0]+(-0.454718665581)*x[1]+(0.405403171722)*x[2]
        s[1,1,2,0]=(0.461750123507)*x[0]+(0.537594040674)*x[1]+(-0.500046587877)*x[2]
        s[1,1,2,1]=(1.07800193455)*x[0]+(0.0757746249706)*x[1]+(0.358223454759)*x[2]
        s[1,2,0,0]=(-1.31360669142)*x[0]+(0.908413426737)*x[1]+(0.986904165818)*x[2]
        s[1,2,0,1]=(-0.250344247716)*x[0]+(0.596746427887)*x[1]+(0.846846599186)*x[2]
        s[1,2,1,0]=(1.65761753214)*x[0]+(0.0969407749043)*x[1]+(0.197913502281)*x[2]
        s[1,2,1,1]=(0.808454446977)*x[0]+(-0.107176416129)*x[1]+(-0.115562549181)*x[2]
        s[1,2,2,0]=(-0.410108442793)*x[0]+(0.250496842214)*x[1]+(0.403806076052)*x[2]
        s[1,2,2,1]=(1.58110488679)*x[0]+(-0.171583112102)*x[1]+(0.441813895222)*x[2]
        s[1,3,0,0]=(-0.519973841295)*x[0]+(0.14862500493)*x[1]+(-0.0257159960311)*x[2]
        s[1,3,0,1]=(-0.710423889723)*x[0]+(0.0941990549739)*x[1]+(-0.147877062943)*x[2]
        s[1,3,1,0]=(-0.19917762038)*x[0]+(-1.06873791711)*x[1]+(1.37688275432)*x[2]
        s[1,3,1,1]=(-0.570477493665)*x[0]+(0.0705814270652)*x[1]+(-1.10186231057)*x[2]
        s[1,3,2,0]=(-1.04891017002)*x[0]+(-0.136784107636)*x[1]+(-1.65953016693)*x[2]
        s[1,3,2,1]=(-0.674805716398)*x[0]+(-0.211803508128)*x[1]+(-0.116463503974)*x[2]
        s[1,4,0,0]=(0.269217220297)*x[0]+(0.810722194772)*x[1]+(1.60977315112)*x[2]
        s[1,4,0,1]=(-0.0213286766746)*x[0]+(1.0974274553)*x[1]+(0.275633207025)*x[2]
        s[1,4,1,0]=(-0.601230245409)*x[0]+(-0.756418955933)*x[1]+(-0.46194303915)*x[2]
        s[1,4,1,1]=(-0.0907667209143)*x[0]+(-1.86623564673)*x[1]+(0.552729474701)*x[2]
        s[1,4,2,0]=(-1.09694541489)*x[0]+(-0.496193881555)*x[1]+(-0.0126133249168)*x[2]
        s[1,4,2,1]=(-0.648959246633)*x[0]+(-0.60655325639)*x[1]+(0.438258051102)*x[2]
        s[2,0,0,0]=(-0.100512547019)*x[0]+(-1.37291269079)*x[1]+(1.63084319876)*x[2]
        s[2,0,0,1]=(-0.50510326916)*x[0]+(0.308799737351)*x[1]+(1.63506761477)*x[2]
        s[2,0,1,0]=(0.708408519723)*x[0]+(-0.0759833841695)*x[1]+(-0.980974095374)*x[2]
        s[2,0,1,1]=(-1.59727324143)*x[0]+(0.995250195962)*x[1]+(0.169527316083)*x[2]
        s[2,0,2,0]=(0.766935817054)*x[0]+(0.455564399031)*x[1]+(0.605772951782)*x[2]
        s[2,0,2,1]=(-0.820755214894)*x[0]+(-0.43359371038)*x[1]+(-0.0942398466964)*x[2]
        s[2,1,0,0]=(0.390476682801)*x[0]+(1.11627447827)*x[1]+(0.670843405466)*x[2]
        s[2,1,0,1]=(-0.433594870483)*x[0]+(-0.940710004402)*x[1]+(1.10756041487)*x[2]
        s[2,1,1,0]=(-1.21880018868)*x[0]+(0.380422307344)*x[1]+(-0.864017731789)*x[2]
        s[2,1,1,1]=(-0.194168346974)*x[0]+(0.324183983028)*x[1]+(0.741456579501)*x[2]
        s[2,1,2,0]=(-0.134211025912)*x[0]+(0.497311041207)*x[1]+(0.33014196376)*x[2]
        s[2,1,2,1]=(-1.3501981937)*x[0]+(1.67601514859)*x[1]+(-1.60084725544)*x[2]
        s[2,2,0,0]=(-0.8367457236)*x[0]+(-1.15628524562)*x[1]+(0.342983125417)*x[2]
        s[2,2,0,1]=(0.956733778283)*x[0]+(-0.404880336577)*x[1]+(-0.350927185105)*x[2]
        s[2,2,1,0]=(0.854842449281)*x[0]+(-0.185001304765)*x[1]+(1.15245157168)*x[2]
        s[2,2,1,1]=(0.00584831242623)*x[0]+(-0.562950548984)*x[1]+(0.785766513431)*x[2]
        s[2,2,2,0]=(0.272759945164)*x[0]+(-0.568207376307)*x[1]+(-0.155997758854)*x[2]
        s[2,2,2,1]=(-0.105324133175)*x[0]+(1.11807221711)*x[1]+(0.122751975298)*x[2]
        s[2,3,0,0]=(0.685932400265)*x[0]+(-0.666847052302)*x[1]+(0.596321369477)*x[2]
        s[2,3,0,1]=(0.335067153518)*x[0]+(0.268453607431)*x[1]+(0.247881738828)*x[2]
        s[2,3,1,0]=(-1.89084909179)*x[0]+(-0.529751828744)*x[1]+(-0.959358202828)*x[2]
        s[2,3,1,1]=(1.06725689989)*x[0]+(-0.709878517637)*x[1]+(0.968886366535)*x[2]
        s[2,3,2,0]=(-1.63466018652)*x[0]+(0.517481248484)*x[1]+(0.109232845638)*x[2]
        s[2,3,2,1]=(0.0716425266603)*x[0]+(-0.58040972115)*x[1]+(0.346948738894)*x[2]
        s[2,4,0,0]=(0.193599854213)*x[0]+(-1.52744079327)*x[1]+(-0.0377038338183)*x[2]
        s[2,4,0,1]=(-0.0296855161658)*x[0]+(0.266976994363)*x[1]+(-0.832771979863)*x[2]
        s[2,4,1,0]=(-0.885217283694)*x[0]+(0.657425725373)*x[1]+(0.385584782531)*x[2]
        s[2,4,1,1]=(-0.249912733602)*x[0]+(0.156704487203)*x[1]+(0.000393408500501)*x[2]
        s[2,4,2,0]=(-0.083303562971)*x[0]+(0.748009334975)*x[1]+(-0.411323946112)*x[2]
        s[2,4,2,1]=(0.143782087293)*x[0]+(-1.86497054401)*x[1]+(-0.398161751889)*x[2]
        s[3,0,0,0]=(-0.531041035506)*x[0]+(0.998737023878)*x[1]+(-0.719326852701)*x[2]
        s[3,0,0,1]=(-1.10320774278)*x[0]+(-1.27795876669)*x[1]+(0.57451462957)*x[2]
        s[3,0,1,0]=(-0.319275578502)*x[0]+(-1.51721910203)*x[1]+(-0.677470578443)*x[2]
        s[3,0,1,1]=(0.7584807917)*x[0]+(0.0523998746885)*x[1]+(0.502182688228)*x[2]
        s[3,0,2,0]=(-0.905809892125)*x[0]+(0.479751975382)*x[1]+(-0.0778539387977)*x[2]
        s[3,0,2,1]=(-0.724891388357)*x[0]+(0.941493554841)*x[1]+(-1.51514565838)*x[2]
        s[3,1,0,0]=(0.815118793475)*x[0]+(0.0503368907581)*x[1]+(-0.502091253213)*x[2]
        s[3,1,0,1]=(-1.50859117948)*x[0]+(0.31279517213)*x[1]+(-0.339295035594)*x[2]
        s[3,1,1,0]=(0.246825549192)*x[0]+(0.687251908372)*x[1]+(0.133971781776)*x[2]
        s[3,1,1,1]=(0.470105585328)*x[0]+(0.382971101979)*x[1]+(-0.469397699886)*x[2]
        s[3,1,2,0]=(-1.45842815542)*x[0]+(-1.00979255513)*x[1]+(1.45386034482)*x[2]
        s[3,1,2,1]=(0.172130164577)*x[0]+(1.25034739991)*x[1]+(-1.15953423359)*x[2]
        s[3,2,0,0]=(-0.58261393055)*x[0]+(0.730381032663)*x[1]+(1.29377076249)*x[2]
        s[3,2,0,1]=(0.696715362351)*x[0]+(-1.37598351546)*x[1]+(-1.26545085967)*x[2]
        s[3,2,1,0]=(-0.0511282610422)*x[0]+(0.58041679118)*x[1]+(-0.0810357412151)*x[2]
        s[3,2,1,1]=(-0.750057228803)*x[0]+(-0.750703125074)*x[1]+(-1.21748956536)*x[2]
        s[3,2,2,0]=(-0.455746666635)*x[0]+(-0.720020627516)*x[1]+(-0.0911281889971)*x[2]
        s[3,2,2,1]=(-0.0681894163236)*x[0]+(0.101592029278)*x[1]+(1.10954864406)*x[2]
        s[3,3,0,0]=(0.433898443791)*x[0]+(0.901180030496)*x[1]+(0.795641263874)*x[2]
        s[3,3,0,1]=(-0.213306810738)*x[0]+(-0.852539239286)*x[1]+(-1.25905416259)*x[2]
        s[3,3,1,0]=(-0.668307384473)*x[0]+(-1.37753879862)*x[1]+(-0.593731800903)*x[2]
        s[3,3,1,1]=(0.815464294514)*x[0]+(0.835389640785)*x[1]+(0.933029468258)*x[2]
        s[3,3,2,0]=(0.565194274444)*x[0]+(0.584119707001)*x[1]+(-0.53812189952)*x[2]
        s[3,3,2,1]=(-0.903604209314)*x[0]+(0.370887007483)*x[1]+(-0.548135915608)*x[2]
        s[3,4,0,0]=(-0.819290139492)*x[0]+(-0.256800498012)*x[1]+(0.350363336935)*x[2]
        s[3,4,0,1]=(0.863170182723)*x[0]+(0.0101766489337)*x[1]+(1.34009635664)*x[2]
        s[3,4,1,0]=(1.03350734136)*x[0]+(1.37672500233)*x[1]+(1.35134434663)*x[2]
        s[3,4,1,1]=(1.4883144839)*x[0]+(0.410688244039)*x[1]+(0.759267998436)*x[2]
        s[3,4,2,0]=(0.373190738096)*x[0]+(1.22136515279)*x[1]+(-0.0365030690962)*x[2]
        s[3,4,2,1]=(-0.607437402737)*x[0]+(0.750130654364)*x[1]+(0.084778884789)*x[2]
        ref[0,0,0,0]=(0.847845815599)*x_ref[0]+(-0.610234211476)*x_ref[1]+(-0.627625510082)*x_ref[2]
        ref[0,0,0,1]=(0.0126184651041)*x_ref[0]+(-0.0651294063564)*x_ref[1]+(-0.274615935201)*x_ref[2]
        ref[0,0,1,0]=(-0.545925417969)*x_ref[0]+(0.421929302201)*x_ref[1]+(-0.910418625383)*x_ref[2]
        ref[0,0,1,1]=(1.58754252937)*x_ref[0]+(0.110111591047)*x_ref[1]+(-1.64780563736)*x_ref[2]
        ref[0,0,2,0]=(-0.510331643992)*x_ref[0]+(0.827685326542)*x_ref[1]+(1.31447962917)*x_ref[2]
        ref[0,0,2,1]=(-1.29575211142)*x_ref[0]+(-0.318096803401)*x_ref[1]+(-0.274660613777)*x_ref[2]
        ref[0,1,0,0]=(0.77444944835)*x_ref[0]+(-1.0893411122)*x_ref[1]+(0.331702744143)*x_ref[2]
        ref[0,1,0,1]=(0.624693916432)*x_ref[0]+(-0.306552086758)*x_ref[1]+(-0.845524417807)*x_ref[2]
        ref[0,1,1,0]=(0.0219492668609)*x_ref[0]+(-1.13712865204)*x_ref[1]+(-0.193051406533)*x_ref[2]
        ref[0,1,1,1]=(-1.32696889246)*x_ref[0]+(1.63857177904)*x_ref[1]+(0.194095476936)*x_ref[2]
        ref[0,1,2,0]=(1.19169978642)*x_ref[0]+(-1.88274910505)*x_ref[1]+(0.321438561021)*x_ref[2]
        ref[0,1,2,1]=(-0.765748081809)*x_ref[0]+(-0.163422665563)*x_ref[1]+(0.256721885141)*x_ref[2]
        ref[0,2,0,0]=(-0.863281319991)*x_ref[0]+(-0.021828308413)*x_ref[1]+(0.868312889989)*x_ref[2]
        ref[0,2,0,1]=(-0.57479341478)*x_ref[0]+(0.404861121887)*x_ref[1]+(-0.274037007699)*x_ref[2]
        ref[0,2,1,0]=(-0.529171478131)*x_ref[0]+(0.836001872018)*x_ref[1]+(-1.12226189724)*x_ref[2]
        ref[0,2,1,1]=(-0.821929160411)*x_ref[0]+(0.0466762602824)*x_ref[1]+(-0.0514331500765)*x_ref[2]
        ref[0,2,2,0]=(-0.605940618544)*x_ref[0]+(-1.54379919132)*x_ref[1]+(1.63243021269)*x_ref[2]
        ref[0,2,2,1]=(-1.49695860655)*x_ref[0]+(-0.601345463475)*x_ref[1]+(-0.753443592162)*x_ref[2]
        ref[0,3,0,0]=(1.33514081849)*x_ref[0]+(0.955849232345)*x_ref[1]+(-0.872221354599)*x_ref[2]
        ref[0,3,0,1]=(0.939731890901)*x_ref[0]+(-1.16116012822)*x_ref[1]+(-0.601602002743)*x_ref[2]
        ref[0,3,1,0]=(-0.675545889094)*x_ref[0]+(0.736755090077)*x_ref[1]+(-0.437370345974)*x_ref[2]
        ref[0,3,1,1]=(-1.17899966213)*x_ref[0]+(0.422125349488)*x_ref[1]+(-0.815649229407)*x_ref[2]
        ref[0,3,2,0]=(0.0773790438387)*x_ref[0]+(0.148378160687)*x_ref[1]+(-1.0156040466)*x_ref[2]
        ref[0,3,2,1]=(0.519422925349)*x_ref[0]+(0.189993892084)*x_ref[1]+(1.59654399673)*x_ref[2]
        ref[0,4,0,0]=(-0.0593352727817)*x_ref[0]+(0.0865384232804)*x_ref[1]+(-0.316122229956)*x_ref[2]
        ref[0,4,0,1]=(0.935220504817)*x_ref[0]+(0.868143343952)*x_ref[1]+(-0.470286174039)*x_ref[2]
        ref[0,4,1,0]=(0.763506086212)*x_ref[0]+(-0.0756795330833)*x_ref[1]+(1.06812672934)*x_ref[2]
        ref[0,4,1,1]=(-0.446580256351)*x_ref[0]+(0.653986880233)*x_ref[1]+(0.634364696873)*x_ref[2]
        ref[0,4,2,0]=(-0.72871248403)*x_ref[0]+(0.0839150289011)*x_ref[1]+(0.178899292496)*x_ref[2]
        ref[0,4,2,1]=(1.30808288445)*x_ref[0]+(0.548931211195)*x_ref[1]+(-0.971585668771)*x_ref[2]
        ref[1,0,0,0]=(-0.737171903787)*x_ref[0]+(-0.137110160891)*x_ref[1]+(1.24817883553)*x_ref[2]
        ref[1,0,0,1]=(0.589824248199)*x_ref[0]+(-0.191688810075)*x_ref[1]+(1.33820051734)*x_ref[2]
        ref[1,0,1,0]=(-0.598063103136)*x_ref[0]+(0.347705502684)*x_ref[1]+(0.490747627936)*x_ref[2]
        ref[1,0,1,1]=(0.287829427303)*x_ref[0]+(-0.853875346674)*x_ref[1]+(-0.227722773909)*x_ref[2]
        ref[1,0,2,0]=(-1.22183236343)*x_ref[0]+(-0.162406713796)*x_ref[1]+(-0.436433884201)*x_ref[2]
        ref[1,0,2,1]=(0.929372688401)*x_ref[0]+(0.469931776805)*x_ref[1]+(0.739110972137)*x_ref[2]
        ref[1,1,0,0]=(0.391824586)*x_ref[0]+(-0.551498198462)*x_ref[1]+(-0.588935046796)*x_ref[2]
        ref[1,1,0,1]=(0.849805745827)*x_ref[0]+(0.557121315313)*x_ref[1]+(1.80284061615)*x_ref[2]
        ref[1,1,1,0]=(-1.62270432051)*x_ref[0]+(-1.55943636923)*x_ref[1]+(0.629204169009)*x_ref[2]
        ref[1,1,1,1]=(-0.191271118514)*x_ref[0]+(-0.454718665581)*x_ref[1]+(0.405403171722)*x_ref[2]
        ref[1,1,2,0]=(0.461750123507)*x_ref[0]+(0.537594040674)*x_ref[1]+(-0.500046587877)*x_ref[2]
        ref[1,1,2,1]=(1.07800193455)*x_ref[0]+(0.0757746249706)*x_ref[1]+(0.358223454759)*x_ref[2]
        ref[1,2,0,0]=(-1.31360669142)*x_ref[0]+(0.908413426737)*x_ref[1]+(0.986904165818)*x_ref[2]
        ref[1,2,0,1]=(-0.250344247716)*x_ref[0]+(0.596746427887)*x_ref[1]+(0.846846599186)*x_ref[2]
        ref[1,2,1,0]=(1.65761753214)*x_ref[0]+(0.0969407749043)*x_ref[1]+(0.197913502281)*x_ref[2]
        ref[1,2,1,1]=(0.808454446977)*x_ref[0]+(-0.107176416129)*x_ref[1]+(-0.115562549181)*x_ref[2]
        ref[1,2,2,0]=(-0.410108442793)*x_ref[0]+(0.250496842214)*x_ref[1]+(0.403806076052)*x_ref[2]
        ref[1,2,2,1]=(1.58110488679)*x_ref[0]+(-0.171583112102)*x_ref[1]+(0.441813895222)*x_ref[2]
        ref[1,3,0,0]=(-0.519973841295)*x_ref[0]+(0.14862500493)*x_ref[1]+(-0.0257159960311)*x_ref[2]
        ref[1,3,0,1]=(-0.710423889723)*x_ref[0]+(0.0941990549739)*x_ref[1]+(-0.147877062943)*x_ref[2]
        ref[1,3,1,0]=(-0.19917762038)*x_ref[0]+(-1.06873791711)*x_ref[1]+(1.37688275432)*x_ref[2]
        ref[1,3,1,1]=(-0.570477493665)*x_ref[0]+(0.0705814270652)*x_ref[1]+(-1.10186231057)*x_ref[2]
        ref[1,3,2,0]=(-1.04891017002)*x_ref[0]+(-0.136784107636)*x_ref[1]+(-1.65953016693)*x_ref[2]
        ref[1,3,2,1]=(-0.674805716398)*x_ref[0]+(-0.211803508128)*x_ref[1]+(-0.116463503974)*x_ref[2]
        ref[1,4,0,0]=(0.269217220297)*x_ref[0]+(0.810722194772)*x_ref[1]+(1.60977315112)*x_ref[2]
        ref[1,4,0,1]=(-0.0213286766746)*x_ref[0]+(1.0974274553)*x_ref[1]+(0.275633207025)*x_ref[2]
        ref[1,4,1,0]=(-0.601230245409)*x_ref[0]+(-0.756418955933)*x_ref[1]+(-0.46194303915)*x_ref[2]
        ref[1,4,1,1]=(-0.0907667209143)*x_ref[0]+(-1.86623564673)*x_ref[1]+(0.552729474701)*x_ref[2]
        ref[1,4,2,0]=(-1.09694541489)*x_ref[0]+(-0.496193881555)*x_ref[1]+(-0.0126133249168)*x_ref[2]
        ref[1,4,2,1]=(-0.648959246633)*x_ref[0]+(-0.60655325639)*x_ref[1]+(0.438258051102)*x_ref[2]
        ref[2,0,0,0]=(-0.100512547019)*x_ref[0]+(-1.37291269079)*x_ref[1]+(1.63084319876)*x_ref[2]
        ref[2,0,0,1]=(-0.50510326916)*x_ref[0]+(0.308799737351)*x_ref[1]+(1.63506761477)*x_ref[2]
        ref[2,0,1,0]=(0.708408519723)*x_ref[0]+(-0.0759833841695)*x_ref[1]+(-0.980974095374)*x_ref[2]
        ref[2,0,1,1]=(-1.59727324143)*x_ref[0]+(0.995250195962)*x_ref[1]+(0.169527316083)*x_ref[2]
        ref[2,0,2,0]=(0.766935817054)*x_ref[0]+(0.455564399031)*x_ref[1]+(0.605772951782)*x_ref[2]
        ref[2,0,2,1]=(-0.820755214894)*x_ref[0]+(-0.43359371038)*x_ref[1]+(-0.0942398466964)*x_ref[2]
        ref[2,1,0,0]=(0.390476682801)*x_ref[0]+(1.11627447827)*x_ref[1]+(0.670843405466)*x_ref[2]
        ref[2,1,0,1]=(-0.433594870483)*x_ref[0]+(-0.940710004402)*x_ref[1]+(1.10756041487)*x_ref[2]
        ref[2,1,1,0]=(-1.21880018868)*x_ref[0]+(0.380422307344)*x_ref[1]+(-0.864017731789)*x_ref[2]
        ref[2,1,1,1]=(-0.194168346974)*x_ref[0]+(0.324183983028)*x_ref[1]+(0.741456579501)*x_ref[2]
        ref[2,1,2,0]=(-0.134211025912)*x_ref[0]+(0.497311041207)*x_ref[1]+(0.33014196376)*x_ref[2]
        ref[2,1,2,1]=(-1.3501981937)*x_ref[0]+(1.67601514859)*x_ref[1]+(-1.60084725544)*x_ref[2]
        ref[2,2,0,0]=(-0.8367457236)*x_ref[0]+(-1.15628524562)*x_ref[1]+(0.342983125417)*x_ref[2]
        ref[2,2,0,1]=(0.956733778283)*x_ref[0]+(-0.404880336577)*x_ref[1]+(-0.350927185105)*x_ref[2]
        ref[2,2,1,0]=(0.854842449281)*x_ref[0]+(-0.185001304765)*x_ref[1]+(1.15245157168)*x_ref[2]
        ref[2,2,1,1]=(0.00584831242623)*x_ref[0]+(-0.562950548984)*x_ref[1]+(0.785766513431)*x_ref[2]
        ref[2,2,2,0]=(0.272759945164)*x_ref[0]+(-0.568207376307)*x_ref[1]+(-0.155997758854)*x_ref[2]
        ref[2,2,2,1]=(-0.105324133175)*x_ref[0]+(1.11807221711)*x_ref[1]+(0.122751975298)*x_ref[2]
        ref[2,3,0,0]=(0.685932400265)*x_ref[0]+(-0.666847052302)*x_ref[1]+(0.596321369477)*x_ref[2]
        ref[2,3,0,1]=(0.335067153518)*x_ref[0]+(0.268453607431)*x_ref[1]+(0.247881738828)*x_ref[2]
        ref[2,3,1,0]=(-1.89084909179)*x_ref[0]+(-0.529751828744)*x_ref[1]+(-0.959358202828)*x_ref[2]
        ref[2,3,1,1]=(1.06725689989)*x_ref[0]+(-0.709878517637)*x_ref[1]+(0.968886366535)*x_ref[2]
        ref[2,3,2,0]=(-1.63466018652)*x_ref[0]+(0.517481248484)*x_ref[1]+(0.109232845638)*x_ref[2]
        ref[2,3,2,1]=(0.0716425266603)*x_ref[0]+(-0.58040972115)*x_ref[1]+(0.346948738894)*x_ref[2]
        ref[2,4,0,0]=(0.193599854213)*x_ref[0]+(-1.52744079327)*x_ref[1]+(-0.0377038338183)*x_ref[2]
        ref[2,4,0,1]=(-0.0296855161658)*x_ref[0]+(0.266976994363)*x_ref[1]+(-0.832771979863)*x_ref[2]
        ref[2,4,1,0]=(-0.885217283694)*x_ref[0]+(0.657425725373)*x_ref[1]+(0.385584782531)*x_ref[2]
        ref[2,4,1,1]=(-0.249912733602)*x_ref[0]+(0.156704487203)*x_ref[1]+(0.000393408500501)*x_ref[2]
        ref[2,4,2,0]=(-0.083303562971)*x_ref[0]+(0.748009334975)*x_ref[1]+(-0.411323946112)*x_ref[2]
        ref[2,4,2,1]=(0.143782087293)*x_ref[0]+(-1.86497054401)*x_ref[1]+(-0.398161751889)*x_ref[2]
        ref[3,0,0,0]=(-0.531041035506)*x_ref[0]+(0.998737023878)*x_ref[1]+(-0.719326852701)*x_ref[2]
        ref[3,0,0,1]=(-1.10320774278)*x_ref[0]+(-1.27795876669)*x_ref[1]+(0.57451462957)*x_ref[2]
        ref[3,0,1,0]=(-0.319275578502)*x_ref[0]+(-1.51721910203)*x_ref[1]+(-0.677470578443)*x_ref[2]
        ref[3,0,1,1]=(0.7584807917)*x_ref[0]+(0.0523998746885)*x_ref[1]+(0.502182688228)*x_ref[2]
        ref[3,0,2,0]=(-0.905809892125)*x_ref[0]+(0.479751975382)*x_ref[1]+(-0.0778539387977)*x_ref[2]
        ref[3,0,2,1]=(-0.724891388357)*x_ref[0]+(0.941493554841)*x_ref[1]+(-1.51514565838)*x_ref[2]
        ref[3,1,0,0]=(0.815118793475)*x_ref[0]+(0.0503368907581)*x_ref[1]+(-0.502091253213)*x_ref[2]
        ref[3,1,0,1]=(-1.50859117948)*x_ref[0]+(0.31279517213)*x_ref[1]+(-0.339295035594)*x_ref[2]
        ref[3,1,1,0]=(0.246825549192)*x_ref[0]+(0.687251908372)*x_ref[1]+(0.133971781776)*x_ref[2]
        ref[3,1,1,1]=(0.470105585328)*x_ref[0]+(0.382971101979)*x_ref[1]+(-0.469397699886)*x_ref[2]
        ref[3,1,2,0]=(-1.45842815542)*x_ref[0]+(-1.00979255513)*x_ref[1]+(1.45386034482)*x_ref[2]
        ref[3,1,2,1]=(0.172130164577)*x_ref[0]+(1.25034739991)*x_ref[1]+(-1.15953423359)*x_ref[2]
        ref[3,2,0,0]=(-0.58261393055)*x_ref[0]+(0.730381032663)*x_ref[1]+(1.29377076249)*x_ref[2]
        ref[3,2,0,1]=(0.696715362351)*x_ref[0]+(-1.37598351546)*x_ref[1]+(-1.26545085967)*x_ref[2]
        ref[3,2,1,0]=(-0.0511282610422)*x_ref[0]+(0.58041679118)*x_ref[1]+(-0.0810357412151)*x_ref[2]
        ref[3,2,1,1]=(-0.750057228803)*x_ref[0]+(-0.750703125074)*x_ref[1]+(-1.21748956536)*x_ref[2]
        ref[3,2,2,0]=(-0.455746666635)*x_ref[0]+(-0.720020627516)*x_ref[1]+(-0.0911281889971)*x_ref[2]
        ref[3,2,2,1]=(-0.0681894163236)*x_ref[0]+(0.101592029278)*x_ref[1]+(1.10954864406)*x_ref[2]
        ref[3,3,0,0]=(0.433898443791)*x_ref[0]+(0.901180030496)*x_ref[1]+(0.795641263874)*x_ref[2]
        ref[3,3,0,1]=(-0.213306810738)*x_ref[0]+(-0.852539239286)*x_ref[1]+(-1.25905416259)*x_ref[2]
        ref[3,3,1,0]=(-0.668307384473)*x_ref[0]+(-1.37753879862)*x_ref[1]+(-0.593731800903)*x_ref[2]
        ref[3,3,1,1]=(0.815464294514)*x_ref[0]+(0.835389640785)*x_ref[1]+(0.933029468258)*x_ref[2]
        ref[3,3,2,0]=(0.565194274444)*x_ref[0]+(0.584119707001)*x_ref[1]+(-0.53812189952)*x_ref[2]
        ref[3,3,2,1]=(-0.903604209314)*x_ref[0]+(0.370887007483)*x_ref[1]+(-0.548135915608)*x_ref[2]
        ref[3,4,0,0]=(-0.819290139492)*x_ref[0]+(-0.256800498012)*x_ref[1]+(0.350363336935)*x_ref[2]
        ref[3,4,0,1]=(0.863170182723)*x_ref[0]+(0.0101766489337)*x_ref[1]+(1.34009635664)*x_ref[2]
        ref[3,4,1,0]=(1.03350734136)*x_ref[0]+(1.37672500233)*x_ref[1]+(1.35134434663)*x_ref[2]
        ref[3,4,1,1]=(1.4883144839)*x_ref[0]+(0.410688244039)*x_ref[1]+(0.759267998436)*x_ref[2]
        ref[3,4,2,0]=(0.373190738096)*x_ref[0]+(1.22136515279)*x_ref[1]+(-0.0365030690962)*x_ref[2]
        ref[3,4,2,1]=(-0.607437402737)*x_ref[0]+(0.750130654364)*x_ref[1]+(0.084778884789)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_ReducedContinuousFunction_rank0(self):
      """
      tests interpolate for rank 0 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedContinuousFunction onto ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(),dim=dim)
      s=Data(0,(),w)
      ref=Data(0,(),w_ref)
      if dim==2:
        s=(-1.64974193328)*x[0]+(1.68675911612)*x[1]
        ref=(-1.64974193328)*x_ref[0]+(1.68675911612)*x_ref[1]
      else:
        s=(-0.517718083008)*x[0]+(0.479679773182)*x[1]+(0.72459612383)*x[2]
        ref=(-0.517718083008)*x_ref[0]+(0.479679773182)*x_ref[1]+(0.72459612383)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_ReducedContinuousFunction_rank1(self):
      """
      tests interpolate for rank 1 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedContinuousFunction onto ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(2,),dim=dim)
      s=Data(0,(2,),w)
      ref=Data(0,(2,),w_ref)
      if dim==2:
        s[0]=(-0.415290633997)*x[0]+(0.118775036022)*x[1]
        s[1]=(-0.833749289762)*x[0]+(0.121297221921)*x[1]
        ref[0]=(-0.415290633997)*x[0]+(0.118775036022)*x[1]
        ref[1]=(-0.833749289762)*x[0]+(0.121297221921)*x[1]
      else:
        s[0]=(-0.336174941008)*x[0]+(-0.844918885743)*x[1]+(-0.410649979186)*x[2]
        s[1]=(0.112527237498)*x[0]+(-0.867632388528)*x[1]+(1.42359104479)*x[2]
        ref[0]=(-0.336174941008)*x[0]+(-0.844918885743)*x[1]+(-0.410649979186)*x[2]
        ref[1]=(0.112527237498)*x[0]+(-0.867632388528)*x[1]+(1.42359104479)*x[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_ReducedContinuousFunction_rank2(self):
      """
      tests interpolate for rank 2 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedContinuousFunction onto ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5),dim=dim)
      s=Data(0,(4, 5),w)
      ref=Data(0,(4, 5),w_ref)
      if dim==2:
        s[0,0]=(-0.833920166667)*x[0]+(-0.105734397941)*x[1]
        s[0,1]=(-0.0535194301789)*x[0]+(1.2869199411)*x[1]
        s[0,2]=(-1.81783608134)*x[0]+(-0.0858549172679)*x[1]
        s[0,3]=(0.133570573696)*x[0]+(0.398281712973)*x[1]
        s[0,4]=(-1.2514060292)*x[0]+(-0.24743946144)*x[1]
        s[1,0]=(0.917760776668)*x[0]+(0.179758313228)*x[1]
        s[1,1]=(0.677772149724)*x[0]+(0.762705763999)*x[1]
        s[1,2]=(-0.505172381789)*x[0]+(0.412365083581)*x[1]
        s[1,3]=(0.0481873177823)*x[0]+(-0.449871653505)*x[1]
        s[1,4]=(-1.43052283016)*x[0]+(0.553426132082)*x[1]
        s[2,0]=(0.337172157552)*x[0]+(1.56154138911)*x[1]
        s[2,1]=(-0.338189138128)*x[0]+(-0.62614063612)*x[1]
        s[2,2]=(-1.51606676783)*x[0]+(-0.13283454533)*x[1]
        s[2,3]=(0.727068106091)*x[0]+(1.00949960235)*x[1]
        s[2,4]=(0.297941246402)*x[0]+(0.952119168452)*x[1]
        s[3,0]=(0.724157315025)*x[0]+(0.962405533793)*x[1]
        s[3,1]=(-0.239168209123)*x[0]+(1.28280794616)*x[1]
        s[3,2]=(-0.0336024791682)*x[0]+(-0.775860593407)*x[1]
        s[3,3]=(-0.118809020482)*x[0]+(0.51890714417)*x[1]
        s[3,4]=(0.374315302545)*x[0]+(-0.258942994)*x[1]
        ref[0,0]=(-0.833920166667)*x_ref[0]+(-0.105734397941)*x_ref[1]
        ref[0,1]=(-0.0535194301789)*x_ref[0]+(1.2869199411)*x_ref[1]
        ref[0,2]=(-1.81783608134)*x_ref[0]+(-0.0858549172679)*x_ref[1]
        ref[0,3]=(0.133570573696)*x_ref[0]+(0.398281712973)*x_ref[1]
        ref[0,4]=(-1.2514060292)*x_ref[0]+(-0.24743946144)*x_ref[1]
        ref[1,0]=(0.917760776668)*x_ref[0]+(0.179758313228)*x_ref[1]
        ref[1,1]=(0.677772149724)*x_ref[0]+(0.762705763999)*x_ref[1]
        ref[1,2]=(-0.505172381789)*x_ref[0]+(0.412365083581)*x_ref[1]
        ref[1,3]=(0.0481873177823)*x_ref[0]+(-0.449871653505)*x_ref[1]
        ref[1,4]=(-1.43052283016)*x_ref[0]+(0.553426132082)*x_ref[1]
        ref[2,0]=(0.337172157552)*x_ref[0]+(1.56154138911)*x_ref[1]
        ref[2,1]=(-0.338189138128)*x_ref[0]+(-0.62614063612)*x_ref[1]
        ref[2,2]=(-1.51606676783)*x_ref[0]+(-0.13283454533)*x_ref[1]
        ref[2,3]=(0.727068106091)*x_ref[0]+(1.00949960235)*x_ref[1]
        ref[2,4]=(0.297941246402)*x_ref[0]+(0.952119168452)*x_ref[1]
        ref[3,0]=(0.724157315025)*x_ref[0]+(0.962405533793)*x_ref[1]
        ref[3,1]=(-0.239168209123)*x_ref[0]+(1.28280794616)*x_ref[1]
        ref[3,2]=(-0.0336024791682)*x_ref[0]+(-0.775860593407)*x_ref[1]
        ref[3,3]=(-0.118809020482)*x_ref[0]+(0.51890714417)*x_ref[1]
        ref[3,4]=(0.374315302545)*x_ref[0]+(-0.258942994)*x_ref[1]
      else:
        s[0,0]=(1.18033303246)*x[0]+(0.59372740835)*x[1]+(-0.684959540977)*x[2]
        s[0,1]=(0.65301033503)*x[0]+(-1.17147722359)*x[1]+(0.455404166733)*x[2]
        s[0,2]=(-0.412048337979)*x[0]+(-1.24933561194)*x[1]+(-0.636287202752)*x[2]
        s[0,3]=(0.122156598551)*x[0]+(0.103952485283)*x[1]+(-0.564471099255)*x[2]
        s[0,4]=(0.737873603859)*x[0]+(0.139615778712)*x[1]+(0.523544473477)*x[2]
        s[1,0]=(0.0198124265514)*x[0]+(-0.335073882197)*x[1]+(-0.805965691618)*x[2]
        s[1,1]=(-1.28485879121)*x[0]+(-0.618847596536)*x[1]+(1.32879899911)*x[2]
        s[1,2]=(1.09319515176)*x[0]+(0.543230471078)*x[1]+(1.56430808151)*x[2]
        s[1,3]=(0.541425658238)*x[0]+(0.0140217060596)*x[1]+(0.603558416867)*x[2]
        s[1,4]=(0.303110106189)*x[0]+(-0.103009794147)*x[1]+(-1.68324105571)*x[2]
        s[2,0]=(-1.65609235169)*x[0]+(-0.290040263496)*x[1]+(0.477073850071)*x[2]
        s[2,1]=(-0.897378807931)*x[0]+(0.191583115578)*x[1]+(0.213226828436)*x[2]
        s[2,2]=(0.523724313062)*x[0]+(0.214413673529)*x[1]+(0.533345615094)*x[2]
        s[2,3]=(-0.994043729754)*x[0]+(-0.307957055273)*x[1]+(0.0422995421427)*x[2]
        s[2,4]=(1.22590973976)*x[0]+(-1.65014223339)*x[1]+(-1.59726996804)*x[2]
        s[3,0]=(-0.0870943824713)*x[0]+(-0.45766718732)*x[1]+(-0.878874753774)*x[2]
        s[3,1]=(0.244470697307)*x[0]+(-0.0429568231364)*x[1]+(-1.57098356615)*x[2]
        s[3,2]=(1.35091699523)*x[0]+(-0.127297590922)*x[1]+(-0.556702799475)*x[2]
        s[3,3]=(0.189117166393)*x[0]+(0.317562328009)*x[1]+(-0.59788285723)*x[2]
        s[3,4]=(-0.155355776878)*x[0]+(-0.616350184089)*x[1]+(0.296083681054)*x[2]
        ref[0,0]=(1.18033303246)*x_ref[0]+(0.59372740835)*x_ref[1]+(-0.684959540977)*x_ref[2]
        ref[0,1]=(0.65301033503)*x_ref[0]+(-1.17147722359)*x_ref[1]+(0.455404166733)*x_ref[2]
        ref[0,2]=(-0.412048337979)*x_ref[0]+(-1.24933561194)*x_ref[1]+(-0.636287202752)*x_ref[2]
        ref[0,3]=(0.122156598551)*x_ref[0]+(0.103952485283)*x_ref[1]+(-0.564471099255)*x_ref[2]
        ref[0,4]=(0.737873603859)*x_ref[0]+(0.139615778712)*x_ref[1]+(0.523544473477)*x_ref[2]
        ref[1,0]=(0.0198124265514)*x_ref[0]+(-0.335073882197)*x_ref[1]+(-0.805965691618)*x_ref[2]
        ref[1,1]=(-1.28485879121)*x_ref[0]+(-0.618847596536)*x_ref[1]+(1.32879899911)*x_ref[2]
        ref[1,2]=(1.09319515176)*x_ref[0]+(0.543230471078)*x_ref[1]+(1.56430808151)*x_ref[2]
        ref[1,3]=(0.541425658238)*x_ref[0]+(0.0140217060596)*x_ref[1]+(0.603558416867)*x_ref[2]
        ref[1,4]=(0.303110106189)*x_ref[0]+(-0.103009794147)*x_ref[1]+(-1.68324105571)*x_ref[2]
        ref[2,0]=(-1.65609235169)*x_ref[0]+(-0.290040263496)*x_ref[1]+(0.477073850071)*x_ref[2]
        ref[2,1]=(-0.897378807931)*x_ref[0]+(0.191583115578)*x_ref[1]+(0.213226828436)*x_ref[2]
        ref[2,2]=(0.523724313062)*x_ref[0]+(0.214413673529)*x_ref[1]+(0.533345615094)*x_ref[2]
        ref[2,3]=(-0.994043729754)*x_ref[0]+(-0.307957055273)*x_ref[1]+(0.0422995421427)*x_ref[2]
        ref[2,4]=(1.22590973976)*x_ref[0]+(-1.65014223339)*x_ref[1]+(-1.59726996804)*x_ref[2]
        ref[3,0]=(-0.0870943824713)*x_ref[0]+(-0.45766718732)*x_ref[1]+(-0.878874753774)*x_ref[2]
        ref[3,1]=(0.244470697307)*x_ref[0]+(-0.0429568231364)*x_ref[1]+(-1.57098356615)*x_ref[2]
        ref[3,2]=(1.35091699523)*x_ref[0]+(-0.127297590922)*x_ref[1]+(-0.556702799475)*x_ref[2]
        ref[3,3]=(0.189117166393)*x_ref[0]+(0.317562328009)*x_ref[1]+(-0.59788285723)*x_ref[2]
        ref[3,4]=(-0.155355776878)*x_ref[0]+(-0.616350184089)*x_ref[1]+(0.296083681054)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_ReducedContinuousFunction_rank3(self):
      """
      tests interpolate for rank 3 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedContinuousFunction onto ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(6, 2, 2),dim=dim)
      s=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2),w_ref)
      if dim==2:
        s[0,0,0]=(0.0730549738553)*x[0]+(0.585649475666)*x[1]
        s[0,0,1]=(-0.449075310223)*x[0]+(1.39116964942)*x[1]
        s[0,1,0]=(0.0210032238157)*x[0]+(1.33904896697)*x[1]
        s[0,1,1]=(0.556125399236)*x[0]+(-0.00311484949343)*x[1]
        s[1,0,0]=(0.938296971476)*x[0]+(1.13223213262)*x[1]
        s[1,0,1]=(0.744133292867)*x[0]+(0.0271311625871)*x[1]
        s[1,1,0]=(-0.0478922372672)*x[0]+(-1.08882574207)*x[1]
        s[1,1,1]=(0.738328994373)*x[0]+(-0.730449948444)*x[1]
        s[2,0,0]=(0.66593099915)*x[0]+(-0.185918605751)*x[1]
        s[2,0,1]=(0.450972410432)*x[0]+(1.3393675839)*x[1]
        s[2,1,0]=(0.897370528228)*x[0]+(0.765127571621)*x[1]
        s[2,1,1]=(-0.349658725406)*x[0]+(-0.536332219959)*x[1]
        s[3,0,0]=(1.11112898963)*x[0]+(1.50253301112)*x[1]
        s[3,0,1]=(0.238957188525)*x[0]+(0.775368032877)*x[1]
        s[3,1,0]=(0.612991596596)*x[0]+(0.0718037077356)*x[1]
        s[3,1,1]=(-0.40039392925)*x[0]+(-1.58712410632)*x[1]
        s[4,0,0]=(0.23241254342)*x[0]+(-0.547054930927)*x[1]
        s[4,0,1]=(-0.976995206698)*x[0]+(1.251791534)*x[1]
        s[4,1,0]=(-1.13243662763)*x[0]+(0.56739947752)*x[1]
        s[4,1,1]=(-0.543613180121)*x[0]+(0.649805921416)*x[1]
        s[5,0,0]=(-0.0690689383083)*x[0]+(0.818563064557)*x[1]
        s[5,0,1]=(0.362899720121)*x[0]+(-0.112604112729)*x[1]
        s[5,1,0]=(-0.291756903179)*x[0]+(-0.538221294595)*x[1]
        s[5,1,1]=(0.60406331286)*x[0]+(0.228744896658)*x[1]
        ref[0,0,0]=(0.0730549738553)*x_ref[0]+(0.585649475666)*x_ref[1]
        ref[0,0,1]=(-0.449075310223)*x_ref[0]+(1.39116964942)*x_ref[1]
        ref[0,1,0]=(0.0210032238157)*x_ref[0]+(1.33904896697)*x_ref[1]
        ref[0,1,1]=(0.556125399236)*x_ref[0]+(-0.00311484949343)*x_ref[1]
        ref[1,0,0]=(0.938296971476)*x_ref[0]+(1.13223213262)*x_ref[1]
        ref[1,0,1]=(0.744133292867)*x_ref[0]+(0.0271311625871)*x_ref[1]
        ref[1,1,0]=(-0.0478922372672)*x_ref[0]+(-1.08882574207)*x_ref[1]
        ref[1,1,1]=(0.738328994373)*x_ref[0]+(-0.730449948444)*x_ref[1]
        ref[2,0,0]=(0.66593099915)*x_ref[0]+(-0.185918605751)*x_ref[1]
        ref[2,0,1]=(0.450972410432)*x_ref[0]+(1.3393675839)*x_ref[1]
        ref[2,1,0]=(0.897370528228)*x_ref[0]+(0.765127571621)*x_ref[1]
        ref[2,1,1]=(-0.349658725406)*x_ref[0]+(-0.536332219959)*x_ref[1]
        ref[3,0,0]=(1.11112898963)*x_ref[0]+(1.50253301112)*x_ref[1]
        ref[3,0,1]=(0.238957188525)*x_ref[0]+(0.775368032877)*x_ref[1]
        ref[3,1,0]=(0.612991596596)*x_ref[0]+(0.0718037077356)*x_ref[1]
        ref[3,1,1]=(-0.40039392925)*x_ref[0]+(-1.58712410632)*x_ref[1]
        ref[4,0,0]=(0.23241254342)*x_ref[0]+(-0.547054930927)*x_ref[1]
        ref[4,0,1]=(-0.976995206698)*x_ref[0]+(1.251791534)*x_ref[1]
        ref[4,1,0]=(-1.13243662763)*x_ref[0]+(0.56739947752)*x_ref[1]
        ref[4,1,1]=(-0.543613180121)*x_ref[0]+(0.649805921416)*x_ref[1]
        ref[5,0,0]=(-0.0690689383083)*x_ref[0]+(0.818563064557)*x_ref[1]
        ref[5,0,1]=(0.362899720121)*x_ref[0]+(-0.112604112729)*x_ref[1]
        ref[5,1,0]=(-0.291756903179)*x_ref[0]+(-0.538221294595)*x_ref[1]
        ref[5,1,1]=(0.60406331286)*x_ref[0]+(0.228744896658)*x_ref[1]
      else:
        s[0,0,0]=(0.811538724853)*x[0]+(-0.347250364239)*x[1]+(-0.116011094391)*x[2]
        s[0,0,1]=(1.69112205586)*x[0]+(-1.04197300041)*x[1]+(0.849118123462)*x[2]
        s[0,1,0]=(1.30816943778)*x[0]+(1.01636941747)*x[1]+(-0.849746116393)*x[2]
        s[0,1,1]=(0.888683598925)*x[0]+(-1.60930356378)*x[1]+(0.522151570265)*x[2]
        s[1,0,0]=(1.03821316703)*x[0]+(-1.19647941504)*x[1]+(-1.08543246224)*x[2]
        s[1,0,1]=(-0.314182278588)*x[0]+(0.144825414517)*x[1]+(0.162613879231)*x[2]
        s[1,1,0]=(0.0416209427382)*x[0]+(0.0390092175082)*x[1]+(1.55616366687)*x[2]
        s[1,1,1]=(-0.577251716148)*x[0]+(0.717529140134)*x[1]+(-0.567199280535)*x[2]
        s[2,0,0]=(1.26716303575)*x[0]+(0.119576983216)*x[1]+(-0.602181033831)*x[2]
        s[2,0,1]=(0.533935330879)*x[0]+(0.292003869006)*x[1]+(-0.0917835680855)*x[2]
        s[2,1,0]=(-1.50218600857)*x[0]+(1.23418550746)*x[1]+(-0.600240530145)*x[2]
        s[2,1,1]=(0.689151741161)*x[0]+(0.909077564156)*x[1]+(0.924638961344)*x[2]
        s[3,0,0]=(-0.763411711557)*x[0]+(-0.862413521307)*x[1]+(1.49095932554)*x[2]
        s[3,0,1]=(-0.215589144977)*x[0]+(0.305301450314)*x[1]+(-0.468853406603)*x[2]
        s[3,1,0]=(-0.433045929433)*x[0]+(-0.907669929171)*x[1]+(0.917462300039)*x[2]
        s[3,1,1]=(0.229593212783)*x[0]+(-1.79371444035)*x[1]+(-0.848409349534)*x[2]
        s[4,0,0]=(1.40249316312)*x[0]+(-0.766609553683)*x[1]+(0.337911083882)*x[2]
        s[4,0,1]=(0.728650674867)*x[0]+(0.631222570369)*x[1]+(1.2491315131)*x[2]
        s[4,1,0]=(1.03998601102)*x[0]+(-1.81204917532)*x[1]+(0.781499164268)*x[2]
        s[4,1,1]=(-0.151999545303)*x[0]+(-0.767585861167)*x[1]+(1.17261654125)*x[2]
        s[5,0,0]=(0.0632249575346)*x[0]+(0.333565970302)*x[1]+(-1.66679001354)*x[2]
        s[5,0,1]=(0.945677231929)*x[0]+(-1.15936834775)*x[1]+(-0.644891644457)*x[2]
        s[5,1,0]=(1.08939087839)*x[0]+(0.687932839014)*x[1]+(-0.473426758917)*x[2]
        s[5,1,1]=(0.0349915672648)*x[0]+(-1.48593425129)*x[1]+(0.145421756236)*x[2]
        ref[0,0,0]=(0.811538724853)*x_ref[0]+(-0.347250364239)*x_ref[1]+(-0.116011094391)*x_ref[2]
        ref[0,0,1]=(1.69112205586)*x_ref[0]+(-1.04197300041)*x_ref[1]+(0.849118123462)*x_ref[2]
        ref[0,1,0]=(1.30816943778)*x_ref[0]+(1.01636941747)*x_ref[1]+(-0.849746116393)*x_ref[2]
        ref[0,1,1]=(0.888683598925)*x_ref[0]+(-1.60930356378)*x_ref[1]+(0.522151570265)*x_ref[2]
        ref[1,0,0]=(1.03821316703)*x_ref[0]+(-1.19647941504)*x_ref[1]+(-1.08543246224)*x_ref[2]
        ref[1,0,1]=(-0.314182278588)*x_ref[0]+(0.144825414517)*x_ref[1]+(0.162613879231)*x_ref[2]
        ref[1,1,0]=(0.0416209427382)*x_ref[0]+(0.0390092175082)*x_ref[1]+(1.55616366687)*x_ref[2]
        ref[1,1,1]=(-0.577251716148)*x_ref[0]+(0.717529140134)*x_ref[1]+(-0.567199280535)*x_ref[2]
        ref[2,0,0]=(1.26716303575)*x_ref[0]+(0.119576983216)*x_ref[1]+(-0.602181033831)*x_ref[2]
        ref[2,0,1]=(0.533935330879)*x_ref[0]+(0.292003869006)*x_ref[1]+(-0.0917835680855)*x_ref[2]
        ref[2,1,0]=(-1.50218600857)*x_ref[0]+(1.23418550746)*x_ref[1]+(-0.600240530145)*x_ref[2]
        ref[2,1,1]=(0.689151741161)*x_ref[0]+(0.909077564156)*x_ref[1]+(0.924638961344)*x_ref[2]
        ref[3,0,0]=(-0.763411711557)*x_ref[0]+(-0.862413521307)*x_ref[1]+(1.49095932554)*x_ref[2]
        ref[3,0,1]=(-0.215589144977)*x_ref[0]+(0.305301450314)*x_ref[1]+(-0.468853406603)*x_ref[2]
        ref[3,1,0]=(-0.433045929433)*x_ref[0]+(-0.907669929171)*x_ref[1]+(0.917462300039)*x_ref[2]
        ref[3,1,1]=(0.229593212783)*x_ref[0]+(-1.79371444035)*x_ref[1]+(-0.848409349534)*x_ref[2]
        ref[4,0,0]=(1.40249316312)*x_ref[0]+(-0.766609553683)*x_ref[1]+(0.337911083882)*x_ref[2]
        ref[4,0,1]=(0.728650674867)*x_ref[0]+(0.631222570369)*x_ref[1]+(1.2491315131)*x_ref[2]
        ref[4,1,0]=(1.03998601102)*x_ref[0]+(-1.81204917532)*x_ref[1]+(0.781499164268)*x_ref[2]
        ref[4,1,1]=(-0.151999545303)*x_ref[0]+(-0.767585861167)*x_ref[1]+(1.17261654125)*x_ref[2]
        ref[5,0,0]=(0.0632249575346)*x_ref[0]+(0.333565970302)*x_ref[1]+(-1.66679001354)*x_ref[2]
        ref[5,0,1]=(0.945677231929)*x_ref[0]+(-1.15936834775)*x_ref[1]+(-0.644891644457)*x_ref[2]
        ref[5,1,0]=(1.08939087839)*x_ref[0]+(0.687932839014)*x_ref[1]+(-0.473426758917)*x_ref[2]
        ref[5,1,1]=(0.0349915672648)*x_ref[0]+(-1.48593425129)*x_ref[1]+(0.145421756236)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_ReducedContinuousFunction_rank4(self):
      """
      tests interpolate for rank 4 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedContinuousFunction onto ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5, 3, 2),dim=dim)
      s=Data(0,(4, 5, 3, 2),w)
      ref=Data(0,(4, 5, 3, 2),w_ref)
      if dim==2:
        s[0,0,0,0]=(0.609727033659)*x[0]+(-0.901858940633)*x[1]
        s[0,0,0,1]=(-0.221259461243)*x[0]+(0.351136340908)*x[1]
        s[0,0,1,0]=(0.497173734282)*x[0]+(0.899142906899)*x[1]
        s[0,0,1,1]=(-1.55206649551)*x[0]+(-0.680257384044)*x[1]
        s[0,0,2,0]=(-0.932033638033)*x[0]+(-0.46503758126)*x[1]
        s[0,0,2,1]=(-0.0372650123525)*x[0]+(0.0173714029098)*x[1]
        s[0,1,0,0]=(0.855963098465)*x[0]+(0.760176320898)*x[1]
        s[0,1,0,1]=(1.37546120327)*x[0]+(1.72916158331)*x[1]
        s[0,1,1,0]=(0.25469861613)*x[0]+(0.71445716033)*x[1]
        s[0,1,1,1]=(0.198455659765)*x[0]+(0.0410276829495)*x[1]
        s[0,1,2,0]=(-0.628080427667)*x[0]+(-0.0197206025984)*x[1]
        s[0,1,2,1]=(1.26759029895)*x[0]+(1.15914953253)*x[1]
        s[0,2,0,0]=(-1.43203053441)*x[0]+(0.366301170069)*x[1]
        s[0,2,0,1]=(0.420527802229)*x[0]+(-0.709209747415)*x[1]
        s[0,2,1,0]=(0.399042264829)*x[0]+(0.0453132754731)*x[1]
        s[0,2,1,1]=(1.2074346062)*x[0]+(-1.14163410992)*x[1]
        s[0,2,2,0]=(-0.00270984158569)*x[0]+(-1.63216200182)*x[1]
        s[0,2,2,1]=(0.174276088847)*x[0]+(-1.04789472231)*x[1]
        s[0,3,0,0]=(-0.47704729036)*x[0]+(-1.2550637204)*x[1]
        s[0,3,0,1]=(0.348356873191)*x[0]+(-0.796668578265)*x[1]
        s[0,3,1,0]=(-0.52724725097)*x[0]+(1.26163123592)*x[1]
        s[0,3,1,1]=(-1.52681799261)*x[0]+(0.936041931735)*x[1]
        s[0,3,2,0]=(1.33689673219)*x[0]+(-1.34901214018)*x[1]
        s[0,3,2,1]=(1.57651286047)*x[0]+(0.245413762864)*x[1]
        s[0,4,0,0]=(0.194500181498)*x[0]+(-0.0231951844441)*x[1]
        s[0,4,0,1]=(-1.49372076697)*x[0]+(-0.111369678181)*x[1]
        s[0,4,1,0]=(-0.74121646836)*x[0]+(-0.817740156307)*x[1]
        s[0,4,1,1]=(-0.233643397123)*x[0]+(0.672715068796)*x[1]
        s[0,4,2,0]=(1.05850947579)*x[0]+(0.386484386191)*x[1]
        s[0,4,2,1]=(1.45105922963)*x[0]+(-0.367458625996)*x[1]
        s[1,0,0,0]=(-0.391803788669)*x[0]+(0.797592300662)*x[1]
        s[1,0,0,1]=(-0.205299072883)*x[0]+(0.677427935558)*x[1]
        s[1,0,1,0]=(1.56225329031)*x[0]+(1.17468092515)*x[1]
        s[1,0,1,1]=(1.4586005658)*x[0]+(1.36481062943)*x[1]
        s[1,0,2,0]=(-1.53281881413)*x[0]+(0.207028833974)*x[1]
        s[1,0,2,1]=(0.0469239691648)*x[0]+(1.03547812705)*x[1]
        s[1,1,0,0]=(1.39614335897)*x[0]+(0.82046489695)*x[1]
        s[1,1,0,1]=(-0.775462741637)*x[0]+(-0.275081761646)*x[1]
        s[1,1,1,0]=(0.173353028608)*x[0]+(-1.05620431816)*x[1]
        s[1,1,1,1]=(-0.464192870247)*x[0]+(0.212204784206)*x[1]
        s[1,1,2,0]=(-0.383813531866)*x[0]+(-0.0773759374234)*x[1]
        s[1,1,2,1]=(-1.07811385759)*x[0]+(-0.457232330819)*x[1]
        s[1,2,0,0]=(-0.295869147576)*x[0]+(0.166843915631)*x[1]
        s[1,2,0,1]=(0.571421034325)*x[0]+(1.46722583245)*x[1]
        s[1,2,1,0]=(-0.34494944257)*x[0]+(-0.286867052491)*x[1]
        s[1,2,1,1]=(1.43128885761)*x[0]+(0.21277887035)*x[1]
        s[1,2,2,0]=(-1.19397187873)*x[0]+(0.126397665227)*x[1]
        s[1,2,2,1]=(0.171368751954)*x[0]+(-1.12100254625)*x[1]
        s[1,3,0,0]=(0.0975333077604)*x[0]+(0.583524660188)*x[1]
        s[1,3,0,1]=(0.0271450955458)*x[0]+(0.368978332867)*x[1]
        s[1,3,1,0]=(1.22806968486)*x[0]+(-0.569932748468)*x[1]
        s[1,3,1,1]=(-0.691989839521)*x[0]+(-0.829822182922)*x[1]
        s[1,3,2,0]=(0.743776534098)*x[0]+(1.3668937127)*x[1]
        s[1,3,2,1]=(0.192206575204)*x[0]+(0.96576702824)*x[1]
        s[1,4,0,0]=(0.149381460079)*x[0]+(-0.987875538036)*x[1]
        s[1,4,0,1]=(-0.706855836683)*x[0]+(0.0410884211631)*x[1]
        s[1,4,1,0]=(-0.222650274)*x[0]+(0.59589242048)*x[1]
        s[1,4,1,1]=(0.541489186743)*x[0]+(-1.22543996074)*x[1]
        s[1,4,2,0]=(0.630234824652)*x[0]+(0.508436575179)*x[1]
        s[1,4,2,1]=(0.990062359294)*x[0]+(-0.639884693407)*x[1]
        s[2,0,0,0]=(0.378887461733)*x[0]+(1.71999809344)*x[1]
        s[2,0,0,1]=(0.182412683544)*x[0]+(-0.401183243877)*x[1]
        s[2,0,1,0]=(0.325775290396)*x[0]+(-0.656088786126)*x[1]
        s[2,0,1,1]=(0.316355229742)*x[0]+(1.54087545195)*x[1]
        s[2,0,2,0]=(1.14203881183)*x[0]+(0.703207191364)*x[1]
        s[2,0,2,1]=(-1.05211790226)*x[0]+(1.68596483036)*x[1]
        s[2,1,0,0]=(0.704109387934)*x[0]+(-0.589003921574)*x[1]
        s[2,1,0,1]=(-0.382489922028)*x[0]+(0.387647642942)*x[1]
        s[2,1,1,0]=(-0.0343434958891)*x[0]+(-0.119176404279)*x[1]
        s[2,1,1,1]=(0.567853527929)*x[0]+(1.32523261969)*x[1]
        s[2,1,2,0]=(1.03496000653)*x[0]+(0.110203591459)*x[1]
        s[2,1,2,1]=(0.560149997806)*x[0]+(-1.62893742873)*x[1]
        s[2,2,0,0]=(1.1801792312)*x[0]+(1.1912385935)*x[1]
        s[2,2,0,1]=(0.936932817634)*x[0]+(-1.49082021974)*x[1]
        s[2,2,1,0]=(-0.305582892069)*x[0]+(-0.267235698563)*x[1]
        s[2,2,1,1]=(0.107449730184)*x[0]+(-0.325267993615)*x[1]
        s[2,2,2,0]=(-1.08118986347)*x[0]+(0.144190727334)*x[1]
        s[2,2,2,1]=(1.44755060146)*x[0]+(-0.127612338817)*x[1]
        s[2,3,0,0]=(-0.666943112999)*x[0]+(-0.780402371989)*x[1]
        s[2,3,0,1]=(0.724206717771)*x[0]+(-0.523593908278)*x[1]
        s[2,3,1,0]=(1.86101197199)*x[0]+(-0.710894911302)*x[1]
        s[2,3,1,1]=(-1.74409232023)*x[0]+(0.407821816271)*x[1]
        s[2,3,2,0]=(0.646304172575)*x[0]+(0.0132628778543)*x[1]
        s[2,3,2,1]=(-0.746041438673)*x[0]+(0.516304872741)*x[1]
        s[2,4,0,0]=(0.0326900201283)*x[0]+(-0.533021156401)*x[1]
        s[2,4,0,1]=(0.088598956981)*x[0]+(0.764307970269)*x[1]
        s[2,4,1,0]=(-0.749553477756)*x[0]+(0.379646792939)*x[1]
        s[2,4,1,1]=(-0.243635321378)*x[0]+(-0.470022600261)*x[1]
        s[2,4,2,0]=(0.586398757247)*x[0]+(-0.116982897683)*x[1]
        s[2,4,2,1]=(-0.989012121879)*x[0]+(-1.25006535257)*x[1]
        s[3,0,0,0]=(1.25228306081)*x[0]+(-0.0932717035896)*x[1]
        s[3,0,0,1]=(1.5975516771)*x[0]+(-0.177733286111)*x[1]
        s[3,0,1,0]=(-0.659910070911)*x[0]+(-0.0999498944812)*x[1]
        s[3,0,1,1]=(-0.299870273662)*x[0]+(-1.22977656744)*x[1]
        s[3,0,2,0]=(-0.158211581143)*x[0]+(-0.141421380503)*x[1]
        s[3,0,2,1]=(-1.48226452788)*x[0]+(-0.731264943309)*x[1]
        s[3,1,0,0]=(-1.64589827853)*x[0]+(1.19692573998)*x[1]
        s[3,1,0,1]=(0.361329655357)*x[0]+(0.234310482171)*x[1]
        s[3,1,1,0]=(-1.14077649516)*x[0]+(0.132657084065)*x[1]
        s[3,1,1,1]=(1.41067975663)*x[0]+(0.818243117884)*x[1]
        s[3,1,2,0]=(-1.53137884342)*x[0]+(-0.0127805333093)*x[1]
        s[3,1,2,1]=(0.240455755486)*x[0]+(-1.02361753893)*x[1]
        s[3,2,0,0]=(-0.334803473072)*x[0]+(-0.807452510423)*x[1]
        s[3,2,0,1]=(0.617324536974)*x[0]+(-0.733163471774)*x[1]
        s[3,2,1,0]=(-0.634580050897)*x[0]+(0.362106278168)*x[1]
        s[3,2,1,1]=(-0.21525531818)*x[0]+(0.982902540898)*x[1]
        s[3,2,2,0]=(-0.590682803195)*x[0]+(-1.27537880632)*x[1]
        s[3,2,2,1]=(0.142578313438)*x[0]+(0.212341687291)*x[1]
        s[3,3,0,0]=(-0.758345317216)*x[0]+(-0.971914631359)*x[1]
        s[3,3,0,1]=(0.569463909234)*x[0]+(0.498848917338)*x[1]
        s[3,3,1,0]=(-0.771456575632)*x[0]+(-0.130456072944)*x[1]
        s[3,3,1,1]=(0.811799269245)*x[0]+(-0.829647203396)*x[1]
        s[3,3,2,0]=(-0.814979427693)*x[0]+(-0.0416092796813)*x[1]
        s[3,3,2,1]=(0.933434189127)*x[0]+(0.60794704936)*x[1]
        s[3,4,0,0]=(0.160280566478)*x[0]+(-0.0141616040181)*x[1]
        s[3,4,0,1]=(-1.34835766268)*x[0]+(0.178951225861)*x[1]
        s[3,4,1,0]=(1.67632814658)*x[0]+(1.31316096483)*x[1]
        s[3,4,1,1]=(-0.603403275376)*x[0]+(0.0607753526431)*x[1]
        s[3,4,2,0]=(-0.178440295473)*x[0]+(0.318102694371)*x[1]
        s[3,4,2,1]=(0.781285160749)*x[0]+(0.625651915981)*x[1]
        ref[0,0,0,0]=(0.609727033659)*x_ref[0]+(-0.901858940633)*x_ref[1]
        ref[0,0,0,1]=(-0.221259461243)*x_ref[0]+(0.351136340908)*x_ref[1]
        ref[0,0,1,0]=(0.497173734282)*x_ref[0]+(0.899142906899)*x_ref[1]
        ref[0,0,1,1]=(-1.55206649551)*x_ref[0]+(-0.680257384044)*x_ref[1]
        ref[0,0,2,0]=(-0.932033638033)*x_ref[0]+(-0.46503758126)*x_ref[1]
        ref[0,0,2,1]=(-0.0372650123525)*x_ref[0]+(0.0173714029098)*x_ref[1]
        ref[0,1,0,0]=(0.855963098465)*x_ref[0]+(0.760176320898)*x_ref[1]
        ref[0,1,0,1]=(1.37546120327)*x_ref[0]+(1.72916158331)*x_ref[1]
        ref[0,1,1,0]=(0.25469861613)*x_ref[0]+(0.71445716033)*x_ref[1]
        ref[0,1,1,1]=(0.198455659765)*x_ref[0]+(0.0410276829495)*x_ref[1]
        ref[0,1,2,0]=(-0.628080427667)*x_ref[0]+(-0.0197206025984)*x_ref[1]
        ref[0,1,2,1]=(1.26759029895)*x_ref[0]+(1.15914953253)*x_ref[1]
        ref[0,2,0,0]=(-1.43203053441)*x_ref[0]+(0.366301170069)*x_ref[1]
        ref[0,2,0,1]=(0.420527802229)*x_ref[0]+(-0.709209747415)*x_ref[1]
        ref[0,2,1,0]=(0.399042264829)*x_ref[0]+(0.0453132754731)*x_ref[1]
        ref[0,2,1,1]=(1.2074346062)*x_ref[0]+(-1.14163410992)*x_ref[1]
        ref[0,2,2,0]=(-0.00270984158569)*x_ref[0]+(-1.63216200182)*x_ref[1]
        ref[0,2,2,1]=(0.174276088847)*x_ref[0]+(-1.04789472231)*x_ref[1]
        ref[0,3,0,0]=(-0.47704729036)*x_ref[0]+(-1.2550637204)*x_ref[1]
        ref[0,3,0,1]=(0.348356873191)*x_ref[0]+(-0.796668578265)*x_ref[1]
        ref[0,3,1,0]=(-0.52724725097)*x_ref[0]+(1.26163123592)*x_ref[1]
        ref[0,3,1,1]=(-1.52681799261)*x_ref[0]+(0.936041931735)*x_ref[1]
        ref[0,3,2,0]=(1.33689673219)*x_ref[0]+(-1.34901214018)*x_ref[1]
        ref[0,3,2,1]=(1.57651286047)*x_ref[0]+(0.245413762864)*x_ref[1]
        ref[0,4,0,0]=(0.194500181498)*x_ref[0]+(-0.0231951844441)*x_ref[1]
        ref[0,4,0,1]=(-1.49372076697)*x_ref[0]+(-0.111369678181)*x_ref[1]
        ref[0,4,1,0]=(-0.74121646836)*x_ref[0]+(-0.817740156307)*x_ref[1]
        ref[0,4,1,1]=(-0.233643397123)*x_ref[0]+(0.672715068796)*x_ref[1]
        ref[0,4,2,0]=(1.05850947579)*x_ref[0]+(0.386484386191)*x_ref[1]
        ref[0,4,2,1]=(1.45105922963)*x_ref[0]+(-0.367458625996)*x_ref[1]
        ref[1,0,0,0]=(-0.391803788669)*x_ref[0]+(0.797592300662)*x_ref[1]
        ref[1,0,0,1]=(-0.205299072883)*x_ref[0]+(0.677427935558)*x_ref[1]
        ref[1,0,1,0]=(1.56225329031)*x_ref[0]+(1.17468092515)*x_ref[1]
        ref[1,0,1,1]=(1.4586005658)*x_ref[0]+(1.36481062943)*x_ref[1]
        ref[1,0,2,0]=(-1.53281881413)*x_ref[0]+(0.207028833974)*x_ref[1]
        ref[1,0,2,1]=(0.0469239691648)*x_ref[0]+(1.03547812705)*x_ref[1]
        ref[1,1,0,0]=(1.39614335897)*x_ref[0]+(0.82046489695)*x_ref[1]
        ref[1,1,0,1]=(-0.775462741637)*x_ref[0]+(-0.275081761646)*x_ref[1]
        ref[1,1,1,0]=(0.173353028608)*x_ref[0]+(-1.05620431816)*x_ref[1]
        ref[1,1,1,1]=(-0.464192870247)*x_ref[0]+(0.212204784206)*x_ref[1]
        ref[1,1,2,0]=(-0.383813531866)*x_ref[0]+(-0.0773759374234)*x_ref[1]
        ref[1,1,2,1]=(-1.07811385759)*x_ref[0]+(-0.457232330819)*x_ref[1]
        ref[1,2,0,0]=(-0.295869147576)*x_ref[0]+(0.166843915631)*x_ref[1]
        ref[1,2,0,1]=(0.571421034325)*x_ref[0]+(1.46722583245)*x_ref[1]
        ref[1,2,1,0]=(-0.34494944257)*x_ref[0]+(-0.286867052491)*x_ref[1]
        ref[1,2,1,1]=(1.43128885761)*x_ref[0]+(0.21277887035)*x_ref[1]
        ref[1,2,2,0]=(-1.19397187873)*x_ref[0]+(0.126397665227)*x_ref[1]
        ref[1,2,2,1]=(0.171368751954)*x_ref[0]+(-1.12100254625)*x_ref[1]
        ref[1,3,0,0]=(0.0975333077604)*x_ref[0]+(0.583524660188)*x_ref[1]
        ref[1,3,0,1]=(0.0271450955458)*x_ref[0]+(0.368978332867)*x_ref[1]
        ref[1,3,1,0]=(1.22806968486)*x_ref[0]+(-0.569932748468)*x_ref[1]
        ref[1,3,1,1]=(-0.691989839521)*x_ref[0]+(-0.829822182922)*x_ref[1]
        ref[1,3,2,0]=(0.743776534098)*x_ref[0]+(1.3668937127)*x_ref[1]
        ref[1,3,2,1]=(0.192206575204)*x_ref[0]+(0.96576702824)*x_ref[1]
        ref[1,4,0,0]=(0.149381460079)*x_ref[0]+(-0.987875538036)*x_ref[1]
        ref[1,4,0,1]=(-0.706855836683)*x_ref[0]+(0.0410884211631)*x_ref[1]
        ref[1,4,1,0]=(-0.222650274)*x_ref[0]+(0.59589242048)*x_ref[1]
        ref[1,4,1,1]=(0.541489186743)*x_ref[0]+(-1.22543996074)*x_ref[1]
        ref[1,4,2,0]=(0.630234824652)*x_ref[0]+(0.508436575179)*x_ref[1]
        ref[1,4,2,1]=(0.990062359294)*x_ref[0]+(-0.639884693407)*x_ref[1]
        ref[2,0,0,0]=(0.378887461733)*x_ref[0]+(1.71999809344)*x_ref[1]
        ref[2,0,0,1]=(0.182412683544)*x_ref[0]+(-0.401183243877)*x_ref[1]
        ref[2,0,1,0]=(0.325775290396)*x_ref[0]+(-0.656088786126)*x_ref[1]
        ref[2,0,1,1]=(0.316355229742)*x_ref[0]+(1.54087545195)*x_ref[1]
        ref[2,0,2,0]=(1.14203881183)*x_ref[0]+(0.703207191364)*x_ref[1]
        ref[2,0,2,1]=(-1.05211790226)*x_ref[0]+(1.68596483036)*x_ref[1]
        ref[2,1,0,0]=(0.704109387934)*x_ref[0]+(-0.589003921574)*x_ref[1]
        ref[2,1,0,1]=(-0.382489922028)*x_ref[0]+(0.387647642942)*x_ref[1]
        ref[2,1,1,0]=(-0.0343434958891)*x_ref[0]+(-0.119176404279)*x_ref[1]
        ref[2,1,1,1]=(0.567853527929)*x_ref[0]+(1.32523261969)*x_ref[1]
        ref[2,1,2,0]=(1.03496000653)*x_ref[0]+(0.110203591459)*x_ref[1]
        ref[2,1,2,1]=(0.560149997806)*x_ref[0]+(-1.62893742873)*x_ref[1]
        ref[2,2,0,0]=(1.1801792312)*x_ref[0]+(1.1912385935)*x_ref[1]
        ref[2,2,0,1]=(0.936932817634)*x_ref[0]+(-1.49082021974)*x_ref[1]
        ref[2,2,1,0]=(-0.305582892069)*x_ref[0]+(-0.267235698563)*x_ref[1]
        ref[2,2,1,1]=(0.107449730184)*x_ref[0]+(-0.325267993615)*x_ref[1]
        ref[2,2,2,0]=(-1.08118986347)*x_ref[0]+(0.144190727334)*x_ref[1]
        ref[2,2,2,1]=(1.44755060146)*x_ref[0]+(-0.127612338817)*x_ref[1]
        ref[2,3,0,0]=(-0.666943112999)*x_ref[0]+(-0.780402371989)*x_ref[1]
        ref[2,3,0,1]=(0.724206717771)*x_ref[0]+(-0.523593908278)*x_ref[1]
        ref[2,3,1,0]=(1.86101197199)*x_ref[0]+(-0.710894911302)*x_ref[1]
        ref[2,3,1,1]=(-1.74409232023)*x_ref[0]+(0.407821816271)*x_ref[1]
        ref[2,3,2,0]=(0.646304172575)*x_ref[0]+(0.0132628778543)*x_ref[1]
        ref[2,3,2,1]=(-0.746041438673)*x_ref[0]+(0.516304872741)*x_ref[1]
        ref[2,4,0,0]=(0.0326900201283)*x_ref[0]+(-0.533021156401)*x_ref[1]
        ref[2,4,0,1]=(0.088598956981)*x_ref[0]+(0.764307970269)*x_ref[1]
        ref[2,4,1,0]=(-0.749553477756)*x_ref[0]+(0.379646792939)*x_ref[1]
        ref[2,4,1,1]=(-0.243635321378)*x_ref[0]+(-0.470022600261)*x_ref[1]
        ref[2,4,2,0]=(0.586398757247)*x_ref[0]+(-0.116982897683)*x_ref[1]
        ref[2,4,2,1]=(-0.989012121879)*x_ref[0]+(-1.25006535257)*x_ref[1]
        ref[3,0,0,0]=(1.25228306081)*x_ref[0]+(-0.0932717035896)*x_ref[1]
        ref[3,0,0,1]=(1.5975516771)*x_ref[0]+(-0.177733286111)*x_ref[1]
        ref[3,0,1,0]=(-0.659910070911)*x_ref[0]+(-0.0999498944812)*x_ref[1]
        ref[3,0,1,1]=(-0.299870273662)*x_ref[0]+(-1.22977656744)*x_ref[1]
        ref[3,0,2,0]=(-0.158211581143)*x_ref[0]+(-0.141421380503)*x_ref[1]
        ref[3,0,2,1]=(-1.48226452788)*x_ref[0]+(-0.731264943309)*x_ref[1]
        ref[3,1,0,0]=(-1.64589827853)*x_ref[0]+(1.19692573998)*x_ref[1]
        ref[3,1,0,1]=(0.361329655357)*x_ref[0]+(0.234310482171)*x_ref[1]
        ref[3,1,1,0]=(-1.14077649516)*x_ref[0]+(0.132657084065)*x_ref[1]
        ref[3,1,1,1]=(1.41067975663)*x_ref[0]+(0.818243117884)*x_ref[1]
        ref[3,1,2,0]=(-1.53137884342)*x_ref[0]+(-0.0127805333093)*x_ref[1]
        ref[3,1,2,1]=(0.240455755486)*x_ref[0]+(-1.02361753893)*x_ref[1]
        ref[3,2,0,0]=(-0.334803473072)*x_ref[0]+(-0.807452510423)*x_ref[1]
        ref[3,2,0,1]=(0.617324536974)*x_ref[0]+(-0.733163471774)*x_ref[1]
        ref[3,2,1,0]=(-0.634580050897)*x_ref[0]+(0.362106278168)*x_ref[1]
        ref[3,2,1,1]=(-0.21525531818)*x_ref[0]+(0.982902540898)*x_ref[1]
        ref[3,2,2,0]=(-0.590682803195)*x_ref[0]+(-1.27537880632)*x_ref[1]
        ref[3,2,2,1]=(0.142578313438)*x_ref[0]+(0.212341687291)*x_ref[1]
        ref[3,3,0,0]=(-0.758345317216)*x_ref[0]+(-0.971914631359)*x_ref[1]
        ref[3,3,0,1]=(0.569463909234)*x_ref[0]+(0.498848917338)*x_ref[1]
        ref[3,3,1,0]=(-0.771456575632)*x_ref[0]+(-0.130456072944)*x_ref[1]
        ref[3,3,1,1]=(0.811799269245)*x_ref[0]+(-0.829647203396)*x_ref[1]
        ref[3,3,2,0]=(-0.814979427693)*x_ref[0]+(-0.0416092796813)*x_ref[1]
        ref[3,3,2,1]=(0.933434189127)*x_ref[0]+(0.60794704936)*x_ref[1]
        ref[3,4,0,0]=(0.160280566478)*x_ref[0]+(-0.0141616040181)*x_ref[1]
        ref[3,4,0,1]=(-1.34835766268)*x_ref[0]+(0.178951225861)*x_ref[1]
        ref[3,4,1,0]=(1.67632814658)*x_ref[0]+(1.31316096483)*x_ref[1]
        ref[3,4,1,1]=(-0.603403275376)*x_ref[0]+(0.0607753526431)*x_ref[1]
        ref[3,4,2,0]=(-0.178440295473)*x_ref[0]+(0.318102694371)*x_ref[1]
        ref[3,4,2,1]=(0.781285160749)*x_ref[0]+(0.625651915981)*x_ref[1]
      else:
        s[0,0,0,0]=(0.847845815599)*x[0]+(-0.610234211476)*x[1]+(-0.627625510082)*x[2]
        s[0,0,0,1]=(0.0126184651041)*x[0]+(-0.0651294063564)*x[1]+(-0.274615935201)*x[2]
        s[0,0,1,0]=(-0.545925417969)*x[0]+(0.421929302201)*x[1]+(-0.910418625383)*x[2]
        s[0,0,1,1]=(1.58754252937)*x[0]+(0.110111591047)*x[1]+(-1.64780563736)*x[2]
        s[0,0,2,0]=(-0.510331643992)*x[0]+(0.827685326542)*x[1]+(1.31447962917)*x[2]
        s[0,0,2,1]=(-1.29575211142)*x[0]+(-0.318096803401)*x[1]+(-0.274660613777)*x[2]
        s[0,1,0,0]=(0.77444944835)*x[0]+(-1.0893411122)*x[1]+(0.331702744143)*x[2]
        s[0,1,0,1]=(0.624693916432)*x[0]+(-0.306552086758)*x[1]+(-0.845524417807)*x[2]
        s[0,1,1,0]=(0.0219492668609)*x[0]+(-1.13712865204)*x[1]+(-0.193051406533)*x[2]
        s[0,1,1,1]=(-1.32696889246)*x[0]+(1.63857177904)*x[1]+(0.194095476936)*x[2]
        s[0,1,2,0]=(1.19169978642)*x[0]+(-1.88274910505)*x[1]+(0.321438561021)*x[2]
        s[0,1,2,1]=(-0.765748081809)*x[0]+(-0.163422665563)*x[1]+(0.256721885141)*x[2]
        s[0,2,0,0]=(-0.863281319991)*x[0]+(-0.021828308413)*x[1]+(0.868312889989)*x[2]
        s[0,2,0,1]=(-0.57479341478)*x[0]+(0.404861121887)*x[1]+(-0.274037007699)*x[2]
        s[0,2,1,0]=(-0.529171478131)*x[0]+(0.836001872018)*x[1]+(-1.12226189724)*x[2]
        s[0,2,1,1]=(-0.821929160411)*x[0]+(0.0466762602824)*x[1]+(-0.0514331500765)*x[2]
        s[0,2,2,0]=(-0.605940618544)*x[0]+(-1.54379919132)*x[1]+(1.63243021269)*x[2]
        s[0,2,2,1]=(-1.49695860655)*x[0]+(-0.601345463475)*x[1]+(-0.753443592162)*x[2]
        s[0,3,0,0]=(1.33514081849)*x[0]+(0.955849232345)*x[1]+(-0.872221354599)*x[2]
        s[0,3,0,1]=(0.939731890901)*x[0]+(-1.16116012822)*x[1]+(-0.601602002743)*x[2]
        s[0,3,1,0]=(-0.675545889094)*x[0]+(0.736755090077)*x[1]+(-0.437370345974)*x[2]
        s[0,3,1,1]=(-1.17899966213)*x[0]+(0.422125349488)*x[1]+(-0.815649229407)*x[2]
        s[0,3,2,0]=(0.0773790438387)*x[0]+(0.148378160687)*x[1]+(-1.0156040466)*x[2]
        s[0,3,2,1]=(0.519422925349)*x[0]+(0.189993892084)*x[1]+(1.59654399673)*x[2]
        s[0,4,0,0]=(-0.0593352727817)*x[0]+(0.0865384232804)*x[1]+(-0.316122229956)*x[2]
        s[0,4,0,1]=(0.935220504817)*x[0]+(0.868143343952)*x[1]+(-0.470286174039)*x[2]
        s[0,4,1,0]=(0.763506086212)*x[0]+(-0.0756795330833)*x[1]+(1.06812672934)*x[2]
        s[0,4,1,1]=(-0.446580256351)*x[0]+(0.653986880233)*x[1]+(0.634364696873)*x[2]
        s[0,4,2,0]=(-0.72871248403)*x[0]+(0.0839150289011)*x[1]+(0.178899292496)*x[2]
        s[0,4,2,1]=(1.30808288445)*x[0]+(0.548931211195)*x[1]+(-0.971585668771)*x[2]
        s[1,0,0,0]=(-0.737171903787)*x[0]+(-0.137110160891)*x[1]+(1.24817883553)*x[2]
        s[1,0,0,1]=(0.589824248199)*x[0]+(-0.191688810075)*x[1]+(1.33820051734)*x[2]
        s[1,0,1,0]=(-0.598063103136)*x[0]+(0.347705502684)*x[1]+(0.490747627936)*x[2]
        s[1,0,1,1]=(0.287829427303)*x[0]+(-0.853875346674)*x[1]+(-0.227722773909)*x[2]
        s[1,0,2,0]=(-1.22183236343)*x[0]+(-0.162406713796)*x[1]+(-0.436433884201)*x[2]
        s[1,0,2,1]=(0.929372688401)*x[0]+(0.469931776805)*x[1]+(0.739110972137)*x[2]
        s[1,1,0,0]=(0.391824586)*x[0]+(-0.551498198462)*x[1]+(-0.588935046796)*x[2]
        s[1,1,0,1]=(0.849805745827)*x[0]+(0.557121315313)*x[1]+(1.80284061615)*x[2]
        s[1,1,1,0]=(-1.62270432051)*x[0]+(-1.55943636923)*x[1]+(0.629204169009)*x[2]
        s[1,1,1,1]=(-0.191271118514)*x[0]+(-0.454718665581)*x[1]+(0.405403171722)*x[2]
        s[1,1,2,0]=(0.461750123507)*x[0]+(0.537594040674)*x[1]+(-0.500046587877)*x[2]
        s[1,1,2,1]=(1.07800193455)*x[0]+(0.0757746249706)*x[1]+(0.358223454759)*x[2]
        s[1,2,0,0]=(-1.31360669142)*x[0]+(0.908413426737)*x[1]+(0.986904165818)*x[2]
        s[1,2,0,1]=(-0.250344247716)*x[0]+(0.596746427887)*x[1]+(0.846846599186)*x[2]
        s[1,2,1,0]=(1.65761753214)*x[0]+(0.0969407749043)*x[1]+(0.197913502281)*x[2]
        s[1,2,1,1]=(0.808454446977)*x[0]+(-0.107176416129)*x[1]+(-0.115562549181)*x[2]
        s[1,2,2,0]=(-0.410108442793)*x[0]+(0.250496842214)*x[1]+(0.403806076052)*x[2]
        s[1,2,2,1]=(1.58110488679)*x[0]+(-0.171583112102)*x[1]+(0.441813895222)*x[2]
        s[1,3,0,0]=(-0.519973841295)*x[0]+(0.14862500493)*x[1]+(-0.0257159960311)*x[2]
        s[1,3,0,1]=(-0.710423889723)*x[0]+(0.0941990549739)*x[1]+(-0.147877062943)*x[2]
        s[1,3,1,0]=(-0.19917762038)*x[0]+(-1.06873791711)*x[1]+(1.37688275432)*x[2]
        s[1,3,1,1]=(-0.570477493665)*x[0]+(0.0705814270652)*x[1]+(-1.10186231057)*x[2]
        s[1,3,2,0]=(-1.04891017002)*x[0]+(-0.136784107636)*x[1]+(-1.65953016693)*x[2]
        s[1,3,2,1]=(-0.674805716398)*x[0]+(-0.211803508128)*x[1]+(-0.116463503974)*x[2]
        s[1,4,0,0]=(0.269217220297)*x[0]+(0.810722194772)*x[1]+(1.60977315112)*x[2]
        s[1,4,0,1]=(-0.0213286766746)*x[0]+(1.0974274553)*x[1]+(0.275633207025)*x[2]
        s[1,4,1,0]=(-0.601230245409)*x[0]+(-0.756418955933)*x[1]+(-0.46194303915)*x[2]
        s[1,4,1,1]=(-0.0907667209143)*x[0]+(-1.86623564673)*x[1]+(0.552729474701)*x[2]
        s[1,4,2,0]=(-1.09694541489)*x[0]+(-0.496193881555)*x[1]+(-0.0126133249168)*x[2]
        s[1,4,2,1]=(-0.648959246633)*x[0]+(-0.60655325639)*x[1]+(0.438258051102)*x[2]
        s[2,0,0,0]=(-0.100512547019)*x[0]+(-1.37291269079)*x[1]+(1.63084319876)*x[2]
        s[2,0,0,1]=(-0.50510326916)*x[0]+(0.308799737351)*x[1]+(1.63506761477)*x[2]
        s[2,0,1,0]=(0.708408519723)*x[0]+(-0.0759833841695)*x[1]+(-0.980974095374)*x[2]
        s[2,0,1,1]=(-1.59727324143)*x[0]+(0.995250195962)*x[1]+(0.169527316083)*x[2]
        s[2,0,2,0]=(0.766935817054)*x[0]+(0.455564399031)*x[1]+(0.605772951782)*x[2]
        s[2,0,2,1]=(-0.820755214894)*x[0]+(-0.43359371038)*x[1]+(-0.0942398466964)*x[2]
        s[2,1,0,0]=(0.390476682801)*x[0]+(1.11627447827)*x[1]+(0.670843405466)*x[2]
        s[2,1,0,1]=(-0.433594870483)*x[0]+(-0.940710004402)*x[1]+(1.10756041487)*x[2]
        s[2,1,1,0]=(-1.21880018868)*x[0]+(0.380422307344)*x[1]+(-0.864017731789)*x[2]
        s[2,1,1,1]=(-0.194168346974)*x[0]+(0.324183983028)*x[1]+(0.741456579501)*x[2]
        s[2,1,2,0]=(-0.134211025912)*x[0]+(0.497311041207)*x[1]+(0.33014196376)*x[2]
        s[2,1,2,1]=(-1.3501981937)*x[0]+(1.67601514859)*x[1]+(-1.60084725544)*x[2]
        s[2,2,0,0]=(-0.8367457236)*x[0]+(-1.15628524562)*x[1]+(0.342983125417)*x[2]
        s[2,2,0,1]=(0.956733778283)*x[0]+(-0.404880336577)*x[1]+(-0.350927185105)*x[2]
        s[2,2,1,0]=(0.854842449281)*x[0]+(-0.185001304765)*x[1]+(1.15245157168)*x[2]
        s[2,2,1,1]=(0.00584831242623)*x[0]+(-0.562950548984)*x[1]+(0.785766513431)*x[2]
        s[2,2,2,0]=(0.272759945164)*x[0]+(-0.568207376307)*x[1]+(-0.155997758854)*x[2]
        s[2,2,2,1]=(-0.105324133175)*x[0]+(1.11807221711)*x[1]+(0.122751975298)*x[2]
        s[2,3,0,0]=(0.685932400265)*x[0]+(-0.666847052302)*x[1]+(0.596321369477)*x[2]
        s[2,3,0,1]=(0.335067153518)*x[0]+(0.268453607431)*x[1]+(0.247881738828)*x[2]
        s[2,3,1,0]=(-1.89084909179)*x[0]+(-0.529751828744)*x[1]+(-0.959358202828)*x[2]
        s[2,3,1,1]=(1.06725689989)*x[0]+(-0.709878517637)*x[1]+(0.968886366535)*x[2]
        s[2,3,2,0]=(-1.63466018652)*x[0]+(0.517481248484)*x[1]+(0.109232845638)*x[2]
        s[2,3,2,1]=(0.0716425266603)*x[0]+(-0.58040972115)*x[1]+(0.346948738894)*x[2]
        s[2,4,0,0]=(0.193599854213)*x[0]+(-1.52744079327)*x[1]+(-0.0377038338183)*x[2]
        s[2,4,0,1]=(-0.0296855161658)*x[0]+(0.266976994363)*x[1]+(-0.832771979863)*x[2]
        s[2,4,1,0]=(-0.885217283694)*x[0]+(0.657425725373)*x[1]+(0.385584782531)*x[2]
        s[2,4,1,1]=(-0.249912733602)*x[0]+(0.156704487203)*x[1]+(0.000393408500501)*x[2]
        s[2,4,2,0]=(-0.083303562971)*x[0]+(0.748009334975)*x[1]+(-0.411323946112)*x[2]
        s[2,4,2,1]=(0.143782087293)*x[0]+(-1.86497054401)*x[1]+(-0.398161751889)*x[2]
        s[3,0,0,0]=(-0.531041035506)*x[0]+(0.998737023878)*x[1]+(-0.719326852701)*x[2]
        s[3,0,0,1]=(-1.10320774278)*x[0]+(-1.27795876669)*x[1]+(0.57451462957)*x[2]
        s[3,0,1,0]=(-0.319275578502)*x[0]+(-1.51721910203)*x[1]+(-0.677470578443)*x[2]
        s[3,0,1,1]=(0.7584807917)*x[0]+(0.0523998746885)*x[1]+(0.502182688228)*x[2]
        s[3,0,2,0]=(-0.905809892125)*x[0]+(0.479751975382)*x[1]+(-0.0778539387977)*x[2]
        s[3,0,2,1]=(-0.724891388357)*x[0]+(0.941493554841)*x[1]+(-1.51514565838)*x[2]
        s[3,1,0,0]=(0.815118793475)*x[0]+(0.0503368907581)*x[1]+(-0.502091253213)*x[2]
        s[3,1,0,1]=(-1.50859117948)*x[0]+(0.31279517213)*x[1]+(-0.339295035594)*x[2]
        s[3,1,1,0]=(0.246825549192)*x[0]+(0.687251908372)*x[1]+(0.133971781776)*x[2]
        s[3,1,1,1]=(0.470105585328)*x[0]+(0.382971101979)*x[1]+(-0.469397699886)*x[2]
        s[3,1,2,0]=(-1.45842815542)*x[0]+(-1.00979255513)*x[1]+(1.45386034482)*x[2]
        s[3,1,2,1]=(0.172130164577)*x[0]+(1.25034739991)*x[1]+(-1.15953423359)*x[2]
        s[3,2,0,0]=(-0.58261393055)*x[0]+(0.730381032663)*x[1]+(1.29377076249)*x[2]
        s[3,2,0,1]=(0.696715362351)*x[0]+(-1.37598351546)*x[1]+(-1.26545085967)*x[2]
        s[3,2,1,0]=(-0.0511282610422)*x[0]+(0.58041679118)*x[1]+(-0.0810357412151)*x[2]
        s[3,2,1,1]=(-0.750057228803)*x[0]+(-0.750703125074)*x[1]+(-1.21748956536)*x[2]
        s[3,2,2,0]=(-0.455746666635)*x[0]+(-0.720020627516)*x[1]+(-0.0911281889971)*x[2]
        s[3,2,2,1]=(-0.0681894163236)*x[0]+(0.101592029278)*x[1]+(1.10954864406)*x[2]
        s[3,3,0,0]=(0.433898443791)*x[0]+(0.901180030496)*x[1]+(0.795641263874)*x[2]
        s[3,3,0,1]=(-0.213306810738)*x[0]+(-0.852539239286)*x[1]+(-1.25905416259)*x[2]
        s[3,3,1,0]=(-0.668307384473)*x[0]+(-1.37753879862)*x[1]+(-0.593731800903)*x[2]
        s[3,3,1,1]=(0.815464294514)*x[0]+(0.835389640785)*x[1]+(0.933029468258)*x[2]
        s[3,3,2,0]=(0.565194274444)*x[0]+(0.584119707001)*x[1]+(-0.53812189952)*x[2]
        s[3,3,2,1]=(-0.903604209314)*x[0]+(0.370887007483)*x[1]+(-0.548135915608)*x[2]
        s[3,4,0,0]=(-0.819290139492)*x[0]+(-0.256800498012)*x[1]+(0.350363336935)*x[2]
        s[3,4,0,1]=(0.863170182723)*x[0]+(0.0101766489337)*x[1]+(1.34009635664)*x[2]
        s[3,4,1,0]=(1.03350734136)*x[0]+(1.37672500233)*x[1]+(1.35134434663)*x[2]
        s[3,4,1,1]=(1.4883144839)*x[0]+(0.410688244039)*x[1]+(0.759267998436)*x[2]
        s[3,4,2,0]=(0.373190738096)*x[0]+(1.22136515279)*x[1]+(-0.0365030690962)*x[2]
        s[3,4,2,1]=(-0.607437402737)*x[0]+(0.750130654364)*x[1]+(0.084778884789)*x[2]
        ref[0,0,0,0]=(0.847845815599)*x_ref[0]+(-0.610234211476)*x_ref[1]+(-0.627625510082)*x_ref[2]
        ref[0,0,0,1]=(0.0126184651041)*x_ref[0]+(-0.0651294063564)*x_ref[1]+(-0.274615935201)*x_ref[2]
        ref[0,0,1,0]=(-0.545925417969)*x_ref[0]+(0.421929302201)*x_ref[1]+(-0.910418625383)*x_ref[2]
        ref[0,0,1,1]=(1.58754252937)*x_ref[0]+(0.110111591047)*x_ref[1]+(-1.64780563736)*x_ref[2]
        ref[0,0,2,0]=(-0.510331643992)*x_ref[0]+(0.827685326542)*x_ref[1]+(1.31447962917)*x_ref[2]
        ref[0,0,2,1]=(-1.29575211142)*x_ref[0]+(-0.318096803401)*x_ref[1]+(-0.274660613777)*x_ref[2]
        ref[0,1,0,0]=(0.77444944835)*x_ref[0]+(-1.0893411122)*x_ref[1]+(0.331702744143)*x_ref[2]
        ref[0,1,0,1]=(0.624693916432)*x_ref[0]+(-0.306552086758)*x_ref[1]+(-0.845524417807)*x_ref[2]
        ref[0,1,1,0]=(0.0219492668609)*x_ref[0]+(-1.13712865204)*x_ref[1]+(-0.193051406533)*x_ref[2]
        ref[0,1,1,1]=(-1.32696889246)*x_ref[0]+(1.63857177904)*x_ref[1]+(0.194095476936)*x_ref[2]
        ref[0,1,2,0]=(1.19169978642)*x_ref[0]+(-1.88274910505)*x_ref[1]+(0.321438561021)*x_ref[2]
        ref[0,1,2,1]=(-0.765748081809)*x_ref[0]+(-0.163422665563)*x_ref[1]+(0.256721885141)*x_ref[2]
        ref[0,2,0,0]=(-0.863281319991)*x_ref[0]+(-0.021828308413)*x_ref[1]+(0.868312889989)*x_ref[2]
        ref[0,2,0,1]=(-0.57479341478)*x_ref[0]+(0.404861121887)*x_ref[1]+(-0.274037007699)*x_ref[2]
        ref[0,2,1,0]=(-0.529171478131)*x_ref[0]+(0.836001872018)*x_ref[1]+(-1.12226189724)*x_ref[2]
        ref[0,2,1,1]=(-0.821929160411)*x_ref[0]+(0.0466762602824)*x_ref[1]+(-0.0514331500765)*x_ref[2]
        ref[0,2,2,0]=(-0.605940618544)*x_ref[0]+(-1.54379919132)*x_ref[1]+(1.63243021269)*x_ref[2]
        ref[0,2,2,1]=(-1.49695860655)*x_ref[0]+(-0.601345463475)*x_ref[1]+(-0.753443592162)*x_ref[2]
        ref[0,3,0,0]=(1.33514081849)*x_ref[0]+(0.955849232345)*x_ref[1]+(-0.872221354599)*x_ref[2]
        ref[0,3,0,1]=(0.939731890901)*x_ref[0]+(-1.16116012822)*x_ref[1]+(-0.601602002743)*x_ref[2]
        ref[0,3,1,0]=(-0.675545889094)*x_ref[0]+(0.736755090077)*x_ref[1]+(-0.437370345974)*x_ref[2]
        ref[0,3,1,1]=(-1.17899966213)*x_ref[0]+(0.422125349488)*x_ref[1]+(-0.815649229407)*x_ref[2]
        ref[0,3,2,0]=(0.0773790438387)*x_ref[0]+(0.148378160687)*x_ref[1]+(-1.0156040466)*x_ref[2]
        ref[0,3,2,1]=(0.519422925349)*x_ref[0]+(0.189993892084)*x_ref[1]+(1.59654399673)*x_ref[2]
        ref[0,4,0,0]=(-0.0593352727817)*x_ref[0]+(0.0865384232804)*x_ref[1]+(-0.316122229956)*x_ref[2]
        ref[0,4,0,1]=(0.935220504817)*x_ref[0]+(0.868143343952)*x_ref[1]+(-0.470286174039)*x_ref[2]
        ref[0,4,1,0]=(0.763506086212)*x_ref[0]+(-0.0756795330833)*x_ref[1]+(1.06812672934)*x_ref[2]
        ref[0,4,1,1]=(-0.446580256351)*x_ref[0]+(0.653986880233)*x_ref[1]+(0.634364696873)*x_ref[2]
        ref[0,4,2,0]=(-0.72871248403)*x_ref[0]+(0.0839150289011)*x_ref[1]+(0.178899292496)*x_ref[2]
        ref[0,4,2,1]=(1.30808288445)*x_ref[0]+(0.548931211195)*x_ref[1]+(-0.971585668771)*x_ref[2]
        ref[1,0,0,0]=(-0.737171903787)*x_ref[0]+(-0.137110160891)*x_ref[1]+(1.24817883553)*x_ref[2]
        ref[1,0,0,1]=(0.589824248199)*x_ref[0]+(-0.191688810075)*x_ref[1]+(1.33820051734)*x_ref[2]
        ref[1,0,1,0]=(-0.598063103136)*x_ref[0]+(0.347705502684)*x_ref[1]+(0.490747627936)*x_ref[2]
        ref[1,0,1,1]=(0.287829427303)*x_ref[0]+(-0.853875346674)*x_ref[1]+(-0.227722773909)*x_ref[2]
        ref[1,0,2,0]=(-1.22183236343)*x_ref[0]+(-0.162406713796)*x_ref[1]+(-0.436433884201)*x_ref[2]
        ref[1,0,2,1]=(0.929372688401)*x_ref[0]+(0.469931776805)*x_ref[1]+(0.739110972137)*x_ref[2]
        ref[1,1,0,0]=(0.391824586)*x_ref[0]+(-0.551498198462)*x_ref[1]+(-0.588935046796)*x_ref[2]
        ref[1,1,0,1]=(0.849805745827)*x_ref[0]+(0.557121315313)*x_ref[1]+(1.80284061615)*x_ref[2]
        ref[1,1,1,0]=(-1.62270432051)*x_ref[0]+(-1.55943636923)*x_ref[1]+(0.629204169009)*x_ref[2]
        ref[1,1,1,1]=(-0.191271118514)*x_ref[0]+(-0.454718665581)*x_ref[1]+(0.405403171722)*x_ref[2]
        ref[1,1,2,0]=(0.461750123507)*x_ref[0]+(0.537594040674)*x_ref[1]+(-0.500046587877)*x_ref[2]
        ref[1,1,2,1]=(1.07800193455)*x_ref[0]+(0.0757746249706)*x_ref[1]+(0.358223454759)*x_ref[2]
        ref[1,2,0,0]=(-1.31360669142)*x_ref[0]+(0.908413426737)*x_ref[1]+(0.986904165818)*x_ref[2]
        ref[1,2,0,1]=(-0.250344247716)*x_ref[0]+(0.596746427887)*x_ref[1]+(0.846846599186)*x_ref[2]
        ref[1,2,1,0]=(1.65761753214)*x_ref[0]+(0.0969407749043)*x_ref[1]+(0.197913502281)*x_ref[2]
        ref[1,2,1,1]=(0.808454446977)*x_ref[0]+(-0.107176416129)*x_ref[1]+(-0.115562549181)*x_ref[2]
        ref[1,2,2,0]=(-0.410108442793)*x_ref[0]+(0.250496842214)*x_ref[1]+(0.403806076052)*x_ref[2]
        ref[1,2,2,1]=(1.58110488679)*x_ref[0]+(-0.171583112102)*x_ref[1]+(0.441813895222)*x_ref[2]
        ref[1,3,0,0]=(-0.519973841295)*x_ref[0]+(0.14862500493)*x_ref[1]+(-0.0257159960311)*x_ref[2]
        ref[1,3,0,1]=(-0.710423889723)*x_ref[0]+(0.0941990549739)*x_ref[1]+(-0.147877062943)*x_ref[2]
        ref[1,3,1,0]=(-0.19917762038)*x_ref[0]+(-1.06873791711)*x_ref[1]+(1.37688275432)*x_ref[2]
        ref[1,3,1,1]=(-0.570477493665)*x_ref[0]+(0.0705814270652)*x_ref[1]+(-1.10186231057)*x_ref[2]
        ref[1,3,2,0]=(-1.04891017002)*x_ref[0]+(-0.136784107636)*x_ref[1]+(-1.65953016693)*x_ref[2]
        ref[1,3,2,1]=(-0.674805716398)*x_ref[0]+(-0.211803508128)*x_ref[1]+(-0.116463503974)*x_ref[2]
        ref[1,4,0,0]=(0.269217220297)*x_ref[0]+(0.810722194772)*x_ref[1]+(1.60977315112)*x_ref[2]
        ref[1,4,0,1]=(-0.0213286766746)*x_ref[0]+(1.0974274553)*x_ref[1]+(0.275633207025)*x_ref[2]
        ref[1,4,1,0]=(-0.601230245409)*x_ref[0]+(-0.756418955933)*x_ref[1]+(-0.46194303915)*x_ref[2]
        ref[1,4,1,1]=(-0.0907667209143)*x_ref[0]+(-1.86623564673)*x_ref[1]+(0.552729474701)*x_ref[2]
        ref[1,4,2,0]=(-1.09694541489)*x_ref[0]+(-0.496193881555)*x_ref[1]+(-0.0126133249168)*x_ref[2]
        ref[1,4,2,1]=(-0.648959246633)*x_ref[0]+(-0.60655325639)*x_ref[1]+(0.438258051102)*x_ref[2]
        ref[2,0,0,0]=(-0.100512547019)*x_ref[0]+(-1.37291269079)*x_ref[1]+(1.63084319876)*x_ref[2]
        ref[2,0,0,1]=(-0.50510326916)*x_ref[0]+(0.308799737351)*x_ref[1]+(1.63506761477)*x_ref[2]
        ref[2,0,1,0]=(0.708408519723)*x_ref[0]+(-0.0759833841695)*x_ref[1]+(-0.980974095374)*x_ref[2]
        ref[2,0,1,1]=(-1.59727324143)*x_ref[0]+(0.995250195962)*x_ref[1]+(0.169527316083)*x_ref[2]
        ref[2,0,2,0]=(0.766935817054)*x_ref[0]+(0.455564399031)*x_ref[1]+(0.605772951782)*x_ref[2]
        ref[2,0,2,1]=(-0.820755214894)*x_ref[0]+(-0.43359371038)*x_ref[1]+(-0.0942398466964)*x_ref[2]
        ref[2,1,0,0]=(0.390476682801)*x_ref[0]+(1.11627447827)*x_ref[1]+(0.670843405466)*x_ref[2]
        ref[2,1,0,1]=(-0.433594870483)*x_ref[0]+(-0.940710004402)*x_ref[1]+(1.10756041487)*x_ref[2]
        ref[2,1,1,0]=(-1.21880018868)*x_ref[0]+(0.380422307344)*x_ref[1]+(-0.864017731789)*x_ref[2]
        ref[2,1,1,1]=(-0.194168346974)*x_ref[0]+(0.324183983028)*x_ref[1]+(0.741456579501)*x_ref[2]
        ref[2,1,2,0]=(-0.134211025912)*x_ref[0]+(0.497311041207)*x_ref[1]+(0.33014196376)*x_ref[2]
        ref[2,1,2,1]=(-1.3501981937)*x_ref[0]+(1.67601514859)*x_ref[1]+(-1.60084725544)*x_ref[2]
        ref[2,2,0,0]=(-0.8367457236)*x_ref[0]+(-1.15628524562)*x_ref[1]+(0.342983125417)*x_ref[2]
        ref[2,2,0,1]=(0.956733778283)*x_ref[0]+(-0.404880336577)*x_ref[1]+(-0.350927185105)*x_ref[2]
        ref[2,2,1,0]=(0.854842449281)*x_ref[0]+(-0.185001304765)*x_ref[1]+(1.15245157168)*x_ref[2]
        ref[2,2,1,1]=(0.00584831242623)*x_ref[0]+(-0.562950548984)*x_ref[1]+(0.785766513431)*x_ref[2]
        ref[2,2,2,0]=(0.272759945164)*x_ref[0]+(-0.568207376307)*x_ref[1]+(-0.155997758854)*x_ref[2]
        ref[2,2,2,1]=(-0.105324133175)*x_ref[0]+(1.11807221711)*x_ref[1]+(0.122751975298)*x_ref[2]
        ref[2,3,0,0]=(0.685932400265)*x_ref[0]+(-0.666847052302)*x_ref[1]+(0.596321369477)*x_ref[2]
        ref[2,3,0,1]=(0.335067153518)*x_ref[0]+(0.268453607431)*x_ref[1]+(0.247881738828)*x_ref[2]
        ref[2,3,1,0]=(-1.89084909179)*x_ref[0]+(-0.529751828744)*x_ref[1]+(-0.959358202828)*x_ref[2]
        ref[2,3,1,1]=(1.06725689989)*x_ref[0]+(-0.709878517637)*x_ref[1]+(0.968886366535)*x_ref[2]
        ref[2,3,2,0]=(-1.63466018652)*x_ref[0]+(0.517481248484)*x_ref[1]+(0.109232845638)*x_ref[2]
        ref[2,3,2,1]=(0.0716425266603)*x_ref[0]+(-0.58040972115)*x_ref[1]+(0.346948738894)*x_ref[2]
        ref[2,4,0,0]=(0.193599854213)*x_ref[0]+(-1.52744079327)*x_ref[1]+(-0.0377038338183)*x_ref[2]
        ref[2,4,0,1]=(-0.0296855161658)*x_ref[0]+(0.266976994363)*x_ref[1]+(-0.832771979863)*x_ref[2]
        ref[2,4,1,0]=(-0.885217283694)*x_ref[0]+(0.657425725373)*x_ref[1]+(0.385584782531)*x_ref[2]
        ref[2,4,1,1]=(-0.249912733602)*x_ref[0]+(0.156704487203)*x_ref[1]+(0.000393408500501)*x_ref[2]
        ref[2,4,2,0]=(-0.083303562971)*x_ref[0]+(0.748009334975)*x_ref[1]+(-0.411323946112)*x_ref[2]
        ref[2,4,2,1]=(0.143782087293)*x_ref[0]+(-1.86497054401)*x_ref[1]+(-0.398161751889)*x_ref[2]
        ref[3,0,0,0]=(-0.531041035506)*x_ref[0]+(0.998737023878)*x_ref[1]+(-0.719326852701)*x_ref[2]
        ref[3,0,0,1]=(-1.10320774278)*x_ref[0]+(-1.27795876669)*x_ref[1]+(0.57451462957)*x_ref[2]
        ref[3,0,1,0]=(-0.319275578502)*x_ref[0]+(-1.51721910203)*x_ref[1]+(-0.677470578443)*x_ref[2]
        ref[3,0,1,1]=(0.7584807917)*x_ref[0]+(0.0523998746885)*x_ref[1]+(0.502182688228)*x_ref[2]
        ref[3,0,2,0]=(-0.905809892125)*x_ref[0]+(0.479751975382)*x_ref[1]+(-0.0778539387977)*x_ref[2]
        ref[3,0,2,1]=(-0.724891388357)*x_ref[0]+(0.941493554841)*x_ref[1]+(-1.51514565838)*x_ref[2]
        ref[3,1,0,0]=(0.815118793475)*x_ref[0]+(0.0503368907581)*x_ref[1]+(-0.502091253213)*x_ref[2]
        ref[3,1,0,1]=(-1.50859117948)*x_ref[0]+(0.31279517213)*x_ref[1]+(-0.339295035594)*x_ref[2]
        ref[3,1,1,0]=(0.246825549192)*x_ref[0]+(0.687251908372)*x_ref[1]+(0.133971781776)*x_ref[2]
        ref[3,1,1,1]=(0.470105585328)*x_ref[0]+(0.382971101979)*x_ref[1]+(-0.469397699886)*x_ref[2]
        ref[3,1,2,0]=(-1.45842815542)*x_ref[0]+(-1.00979255513)*x_ref[1]+(1.45386034482)*x_ref[2]
        ref[3,1,2,1]=(0.172130164577)*x_ref[0]+(1.25034739991)*x_ref[1]+(-1.15953423359)*x_ref[2]
        ref[3,2,0,0]=(-0.58261393055)*x_ref[0]+(0.730381032663)*x_ref[1]+(1.29377076249)*x_ref[2]
        ref[3,2,0,1]=(0.696715362351)*x_ref[0]+(-1.37598351546)*x_ref[1]+(-1.26545085967)*x_ref[2]
        ref[3,2,1,0]=(-0.0511282610422)*x_ref[0]+(0.58041679118)*x_ref[1]+(-0.0810357412151)*x_ref[2]
        ref[3,2,1,1]=(-0.750057228803)*x_ref[0]+(-0.750703125074)*x_ref[1]+(-1.21748956536)*x_ref[2]
        ref[3,2,2,0]=(-0.455746666635)*x_ref[0]+(-0.720020627516)*x_ref[1]+(-0.0911281889971)*x_ref[2]
        ref[3,2,2,1]=(-0.0681894163236)*x_ref[0]+(0.101592029278)*x_ref[1]+(1.10954864406)*x_ref[2]
        ref[3,3,0,0]=(0.433898443791)*x_ref[0]+(0.901180030496)*x_ref[1]+(0.795641263874)*x_ref[2]
        ref[3,3,0,1]=(-0.213306810738)*x_ref[0]+(-0.852539239286)*x_ref[1]+(-1.25905416259)*x_ref[2]
        ref[3,3,1,0]=(-0.668307384473)*x_ref[0]+(-1.37753879862)*x_ref[1]+(-0.593731800903)*x_ref[2]
        ref[3,3,1,1]=(0.815464294514)*x_ref[0]+(0.835389640785)*x_ref[1]+(0.933029468258)*x_ref[2]
        ref[3,3,2,0]=(0.565194274444)*x_ref[0]+(0.584119707001)*x_ref[1]+(-0.53812189952)*x_ref[2]
        ref[3,3,2,1]=(-0.903604209314)*x_ref[0]+(0.370887007483)*x_ref[1]+(-0.548135915608)*x_ref[2]
        ref[3,4,0,0]=(-0.819290139492)*x_ref[0]+(-0.256800498012)*x_ref[1]+(0.350363336935)*x_ref[2]
        ref[3,4,0,1]=(0.863170182723)*x_ref[0]+(0.0101766489337)*x_ref[1]+(1.34009635664)*x_ref[2]
        ref[3,4,1,0]=(1.03350734136)*x_ref[0]+(1.37672500233)*x_ref[1]+(1.35134434663)*x_ref[2]
        ref[3,4,1,1]=(1.4883144839)*x_ref[0]+(0.410688244039)*x_ref[1]+(0.759267998436)*x_ref[2]
        ref[3,4,2,0]=(0.373190738096)*x_ref[0]+(1.22136515279)*x_ref[1]+(-0.0365030690962)*x_ref[2]
        ref[3,4,2,1]=(-0.607437402737)*x_ref[0]+(0.750130654364)*x_ref[1]+(0.084778884789)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_ReducedFunctionOnContactZero_rank0(self):
      """
      tests interpolate for rank 0 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactZero onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactZero(self.domain)
      x=w.getX()
      arg=Symbol(shape=(),dim=dim)
      s=Data(0,(),w)
      ref=Data(0,(),w_ref)
      if dim==2:
        s=(-0.478526643653)*x[0]**o+(-0.798079781236)*x[0]+(-0.499553919498)*x[1]**o+(0.336723069414)*x[1]
        ref=(-0.478526643653)*x_ref[0]**o+(-0.798079781236)*x_ref[0]+(-0.499553919498)*x_ref[1]**o+(0.336723069414)*x_ref[1]
      else:
        s=(-0.35871218678)*x[0]**o+(0.954679093985)*x[0]+(-0.127722494434)*x[1]**o+(0.0119904093364)*x[1]+(-0.647695475743)*x[2]**o+(-0.522381915873)*x[2]
        ref=(-0.35871218678)*x_ref[0]**o+(0.954679093985)*x_ref[0]+(-0.127722494434)*x_ref[1]**o+(0.0119904093364)*x_ref[1]+(-0.647695475743)*x_ref[2]**o+(-0.522381915873)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_ReducedFunctionOnContactZero_rank1(self):
      """
      tests interpolate for rank 1 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactZero onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactZero(self.domain)
      x=w.getX()
      arg=Symbol(shape=(2,),dim=dim)
      s=Data(0,(2,),w)
      ref=Data(0,(2,),w_ref)
      if dim==2:
        s[0]=(0.0333528524993)*x[0]**o+(-0.534169045864)*x[0]+(0.234758321366)*x[1]**o+(0.395702014106)*x[1]
        s[1]=(-0.559512507686)*x[0]**o+(0.473831121701)*x[0]+(0.315500482589)*x[1]**o+(0.659175367015)*x[1]
        ref[0]=(0.0333528524993)*x_ref[0]**o+(-0.534169045864)*x_ref[0]+(0.234758321366)*x_ref[1]**o+(0.395702014106)*x_ref[1]
        ref[1]=(-0.559512507686)*x_ref[0]**o+(0.473831121701)*x_ref[0]+(0.315500482589)*x_ref[1]**o+(0.659175367015)*x_ref[1]
      else:
        s[0]=(0.920431380976)*x[0]**o+(0.203182015633)*x[0]+(-0.30119291643)*x[1]**o+(0.743123673038)*x[1]+(0.158345217723)*x[2]**o+(-0.106151059285)*x[2]
        s[1]=(-0.246067927964)*x[0]**o+(0.249705802315)*x[0]+(0.87511193184)*x[1]**o+(0.174791560577)*x[1]+(-0.717945042302)*x[2]**o+(0.558392243221)*x[2]
        ref[0]=(0.920431380976)*x_ref[0]**o+(0.203182015633)*x_ref[0]+(-0.30119291643)*x_ref[1]**o+(0.743123673038)*x_ref[1]+(0.158345217723)*x_ref[2]**o+(-0.106151059285)*x_ref[2]
        ref[1]=(-0.246067927964)*x_ref[0]**o+(0.249705802315)*x_ref[0]+(0.87511193184)*x_ref[1]**o+(0.174791560577)*x_ref[1]+(-0.717945042302)*x_ref[2]**o+(0.558392243221)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_ReducedFunctionOnContactZero_rank2(self):
      """
      tests interpolate for rank 2 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactZero onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactZero(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5),dim=dim)
      s=Data(0,(4, 5),w)
      ref=Data(0,(4, 5),w_ref)
      if dim==2:
        s[0,0]=(-0.238674589068)*x[0]**o+(0.845146310144)*x[0]+(0.51393009023)*x[1]**o+(0.987659698068)*x[1]
        s[0,1]=(0.525738867968)*x[0]**o+(-0.541924459594)*x[0]+(-0.345083945286)*x[1]**o+(0.892536321695)*x[1]
        s[0,2]=(0.467434483897)*x[0]**o+(0.673356246793)*x[0]+(0.373066330502)*x[1]**o+(0.455273174229)*x[1]
        s[0,3]=(-0.39771044233)*x[0]**o+(0.890234781078)*x[0]+(-0.11624722564)*x[1]**o+(0.995857794085)*x[1]
        s[0,4]=(0.795987536438)*x[0]**o+(-0.683413643902)*x[0]+(-0.0517246803869)*x[1]**o+(0.149282877675)*x[1]
        s[1,0]=(0.215953459935)*x[0]**o+(0.0124143229743)*x[0]+(0.719573519094)*x[1]**o+(0.120838088026)*x[1]
        s[1,1]=(0.195894110038)*x[0]**o+(-0.481212957341)*x[0]+(-0.579369726661)*x[1]**o+(-0.825868817932)*x[1]
        s[1,2]=(0.242091565308)*x[0]**o+(-0.342866600818)*x[0]+(0.871948019343)*x[1]**o+(0.925599757496)*x[1]
        s[1,3]=(-0.228955457627)*x[0]**o+(-0.618007692175)*x[0]+(0.641736066096)*x[1]**o+(-0.648940539231)*x[1]
        s[1,4]=(0.949258368604)*x[0]**o+(0.46037118633)*x[0]+(0.264262232255)*x[1]**o+(-0.204687228041)*x[1]
        s[2,0]=(0.201719829764)*x[0]**o+(-0.758921537456)*x[0]+(0.809565809407)*x[1]**o+(0.18932187303)*x[1]
        s[2,1]=(0.0400837555441)*x[0]**o+(0.703393740665)*x[0]+(0.989844901722)*x[1]**o+(-0.920311703188)*x[1]
        s[2,2]=(-0.1127643095)*x[0]**o+(-0.181508597843)*x[0]+(0.845854218374)*x[1]**o+(-0.401564965436)*x[1]
        s[2,3]=(0.725120155616)*x[0]**o+(0.0408162472823)*x[0]+(0.849016789586)*x[1]**o+(-0.0894725097851)*x[1]
        s[2,4]=(0.00712438942747)*x[0]**o+(0.430728482242)*x[0]+(-0.630571782961)*x[1]**o+(-0.292289123038)*x[1]
        s[3,0]=(0.604980765238)*x[0]**o+(-0.545924520431)*x[0]+(0.365986250184)*x[1]**o+(-0.717572291319)*x[1]
        s[3,1]=(-0.105418444539)*x[0]**o+(-0.301186945237)*x[0]+(0.298798693172)*x[1]**o+(0.305159917178)*x[1]
        s[3,2]=(-0.116290679999)*x[0]**o+(-0.814717358777)*x[0]+(0.463687518523)*x[1]**o+(-0.142856072677)*x[1]
        s[3,3]=(-0.0933248161262)*x[0]**o+(-0.745036857332)*x[0]+(-0.143025048876)*x[1]**o+(-0.317322119337)*x[1]
        s[3,4]=(0.340645005623)*x[0]**o+(-0.369235909154)*x[0]+(-0.739698569483)*x[1]**o+(0.296590339275)*x[1]
        ref[0,0]=(-0.238674589068)*x_ref[0]**o+(0.845146310144)*x_ref[0]+(0.51393009023)*x_ref[1]**o+(0.987659698068)*x_ref[1]
        ref[0,1]=(0.525738867968)*x_ref[0]**o+(-0.541924459594)*x_ref[0]+(-0.345083945286)*x_ref[1]**o+(0.892536321695)*x_ref[1]
        ref[0,2]=(0.467434483897)*x_ref[0]**o+(0.673356246793)*x_ref[0]+(0.373066330502)*x_ref[1]**o+(0.455273174229)*x_ref[1]
        ref[0,3]=(-0.39771044233)*x_ref[0]**o+(0.890234781078)*x_ref[0]+(-0.11624722564)*x_ref[1]**o+(0.995857794085)*x_ref[1]
        ref[0,4]=(0.795987536438)*x_ref[0]**o+(-0.683413643902)*x_ref[0]+(-0.0517246803869)*x_ref[1]**o+(0.149282877675)*x_ref[1]
        ref[1,0]=(0.215953459935)*x_ref[0]**o+(0.0124143229743)*x_ref[0]+(0.719573519094)*x_ref[1]**o+(0.120838088026)*x_ref[1]
        ref[1,1]=(0.195894110038)*x_ref[0]**o+(-0.481212957341)*x_ref[0]+(-0.579369726661)*x_ref[1]**o+(-0.825868817932)*x_ref[1]
        ref[1,2]=(0.242091565308)*x_ref[0]**o+(-0.342866600818)*x_ref[0]+(0.871948019343)*x_ref[1]**o+(0.925599757496)*x_ref[1]
        ref[1,3]=(-0.228955457627)*x_ref[0]**o+(-0.618007692175)*x_ref[0]+(0.641736066096)*x_ref[1]**o+(-0.648940539231)*x_ref[1]
        ref[1,4]=(0.949258368604)*x_ref[0]**o+(0.46037118633)*x_ref[0]+(0.264262232255)*x_ref[1]**o+(-0.204687228041)*x_ref[1]
        ref[2,0]=(0.201719829764)*x_ref[0]**o+(-0.758921537456)*x_ref[0]+(0.809565809407)*x_ref[1]**o+(0.18932187303)*x_ref[1]
        ref[2,1]=(0.0400837555441)*x_ref[0]**o+(0.703393740665)*x_ref[0]+(0.989844901722)*x_ref[1]**o+(-0.920311703188)*x_ref[1]
        ref[2,2]=(-0.1127643095)*x_ref[0]**o+(-0.181508597843)*x_ref[0]+(0.845854218374)*x_ref[1]**o+(-0.401564965436)*x_ref[1]
        ref[2,3]=(0.725120155616)*x_ref[0]**o+(0.0408162472823)*x_ref[0]+(0.849016789586)*x_ref[1]**o+(-0.0894725097851)*x_ref[1]
        ref[2,4]=(0.00712438942747)*x_ref[0]**o+(0.430728482242)*x_ref[0]+(-0.630571782961)*x_ref[1]**o+(-0.292289123038)*x_ref[1]
        ref[3,0]=(0.604980765238)*x_ref[0]**o+(-0.545924520431)*x_ref[0]+(0.365986250184)*x_ref[1]**o+(-0.717572291319)*x_ref[1]
        ref[3,1]=(-0.105418444539)*x_ref[0]**o+(-0.301186945237)*x_ref[0]+(0.298798693172)*x_ref[1]**o+(0.305159917178)*x_ref[1]
        ref[3,2]=(-0.116290679999)*x_ref[0]**o+(-0.814717358777)*x_ref[0]+(0.463687518523)*x_ref[1]**o+(-0.142856072677)*x_ref[1]
        ref[3,3]=(-0.0933248161262)*x_ref[0]**o+(-0.745036857332)*x_ref[0]+(-0.143025048876)*x_ref[1]**o+(-0.317322119337)*x_ref[1]
        ref[3,4]=(0.340645005623)*x_ref[0]**o+(-0.369235909154)*x_ref[0]+(-0.739698569483)*x_ref[1]**o+(0.296590339275)*x_ref[1]
      else:
        s[0,0]=(-0.343988908393)*x[0]**o+(-0.249046543972)*x[0]+(-0.94669255683)*x[1]**o+(-0.125519622379)*x[1]+(-0.238439423996)*x[2]**o+(-0.270484426474)*x[2]
        s[0,1]=(-0.81960628266)*x[0]**o+(-0.752592323006)*x[0]+(-0.608566310221)*x[1]**o+(0.136425226211)*x[1]+(-0.417282843542)*x[2]**o+(0.510888775001)*x[2]
        s[0,2]=(-0.469706099858)*x[0]**o+(0.948757046189)*x[0]+(-0.752290984326)*x[1]**o+(0.423515758928)*x[1]+(0.031409477618)*x[2]**o+(0.234409805802)*x[2]
        s[0,3]=(-0.900755988795)*x[0]**o+(-0.767761655222)*x[0]+(0.569180403754)*x[1]**o+(-0.233362758546)*x[1]+(0.569347237378)*x[2]**o+(0.612139565636)*x[2]
        s[0,4]=(0.885682511121)*x[0]**o+(0.332415602786)*x[0]+(0.350335006384)*x[1]**o+(0.613490643585)*x[1]+(0.444593860725)*x[2]**o+(-0.000225980787608)*x[2]
        s[1,0]=(-0.584959823379)*x[0]**o+(-0.964957485122)*x[0]+(0.289141634104)*x[1]**o+(0.263120358818)*x[1]+(-0.028745699583)*x[2]**o+(0.982902290525)*x[2]
        s[1,1]=(-0.191593204495)*x[0]**o+(-0.604908583526)*x[0]+(0.804988763905)*x[1]**o+(0.741296673588)*x[1]+(0.808775189028)*x[2]**o+(0.320306927549)*x[2]
        s[1,2]=(0.658017563103)*x[0]**o+(0.150282440579)*x[0]+(0.988184604156)*x[1]**o+(-0.168752859833)*x[1]+(0.0248149744777)*x[2]**o+(0.158431888663)*x[2]
        s[1,3]=(0.482286527364)*x[0]**o+(0.794842099146)*x[0]+(-0.103524158001)*x[1]**o+(0.834955012893)*x[1]+(-0.724721525621)*x[2]**o+(-0.581451689942)*x[2]
        s[1,4]=(0.987701027083)*x[0]**o+(0.358477587611)*x[0]+(0.0447031116415)*x[1]**o+(-0.63585414781)*x[1]+(-0.493573694258)*x[2]**o+(-0.670666093134)*x[2]
        s[2,0]=(0.156348690681)*x[0]**o+(0.181907398102)*x[0]+(0.810783963046)*x[1]**o+(-0.0420809525478)*x[1]+(0.455236439578)*x[2]**o+(0.68359015036)*x[2]
        s[2,1]=(-0.277203778658)*x[0]**o+(0.906301662318)*x[0]+(0.507233384623)*x[1]**o+(-0.905644178764)*x[1]+(-0.116730257787)*x[2]**o+(-0.205065773315)*x[2]
        s[2,2]=(-0.951427408426)*x[0]**o+(0.418017573994)*x[0]+(-0.707741913019)*x[1]**o+(-0.517554096797)*x[1]+(-0.612052430927)*x[2]**o+(-0.831045221937)*x[2]
        s[2,3]=(0.0142781625544)*x[0]**o+(-0.0207359350934)*x[0]+(0.69086283301)*x[1]**o+(-0.487396868956)*x[1]+(0.868531537118)*x[2]**o+(-0.269368930177)*x[2]
        s[2,4]=(-0.657537261147)*x[0]**o+(0.411006377043)*x[0]+(0.641800881279)*x[1]**o+(-0.179106096535)*x[1]+(0.413169503059)*x[2]**o+(0.0796148767789)*x[2]
        s[3,0]=(-0.0465615058741)*x[0]**o+(0.497229745315)*x[0]+(0.048895565684)*x[1]**o+(0.881074858803)*x[1]+(-0.0904556886937)*x[2]**o+(-0.0486834982676)*x[2]
        s[3,1]=(0.472154117103)*x[0]**o+(-0.841976825688)*x[0]+(-0.840760456295)*x[1]**o+(-0.461023286336)*x[1]+(0.680506149471)*x[2]**o+(0.503845111856)*x[2]
        s[3,2]=(0.166611223306)*x[0]**o+(0.308064312352)*x[0]+(0.850239399334)*x[1]**o+(0.127827758007)*x[1]+(0.306675924384)*x[2]**o+(-0.35563117529)*x[2]
        s[3,3]=(0.0854397842075)*x[0]**o+(0.696191334187)*x[0]+(0.719296439336)*x[1]**o+(0.703852484675)*x[1]+(-0.841440898463)*x[2]**o+(-0.688038437262)*x[2]
        s[3,4]=(0.775596800509)*x[0]**o+(-0.0380327958147)*x[0]+(0.0582036420384)*x[1]**o+(0.0396609528486)*x[1]+(-0.449051573608)*x[2]**o+(0.139175147549)*x[2]
        ref[0,0]=(-0.343988908393)*x_ref[0]**o+(-0.249046543972)*x_ref[0]+(-0.94669255683)*x_ref[1]**o+(-0.125519622379)*x_ref[1]+(-0.238439423996)*x_ref[2]**o+(-0.270484426474)*x_ref[2]
        ref[0,1]=(-0.81960628266)*x_ref[0]**o+(-0.752592323006)*x_ref[0]+(-0.608566310221)*x_ref[1]**o+(0.136425226211)*x_ref[1]+(-0.417282843542)*x_ref[2]**o+(0.510888775001)*x_ref[2]
        ref[0,2]=(-0.469706099858)*x_ref[0]**o+(0.948757046189)*x_ref[0]+(-0.752290984326)*x_ref[1]**o+(0.423515758928)*x_ref[1]+(0.031409477618)*x_ref[2]**o+(0.234409805802)*x_ref[2]
        ref[0,3]=(-0.900755988795)*x_ref[0]**o+(-0.767761655222)*x_ref[0]+(0.569180403754)*x_ref[1]**o+(-0.233362758546)*x_ref[1]+(0.569347237378)*x_ref[2]**o+(0.612139565636)*x_ref[2]
        ref[0,4]=(0.885682511121)*x_ref[0]**o+(0.332415602786)*x_ref[0]+(0.350335006384)*x_ref[1]**o+(0.613490643585)*x_ref[1]+(0.444593860725)*x_ref[2]**o+(-0.000225980787608)*x_ref[2]
        ref[1,0]=(-0.584959823379)*x_ref[0]**o+(-0.964957485122)*x_ref[0]+(0.289141634104)*x_ref[1]**o+(0.263120358818)*x_ref[1]+(-0.028745699583)*x_ref[2]**o+(0.982902290525)*x_ref[2]
        ref[1,1]=(-0.191593204495)*x_ref[0]**o+(-0.604908583526)*x_ref[0]+(0.804988763905)*x_ref[1]**o+(0.741296673588)*x_ref[1]+(0.808775189028)*x_ref[2]**o+(0.320306927549)*x_ref[2]
        ref[1,2]=(0.658017563103)*x_ref[0]**o+(0.150282440579)*x_ref[0]+(0.988184604156)*x_ref[1]**o+(-0.168752859833)*x_ref[1]+(0.0248149744777)*x_ref[2]**o+(0.158431888663)*x_ref[2]
        ref[1,3]=(0.482286527364)*x_ref[0]**o+(0.794842099146)*x_ref[0]+(-0.103524158001)*x_ref[1]**o+(0.834955012893)*x_ref[1]+(-0.724721525621)*x_ref[2]**o+(-0.581451689942)*x_ref[2]
        ref[1,4]=(0.987701027083)*x_ref[0]**o+(0.358477587611)*x_ref[0]+(0.0447031116415)*x_ref[1]**o+(-0.63585414781)*x_ref[1]+(-0.493573694258)*x_ref[2]**o+(-0.670666093134)*x_ref[2]
        ref[2,0]=(0.156348690681)*x_ref[0]**o+(0.181907398102)*x_ref[0]+(0.810783963046)*x_ref[1]**o+(-0.0420809525478)*x_ref[1]+(0.455236439578)*x_ref[2]**o+(0.68359015036)*x_ref[2]
        ref[2,1]=(-0.277203778658)*x_ref[0]**o+(0.906301662318)*x_ref[0]+(0.507233384623)*x_ref[1]**o+(-0.905644178764)*x_ref[1]+(-0.116730257787)*x_ref[2]**o+(-0.205065773315)*x_ref[2]
        ref[2,2]=(-0.951427408426)*x_ref[0]**o+(0.418017573994)*x_ref[0]+(-0.707741913019)*x_ref[1]**o+(-0.517554096797)*x_ref[1]+(-0.612052430927)*x_ref[2]**o+(-0.831045221937)*x_ref[2]
        ref[2,3]=(0.0142781625544)*x_ref[0]**o+(-0.0207359350934)*x_ref[0]+(0.69086283301)*x_ref[1]**o+(-0.487396868956)*x_ref[1]+(0.868531537118)*x_ref[2]**o+(-0.269368930177)*x_ref[2]
        ref[2,4]=(-0.657537261147)*x_ref[0]**o+(0.411006377043)*x_ref[0]+(0.641800881279)*x_ref[1]**o+(-0.179106096535)*x_ref[1]+(0.413169503059)*x_ref[2]**o+(0.0796148767789)*x_ref[2]
        ref[3,0]=(-0.0465615058741)*x_ref[0]**o+(0.497229745315)*x_ref[0]+(0.048895565684)*x_ref[1]**o+(0.881074858803)*x_ref[1]+(-0.0904556886937)*x_ref[2]**o+(-0.0486834982676)*x_ref[2]
        ref[3,1]=(0.472154117103)*x_ref[0]**o+(-0.841976825688)*x_ref[0]+(-0.840760456295)*x_ref[1]**o+(-0.461023286336)*x_ref[1]+(0.680506149471)*x_ref[2]**o+(0.503845111856)*x_ref[2]
        ref[3,2]=(0.166611223306)*x_ref[0]**o+(0.308064312352)*x_ref[0]+(0.850239399334)*x_ref[1]**o+(0.127827758007)*x_ref[1]+(0.306675924384)*x_ref[2]**o+(-0.35563117529)*x_ref[2]
        ref[3,3]=(0.0854397842075)*x_ref[0]**o+(0.696191334187)*x_ref[0]+(0.719296439336)*x_ref[1]**o+(0.703852484675)*x_ref[1]+(-0.841440898463)*x_ref[2]**o+(-0.688038437262)*x_ref[2]
        ref[3,4]=(0.775596800509)*x_ref[0]**o+(-0.0380327958147)*x_ref[0]+(0.0582036420384)*x_ref[1]**o+(0.0396609528486)*x_ref[1]+(-0.449051573608)*x_ref[2]**o+(0.139175147549)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_ReducedFunctionOnContactZero_rank3(self):
      """
      tests interpolate for rank 3 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactZero onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactZero(self.domain)
      x=w.getX()
      arg=Symbol(shape=(6, 2, 2),dim=dim)
      s=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2),w_ref)
      if dim==2:
        s[0,0,0]=(0.748658213139)*x[0]**o+(0.588706109747)*x[0]+(-0.462395788697)*x[1]**o+(0.558348215492)*x[1]
        s[0,0,1]=(0.629957777572)*x[0]**o+(0.714493090646)*x[0]+(-0.491054832228)*x[1]**o+(0.710860562012)*x[1]
        s[0,1,0]=(-0.775354422191)*x[0]**o+(0.152096744898)*x[0]+(0.717568022676)*x[1]**o+(0.749449532926)*x[1]
        s[0,1,1]=(0.883871444687)*x[0]**o+(0.149367558371)*x[0]+(0.986279594706)*x[1]**o+(0.537229955955)*x[1]
        s[1,0,0]=(0.564688441996)*x[0]**o+(-0.206862774902)*x[0]+(0.522967069439)*x[1]**o+(-0.0390132413525)*x[1]
        s[1,0,1]=(0.706775860383)*x[0]**o+(0.973359689319)*x[0]+(0.567062178556)*x[1]**o+(-0.00264874871685)*x[1]
        s[1,1,0]=(-0.385131437493)*x[0]**o+(0.93223208004)*x[0]+(0.915604601744)*x[1]**o+(-0.604136431942)*x[1]
        s[1,1,1]=(-0.58509656422)*x[0]**o+(-0.46833032365)*x[0]+(0.143097299013)*x[1]**o+(-0.685071549088)*x[1]
        s[2,0,0]=(-0.987452159168)*x[0]**o+(0.609343916634)*x[0]+(0.425443014801)*x[1]**o+(0.58099336693)*x[1]
        s[2,0,1]=(0.957582337028)*x[0]**o+(0.62772269136)*x[0]+(-0.783891315097)*x[1]**o+(-0.77280642716)*x[1]
        s[2,1,0]=(0.327068296677)*x[0]**o+(0.0579535708335)*x[0]+(-0.798372147412)*x[1]**o+(-0.517396772711)*x[1]
        s[2,1,1]=(-0.863293845828)*x[0]**o+(0.688257458796)*x[0]+(0.777044937444)*x[1]**o+(0.210894669552)*x[1]
        s[3,0,0]=(0.641389559724)*x[0]**o+(-0.115831890978)*x[0]+(0.0852026159583)*x[1]**o+(0.101596282721)*x[1]
        s[3,0,1]=(0.0869751746538)*x[0]**o+(0.8310099269)*x[0]+(0.915409474965)*x[1]**o+(-0.140400418634)*x[1]
        s[3,1,0]=(-0.782218646979)*x[0]**o+(0.259242630384)*x[0]+(0.474259881734)*x[1]**o+(-0.339247837819)*x[1]
        s[3,1,1]=(0.110237392658)*x[0]**o+(-0.631167781024)*x[0]+(-0.901066615275)*x[1]**o+(0.380918387854)*x[1]
        s[4,0,0]=(-0.377519984827)*x[0]**o+(-0.77772320212)*x[0]+(0.441380184654)*x[1]**o+(0.601046552584)*x[1]
        s[4,0,1]=(-0.563858988542)*x[0]**o+(0.00445139485154)*x[0]+(-0.294713455734)*x[1]**o+(-0.863742009505)*x[1]
        s[4,1,0]=(-0.0472447099394)*x[0]**o+(-0.192880217627)*x[0]+(-0.299507318615)*x[1]**o+(-0.372320397063)*x[1]
        s[4,1,1]=(-0.81522394336)*x[0]**o+(0.822036737098)*x[0]+(0.944437317648)*x[1]**o+(-0.614529825138)*x[1]
        s[5,0,0]=(0.817613090618)*x[0]**o+(-0.611860825889)*x[0]+(-0.425821334232)*x[1]**o+(0.657829198468)*x[1]
        s[5,0,1]=(-0.562766866656)*x[0]**o+(0.374639711066)*x[0]+(0.5381903866)*x[1]**o+(0.642409148669)*x[1]
        s[5,1,0]=(0.98365300543)*x[0]**o+(0.95143935497)*x[0]+(-0.186391221276)*x[1]**o+(0.99144952063)*x[1]
        s[5,1,1]=(0.422131909022)*x[0]**o+(-0.503167260902)*x[0]+(-0.708590674746)*x[1]**o+(0.164597708191)*x[1]
        ref[0,0,0]=(0.748658213139)*x_ref[0]**o+(0.588706109747)*x_ref[0]+(-0.462395788697)*x_ref[1]**o+(0.558348215492)*x_ref[1]
        ref[0,0,1]=(0.629957777572)*x_ref[0]**o+(0.714493090646)*x_ref[0]+(-0.491054832228)*x_ref[1]**o+(0.710860562012)*x_ref[1]
        ref[0,1,0]=(-0.775354422191)*x_ref[0]**o+(0.152096744898)*x_ref[0]+(0.717568022676)*x_ref[1]**o+(0.749449532926)*x_ref[1]
        ref[0,1,1]=(0.883871444687)*x_ref[0]**o+(0.149367558371)*x_ref[0]+(0.986279594706)*x_ref[1]**o+(0.537229955955)*x_ref[1]
        ref[1,0,0]=(0.564688441996)*x_ref[0]**o+(-0.206862774902)*x_ref[0]+(0.522967069439)*x_ref[1]**o+(-0.0390132413525)*x_ref[1]
        ref[1,0,1]=(0.706775860383)*x_ref[0]**o+(0.973359689319)*x_ref[0]+(0.567062178556)*x_ref[1]**o+(-0.00264874871685)*x_ref[1]
        ref[1,1,0]=(-0.385131437493)*x_ref[0]**o+(0.93223208004)*x_ref[0]+(0.915604601744)*x_ref[1]**o+(-0.604136431942)*x_ref[1]
        ref[1,1,1]=(-0.58509656422)*x_ref[0]**o+(-0.46833032365)*x_ref[0]+(0.143097299013)*x_ref[1]**o+(-0.685071549088)*x_ref[1]
        ref[2,0,0]=(-0.987452159168)*x_ref[0]**o+(0.609343916634)*x_ref[0]+(0.425443014801)*x_ref[1]**o+(0.58099336693)*x_ref[1]
        ref[2,0,1]=(0.957582337028)*x_ref[0]**o+(0.62772269136)*x_ref[0]+(-0.783891315097)*x_ref[1]**o+(-0.77280642716)*x_ref[1]
        ref[2,1,0]=(0.327068296677)*x_ref[0]**o+(0.0579535708335)*x_ref[0]+(-0.798372147412)*x_ref[1]**o+(-0.517396772711)*x_ref[1]
        ref[2,1,1]=(-0.863293845828)*x_ref[0]**o+(0.688257458796)*x_ref[0]+(0.777044937444)*x_ref[1]**o+(0.210894669552)*x_ref[1]
        ref[3,0,0]=(0.641389559724)*x_ref[0]**o+(-0.115831890978)*x_ref[0]+(0.0852026159583)*x_ref[1]**o+(0.101596282721)*x_ref[1]
        ref[3,0,1]=(0.0869751746538)*x_ref[0]**o+(0.8310099269)*x_ref[0]+(0.915409474965)*x_ref[1]**o+(-0.140400418634)*x_ref[1]
        ref[3,1,0]=(-0.782218646979)*x_ref[0]**o+(0.259242630384)*x_ref[0]+(0.474259881734)*x_ref[1]**o+(-0.339247837819)*x_ref[1]
        ref[3,1,1]=(0.110237392658)*x_ref[0]**o+(-0.631167781024)*x_ref[0]+(-0.901066615275)*x_ref[1]**o+(0.380918387854)*x_ref[1]
        ref[4,0,0]=(-0.377519984827)*x_ref[0]**o+(-0.77772320212)*x_ref[0]+(0.441380184654)*x_ref[1]**o+(0.601046552584)*x_ref[1]
        ref[4,0,1]=(-0.563858988542)*x_ref[0]**o+(0.00445139485154)*x_ref[0]+(-0.294713455734)*x_ref[1]**o+(-0.863742009505)*x_ref[1]
        ref[4,1,0]=(-0.0472447099394)*x_ref[0]**o+(-0.192880217627)*x_ref[0]+(-0.299507318615)*x_ref[1]**o+(-0.372320397063)*x_ref[1]
        ref[4,1,1]=(-0.81522394336)*x_ref[0]**o+(0.822036737098)*x_ref[0]+(0.944437317648)*x_ref[1]**o+(-0.614529825138)*x_ref[1]
        ref[5,0,0]=(0.817613090618)*x_ref[0]**o+(-0.611860825889)*x_ref[0]+(-0.425821334232)*x_ref[1]**o+(0.657829198468)*x_ref[1]
        ref[5,0,1]=(-0.562766866656)*x_ref[0]**o+(0.374639711066)*x_ref[0]+(0.5381903866)*x_ref[1]**o+(0.642409148669)*x_ref[1]
        ref[5,1,0]=(0.98365300543)*x_ref[0]**o+(0.95143935497)*x_ref[0]+(-0.186391221276)*x_ref[1]**o+(0.99144952063)*x_ref[1]
        ref[5,1,1]=(0.422131909022)*x_ref[0]**o+(-0.503167260902)*x_ref[0]+(-0.708590674746)*x_ref[1]**o+(0.164597708191)*x_ref[1]
      else:
        s[0,0,0]=(0.286638887843)*x[0]**o+(0.522689277676)*x[0]+(0.467954875136)*x[1]**o+(0.445711366876)*x[1]+(-0.669973962002)*x[2]**o+(0.181395269699)*x[2]
        s[0,0,1]=(0.353016816809)*x[0]**o+(0.991829804601)*x[0]+(-0.258315747172)*x[1]**o+(0.543990055925)*x[1]+(-0.769057570955)*x[2]**o+(0.233571124807)*x[2]
        s[0,1,0]=(0.918139015102)*x[0]**o+(-0.947988526027)*x[0]+(0.331622282377)*x[1]**o+(-0.777541428786)*x[1]+(-0.664961721637)*x[2]**o+(-0.484643482585)*x[2]
        s[0,1,1]=(0.140004476479)*x[0]**o+(-0.610763924636)*x[0]+(0.789189906539)*x[1]**o+(-0.590757211504)*x[1]+(-0.0725209360897)*x[2]**o+(-0.766615663399)*x[2]
        s[1,0,0]=(0.725731333346)*x[0]**o+(-0.131718022099)*x[0]+(0.627777941816)*x[1]**o+(-0.482136262357)*x[1]+(0.413942360645)*x[2]**o+(-0.987161567454)*x[2]
        s[1,0,1]=(0.975906575301)*x[0]**o+(0.642044782609)*x[0]+(0.670119948857)*x[1]**o+(-0.93125168252)*x[1]+(-0.553725567396)*x[2]**o+(0.609585560905)*x[2]
        s[1,1,0]=(-0.073682338522)*x[0]**o+(-0.391983504903)*x[0]+(0.252884640925)*x[1]**o+(0.474040995664)*x[1]+(-0.763628395477)*x[2]**o+(-0.753583810078)*x[2]
        s[1,1,1]=(-0.680881408584)*x[0]**o+(-0.546411376016)*x[0]+(0.85433594417)*x[1]**o+(-0.990738365487)*x[1]+(-0.202162243501)*x[2]**o+(0.256411615615)*x[2]
        s[2,0,0]=(0.79289192889)*x[0]**o+(0.243831874798)*x[0]+(0.681415436664)*x[1]**o+(0.223205387356)*x[1]+(0.517155768633)*x[2]**o+(-0.283058167273)*x[2]
        s[2,0,1]=(-0.608869102346)*x[0]**o+(0.496990925284)*x[0]+(0.925407693967)*x[1]**o+(-0.358933716971)*x[1]+(-0.328654182706)*x[2]**o+(0.835965636453)*x[2]
        s[2,1,0]=(-0.830133878676)*x[0]**o+(-0.123104434072)*x[0]+(-0.662614589611)*x[1]**o+(-0.288827930561)*x[1]+(-0.411343484191)*x[2]**o+(0.632500606896)*x[2]
        s[2,1,1]=(0.272639418094)*x[0]**o+(-0.897989787007)*x[0]+(-0.838744589589)*x[1]**o+(0.492441488123)*x[1]+(-0.634424482735)*x[2]**o+(-0.509179635903)*x[2]
        s[3,0,0]=(-0.810786146104)*x[0]**o+(-0.582309522585)*x[0]+(-0.0537085055431)*x[1]**o+(0.230345910939)*x[1]+(0.482750960353)*x[2]**o+(0.248433602737)*x[2]
        s[3,0,1]=(0.148959585193)*x[0]**o+(0.222768449067)*x[0]+(0.819219237595)*x[1]**o+(0.386479159121)*x[1]+(-0.0808464606991)*x[2]**o+(-0.0578651077709)*x[2]
        s[3,1,0]=(-0.665745862321)*x[0]**o+(-0.864332853905)*x[0]+(-0.622626855999)*x[1]**o+(-0.961729702089)*x[1]+(-0.0528887263986)*x[2]**o+(0.203930539621)*x[2]
        s[3,1,1]=(0.216834395994)*x[0]**o+(0.14686771427)*x[0]+(-0.634838825712)*x[1]**o+(-0.0741875710555)*x[1]+(0.49863557383)*x[2]**o+(0.128134148051)*x[2]
        s[4,0,0]=(-0.0429185773768)*x[0]**o+(-0.884386434806)*x[0]+(0.590953595756)*x[1]**o+(0.229045075384)*x[1]+(0.289339613114)*x[2]**o+(0.799614281102)*x[2]
        s[4,0,1]=(0.401237094734)*x[0]**o+(0.374532696487)*x[0]+(0.382784705093)*x[1]**o+(-0.00752495997413)*x[1]+(-0.693554885405)*x[2]**o+(-0.0030745706685)*x[2]
        s[4,1,0]=(0.746124554182)*x[0]**o+(-0.863459707092)*x[0]+(0.586647115827)*x[1]**o+(-0.94164686768)*x[1]+(0.700937522698)*x[2]**o+(-0.0464264819588)*x[2]
        s[4,1,1]=(0.33270904188)*x[0]**o+(0.90735243167)*x[0]+(-0.479962340673)*x[1]**o+(-0.860540809179)*x[1]+(0.396295831574)*x[2]**o+(-0.47164342577)*x[2]
        s[5,0,0]=(-0.518396400617)*x[0]**o+(0.0893385548496)*x[0]+(0.695842336307)*x[1]**o+(0.552511723893)*x[1]+(0.0636197708084)*x[2]**o+(0.0221143500177)*x[2]
        s[5,0,1]=(0.612931629659)*x[0]**o+(-0.898561600235)*x[0]+(0.231387395725)*x[1]**o+(0.728648837106)*x[1]+(-0.819722749156)*x[2]**o+(-0.987410960592)*x[2]
        s[5,1,0]=(0.228541632142)*x[0]**o+(0.0352301546347)*x[0]+(-0.457859957282)*x[1]**o+(0.126487825769)*x[1]+(-0.685267391542)*x[2]**o+(-0.695662688997)*x[2]
        s[5,1,1]=(0.892154759933)*x[0]**o+(0.0404556267055)*x[0]+(-0.509790912326)*x[1]**o+(0.544457102938)*x[1]+(-0.110692172028)*x[2]**o+(-0.111481287291)*x[2]
        ref[0,0,0]=(0.286638887843)*x_ref[0]**o+(0.522689277676)*x_ref[0]+(0.467954875136)*x_ref[1]**o+(0.445711366876)*x_ref[1]+(-0.669973962002)*x_ref[2]**o+(0.181395269699)*x_ref[2]
        ref[0,0,1]=(0.353016816809)*x_ref[0]**o+(0.991829804601)*x_ref[0]+(-0.258315747172)*x_ref[1]**o+(0.543990055925)*x_ref[1]+(-0.769057570955)*x_ref[2]**o+(0.233571124807)*x_ref[2]
        ref[0,1,0]=(0.918139015102)*x_ref[0]**o+(-0.947988526027)*x_ref[0]+(0.331622282377)*x_ref[1]**o+(-0.777541428786)*x_ref[1]+(-0.664961721637)*x_ref[2]**o+(-0.484643482585)*x_ref[2]
        ref[0,1,1]=(0.140004476479)*x_ref[0]**o+(-0.610763924636)*x_ref[0]+(0.789189906539)*x_ref[1]**o+(-0.590757211504)*x_ref[1]+(-0.0725209360897)*x_ref[2]**o+(-0.766615663399)*x_ref[2]
        ref[1,0,0]=(0.725731333346)*x_ref[0]**o+(-0.131718022099)*x_ref[0]+(0.627777941816)*x_ref[1]**o+(-0.482136262357)*x_ref[1]+(0.413942360645)*x_ref[2]**o+(-0.987161567454)*x_ref[2]
        ref[1,0,1]=(0.975906575301)*x_ref[0]**o+(0.642044782609)*x_ref[0]+(0.670119948857)*x_ref[1]**o+(-0.93125168252)*x_ref[1]+(-0.553725567396)*x_ref[2]**o+(0.609585560905)*x_ref[2]
        ref[1,1,0]=(-0.073682338522)*x_ref[0]**o+(-0.391983504903)*x_ref[0]+(0.252884640925)*x_ref[1]**o+(0.474040995664)*x_ref[1]+(-0.763628395477)*x_ref[2]**o+(-0.753583810078)*x_ref[2]
        ref[1,1,1]=(-0.680881408584)*x_ref[0]**o+(-0.546411376016)*x_ref[0]+(0.85433594417)*x_ref[1]**o+(-0.990738365487)*x_ref[1]+(-0.202162243501)*x_ref[2]**o+(0.256411615615)*x_ref[2]
        ref[2,0,0]=(0.79289192889)*x_ref[0]**o+(0.243831874798)*x_ref[0]+(0.681415436664)*x_ref[1]**o+(0.223205387356)*x_ref[1]+(0.517155768633)*x_ref[2]**o+(-0.283058167273)*x_ref[2]
        ref[2,0,1]=(-0.608869102346)*x_ref[0]**o+(0.496990925284)*x_ref[0]+(0.925407693967)*x_ref[1]**o+(-0.358933716971)*x_ref[1]+(-0.328654182706)*x_ref[2]**o+(0.835965636453)*x_ref[2]
        ref[2,1,0]=(-0.830133878676)*x_ref[0]**o+(-0.123104434072)*x_ref[0]+(-0.662614589611)*x_ref[1]**o+(-0.288827930561)*x_ref[1]+(-0.411343484191)*x_ref[2]**o+(0.632500606896)*x_ref[2]
        ref[2,1,1]=(0.272639418094)*x_ref[0]**o+(-0.897989787007)*x_ref[0]+(-0.838744589589)*x_ref[1]**o+(0.492441488123)*x_ref[1]+(-0.634424482735)*x_ref[2]**o+(-0.509179635903)*x_ref[2]
        ref[3,0,0]=(-0.810786146104)*x_ref[0]**o+(-0.582309522585)*x_ref[0]+(-0.0537085055431)*x_ref[1]**o+(0.230345910939)*x_ref[1]+(0.482750960353)*x_ref[2]**o+(0.248433602737)*x_ref[2]
        ref[3,0,1]=(0.148959585193)*x_ref[0]**o+(0.222768449067)*x_ref[0]+(0.819219237595)*x_ref[1]**o+(0.386479159121)*x_ref[1]+(-0.0808464606991)*x_ref[2]**o+(-0.0578651077709)*x_ref[2]
        ref[3,1,0]=(-0.665745862321)*x_ref[0]**o+(-0.864332853905)*x_ref[0]+(-0.622626855999)*x_ref[1]**o+(-0.961729702089)*x_ref[1]+(-0.0528887263986)*x_ref[2]**o+(0.203930539621)*x_ref[2]
        ref[3,1,1]=(0.216834395994)*x_ref[0]**o+(0.14686771427)*x_ref[0]+(-0.634838825712)*x_ref[1]**o+(-0.0741875710555)*x_ref[1]+(0.49863557383)*x_ref[2]**o+(0.128134148051)*x_ref[2]
        ref[4,0,0]=(-0.0429185773768)*x_ref[0]**o+(-0.884386434806)*x_ref[0]+(0.590953595756)*x_ref[1]**o+(0.229045075384)*x_ref[1]+(0.289339613114)*x_ref[2]**o+(0.799614281102)*x_ref[2]
        ref[4,0,1]=(0.401237094734)*x_ref[0]**o+(0.374532696487)*x_ref[0]+(0.382784705093)*x_ref[1]**o+(-0.00752495997413)*x_ref[1]+(-0.693554885405)*x_ref[2]**o+(-0.0030745706685)*x_ref[2]
        ref[4,1,0]=(0.746124554182)*x_ref[0]**o+(-0.863459707092)*x_ref[0]+(0.586647115827)*x_ref[1]**o+(-0.94164686768)*x_ref[1]+(0.700937522698)*x_ref[2]**o+(-0.0464264819588)*x_ref[2]
        ref[4,1,1]=(0.33270904188)*x_ref[0]**o+(0.90735243167)*x_ref[0]+(-0.479962340673)*x_ref[1]**o+(-0.860540809179)*x_ref[1]+(0.396295831574)*x_ref[2]**o+(-0.47164342577)*x_ref[2]
        ref[5,0,0]=(-0.518396400617)*x_ref[0]**o+(0.0893385548496)*x_ref[0]+(0.695842336307)*x_ref[1]**o+(0.552511723893)*x_ref[1]+(0.0636197708084)*x_ref[2]**o+(0.0221143500177)*x_ref[2]
        ref[5,0,1]=(0.612931629659)*x_ref[0]**o+(-0.898561600235)*x_ref[0]+(0.231387395725)*x_ref[1]**o+(0.728648837106)*x_ref[1]+(-0.819722749156)*x_ref[2]**o+(-0.987410960592)*x_ref[2]
        ref[5,1,0]=(0.228541632142)*x_ref[0]**o+(0.0352301546347)*x_ref[0]+(-0.457859957282)*x_ref[1]**o+(0.126487825769)*x_ref[1]+(-0.685267391542)*x_ref[2]**o+(-0.695662688997)*x_ref[2]
        ref[5,1,1]=(0.892154759933)*x_ref[0]**o+(0.0404556267055)*x_ref[0]+(-0.509790912326)*x_ref[1]**o+(0.544457102938)*x_ref[1]+(-0.110692172028)*x_ref[2]**o+(-0.111481287291)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_ReducedFunctionOnContactZero_rank4(self):
      """
      tests interpolate for rank 4 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactZero onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactZero(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5, 3, 2),dim=dim)
      s=Data(0,(4, 5, 3, 2),w)
      ref=Data(0,(4, 5, 3, 2),w_ref)
      if dim==2:
        s[0,0,0,0]=(-0.0573009878147)*x[0]**o+(-0.350301437886)*x[0]+(0.253659046023)*x[1]**o+(0.658249302609)*x[1]
        s[0,0,0,1]=(0.830676080978)*x[0]**o+(-0.809626423382)*x[0]+(0.370613645219)*x[1]**o+(-0.172834681384)*x[1]
        s[0,0,1,0]=(-0.735478785962)*x[0]**o+(0.512724546806)*x[0]+(-0.717033770258)*x[1]**o+(-0.0946841286843)*x[1]
        s[0,0,1,1]=(-0.715337226505)*x[0]**o+(0.1253433121)*x[0]+(0.230909585243)*x[1]**o+(0.470778883904)*x[1]
        s[0,0,2,0]=(0.949793126568)*x[0]**o+(-0.12899598886)*x[0]+(-0.749733392851)*x[1]**o+(-0.340807421479)*x[1]
        s[0,0,2,1]=(0.261609659356)*x[0]**o+(0.336166427172)*x[0]+(0.657111532027)*x[1]**o+(-0.614814002143)*x[1]
        s[0,1,0,0]=(-0.0275060361172)*x[0]**o+(0.966092296743)*x[0]+(-0.532890777604)*x[1]**o+(0.517696025589)*x[1]
        s[0,1,0,1]=(0.00166569160565)*x[0]**o+(0.236625486597)*x[0]+(0.893101928693)*x[1]**o+(-0.574929038375)*x[1]
        s[0,1,1,0]=(0.872269789656)*x[0]**o+(-0.905740937959)*x[0]+(0.719538254619)*x[1]**o+(0.711635238494)*x[1]
        s[0,1,1,1]=(0.801801659725)*x[0]**o+(0.139019380599)*x[0]+(0.708524489966)*x[1]**o+(-0.77122453375)*x[1]
        s[0,1,2,0]=(-0.0481701009478)*x[0]**o+(0.167249653575)*x[0]+(0.0551704085528)*x[1]**o+(0.505944176073)*x[1]
        s[0,1,2,1]=(0.185898813821)*x[0]**o+(-0.750605793976)*x[0]+(-0.172006188497)*x[1]**o+(-0.0840963686282)*x[1]
        s[0,2,0,0]=(0.499076840583)*x[0]**o+(-0.230482058386)*x[0]+(0.522046309086)*x[1]**o+(-0.445372865956)*x[1]
        s[0,2,0,1]=(0.643144964505)*x[0]**o+(0.754412438054)*x[0]+(-0.527488849658)*x[1]**o+(-0.781951420648)*x[1]
        s[0,2,1,0]=(-0.428090027626)*x[0]**o+(0.941724629359)*x[0]+(-0.888361304631)*x[1]**o+(-0.193082322033)*x[1]
        s[0,2,1,1]=(0.737854520423)*x[0]**o+(-0.600462005647)*x[0]+(0.523944735668)*x[1]**o+(-0.88314904941)*x[1]
        s[0,2,2,0]=(0.401651360188)*x[0]**o+(0.605266809429)*x[0]+(0.114190031414)*x[1]**o+(-0.0554731448399)*x[1]
        s[0,2,2,1]=(0.49575503694)*x[0]**o+(0.842572557458)*x[0]+(-0.65557628937)*x[1]**o+(-0.110654902831)*x[1]
        s[0,3,0,0]=(-0.914580503709)*x[0]**o+(0.568110262697)*x[0]+(0.465998898456)*x[1]**o+(0.668680449802)*x[1]
        s[0,3,0,1]=(-0.406517611176)*x[0]**o+(0.751773194945)*x[0]+(-0.546709674237)*x[1]**o+(0.830905534576)*x[1]
        s[0,3,1,0]=(0.256170881332)*x[0]**o+(0.785200644094)*x[0]+(-0.0949323527814)*x[1]**o+(0.437799246842)*x[1]
        s[0,3,1,1]=(0.388926885643)*x[0]**o+(0.0649842722847)*x[0]+(-0.570138074228)*x[1]**o+(0.0125008958832)*x[1]
        s[0,3,2,0]=(-0.693124111229)*x[0]**o+(-0.135531989861)*x[0]+(-0.484364583498)*x[1]**o+(0.142028440685)*x[1]
        s[0,3,2,1]=(0.162852074905)*x[0]**o+(-0.211709648647)*x[0]+(-0.424964818278)*x[1]**o+(-0.0311289175964)*x[1]
        s[0,4,0,0]=(-0.893934248272)*x[0]**o+(-0.883334046545)*x[0]+(0.141127275532)*x[1]**o+(0.790251212849)*x[1]
        s[0,4,0,1]=(0.751030754129)*x[0]**o+(-0.0463112594115)*x[0]+(0.894754601054)*x[1]**o+(-0.986137408191)*x[1]
        s[0,4,1,0]=(-0.994216054583)*x[0]**o+(0.989450915377)*x[0]+(0.202383847504)*x[1]**o+(-0.829413728527)*x[1]
        s[0,4,1,1]=(-0.70581665488)*x[0]**o+(0.387180461053)*x[0]+(0.925669527701)*x[1]**o+(0.422052343171)*x[1]
        s[0,4,2,0]=(-0.498553799214)*x[0]**o+(0.70559513684)*x[0]+(-0.595097550839)*x[1]**o+(-0.177703117092)*x[1]
        s[0,4,2,1]=(-0.703437824074)*x[0]**o+(-0.574613675229)*x[0]+(-0.699959931134)*x[1]**o+(-0.503745753701)*x[1]
        s[1,0,0,0]=(-0.0673123845899)*x[0]**o+(-0.0603073393325)*x[0]+(0.724127339391)*x[1]**o+(-0.0875500989225)*x[1]
        s[1,0,0,1]=(0.0766117819496)*x[0]**o+(-0.489029658255)*x[0]+(0.602970700568)*x[1]**o+(0.624180485873)*x[1]
        s[1,0,1,0]=(0.537236166297)*x[0]**o+(0.805657082915)*x[0]+(-0.0263572051431)*x[1]**o+(0.683790156906)*x[1]
        s[1,0,1,1]=(0.55427198724)*x[0]**o+(0.631937641401)*x[0]+(-0.634744046324)*x[1]**o+(0.824382447181)*x[1]
        s[1,0,2,0]=(0.694551886505)*x[0]**o+(-0.123561028797)*x[0]+(-0.0237233372904)*x[1]**o+(-0.822938415295)*x[1]
        s[1,0,2,1]=(0.293944558832)*x[0]**o+(-0.34741820803)*x[0]+(-0.471269455638)*x[1]**o+(0.716959839747)*x[1]
        s[1,1,0,0]=(0.409876005614)*x[0]**o+(-0.322385353356)*x[0]+(0.362023900558)*x[1]**o+(-0.974915188765)*x[1]
        s[1,1,0,1]=(-0.823587457611)*x[0]**o+(-0.315566397595)*x[0]+(-0.681951254575)*x[1]**o+(-0.945803349426)*x[1]
        s[1,1,1,0]=(0.782950007967)*x[0]**o+(0.767220278425)*x[0]+(0.109277917483)*x[1]**o+(-0.336228598202)*x[1]
        s[1,1,1,1]=(-0.969811850973)*x[0]**o+(0.453588680715)*x[0]+(-0.296932776592)*x[1]**o+(-0.460668458023)*x[1]
        s[1,1,2,0]=(0.124616545015)*x[0]**o+(0.180238150583)*x[0]+(0.283723867146)*x[1]**o+(0.873774600692)*x[1]
        s[1,1,2,1]=(0.00334202091213)*x[0]**o+(-0.711821714868)*x[0]+(0.638247490899)*x[1]**o+(-0.409696943449)*x[1]
        s[1,2,0,0]=(-0.206885052591)*x[0]**o+(-0.677245253776)*x[0]+(-0.508637512178)*x[1]**o+(0.0862259084563)*x[1]
        s[1,2,0,1]=(0.402289447161)*x[0]**o+(0.242246771389)*x[0]+(0.775293224518)*x[1]**o+(-0.668881714475)*x[1]
        s[1,2,1,0]=(-0.094983969904)*x[0]**o+(0.615306379484)*x[0]+(-0.737839325648)*x[1]**o+(0.457810359747)*x[1]
        s[1,2,1,1]=(0.488668557736)*x[0]**o+(0.188170233286)*x[0]+(0.0947759637418)*x[1]**o+(-0.510206729955)*x[1]
        s[1,2,2,0]=(-0.102204043127)*x[0]**o+(0.584307889877)*x[0]+(-0.779746892074)*x[1]**o+(-0.767757863288)*x[1]
        s[1,2,2,1]=(-0.00230617324952)*x[0]**o+(-0.184632788675)*x[0]+(-0.779494086898)*x[1]**o+(-0.105311421755)*x[1]
        s[1,3,0,0]=(0.373541105531)*x[0]**o+(-0.895909586816)*x[0]+(0.853523284159)*x[1]**o+(0.915229883403)*x[1]
        s[1,3,0,1]=(-0.888315748473)*x[0]**o+(0.877859466924)*x[0]+(-0.830380928425)*x[1]**o+(-0.25922026388)*x[1]
        s[1,3,1,0]=(-0.944775455365)*x[0]**o+(0.694978915121)*x[0]+(0.00346060944841)*x[1]**o+(0.610096191934)*x[1]
        s[1,3,1,1]=(-0.175304757445)*x[0]**o+(0.883688655763)*x[0]+(-0.0260273718271)*x[1]**o+(0.440569935385)*x[1]
        s[1,3,2,0]=(0.973418372218)*x[0]**o+(-0.0725489149125)*x[0]+(-0.790861964708)*x[1]**o+(-0.764398676127)*x[1]
        s[1,3,2,1]=(0.188665821479)*x[0]**o+(-0.694573366343)*x[0]+(-0.860244164818)*x[1]**o+(0.642736876841)*x[1]
        s[1,4,0,0]=(-0.619221357105)*x[0]**o+(-0.0366612230467)*x[0]+(0.920044684933)*x[1]**o+(-0.901158565327)*x[1]
        s[1,4,0,1]=(-0.774227719954)*x[0]**o+(0.471966691541)*x[0]+(0.0706379157745)*x[1]**o+(-0.238713884878)*x[1]
        s[1,4,1,0]=(0.863551422475)*x[0]**o+(0.131078367733)*x[0]+(0.458326651999)*x[1]**o+(-0.750431841864)*x[1]
        s[1,4,1,1]=(0.105763306948)*x[0]**o+(-0.254538567936)*x[0]+(-0.559297954287)*x[1]**o+(0.21981759162)*x[1]
        s[1,4,2,0]=(0.922873574086)*x[0]**o+(-0.853230802788)*x[0]+(-0.982798222021)*x[1]**o+(-0.0866961947026)*x[1]
        s[1,4,2,1]=(-0.21664472336)*x[0]**o+(-0.915876043584)*x[0]+(0.6182964805)*x[1]**o+(0.665515072302)*x[1]
        s[2,0,0,0]=(-0.491230924471)*x[0]**o+(0.509670721146)*x[0]+(-0.44580759248)*x[1]**o+(-0.943168309946)*x[1]
        s[2,0,0,1]=(-0.0895802535424)*x[0]**o+(0.0780292649662)*x[0]+(-0.42293922141)*x[1]**o+(-0.76928974668)*x[1]
        s[2,0,1,0]=(-0.404348433578)*x[0]**o+(-0.986117091427)*x[0]+(0.562569938626)*x[1]**o+(-0.41119675761)*x[1]
        s[2,0,1,1]=(0.517587873308)*x[0]**o+(0.0731347448674)*x[0]+(0.830876774432)*x[1]**o+(-0.690590335528)*x[1]
        s[2,0,2,0]=(0.600931766711)*x[0]**o+(-0.786774940006)*x[0]+(-0.285975800855)*x[1]**o+(0.348122549519)*x[1]
        s[2,0,2,1]=(0.825033481951)*x[0]**o+(-0.744551393479)*x[0]+(0.452879660703)*x[1]**o+(-0.257025658518)*x[1]
        s[2,1,0,0]=(-0.0822184021881)*x[0]**o+(0.955001628077)*x[0]+(-0.878009442237)*x[1]**o+(-0.816970793701)*x[1]
        s[2,1,0,1]=(-0.151473282662)*x[0]**o+(0.937636279558)*x[0]+(-0.969793512643)*x[1]**o+(0.704581030148)*x[1]
        s[2,1,1,0]=(0.76728625844)*x[0]**o+(-0.90294390262)*x[0]+(-0.375443980814)*x[1]**o+(0.276219230561)*x[1]
        s[2,1,1,1]=(-0.994260771722)*x[0]**o+(0.64595057257)*x[0]+(0.486576661947)*x[1]**o+(0.731738848734)*x[1]
        s[2,1,2,0]=(-0.52048273513)*x[0]**o+(-0.51915057929)*x[0]+(0.184855975415)*x[1]**o+(0.417869171454)*x[1]
        s[2,1,2,1]=(-0.371019640816)*x[0]**o+(-0.617014410896)*x[0]+(0.725259821636)*x[1]**o+(-0.0393667759542)*x[1]
        s[2,2,0,0]=(0.070044943001)*x[0]**o+(-0.663066522147)*x[0]+(0.761006491068)*x[1]**o+(-0.137480662114)*x[1]
        s[2,2,0,1]=(-0.394035586896)*x[0]**o+(0.802334732427)*x[0]+(-0.191154645494)*x[1]**o+(-0.278490557185)*x[1]
        s[2,2,1,0]=(0.0664932052956)*x[0]**o+(0.938378455125)*x[0]+(0.997119543949)*x[1]**o+(-0.696820719616)*x[1]
        s[2,2,1,1]=(0.79758586389)*x[0]**o+(0.0393291272874)*x[0]+(0.644980805887)*x[1]**o+(0.61925875842)*x[1]
        s[2,2,2,0]=(0.0353810358089)*x[0]**o+(-0.0134652375886)*x[0]+(-0.942381983803)*x[1]**o+(0.288697669014)*x[1]
        s[2,2,2,1]=(-0.327577257504)*x[0]**o+(0.816374902745)*x[0]+(0.676853003934)*x[1]**o+(0.581394632094)*x[1]
        s[2,3,0,0]=(0.63036222221)*x[0]**o+(-0.899724810809)*x[0]+(0.276882104091)*x[1]**o+(-0.00448820301083)*x[1]
        s[2,3,0,1]=(-0.219899069159)*x[0]**o+(-0.439244974227)*x[0]+(-0.369366024932)*x[1]**o+(-0.161805301574)*x[1]
        s[2,3,1,0]=(0.165202872055)*x[0]**o+(-0.21521932039)*x[0]+(0.635188695229)*x[1]**o+(-0.254785217427)*x[1]
        s[2,3,1,1]=(0.743546803617)*x[0]**o+(0.876160636955)*x[0]+(-0.094730937905)*x[1]**o+(-0.898315912992)*x[1]
        s[2,3,2,0]=(0.946538368626)*x[0]**o+(0.715488078243)*x[0]+(0.830112020529)*x[1]**o+(-0.320605753644)*x[1]
        s[2,3,2,1]=(0.52106888624)*x[0]**o+(-0.938072665733)*x[0]+(-0.682830472037)*x[1]**o+(-0.787442211636)*x[1]
        s[2,4,0,0]=(0.802634158133)*x[0]**o+(0.350923640202)*x[0]+(-0.142397660222)*x[1]**o+(0.768827179039)*x[1]
        s[2,4,0,1]=(0.912207226957)*x[0]**o+(0.302955742414)*x[0]+(-0.125141454725)*x[1]**o+(0.115375955845)*x[1]
        s[2,4,1,0]=(0.88629189066)*x[0]**o+(-0.431145868333)*x[0]+(0.521817592458)*x[1]**o+(0.757372657079)*x[1]
        s[2,4,1,1]=(-0.34619470039)*x[0]**o+(0.0971941022378)*x[0]+(-0.235321219876)*x[1]**o+(0.919340221916)*x[1]
        s[2,4,2,0]=(-0.453744205858)*x[0]**o+(0.386107152737)*x[0]+(-0.53829794769)*x[1]**o+(-0.955985738202)*x[1]
        s[2,4,2,1]=(0.327564798989)*x[0]**o+(0.234176645121)*x[0]+(-0.647242055069)*x[1]**o+(-0.914279617484)*x[1]
        s[3,0,0,0]=(-0.765850994681)*x[0]**o+(0.0969021602734)*x[0]+(-0.003697715708)*x[1]**o+(0.642293903037)*x[1]
        s[3,0,0,1]=(-0.736912760299)*x[0]**o+(-0.496756935434)*x[0]+(0.00160221880808)*x[1]**o+(-0.0275314787811)*x[1]
        s[3,0,1,0]=(0.266302600859)*x[0]**o+(0.962232443249)*x[0]+(0.588094142167)*x[1]**o+(-0.740113379551)*x[1]
        s[3,0,1,1]=(-0.701204282707)*x[0]**o+(0.16377449778)*x[0]+(-0.636725536715)*x[1]**o+(0.925340403623)*x[1]
        s[3,0,2,0]=(-0.473492482064)*x[0]**o+(0.356976796364)*x[0]+(0.28490257392)*x[1]**o+(0.558859192591)*x[1]
        s[3,0,2,1]=(-0.398155056481)*x[0]**o+(-0.693010191754)*x[0]+(-0.457570700744)*x[1]**o+(0.899032980684)*x[1]
        s[3,1,0,0]=(0.676626616862)*x[0]**o+(-0.142127205425)*x[0]+(-0.0703163466342)*x[1]**o+(-0.603767804268)*x[1]
        s[3,1,0,1]=(0.901299559593)*x[0]**o+(-0.319119050502)*x[0]+(-0.230855076893)*x[1]**o+(0.923789250137)*x[1]
        s[3,1,1,0]=(0.0125691057912)*x[0]**o+(0.827401151454)*x[0]+(0.0881382970418)*x[1]**o+(0.12796232574)*x[1]
        s[3,1,1,1]=(-0.544917801861)*x[0]**o+(-0.163923780568)*x[0]+(0.107833995954)*x[1]**o+(-0.347470857919)*x[1]
        s[3,1,2,0]=(-0.127281534891)*x[0]**o+(0.410172113868)*x[0]+(0.516300378909)*x[1]**o+(-0.454031679856)*x[1]
        s[3,1,2,1]=(-0.797619552686)*x[0]**o+(0.0212633210895)*x[0]+(-0.0890328154993)*x[1]**o+(0.755392852372)*x[1]
        s[3,2,0,0]=(0.00976392699076)*x[0]**o+(-0.608813040196)*x[0]+(0.786997931081)*x[1]**o+(0.637450406747)*x[1]
        s[3,2,0,1]=(-0.594915968211)*x[0]**o+(0.909106492284)*x[0]+(0.209774113888)*x[1]**o+(0.926156671099)*x[1]
        s[3,2,1,0]=(0.226007901841)*x[0]**o+(-0.43873163217)*x[0]+(-0.414778768373)*x[1]**o+(0.681053132578)*x[1]
        s[3,2,1,1]=(0.733375971867)*x[0]**o+(0.861188515019)*x[0]+(0.036782797193)*x[1]**o+(0.145181529782)*x[1]
        s[3,2,2,0]=(0.635727397799)*x[0]**o+(0.874128725841)*x[0]+(-0.810169847222)*x[1]**o+(0.234369619159)*x[1]
        s[3,2,2,1]=(-0.137252742544)*x[0]**o+(0.817564886208)*x[0]+(0.524706682726)*x[1]**o+(0.381735970369)*x[1]
        s[3,3,0,0]=(0.537368746859)*x[0]**o+(-0.298760793455)*x[0]+(-0.890807690497)*x[1]**o+(-0.662222369966)*x[1]
        s[3,3,0,1]=(-0.67672238667)*x[0]**o+(-0.305431277061)*x[0]+(-0.435551021687)*x[1]**o+(0.0777923105041)*x[1]
        s[3,3,1,0]=(-0.163284186346)*x[0]**o+(-0.764058727495)*x[0]+(-0.255689425142)*x[1]**o+(0.285528254098)*x[1]
        s[3,3,1,1]=(-0.842634439814)*x[0]**o+(-0.194811378723)*x[0]+(0.814424721344)*x[1]**o+(-0.236826785563)*x[1]
        s[3,3,2,0]=(0.532936591284)*x[0]**o+(-0.920822945577)*x[0]+(-0.677841412043)*x[1]**o+(0.579981868038)*x[1]
        s[3,3,2,1]=(-0.470357673624)*x[0]**o+(-0.972798479564)*x[0]+(0.489692897505)*x[1]**o+(0.213937860142)*x[1]
        s[3,4,0,0]=(-0.169170544642)*x[0]**o+(0.543113611166)*x[0]+(0.536912197355)*x[1]**o+(0.304230092305)*x[1]
        s[3,4,0,1]=(-0.170701528868)*x[0]**o+(0.340606153319)*x[0]+(-0.470528666825)*x[1]**o+(-0.522028583228)*x[1]
        s[3,4,1,0]=(0.0194532320642)*x[0]**o+(0.335898702694)*x[0]+(-0.917025993458)*x[1]**o+(-0.539748486826)*x[1]
        s[3,4,1,1]=(-0.576465573138)*x[0]**o+(-0.906917910142)*x[0]+(-0.313000868178)*x[1]**o+(-0.20607256753)*x[1]
        s[3,4,2,0]=(-0.75394368612)*x[0]**o+(0.658866075222)*x[0]+(0.402200040518)*x[1]**o+(-0.544252067119)*x[1]
        s[3,4,2,1]=(0.0490609014064)*x[0]**o+(-0.873887848405)*x[0]+(0.91186722933)*x[1]**o+(0.375269571672)*x[1]
        ref[0,0,0,0]=(-0.0573009878147)*x_ref[0]**o+(-0.350301437886)*x_ref[0]+(0.253659046023)*x_ref[1]**o+(0.658249302609)*x_ref[1]
        ref[0,0,0,1]=(0.830676080978)*x_ref[0]**o+(-0.809626423382)*x_ref[0]+(0.370613645219)*x_ref[1]**o+(-0.172834681384)*x_ref[1]
        ref[0,0,1,0]=(-0.735478785962)*x_ref[0]**o+(0.512724546806)*x_ref[0]+(-0.717033770258)*x_ref[1]**o+(-0.0946841286843)*x_ref[1]
        ref[0,0,1,1]=(-0.715337226505)*x_ref[0]**o+(0.1253433121)*x_ref[0]+(0.230909585243)*x_ref[1]**o+(0.470778883904)*x_ref[1]
        ref[0,0,2,0]=(0.949793126568)*x_ref[0]**o+(-0.12899598886)*x_ref[0]+(-0.749733392851)*x_ref[1]**o+(-0.340807421479)*x_ref[1]
        ref[0,0,2,1]=(0.261609659356)*x_ref[0]**o+(0.336166427172)*x_ref[0]+(0.657111532027)*x_ref[1]**o+(-0.614814002143)*x_ref[1]
        ref[0,1,0,0]=(-0.0275060361172)*x_ref[0]**o+(0.966092296743)*x_ref[0]+(-0.532890777604)*x_ref[1]**o+(0.517696025589)*x_ref[1]
        ref[0,1,0,1]=(0.00166569160565)*x_ref[0]**o+(0.236625486597)*x_ref[0]+(0.893101928693)*x_ref[1]**o+(-0.574929038375)*x_ref[1]
        ref[0,1,1,0]=(0.872269789656)*x_ref[0]**o+(-0.905740937959)*x_ref[0]+(0.719538254619)*x_ref[1]**o+(0.711635238494)*x_ref[1]
        ref[0,1,1,1]=(0.801801659725)*x_ref[0]**o+(0.139019380599)*x_ref[0]+(0.708524489966)*x_ref[1]**o+(-0.77122453375)*x_ref[1]
        ref[0,1,2,0]=(-0.0481701009478)*x_ref[0]**o+(0.167249653575)*x_ref[0]+(0.0551704085528)*x_ref[1]**o+(0.505944176073)*x_ref[1]
        ref[0,1,2,1]=(0.185898813821)*x_ref[0]**o+(-0.750605793976)*x_ref[0]+(-0.172006188497)*x_ref[1]**o+(-0.0840963686282)*x_ref[1]
        ref[0,2,0,0]=(0.499076840583)*x_ref[0]**o+(-0.230482058386)*x_ref[0]+(0.522046309086)*x_ref[1]**o+(-0.445372865956)*x_ref[1]
        ref[0,2,0,1]=(0.643144964505)*x_ref[0]**o+(0.754412438054)*x_ref[0]+(-0.527488849658)*x_ref[1]**o+(-0.781951420648)*x_ref[1]
        ref[0,2,1,0]=(-0.428090027626)*x_ref[0]**o+(0.941724629359)*x_ref[0]+(-0.888361304631)*x_ref[1]**o+(-0.193082322033)*x_ref[1]
        ref[0,2,1,1]=(0.737854520423)*x_ref[0]**o+(-0.600462005647)*x_ref[0]+(0.523944735668)*x_ref[1]**o+(-0.88314904941)*x_ref[1]
        ref[0,2,2,0]=(0.401651360188)*x_ref[0]**o+(0.605266809429)*x_ref[0]+(0.114190031414)*x_ref[1]**o+(-0.0554731448399)*x_ref[1]
        ref[0,2,2,1]=(0.49575503694)*x_ref[0]**o+(0.842572557458)*x_ref[0]+(-0.65557628937)*x_ref[1]**o+(-0.110654902831)*x_ref[1]
        ref[0,3,0,0]=(-0.914580503709)*x_ref[0]**o+(0.568110262697)*x_ref[0]+(0.465998898456)*x_ref[1]**o+(0.668680449802)*x_ref[1]
        ref[0,3,0,1]=(-0.406517611176)*x_ref[0]**o+(0.751773194945)*x_ref[0]+(-0.546709674237)*x_ref[1]**o+(0.830905534576)*x_ref[1]
        ref[0,3,1,0]=(0.256170881332)*x_ref[0]**o+(0.785200644094)*x_ref[0]+(-0.0949323527814)*x_ref[1]**o+(0.437799246842)*x_ref[1]
        ref[0,3,1,1]=(0.388926885643)*x_ref[0]**o+(0.0649842722847)*x_ref[0]+(-0.570138074228)*x_ref[1]**o+(0.0125008958832)*x_ref[1]
        ref[0,3,2,0]=(-0.693124111229)*x_ref[0]**o+(-0.135531989861)*x_ref[0]+(-0.484364583498)*x_ref[1]**o+(0.142028440685)*x_ref[1]
        ref[0,3,2,1]=(0.162852074905)*x_ref[0]**o+(-0.211709648647)*x_ref[0]+(-0.424964818278)*x_ref[1]**o+(-0.0311289175964)*x_ref[1]
        ref[0,4,0,0]=(-0.893934248272)*x_ref[0]**o+(-0.883334046545)*x_ref[0]+(0.141127275532)*x_ref[1]**o+(0.790251212849)*x_ref[1]
        ref[0,4,0,1]=(0.751030754129)*x_ref[0]**o+(-0.0463112594115)*x_ref[0]+(0.894754601054)*x_ref[1]**o+(-0.986137408191)*x_ref[1]
        ref[0,4,1,0]=(-0.994216054583)*x_ref[0]**o+(0.989450915377)*x_ref[0]+(0.202383847504)*x_ref[1]**o+(-0.829413728527)*x_ref[1]
        ref[0,4,1,1]=(-0.70581665488)*x_ref[0]**o+(0.387180461053)*x_ref[0]+(0.925669527701)*x_ref[1]**o+(0.422052343171)*x_ref[1]
        ref[0,4,2,0]=(-0.498553799214)*x_ref[0]**o+(0.70559513684)*x_ref[0]+(-0.595097550839)*x_ref[1]**o+(-0.177703117092)*x_ref[1]
        ref[0,4,2,1]=(-0.703437824074)*x_ref[0]**o+(-0.574613675229)*x_ref[0]+(-0.699959931134)*x_ref[1]**o+(-0.503745753701)*x_ref[1]
        ref[1,0,0,0]=(-0.0673123845899)*x_ref[0]**o+(-0.0603073393325)*x_ref[0]+(0.724127339391)*x_ref[1]**o+(-0.0875500989225)*x_ref[1]
        ref[1,0,0,1]=(0.0766117819496)*x_ref[0]**o+(-0.489029658255)*x_ref[0]+(0.602970700568)*x_ref[1]**o+(0.624180485873)*x_ref[1]
        ref[1,0,1,0]=(0.537236166297)*x_ref[0]**o+(0.805657082915)*x_ref[0]+(-0.0263572051431)*x_ref[1]**o+(0.683790156906)*x_ref[1]
        ref[1,0,1,1]=(0.55427198724)*x_ref[0]**o+(0.631937641401)*x_ref[0]+(-0.634744046324)*x_ref[1]**o+(0.824382447181)*x_ref[1]
        ref[1,0,2,0]=(0.694551886505)*x_ref[0]**o+(-0.123561028797)*x_ref[0]+(-0.0237233372904)*x_ref[1]**o+(-0.822938415295)*x_ref[1]
        ref[1,0,2,1]=(0.293944558832)*x_ref[0]**o+(-0.34741820803)*x_ref[0]+(-0.471269455638)*x_ref[1]**o+(0.716959839747)*x_ref[1]
        ref[1,1,0,0]=(0.409876005614)*x_ref[0]**o+(-0.322385353356)*x_ref[0]+(0.362023900558)*x_ref[1]**o+(-0.974915188765)*x_ref[1]
        ref[1,1,0,1]=(-0.823587457611)*x_ref[0]**o+(-0.315566397595)*x_ref[0]+(-0.681951254575)*x_ref[1]**o+(-0.945803349426)*x_ref[1]
        ref[1,1,1,0]=(0.782950007967)*x_ref[0]**o+(0.767220278425)*x_ref[0]+(0.109277917483)*x_ref[1]**o+(-0.336228598202)*x_ref[1]
        ref[1,1,1,1]=(-0.969811850973)*x_ref[0]**o+(0.453588680715)*x_ref[0]+(-0.296932776592)*x_ref[1]**o+(-0.460668458023)*x_ref[1]
        ref[1,1,2,0]=(0.124616545015)*x_ref[0]**o+(0.180238150583)*x_ref[0]+(0.283723867146)*x_ref[1]**o+(0.873774600692)*x_ref[1]
        ref[1,1,2,1]=(0.00334202091213)*x_ref[0]**o+(-0.711821714868)*x_ref[0]+(0.638247490899)*x_ref[1]**o+(-0.409696943449)*x_ref[1]
        ref[1,2,0,0]=(-0.206885052591)*x_ref[0]**o+(-0.677245253776)*x_ref[0]+(-0.508637512178)*x_ref[1]**o+(0.0862259084563)*x_ref[1]
        ref[1,2,0,1]=(0.402289447161)*x_ref[0]**o+(0.242246771389)*x_ref[0]+(0.775293224518)*x_ref[1]**o+(-0.668881714475)*x_ref[1]
        ref[1,2,1,0]=(-0.094983969904)*x_ref[0]**o+(0.615306379484)*x_ref[0]+(-0.737839325648)*x_ref[1]**o+(0.457810359747)*x_ref[1]
        ref[1,2,1,1]=(0.488668557736)*x_ref[0]**o+(0.188170233286)*x_ref[0]+(0.0947759637418)*x_ref[1]**o+(-0.510206729955)*x_ref[1]
        ref[1,2,2,0]=(-0.102204043127)*x_ref[0]**o+(0.584307889877)*x_ref[0]+(-0.779746892074)*x_ref[1]**o+(-0.767757863288)*x_ref[1]
        ref[1,2,2,1]=(-0.00230617324952)*x_ref[0]**o+(-0.184632788675)*x_ref[0]+(-0.779494086898)*x_ref[1]**o+(-0.105311421755)*x_ref[1]
        ref[1,3,0,0]=(0.373541105531)*x_ref[0]**o+(-0.895909586816)*x_ref[0]+(0.853523284159)*x_ref[1]**o+(0.915229883403)*x_ref[1]
        ref[1,3,0,1]=(-0.888315748473)*x_ref[0]**o+(0.877859466924)*x_ref[0]+(-0.830380928425)*x_ref[1]**o+(-0.25922026388)*x_ref[1]
        ref[1,3,1,0]=(-0.944775455365)*x_ref[0]**o+(0.694978915121)*x_ref[0]+(0.00346060944841)*x_ref[1]**o+(0.610096191934)*x_ref[1]
        ref[1,3,1,1]=(-0.175304757445)*x_ref[0]**o+(0.883688655763)*x_ref[0]+(-0.0260273718271)*x_ref[1]**o+(0.440569935385)*x_ref[1]
        ref[1,3,2,0]=(0.973418372218)*x_ref[0]**o+(-0.0725489149125)*x_ref[0]+(-0.790861964708)*x_ref[1]**o+(-0.764398676127)*x_ref[1]
        ref[1,3,2,1]=(0.188665821479)*x_ref[0]**o+(-0.694573366343)*x_ref[0]+(-0.860244164818)*x_ref[1]**o+(0.642736876841)*x_ref[1]
        ref[1,4,0,0]=(-0.619221357105)*x_ref[0]**o+(-0.0366612230467)*x_ref[0]+(0.920044684933)*x_ref[1]**o+(-0.901158565327)*x_ref[1]
        ref[1,4,0,1]=(-0.774227719954)*x_ref[0]**o+(0.471966691541)*x_ref[0]+(0.0706379157745)*x_ref[1]**o+(-0.238713884878)*x_ref[1]
        ref[1,4,1,0]=(0.863551422475)*x_ref[0]**o+(0.131078367733)*x_ref[0]+(0.458326651999)*x_ref[1]**o+(-0.750431841864)*x_ref[1]
        ref[1,4,1,1]=(0.105763306948)*x_ref[0]**o+(-0.254538567936)*x_ref[0]+(-0.559297954287)*x_ref[1]**o+(0.21981759162)*x_ref[1]
        ref[1,4,2,0]=(0.922873574086)*x_ref[0]**o+(-0.853230802788)*x_ref[0]+(-0.982798222021)*x_ref[1]**o+(-0.0866961947026)*x_ref[1]
        ref[1,4,2,1]=(-0.21664472336)*x_ref[0]**o+(-0.915876043584)*x_ref[0]+(0.6182964805)*x_ref[1]**o+(0.665515072302)*x_ref[1]
        ref[2,0,0,0]=(-0.491230924471)*x_ref[0]**o+(0.509670721146)*x_ref[0]+(-0.44580759248)*x_ref[1]**o+(-0.943168309946)*x_ref[1]
        ref[2,0,0,1]=(-0.0895802535424)*x_ref[0]**o+(0.0780292649662)*x_ref[0]+(-0.42293922141)*x_ref[1]**o+(-0.76928974668)*x_ref[1]
        ref[2,0,1,0]=(-0.404348433578)*x_ref[0]**o+(-0.986117091427)*x_ref[0]+(0.562569938626)*x_ref[1]**o+(-0.41119675761)*x_ref[1]
        ref[2,0,1,1]=(0.517587873308)*x_ref[0]**o+(0.0731347448674)*x_ref[0]+(0.830876774432)*x_ref[1]**o+(-0.690590335528)*x_ref[1]
        ref[2,0,2,0]=(0.600931766711)*x_ref[0]**o+(-0.786774940006)*x_ref[0]+(-0.285975800855)*x_ref[1]**o+(0.348122549519)*x_ref[1]
        ref[2,0,2,1]=(0.825033481951)*x_ref[0]**o+(-0.744551393479)*x_ref[0]+(0.452879660703)*x_ref[1]**o+(-0.257025658518)*x_ref[1]
        ref[2,1,0,0]=(-0.0822184021881)*x_ref[0]**o+(0.955001628077)*x_ref[0]+(-0.878009442237)*x_ref[1]**o+(-0.816970793701)*x_ref[1]
        ref[2,1,0,1]=(-0.151473282662)*x_ref[0]**o+(0.937636279558)*x_ref[0]+(-0.969793512643)*x_ref[1]**o+(0.704581030148)*x_ref[1]
        ref[2,1,1,0]=(0.76728625844)*x_ref[0]**o+(-0.90294390262)*x_ref[0]+(-0.375443980814)*x_ref[1]**o+(0.276219230561)*x_ref[1]
        ref[2,1,1,1]=(-0.994260771722)*x_ref[0]**o+(0.64595057257)*x_ref[0]+(0.486576661947)*x_ref[1]**o+(0.731738848734)*x_ref[1]
        ref[2,1,2,0]=(-0.52048273513)*x_ref[0]**o+(-0.51915057929)*x_ref[0]+(0.184855975415)*x_ref[1]**o+(0.417869171454)*x_ref[1]
        ref[2,1,2,1]=(-0.371019640816)*x_ref[0]**o+(-0.617014410896)*x_ref[0]+(0.725259821636)*x_ref[1]**o+(-0.0393667759542)*x_ref[1]
        ref[2,2,0,0]=(0.070044943001)*x_ref[0]**o+(-0.663066522147)*x_ref[0]+(0.761006491068)*x_ref[1]**o+(-0.137480662114)*x_ref[1]
        ref[2,2,0,1]=(-0.394035586896)*x_ref[0]**o+(0.802334732427)*x_ref[0]+(-0.191154645494)*x_ref[1]**o+(-0.278490557185)*x_ref[1]
        ref[2,2,1,0]=(0.0664932052956)*x_ref[0]**o+(0.938378455125)*x_ref[0]+(0.997119543949)*x_ref[1]**o+(-0.696820719616)*x_ref[1]
        ref[2,2,1,1]=(0.79758586389)*x_ref[0]**o+(0.0393291272874)*x_ref[0]+(0.644980805887)*x_ref[1]**o+(0.61925875842)*x_ref[1]
        ref[2,2,2,0]=(0.0353810358089)*x_ref[0]**o+(-0.0134652375886)*x_ref[0]+(-0.942381983803)*x_ref[1]**o+(0.288697669014)*x_ref[1]
        ref[2,2,2,1]=(-0.327577257504)*x_ref[0]**o+(0.816374902745)*x_ref[0]+(0.676853003934)*x_ref[1]**o+(0.581394632094)*x_ref[1]
        ref[2,3,0,0]=(0.63036222221)*x_ref[0]**o+(-0.899724810809)*x_ref[0]+(0.276882104091)*x_ref[1]**o+(-0.00448820301083)*x_ref[1]
        ref[2,3,0,1]=(-0.219899069159)*x_ref[0]**o+(-0.439244974227)*x_ref[0]+(-0.369366024932)*x_ref[1]**o+(-0.161805301574)*x_ref[1]
        ref[2,3,1,0]=(0.165202872055)*x_ref[0]**o+(-0.21521932039)*x_ref[0]+(0.635188695229)*x_ref[1]**o+(-0.254785217427)*x_ref[1]
        ref[2,3,1,1]=(0.743546803617)*x_ref[0]**o+(0.876160636955)*x_ref[0]+(-0.094730937905)*x_ref[1]**o+(-0.898315912992)*x_ref[1]
        ref[2,3,2,0]=(0.946538368626)*x_ref[0]**o+(0.715488078243)*x_ref[0]+(0.830112020529)*x_ref[1]**o+(-0.320605753644)*x_ref[1]
        ref[2,3,2,1]=(0.52106888624)*x_ref[0]**o+(-0.938072665733)*x_ref[0]+(-0.682830472037)*x_ref[1]**o+(-0.787442211636)*x_ref[1]
        ref[2,4,0,0]=(0.802634158133)*x_ref[0]**o+(0.350923640202)*x_ref[0]+(-0.142397660222)*x_ref[1]**o+(0.768827179039)*x_ref[1]
        ref[2,4,0,1]=(0.912207226957)*x_ref[0]**o+(0.302955742414)*x_ref[0]+(-0.125141454725)*x_ref[1]**o+(0.115375955845)*x_ref[1]
        ref[2,4,1,0]=(0.88629189066)*x_ref[0]**o+(-0.431145868333)*x_ref[0]+(0.521817592458)*x_ref[1]**o+(0.757372657079)*x_ref[1]
        ref[2,4,1,1]=(-0.34619470039)*x_ref[0]**o+(0.0971941022378)*x_ref[0]+(-0.235321219876)*x_ref[1]**o+(0.919340221916)*x_ref[1]
        ref[2,4,2,0]=(-0.453744205858)*x_ref[0]**o+(0.386107152737)*x_ref[0]+(-0.53829794769)*x_ref[1]**o+(-0.955985738202)*x_ref[1]
        ref[2,4,2,1]=(0.327564798989)*x_ref[0]**o+(0.234176645121)*x_ref[0]+(-0.647242055069)*x_ref[1]**o+(-0.914279617484)*x_ref[1]
        ref[3,0,0,0]=(-0.765850994681)*x_ref[0]**o+(0.0969021602734)*x_ref[0]+(-0.003697715708)*x_ref[1]**o+(0.642293903037)*x_ref[1]
        ref[3,0,0,1]=(-0.736912760299)*x_ref[0]**o+(-0.496756935434)*x_ref[0]+(0.00160221880808)*x_ref[1]**o+(-0.0275314787811)*x_ref[1]
        ref[3,0,1,0]=(0.266302600859)*x_ref[0]**o+(0.962232443249)*x_ref[0]+(0.588094142167)*x_ref[1]**o+(-0.740113379551)*x_ref[1]
        ref[3,0,1,1]=(-0.701204282707)*x_ref[0]**o+(0.16377449778)*x_ref[0]+(-0.636725536715)*x_ref[1]**o+(0.925340403623)*x_ref[1]
        ref[3,0,2,0]=(-0.473492482064)*x_ref[0]**o+(0.356976796364)*x_ref[0]+(0.28490257392)*x_ref[1]**o+(0.558859192591)*x_ref[1]
        ref[3,0,2,1]=(-0.398155056481)*x_ref[0]**o+(-0.693010191754)*x_ref[0]+(-0.457570700744)*x_ref[1]**o+(0.899032980684)*x_ref[1]
        ref[3,1,0,0]=(0.676626616862)*x_ref[0]**o+(-0.142127205425)*x_ref[0]+(-0.0703163466342)*x_ref[1]**o+(-0.603767804268)*x_ref[1]
        ref[3,1,0,1]=(0.901299559593)*x_ref[0]**o+(-0.319119050502)*x_ref[0]+(-0.230855076893)*x_ref[1]**o+(0.923789250137)*x_ref[1]
        ref[3,1,1,0]=(0.0125691057912)*x_ref[0]**o+(0.827401151454)*x_ref[0]+(0.0881382970418)*x_ref[1]**o+(0.12796232574)*x_ref[1]
        ref[3,1,1,1]=(-0.544917801861)*x_ref[0]**o+(-0.163923780568)*x_ref[0]+(0.107833995954)*x_ref[1]**o+(-0.347470857919)*x_ref[1]
        ref[3,1,2,0]=(-0.127281534891)*x_ref[0]**o+(0.410172113868)*x_ref[0]+(0.516300378909)*x_ref[1]**o+(-0.454031679856)*x_ref[1]
        ref[3,1,2,1]=(-0.797619552686)*x_ref[0]**o+(0.0212633210895)*x_ref[0]+(-0.0890328154993)*x_ref[1]**o+(0.755392852372)*x_ref[1]
        ref[3,2,0,0]=(0.00976392699076)*x_ref[0]**o+(-0.608813040196)*x_ref[0]+(0.786997931081)*x_ref[1]**o+(0.637450406747)*x_ref[1]
        ref[3,2,0,1]=(-0.594915968211)*x_ref[0]**o+(0.909106492284)*x_ref[0]+(0.209774113888)*x_ref[1]**o+(0.926156671099)*x_ref[1]
        ref[3,2,1,0]=(0.226007901841)*x_ref[0]**o+(-0.43873163217)*x_ref[0]+(-0.414778768373)*x_ref[1]**o+(0.681053132578)*x_ref[1]
        ref[3,2,1,1]=(0.733375971867)*x_ref[0]**o+(0.861188515019)*x_ref[0]+(0.036782797193)*x_ref[1]**o+(0.145181529782)*x_ref[1]
        ref[3,2,2,0]=(0.635727397799)*x_ref[0]**o+(0.874128725841)*x_ref[0]+(-0.810169847222)*x_ref[1]**o+(0.234369619159)*x_ref[1]
        ref[3,2,2,1]=(-0.137252742544)*x_ref[0]**o+(0.817564886208)*x_ref[0]+(0.524706682726)*x_ref[1]**o+(0.381735970369)*x_ref[1]
        ref[3,3,0,0]=(0.537368746859)*x_ref[0]**o+(-0.298760793455)*x_ref[0]+(-0.890807690497)*x_ref[1]**o+(-0.662222369966)*x_ref[1]
        ref[3,3,0,1]=(-0.67672238667)*x_ref[0]**o+(-0.305431277061)*x_ref[0]+(-0.435551021687)*x_ref[1]**o+(0.0777923105041)*x_ref[1]
        ref[3,3,1,0]=(-0.163284186346)*x_ref[0]**o+(-0.764058727495)*x_ref[0]+(-0.255689425142)*x_ref[1]**o+(0.285528254098)*x_ref[1]
        ref[3,3,1,1]=(-0.842634439814)*x_ref[0]**o+(-0.194811378723)*x_ref[0]+(0.814424721344)*x_ref[1]**o+(-0.236826785563)*x_ref[1]
        ref[3,3,2,0]=(0.532936591284)*x_ref[0]**o+(-0.920822945577)*x_ref[0]+(-0.677841412043)*x_ref[1]**o+(0.579981868038)*x_ref[1]
        ref[3,3,2,1]=(-0.470357673624)*x_ref[0]**o+(-0.972798479564)*x_ref[0]+(0.489692897505)*x_ref[1]**o+(0.213937860142)*x_ref[1]
        ref[3,4,0,0]=(-0.169170544642)*x_ref[0]**o+(0.543113611166)*x_ref[0]+(0.536912197355)*x_ref[1]**o+(0.304230092305)*x_ref[1]
        ref[3,4,0,1]=(-0.170701528868)*x_ref[0]**o+(0.340606153319)*x_ref[0]+(-0.470528666825)*x_ref[1]**o+(-0.522028583228)*x_ref[1]
        ref[3,4,1,0]=(0.0194532320642)*x_ref[0]**o+(0.335898702694)*x_ref[0]+(-0.917025993458)*x_ref[1]**o+(-0.539748486826)*x_ref[1]
        ref[3,4,1,1]=(-0.576465573138)*x_ref[0]**o+(-0.906917910142)*x_ref[0]+(-0.313000868178)*x_ref[1]**o+(-0.20607256753)*x_ref[1]
        ref[3,4,2,0]=(-0.75394368612)*x_ref[0]**o+(0.658866075222)*x_ref[0]+(0.402200040518)*x_ref[1]**o+(-0.544252067119)*x_ref[1]
        ref[3,4,2,1]=(0.0490609014064)*x_ref[0]**o+(-0.873887848405)*x_ref[0]+(0.91186722933)*x_ref[1]**o+(0.375269571672)*x_ref[1]
      else:
        s[0,0,0,0]=(-0.853604830323)*x[0]**o+(0.378465791865)*x[0]+(-0.83364075876)*x[1]**o+(0.212018195408)*x[1]+(0.48856925163)*x[2]**o+(-0.179831169531)*x[2]
        s[0,0,0,1]=(0.868673673397)*x[0]**o+(-0.238560476072)*x[0]+(0.828785612257)*x[1]**o+(0.176759782363)*x[1]+(0.525888840201)*x[2]**o+(0.436158744893)*x[2]
        s[0,0,1,0]=(0.887888476899)*x[0]**o+(0.458452579078)*x[0]+(0.572312971484)*x[1]**o+(0.694648297654)*x[1]+(0.244770581638)*x[2]**o+(-0.326433505221)*x[2]
        s[0,0,1,1]=(0.958595740398)*x[0]**o+(-0.448016561436)*x[0]+(-0.898608139656)*x[1]**o+(0.0189044802348)*x[1]+(0.664802638624)*x[2]**o+(0.675631565244)*x[2]
        s[0,0,2,0]=(-0.0968613941718)*x[0]**o+(-0.580198871721)*x[0]+(-0.695139695389)*x[1]**o+(0.670627088913)*x[1]+(-0.466392817977)*x[2]**o+(-0.414367914263)*x[2]
        s[0,0,2,1]=(0.0563724518255)*x[0]**o+(0.141716006991)*x[0]+(-0.889611317888)*x[1]**o+(0.274193692615)*x[1]+(0.81766066569)*x[2]**o+(0.260242356947)*x[2]
        s[0,1,0,0]=(0.729916171245)*x[0]**o+(-0.0512883787314)*x[0]+(-0.395879068642)*x[1]**o+(0.65153799151)*x[1]+(-0.90747241683)*x[2]**o+(0.556056661876)*x[2]
        s[0,1,0,1]=(-0.336962711878)*x[0]**o+(0.494126376731)*x[0]+(-0.0180540415242)*x[1]**o+(-0.281208191895)*x[1]+(-0.891281821502)*x[2]**o+(0.434438114521)*x[2]
        s[0,1,1,0]=(-0.0203012608797)*x[0]**o+(-0.592593200931)*x[0]+(-0.908091999823)*x[1]**o+(0.917265173937)*x[1]+(0.0113338093354)*x[2]**o+(0.832522528967)*x[2]
        s[0,1,1,1]=(-0.287082151578)*x[0]**o+(-0.882563505538)*x[0]+(-0.805577758903)*x[1]**o+(-0.0877632778656)*x[1]+(-0.4141073805)*x[2]**o+(-0.0809945294401)*x[2]
        s[0,1,2,0]=(-0.745887649438)*x[0]**o+(0.105423480913)*x[0]+(0.642374896965)*x[1]**o+(0.479730302425)*x[1]+(0.334298930575)*x[2]**o+(0.214588275036)*x[2]
        s[0,1,2,1]=(-0.987939537144)*x[0]**o+(-0.889107799492)*x[0]+(-0.133927268357)*x[1]**o+(-0.991450009918)*x[1]+(-0.201645805359)*x[2]**o+(0.481104989077)*x[2]
        s[0,2,0,0]=(0.535918222555)*x[0]**o+(0.243041294608)*x[0]+(-0.205082148634)*x[1]**o+(0.233470577382)*x[1]+(0.398471417689)*x[2]**o+(-0.56437758006)*x[2]
        s[0,2,0,1]=(0.374626206936)*x[0]**o+(-0.800840698958)*x[0]+(0.336427483065)*x[1]**o+(0.143368857985)*x[1]+(0.822396957468)*x[2]**o+(-0.288884862626)*x[2]
        s[0,2,1,0]=(-0.152217915896)*x[0]**o+(0.868324413095)*x[0]+(0.93760108412)*x[1]**o+(-0.584628102458)*x[1]+(-0.678653990131)*x[2]**o+(-0.130768977951)*x[2]
        s[0,2,1,1]=(-0.962809494754)*x[0]**o+(0.057963920559)*x[0]+(-0.68685683212)*x[1]**o+(0.869986883492)*x[1]+(-0.193987500788)*x[2]**o+(0.0357315899177)*x[2]
        s[0,2,2,0]=(-0.296222468619)*x[0]**o+(0.853293489381)*x[0]+(0.886622498642)*x[1]**o+(-0.22952370037)*x[1]+(0.444962301355)*x[2]**o+(0.498735104677)*x[2]
        s[0,2,2,1]=(-0.556796221789)*x[0]**o+(-0.98717630759)*x[0]+(-0.56578841851)*x[1]**o+(-0.769428379953)*x[1]+(0.41417947942)*x[2]**o+(-0.99075609451)*x[2]
        s[0,3,0,0]=(-0.312173122533)*x[0]**o+(-0.0183061811911)*x[0]+(0.632028466217)*x[1]**o+(-0.516798544207)*x[1]+(0.467776726567)*x[2]**o+(-0.0750898474112)*x[2]
        s[0,3,0,1]=(0.151818577102)*x[0]**o+(0.275659634397)*x[0]+(-0.695866418429)*x[1]**o+(-0.158484337329)*x[1]+(-0.717635621103)*x[2]**o+(0.96218471765)*x[2]
        s[0,3,1,0]=(-0.959269035858)*x[0]**o+(-0.694952577107)*x[0]+(0.133713794285)*x[1]**o+(0.945801566586)*x[1]+(0.480976843012)*x[2]**o+(0.514096820167)*x[2]
        s[0,3,1,1]=(-0.696429878723)*x[0]**o+(0.844116956105)*x[0]+(0.986596486924)*x[1]**o+(0.213400513251)*x[1]+(0.168567391983)*x[2]**o+(-0.96577037709)*x[2]
        s[0,3,2,0]=(0.272554167042)*x[0]**o+(-0.134445947881)*x[0]+(-0.620927727232)*x[1]**o+(0.774446825459)*x[1]+(-0.140360641548)*x[2]**o+(-0.393011913595)*x[2]
        s[0,3,2,1]=(0.973937360802)*x[0]**o+(-0.780203033305)*x[0]+(0.62508420402)*x[1]**o+(-0.957359234026)*x[1]+(0.301032273116)*x[2]**o+(0.238130119107)*x[2]
        s[0,4,0,0]=(0.550167138562)*x[0]**o+(-0.975271434354)*x[0]+(-0.90226839553)*x[1]**o+(0.765360109563)*x[1]+(-0.0539842939871)*x[2]**o+(-0.992811018253)*x[2]
        s[0,4,0,1]=(0.115680644235)*x[0]**o+(0.6504959069)*x[0]+(0.0711117194347)*x[1]**o+(0.765247742051)*x[1]+(0.651478097229)*x[2]**o+(0.0605973413193)*x[2]
        s[0,4,1,0]=(-0.339495686607)*x[0]**o+(0.98910452173)*x[0]+(-0.97593906201)*x[1]**o+(-0.961098427995)*x[1]+(-0.970981798475)*x[2]**o+(-0.655846307864)*x[2]
        s[0,4,1,1]=(-0.676422431554)*x[0]**o+(0.17779634387)*x[0]+(-0.101241446408)*x[1]**o+(0.592319052036)*x[1]+(-0.328471629455)*x[2]**o+(0.842981957639)*x[2]
        s[0,4,2,0]=(0.475154370559)*x[0]**o+(-0.861452809594)*x[0]+(0.021220095967)*x[1]**o+(-0.167324332634)*x[1]+(-0.420334858996)*x[2]**o+(-0.243503287995)*x[2]
        s[0,4,2,1]=(0.507818170756)*x[0]**o+(0.0305556062843)*x[0]+(0.657465103712)*x[1]**o+(-0.133306218785)*x[1]+(0.596182931411)*x[2]**o+(0.26354220091)*x[2]
        s[1,0,0,0]=(0.141665798979)*x[0]**o+(0.257839463098)*x[0]+(0.20842719092)*x[1]**o+(0.0606481804876)*x[1]+(0.564188524366)*x[2]**o+(0.276431563089)*x[2]
        s[1,0,0,1]=(-0.475871120292)*x[0]**o+(-0.945285882278)*x[0]+(-0.852943036141)*x[1]**o+(-0.393053044641)*x[1]+(0.894360138187)*x[2]**o+(0.945580362088)*x[2]
        s[1,0,1,0]=(-0.269113046186)*x[0]**o+(0.979226351783)*x[0]+(0.59588973168)*x[1]**o+(0.709182601617)*x[1]+(0.203809169118)*x[2]**o+(0.843792678336)*x[2]
        s[1,0,1,1]=(0.625152197275)*x[0]**o+(0.700601491781)*x[0]+(-0.556266788732)*x[1]**o+(0.621296929469)*x[1]+(0.370627217553)*x[2]**o+(-0.86981081047)*x[2]
        s[1,0,2,0]=(0.337314024742)*x[0]**o+(0.4936522561)*x[0]+(-0.121004948433)*x[1]**o+(0.682816106721)*x[1]+(0.135016952475)*x[2]**o+(0.447788511803)*x[2]
        s[1,0,2,1]=(0.627048609871)*x[0]**o+(0.464430671751)*x[0]+(0.248071099972)*x[1]**o+(0.49700045944)*x[1]+(0.764133255754)*x[2]**o+(-0.407441090409)*x[2]
        s[1,1,0,0]=(-0.891648124769)*x[0]**o+(-0.852500694418)*x[0]+(-0.501966394474)*x[1]**o+(-0.951197952585)*x[1]+(-0.293731401112)*x[2]**o+(0.340542703849)*x[2]
        s[1,1,0,1]=(0.047745963453)*x[0]**o+(0.605499497949)*x[0]+(0.603025931833)*x[1]**o+(0.412711498009)*x[1]+(-0.706970380941)*x[2]**o+(-0.865479677059)*x[2]
        s[1,1,1,0]=(-0.809326436683)*x[0]**o+(0.50103064436)*x[0]+(-0.726029943072)*x[1]**o+(-0.0136545501065)*x[1]+(0.869298450797)*x[2]**o+(-0.0539795856427)*x[2]
        s[1,1,1,1]=(0.822265437858)*x[0]**o+(-0.13686562101)*x[0]+(-0.921877204478)*x[1]**o+(0.436216049238)*x[1]+(0.167449871886)*x[2]**o+(-0.376043518724)*x[2]
        s[1,1,2,0]=(-0.240904110506)*x[0]**o+(-0.709486977651)*x[0]+(0.589748137401)*x[1]**o+(0.997759434219)*x[1]+(0.211495210823)*x[2]**o+(0.857954370359)*x[2]
        s[1,1,2,1]=(-0.872650055395)*x[0]**o+(0.488392921139)*x[0]+(0.456778341472)*x[1]**o+(0.533360514682)*x[1]+(0.660463672454)*x[2]**o+(-0.287728736994)*x[2]
        s[1,2,0,0]=(-0.56865288172)*x[0]**o+(-0.257707278342)*x[0]+(0.492925973387)*x[1]**o+(-0.433521245223)*x[1]+(-0.214711127244)*x[2]**o+(-0.578235847487)*x[2]
        s[1,2,0,1]=(0.502020345167)*x[0]**o+(-0.898730931974)*x[0]+(0.152379872963)*x[1]**o+(0.963247300695)*x[1]+(-0.935219139549)*x[2]**o+(-0.322095550311)*x[2]
        s[1,2,1,0]=(0.755425499434)*x[0]**o+(0.922722006611)*x[0]+(-0.854913831787)*x[1]**o+(0.28200116496)*x[1]+(0.654754619366)*x[2]**o+(-0.87246359532)*x[2]
        s[1,2,1,1]=(0.476533428226)*x[0]**o+(-0.21427962663)*x[0]+(-0.42807986959)*x[1]**o+(-0.453270655163)*x[1]+(-0.0618352146195)*x[2]**o+(-0.0578564702081)*x[2]
        s[1,2,2,0]=(0.836242039415)*x[0]**o+(0.0582519660745)*x[0]+(-0.736245953793)*x[1]**o+(-0.0216606630383)*x[1]+(-0.241691127038)*x[2]**o+(-0.273769285415)*x[2]
        s[1,2,2,1]=(0.956981101891)*x[0]**o+(-0.619600309731)*x[0]+(0.255999571352)*x[1]**o+(0.931443107924)*x[1]+(0.803455662831)*x[2]**o+(0.587378690917)*x[2]
        s[1,3,0,0]=(0.84725602389)*x[0]**o+(0.099929556134)*x[0]+(0.918513623927)*x[1]**o+(-0.540239895987)*x[1]+(0.543336053141)*x[2]**o+(-0.35339148993)*x[2]
        s[1,3,0,1]=(0.0790066953583)*x[0]**o+(-0.622984122413)*x[0]+(0.844477077556)*x[1]**o+(-0.843967484949)*x[1]+(0.382810149795)*x[2]**o+(0.693483174672)*x[2]
        s[1,3,1,0]=(-0.606850566406)*x[0]**o+(-0.613588487235)*x[0]+(-0.982801043706)*x[1]**o+(-0.200157044491)*x[1]+(0.865010142689)*x[2]**o+(-0.926404146796)*x[2]
        s[1,3,1,1]=(-0.926332516404)*x[0]**o+(-0.644403030771)*x[0]+(-0.391941644842)*x[1]**o+(-0.628555138555)*x[1]+(0.584646630734)*x[2]**o+(0.627561535294)*x[2]
        s[1,3,2,0]=(0.777960693446)*x[0]**o+(-0.580698013493)*x[0]+(-0.061588850878)*x[1]**o+(0.909167097211)*x[1]+(-0.346386067681)*x[2]**o+(-0.698593831635)*x[2]
        s[1,3,2,1]=(-0.25740731816)*x[0]**o+(0.0609560824945)*x[0]+(-0.866636970162)*x[1]**o+(0.352496912986)*x[1]+(0.212276243322)*x[2]**o+(-0.29499231054)*x[2]
        s[1,4,0,0]=(0.176828476382)*x[0]**o+(0.592116728712)*x[0]+(-0.969737040463)*x[1]**o+(-0.0140144291993)*x[1]+(-0.203791892462)*x[2]**o+(0.687043221294)*x[2]
        s[1,4,0,1]=(-0.0673562914955)*x[0]**o+(0.104813124019)*x[0]+(0.766386480162)*x[1]**o+(-0.499215759851)*x[1]+(0.163490109505)*x[2]**o+(0.0477449791182)*x[2]
        s[1,4,1,0]=(-0.186436883357)*x[0]**o+(0.771658464504)*x[0]+(0.115845567912)*x[1]**o+(-0.573016245745)*x[1]+(0.759680038826)*x[2]**o+(0.0660145528929)*x[2]
        s[1,4,1,1]=(-0.983746295741)*x[0]**o+(-0.370146292969)*x[0]+(-0.276398856773)*x[1]**o+(-0.765483196375)*x[1]+(-0.0802845394538)*x[2]**o+(0.762085198906)*x[2]
        s[1,4,2,0]=(-0.437323501168)*x[0]**o+(-0.076976193125)*x[0]+(0.780179691457)*x[1]**o+(0.941748527645)*x[1]+(0.921220023779)*x[2]**o+(-0.611350278287)*x[2]
        s[1,4,2,1]=(-0.811077839585)*x[0]**o+(-0.0781699097656)*x[0]+(-0.955979549084)*x[1]**o+(0.742581422414)*x[1]+(-0.905829236752)*x[2]**o+(-0.671257774729)*x[2]
        s[2,0,0,0]=(0.3602498958)*x[0]**o+(0.567952303965)*x[0]+(-0.72126509422)*x[1]**o+(0.652912038045)*x[1]+(-0.448081819441)*x[2]**o+(-0.628140847819)*x[2]
        s[2,0,0,1]=(0.823191679062)*x[0]**o+(-0.12280206123)*x[0]+(0.202149797631)*x[1]**o+(-0.751457177895)*x[1]+(0.367057556408)*x[2]**o+(0.71681510921)*x[2]
        s[2,0,1,0]=(-0.553277801352)*x[0]**o+(-0.695114786088)*x[0]+(-0.506695185331)*x[1]**o+(0.947900606544)*x[1]+(0.393574900465)*x[2]**o+(-0.131907977332)*x[2]
        s[2,0,1,1]=(-0.683260439471)*x[0]**o+(0.108921790681)*x[0]+(0.694361429808)*x[1]**o+(-0.879827908202)*x[1]+(-0.768400141399)*x[2]**o+(-0.869535466479)*x[2]
        s[2,0,2,0]=(-0.377844410367)*x[0]**o+(-0.486876364642)*x[0]+(-0.139990170975)*x[1]**o+(0.521294626989)*x[1]+(0.384721944946)*x[2]**o+(0.912206678795)*x[2]
        s[2,0,2,1]=(0.624159317849)*x[0]**o+(0.910540130065)*x[0]+(-0.879166404339)*x[1]**o+(0.710229335366)*x[1]+(-0.756806360688)*x[2]**o+(-0.169809484076)*x[2]
        s[2,1,0,0]=(-0.887833054092)*x[0]**o+(-0.541562581783)*x[0]+(-0.439734700726)*x[1]**o+(-0.294103567325)*x[1]+(0.621145429347)*x[2]**o+(0.65618994558)*x[2]
        s[2,1,0,1]=(-0.199707709698)*x[0]**o+(0.952895238716)*x[0]+(-0.113807788254)*x[1]**o+(-0.209497731554)*x[1]+(-0.201935349119)*x[2]**o+(0.896172767252)*x[2]
        s[2,1,1,0]=(0.961864846935)*x[0]**o+(0.34037006989)*x[0]+(0.97866208125)*x[1]**o+(0.949937962054)*x[1]+(-0.943237740253)*x[2]**o+(0.492751804379)*x[2]
        s[2,1,1,1]=(0.0364913119154)*x[0]**o+(-0.706276345936)*x[0]+(-0.0562330252965)*x[1]**o+(-0.919546510941)*x[1]+(0.376680654917)*x[2]**o+(-0.184153093526)*x[2]
        s[2,1,2,0]=(-0.135265768495)*x[0]**o+(0.650164259074)*x[0]+(0.224627083284)*x[1]**o+(0.918537471241)*x[1]+(0.662719594601)*x[2]**o+(-0.4451558313)*x[2]
        s[2,1,2,1]=(0.968484347906)*x[0]**o+(0.261215594011)*x[0]+(-0.478839261077)*x[1]**o+(-0.806554334141)*x[1]+(0.576441463783)*x[2]**o+(-0.407338671799)*x[2]
        s[2,2,0,0]=(-0.691105566226)*x[0]**o+(0.860347396649)*x[0]+(0.354066286829)*x[1]**o+(-0.0402175279153)*x[1]+(0.415237302648)*x[2]**o+(0.905209054794)*x[2]
        s[2,2,0,1]=(-0.548678761348)*x[0]**o+(-0.294803736974)*x[0]+(-0.960170319863)*x[1]**o+(-0.27333394559)*x[1]+(0.866348038561)*x[2]**o+(-0.977985070269)*x[2]
        s[2,2,1,0]=(-0.942843839038)*x[0]**o+(-0.868858708934)*x[0]+(-0.208988345792)*x[1]**o+(-0.628702405302)*x[1]+(0.856538967932)*x[2]**o+(-0.940751252093)*x[2]
        s[2,2,1,1]=(0.493835163998)*x[0]**o+(0.236452180515)*x[0]+(0.92903313506)*x[1]**o+(0.817122194244)*x[1]+(-0.498919897775)*x[2]**o+(0.177040227977)*x[2]
        s[2,2,2,0]=(-0.750931164998)*x[0]**o+(0.859659424304)*x[0]+(0.415426727903)*x[1]**o+(-0.468797574814)*x[1]+(-0.294847888148)*x[2]**o+(-0.992000733445)*x[2]
        s[2,2,2,1]=(0.0182583094659)*x[0]**o+(-0.830672513808)*x[0]+(0.37453609414)*x[1]**o+(-0.57449821779)*x[1]+(0.134490719543)*x[2]**o+(-0.609352246473)*x[2]
        s[2,3,0,0]=(-0.872131310391)*x[0]**o+(0.0825093683843)*x[0]+(-0.982804116245)*x[1]**o+(0.22341420851)*x[1]+(-0.231362455053)*x[2]**o+(-0.904631856625)*x[2]
        s[2,3,0,1]=(-0.0582824887741)*x[0]**o+(-0.399156766464)*x[0]+(0.0772313065438)*x[1]**o+(0.00426296268846)*x[1]+(-0.803184293158)*x[2]**o+(-0.17731453976)*x[2]
        s[2,3,1,0]=(0.252580558693)*x[0]**o+(-0.670939935472)*x[0]+(-0.396636904351)*x[1]**o+(-0.774693006552)*x[1]+(0.451637181152)*x[2]**o+(-0.463784221351)*x[2]
        s[2,3,1,1]=(0.126767645109)*x[0]**o+(-0.395030113635)*x[0]+(-0.955454976686)*x[1]**o+(-0.710220994617)*x[1]+(-0.397874215052)*x[2]**o+(-0.0217615150269)*x[2]
        s[2,3,2,0]=(0.990529883771)*x[0]**o+(0.278013247448)*x[0]+(-0.430442262972)*x[1]**o+(-0.720399995623)*x[1]+(0.610560781511)*x[2]**o+(-0.264692262808)*x[2]
        s[2,3,2,1]=(-0.776369460759)*x[0]**o+(0.397198841934)*x[0]+(-0.604527190952)*x[1]**o+(0.94368439705)*x[1]+(-0.88032288089)*x[2]**o+(0.989155014613)*x[2]
        s[2,4,0,0]=(0.457573740068)*x[0]**o+(-0.352003681775)*x[0]+(0.996428644418)*x[1]**o+(-0.348011873806)*x[1]+(-0.58023500034)*x[2]**o+(0.727228005186)*x[2]
        s[2,4,0,1]=(0.50989726873)*x[0]**o+(0.83338864798)*x[0]+(-0.111380969484)*x[1]**o+(-0.278057155647)*x[1]+(0.0022286233284)*x[2]**o+(0.841662666668)*x[2]
        s[2,4,1,0]=(-0.901603581146)*x[0]**o+(0.475205009348)*x[0]+(-0.430007910717)*x[1]**o+(-0.114746630672)*x[1]+(0.847548245931)*x[2]**o+(-0.539570230562)*x[2]
        s[2,4,1,1]=(0.539830165452)*x[0]**o+(-0.415758645187)*x[0]+(0.712970160554)*x[1]**o+(0.833164796207)*x[1]+(-0.65521876468)*x[2]**o+(-0.478072278167)*x[2]
        s[2,4,2,0]=(0.651879536878)*x[0]**o+(0.782082841616)*x[0]+(0.0416994850732)*x[1]**o+(0.0989265074417)*x[1]+(0.417171388449)*x[2]**o+(0.204544481473)*x[2]
        s[2,4,2,1]=(-0.957695309447)*x[0]**o+(0.41057815979)*x[0]+(0.847095166862)*x[1]**o+(0.937811945321)*x[1]+(-0.794606224039)*x[2]**o+(0.021545807724)*x[2]
        s[3,0,0,0]=(-0.521599558272)*x[0]**o+(-0.640095375476)*x[0]+(-0.51854438692)*x[1]**o+(-0.87210774891)*x[1]+(-0.693848689324)*x[2]**o+(-0.261635833944)*x[2]
        s[3,0,0,1]=(0.620197170573)*x[0]**o+(-0.841665901801)*x[0]+(-0.546759437483)*x[1]**o+(-0.168579145925)*x[1]+(-0.576567932552)*x[2]**o+(-0.645039556295)*x[2]
        s[3,0,1,0]=(0.581444084796)*x[0]**o+(-0.0462338578443)*x[0]+(0.117244760003)*x[1]**o+(0.732980884473)*x[1]+(-0.765006403884)*x[2]**o+(-0.543045061256)*x[2]
        s[3,0,1,1]=(0.695840005801)*x[0]**o+(-0.650014497536)*x[0]+(-0.801799125938)*x[1]**o+(-0.482643409576)*x[1]+(-0.416787187079)*x[2]**o+(0.584087905418)*x[2]
        s[3,0,2,0]=(-0.474618797242)*x[0]**o+(0.0189438474293)*x[0]+(-0.982174873996)*x[1]**o+(0.866178328676)*x[1]+(0.208673392242)*x[2]**o+(0.260241441013)*x[2]
        s[3,0,2,1]=(0.764105959702)*x[0]**o+(-0.578329182229)*x[0]+(0.556936430537)*x[1]**o+(0.0927824162943)*x[1]+(0.541941913665)*x[2]**o+(-0.20051953035)*x[2]
        s[3,1,0,0]=(0.467656474356)*x[0]**o+(0.20979187037)*x[0]+(0.830045363621)*x[1]**o+(0.305144956736)*x[1]+(-0.75827792525)*x[2]**o+(-0.336399882938)*x[2]
        s[3,1,0,1]=(-0.608709029175)*x[0]**o+(0.856296131715)*x[0]+(0.740185602478)*x[1]**o+(-0.899456455583)*x[1]+(-0.876657902509)*x[2]**o+(0.364216124717)*x[2]
        s[3,1,1,0]=(-0.19192395812)*x[0]**o+(0.106118249552)*x[0]+(-0.894241926881)*x[1]**o+(-0.985455697871)*x[1]+(-0.782102817165)*x[2]**o+(0.600080360854)*x[2]
        s[3,1,1,1]=(0.721311552863)*x[0]**o+(0.882962028856)*x[0]+(0.150876040487)*x[1]**o+(0.726055027812)*x[1]+(0.379204844464)*x[2]**o+(0.451150402164)*x[2]
        s[3,1,2,0]=(-0.694311490249)*x[0]**o+(-0.0875834002548)*x[0]+(-0.496200108766)*x[1]**o+(-0.151348317151)*x[1]+(0.854949924292)*x[2]**o+(0.915644549228)*x[2]
        s[3,1,2,1]=(0.470664286879)*x[0]**o+(0.706777924428)*x[0]+(-0.402719482127)*x[1]**o+(0.354894757591)*x[1]+(-0.472386640978)*x[2]**o+(0.947399400151)*x[2]
        s[3,2,0,0]=(-0.348423303161)*x[0]**o+(0.555948931484)*x[0]+(-0.43100597992)*x[1]**o+(-0.616049833968)*x[1]+(-0.150797584473)*x[2]**o+(0.520748537644)*x[2]
        s[3,2,0,1]=(0.138677687017)*x[0]**o+(0.18650212002)*x[0]+(0.260025448474)*x[1]**o+(0.0500586777217)*x[1]+(0.588917447067)*x[2]**o+(0.631694471884)*x[2]
        s[3,2,1,0]=(0.732017093416)*x[0]**o+(0.454265152556)*x[0]+(0.656152173314)*x[1]**o+(-0.214651040504)*x[1]+(0.606594898698)*x[2]**o+(-0.944529888173)*x[2]
        s[3,2,1,1]=(0.81276769768)*x[0]**o+(0.955839641144)*x[0]+(-0.597801282701)*x[1]**o+(0.203084441108)*x[1]+(0.371816930827)*x[2]**o+(-0.271843831057)*x[2]
        s[3,2,2,0]=(-0.0175874259272)*x[0]**o+(0.714416611293)*x[0]+(-0.446272670147)*x[1]**o+(-0.749224258055)*x[1]+(0.0434115533854)*x[2]**o+(-0.988008357199)*x[2]
        s[3,2,2,1]=(-0.219583104946)*x[0]**o+(0.207359013184)*x[0]+(-0.891027011221)*x[1]**o+(0.587776096493)*x[1]+(-0.545546707003)*x[2]**o+(-0.964582011253)*x[2]
        s[3,3,0,0]=(-0.540327832705)*x[0]**o+(0.489344676308)*x[0]+(0.663864603046)*x[1]**o+(-0.382248762805)*x[1]+(-0.729410073862)*x[2]**o+(-0.0325319636233)*x[2]
        s[3,3,0,1]=(0.259904022099)*x[0]**o+(-0.954974552252)*x[0]+(0.807969897038)*x[1]**o+(-0.535211587892)*x[1]+(0.401555747708)*x[2]**o+(0.80508329455)*x[2]
        s[3,3,1,0]=(0.172742587833)*x[0]**o+(-0.548601545874)*x[0]+(-0.991112338996)*x[1]**o+(-0.43054209437)*x[1]+(-0.32739577851)*x[2]**o+(-0.774125262769)*x[2]
        s[3,3,1,1]=(0.845744554328)*x[0]**o+(-0.258867050163)*x[0]+(-0.204629368407)*x[1]**o+(-0.821603302256)*x[1]+(-0.828098528219)*x[2]**o+(0.777673858589)*x[2]
        s[3,3,2,0]=(0.852991896102)*x[0]**o+(-0.745196256354)*x[0]+(0.220492132429)*x[1]**o+(-0.953335053523)*x[1]+(-0.324748822468)*x[2]**o+(-0.271392188657)*x[2]
        s[3,3,2,1]=(-0.256699792962)*x[0]**o+(0.670574693398)*x[0]+(0.538127693066)*x[1]**o+(-0.8291145795)*x[1]+(-0.152733016837)*x[2]**o+(0.401659214289)*x[2]
        s[3,4,0,0]=(-0.185294031294)*x[0]**o+(0.0683648982912)*x[0]+(-0.375470928752)*x[1]**o+(-0.67810003631)*x[1]+(-0.189259339657)*x[2]**o+(-0.174734934234)*x[2]
        s[3,4,0,1]=(-0.690376152222)*x[0]**o+(-0.021959548712)*x[0]+(0.816545400525)*x[1]**o+(0.743074566253)*x[1]+(0.907319374762)*x[2]**o+(-0.399104880981)*x[2]
        s[3,4,1,0]=(0.00764653168626)*x[0]**o+(0.418055838525)*x[0]+(-0.711231894711)*x[1]**o+(-0.319907118923)*x[1]+(0.230294344053)*x[2]**o+(0.852841918128)*x[2]
        s[3,4,1,1]=(0.773576800189)*x[0]**o+(-0.258243482679)*x[0]+(-0.217450374857)*x[1]**o+(0.0765524835501)*x[1]+(0.602823749066)*x[2]**o+(0.960046729433)*x[2]
        s[3,4,2,0]=(0.049879518732)*x[0]**o+(0.780027362911)*x[0]+(0.352479979369)*x[1]**o+(-0.960180931719)*x[1]+(0.220657155288)*x[2]**o+(-0.534538076664)*x[2]
        s[3,4,2,1]=(-0.260286357506)*x[0]**o+(0.319391110493)*x[0]+(0.0657794068301)*x[1]**o+(0.254797964539)*x[1]+(-0.495401153451)*x[2]**o+(-0.339881488587)*x[2]
        ref[0,0,0,0]=(-0.853604830323)*x_ref[0]**o+(0.378465791865)*x_ref[0]+(-0.83364075876)*x_ref[1]**o+(0.212018195408)*x_ref[1]+(0.48856925163)*x_ref[2]**o+(-0.179831169531)*x_ref[2]
        ref[0,0,0,1]=(0.868673673397)*x_ref[0]**o+(-0.238560476072)*x_ref[0]+(0.828785612257)*x_ref[1]**o+(0.176759782363)*x_ref[1]+(0.525888840201)*x_ref[2]**o+(0.436158744893)*x_ref[2]
        ref[0,0,1,0]=(0.887888476899)*x_ref[0]**o+(0.458452579078)*x_ref[0]+(0.572312971484)*x_ref[1]**o+(0.694648297654)*x_ref[1]+(0.244770581638)*x_ref[2]**o+(-0.326433505221)*x_ref[2]
        ref[0,0,1,1]=(0.958595740398)*x_ref[0]**o+(-0.448016561436)*x_ref[0]+(-0.898608139656)*x_ref[1]**o+(0.0189044802348)*x_ref[1]+(0.664802638624)*x_ref[2]**o+(0.675631565244)*x_ref[2]
        ref[0,0,2,0]=(-0.0968613941718)*x_ref[0]**o+(-0.580198871721)*x_ref[0]+(-0.695139695389)*x_ref[1]**o+(0.670627088913)*x_ref[1]+(-0.466392817977)*x_ref[2]**o+(-0.414367914263)*x_ref[2]
        ref[0,0,2,1]=(0.0563724518255)*x_ref[0]**o+(0.141716006991)*x_ref[0]+(-0.889611317888)*x_ref[1]**o+(0.274193692615)*x_ref[1]+(0.81766066569)*x_ref[2]**o+(0.260242356947)*x_ref[2]
        ref[0,1,0,0]=(0.729916171245)*x_ref[0]**o+(-0.0512883787314)*x_ref[0]+(-0.395879068642)*x_ref[1]**o+(0.65153799151)*x_ref[1]+(-0.90747241683)*x_ref[2]**o+(0.556056661876)*x_ref[2]
        ref[0,1,0,1]=(-0.336962711878)*x_ref[0]**o+(0.494126376731)*x_ref[0]+(-0.0180540415242)*x_ref[1]**o+(-0.281208191895)*x_ref[1]+(-0.891281821502)*x_ref[2]**o+(0.434438114521)*x_ref[2]
        ref[0,1,1,0]=(-0.0203012608797)*x_ref[0]**o+(-0.592593200931)*x_ref[0]+(-0.908091999823)*x_ref[1]**o+(0.917265173937)*x_ref[1]+(0.0113338093354)*x_ref[2]**o+(0.832522528967)*x_ref[2]
        ref[0,1,1,1]=(-0.287082151578)*x_ref[0]**o+(-0.882563505538)*x_ref[0]+(-0.805577758903)*x_ref[1]**o+(-0.0877632778656)*x_ref[1]+(-0.4141073805)*x_ref[2]**o+(-0.0809945294401)*x_ref[2]
        ref[0,1,2,0]=(-0.745887649438)*x_ref[0]**o+(0.105423480913)*x_ref[0]+(0.642374896965)*x_ref[1]**o+(0.479730302425)*x_ref[1]+(0.334298930575)*x_ref[2]**o+(0.214588275036)*x_ref[2]
        ref[0,1,2,1]=(-0.987939537144)*x_ref[0]**o+(-0.889107799492)*x_ref[0]+(-0.133927268357)*x_ref[1]**o+(-0.991450009918)*x_ref[1]+(-0.201645805359)*x_ref[2]**o+(0.481104989077)*x_ref[2]
        ref[0,2,0,0]=(0.535918222555)*x_ref[0]**o+(0.243041294608)*x_ref[0]+(-0.205082148634)*x_ref[1]**o+(0.233470577382)*x_ref[1]+(0.398471417689)*x_ref[2]**o+(-0.56437758006)*x_ref[2]
        ref[0,2,0,1]=(0.374626206936)*x_ref[0]**o+(-0.800840698958)*x_ref[0]+(0.336427483065)*x_ref[1]**o+(0.143368857985)*x_ref[1]+(0.822396957468)*x_ref[2]**o+(-0.288884862626)*x_ref[2]
        ref[0,2,1,0]=(-0.152217915896)*x_ref[0]**o+(0.868324413095)*x_ref[0]+(0.93760108412)*x_ref[1]**o+(-0.584628102458)*x_ref[1]+(-0.678653990131)*x_ref[2]**o+(-0.130768977951)*x_ref[2]
        ref[0,2,1,1]=(-0.962809494754)*x_ref[0]**o+(0.057963920559)*x_ref[0]+(-0.68685683212)*x_ref[1]**o+(0.869986883492)*x_ref[1]+(-0.193987500788)*x_ref[2]**o+(0.0357315899177)*x_ref[2]
        ref[0,2,2,0]=(-0.296222468619)*x_ref[0]**o+(0.853293489381)*x_ref[0]+(0.886622498642)*x_ref[1]**o+(-0.22952370037)*x_ref[1]+(0.444962301355)*x_ref[2]**o+(0.498735104677)*x_ref[2]
        ref[0,2,2,1]=(-0.556796221789)*x_ref[0]**o+(-0.98717630759)*x_ref[0]+(-0.56578841851)*x_ref[1]**o+(-0.769428379953)*x_ref[1]+(0.41417947942)*x_ref[2]**o+(-0.99075609451)*x_ref[2]
        ref[0,3,0,0]=(-0.312173122533)*x_ref[0]**o+(-0.0183061811911)*x_ref[0]+(0.632028466217)*x_ref[1]**o+(-0.516798544207)*x_ref[1]+(0.467776726567)*x_ref[2]**o+(-0.0750898474112)*x_ref[2]
        ref[0,3,0,1]=(0.151818577102)*x_ref[0]**o+(0.275659634397)*x_ref[0]+(-0.695866418429)*x_ref[1]**o+(-0.158484337329)*x_ref[1]+(-0.717635621103)*x_ref[2]**o+(0.96218471765)*x_ref[2]
        ref[0,3,1,0]=(-0.959269035858)*x_ref[0]**o+(-0.694952577107)*x_ref[0]+(0.133713794285)*x_ref[1]**o+(0.945801566586)*x_ref[1]+(0.480976843012)*x_ref[2]**o+(0.514096820167)*x_ref[2]
        ref[0,3,1,1]=(-0.696429878723)*x_ref[0]**o+(0.844116956105)*x_ref[0]+(0.986596486924)*x_ref[1]**o+(0.213400513251)*x_ref[1]+(0.168567391983)*x_ref[2]**o+(-0.96577037709)*x_ref[2]
        ref[0,3,2,0]=(0.272554167042)*x_ref[0]**o+(-0.134445947881)*x_ref[0]+(-0.620927727232)*x_ref[1]**o+(0.774446825459)*x_ref[1]+(-0.140360641548)*x_ref[2]**o+(-0.393011913595)*x_ref[2]
        ref[0,3,2,1]=(0.973937360802)*x_ref[0]**o+(-0.780203033305)*x_ref[0]+(0.62508420402)*x_ref[1]**o+(-0.957359234026)*x_ref[1]+(0.301032273116)*x_ref[2]**o+(0.238130119107)*x_ref[2]
        ref[0,4,0,0]=(0.550167138562)*x_ref[0]**o+(-0.975271434354)*x_ref[0]+(-0.90226839553)*x_ref[1]**o+(0.765360109563)*x_ref[1]+(-0.0539842939871)*x_ref[2]**o+(-0.992811018253)*x_ref[2]
        ref[0,4,0,1]=(0.115680644235)*x_ref[0]**o+(0.6504959069)*x_ref[0]+(0.0711117194347)*x_ref[1]**o+(0.765247742051)*x_ref[1]+(0.651478097229)*x_ref[2]**o+(0.0605973413193)*x_ref[2]
        ref[0,4,1,0]=(-0.339495686607)*x_ref[0]**o+(0.98910452173)*x_ref[0]+(-0.97593906201)*x_ref[1]**o+(-0.961098427995)*x_ref[1]+(-0.970981798475)*x_ref[2]**o+(-0.655846307864)*x_ref[2]
        ref[0,4,1,1]=(-0.676422431554)*x_ref[0]**o+(0.17779634387)*x_ref[0]+(-0.101241446408)*x_ref[1]**o+(0.592319052036)*x_ref[1]+(-0.328471629455)*x_ref[2]**o+(0.842981957639)*x_ref[2]
        ref[0,4,2,0]=(0.475154370559)*x_ref[0]**o+(-0.861452809594)*x_ref[0]+(0.021220095967)*x_ref[1]**o+(-0.167324332634)*x_ref[1]+(-0.420334858996)*x_ref[2]**o+(-0.243503287995)*x_ref[2]
        ref[0,4,2,1]=(0.507818170756)*x_ref[0]**o+(0.0305556062843)*x_ref[0]+(0.657465103712)*x_ref[1]**o+(-0.133306218785)*x_ref[1]+(0.596182931411)*x_ref[2]**o+(0.26354220091)*x_ref[2]
        ref[1,0,0,0]=(0.141665798979)*x_ref[0]**o+(0.257839463098)*x_ref[0]+(0.20842719092)*x_ref[1]**o+(0.0606481804876)*x_ref[1]+(0.564188524366)*x_ref[2]**o+(0.276431563089)*x_ref[2]
        ref[1,0,0,1]=(-0.475871120292)*x_ref[0]**o+(-0.945285882278)*x_ref[0]+(-0.852943036141)*x_ref[1]**o+(-0.393053044641)*x_ref[1]+(0.894360138187)*x_ref[2]**o+(0.945580362088)*x_ref[2]
        ref[1,0,1,0]=(-0.269113046186)*x_ref[0]**o+(0.979226351783)*x_ref[0]+(0.59588973168)*x_ref[1]**o+(0.709182601617)*x_ref[1]+(0.203809169118)*x_ref[2]**o+(0.843792678336)*x_ref[2]
        ref[1,0,1,1]=(0.625152197275)*x_ref[0]**o+(0.700601491781)*x_ref[0]+(-0.556266788732)*x_ref[1]**o+(0.621296929469)*x_ref[1]+(0.370627217553)*x_ref[2]**o+(-0.86981081047)*x_ref[2]
        ref[1,0,2,0]=(0.337314024742)*x_ref[0]**o+(0.4936522561)*x_ref[0]+(-0.121004948433)*x_ref[1]**o+(0.682816106721)*x_ref[1]+(0.135016952475)*x_ref[2]**o+(0.447788511803)*x_ref[2]
        ref[1,0,2,1]=(0.627048609871)*x_ref[0]**o+(0.464430671751)*x_ref[0]+(0.248071099972)*x_ref[1]**o+(0.49700045944)*x_ref[1]+(0.764133255754)*x_ref[2]**o+(-0.407441090409)*x_ref[2]
        ref[1,1,0,0]=(-0.891648124769)*x_ref[0]**o+(-0.852500694418)*x_ref[0]+(-0.501966394474)*x_ref[1]**o+(-0.951197952585)*x_ref[1]+(-0.293731401112)*x_ref[2]**o+(0.340542703849)*x_ref[2]
        ref[1,1,0,1]=(0.047745963453)*x_ref[0]**o+(0.605499497949)*x_ref[0]+(0.603025931833)*x_ref[1]**o+(0.412711498009)*x_ref[1]+(-0.706970380941)*x_ref[2]**o+(-0.865479677059)*x_ref[2]
        ref[1,1,1,0]=(-0.809326436683)*x_ref[0]**o+(0.50103064436)*x_ref[0]+(-0.726029943072)*x_ref[1]**o+(-0.0136545501065)*x_ref[1]+(0.869298450797)*x_ref[2]**o+(-0.0539795856427)*x_ref[2]
        ref[1,1,1,1]=(0.822265437858)*x_ref[0]**o+(-0.13686562101)*x_ref[0]+(-0.921877204478)*x_ref[1]**o+(0.436216049238)*x_ref[1]+(0.167449871886)*x_ref[2]**o+(-0.376043518724)*x_ref[2]
        ref[1,1,2,0]=(-0.240904110506)*x_ref[0]**o+(-0.709486977651)*x_ref[0]+(0.589748137401)*x_ref[1]**o+(0.997759434219)*x_ref[1]+(0.211495210823)*x_ref[2]**o+(0.857954370359)*x_ref[2]
        ref[1,1,2,1]=(-0.872650055395)*x_ref[0]**o+(0.488392921139)*x_ref[0]+(0.456778341472)*x_ref[1]**o+(0.533360514682)*x_ref[1]+(0.660463672454)*x_ref[2]**o+(-0.287728736994)*x_ref[2]
        ref[1,2,0,0]=(-0.56865288172)*x_ref[0]**o+(-0.257707278342)*x_ref[0]+(0.492925973387)*x_ref[1]**o+(-0.433521245223)*x_ref[1]+(-0.214711127244)*x_ref[2]**o+(-0.578235847487)*x_ref[2]
        ref[1,2,0,1]=(0.502020345167)*x_ref[0]**o+(-0.898730931974)*x_ref[0]+(0.152379872963)*x_ref[1]**o+(0.963247300695)*x_ref[1]+(-0.935219139549)*x_ref[2]**o+(-0.322095550311)*x_ref[2]
        ref[1,2,1,0]=(0.755425499434)*x_ref[0]**o+(0.922722006611)*x_ref[0]+(-0.854913831787)*x_ref[1]**o+(0.28200116496)*x_ref[1]+(0.654754619366)*x_ref[2]**o+(-0.87246359532)*x_ref[2]
        ref[1,2,1,1]=(0.476533428226)*x_ref[0]**o+(-0.21427962663)*x_ref[0]+(-0.42807986959)*x_ref[1]**o+(-0.453270655163)*x_ref[1]+(-0.0618352146195)*x_ref[2]**o+(-0.0578564702081)*x_ref[2]
        ref[1,2,2,0]=(0.836242039415)*x_ref[0]**o+(0.0582519660745)*x_ref[0]+(-0.736245953793)*x_ref[1]**o+(-0.0216606630383)*x_ref[1]+(-0.241691127038)*x_ref[2]**o+(-0.273769285415)*x_ref[2]
        ref[1,2,2,1]=(0.956981101891)*x_ref[0]**o+(-0.619600309731)*x_ref[0]+(0.255999571352)*x_ref[1]**o+(0.931443107924)*x_ref[1]+(0.803455662831)*x_ref[2]**o+(0.587378690917)*x_ref[2]
        ref[1,3,0,0]=(0.84725602389)*x_ref[0]**o+(0.099929556134)*x_ref[0]+(0.918513623927)*x_ref[1]**o+(-0.540239895987)*x_ref[1]+(0.543336053141)*x_ref[2]**o+(-0.35339148993)*x_ref[2]
        ref[1,3,0,1]=(0.0790066953583)*x_ref[0]**o+(-0.622984122413)*x_ref[0]+(0.844477077556)*x_ref[1]**o+(-0.843967484949)*x_ref[1]+(0.382810149795)*x_ref[2]**o+(0.693483174672)*x_ref[2]
        ref[1,3,1,0]=(-0.606850566406)*x_ref[0]**o+(-0.613588487235)*x_ref[0]+(-0.982801043706)*x_ref[1]**o+(-0.200157044491)*x_ref[1]+(0.865010142689)*x_ref[2]**o+(-0.926404146796)*x_ref[2]
        ref[1,3,1,1]=(-0.926332516404)*x_ref[0]**o+(-0.644403030771)*x_ref[0]+(-0.391941644842)*x_ref[1]**o+(-0.628555138555)*x_ref[1]+(0.584646630734)*x_ref[2]**o+(0.627561535294)*x_ref[2]
        ref[1,3,2,0]=(0.777960693446)*x_ref[0]**o+(-0.580698013493)*x_ref[0]+(-0.061588850878)*x_ref[1]**o+(0.909167097211)*x_ref[1]+(-0.346386067681)*x_ref[2]**o+(-0.698593831635)*x_ref[2]
        ref[1,3,2,1]=(-0.25740731816)*x_ref[0]**o+(0.0609560824945)*x_ref[0]+(-0.866636970162)*x_ref[1]**o+(0.352496912986)*x_ref[1]+(0.212276243322)*x_ref[2]**o+(-0.29499231054)*x_ref[2]
        ref[1,4,0,0]=(0.176828476382)*x_ref[0]**o+(0.592116728712)*x_ref[0]+(-0.969737040463)*x_ref[1]**o+(-0.0140144291993)*x_ref[1]+(-0.203791892462)*x_ref[2]**o+(0.687043221294)*x_ref[2]
        ref[1,4,0,1]=(-0.0673562914955)*x_ref[0]**o+(0.104813124019)*x_ref[0]+(0.766386480162)*x_ref[1]**o+(-0.499215759851)*x_ref[1]+(0.163490109505)*x_ref[2]**o+(0.0477449791182)*x_ref[2]
        ref[1,4,1,0]=(-0.186436883357)*x_ref[0]**o+(0.771658464504)*x_ref[0]+(0.115845567912)*x_ref[1]**o+(-0.573016245745)*x_ref[1]+(0.759680038826)*x_ref[2]**o+(0.0660145528929)*x_ref[2]
        ref[1,4,1,1]=(-0.983746295741)*x_ref[0]**o+(-0.370146292969)*x_ref[0]+(-0.276398856773)*x_ref[1]**o+(-0.765483196375)*x_ref[1]+(-0.0802845394538)*x_ref[2]**o+(0.762085198906)*x_ref[2]
        ref[1,4,2,0]=(-0.437323501168)*x_ref[0]**o+(-0.076976193125)*x_ref[0]+(0.780179691457)*x_ref[1]**o+(0.941748527645)*x_ref[1]+(0.921220023779)*x_ref[2]**o+(-0.611350278287)*x_ref[2]
        ref[1,4,2,1]=(-0.811077839585)*x_ref[0]**o+(-0.0781699097656)*x_ref[0]+(-0.955979549084)*x_ref[1]**o+(0.742581422414)*x_ref[1]+(-0.905829236752)*x_ref[2]**o+(-0.671257774729)*x_ref[2]
        ref[2,0,0,0]=(0.3602498958)*x_ref[0]**o+(0.567952303965)*x_ref[0]+(-0.72126509422)*x_ref[1]**o+(0.652912038045)*x_ref[1]+(-0.448081819441)*x_ref[2]**o+(-0.628140847819)*x_ref[2]
        ref[2,0,0,1]=(0.823191679062)*x_ref[0]**o+(-0.12280206123)*x_ref[0]+(0.202149797631)*x_ref[1]**o+(-0.751457177895)*x_ref[1]+(0.367057556408)*x_ref[2]**o+(0.71681510921)*x_ref[2]
        ref[2,0,1,0]=(-0.553277801352)*x_ref[0]**o+(-0.695114786088)*x_ref[0]+(-0.506695185331)*x_ref[1]**o+(0.947900606544)*x_ref[1]+(0.393574900465)*x_ref[2]**o+(-0.131907977332)*x_ref[2]
        ref[2,0,1,1]=(-0.683260439471)*x_ref[0]**o+(0.108921790681)*x_ref[0]+(0.694361429808)*x_ref[1]**o+(-0.879827908202)*x_ref[1]+(-0.768400141399)*x_ref[2]**o+(-0.869535466479)*x_ref[2]
        ref[2,0,2,0]=(-0.377844410367)*x_ref[0]**o+(-0.486876364642)*x_ref[0]+(-0.139990170975)*x_ref[1]**o+(0.521294626989)*x_ref[1]+(0.384721944946)*x_ref[2]**o+(0.912206678795)*x_ref[2]
        ref[2,0,2,1]=(0.624159317849)*x_ref[0]**o+(0.910540130065)*x_ref[0]+(-0.879166404339)*x_ref[1]**o+(0.710229335366)*x_ref[1]+(-0.756806360688)*x_ref[2]**o+(-0.169809484076)*x_ref[2]
        ref[2,1,0,0]=(-0.887833054092)*x_ref[0]**o+(-0.541562581783)*x_ref[0]+(-0.439734700726)*x_ref[1]**o+(-0.294103567325)*x_ref[1]+(0.621145429347)*x_ref[2]**o+(0.65618994558)*x_ref[2]
        ref[2,1,0,1]=(-0.199707709698)*x_ref[0]**o+(0.952895238716)*x_ref[0]+(-0.113807788254)*x_ref[1]**o+(-0.209497731554)*x_ref[1]+(-0.201935349119)*x_ref[2]**o+(0.896172767252)*x_ref[2]
        ref[2,1,1,0]=(0.961864846935)*x_ref[0]**o+(0.34037006989)*x_ref[0]+(0.97866208125)*x_ref[1]**o+(0.949937962054)*x_ref[1]+(-0.943237740253)*x_ref[2]**o+(0.492751804379)*x_ref[2]
        ref[2,1,1,1]=(0.0364913119154)*x_ref[0]**o+(-0.706276345936)*x_ref[0]+(-0.0562330252965)*x_ref[1]**o+(-0.919546510941)*x_ref[1]+(0.376680654917)*x_ref[2]**o+(-0.184153093526)*x_ref[2]
        ref[2,1,2,0]=(-0.135265768495)*x_ref[0]**o+(0.650164259074)*x_ref[0]+(0.224627083284)*x_ref[1]**o+(0.918537471241)*x_ref[1]+(0.662719594601)*x_ref[2]**o+(-0.4451558313)*x_ref[2]
        ref[2,1,2,1]=(0.968484347906)*x_ref[0]**o+(0.261215594011)*x_ref[0]+(-0.478839261077)*x_ref[1]**o+(-0.806554334141)*x_ref[1]+(0.576441463783)*x_ref[2]**o+(-0.407338671799)*x_ref[2]
        ref[2,2,0,0]=(-0.691105566226)*x_ref[0]**o+(0.860347396649)*x_ref[0]+(0.354066286829)*x_ref[1]**o+(-0.0402175279153)*x_ref[1]+(0.415237302648)*x_ref[2]**o+(0.905209054794)*x_ref[2]
        ref[2,2,0,1]=(-0.548678761348)*x_ref[0]**o+(-0.294803736974)*x_ref[0]+(-0.960170319863)*x_ref[1]**o+(-0.27333394559)*x_ref[1]+(0.866348038561)*x_ref[2]**o+(-0.977985070269)*x_ref[2]
        ref[2,2,1,0]=(-0.942843839038)*x_ref[0]**o+(-0.868858708934)*x_ref[0]+(-0.208988345792)*x_ref[1]**o+(-0.628702405302)*x_ref[1]+(0.856538967932)*x_ref[2]**o+(-0.940751252093)*x_ref[2]
        ref[2,2,1,1]=(0.493835163998)*x_ref[0]**o+(0.236452180515)*x_ref[0]+(0.92903313506)*x_ref[1]**o+(0.817122194244)*x_ref[1]+(-0.498919897775)*x_ref[2]**o+(0.177040227977)*x_ref[2]
        ref[2,2,2,0]=(-0.750931164998)*x_ref[0]**o+(0.859659424304)*x_ref[0]+(0.415426727903)*x_ref[1]**o+(-0.468797574814)*x_ref[1]+(-0.294847888148)*x_ref[2]**o+(-0.992000733445)*x_ref[2]
        ref[2,2,2,1]=(0.0182583094659)*x_ref[0]**o+(-0.830672513808)*x_ref[0]+(0.37453609414)*x_ref[1]**o+(-0.57449821779)*x_ref[1]+(0.134490719543)*x_ref[2]**o+(-0.609352246473)*x_ref[2]
        ref[2,3,0,0]=(-0.872131310391)*x_ref[0]**o+(0.0825093683843)*x_ref[0]+(-0.982804116245)*x_ref[1]**o+(0.22341420851)*x_ref[1]+(-0.231362455053)*x_ref[2]**o+(-0.904631856625)*x_ref[2]
        ref[2,3,0,1]=(-0.0582824887741)*x_ref[0]**o+(-0.399156766464)*x_ref[0]+(0.0772313065438)*x_ref[1]**o+(0.00426296268846)*x_ref[1]+(-0.803184293158)*x_ref[2]**o+(-0.17731453976)*x_ref[2]
        ref[2,3,1,0]=(0.252580558693)*x_ref[0]**o+(-0.670939935472)*x_ref[0]+(-0.396636904351)*x_ref[1]**o+(-0.774693006552)*x_ref[1]+(0.451637181152)*x_ref[2]**o+(-0.463784221351)*x_ref[2]
        ref[2,3,1,1]=(0.126767645109)*x_ref[0]**o+(-0.395030113635)*x_ref[0]+(-0.955454976686)*x_ref[1]**o+(-0.710220994617)*x_ref[1]+(-0.397874215052)*x_ref[2]**o+(-0.0217615150269)*x_ref[2]
        ref[2,3,2,0]=(0.990529883771)*x_ref[0]**o+(0.278013247448)*x_ref[0]+(-0.430442262972)*x_ref[1]**o+(-0.720399995623)*x_ref[1]+(0.610560781511)*x_ref[2]**o+(-0.264692262808)*x_ref[2]
        ref[2,3,2,1]=(-0.776369460759)*x_ref[0]**o+(0.397198841934)*x_ref[0]+(-0.604527190952)*x_ref[1]**o+(0.94368439705)*x_ref[1]+(-0.88032288089)*x_ref[2]**o+(0.989155014613)*x_ref[2]
        ref[2,4,0,0]=(0.457573740068)*x_ref[0]**o+(-0.352003681775)*x_ref[0]+(0.996428644418)*x_ref[1]**o+(-0.348011873806)*x_ref[1]+(-0.58023500034)*x_ref[2]**o+(0.727228005186)*x_ref[2]
        ref[2,4,0,1]=(0.50989726873)*x_ref[0]**o+(0.83338864798)*x_ref[0]+(-0.111380969484)*x_ref[1]**o+(-0.278057155647)*x_ref[1]+(0.0022286233284)*x_ref[2]**o+(0.841662666668)*x_ref[2]
        ref[2,4,1,0]=(-0.901603581146)*x_ref[0]**o+(0.475205009348)*x_ref[0]+(-0.430007910717)*x_ref[1]**o+(-0.114746630672)*x_ref[1]+(0.847548245931)*x_ref[2]**o+(-0.539570230562)*x_ref[2]
        ref[2,4,1,1]=(0.539830165452)*x_ref[0]**o+(-0.415758645187)*x_ref[0]+(0.712970160554)*x_ref[1]**o+(0.833164796207)*x_ref[1]+(-0.65521876468)*x_ref[2]**o+(-0.478072278167)*x_ref[2]
        ref[2,4,2,0]=(0.651879536878)*x_ref[0]**o+(0.782082841616)*x_ref[0]+(0.0416994850732)*x_ref[1]**o+(0.0989265074417)*x_ref[1]+(0.417171388449)*x_ref[2]**o+(0.204544481473)*x_ref[2]
        ref[2,4,2,1]=(-0.957695309447)*x_ref[0]**o+(0.41057815979)*x_ref[0]+(0.847095166862)*x_ref[1]**o+(0.937811945321)*x_ref[1]+(-0.794606224039)*x_ref[2]**o+(0.021545807724)*x_ref[2]
        ref[3,0,0,0]=(-0.521599558272)*x_ref[0]**o+(-0.640095375476)*x_ref[0]+(-0.51854438692)*x_ref[1]**o+(-0.87210774891)*x_ref[1]+(-0.693848689324)*x_ref[2]**o+(-0.261635833944)*x_ref[2]
        ref[3,0,0,1]=(0.620197170573)*x_ref[0]**o+(-0.841665901801)*x_ref[0]+(-0.546759437483)*x_ref[1]**o+(-0.168579145925)*x_ref[1]+(-0.576567932552)*x_ref[2]**o+(-0.645039556295)*x_ref[2]
        ref[3,0,1,0]=(0.581444084796)*x_ref[0]**o+(-0.0462338578443)*x_ref[0]+(0.117244760003)*x_ref[1]**o+(0.732980884473)*x_ref[1]+(-0.765006403884)*x_ref[2]**o+(-0.543045061256)*x_ref[2]
        ref[3,0,1,1]=(0.695840005801)*x_ref[0]**o+(-0.650014497536)*x_ref[0]+(-0.801799125938)*x_ref[1]**o+(-0.482643409576)*x_ref[1]+(-0.416787187079)*x_ref[2]**o+(0.584087905418)*x_ref[2]
        ref[3,0,2,0]=(-0.474618797242)*x_ref[0]**o+(0.0189438474293)*x_ref[0]+(-0.982174873996)*x_ref[1]**o+(0.866178328676)*x_ref[1]+(0.208673392242)*x_ref[2]**o+(0.260241441013)*x_ref[2]
        ref[3,0,2,1]=(0.764105959702)*x_ref[0]**o+(-0.578329182229)*x_ref[0]+(0.556936430537)*x_ref[1]**o+(0.0927824162943)*x_ref[1]+(0.541941913665)*x_ref[2]**o+(-0.20051953035)*x_ref[2]
        ref[3,1,0,0]=(0.467656474356)*x_ref[0]**o+(0.20979187037)*x_ref[0]+(0.830045363621)*x_ref[1]**o+(0.305144956736)*x_ref[1]+(-0.75827792525)*x_ref[2]**o+(-0.336399882938)*x_ref[2]
        ref[3,1,0,1]=(-0.608709029175)*x_ref[0]**o+(0.856296131715)*x_ref[0]+(0.740185602478)*x_ref[1]**o+(-0.899456455583)*x_ref[1]+(-0.876657902509)*x_ref[2]**o+(0.364216124717)*x_ref[2]
        ref[3,1,1,0]=(-0.19192395812)*x_ref[0]**o+(0.106118249552)*x_ref[0]+(-0.894241926881)*x_ref[1]**o+(-0.985455697871)*x_ref[1]+(-0.782102817165)*x_ref[2]**o+(0.600080360854)*x_ref[2]
        ref[3,1,1,1]=(0.721311552863)*x_ref[0]**o+(0.882962028856)*x_ref[0]+(0.150876040487)*x_ref[1]**o+(0.726055027812)*x_ref[1]+(0.379204844464)*x_ref[2]**o+(0.451150402164)*x_ref[2]
        ref[3,1,2,0]=(-0.694311490249)*x_ref[0]**o+(-0.0875834002548)*x_ref[0]+(-0.496200108766)*x_ref[1]**o+(-0.151348317151)*x_ref[1]+(0.854949924292)*x_ref[2]**o+(0.915644549228)*x_ref[2]
        ref[3,1,2,1]=(0.470664286879)*x_ref[0]**o+(0.706777924428)*x_ref[0]+(-0.402719482127)*x_ref[1]**o+(0.354894757591)*x_ref[1]+(-0.472386640978)*x_ref[2]**o+(0.947399400151)*x_ref[2]
        ref[3,2,0,0]=(-0.348423303161)*x_ref[0]**o+(0.555948931484)*x_ref[0]+(-0.43100597992)*x_ref[1]**o+(-0.616049833968)*x_ref[1]+(-0.150797584473)*x_ref[2]**o+(0.520748537644)*x_ref[2]
        ref[3,2,0,1]=(0.138677687017)*x_ref[0]**o+(0.18650212002)*x_ref[0]+(0.260025448474)*x_ref[1]**o+(0.0500586777217)*x_ref[1]+(0.588917447067)*x_ref[2]**o+(0.631694471884)*x_ref[2]
        ref[3,2,1,0]=(0.732017093416)*x_ref[0]**o+(0.454265152556)*x_ref[0]+(0.656152173314)*x_ref[1]**o+(-0.214651040504)*x_ref[1]+(0.606594898698)*x_ref[2]**o+(-0.944529888173)*x_ref[2]
        ref[3,2,1,1]=(0.81276769768)*x_ref[0]**o+(0.955839641144)*x_ref[0]+(-0.597801282701)*x_ref[1]**o+(0.203084441108)*x_ref[1]+(0.371816930827)*x_ref[2]**o+(-0.271843831057)*x_ref[2]
        ref[3,2,2,0]=(-0.0175874259272)*x_ref[0]**o+(0.714416611293)*x_ref[0]+(-0.446272670147)*x_ref[1]**o+(-0.749224258055)*x_ref[1]+(0.0434115533854)*x_ref[2]**o+(-0.988008357199)*x_ref[2]
        ref[3,2,2,1]=(-0.219583104946)*x_ref[0]**o+(0.207359013184)*x_ref[0]+(-0.891027011221)*x_ref[1]**o+(0.587776096493)*x_ref[1]+(-0.545546707003)*x_ref[2]**o+(-0.964582011253)*x_ref[2]
        ref[3,3,0,0]=(-0.540327832705)*x_ref[0]**o+(0.489344676308)*x_ref[0]+(0.663864603046)*x_ref[1]**o+(-0.382248762805)*x_ref[1]+(-0.729410073862)*x_ref[2]**o+(-0.0325319636233)*x_ref[2]
        ref[3,3,0,1]=(0.259904022099)*x_ref[0]**o+(-0.954974552252)*x_ref[0]+(0.807969897038)*x_ref[1]**o+(-0.535211587892)*x_ref[1]+(0.401555747708)*x_ref[2]**o+(0.80508329455)*x_ref[2]
        ref[3,3,1,0]=(0.172742587833)*x_ref[0]**o+(-0.548601545874)*x_ref[0]+(-0.991112338996)*x_ref[1]**o+(-0.43054209437)*x_ref[1]+(-0.32739577851)*x_ref[2]**o+(-0.774125262769)*x_ref[2]
        ref[3,3,1,1]=(0.845744554328)*x_ref[0]**o+(-0.258867050163)*x_ref[0]+(-0.204629368407)*x_ref[1]**o+(-0.821603302256)*x_ref[1]+(-0.828098528219)*x_ref[2]**o+(0.777673858589)*x_ref[2]
        ref[3,3,2,0]=(0.852991896102)*x_ref[0]**o+(-0.745196256354)*x_ref[0]+(0.220492132429)*x_ref[1]**o+(-0.953335053523)*x_ref[1]+(-0.324748822468)*x_ref[2]**o+(-0.271392188657)*x_ref[2]
        ref[3,3,2,1]=(-0.256699792962)*x_ref[0]**o+(0.670574693398)*x_ref[0]+(0.538127693066)*x_ref[1]**o+(-0.8291145795)*x_ref[1]+(-0.152733016837)*x_ref[2]**o+(0.401659214289)*x_ref[2]
        ref[3,4,0,0]=(-0.185294031294)*x_ref[0]**o+(0.0683648982912)*x_ref[0]+(-0.375470928752)*x_ref[1]**o+(-0.67810003631)*x_ref[1]+(-0.189259339657)*x_ref[2]**o+(-0.174734934234)*x_ref[2]
        ref[3,4,0,1]=(-0.690376152222)*x_ref[0]**o+(-0.021959548712)*x_ref[0]+(0.816545400525)*x_ref[1]**o+(0.743074566253)*x_ref[1]+(0.907319374762)*x_ref[2]**o+(-0.399104880981)*x_ref[2]
        ref[3,4,1,0]=(0.00764653168626)*x_ref[0]**o+(0.418055838525)*x_ref[0]+(-0.711231894711)*x_ref[1]**o+(-0.319907118923)*x_ref[1]+(0.230294344053)*x_ref[2]**o+(0.852841918128)*x_ref[2]
        ref[3,4,1,1]=(0.773576800189)*x_ref[0]**o+(-0.258243482679)*x_ref[0]+(-0.217450374857)*x_ref[1]**o+(0.0765524835501)*x_ref[1]+(0.602823749066)*x_ref[2]**o+(0.960046729433)*x_ref[2]
        ref[3,4,2,0]=(0.049879518732)*x_ref[0]**o+(0.780027362911)*x_ref[0]+(0.352479979369)*x_ref[1]**o+(-0.960180931719)*x_ref[1]+(0.220657155288)*x_ref[2]**o+(-0.534538076664)*x_ref[2]
        ref[3,4,2,1]=(-0.260286357506)*x_ref[0]**o+(0.319391110493)*x_ref[0]+(0.0657794068301)*x_ref[1]**o+(0.254797964539)*x_ref[1]+(-0.495401153451)*x_ref[2]**o+(-0.339881488587)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_ReducedFunctionOnContactOne_rank0(self):
      """
      tests interpolate for rank 0 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactOne onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactOne(self.domain)
      x=w.getX()
      arg=Symbol(shape=(),dim=dim)
      s=Data(0,(),w)
      ref=Data(0,(),w_ref)
      if dim==2:
        s=(0.992324887623)*x[0]**o+(-0.399021408175)*x[0]+(-0.82044668182)*x[1]**o+(-0.115930766833)*x[1]
        ref=(0.992324887623)*x_ref[0]**o+(-0.399021408175)*x_ref[0]+(-0.82044668182)*x_ref[1]**o+(-0.115930766833)*x_ref[1]
      else:
        s=(-0.917591389532)*x[0]**o+(-0.636147522121)*x[0]+(-0.624726334305)*x[1]**o+(0.253824967861)*x[1]+(0.391376692454)*x[2]**o+(-0.635535244093)*x[2]
        ref=(-0.917591389532)*x_ref[0]**o+(-0.636147522121)*x_ref[0]+(-0.624726334305)*x_ref[1]**o+(0.253824967861)*x_ref[1]+(0.391376692454)*x_ref[2]**o+(-0.635535244093)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_ReducedFunctionOnContactOne_rank1(self):
      """
      tests interpolate for rank 1 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactOne onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactOne(self.domain)
      x=w.getX()
      arg=Symbol(shape=(2,),dim=dim)
      s=Data(0,(2,),w)
      ref=Data(0,(2,),w_ref)
      if dim==2:
        s[0]=(-0.420392705289)*x[0]**o+(-0.988154722022)*x[0]+(-0.865378713801)*x[1]**o+(0.992312897132)*x[1]
        s[1]=(-0.48354228784)*x[0]**o+(0.07109068872)*x[0]+(0.757923528294)*x[1]**o+(0.903265965794)*x[1]
        ref[0]=(-0.420392705289)*x_ref[0]**o+(-0.988154722022)*x_ref[0]+(-0.865378713801)*x_ref[1]**o+(0.992312897132)*x_ref[1]
        ref[1]=(-0.48354228784)*x_ref[0]**o+(0.07109068872)*x_ref[0]+(0.757923528294)*x_ref[1]**o+(0.903265965794)*x_ref[1]
      else:
        s[0]=(-0.119618139601)*x[0]**o+(0.674458706959)*x[0]+(0.283506026525)*x[1]**o+(0.867334648745)*x[1]+(0.793263586089)*x[2]**o+(-0.225225247588)*x[2]
        s[1]=(-0.458483133737)*x[0]**o+(0.636334230541)*x[0]+(0.433152852396)*x[1]**o+(0.946877061205)*x[1]+(-0.633441461314)*x[2]**o+(-0.00536782448735)*x[2]
        ref[0]=(-0.119618139601)*x_ref[0]**o+(0.674458706959)*x_ref[0]+(0.283506026525)*x_ref[1]**o+(0.867334648745)*x_ref[1]+(0.793263586089)*x_ref[2]**o+(-0.225225247588)*x_ref[2]
        ref[1]=(-0.458483133737)*x_ref[0]**o+(0.636334230541)*x_ref[0]+(0.433152852396)*x_ref[1]**o+(0.946877061205)*x_ref[1]+(-0.633441461314)*x_ref[2]**o+(-0.00536782448735)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_ReducedFunctionOnContactOne_rank2(self):
      """
      tests interpolate for rank 2 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactOne onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactOne(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5),dim=dim)
      s=Data(0,(4, 5),w)
      ref=Data(0,(4, 5),w_ref)
      if dim==2:
        s[0,0]=(-0.230877307866)*x[0]**o+(0.215021920364)*x[0]+(0.72160607128)*x[1]**o+(0.288782620524)*x[1]
        s[0,1]=(-0.397089347754)*x[0]**o+(-0.859308303321)*x[0]+(-0.655908162636)*x[1]**o+(-0.75086945631)*x[1]
        s[0,2]=(0.17849428647)*x[0]**o+(0.190815421543)*x[0]+(0.972002884101)*x[1]**o+(0.4108901607)*x[1]
        s[0,3]=(-0.288371936253)*x[0]**o+(-0.394846982668)*x[0]+(0.00326108093218)*x[1]**o+(0.218771269587)*x[1]
        s[0,4]=(0.629096144369)*x[0]**o+(0.820908159894)*x[0]+(0.691458623728)*x[1]**o+(0.214095362021)*x[1]
        s[1,0]=(0.709456114123)*x[0]**o+(-0.837635170883)*x[0]+(0.469741659937)*x[1]**o+(-0.225676816505)*x[1]
        s[1,1]=(-0.429024930015)*x[0]**o+(0.332522034373)*x[0]+(-0.14085265632)*x[1]**o+(0.739644399991)*x[1]
        s[1,2]=(-0.550855064281)*x[0]**o+(0.950491676704)*x[0]+(0.468631317567)*x[1]**o+(-0.78396330626)*x[1]
        s[1,3]=(0.0713337235427)*x[0]**o+(-0.371870909904)*x[0]+(-0.329491599348)*x[1]**o+(0.0765857177272)*x[1]
        s[1,4]=(-0.619922856297)*x[0]**o+(-0.230137321592)*x[0]+(0.883918096123)*x[1]**o+(-0.164797122993)*x[1]
        s[2,0]=(0.577972603997)*x[0]**o+(-0.535427767528)*x[0]+(0.105653927239)*x[1]**o+(-0.803115354047)*x[1]
        s[2,1]=(-0.941298066772)*x[0]**o+(0.633879774328)*x[0]+(-0.769467220886)*x[1]**o+(-0.830635312894)*x[1]
        s[2,2]=(-0.143658254898)*x[0]**o+(0.612045970085)*x[0]+(-0.234784912386)*x[1]**o+(-0.111427283432)*x[1]
        s[2,3]=(-0.0806971603544)*x[0]**o+(-0.478260278718)*x[0]+(0.405214636774)*x[1]**o+(-0.0171544079949)*x[1]
        s[2,4]=(-0.619407716965)*x[0]**o+(0.292456447643)*x[0]+(-0.404895807117)*x[1]**o+(0.955367226581)*x[1]
        s[3,0]=(-0.762723098154)*x[0]**o+(-0.547004242664)*x[0]+(-0.266917906271)*x[1]**o+(0.600697811809)*x[1]
        s[3,1]=(-0.567148999707)*x[0]**o+(0.374593601186)*x[0]+(-0.440814971395)*x[1]**o+(-0.737145147183)*x[1]
        s[3,2]=(0.630637925234)*x[0]**o+(-0.0884747935175)*x[0]+(-0.0319938816017)*x[1]**o+(0.865297163842)*x[1]
        s[3,3]=(-0.811769976054)*x[0]**o+(0.533258797244)*x[0]+(0.735904279517)*x[1]**o+(0.362604687369)*x[1]
        s[3,4]=(-0.711314210671)*x[0]**o+(-0.270182873191)*x[0]+(-0.965300380176)*x[1]**o+(0.50703939773)*x[1]
        ref[0,0]=(-0.230877307866)*x_ref[0]**o+(0.215021920364)*x_ref[0]+(0.72160607128)*x_ref[1]**o+(0.288782620524)*x_ref[1]
        ref[0,1]=(-0.397089347754)*x_ref[0]**o+(-0.859308303321)*x_ref[0]+(-0.655908162636)*x_ref[1]**o+(-0.75086945631)*x_ref[1]
        ref[0,2]=(0.17849428647)*x_ref[0]**o+(0.190815421543)*x_ref[0]+(0.972002884101)*x_ref[1]**o+(0.4108901607)*x_ref[1]
        ref[0,3]=(-0.288371936253)*x_ref[0]**o+(-0.394846982668)*x_ref[0]+(0.00326108093218)*x_ref[1]**o+(0.218771269587)*x_ref[1]
        ref[0,4]=(0.629096144369)*x_ref[0]**o+(0.820908159894)*x_ref[0]+(0.691458623728)*x_ref[1]**o+(0.214095362021)*x_ref[1]
        ref[1,0]=(0.709456114123)*x_ref[0]**o+(-0.837635170883)*x_ref[0]+(0.469741659937)*x_ref[1]**o+(-0.225676816505)*x_ref[1]
        ref[1,1]=(-0.429024930015)*x_ref[0]**o+(0.332522034373)*x_ref[0]+(-0.14085265632)*x_ref[1]**o+(0.739644399991)*x_ref[1]
        ref[1,2]=(-0.550855064281)*x_ref[0]**o+(0.950491676704)*x_ref[0]+(0.468631317567)*x_ref[1]**o+(-0.78396330626)*x_ref[1]
        ref[1,3]=(0.0713337235427)*x_ref[0]**o+(-0.371870909904)*x_ref[0]+(-0.329491599348)*x_ref[1]**o+(0.0765857177272)*x_ref[1]
        ref[1,4]=(-0.619922856297)*x_ref[0]**o+(-0.230137321592)*x_ref[0]+(0.883918096123)*x_ref[1]**o+(-0.164797122993)*x_ref[1]
        ref[2,0]=(0.577972603997)*x_ref[0]**o+(-0.535427767528)*x_ref[0]+(0.105653927239)*x_ref[1]**o+(-0.803115354047)*x_ref[1]
        ref[2,1]=(-0.941298066772)*x_ref[0]**o+(0.633879774328)*x_ref[0]+(-0.769467220886)*x_ref[1]**o+(-0.830635312894)*x_ref[1]
        ref[2,2]=(-0.143658254898)*x_ref[0]**o+(0.612045970085)*x_ref[0]+(-0.234784912386)*x_ref[1]**o+(-0.111427283432)*x_ref[1]
        ref[2,3]=(-0.0806971603544)*x_ref[0]**o+(-0.478260278718)*x_ref[0]+(0.405214636774)*x_ref[1]**o+(-0.0171544079949)*x_ref[1]
        ref[2,4]=(-0.619407716965)*x_ref[0]**o+(0.292456447643)*x_ref[0]+(-0.404895807117)*x_ref[1]**o+(0.955367226581)*x_ref[1]
        ref[3,0]=(-0.762723098154)*x_ref[0]**o+(-0.547004242664)*x_ref[0]+(-0.266917906271)*x_ref[1]**o+(0.600697811809)*x_ref[1]
        ref[3,1]=(-0.567148999707)*x_ref[0]**o+(0.374593601186)*x_ref[0]+(-0.440814971395)*x_ref[1]**o+(-0.737145147183)*x_ref[1]
        ref[3,2]=(0.630637925234)*x_ref[0]**o+(-0.0884747935175)*x_ref[0]+(-0.0319938816017)*x_ref[1]**o+(0.865297163842)*x_ref[1]
        ref[3,3]=(-0.811769976054)*x_ref[0]**o+(0.533258797244)*x_ref[0]+(0.735904279517)*x_ref[1]**o+(0.362604687369)*x_ref[1]
        ref[3,4]=(-0.711314210671)*x_ref[0]**o+(-0.270182873191)*x_ref[0]+(-0.965300380176)*x_ref[1]**o+(0.50703939773)*x_ref[1]
      else:
        s[0,0]=(0.817784424934)*x[0]**o+(0.140829586694)*x[0]+(0.395723095233)*x[1]**o+(-0.0916449245097)*x[1]+(-0.686149802471)*x[2]**o+(0.231879617817)*x[2]
        s[0,1]=(-0.753947196887)*x[0]**o+(0.0184422308138)*x[0]+(0.38127599678)*x[1]**o+(-0.958287760938)*x[1]+(-0.230273819981)*x[2]**o+(-0.922603290238)*x[2]
        s[0,2]=(0.44304320962)*x[0]**o+(0.35396422013)*x[0]+(-0.479605019935)*x[1]**o+(0.289724501875)*x[1]+(0.657449032305)*x[2]**o+(-0.706689994866)*x[2]
        s[0,3]=(0.0214930482601)*x[0]**o+(-0.915710233423)*x[0]+(-0.571112565179)*x[1]**o+(0.837855622741)*x[1]+(-0.402178902773)*x[2]**o+(-0.113536055723)*x[2]
        s[0,4]=(0.0332368088464)*x[0]**o+(0.488860583014)*x[0]+(0.687230732879)*x[1]**o+(0.417177289784)*x[1]+(-0.138583225765)*x[2]**o+(-0.288206976222)*x[2]
        s[1,0]=(-0.571959772672)*x[0]**o+(-0.806968233432)*x[0]+(-0.951330341336)*x[1]**o+(-0.725871940445)*x[1]+(0.473279498914)*x[2]**o+(0.693882912652)*x[2]
        s[1,1]=(-0.925147598095)*x[0]**o+(0.612875097386)*x[0]+(0.625107884558)*x[1]**o+(0.665316395744)*x[1]+(-0.012158847541)*x[2]**o+(0.114880783467)*x[2]
        s[1,2]=(-0.361803346001)*x[0]**o+(0.630266012707)*x[0]+(-0.698127168765)*x[1]**o+(-0.534017552333)*x[1]+(0.968004682982)*x[2]**o+(-0.607970906679)*x[2]
        s[1,3]=(-0.122803946559)*x[0]**o+(-0.506542460419)*x[0]+(-0.993759502512)*x[1]**o+(-0.335687805433)*x[1]+(0.143963376404)*x[2]**o+(-0.980235207299)*x[2]
        s[1,4]=(-0.998831548478)*x[0]**o+(0.985193839232)*x[0]+(-0.310482397827)*x[1]**o+(0.487407650791)*x[1]+(-0.974332702676)*x[2]**o+(-0.0921055039133)*x[2]
        s[2,0]=(-0.160666729317)*x[0]**o+(0.250220171076)*x[0]+(-0.868648917809)*x[1]**o+(-0.711318604851)*x[1]+(0.115301101729)*x[2]**o+(0.231696383247)*x[2]
        s[2,1]=(-0.775210051641)*x[0]**o+(0.586897738775)*x[0]+(0.449891876198)*x[1]**o+(-0.326417063196)*x[1]+(-0.693635538015)*x[2]**o+(-0.180803736532)*x[2]
        s[2,2]=(-0.208290352953)*x[0]**o+(-0.447870020368)*x[0]+(-0.518392174915)*x[1]**o+(-0.183558767418)*x[1]+(-0.496457835849)*x[2]**o+(-0.423284073396)*x[2]
        s[2,3]=(-0.994564109782)*x[0]**o+(-0.229677912537)*x[0]+(0.563307213686)*x[1]**o+(0.922013593154)*x[1]+(-0.371937088648)*x[2]**o+(-0.243995790101)*x[2]
        s[2,4]=(0.108680732756)*x[0]**o+(0.8978812368)*x[0]+(0.672348022342)*x[1]**o+(0.316624749349)*x[1]+(0.304319187223)*x[2]**o+(-0.604575730332)*x[2]
        s[3,0]=(0.0734716079145)*x[0]**o+(-0.627088377782)*x[0]+(0.620453085026)*x[1]**o+(0.331742115021)*x[1]+(-0.166118547603)*x[2]**o+(0.808314484323)*x[2]
        s[3,1]=(-0.0808492395011)*x[0]**o+(-0.0938687820054)*x[0]+(-0.888326387847)*x[1]**o+(-0.608268617177)*x[1]+(-0.316327102759)*x[2]**o+(-0.592763199764)*x[2]
        s[3,2]=(-0.255058846163)*x[0]**o+(0.103273543614)*x[0]+(-0.976707756405)*x[1]**o+(-0.723115640951)*x[1]+(-0.871816562933)*x[2]**o+(0.0248677291123)*x[2]
        s[3,3]=(-0.128009946516)*x[0]**o+(0.285430888656)*x[0]+(0.660763158647)*x[1]**o+(-0.96689913729)*x[1]+(-0.512425011889)*x[2]**o+(0.12926532516)*x[2]
        s[3,4]=(0.374659980563)*x[0]**o+(-0.13005773825)*x[0]+(-0.491643647934)*x[1]**o+(0.536781460391)*x[1]+(0.484891534824)*x[2]**o+(0.764854714727)*x[2]
        ref[0,0]=(0.817784424934)*x_ref[0]**o+(0.140829586694)*x_ref[0]+(0.395723095233)*x_ref[1]**o+(-0.0916449245097)*x_ref[1]+(-0.686149802471)*x_ref[2]**o+(0.231879617817)*x_ref[2]
        ref[0,1]=(-0.753947196887)*x_ref[0]**o+(0.0184422308138)*x_ref[0]+(0.38127599678)*x_ref[1]**o+(-0.958287760938)*x_ref[1]+(-0.230273819981)*x_ref[2]**o+(-0.922603290238)*x_ref[2]
        ref[0,2]=(0.44304320962)*x_ref[0]**o+(0.35396422013)*x_ref[0]+(-0.479605019935)*x_ref[1]**o+(0.289724501875)*x_ref[1]+(0.657449032305)*x_ref[2]**o+(-0.706689994866)*x_ref[2]
        ref[0,3]=(0.0214930482601)*x_ref[0]**o+(-0.915710233423)*x_ref[0]+(-0.571112565179)*x_ref[1]**o+(0.837855622741)*x_ref[1]+(-0.402178902773)*x_ref[2]**o+(-0.113536055723)*x_ref[2]
        ref[0,4]=(0.0332368088464)*x_ref[0]**o+(0.488860583014)*x_ref[0]+(0.687230732879)*x_ref[1]**o+(0.417177289784)*x_ref[1]+(-0.138583225765)*x_ref[2]**o+(-0.288206976222)*x_ref[2]
        ref[1,0]=(-0.571959772672)*x_ref[0]**o+(-0.806968233432)*x_ref[0]+(-0.951330341336)*x_ref[1]**o+(-0.725871940445)*x_ref[1]+(0.473279498914)*x_ref[2]**o+(0.693882912652)*x_ref[2]
        ref[1,1]=(-0.925147598095)*x_ref[0]**o+(0.612875097386)*x_ref[0]+(0.625107884558)*x_ref[1]**o+(0.665316395744)*x_ref[1]+(-0.012158847541)*x_ref[2]**o+(0.114880783467)*x_ref[2]
        ref[1,2]=(-0.361803346001)*x_ref[0]**o+(0.630266012707)*x_ref[0]+(-0.698127168765)*x_ref[1]**o+(-0.534017552333)*x_ref[1]+(0.968004682982)*x_ref[2]**o+(-0.607970906679)*x_ref[2]
        ref[1,3]=(-0.122803946559)*x_ref[0]**o+(-0.506542460419)*x_ref[0]+(-0.993759502512)*x_ref[1]**o+(-0.335687805433)*x_ref[1]+(0.143963376404)*x_ref[2]**o+(-0.980235207299)*x_ref[2]
        ref[1,4]=(-0.998831548478)*x_ref[0]**o+(0.985193839232)*x_ref[0]+(-0.310482397827)*x_ref[1]**o+(0.487407650791)*x_ref[1]+(-0.974332702676)*x_ref[2]**o+(-0.0921055039133)*x_ref[2]
        ref[2,0]=(-0.160666729317)*x_ref[0]**o+(0.250220171076)*x_ref[0]+(-0.868648917809)*x_ref[1]**o+(-0.711318604851)*x_ref[1]+(0.115301101729)*x_ref[2]**o+(0.231696383247)*x_ref[2]
        ref[2,1]=(-0.775210051641)*x_ref[0]**o+(0.586897738775)*x_ref[0]+(0.449891876198)*x_ref[1]**o+(-0.326417063196)*x_ref[1]+(-0.693635538015)*x_ref[2]**o+(-0.180803736532)*x_ref[2]
        ref[2,2]=(-0.208290352953)*x_ref[0]**o+(-0.447870020368)*x_ref[0]+(-0.518392174915)*x_ref[1]**o+(-0.183558767418)*x_ref[1]+(-0.496457835849)*x_ref[2]**o+(-0.423284073396)*x_ref[2]
        ref[2,3]=(-0.994564109782)*x_ref[0]**o+(-0.229677912537)*x_ref[0]+(0.563307213686)*x_ref[1]**o+(0.922013593154)*x_ref[1]+(-0.371937088648)*x_ref[2]**o+(-0.243995790101)*x_ref[2]
        ref[2,4]=(0.108680732756)*x_ref[0]**o+(0.8978812368)*x_ref[0]+(0.672348022342)*x_ref[1]**o+(0.316624749349)*x_ref[1]+(0.304319187223)*x_ref[2]**o+(-0.604575730332)*x_ref[2]
        ref[3,0]=(0.0734716079145)*x_ref[0]**o+(-0.627088377782)*x_ref[0]+(0.620453085026)*x_ref[1]**o+(0.331742115021)*x_ref[1]+(-0.166118547603)*x_ref[2]**o+(0.808314484323)*x_ref[2]
        ref[3,1]=(-0.0808492395011)*x_ref[0]**o+(-0.0938687820054)*x_ref[0]+(-0.888326387847)*x_ref[1]**o+(-0.608268617177)*x_ref[1]+(-0.316327102759)*x_ref[2]**o+(-0.592763199764)*x_ref[2]
        ref[3,2]=(-0.255058846163)*x_ref[0]**o+(0.103273543614)*x_ref[0]+(-0.976707756405)*x_ref[1]**o+(-0.723115640951)*x_ref[1]+(-0.871816562933)*x_ref[2]**o+(0.0248677291123)*x_ref[2]
        ref[3,3]=(-0.128009946516)*x_ref[0]**o+(0.285430888656)*x_ref[0]+(0.660763158647)*x_ref[1]**o+(-0.96689913729)*x_ref[1]+(-0.512425011889)*x_ref[2]**o+(0.12926532516)*x_ref[2]
        ref[3,4]=(0.374659980563)*x_ref[0]**o+(-0.13005773825)*x_ref[0]+(-0.491643647934)*x_ref[1]**o+(0.536781460391)*x_ref[1]+(0.484891534824)*x_ref[2]**o+(0.764854714727)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_ReducedFunctionOnContactOne_rank3(self):
      """
      tests interpolate for rank 3 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactOne onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactOne(self.domain)
      x=w.getX()
      arg=Symbol(shape=(6, 2, 2),dim=dim)
      s=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2),w_ref)
      if dim==2:
        s[0,0,0]=(-0.395079492423)*x[0]**o+(-0.627751628747)*x[0]+(-0.754320526654)*x[1]**o+(-0.498408852477)*x[1]
        s[0,0,1]=(0.498446911285)*x[0]**o+(0.0596046404343)*x[0]+(0.667944256533)*x[1]**o+(0.80046807163)*x[1]
        s[0,1,0]=(-0.0130333830081)*x[0]**o+(0.649913235604)*x[0]+(-0.374473401736)*x[1]**o+(-0.0186132358452)*x[1]
        s[0,1,1]=(-0.390035543793)*x[0]**o+(0.673194387531)*x[0]+(-0.867455391881)*x[1]**o+(-0.510668203571)*x[1]
        s[1,0,0]=(0.978997680408)*x[0]**o+(-0.343461179415)*x[0]+(0.582713911591)*x[1]**o+(-0.165697613343)*x[1]
        s[1,0,1]=(0.0923732379474)*x[0]**o+(-0.585219326869)*x[0]+(0.540073721296)*x[1]**o+(0.027063080312)*x[1]
        s[1,1,0]=(0.590126162972)*x[0]**o+(-0.136675336981)*x[0]+(-0.376003854007)*x[1]**o+(0.966535168787)*x[1]
        s[1,1,1]=(-0.493032436853)*x[0]**o+(0.239853630037)*x[0]+(-0.602522863181)*x[1]**o+(0.33785945991)*x[1]
        s[2,0,0]=(-0.529642904818)*x[0]**o+(-0.43628377125)*x[0]+(-0.996498099393)*x[1]**o+(-0.0591954433263)*x[1]
        s[2,0,1]=(-0.235182234411)*x[0]**o+(0.811387084032)*x[0]+(-0.443296230671)*x[1]**o+(-0.3314686181)*x[1]
        s[2,1,0]=(-0.110109607717)*x[0]**o+(-0.1409604355)*x[0]+(0.284844298465)*x[1]**o+(0.714735331234)*x[1]
        s[2,1,1]=(-0.936889720851)*x[0]**o+(-0.00858197667802)*x[0]+(0.680058166799)*x[1]**o+(0.436911645255)*x[1]
        s[3,0,0]=(-0.478297335601)*x[0]**o+(0.686114547145)*x[0]+(0.234700452186)*x[1]**o+(-0.330970013839)*x[1]
        s[3,0,1]=(0.975338382165)*x[0]**o+(0.607523981612)*x[0]+(-0.616521819955)*x[1]**o+(-0.292932591377)*x[1]
        s[3,1,0]=(0.987717426508)*x[0]**o+(0.6507515036)*x[0]+(-0.195762979545)*x[1]**o+(-0.667650116614)*x[1]
        s[3,1,1]=(0.53886053148)*x[0]**o+(-0.491668085193)*x[0]+(0.486615664224)*x[1]**o+(0.741560383409)*x[1]
        s[4,0,0]=(-0.936467387625)*x[0]**o+(-0.142421597802)*x[0]+(0.733595719048)*x[1]**o+(0.402826551348)*x[1]
        s[4,0,1]=(0.64501254965)*x[0]**o+(-0.660424630011)*x[0]+(-0.221052224259)*x[1]**o+(-0.199679968927)*x[1]
        s[4,1,0]=(0.809693682851)*x[0]**o+(-0.614404151383)*x[0]+(-0.754545121479)*x[1]**o+(-0.603231220423)*x[1]
        s[4,1,1]=(0.709936703222)*x[0]**o+(-0.389296803297)*x[0]+(-0.271161950977)*x[1]**o+(-0.929259781943)*x[1]
        s[5,0,0]=(-0.267950850602)*x[0]**o+(-0.660075303261)*x[0]+(0.0304564867312)*x[1]**o+(-0.42168319765)*x[1]
        s[5,0,1]=(0.137764595135)*x[0]**o+(0.265554426237)*x[0]+(-0.435216862466)*x[1]**o+(-0.338253702328)*x[1]
        s[5,1,0]=(0.858568660192)*x[0]**o+(-0.18371104351)*x[0]+(0.665662082165)*x[1]**o+(0.0684606399813)*x[1]
        s[5,1,1]=(0.897687912811)*x[0]**o+(0.675844284819)*x[0]+(0.0430574862378)*x[1]**o+(0.371728813053)*x[1]
        ref[0,0,0]=(-0.395079492423)*x_ref[0]**o+(-0.627751628747)*x_ref[0]+(-0.754320526654)*x_ref[1]**o+(-0.498408852477)*x_ref[1]
        ref[0,0,1]=(0.498446911285)*x_ref[0]**o+(0.0596046404343)*x_ref[0]+(0.667944256533)*x_ref[1]**o+(0.80046807163)*x_ref[1]
        ref[0,1,0]=(-0.0130333830081)*x_ref[0]**o+(0.649913235604)*x_ref[0]+(-0.374473401736)*x_ref[1]**o+(-0.0186132358452)*x_ref[1]
        ref[0,1,1]=(-0.390035543793)*x_ref[0]**o+(0.673194387531)*x_ref[0]+(-0.867455391881)*x_ref[1]**o+(-0.510668203571)*x_ref[1]
        ref[1,0,0]=(0.978997680408)*x_ref[0]**o+(-0.343461179415)*x_ref[0]+(0.582713911591)*x_ref[1]**o+(-0.165697613343)*x_ref[1]
        ref[1,0,1]=(0.0923732379474)*x_ref[0]**o+(-0.585219326869)*x_ref[0]+(0.540073721296)*x_ref[1]**o+(0.027063080312)*x_ref[1]
        ref[1,1,0]=(0.590126162972)*x_ref[0]**o+(-0.136675336981)*x_ref[0]+(-0.376003854007)*x_ref[1]**o+(0.966535168787)*x_ref[1]
        ref[1,1,1]=(-0.493032436853)*x_ref[0]**o+(0.239853630037)*x_ref[0]+(-0.602522863181)*x_ref[1]**o+(0.33785945991)*x_ref[1]
        ref[2,0,0]=(-0.529642904818)*x_ref[0]**o+(-0.43628377125)*x_ref[0]+(-0.996498099393)*x_ref[1]**o+(-0.0591954433263)*x_ref[1]
        ref[2,0,1]=(-0.235182234411)*x_ref[0]**o+(0.811387084032)*x_ref[0]+(-0.443296230671)*x_ref[1]**o+(-0.3314686181)*x_ref[1]
        ref[2,1,0]=(-0.110109607717)*x_ref[0]**o+(-0.1409604355)*x_ref[0]+(0.284844298465)*x_ref[1]**o+(0.714735331234)*x_ref[1]
        ref[2,1,1]=(-0.936889720851)*x_ref[0]**o+(-0.00858197667802)*x_ref[0]+(0.680058166799)*x_ref[1]**o+(0.436911645255)*x_ref[1]
        ref[3,0,0]=(-0.478297335601)*x_ref[0]**o+(0.686114547145)*x_ref[0]+(0.234700452186)*x_ref[1]**o+(-0.330970013839)*x_ref[1]
        ref[3,0,1]=(0.975338382165)*x_ref[0]**o+(0.607523981612)*x_ref[0]+(-0.616521819955)*x_ref[1]**o+(-0.292932591377)*x_ref[1]
        ref[3,1,0]=(0.987717426508)*x_ref[0]**o+(0.6507515036)*x_ref[0]+(-0.195762979545)*x_ref[1]**o+(-0.667650116614)*x_ref[1]
        ref[3,1,1]=(0.53886053148)*x_ref[0]**o+(-0.491668085193)*x_ref[0]+(0.486615664224)*x_ref[1]**o+(0.741560383409)*x_ref[1]
        ref[4,0,0]=(-0.936467387625)*x_ref[0]**o+(-0.142421597802)*x_ref[0]+(0.733595719048)*x_ref[1]**o+(0.402826551348)*x_ref[1]
        ref[4,0,1]=(0.64501254965)*x_ref[0]**o+(-0.660424630011)*x_ref[0]+(-0.221052224259)*x_ref[1]**o+(-0.199679968927)*x_ref[1]
        ref[4,1,0]=(0.809693682851)*x_ref[0]**o+(-0.614404151383)*x_ref[0]+(-0.754545121479)*x_ref[1]**o+(-0.603231220423)*x_ref[1]
        ref[4,1,1]=(0.709936703222)*x_ref[0]**o+(-0.389296803297)*x_ref[0]+(-0.271161950977)*x_ref[1]**o+(-0.929259781943)*x_ref[1]
        ref[5,0,0]=(-0.267950850602)*x_ref[0]**o+(-0.660075303261)*x_ref[0]+(0.0304564867312)*x_ref[1]**o+(-0.42168319765)*x_ref[1]
        ref[5,0,1]=(0.137764595135)*x_ref[0]**o+(0.265554426237)*x_ref[0]+(-0.435216862466)*x_ref[1]**o+(-0.338253702328)*x_ref[1]
        ref[5,1,0]=(0.858568660192)*x_ref[0]**o+(-0.18371104351)*x_ref[0]+(0.665662082165)*x_ref[1]**o+(0.0684606399813)*x_ref[1]
        ref[5,1,1]=(0.897687912811)*x_ref[0]**o+(0.675844284819)*x_ref[0]+(0.0430574862378)*x_ref[1]**o+(0.371728813053)*x_ref[1]
      else:
        s[0,0,0]=(-0.589511962709)*x[0]**o+(0.159792477428)*x[0]+(0.766030256721)*x[1]**o+(-0.312062526449)*x[1]+(0.64700443985)*x[2]**o+(0.0518757184544)*x[2]
        s[0,0,1]=(-0.894062305961)*x[0]**o+(0.864404945522)*x[0]+(0.560280553158)*x[1]**o+(0.918283462394)*x[1]+(-0.490840733192)*x[2]**o+(0.697691943154)*x[2]
        s[0,1,0]=(-0.0882043414483)*x[0]**o+(-0.096537953899)*x[0]+(0.927057867899)*x[1]**o+(0.77613572949)*x[1]+(-0.987702345599)*x[2]**o+(-0.574645805199)*x[2]
        s[0,1,1]=(-0.66012117034)*x[0]**o+(-0.189179227113)*x[0]+(0.292721630825)*x[1]**o+(0.0170456979372)*x[1]+(0.0183109554333)*x[2]**o+(-0.917454875678)*x[2]
        s[1,0,0]=(0.654605700477)*x[0]**o+(-0.170526279436)*x[0]+(0.343326885913)*x[1]**o+(0.210563849271)*x[1]+(0.534291586724)*x[2]**o+(-0.755048841391)*x[2]
        s[1,0,1]=(-0.364686989491)*x[0]**o+(0.552984183344)*x[0]+(-0.470357667845)*x[1]**o+(-0.464300631313)*x[1]+(-0.438895038261)*x[2]**o+(0.518824978532)*x[2]
        s[1,1,0]=(-0.0456454803782)*x[0]**o+(-0.696772845921)*x[0]+(-0.587520555693)*x[1]**o+(0.676523672069)*x[1]+(-0.905996237596)*x[2]**o+(-0.474417867255)*x[2]
        s[1,1,1]=(0.360939068763)*x[0]**o+(-0.743105635563)*x[0]+(-0.615952199646)*x[1]**o+(-0.538063201399)*x[1]+(-0.898283738708)*x[2]**o+(-0.97634117952)*x[2]
        s[2,0,0]=(-0.185178359078)*x[0]**o+(0.224443281068)*x[0]+(0.754102418405)*x[1]**o+(-0.411697272338)*x[1]+(-0.528900607926)*x[2]**o+(-0.807089685705)*x[2]
        s[2,0,1]=(-0.784480844715)*x[0]**o+(-0.823404986088)*x[0]+(0.749752559055)*x[1]**o+(-0.993803702434)*x[1]+(-0.03194828585)*x[2]**o+(-0.104230991195)*x[2]
        s[2,1,0]=(-0.3975495117)*x[0]**o+(-0.921611510193)*x[0]+(0.364776109304)*x[1]**o+(0.568483551949)*x[1]+(0.92620487977)*x[2]**o+(-0.203075275658)*x[2]
        s[2,1,1]=(0.483830827389)*x[0]**o+(-0.16426527248)*x[0]+(0.829148557161)*x[1]**o+(-0.476785306719)*x[1]+(0.498338738131)*x[2]**o+(-0.335483518227)*x[2]
        s[3,0,0]=(-0.673980780716)*x[0]**o+(0.565441119757)*x[0]+(-0.440135530887)*x[1]**o+(-0.505046260809)*x[1]+(-0.37380888836)*x[2]**o+(0.405503742087)*x[2]
        s[3,0,1]=(-0.330871586339)*x[0]**o+(-0.590160971509)*x[0]+(-0.737041178139)*x[1]**o+(0.998222421798)*x[1]+(0.910277176468)*x[2]**o+(0.429491322427)*x[2]
        s[3,1,0]=(-0.166111866877)*x[0]**o+(-0.758927317121)*x[0]+(0.287263609892)*x[1]**o+(-0.322563818793)*x[1]+(-0.430564368296)*x[2]**o+(0.174322344232)*x[2]
        s[3,1,1]=(-0.661309380996)*x[0]**o+(0.139785030015)*x[0]+(-0.682140312514)*x[1]**o+(-0.654285952086)*x[1]+(0.622815328424)*x[2]**o+(0.485021479372)*x[2]
        s[4,0,0]=(-0.594196524006)*x[0]**o+(-0.539707458079)*x[0]+(-0.793206045899)*x[1]**o+(-0.65733164652)*x[1]+(0.853898902594)*x[2]**o+(-0.68578755422)*x[2]
        s[4,0,1]=(-0.368655085475)*x[0]**o+(0.965754821996)*x[0]+(0.796746162388)*x[1]**o+(-0.0792750556783)*x[1]+(0.865822523139)*x[2]**o+(0.824557599127)*x[2]
        s[4,1,0]=(0.0309342911768)*x[0]**o+(-0.57039195842)*x[0]+(0.314105965219)*x[1]**o+(0.991995648934)*x[1]+(-0.344915004359)*x[2]**o+(0.461898675894)*x[2]
        s[4,1,1]=(0.287549691013)*x[0]**o+(0.342272217628)*x[0]+(-0.469131330709)*x[1]**o+(0.505146423329)*x[1]+(0.347751545765)*x[2]**o+(0.547278978185)*x[2]
        s[5,0,0]=(-0.884852695364)*x[0]**o+(-0.642157963952)*x[0]+(-0.787121736521)*x[1]**o+(0.187833667108)*x[1]+(-0.339842450451)*x[2]**o+(0.174012710957)*x[2]
        s[5,0,1]=(0.28878191697)*x[0]**o+(-0.360576326583)*x[0]+(-0.0945722807072)*x[1]**o+(-0.587351566921)*x[1]+(-0.933568527838)*x[2]**o+(-0.60716101577)*x[2]
        s[5,1,0]=(-0.581405265173)*x[0]**o+(-0.015810740193)*x[0]+(0.919122906998)*x[1]**o+(-0.676080492473)*x[1]+(0.0255111996852)*x[2]**o+(-0.693486219028)*x[2]
        s[5,1,1]=(-0.941020084511)*x[0]**o+(-0.330527617459)*x[0]+(0.98542398649)*x[1]**o+(-0.154193758572)*x[1]+(0.876454677452)*x[2]**o+(-0.278575401803)*x[2]
        ref[0,0,0]=(-0.589511962709)*x_ref[0]**o+(0.159792477428)*x_ref[0]+(0.766030256721)*x_ref[1]**o+(-0.312062526449)*x_ref[1]+(0.64700443985)*x_ref[2]**o+(0.0518757184544)*x_ref[2]
        ref[0,0,1]=(-0.894062305961)*x_ref[0]**o+(0.864404945522)*x_ref[0]+(0.560280553158)*x_ref[1]**o+(0.918283462394)*x_ref[1]+(-0.490840733192)*x_ref[2]**o+(0.697691943154)*x_ref[2]
        ref[0,1,0]=(-0.0882043414483)*x_ref[0]**o+(-0.096537953899)*x_ref[0]+(0.927057867899)*x_ref[1]**o+(0.77613572949)*x_ref[1]+(-0.987702345599)*x_ref[2]**o+(-0.574645805199)*x_ref[2]
        ref[0,1,1]=(-0.66012117034)*x_ref[0]**o+(-0.189179227113)*x_ref[0]+(0.292721630825)*x_ref[1]**o+(0.0170456979372)*x_ref[1]+(0.0183109554333)*x_ref[2]**o+(-0.917454875678)*x_ref[2]
        ref[1,0,0]=(0.654605700477)*x_ref[0]**o+(-0.170526279436)*x_ref[0]+(0.343326885913)*x_ref[1]**o+(0.210563849271)*x_ref[1]+(0.534291586724)*x_ref[2]**o+(-0.755048841391)*x_ref[2]
        ref[1,0,1]=(-0.364686989491)*x_ref[0]**o+(0.552984183344)*x_ref[0]+(-0.470357667845)*x_ref[1]**o+(-0.464300631313)*x_ref[1]+(-0.438895038261)*x_ref[2]**o+(0.518824978532)*x_ref[2]
        ref[1,1,0]=(-0.0456454803782)*x_ref[0]**o+(-0.696772845921)*x_ref[0]+(-0.587520555693)*x_ref[1]**o+(0.676523672069)*x_ref[1]+(-0.905996237596)*x_ref[2]**o+(-0.474417867255)*x_ref[2]
        ref[1,1,1]=(0.360939068763)*x_ref[0]**o+(-0.743105635563)*x_ref[0]+(-0.615952199646)*x_ref[1]**o+(-0.538063201399)*x_ref[1]+(-0.898283738708)*x_ref[2]**o+(-0.97634117952)*x_ref[2]
        ref[2,0,0]=(-0.185178359078)*x_ref[0]**o+(0.224443281068)*x_ref[0]+(0.754102418405)*x_ref[1]**o+(-0.411697272338)*x_ref[1]+(-0.528900607926)*x_ref[2]**o+(-0.807089685705)*x_ref[2]
        ref[2,0,1]=(-0.784480844715)*x_ref[0]**o+(-0.823404986088)*x_ref[0]+(0.749752559055)*x_ref[1]**o+(-0.993803702434)*x_ref[1]+(-0.03194828585)*x_ref[2]**o+(-0.104230991195)*x_ref[2]
        ref[2,1,0]=(-0.3975495117)*x_ref[0]**o+(-0.921611510193)*x_ref[0]+(0.364776109304)*x_ref[1]**o+(0.568483551949)*x_ref[1]+(0.92620487977)*x_ref[2]**o+(-0.203075275658)*x_ref[2]
        ref[2,1,1]=(0.483830827389)*x_ref[0]**o+(-0.16426527248)*x_ref[0]+(0.829148557161)*x_ref[1]**o+(-0.476785306719)*x_ref[1]+(0.498338738131)*x_ref[2]**o+(-0.335483518227)*x_ref[2]
        ref[3,0,0]=(-0.673980780716)*x_ref[0]**o+(0.565441119757)*x_ref[0]+(-0.440135530887)*x_ref[1]**o+(-0.505046260809)*x_ref[1]+(-0.37380888836)*x_ref[2]**o+(0.405503742087)*x_ref[2]
        ref[3,0,1]=(-0.330871586339)*x_ref[0]**o+(-0.590160971509)*x_ref[0]+(-0.737041178139)*x_ref[1]**o+(0.998222421798)*x_ref[1]+(0.910277176468)*x_ref[2]**o+(0.429491322427)*x_ref[2]
        ref[3,1,0]=(-0.166111866877)*x_ref[0]**o+(-0.758927317121)*x_ref[0]+(0.287263609892)*x_ref[1]**o+(-0.322563818793)*x_ref[1]+(-0.430564368296)*x_ref[2]**o+(0.174322344232)*x_ref[2]
        ref[3,1,1]=(-0.661309380996)*x_ref[0]**o+(0.139785030015)*x_ref[0]+(-0.682140312514)*x_ref[1]**o+(-0.654285952086)*x_ref[1]+(0.622815328424)*x_ref[2]**o+(0.485021479372)*x_ref[2]
        ref[4,0,0]=(-0.594196524006)*x_ref[0]**o+(-0.539707458079)*x_ref[0]+(-0.793206045899)*x_ref[1]**o+(-0.65733164652)*x_ref[1]+(0.853898902594)*x_ref[2]**o+(-0.68578755422)*x_ref[2]
        ref[4,0,1]=(-0.368655085475)*x_ref[0]**o+(0.965754821996)*x_ref[0]+(0.796746162388)*x_ref[1]**o+(-0.0792750556783)*x_ref[1]+(0.865822523139)*x_ref[2]**o+(0.824557599127)*x_ref[2]
        ref[4,1,0]=(0.0309342911768)*x_ref[0]**o+(-0.57039195842)*x_ref[0]+(0.314105965219)*x_ref[1]**o+(0.991995648934)*x_ref[1]+(-0.344915004359)*x_ref[2]**o+(0.461898675894)*x_ref[2]
        ref[4,1,1]=(0.287549691013)*x_ref[0]**o+(0.342272217628)*x_ref[0]+(-0.469131330709)*x_ref[1]**o+(0.505146423329)*x_ref[1]+(0.347751545765)*x_ref[2]**o+(0.547278978185)*x_ref[2]
        ref[5,0,0]=(-0.884852695364)*x_ref[0]**o+(-0.642157963952)*x_ref[0]+(-0.787121736521)*x_ref[1]**o+(0.187833667108)*x_ref[1]+(-0.339842450451)*x_ref[2]**o+(0.174012710957)*x_ref[2]
        ref[5,0,1]=(0.28878191697)*x_ref[0]**o+(-0.360576326583)*x_ref[0]+(-0.0945722807072)*x_ref[1]**o+(-0.587351566921)*x_ref[1]+(-0.933568527838)*x_ref[2]**o+(-0.60716101577)*x_ref[2]
        ref[5,1,0]=(-0.581405265173)*x_ref[0]**o+(-0.015810740193)*x_ref[0]+(0.919122906998)*x_ref[1]**o+(-0.676080492473)*x_ref[1]+(0.0255111996852)*x_ref[2]**o+(-0.693486219028)*x_ref[2]
        ref[5,1,1]=(-0.941020084511)*x_ref[0]**o+(-0.330527617459)*x_ref[0]+(0.98542398649)*x_ref[1]**o+(-0.154193758572)*x_ref[1]+(0.876454677452)*x_ref[2]**o+(-0.278575401803)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactZero_fromSymbol_ReducedFunctionOnContactOne_rank4(self):
      """
      tests interpolate for rank 4 Symbol onto the ReducedFunctionOnContactZero

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactOne onto ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactOne(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5, 3, 2),dim=dim)
      s=Data(0,(4, 5, 3, 2),w)
      ref=Data(0,(4, 5, 3, 2),w_ref)
      if dim==2:
        s[0,0,0,0]=(-0.939524201884)*x[0]**o+(-0.294114596857)*x[0]+(-0.662571514204)*x[1]**o+(-0.295055230761)*x[1]
        s[0,0,0,1]=(0.174447802399)*x[0]**o+(0.313121933505)*x[0]+(0.780423776572)*x[1]**o+(0.260329316622)*x[1]
        s[0,0,1,0]=(-0.307907192496)*x[0]**o+(0.924386500426)*x[0]+(-0.34849235247)*x[1]**o+(0.42450311854)*x[1]
        s[0,0,1,1]=(-0.221818525078)*x[0]**o+(0.334658153768)*x[0]+(0.730236845709)*x[1]**o+(0.923645750095)*x[1]
        s[0,0,2,0]=(-0.0573462957426)*x[0]**o+(0.10400906253)*x[0]+(-0.606757296289)*x[1]**o+(-0.171453259849)*x[1]
        s[0,0,2,1]=(-0.653190260251)*x[0]**o+(0.392209523496)*x[0]+(0.984792984469)*x[1]**o+(-0.310847119118)*x[1]
        s[0,1,0,0]=(-0.0980551705202)*x[0]**o+(0.475337517365)*x[0]+(-0.93300381011)*x[1]**o+(0.0593095200419)*x[1]
        s[0,1,0,1]=(0.833302217025)*x[0]**o+(-0.866194985925)*x[0]+(0.337967734176)*x[1]**o+(0.0581478569311)*x[1]
        s[0,1,1,0]=(0.936751875211)*x[0]**o+(-0.217226318173)*x[0]+(0.477011461247)*x[1]**o+(0.221436266198)*x[1]
        s[0,1,1,1]=(-0.537625551791)*x[0]**o+(0.718351842961)*x[0]+(0.0286304587001)*x[1]**o+(-0.721442785761)*x[1]
        s[0,1,2,0]=(-0.738645216931)*x[0]**o+(0.0195980002286)*x[0]+(0.827363980625)*x[1]**o+(-0.743955943985)*x[1]
        s[0,1,2,1]=(-0.145033137282)*x[0]**o+(0.98069402199)*x[0]+(-0.167153611864)*x[1]**o+(-0.577697516598)*x[1]
        s[0,2,0,0]=(0.594926068713)*x[0]**o+(-0.124581510742)*x[0]+(-0.392512448989)*x[1]**o+(-0.989702605289)*x[1]
        s[0,2,0,1]=(-0.208680726694)*x[0]**o+(0.0226857988067)*x[0]+(0.0127574486126)*x[1]**o+(-0.661169282937)*x[1]
        s[0,2,1,0]=(0.43109296375)*x[0]**o+(-0.966243011304)*x[0]+(-0.448863362387)*x[1]**o+(-0.531981406137)*x[1]
        s[0,2,1,1]=(0.427441889618)*x[0]**o+(-0.730863645031)*x[0]+(0.0948615738498)*x[1]**o+(0.349409965532)*x[1]
        s[0,2,2,0]=(0.0857935500808)*x[0]**o+(0.916612852535)*x[0]+(-0.5434740164)*x[1]**o+(0.651670987189)*x[1]
        s[0,2,2,1]=(0.269775748844)*x[0]**o+(-0.351727930666)*x[0]+(0.648234694172)*x[1]**o+(0.714331370173)*x[1]
        s[0,3,0,0]=(0.862594479045)*x[0]**o+(-0.787345711339)*x[0]+(0.1934849213)*x[1]**o+(-0.977224028721)*x[1]
        s[0,3,0,1]=(0.166215962959)*x[0]**o+(0.734752140512)*x[0]+(0.373558020776)*x[1]**o+(-0.589819927874)*x[1]
        s[0,3,1,0]=(0.0289724459782)*x[0]**o+(0.517327391578)*x[0]+(0.109956624336)*x[1]**o+(-0.290855234976)*x[1]
        s[0,3,1,1]=(0.369852037916)*x[0]**o+(0.184556562048)*x[0]+(-0.694630650679)*x[1]**o+(-0.526020870229)*x[1]
        s[0,3,2,0]=(0.397498255324)*x[0]**o+(-0.375327133358)*x[0]+(0.564571187906)*x[1]**o+(-0.295820211045)*x[1]
        s[0,3,2,1]=(0.550532413074)*x[0]**o+(-0.751307255234)*x[0]+(0.323446174542)*x[1]**o+(0.968475158376)*x[1]
        s[0,4,0,0]=(-0.112506956442)*x[0]**o+(-0.432032518771)*x[0]+(0.0205217950343)*x[1]**o+(-0.507714722498)*x[1]
        s[0,4,0,1]=(-0.501932335824)*x[0]**o+(-0.147371573026)*x[0]+(-0.421130372732)*x[1]**o+(-0.958029815205)*x[1]
        s[0,4,1,0]=(0.172226819831)*x[0]**o+(-0.712313983987)*x[0]+(0.859887204781)*x[1]**o+(0.0661613029313)*x[1]
        s[0,4,1,1]=(-0.863367396226)*x[0]**o+(0.150655310131)*x[0]+(-0.446461969683)*x[1]**o+(0.247377395525)*x[1]
        s[0,4,2,0]=(-0.0431980822723)*x[0]**o+(0.676680822195)*x[0]+(-0.613277443463)*x[1]**o+(0.787356918727)*x[1]
        s[0,4,2,1]=(-0.0569719722636)*x[0]**o+(0.641187476483)*x[0]+(-0.303617110099)*x[1]**o+(-0.888820557309)*x[1]
        s[1,0,0,0]=(-0.0552307803637)*x[0]**o+(0.306010657529)*x[0]+(0.488223930134)*x[1]**o+(-0.682392555784)*x[1]
        s[1,0,0,1]=(-0.253901726526)*x[0]**o+(0.73343629071)*x[0]+(-0.390491326134)*x[1]**o+(0.774605582697)*x[1]
        s[1,0,1,0]=(-0.42967954588)*x[0]**o+(-0.751284399784)*x[0]+(0.813738133871)*x[1]**o+(-0.0995163058125)*x[1]
        s[1,0,1,1]=(-0.0880959500234)*x[0]**o+(-0.781527901178)*x[0]+(-0.496899067642)*x[1]**o+(0.55524107606)*x[1]
        s[1,0,2,0]=(0.256546278479)*x[0]**o+(-0.595310894415)*x[0]+(-0.281823345122)*x[1]**o+(-0.953457305769)*x[1]
        s[1,0,2,1]=(0.365266273739)*x[0]**o+(-0.935192596652)*x[0]+(0.740921701447)*x[1]**o+(0.843620235195)*x[1]
        s[1,1,0,0]=(-0.992209271955)*x[0]**o+(0.267499041637)*x[0]+(-0.446633252289)*x[1]**o+(0.357054618942)*x[1]
        s[1,1,0,1]=(0.863355358433)*x[0]**o+(-0.104881968217)*x[0]+(0.0776056200756)*x[1]**o+(0.485205982195)*x[1]
        s[1,1,1,0]=(0.29822878169)*x[0]**o+(0.766559198241)*x[0]+(0.489839749671)*x[1]**o+(-0.694316951865)*x[1]
        s[1,1,1,1]=(-0.947135250126)*x[0]**o+(-0.952537408063)*x[0]+(-0.840569482634)*x[1]**o+(-0.0881389491948)*x[1]
        s[1,1,2,0]=(0.269982185854)*x[0]**o+(0.290850786316)*x[0]+(0.707550793741)*x[1]**o+(-0.583787562553)*x[1]
        s[1,1,2,1]=(-0.998478389198)*x[0]**o+(-0.902713626135)*x[0]+(-0.0809583498397)*x[1]**o+(0.242726761269)*x[1]
        s[1,2,0,0]=(-0.0752124155425)*x[0]**o+(0.287338426631)*x[0]+(-0.873281064449)*x[1]**o+(0.86906507097)*x[1]
        s[1,2,0,1]=(-0.451012489095)*x[0]**o+(0.692877058818)*x[0]+(-0.771278511209)*x[1]**o+(0.325983000966)*x[1]
        s[1,2,1,0]=(-0.633239034154)*x[0]**o+(0.751673044983)*x[0]+(-0.799706248119)*x[1]**o+(0.0783814978548)*x[1]
        s[1,2,1,1]=(-0.496449080562)*x[0]**o+(-0.382869354097)*x[0]+(-0.658792080006)*x[1]**o+(0.578499479656)*x[1]
        s[1,2,2,0]=(0.41041079346)*x[0]**o+(0.552454636943)*x[0]+(0.236050520221)*x[1]**o+(0.477587390293)*x[1]
        s[1,2,2,1]=(0.649795041558)*x[0]**o+(-0.947868231928)*x[0]+(-0.529895928082)*x[1]**o+(0.715695540026)*x[1]
        s[1,3,0,0]=(0.948112901731)*x[0]**o+(-0.374624145588)*x[0]+(0.806640283077)*x[1]**o+(0.729074938843)*x[1]
        s[1,3,0,1]=(0.266956301954)*x[0]**o+(-0.878004354786)*x[0]+(-0.422298760963)*x[1]**o+(-0.251922913127)*x[1]
        s[1,3,1,0]=(0.751159198422)*x[0]**o+(-0.576983418224)*x[0]+(0.427653287761)*x[1]**o+(0.351116956206)*x[1]
        s[1,3,1,1]=(-0.804702781998)*x[0]**o+(-0.218650676384)*x[0]+(-0.832220670207)*x[1]**o+(0.997469993794)*x[1]
        s[1,3,2,0]=(0.111110638087)*x[0]**o+(-0.435831664904)*x[0]+(-0.144481003079)*x[1]**o+(-0.604349850119)*x[1]
        s[1,3,2,1]=(0.470311907248)*x[0]**o+(-0.591709930358)*x[0]+(-0.562662398681)*x[1]**o+(0.0163307873851)*x[1]
        s[1,4,0,0]=(0.684349728229)*x[0]**o+(0.404890565831)*x[0]+(0.922438828147)*x[1]**o+(0.939539648586)*x[1]
        s[1,4,0,1]=(-0.179161968364)*x[0]**o+(0.983188684135)*x[0]+(-0.722578271046)*x[1]**o+(-0.450472366251)*x[1]
        s[1,4,1,0]=(0.243718494318)*x[0]**o+(-0.0251174594804)*x[0]+(0.952890549291)*x[1]**o+(-0.398408844924)*x[1]
        s[1,4,1,1]=(-0.883090916963)*x[0]**o+(-0.236073018867)*x[0]+(-0.245092152479)*x[1]**o+(0.846958498274)*x[1]
        s[1,4,2,0]=(-0.839928016329)*x[0]**o+(-0.677734468188)*x[0]+(-0.123639765438)*x[1]**o+(-0.680672570927)*x[1]
        s[1,4,2,1]=(-0.234629335025)*x[0]**o+(-0.567381745605)*x[0]+(0.623480113756)*x[1]**o+(0.2049427452)*x[1]
        s[2,0,0,0]=(-0.297265663727)*x[0]**o+(0.00899195467866)*x[0]+(0.992475896282)*x[1]**o+(0.863451618621)*x[1]
        s[2,0,0,1]=(-0.24914517208)*x[0]**o+(0.104091943078)*x[0]+(0.762436787588)*x[1]**o+(0.89363657955)*x[1]
        s[2,0,1,0]=(0.264734056412)*x[0]**o+(-0.716868164934)*x[0]+(-0.905285068877)*x[1]**o+(0.0227117696273)*x[1]
        s[2,0,1,1]=(-0.932617617299)*x[0]**o+(0.271634450254)*x[0]+(0.51812940988)*x[1]**o+(-0.484525267705)*x[1]
        s[2,0,2,0]=(0.364599957461)*x[0]**o+(0.818588026862)*x[0]+(0.70366241863)*x[1]**o+(0.43910943526)*x[1]
        s[2,0,2,1]=(0.557891341305)*x[0]**o+(-0.891810635437)*x[0]+(0.649138154483)*x[1]**o+(-0.213951745119)*x[1]
        s[2,1,0,0]=(0.994667066111)*x[0]**o+(0.433110731778)*x[0]+(0.787983438535)*x[1]**o+(0.979717998129)*x[1]
        s[2,1,0,1]=(0.619284628164)*x[0]**o+(0.662998143683)*x[0]+(0.0328849267442)*x[1]**o+(0.765881868994)*x[1]
        s[2,1,1,0]=(0.335939940328)*x[0]**o+(0.556758825457)*x[0]+(-0.524870937642)*x[1]**o+(0.0316796705422)*x[1]
        s[2,1,1,1]=(-0.702249603437)*x[0]**o+(0.544642480464)*x[0]+(0.330520285989)*x[1]**o+(0.313003559197)*x[1]
        s[2,1,2,0]=(0.0294247925115)*x[0]**o+(0.10067678181)*x[0]+(0.608148681239)*x[1]**o+(-0.95697997936)*x[1]
        s[2,1,2,1]=(0.101913506022)*x[0]**o+(-0.423061068258)*x[0]+(0.303518988539)*x[1]**o+(0.0108182161414)*x[1]
        s[2,2,0,0]=(0.390642040424)*x[0]**o+(0.0578342521895)*x[0]+(0.228499563768)*x[1]**o+(-0.944433241107)*x[1]
        s[2,2,0,1]=(0.522633870922)*x[0]**o+(0.724655140578)*x[0]+(-0.664760217849)*x[1]**o+(0.812535008015)*x[1]
        s[2,2,1,0]=(0.27608720491)*x[0]**o+(0.42350834695)*x[0]+(-0.732116287856)*x[1]**o+(0.930674651671)*x[1]
        s[2,2,1,1]=(0.214308063462)*x[0]**o+(0.815707217077)*x[0]+(0.231785784747)*x[1]**o+(-0.954429150944)*x[1]
        s[2,2,2,0]=(0.402814098347)*x[0]**o+(-0.650491293555)*x[0]+(-0.359159927669)*x[1]**o+(0.503842188774)*x[1]
        s[2,2,2,1]=(-0.191195415522)*x[0]**o+(0.980791691347)*x[0]+(0.228597878156)*x[1]**o+(0.535975172993)*x[1]
        s[2,3,0,0]=(0.495749327231)*x[0]**o+(0.311034011895)*x[0]+(-0.532686805928)*x[1]**o+(0.92438969176)*x[1]
        s[2,3,0,1]=(-0.38513599341)*x[0]**o+(-0.463099237819)*x[0]+(-0.992606328662)*x[1]**o+(-0.927920439344)*x[1]
        s[2,3,1,0]=(-0.868022271483)*x[0]**o+(-0.540096192748)*x[0]+(-0.0879858139685)*x[1]**o+(0.534604810517)*x[1]
        s[2,3,1,1]=(-0.686837754683)*x[0]**o+(-0.505585075528)*x[0]+(-0.218155668559)*x[1]**o+(-0.241972887878)*x[1]
        s[2,3,2,0]=(0.86187325655)*x[0]**o+(-0.733958307606)*x[0]+(-0.107393817742)*x[1]**o+(-0.830306745614)*x[1]
        s[2,3,2,1]=(0.404834271017)*x[0]**o+(0.44646270613)*x[0]+(-0.657038236282)*x[1]**o+(-0.540570795414)*x[1]
        s[2,4,0,0]=(-0.216828677229)*x[0]**o+(0.129702603598)*x[0]+(0.556920803376)*x[1]**o+(0.960021941421)*x[1]
        s[2,4,0,1]=(-0.21625137654)*x[0]**o+(-0.19604294366)*x[0]+(-0.889082170851)*x[1]**o+(-0.834967241538)*x[1]
        s[2,4,1,0]=(0.123104286845)*x[0]**o+(-0.43197703752)*x[0]+(0.737850827826)*x[1]**o+(0.344689639656)*x[1]
        s[2,4,1,1]=(0.119471348292)*x[0]**o+(0.948353928609)*x[0]+(-0.726863566036)*x[1]**o+(-0.957158639686)*x[1]
        s[2,4,2,0]=(0.606624368242)*x[0]**o+(-0.129636738653)*x[0]+(0.117469089776)*x[1]**o+(0.766024628396)*x[1]
        s[2,4,2,1]=(-0.83631907859)*x[0]**o+(-0.168433036335)*x[0]+(-0.500663355802)*x[1]**o+(0.0114866397792)*x[1]
        s[3,0,0,0]=(-0.629399168531)*x[0]**o+(-0.464524331386)*x[0]+(0.0406391749081)*x[1]**o+(-0.380881295086)*x[1]
        s[3,0,0,1]=(-0.614191170121)*x[0]**o+(0.90297284642)*x[0]+(0.949653922227)*x[1]**o+(0.947014467705)*x[1]
        s[3,0,1,0]=(0.128075669906)*x[0]**o+(-0.215027529464)*x[0]+(-0.230820603567)*x[1]**o+(0.656672788071)*x[1]
        s[3,0,1,1]=(0.11006414768)*x[0]**o+(0.575548749986)*x[0]+(0.803886215346)*x[1]**o+(0.627503866105)*x[1]
        s[3,0,2,0]=(-0.824524708364)*x[0]**o+(-0.399533304607)*x[0]+(-0.278996789484)*x[1]**o+(-0.139967848513)*x[1]
        s[3,0,2,1]=(-0.657392776251)*x[0]**o+(0.959283975752)*x[0]+(0.990715521701)*x[1]**o+(0.0999777335871)*x[1]
        s[3,1,0,0]=(-0.691053621801)*x[0]**o+(0.189005136467)*x[0]+(0.789186325959)*x[1]**o+(-0.981079749179)*x[1]
        s[3,1,0,1]=(0.228390867858)*x[0]**o+(-0.463941202938)*x[0]+(-0.782094886569)*x[1]**o+(0.464512245943)*x[1]
        s[3,1,1,0]=(-0.819255906497)*x[0]**o+(-0.204510874222)*x[0]+(-0.193473362908)*x[1]**o+(-0.0250165714593)*x[1]
        s[3,1,1,1]=(0.331636523937)*x[0]**o+(-0.0429730777685)*x[0]+(-0.547743299325)*x[1]**o+(-0.187953226002)*x[1]
        s[3,1,2,0]=(0.159370360993)*x[0]**o+(0.883315520778)*x[0]+(0.0933439209551)*x[1]**o+(-0.458319201953)*x[1]
        s[3,1,2,1]=(-0.273991258)*x[0]**o+(0.209739105337)*x[0]+(0.335210395431)*x[1]**o+(-0.91838746555)*x[1]
        s[3,2,0,0]=(-0.0618039563518)*x[0]**o+(-0.59799143228)*x[0]+(0.93203030688)*x[1]**o+(0.581545424351)*x[1]
        s[3,2,0,1]=(-0.321101903544)*x[0]**o+(-0.781260985212)*x[0]+(-0.106128211217)*x[1]**o+(0.913142993455)*x[1]
        s[3,2,1,0]=(0.584486365293)*x[0]**o+(-0.654479654421)*x[0]+(0.579992761397)*x[1]**o+(-0.984753896904)*x[1]
        s[3,2,1,1]=(0.186056629749)*x[0]**o+(-0.889254599383)*x[0]+(0.134406174831)*x[1]**o+(0.843880331117)*x[1]
        s[3,2,2,0]=(0.578447040133)*x[0]**o+(-0.354828599745)*x[0]+(0.943853840713)*x[1]**o+(0.668565941699)*x[1]
        s[3,2,2,1]=(-0.657678353823)*x[0]**o+(0.657623646562)*x[0]+(0.670144253782)*x[1]**o+(0.489533260867)*x[1]
        s[3,3,0,0]=(0.518241767069)*x[0]**o+(0.717113307079)*x[0]+(-0.986081844812)*x[1]**o+(-0.922915734137)*x[1]
        s[3,3,0,1]=(0.934651660463)*x[0]**o+(0.524253175591)*x[0]+(0.519169286504)*x[1]**o+(0.995798377114)*x[1]
        s[3,3,1,0]=(0.396606116937)*x[0]**o+(-0.479342680739)*x[0]+(0.677502711338)*x[1]**o+(-0.591869615963)*x[1]
        s[3,3,1,1]=(0.339631149039)*x[0]**o+(0.44124607198)*x[0]+(0.786875844013)*x[1]**o+(-0.925255814639)*x[1]
        s[3,3,2,0]=(-0.779510842649)*x[0]**o+(0.562505114965)*x[0]+(0.582094163828)*x[1]**o+(0.910521768351)*x[1]
        s[3,3,2,1]=(-0.661315883103)*x[0]**o+(0.226495727911)*x[0]+(-0.464093148756)*x[1]**o+(0.696758174053)*x[1]
        s[3,4,0,0]=(-0.0366658565966)*x[0]**o+(0.78669216474)*x[0]+(-0.784779466718)*x[1]**o+(0.137721843502)*x[1]
        s[3,4,0,1]=(-0.00884301625831)*x[0]**o+(0.25582694645)*x[0]+(-0.73173489905)*x[1]**o+(-0.956569853331)*x[1]
        s[3,4,1,0]=(-0.0884743360746)*x[0]**o+(-0.484605933801)*x[0]+(0.773028853802)*x[1]**o+(-0.13255190562)*x[1]
        s[3,4,1,1]=(0.597859646547)*x[0]**o+(-0.254684950139)*x[0]+(0.292389327091)*x[1]**o+(0.942836804031)*x[1]
        s[3,4,2,0]=(-0.82916540636)*x[0]**o+(0.533192478501)*x[0]+(0.577760569513)*x[1]**o+(-0.812132995663)*x[1]
        s[3,4,2,1]=(-0.0918171874792)*x[0]**o+(-0.232778513191)*x[0]+(-0.57345566951)*x[1]**o+(0.870122102)*x[1]
        ref[0,0,0,0]=(-0.939524201884)*x_ref[0]**o+(-0.294114596857)*x_ref[0]+(-0.662571514204)*x_ref[1]**o+(-0.295055230761)*x_ref[1]
        ref[0,0,0,1]=(0.174447802399)*x_ref[0]**o+(0.313121933505)*x_ref[0]+(0.780423776572)*x_ref[1]**o+(0.260329316622)*x_ref[1]
        ref[0,0,1,0]=(-0.307907192496)*x_ref[0]**o+(0.924386500426)*x_ref[0]+(-0.34849235247)*x_ref[1]**o+(0.42450311854)*x_ref[1]
        ref[0,0,1,1]=(-0.221818525078)*x_ref[0]**o+(0.334658153768)*x_ref[0]+(0.730236845709)*x_ref[1]**o+(0.923645750095)*x_ref[1]
        ref[0,0,2,0]=(-0.0573462957426)*x_ref[0]**o+(0.10400906253)*x_ref[0]+(-0.606757296289)*x_ref[1]**o+(-0.171453259849)*x_ref[1]
        ref[0,0,2,1]=(-0.653190260251)*x_ref[0]**o+(0.392209523496)*x_ref[0]+(0.984792984469)*x_ref[1]**o+(-0.310847119118)*x_ref[1]
        ref[0,1,0,0]=(-0.0980551705202)*x_ref[0]**o+(0.475337517365)*x_ref[0]+(-0.93300381011)*x_ref[1]**o+(0.0593095200419)*x_ref[1]
        ref[0,1,0,1]=(0.833302217025)*x_ref[0]**o+(-0.866194985925)*x_ref[0]+(0.337967734176)*x_ref[1]**o+(0.0581478569311)*x_ref[1]
        ref[0,1,1,0]=(0.936751875211)*x_ref[0]**o+(-0.217226318173)*x_ref[0]+(0.477011461247)*x_ref[1]**o+(0.221436266198)*x_ref[1]
        ref[0,1,1,1]=(-0.537625551791)*x_ref[0]**o+(0.718351842961)*x_ref[0]+(0.0286304587001)*x_ref[1]**o+(-0.721442785761)*x_ref[1]
        ref[0,1,2,0]=(-0.738645216931)*x_ref[0]**o+(0.0195980002286)*x_ref[0]+(0.827363980625)*x_ref[1]**o+(-0.743955943985)*x_ref[1]
        ref[0,1,2,1]=(-0.145033137282)*x_ref[0]**o+(0.98069402199)*x_ref[0]+(-0.167153611864)*x_ref[1]**o+(-0.577697516598)*x_ref[1]
        ref[0,2,0,0]=(0.594926068713)*x_ref[0]**o+(-0.124581510742)*x_ref[0]+(-0.392512448989)*x_ref[1]**o+(-0.989702605289)*x_ref[1]
        ref[0,2,0,1]=(-0.208680726694)*x_ref[0]**o+(0.0226857988067)*x_ref[0]+(0.0127574486126)*x_ref[1]**o+(-0.661169282937)*x_ref[1]
        ref[0,2,1,0]=(0.43109296375)*x_ref[0]**o+(-0.966243011304)*x_ref[0]+(-0.448863362387)*x_ref[1]**o+(-0.531981406137)*x_ref[1]
        ref[0,2,1,1]=(0.427441889618)*x_ref[0]**o+(-0.730863645031)*x_ref[0]+(0.0948615738498)*x_ref[1]**o+(0.349409965532)*x_ref[1]
        ref[0,2,2,0]=(0.0857935500808)*x_ref[0]**o+(0.916612852535)*x_ref[0]+(-0.5434740164)*x_ref[1]**o+(0.651670987189)*x_ref[1]
        ref[0,2,2,1]=(0.269775748844)*x_ref[0]**o+(-0.351727930666)*x_ref[0]+(0.648234694172)*x_ref[1]**o+(0.714331370173)*x_ref[1]
        ref[0,3,0,0]=(0.862594479045)*x_ref[0]**o+(-0.787345711339)*x_ref[0]+(0.1934849213)*x_ref[1]**o+(-0.977224028721)*x_ref[1]
        ref[0,3,0,1]=(0.166215962959)*x_ref[0]**o+(0.734752140512)*x_ref[0]+(0.373558020776)*x_ref[1]**o+(-0.589819927874)*x_ref[1]
        ref[0,3,1,0]=(0.0289724459782)*x_ref[0]**o+(0.517327391578)*x_ref[0]+(0.109956624336)*x_ref[1]**o+(-0.290855234976)*x_ref[1]
        ref[0,3,1,1]=(0.369852037916)*x_ref[0]**o+(0.184556562048)*x_ref[0]+(-0.694630650679)*x_ref[1]**o+(-0.526020870229)*x_ref[1]
        ref[0,3,2,0]=(0.397498255324)*x_ref[0]**o+(-0.375327133358)*x_ref[0]+(0.564571187906)*x_ref[1]**o+(-0.295820211045)*x_ref[1]
        ref[0,3,2,1]=(0.550532413074)*x_ref[0]**o+(-0.751307255234)*x_ref[0]+(0.323446174542)*x_ref[1]**o+(0.968475158376)*x_ref[1]
        ref[0,4,0,0]=(-0.112506956442)*x_ref[0]**o+(-0.432032518771)*x_ref[0]+(0.0205217950343)*x_ref[1]**o+(-0.507714722498)*x_ref[1]
        ref[0,4,0,1]=(-0.501932335824)*x_ref[0]**o+(-0.147371573026)*x_ref[0]+(-0.421130372732)*x_ref[1]**o+(-0.958029815205)*x_ref[1]
        ref[0,4,1,0]=(0.172226819831)*x_ref[0]**o+(-0.712313983987)*x_ref[0]+(0.859887204781)*x_ref[1]**o+(0.0661613029313)*x_ref[1]
        ref[0,4,1,1]=(-0.863367396226)*x_ref[0]**o+(0.150655310131)*x_ref[0]+(-0.446461969683)*x_ref[1]**o+(0.247377395525)*x_ref[1]
        ref[0,4,2,0]=(-0.0431980822723)*x_ref[0]**o+(0.676680822195)*x_ref[0]+(-0.613277443463)*x_ref[1]**o+(0.787356918727)*x_ref[1]
        ref[0,4,2,1]=(-0.0569719722636)*x_ref[0]**o+(0.641187476483)*x_ref[0]+(-0.303617110099)*x_ref[1]**o+(-0.888820557309)*x_ref[1]
        ref[1,0,0,0]=(-0.0552307803637)*x_ref[0]**o+(0.306010657529)*x_ref[0]+(0.488223930134)*x_ref[1]**o+(-0.682392555784)*x_ref[1]
        ref[1,0,0,1]=(-0.253901726526)*x_ref[0]**o+(0.73343629071)*x_ref[0]+(-0.390491326134)*x_ref[1]**o+(0.774605582697)*x_ref[1]
        ref[1,0,1,0]=(-0.42967954588)*x_ref[0]**o+(-0.751284399784)*x_ref[0]+(0.813738133871)*x_ref[1]**o+(-0.0995163058125)*x_ref[1]
        ref[1,0,1,1]=(-0.0880959500234)*x_ref[0]**o+(-0.781527901178)*x_ref[0]+(-0.496899067642)*x_ref[1]**o+(0.55524107606)*x_ref[1]
        ref[1,0,2,0]=(0.256546278479)*x_ref[0]**o+(-0.595310894415)*x_ref[0]+(-0.281823345122)*x_ref[1]**o+(-0.953457305769)*x_ref[1]
        ref[1,0,2,1]=(0.365266273739)*x_ref[0]**o+(-0.935192596652)*x_ref[0]+(0.740921701447)*x_ref[1]**o+(0.843620235195)*x_ref[1]
        ref[1,1,0,0]=(-0.992209271955)*x_ref[0]**o+(0.267499041637)*x_ref[0]+(-0.446633252289)*x_ref[1]**o+(0.357054618942)*x_ref[1]
        ref[1,1,0,1]=(0.863355358433)*x_ref[0]**o+(-0.104881968217)*x_ref[0]+(0.0776056200756)*x_ref[1]**o+(0.485205982195)*x_ref[1]
        ref[1,1,1,0]=(0.29822878169)*x_ref[0]**o+(0.766559198241)*x_ref[0]+(0.489839749671)*x_ref[1]**o+(-0.694316951865)*x_ref[1]
        ref[1,1,1,1]=(-0.947135250126)*x_ref[0]**o+(-0.952537408063)*x_ref[0]+(-0.840569482634)*x_ref[1]**o+(-0.0881389491948)*x_ref[1]
        ref[1,1,2,0]=(0.269982185854)*x_ref[0]**o+(0.290850786316)*x_ref[0]+(0.707550793741)*x_ref[1]**o+(-0.583787562553)*x_ref[1]
        ref[1,1,2,1]=(-0.998478389198)*x_ref[0]**o+(-0.902713626135)*x_ref[0]+(-0.0809583498397)*x_ref[1]**o+(0.242726761269)*x_ref[1]
        ref[1,2,0,0]=(-0.0752124155425)*x_ref[0]**o+(0.287338426631)*x_ref[0]+(-0.873281064449)*x_ref[1]**o+(0.86906507097)*x_ref[1]
        ref[1,2,0,1]=(-0.451012489095)*x_ref[0]**o+(0.692877058818)*x_ref[0]+(-0.771278511209)*x_ref[1]**o+(0.325983000966)*x_ref[1]
        ref[1,2,1,0]=(-0.633239034154)*x_ref[0]**o+(0.751673044983)*x_ref[0]+(-0.799706248119)*x_ref[1]**o+(0.0783814978548)*x_ref[1]
        ref[1,2,1,1]=(-0.496449080562)*x_ref[0]**o+(-0.382869354097)*x_ref[0]+(-0.658792080006)*x_ref[1]**o+(0.578499479656)*x_ref[1]
        ref[1,2,2,0]=(0.41041079346)*x_ref[0]**o+(0.552454636943)*x_ref[0]+(0.236050520221)*x_ref[1]**o+(0.477587390293)*x_ref[1]
        ref[1,2,2,1]=(0.649795041558)*x_ref[0]**o+(-0.947868231928)*x_ref[0]+(-0.529895928082)*x_ref[1]**o+(0.715695540026)*x_ref[1]
        ref[1,3,0,0]=(0.948112901731)*x_ref[0]**o+(-0.374624145588)*x_ref[0]+(0.806640283077)*x_ref[1]**o+(0.729074938843)*x_ref[1]
        ref[1,3,0,1]=(0.266956301954)*x_ref[0]**o+(-0.878004354786)*x_ref[0]+(-0.422298760963)*x_ref[1]**o+(-0.251922913127)*x_ref[1]
        ref[1,3,1,0]=(0.751159198422)*x_ref[0]**o+(-0.576983418224)*x_ref[0]+(0.427653287761)*x_ref[1]**o+(0.351116956206)*x_ref[1]
        ref[1,3,1,1]=(-0.804702781998)*x_ref[0]**o+(-0.218650676384)*x_ref[0]+(-0.832220670207)*x_ref[1]**o+(0.997469993794)*x_ref[1]
        ref[1,3,2,0]=(0.111110638087)*x_ref[0]**o+(-0.435831664904)*x_ref[0]+(-0.144481003079)*x_ref[1]**o+(-0.604349850119)*x_ref[1]
        ref[1,3,2,1]=(0.470311907248)*x_ref[0]**o+(-0.591709930358)*x_ref[0]+(-0.562662398681)*x_ref[1]**o+(0.0163307873851)*x_ref[1]
        ref[1,4,0,0]=(0.684349728229)*x_ref[0]**o+(0.404890565831)*x_ref[0]+(0.922438828147)*x_ref[1]**o+(0.939539648586)*x_ref[1]
        ref[1,4,0,1]=(-0.179161968364)*x_ref[0]**o+(0.983188684135)*x_ref[0]+(-0.722578271046)*x_ref[1]**o+(-0.450472366251)*x_ref[1]
        ref[1,4,1,0]=(0.243718494318)*x_ref[0]**o+(-0.0251174594804)*x_ref[0]+(0.952890549291)*x_ref[1]**o+(-0.398408844924)*x_ref[1]
        ref[1,4,1,1]=(-0.883090916963)*x_ref[0]**o+(-0.236073018867)*x_ref[0]+(-0.245092152479)*x_ref[1]**o+(0.846958498274)*x_ref[1]
        ref[1,4,2,0]=(-0.839928016329)*x_ref[0]**o+(-0.677734468188)*x_ref[0]+(-0.123639765438)*x_ref[1]**o+(-0.680672570927)*x_ref[1]
        ref[1,4,2,1]=(-0.234629335025)*x_ref[0]**o+(-0.567381745605)*x_ref[0]+(0.623480113756)*x_ref[1]**o+(0.2049427452)*x_ref[1]
        ref[2,0,0,0]=(-0.297265663727)*x_ref[0]**o+(0.00899195467866)*x_ref[0]+(0.992475896282)*x_ref[1]**o+(0.863451618621)*x_ref[1]
        ref[2,0,0,1]=(-0.24914517208)*x_ref[0]**o+(0.104091943078)*x_ref[0]+(0.762436787588)*x_ref[1]**o+(0.89363657955)*x_ref[1]
        ref[2,0,1,0]=(0.264734056412)*x_ref[0]**o+(-0.716868164934)*x_ref[0]+(-0.905285068877)*x_ref[1]**o+(0.0227117696273)*x_ref[1]
        ref[2,0,1,1]=(-0.932617617299)*x_ref[0]**o+(0.271634450254)*x_ref[0]+(0.51812940988)*x_ref[1]**o+(-0.484525267705)*x_ref[1]
        ref[2,0,2,0]=(0.364599957461)*x_ref[0]**o+(0.818588026862)*x_ref[0]+(0.70366241863)*x_ref[1]**o+(0.43910943526)*x_ref[1]
        ref[2,0,2,1]=(0.557891341305)*x_ref[0]**o+(-0.891810635437)*x_ref[0]+(0.649138154483)*x_ref[1]**o+(-0.213951745119)*x_ref[1]
        ref[2,1,0,0]=(0.994667066111)*x_ref[0]**o+(0.433110731778)*x_ref[0]+(0.787983438535)*x_ref[1]**o+(0.979717998129)*x_ref[1]
        ref[2,1,0,1]=(0.619284628164)*x_ref[0]**o+(0.662998143683)*x_ref[0]+(0.0328849267442)*x_ref[1]**o+(0.765881868994)*x_ref[1]
        ref[2,1,1,0]=(0.335939940328)*x_ref[0]**o+(0.556758825457)*x_ref[0]+(-0.524870937642)*x_ref[1]**o+(0.0316796705422)*x_ref[1]
        ref[2,1,1,1]=(-0.702249603437)*x_ref[0]**o+(0.544642480464)*x_ref[0]+(0.330520285989)*x_ref[1]**o+(0.313003559197)*x_ref[1]
        ref[2,1,2,0]=(0.0294247925115)*x_ref[0]**o+(0.10067678181)*x_ref[0]+(0.608148681239)*x_ref[1]**o+(-0.95697997936)*x_ref[1]
        ref[2,1,2,1]=(0.101913506022)*x_ref[0]**o+(-0.423061068258)*x_ref[0]+(0.303518988539)*x_ref[1]**o+(0.0108182161414)*x_ref[1]
        ref[2,2,0,0]=(0.390642040424)*x_ref[0]**o+(0.0578342521895)*x_ref[0]+(0.228499563768)*x_ref[1]**o+(-0.944433241107)*x_ref[1]
        ref[2,2,0,1]=(0.522633870922)*x_ref[0]**o+(0.724655140578)*x_ref[0]+(-0.664760217849)*x_ref[1]**o+(0.812535008015)*x_ref[1]
        ref[2,2,1,0]=(0.27608720491)*x_ref[0]**o+(0.42350834695)*x_ref[0]+(-0.732116287856)*x_ref[1]**o+(0.930674651671)*x_ref[1]
        ref[2,2,1,1]=(0.214308063462)*x_ref[0]**o+(0.815707217077)*x_ref[0]+(0.231785784747)*x_ref[1]**o+(-0.954429150944)*x_ref[1]
        ref[2,2,2,0]=(0.402814098347)*x_ref[0]**o+(-0.650491293555)*x_ref[0]+(-0.359159927669)*x_ref[1]**o+(0.503842188774)*x_ref[1]
        ref[2,2,2,1]=(-0.191195415522)*x_ref[0]**o+(0.980791691347)*x_ref[0]+(0.228597878156)*x_ref[1]**o+(0.535975172993)*x_ref[1]
        ref[2,3,0,0]=(0.495749327231)*x_ref[0]**o+(0.311034011895)*x_ref[0]+(-0.532686805928)*x_ref[1]**o+(0.92438969176)*x_ref[1]
        ref[2,3,0,1]=(-0.38513599341)*x_ref[0]**o+(-0.463099237819)*x_ref[0]+(-0.992606328662)*x_ref[1]**o+(-0.927920439344)*x_ref[1]
        ref[2,3,1,0]=(-0.868022271483)*x_ref[0]**o+(-0.540096192748)*x_ref[0]+(-0.0879858139685)*x_ref[1]**o+(0.534604810517)*x_ref[1]
        ref[2,3,1,1]=(-0.686837754683)*x_ref[0]**o+(-0.505585075528)*x_ref[0]+(-0.218155668559)*x_ref[1]**o+(-0.241972887878)*x_ref[1]
        ref[2,3,2,0]=(0.86187325655)*x_ref[0]**o+(-0.733958307606)*x_ref[0]+(-0.107393817742)*x_ref[1]**o+(-0.830306745614)*x_ref[1]
        ref[2,3,2,1]=(0.404834271017)*x_ref[0]**o+(0.44646270613)*x_ref[0]+(-0.657038236282)*x_ref[1]**o+(-0.540570795414)*x_ref[1]
        ref[2,4,0,0]=(-0.216828677229)*x_ref[0]**o+(0.129702603598)*x_ref[0]+(0.556920803376)*x_ref[1]**o+(0.960021941421)*x_ref[1]
        ref[2,4,0,1]=(-0.21625137654)*x_ref[0]**o+(-0.19604294366)*x_ref[0]+(-0.889082170851)*x_ref[1]**o+(-0.834967241538)*x_ref[1]
        ref[2,4,1,0]=(0.123104286845)*x_ref[0]**o+(-0.43197703752)*x_ref[0]+(0.737850827826)*x_ref[1]**o+(0.344689639656)*x_ref[1]
        ref[2,4,1,1]=(0.119471348292)*x_ref[0]**o+(0.948353928609)*x_ref[0]+(-0.726863566036)*x_ref[1]**o+(-0.957158639686)*x_ref[1]
        ref[2,4,2,0]=(0.606624368242)*x_ref[0]**o+(-0.129636738653)*x_ref[0]+(0.117469089776)*x_ref[1]**o+(0.766024628396)*x_ref[1]
        ref[2,4,2,1]=(-0.83631907859)*x_ref[0]**o+(-0.168433036335)*x_ref[0]+(-0.500663355802)*x_ref[1]**o+(0.0114866397792)*x_ref[1]
        ref[3,0,0,0]=(-0.629399168531)*x_ref[0]**o+(-0.464524331386)*x_ref[0]+(0.0406391749081)*x_ref[1]**o+(-0.380881295086)*x_ref[1]
        ref[3,0,0,1]=(-0.614191170121)*x_ref[0]**o+(0.90297284642)*x_ref[0]+(0.949653922227)*x_ref[1]**o+(0.947014467705)*x_ref[1]
        ref[3,0,1,0]=(0.128075669906)*x_ref[0]**o+(-0.215027529464)*x_ref[0]+(-0.230820603567)*x_ref[1]**o+(0.656672788071)*x_ref[1]
        ref[3,0,1,1]=(0.11006414768)*x_ref[0]**o+(0.575548749986)*x_ref[0]+(0.803886215346)*x_ref[1]**o+(0.627503866105)*x_ref[1]
        ref[3,0,2,0]=(-0.824524708364)*x_ref[0]**o+(-0.399533304607)*x_ref[0]+(-0.278996789484)*x_ref[1]**o+(-0.139967848513)*x_ref[1]
        ref[3,0,2,1]=(-0.657392776251)*x_ref[0]**o+(0.959283975752)*x_ref[0]+(0.990715521701)*x_ref[1]**o+(0.0999777335871)*x_ref[1]
        ref[3,1,0,0]=(-0.691053621801)*x_ref[0]**o+(0.189005136467)*x_ref[0]+(0.789186325959)*x_ref[1]**o+(-0.981079749179)*x_ref[1]
        ref[3,1,0,1]=(0.228390867858)*x_ref[0]**o+(-0.463941202938)*x_ref[0]+(-0.782094886569)*x_ref[1]**o+(0.464512245943)*x_ref[1]
        ref[3,1,1,0]=(-0.819255906497)*x_ref[0]**o+(-0.204510874222)*x_ref[0]+(-0.193473362908)*x_ref[1]**o+(-0.0250165714593)*x_ref[1]
        ref[3,1,1,1]=(0.331636523937)*x_ref[0]**o+(-0.0429730777685)*x_ref[0]+(-0.547743299325)*x_ref[1]**o+(-0.187953226002)*x_ref[1]
        ref[3,1,2,0]=(0.159370360993)*x_ref[0]**o+(0.883315520778)*x_ref[0]+(0.0933439209551)*x_ref[1]**o+(-0.458319201953)*x_ref[1]
        ref[3,1,2,1]=(-0.273991258)*x_ref[0]**o+(0.209739105337)*x_ref[0]+(0.335210395431)*x_ref[1]**o+(-0.91838746555)*x_ref[1]
        ref[3,2,0,0]=(-0.0618039563518)*x_ref[0]**o+(-0.59799143228)*x_ref[0]+(0.93203030688)*x_ref[1]**o+(0.581545424351)*x_ref[1]
        ref[3,2,0,1]=(-0.321101903544)*x_ref[0]**o+(-0.781260985212)*x_ref[0]+(-0.106128211217)*x_ref[1]**o+(0.913142993455)*x_ref[1]
        ref[3,2,1,0]=(0.584486365293)*x_ref[0]**o+(-0.654479654421)*x_ref[0]+(0.579992761397)*x_ref[1]**o+(-0.984753896904)*x_ref[1]
        ref[3,2,1,1]=(0.186056629749)*x_ref[0]**o+(-0.889254599383)*x_ref[0]+(0.134406174831)*x_ref[1]**o+(0.843880331117)*x_ref[1]
        ref[3,2,2,0]=(0.578447040133)*x_ref[0]**o+(-0.354828599745)*x_ref[0]+(0.943853840713)*x_ref[1]**o+(0.668565941699)*x_ref[1]
        ref[3,2,2,1]=(-0.657678353823)*x_ref[0]**o+(0.657623646562)*x_ref[0]+(0.670144253782)*x_ref[1]**o+(0.489533260867)*x_ref[1]
        ref[3,3,0,0]=(0.518241767069)*x_ref[0]**o+(0.717113307079)*x_ref[0]+(-0.986081844812)*x_ref[1]**o+(-0.922915734137)*x_ref[1]
        ref[3,3,0,1]=(0.934651660463)*x_ref[0]**o+(0.524253175591)*x_ref[0]+(0.519169286504)*x_ref[1]**o+(0.995798377114)*x_ref[1]
        ref[3,3,1,0]=(0.396606116937)*x_ref[0]**o+(-0.479342680739)*x_ref[0]+(0.677502711338)*x_ref[1]**o+(-0.591869615963)*x_ref[1]
        ref[3,3,1,1]=(0.339631149039)*x_ref[0]**o+(0.44124607198)*x_ref[0]+(0.786875844013)*x_ref[1]**o+(-0.925255814639)*x_ref[1]
        ref[3,3,2,0]=(-0.779510842649)*x_ref[0]**o+(0.562505114965)*x_ref[0]+(0.582094163828)*x_ref[1]**o+(0.910521768351)*x_ref[1]
        ref[3,3,2,1]=(-0.661315883103)*x_ref[0]**o+(0.226495727911)*x_ref[0]+(-0.464093148756)*x_ref[1]**o+(0.696758174053)*x_ref[1]
        ref[3,4,0,0]=(-0.0366658565966)*x_ref[0]**o+(0.78669216474)*x_ref[0]+(-0.784779466718)*x_ref[1]**o+(0.137721843502)*x_ref[1]
        ref[3,4,0,1]=(-0.00884301625831)*x_ref[0]**o+(0.25582694645)*x_ref[0]+(-0.73173489905)*x_ref[1]**o+(-0.956569853331)*x_ref[1]
        ref[3,4,1,0]=(-0.0884743360746)*x_ref[0]**o+(-0.484605933801)*x_ref[0]+(0.773028853802)*x_ref[1]**o+(-0.13255190562)*x_ref[1]
        ref[3,4,1,1]=(0.597859646547)*x_ref[0]**o+(-0.254684950139)*x_ref[0]+(0.292389327091)*x_ref[1]**o+(0.942836804031)*x_ref[1]
        ref[3,4,2,0]=(-0.82916540636)*x_ref[0]**o+(0.533192478501)*x_ref[0]+(0.577760569513)*x_ref[1]**o+(-0.812132995663)*x_ref[1]
        ref[3,4,2,1]=(-0.0918171874792)*x_ref[0]**o+(-0.232778513191)*x_ref[0]+(-0.57345566951)*x_ref[1]**o+(0.870122102)*x_ref[1]
      else:
        s[0,0,0,0]=(-0.419262547354)*x[0]**o+(-0.607792358178)*x[0]+(0.539129711536)*x[1]**o+(-0.954002022975)*x[1]+(0.52016184412)*x[2]**o+(-0.905599414589)*x[2]
        s[0,0,0,1]=(0.552249219683)*x[0]**o+(0.0747535682119)*x[0]+(-0.0550858389381)*x[1]**o+(-0.700756838491)*x[1]+(-0.62569620016)*x[2]**o+(-0.557287205831)*x[2]
        s[0,0,1,0]=(-0.84674266979)*x[0]**o+(-0.266089976589)*x[0]+(-0.994821395425)*x[1]**o+(-0.567614298529)*x[1]+(-0.814756268883)*x[2]**o+(-0.524622381582)*x[2]
        s[0,0,1,1]=(0.160195104873)*x[0]**o+(0.161596910068)*x[0]+(-0.0404024431157)*x[1]**o+(0.299140544701)*x[1]+(0.479120924271)*x[2]**o+(0.375793387197)*x[2]
        s[0,0,2,0]=(0.619011883068)*x[0]**o+(0.453196321806)*x[0]+(0.596546848845)*x[1]**o+(-0.148655559308)*x[1]+(-0.276643963879)*x[2]**o+(-0.556233199969)*x[2]
        s[0,0,2,1]=(-0.44936519278)*x[0]**o+(0.279187353573)*x[0]+(-0.626699833624)*x[1]**o+(0.172543451997)*x[1]+(0.324942859052)*x[2]**o+(0.227082355579)*x[2]
        s[0,1,0,0]=(0.348983572344)*x[0]**o+(-0.692310510135)*x[0]+(0.127542726356)*x[1]**o+(-0.719693998448)*x[1]+(0.190533765375)*x[2]**o+(-0.44411272509)*x[2]
        s[0,1,0,1]=(0.755193842627)*x[0]**o+(-0.0380801278139)*x[0]+(-0.416973165389)*x[1]**o+(-0.407050428042)*x[1]+(-0.803402606403)*x[2]**o+(0.860761604165)*x[2]
        s[0,1,1,0]=(0.903125811803)*x[0]**o+(-0.370434864257)*x[0]+(-0.810880082994)*x[1]**o+(0.214166691855)*x[1]+(-0.62621693665)*x[2]**o+(-0.043360368073)*x[2]
        s[0,1,1,1]=(-0.442516790862)*x[0]**o+(0.912139701807)*x[0]+(0.871611995743)*x[1]**o+(0.0291580448949)*x[1]+(-0.489427959855)*x[2]**o+(-0.950170294127)*x[2]
        s[0,1,2,0]=(-0.0867645917569)*x[0]**o+(0.556462347924)*x[0]+(-0.816554443299)*x[1]**o+(0.818773437796)*x[1]+(0.073905470924)*x[2]**o+(-0.714497094507)*x[2]
        s[0,1,2,1]=(0.699491100428)*x[0]**o+(-0.200275804269)*x[0]+(0.994316913102)*x[1]**o+(0.832890328003)*x[1]+(0.385381865669)*x[2]**o+(0.153056606745)*x[2]
        s[0,2,0,0]=(0.612319496073)*x[0]**o+(-0.301991428044)*x[0]+(-0.536866971844)*x[1]**o+(-0.375311119699)*x[1]+(0.733545441876)*x[2]**o+(0.0177187199528)*x[2]
        s[0,2,0,1]=(0.358991602948)*x[0]**o+(-0.862573091669)*x[0]+(0.464891723814)*x[1]**o+(0.602366461601)*x[1]+(-0.668916509347)*x[2]**o+(0.76048108017)*x[2]
        s[0,2,1,0]=(0.188195665573)*x[0]**o+(-0.927407688437)*x[0]+(-0.473014301986)*x[1]**o+(-0.775114885201)*x[1]+(-0.453403342886)*x[2]**o+(0.006776981492)*x[2]
        s[0,2,1,1]=(-0.700181448543)*x[0]**o+(-0.289180138599)*x[0]+(-0.867022883968)*x[1]**o+(-0.359339607516)*x[1]+(-0.809558496152)*x[2]**o+(-0.576358976651)*x[2]
        s[0,2,2,0]=(-0.557333080207)*x[0]**o+(0.724962853865)*x[0]+(0.995041226791)*x[1]**o+(-0.610460710065)*x[1]+(0.620016428769)*x[2]**o+(-0.449182254452)*x[2]
        s[0,2,2,1]=(-0.937207840292)*x[0]**o+(-0.66582207891)*x[0]+(0.0951881913165)*x[1]**o+(-0.0770694628972)*x[1]+(0.034224640398)*x[2]**o+(0.652057466224)*x[2]
        s[0,3,0,0]=(-0.0236750483037)*x[0]**o+(0.956130793675)*x[0]+(-0.589425964387)*x[1]**o+(-0.204770941511)*x[1]+(0.695255730164)*x[2]**o+(-0.796222694557)*x[2]
        s[0,3,0,1]=(0.138589818728)*x[0]**o+(0.493237559501)*x[0]+(0.139281181588)*x[1]**o+(-0.974201990591)*x[1]+(-0.15401887402)*x[2]**o+(0.23794136041)*x[2]
        s[0,3,1,0]=(-0.332886758751)*x[0]**o+(-0.231549718295)*x[0]+(0.581536450397)*x[1]**o+(0.790063092494)*x[1]+(0.358814129932)*x[2]**o+(-0.110547525643)*x[2]
        s[0,3,1,1]=(0.122775146487)*x[0]**o+(-0.00818530079912)*x[0]+(0.0510575666357)*x[1]**o+(-0.96911992662)*x[1]+(-0.820269176326)*x[2]**o+(0.875945222846)*x[2]
        s[0,3,2,0]=(0.761874596666)*x[0]**o+(0.621369218231)*x[0]+(0.960271412263)*x[1]**o+(-0.808832785376)*x[1]+(0.981428454481)*x[2]**o+(0.393331122254)*x[2]
        s[0,3,2,1]=(-0.646477042209)*x[0]**o+(-0.26169646294)*x[0]+(0.992265690785)*x[1]**o+(-0.678558332435)*x[1]+(-0.285531525349)*x[2]**o+(-0.571647919822)*x[2]
        s[0,4,0,0]=(0.492930260015)*x[0]**o+(-0.647489772768)*x[0]+(-0.381225997009)*x[1]**o+(0.560462598779)*x[1]+(-0.63571677055)*x[2]**o+(0.103213895387)*x[2]
        s[0,4,0,1]=(-0.0362476689653)*x[0]**o+(-0.478992488756)*x[0]+(0.544907192925)*x[1]**o+(-0.678350370776)*x[1]+(-0.0776363475051)*x[2]**o+(-0.803140341684)*x[2]
        s[0,4,1,0]=(0.0787683344872)*x[0]**o+(0.688990188782)*x[0]+(0.257089245341)*x[1]**o+(-0.648663765213)*x[1]+(-0.140632312728)*x[2]**o+(0.676209090063)*x[2]
        s[0,4,1,1]=(0.804994807205)*x[0]**o+(0.74750498671)*x[0]+(-0.00532761506025)*x[1]**o+(-0.295700694165)*x[1]+(-0.209175099115)*x[2]**o+(-0.0126407798374)*x[2]
        s[0,4,2,0]=(-0.789160064467)*x[0]**o+(0.332412023778)*x[0]+(0.242860020473)*x[1]**o+(0.0284658318544)*x[1]+(-0.357559965972)*x[2]**o+(0.57056582293)*x[2]
        s[0,4,2,1]=(-0.861027741096)*x[0]**o+(0.545166238342)*x[0]+(0.950975873381)*x[1]**o+(-0.436709572856)*x[1]+(0.93636977927)*x[2]**o+(-0.405205657349)*x[2]
        s[1,0,0,0]=(0.47880791873)*x[0]**o+(0.037486909403)*x[0]+(0.732218957477)*x[1]**o+(0.274811688271)*x[1]+(0.0686323288694)*x[2]**o+(-0.492347744678)*x[2]
        s[1,0,0,1]=(-0.618259414329)*x[0]**o+(0.630883795613)*x[0]+(0.269141377847)*x[1]**o+(-0.490916952452)*x[1]+(-0.333070678904)*x[2]**o+(0.0484209401349)*x[2]
        s[1,0,1,0]=(-0.906059152197)*x[0]**o+(0.0322820078282)*x[0]+(-0.47807076381)*x[1]**o+(0.0296380181906)*x[1]+(0.553686905633)*x[2]**o+(-0.408333784569)*x[2]
        s[1,0,1,1]=(0.282962689091)*x[0]**o+(-0.11495842442)*x[0]+(0.818142653998)*x[1]**o+(0.563460324128)*x[1]+(-0.993782402791)*x[2]**o+(-0.0193690715193)*x[2]
        s[1,0,2,0]=(0.287962506263)*x[0]**o+(-0.401293374286)*x[0]+(0.173559459587)*x[1]**o+(0.822343918838)*x[1]+(0.888309195417)*x[2]**o+(0.687878298228)*x[2]
        s[1,0,2,1]=(-0.926775125361)*x[0]**o+(0.396396669159)*x[0]+(0.632962652289)*x[1]**o+(-0.915667885738)*x[1]+(-0.31075960693)*x[2]**o+(0.920135418244)*x[2]
        s[1,1,0,0]=(-0.0915500113579)*x[0]**o+(0.489036843145)*x[0]+(0.477982330324)*x[1]**o+(0.703371804742)*x[1]+(-0.471773636091)*x[2]**o+(0.0665811350713)*x[2]
        s[1,1,0,1]=(0.525411480834)*x[0]**o+(-0.369843393433)*x[0]+(-0.7770705966)*x[1]**o+(0.534153802448)*x[1]+(0.309828458492)*x[2]**o+(0.627690678711)*x[2]
        s[1,1,1,0]=(0.149039257889)*x[0]**o+(-0.0585283560518)*x[0]+(-0.0505893024858)*x[1]**o+(0.677281229531)*x[1]+(-0.451950028252)*x[2]**o+(0.356600216864)*x[2]
        s[1,1,1,1]=(-0.528237952467)*x[0]**o+(0.533554438129)*x[0]+(0.741981923136)*x[1]**o+(-0.596922185095)*x[1]+(0.50521173977)*x[2]**o+(-0.247100318829)*x[2]
        s[1,1,2,0]=(0.667977944954)*x[0]**o+(0.306325804541)*x[0]+(-0.654643365837)*x[1]**o+(0.0176380527447)*x[1]+(-0.218012920296)*x[2]**o+(-0.181740681902)*x[2]
        s[1,1,2,1]=(0.440430981908)*x[0]**o+(0.994608754477)*x[0]+(0.114426402744)*x[1]**o+(-0.896222828258)*x[1]+(0.997911423594)*x[2]**o+(0.648328801403)*x[2]
        s[1,2,0,0]=(0.397328767103)*x[0]**o+(0.780110953188)*x[0]+(0.453955649367)*x[1]**o+(0.953839185994)*x[1]+(0.995536671917)*x[2]**o+(0.177278490615)*x[2]
        s[1,2,0,1]=(-0.154393442621)*x[0]**o+(-0.100267841021)*x[0]+(-0.781347482738)*x[1]**o+(0.391441261298)*x[1]+(0.469832428629)*x[2]**o+(0.608531800159)*x[2]
        s[1,2,1,0]=(-0.534167310466)*x[0]**o+(0.414564904301)*x[0]+(-0.0674568638795)*x[1]**o+(0.577721681963)*x[1]+(-0.00750850180411)*x[2]**o+(0.0817658166565)*x[2]
        s[1,2,1,1]=(-0.529547188418)*x[0]**o+(0.267917121456)*x[0]+(0.372449729423)*x[1]**o+(0.887072823755)*x[1]+(0.887821539758)*x[2]**o+(0.249771090594)*x[2]
        s[1,2,2,0]=(0.949056994737)*x[0]**o+(-0.174523839411)*x[0]+(-0.646085737348)*x[1]**o+(0.449358235114)*x[1]+(-0.970748494721)*x[2]**o+(-0.410800591341)*x[2]
        s[1,2,2,1]=(0.879006069022)*x[0]**o+(-0.510106021113)*x[0]+(0.0203703221469)*x[1]**o+(-0.844338135123)*x[1]+(0.752168319634)*x[2]**o+(0.158760999046)*x[2]
        s[1,3,0,0]=(0.845321813876)*x[0]**o+(0.405224606418)*x[0]+(-0.89159606951)*x[1]**o+(-0.949499370695)*x[1]+(0.501499536159)*x[2]**o+(-0.869583954204)*x[2]
        s[1,3,0,1]=(0.0887248259912)*x[0]**o+(0.232925681778)*x[0]+(0.0315176777236)*x[1]**o+(0.00568107770424)*x[1]+(-0.821963787641)*x[2]**o+(-0.634962715535)*x[2]
        s[1,3,1,0]=(0.546210053059)*x[0]**o+(-0.688171108228)*x[0]+(-0.699621490343)*x[1]**o+(-0.850445270177)*x[1]+(-0.725131419114)*x[2]**o+(0.484336843024)*x[2]
        s[1,3,1,1]=(0.278594806693)*x[0]**o+(-0.266710093152)*x[0]+(-0.151515190688)*x[1]**o+(-0.705005965054)*x[1]+(0.811008835405)*x[2]**o+(-0.342637057553)*x[2]
        s[1,3,2,0]=(0.4748294578)*x[0]**o+(-0.0741346092747)*x[0]+(0.148026337115)*x[1]**o+(0.0252686427245)*x[1]+(-0.479120476874)*x[2]**o+(0.0855008343792)*x[2]
        s[1,3,2,1]=(-0.337007818128)*x[0]**o+(-0.690464368815)*x[0]+(0.481925609432)*x[1]**o+(-0.0981705702921)*x[1]+(0.19218353071)*x[2]**o+(0.988337158745)*x[2]
        s[1,4,0,0]=(-0.757351506535)*x[0]**o+(-0.824203825425)*x[0]+(-0.998647750787)*x[1]**o+(-0.111654698732)*x[1]+(-0.713287873387)*x[2]**o+(0.779671510732)*x[2]
        s[1,4,0,1]=(-0.756370998169)*x[0]**o+(0.826863767951)*x[0]+(0.358310449165)*x[1]**o+(0.882430292099)*x[1]+(0.880156909975)*x[2]**o+(0.360411043274)*x[2]
        s[1,4,1,0]=(0.226080011536)*x[0]**o+(-0.740536947501)*x[0]+(-0.650527457572)*x[1]**o+(-0.890015156497)*x[1]+(0.688973071404)*x[2]**o+(0.29237820526)*x[2]
        s[1,4,1,1]=(0.574015626188)*x[0]**o+(-0.675641095131)*x[0]+(0.814289365677)*x[1]**o+(-0.625021588943)*x[1]+(-0.862079110641)*x[2]**o+(0.0363949889967)*x[2]
        s[1,4,2,0]=(0.126659193582)*x[0]**o+(0.16976937315)*x[0]+(0.18822384431)*x[1]**o+(0.546082142359)*x[1]+(0.551365996908)*x[2]**o+(0.395213783677)*x[2]
        s[1,4,2,1]=(-0.292436267133)*x[0]**o+(-0.442787521929)*x[0]+(-0.310488595304)*x[1]**o+(0.0918858003303)*x[1]+(-0.986695921261)*x[2]**o+(-0.52979957068)*x[2]
        s[2,0,0,0]=(0.396669972067)*x[0]**o+(-0.711415400722)*x[0]+(-0.717006050735)*x[1]**o+(-0.0935757817421)*x[1]+(0.743477556176)*x[2]**o+(0.823967976857)*x[2]
        s[2,0,0,1]=(0.488395228667)*x[0]**o+(0.230144035899)*x[0]+(0.125851877066)*x[1]**o+(0.0536284998009)*x[1]+(-0.0114515128334)*x[2]**o+(-0.494039400348)*x[2]
        s[2,0,1,0]=(0.900615108208)*x[0]**o+(0.432765237128)*x[0]+(0.981213121806)*x[1]**o+(-0.116934702769)*x[1]+(-0.612499086627)*x[2]**o+(-0.365572411348)*x[2]
        s[2,0,1,1]=(0.0342670705338)*x[0]**o+(0.54954404771)*x[0]+(0.428653302401)*x[1]**o+(0.537787906346)*x[1]+(-0.64287424457)*x[2]**o+(-0.173877644259)*x[2]
        s[2,0,2,0]=(-0.0510871045727)*x[0]**o+(-0.908007271573)*x[0]+(0.00194326164542)*x[1]**o+(0.466243299714)*x[1]+(-0.433962263339)*x[2]**o+(0.874724229969)*x[2]
        s[2,0,2,1]=(0.666308090297)*x[0]**o+(0.933188439742)*x[0]+(-0.0479445062129)*x[1]**o+(0.0747806162498)*x[1]+(0.911844507192)*x[2]**o+(-0.934538480024)*x[2]
        s[2,1,0,0]=(-0.516290157366)*x[0]**o+(0.437519025532)*x[0]+(-0.170999588789)*x[1]**o+(-0.934096554106)*x[1]+(0.88575800375)*x[2]**o+(0.636966058116)*x[2]
        s[2,1,0,1]=(0.0172669412005)*x[0]**o+(-0.867857912246)*x[0]+(-0.972747832278)*x[1]**o+(-0.780969292213)*x[1]+(-0.418154913379)*x[2]**o+(0.805902597544)*x[2]
        s[2,1,1,0]=(-0.964006922451)*x[0]**o+(0.236797578253)*x[0]+(-0.78802971076)*x[1]**o+(0.704165381448)*x[1]+(0.00112348607329)*x[2]**o+(-0.689065853536)*x[2]
        s[2,1,1,1]=(-0.722392335335)*x[0]**o+(-0.120018753634)*x[0]+(0.216541173904)*x[1]**o+(-0.374710223309)*x[1]+(0.532158723682)*x[2]**o+(0.464637663705)*x[2]
        s[2,1,2,0]=(-0.183637646888)*x[0]**o+(-0.0680674034088)*x[0]+(-0.114129008735)*x[1]**o+(-0.78666547603)*x[1]+(0.0956689943304)*x[2]**o+(0.743296994432)*x[2]
        s[2,1,2,1]=(0.611382565923)*x[0]**o+(-0.768682259494)*x[0]+(-0.540943249593)*x[1]**o+(0.753523431999)*x[1]+(0.610689642267)*x[2]**o+(0.0122811249496)*x[2]
        s[2,2,0,0]=(-0.789273374112)*x[0]**o+(0.0977540682694)*x[0]+(-0.829719260296)*x[1]**o+(0.824773509135)*x[1]+(0.216669854484)*x[2]**o+(-0.764947843182)*x[2]
        s[2,2,0,1]=(-0.340784170043)*x[0]**o+(-0.267437208067)*x[0]+(-0.430973997706)*x[1]**o+(-0.0846177320866)*x[1]+(0.688599546971)*x[2]**o+(0.265513958383)*x[2]
        s[2,2,1,0]=(-0.237677411545)*x[0]**o+(-0.792144835449)*x[0]+(-0.435763499999)*x[1]**o+(0.682920743804)*x[1]+(0.116459199169)*x[2]**o+(0.254527225097)*x[2]
        s[2,2,1,1]=(0.342906428734)*x[0]**o+(0.206470310722)*x[0]+(-0.352429884648)*x[1]**o+(-0.168905312607)*x[1]+(-0.105529665095)*x[2]**o+(0.199873788005)*x[2]
        s[2,2,2,0]=(0.677855511296)*x[0]**o+(0.983953993003)*x[0]+(-0.64997591911)*x[1]**o+(-0.629221981454)*x[1]+(0.990621105566)*x[2]**o+(0.785766133934)*x[2]
        s[2,2,2,1]=(-0.570482617922)*x[0]**o+(-0.151066051792)*x[0]+(-0.511453364326)*x[1]**o+(0.128824691419)*x[1]+(-0.520127324328)*x[2]**o+(-0.157450052117)*x[2]
        s[2,3,0,0]=(-0.423827660795)*x[0]**o+(-0.815943363397)*x[0]+(-0.636070023986)*x[1]**o+(0.251327239067)*x[1]+(-0.401641739847)*x[2]**o+(0.130280903755)*x[2]
        s[2,3,0,1]=(0.0411439121625)*x[0]**o+(0.621609981629)*x[0]+(-0.589059932668)*x[1]**o+(0.543095349334)*x[1]+(-0.0122421869863)*x[2]**o+(-0.776924587359)*x[2]
        s[2,3,1,0]=(0.63252327363)*x[0]**o+(0.275597518383)*x[0]+(0.832198526441)*x[1]**o+(-0.92855918951)*x[1]+(0.490343890624)*x[2]**o+(-0.412580022625)*x[2]
        s[2,3,1,1]=(-0.560907436774)*x[0]**o+(0.413956414518)*x[0]+(-0.849736975879)*x[1]**o+(0.167861053833)*x[1]+(-0.316526371498)*x[2]**o+(-0.431245755915)*x[2]
        s[2,3,2,0]=(0.883757549647)*x[0]**o+(-0.614133317417)*x[0]+(0.923442235162)*x[1]**o+(0.483998522495)*x[1]+(0.649003328717)*x[2]**o+(-0.320181801832)*x[2]
        s[2,3,2,1]=(0.181982086686)*x[0]**o+(-0.88761330228)*x[0]+(-0.752108545831)*x[1]**o+(0.105681418232)*x[1]+(0.211053107745)*x[2]**o+(-0.840650090772)*x[2]
        s[2,4,0,0]=(0.774235220873)*x[0]**o+(-0.394430371883)*x[0]+(-0.82551531411)*x[1]**o+(-0.397374538567)*x[1]+(0.51757884795)*x[2]**o+(-0.878919341817)*x[2]
        s[2,4,0,1]=(-0.282445913776)*x[0]**o+(-0.252614209563)*x[0]+(-0.66810962709)*x[1]**o+(0.912660025157)*x[1]+(0.786200789061)*x[2]**o+(-0.271329907892)*x[2]
        s[2,4,1,0]=(-0.415061697718)*x[0]**o+(-0.577077815899)*x[0]+(-0.479115469184)*x[1]**o+(0.285424287206)*x[1]+(0.580737671113)*x[2]**o+(0.345716282261)*x[2]
        s[2,4,1,1]=(0.103116885191)*x[0]**o+(0.472934825259)*x[0]+(0.192747945705)*x[1]**o+(0.106949097334)*x[1]+(-0.109457290729)*x[2]**o+(0.843345964596)*x[2]
        s[2,4,2,0]=(0.600281530422)*x[0]**o+(-0.427230650838)*x[0]+(-0.868150408946)*x[1]**o+(-0.203647711262)*x[1]+(-0.550477804035)*x[2]**o+(0.415736214172)*x[2]
        s[2,4,2,1]=(0.739232217202)*x[0]**o+(-0.476864848171)*x[0]+(0.367627911492)*x[1]**o+(0.105176582927)*x[1]+(0.482113806667)*x[2]**o+(0.782020005885)*x[2]
        s[3,0,0,0]=(0.938839141593)*x[0]**o+(-0.655793939395)*x[0]+(0.0224780413611)*x[1]**o+(0.252535317383)*x[1]+(-0.887022478757)*x[2]**o+(-0.451912746232)*x[2]
        s[3,0,0,1]=(-0.196994797427)*x[0]**o+(-0.222816920678)*x[0]+(0.37131460044)*x[1]**o+(0.124872956754)*x[1]+(-0.872680499884)*x[2]**o+(-0.490521736465)*x[2]
        s[3,0,1,0]=(-0.977712672434)*x[0]**o+(-0.665877989129)*x[0]+(-0.541103000349)*x[1]**o+(-0.527523471522)*x[1]+(0.469063433285)*x[2]**o+(-0.410680406273)*x[2]
        s[3,0,1,1]=(0.371612180735)*x[0]**o+(-0.882397978651)*x[0]+(-0.333119569273)*x[1]**o+(0.0439599202688)*x[1]+(-0.378698960758)*x[2]**o+(0.373830212832)*x[2]
        s[3,0,2,0]=(0.13552136805)*x[0]**o+(-0.347572305079)*x[0]+(0.194173745453)*x[1]**o+(-0.536374783086)*x[1]+(-0.0928402883882)*x[2]**o+(0.972386889251)*x[2]
        s[3,0,2,1]=(-0.673716579315)*x[0]**o+(0.497683603572)*x[0]+(0.0894501300871)*x[1]**o+(-0.691905501083)*x[1]+(-0.819373609736)*x[2]**o+(0.876955815163)*x[2]
        s[3,1,0,0]=(-0.612778977625)*x[0]**o+(0.898593519513)*x[0]+(-0.579505051936)*x[1]**o+(0.294087297529)*x[1]+(0.713022674221)*x[2]**o+(-0.0994065928128)*x[2]
        s[3,1,0,1]=(-0.141600251938)*x[0]**o+(-0.677815682935)*x[0]+(-0.408920459122)*x[1]**o+(-0.723029003437)*x[1]+(0.481057659884)*x[2]**o+(0.427909771498)*x[2]
        s[3,1,1,0]=(-0.516486224771)*x[0]**o+(0.526484181999)*x[0]+(0.911793059151)*x[1]**o+(0.409808964766)*x[1]+(-0.413591542253)*x[2]**o+(-0.759664326823)*x[2]
        s[3,1,1,1]=(0.905324377656)*x[0]**o+(-0.208896172985)*x[0]+(-0.422109371498)*x[1]**o+(0.491883832748)*x[1]+(-0.203413002012)*x[2]**o+(0.406737219074)*x[2]
        s[3,1,2,0]=(-0.573692277755)*x[0]**o+(0.895762539503)*x[0]+(0.176538736347)*x[1]**o+(-0.847640766515)*x[1]+(0.5650742337)*x[2]**o+(0.503165369442)*x[2]
        s[3,1,2,1]=(0.308521434074)*x[0]**o+(0.616776036606)*x[0]+(-0.405321909102)*x[1]**o+(0.497367404288)*x[1]+(-0.798636270652)*x[2]**o+(-0.337423592684)*x[2]
        s[3,2,0,0]=(-0.394455434548)*x[0]**o+(-0.292212771133)*x[0]+(-0.941613233015)*x[1]**o+(-0.404522244202)*x[1]+(-0.0112893909573)*x[2]**o+(-0.9706166943)*x[2]
        s[3,2,0,1]=(0.627774279675)*x[0]**o+(-0.645334179444)*x[0]+(0.691389142649)*x[1]**o+(0.763379715146)*x[1]+(0.610766030943)*x[2]**o+(-0.786459853785)*x[2]
        s[3,2,1,0]=(-0.0101413740932)*x[0]**o+(-0.0970748510013)*x[0]+(-0.52181886114)*x[1]**o+(-0.612494396935)*x[1]+(0.941988727244)*x[2]**o+(0.0361447091017)*x[2]
        s[3,2,1,1]=(-0.0943447427863)*x[0]**o+(0.708735490064)*x[0]+(-0.352853510872)*x[1]**o+(-0.545593672805)*x[1]+(-0.0106851492926)*x[2]**o+(0.712976402827)*x[2]
        s[3,2,2,0]=(-0.522341854545)*x[0]**o+(0.320365455548)*x[0]+(-0.111682800076)*x[1]**o+(-0.337351991987)*x[1]+(-0.33503967624)*x[2]**o+(-0.215395174878)*x[2]
        s[3,2,2,1]=(-0.191634883402)*x[0]**o+(-0.79050407204)*x[0]+(-0.881959919487)*x[1]**o+(0.962597211649)*x[1]+(0.602181506788)*x[2]**o+(-0.430152854642)*x[2]
        s[3,3,0,0]=(-0.458551188388)*x[0]**o+(0.62195840207)*x[0]+(0.646839412108)*x[1]**o+(-0.597989589884)*x[1]+(-0.160578706146)*x[2]**o+(0.885101198188)*x[2]
        s[3,3,0,1]=(-0.806015868617)*x[0]**o+(0.0617182982024)*x[0]+(0.296661747576)*x[1]**o+(0.381687206096)*x[1]+(0.954310501938)*x[2]**o+(0.112880684912)*x[2]
        s[3,3,1,0]=(0.485202154902)*x[0]**o+(0.528514725346)*x[0]+(0.550444453321)*x[1]**o+(-0.816313566191)*x[1]+(0.945151247472)*x[2]**o+(-0.201955849739)*x[2]
        s[3,3,1,1]=(0.245157893191)*x[0]**o+(-0.281606458201)*x[0]+(-0.312808561115)*x[1]**o+(0.185786713188)*x[1]+(0.996302657312)*x[2]**o+(-0.227884060488)*x[2]
        s[3,3,2,0]=(0.297297617074)*x[0]**o+(0.156994991329)*x[0]+(-0.403988935033)*x[1]**o+(-0.0894931373916)*x[1]+(-0.979214022227)*x[2]**o+(0.287684161706)*x[2]
        s[3,3,2,1]=(-0.287794860211)*x[0]**o+(0.169671378363)*x[0]+(0.0172804553692)*x[1]**o+(-0.973449618671)*x[1]+(0.0652371831894)*x[2]**o+(0.775978144983)*x[2]
        s[3,4,0,0]=(0.54203905332)*x[0]**o+(0.66731094672)*x[0]+(-0.865207826406)*x[1]**o+(0.466596502355)*x[1]+(-0.45336027189)*x[2]**o+(-0.266404323774)*x[2]
        s[3,4,0,1]=(-0.0264326659602)*x[0]**o+(0.102421850054)*x[0]+(-0.672364919167)*x[1]**o+(-0.461816126462)*x[1]+(0.11436848926)*x[2]**o+(-0.78486118431)*x[2]
        s[3,4,1,0]=(-0.941616359303)*x[0]**o+(-0.0519153474588)*x[0]+(0.8597678151)*x[1]**o+(0.164278414756)*x[1]+(0.496859863828)*x[2]**o+(-0.413785115519)*x[2]
        s[3,4,1,1]=(-0.726802133704)*x[0]**o+(0.685989768699)*x[0]+(0.499728992214)*x[1]**o+(0.675799006108)*x[1]+(0.205370350943)*x[2]**o+(0.983979039059)*x[2]
        s[3,4,2,0]=(-0.121723702011)*x[0]**o+(0.350456030225)*x[0]+(0.211968006265)*x[1]**o+(-0.960265509868)*x[1]+(-0.593090834374)*x[2]**o+(-0.631506211212)*x[2]
        s[3,4,2,1]=(0.279791655852)*x[0]**o+(0.236927673275)*x[0]+(0.84188823482)*x[1]**o+(0.208410505206)*x[1]+(0.392354982825)*x[2]**o+(0.671826571516)*x[2]
        ref[0,0,0,0]=(-0.419262547354)*x_ref[0]**o+(-0.607792358178)*x_ref[0]+(0.539129711536)*x_ref[1]**o+(-0.954002022975)*x_ref[1]+(0.52016184412)*x_ref[2]**o+(-0.905599414589)*x_ref[2]
        ref[0,0,0,1]=(0.552249219683)*x_ref[0]**o+(0.0747535682119)*x_ref[0]+(-0.0550858389381)*x_ref[1]**o+(-0.700756838491)*x_ref[1]+(-0.62569620016)*x_ref[2]**o+(-0.557287205831)*x_ref[2]
        ref[0,0,1,0]=(-0.84674266979)*x_ref[0]**o+(-0.266089976589)*x_ref[0]+(-0.994821395425)*x_ref[1]**o+(-0.567614298529)*x_ref[1]+(-0.814756268883)*x_ref[2]**o+(-0.524622381582)*x_ref[2]
        ref[0,0,1,1]=(0.160195104873)*x_ref[0]**o+(0.161596910068)*x_ref[0]+(-0.0404024431157)*x_ref[1]**o+(0.299140544701)*x_ref[1]+(0.479120924271)*x_ref[2]**o+(0.375793387197)*x_ref[2]
        ref[0,0,2,0]=(0.619011883068)*x_ref[0]**o+(0.453196321806)*x_ref[0]+(0.596546848845)*x_ref[1]**o+(-0.148655559308)*x_ref[1]+(-0.276643963879)*x_ref[2]**o+(-0.556233199969)*x_ref[2]
        ref[0,0,2,1]=(-0.44936519278)*x_ref[0]**o+(0.279187353573)*x_ref[0]+(-0.626699833624)*x_ref[1]**o+(0.172543451997)*x_ref[1]+(0.324942859052)*x_ref[2]**o+(0.227082355579)*x_ref[2]
        ref[0,1,0,0]=(0.348983572344)*x_ref[0]**o+(-0.692310510135)*x_ref[0]+(0.127542726356)*x_ref[1]**o+(-0.719693998448)*x_ref[1]+(0.190533765375)*x_ref[2]**o+(-0.44411272509)*x_ref[2]
        ref[0,1,0,1]=(0.755193842627)*x_ref[0]**o+(-0.0380801278139)*x_ref[0]+(-0.416973165389)*x_ref[1]**o+(-0.407050428042)*x_ref[1]+(-0.803402606403)*x_ref[2]**o+(0.860761604165)*x_ref[2]
        ref[0,1,1,0]=(0.903125811803)*x_ref[0]**o+(-0.370434864257)*x_ref[0]+(-0.810880082994)*x_ref[1]**o+(0.214166691855)*x_ref[1]+(-0.62621693665)*x_ref[2]**o+(-0.043360368073)*x_ref[2]
        ref[0,1,1,1]=(-0.442516790862)*x_ref[0]**o+(0.912139701807)*x_ref[0]+(0.871611995743)*x_ref[1]**o+(0.0291580448949)*x_ref[1]+(-0.489427959855)*x_ref[2]**o+(-0.950170294127)*x_ref[2]
        ref[0,1,2,0]=(-0.0867645917569)*x_ref[0]**o+(0.556462347924)*x_ref[0]+(-0.816554443299)*x_ref[1]**o+(0.818773437796)*x_ref[1]+(0.073905470924)*x_ref[2]**o+(-0.714497094507)*x_ref[2]
        ref[0,1,2,1]=(0.699491100428)*x_ref[0]**o+(-0.200275804269)*x_ref[0]+(0.994316913102)*x_ref[1]**o+(0.832890328003)*x_ref[1]+(0.385381865669)*x_ref[2]**o+(0.153056606745)*x_ref[2]
        ref[0,2,0,0]=(0.612319496073)*x_ref[0]**o+(-0.301991428044)*x_ref[0]+(-0.536866971844)*x_ref[1]**o+(-0.375311119699)*x_ref[1]+(0.733545441876)*x_ref[2]**o+(0.0177187199528)*x_ref[2]
        ref[0,2,0,1]=(0.358991602948)*x_ref[0]**o+(-0.862573091669)*x_ref[0]+(0.464891723814)*x_ref[1]**o+(0.602366461601)*x_ref[1]+(-0.668916509347)*x_ref[2]**o+(0.76048108017)*x_ref[2]
        ref[0,2,1,0]=(0.188195665573)*x_ref[0]**o+(-0.927407688437)*x_ref[0]+(-0.473014301986)*x_ref[1]**o+(-0.775114885201)*x_ref[1]+(-0.453403342886)*x_ref[2]**o+(0.006776981492)*x_ref[2]
        ref[0,2,1,1]=(-0.700181448543)*x_ref[0]**o+(-0.289180138599)*x_ref[0]+(-0.867022883968)*x_ref[1]**o+(-0.359339607516)*x_ref[1]+(-0.809558496152)*x_ref[2]**o+(-0.576358976651)*x_ref[2]
        ref[0,2,2,0]=(-0.557333080207)*x_ref[0]**o+(0.724962853865)*x_ref[0]+(0.995041226791)*x_ref[1]**o+(-0.610460710065)*x_ref[1]+(0.620016428769)*x_ref[2]**o+(-0.449182254452)*x_ref[2]
        ref[0,2,2,1]=(-0.937207840292)*x_ref[0]**o+(-0.66582207891)*x_ref[0]+(0.0951881913165)*x_ref[1]**o+(-0.0770694628972)*x_ref[1]+(0.034224640398)*x_ref[2]**o+(0.652057466224)*x_ref[2]
        ref[0,3,0,0]=(-0.0236750483037)*x_ref[0]**o+(0.956130793675)*x_ref[0]+(-0.589425964387)*x_ref[1]**o+(-0.204770941511)*x_ref[1]+(0.695255730164)*x_ref[2]**o+(-0.796222694557)*x_ref[2]
        ref[0,3,0,1]=(0.138589818728)*x_ref[0]**o+(0.493237559501)*x_ref[0]+(0.139281181588)*x_ref[1]**o+(-0.974201990591)*x_ref[1]+(-0.15401887402)*x_ref[2]**o+(0.23794136041)*x_ref[2]
        ref[0,3,1,0]=(-0.332886758751)*x_ref[0]**o+(-0.231549718295)*x_ref[0]+(0.581536450397)*x_ref[1]**o+(0.790063092494)*x_ref[1]+(0.358814129932)*x_ref[2]**o+(-0.110547525643)*x_ref[2]
        ref[0,3,1,1]=(0.122775146487)*x_ref[0]**o+(-0.00818530079912)*x_ref[0]+(0.0510575666357)*x_ref[1]**o+(-0.96911992662)*x_ref[1]+(-0.820269176326)*x_ref[2]**o+(0.875945222846)*x_ref[2]
        ref[0,3,2,0]=(0.761874596666)*x_ref[0]**o+(0.621369218231)*x_ref[0]+(0.960271412263)*x_ref[1]**o+(-0.808832785376)*x_ref[1]+(0.981428454481)*x_ref[2]**o+(0.393331122254)*x_ref[2]
        ref[0,3,2,1]=(-0.646477042209)*x_ref[0]**o+(-0.26169646294)*x_ref[0]+(0.992265690785)*x_ref[1]**o+(-0.678558332435)*x_ref[1]+(-0.285531525349)*x_ref[2]**o+(-0.571647919822)*x_ref[2]
        ref[0,4,0,0]=(0.492930260015)*x_ref[0]**o+(-0.647489772768)*x_ref[0]+(-0.381225997009)*x_ref[1]**o+(0.560462598779)*x_ref[1]+(-0.63571677055)*x_ref[2]**o+(0.103213895387)*x_ref[2]
        ref[0,4,0,1]=(-0.0362476689653)*x_ref[0]**o+(-0.478992488756)*x_ref[0]+(0.544907192925)*x_ref[1]**o+(-0.678350370776)*x_ref[1]+(-0.0776363475051)*x_ref[2]**o+(-0.803140341684)*x_ref[2]
        ref[0,4,1,0]=(0.0787683344872)*x_ref[0]**o+(0.688990188782)*x_ref[0]+(0.257089245341)*x_ref[1]**o+(-0.648663765213)*x_ref[1]+(-0.140632312728)*x_ref[2]**o+(0.676209090063)*x_ref[2]
        ref[0,4,1,1]=(0.804994807205)*x_ref[0]**o+(0.74750498671)*x_ref[0]+(-0.00532761506025)*x_ref[1]**o+(-0.295700694165)*x_ref[1]+(-0.209175099115)*x_ref[2]**o+(-0.0126407798374)*x_ref[2]
        ref[0,4,2,0]=(-0.789160064467)*x_ref[0]**o+(0.332412023778)*x_ref[0]+(0.242860020473)*x_ref[1]**o+(0.0284658318544)*x_ref[1]+(-0.357559965972)*x_ref[2]**o+(0.57056582293)*x_ref[2]
        ref[0,4,2,1]=(-0.861027741096)*x_ref[0]**o+(0.545166238342)*x_ref[0]+(0.950975873381)*x_ref[1]**o+(-0.436709572856)*x_ref[1]+(0.93636977927)*x_ref[2]**o+(-0.405205657349)*x_ref[2]
        ref[1,0,0,0]=(0.47880791873)*x_ref[0]**o+(0.037486909403)*x_ref[0]+(0.732218957477)*x_ref[1]**o+(0.274811688271)*x_ref[1]+(0.0686323288694)*x_ref[2]**o+(-0.492347744678)*x_ref[2]
        ref[1,0,0,1]=(-0.618259414329)*x_ref[0]**o+(0.630883795613)*x_ref[0]+(0.269141377847)*x_ref[1]**o+(-0.490916952452)*x_ref[1]+(-0.333070678904)*x_ref[2]**o+(0.0484209401349)*x_ref[2]
        ref[1,0,1,0]=(-0.906059152197)*x_ref[0]**o+(0.0322820078282)*x_ref[0]+(-0.47807076381)*x_ref[1]**o+(0.0296380181906)*x_ref[1]+(0.553686905633)*x_ref[2]**o+(-0.408333784569)*x_ref[2]
        ref[1,0,1,1]=(0.282962689091)*x_ref[0]**o+(-0.11495842442)*x_ref[0]+(0.818142653998)*x_ref[1]**o+(0.563460324128)*x_ref[1]+(-0.993782402791)*x_ref[2]**o+(-0.0193690715193)*x_ref[2]
        ref[1,0,2,0]=(0.287962506263)*x_ref[0]**o+(-0.401293374286)*x_ref[0]+(0.173559459587)*x_ref[1]**o+(0.822343918838)*x_ref[1]+(0.888309195417)*x_ref[2]**o+(0.687878298228)*x_ref[2]
        ref[1,0,2,1]=(-0.926775125361)*x_ref[0]**o+(0.396396669159)*x_ref[0]+(0.632962652289)*x_ref[1]**o+(-0.915667885738)*x_ref[1]+(-0.31075960693)*x_ref[2]**o+(0.920135418244)*x_ref[2]
        ref[1,1,0,0]=(-0.0915500113579)*x_ref[0]**o+(0.489036843145)*x_ref[0]+(0.477982330324)*x_ref[1]**o+(0.703371804742)*x_ref[1]+(-0.471773636091)*x_ref[2]**o+(0.0665811350713)*x_ref[2]
        ref[1,1,0,1]=(0.525411480834)*x_ref[0]**o+(-0.369843393433)*x_ref[0]+(-0.7770705966)*x_ref[1]**o+(0.534153802448)*x_ref[1]+(0.309828458492)*x_ref[2]**o+(0.627690678711)*x_ref[2]
        ref[1,1,1,0]=(0.149039257889)*x_ref[0]**o+(-0.0585283560518)*x_ref[0]+(-0.0505893024858)*x_ref[1]**o+(0.677281229531)*x_ref[1]+(-0.451950028252)*x_ref[2]**o+(0.356600216864)*x_ref[2]
        ref[1,1,1,1]=(-0.528237952467)*x_ref[0]**o+(0.533554438129)*x_ref[0]+(0.741981923136)*x_ref[1]**o+(-0.596922185095)*x_ref[1]+(0.50521173977)*x_ref[2]**o+(-0.247100318829)*x_ref[2]
        ref[1,1,2,0]=(0.667977944954)*x_ref[0]**o+(0.306325804541)*x_ref[0]+(-0.654643365837)*x_ref[1]**o+(0.0176380527447)*x_ref[1]+(-0.218012920296)*x_ref[2]**o+(-0.181740681902)*x_ref[2]
        ref[1,1,2,1]=(0.440430981908)*x_ref[0]**o+(0.994608754477)*x_ref[0]+(0.114426402744)*x_ref[1]**o+(-0.896222828258)*x_ref[1]+(0.997911423594)*x_ref[2]**o+(0.648328801403)*x_ref[2]
        ref[1,2,0,0]=(0.397328767103)*x_ref[0]**o+(0.780110953188)*x_ref[0]+(0.453955649367)*x_ref[1]**o+(0.953839185994)*x_ref[1]+(0.995536671917)*x_ref[2]**o+(0.177278490615)*x_ref[2]
        ref[1,2,0,1]=(-0.154393442621)*x_ref[0]**o+(-0.100267841021)*x_ref[0]+(-0.781347482738)*x_ref[1]**o+(0.391441261298)*x_ref[1]+(0.469832428629)*x_ref[2]**o+(0.608531800159)*x_ref[2]
        ref[1,2,1,0]=(-0.534167310466)*x_ref[0]**o+(0.414564904301)*x_ref[0]+(-0.0674568638795)*x_ref[1]**o+(0.577721681963)*x_ref[1]+(-0.00750850180411)*x_ref[2]**o+(0.0817658166565)*x_ref[2]
        ref[1,2,1,1]=(-0.529547188418)*x_ref[0]**o+(0.267917121456)*x_ref[0]+(0.372449729423)*x_ref[1]**o+(0.887072823755)*x_ref[1]+(0.887821539758)*x_ref[2]**o+(0.249771090594)*x_ref[2]
        ref[1,2,2,0]=(0.949056994737)*x_ref[0]**o+(-0.174523839411)*x_ref[0]+(-0.646085737348)*x_ref[1]**o+(0.449358235114)*x_ref[1]+(-0.970748494721)*x_ref[2]**o+(-0.410800591341)*x_ref[2]
        ref[1,2,2,1]=(0.879006069022)*x_ref[0]**o+(-0.510106021113)*x_ref[0]+(0.0203703221469)*x_ref[1]**o+(-0.844338135123)*x_ref[1]+(0.752168319634)*x_ref[2]**o+(0.158760999046)*x_ref[2]
        ref[1,3,0,0]=(0.845321813876)*x_ref[0]**o+(0.405224606418)*x_ref[0]+(-0.89159606951)*x_ref[1]**o+(-0.949499370695)*x_ref[1]+(0.501499536159)*x_ref[2]**o+(-0.869583954204)*x_ref[2]
        ref[1,3,0,1]=(0.0887248259912)*x_ref[0]**o+(0.232925681778)*x_ref[0]+(0.0315176777236)*x_ref[1]**o+(0.00568107770424)*x_ref[1]+(-0.821963787641)*x_ref[2]**o+(-0.634962715535)*x_ref[2]
        ref[1,3,1,0]=(0.546210053059)*x_ref[0]**o+(-0.688171108228)*x_ref[0]+(-0.699621490343)*x_ref[1]**o+(-0.850445270177)*x_ref[1]+(-0.725131419114)*x_ref[2]**o+(0.484336843024)*x_ref[2]
        ref[1,3,1,1]=(0.278594806693)*x_ref[0]**o+(-0.266710093152)*x_ref[0]+(-0.151515190688)*x_ref[1]**o+(-0.705005965054)*x_ref[1]+(0.811008835405)*x_ref[2]**o+(-0.342637057553)*x_ref[2]
        ref[1,3,2,0]=(0.4748294578)*x_ref[0]**o+(-0.0741346092747)*x_ref[0]+(0.148026337115)*x_ref[1]**o+(0.0252686427245)*x_ref[1]+(-0.479120476874)*x_ref[2]**o+(0.0855008343792)*x_ref[2]
        ref[1,3,2,1]=(-0.337007818128)*x_ref[0]**o+(-0.690464368815)*x_ref[0]+(0.481925609432)*x_ref[1]**o+(-0.0981705702921)*x_ref[1]+(0.19218353071)*x_ref[2]**o+(0.988337158745)*x_ref[2]
        ref[1,4,0,0]=(-0.757351506535)*x_ref[0]**o+(-0.824203825425)*x_ref[0]+(-0.998647750787)*x_ref[1]**o+(-0.111654698732)*x_ref[1]+(-0.713287873387)*x_ref[2]**o+(0.779671510732)*x_ref[2]
        ref[1,4,0,1]=(-0.756370998169)*x_ref[0]**o+(0.826863767951)*x_ref[0]+(0.358310449165)*x_ref[1]**o+(0.882430292099)*x_ref[1]+(0.880156909975)*x_ref[2]**o+(0.360411043274)*x_ref[2]
        ref[1,4,1,0]=(0.226080011536)*x_ref[0]**o+(-0.740536947501)*x_ref[0]+(-0.650527457572)*x_ref[1]**o+(-0.890015156497)*x_ref[1]+(0.688973071404)*x_ref[2]**o+(0.29237820526)*x_ref[2]
        ref[1,4,1,1]=(0.574015626188)*x_ref[0]**o+(-0.675641095131)*x_ref[0]+(0.814289365677)*x_ref[1]**o+(-0.625021588943)*x_ref[1]+(-0.862079110641)*x_ref[2]**o+(0.0363949889967)*x_ref[2]
        ref[1,4,2,0]=(0.126659193582)*x_ref[0]**o+(0.16976937315)*x_ref[0]+(0.18822384431)*x_ref[1]**o+(0.546082142359)*x_ref[1]+(0.551365996908)*x_ref[2]**o+(0.395213783677)*x_ref[2]
        ref[1,4,2,1]=(-0.292436267133)*x_ref[0]**o+(-0.442787521929)*x_ref[0]+(-0.310488595304)*x_ref[1]**o+(0.0918858003303)*x_ref[1]+(-0.986695921261)*x_ref[2]**o+(-0.52979957068)*x_ref[2]
        ref[2,0,0,0]=(0.396669972067)*x_ref[0]**o+(-0.711415400722)*x_ref[0]+(-0.717006050735)*x_ref[1]**o+(-0.0935757817421)*x_ref[1]+(0.743477556176)*x_ref[2]**o+(0.823967976857)*x_ref[2]
        ref[2,0,0,1]=(0.488395228667)*x_ref[0]**o+(0.230144035899)*x_ref[0]+(0.125851877066)*x_ref[1]**o+(0.0536284998009)*x_ref[1]+(-0.0114515128334)*x_ref[2]**o+(-0.494039400348)*x_ref[2]
        ref[2,0,1,0]=(0.900615108208)*x_ref[0]**o+(0.432765237128)*x_ref[0]+(0.981213121806)*x_ref[1]**o+(-0.116934702769)*x_ref[1]+(-0.612499086627)*x_ref[2]**o+(-0.365572411348)*x_ref[2]
        ref[2,0,1,1]=(0.0342670705338)*x_ref[0]**o+(0.54954404771)*x_ref[0]+(0.428653302401)*x_ref[1]**o+(0.537787906346)*x_ref[1]+(-0.64287424457)*x_ref[2]**o+(-0.173877644259)*x_ref[2]
        ref[2,0,2,0]=(-0.0510871045727)*x_ref[0]**o+(-0.908007271573)*x_ref[0]+(0.00194326164542)*x_ref[1]**o+(0.466243299714)*x_ref[1]+(-0.433962263339)*x_ref[2]**o+(0.874724229969)*x_ref[2]
        ref[2,0,2,1]=(0.666308090297)*x_ref[0]**o+(0.933188439742)*x_ref[0]+(-0.0479445062129)*x_ref[1]**o+(0.0747806162498)*x_ref[1]+(0.911844507192)*x_ref[2]**o+(-0.934538480024)*x_ref[2]
        ref[2,1,0,0]=(-0.516290157366)*x_ref[0]**o+(0.437519025532)*x_ref[0]+(-0.170999588789)*x_ref[1]**o+(-0.934096554106)*x_ref[1]+(0.88575800375)*x_ref[2]**o+(0.636966058116)*x_ref[2]
        ref[2,1,0,1]=(0.0172669412005)*x_ref[0]**o+(-0.867857912246)*x_ref[0]+(-0.972747832278)*x_ref[1]**o+(-0.780969292213)*x_ref[1]+(-0.418154913379)*x_ref[2]**o+(0.805902597544)*x_ref[2]
        ref[2,1,1,0]=(-0.964006922451)*x_ref[0]**o+(0.236797578253)*x_ref[0]+(-0.78802971076)*x_ref[1]**o+(0.704165381448)*x_ref[1]+(0.00112348607329)*x_ref[2]**o+(-0.689065853536)*x_ref[2]
        ref[2,1,1,1]=(-0.722392335335)*x_ref[0]**o+(-0.120018753634)*x_ref[0]+(0.216541173904)*x_ref[1]**o+(-0.374710223309)*x_ref[1]+(0.532158723682)*x_ref[2]**o+(0.464637663705)*x_ref[2]
        ref[2,1,2,0]=(-0.183637646888)*x_ref[0]**o+(-0.0680674034088)*x_ref[0]+(-0.114129008735)*x_ref[1]**o+(-0.78666547603)*x_ref[1]+(0.0956689943304)*x_ref[2]**o+(0.743296994432)*x_ref[2]
        ref[2,1,2,1]=(0.611382565923)*x_ref[0]**o+(-0.768682259494)*x_ref[0]+(-0.540943249593)*x_ref[1]**o+(0.753523431999)*x_ref[1]+(0.610689642267)*x_ref[2]**o+(0.0122811249496)*x_ref[2]
        ref[2,2,0,0]=(-0.789273374112)*x_ref[0]**o+(0.0977540682694)*x_ref[0]+(-0.829719260296)*x_ref[1]**o+(0.824773509135)*x_ref[1]+(0.216669854484)*x_ref[2]**o+(-0.764947843182)*x_ref[2]
        ref[2,2,0,1]=(-0.340784170043)*x_ref[0]**o+(-0.267437208067)*x_ref[0]+(-0.430973997706)*x_ref[1]**o+(-0.0846177320866)*x_ref[1]+(0.688599546971)*x_ref[2]**o+(0.265513958383)*x_ref[2]
        ref[2,2,1,0]=(-0.237677411545)*x_ref[0]**o+(-0.792144835449)*x_ref[0]+(-0.435763499999)*x_ref[1]**o+(0.682920743804)*x_ref[1]+(0.116459199169)*x_ref[2]**o+(0.254527225097)*x_ref[2]
        ref[2,2,1,1]=(0.342906428734)*x_ref[0]**o+(0.206470310722)*x_ref[0]+(-0.352429884648)*x_ref[1]**o+(-0.168905312607)*x_ref[1]+(-0.105529665095)*x_ref[2]**o+(0.199873788005)*x_ref[2]
        ref[2,2,2,0]=(0.677855511296)*x_ref[0]**o+(0.983953993003)*x_ref[0]+(-0.64997591911)*x_ref[1]**o+(-0.629221981454)*x_ref[1]+(0.990621105566)*x_ref[2]**o+(0.785766133934)*x_ref[2]
        ref[2,2,2,1]=(-0.570482617922)*x_ref[0]**o+(-0.151066051792)*x_ref[0]+(-0.511453364326)*x_ref[1]**o+(0.128824691419)*x_ref[1]+(-0.520127324328)*x_ref[2]**o+(-0.157450052117)*x_ref[2]
        ref[2,3,0,0]=(-0.423827660795)*x_ref[0]**o+(-0.815943363397)*x_ref[0]+(-0.636070023986)*x_ref[1]**o+(0.251327239067)*x_ref[1]+(-0.401641739847)*x_ref[2]**o+(0.130280903755)*x_ref[2]
        ref[2,3,0,1]=(0.0411439121625)*x_ref[0]**o+(0.621609981629)*x_ref[0]+(-0.589059932668)*x_ref[1]**o+(0.543095349334)*x_ref[1]+(-0.0122421869863)*x_ref[2]**o+(-0.776924587359)*x_ref[2]
        ref[2,3,1,0]=(0.63252327363)*x_ref[0]**o+(0.275597518383)*x_ref[0]+(0.832198526441)*x_ref[1]**o+(-0.92855918951)*x_ref[1]+(0.490343890624)*x_ref[2]**o+(-0.412580022625)*x_ref[2]
        ref[2,3,1,1]=(-0.560907436774)*x_ref[0]**o+(0.413956414518)*x_ref[0]+(-0.849736975879)*x_ref[1]**o+(0.167861053833)*x_ref[1]+(-0.316526371498)*x_ref[2]**o+(-0.431245755915)*x_ref[2]
        ref[2,3,2,0]=(0.883757549647)*x_ref[0]**o+(-0.614133317417)*x_ref[0]+(0.923442235162)*x_ref[1]**o+(0.483998522495)*x_ref[1]+(0.649003328717)*x_ref[2]**o+(-0.320181801832)*x_ref[2]
        ref[2,3,2,1]=(0.181982086686)*x_ref[0]**o+(-0.88761330228)*x_ref[0]+(-0.752108545831)*x_ref[1]**o+(0.105681418232)*x_ref[1]+(0.211053107745)*x_ref[2]**o+(-0.840650090772)*x_ref[2]
        ref[2,4,0,0]=(0.774235220873)*x_ref[0]**o+(-0.394430371883)*x_ref[0]+(-0.82551531411)*x_ref[1]**o+(-0.397374538567)*x_ref[1]+(0.51757884795)*x_ref[2]**o+(-0.878919341817)*x_ref[2]
        ref[2,4,0,1]=(-0.282445913776)*x_ref[0]**o+(-0.252614209563)*x_ref[0]+(-0.66810962709)*x_ref[1]**o+(0.912660025157)*x_ref[1]+(0.786200789061)*x_ref[2]**o+(-0.271329907892)*x_ref[2]
        ref[2,4,1,0]=(-0.415061697718)*x_ref[0]**o+(-0.577077815899)*x_ref[0]+(-0.479115469184)*x_ref[1]**o+(0.285424287206)*x_ref[1]+(0.580737671113)*x_ref[2]**o+(0.345716282261)*x_ref[2]
        ref[2,4,1,1]=(0.103116885191)*x_ref[0]**o+(0.472934825259)*x_ref[0]+(0.192747945705)*x_ref[1]**o+(0.106949097334)*x_ref[1]+(-0.109457290729)*x_ref[2]**o+(0.843345964596)*x_ref[2]
        ref[2,4,2,0]=(0.600281530422)*x_ref[0]**o+(-0.427230650838)*x_ref[0]+(-0.868150408946)*x_ref[1]**o+(-0.203647711262)*x_ref[1]+(-0.550477804035)*x_ref[2]**o+(0.415736214172)*x_ref[2]
        ref[2,4,2,1]=(0.739232217202)*x_ref[0]**o+(-0.476864848171)*x_ref[0]+(0.367627911492)*x_ref[1]**o+(0.105176582927)*x_ref[1]+(0.482113806667)*x_ref[2]**o+(0.782020005885)*x_ref[2]
        ref[3,0,0,0]=(0.938839141593)*x_ref[0]**o+(-0.655793939395)*x_ref[0]+(0.0224780413611)*x_ref[1]**o+(0.252535317383)*x_ref[1]+(-0.887022478757)*x_ref[2]**o+(-0.451912746232)*x_ref[2]
        ref[3,0,0,1]=(-0.196994797427)*x_ref[0]**o+(-0.222816920678)*x_ref[0]+(0.37131460044)*x_ref[1]**o+(0.124872956754)*x_ref[1]+(-0.872680499884)*x_ref[2]**o+(-0.490521736465)*x_ref[2]
        ref[3,0,1,0]=(-0.977712672434)*x_ref[0]**o+(-0.665877989129)*x_ref[0]+(-0.541103000349)*x_ref[1]**o+(-0.527523471522)*x_ref[1]+(0.469063433285)*x_ref[2]**o+(-0.410680406273)*x_ref[2]
        ref[3,0,1,1]=(0.371612180735)*x_ref[0]**o+(-0.882397978651)*x_ref[0]+(-0.333119569273)*x_ref[1]**o+(0.0439599202688)*x_ref[1]+(-0.378698960758)*x_ref[2]**o+(0.373830212832)*x_ref[2]
        ref[3,0,2,0]=(0.13552136805)*x_ref[0]**o+(-0.347572305079)*x_ref[0]+(0.194173745453)*x_ref[1]**o+(-0.536374783086)*x_ref[1]+(-0.0928402883882)*x_ref[2]**o+(0.972386889251)*x_ref[2]
        ref[3,0,2,1]=(-0.673716579315)*x_ref[0]**o+(0.497683603572)*x_ref[0]+(0.0894501300871)*x_ref[1]**o+(-0.691905501083)*x_ref[1]+(-0.819373609736)*x_ref[2]**o+(0.876955815163)*x_ref[2]
        ref[3,1,0,0]=(-0.612778977625)*x_ref[0]**o+(0.898593519513)*x_ref[0]+(-0.579505051936)*x_ref[1]**o+(0.294087297529)*x_ref[1]+(0.713022674221)*x_ref[2]**o+(-0.0994065928128)*x_ref[2]
        ref[3,1,0,1]=(-0.141600251938)*x_ref[0]**o+(-0.677815682935)*x_ref[0]+(-0.408920459122)*x_ref[1]**o+(-0.723029003437)*x_ref[1]+(0.481057659884)*x_ref[2]**o+(0.427909771498)*x_ref[2]
        ref[3,1,1,0]=(-0.516486224771)*x_ref[0]**o+(0.526484181999)*x_ref[0]+(0.911793059151)*x_ref[1]**o+(0.409808964766)*x_ref[1]+(-0.413591542253)*x_ref[2]**o+(-0.759664326823)*x_ref[2]
        ref[3,1,1,1]=(0.905324377656)*x_ref[0]**o+(-0.208896172985)*x_ref[0]+(-0.422109371498)*x_ref[1]**o+(0.491883832748)*x_ref[1]+(-0.203413002012)*x_ref[2]**o+(0.406737219074)*x_ref[2]
        ref[3,1,2,0]=(-0.573692277755)*x_ref[0]**o+(0.895762539503)*x_ref[0]+(0.176538736347)*x_ref[1]**o+(-0.847640766515)*x_ref[1]+(0.5650742337)*x_ref[2]**o+(0.503165369442)*x_ref[2]
        ref[3,1,2,1]=(0.308521434074)*x_ref[0]**o+(0.616776036606)*x_ref[0]+(-0.405321909102)*x_ref[1]**o+(0.497367404288)*x_ref[1]+(-0.798636270652)*x_ref[2]**o+(-0.337423592684)*x_ref[2]
        ref[3,2,0,0]=(-0.394455434548)*x_ref[0]**o+(-0.292212771133)*x_ref[0]+(-0.941613233015)*x_ref[1]**o+(-0.404522244202)*x_ref[1]+(-0.0112893909573)*x_ref[2]**o+(-0.9706166943)*x_ref[2]
        ref[3,2,0,1]=(0.627774279675)*x_ref[0]**o+(-0.645334179444)*x_ref[0]+(0.691389142649)*x_ref[1]**o+(0.763379715146)*x_ref[1]+(0.610766030943)*x_ref[2]**o+(-0.786459853785)*x_ref[2]
        ref[3,2,1,0]=(-0.0101413740932)*x_ref[0]**o+(-0.0970748510013)*x_ref[0]+(-0.52181886114)*x_ref[1]**o+(-0.612494396935)*x_ref[1]+(0.941988727244)*x_ref[2]**o+(0.0361447091017)*x_ref[2]
        ref[3,2,1,1]=(-0.0943447427863)*x_ref[0]**o+(0.708735490064)*x_ref[0]+(-0.352853510872)*x_ref[1]**o+(-0.545593672805)*x_ref[1]+(-0.0106851492926)*x_ref[2]**o+(0.712976402827)*x_ref[2]
        ref[3,2,2,0]=(-0.522341854545)*x_ref[0]**o+(0.320365455548)*x_ref[0]+(-0.111682800076)*x_ref[1]**o+(-0.337351991987)*x_ref[1]+(-0.33503967624)*x_ref[2]**o+(-0.215395174878)*x_ref[2]
        ref[3,2,2,1]=(-0.191634883402)*x_ref[0]**o+(-0.79050407204)*x_ref[0]+(-0.881959919487)*x_ref[1]**o+(0.962597211649)*x_ref[1]+(0.602181506788)*x_ref[2]**o+(-0.430152854642)*x_ref[2]
        ref[3,3,0,0]=(-0.458551188388)*x_ref[0]**o+(0.62195840207)*x_ref[0]+(0.646839412108)*x_ref[1]**o+(-0.597989589884)*x_ref[1]+(-0.160578706146)*x_ref[2]**o+(0.885101198188)*x_ref[2]
        ref[3,3,0,1]=(-0.806015868617)*x_ref[0]**o+(0.0617182982024)*x_ref[0]+(0.296661747576)*x_ref[1]**o+(0.381687206096)*x_ref[1]+(0.954310501938)*x_ref[2]**o+(0.112880684912)*x_ref[2]
        ref[3,3,1,0]=(0.485202154902)*x_ref[0]**o+(0.528514725346)*x_ref[0]+(0.550444453321)*x_ref[1]**o+(-0.816313566191)*x_ref[1]+(0.945151247472)*x_ref[2]**o+(-0.201955849739)*x_ref[2]
        ref[3,3,1,1]=(0.245157893191)*x_ref[0]**o+(-0.281606458201)*x_ref[0]+(-0.312808561115)*x_ref[1]**o+(0.185786713188)*x_ref[1]+(0.996302657312)*x_ref[2]**o+(-0.227884060488)*x_ref[2]
        ref[3,3,2,0]=(0.297297617074)*x_ref[0]**o+(0.156994991329)*x_ref[0]+(-0.403988935033)*x_ref[1]**o+(-0.0894931373916)*x_ref[1]+(-0.979214022227)*x_ref[2]**o+(0.287684161706)*x_ref[2]
        ref[3,3,2,1]=(-0.287794860211)*x_ref[0]**o+(0.169671378363)*x_ref[0]+(0.0172804553692)*x_ref[1]**o+(-0.973449618671)*x_ref[1]+(0.0652371831894)*x_ref[2]**o+(0.775978144983)*x_ref[2]
        ref[3,4,0,0]=(0.54203905332)*x_ref[0]**o+(0.66731094672)*x_ref[0]+(-0.865207826406)*x_ref[1]**o+(0.466596502355)*x_ref[1]+(-0.45336027189)*x_ref[2]**o+(-0.266404323774)*x_ref[2]
        ref[3,4,0,1]=(-0.0264326659602)*x_ref[0]**o+(0.102421850054)*x_ref[0]+(-0.672364919167)*x_ref[1]**o+(-0.461816126462)*x_ref[1]+(0.11436848926)*x_ref[2]**o+(-0.78486118431)*x_ref[2]
        ref[3,4,1,0]=(-0.941616359303)*x_ref[0]**o+(-0.0519153474588)*x_ref[0]+(0.8597678151)*x_ref[1]**o+(0.164278414756)*x_ref[1]+(0.496859863828)*x_ref[2]**o+(-0.413785115519)*x_ref[2]
        ref[3,4,1,1]=(-0.726802133704)*x_ref[0]**o+(0.685989768699)*x_ref[0]+(0.499728992214)*x_ref[1]**o+(0.675799006108)*x_ref[1]+(0.205370350943)*x_ref[2]**o+(0.983979039059)*x_ref[2]
        ref[3,4,2,0]=(-0.121723702011)*x_ref[0]**o+(0.350456030225)*x_ref[0]+(0.211968006265)*x_ref[1]**o+(-0.960265509868)*x_ref[1]+(-0.593090834374)*x_ref[2]**o+(-0.631506211212)*x_ref[2]
        ref[3,4,2,1]=(0.279791655852)*x_ref[0]**o+(0.236927673275)*x_ref[0]+(0.84188823482)*x_ref[1]**o+(0.208410505206)*x_ref[1]+(0.392354982825)*x_ref[2]**o+(0.671826571516)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromData_ContinuousFunction_rank0(self):
      """
      tests interpolate for rank 0 Data onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ContinuousFunction onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(),w)
      ref=Data(0,(),w_ref)
      if dim==2:
        arg=(-0.0071553637759)*x[0]**o+(0.357615094637)*x[0]+(0.980385970972)*x[1]**o+(0.423505273491)*x[1]
        ref=(-0.0071553637759)*x_ref[0]**o+(0.357615094637)*x_ref[0]+(0.980385970972)*x_ref[1]**o+(0.423505273491)*x_ref[1]
      else:
        arg=(0.546251547514)*x[0]**o+(-0.530405447164)*x[0]+(-0.901465213708)*x[1]**o+(-0.986019835171)*x[1]+(-0.731836172089)*x[2]**o+(0.377317674082)*x[2]
        ref=(0.546251547514)*x_ref[0]**o+(-0.530405447164)*x_ref[0]+(-0.901465213708)*x_ref[1]**o+(-0.986019835171)*x_ref[1]+(-0.731836172089)*x_ref[2]**o+(0.377317674082)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromData_ContinuousFunction_rank1(self):
      """
      tests interpolate for rank 1 Data onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ContinuousFunction onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(2,),w)
      ref=Data(0,(2,),w_ref)
      if dim==2:
        arg[0]=(-0.138216822552)*x[0]**o+(0.908302047805)*x[0]+(-0.247600939209)*x[1]**o+(0.776239000167)*x[1]
        arg[1]=(-0.577926899073)*x[0]**o+(0.714263102346)*x[0]+(-0.343808625475)*x[1]**o+(0.786295563423)*x[1]
        ref[0]=(-0.138216822552)*x_ref[0]**o+(0.908302047805)*x_ref[0]+(-0.247600939209)*x_ref[1]**o+(0.776239000167)*x_ref[1]
        ref[1]=(-0.577926899073)*x_ref[0]**o+(0.714263102346)*x_ref[0]+(-0.343808625475)*x_ref[1]**o+(0.786295563423)*x_ref[1]
      else:
        arg[0]=(0.020232467504)*x[0]**o+(-0.0449727381653)*x[0]+(0.00132143537494)*x[1]**o+(-0.870292414095)*x[1]+(-0.926731511382)*x[2]**o+(-0.398915622332)*x[2]
        arg[1]=(0.203864545604)*x[0]**o+(0.150501673382)*x[0]+(0.132132290114)*x[1]**o+(-0.720617456803)*x[1]+(0.673909788445)*x[2]**o+(0.145376368499)*x[2]
        ref[0]=(0.020232467504)*x_ref[0]**o+(-0.0449727381653)*x_ref[0]+(0.00132143537494)*x_ref[1]**o+(-0.870292414095)*x_ref[1]+(-0.926731511382)*x_ref[2]**o+(-0.398915622332)*x_ref[2]
        ref[1]=(0.203864545604)*x_ref[0]**o+(0.150501673382)*x_ref[0]+(0.132132290114)*x_ref[1]**o+(-0.720617456803)*x_ref[1]+(0.673909788445)*x_ref[2]**o+(0.145376368499)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromData_ContinuousFunction_rank2(self):
      """
      tests interpolate for rank 2 Data onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ContinuousFunction onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5),w)
      ref=Data(0,(4, 5),w_ref)
      if dim==2:
        arg[0,0]=(-0.344685653483)*x[0]**o+(-0.618443209029)*x[0]+(-0.338246837583)*x[1]**o+(-0.709311881709)*x[1]
        arg[0,1]=(-0.176341370817)*x[0]**o+(-0.230030832268)*x[0]+(-0.335163038431)*x[1]**o+(0.278627093006)*x[1]
        arg[0,2]=(0.376270607923)*x[0]**o+(0.895822309578)*x[0]+(0.418316736671)*x[1]**o+(-0.951461687587)*x[1]
        arg[0,3]=(0.69925470532)*x[0]**o+(-0.829917904706)*x[0]+(0.60851481091)*x[1]**o+(-0.880924691692)*x[1]
        arg[0,4]=(0.537570501499)*x[0]**o+(0.584091505535)*x[0]+(-0.0045669307163)*x[1]**o+(0.162714723047)*x[1]
        arg[1,0]=(-0.244615538677)*x[0]**o+(-0.0168359545772)*x[0]+(-0.511097014251)*x[1]**o+(-0.476830089968)*x[1]
        arg[1,1]=(0.350953788855)*x[0]**o+(0.0859931352733)*x[0]+(-0.565664846934)*x[1]**o+(-0.230383210864)*x[1]
        arg[1,2]=(0.787537227138)*x[0]**o+(-0.97589028834)*x[0]+(0.978870857538)*x[1]**o+(0.436540016432)*x[1]
        arg[1,3]=(-0.806777674064)*x[0]**o+(-0.384971723467)*x[0]+(-0.423836322283)*x[1]**o+(0.470609842503)*x[1]
        arg[1,4]=(0.288422170494)*x[0]**o+(-0.467670464119)*x[0]+(-0.616952048135)*x[1]**o+(0.604831335817)*x[1]
        arg[2,0]=(0.0461969205986)*x[0]**o+(0.663083190578)*x[0]+(-0.673597938582)*x[1]**o+(0.217389040208)*x[1]
        arg[2,1]=(-0.32008443048)*x[0]**o+(-0.970854886309)*x[0]+(0.630220968872)*x[1]**o+(0.827824011117)*x[1]
        arg[2,2]=(-0.232867522682)*x[0]**o+(-0.17921445933)*x[0]+(-0.565289194319)*x[1]**o+(0.0251729876325)*x[1]
        arg[2,3]=(-0.862002880133)*x[0]**o+(0.64790108714)*x[0]+(-0.469428882373)*x[1]**o+(0.50581485158)*x[1]
        arg[2,4]=(-0.957389923449)*x[0]**o+(-0.716972488151)*x[0]+(0.848168933491)*x[1]**o+(0.78614576485)*x[1]
        arg[3,0]=(0.417890378265)*x[0]**o+(-0.128884271967)*x[0]+(0.981251819451)*x[1]**o+(0.837945499182)*x[1]
        arg[3,1]=(-0.296791907211)*x[0]**o+(0.913966659909)*x[0]+(0.0171079547828)*x[1]**o+(0.472922286638)*x[1]
        arg[3,2]=(-0.549504779354)*x[0]**o+(0.356989069435)*x[0]+(-0.0978494385269)*x[1]**o+(0.59354694599)*x[1]
        arg[3,3]=(0.0272055308458)*x[0]**o+(0.560184544705)*x[0]+(-0.825452907104)*x[1]**o+(0.701919616153)*x[1]
        arg[3,4]=(-0.33094753848)*x[0]**o+(0.877723269108)*x[0]+(-0.491915661514)*x[1]**o+(-0.98784807255)*x[1]
        ref[0,0]=(-0.344685653483)*x_ref[0]**o+(-0.618443209029)*x_ref[0]+(-0.338246837583)*x_ref[1]**o+(-0.709311881709)*x_ref[1]
        ref[0,1]=(-0.176341370817)*x_ref[0]**o+(-0.230030832268)*x_ref[0]+(-0.335163038431)*x_ref[1]**o+(0.278627093006)*x_ref[1]
        ref[0,2]=(0.376270607923)*x_ref[0]**o+(0.895822309578)*x_ref[0]+(0.418316736671)*x_ref[1]**o+(-0.951461687587)*x_ref[1]
        ref[0,3]=(0.69925470532)*x_ref[0]**o+(-0.829917904706)*x_ref[0]+(0.60851481091)*x_ref[1]**o+(-0.880924691692)*x_ref[1]
        ref[0,4]=(0.537570501499)*x_ref[0]**o+(0.584091505535)*x_ref[0]+(-0.0045669307163)*x_ref[1]**o+(0.162714723047)*x_ref[1]
        ref[1,0]=(-0.244615538677)*x_ref[0]**o+(-0.0168359545772)*x_ref[0]+(-0.511097014251)*x_ref[1]**o+(-0.476830089968)*x_ref[1]
        ref[1,1]=(0.350953788855)*x_ref[0]**o+(0.0859931352733)*x_ref[0]+(-0.565664846934)*x_ref[1]**o+(-0.230383210864)*x_ref[1]
        ref[1,2]=(0.787537227138)*x_ref[0]**o+(-0.97589028834)*x_ref[0]+(0.978870857538)*x_ref[1]**o+(0.436540016432)*x_ref[1]
        ref[1,3]=(-0.806777674064)*x_ref[0]**o+(-0.384971723467)*x_ref[0]+(-0.423836322283)*x_ref[1]**o+(0.470609842503)*x_ref[1]
        ref[1,4]=(0.288422170494)*x_ref[0]**o+(-0.467670464119)*x_ref[0]+(-0.616952048135)*x_ref[1]**o+(0.604831335817)*x_ref[1]
        ref[2,0]=(0.0461969205986)*x_ref[0]**o+(0.663083190578)*x_ref[0]+(-0.673597938582)*x_ref[1]**o+(0.217389040208)*x_ref[1]
        ref[2,1]=(-0.32008443048)*x_ref[0]**o+(-0.970854886309)*x_ref[0]+(0.630220968872)*x_ref[1]**o+(0.827824011117)*x_ref[1]
        ref[2,2]=(-0.232867522682)*x_ref[0]**o+(-0.17921445933)*x_ref[0]+(-0.565289194319)*x_ref[1]**o+(0.0251729876325)*x_ref[1]
        ref[2,3]=(-0.862002880133)*x_ref[0]**o+(0.64790108714)*x_ref[0]+(-0.469428882373)*x_ref[1]**o+(0.50581485158)*x_ref[1]
        ref[2,4]=(-0.957389923449)*x_ref[0]**o+(-0.716972488151)*x_ref[0]+(0.848168933491)*x_ref[1]**o+(0.78614576485)*x_ref[1]
        ref[3,0]=(0.417890378265)*x_ref[0]**o+(-0.128884271967)*x_ref[0]+(0.981251819451)*x_ref[1]**o+(0.837945499182)*x_ref[1]
        ref[3,1]=(-0.296791907211)*x_ref[0]**o+(0.913966659909)*x_ref[0]+(0.0171079547828)*x_ref[1]**o+(0.472922286638)*x_ref[1]
        ref[3,2]=(-0.549504779354)*x_ref[0]**o+(0.356989069435)*x_ref[0]+(-0.0978494385269)*x_ref[1]**o+(0.59354694599)*x_ref[1]
        ref[3,3]=(0.0272055308458)*x_ref[0]**o+(0.560184544705)*x_ref[0]+(-0.825452907104)*x_ref[1]**o+(0.701919616153)*x_ref[1]
        ref[3,4]=(-0.33094753848)*x_ref[0]**o+(0.877723269108)*x_ref[0]+(-0.491915661514)*x_ref[1]**o+(-0.98784807255)*x_ref[1]
      else:
        arg[0,0]=(-0.434721514594)*x[0]**o+(-0.28074158231)*x[0]+(0.102692621316)*x[1]**o+(0.399677492352)*x[1]+(0.39799987301)*x[2]**o+(-0.134547332796)*x[2]
        arg[0,1]=(-0.814630031471)*x[0]**o+(-0.0722898377573)*x[0]+(0.356831166041)*x[1]**o+(0.0556663518903)*x[1]+(-0.335764152958)*x[2]**o+(0.344564519072)*x[2]
        arg[0,2]=(-0.881698115197)*x[0]**o+(-0.030759288922)*x[0]+(-0.23126177081)*x[1]**o+(0.865011678258)*x[1]+(0.961867471932)*x[2]**o+(-0.04809978604)*x[2]
        arg[0,3]=(-0.931319970946)*x[0]**o+(0.807195614255)*x[0]+(0.116305510032)*x[1]**o+(0.337873586448)*x[1]+(0.523766276637)*x[2]**o+(-0.890502808117)*x[2]
        arg[0,4]=(0.952369561411)*x[0]**o+(0.518186569223)*x[0]+(0.184454818734)*x[1]**o+(-0.198618411156)*x[1]+(0.891317797589)*x[2]**o+(-0.312268228065)*x[2]
        arg[1,0]=(-0.770379397538)*x[0]**o+(0.11068893635)*x[0]+(0.435574518842)*x[1]**o+(-0.895094805657)*x[1]+(0.719886389608)*x[2]**o+(0.617254266829)*x[2]
        arg[1,1]=(-0.968566082536)*x[0]**o+(0.70331821608)*x[0]+(-0.256092697223)*x[1]**o+(0.214482838048)*x[1]+(-0.20817108493)*x[2]**o+(-0.741961694045)*x[2]
        arg[1,2]=(0.928194134464)*x[0]**o+(0.748278982508)*x[0]+(0.459001496702)*x[1]**o+(-0.333954496033)*x[1]+(0.0451659439418)*x[2]**o+(0.0355669781059)*x[2]
        arg[1,3]=(-0.285564661682)*x[0]**o+(-0.780972389596)*x[0]+(0.20442278849)*x[1]**o+(-0.425673038027)*x[1]+(-0.886021022849)*x[2]**o+(0.2231241391)*x[2]
        arg[1,4]=(0.48756511391)*x[0]**o+(0.904219456651)*x[0]+(-0.723245954496)*x[1]**o+(-0.876483077689)*x[1]+(0.454913273237)*x[2]**o+(0.907651543551)*x[2]
        arg[2,0]=(-0.490623674798)*x[0]**o+(0.179561344392)*x[0]+(-0.257796709265)*x[1]**o+(0.470216680739)*x[1]+(-0.334946805819)*x[2]**o+(-0.295308708488)*x[2]
        arg[2,1]=(-0.0753731003603)*x[0]**o+(0.848622766023)*x[0]+(-0.168762035291)*x[1]**o+(-0.132048839333)*x[1]+(0.229089231051)*x[2]**o+(0.280032186468)*x[2]
        arg[2,2]=(0.601432167354)*x[0]**o+(-0.0857688429764)*x[0]+(0.556763499853)*x[1]**o+(-0.0953568285359)*x[1]+(-0.396589175262)*x[2]**o+(0.437257052414)*x[2]
        arg[2,3]=(0.239710792478)*x[0]**o+(0.255863660922)*x[0]+(-0.860831814227)*x[1]**o+(0.416057968294)*x[1]+(0.284515440811)*x[2]**o+(-0.0737992548315)*x[2]
        arg[2,4]=(-0.86884373286)*x[0]**o+(0.306811019609)*x[0]+(0.955007347168)*x[1]**o+(0.731959544948)*x[1]+(0.326947842619)*x[2]**o+(0.430618227986)*x[2]
        arg[3,0]=(0.450341287007)*x[0]**o+(-0.0318463261779)*x[0]+(-0.755083793007)*x[1]**o+(0.427706480425)*x[1]+(0.224383931841)*x[2]**o+(-0.801200151542)*x[2]
        arg[3,1]=(-0.750405028813)*x[0]**o+(-0.531057162102)*x[0]+(0.120116357521)*x[1]**o+(-0.247640504917)*x[1]+(-0.360494371461)*x[2]**o+(-0.496404068337)*x[2]
        arg[3,2]=(-0.202775617449)*x[0]**o+(0.833337604045)*x[0]+(0.551285270602)*x[1]**o+(0.266630410234)*x[1]+(0.903041333927)*x[2]**o+(0.527233939028)*x[2]
        arg[3,3]=(0.12892683791)*x[0]**o+(-0.486726243104)*x[0]+(-0.340728397711)*x[1]**o+(-0.670692588507)*x[1]+(0.832250132781)*x[2]**o+(0.276897913372)*x[2]
        arg[3,4]=(0.839441639419)*x[0]**o+(0.362150671815)*x[0]+(0.728370029046)*x[1]**o+(0.149540296968)*x[1]+(-0.0418719598944)*x[2]**o+(-0.331805297414)*x[2]
        ref[0,0]=(-0.434721514594)*x_ref[0]**o+(-0.28074158231)*x_ref[0]+(0.102692621316)*x_ref[1]**o+(0.399677492352)*x_ref[1]+(0.39799987301)*x_ref[2]**o+(-0.134547332796)*x_ref[2]
        ref[0,1]=(-0.814630031471)*x_ref[0]**o+(-0.0722898377573)*x_ref[0]+(0.356831166041)*x_ref[1]**o+(0.0556663518903)*x_ref[1]+(-0.335764152958)*x_ref[2]**o+(0.344564519072)*x_ref[2]
        ref[0,2]=(-0.881698115197)*x_ref[0]**o+(-0.030759288922)*x_ref[0]+(-0.23126177081)*x_ref[1]**o+(0.865011678258)*x_ref[1]+(0.961867471932)*x_ref[2]**o+(-0.04809978604)*x_ref[2]
        ref[0,3]=(-0.931319970946)*x_ref[0]**o+(0.807195614255)*x_ref[0]+(0.116305510032)*x_ref[1]**o+(0.337873586448)*x_ref[1]+(0.523766276637)*x_ref[2]**o+(-0.890502808117)*x_ref[2]
        ref[0,4]=(0.952369561411)*x_ref[0]**o+(0.518186569223)*x_ref[0]+(0.184454818734)*x_ref[1]**o+(-0.198618411156)*x_ref[1]+(0.891317797589)*x_ref[2]**o+(-0.312268228065)*x_ref[2]
        ref[1,0]=(-0.770379397538)*x_ref[0]**o+(0.11068893635)*x_ref[0]+(0.435574518842)*x_ref[1]**o+(-0.895094805657)*x_ref[1]+(0.719886389608)*x_ref[2]**o+(0.617254266829)*x_ref[2]
        ref[1,1]=(-0.968566082536)*x_ref[0]**o+(0.70331821608)*x_ref[0]+(-0.256092697223)*x_ref[1]**o+(0.214482838048)*x_ref[1]+(-0.20817108493)*x_ref[2]**o+(-0.741961694045)*x_ref[2]
        ref[1,2]=(0.928194134464)*x_ref[0]**o+(0.748278982508)*x_ref[0]+(0.459001496702)*x_ref[1]**o+(-0.333954496033)*x_ref[1]+(0.0451659439418)*x_ref[2]**o+(0.0355669781059)*x_ref[2]
        ref[1,3]=(-0.285564661682)*x_ref[0]**o+(-0.780972389596)*x_ref[0]+(0.20442278849)*x_ref[1]**o+(-0.425673038027)*x_ref[1]+(-0.886021022849)*x_ref[2]**o+(0.2231241391)*x_ref[2]
        ref[1,4]=(0.48756511391)*x_ref[0]**o+(0.904219456651)*x_ref[0]+(-0.723245954496)*x_ref[1]**o+(-0.876483077689)*x_ref[1]+(0.454913273237)*x_ref[2]**o+(0.907651543551)*x_ref[2]
        ref[2,0]=(-0.490623674798)*x_ref[0]**o+(0.179561344392)*x_ref[0]+(-0.257796709265)*x_ref[1]**o+(0.470216680739)*x_ref[1]+(-0.334946805819)*x_ref[2]**o+(-0.295308708488)*x_ref[2]
        ref[2,1]=(-0.0753731003603)*x_ref[0]**o+(0.848622766023)*x_ref[0]+(-0.168762035291)*x_ref[1]**o+(-0.132048839333)*x_ref[1]+(0.229089231051)*x_ref[2]**o+(0.280032186468)*x_ref[2]
        ref[2,2]=(0.601432167354)*x_ref[0]**o+(-0.0857688429764)*x_ref[0]+(0.556763499853)*x_ref[1]**o+(-0.0953568285359)*x_ref[1]+(-0.396589175262)*x_ref[2]**o+(0.437257052414)*x_ref[2]
        ref[2,3]=(0.239710792478)*x_ref[0]**o+(0.255863660922)*x_ref[0]+(-0.860831814227)*x_ref[1]**o+(0.416057968294)*x_ref[1]+(0.284515440811)*x_ref[2]**o+(-0.0737992548315)*x_ref[2]
        ref[2,4]=(-0.86884373286)*x_ref[0]**o+(0.306811019609)*x_ref[0]+(0.955007347168)*x_ref[1]**o+(0.731959544948)*x_ref[1]+(0.326947842619)*x_ref[2]**o+(0.430618227986)*x_ref[2]
        ref[3,0]=(0.450341287007)*x_ref[0]**o+(-0.0318463261779)*x_ref[0]+(-0.755083793007)*x_ref[1]**o+(0.427706480425)*x_ref[1]+(0.224383931841)*x_ref[2]**o+(-0.801200151542)*x_ref[2]
        ref[3,1]=(-0.750405028813)*x_ref[0]**o+(-0.531057162102)*x_ref[0]+(0.120116357521)*x_ref[1]**o+(-0.247640504917)*x_ref[1]+(-0.360494371461)*x_ref[2]**o+(-0.496404068337)*x_ref[2]
        ref[3,2]=(-0.202775617449)*x_ref[0]**o+(0.833337604045)*x_ref[0]+(0.551285270602)*x_ref[1]**o+(0.266630410234)*x_ref[1]+(0.903041333927)*x_ref[2]**o+(0.527233939028)*x_ref[2]
        ref[3,3]=(0.12892683791)*x_ref[0]**o+(-0.486726243104)*x_ref[0]+(-0.340728397711)*x_ref[1]**o+(-0.670692588507)*x_ref[1]+(0.832250132781)*x_ref[2]**o+(0.276897913372)*x_ref[2]
        ref[3,4]=(0.839441639419)*x_ref[0]**o+(0.362150671815)*x_ref[0]+(0.728370029046)*x_ref[1]**o+(0.149540296968)*x_ref[1]+(-0.0418719598944)*x_ref[2]**o+(-0.331805297414)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromData_ContinuousFunction_rank3(self):
      """
      tests interpolate for rank 3 Data onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ContinuousFunction onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2),w_ref)
      if dim==2:
        arg[0,0,0]=(-0.475615092269)*x[0]**o+(-0.0893751722575)*x[0]+(0.595881277166)*x[1]**o+(0.397900173915)*x[1]
        arg[0,0,1]=(-0.552614805712)*x[0]**o+(0.14072435445)*x[0]+(0.290231845094)*x[1]**o+(0.573878059041)*x[1]
        arg[0,1,0]=(-0.226148312038)*x[0]**o+(-0.737524507359)*x[0]+(-0.704362709293)*x[1]**o+(-0.197223346266)*x[1]
        arg[0,1,1]=(0.844313529642)*x[0]**o+(-0.948514286846)*x[0]+(0.383754990882)*x[1]**o+(0.629819148632)*x[1]
        arg[1,0,0]=(-0.519122333314)*x[0]**o+(-0.895333584981)*x[0]+(-0.941503934561)*x[1]**o+(0.574440495425)*x[1]
        arg[1,0,1]=(0.34967649382)*x[0]**o+(-0.332598318229)*x[0]+(0.0587818420974)*x[1]**o+(0.0848419508498)*x[1]
        arg[1,1,0]=(0.126943263196)*x[0]**o+(0.20287950383)*x[0]+(-0.305454685691)*x[1]**o+(-0.357695790052)*x[1]
        arg[1,1,1]=(0.104154708153)*x[0]**o+(0.867435765411)*x[0]+(0.656946071961)*x[1]**o+(0.96433811362)*x[1]
        arg[2,0,0]=(-0.407362949457)*x[0]**o+(-0.171021521273)*x[0]+(0.901342300398)*x[1]**o+(-0.0360476097399)*x[1]
        arg[2,0,1]=(-0.709421035652)*x[0]**o+(0.949318093375)*x[0]+(0.155061840049)*x[1]**o+(0.828999317354)*x[1]
        arg[2,1,0]=(-0.573031939083)*x[0]**o+(-0.766186602896)*x[0]+(-0.0483071013367)*x[1]**o+(0.741988754559)*x[1]
        arg[2,1,1]=(-0.826747727748)*x[0]**o+(0.481970220735)*x[0]+(0.735617076451)*x[1]**o+(-0.64482586429)*x[1]
        arg[3,0,0]=(0.0834619008603)*x[0]**o+(-0.122260004323)*x[0]+(-0.193792114997)*x[1]**o+(0.635792828013)*x[1]
        arg[3,0,1]=(0.868630978759)*x[0]**o+(0.54474614716)*x[0]+(-0.514377753448)*x[1]**o+(0.58889412844)*x[1]
        arg[3,1,0]=(-0.233744744882)*x[0]**o+(0.883601383004)*x[0]+(0.575012113003)*x[1]**o+(0.185704140149)*x[1]
        arg[3,1,1]=(-0.948893544046)*x[0]**o+(0.851870038501)*x[0]+(-0.591835259957)*x[1]**o+(-0.958375628808)*x[1]
        arg[4,0,0]=(0.981685480559)*x[0]**o+(0.718249744763)*x[0]+(0.589534430546)*x[1]**o+(0.72294148429)*x[1]
        arg[4,0,1]=(-0.603601154465)*x[0]**o+(0.293688738803)*x[0]+(0.151726724518)*x[1]**o+(0.56316377967)*x[1]
        arg[4,1,0]=(0.539033109011)*x[0]**o+(0.791686885872)*x[0]+(0.878973377358)*x[1]**o+(0.869606341625)*x[1]
        arg[4,1,1]=(-0.0920873290752)*x[0]**o+(-0.132682060309)*x[0]+(0.34023826981)*x[1]**o+(0.585476816741)*x[1]
        arg[5,0,0]=(-0.430989548175)*x[0]**o+(-0.525387682044)*x[0]+(-0.996483829778)*x[1]**o+(-0.564503119535)*x[1]
        arg[5,0,1]=(0.602911018749)*x[0]**o+(-0.348972500704)*x[0]+(0.174681048071)*x[1]**o+(-0.94551825287)*x[1]
        arg[5,1,0]=(0.780926392259)*x[0]**o+(-0.164977652547)*x[0]+(0.224726701017)*x[1]**o+(-0.779109318124)*x[1]
        arg[5,1,1]=(0.470928696402)*x[0]**o+(-0.505871268699)*x[0]+(0.407158974816)*x[1]**o+(0.392856659954)*x[1]
        ref[0,0,0]=(-0.475615092269)*x_ref[0]**o+(-0.0893751722575)*x_ref[0]+(0.595881277166)*x_ref[1]**o+(0.397900173915)*x_ref[1]
        ref[0,0,1]=(-0.552614805712)*x_ref[0]**o+(0.14072435445)*x_ref[0]+(0.290231845094)*x_ref[1]**o+(0.573878059041)*x_ref[1]
        ref[0,1,0]=(-0.226148312038)*x_ref[0]**o+(-0.737524507359)*x_ref[0]+(-0.704362709293)*x_ref[1]**o+(-0.197223346266)*x_ref[1]
        ref[0,1,1]=(0.844313529642)*x_ref[0]**o+(-0.948514286846)*x_ref[0]+(0.383754990882)*x_ref[1]**o+(0.629819148632)*x_ref[1]
        ref[1,0,0]=(-0.519122333314)*x_ref[0]**o+(-0.895333584981)*x_ref[0]+(-0.941503934561)*x_ref[1]**o+(0.574440495425)*x_ref[1]
        ref[1,0,1]=(0.34967649382)*x_ref[0]**o+(-0.332598318229)*x_ref[0]+(0.0587818420974)*x_ref[1]**o+(0.0848419508498)*x_ref[1]
        ref[1,1,0]=(0.126943263196)*x_ref[0]**o+(0.20287950383)*x_ref[0]+(-0.305454685691)*x_ref[1]**o+(-0.357695790052)*x_ref[1]
        ref[1,1,1]=(0.104154708153)*x_ref[0]**o+(0.867435765411)*x_ref[0]+(0.656946071961)*x_ref[1]**o+(0.96433811362)*x_ref[1]
        ref[2,0,0]=(-0.407362949457)*x_ref[0]**o+(-0.171021521273)*x_ref[0]+(0.901342300398)*x_ref[1]**o+(-0.0360476097399)*x_ref[1]
        ref[2,0,1]=(-0.709421035652)*x_ref[0]**o+(0.949318093375)*x_ref[0]+(0.155061840049)*x_ref[1]**o+(0.828999317354)*x_ref[1]
        ref[2,1,0]=(-0.573031939083)*x_ref[0]**o+(-0.766186602896)*x_ref[0]+(-0.0483071013367)*x_ref[1]**o+(0.741988754559)*x_ref[1]
        ref[2,1,1]=(-0.826747727748)*x_ref[0]**o+(0.481970220735)*x_ref[0]+(0.735617076451)*x_ref[1]**o+(-0.64482586429)*x_ref[1]
        ref[3,0,0]=(0.0834619008603)*x_ref[0]**o+(-0.122260004323)*x_ref[0]+(-0.193792114997)*x_ref[1]**o+(0.635792828013)*x_ref[1]
        ref[3,0,1]=(0.868630978759)*x_ref[0]**o+(0.54474614716)*x_ref[0]+(-0.514377753448)*x_ref[1]**o+(0.58889412844)*x_ref[1]
        ref[3,1,0]=(-0.233744744882)*x_ref[0]**o+(0.883601383004)*x_ref[0]+(0.575012113003)*x_ref[1]**o+(0.185704140149)*x_ref[1]
        ref[3,1,1]=(-0.948893544046)*x_ref[0]**o+(0.851870038501)*x_ref[0]+(-0.591835259957)*x_ref[1]**o+(-0.958375628808)*x_ref[1]
        ref[4,0,0]=(0.981685480559)*x_ref[0]**o+(0.718249744763)*x_ref[0]+(0.589534430546)*x_ref[1]**o+(0.72294148429)*x_ref[1]
        ref[4,0,1]=(-0.603601154465)*x_ref[0]**o+(0.293688738803)*x_ref[0]+(0.151726724518)*x_ref[1]**o+(0.56316377967)*x_ref[1]
        ref[4,1,0]=(0.539033109011)*x_ref[0]**o+(0.791686885872)*x_ref[0]+(0.878973377358)*x_ref[1]**o+(0.869606341625)*x_ref[1]
        ref[4,1,1]=(-0.0920873290752)*x_ref[0]**o+(-0.132682060309)*x_ref[0]+(0.34023826981)*x_ref[1]**o+(0.585476816741)*x_ref[1]
        ref[5,0,0]=(-0.430989548175)*x_ref[0]**o+(-0.525387682044)*x_ref[0]+(-0.996483829778)*x_ref[1]**o+(-0.564503119535)*x_ref[1]
        ref[5,0,1]=(0.602911018749)*x_ref[0]**o+(-0.348972500704)*x_ref[0]+(0.174681048071)*x_ref[1]**o+(-0.94551825287)*x_ref[1]
        ref[5,1,0]=(0.780926392259)*x_ref[0]**o+(-0.164977652547)*x_ref[0]+(0.224726701017)*x_ref[1]**o+(-0.779109318124)*x_ref[1]
        ref[5,1,1]=(0.470928696402)*x_ref[0]**o+(-0.505871268699)*x_ref[0]+(0.407158974816)*x_ref[1]**o+(0.392856659954)*x_ref[1]
      else:
        arg[0,0,0]=(0.759590374826)*x[0]**o+(-0.776182110267)*x[0]+(0.713871101133)*x[1]**o+(-0.0991655752543)*x[1]+(0.339491998788)*x[2]**o+(-0.624141085669)*x[2]
        arg[0,0,1]=(-0.948679111098)*x[0]**o+(0.0918139408797)*x[0]+(-0.519481784001)*x[1]**o+(-0.480754367303)*x[1]+(0.996421472879)*x[2]**o+(-0.933032853651)*x[2]
        arg[0,1,0]=(-0.824750788431)*x[0]**o+(0.846735099726)*x[0]+(-0.436835905895)*x[1]**o+(-0.732363323611)*x[1]+(-0.865610972073)*x[2]**o+(0.754759759319)*x[2]
        arg[0,1,1]=(-0.403783782613)*x[0]**o+(0.918875237045)*x[0]+(0.0934155750366)*x[1]**o+(0.832477256884)*x[1]+(0.615080190605)*x[2]**o+(0.000920074523901)*x[2]
        arg[1,0,0]=(0.260531941775)*x[0]**o+(-0.514647294043)*x[0]+(0.237113978946)*x[1]**o+(-0.116626766689)*x[1]+(-0.670422853592)*x[2]**o+(0.574941151693)*x[2]
        arg[1,0,1]=(-0.0133773984155)*x[0]**o+(0.286273972547)*x[0]+(0.0493985332233)*x[1]**o+(-0.567835671454)*x[1]+(0.508314781519)*x[2]**o+(-0.603506084784)*x[2]
        arg[1,1,0]=(0.0237413792908)*x[0]**o+(-0.758893281773)*x[0]+(0.150670689461)*x[1]**o+(0.136386516616)*x[1]+(-0.440203817756)*x[2]**o+(0.796610908529)*x[2]
        arg[1,1,1]=(-0.223549634917)*x[0]**o+(0.396472946824)*x[0]+(0.47830083818)*x[1]**o+(0.059933592631)*x[1]+(0.709371074379)*x[2]**o+(-0.129442424949)*x[2]
        arg[2,0,0]=(0.686170679191)*x[0]**o+(-0.118668021586)*x[0]+(0.033954449978)*x[1]**o+(-0.0508661210104)*x[1]+(0.352824724076)*x[2]**o+(-0.772874133692)*x[2]
        arg[2,0,1]=(0.434879772789)*x[0]**o+(-0.148309931356)*x[0]+(-0.013792688421)*x[1]**o+(0.597899709109)*x[1]+(-0.280336465565)*x[2]**o+(-0.177134087728)*x[2]
        arg[2,1,0]=(-0.636844238715)*x[0]**o+(-0.646269255495)*x[0]+(0.243236542558)*x[1]**o+(0.203480511801)*x[1]+(0.693290035164)*x[2]**o+(0.773354114662)*x[2]
        arg[2,1,1]=(-0.161998867128)*x[0]**o+(0.193608078775)*x[0]+(0.607954851405)*x[1]**o+(0.648078354644)*x[1]+(-0.488891750936)*x[2]**o+(0.912860597431)*x[2]
        arg[3,0,0]=(-0.696909279028)*x[0]**o+(0.039558795649)*x[0]+(0.441121646773)*x[1]**o+(-0.0681790925087)*x[1]+(-0.792437910441)*x[2]**o+(-0.512942395878)*x[2]
        arg[3,0,1]=(-0.264832490825)*x[0]**o+(-0.955976365474)*x[0]+(-0.884641387524)*x[1]**o+(-0.736970630699)*x[1]+(-0.0525378155791)*x[2]**o+(0.149164361449)*x[2]
        arg[3,1,0]=(0.434888951175)*x[0]**o+(-0.218511432369)*x[0]+(0.564377624401)*x[1]**o+(-0.0978716780106)*x[1]+(0.447857191989)*x[2]**o+(-0.466572586856)*x[2]
        arg[3,1,1]=(0.457689985738)*x[0]**o+(0.00322996620178)*x[0]+(0.307787601144)*x[1]**o+(-0.119455639704)*x[1]+(0.70367723461)*x[2]**o+(0.561913096313)*x[2]
        arg[4,0,0]=(-0.759802700221)*x[0]**o+(0.469649690329)*x[0]+(0.0913972297116)*x[1]**o+(0.310277900593)*x[1]+(-0.590266889319)*x[2]**o+(-0.89711266859)*x[2]
        arg[4,0,1]=(0.85506147544)*x[0]**o+(-0.357476737069)*x[0]+(-0.646898911915)*x[1]**o+(-0.674811608049)*x[1]+(0.865265982147)*x[2]**o+(0.720153126392)*x[2]
        arg[4,1,0]=(0.848146446583)*x[0]**o+(0.868166234685)*x[0]+(0.0477110054774)*x[1]**o+(-0.0338098833736)*x[1]+(-0.360024439914)*x[2]**o+(-0.634359083766)*x[2]
        arg[4,1,1]=(-0.672530624126)*x[0]**o+(0.354170615043)*x[0]+(0.486028226561)*x[1]**o+(0.857532578327)*x[1]+(0.15798171243)*x[2]**o+(0.0579857791862)*x[2]
        arg[5,0,0]=(0.653171799956)*x[0]**o+(0.605427034155)*x[0]+(0.7414424011)*x[1]**o+(-0.807505211605)*x[1]+(-0.0411776868808)*x[2]**o+(0.24301961063)*x[2]
        arg[5,0,1]=(-0.336860102074)*x[0]**o+(0.235697646908)*x[0]+(-0.173217953629)*x[1]**o+(0.989643649714)*x[1]+(0.977460054831)*x[2]**o+(-0.743825027125)*x[2]
        arg[5,1,0]=(0.794904875581)*x[0]**o+(0.607548574046)*x[0]+(0.603180304387)*x[1]**o+(0.276995429843)*x[1]+(0.698731315308)*x[2]**o+(-0.966744108727)*x[2]
        arg[5,1,1]=(-0.0230004766002)*x[0]**o+(0.489585418104)*x[0]+(0.382764947116)*x[1]**o+(0.143873532935)*x[1]+(0.932000957196)*x[2]**o+(0.739975795806)*x[2]
        ref[0,0,0]=(0.759590374826)*x_ref[0]**o+(-0.776182110267)*x_ref[0]+(0.713871101133)*x_ref[1]**o+(-0.0991655752543)*x_ref[1]+(0.339491998788)*x_ref[2]**o+(-0.624141085669)*x_ref[2]
        ref[0,0,1]=(-0.948679111098)*x_ref[0]**o+(0.0918139408797)*x_ref[0]+(-0.519481784001)*x_ref[1]**o+(-0.480754367303)*x_ref[1]+(0.996421472879)*x_ref[2]**o+(-0.933032853651)*x_ref[2]
        ref[0,1,0]=(-0.824750788431)*x_ref[0]**o+(0.846735099726)*x_ref[0]+(-0.436835905895)*x_ref[1]**o+(-0.732363323611)*x_ref[1]+(-0.865610972073)*x_ref[2]**o+(0.754759759319)*x_ref[2]
        ref[0,1,1]=(-0.403783782613)*x_ref[0]**o+(0.918875237045)*x_ref[0]+(0.0934155750366)*x_ref[1]**o+(0.832477256884)*x_ref[1]+(0.615080190605)*x_ref[2]**o+(0.000920074523901)*x_ref[2]
        ref[1,0,0]=(0.260531941775)*x_ref[0]**o+(-0.514647294043)*x_ref[0]+(0.237113978946)*x_ref[1]**o+(-0.116626766689)*x_ref[1]+(-0.670422853592)*x_ref[2]**o+(0.574941151693)*x_ref[2]
        ref[1,0,1]=(-0.0133773984155)*x_ref[0]**o+(0.286273972547)*x_ref[0]+(0.0493985332233)*x_ref[1]**o+(-0.567835671454)*x_ref[1]+(0.508314781519)*x_ref[2]**o+(-0.603506084784)*x_ref[2]
        ref[1,1,0]=(0.0237413792908)*x_ref[0]**o+(-0.758893281773)*x_ref[0]+(0.150670689461)*x_ref[1]**o+(0.136386516616)*x_ref[1]+(-0.440203817756)*x_ref[2]**o+(0.796610908529)*x_ref[2]
        ref[1,1,1]=(-0.223549634917)*x_ref[0]**o+(0.396472946824)*x_ref[0]+(0.47830083818)*x_ref[1]**o+(0.059933592631)*x_ref[1]+(0.709371074379)*x_ref[2]**o+(-0.129442424949)*x_ref[2]
        ref[2,0,0]=(0.686170679191)*x_ref[0]**o+(-0.118668021586)*x_ref[0]+(0.033954449978)*x_ref[1]**o+(-0.0508661210104)*x_ref[1]+(0.352824724076)*x_ref[2]**o+(-0.772874133692)*x_ref[2]
        ref[2,0,1]=(0.434879772789)*x_ref[0]**o+(-0.148309931356)*x_ref[0]+(-0.013792688421)*x_ref[1]**o+(0.597899709109)*x_ref[1]+(-0.280336465565)*x_ref[2]**o+(-0.177134087728)*x_ref[2]
        ref[2,1,0]=(-0.636844238715)*x_ref[0]**o+(-0.646269255495)*x_ref[0]+(0.243236542558)*x_ref[1]**o+(0.203480511801)*x_ref[1]+(0.693290035164)*x_ref[2]**o+(0.773354114662)*x_ref[2]
        ref[2,1,1]=(-0.161998867128)*x_ref[0]**o+(0.193608078775)*x_ref[0]+(0.607954851405)*x_ref[1]**o+(0.648078354644)*x_ref[1]+(-0.488891750936)*x_ref[2]**o+(0.912860597431)*x_ref[2]
        ref[3,0,0]=(-0.696909279028)*x_ref[0]**o+(0.039558795649)*x_ref[0]+(0.441121646773)*x_ref[1]**o+(-0.0681790925087)*x_ref[1]+(-0.792437910441)*x_ref[2]**o+(-0.512942395878)*x_ref[2]
        ref[3,0,1]=(-0.264832490825)*x_ref[0]**o+(-0.955976365474)*x_ref[0]+(-0.884641387524)*x_ref[1]**o+(-0.736970630699)*x_ref[1]+(-0.0525378155791)*x_ref[2]**o+(0.149164361449)*x_ref[2]
        ref[3,1,0]=(0.434888951175)*x_ref[0]**o+(-0.218511432369)*x_ref[0]+(0.564377624401)*x_ref[1]**o+(-0.0978716780106)*x_ref[1]+(0.447857191989)*x_ref[2]**o+(-0.466572586856)*x_ref[2]
        ref[3,1,1]=(0.457689985738)*x_ref[0]**o+(0.00322996620178)*x_ref[0]+(0.307787601144)*x_ref[1]**o+(-0.119455639704)*x_ref[1]+(0.70367723461)*x_ref[2]**o+(0.561913096313)*x_ref[2]
        ref[4,0,0]=(-0.759802700221)*x_ref[0]**o+(0.469649690329)*x_ref[0]+(0.0913972297116)*x_ref[1]**o+(0.310277900593)*x_ref[1]+(-0.590266889319)*x_ref[2]**o+(-0.89711266859)*x_ref[2]
        ref[4,0,1]=(0.85506147544)*x_ref[0]**o+(-0.357476737069)*x_ref[0]+(-0.646898911915)*x_ref[1]**o+(-0.674811608049)*x_ref[1]+(0.865265982147)*x_ref[2]**o+(0.720153126392)*x_ref[2]
        ref[4,1,0]=(0.848146446583)*x_ref[0]**o+(0.868166234685)*x_ref[0]+(0.0477110054774)*x_ref[1]**o+(-0.0338098833736)*x_ref[1]+(-0.360024439914)*x_ref[2]**o+(-0.634359083766)*x_ref[2]
        ref[4,1,1]=(-0.672530624126)*x_ref[0]**o+(0.354170615043)*x_ref[0]+(0.486028226561)*x_ref[1]**o+(0.857532578327)*x_ref[1]+(0.15798171243)*x_ref[2]**o+(0.0579857791862)*x_ref[2]
        ref[5,0,0]=(0.653171799956)*x_ref[0]**o+(0.605427034155)*x_ref[0]+(0.7414424011)*x_ref[1]**o+(-0.807505211605)*x_ref[1]+(-0.0411776868808)*x_ref[2]**o+(0.24301961063)*x_ref[2]
        ref[5,0,1]=(-0.336860102074)*x_ref[0]**o+(0.235697646908)*x_ref[0]+(-0.173217953629)*x_ref[1]**o+(0.989643649714)*x_ref[1]+(0.977460054831)*x_ref[2]**o+(-0.743825027125)*x_ref[2]
        ref[5,1,0]=(0.794904875581)*x_ref[0]**o+(0.607548574046)*x_ref[0]+(0.603180304387)*x_ref[1]**o+(0.276995429843)*x_ref[1]+(0.698731315308)*x_ref[2]**o+(-0.966744108727)*x_ref[2]
        ref[5,1,1]=(-0.0230004766002)*x_ref[0]**o+(0.489585418104)*x_ref[0]+(0.382764947116)*x_ref[1]**o+(0.143873532935)*x_ref[1]+(0.932000957196)*x_ref[2]**o+(0.739975795806)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromData_ContinuousFunction_rank4(self):
      """
      tests interpolate for rank 4 Data onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ContinuousFunction onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5, 3, 2),w)
      ref=Data(0,(4, 5, 3, 2),w_ref)
      if dim==2:
        arg[0,0,0,0]=(0.922177773381)*x[0]**o+(0.37324175781)*x[0]+(-0.169208776011)*x[1]**o+(0.72134249236)*x[1]
        arg[0,0,0,1]=(0.85312270129)*x[0]**o+(0.0512919651836)*x[0]+(0.968880249675)*x[1]**o+(0.740058819096)*x[1]
        arg[0,0,1,0]=(-0.018205657901)*x[0]**o+(0.601838710779)*x[0]+(0.211002282731)*x[1]**o+(0.801382123792)*x[1]
        arg[0,0,1,1]=(-0.249128714489)*x[0]**o+(-0.408809542089)*x[0]+(-0.684434920261)*x[1]**o+(0.223674849818)*x[1]
        arg[0,0,2,0]=(0.396627551183)*x[0]**o+(-0.223592778052)*x[0]+(0.227333834259)*x[1]**o+(0.66580805616)*x[1]
        arg[0,0,2,1]=(0.365216692731)*x[0]**o+(-0.670947594503)*x[0]+(0.400260365509)*x[1]**o+(0.10903299253)*x[1]
        arg[0,1,0,0]=(0.823297765985)*x[0]**o+(0.60337946486)*x[0]+(-0.314265083693)*x[1]**o+(-0.00636816227929)*x[1]
        arg[0,1,0,1]=(-0.26216625982)*x[0]**o+(0.767412606588)*x[0]+(-0.284917238441)*x[1]**o+(0.56385025143)*x[1]
        arg[0,1,1,0]=(-0.732326448281)*x[0]**o+(-0.669197323856)*x[0]+(-0.608751752289)*x[1]**o+(-0.853458985188)*x[1]
        arg[0,1,1,1]=(0.637275594334)*x[0]**o+(0.696570850488)*x[0]+(0.012026820657)*x[1]**o+(0.558317439186)*x[1]
        arg[0,1,2,0]=(0.925899092402)*x[0]**o+(0.568045453255)*x[0]+(-0.766701931715)*x[1]**o+(-0.95082586036)*x[1]
        arg[0,1,2,1]=(0.127524211574)*x[0]**o+(-0.65150542164)*x[0]+(-0.365773940068)*x[1]**o+(0.990465711754)*x[1]
        arg[0,2,0,0]=(0.456469052217)*x[0]**o+(0.403391798397)*x[0]+(-0.21921915949)*x[1]**o+(0.171493695945)*x[1]
        arg[0,2,0,1]=(-0.980242506743)*x[0]**o+(0.880225959645)*x[0]+(0.682653186307)*x[1]**o+(-0.561867916377)*x[1]
        arg[0,2,1,0]=(-0.329881998938)*x[0]**o+(0.464976480407)*x[0]+(-0.812583693347)*x[1]**o+(0.0103356333568)*x[1]
        arg[0,2,1,1]=(-0.710350374606)*x[0]**o+(-0.445981415127)*x[0]+(-0.159054554695)*x[1]**o+(-0.0306514318922)*x[1]
        arg[0,2,2,0]=(0.0903142465124)*x[0]**o+(0.872680271888)*x[0]+(0.230242338668)*x[1]**o+(0.557126778217)*x[1]
        arg[0,2,2,1]=(-0.842046475241)*x[0]**o+(0.350544067128)*x[0]+(0.486186516633)*x[1]**o+(-0.550801066658)*x[1]
        arg[0,3,0,0]=(0.740313870334)*x[0]**o+(-0.177105021967)*x[0]+(0.00114640526837)*x[1]**o+(0.349602955151)*x[1]
        arg[0,3,0,1]=(0.898906724506)*x[0]**o+(-0.882729119203)*x[0]+(0.000278645863091)*x[1]**o+(-0.450247260284)*x[1]
        arg[0,3,1,0]=(0.411917691981)*x[0]**o+(0.0461900014479)*x[0]+(0.1608400364)*x[1]**o+(0.130550626888)*x[1]
        arg[0,3,1,1]=(0.610378099059)*x[0]**o+(-0.903971943528)*x[0]+(-0.0170960427518)*x[1]**o+(0.765729630725)*x[1]
        arg[0,3,2,0]=(-0.811421756909)*x[0]**o+(-0.220065414196)*x[0]+(0.529467921065)*x[1]**o+(0.13896086667)*x[1]
        arg[0,3,2,1]=(-0.621021589728)*x[0]**o+(0.181903892108)*x[0]+(0.724296638902)*x[1]**o+(0.651705107858)*x[1]
        arg[0,4,0,0]=(0.323254788695)*x[0]**o+(0.758728125147)*x[0]+(0.0719056895797)*x[1]**o+(-0.486299017994)*x[1]
        arg[0,4,0,1]=(-0.426525215977)*x[0]**o+(-0.70344592953)*x[0]+(-0.467799544116)*x[1]**o+(0.0394017649712)*x[1]
        arg[0,4,1,0]=(-0.559984242808)*x[0]**o+(-0.963639424043)*x[0]+(0.884232194806)*x[1]**o+(0.948111505594)*x[1]
        arg[0,4,1,1]=(-0.825627656302)*x[0]**o+(0.167871540452)*x[0]+(0.0969950885663)*x[1]**o+(-0.477024385485)*x[1]
        arg[0,4,2,0]=(-0.890191205005)*x[0]**o+(-0.291891624031)*x[0]+(-0.924812934321)*x[1]**o+(-0.898567598264)*x[1]
        arg[0,4,2,1]=(0.0552241756422)*x[0]**o+(-0.47878688784)*x[0]+(-0.208893161197)*x[1]**o+(-0.671733441515)*x[1]
        arg[1,0,0,0]=(0.609961055126)*x[0]**o+(0.389225845794)*x[0]+(0.147977819654)*x[1]**o+(-0.75096024248)*x[1]
        arg[1,0,0,1]=(-0.407376527411)*x[0]**o+(-0.696660373977)*x[0]+(-0.803862178008)*x[1]**o+(-0.367087112506)*x[1]
        arg[1,0,1,0]=(0.89146134332)*x[0]**o+(0.388607608477)*x[0]+(0.763190367241)*x[1]**o+(0.169275499463)*x[1]
        arg[1,0,1,1]=(-0.870871474063)*x[0]**o+(-0.21526798243)*x[0]+(0.159168404985)*x[1]**o+(0.715049896538)*x[1]
        arg[1,0,2,0]=(-0.842978554795)*x[0]**o+(0.209649486001)*x[0]+(-0.767629750054)*x[1]**o+(-0.126634026874)*x[1]
        arg[1,0,2,1]=(-0.728571756044)*x[0]**o+(-0.247852313042)*x[0]+(0.818809803616)*x[1]**o+(0.736163163331)*x[1]
        arg[1,1,0,0]=(-0.670608139072)*x[0]**o+(0.330167259903)*x[0]+(0.275974484086)*x[1]**o+(0.492110644529)*x[1]
        arg[1,1,0,1]=(0.724664287226)*x[0]**o+(0.738855592065)*x[0]+(0.511369798951)*x[1]**o+(0.455794112842)*x[1]
        arg[1,1,1,0]=(0.783504599076)*x[0]**o+(-0.558579246854)*x[0]+(0.608044912637)*x[1]**o+(-0.401058633268)*x[1]
        arg[1,1,1,1]=(0.406697861117)*x[0]**o+(-0.31302718156)*x[0]+(-0.999553049864)*x[1]**o+(0.647698243869)*x[1]
        arg[1,1,2,0]=(-0.788346252365)*x[0]**o+(-0.809895265253)*x[0]+(0.441738551485)*x[1]**o+(-0.686785865679)*x[1]
        arg[1,1,2,1]=(0.794571346519)*x[0]**o+(-0.552071697712)*x[0]+(0.77260218173)*x[1]**o+(-0.969584549528)*x[1]
        arg[1,2,0,0]=(-0.867948229325)*x[0]**o+(-0.582387320474)*x[0]+(0.0387726756335)*x[1]**o+(0.4143770831)*x[1]
        arg[1,2,0,1]=(-0.913557875303)*x[0]**o+(-0.517970834439)*x[0]+(-0.922517499063)*x[1]**o+(0.858552054359)*x[1]
        arg[1,2,1,0]=(0.409302016181)*x[0]**o+(-0.684529177441)*x[0]+(0.818122023218)*x[1]**o+(-0.583589419623)*x[1]
        arg[1,2,1,1]=(0.750795883056)*x[0]**o+(-0.710882726624)*x[0]+(0.915210035315)*x[1]**o+(0.802588664027)*x[1]
        arg[1,2,2,0]=(0.639216566804)*x[0]**o+(-0.300954214125)*x[0]+(-0.26724393342)*x[1]**o+(0.426742851293)*x[1]
        arg[1,2,2,1]=(-0.00337538020121)*x[0]**o+(0.283012169215)*x[0]+(0.725040731001)*x[1]**o+(-0.708238265029)*x[1]
        arg[1,3,0,0]=(0.217404922432)*x[0]**o+(-0.590883003502)*x[0]+(0.451142849723)*x[1]**o+(0.624698767392)*x[1]
        arg[1,3,0,1]=(0.0351258739828)*x[0]**o+(-0.784068492686)*x[0]+(0.469956184167)*x[1]**o+(0.634124784387)*x[1]
        arg[1,3,1,0]=(0.998414775431)*x[0]**o+(0.388202946918)*x[0]+(-0.849223443255)*x[1]**o+(-0.804590028218)*x[1]
        arg[1,3,1,1]=(0.0219748851856)*x[0]**o+(0.464069403336)*x[0]+(-0.470759541095)*x[1]**o+(-0.21773573235)*x[1]
        arg[1,3,2,0]=(-0.879175905661)*x[0]**o+(0.100238280861)*x[0]+(0.331472085056)*x[1]**o+(0.0310947651268)*x[1]
        arg[1,3,2,1]=(-0.195155740038)*x[0]**o+(0.532078676231)*x[0]+(-0.863288082011)*x[1]**o+(-0.0905878788265)*x[1]
        arg[1,4,0,0]=(0.470883637487)*x[0]**o+(0.0291028245963)*x[0]+(-0.152462235474)*x[1]**o+(0.890883963301)*x[1]
        arg[1,4,0,1]=(-0.0780797534409)*x[0]**o+(-0.999649003413)*x[0]+(-0.79446765004)*x[1]**o+(-0.820520881428)*x[1]
        arg[1,4,1,0]=(-0.82479629638)*x[0]**o+(-0.28924829922)*x[0]+(-0.960430867949)*x[1]**o+(-0.514730558673)*x[1]
        arg[1,4,1,1]=(-0.0853911110391)*x[0]**o+(0.726747270178)*x[0]+(-0.70925711586)*x[1]**o+(-0.331433386035)*x[1]
        arg[1,4,2,0]=(-0.434350343402)*x[0]**o+(0.335341284415)*x[0]+(0.496818240476)*x[1]**o+(-0.925006191184)*x[1]
        arg[1,4,2,1]=(0.0753516047957)*x[0]**o+(0.171090641529)*x[0]+(-0.836002762987)*x[1]**o+(-0.753836171615)*x[1]
        arg[2,0,0,0]=(-0.331008073553)*x[0]**o+(-0.299051880715)*x[0]+(-0.84520124413)*x[1]**o+(0.780471473975)*x[1]
        arg[2,0,0,1]=(-0.947973851318)*x[0]**o+(0.579142212501)*x[0]+(0.137317760268)*x[1]**o+(-0.379509530229)*x[1]
        arg[2,0,1,0]=(-0.741320730287)*x[0]**o+(0.642110710824)*x[0]+(-0.632595944259)*x[1]**o+(0.944980046528)*x[1]
        arg[2,0,1,1]=(0.616943682269)*x[0]**o+(-0.348189507611)*x[0]+(-0.414080070092)*x[1]**o+(-0.733041081362)*x[1]
        arg[2,0,2,0]=(-0.021261659709)*x[0]**o+(0.790041129031)*x[0]+(0.788378634137)*x[1]**o+(0.391057560074)*x[1]
        arg[2,0,2,1]=(0.563737321295)*x[0]**o+(-0.110516278477)*x[0]+(0.383563652425)*x[1]**o+(-0.857100387101)*x[1]
        arg[2,1,0,0]=(-0.0839346911978)*x[0]**o+(0.699255867073)*x[0]+(-0.698488781835)*x[1]**o+(0.225940657322)*x[1]
        arg[2,1,0,1]=(0.93638601214)*x[0]**o+(-0.593516691147)*x[0]+(-0.445863239521)*x[1]**o+(0.601067357746)*x[1]
        arg[2,1,1,0]=(0.114814804735)*x[0]**o+(0.717600499991)*x[0]+(0.7371911818)*x[1]**o+(0.136012416552)*x[1]
        arg[2,1,1,1]=(0.794499830148)*x[0]**o+(0.0581947415855)*x[0]+(0.626382859976)*x[1]**o+(-0.668272098674)*x[1]
        arg[2,1,2,0]=(0.564628771073)*x[0]**o+(0.102820241267)*x[0]+(0.397960234363)*x[1]**o+(0.379632827896)*x[1]
        arg[2,1,2,1]=(-0.0880639775913)*x[0]**o+(0.325978376801)*x[0]+(-0.505498229733)*x[1]**o+(-0.595922646655)*x[1]
        arg[2,2,0,0]=(-0.132672737928)*x[0]**o+(-0.0291240195523)*x[0]+(-0.00825944211706)*x[1]**o+(0.985993963145)*x[1]
        arg[2,2,0,1]=(0.244321497515)*x[0]**o+(-0.111329005236)*x[0]+(0.769353471123)*x[1]**o+(-0.174257027136)*x[1]
        arg[2,2,1,0]=(0.252286330282)*x[0]**o+(0.450975615825)*x[0]+(-0.515016578606)*x[1]**o+(0.834736757555)*x[1]
        arg[2,2,1,1]=(0.0588273338962)*x[0]**o+(-0.865406338788)*x[0]+(0.482095645149)*x[1]**o+(-0.595126490156)*x[1]
        arg[2,2,2,0]=(0.244580063555)*x[0]**o+(0.576709884892)*x[0]+(-0.212616562122)*x[1]**o+(-0.407508050723)*x[1]
        arg[2,2,2,1]=(0.416825783755)*x[0]**o+(0.430959078988)*x[0]+(-0.398840301481)*x[1]**o+(0.889105964295)*x[1]
        arg[2,3,0,0]=(-0.731872136283)*x[0]**o+(0.237034931745)*x[0]+(0.547737117698)*x[1]**o+(0.364733861661)*x[1]
        arg[2,3,0,1]=(0.971716549689)*x[0]**o+(0.466300970081)*x[0]+(-0.51802162747)*x[1]**o+(-0.859910778018)*x[1]
        arg[2,3,1,0]=(-0.562918219893)*x[0]**o+(-0.0532549959391)*x[0]+(0.842033495359)*x[1]**o+(-0.708554283815)*x[1]
        arg[2,3,1,1]=(0.437125534104)*x[0]**o+(-0.376751615116)*x[0]+(-0.330889383623)*x[1]**o+(0.958884035289)*x[1]
        arg[2,3,2,0]=(-0.194290501566)*x[0]**o+(-0.368131904292)*x[0]+(0.802917650409)*x[1]**o+(0.628603987906)*x[1]
        arg[2,3,2,1]=(0.70219875881)*x[0]**o+(0.822181339005)*x[0]+(0.768635447234)*x[1]**o+(0.871676010439)*x[1]
        arg[2,4,0,0]=(-0.654498700662)*x[0]**o+(0.851143676147)*x[0]+(0.468468395415)*x[1]**o+(-0.447240713576)*x[1]
        arg[2,4,0,1]=(0.893177052112)*x[0]**o+(0.652743480988)*x[0]+(0.726646163682)*x[1]**o+(-0.854895731892)*x[1]
        arg[2,4,1,0]=(0.0540542301334)*x[0]**o+(-0.208654443984)*x[0]+(0.925127576853)*x[1]**o+(-0.932489814868)*x[1]
        arg[2,4,1,1]=(-0.460268214104)*x[0]**o+(-0.303569308977)*x[0]+(-0.381458880316)*x[1]**o+(0.690537661673)*x[1]
        arg[2,4,2,0]=(0.401012366366)*x[0]**o+(-0.970993441679)*x[0]+(0.0249535421254)*x[1]**o+(-0.499737254014)*x[1]
        arg[2,4,2,1]=(-0.692604790859)*x[0]**o+(-0.685292098725)*x[0]+(0.22550357354)*x[1]**o+(0.908152783462)*x[1]
        arg[3,0,0,0]=(-0.634246407254)*x[0]**o+(-0.571223033336)*x[0]+(0.0574319920181)*x[1]**o+(0.123763929908)*x[1]
        arg[3,0,0,1]=(-0.569358219213)*x[0]**o+(-0.107237509632)*x[0]+(0.97672999497)*x[1]**o+(-0.114939366813)*x[1]
        arg[3,0,1,0]=(-0.767885152479)*x[0]**o+(-0.105924102555)*x[0]+(0.539361607398)*x[1]**o+(0.546186446469)*x[1]
        arg[3,0,1,1]=(-0.58818696897)*x[0]**o+(0.629371516566)*x[0]+(-0.598173904376)*x[1]**o+(0.959043112088)*x[1]
        arg[3,0,2,0]=(0.751165393927)*x[0]**o+(0.732151619397)*x[0]+(-0.435276735735)*x[1]**o+(0.947142140722)*x[1]
        arg[3,0,2,1]=(0.463904349547)*x[0]**o+(-0.947270841014)*x[0]+(0.304239969784)*x[1]**o+(-0.233566944292)*x[1]
        arg[3,1,0,0]=(0.961122232395)*x[0]**o+(-0.989040215156)*x[0]+(0.833186746232)*x[1]**o+(0.226673105305)*x[1]
        arg[3,1,0,1]=(-0.156957616578)*x[0]**o+(-0.361665587314)*x[0]+(0.987490273947)*x[1]**o+(-0.890271058746)*x[1]
        arg[3,1,1,0]=(-0.259959222953)*x[0]**o+(-0.696135598749)*x[0]+(0.234411151165)*x[1]**o+(0.783105897256)*x[1]
        arg[3,1,1,1]=(0.020886322199)*x[0]**o+(0.887445555864)*x[0]+(0.709489180254)*x[1]**o+(0.990590112507)*x[1]
        arg[3,1,2,0]=(-0.963790247866)*x[0]**o+(-0.417180971158)*x[0]+(0.303859261646)*x[1]**o+(0.745552024399)*x[1]
        arg[3,1,2,1]=(-0.364448135641)*x[0]**o+(0.760841571414)*x[0]+(-0.0283374186763)*x[1]**o+(-0.491433855748)*x[1]
        arg[3,2,0,0]=(-0.649766142291)*x[0]**o+(-0.177410238297)*x[0]+(0.137316641289)*x[1]**o+(-0.40890299152)*x[1]
        arg[3,2,0,1]=(0.282922984657)*x[0]**o+(-0.660238152943)*x[0]+(0.481726052488)*x[1]**o+(0.976011254218)*x[1]
        arg[3,2,1,0]=(0.989761520473)*x[0]**o+(0.194082255647)*x[0]+(-0.648435682551)*x[1]**o+(-0.721569599311)*x[1]
        arg[3,2,1,1]=(0.734634540883)*x[0]**o+(0.524638529049)*x[0]+(-0.737636791717)*x[1]**o+(-0.995904525822)*x[1]
        arg[3,2,2,0]=(0.422396808663)*x[0]**o+(-0.820639154105)*x[0]+(-0.566207315373)*x[1]**o+(-0.224174711984)*x[1]
        arg[3,2,2,1]=(0.719145374997)*x[0]**o+(-0.885860499468)*x[0]+(0.184062878609)*x[1]**o+(0.47660543007)*x[1]
        arg[3,3,0,0]=(-0.0122290849915)*x[0]**o+(-0.0314298339876)*x[0]+(0.937255407134)*x[1]**o+(-0.754858547072)*x[1]
        arg[3,3,0,1]=(-0.507826956669)*x[0]**o+(0.951205020689)*x[0]+(0.837096245109)*x[1]**o+(-0.576692399518)*x[1]
        arg[3,3,1,0]=(0.602119808238)*x[0]**o+(-0.943937134298)*x[0]+(-0.244250254626)*x[1]**o+(-0.814103542727)*x[1]
        arg[3,3,1,1]=(-0.51811661822)*x[0]**o+(-0.188969078762)*x[0]+(-0.697470816837)*x[1]**o+(-0.741593014841)*x[1]
        arg[3,3,2,0]=(-0.673948933786)*x[0]**o+(0.816404336566)*x[0]+(0.339239090092)*x[1]**o+(-0.0173550947113)*x[1]
        arg[3,3,2,1]=(0.499109852716)*x[0]**o+(0.515501552893)*x[0]+(0.926442939974)*x[1]**o+(0.157227173254)*x[1]
        arg[3,4,0,0]=(-0.1837389273)*x[0]**o+(0.78490229721)*x[0]+(-0.248033788374)*x[1]**o+(0.735136386405)*x[1]
        arg[3,4,0,1]=(0.121169550194)*x[0]**o+(-0.854027732077)*x[0]+(0.937680848475)*x[1]**o+(-0.447184197264)*x[1]
        arg[3,4,1,0]=(-0.788610934694)*x[0]**o+(-0.397825388144)*x[0]+(0.604421361449)*x[1]**o+(0.937184552821)*x[1]
        arg[3,4,1,1]=(-0.493380143496)*x[0]**o+(0.958197000398)*x[0]+(0.282372561751)*x[1]**o+(-0.55322669826)*x[1]
        arg[3,4,2,0]=(0.11333865923)*x[0]**o+(-0.542805511569)*x[0]+(-0.640646701928)*x[1]**o+(-0.342030716597)*x[1]
        arg[3,4,2,1]=(0.505356266297)*x[0]**o+(-0.107422782294)*x[0]+(0.542833447968)*x[1]**o+(-0.615275716592)*x[1]
        ref[0,0,0,0]=(0.922177773381)*x_ref[0]**o+(0.37324175781)*x_ref[0]+(-0.169208776011)*x_ref[1]**o+(0.72134249236)*x_ref[1]
        ref[0,0,0,1]=(0.85312270129)*x_ref[0]**o+(0.0512919651836)*x_ref[0]+(0.968880249675)*x_ref[1]**o+(0.740058819096)*x_ref[1]
        ref[0,0,1,0]=(-0.018205657901)*x_ref[0]**o+(0.601838710779)*x_ref[0]+(0.211002282731)*x_ref[1]**o+(0.801382123792)*x_ref[1]
        ref[0,0,1,1]=(-0.249128714489)*x_ref[0]**o+(-0.408809542089)*x_ref[0]+(-0.684434920261)*x_ref[1]**o+(0.223674849818)*x_ref[1]
        ref[0,0,2,0]=(0.396627551183)*x_ref[0]**o+(-0.223592778052)*x_ref[0]+(0.227333834259)*x_ref[1]**o+(0.66580805616)*x_ref[1]
        ref[0,0,2,1]=(0.365216692731)*x_ref[0]**o+(-0.670947594503)*x_ref[0]+(0.400260365509)*x_ref[1]**o+(0.10903299253)*x_ref[1]
        ref[0,1,0,0]=(0.823297765985)*x_ref[0]**o+(0.60337946486)*x_ref[0]+(-0.314265083693)*x_ref[1]**o+(-0.00636816227929)*x_ref[1]
        ref[0,1,0,1]=(-0.26216625982)*x_ref[0]**o+(0.767412606588)*x_ref[0]+(-0.284917238441)*x_ref[1]**o+(0.56385025143)*x_ref[1]
        ref[0,1,1,0]=(-0.732326448281)*x_ref[0]**o+(-0.669197323856)*x_ref[0]+(-0.608751752289)*x_ref[1]**o+(-0.853458985188)*x_ref[1]
        ref[0,1,1,1]=(0.637275594334)*x_ref[0]**o+(0.696570850488)*x_ref[0]+(0.012026820657)*x_ref[1]**o+(0.558317439186)*x_ref[1]
        ref[0,1,2,0]=(0.925899092402)*x_ref[0]**o+(0.568045453255)*x_ref[0]+(-0.766701931715)*x_ref[1]**o+(-0.95082586036)*x_ref[1]
        ref[0,1,2,1]=(0.127524211574)*x_ref[0]**o+(-0.65150542164)*x_ref[0]+(-0.365773940068)*x_ref[1]**o+(0.990465711754)*x_ref[1]
        ref[0,2,0,0]=(0.456469052217)*x_ref[0]**o+(0.403391798397)*x_ref[0]+(-0.21921915949)*x_ref[1]**o+(0.171493695945)*x_ref[1]
        ref[0,2,0,1]=(-0.980242506743)*x_ref[0]**o+(0.880225959645)*x_ref[0]+(0.682653186307)*x_ref[1]**o+(-0.561867916377)*x_ref[1]
        ref[0,2,1,0]=(-0.329881998938)*x_ref[0]**o+(0.464976480407)*x_ref[0]+(-0.812583693347)*x_ref[1]**o+(0.0103356333568)*x_ref[1]
        ref[0,2,1,1]=(-0.710350374606)*x_ref[0]**o+(-0.445981415127)*x_ref[0]+(-0.159054554695)*x_ref[1]**o+(-0.0306514318922)*x_ref[1]
        ref[0,2,2,0]=(0.0903142465124)*x_ref[0]**o+(0.872680271888)*x_ref[0]+(0.230242338668)*x_ref[1]**o+(0.557126778217)*x_ref[1]
        ref[0,2,2,1]=(-0.842046475241)*x_ref[0]**o+(0.350544067128)*x_ref[0]+(0.486186516633)*x_ref[1]**o+(-0.550801066658)*x_ref[1]
        ref[0,3,0,0]=(0.740313870334)*x_ref[0]**o+(-0.177105021967)*x_ref[0]+(0.00114640526837)*x_ref[1]**o+(0.349602955151)*x_ref[1]
        ref[0,3,0,1]=(0.898906724506)*x_ref[0]**o+(-0.882729119203)*x_ref[0]+(0.000278645863091)*x_ref[1]**o+(-0.450247260284)*x_ref[1]
        ref[0,3,1,0]=(0.411917691981)*x_ref[0]**o+(0.0461900014479)*x_ref[0]+(0.1608400364)*x_ref[1]**o+(0.130550626888)*x_ref[1]
        ref[0,3,1,1]=(0.610378099059)*x_ref[0]**o+(-0.903971943528)*x_ref[0]+(-0.0170960427518)*x_ref[1]**o+(0.765729630725)*x_ref[1]
        ref[0,3,2,0]=(-0.811421756909)*x_ref[0]**o+(-0.220065414196)*x_ref[0]+(0.529467921065)*x_ref[1]**o+(0.13896086667)*x_ref[1]
        ref[0,3,2,1]=(-0.621021589728)*x_ref[0]**o+(0.181903892108)*x_ref[0]+(0.724296638902)*x_ref[1]**o+(0.651705107858)*x_ref[1]
        ref[0,4,0,0]=(0.323254788695)*x_ref[0]**o+(0.758728125147)*x_ref[0]+(0.0719056895797)*x_ref[1]**o+(-0.486299017994)*x_ref[1]
        ref[0,4,0,1]=(-0.426525215977)*x_ref[0]**o+(-0.70344592953)*x_ref[0]+(-0.467799544116)*x_ref[1]**o+(0.0394017649712)*x_ref[1]
        ref[0,4,1,0]=(-0.559984242808)*x_ref[0]**o+(-0.963639424043)*x_ref[0]+(0.884232194806)*x_ref[1]**o+(0.948111505594)*x_ref[1]
        ref[0,4,1,1]=(-0.825627656302)*x_ref[0]**o+(0.167871540452)*x_ref[0]+(0.0969950885663)*x_ref[1]**o+(-0.477024385485)*x_ref[1]
        ref[0,4,2,0]=(-0.890191205005)*x_ref[0]**o+(-0.291891624031)*x_ref[0]+(-0.924812934321)*x_ref[1]**o+(-0.898567598264)*x_ref[1]
        ref[0,4,2,1]=(0.0552241756422)*x_ref[0]**o+(-0.47878688784)*x_ref[0]+(-0.208893161197)*x_ref[1]**o+(-0.671733441515)*x_ref[1]
        ref[1,0,0,0]=(0.609961055126)*x_ref[0]**o+(0.389225845794)*x_ref[0]+(0.147977819654)*x_ref[1]**o+(-0.75096024248)*x_ref[1]
        ref[1,0,0,1]=(-0.407376527411)*x_ref[0]**o+(-0.696660373977)*x_ref[0]+(-0.803862178008)*x_ref[1]**o+(-0.367087112506)*x_ref[1]
        ref[1,0,1,0]=(0.89146134332)*x_ref[0]**o+(0.388607608477)*x_ref[0]+(0.763190367241)*x_ref[1]**o+(0.169275499463)*x_ref[1]
        ref[1,0,1,1]=(-0.870871474063)*x_ref[0]**o+(-0.21526798243)*x_ref[0]+(0.159168404985)*x_ref[1]**o+(0.715049896538)*x_ref[1]
        ref[1,0,2,0]=(-0.842978554795)*x_ref[0]**o+(0.209649486001)*x_ref[0]+(-0.767629750054)*x_ref[1]**o+(-0.126634026874)*x_ref[1]
        ref[1,0,2,1]=(-0.728571756044)*x_ref[0]**o+(-0.247852313042)*x_ref[0]+(0.818809803616)*x_ref[1]**o+(0.736163163331)*x_ref[1]
        ref[1,1,0,0]=(-0.670608139072)*x_ref[0]**o+(0.330167259903)*x_ref[0]+(0.275974484086)*x_ref[1]**o+(0.492110644529)*x_ref[1]
        ref[1,1,0,1]=(0.724664287226)*x_ref[0]**o+(0.738855592065)*x_ref[0]+(0.511369798951)*x_ref[1]**o+(0.455794112842)*x_ref[1]
        ref[1,1,1,0]=(0.783504599076)*x_ref[0]**o+(-0.558579246854)*x_ref[0]+(0.608044912637)*x_ref[1]**o+(-0.401058633268)*x_ref[1]
        ref[1,1,1,1]=(0.406697861117)*x_ref[0]**o+(-0.31302718156)*x_ref[0]+(-0.999553049864)*x_ref[1]**o+(0.647698243869)*x_ref[1]
        ref[1,1,2,0]=(-0.788346252365)*x_ref[0]**o+(-0.809895265253)*x_ref[0]+(0.441738551485)*x_ref[1]**o+(-0.686785865679)*x_ref[1]
        ref[1,1,2,1]=(0.794571346519)*x_ref[0]**o+(-0.552071697712)*x_ref[0]+(0.77260218173)*x_ref[1]**o+(-0.969584549528)*x_ref[1]
        ref[1,2,0,0]=(-0.867948229325)*x_ref[0]**o+(-0.582387320474)*x_ref[0]+(0.0387726756335)*x_ref[1]**o+(0.4143770831)*x_ref[1]
        ref[1,2,0,1]=(-0.913557875303)*x_ref[0]**o+(-0.517970834439)*x_ref[0]+(-0.922517499063)*x_ref[1]**o+(0.858552054359)*x_ref[1]
        ref[1,2,1,0]=(0.409302016181)*x_ref[0]**o+(-0.684529177441)*x_ref[0]+(0.818122023218)*x_ref[1]**o+(-0.583589419623)*x_ref[1]
        ref[1,2,1,1]=(0.750795883056)*x_ref[0]**o+(-0.710882726624)*x_ref[0]+(0.915210035315)*x_ref[1]**o+(0.802588664027)*x_ref[1]
        ref[1,2,2,0]=(0.639216566804)*x_ref[0]**o+(-0.300954214125)*x_ref[0]+(-0.26724393342)*x_ref[1]**o+(0.426742851293)*x_ref[1]
        ref[1,2,2,1]=(-0.00337538020121)*x_ref[0]**o+(0.283012169215)*x_ref[0]+(0.725040731001)*x_ref[1]**o+(-0.708238265029)*x_ref[1]
        ref[1,3,0,0]=(0.217404922432)*x_ref[0]**o+(-0.590883003502)*x_ref[0]+(0.451142849723)*x_ref[1]**o+(0.624698767392)*x_ref[1]
        ref[1,3,0,1]=(0.0351258739828)*x_ref[0]**o+(-0.784068492686)*x_ref[0]+(0.469956184167)*x_ref[1]**o+(0.634124784387)*x_ref[1]
        ref[1,3,1,0]=(0.998414775431)*x_ref[0]**o+(0.388202946918)*x_ref[0]+(-0.849223443255)*x_ref[1]**o+(-0.804590028218)*x_ref[1]
        ref[1,3,1,1]=(0.0219748851856)*x_ref[0]**o+(0.464069403336)*x_ref[0]+(-0.470759541095)*x_ref[1]**o+(-0.21773573235)*x_ref[1]
        ref[1,3,2,0]=(-0.879175905661)*x_ref[0]**o+(0.100238280861)*x_ref[0]+(0.331472085056)*x_ref[1]**o+(0.0310947651268)*x_ref[1]
        ref[1,3,2,1]=(-0.195155740038)*x_ref[0]**o+(0.532078676231)*x_ref[0]+(-0.863288082011)*x_ref[1]**o+(-0.0905878788265)*x_ref[1]
        ref[1,4,0,0]=(0.470883637487)*x_ref[0]**o+(0.0291028245963)*x_ref[0]+(-0.152462235474)*x_ref[1]**o+(0.890883963301)*x_ref[1]
        ref[1,4,0,1]=(-0.0780797534409)*x_ref[0]**o+(-0.999649003413)*x_ref[0]+(-0.79446765004)*x_ref[1]**o+(-0.820520881428)*x_ref[1]
        ref[1,4,1,0]=(-0.82479629638)*x_ref[0]**o+(-0.28924829922)*x_ref[0]+(-0.960430867949)*x_ref[1]**o+(-0.514730558673)*x_ref[1]
        ref[1,4,1,1]=(-0.0853911110391)*x_ref[0]**o+(0.726747270178)*x_ref[0]+(-0.70925711586)*x_ref[1]**o+(-0.331433386035)*x_ref[1]
        ref[1,4,2,0]=(-0.434350343402)*x_ref[0]**o+(0.335341284415)*x_ref[0]+(0.496818240476)*x_ref[1]**o+(-0.925006191184)*x_ref[1]
        ref[1,4,2,1]=(0.0753516047957)*x_ref[0]**o+(0.171090641529)*x_ref[0]+(-0.836002762987)*x_ref[1]**o+(-0.753836171615)*x_ref[1]
        ref[2,0,0,0]=(-0.331008073553)*x_ref[0]**o+(-0.299051880715)*x_ref[0]+(-0.84520124413)*x_ref[1]**o+(0.780471473975)*x_ref[1]
        ref[2,0,0,1]=(-0.947973851318)*x_ref[0]**o+(0.579142212501)*x_ref[0]+(0.137317760268)*x_ref[1]**o+(-0.379509530229)*x_ref[1]
        ref[2,0,1,0]=(-0.741320730287)*x_ref[0]**o+(0.642110710824)*x_ref[0]+(-0.632595944259)*x_ref[1]**o+(0.944980046528)*x_ref[1]
        ref[2,0,1,1]=(0.616943682269)*x_ref[0]**o+(-0.348189507611)*x_ref[0]+(-0.414080070092)*x_ref[1]**o+(-0.733041081362)*x_ref[1]
        ref[2,0,2,0]=(-0.021261659709)*x_ref[0]**o+(0.790041129031)*x_ref[0]+(0.788378634137)*x_ref[1]**o+(0.391057560074)*x_ref[1]
        ref[2,0,2,1]=(0.563737321295)*x_ref[0]**o+(-0.110516278477)*x_ref[0]+(0.383563652425)*x_ref[1]**o+(-0.857100387101)*x_ref[1]
        ref[2,1,0,0]=(-0.0839346911978)*x_ref[0]**o+(0.699255867073)*x_ref[0]+(-0.698488781835)*x_ref[1]**o+(0.225940657322)*x_ref[1]
        ref[2,1,0,1]=(0.93638601214)*x_ref[0]**o+(-0.593516691147)*x_ref[0]+(-0.445863239521)*x_ref[1]**o+(0.601067357746)*x_ref[1]
        ref[2,1,1,0]=(0.114814804735)*x_ref[0]**o+(0.717600499991)*x_ref[0]+(0.7371911818)*x_ref[1]**o+(0.136012416552)*x_ref[1]
        ref[2,1,1,1]=(0.794499830148)*x_ref[0]**o+(0.0581947415855)*x_ref[0]+(0.626382859976)*x_ref[1]**o+(-0.668272098674)*x_ref[1]
        ref[2,1,2,0]=(0.564628771073)*x_ref[0]**o+(0.102820241267)*x_ref[0]+(0.397960234363)*x_ref[1]**o+(0.379632827896)*x_ref[1]
        ref[2,1,2,1]=(-0.0880639775913)*x_ref[0]**o+(0.325978376801)*x_ref[0]+(-0.505498229733)*x_ref[1]**o+(-0.595922646655)*x_ref[1]
        ref[2,2,0,0]=(-0.132672737928)*x_ref[0]**o+(-0.0291240195523)*x_ref[0]+(-0.00825944211706)*x_ref[1]**o+(0.985993963145)*x_ref[1]
        ref[2,2,0,1]=(0.244321497515)*x_ref[0]**o+(-0.111329005236)*x_ref[0]+(0.769353471123)*x_ref[1]**o+(-0.174257027136)*x_ref[1]
        ref[2,2,1,0]=(0.252286330282)*x_ref[0]**o+(0.450975615825)*x_ref[0]+(-0.515016578606)*x_ref[1]**o+(0.834736757555)*x_ref[1]
        ref[2,2,1,1]=(0.0588273338962)*x_ref[0]**o+(-0.865406338788)*x_ref[0]+(0.482095645149)*x_ref[1]**o+(-0.595126490156)*x_ref[1]
        ref[2,2,2,0]=(0.244580063555)*x_ref[0]**o+(0.576709884892)*x_ref[0]+(-0.212616562122)*x_ref[1]**o+(-0.407508050723)*x_ref[1]
        ref[2,2,2,1]=(0.416825783755)*x_ref[0]**o+(0.430959078988)*x_ref[0]+(-0.398840301481)*x_ref[1]**o+(0.889105964295)*x_ref[1]
        ref[2,3,0,0]=(-0.731872136283)*x_ref[0]**o+(0.237034931745)*x_ref[0]+(0.547737117698)*x_ref[1]**o+(0.364733861661)*x_ref[1]
        ref[2,3,0,1]=(0.971716549689)*x_ref[0]**o+(0.466300970081)*x_ref[0]+(-0.51802162747)*x_ref[1]**o+(-0.859910778018)*x_ref[1]
        ref[2,3,1,0]=(-0.562918219893)*x_ref[0]**o+(-0.0532549959391)*x_ref[0]+(0.842033495359)*x_ref[1]**o+(-0.708554283815)*x_ref[1]
        ref[2,3,1,1]=(0.437125534104)*x_ref[0]**o+(-0.376751615116)*x_ref[0]+(-0.330889383623)*x_ref[1]**o+(0.958884035289)*x_ref[1]
        ref[2,3,2,0]=(-0.194290501566)*x_ref[0]**o+(-0.368131904292)*x_ref[0]+(0.802917650409)*x_ref[1]**o+(0.628603987906)*x_ref[1]
        ref[2,3,2,1]=(0.70219875881)*x_ref[0]**o+(0.822181339005)*x_ref[0]+(0.768635447234)*x_ref[1]**o+(0.871676010439)*x_ref[1]
        ref[2,4,0,0]=(-0.654498700662)*x_ref[0]**o+(0.851143676147)*x_ref[0]+(0.468468395415)*x_ref[1]**o+(-0.447240713576)*x_ref[1]
        ref[2,4,0,1]=(0.893177052112)*x_ref[0]**o+(0.652743480988)*x_ref[0]+(0.726646163682)*x_ref[1]**o+(-0.854895731892)*x_ref[1]
        ref[2,4,1,0]=(0.0540542301334)*x_ref[0]**o+(-0.208654443984)*x_ref[0]+(0.925127576853)*x_ref[1]**o+(-0.932489814868)*x_ref[1]
        ref[2,4,1,1]=(-0.460268214104)*x_ref[0]**o+(-0.303569308977)*x_ref[0]+(-0.381458880316)*x_ref[1]**o+(0.690537661673)*x_ref[1]
        ref[2,4,2,0]=(0.401012366366)*x_ref[0]**o+(-0.970993441679)*x_ref[0]+(0.0249535421254)*x_ref[1]**o+(-0.499737254014)*x_ref[1]
        ref[2,4,2,1]=(-0.692604790859)*x_ref[0]**o+(-0.685292098725)*x_ref[0]+(0.22550357354)*x_ref[1]**o+(0.908152783462)*x_ref[1]
        ref[3,0,0,0]=(-0.634246407254)*x_ref[0]**o+(-0.571223033336)*x_ref[0]+(0.0574319920181)*x_ref[1]**o+(0.123763929908)*x_ref[1]
        ref[3,0,0,1]=(-0.569358219213)*x_ref[0]**o+(-0.107237509632)*x_ref[0]+(0.97672999497)*x_ref[1]**o+(-0.114939366813)*x_ref[1]
        ref[3,0,1,0]=(-0.767885152479)*x_ref[0]**o+(-0.105924102555)*x_ref[0]+(0.539361607398)*x_ref[1]**o+(0.546186446469)*x_ref[1]
        ref[3,0,1,1]=(-0.58818696897)*x_ref[0]**o+(0.629371516566)*x_ref[0]+(-0.598173904376)*x_ref[1]**o+(0.959043112088)*x_ref[1]
        ref[3,0,2,0]=(0.751165393927)*x_ref[0]**o+(0.732151619397)*x_ref[0]+(-0.435276735735)*x_ref[1]**o+(0.947142140722)*x_ref[1]
        ref[3,0,2,1]=(0.463904349547)*x_ref[0]**o+(-0.947270841014)*x_ref[0]+(0.304239969784)*x_ref[1]**o+(-0.233566944292)*x_ref[1]
        ref[3,1,0,0]=(0.961122232395)*x_ref[0]**o+(-0.989040215156)*x_ref[0]+(0.833186746232)*x_ref[1]**o+(0.226673105305)*x_ref[1]
        ref[3,1,0,1]=(-0.156957616578)*x_ref[0]**o+(-0.361665587314)*x_ref[0]+(0.987490273947)*x_ref[1]**o+(-0.890271058746)*x_ref[1]
        ref[3,1,1,0]=(-0.259959222953)*x_ref[0]**o+(-0.696135598749)*x_ref[0]+(0.234411151165)*x_ref[1]**o+(0.783105897256)*x_ref[1]
        ref[3,1,1,1]=(0.020886322199)*x_ref[0]**o+(0.887445555864)*x_ref[0]+(0.709489180254)*x_ref[1]**o+(0.990590112507)*x_ref[1]
        ref[3,1,2,0]=(-0.963790247866)*x_ref[0]**o+(-0.417180971158)*x_ref[0]+(0.303859261646)*x_ref[1]**o+(0.745552024399)*x_ref[1]
        ref[3,1,2,1]=(-0.364448135641)*x_ref[0]**o+(0.760841571414)*x_ref[0]+(-0.0283374186763)*x_ref[1]**o+(-0.491433855748)*x_ref[1]
        ref[3,2,0,0]=(-0.649766142291)*x_ref[0]**o+(-0.177410238297)*x_ref[0]+(0.137316641289)*x_ref[1]**o+(-0.40890299152)*x_ref[1]
        ref[3,2,0,1]=(0.282922984657)*x_ref[0]**o+(-0.660238152943)*x_ref[0]+(0.481726052488)*x_ref[1]**o+(0.976011254218)*x_ref[1]
        ref[3,2,1,0]=(0.989761520473)*x_ref[0]**o+(0.194082255647)*x_ref[0]+(-0.648435682551)*x_ref[1]**o+(-0.721569599311)*x_ref[1]
        ref[3,2,1,1]=(0.734634540883)*x_ref[0]**o+(0.524638529049)*x_ref[0]+(-0.737636791717)*x_ref[1]**o+(-0.995904525822)*x_ref[1]
        ref[3,2,2,0]=(0.422396808663)*x_ref[0]**o+(-0.820639154105)*x_ref[0]+(-0.566207315373)*x_ref[1]**o+(-0.224174711984)*x_ref[1]
        ref[3,2,2,1]=(0.719145374997)*x_ref[0]**o+(-0.885860499468)*x_ref[0]+(0.184062878609)*x_ref[1]**o+(0.47660543007)*x_ref[1]
        ref[3,3,0,0]=(-0.0122290849915)*x_ref[0]**o+(-0.0314298339876)*x_ref[0]+(0.937255407134)*x_ref[1]**o+(-0.754858547072)*x_ref[1]
        ref[3,3,0,1]=(-0.507826956669)*x_ref[0]**o+(0.951205020689)*x_ref[0]+(0.837096245109)*x_ref[1]**o+(-0.576692399518)*x_ref[1]
        ref[3,3,1,0]=(0.602119808238)*x_ref[0]**o+(-0.943937134298)*x_ref[0]+(-0.244250254626)*x_ref[1]**o+(-0.814103542727)*x_ref[1]
        ref[3,3,1,1]=(-0.51811661822)*x_ref[0]**o+(-0.188969078762)*x_ref[0]+(-0.697470816837)*x_ref[1]**o+(-0.741593014841)*x_ref[1]
        ref[3,3,2,0]=(-0.673948933786)*x_ref[0]**o+(0.816404336566)*x_ref[0]+(0.339239090092)*x_ref[1]**o+(-0.0173550947113)*x_ref[1]
        ref[3,3,2,1]=(0.499109852716)*x_ref[0]**o+(0.515501552893)*x_ref[0]+(0.926442939974)*x_ref[1]**o+(0.157227173254)*x_ref[1]
        ref[3,4,0,0]=(-0.1837389273)*x_ref[0]**o+(0.78490229721)*x_ref[0]+(-0.248033788374)*x_ref[1]**o+(0.735136386405)*x_ref[1]
        ref[3,4,0,1]=(0.121169550194)*x_ref[0]**o+(-0.854027732077)*x_ref[0]+(0.937680848475)*x_ref[1]**o+(-0.447184197264)*x_ref[1]
        ref[3,4,1,0]=(-0.788610934694)*x_ref[0]**o+(-0.397825388144)*x_ref[0]+(0.604421361449)*x_ref[1]**o+(0.937184552821)*x_ref[1]
        ref[3,4,1,1]=(-0.493380143496)*x_ref[0]**o+(0.958197000398)*x_ref[0]+(0.282372561751)*x_ref[1]**o+(-0.55322669826)*x_ref[1]
        ref[3,4,2,0]=(0.11333865923)*x_ref[0]**o+(-0.542805511569)*x_ref[0]+(-0.640646701928)*x_ref[1]**o+(-0.342030716597)*x_ref[1]
        ref[3,4,2,1]=(0.505356266297)*x_ref[0]**o+(-0.107422782294)*x_ref[0]+(0.542833447968)*x_ref[1]**o+(-0.615275716592)*x_ref[1]
      else:
        arg[0,0,0,0]=(0.507422888369)*x[0]**o+(0.808798513305)*x[0]+(0.51493073805)*x[1]**o+(-0.928527686947)*x[1]+(-0.788010447498)*x[2]**o+(-0.378389833131)*x[2]
        arg[0,0,0,1]=(-0.475851243177)*x[0]**o+(0.746578446988)*x[0]+(0.504778622494)*x[1]**o+(-0.677770398899)*x[1]+(-0.862535648403)*x[2]**o+(0.620414141691)*x[2]
        arg[0,0,1,0]=(-0.361726879582)*x[0]**o+(0.554584127587)*x[0]+(0.436610414772)*x[1]**o+(-0.686534492717)*x[1]+(0.757226061419)*x[2]**o+(0.0948537725389)*x[2]
        arg[0,0,1,1]=(-0.362160696355)*x[0]**o+(0.766524051703)*x[0]+(0.209980857795)*x[1]**o+(-0.478398061437)*x[1]+(-0.410960179122)*x[2]**o+(-0.213272138462)*x[2]
        arg[0,0,2,0]=(0.87154872554)*x[0]**o+(-0.727915301904)*x[0]+(0.683580698056)*x[1]**o+(0.215052521277)*x[1]+(0.267231507898)*x[2]**o+(-0.0124189912449)*x[2]
        arg[0,0,2,1]=(0.631400944876)*x[0]**o+(0.537293628606)*x[0]+(0.944754350994)*x[1]**o+(0.108068908012)*x[1]+(0.890117096365)*x[2]**o+(0.164706867235)*x[2]
        arg[0,1,0,0]=(-0.935871759207)*x[0]**o+(0.350133758077)*x[0]+(-0.966482181015)*x[1]**o+(0.440902701528)*x[1]+(-0.893329541322)*x[2]**o+(0.350317827938)*x[2]
        arg[0,1,0,1]=(0.465305203601)*x[0]**o+(0.711131797928)*x[0]+(0.104091536498)*x[1]**o+(0.484811628127)*x[1]+(0.676878954516)*x[2]**o+(-0.99327016995)*x[2]
        arg[0,1,1,0]=(-0.361189016151)*x[0]**o+(0.962468573492)*x[0]+(-0.158571122564)*x[1]**o+(-0.293583660912)*x[1]+(-0.713699173608)*x[2]**o+(-0.507430163361)*x[2]
        arg[0,1,1,1]=(0.172564527054)*x[0]**o+(0.273438145027)*x[0]+(0.137378266239)*x[1]**o+(-0.675788606841)*x[1]+(-0.362143416121)*x[2]**o+(0.0703897749775)*x[2]
        arg[0,1,2,0]=(-0.941374717271)*x[0]**o+(0.289433165458)*x[0]+(0.318615227028)*x[1]**o+(0.524362568204)*x[1]+(-0.612590305095)*x[2]**o+(-0.117555538957)*x[2]
        arg[0,1,2,1]=(-0.442920825551)*x[0]**o+(-0.454982146579)*x[0]+(0.355353737589)*x[1]**o+(-0.729623917004)*x[1]+(-0.567220335937)*x[2]**o+(0.977652639976)*x[2]
        arg[0,2,0,0]=(0.942485930373)*x[0]**o+(-0.605823961033)*x[0]+(-0.143465779338)*x[1]**o+(0.327354308987)*x[1]+(0.691900417542)*x[2]**o+(-0.514165213849)*x[2]
        arg[0,2,0,1]=(0.018029272977)*x[0]**o+(-0.209992705469)*x[0]+(-0.224614727554)*x[1]**o+(-0.862697462048)*x[1]+(0.274791387928)*x[2]**o+(-0.907445869506)*x[2]
        arg[0,2,1,0]=(0.862755248265)*x[0]**o+(0.020831641939)*x[0]+(0.496662599261)*x[1]**o+(0.79318975953)*x[1]+(0.0912166895541)*x[2]**o+(0.556337610316)*x[2]
        arg[0,2,1,1]=(-0.09652997071)*x[0]**o+(-0.954279421394)*x[0]+(-0.601053524018)*x[1]**o+(0.642528012068)*x[1]+(0.295748113888)*x[2]**o+(-0.318166346186)*x[2]
        arg[0,2,2,0]=(-0.914878854129)*x[0]**o+(0.159201123642)*x[0]+(0.589819056368)*x[1]**o+(-0.715763226823)*x[1]+(-0.444572583567)*x[2]**o+(-0.487866651367)*x[2]
        arg[0,2,2,1]=(-0.71271969175)*x[0]**o+(0.454616308594)*x[0]+(0.510270095021)*x[1]**o+(-0.0385822967955)*x[1]+(-0.519736540313)*x[2]**o+(0.0556733091556)*x[2]
        arg[0,3,0,0]=(0.355768034846)*x[0]**o+(0.152240594627)*x[0]+(-0.81081131843)*x[1]**o+(-0.913373483156)*x[1]+(-0.319859744799)*x[2]**o+(-0.306218435393)*x[2]
        arg[0,3,0,1]=(-0.656932063693)*x[0]**o+(0.832886861783)*x[0]+(-0.417112750683)*x[1]**o+(-0.388190265957)*x[1]+(0.246534947096)*x[2]**o+(-0.896822839189)*x[2]
        arg[0,3,1,0]=(0.204713418891)*x[0]**o+(-0.150097488817)*x[0]+(0.806578426941)*x[1]**o+(-0.590052708995)*x[1]+(-0.758958608419)*x[2]**o+(-0.221973528232)*x[2]
        arg[0,3,1,1]=(-0.854308184618)*x[0]**o+(-0.292526203646)*x[0]+(-0.662403504833)*x[1]**o+(0.0955657982227)*x[1]+(0.69471076173)*x[2]**o+(0.384077309264)*x[2]
        arg[0,3,2,0]=(0.982057200864)*x[0]**o+(-0.64758727704)*x[0]+(0.15941493222)*x[1]**o+(-0.17745437115)*x[1]+(-0.72953252955)*x[2]**o+(0.986979270802)*x[2]
        arg[0,3,2,1]=(0.764526698779)*x[0]**o+(0.22809521115)*x[0]+(-0.0762092087998)*x[1]**o+(0.415212410471)*x[1]+(-0.263638988463)*x[2]**o+(-0.934669942336)*x[2]
        arg[0,4,0,0]=(0.0812656932707)*x[0]**o+(0.0397235062518)*x[0]+(-0.0496912823983)*x[1]**o+(0.189891585189)*x[1]+(0.243537823293)*x[2]**o+(0.461256134208)*x[2]
        arg[0,4,0,1]=(0.167596358861)*x[0]**o+(0.441650690481)*x[0]+(0.638100632205)*x[1]**o+(-0.69240615765)*x[1]+(-0.501855369512)*x[2]**o+(0.769465683782)*x[2]
        arg[0,4,1,0]=(-0.137149398927)*x[0]**o+(-0.186548325285)*x[0]+(-0.504656948352)*x[1]**o+(0.228896656134)*x[1]+(-0.472425358061)*x[2]**o+(-0.0769834754996)*x[2]
        arg[0,4,1,1]=(0.311655648626)*x[0]**o+(0.856513683959)*x[0]+(0.47173626537)*x[1]**o+(0.245497333467)*x[1]+(-0.513727691305)*x[2]**o+(-0.588614785682)*x[2]
        arg[0,4,2,0]=(-0.152907854625)*x[0]**o+(-0.86229950575)*x[0]+(-0.931356414564)*x[1]**o+(-0.915667991864)*x[1]+(0.322316118527)*x[2]**o+(-0.987559462539)*x[2]
        arg[0,4,2,1]=(-0.705947769164)*x[0]**o+(0.104081366627)*x[0]+(-0.540454644392)*x[1]**o+(0.404522880836)*x[1]+(0.146464264141)*x[2]**o+(0.586755030006)*x[2]
        arg[1,0,0,0]=(-0.466614768106)*x[0]**o+(0.482681414943)*x[0]+(-0.176553738108)*x[1]**o+(-0.39162272813)*x[1]+(0.74312023324)*x[2]**o+(0.824538905894)*x[2]
        arg[1,0,0,1]=(-0.9351073631)*x[0]**o+(-0.640226640449)*x[0]+(0.966208762054)*x[1]**o+(0.101792167944)*x[1]+(-0.732983657038)*x[2]**o+(-0.112313375937)*x[2]
        arg[1,0,1,0]=(-0.305403470434)*x[0]**o+(-0.414397930347)*x[0]+(0.356093254322)*x[1]**o+(-0.52607560749)*x[1]+(-0.70070674473)*x[2]**o+(-0.0192901616015)*x[2]
        arg[1,0,1,1]=(0.450046989211)*x[0]**o+(0.602770057268)*x[0]+(0.0193812509113)*x[1]**o+(-0.275615149191)*x[1]+(0.0180468267836)*x[2]**o+(-0.81388255113)*x[2]
        arg[1,0,2,0]=(0.913651483249)*x[0]**o+(-0.92188313743)*x[0]+(0.680007150195)*x[1]**o+(-0.300145584708)*x[1]+(-0.398883520002)*x[2]**o+(-0.752781496463)*x[2]
        arg[1,0,2,1]=(-0.265749827845)*x[0]**o+(0.594379728352)*x[0]+(0.463409943587)*x[1]**o+(-0.85840420426)*x[1]+(-0.705963756674)*x[2]**o+(0.251711112887)*x[2]
        arg[1,1,0,0]=(-0.28895690781)*x[0]**o+(0.476461656166)*x[0]+(0.0692567954597)*x[1]**o+(0.88763437878)*x[1]+(0.0872256197554)*x[2]**o+(0.19467039694)*x[2]
        arg[1,1,0,1]=(0.741045418981)*x[0]**o+(-0.726935151332)*x[0]+(0.899418662131)*x[1]**o+(0.27516497286)*x[1]+(-0.715927474971)*x[2]**o+(-0.709283098732)*x[2]
        arg[1,1,1,0]=(0.991407130174)*x[0]**o+(-0.163484265852)*x[0]+(-0.983254548659)*x[1]**o+(0.0568443855477)*x[1]+(0.309664644588)*x[2]**o+(0.904334718845)*x[2]
        arg[1,1,1,1]=(0.237261672833)*x[0]**o+(0.0861047746345)*x[0]+(-0.494898018662)*x[1]**o+(-0.433075334646)*x[1]+(0.588597346505)*x[2]**o+(-0.185559567553)*x[2]
        arg[1,1,2,0]=(-0.969384091709)*x[0]**o+(-0.0229777223491)*x[0]+(0.0945623059666)*x[1]**o+(0.303543647034)*x[1]+(0.659000135381)*x[2]**o+(0.00873791711254)*x[2]
        arg[1,1,2,1]=(-0.421406731882)*x[0]**o+(-0.5017394377)*x[0]+(0.027747130694)*x[1]**o+(0.327278873996)*x[1]+(-0.754133439304)*x[2]**o+(0.673525803845)*x[2]
        arg[1,2,0,0]=(-0.104620319685)*x[0]**o+(-0.685602393278)*x[0]+(0.972173717014)*x[1]**o+(-0.440573017063)*x[1]+(-0.709529395743)*x[2]**o+(0.634170741457)*x[2]
        arg[1,2,0,1]=(-0.560386754264)*x[0]**o+(0.570894623654)*x[0]+(-0.455122833362)*x[1]**o+(-0.00322505937935)*x[1]+(0.959400336149)*x[2]**o+(0.64014859954)*x[2]
        arg[1,2,1,0]=(0.100585233221)*x[0]**o+(0.939532360957)*x[0]+(-0.900301955112)*x[1]**o+(-0.776375896786)*x[1]+(-0.779373294746)*x[2]**o+(0.546157461116)*x[2]
        arg[1,2,1,1]=(-0.680457315071)*x[0]**o+(-0.0278844539208)*x[0]+(-0.629673590044)*x[1]**o+(0.813218376348)*x[1]+(-0.449484131841)*x[2]**o+(0.641682150048)*x[2]
        arg[1,2,2,0]=(0.60056954386)*x[0]**o+(-0.763169434657)*x[0]+(-0.672890915035)*x[1]**o+(-0.368711154571)*x[1]+(0.476810195318)*x[2]**o+(-0.935071124382)*x[2]
        arg[1,2,2,1]=(0.994100148333)*x[0]**o+(0.471647033381)*x[0]+(0.341677296654)*x[1]**o+(0.466442262068)*x[1]+(-0.0634919249117)*x[2]**o+(0.0637348830912)*x[2]
        arg[1,3,0,0]=(-0.120744286726)*x[0]**o+(0.630378984537)*x[0]+(0.0837817796507)*x[1]**o+(-0.850115274881)*x[1]+(0.574342626561)*x[2]**o+(-0.705409915626)*x[2]
        arg[1,3,0,1]=(-0.51181061391)*x[0]**o+(0.711479747616)*x[0]+(-0.00800928076734)*x[1]**o+(-0.703424580277)*x[1]+(-0.925935026157)*x[2]**o+(0.488175992827)*x[2]
        arg[1,3,1,0]=(0.531879615751)*x[0]**o+(-0.214270521934)*x[0]+(-0.432344726015)*x[1]**o+(-0.107660826449)*x[1]+(0.763406427398)*x[2]**o+(-0.511000150429)*x[2]
        arg[1,3,1,1]=(-0.867467568206)*x[0]**o+(-0.738425961596)*x[0]+(-0.925337667706)*x[1]**o+(-0.586227765107)*x[1]+(0.909987211462)*x[2]**o+(0.533347022582)*x[2]
        arg[1,3,2,0]=(-0.47713113862)*x[0]**o+(-0.130876138883)*x[0]+(-0.607996835981)*x[1]**o+(-0.0306261551172)*x[1]+(0.956207500475)*x[2]**o+(-0.287731719382)*x[2]
        arg[1,3,2,1]=(0.179479486943)*x[0]**o+(0.361266562479)*x[0]+(0.374904472267)*x[1]**o+(0.00788379106301)*x[1]+(-0.34090684265)*x[2]**o+(-0.190875910791)*x[2]
        arg[1,4,0,0]=(0.685464203729)*x[0]**o+(0.0634561930987)*x[0]+(-0.662534110385)*x[1]**o+(0.383849898086)*x[1]+(0.948230524205)*x[2]**o+(-0.347042281976)*x[2]
        arg[1,4,0,1]=(-0.484499755322)*x[0]**o+(-0.765655349243)*x[0]+(0.805331514606)*x[1]**o+(-0.0635912547496)*x[1]+(-0.139457134772)*x[2]**o+(0.227446734221)*x[2]
        arg[1,4,1,0]=(-0.913971229647)*x[0]**o+(0.042531889985)*x[0]+(0.143956410289)*x[1]**o+(0.597990264168)*x[1]+(-0.85926903006)*x[2]**o+(0.205900361792)*x[2]
        arg[1,4,1,1]=(-0.955540264653)*x[0]**o+(0.983439073814)*x[0]+(0.246870351555)*x[1]**o+(-0.610917180843)*x[1]+(-0.0418134447092)*x[2]**o+(-0.58407787472)*x[2]
        arg[1,4,2,0]=(0.787274711453)*x[0]**o+(-0.899651339467)*x[0]+(-0.62195254225)*x[1]**o+(-0.516447289251)*x[1]+(0.267082643601)*x[2]**o+(0.0408186039688)*x[2]
        arg[1,4,2,1]=(-0.485773801895)*x[0]**o+(0.663478067655)*x[0]+(0.928973237869)*x[1]**o+(0.353735145156)*x[1]+(0.128409748978)*x[2]**o+(-0.476479648592)*x[2]
        arg[2,0,0,0]=(0.491574821811)*x[0]**o+(0.455303820526)*x[0]+(-0.747773112075)*x[1]**o+(-0.653255987137)*x[1]+(-0.738293040346)*x[2]**o+(0.948070477272)*x[2]
        arg[2,0,0,1]=(-0.697412782403)*x[0]**o+(-0.738389544586)*x[0]+(0.231728579935)*x[1]**o+(0.248503931032)*x[1]+(-0.561155590703)*x[2]**o+(0.324682113147)*x[2]
        arg[2,0,1,0]=(-0.00507638382064)*x[0]**o+(-0.983105866448)*x[0]+(-0.253141627484)*x[1]**o+(-0.0664178091069)*x[1]+(0.194460220144)*x[2]**o+(-0.174257469198)*x[2]
        arg[2,0,1,1]=(-0.644447560463)*x[0]**o+(0.658703761133)*x[0]+(-0.685392825104)*x[1]**o+(-0.766295260796)*x[1]+(-0.922004388493)*x[2]**o+(-0.0745707720923)*x[2]
        arg[2,0,2,0]=(-0.69924009606)*x[0]**o+(-0.398212597328)*x[0]+(-0.364991550186)*x[1]**o+(0.243474288977)*x[1]+(0.129545162767)*x[2]**o+(0.920611954845)*x[2]
        arg[2,0,2,1]=(-0.494476126875)*x[0]**o+(-0.131503103148)*x[0]+(-0.241670500874)*x[1]**o+(0.40439125905)*x[1]+(0.134166779399)*x[2]**o+(-0.507385751809)*x[2]
        arg[2,1,0,0]=(0.301148115913)*x[0]**o+(0.189635133395)*x[0]+(0.744167404774)*x[1]**o+(0.995754344203)*x[1]+(-0.334760662218)*x[2]**o+(-0.182462923741)*x[2]
        arg[2,1,0,1]=(0.523500022508)*x[0]**o+(-0.257042661558)*x[0]+(-0.940233100381)*x[1]**o+(-0.202431731891)*x[1]+(-0.653480190287)*x[2]**o+(0.993156915081)*x[2]
        arg[2,1,1,0]=(-0.502452234661)*x[0]**o+(-0.711581805303)*x[0]+(0.696450091152)*x[1]**o+(0.436746953802)*x[1]+(0.695969418438)*x[2]**o+(-0.671400022595)*x[2]
        arg[2,1,1,1]=(0.0705944125182)*x[0]**o+(-0.865755834485)*x[0]+(0.921555996641)*x[1]**o+(-0.567855814858)*x[1]+(-0.213455192388)*x[2]**o+(0.820868399259)*x[2]
        arg[2,1,2,0]=(0.744232126707)*x[0]**o+(0.336969176485)*x[0]+(0.339407729288)*x[1]**o+(-0.465154968917)*x[1]+(0.389524167202)*x[2]**o+(0.645394954477)*x[2]
        arg[2,1,2,1]=(-0.299085687516)*x[0]**o+(0.220795864743)*x[0]+(-0.19353913538)*x[1]**o+(0.445131396297)*x[1]+(-0.779605318636)*x[2]**o+(-0.76707264371)*x[2]
        arg[2,2,0,0]=(-0.357585997171)*x[0]**o+(-0.248795296406)*x[0]+(0.0876364714478)*x[1]**o+(-0.682833482179)*x[1]+(0.497052575986)*x[2]**o+(0.893378007889)*x[2]
        arg[2,2,0,1]=(0.38067999218)*x[0]**o+(-0.857246232026)*x[0]+(0.0785818284359)*x[1]**o+(-0.473287028123)*x[1]+(-0.763041318956)*x[2]**o+(-0.1119710251)*x[2]
        arg[2,2,1,0]=(-0.0278856926369)*x[0]**o+(0.435955626553)*x[0]+(-0.676274741782)*x[1]**o+(-0.719721825467)*x[1]+(0.195007734703)*x[2]**o+(0.504771278902)*x[2]
        arg[2,2,1,1]=(0.459547475772)*x[0]**o+(-0.669041681613)*x[0]+(-0.741957874364)*x[1]**o+(0.604585337407)*x[1]+(-0.886718742046)*x[2]**o+(0.338113237761)*x[2]
        arg[2,2,2,0]=(0.392926733678)*x[0]**o+(-0.923680589861)*x[0]+(-0.425503328457)*x[1]**o+(-0.0902669250117)*x[1]+(-0.346726072703)*x[2]**o+(0.541482976709)*x[2]
        arg[2,2,2,1]=(0.134867023616)*x[0]**o+(0.478349607348)*x[0]+(-0.334617836366)*x[1]**o+(-0.74587411744)*x[1]+(-0.880365937567)*x[2]**o+(0.820743064452)*x[2]
        arg[2,3,0,0]=(0.081184173895)*x[0]**o+(0.315011929905)*x[0]+(-0.137083926616)*x[1]**o+(0.103926223299)*x[1]+(0.363270989968)*x[2]**o+(-0.454075563606)*x[2]
        arg[2,3,0,1]=(0.457730701321)*x[0]**o+(0.922345625691)*x[0]+(-0.412933933474)*x[1]**o+(-0.546359655103)*x[1]+(0.719101029933)*x[2]**o+(-0.790522859322)*x[2]
        arg[2,3,1,0]=(-0.811816419841)*x[0]**o+(0.0379298408788)*x[0]+(-0.852643125131)*x[1]**o+(0.76881274254)*x[1]+(0.75214553496)*x[2]**o+(0.800382814403)*x[2]
        arg[2,3,1,1]=(0.524802143618)*x[0]**o+(-0.717457078371)*x[0]+(0.389026887965)*x[1]**o+(-0.231912558721)*x[1]+(-0.394705562333)*x[2]**o+(-0.794723802655)*x[2]
        arg[2,3,2,0]=(-0.232215520303)*x[0]**o+(0.994932250617)*x[0]+(-0.760425274222)*x[1]**o+(0.0910727760744)*x[1]+(0.621642024132)*x[2]**o+(0.263978719304)*x[2]
        arg[2,3,2,1]=(0.515269404424)*x[0]**o+(-0.998266958288)*x[0]+(-0.485078033534)*x[1]**o+(-0.960043247553)*x[1]+(-0.926562413314)*x[2]**o+(0.612710762712)*x[2]
        arg[2,4,0,0]=(0.419808446591)*x[0]**o+(0.132792190191)*x[0]+(-0.690603562736)*x[1]**o+(0.45137757885)*x[1]+(-0.262249041567)*x[2]**o+(-0.432995254783)*x[2]
        arg[2,4,0,1]=(0.62413323935)*x[0]**o+(0.963559148607)*x[0]+(-0.879044261015)*x[1]**o+(-0.504981574957)*x[1]+(-0.575204029973)*x[2]**o+(0.066855791448)*x[2]
        arg[2,4,1,0]=(0.321218662791)*x[0]**o+(0.975103526588)*x[0]+(0.783716185844)*x[1]**o+(-0.816368139389)*x[1]+(0.778169798128)*x[2]**o+(-0.527443068085)*x[2]
        arg[2,4,1,1]=(-0.8801082648)*x[0]**o+(-0.0313845929272)*x[0]+(0.485262237838)*x[1]**o+(0.0778888535573)*x[1]+(-0.900948214929)*x[2]**o+(-0.98830300089)*x[2]
        arg[2,4,2,0]=(-0.996395042226)*x[0]**o+(0.253635852642)*x[0]+(0.180965173659)*x[1]**o+(-0.488294662459)*x[1]+(-0.144583511741)*x[2]**o+(0.285854081388)*x[2]
        arg[2,4,2,1]=(0.00988506292061)*x[0]**o+(-0.00192786315452)*x[0]+(0.720775470647)*x[1]**o+(-0.587673311506)*x[1]+(0.313836238675)*x[2]**o+(0.45975125457)*x[2]
        arg[3,0,0,0]=(-0.822049489749)*x[0]**o+(-0.989589435175)*x[0]+(-0.0943830434118)*x[1]**o+(-0.789229900035)*x[1]+(0.311382619317)*x[2]**o+(-0.316646938685)*x[2]
        arg[3,0,0,1]=(0.501517916677)*x[0]**o+(-0.704502365436)*x[0]+(0.913184728006)*x[1]**o+(-0.843712848791)*x[1]+(0.901479696878)*x[2]**o+(-0.713893336542)*x[2]
        arg[3,0,1,0]=(0.0771592811086)*x[0]**o+(0.796038032206)*x[0]+(0.566256212871)*x[1]**o+(0.394136659986)*x[1]+(-0.794422065388)*x[2]**o+(0.662174992848)*x[2]
        arg[3,0,1,1]=(0.180225115167)*x[0]**o+(0.366965867971)*x[0]+(-0.244015756153)*x[1]**o+(0.433571067335)*x[1]+(0.129075150185)*x[2]**o+(-0.216304414605)*x[2]
        arg[3,0,2,0]=(0.100277967034)*x[0]**o+(-0.590583546786)*x[0]+(-0.595553697152)*x[1]**o+(0.923573252239)*x[1]+(0.578966663184)*x[2]**o+(-0.411082913097)*x[2]
        arg[3,0,2,1]=(-0.575315696811)*x[0]**o+(0.608760285918)*x[0]+(-0.3592074605)*x[1]**o+(0.515119945308)*x[1]+(-0.4000138306)*x[2]**o+(0.477685324937)*x[2]
        arg[3,1,0,0]=(0.0780014276919)*x[0]**o+(-0.0932830671471)*x[0]+(0.39477236642)*x[1]**o+(0.406673135556)*x[1]+(-0.235480707568)*x[2]**o+(-0.872302257144)*x[2]
        arg[3,1,0,1]=(0.170573072646)*x[0]**o+(0.868146561326)*x[0]+(0.262136343896)*x[1]**o+(0.192343529912)*x[1]+(0.157400965693)*x[2]**o+(-0.932981619473)*x[2]
        arg[3,1,1,0]=(0.891433179291)*x[0]**o+(-0.281396449997)*x[0]+(-0.0803173619772)*x[1]**o+(-0.378460054356)*x[1]+(-0.581578656014)*x[2]**o+(-0.65856067509)*x[2]
        arg[3,1,1,1]=(0.343599956626)*x[0]**o+(-0.92063694125)*x[0]+(0.440698102072)*x[1]**o+(0.767124166388)*x[1]+(0.050964317543)*x[2]**o+(1.35605562239e-05)*x[2]
        arg[3,1,2,0]=(-0.614268150987)*x[0]**o+(0.363806471556)*x[0]+(0.92193457905)*x[1]**o+(0.157919580642)*x[1]+(-0.955575321124)*x[2]**o+(-0.984808437666)*x[2]
        arg[3,1,2,1]=(0.030099648966)*x[0]**o+(-0.375999369491)*x[0]+(-0.0992643926008)*x[1]**o+(0.109209161093)*x[1]+(-0.974043780088)*x[2]**o+(0.908049658745)*x[2]
        arg[3,2,0,0]=(0.297771341106)*x[0]**o+(-0.637802228169)*x[0]+(0.398100441783)*x[1]**o+(-0.0643425181651)*x[1]+(0.662775339033)*x[2]**o+(0.744835182052)*x[2]
        arg[3,2,0,1]=(0.312026420583)*x[0]**o+(0.865855052574)*x[0]+(0.0195001783165)*x[1]**o+(0.537899238473)*x[1]+(0.554893357481)*x[2]**o+(0.71555610145)*x[2]
        arg[3,2,1,0]=(-0.199138146186)*x[0]**o+(0.827546489855)*x[0]+(-0.480965464736)*x[1]**o+(-0.473796093187)*x[1]+(0.880264675049)*x[2]**o+(0.544506670844)*x[2]
        arg[3,2,1,1]=(0.773798005008)*x[0]**o+(0.744945241634)*x[0]+(0.548170564608)*x[1]**o+(-0.560503692823)*x[1]+(-0.435630215427)*x[2]**o+(0.080015730707)*x[2]
        arg[3,2,2,0]=(0.36735714937)*x[0]**o+(-0.867071557873)*x[0]+(0.936135650431)*x[1]**o+(-0.432533600524)*x[1]+(-0.0224430204069)*x[2]**o+(0.218590302621)*x[2]
        arg[3,2,2,1]=(0.932808382499)*x[0]**o+(-0.0970016713319)*x[0]+(0.458943467579)*x[1]**o+(-0.134338789898)*x[1]+(-0.426892557314)*x[2]**o+(0.627071826863)*x[2]
        arg[3,3,0,0]=(-0.623804149075)*x[0]**o+(0.910423590172)*x[0]+(0.465938181097)*x[1]**o+(0.495479245391)*x[1]+(0.693745442185)*x[2]**o+(-0.437014226757)*x[2]
        arg[3,3,0,1]=(0.598807017496)*x[0]**o+(-0.112361840211)*x[0]+(-0.550899685213)*x[1]**o+(-0.0478222019564)*x[1]+(-0.0110129517693)*x[2]**o+(-0.059534786374)*x[2]
        arg[3,3,1,0]=(-0.561155633409)*x[0]**o+(-0.463563057636)*x[0]+(-0.170463996009)*x[1]**o+(-0.214509204942)*x[1]+(0.851200198666)*x[2]**o+(-0.721036329689)*x[2]
        arg[3,3,1,1]=(0.608666240405)*x[0]**o+(0.814232455115)*x[0]+(-0.604645629689)*x[1]**o+(0.960976509673)*x[1]+(0.365807996219)*x[2]**o+(-0.87491909238)*x[2]
        arg[3,3,2,0]=(0.120126988418)*x[0]**o+(-0.745687314825)*x[0]+(-0.332697826984)*x[1]**o+(-0.823683406516)*x[1]+(0.305444079613)*x[2]**o+(0.901972115441)*x[2]
        arg[3,3,2,1]=(0.0470293848904)*x[0]**o+(-0.195496754814)*x[0]+(0.671870114105)*x[1]**o+(0.170273146371)*x[1]+(-0.851927986188)*x[2]**o+(0.772192863973)*x[2]
        arg[3,4,0,0]=(-0.95392341727)*x[0]**o+(-0.665694215521)*x[0]+(-0.0247396499378)*x[1]**o+(0.441352857387)*x[1]+(0.767156547791)*x[2]**o+(-0.14696372018)*x[2]
        arg[3,4,0,1]=(0.209761166815)*x[0]**o+(0.978934473425)*x[0]+(0.943806891053)*x[1]**o+(-0.756619917601)*x[1]+(0.635699949764)*x[2]**o+(-0.345248269568)*x[2]
        arg[3,4,1,0]=(0.748020785785)*x[0]**o+(0.311104549542)*x[0]+(0.739127551248)*x[1]**o+(-0.635238575859)*x[1]+(0.623898875653)*x[2]**o+(0.343407392068)*x[2]
        arg[3,4,1,1]=(-0.422500892471)*x[0]**o+(0.0688495013346)*x[0]+(-0.919653906232)*x[1]**o+(0.531036586396)*x[1]+(-0.22487910672)*x[2]**o+(-0.848225994541)*x[2]
        arg[3,4,2,0]=(0.723965744673)*x[0]**o+(0.558602959224)*x[0]+(0.699539243308)*x[1]**o+(-0.597426541715)*x[1]+(-0.0128675070522)*x[2]**o+(-0.874796626242)*x[2]
        arg[3,4,2,1]=(-0.962970021216)*x[0]**o+(0.532886481874)*x[0]+(-0.523159705965)*x[1]**o+(-0.82887407638)*x[1]+(0.874288116843)*x[2]**o+(0.520024515804)*x[2]
        ref[0,0,0,0]=(0.507422888369)*x_ref[0]**o+(0.808798513305)*x_ref[0]+(0.51493073805)*x_ref[1]**o+(-0.928527686947)*x_ref[1]+(-0.788010447498)*x_ref[2]**o+(-0.378389833131)*x_ref[2]
        ref[0,0,0,1]=(-0.475851243177)*x_ref[0]**o+(0.746578446988)*x_ref[0]+(0.504778622494)*x_ref[1]**o+(-0.677770398899)*x_ref[1]+(-0.862535648403)*x_ref[2]**o+(0.620414141691)*x_ref[2]
        ref[0,0,1,0]=(-0.361726879582)*x_ref[0]**o+(0.554584127587)*x_ref[0]+(0.436610414772)*x_ref[1]**o+(-0.686534492717)*x_ref[1]+(0.757226061419)*x_ref[2]**o+(0.0948537725389)*x_ref[2]
        ref[0,0,1,1]=(-0.362160696355)*x_ref[0]**o+(0.766524051703)*x_ref[0]+(0.209980857795)*x_ref[1]**o+(-0.478398061437)*x_ref[1]+(-0.410960179122)*x_ref[2]**o+(-0.213272138462)*x_ref[2]
        ref[0,0,2,0]=(0.87154872554)*x_ref[0]**o+(-0.727915301904)*x_ref[0]+(0.683580698056)*x_ref[1]**o+(0.215052521277)*x_ref[1]+(0.267231507898)*x_ref[2]**o+(-0.0124189912449)*x_ref[2]
        ref[0,0,2,1]=(0.631400944876)*x_ref[0]**o+(0.537293628606)*x_ref[0]+(0.944754350994)*x_ref[1]**o+(0.108068908012)*x_ref[1]+(0.890117096365)*x_ref[2]**o+(0.164706867235)*x_ref[2]
        ref[0,1,0,0]=(-0.935871759207)*x_ref[0]**o+(0.350133758077)*x_ref[0]+(-0.966482181015)*x_ref[1]**o+(0.440902701528)*x_ref[1]+(-0.893329541322)*x_ref[2]**o+(0.350317827938)*x_ref[2]
        ref[0,1,0,1]=(0.465305203601)*x_ref[0]**o+(0.711131797928)*x_ref[0]+(0.104091536498)*x_ref[1]**o+(0.484811628127)*x_ref[1]+(0.676878954516)*x_ref[2]**o+(-0.99327016995)*x_ref[2]
        ref[0,1,1,0]=(-0.361189016151)*x_ref[0]**o+(0.962468573492)*x_ref[0]+(-0.158571122564)*x_ref[1]**o+(-0.293583660912)*x_ref[1]+(-0.713699173608)*x_ref[2]**o+(-0.507430163361)*x_ref[2]
        ref[0,1,1,1]=(0.172564527054)*x_ref[0]**o+(0.273438145027)*x_ref[0]+(0.137378266239)*x_ref[1]**o+(-0.675788606841)*x_ref[1]+(-0.362143416121)*x_ref[2]**o+(0.0703897749775)*x_ref[2]
        ref[0,1,2,0]=(-0.941374717271)*x_ref[0]**o+(0.289433165458)*x_ref[0]+(0.318615227028)*x_ref[1]**o+(0.524362568204)*x_ref[1]+(-0.612590305095)*x_ref[2]**o+(-0.117555538957)*x_ref[2]
        ref[0,1,2,1]=(-0.442920825551)*x_ref[0]**o+(-0.454982146579)*x_ref[0]+(0.355353737589)*x_ref[1]**o+(-0.729623917004)*x_ref[1]+(-0.567220335937)*x_ref[2]**o+(0.977652639976)*x_ref[2]
        ref[0,2,0,0]=(0.942485930373)*x_ref[0]**o+(-0.605823961033)*x_ref[0]+(-0.143465779338)*x_ref[1]**o+(0.327354308987)*x_ref[1]+(0.691900417542)*x_ref[2]**o+(-0.514165213849)*x_ref[2]
        ref[0,2,0,1]=(0.018029272977)*x_ref[0]**o+(-0.209992705469)*x_ref[0]+(-0.224614727554)*x_ref[1]**o+(-0.862697462048)*x_ref[1]+(0.274791387928)*x_ref[2]**o+(-0.907445869506)*x_ref[2]
        ref[0,2,1,0]=(0.862755248265)*x_ref[0]**o+(0.020831641939)*x_ref[0]+(0.496662599261)*x_ref[1]**o+(0.79318975953)*x_ref[1]+(0.0912166895541)*x_ref[2]**o+(0.556337610316)*x_ref[2]
        ref[0,2,1,1]=(-0.09652997071)*x_ref[0]**o+(-0.954279421394)*x_ref[0]+(-0.601053524018)*x_ref[1]**o+(0.642528012068)*x_ref[1]+(0.295748113888)*x_ref[2]**o+(-0.318166346186)*x_ref[2]
        ref[0,2,2,0]=(-0.914878854129)*x_ref[0]**o+(0.159201123642)*x_ref[0]+(0.589819056368)*x_ref[1]**o+(-0.715763226823)*x_ref[1]+(-0.444572583567)*x_ref[2]**o+(-0.487866651367)*x_ref[2]
        ref[0,2,2,1]=(-0.71271969175)*x_ref[0]**o+(0.454616308594)*x_ref[0]+(0.510270095021)*x_ref[1]**o+(-0.0385822967955)*x_ref[1]+(-0.519736540313)*x_ref[2]**o+(0.0556733091556)*x_ref[2]
        ref[0,3,0,0]=(0.355768034846)*x_ref[0]**o+(0.152240594627)*x_ref[0]+(-0.81081131843)*x_ref[1]**o+(-0.913373483156)*x_ref[1]+(-0.319859744799)*x_ref[2]**o+(-0.306218435393)*x_ref[2]
        ref[0,3,0,1]=(-0.656932063693)*x_ref[0]**o+(0.832886861783)*x_ref[0]+(-0.417112750683)*x_ref[1]**o+(-0.388190265957)*x_ref[1]+(0.246534947096)*x_ref[2]**o+(-0.896822839189)*x_ref[2]
        ref[0,3,1,0]=(0.204713418891)*x_ref[0]**o+(-0.150097488817)*x_ref[0]+(0.806578426941)*x_ref[1]**o+(-0.590052708995)*x_ref[1]+(-0.758958608419)*x_ref[2]**o+(-0.221973528232)*x_ref[2]
        ref[0,3,1,1]=(-0.854308184618)*x_ref[0]**o+(-0.292526203646)*x_ref[0]+(-0.662403504833)*x_ref[1]**o+(0.0955657982227)*x_ref[1]+(0.69471076173)*x_ref[2]**o+(0.384077309264)*x_ref[2]
        ref[0,3,2,0]=(0.982057200864)*x_ref[0]**o+(-0.64758727704)*x_ref[0]+(0.15941493222)*x_ref[1]**o+(-0.17745437115)*x_ref[1]+(-0.72953252955)*x_ref[2]**o+(0.986979270802)*x_ref[2]
        ref[0,3,2,1]=(0.764526698779)*x_ref[0]**o+(0.22809521115)*x_ref[0]+(-0.0762092087998)*x_ref[1]**o+(0.415212410471)*x_ref[1]+(-0.263638988463)*x_ref[2]**o+(-0.934669942336)*x_ref[2]
        ref[0,4,0,0]=(0.0812656932707)*x_ref[0]**o+(0.0397235062518)*x_ref[0]+(-0.0496912823983)*x_ref[1]**o+(0.189891585189)*x_ref[1]+(0.243537823293)*x_ref[2]**o+(0.461256134208)*x_ref[2]
        ref[0,4,0,1]=(0.167596358861)*x_ref[0]**o+(0.441650690481)*x_ref[0]+(0.638100632205)*x_ref[1]**o+(-0.69240615765)*x_ref[1]+(-0.501855369512)*x_ref[2]**o+(0.769465683782)*x_ref[2]
        ref[0,4,1,0]=(-0.137149398927)*x_ref[0]**o+(-0.186548325285)*x_ref[0]+(-0.504656948352)*x_ref[1]**o+(0.228896656134)*x_ref[1]+(-0.472425358061)*x_ref[2]**o+(-0.0769834754996)*x_ref[2]
        ref[0,4,1,1]=(0.311655648626)*x_ref[0]**o+(0.856513683959)*x_ref[0]+(0.47173626537)*x_ref[1]**o+(0.245497333467)*x_ref[1]+(-0.513727691305)*x_ref[2]**o+(-0.588614785682)*x_ref[2]
        ref[0,4,2,0]=(-0.152907854625)*x_ref[0]**o+(-0.86229950575)*x_ref[0]+(-0.931356414564)*x_ref[1]**o+(-0.915667991864)*x_ref[1]+(0.322316118527)*x_ref[2]**o+(-0.987559462539)*x_ref[2]
        ref[0,4,2,1]=(-0.705947769164)*x_ref[0]**o+(0.104081366627)*x_ref[0]+(-0.540454644392)*x_ref[1]**o+(0.404522880836)*x_ref[1]+(0.146464264141)*x_ref[2]**o+(0.586755030006)*x_ref[2]
        ref[1,0,0,0]=(-0.466614768106)*x_ref[0]**o+(0.482681414943)*x_ref[0]+(-0.176553738108)*x_ref[1]**o+(-0.39162272813)*x_ref[1]+(0.74312023324)*x_ref[2]**o+(0.824538905894)*x_ref[2]
        ref[1,0,0,1]=(-0.9351073631)*x_ref[0]**o+(-0.640226640449)*x_ref[0]+(0.966208762054)*x_ref[1]**o+(0.101792167944)*x_ref[1]+(-0.732983657038)*x_ref[2]**o+(-0.112313375937)*x_ref[2]
        ref[1,0,1,0]=(-0.305403470434)*x_ref[0]**o+(-0.414397930347)*x_ref[0]+(0.356093254322)*x_ref[1]**o+(-0.52607560749)*x_ref[1]+(-0.70070674473)*x_ref[2]**o+(-0.0192901616015)*x_ref[2]
        ref[1,0,1,1]=(0.450046989211)*x_ref[0]**o+(0.602770057268)*x_ref[0]+(0.0193812509113)*x_ref[1]**o+(-0.275615149191)*x_ref[1]+(0.0180468267836)*x_ref[2]**o+(-0.81388255113)*x_ref[2]
        ref[1,0,2,0]=(0.913651483249)*x_ref[0]**o+(-0.92188313743)*x_ref[0]+(0.680007150195)*x_ref[1]**o+(-0.300145584708)*x_ref[1]+(-0.398883520002)*x_ref[2]**o+(-0.752781496463)*x_ref[2]
        ref[1,0,2,1]=(-0.265749827845)*x_ref[0]**o+(0.594379728352)*x_ref[0]+(0.463409943587)*x_ref[1]**o+(-0.85840420426)*x_ref[1]+(-0.705963756674)*x_ref[2]**o+(0.251711112887)*x_ref[2]
        ref[1,1,0,0]=(-0.28895690781)*x_ref[0]**o+(0.476461656166)*x_ref[0]+(0.0692567954597)*x_ref[1]**o+(0.88763437878)*x_ref[1]+(0.0872256197554)*x_ref[2]**o+(0.19467039694)*x_ref[2]
        ref[1,1,0,1]=(0.741045418981)*x_ref[0]**o+(-0.726935151332)*x_ref[0]+(0.899418662131)*x_ref[1]**o+(0.27516497286)*x_ref[1]+(-0.715927474971)*x_ref[2]**o+(-0.709283098732)*x_ref[2]
        ref[1,1,1,0]=(0.991407130174)*x_ref[0]**o+(-0.163484265852)*x_ref[0]+(-0.983254548659)*x_ref[1]**o+(0.0568443855477)*x_ref[1]+(0.309664644588)*x_ref[2]**o+(0.904334718845)*x_ref[2]
        ref[1,1,1,1]=(0.237261672833)*x_ref[0]**o+(0.0861047746345)*x_ref[0]+(-0.494898018662)*x_ref[1]**o+(-0.433075334646)*x_ref[1]+(0.588597346505)*x_ref[2]**o+(-0.185559567553)*x_ref[2]
        ref[1,1,2,0]=(-0.969384091709)*x_ref[0]**o+(-0.0229777223491)*x_ref[0]+(0.0945623059666)*x_ref[1]**o+(0.303543647034)*x_ref[1]+(0.659000135381)*x_ref[2]**o+(0.00873791711254)*x_ref[2]
        ref[1,1,2,1]=(-0.421406731882)*x_ref[0]**o+(-0.5017394377)*x_ref[0]+(0.027747130694)*x_ref[1]**o+(0.327278873996)*x_ref[1]+(-0.754133439304)*x_ref[2]**o+(0.673525803845)*x_ref[2]
        ref[1,2,0,0]=(-0.104620319685)*x_ref[0]**o+(-0.685602393278)*x_ref[0]+(0.972173717014)*x_ref[1]**o+(-0.440573017063)*x_ref[1]+(-0.709529395743)*x_ref[2]**o+(0.634170741457)*x_ref[2]
        ref[1,2,0,1]=(-0.560386754264)*x_ref[0]**o+(0.570894623654)*x_ref[0]+(-0.455122833362)*x_ref[1]**o+(-0.00322505937935)*x_ref[1]+(0.959400336149)*x_ref[2]**o+(0.64014859954)*x_ref[2]
        ref[1,2,1,0]=(0.100585233221)*x_ref[0]**o+(0.939532360957)*x_ref[0]+(-0.900301955112)*x_ref[1]**o+(-0.776375896786)*x_ref[1]+(-0.779373294746)*x_ref[2]**o+(0.546157461116)*x_ref[2]
        ref[1,2,1,1]=(-0.680457315071)*x_ref[0]**o+(-0.0278844539208)*x_ref[0]+(-0.629673590044)*x_ref[1]**o+(0.813218376348)*x_ref[1]+(-0.449484131841)*x_ref[2]**o+(0.641682150048)*x_ref[2]
        ref[1,2,2,0]=(0.60056954386)*x_ref[0]**o+(-0.763169434657)*x_ref[0]+(-0.672890915035)*x_ref[1]**o+(-0.368711154571)*x_ref[1]+(0.476810195318)*x_ref[2]**o+(-0.935071124382)*x_ref[2]
        ref[1,2,2,1]=(0.994100148333)*x_ref[0]**o+(0.471647033381)*x_ref[0]+(0.341677296654)*x_ref[1]**o+(0.466442262068)*x_ref[1]+(-0.0634919249117)*x_ref[2]**o+(0.0637348830912)*x_ref[2]
        ref[1,3,0,0]=(-0.120744286726)*x_ref[0]**o+(0.630378984537)*x_ref[0]+(0.0837817796507)*x_ref[1]**o+(-0.850115274881)*x_ref[1]+(0.574342626561)*x_ref[2]**o+(-0.705409915626)*x_ref[2]
        ref[1,3,0,1]=(-0.51181061391)*x_ref[0]**o+(0.711479747616)*x_ref[0]+(-0.00800928076734)*x_ref[1]**o+(-0.703424580277)*x_ref[1]+(-0.925935026157)*x_ref[2]**o+(0.488175992827)*x_ref[2]
        ref[1,3,1,0]=(0.531879615751)*x_ref[0]**o+(-0.214270521934)*x_ref[0]+(-0.432344726015)*x_ref[1]**o+(-0.107660826449)*x_ref[1]+(0.763406427398)*x_ref[2]**o+(-0.511000150429)*x_ref[2]
        ref[1,3,1,1]=(-0.867467568206)*x_ref[0]**o+(-0.738425961596)*x_ref[0]+(-0.925337667706)*x_ref[1]**o+(-0.586227765107)*x_ref[1]+(0.909987211462)*x_ref[2]**o+(0.533347022582)*x_ref[2]
        ref[1,3,2,0]=(-0.47713113862)*x_ref[0]**o+(-0.130876138883)*x_ref[0]+(-0.607996835981)*x_ref[1]**o+(-0.0306261551172)*x_ref[1]+(0.956207500475)*x_ref[2]**o+(-0.287731719382)*x_ref[2]
        ref[1,3,2,1]=(0.179479486943)*x_ref[0]**o+(0.361266562479)*x_ref[0]+(0.374904472267)*x_ref[1]**o+(0.00788379106301)*x_ref[1]+(-0.34090684265)*x_ref[2]**o+(-0.190875910791)*x_ref[2]
        ref[1,4,0,0]=(0.685464203729)*x_ref[0]**o+(0.0634561930987)*x_ref[0]+(-0.662534110385)*x_ref[1]**o+(0.383849898086)*x_ref[1]+(0.948230524205)*x_ref[2]**o+(-0.347042281976)*x_ref[2]
        ref[1,4,0,1]=(-0.484499755322)*x_ref[0]**o+(-0.765655349243)*x_ref[0]+(0.805331514606)*x_ref[1]**o+(-0.0635912547496)*x_ref[1]+(-0.139457134772)*x_ref[2]**o+(0.227446734221)*x_ref[2]
        ref[1,4,1,0]=(-0.913971229647)*x_ref[0]**o+(0.042531889985)*x_ref[0]+(0.143956410289)*x_ref[1]**o+(0.597990264168)*x_ref[1]+(-0.85926903006)*x_ref[2]**o+(0.205900361792)*x_ref[2]
        ref[1,4,1,1]=(-0.955540264653)*x_ref[0]**o+(0.983439073814)*x_ref[0]+(0.246870351555)*x_ref[1]**o+(-0.610917180843)*x_ref[1]+(-0.0418134447092)*x_ref[2]**o+(-0.58407787472)*x_ref[2]
        ref[1,4,2,0]=(0.787274711453)*x_ref[0]**o+(-0.899651339467)*x_ref[0]+(-0.62195254225)*x_ref[1]**o+(-0.516447289251)*x_ref[1]+(0.267082643601)*x_ref[2]**o+(0.0408186039688)*x_ref[2]
        ref[1,4,2,1]=(-0.485773801895)*x_ref[0]**o+(0.663478067655)*x_ref[0]+(0.928973237869)*x_ref[1]**o+(0.353735145156)*x_ref[1]+(0.128409748978)*x_ref[2]**o+(-0.476479648592)*x_ref[2]
        ref[2,0,0,0]=(0.491574821811)*x_ref[0]**o+(0.455303820526)*x_ref[0]+(-0.747773112075)*x_ref[1]**o+(-0.653255987137)*x_ref[1]+(-0.738293040346)*x_ref[2]**o+(0.948070477272)*x_ref[2]
        ref[2,0,0,1]=(-0.697412782403)*x_ref[0]**o+(-0.738389544586)*x_ref[0]+(0.231728579935)*x_ref[1]**o+(0.248503931032)*x_ref[1]+(-0.561155590703)*x_ref[2]**o+(0.324682113147)*x_ref[2]
        ref[2,0,1,0]=(-0.00507638382064)*x_ref[0]**o+(-0.983105866448)*x_ref[0]+(-0.253141627484)*x_ref[1]**o+(-0.0664178091069)*x_ref[1]+(0.194460220144)*x_ref[2]**o+(-0.174257469198)*x_ref[2]
        ref[2,0,1,1]=(-0.644447560463)*x_ref[0]**o+(0.658703761133)*x_ref[0]+(-0.685392825104)*x_ref[1]**o+(-0.766295260796)*x_ref[1]+(-0.922004388493)*x_ref[2]**o+(-0.0745707720923)*x_ref[2]
        ref[2,0,2,0]=(-0.69924009606)*x_ref[0]**o+(-0.398212597328)*x_ref[0]+(-0.364991550186)*x_ref[1]**o+(0.243474288977)*x_ref[1]+(0.129545162767)*x_ref[2]**o+(0.920611954845)*x_ref[2]
        ref[2,0,2,1]=(-0.494476126875)*x_ref[0]**o+(-0.131503103148)*x_ref[0]+(-0.241670500874)*x_ref[1]**o+(0.40439125905)*x_ref[1]+(0.134166779399)*x_ref[2]**o+(-0.507385751809)*x_ref[2]
        ref[2,1,0,0]=(0.301148115913)*x_ref[0]**o+(0.189635133395)*x_ref[0]+(0.744167404774)*x_ref[1]**o+(0.995754344203)*x_ref[1]+(-0.334760662218)*x_ref[2]**o+(-0.182462923741)*x_ref[2]
        ref[2,1,0,1]=(0.523500022508)*x_ref[0]**o+(-0.257042661558)*x_ref[0]+(-0.940233100381)*x_ref[1]**o+(-0.202431731891)*x_ref[1]+(-0.653480190287)*x_ref[2]**o+(0.993156915081)*x_ref[2]
        ref[2,1,1,0]=(-0.502452234661)*x_ref[0]**o+(-0.711581805303)*x_ref[0]+(0.696450091152)*x_ref[1]**o+(0.436746953802)*x_ref[1]+(0.695969418438)*x_ref[2]**o+(-0.671400022595)*x_ref[2]
        ref[2,1,1,1]=(0.0705944125182)*x_ref[0]**o+(-0.865755834485)*x_ref[0]+(0.921555996641)*x_ref[1]**o+(-0.567855814858)*x_ref[1]+(-0.213455192388)*x_ref[2]**o+(0.820868399259)*x_ref[2]
        ref[2,1,2,0]=(0.744232126707)*x_ref[0]**o+(0.336969176485)*x_ref[0]+(0.339407729288)*x_ref[1]**o+(-0.465154968917)*x_ref[1]+(0.389524167202)*x_ref[2]**o+(0.645394954477)*x_ref[2]
        ref[2,1,2,1]=(-0.299085687516)*x_ref[0]**o+(0.220795864743)*x_ref[0]+(-0.19353913538)*x_ref[1]**o+(0.445131396297)*x_ref[1]+(-0.779605318636)*x_ref[2]**o+(-0.76707264371)*x_ref[2]
        ref[2,2,0,0]=(-0.357585997171)*x_ref[0]**o+(-0.248795296406)*x_ref[0]+(0.0876364714478)*x_ref[1]**o+(-0.682833482179)*x_ref[1]+(0.497052575986)*x_ref[2]**o+(0.893378007889)*x_ref[2]
        ref[2,2,0,1]=(0.38067999218)*x_ref[0]**o+(-0.857246232026)*x_ref[0]+(0.0785818284359)*x_ref[1]**o+(-0.473287028123)*x_ref[1]+(-0.763041318956)*x_ref[2]**o+(-0.1119710251)*x_ref[2]
        ref[2,2,1,0]=(-0.0278856926369)*x_ref[0]**o+(0.435955626553)*x_ref[0]+(-0.676274741782)*x_ref[1]**o+(-0.719721825467)*x_ref[1]+(0.195007734703)*x_ref[2]**o+(0.504771278902)*x_ref[2]
        ref[2,2,1,1]=(0.459547475772)*x_ref[0]**o+(-0.669041681613)*x_ref[0]+(-0.741957874364)*x_ref[1]**o+(0.604585337407)*x_ref[1]+(-0.886718742046)*x_ref[2]**o+(0.338113237761)*x_ref[2]
        ref[2,2,2,0]=(0.392926733678)*x_ref[0]**o+(-0.923680589861)*x_ref[0]+(-0.425503328457)*x_ref[1]**o+(-0.0902669250117)*x_ref[1]+(-0.346726072703)*x_ref[2]**o+(0.541482976709)*x_ref[2]
        ref[2,2,2,1]=(0.134867023616)*x_ref[0]**o+(0.478349607348)*x_ref[0]+(-0.334617836366)*x_ref[1]**o+(-0.74587411744)*x_ref[1]+(-0.880365937567)*x_ref[2]**o+(0.820743064452)*x_ref[2]
        ref[2,3,0,0]=(0.081184173895)*x_ref[0]**o+(0.315011929905)*x_ref[0]+(-0.137083926616)*x_ref[1]**o+(0.103926223299)*x_ref[1]+(0.363270989968)*x_ref[2]**o+(-0.454075563606)*x_ref[2]
        ref[2,3,0,1]=(0.457730701321)*x_ref[0]**o+(0.922345625691)*x_ref[0]+(-0.412933933474)*x_ref[1]**o+(-0.546359655103)*x_ref[1]+(0.719101029933)*x_ref[2]**o+(-0.790522859322)*x_ref[2]
        ref[2,3,1,0]=(-0.811816419841)*x_ref[0]**o+(0.0379298408788)*x_ref[0]+(-0.852643125131)*x_ref[1]**o+(0.76881274254)*x_ref[1]+(0.75214553496)*x_ref[2]**o+(0.800382814403)*x_ref[2]
        ref[2,3,1,1]=(0.524802143618)*x_ref[0]**o+(-0.717457078371)*x_ref[0]+(0.389026887965)*x_ref[1]**o+(-0.231912558721)*x_ref[1]+(-0.394705562333)*x_ref[2]**o+(-0.794723802655)*x_ref[2]
        ref[2,3,2,0]=(-0.232215520303)*x_ref[0]**o+(0.994932250617)*x_ref[0]+(-0.760425274222)*x_ref[1]**o+(0.0910727760744)*x_ref[1]+(0.621642024132)*x_ref[2]**o+(0.263978719304)*x_ref[2]
        ref[2,3,2,1]=(0.515269404424)*x_ref[0]**o+(-0.998266958288)*x_ref[0]+(-0.485078033534)*x_ref[1]**o+(-0.960043247553)*x_ref[1]+(-0.926562413314)*x_ref[2]**o+(0.612710762712)*x_ref[2]
        ref[2,4,0,0]=(0.419808446591)*x_ref[0]**o+(0.132792190191)*x_ref[0]+(-0.690603562736)*x_ref[1]**o+(0.45137757885)*x_ref[1]+(-0.262249041567)*x_ref[2]**o+(-0.432995254783)*x_ref[2]
        ref[2,4,0,1]=(0.62413323935)*x_ref[0]**o+(0.963559148607)*x_ref[0]+(-0.879044261015)*x_ref[1]**o+(-0.504981574957)*x_ref[1]+(-0.575204029973)*x_ref[2]**o+(0.066855791448)*x_ref[2]
        ref[2,4,1,0]=(0.321218662791)*x_ref[0]**o+(0.975103526588)*x_ref[0]+(0.783716185844)*x_ref[1]**o+(-0.816368139389)*x_ref[1]+(0.778169798128)*x_ref[2]**o+(-0.527443068085)*x_ref[2]
        ref[2,4,1,1]=(-0.8801082648)*x_ref[0]**o+(-0.0313845929272)*x_ref[0]+(0.485262237838)*x_ref[1]**o+(0.0778888535573)*x_ref[1]+(-0.900948214929)*x_ref[2]**o+(-0.98830300089)*x_ref[2]
        ref[2,4,2,0]=(-0.996395042226)*x_ref[0]**o+(0.253635852642)*x_ref[0]+(0.180965173659)*x_ref[1]**o+(-0.488294662459)*x_ref[1]+(-0.144583511741)*x_ref[2]**o+(0.285854081388)*x_ref[2]
        ref[2,4,2,1]=(0.00988506292061)*x_ref[0]**o+(-0.00192786315452)*x_ref[0]+(0.720775470647)*x_ref[1]**o+(-0.587673311506)*x_ref[1]+(0.313836238675)*x_ref[2]**o+(0.45975125457)*x_ref[2]
        ref[3,0,0,0]=(-0.822049489749)*x_ref[0]**o+(-0.989589435175)*x_ref[0]+(-0.0943830434118)*x_ref[1]**o+(-0.789229900035)*x_ref[1]+(0.311382619317)*x_ref[2]**o+(-0.316646938685)*x_ref[2]
        ref[3,0,0,1]=(0.501517916677)*x_ref[0]**o+(-0.704502365436)*x_ref[0]+(0.913184728006)*x_ref[1]**o+(-0.843712848791)*x_ref[1]+(0.901479696878)*x_ref[2]**o+(-0.713893336542)*x_ref[2]
        ref[3,0,1,0]=(0.0771592811086)*x_ref[0]**o+(0.796038032206)*x_ref[0]+(0.566256212871)*x_ref[1]**o+(0.394136659986)*x_ref[1]+(-0.794422065388)*x_ref[2]**o+(0.662174992848)*x_ref[2]
        ref[3,0,1,1]=(0.180225115167)*x_ref[0]**o+(0.366965867971)*x_ref[0]+(-0.244015756153)*x_ref[1]**o+(0.433571067335)*x_ref[1]+(0.129075150185)*x_ref[2]**o+(-0.216304414605)*x_ref[2]
        ref[3,0,2,0]=(0.100277967034)*x_ref[0]**o+(-0.590583546786)*x_ref[0]+(-0.595553697152)*x_ref[1]**o+(0.923573252239)*x_ref[1]+(0.578966663184)*x_ref[2]**o+(-0.411082913097)*x_ref[2]
        ref[3,0,2,1]=(-0.575315696811)*x_ref[0]**o+(0.608760285918)*x_ref[0]+(-0.3592074605)*x_ref[1]**o+(0.515119945308)*x_ref[1]+(-0.4000138306)*x_ref[2]**o+(0.477685324937)*x_ref[2]
        ref[3,1,0,0]=(0.0780014276919)*x_ref[0]**o+(-0.0932830671471)*x_ref[0]+(0.39477236642)*x_ref[1]**o+(0.406673135556)*x_ref[1]+(-0.235480707568)*x_ref[2]**o+(-0.872302257144)*x_ref[2]
        ref[3,1,0,1]=(0.170573072646)*x_ref[0]**o+(0.868146561326)*x_ref[0]+(0.262136343896)*x_ref[1]**o+(0.192343529912)*x_ref[1]+(0.157400965693)*x_ref[2]**o+(-0.932981619473)*x_ref[2]
        ref[3,1,1,0]=(0.891433179291)*x_ref[0]**o+(-0.281396449997)*x_ref[0]+(-0.0803173619772)*x_ref[1]**o+(-0.378460054356)*x_ref[1]+(-0.581578656014)*x_ref[2]**o+(-0.65856067509)*x_ref[2]
        ref[3,1,1,1]=(0.343599956626)*x_ref[0]**o+(-0.92063694125)*x_ref[0]+(0.440698102072)*x_ref[1]**o+(0.767124166388)*x_ref[1]+(0.050964317543)*x_ref[2]**o+(1.35605562239e-05)*x_ref[2]
        ref[3,1,2,0]=(-0.614268150987)*x_ref[0]**o+(0.363806471556)*x_ref[0]+(0.92193457905)*x_ref[1]**o+(0.157919580642)*x_ref[1]+(-0.955575321124)*x_ref[2]**o+(-0.984808437666)*x_ref[2]
        ref[3,1,2,1]=(0.030099648966)*x_ref[0]**o+(-0.375999369491)*x_ref[0]+(-0.0992643926008)*x_ref[1]**o+(0.109209161093)*x_ref[1]+(-0.974043780088)*x_ref[2]**o+(0.908049658745)*x_ref[2]
        ref[3,2,0,0]=(0.297771341106)*x_ref[0]**o+(-0.637802228169)*x_ref[0]+(0.398100441783)*x_ref[1]**o+(-0.0643425181651)*x_ref[1]+(0.662775339033)*x_ref[2]**o+(0.744835182052)*x_ref[2]
        ref[3,2,0,1]=(0.312026420583)*x_ref[0]**o+(0.865855052574)*x_ref[0]+(0.0195001783165)*x_ref[1]**o+(0.537899238473)*x_ref[1]+(0.554893357481)*x_ref[2]**o+(0.71555610145)*x_ref[2]
        ref[3,2,1,0]=(-0.199138146186)*x_ref[0]**o+(0.827546489855)*x_ref[0]+(-0.480965464736)*x_ref[1]**o+(-0.473796093187)*x_ref[1]+(0.880264675049)*x_ref[2]**o+(0.544506670844)*x_ref[2]
        ref[3,2,1,1]=(0.773798005008)*x_ref[0]**o+(0.744945241634)*x_ref[0]+(0.548170564608)*x_ref[1]**o+(-0.560503692823)*x_ref[1]+(-0.435630215427)*x_ref[2]**o+(0.080015730707)*x_ref[2]
        ref[3,2,2,0]=(0.36735714937)*x_ref[0]**o+(-0.867071557873)*x_ref[0]+(0.936135650431)*x_ref[1]**o+(-0.432533600524)*x_ref[1]+(-0.0224430204069)*x_ref[2]**o+(0.218590302621)*x_ref[2]
        ref[3,2,2,1]=(0.932808382499)*x_ref[0]**o+(-0.0970016713319)*x_ref[0]+(0.458943467579)*x_ref[1]**o+(-0.134338789898)*x_ref[1]+(-0.426892557314)*x_ref[2]**o+(0.627071826863)*x_ref[2]
        ref[3,3,0,0]=(-0.623804149075)*x_ref[0]**o+(0.910423590172)*x_ref[0]+(0.465938181097)*x_ref[1]**o+(0.495479245391)*x_ref[1]+(0.693745442185)*x_ref[2]**o+(-0.437014226757)*x_ref[2]
        ref[3,3,0,1]=(0.598807017496)*x_ref[0]**o+(-0.112361840211)*x_ref[0]+(-0.550899685213)*x_ref[1]**o+(-0.0478222019564)*x_ref[1]+(-0.0110129517693)*x_ref[2]**o+(-0.059534786374)*x_ref[2]
        ref[3,3,1,0]=(-0.561155633409)*x_ref[0]**o+(-0.463563057636)*x_ref[0]+(-0.170463996009)*x_ref[1]**o+(-0.214509204942)*x_ref[1]+(0.851200198666)*x_ref[2]**o+(-0.721036329689)*x_ref[2]
        ref[3,3,1,1]=(0.608666240405)*x_ref[0]**o+(0.814232455115)*x_ref[0]+(-0.604645629689)*x_ref[1]**o+(0.960976509673)*x_ref[1]+(0.365807996219)*x_ref[2]**o+(-0.87491909238)*x_ref[2]
        ref[3,3,2,0]=(0.120126988418)*x_ref[0]**o+(-0.745687314825)*x_ref[0]+(-0.332697826984)*x_ref[1]**o+(-0.823683406516)*x_ref[1]+(0.305444079613)*x_ref[2]**o+(0.901972115441)*x_ref[2]
        ref[3,3,2,1]=(0.0470293848904)*x_ref[0]**o+(-0.195496754814)*x_ref[0]+(0.671870114105)*x_ref[1]**o+(0.170273146371)*x_ref[1]+(-0.851927986188)*x_ref[2]**o+(0.772192863973)*x_ref[2]
        ref[3,4,0,0]=(-0.95392341727)*x_ref[0]**o+(-0.665694215521)*x_ref[0]+(-0.0247396499378)*x_ref[1]**o+(0.441352857387)*x_ref[1]+(0.767156547791)*x_ref[2]**o+(-0.14696372018)*x_ref[2]
        ref[3,4,0,1]=(0.209761166815)*x_ref[0]**o+(0.978934473425)*x_ref[0]+(0.943806891053)*x_ref[1]**o+(-0.756619917601)*x_ref[1]+(0.635699949764)*x_ref[2]**o+(-0.345248269568)*x_ref[2]
        ref[3,4,1,0]=(0.748020785785)*x_ref[0]**o+(0.311104549542)*x_ref[0]+(0.739127551248)*x_ref[1]**o+(-0.635238575859)*x_ref[1]+(0.623898875653)*x_ref[2]**o+(0.343407392068)*x_ref[2]
        ref[3,4,1,1]=(-0.422500892471)*x_ref[0]**o+(0.0688495013346)*x_ref[0]+(-0.919653906232)*x_ref[1]**o+(0.531036586396)*x_ref[1]+(-0.22487910672)*x_ref[2]**o+(-0.848225994541)*x_ref[2]
        ref[3,4,2,0]=(0.723965744673)*x_ref[0]**o+(0.558602959224)*x_ref[0]+(0.699539243308)*x_ref[1]**o+(-0.597426541715)*x_ref[1]+(-0.0128675070522)*x_ref[2]**o+(-0.874796626242)*x_ref[2]
        ref[3,4,2,1]=(-0.962970021216)*x_ref[0]**o+(0.532886481874)*x_ref[0]+(-0.523159705965)*x_ref[1]**o+(-0.82887407638)*x_ref[1]+(0.874288116843)*x_ref[2]**o+(0.520024515804)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromData_Solution_rank0(self):
      """
      tests interpolate for rank 0 Data onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from Solution onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Data(0,(),w)
      ref=Data(0,(),w_ref)
      if dim==2:
        arg=(-0.0603413935246)*x[0]**o+(-0.70184894912)*x[0]+(-0.206341392562)*x[1]**o+(0.892585857385)*x[1]
        ref=(-0.0603413935246)*x_ref[0]**o+(-0.70184894912)*x_ref[0]+(-0.206341392562)*x_ref[1]**o+(0.892585857385)*x_ref[1]
      else:
        arg=(0.120763612793)*x[0]**o+(-0.298506957699)*x[0]+(0.0620633729217)*x[1]**o+(0.807000465544)*x[1]+(-0.564443939995)*x[2]**o+(0.820783035273)*x[2]
        ref=(0.120763612793)*x_ref[0]**o+(-0.298506957699)*x_ref[0]+(0.0620633729217)*x_ref[1]**o+(0.807000465544)*x_ref[1]+(-0.564443939995)*x_ref[2]**o+(0.820783035273)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromData_Solution_rank1(self):
      """
      tests interpolate for rank 1 Data onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from Solution onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Data(0,(2,),w)
      ref=Data(0,(2,),w_ref)
      if dim==2:
        arg[0]=(-0.69193390973)*x[0]**o+(-0.768888031033)*x[0]+(-0.813493496124)*x[1]**o+(-0.767059499773)*x[1]
        arg[1]=(0.111870073987)*x[0]**o+(-0.196406293238)*x[0]+(-0.10683601134)*x[1]**o+(0.69972228684)*x[1]
        ref[0]=(-0.69193390973)*x_ref[0]**o+(-0.768888031033)*x_ref[0]+(-0.813493496124)*x_ref[1]**o+(-0.767059499773)*x_ref[1]
        ref[1]=(0.111870073987)*x_ref[0]**o+(-0.196406293238)*x_ref[0]+(-0.10683601134)*x_ref[1]**o+(0.69972228684)*x_ref[1]
      else:
        arg[0]=(-0.20606732369)*x[0]**o+(0.773916734719)*x[0]+(-0.915594370888)*x[1]**o+(-0.684202475504)*x[1]+(0.161644377631)*x[2]**o+(-0.834568364942)*x[2]
        arg[1]=(-0.475848931399)*x[0]**o+(-0.967325026091)*x[0]+(0.371032251427)*x[1]**o+(0.976933923378)*x[1]+(0.326121409381)*x[2]**o+(0.880666151318)*x[2]
        ref[0]=(-0.20606732369)*x_ref[0]**o+(0.773916734719)*x_ref[0]+(-0.915594370888)*x_ref[1]**o+(-0.684202475504)*x_ref[1]+(0.161644377631)*x_ref[2]**o+(-0.834568364942)*x_ref[2]
        ref[1]=(-0.475848931399)*x_ref[0]**o+(-0.967325026091)*x_ref[0]+(0.371032251427)*x_ref[1]**o+(0.976933923378)*x_ref[1]+(0.326121409381)*x_ref[2]**o+(0.880666151318)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromData_Solution_rank2(self):
      """
      tests interpolate for rank 2 Data onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from Solution onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5),w)
      ref=Data(0,(4, 5),w_ref)
      if dim==2:
        arg[0,0]=(-0.664616598004)*x[0]**o+(0.159791014359)*x[0]+(-0.3747223278)*x[1]**o+(0.302802720584)*x[1]
        arg[0,1]=(-0.334311194265)*x[0]**o+(0.865855840771)*x[0]+(-0.0184933229609)*x[1]**o+(0.589277746683)*x[1]
        arg[0,2]=(0.685266031246)*x[0]**o+(0.846357835994)*x[0]+(0.233422031025)*x[1]**o+(0.391225449794)*x[1]
        arg[0,3]=(-0.889412463645)*x[0]**o+(0.336985584587)*x[0]+(-0.311629085429)*x[1]**o+(-0.574756125055)*x[1]
        arg[0,4]=(-0.786434255596)*x[0]**o+(0.897529643329)*x[0]+(0.112631455142)*x[1]**o+(0.501957817255)*x[1]
        arg[1,0]=(-0.841995669352)*x[0]**o+(-0.372241861224)*x[0]+(0.669429619157)*x[1]**o+(0.913007793035)*x[1]
        arg[1,1]=(-0.965486931849)*x[0]**o+(-0.163330142621)*x[0]+(0.302996628649)*x[1]**o+(-0.214176383433)*x[1]
        arg[1,2]=(-0.717841677595)*x[0]**o+(-0.560453345617)*x[0]+(0.885221610478)*x[1]**o+(0.0925933897521)*x[1]
        arg[1,3]=(0.827380987548)*x[0]**o+(-0.185460343082)*x[0]+(-0.618848726331)*x[1]**o+(-0.25427152262)*x[1]
        arg[1,4]=(-0.232848766689)*x[0]**o+(-0.812427149527)*x[0]+(-0.6328125174)*x[1]**o+(0.377849173478)*x[1]
        arg[2,0]=(0.725568000689)*x[0]**o+(-0.0119648722611)*x[0]+(0.0551179058345)*x[1]**o+(-0.313601095535)*x[1]
        arg[2,1]=(-0.371331280463)*x[0]**o+(0.164812693414)*x[0]+(0.140689113041)*x[1]**o+(0.866863122864)*x[1]
        arg[2,2]=(0.445443472696)*x[0]**o+(0.885420986537)*x[0]+(0.434326737693)*x[1]**o+(0.013677800309)*x[1]
        arg[2,3]=(0.186221146816)*x[0]**o+(0.919506151782)*x[0]+(-0.163957717851)*x[1]**o+(-0.533309231769)*x[1]
        arg[2,4]=(0.137164729623)*x[0]**o+(0.561715777859)*x[0]+(0.33724606372)*x[1]**o+(-0.554531877504)*x[1]
        arg[3,0]=(-0.0632399630146)*x[0]**o+(0.712014652956)*x[0]+(0.107109261787)*x[1]**o+(0.228670262705)*x[1]
        arg[3,1]=(-0.973793135038)*x[0]**o+(0.923465546039)*x[0]+(0.189825544105)*x[1]**o+(-0.33784496447)*x[1]
        arg[3,2]=(-0.12686258826)*x[0]**o+(-0.582653698893)*x[0]+(0.860503162549)*x[1]**o+(-0.477305983221)*x[1]
        arg[3,3]=(-0.407988306429)*x[0]**o+(-0.174396602789)*x[0]+(0.646815890222)*x[1]**o+(-0.450974163079)*x[1]
        arg[3,4]=(0.162458379489)*x[0]**o+(-0.0569354582388)*x[0]+(0.0463216806125)*x[1]**o+(-0.381777259635)*x[1]
        ref[0,0]=(-0.664616598004)*x_ref[0]**o+(0.159791014359)*x_ref[0]+(-0.3747223278)*x_ref[1]**o+(0.302802720584)*x_ref[1]
        ref[0,1]=(-0.334311194265)*x_ref[0]**o+(0.865855840771)*x_ref[0]+(-0.0184933229609)*x_ref[1]**o+(0.589277746683)*x_ref[1]
        ref[0,2]=(0.685266031246)*x_ref[0]**o+(0.846357835994)*x_ref[0]+(0.233422031025)*x_ref[1]**o+(0.391225449794)*x_ref[1]
        ref[0,3]=(-0.889412463645)*x_ref[0]**o+(0.336985584587)*x_ref[0]+(-0.311629085429)*x_ref[1]**o+(-0.574756125055)*x_ref[1]
        ref[0,4]=(-0.786434255596)*x_ref[0]**o+(0.897529643329)*x_ref[0]+(0.112631455142)*x_ref[1]**o+(0.501957817255)*x_ref[1]
        ref[1,0]=(-0.841995669352)*x_ref[0]**o+(-0.372241861224)*x_ref[0]+(0.669429619157)*x_ref[1]**o+(0.913007793035)*x_ref[1]
        ref[1,1]=(-0.965486931849)*x_ref[0]**o+(-0.163330142621)*x_ref[0]+(0.302996628649)*x_ref[1]**o+(-0.214176383433)*x_ref[1]
        ref[1,2]=(-0.717841677595)*x_ref[0]**o+(-0.560453345617)*x_ref[0]+(0.885221610478)*x_ref[1]**o+(0.0925933897521)*x_ref[1]
        ref[1,3]=(0.827380987548)*x_ref[0]**o+(-0.185460343082)*x_ref[0]+(-0.618848726331)*x_ref[1]**o+(-0.25427152262)*x_ref[1]
        ref[1,4]=(-0.232848766689)*x_ref[0]**o+(-0.812427149527)*x_ref[0]+(-0.6328125174)*x_ref[1]**o+(0.377849173478)*x_ref[1]
        ref[2,0]=(0.725568000689)*x_ref[0]**o+(-0.0119648722611)*x_ref[0]+(0.0551179058345)*x_ref[1]**o+(-0.313601095535)*x_ref[1]
        ref[2,1]=(-0.371331280463)*x_ref[0]**o+(0.164812693414)*x_ref[0]+(0.140689113041)*x_ref[1]**o+(0.866863122864)*x_ref[1]
        ref[2,2]=(0.445443472696)*x_ref[0]**o+(0.885420986537)*x_ref[0]+(0.434326737693)*x_ref[1]**o+(0.013677800309)*x_ref[1]
        ref[2,3]=(0.186221146816)*x_ref[0]**o+(0.919506151782)*x_ref[0]+(-0.163957717851)*x_ref[1]**o+(-0.533309231769)*x_ref[1]
        ref[2,4]=(0.137164729623)*x_ref[0]**o+(0.561715777859)*x_ref[0]+(0.33724606372)*x_ref[1]**o+(-0.554531877504)*x_ref[1]
        ref[3,0]=(-0.0632399630146)*x_ref[0]**o+(0.712014652956)*x_ref[0]+(0.107109261787)*x_ref[1]**o+(0.228670262705)*x_ref[1]
        ref[3,1]=(-0.973793135038)*x_ref[0]**o+(0.923465546039)*x_ref[0]+(0.189825544105)*x_ref[1]**o+(-0.33784496447)*x_ref[1]
        ref[3,2]=(-0.12686258826)*x_ref[0]**o+(-0.582653698893)*x_ref[0]+(0.860503162549)*x_ref[1]**o+(-0.477305983221)*x_ref[1]
        ref[3,3]=(-0.407988306429)*x_ref[0]**o+(-0.174396602789)*x_ref[0]+(0.646815890222)*x_ref[1]**o+(-0.450974163079)*x_ref[1]
        ref[3,4]=(0.162458379489)*x_ref[0]**o+(-0.0569354582388)*x_ref[0]+(0.0463216806125)*x_ref[1]**o+(-0.381777259635)*x_ref[1]
      else:
        arg[0,0]=(0.865079887449)*x[0]**o+(-0.833757275547)*x[0]+(-0.967658088049)*x[1]**o+(-0.168314490396)*x[1]+(0.151618201772)*x[2]**o+(-0.251062354355)*x[2]
        arg[0,1]=(0.110207997589)*x[0]**o+(-0.510994295641)*x[0]+(0.855215902178)*x[1]**o+(0.941502315255)*x[1]+(-0.794874945971)*x[2]**o+(-0.309190709938)*x[2]
        arg[0,2]=(0.71701217173)*x[0]**o+(-0.221333357165)*x[0]+(0.689963224026)*x[1]**o+(0.221517236085)*x[1]+(-0.52608140393)*x[2]**o+(0.371488496481)*x[2]
        arg[0,3]=(0.618194041415)*x[0]**o+(0.454387091678)*x[0]+(-0.707329629946)*x[1]**o+(0.41972715681)*x[1]+(0.0180789814415)*x[2]**o+(-0.176487980225)*x[2]
        arg[0,4]=(0.329275243989)*x[0]**o+(-0.679005993002)*x[0]+(-0.261017436418)*x[1]**o+(0.151780687062)*x[1]+(-0.480936330475)*x[2]**o+(-0.697077234587)*x[2]
        arg[1,0]=(0.720913763906)*x[0]**o+(0.283486705277)*x[0]+(0.498289077088)*x[1]**o+(0.657543220863)*x[1]+(-0.666293219679)*x[2]**o+(0.901456618353)*x[2]
        arg[1,1]=(-0.984526350563)*x[0]**o+(0.677118292603)*x[0]+(0.614707818107)*x[1]**o+(0.272173514554)*x[1]+(0.41492971828)*x[2]**o+(0.274511869871)*x[2]
        arg[1,2]=(0.322224985373)*x[0]**o+(0.567272204545)*x[0]+(-0.437313912332)*x[1]**o+(0.847486025896)*x[1]+(0.769434908781)*x[2]**o+(0.325917607497)*x[2]
        arg[1,3]=(0.709357837264)*x[0]**o+(0.927425103914)*x[0]+(-0.622876388501)*x[1]**o+(-0.296141603555)*x[1]+(-0.038448878923)*x[2]**o+(-0.320692580169)*x[2]
        arg[1,4]=(-0.27890333885)*x[0]**o+(0.803346569394)*x[0]+(0.00665934218255)*x[1]**o+(-0.89057979428)*x[1]+(-0.727135553706)*x[2]**o+(0.892946022497)*x[2]
        arg[2,0]=(0.579951394388)*x[0]**o+(0.0406506913385)*x[0]+(0.363944368798)*x[1]**o+(0.582732756951)*x[1]+(0.542548838027)*x[2]**o+(0.365742797343)*x[2]
        arg[2,1]=(0.253260777874)*x[0]**o+(-0.69077350875)*x[0]+(0.458080700492)*x[1]**o+(-0.79224006672)*x[1]+(-0.504865679524)*x[2]**o+(0.681425632809)*x[2]
        arg[2,2]=(0.600427718261)*x[0]**o+(0.91761242151)*x[0]+(0.602316047463)*x[1]**o+(0.890383614185)*x[1]+(-0.386691039525)*x[2]**o+(0.688659949109)*x[2]
        arg[2,3]=(-0.181338810914)*x[0]**o+(0.933744241999)*x[0]+(0.0212774093137)*x[1]**o+(0.284972854884)*x[1]+(-0.150219922419)*x[2]**o+(0.0878011402107)*x[2]
        arg[2,4]=(-0.848985879538)*x[0]**o+(0.298962025531)*x[0]+(0.593300576179)*x[1]**o+(0.301509253541)*x[1]+(0.971171084219)*x[2]**o+(-0.973386482857)*x[2]
        arg[3,0]=(-0.735608389534)*x[0]**o+(0.44902436426)*x[0]+(0.0536456095512)*x[1]**o+(0.208969935548)*x[1]+(-0.787609300211)*x[2]**o+(0.726024812477)*x[2]
        arg[3,1]=(0.200882171943)*x[0]**o+(-0.644461780745)*x[0]+(-0.0995704510264)*x[1]**o+(-0.857730241944)*x[1]+(-0.372302969302)*x[2]**o+(-0.171367954739)*x[2]
        arg[3,2]=(-0.673606332627)*x[0]**o+(-0.828183678442)*x[0]+(-0.236856099365)*x[1]**o+(-0.266339053288)*x[1]+(-0.972034740204)*x[2]**o+(-0.33151750348)*x[2]
        arg[3,3]=(-0.312470720017)*x[0]**o+(-0.657433718629)*x[0]+(-0.984940036612)*x[1]**o+(0.197707053145)*x[1]+(0.512009064643)*x[2]**o+(0.855601033248)*x[2]
        arg[3,4]=(-0.674854508228)*x[0]**o+(-0.394035682831)*x[0]+(-0.749446232511)*x[1]**o+(0.866992763518)*x[1]+(-0.945837361158)*x[2]**o+(0.593490782481)*x[2]
        ref[0,0]=(0.865079887449)*x_ref[0]**o+(-0.833757275547)*x_ref[0]+(-0.967658088049)*x_ref[1]**o+(-0.168314490396)*x_ref[1]+(0.151618201772)*x_ref[2]**o+(-0.251062354355)*x_ref[2]
        ref[0,1]=(0.110207997589)*x_ref[0]**o+(-0.510994295641)*x_ref[0]+(0.855215902178)*x_ref[1]**o+(0.941502315255)*x_ref[1]+(-0.794874945971)*x_ref[2]**o+(-0.309190709938)*x_ref[2]
        ref[0,2]=(0.71701217173)*x_ref[0]**o+(-0.221333357165)*x_ref[0]+(0.689963224026)*x_ref[1]**o+(0.221517236085)*x_ref[1]+(-0.52608140393)*x_ref[2]**o+(0.371488496481)*x_ref[2]
        ref[0,3]=(0.618194041415)*x_ref[0]**o+(0.454387091678)*x_ref[0]+(-0.707329629946)*x_ref[1]**o+(0.41972715681)*x_ref[1]+(0.0180789814415)*x_ref[2]**o+(-0.176487980225)*x_ref[2]
        ref[0,4]=(0.329275243989)*x_ref[0]**o+(-0.679005993002)*x_ref[0]+(-0.261017436418)*x_ref[1]**o+(0.151780687062)*x_ref[1]+(-0.480936330475)*x_ref[2]**o+(-0.697077234587)*x_ref[2]
        ref[1,0]=(0.720913763906)*x_ref[0]**o+(0.283486705277)*x_ref[0]+(0.498289077088)*x_ref[1]**o+(0.657543220863)*x_ref[1]+(-0.666293219679)*x_ref[2]**o+(0.901456618353)*x_ref[2]
        ref[1,1]=(-0.984526350563)*x_ref[0]**o+(0.677118292603)*x_ref[0]+(0.614707818107)*x_ref[1]**o+(0.272173514554)*x_ref[1]+(0.41492971828)*x_ref[2]**o+(0.274511869871)*x_ref[2]
        ref[1,2]=(0.322224985373)*x_ref[0]**o+(0.567272204545)*x_ref[0]+(-0.437313912332)*x_ref[1]**o+(0.847486025896)*x_ref[1]+(0.769434908781)*x_ref[2]**o+(0.325917607497)*x_ref[2]
        ref[1,3]=(0.709357837264)*x_ref[0]**o+(0.927425103914)*x_ref[0]+(-0.622876388501)*x_ref[1]**o+(-0.296141603555)*x_ref[1]+(-0.038448878923)*x_ref[2]**o+(-0.320692580169)*x_ref[2]
        ref[1,4]=(-0.27890333885)*x_ref[0]**o+(0.803346569394)*x_ref[0]+(0.00665934218255)*x_ref[1]**o+(-0.89057979428)*x_ref[1]+(-0.727135553706)*x_ref[2]**o+(0.892946022497)*x_ref[2]
        ref[2,0]=(0.579951394388)*x_ref[0]**o+(0.0406506913385)*x_ref[0]+(0.363944368798)*x_ref[1]**o+(0.582732756951)*x_ref[1]+(0.542548838027)*x_ref[2]**o+(0.365742797343)*x_ref[2]
        ref[2,1]=(0.253260777874)*x_ref[0]**o+(-0.69077350875)*x_ref[0]+(0.458080700492)*x_ref[1]**o+(-0.79224006672)*x_ref[1]+(-0.504865679524)*x_ref[2]**o+(0.681425632809)*x_ref[2]
        ref[2,2]=(0.600427718261)*x_ref[0]**o+(0.91761242151)*x_ref[0]+(0.602316047463)*x_ref[1]**o+(0.890383614185)*x_ref[1]+(-0.386691039525)*x_ref[2]**o+(0.688659949109)*x_ref[2]
        ref[2,3]=(-0.181338810914)*x_ref[0]**o+(0.933744241999)*x_ref[0]+(0.0212774093137)*x_ref[1]**o+(0.284972854884)*x_ref[1]+(-0.150219922419)*x_ref[2]**o+(0.0878011402107)*x_ref[2]
        ref[2,4]=(-0.848985879538)*x_ref[0]**o+(0.298962025531)*x_ref[0]+(0.593300576179)*x_ref[1]**o+(0.301509253541)*x_ref[1]+(0.971171084219)*x_ref[2]**o+(-0.973386482857)*x_ref[2]
        ref[3,0]=(-0.735608389534)*x_ref[0]**o+(0.44902436426)*x_ref[0]+(0.0536456095512)*x_ref[1]**o+(0.208969935548)*x_ref[1]+(-0.787609300211)*x_ref[2]**o+(0.726024812477)*x_ref[2]
        ref[3,1]=(0.200882171943)*x_ref[0]**o+(-0.644461780745)*x_ref[0]+(-0.0995704510264)*x_ref[1]**o+(-0.857730241944)*x_ref[1]+(-0.372302969302)*x_ref[2]**o+(-0.171367954739)*x_ref[2]
        ref[3,2]=(-0.673606332627)*x_ref[0]**o+(-0.828183678442)*x_ref[0]+(-0.236856099365)*x_ref[1]**o+(-0.266339053288)*x_ref[1]+(-0.972034740204)*x_ref[2]**o+(-0.33151750348)*x_ref[2]
        ref[3,3]=(-0.312470720017)*x_ref[0]**o+(-0.657433718629)*x_ref[0]+(-0.984940036612)*x_ref[1]**o+(0.197707053145)*x_ref[1]+(0.512009064643)*x_ref[2]**o+(0.855601033248)*x_ref[2]
        ref[3,4]=(-0.674854508228)*x_ref[0]**o+(-0.394035682831)*x_ref[0]+(-0.749446232511)*x_ref[1]**o+(0.866992763518)*x_ref[1]+(-0.945837361158)*x_ref[2]**o+(0.593490782481)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromData_Solution_rank3(self):
      """
      tests interpolate for rank 3 Data onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from Solution onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2),w_ref)
      if dim==2:
        arg[0,0,0]=(-0.511844892619)*x[0]**o+(0.72447480361)*x[0]+(-0.415725587152)*x[1]**o+(-0.495233818217)*x[1]
        arg[0,0,1]=(0.513861713949)*x[0]**o+(-0.918192893835)*x[0]+(0.563871531176)*x[1]**o+(0.885768393284)*x[1]
        arg[0,1,0]=(-0.412320762031)*x[0]**o+(0.576776422795)*x[0]+(0.0103535112247)*x[1]**o+(-0.738144219037)*x[1]
        arg[0,1,1]=(0.69941105604)*x[0]**o+(0.923470022558)*x[0]+(0.568449180547)*x[1]**o+(0.361429609343)*x[1]
        arg[1,0,0]=(-0.574139432427)*x[0]**o+(0.968980609212)*x[0]+(-0.198809151818)*x[1]**o+(0.751623099192)*x[1]
        arg[1,0,1]=(-0.743433955873)*x[0]**o+(-0.608013190658)*x[0]+(-0.0625473402572)*x[1]**o+(0.168418241981)*x[1]
        arg[1,1,0]=(-0.632180449517)*x[0]**o+(-0.892666803222)*x[0]+(-0.558127678438)*x[1]**o+(0.212462582041)*x[1]
        arg[1,1,1]=(-0.666381769622)*x[0]**o+(-0.0299308846938)*x[0]+(-0.450221481906)*x[1]**o+(0.701099654576)*x[1]
        arg[2,0,0]=(-0.927832262778)*x[0]**o+(0.901665098705)*x[0]+(-0.0462955000541)*x[1]**o+(-0.92139600993)*x[1]
        arg[2,0,1]=(-0.813732112672)*x[0]**o+(0.569194058283)*x[0]+(-0.527298120198)*x[1]**o+(-0.531607593369)*x[1]
        arg[2,1,0]=(-0.760238058111)*x[0]**o+(0.628718839289)*x[0]+(-0.420066582474)*x[1]**o+(-0.205537665056)*x[1]
        arg[2,1,1]=(0.421784629744)*x[0]**o+(-0.781451031751)*x[0]+(-0.0401156726902)*x[1]**o+(0.437044655245)*x[1]
        arg[3,0,0]=(0.324136561188)*x[0]**o+(-0.920235828584)*x[0]+(0.677675509043)*x[1]**o+(0.322337884254)*x[1]
        arg[3,0,1]=(-0.0711813236306)*x[0]**o+(0.0770312838088)*x[0]+(0.259613502243)*x[1]**o+(0.508154611576)*x[1]
        arg[3,1,0]=(0.944048224599)*x[0]**o+(0.039954080407)*x[0]+(-0.503309434151)*x[1]**o+(0.747109874504)*x[1]
        arg[3,1,1]=(0.692616323513)*x[0]**o+(0.0114568310884)*x[0]+(0.0708318026263)*x[1]**o+(0.822167043168)*x[1]
        arg[4,0,0]=(0.392645136008)*x[0]**o+(0.160054219263)*x[0]+(-0.32320145108)*x[1]**o+(0.598963113077)*x[1]
        arg[4,0,1]=(0.124778736846)*x[0]**o+(0.949589553291)*x[0]+(0.0985585514202)*x[1]**o+(-0.536684039884)*x[1]
        arg[4,1,0]=(-0.983690591076)*x[0]**o+(-0.115321512368)*x[0]+(0.0875590878092)*x[1]**o+(0.992162964289)*x[1]
        arg[4,1,1]=(-0.89264628952)*x[0]**o+(0.414589880453)*x[0]+(-0.40959764673)*x[1]**o+(-0.261016337239)*x[1]
        arg[5,0,0]=(-0.613553175002)*x[0]**o+(0.320685999723)*x[0]+(-0.523099455238)*x[1]**o+(0.855780524769)*x[1]
        arg[5,0,1]=(-0.45911360029)*x[0]**o+(-0.285942390871)*x[0]+(0.7279702976)*x[1]**o+(0.981483922949)*x[1]
        arg[5,1,0]=(0.577254626037)*x[0]**o+(0.146911866492)*x[0]+(-0.414545328014)*x[1]**o+(-0.979179708006)*x[1]
        arg[5,1,1]=(-0.838392720256)*x[0]**o+(-0.51717223091)*x[0]+(0.635616756614)*x[1]**o+(0.112711970543)*x[1]
        ref[0,0,0]=(-0.511844892619)*x_ref[0]**o+(0.72447480361)*x_ref[0]+(-0.415725587152)*x_ref[1]**o+(-0.495233818217)*x_ref[1]
        ref[0,0,1]=(0.513861713949)*x_ref[0]**o+(-0.918192893835)*x_ref[0]+(0.563871531176)*x_ref[1]**o+(0.885768393284)*x_ref[1]
        ref[0,1,0]=(-0.412320762031)*x_ref[0]**o+(0.576776422795)*x_ref[0]+(0.0103535112247)*x_ref[1]**o+(-0.738144219037)*x_ref[1]
        ref[0,1,1]=(0.69941105604)*x_ref[0]**o+(0.923470022558)*x_ref[0]+(0.568449180547)*x_ref[1]**o+(0.361429609343)*x_ref[1]
        ref[1,0,0]=(-0.574139432427)*x_ref[0]**o+(0.968980609212)*x_ref[0]+(-0.198809151818)*x_ref[1]**o+(0.751623099192)*x_ref[1]
        ref[1,0,1]=(-0.743433955873)*x_ref[0]**o+(-0.608013190658)*x_ref[0]+(-0.0625473402572)*x_ref[1]**o+(0.168418241981)*x_ref[1]
        ref[1,1,0]=(-0.632180449517)*x_ref[0]**o+(-0.892666803222)*x_ref[0]+(-0.558127678438)*x_ref[1]**o+(0.212462582041)*x_ref[1]
        ref[1,1,1]=(-0.666381769622)*x_ref[0]**o+(-0.0299308846938)*x_ref[0]+(-0.450221481906)*x_ref[1]**o+(0.701099654576)*x_ref[1]
        ref[2,0,0]=(-0.927832262778)*x_ref[0]**o+(0.901665098705)*x_ref[0]+(-0.0462955000541)*x_ref[1]**o+(-0.92139600993)*x_ref[1]
        ref[2,0,1]=(-0.813732112672)*x_ref[0]**o+(0.569194058283)*x_ref[0]+(-0.527298120198)*x_ref[1]**o+(-0.531607593369)*x_ref[1]
        ref[2,1,0]=(-0.760238058111)*x_ref[0]**o+(0.628718839289)*x_ref[0]+(-0.420066582474)*x_ref[1]**o+(-0.205537665056)*x_ref[1]
        ref[2,1,1]=(0.421784629744)*x_ref[0]**o+(-0.781451031751)*x_ref[0]+(-0.0401156726902)*x_ref[1]**o+(0.437044655245)*x_ref[1]
        ref[3,0,0]=(0.324136561188)*x_ref[0]**o+(-0.920235828584)*x_ref[0]+(0.677675509043)*x_ref[1]**o+(0.322337884254)*x_ref[1]
        ref[3,0,1]=(-0.0711813236306)*x_ref[0]**o+(0.0770312838088)*x_ref[0]+(0.259613502243)*x_ref[1]**o+(0.508154611576)*x_ref[1]
        ref[3,1,0]=(0.944048224599)*x_ref[0]**o+(0.039954080407)*x_ref[0]+(-0.503309434151)*x_ref[1]**o+(0.747109874504)*x_ref[1]
        ref[3,1,1]=(0.692616323513)*x_ref[0]**o+(0.0114568310884)*x_ref[0]+(0.0708318026263)*x_ref[1]**o+(0.822167043168)*x_ref[1]
        ref[4,0,0]=(0.392645136008)*x_ref[0]**o+(0.160054219263)*x_ref[0]+(-0.32320145108)*x_ref[1]**o+(0.598963113077)*x_ref[1]
        ref[4,0,1]=(0.124778736846)*x_ref[0]**o+(0.949589553291)*x_ref[0]+(0.0985585514202)*x_ref[1]**o+(-0.536684039884)*x_ref[1]
        ref[4,1,0]=(-0.983690591076)*x_ref[0]**o+(-0.115321512368)*x_ref[0]+(0.0875590878092)*x_ref[1]**o+(0.992162964289)*x_ref[1]
        ref[4,1,1]=(-0.89264628952)*x_ref[0]**o+(0.414589880453)*x_ref[0]+(-0.40959764673)*x_ref[1]**o+(-0.261016337239)*x_ref[1]
        ref[5,0,0]=(-0.613553175002)*x_ref[0]**o+(0.320685999723)*x_ref[0]+(-0.523099455238)*x_ref[1]**o+(0.855780524769)*x_ref[1]
        ref[5,0,1]=(-0.45911360029)*x_ref[0]**o+(-0.285942390871)*x_ref[0]+(0.7279702976)*x_ref[1]**o+(0.981483922949)*x_ref[1]
        ref[5,1,0]=(0.577254626037)*x_ref[0]**o+(0.146911866492)*x_ref[0]+(-0.414545328014)*x_ref[1]**o+(-0.979179708006)*x_ref[1]
        ref[5,1,1]=(-0.838392720256)*x_ref[0]**o+(-0.51717223091)*x_ref[0]+(0.635616756614)*x_ref[1]**o+(0.112711970543)*x_ref[1]
      else:
        arg[0,0,0]=(0.807360140818)*x[0]**o+(-0.399141641865)*x[0]+(0.752910658124)*x[1]**o+(0.0998734354996)*x[1]+(-0.146785318133)*x[2]**o+(-0.393796605729)*x[2]
        arg[0,0,1]=(-0.502209573844)*x[0]**o+(0.501684846477)*x[0]+(0.634537290794)*x[1]**o+(0.481836589517)*x[1]+(-0.0221536339712)*x[2]**o+(-0.868693970024)*x[2]
        arg[0,1,0]=(-0.268854282014)*x[0]**o+(0.0516535086968)*x[0]+(-0.505997013698)*x[1]**o+(0.414205349613)*x[1]+(0.0119995662599)*x[2]**o+(0.584354222675)*x[2]
        arg[0,1,1]=(0.81923283939)*x[0]**o+(-0.174637610292)*x[0]+(0.283785251937)*x[1]**o+(0.716260504685)*x[1]+(-0.957279283093)*x[2]**o+(0.632750238692)*x[2]
        arg[1,0,0]=(0.601003888536)*x[0]**o+(-0.575011763358)*x[0]+(0.095159775614)*x[1]**o+(0.141200996742)*x[1]+(0.0173869440863)*x[2]**o+(0.478733688259)*x[2]
        arg[1,0,1]=(-0.783826426764)*x[0]**o+(-0.40604430175)*x[0]+(-0.4574879786)*x[1]**o+(0.020222886424)*x[1]+(0.225419750344)*x[2]**o+(-0.530026190372)*x[2]
        arg[1,1,0]=(0.542744341911)*x[0]**o+(-0.501169367089)*x[0]+(0.447661946532)*x[1]**o+(0.384528043888)*x[1]+(-0.599271261928)*x[2]**o+(-0.535447844217)*x[2]
        arg[1,1,1]=(-0.457163465824)*x[0]**o+(0.0475382663524)*x[0]+(0.369626614251)*x[1]**o+(0.1125159395)*x[1]+(-0.252442923096)*x[2]**o+(0.0717205095822)*x[2]
        arg[2,0,0]=(0.341536174626)*x[0]**o+(0.4408059057)*x[0]+(0.196877383546)*x[1]**o+(0.903072917866)*x[1]+(0.423403716275)*x[2]**o+(-0.521563534689)*x[2]
        arg[2,0,1]=(0.553187361262)*x[0]**o+(-0.539057536895)*x[0]+(-0.814990210449)*x[1]**o+(-0.418614789609)*x[1]+(0.857298489434)*x[2]**o+(0.629708672)*x[2]
        arg[2,1,0]=(0.494803348038)*x[0]**o+(0.143644562028)*x[0]+(-0.0425813919235)*x[1]**o+(-0.818562153097)*x[1]+(-0.730690839032)*x[2]**o+(-0.569426137866)*x[2]
        arg[2,1,1]=(-0.573254727715)*x[0]**o+(-0.536459946225)*x[0]+(-0.244717967781)*x[1]**o+(-0.67463285343)*x[1]+(-0.383959754049)*x[2]**o+(-0.87595126443)*x[2]
        arg[3,0,0]=(0.758065154533)*x[0]**o+(0.634049823109)*x[0]+(0.851352543082)*x[1]**o+(0.302790462394)*x[1]+(-0.920790730269)*x[2]**o+(0.460419670912)*x[2]
        arg[3,0,1]=(0.415682634939)*x[0]**o+(-0.664821834506)*x[0]+(-0.314068887264)*x[1]**o+(-0.322413102029)*x[1]+(-0.498768865759)*x[2]**o+(-0.165338012626)*x[2]
        arg[3,1,0]=(-0.398599595126)*x[0]**o+(0.744397086526)*x[0]+(-0.02321839482)*x[1]**o+(0.989743131098)*x[1]+(0.0713981107777)*x[2]**o+(-0.7862942966)*x[2]
        arg[3,1,1]=(0.90835870295)*x[0]**o+(0.285965355995)*x[0]+(-0.302780058578)*x[1]**o+(0.396923771953)*x[1]+(0.717312324488)*x[2]**o+(0.967488138744)*x[2]
        arg[4,0,0]=(-0.286827561624)*x[0]**o+(0.67973673275)*x[0]+(0.325169520671)*x[1]**o+(0.482002981846)*x[1]+(-0.826037251282)*x[2]**o+(-0.0567054906659)*x[2]
        arg[4,0,1]=(-0.667297458554)*x[0]**o+(-0.207369757609)*x[0]+(-0.982746021703)*x[1]**o+(0.963677089363)*x[1]+(0.917328237047)*x[2]**o+(-0.684386577728)*x[2]
        arg[4,1,0]=(-0.835525763762)*x[0]**o+(-0.113324184435)*x[0]+(0.465855661969)*x[1]**o+(0.656047307997)*x[1]+(0.331675778117)*x[2]**o+(-0.686067177565)*x[2]
        arg[4,1,1]=(-0.468668496499)*x[0]**o+(0.295197251458)*x[0]+(0.065102699741)*x[1]**o+(-0.378381176309)*x[1]+(0.933627973066)*x[2]**o+(-0.91462136931)*x[2]
        arg[5,0,0]=(-0.161916994488)*x[0]**o+(0.3070005414)*x[0]+(0.195946524325)*x[1]**o+(-0.453290146527)*x[1]+(0.751532065651)*x[2]**o+(0.82676125654)*x[2]
        arg[5,0,1]=(-0.925362363492)*x[0]**o+(-0.0295351854097)*x[0]+(0.533980764998)*x[1]**o+(0.128080489262)*x[1]+(0.502858052015)*x[2]**o+(0.813196540163)*x[2]
        arg[5,1,0]=(-0.740146099318)*x[0]**o+(0.27623528724)*x[0]+(-0.361620073401)*x[1]**o+(-0.310534085108)*x[1]+(-0.0363428185757)*x[2]**o+(-0.145397849357)*x[2]
        arg[5,1,1]=(0.362089087335)*x[0]**o+(0.539359659929)*x[0]+(0.00904990097799)*x[1]**o+(-0.150401175722)*x[1]+(-0.20465882105)*x[2]**o+(-0.519331232192)*x[2]
        ref[0,0,0]=(0.807360140818)*x_ref[0]**o+(-0.399141641865)*x_ref[0]+(0.752910658124)*x_ref[1]**o+(0.0998734354996)*x_ref[1]+(-0.146785318133)*x_ref[2]**o+(-0.393796605729)*x_ref[2]
        ref[0,0,1]=(-0.502209573844)*x_ref[0]**o+(0.501684846477)*x_ref[0]+(0.634537290794)*x_ref[1]**o+(0.481836589517)*x_ref[1]+(-0.0221536339712)*x_ref[2]**o+(-0.868693970024)*x_ref[2]
        ref[0,1,0]=(-0.268854282014)*x_ref[0]**o+(0.0516535086968)*x_ref[0]+(-0.505997013698)*x_ref[1]**o+(0.414205349613)*x_ref[1]+(0.0119995662599)*x_ref[2]**o+(0.584354222675)*x_ref[2]
        ref[0,1,1]=(0.81923283939)*x_ref[0]**o+(-0.174637610292)*x_ref[0]+(0.283785251937)*x_ref[1]**o+(0.716260504685)*x_ref[1]+(-0.957279283093)*x_ref[2]**o+(0.632750238692)*x_ref[2]
        ref[1,0,0]=(0.601003888536)*x_ref[0]**o+(-0.575011763358)*x_ref[0]+(0.095159775614)*x_ref[1]**o+(0.141200996742)*x_ref[1]+(0.0173869440863)*x_ref[2]**o+(0.478733688259)*x_ref[2]
        ref[1,0,1]=(-0.783826426764)*x_ref[0]**o+(-0.40604430175)*x_ref[0]+(-0.4574879786)*x_ref[1]**o+(0.020222886424)*x_ref[1]+(0.225419750344)*x_ref[2]**o+(-0.530026190372)*x_ref[2]
        ref[1,1,0]=(0.542744341911)*x_ref[0]**o+(-0.501169367089)*x_ref[0]+(0.447661946532)*x_ref[1]**o+(0.384528043888)*x_ref[1]+(-0.599271261928)*x_ref[2]**o+(-0.535447844217)*x_ref[2]
        ref[1,1,1]=(-0.457163465824)*x_ref[0]**o+(0.0475382663524)*x_ref[0]+(0.369626614251)*x_ref[1]**o+(0.1125159395)*x_ref[1]+(-0.252442923096)*x_ref[2]**o+(0.0717205095822)*x_ref[2]
        ref[2,0,0]=(0.341536174626)*x_ref[0]**o+(0.4408059057)*x_ref[0]+(0.196877383546)*x_ref[1]**o+(0.903072917866)*x_ref[1]+(0.423403716275)*x_ref[2]**o+(-0.521563534689)*x_ref[2]
        ref[2,0,1]=(0.553187361262)*x_ref[0]**o+(-0.539057536895)*x_ref[0]+(-0.814990210449)*x_ref[1]**o+(-0.418614789609)*x_ref[1]+(0.857298489434)*x_ref[2]**o+(0.629708672)*x_ref[2]
        ref[2,1,0]=(0.494803348038)*x_ref[0]**o+(0.143644562028)*x_ref[0]+(-0.0425813919235)*x_ref[1]**o+(-0.818562153097)*x_ref[1]+(-0.730690839032)*x_ref[2]**o+(-0.569426137866)*x_ref[2]
        ref[2,1,1]=(-0.573254727715)*x_ref[0]**o+(-0.536459946225)*x_ref[0]+(-0.244717967781)*x_ref[1]**o+(-0.67463285343)*x_ref[1]+(-0.383959754049)*x_ref[2]**o+(-0.87595126443)*x_ref[2]
        ref[3,0,0]=(0.758065154533)*x_ref[0]**o+(0.634049823109)*x_ref[0]+(0.851352543082)*x_ref[1]**o+(0.302790462394)*x_ref[1]+(-0.920790730269)*x_ref[2]**o+(0.460419670912)*x_ref[2]
        ref[3,0,1]=(0.415682634939)*x_ref[0]**o+(-0.664821834506)*x_ref[0]+(-0.314068887264)*x_ref[1]**o+(-0.322413102029)*x_ref[1]+(-0.498768865759)*x_ref[2]**o+(-0.165338012626)*x_ref[2]
        ref[3,1,0]=(-0.398599595126)*x_ref[0]**o+(0.744397086526)*x_ref[0]+(-0.02321839482)*x_ref[1]**o+(0.989743131098)*x_ref[1]+(0.0713981107777)*x_ref[2]**o+(-0.7862942966)*x_ref[2]
        ref[3,1,1]=(0.90835870295)*x_ref[0]**o+(0.285965355995)*x_ref[0]+(-0.302780058578)*x_ref[1]**o+(0.396923771953)*x_ref[1]+(0.717312324488)*x_ref[2]**o+(0.967488138744)*x_ref[2]
        ref[4,0,0]=(-0.286827561624)*x_ref[0]**o+(0.67973673275)*x_ref[0]+(0.325169520671)*x_ref[1]**o+(0.482002981846)*x_ref[1]+(-0.826037251282)*x_ref[2]**o+(-0.0567054906659)*x_ref[2]
        ref[4,0,1]=(-0.667297458554)*x_ref[0]**o+(-0.207369757609)*x_ref[0]+(-0.982746021703)*x_ref[1]**o+(0.963677089363)*x_ref[1]+(0.917328237047)*x_ref[2]**o+(-0.684386577728)*x_ref[2]
        ref[4,1,0]=(-0.835525763762)*x_ref[0]**o+(-0.113324184435)*x_ref[0]+(0.465855661969)*x_ref[1]**o+(0.656047307997)*x_ref[1]+(0.331675778117)*x_ref[2]**o+(-0.686067177565)*x_ref[2]
        ref[4,1,1]=(-0.468668496499)*x_ref[0]**o+(0.295197251458)*x_ref[0]+(0.065102699741)*x_ref[1]**o+(-0.378381176309)*x_ref[1]+(0.933627973066)*x_ref[2]**o+(-0.91462136931)*x_ref[2]
        ref[5,0,0]=(-0.161916994488)*x_ref[0]**o+(0.3070005414)*x_ref[0]+(0.195946524325)*x_ref[1]**o+(-0.453290146527)*x_ref[1]+(0.751532065651)*x_ref[2]**o+(0.82676125654)*x_ref[2]
        ref[5,0,1]=(-0.925362363492)*x_ref[0]**o+(-0.0295351854097)*x_ref[0]+(0.533980764998)*x_ref[1]**o+(0.128080489262)*x_ref[1]+(0.502858052015)*x_ref[2]**o+(0.813196540163)*x_ref[2]
        ref[5,1,0]=(-0.740146099318)*x_ref[0]**o+(0.27623528724)*x_ref[0]+(-0.361620073401)*x_ref[1]**o+(-0.310534085108)*x_ref[1]+(-0.0363428185757)*x_ref[2]**o+(-0.145397849357)*x_ref[2]
        ref[5,1,1]=(0.362089087335)*x_ref[0]**o+(0.539359659929)*x_ref[0]+(0.00904990097799)*x_ref[1]**o+(-0.150401175722)*x_ref[1]+(-0.20465882105)*x_ref[2]**o+(-0.519331232192)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromData_Solution_rank4(self):
      """
      tests interpolate for rank 4 Data onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from Solution onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5, 3, 2),w)
      ref=Data(0,(4, 5, 3, 2),w_ref)
      if dim==2:
        arg[0,0,0,0]=(0.14016992168)*x[0]**o+(-0.396856222091)*x[0]+(0.747446299477)*x[1]**o+(-0.299063042467)*x[1]
        arg[0,0,0,1]=(0.126297805139)*x[0]**o+(0.317089000483)*x[0]+(0.0418478916266)*x[1]**o+(-0.892825689806)*x[1]
        arg[0,0,1,0]=(0.127263485532)*x[0]**o+(0.400880552184)*x[0]+(-0.858713979216)*x[1]**o+(0.238693631738)*x[1]
        arg[0,0,1,1]=(-0.985703521737)*x[0]**o+(-0.0192845851979)*x[0]+(-0.442493677109)*x[1]**o+(0.188038174612)*x[1]
        arg[0,0,2,0]=(0.814303535641)*x[0]**o+(-0.201938227136)*x[0]+(-0.134135136581)*x[1]**o+(0.939272592089)*x[1]
        arg[0,0,2,1]=(-0.609570412575)*x[0]**o+(0.181669701944)*x[0]+(0.882708213835)*x[1]**o+(0.315561722535)*x[1]
        arg[0,1,0,0]=(0.180272217085)*x[0]**o+(-0.921231698951)*x[0]+(-0.670170775864)*x[1]**o+(0.36959006719)*x[1]
        arg[0,1,0,1]=(-0.907447754449)*x[0]**o+(0.984356686322)*x[0]+(0.300621254386)*x[1]**o+(0.491995830235)*x[1]
        arg[0,1,1,0]=(-0.535808887861)*x[0]**o+(0.991647965865)*x[0]+(-0.505667913884)*x[1]**o+(-0.403245981754)*x[1]
        arg[0,1,1,1]=(-0.315687983887)*x[0]**o+(-0.570118136173)*x[0]+(0.327842038508)*x[1]**o+(-0.475833876716)*x[1]
        arg[0,1,2,0]=(0.598143043008)*x[0]**o+(0.119789453354)*x[0]+(-0.307994361027)*x[1]**o+(-0.747443620209)*x[1]
        arg[0,1,2,1]=(-0.384576138426)*x[0]**o+(-0.21431195392)*x[0]+(0.622157923623)*x[1]**o+(-0.535976986322)*x[1]
        arg[0,2,0,0]=(-0.823640169824)*x[0]**o+(-0.946436729867)*x[0]+(0.158819942161)*x[1]**o+(-0.175783146941)*x[1]
        arg[0,2,0,1]=(0.31586435609)*x[0]**o+(0.480823809413)*x[0]+(-0.155590673671)*x[1]**o+(-0.283774897002)*x[1]
        arg[0,2,1,0]=(0.973677960338)*x[0]**o+(-0.42119441385)*x[0]+(0.936434159723)*x[1]**o+(-0.271328279678)*x[1]
        arg[0,2,1,1]=(0.995977746066)*x[0]**o+(0.115566554312)*x[0]+(0.6023712777)*x[1]**o+(0.523708594385)*x[1]
        arg[0,2,2,0]=(0.690331991941)*x[0]**o+(0.708864222789)*x[0]+(-0.0209681974418)*x[1]**o+(-0.402664547241)*x[1]
        arg[0,2,2,1]=(0.793403379571)*x[0]**o+(-0.44195907862)*x[0]+(-0.750943552401)*x[1]**o+(0.87817105061)*x[1]
        arg[0,3,0,0]=(-0.960650344452)*x[0]**o+(0.918969286865)*x[0]+(-0.788611475275)*x[1]**o+(-0.917265819096)*x[1]
        arg[0,3,0,1]=(0.379234695969)*x[0]**o+(0.288784268667)*x[0]+(-0.231788371542)*x[1]**o+(-0.521488838978)*x[1]
        arg[0,3,1,0]=(-0.482353410226)*x[0]**o+(0.666269361063)*x[0]+(0.207628082266)*x[1]**o+(0.890536697637)*x[1]
        arg[0,3,1,1]=(-0.256826971067)*x[0]**o+(0.244573109457)*x[0]+(0.435182409565)*x[1]**o+(-0.98882467576)*x[1]
        arg[0,3,2,0]=(0.727681501266)*x[0]**o+(0.174117939756)*x[0]+(0.626998345198)*x[1]**o+(0.666518756047)*x[1]
        arg[0,3,2,1]=(-0.125855300157)*x[0]**o+(-0.750291117974)*x[0]+(0.840722993443)*x[1]**o+(0.0798848819291)*x[1]
        arg[0,4,0,0]=(-0.156808480304)*x[0]**o+(-0.864855000875)*x[0]+(-0.825786767174)*x[1]**o+(-0.563999207624)*x[1]
        arg[0,4,0,1]=(-0.890116284779)*x[0]**o+(0.795471417269)*x[0]+(0.326454331959)*x[1]**o+(-0.376644334859)*x[1]
        arg[0,4,1,0]=(-0.453181887597)*x[0]**o+(0.488149970589)*x[0]+(0.506352954527)*x[1]**o+(0.971016325918)*x[1]
        arg[0,4,1,1]=(0.156976484311)*x[0]**o+(0.988818166411)*x[0]+(0.948579296281)*x[1]**o+(-0.80303322078)*x[1]
        arg[0,4,2,0]=(-0.0625333956516)*x[0]**o+(0.357609677642)*x[0]+(0.00245182913993)*x[1]**o+(-0.162058851234)*x[1]
        arg[0,4,2,1]=(0.645504631508)*x[0]**o+(-0.0107543576265)*x[0]+(0.15490632437)*x[1]**o+(0.841985786269)*x[1]
        arg[1,0,0,0]=(0.703923364197)*x[0]**o+(-0.424818178123)*x[0]+(-0.744399116071)*x[1]**o+(0.303429987787)*x[1]
        arg[1,0,0,1]=(-0.92849671168)*x[0]**o+(0.050597602551)*x[0]+(0.91000243103)*x[1]**o+(-0.742775174195)*x[1]
        arg[1,0,1,0]=(0.281218131435)*x[0]**o+(0.171029842909)*x[0]+(-0.65806576709)*x[1]**o+(-0.718069456213)*x[1]
        arg[1,0,1,1]=(0.312255067232)*x[0]**o+(0.566401030665)*x[0]+(-0.941709776271)*x[1]**o+(0.0111896041808)*x[1]
        arg[1,0,2,0]=(-0.886019321094)*x[0]**o+(-0.688815756004)*x[0]+(-0.595491288141)*x[1]**o+(0.343402495621)*x[1]
        arg[1,0,2,1]=(-0.681063845863)*x[0]**o+(-0.676550403066)*x[0]+(0.186106532969)*x[1]**o+(0.181779537727)*x[1]
        arg[1,1,0,0]=(0.128588939989)*x[0]**o+(0.477711214734)*x[0]+(-0.17697460919)*x[1]**o+(0.0217208670244)*x[1]
        arg[1,1,0,1]=(-0.652028522296)*x[0]**o+(-0.503146425209)*x[0]+(-0.427973128189)*x[1]**o+(-0.182775422425)*x[1]
        arg[1,1,1,0]=(-0.0499349942904)*x[0]**o+(-0.0183804745613)*x[0]+(-0.912604094211)*x[1]**o+(-0.949181938689)*x[1]
        arg[1,1,1,1]=(-0.472737626621)*x[0]**o+(-0.142195314687)*x[0]+(0.580775749383)*x[1]**o+(-0.832549796746)*x[1]
        arg[1,1,2,0]=(-0.288756468146)*x[0]**o+(-0.50657056474)*x[0]+(-0.809713268333)*x[1]**o+(0.598450102365)*x[1]
        arg[1,1,2,1]=(-0.259779636241)*x[0]**o+(-0.227925001879)*x[0]+(-0.0752800702413)*x[1]**o+(-0.438841969161)*x[1]
        arg[1,2,0,0]=(-0.617835522076)*x[0]**o+(-0.969275786274)*x[0]+(0.899717033555)*x[1]**o+(0.77648944909)*x[1]
        arg[1,2,0,1]=(-0.0619395581419)*x[0]**o+(0.926394874574)*x[0]+(0.984605961354)*x[1]**o+(0.253347734181)*x[1]
        arg[1,2,1,0]=(0.748474342269)*x[0]**o+(0.0794003341104)*x[0]+(-0.244607682532)*x[1]**o+(-0.228787873564)*x[1]
        arg[1,2,1,1]=(-0.266853205536)*x[0]**o+(0.529714058158)*x[0]+(-0.421599080872)*x[1]**o+(-0.595557599245)*x[1]
        arg[1,2,2,0]=(-0.377962443753)*x[0]**o+(0.609550409525)*x[0]+(0.951904771097)*x[1]**o+(-0.123551801933)*x[1]
        arg[1,2,2,1]=(0.3262671136)*x[0]**o+(-0.574281078833)*x[0]+(0.308261022622)*x[1]**o+(-0.784762723843)*x[1]
        arg[1,3,0,0]=(-0.419601410829)*x[0]**o+(-0.576064062071)*x[0]+(-0.431574278091)*x[1]**o+(-0.264303418881)*x[1]
        arg[1,3,0,1]=(0.176058414693)*x[0]**o+(0.672752046411)*x[0]+(-0.8579992002)*x[1]**o+(-0.135904959185)*x[1]
        arg[1,3,1,0]=(0.84480795105)*x[0]**o+(0.846846799225)*x[0]+(0.134417432017)*x[1]**o+(-0.725316162128)*x[1]
        arg[1,3,1,1]=(-0.102162279538)*x[0]**o+(0.451180031789)*x[0]+(0.490175190736)*x[1]**o+(0.811823281177)*x[1]
        arg[1,3,2,0]=(-0.582177348293)*x[0]**o+(-0.74631626712)*x[0]+(-0.275524221012)*x[1]**o+(-0.658276720144)*x[1]
        arg[1,3,2,1]=(0.304511057297)*x[0]**o+(-0.354531411968)*x[0]+(0.0944491594546)*x[1]**o+(0.619039121597)*x[1]
        arg[1,4,0,0]=(0.754898520324)*x[0]**o+(-0.28914218119)*x[0]+(0.8660675945)*x[1]**o+(0.526777161228)*x[1]
        arg[1,4,0,1]=(-0.820150637429)*x[0]**o+(-0.399203443857)*x[0]+(0.118863382167)*x[1]**o+(0.269335037373)*x[1]
        arg[1,4,1,0]=(-0.669389232967)*x[0]**o+(-0.119080399209)*x[0]+(0.999919873616)*x[1]**o+(0.733390578378)*x[1]
        arg[1,4,1,1]=(-0.85147101348)*x[0]**o+(-0.0453996523642)*x[0]+(0.00891698330334)*x[1]**o+(-0.27128670805)*x[1]
        arg[1,4,2,0]=(-0.946558097166)*x[0]**o+(0.0141300604882)*x[0]+(0.04075090617)*x[1]**o+(-0.141095386138)*x[1]
        arg[1,4,2,1]=(0.613700159605)*x[0]**o+(0.115107829201)*x[0]+(0.907429968641)*x[1]**o+(0.901820725424)*x[1]
        arg[2,0,0,0]=(-0.629486588984)*x[0]**o+(-0.590908868493)*x[0]+(0.689333052519)*x[1]**o+(0.22096464697)*x[1]
        arg[2,0,0,1]=(-0.475364708492)*x[0]**o+(-0.3195126467)*x[0]+(-0.688082519853)*x[1]**o+(-0.889722517035)*x[1]
        arg[2,0,1,0]=(0.931261765284)*x[0]**o+(0.399101548845)*x[0]+(-0.185342345742)*x[1]**o+(0.188927724838)*x[1]
        arg[2,0,1,1]=(-0.195889209768)*x[0]**o+(-0.0878952476483)*x[0]+(0.636980893598)*x[1]**o+(0.827709469156)*x[1]
        arg[2,0,2,0]=(0.644658731705)*x[0]**o+(0.275771474382)*x[0]+(0.0802903363405)*x[1]**o+(-0.157785300253)*x[1]
        arg[2,0,2,1]=(0.324153739483)*x[0]**o+(-0.978043993375)*x[0]+(0.440060321393)*x[1]**o+(0.103952166755)*x[1]
        arg[2,1,0,0]=(-0.233567312787)*x[0]**o+(0.139529308019)*x[0]+(-0.200168759944)*x[1]**o+(-0.629057762953)*x[1]
        arg[2,1,0,1]=(-0.407808993281)*x[0]**o+(-0.208049914815)*x[0]+(0.676817990818)*x[1]**o+(-0.27174220248)*x[1]
        arg[2,1,1,0]=(-0.159113456499)*x[0]**o+(0.586749846436)*x[0]+(0.112658884741)*x[1]**o+(0.521917298039)*x[1]
        arg[2,1,1,1]=(-0.377986619436)*x[0]**o+(0.920912593255)*x[0]+(0.61285598265)*x[1]**o+(0.491316381102)*x[1]
        arg[2,1,2,0]=(-0.594573349393)*x[0]**o+(-0.784940398408)*x[0]+(0.0602427709094)*x[1]**o+(0.381334187216)*x[1]
        arg[2,1,2,1]=(-0.359402310466)*x[0]**o+(0.243014104221)*x[0]+(0.960587701667)*x[1]**o+(0.653124687952)*x[1]
        arg[2,2,0,0]=(0.667031182369)*x[0]**o+(0.099374903559)*x[0]+(0.00279841725989)*x[1]**o+(-0.413751543412)*x[1]
        arg[2,2,0,1]=(0.793557898241)*x[0]**o+(0.146572317359)*x[0]+(0.812964170008)*x[1]**o+(-0.988110793397)*x[1]
        arg[2,2,1,0]=(0.900661246137)*x[0]**o+(-0.997975978586)*x[0]+(0.63300171387)*x[1]**o+(-0.103878548778)*x[1]
        arg[2,2,1,1]=(-0.456464050469)*x[0]**o+(0.830699811714)*x[0]+(-0.186689927425)*x[1]**o+(-0.100170151164)*x[1]
        arg[2,2,2,0]=(0.732282428164)*x[0]**o+(0.928947589228)*x[0]+(-0.282773641579)*x[1]**o+(-0.927533112027)*x[1]
        arg[2,2,2,1]=(0.768111868708)*x[0]**o+(-0.308231472858)*x[0]+(0.890807565689)*x[1]**o+(-0.731196891492)*x[1]
        arg[2,3,0,0]=(0.559658559574)*x[0]**o+(0.104536615096)*x[0]+(-0.162594824125)*x[1]**o+(-0.172859200721)*x[1]
        arg[2,3,0,1]=(0.769297853152)*x[0]**o+(0.346598673904)*x[0]+(0.0313124779153)*x[1]**o+(-0.751767979251)*x[1]
        arg[2,3,1,0]=(0.870887050816)*x[0]**o+(0.593904859463)*x[0]+(-0.108083837899)*x[1]**o+(0.5467383682)*x[1]
        arg[2,3,1,1]=(0.244253227645)*x[0]**o+(0.911176738654)*x[0]+(-0.497855487714)*x[1]**o+(0.226443212585)*x[1]
        arg[2,3,2,0]=(-0.599893560596)*x[0]**o+(-0.893030581717)*x[0]+(-0.745249903413)*x[1]**o+(0.563740068619)*x[1]
        arg[2,3,2,1]=(-0.198088241103)*x[0]**o+(-0.171306730647)*x[0]+(-0.917417138299)*x[1]**o+(0.233920617125)*x[1]
        arg[2,4,0,0]=(0.804585270618)*x[0]**o+(0.3784879961)*x[0]+(0.621076476928)*x[1]**o+(-0.534671153952)*x[1]
        arg[2,4,0,1]=(0.380555159947)*x[0]**o+(-0.630487045841)*x[0]+(0.917413683072)*x[1]**o+(0.491925783834)*x[1]
        arg[2,4,1,0]=(0.946642644964)*x[0]**o+(-0.236438579482)*x[0]+(0.525585953739)*x[1]**o+(0.499940951133)*x[1]
        arg[2,4,1,1]=(0.827450889548)*x[0]**o+(-0.73264543626)*x[0]+(0.0626492901072)*x[1]**o+(-0.882759632862)*x[1]
        arg[2,4,2,0]=(-0.0671814496498)*x[0]**o+(0.108634446052)*x[0]+(-0.421081640868)*x[1]**o+(-0.546455336395)*x[1]
        arg[2,4,2,1]=(-0.601339784479)*x[0]**o+(-0.206937965916)*x[0]+(-0.849120792543)*x[1]**o+(0.615842037441)*x[1]
        arg[3,0,0,0]=(0.640466650913)*x[0]**o+(-0.936176165762)*x[0]+(0.12758686525)*x[1]**o+(-0.104771286099)*x[1]
        arg[3,0,0,1]=(-0.868109993006)*x[0]**o+(0.470815003671)*x[0]+(-0.301208743474)*x[1]**o+(-0.296271408095)*x[1]
        arg[3,0,1,0]=(0.66584252787)*x[0]**o+(-0.889549573986)*x[0]+(-0.0876076820512)*x[1]**o+(-0.249000029178)*x[1]
        arg[3,0,1,1]=(0.289388044606)*x[0]**o+(0.674519856916)*x[0]+(-0.721031236925)*x[1]**o+(0.093281544052)*x[1]
        arg[3,0,2,0]=(-0.0558863543119)*x[0]**o+(-0.367544430651)*x[0]+(-0.85580152565)*x[1]**o+(-0.140166367203)*x[1]
        arg[3,0,2,1]=(0.600709511675)*x[0]**o+(0.286322916564)*x[0]+(-0.544094007372)*x[1]**o+(0.470305833347)*x[1]
        arg[3,1,0,0]=(-0.828780197168)*x[0]**o+(-0.568096532278)*x[0]+(0.435127414299)*x[1]**o+(0.96890877255)*x[1]
        arg[3,1,0,1]=(0.191455614175)*x[0]**o+(-0.99507613827)*x[0]+(0.0735829936134)*x[1]**o+(0.635849313153)*x[1]
        arg[3,1,1,0]=(-0.839049013591)*x[0]**o+(-0.718731394184)*x[0]+(0.575853740085)*x[1]**o+(-0.57924857477)*x[1]
        arg[3,1,1,1]=(0.116876810453)*x[0]**o+(0.145727923885)*x[0]+(0.0155069936368)*x[1]**o+(0.223214885433)*x[1]
        arg[3,1,2,0]=(-0.674811720223)*x[0]**o+(0.651640807063)*x[0]+(-0.652022625703)*x[1]**o+(-0.119110788344)*x[1]
        arg[3,1,2,1]=(-0.544748794193)*x[0]**o+(-0.689440165483)*x[0]+(0.936900319026)*x[1]**o+(-0.337287311564)*x[1]
        arg[3,2,0,0]=(-0.355059804554)*x[0]**o+(0.458306329535)*x[0]+(0.172261907494)*x[1]**o+(0.498548138594)*x[1]
        arg[3,2,0,1]=(0.624420187129)*x[0]**o+(0.808931844722)*x[0]+(-0.690635894194)*x[1]**o+(-0.156122444554)*x[1]
        arg[3,2,1,0]=(-0.190029529776)*x[0]**o+(0.925600418137)*x[0]+(0.601040324953)*x[1]**o+(0.615485749173)*x[1]
        arg[3,2,1,1]=(0.679796674662)*x[0]**o+(0.879285148869)*x[0]+(0.0476373514288)*x[1]**o+(0.37655760501)*x[1]
        arg[3,2,2,0]=(0.231748550393)*x[0]**o+(0.209858138221)*x[0]+(-0.88372579866)*x[1]**o+(0.30534744967)*x[1]
        arg[3,2,2,1]=(-0.772177697671)*x[0]**o+(0.012439044435)*x[0]+(-0.293218976284)*x[1]**o+(0.306955066595)*x[1]
        arg[3,3,0,0]=(0.387256929042)*x[0]**o+(-0.434015953267)*x[0]+(-0.098736946868)*x[1]**o+(-0.0837712534186)*x[1]
        arg[3,3,0,1]=(0.396246252967)*x[0]**o+(-0.338955181605)*x[0]+(-0.353504371554)*x[1]**o+(-0.969811402293)*x[1]
        arg[3,3,1,0]=(-0.336508964723)*x[0]**o+(-0.411882233342)*x[0]+(-0.603965321132)*x[1]**o+(0.177221705416)*x[1]
        arg[3,3,1,1]=(0.786971174387)*x[0]**o+(0.759003522166)*x[0]+(-0.0961289050409)*x[1]**o+(-0.0811258682982)*x[1]
        arg[3,3,2,0]=(0.526266925685)*x[0]**o+(-0.113145219611)*x[0]+(-0.208427816884)*x[1]**o+(0.291484710079)*x[1]
        arg[3,3,2,1]=(-0.591153907468)*x[0]**o+(-0.342892047641)*x[0]+(0.865954283634)*x[1]**o+(-0.22112030903)*x[1]
        arg[3,4,0,0]=(0.755162128186)*x[0]**o+(0.494457469003)*x[0]+(0.889254517137)*x[1]**o+(0.880654056969)*x[1]
        arg[3,4,0,1]=(0.450983401629)*x[0]**o+(-0.0855489110466)*x[0]+(0.0601625202676)*x[1]**o+(-0.0730383237708)*x[1]
        arg[3,4,1,0]=(0.495514588456)*x[0]**o+(-0.0895269876334)*x[0]+(-0.89729630673)*x[1]**o+(0.759465768614)*x[1]
        arg[3,4,1,1]=(-0.666472982468)*x[0]**o+(-0.251715028171)*x[0]+(-0.0623287375223)*x[1]**o+(-0.895663326995)*x[1]
        arg[3,4,2,0]=(0.489737923112)*x[0]**o+(0.868066981889)*x[0]+(0.0371747456704)*x[1]**o+(-0.528951636028)*x[1]
        arg[3,4,2,1]=(0.556312598455)*x[0]**o+(-0.0979773350236)*x[0]+(-0.82785928691)*x[1]**o+(-0.567596765858)*x[1]
        ref[0,0,0,0]=(0.14016992168)*x_ref[0]**o+(-0.396856222091)*x_ref[0]+(0.747446299477)*x_ref[1]**o+(-0.299063042467)*x_ref[1]
        ref[0,0,0,1]=(0.126297805139)*x_ref[0]**o+(0.317089000483)*x_ref[0]+(0.0418478916266)*x_ref[1]**o+(-0.892825689806)*x_ref[1]
        ref[0,0,1,0]=(0.127263485532)*x_ref[0]**o+(0.400880552184)*x_ref[0]+(-0.858713979216)*x_ref[1]**o+(0.238693631738)*x_ref[1]
        ref[0,0,1,1]=(-0.985703521737)*x_ref[0]**o+(-0.0192845851979)*x_ref[0]+(-0.442493677109)*x_ref[1]**o+(0.188038174612)*x_ref[1]
        ref[0,0,2,0]=(0.814303535641)*x_ref[0]**o+(-0.201938227136)*x_ref[0]+(-0.134135136581)*x_ref[1]**o+(0.939272592089)*x_ref[1]
        ref[0,0,2,1]=(-0.609570412575)*x_ref[0]**o+(0.181669701944)*x_ref[0]+(0.882708213835)*x_ref[1]**o+(0.315561722535)*x_ref[1]
        ref[0,1,0,0]=(0.180272217085)*x_ref[0]**o+(-0.921231698951)*x_ref[0]+(-0.670170775864)*x_ref[1]**o+(0.36959006719)*x_ref[1]
        ref[0,1,0,1]=(-0.907447754449)*x_ref[0]**o+(0.984356686322)*x_ref[0]+(0.300621254386)*x_ref[1]**o+(0.491995830235)*x_ref[1]
        ref[0,1,1,0]=(-0.535808887861)*x_ref[0]**o+(0.991647965865)*x_ref[0]+(-0.505667913884)*x_ref[1]**o+(-0.403245981754)*x_ref[1]
        ref[0,1,1,1]=(-0.315687983887)*x_ref[0]**o+(-0.570118136173)*x_ref[0]+(0.327842038508)*x_ref[1]**o+(-0.475833876716)*x_ref[1]
        ref[0,1,2,0]=(0.598143043008)*x_ref[0]**o+(0.119789453354)*x_ref[0]+(-0.307994361027)*x_ref[1]**o+(-0.747443620209)*x_ref[1]
        ref[0,1,2,1]=(-0.384576138426)*x_ref[0]**o+(-0.21431195392)*x_ref[0]+(0.622157923623)*x_ref[1]**o+(-0.535976986322)*x_ref[1]
        ref[0,2,0,0]=(-0.823640169824)*x_ref[0]**o+(-0.946436729867)*x_ref[0]+(0.158819942161)*x_ref[1]**o+(-0.175783146941)*x_ref[1]
        ref[0,2,0,1]=(0.31586435609)*x_ref[0]**o+(0.480823809413)*x_ref[0]+(-0.155590673671)*x_ref[1]**o+(-0.283774897002)*x_ref[1]
        ref[0,2,1,0]=(0.973677960338)*x_ref[0]**o+(-0.42119441385)*x_ref[0]+(0.936434159723)*x_ref[1]**o+(-0.271328279678)*x_ref[1]
        ref[0,2,1,1]=(0.995977746066)*x_ref[0]**o+(0.115566554312)*x_ref[0]+(0.6023712777)*x_ref[1]**o+(0.523708594385)*x_ref[1]
        ref[0,2,2,0]=(0.690331991941)*x_ref[0]**o+(0.708864222789)*x_ref[0]+(-0.0209681974418)*x_ref[1]**o+(-0.402664547241)*x_ref[1]
        ref[0,2,2,1]=(0.793403379571)*x_ref[0]**o+(-0.44195907862)*x_ref[0]+(-0.750943552401)*x_ref[1]**o+(0.87817105061)*x_ref[1]
        ref[0,3,0,0]=(-0.960650344452)*x_ref[0]**o+(0.918969286865)*x_ref[0]+(-0.788611475275)*x_ref[1]**o+(-0.917265819096)*x_ref[1]
        ref[0,3,0,1]=(0.379234695969)*x_ref[0]**o+(0.288784268667)*x_ref[0]+(-0.231788371542)*x_ref[1]**o+(-0.521488838978)*x_ref[1]
        ref[0,3,1,0]=(-0.482353410226)*x_ref[0]**o+(0.666269361063)*x_ref[0]+(0.207628082266)*x_ref[1]**o+(0.890536697637)*x_ref[1]
        ref[0,3,1,1]=(-0.256826971067)*x_ref[0]**o+(0.244573109457)*x_ref[0]+(0.435182409565)*x_ref[1]**o+(-0.98882467576)*x_ref[1]
        ref[0,3,2,0]=(0.727681501266)*x_ref[0]**o+(0.174117939756)*x_ref[0]+(0.626998345198)*x_ref[1]**o+(0.666518756047)*x_ref[1]
        ref[0,3,2,1]=(-0.125855300157)*x_ref[0]**o+(-0.750291117974)*x_ref[0]+(0.840722993443)*x_ref[1]**o+(0.0798848819291)*x_ref[1]
        ref[0,4,0,0]=(-0.156808480304)*x_ref[0]**o+(-0.864855000875)*x_ref[0]+(-0.825786767174)*x_ref[1]**o+(-0.563999207624)*x_ref[1]
        ref[0,4,0,1]=(-0.890116284779)*x_ref[0]**o+(0.795471417269)*x_ref[0]+(0.326454331959)*x_ref[1]**o+(-0.376644334859)*x_ref[1]
        ref[0,4,1,0]=(-0.453181887597)*x_ref[0]**o+(0.488149970589)*x_ref[0]+(0.506352954527)*x_ref[1]**o+(0.971016325918)*x_ref[1]
        ref[0,4,1,1]=(0.156976484311)*x_ref[0]**o+(0.988818166411)*x_ref[0]+(0.948579296281)*x_ref[1]**o+(-0.80303322078)*x_ref[1]
        ref[0,4,2,0]=(-0.0625333956516)*x_ref[0]**o+(0.357609677642)*x_ref[0]+(0.00245182913993)*x_ref[1]**o+(-0.162058851234)*x_ref[1]
        ref[0,4,2,1]=(0.645504631508)*x_ref[0]**o+(-0.0107543576265)*x_ref[0]+(0.15490632437)*x_ref[1]**o+(0.841985786269)*x_ref[1]
        ref[1,0,0,0]=(0.703923364197)*x_ref[0]**o+(-0.424818178123)*x_ref[0]+(-0.744399116071)*x_ref[1]**o+(0.303429987787)*x_ref[1]
        ref[1,0,0,1]=(-0.92849671168)*x_ref[0]**o+(0.050597602551)*x_ref[0]+(0.91000243103)*x_ref[1]**o+(-0.742775174195)*x_ref[1]
        ref[1,0,1,0]=(0.281218131435)*x_ref[0]**o+(0.171029842909)*x_ref[0]+(-0.65806576709)*x_ref[1]**o+(-0.718069456213)*x_ref[1]
        ref[1,0,1,1]=(0.312255067232)*x_ref[0]**o+(0.566401030665)*x_ref[0]+(-0.941709776271)*x_ref[1]**o+(0.0111896041808)*x_ref[1]
        ref[1,0,2,0]=(-0.886019321094)*x_ref[0]**o+(-0.688815756004)*x_ref[0]+(-0.595491288141)*x_ref[1]**o+(0.343402495621)*x_ref[1]
        ref[1,0,2,1]=(-0.681063845863)*x_ref[0]**o+(-0.676550403066)*x_ref[0]+(0.186106532969)*x_ref[1]**o+(0.181779537727)*x_ref[1]
        ref[1,1,0,0]=(0.128588939989)*x_ref[0]**o+(0.477711214734)*x_ref[0]+(-0.17697460919)*x_ref[1]**o+(0.0217208670244)*x_ref[1]
        ref[1,1,0,1]=(-0.652028522296)*x_ref[0]**o+(-0.503146425209)*x_ref[0]+(-0.427973128189)*x_ref[1]**o+(-0.182775422425)*x_ref[1]
        ref[1,1,1,0]=(-0.0499349942904)*x_ref[0]**o+(-0.0183804745613)*x_ref[0]+(-0.912604094211)*x_ref[1]**o+(-0.949181938689)*x_ref[1]
        ref[1,1,1,1]=(-0.472737626621)*x_ref[0]**o+(-0.142195314687)*x_ref[0]+(0.580775749383)*x_ref[1]**o+(-0.832549796746)*x_ref[1]
        ref[1,1,2,0]=(-0.288756468146)*x_ref[0]**o+(-0.50657056474)*x_ref[0]+(-0.809713268333)*x_ref[1]**o+(0.598450102365)*x_ref[1]
        ref[1,1,2,1]=(-0.259779636241)*x_ref[0]**o+(-0.227925001879)*x_ref[0]+(-0.0752800702413)*x_ref[1]**o+(-0.438841969161)*x_ref[1]
        ref[1,2,0,0]=(-0.617835522076)*x_ref[0]**o+(-0.969275786274)*x_ref[0]+(0.899717033555)*x_ref[1]**o+(0.77648944909)*x_ref[1]
        ref[1,2,0,1]=(-0.0619395581419)*x_ref[0]**o+(0.926394874574)*x_ref[0]+(0.984605961354)*x_ref[1]**o+(0.253347734181)*x_ref[1]
        ref[1,2,1,0]=(0.748474342269)*x_ref[0]**o+(0.0794003341104)*x_ref[0]+(-0.244607682532)*x_ref[1]**o+(-0.228787873564)*x_ref[1]
        ref[1,2,1,1]=(-0.266853205536)*x_ref[0]**o+(0.529714058158)*x_ref[0]+(-0.421599080872)*x_ref[1]**o+(-0.595557599245)*x_ref[1]
        ref[1,2,2,0]=(-0.377962443753)*x_ref[0]**o+(0.609550409525)*x_ref[0]+(0.951904771097)*x_ref[1]**o+(-0.123551801933)*x_ref[1]
        ref[1,2,2,1]=(0.3262671136)*x_ref[0]**o+(-0.574281078833)*x_ref[0]+(0.308261022622)*x_ref[1]**o+(-0.784762723843)*x_ref[1]
        ref[1,3,0,0]=(-0.419601410829)*x_ref[0]**o+(-0.576064062071)*x_ref[0]+(-0.431574278091)*x_ref[1]**o+(-0.264303418881)*x_ref[1]
        ref[1,3,0,1]=(0.176058414693)*x_ref[0]**o+(0.672752046411)*x_ref[0]+(-0.8579992002)*x_ref[1]**o+(-0.135904959185)*x_ref[1]
        ref[1,3,1,0]=(0.84480795105)*x_ref[0]**o+(0.846846799225)*x_ref[0]+(0.134417432017)*x_ref[1]**o+(-0.725316162128)*x_ref[1]
        ref[1,3,1,1]=(-0.102162279538)*x_ref[0]**o+(0.451180031789)*x_ref[0]+(0.490175190736)*x_ref[1]**o+(0.811823281177)*x_ref[1]
        ref[1,3,2,0]=(-0.582177348293)*x_ref[0]**o+(-0.74631626712)*x_ref[0]+(-0.275524221012)*x_ref[1]**o+(-0.658276720144)*x_ref[1]
        ref[1,3,2,1]=(0.304511057297)*x_ref[0]**o+(-0.354531411968)*x_ref[0]+(0.0944491594546)*x_ref[1]**o+(0.619039121597)*x_ref[1]
        ref[1,4,0,0]=(0.754898520324)*x_ref[0]**o+(-0.28914218119)*x_ref[0]+(0.8660675945)*x_ref[1]**o+(0.526777161228)*x_ref[1]
        ref[1,4,0,1]=(-0.820150637429)*x_ref[0]**o+(-0.399203443857)*x_ref[0]+(0.118863382167)*x_ref[1]**o+(0.269335037373)*x_ref[1]
        ref[1,4,1,0]=(-0.669389232967)*x_ref[0]**o+(-0.119080399209)*x_ref[0]+(0.999919873616)*x_ref[1]**o+(0.733390578378)*x_ref[1]
        ref[1,4,1,1]=(-0.85147101348)*x_ref[0]**o+(-0.0453996523642)*x_ref[0]+(0.00891698330334)*x_ref[1]**o+(-0.27128670805)*x_ref[1]
        ref[1,4,2,0]=(-0.946558097166)*x_ref[0]**o+(0.0141300604882)*x_ref[0]+(0.04075090617)*x_ref[1]**o+(-0.141095386138)*x_ref[1]
        ref[1,4,2,1]=(0.613700159605)*x_ref[0]**o+(0.115107829201)*x_ref[0]+(0.907429968641)*x_ref[1]**o+(0.901820725424)*x_ref[1]
        ref[2,0,0,0]=(-0.629486588984)*x_ref[0]**o+(-0.590908868493)*x_ref[0]+(0.689333052519)*x_ref[1]**o+(0.22096464697)*x_ref[1]
        ref[2,0,0,1]=(-0.475364708492)*x_ref[0]**o+(-0.3195126467)*x_ref[0]+(-0.688082519853)*x_ref[1]**o+(-0.889722517035)*x_ref[1]
        ref[2,0,1,0]=(0.931261765284)*x_ref[0]**o+(0.399101548845)*x_ref[0]+(-0.185342345742)*x_ref[1]**o+(0.188927724838)*x_ref[1]
        ref[2,0,1,1]=(-0.195889209768)*x_ref[0]**o+(-0.0878952476483)*x_ref[0]+(0.636980893598)*x_ref[1]**o+(0.827709469156)*x_ref[1]
        ref[2,0,2,0]=(0.644658731705)*x_ref[0]**o+(0.275771474382)*x_ref[0]+(0.0802903363405)*x_ref[1]**o+(-0.157785300253)*x_ref[1]
        ref[2,0,2,1]=(0.324153739483)*x_ref[0]**o+(-0.978043993375)*x_ref[0]+(0.440060321393)*x_ref[1]**o+(0.103952166755)*x_ref[1]
        ref[2,1,0,0]=(-0.233567312787)*x_ref[0]**o+(0.139529308019)*x_ref[0]+(-0.200168759944)*x_ref[1]**o+(-0.629057762953)*x_ref[1]
        ref[2,1,0,1]=(-0.407808993281)*x_ref[0]**o+(-0.208049914815)*x_ref[0]+(0.676817990818)*x_ref[1]**o+(-0.27174220248)*x_ref[1]
        ref[2,1,1,0]=(-0.159113456499)*x_ref[0]**o+(0.586749846436)*x_ref[0]+(0.112658884741)*x_ref[1]**o+(0.521917298039)*x_ref[1]
        ref[2,1,1,1]=(-0.377986619436)*x_ref[0]**o+(0.920912593255)*x_ref[0]+(0.61285598265)*x_ref[1]**o+(0.491316381102)*x_ref[1]
        ref[2,1,2,0]=(-0.594573349393)*x_ref[0]**o+(-0.784940398408)*x_ref[0]+(0.0602427709094)*x_ref[1]**o+(0.381334187216)*x_ref[1]
        ref[2,1,2,1]=(-0.359402310466)*x_ref[0]**o+(0.243014104221)*x_ref[0]+(0.960587701667)*x_ref[1]**o+(0.653124687952)*x_ref[1]
        ref[2,2,0,0]=(0.667031182369)*x_ref[0]**o+(0.099374903559)*x_ref[0]+(0.00279841725989)*x_ref[1]**o+(-0.413751543412)*x_ref[1]
        ref[2,2,0,1]=(0.793557898241)*x_ref[0]**o+(0.146572317359)*x_ref[0]+(0.812964170008)*x_ref[1]**o+(-0.988110793397)*x_ref[1]
        ref[2,2,1,0]=(0.900661246137)*x_ref[0]**o+(-0.997975978586)*x_ref[0]+(0.63300171387)*x_ref[1]**o+(-0.103878548778)*x_ref[1]
        ref[2,2,1,1]=(-0.456464050469)*x_ref[0]**o+(0.830699811714)*x_ref[0]+(-0.186689927425)*x_ref[1]**o+(-0.100170151164)*x_ref[1]
        ref[2,2,2,0]=(0.732282428164)*x_ref[0]**o+(0.928947589228)*x_ref[0]+(-0.282773641579)*x_ref[1]**o+(-0.927533112027)*x_ref[1]
        ref[2,2,2,1]=(0.768111868708)*x_ref[0]**o+(-0.308231472858)*x_ref[0]+(0.890807565689)*x_ref[1]**o+(-0.731196891492)*x_ref[1]
        ref[2,3,0,0]=(0.559658559574)*x_ref[0]**o+(0.104536615096)*x_ref[0]+(-0.162594824125)*x_ref[1]**o+(-0.172859200721)*x_ref[1]
        ref[2,3,0,1]=(0.769297853152)*x_ref[0]**o+(0.346598673904)*x_ref[0]+(0.0313124779153)*x_ref[1]**o+(-0.751767979251)*x_ref[1]
        ref[2,3,1,0]=(0.870887050816)*x_ref[0]**o+(0.593904859463)*x_ref[0]+(-0.108083837899)*x_ref[1]**o+(0.5467383682)*x_ref[1]
        ref[2,3,1,1]=(0.244253227645)*x_ref[0]**o+(0.911176738654)*x_ref[0]+(-0.497855487714)*x_ref[1]**o+(0.226443212585)*x_ref[1]
        ref[2,3,2,0]=(-0.599893560596)*x_ref[0]**o+(-0.893030581717)*x_ref[0]+(-0.745249903413)*x_ref[1]**o+(0.563740068619)*x_ref[1]
        ref[2,3,2,1]=(-0.198088241103)*x_ref[0]**o+(-0.171306730647)*x_ref[0]+(-0.917417138299)*x_ref[1]**o+(0.233920617125)*x_ref[1]
        ref[2,4,0,0]=(0.804585270618)*x_ref[0]**o+(0.3784879961)*x_ref[0]+(0.621076476928)*x_ref[1]**o+(-0.534671153952)*x_ref[1]
        ref[2,4,0,1]=(0.380555159947)*x_ref[0]**o+(-0.630487045841)*x_ref[0]+(0.917413683072)*x_ref[1]**o+(0.491925783834)*x_ref[1]
        ref[2,4,1,0]=(0.946642644964)*x_ref[0]**o+(-0.236438579482)*x_ref[0]+(0.525585953739)*x_ref[1]**o+(0.499940951133)*x_ref[1]
        ref[2,4,1,1]=(0.827450889548)*x_ref[0]**o+(-0.73264543626)*x_ref[0]+(0.0626492901072)*x_ref[1]**o+(-0.882759632862)*x_ref[1]
        ref[2,4,2,0]=(-0.0671814496498)*x_ref[0]**o+(0.108634446052)*x_ref[0]+(-0.421081640868)*x_ref[1]**o+(-0.546455336395)*x_ref[1]
        ref[2,4,2,1]=(-0.601339784479)*x_ref[0]**o+(-0.206937965916)*x_ref[0]+(-0.849120792543)*x_ref[1]**o+(0.615842037441)*x_ref[1]
        ref[3,0,0,0]=(0.640466650913)*x_ref[0]**o+(-0.936176165762)*x_ref[0]+(0.12758686525)*x_ref[1]**o+(-0.104771286099)*x_ref[1]
        ref[3,0,0,1]=(-0.868109993006)*x_ref[0]**o+(0.470815003671)*x_ref[0]+(-0.301208743474)*x_ref[1]**o+(-0.296271408095)*x_ref[1]
        ref[3,0,1,0]=(0.66584252787)*x_ref[0]**o+(-0.889549573986)*x_ref[0]+(-0.0876076820512)*x_ref[1]**o+(-0.249000029178)*x_ref[1]
        ref[3,0,1,1]=(0.289388044606)*x_ref[0]**o+(0.674519856916)*x_ref[0]+(-0.721031236925)*x_ref[1]**o+(0.093281544052)*x_ref[1]
        ref[3,0,2,0]=(-0.0558863543119)*x_ref[0]**o+(-0.367544430651)*x_ref[0]+(-0.85580152565)*x_ref[1]**o+(-0.140166367203)*x_ref[1]
        ref[3,0,2,1]=(0.600709511675)*x_ref[0]**o+(0.286322916564)*x_ref[0]+(-0.544094007372)*x_ref[1]**o+(0.470305833347)*x_ref[1]
        ref[3,1,0,0]=(-0.828780197168)*x_ref[0]**o+(-0.568096532278)*x_ref[0]+(0.435127414299)*x_ref[1]**o+(0.96890877255)*x_ref[1]
        ref[3,1,0,1]=(0.191455614175)*x_ref[0]**o+(-0.99507613827)*x_ref[0]+(0.0735829936134)*x_ref[1]**o+(0.635849313153)*x_ref[1]
        ref[3,1,1,0]=(-0.839049013591)*x_ref[0]**o+(-0.718731394184)*x_ref[0]+(0.575853740085)*x_ref[1]**o+(-0.57924857477)*x_ref[1]
        ref[3,1,1,1]=(0.116876810453)*x_ref[0]**o+(0.145727923885)*x_ref[0]+(0.0155069936368)*x_ref[1]**o+(0.223214885433)*x_ref[1]
        ref[3,1,2,0]=(-0.674811720223)*x_ref[0]**o+(0.651640807063)*x_ref[0]+(-0.652022625703)*x_ref[1]**o+(-0.119110788344)*x_ref[1]
        ref[3,1,2,1]=(-0.544748794193)*x_ref[0]**o+(-0.689440165483)*x_ref[0]+(0.936900319026)*x_ref[1]**o+(-0.337287311564)*x_ref[1]
        ref[3,2,0,0]=(-0.355059804554)*x_ref[0]**o+(0.458306329535)*x_ref[0]+(0.172261907494)*x_ref[1]**o+(0.498548138594)*x_ref[1]
        ref[3,2,0,1]=(0.624420187129)*x_ref[0]**o+(0.808931844722)*x_ref[0]+(-0.690635894194)*x_ref[1]**o+(-0.156122444554)*x_ref[1]
        ref[3,2,1,0]=(-0.190029529776)*x_ref[0]**o+(0.925600418137)*x_ref[0]+(0.601040324953)*x_ref[1]**o+(0.615485749173)*x_ref[1]
        ref[3,2,1,1]=(0.679796674662)*x_ref[0]**o+(0.879285148869)*x_ref[0]+(0.0476373514288)*x_ref[1]**o+(0.37655760501)*x_ref[1]
        ref[3,2,2,0]=(0.231748550393)*x_ref[0]**o+(0.209858138221)*x_ref[0]+(-0.88372579866)*x_ref[1]**o+(0.30534744967)*x_ref[1]
        ref[3,2,2,1]=(-0.772177697671)*x_ref[0]**o+(0.012439044435)*x_ref[0]+(-0.293218976284)*x_ref[1]**o+(0.306955066595)*x_ref[1]
        ref[3,3,0,0]=(0.387256929042)*x_ref[0]**o+(-0.434015953267)*x_ref[0]+(-0.098736946868)*x_ref[1]**o+(-0.0837712534186)*x_ref[1]
        ref[3,3,0,1]=(0.396246252967)*x_ref[0]**o+(-0.338955181605)*x_ref[0]+(-0.353504371554)*x_ref[1]**o+(-0.969811402293)*x_ref[1]
        ref[3,3,1,0]=(-0.336508964723)*x_ref[0]**o+(-0.411882233342)*x_ref[0]+(-0.603965321132)*x_ref[1]**o+(0.177221705416)*x_ref[1]
        ref[3,3,1,1]=(0.786971174387)*x_ref[0]**o+(0.759003522166)*x_ref[0]+(-0.0961289050409)*x_ref[1]**o+(-0.0811258682982)*x_ref[1]
        ref[3,3,2,0]=(0.526266925685)*x_ref[0]**o+(-0.113145219611)*x_ref[0]+(-0.208427816884)*x_ref[1]**o+(0.291484710079)*x_ref[1]
        ref[3,3,2,1]=(-0.591153907468)*x_ref[0]**o+(-0.342892047641)*x_ref[0]+(0.865954283634)*x_ref[1]**o+(-0.22112030903)*x_ref[1]
        ref[3,4,0,0]=(0.755162128186)*x_ref[0]**o+(0.494457469003)*x_ref[0]+(0.889254517137)*x_ref[1]**o+(0.880654056969)*x_ref[1]
        ref[3,4,0,1]=(0.450983401629)*x_ref[0]**o+(-0.0855489110466)*x_ref[0]+(0.0601625202676)*x_ref[1]**o+(-0.0730383237708)*x_ref[1]
        ref[3,4,1,0]=(0.495514588456)*x_ref[0]**o+(-0.0895269876334)*x_ref[0]+(-0.89729630673)*x_ref[1]**o+(0.759465768614)*x_ref[1]
        ref[3,4,1,1]=(-0.666472982468)*x_ref[0]**o+(-0.251715028171)*x_ref[0]+(-0.0623287375223)*x_ref[1]**o+(-0.895663326995)*x_ref[1]
        ref[3,4,2,0]=(0.489737923112)*x_ref[0]**o+(0.868066981889)*x_ref[0]+(0.0371747456704)*x_ref[1]**o+(-0.528951636028)*x_ref[1]
        ref[3,4,2,1]=(0.556312598455)*x_ref[0]**o+(-0.0979773350236)*x_ref[0]+(-0.82785928691)*x_ref[1]**o+(-0.567596765858)*x_ref[1]
      else:
        arg[0,0,0,0]=(0.952750275738)*x[0]**o+(0.971489204087)*x[0]+(-0.320706487584)*x[1]**o+(-0.853260471578)*x[1]+(-0.696434862068)*x[2]**o+(0.641824397433)*x[2]
        arg[0,0,0,1]=(0.115484192786)*x[0]**o+(0.147813687309)*x[0]+(-0.998010548186)*x[1]**o+(0.0274712088882)*x[1]+(0.136937971489)*x[2]**o+(-0.657986135444)*x[2]
        arg[0,0,1,0]=(0.805857504779)*x[0]**o+(0.196278916938)*x[0]+(-0.969340785263)*x[1]**o+(-0.543890174888)*x[1]+(0.49118423357)*x[2]**o+(-0.883493800593)*x[2]
        arg[0,0,1,1]=(-0.808802136474)*x[0]**o+(-0.866039395649)*x[0]+(0.0339207025941)*x[1]**o+(-0.441368773534)*x[1]+(0.673688211653)*x[2]**o+(0.917894389398)*x[2]
        arg[0,0,2,0]=(0.0459902210078)*x[0]**o+(0.862332666528)*x[0]+(-0.547792306389)*x[1]**o+(0.722176431732)*x[1]+(-0.511278989032)*x[2]**o+(-0.692775336577)*x[2]
        arg[0,0,2,1]=(-0.641904406513)*x[0]**o+(-0.854450656063)*x[0]+(0.81138894849)*x[1]**o+(0.328886625074)*x[1]+(0.400935831363)*x[2]**o+(-0.25184059766)*x[2]
        arg[0,1,0,0]=(0.69922840942)*x[0]**o+(-0.798085508443)*x[0]+(-0.85191383431)*x[1]**o+(-0.162940829331)*x[1]+(0.504020642931)*x[2]**o+(0.0950957436173)*x[2]
        arg[0,1,0,1]=(0.28901429037)*x[0]**o+(-0.624508745729)*x[0]+(0.921288077478)*x[1]**o+(-0.769207901119)*x[1]+(0.0852374599763)*x[2]**o+(-0.402583033986)*x[2]
        arg[0,1,1,0]=(0.715817282215)*x[0]**o+(0.68735256665)*x[0]+(0.610212058649)*x[1]**o+(0.53560824609)*x[1]+(-0.238908897264)*x[2]**o+(0.619799158474)*x[2]
        arg[0,1,1,1]=(-0.371797414191)*x[0]**o+(0.106263350514)*x[0]+(0.583768355362)*x[1]**o+(0.818797667369)*x[1]+(0.0567571152493)*x[2]**o+(-0.242244867169)*x[2]
        arg[0,1,2,0]=(0.190480659729)*x[0]**o+(-0.632415604771)*x[0]+(0.252513826138)*x[1]**o+(0.874732840469)*x[1]+(-0.367769326078)*x[2]**o+(0.364453292404)*x[2]
        arg[0,1,2,1]=(-0.704507136581)*x[0]**o+(-0.614211737547)*x[0]+(0.533383507313)*x[1]**o+(-0.940007583478)*x[1]+(0.558260002746)*x[2]**o+(0.742565692751)*x[2]
        arg[0,2,0,0]=(0.132581156766)*x[0]**o+(0.671793183183)*x[0]+(-0.308255110769)*x[1]**o+(-0.956002957061)*x[1]+(0.300091541301)*x[2]**o+(-0.714260746791)*x[2]
        arg[0,2,0,1]=(-0.836648080859)*x[0]**o+(0.123813861017)*x[0]+(0.866569934894)*x[1]**o+(-0.957824572351)*x[1]+(-0.904478410952)*x[2]**o+(0.691673758521)*x[2]
        arg[0,2,1,0]=(0.411624721878)*x[0]**o+(0.594054722352)*x[0]+(0.150177701135)*x[1]**o+(0.945855261084)*x[1]+(0.295930158817)*x[2]**o+(-0.607980954424)*x[2]
        arg[0,2,1,1]=(-0.0777741308373)*x[0]**o+(0.697501295655)*x[0]+(0.981797797329)*x[1]**o+(-0.671827077949)*x[1]+(0.292310146229)*x[2]**o+(0.484652357857)*x[2]
        arg[0,2,2,0]=(0.194910469807)*x[0]**o+(-0.70008515623)*x[0]+(0.192808336639)*x[1]**o+(-0.962764477371)*x[1]+(-0.34292240753)*x[2]**o+(0.525470829202)*x[2]
        arg[0,2,2,1]=(0.850368420228)*x[0]**o+(-0.516355386368)*x[0]+(0.395764367873)*x[1]**o+(0.320830752607)*x[1]+(-0.26288679044)*x[2]**o+(0.174457723191)*x[2]
        arg[0,3,0,0]=(-0.713987556845)*x[0]**o+(0.0391066248253)*x[0]+(-0.63911733479)*x[1]**o+(-0.119953032121)*x[1]+(0.697879836447)*x[2]**o+(-0.474957057702)*x[2]
        arg[0,3,0,1]=(-0.128614897423)*x[0]**o+(0.7166036403)*x[0]+(0.678415978447)*x[1]**o+(-0.191453169618)*x[1]+(-0.755261361956)*x[2]**o+(-0.839956004196)*x[2]
        arg[0,3,1,0]=(0.258158740934)*x[0]**o+(-0.412199346967)*x[0]+(-0.781106341272)*x[1]**o+(-0.561162823101)*x[1]+(-0.59549864377)*x[2]**o+(-0.748038415721)*x[2]
        arg[0,3,1,1]=(0.68950222305)*x[0]**o+(-0.904027774445)*x[0]+(0.769744453441)*x[1]**o+(-0.265549152752)*x[1]+(-0.375978837131)*x[2]**o+(-0.387010981761)*x[2]
        arg[0,3,2,0]=(0.46852681319)*x[0]**o+(-0.887650258615)*x[0]+(-0.780275624319)*x[1]**o+(0.527897372599)*x[1]+(0.889281778411)*x[2]**o+(0.82538563877)*x[2]
        arg[0,3,2,1]=(-0.817514448184)*x[0]**o+(-0.230190768519)*x[0]+(-0.101550849344)*x[1]**o+(0.1654885857)*x[1]+(-0.231317749498)*x[2]**o+(0.580153306395)*x[2]
        arg[0,4,0,0]=(0.394993007242)*x[0]**o+(0.305948764395)*x[0]+(-0.66853441368)*x[1]**o+(0.279027498094)*x[1]+(0.161366934709)*x[2]**o+(-0.742002471552)*x[2]
        arg[0,4,0,1]=(-0.661214709471)*x[0]**o+(0.548880317935)*x[0]+(0.472682748503)*x[1]**o+(0.266797505699)*x[1]+(0.405618413683)*x[2]**o+(-0.0539677542667)*x[2]
        arg[0,4,1,0]=(-0.713892424429)*x[0]**o+(0.568075831592)*x[0]+(-0.956180895599)*x[1]**o+(0.63618957695)*x[1]+(0.983696160047)*x[2]**o+(-0.308157929002)*x[2]
        arg[0,4,1,1]=(-0.666582919353)*x[0]**o+(0.0816884254484)*x[0]+(0.190810048896)*x[1]**o+(0.482322905132)*x[1]+(-0.576597070954)*x[2]**o+(-0.722381611553)*x[2]
        arg[0,4,2,0]=(-0.0229650731041)*x[0]**o+(-0.0867816754979)*x[0]+(0.0905042102365)*x[1]**o+(-0.994963424765)*x[1]+(0.74504757815)*x[2]**o+(0.71462549644)*x[2]
        arg[0,4,2,1]=(-0.0464278496147)*x[0]**o+(0.122501026996)*x[0]+(-0.049450141901)*x[1]**o+(-0.388739464086)*x[1]+(-0.151389885067)*x[2]**o+(-0.0316948088803)*x[2]
        arg[1,0,0,0]=(-0.303653386851)*x[0]**o+(-0.435866753659)*x[0]+(-0.419078175082)*x[1]**o+(-0.907595244917)*x[1]+(0.442156528837)*x[2]**o+(-0.76481785523)*x[2]
        arg[1,0,0,1]=(0.879454551618)*x[0]**o+(-0.530789858588)*x[0]+(0.942203776542)*x[1]**o+(0.864362935436)*x[1]+(-0.252832164588)*x[2]**o+(0.00758252581015)*x[2]
        arg[1,0,1,0]=(0.522433142898)*x[0]**o+(0.564002257569)*x[0]+(0.149811689377)*x[1]**o+(-0.757726948165)*x[1]+(-0.767052869719)*x[2]**o+(0.743541806001)*x[2]
        arg[1,0,1,1]=(0.535612198186)*x[0]**o+(0.0874480088401)*x[0]+(0.913151292296)*x[1]**o+(-0.495024655308)*x[1]+(-0.42308905182)*x[2]**o+(-0.983623551153)*x[2]
        arg[1,0,2,0]=(0.587221456352)*x[0]**o+(0.0384438535009)*x[0]+(0.301554087927)*x[1]**o+(-0.944074061806)*x[1]+(0.808195432991)*x[2]**o+(-0.46545743768)*x[2]
        arg[1,0,2,1]=(0.136251169669)*x[0]**o+(0.772190271875)*x[0]+(-0.490981224022)*x[1]**o+(-0.191915588448)*x[1]+(0.792617864405)*x[2]**o+(0.139425359773)*x[2]
        arg[1,1,0,0]=(0.0440493146329)*x[0]**o+(-0.948951787109)*x[0]+(-0.564104026105)*x[1]**o+(0.177457883377)*x[1]+(0.578169345377)*x[2]**o+(0.35574407049)*x[2]
        arg[1,1,0,1]=(-0.740305651819)*x[0]**o+(-0.340976006932)*x[0]+(-0.657257828033)*x[1]**o+(-0.456240473558)*x[1]+(0.526236681039)*x[2]**o+(0.804762801115)*x[2]
        arg[1,1,1,0]=(-0.753480536304)*x[0]**o+(0.424080516297)*x[0]+(-0.97529611006)*x[1]**o+(-0.511450162624)*x[1]+(-0.248982226385)*x[2]**o+(0.497803766242)*x[2]
        arg[1,1,1,1]=(-0.273280180629)*x[0]**o+(0.798945176097)*x[0]+(0.552152546131)*x[1]**o+(0.737594404744)*x[1]+(0.0339269287984)*x[2]**o+(-0.773791176515)*x[2]
        arg[1,1,2,0]=(0.553404075378)*x[0]**o+(-0.630969888585)*x[0]+(0.455072287913)*x[1]**o+(0.145928503781)*x[1]+(0.632480470504)*x[2]**o+(0.363815684296)*x[2]
        arg[1,1,2,1]=(-0.426756293656)*x[0]**o+(-0.449867410734)*x[0]+(-0.875891527959)*x[1]**o+(-0.125243159763)*x[1]+(0.887344501669)*x[2]**o+(-0.0569005112264)*x[2]
        arg[1,2,0,0]=(0.762313320036)*x[0]**o+(-0.514266376348)*x[0]+(-0.915248032301)*x[1]**o+(0.50418081892)*x[1]+(-0.867662577318)*x[2]**o+(0.731635970592)*x[2]
        arg[1,2,0,1]=(-0.0866106252969)*x[0]**o+(0.063250309716)*x[0]+(0.598723588976)*x[1]**o+(0.317235331686)*x[1]+(-0.412892601258)*x[2]**o+(0.771147670726)*x[2]
        arg[1,2,1,0]=(0.57639148624)*x[0]**o+(0.773916349481)*x[0]+(0.401319543026)*x[1]**o+(-0.510387398986)*x[1]+(-0.796773375436)*x[2]**o+(-0.765544756429)*x[2]
        arg[1,2,1,1]=(-0.686952109389)*x[0]**o+(0.428892331428)*x[0]+(-0.139971762047)*x[1]**o+(-0.747581454837)*x[1]+(0.152196734461)*x[2]**o+(-0.778424993847)*x[2]
        arg[1,2,2,0]=(0.729199771391)*x[0]**o+(0.185870258363)*x[0]+(0.996198489286)*x[1]**o+(0.0343703739018)*x[1]+(-0.6637833208)*x[2]**o+(-0.738510210409)*x[2]
        arg[1,2,2,1]=(0.705773326438)*x[0]**o+(-0.633406525448)*x[0]+(0.252620923822)*x[1]**o+(-0.760785834478)*x[1]+(0.995411456622)*x[2]**o+(-0.365718663741)*x[2]
        arg[1,3,0,0]=(0.826142335301)*x[0]**o+(-0.869335317945)*x[0]+(0.660360312658)*x[1]**o+(0.96157015332)*x[1]+(-0.268246324116)*x[2]**o+(-0.575288823295)*x[2]
        arg[1,3,0,1]=(0.620117943072)*x[0]**o+(0.51267609202)*x[0]+(-0.882468485772)*x[1]**o+(0.771021246475)*x[1]+(0.398548927211)*x[2]**o+(0.275983192446)*x[2]
        arg[1,3,1,0]=(0.716523839796)*x[0]**o+(-0.0868143348547)*x[0]+(-0.456384710114)*x[1]**o+(-0.646489063979)*x[1]+(0.594500214771)*x[2]**o+(-0.299956296467)*x[2]
        arg[1,3,1,1]=(-0.453257208906)*x[0]**o+(0.583260425119)*x[0]+(0.263463648634)*x[1]**o+(0.493693446665)*x[1]+(-0.529884773858)*x[2]**o+(0.220733486264)*x[2]
        arg[1,3,2,0]=(0.669052014654)*x[0]**o+(-0.247902780475)*x[0]+(0.44071786167)*x[1]**o+(-0.21199653848)*x[1]+(-0.0713603448391)*x[2]**o+(-0.66681439981)*x[2]
        arg[1,3,2,1]=(-0.341533813758)*x[0]**o+(0.770581995272)*x[0]+(0.03376913861)*x[1]**o+(0.000270719073091)*x[1]+(-0.774936686899)*x[2]**o+(-0.0196300167016)*x[2]
        arg[1,4,0,0]=(-0.869695667108)*x[0]**o+(0.515676789402)*x[0]+(0.67731822534)*x[1]**o+(0.126887846346)*x[1]+(0.843200966455)*x[2]**o+(-0.59930363301)*x[2]
        arg[1,4,0,1]=(0.909369352728)*x[0]**o+(-0.110828876636)*x[0]+(-0.65561953478)*x[1]**o+(0.598491879954)*x[1]+(-0.557567770163)*x[2]**o+(-0.391108926502)*x[2]
        arg[1,4,1,0]=(0.320646291934)*x[0]**o+(0.164062007697)*x[0]+(-0.546323998218)*x[1]**o+(0.72651711346)*x[1]+(-0.613496738898)*x[2]**o+(0.430761082137)*x[2]
        arg[1,4,1,1]=(-0.721651561555)*x[0]**o+(-0.0647310624509)*x[0]+(-0.360525345485)*x[1]**o+(0.408239277468)*x[1]+(0.600551705094)*x[2]**o+(0.506237401392)*x[2]
        arg[1,4,2,0]=(-0.391702965605)*x[0]**o+(-0.574470582913)*x[0]+(-0.729724609696)*x[1]**o+(0.91381180302)*x[1]+(-0.544328104929)*x[2]**o+(-0.516992534607)*x[2]
        arg[1,4,2,1]=(-0.0409666962711)*x[0]**o+(-0.524913112823)*x[0]+(-0.677030607856)*x[1]**o+(-0.530859496807)*x[1]+(0.137890059737)*x[2]**o+(0.0669856870322)*x[2]
        arg[2,0,0,0]=(0.10369318023)*x[0]**o+(0.345691611083)*x[0]+(-0.222795950561)*x[1]**o+(-0.797690270935)*x[1]+(-0.703457203234)*x[2]**o+(-0.751193209871)*x[2]
        arg[2,0,0,1]=(0.166457278787)*x[0]**o+(-0.0304766240065)*x[0]+(0.159987374581)*x[1]**o+(0.828012214748)*x[1]+(0.0670696566332)*x[2]**o+(-0.498252024306)*x[2]
        arg[2,0,1,0]=(0.901095992383)*x[0]**o+(0.400296735981)*x[0]+(0.394687295672)*x[1]**o+(-0.986496611224)*x[1]+(-0.768945602918)*x[2]**o+(-0.782762026735)*x[2]
        arg[2,0,1,1]=(0.47969549703)*x[0]**o+(0.981251232699)*x[0]+(-0.69807700503)*x[1]**o+(-0.813549271424)*x[1]+(-0.135970806025)*x[2]**o+(0.288738861745)*x[2]
        arg[2,0,2,0]=(0.315262113284)*x[0]**o+(0.0380265323361)*x[0]+(0.259894600802)*x[1]**o+(0.538712801332)*x[1]+(-0.385783597257)*x[2]**o+(0.666670257561)*x[2]
        arg[2,0,2,1]=(-0.756715539091)*x[0]**o+(-0.823809574081)*x[0]+(-0.283842954238)*x[1]**o+(-0.640166311159)*x[1]+(-0.39947011053)*x[2]**o+(-0.393543798757)*x[2]
        arg[2,1,0,0]=(-0.292033523499)*x[0]**o+(-0.940246412652)*x[0]+(-0.327571437608)*x[1]**o+(0.837077706655)*x[1]+(0.446471499818)*x[2]**o+(-0.747249149016)*x[2]
        arg[2,1,0,1]=(-0.867504469067)*x[0]**o+(0.635684140727)*x[0]+(0.488043128661)*x[1]**o+(-0.163389002402)*x[1]+(0.0440527250714)*x[2]**o+(0.605766722432)*x[2]
        arg[2,1,1,0]=(-0.0622304802635)*x[0]**o+(0.607299318698)*x[0]+(0.974933906045)*x[1]**o+(0.0632442428777)*x[1]+(0.0672808299851)*x[2]**o+(-0.555700803374)*x[2]
        arg[2,1,1,1]=(-0.350438453721)*x[0]**o+(-0.826289294567)*x[0]+(-0.473764081208)*x[1]**o+(-0.247804269494)*x[1]+(0.834301955859)*x[2]**o+(0.443058363661)*x[2]
        arg[2,1,2,0]=(0.5046026606)*x[0]**o+(0.305502438052)*x[0]+(-0.201816724695)*x[1]**o+(-0.101544576856)*x[1]+(0.729817329051)*x[2]**o+(-0.851479834334)*x[2]
        arg[2,1,2,1]=(0.917039424926)*x[0]**o+(0.260565966637)*x[0]+(-0.595846674057)*x[1]**o+(0.654773238108)*x[1]+(0.544664592565)*x[2]**o+(0.363343885358)*x[2]
        arg[2,2,0,0]=(0.243879200917)*x[0]**o+(0.335612613868)*x[0]+(-0.287066716076)*x[1]**o+(0.506206687739)*x[1]+(-0.902513036448)*x[2]**o+(-0.427017746025)*x[2]
        arg[2,2,0,1]=(0.965102871162)*x[0]**o+(0.161476912964)*x[0]+(0.97007160306)*x[1]**o+(0.870104939402)*x[1]+(0.552404991536)*x[2]**o+(-0.319082395552)*x[2]
        arg[2,2,1,0]=(0.17611459545)*x[0]**o+(-0.457419791602)*x[0]+(-0.00217752259743)*x[1]**o+(-0.632177103931)*x[1]+(0.521123797224)*x[2]**o+(0.427935307605)*x[2]
        arg[2,2,1,1]=(-0.993298850316)*x[0]**o+(0.277763727112)*x[0]+(0.849464768611)*x[1]**o+(0.890919639607)*x[1]+(0.01030894969)*x[2]**o+(-0.76051772768)*x[2]
        arg[2,2,2,0]=(-0.622887788117)*x[0]**o+(0.40187802561)*x[0]+(0.836685367934)*x[1]**o+(0.700379534201)*x[1]+(0.683627950354)*x[2]**o+(0.445351804296)*x[2]
        arg[2,2,2,1]=(0.124965536288)*x[0]**o+(-0.198462168996)*x[0]+(-0.657084126986)*x[1]**o+(-0.420469996385)*x[1]+(0.237164091597)*x[2]**o+(0.161926160295)*x[2]
        arg[2,3,0,0]=(-0.622682941897)*x[0]**o+(-0.6279729398)*x[0]+(0.189086870792)*x[1]**o+(-0.350822257394)*x[1]+(-0.505481371359)*x[2]**o+(0.812944216714)*x[2]
        arg[2,3,0,1]=(-0.246704719057)*x[0]**o+(0.513716384141)*x[0]+(-0.264914192193)*x[1]**o+(-0.32984984663)*x[1]+(-0.80409813849)*x[2]**o+(-0.825568736974)*x[2]
        arg[2,3,1,0]=(-0.534782640399)*x[0]**o+(-0.917823259622)*x[0]+(0.691827074405)*x[1]**o+(-0.818629872154)*x[1]+(0.870046992385)*x[2]**o+(0.00423218787803)*x[2]
        arg[2,3,1,1]=(0.633068676613)*x[0]**o+(-0.895613811402)*x[0]+(0.62242069294)*x[1]**o+(-0.272136050955)*x[1]+(-0.199008776357)*x[2]**o+(0.836119124508)*x[2]
        arg[2,3,2,0]=(0.335884106464)*x[0]**o+(-0.857917873801)*x[0]+(-0.42653278071)*x[1]**o+(0.375200781448)*x[1]+(0.378427453722)*x[2]**o+(0.398603623445)*x[2]
        arg[2,3,2,1]=(0.0602499604703)*x[0]**o+(-0.921459358089)*x[0]+(0.558180675902)*x[1]**o+(0.0863734870743)*x[1]+(0.215167095483)*x[2]**o+(-0.37908830738)*x[2]
        arg[2,4,0,0]=(0.866321987007)*x[0]**o+(0.81950553744)*x[0]+(0.0422552872889)*x[1]**o+(0.369328458453)*x[1]+(0.0547909452885)*x[2]**o+(-0.396367133883)*x[2]
        arg[2,4,0,1]=(0.438824337571)*x[0]**o+(0.211435787811)*x[0]+(-0.801273340861)*x[1]**o+(-0.181654271172)*x[1]+(0.374171148243)*x[2]**o+(0.945633108881)*x[2]
        arg[2,4,1,0]=(0.588044951473)*x[0]**o+(0.401384824924)*x[0]+(-0.191996429616)*x[1]**o+(0.0472579697166)*x[1]+(0.470962958517)*x[2]**o+(0.52718875619)*x[2]
        arg[2,4,1,1]=(0.371613549998)*x[0]**o+(-0.414998926249)*x[0]+(-0.762013182749)*x[1]**o+(0.796144230249)*x[1]+(-0.555041053148)*x[2]**o+(-0.595298275916)*x[2]
        arg[2,4,2,0]=(-0.992527136862)*x[0]**o+(-0.995041267483)*x[0]+(-0.730305990844)*x[1]**o+(-0.52470798566)*x[1]+(-0.938945944067)*x[2]**o+(0.33396135548)*x[2]
        arg[2,4,2,1]=(0.0333921786723)*x[0]**o+(0.707990374515)*x[0]+(-0.106667413755)*x[1]**o+(0.282989655082)*x[1]+(-0.152535982788)*x[2]**o+(-0.75130096727)*x[2]
        arg[3,0,0,0]=(-0.822675771493)*x[0]**o+(0.726468488157)*x[0]+(0.972567560814)*x[1]**o+(0.308266449093)*x[1]+(0.71069864654)*x[2]**o+(-0.683195491271)*x[2]
        arg[3,0,0,1]=(-0.505155072199)*x[0]**o+(0.339013729552)*x[0]+(-0.81565931012)*x[1]**o+(0.362458680494)*x[1]+(0.960221320064)*x[2]**o+(-0.17234580875)*x[2]
        arg[3,0,1,0]=(0.67972657926)*x[0]**o+(0.143085543693)*x[0]+(-0.699516563264)*x[1]**o+(-0.892226133751)*x[1]+(-0.898541541049)*x[2]**o+(-0.736249681714)*x[2]
        arg[3,0,1,1]=(-0.144423319215)*x[0]**o+(-0.509713525676)*x[0]+(0.347584762861)*x[1]**o+(0.666169972794)*x[1]+(0.691734080163)*x[2]**o+(0.9338062148)*x[2]
        arg[3,0,2,0]=(-0.47323843589)*x[0]**o+(0.0819655059363)*x[0]+(0.0863030211133)*x[1]**o+(0.127803089307)*x[1]+(-0.155852489756)*x[2]**o+(-0.00470398386987)*x[2]
        arg[3,0,2,1]=(0.372452726232)*x[0]**o+(0.133808609857)*x[0]+(0.962945642209)*x[1]**o+(0.0446609213929)*x[1]+(-0.446207951293)*x[2]**o+(0.0400733439868)*x[2]
        arg[3,1,0,0]=(-0.668575734235)*x[0]**o+(0.905913572459)*x[0]+(-0.321327880802)*x[1]**o+(0.0263209456188)*x[1]+(-0.0355325780368)*x[2]**o+(-0.624130395105)*x[2]
        arg[3,1,0,1]=(0.859810828483)*x[0]**o+(-0.235372125226)*x[0]+(-0.16715244612)*x[1]**o+(-0.793399872713)*x[1]+(0.303973795556)*x[2]**o+(0.176740655638)*x[2]
        arg[3,1,1,0]=(0.736448874258)*x[0]**o+(-0.448956585466)*x[0]+(0.782095902992)*x[1]**o+(0.796524196866)*x[1]+(-0.732812289541)*x[2]**o+(0.687089667291)*x[2]
        arg[3,1,1,1]=(-0.797708068194)*x[0]**o+(-0.691308116296)*x[0]+(-0.644065410151)*x[1]**o+(-0.929624672017)*x[1]+(0.265040940279)*x[2]**o+(0.424929324889)*x[2]
        arg[3,1,2,0]=(0.555032482777)*x[0]**o+(-0.564617314931)*x[0]+(-0.612721853869)*x[1]**o+(-0.048466699593)*x[1]+(0.753265037897)*x[2]**o+(0.913086239874)*x[2]
        arg[3,1,2,1]=(0.580881807771)*x[0]**o+(0.970973128604)*x[0]+(-0.603610563753)*x[1]**o+(0.979985814438)*x[1]+(-0.639531987694)*x[2]**o+(-0.773094138073)*x[2]
        arg[3,2,0,0]=(0.574306840544)*x[0]**o+(0.449094769988)*x[0]+(-0.406303076614)*x[1]**o+(-0.776648781906)*x[1]+(-0.59829476647)*x[2]**o+(0.257632446666)*x[2]
        arg[3,2,0,1]=(0.689106127051)*x[0]**o+(0.717184851708)*x[0]+(-0.495581943615)*x[1]**o+(-0.30329139788)*x[1]+(0.901753128535)*x[2]**o+(0.48496505727)*x[2]
        arg[3,2,1,0]=(0.252050902256)*x[0]**o+(-0.0414281706943)*x[0]+(-0.284125627654)*x[1]**o+(0.477450554269)*x[1]+(0.849202242148)*x[2]**o+(0.211826565889)*x[2]
        arg[3,2,1,1]=(-0.319900673571)*x[0]**o+(0.965993723739)*x[0]+(0.671360092314)*x[1]**o+(-0.652805407934)*x[1]+(-0.0352316646853)*x[2]**o+(-0.442493499306)*x[2]
        arg[3,2,2,0]=(0.525577660439)*x[0]**o+(0.660882378915)*x[0]+(-0.419826738105)*x[1]**o+(-0.798251889077)*x[1]+(-0.430382393461)*x[2]**o+(0.714357242496)*x[2]
        arg[3,2,2,1]=(-0.200634944034)*x[0]**o+(0.211086366034)*x[0]+(0.116289218188)*x[1]**o+(-0.785792860501)*x[1]+(0.763320327438)*x[2]**o+(-0.457355826175)*x[2]
        arg[3,3,0,0]=(-0.890873783782)*x[0]**o+(0.322610697657)*x[0]+(-0.351437390332)*x[1]**o+(-0.720698169708)*x[1]+(-0.664444440194)*x[2]**o+(0.274865394829)*x[2]
        arg[3,3,0,1]=(-0.384885652342)*x[0]**o+(-0.409799972422)*x[0]+(0.679315041587)*x[1]**o+(0.0859788488759)*x[1]+(-0.136173937432)*x[2]**o+(-0.136591429212)*x[2]
        arg[3,3,1,0]=(0.147934741321)*x[0]**o+(0.246941956666)*x[0]+(0.696831123708)*x[1]**o+(-0.905966322958)*x[1]+(0.672705695255)*x[2]**o+(-0.776017262395)*x[2]
        arg[3,3,1,1]=(0.933982654679)*x[0]**o+(-0.367246348547)*x[0]+(-0.254076834401)*x[1]**o+(0.148617981234)*x[1]+(-0.486881004358)*x[2]**o+(0.636371570082)*x[2]
        arg[3,3,2,0]=(-0.536125392071)*x[0]**o+(-0.663380779479)*x[0]+(0.0655694689011)*x[1]**o+(0.0390002326894)*x[1]+(-0.731730129742)*x[2]**o+(-0.578791485067)*x[2]
        arg[3,3,2,1]=(0.705873902409)*x[0]**o+(-0.0444303506955)*x[0]+(-0.978519191225)*x[1]**o+(-0.0280878186958)*x[1]+(0.182378726121)*x[2]**o+(-0.555345919696)*x[2]
        arg[3,4,0,0]=(0.945945862395)*x[0]**o+(-0.448487950632)*x[0]+(-0.477570573229)*x[1]**o+(0.313871838624)*x[1]+(0.237944220287)*x[2]**o+(-0.218095130482)*x[2]
        arg[3,4,0,1]=(-0.758647277269)*x[0]**o+(-0.550604667208)*x[0]+(-0.262944399053)*x[1]**o+(0.994302652859)*x[1]+(0.363440310008)*x[2]**o+(-0.127863970613)*x[2]
        arg[3,4,1,0]=(0.667202433965)*x[0]**o+(0.954901068821)*x[0]+(0.719745510318)*x[1]**o+(-0.763882742007)*x[1]+(0.880927476123)*x[2]**o+(0.102034630893)*x[2]
        arg[3,4,1,1]=(-0.684644024456)*x[0]**o+(0.88037275093)*x[0]+(-0.772213766289)*x[1]**o+(-0.702615214094)*x[1]+(-0.25661708053)*x[2]**o+(0.126779040694)*x[2]
        arg[3,4,2,0]=(-0.589611385303)*x[0]**o+(0.638712635252)*x[0]+(0.745450741856)*x[1]**o+(0.399494092815)*x[1]+(-0.717293809038)*x[2]**o+(0.305662737751)*x[2]
        arg[3,4,2,1]=(0.0572426997796)*x[0]**o+(-0.764600144578)*x[0]+(0.482545219565)*x[1]**o+(0.525883069411)*x[1]+(-0.612224327195)*x[2]**o+(0.354344820256)*x[2]
        ref[0,0,0,0]=(0.952750275738)*x_ref[0]**o+(0.971489204087)*x_ref[0]+(-0.320706487584)*x_ref[1]**o+(-0.853260471578)*x_ref[1]+(-0.696434862068)*x_ref[2]**o+(0.641824397433)*x_ref[2]
        ref[0,0,0,1]=(0.115484192786)*x_ref[0]**o+(0.147813687309)*x_ref[0]+(-0.998010548186)*x_ref[1]**o+(0.0274712088882)*x_ref[1]+(0.136937971489)*x_ref[2]**o+(-0.657986135444)*x_ref[2]
        ref[0,0,1,0]=(0.805857504779)*x_ref[0]**o+(0.196278916938)*x_ref[0]+(-0.969340785263)*x_ref[1]**o+(-0.543890174888)*x_ref[1]+(0.49118423357)*x_ref[2]**o+(-0.883493800593)*x_ref[2]
        ref[0,0,1,1]=(-0.808802136474)*x_ref[0]**o+(-0.866039395649)*x_ref[0]+(0.0339207025941)*x_ref[1]**o+(-0.441368773534)*x_ref[1]+(0.673688211653)*x_ref[2]**o+(0.917894389398)*x_ref[2]
        ref[0,0,2,0]=(0.0459902210078)*x_ref[0]**o+(0.862332666528)*x_ref[0]+(-0.547792306389)*x_ref[1]**o+(0.722176431732)*x_ref[1]+(-0.511278989032)*x_ref[2]**o+(-0.692775336577)*x_ref[2]
        ref[0,0,2,1]=(-0.641904406513)*x_ref[0]**o+(-0.854450656063)*x_ref[0]+(0.81138894849)*x_ref[1]**o+(0.328886625074)*x_ref[1]+(0.400935831363)*x_ref[2]**o+(-0.25184059766)*x_ref[2]
        ref[0,1,0,0]=(0.69922840942)*x_ref[0]**o+(-0.798085508443)*x_ref[0]+(-0.85191383431)*x_ref[1]**o+(-0.162940829331)*x_ref[1]+(0.504020642931)*x_ref[2]**o+(0.0950957436173)*x_ref[2]
        ref[0,1,0,1]=(0.28901429037)*x_ref[0]**o+(-0.624508745729)*x_ref[0]+(0.921288077478)*x_ref[1]**o+(-0.769207901119)*x_ref[1]+(0.0852374599763)*x_ref[2]**o+(-0.402583033986)*x_ref[2]
        ref[0,1,1,0]=(0.715817282215)*x_ref[0]**o+(0.68735256665)*x_ref[0]+(0.610212058649)*x_ref[1]**o+(0.53560824609)*x_ref[1]+(-0.238908897264)*x_ref[2]**o+(0.619799158474)*x_ref[2]
        ref[0,1,1,1]=(-0.371797414191)*x_ref[0]**o+(0.106263350514)*x_ref[0]+(0.583768355362)*x_ref[1]**o+(0.818797667369)*x_ref[1]+(0.0567571152493)*x_ref[2]**o+(-0.242244867169)*x_ref[2]
        ref[0,1,2,0]=(0.190480659729)*x_ref[0]**o+(-0.632415604771)*x_ref[0]+(0.252513826138)*x_ref[1]**o+(0.874732840469)*x_ref[1]+(-0.367769326078)*x_ref[2]**o+(0.364453292404)*x_ref[2]
        ref[0,1,2,1]=(-0.704507136581)*x_ref[0]**o+(-0.614211737547)*x_ref[0]+(0.533383507313)*x_ref[1]**o+(-0.940007583478)*x_ref[1]+(0.558260002746)*x_ref[2]**o+(0.742565692751)*x_ref[2]
        ref[0,2,0,0]=(0.132581156766)*x_ref[0]**o+(0.671793183183)*x_ref[0]+(-0.308255110769)*x_ref[1]**o+(-0.956002957061)*x_ref[1]+(0.300091541301)*x_ref[2]**o+(-0.714260746791)*x_ref[2]
        ref[0,2,0,1]=(-0.836648080859)*x_ref[0]**o+(0.123813861017)*x_ref[0]+(0.866569934894)*x_ref[1]**o+(-0.957824572351)*x_ref[1]+(-0.904478410952)*x_ref[2]**o+(0.691673758521)*x_ref[2]
        ref[0,2,1,0]=(0.411624721878)*x_ref[0]**o+(0.594054722352)*x_ref[0]+(0.150177701135)*x_ref[1]**o+(0.945855261084)*x_ref[1]+(0.295930158817)*x_ref[2]**o+(-0.607980954424)*x_ref[2]
        ref[0,2,1,1]=(-0.0777741308373)*x_ref[0]**o+(0.697501295655)*x_ref[0]+(0.981797797329)*x_ref[1]**o+(-0.671827077949)*x_ref[1]+(0.292310146229)*x_ref[2]**o+(0.484652357857)*x_ref[2]
        ref[0,2,2,0]=(0.194910469807)*x_ref[0]**o+(-0.70008515623)*x_ref[0]+(0.192808336639)*x_ref[1]**o+(-0.962764477371)*x_ref[1]+(-0.34292240753)*x_ref[2]**o+(0.525470829202)*x_ref[2]
        ref[0,2,2,1]=(0.850368420228)*x_ref[0]**o+(-0.516355386368)*x_ref[0]+(0.395764367873)*x_ref[1]**o+(0.320830752607)*x_ref[1]+(-0.26288679044)*x_ref[2]**o+(0.174457723191)*x_ref[2]
        ref[0,3,0,0]=(-0.713987556845)*x_ref[0]**o+(0.0391066248253)*x_ref[0]+(-0.63911733479)*x_ref[1]**o+(-0.119953032121)*x_ref[1]+(0.697879836447)*x_ref[2]**o+(-0.474957057702)*x_ref[2]
        ref[0,3,0,1]=(-0.128614897423)*x_ref[0]**o+(0.7166036403)*x_ref[0]+(0.678415978447)*x_ref[1]**o+(-0.191453169618)*x_ref[1]+(-0.755261361956)*x_ref[2]**o+(-0.839956004196)*x_ref[2]
        ref[0,3,1,0]=(0.258158740934)*x_ref[0]**o+(-0.412199346967)*x_ref[0]+(-0.781106341272)*x_ref[1]**o+(-0.561162823101)*x_ref[1]+(-0.59549864377)*x_ref[2]**o+(-0.748038415721)*x_ref[2]
        ref[0,3,1,1]=(0.68950222305)*x_ref[0]**o+(-0.904027774445)*x_ref[0]+(0.769744453441)*x_ref[1]**o+(-0.265549152752)*x_ref[1]+(-0.375978837131)*x_ref[2]**o+(-0.387010981761)*x_ref[2]
        ref[0,3,2,0]=(0.46852681319)*x_ref[0]**o+(-0.887650258615)*x_ref[0]+(-0.780275624319)*x_ref[1]**o+(0.527897372599)*x_ref[1]+(0.889281778411)*x_ref[2]**o+(0.82538563877)*x_ref[2]
        ref[0,3,2,1]=(-0.817514448184)*x_ref[0]**o+(-0.230190768519)*x_ref[0]+(-0.101550849344)*x_ref[1]**o+(0.1654885857)*x_ref[1]+(-0.231317749498)*x_ref[2]**o+(0.580153306395)*x_ref[2]
        ref[0,4,0,0]=(0.394993007242)*x_ref[0]**o+(0.305948764395)*x_ref[0]+(-0.66853441368)*x_ref[1]**o+(0.279027498094)*x_ref[1]+(0.161366934709)*x_ref[2]**o+(-0.742002471552)*x_ref[2]
        ref[0,4,0,1]=(-0.661214709471)*x_ref[0]**o+(0.548880317935)*x_ref[0]+(0.472682748503)*x_ref[1]**o+(0.266797505699)*x_ref[1]+(0.405618413683)*x_ref[2]**o+(-0.0539677542667)*x_ref[2]
        ref[0,4,1,0]=(-0.713892424429)*x_ref[0]**o+(0.568075831592)*x_ref[0]+(-0.956180895599)*x_ref[1]**o+(0.63618957695)*x_ref[1]+(0.983696160047)*x_ref[2]**o+(-0.308157929002)*x_ref[2]
        ref[0,4,1,1]=(-0.666582919353)*x_ref[0]**o+(0.0816884254484)*x_ref[0]+(0.190810048896)*x_ref[1]**o+(0.482322905132)*x_ref[1]+(-0.576597070954)*x_ref[2]**o+(-0.722381611553)*x_ref[2]
        ref[0,4,2,0]=(-0.0229650731041)*x_ref[0]**o+(-0.0867816754979)*x_ref[0]+(0.0905042102365)*x_ref[1]**o+(-0.994963424765)*x_ref[1]+(0.74504757815)*x_ref[2]**o+(0.71462549644)*x_ref[2]
        ref[0,4,2,1]=(-0.0464278496147)*x_ref[0]**o+(0.122501026996)*x_ref[0]+(-0.049450141901)*x_ref[1]**o+(-0.388739464086)*x_ref[1]+(-0.151389885067)*x_ref[2]**o+(-0.0316948088803)*x_ref[2]
        ref[1,0,0,0]=(-0.303653386851)*x_ref[0]**o+(-0.435866753659)*x_ref[0]+(-0.419078175082)*x_ref[1]**o+(-0.907595244917)*x_ref[1]+(0.442156528837)*x_ref[2]**o+(-0.76481785523)*x_ref[2]
        ref[1,0,0,1]=(0.879454551618)*x_ref[0]**o+(-0.530789858588)*x_ref[0]+(0.942203776542)*x_ref[1]**o+(0.864362935436)*x_ref[1]+(-0.252832164588)*x_ref[2]**o+(0.00758252581015)*x_ref[2]
        ref[1,0,1,0]=(0.522433142898)*x_ref[0]**o+(0.564002257569)*x_ref[0]+(0.149811689377)*x_ref[1]**o+(-0.757726948165)*x_ref[1]+(-0.767052869719)*x_ref[2]**o+(0.743541806001)*x_ref[2]
        ref[1,0,1,1]=(0.535612198186)*x_ref[0]**o+(0.0874480088401)*x_ref[0]+(0.913151292296)*x_ref[1]**o+(-0.495024655308)*x_ref[1]+(-0.42308905182)*x_ref[2]**o+(-0.983623551153)*x_ref[2]
        ref[1,0,2,0]=(0.587221456352)*x_ref[0]**o+(0.0384438535009)*x_ref[0]+(0.301554087927)*x_ref[1]**o+(-0.944074061806)*x_ref[1]+(0.808195432991)*x_ref[2]**o+(-0.46545743768)*x_ref[2]
        ref[1,0,2,1]=(0.136251169669)*x_ref[0]**o+(0.772190271875)*x_ref[0]+(-0.490981224022)*x_ref[1]**o+(-0.191915588448)*x_ref[1]+(0.792617864405)*x_ref[2]**o+(0.139425359773)*x_ref[2]
        ref[1,1,0,0]=(0.0440493146329)*x_ref[0]**o+(-0.948951787109)*x_ref[0]+(-0.564104026105)*x_ref[1]**o+(0.177457883377)*x_ref[1]+(0.578169345377)*x_ref[2]**o+(0.35574407049)*x_ref[2]
        ref[1,1,0,1]=(-0.740305651819)*x_ref[0]**o+(-0.340976006932)*x_ref[0]+(-0.657257828033)*x_ref[1]**o+(-0.456240473558)*x_ref[1]+(0.526236681039)*x_ref[2]**o+(0.804762801115)*x_ref[2]
        ref[1,1,1,0]=(-0.753480536304)*x_ref[0]**o+(0.424080516297)*x_ref[0]+(-0.97529611006)*x_ref[1]**o+(-0.511450162624)*x_ref[1]+(-0.248982226385)*x_ref[2]**o+(0.497803766242)*x_ref[2]
        ref[1,1,1,1]=(-0.273280180629)*x_ref[0]**o+(0.798945176097)*x_ref[0]+(0.552152546131)*x_ref[1]**o+(0.737594404744)*x_ref[1]+(0.0339269287984)*x_ref[2]**o+(-0.773791176515)*x_ref[2]
        ref[1,1,2,0]=(0.553404075378)*x_ref[0]**o+(-0.630969888585)*x_ref[0]+(0.455072287913)*x_ref[1]**o+(0.145928503781)*x_ref[1]+(0.632480470504)*x_ref[2]**o+(0.363815684296)*x_ref[2]
        ref[1,1,2,1]=(-0.426756293656)*x_ref[0]**o+(-0.449867410734)*x_ref[0]+(-0.875891527959)*x_ref[1]**o+(-0.125243159763)*x_ref[1]+(0.887344501669)*x_ref[2]**o+(-0.0569005112264)*x_ref[2]
        ref[1,2,0,0]=(0.762313320036)*x_ref[0]**o+(-0.514266376348)*x_ref[0]+(-0.915248032301)*x_ref[1]**o+(0.50418081892)*x_ref[1]+(-0.867662577318)*x_ref[2]**o+(0.731635970592)*x_ref[2]
        ref[1,2,0,1]=(-0.0866106252969)*x_ref[0]**o+(0.063250309716)*x_ref[0]+(0.598723588976)*x_ref[1]**o+(0.317235331686)*x_ref[1]+(-0.412892601258)*x_ref[2]**o+(0.771147670726)*x_ref[2]
        ref[1,2,1,0]=(0.57639148624)*x_ref[0]**o+(0.773916349481)*x_ref[0]+(0.401319543026)*x_ref[1]**o+(-0.510387398986)*x_ref[1]+(-0.796773375436)*x_ref[2]**o+(-0.765544756429)*x_ref[2]
        ref[1,2,1,1]=(-0.686952109389)*x_ref[0]**o+(0.428892331428)*x_ref[0]+(-0.139971762047)*x_ref[1]**o+(-0.747581454837)*x_ref[1]+(0.152196734461)*x_ref[2]**o+(-0.778424993847)*x_ref[2]
        ref[1,2,2,0]=(0.729199771391)*x_ref[0]**o+(0.185870258363)*x_ref[0]+(0.996198489286)*x_ref[1]**o+(0.0343703739018)*x_ref[1]+(-0.6637833208)*x_ref[2]**o+(-0.738510210409)*x_ref[2]
        ref[1,2,2,1]=(0.705773326438)*x_ref[0]**o+(-0.633406525448)*x_ref[0]+(0.252620923822)*x_ref[1]**o+(-0.760785834478)*x_ref[1]+(0.995411456622)*x_ref[2]**o+(-0.365718663741)*x_ref[2]
        ref[1,3,0,0]=(0.826142335301)*x_ref[0]**o+(-0.869335317945)*x_ref[0]+(0.660360312658)*x_ref[1]**o+(0.96157015332)*x_ref[1]+(-0.268246324116)*x_ref[2]**o+(-0.575288823295)*x_ref[2]
        ref[1,3,0,1]=(0.620117943072)*x_ref[0]**o+(0.51267609202)*x_ref[0]+(-0.882468485772)*x_ref[1]**o+(0.771021246475)*x_ref[1]+(0.398548927211)*x_ref[2]**o+(0.275983192446)*x_ref[2]
        ref[1,3,1,0]=(0.716523839796)*x_ref[0]**o+(-0.0868143348547)*x_ref[0]+(-0.456384710114)*x_ref[1]**o+(-0.646489063979)*x_ref[1]+(0.594500214771)*x_ref[2]**o+(-0.299956296467)*x_ref[2]
        ref[1,3,1,1]=(-0.453257208906)*x_ref[0]**o+(0.583260425119)*x_ref[0]+(0.263463648634)*x_ref[1]**o+(0.493693446665)*x_ref[1]+(-0.529884773858)*x_ref[2]**o+(0.220733486264)*x_ref[2]
        ref[1,3,2,0]=(0.669052014654)*x_ref[0]**o+(-0.247902780475)*x_ref[0]+(0.44071786167)*x_ref[1]**o+(-0.21199653848)*x_ref[1]+(-0.0713603448391)*x_ref[2]**o+(-0.66681439981)*x_ref[2]
        ref[1,3,2,1]=(-0.341533813758)*x_ref[0]**o+(0.770581995272)*x_ref[0]+(0.03376913861)*x_ref[1]**o+(0.000270719073091)*x_ref[1]+(-0.774936686899)*x_ref[2]**o+(-0.0196300167016)*x_ref[2]
        ref[1,4,0,0]=(-0.869695667108)*x_ref[0]**o+(0.515676789402)*x_ref[0]+(0.67731822534)*x_ref[1]**o+(0.126887846346)*x_ref[1]+(0.843200966455)*x_ref[2]**o+(-0.59930363301)*x_ref[2]
        ref[1,4,0,1]=(0.909369352728)*x_ref[0]**o+(-0.110828876636)*x_ref[0]+(-0.65561953478)*x_ref[1]**o+(0.598491879954)*x_ref[1]+(-0.557567770163)*x_ref[2]**o+(-0.391108926502)*x_ref[2]
        ref[1,4,1,0]=(0.320646291934)*x_ref[0]**o+(0.164062007697)*x_ref[0]+(-0.546323998218)*x_ref[1]**o+(0.72651711346)*x_ref[1]+(-0.613496738898)*x_ref[2]**o+(0.430761082137)*x_ref[2]
        ref[1,4,1,1]=(-0.721651561555)*x_ref[0]**o+(-0.0647310624509)*x_ref[0]+(-0.360525345485)*x_ref[1]**o+(0.408239277468)*x_ref[1]+(0.600551705094)*x_ref[2]**o+(0.506237401392)*x_ref[2]
        ref[1,4,2,0]=(-0.391702965605)*x_ref[0]**o+(-0.574470582913)*x_ref[0]+(-0.729724609696)*x_ref[1]**o+(0.91381180302)*x_ref[1]+(-0.544328104929)*x_ref[2]**o+(-0.516992534607)*x_ref[2]
        ref[1,4,2,1]=(-0.0409666962711)*x_ref[0]**o+(-0.524913112823)*x_ref[0]+(-0.677030607856)*x_ref[1]**o+(-0.530859496807)*x_ref[1]+(0.137890059737)*x_ref[2]**o+(0.0669856870322)*x_ref[2]
        ref[2,0,0,0]=(0.10369318023)*x_ref[0]**o+(0.345691611083)*x_ref[0]+(-0.222795950561)*x_ref[1]**o+(-0.797690270935)*x_ref[1]+(-0.703457203234)*x_ref[2]**o+(-0.751193209871)*x_ref[2]
        ref[2,0,0,1]=(0.166457278787)*x_ref[0]**o+(-0.0304766240065)*x_ref[0]+(0.159987374581)*x_ref[1]**o+(0.828012214748)*x_ref[1]+(0.0670696566332)*x_ref[2]**o+(-0.498252024306)*x_ref[2]
        ref[2,0,1,0]=(0.901095992383)*x_ref[0]**o+(0.400296735981)*x_ref[0]+(0.394687295672)*x_ref[1]**o+(-0.986496611224)*x_ref[1]+(-0.768945602918)*x_ref[2]**o+(-0.782762026735)*x_ref[2]
        ref[2,0,1,1]=(0.47969549703)*x_ref[0]**o+(0.981251232699)*x_ref[0]+(-0.69807700503)*x_ref[1]**o+(-0.813549271424)*x_ref[1]+(-0.135970806025)*x_ref[2]**o+(0.288738861745)*x_ref[2]
        ref[2,0,2,0]=(0.315262113284)*x_ref[0]**o+(0.0380265323361)*x_ref[0]+(0.259894600802)*x_ref[1]**o+(0.538712801332)*x_ref[1]+(-0.385783597257)*x_ref[2]**o+(0.666670257561)*x_ref[2]
        ref[2,0,2,1]=(-0.756715539091)*x_ref[0]**o+(-0.823809574081)*x_ref[0]+(-0.283842954238)*x_ref[1]**o+(-0.640166311159)*x_ref[1]+(-0.39947011053)*x_ref[2]**o+(-0.393543798757)*x_ref[2]
        ref[2,1,0,0]=(-0.292033523499)*x_ref[0]**o+(-0.940246412652)*x_ref[0]+(-0.327571437608)*x_ref[1]**o+(0.837077706655)*x_ref[1]+(0.446471499818)*x_ref[2]**o+(-0.747249149016)*x_ref[2]
        ref[2,1,0,1]=(-0.867504469067)*x_ref[0]**o+(0.635684140727)*x_ref[0]+(0.488043128661)*x_ref[1]**o+(-0.163389002402)*x_ref[1]+(0.0440527250714)*x_ref[2]**o+(0.605766722432)*x_ref[2]
        ref[2,1,1,0]=(-0.0622304802635)*x_ref[0]**o+(0.607299318698)*x_ref[0]+(0.974933906045)*x_ref[1]**o+(0.0632442428777)*x_ref[1]+(0.0672808299851)*x_ref[2]**o+(-0.555700803374)*x_ref[2]
        ref[2,1,1,1]=(-0.350438453721)*x_ref[0]**o+(-0.826289294567)*x_ref[0]+(-0.473764081208)*x_ref[1]**o+(-0.247804269494)*x_ref[1]+(0.834301955859)*x_ref[2]**o+(0.443058363661)*x_ref[2]
        ref[2,1,2,0]=(0.5046026606)*x_ref[0]**o+(0.305502438052)*x_ref[0]+(-0.201816724695)*x_ref[1]**o+(-0.101544576856)*x_ref[1]+(0.729817329051)*x_ref[2]**o+(-0.851479834334)*x_ref[2]
        ref[2,1,2,1]=(0.917039424926)*x_ref[0]**o+(0.260565966637)*x_ref[0]+(-0.595846674057)*x_ref[1]**o+(0.654773238108)*x_ref[1]+(0.544664592565)*x_ref[2]**o+(0.363343885358)*x_ref[2]
        ref[2,2,0,0]=(0.243879200917)*x_ref[0]**o+(0.335612613868)*x_ref[0]+(-0.287066716076)*x_ref[1]**o+(0.506206687739)*x_ref[1]+(-0.902513036448)*x_ref[2]**o+(-0.427017746025)*x_ref[2]
        ref[2,2,0,1]=(0.965102871162)*x_ref[0]**o+(0.161476912964)*x_ref[0]+(0.97007160306)*x_ref[1]**o+(0.870104939402)*x_ref[1]+(0.552404991536)*x_ref[2]**o+(-0.319082395552)*x_ref[2]
        ref[2,2,1,0]=(0.17611459545)*x_ref[0]**o+(-0.457419791602)*x_ref[0]+(-0.00217752259743)*x_ref[1]**o+(-0.632177103931)*x_ref[1]+(0.521123797224)*x_ref[2]**o+(0.427935307605)*x_ref[2]
        ref[2,2,1,1]=(-0.993298850316)*x_ref[0]**o+(0.277763727112)*x_ref[0]+(0.849464768611)*x_ref[1]**o+(0.890919639607)*x_ref[1]+(0.01030894969)*x_ref[2]**o+(-0.76051772768)*x_ref[2]
        ref[2,2,2,0]=(-0.622887788117)*x_ref[0]**o+(0.40187802561)*x_ref[0]+(0.836685367934)*x_ref[1]**o+(0.700379534201)*x_ref[1]+(0.683627950354)*x_ref[2]**o+(0.445351804296)*x_ref[2]
        ref[2,2,2,1]=(0.124965536288)*x_ref[0]**o+(-0.198462168996)*x_ref[0]+(-0.657084126986)*x_ref[1]**o+(-0.420469996385)*x_ref[1]+(0.237164091597)*x_ref[2]**o+(0.161926160295)*x_ref[2]
        ref[2,3,0,0]=(-0.622682941897)*x_ref[0]**o+(-0.6279729398)*x_ref[0]+(0.189086870792)*x_ref[1]**o+(-0.350822257394)*x_ref[1]+(-0.505481371359)*x_ref[2]**o+(0.812944216714)*x_ref[2]
        ref[2,3,0,1]=(-0.246704719057)*x_ref[0]**o+(0.513716384141)*x_ref[0]+(-0.264914192193)*x_ref[1]**o+(-0.32984984663)*x_ref[1]+(-0.80409813849)*x_ref[2]**o+(-0.825568736974)*x_ref[2]
        ref[2,3,1,0]=(-0.534782640399)*x_ref[0]**o+(-0.917823259622)*x_ref[0]+(0.691827074405)*x_ref[1]**o+(-0.818629872154)*x_ref[1]+(0.870046992385)*x_ref[2]**o+(0.00423218787803)*x_ref[2]
        ref[2,3,1,1]=(0.633068676613)*x_ref[0]**o+(-0.895613811402)*x_ref[0]+(0.62242069294)*x_ref[1]**o+(-0.272136050955)*x_ref[1]+(-0.199008776357)*x_ref[2]**o+(0.836119124508)*x_ref[2]
        ref[2,3,2,0]=(0.335884106464)*x_ref[0]**o+(-0.857917873801)*x_ref[0]+(-0.42653278071)*x_ref[1]**o+(0.375200781448)*x_ref[1]+(0.378427453722)*x_ref[2]**o+(0.398603623445)*x_ref[2]
        ref[2,3,2,1]=(0.0602499604703)*x_ref[0]**o+(-0.921459358089)*x_ref[0]+(0.558180675902)*x_ref[1]**o+(0.0863734870743)*x_ref[1]+(0.215167095483)*x_ref[2]**o+(-0.37908830738)*x_ref[2]
        ref[2,4,0,0]=(0.866321987007)*x_ref[0]**o+(0.81950553744)*x_ref[0]+(0.0422552872889)*x_ref[1]**o+(0.369328458453)*x_ref[1]+(0.0547909452885)*x_ref[2]**o+(-0.396367133883)*x_ref[2]
        ref[2,4,0,1]=(0.438824337571)*x_ref[0]**o+(0.211435787811)*x_ref[0]+(-0.801273340861)*x_ref[1]**o+(-0.181654271172)*x_ref[1]+(0.374171148243)*x_ref[2]**o+(0.945633108881)*x_ref[2]
        ref[2,4,1,0]=(0.588044951473)*x_ref[0]**o+(0.401384824924)*x_ref[0]+(-0.191996429616)*x_ref[1]**o+(0.0472579697166)*x_ref[1]+(0.470962958517)*x_ref[2]**o+(0.52718875619)*x_ref[2]
        ref[2,4,1,1]=(0.371613549998)*x_ref[0]**o+(-0.414998926249)*x_ref[0]+(-0.762013182749)*x_ref[1]**o+(0.796144230249)*x_ref[1]+(-0.555041053148)*x_ref[2]**o+(-0.595298275916)*x_ref[2]
        ref[2,4,2,0]=(-0.992527136862)*x_ref[0]**o+(-0.995041267483)*x_ref[0]+(-0.730305990844)*x_ref[1]**o+(-0.52470798566)*x_ref[1]+(-0.938945944067)*x_ref[2]**o+(0.33396135548)*x_ref[2]
        ref[2,4,2,1]=(0.0333921786723)*x_ref[0]**o+(0.707990374515)*x_ref[0]+(-0.106667413755)*x_ref[1]**o+(0.282989655082)*x_ref[1]+(-0.152535982788)*x_ref[2]**o+(-0.75130096727)*x_ref[2]
        ref[3,0,0,0]=(-0.822675771493)*x_ref[0]**o+(0.726468488157)*x_ref[0]+(0.972567560814)*x_ref[1]**o+(0.308266449093)*x_ref[1]+(0.71069864654)*x_ref[2]**o+(-0.683195491271)*x_ref[2]
        ref[3,0,0,1]=(-0.505155072199)*x_ref[0]**o+(0.339013729552)*x_ref[0]+(-0.81565931012)*x_ref[1]**o+(0.362458680494)*x_ref[1]+(0.960221320064)*x_ref[2]**o+(-0.17234580875)*x_ref[2]
        ref[3,0,1,0]=(0.67972657926)*x_ref[0]**o+(0.143085543693)*x_ref[0]+(-0.699516563264)*x_ref[1]**o+(-0.892226133751)*x_ref[1]+(-0.898541541049)*x_ref[2]**o+(-0.736249681714)*x_ref[2]
        ref[3,0,1,1]=(-0.144423319215)*x_ref[0]**o+(-0.509713525676)*x_ref[0]+(0.347584762861)*x_ref[1]**o+(0.666169972794)*x_ref[1]+(0.691734080163)*x_ref[2]**o+(0.9338062148)*x_ref[2]
        ref[3,0,2,0]=(-0.47323843589)*x_ref[0]**o+(0.0819655059363)*x_ref[0]+(0.0863030211133)*x_ref[1]**o+(0.127803089307)*x_ref[1]+(-0.155852489756)*x_ref[2]**o+(-0.00470398386987)*x_ref[2]
        ref[3,0,2,1]=(0.372452726232)*x_ref[0]**o+(0.133808609857)*x_ref[0]+(0.962945642209)*x_ref[1]**o+(0.0446609213929)*x_ref[1]+(-0.446207951293)*x_ref[2]**o+(0.0400733439868)*x_ref[2]
        ref[3,1,0,0]=(-0.668575734235)*x_ref[0]**o+(0.905913572459)*x_ref[0]+(-0.321327880802)*x_ref[1]**o+(0.0263209456188)*x_ref[1]+(-0.0355325780368)*x_ref[2]**o+(-0.624130395105)*x_ref[2]
        ref[3,1,0,1]=(0.859810828483)*x_ref[0]**o+(-0.235372125226)*x_ref[0]+(-0.16715244612)*x_ref[1]**o+(-0.793399872713)*x_ref[1]+(0.303973795556)*x_ref[2]**o+(0.176740655638)*x_ref[2]
        ref[3,1,1,0]=(0.736448874258)*x_ref[0]**o+(-0.448956585466)*x_ref[0]+(0.782095902992)*x_ref[1]**o+(0.796524196866)*x_ref[1]+(-0.732812289541)*x_ref[2]**o+(0.687089667291)*x_ref[2]
        ref[3,1,1,1]=(-0.797708068194)*x_ref[0]**o+(-0.691308116296)*x_ref[0]+(-0.644065410151)*x_ref[1]**o+(-0.929624672017)*x_ref[1]+(0.265040940279)*x_ref[2]**o+(0.424929324889)*x_ref[2]
        ref[3,1,2,0]=(0.555032482777)*x_ref[0]**o+(-0.564617314931)*x_ref[0]+(-0.612721853869)*x_ref[1]**o+(-0.048466699593)*x_ref[1]+(0.753265037897)*x_ref[2]**o+(0.913086239874)*x_ref[2]
        ref[3,1,2,1]=(0.580881807771)*x_ref[0]**o+(0.970973128604)*x_ref[0]+(-0.603610563753)*x_ref[1]**o+(0.979985814438)*x_ref[1]+(-0.639531987694)*x_ref[2]**o+(-0.773094138073)*x_ref[2]
        ref[3,2,0,0]=(0.574306840544)*x_ref[0]**o+(0.449094769988)*x_ref[0]+(-0.406303076614)*x_ref[1]**o+(-0.776648781906)*x_ref[1]+(-0.59829476647)*x_ref[2]**o+(0.257632446666)*x_ref[2]
        ref[3,2,0,1]=(0.689106127051)*x_ref[0]**o+(0.717184851708)*x_ref[0]+(-0.495581943615)*x_ref[1]**o+(-0.30329139788)*x_ref[1]+(0.901753128535)*x_ref[2]**o+(0.48496505727)*x_ref[2]
        ref[3,2,1,0]=(0.252050902256)*x_ref[0]**o+(-0.0414281706943)*x_ref[0]+(-0.284125627654)*x_ref[1]**o+(0.477450554269)*x_ref[1]+(0.849202242148)*x_ref[2]**o+(0.211826565889)*x_ref[2]
        ref[3,2,1,1]=(-0.319900673571)*x_ref[0]**o+(0.965993723739)*x_ref[0]+(0.671360092314)*x_ref[1]**o+(-0.652805407934)*x_ref[1]+(-0.0352316646853)*x_ref[2]**o+(-0.442493499306)*x_ref[2]
        ref[3,2,2,0]=(0.525577660439)*x_ref[0]**o+(0.660882378915)*x_ref[0]+(-0.419826738105)*x_ref[1]**o+(-0.798251889077)*x_ref[1]+(-0.430382393461)*x_ref[2]**o+(0.714357242496)*x_ref[2]
        ref[3,2,2,1]=(-0.200634944034)*x_ref[0]**o+(0.211086366034)*x_ref[0]+(0.116289218188)*x_ref[1]**o+(-0.785792860501)*x_ref[1]+(0.763320327438)*x_ref[2]**o+(-0.457355826175)*x_ref[2]
        ref[3,3,0,0]=(-0.890873783782)*x_ref[0]**o+(0.322610697657)*x_ref[0]+(-0.351437390332)*x_ref[1]**o+(-0.720698169708)*x_ref[1]+(-0.664444440194)*x_ref[2]**o+(0.274865394829)*x_ref[2]
        ref[3,3,0,1]=(-0.384885652342)*x_ref[0]**o+(-0.409799972422)*x_ref[0]+(0.679315041587)*x_ref[1]**o+(0.0859788488759)*x_ref[1]+(-0.136173937432)*x_ref[2]**o+(-0.136591429212)*x_ref[2]
        ref[3,3,1,0]=(0.147934741321)*x_ref[0]**o+(0.246941956666)*x_ref[0]+(0.696831123708)*x_ref[1]**o+(-0.905966322958)*x_ref[1]+(0.672705695255)*x_ref[2]**o+(-0.776017262395)*x_ref[2]
        ref[3,3,1,1]=(0.933982654679)*x_ref[0]**o+(-0.367246348547)*x_ref[0]+(-0.254076834401)*x_ref[1]**o+(0.148617981234)*x_ref[1]+(-0.486881004358)*x_ref[2]**o+(0.636371570082)*x_ref[2]
        ref[3,3,2,0]=(-0.536125392071)*x_ref[0]**o+(-0.663380779479)*x_ref[0]+(0.0655694689011)*x_ref[1]**o+(0.0390002326894)*x_ref[1]+(-0.731730129742)*x_ref[2]**o+(-0.578791485067)*x_ref[2]
        ref[3,3,2,1]=(0.705873902409)*x_ref[0]**o+(-0.0444303506955)*x_ref[0]+(-0.978519191225)*x_ref[1]**o+(-0.0280878186958)*x_ref[1]+(0.182378726121)*x_ref[2]**o+(-0.555345919696)*x_ref[2]
        ref[3,4,0,0]=(0.945945862395)*x_ref[0]**o+(-0.448487950632)*x_ref[0]+(-0.477570573229)*x_ref[1]**o+(0.313871838624)*x_ref[1]+(0.237944220287)*x_ref[2]**o+(-0.218095130482)*x_ref[2]
        ref[3,4,0,1]=(-0.758647277269)*x_ref[0]**o+(-0.550604667208)*x_ref[0]+(-0.262944399053)*x_ref[1]**o+(0.994302652859)*x_ref[1]+(0.363440310008)*x_ref[2]**o+(-0.127863970613)*x_ref[2]
        ref[3,4,1,0]=(0.667202433965)*x_ref[0]**o+(0.954901068821)*x_ref[0]+(0.719745510318)*x_ref[1]**o+(-0.763882742007)*x_ref[1]+(0.880927476123)*x_ref[2]**o+(0.102034630893)*x_ref[2]
        ref[3,4,1,1]=(-0.684644024456)*x_ref[0]**o+(0.88037275093)*x_ref[0]+(-0.772213766289)*x_ref[1]**o+(-0.702615214094)*x_ref[1]+(-0.25661708053)*x_ref[2]**o+(0.126779040694)*x_ref[2]
        ref[3,4,2,0]=(-0.589611385303)*x_ref[0]**o+(0.638712635252)*x_ref[0]+(0.745450741856)*x_ref[1]**o+(0.399494092815)*x_ref[1]+(-0.717293809038)*x_ref[2]**o+(0.305662737751)*x_ref[2]
        ref[3,4,2,1]=(0.0572426997796)*x_ref[0]**o+(-0.764600144578)*x_ref[0]+(0.482545219565)*x_ref[1]**o+(0.525883069411)*x_ref[1]+(-0.612224327195)*x_ref[2]**o+(0.354344820256)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromData_ReducedSolution_rank0(self):
      """
      tests interpolate for rank 0 Data onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedSolution onto ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Data(0,(),w)
      ref=Data(0,(),w_ref)
      if dim==2:
        arg=(-0.100067148208)*x[0]+(-0.8375743725)*x[1]
        ref=(-0.100067148208)*x_ref[0]+(-0.8375743725)*x_ref[1]
      else:
        arg=(-0.247343416726)*x[0]+(0.604193484164)*x[1]+(-0.219899438287)*x[2]
        ref=(-0.247343416726)*x_ref[0]+(0.604193484164)*x_ref[1]+(-0.219899438287)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromData_ReducedSolution_rank1(self):
      """
      tests interpolate for rank 1 Data onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedSolution onto ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Data(0,(2,),w)
      ref=Data(0,(2,),w_ref)
      if dim==2:
        arg[0]=(-1.66196653104)*x[0]+(1.29891335227)*x[1]
        arg[1]=(-0.347844771248)*x[0]+(-0.404726300584)*x[1]
        ref[0]=(-1.66196653104)*x[0]+(1.29891335227)*x[1]
        ref[1]=(-0.347844771248)*x[0]+(-0.404726300584)*x[1]
      else:
        arg[0]=(0.334094742067)*x[0]+(-0.619063646554)*x[1]+(0.538572422001)*x[2]
        arg[1]=(0.673290433004)*x[0]+(0.169501972843)*x[1]+(-0.833663493893)*x[2]
        ref[0]=(0.334094742067)*x[0]+(-0.619063646554)*x[1]+(0.538572422001)*x[2]
        ref[1]=(0.673290433004)*x[0]+(0.169501972843)*x[1]+(-0.833663493893)*x[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromData_ReducedSolution_rank2(self):
      """
      tests interpolate for rank 2 Data onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedSolution onto ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5),w)
      ref=Data(0,(4, 5),w_ref)
      if dim==2:
        arg[0,0]=(0.00153698933322)*x[0]+(0.392281645764)*x[1]
        arg[0,1]=(-1.2794655544)*x[0]+(-0.373519639818)*x[1]
        arg[0,2]=(0.30127033321)*x[0]+(-0.0996865853738)*x[1]
        arg[0,3]=(1.23405799349)*x[0]+(0.498558200339)*x[1]
        arg[0,4]=(0.85519350742)*x[0]+(-0.19188046135)*x[1]
        arg[1,0]=(0.417367834962)*x[0]+(0.992318201639)*x[1]
        arg[1,1]=(-0.646432128833)*x[0]+(-0.872750321582)*x[1]
        arg[1,2]=(-0.806117185061)*x[0]+(-0.89396408325)*x[1]
        arg[1,3]=(0.768017237627)*x[0]+(-1.2904111912)*x[1]
        arg[1,4]=(-0.0159685523188)*x[0]+(0.401410193169)*x[1]
        arg[2,0]=(-0.844501716181)*x[0]+(-0.123115343819)*x[1]
        arg[2,1]=(1.08441909409)*x[0]+(0.152432282539)*x[1]
        arg[2,2]=(-0.60551037413)*x[0]+(1.15692828826)*x[1]
        arg[2,3]=(-1.21509168541)*x[0]+(1.03683236377)*x[1]
        arg[2,4]=(1.16850730134)*x[0]+(0.162202615909)*x[1]
        arg[3,0]=(0.727772166897)*x[0]+(-0.296815840935)*x[1]
        arg[3,1]=(-0.0880577151489)*x[0]+(1.66427691426)*x[1]
        arg[3,2]=(0.415144534458)*x[0]+(-0.555833723295)*x[1]
        arg[3,3]=(1.09444635829)*x[0]+(-0.335209417536)*x[1]
        arg[3,4]=(-1.05047769585)*x[0]+(0.684427039986)*x[1]
        ref[0,0]=(0.00153698933322)*x_ref[0]+(0.392281645764)*x_ref[1]
        ref[0,1]=(-1.2794655544)*x_ref[0]+(-0.373519639818)*x_ref[1]
        ref[0,2]=(0.30127033321)*x_ref[0]+(-0.0996865853738)*x_ref[1]
        ref[0,3]=(1.23405799349)*x_ref[0]+(0.498558200339)*x_ref[1]
        ref[0,4]=(0.85519350742)*x_ref[0]+(-0.19188046135)*x_ref[1]
        ref[1,0]=(0.417367834962)*x_ref[0]+(0.992318201639)*x_ref[1]
        ref[1,1]=(-0.646432128833)*x_ref[0]+(-0.872750321582)*x_ref[1]
        ref[1,2]=(-0.806117185061)*x_ref[0]+(-0.89396408325)*x_ref[1]
        ref[1,3]=(0.768017237627)*x_ref[0]+(-1.2904111912)*x_ref[1]
        ref[1,4]=(-0.0159685523188)*x_ref[0]+(0.401410193169)*x_ref[1]
        ref[2,0]=(-0.844501716181)*x_ref[0]+(-0.123115343819)*x_ref[1]
        ref[2,1]=(1.08441909409)*x_ref[0]+(0.152432282539)*x_ref[1]
        ref[2,2]=(-0.60551037413)*x_ref[0]+(1.15692828826)*x_ref[1]
        ref[2,3]=(-1.21509168541)*x_ref[0]+(1.03683236377)*x_ref[1]
        ref[2,4]=(1.16850730134)*x_ref[0]+(0.162202615909)*x_ref[1]
        ref[3,0]=(0.727772166897)*x_ref[0]+(-0.296815840935)*x_ref[1]
        ref[3,1]=(-0.0880577151489)*x_ref[0]+(1.66427691426)*x_ref[1]
        ref[3,2]=(0.415144534458)*x_ref[0]+(-0.555833723295)*x_ref[1]
        ref[3,3]=(1.09444635829)*x_ref[0]+(-0.335209417536)*x_ref[1]
        ref[3,4]=(-1.05047769585)*x_ref[0]+(0.684427039986)*x_ref[1]
      else:
        arg[0,0]=(-0.611607640773)*x[0]+(0.584485971628)*x[1]+(0.0090316337643)*x[2]
        arg[0,1]=(0.905278285857)*x[0]+(0.332523151242)*x[1]+(0.9909841807)*x[2]
        arg[0,2]=(-1.06629333132)*x[0]+(-0.38478180064)*x[1]+(-0.63309211256)*x[2]
        arg[0,3]=(0.122436386344)*x[0]+(-0.218752671204)*x[1]+(-0.746695172216)*x[2]
        arg[0,4]=(1.03152124874)*x[0]+(-1.11327271563)*x[1]+(-0.257591112135)*x[2]
        arg[1,0]=(0.969019600299)*x[0]+(-0.320279309689)*x[1]+(0.327767746545)*x[2]
        arg[1,1]=(0.291023069052)*x[0]+(0.78112901064)*x[1]+(0.879257239042)*x[2]
        arg[1,2]=(-0.131262891536)*x[0]+(-0.0109690229153)*x[1]+(-0.171900332406)*x[2]
        arg[1,3]=(-1.32272684486)*x[0]+(-0.256869536085)*x[1]+(0.0308524567462)*x[2]
        arg[1,4]=(1.30707061279)*x[0]+(0.972866282634)*x[1]+(-0.474948537882)*x[2]
        arg[2,0]=(0.169034210421)*x[0]+(-1.08714896226)*x[1]+(-0.636838450708)*x[2]
        arg[2,1]=(-0.470752780773)*x[0]+(-0.695286088059)*x[1]+(-1.19197058712)*x[2]
        arg[2,2]=(0.701403721228)*x[0]+(0.551002482436)*x[1]+(-0.295853851234)*x[2]
        arg[2,3]=(-0.796431361745)*x[0]+(0.0581884766856)*x[1]+(-1.13668563089)*x[2]
        arg[2,4]=(-0.733185798017)*x[0]+(0.178900103735)*x[1]+(-0.845671526781)*x[2]
        arg[3,0]=(-1.47064927107)*x[0]+(1.33393324965)*x[1]+(-0.767393460937)*x[2]
        arg[3,1]=(1.36784513226)*x[0]+(0.100008910431)*x[1]+(-1.09210256713)*x[2]
        arg[3,2]=(-0.873076669977)*x[0]+(0.367213631778)*x[1]+(0.0698296920039)*x[2]
        arg[3,3]=(-1.76609214158)*x[0]+(-1.26186286566)*x[1]+(0.377817035113)*x[2]
        arg[3,4]=(-1.28261709396)*x[0]+(0.240480767533)*x[1]+(0.307813279357)*x[2]
        ref[0,0]=(-0.611607640773)*x_ref[0]+(0.584485971628)*x_ref[1]+(0.0090316337643)*x_ref[2]
        ref[0,1]=(0.905278285857)*x_ref[0]+(0.332523151242)*x_ref[1]+(0.9909841807)*x_ref[2]
        ref[0,2]=(-1.06629333132)*x_ref[0]+(-0.38478180064)*x_ref[1]+(-0.63309211256)*x_ref[2]
        ref[0,3]=(0.122436386344)*x_ref[0]+(-0.218752671204)*x_ref[1]+(-0.746695172216)*x_ref[2]
        ref[0,4]=(1.03152124874)*x_ref[0]+(-1.11327271563)*x_ref[1]+(-0.257591112135)*x_ref[2]
        ref[1,0]=(0.969019600299)*x_ref[0]+(-0.320279309689)*x_ref[1]+(0.327767746545)*x_ref[2]
        ref[1,1]=(0.291023069052)*x_ref[0]+(0.78112901064)*x_ref[1]+(0.879257239042)*x_ref[2]
        ref[1,2]=(-0.131262891536)*x_ref[0]+(-0.0109690229153)*x_ref[1]+(-0.171900332406)*x_ref[2]
        ref[1,3]=(-1.32272684486)*x_ref[0]+(-0.256869536085)*x_ref[1]+(0.0308524567462)*x_ref[2]
        ref[1,4]=(1.30707061279)*x_ref[0]+(0.972866282634)*x_ref[1]+(-0.474948537882)*x_ref[2]
        ref[2,0]=(0.169034210421)*x_ref[0]+(-1.08714896226)*x_ref[1]+(-0.636838450708)*x_ref[2]
        ref[2,1]=(-0.470752780773)*x_ref[0]+(-0.695286088059)*x_ref[1]+(-1.19197058712)*x_ref[2]
        ref[2,2]=(0.701403721228)*x_ref[0]+(0.551002482436)*x_ref[1]+(-0.295853851234)*x_ref[2]
        ref[2,3]=(-0.796431361745)*x_ref[0]+(0.0581884766856)*x_ref[1]+(-1.13668563089)*x_ref[2]
        ref[2,4]=(-0.733185798017)*x_ref[0]+(0.178900103735)*x_ref[1]+(-0.845671526781)*x_ref[2]
        ref[3,0]=(-1.47064927107)*x_ref[0]+(1.33393324965)*x_ref[1]+(-0.767393460937)*x_ref[2]
        ref[3,1]=(1.36784513226)*x_ref[0]+(0.100008910431)*x_ref[1]+(-1.09210256713)*x_ref[2]
        ref[3,2]=(-0.873076669977)*x_ref[0]+(0.367213631778)*x_ref[1]+(0.0698296920039)*x_ref[2]
        ref[3,3]=(-1.76609214158)*x_ref[0]+(-1.26186286566)*x_ref[1]+(0.377817035113)*x_ref[2]
        ref[3,4]=(-1.28261709396)*x_ref[0]+(0.240480767533)*x_ref[1]+(0.307813279357)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromData_ReducedSolution_rank3(self):
      """
      tests interpolate for rank 3 Data onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedSolution onto ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2),w_ref)
      if dim==2:
        arg[0,0,0]=(-0.423505169404)*x[0]+(-1.10390334264)*x[1]
        arg[0,0,1]=(-0.181820089254)*x[0]+(-0.374602268791)*x[1]
        arg[0,1,0]=(-1.36906917075)*x[0]+(0.26554385908)*x[1]
        arg[0,1,1]=(-0.280661572374)*x[0]+(-0.239232524287)*x[1]
        arg[1,0,0]=(-1.25670251934)*x[0]+(-0.429353977735)*x[1]
        arg[1,0,1]=(-0.747928970735)*x[0]+(1.17874317611)*x[1]
        arg[1,1,0]=(-0.548814611802)*x[0]+(-0.0249315764285)*x[1]
        arg[1,1,1]=(-0.481123117522)*x[0]+(1.23402203879)*x[1]
        arg[2,0,0]=(-0.0639512689884)*x[0]+(-0.0278213371376)*x[1]
        arg[2,0,1]=(-1.17292797621)*x[0]+(0.42140600304)*x[1]
        arg[2,1,0]=(-0.358052601083)*x[0]+(-1.66807880791)*x[1]
        arg[2,1,1]=(-0.291433251349)*x[0]+(0.111473162108)*x[1]
        arg[3,0,0]=(0.742643625989)*x[0]+(-1.29214106228)*x[1]
        arg[3,0,1]=(0.319887321826)*x[0]+(-0.40213369238)*x[1]
        arg[3,1,0]=(-0.449412064097)*x[0]+(-1.10654262009)*x[1]
        arg[3,1,1]=(1.19722227632)*x[0]+(1.3455799026)*x[1]
        arg[4,0,0]=(-1.77453188487)*x[0]+(0.634838793872)*x[1]
        arg[4,0,1]=(-0.215461066907)*x[0]+(1.11635233844)*x[1]
        arg[4,1,0]=(0.835816652592)*x[0]+(0.668378554214)*x[1]
        arg[4,1,1]=(-1.22133225406)*x[0]+(-0.148780252398)*x[1]
        arg[5,0,0]=(0.320245409639)*x[0]+(-1.08828513776)*x[1]
        arg[5,0,1]=(-0.1865835417)*x[0]+(-0.353282061874)*x[1]
        arg[5,1,0]=(1.38858040685)*x[0]+(0.166398261098)*x[1]
        arg[5,1,1]=(0.947718875755)*x[0]+(-1.06171934302)*x[1]
        ref[0,0,0]=(-0.423505169404)*x_ref[0]+(-1.10390334264)*x_ref[1]
        ref[0,0,1]=(-0.181820089254)*x_ref[0]+(-0.374602268791)*x_ref[1]
        ref[0,1,0]=(-1.36906917075)*x_ref[0]+(0.26554385908)*x_ref[1]
        ref[0,1,1]=(-0.280661572374)*x_ref[0]+(-0.239232524287)*x_ref[1]
        ref[1,0,0]=(-1.25670251934)*x_ref[0]+(-0.429353977735)*x_ref[1]
        ref[1,0,1]=(-0.747928970735)*x_ref[0]+(1.17874317611)*x_ref[1]
        ref[1,1,0]=(-0.548814611802)*x_ref[0]+(-0.0249315764285)*x_ref[1]
        ref[1,1,1]=(-0.481123117522)*x_ref[0]+(1.23402203879)*x_ref[1]
        ref[2,0,0]=(-0.0639512689884)*x_ref[0]+(-0.0278213371376)*x_ref[1]
        ref[2,0,1]=(-1.17292797621)*x_ref[0]+(0.42140600304)*x_ref[1]
        ref[2,1,0]=(-0.358052601083)*x_ref[0]+(-1.66807880791)*x_ref[1]
        ref[2,1,1]=(-0.291433251349)*x_ref[0]+(0.111473162108)*x_ref[1]
        ref[3,0,0]=(0.742643625989)*x_ref[0]+(-1.29214106228)*x_ref[1]
        ref[3,0,1]=(0.319887321826)*x_ref[0]+(-0.40213369238)*x_ref[1]
        ref[3,1,0]=(-0.449412064097)*x_ref[0]+(-1.10654262009)*x_ref[1]
        ref[3,1,1]=(1.19722227632)*x_ref[0]+(1.3455799026)*x_ref[1]
        ref[4,0,0]=(-1.77453188487)*x_ref[0]+(0.634838793872)*x_ref[1]
        ref[4,0,1]=(-0.215461066907)*x_ref[0]+(1.11635233844)*x_ref[1]
        ref[4,1,0]=(0.835816652592)*x_ref[0]+(0.668378554214)*x_ref[1]
        ref[4,1,1]=(-1.22133225406)*x_ref[0]+(-0.148780252398)*x_ref[1]
        ref[5,0,0]=(0.320245409639)*x_ref[0]+(-1.08828513776)*x_ref[1]
        ref[5,0,1]=(-0.1865835417)*x_ref[0]+(-0.353282061874)*x_ref[1]
        ref[5,1,0]=(1.38858040685)*x_ref[0]+(0.166398261098)*x_ref[1]
        ref[5,1,1]=(0.947718875755)*x_ref[0]+(-1.06171934302)*x_ref[1]
      else:
        arg[0,0,0]=(-0.892096322478)*x[0]+(0.720363183451)*x[1]+(0.600609272191)*x[2]
        arg[0,0,1]=(-0.418024461935)*x[0]+(-0.819275292035)*x[1]+(0.134962557692)*x[2]
        arg[0,1,0]=(-0.449783838361)*x[0]+(0.943677173533)*x[1]+(0.209424190991)*x[2]
        arg[0,1,1]=(-0.424386775004)*x[0]+(-0.213024388094)*x[1]+(-1.05927943384)*x[2]
        arg[1,0,0]=(-0.699504096539)*x[0]+(-0.153570308377)*x[1]+(0.576902392658)*x[2]
        arg[1,0,1]=(-0.426914055661)*x[0]+(-0.330405267059)*x[1]+(1.07656632155)*x[2]
        arg[1,1,0]=(0.775238005352)*x[0]+(1.11012072575)*x[1]+(0.764627098603)*x[2]
        arg[1,1,1]=(0.0130424708408)*x[0]+(-0.671778102107)*x[1]+(0.780684368983)*x[2]
        arg[2,0,0]=(-0.0512224258498)*x[0]+(-1.50515803984)*x[1]+(0.342824143946)*x[2]
        arg[2,0,1]=(0.117541624176)*x[0]+(0.372415524552)*x[1]+(0.255218095016)*x[2]
        arg[2,1,0]=(-0.574508895703)*x[0]+(0.345217292359)*x[1]+(0.209211956821)*x[2]
        arg[2,1,1]=(0.93643587147)*x[0]+(-0.096209009902)*x[1]+(0.205150859527)*x[2]
        arg[3,0,0]=(1.7541929884)*x[0]+(0.169061478447)*x[1]+(0.534746695515)*x[2]
        arg[3,0,1]=(0.442329266782)*x[0]+(0.334971710414)*x[1]+(1.66892557698)*x[2]
        arg[3,1,0]=(0.316436716758)*x[0]+(-1.25442030987)*x[1]+(0.372183105374)*x[2]
        arg[3,1,1]=(-0.254157820265)*x[0]+(0.287381300023)*x[1]+(0.0574117423352)*x[2]
        arg[4,0,0]=(-1.08380257521)*x[0]+(-1.31461890032)*x[1]+(-0.503802775099)*x[2]
        arg[4,0,1]=(-0.558293486798)*x[0]+(0.702317635925)*x[1]+(0.463814829137)*x[2]
        arg[4,1,0]=(-0.677160971974)*x[0]+(-0.14921275312)*x[1]+(-1.6501459435)*x[2]
        arg[4,1,1]=(-0.176672532474)*x[0]+(0.945017605811)*x[1]+(-0.790342939788)*x[2]
        arg[5,0,0]=(-1.45668070675)*x[0]+(-0.404957129204)*x[1]+(1.37383065954)*x[2]
        arg[5,0,1]=(0.53829326756)*x[0]+(0.543333031479)*x[1]+(-0.854057296362)*x[2]
        arg[5,1,0]=(0.805864283921)*x[0]+(-0.00186634953513)*x[1]+(-1.97111362539)*x[2]
        arg[5,1,1]=(-0.387746238967)*x[0]+(-0.739158353882)*x[1]+(-0.134082808118)*x[2]
        ref[0,0,0]=(-0.892096322478)*x_ref[0]+(0.720363183451)*x_ref[1]+(0.600609272191)*x_ref[2]
        ref[0,0,1]=(-0.418024461935)*x_ref[0]+(-0.819275292035)*x_ref[1]+(0.134962557692)*x_ref[2]
        ref[0,1,0]=(-0.449783838361)*x_ref[0]+(0.943677173533)*x_ref[1]+(0.209424190991)*x_ref[2]
        ref[0,1,1]=(-0.424386775004)*x_ref[0]+(-0.213024388094)*x_ref[1]+(-1.05927943384)*x_ref[2]
        ref[1,0,0]=(-0.699504096539)*x_ref[0]+(-0.153570308377)*x_ref[1]+(0.576902392658)*x_ref[2]
        ref[1,0,1]=(-0.426914055661)*x_ref[0]+(-0.330405267059)*x_ref[1]+(1.07656632155)*x_ref[2]
        ref[1,1,0]=(0.775238005352)*x_ref[0]+(1.11012072575)*x_ref[1]+(0.764627098603)*x_ref[2]
        ref[1,1,1]=(0.0130424708408)*x_ref[0]+(-0.671778102107)*x_ref[1]+(0.780684368983)*x_ref[2]
        ref[2,0,0]=(-0.0512224258498)*x_ref[0]+(-1.50515803984)*x_ref[1]+(0.342824143946)*x_ref[2]
        ref[2,0,1]=(0.117541624176)*x_ref[0]+(0.372415524552)*x_ref[1]+(0.255218095016)*x_ref[2]
        ref[2,1,0]=(-0.574508895703)*x_ref[0]+(0.345217292359)*x_ref[1]+(0.209211956821)*x_ref[2]
        ref[2,1,1]=(0.93643587147)*x_ref[0]+(-0.096209009902)*x_ref[1]+(0.205150859527)*x_ref[2]
        ref[3,0,0]=(1.7541929884)*x_ref[0]+(0.169061478447)*x_ref[1]+(0.534746695515)*x_ref[2]
        ref[3,0,1]=(0.442329266782)*x_ref[0]+(0.334971710414)*x_ref[1]+(1.66892557698)*x_ref[2]
        ref[3,1,0]=(0.316436716758)*x_ref[0]+(-1.25442030987)*x_ref[1]+(0.372183105374)*x_ref[2]
        ref[3,1,1]=(-0.254157820265)*x_ref[0]+(0.287381300023)*x_ref[1]+(0.0574117423352)*x_ref[2]
        ref[4,0,0]=(-1.08380257521)*x_ref[0]+(-1.31461890032)*x_ref[1]+(-0.503802775099)*x_ref[2]
        ref[4,0,1]=(-0.558293486798)*x_ref[0]+(0.702317635925)*x_ref[1]+(0.463814829137)*x_ref[2]
        ref[4,1,0]=(-0.677160971974)*x_ref[0]+(-0.14921275312)*x_ref[1]+(-1.6501459435)*x_ref[2]
        ref[4,1,1]=(-0.176672532474)*x_ref[0]+(0.945017605811)*x_ref[1]+(-0.790342939788)*x_ref[2]
        ref[5,0,0]=(-1.45668070675)*x_ref[0]+(-0.404957129204)*x_ref[1]+(1.37383065954)*x_ref[2]
        ref[5,0,1]=(0.53829326756)*x_ref[0]+(0.543333031479)*x_ref[1]+(-0.854057296362)*x_ref[2]
        ref[5,1,0]=(0.805864283921)*x_ref[0]+(-0.00186634953513)*x_ref[1]+(-1.97111362539)*x_ref[2]
        ref[5,1,1]=(-0.387746238967)*x_ref[0]+(-0.739158353882)*x_ref[1]+(-0.134082808118)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromData_ReducedSolution_rank4(self):
      """
      tests interpolate for rank 4 Data onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedSolution onto ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5, 3, 2),w)
      ref=Data(0,(4, 5, 3, 2),w_ref)
      if dim==2:
        arg[0,0,0,0]=(-0.754311658086)*x[0]+(0.235157034342)*x[1]
        arg[0,0,0,1]=(-1.33786315024)*x[0]+(0.243349279742)*x[1]
        arg[0,0,1,0]=(1.48657750088)*x[0]+(1.09853108305)*x[1]
        arg[0,0,1,1]=(1.4501362266)*x[0]+(0.824811567155)*x[1]
        arg[0,0,2,0]=(1.12567566082)*x[0]+(0.663525681174)*x[1]
        arg[0,0,2,1]=(1.48991565019)*x[0]+(-0.680795365011)*x[1]
        arg[0,1,0,0]=(0.268571449528)*x[0]+(0.818595834672)*x[1]
        arg[0,1,0,1]=(-1.55490945555)*x[0]+(0.783595197888)*x[1]
        arg[0,1,1,0]=(0.913201382585)*x[0]+(0.855491379376)*x[1]
        arg[0,1,1,1]=(-0.179192075116)*x[0]+(-0.690415079523)*x[1]
        arg[0,1,2,0]=(-0.0393574502912)*x[0]+(-0.690157650634)*x[1]
        arg[0,1,2,1]=(0.784216863217)*x[0]+(0.498735486159)*x[1]
        arg[0,2,0,0]=(0.137939554165)*x[0]+(-1.33961519113)*x[1]
        arg[0,2,0,1]=(-1.03897311736)*x[0]+(-1.37797405312)*x[1]
        arg[0,2,1,0]=(0.199812003756)*x[0]+(-0.0388499674037)*x[1]
        arg[0,2,1,1]=(1.15530534202)*x[0]+(-1.28280869078)*x[1]
        arg[0,2,2,0]=(0.836447791155)*x[0]+(1.2627066026)*x[1]
        arg[0,2,2,1]=(-0.584608492688)*x[0]+(-1.55305974305)*x[1]
        arg[0,3,0,0]=(-0.661952381799)*x[0]+(-1.27062403778)*x[1]
        arg[0,3,0,1]=(-1.47789532259)*x[0]+(-0.279583524936)*x[1]
        arg[0,3,1,0]=(1.40729771357)*x[0]+(0.0971284144528)*x[1]
        arg[0,3,1,1]=(-0.459167955911)*x[0]+(-1.29033281524)*x[1]
        arg[0,3,2,0]=(-1.63189436824)*x[0]+(0.75729808422)*x[1]
        arg[0,3,2,1]=(0.522479828718)*x[0]+(-0.0231299515383)*x[1]
        arg[0,4,0,0]=(0.241891446615)*x[0]+(0.803327107651)*x[1]
        arg[0,4,0,1]=(0.273613502836)*x[0]+(-0.0115180464171)*x[1]
        arg[0,4,1,0]=(1.96219818696)*x[0]+(-0.349137094743)*x[1]
        arg[0,4,1,1]=(0.365690821719)*x[0]+(-1.6943347181)*x[1]
        arg[0,4,2,0]=(-0.283970779872)*x[0]+(-0.258537179804)*x[1]
        arg[0,4,2,1]=(-0.973333430598)*x[0]+(1.48706276212)*x[1]
        arg[1,0,0,0]=(1.18757466971)*x[0]+(0.838804656357)*x[1]
        arg[1,0,0,1]=(0.840859904702)*x[0]+(1.09955195682)*x[1]
        arg[1,0,1,0]=(1.46190513227)*x[0]+(-0.596959830422)*x[1]
        arg[1,0,1,1]=(0.718689296745)*x[0]+(-0.628074620222)*x[1]
        arg[1,0,2,0]=(0.440761293041)*x[0]+(0.115800854194)*x[1]
        arg[1,0,2,1]=(1.34891146904)*x[0]+(0.813690260652)*x[1]
        arg[1,1,0,0]=(0.183071638098)*x[0]+(-1.37996210755)*x[1]
        arg[1,1,0,1]=(0.258142115988)*x[0]+(-0.757709381529)*x[1]
        arg[1,1,1,0]=(1.72392829309)*x[0]+(0.444995102781)*x[1]
        arg[1,1,1,1]=(-0.380898209988)*x[0]+(-0.0950246860347)*x[1]
        arg[1,1,2,0]=(0.203007226016)*x[0]+(1.09786001832)*x[1]
        arg[1,1,2,1]=(-0.502500865341)*x[0]+(-0.629980527995)*x[1]
        arg[1,2,0,0]=(0.482549518436)*x[0]+(0.565840824146)*x[1]
        arg[1,2,0,1]=(0.355818565068)*x[0]+(-0.78273295574)*x[1]
        arg[1,2,1,0]=(1.79204033071)*x[0]+(-0.698040851585)*x[1]
        arg[1,2,1,1]=(-0.534827741044)*x[0]+(0.00943131627498)*x[1]
        arg[1,2,2,0]=(-1.00663524444)*x[0]+(0.0207854553456)*x[1]
        arg[1,2,2,1]=(-0.338048040159)*x[0]+(-0.380607406217)*x[1]
        arg[1,3,0,0]=(0.586310549044)*x[0]+(0.276326798805)*x[1]
        arg[1,3,0,1]=(-0.260482580702)*x[0]+(-0.349279395321)*x[1]
        arg[1,3,1,0]=(-0.196533829895)*x[0]+(0.614805272738)*x[1]
        arg[1,3,1,1]=(-0.363176483895)*x[0]+(-0.925817017731)*x[1]
        arg[1,3,2,0]=(-1.24182075421)*x[0]+(0.215966446013)*x[1]
        arg[1,3,2,1]=(1.07546652642)*x[0]+(0.383803991974)*x[1]
        arg[1,4,0,0]=(0.199312303525)*x[0]+(-0.314520883859)*x[1]
        arg[1,4,0,1]=(-1.56890826152)*x[0]+(-0.133086662241)*x[1]
        arg[1,4,1,0]=(1.1307241936)*x[0]+(-1.32905744984)*x[1]
        arg[1,4,1,1]=(-0.385991455298)*x[0]+(0.292913024507)*x[1]
        arg[1,4,2,0]=(1.14561678056)*x[0]+(-0.662696162276)*x[1]
        arg[1,4,2,1]=(-1.49533948954)*x[0]+(1.43396138744)*x[1]
        arg[2,0,0,0]=(-0.63160396493)*x[0]+(0.813120998776)*x[1]
        arg[2,0,0,1]=(0.65598998434)*x[0]+(-1.02445537553)*x[1]
        arg[2,0,1,0]=(-1.13473104075)*x[0]+(0.057331183449)*x[1]
        arg[2,0,1,1]=(-0.154892482107)*x[0]+(0.481969957579)*x[1]
        arg[2,0,2,0]=(0.521916671496)*x[0]+(0.546676943059)*x[1]
        arg[2,0,2,1]=(-1.68919379028)*x[0]+(1.08292038981)*x[1]
        arg[2,1,0,0]=(-0.643846595888)*x[0]+(0.528306097755)*x[1]
        arg[2,1,0,1]=(1.3745090497)*x[0]+(-0.0801648509184)*x[1]
        arg[2,1,1,0]=(0.129096817054)*x[0]+(0.90750370855)*x[1]
        arg[2,1,1,1]=(0.539309513305)*x[0]+(1.34949470961)*x[1]
        arg[2,1,2,0]=(1.4189703896)*x[0]+(-1.75011201893)*x[1]
        arg[2,1,2,1]=(-1.09325548386)*x[0]+(0.148279301716)*x[1]
        arg[2,2,0,0]=(-1.25785133743)*x[0]+(0.275071306594)*x[1]
        arg[2,2,0,1]=(-0.550152068761)*x[0]+(-1.20422484275)*x[1]
        arg[2,2,1,0]=(-0.102062863668)*x[0]+(-1.34431887541)*x[1]
        arg[2,2,1,1]=(0.103354724169)*x[0]+(-0.941952180446)*x[1]
        arg[2,2,2,0]=(1.2090339604)*x[0]+(-0.330357976796)*x[1]
        arg[2,2,2,1]=(0.203162449188)*x[0]+(-1.03640702014)*x[1]
        arg[2,3,0,0]=(-0.324576353514)*x[0]+(0.582614539267)*x[1]
        arg[2,3,0,1]=(-0.213823136215)*x[0]+(-0.473199105214)*x[1]
        arg[2,3,1,0]=(0.395144702599)*x[0]+(0.0358194467152)*x[1]
        arg[2,3,1,1]=(1.32860563396)*x[0]+(-0.577308310732)*x[1]
        arg[2,3,2,0]=(0.994205029432)*x[0]+(1.13433517636)*x[1]
        arg[2,3,2,1]=(-0.488499286742)*x[0]+(0.224253820738)*x[1]
        arg[2,4,0,0]=(1.48035663008)*x[0]+(-1.09697986914)*x[1]
        arg[2,4,0,1]=(-1.09991915151)*x[0]+(-0.557951199985)*x[1]
        arg[2,4,1,0]=(-1.23862699804)*x[0]+(-0.230602187713)*x[1]
        arg[2,4,1,1]=(-0.922609370448)*x[0]+(1.76673953413)*x[1]
        arg[2,4,2,0]=(0.731769419709)*x[0]+(-1.1094516608)*x[1]
        arg[2,4,2,1]=(-0.145672504805)*x[0]+(0.168194421355)*x[1]
        arg[3,0,0,0]=(1.02559707602)*x[0]+(0.260431368523)*x[1]
        arg[3,0,0,1]=(0.111899568824)*x[0]+(-0.90725475423)*x[1]
        arg[3,0,1,0]=(-0.226547745683)*x[0]+(-0.232830393372)*x[1]
        arg[3,0,1,1]=(0.347004318471)*x[0]+(-0.196130970001)*x[1]
        arg[3,0,2,0]=(-1.72155871442)*x[0]+(-1.28372407452)*x[1]
        arg[3,0,2,1]=(1.43817914693)*x[0]+(-0.354093253684)*x[1]
        arg[3,1,0,0]=(-0.293939245102)*x[0]+(0.0255562139103)*x[1]
        arg[3,1,0,1]=(0.139306213018)*x[0]+(0.595062226642)*x[1]
        arg[3,1,1,0]=(-0.670339920672)*x[0]+(0.387520583192)*x[1]
        arg[3,1,1,1]=(-0.0323325470954)*x[0]+(-0.681162999279)*x[1]
        arg[3,1,2,0]=(0.235275048739)*x[0]+(0.144725473773)*x[1]
        arg[3,1,2,1]=(0.132211795214)*x[0]+(0.262780216845)*x[1]
        arg[3,2,0,0]=(0.590514326163)*x[0]+(-0.84273677918)*x[1]
        arg[3,2,0,1]=(-0.925181556185)*x[0]+(-0.300244447001)*x[1]
        arg[3,2,1,0]=(-0.0509700795112)*x[0]+(-1.31919389616)*x[1]
        arg[3,2,1,1]=(1.50577291846)*x[0]+(-0.219284607769)*x[1]
        arg[3,2,2,0]=(-0.0460616673346)*x[0]+(-0.690312685466)*x[1]
        arg[3,2,2,1]=(-0.673765477936)*x[0]+(-0.0965860475641)*x[1]
        arg[3,3,0,0]=(-0.6577594174)*x[0]+(0.171886277016)*x[1]
        arg[3,3,0,1]=(-1.08810434718)*x[0]+(0.333153250183)*x[1]
        arg[3,3,1,0]=(0.150968371983)*x[0]+(0.290859317391)*x[1]
        arg[3,3,1,1]=(1.62096265131)*x[0]+(-1.10309602741)*x[1]
        arg[3,3,2,0]=(-0.932063639972)*x[0]+(-0.897131561126)*x[1]
        arg[3,3,2,1]=(-0.819110834345)*x[0]+(1.59159109562)*x[1]
        arg[3,4,0,0]=(0.272662931155)*x[0]+(0.69026285756)*x[1]
        arg[3,4,0,1]=(-0.600556370757)*x[0]+(-0.0853533546302)*x[1]
        arg[3,4,1,0]=(1.18544735281)*x[0]+(0.217751189034)*x[1]
        arg[3,4,1,1]=(-0.594609531358)*x[0]+(0.882158349788)*x[1]
        arg[3,4,2,0]=(1.18876765552)*x[0]+(-0.411481586979)*x[1]
        arg[3,4,2,1]=(-1.093067792)*x[0]+(-0.266744260768)*x[1]
        ref[0,0,0,0]=(-0.754311658086)*x_ref[0]+(0.235157034342)*x_ref[1]
        ref[0,0,0,1]=(-1.33786315024)*x_ref[0]+(0.243349279742)*x_ref[1]
        ref[0,0,1,0]=(1.48657750088)*x_ref[0]+(1.09853108305)*x_ref[1]
        ref[0,0,1,1]=(1.4501362266)*x_ref[0]+(0.824811567155)*x_ref[1]
        ref[0,0,2,0]=(1.12567566082)*x_ref[0]+(0.663525681174)*x_ref[1]
        ref[0,0,2,1]=(1.48991565019)*x_ref[0]+(-0.680795365011)*x_ref[1]
        ref[0,1,0,0]=(0.268571449528)*x_ref[0]+(0.818595834672)*x_ref[1]
        ref[0,1,0,1]=(-1.55490945555)*x_ref[0]+(0.783595197888)*x_ref[1]
        ref[0,1,1,0]=(0.913201382585)*x_ref[0]+(0.855491379376)*x_ref[1]
        ref[0,1,1,1]=(-0.179192075116)*x_ref[0]+(-0.690415079523)*x_ref[1]
        ref[0,1,2,0]=(-0.0393574502912)*x_ref[0]+(-0.690157650634)*x_ref[1]
        ref[0,1,2,1]=(0.784216863217)*x_ref[0]+(0.498735486159)*x_ref[1]
        ref[0,2,0,0]=(0.137939554165)*x_ref[0]+(-1.33961519113)*x_ref[1]
        ref[0,2,0,1]=(-1.03897311736)*x_ref[0]+(-1.37797405312)*x_ref[1]
        ref[0,2,1,0]=(0.199812003756)*x_ref[0]+(-0.0388499674037)*x_ref[1]
        ref[0,2,1,1]=(1.15530534202)*x_ref[0]+(-1.28280869078)*x_ref[1]
        ref[0,2,2,0]=(0.836447791155)*x_ref[0]+(1.2627066026)*x_ref[1]
        ref[0,2,2,1]=(-0.584608492688)*x_ref[0]+(-1.55305974305)*x_ref[1]
        ref[0,3,0,0]=(-0.661952381799)*x_ref[0]+(-1.27062403778)*x_ref[1]
        ref[0,3,0,1]=(-1.47789532259)*x_ref[0]+(-0.279583524936)*x_ref[1]
        ref[0,3,1,0]=(1.40729771357)*x_ref[0]+(0.0971284144528)*x_ref[1]
        ref[0,3,1,1]=(-0.459167955911)*x_ref[0]+(-1.29033281524)*x_ref[1]
        ref[0,3,2,0]=(-1.63189436824)*x_ref[0]+(0.75729808422)*x_ref[1]
        ref[0,3,2,1]=(0.522479828718)*x_ref[0]+(-0.0231299515383)*x_ref[1]
        ref[0,4,0,0]=(0.241891446615)*x_ref[0]+(0.803327107651)*x_ref[1]
        ref[0,4,0,1]=(0.273613502836)*x_ref[0]+(-0.0115180464171)*x_ref[1]
        ref[0,4,1,0]=(1.96219818696)*x_ref[0]+(-0.349137094743)*x_ref[1]
        ref[0,4,1,1]=(0.365690821719)*x_ref[0]+(-1.6943347181)*x_ref[1]
        ref[0,4,2,0]=(-0.283970779872)*x_ref[0]+(-0.258537179804)*x_ref[1]
        ref[0,4,2,1]=(-0.973333430598)*x_ref[0]+(1.48706276212)*x_ref[1]
        ref[1,0,0,0]=(1.18757466971)*x_ref[0]+(0.838804656357)*x_ref[1]
        ref[1,0,0,1]=(0.840859904702)*x_ref[0]+(1.09955195682)*x_ref[1]
        ref[1,0,1,0]=(1.46190513227)*x_ref[0]+(-0.596959830422)*x_ref[1]
        ref[1,0,1,1]=(0.718689296745)*x_ref[0]+(-0.628074620222)*x_ref[1]
        ref[1,0,2,0]=(0.440761293041)*x_ref[0]+(0.115800854194)*x_ref[1]
        ref[1,0,2,1]=(1.34891146904)*x_ref[0]+(0.813690260652)*x_ref[1]
        ref[1,1,0,0]=(0.183071638098)*x_ref[0]+(-1.37996210755)*x_ref[1]
        ref[1,1,0,1]=(0.258142115988)*x_ref[0]+(-0.757709381529)*x_ref[1]
        ref[1,1,1,0]=(1.72392829309)*x_ref[0]+(0.444995102781)*x_ref[1]
        ref[1,1,1,1]=(-0.380898209988)*x_ref[0]+(-0.0950246860347)*x_ref[1]
        ref[1,1,2,0]=(0.203007226016)*x_ref[0]+(1.09786001832)*x_ref[1]
        ref[1,1,2,1]=(-0.502500865341)*x_ref[0]+(-0.629980527995)*x_ref[1]
        ref[1,2,0,0]=(0.482549518436)*x_ref[0]+(0.565840824146)*x_ref[1]
        ref[1,2,0,1]=(0.355818565068)*x_ref[0]+(-0.78273295574)*x_ref[1]
        ref[1,2,1,0]=(1.79204033071)*x_ref[0]+(-0.698040851585)*x_ref[1]
        ref[1,2,1,1]=(-0.534827741044)*x_ref[0]+(0.00943131627498)*x_ref[1]
        ref[1,2,2,0]=(-1.00663524444)*x_ref[0]+(0.0207854553456)*x_ref[1]
        ref[1,2,2,1]=(-0.338048040159)*x_ref[0]+(-0.380607406217)*x_ref[1]
        ref[1,3,0,0]=(0.586310549044)*x_ref[0]+(0.276326798805)*x_ref[1]
        ref[1,3,0,1]=(-0.260482580702)*x_ref[0]+(-0.349279395321)*x_ref[1]
        ref[1,3,1,0]=(-0.196533829895)*x_ref[0]+(0.614805272738)*x_ref[1]
        ref[1,3,1,1]=(-0.363176483895)*x_ref[0]+(-0.925817017731)*x_ref[1]
        ref[1,3,2,0]=(-1.24182075421)*x_ref[0]+(0.215966446013)*x_ref[1]
        ref[1,3,2,1]=(1.07546652642)*x_ref[0]+(0.383803991974)*x_ref[1]
        ref[1,4,0,0]=(0.199312303525)*x_ref[0]+(-0.314520883859)*x_ref[1]
        ref[1,4,0,1]=(-1.56890826152)*x_ref[0]+(-0.133086662241)*x_ref[1]
        ref[1,4,1,0]=(1.1307241936)*x_ref[0]+(-1.32905744984)*x_ref[1]
        ref[1,4,1,1]=(-0.385991455298)*x_ref[0]+(0.292913024507)*x_ref[1]
        ref[1,4,2,0]=(1.14561678056)*x_ref[0]+(-0.662696162276)*x_ref[1]
        ref[1,4,2,1]=(-1.49533948954)*x_ref[0]+(1.43396138744)*x_ref[1]
        ref[2,0,0,0]=(-0.63160396493)*x_ref[0]+(0.813120998776)*x_ref[1]
        ref[2,0,0,1]=(0.65598998434)*x_ref[0]+(-1.02445537553)*x_ref[1]
        ref[2,0,1,0]=(-1.13473104075)*x_ref[0]+(0.057331183449)*x_ref[1]
        ref[2,0,1,1]=(-0.154892482107)*x_ref[0]+(0.481969957579)*x_ref[1]
        ref[2,0,2,0]=(0.521916671496)*x_ref[0]+(0.546676943059)*x_ref[1]
        ref[2,0,2,1]=(-1.68919379028)*x_ref[0]+(1.08292038981)*x_ref[1]
        ref[2,1,0,0]=(-0.643846595888)*x_ref[0]+(0.528306097755)*x_ref[1]
        ref[2,1,0,1]=(1.3745090497)*x_ref[0]+(-0.0801648509184)*x_ref[1]
        ref[2,1,1,0]=(0.129096817054)*x_ref[0]+(0.90750370855)*x_ref[1]
        ref[2,1,1,1]=(0.539309513305)*x_ref[0]+(1.34949470961)*x_ref[1]
        ref[2,1,2,0]=(1.4189703896)*x_ref[0]+(-1.75011201893)*x_ref[1]
        ref[2,1,2,1]=(-1.09325548386)*x_ref[0]+(0.148279301716)*x_ref[1]
        ref[2,2,0,0]=(-1.25785133743)*x_ref[0]+(0.275071306594)*x_ref[1]
        ref[2,2,0,1]=(-0.550152068761)*x_ref[0]+(-1.20422484275)*x_ref[1]
        ref[2,2,1,0]=(-0.102062863668)*x_ref[0]+(-1.34431887541)*x_ref[1]
        ref[2,2,1,1]=(0.103354724169)*x_ref[0]+(-0.941952180446)*x_ref[1]
        ref[2,2,2,0]=(1.2090339604)*x_ref[0]+(-0.330357976796)*x_ref[1]
        ref[2,2,2,1]=(0.203162449188)*x_ref[0]+(-1.03640702014)*x_ref[1]
        ref[2,3,0,0]=(-0.324576353514)*x_ref[0]+(0.582614539267)*x_ref[1]
        ref[2,3,0,1]=(-0.213823136215)*x_ref[0]+(-0.473199105214)*x_ref[1]
        ref[2,3,1,0]=(0.395144702599)*x_ref[0]+(0.0358194467152)*x_ref[1]
        ref[2,3,1,1]=(1.32860563396)*x_ref[0]+(-0.577308310732)*x_ref[1]
        ref[2,3,2,0]=(0.994205029432)*x_ref[0]+(1.13433517636)*x_ref[1]
        ref[2,3,2,1]=(-0.488499286742)*x_ref[0]+(0.224253820738)*x_ref[1]
        ref[2,4,0,0]=(1.48035663008)*x_ref[0]+(-1.09697986914)*x_ref[1]
        ref[2,4,0,1]=(-1.09991915151)*x_ref[0]+(-0.557951199985)*x_ref[1]
        ref[2,4,1,0]=(-1.23862699804)*x_ref[0]+(-0.230602187713)*x_ref[1]
        ref[2,4,1,1]=(-0.922609370448)*x_ref[0]+(1.76673953413)*x_ref[1]
        ref[2,4,2,0]=(0.731769419709)*x_ref[0]+(-1.1094516608)*x_ref[1]
        ref[2,4,2,1]=(-0.145672504805)*x_ref[0]+(0.168194421355)*x_ref[1]
        ref[3,0,0,0]=(1.02559707602)*x_ref[0]+(0.260431368523)*x_ref[1]
        ref[3,0,0,1]=(0.111899568824)*x_ref[0]+(-0.90725475423)*x_ref[1]
        ref[3,0,1,0]=(-0.226547745683)*x_ref[0]+(-0.232830393372)*x_ref[1]
        ref[3,0,1,1]=(0.347004318471)*x_ref[0]+(-0.196130970001)*x_ref[1]
        ref[3,0,2,0]=(-1.72155871442)*x_ref[0]+(-1.28372407452)*x_ref[1]
        ref[3,0,2,1]=(1.43817914693)*x_ref[0]+(-0.354093253684)*x_ref[1]
        ref[3,1,0,0]=(-0.293939245102)*x_ref[0]+(0.0255562139103)*x_ref[1]
        ref[3,1,0,1]=(0.139306213018)*x_ref[0]+(0.595062226642)*x_ref[1]
        ref[3,1,1,0]=(-0.670339920672)*x_ref[0]+(0.387520583192)*x_ref[1]
        ref[3,1,1,1]=(-0.0323325470954)*x_ref[0]+(-0.681162999279)*x_ref[1]
        ref[3,1,2,0]=(0.235275048739)*x_ref[0]+(0.144725473773)*x_ref[1]
        ref[3,1,2,1]=(0.132211795214)*x_ref[0]+(0.262780216845)*x_ref[1]
        ref[3,2,0,0]=(0.590514326163)*x_ref[0]+(-0.84273677918)*x_ref[1]
        ref[3,2,0,1]=(-0.925181556185)*x_ref[0]+(-0.300244447001)*x_ref[1]
        ref[3,2,1,0]=(-0.0509700795112)*x_ref[0]+(-1.31919389616)*x_ref[1]
        ref[3,2,1,1]=(1.50577291846)*x_ref[0]+(-0.219284607769)*x_ref[1]
        ref[3,2,2,0]=(-0.0460616673346)*x_ref[0]+(-0.690312685466)*x_ref[1]
        ref[3,2,2,1]=(-0.673765477936)*x_ref[0]+(-0.0965860475641)*x_ref[1]
        ref[3,3,0,0]=(-0.6577594174)*x_ref[0]+(0.171886277016)*x_ref[1]
        ref[3,3,0,1]=(-1.08810434718)*x_ref[0]+(0.333153250183)*x_ref[1]
        ref[3,3,1,0]=(0.150968371983)*x_ref[0]+(0.290859317391)*x_ref[1]
        ref[3,3,1,1]=(1.62096265131)*x_ref[0]+(-1.10309602741)*x_ref[1]
        ref[3,3,2,0]=(-0.932063639972)*x_ref[0]+(-0.897131561126)*x_ref[1]
        ref[3,3,2,1]=(-0.819110834345)*x_ref[0]+(1.59159109562)*x_ref[1]
        ref[3,4,0,0]=(0.272662931155)*x_ref[0]+(0.69026285756)*x_ref[1]
        ref[3,4,0,1]=(-0.600556370757)*x_ref[0]+(-0.0853533546302)*x_ref[1]
        ref[3,4,1,0]=(1.18544735281)*x_ref[0]+(0.217751189034)*x_ref[1]
        ref[3,4,1,1]=(-0.594609531358)*x_ref[0]+(0.882158349788)*x_ref[1]
        ref[3,4,2,0]=(1.18876765552)*x_ref[0]+(-0.411481586979)*x_ref[1]
        ref[3,4,2,1]=(-1.093067792)*x_ref[0]+(-0.266744260768)*x_ref[1]
      else:
        arg[0,0,0,0]=(-1.29218217256)*x[0]+(-0.382937892213)*x[1]+(0.14363896447)*x[2]
        arg[0,0,0,1]=(1.39856942421)*x[0]+(0.302082267835)*x[1]+(-0.40870736584)*x[2]
        arg[0,0,1,0]=(0.523846108043)*x[0]+(1.19961531204)*x[1]+(0.838881761349)*x[2]
        arg[0,0,1,1]=(0.634542282764)*x[0]+(-0.475514686466)*x[1]+(-1.68745428486)*x[2]
        arg[0,0,2,0]=(1.16379585731)*x[0]+(-0.928037595416)*x[1]+(0.998857178353)*x[2]
        arg[0,0,2,1]=(-0.804515607022)*x[0]+(-0.915469493439)*x[1]+(-0.143888000623)*x[2]
        arg[0,1,0,0]=(0.925843155486)*x[0]+(-0.168243418953)*x[1]+(-0.887839555187)*x[2]
        arg[0,1,0,1]=(-0.913103105329)*x[0]+(1.38011681732)*x[1]+(-0.992478793291)*x[2]
        arg[0,1,1,0]=(0.759685289903)*x[0]+(-0.579481939982)*x[1]+(0.577682929138)*x[2]
        arg[0,1,1,1]=(0.813155548228)*x[0]+(-0.178317843217)*x[1]+(-0.0460113194526)*x[2]
        arg[0,1,2,0]=(-0.195764860141)*x[0]+(1.44256979628)*x[1]+(0.283992441343)*x[2]
        arg[0,1,2,1]=(-0.921229459072)*x[0]+(-1.19295761547)*x[1]+(0.215736801944)*x[2]
        arg[0,2,0,0]=(-0.741726331752)*x[0]+(-0.225585937382)*x[1]+(1.12654556466)*x[2]
        arg[0,2,0,1]=(-0.709656162421)*x[0]+(0.298862010425)*x[1]+(-0.502443284827)*x[2]
        arg[0,2,1,0]=(-1.01166188669)*x[0]+(-1.43763408953)*x[1]+(-0.835160398488)*x[2]
        arg[0,2,1,1]=(1.80968424545)*x[0]+(-0.802666103495)*x[1]+(0.290283606115)*x[2]
        arg[0,2,2,0]=(0.346481981585)*x[0]+(-0.444176962215)*x[1]+(0.689127612227)*x[2]
        arg[0,2,2,1]=(1.06925494095)*x[0]+(0.452854988449)*x[1]+(-1.19974690364)*x[2]
        arg[0,3,0,0]=(-0.534896036328)*x[0]+(-0.312000770615)*x[1]+(-0.440033746693)*x[2]
        arg[0,3,0,1]=(0.580351431838)*x[0]+(-0.36623848451)*x[1]+(-0.507837490037)*x[2]
        arg[0,3,1,0]=(0.383180910522)*x[0]+(-0.0907152443563)*x[1]+(-0.200103743943)*x[2]
        arg[0,3,1,1]=(0.0160194687513)*x[0]+(0.862509612939)*x[1]+(-1.14643594474)*x[2]
        arg[0,3,2,0]=(-0.224390850199)*x[0]+(-0.176927715447)*x[1]+(0.213480525831)*x[2]
        arg[0,3,2,1]=(-0.0508511448148)*x[0]+(-1.28943467389)*x[1]+(0.902285302487)*x[2]
        arg[0,4,0,0]=(-0.633106625232)*x[0]+(-0.430651917485)*x[1]+(0.583149841952)*x[2]
        arg[0,4,0,1]=(0.579035528956)*x[0]+(0.159439682519)*x[1]+(-0.0926277063021)*x[2]
        arg[0,4,1,0]=(0.00484756366973)*x[0]+(0.0324822270755)*x[1]+(1.20052413378)*x[2]
        arg[0,4,1,1]=(-0.337383994688)*x[0]+(0.902076367981)*x[1]+(-0.1956377395)*x[2]
        arg[0,4,2,0]=(0.928434343456)*x[0]+(0.0376569884355)*x[1]+(-0.155274118568)*x[2]
        arg[0,4,2,1]=(-0.541990213238)*x[0]+(-0.0165076697829)*x[1]+(-0.210319224859)*x[2]
        arg[1,0,0,0]=(-1.67762025181)*x[0]+(1.61434755579)*x[1]+(0.357272049999)*x[2]
        arg[1,0,0,1]=(-1.62532544549)*x[0]+(-0.271426231027)*x[1]+(0.338680991778)*x[2]
        arg[1,0,1,0]=(1.73523408033)*x[0]+(1.11987092368)*x[1]+(0.126769364273)*x[2]
        arg[1,0,1,1]=(-1.49220786558)*x[0]+(1.51915566092)*x[1]+(1.24788192157)*x[2]
        arg[1,0,2,0]=(-0.687808766765)*x[0]+(-1.02318754737)*x[1]+(0.590653972416)*x[2]
        arg[1,0,2,1]=(0.756067354802)*x[0]+(-0.66964706965)*x[1]+(-1.18986499158)*x[2]
        arg[1,1,0,0]=(0.371345732177)*x[0]+(0.212739617115)*x[1]+(-1.13439047405)*x[2]
        arg[1,1,0,1]=(0.525761804111)*x[0]+(0.144130399691)*x[1]+(0.715517587677)*x[2]
        arg[1,1,1,0]=(-0.376160293215)*x[0]+(-0.469964935263)*x[1]+(0.413518194707)*x[2]
        arg[1,1,1,1]=(-0.787748398144)*x[0]+(-0.810090402237)*x[1]+(-0.50127109256)*x[2]
        arg[1,1,2,0]=(0.654555083692)*x[0]+(-0.142134700515)*x[1]+(-1.55891930521)*x[2]
        arg[1,1,2,1]=(-0.423058885456)*x[0]+(-0.65930158641)*x[1]+(0.94440057744)*x[2]
        arg[1,2,0,0]=(-0.924832007053)*x[0]+(1.0154890382)*x[1]+(0.11938079748)*x[2]
        arg[1,2,0,1]=(0.134642686931)*x[0]+(-1.30908067998)*x[1]+(0.367206139074)*x[2]
        arg[1,2,1,0]=(1.55524055201)*x[0]+(0.727208660637)*x[1]+(0.809810917377)*x[2]
        arg[1,2,1,1]=(0.94813391462)*x[0]+(-1.49402524308)*x[1]+(0.367824542434)*x[2]
        arg[1,2,2,0]=(-0.608295798731)*x[0]+(-0.142302252814)*x[1]+(-0.986529251308)*x[2]
        arg[1,2,2,1]=(0.0259394619607)*x[0]+(-1.60728202467)*x[1]+(0.312534656619)*x[2]
        arg[1,3,0,0]=(-1.34063373729)*x[0]+(-0.268499839329)*x[1]+(-0.335125886299)*x[2]
        arg[1,3,0,1]=(1.53280698072)*x[0]+(1.24369684711)*x[1]+(0.766369156132)*x[2]
        arg[1,3,1,0]=(0.902067176876)*x[0]+(0.791102994814)*x[1]+(-0.89266430493)*x[2]
        arg[1,3,1,1]=(0.704997629232)*x[0]+(-0.551611360332)*x[1]+(1.39134537491)*x[2]
        arg[1,3,2,0]=(-0.151690675211)*x[0]+(-1.01762975893)*x[1]+(-0.627493214284)*x[2]
        arg[1,3,2,1]=(1.00936645922)*x[0]+(0.0322458997248)*x[1]+(1.45039938194)*x[2]
        arg[1,4,0,0]=(-0.0916390388904)*x[0]+(-0.783111746469)*x[1]+(0.389725342225)*x[2]
        arg[1,4,0,1]=(1.18859328243)*x[0]+(0.871689661558)*x[1]+(-0.775464713448)*x[2]
        arg[1,4,1,0]=(0.913118342488)*x[0]+(1.53285648884)*x[1]+(0.0947365101243)*x[2]
        arg[1,4,1,1]=(-1.00735841224)*x[0]+(-1.12544193607)*x[1]+(0.267347229372)*x[2]
        arg[1,4,2,0]=(0.285943438284)*x[0]+(-1.0889389244)*x[1]+(-1.03054842327)*x[2]
        arg[1,4,2,1]=(-1.5768940139)*x[0]+(0.91237754587)*x[1]+(1.30220394199)*x[2]
        arg[2,0,0,0]=(-0.0634935883277)*x[0]+(0.157304131712)*x[1]+(-1.2995042074)*x[2]
        arg[2,0,0,1]=(-0.802705266871)*x[0]+(0.238233162774)*x[1]+(1.29463414013)*x[2]
        arg[2,0,1,0]=(-0.925468462671)*x[0]+(0.870960795966)*x[1]+(0.156522765713)*x[2]
        arg[2,0,1,1]=(-0.546666833309)*x[0]+(-1.14649607995)*x[1]+(-1.53906505048)*x[2]
        arg[2,0,2,0]=(0.464952910857)*x[0]+(0.408605786027)*x[1]+(-0.886974881114)*x[2]
        arg[2,0,2,1]=(0.491723247948)*x[0]+(0.194616420077)*x[1]+(0.409049782749)*x[2]
        arg[2,1,0,0]=(-1.19836883114)*x[0]+(1.0622722337)*x[1]+(-0.109193867727)*x[2]
        arg[2,1,0,1]=(1.01973600383)*x[0]+(-0.154482220539)*x[1]+(0.378696371956)*x[2]
        arg[2,1,1,0]=(-0.0812015296246)*x[0]+(-1.10252794568)*x[1]+(-0.00585426581857)*x[2]
        arg[2,1,1,1]=(0.536042665272)*x[0]+(0.685507117255)*x[1]+(-0.179652683139)*x[2]
        arg[2,1,2,0]=(-0.36750467811)*x[0]+(-1.23674550869)*x[1]+(-0.762905702801)*x[2]
        arg[2,1,2,1]=(0.329665965651)*x[0]+(0.101195810737)*x[1]+(-0.610015052375)*x[2]
        arg[2,2,0,0]=(-0.81644635185)*x[0]+(-0.604578476306)*x[1]+(0.0440925706991)*x[2]
        arg[2,2,0,1]=(-0.958241437971)*x[0]+(0.38563498206)*x[1]+(0.913406762318)*x[2]
        arg[2,2,1,0]=(1.22027492224)*x[0]+(-1.00726209939)*x[1]+(-1.05083068303)*x[2]
        arg[2,2,1,1]=(-1.47561192645)*x[0]+(-0.903549355608)*x[1]+(0.236771324671)*x[2]
        arg[2,2,2,0]=(0.284995864449)*x[0]+(-0.175760829921)*x[1]+(-0.720607220001)*x[2]
        arg[2,2,2,1]=(-0.535894515963)*x[0]+(-0.821302957162)*x[1]+(-1.75141540241)*x[2]
        arg[2,3,0,0]=(-0.333082106674)*x[0]+(0.579471826412)*x[1]+(0.561184138108)*x[2]
        arg[2,3,0,1]=(1.34803519557)*x[0]+(-0.441299088849)*x[1]+(-1.04945951616)*x[2]
        arg[2,3,1,0]=(0.299011096602)*x[0]+(1.32726458235)*x[1]+(-0.196298687359)*x[2]
        arg[2,3,1,1]=(1.48698834204)*x[0]+(0.527221496971)*x[1]+(0.0819813519422)*x[2]
        arg[2,3,2,0]=(0.681680955153)*x[0]+(0.503653972357)*x[1]+(0.434669347891)*x[2]
        arg[2,3,2,1]=(-0.152586789311)*x[0]+(-0.765242635579)*x[1]+(0.712663994387)*x[2]
        arg[2,4,0,0]=(0.630152300463)*x[0]+(1.12366733588)*x[1]+(-0.662966700638)*x[2]
        arg[2,4,0,1]=(1.02796709755)*x[0]+(-0.24093881584)*x[1]+(1.79445417019)*x[2]
        arg[2,4,1,0]=(-0.954575760758)*x[0]+(0.16331782608)*x[1]+(1.38309921277)*x[2]
        arg[2,4,1,1]=(0.536092514281)*x[0]+(-0.276364952673)*x[1]+(0.779619701528)*x[2]
        arg[2,4,2,0]=(0.267270444141)*x[0]+(-0.810746424331)*x[1]+(-0.733877853264)*x[2]
        arg[2,4,2,1]=(-0.0641688847923)*x[0]+(-0.386279280667)*x[1]+(0.0637571595255)*x[2]
        arg[3,0,0,0]=(0.469903180687)*x[0]+(-0.793812397342)*x[1]+(1.60841921779)*x[2]
        arg[3,0,0,1]=(0.720057600102)*x[0]+(0.502582253261)*x[1]+(0.460147791003)*x[2]
        arg[3,0,1,0]=(0.0327862564137)*x[0]+(-1.142962965)*x[1]+(0.114799131894)*x[2]
        arg[3,0,1,1]=(-0.997412656978)*x[0]+(-0.678175468576)*x[1]+(-0.152663147682)*x[2]
        arg[3,0,2,0]=(0.810424599078)*x[0]+(0.75522727786)*x[1]+(-0.205880941501)*x[2]
        arg[3,0,2,1]=(0.353601949476)*x[0]+(0.0963200813379)*x[1]+(0.0578829287146)*x[2]
        arg[3,1,0,0]=(0.346590730062)*x[0]+(1.07953790092)*x[1]+(0.647586851217)*x[2]
        arg[3,1,0,1]=(0.889879195334)*x[0]+(-0.352788186249)*x[1]+(0.434900135803)*x[2]
        arg[3,1,1,0]=(-0.721742734959)*x[0]+(0.166066672134)*x[1]+(-0.489266924809)*x[2]
        arg[3,1,1,1]=(-0.810487755181)*x[0]+(0.0354916979253)*x[1]+(0.0735520654395)*x[2]
        arg[3,1,2,0]=(1.02414338939)*x[0]+(-0.851076284556)*x[1]+(-0.232174494405)*x[2]
        arg[3,1,2,1]=(-0.327246566026)*x[0]+(-0.0499396023605)*x[1]+(0.611585402379)*x[2]
        arg[3,2,0,0]=(-0.365757628557)*x[0]+(0.868697342115)*x[1]+(-1.102545972)*x[2]
        arg[3,2,0,1]=(0.628592129227)*x[0]+(1.25545515311)*x[1]+(-1.0577476264)*x[2]
        arg[3,2,1,0]=(0.160995978044)*x[0]+(-0.947389499556)*x[1]+(-0.870476160656)*x[2]
        arg[3,2,1,1]=(0.910787139938)*x[0]+(-0.0681493345139)*x[1]+(-0.561910153781)*x[2]
        arg[3,2,2,0]=(0.0761644634428)*x[0]+(-0.700940967343)*x[1]+(1.19407210904)*x[2]
        arg[3,2,2,1]=(1.75717437065)*x[0]+(-0.144457317494)*x[1]+(-0.523233453079)*x[2]
        arg[3,3,0,0]=(0.448782872606)*x[0]+(0.650464256547)*x[1]+(1.19476547456)*x[2]
        arg[3,3,0,1]=(0.118586027201)*x[0]+(0.449971816918)*x[1]+(-0.510812733389)*x[2]
        arg[3,3,1,0]=(-0.330032265081)*x[0]+(-0.13657534891)*x[1]+(0.456365289556)*x[2]
        arg[3,3,1,1]=(0.697584212635)*x[0]+(0.897604241724)*x[1]+(1.31517116663)*x[2]
        arg[3,3,2,0]=(-0.940538707365)*x[0]+(-0.334862221665)*x[1]+(0.0969742096754)*x[2]
        arg[3,3,2,1]=(-0.985788813516)*x[0]+(0.348389725548)*x[1]+(-0.260816590361)*x[2]
        arg[3,4,0,0]=(0.754692421417)*x[0]+(0.775985781769)*x[1]+(-0.0866159222457)*x[2]
        arg[3,4,0,1]=(-1.17670502429)*x[0]+(1.12268279393)*x[1]+(-1.1279822095)*x[2]
        arg[3,4,1,0]=(-1.48674370666)*x[0]+(1.26685445713)*x[1]+(-0.0522828809306)*x[2]
        arg[3,4,1,1]=(0.401760689843)*x[0]+(-1.35707797971)*x[1]+(-0.0580654273828)*x[2]
        arg[3,4,2,0]=(-1.40697830852)*x[0]+(0.933923369958)*x[1]+(0.230152169159)*x[2]
        arg[3,4,2,1]=(-1.21242130555)*x[0]+(0.858316339663)*x[1]+(-0.839747078526)*x[2]
        ref[0,0,0,0]=(-1.29218217256)*x_ref[0]+(-0.382937892213)*x_ref[1]+(0.14363896447)*x_ref[2]
        ref[0,0,0,1]=(1.39856942421)*x_ref[0]+(0.302082267835)*x_ref[1]+(-0.40870736584)*x_ref[2]
        ref[0,0,1,0]=(0.523846108043)*x_ref[0]+(1.19961531204)*x_ref[1]+(0.838881761349)*x_ref[2]
        ref[0,0,1,1]=(0.634542282764)*x_ref[0]+(-0.475514686466)*x_ref[1]+(-1.68745428486)*x_ref[2]
        ref[0,0,2,0]=(1.16379585731)*x_ref[0]+(-0.928037595416)*x_ref[1]+(0.998857178353)*x_ref[2]
        ref[0,0,2,1]=(-0.804515607022)*x_ref[0]+(-0.915469493439)*x_ref[1]+(-0.143888000623)*x_ref[2]
        ref[0,1,0,0]=(0.925843155486)*x_ref[0]+(-0.168243418953)*x_ref[1]+(-0.887839555187)*x_ref[2]
        ref[0,1,0,1]=(-0.913103105329)*x_ref[0]+(1.38011681732)*x_ref[1]+(-0.992478793291)*x_ref[2]
        ref[0,1,1,0]=(0.759685289903)*x_ref[0]+(-0.579481939982)*x_ref[1]+(0.577682929138)*x_ref[2]
        ref[0,1,1,1]=(0.813155548228)*x_ref[0]+(-0.178317843217)*x_ref[1]+(-0.0460113194526)*x_ref[2]
        ref[0,1,2,0]=(-0.195764860141)*x_ref[0]+(1.44256979628)*x_ref[1]+(0.283992441343)*x_ref[2]
        ref[0,1,2,1]=(-0.921229459072)*x_ref[0]+(-1.19295761547)*x_ref[1]+(0.215736801944)*x_ref[2]
        ref[0,2,0,0]=(-0.741726331752)*x_ref[0]+(-0.225585937382)*x_ref[1]+(1.12654556466)*x_ref[2]
        ref[0,2,0,1]=(-0.709656162421)*x_ref[0]+(0.298862010425)*x_ref[1]+(-0.502443284827)*x_ref[2]
        ref[0,2,1,0]=(-1.01166188669)*x_ref[0]+(-1.43763408953)*x_ref[1]+(-0.835160398488)*x_ref[2]
        ref[0,2,1,1]=(1.80968424545)*x_ref[0]+(-0.802666103495)*x_ref[1]+(0.290283606115)*x_ref[2]
        ref[0,2,2,0]=(0.346481981585)*x_ref[0]+(-0.444176962215)*x_ref[1]+(0.689127612227)*x_ref[2]
        ref[0,2,2,1]=(1.06925494095)*x_ref[0]+(0.452854988449)*x_ref[1]+(-1.19974690364)*x_ref[2]
        ref[0,3,0,0]=(-0.534896036328)*x_ref[0]+(-0.312000770615)*x_ref[1]+(-0.440033746693)*x_ref[2]
        ref[0,3,0,1]=(0.580351431838)*x_ref[0]+(-0.36623848451)*x_ref[1]+(-0.507837490037)*x_ref[2]
        ref[0,3,1,0]=(0.383180910522)*x_ref[0]+(-0.0907152443563)*x_ref[1]+(-0.200103743943)*x_ref[2]
        ref[0,3,1,1]=(0.0160194687513)*x_ref[0]+(0.862509612939)*x_ref[1]+(-1.14643594474)*x_ref[2]
        ref[0,3,2,0]=(-0.224390850199)*x_ref[0]+(-0.176927715447)*x_ref[1]+(0.213480525831)*x_ref[2]
        ref[0,3,2,1]=(-0.0508511448148)*x_ref[0]+(-1.28943467389)*x_ref[1]+(0.902285302487)*x_ref[2]
        ref[0,4,0,0]=(-0.633106625232)*x_ref[0]+(-0.430651917485)*x_ref[1]+(0.583149841952)*x_ref[2]
        ref[0,4,0,1]=(0.579035528956)*x_ref[0]+(0.159439682519)*x_ref[1]+(-0.0926277063021)*x_ref[2]
        ref[0,4,1,0]=(0.00484756366973)*x_ref[0]+(0.0324822270755)*x_ref[1]+(1.20052413378)*x_ref[2]
        ref[0,4,1,1]=(-0.337383994688)*x_ref[0]+(0.902076367981)*x_ref[1]+(-0.1956377395)*x_ref[2]
        ref[0,4,2,0]=(0.928434343456)*x_ref[0]+(0.0376569884355)*x_ref[1]+(-0.155274118568)*x_ref[2]
        ref[0,4,2,1]=(-0.541990213238)*x_ref[0]+(-0.0165076697829)*x_ref[1]+(-0.210319224859)*x_ref[2]
        ref[1,0,0,0]=(-1.67762025181)*x_ref[0]+(1.61434755579)*x_ref[1]+(0.357272049999)*x_ref[2]
        ref[1,0,0,1]=(-1.62532544549)*x_ref[0]+(-0.271426231027)*x_ref[1]+(0.338680991778)*x_ref[2]
        ref[1,0,1,0]=(1.73523408033)*x_ref[0]+(1.11987092368)*x_ref[1]+(0.126769364273)*x_ref[2]
        ref[1,0,1,1]=(-1.49220786558)*x_ref[0]+(1.51915566092)*x_ref[1]+(1.24788192157)*x_ref[2]
        ref[1,0,2,0]=(-0.687808766765)*x_ref[0]+(-1.02318754737)*x_ref[1]+(0.590653972416)*x_ref[2]
        ref[1,0,2,1]=(0.756067354802)*x_ref[0]+(-0.66964706965)*x_ref[1]+(-1.18986499158)*x_ref[2]
        ref[1,1,0,0]=(0.371345732177)*x_ref[0]+(0.212739617115)*x_ref[1]+(-1.13439047405)*x_ref[2]
        ref[1,1,0,1]=(0.525761804111)*x_ref[0]+(0.144130399691)*x_ref[1]+(0.715517587677)*x_ref[2]
        ref[1,1,1,0]=(-0.376160293215)*x_ref[0]+(-0.469964935263)*x_ref[1]+(0.413518194707)*x_ref[2]
        ref[1,1,1,1]=(-0.787748398144)*x_ref[0]+(-0.810090402237)*x_ref[1]+(-0.50127109256)*x_ref[2]
        ref[1,1,2,0]=(0.654555083692)*x_ref[0]+(-0.142134700515)*x_ref[1]+(-1.55891930521)*x_ref[2]
        ref[1,1,2,1]=(-0.423058885456)*x_ref[0]+(-0.65930158641)*x_ref[1]+(0.94440057744)*x_ref[2]
        ref[1,2,0,0]=(-0.924832007053)*x_ref[0]+(1.0154890382)*x_ref[1]+(0.11938079748)*x_ref[2]
        ref[1,2,0,1]=(0.134642686931)*x_ref[0]+(-1.30908067998)*x_ref[1]+(0.367206139074)*x_ref[2]
        ref[1,2,1,0]=(1.55524055201)*x_ref[0]+(0.727208660637)*x_ref[1]+(0.809810917377)*x_ref[2]
        ref[1,2,1,1]=(0.94813391462)*x_ref[0]+(-1.49402524308)*x_ref[1]+(0.367824542434)*x_ref[2]
        ref[1,2,2,0]=(-0.608295798731)*x_ref[0]+(-0.142302252814)*x_ref[1]+(-0.986529251308)*x_ref[2]
        ref[1,2,2,1]=(0.0259394619607)*x_ref[0]+(-1.60728202467)*x_ref[1]+(0.312534656619)*x_ref[2]
        ref[1,3,0,0]=(-1.34063373729)*x_ref[0]+(-0.268499839329)*x_ref[1]+(-0.335125886299)*x_ref[2]
        ref[1,3,0,1]=(1.53280698072)*x_ref[0]+(1.24369684711)*x_ref[1]+(0.766369156132)*x_ref[2]
        ref[1,3,1,0]=(0.902067176876)*x_ref[0]+(0.791102994814)*x_ref[1]+(-0.89266430493)*x_ref[2]
        ref[1,3,1,1]=(0.704997629232)*x_ref[0]+(-0.551611360332)*x_ref[1]+(1.39134537491)*x_ref[2]
        ref[1,3,2,0]=(-0.151690675211)*x_ref[0]+(-1.01762975893)*x_ref[1]+(-0.627493214284)*x_ref[2]
        ref[1,3,2,1]=(1.00936645922)*x_ref[0]+(0.0322458997248)*x_ref[1]+(1.45039938194)*x_ref[2]
        ref[1,4,0,0]=(-0.0916390388904)*x_ref[0]+(-0.783111746469)*x_ref[1]+(0.389725342225)*x_ref[2]
        ref[1,4,0,1]=(1.18859328243)*x_ref[0]+(0.871689661558)*x_ref[1]+(-0.775464713448)*x_ref[2]
        ref[1,4,1,0]=(0.913118342488)*x_ref[0]+(1.53285648884)*x_ref[1]+(0.0947365101243)*x_ref[2]
        ref[1,4,1,1]=(-1.00735841224)*x_ref[0]+(-1.12544193607)*x_ref[1]+(0.267347229372)*x_ref[2]
        ref[1,4,2,0]=(0.285943438284)*x_ref[0]+(-1.0889389244)*x_ref[1]+(-1.03054842327)*x_ref[2]
        ref[1,4,2,1]=(-1.5768940139)*x_ref[0]+(0.91237754587)*x_ref[1]+(1.30220394199)*x_ref[2]
        ref[2,0,0,0]=(-0.0634935883277)*x_ref[0]+(0.157304131712)*x_ref[1]+(-1.2995042074)*x_ref[2]
        ref[2,0,0,1]=(-0.802705266871)*x_ref[0]+(0.238233162774)*x_ref[1]+(1.29463414013)*x_ref[2]
        ref[2,0,1,0]=(-0.925468462671)*x_ref[0]+(0.870960795966)*x_ref[1]+(0.156522765713)*x_ref[2]
        ref[2,0,1,1]=(-0.546666833309)*x_ref[0]+(-1.14649607995)*x_ref[1]+(-1.53906505048)*x_ref[2]
        ref[2,0,2,0]=(0.464952910857)*x_ref[0]+(0.408605786027)*x_ref[1]+(-0.886974881114)*x_ref[2]
        ref[2,0,2,1]=(0.491723247948)*x_ref[0]+(0.194616420077)*x_ref[1]+(0.409049782749)*x_ref[2]
        ref[2,1,0,0]=(-1.19836883114)*x_ref[0]+(1.0622722337)*x_ref[1]+(-0.109193867727)*x_ref[2]
        ref[2,1,0,1]=(1.01973600383)*x_ref[0]+(-0.154482220539)*x_ref[1]+(0.378696371956)*x_ref[2]
        ref[2,1,1,0]=(-0.0812015296246)*x_ref[0]+(-1.10252794568)*x_ref[1]+(-0.00585426581857)*x_ref[2]
        ref[2,1,1,1]=(0.536042665272)*x_ref[0]+(0.685507117255)*x_ref[1]+(-0.179652683139)*x_ref[2]
        ref[2,1,2,0]=(-0.36750467811)*x_ref[0]+(-1.23674550869)*x_ref[1]+(-0.762905702801)*x_ref[2]
        ref[2,1,2,1]=(0.329665965651)*x_ref[0]+(0.101195810737)*x_ref[1]+(-0.610015052375)*x_ref[2]
        ref[2,2,0,0]=(-0.81644635185)*x_ref[0]+(-0.604578476306)*x_ref[1]+(0.0440925706991)*x_ref[2]
        ref[2,2,0,1]=(-0.958241437971)*x_ref[0]+(0.38563498206)*x_ref[1]+(0.913406762318)*x_ref[2]
        ref[2,2,1,0]=(1.22027492224)*x_ref[0]+(-1.00726209939)*x_ref[1]+(-1.05083068303)*x_ref[2]
        ref[2,2,1,1]=(-1.47561192645)*x_ref[0]+(-0.903549355608)*x_ref[1]+(0.236771324671)*x_ref[2]
        ref[2,2,2,0]=(0.284995864449)*x_ref[0]+(-0.175760829921)*x_ref[1]+(-0.720607220001)*x_ref[2]
        ref[2,2,2,1]=(-0.535894515963)*x_ref[0]+(-0.821302957162)*x_ref[1]+(-1.75141540241)*x_ref[2]
        ref[2,3,0,0]=(-0.333082106674)*x_ref[0]+(0.579471826412)*x_ref[1]+(0.561184138108)*x_ref[2]
        ref[2,3,0,1]=(1.34803519557)*x_ref[0]+(-0.441299088849)*x_ref[1]+(-1.04945951616)*x_ref[2]
        ref[2,3,1,0]=(0.299011096602)*x_ref[0]+(1.32726458235)*x_ref[1]+(-0.196298687359)*x_ref[2]
        ref[2,3,1,1]=(1.48698834204)*x_ref[0]+(0.527221496971)*x_ref[1]+(0.0819813519422)*x_ref[2]
        ref[2,3,2,0]=(0.681680955153)*x_ref[0]+(0.503653972357)*x_ref[1]+(0.434669347891)*x_ref[2]
        ref[2,3,2,1]=(-0.152586789311)*x_ref[0]+(-0.765242635579)*x_ref[1]+(0.712663994387)*x_ref[2]
        ref[2,4,0,0]=(0.630152300463)*x_ref[0]+(1.12366733588)*x_ref[1]+(-0.662966700638)*x_ref[2]
        ref[2,4,0,1]=(1.02796709755)*x_ref[0]+(-0.24093881584)*x_ref[1]+(1.79445417019)*x_ref[2]
        ref[2,4,1,0]=(-0.954575760758)*x_ref[0]+(0.16331782608)*x_ref[1]+(1.38309921277)*x_ref[2]
        ref[2,4,1,1]=(0.536092514281)*x_ref[0]+(-0.276364952673)*x_ref[1]+(0.779619701528)*x_ref[2]
        ref[2,4,2,0]=(0.267270444141)*x_ref[0]+(-0.810746424331)*x_ref[1]+(-0.733877853264)*x_ref[2]
        ref[2,4,2,1]=(-0.0641688847923)*x_ref[0]+(-0.386279280667)*x_ref[1]+(0.0637571595255)*x_ref[2]
        ref[3,0,0,0]=(0.469903180687)*x_ref[0]+(-0.793812397342)*x_ref[1]+(1.60841921779)*x_ref[2]
        ref[3,0,0,1]=(0.720057600102)*x_ref[0]+(0.502582253261)*x_ref[1]+(0.460147791003)*x_ref[2]
        ref[3,0,1,0]=(0.0327862564137)*x_ref[0]+(-1.142962965)*x_ref[1]+(0.114799131894)*x_ref[2]
        ref[3,0,1,1]=(-0.997412656978)*x_ref[0]+(-0.678175468576)*x_ref[1]+(-0.152663147682)*x_ref[2]
        ref[3,0,2,0]=(0.810424599078)*x_ref[0]+(0.75522727786)*x_ref[1]+(-0.205880941501)*x_ref[2]
        ref[3,0,2,1]=(0.353601949476)*x_ref[0]+(0.0963200813379)*x_ref[1]+(0.0578829287146)*x_ref[2]
        ref[3,1,0,0]=(0.346590730062)*x_ref[0]+(1.07953790092)*x_ref[1]+(0.647586851217)*x_ref[2]
        ref[3,1,0,1]=(0.889879195334)*x_ref[0]+(-0.352788186249)*x_ref[1]+(0.434900135803)*x_ref[2]
        ref[3,1,1,0]=(-0.721742734959)*x_ref[0]+(0.166066672134)*x_ref[1]+(-0.489266924809)*x_ref[2]
        ref[3,1,1,1]=(-0.810487755181)*x_ref[0]+(0.0354916979253)*x_ref[1]+(0.0735520654395)*x_ref[2]
        ref[3,1,2,0]=(1.02414338939)*x_ref[0]+(-0.851076284556)*x_ref[1]+(-0.232174494405)*x_ref[2]
        ref[3,1,2,1]=(-0.327246566026)*x_ref[0]+(-0.0499396023605)*x_ref[1]+(0.611585402379)*x_ref[2]
        ref[3,2,0,0]=(-0.365757628557)*x_ref[0]+(0.868697342115)*x_ref[1]+(-1.102545972)*x_ref[2]
        ref[3,2,0,1]=(0.628592129227)*x_ref[0]+(1.25545515311)*x_ref[1]+(-1.0577476264)*x_ref[2]
        ref[3,2,1,0]=(0.160995978044)*x_ref[0]+(-0.947389499556)*x_ref[1]+(-0.870476160656)*x_ref[2]
        ref[3,2,1,1]=(0.910787139938)*x_ref[0]+(-0.0681493345139)*x_ref[1]+(-0.561910153781)*x_ref[2]
        ref[3,2,2,0]=(0.0761644634428)*x_ref[0]+(-0.700940967343)*x_ref[1]+(1.19407210904)*x_ref[2]
        ref[3,2,2,1]=(1.75717437065)*x_ref[0]+(-0.144457317494)*x_ref[1]+(-0.523233453079)*x_ref[2]
        ref[3,3,0,0]=(0.448782872606)*x_ref[0]+(0.650464256547)*x_ref[1]+(1.19476547456)*x_ref[2]
        ref[3,3,0,1]=(0.118586027201)*x_ref[0]+(0.449971816918)*x_ref[1]+(-0.510812733389)*x_ref[2]
        ref[3,3,1,0]=(-0.330032265081)*x_ref[0]+(-0.13657534891)*x_ref[1]+(0.456365289556)*x_ref[2]
        ref[3,3,1,1]=(0.697584212635)*x_ref[0]+(0.897604241724)*x_ref[1]+(1.31517116663)*x_ref[2]
        ref[3,3,2,0]=(-0.940538707365)*x_ref[0]+(-0.334862221665)*x_ref[1]+(0.0969742096754)*x_ref[2]
        ref[3,3,2,1]=(-0.985788813516)*x_ref[0]+(0.348389725548)*x_ref[1]+(-0.260816590361)*x_ref[2]
        ref[3,4,0,0]=(0.754692421417)*x_ref[0]+(0.775985781769)*x_ref[1]+(-0.0866159222457)*x_ref[2]
        ref[3,4,0,1]=(-1.17670502429)*x_ref[0]+(1.12268279393)*x_ref[1]+(-1.1279822095)*x_ref[2]
        ref[3,4,1,0]=(-1.48674370666)*x_ref[0]+(1.26685445713)*x_ref[1]+(-0.0522828809306)*x_ref[2]
        ref[3,4,1,1]=(0.401760689843)*x_ref[0]+(-1.35707797971)*x_ref[1]+(-0.0580654273828)*x_ref[2]
        ref[3,4,2,0]=(-1.40697830852)*x_ref[0]+(0.933923369958)*x_ref[1]+(0.230152169159)*x_ref[2]
        ref[3,4,2,1]=(-1.21242130555)*x_ref[0]+(0.858316339663)*x_ref[1]+(-0.839747078526)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromData_ReducedContinuousFunction_rank0(self):
      """
      tests interpolate for rank 0 Data onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedContinuousFunction onto ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(),w)
      ref=Data(0,(),w_ref)
      if dim==2:
        arg=(-0.100067148208)*x[0]+(-0.8375743725)*x[1]
        ref=(-0.100067148208)*x_ref[0]+(-0.8375743725)*x_ref[1]
      else:
        arg=(-0.247343416726)*x[0]+(0.604193484164)*x[1]+(-0.219899438287)*x[2]
        ref=(-0.247343416726)*x_ref[0]+(0.604193484164)*x_ref[1]+(-0.219899438287)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromData_ReducedContinuousFunction_rank1(self):
      """
      tests interpolate for rank 1 Data onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedContinuousFunction onto ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(2,),w)
      ref=Data(0,(2,),w_ref)
      if dim==2:
        arg[0]=(-1.66196653104)*x[0]+(1.29891335227)*x[1]
        arg[1]=(-0.347844771248)*x[0]+(-0.404726300584)*x[1]
        ref[0]=(-1.66196653104)*x[0]+(1.29891335227)*x[1]
        ref[1]=(-0.347844771248)*x[0]+(-0.404726300584)*x[1]
      else:
        arg[0]=(0.334094742067)*x[0]+(-0.619063646554)*x[1]+(0.538572422001)*x[2]
        arg[1]=(0.673290433004)*x[0]+(0.169501972843)*x[1]+(-0.833663493893)*x[2]
        ref[0]=(0.334094742067)*x[0]+(-0.619063646554)*x[1]+(0.538572422001)*x[2]
        ref[1]=(0.673290433004)*x[0]+(0.169501972843)*x[1]+(-0.833663493893)*x[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromData_ReducedContinuousFunction_rank2(self):
      """
      tests interpolate for rank 2 Data onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedContinuousFunction onto ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5),w)
      ref=Data(0,(4, 5),w_ref)
      if dim==2:
        arg[0,0]=(0.00153698933322)*x[0]+(0.392281645764)*x[1]
        arg[0,1]=(-1.2794655544)*x[0]+(-0.373519639818)*x[1]
        arg[0,2]=(0.30127033321)*x[0]+(-0.0996865853738)*x[1]
        arg[0,3]=(1.23405799349)*x[0]+(0.498558200339)*x[1]
        arg[0,4]=(0.85519350742)*x[0]+(-0.19188046135)*x[1]
        arg[1,0]=(0.417367834962)*x[0]+(0.992318201639)*x[1]
        arg[1,1]=(-0.646432128833)*x[0]+(-0.872750321582)*x[1]
        arg[1,2]=(-0.806117185061)*x[0]+(-0.89396408325)*x[1]
        arg[1,3]=(0.768017237627)*x[0]+(-1.2904111912)*x[1]
        arg[1,4]=(-0.0159685523188)*x[0]+(0.401410193169)*x[1]
        arg[2,0]=(-0.844501716181)*x[0]+(-0.123115343819)*x[1]
        arg[2,1]=(1.08441909409)*x[0]+(0.152432282539)*x[1]
        arg[2,2]=(-0.60551037413)*x[0]+(1.15692828826)*x[1]
        arg[2,3]=(-1.21509168541)*x[0]+(1.03683236377)*x[1]
        arg[2,4]=(1.16850730134)*x[0]+(0.162202615909)*x[1]
        arg[3,0]=(0.727772166897)*x[0]+(-0.296815840935)*x[1]
        arg[3,1]=(-0.0880577151489)*x[0]+(1.66427691426)*x[1]
        arg[3,2]=(0.415144534458)*x[0]+(-0.555833723295)*x[1]
        arg[3,3]=(1.09444635829)*x[0]+(-0.335209417536)*x[1]
        arg[3,4]=(-1.05047769585)*x[0]+(0.684427039986)*x[1]
        ref[0,0]=(0.00153698933322)*x_ref[0]+(0.392281645764)*x_ref[1]
        ref[0,1]=(-1.2794655544)*x_ref[0]+(-0.373519639818)*x_ref[1]
        ref[0,2]=(0.30127033321)*x_ref[0]+(-0.0996865853738)*x_ref[1]
        ref[0,3]=(1.23405799349)*x_ref[0]+(0.498558200339)*x_ref[1]
        ref[0,4]=(0.85519350742)*x_ref[0]+(-0.19188046135)*x_ref[1]
        ref[1,0]=(0.417367834962)*x_ref[0]+(0.992318201639)*x_ref[1]
        ref[1,1]=(-0.646432128833)*x_ref[0]+(-0.872750321582)*x_ref[1]
        ref[1,2]=(-0.806117185061)*x_ref[0]+(-0.89396408325)*x_ref[1]
        ref[1,3]=(0.768017237627)*x_ref[0]+(-1.2904111912)*x_ref[1]
        ref[1,4]=(-0.0159685523188)*x_ref[0]+(0.401410193169)*x_ref[1]
        ref[2,0]=(-0.844501716181)*x_ref[0]+(-0.123115343819)*x_ref[1]
        ref[2,1]=(1.08441909409)*x_ref[0]+(0.152432282539)*x_ref[1]
        ref[2,2]=(-0.60551037413)*x_ref[0]+(1.15692828826)*x_ref[1]
        ref[2,3]=(-1.21509168541)*x_ref[0]+(1.03683236377)*x_ref[1]
        ref[2,4]=(1.16850730134)*x_ref[0]+(0.162202615909)*x_ref[1]
        ref[3,0]=(0.727772166897)*x_ref[0]+(-0.296815840935)*x_ref[1]
        ref[3,1]=(-0.0880577151489)*x_ref[0]+(1.66427691426)*x_ref[1]
        ref[3,2]=(0.415144534458)*x_ref[0]+(-0.555833723295)*x_ref[1]
        ref[3,3]=(1.09444635829)*x_ref[0]+(-0.335209417536)*x_ref[1]
        ref[3,4]=(-1.05047769585)*x_ref[0]+(0.684427039986)*x_ref[1]
      else:
        arg[0,0]=(-0.611607640773)*x[0]+(0.584485971628)*x[1]+(0.0090316337643)*x[2]
        arg[0,1]=(0.905278285857)*x[0]+(0.332523151242)*x[1]+(0.9909841807)*x[2]
        arg[0,2]=(-1.06629333132)*x[0]+(-0.38478180064)*x[1]+(-0.63309211256)*x[2]
        arg[0,3]=(0.122436386344)*x[0]+(-0.218752671204)*x[1]+(-0.746695172216)*x[2]
        arg[0,4]=(1.03152124874)*x[0]+(-1.11327271563)*x[1]+(-0.257591112135)*x[2]
        arg[1,0]=(0.969019600299)*x[0]+(-0.320279309689)*x[1]+(0.327767746545)*x[2]
        arg[1,1]=(0.291023069052)*x[0]+(0.78112901064)*x[1]+(0.879257239042)*x[2]
        arg[1,2]=(-0.131262891536)*x[0]+(-0.0109690229153)*x[1]+(-0.171900332406)*x[2]
        arg[1,3]=(-1.32272684486)*x[0]+(-0.256869536085)*x[1]+(0.0308524567462)*x[2]
        arg[1,4]=(1.30707061279)*x[0]+(0.972866282634)*x[1]+(-0.474948537882)*x[2]
        arg[2,0]=(0.169034210421)*x[0]+(-1.08714896226)*x[1]+(-0.636838450708)*x[2]
        arg[2,1]=(-0.470752780773)*x[0]+(-0.695286088059)*x[1]+(-1.19197058712)*x[2]
        arg[2,2]=(0.701403721228)*x[0]+(0.551002482436)*x[1]+(-0.295853851234)*x[2]
        arg[2,3]=(-0.796431361745)*x[0]+(0.0581884766856)*x[1]+(-1.13668563089)*x[2]
        arg[2,4]=(-0.733185798017)*x[0]+(0.178900103735)*x[1]+(-0.845671526781)*x[2]
        arg[3,0]=(-1.47064927107)*x[0]+(1.33393324965)*x[1]+(-0.767393460937)*x[2]
        arg[3,1]=(1.36784513226)*x[0]+(0.100008910431)*x[1]+(-1.09210256713)*x[2]
        arg[3,2]=(-0.873076669977)*x[0]+(0.367213631778)*x[1]+(0.0698296920039)*x[2]
        arg[3,3]=(-1.76609214158)*x[0]+(-1.26186286566)*x[1]+(0.377817035113)*x[2]
        arg[3,4]=(-1.28261709396)*x[0]+(0.240480767533)*x[1]+(0.307813279357)*x[2]
        ref[0,0]=(-0.611607640773)*x_ref[0]+(0.584485971628)*x_ref[1]+(0.0090316337643)*x_ref[2]
        ref[0,1]=(0.905278285857)*x_ref[0]+(0.332523151242)*x_ref[1]+(0.9909841807)*x_ref[2]
        ref[0,2]=(-1.06629333132)*x_ref[0]+(-0.38478180064)*x_ref[1]+(-0.63309211256)*x_ref[2]
        ref[0,3]=(0.122436386344)*x_ref[0]+(-0.218752671204)*x_ref[1]+(-0.746695172216)*x_ref[2]
        ref[0,4]=(1.03152124874)*x_ref[0]+(-1.11327271563)*x_ref[1]+(-0.257591112135)*x_ref[2]
        ref[1,0]=(0.969019600299)*x_ref[0]+(-0.320279309689)*x_ref[1]+(0.327767746545)*x_ref[2]
        ref[1,1]=(0.291023069052)*x_ref[0]+(0.78112901064)*x_ref[1]+(0.879257239042)*x_ref[2]
        ref[1,2]=(-0.131262891536)*x_ref[0]+(-0.0109690229153)*x_ref[1]+(-0.171900332406)*x_ref[2]
        ref[1,3]=(-1.32272684486)*x_ref[0]+(-0.256869536085)*x_ref[1]+(0.0308524567462)*x_ref[2]
        ref[1,4]=(1.30707061279)*x_ref[0]+(0.972866282634)*x_ref[1]+(-0.474948537882)*x_ref[2]
        ref[2,0]=(0.169034210421)*x_ref[0]+(-1.08714896226)*x_ref[1]+(-0.636838450708)*x_ref[2]
        ref[2,1]=(-0.470752780773)*x_ref[0]+(-0.695286088059)*x_ref[1]+(-1.19197058712)*x_ref[2]
        ref[2,2]=(0.701403721228)*x_ref[0]+(0.551002482436)*x_ref[1]+(-0.295853851234)*x_ref[2]
        ref[2,3]=(-0.796431361745)*x_ref[0]+(0.0581884766856)*x_ref[1]+(-1.13668563089)*x_ref[2]
        ref[2,4]=(-0.733185798017)*x_ref[0]+(0.178900103735)*x_ref[1]+(-0.845671526781)*x_ref[2]
        ref[3,0]=(-1.47064927107)*x_ref[0]+(1.33393324965)*x_ref[1]+(-0.767393460937)*x_ref[2]
        ref[3,1]=(1.36784513226)*x_ref[0]+(0.100008910431)*x_ref[1]+(-1.09210256713)*x_ref[2]
        ref[3,2]=(-0.873076669977)*x_ref[0]+(0.367213631778)*x_ref[1]+(0.0698296920039)*x_ref[2]
        ref[3,3]=(-1.76609214158)*x_ref[0]+(-1.26186286566)*x_ref[1]+(0.377817035113)*x_ref[2]
        ref[3,4]=(-1.28261709396)*x_ref[0]+(0.240480767533)*x_ref[1]+(0.307813279357)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromData_ReducedContinuousFunction_rank3(self):
      """
      tests interpolate for rank 3 Data onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedContinuousFunction onto ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2),w_ref)
      if dim==2:
        arg[0,0,0]=(-0.423505169404)*x[0]+(-1.10390334264)*x[1]
        arg[0,0,1]=(-0.181820089254)*x[0]+(-0.374602268791)*x[1]
        arg[0,1,0]=(-1.36906917075)*x[0]+(0.26554385908)*x[1]
        arg[0,1,1]=(-0.280661572374)*x[0]+(-0.239232524287)*x[1]
        arg[1,0,0]=(-1.25670251934)*x[0]+(-0.429353977735)*x[1]
        arg[1,0,1]=(-0.747928970735)*x[0]+(1.17874317611)*x[1]
        arg[1,1,0]=(-0.548814611802)*x[0]+(-0.0249315764285)*x[1]
        arg[1,1,1]=(-0.481123117522)*x[0]+(1.23402203879)*x[1]
        arg[2,0,0]=(-0.0639512689884)*x[0]+(-0.0278213371376)*x[1]
        arg[2,0,1]=(-1.17292797621)*x[0]+(0.42140600304)*x[1]
        arg[2,1,0]=(-0.358052601083)*x[0]+(-1.66807880791)*x[1]
        arg[2,1,1]=(-0.291433251349)*x[0]+(0.111473162108)*x[1]
        arg[3,0,0]=(0.742643625989)*x[0]+(-1.29214106228)*x[1]
        arg[3,0,1]=(0.319887321826)*x[0]+(-0.40213369238)*x[1]
        arg[3,1,0]=(-0.449412064097)*x[0]+(-1.10654262009)*x[1]
        arg[3,1,1]=(1.19722227632)*x[0]+(1.3455799026)*x[1]
        arg[4,0,0]=(-1.77453188487)*x[0]+(0.634838793872)*x[1]
        arg[4,0,1]=(-0.215461066907)*x[0]+(1.11635233844)*x[1]
        arg[4,1,0]=(0.835816652592)*x[0]+(0.668378554214)*x[1]
        arg[4,1,1]=(-1.22133225406)*x[0]+(-0.148780252398)*x[1]
        arg[5,0,0]=(0.320245409639)*x[0]+(-1.08828513776)*x[1]
        arg[5,0,1]=(-0.1865835417)*x[0]+(-0.353282061874)*x[1]
        arg[5,1,0]=(1.38858040685)*x[0]+(0.166398261098)*x[1]
        arg[5,1,1]=(0.947718875755)*x[0]+(-1.06171934302)*x[1]
        ref[0,0,0]=(-0.423505169404)*x_ref[0]+(-1.10390334264)*x_ref[1]
        ref[0,0,1]=(-0.181820089254)*x_ref[0]+(-0.374602268791)*x_ref[1]
        ref[0,1,0]=(-1.36906917075)*x_ref[0]+(0.26554385908)*x_ref[1]
        ref[0,1,1]=(-0.280661572374)*x_ref[0]+(-0.239232524287)*x_ref[1]
        ref[1,0,0]=(-1.25670251934)*x_ref[0]+(-0.429353977735)*x_ref[1]
        ref[1,0,1]=(-0.747928970735)*x_ref[0]+(1.17874317611)*x_ref[1]
        ref[1,1,0]=(-0.548814611802)*x_ref[0]+(-0.0249315764285)*x_ref[1]
        ref[1,1,1]=(-0.481123117522)*x_ref[0]+(1.23402203879)*x_ref[1]
        ref[2,0,0]=(-0.0639512689884)*x_ref[0]+(-0.0278213371376)*x_ref[1]
        ref[2,0,1]=(-1.17292797621)*x_ref[0]+(0.42140600304)*x_ref[1]
        ref[2,1,0]=(-0.358052601083)*x_ref[0]+(-1.66807880791)*x_ref[1]
        ref[2,1,1]=(-0.291433251349)*x_ref[0]+(0.111473162108)*x_ref[1]
        ref[3,0,0]=(0.742643625989)*x_ref[0]+(-1.29214106228)*x_ref[1]
        ref[3,0,1]=(0.319887321826)*x_ref[0]+(-0.40213369238)*x_ref[1]
        ref[3,1,0]=(-0.449412064097)*x_ref[0]+(-1.10654262009)*x_ref[1]
        ref[3,1,1]=(1.19722227632)*x_ref[0]+(1.3455799026)*x_ref[1]
        ref[4,0,0]=(-1.77453188487)*x_ref[0]+(0.634838793872)*x_ref[1]
        ref[4,0,1]=(-0.215461066907)*x_ref[0]+(1.11635233844)*x_ref[1]
        ref[4,1,0]=(0.835816652592)*x_ref[0]+(0.668378554214)*x_ref[1]
        ref[4,1,1]=(-1.22133225406)*x_ref[0]+(-0.148780252398)*x_ref[1]
        ref[5,0,0]=(0.320245409639)*x_ref[0]+(-1.08828513776)*x_ref[1]
        ref[5,0,1]=(-0.1865835417)*x_ref[0]+(-0.353282061874)*x_ref[1]
        ref[5,1,0]=(1.38858040685)*x_ref[0]+(0.166398261098)*x_ref[1]
        ref[5,1,1]=(0.947718875755)*x_ref[0]+(-1.06171934302)*x_ref[1]
      else:
        arg[0,0,0]=(-0.892096322478)*x[0]+(0.720363183451)*x[1]+(0.600609272191)*x[2]
        arg[0,0,1]=(-0.418024461935)*x[0]+(-0.819275292035)*x[1]+(0.134962557692)*x[2]
        arg[0,1,0]=(-0.449783838361)*x[0]+(0.943677173533)*x[1]+(0.209424190991)*x[2]
        arg[0,1,1]=(-0.424386775004)*x[0]+(-0.213024388094)*x[1]+(-1.05927943384)*x[2]
        arg[1,0,0]=(-0.699504096539)*x[0]+(-0.153570308377)*x[1]+(0.576902392658)*x[2]
        arg[1,0,1]=(-0.426914055661)*x[0]+(-0.330405267059)*x[1]+(1.07656632155)*x[2]
        arg[1,1,0]=(0.775238005352)*x[0]+(1.11012072575)*x[1]+(0.764627098603)*x[2]
        arg[1,1,1]=(0.0130424708408)*x[0]+(-0.671778102107)*x[1]+(0.780684368983)*x[2]
        arg[2,0,0]=(-0.0512224258498)*x[0]+(-1.50515803984)*x[1]+(0.342824143946)*x[2]
        arg[2,0,1]=(0.117541624176)*x[0]+(0.372415524552)*x[1]+(0.255218095016)*x[2]
        arg[2,1,0]=(-0.574508895703)*x[0]+(0.345217292359)*x[1]+(0.209211956821)*x[2]
        arg[2,1,1]=(0.93643587147)*x[0]+(-0.096209009902)*x[1]+(0.205150859527)*x[2]
        arg[3,0,0]=(1.7541929884)*x[0]+(0.169061478447)*x[1]+(0.534746695515)*x[2]
        arg[3,0,1]=(0.442329266782)*x[0]+(0.334971710414)*x[1]+(1.66892557698)*x[2]
        arg[3,1,0]=(0.316436716758)*x[0]+(-1.25442030987)*x[1]+(0.372183105374)*x[2]
        arg[3,1,1]=(-0.254157820265)*x[0]+(0.287381300023)*x[1]+(0.0574117423352)*x[2]
        arg[4,0,0]=(-1.08380257521)*x[0]+(-1.31461890032)*x[1]+(-0.503802775099)*x[2]
        arg[4,0,1]=(-0.558293486798)*x[0]+(0.702317635925)*x[1]+(0.463814829137)*x[2]
        arg[4,1,0]=(-0.677160971974)*x[0]+(-0.14921275312)*x[1]+(-1.6501459435)*x[2]
        arg[4,1,1]=(-0.176672532474)*x[0]+(0.945017605811)*x[1]+(-0.790342939788)*x[2]
        arg[5,0,0]=(-1.45668070675)*x[0]+(-0.404957129204)*x[1]+(1.37383065954)*x[2]
        arg[5,0,1]=(0.53829326756)*x[0]+(0.543333031479)*x[1]+(-0.854057296362)*x[2]
        arg[5,1,0]=(0.805864283921)*x[0]+(-0.00186634953513)*x[1]+(-1.97111362539)*x[2]
        arg[5,1,1]=(-0.387746238967)*x[0]+(-0.739158353882)*x[1]+(-0.134082808118)*x[2]
        ref[0,0,0]=(-0.892096322478)*x_ref[0]+(0.720363183451)*x_ref[1]+(0.600609272191)*x_ref[2]
        ref[0,0,1]=(-0.418024461935)*x_ref[0]+(-0.819275292035)*x_ref[1]+(0.134962557692)*x_ref[2]
        ref[0,1,0]=(-0.449783838361)*x_ref[0]+(0.943677173533)*x_ref[1]+(0.209424190991)*x_ref[2]
        ref[0,1,1]=(-0.424386775004)*x_ref[0]+(-0.213024388094)*x_ref[1]+(-1.05927943384)*x_ref[2]
        ref[1,0,0]=(-0.699504096539)*x_ref[0]+(-0.153570308377)*x_ref[1]+(0.576902392658)*x_ref[2]
        ref[1,0,1]=(-0.426914055661)*x_ref[0]+(-0.330405267059)*x_ref[1]+(1.07656632155)*x_ref[2]
        ref[1,1,0]=(0.775238005352)*x_ref[0]+(1.11012072575)*x_ref[1]+(0.764627098603)*x_ref[2]
        ref[1,1,1]=(0.0130424708408)*x_ref[0]+(-0.671778102107)*x_ref[1]+(0.780684368983)*x_ref[2]
        ref[2,0,0]=(-0.0512224258498)*x_ref[0]+(-1.50515803984)*x_ref[1]+(0.342824143946)*x_ref[2]
        ref[2,0,1]=(0.117541624176)*x_ref[0]+(0.372415524552)*x_ref[1]+(0.255218095016)*x_ref[2]
        ref[2,1,0]=(-0.574508895703)*x_ref[0]+(0.345217292359)*x_ref[1]+(0.209211956821)*x_ref[2]
        ref[2,1,1]=(0.93643587147)*x_ref[0]+(-0.096209009902)*x_ref[1]+(0.205150859527)*x_ref[2]
        ref[3,0,0]=(1.7541929884)*x_ref[0]+(0.169061478447)*x_ref[1]+(0.534746695515)*x_ref[2]
        ref[3,0,1]=(0.442329266782)*x_ref[0]+(0.334971710414)*x_ref[1]+(1.66892557698)*x_ref[2]
        ref[3,1,0]=(0.316436716758)*x_ref[0]+(-1.25442030987)*x_ref[1]+(0.372183105374)*x_ref[2]
        ref[3,1,1]=(-0.254157820265)*x_ref[0]+(0.287381300023)*x_ref[1]+(0.0574117423352)*x_ref[2]
        ref[4,0,0]=(-1.08380257521)*x_ref[0]+(-1.31461890032)*x_ref[1]+(-0.503802775099)*x_ref[2]
        ref[4,0,1]=(-0.558293486798)*x_ref[0]+(0.702317635925)*x_ref[1]+(0.463814829137)*x_ref[2]
        ref[4,1,0]=(-0.677160971974)*x_ref[0]+(-0.14921275312)*x_ref[1]+(-1.6501459435)*x_ref[2]
        ref[4,1,1]=(-0.176672532474)*x_ref[0]+(0.945017605811)*x_ref[1]+(-0.790342939788)*x_ref[2]
        ref[5,0,0]=(-1.45668070675)*x_ref[0]+(-0.404957129204)*x_ref[1]+(1.37383065954)*x_ref[2]
        ref[5,0,1]=(0.53829326756)*x_ref[0]+(0.543333031479)*x_ref[1]+(-0.854057296362)*x_ref[2]
        ref[5,1,0]=(0.805864283921)*x_ref[0]+(-0.00186634953513)*x_ref[1]+(-1.97111362539)*x_ref[2]
        ref[5,1,1]=(-0.387746238967)*x_ref[0]+(-0.739158353882)*x_ref[1]+(-0.134082808118)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromData_ReducedContinuousFunction_rank4(self):
      """
      tests interpolate for rank 4 Data onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedContinuousFunction onto ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5, 3, 2),w)
      ref=Data(0,(4, 5, 3, 2),w_ref)
      if dim==2:
        arg[0,0,0,0]=(-0.754311658086)*x[0]+(0.235157034342)*x[1]
        arg[0,0,0,1]=(-1.33786315024)*x[0]+(0.243349279742)*x[1]
        arg[0,0,1,0]=(1.48657750088)*x[0]+(1.09853108305)*x[1]
        arg[0,0,1,1]=(1.4501362266)*x[0]+(0.824811567155)*x[1]
        arg[0,0,2,0]=(1.12567566082)*x[0]+(0.663525681174)*x[1]
        arg[0,0,2,1]=(1.48991565019)*x[0]+(-0.680795365011)*x[1]
        arg[0,1,0,0]=(0.268571449528)*x[0]+(0.818595834672)*x[1]
        arg[0,1,0,1]=(-1.55490945555)*x[0]+(0.783595197888)*x[1]
        arg[0,1,1,0]=(0.913201382585)*x[0]+(0.855491379376)*x[1]
        arg[0,1,1,1]=(-0.179192075116)*x[0]+(-0.690415079523)*x[1]
        arg[0,1,2,0]=(-0.0393574502912)*x[0]+(-0.690157650634)*x[1]
        arg[0,1,2,1]=(0.784216863217)*x[0]+(0.498735486159)*x[1]
        arg[0,2,0,0]=(0.137939554165)*x[0]+(-1.33961519113)*x[1]
        arg[0,2,0,1]=(-1.03897311736)*x[0]+(-1.37797405312)*x[1]
        arg[0,2,1,0]=(0.199812003756)*x[0]+(-0.0388499674037)*x[1]
        arg[0,2,1,1]=(1.15530534202)*x[0]+(-1.28280869078)*x[1]
        arg[0,2,2,0]=(0.836447791155)*x[0]+(1.2627066026)*x[1]
        arg[0,2,2,1]=(-0.584608492688)*x[0]+(-1.55305974305)*x[1]
        arg[0,3,0,0]=(-0.661952381799)*x[0]+(-1.27062403778)*x[1]
        arg[0,3,0,1]=(-1.47789532259)*x[0]+(-0.279583524936)*x[1]
        arg[0,3,1,0]=(1.40729771357)*x[0]+(0.0971284144528)*x[1]
        arg[0,3,1,1]=(-0.459167955911)*x[0]+(-1.29033281524)*x[1]
        arg[0,3,2,0]=(-1.63189436824)*x[0]+(0.75729808422)*x[1]
        arg[0,3,2,1]=(0.522479828718)*x[0]+(-0.0231299515383)*x[1]
        arg[0,4,0,0]=(0.241891446615)*x[0]+(0.803327107651)*x[1]
        arg[0,4,0,1]=(0.273613502836)*x[0]+(-0.0115180464171)*x[1]
        arg[0,4,1,0]=(1.96219818696)*x[0]+(-0.349137094743)*x[1]
        arg[0,4,1,1]=(0.365690821719)*x[0]+(-1.6943347181)*x[1]
        arg[0,4,2,0]=(-0.283970779872)*x[0]+(-0.258537179804)*x[1]
        arg[0,4,2,1]=(-0.973333430598)*x[0]+(1.48706276212)*x[1]
        arg[1,0,0,0]=(1.18757466971)*x[0]+(0.838804656357)*x[1]
        arg[1,0,0,1]=(0.840859904702)*x[0]+(1.09955195682)*x[1]
        arg[1,0,1,0]=(1.46190513227)*x[0]+(-0.596959830422)*x[1]
        arg[1,0,1,1]=(0.718689296745)*x[0]+(-0.628074620222)*x[1]
        arg[1,0,2,0]=(0.440761293041)*x[0]+(0.115800854194)*x[1]
        arg[1,0,2,1]=(1.34891146904)*x[0]+(0.813690260652)*x[1]
        arg[1,1,0,0]=(0.183071638098)*x[0]+(-1.37996210755)*x[1]
        arg[1,1,0,1]=(0.258142115988)*x[0]+(-0.757709381529)*x[1]
        arg[1,1,1,0]=(1.72392829309)*x[0]+(0.444995102781)*x[1]
        arg[1,1,1,1]=(-0.380898209988)*x[0]+(-0.0950246860347)*x[1]
        arg[1,1,2,0]=(0.203007226016)*x[0]+(1.09786001832)*x[1]
        arg[1,1,2,1]=(-0.502500865341)*x[0]+(-0.629980527995)*x[1]
        arg[1,2,0,0]=(0.482549518436)*x[0]+(0.565840824146)*x[1]
        arg[1,2,0,1]=(0.355818565068)*x[0]+(-0.78273295574)*x[1]
        arg[1,2,1,0]=(1.79204033071)*x[0]+(-0.698040851585)*x[1]
        arg[1,2,1,1]=(-0.534827741044)*x[0]+(0.00943131627498)*x[1]
        arg[1,2,2,0]=(-1.00663524444)*x[0]+(0.0207854553456)*x[1]
        arg[1,2,2,1]=(-0.338048040159)*x[0]+(-0.380607406217)*x[1]
        arg[1,3,0,0]=(0.586310549044)*x[0]+(0.276326798805)*x[1]
        arg[1,3,0,1]=(-0.260482580702)*x[0]+(-0.349279395321)*x[1]
        arg[1,3,1,0]=(-0.196533829895)*x[0]+(0.614805272738)*x[1]
        arg[1,3,1,1]=(-0.363176483895)*x[0]+(-0.925817017731)*x[1]
        arg[1,3,2,0]=(-1.24182075421)*x[0]+(0.215966446013)*x[1]
        arg[1,3,2,1]=(1.07546652642)*x[0]+(0.383803991974)*x[1]
        arg[1,4,0,0]=(0.199312303525)*x[0]+(-0.314520883859)*x[1]
        arg[1,4,0,1]=(-1.56890826152)*x[0]+(-0.133086662241)*x[1]
        arg[1,4,1,0]=(1.1307241936)*x[0]+(-1.32905744984)*x[1]
        arg[1,4,1,1]=(-0.385991455298)*x[0]+(0.292913024507)*x[1]
        arg[1,4,2,0]=(1.14561678056)*x[0]+(-0.662696162276)*x[1]
        arg[1,4,2,1]=(-1.49533948954)*x[0]+(1.43396138744)*x[1]
        arg[2,0,0,0]=(-0.63160396493)*x[0]+(0.813120998776)*x[1]
        arg[2,0,0,1]=(0.65598998434)*x[0]+(-1.02445537553)*x[1]
        arg[2,0,1,0]=(-1.13473104075)*x[0]+(0.057331183449)*x[1]
        arg[2,0,1,1]=(-0.154892482107)*x[0]+(0.481969957579)*x[1]
        arg[2,0,2,0]=(0.521916671496)*x[0]+(0.546676943059)*x[1]
        arg[2,0,2,1]=(-1.68919379028)*x[0]+(1.08292038981)*x[1]
        arg[2,1,0,0]=(-0.643846595888)*x[0]+(0.528306097755)*x[1]
        arg[2,1,0,1]=(1.3745090497)*x[0]+(-0.0801648509184)*x[1]
        arg[2,1,1,0]=(0.129096817054)*x[0]+(0.90750370855)*x[1]
        arg[2,1,1,1]=(0.539309513305)*x[0]+(1.34949470961)*x[1]
        arg[2,1,2,0]=(1.4189703896)*x[0]+(-1.75011201893)*x[1]
        arg[2,1,2,1]=(-1.09325548386)*x[0]+(0.148279301716)*x[1]
        arg[2,2,0,0]=(-1.25785133743)*x[0]+(0.275071306594)*x[1]
        arg[2,2,0,1]=(-0.550152068761)*x[0]+(-1.20422484275)*x[1]
        arg[2,2,1,0]=(-0.102062863668)*x[0]+(-1.34431887541)*x[1]
        arg[2,2,1,1]=(0.103354724169)*x[0]+(-0.941952180446)*x[1]
        arg[2,2,2,0]=(1.2090339604)*x[0]+(-0.330357976796)*x[1]
        arg[2,2,2,1]=(0.203162449188)*x[0]+(-1.03640702014)*x[1]
        arg[2,3,0,0]=(-0.324576353514)*x[0]+(0.582614539267)*x[1]
        arg[2,3,0,1]=(-0.213823136215)*x[0]+(-0.473199105214)*x[1]
        arg[2,3,1,0]=(0.395144702599)*x[0]+(0.0358194467152)*x[1]
        arg[2,3,1,1]=(1.32860563396)*x[0]+(-0.577308310732)*x[1]
        arg[2,3,2,0]=(0.994205029432)*x[0]+(1.13433517636)*x[1]
        arg[2,3,2,1]=(-0.488499286742)*x[0]+(0.224253820738)*x[1]
        arg[2,4,0,0]=(1.48035663008)*x[0]+(-1.09697986914)*x[1]
        arg[2,4,0,1]=(-1.09991915151)*x[0]+(-0.557951199985)*x[1]
        arg[2,4,1,0]=(-1.23862699804)*x[0]+(-0.230602187713)*x[1]
        arg[2,4,1,1]=(-0.922609370448)*x[0]+(1.76673953413)*x[1]
        arg[2,4,2,0]=(0.731769419709)*x[0]+(-1.1094516608)*x[1]
        arg[2,4,2,1]=(-0.145672504805)*x[0]+(0.168194421355)*x[1]
        arg[3,0,0,0]=(1.02559707602)*x[0]+(0.260431368523)*x[1]
        arg[3,0,0,1]=(0.111899568824)*x[0]+(-0.90725475423)*x[1]
        arg[3,0,1,0]=(-0.226547745683)*x[0]+(-0.232830393372)*x[1]
        arg[3,0,1,1]=(0.347004318471)*x[0]+(-0.196130970001)*x[1]
        arg[3,0,2,0]=(-1.72155871442)*x[0]+(-1.28372407452)*x[1]
        arg[3,0,2,1]=(1.43817914693)*x[0]+(-0.354093253684)*x[1]
        arg[3,1,0,0]=(-0.293939245102)*x[0]+(0.0255562139103)*x[1]
        arg[3,1,0,1]=(0.139306213018)*x[0]+(0.595062226642)*x[1]
        arg[3,1,1,0]=(-0.670339920672)*x[0]+(0.387520583192)*x[1]
        arg[3,1,1,1]=(-0.0323325470954)*x[0]+(-0.681162999279)*x[1]
        arg[3,1,2,0]=(0.235275048739)*x[0]+(0.144725473773)*x[1]
        arg[3,1,2,1]=(0.132211795214)*x[0]+(0.262780216845)*x[1]
        arg[3,2,0,0]=(0.590514326163)*x[0]+(-0.84273677918)*x[1]
        arg[3,2,0,1]=(-0.925181556185)*x[0]+(-0.300244447001)*x[1]
        arg[3,2,1,0]=(-0.0509700795112)*x[0]+(-1.31919389616)*x[1]
        arg[3,2,1,1]=(1.50577291846)*x[0]+(-0.219284607769)*x[1]
        arg[3,2,2,0]=(-0.0460616673346)*x[0]+(-0.690312685466)*x[1]
        arg[3,2,2,1]=(-0.673765477936)*x[0]+(-0.0965860475641)*x[1]
        arg[3,3,0,0]=(-0.6577594174)*x[0]+(0.171886277016)*x[1]
        arg[3,3,0,1]=(-1.08810434718)*x[0]+(0.333153250183)*x[1]
        arg[3,3,1,0]=(0.150968371983)*x[0]+(0.290859317391)*x[1]
        arg[3,3,1,1]=(1.62096265131)*x[0]+(-1.10309602741)*x[1]
        arg[3,3,2,0]=(-0.932063639972)*x[0]+(-0.897131561126)*x[1]
        arg[3,3,2,1]=(-0.819110834345)*x[0]+(1.59159109562)*x[1]
        arg[3,4,0,0]=(0.272662931155)*x[0]+(0.69026285756)*x[1]
        arg[3,4,0,1]=(-0.600556370757)*x[0]+(-0.0853533546302)*x[1]
        arg[3,4,1,0]=(1.18544735281)*x[0]+(0.217751189034)*x[1]
        arg[3,4,1,1]=(-0.594609531358)*x[0]+(0.882158349788)*x[1]
        arg[3,4,2,0]=(1.18876765552)*x[0]+(-0.411481586979)*x[1]
        arg[3,4,2,1]=(-1.093067792)*x[0]+(-0.266744260768)*x[1]
        ref[0,0,0,0]=(-0.754311658086)*x_ref[0]+(0.235157034342)*x_ref[1]
        ref[0,0,0,1]=(-1.33786315024)*x_ref[0]+(0.243349279742)*x_ref[1]
        ref[0,0,1,0]=(1.48657750088)*x_ref[0]+(1.09853108305)*x_ref[1]
        ref[0,0,1,1]=(1.4501362266)*x_ref[0]+(0.824811567155)*x_ref[1]
        ref[0,0,2,0]=(1.12567566082)*x_ref[0]+(0.663525681174)*x_ref[1]
        ref[0,0,2,1]=(1.48991565019)*x_ref[0]+(-0.680795365011)*x_ref[1]
        ref[0,1,0,0]=(0.268571449528)*x_ref[0]+(0.818595834672)*x_ref[1]
        ref[0,1,0,1]=(-1.55490945555)*x_ref[0]+(0.783595197888)*x_ref[1]
        ref[0,1,1,0]=(0.913201382585)*x_ref[0]+(0.855491379376)*x_ref[1]
        ref[0,1,1,1]=(-0.179192075116)*x_ref[0]+(-0.690415079523)*x_ref[1]
        ref[0,1,2,0]=(-0.0393574502912)*x_ref[0]+(-0.690157650634)*x_ref[1]
        ref[0,1,2,1]=(0.784216863217)*x_ref[0]+(0.498735486159)*x_ref[1]
        ref[0,2,0,0]=(0.137939554165)*x_ref[0]+(-1.33961519113)*x_ref[1]
        ref[0,2,0,1]=(-1.03897311736)*x_ref[0]+(-1.37797405312)*x_ref[1]
        ref[0,2,1,0]=(0.199812003756)*x_ref[0]+(-0.0388499674037)*x_ref[1]
        ref[0,2,1,1]=(1.15530534202)*x_ref[0]+(-1.28280869078)*x_ref[1]
        ref[0,2,2,0]=(0.836447791155)*x_ref[0]+(1.2627066026)*x_ref[1]
        ref[0,2,2,1]=(-0.584608492688)*x_ref[0]+(-1.55305974305)*x_ref[1]
        ref[0,3,0,0]=(-0.661952381799)*x_ref[0]+(-1.27062403778)*x_ref[1]
        ref[0,3,0,1]=(-1.47789532259)*x_ref[0]+(-0.279583524936)*x_ref[1]
        ref[0,3,1,0]=(1.40729771357)*x_ref[0]+(0.0971284144528)*x_ref[1]
        ref[0,3,1,1]=(-0.459167955911)*x_ref[0]+(-1.29033281524)*x_ref[1]
        ref[0,3,2,0]=(-1.63189436824)*x_ref[0]+(0.75729808422)*x_ref[1]
        ref[0,3,2,1]=(0.522479828718)*x_ref[0]+(-0.0231299515383)*x_ref[1]
        ref[0,4,0,0]=(0.241891446615)*x_ref[0]+(0.803327107651)*x_ref[1]
        ref[0,4,0,1]=(0.273613502836)*x_ref[0]+(-0.0115180464171)*x_ref[1]
        ref[0,4,1,0]=(1.96219818696)*x_ref[0]+(-0.349137094743)*x_ref[1]
        ref[0,4,1,1]=(0.365690821719)*x_ref[0]+(-1.6943347181)*x_ref[1]
        ref[0,4,2,0]=(-0.283970779872)*x_ref[0]+(-0.258537179804)*x_ref[1]
        ref[0,4,2,1]=(-0.973333430598)*x_ref[0]+(1.48706276212)*x_ref[1]
        ref[1,0,0,0]=(1.18757466971)*x_ref[0]+(0.838804656357)*x_ref[1]
        ref[1,0,0,1]=(0.840859904702)*x_ref[0]+(1.09955195682)*x_ref[1]
        ref[1,0,1,0]=(1.46190513227)*x_ref[0]+(-0.596959830422)*x_ref[1]
        ref[1,0,1,1]=(0.718689296745)*x_ref[0]+(-0.628074620222)*x_ref[1]
        ref[1,0,2,0]=(0.440761293041)*x_ref[0]+(0.115800854194)*x_ref[1]
        ref[1,0,2,1]=(1.34891146904)*x_ref[0]+(0.813690260652)*x_ref[1]
        ref[1,1,0,0]=(0.183071638098)*x_ref[0]+(-1.37996210755)*x_ref[1]
        ref[1,1,0,1]=(0.258142115988)*x_ref[0]+(-0.757709381529)*x_ref[1]
        ref[1,1,1,0]=(1.72392829309)*x_ref[0]+(0.444995102781)*x_ref[1]
        ref[1,1,1,1]=(-0.380898209988)*x_ref[0]+(-0.0950246860347)*x_ref[1]
        ref[1,1,2,0]=(0.203007226016)*x_ref[0]+(1.09786001832)*x_ref[1]
        ref[1,1,2,1]=(-0.502500865341)*x_ref[0]+(-0.629980527995)*x_ref[1]
        ref[1,2,0,0]=(0.482549518436)*x_ref[0]+(0.565840824146)*x_ref[1]
        ref[1,2,0,1]=(0.355818565068)*x_ref[0]+(-0.78273295574)*x_ref[1]
        ref[1,2,1,0]=(1.79204033071)*x_ref[0]+(-0.698040851585)*x_ref[1]
        ref[1,2,1,1]=(-0.534827741044)*x_ref[0]+(0.00943131627498)*x_ref[1]
        ref[1,2,2,0]=(-1.00663524444)*x_ref[0]+(0.0207854553456)*x_ref[1]
        ref[1,2,2,1]=(-0.338048040159)*x_ref[0]+(-0.380607406217)*x_ref[1]
        ref[1,3,0,0]=(0.586310549044)*x_ref[0]+(0.276326798805)*x_ref[1]
        ref[1,3,0,1]=(-0.260482580702)*x_ref[0]+(-0.349279395321)*x_ref[1]
        ref[1,3,1,0]=(-0.196533829895)*x_ref[0]+(0.614805272738)*x_ref[1]
        ref[1,3,1,1]=(-0.363176483895)*x_ref[0]+(-0.925817017731)*x_ref[1]
        ref[1,3,2,0]=(-1.24182075421)*x_ref[0]+(0.215966446013)*x_ref[1]
        ref[1,3,2,1]=(1.07546652642)*x_ref[0]+(0.383803991974)*x_ref[1]
        ref[1,4,0,0]=(0.199312303525)*x_ref[0]+(-0.314520883859)*x_ref[1]
        ref[1,4,0,1]=(-1.56890826152)*x_ref[0]+(-0.133086662241)*x_ref[1]
        ref[1,4,1,0]=(1.1307241936)*x_ref[0]+(-1.32905744984)*x_ref[1]
        ref[1,4,1,1]=(-0.385991455298)*x_ref[0]+(0.292913024507)*x_ref[1]
        ref[1,4,2,0]=(1.14561678056)*x_ref[0]+(-0.662696162276)*x_ref[1]
        ref[1,4,2,1]=(-1.49533948954)*x_ref[0]+(1.43396138744)*x_ref[1]
        ref[2,0,0,0]=(-0.63160396493)*x_ref[0]+(0.813120998776)*x_ref[1]
        ref[2,0,0,1]=(0.65598998434)*x_ref[0]+(-1.02445537553)*x_ref[1]
        ref[2,0,1,0]=(-1.13473104075)*x_ref[0]+(0.057331183449)*x_ref[1]
        ref[2,0,1,1]=(-0.154892482107)*x_ref[0]+(0.481969957579)*x_ref[1]
        ref[2,0,2,0]=(0.521916671496)*x_ref[0]+(0.546676943059)*x_ref[1]
        ref[2,0,2,1]=(-1.68919379028)*x_ref[0]+(1.08292038981)*x_ref[1]
        ref[2,1,0,0]=(-0.643846595888)*x_ref[0]+(0.528306097755)*x_ref[1]
        ref[2,1,0,1]=(1.3745090497)*x_ref[0]+(-0.0801648509184)*x_ref[1]
        ref[2,1,1,0]=(0.129096817054)*x_ref[0]+(0.90750370855)*x_ref[1]
        ref[2,1,1,1]=(0.539309513305)*x_ref[0]+(1.34949470961)*x_ref[1]
        ref[2,1,2,0]=(1.4189703896)*x_ref[0]+(-1.75011201893)*x_ref[1]
        ref[2,1,2,1]=(-1.09325548386)*x_ref[0]+(0.148279301716)*x_ref[1]
        ref[2,2,0,0]=(-1.25785133743)*x_ref[0]+(0.275071306594)*x_ref[1]
        ref[2,2,0,1]=(-0.550152068761)*x_ref[0]+(-1.20422484275)*x_ref[1]
        ref[2,2,1,0]=(-0.102062863668)*x_ref[0]+(-1.34431887541)*x_ref[1]
        ref[2,2,1,1]=(0.103354724169)*x_ref[0]+(-0.941952180446)*x_ref[1]
        ref[2,2,2,0]=(1.2090339604)*x_ref[0]+(-0.330357976796)*x_ref[1]
        ref[2,2,2,1]=(0.203162449188)*x_ref[0]+(-1.03640702014)*x_ref[1]
        ref[2,3,0,0]=(-0.324576353514)*x_ref[0]+(0.582614539267)*x_ref[1]
        ref[2,3,0,1]=(-0.213823136215)*x_ref[0]+(-0.473199105214)*x_ref[1]
        ref[2,3,1,0]=(0.395144702599)*x_ref[0]+(0.0358194467152)*x_ref[1]
        ref[2,3,1,1]=(1.32860563396)*x_ref[0]+(-0.577308310732)*x_ref[1]
        ref[2,3,2,0]=(0.994205029432)*x_ref[0]+(1.13433517636)*x_ref[1]
        ref[2,3,2,1]=(-0.488499286742)*x_ref[0]+(0.224253820738)*x_ref[1]
        ref[2,4,0,0]=(1.48035663008)*x_ref[0]+(-1.09697986914)*x_ref[1]
        ref[2,4,0,1]=(-1.09991915151)*x_ref[0]+(-0.557951199985)*x_ref[1]
        ref[2,4,1,0]=(-1.23862699804)*x_ref[0]+(-0.230602187713)*x_ref[1]
        ref[2,4,1,1]=(-0.922609370448)*x_ref[0]+(1.76673953413)*x_ref[1]
        ref[2,4,2,0]=(0.731769419709)*x_ref[0]+(-1.1094516608)*x_ref[1]
        ref[2,4,2,1]=(-0.145672504805)*x_ref[0]+(0.168194421355)*x_ref[1]
        ref[3,0,0,0]=(1.02559707602)*x_ref[0]+(0.260431368523)*x_ref[1]
        ref[3,0,0,1]=(0.111899568824)*x_ref[0]+(-0.90725475423)*x_ref[1]
        ref[3,0,1,0]=(-0.226547745683)*x_ref[0]+(-0.232830393372)*x_ref[1]
        ref[3,0,1,1]=(0.347004318471)*x_ref[0]+(-0.196130970001)*x_ref[1]
        ref[3,0,2,0]=(-1.72155871442)*x_ref[0]+(-1.28372407452)*x_ref[1]
        ref[3,0,2,1]=(1.43817914693)*x_ref[0]+(-0.354093253684)*x_ref[1]
        ref[3,1,0,0]=(-0.293939245102)*x_ref[0]+(0.0255562139103)*x_ref[1]
        ref[3,1,0,1]=(0.139306213018)*x_ref[0]+(0.595062226642)*x_ref[1]
        ref[3,1,1,0]=(-0.670339920672)*x_ref[0]+(0.387520583192)*x_ref[1]
        ref[3,1,1,1]=(-0.0323325470954)*x_ref[0]+(-0.681162999279)*x_ref[1]
        ref[3,1,2,0]=(0.235275048739)*x_ref[0]+(0.144725473773)*x_ref[1]
        ref[3,1,2,1]=(0.132211795214)*x_ref[0]+(0.262780216845)*x_ref[1]
        ref[3,2,0,0]=(0.590514326163)*x_ref[0]+(-0.84273677918)*x_ref[1]
        ref[3,2,0,1]=(-0.925181556185)*x_ref[0]+(-0.300244447001)*x_ref[1]
        ref[3,2,1,0]=(-0.0509700795112)*x_ref[0]+(-1.31919389616)*x_ref[1]
        ref[3,2,1,1]=(1.50577291846)*x_ref[0]+(-0.219284607769)*x_ref[1]
        ref[3,2,2,0]=(-0.0460616673346)*x_ref[0]+(-0.690312685466)*x_ref[1]
        ref[3,2,2,1]=(-0.673765477936)*x_ref[0]+(-0.0965860475641)*x_ref[1]
        ref[3,3,0,0]=(-0.6577594174)*x_ref[0]+(0.171886277016)*x_ref[1]
        ref[3,3,0,1]=(-1.08810434718)*x_ref[0]+(0.333153250183)*x_ref[1]
        ref[3,3,1,0]=(0.150968371983)*x_ref[0]+(0.290859317391)*x_ref[1]
        ref[3,3,1,1]=(1.62096265131)*x_ref[0]+(-1.10309602741)*x_ref[1]
        ref[3,3,2,0]=(-0.932063639972)*x_ref[0]+(-0.897131561126)*x_ref[1]
        ref[3,3,2,1]=(-0.819110834345)*x_ref[0]+(1.59159109562)*x_ref[1]
        ref[3,4,0,0]=(0.272662931155)*x_ref[0]+(0.69026285756)*x_ref[1]
        ref[3,4,0,1]=(-0.600556370757)*x_ref[0]+(-0.0853533546302)*x_ref[1]
        ref[3,4,1,0]=(1.18544735281)*x_ref[0]+(0.217751189034)*x_ref[1]
        ref[3,4,1,1]=(-0.594609531358)*x_ref[0]+(0.882158349788)*x_ref[1]
        ref[3,4,2,0]=(1.18876765552)*x_ref[0]+(-0.411481586979)*x_ref[1]
        ref[3,4,2,1]=(-1.093067792)*x_ref[0]+(-0.266744260768)*x_ref[1]
      else:
        arg[0,0,0,0]=(-1.29218217256)*x[0]+(-0.382937892213)*x[1]+(0.14363896447)*x[2]
        arg[0,0,0,1]=(1.39856942421)*x[0]+(0.302082267835)*x[1]+(-0.40870736584)*x[2]
        arg[0,0,1,0]=(0.523846108043)*x[0]+(1.19961531204)*x[1]+(0.838881761349)*x[2]
        arg[0,0,1,1]=(0.634542282764)*x[0]+(-0.475514686466)*x[1]+(-1.68745428486)*x[2]
        arg[0,0,2,0]=(1.16379585731)*x[0]+(-0.928037595416)*x[1]+(0.998857178353)*x[2]
        arg[0,0,2,1]=(-0.804515607022)*x[0]+(-0.915469493439)*x[1]+(-0.143888000623)*x[2]
        arg[0,1,0,0]=(0.925843155486)*x[0]+(-0.168243418953)*x[1]+(-0.887839555187)*x[2]
        arg[0,1,0,1]=(-0.913103105329)*x[0]+(1.38011681732)*x[1]+(-0.992478793291)*x[2]
        arg[0,1,1,0]=(0.759685289903)*x[0]+(-0.579481939982)*x[1]+(0.577682929138)*x[2]
        arg[0,1,1,1]=(0.813155548228)*x[0]+(-0.178317843217)*x[1]+(-0.0460113194526)*x[2]
        arg[0,1,2,0]=(-0.195764860141)*x[0]+(1.44256979628)*x[1]+(0.283992441343)*x[2]
        arg[0,1,2,1]=(-0.921229459072)*x[0]+(-1.19295761547)*x[1]+(0.215736801944)*x[2]
        arg[0,2,0,0]=(-0.741726331752)*x[0]+(-0.225585937382)*x[1]+(1.12654556466)*x[2]
        arg[0,2,0,1]=(-0.709656162421)*x[0]+(0.298862010425)*x[1]+(-0.502443284827)*x[2]
        arg[0,2,1,0]=(-1.01166188669)*x[0]+(-1.43763408953)*x[1]+(-0.835160398488)*x[2]
        arg[0,2,1,1]=(1.80968424545)*x[0]+(-0.802666103495)*x[1]+(0.290283606115)*x[2]
        arg[0,2,2,0]=(0.346481981585)*x[0]+(-0.444176962215)*x[1]+(0.689127612227)*x[2]
        arg[0,2,2,1]=(1.06925494095)*x[0]+(0.452854988449)*x[1]+(-1.19974690364)*x[2]
        arg[0,3,0,0]=(-0.534896036328)*x[0]+(-0.312000770615)*x[1]+(-0.440033746693)*x[2]
        arg[0,3,0,1]=(0.580351431838)*x[0]+(-0.36623848451)*x[1]+(-0.507837490037)*x[2]
        arg[0,3,1,0]=(0.383180910522)*x[0]+(-0.0907152443563)*x[1]+(-0.200103743943)*x[2]
        arg[0,3,1,1]=(0.0160194687513)*x[0]+(0.862509612939)*x[1]+(-1.14643594474)*x[2]
        arg[0,3,2,0]=(-0.224390850199)*x[0]+(-0.176927715447)*x[1]+(0.213480525831)*x[2]
        arg[0,3,2,1]=(-0.0508511448148)*x[0]+(-1.28943467389)*x[1]+(0.902285302487)*x[2]
        arg[0,4,0,0]=(-0.633106625232)*x[0]+(-0.430651917485)*x[1]+(0.583149841952)*x[2]
        arg[0,4,0,1]=(0.579035528956)*x[0]+(0.159439682519)*x[1]+(-0.0926277063021)*x[2]
        arg[0,4,1,0]=(0.00484756366973)*x[0]+(0.0324822270755)*x[1]+(1.20052413378)*x[2]
        arg[0,4,1,1]=(-0.337383994688)*x[0]+(0.902076367981)*x[1]+(-0.1956377395)*x[2]
        arg[0,4,2,0]=(0.928434343456)*x[0]+(0.0376569884355)*x[1]+(-0.155274118568)*x[2]
        arg[0,4,2,1]=(-0.541990213238)*x[0]+(-0.0165076697829)*x[1]+(-0.210319224859)*x[2]
        arg[1,0,0,0]=(-1.67762025181)*x[0]+(1.61434755579)*x[1]+(0.357272049999)*x[2]
        arg[1,0,0,1]=(-1.62532544549)*x[0]+(-0.271426231027)*x[1]+(0.338680991778)*x[2]
        arg[1,0,1,0]=(1.73523408033)*x[0]+(1.11987092368)*x[1]+(0.126769364273)*x[2]
        arg[1,0,1,1]=(-1.49220786558)*x[0]+(1.51915566092)*x[1]+(1.24788192157)*x[2]
        arg[1,0,2,0]=(-0.687808766765)*x[0]+(-1.02318754737)*x[1]+(0.590653972416)*x[2]
        arg[1,0,2,1]=(0.756067354802)*x[0]+(-0.66964706965)*x[1]+(-1.18986499158)*x[2]
        arg[1,1,0,0]=(0.371345732177)*x[0]+(0.212739617115)*x[1]+(-1.13439047405)*x[2]
        arg[1,1,0,1]=(0.525761804111)*x[0]+(0.144130399691)*x[1]+(0.715517587677)*x[2]
        arg[1,1,1,0]=(-0.376160293215)*x[0]+(-0.469964935263)*x[1]+(0.413518194707)*x[2]
        arg[1,1,1,1]=(-0.787748398144)*x[0]+(-0.810090402237)*x[1]+(-0.50127109256)*x[2]
        arg[1,1,2,0]=(0.654555083692)*x[0]+(-0.142134700515)*x[1]+(-1.55891930521)*x[2]
        arg[1,1,2,1]=(-0.423058885456)*x[0]+(-0.65930158641)*x[1]+(0.94440057744)*x[2]
        arg[1,2,0,0]=(-0.924832007053)*x[0]+(1.0154890382)*x[1]+(0.11938079748)*x[2]
        arg[1,2,0,1]=(0.134642686931)*x[0]+(-1.30908067998)*x[1]+(0.367206139074)*x[2]
        arg[1,2,1,0]=(1.55524055201)*x[0]+(0.727208660637)*x[1]+(0.809810917377)*x[2]
        arg[1,2,1,1]=(0.94813391462)*x[0]+(-1.49402524308)*x[1]+(0.367824542434)*x[2]
        arg[1,2,2,0]=(-0.608295798731)*x[0]+(-0.142302252814)*x[1]+(-0.986529251308)*x[2]
        arg[1,2,2,1]=(0.0259394619607)*x[0]+(-1.60728202467)*x[1]+(0.312534656619)*x[2]
        arg[1,3,0,0]=(-1.34063373729)*x[0]+(-0.268499839329)*x[1]+(-0.335125886299)*x[2]
        arg[1,3,0,1]=(1.53280698072)*x[0]+(1.24369684711)*x[1]+(0.766369156132)*x[2]
        arg[1,3,1,0]=(0.902067176876)*x[0]+(0.791102994814)*x[1]+(-0.89266430493)*x[2]
        arg[1,3,1,1]=(0.704997629232)*x[0]+(-0.551611360332)*x[1]+(1.39134537491)*x[2]
        arg[1,3,2,0]=(-0.151690675211)*x[0]+(-1.01762975893)*x[1]+(-0.627493214284)*x[2]
        arg[1,3,2,1]=(1.00936645922)*x[0]+(0.0322458997248)*x[1]+(1.45039938194)*x[2]
        arg[1,4,0,0]=(-0.0916390388904)*x[0]+(-0.783111746469)*x[1]+(0.389725342225)*x[2]
        arg[1,4,0,1]=(1.18859328243)*x[0]+(0.871689661558)*x[1]+(-0.775464713448)*x[2]
        arg[1,4,1,0]=(0.913118342488)*x[0]+(1.53285648884)*x[1]+(0.0947365101243)*x[2]
        arg[1,4,1,1]=(-1.00735841224)*x[0]+(-1.12544193607)*x[1]+(0.267347229372)*x[2]
        arg[1,4,2,0]=(0.285943438284)*x[0]+(-1.0889389244)*x[1]+(-1.03054842327)*x[2]
        arg[1,4,2,1]=(-1.5768940139)*x[0]+(0.91237754587)*x[1]+(1.30220394199)*x[2]
        arg[2,0,0,0]=(-0.0634935883277)*x[0]+(0.157304131712)*x[1]+(-1.2995042074)*x[2]
        arg[2,0,0,1]=(-0.802705266871)*x[0]+(0.238233162774)*x[1]+(1.29463414013)*x[2]
        arg[2,0,1,0]=(-0.925468462671)*x[0]+(0.870960795966)*x[1]+(0.156522765713)*x[2]
        arg[2,0,1,1]=(-0.546666833309)*x[0]+(-1.14649607995)*x[1]+(-1.53906505048)*x[2]
        arg[2,0,2,0]=(0.464952910857)*x[0]+(0.408605786027)*x[1]+(-0.886974881114)*x[2]
        arg[2,0,2,1]=(0.491723247948)*x[0]+(0.194616420077)*x[1]+(0.409049782749)*x[2]
        arg[2,1,0,0]=(-1.19836883114)*x[0]+(1.0622722337)*x[1]+(-0.109193867727)*x[2]
        arg[2,1,0,1]=(1.01973600383)*x[0]+(-0.154482220539)*x[1]+(0.378696371956)*x[2]
        arg[2,1,1,0]=(-0.0812015296246)*x[0]+(-1.10252794568)*x[1]+(-0.00585426581857)*x[2]
        arg[2,1,1,1]=(0.536042665272)*x[0]+(0.685507117255)*x[1]+(-0.179652683139)*x[2]
        arg[2,1,2,0]=(-0.36750467811)*x[0]+(-1.23674550869)*x[1]+(-0.762905702801)*x[2]
        arg[2,1,2,1]=(0.329665965651)*x[0]+(0.101195810737)*x[1]+(-0.610015052375)*x[2]
        arg[2,2,0,0]=(-0.81644635185)*x[0]+(-0.604578476306)*x[1]+(0.0440925706991)*x[2]
        arg[2,2,0,1]=(-0.958241437971)*x[0]+(0.38563498206)*x[1]+(0.913406762318)*x[2]
        arg[2,2,1,0]=(1.22027492224)*x[0]+(-1.00726209939)*x[1]+(-1.05083068303)*x[2]
        arg[2,2,1,1]=(-1.47561192645)*x[0]+(-0.903549355608)*x[1]+(0.236771324671)*x[2]
        arg[2,2,2,0]=(0.284995864449)*x[0]+(-0.175760829921)*x[1]+(-0.720607220001)*x[2]
        arg[2,2,2,1]=(-0.535894515963)*x[0]+(-0.821302957162)*x[1]+(-1.75141540241)*x[2]
        arg[2,3,0,0]=(-0.333082106674)*x[0]+(0.579471826412)*x[1]+(0.561184138108)*x[2]
        arg[2,3,0,1]=(1.34803519557)*x[0]+(-0.441299088849)*x[1]+(-1.04945951616)*x[2]
        arg[2,3,1,0]=(0.299011096602)*x[0]+(1.32726458235)*x[1]+(-0.196298687359)*x[2]
        arg[2,3,1,1]=(1.48698834204)*x[0]+(0.527221496971)*x[1]+(0.0819813519422)*x[2]
        arg[2,3,2,0]=(0.681680955153)*x[0]+(0.503653972357)*x[1]+(0.434669347891)*x[2]
        arg[2,3,2,1]=(-0.152586789311)*x[0]+(-0.765242635579)*x[1]+(0.712663994387)*x[2]
        arg[2,4,0,0]=(0.630152300463)*x[0]+(1.12366733588)*x[1]+(-0.662966700638)*x[2]
        arg[2,4,0,1]=(1.02796709755)*x[0]+(-0.24093881584)*x[1]+(1.79445417019)*x[2]
        arg[2,4,1,0]=(-0.954575760758)*x[0]+(0.16331782608)*x[1]+(1.38309921277)*x[2]
        arg[2,4,1,1]=(0.536092514281)*x[0]+(-0.276364952673)*x[1]+(0.779619701528)*x[2]
        arg[2,4,2,0]=(0.267270444141)*x[0]+(-0.810746424331)*x[1]+(-0.733877853264)*x[2]
        arg[2,4,2,1]=(-0.0641688847923)*x[0]+(-0.386279280667)*x[1]+(0.0637571595255)*x[2]
        arg[3,0,0,0]=(0.469903180687)*x[0]+(-0.793812397342)*x[1]+(1.60841921779)*x[2]
        arg[3,0,0,1]=(0.720057600102)*x[0]+(0.502582253261)*x[1]+(0.460147791003)*x[2]
        arg[3,0,1,0]=(0.0327862564137)*x[0]+(-1.142962965)*x[1]+(0.114799131894)*x[2]
        arg[3,0,1,1]=(-0.997412656978)*x[0]+(-0.678175468576)*x[1]+(-0.152663147682)*x[2]
        arg[3,0,2,0]=(0.810424599078)*x[0]+(0.75522727786)*x[1]+(-0.205880941501)*x[2]
        arg[3,0,2,1]=(0.353601949476)*x[0]+(0.0963200813379)*x[1]+(0.0578829287146)*x[2]
        arg[3,1,0,0]=(0.346590730062)*x[0]+(1.07953790092)*x[1]+(0.647586851217)*x[2]
        arg[3,1,0,1]=(0.889879195334)*x[0]+(-0.352788186249)*x[1]+(0.434900135803)*x[2]
        arg[3,1,1,0]=(-0.721742734959)*x[0]+(0.166066672134)*x[1]+(-0.489266924809)*x[2]
        arg[3,1,1,1]=(-0.810487755181)*x[0]+(0.0354916979253)*x[1]+(0.0735520654395)*x[2]
        arg[3,1,2,0]=(1.02414338939)*x[0]+(-0.851076284556)*x[1]+(-0.232174494405)*x[2]
        arg[3,1,2,1]=(-0.327246566026)*x[0]+(-0.0499396023605)*x[1]+(0.611585402379)*x[2]
        arg[3,2,0,0]=(-0.365757628557)*x[0]+(0.868697342115)*x[1]+(-1.102545972)*x[2]
        arg[3,2,0,1]=(0.628592129227)*x[0]+(1.25545515311)*x[1]+(-1.0577476264)*x[2]
        arg[3,2,1,0]=(0.160995978044)*x[0]+(-0.947389499556)*x[1]+(-0.870476160656)*x[2]
        arg[3,2,1,1]=(0.910787139938)*x[0]+(-0.0681493345139)*x[1]+(-0.561910153781)*x[2]
        arg[3,2,2,0]=(0.0761644634428)*x[0]+(-0.700940967343)*x[1]+(1.19407210904)*x[2]
        arg[3,2,2,1]=(1.75717437065)*x[0]+(-0.144457317494)*x[1]+(-0.523233453079)*x[2]
        arg[3,3,0,0]=(0.448782872606)*x[0]+(0.650464256547)*x[1]+(1.19476547456)*x[2]
        arg[3,3,0,1]=(0.118586027201)*x[0]+(0.449971816918)*x[1]+(-0.510812733389)*x[2]
        arg[3,3,1,0]=(-0.330032265081)*x[0]+(-0.13657534891)*x[1]+(0.456365289556)*x[2]
        arg[3,3,1,1]=(0.697584212635)*x[0]+(0.897604241724)*x[1]+(1.31517116663)*x[2]
        arg[3,3,2,0]=(-0.940538707365)*x[0]+(-0.334862221665)*x[1]+(0.0969742096754)*x[2]
        arg[3,3,2,1]=(-0.985788813516)*x[0]+(0.348389725548)*x[1]+(-0.260816590361)*x[2]
        arg[3,4,0,0]=(0.754692421417)*x[0]+(0.775985781769)*x[1]+(-0.0866159222457)*x[2]
        arg[3,4,0,1]=(-1.17670502429)*x[0]+(1.12268279393)*x[1]+(-1.1279822095)*x[2]
        arg[3,4,1,0]=(-1.48674370666)*x[0]+(1.26685445713)*x[1]+(-0.0522828809306)*x[2]
        arg[3,4,1,1]=(0.401760689843)*x[0]+(-1.35707797971)*x[1]+(-0.0580654273828)*x[2]
        arg[3,4,2,0]=(-1.40697830852)*x[0]+(0.933923369958)*x[1]+(0.230152169159)*x[2]
        arg[3,4,2,1]=(-1.21242130555)*x[0]+(0.858316339663)*x[1]+(-0.839747078526)*x[2]
        ref[0,0,0,0]=(-1.29218217256)*x_ref[0]+(-0.382937892213)*x_ref[1]+(0.14363896447)*x_ref[2]
        ref[0,0,0,1]=(1.39856942421)*x_ref[0]+(0.302082267835)*x_ref[1]+(-0.40870736584)*x_ref[2]
        ref[0,0,1,0]=(0.523846108043)*x_ref[0]+(1.19961531204)*x_ref[1]+(0.838881761349)*x_ref[2]
        ref[0,0,1,1]=(0.634542282764)*x_ref[0]+(-0.475514686466)*x_ref[1]+(-1.68745428486)*x_ref[2]
        ref[0,0,2,0]=(1.16379585731)*x_ref[0]+(-0.928037595416)*x_ref[1]+(0.998857178353)*x_ref[2]
        ref[0,0,2,1]=(-0.804515607022)*x_ref[0]+(-0.915469493439)*x_ref[1]+(-0.143888000623)*x_ref[2]
        ref[0,1,0,0]=(0.925843155486)*x_ref[0]+(-0.168243418953)*x_ref[1]+(-0.887839555187)*x_ref[2]
        ref[0,1,0,1]=(-0.913103105329)*x_ref[0]+(1.38011681732)*x_ref[1]+(-0.992478793291)*x_ref[2]
        ref[0,1,1,0]=(0.759685289903)*x_ref[0]+(-0.579481939982)*x_ref[1]+(0.577682929138)*x_ref[2]
        ref[0,1,1,1]=(0.813155548228)*x_ref[0]+(-0.178317843217)*x_ref[1]+(-0.0460113194526)*x_ref[2]
        ref[0,1,2,0]=(-0.195764860141)*x_ref[0]+(1.44256979628)*x_ref[1]+(0.283992441343)*x_ref[2]
        ref[0,1,2,1]=(-0.921229459072)*x_ref[0]+(-1.19295761547)*x_ref[1]+(0.215736801944)*x_ref[2]
        ref[0,2,0,0]=(-0.741726331752)*x_ref[0]+(-0.225585937382)*x_ref[1]+(1.12654556466)*x_ref[2]
        ref[0,2,0,1]=(-0.709656162421)*x_ref[0]+(0.298862010425)*x_ref[1]+(-0.502443284827)*x_ref[2]
        ref[0,2,1,0]=(-1.01166188669)*x_ref[0]+(-1.43763408953)*x_ref[1]+(-0.835160398488)*x_ref[2]
        ref[0,2,1,1]=(1.80968424545)*x_ref[0]+(-0.802666103495)*x_ref[1]+(0.290283606115)*x_ref[2]
        ref[0,2,2,0]=(0.346481981585)*x_ref[0]+(-0.444176962215)*x_ref[1]+(0.689127612227)*x_ref[2]
        ref[0,2,2,1]=(1.06925494095)*x_ref[0]+(0.452854988449)*x_ref[1]+(-1.19974690364)*x_ref[2]
        ref[0,3,0,0]=(-0.534896036328)*x_ref[0]+(-0.312000770615)*x_ref[1]+(-0.440033746693)*x_ref[2]
        ref[0,3,0,1]=(0.580351431838)*x_ref[0]+(-0.36623848451)*x_ref[1]+(-0.507837490037)*x_ref[2]
        ref[0,3,1,0]=(0.383180910522)*x_ref[0]+(-0.0907152443563)*x_ref[1]+(-0.200103743943)*x_ref[2]
        ref[0,3,1,1]=(0.0160194687513)*x_ref[0]+(0.862509612939)*x_ref[1]+(-1.14643594474)*x_ref[2]
        ref[0,3,2,0]=(-0.224390850199)*x_ref[0]+(-0.176927715447)*x_ref[1]+(0.213480525831)*x_ref[2]
        ref[0,3,2,1]=(-0.0508511448148)*x_ref[0]+(-1.28943467389)*x_ref[1]+(0.902285302487)*x_ref[2]
        ref[0,4,0,0]=(-0.633106625232)*x_ref[0]+(-0.430651917485)*x_ref[1]+(0.583149841952)*x_ref[2]
        ref[0,4,0,1]=(0.579035528956)*x_ref[0]+(0.159439682519)*x_ref[1]+(-0.0926277063021)*x_ref[2]
        ref[0,4,1,0]=(0.00484756366973)*x_ref[0]+(0.0324822270755)*x_ref[1]+(1.20052413378)*x_ref[2]
        ref[0,4,1,1]=(-0.337383994688)*x_ref[0]+(0.902076367981)*x_ref[1]+(-0.1956377395)*x_ref[2]
        ref[0,4,2,0]=(0.928434343456)*x_ref[0]+(0.0376569884355)*x_ref[1]+(-0.155274118568)*x_ref[2]
        ref[0,4,2,1]=(-0.541990213238)*x_ref[0]+(-0.0165076697829)*x_ref[1]+(-0.210319224859)*x_ref[2]
        ref[1,0,0,0]=(-1.67762025181)*x_ref[0]+(1.61434755579)*x_ref[1]+(0.357272049999)*x_ref[2]
        ref[1,0,0,1]=(-1.62532544549)*x_ref[0]+(-0.271426231027)*x_ref[1]+(0.338680991778)*x_ref[2]
        ref[1,0,1,0]=(1.73523408033)*x_ref[0]+(1.11987092368)*x_ref[1]+(0.126769364273)*x_ref[2]
        ref[1,0,1,1]=(-1.49220786558)*x_ref[0]+(1.51915566092)*x_ref[1]+(1.24788192157)*x_ref[2]
        ref[1,0,2,0]=(-0.687808766765)*x_ref[0]+(-1.02318754737)*x_ref[1]+(0.590653972416)*x_ref[2]
        ref[1,0,2,1]=(0.756067354802)*x_ref[0]+(-0.66964706965)*x_ref[1]+(-1.18986499158)*x_ref[2]
        ref[1,1,0,0]=(0.371345732177)*x_ref[0]+(0.212739617115)*x_ref[1]+(-1.13439047405)*x_ref[2]
        ref[1,1,0,1]=(0.525761804111)*x_ref[0]+(0.144130399691)*x_ref[1]+(0.715517587677)*x_ref[2]
        ref[1,1,1,0]=(-0.376160293215)*x_ref[0]+(-0.469964935263)*x_ref[1]+(0.413518194707)*x_ref[2]
        ref[1,1,1,1]=(-0.787748398144)*x_ref[0]+(-0.810090402237)*x_ref[1]+(-0.50127109256)*x_ref[2]
        ref[1,1,2,0]=(0.654555083692)*x_ref[0]+(-0.142134700515)*x_ref[1]+(-1.55891930521)*x_ref[2]
        ref[1,1,2,1]=(-0.423058885456)*x_ref[0]+(-0.65930158641)*x_ref[1]+(0.94440057744)*x_ref[2]
        ref[1,2,0,0]=(-0.924832007053)*x_ref[0]+(1.0154890382)*x_ref[1]+(0.11938079748)*x_ref[2]
        ref[1,2,0,1]=(0.134642686931)*x_ref[0]+(-1.30908067998)*x_ref[1]+(0.367206139074)*x_ref[2]
        ref[1,2,1,0]=(1.55524055201)*x_ref[0]+(0.727208660637)*x_ref[1]+(0.809810917377)*x_ref[2]
        ref[1,2,1,1]=(0.94813391462)*x_ref[0]+(-1.49402524308)*x_ref[1]+(0.367824542434)*x_ref[2]
        ref[1,2,2,0]=(-0.608295798731)*x_ref[0]+(-0.142302252814)*x_ref[1]+(-0.986529251308)*x_ref[2]
        ref[1,2,2,1]=(0.0259394619607)*x_ref[0]+(-1.60728202467)*x_ref[1]+(0.312534656619)*x_ref[2]
        ref[1,3,0,0]=(-1.34063373729)*x_ref[0]+(-0.268499839329)*x_ref[1]+(-0.335125886299)*x_ref[2]
        ref[1,3,0,1]=(1.53280698072)*x_ref[0]+(1.24369684711)*x_ref[1]+(0.766369156132)*x_ref[2]
        ref[1,3,1,0]=(0.902067176876)*x_ref[0]+(0.791102994814)*x_ref[1]+(-0.89266430493)*x_ref[2]
        ref[1,3,1,1]=(0.704997629232)*x_ref[0]+(-0.551611360332)*x_ref[1]+(1.39134537491)*x_ref[2]
        ref[1,3,2,0]=(-0.151690675211)*x_ref[0]+(-1.01762975893)*x_ref[1]+(-0.627493214284)*x_ref[2]
        ref[1,3,2,1]=(1.00936645922)*x_ref[0]+(0.0322458997248)*x_ref[1]+(1.45039938194)*x_ref[2]
        ref[1,4,0,0]=(-0.0916390388904)*x_ref[0]+(-0.783111746469)*x_ref[1]+(0.389725342225)*x_ref[2]
        ref[1,4,0,1]=(1.18859328243)*x_ref[0]+(0.871689661558)*x_ref[1]+(-0.775464713448)*x_ref[2]
        ref[1,4,1,0]=(0.913118342488)*x_ref[0]+(1.53285648884)*x_ref[1]+(0.0947365101243)*x_ref[2]
        ref[1,4,1,1]=(-1.00735841224)*x_ref[0]+(-1.12544193607)*x_ref[1]+(0.267347229372)*x_ref[2]
        ref[1,4,2,0]=(0.285943438284)*x_ref[0]+(-1.0889389244)*x_ref[1]+(-1.03054842327)*x_ref[2]
        ref[1,4,2,1]=(-1.5768940139)*x_ref[0]+(0.91237754587)*x_ref[1]+(1.30220394199)*x_ref[2]
        ref[2,0,0,0]=(-0.0634935883277)*x_ref[0]+(0.157304131712)*x_ref[1]+(-1.2995042074)*x_ref[2]
        ref[2,0,0,1]=(-0.802705266871)*x_ref[0]+(0.238233162774)*x_ref[1]+(1.29463414013)*x_ref[2]
        ref[2,0,1,0]=(-0.925468462671)*x_ref[0]+(0.870960795966)*x_ref[1]+(0.156522765713)*x_ref[2]
        ref[2,0,1,1]=(-0.546666833309)*x_ref[0]+(-1.14649607995)*x_ref[1]+(-1.53906505048)*x_ref[2]
        ref[2,0,2,0]=(0.464952910857)*x_ref[0]+(0.408605786027)*x_ref[1]+(-0.886974881114)*x_ref[2]
        ref[2,0,2,1]=(0.491723247948)*x_ref[0]+(0.194616420077)*x_ref[1]+(0.409049782749)*x_ref[2]
        ref[2,1,0,0]=(-1.19836883114)*x_ref[0]+(1.0622722337)*x_ref[1]+(-0.109193867727)*x_ref[2]
        ref[2,1,0,1]=(1.01973600383)*x_ref[0]+(-0.154482220539)*x_ref[1]+(0.378696371956)*x_ref[2]
        ref[2,1,1,0]=(-0.0812015296246)*x_ref[0]+(-1.10252794568)*x_ref[1]+(-0.00585426581857)*x_ref[2]
        ref[2,1,1,1]=(0.536042665272)*x_ref[0]+(0.685507117255)*x_ref[1]+(-0.179652683139)*x_ref[2]
        ref[2,1,2,0]=(-0.36750467811)*x_ref[0]+(-1.23674550869)*x_ref[1]+(-0.762905702801)*x_ref[2]
        ref[2,1,2,1]=(0.329665965651)*x_ref[0]+(0.101195810737)*x_ref[1]+(-0.610015052375)*x_ref[2]
        ref[2,2,0,0]=(-0.81644635185)*x_ref[0]+(-0.604578476306)*x_ref[1]+(0.0440925706991)*x_ref[2]
        ref[2,2,0,1]=(-0.958241437971)*x_ref[0]+(0.38563498206)*x_ref[1]+(0.913406762318)*x_ref[2]
        ref[2,2,1,0]=(1.22027492224)*x_ref[0]+(-1.00726209939)*x_ref[1]+(-1.05083068303)*x_ref[2]
        ref[2,2,1,1]=(-1.47561192645)*x_ref[0]+(-0.903549355608)*x_ref[1]+(0.236771324671)*x_ref[2]
        ref[2,2,2,0]=(0.284995864449)*x_ref[0]+(-0.175760829921)*x_ref[1]+(-0.720607220001)*x_ref[2]
        ref[2,2,2,1]=(-0.535894515963)*x_ref[0]+(-0.821302957162)*x_ref[1]+(-1.75141540241)*x_ref[2]
        ref[2,3,0,0]=(-0.333082106674)*x_ref[0]+(0.579471826412)*x_ref[1]+(0.561184138108)*x_ref[2]
        ref[2,3,0,1]=(1.34803519557)*x_ref[0]+(-0.441299088849)*x_ref[1]+(-1.04945951616)*x_ref[2]
        ref[2,3,1,0]=(0.299011096602)*x_ref[0]+(1.32726458235)*x_ref[1]+(-0.196298687359)*x_ref[2]
        ref[2,3,1,1]=(1.48698834204)*x_ref[0]+(0.527221496971)*x_ref[1]+(0.0819813519422)*x_ref[2]
        ref[2,3,2,0]=(0.681680955153)*x_ref[0]+(0.503653972357)*x_ref[1]+(0.434669347891)*x_ref[2]
        ref[2,3,2,1]=(-0.152586789311)*x_ref[0]+(-0.765242635579)*x_ref[1]+(0.712663994387)*x_ref[2]
        ref[2,4,0,0]=(0.630152300463)*x_ref[0]+(1.12366733588)*x_ref[1]+(-0.662966700638)*x_ref[2]
        ref[2,4,0,1]=(1.02796709755)*x_ref[0]+(-0.24093881584)*x_ref[1]+(1.79445417019)*x_ref[2]
        ref[2,4,1,0]=(-0.954575760758)*x_ref[0]+(0.16331782608)*x_ref[1]+(1.38309921277)*x_ref[2]
        ref[2,4,1,1]=(0.536092514281)*x_ref[0]+(-0.276364952673)*x_ref[1]+(0.779619701528)*x_ref[2]
        ref[2,4,2,0]=(0.267270444141)*x_ref[0]+(-0.810746424331)*x_ref[1]+(-0.733877853264)*x_ref[2]
        ref[2,4,2,1]=(-0.0641688847923)*x_ref[0]+(-0.386279280667)*x_ref[1]+(0.0637571595255)*x_ref[2]
        ref[3,0,0,0]=(0.469903180687)*x_ref[0]+(-0.793812397342)*x_ref[1]+(1.60841921779)*x_ref[2]
        ref[3,0,0,1]=(0.720057600102)*x_ref[0]+(0.502582253261)*x_ref[1]+(0.460147791003)*x_ref[2]
        ref[3,0,1,0]=(0.0327862564137)*x_ref[0]+(-1.142962965)*x_ref[1]+(0.114799131894)*x_ref[2]
        ref[3,0,1,1]=(-0.997412656978)*x_ref[0]+(-0.678175468576)*x_ref[1]+(-0.152663147682)*x_ref[2]
        ref[3,0,2,0]=(0.810424599078)*x_ref[0]+(0.75522727786)*x_ref[1]+(-0.205880941501)*x_ref[2]
        ref[3,0,2,1]=(0.353601949476)*x_ref[0]+(0.0963200813379)*x_ref[1]+(0.0578829287146)*x_ref[2]
        ref[3,1,0,0]=(0.346590730062)*x_ref[0]+(1.07953790092)*x_ref[1]+(0.647586851217)*x_ref[2]
        ref[3,1,0,1]=(0.889879195334)*x_ref[0]+(-0.352788186249)*x_ref[1]+(0.434900135803)*x_ref[2]
        ref[3,1,1,0]=(-0.721742734959)*x_ref[0]+(0.166066672134)*x_ref[1]+(-0.489266924809)*x_ref[2]
        ref[3,1,1,1]=(-0.810487755181)*x_ref[0]+(0.0354916979253)*x_ref[1]+(0.0735520654395)*x_ref[2]
        ref[3,1,2,0]=(1.02414338939)*x_ref[0]+(-0.851076284556)*x_ref[1]+(-0.232174494405)*x_ref[2]
        ref[3,1,2,1]=(-0.327246566026)*x_ref[0]+(-0.0499396023605)*x_ref[1]+(0.611585402379)*x_ref[2]
        ref[3,2,0,0]=(-0.365757628557)*x_ref[0]+(0.868697342115)*x_ref[1]+(-1.102545972)*x_ref[2]
        ref[3,2,0,1]=(0.628592129227)*x_ref[0]+(1.25545515311)*x_ref[1]+(-1.0577476264)*x_ref[2]
        ref[3,2,1,0]=(0.160995978044)*x_ref[0]+(-0.947389499556)*x_ref[1]+(-0.870476160656)*x_ref[2]
        ref[3,2,1,1]=(0.910787139938)*x_ref[0]+(-0.0681493345139)*x_ref[1]+(-0.561910153781)*x_ref[2]
        ref[3,2,2,0]=(0.0761644634428)*x_ref[0]+(-0.700940967343)*x_ref[1]+(1.19407210904)*x_ref[2]
        ref[3,2,2,1]=(1.75717437065)*x_ref[0]+(-0.144457317494)*x_ref[1]+(-0.523233453079)*x_ref[2]
        ref[3,3,0,0]=(0.448782872606)*x_ref[0]+(0.650464256547)*x_ref[1]+(1.19476547456)*x_ref[2]
        ref[3,3,0,1]=(0.118586027201)*x_ref[0]+(0.449971816918)*x_ref[1]+(-0.510812733389)*x_ref[2]
        ref[3,3,1,0]=(-0.330032265081)*x_ref[0]+(-0.13657534891)*x_ref[1]+(0.456365289556)*x_ref[2]
        ref[3,3,1,1]=(0.697584212635)*x_ref[0]+(0.897604241724)*x_ref[1]+(1.31517116663)*x_ref[2]
        ref[3,3,2,0]=(-0.940538707365)*x_ref[0]+(-0.334862221665)*x_ref[1]+(0.0969742096754)*x_ref[2]
        ref[3,3,2,1]=(-0.985788813516)*x_ref[0]+(0.348389725548)*x_ref[1]+(-0.260816590361)*x_ref[2]
        ref[3,4,0,0]=(0.754692421417)*x_ref[0]+(0.775985781769)*x_ref[1]+(-0.0866159222457)*x_ref[2]
        ref[3,4,0,1]=(-1.17670502429)*x_ref[0]+(1.12268279393)*x_ref[1]+(-1.1279822095)*x_ref[2]
        ref[3,4,1,0]=(-1.48674370666)*x_ref[0]+(1.26685445713)*x_ref[1]+(-0.0522828809306)*x_ref[2]
        ref[3,4,1,1]=(0.401760689843)*x_ref[0]+(-1.35707797971)*x_ref[1]+(-0.0580654273828)*x_ref[2]
        ref[3,4,2,0]=(-1.40697830852)*x_ref[0]+(0.933923369958)*x_ref[1]+(0.230152169159)*x_ref[2]
        ref[3,4,2,1]=(-1.21242130555)*x_ref[0]+(0.858316339663)*x_ref[1]+(-0.839747078526)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromData_ReducedFunctionOnContactZero_rank0(self):
      """
      tests interpolate for rank 0 Data onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactZero onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactZero(self.domain)
      x=w.getX()
      arg=Data(0,(),w)
      ref=Data(0,(),w_ref)
      if dim==2:
        arg=(-0.485106931655)*x[0]**o+(-0.45911691249)*x[0]+(-0.780654139819)*x[1]**o+(-0.496275334303)*x[1]
        ref=(-0.485106931655)*x_ref[0]**o+(-0.45911691249)*x_ref[0]+(-0.780654139819)*x_ref[1]**o+(-0.496275334303)*x_ref[1]
      else:
        arg=(-0.312233545328)*x[0]**o+(0.386652391857)*x[0]+(0.893938213685)*x[1]**o+(-0.109836308832)*x[1]+(0.293419600866)*x[2]**o+(-0.802363407879)*x[2]
        ref=(-0.312233545328)*x_ref[0]**o+(0.386652391857)*x_ref[0]+(0.893938213685)*x_ref[1]**o+(-0.109836308832)*x_ref[1]+(0.293419600866)*x_ref[2]**o+(-0.802363407879)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromData_ReducedFunctionOnContactZero_rank1(self):
      """
      tests interpolate for rank 1 Data onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactZero onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactZero(self.domain)
      x=w.getX()
      arg=Data(0,(2,),w)
      ref=Data(0,(2,),w_ref)
      if dim==2:
        arg[0]=(0.383147346126)*x[0]**o+(-0.103098043084)*x[0]+(0.389581571033)*x[1]**o+(0.647441523744)*x[1]
        arg[1]=(-0.0718078186662)*x[0]**o+(-0.975209175853)*x[0]+(-0.62913306056)*x[1]**o+(0.648038487393)*x[1]
        ref[0]=(0.383147346126)*x_ref[0]**o+(-0.103098043084)*x_ref[0]+(0.389581571033)*x_ref[1]**o+(0.647441523744)*x_ref[1]
        ref[1]=(-0.0718078186662)*x_ref[0]**o+(-0.975209175853)*x_ref[0]+(-0.62913306056)*x_ref[1]**o+(0.648038487393)*x_ref[1]
      else:
        arg[0]=(-0.392958184101)*x[0]**o+(-0.529709692826)*x[0]+(-0.834422900831)*x[1]**o+(0.52562214773)*x[1]+(-0.726762801989)*x[2]**o+(-0.329225625652)*x[2]
        arg[1]=(-0.388486489727)*x[0]**o+(0.62353995706)*x[0]+(0.0198506797091)*x[1]**o+(0.744336375443)*x[1]+(0.115966573066)*x[2]**o+(-0.195838697299)*x[2]
        ref[0]=(-0.392958184101)*x_ref[0]**o+(-0.529709692826)*x_ref[0]+(-0.834422900831)*x_ref[1]**o+(0.52562214773)*x_ref[1]+(-0.726762801989)*x_ref[2]**o+(-0.329225625652)*x_ref[2]
        ref[1]=(-0.388486489727)*x_ref[0]**o+(0.62353995706)*x_ref[0]+(0.0198506797091)*x_ref[1]**o+(0.744336375443)*x_ref[1]+(0.115966573066)*x_ref[2]**o+(-0.195838697299)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromData_ReducedFunctionOnContactZero_rank2(self):
      """
      tests interpolate for rank 2 Data onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactZero onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactZero(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5),w)
      ref=Data(0,(4, 5),w_ref)
      if dim==2:
        arg[0,0]=(-0.657915437562)*x[0]**o+(-0.545113889022)*x[0]+(0.531900702747)*x[1]**o+(0.593791195717)*x[1]
        arg[0,1]=(0.051080053648)*x[0]**o+(-0.693135614033)*x[0]+(0.132124625293)*x[1]**o+(0.790120778986)*x[1]
        arg[0,2]=(-0.474326324304)*x[0]**o+(0.844119715746)*x[0]+(-0.107139623866)*x[1]**o+(0.456623369855)*x[1]
        arg[0,3]=(-0.582421376463)*x[0]**o+(0.972810074845)*x[0]+(0.875278994526)*x[1]**o+(-0.382656553829)*x[1]
        arg[0,4]=(-0.904956623057)*x[0]**o+(0.701968065151)*x[0]+(-0.710231782855)*x[1]**o+(-0.123456684979)*x[1]
        arg[1,0]=(0.361017539866)*x[0]**o+(0.865249575899)*x[0]+(0.529515655309)*x[1]**o+(0.161402780958)*x[1]
        arg[1,1]=(0.813945406612)*x[0]**o+(0.801153720706)*x[0]+(-0.254227034266)*x[1]**o+(-0.422848409284)*x[1]
        arg[1,2]=(0.56207473833)*x[0]**o+(-0.831638855484)*x[0]+(-0.584619648965)*x[1]**o+(0.453944553892)*x[1]
        arg[1,3]=(0.25526821881)*x[0]**o+(-0.436135120045)*x[0]+(0.524842439137)*x[1]**o+(0.568007117728)*x[1]
        arg[1,4]=(-0.0587551077421)*x[0]**o+(-0.434635762185)*x[0]+(-0.592094375986)*x[1]**o+(0.873123434395)*x[1]
        arg[2,0]=(0.266819439236)*x[0]**o+(-0.45156140915)*x[0]+(0.436756372775)*x[1]**o+(0.12295128383)*x[1]
        arg[2,1]=(0.347128677451)*x[0]**o+(-0.0983169467916)*x[0]+(-0.414573209624)*x[1]**o+(-0.171300108487)*x[1]
        arg[2,2]=(-0.279695154296)*x[0]**o+(-0.911450144779)*x[0]+(0.5055340448)*x[1]**o+(-0.858058864152)*x[1]
        arg[2,3]=(-0.53768906863)*x[0]**o+(-0.0626923004303)*x[0]+(0.72295545269)*x[1]**o+(-0.77070425804)*x[1]
        arg[2,4]=(0.18215032008)*x[0]**o+(-0.965014882526)*x[0]+(0.596564813119)*x[1]**o+(0.0817024840554)*x[1]
        arg[3,0]=(-0.572404596368)*x[0]**o+(0.488365635829)*x[0]+(0.145439800826)*x[1]**o+(-0.337589667899)*x[1]
        arg[3,1]=(-0.628977426654)*x[0]**o+(0.22731945375)*x[0]+(0.989224773975)*x[1]**o+(0.0691553279499)*x[1]
        arg[3,2]=(-0.43609079916)*x[0]**o+(0.966113801416)*x[0]+(-0.853136416278)*x[1]**o+(-0.0125151911906)*x[1]
        arg[3,3]=(0.340159347855)*x[0]**o+(-0.254775013843)*x[0]+(0.795261224144)*x[1]**o+(-0.692255020659)*x[1]
        arg[3,4]=(-0.153521724504)*x[0]**o+(-0.681779299881)*x[0]+(0.693860779041)*x[1]**o+(0.882012604794)*x[1]
        ref[0,0]=(-0.657915437562)*x_ref[0]**o+(-0.545113889022)*x_ref[0]+(0.531900702747)*x_ref[1]**o+(0.593791195717)*x_ref[1]
        ref[0,1]=(0.051080053648)*x_ref[0]**o+(-0.693135614033)*x_ref[0]+(0.132124625293)*x_ref[1]**o+(0.790120778986)*x_ref[1]
        ref[0,2]=(-0.474326324304)*x_ref[0]**o+(0.844119715746)*x_ref[0]+(-0.107139623866)*x_ref[1]**o+(0.456623369855)*x_ref[1]
        ref[0,3]=(-0.582421376463)*x_ref[0]**o+(0.972810074845)*x_ref[0]+(0.875278994526)*x_ref[1]**o+(-0.382656553829)*x_ref[1]
        ref[0,4]=(-0.904956623057)*x_ref[0]**o+(0.701968065151)*x_ref[0]+(-0.710231782855)*x_ref[1]**o+(-0.123456684979)*x_ref[1]
        ref[1,0]=(0.361017539866)*x_ref[0]**o+(0.865249575899)*x_ref[0]+(0.529515655309)*x_ref[1]**o+(0.161402780958)*x_ref[1]
        ref[1,1]=(0.813945406612)*x_ref[0]**o+(0.801153720706)*x_ref[0]+(-0.254227034266)*x_ref[1]**o+(-0.422848409284)*x_ref[1]
        ref[1,2]=(0.56207473833)*x_ref[0]**o+(-0.831638855484)*x_ref[0]+(-0.584619648965)*x_ref[1]**o+(0.453944553892)*x_ref[1]
        ref[1,3]=(0.25526821881)*x_ref[0]**o+(-0.436135120045)*x_ref[0]+(0.524842439137)*x_ref[1]**o+(0.568007117728)*x_ref[1]
        ref[1,4]=(-0.0587551077421)*x_ref[0]**o+(-0.434635762185)*x_ref[0]+(-0.592094375986)*x_ref[1]**o+(0.873123434395)*x_ref[1]
        ref[2,0]=(0.266819439236)*x_ref[0]**o+(-0.45156140915)*x_ref[0]+(0.436756372775)*x_ref[1]**o+(0.12295128383)*x_ref[1]
        ref[2,1]=(0.347128677451)*x_ref[0]**o+(-0.0983169467916)*x_ref[0]+(-0.414573209624)*x_ref[1]**o+(-0.171300108487)*x_ref[1]
        ref[2,2]=(-0.279695154296)*x_ref[0]**o+(-0.911450144779)*x_ref[0]+(0.5055340448)*x_ref[1]**o+(-0.858058864152)*x_ref[1]
        ref[2,3]=(-0.53768906863)*x_ref[0]**o+(-0.0626923004303)*x_ref[0]+(0.72295545269)*x_ref[1]**o+(-0.77070425804)*x_ref[1]
        ref[2,4]=(0.18215032008)*x_ref[0]**o+(-0.965014882526)*x_ref[0]+(0.596564813119)*x_ref[1]**o+(0.0817024840554)*x_ref[1]
        ref[3,0]=(-0.572404596368)*x_ref[0]**o+(0.488365635829)*x_ref[0]+(0.145439800826)*x_ref[1]**o+(-0.337589667899)*x_ref[1]
        ref[3,1]=(-0.628977426654)*x_ref[0]**o+(0.22731945375)*x_ref[0]+(0.989224773975)*x_ref[1]**o+(0.0691553279499)*x_ref[1]
        ref[3,2]=(-0.43609079916)*x_ref[0]**o+(0.966113801416)*x_ref[0]+(-0.853136416278)*x_ref[1]**o+(-0.0125151911906)*x_ref[1]
        ref[3,3]=(0.340159347855)*x_ref[0]**o+(-0.254775013843)*x_ref[0]+(0.795261224144)*x_ref[1]**o+(-0.692255020659)*x_ref[1]
        ref[3,4]=(-0.153521724504)*x_ref[0]**o+(-0.681779299881)*x_ref[0]+(0.693860779041)*x_ref[1]**o+(0.882012604794)*x_ref[1]
      else:
        arg[0,0]=(-0.0201751669244)*x[0]**o+(0.337967826628)*x[0]+(0.262482967798)*x[1]**o+(-0.192863458488)*x[1]+(-0.696822606165)*x[2]**o+(-0.476556518002)*x[2]
        arg[0,1]=(-0.546277895941)*x[0]**o+(-0.118793125394)*x[0]+(0.987786322041)*x[1]**o+(-0.241472898602)*x[1]+(-0.454308453322)*x[2]**o+(-0.27057260464)*x[2]
        arg[0,2]=(0.383631846628)*x[0]**o+(0.979909217026)*x[0]+(-0.0503694684178)*x[1]**o+(-0.748153155067)*x[1]+(-0.721832849635)*x[2]**o+(0.144937710994)*x[2]
        arg[0,3]=(-0.540144273345)*x[0]**o+(-0.986853732479)*x[0]+(-0.14527387031)*x[1]**o+(0.329815126012)*x[1]+(0.566431907689)*x[2]**o+(-0.323179211167)*x[2]
        arg[0,4]=(-0.448624394254)*x[0]**o+(-0.272742080619)*x[0]+(-0.451405020267)*x[1]**o+(0.173204890481)*x[1]+(-0.0122463571841)*x[2]**o+(-0.595932249794)*x[2]
        arg[1,0]=(0.984160677776)*x[0]**o+(-0.243512460216)*x[0]+(0.82590496761)*x[1]**o+(-0.448270326787)*x[1]+(0.286008153265)*x[2]**o+(0.804229889769)*x[2]
        arg[1,1]=(0.531762548093)*x[0]**o+(0.211455783615)*x[0]+(0.836058347792)*x[1]**o+(0.806904570574)*x[1]+(-0.052512287992)*x[2]**o+(0.998679219355)*x[2]
        arg[1,2]=(0.462615968956)*x[0]**o+(0.960862689218)*x[0]+(-0.693932866844)*x[1]**o+(0.881110014782)*x[1]+(0.976329091354)*x[2]**o+(-0.647483316997)*x[2]
        arg[1,3]=(-0.764181748932)*x[0]**o+(-0.962197871121)*x[0]+(-0.143747029438)*x[1]**o+(0.913962833322)*x[1]+(0.165013597835)*x[2]**o+(0.166877976184)*x[2]
        arg[1,4]=(0.687688905493)*x[0]**o+(-0.152816071208)*x[0]+(-0.277974702822)*x[1]**o+(-0.389918839086)*x[1]+(0.229658530129)*x[2]**o+(-0.69924627348)*x[2]
        arg[2,0]=(0.0598215047949)*x[0]**o+(0.592389000293)*x[0]+(0.00886573296364)*x[1]**o+(0.339086619934)*x[1]+(0.476600446349)*x[2]**o+(0.767524427684)*x[2]
        arg[2,1]=(0.525936212445)*x[0]**o+(0.44607860423)*x[0]+(0.0397184541609)*x[1]**o+(-0.424118244789)*x[1]+(-0.273077104391)*x[2]**o+(0.435027185784)*x[2]
        arg[2,2]=(0.672992314995)*x[0]**o+(0.993339100191)*x[0]+(0.488983819898)*x[1]**o+(0.568579529177)*x[1]+(-0.321282052211)*x[2]**o+(-0.268847753649)*x[2]
        arg[2,3]=(-0.0536748741467)*x[0]**o+(-0.195658514433)*x[0]+(-0.931652264383)*x[1]**o+(-0.89957282695)*x[1]+(-0.811088135231)*x[2]**o+(0.0699177832759)*x[2]
        arg[2,4]=(-0.707875141752)*x[0]**o+(-0.00297935832343)*x[0]+(-0.959580991547)*x[1]**o+(0.0787110832272)*x[1]+(-0.754646316678)*x[2]**o+(-0.432731727467)*x[2]
        arg[3,0]=(0.728853463111)*x[0]**o+(-0.681375546727)*x[0]+(-0.283722278591)*x[1]**o+(0.950522890527)*x[1]+(0.608194817491)*x[2]**o+(0.571156129477)*x[2]
        arg[3,1]=(-0.0208969501228)*x[0]**o+(0.572596028151)*x[0]+(0.811179009214)*x[1]**o+(0.519098352581)*x[1]+(0.154527510954)*x[2]**o+(-0.21413742337)*x[2]
        arg[3,2]=(0.347209642455)*x[0]**o+(-0.114911324326)*x[0]+(-0.104561779357)*x[1]**o+(0.848703366841)*x[1]+(0.680312939858)*x[2]**o+(-0.680566404979)*x[2]
        arg[3,3]=(0.342687985669)*x[0]**o+(0.443470953221)*x[0]+(-0.849668173915)*x[1]**o+(0.337689225078)*x[1]+(-0.35083505383)*x[2]**o+(-0.284627678739)*x[2]
        arg[3,4]=(0.661690111131)*x[0]**o+(-0.0918926979359)*x[0]+(-0.587753600932)*x[1]**o+(-0.383115718043)*x[1]+(-0.0696588695623)*x[2]**o+(-0.799630628972)*x[2]
        ref[0,0]=(-0.0201751669244)*x_ref[0]**o+(0.337967826628)*x_ref[0]+(0.262482967798)*x_ref[1]**o+(-0.192863458488)*x_ref[1]+(-0.696822606165)*x_ref[2]**o+(-0.476556518002)*x_ref[2]
        ref[0,1]=(-0.546277895941)*x_ref[0]**o+(-0.118793125394)*x_ref[0]+(0.987786322041)*x_ref[1]**o+(-0.241472898602)*x_ref[1]+(-0.454308453322)*x_ref[2]**o+(-0.27057260464)*x_ref[2]
        ref[0,2]=(0.383631846628)*x_ref[0]**o+(0.979909217026)*x_ref[0]+(-0.0503694684178)*x_ref[1]**o+(-0.748153155067)*x_ref[1]+(-0.721832849635)*x_ref[2]**o+(0.144937710994)*x_ref[2]
        ref[0,3]=(-0.540144273345)*x_ref[0]**o+(-0.986853732479)*x_ref[0]+(-0.14527387031)*x_ref[1]**o+(0.329815126012)*x_ref[1]+(0.566431907689)*x_ref[2]**o+(-0.323179211167)*x_ref[2]
        ref[0,4]=(-0.448624394254)*x_ref[0]**o+(-0.272742080619)*x_ref[0]+(-0.451405020267)*x_ref[1]**o+(0.173204890481)*x_ref[1]+(-0.0122463571841)*x_ref[2]**o+(-0.595932249794)*x_ref[2]
        ref[1,0]=(0.984160677776)*x_ref[0]**o+(-0.243512460216)*x_ref[0]+(0.82590496761)*x_ref[1]**o+(-0.448270326787)*x_ref[1]+(0.286008153265)*x_ref[2]**o+(0.804229889769)*x_ref[2]
        ref[1,1]=(0.531762548093)*x_ref[0]**o+(0.211455783615)*x_ref[0]+(0.836058347792)*x_ref[1]**o+(0.806904570574)*x_ref[1]+(-0.052512287992)*x_ref[2]**o+(0.998679219355)*x_ref[2]
        ref[1,2]=(0.462615968956)*x_ref[0]**o+(0.960862689218)*x_ref[0]+(-0.693932866844)*x_ref[1]**o+(0.881110014782)*x_ref[1]+(0.976329091354)*x_ref[2]**o+(-0.647483316997)*x_ref[2]
        ref[1,3]=(-0.764181748932)*x_ref[0]**o+(-0.962197871121)*x_ref[0]+(-0.143747029438)*x_ref[1]**o+(0.913962833322)*x_ref[1]+(0.165013597835)*x_ref[2]**o+(0.166877976184)*x_ref[2]
        ref[1,4]=(0.687688905493)*x_ref[0]**o+(-0.152816071208)*x_ref[0]+(-0.277974702822)*x_ref[1]**o+(-0.389918839086)*x_ref[1]+(0.229658530129)*x_ref[2]**o+(-0.69924627348)*x_ref[2]
        ref[2,0]=(0.0598215047949)*x_ref[0]**o+(0.592389000293)*x_ref[0]+(0.00886573296364)*x_ref[1]**o+(0.339086619934)*x_ref[1]+(0.476600446349)*x_ref[2]**o+(0.767524427684)*x_ref[2]
        ref[2,1]=(0.525936212445)*x_ref[0]**o+(0.44607860423)*x_ref[0]+(0.0397184541609)*x_ref[1]**o+(-0.424118244789)*x_ref[1]+(-0.273077104391)*x_ref[2]**o+(0.435027185784)*x_ref[2]
        ref[2,2]=(0.672992314995)*x_ref[0]**o+(0.993339100191)*x_ref[0]+(0.488983819898)*x_ref[1]**o+(0.568579529177)*x_ref[1]+(-0.321282052211)*x_ref[2]**o+(-0.268847753649)*x_ref[2]
        ref[2,3]=(-0.0536748741467)*x_ref[0]**o+(-0.195658514433)*x_ref[0]+(-0.931652264383)*x_ref[1]**o+(-0.89957282695)*x_ref[1]+(-0.811088135231)*x_ref[2]**o+(0.0699177832759)*x_ref[2]
        ref[2,4]=(-0.707875141752)*x_ref[0]**o+(-0.00297935832343)*x_ref[0]+(-0.959580991547)*x_ref[1]**o+(0.0787110832272)*x_ref[1]+(-0.754646316678)*x_ref[2]**o+(-0.432731727467)*x_ref[2]
        ref[3,0]=(0.728853463111)*x_ref[0]**o+(-0.681375546727)*x_ref[0]+(-0.283722278591)*x_ref[1]**o+(0.950522890527)*x_ref[1]+(0.608194817491)*x_ref[2]**o+(0.571156129477)*x_ref[2]
        ref[3,1]=(-0.0208969501228)*x_ref[0]**o+(0.572596028151)*x_ref[0]+(0.811179009214)*x_ref[1]**o+(0.519098352581)*x_ref[1]+(0.154527510954)*x_ref[2]**o+(-0.21413742337)*x_ref[2]
        ref[3,2]=(0.347209642455)*x_ref[0]**o+(-0.114911324326)*x_ref[0]+(-0.104561779357)*x_ref[1]**o+(0.848703366841)*x_ref[1]+(0.680312939858)*x_ref[2]**o+(-0.680566404979)*x_ref[2]
        ref[3,3]=(0.342687985669)*x_ref[0]**o+(0.443470953221)*x_ref[0]+(-0.849668173915)*x_ref[1]**o+(0.337689225078)*x_ref[1]+(-0.35083505383)*x_ref[2]**o+(-0.284627678739)*x_ref[2]
        ref[3,4]=(0.661690111131)*x_ref[0]**o+(-0.0918926979359)*x_ref[0]+(-0.587753600932)*x_ref[1]**o+(-0.383115718043)*x_ref[1]+(-0.0696588695623)*x_ref[2]**o+(-0.799630628972)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromData_ReducedFunctionOnContactZero_rank3(self):
      """
      tests interpolate for rank 3 Data onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactZero onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactZero(self.domain)
      x=w.getX()
      arg=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2),w_ref)
      if dim==2:
        arg[0,0,0]=(-0.906705618142)*x[0]**o+(0.458982439103)*x[0]+(0.556464353903)*x[1]**o+(0.733236686234)*x[1]
        arg[0,0,1]=(-0.192889954523)*x[0]**o+(-0.814332539133)*x[0]+(0.446164214282)*x[1]**o+(0.032291315446)*x[1]
        arg[0,1,0]=(0.757081929199)*x[0]**o+(-0.3938524931)*x[0]+(-0.458060132115)*x[1]**o+(0.471831491182)*x[1]
        arg[0,1,1]=(0.459868785407)*x[0]**o+(0.513707183316)*x[0]+(-0.9682413377)*x[1]**o+(-0.49496265187)*x[1]
        arg[1,0,0]=(0.546655212697)*x[0]**o+(-0.296125788768)*x[0]+(0.570033305927)*x[1]**o+(-0.818919094317)*x[1]
        arg[1,0,1]=(-0.954422252892)*x[0]**o+(-0.925035286684)*x[0]+(0.171311127546)*x[1]**o+(-0.584606597796)*x[1]
        arg[1,1,0]=(-0.203956024883)*x[0]**o+(-0.933117428128)*x[0]+(-0.745101180472)*x[1]**o+(0.0390208509919)*x[1]
        arg[1,1,1]=(-0.67467593628)*x[0]**o+(0.42281003355)*x[0]+(-0.495401176467)*x[1]**o+(0.993257306842)*x[1]
        arg[2,0,0]=(-0.204011890956)*x[0]**o+(0.730452092671)*x[0]+(-0.0486718200506)*x[1]**o+(0.691928726795)*x[1]
        arg[2,0,1]=(0.381571380631)*x[0]**o+(0.504896621541)*x[0]+(-0.229805983176)*x[1]**o+(0.281401365705)*x[1]
        arg[2,1,0]=(0.578852751326)*x[0]**o+(-0.38588684985)*x[0]+(-0.630908284751)*x[1]**o+(0.237972319424)*x[1]
        arg[2,1,1]=(-0.928366568501)*x[0]**o+(0.254484026833)*x[0]+(0.903931940538)*x[1]**o+(-0.370818194785)*x[1]
        arg[3,0,0]=(-0.216012465475)*x[0]**o+(-0.938274177001)*x[0]+(0.0674408710328)*x[1]**o+(-0.731594784098)*x[1]
        arg[3,0,1]=(-0.156790905207)*x[0]**o+(0.284403081253)*x[0]+(0.3754228052)*x[1]**o+(-0.772758808958)*x[1]
        arg[3,1,0]=(0.89987768188)*x[0]**o+(0.0177886564195)*x[0]+(0.102329016274)*x[1]**o+(0.474039949682)*x[1]
        arg[3,1,1]=(0.0416243959618)*x[0]**o+(0.739593496294)*x[0]+(-0.0215969068381)*x[1]**o+(0.203954403681)*x[1]
        arg[4,0,0]=(-0.479380091881)*x[0]**o+(0.973630227471)*x[0]+(0.372617682727)*x[1]**o+(0.721993565094)*x[1]
        arg[4,0,1]=(0.360211306746)*x[0]**o+(0.260993313325)*x[0]+(0.252635752916)*x[1]**o+(0.534875843061)*x[1]
        arg[4,1,0]=(-0.609695114069)*x[0]**o+(0.283865161572)*x[0]+(-0.607441168016)*x[1]**o+(-0.838279309054)*x[1]
        arg[4,1,1]=(0.984921561958)*x[0]**o+(-0.352255390774)*x[0]+(-0.770706841576)*x[1]**o+(0.456415771191)*x[1]
        arg[5,0,0]=(0.903410389973)*x[0]**o+(0.609651342058)*x[0]+(-0.0890705788805)*x[1]**o+(0.613493629434)*x[1]
        arg[5,0,1]=(-0.977979577058)*x[0]**o+(-0.121665631763)*x[0]+(0.466897174395)*x[1]**o+(-0.438819940752)*x[1]
        arg[5,1,0]=(-0.537313389905)*x[0]**o+(-0.0401503571218)*x[0]+(-0.454563705252)*x[1]**o+(-0.728276295145)*x[1]
        arg[5,1,1]=(0.858855332454)*x[0]**o+(0.378938106473)*x[0]+(0.908957690039)*x[1]**o+(0.989040980713)*x[1]
        ref[0,0,0]=(-0.906705618142)*x_ref[0]**o+(0.458982439103)*x_ref[0]+(0.556464353903)*x_ref[1]**o+(0.733236686234)*x_ref[1]
        ref[0,0,1]=(-0.192889954523)*x_ref[0]**o+(-0.814332539133)*x_ref[0]+(0.446164214282)*x_ref[1]**o+(0.032291315446)*x_ref[1]
        ref[0,1,0]=(0.757081929199)*x_ref[0]**o+(-0.3938524931)*x_ref[0]+(-0.458060132115)*x_ref[1]**o+(0.471831491182)*x_ref[1]
        ref[0,1,1]=(0.459868785407)*x_ref[0]**o+(0.513707183316)*x_ref[0]+(-0.9682413377)*x_ref[1]**o+(-0.49496265187)*x_ref[1]
        ref[1,0,0]=(0.546655212697)*x_ref[0]**o+(-0.296125788768)*x_ref[0]+(0.570033305927)*x_ref[1]**o+(-0.818919094317)*x_ref[1]
        ref[1,0,1]=(-0.954422252892)*x_ref[0]**o+(-0.925035286684)*x_ref[0]+(0.171311127546)*x_ref[1]**o+(-0.584606597796)*x_ref[1]
        ref[1,1,0]=(-0.203956024883)*x_ref[0]**o+(-0.933117428128)*x_ref[0]+(-0.745101180472)*x_ref[1]**o+(0.0390208509919)*x_ref[1]
        ref[1,1,1]=(-0.67467593628)*x_ref[0]**o+(0.42281003355)*x_ref[0]+(-0.495401176467)*x_ref[1]**o+(0.993257306842)*x_ref[1]
        ref[2,0,0]=(-0.204011890956)*x_ref[0]**o+(0.730452092671)*x_ref[0]+(-0.0486718200506)*x_ref[1]**o+(0.691928726795)*x_ref[1]
        ref[2,0,1]=(0.381571380631)*x_ref[0]**o+(0.504896621541)*x_ref[0]+(-0.229805983176)*x_ref[1]**o+(0.281401365705)*x_ref[1]
        ref[2,1,0]=(0.578852751326)*x_ref[0]**o+(-0.38588684985)*x_ref[0]+(-0.630908284751)*x_ref[1]**o+(0.237972319424)*x_ref[1]
        ref[2,1,1]=(-0.928366568501)*x_ref[0]**o+(0.254484026833)*x_ref[0]+(0.903931940538)*x_ref[1]**o+(-0.370818194785)*x_ref[1]
        ref[3,0,0]=(-0.216012465475)*x_ref[0]**o+(-0.938274177001)*x_ref[0]+(0.0674408710328)*x_ref[1]**o+(-0.731594784098)*x_ref[1]
        ref[3,0,1]=(-0.156790905207)*x_ref[0]**o+(0.284403081253)*x_ref[0]+(0.3754228052)*x_ref[1]**o+(-0.772758808958)*x_ref[1]
        ref[3,1,0]=(0.89987768188)*x_ref[0]**o+(0.0177886564195)*x_ref[0]+(0.102329016274)*x_ref[1]**o+(0.474039949682)*x_ref[1]
        ref[3,1,1]=(0.0416243959618)*x_ref[0]**o+(0.739593496294)*x_ref[0]+(-0.0215969068381)*x_ref[1]**o+(0.203954403681)*x_ref[1]
        ref[4,0,0]=(-0.479380091881)*x_ref[0]**o+(0.973630227471)*x_ref[0]+(0.372617682727)*x_ref[1]**o+(0.721993565094)*x_ref[1]
        ref[4,0,1]=(0.360211306746)*x_ref[0]**o+(0.260993313325)*x_ref[0]+(0.252635752916)*x_ref[1]**o+(0.534875843061)*x_ref[1]
        ref[4,1,0]=(-0.609695114069)*x_ref[0]**o+(0.283865161572)*x_ref[0]+(-0.607441168016)*x_ref[1]**o+(-0.838279309054)*x_ref[1]
        ref[4,1,1]=(0.984921561958)*x_ref[0]**o+(-0.352255390774)*x_ref[0]+(-0.770706841576)*x_ref[1]**o+(0.456415771191)*x_ref[1]
        ref[5,0,0]=(0.903410389973)*x_ref[0]**o+(0.609651342058)*x_ref[0]+(-0.0890705788805)*x_ref[1]**o+(0.613493629434)*x_ref[1]
        ref[5,0,1]=(-0.977979577058)*x_ref[0]**o+(-0.121665631763)*x_ref[0]+(0.466897174395)*x_ref[1]**o+(-0.438819940752)*x_ref[1]
        ref[5,1,0]=(-0.537313389905)*x_ref[0]**o+(-0.0401503571218)*x_ref[0]+(-0.454563705252)*x_ref[1]**o+(-0.728276295145)*x_ref[1]
        ref[5,1,1]=(0.858855332454)*x_ref[0]**o+(0.378938106473)*x_ref[0]+(0.908957690039)*x_ref[1]**o+(0.989040980713)*x_ref[1]
      else:
        arg[0,0,0]=(-0.550852990025)*x[0]**o+(0.578705971531)*x[0]+(0.9533401901)*x[1]**o+(0.233181910221)*x[1]+(-0.143544272432)*x[2]**o+(-0.986513253736)*x[2]
        arg[0,0,1]=(-0.326417636931)*x[0]**o+(0.0910236969351)*x[0]+(-0.8240642668)*x[1]**o+(-0.242359389714)*x[1]+(0.266813299647)*x[2]**o+(-0.364956839435)*x[2]
        arg[0,1,0]=(-0.777107068999)*x[0]**o+(0.201120303703)*x[0]+(-0.684906707463)*x[1]**o+(-0.9620762843)*x[1]+(-0.314789913974)*x[2]**o+(-0.70682592319)*x[2]
        arg[0,1,1]=(-0.980292769796)*x[0]**o+(-0.827763529178)*x[0]+(0.256770279946)*x[1]**o+(0.619665292369)*x[1]+(0.014752087873)*x[2]**o+(-0.810044916486)*x[2]
        arg[1,0,0]=(-0.51164541516)*x[0]**o+(-0.429646605692)*x[0]+(-0.844516540906)*x[1]**o+(-0.0334629941654)*x[1]+(0.942164301071)*x[2]**o+(0.129008643344)*x[2]
        arg[1,0,1]=(-0.545329352463)*x[0]**o+(-0.0601985885543)*x[0]+(0.959752895562)*x[1]**o+(-0.617790988468)*x[1]+(-0.352323774737)*x[2]**o+(0.0854549755648)*x[2]
        arg[1,1,0]=(0.163270290732)*x[0]**o+(0.715418689409)*x[0]+(-0.892736094727)*x[1]**o+(-0.250085969902)*x[1]+(-0.323686607391)*x[2]**o+(-0.808822044566)*x[2]
        arg[1,1,1]=(-0.582858228848)*x[0]**o+(0.245356494646)*x[0]+(0.253867860186)*x[1]**o+(0.294533212339)*x[1]+(0.802766375942)*x[2]**o+(-0.142246487978)*x[2]
        arg[2,0,0]=(0.559682931749)*x[0]**o+(0.349781768104)*x[0]+(-0.903942828138)*x[1]**o+(0.285790428402)*x[1]+(0.334765408827)*x[2]**o+(-0.319553167552)*x[2]
        arg[2,0,1]=(-0.33958138616)*x[0]**o+(-0.476356251551)*x[0]+(0.222685919591)*x[1]**o+(0.947677379274)*x[1]+(-0.238198083505)*x[2]**o+(-0.158835521934)*x[2]
        arg[2,1,0]=(0.233005925554)*x[0]**o+(-0.0885827512025)*x[0]+(0.99298235997)*x[1]**o+(-0.972864413346)*x[1]+(-0.328337965331)*x[2]**o+(0.548449226427)*x[2]
        arg[2,1,1]=(-0.682434615287)*x[0]**o+(-0.681560497743)*x[0]+(-0.0427998840708)*x[1]**o+(-0.640532507401)*x[1]+(0.0600572641903)*x[2]**o+(0.580127097835)*x[2]
        arg[3,0,0]=(0.627556962303)*x[0]**o+(-0.2282646452)*x[0]+(0.201159933134)*x[1]**o+(-0.0874921144181)*x[1]+(0.487026890349)*x[2]**o+(-0.044256665504)*x[2]
        arg[3,0,1]=(-0.202969525043)*x[0]**o+(0.0445621391034)*x[0]+(-0.763479658456)*x[1]**o+(-0.600818445386)*x[1]+(-0.442800171339)*x[2]**o+(0.0422443400421)*x[2]
        arg[3,1,0]=(-0.422348404642)*x[0]**o+(0.723277196982)*x[0]+(0.882658191301)*x[1]**o+(-0.415218341669)*x[1]+(0.834911332561)*x[2]**o+(-0.843638850779)*x[2]
        arg[3,1,1]=(-0.685037113836)*x[0]**o+(0.119526696821)*x[0]+(0.732784440424)*x[1]**o+(0.156993935952)*x[1]+(-0.565837827825)*x[2]**o+(0.667635603478)*x[2]
        arg[4,0,0]=(-0.18210082527)*x[0]**o+(-0.00639006601355)*x[0]+(0.633771301081)*x[1]**o+(0.765450011643)*x[1]+(-0.323701472535)*x[2]**o+(0.841246809469)*x[2]
        arg[4,0,1]=(0.345847635107)*x[0]**o+(0.910917657816)*x[0]+(0.329667803527)*x[1]**o+(-0.0638797846725)*x[1]+(-0.705758696202)*x[2]**o+(0.39567870447)*x[2]
        arg[4,1,0]=(0.700805557734)*x[0]**o+(0.215376719066)*x[0]+(0.057212023986)*x[1]**o+(-0.601972647211)*x[1]+(-0.0163323501628)*x[2]**o+(-0.182314452762)*x[2]
        arg[4,1,1]=(-0.25633784052)*x[0]**o+(-0.36128209808)*x[0]+(-0.580660119759)*x[1]**o+(-0.718361413439)*x[1]+(0.294752937469)*x[2]**o+(0.479425008353)*x[2]
        arg[5,0,0]=(0.0250768411622)*x[0]**o+(-0.954437161073)*x[0]+(0.314623130846)*x[1]**o+(-0.915911802082)*x[1]+(-0.293481557155)*x[2]**o+(0.193040417512)*x[2]
        arg[5,0,1]=(-0.260552760825)*x[0]**o+(-0.993318746791)*x[0]+(-0.0888187740235)*x[1]**o+(0.228899511722)*x[1]+(0.667467747688)*x[2]**o+(0.210051895803)*x[2]
        arg[5,1,0]=(-0.925551425372)*x[0]**o+(-0.051063099903)*x[0]+(0.0607054862843)*x[1]**o+(0.778043999849)*x[1]+(-0.393429559505)*x[2]**o+(0.235480458425)*x[2]
        arg[5,1,1]=(0.00188621198193)*x[0]**o+(-0.529444982511)*x[0]+(0.766032106394)*x[1]**o+(-0.554844276584)*x[1]+(0.793956883291)*x[2]**o+(0.221253426455)*x[2]
        ref[0,0,0]=(-0.550852990025)*x_ref[0]**o+(0.578705971531)*x_ref[0]+(0.9533401901)*x_ref[1]**o+(0.233181910221)*x_ref[1]+(-0.143544272432)*x_ref[2]**o+(-0.986513253736)*x_ref[2]
        ref[0,0,1]=(-0.326417636931)*x_ref[0]**o+(0.0910236969351)*x_ref[0]+(-0.8240642668)*x_ref[1]**o+(-0.242359389714)*x_ref[1]+(0.266813299647)*x_ref[2]**o+(-0.364956839435)*x_ref[2]
        ref[0,1,0]=(-0.777107068999)*x_ref[0]**o+(0.201120303703)*x_ref[0]+(-0.684906707463)*x_ref[1]**o+(-0.9620762843)*x_ref[1]+(-0.314789913974)*x_ref[2]**o+(-0.70682592319)*x_ref[2]
        ref[0,1,1]=(-0.980292769796)*x_ref[0]**o+(-0.827763529178)*x_ref[0]+(0.256770279946)*x_ref[1]**o+(0.619665292369)*x_ref[1]+(0.014752087873)*x_ref[2]**o+(-0.810044916486)*x_ref[2]
        ref[1,0,0]=(-0.51164541516)*x_ref[0]**o+(-0.429646605692)*x_ref[0]+(-0.844516540906)*x_ref[1]**o+(-0.0334629941654)*x_ref[1]+(0.942164301071)*x_ref[2]**o+(0.129008643344)*x_ref[2]
        ref[1,0,1]=(-0.545329352463)*x_ref[0]**o+(-0.0601985885543)*x_ref[0]+(0.959752895562)*x_ref[1]**o+(-0.617790988468)*x_ref[1]+(-0.352323774737)*x_ref[2]**o+(0.0854549755648)*x_ref[2]
        ref[1,1,0]=(0.163270290732)*x_ref[0]**o+(0.715418689409)*x_ref[0]+(-0.892736094727)*x_ref[1]**o+(-0.250085969902)*x_ref[1]+(-0.323686607391)*x_ref[2]**o+(-0.808822044566)*x_ref[2]
        ref[1,1,1]=(-0.582858228848)*x_ref[0]**o+(0.245356494646)*x_ref[0]+(0.253867860186)*x_ref[1]**o+(0.294533212339)*x_ref[1]+(0.802766375942)*x_ref[2]**o+(-0.142246487978)*x_ref[2]
        ref[2,0,0]=(0.559682931749)*x_ref[0]**o+(0.349781768104)*x_ref[0]+(-0.903942828138)*x_ref[1]**o+(0.285790428402)*x_ref[1]+(0.334765408827)*x_ref[2]**o+(-0.319553167552)*x_ref[2]
        ref[2,0,1]=(-0.33958138616)*x_ref[0]**o+(-0.476356251551)*x_ref[0]+(0.222685919591)*x_ref[1]**o+(0.947677379274)*x_ref[1]+(-0.238198083505)*x_ref[2]**o+(-0.158835521934)*x_ref[2]
        ref[2,1,0]=(0.233005925554)*x_ref[0]**o+(-0.0885827512025)*x_ref[0]+(0.99298235997)*x_ref[1]**o+(-0.972864413346)*x_ref[1]+(-0.328337965331)*x_ref[2]**o+(0.548449226427)*x_ref[2]
        ref[2,1,1]=(-0.682434615287)*x_ref[0]**o+(-0.681560497743)*x_ref[0]+(-0.0427998840708)*x_ref[1]**o+(-0.640532507401)*x_ref[1]+(0.0600572641903)*x_ref[2]**o+(0.580127097835)*x_ref[2]
        ref[3,0,0]=(0.627556962303)*x_ref[0]**o+(-0.2282646452)*x_ref[0]+(0.201159933134)*x_ref[1]**o+(-0.0874921144181)*x_ref[1]+(0.487026890349)*x_ref[2]**o+(-0.044256665504)*x_ref[2]
        ref[3,0,1]=(-0.202969525043)*x_ref[0]**o+(0.0445621391034)*x_ref[0]+(-0.763479658456)*x_ref[1]**o+(-0.600818445386)*x_ref[1]+(-0.442800171339)*x_ref[2]**o+(0.0422443400421)*x_ref[2]
        ref[3,1,0]=(-0.422348404642)*x_ref[0]**o+(0.723277196982)*x_ref[0]+(0.882658191301)*x_ref[1]**o+(-0.415218341669)*x_ref[1]+(0.834911332561)*x_ref[2]**o+(-0.843638850779)*x_ref[2]
        ref[3,1,1]=(-0.685037113836)*x_ref[0]**o+(0.119526696821)*x_ref[0]+(0.732784440424)*x_ref[1]**o+(0.156993935952)*x_ref[1]+(-0.565837827825)*x_ref[2]**o+(0.667635603478)*x_ref[2]
        ref[4,0,0]=(-0.18210082527)*x_ref[0]**o+(-0.00639006601355)*x_ref[0]+(0.633771301081)*x_ref[1]**o+(0.765450011643)*x_ref[1]+(-0.323701472535)*x_ref[2]**o+(0.841246809469)*x_ref[2]
        ref[4,0,1]=(0.345847635107)*x_ref[0]**o+(0.910917657816)*x_ref[0]+(0.329667803527)*x_ref[1]**o+(-0.0638797846725)*x_ref[1]+(-0.705758696202)*x_ref[2]**o+(0.39567870447)*x_ref[2]
        ref[4,1,0]=(0.700805557734)*x_ref[0]**o+(0.215376719066)*x_ref[0]+(0.057212023986)*x_ref[1]**o+(-0.601972647211)*x_ref[1]+(-0.0163323501628)*x_ref[2]**o+(-0.182314452762)*x_ref[2]
        ref[4,1,1]=(-0.25633784052)*x_ref[0]**o+(-0.36128209808)*x_ref[0]+(-0.580660119759)*x_ref[1]**o+(-0.718361413439)*x_ref[1]+(0.294752937469)*x_ref[2]**o+(0.479425008353)*x_ref[2]
        ref[5,0,0]=(0.0250768411622)*x_ref[0]**o+(-0.954437161073)*x_ref[0]+(0.314623130846)*x_ref[1]**o+(-0.915911802082)*x_ref[1]+(-0.293481557155)*x_ref[2]**o+(0.193040417512)*x_ref[2]
        ref[5,0,1]=(-0.260552760825)*x_ref[0]**o+(-0.993318746791)*x_ref[0]+(-0.0888187740235)*x_ref[1]**o+(0.228899511722)*x_ref[1]+(0.667467747688)*x_ref[2]**o+(0.210051895803)*x_ref[2]
        ref[5,1,0]=(-0.925551425372)*x_ref[0]**o+(-0.051063099903)*x_ref[0]+(0.0607054862843)*x_ref[1]**o+(0.778043999849)*x_ref[1]+(-0.393429559505)*x_ref[2]**o+(0.235480458425)*x_ref[2]
        ref[5,1,1]=(0.00188621198193)*x_ref[0]**o+(-0.529444982511)*x_ref[0]+(0.766032106394)*x_ref[1]**o+(-0.554844276584)*x_ref[1]+(0.793956883291)*x_ref[2]**o+(0.221253426455)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromData_ReducedFunctionOnContactZero_rank4(self):
      """
      tests interpolate for rank 4 Data onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactZero onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactZero(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5, 3, 2),w)
      ref=Data(0,(4, 5, 3, 2),w_ref)
      if dim==2:
        arg[0,0,0,0]=(0.358768976574)*x[0]**o+(-0.231637053165)*x[0]+(0.798891506345)*x[1]**o+(0.292325388353)*x[1]
        arg[0,0,0,1]=(0.53301689922)*x[0]**o+(-0.761786870664)*x[0]+(0.284067900826)*x[1]**o+(-0.688662082205)*x[1]
        arg[0,0,1,0]=(-0.721988036113)*x[0]**o+(-0.743892757815)*x[0]+(-0.796787022375)*x[1]**o+(-0.499468412523)*x[1]
        arg[0,0,1,1]=(0.0710691488306)*x[0]**o+(0.0210385141456)*x[0]+(-0.709342417683)*x[1]**o+(0.901794071115)*x[1]
        arg[0,0,2,0]=(-0.800537263339)*x[0]**o+(0.583114786539)*x[0]+(0.422752622522)*x[1]**o+(-0.0954981460748)*x[1]
        arg[0,0,2,1]=(-0.0805357065445)*x[0]**o+(0.476377895547)*x[0]+(0.242203912566)*x[1]**o+(-0.479971540043)*x[1]
        arg[0,1,0,0]=(-0.476356823162)*x[0]**o+(-0.438853622252)*x[0]+(-0.937228141746)*x[1]**o+(0.366496742588)*x[1]
        arg[0,1,0,1]=(-0.496918253956)*x[0]**o+(-0.0227549061268)*x[0]+(0.830195584768)*x[1]**o+(-0.114624902423)*x[1]
        arg[0,1,1,0]=(0.431291036997)*x[0]**o+(0.807713720258)*x[0]+(0.730057543333)*x[1]**o+(0.635171725232)*x[1]
        arg[0,1,1,1]=(0.784425995411)*x[0]**o+(-0.397510494899)*x[0]+(-0.579911740115)*x[1]**o+(0.130426014607)*x[1]
        arg[0,1,2,0]=(0.473613650641)*x[0]**o+(-0.155617741177)*x[0]+(0.873237852103)*x[1]**o+(-0.749395152414)*x[1]
        arg[0,1,2,1]=(0.367649559431)*x[0]**o+(0.830181309644)*x[0]+(0.87726968147)*x[1]**o+(-0.450304793682)*x[1]
        arg[0,2,0,0]=(-0.700522876654)*x[0]**o+(-0.998187126003)*x[0]+(0.732491610173)*x[1]**o+(0.648679267)*x[1]
        arg[0,2,0,1]=(0.499746141285)*x[0]**o+(-0.730090975806)*x[0]+(-0.525566438654)*x[1]**o+(0.212469850421)*x[1]
        arg[0,2,1,0]=(-0.847752916298)*x[0]**o+(0.43510472619)*x[0]+(0.815305936548)*x[1]**o+(-0.143802676552)*x[1]
        arg[0,2,1,1]=(-0.453572376958)*x[0]**o+(-0.913023531929)*x[0]+(0.743328127665)*x[1]**o+(0.152586122591)*x[1]
        arg[0,2,2,0]=(0.526939334508)*x[0]**o+(0.247268722057)*x[0]+(-0.176432621481)*x[1]**o+(-0.510110202196)*x[1]
        arg[0,2,2,1]=(0.139685732003)*x[0]**o+(0.278304455867)*x[0]+(0.517953769616)*x[1]**o+(-0.272384261701)*x[1]
        arg[0,3,0,0]=(-0.38867907688)*x[0]**o+(-0.73043975055)*x[0]+(-0.14293243901)*x[1]**o+(0.0601042854043)*x[1]
        arg[0,3,0,1]=(-0.163055949188)*x[0]**o+(-0.035877062482)*x[0]+(0.0396615423106)*x[1]**o+(-0.870062303275)*x[1]
        arg[0,3,1,0]=(0.283839779914)*x[0]**o+(-0.519316925197)*x[0]+(-0.437401973959)*x[1]**o+(-0.823707232663)*x[1]
        arg[0,3,1,1]=(0.700305425892)*x[0]**o+(0.0299761521094)*x[0]+(-0.993032151025)*x[1]**o+(0.633293413221)*x[1]
        arg[0,3,2,0]=(-0.745357256365)*x[0]**o+(-0.451553456961)*x[0]+(0.05789646302)*x[1]**o+(-0.755901155251)*x[1]
        arg[0,3,2,1]=(-0.57330840403)*x[0]**o+(-0.589361483751)*x[0]+(-0.038677123513)*x[1]**o+(0.417332032168)*x[1]
        arg[0,4,0,0]=(0.759655161531)*x[0]**o+(0.737483461328)*x[0]+(0.573158629937)*x[1]**o+(-0.454191109106)*x[1]
        arg[0,4,0,1]=(-0.232982406645)*x[0]**o+(-0.926030038179)*x[0]+(-0.504256834465)*x[1]**o+(-0.107703527058)*x[1]
        arg[0,4,1,0]=(0.898411697016)*x[0]**o+(0.642045536742)*x[0]+(-0.953059791895)*x[1]**o+(-0.67501710093)*x[1]
        arg[0,4,1,1]=(0.300380574165)*x[0]**o+(0.204591960115)*x[0]+(-0.308849697562)*x[1]**o+(-0.489495887357)*x[1]
        arg[0,4,2,0]=(-0.814560286716)*x[0]**o+(0.154235599953)*x[0]+(0.279329273498)*x[1]**o+(-0.674063978926)*x[1]
        arg[0,4,2,1]=(-0.718018870365)*x[0]**o+(-0.735982858424)*x[0]+(-0.0132197132926)*x[1]**o+(0.398939587543)*x[1]
        arg[1,0,0,0]=(0.0559995351725)*x[0]**o+(-0.164549619892)*x[0]+(-0.794242529408)*x[1]**o+(0.0927822314785)*x[1]
        arg[1,0,0,1]=(-0.894238542341)*x[0]**o+(0.378559247473)*x[0]+(-0.183957272639)*x[1]**o+(0.486358968942)*x[1]
        arg[1,0,1,0]=(0.931288801001)*x[0]**o+(0.223815769968)*x[0]+(-0.719591906328)*x[1]**o+(-0.743644741105)*x[1]
        arg[1,0,1,1]=(0.156646029123)*x[0]**o+(0.306942879419)*x[0]+(0.747006830674)*x[1]**o+(-0.876534912274)*x[1]
        arg[1,0,2,0]=(-0.496819081999)*x[0]**o+(-0.939227402425)*x[0]+(0.0566331712182)*x[1]**o+(0.565724364661)*x[1]
        arg[1,0,2,1]=(0.903706429954)*x[0]**o+(0.667947320307)*x[0]+(0.97140855373)*x[1]**o+(0.924095235752)*x[1]
        arg[1,1,0,0]=(0.526180197299)*x[0]**o+(0.435493351674)*x[0]+(0.400107077848)*x[1]**o+(-0.704760833758)*x[1]
        arg[1,1,0,1]=(0.0723740015004)*x[0]**o+(-0.405859051371)*x[0]+(0.200787757511)*x[1]**o+(0.41819184563)*x[1]
        arg[1,1,1,0]=(-0.0320951655627)*x[0]**o+(0.235068445821)*x[0]+(0.450383349599)*x[1]**o+(-0.801487424765)*x[1]
        arg[1,1,1,1]=(-0.52478892744)*x[0]**o+(-0.449111297125)*x[0]+(0.731062182014)*x[1]**o+(0.533619608577)*x[1]
        arg[1,1,2,0]=(0.0889794057057)*x[0]**o+(-0.971856981571)*x[0]+(0.662216408922)*x[1]**o+(0.164089024468)*x[1]
        arg[1,1,2,1]=(-0.196374963647)*x[0]**o+(-0.739360211802)*x[0]+(0.916611650753)*x[1]**o+(-0.260102318115)*x[1]
        arg[1,2,0,0]=(0.627763572103)*x[0]**o+(-0.893900847319)*x[0]+(0.907548446417)*x[1]**o+(0.0687906203663)*x[1]
        arg[1,2,0,1]=(0.59755498682)*x[0]**o+(-0.457108009787)*x[0]+(0.71562139654)*x[1]**o+(-0.300872832559)*x[1]
        arg[1,2,1,0]=(-0.479641829834)*x[0]**o+(0.297188513722)*x[0]+(0.0227730978493)*x[1]**o+(0.199329665406)*x[1]
        arg[1,2,1,1]=(0.918974940101)*x[0]**o+(0.595194981816)*x[0]+(0.760030152554)*x[1]**o+(-0.798899878486)*x[1]
        arg[1,2,2,0]=(-0.878334511842)*x[0]**o+(0.910857236287)*x[0]+(0.0164747567997)*x[1]**o+(-0.876970888445)*x[1]
        arg[1,2,2,1]=(0.29445809181)*x[0]**o+(0.837849454931)*x[0]+(0.0759054371211)*x[1]**o+(0.501243080372)*x[1]
        arg[1,3,0,0]=(-0.529006436258)*x[0]**o+(-0.932325855538)*x[0]+(-0.561472972374)*x[1]**o+(0.845125736656)*x[1]
        arg[1,3,0,1]=(-0.843008257488)*x[0]**o+(0.614118623012)*x[0]+(-0.167015137447)*x[1]**o+(0.158701952625)*x[1]
        arg[1,3,1,0]=(-0.662966916148)*x[0]**o+(-0.443297576512)*x[0]+(0.295988608077)*x[1]**o+(0.059506908615)*x[1]
        arg[1,3,1,1]=(0.864199978014)*x[0]**o+(0.138723519602)*x[0]+(0.144964839302)*x[1]**o+(0.239863709389)*x[1]
        arg[1,3,2,0]=(0.479006097431)*x[0]**o+(0.542168797821)*x[0]+(0.561074562163)*x[1]**o+(-0.133365787275)*x[1]
        arg[1,3,2,1]=(-0.901033449452)*x[0]**o+(-0.620245150074)*x[0]+(0.598461569697)*x[1]**o+(-0.2213246854)*x[1]
        arg[1,4,0,0]=(-0.577067401348)*x[0]**o+(0.486154324874)*x[0]+(0.809205150442)*x[1]**o+(-0.218206418459)*x[1]
        arg[1,4,0,1]=(0.726556862066)*x[0]**o+(0.0293695509867)*x[0]+(0.601938348193)*x[1]**o+(0.967332697531)*x[1]
        arg[1,4,1,0]=(-0.675624388054)*x[0]**o+(-0.871613723214)*x[0]+(0.605519387693)*x[1]**o+(0.287186894411)*x[1]
        arg[1,4,1,1]=(-0.689904898214)*x[0]**o+(-0.918236705665)*x[0]+(-0.892168622177)*x[1]**o+(0.507441662612)*x[1]
        arg[1,4,2,0]=(0.845768980712)*x[0]**o+(-0.458467508114)*x[0]+(-0.518185249391)*x[1]**o+(0.829945330327)*x[1]
        arg[1,4,2,1]=(0.776852312178)*x[0]**o+(-0.268810999469)*x[0]+(-0.576757148841)*x[1]**o+(-0.50628294802)*x[1]
        arg[2,0,0,0]=(-0.686840219939)*x[0]**o+(0.6861172847)*x[0]+(0.240362263052)*x[1]**o+(0.962361523884)*x[1]
        arg[2,0,0,1]=(-0.943837188119)*x[0]**o+(0.829832768268)*x[0]+(0.377590124133)*x[1]**o+(-0.171714144962)*x[1]
        arg[2,0,1,0]=(0.862809148447)*x[0]**o+(0.0573402136489)*x[0]+(-0.743311826013)*x[1]**o+(0.427809313437)*x[1]
        arg[2,0,1,1]=(0.377380819576)*x[0]**o+(-0.223280285041)*x[0]+(0.178628658562)*x[1]**o+(0.121964611877)*x[1]
        arg[2,0,2,0]=(0.41723758689)*x[0]**o+(0.256039203883)*x[0]+(0.255358776676)*x[1]**o+(-0.881665964974)*x[1]
        arg[2,0,2,1]=(-0.0797103024499)*x[0]**o+(0.128068931414)*x[0]+(-0.758317834409)*x[1]**o+(0.654657809183)*x[1]
        arg[2,1,0,0]=(0.603648218133)*x[0]**o+(-0.286780270729)*x[0]+(-0.341108067214)*x[1]**o+(-0.769667356087)*x[1]
        arg[2,1,0,1]=(0.844044270035)*x[0]**o+(0.0585907857764)*x[0]+(-0.578232785252)*x[1]**o+(-0.40959017859)*x[1]
        arg[2,1,1,0]=(0.599709567393)*x[0]**o+(0.307092449739)*x[0]+(-0.402950586215)*x[1]**o+(0.0674239470451)*x[1]
        arg[2,1,1,1]=(0.182608249961)*x[0]**o+(0.373533584601)*x[0]+(0.40949869082)*x[1]**o+(0.696333617699)*x[1]
        arg[2,1,2,0]=(0.0462891111054)*x[0]**o+(0.371258293292)*x[0]+(0.782976187716)*x[1]**o+(0.117976017999)*x[1]
        arg[2,1,2,1]=(0.891841121526)*x[0]**o+(-0.998971173204)*x[0]+(0.0286712507928)*x[1]**o+(-0.0198507022516)*x[1]
        arg[2,2,0,0]=(0.284620902703)*x[0]**o+(0.936231203073)*x[0]+(0.829209752046)*x[1]**o+(-0.94940982125)*x[1]
        arg[2,2,0,1]=(-0.647956027376)*x[0]**o+(0.923690718922)*x[0]+(-0.0322423837346)*x[1]**o+(0.124962508606)*x[1]
        arg[2,2,1,0]=(-0.0696010411705)*x[0]**o+(-0.161672936739)*x[0]+(-0.997445762186)*x[1]**o+(0.177962302014)*x[1]
        arg[2,2,1,1]=(-0.897657700296)*x[0]**o+(-0.467866721175)*x[0]+(-0.560034332007)*x[1]**o+(-0.861509898295)*x[1]
        arg[2,2,2,0]=(-0.100880174048)*x[0]**o+(0.678568254695)*x[0]+(-0.83289411684)*x[1]**o+(0.680564393149)*x[1]
        arg[2,2,2,1]=(0.546159181491)*x[0]**o+(0.720296622966)*x[0]+(0.939046941118)*x[1]**o+(-0.716147554733)*x[1]
        arg[2,3,0,0]=(-0.921073162235)*x[0]**o+(0.979925882289)*x[0]+(-0.0938642966949)*x[1]**o+(0.482093047223)*x[1]
        arg[2,3,0,1]=(0.0709940956822)*x[0]**o+(-0.691401017243)*x[0]+(0.447000814029)*x[1]**o+(0.0575856240533)*x[1]
        arg[2,3,1,0]=(0.562800393365)*x[0]**o+(0.657646549915)*x[0]+(-0.835326361689)*x[1]**o+(0.3487960687)*x[1]
        arg[2,3,1,1]=(0.106668867409)*x[0]**o+(0.436327240765)*x[0]+(-0.585051261259)*x[1]**o+(0.616655338667)*x[1]
        arg[2,3,2,0]=(-0.501849628176)*x[0]**o+(-0.284722503916)*x[0]+(-0.892470199837)*x[1]**o+(-0.979942541795)*x[1]
        arg[2,3,2,1]=(0.236759540073)*x[0]**o+(-0.806931113769)*x[0]+(-0.225220003475)*x[1]**o+(-0.95183239844)*x[1]
        arg[2,4,0,0]=(-0.684823909581)*x[0]**o+(0.445615228059)*x[0]+(-0.638879895767)*x[1]**o+(-0.335505874088)*x[1]
        arg[2,4,0,1]=(-0.196708460612)*x[0]**o+(0.981592996809)*x[0]+(0.435774317028)*x[1]**o+(-0.0952200663109)*x[1]
        arg[2,4,1,0]=(-0.552759474764)*x[0]**o+(-0.614031173545)*x[0]+(0.304375542871)*x[1]**o+(-0.285131901533)*x[1]
        arg[2,4,1,1]=(0.147812951421)*x[0]**o+(-0.78533487681)*x[0]+(0.942205524401)*x[1]**o+(0.166280215075)*x[1]
        arg[2,4,2,0]=(0.843067982563)*x[0]**o+(0.0116520710289)*x[0]+(0.927948889059)*x[1]**o+(-0.395251919577)*x[1]
        arg[2,4,2,1]=(-0.316538010007)*x[0]**o+(-0.139262497126)*x[0]+(0.645409249217)*x[1]**o+(0.192949521992)*x[1]
        arg[3,0,0,0]=(0.88179290666)*x[0]**o+(0.337853287365)*x[0]+(-0.590219587193)*x[1]**o+(-0.827051504196)*x[1]
        arg[3,0,0,1]=(-0.104526336997)*x[0]**o+(0.299097012282)*x[0]+(-0.348493995853)*x[1]**o+(-0.780034322304)*x[1]
        arg[3,0,1,0]=(0.358724159445)*x[0]**o+(-0.0338089482488)*x[0]+(-0.0423054506456)*x[1]**o+(0.67180489557)*x[1]
        arg[3,0,1,1]=(-0.778880095665)*x[0]**o+(-0.448812173928)*x[0]+(-0.823686122382)*x[1]**o+(0.636028922656)*x[1]
        arg[3,0,2,0]=(0.146891002069)*x[0]**o+(-0.368903199628)*x[0]+(-0.596272077557)*x[1]**o+(0.703510683225)*x[1]
        arg[3,0,2,1]=(0.357416423547)*x[0]**o+(-0.103754905142)*x[0]+(0.32026011189)*x[1]**o+(0.755648597627)*x[1]
        arg[3,1,0,0]=(0.399082464238)*x[0]**o+(-0.12611921797)*x[0]+(0.426552225948)*x[1]**o+(-0.228714297286)*x[1]
        arg[3,1,0,1]=(-0.638411026717)*x[0]**o+(-0.342143922306)*x[0]+(-0.502107802626)*x[1]**o+(-0.881045304848)*x[1]
        arg[3,1,1,0]=(0.411134247295)*x[0]**o+(-0.216585727086)*x[0]+(0.270013339156)*x[1]**o+(0.251581243494)*x[1]
        arg[3,1,1,1]=(0.788758848294)*x[0]**o+(0.345181621902)*x[0]+(0.102843710598)*x[1]**o+(0.0561071077516)*x[1]
        arg[3,1,2,0]=(0.232441162347)*x[0]**o+(-0.687544250386)*x[0]+(0.369065746372)*x[1]**o+(0.628109564412)*x[1]
        arg[3,1,2,1]=(-0.912295044312)*x[0]**o+(0.847830497468)*x[0]+(0.776648618003)*x[1]**o+(0.522638388084)*x[1]
        arg[3,2,0,0]=(0.779333120942)*x[0]**o+(-0.0747119967046)*x[0]+(-0.594363282198)*x[1]**o+(0.513922159444)*x[1]
        arg[3,2,0,1]=(0.877492427404)*x[0]**o+(-0.191727825399)*x[0]+(0.0881296554818)*x[1]**o+(-0.408407503067)*x[1]
        arg[3,2,1,0]=(0.29696771215)*x[0]**o+(-0.914882727138)*x[0]+(0.684156831385)*x[1]**o+(0.142293411327)*x[1]
        arg[3,2,1,1]=(0.105927449304)*x[0]**o+(-0.863339661598)*x[0]+(-0.815550662607)*x[1]**o+(-0.261034715274)*x[1]
        arg[3,2,2,0]=(0.944082529871)*x[0]**o+(-0.449259627627)*x[0]+(-0.560526159016)*x[1]**o+(0.609050500798)*x[1]
        arg[3,2,2,1]=(0.529508870267)*x[0]**o+(0.627216712239)*x[0]+(-0.0148122228382)*x[1]**o+(0.0743274653452)*x[1]
        arg[3,3,0,0]=(-0.30609697448)*x[0]**o+(0.910972095956)*x[0]+(0.256274444924)*x[1]**o+(0.229096821635)*x[1]
        arg[3,3,0,1]=(-0.510323627078)*x[0]**o+(0.308418491165)*x[0]+(0.786418296678)*x[1]**o+(-0.248788086479)*x[1]
        arg[3,3,1,0]=(-0.981107944688)*x[0]**o+(-0.331637054173)*x[0]+(-0.972981902152)*x[1]**o+(-0.389364980465)*x[1]
        arg[3,3,1,1]=(-0.839880147367)*x[0]**o+(-0.911569972467)*x[0]+(0.209642332468)*x[1]**o+(0.0926875442137)*x[1]
        arg[3,3,2,0]=(-0.22724314378)*x[0]**o+(-0.0396604383619)*x[0]+(0.906829234649)*x[1]**o+(-0.862410550066)*x[1]
        arg[3,3,2,1]=(-0.412816401935)*x[0]**o+(-0.848442294241)*x[0]+(-0.599302206188)*x[1]**o+(0.719403009819)*x[1]
        arg[3,4,0,0]=(0.259283054664)*x[0]**o+(0.0983272120339)*x[0]+(-0.538373082787)*x[1]**o+(0.950164435617)*x[1]
        arg[3,4,0,1]=(0.633594236794)*x[0]**o+(0.120984469179)*x[0]+(0.472771297982)*x[1]**o+(0.873015785724)*x[1]
        arg[3,4,1,0]=(0.913524970057)*x[0]**o+(0.106590018424)*x[0]+(0.124026494061)*x[1]**o+(-0.254989594971)*x[1]
        arg[3,4,1,1]=(0.945528885492)*x[0]**o+(-0.323205519576)*x[0]+(-0.467294290307)*x[1]**o+(-0.557033486306)*x[1]
        arg[3,4,2,0]=(-0.268364682764)*x[0]**o+(0.0289685280025)*x[0]+(0.0493964636013)*x[1]**o+(0.872120137365)*x[1]
        arg[3,4,2,1]=(-0.266698796897)*x[0]**o+(-0.679148300702)*x[0]+(0.337653394795)*x[1]**o+(0.186866149716)*x[1]
        ref[0,0,0,0]=(0.358768976574)*x_ref[0]**o+(-0.231637053165)*x_ref[0]+(0.798891506345)*x_ref[1]**o+(0.292325388353)*x_ref[1]
        ref[0,0,0,1]=(0.53301689922)*x_ref[0]**o+(-0.761786870664)*x_ref[0]+(0.284067900826)*x_ref[1]**o+(-0.688662082205)*x_ref[1]
        ref[0,0,1,0]=(-0.721988036113)*x_ref[0]**o+(-0.743892757815)*x_ref[0]+(-0.796787022375)*x_ref[1]**o+(-0.499468412523)*x_ref[1]
        ref[0,0,1,1]=(0.0710691488306)*x_ref[0]**o+(0.0210385141456)*x_ref[0]+(-0.709342417683)*x_ref[1]**o+(0.901794071115)*x_ref[1]
        ref[0,0,2,0]=(-0.800537263339)*x_ref[0]**o+(0.583114786539)*x_ref[0]+(0.422752622522)*x_ref[1]**o+(-0.0954981460748)*x_ref[1]
        ref[0,0,2,1]=(-0.0805357065445)*x_ref[0]**o+(0.476377895547)*x_ref[0]+(0.242203912566)*x_ref[1]**o+(-0.479971540043)*x_ref[1]
        ref[0,1,0,0]=(-0.476356823162)*x_ref[0]**o+(-0.438853622252)*x_ref[0]+(-0.937228141746)*x_ref[1]**o+(0.366496742588)*x_ref[1]
        ref[0,1,0,1]=(-0.496918253956)*x_ref[0]**o+(-0.0227549061268)*x_ref[0]+(0.830195584768)*x_ref[1]**o+(-0.114624902423)*x_ref[1]
        ref[0,1,1,0]=(0.431291036997)*x_ref[0]**o+(0.807713720258)*x_ref[0]+(0.730057543333)*x_ref[1]**o+(0.635171725232)*x_ref[1]
        ref[0,1,1,1]=(0.784425995411)*x_ref[0]**o+(-0.397510494899)*x_ref[0]+(-0.579911740115)*x_ref[1]**o+(0.130426014607)*x_ref[1]
        ref[0,1,2,0]=(0.473613650641)*x_ref[0]**o+(-0.155617741177)*x_ref[0]+(0.873237852103)*x_ref[1]**o+(-0.749395152414)*x_ref[1]
        ref[0,1,2,1]=(0.367649559431)*x_ref[0]**o+(0.830181309644)*x_ref[0]+(0.87726968147)*x_ref[1]**o+(-0.450304793682)*x_ref[1]
        ref[0,2,0,0]=(-0.700522876654)*x_ref[0]**o+(-0.998187126003)*x_ref[0]+(0.732491610173)*x_ref[1]**o+(0.648679267)*x_ref[1]
        ref[0,2,0,1]=(0.499746141285)*x_ref[0]**o+(-0.730090975806)*x_ref[0]+(-0.525566438654)*x_ref[1]**o+(0.212469850421)*x_ref[1]
        ref[0,2,1,0]=(-0.847752916298)*x_ref[0]**o+(0.43510472619)*x_ref[0]+(0.815305936548)*x_ref[1]**o+(-0.143802676552)*x_ref[1]
        ref[0,2,1,1]=(-0.453572376958)*x_ref[0]**o+(-0.913023531929)*x_ref[0]+(0.743328127665)*x_ref[1]**o+(0.152586122591)*x_ref[1]
        ref[0,2,2,0]=(0.526939334508)*x_ref[0]**o+(0.247268722057)*x_ref[0]+(-0.176432621481)*x_ref[1]**o+(-0.510110202196)*x_ref[1]
        ref[0,2,2,1]=(0.139685732003)*x_ref[0]**o+(0.278304455867)*x_ref[0]+(0.517953769616)*x_ref[1]**o+(-0.272384261701)*x_ref[1]
        ref[0,3,0,0]=(-0.38867907688)*x_ref[0]**o+(-0.73043975055)*x_ref[0]+(-0.14293243901)*x_ref[1]**o+(0.0601042854043)*x_ref[1]
        ref[0,3,0,1]=(-0.163055949188)*x_ref[0]**o+(-0.035877062482)*x_ref[0]+(0.0396615423106)*x_ref[1]**o+(-0.870062303275)*x_ref[1]
        ref[0,3,1,0]=(0.283839779914)*x_ref[0]**o+(-0.519316925197)*x_ref[0]+(-0.437401973959)*x_ref[1]**o+(-0.823707232663)*x_ref[1]
        ref[0,3,1,1]=(0.700305425892)*x_ref[0]**o+(0.0299761521094)*x_ref[0]+(-0.993032151025)*x_ref[1]**o+(0.633293413221)*x_ref[1]
        ref[0,3,2,0]=(-0.745357256365)*x_ref[0]**o+(-0.451553456961)*x_ref[0]+(0.05789646302)*x_ref[1]**o+(-0.755901155251)*x_ref[1]
        ref[0,3,2,1]=(-0.57330840403)*x_ref[0]**o+(-0.589361483751)*x_ref[0]+(-0.038677123513)*x_ref[1]**o+(0.417332032168)*x_ref[1]
        ref[0,4,0,0]=(0.759655161531)*x_ref[0]**o+(0.737483461328)*x_ref[0]+(0.573158629937)*x_ref[1]**o+(-0.454191109106)*x_ref[1]
        ref[0,4,0,1]=(-0.232982406645)*x_ref[0]**o+(-0.926030038179)*x_ref[0]+(-0.504256834465)*x_ref[1]**o+(-0.107703527058)*x_ref[1]
        ref[0,4,1,0]=(0.898411697016)*x_ref[0]**o+(0.642045536742)*x_ref[0]+(-0.953059791895)*x_ref[1]**o+(-0.67501710093)*x_ref[1]
        ref[0,4,1,1]=(0.300380574165)*x_ref[0]**o+(0.204591960115)*x_ref[0]+(-0.308849697562)*x_ref[1]**o+(-0.489495887357)*x_ref[1]
        ref[0,4,2,0]=(-0.814560286716)*x_ref[0]**o+(0.154235599953)*x_ref[0]+(0.279329273498)*x_ref[1]**o+(-0.674063978926)*x_ref[1]
        ref[0,4,2,1]=(-0.718018870365)*x_ref[0]**o+(-0.735982858424)*x_ref[0]+(-0.0132197132926)*x_ref[1]**o+(0.398939587543)*x_ref[1]
        ref[1,0,0,0]=(0.0559995351725)*x_ref[0]**o+(-0.164549619892)*x_ref[0]+(-0.794242529408)*x_ref[1]**o+(0.0927822314785)*x_ref[1]
        ref[1,0,0,1]=(-0.894238542341)*x_ref[0]**o+(0.378559247473)*x_ref[0]+(-0.183957272639)*x_ref[1]**o+(0.486358968942)*x_ref[1]
        ref[1,0,1,0]=(0.931288801001)*x_ref[0]**o+(0.223815769968)*x_ref[0]+(-0.719591906328)*x_ref[1]**o+(-0.743644741105)*x_ref[1]
        ref[1,0,1,1]=(0.156646029123)*x_ref[0]**o+(0.306942879419)*x_ref[0]+(0.747006830674)*x_ref[1]**o+(-0.876534912274)*x_ref[1]
        ref[1,0,2,0]=(-0.496819081999)*x_ref[0]**o+(-0.939227402425)*x_ref[0]+(0.0566331712182)*x_ref[1]**o+(0.565724364661)*x_ref[1]
        ref[1,0,2,1]=(0.903706429954)*x_ref[0]**o+(0.667947320307)*x_ref[0]+(0.97140855373)*x_ref[1]**o+(0.924095235752)*x_ref[1]
        ref[1,1,0,0]=(0.526180197299)*x_ref[0]**o+(0.435493351674)*x_ref[0]+(0.400107077848)*x_ref[1]**o+(-0.704760833758)*x_ref[1]
        ref[1,1,0,1]=(0.0723740015004)*x_ref[0]**o+(-0.405859051371)*x_ref[0]+(0.200787757511)*x_ref[1]**o+(0.41819184563)*x_ref[1]
        ref[1,1,1,0]=(-0.0320951655627)*x_ref[0]**o+(0.235068445821)*x_ref[0]+(0.450383349599)*x_ref[1]**o+(-0.801487424765)*x_ref[1]
        ref[1,1,1,1]=(-0.52478892744)*x_ref[0]**o+(-0.449111297125)*x_ref[0]+(0.731062182014)*x_ref[1]**o+(0.533619608577)*x_ref[1]
        ref[1,1,2,0]=(0.0889794057057)*x_ref[0]**o+(-0.971856981571)*x_ref[0]+(0.662216408922)*x_ref[1]**o+(0.164089024468)*x_ref[1]
        ref[1,1,2,1]=(-0.196374963647)*x_ref[0]**o+(-0.739360211802)*x_ref[0]+(0.916611650753)*x_ref[1]**o+(-0.260102318115)*x_ref[1]
        ref[1,2,0,0]=(0.627763572103)*x_ref[0]**o+(-0.893900847319)*x_ref[0]+(0.907548446417)*x_ref[1]**o+(0.0687906203663)*x_ref[1]
        ref[1,2,0,1]=(0.59755498682)*x_ref[0]**o+(-0.457108009787)*x_ref[0]+(0.71562139654)*x_ref[1]**o+(-0.300872832559)*x_ref[1]
        ref[1,2,1,0]=(-0.479641829834)*x_ref[0]**o+(0.297188513722)*x_ref[0]+(0.0227730978493)*x_ref[1]**o+(0.199329665406)*x_ref[1]
        ref[1,2,1,1]=(0.918974940101)*x_ref[0]**o+(0.595194981816)*x_ref[0]+(0.760030152554)*x_ref[1]**o+(-0.798899878486)*x_ref[1]
        ref[1,2,2,0]=(-0.878334511842)*x_ref[0]**o+(0.910857236287)*x_ref[0]+(0.0164747567997)*x_ref[1]**o+(-0.876970888445)*x_ref[1]
        ref[1,2,2,1]=(0.29445809181)*x_ref[0]**o+(0.837849454931)*x_ref[0]+(0.0759054371211)*x_ref[1]**o+(0.501243080372)*x_ref[1]
        ref[1,3,0,0]=(-0.529006436258)*x_ref[0]**o+(-0.932325855538)*x_ref[0]+(-0.561472972374)*x_ref[1]**o+(0.845125736656)*x_ref[1]
        ref[1,3,0,1]=(-0.843008257488)*x_ref[0]**o+(0.614118623012)*x_ref[0]+(-0.167015137447)*x_ref[1]**o+(0.158701952625)*x_ref[1]
        ref[1,3,1,0]=(-0.662966916148)*x_ref[0]**o+(-0.443297576512)*x_ref[0]+(0.295988608077)*x_ref[1]**o+(0.059506908615)*x_ref[1]
        ref[1,3,1,1]=(0.864199978014)*x_ref[0]**o+(0.138723519602)*x_ref[0]+(0.144964839302)*x_ref[1]**o+(0.239863709389)*x_ref[1]
        ref[1,3,2,0]=(0.479006097431)*x_ref[0]**o+(0.542168797821)*x_ref[0]+(0.561074562163)*x_ref[1]**o+(-0.133365787275)*x_ref[1]
        ref[1,3,2,1]=(-0.901033449452)*x_ref[0]**o+(-0.620245150074)*x_ref[0]+(0.598461569697)*x_ref[1]**o+(-0.2213246854)*x_ref[1]
        ref[1,4,0,0]=(-0.577067401348)*x_ref[0]**o+(0.486154324874)*x_ref[0]+(0.809205150442)*x_ref[1]**o+(-0.218206418459)*x_ref[1]
        ref[1,4,0,1]=(0.726556862066)*x_ref[0]**o+(0.0293695509867)*x_ref[0]+(0.601938348193)*x_ref[1]**o+(0.967332697531)*x_ref[1]
        ref[1,4,1,0]=(-0.675624388054)*x_ref[0]**o+(-0.871613723214)*x_ref[0]+(0.605519387693)*x_ref[1]**o+(0.287186894411)*x_ref[1]
        ref[1,4,1,1]=(-0.689904898214)*x_ref[0]**o+(-0.918236705665)*x_ref[0]+(-0.892168622177)*x_ref[1]**o+(0.507441662612)*x_ref[1]
        ref[1,4,2,0]=(0.845768980712)*x_ref[0]**o+(-0.458467508114)*x_ref[0]+(-0.518185249391)*x_ref[1]**o+(0.829945330327)*x_ref[1]
        ref[1,4,2,1]=(0.776852312178)*x_ref[0]**o+(-0.268810999469)*x_ref[0]+(-0.576757148841)*x_ref[1]**o+(-0.50628294802)*x_ref[1]
        ref[2,0,0,0]=(-0.686840219939)*x_ref[0]**o+(0.6861172847)*x_ref[0]+(0.240362263052)*x_ref[1]**o+(0.962361523884)*x_ref[1]
        ref[2,0,0,1]=(-0.943837188119)*x_ref[0]**o+(0.829832768268)*x_ref[0]+(0.377590124133)*x_ref[1]**o+(-0.171714144962)*x_ref[1]
        ref[2,0,1,0]=(0.862809148447)*x_ref[0]**o+(0.0573402136489)*x_ref[0]+(-0.743311826013)*x_ref[1]**o+(0.427809313437)*x_ref[1]
        ref[2,0,1,1]=(0.377380819576)*x_ref[0]**o+(-0.223280285041)*x_ref[0]+(0.178628658562)*x_ref[1]**o+(0.121964611877)*x_ref[1]
        ref[2,0,2,0]=(0.41723758689)*x_ref[0]**o+(0.256039203883)*x_ref[0]+(0.255358776676)*x_ref[1]**o+(-0.881665964974)*x_ref[1]
        ref[2,0,2,1]=(-0.0797103024499)*x_ref[0]**o+(0.128068931414)*x_ref[0]+(-0.758317834409)*x_ref[1]**o+(0.654657809183)*x_ref[1]
        ref[2,1,0,0]=(0.603648218133)*x_ref[0]**o+(-0.286780270729)*x_ref[0]+(-0.341108067214)*x_ref[1]**o+(-0.769667356087)*x_ref[1]
        ref[2,1,0,1]=(0.844044270035)*x_ref[0]**o+(0.0585907857764)*x_ref[0]+(-0.578232785252)*x_ref[1]**o+(-0.40959017859)*x_ref[1]
        ref[2,1,1,0]=(0.599709567393)*x_ref[0]**o+(0.307092449739)*x_ref[0]+(-0.402950586215)*x_ref[1]**o+(0.0674239470451)*x_ref[1]
        ref[2,1,1,1]=(0.182608249961)*x_ref[0]**o+(0.373533584601)*x_ref[0]+(0.40949869082)*x_ref[1]**o+(0.696333617699)*x_ref[1]
        ref[2,1,2,0]=(0.0462891111054)*x_ref[0]**o+(0.371258293292)*x_ref[0]+(0.782976187716)*x_ref[1]**o+(0.117976017999)*x_ref[1]
        ref[2,1,2,1]=(0.891841121526)*x_ref[0]**o+(-0.998971173204)*x_ref[0]+(0.0286712507928)*x_ref[1]**o+(-0.0198507022516)*x_ref[1]
        ref[2,2,0,0]=(0.284620902703)*x_ref[0]**o+(0.936231203073)*x_ref[0]+(0.829209752046)*x_ref[1]**o+(-0.94940982125)*x_ref[1]
        ref[2,2,0,1]=(-0.647956027376)*x_ref[0]**o+(0.923690718922)*x_ref[0]+(-0.0322423837346)*x_ref[1]**o+(0.124962508606)*x_ref[1]
        ref[2,2,1,0]=(-0.0696010411705)*x_ref[0]**o+(-0.161672936739)*x_ref[0]+(-0.997445762186)*x_ref[1]**o+(0.177962302014)*x_ref[1]
        ref[2,2,1,1]=(-0.897657700296)*x_ref[0]**o+(-0.467866721175)*x_ref[0]+(-0.560034332007)*x_ref[1]**o+(-0.861509898295)*x_ref[1]
        ref[2,2,2,0]=(-0.100880174048)*x_ref[0]**o+(0.678568254695)*x_ref[0]+(-0.83289411684)*x_ref[1]**o+(0.680564393149)*x_ref[1]
        ref[2,2,2,1]=(0.546159181491)*x_ref[0]**o+(0.720296622966)*x_ref[0]+(0.939046941118)*x_ref[1]**o+(-0.716147554733)*x_ref[1]
        ref[2,3,0,0]=(-0.921073162235)*x_ref[0]**o+(0.979925882289)*x_ref[0]+(-0.0938642966949)*x_ref[1]**o+(0.482093047223)*x_ref[1]
        ref[2,3,0,1]=(0.0709940956822)*x_ref[0]**o+(-0.691401017243)*x_ref[0]+(0.447000814029)*x_ref[1]**o+(0.0575856240533)*x_ref[1]
        ref[2,3,1,0]=(0.562800393365)*x_ref[0]**o+(0.657646549915)*x_ref[0]+(-0.835326361689)*x_ref[1]**o+(0.3487960687)*x_ref[1]
        ref[2,3,1,1]=(0.106668867409)*x_ref[0]**o+(0.436327240765)*x_ref[0]+(-0.585051261259)*x_ref[1]**o+(0.616655338667)*x_ref[1]
        ref[2,3,2,0]=(-0.501849628176)*x_ref[0]**o+(-0.284722503916)*x_ref[0]+(-0.892470199837)*x_ref[1]**o+(-0.979942541795)*x_ref[1]
        ref[2,3,2,1]=(0.236759540073)*x_ref[0]**o+(-0.806931113769)*x_ref[0]+(-0.225220003475)*x_ref[1]**o+(-0.95183239844)*x_ref[1]
        ref[2,4,0,0]=(-0.684823909581)*x_ref[0]**o+(0.445615228059)*x_ref[0]+(-0.638879895767)*x_ref[1]**o+(-0.335505874088)*x_ref[1]
        ref[2,4,0,1]=(-0.196708460612)*x_ref[0]**o+(0.981592996809)*x_ref[0]+(0.435774317028)*x_ref[1]**o+(-0.0952200663109)*x_ref[1]
        ref[2,4,1,0]=(-0.552759474764)*x_ref[0]**o+(-0.614031173545)*x_ref[0]+(0.304375542871)*x_ref[1]**o+(-0.285131901533)*x_ref[1]
        ref[2,4,1,1]=(0.147812951421)*x_ref[0]**o+(-0.78533487681)*x_ref[0]+(0.942205524401)*x_ref[1]**o+(0.166280215075)*x_ref[1]
        ref[2,4,2,0]=(0.843067982563)*x_ref[0]**o+(0.0116520710289)*x_ref[0]+(0.927948889059)*x_ref[1]**o+(-0.395251919577)*x_ref[1]
        ref[2,4,2,1]=(-0.316538010007)*x_ref[0]**o+(-0.139262497126)*x_ref[0]+(0.645409249217)*x_ref[1]**o+(0.192949521992)*x_ref[1]
        ref[3,0,0,0]=(0.88179290666)*x_ref[0]**o+(0.337853287365)*x_ref[0]+(-0.590219587193)*x_ref[1]**o+(-0.827051504196)*x_ref[1]
        ref[3,0,0,1]=(-0.104526336997)*x_ref[0]**o+(0.299097012282)*x_ref[0]+(-0.348493995853)*x_ref[1]**o+(-0.780034322304)*x_ref[1]
        ref[3,0,1,0]=(0.358724159445)*x_ref[0]**o+(-0.0338089482488)*x_ref[0]+(-0.0423054506456)*x_ref[1]**o+(0.67180489557)*x_ref[1]
        ref[3,0,1,1]=(-0.778880095665)*x_ref[0]**o+(-0.448812173928)*x_ref[0]+(-0.823686122382)*x_ref[1]**o+(0.636028922656)*x_ref[1]
        ref[3,0,2,0]=(0.146891002069)*x_ref[0]**o+(-0.368903199628)*x_ref[0]+(-0.596272077557)*x_ref[1]**o+(0.703510683225)*x_ref[1]
        ref[3,0,2,1]=(0.357416423547)*x_ref[0]**o+(-0.103754905142)*x_ref[0]+(0.32026011189)*x_ref[1]**o+(0.755648597627)*x_ref[1]
        ref[3,1,0,0]=(0.399082464238)*x_ref[0]**o+(-0.12611921797)*x_ref[0]+(0.426552225948)*x_ref[1]**o+(-0.228714297286)*x_ref[1]
        ref[3,1,0,1]=(-0.638411026717)*x_ref[0]**o+(-0.342143922306)*x_ref[0]+(-0.502107802626)*x_ref[1]**o+(-0.881045304848)*x_ref[1]
        ref[3,1,1,0]=(0.411134247295)*x_ref[0]**o+(-0.216585727086)*x_ref[0]+(0.270013339156)*x_ref[1]**o+(0.251581243494)*x_ref[1]
        ref[3,1,1,1]=(0.788758848294)*x_ref[0]**o+(0.345181621902)*x_ref[0]+(0.102843710598)*x_ref[1]**o+(0.0561071077516)*x_ref[1]
        ref[3,1,2,0]=(0.232441162347)*x_ref[0]**o+(-0.687544250386)*x_ref[0]+(0.369065746372)*x_ref[1]**o+(0.628109564412)*x_ref[1]
        ref[3,1,2,1]=(-0.912295044312)*x_ref[0]**o+(0.847830497468)*x_ref[0]+(0.776648618003)*x_ref[1]**o+(0.522638388084)*x_ref[1]
        ref[3,2,0,0]=(0.779333120942)*x_ref[0]**o+(-0.0747119967046)*x_ref[0]+(-0.594363282198)*x_ref[1]**o+(0.513922159444)*x_ref[1]
        ref[3,2,0,1]=(0.877492427404)*x_ref[0]**o+(-0.191727825399)*x_ref[0]+(0.0881296554818)*x_ref[1]**o+(-0.408407503067)*x_ref[1]
        ref[3,2,1,0]=(0.29696771215)*x_ref[0]**o+(-0.914882727138)*x_ref[0]+(0.684156831385)*x_ref[1]**o+(0.142293411327)*x_ref[1]
        ref[3,2,1,1]=(0.105927449304)*x_ref[0]**o+(-0.863339661598)*x_ref[0]+(-0.815550662607)*x_ref[1]**o+(-0.261034715274)*x_ref[1]
        ref[3,2,2,0]=(0.944082529871)*x_ref[0]**o+(-0.449259627627)*x_ref[0]+(-0.560526159016)*x_ref[1]**o+(0.609050500798)*x_ref[1]
        ref[3,2,2,1]=(0.529508870267)*x_ref[0]**o+(0.627216712239)*x_ref[0]+(-0.0148122228382)*x_ref[1]**o+(0.0743274653452)*x_ref[1]
        ref[3,3,0,0]=(-0.30609697448)*x_ref[0]**o+(0.910972095956)*x_ref[0]+(0.256274444924)*x_ref[1]**o+(0.229096821635)*x_ref[1]
        ref[3,3,0,1]=(-0.510323627078)*x_ref[0]**o+(0.308418491165)*x_ref[0]+(0.786418296678)*x_ref[1]**o+(-0.248788086479)*x_ref[1]
        ref[3,3,1,0]=(-0.981107944688)*x_ref[0]**o+(-0.331637054173)*x_ref[0]+(-0.972981902152)*x_ref[1]**o+(-0.389364980465)*x_ref[1]
        ref[3,3,1,1]=(-0.839880147367)*x_ref[0]**o+(-0.911569972467)*x_ref[0]+(0.209642332468)*x_ref[1]**o+(0.0926875442137)*x_ref[1]
        ref[3,3,2,0]=(-0.22724314378)*x_ref[0]**o+(-0.0396604383619)*x_ref[0]+(0.906829234649)*x_ref[1]**o+(-0.862410550066)*x_ref[1]
        ref[3,3,2,1]=(-0.412816401935)*x_ref[0]**o+(-0.848442294241)*x_ref[0]+(-0.599302206188)*x_ref[1]**o+(0.719403009819)*x_ref[1]
        ref[3,4,0,0]=(0.259283054664)*x_ref[0]**o+(0.0983272120339)*x_ref[0]+(-0.538373082787)*x_ref[1]**o+(0.950164435617)*x_ref[1]
        ref[3,4,0,1]=(0.633594236794)*x_ref[0]**o+(0.120984469179)*x_ref[0]+(0.472771297982)*x_ref[1]**o+(0.873015785724)*x_ref[1]
        ref[3,4,1,0]=(0.913524970057)*x_ref[0]**o+(0.106590018424)*x_ref[0]+(0.124026494061)*x_ref[1]**o+(-0.254989594971)*x_ref[1]
        ref[3,4,1,1]=(0.945528885492)*x_ref[0]**o+(-0.323205519576)*x_ref[0]+(-0.467294290307)*x_ref[1]**o+(-0.557033486306)*x_ref[1]
        ref[3,4,2,0]=(-0.268364682764)*x_ref[0]**o+(0.0289685280025)*x_ref[0]+(0.0493964636013)*x_ref[1]**o+(0.872120137365)*x_ref[1]
        ref[3,4,2,1]=(-0.266698796897)*x_ref[0]**o+(-0.679148300702)*x_ref[0]+(0.337653394795)*x_ref[1]**o+(0.186866149716)*x_ref[1]
      else:
        arg[0,0,0,0]=(-0.928996629952)*x[0]**o+(-0.171383074114)*x[0]+(-0.324423798901)*x[1]**o+(-0.801354053962)*x[1]+(0.365256142808)*x[2]**o+(0.126262208857)*x[2]
        arg[0,0,0,1]=(0.326024843626)*x[0]**o+(-0.0091566240426)*x[0]+(-0.0396235158589)*x[1]**o+(-0.753302577837)*x[1]+(-0.38709916587)*x[2]**o+(-0.28660316595)*x[2]
        arg[0,0,1,0]=(0.755338255506)*x[0]**o+(0.851895434343)*x[0]+(0.419119464402)*x[1]**o+(0.670733085023)*x[1]+(0.819398310958)*x[2]**o+(0.476185335773)*x[2]
        arg[0,0,1,1]=(-0.981416259131)*x[0]**o+(-0.97000252267)*x[0]+(-0.132624451699)*x[1]**o+(-0.261428370649)*x[1]+(-0.349245951727)*x[2]**o+(-0.717343195877)*x[2]
        arg[0,0,2,0]=(-0.874826075266)*x[0]**o+(-0.399307905707)*x[0]+(0.350873599317)*x[1]**o+(0.15861651992)*x[1]+(0.396279933216)*x[2]**o+(0.92332515708)*x[2]
        arg[0,0,2,1]=(-0.183596683527)*x[0]**o+(-0.115194639348)*x[0]+(0.616111723618)*x[1]**o+(-0.402157657669)*x[1]+(0.44145594193)*x[2]**o+(0.686052407784)*x[2]
        arg[0,1,0,0]=(-0.669169178244)*x[0]**o+(0.835937801609)*x[0]+(0.468805302181)*x[1]**o+(-0.627292634023)*x[1]+(-0.674361231979)*x[2]**o+(-0.00929181785129)*x[2]
        arg[0,1,0,1]=(-0.568131655705)*x[0]**o+(0.690366535852)*x[0]+(-0.212524396914)*x[1]**o+(-0.767983769407)*x[1]+(0.477500128569)*x[2]**o+(-0.0190887598644)*x[2]
        arg[0,1,1,0]=(-0.374170635822)*x[0]**o+(0.0169009974558)*x[0]+(-0.162519103357)*x[1]**o+(0.786991144449)*x[1]+(0.0590294918131)*x[2]**o+(-0.213442509309)*x[2]
        arg[0,1,1,1]=(-0.558268657439)*x[0]**o+(0.584734933669)*x[0]+(-0.650790013689)*x[1]**o+(-0.192849048407)*x[1]+(0.505458636714)*x[2]**o+(-0.585174281371)*x[2]
        arg[0,1,2,0]=(-0.933243590775)*x[0]**o+(0.528060249767)*x[0]+(-0.763255746252)*x[1]**o+(0.771589118163)*x[1]+(-0.641287604338)*x[2]**o+(0.379566401585)*x[2]
        arg[0,1,2,1]=(-0.836976373935)*x[0]**o+(-0.969434110974)*x[0]+(0.758148372888)*x[1]**o+(-0.89212062535)*x[1]+(0.788693820015)*x[2]**o+(-0.620704367414)*x[2]
        arg[0,2,0,0]=(0.0163117360814)*x[0]**o+(-0.390867752068)*x[0]+(-0.725068297982)*x[1]**o+(-0.680312496646)*x[1]+(0.069273494969)*x[2]**o+(-0.576122275696)*x[2]
        arg[0,2,0,1]=(0.461771197614)*x[0]**o+(-0.767541404109)*x[0]+(-0.694279263622)*x[1]**o+(-0.721448812749)*x[1]+(0.248787896584)*x[2]**o+(0.237075401327)*x[2]
        arg[0,2,1,0]=(0.381987012965)*x[0]**o+(-0.288236233238)*x[0]+(-0.519012025084)*x[1]**o+(-0.256658293082)*x[1]+(0.807322167368)*x[2]**o+(-0.261916179859)*x[2]
        arg[0,2,1,1]=(0.257241267407)*x[0]**o+(-0.470612990721)*x[0]+(-0.208612656448)*x[1]**o+(0.839646962914)*x[1]+(0.916675150494)*x[2]**o+(-0.165468351005)*x[2]
        arg[0,2,2,0]=(0.92569232521)*x[0]**o+(0.809982458308)*x[0]+(-0.860530305336)*x[1]**o+(0.261687622817)*x[1]+(0.948003684651)*x[2]**o+(-0.473133742361)*x[2]
        arg[0,2,2,1]=(-0.113846814874)*x[0]**o+(0.680846459377)*x[0]+(0.920219149741)*x[1]**o+(0.172027554503)*x[1]+(0.475330041115)*x[2]**o+(-0.176218891429)*x[2]
        arg[0,3,0,0]=(-0.948638294297)*x[0]**o+(-0.0411621070703)*x[0]+(-0.294759271027)*x[1]**o+(-0.903733455086)*x[1]+(-0.995253733593)*x[2]**o+(0.106726960462)*x[2]
        arg[0,3,0,1]=(0.1592709614)*x[0]**o+(0.0683266445293)*x[0]+(0.336092506745)*x[1]**o+(-0.209047739881)*x[1]+(-0.330417243459)*x[2]**o+(-0.809242647755)*x[2]
        arg[0,3,1,0]=(-0.680637516359)*x[0]**o+(-0.713405815404)*x[0]+(-0.484624435043)*x[1]**o+(-0.279314462332)*x[1]+(-0.458856066504)*x[2]**o+(0.0588111305227)*x[2]
        arg[0,3,1,1]=(-0.853676338655)*x[0]**o+(-0.651539814195)*x[0]+(-0.109727277725)*x[1]**o+(0.0139006407942)*x[1]+(0.231943245426)*x[2]**o+(0.42972517233)*x[2]
        arg[0,3,2,0]=(-0.60799269765)*x[0]**o+(-0.802413269128)*x[0]+(0.151362498863)*x[1]**o+(-0.678226252875)*x[1]+(-0.867117685659)*x[2]**o+(-0.711982116071)*x[2]
        arg[0,3,2,1]=(-0.243012743941)*x[0]**o+(-0.692614579016)*x[0]+(0.0986817641914)*x[1]**o+(-0.495735862695)*x[1]+(-0.430040681653)*x[2]**o+(0.0108928722284)*x[2]
        arg[0,4,0,0]=(0.0989402181558)*x[0]**o+(-0.60096719882)*x[0]+(-0.324253928432)*x[1]**o+(-0.862723687559)*x[1]+(-0.34745416427)*x[2]**o+(0.936559074873)*x[2]
        arg[0,4,0,1]=(0.711021967566)*x[0]**o+(-0.939408401113)*x[0]+(-0.218100921633)*x[1]**o+(0.92835194875)*x[1]+(-0.353936913865)*x[2]**o+(-0.547266090928)*x[2]
        arg[0,4,1,0]=(0.172231614188)*x[0]**o+(-0.734622090671)*x[0]+(0.432686916647)*x[1]**o+(0.552595350492)*x[1]+(0.779662641076)*x[2]**o+(-0.0933220492779)*x[2]
        arg[0,4,1,1]=(0.996776716866)*x[0]**o+(-0.340786520686)*x[0]+(0.260075214171)*x[1]**o+(0.00385504852496)*x[1]+(0.377516772127)*x[2]**o+(-0.341751634181)*x[2]
        arg[0,4,2,0]=(-0.655642896664)*x[0]**o+(-0.840525173737)*x[0]+(-0.855343721564)*x[1]**o+(0.253558571524)*x[1]+(0.0463733059925)*x[2]**o+(-0.116359866912)*x[2]
        arg[0,4,2,1]=(0.131619661414)*x[0]**o+(-0.162067255954)*x[0]+(0.562536831367)*x[1]**o+(-0.940412472035)*x[1]+(-0.282265762783)*x[2]**o+(0.634711607923)*x[2]
        arg[1,0,0,0]=(0.484071808097)*x[0]**o+(0.566763524691)*x[0]+(-0.658614173653)*x[1]**o+(-0.140192232895)*x[1]+(-0.310114236409)*x[2]**o+(-0.918532401595)*x[2]
        arg[1,0,0,1]=(-0.81190380441)*x[0]**o+(0.644439878497)*x[0]+(0.479583403289)*x[1]**o+(-0.493318882779)*x[1]+(-0.424704492175)*x[2]**o+(0.700624317732)*x[2]
        arg[1,0,1,0]=(0.766650182435)*x[0]**o+(0.226365545129)*x[0]+(0.510828945971)*x[1]**o+(0.283221954498)*x[1]+(0.817303877886)*x[2]**o+(-0.245709044628)*x[2]
        arg[1,0,1,1]=(0.194787112664)*x[0]**o+(-0.879306736922)*x[0]+(0.185381291627)*x[1]**o+(-0.510692743975)*x[1]+(-0.422555014055)*x[2]**o+(0.556629919959)*x[2]
        arg[1,0,2,0]=(-0.958345828895)*x[0]**o+(-0.667735551013)*x[0]+(0.278271284225)*x[1]**o+(-0.0847995194757)*x[1]+(0.902184171441)*x[2]**o+(0.763125168854)*x[2]
        arg[1,0,2,1]=(-0.0766429670195)*x[0]**o+(0.894421139029)*x[0]+(-0.331560314101)*x[1]**o+(-0.0253297623997)*x[1]+(0.13406045967)*x[2]**o+(-0.326681111625)*x[2]
        arg[1,1,0,0]=(-0.979406855838)*x[0]**o+(0.528849909402)*x[0]+(-0.0486159476737)*x[1]**o+(-0.898392512695)*x[1]+(-0.678328327407)*x[2]**o+(0.644201802057)*x[2]
        arg[1,1,0,1]=(-0.977485554605)*x[0]**o+(0.794799941538)*x[0]+(-0.00237183848504)*x[1]**o+(-0.633497483149)*x[1]+(0.746593853568)*x[2]**o+(0.197741126365)*x[2]
        arg[1,1,1,0]=(0.379985506182)*x[0]**o+(0.69889144234)*x[0]+(-0.507293887409)*x[1]**o+(-0.727675545291)*x[1]+(-0.467064531693)*x[2]**o+(0.130273930089)*x[2]
        arg[1,1,1,1]=(-0.330627830819)*x[0]**o+(-0.0311542935889)*x[0]+(-0.0671494148749)*x[1]**o+(0.583605469163)*x[1]+(-0.172083996069)*x[2]**o+(0.666718011074)*x[2]
        arg[1,1,2,0]=(0.539618381063)*x[0]**o+(-0.546796237888)*x[0]+(0.857911750392)*x[1]**o+(0.119552731463)*x[1]+(0.329286820109)*x[2]**o+(0.637240649087)*x[2]
        arg[1,1,2,1]=(-0.828603071334)*x[0]**o+(0.861799149065)*x[0]+(0.615224040538)*x[1]**o+(-0.0868985543263)*x[1]+(-0.206168997809)*x[2]**o+(0.311103719286)*x[2]
        arg[1,2,0,0]=(0.695106867715)*x[0]**o+(-0.533303418727)*x[0]+(-0.393936955207)*x[1]**o+(-0.407635261686)*x[1]+(0.604346269147)*x[2]**o+(-0.10213061533)*x[2]
        arg[1,2,0,1]=(0.338035309929)*x[0]**o+(0.517135412671)*x[0]+(0.277863735829)*x[1]**o+(-0.703417079398)*x[1]+(0.369398635984)*x[2]**o+(-0.952905070601)*x[2]
        arg[1,2,1,0]=(0.937866603297)*x[0]**o+(0.436121631708)*x[0]+(0.568362098461)*x[1]**o+(0.223837722984)*x[1]+(-0.236896910941)*x[2]**o+(0.316114408308)*x[2]
        arg[1,2,1,1]=(0.285714491932)*x[0]**o+(0.633321537987)*x[0]+(0.953737237202)*x[1]**o+(0.732065555924)*x[1]+(-0.515759068197)*x[2]**o+(-0.519775334889)*x[2]
        arg[1,2,2,0]=(-0.0610222303479)*x[0]**o+(0.135974983944)*x[0]+(0.450213393447)*x[1]**o+(-0.44747541715)*x[1]+(-0.640660207187)*x[2]**o+(0.348457261119)*x[2]
        arg[1,2,2,1]=(0.131753033369)*x[0]**o+(-0.311359549998)*x[0]+(0.445707767235)*x[1]**o+(-0.710795569938)*x[1]+(-0.746038648477)*x[2]**o+(-0.210414773188)*x[2]
        arg[1,3,0,0]=(-0.323684061289)*x[0]**o+(-0.0937091220282)*x[0]+(-0.827884898323)*x[1]**o+(0.604442907568)*x[1]+(0.620923596379)*x[2]**o+(0.851758170763)*x[2]
        arg[1,3,0,1]=(-0.269239872573)*x[0]**o+(-0.603248280988)*x[0]+(0.603804530326)*x[1]**o+(-0.331910095046)*x[1]+(-0.653877236308)*x[2]**o+(0.115434865132)*x[2]
        arg[1,3,1,0]=(0.0531798412869)*x[0]**o+(0.292950346899)*x[0]+(-0.634376081823)*x[1]**o+(-0.775427771578)*x[1]+(-0.987113467149)*x[2]**o+(0.757980444447)*x[2]
        arg[1,3,1,1]=(0.757807292214)*x[0]**o+(0.81778478971)*x[0]+(-0.619873148405)*x[1]**o+(0.0129855659138)*x[1]+(-0.282861093681)*x[2]**o+(0.667211104045)*x[2]
        arg[1,3,2,0]=(-0.915205606853)*x[0]**o+(0.918704643459)*x[0]+(0.143238155504)*x[1]**o+(0.0285686377214)*x[1]+(-0.848712843243)*x[2]**o+(0.058647124862)*x[2]
        arg[1,3,2,1]=(0.441487712831)*x[0]**o+(-0.626878272507)*x[0]+(-0.862579662295)*x[1]**o+(0.204629243832)*x[1]+(0.964143347265)*x[2]**o+(-0.399125844282)*x[2]
        arg[1,4,0,0]=(-0.349791275126)*x[0]**o+(-0.140293844776)*x[0]+(-0.693879671138)*x[1]**o+(0.429290435637)*x[1]+(-0.892670314048)*x[2]**o+(-0.380777380409)*x[2]
        arg[1,4,0,1]=(0.528712424647)*x[0]**o+(0.449268002671)*x[0]+(-0.202472214961)*x[1]**o+(-0.374762208851)*x[1]+(-0.625590920648)*x[2]**o+(-0.000776205555531)*x[2]
        arg[1,4,1,0]=(0.00425881397184)*x[0]**o+(0.265831686248)*x[0]+(0.757626547866)*x[1]**o+(-0.934207064756)*x[1]+(0.541356932538)*x[2]**o+(0.217841597089)*x[2]
        arg[1,4,1,1]=(-0.02422207106)*x[0]**o+(-0.807018149115)*x[0]+(-0.402347046418)*x[1]**o+(-0.0548851919048)*x[1]+(0.850202044077)*x[2]**o+(-0.889576261979)*x[2]
        arg[1,4,2,0]=(0.487059068561)*x[0]**o+(0.338254525003)*x[0]+(-0.387019663154)*x[1]**o+(-0.436969552236)*x[1]+(-0.747690766603)*x[2]**o+(0.265854907003)*x[2]
        arg[1,4,2,1]=(0.687562221132)*x[0]**o+(0.4755782976)*x[0]+(-0.456287111804)*x[1]**o+(0.412138123928)*x[1]+(0.637920334647)*x[2]**o+(0.817007725525)*x[2]
        arg[2,0,0,0]=(-0.728166422776)*x[0]**o+(0.108904177195)*x[0]+(0.383848108635)*x[1]**o+(-0.19252707292)*x[1]+(-0.279319122495)*x[2]**o+(-0.0530727745585)*x[2]
        arg[2,0,0,1]=(0.380827452272)*x[0]**o+(-0.133776280687)*x[0]+(-0.174731551283)*x[1]**o+(-0.369231086652)*x[1]+(-0.0190771818046)*x[2]**o+(-0.331873068919)*x[2]
        arg[2,0,1,0]=(0.653644298826)*x[0]**o+(-0.916110182689)*x[0]+(0.0510855252492)*x[1]**o+(-0.934192949208)*x[1]+(0.572048462784)*x[2]**o+(-0.451401856863)*x[2]
        arg[2,0,1,1]=(-0.394958497502)*x[0]**o+(-0.690752972227)*x[0]+(-0.436352069349)*x[1]**o+(0.933524576573)*x[1]+(-0.524344161699)*x[2]**o+(0.526748792776)*x[2]
        arg[2,0,2,0]=(0.845649523782)*x[0]**o+(0.63927747558)*x[0]+(-0.534192218216)*x[1]**o+(0.714500925459)*x[1]+(0.156077814898)*x[2]**o+(-0.193712778792)*x[2]
        arg[2,0,2,1]=(0.809636640413)*x[0]**o+(-0.930654037481)*x[0]+(0.509441692942)*x[1]**o+(0.881563824585)*x[1]+(0.101333749057)*x[2]**o+(0.418791722704)*x[2]
        arg[2,1,0,0]=(0.289481281745)*x[0]**o+(-0.98279193802)*x[0]+(-0.443382747319)*x[1]**o+(-0.281710992472)*x[1]+(0.426349350391)*x[2]**o+(-0.870192121251)*x[2]
        arg[2,1,0,1]=(0.837046002753)*x[0]**o+(-0.0254166983371)*x[0]+(0.30178635814)*x[1]**o+(0.973915444924)*x[1]+(0.298631524734)*x[2]**o+(-0.422490970248)*x[2]
        arg[2,1,1,0]=(0.869799082394)*x[0]**o+(0.376436513628)*x[0]+(-0.74735679118)*x[1]**o+(0.477184335901)*x[1]+(-0.797472924916)*x[2]**o+(-0.568827873964)*x[2]
        arg[2,1,1,1]=(0.993441009779)*x[0]**o+(-0.264888299493)*x[0]+(0.861197560367)*x[1]**o+(-0.769058084144)*x[1]+(-0.86221947415)*x[2]**o+(-0.617024169672)*x[2]
        arg[2,1,2,0]=(-0.89303691749)*x[0]**o+(-0.390287412175)*x[0]+(-0.312883084291)*x[1]**o+(-0.432368502641)*x[1]+(-0.121476666357)*x[2]**o+(0.416361314288)*x[2]
        arg[2,1,2,1]=(-0.622214426504)*x[0]**o+(-0.595279134338)*x[0]+(0.000927173498715)*x[1]**o+(0.783001179117)*x[1]+(-0.956003738061)*x[2]**o+(0.482321214492)*x[2]
        arg[2,2,0,0]=(-0.0047994639068)*x[0]**o+(0.478805093492)*x[0]+(0.747825846519)*x[1]**o+(0.0756064430807)*x[1]+(-0.470534826525)*x[2]**o+(-0.421524571831)*x[2]
        arg[2,2,0,1]=(-0.0569198078126)*x[0]**o+(-0.36284856463)*x[0]+(-0.374343856016)*x[1]**o+(0.193470734451)*x[1]+(0.425350990791)*x[2]**o+(-0.903309757487)*x[2]
        arg[2,2,1,0]=(0.602075251229)*x[0]**o+(-0.558163518701)*x[0]+(-0.00146699467501)*x[1]**o+(0.437116756605)*x[1]+(-0.252696658318)*x[2]**o+(0.154017654456)*x[2]
        arg[2,2,1,1]=(0.226970288166)*x[0]**o+(0.844076458869)*x[0]+(0.231874137232)*x[1]**o+(-0.270233945335)*x[1]+(0.365979082276)*x[2]**o+(-0.937144802776)*x[2]
        arg[2,2,2,0]=(0.662440813722)*x[0]**o+(-0.600830429698)*x[0]+(0.248436486724)*x[1]**o+(0.73439963812)*x[1]+(-0.406317484259)*x[2]**o+(-0.406641354653)*x[2]
        arg[2,2,2,1]=(0.948630940404)*x[0]**o+(-0.695098513788)*x[0]+(-0.819843883615)*x[1]**o+(0.0479609386435)*x[1]+(-0.968033045837)*x[2]**o+(0.199490071238)*x[2]
        arg[2,3,0,0]=(0.967996364186)*x[0]**o+(0.562562580668)*x[0]+(0.660658005197)*x[1]**o+(-0.00126241998808)*x[1]+(-0.0188273819622)*x[2]**o+(-0.798370566647)*x[2]
        arg[2,3,0,1]=(0.108137383906)*x[0]**o+(0.230815090379)*x[0]+(-0.0817434914408)*x[1]**o+(0.256977068172)*x[1]+(0.323454379512)*x[2]**o+(0.0382049771532)*x[2]
        arg[2,3,1,0]=(-0.279443973124)*x[0]**o+(-0.948014830608)*x[0]+(-0.237787814838)*x[1]**o+(-0.880632864876)*x[1]+(0.305230465688)*x[2]**o+(0.722139326346)*x[2]
        arg[2,3,1,1]=(-0.192752669063)*x[0]**o+(0.763575125553)*x[0]+(-0.317096602795)*x[1]**o+(-0.241479822867)*x[1]+(0.643154065804)*x[2]**o+(0.903250562165)*x[2]
        arg[2,3,2,0]=(0.58153852366)*x[0]**o+(-0.131284078682)*x[0]+(0.803022288461)*x[1]**o+(0.862074208938)*x[1]+(0.934648335783)*x[2]**o+(0.720337200812)*x[2]
        arg[2,3,2,1]=(-0.866755142782)*x[0]**o+(0.299995782189)*x[0]+(0.830022439718)*x[1]**o+(0.787775630269)*x[1]+(-0.436980900353)*x[2]**o+(0.436719598692)*x[2]
        arg[2,4,0,0]=(-0.416946111972)*x[0]**o+(-0.342692819134)*x[0]+(-0.265712927184)*x[1]**o+(-0.503154497528)*x[1]+(0.436447837569)*x[2]**o+(-0.514226068117)*x[2]
        arg[2,4,0,1]=(-0.915995315837)*x[0]**o+(-0.562870475868)*x[0]+(0.158881525561)*x[1]**o+(-0.458461594412)*x[1]+(-0.117927681987)*x[2]**o+(-0.697751714189)*x[2]
        arg[2,4,1,0]=(-0.961291698815)*x[0]**o+(0.457487466485)*x[0]+(0.170987701633)*x[1]**o+(-0.14226156627)*x[1]+(-0.663462703057)*x[2]**o+(-0.0307934454283)*x[2]
        arg[2,4,1,1]=(0.588162542138)*x[0]**o+(-0.761783611043)*x[0]+(0.510241246326)*x[1]**o+(0.805053446677)*x[1]+(-0.851391511147)*x[2]**o+(-0.0976037736598)*x[2]
        arg[2,4,2,0]=(0.649047985858)*x[0]**o+(0.677435498282)*x[0]+(-0.620777977748)*x[1]**o+(-0.209440794016)*x[1]+(0.58009322661)*x[2]**o+(0.56317180284)*x[2]
        arg[2,4,2,1]=(-0.975596464331)*x[0]**o+(0.759364271658)*x[0]+(0.43142780149)*x[1]**o+(-0.701043203786)*x[1]+(-0.648159388134)*x[2]**o+(-0.129618261144)*x[2]
        arg[3,0,0,0]=(-0.0207100058532)*x[0]**o+(-0.829637327952)*x[0]+(-0.0326040806041)*x[1]**o+(0.652320712227)*x[1]+(-0.509435869246)*x[2]**o+(-0.66044392521)*x[2]
        arg[3,0,0,1]=(-0.312677151974)*x[0]**o+(0.0204494340128)*x[0]+(-0.989457740146)*x[1]**o+(-0.0606077687298)*x[1]+(-0.527267756606)*x[2]**o+(0.496006274201)*x[2]
        arg[3,0,1,0]=(0.842871894549)*x[0]**o+(-0.577687338895)*x[0]+(0.414019506131)*x[1]**o+(0.0410521331015)*x[1]+(-0.830482660184)*x[2]**o+(0.688692187619)*x[2]
        arg[3,0,1,1]=(-0.920070939376)*x[0]**o+(0.632972952755)*x[0]+(0.944186412149)*x[1]**o+(0.620319712728)*x[1]+(-0.171433727161)*x[2]**o+(0.267125744191)*x[2]
        arg[3,0,2,0]=(-0.0669535338492)*x[0]**o+(0.249181230746)*x[0]+(-0.147448553328)*x[1]**o+(-0.459701695839)*x[1]+(0.378458172384)*x[2]**o+(0.916594147889)*x[2]
        arg[3,0,2,1]=(0.0720111269432)*x[0]**o+(-0.390032127319)*x[0]+(-0.115565740734)*x[1]**o+(0.501210832815)*x[1]+(0.568129023658)*x[2]**o+(-0.303909080042)*x[2]
        arg[3,1,0,0]=(0.205675480731)*x[0]**o+(-0.843093328795)*x[0]+(-0.812415573635)*x[1]**o+(-0.706806177394)*x[1]+(-0.158442718286)*x[2]**o+(0.15187176613)*x[2]
        arg[3,1,0,1]=(0.149669308017)*x[0]**o+(0.487445503149)*x[0]+(-0.27973905833)*x[1]**o+(-0.504992462225)*x[1]+(-0.728166691522)*x[2]**o+(0.734846267391)*x[2]
        arg[3,1,1,0]=(0.0410578015251)*x[0]**o+(-0.525001495676)*x[0]+(-0.512685697472)*x[1]**o+(-0.284314686987)*x[1]+(0.888112049081)*x[2]**o+(0.425466793802)*x[2]
        arg[3,1,1,1]=(-0.374237047043)*x[0]**o+(-0.00078724690461)*x[0]+(0.849952487348)*x[1]**o+(-0.654945893799)*x[1]+(-0.536739132177)*x[2]**o+(0.20919294066)*x[2]
        arg[3,1,2,0]=(0.685749084068)*x[0]**o+(-0.730378478543)*x[0]+(-0.918072746761)*x[1]**o+(-0.706713872062)*x[1]+(-0.486100957526)*x[2]**o+(0.95736939257)*x[2]
        arg[3,1,2,1]=(-0.287859736321)*x[0]**o+(-0.271882215207)*x[0]+(-0.673357024623)*x[1]**o+(0.702867457627)*x[1]+(0.78445294416)*x[2]**o+(0.613888365259)*x[2]
        arg[3,2,0,0]=(0.568506008135)*x[0]**o+(-0.825947283734)*x[0]+(0.687369924271)*x[1]**o+(-0.866597782569)*x[1]+(-0.332810902355)*x[2]**o+(0.499502049139)*x[2]
        arg[3,2,0,1]=(0.757251773732)*x[0]**o+(0.293337812465)*x[0]+(0.686709285382)*x[1]**o+(-0.084997189402)*x[1]+(-0.338903817381)*x[2]**o+(-0.787646799698)*x[2]
        arg[3,2,1,0]=(-0.580327419983)*x[0]**o+(-0.603952498996)*x[0]+(0.523057988618)*x[1]**o+(-0.0835187394107)*x[1]+(0.459827887147)*x[2]**o+(0.689021201042)*x[2]
        arg[3,2,1,1]=(-0.41923416619)*x[0]**o+(0.0406426957237)*x[0]+(-0.908041008186)*x[1]**o+(0.382097138462)*x[1]+(0.66835706849)*x[2]**o+(0.355849275378)*x[2]
        arg[3,2,2,0]=(-0.133756767108)*x[0]**o+(-0.233743496333)*x[0]+(-0.661420120856)*x[1]**o+(0.788722576584)*x[1]+(-0.131764585707)*x[2]**o+(-0.781004479871)*x[2]
        arg[3,2,2,1]=(0.0308645139784)*x[0]**o+(-0.194986246344)*x[0]+(-0.267418187303)*x[1]**o+(0.332143733168)*x[1]+(0.513266431894)*x[2]**o+(0.594016981473)*x[2]
        arg[3,3,0,0]=(0.644314102613)*x[0]**o+(-0.977355279476)*x[0]+(0.516452323667)*x[1]**o+(0.172873579919)*x[1]+(0.778904819969)*x[2]**o+(0.559689750425)*x[2]
        arg[3,3,0,1]=(0.736135986201)*x[0]**o+(0.366918250826)*x[0]+(-0.0547007988818)*x[1]**o+(-0.830174436433)*x[1]+(0.613440124338)*x[2]**o+(0.574992469591)*x[2]
        arg[3,3,1,0]=(0.324504938146)*x[0]**o+(-0.915240176689)*x[0]+(-0.398105336249)*x[1]**o+(-0.526498343533)*x[1]+(0.43264573336)*x[2]**o+(0.393008086529)*x[2]
        arg[3,3,1,1]=(0.926367438323)*x[0]**o+(0.526364305462)*x[0]+(0.15596582236)*x[1]**o+(0.165158439711)*x[1]+(-0.508505348973)*x[2]**o+(-0.603637833992)*x[2]
        arg[3,3,2,0]=(0.661333046004)*x[0]**o+(0.425320373512)*x[0]+(-0.810809051016)*x[1]**o+(0.20446843023)*x[1]+(0.685136412413)*x[2]**o+(-0.0281127019894)*x[2]
        arg[3,3,2,1]=(0.851742499007)*x[0]**o+(-0.405506974975)*x[0]+(-0.296768983731)*x[1]**o+(0.389557354929)*x[1]+(0.335108341463)*x[2]**o+(0.0313094433351)*x[2]
        arg[3,4,0,0]=(0.0415035498649)*x[0]**o+(0.827106496882)*x[0]+(-0.481831487377)*x[1]**o+(-0.14935685817)*x[1]+(0.354765095566)*x[2]**o+(0.521898910137)*x[2]
        arg[3,4,0,1]=(-0.36328099035)*x[0]**o+(-0.243342386595)*x[0]+(0.0475935188475)*x[1]**o+(0.845308417697)*x[1]+(0.315259620082)*x[2]**o+(0.541313742759)*x[2]
        arg[3,4,1,0]=(-0.66364149672)*x[0]**o+(0.987505978353)*x[0]+(-0.337659775957)*x[1]**o+(0.197277151989)*x[1]+(-0.431260870605)*x[2]**o+(-0.803375909471)*x[2]
        arg[3,4,1,1]=(0.722104365428)*x[0]**o+(0.168416682879)*x[0]+(-0.415740544748)*x[1]**o+(-0.437836696476)*x[1]+(-0.280581752598)*x[2]**o+(0.434326371762)*x[2]
        arg[3,4,2,0]=(0.673144889684)*x[0]**o+(-0.0628529591764)*x[0]+(0.0161458704408)*x[1]**o+(-0.439966404278)*x[1]+(-0.963402070447)*x[2]**o+(0.308312287021)*x[2]
        arg[3,4,2,1]=(0.649131384887)*x[0]**o+(-0.825086043001)*x[0]+(0.204615530143)*x[1]**o+(0.00801105023644)*x[1]+(-0.234754758061)*x[2]**o+(0.659650083864)*x[2]
        ref[0,0,0,0]=(-0.928996629952)*x_ref[0]**o+(-0.171383074114)*x_ref[0]+(-0.324423798901)*x_ref[1]**o+(-0.801354053962)*x_ref[1]+(0.365256142808)*x_ref[2]**o+(0.126262208857)*x_ref[2]
        ref[0,0,0,1]=(0.326024843626)*x_ref[0]**o+(-0.0091566240426)*x_ref[0]+(-0.0396235158589)*x_ref[1]**o+(-0.753302577837)*x_ref[1]+(-0.38709916587)*x_ref[2]**o+(-0.28660316595)*x_ref[2]
        ref[0,0,1,0]=(0.755338255506)*x_ref[0]**o+(0.851895434343)*x_ref[0]+(0.419119464402)*x_ref[1]**o+(0.670733085023)*x_ref[1]+(0.819398310958)*x_ref[2]**o+(0.476185335773)*x_ref[2]
        ref[0,0,1,1]=(-0.981416259131)*x_ref[0]**o+(-0.97000252267)*x_ref[0]+(-0.132624451699)*x_ref[1]**o+(-0.261428370649)*x_ref[1]+(-0.349245951727)*x_ref[2]**o+(-0.717343195877)*x_ref[2]
        ref[0,0,2,0]=(-0.874826075266)*x_ref[0]**o+(-0.399307905707)*x_ref[0]+(0.350873599317)*x_ref[1]**o+(0.15861651992)*x_ref[1]+(0.396279933216)*x_ref[2]**o+(0.92332515708)*x_ref[2]
        ref[0,0,2,1]=(-0.183596683527)*x_ref[0]**o+(-0.115194639348)*x_ref[0]+(0.616111723618)*x_ref[1]**o+(-0.402157657669)*x_ref[1]+(0.44145594193)*x_ref[2]**o+(0.686052407784)*x_ref[2]
        ref[0,1,0,0]=(-0.669169178244)*x_ref[0]**o+(0.835937801609)*x_ref[0]+(0.468805302181)*x_ref[1]**o+(-0.627292634023)*x_ref[1]+(-0.674361231979)*x_ref[2]**o+(-0.00929181785129)*x_ref[2]
        ref[0,1,0,1]=(-0.568131655705)*x_ref[0]**o+(0.690366535852)*x_ref[0]+(-0.212524396914)*x_ref[1]**o+(-0.767983769407)*x_ref[1]+(0.477500128569)*x_ref[2]**o+(-0.0190887598644)*x_ref[2]
        ref[0,1,1,0]=(-0.374170635822)*x_ref[0]**o+(0.0169009974558)*x_ref[0]+(-0.162519103357)*x_ref[1]**o+(0.786991144449)*x_ref[1]+(0.0590294918131)*x_ref[2]**o+(-0.213442509309)*x_ref[2]
        ref[0,1,1,1]=(-0.558268657439)*x_ref[0]**o+(0.584734933669)*x_ref[0]+(-0.650790013689)*x_ref[1]**o+(-0.192849048407)*x_ref[1]+(0.505458636714)*x_ref[2]**o+(-0.585174281371)*x_ref[2]
        ref[0,1,2,0]=(-0.933243590775)*x_ref[0]**o+(0.528060249767)*x_ref[0]+(-0.763255746252)*x_ref[1]**o+(0.771589118163)*x_ref[1]+(-0.641287604338)*x_ref[2]**o+(0.379566401585)*x_ref[2]
        ref[0,1,2,1]=(-0.836976373935)*x_ref[0]**o+(-0.969434110974)*x_ref[0]+(0.758148372888)*x_ref[1]**o+(-0.89212062535)*x_ref[1]+(0.788693820015)*x_ref[2]**o+(-0.620704367414)*x_ref[2]
        ref[0,2,0,0]=(0.0163117360814)*x_ref[0]**o+(-0.390867752068)*x_ref[0]+(-0.725068297982)*x_ref[1]**o+(-0.680312496646)*x_ref[1]+(0.069273494969)*x_ref[2]**o+(-0.576122275696)*x_ref[2]
        ref[0,2,0,1]=(0.461771197614)*x_ref[0]**o+(-0.767541404109)*x_ref[0]+(-0.694279263622)*x_ref[1]**o+(-0.721448812749)*x_ref[1]+(0.248787896584)*x_ref[2]**o+(0.237075401327)*x_ref[2]
        ref[0,2,1,0]=(0.381987012965)*x_ref[0]**o+(-0.288236233238)*x_ref[0]+(-0.519012025084)*x_ref[1]**o+(-0.256658293082)*x_ref[1]+(0.807322167368)*x_ref[2]**o+(-0.261916179859)*x_ref[2]
        ref[0,2,1,1]=(0.257241267407)*x_ref[0]**o+(-0.470612990721)*x_ref[0]+(-0.208612656448)*x_ref[1]**o+(0.839646962914)*x_ref[1]+(0.916675150494)*x_ref[2]**o+(-0.165468351005)*x_ref[2]
        ref[0,2,2,0]=(0.92569232521)*x_ref[0]**o+(0.809982458308)*x_ref[0]+(-0.860530305336)*x_ref[1]**o+(0.261687622817)*x_ref[1]+(0.948003684651)*x_ref[2]**o+(-0.473133742361)*x_ref[2]
        ref[0,2,2,1]=(-0.113846814874)*x_ref[0]**o+(0.680846459377)*x_ref[0]+(0.920219149741)*x_ref[1]**o+(0.172027554503)*x_ref[1]+(0.475330041115)*x_ref[2]**o+(-0.176218891429)*x_ref[2]
        ref[0,3,0,0]=(-0.948638294297)*x_ref[0]**o+(-0.0411621070703)*x_ref[0]+(-0.294759271027)*x_ref[1]**o+(-0.903733455086)*x_ref[1]+(-0.995253733593)*x_ref[2]**o+(0.106726960462)*x_ref[2]
        ref[0,3,0,1]=(0.1592709614)*x_ref[0]**o+(0.0683266445293)*x_ref[0]+(0.336092506745)*x_ref[1]**o+(-0.209047739881)*x_ref[1]+(-0.330417243459)*x_ref[2]**o+(-0.809242647755)*x_ref[2]
        ref[0,3,1,0]=(-0.680637516359)*x_ref[0]**o+(-0.713405815404)*x_ref[0]+(-0.484624435043)*x_ref[1]**o+(-0.279314462332)*x_ref[1]+(-0.458856066504)*x_ref[2]**o+(0.0588111305227)*x_ref[2]
        ref[0,3,1,1]=(-0.853676338655)*x_ref[0]**o+(-0.651539814195)*x_ref[0]+(-0.109727277725)*x_ref[1]**o+(0.0139006407942)*x_ref[1]+(0.231943245426)*x_ref[2]**o+(0.42972517233)*x_ref[2]
        ref[0,3,2,0]=(-0.60799269765)*x_ref[0]**o+(-0.802413269128)*x_ref[0]+(0.151362498863)*x_ref[1]**o+(-0.678226252875)*x_ref[1]+(-0.867117685659)*x_ref[2]**o+(-0.711982116071)*x_ref[2]
        ref[0,3,2,1]=(-0.243012743941)*x_ref[0]**o+(-0.692614579016)*x_ref[0]+(0.0986817641914)*x_ref[1]**o+(-0.495735862695)*x_ref[1]+(-0.430040681653)*x_ref[2]**o+(0.0108928722284)*x_ref[2]
        ref[0,4,0,0]=(0.0989402181558)*x_ref[0]**o+(-0.60096719882)*x_ref[0]+(-0.324253928432)*x_ref[1]**o+(-0.862723687559)*x_ref[1]+(-0.34745416427)*x_ref[2]**o+(0.936559074873)*x_ref[2]
        ref[0,4,0,1]=(0.711021967566)*x_ref[0]**o+(-0.939408401113)*x_ref[0]+(-0.218100921633)*x_ref[1]**o+(0.92835194875)*x_ref[1]+(-0.353936913865)*x_ref[2]**o+(-0.547266090928)*x_ref[2]
        ref[0,4,1,0]=(0.172231614188)*x_ref[0]**o+(-0.734622090671)*x_ref[0]+(0.432686916647)*x_ref[1]**o+(0.552595350492)*x_ref[1]+(0.779662641076)*x_ref[2]**o+(-0.0933220492779)*x_ref[2]
        ref[0,4,1,1]=(0.996776716866)*x_ref[0]**o+(-0.340786520686)*x_ref[0]+(0.260075214171)*x_ref[1]**o+(0.00385504852496)*x_ref[1]+(0.377516772127)*x_ref[2]**o+(-0.341751634181)*x_ref[2]
        ref[0,4,2,0]=(-0.655642896664)*x_ref[0]**o+(-0.840525173737)*x_ref[0]+(-0.855343721564)*x_ref[1]**o+(0.253558571524)*x_ref[1]+(0.0463733059925)*x_ref[2]**o+(-0.116359866912)*x_ref[2]
        ref[0,4,2,1]=(0.131619661414)*x_ref[0]**o+(-0.162067255954)*x_ref[0]+(0.562536831367)*x_ref[1]**o+(-0.940412472035)*x_ref[1]+(-0.282265762783)*x_ref[2]**o+(0.634711607923)*x_ref[2]
        ref[1,0,0,0]=(0.484071808097)*x_ref[0]**o+(0.566763524691)*x_ref[0]+(-0.658614173653)*x_ref[1]**o+(-0.140192232895)*x_ref[1]+(-0.310114236409)*x_ref[2]**o+(-0.918532401595)*x_ref[2]
        ref[1,0,0,1]=(-0.81190380441)*x_ref[0]**o+(0.644439878497)*x_ref[0]+(0.479583403289)*x_ref[1]**o+(-0.493318882779)*x_ref[1]+(-0.424704492175)*x_ref[2]**o+(0.700624317732)*x_ref[2]
        ref[1,0,1,0]=(0.766650182435)*x_ref[0]**o+(0.226365545129)*x_ref[0]+(0.510828945971)*x_ref[1]**o+(0.283221954498)*x_ref[1]+(0.817303877886)*x_ref[2]**o+(-0.245709044628)*x_ref[2]
        ref[1,0,1,1]=(0.194787112664)*x_ref[0]**o+(-0.879306736922)*x_ref[0]+(0.185381291627)*x_ref[1]**o+(-0.510692743975)*x_ref[1]+(-0.422555014055)*x_ref[2]**o+(0.556629919959)*x_ref[2]
        ref[1,0,2,0]=(-0.958345828895)*x_ref[0]**o+(-0.667735551013)*x_ref[0]+(0.278271284225)*x_ref[1]**o+(-0.0847995194757)*x_ref[1]+(0.902184171441)*x_ref[2]**o+(0.763125168854)*x_ref[2]
        ref[1,0,2,1]=(-0.0766429670195)*x_ref[0]**o+(0.894421139029)*x_ref[0]+(-0.331560314101)*x_ref[1]**o+(-0.0253297623997)*x_ref[1]+(0.13406045967)*x_ref[2]**o+(-0.326681111625)*x_ref[2]
        ref[1,1,0,0]=(-0.979406855838)*x_ref[0]**o+(0.528849909402)*x_ref[0]+(-0.0486159476737)*x_ref[1]**o+(-0.898392512695)*x_ref[1]+(-0.678328327407)*x_ref[2]**o+(0.644201802057)*x_ref[2]
        ref[1,1,0,1]=(-0.977485554605)*x_ref[0]**o+(0.794799941538)*x_ref[0]+(-0.00237183848504)*x_ref[1]**o+(-0.633497483149)*x_ref[1]+(0.746593853568)*x_ref[2]**o+(0.197741126365)*x_ref[2]
        ref[1,1,1,0]=(0.379985506182)*x_ref[0]**o+(0.69889144234)*x_ref[0]+(-0.507293887409)*x_ref[1]**o+(-0.727675545291)*x_ref[1]+(-0.467064531693)*x_ref[2]**o+(0.130273930089)*x_ref[2]
        ref[1,1,1,1]=(-0.330627830819)*x_ref[0]**o+(-0.0311542935889)*x_ref[0]+(-0.0671494148749)*x_ref[1]**o+(0.583605469163)*x_ref[1]+(-0.172083996069)*x_ref[2]**o+(0.666718011074)*x_ref[2]
        ref[1,1,2,0]=(0.539618381063)*x_ref[0]**o+(-0.546796237888)*x_ref[0]+(0.857911750392)*x_ref[1]**o+(0.119552731463)*x_ref[1]+(0.329286820109)*x_ref[2]**o+(0.637240649087)*x_ref[2]
        ref[1,1,2,1]=(-0.828603071334)*x_ref[0]**o+(0.861799149065)*x_ref[0]+(0.615224040538)*x_ref[1]**o+(-0.0868985543263)*x_ref[1]+(-0.206168997809)*x_ref[2]**o+(0.311103719286)*x_ref[2]
        ref[1,2,0,0]=(0.695106867715)*x_ref[0]**o+(-0.533303418727)*x_ref[0]+(-0.393936955207)*x_ref[1]**o+(-0.407635261686)*x_ref[1]+(0.604346269147)*x_ref[2]**o+(-0.10213061533)*x_ref[2]
        ref[1,2,0,1]=(0.338035309929)*x_ref[0]**o+(0.517135412671)*x_ref[0]+(0.277863735829)*x_ref[1]**o+(-0.703417079398)*x_ref[1]+(0.369398635984)*x_ref[2]**o+(-0.952905070601)*x_ref[2]
        ref[1,2,1,0]=(0.937866603297)*x_ref[0]**o+(0.436121631708)*x_ref[0]+(0.568362098461)*x_ref[1]**o+(0.223837722984)*x_ref[1]+(-0.236896910941)*x_ref[2]**o+(0.316114408308)*x_ref[2]
        ref[1,2,1,1]=(0.285714491932)*x_ref[0]**o+(0.633321537987)*x_ref[0]+(0.953737237202)*x_ref[1]**o+(0.732065555924)*x_ref[1]+(-0.515759068197)*x_ref[2]**o+(-0.519775334889)*x_ref[2]
        ref[1,2,2,0]=(-0.0610222303479)*x_ref[0]**o+(0.135974983944)*x_ref[0]+(0.450213393447)*x_ref[1]**o+(-0.44747541715)*x_ref[1]+(-0.640660207187)*x_ref[2]**o+(0.348457261119)*x_ref[2]
        ref[1,2,2,1]=(0.131753033369)*x_ref[0]**o+(-0.311359549998)*x_ref[0]+(0.445707767235)*x_ref[1]**o+(-0.710795569938)*x_ref[1]+(-0.746038648477)*x_ref[2]**o+(-0.210414773188)*x_ref[2]
        ref[1,3,0,0]=(-0.323684061289)*x_ref[0]**o+(-0.0937091220282)*x_ref[0]+(-0.827884898323)*x_ref[1]**o+(0.604442907568)*x_ref[1]+(0.620923596379)*x_ref[2]**o+(0.851758170763)*x_ref[2]
        ref[1,3,0,1]=(-0.269239872573)*x_ref[0]**o+(-0.603248280988)*x_ref[0]+(0.603804530326)*x_ref[1]**o+(-0.331910095046)*x_ref[1]+(-0.653877236308)*x_ref[2]**o+(0.115434865132)*x_ref[2]
        ref[1,3,1,0]=(0.0531798412869)*x_ref[0]**o+(0.292950346899)*x_ref[0]+(-0.634376081823)*x_ref[1]**o+(-0.775427771578)*x_ref[1]+(-0.987113467149)*x_ref[2]**o+(0.757980444447)*x_ref[2]
        ref[1,3,1,1]=(0.757807292214)*x_ref[0]**o+(0.81778478971)*x_ref[0]+(-0.619873148405)*x_ref[1]**o+(0.0129855659138)*x_ref[1]+(-0.282861093681)*x_ref[2]**o+(0.667211104045)*x_ref[2]
        ref[1,3,2,0]=(-0.915205606853)*x_ref[0]**o+(0.918704643459)*x_ref[0]+(0.143238155504)*x_ref[1]**o+(0.0285686377214)*x_ref[1]+(-0.848712843243)*x_ref[2]**o+(0.058647124862)*x_ref[2]
        ref[1,3,2,1]=(0.441487712831)*x_ref[0]**o+(-0.626878272507)*x_ref[0]+(-0.862579662295)*x_ref[1]**o+(0.204629243832)*x_ref[1]+(0.964143347265)*x_ref[2]**o+(-0.399125844282)*x_ref[2]
        ref[1,4,0,0]=(-0.349791275126)*x_ref[0]**o+(-0.140293844776)*x_ref[0]+(-0.693879671138)*x_ref[1]**o+(0.429290435637)*x_ref[1]+(-0.892670314048)*x_ref[2]**o+(-0.380777380409)*x_ref[2]
        ref[1,4,0,1]=(0.528712424647)*x_ref[0]**o+(0.449268002671)*x_ref[0]+(-0.202472214961)*x_ref[1]**o+(-0.374762208851)*x_ref[1]+(-0.625590920648)*x_ref[2]**o+(-0.000776205555531)*x_ref[2]
        ref[1,4,1,0]=(0.00425881397184)*x_ref[0]**o+(0.265831686248)*x_ref[0]+(0.757626547866)*x_ref[1]**o+(-0.934207064756)*x_ref[1]+(0.541356932538)*x_ref[2]**o+(0.217841597089)*x_ref[2]
        ref[1,4,1,1]=(-0.02422207106)*x_ref[0]**o+(-0.807018149115)*x_ref[0]+(-0.402347046418)*x_ref[1]**o+(-0.0548851919048)*x_ref[1]+(0.850202044077)*x_ref[2]**o+(-0.889576261979)*x_ref[2]
        ref[1,4,2,0]=(0.487059068561)*x_ref[0]**o+(0.338254525003)*x_ref[0]+(-0.387019663154)*x_ref[1]**o+(-0.436969552236)*x_ref[1]+(-0.747690766603)*x_ref[2]**o+(0.265854907003)*x_ref[2]
        ref[1,4,2,1]=(0.687562221132)*x_ref[0]**o+(0.4755782976)*x_ref[0]+(-0.456287111804)*x_ref[1]**o+(0.412138123928)*x_ref[1]+(0.637920334647)*x_ref[2]**o+(0.817007725525)*x_ref[2]
        ref[2,0,0,0]=(-0.728166422776)*x_ref[0]**o+(0.108904177195)*x_ref[0]+(0.383848108635)*x_ref[1]**o+(-0.19252707292)*x_ref[1]+(-0.279319122495)*x_ref[2]**o+(-0.0530727745585)*x_ref[2]
        ref[2,0,0,1]=(0.380827452272)*x_ref[0]**o+(-0.133776280687)*x_ref[0]+(-0.174731551283)*x_ref[1]**o+(-0.369231086652)*x_ref[1]+(-0.0190771818046)*x_ref[2]**o+(-0.331873068919)*x_ref[2]
        ref[2,0,1,0]=(0.653644298826)*x_ref[0]**o+(-0.916110182689)*x_ref[0]+(0.0510855252492)*x_ref[1]**o+(-0.934192949208)*x_ref[1]+(0.572048462784)*x_ref[2]**o+(-0.451401856863)*x_ref[2]
        ref[2,0,1,1]=(-0.394958497502)*x_ref[0]**o+(-0.690752972227)*x_ref[0]+(-0.436352069349)*x_ref[1]**o+(0.933524576573)*x_ref[1]+(-0.524344161699)*x_ref[2]**o+(0.526748792776)*x_ref[2]
        ref[2,0,2,0]=(0.845649523782)*x_ref[0]**o+(0.63927747558)*x_ref[0]+(-0.534192218216)*x_ref[1]**o+(0.714500925459)*x_ref[1]+(0.156077814898)*x_ref[2]**o+(-0.193712778792)*x_ref[2]
        ref[2,0,2,1]=(0.809636640413)*x_ref[0]**o+(-0.930654037481)*x_ref[0]+(0.509441692942)*x_ref[1]**o+(0.881563824585)*x_ref[1]+(0.101333749057)*x_ref[2]**o+(0.418791722704)*x_ref[2]
        ref[2,1,0,0]=(0.289481281745)*x_ref[0]**o+(-0.98279193802)*x_ref[0]+(-0.443382747319)*x_ref[1]**o+(-0.281710992472)*x_ref[1]+(0.426349350391)*x_ref[2]**o+(-0.870192121251)*x_ref[2]
        ref[2,1,0,1]=(0.837046002753)*x_ref[0]**o+(-0.0254166983371)*x_ref[0]+(0.30178635814)*x_ref[1]**o+(0.973915444924)*x_ref[1]+(0.298631524734)*x_ref[2]**o+(-0.422490970248)*x_ref[2]
        ref[2,1,1,0]=(0.869799082394)*x_ref[0]**o+(0.376436513628)*x_ref[0]+(-0.74735679118)*x_ref[1]**o+(0.477184335901)*x_ref[1]+(-0.797472924916)*x_ref[2]**o+(-0.568827873964)*x_ref[2]
        ref[2,1,1,1]=(0.993441009779)*x_ref[0]**o+(-0.264888299493)*x_ref[0]+(0.861197560367)*x_ref[1]**o+(-0.769058084144)*x_ref[1]+(-0.86221947415)*x_ref[2]**o+(-0.617024169672)*x_ref[2]
        ref[2,1,2,0]=(-0.89303691749)*x_ref[0]**o+(-0.390287412175)*x_ref[0]+(-0.312883084291)*x_ref[1]**o+(-0.432368502641)*x_ref[1]+(-0.121476666357)*x_ref[2]**o+(0.416361314288)*x_ref[2]
        ref[2,1,2,1]=(-0.622214426504)*x_ref[0]**o+(-0.595279134338)*x_ref[0]+(0.000927173498715)*x_ref[1]**o+(0.783001179117)*x_ref[1]+(-0.956003738061)*x_ref[2]**o+(0.482321214492)*x_ref[2]
        ref[2,2,0,0]=(-0.0047994639068)*x_ref[0]**o+(0.478805093492)*x_ref[0]+(0.747825846519)*x_ref[1]**o+(0.0756064430807)*x_ref[1]+(-0.470534826525)*x_ref[2]**o+(-0.421524571831)*x_ref[2]
        ref[2,2,0,1]=(-0.0569198078126)*x_ref[0]**o+(-0.36284856463)*x_ref[0]+(-0.374343856016)*x_ref[1]**o+(0.193470734451)*x_ref[1]+(0.425350990791)*x_ref[2]**o+(-0.903309757487)*x_ref[2]
        ref[2,2,1,0]=(0.602075251229)*x_ref[0]**o+(-0.558163518701)*x_ref[0]+(-0.00146699467501)*x_ref[1]**o+(0.437116756605)*x_ref[1]+(-0.252696658318)*x_ref[2]**o+(0.154017654456)*x_ref[2]
        ref[2,2,1,1]=(0.226970288166)*x_ref[0]**o+(0.844076458869)*x_ref[0]+(0.231874137232)*x_ref[1]**o+(-0.270233945335)*x_ref[1]+(0.365979082276)*x_ref[2]**o+(-0.937144802776)*x_ref[2]
        ref[2,2,2,0]=(0.662440813722)*x_ref[0]**o+(-0.600830429698)*x_ref[0]+(0.248436486724)*x_ref[1]**o+(0.73439963812)*x_ref[1]+(-0.406317484259)*x_ref[2]**o+(-0.406641354653)*x_ref[2]
        ref[2,2,2,1]=(0.948630940404)*x_ref[0]**o+(-0.695098513788)*x_ref[0]+(-0.819843883615)*x_ref[1]**o+(0.0479609386435)*x_ref[1]+(-0.968033045837)*x_ref[2]**o+(0.199490071238)*x_ref[2]
        ref[2,3,0,0]=(0.967996364186)*x_ref[0]**o+(0.562562580668)*x_ref[0]+(0.660658005197)*x_ref[1]**o+(-0.00126241998808)*x_ref[1]+(-0.0188273819622)*x_ref[2]**o+(-0.798370566647)*x_ref[2]
        ref[2,3,0,1]=(0.108137383906)*x_ref[0]**o+(0.230815090379)*x_ref[0]+(-0.0817434914408)*x_ref[1]**o+(0.256977068172)*x_ref[1]+(0.323454379512)*x_ref[2]**o+(0.0382049771532)*x_ref[2]
        ref[2,3,1,0]=(-0.279443973124)*x_ref[0]**o+(-0.948014830608)*x_ref[0]+(-0.237787814838)*x_ref[1]**o+(-0.880632864876)*x_ref[1]+(0.305230465688)*x_ref[2]**o+(0.722139326346)*x_ref[2]
        ref[2,3,1,1]=(-0.192752669063)*x_ref[0]**o+(0.763575125553)*x_ref[0]+(-0.317096602795)*x_ref[1]**o+(-0.241479822867)*x_ref[1]+(0.643154065804)*x_ref[2]**o+(0.903250562165)*x_ref[2]
        ref[2,3,2,0]=(0.58153852366)*x_ref[0]**o+(-0.131284078682)*x_ref[0]+(0.803022288461)*x_ref[1]**o+(0.862074208938)*x_ref[1]+(0.934648335783)*x_ref[2]**o+(0.720337200812)*x_ref[2]
        ref[2,3,2,1]=(-0.866755142782)*x_ref[0]**o+(0.299995782189)*x_ref[0]+(0.830022439718)*x_ref[1]**o+(0.787775630269)*x_ref[1]+(-0.436980900353)*x_ref[2]**o+(0.436719598692)*x_ref[2]
        ref[2,4,0,0]=(-0.416946111972)*x_ref[0]**o+(-0.342692819134)*x_ref[0]+(-0.265712927184)*x_ref[1]**o+(-0.503154497528)*x_ref[1]+(0.436447837569)*x_ref[2]**o+(-0.514226068117)*x_ref[2]
        ref[2,4,0,1]=(-0.915995315837)*x_ref[0]**o+(-0.562870475868)*x_ref[0]+(0.158881525561)*x_ref[1]**o+(-0.458461594412)*x_ref[1]+(-0.117927681987)*x_ref[2]**o+(-0.697751714189)*x_ref[2]
        ref[2,4,1,0]=(-0.961291698815)*x_ref[0]**o+(0.457487466485)*x_ref[0]+(0.170987701633)*x_ref[1]**o+(-0.14226156627)*x_ref[1]+(-0.663462703057)*x_ref[2]**o+(-0.0307934454283)*x_ref[2]
        ref[2,4,1,1]=(0.588162542138)*x_ref[0]**o+(-0.761783611043)*x_ref[0]+(0.510241246326)*x_ref[1]**o+(0.805053446677)*x_ref[1]+(-0.851391511147)*x_ref[2]**o+(-0.0976037736598)*x_ref[2]
        ref[2,4,2,0]=(0.649047985858)*x_ref[0]**o+(0.677435498282)*x_ref[0]+(-0.620777977748)*x_ref[1]**o+(-0.209440794016)*x_ref[1]+(0.58009322661)*x_ref[2]**o+(0.56317180284)*x_ref[2]
        ref[2,4,2,1]=(-0.975596464331)*x_ref[0]**o+(0.759364271658)*x_ref[0]+(0.43142780149)*x_ref[1]**o+(-0.701043203786)*x_ref[1]+(-0.648159388134)*x_ref[2]**o+(-0.129618261144)*x_ref[2]
        ref[3,0,0,0]=(-0.0207100058532)*x_ref[0]**o+(-0.829637327952)*x_ref[0]+(-0.0326040806041)*x_ref[1]**o+(0.652320712227)*x_ref[1]+(-0.509435869246)*x_ref[2]**o+(-0.66044392521)*x_ref[2]
        ref[3,0,0,1]=(-0.312677151974)*x_ref[0]**o+(0.0204494340128)*x_ref[0]+(-0.989457740146)*x_ref[1]**o+(-0.0606077687298)*x_ref[1]+(-0.527267756606)*x_ref[2]**o+(0.496006274201)*x_ref[2]
        ref[3,0,1,0]=(0.842871894549)*x_ref[0]**o+(-0.577687338895)*x_ref[0]+(0.414019506131)*x_ref[1]**o+(0.0410521331015)*x_ref[1]+(-0.830482660184)*x_ref[2]**o+(0.688692187619)*x_ref[2]
        ref[3,0,1,1]=(-0.920070939376)*x_ref[0]**o+(0.632972952755)*x_ref[0]+(0.944186412149)*x_ref[1]**o+(0.620319712728)*x_ref[1]+(-0.171433727161)*x_ref[2]**o+(0.267125744191)*x_ref[2]
        ref[3,0,2,0]=(-0.0669535338492)*x_ref[0]**o+(0.249181230746)*x_ref[0]+(-0.147448553328)*x_ref[1]**o+(-0.459701695839)*x_ref[1]+(0.378458172384)*x_ref[2]**o+(0.916594147889)*x_ref[2]
        ref[3,0,2,1]=(0.0720111269432)*x_ref[0]**o+(-0.390032127319)*x_ref[0]+(-0.115565740734)*x_ref[1]**o+(0.501210832815)*x_ref[1]+(0.568129023658)*x_ref[2]**o+(-0.303909080042)*x_ref[2]
        ref[3,1,0,0]=(0.205675480731)*x_ref[0]**o+(-0.843093328795)*x_ref[0]+(-0.812415573635)*x_ref[1]**o+(-0.706806177394)*x_ref[1]+(-0.158442718286)*x_ref[2]**o+(0.15187176613)*x_ref[2]
        ref[3,1,0,1]=(0.149669308017)*x_ref[0]**o+(0.487445503149)*x_ref[0]+(-0.27973905833)*x_ref[1]**o+(-0.504992462225)*x_ref[1]+(-0.728166691522)*x_ref[2]**o+(0.734846267391)*x_ref[2]
        ref[3,1,1,0]=(0.0410578015251)*x_ref[0]**o+(-0.525001495676)*x_ref[0]+(-0.512685697472)*x_ref[1]**o+(-0.284314686987)*x_ref[1]+(0.888112049081)*x_ref[2]**o+(0.425466793802)*x_ref[2]
        ref[3,1,1,1]=(-0.374237047043)*x_ref[0]**o+(-0.00078724690461)*x_ref[0]+(0.849952487348)*x_ref[1]**o+(-0.654945893799)*x_ref[1]+(-0.536739132177)*x_ref[2]**o+(0.20919294066)*x_ref[2]
        ref[3,1,2,0]=(0.685749084068)*x_ref[0]**o+(-0.730378478543)*x_ref[0]+(-0.918072746761)*x_ref[1]**o+(-0.706713872062)*x_ref[1]+(-0.486100957526)*x_ref[2]**o+(0.95736939257)*x_ref[2]
        ref[3,1,2,1]=(-0.287859736321)*x_ref[0]**o+(-0.271882215207)*x_ref[0]+(-0.673357024623)*x_ref[1]**o+(0.702867457627)*x_ref[1]+(0.78445294416)*x_ref[2]**o+(0.613888365259)*x_ref[2]
        ref[3,2,0,0]=(0.568506008135)*x_ref[0]**o+(-0.825947283734)*x_ref[0]+(0.687369924271)*x_ref[1]**o+(-0.866597782569)*x_ref[1]+(-0.332810902355)*x_ref[2]**o+(0.499502049139)*x_ref[2]
        ref[3,2,0,1]=(0.757251773732)*x_ref[0]**o+(0.293337812465)*x_ref[0]+(0.686709285382)*x_ref[1]**o+(-0.084997189402)*x_ref[1]+(-0.338903817381)*x_ref[2]**o+(-0.787646799698)*x_ref[2]
        ref[3,2,1,0]=(-0.580327419983)*x_ref[0]**o+(-0.603952498996)*x_ref[0]+(0.523057988618)*x_ref[1]**o+(-0.0835187394107)*x_ref[1]+(0.459827887147)*x_ref[2]**o+(0.689021201042)*x_ref[2]
        ref[3,2,1,1]=(-0.41923416619)*x_ref[0]**o+(0.0406426957237)*x_ref[0]+(-0.908041008186)*x_ref[1]**o+(0.382097138462)*x_ref[1]+(0.66835706849)*x_ref[2]**o+(0.355849275378)*x_ref[2]
        ref[3,2,2,0]=(-0.133756767108)*x_ref[0]**o+(-0.233743496333)*x_ref[0]+(-0.661420120856)*x_ref[1]**o+(0.788722576584)*x_ref[1]+(-0.131764585707)*x_ref[2]**o+(-0.781004479871)*x_ref[2]
        ref[3,2,2,1]=(0.0308645139784)*x_ref[0]**o+(-0.194986246344)*x_ref[0]+(-0.267418187303)*x_ref[1]**o+(0.332143733168)*x_ref[1]+(0.513266431894)*x_ref[2]**o+(0.594016981473)*x_ref[2]
        ref[3,3,0,0]=(0.644314102613)*x_ref[0]**o+(-0.977355279476)*x_ref[0]+(0.516452323667)*x_ref[1]**o+(0.172873579919)*x_ref[1]+(0.778904819969)*x_ref[2]**o+(0.559689750425)*x_ref[2]
        ref[3,3,0,1]=(0.736135986201)*x_ref[0]**o+(0.366918250826)*x_ref[0]+(-0.0547007988818)*x_ref[1]**o+(-0.830174436433)*x_ref[1]+(0.613440124338)*x_ref[2]**o+(0.574992469591)*x_ref[2]
        ref[3,3,1,0]=(0.324504938146)*x_ref[0]**o+(-0.915240176689)*x_ref[0]+(-0.398105336249)*x_ref[1]**o+(-0.526498343533)*x_ref[1]+(0.43264573336)*x_ref[2]**o+(0.393008086529)*x_ref[2]
        ref[3,3,1,1]=(0.926367438323)*x_ref[0]**o+(0.526364305462)*x_ref[0]+(0.15596582236)*x_ref[1]**o+(0.165158439711)*x_ref[1]+(-0.508505348973)*x_ref[2]**o+(-0.603637833992)*x_ref[2]
        ref[3,3,2,0]=(0.661333046004)*x_ref[0]**o+(0.425320373512)*x_ref[0]+(-0.810809051016)*x_ref[1]**o+(0.20446843023)*x_ref[1]+(0.685136412413)*x_ref[2]**o+(-0.0281127019894)*x_ref[2]
        ref[3,3,2,1]=(0.851742499007)*x_ref[0]**o+(-0.405506974975)*x_ref[0]+(-0.296768983731)*x_ref[1]**o+(0.389557354929)*x_ref[1]+(0.335108341463)*x_ref[2]**o+(0.0313094433351)*x_ref[2]
        ref[3,4,0,0]=(0.0415035498649)*x_ref[0]**o+(0.827106496882)*x_ref[0]+(-0.481831487377)*x_ref[1]**o+(-0.14935685817)*x_ref[1]+(0.354765095566)*x_ref[2]**o+(0.521898910137)*x_ref[2]
        ref[3,4,0,1]=(-0.36328099035)*x_ref[0]**o+(-0.243342386595)*x_ref[0]+(0.0475935188475)*x_ref[1]**o+(0.845308417697)*x_ref[1]+(0.315259620082)*x_ref[2]**o+(0.541313742759)*x_ref[2]
        ref[3,4,1,0]=(-0.66364149672)*x_ref[0]**o+(0.987505978353)*x_ref[0]+(-0.337659775957)*x_ref[1]**o+(0.197277151989)*x_ref[1]+(-0.431260870605)*x_ref[2]**o+(-0.803375909471)*x_ref[2]
        ref[3,4,1,1]=(0.722104365428)*x_ref[0]**o+(0.168416682879)*x_ref[0]+(-0.415740544748)*x_ref[1]**o+(-0.437836696476)*x_ref[1]+(-0.280581752598)*x_ref[2]**o+(0.434326371762)*x_ref[2]
        ref[3,4,2,0]=(0.673144889684)*x_ref[0]**o+(-0.0628529591764)*x_ref[0]+(0.0161458704408)*x_ref[1]**o+(-0.439966404278)*x_ref[1]+(-0.963402070447)*x_ref[2]**o+(0.308312287021)*x_ref[2]
        ref[3,4,2,1]=(0.649131384887)*x_ref[0]**o+(-0.825086043001)*x_ref[0]+(0.204615530143)*x_ref[1]**o+(0.00801105023644)*x_ref[1]+(-0.234754758061)*x_ref[2]**o+(0.659650083864)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromData_ReducedFunctionOnContactOne_rank0(self):
      """
      tests interpolate for rank 0 Data onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactOne onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactOne(self.domain)
      x=w.getX()
      arg=Data(0,(),w)
      ref=Data(0,(),w_ref)
      if dim==2:
        arg=(-0.935068705503)*x[0]**o+(-0.843768568975)*x[0]+(0.220151879481)*x[1]**o+(0.629247769917)*x[1]
        ref=(-0.935068705503)*x_ref[0]**o+(-0.843768568975)*x_ref[0]+(0.220151879481)*x_ref[1]**o+(0.629247769917)*x_ref[1]
      else:
        arg=(0.204715133488)*x[0]**o+(-0.933589384176)*x[0]+(-0.651477613601)*x[1]**o+(-0.128099401086)*x[1]+(0.0507915334433)*x[2]**o+(0.63225604515)*x[2]
        ref=(0.204715133488)*x_ref[0]**o+(-0.933589384176)*x_ref[0]+(-0.651477613601)*x_ref[1]**o+(-0.128099401086)*x_ref[1]+(0.0507915334433)*x_ref[2]**o+(0.63225604515)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromData_ReducedFunctionOnContactOne_rank1(self):
      """
      tests interpolate for rank 1 Data onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactOne onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactOne(self.domain)
      x=w.getX()
      arg=Data(0,(2,),w)
      ref=Data(0,(2,),w_ref)
      if dim==2:
        arg[0]=(0.693671772608)*x[0]**o+(0.260200526395)*x[0]+(-0.99202032752)*x[1]**o+(0.698503413133)*x[1]
        arg[1]=(-0.549448913905)*x[0]**o+(-0.728769679965)*x[0]+(0.415132041969)*x[1]**o+(-0.106391482954)*x[1]
        ref[0]=(0.693671772608)*x_ref[0]**o+(0.260200526395)*x_ref[0]+(-0.99202032752)*x_ref[1]**o+(0.698503413133)*x_ref[1]
        ref[1]=(-0.549448913905)*x_ref[0]**o+(-0.728769679965)*x_ref[0]+(0.415132041969)*x_ref[1]**o+(-0.106391482954)*x_ref[1]
      else:
        arg[0]=(0.479813279739)*x[0]**o+(0.940795516779)*x[0]+(-0.671143421639)*x[1]**o+(-0.485412476854)*x[1]+(-0.962114397463)*x[2]**o+(0.0250734884375)*x[2]
        arg[1]=(0.703481492279)*x[0]**o+(0.298972367312)*x[0]+(0.22386003834)*x[1]**o+(-0.726486516907)*x[1]+(0.0556568111911)*x[2]**o+(-0.603711754869)*x[2]
        ref[0]=(0.479813279739)*x_ref[0]**o+(0.940795516779)*x_ref[0]+(-0.671143421639)*x_ref[1]**o+(-0.485412476854)*x_ref[1]+(-0.962114397463)*x_ref[2]**o+(0.0250734884375)*x_ref[2]
        ref[1]=(0.703481492279)*x_ref[0]**o+(0.298972367312)*x_ref[0]+(0.22386003834)*x_ref[1]**o+(-0.726486516907)*x_ref[1]+(0.0556568111911)*x_ref[2]**o+(-0.603711754869)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromData_ReducedFunctionOnContactOne_rank2(self):
      """
      tests interpolate for rank 2 Data onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactOne onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactOne(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5),w)
      ref=Data(0,(4, 5),w_ref)
      if dim==2:
        arg[0,0]=(0.436439283545)*x[0]**o+(0.705849609672)*x[0]+(-0.261476237133)*x[1]**o+(-0.10504570237)*x[1]
        arg[0,1]=(0.035208834696)*x[0]**o+(0.590058421274)*x[0]+(-0.532905126944)*x[1]**o+(0.126572388319)*x[1]
        arg[0,2]=(0.985266120604)*x[0]**o+(-0.465951670003)*x[0]+(0.855881157118)*x[1]**o+(0.00932326399668)*x[1]
        arg[0,3]=(0.867525798771)*x[0]**o+(-0.480556561488)*x[0]+(-0.958556900472)*x[1]**o+(-0.764939529718)*x[1]
        arg[0,4]=(-0.51062937871)*x[0]**o+(-0.135415437913)*x[0]+(0.340077694039)*x[1]**o+(0.874347806916)*x[1]
        arg[1,0]=(0.636810684661)*x[0]**o+(0.774651780754)*x[0]+(-0.314130997812)*x[1]**o+(-0.422706240721)*x[1]
        arg[1,1]=(0.495122166216)*x[0]**o+(0.757099858142)*x[0]+(-0.25861054515)*x[1]**o+(0.612180732826)*x[1]
        arg[1,2]=(-0.909636123949)*x[0]**o+(-0.0434375137409)*x[0]+(0.100612174358)*x[1]**o+(0.787104124194)*x[1]
        arg[1,3]=(-0.499307726848)*x[0]**o+(0.013205984499)*x[0]+(0.858074302957)*x[1]**o+(-0.630926261217)*x[1]
        arg[1,4]=(-0.410815849264)*x[0]**o+(0.820939326325)*x[0]+(-0.415370205615)*x[1]**o+(0.0602438669371)*x[1]
        arg[2,0]=(0.642545275281)*x[0]**o+(0.872239548588)*x[0]+(0.446294659843)*x[1]**o+(0.929350720888)*x[1]
        arg[2,1]=(0.182218495486)*x[0]**o+(-0.393184556589)*x[0]+(0.42311545961)*x[1]**o+(-0.479636250878)*x[1]
        arg[2,2]=(0.874574618473)*x[0]**o+(0.879031610251)*x[0]+(0.686157301617)*x[1]**o+(0.706917527697)*x[1]
        arg[2,3]=(0.614189029029)*x[0]**o+(0.0973950074653)*x[0]+(0.626052996278)*x[1]**o+(0.75043535006)*x[1]
        arg[2,4]=(0.386209196978)*x[0]**o+(0.644169312279)*x[0]+(-0.554562346917)*x[1]**o+(-0.474292025255)*x[1]
        arg[3,0]=(0.015012073686)*x[0]**o+(-0.665804159938)*x[0]+(-0.849030247328)*x[1]**o+(-0.936779648988)*x[1]
        arg[3,1]=(-0.818194334698)*x[0]**o+(-0.569993362998)*x[0]+(-0.806235611766)*x[1]**o+(0.35347475408)*x[1]
        arg[3,2]=(-0.227117779134)*x[0]**o+(-0.981285175536)*x[0]+(-0.226804205345)*x[1]**o+(-0.0134258740334)*x[1]
        arg[3,3]=(0.126716245369)*x[0]**o+(0.135476076345)*x[0]+(-0.641421845681)*x[1]**o+(0.0595388928019)*x[1]
        arg[3,4]=(-0.658260624128)*x[0]**o+(0.124989215683)*x[0]+(-0.949146047463)*x[1]**o+(0.805277938593)*x[1]
        ref[0,0]=(0.436439283545)*x_ref[0]**o+(0.705849609672)*x_ref[0]+(-0.261476237133)*x_ref[1]**o+(-0.10504570237)*x_ref[1]
        ref[0,1]=(0.035208834696)*x_ref[0]**o+(0.590058421274)*x_ref[0]+(-0.532905126944)*x_ref[1]**o+(0.126572388319)*x_ref[1]
        ref[0,2]=(0.985266120604)*x_ref[0]**o+(-0.465951670003)*x_ref[0]+(0.855881157118)*x_ref[1]**o+(0.00932326399668)*x_ref[1]
        ref[0,3]=(0.867525798771)*x_ref[0]**o+(-0.480556561488)*x_ref[0]+(-0.958556900472)*x_ref[1]**o+(-0.764939529718)*x_ref[1]
        ref[0,4]=(-0.51062937871)*x_ref[0]**o+(-0.135415437913)*x_ref[0]+(0.340077694039)*x_ref[1]**o+(0.874347806916)*x_ref[1]
        ref[1,0]=(0.636810684661)*x_ref[0]**o+(0.774651780754)*x_ref[0]+(-0.314130997812)*x_ref[1]**o+(-0.422706240721)*x_ref[1]
        ref[1,1]=(0.495122166216)*x_ref[0]**o+(0.757099858142)*x_ref[0]+(-0.25861054515)*x_ref[1]**o+(0.612180732826)*x_ref[1]
        ref[1,2]=(-0.909636123949)*x_ref[0]**o+(-0.0434375137409)*x_ref[0]+(0.100612174358)*x_ref[1]**o+(0.787104124194)*x_ref[1]
        ref[1,3]=(-0.499307726848)*x_ref[0]**o+(0.013205984499)*x_ref[0]+(0.858074302957)*x_ref[1]**o+(-0.630926261217)*x_ref[1]
        ref[1,4]=(-0.410815849264)*x_ref[0]**o+(0.820939326325)*x_ref[0]+(-0.415370205615)*x_ref[1]**o+(0.0602438669371)*x_ref[1]
        ref[2,0]=(0.642545275281)*x_ref[0]**o+(0.872239548588)*x_ref[0]+(0.446294659843)*x_ref[1]**o+(0.929350720888)*x_ref[1]
        ref[2,1]=(0.182218495486)*x_ref[0]**o+(-0.393184556589)*x_ref[0]+(0.42311545961)*x_ref[1]**o+(-0.479636250878)*x_ref[1]
        ref[2,2]=(0.874574618473)*x_ref[0]**o+(0.879031610251)*x_ref[0]+(0.686157301617)*x_ref[1]**o+(0.706917527697)*x_ref[1]
        ref[2,3]=(0.614189029029)*x_ref[0]**o+(0.0973950074653)*x_ref[0]+(0.626052996278)*x_ref[1]**o+(0.75043535006)*x_ref[1]
        ref[2,4]=(0.386209196978)*x_ref[0]**o+(0.644169312279)*x_ref[0]+(-0.554562346917)*x_ref[1]**o+(-0.474292025255)*x_ref[1]
        ref[3,0]=(0.015012073686)*x_ref[0]**o+(-0.665804159938)*x_ref[0]+(-0.849030247328)*x_ref[1]**o+(-0.936779648988)*x_ref[1]
        ref[3,1]=(-0.818194334698)*x_ref[0]**o+(-0.569993362998)*x_ref[0]+(-0.806235611766)*x_ref[1]**o+(0.35347475408)*x_ref[1]
        ref[3,2]=(-0.227117779134)*x_ref[0]**o+(-0.981285175536)*x_ref[0]+(-0.226804205345)*x_ref[1]**o+(-0.0134258740334)*x_ref[1]
        ref[3,3]=(0.126716245369)*x_ref[0]**o+(0.135476076345)*x_ref[0]+(-0.641421845681)*x_ref[1]**o+(0.0595388928019)*x_ref[1]
        ref[3,4]=(-0.658260624128)*x_ref[0]**o+(0.124989215683)*x_ref[0]+(-0.949146047463)*x_ref[1]**o+(0.805277938593)*x_ref[1]
      else:
        arg[0,0]=(-0.859332605724)*x[0]**o+(-0.239005430756)*x[0]+(-0.52959833033)*x[1]**o+(0.725877930655)*x[1]+(-0.0348436114161)*x[2]**o+(0.47287744745)*x[2]
        arg[0,1]=(-0.0816886854463)*x[0]**o+(-0.626507446322)*x[0]+(-0.93804868959)*x[1]**o+(-0.450134311929)*x[1]+(0.0957946938715)*x[2]**o+(-0.236994496198)*x[2]
        arg[0,2]=(-0.669441310039)*x[0]**o+(-0.406451428536)*x[0]+(-0.292718530458)*x[1]**o+(-0.0948016380336)*x[1]+(-0.0943083566957)*x[2]**o+(0.747978324024)*x[2]
        arg[0,3]=(-0.0350228116485)*x[0]**o+(-0.967728170529)*x[0]+(-0.16425180976)*x[1]**o+(-0.141334783297)*x[1]+(0.179458630309)*x[2]**o+(0.45080506919)*x[2]
        arg[0,4]=(0.909578721407)*x[0]**o+(-0.341028350711)*x[0]+(-0.937703389458)*x[1]**o+(-0.845025015446)*x[1]+(-0.762836324997)*x[2]**o+(0.772817997464)*x[2]
        arg[1,0]=(0.75675349915)*x[0]**o+(-0.208827138169)*x[0]+(-0.0146742840091)*x[1]**o+(-0.1486215734)*x[1]+(0.205344647252)*x[2]**o+(-0.409293867732)*x[2]
        arg[1,1]=(-0.215359027624)*x[0]**o+(0.220661567012)*x[0]+(0.122954614324)*x[1]**o+(-0.888828539035)*x[1]+(-0.860535003713)*x[2]**o+(-0.582396229154)*x[2]
        arg[1,2]=(-0.252703089432)*x[0]**o+(0.294168447756)*x[0]+(0.323290819691)*x[1]**o+(-0.0440370164755)*x[1]+(0.487110700327)*x[2]**o+(-0.269109129573)*x[2]
        arg[1,3]=(0.0653521784751)*x[0]**o+(0.328048279094)*x[0]+(-0.414467594535)*x[1]**o+(0.436061182815)*x[1]+(0.483513283452)*x[2]**o+(-0.744560862926)*x[2]
        arg[1,4]=(0.0102955000926)*x[0]**o+(-0.636994970624)*x[0]+(0.83732393089)*x[1]**o+(0.914683806871)*x[1]+(0.574439856221)*x[2]**o+(-0.096821482591)*x[2]
        arg[2,0]=(-0.129692996812)*x[0]**o+(-0.617534967242)*x[0]+(0.299770969796)*x[1]**o+(0.93870530376)*x[1]+(-0.864448551767)*x[2]**o+(0.755129965839)*x[2]
        arg[2,1]=(-0.354122778719)*x[0]**o+(0.576654786045)*x[0]+(-0.965116346607)*x[1]**o+(0.0705856628847)*x[1]+(0.817059960545)*x[2]**o+(-0.597731384232)*x[2]
        arg[2,2]=(-0.705190107591)*x[0]**o+(0.876019062019)*x[0]+(-0.732733986501)*x[1]**o+(0.505334922081)*x[1]+(-0.616601928994)*x[2]**o+(-0.332268983863)*x[2]
        arg[2,3]=(0.663768017414)*x[0]**o+(-0.431682444208)*x[0]+(0.186159158088)*x[1]**o+(-0.689766933274)*x[1]+(-0.765854811539)*x[2]**o+(-0.402774599632)*x[2]
        arg[2,4]=(-0.698031877485)*x[0]**o+(0.914418269242)*x[0]+(-0.693564630363)*x[1]**o+(-0.18289238868)*x[1]+(0.988158500287)*x[2]**o+(-0.33952288549)*x[2]
        arg[3,0]=(-0.188499634873)*x[0]**o+(-0.10427816897)*x[0]+(-0.772866404794)*x[1]**o+(0.841928986479)*x[1]+(-0.866955639695)*x[2]**o+(-0.152515560713)*x[2]
        arg[3,1]=(0.937695460564)*x[0]**o+(-0.551079526492)*x[0]+(0.353043727721)*x[1]**o+(-0.604814667118)*x[1]+(0.200913251642)*x[2]**o+(-0.0497427852682)*x[2]
        arg[3,2]=(-0.49130784927)*x[0]**o+(0.537445967432)*x[0]+(-0.408371200391)*x[1]**o+(0.945593173362)*x[1]+(-0.0956969348675)*x[2]**o+(-0.174133549441)*x[2]
        arg[3,3]=(0.847790369559)*x[0]**o+(0.743993523287)*x[0]+(-0.810126540693)*x[1]**o+(-0.775699501886)*x[1]+(-0.479687101313)*x[2]**o+(0.627116825116)*x[2]
        arg[3,4]=(0.352455163407)*x[0]**o+(0.86343850667)*x[0]+(0.126742974996)*x[1]**o+(0.0175699888323)*x[1]+(0.964899127449)*x[2]**o+(-0.206749046822)*x[2]
        ref[0,0]=(-0.859332605724)*x_ref[0]**o+(-0.239005430756)*x_ref[0]+(-0.52959833033)*x_ref[1]**o+(0.725877930655)*x_ref[1]+(-0.0348436114161)*x_ref[2]**o+(0.47287744745)*x_ref[2]
        ref[0,1]=(-0.0816886854463)*x_ref[0]**o+(-0.626507446322)*x_ref[0]+(-0.93804868959)*x_ref[1]**o+(-0.450134311929)*x_ref[1]+(0.0957946938715)*x_ref[2]**o+(-0.236994496198)*x_ref[2]
        ref[0,2]=(-0.669441310039)*x_ref[0]**o+(-0.406451428536)*x_ref[0]+(-0.292718530458)*x_ref[1]**o+(-0.0948016380336)*x_ref[1]+(-0.0943083566957)*x_ref[2]**o+(0.747978324024)*x_ref[2]
        ref[0,3]=(-0.0350228116485)*x_ref[0]**o+(-0.967728170529)*x_ref[0]+(-0.16425180976)*x_ref[1]**o+(-0.141334783297)*x_ref[1]+(0.179458630309)*x_ref[2]**o+(0.45080506919)*x_ref[2]
        ref[0,4]=(0.909578721407)*x_ref[0]**o+(-0.341028350711)*x_ref[0]+(-0.937703389458)*x_ref[1]**o+(-0.845025015446)*x_ref[1]+(-0.762836324997)*x_ref[2]**o+(0.772817997464)*x_ref[2]
        ref[1,0]=(0.75675349915)*x_ref[0]**o+(-0.208827138169)*x_ref[0]+(-0.0146742840091)*x_ref[1]**o+(-0.1486215734)*x_ref[1]+(0.205344647252)*x_ref[2]**o+(-0.409293867732)*x_ref[2]
        ref[1,1]=(-0.215359027624)*x_ref[0]**o+(0.220661567012)*x_ref[0]+(0.122954614324)*x_ref[1]**o+(-0.888828539035)*x_ref[1]+(-0.860535003713)*x_ref[2]**o+(-0.582396229154)*x_ref[2]
        ref[1,2]=(-0.252703089432)*x_ref[0]**o+(0.294168447756)*x_ref[0]+(0.323290819691)*x_ref[1]**o+(-0.0440370164755)*x_ref[1]+(0.487110700327)*x_ref[2]**o+(-0.269109129573)*x_ref[2]
        ref[1,3]=(0.0653521784751)*x_ref[0]**o+(0.328048279094)*x_ref[0]+(-0.414467594535)*x_ref[1]**o+(0.436061182815)*x_ref[1]+(0.483513283452)*x_ref[2]**o+(-0.744560862926)*x_ref[2]
        ref[1,4]=(0.0102955000926)*x_ref[0]**o+(-0.636994970624)*x_ref[0]+(0.83732393089)*x_ref[1]**o+(0.914683806871)*x_ref[1]+(0.574439856221)*x_ref[2]**o+(-0.096821482591)*x_ref[2]
        ref[2,0]=(-0.129692996812)*x_ref[0]**o+(-0.617534967242)*x_ref[0]+(0.299770969796)*x_ref[1]**o+(0.93870530376)*x_ref[1]+(-0.864448551767)*x_ref[2]**o+(0.755129965839)*x_ref[2]
        ref[2,1]=(-0.354122778719)*x_ref[0]**o+(0.576654786045)*x_ref[0]+(-0.965116346607)*x_ref[1]**o+(0.0705856628847)*x_ref[1]+(0.817059960545)*x_ref[2]**o+(-0.597731384232)*x_ref[2]
        ref[2,2]=(-0.705190107591)*x_ref[0]**o+(0.876019062019)*x_ref[0]+(-0.732733986501)*x_ref[1]**o+(0.505334922081)*x_ref[1]+(-0.616601928994)*x_ref[2]**o+(-0.332268983863)*x_ref[2]
        ref[2,3]=(0.663768017414)*x_ref[0]**o+(-0.431682444208)*x_ref[0]+(0.186159158088)*x_ref[1]**o+(-0.689766933274)*x_ref[1]+(-0.765854811539)*x_ref[2]**o+(-0.402774599632)*x_ref[2]
        ref[2,4]=(-0.698031877485)*x_ref[0]**o+(0.914418269242)*x_ref[0]+(-0.693564630363)*x_ref[1]**o+(-0.18289238868)*x_ref[1]+(0.988158500287)*x_ref[2]**o+(-0.33952288549)*x_ref[2]
        ref[3,0]=(-0.188499634873)*x_ref[0]**o+(-0.10427816897)*x_ref[0]+(-0.772866404794)*x_ref[1]**o+(0.841928986479)*x_ref[1]+(-0.866955639695)*x_ref[2]**o+(-0.152515560713)*x_ref[2]
        ref[3,1]=(0.937695460564)*x_ref[0]**o+(-0.551079526492)*x_ref[0]+(0.353043727721)*x_ref[1]**o+(-0.604814667118)*x_ref[1]+(0.200913251642)*x_ref[2]**o+(-0.0497427852682)*x_ref[2]
        ref[3,2]=(-0.49130784927)*x_ref[0]**o+(0.537445967432)*x_ref[0]+(-0.408371200391)*x_ref[1]**o+(0.945593173362)*x_ref[1]+(-0.0956969348675)*x_ref[2]**o+(-0.174133549441)*x_ref[2]
        ref[3,3]=(0.847790369559)*x_ref[0]**o+(0.743993523287)*x_ref[0]+(-0.810126540693)*x_ref[1]**o+(-0.775699501886)*x_ref[1]+(-0.479687101313)*x_ref[2]**o+(0.627116825116)*x_ref[2]
        ref[3,4]=(0.352455163407)*x_ref[0]**o+(0.86343850667)*x_ref[0]+(0.126742974996)*x_ref[1]**o+(0.0175699888323)*x_ref[1]+(0.964899127449)*x_ref[2]**o+(-0.206749046822)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromData_ReducedFunctionOnContactOne_rank3(self):
      """
      tests interpolate for rank 3 Data onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactOne onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactOne(self.domain)
      x=w.getX()
      arg=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2),w_ref)
      if dim==2:
        arg[0,0,0]=(-0.396745847367)*x[0]**o+(-0.700719490691)*x[0]+(0.117558242545)*x[1]**o+(-0.161976142849)*x[1]
        arg[0,0,1]=(-0.490732277107)*x[0]**o+(0.526833149216)*x[0]+(-0.136309303122)*x[1]**o+(0.629465649533)*x[1]
        arg[0,1,0]=(-0.242474968121)*x[0]**o+(-0.76270698332)*x[0]+(-0.0979504547153)*x[1]**o+(-0.0743521177641)*x[1]
        arg[0,1,1]=(0.641246476843)*x[0]**o+(0.0158147797433)*x[0]+(-0.213898399703)*x[1]**o+(-0.378401519675)*x[1]
        arg[1,0,0]=(0.603700544573)*x[0]**o+(-0.993751144334)*x[0]+(0.466222025692)*x[1]**o+(0.457689495241)*x[1]
        arg[1,0,1]=(-0.727287858841)*x[0]**o+(-0.177841994955)*x[0]+(0.720237448692)*x[1]**o+(0.716175629178)*x[1]
        arg[1,1,0]=(0.784514682242)*x[0]**o+(0.681214612353)*x[0]+(0.404266860166)*x[1]**o+(0.871218157219)*x[1]
        arg[1,1,1]=(-0.918634586367)*x[0]**o+(-0.722365105889)*x[0]+(-0.0337202979587)*x[1]**o+(-0.487790376172)*x[1]
        arg[2,0,0]=(0.159515902499)*x[0]**o+(-0.0650417831188)*x[0]+(0.107260481333)*x[1]**o+(0.469096212511)*x[1]
        arg[2,0,1]=(0.0436003865409)*x[0]**o+(-0.67488837592)*x[0]+(0.547761443272)*x[1]**o+(-0.69222031017)*x[1]
        arg[2,1,0]=(-0.153636036391)*x[0]**o+(0.0415911821723)*x[0]+(0.421990370678)*x[1]**o+(-0.128883832414)*x[1]
        arg[2,1,1]=(0.513420393955)*x[0]**o+(-0.769636964711)*x[0]+(0.874383588901)*x[1]**o+(-0.357923227767)*x[1]
        arg[3,0,0]=(0.943275907357)*x[0]**o+(0.372378913231)*x[0]+(0.559656674007)*x[1]**o+(-0.878007664293)*x[1]
        arg[3,0,1]=(-0.541535388034)*x[0]**o+(-0.0510200932939)*x[0]+(-0.448618747155)*x[1]**o+(0.693514427768)*x[1]
        arg[3,1,0]=(-0.287622342268)*x[0]**o+(0.8333134988)*x[0]+(-0.691930501782)*x[1]**o+(-0.659618596505)*x[1]
        arg[3,1,1]=(0.44446133556)*x[0]**o+(0.697704081094)*x[0]+(0.0679863572322)*x[1]**o+(-0.949443102916)*x[1]
        arg[4,0,0]=(0.546245770076)*x[0]**o+(-0.466891879347)*x[0]+(-0.608261901126)*x[1]**o+(0.669561643667)*x[1]
        arg[4,0,1]=(-0.522497485476)*x[0]**o+(0.70886716903)*x[0]+(-0.296815361132)*x[1]**o+(0.500540583082)*x[1]
        arg[4,1,0]=(-0.841653490079)*x[0]**o+(0.400589086976)*x[0]+(0.865994660457)*x[1]**o+(0.0205817210969)*x[1]
        arg[4,1,1]=(-0.64369513787)*x[0]**o+(0.883032618928)*x[0]+(-0.178766897224)*x[1]**o+(0.748041632322)*x[1]
        arg[5,0,0]=(0.471810615096)*x[0]**o+(-0.140340964274)*x[0]+(-0.7063314968)*x[1]**o+(0.0524231879837)*x[1]
        arg[5,0,1]=(-0.51339482213)*x[0]**o+(-0.81447661092)*x[0]+(0.437085868301)*x[1]**o+(0.862678017842)*x[1]
        arg[5,1,0]=(-0.752858172473)*x[0]**o+(0.325827525243)*x[0]+(-0.702286757187)*x[1]**o+(-0.467810951898)*x[1]
        arg[5,1,1]=(-0.256879797442)*x[0]**o+(0.257067666229)*x[0]+(0.912308244411)*x[1]**o+(0.0620337856352)*x[1]
        ref[0,0,0]=(-0.396745847367)*x_ref[0]**o+(-0.700719490691)*x_ref[0]+(0.117558242545)*x_ref[1]**o+(-0.161976142849)*x_ref[1]
        ref[0,0,1]=(-0.490732277107)*x_ref[0]**o+(0.526833149216)*x_ref[0]+(-0.136309303122)*x_ref[1]**o+(0.629465649533)*x_ref[1]
        ref[0,1,0]=(-0.242474968121)*x_ref[0]**o+(-0.76270698332)*x_ref[0]+(-0.0979504547153)*x_ref[1]**o+(-0.0743521177641)*x_ref[1]
        ref[0,1,1]=(0.641246476843)*x_ref[0]**o+(0.0158147797433)*x_ref[0]+(-0.213898399703)*x_ref[1]**o+(-0.378401519675)*x_ref[1]
        ref[1,0,0]=(0.603700544573)*x_ref[0]**o+(-0.993751144334)*x_ref[0]+(0.466222025692)*x_ref[1]**o+(0.457689495241)*x_ref[1]
        ref[1,0,1]=(-0.727287858841)*x_ref[0]**o+(-0.177841994955)*x_ref[0]+(0.720237448692)*x_ref[1]**o+(0.716175629178)*x_ref[1]
        ref[1,1,0]=(0.784514682242)*x_ref[0]**o+(0.681214612353)*x_ref[0]+(0.404266860166)*x_ref[1]**o+(0.871218157219)*x_ref[1]
        ref[1,1,1]=(-0.918634586367)*x_ref[0]**o+(-0.722365105889)*x_ref[0]+(-0.0337202979587)*x_ref[1]**o+(-0.487790376172)*x_ref[1]
        ref[2,0,0]=(0.159515902499)*x_ref[0]**o+(-0.0650417831188)*x_ref[0]+(0.107260481333)*x_ref[1]**o+(0.469096212511)*x_ref[1]
        ref[2,0,1]=(0.0436003865409)*x_ref[0]**o+(-0.67488837592)*x_ref[0]+(0.547761443272)*x_ref[1]**o+(-0.69222031017)*x_ref[1]
        ref[2,1,0]=(-0.153636036391)*x_ref[0]**o+(0.0415911821723)*x_ref[0]+(0.421990370678)*x_ref[1]**o+(-0.128883832414)*x_ref[1]
        ref[2,1,1]=(0.513420393955)*x_ref[0]**o+(-0.769636964711)*x_ref[0]+(0.874383588901)*x_ref[1]**o+(-0.357923227767)*x_ref[1]
        ref[3,0,0]=(0.943275907357)*x_ref[0]**o+(0.372378913231)*x_ref[0]+(0.559656674007)*x_ref[1]**o+(-0.878007664293)*x_ref[1]
        ref[3,0,1]=(-0.541535388034)*x_ref[0]**o+(-0.0510200932939)*x_ref[0]+(-0.448618747155)*x_ref[1]**o+(0.693514427768)*x_ref[1]
        ref[3,1,0]=(-0.287622342268)*x_ref[0]**o+(0.8333134988)*x_ref[0]+(-0.691930501782)*x_ref[1]**o+(-0.659618596505)*x_ref[1]
        ref[3,1,1]=(0.44446133556)*x_ref[0]**o+(0.697704081094)*x_ref[0]+(0.0679863572322)*x_ref[1]**o+(-0.949443102916)*x_ref[1]
        ref[4,0,0]=(0.546245770076)*x_ref[0]**o+(-0.466891879347)*x_ref[0]+(-0.608261901126)*x_ref[1]**o+(0.669561643667)*x_ref[1]
        ref[4,0,1]=(-0.522497485476)*x_ref[0]**o+(0.70886716903)*x_ref[0]+(-0.296815361132)*x_ref[1]**o+(0.500540583082)*x_ref[1]
        ref[4,1,0]=(-0.841653490079)*x_ref[0]**o+(0.400589086976)*x_ref[0]+(0.865994660457)*x_ref[1]**o+(0.0205817210969)*x_ref[1]
        ref[4,1,1]=(-0.64369513787)*x_ref[0]**o+(0.883032618928)*x_ref[0]+(-0.178766897224)*x_ref[1]**o+(0.748041632322)*x_ref[1]
        ref[5,0,0]=(0.471810615096)*x_ref[0]**o+(-0.140340964274)*x_ref[0]+(-0.7063314968)*x_ref[1]**o+(0.0524231879837)*x_ref[1]
        ref[5,0,1]=(-0.51339482213)*x_ref[0]**o+(-0.81447661092)*x_ref[0]+(0.437085868301)*x_ref[1]**o+(0.862678017842)*x_ref[1]
        ref[5,1,0]=(-0.752858172473)*x_ref[0]**o+(0.325827525243)*x_ref[0]+(-0.702286757187)*x_ref[1]**o+(-0.467810951898)*x_ref[1]
        ref[5,1,1]=(-0.256879797442)*x_ref[0]**o+(0.257067666229)*x_ref[0]+(0.912308244411)*x_ref[1]**o+(0.0620337856352)*x_ref[1]
      else:
        arg[0,0,0]=(0.564502533309)*x[0]**o+(-0.237721559752)*x[0]+(0.225601658644)*x[1]**o+(0.967190188292)*x[1]+(0.471760897542)*x[2]**o+(0.79404420728)*x[2]
        arg[0,0,1]=(-0.373762346136)*x[0]**o+(-0.643766021731)*x[0]+(0.088028383792)*x[1]**o+(-0.541163410313)*x[1]+(0.463315911637)*x[2]**o+(-0.596740807595)*x[2]
        arg[0,1,0]=(-0.526694453842)*x[0]**o+(0.405010889931)*x[0]+(-0.0153807962972)*x[1]**o+(-0.0446620712949)*x[1]+(0.00553918668197)*x[2]**o+(0.72170514564)*x[2]
        arg[0,1,1]=(-0.00750073616089)*x[0]**o+(-0.60431903208)*x[0]+(0.790032415931)*x[1]**o+(0.272709398336)*x[1]+(0.228539600712)*x[2]**o+(0.62284140325)*x[2]
        arg[1,0,0]=(-0.892797595418)*x[0]**o+(-0.151608141828)*x[0]+(0.313142074823)*x[1]**o+(0.80280439466)*x[1]+(-0.593056273594)*x[2]**o+(-0.41551679707)*x[2]
        arg[1,0,1]=(-0.385715092974)*x[0]**o+(-0.77884107065)*x[0]+(-0.0804019137225)*x[1]**o+(-0.240129932101)*x[1]+(0.238145788042)*x[2]**o+(-0.806337666628)*x[2]
        arg[1,1,0]=(0.940492459417)*x[0]**o+(-0.0054460073194)*x[0]+(-0.697916565168)*x[1]**o+(-0.949456184725)*x[1]+(-0.115461214789)*x[2]**o+(-0.403742409948)*x[2]
        arg[1,1,1]=(-0.711716124415)*x[0]**o+(0.586131021871)*x[0]+(0.625988666837)*x[1]**o+(0.638433219635)*x[1]+(0.971169174015)*x[2]**o+(-0.064949626884)*x[2]
        arg[2,0,0]=(-0.891973333325)*x[0]**o+(0.76090478364)*x[0]+(0.439956136239)*x[1]**o+(-0.74807071296)*x[1]+(-0.209999803467)*x[2]**o+(0.404396254503)*x[2]
        arg[2,0,1]=(0.559776288735)*x[0]**o+(0.284643618707)*x[0]+(0.0527799359967)*x[1]**o+(-0.201660204363)*x[1]+(0.590722655898)*x[2]**o+(-0.24940757474)*x[2]
        arg[2,1,0]=(-0.747588234432)*x[0]**o+(0.0196702046746)*x[0]+(0.0454238912973)*x[1]**o+(0.174053915643)*x[1]+(-0.868554688657)*x[2]**o+(-0.127425849813)*x[2]
        arg[2,1,1]=(-0.324276984008)*x[0]**o+(-0.494912096003)*x[0]+(-0.780953487415)*x[1]**o+(0.225787179491)*x[1]+(-0.760330660035)*x[2]**o+(0.0440722700759)*x[2]
        arg[3,0,0]=(0.37479028021)*x[0]**o+(-0.869009335783)*x[0]+(0.811673580204)*x[1]**o+(0.521877505351)*x[1]+(-0.508710979256)*x[2]**o+(0.558085136638)*x[2]
        arg[3,0,1]=(0.815143535066)*x[0]**o+(0.867977183071)*x[0]+(0.0315543942614)*x[1]**o+(0.786379597601)*x[1]+(0.0433279381339)*x[2]**o+(-0.56554445624)*x[2]
        arg[3,1,0]=(-0.948224238625)*x[0]**o+(-0.759282528831)*x[0]+(0.511802336753)*x[1]**o+(0.331073342499)*x[1]+(0.295241697427)*x[2]**o+(0.154864664771)*x[2]
        arg[3,1,1]=(-0.982634132939)*x[0]**o+(-0.0762014431695)*x[0]+(-0.0797462613351)*x[1]**o+(-0.512305271957)*x[1]+(-0.348883505418)*x[2]**o+(0.00580412526927)*x[2]
        arg[4,0,0]=(0.186597109908)*x[0]**o+(0.051957206238)*x[0]+(0.572882754553)*x[1]**o+(-0.114338954084)*x[1]+(-0.517239439578)*x[2]**o+(-0.298287404607)*x[2]
        arg[4,0,1]=(0.637628569107)*x[0]**o+(0.228562669958)*x[0]+(0.053657321561)*x[1]**o+(0.762909608846)*x[1]+(0.873936819574)*x[2]**o+(-0.20000969514)*x[2]
        arg[4,1,0]=(-0.717459489066)*x[0]**o+(-0.462174191129)*x[0]+(-0.70491180102)*x[1]**o+(-0.406072010622)*x[1]+(-0.903872259447)*x[2]**o+(0.487491537826)*x[2]
        arg[4,1,1]=(0.947654286923)*x[0]**o+(-0.228830597313)*x[0]+(-0.501442713655)*x[1]**o+(0.282437368749)*x[1]+(0.543592950002)*x[2]**o+(0.5400198247)*x[2]
        arg[5,0,0]=(0.144349494033)*x[0]**o+(-0.635328703132)*x[0]+(-0.797851162499)*x[1]**o+(0.356874780185)*x[1]+(-0.205725457569)*x[2]**o+(-0.674962512018)*x[2]
        arg[5,0,1]=(-0.728948504419)*x[0]**o+(0.944337871242)*x[0]+(-0.790388944678)*x[1]**o+(0.59119693076)*x[1]+(0.9569152246)*x[2]**o+(-0.205404245177)*x[2]
        arg[5,1,0]=(0.477597678348)*x[0]**o+(-0.318993929605)*x[0]+(0.481029970286)*x[1]**o+(0.708696918395)*x[1]+(0.0531716082887)*x[2]**o+(-0.847697491061)*x[2]
        arg[5,1,1]=(-0.458572001979)*x[0]**o+(0.582048244354)*x[0]+(-0.040756132547)*x[1]**o+(-0.953201198076)*x[1]+(-0.971859723822)*x[2]**o+(-0.796426143181)*x[2]
        ref[0,0,0]=(0.564502533309)*x_ref[0]**o+(-0.237721559752)*x_ref[0]+(0.225601658644)*x_ref[1]**o+(0.967190188292)*x_ref[1]+(0.471760897542)*x_ref[2]**o+(0.79404420728)*x_ref[2]
        ref[0,0,1]=(-0.373762346136)*x_ref[0]**o+(-0.643766021731)*x_ref[0]+(0.088028383792)*x_ref[1]**o+(-0.541163410313)*x_ref[1]+(0.463315911637)*x_ref[2]**o+(-0.596740807595)*x_ref[2]
        ref[0,1,0]=(-0.526694453842)*x_ref[0]**o+(0.405010889931)*x_ref[0]+(-0.0153807962972)*x_ref[1]**o+(-0.0446620712949)*x_ref[1]+(0.00553918668197)*x_ref[2]**o+(0.72170514564)*x_ref[2]
        ref[0,1,1]=(-0.00750073616089)*x_ref[0]**o+(-0.60431903208)*x_ref[0]+(0.790032415931)*x_ref[1]**o+(0.272709398336)*x_ref[1]+(0.228539600712)*x_ref[2]**o+(0.62284140325)*x_ref[2]
        ref[1,0,0]=(-0.892797595418)*x_ref[0]**o+(-0.151608141828)*x_ref[0]+(0.313142074823)*x_ref[1]**o+(0.80280439466)*x_ref[1]+(-0.593056273594)*x_ref[2]**o+(-0.41551679707)*x_ref[2]
        ref[1,0,1]=(-0.385715092974)*x_ref[0]**o+(-0.77884107065)*x_ref[0]+(-0.0804019137225)*x_ref[1]**o+(-0.240129932101)*x_ref[1]+(0.238145788042)*x_ref[2]**o+(-0.806337666628)*x_ref[2]
        ref[1,1,0]=(0.940492459417)*x_ref[0]**o+(-0.0054460073194)*x_ref[0]+(-0.697916565168)*x_ref[1]**o+(-0.949456184725)*x_ref[1]+(-0.115461214789)*x_ref[2]**o+(-0.403742409948)*x_ref[2]
        ref[1,1,1]=(-0.711716124415)*x_ref[0]**o+(0.586131021871)*x_ref[0]+(0.625988666837)*x_ref[1]**o+(0.638433219635)*x_ref[1]+(0.971169174015)*x_ref[2]**o+(-0.064949626884)*x_ref[2]
        ref[2,0,0]=(-0.891973333325)*x_ref[0]**o+(0.76090478364)*x_ref[0]+(0.439956136239)*x_ref[1]**o+(-0.74807071296)*x_ref[1]+(-0.209999803467)*x_ref[2]**o+(0.404396254503)*x_ref[2]
        ref[2,0,1]=(0.559776288735)*x_ref[0]**o+(0.284643618707)*x_ref[0]+(0.0527799359967)*x_ref[1]**o+(-0.201660204363)*x_ref[1]+(0.590722655898)*x_ref[2]**o+(-0.24940757474)*x_ref[2]
        ref[2,1,0]=(-0.747588234432)*x_ref[0]**o+(0.0196702046746)*x_ref[0]+(0.0454238912973)*x_ref[1]**o+(0.174053915643)*x_ref[1]+(-0.868554688657)*x_ref[2]**o+(-0.127425849813)*x_ref[2]
        ref[2,1,1]=(-0.324276984008)*x_ref[0]**o+(-0.494912096003)*x_ref[0]+(-0.780953487415)*x_ref[1]**o+(0.225787179491)*x_ref[1]+(-0.760330660035)*x_ref[2]**o+(0.0440722700759)*x_ref[2]
        ref[3,0,0]=(0.37479028021)*x_ref[0]**o+(-0.869009335783)*x_ref[0]+(0.811673580204)*x_ref[1]**o+(0.521877505351)*x_ref[1]+(-0.508710979256)*x_ref[2]**o+(0.558085136638)*x_ref[2]
        ref[3,0,1]=(0.815143535066)*x_ref[0]**o+(0.867977183071)*x_ref[0]+(0.0315543942614)*x_ref[1]**o+(0.786379597601)*x_ref[1]+(0.0433279381339)*x_ref[2]**o+(-0.56554445624)*x_ref[2]
        ref[3,1,0]=(-0.948224238625)*x_ref[0]**o+(-0.759282528831)*x_ref[0]+(0.511802336753)*x_ref[1]**o+(0.331073342499)*x_ref[1]+(0.295241697427)*x_ref[2]**o+(0.154864664771)*x_ref[2]
        ref[3,1,1]=(-0.982634132939)*x_ref[0]**o+(-0.0762014431695)*x_ref[0]+(-0.0797462613351)*x_ref[1]**o+(-0.512305271957)*x_ref[1]+(-0.348883505418)*x_ref[2]**o+(0.00580412526927)*x_ref[2]
        ref[4,0,0]=(0.186597109908)*x_ref[0]**o+(0.051957206238)*x_ref[0]+(0.572882754553)*x_ref[1]**o+(-0.114338954084)*x_ref[1]+(-0.517239439578)*x_ref[2]**o+(-0.298287404607)*x_ref[2]
        ref[4,0,1]=(0.637628569107)*x_ref[0]**o+(0.228562669958)*x_ref[0]+(0.053657321561)*x_ref[1]**o+(0.762909608846)*x_ref[1]+(0.873936819574)*x_ref[2]**o+(-0.20000969514)*x_ref[2]
        ref[4,1,0]=(-0.717459489066)*x_ref[0]**o+(-0.462174191129)*x_ref[0]+(-0.70491180102)*x_ref[1]**o+(-0.406072010622)*x_ref[1]+(-0.903872259447)*x_ref[2]**o+(0.487491537826)*x_ref[2]
        ref[4,1,1]=(0.947654286923)*x_ref[0]**o+(-0.228830597313)*x_ref[0]+(-0.501442713655)*x_ref[1]**o+(0.282437368749)*x_ref[1]+(0.543592950002)*x_ref[2]**o+(0.5400198247)*x_ref[2]
        ref[5,0,0]=(0.144349494033)*x_ref[0]**o+(-0.635328703132)*x_ref[0]+(-0.797851162499)*x_ref[1]**o+(0.356874780185)*x_ref[1]+(-0.205725457569)*x_ref[2]**o+(-0.674962512018)*x_ref[2]
        ref[5,0,1]=(-0.728948504419)*x_ref[0]**o+(0.944337871242)*x_ref[0]+(-0.790388944678)*x_ref[1]**o+(0.59119693076)*x_ref[1]+(0.9569152246)*x_ref[2]**o+(-0.205404245177)*x_ref[2]
        ref[5,1,0]=(0.477597678348)*x_ref[0]**o+(-0.318993929605)*x_ref[0]+(0.481029970286)*x_ref[1]**o+(0.708696918395)*x_ref[1]+(0.0531716082887)*x_ref[2]**o+(-0.847697491061)*x_ref[2]
        ref[5,1,1]=(-0.458572001979)*x_ref[0]**o+(0.582048244354)*x_ref[0]+(-0.040756132547)*x_ref[1]**o+(-0.953201198076)*x_ref[1]+(-0.971859723822)*x_ref[2]**o+(-0.796426143181)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromData_ReducedFunctionOnContactOne_rank4(self):
      """
      tests interpolate for rank 4 Data onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactOne onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactOne(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5, 3, 2),w)
      ref=Data(0,(4, 5, 3, 2),w_ref)
      if dim==2:
        arg[0,0,0,0]=(-0.937777610663)*x[0]**o+(-0.0228520946632)*x[0]+(0.642385396583)*x[1]**o+(-0.803821945812)*x[1]
        arg[0,0,0,1]=(-0.398785307492)*x[0]**o+(0.0450288952883)*x[0]+(-0.777780548133)*x[1]**o+(0.0273772099414)*x[1]
        arg[0,0,1,0]=(-0.667280086925)*x[0]**o+(-0.934642710274)*x[0]+(0.565327494491)*x[1]**o+(0.988521769056)*x[1]
        arg[0,0,1,1]=(-0.225562383498)*x[0]**o+(-0.280566031134)*x[0]+(-0.295768954831)*x[1]**o+(0.102759013802)*x[1]
        arg[0,0,2,0]=(-0.179577502546)*x[0]**o+(-0.156932777074)*x[0]+(-0.145616153633)*x[1]**o+(0.971876899135)*x[1]
        arg[0,0,2,1]=(-0.840266409118)*x[0]**o+(0.704356785022)*x[0]+(0.740091470711)*x[1]**o+(0.708478499023)*x[1]
        arg[0,1,0,0]=(-0.487161662806)*x[0]**o+(0.565313908399)*x[0]+(-0.33708697293)*x[1]**o+(0.731675966546)*x[1]
        arg[0,1,0,1]=(-0.135615857963)*x[0]**o+(0.265408777025)*x[0]+(-0.168207260227)*x[1]**o+(-0.598974659317)*x[1]
        arg[0,1,1,0]=(0.0533944010258)*x[0]**o+(0.819248906738)*x[0]+(-0.837005251524)*x[1]**o+(0.119613618595)*x[1]
        arg[0,1,1,1]=(-0.257520627047)*x[0]**o+(0.339264545713)*x[0]+(0.137721228188)*x[1]**o+(-0.512251030979)*x[1]
        arg[0,1,2,0]=(-0.797070396749)*x[0]**o+(0.690173285077)*x[0]+(-0.213206187271)*x[1]**o+(-0.873999125945)*x[1]
        arg[0,1,2,1]=(-0.847662300367)*x[0]**o+(0.785781114266)*x[0]+(-0.456918398811)*x[1]**o+(0.458881079947)*x[1]
        arg[0,2,0,0]=(-0.385269163334)*x[0]**o+(0.135941228964)*x[0]+(0.233245620648)*x[1]**o+(-0.246282209626)*x[1]
        arg[0,2,0,1]=(0.372914636218)*x[0]**o+(-0.160283657003)*x[0]+(0.0565043889781)*x[1]**o+(0.199715522021)*x[1]
        arg[0,2,1,0]=(0.676198390598)*x[0]**o+(0.938025122014)*x[0]+(0.161074696651)*x[1]**o+(-0.155495646964)*x[1]
        arg[0,2,1,1]=(-0.709286924155)*x[0]**o+(-0.173716284776)*x[0]+(0.0576435484694)*x[1]**o+(0.673924697832)*x[1]
        arg[0,2,2,0]=(-0.436468652969)*x[0]**o+(0.117279009082)*x[0]+(-0.731436369191)*x[1]**o+(0.919578503469)*x[1]
        arg[0,2,2,1]=(-0.691042646151)*x[0]**o+(-0.606293725574)*x[0]+(0.657291692759)*x[1]**o+(-0.0146398994729)*x[1]
        arg[0,3,0,0]=(0.365939313719)*x[0]**o+(-0.221236913792)*x[0]+(-0.729594967883)*x[1]**o+(0.0179024719114)*x[1]
        arg[0,3,0,1]=(0.135892133258)*x[0]**o+(0.728069461936)*x[0]+(-0.365691795279)*x[1]**o+(0.877708599922)*x[1]
        arg[0,3,1,0]=(-0.853312456817)*x[0]**o+(-0.713031969049)*x[0]+(0.274755827464)*x[1]**o+(-0.527330699033)*x[1]
        arg[0,3,1,1]=(-0.30358584687)*x[0]**o+(0.871629391968)*x[0]+(0.906915162645)*x[1]**o+(0.499955429041)*x[1]
        arg[0,3,2,0]=(-0.880532587427)*x[0]**o+(0.265729672258)*x[0]+(0.528677136349)*x[1]**o+(-0.224602964925)*x[1]
        arg[0,3,2,1]=(0.158632463532)*x[0]**o+(-0.665418329643)*x[0]+(0.957661069067)*x[1]**o+(-0.23771309434)*x[1]
        arg[0,4,0,0]=(-0.128073514905)*x[0]**o+(-0.452543798055)*x[0]+(0.303808626277)*x[1]**o+(0.841041779759)*x[1]
        arg[0,4,0,1]=(0.0219309964781)*x[0]**o+(-0.949488393443)*x[0]+(0.0436905404663)*x[1]**o+(-0.304527242795)*x[1]
        arg[0,4,1,0]=(-0.792275137775)*x[0]**o+(0.450079829644)*x[0]+(0.162879981637)*x[1]**o+(-0.185839015805)*x[1]
        arg[0,4,1,1]=(0.761932713893)*x[0]**o+(0.932601232945)*x[0]+(-0.606328305783)*x[1]**o+(-0.123973545682)*x[1]
        arg[0,4,2,0]=(-0.514088304908)*x[0]**o+(0.987668774807)*x[0]+(0.866078887168)*x[1]**o+(-0.608518224484)*x[1]
        arg[0,4,2,1]=(-0.427564608028)*x[0]**o+(0.231599614445)*x[0]+(-0.406862229819)*x[1]**o+(0.358456864265)*x[1]
        arg[1,0,0,0]=(0.342092171801)*x[0]**o+(-0.264522675185)*x[0]+(-0.37976917867)*x[1]**o+(0.778596602561)*x[1]
        arg[1,0,0,1]=(0.672156473235)*x[0]**o+(0.320317500856)*x[0]+(0.514206881296)*x[1]**o+(0.602594631705)*x[1]
        arg[1,0,1,0]=(0.914472920293)*x[0]**o+(0.525647943447)*x[0]+(0.685613621597)*x[1]**o+(0.482308039231)*x[1]
        arg[1,0,1,1]=(-0.727982874425)*x[0]**o+(-0.417964463065)*x[0]+(0.825853988063)*x[1]**o+(0.236134760252)*x[1]
        arg[1,0,2,0]=(0.922186381756)*x[0]**o+(-0.140522762044)*x[0]+(-0.282169431666)*x[1]**o+(0.778383988397)*x[1]
        arg[1,0,2,1]=(0.307296736289)*x[0]**o+(0.589844868569)*x[0]+(0.64571457752)*x[1]**o+(0.749333079485)*x[1]
        arg[1,1,0,0]=(0.908345176703)*x[0]**o+(-0.476053147164)*x[0]+(0.0828501189289)*x[1]**o+(0.280657117705)*x[1]
        arg[1,1,0,1]=(0.396128814097)*x[0]**o+(0.368821575482)*x[0]+(-0.916584602334)*x[1]**o+(-0.149377992159)*x[1]
        arg[1,1,1,0]=(-0.38468474313)*x[0]**o+(0.304742499301)*x[0]+(0.638083865846)*x[1]**o+(-0.332046460663)*x[1]
        arg[1,1,1,1]=(0.845097787138)*x[0]**o+(-0.34949498164)*x[0]+(0.229157202572)*x[1]**o+(0.677584672856)*x[1]
        arg[1,1,2,0]=(0.0280692825358)*x[0]**o+(0.567451370107)*x[0]+(0.66934188305)*x[1]**o+(-0.477197213188)*x[1]
        arg[1,1,2,1]=(0.375411487264)*x[0]**o+(0.55086548839)*x[0]+(-0.392135351366)*x[1]**o+(0.91971334589)*x[1]
        arg[1,2,0,0]=(-0.7659270368)*x[0]**o+(-0.338082792509)*x[0]+(-0.26272975232)*x[1]**o+(0.267936454568)*x[1]
        arg[1,2,0,1]=(0.918125455771)*x[0]**o+(0.182896655906)*x[0]+(0.525396835337)*x[1]**o+(0.201918789801)*x[1]
        arg[1,2,1,0]=(-0.54472490266)*x[0]**o+(0.693714832775)*x[0]+(0.898278265794)*x[1]**o+(0.345095187159)*x[1]
        arg[1,2,1,1]=(-0.498607752195)*x[0]**o+(0.84678344253)*x[0]+(0.678992455984)*x[1]**o+(-0.597396262326)*x[1]
        arg[1,2,2,0]=(0.215267998554)*x[0]**o+(0.384479388838)*x[0]+(-0.09078862662)*x[1]**o+(-0.0162185695524)*x[1]
        arg[1,2,2,1]=(-0.335574750173)*x[0]**o+(-0.439933690512)*x[0]+(0.194601817488)*x[1]**o+(0.795181445633)*x[1]
        arg[1,3,0,0]=(0.862180273604)*x[0]**o+(0.276662657248)*x[0]+(-0.908827123328)*x[1]**o+(0.290406540669)*x[1]
        arg[1,3,0,1]=(-0.620441636176)*x[0]**o+(0.448236189204)*x[0]+(-0.503817375869)*x[1]**o+(0.689447469467)*x[1]
        arg[1,3,1,0]=(0.126327572417)*x[0]**o+(-0.476552913848)*x[0]+(0.362273273226)*x[1]**o+(-0.816228830372)*x[1]
        arg[1,3,1,1]=(-0.65091977023)*x[0]**o+(0.0316560718061)*x[0]+(-0.922906273452)*x[1]**o+(-0.584811809954)*x[1]
        arg[1,3,2,0]=(-0.675982262546)*x[0]**o+(0.23518662745)*x[0]+(-0.270229269162)*x[1]**o+(-0.00300656227686)*x[1]
        arg[1,3,2,1]=(0.55691633063)*x[0]**o+(-0.0643123929496)*x[0]+(0.212825067376)*x[1]**o+(-0.0807098593265)*x[1]
        arg[1,4,0,0]=(0.199100268435)*x[0]**o+(-0.358028564112)*x[0]+(0.130301315926)*x[1]**o+(-0.255752658178)*x[1]
        arg[1,4,0,1]=(0.821720568968)*x[0]**o+(0.193634063182)*x[0]+(0.862353589302)*x[1]**o+(-0.42971617129)*x[1]
        arg[1,4,1,0]=(0.51160977767)*x[0]**o+(0.326743318954)*x[0]+(-0.866878934278)*x[1]**o+(-0.927585776234)*x[1]
        arg[1,4,1,1]=(0.299835915861)*x[0]**o+(-0.969027084981)*x[0]+(-0.589772230832)*x[1]**o+(0.382433302029)*x[1]
        arg[1,4,2,0]=(0.473101160384)*x[0]**o+(-0.591124830283)*x[0]+(0.0929141413982)*x[1]**o+(-0.529811769054)*x[1]
        arg[1,4,2,1]=(-0.712363897962)*x[0]**o+(-0.259229026597)*x[0]+(0.00983651162685)*x[1]**o+(-0.329312308231)*x[1]
        arg[2,0,0,0]=(-0.507414918232)*x[0]**o+(0.668167664289)*x[0]+(0.654870826953)*x[1]**o+(-0.411963068213)*x[1]
        arg[2,0,0,1]=(-0.894915678458)*x[0]**o+(0.647579368695)*x[0]+(0.222274441569)*x[1]**o+(0.737972296743)*x[1]
        arg[2,0,1,0]=(-0.457176106717)*x[0]**o+(0.99997698232)*x[0]+(0.524381951875)*x[1]**o+(0.796302331418)*x[1]
        arg[2,0,1,1]=(-0.761912711581)*x[0]**o+(-0.995235985398)*x[0]+(-0.371926591709)*x[1]**o+(-0.37949019966)*x[1]
        arg[2,0,2,0]=(0.402826798397)*x[0]**o+(0.473547871218)*x[0]+(0.801636706721)*x[1]**o+(0.314221308918)*x[1]
        arg[2,0,2,1]=(0.0416846902183)*x[0]**o+(0.815306853966)*x[0]+(-0.526315166764)*x[1]**o+(-0.327935702076)*x[1]
        arg[2,1,0,0]=(-0.53829655239)*x[0]**o+(-0.467914397053)*x[0]+(0.392659650792)*x[1]**o+(0.309531884089)*x[1]
        arg[2,1,0,1]=(-0.768324371377)*x[0]**o+(0.136336455054)*x[0]+(0.0653485752909)*x[1]**o+(-0.0181923467911)*x[1]
        arg[2,1,1,0]=(0.121561771322)*x[0]**o+(-0.794575616334)*x[0]+(0.038194779569)*x[1]**o+(-0.695864614551)*x[1]
        arg[2,1,1,1]=(0.941872575674)*x[0]**o+(0.322067879395)*x[0]+(0.176873583908)*x[1]**o+(0.266964433323)*x[1]
        arg[2,1,2,0]=(0.324335817101)*x[0]**o+(0.609048475305)*x[0]+(0.350973789003)*x[1]**o+(-0.284995830372)*x[1]
        arg[2,1,2,1]=(0.939941132515)*x[0]**o+(0.782468124836)*x[0]+(-0.238539378134)*x[1]**o+(-0.515203802574)*x[1]
        arg[2,2,0,0]=(0.080179317168)*x[0]**o+(-0.555769741953)*x[0]+(0.283304726007)*x[1]**o+(0.844935300942)*x[1]
        arg[2,2,0,1]=(0.52001819752)*x[0]**o+(-0.105912273152)*x[0]+(-0.360523424917)*x[1]**o+(-0.179135541646)*x[1]
        arg[2,2,1,0]=(0.719593478448)*x[0]**o+(-0.722945454687)*x[0]+(0.0316404625439)*x[1]**o+(0.154290328891)*x[1]
        arg[2,2,1,1]=(0.906382302919)*x[0]**o+(-0.137753840588)*x[0]+(-0.626417865101)*x[1]**o+(-0.890642745149)*x[1]
        arg[2,2,2,0]=(0.618763086671)*x[0]**o+(-0.267876013516)*x[0]+(0.582353922702)*x[1]**o+(-0.190278788501)*x[1]
        arg[2,2,2,1]=(0.778824546516)*x[0]**o+(-0.00837218512008)*x[0]+(0.576376019084)*x[1]**o+(-0.173470710629)*x[1]
        arg[2,3,0,0]=(0.351742111948)*x[0]**o+(-0.641801256932)*x[0]+(-0.979232109277)*x[1]**o+(0.973197715414)*x[1]
        arg[2,3,0,1]=(-0.194493570182)*x[0]**o+(-0.24088269914)*x[0]+(-0.594521666623)*x[1]**o+(0.442357460052)*x[1]
        arg[2,3,1,0]=(-0.0677209625916)*x[0]**o+(-0.329246651504)*x[0]+(-0.200109031557)*x[1]**o+(0.0666064469254)*x[1]
        arg[2,3,1,1]=(-0.31026797404)*x[0]**o+(-0.0420778789656)*x[0]+(-0.534104800904)*x[1]**o+(0.940011883171)*x[1]
        arg[2,3,2,0]=(0.572825859789)*x[0]**o+(-0.439819027899)*x[0]+(-0.641214796724)*x[1]**o+(0.281317897435)*x[1]
        arg[2,3,2,1]=(0.410920070755)*x[0]**o+(-0.517259918184)*x[0]+(0.436420469387)*x[1]**o+(0.212276585342)*x[1]
        arg[2,4,0,0]=(0.0537540090315)*x[0]**o+(-0.362577123782)*x[0]+(-0.0134992480023)*x[1]**o+(0.59055317928)*x[1]
        arg[2,4,0,1]=(0.894896469792)*x[0]**o+(-0.594219811149)*x[0]+(0.755490579022)*x[1]**o+(-0.222913251691)*x[1]
        arg[2,4,1,0]=(-0.95471806813)*x[0]**o+(0.364260591437)*x[0]+(-0.51232745259)*x[1]**o+(-0.874795288001)*x[1]
        arg[2,4,1,1]=(-0.146483465086)*x[0]**o+(0.182495641134)*x[0]+(-0.132211746968)*x[1]**o+(0.39915899917)*x[1]
        arg[2,4,2,0]=(0.630856642463)*x[0]**o+(0.568438094437)*x[0]+(0.283670966088)*x[1]**o+(0.200206672418)*x[1]
        arg[2,4,2,1]=(-0.181938802717)*x[0]**o+(0.850684926475)*x[0]+(-0.765889361148)*x[1]**o+(0.750762858672)*x[1]
        arg[3,0,0,0]=(0.618955933151)*x[0]**o+(0.0280502885803)*x[0]+(-0.518282623282)*x[1]**o+(-0.741257199588)*x[1]
        arg[3,0,0,1]=(0.122462861306)*x[0]**o+(-0.103190806023)*x[0]+(0.647959918767)*x[1]**o+(0.531727405032)*x[1]
        arg[3,0,1,0]=(-0.129468664704)*x[0]**o+(0.342288003693)*x[0]+(-0.0268246756133)*x[1]**o+(0.671147920273)*x[1]
        arg[3,0,1,1]=(-0.99153395274)*x[0]**o+(0.584874556271)*x[0]+(-0.29273067599)*x[1]**o+(-0.802100630404)*x[1]
        arg[3,0,2,0]=(0.406033817569)*x[0]**o+(-0.664112618189)*x[0]+(0.970253482906)*x[1]**o+(0.979709129918)*x[1]
        arg[3,0,2,1]=(-0.708798148641)*x[0]**o+(-0.574259437301)*x[0]+(0.305353886124)*x[1]**o+(0.90520799029)*x[1]
        arg[3,1,0,0]=(-0.296777292529)*x[0]**o+(0.788817860427)*x[0]+(-0.817934426475)*x[1]**o+(-0.480405944642)*x[1]
        arg[3,1,0,1]=(0.382866653892)*x[0]**o+(-0.0911340124698)*x[0]+(0.449555911305)*x[1]**o+(0.972590525904)*x[1]
        arg[3,1,1,0]=(0.150349675921)*x[0]**o+(0.316401916978)*x[0]+(0.176755455021)*x[1]**o+(-0.895165552576)*x[1]
        arg[3,1,1,1]=(-0.334194603044)*x[0]**o+(0.40353334527)*x[0]+(0.510619331296)*x[1]**o+(-0.423857458769)*x[1]
        arg[3,1,2,0]=(0.876266380991)*x[0]**o+(-0.1146693887)*x[0]+(0.0388933985336)*x[1]**o+(0.537276050219)*x[1]
        arg[3,1,2,1]=(0.953489127278)*x[0]**o+(-0.40088917811)*x[0]+(0.322112015322)*x[1]**o+(0.570451651396)*x[1]
        arg[3,2,0,0]=(-0.392674815471)*x[0]**o+(0.805725364633)*x[0]+(-0.550131544344)*x[1]**o+(-0.902042986324)*x[1]
        arg[3,2,0,1]=(0.996009652754)*x[0]**o+(0.534022968708)*x[0]+(-0.251810035763)*x[1]**o+(-0.324231235105)*x[1]
        arg[3,2,1,0]=(-0.280691594637)*x[0]**o+(0.716441336919)*x[0]+(0.444717298591)*x[1]**o+(-0.417682576675)*x[1]
        arg[3,2,1,1]=(-0.528033696021)*x[0]**o+(0.105118392268)*x[0]+(0.219345332231)*x[1]**o+(-0.554011705858)*x[1]
        arg[3,2,2,0]=(0.869967952526)*x[0]**o+(-0.285204973916)*x[0]+(0.676958471189)*x[1]**o+(-0.109813007662)*x[1]
        arg[3,2,2,1]=(-0.183338615818)*x[0]**o+(0.124214512553)*x[0]+(-0.00262420395052)*x[1]**o+(0.505099322191)*x[1]
        arg[3,3,0,0]=(-0.77019083751)*x[0]**o+(-0.288304317474)*x[0]+(-0.674724525333)*x[1]**o+(0.6230658909)*x[1]
        arg[3,3,0,1]=(0.4027332007)*x[0]**o+(0.0181383553658)*x[0]+(0.0346834190431)*x[1]**o+(-0.343302420746)*x[1]
        arg[3,3,1,0]=(0.0907452191096)*x[0]**o+(-0.734197819823)*x[0]+(-0.262536956938)*x[1]**o+(-0.838964523281)*x[1]
        arg[3,3,1,1]=(0.362001847336)*x[0]**o+(-0.99196319473)*x[0]+(-0.544565811529)*x[1]**o+(0.0460139495686)*x[1]
        arg[3,3,2,0]=(0.43659913886)*x[0]**o+(-0.42986865456)*x[0]+(-0.298749218949)*x[1]**o+(0.927334871957)*x[1]
        arg[3,3,2,1]=(0.400272592051)*x[0]**o+(0.332756541456)*x[0]+(0.971127734155)*x[1]**o+(0.653965425034)*x[1]
        arg[3,4,0,0]=(0.663781122153)*x[0]**o+(0.697898766724)*x[0]+(0.157432061303)*x[1]**o+(-0.0526003145497)*x[1]
        arg[3,4,0,1]=(-0.463791084222)*x[0]**o+(0.384914785855)*x[0]+(-0.996485418809)*x[1]**o+(-0.519106290889)*x[1]
        arg[3,4,1,0]=(-0.598068247392)*x[0]**o+(0.553482523952)*x[0]+(0.202485340343)*x[1]**o+(-0.540515409196)*x[1]
        arg[3,4,1,1]=(0.134764979841)*x[0]**o+(0.0141136152488)*x[0]+(-0.536476467366)*x[1]**o+(0.323150057577)*x[1]
        arg[3,4,2,0]=(-0.73096888708)*x[0]**o+(0.485529166207)*x[0]+(-0.541219829073)*x[1]**o+(0.471443044424)*x[1]
        arg[3,4,2,1]=(-0.815808733094)*x[0]**o+(0.694490460113)*x[0]+(0.292301816313)*x[1]**o+(0.700014734893)*x[1]
        ref[0,0,0,0]=(-0.937777610663)*x_ref[0]**o+(-0.0228520946632)*x_ref[0]+(0.642385396583)*x_ref[1]**o+(-0.803821945812)*x_ref[1]
        ref[0,0,0,1]=(-0.398785307492)*x_ref[0]**o+(0.0450288952883)*x_ref[0]+(-0.777780548133)*x_ref[1]**o+(0.0273772099414)*x_ref[1]
        ref[0,0,1,0]=(-0.667280086925)*x_ref[0]**o+(-0.934642710274)*x_ref[0]+(0.565327494491)*x_ref[1]**o+(0.988521769056)*x_ref[1]
        ref[0,0,1,1]=(-0.225562383498)*x_ref[0]**o+(-0.280566031134)*x_ref[0]+(-0.295768954831)*x_ref[1]**o+(0.102759013802)*x_ref[1]
        ref[0,0,2,0]=(-0.179577502546)*x_ref[0]**o+(-0.156932777074)*x_ref[0]+(-0.145616153633)*x_ref[1]**o+(0.971876899135)*x_ref[1]
        ref[0,0,2,1]=(-0.840266409118)*x_ref[0]**o+(0.704356785022)*x_ref[0]+(0.740091470711)*x_ref[1]**o+(0.708478499023)*x_ref[1]
        ref[0,1,0,0]=(-0.487161662806)*x_ref[0]**o+(0.565313908399)*x_ref[0]+(-0.33708697293)*x_ref[1]**o+(0.731675966546)*x_ref[1]
        ref[0,1,0,1]=(-0.135615857963)*x_ref[0]**o+(0.265408777025)*x_ref[0]+(-0.168207260227)*x_ref[1]**o+(-0.598974659317)*x_ref[1]
        ref[0,1,1,0]=(0.0533944010258)*x_ref[0]**o+(0.819248906738)*x_ref[0]+(-0.837005251524)*x_ref[1]**o+(0.119613618595)*x_ref[1]
        ref[0,1,1,1]=(-0.257520627047)*x_ref[0]**o+(0.339264545713)*x_ref[0]+(0.137721228188)*x_ref[1]**o+(-0.512251030979)*x_ref[1]
        ref[0,1,2,0]=(-0.797070396749)*x_ref[0]**o+(0.690173285077)*x_ref[0]+(-0.213206187271)*x_ref[1]**o+(-0.873999125945)*x_ref[1]
        ref[0,1,2,1]=(-0.847662300367)*x_ref[0]**o+(0.785781114266)*x_ref[0]+(-0.456918398811)*x_ref[1]**o+(0.458881079947)*x_ref[1]
        ref[0,2,0,0]=(-0.385269163334)*x_ref[0]**o+(0.135941228964)*x_ref[0]+(0.233245620648)*x_ref[1]**o+(-0.246282209626)*x_ref[1]
        ref[0,2,0,1]=(0.372914636218)*x_ref[0]**o+(-0.160283657003)*x_ref[0]+(0.0565043889781)*x_ref[1]**o+(0.199715522021)*x_ref[1]
        ref[0,2,1,0]=(0.676198390598)*x_ref[0]**o+(0.938025122014)*x_ref[0]+(0.161074696651)*x_ref[1]**o+(-0.155495646964)*x_ref[1]
        ref[0,2,1,1]=(-0.709286924155)*x_ref[0]**o+(-0.173716284776)*x_ref[0]+(0.0576435484694)*x_ref[1]**o+(0.673924697832)*x_ref[1]
        ref[0,2,2,0]=(-0.436468652969)*x_ref[0]**o+(0.117279009082)*x_ref[0]+(-0.731436369191)*x_ref[1]**o+(0.919578503469)*x_ref[1]
        ref[0,2,2,1]=(-0.691042646151)*x_ref[0]**o+(-0.606293725574)*x_ref[0]+(0.657291692759)*x_ref[1]**o+(-0.0146398994729)*x_ref[1]
        ref[0,3,0,0]=(0.365939313719)*x_ref[0]**o+(-0.221236913792)*x_ref[0]+(-0.729594967883)*x_ref[1]**o+(0.0179024719114)*x_ref[1]
        ref[0,3,0,1]=(0.135892133258)*x_ref[0]**o+(0.728069461936)*x_ref[0]+(-0.365691795279)*x_ref[1]**o+(0.877708599922)*x_ref[1]
        ref[0,3,1,0]=(-0.853312456817)*x_ref[0]**o+(-0.713031969049)*x_ref[0]+(0.274755827464)*x_ref[1]**o+(-0.527330699033)*x_ref[1]
        ref[0,3,1,1]=(-0.30358584687)*x_ref[0]**o+(0.871629391968)*x_ref[0]+(0.906915162645)*x_ref[1]**o+(0.499955429041)*x_ref[1]
        ref[0,3,2,0]=(-0.880532587427)*x_ref[0]**o+(0.265729672258)*x_ref[0]+(0.528677136349)*x_ref[1]**o+(-0.224602964925)*x_ref[1]
        ref[0,3,2,1]=(0.158632463532)*x_ref[0]**o+(-0.665418329643)*x_ref[0]+(0.957661069067)*x_ref[1]**o+(-0.23771309434)*x_ref[1]
        ref[0,4,0,0]=(-0.128073514905)*x_ref[0]**o+(-0.452543798055)*x_ref[0]+(0.303808626277)*x_ref[1]**o+(0.841041779759)*x_ref[1]
        ref[0,4,0,1]=(0.0219309964781)*x_ref[0]**o+(-0.949488393443)*x_ref[0]+(0.0436905404663)*x_ref[1]**o+(-0.304527242795)*x_ref[1]
        ref[0,4,1,0]=(-0.792275137775)*x_ref[0]**o+(0.450079829644)*x_ref[0]+(0.162879981637)*x_ref[1]**o+(-0.185839015805)*x_ref[1]
        ref[0,4,1,1]=(0.761932713893)*x_ref[0]**o+(0.932601232945)*x_ref[0]+(-0.606328305783)*x_ref[1]**o+(-0.123973545682)*x_ref[1]
        ref[0,4,2,0]=(-0.514088304908)*x_ref[0]**o+(0.987668774807)*x_ref[0]+(0.866078887168)*x_ref[1]**o+(-0.608518224484)*x_ref[1]
        ref[0,4,2,1]=(-0.427564608028)*x_ref[0]**o+(0.231599614445)*x_ref[0]+(-0.406862229819)*x_ref[1]**o+(0.358456864265)*x_ref[1]
        ref[1,0,0,0]=(0.342092171801)*x_ref[0]**o+(-0.264522675185)*x_ref[0]+(-0.37976917867)*x_ref[1]**o+(0.778596602561)*x_ref[1]
        ref[1,0,0,1]=(0.672156473235)*x_ref[0]**o+(0.320317500856)*x_ref[0]+(0.514206881296)*x_ref[1]**o+(0.602594631705)*x_ref[1]
        ref[1,0,1,0]=(0.914472920293)*x_ref[0]**o+(0.525647943447)*x_ref[0]+(0.685613621597)*x_ref[1]**o+(0.482308039231)*x_ref[1]
        ref[1,0,1,1]=(-0.727982874425)*x_ref[0]**o+(-0.417964463065)*x_ref[0]+(0.825853988063)*x_ref[1]**o+(0.236134760252)*x_ref[1]
        ref[1,0,2,0]=(0.922186381756)*x_ref[0]**o+(-0.140522762044)*x_ref[0]+(-0.282169431666)*x_ref[1]**o+(0.778383988397)*x_ref[1]
        ref[1,0,2,1]=(0.307296736289)*x_ref[0]**o+(0.589844868569)*x_ref[0]+(0.64571457752)*x_ref[1]**o+(0.749333079485)*x_ref[1]
        ref[1,1,0,0]=(0.908345176703)*x_ref[0]**o+(-0.476053147164)*x_ref[0]+(0.0828501189289)*x_ref[1]**o+(0.280657117705)*x_ref[1]
        ref[1,1,0,1]=(0.396128814097)*x_ref[0]**o+(0.368821575482)*x_ref[0]+(-0.916584602334)*x_ref[1]**o+(-0.149377992159)*x_ref[1]
        ref[1,1,1,0]=(-0.38468474313)*x_ref[0]**o+(0.304742499301)*x_ref[0]+(0.638083865846)*x_ref[1]**o+(-0.332046460663)*x_ref[1]
        ref[1,1,1,1]=(0.845097787138)*x_ref[0]**o+(-0.34949498164)*x_ref[0]+(0.229157202572)*x_ref[1]**o+(0.677584672856)*x_ref[1]
        ref[1,1,2,0]=(0.0280692825358)*x_ref[0]**o+(0.567451370107)*x_ref[0]+(0.66934188305)*x_ref[1]**o+(-0.477197213188)*x_ref[1]
        ref[1,1,2,1]=(0.375411487264)*x_ref[0]**o+(0.55086548839)*x_ref[0]+(-0.392135351366)*x_ref[1]**o+(0.91971334589)*x_ref[1]
        ref[1,2,0,0]=(-0.7659270368)*x_ref[0]**o+(-0.338082792509)*x_ref[0]+(-0.26272975232)*x_ref[1]**o+(0.267936454568)*x_ref[1]
        ref[1,2,0,1]=(0.918125455771)*x_ref[0]**o+(0.182896655906)*x_ref[0]+(0.525396835337)*x_ref[1]**o+(0.201918789801)*x_ref[1]
        ref[1,2,1,0]=(-0.54472490266)*x_ref[0]**o+(0.693714832775)*x_ref[0]+(0.898278265794)*x_ref[1]**o+(0.345095187159)*x_ref[1]
        ref[1,2,1,1]=(-0.498607752195)*x_ref[0]**o+(0.84678344253)*x_ref[0]+(0.678992455984)*x_ref[1]**o+(-0.597396262326)*x_ref[1]
        ref[1,2,2,0]=(0.215267998554)*x_ref[0]**o+(0.384479388838)*x_ref[0]+(-0.09078862662)*x_ref[1]**o+(-0.0162185695524)*x_ref[1]
        ref[1,2,2,1]=(-0.335574750173)*x_ref[0]**o+(-0.439933690512)*x_ref[0]+(0.194601817488)*x_ref[1]**o+(0.795181445633)*x_ref[1]
        ref[1,3,0,0]=(0.862180273604)*x_ref[0]**o+(0.276662657248)*x_ref[0]+(-0.908827123328)*x_ref[1]**o+(0.290406540669)*x_ref[1]
        ref[1,3,0,1]=(-0.620441636176)*x_ref[0]**o+(0.448236189204)*x_ref[0]+(-0.503817375869)*x_ref[1]**o+(0.689447469467)*x_ref[1]
        ref[1,3,1,0]=(0.126327572417)*x_ref[0]**o+(-0.476552913848)*x_ref[0]+(0.362273273226)*x_ref[1]**o+(-0.816228830372)*x_ref[1]
        ref[1,3,1,1]=(-0.65091977023)*x_ref[0]**o+(0.0316560718061)*x_ref[0]+(-0.922906273452)*x_ref[1]**o+(-0.584811809954)*x_ref[1]
        ref[1,3,2,0]=(-0.675982262546)*x_ref[0]**o+(0.23518662745)*x_ref[0]+(-0.270229269162)*x_ref[1]**o+(-0.00300656227686)*x_ref[1]
        ref[1,3,2,1]=(0.55691633063)*x_ref[0]**o+(-0.0643123929496)*x_ref[0]+(0.212825067376)*x_ref[1]**o+(-0.0807098593265)*x_ref[1]
        ref[1,4,0,0]=(0.199100268435)*x_ref[0]**o+(-0.358028564112)*x_ref[0]+(0.130301315926)*x_ref[1]**o+(-0.255752658178)*x_ref[1]
        ref[1,4,0,1]=(0.821720568968)*x_ref[0]**o+(0.193634063182)*x_ref[0]+(0.862353589302)*x_ref[1]**o+(-0.42971617129)*x_ref[1]
        ref[1,4,1,0]=(0.51160977767)*x_ref[0]**o+(0.326743318954)*x_ref[0]+(-0.866878934278)*x_ref[1]**o+(-0.927585776234)*x_ref[1]
        ref[1,4,1,1]=(0.299835915861)*x_ref[0]**o+(-0.969027084981)*x_ref[0]+(-0.589772230832)*x_ref[1]**o+(0.382433302029)*x_ref[1]
        ref[1,4,2,0]=(0.473101160384)*x_ref[0]**o+(-0.591124830283)*x_ref[0]+(0.0929141413982)*x_ref[1]**o+(-0.529811769054)*x_ref[1]
        ref[1,4,2,1]=(-0.712363897962)*x_ref[0]**o+(-0.259229026597)*x_ref[0]+(0.00983651162685)*x_ref[1]**o+(-0.329312308231)*x_ref[1]
        ref[2,0,0,0]=(-0.507414918232)*x_ref[0]**o+(0.668167664289)*x_ref[0]+(0.654870826953)*x_ref[1]**o+(-0.411963068213)*x_ref[1]
        ref[2,0,0,1]=(-0.894915678458)*x_ref[0]**o+(0.647579368695)*x_ref[0]+(0.222274441569)*x_ref[1]**o+(0.737972296743)*x_ref[1]
        ref[2,0,1,0]=(-0.457176106717)*x_ref[0]**o+(0.99997698232)*x_ref[0]+(0.524381951875)*x_ref[1]**o+(0.796302331418)*x_ref[1]
        ref[2,0,1,1]=(-0.761912711581)*x_ref[0]**o+(-0.995235985398)*x_ref[0]+(-0.371926591709)*x_ref[1]**o+(-0.37949019966)*x_ref[1]
        ref[2,0,2,0]=(0.402826798397)*x_ref[0]**o+(0.473547871218)*x_ref[0]+(0.801636706721)*x_ref[1]**o+(0.314221308918)*x_ref[1]
        ref[2,0,2,1]=(0.0416846902183)*x_ref[0]**o+(0.815306853966)*x_ref[0]+(-0.526315166764)*x_ref[1]**o+(-0.327935702076)*x_ref[1]
        ref[2,1,0,0]=(-0.53829655239)*x_ref[0]**o+(-0.467914397053)*x_ref[0]+(0.392659650792)*x_ref[1]**o+(0.309531884089)*x_ref[1]
        ref[2,1,0,1]=(-0.768324371377)*x_ref[0]**o+(0.136336455054)*x_ref[0]+(0.0653485752909)*x_ref[1]**o+(-0.0181923467911)*x_ref[1]
        ref[2,1,1,0]=(0.121561771322)*x_ref[0]**o+(-0.794575616334)*x_ref[0]+(0.038194779569)*x_ref[1]**o+(-0.695864614551)*x_ref[1]
        ref[2,1,1,1]=(0.941872575674)*x_ref[0]**o+(0.322067879395)*x_ref[0]+(0.176873583908)*x_ref[1]**o+(0.266964433323)*x_ref[1]
        ref[2,1,2,0]=(0.324335817101)*x_ref[0]**o+(0.609048475305)*x_ref[0]+(0.350973789003)*x_ref[1]**o+(-0.284995830372)*x_ref[1]
        ref[2,1,2,1]=(0.939941132515)*x_ref[0]**o+(0.782468124836)*x_ref[0]+(-0.238539378134)*x_ref[1]**o+(-0.515203802574)*x_ref[1]
        ref[2,2,0,0]=(0.080179317168)*x_ref[0]**o+(-0.555769741953)*x_ref[0]+(0.283304726007)*x_ref[1]**o+(0.844935300942)*x_ref[1]
        ref[2,2,0,1]=(0.52001819752)*x_ref[0]**o+(-0.105912273152)*x_ref[0]+(-0.360523424917)*x_ref[1]**o+(-0.179135541646)*x_ref[1]
        ref[2,2,1,0]=(0.719593478448)*x_ref[0]**o+(-0.722945454687)*x_ref[0]+(0.0316404625439)*x_ref[1]**o+(0.154290328891)*x_ref[1]
        ref[2,2,1,1]=(0.906382302919)*x_ref[0]**o+(-0.137753840588)*x_ref[0]+(-0.626417865101)*x_ref[1]**o+(-0.890642745149)*x_ref[1]
        ref[2,2,2,0]=(0.618763086671)*x_ref[0]**o+(-0.267876013516)*x_ref[0]+(0.582353922702)*x_ref[1]**o+(-0.190278788501)*x_ref[1]
        ref[2,2,2,1]=(0.778824546516)*x_ref[0]**o+(-0.00837218512008)*x_ref[0]+(0.576376019084)*x_ref[1]**o+(-0.173470710629)*x_ref[1]
        ref[2,3,0,0]=(0.351742111948)*x_ref[0]**o+(-0.641801256932)*x_ref[0]+(-0.979232109277)*x_ref[1]**o+(0.973197715414)*x_ref[1]
        ref[2,3,0,1]=(-0.194493570182)*x_ref[0]**o+(-0.24088269914)*x_ref[0]+(-0.594521666623)*x_ref[1]**o+(0.442357460052)*x_ref[1]
        ref[2,3,1,0]=(-0.0677209625916)*x_ref[0]**o+(-0.329246651504)*x_ref[0]+(-0.200109031557)*x_ref[1]**o+(0.0666064469254)*x_ref[1]
        ref[2,3,1,1]=(-0.31026797404)*x_ref[0]**o+(-0.0420778789656)*x_ref[0]+(-0.534104800904)*x_ref[1]**o+(0.940011883171)*x_ref[1]
        ref[2,3,2,0]=(0.572825859789)*x_ref[0]**o+(-0.439819027899)*x_ref[0]+(-0.641214796724)*x_ref[1]**o+(0.281317897435)*x_ref[1]
        ref[2,3,2,1]=(0.410920070755)*x_ref[0]**o+(-0.517259918184)*x_ref[0]+(0.436420469387)*x_ref[1]**o+(0.212276585342)*x_ref[1]
        ref[2,4,0,0]=(0.0537540090315)*x_ref[0]**o+(-0.362577123782)*x_ref[0]+(-0.0134992480023)*x_ref[1]**o+(0.59055317928)*x_ref[1]
        ref[2,4,0,1]=(0.894896469792)*x_ref[0]**o+(-0.594219811149)*x_ref[0]+(0.755490579022)*x_ref[1]**o+(-0.222913251691)*x_ref[1]
        ref[2,4,1,0]=(-0.95471806813)*x_ref[0]**o+(0.364260591437)*x_ref[0]+(-0.51232745259)*x_ref[1]**o+(-0.874795288001)*x_ref[1]
        ref[2,4,1,1]=(-0.146483465086)*x_ref[0]**o+(0.182495641134)*x_ref[0]+(-0.132211746968)*x_ref[1]**o+(0.39915899917)*x_ref[1]
        ref[2,4,2,0]=(0.630856642463)*x_ref[0]**o+(0.568438094437)*x_ref[0]+(0.283670966088)*x_ref[1]**o+(0.200206672418)*x_ref[1]
        ref[2,4,2,1]=(-0.181938802717)*x_ref[0]**o+(0.850684926475)*x_ref[0]+(-0.765889361148)*x_ref[1]**o+(0.750762858672)*x_ref[1]
        ref[3,0,0,0]=(0.618955933151)*x_ref[0]**o+(0.0280502885803)*x_ref[0]+(-0.518282623282)*x_ref[1]**o+(-0.741257199588)*x_ref[1]
        ref[3,0,0,1]=(0.122462861306)*x_ref[0]**o+(-0.103190806023)*x_ref[0]+(0.647959918767)*x_ref[1]**o+(0.531727405032)*x_ref[1]
        ref[3,0,1,0]=(-0.129468664704)*x_ref[0]**o+(0.342288003693)*x_ref[0]+(-0.0268246756133)*x_ref[1]**o+(0.671147920273)*x_ref[1]
        ref[3,0,1,1]=(-0.99153395274)*x_ref[0]**o+(0.584874556271)*x_ref[0]+(-0.29273067599)*x_ref[1]**o+(-0.802100630404)*x_ref[1]
        ref[3,0,2,0]=(0.406033817569)*x_ref[0]**o+(-0.664112618189)*x_ref[0]+(0.970253482906)*x_ref[1]**o+(0.979709129918)*x_ref[1]
        ref[3,0,2,1]=(-0.708798148641)*x_ref[0]**o+(-0.574259437301)*x_ref[0]+(0.305353886124)*x_ref[1]**o+(0.90520799029)*x_ref[1]
        ref[3,1,0,0]=(-0.296777292529)*x_ref[0]**o+(0.788817860427)*x_ref[0]+(-0.817934426475)*x_ref[1]**o+(-0.480405944642)*x_ref[1]
        ref[3,1,0,1]=(0.382866653892)*x_ref[0]**o+(-0.0911340124698)*x_ref[0]+(0.449555911305)*x_ref[1]**o+(0.972590525904)*x_ref[1]
        ref[3,1,1,0]=(0.150349675921)*x_ref[0]**o+(0.316401916978)*x_ref[0]+(0.176755455021)*x_ref[1]**o+(-0.895165552576)*x_ref[1]
        ref[3,1,1,1]=(-0.334194603044)*x_ref[0]**o+(0.40353334527)*x_ref[0]+(0.510619331296)*x_ref[1]**o+(-0.423857458769)*x_ref[1]
        ref[3,1,2,0]=(0.876266380991)*x_ref[0]**o+(-0.1146693887)*x_ref[0]+(0.0388933985336)*x_ref[1]**o+(0.537276050219)*x_ref[1]
        ref[3,1,2,1]=(0.953489127278)*x_ref[0]**o+(-0.40088917811)*x_ref[0]+(0.322112015322)*x_ref[1]**o+(0.570451651396)*x_ref[1]
        ref[3,2,0,0]=(-0.392674815471)*x_ref[0]**o+(0.805725364633)*x_ref[0]+(-0.550131544344)*x_ref[1]**o+(-0.902042986324)*x_ref[1]
        ref[3,2,0,1]=(0.996009652754)*x_ref[0]**o+(0.534022968708)*x_ref[0]+(-0.251810035763)*x_ref[1]**o+(-0.324231235105)*x_ref[1]
        ref[3,2,1,0]=(-0.280691594637)*x_ref[0]**o+(0.716441336919)*x_ref[0]+(0.444717298591)*x_ref[1]**o+(-0.417682576675)*x_ref[1]
        ref[3,2,1,1]=(-0.528033696021)*x_ref[0]**o+(0.105118392268)*x_ref[0]+(0.219345332231)*x_ref[1]**o+(-0.554011705858)*x_ref[1]
        ref[3,2,2,0]=(0.869967952526)*x_ref[0]**o+(-0.285204973916)*x_ref[0]+(0.676958471189)*x_ref[1]**o+(-0.109813007662)*x_ref[1]
        ref[3,2,2,1]=(-0.183338615818)*x_ref[0]**o+(0.124214512553)*x_ref[0]+(-0.00262420395052)*x_ref[1]**o+(0.505099322191)*x_ref[1]
        ref[3,3,0,0]=(-0.77019083751)*x_ref[0]**o+(-0.288304317474)*x_ref[0]+(-0.674724525333)*x_ref[1]**o+(0.6230658909)*x_ref[1]
        ref[3,3,0,1]=(0.4027332007)*x_ref[0]**o+(0.0181383553658)*x_ref[0]+(0.0346834190431)*x_ref[1]**o+(-0.343302420746)*x_ref[1]
        ref[3,3,1,0]=(0.0907452191096)*x_ref[0]**o+(-0.734197819823)*x_ref[0]+(-0.262536956938)*x_ref[1]**o+(-0.838964523281)*x_ref[1]
        ref[3,3,1,1]=(0.362001847336)*x_ref[0]**o+(-0.99196319473)*x_ref[0]+(-0.544565811529)*x_ref[1]**o+(0.0460139495686)*x_ref[1]
        ref[3,3,2,0]=(0.43659913886)*x_ref[0]**o+(-0.42986865456)*x_ref[0]+(-0.298749218949)*x_ref[1]**o+(0.927334871957)*x_ref[1]
        ref[3,3,2,1]=(0.400272592051)*x_ref[0]**o+(0.332756541456)*x_ref[0]+(0.971127734155)*x_ref[1]**o+(0.653965425034)*x_ref[1]
        ref[3,4,0,0]=(0.663781122153)*x_ref[0]**o+(0.697898766724)*x_ref[0]+(0.157432061303)*x_ref[1]**o+(-0.0526003145497)*x_ref[1]
        ref[3,4,0,1]=(-0.463791084222)*x_ref[0]**o+(0.384914785855)*x_ref[0]+(-0.996485418809)*x_ref[1]**o+(-0.519106290889)*x_ref[1]
        ref[3,4,1,0]=(-0.598068247392)*x_ref[0]**o+(0.553482523952)*x_ref[0]+(0.202485340343)*x_ref[1]**o+(-0.540515409196)*x_ref[1]
        ref[3,4,1,1]=(0.134764979841)*x_ref[0]**o+(0.0141136152488)*x_ref[0]+(-0.536476467366)*x_ref[1]**o+(0.323150057577)*x_ref[1]
        ref[3,4,2,0]=(-0.73096888708)*x_ref[0]**o+(0.485529166207)*x_ref[0]+(-0.541219829073)*x_ref[1]**o+(0.471443044424)*x_ref[1]
        ref[3,4,2,1]=(-0.815808733094)*x_ref[0]**o+(0.694490460113)*x_ref[0]+(0.292301816313)*x_ref[1]**o+(0.700014734893)*x_ref[1]
      else:
        arg[0,0,0,0]=(0.244520699379)*x[0]**o+(-0.125771252842)*x[0]+(0.654557798928)*x[1]**o+(0.438110948503)*x[1]+(0.399996169353)*x[2]**o+(0.37037141324)*x[2]
        arg[0,0,0,1]=(0.019693348021)*x[0]**o+(-0.509938412509)*x[0]+(-0.425852907243)*x[1]**o+(0.351462945072)*x[1]+(0.628410872426)*x[2]**o+(0.4769252017)*x[2]
        arg[0,0,1,0]=(0.623813767602)*x[0]**o+(-0.940818977456)*x[0]+(-0.00111063869285)*x[1]**o+(-0.577558081394)*x[1]+(-0.30810971068)*x[2]**o+(-0.660149007373)*x[2]
        arg[0,0,1,1]=(0.994105796371)*x[0]**o+(0.199970906966)*x[0]+(0.961509001434)*x[1]**o+(-0.783069602127)*x[1]+(-0.179922863021)*x[2]**o+(-0.386974503089)*x[2]
        arg[0,0,2,0]=(0.932745137122)*x[0]**o+(0.627995830114)*x[0]+(-0.334344490736)*x[1]**o+(0.733210590815)*x[1]+(-0.23424959051)*x[2]**o+(0.140644214958)*x[2]
        arg[0,0,2,1]=(-0.282836519247)*x[0]**o+(-0.744077669768)*x[0]+(0.123898574243)*x[1]**o+(0.562384289004)*x[1]+(0.0757927394637)*x[2]**o+(0.767684067921)*x[2]
        arg[0,1,0,0]=(0.212366202901)*x[0]**o+(-0.679228304269)*x[0]+(0.829706652419)*x[1]**o+(0.24389996073)*x[1]+(0.909367025978)*x[2]**o+(0.280617790407)*x[2]
        arg[0,1,0,1]=(0.760070952632)*x[0]**o+(-0.678309252372)*x[0]+(0.0372281910363)*x[1]**o+(0.0284582814297)*x[1]+(-0.092349250941)*x[2]**o+(0.347713645921)*x[2]
        arg[0,1,1,0]=(0.248410230191)*x[0]**o+(-0.833983202295)*x[0]+(-0.759357278899)*x[1]**o+(0.880038285893)*x[1]+(-0.734799540652)*x[2]**o+(0.493330231104)*x[2]
        arg[0,1,1,1]=(0.0359919724228)*x[0]**o+(-0.0393899014057)*x[0]+(0.185155782488)*x[1]**o+(-0.92692925719)*x[1]+(0.74718037404)*x[2]**o+(-0.283900681763)*x[2]
        arg[0,1,2,0]=(-0.208643544667)*x[0]**o+(0.368200149096)*x[0]+(-0.327032725172)*x[1]**o+(-0.245547905792)*x[1]+(0.0886488214242)*x[2]**o+(0.179139897691)*x[2]
        arg[0,1,2,1]=(-0.0618762861459)*x[0]**o+(0.253419217897)*x[0]+(0.840270125508)*x[1]**o+(0.0223556878387)*x[1]+(0.0560904700176)*x[2]**o+(-0.646231049424)*x[2]
        arg[0,2,0,0]=(0.568966969865)*x[0]**o+(0.786776511348)*x[0]+(0.150595926116)*x[1]**o+(-0.780164417762)*x[1]+(0.510149831242)*x[2]**o+(-0.902871929903)*x[2]
        arg[0,2,0,1]=(-0.817062669957)*x[0]**o+(0.626805486868)*x[0]+(-0.566129786829)*x[1]**o+(0.248418548905)*x[1]+(-0.273995326019)*x[2]**o+(0.402023521018)*x[2]
        arg[0,2,1,0]=(-0.160276235113)*x[0]**o+(-0.0732543937053)*x[0]+(-0.0651595982698)*x[1]**o+(-0.529463476237)*x[1]+(0.346098812945)*x[2]**o+(0.418367016439)*x[2]
        arg[0,2,1,1]=(0.736907408498)*x[0]**o+(-0.123612748157)*x[0]+(0.000834659108651)*x[1]**o+(0.568833761363)*x[1]+(-0.542676953582)*x[2]**o+(0.026182553484)*x[2]
        arg[0,2,2,0]=(0.320985918151)*x[0]**o+(0.0512654311644)*x[0]+(-0.0313412913713)*x[1]**o+(0.295659930913)*x[1]+(-0.989560405329)*x[2]**o+(-0.107344246298)*x[2]
        arg[0,2,2,1]=(-0.957793939949)*x[0]**o+(-0.177591880789)*x[0]+(0.0785829772689)*x[1]**o+(0.121445956612)*x[1]+(-0.384134176508)*x[2]**o+(-0.775433914622)*x[2]
        arg[0,3,0,0]=(-0.130620609021)*x[0]**o+(0.268283814564)*x[0]+(-0.192835222037)*x[1]**o+(0.235341688541)*x[1]+(0.957069315916)*x[2]**o+(-0.708853567529)*x[2]
        arg[0,3,0,1]=(-0.377028600614)*x[0]**o+(-0.858446162753)*x[0]+(-0.850630580491)*x[1]**o+(-0.810676898206)*x[1]+(-0.613330062817)*x[2]**o+(0.714335738947)*x[2]
        arg[0,3,1,0]=(-0.602336649415)*x[0]**o+(-0.166667559558)*x[0]+(0.962948866081)*x[1]**o+(0.939531483467)*x[1]+(0.267767294244)*x[2]**o+(0.247155872428)*x[2]
        arg[0,3,1,1]=(0.838505391049)*x[0]**o+(0.115204438919)*x[0]+(0.989010581013)*x[1]**o+(-0.649789818418)*x[1]+(0.757328444925)*x[2]**o+(0.825158129449)*x[2]
        arg[0,3,2,0]=(0.724488034506)*x[0]**o+(0.0333093325925)*x[0]+(0.864643652652)*x[1]**o+(-0.557517585925)*x[1]+(-0.340534672063)*x[2]**o+(-0.00425705176271)*x[2]
        arg[0,3,2,1]=(-0.366459483792)*x[0]**o+(0.257706980646)*x[0]+(0.821731531023)*x[1]**o+(-0.212501303617)*x[1]+(-0.128977067795)*x[2]**o+(-0.931439847041)*x[2]
        arg[0,4,0,0]=(0.398430239547)*x[0]**o+(-0.00991917699933)*x[0]+(-0.182471763172)*x[1]**o+(0.218327764946)*x[1]+(-0.484510127696)*x[2]**o+(-0.360678193996)*x[2]
        arg[0,4,0,1]=(0.0640680415135)*x[0]**o+(0.671150063087)*x[0]+(0.208610130381)*x[1]**o+(-0.441252393083)*x[1]+(0.365150146582)*x[2]**o+(-0.228346558142)*x[2]
        arg[0,4,1,0]=(-0.852694824303)*x[0]**o+(0.181041494463)*x[0]+(0.898413617086)*x[1]**o+(-0.918988862595)*x[1]+(0.169010143324)*x[2]**o+(0.540458809834)*x[2]
        arg[0,4,1,1]=(-0.213782550366)*x[0]**o+(-0.180557891041)*x[0]+(0.23144059382)*x[1]**o+(0.0609482143429)*x[1]+(0.89256221946)*x[2]**o+(0.977857158982)*x[2]
        arg[0,4,2,0]=(0.320772898841)*x[0]**o+(0.371610189518)*x[0]+(0.692264150983)*x[1]**o+(-0.751154935415)*x[1]+(-0.163272902772)*x[2]**o+(0.0635386628764)*x[2]
        arg[0,4,2,1]=(-0.316885908577)*x[0]**o+(0.476104431241)*x[0]+(-0.647713072489)*x[1]**o+(0.683813449908)*x[1]+(0.699293342945)*x[2]**o+(-0.0373906770097)*x[2]
        arg[1,0,0,0]=(-0.030329265733)*x[0]**o+(-0.0115761157375)*x[0]+(-0.30993823088)*x[1]**o+(0.628877975389)*x[1]+(-0.582940961417)*x[2]**o+(0.154786570457)*x[2]
        arg[1,0,0,1]=(-0.593610753495)*x[0]**o+(-0.822232097764)*x[0]+(-0.465144000698)*x[1]**o+(-0.62576868204)*x[1]+(-0.53588531407)*x[2]**o+(0.119119578421)*x[2]
        arg[1,0,1,0]=(-0.15668734059)*x[0]**o+(-0.950658811047)*x[0]+(-0.267969935748)*x[1]**o+(0.0919415340304)*x[1]+(-0.0445382452662)*x[2]**o+(0.949984936459)*x[2]
        arg[1,0,1,1]=(-0.762474809436)*x[0]**o+(0.794314442281)*x[0]+(0.617742930372)*x[1]**o+(-0.502057530669)*x[1]+(0.42532154189)*x[2]**o+(0.343417687179)*x[2]
        arg[1,0,2,0]=(0.259583418084)*x[0]**o+(0.460122326434)*x[0]+(0.875185242587)*x[1]**o+(-0.621232091699)*x[1]+(0.922536397008)*x[2]**o+(-0.494729334323)*x[2]
        arg[1,0,2,1]=(0.79625863156)*x[0]**o+(-0.980360881506)*x[0]+(0.947721778185)*x[1]**o+(-0.234201025597)*x[1]+(-0.974251961452)*x[2]**o+(-0.300484605111)*x[2]
        arg[1,1,0,0]=(0.719927358834)*x[0]**o+(-0.296944527139)*x[0]+(0.970290716632)*x[1]**o+(0.45656092862)*x[1]+(0.657023751979)*x[2]**o+(-0.703898365316)*x[2]
        arg[1,1,0,1]=(-0.0600633107738)*x[0]**o+(0.525293730667)*x[0]+(0.17955637601)*x[1]**o+(-0.625782540859)*x[1]+(0.205255859488)*x[2]**o+(-0.963128832741)*x[2]
        arg[1,1,1,0]=(0.119721921725)*x[0]**o+(0.957918016103)*x[0]+(-0.574883179285)*x[1]**o+(0.915580622743)*x[1]+(-0.432538225757)*x[2]**o+(-0.937539574986)*x[2]
        arg[1,1,1,1]=(-0.295639299942)*x[0]**o+(-0.703954000058)*x[0]+(-0.185001166055)*x[1]**o+(-0.943933820897)*x[1]+(-0.12692007135)*x[2]**o+(0.476481616585)*x[2]
        arg[1,1,2,0]=(0.247892010246)*x[0]**o+(0.901198196531)*x[0]+(-0.586162224974)*x[1]**o+(-0.934177552379)*x[1]+(0.704265144051)*x[2]**o+(-0.688632953989)*x[2]
        arg[1,1,2,1]=(-0.116039842334)*x[0]**o+(-0.300373025101)*x[0]+(-0.286684372551)*x[1]**o+(-0.452337760059)*x[1]+(0.883210026596)*x[2]**o+(-0.319739612634)*x[2]
        arg[1,2,0,0]=(-0.0326555626249)*x[0]**o+(-0.0933559852573)*x[0]+(0.807443822382)*x[1]**o+(-0.148842350195)*x[1]+(-0.118904471983)*x[2]**o+(-0.740440145747)*x[2]
        arg[1,2,0,1]=(0.313623768488)*x[0]**o+(0.524958399998)*x[0]+(0.876747001712)*x[1]**o+(-0.468458894694)*x[1]+(0.52118935613)*x[2]**o+(-0.776405429712)*x[2]
        arg[1,2,1,0]=(0.1328367779)*x[0]**o+(-0.959628068869)*x[0]+(0.746000325891)*x[1]**o+(-0.454705589649)*x[1]+(0.08534697763)*x[2]**o+(0.174892567303)*x[2]
        arg[1,2,1,1]=(-0.482594997843)*x[0]**o+(0.765628170342)*x[0]+(0.103333249513)*x[1]**o+(0.840778499017)*x[1]+(0.136748193255)*x[2]**o+(0.771613240623)*x[2]
        arg[1,2,2,0]=(-0.305745376076)*x[0]**o+(-0.0464234361529)*x[0]+(-0.163528204043)*x[1]**o+(0.80782719974)*x[1]+(0.131046315567)*x[2]**o+(0.0849217124905)*x[2]
        arg[1,2,2,1]=(0.564915635291)*x[0]**o+(-0.162467524139)*x[0]+(-0.696721556806)*x[1]**o+(0.0172839552576)*x[1]+(-0.630040402586)*x[2]**o+(-0.181232689117)*x[2]
        arg[1,3,0,0]=(-0.768631958549)*x[0]**o+(0.537076196504)*x[0]+(-0.403489871013)*x[1]**o+(0.831966318632)*x[1]+(0.602387061638)*x[2]**o+(0.330495470463)*x[2]
        arg[1,3,0,1]=(0.770878177942)*x[0]**o+(-0.678609489017)*x[0]+(-0.895200519099)*x[1]**o+(0.935093561205)*x[1]+(0.0497551733469)*x[2]**o+(0.820432808383)*x[2]
        arg[1,3,1,0]=(0.052867263447)*x[0]**o+(0.566167926663)*x[0]+(-0.00397870350455)*x[1]**o+(-0.780374460006)*x[1]+(-0.166645199933)*x[2]**o+(-0.867274057327)*x[2]
        arg[1,3,1,1]=(0.0573446476183)*x[0]**o+(-0.699343104682)*x[0]+(0.428624411932)*x[1]**o+(-0.400834963806)*x[1]+(-0.722750021776)*x[2]**o+(0.386821944027)*x[2]
        arg[1,3,2,0]=(0.390777333115)*x[0]**o+(0.191564882434)*x[0]+(-0.774547811812)*x[1]**o+(0.747678394063)*x[1]+(0.662273739757)*x[2]**o+(-0.295629568405)*x[2]
        arg[1,3,2,1]=(-0.160077661734)*x[0]**o+(-0.512286879466)*x[0]+(0.616824388099)*x[1]**o+(0.0257995114324)*x[1]+(0.644270949659)*x[2]**o+(0.243520087896)*x[2]
        arg[1,4,0,0]=(-0.980501571755)*x[0]**o+(0.844184850883)*x[0]+(-0.608509998911)*x[1]**o+(-0.347687895783)*x[1]+(-0.536534228926)*x[2]**o+(0.632821444072)*x[2]
        arg[1,4,0,1]=(-0.881676821515)*x[0]**o+(-0.858728104026)*x[0]+(-0.576516592622)*x[1]**o+(0.695748958418)*x[1]+(0.856547670877)*x[2]**o+(-0.932960091743)*x[2]
        arg[1,4,1,0]=(-0.451435812217)*x[0]**o+(0.963886330308)*x[0]+(0.888532771273)*x[1]**o+(0.807924978471)*x[1]+(0.789117121354)*x[2]**o+(0.577647188972)*x[2]
        arg[1,4,1,1]=(-0.315320308371)*x[0]**o+(0.182798184972)*x[0]+(0.936963228212)*x[1]**o+(-0.634429372225)*x[1]+(0.634343556662)*x[2]**o+(0.975851394002)*x[2]
        arg[1,4,2,0]=(0.858246326961)*x[0]**o+(0.0955812000864)*x[0]+(0.16143064785)*x[1]**o+(0.397013086552)*x[1]+(-0.507188929963)*x[2]**o+(0.49789827039)*x[2]
        arg[1,4,2,1]=(-0.524634529078)*x[0]**o+(0.675491034226)*x[0]+(0.858857975014)*x[1]**o+(-0.57874207047)*x[1]+(0.476551353232)*x[2]**o+(-0.622601386038)*x[2]
        arg[2,0,0,0]=(0.976563609)*x[0]**o+(0.127133211238)*x[0]+(-0.410155025204)*x[1]**o+(-0.909032700787)*x[1]+(-0.197014506331)*x[2]**o+(0.557171133933)*x[2]
        arg[2,0,0,1]=(0.434874625301)*x[0]**o+(-0.576955025374)*x[0]+(-0.466686136319)*x[1]**o+(-0.359616702831)*x[1]+(-0.916050768111)*x[2]**o+(0.900510057507)*x[2]
        arg[2,0,1,0]=(0.615259852986)*x[0]**o+(-0.685399646912)*x[0]+(-0.222742007887)*x[1]**o+(-0.487739486778)*x[1]+(-0.675225266903)*x[2]**o+(0.494623157154)*x[2]
        arg[2,0,1,1]=(-0.89129010918)*x[0]**o+(0.0051571255592)*x[0]+(0.681003954208)*x[1]**o+(0.634041720719)*x[1]+(-0.308587458722)*x[2]**o+(-0.107801539193)*x[2]
        arg[2,0,2,0]=(-0.660496698398)*x[0]**o+(-0.889315862375)*x[0]+(0.64248433679)*x[1]**o+(-0.941990511222)*x[1]+(-0.173668816201)*x[2]**o+(0.237812802184)*x[2]
        arg[2,0,2,1]=(0.847365004226)*x[0]**o+(0.802449170085)*x[0]+(0.504737931993)*x[1]**o+(-0.695546680342)*x[1]+(0.0742668699455)*x[2]**o+(0.396565506083)*x[2]
        arg[2,1,0,0]=(0.812115742561)*x[0]**o+(-0.528091349418)*x[0]+(-0.442834134691)*x[1]**o+(-0.574414379092)*x[1]+(-0.734077338221)*x[2]**o+(0.159306515061)*x[2]
        arg[2,1,0,1]=(-0.259422255041)*x[0]**o+(-0.104016445541)*x[0]+(0.3560854635)*x[1]**o+(0.997015276785)*x[1]+(-0.707278312293)*x[2]**o+(-0.975235879836)*x[2]
        arg[2,1,1,0]=(0.402099188227)*x[0]**o+(0.0320354114274)*x[0]+(0.203384927114)*x[1]**o+(-0.847804170327)*x[1]+(-0.898459903546)*x[2]**o+(0.639388498254)*x[2]
        arg[2,1,1,1]=(-0.0477209380768)*x[0]**o+(0.531574667575)*x[0]+(0.280889682833)*x[1]**o+(-0.444963671116)*x[1]+(0.33935324652)*x[2]**o+(0.943089036339)*x[2]
        arg[2,1,2,0]=(0.396038211375)*x[0]**o+(-0.54994373568)*x[0]+(0.851017234687)*x[1]**o+(-0.739787661885)*x[1]+(-0.368325897478)*x[2]**o+(-0.449096507837)*x[2]
        arg[2,1,2,1]=(0.857714720302)*x[0]**o+(-0.750668366832)*x[0]+(-0.587901076381)*x[1]**o+(-0.24906126763)*x[1]+(0.131707530536)*x[2]**o+(0.900975507373)*x[2]
        arg[2,2,0,0]=(0.783919932392)*x[0]**o+(-0.967669966647)*x[0]+(0.82052580972)*x[1]**o+(-0.711919131449)*x[1]+(-0.396059626351)*x[2]**o+(-0.265805391355)*x[2]
        arg[2,2,0,1]=(-0.0398160197307)*x[0]**o+(-0.238193541)*x[0]+(0.282277679599)*x[1]**o+(-0.0526835304963)*x[1]+(0.590871489195)*x[2]**o+(-0.925835585742)*x[2]
        arg[2,2,1,0]=(-0.825757767738)*x[0]**o+(0.238488063696)*x[0]+(0.109616380527)*x[1]**o+(-0.942941715764)*x[1]+(0.632013079754)*x[2]**o+(0.0517997920543)*x[2]
        arg[2,2,1,1]=(-0.85343230326)*x[0]**o+(0.225247381624)*x[0]+(0.43171428489)*x[1]**o+(0.0937487532231)*x[1]+(0.894064129869)*x[2]**o+(-0.0876265498234)*x[2]
        arg[2,2,2,0]=(0.349294373111)*x[0]**o+(-0.734159680014)*x[0]+(0.0489227393033)*x[1]**o+(-0.245284468757)*x[1]+(-0.346469825649)*x[2]**o+(0.547054093324)*x[2]
        arg[2,2,2,1]=(-0.0478095998047)*x[0]**o+(0.440261344405)*x[0]+(-0.590330499679)*x[1]**o+(-0.961009986328)*x[1]+(0.546190002905)*x[2]**o+(-0.027965996519)*x[2]
        arg[2,3,0,0]=(0.148563550644)*x[0]**o+(0.576704219545)*x[0]+(-0.782168581109)*x[1]**o+(-0.8301245678)*x[1]+(0.937936245682)*x[2]**o+(0.846740948371)*x[2]
        arg[2,3,0,1]=(-0.447832058524)*x[0]**o+(0.385431388609)*x[0]+(0.193985676964)*x[1]**o+(-0.91717180275)*x[1]+(-0.408732230908)*x[2]**o+(0.548173152118)*x[2]
        arg[2,3,1,0]=(0.197592567864)*x[0]**o+(-0.77657127384)*x[0]+(0.266355135001)*x[1]**o+(0.604009981584)*x[1]+(-0.379930450459)*x[2]**o+(0.46952844479)*x[2]
        arg[2,3,1,1]=(0.996803794962)*x[0]**o+(0.319115676981)*x[0]+(0.255552997875)*x[1]**o+(0.21834578367)*x[1]+(0.389966036243)*x[2]**o+(-0.115308080726)*x[2]
        arg[2,3,2,0]=(-0.0244303880675)*x[0]**o+(-0.128929049017)*x[0]+(-0.350933414406)*x[1]**o+(-0.341716440852)*x[1]+(-0.724579076377)*x[2]**o+(0.21963465519)*x[2]
        arg[2,3,2,1]=(-0.92899971478)*x[0]**o+(-0.304496074717)*x[0]+(0.568151427024)*x[1]**o+(0.504629517922)*x[1]+(0.846158711477)*x[2]**o+(0.35936792568)*x[2]
        arg[2,4,0,0]=(-0.350964834352)*x[0]**o+(0.387667833884)*x[0]+(0.547101274223)*x[1]**o+(0.257218757373)*x[1]+(-0.00316748527841)*x[2]**o+(0.314561238615)*x[2]
        arg[2,4,0,1]=(0.408941739151)*x[0]**o+(0.819113792537)*x[0]+(0.532891848781)*x[1]**o+(-0.30688659725)*x[1]+(-0.736611937618)*x[2]**o+(-0.875783706051)*x[2]
        arg[2,4,1,0]=(-0.730885165985)*x[0]**o+(0.453908506302)*x[0]+(-0.617385752505)*x[1]**o+(0.357459789592)*x[1]+(0.286323175805)*x[2]**o+(-0.448879127031)*x[2]
        arg[2,4,1,1]=(-0.295377341281)*x[0]**o+(0.423424371051)*x[0]+(0.524695076927)*x[1]**o+(-0.698499625038)*x[1]+(0.617308008395)*x[2]**o+(-0.423469866581)*x[2]
        arg[2,4,2,0]=(0.104251379635)*x[0]**o+(0.989709098795)*x[0]+(-0.422075396624)*x[1]**o+(-0.254328561231)*x[1]+(-0.502052795074)*x[2]**o+(-0.0851411652168)*x[2]
        arg[2,4,2,1]=(0.379332842082)*x[0]**o+(-0.707504861783)*x[0]+(0.00855075647146)*x[1]**o+(0.792851159033)*x[1]+(-0.692042160791)*x[2]**o+(-0.18327165243)*x[2]
        arg[3,0,0,0]=(0.310700158718)*x[0]**o+(0.285647240187)*x[0]+(0.817947571838)*x[1]**o+(0.808698040916)*x[1]+(0.844184006443)*x[2]**o+(-0.310383215636)*x[2]
        arg[3,0,0,1]=(-0.0173109952616)*x[0]**o+(0.672034452037)*x[0]+(-0.464572256298)*x[1]**o+(0.205799523299)*x[1]+(-0.971979841338)*x[2]**o+(0.261515966001)*x[2]
        arg[3,0,1,0]=(0.0329384377028)*x[0]**o+(0.571501185072)*x[0]+(-0.902222556642)*x[1]**o+(-0.528317784373)*x[1]+(0.331989640881)*x[2]**o+(0.0427505624318)*x[2]
        arg[3,0,1,1]=(0.964350410761)*x[0]**o+(-0.327198989999)*x[0]+(-0.195271523191)*x[1]**o+(0.534663014601)*x[1]+(-0.0208261208706)*x[2]**o+(-0.158019249052)*x[2]
        arg[3,0,2,0]=(-0.622082726008)*x[0]**o+(0.559463883442)*x[0]+(0.841439693313)*x[1]**o+(-0.196363987516)*x[1]+(-0.0143926547396)*x[2]**o+(0.144732603689)*x[2]
        arg[3,0,2,1]=(-0.149531653278)*x[0]**o+(0.354229138322)*x[0]+(-0.507916088829)*x[1]**o+(0.221251580047)*x[1]+(0.197213706274)*x[2]**o+(-0.555645149276)*x[2]
        arg[3,1,0,0]=(0.331020785374)*x[0]**o+(-0.377647773513)*x[0]+(0.563355928949)*x[1]**o+(0.789309216711)*x[1]+(0.203663383875)*x[2]**o+(-0.0333894540645)*x[2]
        arg[3,1,0,1]=(-0.0965613913882)*x[0]**o+(0.18656484323)*x[0]+(-0.138245941339)*x[1]**o+(-0.34549538745)*x[1]+(0.827173403167)*x[2]**o+(-0.873380503939)*x[2]
        arg[3,1,1,0]=(0.932940890033)*x[0]**o+(-0.564883830288)*x[0]+(-0.505052725295)*x[1]**o+(-0.669420210206)*x[1]+(-0.799638096039)*x[2]**o+(0.469400233749)*x[2]
        arg[3,1,1,1]=(-0.9741761069)*x[0]**o+(-0.748125328781)*x[0]+(0.664448241931)*x[1]**o+(0.0957643296979)*x[1]+(0.112211322019)*x[2]**o+(-0.131870290434)*x[2]
        arg[3,1,2,0]=(-0.438609127704)*x[0]**o+(-0.36618286515)*x[0]+(-0.376420011115)*x[1]**o+(-0.67084624599)*x[1]+(0.821679796644)*x[2]**o+(0.80111319897)*x[2]
        arg[3,1,2,1]=(-0.409859432085)*x[0]**o+(0.848767816929)*x[0]+(0.382358459835)*x[1]**o+(-0.750139843631)*x[1]+(0.587152586559)*x[2]**o+(-0.26162952001)*x[2]
        arg[3,2,0,0]=(-0.506329785074)*x[0]**o+(-0.0967745752004)*x[0]+(-0.163316795268)*x[1]**o+(0.0628481921732)*x[1]+(0.395355494367)*x[2]**o+(0.0978901916332)*x[2]
        arg[3,2,0,1]=(0.803590309587)*x[0]**o+(0.470388053389)*x[0]+(0.674942922168)*x[1]**o+(0.223472834762)*x[1]+(0.552332209028)*x[2]**o+(-0.396443498843)*x[2]
        arg[3,2,1,0]=(-0.312384723839)*x[0]**o+(0.392213828608)*x[0]+(0.449465316671)*x[1]**o+(-0.249103089401)*x[1]+(0.601412913458)*x[2]**o+(0.209899368039)*x[2]
        arg[3,2,1,1]=(-0.583776179557)*x[0]**o+(0.485007047754)*x[0]+(-0.0354153633589)*x[1]**o+(-0.898630969916)*x[1]+(-0.266581492795)*x[2]**o+(0.822086680281)*x[2]
        arg[3,2,2,0]=(0.944617094397)*x[0]**o+(-0.173057994833)*x[0]+(0.927832160048)*x[1]**o+(-0.218599594252)*x[1]+(0.349957755011)*x[2]**o+(0.392602744545)*x[2]
        arg[3,2,2,1]=(0.461648084674)*x[0]**o+(0.826213376949)*x[0]+(0.143429054731)*x[1]**o+(-0.214265128007)*x[1]+(0.0317208127607)*x[2]**o+(-0.858361078381)*x[2]
        arg[3,3,0,0]=(0.306607920368)*x[0]**o+(0.147692891688)*x[0]+(-0.562661766456)*x[1]**o+(0.448595564727)*x[1]+(-0.707178218373)*x[2]**o+(0.0655554587866)*x[2]
        arg[3,3,0,1]=(0.135475820362)*x[0]**o+(0.51420718793)*x[0]+(0.343291220736)*x[1]**o+(-0.797532312602)*x[1]+(-0.273431011273)*x[2]**o+(0.135307779253)*x[2]
        arg[3,3,1,0]=(0.488028231073)*x[0]**o+(0.542021806829)*x[0]+(0.91916268609)*x[1]**o+(0.317368273365)*x[1]+(0.314807039395)*x[2]**o+(-0.841583308025)*x[2]
        arg[3,3,1,1]=(-0.182783814917)*x[0]**o+(0.250863481201)*x[0]+(0.755551457003)*x[1]**o+(0.946171548529)*x[1]+(0.0350225436682)*x[2]**o+(0.776059830052)*x[2]
        arg[3,3,2,0]=(-0.116707124994)*x[0]**o+(-0.727010280588)*x[0]+(0.172065389268)*x[1]**o+(0.754164564639)*x[1]+(0.803777635621)*x[2]**o+(0.293490264417)*x[2]
        arg[3,3,2,1]=(0.154235982797)*x[0]**o+(-0.124286021486)*x[0]+(-0.737324748116)*x[1]**o+(-0.136803541581)*x[1]+(-0.608370191604)*x[2]**o+(-0.933599034128)*x[2]
        arg[3,4,0,0]=(0.818061677328)*x[0]**o+(-0.771719613166)*x[0]+(0.742772795251)*x[1]**o+(0.0359285091312)*x[1]+(-0.985656851837)*x[2]**o+(-0.884598206512)*x[2]
        arg[3,4,0,1]=(0.198371732741)*x[0]**o+(0.646496618681)*x[0]+(0.232912446494)*x[1]**o+(0.945619492023)*x[1]+(-0.344370341731)*x[2]**o+(0.31156840357)*x[2]
        arg[3,4,1,0]=(-0.609298834606)*x[0]**o+(0.784697392412)*x[0]+(0.996065619079)*x[1]**o+(-0.284498300043)*x[1]+(-0.652795706803)*x[2]**o+(-0.254746840396)*x[2]
        arg[3,4,1,1]=(-0.0083632822507)*x[0]**o+(0.200923618933)*x[0]+(0.353883930064)*x[1]**o+(0.153257380179)*x[1]+(0.843965464205)*x[2]**o+(0.0418501400703)*x[2]
        arg[3,4,2,0]=(-0.226055622191)*x[0]**o+(-0.701789166696)*x[0]+(0.274080957143)*x[1]**o+(0.0596744660556)*x[1]+(-0.342083152987)*x[2]**o+(0.593518464697)*x[2]
        arg[3,4,2,1]=(-0.288168945023)*x[0]**o+(-0.742841678774)*x[0]+(0.972629532424)*x[1]**o+(0.892100560733)*x[1]+(0.803880474721)*x[2]**o+(-0.0638336211224)*x[2]
        ref[0,0,0,0]=(0.244520699379)*x_ref[0]**o+(-0.125771252842)*x_ref[0]+(0.654557798928)*x_ref[1]**o+(0.438110948503)*x_ref[1]+(0.399996169353)*x_ref[2]**o+(0.37037141324)*x_ref[2]
        ref[0,0,0,1]=(0.019693348021)*x_ref[0]**o+(-0.509938412509)*x_ref[0]+(-0.425852907243)*x_ref[1]**o+(0.351462945072)*x_ref[1]+(0.628410872426)*x_ref[2]**o+(0.4769252017)*x_ref[2]
        ref[0,0,1,0]=(0.623813767602)*x_ref[0]**o+(-0.940818977456)*x_ref[0]+(-0.00111063869285)*x_ref[1]**o+(-0.577558081394)*x_ref[1]+(-0.30810971068)*x_ref[2]**o+(-0.660149007373)*x_ref[2]
        ref[0,0,1,1]=(0.994105796371)*x_ref[0]**o+(0.199970906966)*x_ref[0]+(0.961509001434)*x_ref[1]**o+(-0.783069602127)*x_ref[1]+(-0.179922863021)*x_ref[2]**o+(-0.386974503089)*x_ref[2]
        ref[0,0,2,0]=(0.932745137122)*x_ref[0]**o+(0.627995830114)*x_ref[0]+(-0.334344490736)*x_ref[1]**o+(0.733210590815)*x_ref[1]+(-0.23424959051)*x_ref[2]**o+(0.140644214958)*x_ref[2]
        ref[0,0,2,1]=(-0.282836519247)*x_ref[0]**o+(-0.744077669768)*x_ref[0]+(0.123898574243)*x_ref[1]**o+(0.562384289004)*x_ref[1]+(0.0757927394637)*x_ref[2]**o+(0.767684067921)*x_ref[2]
        ref[0,1,0,0]=(0.212366202901)*x_ref[0]**o+(-0.679228304269)*x_ref[0]+(0.829706652419)*x_ref[1]**o+(0.24389996073)*x_ref[1]+(0.909367025978)*x_ref[2]**o+(0.280617790407)*x_ref[2]
        ref[0,1,0,1]=(0.760070952632)*x_ref[0]**o+(-0.678309252372)*x_ref[0]+(0.0372281910363)*x_ref[1]**o+(0.0284582814297)*x_ref[1]+(-0.092349250941)*x_ref[2]**o+(0.347713645921)*x_ref[2]
        ref[0,1,1,0]=(0.248410230191)*x_ref[0]**o+(-0.833983202295)*x_ref[0]+(-0.759357278899)*x_ref[1]**o+(0.880038285893)*x_ref[1]+(-0.734799540652)*x_ref[2]**o+(0.493330231104)*x_ref[2]
        ref[0,1,1,1]=(0.0359919724228)*x_ref[0]**o+(-0.0393899014057)*x_ref[0]+(0.185155782488)*x_ref[1]**o+(-0.92692925719)*x_ref[1]+(0.74718037404)*x_ref[2]**o+(-0.283900681763)*x_ref[2]
        ref[0,1,2,0]=(-0.208643544667)*x_ref[0]**o+(0.368200149096)*x_ref[0]+(-0.327032725172)*x_ref[1]**o+(-0.245547905792)*x_ref[1]+(0.0886488214242)*x_ref[2]**o+(0.179139897691)*x_ref[2]
        ref[0,1,2,1]=(-0.0618762861459)*x_ref[0]**o+(0.253419217897)*x_ref[0]+(0.840270125508)*x_ref[1]**o+(0.0223556878387)*x_ref[1]+(0.0560904700176)*x_ref[2]**o+(-0.646231049424)*x_ref[2]
        ref[0,2,0,0]=(0.568966969865)*x_ref[0]**o+(0.786776511348)*x_ref[0]+(0.150595926116)*x_ref[1]**o+(-0.780164417762)*x_ref[1]+(0.510149831242)*x_ref[2]**o+(-0.902871929903)*x_ref[2]
        ref[0,2,0,1]=(-0.817062669957)*x_ref[0]**o+(0.626805486868)*x_ref[0]+(-0.566129786829)*x_ref[1]**o+(0.248418548905)*x_ref[1]+(-0.273995326019)*x_ref[2]**o+(0.402023521018)*x_ref[2]
        ref[0,2,1,0]=(-0.160276235113)*x_ref[0]**o+(-0.0732543937053)*x_ref[0]+(-0.0651595982698)*x_ref[1]**o+(-0.529463476237)*x_ref[1]+(0.346098812945)*x_ref[2]**o+(0.418367016439)*x_ref[2]
        ref[0,2,1,1]=(0.736907408498)*x_ref[0]**o+(-0.123612748157)*x_ref[0]+(0.000834659108651)*x_ref[1]**o+(0.568833761363)*x_ref[1]+(-0.542676953582)*x_ref[2]**o+(0.026182553484)*x_ref[2]
        ref[0,2,2,0]=(0.320985918151)*x_ref[0]**o+(0.0512654311644)*x_ref[0]+(-0.0313412913713)*x_ref[1]**o+(0.295659930913)*x_ref[1]+(-0.989560405329)*x_ref[2]**o+(-0.107344246298)*x_ref[2]
        ref[0,2,2,1]=(-0.957793939949)*x_ref[0]**o+(-0.177591880789)*x_ref[0]+(0.0785829772689)*x_ref[1]**o+(0.121445956612)*x_ref[1]+(-0.384134176508)*x_ref[2]**o+(-0.775433914622)*x_ref[2]
        ref[0,3,0,0]=(-0.130620609021)*x_ref[0]**o+(0.268283814564)*x_ref[0]+(-0.192835222037)*x_ref[1]**o+(0.235341688541)*x_ref[1]+(0.957069315916)*x_ref[2]**o+(-0.708853567529)*x_ref[2]
        ref[0,3,0,1]=(-0.377028600614)*x_ref[0]**o+(-0.858446162753)*x_ref[0]+(-0.850630580491)*x_ref[1]**o+(-0.810676898206)*x_ref[1]+(-0.613330062817)*x_ref[2]**o+(0.714335738947)*x_ref[2]
        ref[0,3,1,0]=(-0.602336649415)*x_ref[0]**o+(-0.166667559558)*x_ref[0]+(0.962948866081)*x_ref[1]**o+(0.939531483467)*x_ref[1]+(0.267767294244)*x_ref[2]**o+(0.247155872428)*x_ref[2]
        ref[0,3,1,1]=(0.838505391049)*x_ref[0]**o+(0.115204438919)*x_ref[0]+(0.989010581013)*x_ref[1]**o+(-0.649789818418)*x_ref[1]+(0.757328444925)*x_ref[2]**o+(0.825158129449)*x_ref[2]
        ref[0,3,2,0]=(0.724488034506)*x_ref[0]**o+(0.0333093325925)*x_ref[0]+(0.864643652652)*x_ref[1]**o+(-0.557517585925)*x_ref[1]+(-0.340534672063)*x_ref[2]**o+(-0.00425705176271)*x_ref[2]
        ref[0,3,2,1]=(-0.366459483792)*x_ref[0]**o+(0.257706980646)*x_ref[0]+(0.821731531023)*x_ref[1]**o+(-0.212501303617)*x_ref[1]+(-0.128977067795)*x_ref[2]**o+(-0.931439847041)*x_ref[2]
        ref[0,4,0,0]=(0.398430239547)*x_ref[0]**o+(-0.00991917699933)*x_ref[0]+(-0.182471763172)*x_ref[1]**o+(0.218327764946)*x_ref[1]+(-0.484510127696)*x_ref[2]**o+(-0.360678193996)*x_ref[2]
        ref[0,4,0,1]=(0.0640680415135)*x_ref[0]**o+(0.671150063087)*x_ref[0]+(0.208610130381)*x_ref[1]**o+(-0.441252393083)*x_ref[1]+(0.365150146582)*x_ref[2]**o+(-0.228346558142)*x_ref[2]
        ref[0,4,1,0]=(-0.852694824303)*x_ref[0]**o+(0.181041494463)*x_ref[0]+(0.898413617086)*x_ref[1]**o+(-0.918988862595)*x_ref[1]+(0.169010143324)*x_ref[2]**o+(0.540458809834)*x_ref[2]
        ref[0,4,1,1]=(-0.213782550366)*x_ref[0]**o+(-0.180557891041)*x_ref[0]+(0.23144059382)*x_ref[1]**o+(0.0609482143429)*x_ref[1]+(0.89256221946)*x_ref[2]**o+(0.977857158982)*x_ref[2]
        ref[0,4,2,0]=(0.320772898841)*x_ref[0]**o+(0.371610189518)*x_ref[0]+(0.692264150983)*x_ref[1]**o+(-0.751154935415)*x_ref[1]+(-0.163272902772)*x_ref[2]**o+(0.0635386628764)*x_ref[2]
        ref[0,4,2,1]=(-0.316885908577)*x_ref[0]**o+(0.476104431241)*x_ref[0]+(-0.647713072489)*x_ref[1]**o+(0.683813449908)*x_ref[1]+(0.699293342945)*x_ref[2]**o+(-0.0373906770097)*x_ref[2]
        ref[1,0,0,0]=(-0.030329265733)*x_ref[0]**o+(-0.0115761157375)*x_ref[0]+(-0.30993823088)*x_ref[1]**o+(0.628877975389)*x_ref[1]+(-0.582940961417)*x_ref[2]**o+(0.154786570457)*x_ref[2]
        ref[1,0,0,1]=(-0.593610753495)*x_ref[0]**o+(-0.822232097764)*x_ref[0]+(-0.465144000698)*x_ref[1]**o+(-0.62576868204)*x_ref[1]+(-0.53588531407)*x_ref[2]**o+(0.119119578421)*x_ref[2]
        ref[1,0,1,0]=(-0.15668734059)*x_ref[0]**o+(-0.950658811047)*x_ref[0]+(-0.267969935748)*x_ref[1]**o+(0.0919415340304)*x_ref[1]+(-0.0445382452662)*x_ref[2]**o+(0.949984936459)*x_ref[2]
        ref[1,0,1,1]=(-0.762474809436)*x_ref[0]**o+(0.794314442281)*x_ref[0]+(0.617742930372)*x_ref[1]**o+(-0.502057530669)*x_ref[1]+(0.42532154189)*x_ref[2]**o+(0.343417687179)*x_ref[2]
        ref[1,0,2,0]=(0.259583418084)*x_ref[0]**o+(0.460122326434)*x_ref[0]+(0.875185242587)*x_ref[1]**o+(-0.621232091699)*x_ref[1]+(0.922536397008)*x_ref[2]**o+(-0.494729334323)*x_ref[2]
        ref[1,0,2,1]=(0.79625863156)*x_ref[0]**o+(-0.980360881506)*x_ref[0]+(0.947721778185)*x_ref[1]**o+(-0.234201025597)*x_ref[1]+(-0.974251961452)*x_ref[2]**o+(-0.300484605111)*x_ref[2]
        ref[1,1,0,0]=(0.719927358834)*x_ref[0]**o+(-0.296944527139)*x_ref[0]+(0.970290716632)*x_ref[1]**o+(0.45656092862)*x_ref[1]+(0.657023751979)*x_ref[2]**o+(-0.703898365316)*x_ref[2]
        ref[1,1,0,1]=(-0.0600633107738)*x_ref[0]**o+(0.525293730667)*x_ref[0]+(0.17955637601)*x_ref[1]**o+(-0.625782540859)*x_ref[1]+(0.205255859488)*x_ref[2]**o+(-0.963128832741)*x_ref[2]
        ref[1,1,1,0]=(0.119721921725)*x_ref[0]**o+(0.957918016103)*x_ref[0]+(-0.574883179285)*x_ref[1]**o+(0.915580622743)*x_ref[1]+(-0.432538225757)*x_ref[2]**o+(-0.937539574986)*x_ref[2]
        ref[1,1,1,1]=(-0.295639299942)*x_ref[0]**o+(-0.703954000058)*x_ref[0]+(-0.185001166055)*x_ref[1]**o+(-0.943933820897)*x_ref[1]+(-0.12692007135)*x_ref[2]**o+(0.476481616585)*x_ref[2]
        ref[1,1,2,0]=(0.247892010246)*x_ref[0]**o+(0.901198196531)*x_ref[0]+(-0.586162224974)*x_ref[1]**o+(-0.934177552379)*x_ref[1]+(0.704265144051)*x_ref[2]**o+(-0.688632953989)*x_ref[2]
        ref[1,1,2,1]=(-0.116039842334)*x_ref[0]**o+(-0.300373025101)*x_ref[0]+(-0.286684372551)*x_ref[1]**o+(-0.452337760059)*x_ref[1]+(0.883210026596)*x_ref[2]**o+(-0.319739612634)*x_ref[2]
        ref[1,2,0,0]=(-0.0326555626249)*x_ref[0]**o+(-0.0933559852573)*x_ref[0]+(0.807443822382)*x_ref[1]**o+(-0.148842350195)*x_ref[1]+(-0.118904471983)*x_ref[2]**o+(-0.740440145747)*x_ref[2]
        ref[1,2,0,1]=(0.313623768488)*x_ref[0]**o+(0.524958399998)*x_ref[0]+(0.876747001712)*x_ref[1]**o+(-0.468458894694)*x_ref[1]+(0.52118935613)*x_ref[2]**o+(-0.776405429712)*x_ref[2]
        ref[1,2,1,0]=(0.1328367779)*x_ref[0]**o+(-0.959628068869)*x_ref[0]+(0.746000325891)*x_ref[1]**o+(-0.454705589649)*x_ref[1]+(0.08534697763)*x_ref[2]**o+(0.174892567303)*x_ref[2]
        ref[1,2,1,1]=(-0.482594997843)*x_ref[0]**o+(0.765628170342)*x_ref[0]+(0.103333249513)*x_ref[1]**o+(0.840778499017)*x_ref[1]+(0.136748193255)*x_ref[2]**o+(0.771613240623)*x_ref[2]
        ref[1,2,2,0]=(-0.305745376076)*x_ref[0]**o+(-0.0464234361529)*x_ref[0]+(-0.163528204043)*x_ref[1]**o+(0.80782719974)*x_ref[1]+(0.131046315567)*x_ref[2]**o+(0.0849217124905)*x_ref[2]
        ref[1,2,2,1]=(0.564915635291)*x_ref[0]**o+(-0.162467524139)*x_ref[0]+(-0.696721556806)*x_ref[1]**o+(0.0172839552576)*x_ref[1]+(-0.630040402586)*x_ref[2]**o+(-0.181232689117)*x_ref[2]
        ref[1,3,0,0]=(-0.768631958549)*x_ref[0]**o+(0.537076196504)*x_ref[0]+(-0.403489871013)*x_ref[1]**o+(0.831966318632)*x_ref[1]+(0.602387061638)*x_ref[2]**o+(0.330495470463)*x_ref[2]
        ref[1,3,0,1]=(0.770878177942)*x_ref[0]**o+(-0.678609489017)*x_ref[0]+(-0.895200519099)*x_ref[1]**o+(0.935093561205)*x_ref[1]+(0.0497551733469)*x_ref[2]**o+(0.820432808383)*x_ref[2]
        ref[1,3,1,0]=(0.052867263447)*x_ref[0]**o+(0.566167926663)*x_ref[0]+(-0.00397870350455)*x_ref[1]**o+(-0.780374460006)*x_ref[1]+(-0.166645199933)*x_ref[2]**o+(-0.867274057327)*x_ref[2]
        ref[1,3,1,1]=(0.0573446476183)*x_ref[0]**o+(-0.699343104682)*x_ref[0]+(0.428624411932)*x_ref[1]**o+(-0.400834963806)*x_ref[1]+(-0.722750021776)*x_ref[2]**o+(0.386821944027)*x_ref[2]
        ref[1,3,2,0]=(0.390777333115)*x_ref[0]**o+(0.191564882434)*x_ref[0]+(-0.774547811812)*x_ref[1]**o+(0.747678394063)*x_ref[1]+(0.662273739757)*x_ref[2]**o+(-0.295629568405)*x_ref[2]
        ref[1,3,2,1]=(-0.160077661734)*x_ref[0]**o+(-0.512286879466)*x_ref[0]+(0.616824388099)*x_ref[1]**o+(0.0257995114324)*x_ref[1]+(0.644270949659)*x_ref[2]**o+(0.243520087896)*x_ref[2]
        ref[1,4,0,0]=(-0.980501571755)*x_ref[0]**o+(0.844184850883)*x_ref[0]+(-0.608509998911)*x_ref[1]**o+(-0.347687895783)*x_ref[1]+(-0.536534228926)*x_ref[2]**o+(0.632821444072)*x_ref[2]
        ref[1,4,0,1]=(-0.881676821515)*x_ref[0]**o+(-0.858728104026)*x_ref[0]+(-0.576516592622)*x_ref[1]**o+(0.695748958418)*x_ref[1]+(0.856547670877)*x_ref[2]**o+(-0.932960091743)*x_ref[2]
        ref[1,4,1,0]=(-0.451435812217)*x_ref[0]**o+(0.963886330308)*x_ref[0]+(0.888532771273)*x_ref[1]**o+(0.807924978471)*x_ref[1]+(0.789117121354)*x_ref[2]**o+(0.577647188972)*x_ref[2]
        ref[1,4,1,1]=(-0.315320308371)*x_ref[0]**o+(0.182798184972)*x_ref[0]+(0.936963228212)*x_ref[1]**o+(-0.634429372225)*x_ref[1]+(0.634343556662)*x_ref[2]**o+(0.975851394002)*x_ref[2]
        ref[1,4,2,0]=(0.858246326961)*x_ref[0]**o+(0.0955812000864)*x_ref[0]+(0.16143064785)*x_ref[1]**o+(0.397013086552)*x_ref[1]+(-0.507188929963)*x_ref[2]**o+(0.49789827039)*x_ref[2]
        ref[1,4,2,1]=(-0.524634529078)*x_ref[0]**o+(0.675491034226)*x_ref[0]+(0.858857975014)*x_ref[1]**o+(-0.57874207047)*x_ref[1]+(0.476551353232)*x_ref[2]**o+(-0.622601386038)*x_ref[2]
        ref[2,0,0,0]=(0.976563609)*x_ref[0]**o+(0.127133211238)*x_ref[0]+(-0.410155025204)*x_ref[1]**o+(-0.909032700787)*x_ref[1]+(-0.197014506331)*x_ref[2]**o+(0.557171133933)*x_ref[2]
        ref[2,0,0,1]=(0.434874625301)*x_ref[0]**o+(-0.576955025374)*x_ref[0]+(-0.466686136319)*x_ref[1]**o+(-0.359616702831)*x_ref[1]+(-0.916050768111)*x_ref[2]**o+(0.900510057507)*x_ref[2]
        ref[2,0,1,0]=(0.615259852986)*x_ref[0]**o+(-0.685399646912)*x_ref[0]+(-0.222742007887)*x_ref[1]**o+(-0.487739486778)*x_ref[1]+(-0.675225266903)*x_ref[2]**o+(0.494623157154)*x_ref[2]
        ref[2,0,1,1]=(-0.89129010918)*x_ref[0]**o+(0.0051571255592)*x_ref[0]+(0.681003954208)*x_ref[1]**o+(0.634041720719)*x_ref[1]+(-0.308587458722)*x_ref[2]**o+(-0.107801539193)*x_ref[2]
        ref[2,0,2,0]=(-0.660496698398)*x_ref[0]**o+(-0.889315862375)*x_ref[0]+(0.64248433679)*x_ref[1]**o+(-0.941990511222)*x_ref[1]+(-0.173668816201)*x_ref[2]**o+(0.237812802184)*x_ref[2]
        ref[2,0,2,1]=(0.847365004226)*x_ref[0]**o+(0.802449170085)*x_ref[0]+(0.504737931993)*x_ref[1]**o+(-0.695546680342)*x_ref[1]+(0.0742668699455)*x_ref[2]**o+(0.396565506083)*x_ref[2]
        ref[2,1,0,0]=(0.812115742561)*x_ref[0]**o+(-0.528091349418)*x_ref[0]+(-0.442834134691)*x_ref[1]**o+(-0.574414379092)*x_ref[1]+(-0.734077338221)*x_ref[2]**o+(0.159306515061)*x_ref[2]
        ref[2,1,0,1]=(-0.259422255041)*x_ref[0]**o+(-0.104016445541)*x_ref[0]+(0.3560854635)*x_ref[1]**o+(0.997015276785)*x_ref[1]+(-0.707278312293)*x_ref[2]**o+(-0.975235879836)*x_ref[2]
        ref[2,1,1,0]=(0.402099188227)*x_ref[0]**o+(0.0320354114274)*x_ref[0]+(0.203384927114)*x_ref[1]**o+(-0.847804170327)*x_ref[1]+(-0.898459903546)*x_ref[2]**o+(0.639388498254)*x_ref[2]
        ref[2,1,1,1]=(-0.0477209380768)*x_ref[0]**o+(0.531574667575)*x_ref[0]+(0.280889682833)*x_ref[1]**o+(-0.444963671116)*x_ref[1]+(0.33935324652)*x_ref[2]**o+(0.943089036339)*x_ref[2]
        ref[2,1,2,0]=(0.396038211375)*x_ref[0]**o+(-0.54994373568)*x_ref[0]+(0.851017234687)*x_ref[1]**o+(-0.739787661885)*x_ref[1]+(-0.368325897478)*x_ref[2]**o+(-0.449096507837)*x_ref[2]
        ref[2,1,2,1]=(0.857714720302)*x_ref[0]**o+(-0.750668366832)*x_ref[0]+(-0.587901076381)*x_ref[1]**o+(-0.24906126763)*x_ref[1]+(0.131707530536)*x_ref[2]**o+(0.900975507373)*x_ref[2]
        ref[2,2,0,0]=(0.783919932392)*x_ref[0]**o+(-0.967669966647)*x_ref[0]+(0.82052580972)*x_ref[1]**o+(-0.711919131449)*x_ref[1]+(-0.396059626351)*x_ref[2]**o+(-0.265805391355)*x_ref[2]
        ref[2,2,0,1]=(-0.0398160197307)*x_ref[0]**o+(-0.238193541)*x_ref[0]+(0.282277679599)*x_ref[1]**o+(-0.0526835304963)*x_ref[1]+(0.590871489195)*x_ref[2]**o+(-0.925835585742)*x_ref[2]
        ref[2,2,1,0]=(-0.825757767738)*x_ref[0]**o+(0.238488063696)*x_ref[0]+(0.109616380527)*x_ref[1]**o+(-0.942941715764)*x_ref[1]+(0.632013079754)*x_ref[2]**o+(0.0517997920543)*x_ref[2]
        ref[2,2,1,1]=(-0.85343230326)*x_ref[0]**o+(0.225247381624)*x_ref[0]+(0.43171428489)*x_ref[1]**o+(0.0937487532231)*x_ref[1]+(0.894064129869)*x_ref[2]**o+(-0.0876265498234)*x_ref[2]
        ref[2,2,2,0]=(0.349294373111)*x_ref[0]**o+(-0.734159680014)*x_ref[0]+(0.0489227393033)*x_ref[1]**o+(-0.245284468757)*x_ref[1]+(-0.346469825649)*x_ref[2]**o+(0.547054093324)*x_ref[2]
        ref[2,2,2,1]=(-0.0478095998047)*x_ref[0]**o+(0.440261344405)*x_ref[0]+(-0.590330499679)*x_ref[1]**o+(-0.961009986328)*x_ref[1]+(0.546190002905)*x_ref[2]**o+(-0.027965996519)*x_ref[2]
        ref[2,3,0,0]=(0.148563550644)*x_ref[0]**o+(0.576704219545)*x_ref[0]+(-0.782168581109)*x_ref[1]**o+(-0.8301245678)*x_ref[1]+(0.937936245682)*x_ref[2]**o+(0.846740948371)*x_ref[2]
        ref[2,3,0,1]=(-0.447832058524)*x_ref[0]**o+(0.385431388609)*x_ref[0]+(0.193985676964)*x_ref[1]**o+(-0.91717180275)*x_ref[1]+(-0.408732230908)*x_ref[2]**o+(0.548173152118)*x_ref[2]
        ref[2,3,1,0]=(0.197592567864)*x_ref[0]**o+(-0.77657127384)*x_ref[0]+(0.266355135001)*x_ref[1]**o+(0.604009981584)*x_ref[1]+(-0.379930450459)*x_ref[2]**o+(0.46952844479)*x_ref[2]
        ref[2,3,1,1]=(0.996803794962)*x_ref[0]**o+(0.319115676981)*x_ref[0]+(0.255552997875)*x_ref[1]**o+(0.21834578367)*x_ref[1]+(0.389966036243)*x_ref[2]**o+(-0.115308080726)*x_ref[2]
        ref[2,3,2,0]=(-0.0244303880675)*x_ref[0]**o+(-0.128929049017)*x_ref[0]+(-0.350933414406)*x_ref[1]**o+(-0.341716440852)*x_ref[1]+(-0.724579076377)*x_ref[2]**o+(0.21963465519)*x_ref[2]
        ref[2,3,2,1]=(-0.92899971478)*x_ref[0]**o+(-0.304496074717)*x_ref[0]+(0.568151427024)*x_ref[1]**o+(0.504629517922)*x_ref[1]+(0.846158711477)*x_ref[2]**o+(0.35936792568)*x_ref[2]
        ref[2,4,0,0]=(-0.350964834352)*x_ref[0]**o+(0.387667833884)*x_ref[0]+(0.547101274223)*x_ref[1]**o+(0.257218757373)*x_ref[1]+(-0.00316748527841)*x_ref[2]**o+(0.314561238615)*x_ref[2]
        ref[2,4,0,1]=(0.408941739151)*x_ref[0]**o+(0.819113792537)*x_ref[0]+(0.532891848781)*x_ref[1]**o+(-0.30688659725)*x_ref[1]+(-0.736611937618)*x_ref[2]**o+(-0.875783706051)*x_ref[2]
        ref[2,4,1,0]=(-0.730885165985)*x_ref[0]**o+(0.453908506302)*x_ref[0]+(-0.617385752505)*x_ref[1]**o+(0.357459789592)*x_ref[1]+(0.286323175805)*x_ref[2]**o+(-0.448879127031)*x_ref[2]
        ref[2,4,1,1]=(-0.295377341281)*x_ref[0]**o+(0.423424371051)*x_ref[0]+(0.524695076927)*x_ref[1]**o+(-0.698499625038)*x_ref[1]+(0.617308008395)*x_ref[2]**o+(-0.423469866581)*x_ref[2]
        ref[2,4,2,0]=(0.104251379635)*x_ref[0]**o+(0.989709098795)*x_ref[0]+(-0.422075396624)*x_ref[1]**o+(-0.254328561231)*x_ref[1]+(-0.502052795074)*x_ref[2]**o+(-0.0851411652168)*x_ref[2]
        ref[2,4,2,1]=(0.379332842082)*x_ref[0]**o+(-0.707504861783)*x_ref[0]+(0.00855075647146)*x_ref[1]**o+(0.792851159033)*x_ref[1]+(-0.692042160791)*x_ref[2]**o+(-0.18327165243)*x_ref[2]
        ref[3,0,0,0]=(0.310700158718)*x_ref[0]**o+(0.285647240187)*x_ref[0]+(0.817947571838)*x_ref[1]**o+(0.808698040916)*x_ref[1]+(0.844184006443)*x_ref[2]**o+(-0.310383215636)*x_ref[2]
        ref[3,0,0,1]=(-0.0173109952616)*x_ref[0]**o+(0.672034452037)*x_ref[0]+(-0.464572256298)*x_ref[1]**o+(0.205799523299)*x_ref[1]+(-0.971979841338)*x_ref[2]**o+(0.261515966001)*x_ref[2]
        ref[3,0,1,0]=(0.0329384377028)*x_ref[0]**o+(0.571501185072)*x_ref[0]+(-0.902222556642)*x_ref[1]**o+(-0.528317784373)*x_ref[1]+(0.331989640881)*x_ref[2]**o+(0.0427505624318)*x_ref[2]
        ref[3,0,1,1]=(0.964350410761)*x_ref[0]**o+(-0.327198989999)*x_ref[0]+(-0.195271523191)*x_ref[1]**o+(0.534663014601)*x_ref[1]+(-0.0208261208706)*x_ref[2]**o+(-0.158019249052)*x_ref[2]
        ref[3,0,2,0]=(-0.622082726008)*x_ref[0]**o+(0.559463883442)*x_ref[0]+(0.841439693313)*x_ref[1]**o+(-0.196363987516)*x_ref[1]+(-0.0143926547396)*x_ref[2]**o+(0.144732603689)*x_ref[2]
        ref[3,0,2,1]=(-0.149531653278)*x_ref[0]**o+(0.354229138322)*x_ref[0]+(-0.507916088829)*x_ref[1]**o+(0.221251580047)*x_ref[1]+(0.197213706274)*x_ref[2]**o+(-0.555645149276)*x_ref[2]
        ref[3,1,0,0]=(0.331020785374)*x_ref[0]**o+(-0.377647773513)*x_ref[0]+(0.563355928949)*x_ref[1]**o+(0.789309216711)*x_ref[1]+(0.203663383875)*x_ref[2]**o+(-0.0333894540645)*x_ref[2]
        ref[3,1,0,1]=(-0.0965613913882)*x_ref[0]**o+(0.18656484323)*x_ref[0]+(-0.138245941339)*x_ref[1]**o+(-0.34549538745)*x_ref[1]+(0.827173403167)*x_ref[2]**o+(-0.873380503939)*x_ref[2]
        ref[3,1,1,0]=(0.932940890033)*x_ref[0]**o+(-0.564883830288)*x_ref[0]+(-0.505052725295)*x_ref[1]**o+(-0.669420210206)*x_ref[1]+(-0.799638096039)*x_ref[2]**o+(0.469400233749)*x_ref[2]
        ref[3,1,1,1]=(-0.9741761069)*x_ref[0]**o+(-0.748125328781)*x_ref[0]+(0.664448241931)*x_ref[1]**o+(0.0957643296979)*x_ref[1]+(0.112211322019)*x_ref[2]**o+(-0.131870290434)*x_ref[2]
        ref[3,1,2,0]=(-0.438609127704)*x_ref[0]**o+(-0.36618286515)*x_ref[0]+(-0.376420011115)*x_ref[1]**o+(-0.67084624599)*x_ref[1]+(0.821679796644)*x_ref[2]**o+(0.80111319897)*x_ref[2]
        ref[3,1,2,1]=(-0.409859432085)*x_ref[0]**o+(0.848767816929)*x_ref[0]+(0.382358459835)*x_ref[1]**o+(-0.750139843631)*x_ref[1]+(0.587152586559)*x_ref[2]**o+(-0.26162952001)*x_ref[2]
        ref[3,2,0,0]=(-0.506329785074)*x_ref[0]**o+(-0.0967745752004)*x_ref[0]+(-0.163316795268)*x_ref[1]**o+(0.0628481921732)*x_ref[1]+(0.395355494367)*x_ref[2]**o+(0.0978901916332)*x_ref[2]
        ref[3,2,0,1]=(0.803590309587)*x_ref[0]**o+(0.470388053389)*x_ref[0]+(0.674942922168)*x_ref[1]**o+(0.223472834762)*x_ref[1]+(0.552332209028)*x_ref[2]**o+(-0.396443498843)*x_ref[2]
        ref[3,2,1,0]=(-0.312384723839)*x_ref[0]**o+(0.392213828608)*x_ref[0]+(0.449465316671)*x_ref[1]**o+(-0.249103089401)*x_ref[1]+(0.601412913458)*x_ref[2]**o+(0.209899368039)*x_ref[2]
        ref[3,2,1,1]=(-0.583776179557)*x_ref[0]**o+(0.485007047754)*x_ref[0]+(-0.0354153633589)*x_ref[1]**o+(-0.898630969916)*x_ref[1]+(-0.266581492795)*x_ref[2]**o+(0.822086680281)*x_ref[2]
        ref[3,2,2,0]=(0.944617094397)*x_ref[0]**o+(-0.173057994833)*x_ref[0]+(0.927832160048)*x_ref[1]**o+(-0.218599594252)*x_ref[1]+(0.349957755011)*x_ref[2]**o+(0.392602744545)*x_ref[2]
        ref[3,2,2,1]=(0.461648084674)*x_ref[0]**o+(0.826213376949)*x_ref[0]+(0.143429054731)*x_ref[1]**o+(-0.214265128007)*x_ref[1]+(0.0317208127607)*x_ref[2]**o+(-0.858361078381)*x_ref[2]
        ref[3,3,0,0]=(0.306607920368)*x_ref[0]**o+(0.147692891688)*x_ref[0]+(-0.562661766456)*x_ref[1]**o+(0.448595564727)*x_ref[1]+(-0.707178218373)*x_ref[2]**o+(0.0655554587866)*x_ref[2]
        ref[3,3,0,1]=(0.135475820362)*x_ref[0]**o+(0.51420718793)*x_ref[0]+(0.343291220736)*x_ref[1]**o+(-0.797532312602)*x_ref[1]+(-0.273431011273)*x_ref[2]**o+(0.135307779253)*x_ref[2]
        ref[3,3,1,0]=(0.488028231073)*x_ref[0]**o+(0.542021806829)*x_ref[0]+(0.91916268609)*x_ref[1]**o+(0.317368273365)*x_ref[1]+(0.314807039395)*x_ref[2]**o+(-0.841583308025)*x_ref[2]
        ref[3,3,1,1]=(-0.182783814917)*x_ref[0]**o+(0.250863481201)*x_ref[0]+(0.755551457003)*x_ref[1]**o+(0.946171548529)*x_ref[1]+(0.0350225436682)*x_ref[2]**o+(0.776059830052)*x_ref[2]
        ref[3,3,2,0]=(-0.116707124994)*x_ref[0]**o+(-0.727010280588)*x_ref[0]+(0.172065389268)*x_ref[1]**o+(0.754164564639)*x_ref[1]+(0.803777635621)*x_ref[2]**o+(0.293490264417)*x_ref[2]
        ref[3,3,2,1]=(0.154235982797)*x_ref[0]**o+(-0.124286021486)*x_ref[0]+(-0.737324748116)*x_ref[1]**o+(-0.136803541581)*x_ref[1]+(-0.608370191604)*x_ref[2]**o+(-0.933599034128)*x_ref[2]
        ref[3,4,0,0]=(0.818061677328)*x_ref[0]**o+(-0.771719613166)*x_ref[0]+(0.742772795251)*x_ref[1]**o+(0.0359285091312)*x_ref[1]+(-0.985656851837)*x_ref[2]**o+(-0.884598206512)*x_ref[2]
        ref[3,4,0,1]=(0.198371732741)*x_ref[0]**o+(0.646496618681)*x_ref[0]+(0.232912446494)*x_ref[1]**o+(0.945619492023)*x_ref[1]+(-0.344370341731)*x_ref[2]**o+(0.31156840357)*x_ref[2]
        ref[3,4,1,0]=(-0.609298834606)*x_ref[0]**o+(0.784697392412)*x_ref[0]+(0.996065619079)*x_ref[1]**o+(-0.284498300043)*x_ref[1]+(-0.652795706803)*x_ref[2]**o+(-0.254746840396)*x_ref[2]
        ref[3,4,1,1]=(-0.0083632822507)*x_ref[0]**o+(0.200923618933)*x_ref[0]+(0.353883930064)*x_ref[1]**o+(0.153257380179)*x_ref[1]+(0.843965464205)*x_ref[2]**o+(0.0418501400703)*x_ref[2]
        ref[3,4,2,0]=(-0.226055622191)*x_ref[0]**o+(-0.701789166696)*x_ref[0]+(0.274080957143)*x_ref[1]**o+(0.0596744660556)*x_ref[1]+(-0.342083152987)*x_ref[2]**o+(0.593518464697)*x_ref[2]
        ref[3,4,2,1]=(-0.288168945023)*x_ref[0]**o+(-0.742841678774)*x_ref[0]+(0.972629532424)*x_ref[1]**o+(0.892100560733)*x_ref[1]+(0.803880474721)*x_ref[2]**o+(-0.0638336211224)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromSymbol_ContinuousFunction_rank0(self):
      """
      tests interpolate for rank 0 Symbol onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ContinuousFunction onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(),dim=dim)
      s=Data(0,(),w)
      ref=Data(0,(),w_ref)
      if dim==2:
        s=(0.207629351539)*x[0]**o+(0.601153935253)*x[0]+(0.280159096838)*x[1]**o+(-0.327076958891)*x[1]
        ref=(0.207629351539)*x_ref[0]**o+(0.601153935253)*x_ref[0]+(0.280159096838)*x_ref[1]**o+(-0.327076958891)*x_ref[1]
      else:
        s=(0.870140941982)*x[0]**o+(0.698352761373)*x[0]+(0.143773825221)*x[1]**o+(-0.906122820343)*x[1]+(-0.479637063866)*x[2]**o+(0.76395958357)*x[2]
        ref=(0.870140941982)*x_ref[0]**o+(0.698352761373)*x_ref[0]+(0.143773825221)*x_ref[1]**o+(-0.906122820343)*x_ref[1]+(-0.479637063866)*x_ref[2]**o+(0.76395958357)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromSymbol_ContinuousFunction_rank1(self):
      """
      tests interpolate for rank 1 Symbol onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ContinuousFunction onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(2,),dim=dim)
      s=Data(0,(2,),w)
      ref=Data(0,(2,),w_ref)
      if dim==2:
        s[0]=(-0.218806478928)*x[0]**o+(0.377016390463)*x[0]+(-0.821557051729)*x[1]**o+(-0.291850215951)*x[1]
        s[1]=(0.268489780039)*x[0]**o+(0.682467861682)*x[0]+(0.247435448778)*x[1]**o+(0.299818969839)*x[1]
        ref[0]=(-0.218806478928)*x_ref[0]**o+(0.377016390463)*x_ref[0]+(-0.821557051729)*x_ref[1]**o+(-0.291850215951)*x_ref[1]
        ref[1]=(0.268489780039)*x_ref[0]**o+(0.682467861682)*x_ref[0]+(0.247435448778)*x_ref[1]**o+(0.299818969839)*x_ref[1]
      else:
        s[0]=(0.45901051382)*x[0]**o+(-0.79161426979)*x[0]+(0.447653516381)*x[1]**o+(-0.50384199989)*x[1]+(0.20809418393)*x[2]**o+(0.978047066867)*x[2]
        s[1]=(-0.383595164108)*x[0]**o+(-0.343736899245)*x[0]+(0.0396873129716)*x[1]**o+(-0.385915355146)*x[1]+(-0.853533231426)*x[2]**o+(0.792420068965)*x[2]
        ref[0]=(0.45901051382)*x_ref[0]**o+(-0.79161426979)*x_ref[0]+(0.447653516381)*x_ref[1]**o+(-0.50384199989)*x_ref[1]+(0.20809418393)*x_ref[2]**o+(0.978047066867)*x_ref[2]
        ref[1]=(-0.383595164108)*x_ref[0]**o+(-0.343736899245)*x_ref[0]+(0.0396873129716)*x_ref[1]**o+(-0.385915355146)*x_ref[1]+(-0.853533231426)*x_ref[2]**o+(0.792420068965)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromSymbol_ContinuousFunction_rank2(self):
      """
      tests interpolate for rank 2 Symbol onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ContinuousFunction onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5),dim=dim)
      s=Data(0,(4, 5),w)
      ref=Data(0,(4, 5),w_ref)
      if dim==2:
        s[0,0]=(-0.0353181613333)*x[0]**o+(0.878876352154)*x[0]+(-0.904935867476)*x[1]**o+(-0.321624539514)*x[1]
        s[0,1]=(-0.288092527799)*x[0]**o+(0.928073219222)*x[0]+(-0.110185302788)*x[1]**o+(-0.891915948247)*x[1]
        s[0,2]=(0.0954349617972)*x[0]**o+(-0.140083515966)*x[0]+(-0.827213026204)*x[1]**o+(-0.910012731257)*x[1]
        s[0,3]=(-0.565938692108)*x[0]**o+(0.991031920541)*x[0]+(-0.582526646689)*x[1]**o+(0.0715276554003)*x[1]
        s[0,4]=(0.621703736822)*x[0]**o+(-0.123891976545)*x[0]+(0.284804346655)*x[1]**o+(0.712402142502)*x[1]
        s[1,0]=(0.406855284727)*x[0]**o+(0.481423856624)*x[0]+(0.196078127047)*x[1]**o+(-0.514004426398)*x[1]
        s[1,1]=(-0.378073499305)*x[0]**o+(0.473337934464)*x[0]+(-0.055111166928)*x[1]**o+(-0.0436034347619)*x[1]
        s[1,2]=(-0.124152152221)*x[0]**o+(-0.420813454028)*x[0]+(0.569674875796)*x[1]**o+(-0.101882853127)*x[1]
        s[1,3]=(-0.528314793712)*x[0]**o+(-0.693704904591)*x[0]+(0.676975909426)*x[1]**o+(0.150922079384)*x[1]
        s[1,4]=(-0.738713706919)*x[0]**o+(-0.366958606319)*x[0]+(0.634632816865)*x[1]**o+(0.793822443349)*x[1]
        s[2,0]=(-0.743224960117)*x[0]**o+(0.885444739748)*x[0]+(-0.569356473669)*x[1]**o+(0.0554721350962)*x[1]
        s[2,1]=(-0.685934253863)*x[0]**o+(0.00101896345091)*x[0]+(0.756736504806)*x[1]**o+(-0.823095371277)*x[1]
        s[2,2]=(-0.220514216102)*x[0]**o+(-0.710824329182)*x[0]+(0.838443592125)*x[1]**o+(0.737338692368)*x[1]
        s[2,3]=(0.386990571268)*x[0]**o+(-0.754808523044)*x[0]+(0.259729871523)*x[1]**o+(0.757370896746)*x[1]
        s[2,4]=(0.942933704541)*x[0]**o+(-0.37055099742)*x[0]+(-0.807891788882)*x[1]**o+(0.875578199856)*x[1]
        s[3,0]=(-0.288294610358)*x[0]**o+(-0.547708565934)*x[0]+(0.875419437633)*x[1]**o+(0.25641477553)*x[1]
        s[3,1]=(-0.283600102851)*x[0]**o+(0.682131874062)*x[0]+(-0.124142296952)*x[1]**o+(0.122898656305)*x[1]
        s[3,2]=(0.714735953181)*x[0]**o+(-0.190821251504)*x[0]+(0.273433212059)*x[1]**o+(0.812718970292)*x[1]
        s[3,3]=(0.338475492958)*x[0]**o+(0.863340300202)*x[0]+(-0.512598335671)*x[1]**o+(0.219919160346)*x[1]
        s[3,4]=(-0.735913321392)*x[0]**o+(-0.599322689823)*x[0]+(-0.69249277704)*x[1]**o+(-0.665352731149)*x[1]
        ref[0,0]=(-0.0353181613333)*x_ref[0]**o+(0.878876352154)*x_ref[0]+(-0.904935867476)*x_ref[1]**o+(-0.321624539514)*x_ref[1]
        ref[0,1]=(-0.288092527799)*x_ref[0]**o+(0.928073219222)*x_ref[0]+(-0.110185302788)*x_ref[1]**o+(-0.891915948247)*x_ref[1]
        ref[0,2]=(0.0954349617972)*x_ref[0]**o+(-0.140083515966)*x_ref[0]+(-0.827213026204)*x_ref[1]**o+(-0.910012731257)*x_ref[1]
        ref[0,3]=(-0.565938692108)*x_ref[0]**o+(0.991031920541)*x_ref[0]+(-0.582526646689)*x_ref[1]**o+(0.0715276554003)*x_ref[1]
        ref[0,4]=(0.621703736822)*x_ref[0]**o+(-0.123891976545)*x_ref[0]+(0.284804346655)*x_ref[1]**o+(0.712402142502)*x_ref[1]
        ref[1,0]=(0.406855284727)*x_ref[0]**o+(0.481423856624)*x_ref[0]+(0.196078127047)*x_ref[1]**o+(-0.514004426398)*x_ref[1]
        ref[1,1]=(-0.378073499305)*x_ref[0]**o+(0.473337934464)*x_ref[0]+(-0.055111166928)*x_ref[1]**o+(-0.0436034347619)*x_ref[1]
        ref[1,2]=(-0.124152152221)*x_ref[0]**o+(-0.420813454028)*x_ref[0]+(0.569674875796)*x_ref[1]**o+(-0.101882853127)*x_ref[1]
        ref[1,3]=(-0.528314793712)*x_ref[0]**o+(-0.693704904591)*x_ref[0]+(0.676975909426)*x_ref[1]**o+(0.150922079384)*x_ref[1]
        ref[1,4]=(-0.738713706919)*x_ref[0]**o+(-0.366958606319)*x_ref[0]+(0.634632816865)*x_ref[1]**o+(0.793822443349)*x_ref[1]
        ref[2,0]=(-0.743224960117)*x_ref[0]**o+(0.885444739748)*x_ref[0]+(-0.569356473669)*x_ref[1]**o+(0.0554721350962)*x_ref[1]
        ref[2,1]=(-0.685934253863)*x_ref[0]**o+(0.00101896345091)*x_ref[0]+(0.756736504806)*x_ref[1]**o+(-0.823095371277)*x_ref[1]
        ref[2,2]=(-0.220514216102)*x_ref[0]**o+(-0.710824329182)*x_ref[0]+(0.838443592125)*x_ref[1]**o+(0.737338692368)*x_ref[1]
        ref[2,3]=(0.386990571268)*x_ref[0]**o+(-0.754808523044)*x_ref[0]+(0.259729871523)*x_ref[1]**o+(0.757370896746)*x_ref[1]
        ref[2,4]=(0.942933704541)*x_ref[0]**o+(-0.37055099742)*x_ref[0]+(-0.807891788882)*x_ref[1]**o+(0.875578199856)*x_ref[1]
        ref[3,0]=(-0.288294610358)*x_ref[0]**o+(-0.547708565934)*x_ref[0]+(0.875419437633)*x_ref[1]**o+(0.25641477553)*x_ref[1]
        ref[3,1]=(-0.283600102851)*x_ref[0]**o+(0.682131874062)*x_ref[0]+(-0.124142296952)*x_ref[1]**o+(0.122898656305)*x_ref[1]
        ref[3,2]=(0.714735953181)*x_ref[0]**o+(-0.190821251504)*x_ref[0]+(0.273433212059)*x_ref[1]**o+(0.812718970292)*x_ref[1]
        ref[3,3]=(0.338475492958)*x_ref[0]**o+(0.863340300202)*x_ref[0]+(-0.512598335671)*x_ref[1]**o+(0.219919160346)*x_ref[1]
        ref[3,4]=(-0.735913321392)*x_ref[0]**o+(-0.599322689823)*x_ref[0]+(-0.69249277704)*x_ref[1]**o+(-0.665352731149)*x_ref[1]
      else:
        s[0,0]=(0.430293433102)*x[0]**o+(-0.857172626308)*x[0]+(0.868470915818)*x[1]**o+(-0.882235724566)*x[1]+(0.284271991649)*x[2]**o+(-0.459504355652)*x[2]
        s[0,1]=(-0.679633702459)*x[0]**o+(-0.222491254643)*x[0]+(0.286851801882)*x[1]**o+(-0.844507424335)*x[1]+(-0.808810961797)*x[2]**o+(0.139670580428)*x[2]
        s[0,2]=(0.682748402275)*x[0]**o+(-0.392534760514)*x[0]+(0.716754522284)*x[1]**o+(-0.151837258954)*x[1]+(0.0826469913138)*x[2]**o+(-0.316631816663)*x[2]
        s[0,3]=(0.551943486674)*x[0]**o+(0.946320480258)*x[0]+(0.75350645127)*x[1]**o+(0.543509582247)*x[1]+(-0.97668108246)*x[2]**o+(-0.910891750491)*x[2]
        s[0,4]=(-0.317631622813)*x[0]**o+(0.647217229231)*x[0]+(0.770238984569)*x[1]**o+(0.873217492832)*x[1]+(0.512276250922)*x[2]**o+(0.282068518751)*x[2]
        s[1,0]=(-0.23327675725)*x[0]**o+(0.562413317314)*x[0]+(-0.0812857523424)*x[1]**o+(-0.721924983109)*x[1]+(0.775846134529)*x[2]**o+(-0.14640332198)*x[2]
        s[1,1]=(0.00265339866254)*x[0]**o+(0.486208331625)*x[0]+(-0.530584135285)*x[1]**o+(-0.84550455983)*x[1]+(-0.296142439781)*x[2]**o+(-0.595262357569)*x[2]
        s[1,2]=(0.985928898395)*x[0]**o+(-0.681211427726)*x[0]+(0.926263452555)*x[1]**o+(-0.435493892249)*x[1]+(-0.312630433899)*x[2]**o+(0.584197447454)*x[2]
        s[1,3]=(-0.558671284798)*x[0]**o+(0.837591478004)*x[0]+(0.00535393855178)*x[1]**o+(-0.662459914024)*x[1]+(0.558541008138)*x[2]**o+(-0.314047626792)*x[2]
        s[1,4]=(-0.300759983439)*x[0]**o+(0.354828936226)*x[0]+(-0.986139955736)*x[1]**o+(0.202600852457)*x[1]+(-0.927794980602)*x[2]**o+(-0.359202012865)*x[2]
        s[2,0]=(0.829016217959)*x[0]**o+(-0.423599278647)*x[0]+(-0.40846564457)*x[1]**o+(-0.869126229204)*x[1]+(0.473022094753)*x[2]**o+(0.910852581174)*x[2]
        s[2,1]=(-0.706119428033)*x[0]**o+(0.0566333827809)*x[0]+(0.809800273677)*x[1]**o+(-0.588604664572)*x[1]+(-0.285988770457)*x[2]**o+(-0.904317124653)*x[2]
        s[2,2]=(0.449032734821)*x[0]**o+(0.896383171746)*x[0]+(-0.852643286424)*x[1]**o+(0.650126054768)*x[1]+(0.255148390617)*x[2]**o+(0.38493564187)*x[2]
        s[2,3]=(0.269546556328)*x[0]**o+(0.217209115989)*x[0]+(0.0299948255851)*x[1]**o+(0.966502659562)*x[1]+(-0.837125730275)*x[2]**o+(0.192194381152)*x[2]
        s[2,4]=(0.749886797267)*x[0]**o+(-0.358784003038)*x[0]+(0.458790223998)*x[1]**o+(-0.868481406026)*x[1]+(-0.856425916274)*x[2]**o+(-0.177865937666)*x[2]
        s[3,0]=(0.944647107347)*x[0]**o+(-0.599969226039)*x[0]+(0.606271189912)*x[1]**o+(0.515347418734)*x[1]+(-0.239547882059)*x[2]**o+(0.179383932895)*x[2]
        s[3,1]=(0.202273995087)*x[0]**o+(0.559445783871)*x[0]+(-0.358032565476)*x[1]**o+(-0.512085149684)*x[1]+(0.888993422451)*x[2]**o+(0.0854392659772)*x[2]
        s[3,2]=(0.367717494858)*x[0]**o+(-0.14816870753)*x[0]+(0.99339026962)*x[1]**o+(0.705222667312)*x[1]+(0.700295737391)*x[2]**o+(0.403996497394)*x[2]
        s[3,3]=(-0.104867279309)*x[0]**o+(0.814918041051)*x[0]+(0.252610473279)*x[1]**o+(-0.744193407199)*x[1]+(0.0189009064209)*x[2]**o+(0.314571922242)*x[2]
        s[3,4]=(-0.622691514266)*x[0]**o+(-0.964201372823)*x[0]+(-0.411043099228)*x[1]**o+(0.142020161136)*x[1]+(-0.587923250915)*x[2]**o+(0.37852533939)*x[2]
        ref[0,0]=(0.430293433102)*x_ref[0]**o+(-0.857172626308)*x_ref[0]+(0.868470915818)*x_ref[1]**o+(-0.882235724566)*x_ref[1]+(0.284271991649)*x_ref[2]**o+(-0.459504355652)*x_ref[2]
        ref[0,1]=(-0.679633702459)*x_ref[0]**o+(-0.222491254643)*x_ref[0]+(0.286851801882)*x_ref[1]**o+(-0.844507424335)*x_ref[1]+(-0.808810961797)*x_ref[2]**o+(0.139670580428)*x_ref[2]
        ref[0,2]=(0.682748402275)*x_ref[0]**o+(-0.392534760514)*x_ref[0]+(0.716754522284)*x_ref[1]**o+(-0.151837258954)*x_ref[1]+(0.0826469913138)*x_ref[2]**o+(-0.316631816663)*x_ref[2]
        ref[0,3]=(0.551943486674)*x_ref[0]**o+(0.946320480258)*x_ref[0]+(0.75350645127)*x_ref[1]**o+(0.543509582247)*x_ref[1]+(-0.97668108246)*x_ref[2]**o+(-0.910891750491)*x_ref[2]
        ref[0,4]=(-0.317631622813)*x_ref[0]**o+(0.647217229231)*x_ref[0]+(0.770238984569)*x_ref[1]**o+(0.873217492832)*x_ref[1]+(0.512276250922)*x_ref[2]**o+(0.282068518751)*x_ref[2]
        ref[1,0]=(-0.23327675725)*x_ref[0]**o+(0.562413317314)*x_ref[0]+(-0.0812857523424)*x_ref[1]**o+(-0.721924983109)*x_ref[1]+(0.775846134529)*x_ref[2]**o+(-0.14640332198)*x_ref[2]
        ref[1,1]=(0.00265339866254)*x_ref[0]**o+(0.486208331625)*x_ref[0]+(-0.530584135285)*x_ref[1]**o+(-0.84550455983)*x_ref[1]+(-0.296142439781)*x_ref[2]**o+(-0.595262357569)*x_ref[2]
        ref[1,2]=(0.985928898395)*x_ref[0]**o+(-0.681211427726)*x_ref[0]+(0.926263452555)*x_ref[1]**o+(-0.435493892249)*x_ref[1]+(-0.312630433899)*x_ref[2]**o+(0.584197447454)*x_ref[2]
        ref[1,3]=(-0.558671284798)*x_ref[0]**o+(0.837591478004)*x_ref[0]+(0.00535393855178)*x_ref[1]**o+(-0.662459914024)*x_ref[1]+(0.558541008138)*x_ref[2]**o+(-0.314047626792)*x_ref[2]
        ref[1,4]=(-0.300759983439)*x_ref[0]**o+(0.354828936226)*x_ref[0]+(-0.986139955736)*x_ref[1]**o+(0.202600852457)*x_ref[1]+(-0.927794980602)*x_ref[2]**o+(-0.359202012865)*x_ref[2]
        ref[2,0]=(0.829016217959)*x_ref[0]**o+(-0.423599278647)*x_ref[0]+(-0.40846564457)*x_ref[1]**o+(-0.869126229204)*x_ref[1]+(0.473022094753)*x_ref[2]**o+(0.910852581174)*x_ref[2]
        ref[2,1]=(-0.706119428033)*x_ref[0]**o+(0.0566333827809)*x_ref[0]+(0.809800273677)*x_ref[1]**o+(-0.588604664572)*x_ref[1]+(-0.285988770457)*x_ref[2]**o+(-0.904317124653)*x_ref[2]
        ref[2,2]=(0.449032734821)*x_ref[0]**o+(0.896383171746)*x_ref[0]+(-0.852643286424)*x_ref[1]**o+(0.650126054768)*x_ref[1]+(0.255148390617)*x_ref[2]**o+(0.38493564187)*x_ref[2]
        ref[2,3]=(0.269546556328)*x_ref[0]**o+(0.217209115989)*x_ref[0]+(0.0299948255851)*x_ref[1]**o+(0.966502659562)*x_ref[1]+(-0.837125730275)*x_ref[2]**o+(0.192194381152)*x_ref[2]
        ref[2,4]=(0.749886797267)*x_ref[0]**o+(-0.358784003038)*x_ref[0]+(0.458790223998)*x_ref[1]**o+(-0.868481406026)*x_ref[1]+(-0.856425916274)*x_ref[2]**o+(-0.177865937666)*x_ref[2]
        ref[3,0]=(0.944647107347)*x_ref[0]**o+(-0.599969226039)*x_ref[0]+(0.606271189912)*x_ref[1]**o+(0.515347418734)*x_ref[1]+(-0.239547882059)*x_ref[2]**o+(0.179383932895)*x_ref[2]
        ref[3,1]=(0.202273995087)*x_ref[0]**o+(0.559445783871)*x_ref[0]+(-0.358032565476)*x_ref[1]**o+(-0.512085149684)*x_ref[1]+(0.888993422451)*x_ref[2]**o+(0.0854392659772)*x_ref[2]
        ref[3,2]=(0.367717494858)*x_ref[0]**o+(-0.14816870753)*x_ref[0]+(0.99339026962)*x_ref[1]**o+(0.705222667312)*x_ref[1]+(0.700295737391)*x_ref[2]**o+(0.403996497394)*x_ref[2]
        ref[3,3]=(-0.104867279309)*x_ref[0]**o+(0.814918041051)*x_ref[0]+(0.252610473279)*x_ref[1]**o+(-0.744193407199)*x_ref[1]+(0.0189009064209)*x_ref[2]**o+(0.314571922242)*x_ref[2]
        ref[3,4]=(-0.622691514266)*x_ref[0]**o+(-0.964201372823)*x_ref[0]+(-0.411043099228)*x_ref[1]**o+(0.142020161136)*x_ref[1]+(-0.587923250915)*x_ref[2]**o+(0.37852533939)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromSymbol_ContinuousFunction_rank3(self):
      """
      tests interpolate for rank 3 Symbol onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ContinuousFunction onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(6, 2, 2),dim=dim)
      s=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2),w_ref)
      if dim==2:
        s[0,0,0]=(0.486126148472)*x[0]**o+(-0.225402743005)*x[0]+(0.182927976639)*x[1]**o+(0.620824619579)*x[1]
        s[0,0,1]=(-0.407409056531)*x[0]**o+(-0.833838057228)*x[0]+(0.644035353162)*x[1]**o+(-0.672679458606)*x[1]
        s[0,1,0]=(-0.581856850287)*x[0]**o+(-0.0618093338242)*x[0]+(0.684919787457)*x[1]**o+(-0.788915767932)*x[1]
        s[0,1,1]=(-0.224040348858)*x[0]**o+(-0.103067046916)*x[0]+(-0.578696291301)*x[1]**o+(0.00211758484966)*x[1]
        s[1,0,0]=(0.979651701027)*x[0]**o+(-0.106752935345)*x[0]+(0.109182674529)*x[1]**o+(-0.487877441364)*x[1]
        s[1,0,1]=(0.108662013179)*x[0]**o+(0.309241035214)*x[0]+(0.666605783286)*x[1]**o+(-0.631752854289)*x[1]
        s[1,1,0]=(0.171413355921)*x[0]**o+(-0.468708755515)*x[0]+(0.668398470024)*x[1]**o+(0.12499650775)*x[1]
        s[1,1,1]=(0.203364381723)*x[0]**o+(0.149271421925)*x[0]+(0.0393327688768)*x[1]**o+(-0.342283671904)*x[1]
        s[2,0,0]=(0.426437033887)*x[0]**o+(-0.0681592307883)*x[0]+(-0.96231688885)*x[1]**o+(-0.74406098717)*x[1]
        s[2,0,1]=(0.817824016472)*x[0]**o+(-0.0332805444397)*x[0]+(0.274976215212)*x[1]**o+(0.0486739871191)*x[1]
        s[2,1,0]=(0.201535943822)*x[0]**o+(-0.620687964831)*x[0]+(0.675814320754)*x[1]**o+(-0.865640523715)*x[1]
        s[2,1,1]=(-0.89105770032)*x[0]**o+(0.730291041637)*x[0]+(-0.0379223326195)*x[1]**o+(0.874877694224)*x[1]
        s[3,0,0]=(-0.562138233363)*x[0]**o+(-0.211931551199)*x[0]+(0.245533602059)*x[1]**o+(-0.752149846924)*x[1]
        s[3,0,1]=(-0.54251114713)*x[0]**o+(0.663375258793)*x[0]+(0.787787067042)*x[1]**o+(-0.128869775715)*x[1]
        s[3,1,0]=(-0.317250597186)*x[0]**o+(-0.92812680953)*x[0]+(-0.787378775549)*x[1]**o+(0.0495460276628)*x[1]
        s[3,1,1]=(0.41215674863)*x[0]**o+(-0.647351893065)*x[0]+(0.910056539934)*x[1]**o+(-0.931747263108)*x[1]
        s[4,0,0]=(-0.239940821453)*x[0]**o+(0.811323165415)*x[0]+(0.524171970463)*x[1]**o+(0.919862325215)*x[1]
        s[4,0,1]=(-0.812864948023)*x[0]**o+(0.966779570342)*x[0]+(-0.528240223073)*x[1]**o+(0.532264572013)*x[1]
        s[4,1,0]=(-0.905620729062)*x[0]**o+(-0.532558330506)*x[0]+(-0.828115659854)*x[1]**o+(-0.0539571069974)*x[1]
        s[4,1,1]=(-0.456328687883)*x[0]**o+(0.198344289976)*x[0]+(0.255309661612)*x[1]**o+(-0.530390597461)*x[1]
        s[5,0,0]=(-0.525932317875)*x[0]**o+(0.959390874707)*x[0]+(-0.860006819486)*x[1]**o+(0.485364243112)*x[1]
        s[5,0,1]=(0.41405164663)*x[0]**o+(0.738240577168)*x[0]+(-0.158776067456)*x[1]**o+(-0.0120083238952)*x[1]
        s[5,1,0]=(0.915095618736)*x[0]**o+(-0.180612310391)*x[0]+(0.859456959672)*x[1]**o+(-0.230245385712)*x[1]
        s[5,1,1]=(0.195626998508)*x[0]**o+(-0.895957896446)*x[0]+(-0.674564217018)*x[1]**o+(0.182147320643)*x[1]
        ref[0,0,0]=(0.486126148472)*x_ref[0]**o+(-0.225402743005)*x_ref[0]+(0.182927976639)*x_ref[1]**o+(0.620824619579)*x_ref[1]
        ref[0,0,1]=(-0.407409056531)*x_ref[0]**o+(-0.833838057228)*x_ref[0]+(0.644035353162)*x_ref[1]**o+(-0.672679458606)*x_ref[1]
        ref[0,1,0]=(-0.581856850287)*x_ref[0]**o+(-0.0618093338242)*x_ref[0]+(0.684919787457)*x_ref[1]**o+(-0.788915767932)*x_ref[1]
        ref[0,1,1]=(-0.224040348858)*x_ref[0]**o+(-0.103067046916)*x_ref[0]+(-0.578696291301)*x_ref[1]**o+(0.00211758484966)*x_ref[1]
        ref[1,0,0]=(0.979651701027)*x_ref[0]**o+(-0.106752935345)*x_ref[0]+(0.109182674529)*x_ref[1]**o+(-0.487877441364)*x_ref[1]
        ref[1,0,1]=(0.108662013179)*x_ref[0]**o+(0.309241035214)*x_ref[0]+(0.666605783286)*x_ref[1]**o+(-0.631752854289)*x_ref[1]
        ref[1,1,0]=(0.171413355921)*x_ref[0]**o+(-0.468708755515)*x_ref[0]+(0.668398470024)*x_ref[1]**o+(0.12499650775)*x_ref[1]
        ref[1,1,1]=(0.203364381723)*x_ref[0]**o+(0.149271421925)*x_ref[0]+(0.0393327688768)*x_ref[1]**o+(-0.342283671904)*x_ref[1]
        ref[2,0,0]=(0.426437033887)*x_ref[0]**o+(-0.0681592307883)*x_ref[0]+(-0.96231688885)*x_ref[1]**o+(-0.74406098717)*x_ref[1]
        ref[2,0,1]=(0.817824016472)*x_ref[0]**o+(-0.0332805444397)*x_ref[0]+(0.274976215212)*x_ref[1]**o+(0.0486739871191)*x_ref[1]
        ref[2,1,0]=(0.201535943822)*x_ref[0]**o+(-0.620687964831)*x_ref[0]+(0.675814320754)*x_ref[1]**o+(-0.865640523715)*x_ref[1]
        ref[2,1,1]=(-0.89105770032)*x_ref[0]**o+(0.730291041637)*x_ref[0]+(-0.0379223326195)*x_ref[1]**o+(0.874877694224)*x_ref[1]
        ref[3,0,0]=(-0.562138233363)*x_ref[0]**o+(-0.211931551199)*x_ref[0]+(0.245533602059)*x_ref[1]**o+(-0.752149846924)*x_ref[1]
        ref[3,0,1]=(-0.54251114713)*x_ref[0]**o+(0.663375258793)*x_ref[0]+(0.787787067042)*x_ref[1]**o+(-0.128869775715)*x_ref[1]
        ref[3,1,0]=(-0.317250597186)*x_ref[0]**o+(-0.92812680953)*x_ref[0]+(-0.787378775549)*x_ref[1]**o+(0.0495460276628)*x_ref[1]
        ref[3,1,1]=(0.41215674863)*x_ref[0]**o+(-0.647351893065)*x_ref[0]+(0.910056539934)*x_ref[1]**o+(-0.931747263108)*x_ref[1]
        ref[4,0,0]=(-0.239940821453)*x_ref[0]**o+(0.811323165415)*x_ref[0]+(0.524171970463)*x_ref[1]**o+(0.919862325215)*x_ref[1]
        ref[4,0,1]=(-0.812864948023)*x_ref[0]**o+(0.966779570342)*x_ref[0]+(-0.528240223073)*x_ref[1]**o+(0.532264572013)*x_ref[1]
        ref[4,1,0]=(-0.905620729062)*x_ref[0]**o+(-0.532558330506)*x_ref[0]+(-0.828115659854)*x_ref[1]**o+(-0.0539571069974)*x_ref[1]
        ref[4,1,1]=(-0.456328687883)*x_ref[0]**o+(0.198344289976)*x_ref[0]+(0.255309661612)*x_ref[1]**o+(-0.530390597461)*x_ref[1]
        ref[5,0,0]=(-0.525932317875)*x_ref[0]**o+(0.959390874707)*x_ref[0]+(-0.860006819486)*x_ref[1]**o+(0.485364243112)*x_ref[1]
        ref[5,0,1]=(0.41405164663)*x_ref[0]**o+(0.738240577168)*x_ref[0]+(-0.158776067456)*x_ref[1]**o+(-0.0120083238952)*x_ref[1]
        ref[5,1,0]=(0.915095618736)*x_ref[0]**o+(-0.180612310391)*x_ref[0]+(0.859456959672)*x_ref[1]**o+(-0.230245385712)*x_ref[1]
        ref[5,1,1]=(0.195626998508)*x_ref[0]**o+(-0.895957896446)*x_ref[0]+(-0.674564217018)*x_ref[1]**o+(0.182147320643)*x_ref[1]
      else:
        s[0,0,0]=(-0.386562865086)*x[0]**o+(-0.101923658127)*x[0]+(0.366033434213)*x[1]**o+(-0.458022094261)*x[1]+(-0.269807341125)*x[2]**o+(0.210015891017)*x[2]
        s[0,0,1]=(-0.232605620109)*x[0]**o+(-0.89635125143)*x[0]+(-0.344907210539)*x[1]**o+(-0.117272497457)*x[1]+(0.86877312028)*x[2]**o+(-0.984463097169)*x[2]
        s[0,1,0]=(-0.734186269208)*x[0]**o+(-0.955220234973)*x[0]+(0.355373019012)*x[1]**o+(-0.0259995318533)*x[1]+(-0.923817606426)*x[2]**o+(0.707611624363)*x[2]
        s[0,1,1]=(-0.65807304305)*x[0]**o+(-0.546854762897)*x[0]+(-0.041383885518)*x[1]**o+(-0.847656164394)*x[1]+(-0.69366771235)*x[2]**o+(-0.00594720800349)*x[2]
        s[1,0,0]=(0.4916488707)*x[0]**o+(0.610932307599)*x[0]+(-0.634836490924)*x[1]**o+(0.190377750049)*x[1]+(0.839648718767)*x[2]**o+(-0.697199112832)*x[2]
        s[1,0,1]=(-0.733462530162)*x[0]**o+(0.897438124381)*x[0]+(-0.284428270428)*x[1]**o+(-0.490325551864)*x[1]+(-0.425270047571)*x[2]**o+(-0.0169167898593)*x[2]
        s[1,1,0]=(-0.1134205794)*x[0]**o+(-0.589938051075)*x[0]+(-0.551954550243)*x[1]**o+(-0.828713609462)*x[1]+(-0.14740902465)*x[2]**o+(-0.0643506708023)*x[2]
        s[1,1,1]=(0.875176575479)*x[0]**o+(-0.436379792297)*x[0]+(-0.238769521024)*x[1]**o+(-0.0883649176657)*x[1]+(-0.268393504724)*x[2]**o+(-0.331629361371)*x[2]
        s[2,0,0]=(-0.445061932808)*x[0]**o+(-0.136283305342)*x[0]+(0.109782590986)*x[1]**o+(0.664151679353)*x[1]+(0.77415319639)*x[2]**o+(0.918600325668)*x[2]
        s[2,0,1]=(-0.966582148735)*x[0]**o+(0.90850899638)*x[0]+(0.860043581705)*x[1]**o+(0.555164541705)*x[1]+(-0.0881641948474)*x[2]**o+(-0.437393798496)*x[2]
        s[2,1,0]=(0.424061230847)*x[0]**o+(0.426231869756)*x[0]+(0.20042430919)*x[1]**o+(-0.392259834985)*x[1]+(0.56903598931)*x[2]**o+(-0.713346863994)*x[2]
        s[2,1,1]=(0.391005749137)*x[0]**o+(-0.141533299622)*x[0]+(-0.54549941972)*x[1]**o+(-0.551542113521)*x[1]+(0.461175241452)*x[2]**o+(0.946400145746)*x[2]
        s[3,0,0]=(0.690778432543)*x[0]**o+(0.0987975519633)*x[0]+(0.600232607252)*x[1]**o+(-0.403160580412)*x[1]+(-0.259142718629)*x[2]**o+(0.125408478863)*x[2]
        s[3,0,1]=(0.45235548032)*x[0]**o+(0.465495192061)*x[0]+(0.605495566594)*x[1]**o+(0.0665669455329)*x[1]+(-0.528374124117)*x[2]**o+(0.971371028924)*x[2]
        s[3,1,0]=(-0.868740120635)*x[0]**o+(0.94078043392)*x[0]+(-0.924836120128)*x[1]**o+(0.0436824783429)*x[1]+(0.436490992322)*x[2]**o+(0.563162883509)*x[2]
        s[3,1,1]=(-0.148701893094)*x[0]**o+(0.114671183607)*x[0]+(0.40845144069)*x[1]**o+(-0.35749472879)*x[1]+(-0.240456109591)*x[2]**o+(-0.959187876519)*x[2]
        s[4,0,0]=(-0.902322503602)*x[0]**o+(-0.0870613571753)*x[0]+(0.604597216194)*x[1]**o+(0.484694096025)*x[1]+(0.733216575545)*x[2]**o+(0.981143903885)*x[2]
        s[4,0,1]=(-0.9183984444)*x[0]**o+(0.222613734722)*x[0]+(0.472361095251)*x[1]**o+(-0.182413993184)*x[1]+(0.404737782329)*x[2]**o+(-0.462634159071)*x[2]
        s[4,1,0]=(-0.577317659134)*x[0]**o+(0.863519222199)*x[0]+(-0.955859899586)*x[1]**o+(0.898032633657)*x[1]+(0.000203504014658)*x[2]**o+(0.84858789073)*x[2]
        s[4,1,1]=(0.103931145317)*x[0]**o+(-0.491733208237)*x[0]+(-0.679616928853)*x[1]**o+(0.779966695883)*x[1]+(0.19391391729)*x[2]**o+(0.237260204789)*x[2]
        s[5,0,0]=(-0.587657719833)*x[0]**o+(0.0677059438625)*x[0]+(-0.070159254762)*x[1]**o+(-0.725676771128)*x[1]+(0.563139108018)*x[2]**o+(-0.182136289146)*x[2]
        s[5,0,1]=(0.647754539947)*x[0]**o+(-0.238569428669)*x[0]+(-0.0352701585821)*x[1]**o+(0.117211931696)*x[1]+(-0.749719337272)*x[2]**o+(-0.0572910818861)*x[2]
        s[5,1,0]=(0.0802652788129)*x[0]**o+(-0.320772823979)*x[0]+(0.875989601034)*x[1]**o+(0.56803751787)*x[1]+(0.115080146257)*x[2]**o+(0.960937482841)*x[2]
        s[5,1,1]=(0.374333508365)*x[0]**o+(-0.198321408411)*x[0]+(0.550730384308)*x[1]**o+(0.980522161961)*x[1]+(-0.523593924556)*x[2]**o+(-0.0882497539881)*x[2]
        ref[0,0,0]=(-0.386562865086)*x_ref[0]**o+(-0.101923658127)*x_ref[0]+(0.366033434213)*x_ref[1]**o+(-0.458022094261)*x_ref[1]+(-0.269807341125)*x_ref[2]**o+(0.210015891017)*x_ref[2]
        ref[0,0,1]=(-0.232605620109)*x_ref[0]**o+(-0.89635125143)*x_ref[0]+(-0.344907210539)*x_ref[1]**o+(-0.117272497457)*x_ref[1]+(0.86877312028)*x_ref[2]**o+(-0.984463097169)*x_ref[2]
        ref[0,1,0]=(-0.734186269208)*x_ref[0]**o+(-0.955220234973)*x_ref[0]+(0.355373019012)*x_ref[1]**o+(-0.0259995318533)*x_ref[1]+(-0.923817606426)*x_ref[2]**o+(0.707611624363)*x_ref[2]
        ref[0,1,1]=(-0.65807304305)*x_ref[0]**o+(-0.546854762897)*x_ref[0]+(-0.041383885518)*x_ref[1]**o+(-0.847656164394)*x_ref[1]+(-0.69366771235)*x_ref[2]**o+(-0.00594720800349)*x_ref[2]
        ref[1,0,0]=(0.4916488707)*x_ref[0]**o+(0.610932307599)*x_ref[0]+(-0.634836490924)*x_ref[1]**o+(0.190377750049)*x_ref[1]+(0.839648718767)*x_ref[2]**o+(-0.697199112832)*x_ref[2]
        ref[1,0,1]=(-0.733462530162)*x_ref[0]**o+(0.897438124381)*x_ref[0]+(-0.284428270428)*x_ref[1]**o+(-0.490325551864)*x_ref[1]+(-0.425270047571)*x_ref[2]**o+(-0.0169167898593)*x_ref[2]
        ref[1,1,0]=(-0.1134205794)*x_ref[0]**o+(-0.589938051075)*x_ref[0]+(-0.551954550243)*x_ref[1]**o+(-0.828713609462)*x_ref[1]+(-0.14740902465)*x_ref[2]**o+(-0.0643506708023)*x_ref[2]
        ref[1,1,1]=(0.875176575479)*x_ref[0]**o+(-0.436379792297)*x_ref[0]+(-0.238769521024)*x_ref[1]**o+(-0.0883649176657)*x_ref[1]+(-0.268393504724)*x_ref[2]**o+(-0.331629361371)*x_ref[2]
        ref[2,0,0]=(-0.445061932808)*x_ref[0]**o+(-0.136283305342)*x_ref[0]+(0.109782590986)*x_ref[1]**o+(0.664151679353)*x_ref[1]+(0.77415319639)*x_ref[2]**o+(0.918600325668)*x_ref[2]
        ref[2,0,1]=(-0.966582148735)*x_ref[0]**o+(0.90850899638)*x_ref[0]+(0.860043581705)*x_ref[1]**o+(0.555164541705)*x_ref[1]+(-0.0881641948474)*x_ref[2]**o+(-0.437393798496)*x_ref[2]
        ref[2,1,0]=(0.424061230847)*x_ref[0]**o+(0.426231869756)*x_ref[0]+(0.20042430919)*x_ref[1]**o+(-0.392259834985)*x_ref[1]+(0.56903598931)*x_ref[2]**o+(-0.713346863994)*x_ref[2]
        ref[2,1,1]=(0.391005749137)*x_ref[0]**o+(-0.141533299622)*x_ref[0]+(-0.54549941972)*x_ref[1]**o+(-0.551542113521)*x_ref[1]+(0.461175241452)*x_ref[2]**o+(0.946400145746)*x_ref[2]
        ref[3,0,0]=(0.690778432543)*x_ref[0]**o+(0.0987975519633)*x_ref[0]+(0.600232607252)*x_ref[1]**o+(-0.403160580412)*x_ref[1]+(-0.259142718629)*x_ref[2]**o+(0.125408478863)*x_ref[2]
        ref[3,0,1]=(0.45235548032)*x_ref[0]**o+(0.465495192061)*x_ref[0]+(0.605495566594)*x_ref[1]**o+(0.0665669455329)*x_ref[1]+(-0.528374124117)*x_ref[2]**o+(0.971371028924)*x_ref[2]
        ref[3,1,0]=(-0.868740120635)*x_ref[0]**o+(0.94078043392)*x_ref[0]+(-0.924836120128)*x_ref[1]**o+(0.0436824783429)*x_ref[1]+(0.436490992322)*x_ref[2]**o+(0.563162883509)*x_ref[2]
        ref[3,1,1]=(-0.148701893094)*x_ref[0]**o+(0.114671183607)*x_ref[0]+(0.40845144069)*x_ref[1]**o+(-0.35749472879)*x_ref[1]+(-0.240456109591)*x_ref[2]**o+(-0.959187876519)*x_ref[2]
        ref[4,0,0]=(-0.902322503602)*x_ref[0]**o+(-0.0870613571753)*x_ref[0]+(0.604597216194)*x_ref[1]**o+(0.484694096025)*x_ref[1]+(0.733216575545)*x_ref[2]**o+(0.981143903885)*x_ref[2]
        ref[4,0,1]=(-0.9183984444)*x_ref[0]**o+(0.222613734722)*x_ref[0]+(0.472361095251)*x_ref[1]**o+(-0.182413993184)*x_ref[1]+(0.404737782329)*x_ref[2]**o+(-0.462634159071)*x_ref[2]
        ref[4,1,0]=(-0.577317659134)*x_ref[0]**o+(0.863519222199)*x_ref[0]+(-0.955859899586)*x_ref[1]**o+(0.898032633657)*x_ref[1]+(0.000203504014658)*x_ref[2]**o+(0.84858789073)*x_ref[2]
        ref[4,1,1]=(0.103931145317)*x_ref[0]**o+(-0.491733208237)*x_ref[0]+(-0.679616928853)*x_ref[1]**o+(0.779966695883)*x_ref[1]+(0.19391391729)*x_ref[2]**o+(0.237260204789)*x_ref[2]
        ref[5,0,0]=(-0.587657719833)*x_ref[0]**o+(0.0677059438625)*x_ref[0]+(-0.070159254762)*x_ref[1]**o+(-0.725676771128)*x_ref[1]+(0.563139108018)*x_ref[2]**o+(-0.182136289146)*x_ref[2]
        ref[5,0,1]=(0.647754539947)*x_ref[0]**o+(-0.238569428669)*x_ref[0]+(-0.0352701585821)*x_ref[1]**o+(0.117211931696)*x_ref[1]+(-0.749719337272)*x_ref[2]**o+(-0.0572910818861)*x_ref[2]
        ref[5,1,0]=(0.0802652788129)*x_ref[0]**o+(-0.320772823979)*x_ref[0]+(0.875989601034)*x_ref[1]**o+(0.56803751787)*x_ref[1]+(0.115080146257)*x_ref[2]**o+(0.960937482841)*x_ref[2]
        ref[5,1,1]=(0.374333508365)*x_ref[0]**o+(-0.198321408411)*x_ref[0]+(0.550730384308)*x_ref[1]**o+(0.980522161961)*x_ref[1]+(-0.523593924556)*x_ref[2]**o+(-0.0882497539881)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromSymbol_ContinuousFunction_rank4(self):
      """
      tests interpolate for rank 4 Symbol onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ContinuousFunction onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5, 3, 2),dim=dim)
      s=Data(0,(4, 5, 3, 2),w)
      ref=Data(0,(4, 5, 3, 2),w_ref)
      if dim==2:
        s[0,0,0,0]=(0.222648011315)*x[0]**o+(0.42721508374)*x[0]+(-0.895044476361)*x[1]**o+(-0.42954458773)*x[1]
        s[0,0,0,1]=(0.141549825966)*x[0]**o+(0.28788545297)*x[0]+(0.595616465531)*x[1]**o+(-0.780212218036)*x[1]
        s[0,0,1,0]=(-0.206706340827)*x[0]**o+(-0.133810198949)*x[0]+(0.257781241939)*x[1]**o+(0.597910683263)*x[1]
        s[0,0,1,1]=(-0.121459945992)*x[0]**o+(-0.766623739533)*x[0]+(0.239967670821)*x[1]**o+(-0.388414165885)*x[1]
        s[0,0,2,0]=(0.146250309234)*x[0]**o+(-0.49783173353)*x[0]+(-0.244780515634)*x[1]**o+(-0.314249479259)*x[1]
        s[0,0,2,1]=(-0.270630697677)*x[0]**o+(0.0407254161321)*x[0]+(0.433386393453)*x[1]**o+(0.0801094288602)*x[1]
        s[0,1,0,0]=(0.748843675191)*x[0]**o+(-0.0720203817557)*x[0]+(-0.593596340641)*x[1]**o+(0.347464092831)*x[1]
        s[0,1,0,1]=(-0.395277643253)*x[0]**o+(0.664246198151)*x[0]+(-0.531558963654)*x[1]**o+(-0.809864143072)*x[1]
        s[0,1,1,0]=(0.193097468131)*x[0]**o+(-0.630186168256)*x[0]+(0.0615850267982)*x[1]**o+(-0.570479656519)*x[1]
        s[0,1,1,1]=(-0.368327911525)*x[0]**o+(-0.407914196122)*x[0]+(0.979471770727)*x[1]**o+(0.983952824078)*x[1]
        s[0,1,2,0]=(0.0767864730507)*x[0]**o+(0.114382875063)*x[0]+(-0.147666698894)*x[1]**o+(0.857285661664)*x[1]
        s[0,1,2,1]=(-0.122027577603)*x[0]**o+(-0.590353974558)*x[0]+(-0.667221707908)*x[1]**o+(0.745800225071)*x[1]
        s[0,2,0,0]=(-0.88730273713)*x[0]**o+(0.190743538467)*x[0]+(-0.938661510873)*x[1]**o+(-0.338310116385)*x[1]
        s[0,2,0,1]=(-0.141121464144)*x[0]**o+(0.850394068231)*x[0]+(0.0971484151963)*x[1]**o+(-0.960454373066)*x[1]
        s[0,2,1,0]=(0.824172674079)*x[0]**o+(0.26884586085)*x[0]+(-0.126014056897)*x[1]**o+(0.2784109285)*x[1]
        s[0,2,1,1]=(-0.360983815477)*x[0]**o+(0.714915900407)*x[0]+(0.767809732657)*x[1]**o+(-0.37952152499)*x[1]
        s[0,2,2,0]=(0.161042364518)*x[0]**o+(-0.643042351924)*x[0]+(0.252496991843)*x[1]**o+(-0.979633048076)*x[1]
        s[0,2,2,1]=(0.603846956362)*x[0]**o+(0.700541009601)*x[0]+(-0.63935941149)*x[1]**o+(-0.933149153774)*x[1]
        s[0,3,0,0]=(0.079280423825)*x[0]**o+(-0.716376532932)*x[0]+(-0.502244268586)*x[1]**o+(-0.959402022113)*x[1]
        s[0,3,0,1]=(-0.00511781253292)*x[0]**o+(-0.450787404264)*x[0]+(-0.474024181711)*x[1]**o+(0.456830901009)*x[1]
        s[0,3,1,0]=(0.111583663671)*x[0]**o+(0.102696091218)*x[0]+(0.523533956986)*x[1]**o+(-0.588614638579)*x[1]
        s[0,3,1,1]=(0.58778089432)*x[0]**o+(0.0740732890571)*x[0]+(-0.836011518186)*x[1]**o+(-0.108700168991)*x[1]
        s[0,3,2,0]=(-0.476011451886)*x[0]**o+(-0.773655516146)*x[0]+(-0.682784970237)*x[1]**o+(0.138275892754)*x[1]
        s[0,3,2,1]=(-0.214407387481)*x[0]**o+(-0.418064963679)*x[0]+(0.261699528297)*x[1]**o+(0.0441030472265)*x[1]
        s[0,4,0,0]=(-0.184136692471)*x[0]**o+(0.757765208488)*x[0]+(-0.873091970596)*x[1]**o+(0.350453919065)*x[1]
        s[0,4,0,1]=(-0.885266586543)*x[0]**o+(0.191822948672)*x[0]+(0.333201768343)*x[1]**o+(0.56090642241)*x[1]
        s[0,4,1,0]=(0.22567370118)*x[0]**o+(0.477010002185)*x[0]+(-0.135849171523)*x[1]**o+(-0.399167662154)*x[1]
        s[0,4,1,1]=(-0.830942601229)*x[0]**o+(0.996133387607)*x[0]+(0.518365254824)*x[1]**o+(-0.316479216888)*x[1]
        s[0,4,2,0]=(-0.21777913967)*x[0]**o+(-0.234792212377)*x[0]+(-0.551676836277)*x[1]**o+(-0.594255963868)*x[1]
        s[0,4,2,1]=(0.261945319278)*x[0]**o+(-0.0895552648139)*x[0]+(0.419366366098)*x[1]**o+(0.952305260972)*x[1]
        s[1,0,0,0]=(-0.940686110562)*x[0]**o+(-0.283753475589)*x[0]+(-0.38170291279)*x[1]**o+(0.800953830123)*x[1]
        s[1,0,0,1]=(-0.674895560431)*x[0]**o+(0.246943746198)*x[0]+(0.33525110471)*x[1]**o+(-0.365874383263)*x[1]
        s[1,0,1,0]=(0.00215505252567)*x[0]**o+(-0.859263824986)*x[0]+(-0.509512919301)*x[1]**o+(-0.413848698442)*x[1]
        s[1,0,1,1]=(0.700734957272)*x[0]**o+(0.974253812785)*x[0]+(-0.161730087557)*x[1]**o+(0.163662044371)*x[1]
        s[1,0,2,0]=(-0.961772561059)*x[0]**o+(0.949683155983)*x[0]+(-0.85122085071)*x[1]**o+(0.480287755677)*x[1]
        s[1,0,2,1]=(-0.560545819014)*x[0]**o+(0.574588583212)*x[0]+(-0.0973249975595)*x[1]**o+(0.635962665135)*x[1]
        s[1,1,0,0]=(0.362894470984)*x[0]**o+(-0.550857559086)*x[0]+(0.250693089264)*x[1]**o+(0.620977354637)*x[1]
        s[1,1,0,1]=(0.843905294027)*x[0]**o+(0.533693790759)*x[0]+(-0.29073001293)*x[1]**o+(-0.0166976969594)*x[1]
        s[1,1,1,0]=(-0.710809602785)*x[0]**o+(-0.886308169232)*x[0]+(-0.0578420674111)*x[1]**o+(0.113973788995)*x[1]
        s[1,1,1,1]=(-0.206206827837)*x[0]**o+(0.666323404382)*x[0]+(-0.602962857934)*x[1]**o+(0.0520638745337)*x[1]
        s[1,1,2,0]=(0.0999097614108)*x[0]**o+(-0.813782256512)*x[0]+(0.543364309707)*x[1]**o+(0.526696582979)*x[1]
        s[1,1,2,1]=(0.9692802149)*x[0]**o+(0.822581430545)*x[0]+(-0.42796571194)*x[1]**o+(-0.98187357708)*x[1]
        s[1,2,0,0]=(0.546276944309)*x[0]**o+(-0.374798696965)*x[0]+(-0.531415779779)*x[1]**o+(0.636501153213)*x[1]
        s[1,2,0,1]=(0.591914536156)*x[0]**o+(0.436968785918)*x[0]+(-0.0643795810318)*x[1]**o+(-0.103071630867)*x[1]
        s[1,2,1,0]=(0.0556003374382)*x[0]**o+(0.321314284536)*x[0]+(0.428484115069)*x[1]**o+(-0.788818848309)*x[1]
        s[1,2,1,1]=(0.708201805858)*x[0]**o+(-0.447922785627)*x[0]+(0.720006160805)*x[1]**o+(-0.195158798158)*x[1]
        s[1,2,2,0]=(-0.191158036071)*x[0]**o+(-0.24071498725)*x[0]+(0.0867135968142)*x[1]**o+(-0.381118927393)*x[1]
        s[1,2,2,1]=(0.101200398285)*x[0]**o+(0.319160878245)*x[0]+(0.823111493698)*x[1]**o+(0.373915251689)*x[1]
        s[1,3,0,0]=(0.418534190311)*x[0]**o+(0.38426953402)*x[0]+(0.202103889162)*x[1]**o+(-0.987162870942)*x[1]
        s[1,3,0,1]=(0.506778960729)*x[0]**o+(0.66657084314)*x[0]+(-0.348650547674)*x[1]**o+(-0.175750694648)*x[1]
        s[1,3,1,0]=(-0.723440967066)*x[0]**o+(-0.0953195217426)*x[0]+(0.701375987002)*x[1]**o+(0.789666038518)*x[1]
        s[1,3,1,1]=(-0.298531731302)*x[0]**o+(0.700679845377)*x[0]+(0.940832077343)*x[1]**o+(0.950445857347)*x[1]
        s[1,3,2,0]=(-0.891108074117)*x[0]**o+(0.864924848307)*x[0]+(0.266041420151)*x[1]**o+(0.552064290037)*x[1]
        s[1,3,2,1]=(-0.457816415507)*x[0]**o+(-0.0349980783656)*x[0]+(-0.487164584531)*x[1]**o+(0.781198707595)*x[1]
        s[1,4,0,0]=(-0.165226035876)*x[0]**o+(0.744411614612)*x[0]+(-0.583420396138)*x[1]**o+(0.0705229778455)*x[1]
        s[1,4,0,1]=(0.0891155162281)*x[0]**o+(-0.880960952088)*x[0]+(0.886639784408)*x[1]**o+(0.882570908394)*x[1]
        s[1,4,1,0]=(0.165376342322)*x[0]**o+(0.946226681425)*x[0]+(-0.692718922043)*x[1]**o+(-0.743234216196)*x[1]
        s[1,4,1,1]=(0.710740507812)*x[0]**o+(0.997738475549)*x[0]+(-0.465912186923)*x[1]**o+(-0.00201986786631)*x[1]
        s[1,4,2,0]=(0.374989766617)*x[0]**o+(-0.405268600342)*x[0]+(0.952553167679)*x[1]**o+(-0.0300346590056)*x[1]
        s[1,4,2,1]=(0.596218440945)*x[0]**o+(0.792458254573)*x[0]+(0.277317899686)*x[1]**o+(0.959183907598)*x[1]
        s[2,0,0,0]=(0.755363954311)*x[0]**o+(0.110444956303)*x[0]+(-0.992020571874)*x[1]**o+(-0.0319378081217)*x[1]
        s[2,0,0,1]=(0.955184354029)*x[0]**o+(0.695896419253)*x[0]+(0.36332796725)*x[1]**o+(-0.317618535271)*x[1]
        s[2,0,1,0]=(-0.609579319942)*x[0]**o+(0.964032314592)*x[0]+(-0.997676133302)*x[1]**o+(0.882837207968)*x[1]
        s[2,0,1,1]=(-0.856804288862)*x[0]**o+(-0.28105270207)*x[0]+(-0.630216743999)*x[1]**o+(-0.698401891968)*x[1]
        s[2,0,2,0]=(0.284068678209)*x[0]**o+(-0.207077555458)*x[0]+(0.602851480909)*x[1]**o+(-0.754913042436)*x[1]
        s[2,0,2,1]=(-0.250223707591)*x[0]**o+(0.35320467306)*x[0]+(0.389011278513)*x[1]**o+(0.226530933139)*x[1]
        s[2,1,0,0]=(-0.941260683043)*x[0]**o+(0.873373110513)*x[0]+(-0.766222917713)*x[1]**o+(-0.553221037926)*x[1]
        s[2,1,0,1]=(-0.0826999421931)*x[0]**o+(0.912535931146)*x[0]+(0.757104222181)*x[1]**o+(-0.285277618808)*x[1]
        s[2,1,1,0]=(0.3338465047)*x[0]**o+(0.781271516683)*x[0]+(-0.879533691783)*x[1]**o+(-0.714418961015)*x[1]
        s[2,1,1,1]=(0.149738013539)*x[0]**o+(0.591368145868)*x[0]+(0.0850187448147)*x[1]**o+(-0.197535932669)*x[1]
        s[2,1,2,0]=(0.919954979514)*x[0]**o+(0.910109312902)*x[0]+(-0.066392739623)*x[1]**o+(0.72593933626)*x[1]
        s[2,1,2,1]=(0.132310708599)*x[0]**o+(-0.651825160189)*x[0]+(-0.763887217289)*x[1]**o+(0.693036971149)*x[1]
        s[2,2,0,0]=(0.444176449376)*x[0]**o+(0.0925744377816)*x[0]+(-0.552171800039)*x[1]**o+(-0.0561642176587)*x[1]
        s[2,2,0,1]=(-0.319291518868)*x[0]**o+(-0.937039777249)*x[0]+(0.977602989302)*x[1]**o+(0.232130154058)*x[1]
        s[2,2,1,0]=(-0.0903492311881)*x[0]**o+(0.7908165014)*x[0]+(-0.427663715489)*x[1]**o+(-0.644083387831)*x[1]
        s[2,2,1,1]=(0.866462742024)*x[0]**o+(-0.321161526982)*x[0]+(0.467946139031)*x[1]**o+(0.981155836279)*x[1]
        s[2,2,2,0]=(-0.946044628595)*x[0]**o+(-0.513431569734)*x[0]+(0.205205524474)*x[1]**o+(-0.299199260153)*x[1]
        s[2,2,2,1]=(0.0871796517538)*x[0]**o+(-0.499117781496)*x[0]+(-0.395420669586)*x[1]**o+(-0.740740744789)*x[1]
        s[2,3,0,0]=(-0.663617561023)*x[0]**o+(0.436824463783)*x[0]+(-0.627409472686)*x[1]**o+(-0.68586382338)*x[1]
        s[2,3,0,1]=(-0.100973464748)*x[0]**o+(-0.376217782462)*x[0]+(0.358787883332)*x[1]**o+(0.041205949572)*x[1]
        s[2,3,1,0]=(-0.18138462543)*x[0]**o+(0.212059079029)*x[0]+(0.259368203273)*x[1]**o+(0.0823245655952)*x[1]
        s[2,3,1,1]=(0.143918794807)*x[0]**o+(0.41064856854)*x[0]+(0.591128358341)*x[1]**o+(0.974911718087)*x[1]
        s[2,3,2,0]=(0.215121419628)*x[0]**o+(-0.202266227576)*x[0]+(0.360645018302)*x[1]**o+(-0.530217372293)*x[1]
        s[2,3,2,1]=(-0.645683652927)*x[0]**o+(0.0783849074368)*x[0]+(0.446495488766)*x[1]**o+(-0.510345834163)*x[1]
        s[2,4,0,0]=(-0.889866222706)*x[0]**o+(0.167795911139)*x[0]+(0.342469422898)*x[1]**o+(0.829595836798)*x[1]
        s[2,4,0,1]=(0.149554083927)*x[0]**o+(-0.822909234878)*x[0]+(0.252083851924)*x[1]**o+(-0.316759440069)*x[1]
        s[2,4,1,0]=(0.903989662182)*x[0]**o+(-0.00199692638164)*x[0]+(0.183216868853)*x[1]**o+(0.291683747052)*x[1]
        s[2,4,1,1]=(-0.837481381147)*x[0]**o+(-0.49767333079)*x[0]+(0.379317825858)*x[1]**o+(0.656734345135)*x[1]
        s[2,4,2,0]=(0.575327981246)*x[0]**o+(-0.32472740434)*x[0]+(-0.615235966928)*x[1]**o+(0.859062569875)*x[1]
        s[2,4,2,1]=(0.997096808384)*x[0]**o+(0.176877547637)*x[0]+(0.944330419866)*x[1]**o+(0.09534223881)*x[1]
        s[3,0,0,0]=(-0.0767412809661)*x[0]**o+(0.184419353055)*x[0]+(0.450559868822)*x[1]**o+(-0.482074341734)*x[1]
        s[3,0,0,1]=(0.500618673994)*x[0]**o+(-0.428845431851)*x[0]+(-0.593972292401)*x[1]**o+(-0.590749326466)*x[1]
        s[3,0,1,0]=(-0.274065531705)*x[0]**o+(-0.0749212487474)*x[0]+(0.179752364175)*x[1]**o+(-0.960772511585)*x[1]
        s[3,0,1,1]=(-0.238772579502)*x[0]**o+(0.489035124726)*x[0]+(-0.664791553185)*x[1]**o+(-0.104025806782)*x[1]
        s[3,0,2,0]=(0.982870092816)*x[0]**o+(-0.888195654437)*x[0]+(0.440388652872)*x[1]**o+(0.805316867841)*x[1]
        s[3,0,2,1]=(0.0228989287015)*x[0]**o+(0.148291709924)*x[0]+(-0.686198603954)*x[1]**o+(-0.821746591235)*x[1]
        s[3,1,0,0]=(0.00240454431321)*x[0]**o+(0.61269586693)*x[0]+(0.341226969118)*x[1]**o+(0.124208016094)*x[1]
        s[3,1,0,1]=(0.0701234542733)*x[0]**o+(0.895384612471)*x[0]+(-0.922625286693)*x[1]**o+(-0.752021767621)*x[1]
        s[3,1,1,0]=(0.771977634017)*x[0]**o+(-0.912502253275)*x[0]+(-0.320855123076)*x[1]**o+(-0.521053436714)*x[1]
        s[3,1,1,1]=(0.291967128777)*x[0]**o+(0.0136675345425)*x[0]+(-0.240499641569)*x[1]**o+(0.120459293301)*x[1]
        s[3,1,2,0]=(-0.165796720823)*x[0]**o+(-0.564849404668)*x[0]+(-0.745995713866)*x[1]**o+(0.105168429062)*x[1]
        s[3,1,2,1]=(0.703303894233)*x[0]**o+(0.198015398782)*x[0]+(-0.399667205564)*x[1]**o+(-0.431944087803)*x[1]
        s[3,2,0,0]=(-0.859984329126)*x[0]**o+(-0.1985647639)*x[0]+(0.0785405575628)*x[1]**o+(-0.91703866491)*x[1]
        s[3,2,0,1]=(0.142256934845)*x[0]**o+(-0.9608102834)*x[0]+(-0.573917350955)*x[1]**o+(-0.115513923505)*x[1]
        s[3,2,1,0]=(0.0929359330483)*x[0]**o+(-0.870599213251)*x[0]+(-0.47846799921)*x[1]**o+(-0.560581415535)*x[1]
        s[3,2,1,1]=(0.148783020471)*x[0]**o+(0.735514599027)*x[0]+(0.839043206017)*x[1]**o+(0.89788839722)*x[1]
        s[3,2,2,0]=(0.838672369672)*x[0]**o+(0.508289848942)*x[0]+(-0.726948781664)*x[1]**o+(-0.766210549368)*x[1]
        s[3,2,2,1]=(-0.934406814466)*x[0]**o+(0.782704000359)*x[0]+(0.813704047884)*x[1]**o+(0.810651175197)*x[1]
        s[3,3,0,0]=(0.0808252741173)*x[0]**o+(-0.863997858352)*x[0]+(-0.486533766186)*x[1]**o+(0.696138206258)*x[1]
        s[3,3,0,1]=(0.828675714346)*x[0]**o+(0.533631275087)*x[0]+(0.767869622158)*x[1]**o+(0.127624967621)*x[1]
        s[3,3,1,0]=(0.486100829039)*x[0]**o+(-0.0150390173975)*x[0]+(0.0799449336303)*x[1]**o+(-0.680174222069)*x[1]
        s[3,3,1,1]=(-0.163578245514)*x[0]**o+(0.664275094053)*x[0]+(-0.983443827553)*x[1]**o+(0.0384576043866)*x[1]
        s[3,3,2,0]=(0.098732247123)*x[0]**o+(-0.500859620961)*x[0]+(0.781489169465)*x[1]**o+(-0.593888761671)*x[1]
        s[3,3,2,1]=(0.207503649878)*x[0]**o+(-0.782987281123)*x[0]+(0.401857033567)*x[1]**o+(-0.561255499698)*x[1]
        s[3,4,0,0]=(-0.703304703621)*x[0]**o+(0.900854331554)*x[0]+(-0.660621668968)*x[1]**o+(-0.339993374651)*x[1]
        s[3,4,0,1]=(-0.113871007693)*x[0]**o+(-0.91823246062)*x[0]+(-0.328204536528)*x[1]**o+(0.811186991056)*x[1]
        s[3,4,1,0]=(0.118365746728)*x[0]**o+(0.966348151963)*x[0]+(0.734158908606)*x[1]**o+(-0.793152919357)*x[1]
        s[3,4,1,1]=(-0.865632981044)*x[0]**o+(-0.533667091846)*x[0]+(0.539354750567)*x[1]**o+(-0.436274385593)*x[1]
        s[3,4,2,0]=(0.0308218777522)*x[0]**o+(0.135254149275)*x[0]+(0.541297536585)*x[1]**o+(0.969869993827)*x[1]
        s[3,4,2,1]=(-0.0120021630149)*x[0]**o+(-0.641342077878)*x[0]+(-0.605491166275)*x[1]**o+(-0.170165570072)*x[1]
        ref[0,0,0,0]=(0.222648011315)*x_ref[0]**o+(0.42721508374)*x_ref[0]+(-0.895044476361)*x_ref[1]**o+(-0.42954458773)*x_ref[1]
        ref[0,0,0,1]=(0.141549825966)*x_ref[0]**o+(0.28788545297)*x_ref[0]+(0.595616465531)*x_ref[1]**o+(-0.780212218036)*x_ref[1]
        ref[0,0,1,0]=(-0.206706340827)*x_ref[0]**o+(-0.133810198949)*x_ref[0]+(0.257781241939)*x_ref[1]**o+(0.597910683263)*x_ref[1]
        ref[0,0,1,1]=(-0.121459945992)*x_ref[0]**o+(-0.766623739533)*x_ref[0]+(0.239967670821)*x_ref[1]**o+(-0.388414165885)*x_ref[1]
        ref[0,0,2,0]=(0.146250309234)*x_ref[0]**o+(-0.49783173353)*x_ref[0]+(-0.244780515634)*x_ref[1]**o+(-0.314249479259)*x_ref[1]
        ref[0,0,2,1]=(-0.270630697677)*x_ref[0]**o+(0.0407254161321)*x_ref[0]+(0.433386393453)*x_ref[1]**o+(0.0801094288602)*x_ref[1]
        ref[0,1,0,0]=(0.748843675191)*x_ref[0]**o+(-0.0720203817557)*x_ref[0]+(-0.593596340641)*x_ref[1]**o+(0.347464092831)*x_ref[1]
        ref[0,1,0,1]=(-0.395277643253)*x_ref[0]**o+(0.664246198151)*x_ref[0]+(-0.531558963654)*x_ref[1]**o+(-0.809864143072)*x_ref[1]
        ref[0,1,1,0]=(0.193097468131)*x_ref[0]**o+(-0.630186168256)*x_ref[0]+(0.0615850267982)*x_ref[1]**o+(-0.570479656519)*x_ref[1]
        ref[0,1,1,1]=(-0.368327911525)*x_ref[0]**o+(-0.407914196122)*x_ref[0]+(0.979471770727)*x_ref[1]**o+(0.983952824078)*x_ref[1]
        ref[0,1,2,0]=(0.0767864730507)*x_ref[0]**o+(0.114382875063)*x_ref[0]+(-0.147666698894)*x_ref[1]**o+(0.857285661664)*x_ref[1]
        ref[0,1,2,1]=(-0.122027577603)*x_ref[0]**o+(-0.590353974558)*x_ref[0]+(-0.667221707908)*x_ref[1]**o+(0.745800225071)*x_ref[1]
        ref[0,2,0,0]=(-0.88730273713)*x_ref[0]**o+(0.190743538467)*x_ref[0]+(-0.938661510873)*x_ref[1]**o+(-0.338310116385)*x_ref[1]
        ref[0,2,0,1]=(-0.141121464144)*x_ref[0]**o+(0.850394068231)*x_ref[0]+(0.0971484151963)*x_ref[1]**o+(-0.960454373066)*x_ref[1]
        ref[0,2,1,0]=(0.824172674079)*x_ref[0]**o+(0.26884586085)*x_ref[0]+(-0.126014056897)*x_ref[1]**o+(0.2784109285)*x_ref[1]
        ref[0,2,1,1]=(-0.360983815477)*x_ref[0]**o+(0.714915900407)*x_ref[0]+(0.767809732657)*x_ref[1]**o+(-0.37952152499)*x_ref[1]
        ref[0,2,2,0]=(0.161042364518)*x_ref[0]**o+(-0.643042351924)*x_ref[0]+(0.252496991843)*x_ref[1]**o+(-0.979633048076)*x_ref[1]
        ref[0,2,2,1]=(0.603846956362)*x_ref[0]**o+(0.700541009601)*x_ref[0]+(-0.63935941149)*x_ref[1]**o+(-0.933149153774)*x_ref[1]
        ref[0,3,0,0]=(0.079280423825)*x_ref[0]**o+(-0.716376532932)*x_ref[0]+(-0.502244268586)*x_ref[1]**o+(-0.959402022113)*x_ref[1]
        ref[0,3,0,1]=(-0.00511781253292)*x_ref[0]**o+(-0.450787404264)*x_ref[0]+(-0.474024181711)*x_ref[1]**o+(0.456830901009)*x_ref[1]
        ref[0,3,1,0]=(0.111583663671)*x_ref[0]**o+(0.102696091218)*x_ref[0]+(0.523533956986)*x_ref[1]**o+(-0.588614638579)*x_ref[1]
        ref[0,3,1,1]=(0.58778089432)*x_ref[0]**o+(0.0740732890571)*x_ref[0]+(-0.836011518186)*x_ref[1]**o+(-0.108700168991)*x_ref[1]
        ref[0,3,2,0]=(-0.476011451886)*x_ref[0]**o+(-0.773655516146)*x_ref[0]+(-0.682784970237)*x_ref[1]**o+(0.138275892754)*x_ref[1]
        ref[0,3,2,1]=(-0.214407387481)*x_ref[0]**o+(-0.418064963679)*x_ref[0]+(0.261699528297)*x_ref[1]**o+(0.0441030472265)*x_ref[1]
        ref[0,4,0,0]=(-0.184136692471)*x_ref[0]**o+(0.757765208488)*x_ref[0]+(-0.873091970596)*x_ref[1]**o+(0.350453919065)*x_ref[1]
        ref[0,4,0,1]=(-0.885266586543)*x_ref[0]**o+(0.191822948672)*x_ref[0]+(0.333201768343)*x_ref[1]**o+(0.56090642241)*x_ref[1]
        ref[0,4,1,0]=(0.22567370118)*x_ref[0]**o+(0.477010002185)*x_ref[0]+(-0.135849171523)*x_ref[1]**o+(-0.399167662154)*x_ref[1]
        ref[0,4,1,1]=(-0.830942601229)*x_ref[0]**o+(0.996133387607)*x_ref[0]+(0.518365254824)*x_ref[1]**o+(-0.316479216888)*x_ref[1]
        ref[0,4,2,0]=(-0.21777913967)*x_ref[0]**o+(-0.234792212377)*x_ref[0]+(-0.551676836277)*x_ref[1]**o+(-0.594255963868)*x_ref[1]
        ref[0,4,2,1]=(0.261945319278)*x_ref[0]**o+(-0.0895552648139)*x_ref[0]+(0.419366366098)*x_ref[1]**o+(0.952305260972)*x_ref[1]
        ref[1,0,0,0]=(-0.940686110562)*x_ref[0]**o+(-0.283753475589)*x_ref[0]+(-0.38170291279)*x_ref[1]**o+(0.800953830123)*x_ref[1]
        ref[1,0,0,1]=(-0.674895560431)*x_ref[0]**o+(0.246943746198)*x_ref[0]+(0.33525110471)*x_ref[1]**o+(-0.365874383263)*x_ref[1]
        ref[1,0,1,0]=(0.00215505252567)*x_ref[0]**o+(-0.859263824986)*x_ref[0]+(-0.509512919301)*x_ref[1]**o+(-0.413848698442)*x_ref[1]
        ref[1,0,1,1]=(0.700734957272)*x_ref[0]**o+(0.974253812785)*x_ref[0]+(-0.161730087557)*x_ref[1]**o+(0.163662044371)*x_ref[1]
        ref[1,0,2,0]=(-0.961772561059)*x_ref[0]**o+(0.949683155983)*x_ref[0]+(-0.85122085071)*x_ref[1]**o+(0.480287755677)*x_ref[1]
        ref[1,0,2,1]=(-0.560545819014)*x_ref[0]**o+(0.574588583212)*x_ref[0]+(-0.0973249975595)*x_ref[1]**o+(0.635962665135)*x_ref[1]
        ref[1,1,0,0]=(0.362894470984)*x_ref[0]**o+(-0.550857559086)*x_ref[0]+(0.250693089264)*x_ref[1]**o+(0.620977354637)*x_ref[1]
        ref[1,1,0,1]=(0.843905294027)*x_ref[0]**o+(0.533693790759)*x_ref[0]+(-0.29073001293)*x_ref[1]**o+(-0.0166976969594)*x_ref[1]
        ref[1,1,1,0]=(-0.710809602785)*x_ref[0]**o+(-0.886308169232)*x_ref[0]+(-0.0578420674111)*x_ref[1]**o+(0.113973788995)*x_ref[1]
        ref[1,1,1,1]=(-0.206206827837)*x_ref[0]**o+(0.666323404382)*x_ref[0]+(-0.602962857934)*x_ref[1]**o+(0.0520638745337)*x_ref[1]
        ref[1,1,2,0]=(0.0999097614108)*x_ref[0]**o+(-0.813782256512)*x_ref[0]+(0.543364309707)*x_ref[1]**o+(0.526696582979)*x_ref[1]
        ref[1,1,2,1]=(0.9692802149)*x_ref[0]**o+(0.822581430545)*x_ref[0]+(-0.42796571194)*x_ref[1]**o+(-0.98187357708)*x_ref[1]
        ref[1,2,0,0]=(0.546276944309)*x_ref[0]**o+(-0.374798696965)*x_ref[0]+(-0.531415779779)*x_ref[1]**o+(0.636501153213)*x_ref[1]
        ref[1,2,0,1]=(0.591914536156)*x_ref[0]**o+(0.436968785918)*x_ref[0]+(-0.0643795810318)*x_ref[1]**o+(-0.103071630867)*x_ref[1]
        ref[1,2,1,0]=(0.0556003374382)*x_ref[0]**o+(0.321314284536)*x_ref[0]+(0.428484115069)*x_ref[1]**o+(-0.788818848309)*x_ref[1]
        ref[1,2,1,1]=(0.708201805858)*x_ref[0]**o+(-0.447922785627)*x_ref[0]+(0.720006160805)*x_ref[1]**o+(-0.195158798158)*x_ref[1]
        ref[1,2,2,0]=(-0.191158036071)*x_ref[0]**o+(-0.24071498725)*x_ref[0]+(0.0867135968142)*x_ref[1]**o+(-0.381118927393)*x_ref[1]
        ref[1,2,2,1]=(0.101200398285)*x_ref[0]**o+(0.319160878245)*x_ref[0]+(0.823111493698)*x_ref[1]**o+(0.373915251689)*x_ref[1]
        ref[1,3,0,0]=(0.418534190311)*x_ref[0]**o+(0.38426953402)*x_ref[0]+(0.202103889162)*x_ref[1]**o+(-0.987162870942)*x_ref[1]
        ref[1,3,0,1]=(0.506778960729)*x_ref[0]**o+(0.66657084314)*x_ref[0]+(-0.348650547674)*x_ref[1]**o+(-0.175750694648)*x_ref[1]
        ref[1,3,1,0]=(-0.723440967066)*x_ref[0]**o+(-0.0953195217426)*x_ref[0]+(0.701375987002)*x_ref[1]**o+(0.789666038518)*x_ref[1]
        ref[1,3,1,1]=(-0.298531731302)*x_ref[0]**o+(0.700679845377)*x_ref[0]+(0.940832077343)*x_ref[1]**o+(0.950445857347)*x_ref[1]
        ref[1,3,2,0]=(-0.891108074117)*x_ref[0]**o+(0.864924848307)*x_ref[0]+(0.266041420151)*x_ref[1]**o+(0.552064290037)*x_ref[1]
        ref[1,3,2,1]=(-0.457816415507)*x_ref[0]**o+(-0.0349980783656)*x_ref[0]+(-0.487164584531)*x_ref[1]**o+(0.781198707595)*x_ref[1]
        ref[1,4,0,0]=(-0.165226035876)*x_ref[0]**o+(0.744411614612)*x_ref[0]+(-0.583420396138)*x_ref[1]**o+(0.0705229778455)*x_ref[1]
        ref[1,4,0,1]=(0.0891155162281)*x_ref[0]**o+(-0.880960952088)*x_ref[0]+(0.886639784408)*x_ref[1]**o+(0.882570908394)*x_ref[1]
        ref[1,4,1,0]=(0.165376342322)*x_ref[0]**o+(0.946226681425)*x_ref[0]+(-0.692718922043)*x_ref[1]**o+(-0.743234216196)*x_ref[1]
        ref[1,4,1,1]=(0.710740507812)*x_ref[0]**o+(0.997738475549)*x_ref[0]+(-0.465912186923)*x_ref[1]**o+(-0.00201986786631)*x_ref[1]
        ref[1,4,2,0]=(0.374989766617)*x_ref[0]**o+(-0.405268600342)*x_ref[0]+(0.952553167679)*x_ref[1]**o+(-0.0300346590056)*x_ref[1]
        ref[1,4,2,1]=(0.596218440945)*x_ref[0]**o+(0.792458254573)*x_ref[0]+(0.277317899686)*x_ref[1]**o+(0.959183907598)*x_ref[1]
        ref[2,0,0,0]=(0.755363954311)*x_ref[0]**o+(0.110444956303)*x_ref[0]+(-0.992020571874)*x_ref[1]**o+(-0.0319378081217)*x_ref[1]
        ref[2,0,0,1]=(0.955184354029)*x_ref[0]**o+(0.695896419253)*x_ref[0]+(0.36332796725)*x_ref[1]**o+(-0.317618535271)*x_ref[1]
        ref[2,0,1,0]=(-0.609579319942)*x_ref[0]**o+(0.964032314592)*x_ref[0]+(-0.997676133302)*x_ref[1]**o+(0.882837207968)*x_ref[1]
        ref[2,0,1,1]=(-0.856804288862)*x_ref[0]**o+(-0.28105270207)*x_ref[0]+(-0.630216743999)*x_ref[1]**o+(-0.698401891968)*x_ref[1]
        ref[2,0,2,0]=(0.284068678209)*x_ref[0]**o+(-0.207077555458)*x_ref[0]+(0.602851480909)*x_ref[1]**o+(-0.754913042436)*x_ref[1]
        ref[2,0,2,1]=(-0.250223707591)*x_ref[0]**o+(0.35320467306)*x_ref[0]+(0.389011278513)*x_ref[1]**o+(0.226530933139)*x_ref[1]
        ref[2,1,0,0]=(-0.941260683043)*x_ref[0]**o+(0.873373110513)*x_ref[0]+(-0.766222917713)*x_ref[1]**o+(-0.553221037926)*x_ref[1]
        ref[2,1,0,1]=(-0.0826999421931)*x_ref[0]**o+(0.912535931146)*x_ref[0]+(0.757104222181)*x_ref[1]**o+(-0.285277618808)*x_ref[1]
        ref[2,1,1,0]=(0.3338465047)*x_ref[0]**o+(0.781271516683)*x_ref[0]+(-0.879533691783)*x_ref[1]**o+(-0.714418961015)*x_ref[1]
        ref[2,1,1,1]=(0.149738013539)*x_ref[0]**o+(0.591368145868)*x_ref[0]+(0.0850187448147)*x_ref[1]**o+(-0.197535932669)*x_ref[1]
        ref[2,1,2,0]=(0.919954979514)*x_ref[0]**o+(0.910109312902)*x_ref[0]+(-0.066392739623)*x_ref[1]**o+(0.72593933626)*x_ref[1]
        ref[2,1,2,1]=(0.132310708599)*x_ref[0]**o+(-0.651825160189)*x_ref[0]+(-0.763887217289)*x_ref[1]**o+(0.693036971149)*x_ref[1]
        ref[2,2,0,0]=(0.444176449376)*x_ref[0]**o+(0.0925744377816)*x_ref[0]+(-0.552171800039)*x_ref[1]**o+(-0.0561642176587)*x_ref[1]
        ref[2,2,0,1]=(-0.319291518868)*x_ref[0]**o+(-0.937039777249)*x_ref[0]+(0.977602989302)*x_ref[1]**o+(0.232130154058)*x_ref[1]
        ref[2,2,1,0]=(-0.0903492311881)*x_ref[0]**o+(0.7908165014)*x_ref[0]+(-0.427663715489)*x_ref[1]**o+(-0.644083387831)*x_ref[1]
        ref[2,2,1,1]=(0.866462742024)*x_ref[0]**o+(-0.321161526982)*x_ref[0]+(0.467946139031)*x_ref[1]**o+(0.981155836279)*x_ref[1]
        ref[2,2,2,0]=(-0.946044628595)*x_ref[0]**o+(-0.513431569734)*x_ref[0]+(0.205205524474)*x_ref[1]**o+(-0.299199260153)*x_ref[1]
        ref[2,2,2,1]=(0.0871796517538)*x_ref[0]**o+(-0.499117781496)*x_ref[0]+(-0.395420669586)*x_ref[1]**o+(-0.740740744789)*x_ref[1]
        ref[2,3,0,0]=(-0.663617561023)*x_ref[0]**o+(0.436824463783)*x_ref[0]+(-0.627409472686)*x_ref[1]**o+(-0.68586382338)*x_ref[1]
        ref[2,3,0,1]=(-0.100973464748)*x_ref[0]**o+(-0.376217782462)*x_ref[0]+(0.358787883332)*x_ref[1]**o+(0.041205949572)*x_ref[1]
        ref[2,3,1,0]=(-0.18138462543)*x_ref[0]**o+(0.212059079029)*x_ref[0]+(0.259368203273)*x_ref[1]**o+(0.0823245655952)*x_ref[1]
        ref[2,3,1,1]=(0.143918794807)*x_ref[0]**o+(0.41064856854)*x_ref[0]+(0.591128358341)*x_ref[1]**o+(0.974911718087)*x_ref[1]
        ref[2,3,2,0]=(0.215121419628)*x_ref[0]**o+(-0.202266227576)*x_ref[0]+(0.360645018302)*x_ref[1]**o+(-0.530217372293)*x_ref[1]
        ref[2,3,2,1]=(-0.645683652927)*x_ref[0]**o+(0.0783849074368)*x_ref[0]+(0.446495488766)*x_ref[1]**o+(-0.510345834163)*x_ref[1]
        ref[2,4,0,0]=(-0.889866222706)*x_ref[0]**o+(0.167795911139)*x_ref[0]+(0.342469422898)*x_ref[1]**o+(0.829595836798)*x_ref[1]
        ref[2,4,0,1]=(0.149554083927)*x_ref[0]**o+(-0.822909234878)*x_ref[0]+(0.252083851924)*x_ref[1]**o+(-0.316759440069)*x_ref[1]
        ref[2,4,1,0]=(0.903989662182)*x_ref[0]**o+(-0.00199692638164)*x_ref[0]+(0.183216868853)*x_ref[1]**o+(0.291683747052)*x_ref[1]
        ref[2,4,1,1]=(-0.837481381147)*x_ref[0]**o+(-0.49767333079)*x_ref[0]+(0.379317825858)*x_ref[1]**o+(0.656734345135)*x_ref[1]
        ref[2,4,2,0]=(0.575327981246)*x_ref[0]**o+(-0.32472740434)*x_ref[0]+(-0.615235966928)*x_ref[1]**o+(0.859062569875)*x_ref[1]
        ref[2,4,2,1]=(0.997096808384)*x_ref[0]**o+(0.176877547637)*x_ref[0]+(0.944330419866)*x_ref[1]**o+(0.09534223881)*x_ref[1]
        ref[3,0,0,0]=(-0.0767412809661)*x_ref[0]**o+(0.184419353055)*x_ref[0]+(0.450559868822)*x_ref[1]**o+(-0.482074341734)*x_ref[1]
        ref[3,0,0,1]=(0.500618673994)*x_ref[0]**o+(-0.428845431851)*x_ref[0]+(-0.593972292401)*x_ref[1]**o+(-0.590749326466)*x_ref[1]
        ref[3,0,1,0]=(-0.274065531705)*x_ref[0]**o+(-0.0749212487474)*x_ref[0]+(0.179752364175)*x_ref[1]**o+(-0.960772511585)*x_ref[1]
        ref[3,0,1,1]=(-0.238772579502)*x_ref[0]**o+(0.489035124726)*x_ref[0]+(-0.664791553185)*x_ref[1]**o+(-0.104025806782)*x_ref[1]
        ref[3,0,2,0]=(0.982870092816)*x_ref[0]**o+(-0.888195654437)*x_ref[0]+(0.440388652872)*x_ref[1]**o+(0.805316867841)*x_ref[1]
        ref[3,0,2,1]=(0.0228989287015)*x_ref[0]**o+(0.148291709924)*x_ref[0]+(-0.686198603954)*x_ref[1]**o+(-0.821746591235)*x_ref[1]
        ref[3,1,0,0]=(0.00240454431321)*x_ref[0]**o+(0.61269586693)*x_ref[0]+(0.341226969118)*x_ref[1]**o+(0.124208016094)*x_ref[1]
        ref[3,1,0,1]=(0.0701234542733)*x_ref[0]**o+(0.895384612471)*x_ref[0]+(-0.922625286693)*x_ref[1]**o+(-0.752021767621)*x_ref[1]
        ref[3,1,1,0]=(0.771977634017)*x_ref[0]**o+(-0.912502253275)*x_ref[0]+(-0.320855123076)*x_ref[1]**o+(-0.521053436714)*x_ref[1]
        ref[3,1,1,1]=(0.291967128777)*x_ref[0]**o+(0.0136675345425)*x_ref[0]+(-0.240499641569)*x_ref[1]**o+(0.120459293301)*x_ref[1]
        ref[3,1,2,0]=(-0.165796720823)*x_ref[0]**o+(-0.564849404668)*x_ref[0]+(-0.745995713866)*x_ref[1]**o+(0.105168429062)*x_ref[1]
        ref[3,1,2,1]=(0.703303894233)*x_ref[0]**o+(0.198015398782)*x_ref[0]+(-0.399667205564)*x_ref[1]**o+(-0.431944087803)*x_ref[1]
        ref[3,2,0,0]=(-0.859984329126)*x_ref[0]**o+(-0.1985647639)*x_ref[0]+(0.0785405575628)*x_ref[1]**o+(-0.91703866491)*x_ref[1]
        ref[3,2,0,1]=(0.142256934845)*x_ref[0]**o+(-0.9608102834)*x_ref[0]+(-0.573917350955)*x_ref[1]**o+(-0.115513923505)*x_ref[1]
        ref[3,2,1,0]=(0.0929359330483)*x_ref[0]**o+(-0.870599213251)*x_ref[0]+(-0.47846799921)*x_ref[1]**o+(-0.560581415535)*x_ref[1]
        ref[3,2,1,1]=(0.148783020471)*x_ref[0]**o+(0.735514599027)*x_ref[0]+(0.839043206017)*x_ref[1]**o+(0.89788839722)*x_ref[1]
        ref[3,2,2,0]=(0.838672369672)*x_ref[0]**o+(0.508289848942)*x_ref[0]+(-0.726948781664)*x_ref[1]**o+(-0.766210549368)*x_ref[1]
        ref[3,2,2,1]=(-0.934406814466)*x_ref[0]**o+(0.782704000359)*x_ref[0]+(0.813704047884)*x_ref[1]**o+(0.810651175197)*x_ref[1]
        ref[3,3,0,0]=(0.0808252741173)*x_ref[0]**o+(-0.863997858352)*x_ref[0]+(-0.486533766186)*x_ref[1]**o+(0.696138206258)*x_ref[1]
        ref[3,3,0,1]=(0.828675714346)*x_ref[0]**o+(0.533631275087)*x_ref[0]+(0.767869622158)*x_ref[1]**o+(0.127624967621)*x_ref[1]
        ref[3,3,1,0]=(0.486100829039)*x_ref[0]**o+(-0.0150390173975)*x_ref[0]+(0.0799449336303)*x_ref[1]**o+(-0.680174222069)*x_ref[1]
        ref[3,3,1,1]=(-0.163578245514)*x_ref[0]**o+(0.664275094053)*x_ref[0]+(-0.983443827553)*x_ref[1]**o+(0.0384576043866)*x_ref[1]
        ref[3,3,2,0]=(0.098732247123)*x_ref[0]**o+(-0.500859620961)*x_ref[0]+(0.781489169465)*x_ref[1]**o+(-0.593888761671)*x_ref[1]
        ref[3,3,2,1]=(0.207503649878)*x_ref[0]**o+(-0.782987281123)*x_ref[0]+(0.401857033567)*x_ref[1]**o+(-0.561255499698)*x_ref[1]
        ref[3,4,0,0]=(-0.703304703621)*x_ref[0]**o+(0.900854331554)*x_ref[0]+(-0.660621668968)*x_ref[1]**o+(-0.339993374651)*x_ref[1]
        ref[3,4,0,1]=(-0.113871007693)*x_ref[0]**o+(-0.91823246062)*x_ref[0]+(-0.328204536528)*x_ref[1]**o+(0.811186991056)*x_ref[1]
        ref[3,4,1,0]=(0.118365746728)*x_ref[0]**o+(0.966348151963)*x_ref[0]+(0.734158908606)*x_ref[1]**o+(-0.793152919357)*x_ref[1]
        ref[3,4,1,1]=(-0.865632981044)*x_ref[0]**o+(-0.533667091846)*x_ref[0]+(0.539354750567)*x_ref[1]**o+(-0.436274385593)*x_ref[1]
        ref[3,4,2,0]=(0.0308218777522)*x_ref[0]**o+(0.135254149275)*x_ref[0]+(0.541297536585)*x_ref[1]**o+(0.969869993827)*x_ref[1]
        ref[3,4,2,1]=(-0.0120021630149)*x_ref[0]**o+(-0.641342077878)*x_ref[0]+(-0.605491166275)*x_ref[1]**o+(-0.170165570072)*x_ref[1]
      else:
        s[0,0,0,0]=(-0.645055264384)*x[0]**o+(-0.193054732885)*x[0]+(0.0857819007815)*x[1]**o+(-0.461528553026)*x[1]+(-0.57134367637)*x[2]**o+(-0.74286427921)*x[2]
        s[0,0,0,1]=(0.487034525946)*x[0]**o+(-0.714187726361)*x[0]+(-0.606957760182)*x[1]**o+(0.0779147207958)*x[1]+(-0.29526614633)*x[2]**o+(-0.458434997173)*x[2]
        s[0,0,1,0]=(0.156747343036)*x[0]**o+(-0.341557748989)*x[0]+(0.158761045238)*x[1]**o+(-0.801472587987)*x[1]+(-0.271446861129)*x[2]**o+(0.900123872896)*x[2]
        s[0,0,1,1]=(0.573494874527)*x[0]**o+(-0.0243920948082)*x[0]+(0.0948894747825)*x[1]**o+(0.65093963776)*x[1]+(0.671075295673)*x[2]**o+(-0.864463024946)*x[2]
        s[0,0,2,0]=(0.725244561373)*x[0]**o+(-0.996337933154)*x[0]+(0.767557173446)*x[1]**o+(0.949472898375)*x[1]+(0.34694891892)*x[2]**o+(-0.0275257881948)*x[2]
        s[0,0,2,1]=(0.172147320675)*x[0]**o+(-0.308830513267)*x[0]+(-0.186533796188)*x[1]**o+(-0.253183031716)*x[1]+(-0.0558420387348)*x[2]**o+(0.639581162188)*x[2]
        s[0,1,0,0]=(-0.201128976272)*x[0]**o+(-0.0604529403788)*x[0]+(0.687516667804)*x[1]**o+(0.0463366971989)*x[1]+(-0.512099164154)*x[2]**o+(-0.562496472369)*x[2]
        s[0,1,0,1]=(-0.579867899667)*x[0]**o+(0.481671792856)*x[0]+(-0.534848750787)*x[1]**o+(-0.681855536562)*x[1]+(-0.138906106494)*x[2]**o+(0.85816397707)*x[2]
        s[0,1,1,0]=(0.361813239327)*x[0]**o+(-0.371707128692)*x[0]+(0.1413931258)*x[1]**o+(-0.222686191434)*x[1]+(0.429860839989)*x[2]**o+(-0.854238845295)*x[2]
        s[0,1,1,1]=(0.777481754216)*x[0]**o+(0.559920012089)*x[0]+(-0.708494735928)*x[1]**o+(-0.340573378074)*x[1]+(-0.658897461649)*x[2]**o+(0.0686343535795)*x[2]
        s[0,1,2,0]=(0.548617122525)*x[0]**o+(0.142683302153)*x[0]+(0.599377799663)*x[1]**o+(-0.0941501509169)*x[1]+(0.899613162974)*x[2]**o+(0.437942658389)*x[2]
        s[0,1,2,1]=(0.055538446966)*x[0]**o+(0.213360227501)*x[0]+(-0.929499951666)*x[1]**o+(0.336982626422)*x[1]+(-0.656888335372)*x[2]**o+(0.407882483181)*x[2]
        s[0,2,0,0]=(-0.874997203733)*x[0]**o+(0.759216265112)*x[0]+(0.948642776931)*x[1]**o+(-0.242980793949)*x[1]+(0.677616559342)*x[2]**o+(0.341859949626)*x[2]
        s[0,2,0,1]=(-0.181175585757)*x[0]**o+(0.335483796366)*x[0]+(-0.664259907913)*x[1]**o+(0.906747760538)*x[1]+(-0.984393889605)*x[2]**o+(-0.812569900067)*x[2]
        s[0,2,1,0]=(0.851036684294)*x[0]**o+(-0.751387632375)*x[0]+(0.724388989617)*x[1]**o+(0.384230212684)*x[1]+(-0.449333509528)*x[2]**o+(0.784460454553)*x[2]
        s[0,2,1,1]=(-0.609413988379)*x[0]**o+(-0.493285779523)*x[0]+(-0.590691722184)*x[1]**o+(-0.607626442728)*x[1]+(0.0625828744206)*x[2]**o+(-0.15945730222)*x[2]
        s[0,2,2,0]=(0.766872578253)*x[0]**o+(-0.590526688805)*x[0]+(0.222323591845)*x[1]**o+(0.512451311011)*x[1]+(-0.160540733556)*x[2]**o+(0.5770077018)*x[2]
        s[0,2,2,1]=(-0.729525005374)*x[0]**o+(0.986018027885)*x[0]+(0.25197251588)*x[1]**o+(0.0450480112291)*x[1]+(-0.869546576401)*x[2]**o+(-0.290965290154)*x[2]
        s[0,3,0,0]=(-0.184322684185)*x[0]**o+(0.751826009862)*x[0]+(-0.697266959303)*x[1]**o+(0.924909596435)*x[1]+(-0.638157491307)*x[2]**o+(0.236651962157)*x[2]
        s[0,3,0,1]=(0.706826305243)*x[0]**o+(-0.946439184553)*x[0]+(0.168150324173)*x[1]**o+(-0.792980290572)*x[1]+(0.61272398393)*x[2]**o+(0.961138901449)*x[2]
        s[0,3,1,0]=(0.933288270936)*x[0]**o+(-0.55682855224)*x[0]+(0.883604318263)*x[1]**o+(0.816211062971)*x[1]+(0.260246780311)*x[2]**o+(0.905516381287)*x[2]
        s[0,3,1,1]=(0.404420727507)*x[0]**o+(0.471967111659)*x[0]+(0.980687681241)*x[1]**o+(-0.0887974405176)*x[1]+(0.0934146142552)*x[2]**o+(0.646580252159)*x[2]
        s[0,3,2,0]=(-0.921777598275)*x[0]**o+(0.499527698309)*x[0]+(0.664601445292)*x[1]**o+(-0.851079448204)*x[1]+(-0.769256802656)*x[2]**o+(0.730819279497)*x[2]
        s[0,3,2,1]=(-0.245507455509)*x[0]**o+(0.357764745233)*x[0]+(0.88892212086)*x[1]**o+(-0.819843178794)*x[1]+(-0.203314043822)*x[2]**o+(0.224643592554)*x[2]
        s[0,4,0,0]=(0.343174611172)*x[0]**o+(-0.701162898726)*x[0]+(-0.447921000153)*x[1]**o+(0.206599993047)*x[1]+(-0.461911660893)*x[2]**o+(-0.757271316865)*x[2]
        s[0,4,0,1]=(-0.55800009078)*x[0]**o+(-0.39370288163)*x[0]+(-0.35379408894)*x[1]**o+(-0.214762670331)*x[1]+(0.725231115203)*x[2]**o+(0.765815232027)*x[2]
        s[0,4,1,0]=(0.440761988298)*x[0]**o+(-0.798551732493)*x[0]+(-0.955319652492)*x[1]**o+(-0.50319720075)*x[1]+(-0.213217943103)*x[2]**o+(-0.882735934069)*x[2]
        s[0,4,1,1]=(0.441117080069)*x[0]**o+(-0.748338538161)*x[0]+(0.956112036119)*x[1]**o+(-0.960896721051)*x[1]+(-0.291350828791)*x[2]**o+(-0.488896289566)*x[2]
        s[0,4,2,0]=(-0.855035257555)*x[0]**o+(-0.579064017558)*x[0]+(-0.501257230663)*x[1]**o+(-0.956216916996)*x[1]+(-0.860898494221)*x[2]**o+(-0.830087013431)*x[2]
        s[0,4,2,1]=(-0.868978503525)*x[0]**o+(0.162013861935)*x[0]+(-0.311759384766)*x[1]**o+(0.399068885633)*x[1]+(-0.569888913523)*x[2]**o+(-0.394173563136)*x[2]
        s[1,0,0,0]=(0.215242187817)*x[0]**o+(-0.215197471124)*x[0]+(0.779932221493)*x[1]**o+(-0.365740975053)*x[1]+(0.601368178784)*x[2]**o+(-0.170706718888)*x[2]
        s[1,0,0,1]=(0.862102162389)*x[0]**o+(-0.815403119428)*x[0]+(0.269334729254)*x[1]**o+(-0.511457037588)*x[1]+(0.00982312570019)*x[2]**o+(-0.407471705546)*x[2]
        s[1,0,1,0]=(-0.728800252524)*x[0]**o+(-0.178562348078)*x[0]+(0.711141895031)*x[1]**o+(-0.0037290743948)*x[1]+(-0.353455013195)*x[2]**o+(-0.0812715065083)*x[2]
        s[1,0,1,1]=(-0.689013409151)*x[0]**o+(-0.467030398964)*x[0]+(0.295472079898)*x[1]**o+(0.803549469534)*x[1]+(-0.360809851719)*x[2]**o+(-0.563821123318)*x[2]
        s[1,0,2,0]=(0.238079356258)*x[0]**o+(-0.385019487483)*x[0]+(0.864174173897)*x[1]**o+(-0.338065985154)*x[1]+(-0.516837345006)*x[2]**o+(0.611410140553)*x[2]
        s[1,0,2,1]=(0.937340721244)*x[0]**o+(-0.0936947975478)*x[0]+(0.313991879475)*x[1]**o+(-0.126811107707)*x[1]+(-0.627872015529)*x[2]**o+(-0.981891626964)*x[2]
        s[1,1,0,0]=(-0.0675382298198)*x[0]**o+(-0.783093574363)*x[0]+(-0.377941739502)*x[1]**o+(-0.0833668456557)*x[1]+(-0.157483560616)*x[2]**o+(0.686957855465)*x[2]
        s[1,1,0,1]=(0.151028535239)*x[0]**o+(0.992218304228)*x[0]+(-0.686192558465)*x[1]**o+(-0.0130549001993)*x[1]+(0.578982765961)*x[2]**o+(0.814271616163)*x[2]
        s[1,1,1,0]=(0.384343377969)*x[0]**o+(0.463218368911)*x[0]+(0.35576368147)*x[1]**o+(-0.834867049786)*x[1]+(-0.274552267393)*x[2]**o+(0.623371271421)*x[2]
        s[1,1,1,1]=(-0.61717606851)*x[0]**o+(-0.578562595163)*x[0]+(0.0518387545717)*x[1]**o+(-0.30815303855)*x[1]+(0.556738906305)*x[2]**o+(0.38164058851)*x[2]
        s[1,1,2,0]=(0.52042073171)*x[0]**o+(0.698894225253)*x[0]+(0.120169831848)*x[1]**o+(0.238450852883)*x[1]+(-0.552966574246)*x[2]**o+(0.661290976968)*x[2]
        s[1,1,2,1]=(-0.428668260768)*x[0]**o+(0.568751515596)*x[0]+(-0.0768903249884)*x[1]**o+(0.768814424639)*x[1]+(0.424759560455)*x[2]**o+(-0.690938863922)*x[2]
        s[1,2,0,0]=(-0.5443917305)*x[0]**o+(-0.0595999390634)*x[0]+(0.901507044764)*x[1]**o+(-0.681191541382)*x[1]+(-0.123122907072)*x[2]**o+(-0.152869512816)*x[2]
        s[1,2,0,1]=(-0.0145795630242)*x[0]**o+(-0.53313358583)*x[0]+(0.629028937665)*x[1]**o+(-0.539546800632)*x[1]+(-0.252160197397)*x[2]**o+(0.0133469506662)*x[2]
        s[1,2,1,0]=(0.00594869891786)*x[0]**o+(-0.56813890169)*x[0]+(-0.685679553166)*x[1]**o+(0.81727698867)*x[1]+(-0.895427226202)*x[2]**o+(0.140096007363)*x[2]
        s[1,2,1,1]=(-0.770028732259)*x[0]**o+(0.827053138879)*x[0]+(-0.917989309552)*x[1]**o+(-0.0517956582083)*x[1]+(-0.266844746028)*x[2]**o+(-0.99294686081)*x[2]
        s[1,2,2,0]=(0.0529054005828)*x[0]**o+(-0.807219927076)*x[0]+(-0.296037864436)*x[1]**o+(-0.617547075266)*x[1]+(-0.966016304179)*x[2]**o+(-0.168793187561)*x[2]
        s[1,2,2,1]=(0.868690089119)*x[0]**o+(-0.610624824567)*x[0]+(-0.411113678451)*x[1]**o+(0.103726193007)*x[1]+(-0.148892296878)*x[2]**o+(-0.195079797362)*x[2]
        s[1,3,0,0]=(0.248806387929)*x[0]**o+(-0.683762743914)*x[0]+(0.460928199336)*x[1]**o+(0.338430932923)*x[1]+(0.589813218945)*x[2]**o+(-0.632423181868)*x[2]
        s[1,3,0,1]=(-0.338476160834)*x[0]**o+(0.491753392304)*x[0]+(-0.590351153797)*x[1]**o+(-0.448026649636)*x[1]+(-0.806930505259)*x[2]**o+(-0.262565462369)*x[2]
        s[1,3,1,0]=(0.808414907549)*x[0]**o+(0.156426917662)*x[0]+(0.33216874106)*x[1]**o+(-0.714791665494)*x[1]+(-0.163168903301)*x[2]**o+(-0.0423061478551)*x[2]
        s[1,3,1,1]=(0.72498761831)*x[0]**o+(0.08194542863)*x[0]+(-0.566316087403)*x[1]**o+(0.49486476938)*x[1]+(0.744201752122)*x[2]**o+(0.766413325553)*x[2]
        s[1,3,2,0]=(0.793853329178)*x[0]**o+(-0.883754372376)*x[0]+(-0.991903018183)*x[1]**o+(0.279576133295)*x[1]+(-0.0364449658341)*x[2]**o+(-0.85024249755)*x[2]
        s[1,3,2,1]=(0.308927107876)*x[0]**o+(-0.995603136748)*x[0]+(-0.15217214087)*x[1]**o+(-0.554310725129)*x[1]+(0.69107000546)*x[2]**o+(0.353155450955)*x[2]
        s[1,4,0,0]=(0.696506340076)*x[0]**o+(0.889728425166)*x[0]+(-0.858268204558)*x[1]**o+(0.353251100246)*x[1]+(-0.974872511432)*x[2]**o+(-0.878064243518)*x[2]
        s[1,4,0,1]=(-0.232678527901)*x[0]**o+(-0.60738965282)*x[0]+(-0.629841924447)*x[1]**o+(0.0731692547228)*x[1]+(-0.457280478523)*x[2]**o+(-0.309371862287)*x[2]
        s[1,4,1,0]=(-0.0254189247267)*x[0]**o+(-0.160538018478)*x[0]+(-0.757800049379)*x[1]**o+(0.108707596517)*x[1]+(-0.182095912638)*x[2]**o+(0.565356253873)*x[2]
        s[1,4,1,1]=(0.353936491579)*x[0]**o+(0.368146351094)*x[0]+(-0.0271653916784)*x[1]**o+(0.7863197856)*x[1]+(-0.0642828131827)*x[2]**o+(0.946295880643)*x[2]
        s[1,4,2,0]=(-0.151347070208)*x[0]**o+(-0.490039530717)*x[0]+(-0.222861393521)*x[1]**o+(0.160218722103)*x[1]+(-0.881185613005)*x[2]**o+(0.453889724466)*x[2]
        s[1,4,2,1]=(0.0185253314435)*x[0]**o+(0.824691270585)*x[0]+(-0.175025465282)*x[1]**o+(-0.946563236595)*x[1]+(0.915012725055)*x[2]**o+(-0.14949108932)*x[2]
        s[2,0,0,0]=(0.608065273397)*x[0]**o+(0.103537849738)*x[0]+(0.640722456034)*x[1]**o+(0.707059952842)*x[1]+(0.958670726841)*x[2]**o+(0.178333168266)*x[2]
        s[2,0,0,1]=(0.762096192615)*x[0]**o+(-0.91311774327)*x[0]+(-0.636481030154)*x[1]**o+(0.0485431700913)*x[1]+(-0.76456752608)*x[2]**o+(0.988366648174)*x[2]
        s[2,0,1,0]=(-0.997545907902)*x[0]**o+(0.0386694282297)*x[0]+(-0.95454977224)*x[1]**o+(-0.472500001853)*x[1]+(-0.315668831265)*x[2]**o+(-0.0911950672323)*x[2]
        s[2,0,1,1]=(0.758639431658)*x[0]**o+(-0.35267267476)*x[0]+(0.733519690739)*x[1]**o+(0.687429543066)*x[1]+(0.748932363975)*x[2]**o+(-0.360330977046)*x[2]
        s[2,0,2,0]=(0.748882354686)*x[0]**o+(0.398195068825)*x[0]+(0.268707244918)*x[1]**o+(-0.906848889273)*x[1]+(0.8203042296)*x[2]**o+(-0.388209014017)*x[2]
        s[2,0,2,1]=(-0.622158650545)*x[0]**o+(0.757162985883)*x[0]+(0.213276305935)*x[1]**o+(0.376995936453)*x[1]+(-0.919851919774)*x[2]**o+(-0.341006967087)*x[2]
        s[2,1,0,0]=(0.45283405067)*x[0]**o+(-0.456183034235)*x[0]+(-0.211030502465)*x[1]**o+(0.874532774776)*x[1]+(-0.245927412434)*x[2]**o+(0.426470240035)*x[2]
        s[2,1,0,1]=(-0.167484852552)*x[0]**o+(0.23618650325)*x[0]+(-0.256431628436)*x[1]**o+(-0.758314815145)*x[1]+(-0.553126534983)*x[2]**o+(-0.993390924398)*x[2]
        s[2,1,1,0]=(-0.579896706714)*x[0]**o+(0.0247821970864)*x[0]+(-0.485603356124)*x[1]**o+(0.610550482437)*x[1]+(-0.788558005559)*x[2]**o+(-0.716855983085)*x[2]
        s[2,1,1,1]=(0.011213932334)*x[0]**o+(-0.606541681102)*x[0]+(-0.783918973118)*x[1]**o+(-0.129337676239)*x[1]+(-0.681878469428)*x[2]**o+(-0.818559911243)*x[2]
        s[2,1,2,0]=(0.806311020506)*x[0]**o+(0.348481209449)*x[0]+(0.147356878845)*x[1]**o+(0.481921509802)*x[1]+(0.445429036477)*x[2]**o+(0.113745424091)*x[2]
        s[2,1,2,1]=(-0.587849278852)*x[0]**o+(-0.141739080759)*x[0]+(-0.0431697185243)*x[1]**o+(0.845205068629)*x[1]+(0.485473344628)*x[2]**o+(0.416700333568)*x[2]
        s[2,2,0,0]=(-0.36580939154)*x[0]**o+(-0.826777569794)*x[0]+(-0.322507767896)*x[1]**o+(0.697307401686)*x[1]+(0.52326834016)*x[2]**o+(-0.31919043817)*x[2]
        s[2,2,0,1]=(-0.130664405569)*x[0]**o+(0.823599150158)*x[0]+(0.751486085263)*x[1]**o+(0.150119789292)*x[1]+(-0.307393365311)*x[2]**o+(0.919517717122)*x[2]
        s[2,2,1,0]=(0.254288128084)*x[0]**o+(0.580902444535)*x[0]+(0.90113516563)*x[1]**o+(0.805502401301)*x[1]+(-0.483775735536)*x[2]**o+(-0.846660440246)*x[2]
        s[2,2,1,1]=(-0.481100104989)*x[0]**o+(0.459159658041)*x[0]+(-0.886023770838)*x[1]**o+(0.693304415006)*x[1]+(-0.490424960617)*x[2]**o+(0.618464834808)*x[2]
        s[2,2,2,0]=(0.851037708736)*x[0]**o+(0.973238080753)*x[0]+(0.617606926402)*x[1]**o+(-0.0129774413768)*x[1]+(0.342094147972)*x[2]**o+(-0.698988316685)*x[2]
        s[2,2,2,1]=(-0.571806691565)*x[0]**o+(-0.488244261444)*x[0]+(-0.408779266509)*x[1]**o+(-0.688083559849)*x[1]+(-0.668712586564)*x[2]**o+(0.811908851853)*x[2]
        s[2,3,0,0]=(0.305354422638)*x[0]**o+(-0.347372904868)*x[0]+(0.147767334473)*x[1]**o+(0.597885123439)*x[1]+(-0.532183927826)*x[2]**o+(0.778189138615)*x[2]
        s[2,3,0,1]=(0.986328831124)*x[0]**o+(0.237847460579)*x[0]+(0.350775306824)*x[1]**o+(-0.750995843201)*x[1]+(0.85575809431)*x[2]**o+(0.624236482292)*x[2]
        s[2,3,1,0]=(-0.418600918765)*x[0]**o+(-0.958489157688)*x[0]+(-0.524398996544)*x[1]**o+(0.186795580218)*x[1]+(-0.364079386452)*x[2]**o+(-0.330796017113)*x[2]
        s[2,3,1,1]=(0.632410928458)*x[0]**o+(-0.557450605906)*x[0]+(-0.699741830058)*x[1]**o+(0.867693032849)*x[1]+(-0.770879368311)*x[2]**o+(0.972808988148)*x[2]
        s[2,3,2,0]=(0.283379195721)*x[0]**o+(-0.636902680473)*x[0]+(0.609885794066)*x[1]**o+(0.831554040093)*x[1]+(-0.330474761682)*x[2]**o+(0.000310556070561)*x[2]
        s[2,3,2,1]=(0.363575347708)*x[0]**o+(-0.327653113631)*x[0]+(-0.842370028303)*x[1]**o+(0.115573924346)*x[1]+(0.0903633504982)*x[2]**o+(-0.619568111717)*x[2]
        s[2,4,0,0]=(0.319164526641)*x[0]**o+(0.45922058507)*x[0]+(0.26273868718)*x[1]**o+(0.919180620189)*x[1]+(0.68231305095)*x[2]**o+(-0.0793024563191)*x[2]
        s[2,4,0,1]=(0.966152397311)*x[0]**o+(-0.323284283817)*x[0]+(0.68073876378)*x[1]**o+(0.353025011982)*x[1]+(0.907154492113)*x[2]**o+(0.371499834679)*x[2]
        s[2,4,1,0]=(0.441142086658)*x[0]**o+(-0.374316420849)*x[0]+(0.886357340085)*x[1]**o+(0.172424726539)*x[1]+(0.416195908431)*x[2]**o+(0.374779208985)*x[2]
        s[2,4,1,1]=(0.809132101144)*x[0]**o+(0.33952471205)*x[0]+(0.159476692789)*x[1]**o+(-0.719297128466)*x[1]+(-0.395244031022)*x[2]**o+(-0.942445288252)*x[2]
        s[2,4,2,0]=(-0.0269665566962)*x[0]**o+(-0.932688309921)*x[0]+(0.980071332595)*x[1]**o+(0.778605742399)*x[1]+(-0.69781881904)*x[2]**o+(0.919684867278)*x[2]
        s[2,4,2,1]=(0.440810804674)*x[0]**o+(0.642802841983)*x[0]+(-0.151896393227)*x[1]**o+(0.5114517147)*x[1]+(0.202606346779)*x[2]**o+(0.145680766426)*x[2]
        s[3,0,0,0]=(0.854905410474)*x[0]**o+(-0.883160499153)*x[0]+(-0.0518436529324)*x[1]**o+(0.472365408955)*x[1]+(-0.49219002513)*x[2]**o+(0.232861430592)*x[2]
        s[3,0,0,1]=(-0.046116184691)*x[0]**o+(0.596829797988)*x[0]+(-0.144691397205)*x[1]**o+(-0.252432054327)*x[1]+(0.0799656699566)*x[2]**o+(-0.220413254006)*x[2]
        s[3,0,1,0]=(0.675157805468)*x[0]**o+(0.655696700297)*x[0]+(0.474709137832)*x[1]**o+(0.121302534807)*x[1]+(-0.718410825564)*x[2]**o+(-0.671917363185)*x[2]
        s[3,0,1,1]=(-0.514174913964)*x[0]**o+(-0.0161999364854)*x[0]+(-0.38933612074)*x[1]**o+(-0.629975655537)*x[1]+(0.488451340059)*x[2]**o+(-0.583580208023)*x[2]
        s[3,0,2,0]=(-0.56947915393)*x[0]**o+(0.434845155767)*x[0]+(-0.968993748561)*x[1]**o+(-0.510861471635)*x[1]+(-0.288591065135)*x[2]**o+(-0.230425132856)*x[2]
        s[3,0,2,1]=(-0.374979517517)*x[0]**o+(-0.0163569866605)*x[0]+(0.75837797441)*x[1]**o+(-0.632240609496)*x[1]+(0.422914146097)*x[2]**o+(0.951370365751)*x[2]
        s[3,1,0,0]=(-0.991849908442)*x[0]**o+(-0.00797307615614)*x[0]+(-0.464488377523)*x[1]**o+(0.183107151355)*x[1]+(-0.470200932584)*x[2]**o+(-0.37271407313)*x[2]
        s[3,1,0,1]=(0.714765401281)*x[0]**o+(0.604856800205)*x[0]+(-0.550067009901)*x[1]**o+(-0.5869512121)*x[1]+(0.620821251646)*x[2]**o+(-0.897762101083)*x[2]
        s[3,1,1,0]=(0.347567608921)*x[0]**o+(0.425122715827)*x[0]+(-0.802475224849)*x[1]**o+(-0.522837292632)*x[1]+(-0.160768805832)*x[2]**o+(-0.445406093759)*x[2]
        s[3,1,1,1]=(0.320284660978)*x[0]**o+(0.252323047651)*x[0]+(-0.615677083087)*x[1]**o+(-0.85623985966)*x[1]+(-0.0480138503943)*x[2]**o+(0.106970090808)*x[2]
        s[3,1,2,0]=(0.744223828984)*x[0]**o+(0.0646702358772)*x[0]+(-0.780189264654)*x[1]**o+(-0.946938785995)*x[1]+(-0.73270156813)*x[2]**o+(-0.240670757875)*x[2]
        s[3,1,2,1]=(0.24593668579)*x[0]**o+(0.0666411656061)*x[0]+(0.278029304996)*x[1]**o+(-0.543159254692)*x[1]+(-0.394304458971)*x[2]**o+(0.0789963203485)*x[2]
        s[3,2,0,0]=(-0.129597165729)*x[0]**o+(-0.375126607602)*x[0]+(-0.133573059675)*x[1]**o+(-0.222965614248)*x[1]+(0.482498045295)*x[2]**o+(0.666380815962)*x[2]
        s[3,2,0,1]=(-0.298314889544)*x[0]**o+(-0.200460718422)*x[0]+(0.969934227091)*x[1]**o+(0.982204235633)*x[1]+(-0.875400986072)*x[2]**o+(-0.962005163625)*x[2]
        s[3,2,1,0]=(-0.744314645975)*x[0]**o+(0.899141755704)*x[0]+(0.738788130658)*x[1]**o+(-0.465548783282)*x[1]+(0.27881485753)*x[2]**o+(-0.711524090298)*x[2]
        s[3,2,1,1]=(-0.698617989028)*x[0]**o+(0.082546181429)*x[0]+(-0.611282109662)*x[1]**o+(-0.28088009072)*x[1]+(-0.260638676131)*x[2]**o+(-0.554757759687)*x[2]
        s[3,2,2,0]=(-0.934985159158)*x[0]**o+(0.366271859175)*x[0]+(0.55018247768)*x[1]**o+(0.114993902724)*x[1]+(0.196571047024)*x[2]**o+(-0.889655441435)*x[2]
        s[3,2,2,1]=(-0.76297489993)*x[0]**o+(-0.00479337140756)*x[0]+(0.853999531704)*x[1]**o+(-0.563106979943)*x[1]+(-0.925873572722)*x[2]**o+(-0.0277052508628)*x[2]
        s[3,3,0,0]=(-0.138137971158)*x[0]**o+(-0.41896080496)*x[0]+(0.0482713349343)*x[1]**o+(-0.21852867848)*x[1]+(0.0190850076189)*x[2]**o+(0.583969066138)*x[2]
        s[3,3,0,1]=(0.806413890451)*x[0]**o+(-0.863828929679)*x[0]+(-0.152284946303)*x[1]**o+(-0.41164748505)*x[1]+(-0.206330733354)*x[2]**o+(0.127114872982)*x[2]
        s[3,3,1,0]=(-0.418785729088)*x[0]**o+(-0.0658589519492)*x[0]+(-0.696602204394)*x[1]**o+(-0.725906543227)*x[1]+(0.968658365691)*x[2]**o+(-0.262448319217)*x[2]
        s[3,3,1,1]=(-0.189305081379)*x[0]**o+(-0.709824241653)*x[0]+(-0.799048663139)*x[1]**o+(0.0420441850736)*x[1]+(-0.443839189643)*x[2]**o+(0.867954924509)*x[2]
        s[3,3,2,0]=(0.56856531819)*x[0]**o+(0.227835800084)*x[0]+(-0.521859405163)*x[1]**o+(-0.869003176683)*x[1]+(-0.684378595493)*x[2]**o+(0.544243688121)*x[2]
        s[3,3,2,1]=(0.714635391612)*x[0]**o+(-0.718360328239)*x[0]+(-0.510003226793)*x[1]**o+(-0.857292216621)*x[1]+(-0.295240904568)*x[2]**o+(0.340409128873)*x[2]
        s[3,4,0,0]=(-0.715361279417)*x[0]**o+(-0.829022388935)*x[0]+(0.56459861288)*x[1]**o+(0.879337406251)*x[1]+(0.423698189396)*x[2]**o+(-0.763053018627)*x[2]
        s[3,4,0,1]=(0.282251024454)*x[0]**o+(-0.275549571404)*x[0]+(0.416099858947)*x[1]**o+(0.304537400335)*x[1]+(0.846027568045)*x[2]**o+(-0.00236635940042)*x[2]
        s[3,4,1,0]=(-0.396026824114)*x[0]**o+(-0.967573299876)*x[0]+(0.419395952325)*x[1]**o+(-0.0769940862315)*x[1]+(-0.451079361856)*x[2]**o+(0.891465362708)*x[2]
        s[3,4,1,1]=(-0.629373368312)*x[0]**o+(-0.0463131060729)*x[0]+(-0.285992266736)*x[1]**o+(0.852693491625)*x[1]+(0.101111799967)*x[2]**o+(-0.982099200191)*x[2]
        s[3,4,2,0]=(-0.840902275944)*x[0]**o+(0.578695985974)*x[0]+(0.533595808779)*x[1]**o+(0.970844141257)*x[1]+(-0.942252307966)*x[2]**o+(-0.051129339745)*x[2]
        s[3,4,2,1]=(-0.981944223126)*x[0]**o+(-0.0930019659212)*x[0]+(-0.239359393705)*x[1]**o+(-0.277892188253)*x[1]+(0.447927318111)*x[2]**o+(-0.860595690016)*x[2]
        ref[0,0,0,0]=(-0.645055264384)*x_ref[0]**o+(-0.193054732885)*x_ref[0]+(0.0857819007815)*x_ref[1]**o+(-0.461528553026)*x_ref[1]+(-0.57134367637)*x_ref[2]**o+(-0.74286427921)*x_ref[2]
        ref[0,0,0,1]=(0.487034525946)*x_ref[0]**o+(-0.714187726361)*x_ref[0]+(-0.606957760182)*x_ref[1]**o+(0.0779147207958)*x_ref[1]+(-0.29526614633)*x_ref[2]**o+(-0.458434997173)*x_ref[2]
        ref[0,0,1,0]=(0.156747343036)*x_ref[0]**o+(-0.341557748989)*x_ref[0]+(0.158761045238)*x_ref[1]**o+(-0.801472587987)*x_ref[1]+(-0.271446861129)*x_ref[2]**o+(0.900123872896)*x_ref[2]
        ref[0,0,1,1]=(0.573494874527)*x_ref[0]**o+(-0.0243920948082)*x_ref[0]+(0.0948894747825)*x_ref[1]**o+(0.65093963776)*x_ref[1]+(0.671075295673)*x_ref[2]**o+(-0.864463024946)*x_ref[2]
        ref[0,0,2,0]=(0.725244561373)*x_ref[0]**o+(-0.996337933154)*x_ref[0]+(0.767557173446)*x_ref[1]**o+(0.949472898375)*x_ref[1]+(0.34694891892)*x_ref[2]**o+(-0.0275257881948)*x_ref[2]
        ref[0,0,2,1]=(0.172147320675)*x_ref[0]**o+(-0.308830513267)*x_ref[0]+(-0.186533796188)*x_ref[1]**o+(-0.253183031716)*x_ref[1]+(-0.0558420387348)*x_ref[2]**o+(0.639581162188)*x_ref[2]
        ref[0,1,0,0]=(-0.201128976272)*x_ref[0]**o+(-0.0604529403788)*x_ref[0]+(0.687516667804)*x_ref[1]**o+(0.0463366971989)*x_ref[1]+(-0.512099164154)*x_ref[2]**o+(-0.562496472369)*x_ref[2]
        ref[0,1,0,1]=(-0.579867899667)*x_ref[0]**o+(0.481671792856)*x_ref[0]+(-0.534848750787)*x_ref[1]**o+(-0.681855536562)*x_ref[1]+(-0.138906106494)*x_ref[2]**o+(0.85816397707)*x_ref[2]
        ref[0,1,1,0]=(0.361813239327)*x_ref[0]**o+(-0.371707128692)*x_ref[0]+(0.1413931258)*x_ref[1]**o+(-0.222686191434)*x_ref[1]+(0.429860839989)*x_ref[2]**o+(-0.854238845295)*x_ref[2]
        ref[0,1,1,1]=(0.777481754216)*x_ref[0]**o+(0.559920012089)*x_ref[0]+(-0.708494735928)*x_ref[1]**o+(-0.340573378074)*x_ref[1]+(-0.658897461649)*x_ref[2]**o+(0.0686343535795)*x_ref[2]
        ref[0,1,2,0]=(0.548617122525)*x_ref[0]**o+(0.142683302153)*x_ref[0]+(0.599377799663)*x_ref[1]**o+(-0.0941501509169)*x_ref[1]+(0.899613162974)*x_ref[2]**o+(0.437942658389)*x_ref[2]
        ref[0,1,2,1]=(0.055538446966)*x_ref[0]**o+(0.213360227501)*x_ref[0]+(-0.929499951666)*x_ref[1]**o+(0.336982626422)*x_ref[1]+(-0.656888335372)*x_ref[2]**o+(0.407882483181)*x_ref[2]
        ref[0,2,0,0]=(-0.874997203733)*x_ref[0]**o+(0.759216265112)*x_ref[0]+(0.948642776931)*x_ref[1]**o+(-0.242980793949)*x_ref[1]+(0.677616559342)*x_ref[2]**o+(0.341859949626)*x_ref[2]
        ref[0,2,0,1]=(-0.181175585757)*x_ref[0]**o+(0.335483796366)*x_ref[0]+(-0.664259907913)*x_ref[1]**o+(0.906747760538)*x_ref[1]+(-0.984393889605)*x_ref[2]**o+(-0.812569900067)*x_ref[2]
        ref[0,2,1,0]=(0.851036684294)*x_ref[0]**o+(-0.751387632375)*x_ref[0]+(0.724388989617)*x_ref[1]**o+(0.384230212684)*x_ref[1]+(-0.449333509528)*x_ref[2]**o+(0.784460454553)*x_ref[2]
        ref[0,2,1,1]=(-0.609413988379)*x_ref[0]**o+(-0.493285779523)*x_ref[0]+(-0.590691722184)*x_ref[1]**o+(-0.607626442728)*x_ref[1]+(0.0625828744206)*x_ref[2]**o+(-0.15945730222)*x_ref[2]
        ref[0,2,2,0]=(0.766872578253)*x_ref[0]**o+(-0.590526688805)*x_ref[0]+(0.222323591845)*x_ref[1]**o+(0.512451311011)*x_ref[1]+(-0.160540733556)*x_ref[2]**o+(0.5770077018)*x_ref[2]
        ref[0,2,2,1]=(-0.729525005374)*x_ref[0]**o+(0.986018027885)*x_ref[0]+(0.25197251588)*x_ref[1]**o+(0.0450480112291)*x_ref[1]+(-0.869546576401)*x_ref[2]**o+(-0.290965290154)*x_ref[2]
        ref[0,3,0,0]=(-0.184322684185)*x_ref[0]**o+(0.751826009862)*x_ref[0]+(-0.697266959303)*x_ref[1]**o+(0.924909596435)*x_ref[1]+(-0.638157491307)*x_ref[2]**o+(0.236651962157)*x_ref[2]
        ref[0,3,0,1]=(0.706826305243)*x_ref[0]**o+(-0.946439184553)*x_ref[0]+(0.168150324173)*x_ref[1]**o+(-0.792980290572)*x_ref[1]+(0.61272398393)*x_ref[2]**o+(0.961138901449)*x_ref[2]
        ref[0,3,1,0]=(0.933288270936)*x_ref[0]**o+(-0.55682855224)*x_ref[0]+(0.883604318263)*x_ref[1]**o+(0.816211062971)*x_ref[1]+(0.260246780311)*x_ref[2]**o+(0.905516381287)*x_ref[2]
        ref[0,3,1,1]=(0.404420727507)*x_ref[0]**o+(0.471967111659)*x_ref[0]+(0.980687681241)*x_ref[1]**o+(-0.0887974405176)*x_ref[1]+(0.0934146142552)*x_ref[2]**o+(0.646580252159)*x_ref[2]
        ref[0,3,2,0]=(-0.921777598275)*x_ref[0]**o+(0.499527698309)*x_ref[0]+(0.664601445292)*x_ref[1]**o+(-0.851079448204)*x_ref[1]+(-0.769256802656)*x_ref[2]**o+(0.730819279497)*x_ref[2]
        ref[0,3,2,1]=(-0.245507455509)*x_ref[0]**o+(0.357764745233)*x_ref[0]+(0.88892212086)*x_ref[1]**o+(-0.819843178794)*x_ref[1]+(-0.203314043822)*x_ref[2]**o+(0.224643592554)*x_ref[2]
        ref[0,4,0,0]=(0.343174611172)*x_ref[0]**o+(-0.701162898726)*x_ref[0]+(-0.447921000153)*x_ref[1]**o+(0.206599993047)*x_ref[1]+(-0.461911660893)*x_ref[2]**o+(-0.757271316865)*x_ref[2]
        ref[0,4,0,1]=(-0.55800009078)*x_ref[0]**o+(-0.39370288163)*x_ref[0]+(-0.35379408894)*x_ref[1]**o+(-0.214762670331)*x_ref[1]+(0.725231115203)*x_ref[2]**o+(0.765815232027)*x_ref[2]
        ref[0,4,1,0]=(0.440761988298)*x_ref[0]**o+(-0.798551732493)*x_ref[0]+(-0.955319652492)*x_ref[1]**o+(-0.50319720075)*x_ref[1]+(-0.213217943103)*x_ref[2]**o+(-0.882735934069)*x_ref[2]
        ref[0,4,1,1]=(0.441117080069)*x_ref[0]**o+(-0.748338538161)*x_ref[0]+(0.956112036119)*x_ref[1]**o+(-0.960896721051)*x_ref[1]+(-0.291350828791)*x_ref[2]**o+(-0.488896289566)*x_ref[2]
        ref[0,4,2,0]=(-0.855035257555)*x_ref[0]**o+(-0.579064017558)*x_ref[0]+(-0.501257230663)*x_ref[1]**o+(-0.956216916996)*x_ref[1]+(-0.860898494221)*x_ref[2]**o+(-0.830087013431)*x_ref[2]
        ref[0,4,2,1]=(-0.868978503525)*x_ref[0]**o+(0.162013861935)*x_ref[0]+(-0.311759384766)*x_ref[1]**o+(0.399068885633)*x_ref[1]+(-0.569888913523)*x_ref[2]**o+(-0.394173563136)*x_ref[2]
        ref[1,0,0,0]=(0.215242187817)*x_ref[0]**o+(-0.215197471124)*x_ref[0]+(0.779932221493)*x_ref[1]**o+(-0.365740975053)*x_ref[1]+(0.601368178784)*x_ref[2]**o+(-0.170706718888)*x_ref[2]
        ref[1,0,0,1]=(0.862102162389)*x_ref[0]**o+(-0.815403119428)*x_ref[0]+(0.269334729254)*x_ref[1]**o+(-0.511457037588)*x_ref[1]+(0.00982312570019)*x_ref[2]**o+(-0.407471705546)*x_ref[2]
        ref[1,0,1,0]=(-0.728800252524)*x_ref[0]**o+(-0.178562348078)*x_ref[0]+(0.711141895031)*x_ref[1]**o+(-0.0037290743948)*x_ref[1]+(-0.353455013195)*x_ref[2]**o+(-0.0812715065083)*x_ref[2]
        ref[1,0,1,1]=(-0.689013409151)*x_ref[0]**o+(-0.467030398964)*x_ref[0]+(0.295472079898)*x_ref[1]**o+(0.803549469534)*x_ref[1]+(-0.360809851719)*x_ref[2]**o+(-0.563821123318)*x_ref[2]
        ref[1,0,2,0]=(0.238079356258)*x_ref[0]**o+(-0.385019487483)*x_ref[0]+(0.864174173897)*x_ref[1]**o+(-0.338065985154)*x_ref[1]+(-0.516837345006)*x_ref[2]**o+(0.611410140553)*x_ref[2]
        ref[1,0,2,1]=(0.937340721244)*x_ref[0]**o+(-0.0936947975478)*x_ref[0]+(0.313991879475)*x_ref[1]**o+(-0.126811107707)*x_ref[1]+(-0.627872015529)*x_ref[2]**o+(-0.981891626964)*x_ref[2]
        ref[1,1,0,0]=(-0.0675382298198)*x_ref[0]**o+(-0.783093574363)*x_ref[0]+(-0.377941739502)*x_ref[1]**o+(-0.0833668456557)*x_ref[1]+(-0.157483560616)*x_ref[2]**o+(0.686957855465)*x_ref[2]
        ref[1,1,0,1]=(0.151028535239)*x_ref[0]**o+(0.992218304228)*x_ref[0]+(-0.686192558465)*x_ref[1]**o+(-0.0130549001993)*x_ref[1]+(0.578982765961)*x_ref[2]**o+(0.814271616163)*x_ref[2]
        ref[1,1,1,0]=(0.384343377969)*x_ref[0]**o+(0.463218368911)*x_ref[0]+(0.35576368147)*x_ref[1]**o+(-0.834867049786)*x_ref[1]+(-0.274552267393)*x_ref[2]**o+(0.623371271421)*x_ref[2]
        ref[1,1,1,1]=(-0.61717606851)*x_ref[0]**o+(-0.578562595163)*x_ref[0]+(0.0518387545717)*x_ref[1]**o+(-0.30815303855)*x_ref[1]+(0.556738906305)*x_ref[2]**o+(0.38164058851)*x_ref[2]
        ref[1,1,2,0]=(0.52042073171)*x_ref[0]**o+(0.698894225253)*x_ref[0]+(0.120169831848)*x_ref[1]**o+(0.238450852883)*x_ref[1]+(-0.552966574246)*x_ref[2]**o+(0.661290976968)*x_ref[2]
        ref[1,1,2,1]=(-0.428668260768)*x_ref[0]**o+(0.568751515596)*x_ref[0]+(-0.0768903249884)*x_ref[1]**o+(0.768814424639)*x_ref[1]+(0.424759560455)*x_ref[2]**o+(-0.690938863922)*x_ref[2]
        ref[1,2,0,0]=(-0.5443917305)*x_ref[0]**o+(-0.0595999390634)*x_ref[0]+(0.901507044764)*x_ref[1]**o+(-0.681191541382)*x_ref[1]+(-0.123122907072)*x_ref[2]**o+(-0.152869512816)*x_ref[2]
        ref[1,2,0,1]=(-0.0145795630242)*x_ref[0]**o+(-0.53313358583)*x_ref[0]+(0.629028937665)*x_ref[1]**o+(-0.539546800632)*x_ref[1]+(-0.252160197397)*x_ref[2]**o+(0.0133469506662)*x_ref[2]
        ref[1,2,1,0]=(0.00594869891786)*x_ref[0]**o+(-0.56813890169)*x_ref[0]+(-0.685679553166)*x_ref[1]**o+(0.81727698867)*x_ref[1]+(-0.895427226202)*x_ref[2]**o+(0.140096007363)*x_ref[2]
        ref[1,2,1,1]=(-0.770028732259)*x_ref[0]**o+(0.827053138879)*x_ref[0]+(-0.917989309552)*x_ref[1]**o+(-0.0517956582083)*x_ref[1]+(-0.266844746028)*x_ref[2]**o+(-0.99294686081)*x_ref[2]
        ref[1,2,2,0]=(0.0529054005828)*x_ref[0]**o+(-0.807219927076)*x_ref[0]+(-0.296037864436)*x_ref[1]**o+(-0.617547075266)*x_ref[1]+(-0.966016304179)*x_ref[2]**o+(-0.168793187561)*x_ref[2]
        ref[1,2,2,1]=(0.868690089119)*x_ref[0]**o+(-0.610624824567)*x_ref[0]+(-0.411113678451)*x_ref[1]**o+(0.103726193007)*x_ref[1]+(-0.148892296878)*x_ref[2]**o+(-0.195079797362)*x_ref[2]
        ref[1,3,0,0]=(0.248806387929)*x_ref[0]**o+(-0.683762743914)*x_ref[0]+(0.460928199336)*x_ref[1]**o+(0.338430932923)*x_ref[1]+(0.589813218945)*x_ref[2]**o+(-0.632423181868)*x_ref[2]
        ref[1,3,0,1]=(-0.338476160834)*x_ref[0]**o+(0.491753392304)*x_ref[0]+(-0.590351153797)*x_ref[1]**o+(-0.448026649636)*x_ref[1]+(-0.806930505259)*x_ref[2]**o+(-0.262565462369)*x_ref[2]
        ref[1,3,1,0]=(0.808414907549)*x_ref[0]**o+(0.156426917662)*x_ref[0]+(0.33216874106)*x_ref[1]**o+(-0.714791665494)*x_ref[1]+(-0.163168903301)*x_ref[2]**o+(-0.0423061478551)*x_ref[2]
        ref[1,3,1,1]=(0.72498761831)*x_ref[0]**o+(0.08194542863)*x_ref[0]+(-0.566316087403)*x_ref[1]**o+(0.49486476938)*x_ref[1]+(0.744201752122)*x_ref[2]**o+(0.766413325553)*x_ref[2]
        ref[1,3,2,0]=(0.793853329178)*x_ref[0]**o+(-0.883754372376)*x_ref[0]+(-0.991903018183)*x_ref[1]**o+(0.279576133295)*x_ref[1]+(-0.0364449658341)*x_ref[2]**o+(-0.85024249755)*x_ref[2]
        ref[1,3,2,1]=(0.308927107876)*x_ref[0]**o+(-0.995603136748)*x_ref[0]+(-0.15217214087)*x_ref[1]**o+(-0.554310725129)*x_ref[1]+(0.69107000546)*x_ref[2]**o+(0.353155450955)*x_ref[2]
        ref[1,4,0,0]=(0.696506340076)*x_ref[0]**o+(0.889728425166)*x_ref[0]+(-0.858268204558)*x_ref[1]**o+(0.353251100246)*x_ref[1]+(-0.974872511432)*x_ref[2]**o+(-0.878064243518)*x_ref[2]
        ref[1,4,0,1]=(-0.232678527901)*x_ref[0]**o+(-0.60738965282)*x_ref[0]+(-0.629841924447)*x_ref[1]**o+(0.0731692547228)*x_ref[1]+(-0.457280478523)*x_ref[2]**o+(-0.309371862287)*x_ref[2]
        ref[1,4,1,0]=(-0.0254189247267)*x_ref[0]**o+(-0.160538018478)*x_ref[0]+(-0.757800049379)*x_ref[1]**o+(0.108707596517)*x_ref[1]+(-0.182095912638)*x_ref[2]**o+(0.565356253873)*x_ref[2]
        ref[1,4,1,1]=(0.353936491579)*x_ref[0]**o+(0.368146351094)*x_ref[0]+(-0.0271653916784)*x_ref[1]**o+(0.7863197856)*x_ref[1]+(-0.0642828131827)*x_ref[2]**o+(0.946295880643)*x_ref[2]
        ref[1,4,2,0]=(-0.151347070208)*x_ref[0]**o+(-0.490039530717)*x_ref[0]+(-0.222861393521)*x_ref[1]**o+(0.160218722103)*x_ref[1]+(-0.881185613005)*x_ref[2]**o+(0.453889724466)*x_ref[2]
        ref[1,4,2,1]=(0.0185253314435)*x_ref[0]**o+(0.824691270585)*x_ref[0]+(-0.175025465282)*x_ref[1]**o+(-0.946563236595)*x_ref[1]+(0.915012725055)*x_ref[2]**o+(-0.14949108932)*x_ref[2]
        ref[2,0,0,0]=(0.608065273397)*x_ref[0]**o+(0.103537849738)*x_ref[0]+(0.640722456034)*x_ref[1]**o+(0.707059952842)*x_ref[1]+(0.958670726841)*x_ref[2]**o+(0.178333168266)*x_ref[2]
        ref[2,0,0,1]=(0.762096192615)*x_ref[0]**o+(-0.91311774327)*x_ref[0]+(-0.636481030154)*x_ref[1]**o+(0.0485431700913)*x_ref[1]+(-0.76456752608)*x_ref[2]**o+(0.988366648174)*x_ref[2]
        ref[2,0,1,0]=(-0.997545907902)*x_ref[0]**o+(0.0386694282297)*x_ref[0]+(-0.95454977224)*x_ref[1]**o+(-0.472500001853)*x_ref[1]+(-0.315668831265)*x_ref[2]**o+(-0.0911950672323)*x_ref[2]
        ref[2,0,1,1]=(0.758639431658)*x_ref[0]**o+(-0.35267267476)*x_ref[0]+(0.733519690739)*x_ref[1]**o+(0.687429543066)*x_ref[1]+(0.748932363975)*x_ref[2]**o+(-0.360330977046)*x_ref[2]
        ref[2,0,2,0]=(0.748882354686)*x_ref[0]**o+(0.398195068825)*x_ref[0]+(0.268707244918)*x_ref[1]**o+(-0.906848889273)*x_ref[1]+(0.8203042296)*x_ref[2]**o+(-0.388209014017)*x_ref[2]
        ref[2,0,2,1]=(-0.622158650545)*x_ref[0]**o+(0.757162985883)*x_ref[0]+(0.213276305935)*x_ref[1]**o+(0.376995936453)*x_ref[1]+(-0.919851919774)*x_ref[2]**o+(-0.341006967087)*x_ref[2]
        ref[2,1,0,0]=(0.45283405067)*x_ref[0]**o+(-0.456183034235)*x_ref[0]+(-0.211030502465)*x_ref[1]**o+(0.874532774776)*x_ref[1]+(-0.245927412434)*x_ref[2]**o+(0.426470240035)*x_ref[2]
        ref[2,1,0,1]=(-0.167484852552)*x_ref[0]**o+(0.23618650325)*x_ref[0]+(-0.256431628436)*x_ref[1]**o+(-0.758314815145)*x_ref[1]+(-0.553126534983)*x_ref[2]**o+(-0.993390924398)*x_ref[2]
        ref[2,1,1,0]=(-0.579896706714)*x_ref[0]**o+(0.0247821970864)*x_ref[0]+(-0.485603356124)*x_ref[1]**o+(0.610550482437)*x_ref[1]+(-0.788558005559)*x_ref[2]**o+(-0.716855983085)*x_ref[2]
        ref[2,1,1,1]=(0.011213932334)*x_ref[0]**o+(-0.606541681102)*x_ref[0]+(-0.783918973118)*x_ref[1]**o+(-0.129337676239)*x_ref[1]+(-0.681878469428)*x_ref[2]**o+(-0.818559911243)*x_ref[2]
        ref[2,1,2,0]=(0.806311020506)*x_ref[0]**o+(0.348481209449)*x_ref[0]+(0.147356878845)*x_ref[1]**o+(0.481921509802)*x_ref[1]+(0.445429036477)*x_ref[2]**o+(0.113745424091)*x_ref[2]
        ref[2,1,2,1]=(-0.587849278852)*x_ref[0]**o+(-0.141739080759)*x_ref[0]+(-0.0431697185243)*x_ref[1]**o+(0.845205068629)*x_ref[1]+(0.485473344628)*x_ref[2]**o+(0.416700333568)*x_ref[2]
        ref[2,2,0,0]=(-0.36580939154)*x_ref[0]**o+(-0.826777569794)*x_ref[0]+(-0.322507767896)*x_ref[1]**o+(0.697307401686)*x_ref[1]+(0.52326834016)*x_ref[2]**o+(-0.31919043817)*x_ref[2]
        ref[2,2,0,1]=(-0.130664405569)*x_ref[0]**o+(0.823599150158)*x_ref[0]+(0.751486085263)*x_ref[1]**o+(0.150119789292)*x_ref[1]+(-0.307393365311)*x_ref[2]**o+(0.919517717122)*x_ref[2]
        ref[2,2,1,0]=(0.254288128084)*x_ref[0]**o+(0.580902444535)*x_ref[0]+(0.90113516563)*x_ref[1]**o+(0.805502401301)*x_ref[1]+(-0.483775735536)*x_ref[2]**o+(-0.846660440246)*x_ref[2]
        ref[2,2,1,1]=(-0.481100104989)*x_ref[0]**o+(0.459159658041)*x_ref[0]+(-0.886023770838)*x_ref[1]**o+(0.693304415006)*x_ref[1]+(-0.490424960617)*x_ref[2]**o+(0.618464834808)*x_ref[2]
        ref[2,2,2,0]=(0.851037708736)*x_ref[0]**o+(0.973238080753)*x_ref[0]+(0.617606926402)*x_ref[1]**o+(-0.0129774413768)*x_ref[1]+(0.342094147972)*x_ref[2]**o+(-0.698988316685)*x_ref[2]
        ref[2,2,2,1]=(-0.571806691565)*x_ref[0]**o+(-0.488244261444)*x_ref[0]+(-0.408779266509)*x_ref[1]**o+(-0.688083559849)*x_ref[1]+(-0.668712586564)*x_ref[2]**o+(0.811908851853)*x_ref[2]
        ref[2,3,0,0]=(0.305354422638)*x_ref[0]**o+(-0.347372904868)*x_ref[0]+(0.147767334473)*x_ref[1]**o+(0.597885123439)*x_ref[1]+(-0.532183927826)*x_ref[2]**o+(0.778189138615)*x_ref[2]
        ref[2,3,0,1]=(0.986328831124)*x_ref[0]**o+(0.237847460579)*x_ref[0]+(0.350775306824)*x_ref[1]**o+(-0.750995843201)*x_ref[1]+(0.85575809431)*x_ref[2]**o+(0.624236482292)*x_ref[2]
        ref[2,3,1,0]=(-0.418600918765)*x_ref[0]**o+(-0.958489157688)*x_ref[0]+(-0.524398996544)*x_ref[1]**o+(0.186795580218)*x_ref[1]+(-0.364079386452)*x_ref[2]**o+(-0.330796017113)*x_ref[2]
        ref[2,3,1,1]=(0.632410928458)*x_ref[0]**o+(-0.557450605906)*x_ref[0]+(-0.699741830058)*x_ref[1]**o+(0.867693032849)*x_ref[1]+(-0.770879368311)*x_ref[2]**o+(0.972808988148)*x_ref[2]
        ref[2,3,2,0]=(0.283379195721)*x_ref[0]**o+(-0.636902680473)*x_ref[0]+(0.609885794066)*x_ref[1]**o+(0.831554040093)*x_ref[1]+(-0.330474761682)*x_ref[2]**o+(0.000310556070561)*x_ref[2]
        ref[2,3,2,1]=(0.363575347708)*x_ref[0]**o+(-0.327653113631)*x_ref[0]+(-0.842370028303)*x_ref[1]**o+(0.115573924346)*x_ref[1]+(0.0903633504982)*x_ref[2]**o+(-0.619568111717)*x_ref[2]
        ref[2,4,0,0]=(0.319164526641)*x_ref[0]**o+(0.45922058507)*x_ref[0]+(0.26273868718)*x_ref[1]**o+(0.919180620189)*x_ref[1]+(0.68231305095)*x_ref[2]**o+(-0.0793024563191)*x_ref[2]
        ref[2,4,0,1]=(0.966152397311)*x_ref[0]**o+(-0.323284283817)*x_ref[0]+(0.68073876378)*x_ref[1]**o+(0.353025011982)*x_ref[1]+(0.907154492113)*x_ref[2]**o+(0.371499834679)*x_ref[2]
        ref[2,4,1,0]=(0.441142086658)*x_ref[0]**o+(-0.374316420849)*x_ref[0]+(0.886357340085)*x_ref[1]**o+(0.172424726539)*x_ref[1]+(0.416195908431)*x_ref[2]**o+(0.374779208985)*x_ref[2]
        ref[2,4,1,1]=(0.809132101144)*x_ref[0]**o+(0.33952471205)*x_ref[0]+(0.159476692789)*x_ref[1]**o+(-0.719297128466)*x_ref[1]+(-0.395244031022)*x_ref[2]**o+(-0.942445288252)*x_ref[2]
        ref[2,4,2,0]=(-0.0269665566962)*x_ref[0]**o+(-0.932688309921)*x_ref[0]+(0.980071332595)*x_ref[1]**o+(0.778605742399)*x_ref[1]+(-0.69781881904)*x_ref[2]**o+(0.919684867278)*x_ref[2]
        ref[2,4,2,1]=(0.440810804674)*x_ref[0]**o+(0.642802841983)*x_ref[0]+(-0.151896393227)*x_ref[1]**o+(0.5114517147)*x_ref[1]+(0.202606346779)*x_ref[2]**o+(0.145680766426)*x_ref[2]
        ref[3,0,0,0]=(0.854905410474)*x_ref[0]**o+(-0.883160499153)*x_ref[0]+(-0.0518436529324)*x_ref[1]**o+(0.472365408955)*x_ref[1]+(-0.49219002513)*x_ref[2]**o+(0.232861430592)*x_ref[2]
        ref[3,0,0,1]=(-0.046116184691)*x_ref[0]**o+(0.596829797988)*x_ref[0]+(-0.144691397205)*x_ref[1]**o+(-0.252432054327)*x_ref[1]+(0.0799656699566)*x_ref[2]**o+(-0.220413254006)*x_ref[2]
        ref[3,0,1,0]=(0.675157805468)*x_ref[0]**o+(0.655696700297)*x_ref[0]+(0.474709137832)*x_ref[1]**o+(0.121302534807)*x_ref[1]+(-0.718410825564)*x_ref[2]**o+(-0.671917363185)*x_ref[2]
        ref[3,0,1,1]=(-0.514174913964)*x_ref[0]**o+(-0.0161999364854)*x_ref[0]+(-0.38933612074)*x_ref[1]**o+(-0.629975655537)*x_ref[1]+(0.488451340059)*x_ref[2]**o+(-0.583580208023)*x_ref[2]
        ref[3,0,2,0]=(-0.56947915393)*x_ref[0]**o+(0.434845155767)*x_ref[0]+(-0.968993748561)*x_ref[1]**o+(-0.510861471635)*x_ref[1]+(-0.288591065135)*x_ref[2]**o+(-0.230425132856)*x_ref[2]
        ref[3,0,2,1]=(-0.374979517517)*x_ref[0]**o+(-0.0163569866605)*x_ref[0]+(0.75837797441)*x_ref[1]**o+(-0.632240609496)*x_ref[1]+(0.422914146097)*x_ref[2]**o+(0.951370365751)*x_ref[2]
        ref[3,1,0,0]=(-0.991849908442)*x_ref[0]**o+(-0.00797307615614)*x_ref[0]+(-0.464488377523)*x_ref[1]**o+(0.183107151355)*x_ref[1]+(-0.470200932584)*x_ref[2]**o+(-0.37271407313)*x_ref[2]
        ref[3,1,0,1]=(0.714765401281)*x_ref[0]**o+(0.604856800205)*x_ref[0]+(-0.550067009901)*x_ref[1]**o+(-0.5869512121)*x_ref[1]+(0.620821251646)*x_ref[2]**o+(-0.897762101083)*x_ref[2]
        ref[3,1,1,0]=(0.347567608921)*x_ref[0]**o+(0.425122715827)*x_ref[0]+(-0.802475224849)*x_ref[1]**o+(-0.522837292632)*x_ref[1]+(-0.160768805832)*x_ref[2]**o+(-0.445406093759)*x_ref[2]
        ref[3,1,1,1]=(0.320284660978)*x_ref[0]**o+(0.252323047651)*x_ref[0]+(-0.615677083087)*x_ref[1]**o+(-0.85623985966)*x_ref[1]+(-0.0480138503943)*x_ref[2]**o+(0.106970090808)*x_ref[2]
        ref[3,1,2,0]=(0.744223828984)*x_ref[0]**o+(0.0646702358772)*x_ref[0]+(-0.780189264654)*x_ref[1]**o+(-0.946938785995)*x_ref[1]+(-0.73270156813)*x_ref[2]**o+(-0.240670757875)*x_ref[2]
        ref[3,1,2,1]=(0.24593668579)*x_ref[0]**o+(0.0666411656061)*x_ref[0]+(0.278029304996)*x_ref[1]**o+(-0.543159254692)*x_ref[1]+(-0.394304458971)*x_ref[2]**o+(0.0789963203485)*x_ref[2]
        ref[3,2,0,0]=(-0.129597165729)*x_ref[0]**o+(-0.375126607602)*x_ref[0]+(-0.133573059675)*x_ref[1]**o+(-0.222965614248)*x_ref[1]+(0.482498045295)*x_ref[2]**o+(0.666380815962)*x_ref[2]
        ref[3,2,0,1]=(-0.298314889544)*x_ref[0]**o+(-0.200460718422)*x_ref[0]+(0.969934227091)*x_ref[1]**o+(0.982204235633)*x_ref[1]+(-0.875400986072)*x_ref[2]**o+(-0.962005163625)*x_ref[2]
        ref[3,2,1,0]=(-0.744314645975)*x_ref[0]**o+(0.899141755704)*x_ref[0]+(0.738788130658)*x_ref[1]**o+(-0.465548783282)*x_ref[1]+(0.27881485753)*x_ref[2]**o+(-0.711524090298)*x_ref[2]
        ref[3,2,1,1]=(-0.698617989028)*x_ref[0]**o+(0.082546181429)*x_ref[0]+(-0.611282109662)*x_ref[1]**o+(-0.28088009072)*x_ref[1]+(-0.260638676131)*x_ref[2]**o+(-0.554757759687)*x_ref[2]
        ref[3,2,2,0]=(-0.934985159158)*x_ref[0]**o+(0.366271859175)*x_ref[0]+(0.55018247768)*x_ref[1]**o+(0.114993902724)*x_ref[1]+(0.196571047024)*x_ref[2]**o+(-0.889655441435)*x_ref[2]
        ref[3,2,2,1]=(-0.76297489993)*x_ref[0]**o+(-0.00479337140756)*x_ref[0]+(0.853999531704)*x_ref[1]**o+(-0.563106979943)*x_ref[1]+(-0.925873572722)*x_ref[2]**o+(-0.0277052508628)*x_ref[2]
        ref[3,3,0,0]=(-0.138137971158)*x_ref[0]**o+(-0.41896080496)*x_ref[0]+(0.0482713349343)*x_ref[1]**o+(-0.21852867848)*x_ref[1]+(0.0190850076189)*x_ref[2]**o+(0.583969066138)*x_ref[2]
        ref[3,3,0,1]=(0.806413890451)*x_ref[0]**o+(-0.863828929679)*x_ref[0]+(-0.152284946303)*x_ref[1]**o+(-0.41164748505)*x_ref[1]+(-0.206330733354)*x_ref[2]**o+(0.127114872982)*x_ref[2]
        ref[3,3,1,0]=(-0.418785729088)*x_ref[0]**o+(-0.0658589519492)*x_ref[0]+(-0.696602204394)*x_ref[1]**o+(-0.725906543227)*x_ref[1]+(0.968658365691)*x_ref[2]**o+(-0.262448319217)*x_ref[2]
        ref[3,3,1,1]=(-0.189305081379)*x_ref[0]**o+(-0.709824241653)*x_ref[0]+(-0.799048663139)*x_ref[1]**o+(0.0420441850736)*x_ref[1]+(-0.443839189643)*x_ref[2]**o+(0.867954924509)*x_ref[2]
        ref[3,3,2,0]=(0.56856531819)*x_ref[0]**o+(0.227835800084)*x_ref[0]+(-0.521859405163)*x_ref[1]**o+(-0.869003176683)*x_ref[1]+(-0.684378595493)*x_ref[2]**o+(0.544243688121)*x_ref[2]
        ref[3,3,2,1]=(0.714635391612)*x_ref[0]**o+(-0.718360328239)*x_ref[0]+(-0.510003226793)*x_ref[1]**o+(-0.857292216621)*x_ref[1]+(-0.295240904568)*x_ref[2]**o+(0.340409128873)*x_ref[2]
        ref[3,4,0,0]=(-0.715361279417)*x_ref[0]**o+(-0.829022388935)*x_ref[0]+(0.56459861288)*x_ref[1]**o+(0.879337406251)*x_ref[1]+(0.423698189396)*x_ref[2]**o+(-0.763053018627)*x_ref[2]
        ref[3,4,0,1]=(0.282251024454)*x_ref[0]**o+(-0.275549571404)*x_ref[0]+(0.416099858947)*x_ref[1]**o+(0.304537400335)*x_ref[1]+(0.846027568045)*x_ref[2]**o+(-0.00236635940042)*x_ref[2]
        ref[3,4,1,0]=(-0.396026824114)*x_ref[0]**o+(-0.967573299876)*x_ref[0]+(0.419395952325)*x_ref[1]**o+(-0.0769940862315)*x_ref[1]+(-0.451079361856)*x_ref[2]**o+(0.891465362708)*x_ref[2]
        ref[3,4,1,1]=(-0.629373368312)*x_ref[0]**o+(-0.0463131060729)*x_ref[0]+(-0.285992266736)*x_ref[1]**o+(0.852693491625)*x_ref[1]+(0.101111799967)*x_ref[2]**o+(-0.982099200191)*x_ref[2]
        ref[3,4,2,0]=(-0.840902275944)*x_ref[0]**o+(0.578695985974)*x_ref[0]+(0.533595808779)*x_ref[1]**o+(0.970844141257)*x_ref[1]+(-0.942252307966)*x_ref[2]**o+(-0.051129339745)*x_ref[2]
        ref[3,4,2,1]=(-0.981944223126)*x_ref[0]**o+(-0.0930019659212)*x_ref[0]+(-0.239359393705)*x_ref[1]**o+(-0.277892188253)*x_ref[1]+(0.447927318111)*x_ref[2]**o+(-0.860595690016)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromSymbol_Solution_rank0(self):
      """
      tests interpolate for rank 0 Symbol onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from Solution onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(),dim=dim)
      s=Data(0,(),w)
      ref=Data(0,(),w_ref)
      if dim==2:
        s=(-0.420247061675)*x[0]**o+(0.845150024832)*x[0]+(-0.70586438118)*x[1]**o+(0.336975044874)*x[1]
        ref=(-0.420247061675)*x_ref[0]**o+(0.845150024832)*x_ref[0]+(-0.70586438118)*x_ref[1]**o+(0.336975044874)*x_ref[1]
      else:
        s=(-0.280517761942)*x[0]**o+(-0.612238168916)*x[0]+(-0.103863621547)*x[1]**o+(-0.0303139932438)*x[1]+(0.216524064885)*x[2]**o+(-0.722384143847)*x[2]
        ref=(-0.280517761942)*x_ref[0]**o+(-0.612238168916)*x_ref[0]+(-0.103863621547)*x_ref[1]**o+(-0.0303139932438)*x_ref[1]+(0.216524064885)*x_ref[2]**o+(-0.722384143847)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromSymbol_Solution_rank1(self):
      """
      tests interpolate for rank 1 Symbol onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from Solution onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(2,),dim=dim)
      s=Data(0,(2,),w)
      ref=Data(0,(2,),w_ref)
      if dim==2:
        s[0]=(0.572752317912)*x[0]**o+(-0.557996031811)*x[0]+(-0.105666248913)*x[1]**o+(0.285149455719)*x[1]
        s[1]=(0.83173456891)*x[0]**o+(-0.347928678237)*x[0]+(0.326013379698)*x[1]**o+(0.949427685784)*x[1]
        ref[0]=(0.572752317912)*x_ref[0]**o+(-0.557996031811)*x_ref[0]+(-0.105666248913)*x_ref[1]**o+(0.285149455719)*x_ref[1]
        ref[1]=(0.83173456891)*x_ref[0]**o+(-0.347928678237)*x_ref[0]+(0.326013379698)*x_ref[1]**o+(0.949427685784)*x_ref[1]
      else:
        s[0]=(0.254569308395)*x[0]**o+(-0.199165607071)*x[0]+(0.368026751091)*x[1]**o+(0.749363957437)*x[1]+(0.784765582138)*x[2]**o+(-0.385360238329)*x[2]
        s[1]=(0.915245931081)*x[0]**o+(-0.427166268095)*x[0]+(-0.310327593132)*x[1]**o+(0.0685458597459)*x[1]+(0.951457086556)*x[2]**o+(-0.103882183326)*x[2]
        ref[0]=(0.254569308395)*x_ref[0]**o+(-0.199165607071)*x_ref[0]+(0.368026751091)*x_ref[1]**o+(0.749363957437)*x_ref[1]+(0.784765582138)*x_ref[2]**o+(-0.385360238329)*x_ref[2]
        ref[1]=(0.915245931081)*x_ref[0]**o+(-0.427166268095)*x_ref[0]+(-0.310327593132)*x_ref[1]**o+(0.0685458597459)*x_ref[1]+(0.951457086556)*x_ref[2]**o+(-0.103882183326)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromSymbol_Solution_rank2(self):
      """
      tests interpolate for rank 2 Symbol onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from Solution onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5),dim=dim)
      s=Data(0,(4, 5),w)
      ref=Data(0,(4, 5),w_ref)
      if dim==2:
        s[0,0]=(0.835384508525)*x[0]**o+(-0.0883356828618)*x[0]+(0.540787143442)*x[1]**o+(-0.844850006869)*x[1]
        s[0,1]=(-0.604551369282)*x[0]**o+(0.364670860569)*x[0]+(0.0483050521734)*x[1]**o+(-0.203895463114)*x[1]
        s[0,2]=(-0.815610974203)*x[0]**o+(0.833865082306)*x[0]+(0.0720866771967)*x[1]**o+(-0.247530114868)*x[1]
        s[0,3]=(-0.735317110832)*x[0]**o+(-0.148571378133)*x[0]+(-0.231962284467)*x[1]**o+(0.636603292819)*x[1]
        s[0,4]=(-0.649798394504)*x[0]**o+(0.185726064609)*x[0]+(0.484279786833)*x[1]**o+(0.519453975218)*x[1]
        s[1,0]=(-0.79002150838)*x[0]**o+(-0.899574857585)*x[0]+(-0.233297202379)*x[1]**o+(0.352419375812)*x[1]
        s[1,1]=(0.607990106144)*x[0]**o+(-0.916411570588)*x[0]+(-0.172031442436)*x[1]**o+(0.795245984279)*x[1]
        s[1,2]=(-0.534291044354)*x[0]**o+(0.0230033818057)*x[0]+(0.399956972297)*x[1]**o+(-0.115654728987)*x[1]
        s[1,3]=(0.0920349813015)*x[0]**o+(-0.0312648034967)*x[0]+(0.347084284907)*x[1]**o+(-0.796084479949)*x[1]
        s[1,4]=(0.341416390322)*x[0]**o+(0.400112572704)*x[0]+(-0.0833860476359)*x[1]**o+(0.999568411991)*x[1]
        s[2,0]=(0.128164199552)*x[0]**o+(0.812514973535)*x[0]+(-0.118931795809)*x[1]**o+(-0.375359555032)*x[1]
        s[2,1]=(-0.288738444027)*x[0]**o+(0.572163772906)*x[0]+(0.184909580584)*x[1]**o+(-0.727127325602)*x[1]
        s[2,2]=(0.170486550408)*x[0]**o+(0.493448396565)*x[0]+(0.00435735862836)*x[1]**o+(0.250237688279)*x[1]
        s[2,3]=(-0.7062351645)*x[0]**o+(-0.0122464096259)*x[0]+(-0.976360268944)*x[1]**o+(0.265587257351)*x[1]
        s[2,4]=(-0.710741742377)*x[0]**o+(0.111377096886)*x[0]+(0.518621968579)*x[1]**o+(-0.264799841387)*x[1]
        s[3,0]=(0.661089560632)*x[0]**o+(-0.175284355246)*x[0]+(0.434737404117)*x[1]**o+(-0.569800530193)*x[1]
        s[3,1]=(-0.581369919535)*x[0]**o+(-0.599162778688)*x[0]+(-0.772449383214)*x[1]**o+(-0.286495247533)*x[1]
        s[3,2]=(0.350919520995)*x[0]**o+(0.2325739414)*x[0]+(0.744285468619)*x[1]**o+(-0.603444606663)*x[1]
        s[3,3]=(-0.946270717928)*x[0]**o+(0.944211313875)*x[0]+(0.128830208798)*x[1]**o+(-0.108311954532)*x[1]
        s[3,4]=(-0.0704013779853)*x[0]**o+(-0.334719508353)*x[0]+(0.39364248466)*x[1]**o+(-0.894987529202)*x[1]
        ref[0,0]=(0.835384508525)*x_ref[0]**o+(-0.0883356828618)*x_ref[0]+(0.540787143442)*x_ref[1]**o+(-0.844850006869)*x_ref[1]
        ref[0,1]=(-0.604551369282)*x_ref[0]**o+(0.364670860569)*x_ref[0]+(0.0483050521734)*x_ref[1]**o+(-0.203895463114)*x_ref[1]
        ref[0,2]=(-0.815610974203)*x_ref[0]**o+(0.833865082306)*x_ref[0]+(0.0720866771967)*x_ref[1]**o+(-0.247530114868)*x_ref[1]
        ref[0,3]=(-0.735317110832)*x_ref[0]**o+(-0.148571378133)*x_ref[0]+(-0.231962284467)*x_ref[1]**o+(0.636603292819)*x_ref[1]
        ref[0,4]=(-0.649798394504)*x_ref[0]**o+(0.185726064609)*x_ref[0]+(0.484279786833)*x_ref[1]**o+(0.519453975218)*x_ref[1]
        ref[1,0]=(-0.79002150838)*x_ref[0]**o+(-0.899574857585)*x_ref[0]+(-0.233297202379)*x_ref[1]**o+(0.352419375812)*x_ref[1]
        ref[1,1]=(0.607990106144)*x_ref[0]**o+(-0.916411570588)*x_ref[0]+(-0.172031442436)*x_ref[1]**o+(0.795245984279)*x_ref[1]
        ref[1,2]=(-0.534291044354)*x_ref[0]**o+(0.0230033818057)*x_ref[0]+(0.399956972297)*x_ref[1]**o+(-0.115654728987)*x_ref[1]
        ref[1,3]=(0.0920349813015)*x_ref[0]**o+(-0.0312648034967)*x_ref[0]+(0.347084284907)*x_ref[1]**o+(-0.796084479949)*x_ref[1]
        ref[1,4]=(0.341416390322)*x_ref[0]**o+(0.400112572704)*x_ref[0]+(-0.0833860476359)*x_ref[1]**o+(0.999568411991)*x_ref[1]
        ref[2,0]=(0.128164199552)*x_ref[0]**o+(0.812514973535)*x_ref[0]+(-0.118931795809)*x_ref[1]**o+(-0.375359555032)*x_ref[1]
        ref[2,1]=(-0.288738444027)*x_ref[0]**o+(0.572163772906)*x_ref[0]+(0.184909580584)*x_ref[1]**o+(-0.727127325602)*x_ref[1]
        ref[2,2]=(0.170486550408)*x_ref[0]**o+(0.493448396565)*x_ref[0]+(0.00435735862836)*x_ref[1]**o+(0.250237688279)*x_ref[1]
        ref[2,3]=(-0.7062351645)*x_ref[0]**o+(-0.0122464096259)*x_ref[0]+(-0.976360268944)*x_ref[1]**o+(0.265587257351)*x_ref[1]
        ref[2,4]=(-0.710741742377)*x_ref[0]**o+(0.111377096886)*x_ref[0]+(0.518621968579)*x_ref[1]**o+(-0.264799841387)*x_ref[1]
        ref[3,0]=(0.661089560632)*x_ref[0]**o+(-0.175284355246)*x_ref[0]+(0.434737404117)*x_ref[1]**o+(-0.569800530193)*x_ref[1]
        ref[3,1]=(-0.581369919535)*x_ref[0]**o+(-0.599162778688)*x_ref[0]+(-0.772449383214)*x_ref[1]**o+(-0.286495247533)*x_ref[1]
        ref[3,2]=(0.350919520995)*x_ref[0]**o+(0.2325739414)*x_ref[0]+(0.744285468619)*x_ref[1]**o+(-0.603444606663)*x_ref[1]
        ref[3,3]=(-0.946270717928)*x_ref[0]**o+(0.944211313875)*x_ref[0]+(0.128830208798)*x_ref[1]**o+(-0.108311954532)*x_ref[1]
        ref[3,4]=(-0.0704013779853)*x_ref[0]**o+(-0.334719508353)*x_ref[0]+(0.39364248466)*x_ref[1]**o+(-0.894987529202)*x_ref[1]
      else:
        s[0,0]=(0.99056578049)*x[0]**o+(0.509128967554)*x[0]+(0.171080190069)*x[1]**o+(0.522380534005)*x[1]+(0.431016370389)*x[2]**o+(-0.993934787463)*x[2]
        s[0,1]=(0.440865158692)*x[0]**o+(0.469110688262)*x[0]+(-0.44415137348)*x[1]**o+(-0.592580419437)*x[1]+(0.725915122511)*x[2]**o+(0.978265572749)*x[2]
        s[0,2]=(0.759448293392)*x[0]**o+(-0.673851848255)*x[0]+(-0.850110053326)*x[1]**o+(0.542563968866)*x[1]+(-0.593647236451)*x[2]**o+(0.238446783033)*x[2]
        s[0,3]=(0.124244333648)*x[0]**o+(-0.72755263687)*x[0]+(-0.694167850492)*x[1]**o+(-0.613367096258)*x[1]+(0.750711571202)*x[2]**o+(0.931327127719)*x[2]
        s[0,4]=(-0.501005188112)*x[0]**o+(-0.844862164905)*x[0]+(-0.771531270162)*x[1]**o+(-0.379614684646)*x[1]+(0.668772926463)*x[2]**o+(0.452222970598)*x[2]
        s[1,0]=(-0.903660331121)*x[0]**o+(0.885970144397)*x[0]+(-0.589427430828)*x[1]**o+(0.149545558395)*x[1]+(-0.672063110451)*x[2]**o+(0.809659958716)*x[2]
        s[1,1]=(0.443858030252)*x[0]**o+(0.268545734898)*x[0]+(-0.723221753395)*x[1]**o+(-0.497164094332)*x[1]+(-0.38727254966)*x[2]**o+(-0.713966847727)*x[2]
        s[1,2]=(-0.141155739442)*x[0]**o+(0.147814163801)*x[0]+(0.30705733457)*x[1]**o+(0.671290544062)*x[1]+(-0.48022640662)*x[2]**o+(-0.258276084908)*x[2]
        s[1,3]=(-0.23260749408)*x[0]**o+(0.685806077106)*x[0]+(-0.802672585899)*x[1]**o+(0.220698760546)*x[1]+(-0.930587997597)*x[2]**o+(0.131923598812)*x[2]
        s[1,4]=(-0.0116512039336)*x[0]**o+(0.949770706957)*x[0]+(-0.315276290308)*x[1]**o+(0.385326039063)*x[1]+(0.508964415622)*x[2]**o+(0.813022533544)*x[2]
        s[2,0]=(0.214620768745)*x[0]**o+(-0.378543435195)*x[0]+(-0.415588213697)*x[1]**o+(0.390408636588)*x[1]+(0.89210598718)*x[2]**o+(0.649105001473)*x[2]
        s[2,1]=(-0.9290010077)*x[0]**o+(-0.819789531059)*x[0]+(-0.31733087944)*x[1]**o+(0.18679256152)*x[1]+(-0.595489801854)*x[2]**o+(0.376682692017)*x[2]
        s[2,2]=(-0.396100840781)*x[0]**o+(-0.962832400773)*x[0]+(-0.97170822186)*x[1]**o+(0.153092378797)*x[1]+(-0.102354054108)*x[2]**o+(0.0569186680057)*x[2]
        s[2,3]=(-0.736586445243)*x[0]**o+(-0.453505210464)*x[0]+(0.0894043232592)*x[1]**o+(0.352561026525)*x[1]+(0.950731094588)*x[2]**o+(0.170774557675)*x[2]
        s[2,4]=(-0.742839390272)*x[0]**o+(0.62294122589)*x[0]+(-0.543260730767)*x[1]**o+(0.189330809244)*x[1]+(-0.603776416284)*x[2]**o+(0.875279918494)*x[2]
        s[3,0]=(-0.409012740744)*x[0]**o+(-0.245267437205)*x[0]+(0.818223917843)*x[1]**o+(-0.962578887383)*x[1]+(-0.51108067434)*x[2]**o+(-0.0692224100799)*x[2]
        s[3,1]=(0.701052191719)*x[0]**o+(-0.611563296923)*x[0]+(0.584279125155)*x[1]**o+(0.324940331516)*x[1]+(-0.0474533585109)*x[2]**o+(-0.226269391305)*x[2]
        s[3,2]=(0.601966725574)*x[0]**o+(0.698971273869)*x[0]+(0.832499644008)*x[1]**o+(-0.680484279276)*x[1]+(0.557754387767)*x[2]**o+(0.589582740674)*x[2]
        s[3,3]=(0.555182612378)*x[0]**o+(-0.548416585377)*x[0]+(0.171087446868)*x[1]**o+(0.252969734012)*x[1]+(-0.291200177721)*x[2]**o+(0.591022017095)*x[2]
        s[3,4]=(0.358263224328)*x[0]**o+(0.618229036593)*x[0]+(-0.853797452875)*x[1]**o+(-0.689076935276)*x[1]+(-0.705975774167)*x[2]**o+(-0.767101969914)*x[2]
        ref[0,0]=(0.99056578049)*x_ref[0]**o+(0.509128967554)*x_ref[0]+(0.171080190069)*x_ref[1]**o+(0.522380534005)*x_ref[1]+(0.431016370389)*x_ref[2]**o+(-0.993934787463)*x_ref[2]
        ref[0,1]=(0.440865158692)*x_ref[0]**o+(0.469110688262)*x_ref[0]+(-0.44415137348)*x_ref[1]**o+(-0.592580419437)*x_ref[1]+(0.725915122511)*x_ref[2]**o+(0.978265572749)*x_ref[2]
        ref[0,2]=(0.759448293392)*x_ref[0]**o+(-0.673851848255)*x_ref[0]+(-0.850110053326)*x_ref[1]**o+(0.542563968866)*x_ref[1]+(-0.593647236451)*x_ref[2]**o+(0.238446783033)*x_ref[2]
        ref[0,3]=(0.124244333648)*x_ref[0]**o+(-0.72755263687)*x_ref[0]+(-0.694167850492)*x_ref[1]**o+(-0.613367096258)*x_ref[1]+(0.750711571202)*x_ref[2]**o+(0.931327127719)*x_ref[2]
        ref[0,4]=(-0.501005188112)*x_ref[0]**o+(-0.844862164905)*x_ref[0]+(-0.771531270162)*x_ref[1]**o+(-0.379614684646)*x_ref[1]+(0.668772926463)*x_ref[2]**o+(0.452222970598)*x_ref[2]
        ref[1,0]=(-0.903660331121)*x_ref[0]**o+(0.885970144397)*x_ref[0]+(-0.589427430828)*x_ref[1]**o+(0.149545558395)*x_ref[1]+(-0.672063110451)*x_ref[2]**o+(0.809659958716)*x_ref[2]
        ref[1,1]=(0.443858030252)*x_ref[0]**o+(0.268545734898)*x_ref[0]+(-0.723221753395)*x_ref[1]**o+(-0.497164094332)*x_ref[1]+(-0.38727254966)*x_ref[2]**o+(-0.713966847727)*x_ref[2]
        ref[1,2]=(-0.141155739442)*x_ref[0]**o+(0.147814163801)*x_ref[0]+(0.30705733457)*x_ref[1]**o+(0.671290544062)*x_ref[1]+(-0.48022640662)*x_ref[2]**o+(-0.258276084908)*x_ref[2]
        ref[1,3]=(-0.23260749408)*x_ref[0]**o+(0.685806077106)*x_ref[0]+(-0.802672585899)*x_ref[1]**o+(0.220698760546)*x_ref[1]+(-0.930587997597)*x_ref[2]**o+(0.131923598812)*x_ref[2]
        ref[1,4]=(-0.0116512039336)*x_ref[0]**o+(0.949770706957)*x_ref[0]+(-0.315276290308)*x_ref[1]**o+(0.385326039063)*x_ref[1]+(0.508964415622)*x_ref[2]**o+(0.813022533544)*x_ref[2]
        ref[2,0]=(0.214620768745)*x_ref[0]**o+(-0.378543435195)*x_ref[0]+(-0.415588213697)*x_ref[1]**o+(0.390408636588)*x_ref[1]+(0.89210598718)*x_ref[2]**o+(0.649105001473)*x_ref[2]
        ref[2,1]=(-0.9290010077)*x_ref[0]**o+(-0.819789531059)*x_ref[0]+(-0.31733087944)*x_ref[1]**o+(0.18679256152)*x_ref[1]+(-0.595489801854)*x_ref[2]**o+(0.376682692017)*x_ref[2]
        ref[2,2]=(-0.396100840781)*x_ref[0]**o+(-0.962832400773)*x_ref[0]+(-0.97170822186)*x_ref[1]**o+(0.153092378797)*x_ref[1]+(-0.102354054108)*x_ref[2]**o+(0.0569186680057)*x_ref[2]
        ref[2,3]=(-0.736586445243)*x_ref[0]**o+(-0.453505210464)*x_ref[0]+(0.0894043232592)*x_ref[1]**o+(0.352561026525)*x_ref[1]+(0.950731094588)*x_ref[2]**o+(0.170774557675)*x_ref[2]
        ref[2,4]=(-0.742839390272)*x_ref[0]**o+(0.62294122589)*x_ref[0]+(-0.543260730767)*x_ref[1]**o+(0.189330809244)*x_ref[1]+(-0.603776416284)*x_ref[2]**o+(0.875279918494)*x_ref[2]
        ref[3,0]=(-0.409012740744)*x_ref[0]**o+(-0.245267437205)*x_ref[0]+(0.818223917843)*x_ref[1]**o+(-0.962578887383)*x_ref[1]+(-0.51108067434)*x_ref[2]**o+(-0.0692224100799)*x_ref[2]
        ref[3,1]=(0.701052191719)*x_ref[0]**o+(-0.611563296923)*x_ref[0]+(0.584279125155)*x_ref[1]**o+(0.324940331516)*x_ref[1]+(-0.0474533585109)*x_ref[2]**o+(-0.226269391305)*x_ref[2]
        ref[3,2]=(0.601966725574)*x_ref[0]**o+(0.698971273869)*x_ref[0]+(0.832499644008)*x_ref[1]**o+(-0.680484279276)*x_ref[1]+(0.557754387767)*x_ref[2]**o+(0.589582740674)*x_ref[2]
        ref[3,3]=(0.555182612378)*x_ref[0]**o+(-0.548416585377)*x_ref[0]+(0.171087446868)*x_ref[1]**o+(0.252969734012)*x_ref[1]+(-0.291200177721)*x_ref[2]**o+(0.591022017095)*x_ref[2]
        ref[3,4]=(0.358263224328)*x_ref[0]**o+(0.618229036593)*x_ref[0]+(-0.853797452875)*x_ref[1]**o+(-0.689076935276)*x_ref[1]+(-0.705975774167)*x_ref[2]**o+(-0.767101969914)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromSymbol_Solution_rank3(self):
      """
      tests interpolate for rank 3 Symbol onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from Solution onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(6, 2, 2),dim=dim)
      s=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2),w_ref)
      if dim==2:
        s[0,0,0]=(0.551903191753)*x[0]**o+(0.134167484401)*x[0]+(-0.970029226665)*x[1]**o+(-0.505769142138)*x[1]
        s[0,0,1]=(0.527481367323)*x[0]**o+(0.383023560586)*x[0]+(0.534661490175)*x[1]**o+(-0.454003082098)*x[1]
        s[0,1,0]=(-0.532872138974)*x[0]**o+(-0.858154622463)*x[0]+(-0.334227178563)*x[1]**o+(0.818853150256)*x[1]
        s[0,1,1]=(-0.402982114556)*x[0]**o+(-0.415574274216)*x[0]+(-0.41876444792)*x[1]**o+(0.944724142146)*x[1]
        s[1,0,0]=(0.10201141991)*x[0]**o+(-0.524044051877)*x[0]+(-0.143688302936)*x[1]**o+(0.669583775819)*x[1]
        s[1,0,1]=(-0.207189538013)*x[0]**o+(-0.293524466257)*x[0]+(0.742423576418)*x[1]**o+(0.698550598519)*x[1]
        s[1,1,0]=(0.589159582266)*x[0]**o+(-0.118585184595)*x[0]+(-0.289302989425)*x[1]**o+(0.408124429254)*x[1]
        s[1,1,1]=(0.432211891066)*x[0]**o+(0.606915869583)*x[0]+(0.945265653758)*x[1]**o+(-0.555752925873)*x[1]
        s[2,0,0]=(-0.0432578004806)*x[0]**o+(0.536208710346)*x[0]+(-0.765816194908)*x[1]**o+(0.0558566372534)*x[1]
        s[2,0,1]=(0.596807480612)*x[0]**o+(-0.0117541181841)*x[0]+(-0.167126829104)*x[1]**o+(0.810853013382)*x[1]
        s[2,1,0]=(0.1865308646)*x[0]**o+(-0.35633893825)*x[0]+(0.0578054987569)*x[1]**o+(0.454879409627)*x[1]
        s[2,1,1]=(0.977442292262)*x[0]**o+(-0.639419194896)*x[0]+(0.660710610339)*x[1]**o+(0.737541315152)*x[1]
        s[3,0,0]=(-0.448512485148)*x[0]**o+(0.0195770337028)*x[0]+(0.476589731682)*x[1]**o+(0.252822613388)*x[1]
        s[3,0,1]=(0.168702994111)*x[0]**o+(-0.262429046534)*x[0]+(-0.336163311284)*x[1]**o+(-0.496715703947)*x[1]
        s[3,1,0]=(0.527929126799)*x[0]**o+(-0.524648338276)*x[0]+(-0.639355259144)*x[1]**o+(0.944748719208)*x[1]
        s[3,1,1]=(0.544347152911)*x[0]**o+(0.411148837164)*x[0]+(-0.0708716357747)*x[1]**o+(0.83241698331)*x[1]
        s[4,0,0]=(-0.633639608573)*x[0]**o+(-0.956364543284)*x[0]+(-0.785908625875)*x[1]**o+(0.930061167858)*x[1]
        s[4,0,1]=(0.609309892405)*x[0]**o+(0.230873371053)*x[0]+(0.787017185481)*x[1]**o+(0.198757600328)*x[1]
        s[4,1,0]=(-0.697537179974)*x[0]**o+(-0.0523509220171)*x[0]+(0.279820592879)*x[1]**o+(0.763446870577)*x[1]
        s[4,1,1]=(-0.489219146209)*x[0]**o+(-0.145758299279)*x[0]+(0.565938821575)*x[1]**o+(-0.98703168748)*x[1]
        s[5,0,0]=(0.0839742626164)*x[0]**o+(0.195476369019)*x[0]+(-0.856490669921)*x[1]**o+(-0.471313686326)*x[1]
        s[5,0,1]=(-0.612045283736)*x[0]**o+(0.9330978976)*x[0]+(0.554623000698)*x[1]**o+(-0.0447734703966)*x[1]
        s[5,1,0]=(0.137284580652)*x[0]**o+(-0.576384951707)*x[0]+(-0.18207486832)*x[1]**o+(0.581595310151)*x[1]
        s[5,1,1]=(0.0169341997249)*x[0]**o+(0.0681222507261)*x[0]+(0.0252651671358)*x[1]**o+(-0.884122676141)*x[1]
        ref[0,0,0]=(0.551903191753)*x_ref[0]**o+(0.134167484401)*x_ref[0]+(-0.970029226665)*x_ref[1]**o+(-0.505769142138)*x_ref[1]
        ref[0,0,1]=(0.527481367323)*x_ref[0]**o+(0.383023560586)*x_ref[0]+(0.534661490175)*x_ref[1]**o+(-0.454003082098)*x_ref[1]
        ref[0,1,0]=(-0.532872138974)*x_ref[0]**o+(-0.858154622463)*x_ref[0]+(-0.334227178563)*x_ref[1]**o+(0.818853150256)*x_ref[1]
        ref[0,1,1]=(-0.402982114556)*x_ref[0]**o+(-0.415574274216)*x_ref[0]+(-0.41876444792)*x_ref[1]**o+(0.944724142146)*x_ref[1]
        ref[1,0,0]=(0.10201141991)*x_ref[0]**o+(-0.524044051877)*x_ref[0]+(-0.143688302936)*x_ref[1]**o+(0.669583775819)*x_ref[1]
        ref[1,0,1]=(-0.207189538013)*x_ref[0]**o+(-0.293524466257)*x_ref[0]+(0.742423576418)*x_ref[1]**o+(0.698550598519)*x_ref[1]
        ref[1,1,0]=(0.589159582266)*x_ref[0]**o+(-0.118585184595)*x_ref[0]+(-0.289302989425)*x_ref[1]**o+(0.408124429254)*x_ref[1]
        ref[1,1,1]=(0.432211891066)*x_ref[0]**o+(0.606915869583)*x_ref[0]+(0.945265653758)*x_ref[1]**o+(-0.555752925873)*x_ref[1]
        ref[2,0,0]=(-0.0432578004806)*x_ref[0]**o+(0.536208710346)*x_ref[0]+(-0.765816194908)*x_ref[1]**o+(0.0558566372534)*x_ref[1]
        ref[2,0,1]=(0.596807480612)*x_ref[0]**o+(-0.0117541181841)*x_ref[0]+(-0.167126829104)*x_ref[1]**o+(0.810853013382)*x_ref[1]
        ref[2,1,0]=(0.1865308646)*x_ref[0]**o+(-0.35633893825)*x_ref[0]+(0.0578054987569)*x_ref[1]**o+(0.454879409627)*x_ref[1]
        ref[2,1,1]=(0.977442292262)*x_ref[0]**o+(-0.639419194896)*x_ref[0]+(0.660710610339)*x_ref[1]**o+(0.737541315152)*x_ref[1]
        ref[3,0,0]=(-0.448512485148)*x_ref[0]**o+(0.0195770337028)*x_ref[0]+(0.476589731682)*x_ref[1]**o+(0.252822613388)*x_ref[1]
        ref[3,0,1]=(0.168702994111)*x_ref[0]**o+(-0.262429046534)*x_ref[0]+(-0.336163311284)*x_ref[1]**o+(-0.496715703947)*x_ref[1]
        ref[3,1,0]=(0.527929126799)*x_ref[0]**o+(-0.524648338276)*x_ref[0]+(-0.639355259144)*x_ref[1]**o+(0.944748719208)*x_ref[1]
        ref[3,1,1]=(0.544347152911)*x_ref[0]**o+(0.411148837164)*x_ref[0]+(-0.0708716357747)*x_ref[1]**o+(0.83241698331)*x_ref[1]
        ref[4,0,0]=(-0.633639608573)*x_ref[0]**o+(-0.956364543284)*x_ref[0]+(-0.785908625875)*x_ref[1]**o+(0.930061167858)*x_ref[1]
        ref[4,0,1]=(0.609309892405)*x_ref[0]**o+(0.230873371053)*x_ref[0]+(0.787017185481)*x_ref[1]**o+(0.198757600328)*x_ref[1]
        ref[4,1,0]=(-0.697537179974)*x_ref[0]**o+(-0.0523509220171)*x_ref[0]+(0.279820592879)*x_ref[1]**o+(0.763446870577)*x_ref[1]
        ref[4,1,1]=(-0.489219146209)*x_ref[0]**o+(-0.145758299279)*x_ref[0]+(0.565938821575)*x_ref[1]**o+(-0.98703168748)*x_ref[1]
        ref[5,0,0]=(0.0839742626164)*x_ref[0]**o+(0.195476369019)*x_ref[0]+(-0.856490669921)*x_ref[1]**o+(-0.471313686326)*x_ref[1]
        ref[5,0,1]=(-0.612045283736)*x_ref[0]**o+(0.9330978976)*x_ref[0]+(0.554623000698)*x_ref[1]**o+(-0.0447734703966)*x_ref[1]
        ref[5,1,0]=(0.137284580652)*x_ref[0]**o+(-0.576384951707)*x_ref[0]+(-0.18207486832)*x_ref[1]**o+(0.581595310151)*x_ref[1]
        ref[5,1,1]=(0.0169341997249)*x_ref[0]**o+(0.0681222507261)*x_ref[0]+(0.0252651671358)*x_ref[1]**o+(-0.884122676141)*x_ref[1]
      else:
        s[0,0,0]=(0.692580506423)*x[0]**o+(-0.739196213794)*x[0]+(-0.754727396858)*x[1]**o+(0.85754978429)*x[1]+(0.320776017021)*x[2]**o+(0.496406747735)*x[2]
        s[0,0,1]=(-0.368178763519)*x[0]**o+(0.443073964944)*x[0]+(-0.441300717299)*x[1]**o+(-0.642365449833)*x[1]+(-0.411121382093)*x[2]**o+(0.298351368421)*x[2]
        s[0,1,0]=(0.124028838378)*x[0]**o+(0.364040911128)*x[0]+(-0.816006030587)*x[1]**o+(-0.342165428091)*x[1]+(-0.565296255652)*x[2]**o+(0.10324169783)*x[2]
        s[0,1,1]=(-0.310115517857)*x[0]**o+(0.408326426387)*x[0]+(0.522912752025)*x[1]**o+(0.637782282686)*x[1]+(-0.386389984613)*x[2]**o+(0.566324706521)*x[2]
        s[1,0,0]=(-0.154331234746)*x[0]**o+(-0.436808593217)*x[0]+(0.00168591908065)*x[1]**o+(-0.786939679968)*x[1]+(-0.914492955074)*x[2]**o+(0.268713986856)*x[2]
        s[1,0,1]=(-0.583647316282)*x[0]**o+(0.893582962788)*x[0]+(0.764771722881)*x[1]**o+(-0.358865375935)*x[1]+(-0.157963402459)*x[2]**o+(-0.606181853567)*x[2]
        s[1,1,0]=(-0.655042363999)*x[0]**o+(0.746261777599)*x[0]+(-0.263881826688)*x[1]**o+(0.996006722883)*x[1]+(0.363648087969)*x[2]**o+(-0.448676062094)*x[2]
        s[1,1,1]=(0.123310902924)*x[0]**o+(0.18934896301)*x[0]+(0.520444019722)*x[1]**o+(0.163046210167)*x[1]+(0.191271174203)*x[2]**o+(-0.603399653729)*x[2]
        s[2,0,0]=(0.556843419598)*x[0]**o+(0.352145721987)*x[0]+(0.311744047933)*x[1]**o+(0.393374763098)*x[1]+(-0.411522712826)*x[2]**o+(-0.857749946733)*x[2]
        s[2,0,1]=(0.716861034223)*x[0]**o+(0.574178184871)*x[0]+(0.0537394109557)*x[1]**o+(0.468155056849)*x[1]+(0.579460976425)*x[2]**o+(0.699492008017)*x[2]
        s[2,1,0]=(-0.93398841558)*x[0]**o+(-0.697231862042)*x[0]+(0.109026477391)*x[1]**o+(0.634335059223)*x[1]+(-0.960274632681)*x[2]**o+(0.111578852692)*x[2]
        s[2,1,1]=(-0.183177179838)*x[0]**o+(0.507798545721)*x[0]+(0.762549005034)*x[1]**o+(0.135498023818)*x[1]+(-0.766065132764)*x[2]**o+(-0.622877452314)*x[2]
        s[3,0,0]=(0.873113007323)*x[0]**o+(-0.140853976021)*x[0]+(0.784413539889)*x[1]**o+(-0.488650797745)*x[1]+(0.452977221701)*x[2]**o+(0.913380695175)*x[2]
        s[3,0,1]=(0.282641502309)*x[0]**o+(-0.340827819728)*x[0]+(-0.54633943683)*x[1]**o+(-0.121403331732)*x[1]+(-0.915298514083)*x[2]**o+(-0.226945838638)*x[2]
        s[3,1,0]=(0.697235623378)*x[0]**o+(-0.124366296878)*x[0]+(-0.402714117894)*x[1]**o+(0.847633014733)*x[1]+(-0.112276438729)*x[2]**o+(-0.563344547848)*x[2]
        s[3,1,1]=(0.029600044738)*x[0]**o+(0.410269123343)*x[0]+(-0.92058774924)*x[1]**o+(-0.764026992207)*x[1]+(0.941619693688)*x[2]**o+(0.59508687872)*x[2]
        s[4,0,0]=(-0.782484223278)*x[0]**o+(-0.463642056406)*x[0]+(0.693714378966)*x[1]**o+(-0.409388955972)*x[1]+(-0.684017609615)*x[2]**o+(0.212037484357)*x[2]
        s[4,0,1]=(-0.507499040314)*x[0]**o+(0.628756911098)*x[0]+(0.582941469574)*x[1]**o+(-0.907597519745)*x[1]+(-0.674162640518)*x[2]**o+(0.343602419355)*x[2]
        s[4,1,0]=(0.718884212471)*x[0]**o+(-0.773542451968)*x[0]+(-0.558458416647)*x[1]**o+(0.246124177952)*x[1]+(0.301868653091)*x[2]**o+(-0.747134856013)*x[2]
        s[4,1,1]=(-0.826106273057)*x[0]**o+(0.810523783505)*x[0]+(-0.654586275299)*x[1]**o+(-0.616385894964)*x[1]+(0.254151903925)*x[2]**o+(0.890878608639)*x[2]
        s[5,0,0]=(0.674499531922)*x[0]**o+(-0.780390926945)*x[0]+(0.950129766744)*x[1]**o+(-0.60369977837)*x[1]+(-0.012121074813)*x[2]**o+(0.281422122495)*x[2]
        s[5,0,1]=(0.834522181864)*x[0]**o+(0.594872843083)*x[0]+(-0.921495736486)*x[1]**o+(0.873833748135)*x[1]+(-0.683068734294)*x[2]**o+(-0.234163027333)*x[2]
        s[5,1,0]=(-0.762343975994)*x[0]**o+(0.328522804445)*x[0]+(-0.684121978071)*x[1]**o+(0.240663011598)*x[1]+(0.781441947029)*x[2]**o+(0.63503560345)*x[2]
        s[5,1,1]=(-0.967543985688)*x[0]**o+(-0.141323317442)*x[0]+(-0.306422683038)*x[1]**o+(-0.935081886909)*x[1]+(-0.142084729038)*x[2]**o+(0.668337592761)*x[2]
        ref[0,0,0]=(0.692580506423)*x_ref[0]**o+(-0.739196213794)*x_ref[0]+(-0.754727396858)*x_ref[1]**o+(0.85754978429)*x_ref[1]+(0.320776017021)*x_ref[2]**o+(0.496406747735)*x_ref[2]
        ref[0,0,1]=(-0.368178763519)*x_ref[0]**o+(0.443073964944)*x_ref[0]+(-0.441300717299)*x_ref[1]**o+(-0.642365449833)*x_ref[1]+(-0.411121382093)*x_ref[2]**o+(0.298351368421)*x_ref[2]
        ref[0,1,0]=(0.124028838378)*x_ref[0]**o+(0.364040911128)*x_ref[0]+(-0.816006030587)*x_ref[1]**o+(-0.342165428091)*x_ref[1]+(-0.565296255652)*x_ref[2]**o+(0.10324169783)*x_ref[2]
        ref[0,1,1]=(-0.310115517857)*x_ref[0]**o+(0.408326426387)*x_ref[0]+(0.522912752025)*x_ref[1]**o+(0.637782282686)*x_ref[1]+(-0.386389984613)*x_ref[2]**o+(0.566324706521)*x_ref[2]
        ref[1,0,0]=(-0.154331234746)*x_ref[0]**o+(-0.436808593217)*x_ref[0]+(0.00168591908065)*x_ref[1]**o+(-0.786939679968)*x_ref[1]+(-0.914492955074)*x_ref[2]**o+(0.268713986856)*x_ref[2]
        ref[1,0,1]=(-0.583647316282)*x_ref[0]**o+(0.893582962788)*x_ref[0]+(0.764771722881)*x_ref[1]**o+(-0.358865375935)*x_ref[1]+(-0.157963402459)*x_ref[2]**o+(-0.606181853567)*x_ref[2]
        ref[1,1,0]=(-0.655042363999)*x_ref[0]**o+(0.746261777599)*x_ref[0]+(-0.263881826688)*x_ref[1]**o+(0.996006722883)*x_ref[1]+(0.363648087969)*x_ref[2]**o+(-0.448676062094)*x_ref[2]
        ref[1,1,1]=(0.123310902924)*x_ref[0]**o+(0.18934896301)*x_ref[0]+(0.520444019722)*x_ref[1]**o+(0.163046210167)*x_ref[1]+(0.191271174203)*x_ref[2]**o+(-0.603399653729)*x_ref[2]
        ref[2,0,0]=(0.556843419598)*x_ref[0]**o+(0.352145721987)*x_ref[0]+(0.311744047933)*x_ref[1]**o+(0.393374763098)*x_ref[1]+(-0.411522712826)*x_ref[2]**o+(-0.857749946733)*x_ref[2]
        ref[2,0,1]=(0.716861034223)*x_ref[0]**o+(0.574178184871)*x_ref[0]+(0.0537394109557)*x_ref[1]**o+(0.468155056849)*x_ref[1]+(0.579460976425)*x_ref[2]**o+(0.699492008017)*x_ref[2]
        ref[2,1,0]=(-0.93398841558)*x_ref[0]**o+(-0.697231862042)*x_ref[0]+(0.109026477391)*x_ref[1]**o+(0.634335059223)*x_ref[1]+(-0.960274632681)*x_ref[2]**o+(0.111578852692)*x_ref[2]
        ref[2,1,1]=(-0.183177179838)*x_ref[0]**o+(0.507798545721)*x_ref[0]+(0.762549005034)*x_ref[1]**o+(0.135498023818)*x_ref[1]+(-0.766065132764)*x_ref[2]**o+(-0.622877452314)*x_ref[2]
        ref[3,0,0]=(0.873113007323)*x_ref[0]**o+(-0.140853976021)*x_ref[0]+(0.784413539889)*x_ref[1]**o+(-0.488650797745)*x_ref[1]+(0.452977221701)*x_ref[2]**o+(0.913380695175)*x_ref[2]
        ref[3,0,1]=(0.282641502309)*x_ref[0]**o+(-0.340827819728)*x_ref[0]+(-0.54633943683)*x_ref[1]**o+(-0.121403331732)*x_ref[1]+(-0.915298514083)*x_ref[2]**o+(-0.226945838638)*x_ref[2]
        ref[3,1,0]=(0.697235623378)*x_ref[0]**o+(-0.124366296878)*x_ref[0]+(-0.402714117894)*x_ref[1]**o+(0.847633014733)*x_ref[1]+(-0.112276438729)*x_ref[2]**o+(-0.563344547848)*x_ref[2]
        ref[3,1,1]=(0.029600044738)*x_ref[0]**o+(0.410269123343)*x_ref[0]+(-0.92058774924)*x_ref[1]**o+(-0.764026992207)*x_ref[1]+(0.941619693688)*x_ref[2]**o+(0.59508687872)*x_ref[2]
        ref[4,0,0]=(-0.782484223278)*x_ref[0]**o+(-0.463642056406)*x_ref[0]+(0.693714378966)*x_ref[1]**o+(-0.409388955972)*x_ref[1]+(-0.684017609615)*x_ref[2]**o+(0.212037484357)*x_ref[2]
        ref[4,0,1]=(-0.507499040314)*x_ref[0]**o+(0.628756911098)*x_ref[0]+(0.582941469574)*x_ref[1]**o+(-0.907597519745)*x_ref[1]+(-0.674162640518)*x_ref[2]**o+(0.343602419355)*x_ref[2]
        ref[4,1,0]=(0.718884212471)*x_ref[0]**o+(-0.773542451968)*x_ref[0]+(-0.558458416647)*x_ref[1]**o+(0.246124177952)*x_ref[1]+(0.301868653091)*x_ref[2]**o+(-0.747134856013)*x_ref[2]
        ref[4,1,1]=(-0.826106273057)*x_ref[0]**o+(0.810523783505)*x_ref[0]+(-0.654586275299)*x_ref[1]**o+(-0.616385894964)*x_ref[1]+(0.254151903925)*x_ref[2]**o+(0.890878608639)*x_ref[2]
        ref[5,0,0]=(0.674499531922)*x_ref[0]**o+(-0.780390926945)*x_ref[0]+(0.950129766744)*x_ref[1]**o+(-0.60369977837)*x_ref[1]+(-0.012121074813)*x_ref[2]**o+(0.281422122495)*x_ref[2]
        ref[5,0,1]=(0.834522181864)*x_ref[0]**o+(0.594872843083)*x_ref[0]+(-0.921495736486)*x_ref[1]**o+(0.873833748135)*x_ref[1]+(-0.683068734294)*x_ref[2]**o+(-0.234163027333)*x_ref[2]
        ref[5,1,0]=(-0.762343975994)*x_ref[0]**o+(0.328522804445)*x_ref[0]+(-0.684121978071)*x_ref[1]**o+(0.240663011598)*x_ref[1]+(0.781441947029)*x_ref[2]**o+(0.63503560345)*x_ref[2]
        ref[5,1,1]=(-0.967543985688)*x_ref[0]**o+(-0.141323317442)*x_ref[0]+(-0.306422683038)*x_ref[1]**o+(-0.935081886909)*x_ref[1]+(-0.142084729038)*x_ref[2]**o+(0.668337592761)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromSymbol_Solution_rank4(self):
      """
      tests interpolate for rank 4 Symbol onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from Solution onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5, 3, 2),dim=dim)
      s=Data(0,(4, 5, 3, 2),w)
      ref=Data(0,(4, 5, 3, 2),w_ref)
      if dim==2:
        s[0,0,0,0]=(-0.547754951171)*x[0]**o+(0.356610809044)*x[0]+(0.452044983539)*x[1]**o+(-0.821341147164)*x[1]
        s[0,0,0,1]=(0.509267661384)*x[0]**o+(0.667069615676)*x[0]+(0.344083346522)*x[1]**o+(-0.281070233626)*x[1]
        s[0,0,1,0]=(0.910153339127)*x[0]**o+(-0.134026992189)*x[0]+(0.283156493736)*x[1]**o+(-0.236635156141)*x[1]
        s[0,0,1,1]=(-0.486341299233)*x[0]**o+(0.969619867206)*x[0]+(0.145365694427)*x[1]**o+(0.395899354311)*x[1]
        s[0,0,2,0]=(-0.683524112594)*x[0]**o+(-0.191053076108)*x[0]+(-0.156617335102)*x[1]**o+(-0.908072509242)*x[1]
        s[0,0,2,1]=(-0.329288530406)*x[0]**o+(0.861655757717)*x[0]+(-0.423717545177)*x[1]**o+(0.357896568135)*x[1]
        s[0,1,0,0]=(0.454682074738)*x[0]**o+(0.242640789048)*x[0]+(-0.465273320791)*x[1]**o+(-0.523776617273)*x[1]
        s[0,1,0,1]=(0.299812377446)*x[0]**o+(0.258743885127)*x[0]+(-0.545559145397)*x[1]**o+(-0.703973878214)*x[1]
        s[0,1,1,0]=(-0.240052819623)*x[0]**o+(-0.648241629125)*x[0]+(0.550226821237)*x[1]**o+(0.586254427297)*x[1]
        s[0,1,1,1]=(0.232594672088)*x[0]**o+(-0.082742546667)*x[0]+(0.695512543415)*x[1]**o+(0.695609364591)*x[1]
        s[0,1,2,0]=(-0.337129430207)*x[0]**o+(-0.0996108162262)*x[0]+(0.520279135444)*x[1]**o+(-0.0419680101154)*x[1]
        s[0,1,2,1]=(0.107050704386)*x[0]**o+(-0.0108163145609)*x[0]+(-0.555986163214)*x[1]**o+(-0.264372389969)*x[1]
        s[0,2,0,0]=(0.558185714977)*x[0]**o+(0.718930754697)*x[0]+(0.661967682247)*x[1]**o+(0.968289790196)*x[1]
        s[0,2,0,1]=(-0.885308413126)*x[0]**o+(-0.000196750280455)*x[0]+(0.162337198564)*x[1]**o+(-0.372085076741)*x[1]
        s[0,2,1,0]=(0.732654846787)*x[0]**o+(0.490322343819)*x[0]+(0.253518225543)*x[1]**o+(0.755915810471)*x[1]
        s[0,2,1,1]=(0.686091929471)*x[0]**o+(0.968804203166)*x[0]+(-0.874263118503)*x[1]**o+(0.792892684533)*x[1]
        s[0,2,2,0]=(-0.538165611113)*x[0]**o+(0.435640729152)*x[0]+(0.53052383117)*x[1]**o+(-0.152917182113)*x[1]
        s[0,2,2,1]=(0.959376322991)*x[0]**o+(0.295428309997)*x[0]+(0.774755280624)*x[1]**o+(0.699112429694)*x[1]
        s[0,3,0,0]=(-0.879327491632)*x[0]**o+(-0.210915363483)*x[0]+(0.677720758543)*x[1]**o+(-0.156843338639)*x[1]
        s[0,3,0,1]=(-0.877749774331)*x[0]**o+(-0.0590688859445)*x[0]+(0.372427864404)*x[1]**o+(0.865604489942)*x[1]
        s[0,3,1,0]=(-0.479636139873)*x[0]**o+(-0.2440095302)*x[0]+(0.291946443647)*x[1]**o+(-0.446912115557)*x[1]
        s[0,3,1,1]=(-0.666968887345)*x[0]**o+(-0.530382039479)*x[0]+(-0.954360470015)*x[1]**o+(0.827870552445)*x[1]
        s[0,3,2,0]=(-0.697008931895)*x[0]**o+(-0.506024200287)*x[0]+(-0.284218723212)*x[1]**o+(-0.12154006973)*x[1]
        s[0,3,2,1]=(0.182079912323)*x[0]**o+(0.128753570425)*x[0]+(-0.610041297244)*x[1]**o+(0.167312633663)*x[1]
        s[0,4,0,0]=(0.905082824851)*x[0]**o+(-0.850946080497)*x[0]+(-0.234019076337)*x[1]**o+(-0.746004283581)*x[1]
        s[0,4,0,1]=(0.252751160265)*x[0]**o+(0.783081625822)*x[0]+(-0.18400975082)*x[1]**o+(0.729096084753)*x[1]
        s[0,4,1,0]=(-0.646745752044)*x[0]**o+(-0.95019218163)*x[0]+(-0.739473363344)*x[1]**o+(0.53887852033)*x[1]
        s[0,4,1,1]=(0.190710071596)*x[0]**o+(0.363498800572)*x[0]+(-0.166712144832)*x[1]**o+(0.383842690034)*x[1]
        s[0,4,2,0]=(-0.252312977399)*x[0]**o+(-0.181431558994)*x[0]+(-0.496223516025)*x[1]**o+(0.262053197788)*x[1]
        s[0,4,2,1]=(0.740944095813)*x[0]**o+(0.350982435679)*x[0]+(0.540773567147)*x[1]**o+(0.154742779719)*x[1]
        s[1,0,0,0]=(-0.204518679107)*x[0]**o+(0.224833545775)*x[0]+(0.701576397345)*x[1]**o+(0.955758264121)*x[1]
        s[1,0,0,1]=(0.98822877512)*x[0]**o+(-0.252532184922)*x[0]+(-0.999543053077)*x[1]**o+(0.20420747276)*x[1]
        s[1,0,1,0]=(-0.435694974363)*x[0]**o+(0.106116746803)*x[0]+(-0.614925089687)*x[1]**o+(-0.433482250787)*x[1]
        s[1,0,1,1]=(-0.967905352631)*x[0]**o+(0.733630963983)*x[0]+(0.561497813177)*x[1]**o+(-0.254199898967)*x[1]
        s[1,0,2,0]=(0.192509982098)*x[0]**o+(0.30844145669)*x[0]+(-0.537990978303)*x[1]**o+(-0.971691014587)*x[1]
        s[1,0,2,1]=(-0.642601469866)*x[0]**o+(0.828976360334)*x[0]+(-0.166247518535)*x[1]**o+(-0.394356610568)*x[1]
        s[1,1,0,0]=(-0.190023582376)*x[0]**o+(0.00308098708653)*x[0]+(0.616327311292)*x[1]**o+(0.50233410734)*x[1]
        s[1,1,0,1]=(-0.421733782097)*x[0]**o+(-0.81738567824)*x[0]+(0.0102253671566)*x[1]**o+(0.977732802241)*x[1]
        s[1,1,1,0]=(-0.932299813732)*x[0]**o+(-0.570728228193)*x[0]+(0.906998960465)*x[1]**o+(0.276967285338)*x[1]
        s[1,1,1,1]=(0.361866174533)*x[0]**o+(-0.948833321247)*x[0]+(0.0267950613442)*x[1]**o+(0.305540794357)*x[1]
        s[1,1,2,0]=(-0.107562715773)*x[0]**o+(0.884548510332)*x[0]+(-0.641236013271)*x[1]**o+(0.835011288403)*x[1]
        s[1,1,2,1]=(-0.447112184269)*x[0]**o+(0.68668033563)*x[0]+(-0.643123622354)*x[1]**o+(-0.373398061272)*x[1]
        s[1,2,0,0]=(0.566476213386)*x[0]**o+(-0.660978862942)*x[0]+(-0.250064101238)*x[1]**o+(0.0617654537116)*x[1]
        s[1,2,0,1]=(-0.0220375595157)*x[0]**o+(-0.664682506405)*x[0]+(-0.810443003238)*x[1]**o+(-0.704430911362)*x[1]
        s[1,2,1,0]=(-0.66837513872)*x[0]**o+(0.518574783899)*x[0]+(0.456254450127)*x[1]**o+(-0.0264418287977)*x[1]
        s[1,2,1,1]=(0.518371474153)*x[0]**o+(0.305863788515)*x[0]+(0.641786832956)*x[1]**o+(0.957403546249)*x[1]
        s[1,2,2,0]=(-0.702485305168)*x[0]**o+(0.810129172394)*x[0]+(0.481384977464)*x[1]**o+(-0.552283897592)*x[1]
        s[1,2,2,1]=(-0.220714257324)*x[0]**o+(0.658520871538)*x[0]+(-0.156197468389)*x[1]**o+(0.406317861261)*x[1]
        s[1,3,0,0]=(-0.833421663125)*x[0]**o+(-0.23885966657)*x[0]+(-0.0853110945734)*x[1]**o+(0.134587357668)*x[1]
        s[1,3,0,1]=(0.983003072543)*x[0]**o+(-0.444298839588)*x[0]+(-0.555821641978)*x[1]**o+(-0.934198804069)*x[1]
        s[1,3,1,0]=(-0.321902025108)*x[0]**o+(0.373296267434)*x[0]+(-0.963121149124)*x[1]**o+(0.0147111484841)*x[1]
        s[1,3,1,1]=(-0.686361701679)*x[0]**o+(0.322135914568)*x[0]+(0.0577830897835)*x[1]**o+(0.577239387766)*x[1]
        s[1,3,2,0]=(0.722944586976)*x[0]**o+(0.789741801778)*x[0]+(0.175208515485)*x[1]**o+(-0.71289312398)*x[1]
        s[1,3,2,1]=(-0.280608530342)*x[0]**o+(0.0860239950894)*x[0]+(0.695636585879)*x[1]**o+(-0.960594121577)*x[1]
        s[1,4,0,0]=(-0.852503330892)*x[0]**o+(-0.602234774377)*x[0]+(0.198091265715)*x[1]**o+(0.777503337768)*x[1]
        s[1,4,0,1]=(-0.177137751594)*x[0]**o+(-0.490189065267)*x[0]+(-0.8576459288)*x[1]**o+(-0.074100447778)*x[1]
        s[1,4,1,0]=(-0.334129920953)*x[0]**o+(-0.337245487943)*x[0]+(-0.839457764245)*x[1]**o+(-0.58152129024)*x[1]
        s[1,4,1,1]=(0.0937571716669)*x[0]**o+(0.592103688833)*x[0]+(0.15448940004)*x[1]**o+(-0.203463329576)*x[1]
        s[1,4,2,0]=(0.139741892415)*x[0]**o+(0.611539882432)*x[0]+(-0.407889172421)*x[1]**o+(0.891469847051)*x[1]
        s[1,4,2,1]=(0.95147411312)*x[0]**o+(0.0998931787148)*x[0]+(0.466902306753)*x[1]**o+(-0.718472606772)*x[1]
        s[2,0,0,0]=(-0.130008770209)*x[0]**o+(0.809915931169)*x[0]+(-0.732276845472)*x[1]**o+(-0.192903892405)*x[1]
        s[2,0,0,1]=(-0.0119712505975)*x[0]**o+(0.656694120973)*x[0]+(0.6862601083)*x[1]**o+(-0.528666986412)*x[1]
        s[2,0,1,0]=(0.28678233655)*x[0]**o+(-0.37339891868)*x[0]+(-0.946168471623)*x[1]**o+(0.762233026568)*x[1]
        s[2,0,1,1]=(0.778843713994)*x[0]**o+(0.817766330368)*x[0]+(-0.300918496467)*x[1]**o+(0.442988971838)*x[1]
        s[2,0,2,0]=(0.207449688136)*x[0]**o+(0.10275570242)*x[0]+(-0.520466147913)*x[1]**o+(0.33346010447)*x[1]
        s[2,0,2,1]=(0.709221342424)*x[0]**o+(0.159144243448)*x[0]+(0.685824812808)*x[1]**o+(0.50901941527)*x[1]
        s[2,1,0,0]=(0.0244924098009)*x[0]**o+(-0.770942809125)*x[0]+(0.372626872676)*x[1]**o+(-0.714320503926)*x[1]
        s[2,1,0,1]=(-0.105052119318)*x[0]**o+(0.573319234378)*x[0]+(0.0995762579843)*x[1]**o+(0.579088439928)*x[1]
        s[2,1,1,0]=(-0.572394543041)*x[0]**o+(0.1606374212)*x[0]+(-0.801731532404)*x[1]**o+(-0.644110219424)*x[1]
        s[2,1,1,1]=(0.914053075068)*x[0]**o+(-0.230478556116)*x[0]+(-0.00343924084847)*x[1]**o+(-0.53144169108)*x[1]
        s[2,1,2,0]=(-0.743362569446)*x[0]**o+(-0.0428045574774)*x[0]+(-0.503190273025)*x[1]**o+(0.831658185042)*x[1]
        s[2,1,2,1]=(0.176721891893)*x[0]**o+(-0.460950049697)*x[0]+(0.45677887739)*x[1]**o+(-0.853283022367)*x[1]
        s[2,2,0,0]=(-0.585337031786)*x[0]**o+(0.901183590413)*x[0]+(-0.746305431471)*x[1]**o+(0.0439043730531)*x[1]
        s[2,2,0,1]=(0.594349646931)*x[0]**o+(-0.431546660621)*x[0]+(-0.962096864047)*x[1]**o+(0.955112341273)*x[1]
        s[2,2,1,0]=(-0.857877499011)*x[0]**o+(0.803223671127)*x[0]+(-0.995806495668)*x[1]**o+(-0.267707433165)*x[1]
        s[2,2,1,1]=(0.968406110131)*x[0]**o+(-0.0208982523366)*x[0]+(-0.000406237500393)*x[1]**o+(-0.334429552696)*x[1]
        s[2,2,2,0]=(-0.960828307746)*x[0]**o+(0.235004476043)*x[0]+(0.0456041210872)*x[1]**o+(0.323144610527)*x[1]
        s[2,2,2,1]=(0.538691072674)*x[0]**o+(-0.737490055104)*x[0]+(-0.96703538134)*x[1]**o+(0.727057872325)*x[1]
        s[2,3,0,0]=(0.475319703318)*x[0]**o+(0.863857783409)*x[0]+(0.0891089574388)*x[1]**o+(0.121619153024)*x[1]
        s[2,3,0,1]=(0.68881433038)*x[0]**o+(-0.812908863657)*x[0]+(0.687129220133)*x[1]**o+(0.580728857382)*x[1]
        s[2,3,1,0]=(0.798247442811)*x[0]**o+(0.600332572583)*x[0]+(0.629676907787)*x[1]**o+(0.0214779823831)*x[1]
        s[2,3,1,1]=(0.446783692926)*x[0]**o+(-0.579857496036)*x[0]+(0.669355002891)*x[1]**o+(-0.104502660981)*x[1]
        s[2,3,2,0]=(0.874126641785)*x[0]**o+(0.244866909003)*x[0]+(-0.0181287421091)*x[1]**o+(-0.728707148938)*x[1]
        s[2,3,2,1]=(-0.716069945597)*x[0]**o+(0.632201538129)*x[0]+(-0.538839892852)*x[1]**o+(-0.886572197711)*x[1]
        s[2,4,0,0]=(-0.422214071919)*x[0]**o+(0.276563507343)*x[0]+(0.481338034453)*x[1]**o+(-0.972870285915)*x[1]
        s[2,4,0,1]=(0.740441652804)*x[0]**o+(0.447067551217)*x[0]+(-0.310919183692)*x[1]**o+(0.636513538883)*x[1]
        s[2,4,1,0]=(-0.599111504454)*x[0]**o+(-0.250476629717)*x[0]+(-0.658311629855)*x[1]**o+(0.492577876045)*x[1]
        s[2,4,1,1]=(0.107735805283)*x[0]**o+(0.208970264385)*x[0]+(-0.147382019475)*x[1]**o+(-0.406553806193)*x[1]
        s[2,4,2,0]=(0.925310626624)*x[0]**o+(0.0400900248412)*x[0]+(-0.431657024276)*x[1]**o+(-0.153834430713)*x[1]
        s[2,4,2,1]=(-0.713453580612)*x[0]**o+(0.391780861442)*x[0]+(-0.940123468631)*x[1]**o+(-0.937804820594)*x[1]
        s[3,0,0,0]=(-0.272038820171)*x[0]**o+(0.0722677637085)*x[0]+(-0.81552430022)*x[1]**o+(-0.605100343524)*x[1]
        s[3,0,0,1]=(-0.869234792389)*x[0]**o+(0.133360720775)*x[0]+(0.334759295473)*x[1]**o+(0.891704395665)*x[1]
        s[3,0,1,0]=(0.283196831063)*x[0]**o+(-0.433641717885)*x[0]+(0.821493597367)*x[1]**o+(0.0757494621194)*x[1]
        s[3,0,1,1]=(-0.970851846713)*x[0]**o+(0.420332346058)*x[0]+(-0.128433226815)*x[1]**o+(0.355540827779)*x[1]
        s[3,0,2,0]=(-0.855684873323)*x[0]**o+(-0.896780938471)*x[0]+(-0.910323225744)*x[1]**o+(-0.648707265693)*x[1]
        s[3,0,2,1]=(0.224013589567)*x[0]**o+(-0.109904216055)*x[0]+(0.86374013156)*x[1]**o+(0.474691072866)*x[1]
        s[3,1,0,0]=(-0.497944848072)*x[0]**o+(0.146776775946)*x[0]+(0.526474245297)*x[1]**o+(-0.835737531991)*x[1]
        s[3,1,0,1]=(-0.421150013578)*x[0]**o+(-0.784402607666)*x[0]+(-0.0805148903284)*x[1]**o+(-0.52347503704)*x[1]
        s[3,1,1,0]=(0.671180085936)*x[0]**o+(-0.0708908978891)*x[0]+(0.80005635402)*x[1]**o+(0.250773812305)*x[1]
        s[3,1,1,1]=(-0.740517458065)*x[0]**o+(-0.164975700504)*x[0]+(-0.954776148663)*x[1]**o+(0.0662528804032)*x[1]
        s[3,1,2,0]=(-0.0487118351329)*x[0]**o+(0.0172491914156)*x[0]+(0.459297824842)*x[1]**o+(0.931441363698)*x[1]
        s[3,1,2,1]=(0.914541867519)*x[0]**o+(-0.421726383007)*x[0]+(0.932990305149)*x[1]**o+(0.00708080928476)*x[1]
        s[3,2,0,0]=(-0.268505299798)*x[0]**o+(-0.884105078203)*x[0]+(-0.343177738426)*x[1]**o+(0.925594916146)*x[1]
        s[3,2,0,1]=(0.792964128575)*x[0]**o+(0.45312934456)*x[0]+(-0.190272595857)*x[1]**o+(-0.714484743775)*x[1]
        s[3,2,1,0]=(-0.389811244023)*x[0]**o+(-0.230272427621)*x[0]+(-0.062642727789)*x[1]**o+(0.333179046361)*x[1]
        s[3,2,1,1]=(0.468980019323)*x[0]**o+(-0.213426322608)*x[0]+(0.806778675985)*x[1]**o+(-0.031962815429)*x[1]
        s[3,2,2,0]=(0.187153153145)*x[0]**o+(0.440459591208)*x[0]+(-0.766229604202)*x[1]**o+(0.630228576743)*x[1]
        s[3,2,2,1]=(-0.362635752315)*x[0]**o+(-0.356960434705)*x[0]+(-0.209248043764)*x[1]**o+(0.70828385566)*x[1]
        s[3,3,0,0]=(-0.680482417515)*x[0]**o+(-0.90459474138)*x[0]+(-0.568552053392)*x[1]**o+(0.394198411912)*x[1]
        s[3,3,0,1]=(-0.0932148057029)*x[0]**o+(0.618628605947)*x[0]+(0.0470711457335)*x[1]**o+(0.802136993857)*x[1]
        s[3,3,1,0]=(-0.442943896794)*x[0]**o+(0.57385228082)*x[0]+(0.916092493461)*x[1]**o+(-0.913579167434)*x[1]
        s[3,3,1,1]=(0.976285532615)*x[0]**o+(-0.327052093737)*x[0]+(0.731622827093)*x[1]**o+(0.198510813385)*x[1]
        s[3,3,2,0]=(-0.921649918257)*x[0]**o+(-0.954803772809)*x[0]+(0.64789210876)*x[1]**o+(-0.928730411729)*x[1]
        s[3,3,2,1]=(0.227719394295)*x[0]**o+(-0.512300073514)*x[0]+(-0.401469395128)*x[1]**o+(-0.229248271712)*x[1]
        s[3,4,0,0]=(0.655903201215)*x[0]**o+(0.285189680631)*x[0]+(-0.183566536361)*x[1]**o+(-0.18428580448)*x[1]
        s[3,4,0,1]=(-0.0282806177857)*x[0]**o+(-0.29856051731)*x[0]+(0.120325396195)*x[1]**o+(-0.29397064071)*x[1]
        s[3,4,1,0]=(-0.640500611083)*x[0]**o+(0.979817975078)*x[0]+(0.577156862636)*x[1]**o+(-0.00170003402221)*x[1]
        s[3,4,1,1]=(0.681055102015)*x[0]**o+(-0.656903522238)*x[0]+(0.470845699959)*x[1]**o+(-0.973850290239)*x[1]
        s[3,4,2,0]=(-0.941893654596)*x[0]**o+(0.96514461809)*x[0]+(0.822665841321)*x[1]**o+(-0.826621500352)*x[1]
        s[3,4,2,1]=(0.0808254384437)*x[0]**o+(0.191567292517)*x[0]+(-0.48838584117)*x[1]**o+(-0.472758040235)*x[1]
        ref[0,0,0,0]=(-0.547754951171)*x_ref[0]**o+(0.356610809044)*x_ref[0]+(0.452044983539)*x_ref[1]**o+(-0.821341147164)*x_ref[1]
        ref[0,0,0,1]=(0.509267661384)*x_ref[0]**o+(0.667069615676)*x_ref[0]+(0.344083346522)*x_ref[1]**o+(-0.281070233626)*x_ref[1]
        ref[0,0,1,0]=(0.910153339127)*x_ref[0]**o+(-0.134026992189)*x_ref[0]+(0.283156493736)*x_ref[1]**o+(-0.236635156141)*x_ref[1]
        ref[0,0,1,1]=(-0.486341299233)*x_ref[0]**o+(0.969619867206)*x_ref[0]+(0.145365694427)*x_ref[1]**o+(0.395899354311)*x_ref[1]
        ref[0,0,2,0]=(-0.683524112594)*x_ref[0]**o+(-0.191053076108)*x_ref[0]+(-0.156617335102)*x_ref[1]**o+(-0.908072509242)*x_ref[1]
        ref[0,0,2,1]=(-0.329288530406)*x_ref[0]**o+(0.861655757717)*x_ref[0]+(-0.423717545177)*x_ref[1]**o+(0.357896568135)*x_ref[1]
        ref[0,1,0,0]=(0.454682074738)*x_ref[0]**o+(0.242640789048)*x_ref[0]+(-0.465273320791)*x_ref[1]**o+(-0.523776617273)*x_ref[1]
        ref[0,1,0,1]=(0.299812377446)*x_ref[0]**o+(0.258743885127)*x_ref[0]+(-0.545559145397)*x_ref[1]**o+(-0.703973878214)*x_ref[1]
        ref[0,1,1,0]=(-0.240052819623)*x_ref[0]**o+(-0.648241629125)*x_ref[0]+(0.550226821237)*x_ref[1]**o+(0.586254427297)*x_ref[1]
        ref[0,1,1,1]=(0.232594672088)*x_ref[0]**o+(-0.082742546667)*x_ref[0]+(0.695512543415)*x_ref[1]**o+(0.695609364591)*x_ref[1]
        ref[0,1,2,0]=(-0.337129430207)*x_ref[0]**o+(-0.0996108162262)*x_ref[0]+(0.520279135444)*x_ref[1]**o+(-0.0419680101154)*x_ref[1]
        ref[0,1,2,1]=(0.107050704386)*x_ref[0]**o+(-0.0108163145609)*x_ref[0]+(-0.555986163214)*x_ref[1]**o+(-0.264372389969)*x_ref[1]
        ref[0,2,0,0]=(0.558185714977)*x_ref[0]**o+(0.718930754697)*x_ref[0]+(0.661967682247)*x_ref[1]**o+(0.968289790196)*x_ref[1]
        ref[0,2,0,1]=(-0.885308413126)*x_ref[0]**o+(-0.000196750280455)*x_ref[0]+(0.162337198564)*x_ref[1]**o+(-0.372085076741)*x_ref[1]
        ref[0,2,1,0]=(0.732654846787)*x_ref[0]**o+(0.490322343819)*x_ref[0]+(0.253518225543)*x_ref[1]**o+(0.755915810471)*x_ref[1]
        ref[0,2,1,1]=(0.686091929471)*x_ref[0]**o+(0.968804203166)*x_ref[0]+(-0.874263118503)*x_ref[1]**o+(0.792892684533)*x_ref[1]
        ref[0,2,2,0]=(-0.538165611113)*x_ref[0]**o+(0.435640729152)*x_ref[0]+(0.53052383117)*x_ref[1]**o+(-0.152917182113)*x_ref[1]
        ref[0,2,2,1]=(0.959376322991)*x_ref[0]**o+(0.295428309997)*x_ref[0]+(0.774755280624)*x_ref[1]**o+(0.699112429694)*x_ref[1]
        ref[0,3,0,0]=(-0.879327491632)*x_ref[0]**o+(-0.210915363483)*x_ref[0]+(0.677720758543)*x_ref[1]**o+(-0.156843338639)*x_ref[1]
        ref[0,3,0,1]=(-0.877749774331)*x_ref[0]**o+(-0.0590688859445)*x_ref[0]+(0.372427864404)*x_ref[1]**o+(0.865604489942)*x_ref[1]
        ref[0,3,1,0]=(-0.479636139873)*x_ref[0]**o+(-0.2440095302)*x_ref[0]+(0.291946443647)*x_ref[1]**o+(-0.446912115557)*x_ref[1]
        ref[0,3,1,1]=(-0.666968887345)*x_ref[0]**o+(-0.530382039479)*x_ref[0]+(-0.954360470015)*x_ref[1]**o+(0.827870552445)*x_ref[1]
        ref[0,3,2,0]=(-0.697008931895)*x_ref[0]**o+(-0.506024200287)*x_ref[0]+(-0.284218723212)*x_ref[1]**o+(-0.12154006973)*x_ref[1]
        ref[0,3,2,1]=(0.182079912323)*x_ref[0]**o+(0.128753570425)*x_ref[0]+(-0.610041297244)*x_ref[1]**o+(0.167312633663)*x_ref[1]
        ref[0,4,0,0]=(0.905082824851)*x_ref[0]**o+(-0.850946080497)*x_ref[0]+(-0.234019076337)*x_ref[1]**o+(-0.746004283581)*x_ref[1]
        ref[0,4,0,1]=(0.252751160265)*x_ref[0]**o+(0.783081625822)*x_ref[0]+(-0.18400975082)*x_ref[1]**o+(0.729096084753)*x_ref[1]
        ref[0,4,1,0]=(-0.646745752044)*x_ref[0]**o+(-0.95019218163)*x_ref[0]+(-0.739473363344)*x_ref[1]**o+(0.53887852033)*x_ref[1]
        ref[0,4,1,1]=(0.190710071596)*x_ref[0]**o+(0.363498800572)*x_ref[0]+(-0.166712144832)*x_ref[1]**o+(0.383842690034)*x_ref[1]
        ref[0,4,2,0]=(-0.252312977399)*x_ref[0]**o+(-0.181431558994)*x_ref[0]+(-0.496223516025)*x_ref[1]**o+(0.262053197788)*x_ref[1]
        ref[0,4,2,1]=(0.740944095813)*x_ref[0]**o+(0.350982435679)*x_ref[0]+(0.540773567147)*x_ref[1]**o+(0.154742779719)*x_ref[1]
        ref[1,0,0,0]=(-0.204518679107)*x_ref[0]**o+(0.224833545775)*x_ref[0]+(0.701576397345)*x_ref[1]**o+(0.955758264121)*x_ref[1]
        ref[1,0,0,1]=(0.98822877512)*x_ref[0]**o+(-0.252532184922)*x_ref[0]+(-0.999543053077)*x_ref[1]**o+(0.20420747276)*x_ref[1]
        ref[1,0,1,0]=(-0.435694974363)*x_ref[0]**o+(0.106116746803)*x_ref[0]+(-0.614925089687)*x_ref[1]**o+(-0.433482250787)*x_ref[1]
        ref[1,0,1,1]=(-0.967905352631)*x_ref[0]**o+(0.733630963983)*x_ref[0]+(0.561497813177)*x_ref[1]**o+(-0.254199898967)*x_ref[1]
        ref[1,0,2,0]=(0.192509982098)*x_ref[0]**o+(0.30844145669)*x_ref[0]+(-0.537990978303)*x_ref[1]**o+(-0.971691014587)*x_ref[1]
        ref[1,0,2,1]=(-0.642601469866)*x_ref[0]**o+(0.828976360334)*x_ref[0]+(-0.166247518535)*x_ref[1]**o+(-0.394356610568)*x_ref[1]
        ref[1,1,0,0]=(-0.190023582376)*x_ref[0]**o+(0.00308098708653)*x_ref[0]+(0.616327311292)*x_ref[1]**o+(0.50233410734)*x_ref[1]
        ref[1,1,0,1]=(-0.421733782097)*x_ref[0]**o+(-0.81738567824)*x_ref[0]+(0.0102253671566)*x_ref[1]**o+(0.977732802241)*x_ref[1]
        ref[1,1,1,0]=(-0.932299813732)*x_ref[0]**o+(-0.570728228193)*x_ref[0]+(0.906998960465)*x_ref[1]**o+(0.276967285338)*x_ref[1]
        ref[1,1,1,1]=(0.361866174533)*x_ref[0]**o+(-0.948833321247)*x_ref[0]+(0.0267950613442)*x_ref[1]**o+(0.305540794357)*x_ref[1]
        ref[1,1,2,0]=(-0.107562715773)*x_ref[0]**o+(0.884548510332)*x_ref[0]+(-0.641236013271)*x_ref[1]**o+(0.835011288403)*x_ref[1]
        ref[1,1,2,1]=(-0.447112184269)*x_ref[0]**o+(0.68668033563)*x_ref[0]+(-0.643123622354)*x_ref[1]**o+(-0.373398061272)*x_ref[1]
        ref[1,2,0,0]=(0.566476213386)*x_ref[0]**o+(-0.660978862942)*x_ref[0]+(-0.250064101238)*x_ref[1]**o+(0.0617654537116)*x_ref[1]
        ref[1,2,0,1]=(-0.0220375595157)*x_ref[0]**o+(-0.664682506405)*x_ref[0]+(-0.810443003238)*x_ref[1]**o+(-0.704430911362)*x_ref[1]
        ref[1,2,1,0]=(-0.66837513872)*x_ref[0]**o+(0.518574783899)*x_ref[0]+(0.456254450127)*x_ref[1]**o+(-0.0264418287977)*x_ref[1]
        ref[1,2,1,1]=(0.518371474153)*x_ref[0]**o+(0.305863788515)*x_ref[0]+(0.641786832956)*x_ref[1]**o+(0.957403546249)*x_ref[1]
        ref[1,2,2,0]=(-0.702485305168)*x_ref[0]**o+(0.810129172394)*x_ref[0]+(0.481384977464)*x_ref[1]**o+(-0.552283897592)*x_ref[1]
        ref[1,2,2,1]=(-0.220714257324)*x_ref[0]**o+(0.658520871538)*x_ref[0]+(-0.156197468389)*x_ref[1]**o+(0.406317861261)*x_ref[1]
        ref[1,3,0,0]=(-0.833421663125)*x_ref[0]**o+(-0.23885966657)*x_ref[0]+(-0.0853110945734)*x_ref[1]**o+(0.134587357668)*x_ref[1]
        ref[1,3,0,1]=(0.983003072543)*x_ref[0]**o+(-0.444298839588)*x_ref[0]+(-0.555821641978)*x_ref[1]**o+(-0.934198804069)*x_ref[1]
        ref[1,3,1,0]=(-0.321902025108)*x_ref[0]**o+(0.373296267434)*x_ref[0]+(-0.963121149124)*x_ref[1]**o+(0.0147111484841)*x_ref[1]
        ref[1,3,1,1]=(-0.686361701679)*x_ref[0]**o+(0.322135914568)*x_ref[0]+(0.0577830897835)*x_ref[1]**o+(0.577239387766)*x_ref[1]
        ref[1,3,2,0]=(0.722944586976)*x_ref[0]**o+(0.789741801778)*x_ref[0]+(0.175208515485)*x_ref[1]**o+(-0.71289312398)*x_ref[1]
        ref[1,3,2,1]=(-0.280608530342)*x_ref[0]**o+(0.0860239950894)*x_ref[0]+(0.695636585879)*x_ref[1]**o+(-0.960594121577)*x_ref[1]
        ref[1,4,0,0]=(-0.852503330892)*x_ref[0]**o+(-0.602234774377)*x_ref[0]+(0.198091265715)*x_ref[1]**o+(0.777503337768)*x_ref[1]
        ref[1,4,0,1]=(-0.177137751594)*x_ref[0]**o+(-0.490189065267)*x_ref[0]+(-0.8576459288)*x_ref[1]**o+(-0.074100447778)*x_ref[1]
        ref[1,4,1,0]=(-0.334129920953)*x_ref[0]**o+(-0.337245487943)*x_ref[0]+(-0.839457764245)*x_ref[1]**o+(-0.58152129024)*x_ref[1]
        ref[1,4,1,1]=(0.0937571716669)*x_ref[0]**o+(0.592103688833)*x_ref[0]+(0.15448940004)*x_ref[1]**o+(-0.203463329576)*x_ref[1]
        ref[1,4,2,0]=(0.139741892415)*x_ref[0]**o+(0.611539882432)*x_ref[0]+(-0.407889172421)*x_ref[1]**o+(0.891469847051)*x_ref[1]
        ref[1,4,2,1]=(0.95147411312)*x_ref[0]**o+(0.0998931787148)*x_ref[0]+(0.466902306753)*x_ref[1]**o+(-0.718472606772)*x_ref[1]
        ref[2,0,0,0]=(-0.130008770209)*x_ref[0]**o+(0.809915931169)*x_ref[0]+(-0.732276845472)*x_ref[1]**o+(-0.192903892405)*x_ref[1]
        ref[2,0,0,1]=(-0.0119712505975)*x_ref[0]**o+(0.656694120973)*x_ref[0]+(0.6862601083)*x_ref[1]**o+(-0.528666986412)*x_ref[1]
        ref[2,0,1,0]=(0.28678233655)*x_ref[0]**o+(-0.37339891868)*x_ref[0]+(-0.946168471623)*x_ref[1]**o+(0.762233026568)*x_ref[1]
        ref[2,0,1,1]=(0.778843713994)*x_ref[0]**o+(0.817766330368)*x_ref[0]+(-0.300918496467)*x_ref[1]**o+(0.442988971838)*x_ref[1]
        ref[2,0,2,0]=(0.207449688136)*x_ref[0]**o+(0.10275570242)*x_ref[0]+(-0.520466147913)*x_ref[1]**o+(0.33346010447)*x_ref[1]
        ref[2,0,2,1]=(0.709221342424)*x_ref[0]**o+(0.159144243448)*x_ref[0]+(0.685824812808)*x_ref[1]**o+(0.50901941527)*x_ref[1]
        ref[2,1,0,0]=(0.0244924098009)*x_ref[0]**o+(-0.770942809125)*x_ref[0]+(0.372626872676)*x_ref[1]**o+(-0.714320503926)*x_ref[1]
        ref[2,1,0,1]=(-0.105052119318)*x_ref[0]**o+(0.573319234378)*x_ref[0]+(0.0995762579843)*x_ref[1]**o+(0.579088439928)*x_ref[1]
        ref[2,1,1,0]=(-0.572394543041)*x_ref[0]**o+(0.1606374212)*x_ref[0]+(-0.801731532404)*x_ref[1]**o+(-0.644110219424)*x_ref[1]
        ref[2,1,1,1]=(0.914053075068)*x_ref[0]**o+(-0.230478556116)*x_ref[0]+(-0.00343924084847)*x_ref[1]**o+(-0.53144169108)*x_ref[1]
        ref[2,1,2,0]=(-0.743362569446)*x_ref[0]**o+(-0.0428045574774)*x_ref[0]+(-0.503190273025)*x_ref[1]**o+(0.831658185042)*x_ref[1]
        ref[2,1,2,1]=(0.176721891893)*x_ref[0]**o+(-0.460950049697)*x_ref[0]+(0.45677887739)*x_ref[1]**o+(-0.853283022367)*x_ref[1]
        ref[2,2,0,0]=(-0.585337031786)*x_ref[0]**o+(0.901183590413)*x_ref[0]+(-0.746305431471)*x_ref[1]**o+(0.0439043730531)*x_ref[1]
        ref[2,2,0,1]=(0.594349646931)*x_ref[0]**o+(-0.431546660621)*x_ref[0]+(-0.962096864047)*x_ref[1]**o+(0.955112341273)*x_ref[1]
        ref[2,2,1,0]=(-0.857877499011)*x_ref[0]**o+(0.803223671127)*x_ref[0]+(-0.995806495668)*x_ref[1]**o+(-0.267707433165)*x_ref[1]
        ref[2,2,1,1]=(0.968406110131)*x_ref[0]**o+(-0.0208982523366)*x_ref[0]+(-0.000406237500393)*x_ref[1]**o+(-0.334429552696)*x_ref[1]
        ref[2,2,2,0]=(-0.960828307746)*x_ref[0]**o+(0.235004476043)*x_ref[0]+(0.0456041210872)*x_ref[1]**o+(0.323144610527)*x_ref[1]
        ref[2,2,2,1]=(0.538691072674)*x_ref[0]**o+(-0.737490055104)*x_ref[0]+(-0.96703538134)*x_ref[1]**o+(0.727057872325)*x_ref[1]
        ref[2,3,0,0]=(0.475319703318)*x_ref[0]**o+(0.863857783409)*x_ref[0]+(0.0891089574388)*x_ref[1]**o+(0.121619153024)*x_ref[1]
        ref[2,3,0,1]=(0.68881433038)*x_ref[0]**o+(-0.812908863657)*x_ref[0]+(0.687129220133)*x_ref[1]**o+(0.580728857382)*x_ref[1]
        ref[2,3,1,0]=(0.798247442811)*x_ref[0]**o+(0.600332572583)*x_ref[0]+(0.629676907787)*x_ref[1]**o+(0.0214779823831)*x_ref[1]
        ref[2,3,1,1]=(0.446783692926)*x_ref[0]**o+(-0.579857496036)*x_ref[0]+(0.669355002891)*x_ref[1]**o+(-0.104502660981)*x_ref[1]
        ref[2,3,2,0]=(0.874126641785)*x_ref[0]**o+(0.244866909003)*x_ref[0]+(-0.0181287421091)*x_ref[1]**o+(-0.728707148938)*x_ref[1]
        ref[2,3,2,1]=(-0.716069945597)*x_ref[0]**o+(0.632201538129)*x_ref[0]+(-0.538839892852)*x_ref[1]**o+(-0.886572197711)*x_ref[1]
        ref[2,4,0,0]=(-0.422214071919)*x_ref[0]**o+(0.276563507343)*x_ref[0]+(0.481338034453)*x_ref[1]**o+(-0.972870285915)*x_ref[1]
        ref[2,4,0,1]=(0.740441652804)*x_ref[0]**o+(0.447067551217)*x_ref[0]+(-0.310919183692)*x_ref[1]**o+(0.636513538883)*x_ref[1]
        ref[2,4,1,0]=(-0.599111504454)*x_ref[0]**o+(-0.250476629717)*x_ref[0]+(-0.658311629855)*x_ref[1]**o+(0.492577876045)*x_ref[1]
        ref[2,4,1,1]=(0.107735805283)*x_ref[0]**o+(0.208970264385)*x_ref[0]+(-0.147382019475)*x_ref[1]**o+(-0.406553806193)*x_ref[1]
        ref[2,4,2,0]=(0.925310626624)*x_ref[0]**o+(0.0400900248412)*x_ref[0]+(-0.431657024276)*x_ref[1]**o+(-0.153834430713)*x_ref[1]
        ref[2,4,2,1]=(-0.713453580612)*x_ref[0]**o+(0.391780861442)*x_ref[0]+(-0.940123468631)*x_ref[1]**o+(-0.937804820594)*x_ref[1]
        ref[3,0,0,0]=(-0.272038820171)*x_ref[0]**o+(0.0722677637085)*x_ref[0]+(-0.81552430022)*x_ref[1]**o+(-0.605100343524)*x_ref[1]
        ref[3,0,0,1]=(-0.869234792389)*x_ref[0]**o+(0.133360720775)*x_ref[0]+(0.334759295473)*x_ref[1]**o+(0.891704395665)*x_ref[1]
        ref[3,0,1,0]=(0.283196831063)*x_ref[0]**o+(-0.433641717885)*x_ref[0]+(0.821493597367)*x_ref[1]**o+(0.0757494621194)*x_ref[1]
        ref[3,0,1,1]=(-0.970851846713)*x_ref[0]**o+(0.420332346058)*x_ref[0]+(-0.128433226815)*x_ref[1]**o+(0.355540827779)*x_ref[1]
        ref[3,0,2,0]=(-0.855684873323)*x_ref[0]**o+(-0.896780938471)*x_ref[0]+(-0.910323225744)*x_ref[1]**o+(-0.648707265693)*x_ref[1]
        ref[3,0,2,1]=(0.224013589567)*x_ref[0]**o+(-0.109904216055)*x_ref[0]+(0.86374013156)*x_ref[1]**o+(0.474691072866)*x_ref[1]
        ref[3,1,0,0]=(-0.497944848072)*x_ref[0]**o+(0.146776775946)*x_ref[0]+(0.526474245297)*x_ref[1]**o+(-0.835737531991)*x_ref[1]
        ref[3,1,0,1]=(-0.421150013578)*x_ref[0]**o+(-0.784402607666)*x_ref[0]+(-0.0805148903284)*x_ref[1]**o+(-0.52347503704)*x_ref[1]
        ref[3,1,1,0]=(0.671180085936)*x_ref[0]**o+(-0.0708908978891)*x_ref[0]+(0.80005635402)*x_ref[1]**o+(0.250773812305)*x_ref[1]
        ref[3,1,1,1]=(-0.740517458065)*x_ref[0]**o+(-0.164975700504)*x_ref[0]+(-0.954776148663)*x_ref[1]**o+(0.0662528804032)*x_ref[1]
        ref[3,1,2,0]=(-0.0487118351329)*x_ref[0]**o+(0.0172491914156)*x_ref[0]+(0.459297824842)*x_ref[1]**o+(0.931441363698)*x_ref[1]
        ref[3,1,2,1]=(0.914541867519)*x_ref[0]**o+(-0.421726383007)*x_ref[0]+(0.932990305149)*x_ref[1]**o+(0.00708080928476)*x_ref[1]
        ref[3,2,0,0]=(-0.268505299798)*x_ref[0]**o+(-0.884105078203)*x_ref[0]+(-0.343177738426)*x_ref[1]**o+(0.925594916146)*x_ref[1]
        ref[3,2,0,1]=(0.792964128575)*x_ref[0]**o+(0.45312934456)*x_ref[0]+(-0.190272595857)*x_ref[1]**o+(-0.714484743775)*x_ref[1]
        ref[3,2,1,0]=(-0.389811244023)*x_ref[0]**o+(-0.230272427621)*x_ref[0]+(-0.062642727789)*x_ref[1]**o+(0.333179046361)*x_ref[1]
        ref[3,2,1,1]=(0.468980019323)*x_ref[0]**o+(-0.213426322608)*x_ref[0]+(0.806778675985)*x_ref[1]**o+(-0.031962815429)*x_ref[1]
        ref[3,2,2,0]=(0.187153153145)*x_ref[0]**o+(0.440459591208)*x_ref[0]+(-0.766229604202)*x_ref[1]**o+(0.630228576743)*x_ref[1]
        ref[3,2,2,1]=(-0.362635752315)*x_ref[0]**o+(-0.356960434705)*x_ref[0]+(-0.209248043764)*x_ref[1]**o+(0.70828385566)*x_ref[1]
        ref[3,3,0,0]=(-0.680482417515)*x_ref[0]**o+(-0.90459474138)*x_ref[0]+(-0.568552053392)*x_ref[1]**o+(0.394198411912)*x_ref[1]
        ref[3,3,0,1]=(-0.0932148057029)*x_ref[0]**o+(0.618628605947)*x_ref[0]+(0.0470711457335)*x_ref[1]**o+(0.802136993857)*x_ref[1]
        ref[3,3,1,0]=(-0.442943896794)*x_ref[0]**o+(0.57385228082)*x_ref[0]+(0.916092493461)*x_ref[1]**o+(-0.913579167434)*x_ref[1]
        ref[3,3,1,1]=(0.976285532615)*x_ref[0]**o+(-0.327052093737)*x_ref[0]+(0.731622827093)*x_ref[1]**o+(0.198510813385)*x_ref[1]
        ref[3,3,2,0]=(-0.921649918257)*x_ref[0]**o+(-0.954803772809)*x_ref[0]+(0.64789210876)*x_ref[1]**o+(-0.928730411729)*x_ref[1]
        ref[3,3,2,1]=(0.227719394295)*x_ref[0]**o+(-0.512300073514)*x_ref[0]+(-0.401469395128)*x_ref[1]**o+(-0.229248271712)*x_ref[1]
        ref[3,4,0,0]=(0.655903201215)*x_ref[0]**o+(0.285189680631)*x_ref[0]+(-0.183566536361)*x_ref[1]**o+(-0.18428580448)*x_ref[1]
        ref[3,4,0,1]=(-0.0282806177857)*x_ref[0]**o+(-0.29856051731)*x_ref[0]+(0.120325396195)*x_ref[1]**o+(-0.29397064071)*x_ref[1]
        ref[3,4,1,0]=(-0.640500611083)*x_ref[0]**o+(0.979817975078)*x_ref[0]+(0.577156862636)*x_ref[1]**o+(-0.00170003402221)*x_ref[1]
        ref[3,4,1,1]=(0.681055102015)*x_ref[0]**o+(-0.656903522238)*x_ref[0]+(0.470845699959)*x_ref[1]**o+(-0.973850290239)*x_ref[1]
        ref[3,4,2,0]=(-0.941893654596)*x_ref[0]**o+(0.96514461809)*x_ref[0]+(0.822665841321)*x_ref[1]**o+(-0.826621500352)*x_ref[1]
        ref[3,4,2,1]=(0.0808254384437)*x_ref[0]**o+(0.191567292517)*x_ref[0]+(-0.48838584117)*x_ref[1]**o+(-0.472758040235)*x_ref[1]
      else:
        s[0,0,0,0]=(0.186525987907)*x[0]**o+(-0.690917154378)*x[0]+(0.832803193217)*x[1]**o+(-0.0440294807629)*x[1]+(0.936927757561)*x[2]**o+(-0.120763864664)*x[2]
        s[0,0,0,1]=(0.0113224706745)*x[0]**o+(0.522015747536)*x[0]+(-0.281875411547)*x[1]**o+(0.605176091896)*x[1]+(-0.448130660869)*x[2]**o+(0.557667991786)*x[2]
        s[0,0,1,0]=(-0.5115411507)*x[0]**o+(0.568051998295)*x[0]+(0.722442651334)*x[1]**o+(0.410822384229)*x[1]+(0.235364385145)*x[2]**o+(0.881645768144)*x[2]
        s[0,0,1,1]=(0.365191539085)*x[0]**o+(0.0518813698411)*x[0]+(0.179359856471)*x[1]**o+(-0.634486389415)*x[1]+(0.311578648976)*x[2]**o+(-0.157034397126)*x[2]
        s[0,0,2,0]=(0.340474667312)*x[0]**o+(-0.761761779074)*x[0]+(-0.118701824049)*x[1]**o+(0.0922924561625)*x[1]+(0.132924567807)*x[2]**o+(-0.0694502802118)*x[2]
        s[0,0,2,1]=(0.21666407868)*x[0]**o+(0.65101030565)*x[0]+(0.0978745307399)*x[1]**o+(-0.592357988084)*x[1]+(-0.605382873404)*x[2]**o+(-0.899153475467)*x[2]
        s[0,1,0,0]=(-0.239693597874)*x[0]**o+(-0.265899323452)*x[0]+(-0.210985518909)*x[1]**o+(-0.408445623906)*x[1]+(0.350622495806)*x[2]**o+(-0.534920839922)*x[2]
        s[0,1,0,1]=(-0.596708903276)*x[0]**o+(0.241677381533)*x[0]+(-0.327063865807)*x[1]**o+(0.731700322335)*x[1]+(-0.29224310646)*x[2]**o+(-0.177855845822)*x[2]
        s[0,1,1,0]=(0.13119146515)*x[0]**o+(0.24155441457)*x[0]+(-0.730354339458)*x[1]**o+(0.17509896433)*x[1]+(0.602886357198)*x[2]**o+(-0.300613826457)*x[2]
        s[0,1,1,1]=(-0.528650609789)*x[0]**o+(0.730375830185)*x[0]+(0.706889523794)*x[1]**o+(-0.280144821714)*x[1]+(0.261124676699)*x[2]**o+(0.230842251108)*x[2]
        s[0,1,2,0]=(0.0867102525865)*x[0]**o+(0.238707033898)*x[0]+(-0.914453619505)*x[1]**o+(0.224533380834)*x[1]+(0.406716858519)*x[2]**o+(-0.347408293194)*x[2]
        s[0,1,2,1]=(-0.567166350757)*x[0]**o+(-0.156169071272)*x[0]+(0.15846212695)*x[1]**o+(-0.0254016167751)*x[1]+(0.707956392472)*x[2]**o+(-0.510965658867)*x[2]
        s[0,2,0,0]=(-0.181235844201)*x[0]**o+(0.936656137873)*x[0]+(0.830895834923)*x[1]**o+(0.752514827093)*x[1]+(0.478651458614)*x[2]**o+(0.353659575913)*x[2]
        s[0,2,0,1]=(-0.0466341076413)*x[0]**o+(-0.137913043533)*x[0]+(0.569602506318)*x[1]**o+(0.453951166085)*x[1]+(-0.736500514341)*x[2]**o+(0.494877047573)*x[2]
        s[0,2,1,0]=(-0.774800275665)*x[0]**o+(0.320634390869)*x[0]+(-0.801898551402)*x[1]**o+(0.680379650404)*x[1]+(0.115138386556)*x[2]**o+(0.847285566367)*x[2]
        s[0,2,1,1]=(-0.948363366443)*x[0]**o+(-0.401155889062)*x[0]+(0.602001652794)*x[1]**o+(0.881111991966)*x[1]+(-0.407826312421)*x[2]**o+(-0.385793087535)*x[2]
        s[0,2,2,0]=(-0.0498782718626)*x[0]**o+(0.278903888089)*x[0]+(0.28455096215)*x[1]**o+(0.165496315478)*x[1]+(-0.516044235314)*x[2]**o+(-0.320130768929)*x[2]
        s[0,2,2,1]=(-0.0313287552123)*x[0]**o+(0.795534322833)*x[0]+(0.0672014159699)*x[1]**o+(-0.232632859958)*x[1]+(0.968980662087)*x[2]**o+(-0.0437975661275)*x[2]
        s[0,3,0,0]=(0.879723765854)*x[0]**o+(0.616502035178)*x[0]+(-0.888143209368)*x[1]**o+(-0.954918894054)*x[1]+(-0.0649197025038)*x[2]**o+(-0.865957161833)*x[2]
        s[0,3,0,1]=(0.550691528997)*x[0]**o+(0.136724942999)*x[0]+(0.474412879455)*x[1]**o+(0.677263937069)*x[1]+(0.453101604697)*x[2]**o+(0.133742079633)*x[2]
        s[0,3,1,0]=(-0.970027293215)*x[0]**o+(-0.610696832357)*x[0]+(-0.248732897836)*x[1]**o+(0.957868248702)*x[1]+(0.334784026675)*x[2]**o+(0.696895349867)*x[2]
        s[0,3,1,1]=(-0.672808837142)*x[0]**o+(-0.0553936755153)*x[0]+(-0.00732320498388)*x[1]**o+(0.49021088588)*x[1]+(0.907554472651)*x[2]**o+(-0.058244899423)*x[2]
        s[0,3,2,0]=(0.0159001870399)*x[0]**o+(0.0227678146243)*x[0]+(0.805708466496)*x[1]**o+(0.785990910247)*x[1]+(-0.725672039232)*x[2]**o+(-0.162870251036)*x[2]
        s[0,3,2,1]=(0.520705647829)*x[0]**o+(-0.310310041859)*x[0]+(-0.317118647815)*x[1]**o+(0.534773934168)*x[1]+(-0.268620331386)*x[2]**o+(0.334022726024)*x[2]
        s[0,4,0,0]=(0.529035051582)*x[0]**o+(0.566819545868)*x[0]+(0.618896518995)*x[1]**o+(0.263612275958)*x[1]+(-0.618741920385)*x[2]**o+(-0.0264643721073)*x[2]
        s[0,4,0,1]=(0.860757010858)*x[0]**o+(-0.706974137596)*x[0]+(0.0612556623739)*x[1]**o+(0.722512189141)*x[1]+(-0.676611378845)*x[2]**o+(-0.615875915177)*x[2]
        s[0,4,1,0]=(-0.186509714673)*x[0]**o+(0.0351164428631)*x[0]+(-0.173768366165)*x[1]**o+(-0.426776871363)*x[1]+(-0.0678988589984)*x[2]**o+(-0.0164388017801)*x[2]
        s[0,4,1,1]=(-0.528750419399)*x[0]**o+(-0.0188958007868)*x[0]+(-0.381850037026)*x[1]**o+(-0.366474181406)*x[1]+(-0.0157097200919)*x[2]**o+(0.750818168768)*x[2]
        s[0,4,2,0]=(0.962983200785)*x[0]**o+(-0.904671719523)*x[0]+(0.83012715029)*x[1]**o+(0.20786351761)*x[1]+(0.61032830083)*x[2]**o+(0.999068101126)*x[2]
        s[0,4,2,1]=(0.369225553113)*x[0]**o+(0.370643799864)*x[0]+(0.562458197002)*x[1]**o+(0.864829790167)*x[1]+(-0.507288312866)*x[2]**o+(-0.615656522393)*x[2]
        s[1,0,0,0]=(0.0208552161791)*x[0]**o+(0.64767817505)*x[0]+(-0.913625079536)*x[1]**o+(0.214341575088)*x[1]+(-0.369358777546)*x[2]**o+(-0.744597104081)*x[2]
        s[1,0,0,1]=(0.688956912909)*x[0]**o+(0.130856033181)*x[0]+(-0.754570690839)*x[1]**o+(-0.892591311673)*x[1]+(0.121513821098)*x[2]**o+(0.0499651928086)*x[2]
        s[1,0,1,0]=(0.488371732492)*x[0]**o+(-0.0899662322511)*x[0]+(-0.184052688998)*x[1]**o+(-0.504791015013)*x[1]+(0.939413170478)*x[2]**o+(-0.371392738292)*x[2]
        s[1,0,1,1]=(-0.634015136517)*x[0]**o+(0.189031198575)*x[0]+(-0.187597065324)*x[1]**o+(0.417035820606)*x[1]+(-0.640693898356)*x[2]**o+(-0.356999823488)*x[2]
        s[1,0,2,0]=(-0.299739907169)*x[0]**o+(0.97973415315)*x[0]+(0.172728079372)*x[1]**o+(-0.219250264223)*x[1]+(0.0377842392779)*x[2]**o+(0.568605450772)*x[2]
        s[1,0,2,1]=(0.668822572798)*x[0]**o+(-0.464904703232)*x[0]+(-0.0438179224843)*x[1]**o+(0.972577571624)*x[1]+(0.200473119295)*x[2]**o+(0.769536551131)*x[2]
        s[1,1,0,0]=(-0.5952291144)*x[0]**o+(0.215912343317)*x[0]+(-0.187871783507)*x[1]**o+(0.000141658119764)*x[1]+(0.652350178895)*x[2]**o+(0.748942594082)*x[2]
        s[1,1,0,1]=(0.978202121028)*x[0]**o+(-0.270952075217)*x[0]+(0.379211795706)*x[1]**o+(0.570614905933)*x[1]+(0.531348311358)*x[2]**o+(-0.774802192871)*x[2]
        s[1,1,1,0]=(-0.173449381518)*x[0]**o+(-0.788533061983)*x[0]+(-0.159503791119)*x[1]**o+(-0.182774291708)*x[1]+(0.493985220422)*x[2]**o+(-0.442523611538)*x[2]
        s[1,1,1,1]=(-0.196642522444)*x[0]**o+(0.353038689926)*x[0]+(0.739133358128)*x[1]**o+(0.384564329164)*x[1]+(0.00724850200888)*x[2]**o+(-0.534791252863)*x[2]
        s[1,1,2,0]=(-0.742601262273)*x[0]**o+(0.870936220364)*x[0]+(0.173515343981)*x[1]**o+(-0.924818265666)*x[1]+(-0.786225297604)*x[2]**o+(0.723191951357)*x[2]
        s[1,1,2,1]=(0.584821708581)*x[0]**o+(0.233692617647)*x[0]+(0.917976018627)*x[1]**o+(0.0411719886893)*x[1]+(-0.731518066297)*x[2]**o+(-0.886641339023)*x[2]
        s[1,2,0,0]=(0.879019029124)*x[0]**o+(0.198224253024)*x[0]+(-0.508879741455)*x[1]**o+(-0.0493693577221)*x[1]+(-0.200204490189)*x[2]**o+(-0.558819359406)*x[2]
        s[1,2,0,1]=(0.343696269765)*x[0]**o+(-0.694690909901)*x[0]+(0.562857727334)*x[1]**o+(0.170105867024)*x[1]+(0.300442565793)*x[2]**o+(-0.78443966073)*x[2]
        s[1,2,1,0]=(-0.104227942738)*x[0]**o+(-0.631808216742)*x[0]+(-0.688416357176)*x[1]**o+(0.543474609002)*x[1]+(-0.475674253864)*x[2]**o+(-0.622363009599)*x[2]
        s[1,2,1,1]=(-0.187993381022)*x[0]**o+(-0.112771930635)*x[0]+(0.231591342365)*x[1]**o+(-0.619972979708)*x[1]+(-0.633470156859)*x[2]**o+(0.726102617311)*x[2]
        s[1,2,2,0]=(-0.0929871065489)*x[0]**o+(-0.741493547909)*x[0]+(-0.500399374101)*x[1]**o+(0.489221111027)*x[1]+(-0.538850386612)*x[2]**o+(-0.366843628744)*x[2]
        s[1,2,2,1]=(-0.449265426222)*x[0]**o+(-0.711133901403)*x[0]+(-0.671933617866)*x[1]**o+(-0.75650179505)*x[1]+(-0.441794206618)*x[2]**o+(-0.679799182186)*x[2]
        s[1,3,0,0]=(0.0794555795393)*x[0]**o+(-0.591466474089)*x[0]+(0.537807349728)*x[1]**o+(-0.0295904103007)*x[1]+(-0.206212961776)*x[2]**o+(0.633837648242)*x[2]
        s[1,3,0,1]=(0.232707718354)*x[0]**o+(-0.310438878448)*x[0]+(-0.0844732229089)*x[1]**o+(0.507469871796)*x[1]+(-0.391898028012)*x[2]**o+(-0.374582716512)*x[2]
        s[1,3,1,0]=(-0.166926427703)*x[0]**o+(-0.926298908928)*x[0]+(0.244425001632)*x[1]**o+(0.284615289051)*x[1]+(-0.675054769717)*x[2]**o+(-0.0765552057049)*x[2]
        s[1,3,1,1]=(-0.469538130468)*x[0]**o+(-0.110820901289)*x[0]+(-0.608743940572)*x[1]**o+(0.0606943645263)*x[1]+(-0.211127814891)*x[2]**o+(-0.0474195944349)*x[2]
        s[1,3,2,0]=(0.769147852508)*x[0]**o+(-0.500794487532)*x[0]+(-0.0971530930356)*x[1]**o+(0.282178880991)*x[1]+(-0.175125963785)*x[2]**o+(0.0567340005296)*x[2]
        s[1,3,2,1]=(0.14422447964)*x[0]**o+(-0.9056785988)*x[0]+(-0.962346923857)*x[1]**o+(0.64076498247)*x[1]+(0.77680599777)*x[2]**o+(0.450703438151)*x[2]
        s[1,4,0,0]=(0.421930869494)*x[0]**o+(0.889038749248)*x[0]+(-0.989075156799)*x[1]**o+(-0.614721833409)*x[1]+(0.112500578762)*x[2]**o+(-0.47083438608)*x[2]
        s[1,4,0,1]=(-0.0180470731329)*x[0]**o+(-0.47539878993)*x[0]+(0.317084518451)*x[1]**o+(0.237846416515)*x[1]+(0.375600671255)*x[2]**o+(0.66276020876)*x[2]
        s[1,4,1,0]=(0.125983672606)*x[0]**o+(0.595653397427)*x[0]+(0.31474692021)*x[1]**o+(0.234488164569)*x[1]+(0.618714006094)*x[2]**o+(-0.160776384119)*x[2]
        s[1,4,1,1]=(0.951917864448)*x[0]**o+(-0.33473954392)*x[0]+(-0.0378707336313)*x[1]**o+(-0.238158748773)*x[1]+(0.574999874285)*x[2]**o+(0.232302306417)*x[2]
        s[1,4,2,0]=(0.505450486119)*x[0]**o+(-0.370259193476)*x[0]+(-0.663053563021)*x[1]**o+(0.669872208578)*x[1]+(0.931794239064)*x[2]**o+(0.133184889523)*x[2]
        s[1,4,2,1]=(-0.937476724928)*x[0]**o+(0.110843923089)*x[0]+(-0.425245370966)*x[1]**o+(0.409116775141)*x[1]+(-0.273590511289)*x[2]**o+(0.441443512397)*x[2]
        s[2,0,0,0]=(0.0599017312855)*x[0]**o+(-0.941401999676)*x[0]+(0.379283945566)*x[1]**o+(-0.336352358178)*x[1]+(-0.514408042486)*x[2]**o+(-0.169011791163)*x[2]
        s[2,0,0,1]=(-0.269712116236)*x[0]**o+(-0.349623235155)*x[0]+(0.522466035838)*x[1]**o+(0.427111359806)*x[1]+(-0.0296663851565)*x[2]**o+(-0.0191832326161)*x[2]
        s[2,0,1,0]=(0.14892181923)*x[0]**o+(0.269460608852)*x[0]+(-0.0343896163306)*x[1]**o+(0.590044957948)*x[1]+(0.946719737501)*x[2]**o+(0.315742744827)*x[2]
        s[2,0,1,1]=(-0.388655118673)*x[0]**o+(-0.00180659903075)*x[0]+(0.969087798155)*x[1]**o+(0.86635225912)*x[1]+(-0.761434218549)*x[2]**o+(-0.323637696199)*x[2]
        s[2,0,2,0]=(0.588641788156)*x[0]**o+(-0.00912456965621)*x[0]+(-0.0266085527484)*x[1]**o+(0.0598075913127)*x[1]+(-0.658095450927)*x[2]**o+(0.110978909962)*x[2]
        s[2,0,2,1]=(0.139641588755)*x[0]**o+(0.593809622136)*x[0]+(-0.259839406352)*x[1]**o+(0.848541675808)*x[1]+(-0.860886035055)*x[2]**o+(-0.489752174489)*x[2]
        s[2,1,0,0]=(0.850328373883)*x[0]**o+(0.675383216776)*x[0]+(-0.117397818856)*x[1]**o+(-0.344801013716)*x[1]+(-0.887091513121)*x[2]**o+(0.584872468856)*x[2]
        s[2,1,0,1]=(-0.745901868498)*x[0]**o+(-0.407149004368)*x[0]+(-0.182253232897)*x[1]**o+(-0.203000014779)*x[1]+(-0.172710156372)*x[2]**o+(0.127673047341)*x[2]
        s[2,1,1,0]=(0.655182501192)*x[0]**o+(-0.17668945533)*x[0]+(0.717711469841)*x[1]**o+(0.595777555767)*x[1]+(-0.974184895311)*x[2]**o+(-0.0299700820305)*x[2]
        s[2,1,1,1]=(0.707793528102)*x[0]**o+(-0.0624977281364)*x[0]+(0.623777123073)*x[1]**o+(-0.794217090978)*x[1]+(0.542320017675)*x[2]**o+(0.903852498061)*x[2]
        s[2,1,2,0]=(0.713295120131)*x[0]**o+(-0.174892054913)*x[0]+(-0.251138558622)*x[1]**o+(-0.77735156875)*x[1]+(-0.94176928221)*x[2]**o+(-0.0244708586622)*x[2]
        s[2,1,2,1]=(0.933976789519)*x[0]**o+(0.90177842055)*x[0]+(-0.343254953267)*x[1]**o+(0.713448014636)*x[1]+(-0.468875743251)*x[2]**o+(0.350869684481)*x[2]
        s[2,2,0,0]=(0.390367713904)*x[0]**o+(-0.601608385293)*x[0]+(-0.757340195841)*x[1]**o+(0.360436452755)*x[1]+(0.291087135717)*x[2]**o+(0.316481151314)*x[2]
        s[2,2,0,1]=(0.111600306678)*x[0]**o+(0.823704129888)*x[0]+(0.682856997317)*x[1]**o+(-0.523736197366)*x[1]+(-0.100121183783)*x[2]**o+(-0.666230008262)*x[2]
        s[2,2,1,0]=(0.173360884658)*x[0]**o+(0.949511779713)*x[0]+(0.259889008248)*x[1]**o+(-0.942214191047)*x[1]+(-0.48690925565)*x[2]**o+(-0.264798193102)*x[2]
        s[2,2,1,1]=(0.483759255276)*x[0]**o+(-0.657669281632)*x[0]+(-0.154764754082)*x[1]**o+(-0.881550854822)*x[1]+(0.523812678208)*x[2]**o+(-0.979914768652)*x[2]
        s[2,2,2,0]=(0.137127593491)*x[0]**o+(-0.104908009627)*x[0]+(0.579522653005)*x[1]**o+(0.844822805269)*x[1]+(0.0328079119318)*x[2]**o+(-0.26897713517)*x[2]
        s[2,2,2,1]=(0.377116909347)*x[0]**o+(0.752578358605)*x[0]+(-0.467992979597)*x[1]**o+(0.254761405884)*x[1]+(-0.391559118991)*x[2]**o+(-0.61268413168)*x[2]
        s[2,3,0,0]=(-0.989947766953)*x[0]**o+(0.830717454791)*x[0]+(-0.473161731716)*x[1]**o+(0.625028238242)*x[1]+(-0.546708564025)*x[2]**o+(-0.691027642154)*x[2]
        s[2,3,0,1]=(0.650495990975)*x[0]**o+(-0.654863347162)*x[0]+(0.638342908241)*x[1]**o+(-0.0485838638503)*x[1]+(-0.632091721289)*x[2]**o+(0.791203973276)*x[2]
        s[2,3,1,0]=(0.266156252436)*x[0]**o+(0.272153723938)*x[0]+(-0.159895797204)*x[1]**o+(-0.640350200995)*x[1]+(0.990601940876)*x[2]**o+(-0.0816835237084)*x[2]
        s[2,3,1,1]=(0.562807345614)*x[0]**o+(0.544038968862)*x[0]+(0.878928928869)*x[1]**o+(0.404794709914)*x[1]+(0.269928450336)*x[2]**o+(0.869832826526)*x[2]
        s[2,3,2,0]=(-0.899170106614)*x[0]**o+(-0.031251129011)*x[0]+(0.0696989801411)*x[1]**o+(0.89839882525)*x[1]+(0.99905365333)*x[2]**o+(-0.856781333049)*x[2]
        s[2,3,2,1]=(0.980091885122)*x[0]**o+(-0.12832421228)*x[0]+(0.0798899702085)*x[1]**o+(-0.729861980386)*x[1]+(-0.970268210161)*x[2]**o+(0.319025854228)*x[2]
        s[2,4,0,0]=(0.891703788308)*x[0]**o+(-0.0724026917568)*x[0]+(-0.74794796925)*x[1]**o+(-0.339878454118)*x[1]+(0.413228256742)*x[2]**o+(-0.371024671724)*x[2]
        s[2,4,0,1]=(0.640892618319)*x[0]**o+(-0.44510403082)*x[0]+(-0.100468205238)*x[1]**o+(0.923901932578)*x[1]+(0.731059415968)*x[2]**o+(-0.609224901759)*x[2]
        s[2,4,1,0]=(0.130333662915)*x[0]**o+(0.394752475547)*x[0]+(-0.11866957093)*x[1]**o+(0.980649245967)*x[1]+(-0.602353311004)*x[2]**o+(0.406199522673)*x[2]
        s[2,4,1,1]=(0.0665437323779)*x[0]**o+(0.952105166153)*x[0]+(0.488465603925)*x[1]**o+(0.0178900500949)*x[1]+(0.206584014652)*x[2]**o+(0.446123239912)*x[2]
        s[2,4,2,0]=(-0.792540868222)*x[0]**o+(0.0265488816348)*x[0]+(-0.693338065664)*x[1]**o+(-0.682629782298)*x[1]+(-0.778972053935)*x[2]**o+(-0.0274648446373)*x[2]
        s[2,4,2,1]=(0.916389263123)*x[0]**o+(-0.662528447313)*x[0]+(-0.0703995406002)*x[1]**o+(0.385180574963)*x[1]+(-0.855872398416)*x[2]**o+(0.5530510507)*x[2]
        s[3,0,0,0]=(-0.306821143583)*x[0]**o+(-0.962173177288)*x[0]+(0.993016856406)*x[1]**o+(-0.579698238302)*x[1]+(0.571895736521)*x[2]**o+(-0.88966756316)*x[2]
        s[3,0,0,1]=(0.719926546559)*x[0]**o+(-0.0858941607816)*x[0]+(0.807242676458)*x[1]**o+(-0.190440184356)*x[1]+(-0.136927444751)*x[2]**o+(0.869886135543)*x[2]
        s[3,0,1,0]=(0.892148659939)*x[0]**o+(-0.482479220378)*x[0]+(0.503671753254)*x[1]**o+(-0.552769467575)*x[1]+(0.138159512264)*x[2]**o+(0.102674771614)*x[2]
        s[3,0,1,1]=(0.185256299506)*x[0]**o+(-0.369351326706)*x[0]+(0.220953483691)*x[1]**o+(-0.0131034510397)*x[1]+(0.956037877433)*x[2]**o+(0.568289950328)*x[2]
        s[3,0,2,0]=(0.045864662475)*x[0]**o+(0.0183434079897)*x[0]+(-0.428111425773)*x[1]**o+(0.254667250574)*x[1]+(-0.983322740032)*x[2]**o+(0.877555405709)*x[2]
        s[3,0,2,1]=(-0.408521811841)*x[0]**o+(0.00640043590699)*x[0]+(-0.105754378317)*x[1]**o+(-0.952486645506)*x[1]+(-0.725225188014)*x[2]**o+(-0.205556627888)*x[2]
        s[3,1,0,0]=(-0.944493178271)*x[0]**o+(0.810364339849)*x[0]+(0.705003782307)*x[1]**o+(-0.880195742246)*x[1]+(-0.843196216793)*x[2]**o+(-0.971449880924)*x[2]
        s[3,1,0,1]=(0.150616114959)*x[0]**o+(-0.717321007987)*x[0]+(0.194099301116)*x[1]**o+(-0.0345362551173)*x[1]+(0.375587089675)*x[2]**o+(-0.409625637147)*x[2]
        s[3,1,1,0]=(0.0802340794764)*x[0]**o+(0.0153666337202)*x[0]+(-0.782772322072)*x[1]**o+(-0.0965866811611)*x[1]+(0.725231231559)*x[2]**o+(-0.452016239379)*x[2]
        s[3,1,1,1]=(0.546956847386)*x[0]**o+(0.839322197764)*x[0]+(-0.735023211892)*x[1]**o+(-0.41772545319)*x[1]+(-0.843610604975)*x[2]**o+(-0.145546519925)*x[2]
        s[3,1,2,0]=(0.729170600381)*x[0]**o+(-0.551244794813)*x[0]+(0.846396830038)*x[1]**o+(-0.759556129789)*x[1]+(0.136640189225)*x[2]**o+(-0.379654525677)*x[2]
        s[3,1,2,1]=(-0.953857643507)*x[0]**o+(-0.304074878681)*x[0]+(0.472716648295)*x[1]**o+(-0.179292020611)*x[1]+(0.909646409116)*x[2]**o+(-0.149337148625)*x[2]
        s[3,2,0,0]=(-0.165651506554)*x[0]**o+(0.410286556434)*x[0]+(-0.385291761561)*x[1]**o+(-0.932290142267)*x[1]+(0.562269759333)*x[2]**o+(-0.516760168836)*x[2]
        s[3,2,0,1]=(-0.874391348276)*x[0]**o+(-0.626614642393)*x[0]+(0.135326235872)*x[1]**o+(-0.169497542734)*x[1]+(0.453312712391)*x[2]**o+(0.699294330345)*x[2]
        s[3,2,1,0]=(0.30374985921)*x[0]**o+(-0.66286753305)*x[0]+(-0.504388407872)*x[1]**o+(-0.0864778873713)*x[1]+(0.7780825189)*x[2]**o+(0.607545865231)*x[2]
        s[3,2,1,1]=(0.323217898368)*x[0]**o+(0.256285070904)*x[0]+(0.32326826477)*x[1]**o+(-0.983183590335)*x[1]+(0.530605958132)*x[2]**o+(-0.109640854019)*x[2]
        s[3,2,2,0]=(0.587862067621)*x[0]**o+(-0.951653317735)*x[0]+(0.089519431923)*x[1]**o+(-0.780330592384)*x[1]+(-0.0483676475341)*x[2]**o+(0.766538955993)*x[2]
        s[3,2,2,1]=(-0.957533418498)*x[0]**o+(-0.681231272445)*x[0]+(-0.0918289718767)*x[1]**o+(0.38725634145)*x[1]+(0.26451874534)*x[2]**o+(-0.901046807078)*x[2]
        s[3,3,0,0]=(0.604215052532)*x[0]**o+(-0.0188225127963)*x[0]+(-0.544365413309)*x[1]**o+(-0.837547618266)*x[1]+(0.506665562924)*x[2]**o+(0.868580855034)*x[2]
        s[3,3,0,1]=(0.307620045896)*x[0]**o+(-0.941702619274)*x[0]+(0.467078433616)*x[1]**o+(-0.525971373276)*x[1]+(-0.438681578224)*x[2]**o+(-0.865538708837)*x[2]
        s[3,3,1,0]=(-0.128299566207)*x[0]**o+(0.0526714707626)*x[0]+(-0.194464943133)*x[1]**o+(0.848343510106)*x[1]+(-0.843042290362)*x[2]**o+(-0.384441684551)*x[2]
        s[3,3,1,1]=(-0.0202567344895)*x[0]**o+(0.919037490786)*x[0]+(0.953616123775)*x[1]**o+(-0.359010584029)*x[1]+(-0.892651116514)*x[2]**o+(0.900116594426)*x[2]
        s[3,3,2,0]=(-0.196701204528)*x[0]**o+(0.173992581268)*x[0]+(0.0304200566544)*x[1]**o+(-0.562311121578)*x[1]+(-0.349902986645)*x[2]**o+(0.137392211876)*x[2]
        s[3,3,2,1]=(-0.264756448582)*x[0]**o+(0.104810865018)*x[0]+(-0.550559232481)*x[1]**o+(-0.621292330017)*x[1]+(-0.697096182063)*x[2]**o+(-0.137686685739)*x[2]
        s[3,4,0,0]=(-0.0500780621237)*x[0]**o+(-0.842196022641)*x[0]+(-0.375688956182)*x[1]**o+(0.293383514141)*x[1]+(-0.679863519997)*x[2]**o+(-0.108799395703)*x[2]
        s[3,4,0,1]=(-0.825663711773)*x[0]**o+(0.0811924383936)*x[0]+(0.658753764102)*x[1]**o+(0.213557161034)*x[1]+(0.048778548639)*x[2]**o+(0.308220587221)*x[2]
        s[3,4,1,0]=(-0.0366332011016)*x[0]**o+(-0.935060469197)*x[0]+(0.52021729799)*x[1]**o+(0.270580190954)*x[1]+(0.690657153532)*x[2]**o+(0.572985687766)*x[2]
        s[3,4,1,1]=(-0.358041857957)*x[0]**o+(0.151420239879)*x[0]+(0.578841565624)*x[1]**o+(0.379085642891)*x[1]+(0.231447762893)*x[2]**o+(-0.494885835203)*x[2]
        s[3,4,2,0]=(0.689670410742)*x[0]**o+(-0.469898987295)*x[0]+(-0.617664742358)*x[1]**o+(-0.92098566662)*x[1]+(0.535497980872)*x[2]**o+(-0.660036711933)*x[2]
        s[3,4,2,1]=(-0.443846183672)*x[0]**o+(-0.0606185138497)*x[0]+(0.902723111093)*x[1]**o+(-0.201894548349)*x[1]+(0.650998336304)*x[2]**o+(-0.936010399236)*x[2]
        ref[0,0,0,0]=(0.186525987907)*x_ref[0]**o+(-0.690917154378)*x_ref[0]+(0.832803193217)*x_ref[1]**o+(-0.0440294807629)*x_ref[1]+(0.936927757561)*x_ref[2]**o+(-0.120763864664)*x_ref[2]
        ref[0,0,0,1]=(0.0113224706745)*x_ref[0]**o+(0.522015747536)*x_ref[0]+(-0.281875411547)*x_ref[1]**o+(0.605176091896)*x_ref[1]+(-0.448130660869)*x_ref[2]**o+(0.557667991786)*x_ref[2]
        ref[0,0,1,0]=(-0.5115411507)*x_ref[0]**o+(0.568051998295)*x_ref[0]+(0.722442651334)*x_ref[1]**o+(0.410822384229)*x_ref[1]+(0.235364385145)*x_ref[2]**o+(0.881645768144)*x_ref[2]
        ref[0,0,1,1]=(0.365191539085)*x_ref[0]**o+(0.0518813698411)*x_ref[0]+(0.179359856471)*x_ref[1]**o+(-0.634486389415)*x_ref[1]+(0.311578648976)*x_ref[2]**o+(-0.157034397126)*x_ref[2]
        ref[0,0,2,0]=(0.340474667312)*x_ref[0]**o+(-0.761761779074)*x_ref[0]+(-0.118701824049)*x_ref[1]**o+(0.0922924561625)*x_ref[1]+(0.132924567807)*x_ref[2]**o+(-0.0694502802118)*x_ref[2]
        ref[0,0,2,1]=(0.21666407868)*x_ref[0]**o+(0.65101030565)*x_ref[0]+(0.0978745307399)*x_ref[1]**o+(-0.592357988084)*x_ref[1]+(-0.605382873404)*x_ref[2]**o+(-0.899153475467)*x_ref[2]
        ref[0,1,0,0]=(-0.239693597874)*x_ref[0]**o+(-0.265899323452)*x_ref[0]+(-0.210985518909)*x_ref[1]**o+(-0.408445623906)*x_ref[1]+(0.350622495806)*x_ref[2]**o+(-0.534920839922)*x_ref[2]
        ref[0,1,0,1]=(-0.596708903276)*x_ref[0]**o+(0.241677381533)*x_ref[0]+(-0.327063865807)*x_ref[1]**o+(0.731700322335)*x_ref[1]+(-0.29224310646)*x_ref[2]**o+(-0.177855845822)*x_ref[2]
        ref[0,1,1,0]=(0.13119146515)*x_ref[0]**o+(0.24155441457)*x_ref[0]+(-0.730354339458)*x_ref[1]**o+(0.17509896433)*x_ref[1]+(0.602886357198)*x_ref[2]**o+(-0.300613826457)*x_ref[2]
        ref[0,1,1,1]=(-0.528650609789)*x_ref[0]**o+(0.730375830185)*x_ref[0]+(0.706889523794)*x_ref[1]**o+(-0.280144821714)*x_ref[1]+(0.261124676699)*x_ref[2]**o+(0.230842251108)*x_ref[2]
        ref[0,1,2,0]=(0.0867102525865)*x_ref[0]**o+(0.238707033898)*x_ref[0]+(-0.914453619505)*x_ref[1]**o+(0.224533380834)*x_ref[1]+(0.406716858519)*x_ref[2]**o+(-0.347408293194)*x_ref[2]
        ref[0,1,2,1]=(-0.567166350757)*x_ref[0]**o+(-0.156169071272)*x_ref[0]+(0.15846212695)*x_ref[1]**o+(-0.0254016167751)*x_ref[1]+(0.707956392472)*x_ref[2]**o+(-0.510965658867)*x_ref[2]
        ref[0,2,0,0]=(-0.181235844201)*x_ref[0]**o+(0.936656137873)*x_ref[0]+(0.830895834923)*x_ref[1]**o+(0.752514827093)*x_ref[1]+(0.478651458614)*x_ref[2]**o+(0.353659575913)*x_ref[2]
        ref[0,2,0,1]=(-0.0466341076413)*x_ref[0]**o+(-0.137913043533)*x_ref[0]+(0.569602506318)*x_ref[1]**o+(0.453951166085)*x_ref[1]+(-0.736500514341)*x_ref[2]**o+(0.494877047573)*x_ref[2]
        ref[0,2,1,0]=(-0.774800275665)*x_ref[0]**o+(0.320634390869)*x_ref[0]+(-0.801898551402)*x_ref[1]**o+(0.680379650404)*x_ref[1]+(0.115138386556)*x_ref[2]**o+(0.847285566367)*x_ref[2]
        ref[0,2,1,1]=(-0.948363366443)*x_ref[0]**o+(-0.401155889062)*x_ref[0]+(0.602001652794)*x_ref[1]**o+(0.881111991966)*x_ref[1]+(-0.407826312421)*x_ref[2]**o+(-0.385793087535)*x_ref[2]
        ref[0,2,2,0]=(-0.0498782718626)*x_ref[0]**o+(0.278903888089)*x_ref[0]+(0.28455096215)*x_ref[1]**o+(0.165496315478)*x_ref[1]+(-0.516044235314)*x_ref[2]**o+(-0.320130768929)*x_ref[2]
        ref[0,2,2,1]=(-0.0313287552123)*x_ref[0]**o+(0.795534322833)*x_ref[0]+(0.0672014159699)*x_ref[1]**o+(-0.232632859958)*x_ref[1]+(0.968980662087)*x_ref[2]**o+(-0.0437975661275)*x_ref[2]
        ref[0,3,0,0]=(0.879723765854)*x_ref[0]**o+(0.616502035178)*x_ref[0]+(-0.888143209368)*x_ref[1]**o+(-0.954918894054)*x_ref[1]+(-0.0649197025038)*x_ref[2]**o+(-0.865957161833)*x_ref[2]
        ref[0,3,0,1]=(0.550691528997)*x_ref[0]**o+(0.136724942999)*x_ref[0]+(0.474412879455)*x_ref[1]**o+(0.677263937069)*x_ref[1]+(0.453101604697)*x_ref[2]**o+(0.133742079633)*x_ref[2]
        ref[0,3,1,0]=(-0.970027293215)*x_ref[0]**o+(-0.610696832357)*x_ref[0]+(-0.248732897836)*x_ref[1]**o+(0.957868248702)*x_ref[1]+(0.334784026675)*x_ref[2]**o+(0.696895349867)*x_ref[2]
        ref[0,3,1,1]=(-0.672808837142)*x_ref[0]**o+(-0.0553936755153)*x_ref[0]+(-0.00732320498388)*x_ref[1]**o+(0.49021088588)*x_ref[1]+(0.907554472651)*x_ref[2]**o+(-0.058244899423)*x_ref[2]
        ref[0,3,2,0]=(0.0159001870399)*x_ref[0]**o+(0.0227678146243)*x_ref[0]+(0.805708466496)*x_ref[1]**o+(0.785990910247)*x_ref[1]+(-0.725672039232)*x_ref[2]**o+(-0.162870251036)*x_ref[2]
        ref[0,3,2,1]=(0.520705647829)*x_ref[0]**o+(-0.310310041859)*x_ref[0]+(-0.317118647815)*x_ref[1]**o+(0.534773934168)*x_ref[1]+(-0.268620331386)*x_ref[2]**o+(0.334022726024)*x_ref[2]
        ref[0,4,0,0]=(0.529035051582)*x_ref[0]**o+(0.566819545868)*x_ref[0]+(0.618896518995)*x_ref[1]**o+(0.263612275958)*x_ref[1]+(-0.618741920385)*x_ref[2]**o+(-0.0264643721073)*x_ref[2]
        ref[0,4,0,1]=(0.860757010858)*x_ref[0]**o+(-0.706974137596)*x_ref[0]+(0.0612556623739)*x_ref[1]**o+(0.722512189141)*x_ref[1]+(-0.676611378845)*x_ref[2]**o+(-0.615875915177)*x_ref[2]
        ref[0,4,1,0]=(-0.186509714673)*x_ref[0]**o+(0.0351164428631)*x_ref[0]+(-0.173768366165)*x_ref[1]**o+(-0.426776871363)*x_ref[1]+(-0.0678988589984)*x_ref[2]**o+(-0.0164388017801)*x_ref[2]
        ref[0,4,1,1]=(-0.528750419399)*x_ref[0]**o+(-0.0188958007868)*x_ref[0]+(-0.381850037026)*x_ref[1]**o+(-0.366474181406)*x_ref[1]+(-0.0157097200919)*x_ref[2]**o+(0.750818168768)*x_ref[2]
        ref[0,4,2,0]=(0.962983200785)*x_ref[0]**o+(-0.904671719523)*x_ref[0]+(0.83012715029)*x_ref[1]**o+(0.20786351761)*x_ref[1]+(0.61032830083)*x_ref[2]**o+(0.999068101126)*x_ref[2]
        ref[0,4,2,1]=(0.369225553113)*x_ref[0]**o+(0.370643799864)*x_ref[0]+(0.562458197002)*x_ref[1]**o+(0.864829790167)*x_ref[1]+(-0.507288312866)*x_ref[2]**o+(-0.615656522393)*x_ref[2]
        ref[1,0,0,0]=(0.0208552161791)*x_ref[0]**o+(0.64767817505)*x_ref[0]+(-0.913625079536)*x_ref[1]**o+(0.214341575088)*x_ref[1]+(-0.369358777546)*x_ref[2]**o+(-0.744597104081)*x_ref[2]
        ref[1,0,0,1]=(0.688956912909)*x_ref[0]**o+(0.130856033181)*x_ref[0]+(-0.754570690839)*x_ref[1]**o+(-0.892591311673)*x_ref[1]+(0.121513821098)*x_ref[2]**o+(0.0499651928086)*x_ref[2]
        ref[1,0,1,0]=(0.488371732492)*x_ref[0]**o+(-0.0899662322511)*x_ref[0]+(-0.184052688998)*x_ref[1]**o+(-0.504791015013)*x_ref[1]+(0.939413170478)*x_ref[2]**o+(-0.371392738292)*x_ref[2]
        ref[1,0,1,1]=(-0.634015136517)*x_ref[0]**o+(0.189031198575)*x_ref[0]+(-0.187597065324)*x_ref[1]**o+(0.417035820606)*x_ref[1]+(-0.640693898356)*x_ref[2]**o+(-0.356999823488)*x_ref[2]
        ref[1,0,2,0]=(-0.299739907169)*x_ref[0]**o+(0.97973415315)*x_ref[0]+(0.172728079372)*x_ref[1]**o+(-0.219250264223)*x_ref[1]+(0.0377842392779)*x_ref[2]**o+(0.568605450772)*x_ref[2]
        ref[1,0,2,1]=(0.668822572798)*x_ref[0]**o+(-0.464904703232)*x_ref[0]+(-0.0438179224843)*x_ref[1]**o+(0.972577571624)*x_ref[1]+(0.200473119295)*x_ref[2]**o+(0.769536551131)*x_ref[2]
        ref[1,1,0,0]=(-0.5952291144)*x_ref[0]**o+(0.215912343317)*x_ref[0]+(-0.187871783507)*x_ref[1]**o+(0.000141658119764)*x_ref[1]+(0.652350178895)*x_ref[2]**o+(0.748942594082)*x_ref[2]
        ref[1,1,0,1]=(0.978202121028)*x_ref[0]**o+(-0.270952075217)*x_ref[0]+(0.379211795706)*x_ref[1]**o+(0.570614905933)*x_ref[1]+(0.531348311358)*x_ref[2]**o+(-0.774802192871)*x_ref[2]
        ref[1,1,1,0]=(-0.173449381518)*x_ref[0]**o+(-0.788533061983)*x_ref[0]+(-0.159503791119)*x_ref[1]**o+(-0.182774291708)*x_ref[1]+(0.493985220422)*x_ref[2]**o+(-0.442523611538)*x_ref[2]
        ref[1,1,1,1]=(-0.196642522444)*x_ref[0]**o+(0.353038689926)*x_ref[0]+(0.739133358128)*x_ref[1]**o+(0.384564329164)*x_ref[1]+(0.00724850200888)*x_ref[2]**o+(-0.534791252863)*x_ref[2]
        ref[1,1,2,0]=(-0.742601262273)*x_ref[0]**o+(0.870936220364)*x_ref[0]+(0.173515343981)*x_ref[1]**o+(-0.924818265666)*x_ref[1]+(-0.786225297604)*x_ref[2]**o+(0.723191951357)*x_ref[2]
        ref[1,1,2,1]=(0.584821708581)*x_ref[0]**o+(0.233692617647)*x_ref[0]+(0.917976018627)*x_ref[1]**o+(0.0411719886893)*x_ref[1]+(-0.731518066297)*x_ref[2]**o+(-0.886641339023)*x_ref[2]
        ref[1,2,0,0]=(0.879019029124)*x_ref[0]**o+(0.198224253024)*x_ref[0]+(-0.508879741455)*x_ref[1]**o+(-0.0493693577221)*x_ref[1]+(-0.200204490189)*x_ref[2]**o+(-0.558819359406)*x_ref[2]
        ref[1,2,0,1]=(0.343696269765)*x_ref[0]**o+(-0.694690909901)*x_ref[0]+(0.562857727334)*x_ref[1]**o+(0.170105867024)*x_ref[1]+(0.300442565793)*x_ref[2]**o+(-0.78443966073)*x_ref[2]
        ref[1,2,1,0]=(-0.104227942738)*x_ref[0]**o+(-0.631808216742)*x_ref[0]+(-0.688416357176)*x_ref[1]**o+(0.543474609002)*x_ref[1]+(-0.475674253864)*x_ref[2]**o+(-0.622363009599)*x_ref[2]
        ref[1,2,1,1]=(-0.187993381022)*x_ref[0]**o+(-0.112771930635)*x_ref[0]+(0.231591342365)*x_ref[1]**o+(-0.619972979708)*x_ref[1]+(-0.633470156859)*x_ref[2]**o+(0.726102617311)*x_ref[2]
        ref[1,2,2,0]=(-0.0929871065489)*x_ref[0]**o+(-0.741493547909)*x_ref[0]+(-0.500399374101)*x_ref[1]**o+(0.489221111027)*x_ref[1]+(-0.538850386612)*x_ref[2]**o+(-0.366843628744)*x_ref[2]
        ref[1,2,2,1]=(-0.449265426222)*x_ref[0]**o+(-0.711133901403)*x_ref[0]+(-0.671933617866)*x_ref[1]**o+(-0.75650179505)*x_ref[1]+(-0.441794206618)*x_ref[2]**o+(-0.679799182186)*x_ref[2]
        ref[1,3,0,0]=(0.0794555795393)*x_ref[0]**o+(-0.591466474089)*x_ref[0]+(0.537807349728)*x_ref[1]**o+(-0.0295904103007)*x_ref[1]+(-0.206212961776)*x_ref[2]**o+(0.633837648242)*x_ref[2]
        ref[1,3,0,1]=(0.232707718354)*x_ref[0]**o+(-0.310438878448)*x_ref[0]+(-0.0844732229089)*x_ref[1]**o+(0.507469871796)*x_ref[1]+(-0.391898028012)*x_ref[2]**o+(-0.374582716512)*x_ref[2]
        ref[1,3,1,0]=(-0.166926427703)*x_ref[0]**o+(-0.926298908928)*x_ref[0]+(0.244425001632)*x_ref[1]**o+(0.284615289051)*x_ref[1]+(-0.675054769717)*x_ref[2]**o+(-0.0765552057049)*x_ref[2]
        ref[1,3,1,1]=(-0.469538130468)*x_ref[0]**o+(-0.110820901289)*x_ref[0]+(-0.608743940572)*x_ref[1]**o+(0.0606943645263)*x_ref[1]+(-0.211127814891)*x_ref[2]**o+(-0.0474195944349)*x_ref[2]
        ref[1,3,2,0]=(0.769147852508)*x_ref[0]**o+(-0.500794487532)*x_ref[0]+(-0.0971530930356)*x_ref[1]**o+(0.282178880991)*x_ref[1]+(-0.175125963785)*x_ref[2]**o+(0.0567340005296)*x_ref[2]
        ref[1,3,2,1]=(0.14422447964)*x_ref[0]**o+(-0.9056785988)*x_ref[0]+(-0.962346923857)*x_ref[1]**o+(0.64076498247)*x_ref[1]+(0.77680599777)*x_ref[2]**o+(0.450703438151)*x_ref[2]
        ref[1,4,0,0]=(0.421930869494)*x_ref[0]**o+(0.889038749248)*x_ref[0]+(-0.989075156799)*x_ref[1]**o+(-0.614721833409)*x_ref[1]+(0.112500578762)*x_ref[2]**o+(-0.47083438608)*x_ref[2]
        ref[1,4,0,1]=(-0.0180470731329)*x_ref[0]**o+(-0.47539878993)*x_ref[0]+(0.317084518451)*x_ref[1]**o+(0.237846416515)*x_ref[1]+(0.375600671255)*x_ref[2]**o+(0.66276020876)*x_ref[2]
        ref[1,4,1,0]=(0.125983672606)*x_ref[0]**o+(0.595653397427)*x_ref[0]+(0.31474692021)*x_ref[1]**o+(0.234488164569)*x_ref[1]+(0.618714006094)*x_ref[2]**o+(-0.160776384119)*x_ref[2]
        ref[1,4,1,1]=(0.951917864448)*x_ref[0]**o+(-0.33473954392)*x_ref[0]+(-0.0378707336313)*x_ref[1]**o+(-0.238158748773)*x_ref[1]+(0.574999874285)*x_ref[2]**o+(0.232302306417)*x_ref[2]
        ref[1,4,2,0]=(0.505450486119)*x_ref[0]**o+(-0.370259193476)*x_ref[0]+(-0.663053563021)*x_ref[1]**o+(0.669872208578)*x_ref[1]+(0.931794239064)*x_ref[2]**o+(0.133184889523)*x_ref[2]
        ref[1,4,2,1]=(-0.937476724928)*x_ref[0]**o+(0.110843923089)*x_ref[0]+(-0.425245370966)*x_ref[1]**o+(0.409116775141)*x_ref[1]+(-0.273590511289)*x_ref[2]**o+(0.441443512397)*x_ref[2]
        ref[2,0,0,0]=(0.0599017312855)*x_ref[0]**o+(-0.941401999676)*x_ref[0]+(0.379283945566)*x_ref[1]**o+(-0.336352358178)*x_ref[1]+(-0.514408042486)*x_ref[2]**o+(-0.169011791163)*x_ref[2]
        ref[2,0,0,1]=(-0.269712116236)*x_ref[0]**o+(-0.349623235155)*x_ref[0]+(0.522466035838)*x_ref[1]**o+(0.427111359806)*x_ref[1]+(-0.0296663851565)*x_ref[2]**o+(-0.0191832326161)*x_ref[2]
        ref[2,0,1,0]=(0.14892181923)*x_ref[0]**o+(0.269460608852)*x_ref[0]+(-0.0343896163306)*x_ref[1]**o+(0.590044957948)*x_ref[1]+(0.946719737501)*x_ref[2]**o+(0.315742744827)*x_ref[2]
        ref[2,0,1,1]=(-0.388655118673)*x_ref[0]**o+(-0.00180659903075)*x_ref[0]+(0.969087798155)*x_ref[1]**o+(0.86635225912)*x_ref[1]+(-0.761434218549)*x_ref[2]**o+(-0.323637696199)*x_ref[2]
        ref[2,0,2,0]=(0.588641788156)*x_ref[0]**o+(-0.00912456965621)*x_ref[0]+(-0.0266085527484)*x_ref[1]**o+(0.0598075913127)*x_ref[1]+(-0.658095450927)*x_ref[2]**o+(0.110978909962)*x_ref[2]
        ref[2,0,2,1]=(0.139641588755)*x_ref[0]**o+(0.593809622136)*x_ref[0]+(-0.259839406352)*x_ref[1]**o+(0.848541675808)*x_ref[1]+(-0.860886035055)*x_ref[2]**o+(-0.489752174489)*x_ref[2]
        ref[2,1,0,0]=(0.850328373883)*x_ref[0]**o+(0.675383216776)*x_ref[0]+(-0.117397818856)*x_ref[1]**o+(-0.344801013716)*x_ref[1]+(-0.887091513121)*x_ref[2]**o+(0.584872468856)*x_ref[2]
        ref[2,1,0,1]=(-0.745901868498)*x_ref[0]**o+(-0.407149004368)*x_ref[0]+(-0.182253232897)*x_ref[1]**o+(-0.203000014779)*x_ref[1]+(-0.172710156372)*x_ref[2]**o+(0.127673047341)*x_ref[2]
        ref[2,1,1,0]=(0.655182501192)*x_ref[0]**o+(-0.17668945533)*x_ref[0]+(0.717711469841)*x_ref[1]**o+(0.595777555767)*x_ref[1]+(-0.974184895311)*x_ref[2]**o+(-0.0299700820305)*x_ref[2]
        ref[2,1,1,1]=(0.707793528102)*x_ref[0]**o+(-0.0624977281364)*x_ref[0]+(0.623777123073)*x_ref[1]**o+(-0.794217090978)*x_ref[1]+(0.542320017675)*x_ref[2]**o+(0.903852498061)*x_ref[2]
        ref[2,1,2,0]=(0.713295120131)*x_ref[0]**o+(-0.174892054913)*x_ref[0]+(-0.251138558622)*x_ref[1]**o+(-0.77735156875)*x_ref[1]+(-0.94176928221)*x_ref[2]**o+(-0.0244708586622)*x_ref[2]
        ref[2,1,2,1]=(0.933976789519)*x_ref[0]**o+(0.90177842055)*x_ref[0]+(-0.343254953267)*x_ref[1]**o+(0.713448014636)*x_ref[1]+(-0.468875743251)*x_ref[2]**o+(0.350869684481)*x_ref[2]
        ref[2,2,0,0]=(0.390367713904)*x_ref[0]**o+(-0.601608385293)*x_ref[0]+(-0.757340195841)*x_ref[1]**o+(0.360436452755)*x_ref[1]+(0.291087135717)*x_ref[2]**o+(0.316481151314)*x_ref[2]
        ref[2,2,0,1]=(0.111600306678)*x_ref[0]**o+(0.823704129888)*x_ref[0]+(0.682856997317)*x_ref[1]**o+(-0.523736197366)*x_ref[1]+(-0.100121183783)*x_ref[2]**o+(-0.666230008262)*x_ref[2]
        ref[2,2,1,0]=(0.173360884658)*x_ref[0]**o+(0.949511779713)*x_ref[0]+(0.259889008248)*x_ref[1]**o+(-0.942214191047)*x_ref[1]+(-0.48690925565)*x_ref[2]**o+(-0.264798193102)*x_ref[2]
        ref[2,2,1,1]=(0.483759255276)*x_ref[0]**o+(-0.657669281632)*x_ref[0]+(-0.154764754082)*x_ref[1]**o+(-0.881550854822)*x_ref[1]+(0.523812678208)*x_ref[2]**o+(-0.979914768652)*x_ref[2]
        ref[2,2,2,0]=(0.137127593491)*x_ref[0]**o+(-0.104908009627)*x_ref[0]+(0.579522653005)*x_ref[1]**o+(0.844822805269)*x_ref[1]+(0.0328079119318)*x_ref[2]**o+(-0.26897713517)*x_ref[2]
        ref[2,2,2,1]=(0.377116909347)*x_ref[0]**o+(0.752578358605)*x_ref[0]+(-0.467992979597)*x_ref[1]**o+(0.254761405884)*x_ref[1]+(-0.391559118991)*x_ref[2]**o+(-0.61268413168)*x_ref[2]
        ref[2,3,0,0]=(-0.989947766953)*x_ref[0]**o+(0.830717454791)*x_ref[0]+(-0.473161731716)*x_ref[1]**o+(0.625028238242)*x_ref[1]+(-0.546708564025)*x_ref[2]**o+(-0.691027642154)*x_ref[2]
        ref[2,3,0,1]=(0.650495990975)*x_ref[0]**o+(-0.654863347162)*x_ref[0]+(0.638342908241)*x_ref[1]**o+(-0.0485838638503)*x_ref[1]+(-0.632091721289)*x_ref[2]**o+(0.791203973276)*x_ref[2]
        ref[2,3,1,0]=(0.266156252436)*x_ref[0]**o+(0.272153723938)*x_ref[0]+(-0.159895797204)*x_ref[1]**o+(-0.640350200995)*x_ref[1]+(0.990601940876)*x_ref[2]**o+(-0.0816835237084)*x_ref[2]
        ref[2,3,1,1]=(0.562807345614)*x_ref[0]**o+(0.544038968862)*x_ref[0]+(0.878928928869)*x_ref[1]**o+(0.404794709914)*x_ref[1]+(0.269928450336)*x_ref[2]**o+(0.869832826526)*x_ref[2]
        ref[2,3,2,0]=(-0.899170106614)*x_ref[0]**o+(-0.031251129011)*x_ref[0]+(0.0696989801411)*x_ref[1]**o+(0.89839882525)*x_ref[1]+(0.99905365333)*x_ref[2]**o+(-0.856781333049)*x_ref[2]
        ref[2,3,2,1]=(0.980091885122)*x_ref[0]**o+(-0.12832421228)*x_ref[0]+(0.0798899702085)*x_ref[1]**o+(-0.729861980386)*x_ref[1]+(-0.970268210161)*x_ref[2]**o+(0.319025854228)*x_ref[2]
        ref[2,4,0,0]=(0.891703788308)*x_ref[0]**o+(-0.0724026917568)*x_ref[0]+(-0.74794796925)*x_ref[1]**o+(-0.339878454118)*x_ref[1]+(0.413228256742)*x_ref[2]**o+(-0.371024671724)*x_ref[2]
        ref[2,4,0,1]=(0.640892618319)*x_ref[0]**o+(-0.44510403082)*x_ref[0]+(-0.100468205238)*x_ref[1]**o+(0.923901932578)*x_ref[1]+(0.731059415968)*x_ref[2]**o+(-0.609224901759)*x_ref[2]
        ref[2,4,1,0]=(0.130333662915)*x_ref[0]**o+(0.394752475547)*x_ref[0]+(-0.11866957093)*x_ref[1]**o+(0.980649245967)*x_ref[1]+(-0.602353311004)*x_ref[2]**o+(0.406199522673)*x_ref[2]
        ref[2,4,1,1]=(0.0665437323779)*x_ref[0]**o+(0.952105166153)*x_ref[0]+(0.488465603925)*x_ref[1]**o+(0.0178900500949)*x_ref[1]+(0.206584014652)*x_ref[2]**o+(0.446123239912)*x_ref[2]
        ref[2,4,2,0]=(-0.792540868222)*x_ref[0]**o+(0.0265488816348)*x_ref[0]+(-0.693338065664)*x_ref[1]**o+(-0.682629782298)*x_ref[1]+(-0.778972053935)*x_ref[2]**o+(-0.0274648446373)*x_ref[2]
        ref[2,4,2,1]=(0.916389263123)*x_ref[0]**o+(-0.662528447313)*x_ref[0]+(-0.0703995406002)*x_ref[1]**o+(0.385180574963)*x_ref[1]+(-0.855872398416)*x_ref[2]**o+(0.5530510507)*x_ref[2]
        ref[3,0,0,0]=(-0.306821143583)*x_ref[0]**o+(-0.962173177288)*x_ref[0]+(0.993016856406)*x_ref[1]**o+(-0.579698238302)*x_ref[1]+(0.571895736521)*x_ref[2]**o+(-0.88966756316)*x_ref[2]
        ref[3,0,0,1]=(0.719926546559)*x_ref[0]**o+(-0.0858941607816)*x_ref[0]+(0.807242676458)*x_ref[1]**o+(-0.190440184356)*x_ref[1]+(-0.136927444751)*x_ref[2]**o+(0.869886135543)*x_ref[2]
        ref[3,0,1,0]=(0.892148659939)*x_ref[0]**o+(-0.482479220378)*x_ref[0]+(0.503671753254)*x_ref[1]**o+(-0.552769467575)*x_ref[1]+(0.138159512264)*x_ref[2]**o+(0.102674771614)*x_ref[2]
        ref[3,0,1,1]=(0.185256299506)*x_ref[0]**o+(-0.369351326706)*x_ref[0]+(0.220953483691)*x_ref[1]**o+(-0.0131034510397)*x_ref[1]+(0.956037877433)*x_ref[2]**o+(0.568289950328)*x_ref[2]
        ref[3,0,2,0]=(0.045864662475)*x_ref[0]**o+(0.0183434079897)*x_ref[0]+(-0.428111425773)*x_ref[1]**o+(0.254667250574)*x_ref[1]+(-0.983322740032)*x_ref[2]**o+(0.877555405709)*x_ref[2]
        ref[3,0,2,1]=(-0.408521811841)*x_ref[0]**o+(0.00640043590699)*x_ref[0]+(-0.105754378317)*x_ref[1]**o+(-0.952486645506)*x_ref[1]+(-0.725225188014)*x_ref[2]**o+(-0.205556627888)*x_ref[2]
        ref[3,1,0,0]=(-0.944493178271)*x_ref[0]**o+(0.810364339849)*x_ref[0]+(0.705003782307)*x_ref[1]**o+(-0.880195742246)*x_ref[1]+(-0.843196216793)*x_ref[2]**o+(-0.971449880924)*x_ref[2]
        ref[3,1,0,1]=(0.150616114959)*x_ref[0]**o+(-0.717321007987)*x_ref[0]+(0.194099301116)*x_ref[1]**o+(-0.0345362551173)*x_ref[1]+(0.375587089675)*x_ref[2]**o+(-0.409625637147)*x_ref[2]
        ref[3,1,1,0]=(0.0802340794764)*x_ref[0]**o+(0.0153666337202)*x_ref[0]+(-0.782772322072)*x_ref[1]**o+(-0.0965866811611)*x_ref[1]+(0.725231231559)*x_ref[2]**o+(-0.452016239379)*x_ref[2]
        ref[3,1,1,1]=(0.546956847386)*x_ref[0]**o+(0.839322197764)*x_ref[0]+(-0.735023211892)*x_ref[1]**o+(-0.41772545319)*x_ref[1]+(-0.843610604975)*x_ref[2]**o+(-0.145546519925)*x_ref[2]
        ref[3,1,2,0]=(0.729170600381)*x_ref[0]**o+(-0.551244794813)*x_ref[0]+(0.846396830038)*x_ref[1]**o+(-0.759556129789)*x_ref[1]+(0.136640189225)*x_ref[2]**o+(-0.379654525677)*x_ref[2]
        ref[3,1,2,1]=(-0.953857643507)*x_ref[0]**o+(-0.304074878681)*x_ref[0]+(0.472716648295)*x_ref[1]**o+(-0.179292020611)*x_ref[1]+(0.909646409116)*x_ref[2]**o+(-0.149337148625)*x_ref[2]
        ref[3,2,0,0]=(-0.165651506554)*x_ref[0]**o+(0.410286556434)*x_ref[0]+(-0.385291761561)*x_ref[1]**o+(-0.932290142267)*x_ref[1]+(0.562269759333)*x_ref[2]**o+(-0.516760168836)*x_ref[2]
        ref[3,2,0,1]=(-0.874391348276)*x_ref[0]**o+(-0.626614642393)*x_ref[0]+(0.135326235872)*x_ref[1]**o+(-0.169497542734)*x_ref[1]+(0.453312712391)*x_ref[2]**o+(0.699294330345)*x_ref[2]
        ref[3,2,1,0]=(0.30374985921)*x_ref[0]**o+(-0.66286753305)*x_ref[0]+(-0.504388407872)*x_ref[1]**o+(-0.0864778873713)*x_ref[1]+(0.7780825189)*x_ref[2]**o+(0.607545865231)*x_ref[2]
        ref[3,2,1,1]=(0.323217898368)*x_ref[0]**o+(0.256285070904)*x_ref[0]+(0.32326826477)*x_ref[1]**o+(-0.983183590335)*x_ref[1]+(0.530605958132)*x_ref[2]**o+(-0.109640854019)*x_ref[2]
        ref[3,2,2,0]=(0.587862067621)*x_ref[0]**o+(-0.951653317735)*x_ref[0]+(0.089519431923)*x_ref[1]**o+(-0.780330592384)*x_ref[1]+(-0.0483676475341)*x_ref[2]**o+(0.766538955993)*x_ref[2]
        ref[3,2,2,1]=(-0.957533418498)*x_ref[0]**o+(-0.681231272445)*x_ref[0]+(-0.0918289718767)*x_ref[1]**o+(0.38725634145)*x_ref[1]+(0.26451874534)*x_ref[2]**o+(-0.901046807078)*x_ref[2]
        ref[3,3,0,0]=(0.604215052532)*x_ref[0]**o+(-0.0188225127963)*x_ref[0]+(-0.544365413309)*x_ref[1]**o+(-0.837547618266)*x_ref[1]+(0.506665562924)*x_ref[2]**o+(0.868580855034)*x_ref[2]
        ref[3,3,0,1]=(0.307620045896)*x_ref[0]**o+(-0.941702619274)*x_ref[0]+(0.467078433616)*x_ref[1]**o+(-0.525971373276)*x_ref[1]+(-0.438681578224)*x_ref[2]**o+(-0.865538708837)*x_ref[2]
        ref[3,3,1,0]=(-0.128299566207)*x_ref[0]**o+(0.0526714707626)*x_ref[0]+(-0.194464943133)*x_ref[1]**o+(0.848343510106)*x_ref[1]+(-0.843042290362)*x_ref[2]**o+(-0.384441684551)*x_ref[2]
        ref[3,3,1,1]=(-0.0202567344895)*x_ref[0]**o+(0.919037490786)*x_ref[0]+(0.953616123775)*x_ref[1]**o+(-0.359010584029)*x_ref[1]+(-0.892651116514)*x_ref[2]**o+(0.900116594426)*x_ref[2]
        ref[3,3,2,0]=(-0.196701204528)*x_ref[0]**o+(0.173992581268)*x_ref[0]+(0.0304200566544)*x_ref[1]**o+(-0.562311121578)*x_ref[1]+(-0.349902986645)*x_ref[2]**o+(0.137392211876)*x_ref[2]
        ref[3,3,2,1]=(-0.264756448582)*x_ref[0]**o+(0.104810865018)*x_ref[0]+(-0.550559232481)*x_ref[1]**o+(-0.621292330017)*x_ref[1]+(-0.697096182063)*x_ref[2]**o+(-0.137686685739)*x_ref[2]
        ref[3,4,0,0]=(-0.0500780621237)*x_ref[0]**o+(-0.842196022641)*x_ref[0]+(-0.375688956182)*x_ref[1]**o+(0.293383514141)*x_ref[1]+(-0.679863519997)*x_ref[2]**o+(-0.108799395703)*x_ref[2]
        ref[3,4,0,1]=(-0.825663711773)*x_ref[0]**o+(0.0811924383936)*x_ref[0]+(0.658753764102)*x_ref[1]**o+(0.213557161034)*x_ref[1]+(0.048778548639)*x_ref[2]**o+(0.308220587221)*x_ref[2]
        ref[3,4,1,0]=(-0.0366332011016)*x_ref[0]**o+(-0.935060469197)*x_ref[0]+(0.52021729799)*x_ref[1]**o+(0.270580190954)*x_ref[1]+(0.690657153532)*x_ref[2]**o+(0.572985687766)*x_ref[2]
        ref[3,4,1,1]=(-0.358041857957)*x_ref[0]**o+(0.151420239879)*x_ref[0]+(0.578841565624)*x_ref[1]**o+(0.379085642891)*x_ref[1]+(0.231447762893)*x_ref[2]**o+(-0.494885835203)*x_ref[2]
        ref[3,4,2,0]=(0.689670410742)*x_ref[0]**o+(-0.469898987295)*x_ref[0]+(-0.617664742358)*x_ref[1]**o+(-0.92098566662)*x_ref[1]+(0.535497980872)*x_ref[2]**o+(-0.660036711933)*x_ref[2]
        ref[3,4,2,1]=(-0.443846183672)*x_ref[0]**o+(-0.0606185138497)*x_ref[0]+(0.902723111093)*x_ref[1]**o+(-0.201894548349)*x_ref[1]+(0.650998336304)*x_ref[2]**o+(-0.936010399236)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromSymbol_ReducedSolution_rank0(self):
      """
      tests interpolate for rank 0 Symbol onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedSolution onto ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(),dim=dim)
      s=Data(0,(),w)
      ref=Data(0,(),w_ref)
      if dim==2:
        s=(0.14488970011)*x[0]+(-0.430489267095)*x[1]
        ref=(0.14488970011)*x_ref[0]+(-0.430489267095)*x_ref[1]
      else:
        s=(-0.0442386353051)*x[0]+(0.0394615944862)*x[1]+(-1.29904883856)*x[2]
        ref=(-0.0442386353051)*x_ref[0]+(0.0394615944862)*x_ref[1]+(-1.29904883856)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromSymbol_ReducedSolution_rank1(self):
      """
      tests interpolate for rank 1 Symbol onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedSolution onto ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(2,),dim=dim)
      s=Data(0,(2,),w)
      ref=Data(0,(2,),w_ref)
      if dim==2:
        s[0]=(0.0426949725928)*x[0]+(0.796580332975)*x[1]
        s[1]=(0.297643604255)*x[0]+(0.501403166746)*x[1]
        ref[0]=(0.0426949725928)*x[0]+(0.796580332975)*x[1]
        ref[1]=(0.297643604255)*x[0]+(0.501403166746)*x[1]
      else:
        s[0]=(0.820713117949)*x[0]+(0.0164537336201)*x[1]+(-0.130931974587)*x[2]
        s[1]=(-0.155153784636)*x[0]+(-0.48729229521)*x[1]+(-0.875514015614)*x[2]
        ref[0]=(0.820713117949)*x[0]+(0.0164537336201)*x[1]+(-0.130931974587)*x[2]
        ref[1]=(-0.155153784636)*x[0]+(-0.48729229521)*x[1]+(-0.875514015614)*x[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromSymbol_ReducedSolution_rank2(self):
      """
      tests interpolate for rank 2 Symbol onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedSolution onto ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5),dim=dim)
      s=Data(0,(4, 5),w)
      ref=Data(0,(4, 5),w_ref)
      if dim==2:
        s[0,0]=(0.918593737351)*x[0]+(-0.183298779344)*x[1]
        s[0,1]=(1.01407474954)*x[0]+(-1.14888338084)*x[1]
        s[0,2]=(-0.0702606564619)*x[0]+(-0.104586136782)*x[1]
        s[0,3]=(-1.00520814193)*x[0]+(-0.959854266357)*x[1]
        s[0,4]=(-1.11463618446)*x[0]+(-0.794825170571)*x[1]
        s[1,0]=(-1.14418607702)*x[0]+(-0.273235396058)*x[1]
        s[1,1]=(0.641683109978)*x[0]+(0.685097028758)*x[1]
        s[1,2]=(0.833533353805)*x[0]+(-0.769590460713)*x[1]
        s[1,3]=(1.44918354254)*x[0]+(0.78941323689)*x[1]
        s[1,4]=(0.968133426719)*x[0]+(-0.923562755675)*x[1]
        s[2,0]=(1.58053224399)*x[0]+(0.600967207116)*x[1]
        s[2,1]=(1.04054361996)*x[0]+(-1.71519167959)*x[1]
        s[2,2]=(-1.55202351352)*x[0]+(-1.06650593034)*x[1]
        s[2,3]=(0.894986308301)*x[0]+(-0.767078277329)*x[1]
        s[2,4]=(-0.804853431103)*x[0]+(-0.44572378652)*x[1]
        s[3,0]=(0.90347844754)*x[0]+(0.357148881796)*x[1]
        s[3,1]=(0.0552482783049)*x[0]+(-0.652692619264)*x[1]
        s[3,2]=(-0.251809701815)*x[0]+(1.89155992551)*x[1]
        s[3,3]=(0.43181695535)*x[0]+(-0.228997029489)*x[1]
        s[3,4]=(-0.522831785326)*x[0]+(-0.372437316342)*x[1]
        ref[0,0]=(0.918593737351)*x_ref[0]+(-0.183298779344)*x_ref[1]
        ref[0,1]=(1.01407474954)*x_ref[0]+(-1.14888338084)*x_ref[1]
        ref[0,2]=(-0.0702606564619)*x_ref[0]+(-0.104586136782)*x_ref[1]
        ref[0,3]=(-1.00520814193)*x_ref[0]+(-0.959854266357)*x_ref[1]
        ref[0,4]=(-1.11463618446)*x_ref[0]+(-0.794825170571)*x_ref[1]
        ref[1,0]=(-1.14418607702)*x_ref[0]+(-0.273235396058)*x_ref[1]
        ref[1,1]=(0.641683109978)*x_ref[0]+(0.685097028758)*x_ref[1]
        ref[1,2]=(0.833533353805)*x_ref[0]+(-0.769590460713)*x_ref[1]
        ref[1,3]=(1.44918354254)*x_ref[0]+(0.78941323689)*x_ref[1]
        ref[1,4]=(0.968133426719)*x_ref[0]+(-0.923562755675)*x_ref[1]
        ref[2,0]=(1.58053224399)*x_ref[0]+(0.600967207116)*x_ref[1]
        ref[2,1]=(1.04054361996)*x_ref[0]+(-1.71519167959)*x_ref[1]
        ref[2,2]=(-1.55202351352)*x_ref[0]+(-1.06650593034)*x_ref[1]
        ref[2,3]=(0.894986308301)*x_ref[0]+(-0.767078277329)*x_ref[1]
        ref[2,4]=(-0.804853431103)*x_ref[0]+(-0.44572378652)*x_ref[1]
        ref[3,0]=(0.90347844754)*x_ref[0]+(0.357148881796)*x_ref[1]
        ref[3,1]=(0.0552482783049)*x_ref[0]+(-0.652692619264)*x_ref[1]
        ref[3,2]=(-0.251809701815)*x_ref[0]+(1.89155992551)*x_ref[1]
        ref[3,3]=(0.43181695535)*x_ref[0]+(-0.228997029489)*x_ref[1]
        ref[3,4]=(-0.522831785326)*x_ref[0]+(-0.372437316342)*x_ref[1]
      else:
        s[0,0]=(0.221916152348)*x[0]+(1.46835470229)*x[1]+(0.300487757142)*x[2]
        s[0,1]=(-0.15334206717)*x[0]+(0.506590749852)*x[1]+(0.477134329524)*x[2]
        s[0,2]=(0.786821788538)*x[0]+(-0.0141797716133)*x[1]+(-1.67421275557)*x[2]
        s[0,3]=(-1.14438881368)*x[0]+(-0.621323583056)*x[1]+(-0.0198865298096)*x[2]
        s[0,4]=(-0.195154511305)*x[0]+(-0.237596745083)*x[1]+(-0.36680113021)*x[2]
        s[1,0]=(-0.0476767967515)*x[0]+(-1.19950396856)*x[1]+(0.12823672742)*x[2]
        s[1,1]=(-0.420573984429)*x[0]+(0.100192190413)*x[1]+(1.30995115077)*x[2]
        s[1,2]=(1.34087691694)*x[0]+(-0.151927765846)*x[1]+(0.417185746786)*x[2]
        s[1,3]=(-0.383933595451)*x[0]+(0.208340861893)*x[1]+(0.947878917669)*x[2]
        s[1,4]=(-0.143686438189)*x[0]+(-0.00211348684153)*x[1]+(-0.859362566827)*x[2]
        s[2,0]=(0.447423746179)*x[0]+(-0.636241203952)*x[1]+(-0.0949114725339)*x[2]
        s[2,1]=(-0.0413740104938)*x[0]+(0.587001644427)*x[1]+(-0.544886520536)*x[2]
        s[2,2]=(-1.07017617383)*x[0]+(-0.762147671415)*x[1]+(-0.248360379513)*x[2]
        s[2,3]=(0.391720300962)*x[0]+(1.6521113321)*x[1]+(-0.408205550425)*x[2]
        s[2,4]=(1.2369249321)*x[0]+(0.952522466347)*x[1]+(-0.404645424067)*x[2]
        s[3,0]=(0.62410800627)*x[0]+(-0.308929187117)*x[1]+(-0.703796873161)*x[2]
        s[3,1]=(0.676590956976)*x[0]+(-1.39642714827)*x[1]+(0.198267797885)*x[2]
        s[3,2]=(-0.202804694134)*x[0]+(0.187577851273)*x[1]+(-0.700960966206)*x[2]
        s[3,3]=(-0.799833564134)*x[0]+(-0.455781630028)*x[1]+(0.453451623154)*x[2]
        s[3,4]=(-0.660373484066)*x[0]+(-1.55472528981)*x[1]+(0.0751348530851)*x[2]
        ref[0,0]=(0.221916152348)*x_ref[0]+(1.46835470229)*x_ref[1]+(0.300487757142)*x_ref[2]
        ref[0,1]=(-0.15334206717)*x_ref[0]+(0.506590749852)*x_ref[1]+(0.477134329524)*x_ref[2]
        ref[0,2]=(0.786821788538)*x_ref[0]+(-0.0141797716133)*x_ref[1]+(-1.67421275557)*x_ref[2]
        ref[0,3]=(-1.14438881368)*x_ref[0]+(-0.621323583056)*x_ref[1]+(-0.0198865298096)*x_ref[2]
        ref[0,4]=(-0.195154511305)*x_ref[0]+(-0.237596745083)*x_ref[1]+(-0.36680113021)*x_ref[2]
        ref[1,0]=(-0.0476767967515)*x_ref[0]+(-1.19950396856)*x_ref[1]+(0.12823672742)*x_ref[2]
        ref[1,1]=(-0.420573984429)*x_ref[0]+(0.100192190413)*x_ref[1]+(1.30995115077)*x_ref[2]
        ref[1,2]=(1.34087691694)*x_ref[0]+(-0.151927765846)*x_ref[1]+(0.417185746786)*x_ref[2]
        ref[1,3]=(-0.383933595451)*x_ref[0]+(0.208340861893)*x_ref[1]+(0.947878917669)*x_ref[2]
        ref[1,4]=(-0.143686438189)*x_ref[0]+(-0.00211348684153)*x_ref[1]+(-0.859362566827)*x_ref[2]
        ref[2,0]=(0.447423746179)*x_ref[0]+(-0.636241203952)*x_ref[1]+(-0.0949114725339)*x_ref[2]
        ref[2,1]=(-0.0413740104938)*x_ref[0]+(0.587001644427)*x_ref[1]+(-0.544886520536)*x_ref[2]
        ref[2,2]=(-1.07017617383)*x_ref[0]+(-0.762147671415)*x_ref[1]+(-0.248360379513)*x_ref[2]
        ref[2,3]=(0.391720300962)*x_ref[0]+(1.6521113321)*x_ref[1]+(-0.408205550425)*x_ref[2]
        ref[2,4]=(1.2369249321)*x_ref[0]+(0.952522466347)*x_ref[1]+(-0.404645424067)*x_ref[2]
        ref[3,0]=(0.62410800627)*x_ref[0]+(-0.308929187117)*x_ref[1]+(-0.703796873161)*x_ref[2]
        ref[3,1]=(0.676590956976)*x_ref[0]+(-1.39642714827)*x_ref[1]+(0.198267797885)*x_ref[2]
        ref[3,2]=(-0.202804694134)*x_ref[0]+(0.187577851273)*x_ref[1]+(-0.700960966206)*x_ref[2]
        ref[3,3]=(-0.799833564134)*x_ref[0]+(-0.455781630028)*x_ref[1]+(0.453451623154)*x_ref[2]
        ref[3,4]=(-0.660373484066)*x_ref[0]+(-1.55472528981)*x_ref[1]+(0.0751348530851)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromSymbol_ReducedSolution_rank3(self):
      """
      tests interpolate for rank 3 Symbol onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedSolution onto ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(6, 2, 2),dim=dim)
      s=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2),w_ref)
      if dim==2:
        s[0,0,0]=(-1.78813432343)*x[0]+(1.60161751633)*x[1]
        s[0,0,1]=(-1.49849771585)*x[0]+(-0.635435863643)*x[1]
        s[0,1,0]=(-0.906731346391)*x[0]+(0.533678119416)*x[1]
        s[0,1,1]=(-0.792521183513)*x[0]+(0.260998720452)*x[1]
        s[1,0,0]=(-0.710279314945)*x[0]+(0.38834997642)*x[1]
        s[1,0,1]=(-0.168938079833)*x[0]+(-0.502425332838)*x[1]
        s[1,1,0]=(-0.358550156297)*x[0]+(-0.650005639384)*x[1]
        s[1,1,1]=(-1.57793968521)*x[0]+(-0.46781871273)*x[1]
        s[2,0,0]=(0.0112676061889)*x[0]+(0.925017077862)*x[1]
        s[2,0,1]=(-0.109253916051)*x[0]+(-0.38189031862)*x[1]
        s[2,1,0]=(0.777175068327)*x[0]+(1.00196875887)*x[1]
        s[2,1,1]=(-0.27343070108)*x[0]+(-0.11181493242)*x[1]
        s[3,0,0]=(0.321226287504)*x[0]+(0.173667688124)*x[1]
        s[3,0,1]=(1.67405481183)*x[0]+(-0.981404244518)*x[1]
        s[3,1,0]=(0.129874894836)*x[0]+(0.341748165587)*x[1]
        s[3,1,1]=(-0.545737004242)*x[0]+(-1.20936137818)*x[1]
        s[4,0,0]=(0.389007738117)*x[0]+(-0.435046969057)*x[1]
        s[4,0,1]=(1.50296802873)*x[0]+(-0.878616436543)*x[1]
        s[4,1,0]=(1.16067694875)*x[0]+(-0.954317497512)*x[1]
        s[4,1,1]=(1.14634534638)*x[0]+(-0.63981762551)*x[1]
        s[5,0,0]=(0.803750299983)*x[0]+(-0.383611041681)*x[1]
        s[5,0,1]=(0.51863884687)*x[0]+(1.32891347835)*x[1]
        s[5,1,0]=(-0.0775425248593)*x[0]+(-0.0375631918508)*x[1]
        s[5,1,1]=(1.57167118663)*x[0]+(-0.649960686486)*x[1]
        ref[0,0,0]=(-1.78813432343)*x_ref[0]+(1.60161751633)*x_ref[1]
        ref[0,0,1]=(-1.49849771585)*x_ref[0]+(-0.635435863643)*x_ref[1]
        ref[0,1,0]=(-0.906731346391)*x_ref[0]+(0.533678119416)*x_ref[1]
        ref[0,1,1]=(-0.792521183513)*x_ref[0]+(0.260998720452)*x_ref[1]
        ref[1,0,0]=(-0.710279314945)*x_ref[0]+(0.38834997642)*x_ref[1]
        ref[1,0,1]=(-0.168938079833)*x_ref[0]+(-0.502425332838)*x_ref[1]
        ref[1,1,0]=(-0.358550156297)*x_ref[0]+(-0.650005639384)*x_ref[1]
        ref[1,1,1]=(-1.57793968521)*x_ref[0]+(-0.46781871273)*x_ref[1]
        ref[2,0,0]=(0.0112676061889)*x_ref[0]+(0.925017077862)*x_ref[1]
        ref[2,0,1]=(-0.109253916051)*x_ref[0]+(-0.38189031862)*x_ref[1]
        ref[2,1,0]=(0.777175068327)*x_ref[0]+(1.00196875887)*x_ref[1]
        ref[2,1,1]=(-0.27343070108)*x_ref[0]+(-0.11181493242)*x_ref[1]
        ref[3,0,0]=(0.321226287504)*x_ref[0]+(0.173667688124)*x_ref[1]
        ref[3,0,1]=(1.67405481183)*x_ref[0]+(-0.981404244518)*x_ref[1]
        ref[3,1,0]=(0.129874894836)*x_ref[0]+(0.341748165587)*x_ref[1]
        ref[3,1,1]=(-0.545737004242)*x_ref[0]+(-1.20936137818)*x_ref[1]
        ref[4,0,0]=(0.389007738117)*x_ref[0]+(-0.435046969057)*x_ref[1]
        ref[4,0,1]=(1.50296802873)*x_ref[0]+(-0.878616436543)*x_ref[1]
        ref[4,1,0]=(1.16067694875)*x_ref[0]+(-0.954317497512)*x_ref[1]
        ref[4,1,1]=(1.14634534638)*x_ref[0]+(-0.63981762551)*x_ref[1]
        ref[5,0,0]=(0.803750299983)*x_ref[0]+(-0.383611041681)*x_ref[1]
        ref[5,0,1]=(0.51863884687)*x_ref[0]+(1.32891347835)*x_ref[1]
        ref[5,1,0]=(-0.0775425248593)*x_ref[0]+(-0.0375631918508)*x_ref[1]
        ref[5,1,1]=(1.57167118663)*x_ref[0]+(-0.649960686486)*x_ref[1]
      else:
        s[0,0,0]=(1.00224477774)*x[0]+(0.276541980132)*x[1]+(-1.61268557504)*x[2]
        s[0,0,1]=(0.182197948486)*x[0]+(-0.314887566213)*x[1]+(0.172753958549)*x[2]
        s[0,1,0]=(-0.225255532228)*x[0]+(1.64823628561)*x[1]+(0.00289115479583)*x[2]
        s[0,1,1]=(-0.0081734432798)*x[0]+(-1.40127403863)*x[1]+(-0.878765016785)*x[2]
        s[1,0,0]=(-0.88802398859)*x[0]+(0.980539857729)*x[1]+(0.106008000545)*x[2]
        s[1,0,1]=(0.413338006112)*x[0]+(0.238889884193)*x[1]+(0.216982176844)*x[2]
        s[1,1,0]=(-0.375359689292)*x[0]+(0.656302411812)*x[1]+(-1.13744732251)*x[2]
        s[1,1,1]=(-0.768638146497)*x[0]+(0.382131491456)*x[1]+(-0.0473187661226)*x[2]
        s[2,0,0]=(0.727474830988)*x[0]+(-1.15897383904)*x[1]+(1.03107954393)*x[2]
        s[2,0,1]=(-0.565279703477)*x[0]+(0.952275155472)*x[1]+(1.72082557166)*x[2]
        s[2,1,0]=(-0.504951356904)*x[0]+(0.0945851323338)*x[1]+(-1.18137582843)*x[2]
        s[2,1,1]=(0.441559000376)*x[0]+(0.808481091433)*x[1]+(-0.185790180692)*x[2]
        s[3,0,0]=(1.097525477)*x[0]+(1.09598373235)*x[1]+(-0.225656362018)*x[2]
        s[3,0,1]=(-1.64387091178)*x[0]+(0.161580847642)*x[1]+(0.616741154516)*x[2]
        s[3,1,0]=(0.518052869517)*x[0]+(-0.491037975466)*x[1]+(-0.320485227716)*x[2]
        s[3,1,1]=(0.0299906539271)*x[0]+(-0.161268046678)*x[1]+(-0.933417226819)*x[2]
        s[4,0,0]=(-1.45814177886)*x[0]+(-1.01057484083)*x[1]+(1.05668957085)*x[2]
        s[4,0,1]=(0.335720229493)*x[0]+(0.155778050975)*x[1]+(-0.479943899005)*x[2]
        s[4,1,0]=(0.424166905697)*x[0]+(1.04188647689)*x[1]+(-0.154173690871)*x[2]
        s[4,1,1]=(-0.663564713586)*x[0]+(-0.744257330117)*x[1]+(-0.586452598078)*x[2]
        s[5,0,0]=(-0.308954778807)*x[0]+(0.0799719560375)*x[1]+(0.128085782808)*x[2]
        s[5,0,1]=(0.548010504324)*x[0]+(-0.676431304167)*x[1]+(0.12659732457)*x[2]
        s[5,1,0]=(0.17131336527)*x[0]+(-1.54908008145)*x[1]+(0.212836465224)*x[2]
        s[5,1,1]=(-0.834644454471)*x[0]+(1.52867678218)*x[1]+(1.19656096059)*x[2]
        ref[0,0,0]=(1.00224477774)*x_ref[0]+(0.276541980132)*x_ref[1]+(-1.61268557504)*x_ref[2]
        ref[0,0,1]=(0.182197948486)*x_ref[0]+(-0.314887566213)*x_ref[1]+(0.172753958549)*x_ref[2]
        ref[0,1,0]=(-0.225255532228)*x_ref[0]+(1.64823628561)*x_ref[1]+(0.00289115479583)*x_ref[2]
        ref[0,1,1]=(-0.0081734432798)*x_ref[0]+(-1.40127403863)*x_ref[1]+(-0.878765016785)*x_ref[2]
        ref[1,0,0]=(-0.88802398859)*x_ref[0]+(0.980539857729)*x_ref[1]+(0.106008000545)*x_ref[2]
        ref[1,0,1]=(0.413338006112)*x_ref[0]+(0.238889884193)*x_ref[1]+(0.216982176844)*x_ref[2]
        ref[1,1,0]=(-0.375359689292)*x_ref[0]+(0.656302411812)*x_ref[1]+(-1.13744732251)*x_ref[2]
        ref[1,1,1]=(-0.768638146497)*x_ref[0]+(0.382131491456)*x_ref[1]+(-0.0473187661226)*x_ref[2]
        ref[2,0,0]=(0.727474830988)*x_ref[0]+(-1.15897383904)*x_ref[1]+(1.03107954393)*x_ref[2]
        ref[2,0,1]=(-0.565279703477)*x_ref[0]+(0.952275155472)*x_ref[1]+(1.72082557166)*x_ref[2]
        ref[2,1,0]=(-0.504951356904)*x_ref[0]+(0.0945851323338)*x_ref[1]+(-1.18137582843)*x_ref[2]
        ref[2,1,1]=(0.441559000376)*x_ref[0]+(0.808481091433)*x_ref[1]+(-0.185790180692)*x_ref[2]
        ref[3,0,0]=(1.097525477)*x_ref[0]+(1.09598373235)*x_ref[1]+(-0.225656362018)*x_ref[2]
        ref[3,0,1]=(-1.64387091178)*x_ref[0]+(0.161580847642)*x_ref[1]+(0.616741154516)*x_ref[2]
        ref[3,1,0]=(0.518052869517)*x_ref[0]+(-0.491037975466)*x_ref[1]+(-0.320485227716)*x_ref[2]
        ref[3,1,1]=(0.0299906539271)*x_ref[0]+(-0.161268046678)*x_ref[1]+(-0.933417226819)*x_ref[2]
        ref[4,0,0]=(-1.45814177886)*x_ref[0]+(-1.01057484083)*x_ref[1]+(1.05668957085)*x_ref[2]
        ref[4,0,1]=(0.335720229493)*x_ref[0]+(0.155778050975)*x_ref[1]+(-0.479943899005)*x_ref[2]
        ref[4,1,0]=(0.424166905697)*x_ref[0]+(1.04188647689)*x_ref[1]+(-0.154173690871)*x_ref[2]
        ref[4,1,1]=(-0.663564713586)*x_ref[0]+(-0.744257330117)*x_ref[1]+(-0.586452598078)*x_ref[2]
        ref[5,0,0]=(-0.308954778807)*x_ref[0]+(0.0799719560375)*x_ref[1]+(0.128085782808)*x_ref[2]
        ref[5,0,1]=(0.548010504324)*x_ref[0]+(-0.676431304167)*x_ref[1]+(0.12659732457)*x_ref[2]
        ref[5,1,0]=(0.17131336527)*x_ref[0]+(-1.54908008145)*x_ref[1]+(0.212836465224)*x_ref[2]
        ref[5,1,1]=(-0.834644454471)*x_ref[0]+(1.52867678218)*x_ref[1]+(1.19656096059)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromSymbol_ReducedSolution_rank4(self):
      """
      tests interpolate for rank 4 Symbol onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedSolution onto ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5, 3, 2),dim=dim)
      s=Data(0,(4, 5, 3, 2),w)
      ref=Data(0,(4, 5, 3, 2),w_ref)
      if dim==2:
        s[0,0,0,0]=(-0.110140466592)*x[0]+(0.758992308989)*x[1]
        s[0,0,0,1]=(-1.27611018159)*x[0]+(-1.21469838588)*x[1]
        s[0,0,1,0]=(-0.695719260931)*x[0]+(-0.631650774678)*x[1]
        s[0,0,1,1]=(-0.0446786678677)*x[0]+(-0.547249041592)*x[1]
        s[0,0,2,0]=(0.824673646909)*x[0]+(-1.53499816213)*x[1]
        s[0,0,2,1]=(0.902244689111)*x[0]+(0.194498326222)*x[1]
        s[0,1,0,0]=(-1.29097330044)*x[0]+(0.0328620774092)*x[1]
        s[0,1,0,1]=(1.20419739576)*x[0]+(-1.25415670694)*x[1]
        s[0,1,1,0]=(0.825311334936)*x[0]+(0.732241589298)*x[1]
        s[0,1,1,1]=(0.239743697336)*x[0]+(0.446701903969)*x[1]
        s[0,1,2,0]=(-0.208525758105)*x[0]+(0.564727160849)*x[1]
        s[0,1,2,1]=(0.425352221432)*x[0]+(-0.439885368017)*x[1]
        s[0,2,0,0]=(0.089403990032)*x[0]+(0.927312736507)*x[1]
        s[0,2,0,1]=(-1.98342026057)*x[0]+(0.908824462154)*x[1]
        s[0,2,1,0]=(-1.02677966987)*x[0]+(1.64009929611)*x[1]
        s[0,2,1,1]=(1.18506229959)*x[0]+(-0.336041064484)*x[1]
        s[0,2,2,0]=(0.124121400983)*x[0]+(0.169858827797)*x[1]
        s[0,2,2,1]=(0.0171983721215)*x[0]+(-0.679377487833)*x[1]
        s[0,3,0,0]=(0.707474586567)*x[0]+(-0.327636474016)*x[1]
        s[0,3,0,1]=(-0.183596474381)*x[0]+(-1.07263498957)*x[1]
        s[0,3,1,0]=(1.49812660538)*x[0]+(-0.313209529867)*x[1]
        s[0,3,1,1]=(-0.678162796237)*x[0]+(-0.823724533082)*x[1]
        s[0,3,2,0]=(-0.79806833841)*x[0]+(0.865576104504)*x[1]
        s[0,3,2,1]=(0.941864201983)*x[0]+(0.734846756813)*x[1]
        s[0,4,0,0]=(-0.205171531689)*x[0]+(0.299610959481)*x[1]
        s[0,4,0,1]=(0.643430728461)*x[0]+(0.584994687875)*x[1]
        s[0,4,1,0]=(-0.408681959359)*x[0]+(0.601772358172)*x[1]
        s[0,4,1,1]=(-1.44472725121)*x[0]+(-0.0899809192849)*x[1]
        s[0,4,2,0]=(0.722764720152)*x[0]+(-0.407449476647)*x[1]
        s[0,4,2,1]=(0.166269375199)*x[0]+(1.20016653459)*x[1]
        s[1,0,0,0]=(-0.764665577343)*x[0]+(0.15492015266)*x[1]
        s[1,0,0,1]=(0.822023595189)*x[0]+(0.00511531586673)*x[1]
        s[1,0,1,0]=(0.0840294388025)*x[0]+(0.393606253186)*x[1]
        s[1,0,1,1]=(-0.702650062915)*x[0]+(-1.10361836979)*x[1]
        s[1,0,2,0]=(-1.34569564836)*x[0]+(-1.50816431101)*x[1]
        s[1,0,2,1]=(1.11337280315)*x[0]+(-0.0771805114838)*x[1]
        s[1,1,0,0]=(-0.0695861126454)*x[0]+(-0.437020174051)*x[1]
        s[1,1,0,1]=(1.68924343517)*x[0]+(0.706853885933)*x[1]
        s[1,1,1,0]=(0.0806933170819)*x[0]+(-0.974818406182)*x[1]
        s[1,1,1,1]=(-0.236836420312)*x[0]+(-0.352629546416)*x[1]
        s[1,1,2,0]=(-0.312697720062)*x[0]+(-0.977312197678)*x[1]
        s[1,1,2,1]=(-0.263563591998)*x[0]+(-0.899465194216)*x[1]
        s[1,2,0,0]=(1.0599010032)*x[0]+(0.625641179018)*x[1]
        s[1,2,0,1]=(0.42589369081)*x[0]+(0.63136683361)*x[1]
        s[1,2,1,0]=(0.629519575902)*x[0]+(-1.03125707286)*x[1]
        s[1,2,1,1]=(0.479272369893)*x[0]+(-1.63724584249)*x[1]
        s[1,2,2,0]=(1.48877937845)*x[0]+(-0.802308310767)*x[1]
        s[1,2,2,1]=(0.374636882204)*x[0]+(0.187733613588)*x[1]
        s[1,3,0,0]=(0.360148707925)*x[0]+(1.48869239048)*x[1]
        s[1,3,0,1]=(-1.29194970756)*x[0]+(-0.433561540189)*x[1]
        s[1,3,1,0]=(0.696118719028)*x[0]+(1.73199962229)*x[1]
        s[1,3,1,1]=(1.63831809651)*x[0]+(0.70687058079)*x[1]
        s[1,3,2,0]=(1.03614006811)*x[0]+(-0.783037005502)*x[1]
        s[1,3,2,1]=(0.351931393443)*x[0]+(-0.720060584954)*x[1]
        s[1,4,0,0]=(0.856138152361)*x[0]+(-0.0733689526147)*x[1]
        s[1,4,0,1]=(-0.256696810525)*x[0]+(1.57547133796)*x[1]
        s[1,4,1,0]=(0.260933620175)*x[0]+(-0.528418816229)*x[1]
        s[1,4,1,1]=(-1.14624698517)*x[0]+(0.537831890429)*x[1]
        s[1,4,2,0]=(-1.11328030308)*x[0]+(0.477599527943)*x[1]
        s[1,4,2,1]=(-1.52135700331)*x[0]+(1.18803329651)*x[1]
        s[2,0,0,0]=(1.02103992957)*x[0]+(1.3428520083)*x[1]
        s[2,0,0,1]=(0.74126089694)*x[0]+(-0.747092126031)*x[1]
        s[2,0,1,0]=(-0.300534057945)*x[0]+(-0.0679226717228)*x[1]
        s[2,0,1,1]=(0.205780243466)*x[0]+(1.89346679377)*x[1]
        s[2,0,2,0]=(0.711997044871)*x[0]+(0.614186591236)*x[1]
        s[2,0,2,1]=(0.562225764659)*x[0]+(-0.75210737336)*x[1]
        s[2,1,0,0]=(1.51024763079)*x[0]+(0.0655411043245)*x[1]
        s[2,1,0,1]=(-1.60270171758)*x[0]+(-0.555919918868)*x[1]
        s[2,1,1,0]=(0.0117250633883)*x[0]+(1.06089889984)*x[1]
        s[2,1,1,1]=(1.41839826051)*x[0]+(-1.11883387918)*x[1]
        s[2,1,2,0]=(-1.47481010167)*x[0]+(0.21986710996)*x[1]
        s[2,1,2,1]=(-0.890780004667)*x[0]+(0.68206435926)*x[1]
        s[2,2,0,0]=(0.00647580761798)*x[0]+(1.08988646609)*x[1]
        s[2,2,0,1]=(0.107576626365)*x[0]+(0.569541010093)*x[1]
        s[2,2,1,0]=(0.655745883134)*x[0]+(1.59781027174)*x[1]
        s[2,2,1,1]=(0.423699701576)*x[0]+(-0.854187892248)*x[1]
        s[2,2,2,0]=(-0.904849730929)*x[0]+(0.235074145696)*x[1]
        s[2,2,2,1]=(0.981525463443)*x[0]+(0.701832385796)*x[1]
        s[2,3,0,0]=(-0.489533883573)*x[0]+(-0.0213717056714)*x[1]
        s[2,3,0,1]=(0.516832539105)*x[0]+(-1.11995329355)*x[1]
        s[2,3,1,0]=(0.287740517981)*x[0]+(-1.3210942817)*x[1]
        s[2,3,1,1]=(0.179604184524)*x[0]+(-1.63664715586)*x[1]
        s[2,3,2,0]=(-0.492296320142)*x[0]+(-0.502235503805)*x[1]
        s[2,3,2,1]=(-0.653587673689)*x[0]+(0.5466761379)*x[1]
        s[2,4,0,0]=(1.37393629094)*x[0]+(0.645991033265)*x[1]
        s[2,4,0,1]=(-0.108057947979)*x[0]+(-0.761267645781)*x[1]
        s[2,4,1,0]=(-0.0900524038235)*x[0]+(0.16773443574)*x[1]
        s[2,4,1,1]=(0.578749842767)*x[0]+(1.18687780739)*x[1]
        s[2,4,2,0]=(-0.349879331342)*x[0]+(-0.214411558047)*x[1]
        s[2,4,2,1]=(-0.875945724127)*x[0]+(-1.36065436643)*x[1]
        s[3,0,0,0]=(0.0488555315168)*x[0]+(-0.82382271801)*x[1]
        s[3,0,0,1]=(1.24017449448)*x[0]+(-0.911198047419)*x[1]
        s[3,0,1,0]=(0.30200654088)*x[0]+(-1.11559310554)*x[1]
        s[3,0,1,1]=(0.67275005347)*x[0]+(-0.645034711632)*x[1]
        s[3,0,2,0]=(-0.670229614054)*x[0]+(0.359423039794)*x[1]
        s[3,0,2,1]=(1.14988130646)*x[0]+(0.86614224885)*x[1]
        s[3,1,0,0]=(0.705716005963)*x[0]+(-1.15357209623)*x[1]
        s[3,1,0,1]=(0.31067508882)*x[0]+(0.960823286011)*x[1]
        s[3,1,1,0]=(0.866232494898)*x[0]+(-0.65162103259)*x[1]
        s[3,1,1,1]=(-0.452071783562)*x[0]+(-0.211956993156)*x[1]
        s[3,1,2,0]=(-1.32137973484)*x[0]+(1.49152904496)*x[1]
        s[3,1,2,1]=(0.854595989445)*x[0]+(1.11534387313)*x[1]
        s[3,2,0,0]=(0.789093645939)*x[0]+(-0.623866327905)*x[1]
        s[3,2,0,1]=(-0.649118397874)*x[0]+(-0.510923012103)*x[1]
        s[3,2,1,0]=(-0.263103480764)*x[0]+(0.0353171690366)*x[1]
        s[3,2,1,1]=(-0.910441885706)*x[0]+(-0.572413584482)*x[1]
        s[3,2,2,0]=(-0.764615254979)*x[0]+(-0.257894213166)*x[1]
        s[3,2,2,1]=(0.82110723418)*x[0]+(-0.881337133364)*x[1]
        s[3,3,0,0]=(-0.978909443882)*x[0]+(0.95340950915)*x[1]
        s[3,3,0,1]=(-0.121221461759)*x[0]+(-1.68773270918)*x[1]
        s[3,3,1,0]=(-0.150032516264)*x[0]+(0.441210519728)*x[1]
        s[3,3,1,1]=(-0.618694838164)*x[0]+(0.379748674791)*x[1]
        s[3,3,2,0]=(0.208366138002)*x[0]+(1.04701688993)*x[1]
        s[3,3,2,1]=(0.207672316371)*x[0]+(-1.04311602421)*x[1]
        s[3,4,0,0]=(0.404190185535)*x[0]+(0.206083873066)*x[1]
        s[3,4,0,1]=(0.928741940014)*x[0]+(0.126856418039)*x[1]
        s[3,4,1,0]=(0.0831637965324)*x[0]+(0.198288732425)*x[1]
        s[3,4,1,1]=(0.602722196536)*x[0]+(-0.256860860163)*x[1]
        s[3,4,2,0]=(1.11330980537)*x[0]+(0.130706767542)*x[1]
        s[3,4,2,1]=(0.638478712945)*x[0]+(0.558803659636)*x[1]
        ref[0,0,0,0]=(-0.110140466592)*x_ref[0]+(0.758992308989)*x_ref[1]
        ref[0,0,0,1]=(-1.27611018159)*x_ref[0]+(-1.21469838588)*x_ref[1]
        ref[0,0,1,0]=(-0.695719260931)*x_ref[0]+(-0.631650774678)*x_ref[1]
        ref[0,0,1,1]=(-0.0446786678677)*x_ref[0]+(-0.547249041592)*x_ref[1]
        ref[0,0,2,0]=(0.824673646909)*x_ref[0]+(-1.53499816213)*x_ref[1]
        ref[0,0,2,1]=(0.902244689111)*x_ref[0]+(0.194498326222)*x_ref[1]
        ref[0,1,0,0]=(-1.29097330044)*x_ref[0]+(0.0328620774092)*x_ref[1]
        ref[0,1,0,1]=(1.20419739576)*x_ref[0]+(-1.25415670694)*x_ref[1]
        ref[0,1,1,0]=(0.825311334936)*x_ref[0]+(0.732241589298)*x_ref[1]
        ref[0,1,1,1]=(0.239743697336)*x_ref[0]+(0.446701903969)*x_ref[1]
        ref[0,1,2,0]=(-0.208525758105)*x_ref[0]+(0.564727160849)*x_ref[1]
        ref[0,1,2,1]=(0.425352221432)*x_ref[0]+(-0.439885368017)*x_ref[1]
        ref[0,2,0,0]=(0.089403990032)*x_ref[0]+(0.927312736507)*x_ref[1]
        ref[0,2,0,1]=(-1.98342026057)*x_ref[0]+(0.908824462154)*x_ref[1]
        ref[0,2,1,0]=(-1.02677966987)*x_ref[0]+(1.64009929611)*x_ref[1]
        ref[0,2,1,1]=(1.18506229959)*x_ref[0]+(-0.336041064484)*x_ref[1]
        ref[0,2,2,0]=(0.124121400983)*x_ref[0]+(0.169858827797)*x_ref[1]
        ref[0,2,2,1]=(0.0171983721215)*x_ref[0]+(-0.679377487833)*x_ref[1]
        ref[0,3,0,0]=(0.707474586567)*x_ref[0]+(-0.327636474016)*x_ref[1]
        ref[0,3,0,1]=(-0.183596474381)*x_ref[0]+(-1.07263498957)*x_ref[1]
        ref[0,3,1,0]=(1.49812660538)*x_ref[0]+(-0.313209529867)*x_ref[1]
        ref[0,3,1,1]=(-0.678162796237)*x_ref[0]+(-0.823724533082)*x_ref[1]
        ref[0,3,2,0]=(-0.79806833841)*x_ref[0]+(0.865576104504)*x_ref[1]
        ref[0,3,2,1]=(0.941864201983)*x_ref[0]+(0.734846756813)*x_ref[1]
        ref[0,4,0,0]=(-0.205171531689)*x_ref[0]+(0.299610959481)*x_ref[1]
        ref[0,4,0,1]=(0.643430728461)*x_ref[0]+(0.584994687875)*x_ref[1]
        ref[0,4,1,0]=(-0.408681959359)*x_ref[0]+(0.601772358172)*x_ref[1]
        ref[0,4,1,1]=(-1.44472725121)*x_ref[0]+(-0.0899809192849)*x_ref[1]
        ref[0,4,2,0]=(0.722764720152)*x_ref[0]+(-0.407449476647)*x_ref[1]
        ref[0,4,2,1]=(0.166269375199)*x_ref[0]+(1.20016653459)*x_ref[1]
        ref[1,0,0,0]=(-0.764665577343)*x_ref[0]+(0.15492015266)*x_ref[1]
        ref[1,0,0,1]=(0.822023595189)*x_ref[0]+(0.00511531586673)*x_ref[1]
        ref[1,0,1,0]=(0.0840294388025)*x_ref[0]+(0.393606253186)*x_ref[1]
        ref[1,0,1,1]=(-0.702650062915)*x_ref[0]+(-1.10361836979)*x_ref[1]
        ref[1,0,2,0]=(-1.34569564836)*x_ref[0]+(-1.50816431101)*x_ref[1]
        ref[1,0,2,1]=(1.11337280315)*x_ref[0]+(-0.0771805114838)*x_ref[1]
        ref[1,1,0,0]=(-0.0695861126454)*x_ref[0]+(-0.437020174051)*x_ref[1]
        ref[1,1,0,1]=(1.68924343517)*x_ref[0]+(0.706853885933)*x_ref[1]
        ref[1,1,1,0]=(0.0806933170819)*x_ref[0]+(-0.974818406182)*x_ref[1]
        ref[1,1,1,1]=(-0.236836420312)*x_ref[0]+(-0.352629546416)*x_ref[1]
        ref[1,1,2,0]=(-0.312697720062)*x_ref[0]+(-0.977312197678)*x_ref[1]
        ref[1,1,2,1]=(-0.263563591998)*x_ref[0]+(-0.899465194216)*x_ref[1]
        ref[1,2,0,0]=(1.0599010032)*x_ref[0]+(0.625641179018)*x_ref[1]
        ref[1,2,0,1]=(0.42589369081)*x_ref[0]+(0.63136683361)*x_ref[1]
        ref[1,2,1,0]=(0.629519575902)*x_ref[0]+(-1.03125707286)*x_ref[1]
        ref[1,2,1,1]=(0.479272369893)*x_ref[0]+(-1.63724584249)*x_ref[1]
        ref[1,2,2,0]=(1.48877937845)*x_ref[0]+(-0.802308310767)*x_ref[1]
        ref[1,2,2,1]=(0.374636882204)*x_ref[0]+(0.187733613588)*x_ref[1]
        ref[1,3,0,0]=(0.360148707925)*x_ref[0]+(1.48869239048)*x_ref[1]
        ref[1,3,0,1]=(-1.29194970756)*x_ref[0]+(-0.433561540189)*x_ref[1]
        ref[1,3,1,0]=(0.696118719028)*x_ref[0]+(1.73199962229)*x_ref[1]
        ref[1,3,1,1]=(1.63831809651)*x_ref[0]+(0.70687058079)*x_ref[1]
        ref[1,3,2,0]=(1.03614006811)*x_ref[0]+(-0.783037005502)*x_ref[1]
        ref[1,3,2,1]=(0.351931393443)*x_ref[0]+(-0.720060584954)*x_ref[1]
        ref[1,4,0,0]=(0.856138152361)*x_ref[0]+(-0.0733689526147)*x_ref[1]
        ref[1,4,0,1]=(-0.256696810525)*x_ref[0]+(1.57547133796)*x_ref[1]
        ref[1,4,1,0]=(0.260933620175)*x_ref[0]+(-0.528418816229)*x_ref[1]
        ref[1,4,1,1]=(-1.14624698517)*x_ref[0]+(0.537831890429)*x_ref[1]
        ref[1,4,2,0]=(-1.11328030308)*x_ref[0]+(0.477599527943)*x_ref[1]
        ref[1,4,2,1]=(-1.52135700331)*x_ref[0]+(1.18803329651)*x_ref[1]
        ref[2,0,0,0]=(1.02103992957)*x_ref[0]+(1.3428520083)*x_ref[1]
        ref[2,0,0,1]=(0.74126089694)*x_ref[0]+(-0.747092126031)*x_ref[1]
        ref[2,0,1,0]=(-0.300534057945)*x_ref[0]+(-0.0679226717228)*x_ref[1]
        ref[2,0,1,1]=(0.205780243466)*x_ref[0]+(1.89346679377)*x_ref[1]
        ref[2,0,2,0]=(0.711997044871)*x_ref[0]+(0.614186591236)*x_ref[1]
        ref[2,0,2,1]=(0.562225764659)*x_ref[0]+(-0.75210737336)*x_ref[1]
        ref[2,1,0,0]=(1.51024763079)*x_ref[0]+(0.0655411043245)*x_ref[1]
        ref[2,1,0,1]=(-1.60270171758)*x_ref[0]+(-0.555919918868)*x_ref[1]
        ref[2,1,1,0]=(0.0117250633883)*x_ref[0]+(1.06089889984)*x_ref[1]
        ref[2,1,1,1]=(1.41839826051)*x_ref[0]+(-1.11883387918)*x_ref[1]
        ref[2,1,2,0]=(-1.47481010167)*x_ref[0]+(0.21986710996)*x_ref[1]
        ref[2,1,2,1]=(-0.890780004667)*x_ref[0]+(0.68206435926)*x_ref[1]
        ref[2,2,0,0]=(0.00647580761798)*x_ref[0]+(1.08988646609)*x_ref[1]
        ref[2,2,0,1]=(0.107576626365)*x_ref[0]+(0.569541010093)*x_ref[1]
        ref[2,2,1,0]=(0.655745883134)*x_ref[0]+(1.59781027174)*x_ref[1]
        ref[2,2,1,1]=(0.423699701576)*x_ref[0]+(-0.854187892248)*x_ref[1]
        ref[2,2,2,0]=(-0.904849730929)*x_ref[0]+(0.235074145696)*x_ref[1]
        ref[2,2,2,1]=(0.981525463443)*x_ref[0]+(0.701832385796)*x_ref[1]
        ref[2,3,0,0]=(-0.489533883573)*x_ref[0]+(-0.0213717056714)*x_ref[1]
        ref[2,3,0,1]=(0.516832539105)*x_ref[0]+(-1.11995329355)*x_ref[1]
        ref[2,3,1,0]=(0.287740517981)*x_ref[0]+(-1.3210942817)*x_ref[1]
        ref[2,3,1,1]=(0.179604184524)*x_ref[0]+(-1.63664715586)*x_ref[1]
        ref[2,3,2,0]=(-0.492296320142)*x_ref[0]+(-0.502235503805)*x_ref[1]
        ref[2,3,2,1]=(-0.653587673689)*x_ref[0]+(0.5466761379)*x_ref[1]
        ref[2,4,0,0]=(1.37393629094)*x_ref[0]+(0.645991033265)*x_ref[1]
        ref[2,4,0,1]=(-0.108057947979)*x_ref[0]+(-0.761267645781)*x_ref[1]
        ref[2,4,1,0]=(-0.0900524038235)*x_ref[0]+(0.16773443574)*x_ref[1]
        ref[2,4,1,1]=(0.578749842767)*x_ref[0]+(1.18687780739)*x_ref[1]
        ref[2,4,2,0]=(-0.349879331342)*x_ref[0]+(-0.214411558047)*x_ref[1]
        ref[2,4,2,1]=(-0.875945724127)*x_ref[0]+(-1.36065436643)*x_ref[1]
        ref[3,0,0,0]=(0.0488555315168)*x_ref[0]+(-0.82382271801)*x_ref[1]
        ref[3,0,0,1]=(1.24017449448)*x_ref[0]+(-0.911198047419)*x_ref[1]
        ref[3,0,1,0]=(0.30200654088)*x_ref[0]+(-1.11559310554)*x_ref[1]
        ref[3,0,1,1]=(0.67275005347)*x_ref[0]+(-0.645034711632)*x_ref[1]
        ref[3,0,2,0]=(-0.670229614054)*x_ref[0]+(0.359423039794)*x_ref[1]
        ref[3,0,2,1]=(1.14988130646)*x_ref[0]+(0.86614224885)*x_ref[1]
        ref[3,1,0,0]=(0.705716005963)*x_ref[0]+(-1.15357209623)*x_ref[1]
        ref[3,1,0,1]=(0.31067508882)*x_ref[0]+(0.960823286011)*x_ref[1]
        ref[3,1,1,0]=(0.866232494898)*x_ref[0]+(-0.65162103259)*x_ref[1]
        ref[3,1,1,1]=(-0.452071783562)*x_ref[0]+(-0.211956993156)*x_ref[1]
        ref[3,1,2,0]=(-1.32137973484)*x_ref[0]+(1.49152904496)*x_ref[1]
        ref[3,1,2,1]=(0.854595989445)*x_ref[0]+(1.11534387313)*x_ref[1]
        ref[3,2,0,0]=(0.789093645939)*x_ref[0]+(-0.623866327905)*x_ref[1]
        ref[3,2,0,1]=(-0.649118397874)*x_ref[0]+(-0.510923012103)*x_ref[1]
        ref[3,2,1,0]=(-0.263103480764)*x_ref[0]+(0.0353171690366)*x_ref[1]
        ref[3,2,1,1]=(-0.910441885706)*x_ref[0]+(-0.572413584482)*x_ref[1]
        ref[3,2,2,0]=(-0.764615254979)*x_ref[0]+(-0.257894213166)*x_ref[1]
        ref[3,2,2,1]=(0.82110723418)*x_ref[0]+(-0.881337133364)*x_ref[1]
        ref[3,3,0,0]=(-0.978909443882)*x_ref[0]+(0.95340950915)*x_ref[1]
        ref[3,3,0,1]=(-0.121221461759)*x_ref[0]+(-1.68773270918)*x_ref[1]
        ref[3,3,1,0]=(-0.150032516264)*x_ref[0]+(0.441210519728)*x_ref[1]
        ref[3,3,1,1]=(-0.618694838164)*x_ref[0]+(0.379748674791)*x_ref[1]
        ref[3,3,2,0]=(0.208366138002)*x_ref[0]+(1.04701688993)*x_ref[1]
        ref[3,3,2,1]=(0.207672316371)*x_ref[0]+(-1.04311602421)*x_ref[1]
        ref[3,4,0,0]=(0.404190185535)*x_ref[0]+(0.206083873066)*x_ref[1]
        ref[3,4,0,1]=(0.928741940014)*x_ref[0]+(0.126856418039)*x_ref[1]
        ref[3,4,1,0]=(0.0831637965324)*x_ref[0]+(0.198288732425)*x_ref[1]
        ref[3,4,1,1]=(0.602722196536)*x_ref[0]+(-0.256860860163)*x_ref[1]
        ref[3,4,2,0]=(1.11330980537)*x_ref[0]+(0.130706767542)*x_ref[1]
        ref[3,4,2,1]=(0.638478712945)*x_ref[0]+(0.558803659636)*x_ref[1]
      else:
        s[0,0,0,0]=(-0.678750626234)*x[0]+(0.707898134345)*x[1]+(1.47682355575)*x[2]
        s[0,0,0,1]=(-1.08665921875)*x[0]+(-1.03746577329)*x[1]+(0.459220112015)*x[2]
        s[0,0,1,0]=(-0.0924850687831)*x[0]+(-1.21392495875)*x[1]+(-0.335482829106)*x[2]
        s[0,0,1,1]=(-0.796044372195)*x[0]+(-1.61600945946)*x[1]+(-0.917678370431)*x[2]
        s[0,0,2,0]=(-1.17314148071)*x[0]+(0.594053121363)*x[1]+(-1.0751981685)*x[2]
        s[0,0,2,1]=(-0.993747266312)*x[0]+(0.647546031964)*x[1]+(0.789233440797)*x[2]
        s[0,1,0,0]=(-0.538620007403)*x[0]+(0.572432466547)*x[1]+(1.59224293356)*x[2]
        s[0,1,0,1]=(-0.315217413376)*x[0]+(1.30492005409)*x[1]+(-0.04944666559)*x[2]
        s[0,1,1,0]=(-0.734247343541)*x[0]+(1.14104633611)*x[1]+(-0.0424483055941)*x[2]
        s[0,1,1,1]=(-0.521727902651)*x[0]+(-0.0302866863619)*x[1]+(-0.0378416654735)*x[2]
        s[0,1,2,0]=(0.319532436452)*x[0]+(0.312388869947)*x[1]+(0.237025751349)*x[2]
        s[0,1,2,1]=(0.713884175591)*x[0]+(1.10911840343)*x[1]+(-1.4943567087)*x[2]
        s[0,2,0,0]=(-0.599254236697)*x[0]+(-0.190373139957)*x[1]+(-1.75168866737)*x[2]
        s[0,2,0,1]=(0.396782073165)*x[0]+(-0.252470841512)*x[1]+(-0.463651254287)*x[2]
        s[0,2,1,0]=(1.03549889185)*x[0]+(0.225894922761)*x[1]+(-0.609139538729)*x[2]
        s[0,2,1,1]=(0.289458873955)*x[0]+(-0.136668874286)*x[1]+(0.474827071819)*x[2]
        s[0,2,2,0]=(0.654253727057)*x[0]+(-0.998748120229)*x[1]+(-0.107718066457)*x[2]
        s[0,2,2,1]=(0.256710826049)*x[0]+(0.309515575678)*x[1]+(-1.60310813555)*x[2]
        s[0,3,0,0]=(1.70779242771)*x[0]+(-0.530818399689)*x[1]+(-0.516799319419)*x[2]
        s[0,3,0,1]=(-0.848868447677)*x[0]+(-0.301212439564)*x[1]+(-0.423135852525)*x[2]
        s[0,3,1,0]=(1.0807833809)*x[0]+(-1.20726021059)*x[1]+(0.565314004443)*x[2]
        s[0,3,1,1]=(-0.660234771003)*x[0]+(-0.443966617085)*x[1]+(-0.185381740077)*x[2]
        s[0,3,2,0]=(1.6260513125)*x[0]+(0.145626479867)*x[1]+(-0.808965403823)*x[2]
        s[0,3,2,1]=(-0.503772598493)*x[0]+(-1.63975947666)*x[1]+(0.255681416016)*x[2]
        s[0,4,0,0]=(0.0818777313252)*x[0]+(0.428155907605)*x[1]+(-0.642991998051)*x[2]
        s[0,4,0,1]=(-0.145032953266)*x[0]+(-0.7645425853)*x[1]+(0.458784780224)*x[2]
        s[0,4,1,0]=(-0.589583511871)*x[0]+(-0.0743175116482)*x[1]+(0.285113485271)*x[2]
        s[0,4,1,1]=(-1.37305892557)*x[0]+(0.917822397515)*x[1]+(-0.378127728174)*x[2]
        s[0,4,2,0]=(1.16505232525)*x[0]+(1.10944900772)*x[1]+(-0.95491955738)*x[2]
        s[0,4,2,1]=(-1.12539432931)*x[0]+(-0.290265022179)*x[1]+(-1.20809121853)*x[2]
        s[1,0,0,0]=(-0.859420732972)*x[0]+(0.564058561557)*x[1]+(-0.795155516002)*x[2]
        s[1,0,0,1]=(-0.314313522693)*x[0]+(1.46275003784)*x[1]+(0.285596389281)*x[2]
        s[1,0,1,0]=(-0.513360534744)*x[0]+(-0.0532410726117)*x[1]+(-0.122433325652)*x[2]
        s[1,0,1,1]=(0.527488940208)*x[0]+(0.0760609834598)*x[1]+(-0.612300815838)*x[2]
        s[1,0,2,0]=(0.302549971797)*x[0]+(-0.710901255568)*x[1]+(0.955062818656)*x[2]
        s[1,0,2,1]=(0.126520802001)*x[0]+(-1.07564234283)*x[1]+(-0.669526781403)*x[2]
        s[1,1,0,0]=(1.50804296428)*x[0]+(0.111903566342)*x[1]+(0.465876365517)*x[2]
        s[1,1,0,1]=(-1.04380726708)*x[0]+(0.148245186406)*x[1]+(-1.06801443458)*x[2]
        s[1,1,1,0]=(1.48236374583)*x[0]+(-0.00794102905726)*x[1]+(-1.35613430084)*x[2]
        s[1,1,1,1]=(1.04119256641)*x[0]+(-1.84075261029)*x[1]+(-0.776969168414)*x[2]
        s[1,1,2,0]=(-0.356539020393)*x[0]+(1.70797570799)*x[1]+(-0.525736126605)*x[2]
        s[1,1,2,1]=(1.62559049194)*x[0]+(-0.482482804647)*x[1]+(0.631790498783)*x[2]
        s[1,2,0,0]=(0.499315452031)*x[0]+(-0.296700233772)*x[1]+(1.00111627057)*x[2]
        s[1,2,0,1]=(1.0433119055)*x[0]+(-0.269957745366)*x[1]+(0.591153091169)*x[2]
        s[1,2,1,0]=(0.502068262492)*x[0]+(-0.849037462803)*x[1]+(-1.56692020458)*x[2]
        s[1,2,1,1]=(1.2984795003)*x[0]+(-1.07653573134)*x[1]+(0.487232329161)*x[2]
        s[1,2,2,0]=(-0.109333568443)*x[0]+(0.687162555279)*x[1]+(-0.812636431387)*x[2]
        s[1,2,2,1]=(0.202326014817)*x[0]+(1.23297194202)*x[1]+(1.41163217274)*x[2]
        s[1,3,0,0]=(0.124740742544)*x[0]+(-0.500878894409)*x[1]+(-0.376438702019)*x[2]
        s[1,3,0,1]=(-0.540234624582)*x[0]+(-0.400577181452)*x[1]+(0.879215681428)*x[2]
        s[1,3,1,0]=(0.0426835015338)*x[0]+(-0.726976626459)*x[1]+(1.32655296741)*x[2]
        s[1,3,1,1]=(-0.120969369352)*x[0]+(-0.426853376816)*x[1]+(1.88787704976)*x[2]
        s[1,3,2,0]=(-0.0895115836038)*x[0]+(-1.17409640569)*x[1]+(-0.874853748928)*x[2]
        s[1,3,2,1]=(-0.0664677570269)*x[0]+(0.708204452159)*x[1]+(-0.26397721498)*x[2]
        s[1,4,0,0]=(0.0236750184919)*x[0]+(-1.42225228911)*x[1]+(0.399502903376)*x[2]
        s[1,4,0,1]=(1.03400366274)*x[0]+(-0.442018365874)*x[1]+(1.02491120541)*x[2]
        s[1,4,1,0]=(-0.150528717616)*x[0]+(-0.512236709632)*x[1]+(-0.0644728802768)*x[2]
        s[1,4,1,1]=(-0.998520493595)*x[0]+(-0.507010096027)*x[1]+(-0.889168150742)*x[2]
        s[1,4,2,0]=(0.179165278401)*x[0]+(1.07136931045)*x[1]+(-0.790101000847)*x[2]
        s[1,4,2,1]=(0.376877479598)*x[0]+(-0.25979387806)*x[1]+(0.446682478544)*x[2]
        s[2,0,0,0]=(0.365668617405)*x[0]+(1.44268125148)*x[1]+(1.65785163405)*x[2]
        s[2,0,0,1]=(0.576180639993)*x[0]+(-0.0571374651916)*x[1]+(-0.0854307657892)*x[2]
        s[2,0,1,0]=(0.0474860699991)*x[0]+(-1.47716457199)*x[1]+(0.87300241949)*x[2]
        s[2,0,1,1]=(-0.293608884991)*x[0]+(0.0963236497693)*x[1]+(-1.31217901681)*x[2]
        s[2,0,2,0]=(-0.725125904522)*x[0]+(0.15259525725)*x[1]+(0.0624842579957)*x[2]
        s[2,0,2,1]=(-0.374667932076)*x[0]+(-0.586704716252)*x[1]+(-0.208259155061)*x[2]
        s[2,1,0,0]=(0.480010407761)*x[0]+(-1.78856630141)*x[1]+(0.296814589368)*x[2]
        s[2,1,0,1]=(-0.131871389585)*x[0]+(0.0984653022698)*x[1]+(0.222785695838)*x[2]
        s[2,1,1,0]=(-0.4095594348)*x[0]+(-0.746059065385)*x[1]+(1.72282248984)*x[2]
        s[2,1,1,1]=(0.841882742791)*x[0]+(0.698803891138)*x[1]+(0.201279240318)*x[2]
        s[2,1,2,0]=(-1.52503851146)*x[0]+(0.746778574384)*x[1]+(0.820127798751)*x[2]
        s[2,1,2,1]=(0.578616447592)*x[0]+(-0.232884112626)*x[1]+(0.719949134498)*x[2]
        s[2,2,0,0]=(-0.250869865455)*x[0]+(0.0457838553118)*x[1]+(0.694605224088)*x[2]
        s[2,2,0,1]=(-0.344703134894)*x[0]+(-0.945296587028)*x[1]+(-0.408991625518)*x[2]
        s[2,2,1,0]=(1.15842118069)*x[0]+(0.385018100988)*x[1]+(-0.298867420422)*x[2]
        s[2,2,1,1]=(-0.992212925836)*x[0]+(-0.303921757453)*x[1]+(1.90996417539)*x[2]
        s[2,2,2,0]=(0.139346086335)*x[0]+(-1.5255000175)*x[1]+(-0.418223280061)*x[2]
        s[2,2,2,1]=(-0.287165354844)*x[0]+(-0.563470113789)*x[1]+(-0.267078242542)*x[2]
        s[2,3,0,0]=(-0.877956931038)*x[0]+(-0.587379404687)*x[1]+(0.223772989101)*x[2]
        s[2,3,0,1]=(0.450240028907)*x[0]+(0.933649504645)*x[1]+(0.922982912622)*x[2]
        s[2,3,1,0]=(-0.695647633954)*x[0]+(0.675123237641)*x[1]+(0.333013288603)*x[2]
        s[2,3,1,1]=(-0.638319791165)*x[0]+(-1.01905860587)*x[1]+(-0.0829897514576)*x[2]
        s[2,3,2,0]=(-0.85507363839)*x[0]+(0.213154292163)*x[1]+(-1.19634907609)*x[2]
        s[2,3,2,1]=(0.51969848742)*x[0]+(-0.103353752424)*x[1]+(1.38100947757)*x[2]
        s[2,4,0,0]=(-1.42264362114)*x[0]+(-0.0622861012095)*x[1]+(1.11415860946)*x[2]
        s[2,4,0,1]=(1.03146522202)*x[0]+(-0.59775537826)*x[1]+(0.455451132416)*x[2]
        s[2,4,1,0]=(-1.42367400988)*x[0]+(0.468019196451)*x[1]+(-0.0449222277683)*x[2]
        s[2,4,1,1]=(-0.337745267456)*x[0]+(-1.24573098041)*x[1]+(0.591411642381)*x[2]
        s[2,4,2,0]=(-0.773220236551)*x[0]+(-0.92934639777)*x[1]+(-0.323666905212)*x[2]
        s[2,4,2,1]=(-1.25416565442)*x[0]+(0.10804460369)*x[1]+(1.3398483009)*x[2]
        s[3,0,0,0]=(-0.187928573247)*x[0]+(0.952793598709)*x[1]+(-0.651544295118)*x[2]
        s[3,0,0,1]=(-1.18183645576)*x[0]+(-0.767480258477)*x[1]+(0.0733909569555)*x[2]
        s[3,0,1,0]=(0.825156828887)*x[0]+(0.900307840844)*x[1]+(-1.03153118659)*x[2]
        s[3,0,1,1]=(-0.150209523447)*x[0]+(1.07490578747)*x[1]+(-1.6916184583)*x[2]
        s[3,0,2,0]=(-1.07041023688)*x[0]+(-0.420347448824)*x[1]+(-0.556885268114)*x[2]
        s[3,0,2,1]=(-1.30609527635)*x[0]+(-0.10753520843)*x[1]+(0.240258152808)*x[2]
        s[3,1,0,0]=(0.899651818251)*x[0]+(0.00909698197696)*x[1]+(-1.64985946319)*x[2]
        s[3,1,0,1]=(-0.880471412731)*x[0]+(0.678893404261)*x[1]+(0.288497390335)*x[2]
        s[3,1,1,0]=(-0.00910924813519)*x[0]+(-1.30365942504)*x[1]+(-0.568919628746)*x[2]
        s[3,1,1,1]=(-0.811256892653)*x[0]+(-0.0284400402628)*x[1]+(0.727386097819)*x[2]
        s[3,1,2,0]=(0.105755186759)*x[0]+(0.196242692495)*x[1]+(0.358516596542)*x[2]
        s[3,1,2,1]=(-0.37985806916)*x[0]+(0.44810760494)*x[1]+(0.633807267038)*x[2]
        s[3,2,0,0]=(1.34921360449)*x[0]+(-0.9349137549)*x[1]+(-0.272245661092)*x[2]
        s[3,2,0,1]=(1.21020688092)*x[0]+(1.1296074258)*x[1]+(0.000391430715188)*x[2]
        s[3,2,1,0]=(-1.25769018387)*x[0]+(-0.786491629941)*x[1]+(-0.862105545664)*x[2]
        s[3,2,1,1]=(0.668529227552)*x[0]+(-0.983024344824)*x[1]+(0.74846424728)*x[2]
        s[3,2,2,0]=(0.45487084108)*x[0]+(0.855491649411)*x[1]+(-0.104621648479)*x[2]
        s[3,2,2,1]=(0.802451481866)*x[0]+(-0.137058520895)*x[1]+(0.573940225359)*x[2]
        s[3,3,0,0]=(0.468999526003)*x[0]+(0.504917415809)*x[1]+(-1.39187140958)*x[2]
        s[3,3,0,1]=(0.501007631838)*x[0]+(-0.285379909295)*x[1]+(0.257752560803)*x[2]
        s[3,3,1,0]=(-1.01423550219)*x[0]+(-0.749951834747)*x[1]+(-0.156550300135)*x[2]
        s[3,3,1,1]=(0.190954159364)*x[0]+(0.985690213898)*x[1]+(-0.942349242057)*x[2]
        s[3,3,2,0]=(-1.04776017317)*x[0]+(1.3978257599)*x[1]+(-0.727385312507)*x[2]
        s[3,3,2,1]=(0.118574662458)*x[0]+(0.763657521414)*x[1]+(-0.947854404873)*x[2]
        s[3,4,0,0]=(-1.42375158871)*x[0]+(1.02577770584)*x[1]+(1.7262667856)*x[2]
        s[3,4,0,1]=(0.819198798755)*x[0]+(-0.177472792322)*x[1]+(-0.495119104783)*x[2]
        s[3,4,1,0]=(-1.44070928104)*x[0]+(-0.67376984043)*x[1]+(0.314203511381)*x[2]
        s[3,4,1,1]=(-0.0417551422406)*x[0]+(-0.446166510311)*x[1]+(-0.303922120635)*x[2]
        s[3,4,2,0]=(0.161905825888)*x[0]+(-0.433001417416)*x[1]+(0.713602984094)*x[2]
        s[3,4,2,1]=(1.29579174225)*x[0]+(-0.136051917273)*x[1]+(0.270836816064)*x[2]
        ref[0,0,0,0]=(-0.678750626234)*x_ref[0]+(0.707898134345)*x_ref[1]+(1.47682355575)*x_ref[2]
        ref[0,0,0,1]=(-1.08665921875)*x_ref[0]+(-1.03746577329)*x_ref[1]+(0.459220112015)*x_ref[2]
        ref[0,0,1,0]=(-0.0924850687831)*x_ref[0]+(-1.21392495875)*x_ref[1]+(-0.335482829106)*x_ref[2]
        ref[0,0,1,1]=(-0.796044372195)*x_ref[0]+(-1.61600945946)*x_ref[1]+(-0.917678370431)*x_ref[2]
        ref[0,0,2,0]=(-1.17314148071)*x_ref[0]+(0.594053121363)*x_ref[1]+(-1.0751981685)*x_ref[2]
        ref[0,0,2,1]=(-0.993747266312)*x_ref[0]+(0.647546031964)*x_ref[1]+(0.789233440797)*x_ref[2]
        ref[0,1,0,0]=(-0.538620007403)*x_ref[0]+(0.572432466547)*x_ref[1]+(1.59224293356)*x_ref[2]
        ref[0,1,0,1]=(-0.315217413376)*x_ref[0]+(1.30492005409)*x_ref[1]+(-0.04944666559)*x_ref[2]
        ref[0,1,1,0]=(-0.734247343541)*x_ref[0]+(1.14104633611)*x_ref[1]+(-0.0424483055941)*x_ref[2]
        ref[0,1,1,1]=(-0.521727902651)*x_ref[0]+(-0.0302866863619)*x_ref[1]+(-0.0378416654735)*x_ref[2]
        ref[0,1,2,0]=(0.319532436452)*x_ref[0]+(0.312388869947)*x_ref[1]+(0.237025751349)*x_ref[2]
        ref[0,1,2,1]=(0.713884175591)*x_ref[0]+(1.10911840343)*x_ref[1]+(-1.4943567087)*x_ref[2]
        ref[0,2,0,0]=(-0.599254236697)*x_ref[0]+(-0.190373139957)*x_ref[1]+(-1.75168866737)*x_ref[2]
        ref[0,2,0,1]=(0.396782073165)*x_ref[0]+(-0.252470841512)*x_ref[1]+(-0.463651254287)*x_ref[2]
        ref[0,2,1,0]=(1.03549889185)*x_ref[0]+(0.225894922761)*x_ref[1]+(-0.609139538729)*x_ref[2]
        ref[0,2,1,1]=(0.289458873955)*x_ref[0]+(-0.136668874286)*x_ref[1]+(0.474827071819)*x_ref[2]
        ref[0,2,2,0]=(0.654253727057)*x_ref[0]+(-0.998748120229)*x_ref[1]+(-0.107718066457)*x_ref[2]
        ref[0,2,2,1]=(0.256710826049)*x_ref[0]+(0.309515575678)*x_ref[1]+(-1.60310813555)*x_ref[2]
        ref[0,3,0,0]=(1.70779242771)*x_ref[0]+(-0.530818399689)*x_ref[1]+(-0.516799319419)*x_ref[2]
        ref[0,3,0,1]=(-0.848868447677)*x_ref[0]+(-0.301212439564)*x_ref[1]+(-0.423135852525)*x_ref[2]
        ref[0,3,1,0]=(1.0807833809)*x_ref[0]+(-1.20726021059)*x_ref[1]+(0.565314004443)*x_ref[2]
        ref[0,3,1,1]=(-0.660234771003)*x_ref[0]+(-0.443966617085)*x_ref[1]+(-0.185381740077)*x_ref[2]
        ref[0,3,2,0]=(1.6260513125)*x_ref[0]+(0.145626479867)*x_ref[1]+(-0.808965403823)*x_ref[2]
        ref[0,3,2,1]=(-0.503772598493)*x_ref[0]+(-1.63975947666)*x_ref[1]+(0.255681416016)*x_ref[2]
        ref[0,4,0,0]=(0.0818777313252)*x_ref[0]+(0.428155907605)*x_ref[1]+(-0.642991998051)*x_ref[2]
        ref[0,4,0,1]=(-0.145032953266)*x_ref[0]+(-0.7645425853)*x_ref[1]+(0.458784780224)*x_ref[2]
        ref[0,4,1,0]=(-0.589583511871)*x_ref[0]+(-0.0743175116482)*x_ref[1]+(0.285113485271)*x_ref[2]
        ref[0,4,1,1]=(-1.37305892557)*x_ref[0]+(0.917822397515)*x_ref[1]+(-0.378127728174)*x_ref[2]
        ref[0,4,2,0]=(1.16505232525)*x_ref[0]+(1.10944900772)*x_ref[1]+(-0.95491955738)*x_ref[2]
        ref[0,4,2,1]=(-1.12539432931)*x_ref[0]+(-0.290265022179)*x_ref[1]+(-1.20809121853)*x_ref[2]
        ref[1,0,0,0]=(-0.859420732972)*x_ref[0]+(0.564058561557)*x_ref[1]+(-0.795155516002)*x_ref[2]
        ref[1,0,0,1]=(-0.314313522693)*x_ref[0]+(1.46275003784)*x_ref[1]+(0.285596389281)*x_ref[2]
        ref[1,0,1,0]=(-0.513360534744)*x_ref[0]+(-0.0532410726117)*x_ref[1]+(-0.122433325652)*x_ref[2]
        ref[1,0,1,1]=(0.527488940208)*x_ref[0]+(0.0760609834598)*x_ref[1]+(-0.612300815838)*x_ref[2]
        ref[1,0,2,0]=(0.302549971797)*x_ref[0]+(-0.710901255568)*x_ref[1]+(0.955062818656)*x_ref[2]
        ref[1,0,2,1]=(0.126520802001)*x_ref[0]+(-1.07564234283)*x_ref[1]+(-0.669526781403)*x_ref[2]
        ref[1,1,0,0]=(1.50804296428)*x_ref[0]+(0.111903566342)*x_ref[1]+(0.465876365517)*x_ref[2]
        ref[1,1,0,1]=(-1.04380726708)*x_ref[0]+(0.148245186406)*x_ref[1]+(-1.06801443458)*x_ref[2]
        ref[1,1,1,0]=(1.48236374583)*x_ref[0]+(-0.00794102905726)*x_ref[1]+(-1.35613430084)*x_ref[2]
        ref[1,1,1,1]=(1.04119256641)*x_ref[0]+(-1.84075261029)*x_ref[1]+(-0.776969168414)*x_ref[2]
        ref[1,1,2,0]=(-0.356539020393)*x_ref[0]+(1.70797570799)*x_ref[1]+(-0.525736126605)*x_ref[2]
        ref[1,1,2,1]=(1.62559049194)*x_ref[0]+(-0.482482804647)*x_ref[1]+(0.631790498783)*x_ref[2]
        ref[1,2,0,0]=(0.499315452031)*x_ref[0]+(-0.296700233772)*x_ref[1]+(1.00111627057)*x_ref[2]
        ref[1,2,0,1]=(1.0433119055)*x_ref[0]+(-0.269957745366)*x_ref[1]+(0.591153091169)*x_ref[2]
        ref[1,2,1,0]=(0.502068262492)*x_ref[0]+(-0.849037462803)*x_ref[1]+(-1.56692020458)*x_ref[2]
        ref[1,2,1,1]=(1.2984795003)*x_ref[0]+(-1.07653573134)*x_ref[1]+(0.487232329161)*x_ref[2]
        ref[1,2,2,0]=(-0.109333568443)*x_ref[0]+(0.687162555279)*x_ref[1]+(-0.812636431387)*x_ref[2]
        ref[1,2,2,1]=(0.202326014817)*x_ref[0]+(1.23297194202)*x_ref[1]+(1.41163217274)*x_ref[2]
        ref[1,3,0,0]=(0.124740742544)*x_ref[0]+(-0.500878894409)*x_ref[1]+(-0.376438702019)*x_ref[2]
        ref[1,3,0,1]=(-0.540234624582)*x_ref[0]+(-0.400577181452)*x_ref[1]+(0.879215681428)*x_ref[2]
        ref[1,3,1,0]=(0.0426835015338)*x_ref[0]+(-0.726976626459)*x_ref[1]+(1.32655296741)*x_ref[2]
        ref[1,3,1,1]=(-0.120969369352)*x_ref[0]+(-0.426853376816)*x_ref[1]+(1.88787704976)*x_ref[2]
        ref[1,3,2,0]=(-0.0895115836038)*x_ref[0]+(-1.17409640569)*x_ref[1]+(-0.874853748928)*x_ref[2]
        ref[1,3,2,1]=(-0.0664677570269)*x_ref[0]+(0.708204452159)*x_ref[1]+(-0.26397721498)*x_ref[2]
        ref[1,4,0,0]=(0.0236750184919)*x_ref[0]+(-1.42225228911)*x_ref[1]+(0.399502903376)*x_ref[2]
        ref[1,4,0,1]=(1.03400366274)*x_ref[0]+(-0.442018365874)*x_ref[1]+(1.02491120541)*x_ref[2]
        ref[1,4,1,0]=(-0.150528717616)*x_ref[0]+(-0.512236709632)*x_ref[1]+(-0.0644728802768)*x_ref[2]
        ref[1,4,1,1]=(-0.998520493595)*x_ref[0]+(-0.507010096027)*x_ref[1]+(-0.889168150742)*x_ref[2]
        ref[1,4,2,0]=(0.179165278401)*x_ref[0]+(1.07136931045)*x_ref[1]+(-0.790101000847)*x_ref[2]
        ref[1,4,2,1]=(0.376877479598)*x_ref[0]+(-0.25979387806)*x_ref[1]+(0.446682478544)*x_ref[2]
        ref[2,0,0,0]=(0.365668617405)*x_ref[0]+(1.44268125148)*x_ref[1]+(1.65785163405)*x_ref[2]
        ref[2,0,0,1]=(0.576180639993)*x_ref[0]+(-0.0571374651916)*x_ref[1]+(-0.0854307657892)*x_ref[2]
        ref[2,0,1,0]=(0.0474860699991)*x_ref[0]+(-1.47716457199)*x_ref[1]+(0.87300241949)*x_ref[2]
        ref[2,0,1,1]=(-0.293608884991)*x_ref[0]+(0.0963236497693)*x_ref[1]+(-1.31217901681)*x_ref[2]
        ref[2,0,2,0]=(-0.725125904522)*x_ref[0]+(0.15259525725)*x_ref[1]+(0.0624842579957)*x_ref[2]
        ref[2,0,2,1]=(-0.374667932076)*x_ref[0]+(-0.586704716252)*x_ref[1]+(-0.208259155061)*x_ref[2]
        ref[2,1,0,0]=(0.480010407761)*x_ref[0]+(-1.78856630141)*x_ref[1]+(0.296814589368)*x_ref[2]
        ref[2,1,0,1]=(-0.131871389585)*x_ref[0]+(0.0984653022698)*x_ref[1]+(0.222785695838)*x_ref[2]
        ref[2,1,1,0]=(-0.4095594348)*x_ref[0]+(-0.746059065385)*x_ref[1]+(1.72282248984)*x_ref[2]
        ref[2,1,1,1]=(0.841882742791)*x_ref[0]+(0.698803891138)*x_ref[1]+(0.201279240318)*x_ref[2]
        ref[2,1,2,0]=(-1.52503851146)*x_ref[0]+(0.746778574384)*x_ref[1]+(0.820127798751)*x_ref[2]
        ref[2,1,2,1]=(0.578616447592)*x_ref[0]+(-0.232884112626)*x_ref[1]+(0.719949134498)*x_ref[2]
        ref[2,2,0,0]=(-0.250869865455)*x_ref[0]+(0.0457838553118)*x_ref[1]+(0.694605224088)*x_ref[2]
        ref[2,2,0,1]=(-0.344703134894)*x_ref[0]+(-0.945296587028)*x_ref[1]+(-0.408991625518)*x_ref[2]
        ref[2,2,1,0]=(1.15842118069)*x_ref[0]+(0.385018100988)*x_ref[1]+(-0.298867420422)*x_ref[2]
        ref[2,2,1,1]=(-0.992212925836)*x_ref[0]+(-0.303921757453)*x_ref[1]+(1.90996417539)*x_ref[2]
        ref[2,2,2,0]=(0.139346086335)*x_ref[0]+(-1.5255000175)*x_ref[1]+(-0.418223280061)*x_ref[2]
        ref[2,2,2,1]=(-0.287165354844)*x_ref[0]+(-0.563470113789)*x_ref[1]+(-0.267078242542)*x_ref[2]
        ref[2,3,0,0]=(-0.877956931038)*x_ref[0]+(-0.587379404687)*x_ref[1]+(0.223772989101)*x_ref[2]
        ref[2,3,0,1]=(0.450240028907)*x_ref[0]+(0.933649504645)*x_ref[1]+(0.922982912622)*x_ref[2]
        ref[2,3,1,0]=(-0.695647633954)*x_ref[0]+(0.675123237641)*x_ref[1]+(0.333013288603)*x_ref[2]
        ref[2,3,1,1]=(-0.638319791165)*x_ref[0]+(-1.01905860587)*x_ref[1]+(-0.0829897514576)*x_ref[2]
        ref[2,3,2,0]=(-0.85507363839)*x_ref[0]+(0.213154292163)*x_ref[1]+(-1.19634907609)*x_ref[2]
        ref[2,3,2,1]=(0.51969848742)*x_ref[0]+(-0.103353752424)*x_ref[1]+(1.38100947757)*x_ref[2]
        ref[2,4,0,0]=(-1.42264362114)*x_ref[0]+(-0.0622861012095)*x_ref[1]+(1.11415860946)*x_ref[2]
        ref[2,4,0,1]=(1.03146522202)*x_ref[0]+(-0.59775537826)*x_ref[1]+(0.455451132416)*x_ref[2]
        ref[2,4,1,0]=(-1.42367400988)*x_ref[0]+(0.468019196451)*x_ref[1]+(-0.0449222277683)*x_ref[2]
        ref[2,4,1,1]=(-0.337745267456)*x_ref[0]+(-1.24573098041)*x_ref[1]+(0.591411642381)*x_ref[2]
        ref[2,4,2,0]=(-0.773220236551)*x_ref[0]+(-0.92934639777)*x_ref[1]+(-0.323666905212)*x_ref[2]
        ref[2,4,2,1]=(-1.25416565442)*x_ref[0]+(0.10804460369)*x_ref[1]+(1.3398483009)*x_ref[2]
        ref[3,0,0,0]=(-0.187928573247)*x_ref[0]+(0.952793598709)*x_ref[1]+(-0.651544295118)*x_ref[2]
        ref[3,0,0,1]=(-1.18183645576)*x_ref[0]+(-0.767480258477)*x_ref[1]+(0.0733909569555)*x_ref[2]
        ref[3,0,1,0]=(0.825156828887)*x_ref[0]+(0.900307840844)*x_ref[1]+(-1.03153118659)*x_ref[2]
        ref[3,0,1,1]=(-0.150209523447)*x_ref[0]+(1.07490578747)*x_ref[1]+(-1.6916184583)*x_ref[2]
        ref[3,0,2,0]=(-1.07041023688)*x_ref[0]+(-0.420347448824)*x_ref[1]+(-0.556885268114)*x_ref[2]
        ref[3,0,2,1]=(-1.30609527635)*x_ref[0]+(-0.10753520843)*x_ref[1]+(0.240258152808)*x_ref[2]
        ref[3,1,0,0]=(0.899651818251)*x_ref[0]+(0.00909698197696)*x_ref[1]+(-1.64985946319)*x_ref[2]
        ref[3,1,0,1]=(-0.880471412731)*x_ref[0]+(0.678893404261)*x_ref[1]+(0.288497390335)*x_ref[2]
        ref[3,1,1,0]=(-0.00910924813519)*x_ref[0]+(-1.30365942504)*x_ref[1]+(-0.568919628746)*x_ref[2]
        ref[3,1,1,1]=(-0.811256892653)*x_ref[0]+(-0.0284400402628)*x_ref[1]+(0.727386097819)*x_ref[2]
        ref[3,1,2,0]=(0.105755186759)*x_ref[0]+(0.196242692495)*x_ref[1]+(0.358516596542)*x_ref[2]
        ref[3,1,2,1]=(-0.37985806916)*x_ref[0]+(0.44810760494)*x_ref[1]+(0.633807267038)*x_ref[2]
        ref[3,2,0,0]=(1.34921360449)*x_ref[0]+(-0.9349137549)*x_ref[1]+(-0.272245661092)*x_ref[2]
        ref[3,2,0,1]=(1.21020688092)*x_ref[0]+(1.1296074258)*x_ref[1]+(0.000391430715188)*x_ref[2]
        ref[3,2,1,0]=(-1.25769018387)*x_ref[0]+(-0.786491629941)*x_ref[1]+(-0.862105545664)*x_ref[2]
        ref[3,2,1,1]=(0.668529227552)*x_ref[0]+(-0.983024344824)*x_ref[1]+(0.74846424728)*x_ref[2]
        ref[3,2,2,0]=(0.45487084108)*x_ref[0]+(0.855491649411)*x_ref[1]+(-0.104621648479)*x_ref[2]
        ref[3,2,2,1]=(0.802451481866)*x_ref[0]+(-0.137058520895)*x_ref[1]+(0.573940225359)*x_ref[2]
        ref[3,3,0,0]=(0.468999526003)*x_ref[0]+(0.504917415809)*x_ref[1]+(-1.39187140958)*x_ref[2]
        ref[3,3,0,1]=(0.501007631838)*x_ref[0]+(-0.285379909295)*x_ref[1]+(0.257752560803)*x_ref[2]
        ref[3,3,1,0]=(-1.01423550219)*x_ref[0]+(-0.749951834747)*x_ref[1]+(-0.156550300135)*x_ref[2]
        ref[3,3,1,1]=(0.190954159364)*x_ref[0]+(0.985690213898)*x_ref[1]+(-0.942349242057)*x_ref[2]
        ref[3,3,2,0]=(-1.04776017317)*x_ref[0]+(1.3978257599)*x_ref[1]+(-0.727385312507)*x_ref[2]
        ref[3,3,2,1]=(0.118574662458)*x_ref[0]+(0.763657521414)*x_ref[1]+(-0.947854404873)*x_ref[2]
        ref[3,4,0,0]=(-1.42375158871)*x_ref[0]+(1.02577770584)*x_ref[1]+(1.7262667856)*x_ref[2]
        ref[3,4,0,1]=(0.819198798755)*x_ref[0]+(-0.177472792322)*x_ref[1]+(-0.495119104783)*x_ref[2]
        ref[3,4,1,0]=(-1.44070928104)*x_ref[0]+(-0.67376984043)*x_ref[1]+(0.314203511381)*x_ref[2]
        ref[3,4,1,1]=(-0.0417551422406)*x_ref[0]+(-0.446166510311)*x_ref[1]+(-0.303922120635)*x_ref[2]
        ref[3,4,2,0]=(0.161905825888)*x_ref[0]+(-0.433001417416)*x_ref[1]+(0.713602984094)*x_ref[2]
        ref[3,4,2,1]=(1.29579174225)*x_ref[0]+(-0.136051917273)*x_ref[1]+(0.270836816064)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromSymbol_ReducedContinuousFunction_rank0(self):
      """
      tests interpolate for rank 0 Symbol onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedContinuousFunction onto ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(),dim=dim)
      s=Data(0,(),w)
      ref=Data(0,(),w_ref)
      if dim==2:
        s=(0.14488970011)*x[0]+(-0.430489267095)*x[1]
        ref=(0.14488970011)*x_ref[0]+(-0.430489267095)*x_ref[1]
      else:
        s=(-0.0442386353051)*x[0]+(0.0394615944862)*x[1]+(-1.29904883856)*x[2]
        ref=(-0.0442386353051)*x_ref[0]+(0.0394615944862)*x_ref[1]+(-1.29904883856)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromSymbol_ReducedContinuousFunction_rank1(self):
      """
      tests interpolate for rank 1 Symbol onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedContinuousFunction onto ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(2,),dim=dim)
      s=Data(0,(2,),w)
      ref=Data(0,(2,),w_ref)
      if dim==2:
        s[0]=(0.0426949725928)*x[0]+(0.796580332975)*x[1]
        s[1]=(0.297643604255)*x[0]+(0.501403166746)*x[1]
        ref[0]=(0.0426949725928)*x[0]+(0.796580332975)*x[1]
        ref[1]=(0.297643604255)*x[0]+(0.501403166746)*x[1]
      else:
        s[0]=(0.820713117949)*x[0]+(0.0164537336201)*x[1]+(-0.130931974587)*x[2]
        s[1]=(-0.155153784636)*x[0]+(-0.48729229521)*x[1]+(-0.875514015614)*x[2]
        ref[0]=(0.820713117949)*x[0]+(0.0164537336201)*x[1]+(-0.130931974587)*x[2]
        ref[1]=(-0.155153784636)*x[0]+(-0.48729229521)*x[1]+(-0.875514015614)*x[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromSymbol_ReducedContinuousFunction_rank2(self):
      """
      tests interpolate for rank 2 Symbol onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedContinuousFunction onto ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5),dim=dim)
      s=Data(0,(4, 5),w)
      ref=Data(0,(4, 5),w_ref)
      if dim==2:
        s[0,0]=(0.918593737351)*x[0]+(-0.183298779344)*x[1]
        s[0,1]=(1.01407474954)*x[0]+(-1.14888338084)*x[1]
        s[0,2]=(-0.0702606564619)*x[0]+(-0.104586136782)*x[1]
        s[0,3]=(-1.00520814193)*x[0]+(-0.959854266357)*x[1]
        s[0,4]=(-1.11463618446)*x[0]+(-0.794825170571)*x[1]
        s[1,0]=(-1.14418607702)*x[0]+(-0.273235396058)*x[1]
        s[1,1]=(0.641683109978)*x[0]+(0.685097028758)*x[1]
        s[1,2]=(0.833533353805)*x[0]+(-0.769590460713)*x[1]
        s[1,3]=(1.44918354254)*x[0]+(0.78941323689)*x[1]
        s[1,4]=(0.968133426719)*x[0]+(-0.923562755675)*x[1]
        s[2,0]=(1.58053224399)*x[0]+(0.600967207116)*x[1]
        s[2,1]=(1.04054361996)*x[0]+(-1.71519167959)*x[1]
        s[2,2]=(-1.55202351352)*x[0]+(-1.06650593034)*x[1]
        s[2,3]=(0.894986308301)*x[0]+(-0.767078277329)*x[1]
        s[2,4]=(-0.804853431103)*x[0]+(-0.44572378652)*x[1]
        s[3,0]=(0.90347844754)*x[0]+(0.357148881796)*x[1]
        s[3,1]=(0.0552482783049)*x[0]+(-0.652692619264)*x[1]
        s[3,2]=(-0.251809701815)*x[0]+(1.89155992551)*x[1]
        s[3,3]=(0.43181695535)*x[0]+(-0.228997029489)*x[1]
        s[3,4]=(-0.522831785326)*x[0]+(-0.372437316342)*x[1]
        ref[0,0]=(0.918593737351)*x_ref[0]+(-0.183298779344)*x_ref[1]
        ref[0,1]=(1.01407474954)*x_ref[0]+(-1.14888338084)*x_ref[1]
        ref[0,2]=(-0.0702606564619)*x_ref[0]+(-0.104586136782)*x_ref[1]
        ref[0,3]=(-1.00520814193)*x_ref[0]+(-0.959854266357)*x_ref[1]
        ref[0,4]=(-1.11463618446)*x_ref[0]+(-0.794825170571)*x_ref[1]
        ref[1,0]=(-1.14418607702)*x_ref[0]+(-0.273235396058)*x_ref[1]
        ref[1,1]=(0.641683109978)*x_ref[0]+(0.685097028758)*x_ref[1]
        ref[1,2]=(0.833533353805)*x_ref[0]+(-0.769590460713)*x_ref[1]
        ref[1,3]=(1.44918354254)*x_ref[0]+(0.78941323689)*x_ref[1]
        ref[1,4]=(0.968133426719)*x_ref[0]+(-0.923562755675)*x_ref[1]
        ref[2,0]=(1.58053224399)*x_ref[0]+(0.600967207116)*x_ref[1]
        ref[2,1]=(1.04054361996)*x_ref[0]+(-1.71519167959)*x_ref[1]
        ref[2,2]=(-1.55202351352)*x_ref[0]+(-1.06650593034)*x_ref[1]
        ref[2,3]=(0.894986308301)*x_ref[0]+(-0.767078277329)*x_ref[1]
        ref[2,4]=(-0.804853431103)*x_ref[0]+(-0.44572378652)*x_ref[1]
        ref[3,0]=(0.90347844754)*x_ref[0]+(0.357148881796)*x_ref[1]
        ref[3,1]=(0.0552482783049)*x_ref[0]+(-0.652692619264)*x_ref[1]
        ref[3,2]=(-0.251809701815)*x_ref[0]+(1.89155992551)*x_ref[1]
        ref[3,3]=(0.43181695535)*x_ref[0]+(-0.228997029489)*x_ref[1]
        ref[3,4]=(-0.522831785326)*x_ref[0]+(-0.372437316342)*x_ref[1]
      else:
        s[0,0]=(0.221916152348)*x[0]+(1.46835470229)*x[1]+(0.300487757142)*x[2]
        s[0,1]=(-0.15334206717)*x[0]+(0.506590749852)*x[1]+(0.477134329524)*x[2]
        s[0,2]=(0.786821788538)*x[0]+(-0.0141797716133)*x[1]+(-1.67421275557)*x[2]
        s[0,3]=(-1.14438881368)*x[0]+(-0.621323583056)*x[1]+(-0.0198865298096)*x[2]
        s[0,4]=(-0.195154511305)*x[0]+(-0.237596745083)*x[1]+(-0.36680113021)*x[2]
        s[1,0]=(-0.0476767967515)*x[0]+(-1.19950396856)*x[1]+(0.12823672742)*x[2]
        s[1,1]=(-0.420573984429)*x[0]+(0.100192190413)*x[1]+(1.30995115077)*x[2]
        s[1,2]=(1.34087691694)*x[0]+(-0.151927765846)*x[1]+(0.417185746786)*x[2]
        s[1,3]=(-0.383933595451)*x[0]+(0.208340861893)*x[1]+(0.947878917669)*x[2]
        s[1,4]=(-0.143686438189)*x[0]+(-0.00211348684153)*x[1]+(-0.859362566827)*x[2]
        s[2,0]=(0.447423746179)*x[0]+(-0.636241203952)*x[1]+(-0.0949114725339)*x[2]
        s[2,1]=(-0.0413740104938)*x[0]+(0.587001644427)*x[1]+(-0.544886520536)*x[2]
        s[2,2]=(-1.07017617383)*x[0]+(-0.762147671415)*x[1]+(-0.248360379513)*x[2]
        s[2,3]=(0.391720300962)*x[0]+(1.6521113321)*x[1]+(-0.408205550425)*x[2]
        s[2,4]=(1.2369249321)*x[0]+(0.952522466347)*x[1]+(-0.404645424067)*x[2]
        s[3,0]=(0.62410800627)*x[0]+(-0.308929187117)*x[1]+(-0.703796873161)*x[2]
        s[3,1]=(0.676590956976)*x[0]+(-1.39642714827)*x[1]+(0.198267797885)*x[2]
        s[3,2]=(-0.202804694134)*x[0]+(0.187577851273)*x[1]+(-0.700960966206)*x[2]
        s[3,3]=(-0.799833564134)*x[0]+(-0.455781630028)*x[1]+(0.453451623154)*x[2]
        s[3,4]=(-0.660373484066)*x[0]+(-1.55472528981)*x[1]+(0.0751348530851)*x[2]
        ref[0,0]=(0.221916152348)*x_ref[0]+(1.46835470229)*x_ref[1]+(0.300487757142)*x_ref[2]
        ref[0,1]=(-0.15334206717)*x_ref[0]+(0.506590749852)*x_ref[1]+(0.477134329524)*x_ref[2]
        ref[0,2]=(0.786821788538)*x_ref[0]+(-0.0141797716133)*x_ref[1]+(-1.67421275557)*x_ref[2]
        ref[0,3]=(-1.14438881368)*x_ref[0]+(-0.621323583056)*x_ref[1]+(-0.0198865298096)*x_ref[2]
        ref[0,4]=(-0.195154511305)*x_ref[0]+(-0.237596745083)*x_ref[1]+(-0.36680113021)*x_ref[2]
        ref[1,0]=(-0.0476767967515)*x_ref[0]+(-1.19950396856)*x_ref[1]+(0.12823672742)*x_ref[2]
        ref[1,1]=(-0.420573984429)*x_ref[0]+(0.100192190413)*x_ref[1]+(1.30995115077)*x_ref[2]
        ref[1,2]=(1.34087691694)*x_ref[0]+(-0.151927765846)*x_ref[1]+(0.417185746786)*x_ref[2]
        ref[1,3]=(-0.383933595451)*x_ref[0]+(0.208340861893)*x_ref[1]+(0.947878917669)*x_ref[2]
        ref[1,4]=(-0.143686438189)*x_ref[0]+(-0.00211348684153)*x_ref[1]+(-0.859362566827)*x_ref[2]
        ref[2,0]=(0.447423746179)*x_ref[0]+(-0.636241203952)*x_ref[1]+(-0.0949114725339)*x_ref[2]
        ref[2,1]=(-0.0413740104938)*x_ref[0]+(0.587001644427)*x_ref[1]+(-0.544886520536)*x_ref[2]
        ref[2,2]=(-1.07017617383)*x_ref[0]+(-0.762147671415)*x_ref[1]+(-0.248360379513)*x_ref[2]
        ref[2,3]=(0.391720300962)*x_ref[0]+(1.6521113321)*x_ref[1]+(-0.408205550425)*x_ref[2]
        ref[2,4]=(1.2369249321)*x_ref[0]+(0.952522466347)*x_ref[1]+(-0.404645424067)*x_ref[2]
        ref[3,0]=(0.62410800627)*x_ref[0]+(-0.308929187117)*x_ref[1]+(-0.703796873161)*x_ref[2]
        ref[3,1]=(0.676590956976)*x_ref[0]+(-1.39642714827)*x_ref[1]+(0.198267797885)*x_ref[2]
        ref[3,2]=(-0.202804694134)*x_ref[0]+(0.187577851273)*x_ref[1]+(-0.700960966206)*x_ref[2]
        ref[3,3]=(-0.799833564134)*x_ref[0]+(-0.455781630028)*x_ref[1]+(0.453451623154)*x_ref[2]
        ref[3,4]=(-0.660373484066)*x_ref[0]+(-1.55472528981)*x_ref[1]+(0.0751348530851)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromSymbol_ReducedContinuousFunction_rank3(self):
      """
      tests interpolate for rank 3 Symbol onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedContinuousFunction onto ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(6, 2, 2),dim=dim)
      s=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2),w_ref)
      if dim==2:
        s[0,0,0]=(-1.78813432343)*x[0]+(1.60161751633)*x[1]
        s[0,0,1]=(-1.49849771585)*x[0]+(-0.635435863643)*x[1]
        s[0,1,0]=(-0.906731346391)*x[0]+(0.533678119416)*x[1]
        s[0,1,1]=(-0.792521183513)*x[0]+(0.260998720452)*x[1]
        s[1,0,0]=(-0.710279314945)*x[0]+(0.38834997642)*x[1]
        s[1,0,1]=(-0.168938079833)*x[0]+(-0.502425332838)*x[1]
        s[1,1,0]=(-0.358550156297)*x[0]+(-0.650005639384)*x[1]
        s[1,1,1]=(-1.57793968521)*x[0]+(-0.46781871273)*x[1]
        s[2,0,0]=(0.0112676061889)*x[0]+(0.925017077862)*x[1]
        s[2,0,1]=(-0.109253916051)*x[0]+(-0.38189031862)*x[1]
        s[2,1,0]=(0.777175068327)*x[0]+(1.00196875887)*x[1]
        s[2,1,1]=(-0.27343070108)*x[0]+(-0.11181493242)*x[1]
        s[3,0,0]=(0.321226287504)*x[0]+(0.173667688124)*x[1]
        s[3,0,1]=(1.67405481183)*x[0]+(-0.981404244518)*x[1]
        s[3,1,0]=(0.129874894836)*x[0]+(0.341748165587)*x[1]
        s[3,1,1]=(-0.545737004242)*x[0]+(-1.20936137818)*x[1]
        s[4,0,0]=(0.389007738117)*x[0]+(-0.435046969057)*x[1]
        s[4,0,1]=(1.50296802873)*x[0]+(-0.878616436543)*x[1]
        s[4,1,0]=(1.16067694875)*x[0]+(-0.954317497512)*x[1]
        s[4,1,1]=(1.14634534638)*x[0]+(-0.63981762551)*x[1]
        s[5,0,0]=(0.803750299983)*x[0]+(-0.383611041681)*x[1]
        s[5,0,1]=(0.51863884687)*x[0]+(1.32891347835)*x[1]
        s[5,1,0]=(-0.0775425248593)*x[0]+(-0.0375631918508)*x[1]
        s[5,1,1]=(1.57167118663)*x[0]+(-0.649960686486)*x[1]
        ref[0,0,0]=(-1.78813432343)*x_ref[0]+(1.60161751633)*x_ref[1]
        ref[0,0,1]=(-1.49849771585)*x_ref[0]+(-0.635435863643)*x_ref[1]
        ref[0,1,0]=(-0.906731346391)*x_ref[0]+(0.533678119416)*x_ref[1]
        ref[0,1,1]=(-0.792521183513)*x_ref[0]+(0.260998720452)*x_ref[1]
        ref[1,0,0]=(-0.710279314945)*x_ref[0]+(0.38834997642)*x_ref[1]
        ref[1,0,1]=(-0.168938079833)*x_ref[0]+(-0.502425332838)*x_ref[1]
        ref[1,1,0]=(-0.358550156297)*x_ref[0]+(-0.650005639384)*x_ref[1]
        ref[1,1,1]=(-1.57793968521)*x_ref[0]+(-0.46781871273)*x_ref[1]
        ref[2,0,0]=(0.0112676061889)*x_ref[0]+(0.925017077862)*x_ref[1]
        ref[2,0,1]=(-0.109253916051)*x_ref[0]+(-0.38189031862)*x_ref[1]
        ref[2,1,0]=(0.777175068327)*x_ref[0]+(1.00196875887)*x_ref[1]
        ref[2,1,1]=(-0.27343070108)*x_ref[0]+(-0.11181493242)*x_ref[1]
        ref[3,0,0]=(0.321226287504)*x_ref[0]+(0.173667688124)*x_ref[1]
        ref[3,0,1]=(1.67405481183)*x_ref[0]+(-0.981404244518)*x_ref[1]
        ref[3,1,0]=(0.129874894836)*x_ref[0]+(0.341748165587)*x_ref[1]
        ref[3,1,1]=(-0.545737004242)*x_ref[0]+(-1.20936137818)*x_ref[1]
        ref[4,0,0]=(0.389007738117)*x_ref[0]+(-0.435046969057)*x_ref[1]
        ref[4,0,1]=(1.50296802873)*x_ref[0]+(-0.878616436543)*x_ref[1]
        ref[4,1,0]=(1.16067694875)*x_ref[0]+(-0.954317497512)*x_ref[1]
        ref[4,1,1]=(1.14634534638)*x_ref[0]+(-0.63981762551)*x_ref[1]
        ref[5,0,0]=(0.803750299983)*x_ref[0]+(-0.383611041681)*x_ref[1]
        ref[5,0,1]=(0.51863884687)*x_ref[0]+(1.32891347835)*x_ref[1]
        ref[5,1,0]=(-0.0775425248593)*x_ref[0]+(-0.0375631918508)*x_ref[1]
        ref[5,1,1]=(1.57167118663)*x_ref[0]+(-0.649960686486)*x_ref[1]
      else:
        s[0,0,0]=(1.00224477774)*x[0]+(0.276541980132)*x[1]+(-1.61268557504)*x[2]
        s[0,0,1]=(0.182197948486)*x[0]+(-0.314887566213)*x[1]+(0.172753958549)*x[2]
        s[0,1,0]=(-0.225255532228)*x[0]+(1.64823628561)*x[1]+(0.00289115479583)*x[2]
        s[0,1,1]=(-0.0081734432798)*x[0]+(-1.40127403863)*x[1]+(-0.878765016785)*x[2]
        s[1,0,0]=(-0.88802398859)*x[0]+(0.980539857729)*x[1]+(0.106008000545)*x[2]
        s[1,0,1]=(0.413338006112)*x[0]+(0.238889884193)*x[1]+(0.216982176844)*x[2]
        s[1,1,0]=(-0.375359689292)*x[0]+(0.656302411812)*x[1]+(-1.13744732251)*x[2]
        s[1,1,1]=(-0.768638146497)*x[0]+(0.382131491456)*x[1]+(-0.0473187661226)*x[2]
        s[2,0,0]=(0.727474830988)*x[0]+(-1.15897383904)*x[1]+(1.03107954393)*x[2]
        s[2,0,1]=(-0.565279703477)*x[0]+(0.952275155472)*x[1]+(1.72082557166)*x[2]
        s[2,1,0]=(-0.504951356904)*x[0]+(0.0945851323338)*x[1]+(-1.18137582843)*x[2]
        s[2,1,1]=(0.441559000376)*x[0]+(0.808481091433)*x[1]+(-0.185790180692)*x[2]
        s[3,0,0]=(1.097525477)*x[0]+(1.09598373235)*x[1]+(-0.225656362018)*x[2]
        s[3,0,1]=(-1.64387091178)*x[0]+(0.161580847642)*x[1]+(0.616741154516)*x[2]
        s[3,1,0]=(0.518052869517)*x[0]+(-0.491037975466)*x[1]+(-0.320485227716)*x[2]
        s[3,1,1]=(0.0299906539271)*x[0]+(-0.161268046678)*x[1]+(-0.933417226819)*x[2]
        s[4,0,0]=(-1.45814177886)*x[0]+(-1.01057484083)*x[1]+(1.05668957085)*x[2]
        s[4,0,1]=(0.335720229493)*x[0]+(0.155778050975)*x[1]+(-0.479943899005)*x[2]
        s[4,1,0]=(0.424166905697)*x[0]+(1.04188647689)*x[1]+(-0.154173690871)*x[2]
        s[4,1,1]=(-0.663564713586)*x[0]+(-0.744257330117)*x[1]+(-0.586452598078)*x[2]
        s[5,0,0]=(-0.308954778807)*x[0]+(0.0799719560375)*x[1]+(0.128085782808)*x[2]
        s[5,0,1]=(0.548010504324)*x[0]+(-0.676431304167)*x[1]+(0.12659732457)*x[2]
        s[5,1,0]=(0.17131336527)*x[0]+(-1.54908008145)*x[1]+(0.212836465224)*x[2]
        s[5,1,1]=(-0.834644454471)*x[0]+(1.52867678218)*x[1]+(1.19656096059)*x[2]
        ref[0,0,0]=(1.00224477774)*x_ref[0]+(0.276541980132)*x_ref[1]+(-1.61268557504)*x_ref[2]
        ref[0,0,1]=(0.182197948486)*x_ref[0]+(-0.314887566213)*x_ref[1]+(0.172753958549)*x_ref[2]
        ref[0,1,0]=(-0.225255532228)*x_ref[0]+(1.64823628561)*x_ref[1]+(0.00289115479583)*x_ref[2]
        ref[0,1,1]=(-0.0081734432798)*x_ref[0]+(-1.40127403863)*x_ref[1]+(-0.878765016785)*x_ref[2]
        ref[1,0,0]=(-0.88802398859)*x_ref[0]+(0.980539857729)*x_ref[1]+(0.106008000545)*x_ref[2]
        ref[1,0,1]=(0.413338006112)*x_ref[0]+(0.238889884193)*x_ref[1]+(0.216982176844)*x_ref[2]
        ref[1,1,0]=(-0.375359689292)*x_ref[0]+(0.656302411812)*x_ref[1]+(-1.13744732251)*x_ref[2]
        ref[1,1,1]=(-0.768638146497)*x_ref[0]+(0.382131491456)*x_ref[1]+(-0.0473187661226)*x_ref[2]
        ref[2,0,0]=(0.727474830988)*x_ref[0]+(-1.15897383904)*x_ref[1]+(1.03107954393)*x_ref[2]
        ref[2,0,1]=(-0.565279703477)*x_ref[0]+(0.952275155472)*x_ref[1]+(1.72082557166)*x_ref[2]
        ref[2,1,0]=(-0.504951356904)*x_ref[0]+(0.0945851323338)*x_ref[1]+(-1.18137582843)*x_ref[2]
        ref[2,1,1]=(0.441559000376)*x_ref[0]+(0.808481091433)*x_ref[1]+(-0.185790180692)*x_ref[2]
        ref[3,0,0]=(1.097525477)*x_ref[0]+(1.09598373235)*x_ref[1]+(-0.225656362018)*x_ref[2]
        ref[3,0,1]=(-1.64387091178)*x_ref[0]+(0.161580847642)*x_ref[1]+(0.616741154516)*x_ref[2]
        ref[3,1,0]=(0.518052869517)*x_ref[0]+(-0.491037975466)*x_ref[1]+(-0.320485227716)*x_ref[2]
        ref[3,1,1]=(0.0299906539271)*x_ref[0]+(-0.161268046678)*x_ref[1]+(-0.933417226819)*x_ref[2]
        ref[4,0,0]=(-1.45814177886)*x_ref[0]+(-1.01057484083)*x_ref[1]+(1.05668957085)*x_ref[2]
        ref[4,0,1]=(0.335720229493)*x_ref[0]+(0.155778050975)*x_ref[1]+(-0.479943899005)*x_ref[2]
        ref[4,1,0]=(0.424166905697)*x_ref[0]+(1.04188647689)*x_ref[1]+(-0.154173690871)*x_ref[2]
        ref[4,1,1]=(-0.663564713586)*x_ref[0]+(-0.744257330117)*x_ref[1]+(-0.586452598078)*x_ref[2]
        ref[5,0,0]=(-0.308954778807)*x_ref[0]+(0.0799719560375)*x_ref[1]+(0.128085782808)*x_ref[2]
        ref[5,0,1]=(0.548010504324)*x_ref[0]+(-0.676431304167)*x_ref[1]+(0.12659732457)*x_ref[2]
        ref[5,1,0]=(0.17131336527)*x_ref[0]+(-1.54908008145)*x_ref[1]+(0.212836465224)*x_ref[2]
        ref[5,1,1]=(-0.834644454471)*x_ref[0]+(1.52867678218)*x_ref[1]+(1.19656096059)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromSymbol_ReducedContinuousFunction_rank4(self):
      """
      tests interpolate for rank 4 Symbol onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedContinuousFunction onto ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5, 3, 2),dim=dim)
      s=Data(0,(4, 5, 3, 2),w)
      ref=Data(0,(4, 5, 3, 2),w_ref)
      if dim==2:
        s[0,0,0,0]=(-0.110140466592)*x[0]+(0.758992308989)*x[1]
        s[0,0,0,1]=(-1.27611018159)*x[0]+(-1.21469838588)*x[1]
        s[0,0,1,0]=(-0.695719260931)*x[0]+(-0.631650774678)*x[1]
        s[0,0,1,1]=(-0.0446786678677)*x[0]+(-0.547249041592)*x[1]
        s[0,0,2,0]=(0.824673646909)*x[0]+(-1.53499816213)*x[1]
        s[0,0,2,1]=(0.902244689111)*x[0]+(0.194498326222)*x[1]
        s[0,1,0,0]=(-1.29097330044)*x[0]+(0.0328620774092)*x[1]
        s[0,1,0,1]=(1.20419739576)*x[0]+(-1.25415670694)*x[1]
        s[0,1,1,0]=(0.825311334936)*x[0]+(0.732241589298)*x[1]
        s[0,1,1,1]=(0.239743697336)*x[0]+(0.446701903969)*x[1]
        s[0,1,2,0]=(-0.208525758105)*x[0]+(0.564727160849)*x[1]
        s[0,1,2,1]=(0.425352221432)*x[0]+(-0.439885368017)*x[1]
        s[0,2,0,0]=(0.089403990032)*x[0]+(0.927312736507)*x[1]
        s[0,2,0,1]=(-1.98342026057)*x[0]+(0.908824462154)*x[1]
        s[0,2,1,0]=(-1.02677966987)*x[0]+(1.64009929611)*x[1]
        s[0,2,1,1]=(1.18506229959)*x[0]+(-0.336041064484)*x[1]
        s[0,2,2,0]=(0.124121400983)*x[0]+(0.169858827797)*x[1]
        s[0,2,2,1]=(0.0171983721215)*x[0]+(-0.679377487833)*x[1]
        s[0,3,0,0]=(0.707474586567)*x[0]+(-0.327636474016)*x[1]
        s[0,3,0,1]=(-0.183596474381)*x[0]+(-1.07263498957)*x[1]
        s[0,3,1,0]=(1.49812660538)*x[0]+(-0.313209529867)*x[1]
        s[0,3,1,1]=(-0.678162796237)*x[0]+(-0.823724533082)*x[1]
        s[0,3,2,0]=(-0.79806833841)*x[0]+(0.865576104504)*x[1]
        s[0,3,2,1]=(0.941864201983)*x[0]+(0.734846756813)*x[1]
        s[0,4,0,0]=(-0.205171531689)*x[0]+(0.299610959481)*x[1]
        s[0,4,0,1]=(0.643430728461)*x[0]+(0.584994687875)*x[1]
        s[0,4,1,0]=(-0.408681959359)*x[0]+(0.601772358172)*x[1]
        s[0,4,1,1]=(-1.44472725121)*x[0]+(-0.0899809192849)*x[1]
        s[0,4,2,0]=(0.722764720152)*x[0]+(-0.407449476647)*x[1]
        s[0,4,2,1]=(0.166269375199)*x[0]+(1.20016653459)*x[1]
        s[1,0,0,0]=(-0.764665577343)*x[0]+(0.15492015266)*x[1]
        s[1,0,0,1]=(0.822023595189)*x[0]+(0.00511531586673)*x[1]
        s[1,0,1,0]=(0.0840294388025)*x[0]+(0.393606253186)*x[1]
        s[1,0,1,1]=(-0.702650062915)*x[0]+(-1.10361836979)*x[1]
        s[1,0,2,0]=(-1.34569564836)*x[0]+(-1.50816431101)*x[1]
        s[1,0,2,1]=(1.11337280315)*x[0]+(-0.0771805114838)*x[1]
        s[1,1,0,0]=(-0.0695861126454)*x[0]+(-0.437020174051)*x[1]
        s[1,1,0,1]=(1.68924343517)*x[0]+(0.706853885933)*x[1]
        s[1,1,1,0]=(0.0806933170819)*x[0]+(-0.974818406182)*x[1]
        s[1,1,1,1]=(-0.236836420312)*x[0]+(-0.352629546416)*x[1]
        s[1,1,2,0]=(-0.312697720062)*x[0]+(-0.977312197678)*x[1]
        s[1,1,2,1]=(-0.263563591998)*x[0]+(-0.899465194216)*x[1]
        s[1,2,0,0]=(1.0599010032)*x[0]+(0.625641179018)*x[1]
        s[1,2,0,1]=(0.42589369081)*x[0]+(0.63136683361)*x[1]
        s[1,2,1,0]=(0.629519575902)*x[0]+(-1.03125707286)*x[1]
        s[1,2,1,1]=(0.479272369893)*x[0]+(-1.63724584249)*x[1]
        s[1,2,2,0]=(1.48877937845)*x[0]+(-0.802308310767)*x[1]
        s[1,2,2,1]=(0.374636882204)*x[0]+(0.187733613588)*x[1]
        s[1,3,0,0]=(0.360148707925)*x[0]+(1.48869239048)*x[1]
        s[1,3,0,1]=(-1.29194970756)*x[0]+(-0.433561540189)*x[1]
        s[1,3,1,0]=(0.696118719028)*x[0]+(1.73199962229)*x[1]
        s[1,3,1,1]=(1.63831809651)*x[0]+(0.70687058079)*x[1]
        s[1,3,2,0]=(1.03614006811)*x[0]+(-0.783037005502)*x[1]
        s[1,3,2,1]=(0.351931393443)*x[0]+(-0.720060584954)*x[1]
        s[1,4,0,0]=(0.856138152361)*x[0]+(-0.0733689526147)*x[1]
        s[1,4,0,1]=(-0.256696810525)*x[0]+(1.57547133796)*x[1]
        s[1,4,1,0]=(0.260933620175)*x[0]+(-0.528418816229)*x[1]
        s[1,4,1,1]=(-1.14624698517)*x[0]+(0.537831890429)*x[1]
        s[1,4,2,0]=(-1.11328030308)*x[0]+(0.477599527943)*x[1]
        s[1,4,2,1]=(-1.52135700331)*x[0]+(1.18803329651)*x[1]
        s[2,0,0,0]=(1.02103992957)*x[0]+(1.3428520083)*x[1]
        s[2,0,0,1]=(0.74126089694)*x[0]+(-0.747092126031)*x[1]
        s[2,0,1,0]=(-0.300534057945)*x[0]+(-0.0679226717228)*x[1]
        s[2,0,1,1]=(0.205780243466)*x[0]+(1.89346679377)*x[1]
        s[2,0,2,0]=(0.711997044871)*x[0]+(0.614186591236)*x[1]
        s[2,0,2,1]=(0.562225764659)*x[0]+(-0.75210737336)*x[1]
        s[2,1,0,0]=(1.51024763079)*x[0]+(0.0655411043245)*x[1]
        s[2,1,0,1]=(-1.60270171758)*x[0]+(-0.555919918868)*x[1]
        s[2,1,1,0]=(0.0117250633883)*x[0]+(1.06089889984)*x[1]
        s[2,1,1,1]=(1.41839826051)*x[0]+(-1.11883387918)*x[1]
        s[2,1,2,0]=(-1.47481010167)*x[0]+(0.21986710996)*x[1]
        s[2,1,2,1]=(-0.890780004667)*x[0]+(0.68206435926)*x[1]
        s[2,2,0,0]=(0.00647580761798)*x[0]+(1.08988646609)*x[1]
        s[2,2,0,1]=(0.107576626365)*x[0]+(0.569541010093)*x[1]
        s[2,2,1,0]=(0.655745883134)*x[0]+(1.59781027174)*x[1]
        s[2,2,1,1]=(0.423699701576)*x[0]+(-0.854187892248)*x[1]
        s[2,2,2,0]=(-0.904849730929)*x[0]+(0.235074145696)*x[1]
        s[2,2,2,1]=(0.981525463443)*x[0]+(0.701832385796)*x[1]
        s[2,3,0,0]=(-0.489533883573)*x[0]+(-0.0213717056714)*x[1]
        s[2,3,0,1]=(0.516832539105)*x[0]+(-1.11995329355)*x[1]
        s[2,3,1,0]=(0.287740517981)*x[0]+(-1.3210942817)*x[1]
        s[2,3,1,1]=(0.179604184524)*x[0]+(-1.63664715586)*x[1]
        s[2,3,2,0]=(-0.492296320142)*x[0]+(-0.502235503805)*x[1]
        s[2,3,2,1]=(-0.653587673689)*x[0]+(0.5466761379)*x[1]
        s[2,4,0,0]=(1.37393629094)*x[0]+(0.645991033265)*x[1]
        s[2,4,0,1]=(-0.108057947979)*x[0]+(-0.761267645781)*x[1]
        s[2,4,1,0]=(-0.0900524038235)*x[0]+(0.16773443574)*x[1]
        s[2,4,1,1]=(0.578749842767)*x[0]+(1.18687780739)*x[1]
        s[2,4,2,0]=(-0.349879331342)*x[0]+(-0.214411558047)*x[1]
        s[2,4,2,1]=(-0.875945724127)*x[0]+(-1.36065436643)*x[1]
        s[3,0,0,0]=(0.0488555315168)*x[0]+(-0.82382271801)*x[1]
        s[3,0,0,1]=(1.24017449448)*x[0]+(-0.911198047419)*x[1]
        s[3,0,1,0]=(0.30200654088)*x[0]+(-1.11559310554)*x[1]
        s[3,0,1,1]=(0.67275005347)*x[0]+(-0.645034711632)*x[1]
        s[3,0,2,0]=(-0.670229614054)*x[0]+(0.359423039794)*x[1]
        s[3,0,2,1]=(1.14988130646)*x[0]+(0.86614224885)*x[1]
        s[3,1,0,0]=(0.705716005963)*x[0]+(-1.15357209623)*x[1]
        s[3,1,0,1]=(0.31067508882)*x[0]+(0.960823286011)*x[1]
        s[3,1,1,0]=(0.866232494898)*x[0]+(-0.65162103259)*x[1]
        s[3,1,1,1]=(-0.452071783562)*x[0]+(-0.211956993156)*x[1]
        s[3,1,2,0]=(-1.32137973484)*x[0]+(1.49152904496)*x[1]
        s[3,1,2,1]=(0.854595989445)*x[0]+(1.11534387313)*x[1]
        s[3,2,0,0]=(0.789093645939)*x[0]+(-0.623866327905)*x[1]
        s[3,2,0,1]=(-0.649118397874)*x[0]+(-0.510923012103)*x[1]
        s[3,2,1,0]=(-0.263103480764)*x[0]+(0.0353171690366)*x[1]
        s[3,2,1,1]=(-0.910441885706)*x[0]+(-0.572413584482)*x[1]
        s[3,2,2,0]=(-0.764615254979)*x[0]+(-0.257894213166)*x[1]
        s[3,2,2,1]=(0.82110723418)*x[0]+(-0.881337133364)*x[1]
        s[3,3,0,0]=(-0.978909443882)*x[0]+(0.95340950915)*x[1]
        s[3,3,0,1]=(-0.121221461759)*x[0]+(-1.68773270918)*x[1]
        s[3,3,1,0]=(-0.150032516264)*x[0]+(0.441210519728)*x[1]
        s[3,3,1,1]=(-0.618694838164)*x[0]+(0.379748674791)*x[1]
        s[3,3,2,0]=(0.208366138002)*x[0]+(1.04701688993)*x[1]
        s[3,3,2,1]=(0.207672316371)*x[0]+(-1.04311602421)*x[1]
        s[3,4,0,0]=(0.404190185535)*x[0]+(0.206083873066)*x[1]
        s[3,4,0,1]=(0.928741940014)*x[0]+(0.126856418039)*x[1]
        s[3,4,1,0]=(0.0831637965324)*x[0]+(0.198288732425)*x[1]
        s[3,4,1,1]=(0.602722196536)*x[0]+(-0.256860860163)*x[1]
        s[3,4,2,0]=(1.11330980537)*x[0]+(0.130706767542)*x[1]
        s[3,4,2,1]=(0.638478712945)*x[0]+(0.558803659636)*x[1]
        ref[0,0,0,0]=(-0.110140466592)*x_ref[0]+(0.758992308989)*x_ref[1]
        ref[0,0,0,1]=(-1.27611018159)*x_ref[0]+(-1.21469838588)*x_ref[1]
        ref[0,0,1,0]=(-0.695719260931)*x_ref[0]+(-0.631650774678)*x_ref[1]
        ref[0,0,1,1]=(-0.0446786678677)*x_ref[0]+(-0.547249041592)*x_ref[1]
        ref[0,0,2,0]=(0.824673646909)*x_ref[0]+(-1.53499816213)*x_ref[1]
        ref[0,0,2,1]=(0.902244689111)*x_ref[0]+(0.194498326222)*x_ref[1]
        ref[0,1,0,0]=(-1.29097330044)*x_ref[0]+(0.0328620774092)*x_ref[1]
        ref[0,1,0,1]=(1.20419739576)*x_ref[0]+(-1.25415670694)*x_ref[1]
        ref[0,1,1,0]=(0.825311334936)*x_ref[0]+(0.732241589298)*x_ref[1]
        ref[0,1,1,1]=(0.239743697336)*x_ref[0]+(0.446701903969)*x_ref[1]
        ref[0,1,2,0]=(-0.208525758105)*x_ref[0]+(0.564727160849)*x_ref[1]
        ref[0,1,2,1]=(0.425352221432)*x_ref[0]+(-0.439885368017)*x_ref[1]
        ref[0,2,0,0]=(0.089403990032)*x_ref[0]+(0.927312736507)*x_ref[1]
        ref[0,2,0,1]=(-1.98342026057)*x_ref[0]+(0.908824462154)*x_ref[1]
        ref[0,2,1,0]=(-1.02677966987)*x_ref[0]+(1.64009929611)*x_ref[1]
        ref[0,2,1,1]=(1.18506229959)*x_ref[0]+(-0.336041064484)*x_ref[1]
        ref[0,2,2,0]=(0.124121400983)*x_ref[0]+(0.169858827797)*x_ref[1]
        ref[0,2,2,1]=(0.0171983721215)*x_ref[0]+(-0.679377487833)*x_ref[1]
        ref[0,3,0,0]=(0.707474586567)*x_ref[0]+(-0.327636474016)*x_ref[1]
        ref[0,3,0,1]=(-0.183596474381)*x_ref[0]+(-1.07263498957)*x_ref[1]
        ref[0,3,1,0]=(1.49812660538)*x_ref[0]+(-0.313209529867)*x_ref[1]
        ref[0,3,1,1]=(-0.678162796237)*x_ref[0]+(-0.823724533082)*x_ref[1]
        ref[0,3,2,0]=(-0.79806833841)*x_ref[0]+(0.865576104504)*x_ref[1]
        ref[0,3,2,1]=(0.941864201983)*x_ref[0]+(0.734846756813)*x_ref[1]
        ref[0,4,0,0]=(-0.205171531689)*x_ref[0]+(0.299610959481)*x_ref[1]
        ref[0,4,0,1]=(0.643430728461)*x_ref[0]+(0.584994687875)*x_ref[1]
        ref[0,4,1,0]=(-0.408681959359)*x_ref[0]+(0.601772358172)*x_ref[1]
        ref[0,4,1,1]=(-1.44472725121)*x_ref[0]+(-0.0899809192849)*x_ref[1]
        ref[0,4,2,0]=(0.722764720152)*x_ref[0]+(-0.407449476647)*x_ref[1]
        ref[0,4,2,1]=(0.166269375199)*x_ref[0]+(1.20016653459)*x_ref[1]
        ref[1,0,0,0]=(-0.764665577343)*x_ref[0]+(0.15492015266)*x_ref[1]
        ref[1,0,0,1]=(0.822023595189)*x_ref[0]+(0.00511531586673)*x_ref[1]
        ref[1,0,1,0]=(0.0840294388025)*x_ref[0]+(0.393606253186)*x_ref[1]
        ref[1,0,1,1]=(-0.702650062915)*x_ref[0]+(-1.10361836979)*x_ref[1]
        ref[1,0,2,0]=(-1.34569564836)*x_ref[0]+(-1.50816431101)*x_ref[1]
        ref[1,0,2,1]=(1.11337280315)*x_ref[0]+(-0.0771805114838)*x_ref[1]
        ref[1,1,0,0]=(-0.0695861126454)*x_ref[0]+(-0.437020174051)*x_ref[1]
        ref[1,1,0,1]=(1.68924343517)*x_ref[0]+(0.706853885933)*x_ref[1]
        ref[1,1,1,0]=(0.0806933170819)*x_ref[0]+(-0.974818406182)*x_ref[1]
        ref[1,1,1,1]=(-0.236836420312)*x_ref[0]+(-0.352629546416)*x_ref[1]
        ref[1,1,2,0]=(-0.312697720062)*x_ref[0]+(-0.977312197678)*x_ref[1]
        ref[1,1,2,1]=(-0.263563591998)*x_ref[0]+(-0.899465194216)*x_ref[1]
        ref[1,2,0,0]=(1.0599010032)*x_ref[0]+(0.625641179018)*x_ref[1]
        ref[1,2,0,1]=(0.42589369081)*x_ref[0]+(0.63136683361)*x_ref[1]
        ref[1,2,1,0]=(0.629519575902)*x_ref[0]+(-1.03125707286)*x_ref[1]
        ref[1,2,1,1]=(0.479272369893)*x_ref[0]+(-1.63724584249)*x_ref[1]
        ref[1,2,2,0]=(1.48877937845)*x_ref[0]+(-0.802308310767)*x_ref[1]
        ref[1,2,2,1]=(0.374636882204)*x_ref[0]+(0.187733613588)*x_ref[1]
        ref[1,3,0,0]=(0.360148707925)*x_ref[0]+(1.48869239048)*x_ref[1]
        ref[1,3,0,1]=(-1.29194970756)*x_ref[0]+(-0.433561540189)*x_ref[1]
        ref[1,3,1,0]=(0.696118719028)*x_ref[0]+(1.73199962229)*x_ref[1]
        ref[1,3,1,1]=(1.63831809651)*x_ref[0]+(0.70687058079)*x_ref[1]
        ref[1,3,2,0]=(1.03614006811)*x_ref[0]+(-0.783037005502)*x_ref[1]
        ref[1,3,2,1]=(0.351931393443)*x_ref[0]+(-0.720060584954)*x_ref[1]
        ref[1,4,0,0]=(0.856138152361)*x_ref[0]+(-0.0733689526147)*x_ref[1]
        ref[1,4,0,1]=(-0.256696810525)*x_ref[0]+(1.57547133796)*x_ref[1]
        ref[1,4,1,0]=(0.260933620175)*x_ref[0]+(-0.528418816229)*x_ref[1]
        ref[1,4,1,1]=(-1.14624698517)*x_ref[0]+(0.537831890429)*x_ref[1]
        ref[1,4,2,0]=(-1.11328030308)*x_ref[0]+(0.477599527943)*x_ref[1]
        ref[1,4,2,1]=(-1.52135700331)*x_ref[0]+(1.18803329651)*x_ref[1]
        ref[2,0,0,0]=(1.02103992957)*x_ref[0]+(1.3428520083)*x_ref[1]
        ref[2,0,0,1]=(0.74126089694)*x_ref[0]+(-0.747092126031)*x_ref[1]
        ref[2,0,1,0]=(-0.300534057945)*x_ref[0]+(-0.0679226717228)*x_ref[1]
        ref[2,0,1,1]=(0.205780243466)*x_ref[0]+(1.89346679377)*x_ref[1]
        ref[2,0,2,0]=(0.711997044871)*x_ref[0]+(0.614186591236)*x_ref[1]
        ref[2,0,2,1]=(0.562225764659)*x_ref[0]+(-0.75210737336)*x_ref[1]
        ref[2,1,0,0]=(1.51024763079)*x_ref[0]+(0.0655411043245)*x_ref[1]
        ref[2,1,0,1]=(-1.60270171758)*x_ref[0]+(-0.555919918868)*x_ref[1]
        ref[2,1,1,0]=(0.0117250633883)*x_ref[0]+(1.06089889984)*x_ref[1]
        ref[2,1,1,1]=(1.41839826051)*x_ref[0]+(-1.11883387918)*x_ref[1]
        ref[2,1,2,0]=(-1.47481010167)*x_ref[0]+(0.21986710996)*x_ref[1]
        ref[2,1,2,1]=(-0.890780004667)*x_ref[0]+(0.68206435926)*x_ref[1]
        ref[2,2,0,0]=(0.00647580761798)*x_ref[0]+(1.08988646609)*x_ref[1]
        ref[2,2,0,1]=(0.107576626365)*x_ref[0]+(0.569541010093)*x_ref[1]
        ref[2,2,1,0]=(0.655745883134)*x_ref[0]+(1.59781027174)*x_ref[1]
        ref[2,2,1,1]=(0.423699701576)*x_ref[0]+(-0.854187892248)*x_ref[1]
        ref[2,2,2,0]=(-0.904849730929)*x_ref[0]+(0.235074145696)*x_ref[1]
        ref[2,2,2,1]=(0.981525463443)*x_ref[0]+(0.701832385796)*x_ref[1]
        ref[2,3,0,0]=(-0.489533883573)*x_ref[0]+(-0.0213717056714)*x_ref[1]
        ref[2,3,0,1]=(0.516832539105)*x_ref[0]+(-1.11995329355)*x_ref[1]
        ref[2,3,1,0]=(0.287740517981)*x_ref[0]+(-1.3210942817)*x_ref[1]
        ref[2,3,1,1]=(0.179604184524)*x_ref[0]+(-1.63664715586)*x_ref[1]
        ref[2,3,2,0]=(-0.492296320142)*x_ref[0]+(-0.502235503805)*x_ref[1]
        ref[2,3,2,1]=(-0.653587673689)*x_ref[0]+(0.5466761379)*x_ref[1]
        ref[2,4,0,0]=(1.37393629094)*x_ref[0]+(0.645991033265)*x_ref[1]
        ref[2,4,0,1]=(-0.108057947979)*x_ref[0]+(-0.761267645781)*x_ref[1]
        ref[2,4,1,0]=(-0.0900524038235)*x_ref[0]+(0.16773443574)*x_ref[1]
        ref[2,4,1,1]=(0.578749842767)*x_ref[0]+(1.18687780739)*x_ref[1]
        ref[2,4,2,0]=(-0.349879331342)*x_ref[0]+(-0.214411558047)*x_ref[1]
        ref[2,4,2,1]=(-0.875945724127)*x_ref[0]+(-1.36065436643)*x_ref[1]
        ref[3,0,0,0]=(0.0488555315168)*x_ref[0]+(-0.82382271801)*x_ref[1]
        ref[3,0,0,1]=(1.24017449448)*x_ref[0]+(-0.911198047419)*x_ref[1]
        ref[3,0,1,0]=(0.30200654088)*x_ref[0]+(-1.11559310554)*x_ref[1]
        ref[3,0,1,1]=(0.67275005347)*x_ref[0]+(-0.645034711632)*x_ref[1]
        ref[3,0,2,0]=(-0.670229614054)*x_ref[0]+(0.359423039794)*x_ref[1]
        ref[3,0,2,1]=(1.14988130646)*x_ref[0]+(0.86614224885)*x_ref[1]
        ref[3,1,0,0]=(0.705716005963)*x_ref[0]+(-1.15357209623)*x_ref[1]
        ref[3,1,0,1]=(0.31067508882)*x_ref[0]+(0.960823286011)*x_ref[1]
        ref[3,1,1,0]=(0.866232494898)*x_ref[0]+(-0.65162103259)*x_ref[1]
        ref[3,1,1,1]=(-0.452071783562)*x_ref[0]+(-0.211956993156)*x_ref[1]
        ref[3,1,2,0]=(-1.32137973484)*x_ref[0]+(1.49152904496)*x_ref[1]
        ref[3,1,2,1]=(0.854595989445)*x_ref[0]+(1.11534387313)*x_ref[1]
        ref[3,2,0,0]=(0.789093645939)*x_ref[0]+(-0.623866327905)*x_ref[1]
        ref[3,2,0,1]=(-0.649118397874)*x_ref[0]+(-0.510923012103)*x_ref[1]
        ref[3,2,1,0]=(-0.263103480764)*x_ref[0]+(0.0353171690366)*x_ref[1]
        ref[3,2,1,1]=(-0.910441885706)*x_ref[0]+(-0.572413584482)*x_ref[1]
        ref[3,2,2,0]=(-0.764615254979)*x_ref[0]+(-0.257894213166)*x_ref[1]
        ref[3,2,2,1]=(0.82110723418)*x_ref[0]+(-0.881337133364)*x_ref[1]
        ref[3,3,0,0]=(-0.978909443882)*x_ref[0]+(0.95340950915)*x_ref[1]
        ref[3,3,0,1]=(-0.121221461759)*x_ref[0]+(-1.68773270918)*x_ref[1]
        ref[3,3,1,0]=(-0.150032516264)*x_ref[0]+(0.441210519728)*x_ref[1]
        ref[3,3,1,1]=(-0.618694838164)*x_ref[0]+(0.379748674791)*x_ref[1]
        ref[3,3,2,0]=(0.208366138002)*x_ref[0]+(1.04701688993)*x_ref[1]
        ref[3,3,2,1]=(0.207672316371)*x_ref[0]+(-1.04311602421)*x_ref[1]
        ref[3,4,0,0]=(0.404190185535)*x_ref[0]+(0.206083873066)*x_ref[1]
        ref[3,4,0,1]=(0.928741940014)*x_ref[0]+(0.126856418039)*x_ref[1]
        ref[3,4,1,0]=(0.0831637965324)*x_ref[0]+(0.198288732425)*x_ref[1]
        ref[3,4,1,1]=(0.602722196536)*x_ref[0]+(-0.256860860163)*x_ref[1]
        ref[3,4,2,0]=(1.11330980537)*x_ref[0]+(0.130706767542)*x_ref[1]
        ref[3,4,2,1]=(0.638478712945)*x_ref[0]+(0.558803659636)*x_ref[1]
      else:
        s[0,0,0,0]=(-0.678750626234)*x[0]+(0.707898134345)*x[1]+(1.47682355575)*x[2]
        s[0,0,0,1]=(-1.08665921875)*x[0]+(-1.03746577329)*x[1]+(0.459220112015)*x[2]
        s[0,0,1,0]=(-0.0924850687831)*x[0]+(-1.21392495875)*x[1]+(-0.335482829106)*x[2]
        s[0,0,1,1]=(-0.796044372195)*x[0]+(-1.61600945946)*x[1]+(-0.917678370431)*x[2]
        s[0,0,2,0]=(-1.17314148071)*x[0]+(0.594053121363)*x[1]+(-1.0751981685)*x[2]
        s[0,0,2,1]=(-0.993747266312)*x[0]+(0.647546031964)*x[1]+(0.789233440797)*x[2]
        s[0,1,0,0]=(-0.538620007403)*x[0]+(0.572432466547)*x[1]+(1.59224293356)*x[2]
        s[0,1,0,1]=(-0.315217413376)*x[0]+(1.30492005409)*x[1]+(-0.04944666559)*x[2]
        s[0,1,1,0]=(-0.734247343541)*x[0]+(1.14104633611)*x[1]+(-0.0424483055941)*x[2]
        s[0,1,1,1]=(-0.521727902651)*x[0]+(-0.0302866863619)*x[1]+(-0.0378416654735)*x[2]
        s[0,1,2,0]=(0.319532436452)*x[0]+(0.312388869947)*x[1]+(0.237025751349)*x[2]
        s[0,1,2,1]=(0.713884175591)*x[0]+(1.10911840343)*x[1]+(-1.4943567087)*x[2]
        s[0,2,0,0]=(-0.599254236697)*x[0]+(-0.190373139957)*x[1]+(-1.75168866737)*x[2]
        s[0,2,0,1]=(0.396782073165)*x[0]+(-0.252470841512)*x[1]+(-0.463651254287)*x[2]
        s[0,2,1,0]=(1.03549889185)*x[0]+(0.225894922761)*x[1]+(-0.609139538729)*x[2]
        s[0,2,1,1]=(0.289458873955)*x[0]+(-0.136668874286)*x[1]+(0.474827071819)*x[2]
        s[0,2,2,0]=(0.654253727057)*x[0]+(-0.998748120229)*x[1]+(-0.107718066457)*x[2]
        s[0,2,2,1]=(0.256710826049)*x[0]+(0.309515575678)*x[1]+(-1.60310813555)*x[2]
        s[0,3,0,0]=(1.70779242771)*x[0]+(-0.530818399689)*x[1]+(-0.516799319419)*x[2]
        s[0,3,0,1]=(-0.848868447677)*x[0]+(-0.301212439564)*x[1]+(-0.423135852525)*x[2]
        s[0,3,1,0]=(1.0807833809)*x[0]+(-1.20726021059)*x[1]+(0.565314004443)*x[2]
        s[0,3,1,1]=(-0.660234771003)*x[0]+(-0.443966617085)*x[1]+(-0.185381740077)*x[2]
        s[0,3,2,0]=(1.6260513125)*x[0]+(0.145626479867)*x[1]+(-0.808965403823)*x[2]
        s[0,3,2,1]=(-0.503772598493)*x[0]+(-1.63975947666)*x[1]+(0.255681416016)*x[2]
        s[0,4,0,0]=(0.0818777313252)*x[0]+(0.428155907605)*x[1]+(-0.642991998051)*x[2]
        s[0,4,0,1]=(-0.145032953266)*x[0]+(-0.7645425853)*x[1]+(0.458784780224)*x[2]
        s[0,4,1,0]=(-0.589583511871)*x[0]+(-0.0743175116482)*x[1]+(0.285113485271)*x[2]
        s[0,4,1,1]=(-1.37305892557)*x[0]+(0.917822397515)*x[1]+(-0.378127728174)*x[2]
        s[0,4,2,0]=(1.16505232525)*x[0]+(1.10944900772)*x[1]+(-0.95491955738)*x[2]
        s[0,4,2,1]=(-1.12539432931)*x[0]+(-0.290265022179)*x[1]+(-1.20809121853)*x[2]
        s[1,0,0,0]=(-0.859420732972)*x[0]+(0.564058561557)*x[1]+(-0.795155516002)*x[2]
        s[1,0,0,1]=(-0.314313522693)*x[0]+(1.46275003784)*x[1]+(0.285596389281)*x[2]
        s[1,0,1,0]=(-0.513360534744)*x[0]+(-0.0532410726117)*x[1]+(-0.122433325652)*x[2]
        s[1,0,1,1]=(0.527488940208)*x[0]+(0.0760609834598)*x[1]+(-0.612300815838)*x[2]
        s[1,0,2,0]=(0.302549971797)*x[0]+(-0.710901255568)*x[1]+(0.955062818656)*x[2]
        s[1,0,2,1]=(0.126520802001)*x[0]+(-1.07564234283)*x[1]+(-0.669526781403)*x[2]
        s[1,1,0,0]=(1.50804296428)*x[0]+(0.111903566342)*x[1]+(0.465876365517)*x[2]
        s[1,1,0,1]=(-1.04380726708)*x[0]+(0.148245186406)*x[1]+(-1.06801443458)*x[2]
        s[1,1,1,0]=(1.48236374583)*x[0]+(-0.00794102905726)*x[1]+(-1.35613430084)*x[2]
        s[1,1,1,1]=(1.04119256641)*x[0]+(-1.84075261029)*x[1]+(-0.776969168414)*x[2]
        s[1,1,2,0]=(-0.356539020393)*x[0]+(1.70797570799)*x[1]+(-0.525736126605)*x[2]
        s[1,1,2,1]=(1.62559049194)*x[0]+(-0.482482804647)*x[1]+(0.631790498783)*x[2]
        s[1,2,0,0]=(0.499315452031)*x[0]+(-0.296700233772)*x[1]+(1.00111627057)*x[2]
        s[1,2,0,1]=(1.0433119055)*x[0]+(-0.269957745366)*x[1]+(0.591153091169)*x[2]
        s[1,2,1,0]=(0.502068262492)*x[0]+(-0.849037462803)*x[1]+(-1.56692020458)*x[2]
        s[1,2,1,1]=(1.2984795003)*x[0]+(-1.07653573134)*x[1]+(0.487232329161)*x[2]
        s[1,2,2,0]=(-0.109333568443)*x[0]+(0.687162555279)*x[1]+(-0.812636431387)*x[2]
        s[1,2,2,1]=(0.202326014817)*x[0]+(1.23297194202)*x[1]+(1.41163217274)*x[2]
        s[1,3,0,0]=(0.124740742544)*x[0]+(-0.500878894409)*x[1]+(-0.376438702019)*x[2]
        s[1,3,0,1]=(-0.540234624582)*x[0]+(-0.400577181452)*x[1]+(0.879215681428)*x[2]
        s[1,3,1,0]=(0.0426835015338)*x[0]+(-0.726976626459)*x[1]+(1.32655296741)*x[2]
        s[1,3,1,1]=(-0.120969369352)*x[0]+(-0.426853376816)*x[1]+(1.88787704976)*x[2]
        s[1,3,2,0]=(-0.0895115836038)*x[0]+(-1.17409640569)*x[1]+(-0.874853748928)*x[2]
        s[1,3,2,1]=(-0.0664677570269)*x[0]+(0.708204452159)*x[1]+(-0.26397721498)*x[2]
        s[1,4,0,0]=(0.0236750184919)*x[0]+(-1.42225228911)*x[1]+(0.399502903376)*x[2]
        s[1,4,0,1]=(1.03400366274)*x[0]+(-0.442018365874)*x[1]+(1.02491120541)*x[2]
        s[1,4,1,0]=(-0.150528717616)*x[0]+(-0.512236709632)*x[1]+(-0.0644728802768)*x[2]
        s[1,4,1,1]=(-0.998520493595)*x[0]+(-0.507010096027)*x[1]+(-0.889168150742)*x[2]
        s[1,4,2,0]=(0.179165278401)*x[0]+(1.07136931045)*x[1]+(-0.790101000847)*x[2]
        s[1,4,2,1]=(0.376877479598)*x[0]+(-0.25979387806)*x[1]+(0.446682478544)*x[2]
        s[2,0,0,0]=(0.365668617405)*x[0]+(1.44268125148)*x[1]+(1.65785163405)*x[2]
        s[2,0,0,1]=(0.576180639993)*x[0]+(-0.0571374651916)*x[1]+(-0.0854307657892)*x[2]
        s[2,0,1,0]=(0.0474860699991)*x[0]+(-1.47716457199)*x[1]+(0.87300241949)*x[2]
        s[2,0,1,1]=(-0.293608884991)*x[0]+(0.0963236497693)*x[1]+(-1.31217901681)*x[2]
        s[2,0,2,0]=(-0.725125904522)*x[0]+(0.15259525725)*x[1]+(0.0624842579957)*x[2]
        s[2,0,2,1]=(-0.374667932076)*x[0]+(-0.586704716252)*x[1]+(-0.208259155061)*x[2]
        s[2,1,0,0]=(0.480010407761)*x[0]+(-1.78856630141)*x[1]+(0.296814589368)*x[2]
        s[2,1,0,1]=(-0.131871389585)*x[0]+(0.0984653022698)*x[1]+(0.222785695838)*x[2]
        s[2,1,1,0]=(-0.4095594348)*x[0]+(-0.746059065385)*x[1]+(1.72282248984)*x[2]
        s[2,1,1,1]=(0.841882742791)*x[0]+(0.698803891138)*x[1]+(0.201279240318)*x[2]
        s[2,1,2,0]=(-1.52503851146)*x[0]+(0.746778574384)*x[1]+(0.820127798751)*x[2]
        s[2,1,2,1]=(0.578616447592)*x[0]+(-0.232884112626)*x[1]+(0.719949134498)*x[2]
        s[2,2,0,0]=(-0.250869865455)*x[0]+(0.0457838553118)*x[1]+(0.694605224088)*x[2]
        s[2,2,0,1]=(-0.344703134894)*x[0]+(-0.945296587028)*x[1]+(-0.408991625518)*x[2]
        s[2,2,1,0]=(1.15842118069)*x[0]+(0.385018100988)*x[1]+(-0.298867420422)*x[2]
        s[2,2,1,1]=(-0.992212925836)*x[0]+(-0.303921757453)*x[1]+(1.90996417539)*x[2]
        s[2,2,2,0]=(0.139346086335)*x[0]+(-1.5255000175)*x[1]+(-0.418223280061)*x[2]
        s[2,2,2,1]=(-0.287165354844)*x[0]+(-0.563470113789)*x[1]+(-0.267078242542)*x[2]
        s[2,3,0,0]=(-0.877956931038)*x[0]+(-0.587379404687)*x[1]+(0.223772989101)*x[2]
        s[2,3,0,1]=(0.450240028907)*x[0]+(0.933649504645)*x[1]+(0.922982912622)*x[2]
        s[2,3,1,0]=(-0.695647633954)*x[0]+(0.675123237641)*x[1]+(0.333013288603)*x[2]
        s[2,3,1,1]=(-0.638319791165)*x[0]+(-1.01905860587)*x[1]+(-0.0829897514576)*x[2]
        s[2,3,2,0]=(-0.85507363839)*x[0]+(0.213154292163)*x[1]+(-1.19634907609)*x[2]
        s[2,3,2,1]=(0.51969848742)*x[0]+(-0.103353752424)*x[1]+(1.38100947757)*x[2]
        s[2,4,0,0]=(-1.42264362114)*x[0]+(-0.0622861012095)*x[1]+(1.11415860946)*x[2]
        s[2,4,0,1]=(1.03146522202)*x[0]+(-0.59775537826)*x[1]+(0.455451132416)*x[2]
        s[2,4,1,0]=(-1.42367400988)*x[0]+(0.468019196451)*x[1]+(-0.0449222277683)*x[2]
        s[2,4,1,1]=(-0.337745267456)*x[0]+(-1.24573098041)*x[1]+(0.591411642381)*x[2]
        s[2,4,2,0]=(-0.773220236551)*x[0]+(-0.92934639777)*x[1]+(-0.323666905212)*x[2]
        s[2,4,2,1]=(-1.25416565442)*x[0]+(0.10804460369)*x[1]+(1.3398483009)*x[2]
        s[3,0,0,0]=(-0.187928573247)*x[0]+(0.952793598709)*x[1]+(-0.651544295118)*x[2]
        s[3,0,0,1]=(-1.18183645576)*x[0]+(-0.767480258477)*x[1]+(0.0733909569555)*x[2]
        s[3,0,1,0]=(0.825156828887)*x[0]+(0.900307840844)*x[1]+(-1.03153118659)*x[2]
        s[3,0,1,1]=(-0.150209523447)*x[0]+(1.07490578747)*x[1]+(-1.6916184583)*x[2]
        s[3,0,2,0]=(-1.07041023688)*x[0]+(-0.420347448824)*x[1]+(-0.556885268114)*x[2]
        s[3,0,2,1]=(-1.30609527635)*x[0]+(-0.10753520843)*x[1]+(0.240258152808)*x[2]
        s[3,1,0,0]=(0.899651818251)*x[0]+(0.00909698197696)*x[1]+(-1.64985946319)*x[2]
        s[3,1,0,1]=(-0.880471412731)*x[0]+(0.678893404261)*x[1]+(0.288497390335)*x[2]
        s[3,1,1,0]=(-0.00910924813519)*x[0]+(-1.30365942504)*x[1]+(-0.568919628746)*x[2]
        s[3,1,1,1]=(-0.811256892653)*x[0]+(-0.0284400402628)*x[1]+(0.727386097819)*x[2]
        s[3,1,2,0]=(0.105755186759)*x[0]+(0.196242692495)*x[1]+(0.358516596542)*x[2]
        s[3,1,2,1]=(-0.37985806916)*x[0]+(0.44810760494)*x[1]+(0.633807267038)*x[2]
        s[3,2,0,0]=(1.34921360449)*x[0]+(-0.9349137549)*x[1]+(-0.272245661092)*x[2]
        s[3,2,0,1]=(1.21020688092)*x[0]+(1.1296074258)*x[1]+(0.000391430715188)*x[2]
        s[3,2,1,0]=(-1.25769018387)*x[0]+(-0.786491629941)*x[1]+(-0.862105545664)*x[2]
        s[3,2,1,1]=(0.668529227552)*x[0]+(-0.983024344824)*x[1]+(0.74846424728)*x[2]
        s[3,2,2,0]=(0.45487084108)*x[0]+(0.855491649411)*x[1]+(-0.104621648479)*x[2]
        s[3,2,2,1]=(0.802451481866)*x[0]+(-0.137058520895)*x[1]+(0.573940225359)*x[2]
        s[3,3,0,0]=(0.468999526003)*x[0]+(0.504917415809)*x[1]+(-1.39187140958)*x[2]
        s[3,3,0,1]=(0.501007631838)*x[0]+(-0.285379909295)*x[1]+(0.257752560803)*x[2]
        s[3,3,1,0]=(-1.01423550219)*x[0]+(-0.749951834747)*x[1]+(-0.156550300135)*x[2]
        s[3,3,1,1]=(0.190954159364)*x[0]+(0.985690213898)*x[1]+(-0.942349242057)*x[2]
        s[3,3,2,0]=(-1.04776017317)*x[0]+(1.3978257599)*x[1]+(-0.727385312507)*x[2]
        s[3,3,2,1]=(0.118574662458)*x[0]+(0.763657521414)*x[1]+(-0.947854404873)*x[2]
        s[3,4,0,0]=(-1.42375158871)*x[0]+(1.02577770584)*x[1]+(1.7262667856)*x[2]
        s[3,4,0,1]=(0.819198798755)*x[0]+(-0.177472792322)*x[1]+(-0.495119104783)*x[2]
        s[3,4,1,0]=(-1.44070928104)*x[0]+(-0.67376984043)*x[1]+(0.314203511381)*x[2]
        s[3,4,1,1]=(-0.0417551422406)*x[0]+(-0.446166510311)*x[1]+(-0.303922120635)*x[2]
        s[3,4,2,0]=(0.161905825888)*x[0]+(-0.433001417416)*x[1]+(0.713602984094)*x[2]
        s[3,4,2,1]=(1.29579174225)*x[0]+(-0.136051917273)*x[1]+(0.270836816064)*x[2]
        ref[0,0,0,0]=(-0.678750626234)*x_ref[0]+(0.707898134345)*x_ref[1]+(1.47682355575)*x_ref[2]
        ref[0,0,0,1]=(-1.08665921875)*x_ref[0]+(-1.03746577329)*x_ref[1]+(0.459220112015)*x_ref[2]
        ref[0,0,1,0]=(-0.0924850687831)*x_ref[0]+(-1.21392495875)*x_ref[1]+(-0.335482829106)*x_ref[2]
        ref[0,0,1,1]=(-0.796044372195)*x_ref[0]+(-1.61600945946)*x_ref[1]+(-0.917678370431)*x_ref[2]
        ref[0,0,2,0]=(-1.17314148071)*x_ref[0]+(0.594053121363)*x_ref[1]+(-1.0751981685)*x_ref[2]
        ref[0,0,2,1]=(-0.993747266312)*x_ref[0]+(0.647546031964)*x_ref[1]+(0.789233440797)*x_ref[2]
        ref[0,1,0,0]=(-0.538620007403)*x_ref[0]+(0.572432466547)*x_ref[1]+(1.59224293356)*x_ref[2]
        ref[0,1,0,1]=(-0.315217413376)*x_ref[0]+(1.30492005409)*x_ref[1]+(-0.04944666559)*x_ref[2]
        ref[0,1,1,0]=(-0.734247343541)*x_ref[0]+(1.14104633611)*x_ref[1]+(-0.0424483055941)*x_ref[2]
        ref[0,1,1,1]=(-0.521727902651)*x_ref[0]+(-0.0302866863619)*x_ref[1]+(-0.0378416654735)*x_ref[2]
        ref[0,1,2,0]=(0.319532436452)*x_ref[0]+(0.312388869947)*x_ref[1]+(0.237025751349)*x_ref[2]
        ref[0,1,2,1]=(0.713884175591)*x_ref[0]+(1.10911840343)*x_ref[1]+(-1.4943567087)*x_ref[2]
        ref[0,2,0,0]=(-0.599254236697)*x_ref[0]+(-0.190373139957)*x_ref[1]+(-1.75168866737)*x_ref[2]
        ref[0,2,0,1]=(0.396782073165)*x_ref[0]+(-0.252470841512)*x_ref[1]+(-0.463651254287)*x_ref[2]
        ref[0,2,1,0]=(1.03549889185)*x_ref[0]+(0.225894922761)*x_ref[1]+(-0.609139538729)*x_ref[2]
        ref[0,2,1,1]=(0.289458873955)*x_ref[0]+(-0.136668874286)*x_ref[1]+(0.474827071819)*x_ref[2]
        ref[0,2,2,0]=(0.654253727057)*x_ref[0]+(-0.998748120229)*x_ref[1]+(-0.107718066457)*x_ref[2]
        ref[0,2,2,1]=(0.256710826049)*x_ref[0]+(0.309515575678)*x_ref[1]+(-1.60310813555)*x_ref[2]
        ref[0,3,0,0]=(1.70779242771)*x_ref[0]+(-0.530818399689)*x_ref[1]+(-0.516799319419)*x_ref[2]
        ref[0,3,0,1]=(-0.848868447677)*x_ref[0]+(-0.301212439564)*x_ref[1]+(-0.423135852525)*x_ref[2]
        ref[0,3,1,0]=(1.0807833809)*x_ref[0]+(-1.20726021059)*x_ref[1]+(0.565314004443)*x_ref[2]
        ref[0,3,1,1]=(-0.660234771003)*x_ref[0]+(-0.443966617085)*x_ref[1]+(-0.185381740077)*x_ref[2]
        ref[0,3,2,0]=(1.6260513125)*x_ref[0]+(0.145626479867)*x_ref[1]+(-0.808965403823)*x_ref[2]
        ref[0,3,2,1]=(-0.503772598493)*x_ref[0]+(-1.63975947666)*x_ref[1]+(0.255681416016)*x_ref[2]
        ref[0,4,0,0]=(0.0818777313252)*x_ref[0]+(0.428155907605)*x_ref[1]+(-0.642991998051)*x_ref[2]
        ref[0,4,0,1]=(-0.145032953266)*x_ref[0]+(-0.7645425853)*x_ref[1]+(0.458784780224)*x_ref[2]
        ref[0,4,1,0]=(-0.589583511871)*x_ref[0]+(-0.0743175116482)*x_ref[1]+(0.285113485271)*x_ref[2]
        ref[0,4,1,1]=(-1.37305892557)*x_ref[0]+(0.917822397515)*x_ref[1]+(-0.378127728174)*x_ref[2]
        ref[0,4,2,0]=(1.16505232525)*x_ref[0]+(1.10944900772)*x_ref[1]+(-0.95491955738)*x_ref[2]
        ref[0,4,2,1]=(-1.12539432931)*x_ref[0]+(-0.290265022179)*x_ref[1]+(-1.20809121853)*x_ref[2]
        ref[1,0,0,0]=(-0.859420732972)*x_ref[0]+(0.564058561557)*x_ref[1]+(-0.795155516002)*x_ref[2]
        ref[1,0,0,1]=(-0.314313522693)*x_ref[0]+(1.46275003784)*x_ref[1]+(0.285596389281)*x_ref[2]
        ref[1,0,1,0]=(-0.513360534744)*x_ref[0]+(-0.0532410726117)*x_ref[1]+(-0.122433325652)*x_ref[2]
        ref[1,0,1,1]=(0.527488940208)*x_ref[0]+(0.0760609834598)*x_ref[1]+(-0.612300815838)*x_ref[2]
        ref[1,0,2,0]=(0.302549971797)*x_ref[0]+(-0.710901255568)*x_ref[1]+(0.955062818656)*x_ref[2]
        ref[1,0,2,1]=(0.126520802001)*x_ref[0]+(-1.07564234283)*x_ref[1]+(-0.669526781403)*x_ref[2]
        ref[1,1,0,0]=(1.50804296428)*x_ref[0]+(0.111903566342)*x_ref[1]+(0.465876365517)*x_ref[2]
        ref[1,1,0,1]=(-1.04380726708)*x_ref[0]+(0.148245186406)*x_ref[1]+(-1.06801443458)*x_ref[2]
        ref[1,1,1,0]=(1.48236374583)*x_ref[0]+(-0.00794102905726)*x_ref[1]+(-1.35613430084)*x_ref[2]
        ref[1,1,1,1]=(1.04119256641)*x_ref[0]+(-1.84075261029)*x_ref[1]+(-0.776969168414)*x_ref[2]
        ref[1,1,2,0]=(-0.356539020393)*x_ref[0]+(1.70797570799)*x_ref[1]+(-0.525736126605)*x_ref[2]
        ref[1,1,2,1]=(1.62559049194)*x_ref[0]+(-0.482482804647)*x_ref[1]+(0.631790498783)*x_ref[2]
        ref[1,2,0,0]=(0.499315452031)*x_ref[0]+(-0.296700233772)*x_ref[1]+(1.00111627057)*x_ref[2]
        ref[1,2,0,1]=(1.0433119055)*x_ref[0]+(-0.269957745366)*x_ref[1]+(0.591153091169)*x_ref[2]
        ref[1,2,1,0]=(0.502068262492)*x_ref[0]+(-0.849037462803)*x_ref[1]+(-1.56692020458)*x_ref[2]
        ref[1,2,1,1]=(1.2984795003)*x_ref[0]+(-1.07653573134)*x_ref[1]+(0.487232329161)*x_ref[2]
        ref[1,2,2,0]=(-0.109333568443)*x_ref[0]+(0.687162555279)*x_ref[1]+(-0.812636431387)*x_ref[2]
        ref[1,2,2,1]=(0.202326014817)*x_ref[0]+(1.23297194202)*x_ref[1]+(1.41163217274)*x_ref[2]
        ref[1,3,0,0]=(0.124740742544)*x_ref[0]+(-0.500878894409)*x_ref[1]+(-0.376438702019)*x_ref[2]
        ref[1,3,0,1]=(-0.540234624582)*x_ref[0]+(-0.400577181452)*x_ref[1]+(0.879215681428)*x_ref[2]
        ref[1,3,1,0]=(0.0426835015338)*x_ref[0]+(-0.726976626459)*x_ref[1]+(1.32655296741)*x_ref[2]
        ref[1,3,1,1]=(-0.120969369352)*x_ref[0]+(-0.426853376816)*x_ref[1]+(1.88787704976)*x_ref[2]
        ref[1,3,2,0]=(-0.0895115836038)*x_ref[0]+(-1.17409640569)*x_ref[1]+(-0.874853748928)*x_ref[2]
        ref[1,3,2,1]=(-0.0664677570269)*x_ref[0]+(0.708204452159)*x_ref[1]+(-0.26397721498)*x_ref[2]
        ref[1,4,0,0]=(0.0236750184919)*x_ref[0]+(-1.42225228911)*x_ref[1]+(0.399502903376)*x_ref[2]
        ref[1,4,0,1]=(1.03400366274)*x_ref[0]+(-0.442018365874)*x_ref[1]+(1.02491120541)*x_ref[2]
        ref[1,4,1,0]=(-0.150528717616)*x_ref[0]+(-0.512236709632)*x_ref[1]+(-0.0644728802768)*x_ref[2]
        ref[1,4,1,1]=(-0.998520493595)*x_ref[0]+(-0.507010096027)*x_ref[1]+(-0.889168150742)*x_ref[2]
        ref[1,4,2,0]=(0.179165278401)*x_ref[0]+(1.07136931045)*x_ref[1]+(-0.790101000847)*x_ref[2]
        ref[1,4,2,1]=(0.376877479598)*x_ref[0]+(-0.25979387806)*x_ref[1]+(0.446682478544)*x_ref[2]
        ref[2,0,0,0]=(0.365668617405)*x_ref[0]+(1.44268125148)*x_ref[1]+(1.65785163405)*x_ref[2]
        ref[2,0,0,1]=(0.576180639993)*x_ref[0]+(-0.0571374651916)*x_ref[1]+(-0.0854307657892)*x_ref[2]
        ref[2,0,1,0]=(0.0474860699991)*x_ref[0]+(-1.47716457199)*x_ref[1]+(0.87300241949)*x_ref[2]
        ref[2,0,1,1]=(-0.293608884991)*x_ref[0]+(0.0963236497693)*x_ref[1]+(-1.31217901681)*x_ref[2]
        ref[2,0,2,0]=(-0.725125904522)*x_ref[0]+(0.15259525725)*x_ref[1]+(0.0624842579957)*x_ref[2]
        ref[2,0,2,1]=(-0.374667932076)*x_ref[0]+(-0.586704716252)*x_ref[1]+(-0.208259155061)*x_ref[2]
        ref[2,1,0,0]=(0.480010407761)*x_ref[0]+(-1.78856630141)*x_ref[1]+(0.296814589368)*x_ref[2]
        ref[2,1,0,1]=(-0.131871389585)*x_ref[0]+(0.0984653022698)*x_ref[1]+(0.222785695838)*x_ref[2]
        ref[2,1,1,0]=(-0.4095594348)*x_ref[0]+(-0.746059065385)*x_ref[1]+(1.72282248984)*x_ref[2]
        ref[2,1,1,1]=(0.841882742791)*x_ref[0]+(0.698803891138)*x_ref[1]+(0.201279240318)*x_ref[2]
        ref[2,1,2,0]=(-1.52503851146)*x_ref[0]+(0.746778574384)*x_ref[1]+(0.820127798751)*x_ref[2]
        ref[2,1,2,1]=(0.578616447592)*x_ref[0]+(-0.232884112626)*x_ref[1]+(0.719949134498)*x_ref[2]
        ref[2,2,0,0]=(-0.250869865455)*x_ref[0]+(0.0457838553118)*x_ref[1]+(0.694605224088)*x_ref[2]
        ref[2,2,0,1]=(-0.344703134894)*x_ref[0]+(-0.945296587028)*x_ref[1]+(-0.408991625518)*x_ref[2]
        ref[2,2,1,0]=(1.15842118069)*x_ref[0]+(0.385018100988)*x_ref[1]+(-0.298867420422)*x_ref[2]
        ref[2,2,1,1]=(-0.992212925836)*x_ref[0]+(-0.303921757453)*x_ref[1]+(1.90996417539)*x_ref[2]
        ref[2,2,2,0]=(0.139346086335)*x_ref[0]+(-1.5255000175)*x_ref[1]+(-0.418223280061)*x_ref[2]
        ref[2,2,2,1]=(-0.287165354844)*x_ref[0]+(-0.563470113789)*x_ref[1]+(-0.267078242542)*x_ref[2]
        ref[2,3,0,0]=(-0.877956931038)*x_ref[0]+(-0.587379404687)*x_ref[1]+(0.223772989101)*x_ref[2]
        ref[2,3,0,1]=(0.450240028907)*x_ref[0]+(0.933649504645)*x_ref[1]+(0.922982912622)*x_ref[2]
        ref[2,3,1,0]=(-0.695647633954)*x_ref[0]+(0.675123237641)*x_ref[1]+(0.333013288603)*x_ref[2]
        ref[2,3,1,1]=(-0.638319791165)*x_ref[0]+(-1.01905860587)*x_ref[1]+(-0.0829897514576)*x_ref[2]
        ref[2,3,2,0]=(-0.85507363839)*x_ref[0]+(0.213154292163)*x_ref[1]+(-1.19634907609)*x_ref[2]
        ref[2,3,2,1]=(0.51969848742)*x_ref[0]+(-0.103353752424)*x_ref[1]+(1.38100947757)*x_ref[2]
        ref[2,4,0,0]=(-1.42264362114)*x_ref[0]+(-0.0622861012095)*x_ref[1]+(1.11415860946)*x_ref[2]
        ref[2,4,0,1]=(1.03146522202)*x_ref[0]+(-0.59775537826)*x_ref[1]+(0.455451132416)*x_ref[2]
        ref[2,4,1,0]=(-1.42367400988)*x_ref[0]+(0.468019196451)*x_ref[1]+(-0.0449222277683)*x_ref[2]
        ref[2,4,1,1]=(-0.337745267456)*x_ref[0]+(-1.24573098041)*x_ref[1]+(0.591411642381)*x_ref[2]
        ref[2,4,2,0]=(-0.773220236551)*x_ref[0]+(-0.92934639777)*x_ref[1]+(-0.323666905212)*x_ref[2]
        ref[2,4,2,1]=(-1.25416565442)*x_ref[0]+(0.10804460369)*x_ref[1]+(1.3398483009)*x_ref[2]
        ref[3,0,0,0]=(-0.187928573247)*x_ref[0]+(0.952793598709)*x_ref[1]+(-0.651544295118)*x_ref[2]
        ref[3,0,0,1]=(-1.18183645576)*x_ref[0]+(-0.767480258477)*x_ref[1]+(0.0733909569555)*x_ref[2]
        ref[3,0,1,0]=(0.825156828887)*x_ref[0]+(0.900307840844)*x_ref[1]+(-1.03153118659)*x_ref[2]
        ref[3,0,1,1]=(-0.150209523447)*x_ref[0]+(1.07490578747)*x_ref[1]+(-1.6916184583)*x_ref[2]
        ref[3,0,2,0]=(-1.07041023688)*x_ref[0]+(-0.420347448824)*x_ref[1]+(-0.556885268114)*x_ref[2]
        ref[3,0,2,1]=(-1.30609527635)*x_ref[0]+(-0.10753520843)*x_ref[1]+(0.240258152808)*x_ref[2]
        ref[3,1,0,0]=(0.899651818251)*x_ref[0]+(0.00909698197696)*x_ref[1]+(-1.64985946319)*x_ref[2]
        ref[3,1,0,1]=(-0.880471412731)*x_ref[0]+(0.678893404261)*x_ref[1]+(0.288497390335)*x_ref[2]
        ref[3,1,1,0]=(-0.00910924813519)*x_ref[0]+(-1.30365942504)*x_ref[1]+(-0.568919628746)*x_ref[2]
        ref[3,1,1,1]=(-0.811256892653)*x_ref[0]+(-0.0284400402628)*x_ref[1]+(0.727386097819)*x_ref[2]
        ref[3,1,2,0]=(0.105755186759)*x_ref[0]+(0.196242692495)*x_ref[1]+(0.358516596542)*x_ref[2]
        ref[3,1,2,1]=(-0.37985806916)*x_ref[0]+(0.44810760494)*x_ref[1]+(0.633807267038)*x_ref[2]
        ref[3,2,0,0]=(1.34921360449)*x_ref[0]+(-0.9349137549)*x_ref[1]+(-0.272245661092)*x_ref[2]
        ref[3,2,0,1]=(1.21020688092)*x_ref[0]+(1.1296074258)*x_ref[1]+(0.000391430715188)*x_ref[2]
        ref[3,2,1,0]=(-1.25769018387)*x_ref[0]+(-0.786491629941)*x_ref[1]+(-0.862105545664)*x_ref[2]
        ref[3,2,1,1]=(0.668529227552)*x_ref[0]+(-0.983024344824)*x_ref[1]+(0.74846424728)*x_ref[2]
        ref[3,2,2,0]=(0.45487084108)*x_ref[0]+(0.855491649411)*x_ref[1]+(-0.104621648479)*x_ref[2]
        ref[3,2,2,1]=(0.802451481866)*x_ref[0]+(-0.137058520895)*x_ref[1]+(0.573940225359)*x_ref[2]
        ref[3,3,0,0]=(0.468999526003)*x_ref[0]+(0.504917415809)*x_ref[1]+(-1.39187140958)*x_ref[2]
        ref[3,3,0,1]=(0.501007631838)*x_ref[0]+(-0.285379909295)*x_ref[1]+(0.257752560803)*x_ref[2]
        ref[3,3,1,0]=(-1.01423550219)*x_ref[0]+(-0.749951834747)*x_ref[1]+(-0.156550300135)*x_ref[2]
        ref[3,3,1,1]=(0.190954159364)*x_ref[0]+(0.985690213898)*x_ref[1]+(-0.942349242057)*x_ref[2]
        ref[3,3,2,0]=(-1.04776017317)*x_ref[0]+(1.3978257599)*x_ref[1]+(-0.727385312507)*x_ref[2]
        ref[3,3,2,1]=(0.118574662458)*x_ref[0]+(0.763657521414)*x_ref[1]+(-0.947854404873)*x_ref[2]
        ref[3,4,0,0]=(-1.42375158871)*x_ref[0]+(1.02577770584)*x_ref[1]+(1.7262667856)*x_ref[2]
        ref[3,4,0,1]=(0.819198798755)*x_ref[0]+(-0.177472792322)*x_ref[1]+(-0.495119104783)*x_ref[2]
        ref[3,4,1,0]=(-1.44070928104)*x_ref[0]+(-0.67376984043)*x_ref[1]+(0.314203511381)*x_ref[2]
        ref[3,4,1,1]=(-0.0417551422406)*x_ref[0]+(-0.446166510311)*x_ref[1]+(-0.303922120635)*x_ref[2]
        ref[3,4,2,0]=(0.161905825888)*x_ref[0]+(-0.433001417416)*x_ref[1]+(0.713602984094)*x_ref[2]
        ref[3,4,2,1]=(1.29579174225)*x_ref[0]+(-0.136051917273)*x_ref[1]+(0.270836816064)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromSymbol_ReducedFunctionOnContactZero_rank0(self):
      """
      tests interpolate for rank 0 Symbol onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactZero onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactZero(self.domain)
      x=w.getX()
      arg=Symbol(shape=(),dim=dim)
      s=Data(0,(),w)
      ref=Data(0,(),w_ref)
      if dim==2:
        s=(0.52559525257)*x[0]**o+(0.229391539263)*x[0]+(0.948594885998)*x[1]**o+(0.247647100433)*x[1]
        ref=(0.52559525257)*x_ref[0]**o+(0.229391539263)*x_ref[0]+(0.948594885998)*x_ref[1]**o+(0.247647100433)*x_ref[1]
      else:
        s=(-0.386040522546)*x[0]**o+(0.390865081371)*x[0]+(-0.194909055558)*x[1]**o+(0.890024298101)*x[1]+(-0.660390723258)*x[2]**o+(-0.191783645278)*x[2]
        ref=(-0.386040522546)*x_ref[0]**o+(0.390865081371)*x_ref[0]+(-0.194909055558)*x_ref[1]**o+(0.890024298101)*x_ref[1]+(-0.660390723258)*x_ref[2]**o+(-0.191783645278)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromSymbol_ReducedFunctionOnContactZero_rank1(self):
      """
      tests interpolate for rank 1 Symbol onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactZero onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactZero(self.domain)
      x=w.getX()
      arg=Symbol(shape=(2,),dim=dim)
      s=Data(0,(2,),w)
      ref=Data(0,(2,),w_ref)
      if dim==2:
        s[0]=(0.886118109323)*x[0]**o+(-0.430472191748)*x[0]+(0.188986990863)*x[1]**o+(0.123299635287)*x[1]
        s[1]=(-0.406991760476)*x[0]**o+(0.940756190869)*x[0]+(-0.926329857206)*x[1]**o+(-0.752111183385)*x[1]
        ref[0]=(0.886118109323)*x_ref[0]**o+(-0.430472191748)*x_ref[0]+(0.188986990863)*x_ref[1]**o+(0.123299635287)*x_ref[1]
        ref[1]=(-0.406991760476)*x_ref[0]**o+(0.940756190869)*x_ref[0]+(-0.926329857206)*x_ref[1]**o+(-0.752111183385)*x_ref[1]
      else:
        s[0]=(-0.888491776167)*x[0]**o+(0.568362216099)*x[0]+(0.92434143189)*x[1]**o+(0.686002203387)*x[1]+(0.323992600729)*x[2]**o+(-0.150051242182)*x[2]
        s[1]=(0.286049958958)*x[0]**o+(-0.947739219713)*x[0]+(-0.216646074031)*x[1]**o+(0.67646631851)*x[1]+(0.0591555372338)*x[2]**o+(0.949863355854)*x[2]
        ref[0]=(-0.888491776167)*x_ref[0]**o+(0.568362216099)*x_ref[0]+(0.92434143189)*x_ref[1]**o+(0.686002203387)*x_ref[1]+(0.323992600729)*x_ref[2]**o+(-0.150051242182)*x_ref[2]
        ref[1]=(0.286049958958)*x_ref[0]**o+(-0.947739219713)*x_ref[0]+(-0.216646074031)*x_ref[1]**o+(0.67646631851)*x_ref[1]+(0.0591555372338)*x_ref[2]**o+(0.949863355854)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromSymbol_ReducedFunctionOnContactZero_rank2(self):
      """
      tests interpolate for rank 2 Symbol onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactZero onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactZero(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5),dim=dim)
      s=Data(0,(4, 5),w)
      ref=Data(0,(4, 5),w_ref)
      if dim==2:
        s[0,0]=(0.777675758098)*x[0]**o+(-0.473156188055)*x[0]+(-0.83738893374)*x[1]**o+(-0.772375433518)*x[1]
        s[0,1]=(0.690632604654)*x[0]**o+(-0.596947428521)*x[0]+(-0.927206976637)*x[1]**o+(0.416133449904)*x[1]
        s[0,2]=(0.159353873068)*x[0]**o+(-0.544617444157)*x[0]+(-0.384185707939)*x[1]**o+(0.630137005341)*x[1]
        s[0,3]=(-0.677069489949)*x[0]**o+(0.121886164808)*x[0]+(-0.243464410455)*x[1]**o+(-0.424481834693)*x[1]
        s[0,4]=(-0.0586502823616)*x[0]**o+(-0.952781957131)*x[0]+(-0.638027009478)*x[1]**o+(-0.939836467104)*x[1]
        s[1,0]=(0.946604826689)*x[0]**o+(-0.344506753444)*x[0]+(-0.366256116023)*x[1]**o+(0.66571445335)*x[1]
        s[1,1]=(-0.148616405902)*x[0]**o+(0.24049619296)*x[0]+(-0.59987968565)*x[1]**o+(-0.30617564854)*x[1]
        s[1,2]=(-0.0215096239827)*x[0]**o+(0.92491389564)*x[0]+(0.820884930171)*x[1]**o+(0.907170170451)*x[1]
        s[1,3]=(0.151439258285)*x[0]**o+(-0.619065403082)*x[0]+(-0.931701693869)*x[1]**o+(-0.636007349209)*x[1]
        s[1,4]=(0.599937864144)*x[0]**o+(0.856401446192)*x[0]+(0.0498004863454)*x[1]**o+(-0.542473731625)*x[1]
        s[2,0]=(0.114304895833)*x[0]**o+(-0.688107927554)*x[0]+(0.391000230103)*x[1]**o+(-0.406487512048)*x[1]
        s[2,1]=(-0.745374769533)*x[0]**o+(0.797317533966)*x[0]+(-0.667327745105)*x[1]**o+(-0.356428295255)*x[1]
        s[2,2]=(-0.381300049096)*x[0]**o+(-0.187744971624)*x[0]+(0.361881540737)*x[1]**o+(-0.116712686284)*x[1]
        s[2,3]=(-0.156007331641)*x[0]**o+(0.887169440804)*x[0]+(-0.324903456097)*x[1]**o+(0.360748003763)*x[1]
        s[2,4]=(0.712524916579)*x[0]**o+(-0.638314673897)*x[0]+(-0.0308273014936)*x[1]**o+(-0.602816096773)*x[1]
        s[3,0]=(0.269003703906)*x[0]**o+(-0.648611537709)*x[0]+(0.0450380502202)*x[1]**o+(0.261193188134)*x[1]
        s[3,1]=(0.84586203363)*x[0]**o+(0.698542521798)*x[0]+(0.827757021664)*x[1]**o+(-0.297626737453)*x[1]
        s[3,2]=(-0.346844241342)*x[0]**o+(-0.609694742721)*x[0]+(-0.34363112844)*x[1]**o+(0.898190611922)*x[1]
        s[3,3]=(-0.150818980857)*x[0]**o+(-0.0555804912754)*x[0]+(0.543245030743)*x[1]**o+(0.862313486187)*x[1]
        s[3,4]=(0.191954299926)*x[0]**o+(0.218021623666)*x[0]+(-0.354607040653)*x[1]**o+(-0.382222211827)*x[1]
        ref[0,0]=(0.777675758098)*x_ref[0]**o+(-0.473156188055)*x_ref[0]+(-0.83738893374)*x_ref[1]**o+(-0.772375433518)*x_ref[1]
        ref[0,1]=(0.690632604654)*x_ref[0]**o+(-0.596947428521)*x_ref[0]+(-0.927206976637)*x_ref[1]**o+(0.416133449904)*x_ref[1]
        ref[0,2]=(0.159353873068)*x_ref[0]**o+(-0.544617444157)*x_ref[0]+(-0.384185707939)*x_ref[1]**o+(0.630137005341)*x_ref[1]
        ref[0,3]=(-0.677069489949)*x_ref[0]**o+(0.121886164808)*x_ref[0]+(-0.243464410455)*x_ref[1]**o+(-0.424481834693)*x_ref[1]
        ref[0,4]=(-0.0586502823616)*x_ref[0]**o+(-0.952781957131)*x_ref[0]+(-0.638027009478)*x_ref[1]**o+(-0.939836467104)*x_ref[1]
        ref[1,0]=(0.946604826689)*x_ref[0]**o+(-0.344506753444)*x_ref[0]+(-0.366256116023)*x_ref[1]**o+(0.66571445335)*x_ref[1]
        ref[1,1]=(-0.148616405902)*x_ref[0]**o+(0.24049619296)*x_ref[0]+(-0.59987968565)*x_ref[1]**o+(-0.30617564854)*x_ref[1]
        ref[1,2]=(-0.0215096239827)*x_ref[0]**o+(0.92491389564)*x_ref[0]+(0.820884930171)*x_ref[1]**o+(0.907170170451)*x_ref[1]
        ref[1,3]=(0.151439258285)*x_ref[0]**o+(-0.619065403082)*x_ref[0]+(-0.931701693869)*x_ref[1]**o+(-0.636007349209)*x_ref[1]
        ref[1,4]=(0.599937864144)*x_ref[0]**o+(0.856401446192)*x_ref[0]+(0.0498004863454)*x_ref[1]**o+(-0.542473731625)*x_ref[1]
        ref[2,0]=(0.114304895833)*x_ref[0]**o+(-0.688107927554)*x_ref[0]+(0.391000230103)*x_ref[1]**o+(-0.406487512048)*x_ref[1]
        ref[2,1]=(-0.745374769533)*x_ref[0]**o+(0.797317533966)*x_ref[0]+(-0.667327745105)*x_ref[1]**o+(-0.356428295255)*x_ref[1]
        ref[2,2]=(-0.381300049096)*x_ref[0]**o+(-0.187744971624)*x_ref[0]+(0.361881540737)*x_ref[1]**o+(-0.116712686284)*x_ref[1]
        ref[2,3]=(-0.156007331641)*x_ref[0]**o+(0.887169440804)*x_ref[0]+(-0.324903456097)*x_ref[1]**o+(0.360748003763)*x_ref[1]
        ref[2,4]=(0.712524916579)*x_ref[0]**o+(-0.638314673897)*x_ref[0]+(-0.0308273014936)*x_ref[1]**o+(-0.602816096773)*x_ref[1]
        ref[3,0]=(0.269003703906)*x_ref[0]**o+(-0.648611537709)*x_ref[0]+(0.0450380502202)*x_ref[1]**o+(0.261193188134)*x_ref[1]
        ref[3,1]=(0.84586203363)*x_ref[0]**o+(0.698542521798)*x_ref[0]+(0.827757021664)*x_ref[1]**o+(-0.297626737453)*x_ref[1]
        ref[3,2]=(-0.346844241342)*x_ref[0]**o+(-0.609694742721)*x_ref[0]+(-0.34363112844)*x_ref[1]**o+(0.898190611922)*x_ref[1]
        ref[3,3]=(-0.150818980857)*x_ref[0]**o+(-0.0555804912754)*x_ref[0]+(0.543245030743)*x_ref[1]**o+(0.862313486187)*x_ref[1]
        ref[3,4]=(0.191954299926)*x_ref[0]**o+(0.218021623666)*x_ref[0]+(-0.354607040653)*x_ref[1]**o+(-0.382222211827)*x_ref[1]
      else:
        s[0,0]=(0.264155606808)*x[0]**o+(0.865349854552)*x[0]+(0.290185544602)*x[1]**o+(-0.881187362254)*x[1]+(0.0974065632758)*x[2]**o+(-0.0146386375323)*x[2]
        s[0,1]=(-0.553039444966)*x[0]**o+(-0.225032077709)*x[0]+(-0.114709392843)*x[1]**o+(0.476316128194)*x[1]+(-0.157256007297)*x[2]**o+(-0.881306470969)*x[2]
        s[0,2]=(0.204050245804)*x[0]**o+(0.843162287468)*x[0]+(0.479364138104)*x[1]**o+(0.584277416031)*x[1]+(-0.728022519952)*x[2]**o+(0.455950432012)*x[2]
        s[0,3]=(0.337825383533)*x[0]**o+(-0.727944018401)*x[0]+(-0.884967739733)*x[1]**o+(0.0927602193719)*x[1]+(0.693987105436)*x[2]**o+(-0.212308959739)*x[2]
        s[0,4]=(-0.663473315009)*x[0]**o+(-0.534399657707)*x[0]+(0.00834675143069)*x[1]**o+(-0.323653373064)*x[1]+(0.138014603409)*x[2]**o+(-0.833018982749)*x[2]
        s[1,0]=(0.18898839339)*x[0]**o+(-0.0348685084852)*x[0]+(0.822229497116)*x[1]**o+(0.512525434163)*x[1]+(-0.136217897358)*x[2]**o+(0.503370342407)*x[2]
        s[1,1]=(-0.628784998358)*x[0]**o+(-0.253509454631)*x[0]+(-0.192305899603)*x[1]**o+(-0.415871165125)*x[1]+(0.709676594594)*x[2]**o+(0.331428469664)*x[2]
        s[1,2]=(0.459380888002)*x[0]**o+(-0.0693389935675)*x[0]+(-0.853473419272)*x[1]**o+(-0.221441495424)*x[1]+(0.649741007162)*x[2]**o+(0.138511312385)*x[2]
        s[1,3]=(-0.130673271522)*x[0]**o+(-0.273321361436)*x[0]+(-0.799741846799)*x[1]**o+(-0.950206189845)*x[1]+(-0.502011070117)*x[2]**o+(0.202315792805)*x[2]
        s[1,4]=(-0.131491315255)*x[0]**o+(0.814802856108)*x[0]+(0.12368780731)*x[1]**o+(-0.184462314547)*x[1]+(0.430222618358)*x[2]**o+(-0.74005756053)*x[2]
        s[2,0]=(-0.713274359703)*x[0]**o+(-0.0251491873794)*x[0]+(-0.622142303525)*x[1]**o+(0.933649343169)*x[1]+(0.312383777738)*x[2]**o+(0.533383231604)*x[2]
        s[2,1]=(0.311750294112)*x[0]**o+(0.144605685448)*x[0]+(-0.980716557512)*x[1]**o+(0.575451487258)*x[1]+(-0.0552788567322)*x[2]**o+(0.416492300045)*x[2]
        s[2,2]=(-0.543301125737)*x[0]**o+(0.788016744944)*x[0]+(0.244619454713)*x[1]**o+(0.570164703338)*x[1]+(-0.448168767253)*x[2]**o+(-0.346442604379)*x[2]
        s[2,3]=(0.731364271175)*x[0]**o+(-0.787549613302)*x[0]+(0.119788530606)*x[1]**o+(0.287282876919)*x[1]+(-0.235292928704)*x[2]**o+(-0.953636574666)*x[2]
        s[2,4]=(0.449796191117)*x[0]**o+(-0.648747759536)*x[0]+(0.601089847127)*x[1]**o+(0.593778367156)*x[1]+(-0.800274848868)*x[2]**o+(-0.576472670212)*x[2]
        s[3,0]=(0.951920248404)*x[0]**o+(-0.42219089138)*x[0]+(0.728667162964)*x[1]**o+(-0.882554153116)*x[1]+(-0.820351735649)*x[2]**o+(0.169030492382)*x[2]
        s[3,1]=(-0.497772815911)*x[0]**o+(0.798040706328)*x[0]+(-0.197945158571)*x[1]**o+(0.974647527948)*x[1]+(0.903944658321)*x[2]**o+(-0.249832296182)*x[2]
        s[3,2]=(0.98668376271)*x[0]**o+(-0.102390184426)*x[0]+(0.70946055219)*x[1]**o+(-0.493091573905)*x[1]+(-0.778257510928)*x[2]**o+(-0.413832616402)*x[2]
        s[3,3]=(0.62445606614)*x[0]**o+(-0.0898301047789)*x[0]+(0.788538530992)*x[1]**o+(0.702286686509)*x[1]+(-0.487082450751)*x[2]**o+(0.875348478204)*x[2]
        s[3,4]=(0.957689119464)*x[0]**o+(-0.855933895119)*x[0]+(0.119002293838)*x[1]**o+(-0.00826726591134)*x[1]+(0.145310022693)*x[2]**o+(0.842408432594)*x[2]
        ref[0,0]=(0.264155606808)*x_ref[0]**o+(0.865349854552)*x_ref[0]+(0.290185544602)*x_ref[1]**o+(-0.881187362254)*x_ref[1]+(0.0974065632758)*x_ref[2]**o+(-0.0146386375323)*x_ref[2]
        ref[0,1]=(-0.553039444966)*x_ref[0]**o+(-0.225032077709)*x_ref[0]+(-0.114709392843)*x_ref[1]**o+(0.476316128194)*x_ref[1]+(-0.157256007297)*x_ref[2]**o+(-0.881306470969)*x_ref[2]
        ref[0,2]=(0.204050245804)*x_ref[0]**o+(0.843162287468)*x_ref[0]+(0.479364138104)*x_ref[1]**o+(0.584277416031)*x_ref[1]+(-0.728022519952)*x_ref[2]**o+(0.455950432012)*x_ref[2]
        ref[0,3]=(0.337825383533)*x_ref[0]**o+(-0.727944018401)*x_ref[0]+(-0.884967739733)*x_ref[1]**o+(0.0927602193719)*x_ref[1]+(0.693987105436)*x_ref[2]**o+(-0.212308959739)*x_ref[2]
        ref[0,4]=(-0.663473315009)*x_ref[0]**o+(-0.534399657707)*x_ref[0]+(0.00834675143069)*x_ref[1]**o+(-0.323653373064)*x_ref[1]+(0.138014603409)*x_ref[2]**o+(-0.833018982749)*x_ref[2]
        ref[1,0]=(0.18898839339)*x_ref[0]**o+(-0.0348685084852)*x_ref[0]+(0.822229497116)*x_ref[1]**o+(0.512525434163)*x_ref[1]+(-0.136217897358)*x_ref[2]**o+(0.503370342407)*x_ref[2]
        ref[1,1]=(-0.628784998358)*x_ref[0]**o+(-0.253509454631)*x_ref[0]+(-0.192305899603)*x_ref[1]**o+(-0.415871165125)*x_ref[1]+(0.709676594594)*x_ref[2]**o+(0.331428469664)*x_ref[2]
        ref[1,2]=(0.459380888002)*x_ref[0]**o+(-0.0693389935675)*x_ref[0]+(-0.853473419272)*x_ref[1]**o+(-0.221441495424)*x_ref[1]+(0.649741007162)*x_ref[2]**o+(0.138511312385)*x_ref[2]
        ref[1,3]=(-0.130673271522)*x_ref[0]**o+(-0.273321361436)*x_ref[0]+(-0.799741846799)*x_ref[1]**o+(-0.950206189845)*x_ref[1]+(-0.502011070117)*x_ref[2]**o+(0.202315792805)*x_ref[2]
        ref[1,4]=(-0.131491315255)*x_ref[0]**o+(0.814802856108)*x_ref[0]+(0.12368780731)*x_ref[1]**o+(-0.184462314547)*x_ref[1]+(0.430222618358)*x_ref[2]**o+(-0.74005756053)*x_ref[2]
        ref[2,0]=(-0.713274359703)*x_ref[0]**o+(-0.0251491873794)*x_ref[0]+(-0.622142303525)*x_ref[1]**o+(0.933649343169)*x_ref[1]+(0.312383777738)*x_ref[2]**o+(0.533383231604)*x_ref[2]
        ref[2,1]=(0.311750294112)*x_ref[0]**o+(0.144605685448)*x_ref[0]+(-0.980716557512)*x_ref[1]**o+(0.575451487258)*x_ref[1]+(-0.0552788567322)*x_ref[2]**o+(0.416492300045)*x_ref[2]
        ref[2,2]=(-0.543301125737)*x_ref[0]**o+(0.788016744944)*x_ref[0]+(0.244619454713)*x_ref[1]**o+(0.570164703338)*x_ref[1]+(-0.448168767253)*x_ref[2]**o+(-0.346442604379)*x_ref[2]
        ref[2,3]=(0.731364271175)*x_ref[0]**o+(-0.787549613302)*x_ref[0]+(0.119788530606)*x_ref[1]**o+(0.287282876919)*x_ref[1]+(-0.235292928704)*x_ref[2]**o+(-0.953636574666)*x_ref[2]
        ref[2,4]=(0.449796191117)*x_ref[0]**o+(-0.648747759536)*x_ref[0]+(0.601089847127)*x_ref[1]**o+(0.593778367156)*x_ref[1]+(-0.800274848868)*x_ref[2]**o+(-0.576472670212)*x_ref[2]
        ref[3,0]=(0.951920248404)*x_ref[0]**o+(-0.42219089138)*x_ref[0]+(0.728667162964)*x_ref[1]**o+(-0.882554153116)*x_ref[1]+(-0.820351735649)*x_ref[2]**o+(0.169030492382)*x_ref[2]
        ref[3,1]=(-0.497772815911)*x_ref[0]**o+(0.798040706328)*x_ref[0]+(-0.197945158571)*x_ref[1]**o+(0.974647527948)*x_ref[1]+(0.903944658321)*x_ref[2]**o+(-0.249832296182)*x_ref[2]
        ref[3,2]=(0.98668376271)*x_ref[0]**o+(-0.102390184426)*x_ref[0]+(0.70946055219)*x_ref[1]**o+(-0.493091573905)*x_ref[1]+(-0.778257510928)*x_ref[2]**o+(-0.413832616402)*x_ref[2]
        ref[3,3]=(0.62445606614)*x_ref[0]**o+(-0.0898301047789)*x_ref[0]+(0.788538530992)*x_ref[1]**o+(0.702286686509)*x_ref[1]+(-0.487082450751)*x_ref[2]**o+(0.875348478204)*x_ref[2]
        ref[3,4]=(0.957689119464)*x_ref[0]**o+(-0.855933895119)*x_ref[0]+(0.119002293838)*x_ref[1]**o+(-0.00826726591134)*x_ref[1]+(0.145310022693)*x_ref[2]**o+(0.842408432594)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromSymbol_ReducedFunctionOnContactZero_rank3(self):
      """
      tests interpolate for rank 3 Symbol onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactZero onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactZero(self.domain)
      x=w.getX()
      arg=Symbol(shape=(6, 2, 2),dim=dim)
      s=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2),w_ref)
      if dim==2:
        s[0,0,0]=(-0.712152806182)*x[0]**o+(0.553286790231)*x[0]+(-0.725363109396)*x[1]**o+(-0.788986615699)*x[1]
        s[0,0,1]=(0.0651260964364)*x[0]**o+(0.846234983166)*x[0]+(0.220346387636)*x[1]**o+(-0.169715997542)*x[1]
        s[0,1,0]=(-0.312770289544)*x[0]**o+(-0.830517475738)*x[0]+(0.532769330513)*x[1]**o+(-0.502035152634)*x[1]
        s[0,1,1]=(-0.268414866022)*x[0]**o+(0.653788083346)*x[0]+(0.845119607511)*x[1]**o+(0.928640589437)*x[1]
        s[1,0,0]=(0.0174302886823)*x[0]**o+(0.567027146136)*x[0]+(0.658004848684)*x[1]**o+(0.238897231665)*x[1]
        s[1,0,1]=(0.0275230087777)*x[0]**o+(-0.909065705806)*x[0]+(0.616873751656)*x[1]**o+(-0.750647747368)*x[1]
        s[1,1,0]=(0.968790737085)*x[0]**o+(-0.717354080994)*x[0]+(0.322354780033)*x[1]**o+(0.297005507117)*x[1]
        s[1,1,1]=(0.260782453626)*x[0]**o+(-0.294785381256)*x[0]+(0.249751113945)*x[1]**o+(-0.334065019247)*x[1]
        s[2,0,0]=(-0.443277964789)*x[0]**o+(-0.951540439846)*x[0]+(0.714190838936)*x[1]**o+(0.745547080519)*x[1]
        s[2,0,1]=(0.835967278262)*x[0]**o+(0.849721181077)*x[0]+(-0.45248294099)*x[1]**o+(-0.91527746844)*x[1]
        s[2,1,0]=(-0.550753310799)*x[0]**o+(0.0991213939269)*x[0]+(-0.990618565235)*x[1]**o+(-0.101473335456)*x[1]
        s[2,1,1]=(-0.992001639983)*x[0]**o+(0.582614881862)*x[0]+(0.649744791891)*x[1]**o+(0.910908482017)*x[1]
        s[3,0,0]=(-0.615118121008)*x[0]**o+(0.908419887637)*x[0]+(0.764202178909)*x[1]**o+(-0.10825645538)*x[1]
        s[3,0,1]=(0.494014349582)*x[0]**o+(0.340651430733)*x[0]+(0.837654928126)*x[1]**o+(0.118115058337)*x[1]
        s[3,1,0]=(0.800835589492)*x[0]**o+(0.358498940072)*x[0]+(0.347138646821)*x[1]**o+(-0.760451735532)*x[1]
        s[3,1,1]=(0.221808818118)*x[0]**o+(0.76998621836)*x[0]+(-0.184775740422)*x[1]**o+(0.891454523108)*x[1]
        s[4,0,0]=(-0.552924785414)*x[0]**o+(0.00903957955798)*x[0]+(-0.0176781743627)*x[1]**o+(-0.788145401649)*x[1]
        s[4,0,1]=(0.0557272322524)*x[0]**o+(0.37477815689)*x[0]+(0.920309703574)*x[1]**o+(0.478950854112)*x[1]
        s[4,1,0]=(0.282037005502)*x[0]**o+(-0.42297688723)*x[0]+(0.544702991352)*x[1]**o+(0.418508536472)*x[1]
        s[4,1,1]=(-0.0983696927616)*x[0]**o+(-0.814684556832)*x[0]+(0.291195735479)*x[1]**o+(-0.840465495063)*x[1]
        s[5,0,0]=(-0.653946625568)*x[0]**o+(-0.419703492576)*x[0]+(-0.00132821148354)*x[1]**o+(0.845340315129)*x[1]
        s[5,0,1]=(0.389254602951)*x[0]**o+(0.269315721817)*x[0]+(-0.666779865472)*x[1]**o+(-0.721924390904)*x[1]
        s[5,1,0]=(0.0273782807939)*x[0]**o+(-0.629004324686)*x[0]+(0.469241132749)*x[1]**o+(0.101031019558)*x[1]
        s[5,1,1]=(-0.642652252134)*x[0]**o+(0.055584656243)*x[0]+(0.591952970815)*x[1]**o+(0.0560172089007)*x[1]
        ref[0,0,0]=(-0.712152806182)*x_ref[0]**o+(0.553286790231)*x_ref[0]+(-0.725363109396)*x_ref[1]**o+(-0.788986615699)*x_ref[1]
        ref[0,0,1]=(0.0651260964364)*x_ref[0]**o+(0.846234983166)*x_ref[0]+(0.220346387636)*x_ref[1]**o+(-0.169715997542)*x_ref[1]
        ref[0,1,0]=(-0.312770289544)*x_ref[0]**o+(-0.830517475738)*x_ref[0]+(0.532769330513)*x_ref[1]**o+(-0.502035152634)*x_ref[1]
        ref[0,1,1]=(-0.268414866022)*x_ref[0]**o+(0.653788083346)*x_ref[0]+(0.845119607511)*x_ref[1]**o+(0.928640589437)*x_ref[1]
        ref[1,0,0]=(0.0174302886823)*x_ref[0]**o+(0.567027146136)*x_ref[0]+(0.658004848684)*x_ref[1]**o+(0.238897231665)*x_ref[1]
        ref[1,0,1]=(0.0275230087777)*x_ref[0]**o+(-0.909065705806)*x_ref[0]+(0.616873751656)*x_ref[1]**o+(-0.750647747368)*x_ref[1]
        ref[1,1,0]=(0.968790737085)*x_ref[0]**o+(-0.717354080994)*x_ref[0]+(0.322354780033)*x_ref[1]**o+(0.297005507117)*x_ref[1]
        ref[1,1,1]=(0.260782453626)*x_ref[0]**o+(-0.294785381256)*x_ref[0]+(0.249751113945)*x_ref[1]**o+(-0.334065019247)*x_ref[1]
        ref[2,0,0]=(-0.443277964789)*x_ref[0]**o+(-0.951540439846)*x_ref[0]+(0.714190838936)*x_ref[1]**o+(0.745547080519)*x_ref[1]
        ref[2,0,1]=(0.835967278262)*x_ref[0]**o+(0.849721181077)*x_ref[0]+(-0.45248294099)*x_ref[1]**o+(-0.91527746844)*x_ref[1]
        ref[2,1,0]=(-0.550753310799)*x_ref[0]**o+(0.0991213939269)*x_ref[0]+(-0.990618565235)*x_ref[1]**o+(-0.101473335456)*x_ref[1]
        ref[2,1,1]=(-0.992001639983)*x_ref[0]**o+(0.582614881862)*x_ref[0]+(0.649744791891)*x_ref[1]**o+(0.910908482017)*x_ref[1]
        ref[3,0,0]=(-0.615118121008)*x_ref[0]**o+(0.908419887637)*x_ref[0]+(0.764202178909)*x_ref[1]**o+(-0.10825645538)*x_ref[1]
        ref[3,0,1]=(0.494014349582)*x_ref[0]**o+(0.340651430733)*x_ref[0]+(0.837654928126)*x_ref[1]**o+(0.118115058337)*x_ref[1]
        ref[3,1,0]=(0.800835589492)*x_ref[0]**o+(0.358498940072)*x_ref[0]+(0.347138646821)*x_ref[1]**o+(-0.760451735532)*x_ref[1]
        ref[3,1,1]=(0.221808818118)*x_ref[0]**o+(0.76998621836)*x_ref[0]+(-0.184775740422)*x_ref[1]**o+(0.891454523108)*x_ref[1]
        ref[4,0,0]=(-0.552924785414)*x_ref[0]**o+(0.00903957955798)*x_ref[0]+(-0.0176781743627)*x_ref[1]**o+(-0.788145401649)*x_ref[1]
        ref[4,0,1]=(0.0557272322524)*x_ref[0]**o+(0.37477815689)*x_ref[0]+(0.920309703574)*x_ref[1]**o+(0.478950854112)*x_ref[1]
        ref[4,1,0]=(0.282037005502)*x_ref[0]**o+(-0.42297688723)*x_ref[0]+(0.544702991352)*x_ref[1]**o+(0.418508536472)*x_ref[1]
        ref[4,1,1]=(-0.0983696927616)*x_ref[0]**o+(-0.814684556832)*x_ref[0]+(0.291195735479)*x_ref[1]**o+(-0.840465495063)*x_ref[1]
        ref[5,0,0]=(-0.653946625568)*x_ref[0]**o+(-0.419703492576)*x_ref[0]+(-0.00132821148354)*x_ref[1]**o+(0.845340315129)*x_ref[1]
        ref[5,0,1]=(0.389254602951)*x_ref[0]**o+(0.269315721817)*x_ref[0]+(-0.666779865472)*x_ref[1]**o+(-0.721924390904)*x_ref[1]
        ref[5,1,0]=(0.0273782807939)*x_ref[0]**o+(-0.629004324686)*x_ref[0]+(0.469241132749)*x_ref[1]**o+(0.101031019558)*x_ref[1]
        ref[5,1,1]=(-0.642652252134)*x_ref[0]**o+(0.055584656243)*x_ref[0]+(0.591952970815)*x_ref[1]**o+(0.0560172089007)*x_ref[1]
      else:
        s[0,0,0]=(-0.484756778033)*x[0]**o+(-0.231779672771)*x[0]+(0.919660538872)*x[1]**o+(0.695050320072)*x[1]+(-0.705779426734)*x[2]**o+(-0.422079798585)*x[2]
        s[0,0,1]=(-0.274490489231)*x[0]**o+(0.387197647284)*x[0]+(-0.366490824762)*x[1]**o+(-0.145891361722)*x[1]+(0.397994935942)*x[2]**o+(0.945093224566)*x[2]
        s[0,1,0]=(-0.0703394741878)*x[0]**o+(0.772542317636)*x[0]+(-0.798490294743)*x[1]**o+(0.425742369552)*x[1]+(0.876889140151)*x[2]**o+(-0.796003635544)*x[2]
        s[0,1,1]=(0.96374816047)*x[0]**o+(-0.784131619847)*x[0]+(-0.93299245221)*x[1]**o+(0.0531089993902)*x[1]+(0.0843073749033)*x[2]**o+(0.391297554637)*x[2]
        s[1,0,0]=(-0.481999451884)*x[0]**o+(0.751216783461)*x[0]+(-0.771590406041)*x[1]**o+(0.623595993735)*x[1]+(-0.921210206711)*x[2]**o+(-0.744578883939)*x[2]
        s[1,0,1]=(-0.812200901916)*x[0]**o+(-0.208483140217)*x[0]+(0.413360073886)*x[1]**o+(-0.169471108437)*x[1]+(-0.428293613451)*x[2]**o+(-0.444321672841)*x[2]
        s[1,1,0]=(0.434668673877)*x[0]**o+(-0.227506663361)*x[0]+(0.396429343706)*x[1]**o+(-0.513657758836)*x[1]+(-0.957101683551)*x[2]**o+(-0.736199356445)*x[2]
        s[1,1,1]=(0.414698036573)*x[0]**o+(0.664686946926)*x[0]+(0.813892586594)*x[1]**o+(-0.991059972133)*x[1]+(-0.966403253116)*x[2]**o+(0.87196549345)*x[2]
        s[2,0,0]=(0.412695063456)*x[0]**o+(0.500236015073)*x[0]+(0.264707290309)*x[1]**o+(-0.877912823257)*x[1]+(0.702553921733)*x[2]**o+(-0.866356407426)*x[2]
        s[2,0,1]=(-0.736177808565)*x[0]**o+(-0.881438411498)*x[0]+(-0.92649028732)*x[1]**o+(-0.296241009813)*x[1]+(0.435099826882)*x[2]**o+(0.868673212655)*x[2]
        s[2,1,0]=(-0.812274827825)*x[0]**o+(0.808913617552)*x[0]+(-0.30994330994)*x[1]**o+(0.425208195893)*x[1]+(0.155091457675)*x[2]**o+(0.989636493062)*x[2]
        s[2,1,1]=(0.704418763542)*x[0]**o+(0.714721207479)*x[0]+(-0.772435541761)*x[1]**o+(-0.701532388352)*x[1]+(-0.0556099968738)*x[2]**o+(-0.385766816311)*x[2]
        s[3,0,0]=(0.329616772304)*x[0]**o+(0.497122208449)*x[0]+(0.803535729001)*x[1]**o+(-0.0936214856199)*x[1]+(0.568734100002)*x[2]**o+(-0.357279001712)*x[2]
        s[3,0,1]=(-0.885441319041)*x[0]**o+(-0.9962223468)*x[0]+(0.875311867648)*x[1]**o+(0.421410419839)*x[1]+(0.612998402692)*x[2]**o+(0.77679279321)*x[2]
        s[3,1,0]=(0.70421388911)*x[0]**o+(-0.0190681099318)*x[0]+(-0.841932997223)*x[1]**o+(-0.0250788950858)*x[1]+(-0.222261918598)*x[2]**o+(-0.271140805403)*x[2]
        s[3,1,1]=(0.0694954416869)*x[0]**o+(-0.391253608839)*x[0]+(0.0431700251114)*x[1]**o+(0.405848258278)*x[1]+(-0.615615286047)*x[2]**o+(-0.619571625435)*x[2]
        s[4,0,0]=(-0.686666614457)*x[0]**o+(0.545505992881)*x[0]+(0.334621135483)*x[1]**o+(0.751135339301)*x[1]+(0.61064481817)*x[2]**o+(0.482901826784)*x[2]
        s[4,0,1]=(0.775613779362)*x[0]**o+(0.614521704508)*x[0]+(-0.0189207080695)*x[1]**o+(0.228118267253)*x[1]+(-0.4140063649)*x[2]**o+(0.944802560539)*x[2]
        s[4,1,0]=(0.118375936359)*x[0]**o+(-0.889963092371)*x[0]+(-0.511745894234)*x[1]**o+(0.545576927355)*x[1]+(-0.11589270826)*x[2]**o+(0.559538534484)*x[2]
        s[4,1,1]=(-0.933623754993)*x[0]**o+(-0.174346861161)*x[0]+(-0.101529487111)*x[1]**o+(-0.943622750193)*x[1]+(-0.625527859586)*x[2]**o+(-0.463443945073)*x[2]
        s[5,0,0]=(-0.60504471632)*x[0]**o+(-0.783562086238)*x[0]+(0.614590790432)*x[1]**o+(0.421598441357)*x[1]+(0.433099495473)*x[2]**o+(0.658822060627)*x[2]
        s[5,0,1]=(0.951028673162)*x[0]**o+(0.757383296595)*x[0]+(-0.671328274248)*x[1]**o+(0.30758815695)*x[1]+(0.429507454083)*x[2]**o+(-0.0784898035186)*x[2]
        s[5,1,0]=(0.357359564527)*x[0]**o+(0.402138711471)*x[0]+(0.397488404381)*x[1]**o+(0.532261812787)*x[1]+(-0.379695545138)*x[2]**o+(-0.351422772499)*x[2]
        s[5,1,1]=(0.776246461598)*x[0]**o+(-0.552578551)*x[0]+(0.770179118746)*x[1]**o+(0.51956785933)*x[1]+(-0.182066259842)*x[2]**o+(-0.915270193133)*x[2]
        ref[0,0,0]=(-0.484756778033)*x_ref[0]**o+(-0.231779672771)*x_ref[0]+(0.919660538872)*x_ref[1]**o+(0.695050320072)*x_ref[1]+(-0.705779426734)*x_ref[2]**o+(-0.422079798585)*x_ref[2]
        ref[0,0,1]=(-0.274490489231)*x_ref[0]**o+(0.387197647284)*x_ref[0]+(-0.366490824762)*x_ref[1]**o+(-0.145891361722)*x_ref[1]+(0.397994935942)*x_ref[2]**o+(0.945093224566)*x_ref[2]
        ref[0,1,0]=(-0.0703394741878)*x_ref[0]**o+(0.772542317636)*x_ref[0]+(-0.798490294743)*x_ref[1]**o+(0.425742369552)*x_ref[1]+(0.876889140151)*x_ref[2]**o+(-0.796003635544)*x_ref[2]
        ref[0,1,1]=(0.96374816047)*x_ref[0]**o+(-0.784131619847)*x_ref[0]+(-0.93299245221)*x_ref[1]**o+(0.0531089993902)*x_ref[1]+(0.0843073749033)*x_ref[2]**o+(0.391297554637)*x_ref[2]
        ref[1,0,0]=(-0.481999451884)*x_ref[0]**o+(0.751216783461)*x_ref[0]+(-0.771590406041)*x_ref[1]**o+(0.623595993735)*x_ref[1]+(-0.921210206711)*x_ref[2]**o+(-0.744578883939)*x_ref[2]
        ref[1,0,1]=(-0.812200901916)*x_ref[0]**o+(-0.208483140217)*x_ref[0]+(0.413360073886)*x_ref[1]**o+(-0.169471108437)*x_ref[1]+(-0.428293613451)*x_ref[2]**o+(-0.444321672841)*x_ref[2]
        ref[1,1,0]=(0.434668673877)*x_ref[0]**o+(-0.227506663361)*x_ref[0]+(0.396429343706)*x_ref[1]**o+(-0.513657758836)*x_ref[1]+(-0.957101683551)*x_ref[2]**o+(-0.736199356445)*x_ref[2]
        ref[1,1,1]=(0.414698036573)*x_ref[0]**o+(0.664686946926)*x_ref[0]+(0.813892586594)*x_ref[1]**o+(-0.991059972133)*x_ref[1]+(-0.966403253116)*x_ref[2]**o+(0.87196549345)*x_ref[2]
        ref[2,0,0]=(0.412695063456)*x_ref[0]**o+(0.500236015073)*x_ref[0]+(0.264707290309)*x_ref[1]**o+(-0.877912823257)*x_ref[1]+(0.702553921733)*x_ref[2]**o+(-0.866356407426)*x_ref[2]
        ref[2,0,1]=(-0.736177808565)*x_ref[0]**o+(-0.881438411498)*x_ref[0]+(-0.92649028732)*x_ref[1]**o+(-0.296241009813)*x_ref[1]+(0.435099826882)*x_ref[2]**o+(0.868673212655)*x_ref[2]
        ref[2,1,0]=(-0.812274827825)*x_ref[0]**o+(0.808913617552)*x_ref[0]+(-0.30994330994)*x_ref[1]**o+(0.425208195893)*x_ref[1]+(0.155091457675)*x_ref[2]**o+(0.989636493062)*x_ref[2]
        ref[2,1,1]=(0.704418763542)*x_ref[0]**o+(0.714721207479)*x_ref[0]+(-0.772435541761)*x_ref[1]**o+(-0.701532388352)*x_ref[1]+(-0.0556099968738)*x_ref[2]**o+(-0.385766816311)*x_ref[2]
        ref[3,0,0]=(0.329616772304)*x_ref[0]**o+(0.497122208449)*x_ref[0]+(0.803535729001)*x_ref[1]**o+(-0.0936214856199)*x_ref[1]+(0.568734100002)*x_ref[2]**o+(-0.357279001712)*x_ref[2]
        ref[3,0,1]=(-0.885441319041)*x_ref[0]**o+(-0.9962223468)*x_ref[0]+(0.875311867648)*x_ref[1]**o+(0.421410419839)*x_ref[1]+(0.612998402692)*x_ref[2]**o+(0.77679279321)*x_ref[2]
        ref[3,1,0]=(0.70421388911)*x_ref[0]**o+(-0.0190681099318)*x_ref[0]+(-0.841932997223)*x_ref[1]**o+(-0.0250788950858)*x_ref[1]+(-0.222261918598)*x_ref[2]**o+(-0.271140805403)*x_ref[2]
        ref[3,1,1]=(0.0694954416869)*x_ref[0]**o+(-0.391253608839)*x_ref[0]+(0.0431700251114)*x_ref[1]**o+(0.405848258278)*x_ref[1]+(-0.615615286047)*x_ref[2]**o+(-0.619571625435)*x_ref[2]
        ref[4,0,0]=(-0.686666614457)*x_ref[0]**o+(0.545505992881)*x_ref[0]+(0.334621135483)*x_ref[1]**o+(0.751135339301)*x_ref[1]+(0.61064481817)*x_ref[2]**o+(0.482901826784)*x_ref[2]
        ref[4,0,1]=(0.775613779362)*x_ref[0]**o+(0.614521704508)*x_ref[0]+(-0.0189207080695)*x_ref[1]**o+(0.228118267253)*x_ref[1]+(-0.4140063649)*x_ref[2]**o+(0.944802560539)*x_ref[2]
        ref[4,1,0]=(0.118375936359)*x_ref[0]**o+(-0.889963092371)*x_ref[0]+(-0.511745894234)*x_ref[1]**o+(0.545576927355)*x_ref[1]+(-0.11589270826)*x_ref[2]**o+(0.559538534484)*x_ref[2]
        ref[4,1,1]=(-0.933623754993)*x_ref[0]**o+(-0.174346861161)*x_ref[0]+(-0.101529487111)*x_ref[1]**o+(-0.943622750193)*x_ref[1]+(-0.625527859586)*x_ref[2]**o+(-0.463443945073)*x_ref[2]
        ref[5,0,0]=(-0.60504471632)*x_ref[0]**o+(-0.783562086238)*x_ref[0]+(0.614590790432)*x_ref[1]**o+(0.421598441357)*x_ref[1]+(0.433099495473)*x_ref[2]**o+(0.658822060627)*x_ref[2]
        ref[5,0,1]=(0.951028673162)*x_ref[0]**o+(0.757383296595)*x_ref[0]+(-0.671328274248)*x_ref[1]**o+(0.30758815695)*x_ref[1]+(0.429507454083)*x_ref[2]**o+(-0.0784898035186)*x_ref[2]
        ref[5,1,0]=(0.357359564527)*x_ref[0]**o+(0.402138711471)*x_ref[0]+(0.397488404381)*x_ref[1]**o+(0.532261812787)*x_ref[1]+(-0.379695545138)*x_ref[2]**o+(-0.351422772499)*x_ref[2]
        ref[5,1,1]=(0.776246461598)*x_ref[0]**o+(-0.552578551)*x_ref[0]+(0.770179118746)*x_ref[1]**o+(0.51956785933)*x_ref[1]+(-0.182066259842)*x_ref[2]**o+(-0.915270193133)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromSymbol_ReducedFunctionOnContactZero_rank4(self):
      """
      tests interpolate for rank 4 Symbol onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactZero onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactZero(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5, 3, 2),dim=dim)
      s=Data(0,(4, 5, 3, 2),w)
      ref=Data(0,(4, 5, 3, 2),w_ref)
      if dim==2:
        s[0,0,0,0]=(-0.682677446173)*x[0]**o+(0.504486605276)*x[0]+(-0.188432273127)*x[1]**o+(-0.115942662064)*x[1]
        s[0,0,0,1]=(0.0136192617812)*x[0]**o+(-0.538389464927)*x[0]+(0.124702227013)*x[1]**o+(0.501919436502)*x[1]
        s[0,0,1,0]=(0.156652087198)*x[0]**o+(0.337393016047)*x[0]+(-0.448397431209)*x[1]**o+(0.117709269726)*x[1]
        s[0,0,1,1]=(-0.547896793651)*x[0]**o+(-0.747571514787)*x[0]+(-0.340601123893)*x[1]**o+(-0.710572632704)*x[1]
        s[0,0,2,0]=(-0.780567482126)*x[0]**o+(0.245095469386)*x[0]+(0.114230637835)*x[1]**o+(-0.608414329196)*x[1]
        s[0,0,2,1]=(-0.492428786813)*x[0]**o+(0.931675985507)*x[0]+(-0.748267062348)*x[1]**o+(-0.856380494895)*x[1]
        s[0,1,0,0]=(-0.596856834994)*x[0]**o+(0.147456639538)*x[0]+(0.189964473187)*x[1]**o+(0.618844379651)*x[1]
        s[0,1,0,1]=(0.301379428437)*x[0]**o+(-0.970231773668)*x[0]+(-0.298459364955)*x[1]**o+(-0.823167810649)*x[1]
        s[0,1,1,0]=(0.815054294305)*x[0]**o+(-0.79025331211)*x[0]+(-0.348789406252)*x[1]**o+(0.69147436715)*x[1]
        s[0,1,1,1]=(-0.994338848091)*x[0]**o+(-0.352485315079)*x[0]+(-0.825830724905)*x[1]**o+(0.568756966202)*x[1]
        s[0,1,2,0]=(-0.262404141899)*x[0]**o+(0.695058140486)*x[0]+(-0.562413865895)*x[1]**o+(0.744853779046)*x[1]
        s[0,1,2,1]=(-0.526480240707)*x[0]**o+(-0.246633022689)*x[0]+(-0.677508152973)*x[1]**o+(0.263085459331)*x[1]
        s[0,2,0,0]=(-0.440455373866)*x[0]**o+(0.982673077522)*x[0]+(0.267643265744)*x[1]**o+(-0.248708912008)*x[1]
        s[0,2,0,1]=(-0.629497400912)*x[0]**o+(-0.222971392637)*x[0]+(0.476895454043)*x[1]**o+(-0.560588300663)*x[1]
        s[0,2,1,0]=(-0.142675958641)*x[0]**o+(0.858528587711)*x[0]+(-0.848579730169)*x[1]**o+(-0.0813066455284)*x[1]
        s[0,2,1,1]=(0.309891337726)*x[0]**o+(-0.966050939539)*x[0]+(-0.214997020692)*x[1]**o+(0.823855020343)*x[1]
        s[0,2,2,0]=(0.61116679522)*x[0]**o+(0.663744432393)*x[0]+(0.414661416033)*x[1]**o+(-0.996447270631)*x[1]
        s[0,2,2,1]=(-0.634958711796)*x[0]**o+(0.27654613916)*x[0]+(0.157322235818)*x[1]**o+(-0.449130441789)*x[1]
        s[0,3,0,0]=(-0.949513930962)*x[0]**o+(0.43714393009)*x[0]+(0.44401680011)*x[1]**o+(0.775550283269)*x[1]
        s[0,3,0,1]=(-0.560623279771)*x[0]**o+(-0.261230828373)*x[0]+(0.809046529697)*x[1]**o+(-0.00091638239609)*x[1]
        s[0,3,1,0]=(-0.897951425228)*x[0]**o+(-0.854681659889)*x[0]+(-0.600467143561)*x[1]**o+(-0.543812193981)*x[1]
        s[0,3,1,1]=(-0.252640196006)*x[0]**o+(-0.0147679084824)*x[0]+(0.474400135578)*x[1]**o+(-0.649969341279)*x[1]
        s[0,3,2,0]=(0.148265101594)*x[0]**o+(0.865387108497)*x[0]+(-0.853468586257)*x[1]**o+(-0.728220923395)*x[1]
        s[0,3,2,1]=(-0.998415894528)*x[0]**o+(0.385240420153)*x[0]+(-0.970588182269)*x[1]**o+(-0.715015353507)*x[1]
        s[0,4,0,0]=(0.47018041111)*x[0]**o+(-0.709194247739)*x[0]+(-0.219962382546)*x[1]**o+(0.354322367748)*x[1]
        s[0,4,0,1]=(-0.322814742992)*x[0]**o+(0.549832283705)*x[0]+(0.174383028533)*x[1]**o+(0.582821277257)*x[1]
        s[0,4,1,0]=(0.263785420773)*x[0]**o+(-0.072514835468)*x[0]+(-0.857389703063)*x[1]**o+(-0.775322452374)*x[1]
        s[0,4,1,1]=(0.938271810888)*x[0]**o+(-0.823646250497)*x[0]+(-0.685525888071)*x[1]**o+(-0.603286444077)*x[1]
        s[0,4,2,0]=(-0.609415096341)*x[0]**o+(-0.720923292264)*x[0]+(-0.965452230391)*x[1]**o+(0.285193163647)*x[1]
        s[0,4,2,1]=(0.421976649551)*x[0]**o+(0.990010463736)*x[0]+(-0.860923000529)*x[1]**o+(0.726093525288)*x[1]
        s[1,0,0,0]=(-0.996811081802)*x[0]**o+(-0.97863301939)*x[0]+(0.202626167636)*x[1]**o+(0.0905062876049)*x[1]
        s[1,0,0,1]=(-0.364721401486)*x[0]**o+(0.935308652161)*x[0]+(-0.405836642917)*x[1]**o+(0.550390830913)*x[1]
        s[1,0,1,0]=(-0.701902045301)*x[0]**o+(0.531854328517)*x[0]+(-0.732386759539)*x[1]**o+(0.425326883394)*x[1]
        s[1,0,1,1]=(0.447994101591)*x[0]**o+(-0.813809278167)*x[0]+(-0.539345843485)*x[1]**o+(-0.0354180208603)*x[1]
        s[1,0,2,0]=(-0.603871847208)*x[0]**o+(0.842448438568)*x[0]+(-0.7438939901)*x[1]**o+(-0.587219172421)*x[1]
        s[1,0,2,1]=(-0.971267843684)*x[0]**o+(0.259537498927)*x[0]+(-0.698510853299)*x[1]**o+(0.722962896543)*x[1]
        s[1,1,0,0]=(0.594944086583)*x[0]**o+(0.296356236144)*x[0]+(0.0970882705472)*x[1]**o+(0.849524363397)*x[1]
        s[1,1,0,1]=(0.837454132615)*x[0]**o+(0.0858001627187)*x[0]+(0.019269936186)*x[1]**o+(-0.86311221183)*x[1]
        s[1,1,1,0]=(0.947339105706)*x[0]**o+(0.424118233136)*x[0]+(-0.314979294332)*x[1]**o+(0.507664692875)*x[1]
        s[1,1,1,1]=(0.805027334702)*x[0]**o+(0.682733602548)*x[0]+(-0.162752989543)*x[1]**o+(-0.438803102705)*x[1]
        s[1,1,2,0]=(0.24090406784)*x[0]**o+(-0.564828151023)*x[0]+(-0.446384646081)*x[1]**o+(0.06921350334)*x[1]
        s[1,1,2,1]=(0.535823223755)*x[0]**o+(0.0248971256814)*x[0]+(-0.508444148354)*x[1]**o+(0.840622721627)*x[1]
        s[1,2,0,0]=(0.0525491067608)*x[0]**o+(-0.384851762445)*x[0]+(0.305218692316)*x[1]**o+(-0.319267902416)*x[1]
        s[1,2,0,1]=(0.514545608836)*x[0]**o+(0.157544217219)*x[0]+(0.0378829715039)*x[1]**o+(0.778857744661)*x[1]
        s[1,2,1,0]=(0.702907133813)*x[0]**o+(-0.773975432754)*x[0]+(0.489924281654)*x[1]**o+(-0.771122137868)*x[1]
        s[1,2,1,1]=(0.305517959872)*x[0]**o+(-0.229609729031)*x[0]+(-0.999272871312)*x[1]**o+(0.283774230586)*x[1]
        s[1,2,2,0]=(0.943767954042)*x[0]**o+(0.214819458913)*x[0]+(0.883796685301)*x[1]**o+(0.88624077323)*x[1]
        s[1,2,2,1]=(0.904685466034)*x[0]**o+(0.238007243352)*x[0]+(-0.496694568656)*x[1]**o+(0.90503921706)*x[1]
        s[1,3,0,0]=(-0.323719826916)*x[0]**o+(-0.679887105617)*x[0]+(-0.204658991412)*x[1]**o+(-0.782808301573)*x[1]
        s[1,3,0,1]=(-0.915796887204)*x[0]**o+(-0.210816609805)*x[0]+(-0.229804577299)*x[1]**o+(0.224669921335)*x[1]
        s[1,3,1,0]=(0.0502687121641)*x[0]**o+(-0.873227856206)*x[0]+(-0.159031793692)*x[1]**o+(-0.891685753753)*x[1]
        s[1,3,1,1]=(0.492846348595)*x[0]**o+(0.787085648664)*x[0]+(0.65805763747)*x[1]**o+(0.79345020859)*x[1]
        s[1,3,2,0]=(0.186889967831)*x[0]**o+(-0.759385582402)*x[0]+(0.526210288461)*x[1]**o+(0.408440324329)*x[1]
        s[1,3,2,1]=(0.49057760628)*x[0]**o+(0.583558905672)*x[0]+(0.200515771351)*x[1]**o+(-0.0457600002227)*x[1]
        s[1,4,0,0]=(0.986278922497)*x[0]**o+(-0.724078510554)*x[0]+(0.918959062512)*x[1]**o+(-0.696724335474)*x[1]
        s[1,4,0,1]=(-0.624189125377)*x[0]**o+(0.849396531428)*x[0]+(-0.793610811078)*x[1]**o+(0.473468667211)*x[1]
        s[1,4,1,0]=(0.850059248076)*x[0]**o+(-0.167497586421)*x[0]+(-0.401914828635)*x[1]**o+(-0.446166310885)*x[1]
        s[1,4,1,1]=(0.735056540402)*x[0]**o+(0.956860138776)*x[0]+(-0.657753509921)*x[1]**o+(0.104941138398)*x[1]
        s[1,4,2,0]=(0.178383558524)*x[0]**o+(-0.695410327712)*x[0]+(-0.896595978256)*x[1]**o+(0.500721122887)*x[1]
        s[1,4,2,1]=(0.00266985547612)*x[0]**o+(0.569844332841)*x[0]+(0.30469482092)*x[1]**o+(0.18820848487)*x[1]
        s[2,0,0,0]=(0.0726883565313)*x[0]**o+(-0.151895019503)*x[0]+(-0.566412947745)*x[1]**o+(0.605833954559)*x[1]
        s[2,0,0,1]=(-0.759348816055)*x[0]**o+(0.203730579434)*x[0]+(-0.306267994)*x[1]**o+(-0.121655396311)*x[1]
        s[2,0,1,0]=(-0.529988368349)*x[0]**o+(0.265126908621)*x[0]+(-0.127232300402)*x[1]**o+(-0.711018521148)*x[1]
        s[2,0,1,1]=(0.367616519674)*x[0]**o+(-0.856828909791)*x[0]+(-0.988581037916)*x[1]**o+(0.236273236567)*x[1]
        s[2,0,2,0]=(0.980896603038)*x[0]**o+(0.0101391252514)*x[0]+(-0.734647631536)*x[1]**o+(-0.298628688073)*x[1]
        s[2,0,2,1]=(-0.876812266906)*x[0]**o+(-0.0545067479612)*x[0]+(-0.752772584824)*x[1]**o+(-0.797225970025)*x[1]
        s[2,1,0,0]=(0.9776767796)*x[0]**o+(-0.381948430876)*x[0]+(0.705224317124)*x[1]**o+(0.431609349218)*x[1]
        s[2,1,0,1]=(0.0282522834733)*x[0]**o+(-0.519316841837)*x[0]+(-0.898831067576)*x[1]**o+(0.491570698382)*x[1]
        s[2,1,1,0]=(-0.874077440763)*x[0]**o+(0.0050283385939)*x[0]+(-0.45400999477)*x[1]**o+(-0.823942392997)*x[1]
        s[2,1,1,1]=(0.705592988895)*x[0]**o+(0.545242748068)*x[0]+(-0.0240067342318)*x[1]**o+(-0.764691278707)*x[1]
        s[2,1,2,0]=(0.214087039013)*x[0]**o+(0.198346865884)*x[0]+(0.195275868007)*x[1]**o+(-0.615146376705)*x[1]
        s[2,1,2,1]=(0.819578786211)*x[0]**o+(0.238162773292)*x[0]+(-0.488387775705)*x[1]**o+(0.143425392477)*x[1]
        s[2,2,0,0]=(0.41469049634)*x[0]**o+(-0.870291745289)*x[0]+(-0.978682320238)*x[1]**o+(0.315825060763)*x[1]
        s[2,2,0,1]=(-0.725231531597)*x[0]**o+(-0.583383717934)*x[0]+(0.677865725085)*x[1]**o+(0.590055042471)*x[1]
        s[2,2,1,0]=(-0.827316408906)*x[0]**o+(0.152995858286)*x[0]+(-0.864980658862)*x[1]**o+(-0.267986836659)*x[1]
        s[2,2,1,1]=(-0.333272933954)*x[0]**o+(0.339541494142)*x[0]+(0.982274574789)*x[1]**o+(0.488564854285)*x[1]
        s[2,2,2,0]=(0.433303448225)*x[0]**o+(0.325824460514)*x[0]+(-0.418818415293)*x[1]**o+(0.884475442413)*x[1]
        s[2,2,2,1]=(-0.692651589533)*x[0]**o+(0.85015849795)*x[0]+(-0.683352003673)*x[1]**o+(0.458991254181)*x[1]
        s[2,3,0,0]=(-0.754234631384)*x[0]**o+(0.240243331561)*x[0]+(0.545360646104)*x[1]**o+(-0.960857789697)*x[1]
        s[2,3,0,1]=(0.911046185694)*x[0]**o+(-0.909178154095)*x[0]+(-0.330132665955)*x[1]**o+(-0.978660867939)*x[1]
        s[2,3,1,0]=(0.88161349743)*x[0]**o+(0.0812270265162)*x[0]+(-0.584074638899)*x[1]**o+(-0.309263951959)*x[1]
        s[2,3,1,1]=(-0.59736682275)*x[0]**o+(-0.176681221389)*x[0]+(0.750820855296)*x[1]**o+(0.977121067295)*x[1]
        s[2,3,2,0]=(-0.926460632737)*x[0]**o+(-0.465577825838)*x[0]+(0.577728110369)*x[1]**o+(-0.973296874179)*x[1]
        s[2,3,2,1]=(-0.959292420516)*x[0]**o+(-0.173671029796)*x[0]+(0.644855220567)*x[1]**o+(-0.659401238679)*x[1]
        s[2,4,0,0]=(-0.185887734162)*x[0]**o+(-0.184924703674)*x[0]+(0.12829518222)*x[1]**o+(-0.95529948598)*x[1]
        s[2,4,0,1]=(-0.222082370982)*x[0]**o+(0.480067405868)*x[0]+(0.737491570657)*x[1]**o+(-0.0505388125547)*x[1]
        s[2,4,1,0]=(-0.759488182696)*x[0]**o+(-0.209420047186)*x[0]+(-0.257523848658)*x[1]**o+(-0.88701477207)*x[1]
        s[2,4,1,1]=(0.549919898662)*x[0]**o+(-0.204942654253)*x[0]+(0.237781682077)*x[1]**o+(-0.398029223748)*x[1]
        s[2,4,2,0]=(-0.429624077152)*x[0]**o+(0.477737955149)*x[0]+(0.150704189177)*x[1]**o+(-0.651247641141)*x[1]
        s[2,4,2,1]=(-0.894705558322)*x[0]**o+(-0.437755159653)*x[0]+(-0.984539083926)*x[1]**o+(0.771869802974)*x[1]
        s[3,0,0,0]=(0.613531927806)*x[0]**o+(-0.262670681917)*x[0]+(0.272118391563)*x[1]**o+(-0.911450635138)*x[1]
        s[3,0,0,1]=(0.0749962421787)*x[0]**o+(-0.280125180177)*x[0]+(0.535812839231)*x[1]**o+(-0.898511642603)*x[1]
        s[3,0,1,0]=(-0.493682703273)*x[0]**o+(0.564806921005)*x[0]+(-0.424211826783)*x[1]**o+(0.955398127065)*x[1]
        s[3,0,1,1]=(-0.208922295917)*x[0]**o+(0.552971608191)*x[0]+(-0.482834655473)*x[1]**o+(0.916990053757)*x[1]
        s[3,0,2,0]=(0.225448292089)*x[0]**o+(-0.508992793538)*x[0]+(-0.301030316256)*x[1]**o+(0.234012186162)*x[1]
        s[3,0,2,1]=(0.173694397805)*x[0]**o+(-0.0436667050758)*x[0]+(-0.391980496474)*x[1]**o+(0.183288325597)*x[1]
        s[3,1,0,0]=(-0.078216910574)*x[0]**o+(-0.993944069614)*x[0]+(-0.250488879366)*x[1]**o+(0.648115982978)*x[1]
        s[3,1,0,1]=(-0.346993095448)*x[0]**o+(0.630491595383)*x[0]+(0.128199747925)*x[1]**o+(-0.681982197159)*x[1]
        s[3,1,1,0]=(0.0360684354752)*x[0]**o+(-0.8297568105)*x[0]+(0.122927048201)*x[1]**o+(0.0448718398101)*x[1]
        s[3,1,1,1]=(-0.603474854473)*x[0]**o+(-0.435029027744)*x[0]+(0.632649021375)*x[1]**o+(0.414362202135)*x[1]
        s[3,1,2,0]=(-0.912833626509)*x[0]**o+(-0.828668448463)*x[0]+(-0.754484607948)*x[1]**o+(0.503101904128)*x[1]
        s[3,1,2,1]=(0.00649229353309)*x[0]**o+(-0.96124954377)*x[0]+(0.648294131558)*x[1]**o+(-0.957122691907)*x[1]
        s[3,2,0,0]=(0.476913272421)*x[0]**o+(0.0878038200542)*x[0]+(-0.516953661957)*x[1]**o+(0.0134540618391)*x[1]
        s[3,2,0,1]=(0.3748519133)*x[0]**o+(-0.970356842975)*x[0]+(-0.874984768114)*x[1]**o+(-0.0950957723387)*x[1]
        s[3,2,1,0]=(-0.711688301522)*x[0]**o+(-0.20444931953)*x[0]+(0.601925297065)*x[1]**o+(0.939713142428)*x[1]
        s[3,2,1,1]=(-0.869179475562)*x[0]**o+(-0.771823270124)*x[0]+(0.0447051413494)*x[1]**o+(0.769181827448)*x[1]
        s[3,2,2,0]=(0.341502469812)*x[0]**o+(-0.892625503786)*x[0]+(-0.683764736135)*x[1]**o+(-0.208342034139)*x[1]
        s[3,2,2,1]=(-0.876208883831)*x[0]**o+(-0.705748793184)*x[0]+(0.92260186528)*x[1]**o+(-0.437547481021)*x[1]
        s[3,3,0,0]=(-0.387117059014)*x[0]**o+(-0.30316333118)*x[0]+(-0.107906971687)*x[1]**o+(0.27027380043)*x[1]
        s[3,3,0,1]=(0.760401124102)*x[0]**o+(0.0392442137961)*x[0]+(0.527403460784)*x[1]**o+(0.49506478844)*x[1]
        s[3,3,1,0]=(-0.797802219992)*x[0]**o+(-0.718681383351)*x[0]+(0.723820891624)*x[1]**o+(0.851058726763)*x[1]
        s[3,3,1,1]=(-0.477124467307)*x[0]**o+(-0.912639051937)*x[0]+(-0.653399611549)*x[1]**o+(-0.527816963598)*x[1]
        s[3,3,2,0]=(-0.375620015065)*x[0]**o+(0.713563556042)*x[0]+(0.592214060154)*x[1]**o+(-0.663713190063)*x[1]
        s[3,3,2,1]=(0.13839650904)*x[0]**o+(-0.894351932455)*x[0]+(0.130542033693)*x[1]**o+(-0.420895602544)*x[1]
        s[3,4,0,0]=(-0.438194955853)*x[0]**o+(-0.119296803116)*x[0]+(0.683500326777)*x[1]**o+(-0.226368973557)*x[1]
        s[3,4,0,1]=(-0.232920353049)*x[0]**o+(-0.35856926089)*x[0]+(0.882231830184)*x[1]**o+(-0.000957747821531)*x[1]
        s[3,4,1,0]=(-0.0939581521731)*x[0]**o+(0.831315092139)*x[0]+(0.042454046079)*x[1]**o+(0.791725586274)*x[1]
        s[3,4,1,1]=(-0.328628704689)*x[0]**o+(0.434271457185)*x[0]+(0.632929764639)*x[1]**o+(-0.526043410013)*x[1]
        s[3,4,2,0]=(0.429887737881)*x[0]**o+(0.828275504726)*x[0]+(-0.199204123672)*x[1]**o+(-0.363782496284)*x[1]
        s[3,4,2,1]=(-0.446866424171)*x[0]**o+(-0.204025885117)*x[0]+(-0.899560130794)*x[1]**o+(0.184518622033)*x[1]
        ref[0,0,0,0]=(-0.682677446173)*x_ref[0]**o+(0.504486605276)*x_ref[0]+(-0.188432273127)*x_ref[1]**o+(-0.115942662064)*x_ref[1]
        ref[0,0,0,1]=(0.0136192617812)*x_ref[0]**o+(-0.538389464927)*x_ref[0]+(0.124702227013)*x_ref[1]**o+(0.501919436502)*x_ref[1]
        ref[0,0,1,0]=(0.156652087198)*x_ref[0]**o+(0.337393016047)*x_ref[0]+(-0.448397431209)*x_ref[1]**o+(0.117709269726)*x_ref[1]
        ref[0,0,1,1]=(-0.547896793651)*x_ref[0]**o+(-0.747571514787)*x_ref[0]+(-0.340601123893)*x_ref[1]**o+(-0.710572632704)*x_ref[1]
        ref[0,0,2,0]=(-0.780567482126)*x_ref[0]**o+(0.245095469386)*x_ref[0]+(0.114230637835)*x_ref[1]**o+(-0.608414329196)*x_ref[1]
        ref[0,0,2,1]=(-0.492428786813)*x_ref[0]**o+(0.931675985507)*x_ref[0]+(-0.748267062348)*x_ref[1]**o+(-0.856380494895)*x_ref[1]
        ref[0,1,0,0]=(-0.596856834994)*x_ref[0]**o+(0.147456639538)*x_ref[0]+(0.189964473187)*x_ref[1]**o+(0.618844379651)*x_ref[1]
        ref[0,1,0,1]=(0.301379428437)*x_ref[0]**o+(-0.970231773668)*x_ref[0]+(-0.298459364955)*x_ref[1]**o+(-0.823167810649)*x_ref[1]
        ref[0,1,1,0]=(0.815054294305)*x_ref[0]**o+(-0.79025331211)*x_ref[0]+(-0.348789406252)*x_ref[1]**o+(0.69147436715)*x_ref[1]
        ref[0,1,1,1]=(-0.994338848091)*x_ref[0]**o+(-0.352485315079)*x_ref[0]+(-0.825830724905)*x_ref[1]**o+(0.568756966202)*x_ref[1]
        ref[0,1,2,0]=(-0.262404141899)*x_ref[0]**o+(0.695058140486)*x_ref[0]+(-0.562413865895)*x_ref[1]**o+(0.744853779046)*x_ref[1]
        ref[0,1,2,1]=(-0.526480240707)*x_ref[0]**o+(-0.246633022689)*x_ref[0]+(-0.677508152973)*x_ref[1]**o+(0.263085459331)*x_ref[1]
        ref[0,2,0,0]=(-0.440455373866)*x_ref[0]**o+(0.982673077522)*x_ref[0]+(0.267643265744)*x_ref[1]**o+(-0.248708912008)*x_ref[1]
        ref[0,2,0,1]=(-0.629497400912)*x_ref[0]**o+(-0.222971392637)*x_ref[0]+(0.476895454043)*x_ref[1]**o+(-0.560588300663)*x_ref[1]
        ref[0,2,1,0]=(-0.142675958641)*x_ref[0]**o+(0.858528587711)*x_ref[0]+(-0.848579730169)*x_ref[1]**o+(-0.0813066455284)*x_ref[1]
        ref[0,2,1,1]=(0.309891337726)*x_ref[0]**o+(-0.966050939539)*x_ref[0]+(-0.214997020692)*x_ref[1]**o+(0.823855020343)*x_ref[1]
        ref[0,2,2,0]=(0.61116679522)*x_ref[0]**o+(0.663744432393)*x_ref[0]+(0.414661416033)*x_ref[1]**o+(-0.996447270631)*x_ref[1]
        ref[0,2,2,1]=(-0.634958711796)*x_ref[0]**o+(0.27654613916)*x_ref[0]+(0.157322235818)*x_ref[1]**o+(-0.449130441789)*x_ref[1]
        ref[0,3,0,0]=(-0.949513930962)*x_ref[0]**o+(0.43714393009)*x_ref[0]+(0.44401680011)*x_ref[1]**o+(0.775550283269)*x_ref[1]
        ref[0,3,0,1]=(-0.560623279771)*x_ref[0]**o+(-0.261230828373)*x_ref[0]+(0.809046529697)*x_ref[1]**o+(-0.00091638239609)*x_ref[1]
        ref[0,3,1,0]=(-0.897951425228)*x_ref[0]**o+(-0.854681659889)*x_ref[0]+(-0.600467143561)*x_ref[1]**o+(-0.543812193981)*x_ref[1]
        ref[0,3,1,1]=(-0.252640196006)*x_ref[0]**o+(-0.0147679084824)*x_ref[0]+(0.474400135578)*x_ref[1]**o+(-0.649969341279)*x_ref[1]
        ref[0,3,2,0]=(0.148265101594)*x_ref[0]**o+(0.865387108497)*x_ref[0]+(-0.853468586257)*x_ref[1]**o+(-0.728220923395)*x_ref[1]
        ref[0,3,2,1]=(-0.998415894528)*x_ref[0]**o+(0.385240420153)*x_ref[0]+(-0.970588182269)*x_ref[1]**o+(-0.715015353507)*x_ref[1]
        ref[0,4,0,0]=(0.47018041111)*x_ref[0]**o+(-0.709194247739)*x_ref[0]+(-0.219962382546)*x_ref[1]**o+(0.354322367748)*x_ref[1]
        ref[0,4,0,1]=(-0.322814742992)*x_ref[0]**o+(0.549832283705)*x_ref[0]+(0.174383028533)*x_ref[1]**o+(0.582821277257)*x_ref[1]
        ref[0,4,1,0]=(0.263785420773)*x_ref[0]**o+(-0.072514835468)*x_ref[0]+(-0.857389703063)*x_ref[1]**o+(-0.775322452374)*x_ref[1]
        ref[0,4,1,1]=(0.938271810888)*x_ref[0]**o+(-0.823646250497)*x_ref[0]+(-0.685525888071)*x_ref[1]**o+(-0.603286444077)*x_ref[1]
        ref[0,4,2,0]=(-0.609415096341)*x_ref[0]**o+(-0.720923292264)*x_ref[0]+(-0.965452230391)*x_ref[1]**o+(0.285193163647)*x_ref[1]
        ref[0,4,2,1]=(0.421976649551)*x_ref[0]**o+(0.990010463736)*x_ref[0]+(-0.860923000529)*x_ref[1]**o+(0.726093525288)*x_ref[1]
        ref[1,0,0,0]=(-0.996811081802)*x_ref[0]**o+(-0.97863301939)*x_ref[0]+(0.202626167636)*x_ref[1]**o+(0.0905062876049)*x_ref[1]
        ref[1,0,0,1]=(-0.364721401486)*x_ref[0]**o+(0.935308652161)*x_ref[0]+(-0.405836642917)*x_ref[1]**o+(0.550390830913)*x_ref[1]
        ref[1,0,1,0]=(-0.701902045301)*x_ref[0]**o+(0.531854328517)*x_ref[0]+(-0.732386759539)*x_ref[1]**o+(0.425326883394)*x_ref[1]
        ref[1,0,1,1]=(0.447994101591)*x_ref[0]**o+(-0.813809278167)*x_ref[0]+(-0.539345843485)*x_ref[1]**o+(-0.0354180208603)*x_ref[1]
        ref[1,0,2,0]=(-0.603871847208)*x_ref[0]**o+(0.842448438568)*x_ref[0]+(-0.7438939901)*x_ref[1]**o+(-0.587219172421)*x_ref[1]
        ref[1,0,2,1]=(-0.971267843684)*x_ref[0]**o+(0.259537498927)*x_ref[0]+(-0.698510853299)*x_ref[1]**o+(0.722962896543)*x_ref[1]
        ref[1,1,0,0]=(0.594944086583)*x_ref[0]**o+(0.296356236144)*x_ref[0]+(0.0970882705472)*x_ref[1]**o+(0.849524363397)*x_ref[1]
        ref[1,1,0,1]=(0.837454132615)*x_ref[0]**o+(0.0858001627187)*x_ref[0]+(0.019269936186)*x_ref[1]**o+(-0.86311221183)*x_ref[1]
        ref[1,1,1,0]=(0.947339105706)*x_ref[0]**o+(0.424118233136)*x_ref[0]+(-0.314979294332)*x_ref[1]**o+(0.507664692875)*x_ref[1]
        ref[1,1,1,1]=(0.805027334702)*x_ref[0]**o+(0.682733602548)*x_ref[0]+(-0.162752989543)*x_ref[1]**o+(-0.438803102705)*x_ref[1]
        ref[1,1,2,0]=(0.24090406784)*x_ref[0]**o+(-0.564828151023)*x_ref[0]+(-0.446384646081)*x_ref[1]**o+(0.06921350334)*x_ref[1]
        ref[1,1,2,1]=(0.535823223755)*x_ref[0]**o+(0.0248971256814)*x_ref[0]+(-0.508444148354)*x_ref[1]**o+(0.840622721627)*x_ref[1]
        ref[1,2,0,0]=(0.0525491067608)*x_ref[0]**o+(-0.384851762445)*x_ref[0]+(0.305218692316)*x_ref[1]**o+(-0.319267902416)*x_ref[1]
        ref[1,2,0,1]=(0.514545608836)*x_ref[0]**o+(0.157544217219)*x_ref[0]+(0.0378829715039)*x_ref[1]**o+(0.778857744661)*x_ref[1]
        ref[1,2,1,0]=(0.702907133813)*x_ref[0]**o+(-0.773975432754)*x_ref[0]+(0.489924281654)*x_ref[1]**o+(-0.771122137868)*x_ref[1]
        ref[1,2,1,1]=(0.305517959872)*x_ref[0]**o+(-0.229609729031)*x_ref[0]+(-0.999272871312)*x_ref[1]**o+(0.283774230586)*x_ref[1]
        ref[1,2,2,0]=(0.943767954042)*x_ref[0]**o+(0.214819458913)*x_ref[0]+(0.883796685301)*x_ref[1]**o+(0.88624077323)*x_ref[1]
        ref[1,2,2,1]=(0.904685466034)*x_ref[0]**o+(0.238007243352)*x_ref[0]+(-0.496694568656)*x_ref[1]**o+(0.90503921706)*x_ref[1]
        ref[1,3,0,0]=(-0.323719826916)*x_ref[0]**o+(-0.679887105617)*x_ref[0]+(-0.204658991412)*x_ref[1]**o+(-0.782808301573)*x_ref[1]
        ref[1,3,0,1]=(-0.915796887204)*x_ref[0]**o+(-0.210816609805)*x_ref[0]+(-0.229804577299)*x_ref[1]**o+(0.224669921335)*x_ref[1]
        ref[1,3,1,0]=(0.0502687121641)*x_ref[0]**o+(-0.873227856206)*x_ref[0]+(-0.159031793692)*x_ref[1]**o+(-0.891685753753)*x_ref[1]
        ref[1,3,1,1]=(0.492846348595)*x_ref[0]**o+(0.787085648664)*x_ref[0]+(0.65805763747)*x_ref[1]**o+(0.79345020859)*x_ref[1]
        ref[1,3,2,0]=(0.186889967831)*x_ref[0]**o+(-0.759385582402)*x_ref[0]+(0.526210288461)*x_ref[1]**o+(0.408440324329)*x_ref[1]
        ref[1,3,2,1]=(0.49057760628)*x_ref[0]**o+(0.583558905672)*x_ref[0]+(0.200515771351)*x_ref[1]**o+(-0.0457600002227)*x_ref[1]
        ref[1,4,0,0]=(0.986278922497)*x_ref[0]**o+(-0.724078510554)*x_ref[0]+(0.918959062512)*x_ref[1]**o+(-0.696724335474)*x_ref[1]
        ref[1,4,0,1]=(-0.624189125377)*x_ref[0]**o+(0.849396531428)*x_ref[0]+(-0.793610811078)*x_ref[1]**o+(0.473468667211)*x_ref[1]
        ref[1,4,1,0]=(0.850059248076)*x_ref[0]**o+(-0.167497586421)*x_ref[0]+(-0.401914828635)*x_ref[1]**o+(-0.446166310885)*x_ref[1]
        ref[1,4,1,1]=(0.735056540402)*x_ref[0]**o+(0.956860138776)*x_ref[0]+(-0.657753509921)*x_ref[1]**o+(0.104941138398)*x_ref[1]
        ref[1,4,2,0]=(0.178383558524)*x_ref[0]**o+(-0.695410327712)*x_ref[0]+(-0.896595978256)*x_ref[1]**o+(0.500721122887)*x_ref[1]
        ref[1,4,2,1]=(0.00266985547612)*x_ref[0]**o+(0.569844332841)*x_ref[0]+(0.30469482092)*x_ref[1]**o+(0.18820848487)*x_ref[1]
        ref[2,0,0,0]=(0.0726883565313)*x_ref[0]**o+(-0.151895019503)*x_ref[0]+(-0.566412947745)*x_ref[1]**o+(0.605833954559)*x_ref[1]
        ref[2,0,0,1]=(-0.759348816055)*x_ref[0]**o+(0.203730579434)*x_ref[0]+(-0.306267994)*x_ref[1]**o+(-0.121655396311)*x_ref[1]
        ref[2,0,1,0]=(-0.529988368349)*x_ref[0]**o+(0.265126908621)*x_ref[0]+(-0.127232300402)*x_ref[1]**o+(-0.711018521148)*x_ref[1]
        ref[2,0,1,1]=(0.367616519674)*x_ref[0]**o+(-0.856828909791)*x_ref[0]+(-0.988581037916)*x_ref[1]**o+(0.236273236567)*x_ref[1]
        ref[2,0,2,0]=(0.980896603038)*x_ref[0]**o+(0.0101391252514)*x_ref[0]+(-0.734647631536)*x_ref[1]**o+(-0.298628688073)*x_ref[1]
        ref[2,0,2,1]=(-0.876812266906)*x_ref[0]**o+(-0.0545067479612)*x_ref[0]+(-0.752772584824)*x_ref[1]**o+(-0.797225970025)*x_ref[1]
        ref[2,1,0,0]=(0.9776767796)*x_ref[0]**o+(-0.381948430876)*x_ref[0]+(0.705224317124)*x_ref[1]**o+(0.431609349218)*x_ref[1]
        ref[2,1,0,1]=(0.0282522834733)*x_ref[0]**o+(-0.519316841837)*x_ref[0]+(-0.898831067576)*x_ref[1]**o+(0.491570698382)*x_ref[1]
        ref[2,1,1,0]=(-0.874077440763)*x_ref[0]**o+(0.0050283385939)*x_ref[0]+(-0.45400999477)*x_ref[1]**o+(-0.823942392997)*x_ref[1]
        ref[2,1,1,1]=(0.705592988895)*x_ref[0]**o+(0.545242748068)*x_ref[0]+(-0.0240067342318)*x_ref[1]**o+(-0.764691278707)*x_ref[1]
        ref[2,1,2,0]=(0.214087039013)*x_ref[0]**o+(0.198346865884)*x_ref[0]+(0.195275868007)*x_ref[1]**o+(-0.615146376705)*x_ref[1]
        ref[2,1,2,1]=(0.819578786211)*x_ref[0]**o+(0.238162773292)*x_ref[0]+(-0.488387775705)*x_ref[1]**o+(0.143425392477)*x_ref[1]
        ref[2,2,0,0]=(0.41469049634)*x_ref[0]**o+(-0.870291745289)*x_ref[0]+(-0.978682320238)*x_ref[1]**o+(0.315825060763)*x_ref[1]
        ref[2,2,0,1]=(-0.725231531597)*x_ref[0]**o+(-0.583383717934)*x_ref[0]+(0.677865725085)*x_ref[1]**o+(0.590055042471)*x_ref[1]
        ref[2,2,1,0]=(-0.827316408906)*x_ref[0]**o+(0.152995858286)*x_ref[0]+(-0.864980658862)*x_ref[1]**o+(-0.267986836659)*x_ref[1]
        ref[2,2,1,1]=(-0.333272933954)*x_ref[0]**o+(0.339541494142)*x_ref[0]+(0.982274574789)*x_ref[1]**o+(0.488564854285)*x_ref[1]
        ref[2,2,2,0]=(0.433303448225)*x_ref[0]**o+(0.325824460514)*x_ref[0]+(-0.418818415293)*x_ref[1]**o+(0.884475442413)*x_ref[1]
        ref[2,2,2,1]=(-0.692651589533)*x_ref[0]**o+(0.85015849795)*x_ref[0]+(-0.683352003673)*x_ref[1]**o+(0.458991254181)*x_ref[1]
        ref[2,3,0,0]=(-0.754234631384)*x_ref[0]**o+(0.240243331561)*x_ref[0]+(0.545360646104)*x_ref[1]**o+(-0.960857789697)*x_ref[1]
        ref[2,3,0,1]=(0.911046185694)*x_ref[0]**o+(-0.909178154095)*x_ref[0]+(-0.330132665955)*x_ref[1]**o+(-0.978660867939)*x_ref[1]
        ref[2,3,1,0]=(0.88161349743)*x_ref[0]**o+(0.0812270265162)*x_ref[0]+(-0.584074638899)*x_ref[1]**o+(-0.309263951959)*x_ref[1]
        ref[2,3,1,1]=(-0.59736682275)*x_ref[0]**o+(-0.176681221389)*x_ref[0]+(0.750820855296)*x_ref[1]**o+(0.977121067295)*x_ref[1]
        ref[2,3,2,0]=(-0.926460632737)*x_ref[0]**o+(-0.465577825838)*x_ref[0]+(0.577728110369)*x_ref[1]**o+(-0.973296874179)*x_ref[1]
        ref[2,3,2,1]=(-0.959292420516)*x_ref[0]**o+(-0.173671029796)*x_ref[0]+(0.644855220567)*x_ref[1]**o+(-0.659401238679)*x_ref[1]
        ref[2,4,0,0]=(-0.185887734162)*x_ref[0]**o+(-0.184924703674)*x_ref[0]+(0.12829518222)*x_ref[1]**o+(-0.95529948598)*x_ref[1]
        ref[2,4,0,1]=(-0.222082370982)*x_ref[0]**o+(0.480067405868)*x_ref[0]+(0.737491570657)*x_ref[1]**o+(-0.0505388125547)*x_ref[1]
        ref[2,4,1,0]=(-0.759488182696)*x_ref[0]**o+(-0.209420047186)*x_ref[0]+(-0.257523848658)*x_ref[1]**o+(-0.88701477207)*x_ref[1]
        ref[2,4,1,1]=(0.549919898662)*x_ref[0]**o+(-0.204942654253)*x_ref[0]+(0.237781682077)*x_ref[1]**o+(-0.398029223748)*x_ref[1]
        ref[2,4,2,0]=(-0.429624077152)*x_ref[0]**o+(0.477737955149)*x_ref[0]+(0.150704189177)*x_ref[1]**o+(-0.651247641141)*x_ref[1]
        ref[2,4,2,1]=(-0.894705558322)*x_ref[0]**o+(-0.437755159653)*x_ref[0]+(-0.984539083926)*x_ref[1]**o+(0.771869802974)*x_ref[1]
        ref[3,0,0,0]=(0.613531927806)*x_ref[0]**o+(-0.262670681917)*x_ref[0]+(0.272118391563)*x_ref[1]**o+(-0.911450635138)*x_ref[1]
        ref[3,0,0,1]=(0.0749962421787)*x_ref[0]**o+(-0.280125180177)*x_ref[0]+(0.535812839231)*x_ref[1]**o+(-0.898511642603)*x_ref[1]
        ref[3,0,1,0]=(-0.493682703273)*x_ref[0]**o+(0.564806921005)*x_ref[0]+(-0.424211826783)*x_ref[1]**o+(0.955398127065)*x_ref[1]
        ref[3,0,1,1]=(-0.208922295917)*x_ref[0]**o+(0.552971608191)*x_ref[0]+(-0.482834655473)*x_ref[1]**o+(0.916990053757)*x_ref[1]
        ref[3,0,2,0]=(0.225448292089)*x_ref[0]**o+(-0.508992793538)*x_ref[0]+(-0.301030316256)*x_ref[1]**o+(0.234012186162)*x_ref[1]
        ref[3,0,2,1]=(0.173694397805)*x_ref[0]**o+(-0.0436667050758)*x_ref[0]+(-0.391980496474)*x_ref[1]**o+(0.183288325597)*x_ref[1]
        ref[3,1,0,0]=(-0.078216910574)*x_ref[0]**o+(-0.993944069614)*x_ref[0]+(-0.250488879366)*x_ref[1]**o+(0.648115982978)*x_ref[1]
        ref[3,1,0,1]=(-0.346993095448)*x_ref[0]**o+(0.630491595383)*x_ref[0]+(0.128199747925)*x_ref[1]**o+(-0.681982197159)*x_ref[1]
        ref[3,1,1,0]=(0.0360684354752)*x_ref[0]**o+(-0.8297568105)*x_ref[0]+(0.122927048201)*x_ref[1]**o+(0.0448718398101)*x_ref[1]
        ref[3,1,1,1]=(-0.603474854473)*x_ref[0]**o+(-0.435029027744)*x_ref[0]+(0.632649021375)*x_ref[1]**o+(0.414362202135)*x_ref[1]
        ref[3,1,2,0]=(-0.912833626509)*x_ref[0]**o+(-0.828668448463)*x_ref[0]+(-0.754484607948)*x_ref[1]**o+(0.503101904128)*x_ref[1]
        ref[3,1,2,1]=(0.00649229353309)*x_ref[0]**o+(-0.96124954377)*x_ref[0]+(0.648294131558)*x_ref[1]**o+(-0.957122691907)*x_ref[1]
        ref[3,2,0,0]=(0.476913272421)*x_ref[0]**o+(0.0878038200542)*x_ref[0]+(-0.516953661957)*x_ref[1]**o+(0.0134540618391)*x_ref[1]
        ref[3,2,0,1]=(0.3748519133)*x_ref[0]**o+(-0.970356842975)*x_ref[0]+(-0.874984768114)*x_ref[1]**o+(-0.0950957723387)*x_ref[1]
        ref[3,2,1,0]=(-0.711688301522)*x_ref[0]**o+(-0.20444931953)*x_ref[0]+(0.601925297065)*x_ref[1]**o+(0.939713142428)*x_ref[1]
        ref[3,2,1,1]=(-0.869179475562)*x_ref[0]**o+(-0.771823270124)*x_ref[0]+(0.0447051413494)*x_ref[1]**o+(0.769181827448)*x_ref[1]
        ref[3,2,2,0]=(0.341502469812)*x_ref[0]**o+(-0.892625503786)*x_ref[0]+(-0.683764736135)*x_ref[1]**o+(-0.208342034139)*x_ref[1]
        ref[3,2,2,1]=(-0.876208883831)*x_ref[0]**o+(-0.705748793184)*x_ref[0]+(0.92260186528)*x_ref[1]**o+(-0.437547481021)*x_ref[1]
        ref[3,3,0,0]=(-0.387117059014)*x_ref[0]**o+(-0.30316333118)*x_ref[0]+(-0.107906971687)*x_ref[1]**o+(0.27027380043)*x_ref[1]
        ref[3,3,0,1]=(0.760401124102)*x_ref[0]**o+(0.0392442137961)*x_ref[0]+(0.527403460784)*x_ref[1]**o+(0.49506478844)*x_ref[1]
        ref[3,3,1,0]=(-0.797802219992)*x_ref[0]**o+(-0.718681383351)*x_ref[0]+(0.723820891624)*x_ref[1]**o+(0.851058726763)*x_ref[1]
        ref[3,3,1,1]=(-0.477124467307)*x_ref[0]**o+(-0.912639051937)*x_ref[0]+(-0.653399611549)*x_ref[1]**o+(-0.527816963598)*x_ref[1]
        ref[3,3,2,0]=(-0.375620015065)*x_ref[0]**o+(0.713563556042)*x_ref[0]+(0.592214060154)*x_ref[1]**o+(-0.663713190063)*x_ref[1]
        ref[3,3,2,1]=(0.13839650904)*x_ref[0]**o+(-0.894351932455)*x_ref[0]+(0.130542033693)*x_ref[1]**o+(-0.420895602544)*x_ref[1]
        ref[3,4,0,0]=(-0.438194955853)*x_ref[0]**o+(-0.119296803116)*x_ref[0]+(0.683500326777)*x_ref[1]**o+(-0.226368973557)*x_ref[1]
        ref[3,4,0,1]=(-0.232920353049)*x_ref[0]**o+(-0.35856926089)*x_ref[0]+(0.882231830184)*x_ref[1]**o+(-0.000957747821531)*x_ref[1]
        ref[3,4,1,0]=(-0.0939581521731)*x_ref[0]**o+(0.831315092139)*x_ref[0]+(0.042454046079)*x_ref[1]**o+(0.791725586274)*x_ref[1]
        ref[3,4,1,1]=(-0.328628704689)*x_ref[0]**o+(0.434271457185)*x_ref[0]+(0.632929764639)*x_ref[1]**o+(-0.526043410013)*x_ref[1]
        ref[3,4,2,0]=(0.429887737881)*x_ref[0]**o+(0.828275504726)*x_ref[0]+(-0.199204123672)*x_ref[1]**o+(-0.363782496284)*x_ref[1]
        ref[3,4,2,1]=(-0.446866424171)*x_ref[0]**o+(-0.204025885117)*x_ref[0]+(-0.899560130794)*x_ref[1]**o+(0.184518622033)*x_ref[1]
      else:
        s[0,0,0,0]=(0.651971717604)*x[0]**o+(-0.781641304094)*x[0]+(0.535792272807)*x[1]**o+(-0.661083912872)*x[1]+(-0.311952439342)*x[2]**o+(-0.952641714536)*x[2]
        s[0,0,0,1]=(-0.480185392802)*x[0]**o+(-0.957422590142)*x[0]+(-0.25301116344)*x[1]**o+(0.865442078084)*x[1]+(0.836322905432)*x[2]**o+(-0.835302193996)*x[2]
        s[0,0,1,0]=(0.163795654451)*x[0]**o+(0.268353643157)*x[0]+(0.298652899832)*x[1]**o+(0.567324820252)*x[1]+(-0.265398363574)*x[2]**o+(0.204860398351)*x[2]
        s[0,0,1,1]=(0.538423766786)*x[0]**o+(-0.01940169351)*x[0]+(0.294103797101)*x[1]**o+(0.132862986285)*x[1]+(0.995163905667)*x[2]**o+(-0.119153668927)*x[2]
        s[0,0,2,0]=(-0.892916012976)*x[0]**o+(-0.557129189047)*x[0]+(0.0705777480285)*x[1]**o+(0.518661097094)*x[1]+(-0.454082645299)*x[2]**o+(0.632837820367)*x[2]
        s[0,0,2,1]=(0.71834426356)*x[0]**o+(-0.688104502488)*x[0]+(-0.285809102801)*x[1]**o+(0.552799075425)*x[1]+(0.116702254748)*x[2]**o+(-0.34761826549)*x[2]
        s[0,1,0,0]=(0.293581495968)*x[0]**o+(0.201727375498)*x[0]+(-0.416610275343)*x[1]**o+(0.607460451032)*x[1]+(0.0343669057747)*x[2]**o+(-0.530601755969)*x[2]
        s[0,1,0,1]=(0.880681535309)*x[0]**o+(-0.953246533735)*x[0]+(-0.424031774258)*x[1]**o+(-0.52019867795)*x[1]+(-0.250435071705)*x[2]**o+(-0.168661740641)*x[2]
        s[0,1,1,0]=(-0.495577302252)*x[0]**o+(0.792099185033)*x[0]+(0.294589995512)*x[1]**o+(-0.734902995766)*x[1]+(-0.359137354522)*x[2]**o+(-0.0317334336381)*x[2]
        s[0,1,1,1]=(-0.843957318456)*x[0]**o+(0.29759734816)*x[0]+(-0.954015934149)*x[1]**o+(-0.270649016792)*x[1]+(-0.364089303705)*x[2]**o+(-0.396696196833)*x[2]
        s[0,1,2,0]=(0.411843103656)*x[0]**o+(-0.806857649219)*x[0]+(0.0998406396629)*x[1]**o+(0.426060184907)*x[1]+(0.839569241907)*x[2]**o+(-0.22764839333)*x[2]
        s[0,1,2,1]=(-0.970273259116)*x[0]**o+(-0.983160602162)*x[0]+(0.418163962338)*x[1]**o+(-0.110768668492)*x[1]+(0.328089646862)*x[2]**o+(-0.270172456473)*x[2]
        s[0,2,0,0]=(0.683233964281)*x[0]**o+(0.188111552757)*x[0]+(-0.951071798504)*x[1]**o+(-0.0330921159172)*x[1]+(-0.64270161977)*x[2]**o+(-0.499615377327)*x[2]
        s[0,2,0,1]=(0.36794958012)*x[0]**o+(0.398010040747)*x[0]+(-0.309445952639)*x[1]**o+(-0.322623662412)*x[1]+(-0.439692584231)*x[2]**o+(0.00620886333611)*x[2]
        s[0,2,1,0]=(0.588715821634)*x[0]**o+(-0.163253786484)*x[0]+(-0.619853219098)*x[1]**o+(-0.163020284476)*x[1]+(-0.781500850817)*x[2]**o+(-0.13622778235)*x[2]
        s[0,2,1,1]=(0.610025540474)*x[0]**o+(-0.262832084419)*x[0]+(0.109453975557)*x[1]**o+(-0.0910626806898)*x[1]+(0.720567359491)*x[2]**o+(0.221728266865)*x[2]
        s[0,2,2,0]=(-0.967888175709)*x[0]**o+(0.841198387559)*x[0]+(0.0942188974625)*x[1]**o+(0.056715320399)*x[1]+(0.220387799569)*x[2]**o+(-0.991352339366)*x[2]
        s[0,2,2,1]=(-0.243144875987)*x[0]**o+(-0.101607712773)*x[0]+(0.0890556148157)*x[1]**o+(0.968975809324)*x[1]+(-0.680428099066)*x[2]**o+(-0.203127186894)*x[2]
        s[0,3,0,0]=(-0.97698323555)*x[0]**o+(0.723899953965)*x[0]+(0.168932911211)*x[1]**o+(0.210562969565)*x[1]+(0.0241740170862)*x[2]**o+(-0.443543352786)*x[2]
        s[0,3,0,1]=(0.273706631775)*x[0]**o+(-0.064335272978)*x[0]+(-0.289390463262)*x[1]**o+(-0.0911403229125)*x[1]+(0.380520540499)*x[2]**o+(0.930362632813)*x[2]
        s[0,3,1,0]=(0.108247452775)*x[0]**o+(0.670880216846)*x[0]+(0.743592674018)*x[1]**o+(0.110450181904)*x[1]+(0.633556946694)*x[2]**o+(-0.526137817259)*x[2]
        s[0,3,1,1]=(0.770308192835)*x[0]**o+(0.304435220496)*x[0]+(-0.851453559716)*x[1]**o+(-0.549151404489)*x[1]+(-0.512295072491)*x[2]**o+(-0.896216869911)*x[2]
        s[0,3,2,0]=(-0.879389228597)*x[0]**o+(-0.98421803535)*x[0]+(0.310411565504)*x[1]**o+(-0.833601944827)*x[1]+(0.520140917807)*x[2]**o+(0.409319548582)*x[2]
        s[0,3,2,1]=(0.673800312082)*x[0]**o+(0.307843370867)*x[0]+(0.0435576949618)*x[1]**o+(-0.362422816889)*x[1]+(0.96314375221)*x[2]**o+(-0.896763989426)*x[2]
        s[0,4,0,0]=(-0.569321969384)*x[0]**o+(-0.545868340066)*x[0]+(0.951980474226)*x[1]**o+(-0.482241368075)*x[1]+(-0.255170213259)*x[2]**o+(0.841268809686)*x[2]
        s[0,4,0,1]=(-0.837681312588)*x[0]**o+(-0.458077368372)*x[0]+(0.997003677283)*x[1]**o+(0.762129779023)*x[1]+(0.0952433046108)*x[2]**o+(-0.781885974532)*x[2]
        s[0,4,1,0]=(-0.856181540078)*x[0]**o+(-0.841612660714)*x[0]+(0.544038400285)*x[1]**o+(-0.284909634097)*x[1]+(0.11260106121)*x[2]**o+(-0.39480742262)*x[2]
        s[0,4,1,1]=(0.356082675669)*x[0]**o+(0.917688541228)*x[0]+(-0.176959383957)*x[1]**o+(0.416653474575)*x[1]+(-0.662930023412)*x[2]**o+(-0.73010168013)*x[2]
        s[0,4,2,0]=(-0.763454917764)*x[0]**o+(0.302162768082)*x[0]+(0.143386232133)*x[1]**o+(0.91616734981)*x[1]+(-0.813201390394)*x[2]**o+(-0.857982436099)*x[2]
        s[0,4,2,1]=(0.835769822382)*x[0]**o+(0.167610186558)*x[0]+(0.2331699547)*x[1]**o+(0.0015459879901)*x[1]+(0.167167023037)*x[2]**o+(0.550280560723)*x[2]
        s[1,0,0,0]=(-0.217157755228)*x[0]**o+(0.534373560701)*x[0]+(-0.883064731078)*x[1]**o+(0.731486045177)*x[1]+(0.819268782653)*x[2]**o+(0.787662641797)*x[2]
        s[1,0,0,1]=(-0.487819762644)*x[0]**o+(-0.430212238027)*x[0]+(-0.56911636092)*x[1]**o+(-0.728659093202)*x[1]+(0.455173919727)*x[2]**o+(-0.636506423308)*x[2]
        s[1,0,1,0]=(0.900337000667)*x[0]**o+(0.299528369342)*x[0]+(0.681918775341)*x[1]**o+(0.370100893068)*x[1]+(-0.587580308922)*x[2]**o+(0.485274084829)*x[2]
        s[1,0,1,1]=(0.442748593533)*x[0]**o+(-0.0292501463909)*x[0]+(0.551554968843)*x[1]**o+(-0.318313953949)*x[1]+(-0.458151041817)*x[2]**o+(-0.625611121823)*x[2]
        s[1,0,2,0]=(-0.723203702271)*x[0]**o+(0.101313366226)*x[0]+(-0.548133659891)*x[1]**o+(-0.392729656892)*x[1]+(0.46317338606)*x[2]**o+(0.235572773174)*x[2]
        s[1,0,2,1]=(-0.445472509581)*x[0]**o+(-0.552381494615)*x[0]+(0.0815798929496)*x[1]**o+(-0.595946577234)*x[1]+(0.878280129666)*x[2]**o+(-0.274608443481)*x[2]
        s[1,1,0,0]=(0.543392744962)*x[0]**o+(0.79638586218)*x[0]+(0.435895934157)*x[1]**o+(-0.0134442947039)*x[1]+(-0.23586652076)*x[2]**o+(0.120783361228)*x[2]
        s[1,1,0,1]=(-0.50551104509)*x[0]**o+(-0.578612752272)*x[0]+(-0.595688781895)*x[1]**o+(-0.411446622857)*x[1]+(-0.729076372085)*x[2]**o+(0.61997008872)*x[2]
        s[1,1,1,0]=(-0.25176939418)*x[0]**o+(0.0192820953571)*x[0]+(-0.365827108002)*x[1]**o+(-0.58048375365)*x[1]+(-0.109602718093)*x[2]**o+(-0.764760324931)*x[2]
        s[1,1,1,1]=(0.333747531705)*x[0]**o+(-0.975460281758)*x[0]+(-0.263662359116)*x[1]**o+(0.833048103253)*x[1]+(-0.403378941533)*x[2]**o+(0.99255510422)*x[2]
        s[1,1,2,0]=(-0.643575493982)*x[0]**o+(0.408164900892)*x[0]+(0.330669259534)*x[1]**o+(0.696238788141)*x[1]+(0.8187478709)*x[2]**o+(0.269447281859)*x[2]
        s[1,1,2,1]=(0.0975010793595)*x[0]**o+(-0.266707476957)*x[0]+(-0.627484959323)*x[1]**o+(0.189636667701)*x[1]+(0.0634608353528)*x[2]**o+(0.425595191628)*x[2]
        s[1,2,0,0]=(0.144863702314)*x[0]**o+(0.62877495084)*x[0]+(-0.937663622583)*x[1]**o+(0.643925666447)*x[1]+(0.0752876047081)*x[2]**o+(-0.0278560665987)*x[2]
        s[1,2,0,1]=(0.825937768085)*x[0]**o+(-0.350938240562)*x[0]+(-0.316596763726)*x[1]**o+(0.248951654728)*x[1]+(-0.4533348882)*x[2]**o+(-0.840614727297)*x[2]
        s[1,2,1,0]=(0.235071855147)*x[0]**o+(0.818758983424)*x[0]+(-0.962091319923)*x[1]**o+(-0.0687829219781)*x[1]+(0.256236142265)*x[2]**o+(-0.236383412975)*x[2]
        s[1,2,1,1]=(-0.674935027618)*x[0]**o+(0.506392204682)*x[0]+(-0.891135534099)*x[1]**o+(0.200562756224)*x[1]+(-0.509385776169)*x[2]**o+(-0.709115810434)*x[2]
        s[1,2,2,0]=(0.476890911154)*x[0]**o+(-0.716988939897)*x[0]+(-0.708930094063)*x[1]**o+(-0.221417730773)*x[1]+(-0.0530682866975)*x[2]**o+(-0.933479841665)*x[2]
        s[1,2,2,1]=(0.912189530094)*x[0]**o+(-0.287403816104)*x[0]+(-0.755698083102)*x[1]**o+(-0.996716444257)*x[1]+(-0.836456434626)*x[2]**o+(-0.463710726041)*x[2]
        s[1,3,0,0]=(0.253691639919)*x[0]**o+(-0.279433210688)*x[0]+(-0.117635686336)*x[1]**o+(0.790073676599)*x[1]+(-0.559215280716)*x[2]**o+(-0.103190062272)*x[2]
        s[1,3,0,1]=(0.0970821498373)*x[0]**o+(0.814008398347)*x[0]+(0.583184329882)*x[1]**o+(-0.598484860263)*x[1]+(-0.00216176134509)*x[2]**o+(0.14889662547)*x[2]
        s[1,3,1,0]=(-0.730045080625)*x[0]**o+(-0.746336558552)*x[0]+(-0.995897359987)*x[1]**o+(-0.70582565406)*x[1]+(0.36600954398)*x[2]**o+(-0.0843877537778)*x[2]
        s[1,3,1,1]=(-0.714997081413)*x[0]**o+(-0.138823664507)*x[0]+(0.969108046142)*x[1]**o+(-0.201514397247)*x[1]+(-0.438601765508)*x[2]**o+(-0.984456457681)*x[2]
        s[1,3,2,0]=(-0.577419600949)*x[0]**o+(-0.821514372871)*x[0]+(-0.179395810292)*x[1]**o+(-0.904303143215)*x[1]+(0.27616964799)*x[2]**o+(-0.272088845238)*x[2]
        s[1,3,2,1]=(0.31325269807)*x[0]**o+(-0.0134619122423)*x[0]+(-0.641786875798)*x[1]**o+(-0.568855591797)*x[1]+(0.600302453067)*x[2]**o+(0.830044865545)*x[2]
        s[1,4,0,0]=(-0.791605536671)*x[0]**o+(-0.578889866035)*x[0]+(-0.22659053834)*x[1]**o+(0.837093316131)*x[1]+(0.499953644961)*x[2]**o+(-0.289930435498)*x[2]
        s[1,4,0,1]=(0.819284096253)*x[0]**o+(0.272518944584)*x[0]+(0.199276568134)*x[1]**o+(0.132962435684)*x[1]+(-0.896986377734)*x[2]**o+(0.775604010154)*x[2]
        s[1,4,1,0]=(-0.450392824249)*x[0]**o+(0.376154164409)*x[0]+(0.199853639816)*x[1]**o+(-0.679551176507)*x[1]+(0.565506686048)*x[2]**o+(0.391489667485)*x[2]
        s[1,4,1,1]=(-0.0905057426998)*x[0]**o+(-0.749252305812)*x[0]+(-0.21131989398)*x[1]**o+(-0.671560452908)*x[1]+(-0.760505475283)*x[2]**o+(0.763692313655)*x[2]
        s[1,4,2,0]=(-0.219636883594)*x[0]**o+(-0.380991913771)*x[0]+(0.831879211521)*x[1]**o+(-0.0686682700553)*x[1]+(0.843832009742)*x[2]**o+(-0.86627759217)*x[2]
        s[1,4,2,1]=(-0.332685504588)*x[0]**o+(0.589527397306)*x[0]+(0.624226689395)*x[1]**o+(0.0515479774619)*x[1]+(0.156657102499)*x[2]**o+(0.465889992773)*x[2]
        s[2,0,0,0]=(0.250467292043)*x[0]**o+(-0.0658874324582)*x[0]+(-0.0709052912862)*x[1]**o+(0.700837318576)*x[1]+(0.650887276648)*x[2]**o+(0.816187438816)*x[2]
        s[2,0,0,1]=(-0.697069910038)*x[0]**o+(-0.287284875865)*x[0]+(-0.972646385975)*x[1]**o+(-0.430570334599)*x[1]+(0.400826092127)*x[2]**o+(-0.484530830951)*x[2]
        s[2,0,1,0]=(0.58433575578)*x[0]**o+(-0.114741976876)*x[0]+(-0.205615141565)*x[1]**o+(-0.476488876624)*x[1]+(0.323174445651)*x[2]**o+(-0.152550510154)*x[2]
        s[2,0,1,1]=(0.928306678393)*x[0]**o+(0.104020335724)*x[0]+(-0.0416879184197)*x[1]**o+(0.865779079734)*x[1]+(-0.169672240264)*x[2]**o+(0.118778471217)*x[2]
        s[2,0,2,0]=(0.652779555688)*x[0]**o+(-0.0209488281208)*x[0]+(0.809628464928)*x[1]**o+(-0.85121566767)*x[1]+(-0.845109631338)*x[2]**o+(-0.970252635974)*x[2]
        s[2,0,2,1]=(-0.740845790747)*x[0]**o+(-0.705835368678)*x[0]+(0.26040103782)*x[1]**o+(0.814854175049)*x[1]+(-0.00842496430724)*x[2]**o+(0.192519753618)*x[2]
        s[2,1,0,0]=(0.629347590715)*x[0]**o+(0.922723375982)*x[0]+(-0.430492700295)*x[1]**o+(-0.895795670345)*x[1]+(0.708440378414)*x[2]**o+(-0.155782723618)*x[2]
        s[2,1,0,1]=(-0.478612307362)*x[0]**o+(0.645774615123)*x[0]+(-0.461136871779)*x[1]**o+(0.297927339931)*x[1]+(-0.452011143468)*x[2]**o+(0.267002423863)*x[2]
        s[2,1,1,0]=(-0.0591350600103)*x[0]**o+(-0.252358408576)*x[0]+(-0.0412752075681)*x[1]**o+(0.942736455517)*x[1]+(-0.223363982173)*x[2]**o+(-0.752007969767)*x[2]
        s[2,1,1,1]=(0.197290290824)*x[0]**o+(-0.170028529393)*x[0]+(-0.143011200679)*x[1]**o+(-0.694395961945)*x[1]+(0.430761130021)*x[2]**o+(0.298239563293)*x[2]
        s[2,1,2,0]=(0.258474077793)*x[0]**o+(-0.956041015018)*x[0]+(0.360444161221)*x[1]**o+(-0.969922460117)*x[1]+(0.856255540521)*x[2]**o+(0.108872710157)*x[2]
        s[2,1,2,1]=(-0.694400609963)*x[0]**o+(-0.864737376468)*x[0]+(-0.898839555362)*x[1]**o+(-0.655053934442)*x[1]+(-0.511293723446)*x[2]**o+(-0.204242252526)*x[2]
        s[2,2,0,0]=(-0.427946109181)*x[0]**o+(0.435798407553)*x[0]+(-0.567923694982)*x[1]**o+(0.870896124804)*x[1]+(-0.996434693707)*x[2]**o+(-0.683367112977)*x[2]
        s[2,2,0,1]=(-0.219431769721)*x[0]**o+(-0.302922785317)*x[0]+(-0.0998576564441)*x[1]**o+(0.64263522193)*x[1]+(0.53256637446)*x[2]**o+(-0.00387183821982)*x[2]
        s[2,2,1,0]=(-0.174988349945)*x[0]**o+(0.408056661499)*x[0]+(0.55925945151)*x[1]**o+(-0.647959566858)*x[1]+(0.905755316979)*x[2]**o+(0.691774684766)*x[2]
        s[2,2,1,1]=(-0.882995226725)*x[0]**o+(-0.948874015343)*x[0]+(-0.652506601975)*x[1]**o+(-0.137989382315)*x[1]+(0.665695021939)*x[2]**o+(0.163493199569)*x[2]
        s[2,2,2,0]=(0.666421535029)*x[0]**o+(-0.984765122669)*x[0]+(0.399488949215)*x[1]**o+(-0.99963893587)*x[1]+(0.225552018439)*x[2]**o+(0.430770653325)*x[2]
        s[2,2,2,1]=(-0.0284528199834)*x[0]**o+(0.0421195684748)*x[0]+(-0.399757974604)*x[1]**o+(0.305895209328)*x[1]+(0.190920234352)*x[2]**o+(-0.0990327186702)*x[2]
        s[2,3,0,0]=(-0.565899348734)*x[0]**o+(0.636275927936)*x[0]+(0.682710455722)*x[1]**o+(-0.434837962369)*x[1]+(0.252457745239)*x[2]**o+(-0.709529514057)*x[2]
        s[2,3,0,1]=(0.0113021999515)*x[0]**o+(0.451528226044)*x[0]+(-0.705424064371)*x[1]**o+(0.500516985463)*x[1]+(-0.528750586385)*x[2]**o+(0.998214035471)*x[2]
        s[2,3,1,0]=(-0.879296343578)*x[0]**o+(-0.724012783173)*x[0]+(0.51105902172)*x[1]**o+(-0.840460578985)*x[1]+(0.222130858003)*x[2]**o+(0.998500581881)*x[2]
        s[2,3,1,1]=(0.478703623115)*x[0]**o+(-0.94935681973)*x[0]+(-0.609714878079)*x[1]**o+(0.0318988430394)*x[1]+(-0.858395334121)*x[2]**o+(-0.171229770192)*x[2]
        s[2,3,2,0]=(0.0791777006911)*x[0]**o+(-0.306954070098)*x[0]+(-0.182890473769)*x[1]**o+(-0.26384266322)*x[1]+(0.663652835142)*x[2]**o+(0.188229986922)*x[2]
        s[2,3,2,1]=(-0.101910553498)*x[0]**o+(-0.34187147162)*x[0]+(-0.52746320646)*x[1]**o+(-0.891142643574)*x[1]+(-0.84135671258)*x[2]**o+(0.448615749816)*x[2]
        s[2,4,0,0]=(0.928980482137)*x[0]**o+(0.00486879685273)*x[0]+(-0.068529973495)*x[1]**o+(-0.16116665925)*x[1]+(0.247146438634)*x[2]**o+(-0.341715145751)*x[2]
        s[2,4,0,1]=(0.486412050624)*x[0]**o+(-0.929254100014)*x[0]+(0.439267744247)*x[1]**o+(-0.430515713902)*x[1]+(-0.317438643233)*x[2]**o+(-0.0843825737593)*x[2]
        s[2,4,1,0]=(0.0130446306687)*x[0]**o+(0.324310412551)*x[0]+(-0.38112172561)*x[1]**o+(-0.92878637625)*x[1]+(0.32867951654)*x[2]**o+(-0.0315638177916)*x[2]
        s[2,4,1,1]=(-0.254225591431)*x[0]**o+(0.321872838348)*x[0]+(0.179619901508)*x[1]**o+(-0.858713537407)*x[1]+(0.151254493236)*x[2]**o+(-0.340368823543)*x[2]
        s[2,4,2,0]=(-0.0844677926666)*x[0]**o+(0.398017491087)*x[0]+(0.0313455976792)*x[1]**o+(-0.872666391632)*x[1]+(0.301765055045)*x[2]**o+(0.563923225852)*x[2]
        s[2,4,2,1]=(0.814244097177)*x[0]**o+(-0.223957875084)*x[0]+(0.447995085387)*x[1]**o+(0.350953464712)*x[1]+(0.333269027952)*x[2]**o+(-0.941385895584)*x[2]
        s[3,0,0,0]=(0.0789492878053)*x[0]**o+(0.525756966035)*x[0]+(0.87125043555)*x[1]**o+(0.881346392374)*x[1]+(-0.855477786784)*x[2]**o+(-0.991712685096)*x[2]
        s[3,0,0,1]=(-0.0359771254585)*x[0]**o+(0.603218005193)*x[0]+(-0.580729043759)*x[1]**o+(-0.20390195049)*x[1]+(-0.862263281233)*x[2]**o+(-0.196587155144)*x[2]
        s[3,0,1,0]=(0.879750375964)*x[0]**o+(-0.251177194936)*x[0]+(0.191264882539)*x[1]**o+(-0.95611617781)*x[1]+(0.71738769264)*x[2]**o+(-0.691767368606)*x[2]
        s[3,0,1,1]=(-0.761557468157)*x[0]**o+(-0.381655367519)*x[0]+(0.587969140055)*x[1]**o+(0.622246650476)*x[1]+(0.117544678721)*x[2]**o+(-0.576059347687)*x[2]
        s[3,0,2,0]=(-0.587802439618)*x[0]**o+(-0.981432433894)*x[0]+(-0.758496301974)*x[1]**o+(0.656105955022)*x[1]+(-0.689107562745)*x[2]**o+(0.267295196672)*x[2]
        s[3,0,2,1]=(-0.0524224100533)*x[0]**o+(-0.925763321074)*x[0]+(0.218186590694)*x[1]**o+(-0.598934300523)*x[1]+(0.22756773)*x[2]**o+(-0.770210327792)*x[2]
        s[3,1,0,0]=(0.922635777646)*x[0]**o+(-0.278736876358)*x[0]+(0.0631653424614)*x[1]**o+(-0.356448496856)*x[1]+(-0.609671239719)*x[2]**o+(0.189678422301)*x[2]
        s[3,1,0,1]=(-0.509070180487)*x[0]**o+(-0.250043376376)*x[0]+(-0.0697248208177)*x[1]**o+(0.531153766941)*x[1]+(0.240916012233)*x[2]**o+(0.391353341194)*x[2]
        s[3,1,1,0]=(-0.514764390151)*x[0]**o+(-0.347397972881)*x[0]+(0.138729357858)*x[1]**o+(-0.30135683112)*x[1]+(0.541893745985)*x[2]**o+(0.414478801003)*x[2]
        s[3,1,1,1]=(0.866088916369)*x[0]**o+(-0.96729572537)*x[0]+(0.824735563587)*x[1]**o+(-0.479859785014)*x[1]+(-0.175784746993)*x[2]**o+(-0.998407148967)*x[2]
        s[3,1,2,0]=(0.327087092534)*x[0]**o+(0.171000747621)*x[0]+(-0.33910895269)*x[1]**o+(0.349474180167)*x[1]+(0.561935192081)*x[2]**o+(0.358380998288)*x[2]
        s[3,1,2,1]=(0.35646522102)*x[0]**o+(-0.912130691808)*x[0]+(0.796756777424)*x[1]**o+(-0.91884834291)*x[1]+(0.956454722923)*x[2]**o+(0.462859602491)*x[2]
        s[3,2,0,0]=(-0.849637152897)*x[0]**o+(0.353228393186)*x[0]+(-0.725757036845)*x[1]**o+(0.590369089609)*x[1]+(-0.280432657156)*x[2]**o+(0.142413703594)*x[2]
        s[3,2,0,1]=(-0.640766746158)*x[0]**o+(-0.454583129018)*x[0]+(-0.616394565949)*x[1]**o+(0.724166900329)*x[1]+(-0.748682704671)*x[2]**o+(0.934076771628)*x[2]
        s[3,2,1,0]=(-0.824548065846)*x[0]**o+(0.40250159164)*x[0]+(0.763315748823)*x[1]**o+(0.255878553659)*x[1]+(0.872676778336)*x[2]**o+(-0.100430227782)*x[2]
        s[3,2,1,1]=(-0.610248210467)*x[0]**o+(0.160976829271)*x[0]+(-0.384236718764)*x[1]**o+(0.657119635861)*x[1]+(0.161695416084)*x[2]**o+(0.505997006976)*x[2]
        s[3,2,2,0]=(0.220409736038)*x[0]**o+(0.398197530033)*x[0]+(-0.288242008883)*x[1]**o+(0.0832866402504)*x[1]+(0.761030964351)*x[2]**o+(0.751986795183)*x[2]
        s[3,2,2,1]=(-0.396845385157)*x[0]**o+(-0.0792538647037)*x[0]+(-0.327181444898)*x[1]**o+(-0.810682039031)*x[1]+(0.260671755637)*x[2]**o+(-0.242407958043)*x[2]
        s[3,3,0,0]=(-0.663937889328)*x[0]**o+(0.587545393032)*x[0]+(0.210910229094)*x[1]**o+(0.500419329566)*x[1]+(0.202539078075)*x[2]**o+(-0.623007416272)*x[2]
        s[3,3,0,1]=(0.0375980252743)*x[0]**o+(-0.435705054083)*x[0]+(-0.660885021014)*x[1]**o+(-0.0332748037874)*x[1]+(-0.832274220584)*x[2]**o+(0.805002616286)*x[2]
        s[3,3,1,0]=(0.395522385265)*x[0]**o+(0.901865335207)*x[0]+(0.016322521344)*x[1]**o+(-0.532212956041)*x[1]+(0.190751397835)*x[2]**o+(0.234716440942)*x[2]
        s[3,3,1,1]=(0.257090076986)*x[0]**o+(-0.169468215187)*x[0]+(-0.0641337112558)*x[1]**o+(-0.674363367639)*x[1]+(0.13633053895)*x[2]**o+(-0.741438377514)*x[2]
        s[3,3,2,0]=(0.984088274745)*x[0]**o+(-0.669313378065)*x[0]+(0.899763931565)*x[1]**o+(-0.874652895499)*x[1]+(-0.172620100734)*x[2]**o+(-0.635666769897)*x[2]
        s[3,3,2,1]=(0.693112828433)*x[0]**o+(-0.99231490163)*x[0]+(0.525842419539)*x[1]**o+(0.268799565406)*x[1]+(-0.473532039244)*x[2]**o+(0.50555338295)*x[2]
        s[3,4,0,0]=(0.780678215778)*x[0]**o+(0.56287435306)*x[0]+(0.322651378857)*x[1]**o+(0.268655956157)*x[1]+(0.438312596985)*x[2]**o+(0.175512199841)*x[2]
        s[3,4,0,1]=(-0.510399313775)*x[0]**o+(-0.63898491047)*x[0]+(-0.765120717001)*x[1]**o+(0.45570492579)*x[1]+(-0.148930444588)*x[2]**o+(0.458400743585)*x[2]
        s[3,4,1,0]=(-0.629456511378)*x[0]**o+(-0.0956526447002)*x[0]+(0.223599370201)*x[1]**o+(-0.689787947305)*x[1]+(0.834792179813)*x[2]**o+(0.413903455599)*x[2]
        s[3,4,1,1]=(0.89648905371)*x[0]**o+(0.420890001861)*x[0]+(-0.890077307972)*x[1]**o+(0.424785046869)*x[1]+(0.84365123356)*x[2]**o+(0.56417731826)*x[2]
        s[3,4,2,0]=(0.853620031631)*x[0]**o+(-0.719293879422)*x[0]+(-0.811007794139)*x[1]**o+(-0.236043157925)*x[1]+(0.130830104273)*x[2]**o+(-0.554431879475)*x[2]
        s[3,4,2,1]=(-0.555616431129)*x[0]**o+(-0.900929040274)*x[0]+(0.773492786481)*x[1]**o+(-0.445322579241)*x[1]+(0.270832172179)*x[2]**o+(-0.716484089692)*x[2]
        ref[0,0,0,0]=(0.651971717604)*x_ref[0]**o+(-0.781641304094)*x_ref[0]+(0.535792272807)*x_ref[1]**o+(-0.661083912872)*x_ref[1]+(-0.311952439342)*x_ref[2]**o+(-0.952641714536)*x_ref[2]
        ref[0,0,0,1]=(-0.480185392802)*x_ref[0]**o+(-0.957422590142)*x_ref[0]+(-0.25301116344)*x_ref[1]**o+(0.865442078084)*x_ref[1]+(0.836322905432)*x_ref[2]**o+(-0.835302193996)*x_ref[2]
        ref[0,0,1,0]=(0.163795654451)*x_ref[0]**o+(0.268353643157)*x_ref[0]+(0.298652899832)*x_ref[1]**o+(0.567324820252)*x_ref[1]+(-0.265398363574)*x_ref[2]**o+(0.204860398351)*x_ref[2]
        ref[0,0,1,1]=(0.538423766786)*x_ref[0]**o+(-0.01940169351)*x_ref[0]+(0.294103797101)*x_ref[1]**o+(0.132862986285)*x_ref[1]+(0.995163905667)*x_ref[2]**o+(-0.119153668927)*x_ref[2]
        ref[0,0,2,0]=(-0.892916012976)*x_ref[0]**o+(-0.557129189047)*x_ref[0]+(0.0705777480285)*x_ref[1]**o+(0.518661097094)*x_ref[1]+(-0.454082645299)*x_ref[2]**o+(0.632837820367)*x_ref[2]
        ref[0,0,2,1]=(0.71834426356)*x_ref[0]**o+(-0.688104502488)*x_ref[0]+(-0.285809102801)*x_ref[1]**o+(0.552799075425)*x_ref[1]+(0.116702254748)*x_ref[2]**o+(-0.34761826549)*x_ref[2]
        ref[0,1,0,0]=(0.293581495968)*x_ref[0]**o+(0.201727375498)*x_ref[0]+(-0.416610275343)*x_ref[1]**o+(0.607460451032)*x_ref[1]+(0.0343669057747)*x_ref[2]**o+(-0.530601755969)*x_ref[2]
        ref[0,1,0,1]=(0.880681535309)*x_ref[0]**o+(-0.953246533735)*x_ref[0]+(-0.424031774258)*x_ref[1]**o+(-0.52019867795)*x_ref[1]+(-0.250435071705)*x_ref[2]**o+(-0.168661740641)*x_ref[2]
        ref[0,1,1,0]=(-0.495577302252)*x_ref[0]**o+(0.792099185033)*x_ref[0]+(0.294589995512)*x_ref[1]**o+(-0.734902995766)*x_ref[1]+(-0.359137354522)*x_ref[2]**o+(-0.0317334336381)*x_ref[2]
        ref[0,1,1,1]=(-0.843957318456)*x_ref[0]**o+(0.29759734816)*x_ref[0]+(-0.954015934149)*x_ref[1]**o+(-0.270649016792)*x_ref[1]+(-0.364089303705)*x_ref[2]**o+(-0.396696196833)*x_ref[2]
        ref[0,1,2,0]=(0.411843103656)*x_ref[0]**o+(-0.806857649219)*x_ref[0]+(0.0998406396629)*x_ref[1]**o+(0.426060184907)*x_ref[1]+(0.839569241907)*x_ref[2]**o+(-0.22764839333)*x_ref[2]
        ref[0,1,2,1]=(-0.970273259116)*x_ref[0]**o+(-0.983160602162)*x_ref[0]+(0.418163962338)*x_ref[1]**o+(-0.110768668492)*x_ref[1]+(0.328089646862)*x_ref[2]**o+(-0.270172456473)*x_ref[2]
        ref[0,2,0,0]=(0.683233964281)*x_ref[0]**o+(0.188111552757)*x_ref[0]+(-0.951071798504)*x_ref[1]**o+(-0.0330921159172)*x_ref[1]+(-0.64270161977)*x_ref[2]**o+(-0.499615377327)*x_ref[2]
        ref[0,2,0,1]=(0.36794958012)*x_ref[0]**o+(0.398010040747)*x_ref[0]+(-0.309445952639)*x_ref[1]**o+(-0.322623662412)*x_ref[1]+(-0.439692584231)*x_ref[2]**o+(0.00620886333611)*x_ref[2]
        ref[0,2,1,0]=(0.588715821634)*x_ref[0]**o+(-0.163253786484)*x_ref[0]+(-0.619853219098)*x_ref[1]**o+(-0.163020284476)*x_ref[1]+(-0.781500850817)*x_ref[2]**o+(-0.13622778235)*x_ref[2]
        ref[0,2,1,1]=(0.610025540474)*x_ref[0]**o+(-0.262832084419)*x_ref[0]+(0.109453975557)*x_ref[1]**o+(-0.0910626806898)*x_ref[1]+(0.720567359491)*x_ref[2]**o+(0.221728266865)*x_ref[2]
        ref[0,2,2,0]=(-0.967888175709)*x_ref[0]**o+(0.841198387559)*x_ref[0]+(0.0942188974625)*x_ref[1]**o+(0.056715320399)*x_ref[1]+(0.220387799569)*x_ref[2]**o+(-0.991352339366)*x_ref[2]
        ref[0,2,2,1]=(-0.243144875987)*x_ref[0]**o+(-0.101607712773)*x_ref[0]+(0.0890556148157)*x_ref[1]**o+(0.968975809324)*x_ref[1]+(-0.680428099066)*x_ref[2]**o+(-0.203127186894)*x_ref[2]
        ref[0,3,0,0]=(-0.97698323555)*x_ref[0]**o+(0.723899953965)*x_ref[0]+(0.168932911211)*x_ref[1]**o+(0.210562969565)*x_ref[1]+(0.0241740170862)*x_ref[2]**o+(-0.443543352786)*x_ref[2]
        ref[0,3,0,1]=(0.273706631775)*x_ref[0]**o+(-0.064335272978)*x_ref[0]+(-0.289390463262)*x_ref[1]**o+(-0.0911403229125)*x_ref[1]+(0.380520540499)*x_ref[2]**o+(0.930362632813)*x_ref[2]
        ref[0,3,1,0]=(0.108247452775)*x_ref[0]**o+(0.670880216846)*x_ref[0]+(0.743592674018)*x_ref[1]**o+(0.110450181904)*x_ref[1]+(0.633556946694)*x_ref[2]**o+(-0.526137817259)*x_ref[2]
        ref[0,3,1,1]=(0.770308192835)*x_ref[0]**o+(0.304435220496)*x_ref[0]+(-0.851453559716)*x_ref[1]**o+(-0.549151404489)*x_ref[1]+(-0.512295072491)*x_ref[2]**o+(-0.896216869911)*x_ref[2]
        ref[0,3,2,0]=(-0.879389228597)*x_ref[0]**o+(-0.98421803535)*x_ref[0]+(0.310411565504)*x_ref[1]**o+(-0.833601944827)*x_ref[1]+(0.520140917807)*x_ref[2]**o+(0.409319548582)*x_ref[2]
        ref[0,3,2,1]=(0.673800312082)*x_ref[0]**o+(0.307843370867)*x_ref[0]+(0.0435576949618)*x_ref[1]**o+(-0.362422816889)*x_ref[1]+(0.96314375221)*x_ref[2]**o+(-0.896763989426)*x_ref[2]
        ref[0,4,0,0]=(-0.569321969384)*x_ref[0]**o+(-0.545868340066)*x_ref[0]+(0.951980474226)*x_ref[1]**o+(-0.482241368075)*x_ref[1]+(-0.255170213259)*x_ref[2]**o+(0.841268809686)*x_ref[2]
        ref[0,4,0,1]=(-0.837681312588)*x_ref[0]**o+(-0.458077368372)*x_ref[0]+(0.997003677283)*x_ref[1]**o+(0.762129779023)*x_ref[1]+(0.0952433046108)*x_ref[2]**o+(-0.781885974532)*x_ref[2]
        ref[0,4,1,0]=(-0.856181540078)*x_ref[0]**o+(-0.841612660714)*x_ref[0]+(0.544038400285)*x_ref[1]**o+(-0.284909634097)*x_ref[1]+(0.11260106121)*x_ref[2]**o+(-0.39480742262)*x_ref[2]
        ref[0,4,1,1]=(0.356082675669)*x_ref[0]**o+(0.917688541228)*x_ref[0]+(-0.176959383957)*x_ref[1]**o+(0.416653474575)*x_ref[1]+(-0.662930023412)*x_ref[2]**o+(-0.73010168013)*x_ref[2]
        ref[0,4,2,0]=(-0.763454917764)*x_ref[0]**o+(0.302162768082)*x_ref[0]+(0.143386232133)*x_ref[1]**o+(0.91616734981)*x_ref[1]+(-0.813201390394)*x_ref[2]**o+(-0.857982436099)*x_ref[2]
        ref[0,4,2,1]=(0.835769822382)*x_ref[0]**o+(0.167610186558)*x_ref[0]+(0.2331699547)*x_ref[1]**o+(0.0015459879901)*x_ref[1]+(0.167167023037)*x_ref[2]**o+(0.550280560723)*x_ref[2]
        ref[1,0,0,0]=(-0.217157755228)*x_ref[0]**o+(0.534373560701)*x_ref[0]+(-0.883064731078)*x_ref[1]**o+(0.731486045177)*x_ref[1]+(0.819268782653)*x_ref[2]**o+(0.787662641797)*x_ref[2]
        ref[1,0,0,1]=(-0.487819762644)*x_ref[0]**o+(-0.430212238027)*x_ref[0]+(-0.56911636092)*x_ref[1]**o+(-0.728659093202)*x_ref[1]+(0.455173919727)*x_ref[2]**o+(-0.636506423308)*x_ref[2]
        ref[1,0,1,0]=(0.900337000667)*x_ref[0]**o+(0.299528369342)*x_ref[0]+(0.681918775341)*x_ref[1]**o+(0.370100893068)*x_ref[1]+(-0.587580308922)*x_ref[2]**o+(0.485274084829)*x_ref[2]
        ref[1,0,1,1]=(0.442748593533)*x_ref[0]**o+(-0.0292501463909)*x_ref[0]+(0.551554968843)*x_ref[1]**o+(-0.318313953949)*x_ref[1]+(-0.458151041817)*x_ref[2]**o+(-0.625611121823)*x_ref[2]
        ref[1,0,2,0]=(-0.723203702271)*x_ref[0]**o+(0.101313366226)*x_ref[0]+(-0.548133659891)*x_ref[1]**o+(-0.392729656892)*x_ref[1]+(0.46317338606)*x_ref[2]**o+(0.235572773174)*x_ref[2]
        ref[1,0,2,1]=(-0.445472509581)*x_ref[0]**o+(-0.552381494615)*x_ref[0]+(0.0815798929496)*x_ref[1]**o+(-0.595946577234)*x_ref[1]+(0.878280129666)*x_ref[2]**o+(-0.274608443481)*x_ref[2]
        ref[1,1,0,0]=(0.543392744962)*x_ref[0]**o+(0.79638586218)*x_ref[0]+(0.435895934157)*x_ref[1]**o+(-0.0134442947039)*x_ref[1]+(-0.23586652076)*x_ref[2]**o+(0.120783361228)*x_ref[2]
        ref[1,1,0,1]=(-0.50551104509)*x_ref[0]**o+(-0.578612752272)*x_ref[0]+(-0.595688781895)*x_ref[1]**o+(-0.411446622857)*x_ref[1]+(-0.729076372085)*x_ref[2]**o+(0.61997008872)*x_ref[2]
        ref[1,1,1,0]=(-0.25176939418)*x_ref[0]**o+(0.0192820953571)*x_ref[0]+(-0.365827108002)*x_ref[1]**o+(-0.58048375365)*x_ref[1]+(-0.109602718093)*x_ref[2]**o+(-0.764760324931)*x_ref[2]
        ref[1,1,1,1]=(0.333747531705)*x_ref[0]**o+(-0.975460281758)*x_ref[0]+(-0.263662359116)*x_ref[1]**o+(0.833048103253)*x_ref[1]+(-0.403378941533)*x_ref[2]**o+(0.99255510422)*x_ref[2]
        ref[1,1,2,0]=(-0.643575493982)*x_ref[0]**o+(0.408164900892)*x_ref[0]+(0.330669259534)*x_ref[1]**o+(0.696238788141)*x_ref[1]+(0.8187478709)*x_ref[2]**o+(0.269447281859)*x_ref[2]
        ref[1,1,2,1]=(0.0975010793595)*x_ref[0]**o+(-0.266707476957)*x_ref[0]+(-0.627484959323)*x_ref[1]**o+(0.189636667701)*x_ref[1]+(0.0634608353528)*x_ref[2]**o+(0.425595191628)*x_ref[2]
        ref[1,2,0,0]=(0.144863702314)*x_ref[0]**o+(0.62877495084)*x_ref[0]+(-0.937663622583)*x_ref[1]**o+(0.643925666447)*x_ref[1]+(0.0752876047081)*x_ref[2]**o+(-0.0278560665987)*x_ref[2]
        ref[1,2,0,1]=(0.825937768085)*x_ref[0]**o+(-0.350938240562)*x_ref[0]+(-0.316596763726)*x_ref[1]**o+(0.248951654728)*x_ref[1]+(-0.4533348882)*x_ref[2]**o+(-0.840614727297)*x_ref[2]
        ref[1,2,1,0]=(0.235071855147)*x_ref[0]**o+(0.818758983424)*x_ref[0]+(-0.962091319923)*x_ref[1]**o+(-0.0687829219781)*x_ref[1]+(0.256236142265)*x_ref[2]**o+(-0.236383412975)*x_ref[2]
        ref[1,2,1,1]=(-0.674935027618)*x_ref[0]**o+(0.506392204682)*x_ref[0]+(-0.891135534099)*x_ref[1]**o+(0.200562756224)*x_ref[1]+(-0.509385776169)*x_ref[2]**o+(-0.709115810434)*x_ref[2]
        ref[1,2,2,0]=(0.476890911154)*x_ref[0]**o+(-0.716988939897)*x_ref[0]+(-0.708930094063)*x_ref[1]**o+(-0.221417730773)*x_ref[1]+(-0.0530682866975)*x_ref[2]**o+(-0.933479841665)*x_ref[2]
        ref[1,2,2,1]=(0.912189530094)*x_ref[0]**o+(-0.287403816104)*x_ref[0]+(-0.755698083102)*x_ref[1]**o+(-0.996716444257)*x_ref[1]+(-0.836456434626)*x_ref[2]**o+(-0.463710726041)*x_ref[2]
        ref[1,3,0,0]=(0.253691639919)*x_ref[0]**o+(-0.279433210688)*x_ref[0]+(-0.117635686336)*x_ref[1]**o+(0.790073676599)*x_ref[1]+(-0.559215280716)*x_ref[2]**o+(-0.103190062272)*x_ref[2]
        ref[1,3,0,1]=(0.0970821498373)*x_ref[0]**o+(0.814008398347)*x_ref[0]+(0.583184329882)*x_ref[1]**o+(-0.598484860263)*x_ref[1]+(-0.00216176134509)*x_ref[2]**o+(0.14889662547)*x_ref[2]
        ref[1,3,1,0]=(-0.730045080625)*x_ref[0]**o+(-0.746336558552)*x_ref[0]+(-0.995897359987)*x_ref[1]**o+(-0.70582565406)*x_ref[1]+(0.36600954398)*x_ref[2]**o+(-0.0843877537778)*x_ref[2]
        ref[1,3,1,1]=(-0.714997081413)*x_ref[0]**o+(-0.138823664507)*x_ref[0]+(0.969108046142)*x_ref[1]**o+(-0.201514397247)*x_ref[1]+(-0.438601765508)*x_ref[2]**o+(-0.984456457681)*x_ref[2]
        ref[1,3,2,0]=(-0.577419600949)*x_ref[0]**o+(-0.821514372871)*x_ref[0]+(-0.179395810292)*x_ref[1]**o+(-0.904303143215)*x_ref[1]+(0.27616964799)*x_ref[2]**o+(-0.272088845238)*x_ref[2]
        ref[1,3,2,1]=(0.31325269807)*x_ref[0]**o+(-0.0134619122423)*x_ref[0]+(-0.641786875798)*x_ref[1]**o+(-0.568855591797)*x_ref[1]+(0.600302453067)*x_ref[2]**o+(0.830044865545)*x_ref[2]
        ref[1,4,0,0]=(-0.791605536671)*x_ref[0]**o+(-0.578889866035)*x_ref[0]+(-0.22659053834)*x_ref[1]**o+(0.837093316131)*x_ref[1]+(0.499953644961)*x_ref[2]**o+(-0.289930435498)*x_ref[2]
        ref[1,4,0,1]=(0.819284096253)*x_ref[0]**o+(0.272518944584)*x_ref[0]+(0.199276568134)*x_ref[1]**o+(0.132962435684)*x_ref[1]+(-0.896986377734)*x_ref[2]**o+(0.775604010154)*x_ref[2]
        ref[1,4,1,0]=(-0.450392824249)*x_ref[0]**o+(0.376154164409)*x_ref[0]+(0.199853639816)*x_ref[1]**o+(-0.679551176507)*x_ref[1]+(0.565506686048)*x_ref[2]**o+(0.391489667485)*x_ref[2]
        ref[1,4,1,1]=(-0.0905057426998)*x_ref[0]**o+(-0.749252305812)*x_ref[0]+(-0.21131989398)*x_ref[1]**o+(-0.671560452908)*x_ref[1]+(-0.760505475283)*x_ref[2]**o+(0.763692313655)*x_ref[2]
        ref[1,4,2,0]=(-0.219636883594)*x_ref[0]**o+(-0.380991913771)*x_ref[0]+(0.831879211521)*x_ref[1]**o+(-0.0686682700553)*x_ref[1]+(0.843832009742)*x_ref[2]**o+(-0.86627759217)*x_ref[2]
        ref[1,4,2,1]=(-0.332685504588)*x_ref[0]**o+(0.589527397306)*x_ref[0]+(0.624226689395)*x_ref[1]**o+(0.0515479774619)*x_ref[1]+(0.156657102499)*x_ref[2]**o+(0.465889992773)*x_ref[2]
        ref[2,0,0,0]=(0.250467292043)*x_ref[0]**o+(-0.0658874324582)*x_ref[0]+(-0.0709052912862)*x_ref[1]**o+(0.700837318576)*x_ref[1]+(0.650887276648)*x_ref[2]**o+(0.816187438816)*x_ref[2]
        ref[2,0,0,1]=(-0.697069910038)*x_ref[0]**o+(-0.287284875865)*x_ref[0]+(-0.972646385975)*x_ref[1]**o+(-0.430570334599)*x_ref[1]+(0.400826092127)*x_ref[2]**o+(-0.484530830951)*x_ref[2]
        ref[2,0,1,0]=(0.58433575578)*x_ref[0]**o+(-0.114741976876)*x_ref[0]+(-0.205615141565)*x_ref[1]**o+(-0.476488876624)*x_ref[1]+(0.323174445651)*x_ref[2]**o+(-0.152550510154)*x_ref[2]
        ref[2,0,1,1]=(0.928306678393)*x_ref[0]**o+(0.104020335724)*x_ref[0]+(-0.0416879184197)*x_ref[1]**o+(0.865779079734)*x_ref[1]+(-0.169672240264)*x_ref[2]**o+(0.118778471217)*x_ref[2]
        ref[2,0,2,0]=(0.652779555688)*x_ref[0]**o+(-0.0209488281208)*x_ref[0]+(0.809628464928)*x_ref[1]**o+(-0.85121566767)*x_ref[1]+(-0.845109631338)*x_ref[2]**o+(-0.970252635974)*x_ref[2]
        ref[2,0,2,1]=(-0.740845790747)*x_ref[0]**o+(-0.705835368678)*x_ref[0]+(0.26040103782)*x_ref[1]**o+(0.814854175049)*x_ref[1]+(-0.00842496430724)*x_ref[2]**o+(0.192519753618)*x_ref[2]
        ref[2,1,0,0]=(0.629347590715)*x_ref[0]**o+(0.922723375982)*x_ref[0]+(-0.430492700295)*x_ref[1]**o+(-0.895795670345)*x_ref[1]+(0.708440378414)*x_ref[2]**o+(-0.155782723618)*x_ref[2]
        ref[2,1,0,1]=(-0.478612307362)*x_ref[0]**o+(0.645774615123)*x_ref[0]+(-0.461136871779)*x_ref[1]**o+(0.297927339931)*x_ref[1]+(-0.452011143468)*x_ref[2]**o+(0.267002423863)*x_ref[2]
        ref[2,1,1,0]=(-0.0591350600103)*x_ref[0]**o+(-0.252358408576)*x_ref[0]+(-0.0412752075681)*x_ref[1]**o+(0.942736455517)*x_ref[1]+(-0.223363982173)*x_ref[2]**o+(-0.752007969767)*x_ref[2]
        ref[2,1,1,1]=(0.197290290824)*x_ref[0]**o+(-0.170028529393)*x_ref[0]+(-0.143011200679)*x_ref[1]**o+(-0.694395961945)*x_ref[1]+(0.430761130021)*x_ref[2]**o+(0.298239563293)*x_ref[2]
        ref[2,1,2,0]=(0.258474077793)*x_ref[0]**o+(-0.956041015018)*x_ref[0]+(0.360444161221)*x_ref[1]**o+(-0.969922460117)*x_ref[1]+(0.856255540521)*x_ref[2]**o+(0.108872710157)*x_ref[2]
        ref[2,1,2,1]=(-0.694400609963)*x_ref[0]**o+(-0.864737376468)*x_ref[0]+(-0.898839555362)*x_ref[1]**o+(-0.655053934442)*x_ref[1]+(-0.511293723446)*x_ref[2]**o+(-0.204242252526)*x_ref[2]
        ref[2,2,0,0]=(-0.427946109181)*x_ref[0]**o+(0.435798407553)*x_ref[0]+(-0.567923694982)*x_ref[1]**o+(0.870896124804)*x_ref[1]+(-0.996434693707)*x_ref[2]**o+(-0.683367112977)*x_ref[2]
        ref[2,2,0,1]=(-0.219431769721)*x_ref[0]**o+(-0.302922785317)*x_ref[0]+(-0.0998576564441)*x_ref[1]**o+(0.64263522193)*x_ref[1]+(0.53256637446)*x_ref[2]**o+(-0.00387183821982)*x_ref[2]
        ref[2,2,1,0]=(-0.174988349945)*x_ref[0]**o+(0.408056661499)*x_ref[0]+(0.55925945151)*x_ref[1]**o+(-0.647959566858)*x_ref[1]+(0.905755316979)*x_ref[2]**o+(0.691774684766)*x_ref[2]
        ref[2,2,1,1]=(-0.882995226725)*x_ref[0]**o+(-0.948874015343)*x_ref[0]+(-0.652506601975)*x_ref[1]**o+(-0.137989382315)*x_ref[1]+(0.665695021939)*x_ref[2]**o+(0.163493199569)*x_ref[2]
        ref[2,2,2,0]=(0.666421535029)*x_ref[0]**o+(-0.984765122669)*x_ref[0]+(0.399488949215)*x_ref[1]**o+(-0.99963893587)*x_ref[1]+(0.225552018439)*x_ref[2]**o+(0.430770653325)*x_ref[2]
        ref[2,2,2,1]=(-0.0284528199834)*x_ref[0]**o+(0.0421195684748)*x_ref[0]+(-0.399757974604)*x_ref[1]**o+(0.305895209328)*x_ref[1]+(0.190920234352)*x_ref[2]**o+(-0.0990327186702)*x_ref[2]
        ref[2,3,0,0]=(-0.565899348734)*x_ref[0]**o+(0.636275927936)*x_ref[0]+(0.682710455722)*x_ref[1]**o+(-0.434837962369)*x_ref[1]+(0.252457745239)*x_ref[2]**o+(-0.709529514057)*x_ref[2]
        ref[2,3,0,1]=(0.0113021999515)*x_ref[0]**o+(0.451528226044)*x_ref[0]+(-0.705424064371)*x_ref[1]**o+(0.500516985463)*x_ref[1]+(-0.528750586385)*x_ref[2]**o+(0.998214035471)*x_ref[2]
        ref[2,3,1,0]=(-0.879296343578)*x_ref[0]**o+(-0.724012783173)*x_ref[0]+(0.51105902172)*x_ref[1]**o+(-0.840460578985)*x_ref[1]+(0.222130858003)*x_ref[2]**o+(0.998500581881)*x_ref[2]
        ref[2,3,1,1]=(0.478703623115)*x_ref[0]**o+(-0.94935681973)*x_ref[0]+(-0.609714878079)*x_ref[1]**o+(0.0318988430394)*x_ref[1]+(-0.858395334121)*x_ref[2]**o+(-0.171229770192)*x_ref[2]
        ref[2,3,2,0]=(0.0791777006911)*x_ref[0]**o+(-0.306954070098)*x_ref[0]+(-0.182890473769)*x_ref[1]**o+(-0.26384266322)*x_ref[1]+(0.663652835142)*x_ref[2]**o+(0.188229986922)*x_ref[2]
        ref[2,3,2,1]=(-0.101910553498)*x_ref[0]**o+(-0.34187147162)*x_ref[0]+(-0.52746320646)*x_ref[1]**o+(-0.891142643574)*x_ref[1]+(-0.84135671258)*x_ref[2]**o+(0.448615749816)*x_ref[2]
        ref[2,4,0,0]=(0.928980482137)*x_ref[0]**o+(0.00486879685273)*x_ref[0]+(-0.068529973495)*x_ref[1]**o+(-0.16116665925)*x_ref[1]+(0.247146438634)*x_ref[2]**o+(-0.341715145751)*x_ref[2]
        ref[2,4,0,1]=(0.486412050624)*x_ref[0]**o+(-0.929254100014)*x_ref[0]+(0.439267744247)*x_ref[1]**o+(-0.430515713902)*x_ref[1]+(-0.317438643233)*x_ref[2]**o+(-0.0843825737593)*x_ref[2]
        ref[2,4,1,0]=(0.0130446306687)*x_ref[0]**o+(0.324310412551)*x_ref[0]+(-0.38112172561)*x_ref[1]**o+(-0.92878637625)*x_ref[1]+(0.32867951654)*x_ref[2]**o+(-0.0315638177916)*x_ref[2]
        ref[2,4,1,1]=(-0.254225591431)*x_ref[0]**o+(0.321872838348)*x_ref[0]+(0.179619901508)*x_ref[1]**o+(-0.858713537407)*x_ref[1]+(0.151254493236)*x_ref[2]**o+(-0.340368823543)*x_ref[2]
        ref[2,4,2,0]=(-0.0844677926666)*x_ref[0]**o+(0.398017491087)*x_ref[0]+(0.0313455976792)*x_ref[1]**o+(-0.872666391632)*x_ref[1]+(0.301765055045)*x_ref[2]**o+(0.563923225852)*x_ref[2]
        ref[2,4,2,1]=(0.814244097177)*x_ref[0]**o+(-0.223957875084)*x_ref[0]+(0.447995085387)*x_ref[1]**o+(0.350953464712)*x_ref[1]+(0.333269027952)*x_ref[2]**o+(-0.941385895584)*x_ref[2]
        ref[3,0,0,0]=(0.0789492878053)*x_ref[0]**o+(0.525756966035)*x_ref[0]+(0.87125043555)*x_ref[1]**o+(0.881346392374)*x_ref[1]+(-0.855477786784)*x_ref[2]**o+(-0.991712685096)*x_ref[2]
        ref[3,0,0,1]=(-0.0359771254585)*x_ref[0]**o+(0.603218005193)*x_ref[0]+(-0.580729043759)*x_ref[1]**o+(-0.20390195049)*x_ref[1]+(-0.862263281233)*x_ref[2]**o+(-0.196587155144)*x_ref[2]
        ref[3,0,1,0]=(0.879750375964)*x_ref[0]**o+(-0.251177194936)*x_ref[0]+(0.191264882539)*x_ref[1]**o+(-0.95611617781)*x_ref[1]+(0.71738769264)*x_ref[2]**o+(-0.691767368606)*x_ref[2]
        ref[3,0,1,1]=(-0.761557468157)*x_ref[0]**o+(-0.381655367519)*x_ref[0]+(0.587969140055)*x_ref[1]**o+(0.622246650476)*x_ref[1]+(0.117544678721)*x_ref[2]**o+(-0.576059347687)*x_ref[2]
        ref[3,0,2,0]=(-0.587802439618)*x_ref[0]**o+(-0.981432433894)*x_ref[0]+(-0.758496301974)*x_ref[1]**o+(0.656105955022)*x_ref[1]+(-0.689107562745)*x_ref[2]**o+(0.267295196672)*x_ref[2]
        ref[3,0,2,1]=(-0.0524224100533)*x_ref[0]**o+(-0.925763321074)*x_ref[0]+(0.218186590694)*x_ref[1]**o+(-0.598934300523)*x_ref[1]+(0.22756773)*x_ref[2]**o+(-0.770210327792)*x_ref[2]
        ref[3,1,0,0]=(0.922635777646)*x_ref[0]**o+(-0.278736876358)*x_ref[0]+(0.0631653424614)*x_ref[1]**o+(-0.356448496856)*x_ref[1]+(-0.609671239719)*x_ref[2]**o+(0.189678422301)*x_ref[2]
        ref[3,1,0,1]=(-0.509070180487)*x_ref[0]**o+(-0.250043376376)*x_ref[0]+(-0.0697248208177)*x_ref[1]**o+(0.531153766941)*x_ref[1]+(0.240916012233)*x_ref[2]**o+(0.391353341194)*x_ref[2]
        ref[3,1,1,0]=(-0.514764390151)*x_ref[0]**o+(-0.347397972881)*x_ref[0]+(0.138729357858)*x_ref[1]**o+(-0.30135683112)*x_ref[1]+(0.541893745985)*x_ref[2]**o+(0.414478801003)*x_ref[2]
        ref[3,1,1,1]=(0.866088916369)*x_ref[0]**o+(-0.96729572537)*x_ref[0]+(0.824735563587)*x_ref[1]**o+(-0.479859785014)*x_ref[1]+(-0.175784746993)*x_ref[2]**o+(-0.998407148967)*x_ref[2]
        ref[3,1,2,0]=(0.327087092534)*x_ref[0]**o+(0.171000747621)*x_ref[0]+(-0.33910895269)*x_ref[1]**o+(0.349474180167)*x_ref[1]+(0.561935192081)*x_ref[2]**o+(0.358380998288)*x_ref[2]
        ref[3,1,2,1]=(0.35646522102)*x_ref[0]**o+(-0.912130691808)*x_ref[0]+(0.796756777424)*x_ref[1]**o+(-0.91884834291)*x_ref[1]+(0.956454722923)*x_ref[2]**o+(0.462859602491)*x_ref[2]
        ref[3,2,0,0]=(-0.849637152897)*x_ref[0]**o+(0.353228393186)*x_ref[0]+(-0.725757036845)*x_ref[1]**o+(0.590369089609)*x_ref[1]+(-0.280432657156)*x_ref[2]**o+(0.142413703594)*x_ref[2]
        ref[3,2,0,1]=(-0.640766746158)*x_ref[0]**o+(-0.454583129018)*x_ref[0]+(-0.616394565949)*x_ref[1]**o+(0.724166900329)*x_ref[1]+(-0.748682704671)*x_ref[2]**o+(0.934076771628)*x_ref[2]
        ref[3,2,1,0]=(-0.824548065846)*x_ref[0]**o+(0.40250159164)*x_ref[0]+(0.763315748823)*x_ref[1]**o+(0.255878553659)*x_ref[1]+(0.872676778336)*x_ref[2]**o+(-0.100430227782)*x_ref[2]
        ref[3,2,1,1]=(-0.610248210467)*x_ref[0]**o+(0.160976829271)*x_ref[0]+(-0.384236718764)*x_ref[1]**o+(0.657119635861)*x_ref[1]+(0.161695416084)*x_ref[2]**o+(0.505997006976)*x_ref[2]
        ref[3,2,2,0]=(0.220409736038)*x_ref[0]**o+(0.398197530033)*x_ref[0]+(-0.288242008883)*x_ref[1]**o+(0.0832866402504)*x_ref[1]+(0.761030964351)*x_ref[2]**o+(0.751986795183)*x_ref[2]
        ref[3,2,2,1]=(-0.396845385157)*x_ref[0]**o+(-0.0792538647037)*x_ref[0]+(-0.327181444898)*x_ref[1]**o+(-0.810682039031)*x_ref[1]+(0.260671755637)*x_ref[2]**o+(-0.242407958043)*x_ref[2]
        ref[3,3,0,0]=(-0.663937889328)*x_ref[0]**o+(0.587545393032)*x_ref[0]+(0.210910229094)*x_ref[1]**o+(0.500419329566)*x_ref[1]+(0.202539078075)*x_ref[2]**o+(-0.623007416272)*x_ref[2]
        ref[3,3,0,1]=(0.0375980252743)*x_ref[0]**o+(-0.435705054083)*x_ref[0]+(-0.660885021014)*x_ref[1]**o+(-0.0332748037874)*x_ref[1]+(-0.832274220584)*x_ref[2]**o+(0.805002616286)*x_ref[2]
        ref[3,3,1,0]=(0.395522385265)*x_ref[0]**o+(0.901865335207)*x_ref[0]+(0.016322521344)*x_ref[1]**o+(-0.532212956041)*x_ref[1]+(0.190751397835)*x_ref[2]**o+(0.234716440942)*x_ref[2]
        ref[3,3,1,1]=(0.257090076986)*x_ref[0]**o+(-0.169468215187)*x_ref[0]+(-0.0641337112558)*x_ref[1]**o+(-0.674363367639)*x_ref[1]+(0.13633053895)*x_ref[2]**o+(-0.741438377514)*x_ref[2]
        ref[3,3,2,0]=(0.984088274745)*x_ref[0]**o+(-0.669313378065)*x_ref[0]+(0.899763931565)*x_ref[1]**o+(-0.874652895499)*x_ref[1]+(-0.172620100734)*x_ref[2]**o+(-0.635666769897)*x_ref[2]
        ref[3,3,2,1]=(0.693112828433)*x_ref[0]**o+(-0.99231490163)*x_ref[0]+(0.525842419539)*x_ref[1]**o+(0.268799565406)*x_ref[1]+(-0.473532039244)*x_ref[2]**o+(0.50555338295)*x_ref[2]
        ref[3,4,0,0]=(0.780678215778)*x_ref[0]**o+(0.56287435306)*x_ref[0]+(0.322651378857)*x_ref[1]**o+(0.268655956157)*x_ref[1]+(0.438312596985)*x_ref[2]**o+(0.175512199841)*x_ref[2]
        ref[3,4,0,1]=(-0.510399313775)*x_ref[0]**o+(-0.63898491047)*x_ref[0]+(-0.765120717001)*x_ref[1]**o+(0.45570492579)*x_ref[1]+(-0.148930444588)*x_ref[2]**o+(0.458400743585)*x_ref[2]
        ref[3,4,1,0]=(-0.629456511378)*x_ref[0]**o+(-0.0956526447002)*x_ref[0]+(0.223599370201)*x_ref[1]**o+(-0.689787947305)*x_ref[1]+(0.834792179813)*x_ref[2]**o+(0.413903455599)*x_ref[2]
        ref[3,4,1,1]=(0.89648905371)*x_ref[0]**o+(0.420890001861)*x_ref[0]+(-0.890077307972)*x_ref[1]**o+(0.424785046869)*x_ref[1]+(0.84365123356)*x_ref[2]**o+(0.56417731826)*x_ref[2]
        ref[3,4,2,0]=(0.853620031631)*x_ref[0]**o+(-0.719293879422)*x_ref[0]+(-0.811007794139)*x_ref[1]**o+(-0.236043157925)*x_ref[1]+(0.130830104273)*x_ref[2]**o+(-0.554431879475)*x_ref[2]
        ref[3,4,2,1]=(-0.555616431129)*x_ref[0]**o+(-0.900929040274)*x_ref[0]+(0.773492786481)*x_ref[1]**o+(-0.445322579241)*x_ref[1]+(0.270832172179)*x_ref[2]**o+(-0.716484089692)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromSymbol_ReducedFunctionOnContactOne_rank0(self):
      """
      tests interpolate for rank 0 Symbol onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactOne onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactOne(self.domain)
      x=w.getX()
      arg=Symbol(shape=(),dim=dim)
      s=Data(0,(),w)
      ref=Data(0,(),w_ref)
      if dim==2:
        s=(-0.806530631577)*x[0]**o+(-0.341707069796)*x[0]+(0.624950122202)*x[1]**o+(-0.547410220031)*x[1]
        ref=(-0.806530631577)*x_ref[0]**o+(-0.341707069796)*x_ref[0]+(0.624950122202)*x_ref[1]**o+(-0.547410220031)*x_ref[1]
      else:
        s=(-0.227792789776)*x[0]**o+(-0.458918821401)*x[0]+(0.333705314078)*x[1]**o+(0.513071453636)*x[1]+(-0.251497218549)*x[2]**o+(0.447781682612)*x[2]
        ref=(-0.227792789776)*x_ref[0]**o+(-0.458918821401)*x_ref[0]+(0.333705314078)*x_ref[1]**o+(0.513071453636)*x_ref[1]+(-0.251497218549)*x_ref[2]**o+(0.447781682612)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromSymbol_ReducedFunctionOnContactOne_rank1(self):
      """
      tests interpolate for rank 1 Symbol onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactOne onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactOne(self.domain)
      x=w.getX()
      arg=Symbol(shape=(2,),dim=dim)
      s=Data(0,(2,),w)
      ref=Data(0,(2,),w_ref)
      if dim==2:
        s[0]=(-0.432328873396)*x[0]**o+(0.677542875504)*x[0]+(-0.834641628818)*x[1]**o+(0.628835966611)*x[1]
        s[1]=(-0.902852113303)*x[0]**o+(-0.593877706793)*x[0]+(0.395220406099)*x[1]**o+(-0.523411084099)*x[1]
        ref[0]=(-0.432328873396)*x_ref[0]**o+(0.677542875504)*x_ref[0]+(-0.834641628818)*x_ref[1]**o+(0.628835966611)*x_ref[1]
        ref[1]=(-0.902852113303)*x_ref[0]**o+(-0.593877706793)*x_ref[0]+(0.395220406099)*x_ref[1]**o+(-0.523411084099)*x_ref[1]
      else:
        s[0]=(0.297175304924)*x[0]**o+(-0.43517991218)*x[0]+(-0.0313129470041)*x[1]**o+(0.269658750043)*x[1]+(-0.407781379485)*x[2]**o+(-0.605849978158)*x[2]
        s[1]=(-0.525947091801)*x[0]**o+(-0.39352324679)*x[0]+(0.683408550141)*x[1]**o+(-0.623028099041)*x[1]+(0.0817707351822)*x[2]**o+(0.36388186042)*x[2]
        ref[0]=(0.297175304924)*x_ref[0]**o+(-0.43517991218)*x_ref[0]+(-0.0313129470041)*x_ref[1]**o+(0.269658750043)*x_ref[1]+(-0.407781379485)*x_ref[2]**o+(-0.605849978158)*x_ref[2]
        ref[1]=(-0.525947091801)*x_ref[0]**o+(-0.39352324679)*x_ref[0]+(0.683408550141)*x_ref[1]**o+(-0.623028099041)*x_ref[1]+(0.0817707351822)*x_ref[2]**o+(0.36388186042)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromSymbol_ReducedFunctionOnContactOne_rank2(self):
      """
      tests interpolate for rank 2 Symbol onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactOne onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactOne(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5),dim=dim)
      s=Data(0,(4, 5),w)
      ref=Data(0,(4, 5),w_ref)
      if dim==2:
        s[0,0]=(0.406232678425)*x[0]**o+(-0.930963378238)*x[0]+(0.845174140625)*x[1]**o+(0.981192355961)*x[1]
        s[0,1]=(-0.635413173141)*x[0]**o+(-0.497296028209)*x[0]+(-0.994669338217)*x[1]**o+(-0.931999198677)*x[1]
        s[0,2]=(0.0780179414399)*x[0]**o+(0.478842279811)*x[0]+(-0.0352846591054)*x[1]**o+(0.111952755346)*x[1]
        s[0,3]=(-0.934565575643)*x[0]**o+(-0.257127097452)*x[0]+(-0.375100482254)*x[1]**o+(0.26830947979)*x[1]
        s[0,4]=(0.00406708346809)*x[0]**o+(0.428953823715)*x[0]+(0.582342187327)*x[1]**o+(0.915249586352)*x[1]
        s[1,0]=(-0.667807592712)*x[0]**o+(0.16108128518)*x[0]+(0.143438381665)*x[1]**o+(-0.96131677347)*x[1]
        s[1,1]=(0.939048416505)*x[0]**o+(0.953402057565)*x[0]+(-0.541334930887)*x[1]**o+(-0.0575833896416)*x[1]
        s[1,2]=(0.0892590765861)*x[0]**o+(-0.599905917167)*x[0]+(-0.0350800081223)*x[1]**o+(-0.0380326906586)*x[1]
        s[1,3]=(-0.620396486232)*x[0]**o+(-0.147159989485)*x[0]+(-0.975288447854)*x[1]**o+(-0.299830553213)*x[1]
        s[1,4]=(-0.500651954078)*x[0]**o+(0.463784646532)*x[0]+(0.494626131733)*x[1]**o+(0.714313271607)*x[1]
        s[2,0]=(0.999573391428)*x[0]**o+(0.740845556331)*x[0]+(-0.254541944755)*x[1]**o+(0.90861174582)*x[1]
        s[2,1]=(0.106308044175)*x[0]**o+(0.737496999865)*x[0]+(0.345388021114)*x[1]**o+(0.732857975859)*x[1]
        s[2,2]=(0.0488435293882)*x[0]**o+(-0.580187506515)*x[0]+(-0.146074818428)*x[1]**o+(0.0903174437642)*x[1]
        s[2,3]=(0.105892490012)*x[0]**o+(0.0233855588118)*x[0]+(-0.0688234002192)*x[1]**o+(-0.608212758131)*x[1]
        s[2,4]=(0.882922466867)*x[0]**o+(0.346911804995)*x[0]+(-0.247283497286)*x[1]**o+(0.411870299278)*x[1]
        s[3,0]=(0.139595043935)*x[0]**o+(-0.228322879218)*x[0]+(-0.124338355975)*x[1]**o+(0.633626671983)*x[1]
        s[3,1]=(0.0525495199049)*x[0]**o+(0.446874982783)*x[0]+(-0.597381789826)*x[1]**o+(0.232913952793)*x[1]
        s[3,2]=(0.350084550681)*x[0]**o+(-0.329493065869)*x[0]+(0.403975623285)*x[1]**o+(0.560476340931)*x[1]
        s[3,3]=(0.918601061431)*x[0]**o+(0.5842950108)*x[0]+(-0.909414458447)*x[1]**o+(0.208889362019)*x[1]
        s[3,4]=(0.0624223531897)*x[0]**o+(-0.364330980767)*x[0]+(-0.388102867728)*x[1]**o+(0.547027313956)*x[1]
        ref[0,0]=(0.406232678425)*x_ref[0]**o+(-0.930963378238)*x_ref[0]+(0.845174140625)*x_ref[1]**o+(0.981192355961)*x_ref[1]
        ref[0,1]=(-0.635413173141)*x_ref[0]**o+(-0.497296028209)*x_ref[0]+(-0.994669338217)*x_ref[1]**o+(-0.931999198677)*x_ref[1]
        ref[0,2]=(0.0780179414399)*x_ref[0]**o+(0.478842279811)*x_ref[0]+(-0.0352846591054)*x_ref[1]**o+(0.111952755346)*x_ref[1]
        ref[0,3]=(-0.934565575643)*x_ref[0]**o+(-0.257127097452)*x_ref[0]+(-0.375100482254)*x_ref[1]**o+(0.26830947979)*x_ref[1]
        ref[0,4]=(0.00406708346809)*x_ref[0]**o+(0.428953823715)*x_ref[0]+(0.582342187327)*x_ref[1]**o+(0.915249586352)*x_ref[1]
        ref[1,0]=(-0.667807592712)*x_ref[0]**o+(0.16108128518)*x_ref[0]+(0.143438381665)*x_ref[1]**o+(-0.96131677347)*x_ref[1]
        ref[1,1]=(0.939048416505)*x_ref[0]**o+(0.953402057565)*x_ref[0]+(-0.541334930887)*x_ref[1]**o+(-0.0575833896416)*x_ref[1]
        ref[1,2]=(0.0892590765861)*x_ref[0]**o+(-0.599905917167)*x_ref[0]+(-0.0350800081223)*x_ref[1]**o+(-0.0380326906586)*x_ref[1]
        ref[1,3]=(-0.620396486232)*x_ref[0]**o+(-0.147159989485)*x_ref[0]+(-0.975288447854)*x_ref[1]**o+(-0.299830553213)*x_ref[1]
        ref[1,4]=(-0.500651954078)*x_ref[0]**o+(0.463784646532)*x_ref[0]+(0.494626131733)*x_ref[1]**o+(0.714313271607)*x_ref[1]
        ref[2,0]=(0.999573391428)*x_ref[0]**o+(0.740845556331)*x_ref[0]+(-0.254541944755)*x_ref[1]**o+(0.90861174582)*x_ref[1]
        ref[2,1]=(0.106308044175)*x_ref[0]**o+(0.737496999865)*x_ref[0]+(0.345388021114)*x_ref[1]**o+(0.732857975859)*x_ref[1]
        ref[2,2]=(0.0488435293882)*x_ref[0]**o+(-0.580187506515)*x_ref[0]+(-0.146074818428)*x_ref[1]**o+(0.0903174437642)*x_ref[1]
        ref[2,3]=(0.105892490012)*x_ref[0]**o+(0.0233855588118)*x_ref[0]+(-0.0688234002192)*x_ref[1]**o+(-0.608212758131)*x_ref[1]
        ref[2,4]=(0.882922466867)*x_ref[0]**o+(0.346911804995)*x_ref[0]+(-0.247283497286)*x_ref[1]**o+(0.411870299278)*x_ref[1]
        ref[3,0]=(0.139595043935)*x_ref[0]**o+(-0.228322879218)*x_ref[0]+(-0.124338355975)*x_ref[1]**o+(0.633626671983)*x_ref[1]
        ref[3,1]=(0.0525495199049)*x_ref[0]**o+(0.446874982783)*x_ref[0]+(-0.597381789826)*x_ref[1]**o+(0.232913952793)*x_ref[1]
        ref[3,2]=(0.350084550681)*x_ref[0]**o+(-0.329493065869)*x_ref[0]+(0.403975623285)*x_ref[1]**o+(0.560476340931)*x_ref[1]
        ref[3,3]=(0.918601061431)*x_ref[0]**o+(0.5842950108)*x_ref[0]+(-0.909414458447)*x_ref[1]**o+(0.208889362019)*x_ref[1]
        ref[3,4]=(0.0624223531897)*x_ref[0]**o+(-0.364330980767)*x_ref[0]+(-0.388102867728)*x_ref[1]**o+(0.547027313956)*x_ref[1]
      else:
        s[0,0]=(0.127471865714)*x[0]**o+(0.75668808568)*x[0]+(0.0197793355564)*x[1]**o+(0.342611017161)*x[1]+(0.256637655934)*x[2]**o+(0.144098222666)*x[2]
        s[0,1]=(0.460982544361)*x[0]**o+(0.0758913155194)*x[0]+(-0.738649205082)*x[1]**o+(-0.729674519552)*x[1]+(0.134554511124)*x[2]**o+(-0.966054425176)*x[2]
        s[0,2]=(-0.515765669688)*x[0]**o+(0.223475540597)*x[0]+(0.710015035386)*x[1]**o+(-0.810406212241)*x[1]+(-0.706682492015)*x[2]**o+(-0.176836116541)*x[2]
        s[0,3]=(0.849940074716)*x[0]**o+(0.276008832632)*x[0]+(0.753715668597)*x[1]**o+(0.327023435021)*x[1]+(-0.432843830378)*x[2]**o+(-0.89188163433)*x[2]
        s[0,4]=(0.306934523362)*x[0]**o+(0.400359334134)*x[0]+(-0.994806829246)*x[1]**o+(0.0349597549696)*x[1]+(-0.978591943815)*x[2]**o+(0.612187266226)*x[2]
        s[1,0]=(0.0461258495436)*x[0]**o+(-0.959702481081)*x[0]+(-0.3018982478)*x[1]**o+(-0.462251712831)*x[1]+(-0.94462432144)*x[2]**o+(-0.506662578784)*x[2]
        s[1,1]=(0.233144525806)*x[0]**o+(-0.414379147564)*x[0]+(0.38512336385)*x[1]**o+(-0.310763668907)*x[1]+(0.48918840723)*x[2]**o+(0.801469913207)*x[2]
        s[1,2]=(-0.132344623984)*x[0]**o+(-0.242418158654)*x[0]+(-0.618349833846)*x[1]**o+(0.249538312346)*x[1]+(0.466255623863)*x[2]**o+(0.619476167885)*x[2]
        s[1,3]=(-0.600411057229)*x[0]**o+(-0.65398469587)*x[0]+(-0.086416622227)*x[1]**o+(0.497186435444)*x[1]+(0.490006868481)*x[2]**o+(-0.281505238163)*x[2]
        s[1,4]=(-0.833074114751)*x[0]**o+(0.179085322144)*x[0]+(-0.669109910245)*x[1]**o+(0.853858313308)*x[1]+(-0.474984175674)*x[2]**o+(0.0145591425038)*x[2]
        s[2,0]=(-0.463438302204)*x[0]**o+(-0.815969915511)*x[0]+(0.958428072331)*x[1]**o+(-0.760925159788)*x[1]+(0.419348775332)*x[2]**o+(-0.146777978724)*x[2]
        s[2,1]=(0.64963543058)*x[0]**o+(0.713874315445)*x[0]+(0.000795833519698)*x[1]**o+(-0.216757220289)*x[1]+(-0.496572329247)*x[2]**o+(0.535744506796)*x[2]
        s[2,2]=(0.848552125819)*x[0]**o+(0.61034402517)*x[0]+(0.254155499518)*x[1]**o+(0.584452254449)*x[1]+(-0.739215490349)*x[2]**o+(0.553754902653)*x[2]
        s[2,3]=(0.45662694476)*x[0]**o+(0.63839923138)*x[0]+(0.05783248842)*x[1]**o+(0.326555670737)*x[1]+(-0.231728870265)*x[2]**o+(-0.773309888086)*x[2]
        s[2,4]=(-0.848527683977)*x[0]**o+(0.445666786862)*x[0]+(0.96357445665)*x[1]**o+(0.805189746125)*x[1]+(-0.99455326053)*x[2]**o+(-0.149519744556)*x[2]
        s[3,0]=(0.537998019205)*x[0]**o+(-0.160515409613)*x[0]+(-0.518616417074)*x[1]**o+(-0.78931847859)*x[1]+(0.533911685732)*x[2]**o+(0.249570769302)*x[2]
        s[3,1]=(0.127551558633)*x[0]**o+(-0.152792201104)*x[0]+(-0.177992097046)*x[1]**o+(-0.582313615383)*x[1]+(0.180383484767)*x[2]**o+(0.457671553174)*x[2]
        s[3,2]=(0.974312579539)*x[0]**o+(0.0374604823328)*x[0]+(-0.737361607733)*x[1]**o+(0.741091246445)*x[1]+(0.679379795149)*x[2]**o+(-0.904426984523)*x[2]
        s[3,3]=(-0.955715962012)*x[0]**o+(0.136457825359)*x[0]+(0.937865545819)*x[1]**o+(0.39839155392)*x[1]+(0.341983124988)*x[2]**o+(0.521155465952)*x[2]
        s[3,4]=(-0.89730837616)*x[0]**o+(0.615140171681)*x[0]+(-0.71054999877)*x[1]**o+(0.66258279405)*x[1]+(0.241482810749)*x[2]**o+(0.926085913167)*x[2]
        ref[0,0]=(0.127471865714)*x_ref[0]**o+(0.75668808568)*x_ref[0]+(0.0197793355564)*x_ref[1]**o+(0.342611017161)*x_ref[1]+(0.256637655934)*x_ref[2]**o+(0.144098222666)*x_ref[2]
        ref[0,1]=(0.460982544361)*x_ref[0]**o+(0.0758913155194)*x_ref[0]+(-0.738649205082)*x_ref[1]**o+(-0.729674519552)*x_ref[1]+(0.134554511124)*x_ref[2]**o+(-0.966054425176)*x_ref[2]
        ref[0,2]=(-0.515765669688)*x_ref[0]**o+(0.223475540597)*x_ref[0]+(0.710015035386)*x_ref[1]**o+(-0.810406212241)*x_ref[1]+(-0.706682492015)*x_ref[2]**o+(-0.176836116541)*x_ref[2]
        ref[0,3]=(0.849940074716)*x_ref[0]**o+(0.276008832632)*x_ref[0]+(0.753715668597)*x_ref[1]**o+(0.327023435021)*x_ref[1]+(-0.432843830378)*x_ref[2]**o+(-0.89188163433)*x_ref[2]
        ref[0,4]=(0.306934523362)*x_ref[0]**o+(0.400359334134)*x_ref[0]+(-0.994806829246)*x_ref[1]**o+(0.0349597549696)*x_ref[1]+(-0.978591943815)*x_ref[2]**o+(0.612187266226)*x_ref[2]
        ref[1,0]=(0.0461258495436)*x_ref[0]**o+(-0.959702481081)*x_ref[0]+(-0.3018982478)*x_ref[1]**o+(-0.462251712831)*x_ref[1]+(-0.94462432144)*x_ref[2]**o+(-0.506662578784)*x_ref[2]
        ref[1,1]=(0.233144525806)*x_ref[0]**o+(-0.414379147564)*x_ref[0]+(0.38512336385)*x_ref[1]**o+(-0.310763668907)*x_ref[1]+(0.48918840723)*x_ref[2]**o+(0.801469913207)*x_ref[2]
        ref[1,2]=(-0.132344623984)*x_ref[0]**o+(-0.242418158654)*x_ref[0]+(-0.618349833846)*x_ref[1]**o+(0.249538312346)*x_ref[1]+(0.466255623863)*x_ref[2]**o+(0.619476167885)*x_ref[2]
        ref[1,3]=(-0.600411057229)*x_ref[0]**o+(-0.65398469587)*x_ref[0]+(-0.086416622227)*x_ref[1]**o+(0.497186435444)*x_ref[1]+(0.490006868481)*x_ref[2]**o+(-0.281505238163)*x_ref[2]
        ref[1,4]=(-0.833074114751)*x_ref[0]**o+(0.179085322144)*x_ref[0]+(-0.669109910245)*x_ref[1]**o+(0.853858313308)*x_ref[1]+(-0.474984175674)*x_ref[2]**o+(0.0145591425038)*x_ref[2]
        ref[2,0]=(-0.463438302204)*x_ref[0]**o+(-0.815969915511)*x_ref[0]+(0.958428072331)*x_ref[1]**o+(-0.760925159788)*x_ref[1]+(0.419348775332)*x_ref[2]**o+(-0.146777978724)*x_ref[2]
        ref[2,1]=(0.64963543058)*x_ref[0]**o+(0.713874315445)*x_ref[0]+(0.000795833519698)*x_ref[1]**o+(-0.216757220289)*x_ref[1]+(-0.496572329247)*x_ref[2]**o+(0.535744506796)*x_ref[2]
        ref[2,2]=(0.848552125819)*x_ref[0]**o+(0.61034402517)*x_ref[0]+(0.254155499518)*x_ref[1]**o+(0.584452254449)*x_ref[1]+(-0.739215490349)*x_ref[2]**o+(0.553754902653)*x_ref[2]
        ref[2,3]=(0.45662694476)*x_ref[0]**o+(0.63839923138)*x_ref[0]+(0.05783248842)*x_ref[1]**o+(0.326555670737)*x_ref[1]+(-0.231728870265)*x_ref[2]**o+(-0.773309888086)*x_ref[2]
        ref[2,4]=(-0.848527683977)*x_ref[0]**o+(0.445666786862)*x_ref[0]+(0.96357445665)*x_ref[1]**o+(0.805189746125)*x_ref[1]+(-0.99455326053)*x_ref[2]**o+(-0.149519744556)*x_ref[2]
        ref[3,0]=(0.537998019205)*x_ref[0]**o+(-0.160515409613)*x_ref[0]+(-0.518616417074)*x_ref[1]**o+(-0.78931847859)*x_ref[1]+(0.533911685732)*x_ref[2]**o+(0.249570769302)*x_ref[2]
        ref[3,1]=(0.127551558633)*x_ref[0]**o+(-0.152792201104)*x_ref[0]+(-0.177992097046)*x_ref[1]**o+(-0.582313615383)*x_ref[1]+(0.180383484767)*x_ref[2]**o+(0.457671553174)*x_ref[2]
        ref[3,2]=(0.974312579539)*x_ref[0]**o+(0.0374604823328)*x_ref[0]+(-0.737361607733)*x_ref[1]**o+(0.741091246445)*x_ref[1]+(0.679379795149)*x_ref[2]**o+(-0.904426984523)*x_ref[2]
        ref[3,3]=(-0.955715962012)*x_ref[0]**o+(0.136457825359)*x_ref[0]+(0.937865545819)*x_ref[1]**o+(0.39839155392)*x_ref[1]+(0.341983124988)*x_ref[2]**o+(0.521155465952)*x_ref[2]
        ref[3,4]=(-0.89730837616)*x_ref[0]**o+(0.615140171681)*x_ref[0]+(-0.71054999877)*x_ref[1]**o+(0.66258279405)*x_ref[1]+(0.241482810749)*x_ref[2]**o+(0.926085913167)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromSymbol_ReducedFunctionOnContactOne_rank3(self):
      """
      tests interpolate for rank 3 Symbol onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactOne onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactOne(self.domain)
      x=w.getX()
      arg=Symbol(shape=(6, 2, 2),dim=dim)
      s=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2),w_ref)
      if dim==2:
        s[0,0,0]=(0.525970841909)*x[0]**o+(-0.724518605704)*x[0]+(0.0391713787427)*x[1]**o+(0.64004594365)*x[1]
        s[0,0,1]=(-0.601113924997)*x[0]**o+(0.654303363817)*x[0]+(0.830774705813)*x[1]**o+(-0.28075712655)*x[1]
        s[0,1,0]=(0.623015609082)*x[0]**o+(-0.336106614274)*x[0]+(0.184004307562)*x[1]**o+(0.34459670048)*x[1]
        s[0,1,1]=(0.369244348185)*x[0]**o+(-0.340034156943)*x[0]+(-0.263728841525)*x[1]**o+(0.395362860504)*x[1]
        s[1,0,0]=(-0.128245343573)*x[0]**o+(0.321050080112)*x[0]+(-0.759828911874)*x[1]**o+(-0.302784065212)*x[1]
        s[1,0,1]=(0.831781567583)*x[0]**o+(-0.695072817532)*x[0]+(0.765831248069)*x[1]**o+(-0.769559204064)*x[1]
        s[1,1,0]=(0.0231358105158)*x[0]**o+(-0.954466637933)*x[0]+(0.662613084093)*x[1]**o+(0.0859686703063)*x[1]
        s[1,1,1]=(-0.521154903312)*x[0]**o+(-0.556433610043)*x[0]+(0.185456230585)*x[1]**o+(0.033977276066)*x[1]
        s[2,0,0]=(0.629879646753)*x[0]**o+(-0.65521892079)*x[0]+(-0.208826471274)*x[1]**o+(0.737149257339)*x[1]
        s[2,0,1]=(-0.736126565138)*x[0]**o+(0.804035681207)*x[0]+(0.498224979538)*x[1]**o+(-0.959078001162)*x[1]
        s[2,1,0]=(0.286852738029)*x[0]**o+(0.353266502016)*x[0]+(0.940928485256)*x[1]**o+(0.28469674407)*x[1]
        s[2,1,1]=(0.275719311131)*x[0]**o+(0.696627217856)*x[0]+(-0.502165548001)*x[1]**o+(-0.520059660264)*x[1]
        s[3,0,0]=(0.658599467166)*x[0]**o+(-0.0350827779091)*x[0]+(-0.523903710086)*x[1]**o+(-0.844505620942)*x[1]
        s[3,0,1]=(0.480789616648)*x[0]**o+(0.276061116127)*x[0]+(-0.859175724597)*x[1]**o+(0.244549318718)*x[1]
        s[3,1,0]=(-0.542810121335)*x[0]**o+(-0.390621513538)*x[0]+(-0.229917193592)*x[1]**o+(-0.431941808228)*x[1]
        s[3,1,1]=(-0.739568286087)*x[0]**o+(-0.225719095703)*x[0]+(-0.366632490383)*x[1]**o+(-0.342423278749)*x[1]
        s[4,0,0]=(-0.348948968234)*x[0]**o+(0.667848620388)*x[0]+(-0.796730440327)*x[1]**o+(-0.391964865505)*x[1]
        s[4,0,1]=(0.86356141782)*x[0]**o+(0.479677832086)*x[0]+(-0.847297501426)*x[1]**o+(0.184369750148)*x[1]
        s[4,1,0]=(0.239661221119)*x[0]**o+(-0.080303273937)*x[0]+(0.748873062459)*x[1]**o+(-0.330109046822)*x[1]
        s[4,1,1]=(0.24720891844)*x[0]**o+(0.955624455873)*x[0]+(0.916553579019)*x[1]**o+(-0.636220767461)*x[1]
        s[5,0,0]=(0.23234510739)*x[0]**o+(-0.164278159674)*x[0]+(0.0850813643445)*x[1]**o+(0.603719147569)*x[1]
        s[5,0,1]=(-0.678009784208)*x[0]**o+(-0.439176295358)*x[0]+(-0.766581373483)*x[1]**o+(-0.754215528635)*x[1]
        s[5,1,0]=(0.587134653445)*x[0]**o+(-0.0529124327833)*x[0]+(-0.106376381667)*x[1]**o+(-0.606613833859)*x[1]
        s[5,1,1]=(0.0165841802044)*x[0]**o+(0.229033641436)*x[0]+(-0.00243791930366)*x[1]**o+(0.405659544043)*x[1]
        ref[0,0,0]=(0.525970841909)*x_ref[0]**o+(-0.724518605704)*x_ref[0]+(0.0391713787427)*x_ref[1]**o+(0.64004594365)*x_ref[1]
        ref[0,0,1]=(-0.601113924997)*x_ref[0]**o+(0.654303363817)*x_ref[0]+(0.830774705813)*x_ref[1]**o+(-0.28075712655)*x_ref[1]
        ref[0,1,0]=(0.623015609082)*x_ref[0]**o+(-0.336106614274)*x_ref[0]+(0.184004307562)*x_ref[1]**o+(0.34459670048)*x_ref[1]
        ref[0,1,1]=(0.369244348185)*x_ref[0]**o+(-0.340034156943)*x_ref[0]+(-0.263728841525)*x_ref[1]**o+(0.395362860504)*x_ref[1]
        ref[1,0,0]=(-0.128245343573)*x_ref[0]**o+(0.321050080112)*x_ref[0]+(-0.759828911874)*x_ref[1]**o+(-0.302784065212)*x_ref[1]
        ref[1,0,1]=(0.831781567583)*x_ref[0]**o+(-0.695072817532)*x_ref[0]+(0.765831248069)*x_ref[1]**o+(-0.769559204064)*x_ref[1]
        ref[1,1,0]=(0.0231358105158)*x_ref[0]**o+(-0.954466637933)*x_ref[0]+(0.662613084093)*x_ref[1]**o+(0.0859686703063)*x_ref[1]
        ref[1,1,1]=(-0.521154903312)*x_ref[0]**o+(-0.556433610043)*x_ref[0]+(0.185456230585)*x_ref[1]**o+(0.033977276066)*x_ref[1]
        ref[2,0,0]=(0.629879646753)*x_ref[0]**o+(-0.65521892079)*x_ref[0]+(-0.208826471274)*x_ref[1]**o+(0.737149257339)*x_ref[1]
        ref[2,0,1]=(-0.736126565138)*x_ref[0]**o+(0.804035681207)*x_ref[0]+(0.498224979538)*x_ref[1]**o+(-0.959078001162)*x_ref[1]
        ref[2,1,0]=(0.286852738029)*x_ref[0]**o+(0.353266502016)*x_ref[0]+(0.940928485256)*x_ref[1]**o+(0.28469674407)*x_ref[1]
        ref[2,1,1]=(0.275719311131)*x_ref[0]**o+(0.696627217856)*x_ref[0]+(-0.502165548001)*x_ref[1]**o+(-0.520059660264)*x_ref[1]
        ref[3,0,0]=(0.658599467166)*x_ref[0]**o+(-0.0350827779091)*x_ref[0]+(-0.523903710086)*x_ref[1]**o+(-0.844505620942)*x_ref[1]
        ref[3,0,1]=(0.480789616648)*x_ref[0]**o+(0.276061116127)*x_ref[0]+(-0.859175724597)*x_ref[1]**o+(0.244549318718)*x_ref[1]
        ref[3,1,0]=(-0.542810121335)*x_ref[0]**o+(-0.390621513538)*x_ref[0]+(-0.229917193592)*x_ref[1]**o+(-0.431941808228)*x_ref[1]
        ref[3,1,1]=(-0.739568286087)*x_ref[0]**o+(-0.225719095703)*x_ref[0]+(-0.366632490383)*x_ref[1]**o+(-0.342423278749)*x_ref[1]
        ref[4,0,0]=(-0.348948968234)*x_ref[0]**o+(0.667848620388)*x_ref[0]+(-0.796730440327)*x_ref[1]**o+(-0.391964865505)*x_ref[1]
        ref[4,0,1]=(0.86356141782)*x_ref[0]**o+(0.479677832086)*x_ref[0]+(-0.847297501426)*x_ref[1]**o+(0.184369750148)*x_ref[1]
        ref[4,1,0]=(0.239661221119)*x_ref[0]**o+(-0.080303273937)*x_ref[0]+(0.748873062459)*x_ref[1]**o+(-0.330109046822)*x_ref[1]
        ref[4,1,1]=(0.24720891844)*x_ref[0]**o+(0.955624455873)*x_ref[0]+(0.916553579019)*x_ref[1]**o+(-0.636220767461)*x_ref[1]
        ref[5,0,0]=(0.23234510739)*x_ref[0]**o+(-0.164278159674)*x_ref[0]+(0.0850813643445)*x_ref[1]**o+(0.603719147569)*x_ref[1]
        ref[5,0,1]=(-0.678009784208)*x_ref[0]**o+(-0.439176295358)*x_ref[0]+(-0.766581373483)*x_ref[1]**o+(-0.754215528635)*x_ref[1]
        ref[5,1,0]=(0.587134653445)*x_ref[0]**o+(-0.0529124327833)*x_ref[0]+(-0.106376381667)*x_ref[1]**o+(-0.606613833859)*x_ref[1]
        ref[5,1,1]=(0.0165841802044)*x_ref[0]**o+(0.229033641436)*x_ref[0]+(-0.00243791930366)*x_ref[1]**o+(0.405659544043)*x_ref[1]
      else:
        s[0,0,0]=(0.320018603048)*x[0]**o+(-0.695020280658)*x[0]+(-0.255721430929)*x[1]**o+(0.295594488722)*x[1]+(-0.0298772897116)*x[2]**o+(-0.607777786149)*x[2]
        s[0,0,1]=(-0.0539156052122)*x[0]**o+(0.538262769125)*x[0]+(-0.111309490507)*x[1]**o+(-0.793327760235)*x[1]+(-0.789135505737)*x[2]**o+(0.200901242923)*x[2]
        s[0,1,0]=(-0.336577387203)*x[0]**o+(0.210285897539)*x[0]+(0.699559016034)*x[1]**o+(-0.916159806945)*x[1]+(-0.499612048693)*x[2]**o+(0.358485383896)*x[2]
        s[0,1,1]=(-0.47730332075)*x[0]**o+(-0.689133971745)*x[0]+(0.765673037239)*x[1]**o+(0.792799541442)*x[1]+(0.230271541384)*x[2]**o+(0.984973655354)*x[2]
        s[1,0,0]=(-0.338473674404)*x[0]**o+(-0.935684983364)*x[0]+(-0.138901982164)*x[1]**o+(0.699572795452)*x[1]+(-0.199407330193)*x[2]**o+(-0.520709372829)*x[2]
        s[1,0,1]=(0.317819732778)*x[0]**o+(-0.845493146327)*x[0]+(0.905916805401)*x[1]**o+(0.432912349711)*x[1]+(-0.0895289935052)*x[2]**o+(0.585373526078)*x[2]
        s[1,1,0]=(0.946256506041)*x[0]**o+(0.579530928092)*x[0]+(-0.129820000715)*x[1]**o+(-0.80018614234)*x[1]+(-0.672241403531)*x[2]**o+(-0.429302115886)*x[2]
        s[1,1,1]=(0.841539001381)*x[0]**o+(0.91335238179)*x[0]+(-0.886657983812)*x[1]**o+(-0.714058529158)*x[1]+(-0.478059374968)*x[2]**o+(0.289902349534)*x[2]
        s[2,0,0]=(0.716479906976)*x[0]**o+(0.879172628545)*x[0]+(0.702298309498)*x[1]**o+(-0.111784591727)*x[1]+(0.40144635898)*x[2]**o+(-0.234783818652)*x[2]
        s[2,0,1]=(0.122997020314)*x[0]**o+(0.241190251335)*x[0]+(0.814657166415)*x[1]**o+(0.282560032124)*x[1]+(-0.0356499295276)*x[2]**o+(0.166315448017)*x[2]
        s[2,1,0]=(-0.881008094539)*x[0]**o+(0.542764181702)*x[0]+(0.242598400741)*x[1]**o+(0.993781493144)*x[1]+(-0.351534490163)*x[2]**o+(-0.104396569846)*x[2]
        s[2,1,1]=(-0.510128022031)*x[0]**o+(-0.215966730947)*x[0]+(0.393015726048)*x[1]**o+(-0.397942881289)*x[1]+(-0.195641507854)*x[2]**o+(0.266488215631)*x[2]
        s[3,0,0]=(-0.998764822575)*x[0]**o+(0.375416908499)*x[0]+(-0.874575087088)*x[1]**o+(0.0988447992659)*x[1]+(0.424141721707)*x[2]**o+(0.628127032413)*x[2]
        s[3,0,1]=(0.599442581541)*x[0]**o+(-0.618344049198)*x[0]+(-0.496506175349)*x[1]**o+(-0.481350180732)*x[1]+(-0.200782615612)*x[2]**o+(0.0201867652613)*x[2]
        s[3,1,0]=(-0.148594055575)*x[0]**o+(-0.806224651257)*x[0]+(-0.204482720049)*x[1]**o+(-0.49019313701)*x[1]+(0.335531499413)*x[2]**o+(-0.834246885478)*x[2]
        s[3,1,1]=(0.641941231569)*x[0]**o+(-0.90604061382)*x[0]+(0.540142802942)*x[1]**o+(0.72231278143)*x[1]+(-0.859060377851)*x[2]**o+(0.324573297095)*x[2]
        s[4,0,0]=(0.720107219881)*x[0]**o+(-0.62138573659)*x[0]+(-0.226330947503)*x[1]**o+(-0.624699498209)*x[1]+(0.539194640961)*x[2]**o+(0.852063240152)*x[2]
        s[4,0,1]=(0.0942892316041)*x[0]**o+(0.993373220728)*x[0]+(-0.0966784587501)*x[1]**o+(-0.683786013291)*x[1]+(-0.0361512012678)*x[2]**o+(-0.393598599894)*x[2]
        s[4,1,0]=(0.601453506151)*x[0]**o+(-0.307251826677)*x[0]+(0.966431194544)*x[1]**o+(0.244859574503)*x[1]+(-0.965624233728)*x[2]**o+(-0.354176955651)*x[2]
        s[4,1,1]=(0.30845293409)*x[0]**o+(0.485784959358)*x[0]+(-0.339346667554)*x[1]**o+(0.882000066104)*x[1]+(-0.548144598203)*x[2]**o+(-0.872275137718)*x[2]
        s[5,0,0]=(0.0264929806889)*x[0]**o+(-0.420817540591)*x[0]+(0.542643032455)*x[1]**o+(-0.509400295496)*x[1]+(-0.303533536712)*x[2]**o+(0.909439372034)*x[2]
        s[5,0,1]=(0.302318680709)*x[0]**o+(0.610414720074)*x[0]+(-0.253744196335)*x[1]**o+(0.0864740115897)*x[1]+(0.774183311916)*x[2]**o+(0.188320998163)*x[2]
        s[5,1,0]=(-0.341811432391)*x[0]**o+(-0.0568308040717)*x[0]+(0.807309619252)*x[1]**o+(0.947557389711)*x[1]+(-0.142294557146)*x[2]**o+(0.345071619119)*x[2]
        s[5,1,1]=(-0.059329629623)*x[0]**o+(-0.767819910895)*x[0]+(-0.0992852685518)*x[1]**o+(0.118514635891)*x[1]+(0.584899765205)*x[2]**o+(-0.924652287665)*x[2]
        ref[0,0,0]=(0.320018603048)*x_ref[0]**o+(-0.695020280658)*x_ref[0]+(-0.255721430929)*x_ref[1]**o+(0.295594488722)*x_ref[1]+(-0.0298772897116)*x_ref[2]**o+(-0.607777786149)*x_ref[2]
        ref[0,0,1]=(-0.0539156052122)*x_ref[0]**o+(0.538262769125)*x_ref[0]+(-0.111309490507)*x_ref[1]**o+(-0.793327760235)*x_ref[1]+(-0.789135505737)*x_ref[2]**o+(0.200901242923)*x_ref[2]
        ref[0,1,0]=(-0.336577387203)*x_ref[0]**o+(0.210285897539)*x_ref[0]+(0.699559016034)*x_ref[1]**o+(-0.916159806945)*x_ref[1]+(-0.499612048693)*x_ref[2]**o+(0.358485383896)*x_ref[2]
        ref[0,1,1]=(-0.47730332075)*x_ref[0]**o+(-0.689133971745)*x_ref[0]+(0.765673037239)*x_ref[1]**o+(0.792799541442)*x_ref[1]+(0.230271541384)*x_ref[2]**o+(0.984973655354)*x_ref[2]
        ref[1,0,0]=(-0.338473674404)*x_ref[0]**o+(-0.935684983364)*x_ref[0]+(-0.138901982164)*x_ref[1]**o+(0.699572795452)*x_ref[1]+(-0.199407330193)*x_ref[2]**o+(-0.520709372829)*x_ref[2]
        ref[1,0,1]=(0.317819732778)*x_ref[0]**o+(-0.845493146327)*x_ref[0]+(0.905916805401)*x_ref[1]**o+(0.432912349711)*x_ref[1]+(-0.0895289935052)*x_ref[2]**o+(0.585373526078)*x_ref[2]
        ref[1,1,0]=(0.946256506041)*x_ref[0]**o+(0.579530928092)*x_ref[0]+(-0.129820000715)*x_ref[1]**o+(-0.80018614234)*x_ref[1]+(-0.672241403531)*x_ref[2]**o+(-0.429302115886)*x_ref[2]
        ref[1,1,1]=(0.841539001381)*x_ref[0]**o+(0.91335238179)*x_ref[0]+(-0.886657983812)*x_ref[1]**o+(-0.714058529158)*x_ref[1]+(-0.478059374968)*x_ref[2]**o+(0.289902349534)*x_ref[2]
        ref[2,0,0]=(0.716479906976)*x_ref[0]**o+(0.879172628545)*x_ref[0]+(0.702298309498)*x_ref[1]**o+(-0.111784591727)*x_ref[1]+(0.40144635898)*x_ref[2]**o+(-0.234783818652)*x_ref[2]
        ref[2,0,1]=(0.122997020314)*x_ref[0]**o+(0.241190251335)*x_ref[0]+(0.814657166415)*x_ref[1]**o+(0.282560032124)*x_ref[1]+(-0.0356499295276)*x_ref[2]**o+(0.166315448017)*x_ref[2]
        ref[2,1,0]=(-0.881008094539)*x_ref[0]**o+(0.542764181702)*x_ref[0]+(0.242598400741)*x_ref[1]**o+(0.993781493144)*x_ref[1]+(-0.351534490163)*x_ref[2]**o+(-0.104396569846)*x_ref[2]
        ref[2,1,1]=(-0.510128022031)*x_ref[0]**o+(-0.215966730947)*x_ref[0]+(0.393015726048)*x_ref[1]**o+(-0.397942881289)*x_ref[1]+(-0.195641507854)*x_ref[2]**o+(0.266488215631)*x_ref[2]
        ref[3,0,0]=(-0.998764822575)*x_ref[0]**o+(0.375416908499)*x_ref[0]+(-0.874575087088)*x_ref[1]**o+(0.0988447992659)*x_ref[1]+(0.424141721707)*x_ref[2]**o+(0.628127032413)*x_ref[2]
        ref[3,0,1]=(0.599442581541)*x_ref[0]**o+(-0.618344049198)*x_ref[0]+(-0.496506175349)*x_ref[1]**o+(-0.481350180732)*x_ref[1]+(-0.200782615612)*x_ref[2]**o+(0.0201867652613)*x_ref[2]
        ref[3,1,0]=(-0.148594055575)*x_ref[0]**o+(-0.806224651257)*x_ref[0]+(-0.204482720049)*x_ref[1]**o+(-0.49019313701)*x_ref[1]+(0.335531499413)*x_ref[2]**o+(-0.834246885478)*x_ref[2]
        ref[3,1,1]=(0.641941231569)*x_ref[0]**o+(-0.90604061382)*x_ref[0]+(0.540142802942)*x_ref[1]**o+(0.72231278143)*x_ref[1]+(-0.859060377851)*x_ref[2]**o+(0.324573297095)*x_ref[2]
        ref[4,0,0]=(0.720107219881)*x_ref[0]**o+(-0.62138573659)*x_ref[0]+(-0.226330947503)*x_ref[1]**o+(-0.624699498209)*x_ref[1]+(0.539194640961)*x_ref[2]**o+(0.852063240152)*x_ref[2]
        ref[4,0,1]=(0.0942892316041)*x_ref[0]**o+(0.993373220728)*x_ref[0]+(-0.0966784587501)*x_ref[1]**o+(-0.683786013291)*x_ref[1]+(-0.0361512012678)*x_ref[2]**o+(-0.393598599894)*x_ref[2]
        ref[4,1,0]=(0.601453506151)*x_ref[0]**o+(-0.307251826677)*x_ref[0]+(0.966431194544)*x_ref[1]**o+(0.244859574503)*x_ref[1]+(-0.965624233728)*x_ref[2]**o+(-0.354176955651)*x_ref[2]
        ref[4,1,1]=(0.30845293409)*x_ref[0]**o+(0.485784959358)*x_ref[0]+(-0.339346667554)*x_ref[1]**o+(0.882000066104)*x_ref[1]+(-0.548144598203)*x_ref[2]**o+(-0.872275137718)*x_ref[2]
        ref[5,0,0]=(0.0264929806889)*x_ref[0]**o+(-0.420817540591)*x_ref[0]+(0.542643032455)*x_ref[1]**o+(-0.509400295496)*x_ref[1]+(-0.303533536712)*x_ref[2]**o+(0.909439372034)*x_ref[2]
        ref[5,0,1]=(0.302318680709)*x_ref[0]**o+(0.610414720074)*x_ref[0]+(-0.253744196335)*x_ref[1]**o+(0.0864740115897)*x_ref[1]+(0.774183311916)*x_ref[2]**o+(0.188320998163)*x_ref[2]
        ref[5,1,0]=(-0.341811432391)*x_ref[0]**o+(-0.0568308040717)*x_ref[0]+(0.807309619252)*x_ref[1]**o+(0.947557389711)*x_ref[1]+(-0.142294557146)*x_ref[2]**o+(0.345071619119)*x_ref[2]
        ref[5,1,1]=(-0.059329629623)*x_ref[0]**o+(-0.767819910895)*x_ref[0]+(-0.0992852685518)*x_ref[1]**o+(0.118514635891)*x_ref[1]+(0.584899765205)*x_ref[2]**o+(-0.924652287665)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_interpolate_onReducedFunctionOnContactOne_fromSymbol_ReducedFunctionOnContactOne_rank4(self):
      """
      tests interpolate for rank 4 Symbol onto the ReducedFunctionOnContactOne

      assumptions: self.domain supports inpterpolation from ReducedFunctionOnContactOne onto ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedFunctionOnContactOne(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5, 3, 2),dim=dim)
      s=Data(0,(4, 5, 3, 2),w)
      ref=Data(0,(4, 5, 3, 2),w_ref)
      if dim==2:
        s[0,0,0,0]=(0.286414445959)*x[0]**o+(-0.513602739404)*x[0]+(0.415115217751)*x[1]**o+(0.302847208656)*x[1]
        s[0,0,0,1]=(0.259932949392)*x[0]**o+(0.35051585712)*x[0]+(-0.199459717408)*x[1]**o+(0.243907820471)*x[1]
        s[0,0,1,0]=(-0.118977243299)*x[0]**o+(-0.42706283684)*x[0]+(0.538165751605)*x[1]**o+(0.393481400689)*x[1]
        s[0,0,1,1]=(-0.0397069035535)*x[0]**o+(0.861872317511)*x[0]+(-0.15731307612)*x[1]**o+(0.693002169125)*x[1]
        s[0,0,2,0]=(0.815645799181)*x[0]**o+(-0.0776314502273)*x[0]+(-0.593034585971)*x[1]**o+(-0.970680448911)*x[1]
        s[0,0,2,1]=(-0.952302578078)*x[0]**o+(0.651928073409)*x[0]+(-0.239714652084)*x[1]**o+(-0.952274708749)*x[1]
        s[0,1,0,0]=(-0.316151000005)*x[0]**o+(0.216061626502)*x[0]+(-0.882718070647)*x[1]**o+(-0.671547751668)*x[1]
        s[0,1,0,1]=(-0.399909471112)*x[0]**o+(-0.318040874639)*x[0]+(-0.907464404526)*x[1]**o+(0.233304604462)*x[1]
        s[0,1,1,0]=(0.774423529257)*x[0]**o+(0.364389715996)*x[0]+(-0.842201054073)*x[1]**o+(0.404586304258)*x[1]
        s[0,1,1,1]=(0.504496704492)*x[0]**o+(0.622221083753)*x[0]+(0.12478240068)*x[1]**o+(0.353949438798)*x[1]
        s[0,1,2,0]=(-0.845828936398)*x[0]**o+(-0.165296386039)*x[0]+(0.486358102378)*x[1]**o+(-0.761128414131)*x[1]
        s[0,1,2,1]=(-0.587533689601)*x[0]**o+(0.889232467387)*x[0]+(-0.17030097273)*x[1]**o+(-0.939366655195)*x[1]
        s[0,2,0,0]=(-0.583544977678)*x[0]**o+(0.49701340766)*x[0]+(-0.59251363345)*x[1]**o+(0.632063421618)*x[1]
        s[0,2,0,1]=(0.159662476533)*x[0]**o+(-0.926024134426)*x[0]+(-0.359267295402)*x[1]**o+(0.575272568112)*x[1]
        s[0,2,1,0]=(-0.737677964249)*x[0]**o+(0.828179798925)*x[0]+(0.487366022336)*x[1]**o+(-0.80790431335)*x[1]
        s[0,2,1,1]=(-0.633849216356)*x[0]**o+(0.127289254283)*x[0]+(0.282991845718)*x[1]**o+(0.821227139666)*x[1]
        s[0,2,2,0]=(-0.821316223164)*x[0]**o+(0.979879665311)*x[0]+(-0.137787756007)*x[1]**o+(-0.277486407169)*x[1]
        s[0,2,2,1]=(-0.622980962419)*x[0]**o+(-0.696479459661)*x[0]+(-0.410742216866)*x[1]**o+(0.576030773935)*x[1]
        s[0,3,0,0]=(-0.207619411473)*x[0]**o+(-0.469849947705)*x[0]+(-0.671957494397)*x[1]**o+(0.254338352433)*x[1]
        s[0,3,0,1]=(0.243280252346)*x[0]**o+(0.133824860089)*x[0]+(0.0696460542307)*x[1]**o+(0.455743386064)*x[1]
        s[0,3,1,0]=(-0.906691692651)*x[0]**o+(0.787810130121)*x[0]+(-0.219233151952)*x[1]**o+(0.743750472953)*x[1]
        s[0,3,1,1]=(-0.250516865682)*x[0]**o+(-0.780254096438)*x[0]+(-0.739910422511)*x[1]**o+(-0.792068328341)*x[1]
        s[0,3,2,0]=(0.0232686870041)*x[0]**o+(0.0106534323439)*x[0]+(-0.0361419376311)*x[1]**o+(0.566265323633)*x[1]
        s[0,3,2,1]=(-0.638354270703)*x[0]**o+(-0.255988140752)*x[0]+(-0.261309974545)*x[1]**o+(-0.697773476841)*x[1]
        s[0,4,0,0]=(-0.381395765807)*x[0]**o+(-0.365270978705)*x[0]+(0.812150815593)*x[1]**o+(0.719382395957)*x[1]
        s[0,4,0,1]=(0.69594272856)*x[0]**o+(0.278887033989)*x[0]+(0.754272027542)*x[1]**o+(0.793061109177)*x[1]
        s[0,4,1,0]=(0.967293155949)*x[0]**o+(-0.840326330307)*x[0]+(0.299320013494)*x[1]**o+(0.319888429035)*x[1]
        s[0,4,1,1]=(-0.0509734884321)*x[0]**o+(-0.936136061318)*x[0]+(-0.98628193626)*x[1]**o+(0.661193529996)*x[1]
        s[0,4,2,0]=(0.562813543077)*x[0]**o+(0.907184253714)*x[0]+(-0.136123729129)*x[1]**o+(-0.986466864564)*x[1]
        s[0,4,2,1]=(-0.434457154258)*x[0]**o+(-0.574859203931)*x[0]+(-0.383505033613)*x[1]**o+(0.331906199764)*x[1]
        s[1,0,0,0]=(0.0611272547169)*x[0]**o+(-0.136522073182)*x[0]+(0.417326897823)*x[1]**o+(-0.890085105585)*x[1]
        s[1,0,0,1]=(-0.869426624414)*x[0]**o+(-0.186222609222)*x[0]+(0.339956750726)*x[1]**o+(-0.226819210334)*x[1]
        s[1,0,1,0]=(0.399952324098)*x[0]**o+(0.798209020313)*x[0]+(-0.521148352596)*x[1]**o+(0.458003374007)*x[1]
        s[1,0,1,1]=(0.501460721237)*x[0]**o+(0.231248731021)*x[0]+(0.332539535549)*x[1]**o+(0.185347952715)*x[1]
        s[1,0,2,0]=(-0.230801196498)*x[0]**o+(0.823765804775)*x[0]+(-0.258377195949)*x[1]**o+(0.224645264709)*x[1]
        s[1,0,2,1]=(-0.241372395526)*x[0]**o+(0.905482005475)*x[0]+(0.0493139366957)*x[1]**o+(0.541602080684)*x[1]
        s[1,1,0,0]=(0.449706009608)*x[0]**o+(0.359879546784)*x[0]+(-0.493158517575)*x[1]**o+(0.191770207113)*x[1]
        s[1,1,0,1]=(0.957438236069)*x[0]**o+(-0.59909754527)*x[0]+(-0.614098873549)*x[1]**o+(0.0834466945634)*x[1]
        s[1,1,1,0]=(0.146023068688)*x[0]**o+(-0.185385831893)*x[0]+(-0.805313629764)*x[1]**o+(0.230766437006)*x[1]
        s[1,1,1,1]=(0.306071651757)*x[0]**o+(0.921518467891)*x[0]+(0.716057571126)*x[1]**o+(-0.522119477993)*x[1]
        s[1,1,2,0]=(0.684162336448)*x[0]**o+(-0.388731324307)*x[0]+(0.600372268011)*x[1]**o+(-0.771944030767)*x[1]
        s[1,1,2,1]=(0.286204844749)*x[0]**o+(0.400092667734)*x[0]+(0.506091236432)*x[1]**o+(0.858615249035)*x[1]
        s[1,2,0,0]=(-0.786850947851)*x[0]**o+(0.0317031686489)*x[0]+(-0.0562719978878)*x[1]**o+(-0.368745220216)*x[1]
        s[1,2,0,1]=(-0.062898974467)*x[0]**o+(-0.397876556176)*x[0]+(-0.117046688506)*x[1]**o+(-0.548475481806)*x[1]
        s[1,2,1,0]=(-0.478254619878)*x[0]**o+(0.866928591218)*x[0]+(0.4475948677)*x[1]**o+(0.393318603064)*x[1]
        s[1,2,1,1]=(-0.351975278856)*x[0]**o+(-0.377161089123)*x[0]+(0.492009299907)*x[1]**o+(0.569876619926)*x[1]
        s[1,2,2,0]=(-0.556131924542)*x[0]**o+(0.603608830074)*x[0]+(0.409486637292)*x[1]**o+(0.796280414552)*x[1]
        s[1,2,2,1]=(0.52135916937)*x[0]**o+(0.760618210549)*x[0]+(-0.145180244853)*x[1]**o+(0.549313491191)*x[1]
        s[1,3,0,0]=(-0.577089151087)*x[0]**o+(0.821891072229)*x[0]+(0.332425052183)*x[1]**o+(-0.151504483932)*x[1]
        s[1,3,0,1]=(-0.966703491954)*x[0]**o+(0.13997287848)*x[0]+(-0.682812558867)*x[1]**o+(-0.465013684448)*x[1]
        s[1,3,1,0]=(0.908453154364)*x[0]**o+(0.267997575637)*x[0]+(0.587549759517)*x[1]**o+(-0.487274402304)*x[1]
        s[1,3,1,1]=(-0.38747875298)*x[0]**o+(-0.685908980542)*x[0]+(-0.350387154367)*x[1]**o+(0.621711654152)*x[1]
        s[1,3,2,0]=(-0.148266632644)*x[0]**o+(-0.580283160892)*x[0]+(-0.261751144414)*x[1]**o+(0.502757383658)*x[1]
        s[1,3,2,1]=(0.610418736064)*x[0]**o+(0.591832308717)*x[0]+(0.869463161016)*x[1]**o+(0.252614326127)*x[1]
        s[1,4,0,0]=(0.683334561136)*x[0]**o+(0.484163667966)*x[0]+(0.254191241985)*x[1]**o+(0.039976829904)*x[1]
        s[1,4,0,1]=(-0.79220316614)*x[0]**o+(0.749008242295)*x[0]+(-0.539372369578)*x[1]**o+(-0.900518079782)*x[1]
        s[1,4,1,0]=(0.576352091657)*x[0]**o+(0.237846036342)*x[0]+(0.000667246779082)*x[1]**o+(-0.709742732387)*x[1]
        s[1,4,1,1]=(-0.676810333712)*x[0]**o+(0.847889875816)*x[0]+(-0.688138508533)*x[1]**o+(0.133158683641)*x[1]
        s[1,4,2,0]=(0.99415007686)*x[0]**o+(-0.141452245889)*x[0]+(0.775573627086)*x[1]**o+(0.452458721895)*x[1]
        s[1,4,2,1]=(0.994145376118)*x[0]**o+(0.395589592556)*x[0]+(-0.669488954253)*x[1]**o+(0.067046815669)*x[1]
        s[2,0,0,0]=(-0.750281322814)*x[0]**o+(-0.309320623684)*x[0]+(0.626517958661)*x[1]**o+(0.597498890162)*x[1]
        s[2,0,0,1]=(0.271076040325)*x[0]**o+(0.667103080861)*x[0]+(0.422066039225)*x[1]**o+(0.363852331279)*x[1]
        s[2,0,1,0]=(-0.273421011656)*x[0]**o+(-0.0619409465417)*x[0]+(0.0679232143441)*x[1]**o+(-0.720428826448)*x[1]
        s[2,0,1,1]=(-0.647493736809)*x[0]**o+(0.927792745023)*x[0]+(0.888263709553)*x[1]**o+(-0.613668040672)*x[1]
        s[2,0,2,0]=(0.576488068023)*x[0]**o+(0.389605065338)*x[0]+(0.273385863145)*x[1]**o+(0.528205453441)*x[1]
        s[2,0,2,1]=(-0.906661332702)*x[0]**o+(0.0725964985998)*x[0]+(0.184026573733)*x[1]**o+(0.72102096328)*x[1]
        s[2,1,0,0]=(0.0561237068662)*x[0]**o+(0.705088464117)*x[0]+(-0.92649218582)*x[1]**o+(-0.392584665998)*x[1]
        s[2,1,0,1]=(0.590410488233)*x[0]**o+(-0.599541782377)*x[0]+(0.739195238446)*x[1]**o+(-0.388992225425)*x[1]
        s[2,1,1,0]=(0.382347893509)*x[0]**o+(0.65529075645)*x[0]+(0.6281938054)*x[1]**o+(0.122434736283)*x[1]
        s[2,1,1,1]=(-0.286086668538)*x[0]**o+(-0.104121795484)*x[0]+(-0.282055217155)*x[1]**o+(0.173105945475)*x[1]
        s[2,1,2,0]=(-0.287280683011)*x[0]**o+(-0.963651344223)*x[0]+(-0.940112522168)*x[1]**o+(-0.788180434371)*x[1]
        s[2,1,2,1]=(0.61738130828)*x[0]**o+(-0.427708968146)*x[0]+(0.319738892701)*x[1]**o+(0.419063299626)*x[1]
        s[2,2,0,0]=(-0.0499926543896)*x[0]**o+(0.01377519232)*x[0]+(-0.325826729461)*x[1]**o+(-0.392085884375)*x[1]
        s[2,2,0,1]=(-0.0581765276804)*x[0]**o+(-0.208263386025)*x[0]+(0.75198359812)*x[1]**o+(0.97252999759)*x[1]
        s[2,2,1,0]=(-0.726087111179)*x[0]**o+(-0.0573158936919)*x[0]+(-0.742825725839)*x[1]**o+(0.415476090214)*x[1]
        s[2,2,1,1]=(0.917838486533)*x[0]**o+(-0.091211527759)*x[0]+(0.507645127218)*x[1]**o+(-0.742224440501)*x[1]
        s[2,2,2,0]=(-0.610881095534)*x[0]**o+(-0.864358502475)*x[0]+(0.0808508216725)*x[1]**o+(0.199006479357)*x[1]
        s[2,2,2,1]=(-0.946103419369)*x[0]**o+(0.491509709702)*x[0]+(0.612215069813)*x[1]**o+(0.418509110447)*x[1]
        s[2,3,0,0]=(0.0741360472802)*x[0]**o+(-0.994432396766)*x[0]+(-0.332924771563)*x[1]**o+(-0.753108051656)*x[1]
        s[2,3,0,1]=(-0.381576097006)*x[0]**o+(-0.377802667887)*x[0]+(-0.0303870681199)*x[1]**o+(-0.345561266945)*x[1]
        s[2,3,1,0]=(0.642729104556)*x[0]**o+(0.570036794833)*x[0]+(-0.868604403383)*x[1]**o+(-0.671162161412)*x[1]
        s[2,3,1,1]=(0.335963731776)*x[0]**o+(-0.504092976502)*x[0]+(-0.980540079107)*x[1]**o+(0.354777629497)*x[1]
        s[2,3,2,0]=(-0.764056326006)*x[0]**o+(-0.416057085413)*x[0]+(-0.690778483271)*x[1]**o+(-0.760474373055)*x[1]
        s[2,3,2,1]=(0.901089622394)*x[0]**o+(0.13387449325)*x[0]+(0.0138079678955)*x[1]**o+(0.839892772097)*x[1]
        s[2,4,0,0]=(-0.650061300028)*x[0]**o+(-0.349615966399)*x[0]+(0.790768704094)*x[1]**o+(-0.42814753068)*x[1]
        s[2,4,0,1]=(-0.0418088165021)*x[0]**o+(0.150269895143)*x[0]+(-0.529846432011)*x[1]**o+(-0.224508824625)*x[1]
        s[2,4,1,0]=(-0.872715524736)*x[0]**o+(-0.968213567754)*x[0]+(-0.485442904703)*x[1]**o+(-0.201353147903)*x[1]
        s[2,4,1,1]=(0.807375853776)*x[0]**o+(-0.555765909033)*x[0]+(0.509490325151)*x[1]**o+(-0.0402998342759)*x[1]
        s[2,4,2,0]=(0.976103420432)*x[0]**o+(-0.31420590756)*x[0]+(0.965906601649)*x[1]**o+(0.996838317987)*x[1]
        s[2,4,2,1]=(0.927750446685)*x[0]**o+(-0.0669100154417)*x[0]+(0.329400978068)*x[1]**o+(0.673620795081)*x[1]
        s[3,0,0,0]=(0.0339658367621)*x[0]**o+(-0.69057366167)*x[0]+(-0.84841361493)*x[1]**o+(-0.793870889269)*x[1]
        s[3,0,0,1]=(0.580299117541)*x[0]**o+(0.711568306729)*x[0]+(0.68652281169)*x[1]**o+(-0.893997742165)*x[1]
        s[3,0,1,0]=(0.472119483925)*x[0]**o+(0.811060953315)*x[0]+(0.131977099466)*x[1]**o+(0.252305475595)*x[1]
        s[3,0,1,1]=(0.825801074332)*x[0]**o+(-0.494721582584)*x[0]+(0.926019323719)*x[1]**o+(-0.398271744425)*x[1]
        s[3,0,2,0]=(0.456913500128)*x[0]**o+(-0.17032765188)*x[0]+(0.758934382849)*x[1]**o+(-0.260113957998)*x[1]
        s[3,0,2,1]=(-0.903049904081)*x[0]**o+(0.0173063807709)*x[0]+(-0.374982617974)*x[1]**o+(0.334268331811)*x[1]
        s[3,1,0,0]=(-0.22949882827)*x[0]**o+(-0.373561413774)*x[0]+(0.790631265088)*x[1]**o+(-0.75812217101)*x[1]
        s[3,1,0,1]=(0.564879113215)*x[0]**o+(0.33779824151)*x[0]+(0.0367512619169)*x[1]**o+(0.441450704065)*x[1]
        s[3,1,1,0]=(-0.428927604938)*x[0]**o+(-0.523613773862)*x[0]+(0.29478120955)*x[1]**o+(-0.863628205314)*x[1]
        s[3,1,1,1]=(0.459167152825)*x[0]**o+(-0.641899803541)*x[0]+(-0.109951392233)*x[1]**o+(0.399265290858)*x[1]
        s[3,1,2,0]=(0.682558303558)*x[0]**o+(-0.137784370501)*x[0]+(-0.393326255016)*x[1]**o+(-0.678503687693)*x[1]
        s[3,1,2,1]=(-0.0809452179949)*x[0]**o+(0.235685309616)*x[0]+(0.974274730596)*x[1]**o+(-0.894536588546)*x[1]
        s[3,2,0,0]=(-0.0490472374331)*x[0]**o+(-0.473644556364)*x[0]+(-0.382189831802)*x[1]**o+(-0.22504837111)*x[1]
        s[3,2,0,1]=(0.997893391083)*x[0]**o+(0.749295222878)*x[0]+(-0.799214431894)*x[1]**o+(0.57167392758)*x[1]
        s[3,2,1,0]=(-0.064907181579)*x[0]**o+(0.766055162408)*x[0]+(-0.524105307308)*x[1]**o+(0.361103034438)*x[1]
        s[3,2,1,1]=(0.690748511868)*x[0]**o+(0.412961119969)*x[0]+(-0.795479866575)*x[1]**o+(-0.450712265218)*x[1]
        s[3,2,2,0]=(0.34363262378)*x[0]**o+(-0.244910370036)*x[0]+(-0.634877248387)*x[1]**o+(-0.472063405174)*x[1]
        s[3,2,2,1]=(0.246638832327)*x[0]**o+(0.914501562312)*x[0]+(0.751830459279)*x[1]**o+(-0.497325969345)*x[1]
        s[3,3,0,0]=(0.541075396481)*x[0]**o+(-0.634968284805)*x[0]+(-0.231672798016)*x[1]**o+(-0.34422126411)*x[1]
        s[3,3,0,1]=(-0.935640589712)*x[0]**o+(-0.375887818373)*x[0]+(-0.243360729986)*x[1]**o+(-0.0544717388717)*x[1]
        s[3,3,1,0]=(0.154524952927)*x[0]**o+(-0.0863136783351)*x[0]+(-0.653676660981)*x[1]**o+(-0.7454863091)*x[1]
        s[3,3,1,1]=(0.277106623801)*x[0]**o+(-0.913188289955)*x[0]+(-0.79285009308)*x[1]**o+(-0.18601350351)*x[1]
        s[3,3,2,0]=(-0.3323524862)*x[0]**o+(0.94012804832)*x[0]+(0.397258718522)*x[1]**o+(-0.933536176616)*x[1]
        s[3,3,2,1]=(0.969750554915)*x[0]**o+(-0.851907090663)*x[0]+(0.950203728487)*x[1]**o+(-0.712715265376)*x[1]
        s[3,4,0,0]=(0.980063789653)*x[0]**o+(0.935723410963)*x[0]+(0.386913924778)*x[1]**o+(0.180557664607)*x[1]
        s[3,4,0,1]=(0.931878041992)*x[0]**o+(0.88853807113)*x[0]+(-0.187838817545)*x[1]**o+(-0.0251337491704)*x[1]
        s[3,4,1,0]=(-0.479847354936)*x[0]**o+(-0.030224862129)*x[0]+(-0.541789127999)*x[1]**o+(0.240762955814)*x[1]
        s[3,4,1,1]=(-0.917202541548)*x[0]**o+(0.217703883458)*x[0]+(0.248901567101)*x[1]**o+(-0.130053826789)*x[1]
        s[3,4,2,0]=(0.719712207452)*x[0]**o+(-0.606674954161)*x[0]+(0.508851160098)*x[1]**o+(-0.160706213281)*x[1]
        s[3,4,2,1]=(-0.5855419209)*x[0]**o+(-0.689958074797)*x[0]+(-0.782999784933)*x[1]**o+(0.800207229965)*x[1]
        ref[0,0,0,0]=(0.286414445959)*x_ref[0]**o+(-0.513602739404)*x_ref[0]+(0.415115217751)*x_ref[1]**o+(0.302847208656)*x_ref[1]
        ref[0,0,0,1]=(0.259932949392)*x_ref[0]**o+(0.35051585712)*x_ref[0]+(-0.199459717408)*x_ref[1]**o+(0.243907820471)*x_ref[1]
        ref[0,0,1,0]=(-0.118977243299)*x_ref[0]**o+(-0.42706283684)*x_ref[0]+(0.538165751605)*x_ref[1]**o+(0.393481400689)*x_ref[1]
        ref[0,0,1,1]=(-0.0397069035535)*x_ref[0]**o+(0.861872317511)*x_ref[0]+(-0.15731307612)*x_ref[1]**o+(0.693002169125)*x_ref[1]
        ref[0,0,2,0]=(0.815645799181)*x_ref[0]**o+(-0.0776314502273)*x_ref[0]+(-0.593034585971)*x_ref[1]**o+(-0.970680448911)*x_ref[1]
        ref[0,0,2,1]=(-0.952302578078)*x_ref[0]**o+(0.651928073409)*x_ref[0]+(-0.239714652084)*x_ref[1]**o+(-0.952274708749)*x_ref[1]
        ref[0,1,0,0]=(-0.316151000005)*x_ref[0]**o+(0.216061626502)*x_ref[0]+(-0.882718070647)*x_ref[1]**o+(-0.671547751668)*x_ref[1]
        ref[0,1,0,1]=(-0.399909471112)*x_ref[0]**o+(-0.318040874639)*x_ref[0]+(-0.907464404526)*x_ref[1]**o+(0.233304604462)*x_ref[1]
        ref[0,1,1,0]=(0.774423529257)*x_ref[0]**o+(0.364389715996)*x_ref[0]+(-0.842201054073)*x_ref[1]**o+(0.404586304258)*x_ref[1]
        ref[0,1,1,1]=(0.504496704492)*x_ref[0]**o+(0.622221083753)*x_ref[0]+(0.12478240068)*x_ref[1]**o+(0.353949438798)*x_ref[1]
        ref[0,1,2,0]=(-0.845828936398)*x_ref[0]**o+(-0.165296386039)*x_ref[0]+(0.486358102378)*x_ref[1]**o+(-0.761128414131)*x_ref[1]
        ref[0,1,2,1]=(-0.587533689601)*x_ref[0]**o+(0.889232467387)*x_ref[0]+(-0.17030097273)*x_ref[1]**o+(-0.939366655195)*x_ref[1]
        ref[0,2,0,0]=(-0.583544977678)*x_ref[0]**o+(0.49701340766)*x_ref[0]+(-0.59251363345)*x_ref[1]**o+(0.632063421618)*x_ref[1]
        ref[0,2,0,1]=(0.159662476533)*x_ref[0]**o+(-0.926024134426)*x_ref[0]+(-0.359267295402)*x_ref[1]**o+(0.575272568112)*x_ref[1]
        ref[0,2,1,0]=(-0.737677964249)*x_ref[0]**o+(0.828179798925)*x_ref[0]+(0.487366022336)*x_ref[1]**o+(-0.80790431335)*x_ref[1]
        ref[0,2,1,1]=(-0.633849216356)*x_ref[0]**o+(0.127289254283)*x_ref[0]+(0.282991845718)*x_ref[1]**o+(0.821227139666)*x_ref[1]
        ref[0,2,2,0]=(-0.821316223164)*x_ref[0]**o+(0.979879665311)*x_ref[0]+(-0.137787756007)*x_ref[1]**o+(-0.277486407169)*x_ref[1]
        ref[0,2,2,1]=(-0.622980962419)*x_ref[0]**o+(-0.696479459661)*x_ref[0]+(-0.410742216866)*x_ref[1]**o+(0.576030773935)*x_ref[1]
        ref[0,3,0,0]=(-0.207619411473)*x_ref[0]**o+(-0.469849947705)*x_ref[0]+(-0.671957494397)*x_ref[1]**o+(0.254338352433)*x_ref[1]
        ref[0,3,0,1]=(0.243280252346)*x_ref[0]**o+(0.133824860089)*x_ref[0]+(0.0696460542307)*x_ref[1]**o+(0.455743386064)*x_ref[1]
        ref[0,3,1,0]=(-0.906691692651)*x_ref[0]**o+(0.787810130121)*x_ref[0]+(-0.219233151952)*x_ref[1]**o+(0.743750472953)*x_ref[1]
        ref[0,3,1,1]=(-0.250516865682)*x_ref[0]**o+(-0.780254096438)*x_ref[0]+(-0.739910422511)*x_ref[1]**o+(-0.792068328341)*x_ref[1]
        ref[0,3,2,0]=(0.0232686870041)*x_ref[0]**o+(0.0106534323439)*x_ref[0]+(-0.0361419376311)*x_ref[1]**o+(0.566265323633)*x_ref[1]
        ref[0,3,2,1]=(-0.638354270703)*x_ref[0]**o+(-0.255988140752)*x_ref[0]+(-0.261309974545)*x_ref[1]**o+(-0.697773476841)*x_ref[1]
        ref[0,4,0,0]=(-0.381395765807)*x_ref[0]**o+(-0.365270978705)*x_ref[0]+(0.812150815593)*x_ref[1]**o+(0.719382395957)*x_ref[1]
        ref[0,4,0,1]=(0.69594272856)*x_ref[0]**o+(0.278887033989)*x_ref[0]+(0.754272027542)*x_ref[1]**o+(0.793061109177)*x_ref[1]
        ref[0,4,1,0]=(0.967293155949)*x_ref[0]**o+(-0.840326330307)*x_ref[0]+(0.299320013494)*x_ref[1]**o+(0.319888429035)*x_ref[1]
        ref[0,4,1,1]=(-0.0509734884321)*x_ref[0]**o+(-0.936136061318)*x_ref[0]+(-0.98628193626)*x_ref[1]**o+(0.661193529996)*x_ref[1]
        ref[0,4,2,0]=(0.562813543077)*x_ref[0]**o+(0.907184253714)*x_ref[0]+(-0.136123729129)*x_ref[1]**o+(-0.986466864564)*x_ref[1]
        ref[0,4,2,1]=(-0.434457154258)*x_ref[0]**o+(-0.574859203931)*x_ref[0]+(-0.383505033613)*x_ref[1]**o+(0.331906199764)*x_ref[1]
        ref[1,0,0,0]=(0.0611272547169)*x_ref[0]**o+(-0.136522073182)*x_ref[0]+(0.417326897823)*x_ref[1]**o+(-0.890085105585)*x_ref[1]
        ref[1,0,0,1]=(-0.869426624414)*x_ref[0]**o+(-0.186222609222)*x_ref[0]+(0.339956750726)*x_ref[1]**o+(-0.226819210334)*x_ref[1]
        ref[1,0,1,0]=(0.399952324098)*x_ref[0]**o+(0.798209020313)*x_ref[0]+(-0.521148352596)*x_ref[1]**o+(0.458003374007)*x_ref[1]
        ref[1,0,1,1]=(0.501460721237)*x_ref[0]**o+(0.231248731021)*x_ref[0]+(0.332539535549)*x_ref[1]**o+(0.185347952715)*x_ref[1]
        ref[1,0,2,0]=(-0.230801196498)*x_ref[0]**o+(0.823765804775)*x_ref[0]+(-0.258377195949)*x_ref[1]**o+(0.224645264709)*x_ref[1]
        ref[1,0,2,1]=(-0.241372395526)*x_ref[0]**o+(0.905482005475)*x_ref[0]+(0.0493139366957)*x_ref[1]**o+(0.541602080684)*x_ref[1]
        ref[1,1,0,0]=(0.449706009608)*x_ref[0]**o+(0.359879546784)*x_ref[0]+(-0.493158517575)*x_ref[1]**o+(0.191770207113)*x_ref[1]
        ref[1,1,0,1]=(0.957438236069)*x_ref[0]**o+(-0.59909754527)*x_ref[0]+(-0.614098873549)*x_ref[1]**o+(0.0834466945634)*x_ref[1]
        ref[1,1,1,0]=(0.146023068688)*x_ref[0]**o+(-0.185385831893)*x_ref[0]+(-0.805313629764)*x_ref[1]**o+(0.230766437006)*x_ref[1]
        ref[1,1,1,1]=(0.306071651757)*x_ref[0]**o+(0.921518467891)*x_ref[0]+(0.716057571126)*x_ref[1]**o+(-0.522119477993)*x_ref[1]
        ref[1,1,2,0]=(0.684162336448)*x_ref[0]**o+(-0.388731324307)*x_ref[0]+(0.600372268011)*x_ref[1]**o+(-0.771944030767)*x_ref[1]
        ref[1,1,2,1]=(0.286204844749)*x_ref[0]**o+(0.400092667734)*x_ref[0]+(0.506091236432)*x_ref[1]**o+(0.858615249035)*x_ref[1]
        ref[1,2,0,0]=(-0.786850947851)*x_ref[0]**o+(0.0317031686489)*x_ref[0]+(-0.0562719978878)*x_ref[1]**o+(-0.368745220216)*x_ref[1]
        ref[1,2,0,1]=(-0.062898974467)*x_ref[0]**o+(-0.397876556176)*x_ref[0]+(-0.117046688506)*x_ref[1]**o+(-0.548475481806)*x_ref[1]
        ref[1,2,1,0]=(-0.478254619878)*x_ref[0]**o+(0.866928591218)*x_ref[0]+(0.4475948677)*x_ref[1]**o+(0.393318603064)*x_ref[1]
        ref[1,2,1,1]=(-0.351975278856)*x_ref[0]**o+(-0.377161089123)*x_ref[0]+(0.492009299907)*x_ref[1]**o+(0.569876619926)*x_ref[1]
        ref[1,2,2,0]=(-0.556131924542)*x_ref[0]**o+(0.603608830074)*x_ref[0]+(0.409486637292)*x_ref[1]**o+(0.796280414552)*x_ref[1]
        ref[1,2,2,1]=(0.52135916937)*x_ref[0]**o+(0.760618210549)*x_ref[0]+(-0.145180244853)*x_ref[1]**o+(0.549313491191)*x_ref[1]
        ref[1,3,0,0]=(-0.577089151087)*x_ref[0]**o+(0.821891072229)*x_ref[0]+(0.332425052183)*x_ref[1]**o+(-0.151504483932)*x_ref[1]
        ref[1,3,0,1]=(-0.966703491954)*x_ref[0]**o+(0.13997287848)*x_ref[0]+(-0.682812558867)*x_ref[1]**o+(-0.465013684448)*x_ref[1]
        ref[1,3,1,0]=(0.908453154364)*x_ref[0]**o+(0.267997575637)*x_ref[0]+(0.587549759517)*x_ref[1]**o+(-0.487274402304)*x_ref[1]
        ref[1,3,1,1]=(-0.38747875298)*x_ref[0]**o+(-0.685908980542)*x_ref[0]+(-0.350387154367)*x_ref[1]**o+(0.621711654152)*x_ref[1]
        ref[1,3,2,0]=(-0.148266632644)*x_ref[0]**o+(-0.580283160892)*x_ref[0]+(-0.261751144414)*x_ref[1]**o+(0.502757383658)*x_ref[1]
        ref[1,3,2,1]=(0.610418736064)*x_ref[0]**o+(0.591832308717)*x_ref[0]+(0.869463161016)*x_ref[1]**o+(0.252614326127)*x_ref[1]
        ref[1,4,0,0]=(0.683334561136)*x_ref[0]**o+(0.484163667966)*x_ref[0]+(0.254191241985)*x_ref[1]**o+(0.039976829904)*x_ref[1]
        ref[1,4,0,1]=(-0.79220316614)*x_ref[0]**o+(0.749008242295)*x_ref[0]+(-0.539372369578)*x_ref[1]**o+(-0.900518079782)*x_ref[1]
        ref[1,4,1,0]=(0.576352091657)*x_ref[0]**o+(0.237846036342)*x_ref[0]+(0.000667246779082)*x_ref[1]**o+(-0.709742732387)*x_ref[1]
        ref[1,4,1,1]=(-0.676810333712)*x_ref[0]**o+(0.847889875816)*x_ref[0]+(-0.688138508533)*x_ref[1]**o+(0.133158683641)*x_ref[1]
        ref[1,4,2,0]=(0.99415007686)*x_ref[0]**o+(-0.141452245889)*x_ref[0]+(0.775573627086)*x_ref[1]**o+(0.452458721895)*x_ref[1]
        ref[1,4,2,1]=(0.994145376118)*x_ref[0]**o+(0.395589592556)*x_ref[0]+(-0.669488954253)*x_ref[1]**o+(0.067046815669)*x_ref[1]
        ref[2,0,0,0]=(-0.750281322814)*x_ref[0]**o+(-0.309320623684)*x_ref[0]+(0.626517958661)*x_ref[1]**o+(0.597498890162)*x_ref[1]
        ref[2,0,0,1]=(0.271076040325)*x_ref[0]**o+(0.667103080861)*x_ref[0]+(0.422066039225)*x_ref[1]**o+(0.363852331279)*x_ref[1]
        ref[2,0,1,0]=(-0.273421011656)*x_ref[0]**o+(-0.0619409465417)*x_ref[0]+(0.0679232143441)*x_ref[1]**o+(-0.720428826448)*x_ref[1]
        ref[2,0,1,1]=(-0.647493736809)*x_ref[0]**o+(0.927792745023)*x_ref[0]+(0.888263709553)*x_ref[1]**o+(-0.613668040672)*x_ref[1]
        ref[2,0,2,0]=(0.576488068023)*x_ref[0]**o+(0.389605065338)*x_ref[0]+(0.273385863145)*x_ref[1]**o+(0.528205453441)*x_ref[1]
        ref[2,0,2,1]=(-0.906661332702)*x_ref[0]**o+(0.0725964985998)*x_ref[0]+(0.184026573733)*x_ref[1]**o+(0.72102096328)*x_ref[1]
        ref[2,1,0,0]=(0.0561237068662)*x_ref[0]**o+(0.705088464117)*x_ref[0]+(-0.92649218582)*x_ref[1]**o+(-0.392584665998)*x_ref[1]
        ref[2,1,0,1]=(0.590410488233)*x_ref[0]**o+(-0.599541782377)*x_ref[0]+(0.739195238446)*x_ref[1]**o+(-0.388992225425)*x_ref[1]
        ref[2,1,1,0]=(0.382347893509)*x_ref[0]**o+(0.65529075645)*x_ref[0]+(0.6281938054)*x_ref[1]**o+(0.122434736283)*x_ref[1]
        ref[2,1,1,1]=(-0.286086668538)*x_ref[0]**o+(-0.104121795484)*x_ref[0]+(-0.282055217155)*x_ref[1]**o+(0.173105945475)*x_ref[1]
        ref[2,1,2,0]=(-0.287280683011)*x_ref[0]**o+(-0.963651344223)*x_ref[0]+(-0.940112522168)*x_ref[1]**o+(-0.788180434371)*x_ref[1]
        ref[2,1,2,1]=(0.61738130828)*x_ref[0]**o+(-0.427708968146)*x_ref[0]+(0.319738892701)*x_ref[1]**o+(0.419063299626)*x_ref[1]
        ref[2,2,0,0]=(-0.0499926543896)*x_ref[0]**o+(0.01377519232)*x_ref[0]+(-0.325826729461)*x_ref[1]**o+(-0.392085884375)*x_ref[1]
        ref[2,2,0,1]=(-0.0581765276804)*x_ref[0]**o+(-0.208263386025)*x_ref[0]+(0.75198359812)*x_ref[1]**o+(0.97252999759)*x_ref[1]
        ref[2,2,1,0]=(-0.726087111179)*x_ref[0]**o+(-0.0573158936919)*x_ref[0]+(-0.742825725839)*x_ref[1]**o+(0.415476090214)*x_ref[1]
        ref[2,2,1,1]=(0.917838486533)*x_ref[0]**o+(-0.091211527759)*x_ref[0]+(0.507645127218)*x_ref[1]**o+(-0.742224440501)*x_ref[1]
        ref[2,2,2,0]=(-0.610881095534)*x_ref[0]**o+(-0.864358502475)*x_ref[0]+(0.0808508216725)*x_ref[1]**o+(0.199006479357)*x_ref[1]
        ref[2,2,2,1]=(-0.946103419369)*x_ref[0]**o+(0.491509709702)*x_ref[0]+(0.612215069813)*x_ref[1]**o+(0.418509110447)*x_ref[1]
        ref[2,3,0,0]=(0.0741360472802)*x_ref[0]**o+(-0.994432396766)*x_ref[0]+(-0.332924771563)*x_ref[1]**o+(-0.753108051656)*x_ref[1]
        ref[2,3,0,1]=(-0.381576097006)*x_ref[0]**o+(-0.377802667887)*x_ref[0]+(-0.0303870681199)*x_ref[1]**o+(-0.345561266945)*x_ref[1]
        ref[2,3,1,0]=(0.642729104556)*x_ref[0]**o+(0.570036794833)*x_ref[0]+(-0.868604403383)*x_ref[1]**o+(-0.671162161412)*x_ref[1]
        ref[2,3,1,1]=(0.335963731776)*x_ref[0]**o+(-0.504092976502)*x_ref[0]+(-0.980540079107)*x_ref[1]**o+(0.354777629497)*x_ref[1]
        ref[2,3,2,0]=(-0.764056326006)*x_ref[0]**o+(-0.416057085413)*x_ref[0]+(-0.690778483271)*x_ref[1]**o+(-0.760474373055)*x_ref[1]
        ref[2,3,2,1]=(0.901089622394)*x_ref[0]**o+(0.13387449325)*x_ref[0]+(0.0138079678955)*x_ref[1]**o+(0.839892772097)*x_ref[1]
        ref[2,4,0,0]=(-0.650061300028)*x_ref[0]**o+(-0.349615966399)*x_ref[0]+(0.790768704094)*x_ref[1]**o+(-0.42814753068)*x_ref[1]
        ref[2,4,0,1]=(-0.0418088165021)*x_ref[0]**o+(0.150269895143)*x_ref[0]+(-0.529846432011)*x_ref[1]**o+(-0.224508824625)*x_ref[1]
        ref[2,4,1,0]=(-0.872715524736)*x_ref[0]**o+(-0.968213567754)*x_ref[0]+(-0.485442904703)*x_ref[1]**o+(-0.201353147903)*x_ref[1]
        ref[2,4,1,1]=(0.807375853776)*x_ref[0]**o+(-0.555765909033)*x_ref[0]+(0.509490325151)*x_ref[1]**o+(-0.0402998342759)*x_ref[1]
        ref[2,4,2,0]=(0.976103420432)*x_ref[0]**o+(-0.31420590756)*x_ref[0]+(0.965906601649)*x_ref[1]**o+(0.996838317987)*x_ref[1]
        ref[2,4,2,1]=(0.927750446685)*x_ref[0]**o+(-0.0669100154417)*x_ref[0]+(0.329400978068)*x_ref[1]**o+(0.673620795081)*x_ref[1]
        ref[3,0,0,0]=(0.0339658367621)*x_ref[0]**o+(-0.69057366167)*x_ref[0]+(-0.84841361493)*x_ref[1]**o+(-0.793870889269)*x_ref[1]
        ref[3,0,0,1]=(0.580299117541)*x_ref[0]**o+(0.711568306729)*x_ref[0]+(0.68652281169)*x_ref[1]**o+(-0.893997742165)*x_ref[1]
        ref[3,0,1,0]=(0.472119483925)*x_ref[0]**o+(0.811060953315)*x_ref[0]+(0.131977099466)*x_ref[1]**o+(0.252305475595)*x_ref[1]
        ref[3,0,1,1]=(0.825801074332)*x_ref[0]**o+(-0.494721582584)*x_ref[0]+(0.926019323719)*x_ref[1]**o+(-0.398271744425)*x_ref[1]
        ref[3,0,2,0]=(0.456913500128)*x_ref[0]**o+(-0.17032765188)*x_ref[0]+(0.758934382849)*x_ref[1]**o+(-0.260113957998)*x_ref[1]
        ref[3,0,2,1]=(-0.903049904081)*x_ref[0]**o+(0.0173063807709)*x_ref[0]+(-0.374982617974)*x_ref[1]**o+(0.334268331811)*x_ref[1]
        ref[3,1,0,0]=(-0.22949882827)*x_ref[0]**o+(-0.373561413774)*x_ref[0]+(0.790631265088)*x_ref[1]**o+(-0.75812217101)*x_ref[1]
        ref[3,1,0,1]=(0.564879113215)*x_ref[0]**o+(0.33779824151)*x_ref[0]+(0.0367512619169)*x_ref[1]**o+(0.441450704065)*x_ref[1]
        ref[3,1,1,0]=(-0.428927604938)*x_ref[0]**o+(-0.523613773862)*x_ref[0]+(0.29478120955)*x_ref[1]**o+(-0.863628205314)*x_ref[1]
        ref[3,1,1,1]=(0.459167152825)*x_ref[0]**o+(-0.641899803541)*x_ref[0]+(-0.109951392233)*x_ref[1]**o+(0.399265290858)*x_ref[1]
        ref[3,1,2,0]=(0.682558303558)*x_ref[0]**o+(-0.137784370501)*x_ref[0]+(-0.393326255016)*x_ref[1]**o+(-0.678503687693)*x_ref[1]
        ref[3,1,2,1]=(-0.0809452179949)*x_ref[0]**o+(0.235685309616)*x_ref[0]+(0.974274730596)*x_ref[1]**o+(-0.894536588546)*x_ref[1]
        ref[3,2,0,0]=(-0.0490472374331)*x_ref[0]**o+(-0.473644556364)*x_ref[0]+(-0.382189831802)*x_ref[1]**o+(-0.22504837111)*x_ref[1]
        ref[3,2,0,1]=(0.997893391083)*x_ref[0]**o+(0.749295222878)*x_ref[0]+(-0.799214431894)*x_ref[1]**o+(0.57167392758)*x_ref[1]
        ref[3,2,1,0]=(-0.064907181579)*x_ref[0]**o+(0.766055162408)*x_ref[0]+(-0.524105307308)*x_ref[1]**o+(0.361103034438)*x_ref[1]
        ref[3,2,1,1]=(0.690748511868)*x_ref[0]**o+(0.412961119969)*x_ref[0]+(-0.795479866575)*x_ref[1]**o+(-0.450712265218)*x_ref[1]
        ref[3,2,2,0]=(0.34363262378)*x_ref[0]**o+(-0.244910370036)*x_ref[0]+(-0.634877248387)*x_ref[1]**o+(-0.472063405174)*x_ref[1]
        ref[3,2,2,1]=(0.246638832327)*x_ref[0]**o+(0.914501562312)*x_ref[0]+(0.751830459279)*x_ref[1]**o+(-0.497325969345)*x_ref[1]
        ref[3,3,0,0]=(0.541075396481)*x_ref[0]**o+(-0.634968284805)*x_ref[0]+(-0.231672798016)*x_ref[1]**o+(-0.34422126411)*x_ref[1]
        ref[3,3,0,1]=(-0.935640589712)*x_ref[0]**o+(-0.375887818373)*x_ref[0]+(-0.243360729986)*x_ref[1]**o+(-0.0544717388717)*x_ref[1]
        ref[3,3,1,0]=(0.154524952927)*x_ref[0]**o+(-0.0863136783351)*x_ref[0]+(-0.653676660981)*x_ref[1]**o+(-0.7454863091)*x_ref[1]
        ref[3,3,1,1]=(0.277106623801)*x_ref[0]**o+(-0.913188289955)*x_ref[0]+(-0.79285009308)*x_ref[1]**o+(-0.18601350351)*x_ref[1]
        ref[3,3,2,0]=(-0.3323524862)*x_ref[0]**o+(0.94012804832)*x_ref[0]+(0.397258718522)*x_ref[1]**o+(-0.933536176616)*x_ref[1]
        ref[3,3,2,1]=(0.969750554915)*x_ref[0]**o+(-0.851907090663)*x_ref[0]+(0.950203728487)*x_ref[1]**o+(-0.712715265376)*x_ref[1]
        ref[3,4,0,0]=(0.980063789653)*x_ref[0]**o+(0.935723410963)*x_ref[0]+(0.386913924778)*x_ref[1]**o+(0.180557664607)*x_ref[1]
        ref[3,4,0,1]=(0.931878041992)*x_ref[0]**o+(0.88853807113)*x_ref[0]+(-0.187838817545)*x_ref[1]**o+(-0.0251337491704)*x_ref[1]
        ref[3,4,1,0]=(-0.479847354936)*x_ref[0]**o+(-0.030224862129)*x_ref[0]+(-0.541789127999)*x_ref[1]**o+(0.240762955814)*x_ref[1]
        ref[3,4,1,1]=(-0.917202541548)*x_ref[0]**o+(0.217703883458)*x_ref[0]+(0.248901567101)*x_ref[1]**o+(-0.130053826789)*x_ref[1]
        ref[3,4,2,0]=(0.719712207452)*x_ref[0]**o+(-0.606674954161)*x_ref[0]+(0.508851160098)*x_ref[1]**o+(-0.160706213281)*x_ref[1]
        ref[3,4,2,1]=(-0.5855419209)*x_ref[0]**o+(-0.689958074797)*x_ref[0]+(-0.782999784933)*x_ref[1]**o+(0.800207229965)*x_ref[1]
      else:
        s[0,0,0,0]=(-0.0939552237867)*x[0]**o+(-0.233782296435)*x[0]+(0.272117493235)*x[1]**o+(0.938376635468)*x[1]+(0.661832811129)*x[2]**o+(0.282399882837)*x[2]
        s[0,0,0,1]=(0.236203034262)*x[0]**o+(-0.583700442678)*x[0]+(-0.870895994748)*x[1]**o+(0.192686915202)*x[1]+(-0.901022579657)*x[2]**o+(-0.329112379375)*x[2]
        s[0,0,1,0]=(-0.583068685335)*x[0]**o+(0.981024224765)*x[0]+(0.869746509265)*x[1]**o+(0.713473797745)*x[1]+(0.647842443595)*x[2]**o+(0.128105689437)*x[2]
        s[0,0,1,1]=(0.231954364537)*x[0]**o+(0.301876895567)*x[0]+(0.902392392762)*x[1]**o+(0.77394994345)*x[1]+(-0.617945530904)*x[2]**o+(0.596296526597)*x[2]
        s[0,0,2,0]=(-0.848859481595)*x[0]**o+(0.489664239901)*x[0]+(0.101756068383)*x[1]**o+(0.372022728544)*x[1]+(-0.0480662015594)*x[2]**o+(-0.140663588571)*x[2]
        s[0,0,2,1]=(-0.151813501072)*x[0]**o+(-0.0619139652224)*x[0]+(-0.2404019023)*x[1]**o+(0.186875111425)*x[1]+(0.870121943947)*x[2]**o+(-0.401842672093)*x[2]
        s[0,1,0,0]=(-0.862371891518)*x[0]**o+(-0.141519691948)*x[0]+(0.0218465584484)*x[1]**o+(0.474483154145)*x[1]+(0.00642588343357)*x[2]**o+(0.0937465696159)*x[2]
        s[0,1,0,1]=(0.957271234184)*x[0]**o+(0.181371031824)*x[0]+(0.325782983064)*x[1]**o+(-0.758554608166)*x[1]+(0.888493514738)*x[2]**o+(0.360604795131)*x[2]
        s[0,1,1,0]=(0.8810614634)*x[0]**o+(-0.891947928245)*x[0]+(-0.81840638286)*x[1]**o+(-0.338084999054)*x[1]+(-0.408571326721)*x[2]**o+(-0.675124681083)*x[2]
        s[0,1,1,1]=(-0.00418963259364)*x[0]**o+(0.0872512414907)*x[0]+(0.307397770057)*x[1]**o+(-0.784741288514)*x[1]+(-0.288826977888)*x[2]**o+(-0.079246885912)*x[2]
        s[0,1,2,0]=(-0.933072733018)*x[0]**o+(0.141983186826)*x[0]+(-0.512297440539)*x[1]**o+(-0.232926253673)*x[1]+(0.884581006836)*x[2]**o+(-0.196597583936)*x[2]
        s[0,1,2,1]=(-0.264256230447)*x[0]**o+(0.0633721777798)*x[0]+(0.798360353199)*x[1]**o+(-0.323014270481)*x[1]+(0.99252859322)*x[2]**o+(0.933710902797)*x[2]
        s[0,2,0,0]=(0.402384727981)*x[0]**o+(-0.600697619335)*x[0]+(0.0672711555798)*x[1]**o+(-0.905342068352)*x[1]+(0.576003188043)*x[2]**o+(0.136818654256)*x[2]
        s[0,2,0,1]=(0.335215866093)*x[0]**o+(-0.128245834791)*x[0]+(0.214262493314)*x[1]**o+(-0.278381637725)*x[1]+(0.506994464335)*x[2]**o+(-0.426267615741)*x[2]
        s[0,2,1,0]=(0.463302776547)*x[0]**o+(0.836336860793)*x[0]+(0.978118223846)*x[1]**o+(-0.299895814699)*x[1]+(0.816774597805)*x[2]**o+(0.561662450214)*x[2]
        s[0,2,1,1]=(-0.355099999892)*x[0]**o+(-0.867893817303)*x[0]+(0.318137264146)*x[1]**o+(0.996417293091)*x[1]+(0.190720206337)*x[2]**o+(-0.956547404476)*x[2]
        s[0,2,2,0]=(0.636419754289)*x[0]**o+(-0.490121483483)*x[0]+(0.0724785970005)*x[1]**o+(0.654051448577)*x[1]+(-0.682352816989)*x[2]**o+(-0.969295042468)*x[2]
        s[0,2,2,1]=(0.523182385281)*x[0]**o+(0.882794657228)*x[0]+(-0.258639599201)*x[1]**o+(0.870016686554)*x[1]+(0.810959873296)*x[2]**o+(-0.120051000895)*x[2]
        s[0,3,0,0]=(-0.142691620177)*x[0]**o+(-0.26145359895)*x[0]+(0.979353443033)*x[1]**o+(0.812390675638)*x[1]+(0.659244834954)*x[2]**o+(0.951080038113)*x[2]
        s[0,3,0,1]=(-0.969621517275)*x[0]**o+(-0.0841833729288)*x[0]+(0.183088818824)*x[1]**o+(-0.0637411180744)*x[1]+(0.70498987804)*x[2]**o+(0.464729030647)*x[2]
        s[0,3,1,0]=(-0.908811399013)*x[0]**o+(0.469705146462)*x[0]+(0.623185114304)*x[1]**o+(0.156348468944)*x[1]+(-0.0352444815103)*x[2]**o+(-0.0924055149797)*x[2]
        s[0,3,1,1]=(-0.804539460051)*x[0]**o+(-0.64073007907)*x[0]+(0.871919457046)*x[1]**o+(-0.446001955346)*x[1]+(-0.758572789631)*x[2]**o+(-0.943023339532)*x[2]
        s[0,3,2,0]=(0.76012858373)*x[0]**o+(-0.413461345613)*x[0]+(-0.636494843218)*x[1]**o+(0.0576409596138)*x[1]+(0.32356830303)*x[2]**o+(0.66336356169)*x[2]
        s[0,3,2,1]=(-0.682969408206)*x[0]**o+(-0.00378599179446)*x[0]+(0.139592931649)*x[1]**o+(0.230923755162)*x[1]+(-0.359747563013)*x[2]**o+(0.490767562994)*x[2]
        s[0,4,0,0]=(0.40294705204)*x[0]**o+(-0.725991341301)*x[0]+(-0.123299724766)*x[1]**o+(0.783062653249)*x[1]+(0.284875756512)*x[2]**o+(-0.479856616542)*x[2]
        s[0,4,0,1]=(0.671083818646)*x[0]**o+(0.475388684118)*x[0]+(-0.0415590406628)*x[1]**o+(0.523350160591)*x[1]+(-0.869125066985)*x[2]**o+(0.539765124009)*x[2]
        s[0,4,1,0]=(0.517796715592)*x[0]**o+(0.273319492606)*x[0]+(0.131964130241)*x[1]**o+(0.522704060425)*x[1]+(0.945735742195)*x[2]**o+(-0.927060768569)*x[2]
        s[0,4,1,1]=(-0.0877749093432)*x[0]**o+(-0.228856801496)*x[0]+(-0.918861499662)*x[1]**o+(0.783335087368)*x[1]+(0.731374866941)*x[2]**o+(-0.0169316582485)*x[2]
        s[0,4,2,0]=(-0.521914417723)*x[0]**o+(0.100272381296)*x[0]+(-0.37855972365)*x[1]**o+(-0.748174737428)*x[1]+(0.08343326291)*x[2]**o+(-0.284290813001)*x[2]
        s[0,4,2,1]=(-0.622347209837)*x[0]**o+(-0.737347404815)*x[0]+(0.973210808148)*x[1]**o+(0.34940344181)*x[1]+(0.917063169506)*x[2]**o+(0.40443560458)*x[2]
        s[1,0,0,0]=(0.906350070311)*x[0]**o+(-0.514349936373)*x[0]+(-0.0377819340766)*x[1]**o+(-0.868663863385)*x[1]+(-0.910618099331)*x[2]**o+(0.161682748869)*x[2]
        s[1,0,0,1]=(-0.525696726097)*x[0]**o+(-0.564534313649)*x[0]+(0.294413275375)*x[1]**o+(-0.032889914662)*x[1]+(-0.655188068906)*x[2]**o+(-0.544604687619)*x[2]
        s[1,0,1,0]=(0.982917203627)*x[0]**o+(0.23003920025)*x[0]+(0.53970317464)*x[1]**o+(0.488996625882)*x[1]+(0.155567762567)*x[2]**o+(-0.0769333810796)*x[2]
        s[1,0,1,1]=(-0.695407739098)*x[0]**o+(-0.528856183813)*x[0]+(-0.765981763723)*x[1]**o+(-0.747233410366)*x[1]+(-0.217961552142)*x[2]**o+(0.363922845031)*x[2]
        s[1,0,2,0]=(-0.508333136514)*x[0]**o+(-0.0531371629837)*x[0]+(0.147835213524)*x[1]**o+(0.811513270701)*x[1]+(-0.51987266682)*x[2]**o+(-0.840524592697)*x[2]
        s[1,0,2,1]=(0.522917930226)*x[0]**o+(-0.387982699235)*x[0]+(-0.716946321359)*x[1]**o+(0.440470433449)*x[1]+(0.841627656575)*x[2]**o+(0.635104516674)*x[2]
        s[1,1,0,0]=(0.484898204665)*x[0]**o+(-0.753933429744)*x[0]+(-0.275664031761)*x[1]**o+(-0.566814900411)*x[1]+(0.0612476985048)*x[2]**o+(0.437090771267)*x[2]
        s[1,1,0,1]=(-0.182638197905)*x[0]**o+(-0.778469897629)*x[0]+(-0.626963871394)*x[1]**o+(-0.400611638351)*x[1]+(0.895655277012)*x[2]**o+(0.0326121043684)*x[2]
        s[1,1,1,0]=(-0.916569622941)*x[0]**o+(-0.658275271075)*x[0]+(0.250794036795)*x[1]**o+(-0.233593794029)*x[1]+(-0.96809736282)*x[2]**o+(0.656072783817)*x[2]
        s[1,1,1,1]=(0.0171169428141)*x[0]**o+(0.860556263522)*x[0]+(0.345606647004)*x[1]**o+(0.564186697526)*x[1]+(0.588489057339)*x[2]**o+(-0.87110086452)*x[2]
        s[1,1,2,0]=(0.275896407623)*x[0]**o+(0.0122423763916)*x[0]+(0.107452743797)*x[1]**o+(-0.367492719113)*x[1]+(0.884317274048)*x[2]**o+(0.0908799334316)*x[2]
        s[1,1,2,1]=(0.642219111519)*x[0]**o+(-0.183742201216)*x[0]+(-0.327197867043)*x[1]**o+(0.322321062553)*x[1]+(-0.183074011204)*x[2]**o+(0.720021602264)*x[2]
        s[1,2,0,0]=(0.532332942294)*x[0]**o+(0.950878496192)*x[0]+(0.819174829388)*x[1]**o+(0.43412262514)*x[1]+(-0.166925240319)*x[2]**o+(-0.698173204203)*x[2]
        s[1,2,0,1]=(0.259909649155)*x[0]**o+(-0.264967838281)*x[0]+(-0.774874307838)*x[1]**o+(-0.275886773315)*x[1]+(0.801233942875)*x[2]**o+(0.27496052417)*x[2]
        s[1,2,1,0]=(-0.746564397549)*x[0]**o+(-0.690480530678)*x[0]+(0.114986469362)*x[1]**o+(-0.751415850669)*x[1]+(0.119951306293)*x[2]**o+(-0.665012834766)*x[2]
        s[1,2,1,1]=(-0.851672541025)*x[0]**o+(0.235181175896)*x[0]+(-0.62908061934)*x[1]**o+(-0.438672467151)*x[1]+(-0.761945783738)*x[2]**o+(0.580730895385)*x[2]
        s[1,2,2,0]=(0.676776403141)*x[0]**o+(-0.707505868325)*x[0]+(0.724421249054)*x[1]**o+(0.120766197116)*x[1]+(-0.760834341632)*x[2]**o+(-0.689464446912)*x[2]
        s[1,2,2,1]=(0.147028945539)*x[0]**o+(0.199909936204)*x[0]+(0.903168165102)*x[1]**o+(-0.46293792368)*x[1]+(-0.282517651615)*x[2]**o+(0.740619136229)*x[2]
        s[1,3,0,0]=(-0.87747323136)*x[0]**o+(-0.361656418039)*x[0]+(-0.14856291522)*x[1]**o+(0.474044532693)*x[1]+(0.138460936937)*x[2]**o+(-0.145182012384)*x[2]
        s[1,3,0,1]=(0.748334497747)*x[0]**o+(0.892498113089)*x[0]+(0.239113720175)*x[1]**o+(-0.762524411251)*x[1]+(-0.488407873495)*x[2]**o+(0.158979901061)*x[2]
        s[1,3,1,0]=(0.465350922376)*x[0]**o+(-0.654156421791)*x[0]+(0.878703330328)*x[1]**o+(0.537076882157)*x[1]+(0.389722314724)*x[2]**o+(-0.120902843496)*x[2]
        s[1,3,1,1]=(-0.153324789456)*x[0]**o+(-0.842917613229)*x[0]+(0.984971864624)*x[1]**o+(0.349269583591)*x[1]+(0.484775635349)*x[2]**o+(0.0123674100644)*x[2]
        s[1,3,2,0]=(-0.175935804533)*x[0]**o+(0.0654493735159)*x[0]+(0.726667315683)*x[1]**o+(0.979882137896)*x[1]+(0.543445934589)*x[2]**o+(-0.149142988507)*x[2]
        s[1,3,2,1]=(0.235188036633)*x[0]**o+(-0.383249536636)*x[0]+(-0.445575566142)*x[1]**o+(-0.819686823355)*x[1]+(0.170138244522)*x[2]**o+(-0.80311924596)*x[2]
        s[1,4,0,0]=(0.471387483772)*x[0]**o+(-0.591448975176)*x[0]+(-0.833833649412)*x[1]**o+(0.770775874052)*x[1]+(0.537769198907)*x[2]**o+(-0.646003815898)*x[2]
        s[1,4,0,1]=(-0.797500020228)*x[0]**o+(-0.136543576325)*x[0]+(0.11979987113)*x[1]**o+(-0.495999866876)*x[1]+(-0.611958471401)*x[2]**o+(-0.588202607164)*x[2]
        s[1,4,1,0]=(-0.550038387555)*x[0]**o+(0.0844321262213)*x[0]+(0.131005384757)*x[1]**o+(-0.515674980981)*x[1]+(-0.637043856082)*x[2]**o+(0.333864121142)*x[2]
        s[1,4,1,1]=(0.71740532023)*x[0]**o+(-0.868269127293)*x[0]+(0.323641764571)*x[1]**o+(0.820948811255)*x[1]+(0.799638086908)*x[2]**o+(-0.558360429849)*x[2]
        s[1,4,2,0]=(-0.366976697335)*x[0]**o+(-0.121083094264)*x[0]+(0.650559587113)*x[1]**o+(-0.400220219011)*x[1]+(-0.0785928535861)*x[2]**o+(-0.50420364324)*x[2]
        s[1,4,2,1]=(0.872011301914)*x[0]**o+(0.0300770125292)*x[0]+(-0.965040697782)*x[1]**o+(-0.373825300018)*x[1]+(-0.988463549572)*x[2]**o+(0.906686888358)*x[2]
        s[2,0,0,0]=(-0.644460598597)*x[0]**o+(0.69918562566)*x[0]+(0.0995801100352)*x[1]**o+(0.257114765725)*x[1]+(-0.172788858436)*x[2]**o+(-0.625310450189)*x[2]
        s[2,0,0,1]=(0.331738659918)*x[0]**o+(0.34134401217)*x[0]+(0.667229795214)*x[1]**o+(0.248779696936)*x[1]+(0.782703155201)*x[2]**o+(-0.0117889862376)*x[2]
        s[2,0,1,0]=(0.456825869656)*x[0]**o+(0.618685513764)*x[0]+(0.00330761288087)*x[1]**o+(-0.839303906537)*x[1]+(0.16840639247)*x[2]**o+(-0.177323734594)*x[2]
        s[2,0,1,1]=(0.543967300033)*x[0]**o+(0.537337400025)*x[0]+(-0.232374591177)*x[1]**o+(-0.163640094925)*x[1]+(0.146221483058)*x[2]**o+(0.833185534106)*x[2]
        s[2,0,2,0]=(-0.283419495517)*x[0]**o+(-0.0641847006529)*x[0]+(0.933228590846)*x[1]**o+(0.575529900563)*x[1]+(-0.634849100928)*x[2]**o+(-0.898678364957)*x[2]
        s[2,0,2,1]=(0.248202597053)*x[0]**o+(-0.395428705005)*x[0]+(-0.921778142592)*x[1]**o+(0.752699910606)*x[1]+(-0.208939925012)*x[2]**o+(0.252031969391)*x[2]
        s[2,1,0,0]=(0.543045279112)*x[0]**o+(-0.200796280468)*x[0]+(-0.850234504692)*x[1]**o+(-0.608668852127)*x[1]+(-0.773553907964)*x[2]**o+(-0.727318461167)*x[2]
        s[2,1,0,1]=(0.670138313505)*x[0]**o+(0.763313503204)*x[0]+(-0.712024856615)*x[1]**o+(-0.0810633888547)*x[1]+(-0.543339707954)*x[2]**o+(-0.0944422473469)*x[2]
        s[2,1,1,0]=(0.66812399426)*x[0]**o+(-0.00267070537766)*x[0]+(0.704373662345)*x[1]**o+(-0.778778632343)*x[1]+(-0.195017717355)*x[2]**o+(-0.718710830736)*x[2]
        s[2,1,1,1]=(-0.905514219789)*x[0]**o+(-0.899693862646)*x[0]+(-0.791063116492)*x[1]**o+(-0.470046803205)*x[1]+(0.954924769408)*x[2]**o+(0.36799445545)*x[2]
        s[2,1,2,0]=(0.363438950523)*x[0]**o+(-0.788582160746)*x[0]+(0.316623350464)*x[1]**o+(-0.621859877627)*x[1]+(-0.899933912274)*x[2]**o+(-0.663582380926)*x[2]
        s[2,1,2,1]=(0.70767884819)*x[0]**o+(-0.337197298469)*x[0]+(0.329833087038)*x[1]**o+(-0.767476122169)*x[1]+(0.813567450674)*x[2]**o+(-0.230739080738)*x[2]
        s[2,2,0,0]=(0.425629461437)*x[0]**o+(0.736062549141)*x[0]+(0.0766779443487)*x[1]**o+(-0.645685464512)*x[1]+(-0.563156862696)*x[2]**o+(-0.200669234455)*x[2]
        s[2,2,0,1]=(-0.436511105169)*x[0]**o+(-0.784209106822)*x[0]+(0.476448752447)*x[1]**o+(-0.976253075887)*x[1]+(0.744207251461)*x[2]**o+(-0.480585834629)*x[2]
        s[2,2,1,0]=(-0.160519965814)*x[0]**o+(-0.740586061373)*x[0]+(-0.291540051837)*x[1]**o+(0.0186903802094)*x[1]+(-0.456120795038)*x[2]**o+(0.930603860664)*x[2]
        s[2,2,1,1]=(0.835200574398)*x[0]**o+(-0.111618184052)*x[0]+(0.394793418949)*x[1]**o+(0.66759987425)*x[1]+(0.902913963474)*x[2]**o+(-0.840871310598)*x[2]
        s[2,2,2,0]=(-0.745841363439)*x[0]**o+(0.859714040441)*x[0]+(-0.133922374335)*x[1]**o+(-0.539065635521)*x[1]+(-0.0496309374348)*x[2]**o+(-0.3150513125)*x[2]
        s[2,2,2,1]=(-0.121952493914)*x[0]**o+(-0.145565953724)*x[0]+(-0.46956691252)*x[1]**o+(0.594587113542)*x[1]+(0.801503774207)*x[2]**o+(-0.559107902481)*x[2]
        s[2,3,0,0]=(0.107705603601)*x[0]**o+(0.83167538493)*x[0]+(0.719052082882)*x[1]**o+(-0.0418541572577)*x[1]+(-0.760679778738)*x[2]**o+(-0.269573603299)*x[2]
        s[2,3,0,1]=(-0.610961305034)*x[0]**o+(-0.48822367684)*x[0]+(-0.533146614698)*x[1]**o+(-0.0634033507577)*x[1]+(-0.525819529758)*x[2]**o+(0.70089075883)*x[2]
        s[2,3,1,0]=(0.503925514982)*x[0]**o+(0.65957769246)*x[0]+(-0.624626366929)*x[1]**o+(-0.374542181845)*x[1]+(-0.363387031742)*x[2]**o+(-0.00563190329728)*x[2]
        s[2,3,1,1]=(0.337750136392)*x[0]**o+(-0.364300637249)*x[0]+(0.0892494275431)*x[1]**o+(0.958228736151)*x[1]+(0.203359176466)*x[2]**o+(-0.454161162092)*x[2]
        s[2,3,2,0]=(0.171894913416)*x[0]**o+(0.930342123436)*x[0]+(0.888973400724)*x[1]**o+(0.732732070997)*x[1]+(-0.365536399448)*x[2]**o+(0.719027934816)*x[2]
        s[2,3,2,1]=(0.0718345473684)*x[0]**o+(-0.351767950331)*x[0]+(-0.427880627822)*x[1]**o+(0.906554327062)*x[1]+(0.209905901579)*x[2]**o+(0.343983972848)*x[2]
        s[2,4,0,0]=(0.20082437951)*x[0]**o+(0.411679233479)*x[0]+(-0.146683847376)*x[1]**o+(0.619164773872)*x[1]+(0.283942602042)*x[2]**o+(0.328844649922)*x[2]
        s[2,4,0,1]=(0.512237008877)*x[0]**o+(-0.0799414456365)*x[0]+(-0.444485238118)*x[1]**o+(-0.617601180088)*x[1]+(-0.522016560329)*x[2]**o+(0.764966540338)*x[2]
        s[2,4,1,0]=(-0.906295486351)*x[0]**o+(0.213043439406)*x[0]+(-0.841541923905)*x[1]**o+(0.613189806332)*x[1]+(0.98016601466)*x[2]**o+(0.373100080758)*x[2]
        s[2,4,1,1]=(0.421256899132)*x[0]**o+(-0.148027161979)*x[0]+(-0.436551670748)*x[1]**o+(0.15389527845)*x[1]+(-0.202772011213)*x[2]**o+(0.526456981868)*x[2]
        s[2,4,2,0]=(0.326695825605)*x[0]**o+(-0.658196467156)*x[0]+(-0.0229897453445)*x[1]**o+(0.640660125725)*x[1]+(0.604325801088)*x[2]**o+(0.415440861181)*x[2]
        s[2,4,2,1]=(-0.353746711731)*x[0]**o+(0.707902797414)*x[0]+(-0.117816706586)*x[1]**o+(0.767979696436)*x[1]+(0.0361750168523)*x[2]**o+(-0.904298176296)*x[2]
        s[3,0,0,0]=(0.274392483594)*x[0]**o+(-0.142144450689)*x[0]+(-0.308306223778)*x[1]**o+(0.674122485073)*x[1]+(0.259741082605)*x[2]**o+(-0.249672127213)*x[2]
        s[3,0,0,1]=(-0.299212636089)*x[0]**o+(0.670140835575)*x[0]+(0.0513019322721)*x[1]**o+(0.460049892726)*x[1]+(0.167967285754)*x[2]**o+(-0.463520967247)*x[2]
        s[3,0,1,0]=(0.850761515544)*x[0]**o+(0.945654906297)*x[0]+(-0.841039831472)*x[1]**o+(-0.051572515458)*x[1]+(0.750689246331)*x[2]**o+(0.0651065819687)*x[2]
        s[3,0,1,1]=(-0.387082102682)*x[0]**o+(-0.89541016885)*x[0]+(-0.903675915508)*x[1]**o+(-0.268304570988)*x[1]+(-0.259828889903)*x[2]**o+(-0.983650155837)*x[2]
        s[3,0,2,0]=(0.497841641089)*x[0]**o+(0.80522306204)*x[0]+(0.532637885608)*x[1]**o+(-0.345627398376)*x[1]+(-0.768962683025)*x[2]**o+(-0.608176054775)*x[2]
        s[3,0,2,1]=(0.00437007549865)*x[0]**o+(0.847511281463)*x[0]+(0.348033100721)*x[1]**o+(0.348640167731)*x[1]+(-0.482286157201)*x[2]**o+(0.23184848892)*x[2]
        s[3,1,0,0]=(0.79458009346)*x[0]**o+(0.346244735823)*x[0]+(0.274964941045)*x[1]**o+(0.306665745695)*x[1]+(0.752939751818)*x[2]**o+(-0.0212951945504)*x[2]
        s[3,1,0,1]=(0.279860784064)*x[0]**o+(0.728689070796)*x[0]+(0.112157508671)*x[1]**o+(-0.885233951967)*x[1]+(-0.469780984101)*x[2]**o+(0.177020475912)*x[2]
        s[3,1,1,0]=(0.433534890526)*x[0]**o+(0.485733515889)*x[0]+(0.830706912979)*x[1]**o+(0.118568290881)*x[1]+(-0.863800517187)*x[2]**o+(0.737792919447)*x[2]
        s[3,1,1,1]=(0.791784006945)*x[0]**o+(-0.112270136239)*x[0]+(0.874488179673)*x[1]**o+(-0.678555278645)*x[1]+(0.985727158563)*x[2]**o+(0.920137638515)*x[2]
        s[3,1,2,0]=(-0.961254804108)*x[0]**o+(0.243551301383)*x[0]+(-0.311248382242)*x[1]**o+(0.41289285623)*x[1]+(-0.371475600554)*x[2]**o+(-0.0582608041921)*x[2]
        s[3,1,2,1]=(-0.144445942195)*x[0]**o+(-0.418750582693)*x[0]+(-0.468333324126)*x[1]**o+(-0.658823898386)*x[1]+(-0.909121304085)*x[2]**o+(-0.600433830184)*x[2]
        s[3,2,0,0]=(-0.286554311888)*x[0]**o+(-0.737505598582)*x[0]+(0.566521914245)*x[1]**o+(0.498091322078)*x[1]+(0.0202811708798)*x[2]**o+(-0.792584826215)*x[2]
        s[3,2,0,1]=(-0.52573040544)*x[0]**o+(0.256454412031)*x[0]+(-0.771513393908)*x[1]**o+(0.0769633391923)*x[1]+(0.447325513976)*x[2]**o+(0.860889667608)*x[2]
        s[3,2,1,0]=(-0.776650147122)*x[0]**o+(-0.597663916146)*x[0]+(-0.56596948577)*x[1]**o+(0.606515796008)*x[1]+(-0.0569814692195)*x[2]**o+(0.0832162598646)*x[2]
        s[3,2,1,1]=(-0.702983139798)*x[0]**o+(-0.0721500323076)*x[0]+(-0.696083887377)*x[1]**o+(-0.00975653495795)*x[1]+(0.972659180196)*x[2]**o+(0.968527616515)*x[2]
        s[3,2,2,0]=(-0.861690620984)*x[0]**o+(0.169760353347)*x[0]+(-0.774337793749)*x[1]**o+(-0.799362859305)*x[1]+(-0.352790827791)*x[2]**o+(-0.31255981171)*x[2]
        s[3,2,2,1]=(0.264209810017)*x[0]**o+(-0.86560795678)*x[0]+(0.306794805255)*x[1]**o+(0.795495753342)*x[1]+(0.5631729589)*x[2]**o+(-0.901732966437)*x[2]
        s[3,3,0,0]=(0.151220066195)*x[0]**o+(0.294238555124)*x[0]+(0.725179461723)*x[1]**o+(0.823417422915)*x[1]+(0.826457989801)*x[2]**o+(-0.068783769033)*x[2]
        s[3,3,0,1]=(0.965148541899)*x[0]**o+(0.054903011166)*x[0]+(-0.564782216515)*x[1]**o+(-0.825732725252)*x[1]+(-0.172033069024)*x[2]**o+(0.657198365579)*x[2]
        s[3,3,1,0]=(0.695530033492)*x[0]**o+(-0.251400308058)*x[0]+(-0.379416854517)*x[1]**o+(-0.491666460242)*x[1]+(0.215166477617)*x[2]**o+(0.690685608974)*x[2]
        s[3,3,1,1]=(0.377513737173)*x[0]**o+(-0.0172594331468)*x[0]+(0.42799383164)*x[1]**o+(0.0607216253163)*x[1]+(-0.153084202648)*x[2]**o+(0.411927363219)*x[2]
        s[3,3,2,0]=(-0.125731371056)*x[0]**o+(0.33378895482)*x[0]+(-0.448245878593)*x[1]**o+(-0.764746615313)*x[1]+(0.181875287538)*x[2]**o+(-0.581992845032)*x[2]
        s[3,3,2,1]=(-0.584517106374)*x[0]**o+(0.793927805805)*x[0]+(0.300548289158)*x[1]**o+(0.0449525116422)*x[1]+(0.883866474871)*x[2]**o+(0.518036509563)*x[2]
        s[3,4,0,0]=(-0.685669036611)*x[0]**o+(-0.406759680831)*x[0]+(0.020288611842)*x[1]**o+(0.524437172569)*x[1]+(-0.377692301032)*x[2]**o+(-0.969562114494)*x[2]
        s[3,4,0,1]=(0.541982416337)*x[0]**o+(0.839468986455)*x[0]+(-0.542504618993)*x[1]**o+(-0.458056288842)*x[1]+(0.982937308016)*x[2]**o+(0.751237990644)*x[2]
        s[3,4,1,0]=(-0.623461568258)*x[0]**o+(0.469859301643)*x[0]+(0.841657608811)*x[1]**o+(0.819674909109)*x[1]+(-0.655140622922)*x[2]**o+(0.0824474652875)*x[2]
        s[3,4,1,1]=(0.906724250555)*x[0]**o+(0.14855445564)*x[0]+(-0.862379845699)*x[1]**o+(-0.853300535973)*x[1]+(-0.714581251705)*x[2]**o+(-0.956268149641)*x[2]
        s[3,4,2,0]=(-0.721603766365)*x[0]**o+(0.132835246572)*x[0]+(0.0527745535366)*x[1]**o+(-0.161464199846)*x[1]+(-0.551405664077)*x[2]**o+(-0.696496777295)*x[2]
        s[3,4,2,1]=(-0.215666532882)*x[0]**o+(0.145798314909)*x[0]+(0.547835453007)*x[1]**o+(0.6070128325)*x[1]+(0.179306259279)*x[2]**o+(-0.0234805628101)*x[2]
        ref[0,0,0,0]=(-0.0939552237867)*x_ref[0]**o+(-0.233782296435)*x_ref[0]+(0.272117493235)*x_ref[1]**o+(0.938376635468)*x_ref[1]+(0.661832811129)*x_ref[2]**o+(0.282399882837)*x_ref[2]
        ref[0,0,0,1]=(0.236203034262)*x_ref[0]**o+(-0.583700442678)*x_ref[0]+(-0.870895994748)*x_ref[1]**o+(0.192686915202)*x_ref[1]+(-0.901022579657)*x_ref[2]**o+(-0.329112379375)*x_ref[2]
        ref[0,0,1,0]=(-0.583068685335)*x_ref[0]**o+(0.981024224765)*x_ref[0]+(0.869746509265)*x_ref[1]**o+(0.713473797745)*x_ref[1]+(0.647842443595)*x_ref[2]**o+(0.128105689437)*x_ref[2]
        ref[0,0,1,1]=(0.231954364537)*x_ref[0]**o+(0.301876895567)*x_ref[0]+(0.902392392762)*x_ref[1]**o+(0.77394994345)*x_ref[1]+(-0.617945530904)*x_ref[2]**o+(0.596296526597)*x_ref[2]
        ref[0,0,2,0]=(-0.848859481595)*x_ref[0]**o+(0.489664239901)*x_ref[0]+(0.101756068383)*x_ref[1]**o+(0.372022728544)*x_ref[1]+(-0.0480662015594)*x_ref[2]**o+(-0.140663588571)*x_ref[2]
        ref[0,0,2,1]=(-0.151813501072)*x_ref[0]**o+(-0.0619139652224)*x_ref[0]+(-0.2404019023)*x_ref[1]**o+(0.186875111425)*x_ref[1]+(0.870121943947)*x_ref[2]**o+(-0.401842672093)*x_ref[2]
        ref[0,1,0,0]=(-0.862371891518)*x_ref[0]**o+(-0.141519691948)*x_ref[0]+(0.0218465584484)*x_ref[1]**o+(0.474483154145)*x_ref[1]+(0.00642588343357)*x_ref[2]**o+(0.0937465696159)*x_ref[2]
        ref[0,1,0,1]=(0.957271234184)*x_ref[0]**o+(0.181371031824)*x_ref[0]+(0.325782983064)*x_ref[1]**o+(-0.758554608166)*x_ref[1]+(0.888493514738)*x_ref[2]**o+(0.360604795131)*x_ref[2]
        ref[0,1,1,0]=(0.8810614634)*x_ref[0]**o+(-0.891947928245)*x_ref[0]+(-0.81840638286)*x_ref[1]**o+(-0.338084999054)*x_ref[1]+(-0.408571326721)*x_ref[2]**o+(-0.675124681083)*x_ref[2]
        ref[0,1,1,1]=(-0.00418963259364)*x_ref[0]**o+(0.0872512414907)*x_ref[0]+(0.307397770057)*x_ref[1]**o+(-0.784741288514)*x_ref[1]+(-0.288826977888)*x_ref[2]**o+(-0.079246885912)*x_ref[2]
        ref[0,1,2,0]=(-0.933072733018)*x_ref[0]**o+(0.141983186826)*x_ref[0]+(-0.512297440539)*x_ref[1]**o+(-0.232926253673)*x_ref[1]+(0.884581006836)*x_ref[2]**o+(-0.196597583936)*x_ref[2]
        ref[0,1,2,1]=(-0.264256230447)*x_ref[0]**o+(0.0633721777798)*x_ref[0]+(0.798360353199)*x_ref[1]**o+(-0.323014270481)*x_ref[1]+(0.99252859322)*x_ref[2]**o+(0.933710902797)*x_ref[2]
        ref[0,2,0,0]=(0.402384727981)*x_ref[0]**o+(-0.600697619335)*x_ref[0]+(0.0672711555798)*x_ref[1]**o+(-0.905342068352)*x_ref[1]+(0.576003188043)*x_ref[2]**o+(0.136818654256)*x_ref[2]
        ref[0,2,0,1]=(0.335215866093)*x_ref[0]**o+(-0.128245834791)*x_ref[0]+(0.214262493314)*x_ref[1]**o+(-0.278381637725)*x_ref[1]+(0.506994464335)*x_ref[2]**o+(-0.426267615741)*x_ref[2]
        ref[0,2,1,0]=(0.463302776547)*x_ref[0]**o+(0.836336860793)*x_ref[0]+(0.978118223846)*x_ref[1]**o+(-0.299895814699)*x_ref[1]+(0.816774597805)*x_ref[2]**o+(0.561662450214)*x_ref[2]
        ref[0,2,1,1]=(-0.355099999892)*x_ref[0]**o+(-0.867893817303)*x_ref[0]+(0.318137264146)*x_ref[1]**o+(0.996417293091)*x_ref[1]+(0.190720206337)*x_ref[2]**o+(-0.956547404476)*x_ref[2]
        ref[0,2,2,0]=(0.636419754289)*x_ref[0]**o+(-0.490121483483)*x_ref[0]+(0.0724785970005)*x_ref[1]**o+(0.654051448577)*x_ref[1]+(-0.682352816989)*x_ref[2]**o+(-0.969295042468)*x_ref[2]
        ref[0,2,2,1]=(0.523182385281)*x_ref[0]**o+(0.882794657228)*x_ref[0]+(-0.258639599201)*x_ref[1]**o+(0.870016686554)*x_ref[1]+(0.810959873296)*x_ref[2]**o+(-0.120051000895)*x_ref[2]
        ref[0,3,0,0]=(-0.142691620177)*x_ref[0]**o+(-0.26145359895)*x_ref[0]+(0.979353443033)*x_ref[1]**o+(0.812390675638)*x_ref[1]+(0.659244834954)*x_ref[2]**o+(0.951080038113)*x_ref[2]
        ref[0,3,0,1]=(-0.969621517275)*x_ref[0]**o+(-0.0841833729288)*x_ref[0]+(0.183088818824)*x_ref[1]**o+(-0.0637411180744)*x_ref[1]+(0.70498987804)*x_ref[2]**o+(0.464729030647)*x_ref[2]
        ref[0,3,1,0]=(-0.908811399013)*x_ref[0]**o+(0.469705146462)*x_ref[0]+(0.623185114304)*x_ref[1]**o+(0.156348468944)*x_ref[1]+(-0.0352444815103)*x_ref[2]**o+(-0.0924055149797)*x_ref[2]
        ref[0,3,1,1]=(-0.804539460051)*x_ref[0]**o+(-0.64073007907)*x_ref[0]+(0.871919457046)*x_ref[1]**o+(-0.446001955346)*x_ref[1]+(-0.758572789631)*x_ref[2]**o+(-0.943023339532)*x_ref[2]
        ref[0,3,2,0]=(0.76012858373)*x_ref[0]**o+(-0.413461345613)*x_ref[0]+(-0.636494843218)*x_ref[1]**o+(0.0576409596138)*x_ref[1]+(0.32356830303)*x_ref[2]**o+(0.66336356169)*x_ref[2]
        ref[0,3,2,1]=(-0.682969408206)*x_ref[0]**o+(-0.00378599179446)*x_ref[0]+(0.139592931649)*x_ref[1]**o+(0.230923755162)*x_ref[1]+(-0.359747563013)*x_ref[2]**o+(0.490767562994)*x_ref[2]
        ref[0,4,0,0]=(0.40294705204)*x_ref[0]**o+(-0.725991341301)*x_ref[0]+(-0.123299724766)*x_ref[1]**o+(0.783062653249)*x_ref[1]+(0.284875756512)*x_ref[2]**o+(-0.479856616542)*x_ref[2]
        ref[0,4,0,1]=(0.671083818646)*x_ref[0]**o+(0.475388684118)*x_ref[0]+(-0.0415590406628)*x_ref[1]**o+(0.523350160591)*x_ref[1]+(-0.869125066985)*x_ref[2]**o+(0.539765124009)*x_ref[2]
        ref[0,4,1,0]=(0.517796715592)*x_ref[0]**o+(0.273319492606)*x_ref[0]+(0.131964130241)*x_ref[1]**o+(0.522704060425)*x_ref[1]+(0.945735742195)*x_ref[2]**o+(-0.927060768569)*x_ref[2]
        ref[0,4,1,1]=(-0.0877749093432)*x_ref[0]**o+(-0.228856801496)*x_ref[0]+(-0.918861499662)*x_ref[1]**o+(0.783335087368)*x_ref[1]+(0.731374866941)*x_ref[2]**o+(-0.0169316582485)*x_ref[2]
        ref[0,4,2,0]=(-0.521914417723)*x_ref[0]**o+(0.100272381296)*x_ref[0]+(-0.37855972365)*x_ref[1]**o+(-0.748174737428)*x_ref[1]+(0.08343326291)*x_ref[2]**o+(-0.284290813001)*x_ref[2]
        ref[0,4,2,1]=(-0.622347209837)*x_ref[0]**o+(-0.737347404815)*x_ref[0]+(0.973210808148)*x_ref[1]**o+(0.34940344181)*x_ref[1]+(0.917063169506)*x_ref[2]**o+(0.40443560458)*x_ref[2]
        ref[1,0,0,0]=(0.906350070311)*x_ref[0]**o+(-0.514349936373)*x_ref[0]+(-0.0377819340766)*x_ref[1]**o+(-0.868663863385)*x_ref[1]+(-0.910618099331)*x_ref[2]**o+(0.161682748869)*x_ref[2]
        ref[1,0,0,1]=(-0.525696726097)*x_ref[0]**o+(-0.564534313649)*x_ref[0]+(0.294413275375)*x_ref[1]**o+(-0.032889914662)*x_ref[1]+(-0.655188068906)*x_ref[2]**o+(-0.544604687619)*x_ref[2]
        ref[1,0,1,0]=(0.982917203627)*x_ref[0]**o+(0.23003920025)*x_ref[0]+(0.53970317464)*x_ref[1]**o+(0.488996625882)*x_ref[1]+(0.155567762567)*x_ref[2]**o+(-0.0769333810796)*x_ref[2]
        ref[1,0,1,1]=(-0.695407739098)*x_ref[0]**o+(-0.528856183813)*x_ref[0]+(-0.765981763723)*x_ref[1]**o+(-0.747233410366)*x_ref[1]+(-0.217961552142)*x_ref[2]**o+(0.363922845031)*x_ref[2]
        ref[1,0,2,0]=(-0.508333136514)*x_ref[0]**o+(-0.0531371629837)*x_ref[0]+(0.147835213524)*x_ref[1]**o+(0.811513270701)*x_ref[1]+(-0.51987266682)*x_ref[2]**o+(-0.840524592697)*x_ref[2]
        ref[1,0,2,1]=(0.522917930226)*x_ref[0]**o+(-0.387982699235)*x_ref[0]+(-0.716946321359)*x_ref[1]**o+(0.440470433449)*x_ref[1]+(0.841627656575)*x_ref[2]**o+(0.635104516674)*x_ref[2]
        ref[1,1,0,0]=(0.484898204665)*x_ref[0]**o+(-0.753933429744)*x_ref[0]+(-0.275664031761)*x_ref[1]**o+(-0.566814900411)*x_ref[1]+(0.0612476985048)*x_ref[2]**o+(0.437090771267)*x_ref[2]
        ref[1,1,0,1]=(-0.182638197905)*x_ref[0]**o+(-0.778469897629)*x_ref[0]+(-0.626963871394)*x_ref[1]**o+(-0.400611638351)*x_ref[1]+(0.895655277012)*x_ref[2]**o+(0.0326121043684)*x_ref[2]
        ref[1,1,1,0]=(-0.916569622941)*x_ref[0]**o+(-0.658275271075)*x_ref[0]+(0.250794036795)*x_ref[1]**o+(-0.233593794029)*x_ref[1]+(-0.96809736282)*x_ref[2]**o+(0.656072783817)*x_ref[2]
        ref[1,1,1,1]=(0.0171169428141)*x_ref[0]**o+(0.860556263522)*x_ref[0]+(0.345606647004)*x_ref[1]**o+(0.564186697526)*x_ref[1]+(0.588489057339)*x_ref[2]**o+(-0.87110086452)*x_ref[2]
        ref[1,1,2,0]=(0.275896407623)*x_ref[0]**o+(0.0122423763916)*x_ref[0]+(0.107452743797)*x_ref[1]**o+(-0.367492719113)*x_ref[1]+(0.884317274048)*x_ref[2]**o+(0.0908799334316)*x_ref[2]
        ref[1,1,2,1]=(0.642219111519)*x_ref[0]**o+(-0.183742201216)*x_ref[0]+(-0.327197867043)*x_ref[1]**o+(0.322321062553)*x_ref[1]+(-0.183074011204)*x_ref[2]**o+(0.720021602264)*x_ref[2]
        ref[1,2,0,0]=(0.532332942294)*x_ref[0]**o+(0.950878496192)*x_ref[0]+(0.819174829388)*x_ref[1]**o+(0.43412262514)*x_ref[1]+(-0.166925240319)*x_ref[2]**o+(-0.698173204203)*x_ref[2]
        ref[1,2,0,1]=(0.259909649155)*x_ref[0]**o+(-0.264967838281)*x_ref[0]+(-0.774874307838)*x_ref[1]**o+(-0.275886773315)*x_ref[1]+(0.801233942875)*x_ref[2]**o+(0.27496052417)*x_ref[2]
        ref[1,2,1,0]=(-0.746564397549)*x_ref[0]**o+(-0.690480530678)*x_ref[0]+(0.114986469362)*x_ref[1]**o+(-0.751415850669)*x_ref[1]+(0.119951306293)*x_ref[2]**o+(-0.665012834766)*x_ref[2]
        ref[1,2,1,1]=(-0.851672541025)*x_ref[0]**o+(0.235181175896)*x_ref[0]+(-0.62908061934)*x_ref[1]**o+(-0.438672467151)*x_ref[1]+(-0.761945783738)*x_ref[2]**o+(0.580730895385)*x_ref[2]
        ref[1,2,2,0]=(0.676776403141)*x_ref[0]**o+(-0.707505868325)*x_ref[0]+(0.724421249054)*x_ref[1]**o+(0.120766197116)*x_ref[1]+(-0.760834341632)*x_ref[2]**o+(-0.689464446912)*x_ref[2]
        ref[1,2,2,1]=(0.147028945539)*x_ref[0]**o+(0.199909936204)*x_ref[0]+(0.903168165102)*x_ref[1]**o+(-0.46293792368)*x_ref[1]+(-0.282517651615)*x_ref[2]**o+(0.740619136229)*x_ref[2]
        ref[1,3,0,0]=(-0.87747323136)*x_ref[0]**o+(-0.361656418039)*x_ref[0]+(-0.14856291522)*x_ref[1]**o+(0.474044532693)*x_ref[1]+(0.138460936937)*x_ref[2]**o+(-0.145182012384)*x_ref[2]
        ref[1,3,0,1]=(0.748334497747)*x_ref[0]**o+(0.892498113089)*x_ref[0]+(0.239113720175)*x_ref[1]**o+(-0.762524411251)*x_ref[1]+(-0.488407873495)*x_ref[2]**o+(0.158979901061)*x_ref[2]
        ref[1,3,1,0]=(0.465350922376)*x_ref[0]**o+(-0.654156421791)*x_ref[0]+(0.878703330328)*x_ref[1]**o+(0.537076882157)*x_ref[1]+(0.389722314724)*x_ref[2]**o+(-0.120902843496)*x_ref[2]
        ref[1,3,1,1]=(-0.153324789456)*x_ref[0]**o+(-0.842917613229)*x_ref[0]+(0.984971864624)*x_ref[1]**o+(0.349269583591)*x_ref[1]+(0.484775635349)*x_ref[2]**o+(0.0123674100644)*x_ref[2]
        ref[1,3,2,0]=(-0.175935804533)*x_ref[0]**o+(0.0654493735159)*x_ref[0]+(0.726667315683)*x_ref[1]**o+(0.979882137896)*x_ref[1]+(0.543445934589)*x_ref[2]**o+(-0.149142988507)*x_ref[2]
        ref[1,3,2,1]=(0.235188036633)*x_ref[0]**o+(-0.383249536636)*x_ref[0]+(-0.445575566142)*x_ref[1]**o+(-0.819686823355)*x_ref[1]+(0.170138244522)*x_ref[2]**o+(-0.80311924596)*x_ref[2]
        ref[1,4,0,0]=(0.471387483772)*x_ref[0]**o+(-0.591448975176)*x_ref[0]+(-0.833833649412)*x_ref[1]**o+(0.770775874052)*x_ref[1]+(0.537769198907)*x_ref[2]**o+(-0.646003815898)*x_ref[2]
        ref[1,4,0,1]=(-0.797500020228)*x_ref[0]**o+(-0.136543576325)*x_ref[0]+(0.11979987113)*x_ref[1]**o+(-0.495999866876)*x_ref[1]+(-0.611958471401)*x_ref[2]**o+(-0.588202607164)*x_ref[2]
        ref[1,4,1,0]=(-0.550038387555)*x_ref[0]**o+(0.0844321262213)*x_ref[0]+(0.131005384757)*x_ref[1]**o+(-0.515674980981)*x_ref[1]+(-0.637043856082)*x_ref[2]**o+(0.333864121142)*x_ref[2]
        ref[1,4,1,1]=(0.71740532023)*x_ref[0]**o+(-0.868269127293)*x_ref[0]+(0.323641764571)*x_ref[1]**o+(0.820948811255)*x_ref[1]+(0.799638086908)*x_ref[2]**o+(-0.558360429849)*x_ref[2]
        ref[1,4,2,0]=(-0.366976697335)*x_ref[0]**o+(-0.121083094264)*x_ref[0]+(0.650559587113)*x_ref[1]**o+(-0.400220219011)*x_ref[1]+(-0.0785928535861)*x_ref[2]**o+(-0.50420364324)*x_ref[2]
        ref[1,4,2,1]=(0.872011301914)*x_ref[0]**o+(0.0300770125292)*x_ref[0]+(-0.965040697782)*x_ref[1]**o+(-0.373825300018)*x_ref[1]+(-0.988463549572)*x_ref[2]**o+(0.906686888358)*x_ref[2]
        ref[2,0,0,0]=(-0.644460598597)*x_ref[0]**o+(0.69918562566)*x_ref[0]+(0.0995801100352)*x_ref[1]**o+(0.257114765725)*x_ref[1]+(-0.172788858436)*x_ref[2]**o+(-0.625310450189)*x_ref[2]
        ref[2,0,0,1]=(0.331738659918)*x_ref[0]**o+(0.34134401217)*x_ref[0]+(0.667229795214)*x_ref[1]**o+(0.248779696936)*x_ref[1]+(0.782703155201)*x_ref[2]**o+(-0.0117889862376)*x_ref[2]
        ref[2,0,1,0]=(0.456825869656)*x_ref[0]**o+(0.618685513764)*x_ref[0]+(0.00330761288087)*x_ref[1]**o+(-0.839303906537)*x_ref[1]+(0.16840639247)*x_ref[2]**o+(-0.177323734594)*x_ref[2]
        ref[2,0,1,1]=(0.543967300033)*x_ref[0]**o+(0.537337400025)*x_ref[0]+(-0.232374591177)*x_ref[1]**o+(-0.163640094925)*x_ref[1]+(0.146221483058)*x_ref[2]**o+(0.833185534106)*x_ref[2]
        ref[2,0,2,0]=(-0.283419495517)*x_ref[0]**o+(-0.0641847006529)*x_ref[0]+(0.933228590846)*x_ref[1]**o+(0.575529900563)*x_ref[1]+(-0.634849100928)*x_ref[2]**o+(-0.898678364957)*x_ref[2]
        ref[2,0,2,1]=(0.248202597053)*x_ref[0]**o+(-0.395428705005)*x_ref[0]+(-0.921778142592)*x_ref[1]**o+(0.752699910606)*x_ref[1]+(-0.208939925012)*x_ref[2]**o+(0.252031969391)*x_ref[2]
        ref[2,1,0,0]=(0.543045279112)*x_ref[0]**o+(-0.200796280468)*x_ref[0]+(-0.850234504692)*x_ref[1]**o+(-0.608668852127)*x_ref[1]+(-0.773553907964)*x_ref[2]**o+(-0.727318461167)*x_ref[2]
        ref[2,1,0,1]=(0.670138313505)*x_ref[0]**o+(0.763313503204)*x_ref[0]+(-0.712024856615)*x_ref[1]**o+(-0.0810633888547)*x_ref[1]+(-0.543339707954)*x_ref[2]**o+(-0.0944422473469)*x_ref[2]
        ref[2,1,1,0]=(0.66812399426)*x_ref[0]**o+(-0.00267070537766)*x_ref[0]+(0.704373662345)*x_ref[1]**o+(-0.778778632343)*x_ref[1]+(-0.195017717355)*x_ref[2]**o+(-0.718710830736)*x_ref[2]
        ref[2,1,1,1]=(-0.905514219789)*x_ref[0]**o+(-0.899693862646)*x_ref[0]+(-0.791063116492)*x_ref[1]**o+(-0.470046803205)*x_ref[1]+(0.954924769408)*x_ref[2]**o+(0.36799445545)*x_ref[2]
        ref[2,1,2,0]=(0.363438950523)*x_ref[0]**o+(-0.788582160746)*x_ref[0]+(0.316623350464)*x_ref[1]**o+(-0.621859877627)*x_ref[1]+(-0.899933912274)*x_ref[2]**o+(-0.663582380926)*x_ref[2]
        ref[2,1,2,1]=(0.70767884819)*x_ref[0]**o+(-0.337197298469)*x_ref[0]+(0.329833087038)*x_ref[1]**o+(-0.767476122169)*x_ref[1]+(0.813567450674)*x_ref[2]**o+(-0.230739080738)*x_ref[2]
        ref[2,2,0,0]=(0.425629461437)*x_ref[0]**o+(0.736062549141)*x_ref[0]+(0.0766779443487)*x_ref[1]**o+(-0.645685464512)*x_ref[1]+(-0.563156862696)*x_ref[2]**o+(-0.200669234455)*x_ref[2]
        ref[2,2,0,1]=(-0.436511105169)*x_ref[0]**o+(-0.784209106822)*x_ref[0]+(0.476448752447)*x_ref[1]**o+(-0.976253075887)*x_ref[1]+(0.744207251461)*x_ref[2]**o+(-0.480585834629)*x_ref[2]
        ref[2,2,1,0]=(-0.160519965814)*x_ref[0]**o+(-0.740586061373)*x_ref[0]+(-0.291540051837)*x_ref[1]**o+(0.0186903802094)*x_ref[1]+(-0.456120795038)*x_ref[2]**o+(0.930603860664)*x_ref[2]
        ref[2,2,1,1]=(0.835200574398)*x_ref[0]**o+(-0.111618184052)*x_ref[0]+(0.394793418949)*x_ref[1]**o+(0.66759987425)*x_ref[1]+(0.902913963474)*x_ref[2]**o+(-0.840871310598)*x_ref[2]
        ref[2,2,2,0]=(-0.745841363439)*x_ref[0]**o+(0.859714040441)*x_ref[0]+(-0.133922374335)*x_ref[1]**o+(-0.539065635521)*x_ref[1]+(-0.0496309374348)*x_ref[2]**o+(-0.3150513125)*x_ref[2]
        ref[2,2,2,1]=(-0.121952493914)*x_ref[0]**o+(-0.145565953724)*x_ref[0]+(-0.46956691252)*x_ref[1]**o+(0.594587113542)*x_ref[1]+(0.801503774207)*x_ref[2]**o+(-0.559107902481)*x_ref[2]
        ref[2,3,0,0]=(0.107705603601)*x_ref[0]**o+(0.83167538493)*x_ref[0]+(0.719052082882)*x_ref[1]**o+(-0.0418541572577)*x_ref[1]+(-0.760679778738)*x_ref[2]**o+(-0.269573603299)*x_ref[2]
        ref[2,3,0,1]=(-0.610961305034)*x_ref[0]**o+(-0.48822367684)*x_ref[0]+(-0.533146614698)*x_ref[1]**o+(-0.0634033507577)*x_ref[1]+(-0.525819529758)*x_ref[2]**o+(0.70089075883)*x_ref[2]
        ref[2,3,1,0]=(0.503925514982)*x_ref[0]**o+(0.65957769246)*x_ref[0]+(-0.624626366929)*x_ref[1]**o+(-0.374542181845)*x_ref[1]+(-0.363387031742)*x_ref[2]**o+(-0.00563190329728)*x_ref[2]
        ref[2,3,1,1]=(0.337750136392)*x_ref[0]**o+(-0.364300637249)*x_ref[0]+(0.0892494275431)*x_ref[1]**o+(0.958228736151)*x_ref[1]+(0.203359176466)*x_ref[2]**o+(-0.454161162092)*x_ref[2]
        ref[2,3,2,0]=(0.171894913416)*x_ref[0]**o+(0.930342123436)*x_ref[0]+(0.888973400724)*x_ref[1]**o+(0.732732070997)*x_ref[1]+(-0.365536399448)*x_ref[2]**o+(0.719027934816)*x_ref[2]
        ref[2,3,2,1]=(0.0718345473684)*x_ref[0]**o+(-0.351767950331)*x_ref[0]+(-0.427880627822)*x_ref[1]**o+(0.906554327062)*x_ref[1]+(0.209905901579)*x_ref[2]**o+(0.343983972848)*x_ref[2]
        ref[2,4,0,0]=(0.20082437951)*x_ref[0]**o+(0.411679233479)*x_ref[0]+(-0.146683847376)*x_ref[1]**o+(0.619164773872)*x_ref[1]+(0.283942602042)*x_ref[2]**o+(0.328844649922)*x_ref[2]
        ref[2,4,0,1]=(0.512237008877)*x_ref[0]**o+(-0.0799414456365)*x_ref[0]+(-0.444485238118)*x_ref[1]**o+(-0.617601180088)*x_ref[1]+(-0.522016560329)*x_ref[2]**o+(0.764966540338)*x_ref[2]
        ref[2,4,1,0]=(-0.906295486351)*x_ref[0]**o+(0.213043439406)*x_ref[0]+(-0.841541923905)*x_ref[1]**o+(0.613189806332)*x_ref[1]+(0.98016601466)*x_ref[2]**o+(0.373100080758)*x_ref[2]
        ref[2,4,1,1]=(0.421256899132)*x_ref[0]**o+(-0.148027161979)*x_ref[0]+(-0.436551670748)*x_ref[1]**o+(0.15389527845)*x_ref[1]+(-0.202772011213)*x_ref[2]**o+(0.526456981868)*x_ref[2]
        ref[2,4,2,0]=(0.326695825605)*x_ref[0]**o+(-0.658196467156)*x_ref[0]+(-0.0229897453445)*x_ref[1]**o+(0.640660125725)*x_ref[1]+(0.604325801088)*x_ref[2]**o+(0.415440861181)*x_ref[2]
        ref[2,4,2,1]=(-0.353746711731)*x_ref[0]**o+(0.707902797414)*x_ref[0]+(-0.117816706586)*x_ref[1]**o+(0.767979696436)*x_ref[1]+(0.0361750168523)*x_ref[2]**o+(-0.904298176296)*x_ref[2]
        ref[3,0,0,0]=(0.274392483594)*x_ref[0]**o+(-0.142144450689)*x_ref[0]+(-0.308306223778)*x_ref[1]**o+(0.674122485073)*x_ref[1]+(0.259741082605)*x_ref[2]**o+(-0.249672127213)*x_ref[2]
        ref[3,0,0,1]=(-0.299212636089)*x_ref[0]**o+(0.670140835575)*x_ref[0]+(0.0513019322721)*x_ref[1]**o+(0.460049892726)*x_ref[1]+(0.167967285754)*x_ref[2]**o+(-0.463520967247)*x_ref[2]
        ref[3,0,1,0]=(0.850761515544)*x_ref[0]**o+(0.945654906297)*x_ref[0]+(-0.841039831472)*x_ref[1]**o+(-0.051572515458)*x_ref[1]+(0.750689246331)*x_ref[2]**o+(0.0651065819687)*x_ref[2]
        ref[3,0,1,1]=(-0.387082102682)*x_ref[0]**o+(-0.89541016885)*x_ref[0]+(-0.903675915508)*x_ref[1]**o+(-0.268304570988)*x_ref[1]+(-0.259828889903)*x_ref[2]**o+(-0.983650155837)*x_ref[2]
        ref[3,0,2,0]=(0.497841641089)*x_ref[0]**o+(0.80522306204)*x_ref[0]+(0.532637885608)*x_ref[1]**o+(-0.345627398376)*x_ref[1]+(-0.768962683025)*x_ref[2]**o+(-0.608176054775)*x_ref[2]
        ref[3,0,2,1]=(0.00437007549865)*x_ref[0]**o+(0.847511281463)*x_ref[0]+(0.348033100721)*x_ref[1]**o+(0.348640167731)*x_ref[1]+(-0.482286157201)*x_ref[2]**o+(0.23184848892)*x_ref[2]
        ref[3,1,0,0]=(0.79458009346)*x_ref[0]**o+(0.346244735823)*x_ref[0]+(0.274964941045)*x_ref[1]**o+(0.306665745695)*x_ref[1]+(0.752939751818)*x_ref[2]**o+(-0.0212951945504)*x_ref[2]
        ref[3,1,0,1]=(0.279860784064)*x_ref[0]**o+(0.728689070796)*x_ref[0]+(0.112157508671)*x_ref[1]**o+(-0.885233951967)*x_ref[1]+(-0.469780984101)*x_ref[2]**o+(0.177020475912)*x_ref[2]
        ref[3,1,1,0]=(0.433534890526)*x_ref[0]**o+(0.485733515889)*x_ref[0]+(0.830706912979)*x_ref[1]**o+(0.118568290881)*x_ref[1]+(-0.863800517187)*x_ref[2]**o+(0.737792919447)*x_ref[2]
        ref[3,1,1,1]=(0.791784006945)*x_ref[0]**o+(-0.112270136239)*x_ref[0]+(0.874488179673)*x_ref[1]**o+(-0.678555278645)*x_ref[1]+(0.985727158563)*x_ref[2]**o+(0.920137638515)*x_ref[2]
        ref[3,1,2,0]=(-0.961254804108)*x_ref[0]**o+(0.243551301383)*x_ref[0]+(-0.311248382242)*x_ref[1]**o+(0.41289285623)*x_ref[1]+(-0.371475600554)*x_ref[2]**o+(-0.0582608041921)*x_ref[2]
        ref[3,1,2,1]=(-0.144445942195)*x_ref[0]**o+(-0.418750582693)*x_ref[0]+(-0.468333324126)*x_ref[1]**o+(-0.658823898386)*x_ref[1]+(-0.909121304085)*x_ref[2]**o+(-0.600433830184)*x_ref[2]
        ref[3,2,0,0]=(-0.286554311888)*x_ref[0]**o+(-0.737505598582)*x_ref[0]+(0.566521914245)*x_ref[1]**o+(0.498091322078)*x_ref[1]+(0.0202811708798)*x_ref[2]**o+(-0.792584826215)*x_ref[2]
        ref[3,2,0,1]=(-0.52573040544)*x_ref[0]**o+(0.256454412031)*x_ref[0]+(-0.771513393908)*x_ref[1]**o+(0.0769633391923)*x_ref[1]+(0.447325513976)*x_ref[2]**o+(0.860889667608)*x_ref[2]
        ref[3,2,1,0]=(-0.776650147122)*x_ref[0]**o+(-0.597663916146)*x_ref[0]+(-0.56596948577)*x_ref[1]**o+(0.606515796008)*x_ref[1]+(-0.0569814692195)*x_ref[2]**o+(0.0832162598646)*x_ref[2]
        ref[3,2,1,1]=(-0.702983139798)*x_ref[0]**o+(-0.0721500323076)*x_ref[0]+(-0.696083887377)*x_ref[1]**o+(-0.00975653495795)*x_ref[1]+(0.972659180196)*x_ref[2]**o+(0.968527616515)*x_ref[2]
        ref[3,2,2,0]=(-0.861690620984)*x_ref[0]**o+(0.169760353347)*x_ref[0]+(-0.774337793749)*x_ref[1]**o+(-0.799362859305)*x_ref[1]+(-0.352790827791)*x_ref[2]**o+(-0.31255981171)*x_ref[2]
        ref[3,2,2,1]=(0.264209810017)*x_ref[0]**o+(-0.86560795678)*x_ref[0]+(0.306794805255)*x_ref[1]**o+(0.795495753342)*x_ref[1]+(0.5631729589)*x_ref[2]**o+(-0.901732966437)*x_ref[2]
        ref[3,3,0,0]=(0.151220066195)*x_ref[0]**o+(0.294238555124)*x_ref[0]+(0.725179461723)*x_ref[1]**o+(0.823417422915)*x_ref[1]+(0.826457989801)*x_ref[2]**o+(-0.068783769033)*x_ref[2]
        ref[3,3,0,1]=(0.965148541899)*x_ref[0]**o+(0.054903011166)*x_ref[0]+(-0.564782216515)*x_ref[1]**o+(-0.825732725252)*x_ref[1]+(-0.172033069024)*x_ref[2]**o+(0.657198365579)*x_ref[2]
        ref[3,3,1,0]=(0.695530033492)*x_ref[0]**o+(-0.251400308058)*x_ref[0]+(-0.379416854517)*x_ref[1]**o+(-0.491666460242)*x_ref[1]+(0.215166477617)*x_ref[2]**o+(0.690685608974)*x_ref[2]
        ref[3,3,1,1]=(0.377513737173)*x_ref[0]**o+(-0.0172594331468)*x_ref[0]+(0.42799383164)*x_ref[1]**o+(0.0607216253163)*x_ref[1]+(-0.153084202648)*x_ref[2]**o+(0.411927363219)*x_ref[2]
        ref[3,3,2,0]=(-0.125731371056)*x_ref[0]**o+(0.33378895482)*x_ref[0]+(-0.448245878593)*x_ref[1]**o+(-0.764746615313)*x_ref[1]+(0.181875287538)*x_ref[2]**o+(-0.581992845032)*x_ref[2]
        ref[3,3,2,1]=(-0.584517106374)*x_ref[0]**o+(0.793927805805)*x_ref[0]+(0.300548289158)*x_ref[1]**o+(0.0449525116422)*x_ref[1]+(0.883866474871)*x_ref[2]**o+(0.518036509563)*x_ref[2]
        ref[3,4,0,0]=(-0.685669036611)*x_ref[0]**o+(-0.406759680831)*x_ref[0]+(0.020288611842)*x_ref[1]**o+(0.524437172569)*x_ref[1]+(-0.377692301032)*x_ref[2]**o+(-0.969562114494)*x_ref[2]
        ref[3,4,0,1]=(0.541982416337)*x_ref[0]**o+(0.839468986455)*x_ref[0]+(-0.542504618993)*x_ref[1]**o+(-0.458056288842)*x_ref[1]+(0.982937308016)*x_ref[2]**o+(0.751237990644)*x_ref[2]
        ref[3,4,1,0]=(-0.623461568258)*x_ref[0]**o+(0.469859301643)*x_ref[0]+(0.841657608811)*x_ref[1]**o+(0.819674909109)*x_ref[1]+(-0.655140622922)*x_ref[2]**o+(0.0824474652875)*x_ref[2]
        ref[3,4,1,1]=(0.906724250555)*x_ref[0]**o+(0.14855445564)*x_ref[0]+(-0.862379845699)*x_ref[1]**o+(-0.853300535973)*x_ref[1]+(-0.714581251705)*x_ref[2]**o+(-0.956268149641)*x_ref[2]
        ref[3,4,2,0]=(-0.721603766365)*x_ref[0]**o+(0.132835246572)*x_ref[0]+(0.0527745535366)*x_ref[1]**o+(-0.161464199846)*x_ref[1]+(-0.551405664077)*x_ref[2]**o+(-0.696496777295)*x_ref[2]
        ref[3,4,2,1]=(-0.215666532882)*x_ref[0]**o+(0.145798314909)*x_ref[0]+(0.547835453007)*x_ref[1]**o+(0.6070128325)*x_ref[1]+(0.179306259279)*x_ref[2]**o+(-0.0234805628101)*x_ref[2]
      res=interpolate(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 5, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_L2_onFunctionOnContactZero_fromData_rank0(self):
      """
      tests L2-norm of Data on the FunctionOnContactZero

      assumptions: self.domain supports integration on FunctionOnContactZero
      """
      dim=self.domain.getDim()
      w=FunctionOnContactZero(self.domain)
      x=w.getX()
      if dim==2:
        arg=Data(0,(),w)
        arg=(-0.908822474879)*x[0]
        ref=sqrt((0.0)+(0.206489572711))

      else:
        arg=Data(0,(),w)
        arg=(-0.893168521371)*x[0]
        ref=sqrt((0.0)+(0.199437501892))

      res=L2(arg)
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertAlmostEqual(res,ref,int(-log10(self.RES_TOL)),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_L2_onFunctionOnContactZero_fromData_rank1(self):
      """
      tests L2-norm of Data on the FunctionOnContactZero

      assumptions: self.domain supports integration on FunctionOnContactZero
      """
      dim=self.domain.getDim()
      w=FunctionOnContactZero(self.domain)
      x=w.getX()
      if dim==2:
        arg=Data(0,(2,),w)
        arg[0]=(-0.30839170903)*x[0]
        arg[1]=(0.108352504587)*x[1]
        ref=sqrt((0.00391342175007)+(0.0237763615497))

      else:
        arg=Data(0,(3,),w)
        arg[0]=(-0.260572084359)*x[0]
        arg[1]=(0.250696802346)*x[1]
        arg[2]=(-0.498857619202)*x[2]
        ref=sqrt((0.103902603647)+(0.0169744527868))

      res=L2(arg)
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertAlmostEqual(res,ref,int(-log10(self.RES_TOL)),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_L2_onFunctionOnContactZero_fromData_rank2(self):
      """
      tests L2-norm of Data on the FunctionOnContactZero

      assumptions: self.domain supports integration on FunctionOnContactZero
      """
      dim=self.domain.getDim()
      w=FunctionOnContactZero(self.domain)
      x=w.getX()
      if dim==2:
        arg=Data(0,(4, 2),w)
        arg[0,0]=(0.0107777943992)*x[0]
        arg[0,1]=(0.0188929341636)*x[1]
        arg[1,0]=(-0.00268267834861)*x[0]
        arg[1,1]=(-0.656039971477)*x[1]
        arg[2,0]=(-0.997645982047)*x[0]
        arg[2,1]=(0.993049748017)*x[1]
        arg[3,0]=(0.938636169945)*x[0]
        arg[3,1]=(-0.609892243528)*x[1]
        ref=sqrt((0.596287245963)+(0.46911468066))

      else:
        arg=Data(0,(4, 3),w)
        arg[0,0]=(0.128987155564)*x[0]
        arg[0,1]=(-0.582660210551)*x[1]
        arg[0,2]=(-0.605040149822)*x[2]
        arg[1,0]=(0.494540007291)*x[0]
        arg[1,1]=(0.563900397935)*x[1]
        arg[1,2]=(-0.806214220737)*x[2]
        arg[2,0]=(0.878027993704)*x[0]
        arg[2,1]=(0.597165310427)*x[1]
        arg[2,2]=(-0.444798130742)*x[2]
        arg[3,0]=(0.982067213342)*x[0]
        arg[3,1]=(-0.160013302763)*x[1]
        arg[3,2]=(-0.490830124654)*x[2]
        ref=sqrt((0.831500595262)+(0.49914916859))

      res=L2(arg)
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertAlmostEqual(res,ref,int(-log10(self.RES_TOL)),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_L2_onFunctionOnContactZero_fromData_rank3(self):
      """
      tests L2-norm of Data on the FunctionOnContactZero

      assumptions: self.domain supports integration on FunctionOnContactZero
      """
      dim=self.domain.getDim()
      w=FunctionOnContactZero(self.domain)
      x=w.getX()
      if dim==2:
        arg=Data(0,(6, 2, 2),w)
        arg[0,0,0]=(0.83264345248)*x[0]
        arg[0,0,1]=(-0.29697614845)*x[1]
        arg[0,1,0]=(-0.903150797089)*x[0]
        arg[0,1,1]=(-0.569048463149)*x[1]
        arg[1,0,0]=(0.317087339385)*x[0]
        arg[1,0,1]=(0.0120740865344)*x[1]
        arg[1,1,0]=(-0.221145803457)*x[0]
        arg[1,1,1]=(-0.15362409313)*x[1]
        arg[2,0,0]=(0.459428065932)*x[0]
        arg[2,0,1]=(-0.495488213623)*x[1]
        arg[2,1,0]=(0.919114285084)*x[0]
        arg[2,1,1]=(0.54511942052)*x[1]
        arg[3,0,0]=(0.442207797529)*x[0]
        arg[3,0,1]=(0.958341188449)*x[1]
        arg[3,1,0]=(-0.0915205976682)*x[0]
        arg[3,1,1]=(-0.410964460336)*x[1]
        arg[4,0,0]=(-0.243030095673)*x[0]
        arg[4,0,1]=(-0.413661756935)*x[1]
        arg[4,1,0]=(-0.0973014277464)*x[0]
        arg[4,1,1]=(0.204130676874)*x[1]
        arg[5,0,0]=(0.38083750131)*x[0]
        arg[5,0,1]=(0.286331574579)*x[1]
        arg[5,1,0]=(0.583320730408)*x[0]
        arg[5,1,1]=(-0.0899419806181)*x[1]
        ref=sqrt((0.789530406063)+(0.868006693351))

      else:
        arg=Data(0,(6, 2, 3),w)
        arg[0,0,0]=(-0.98411764268)*x[0]
        arg[0,0,1]=(-0.180549567137)*x[1]
        arg[0,0,2]=(0.357923974709)*x[2]
        arg[0,1,0]=(0.67691778218)*x[0]
        arg[0,1,1]=(-0.643267669153)*x[1]
        arg[0,1,2]=(0.452374328301)*x[2]
        arg[1,0,0]=(0.494303694439)*x[0]
        arg[1,0,1]=(0.00767629822214)*x[1]
        arg[1,0,2]=(0.232436713422)*x[2]
        arg[1,1,0]=(-0.698568975983)*x[0]
        arg[1,1,1]=(-0.08483720353)*x[1]
        arg[1,1,2]=(-0.723935799719)*x[2]
        arg[2,0,0]=(0.931313239646)*x[0]
        arg[2,0,1]=(-0.0988070442341)*x[1]
        arg[2,0,2]=(-0.166201798278)*x[2]
        arg[2,1,0]=(-0.733172171406)*x[0]
        arg[2,1,1]=(0.24542706584)*x[1]
        arg[2,1,2]=(0.49170717592)*x[2]
        arg[3,0,0]=(-0.718346096869)*x[0]
        arg[3,0,1]=(0.297442410539)*x[1]
        arg[3,0,2]=(0.0277708718174)*x[2]
        arg[3,1,0]=(0.735100702194)*x[0]
        arg[3,1,1]=(-0.808007526296)*x[1]
        arg[3,1,2]=(-0.423023238313)*x[2]
        arg[4,0,0]=(0.383188529876)*x[0]
        arg[4,0,1]=(-0.190101601814)*x[1]
        arg[4,0,2]=(-0.404380990215)*x[2]
        arg[4,1,0]=(-0.31098204217)*x[0]
        arg[4,1,1]=(-0.715247945432)*x[1]
        arg[4,1,2]=(0.435774772982)*x[2]
        arg[5,0,0]=(-0.0662056096206)*x[0]
        arg[5,0,1]=(0.746710247367)*x[1]
        arg[5,0,2]=(0.753267249301)*x[2]
        arg[5,1,0]=(0.258102962518)*x[0]
        arg[5,1,1]=(-0.0270028483187)*x[1]
        arg[5,1,2]=(-0.971060107954)*x[2]
        ref=sqrt((1.86493016409)+(1.23371587938))

      res=L2(arg)
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertAlmostEqual(res,ref,int(-log10(self.RES_TOL)),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_L2_onFunctionOnContactZero_fromData_rank4(self):
      """
      tests L2-norm of Data on the FunctionOnContactZero

      assumptions: self.domain supports integration on FunctionOnContactZero
      """
      dim=self.domain.getDim()
      w=FunctionOnContactZero(self.domain)
      x=w.getX()
      if dim==2:
        arg=Data(0,(4, 5, 3, 2),w)
        arg[0,0,0,0]=(-0.144741217139)*x[0]
        arg[0,0,0,1]=(-0.188187241012)*x[1]
        arg[0,0,1,0]=(0.056339373733)*x[0]
        arg[0,0,1,1]=(0.222177130759)*x[1]
        arg[0,0,2,0]=(-0.437240978497)*x[0]
        arg[0,0,2,1]=(-0.479213082014)*x[1]
        arg[0,1,0,0]=(0.343204141181)*x[0]
        arg[0,1,0,1]=(0.099683148208)*x[1]
        arg[0,1,1,0]=(0.167657906331)*x[0]
        arg[0,1,1,1]=(0.371064917656)*x[1]
        arg[0,1,2,0]=(-0.175796118619)*x[0]
        arg[0,1,2,1]=(-0.523477567832)*x[1]
        arg[0,2,0,0]=(0.188832282277)*x[0]
        arg[0,2,0,1]=(0.948618356423)*x[1]
        arg[0,2,1,0]=(-0.3835066594)*x[0]
        arg[0,2,1,1]=(0.872875257641)*x[1]
        arg[0,2,2,0]=(-0.98414676194)*x[0]
        arg[0,2,2,1]=(0.59170993756)*x[1]
        arg[0,3,0,0]=(0.875820385471)*x[0]
        arg[0,3,0,1]=(-0.209139794169)*x[1]
        arg[0,3,1,0]=(0.783312979835)*x[0]
        arg[0,3,1,1]=(-0.440048708153)*x[1]
        arg[0,3,2,0]=(-0.906741480158)*x[0]
        arg[0,3,2,1]=(0.63834196559)*x[1]
        arg[0,4,0,0]=(0.0142616166715)*x[0]
        arg[0,4,0,1]=(-0.0745581409622)*x[1]
        arg[0,4,1,0]=(-0.779966728505)*x[0]
        arg[0,4,1,1]=(0.734744791584)*x[1]
        arg[0,4,2,0]=(0.0407627699446)*x[0]
        arg[0,4,2,1]=(-0.629666423098)*x[1]
        arg[1,0,0,0]=(-0.782291768803)*x[0]
        arg[1,0,0,1]=(-0.197799867335)*x[1]
        arg[1,0,1,0]=(-0.487323212409)*x[0]
        arg[1,0,1,1]=(-0.789940409633)*x[1]
        arg[1,0,2,0]=(-0.523871448728)*x[0]
        arg[1,0,2,1]=(-0.622272488658)*x[1]
        arg[1,1,0,0]=(0.370095832941)*x[0]
        arg[1,1,0,1]=(-0.0683624906905)*x[1]
        arg[1,1,1,0]=(-0.180583695959)*x[0]
        arg[1,1,1,1]=(-0.0529097308814)*x[1]
        arg[1,1,2,0]=(0.440088045764)*x[0]
        arg[1,1,2,1]=(-0.207665675854)*x[1]
        arg[1,2,0,0]=(-0.360120735448)*x[0]
        arg[1,2,0,1]=(0.877797020694)*x[1]
        arg[1,2,1,0]=(0.655192721419)*x[0]
        arg[1,2,1,1]=(0.978781227463)*x[1]
        arg[1,2,2,0]=(-0.205886258554)*x[0]
        arg[1,2,2,1]=(-0.0050396100363)*x[1]
        arg[1,3,0,0]=(-0.484332943177)*x[0]
        arg[1,3,0,1]=(0.964697361018)*x[1]
        arg[1,3,1,0]=(0.0815652906824)*x[0]
        arg[1,3,1,1]=(-0.531163025802)*x[1]
        arg[1,3,2,0]=(-0.229615479833)*x[0]
        arg[1,3,2,1]=(0.139971291609)*x[1]
        arg[1,4,0,0]=(0.960839193726)*x[0]
        arg[1,4,0,1]=(-0.9744930023)*x[1]
        arg[1,4,1,0]=(-0.0919000363634)*x[0]
        arg[1,4,1,1]=(0.483909186003)*x[1]
        arg[1,4,2,0]=(0.248597552464)*x[0]
        arg[1,4,2,1]=(0.965872402486)*x[1]
        arg[2,0,0,0]=(-0.935049188033)*x[0]
        arg[2,0,0,1]=(-0.823239964421)*x[1]
        arg[2,0,1,0]=(0.30901193713)*x[0]
        arg[2,0,1,1]=(-0.0538396976182)*x[1]
        arg[2,0,2,0]=(-0.449416473099)*x[0]
        arg[2,0,2,1]=(-0.857785267819)*x[1]
        arg[2,1,0,0]=(-0.167802578334)*x[0]
        arg[2,1,0,1]=(-0.7016101186)*x[1]
        arg[2,1,1,0]=(0.995848317929)*x[0]
        arg[2,1,1,1]=(-0.824595018011)*x[1]
        arg[2,1,2,0]=(-0.826030527686)*x[0]
        arg[2,1,2,1]=(-0.856151725651)*x[1]
        arg[2,2,0,0]=(0.342047605028)*x[0]
        arg[2,2,0,1]=(-0.342370307931)*x[1]
        arg[2,2,1,0]=(-0.540693890831)*x[0]
        arg[2,2,1,1]=(-0.720754897785)*x[1]
        arg[2,2,2,0]=(-0.0941215154824)*x[0]
        arg[2,2,2,1]=(0.115887914141)*x[1]
        arg[2,3,0,0]=(-0.0920059639279)*x[0]
        arg[2,3,0,1]=(0.910948619784)*x[1]
        arg[2,3,1,0]=(-0.40846609126)*x[0]
        arg[2,3,1,1]=(0.542728733954)*x[1]
        arg[2,3,2,0]=(0.451361830368)*x[0]
        arg[2,3,2,1]=(0.0767538617936)*x[1]
        arg[2,4,0,0]=(-0.431757008766)*x[0]
        arg[2,4,0,1]=(0.258988103513)*x[1]
        arg[2,4,1,0]=(-0.584559859464)*x[0]
        arg[2,4,1,1]=(-0.776827606804)*x[1]
        arg[2,4,2,0]=(-0.867078960637)*x[0]
        arg[2,4,2,1]=(-0.0972960135129)*x[1]
        arg[3,0,0,0]=(0.999511727022)*x[0]
        arg[3,0,0,1]=(-0.753218704622)*x[1]
        arg[3,0,1,0]=(0.506439354885)*x[0]
        arg[3,0,1,1]=(-0.0845853282297)*x[1]
        arg[3,0,2,0]=(-0.871475028801)*x[0]
        arg[3,0,2,1]=(-0.69261642657)*x[1]
        arg[3,1,0,0]=(0.549778399933)*x[0]
        arg[3,1,0,1]=(0.246187536149)*x[1]
        arg[3,1,1,0]=(-0.620454200676)*x[0]
        arg[3,1,1,1]=(0.372738564315)*x[1]
        arg[3,1,2,0]=(0.0544097075138)*x[0]
        arg[3,1,2,1]=(-0.0883198944676)*x[1]
        arg[3,2,0,0]=(-0.671057180545)*x[0]
        arg[3,2,0,1]=(-0.118991797907)*x[1]
        arg[3,2,1,0]=(-0.27196730081)*x[0]
        arg[3,2,1,1]=(-0.458807068968)*x[1]
        arg[3,2,2,0]=(-0.31485399083)*x[0]
        arg[3,2,2,1]=(0.487291960328)*x[1]
        arg[3,3,0,0]=(-0.750302723531)*x[0]
        arg[3,3,0,1]=(-0.486428789771)*x[1]
        arg[3,3,1,0]=(-0.295909324594)*x[0]
        arg[3,3,1,1]=(0.325702372596)*x[1]
        arg[3,3,2,0]=(-0.512109540631)*x[0]
        arg[3,3,2,1]=(0.563284721908)*x[1]
        arg[3,4,0,0]=(0.53579406798)*x[0]
        arg[3,4,0,1]=(-0.468431927975)*x[1]
        arg[3,4,1,0]=(0.570851401329)*x[0]
        arg[3,4,1,1]=(0.107791149865)*x[1]
        arg[3,4,2,0]=(-0.211543670157)*x[0]
        arg[3,4,2,1]=(0.848189550468)*x[1]
        ref=sqrt((6.54316712884)+(4.39564064326))

      else:
        arg=Data(0,(4, 5, 3, 3),w)
        arg[0,0,0,0]=(-0.392256494872)*x[0]
        arg[0,0,0,1]=(0.271488478841)*x[1]
        arg[0,0,0,2]=(-0.878268531987)*x[2]
        arg[0,0,1,0]=(0.106903787643)*x[0]
        arg[0,0,1,1]=(-0.117986434516)*x[1]
        arg[0,0,1,2]=(0.912673598522)*x[2]
        arg[0,0,2,0]=(-0.652598945824)*x[0]
        arg[0,0,2,1]=(-0.984195895647)*x[1]
        arg[0,0,2,2]=(-0.137845459589)*x[2]
        arg[0,1,0,0]=(0.696713200721)*x[0]
        arg[0,1,0,1]=(0.480337929223)*x[1]
        arg[0,1,0,2]=(-0.628726114325)*x[2]
        arg[0,1,1,0]=(-0.356503461347)*x[0]
        arg[0,1,1,1]=(-0.64670584045)*x[1]
        arg[0,1,1,2]=(-0.737787618908)*x[2]
        arg[0,1,2,0]=(-0.14207595302)*x[0]
        arg[0,1,2,1]=(-0.573485525334)*x[1]
        arg[0,1,2,2]=(-0.955542178481)*x[2]
        arg[0,2,0,0]=(0.371952883975)*x[0]
        arg[0,2,0,1]=(0.114016178762)*x[1]
        arg[0,2,0,2]=(0.552721218169)*x[2]
        arg[0,2,1,0]=(0.318426742113)*x[0]
        arg[0,2,1,1]=(0.862220239384)*x[1]
        arg[0,2,1,2]=(0.887376889978)*x[2]
        arg[0,2,2,0]=(0.203656742981)*x[0]
        arg[0,2,2,1]=(0.350541335144)*x[1]
        arg[0,2,2,2]=(-0.448401957447)*x[2]
        arg[0,3,0,0]=(-0.349200084042)*x[0]
        arg[0,3,0,1]=(-0.546012602125)*x[1]
        arg[0,3,0,2]=(-0.931488270979)*x[2]
        arg[0,3,1,0]=(0.207457831058)*x[0]
        arg[0,3,1,1]=(0.557715840549)*x[1]
        arg[0,3,1,2]=(-0.978130146744)*x[2]
        arg[0,3,2,0]=(-0.55088967957)*x[0]
        arg[0,3,2,1]=(0.0490328838057)*x[1]
        arg[0,3,2,2]=(0.150209929122)*x[2]
        arg[0,4,0,0]=(-0.484145614698)*x[0]
        arg[0,4,0,1]=(0.393078411279)*x[1]
        arg[0,4,0,2]=(0.0678631863917)*x[2]
        arg[0,4,1,0]=(-0.350980464628)*x[0]
        arg[0,4,1,1]=(-0.784203839564)*x[1]
        arg[0,4,1,2]=(0.636960296147)*x[2]
        arg[0,4,2,0]=(0.592799503581)*x[0]
        arg[0,4,2,1]=(-0.672104833683)*x[1]
        arg[0,4,2,2]=(0.0366914082467)*x[2]
        arg[1,0,0,0]=(-0.147168019774)*x[0]
        arg[1,0,0,1]=(-0.0823637938956)*x[1]
        arg[1,0,0,2]=(-0.852729690176)*x[2]
        arg[1,0,1,0]=(-0.612338430408)*x[0]
        arg[1,0,1,1]=(-0.85820035747)*x[1]
        arg[1,0,1,2]=(-0.463664966162)*x[2]
        arg[1,0,2,0]=(0.274600720491)*x[0]
        arg[1,0,2,1]=(-0.488508234093)*x[1]
        arg[1,0,2,2]=(-0.28251466519)*x[2]
        arg[1,1,0,0]=(-0.0196532462794)*x[0]
        arg[1,1,0,1]=(0.239352528871)*x[1]
        arg[1,1,0,2]=(-0.17054773873)*x[2]
        arg[1,1,1,0]=(-0.2396627789)*x[0]
        arg[1,1,1,1]=(0.868970323003)*x[1]
        arg[1,1,1,2]=(0.401487430312)*x[2]
        arg[1,1,2,0]=(-0.624411449783)*x[0]
        arg[1,1,2,1]=(0.0036634266684)*x[1]
        arg[1,1,2,2]=(0.736129120967)*x[2]
        arg[1,2,0,0]=(0.183756511707)*x[0]
        arg[1,2,0,1]=(-0.288651848639)*x[1]
        arg[1,2,0,2]=(-0.0672121537447)*x[2]
        arg[1,2,1,0]=(-0.323274725936)*x[0]
        arg[1,2,1,1]=(0.298001016025)*x[1]
        arg[1,2,1,2]=(-0.976052460675)*x[2]
        arg[1,2,2,0]=(0.596504441096)*x[0]
        arg[1,2,2,1]=(0.873776068983)*x[1]
        arg[1,2,2,2]=(-0.994068273196)*x[2]
        arg[1,3,0,0]=(-0.495387299681)*x[0]
        arg[1,3,0,1]=(-0.123674756551)*x[1]
        arg[1,3,0,2]=(0.581213818577)*x[2]
        arg[1,3,1,0]=(0.146405749701)*x[0]
        arg[1,3,1,1]=(-0.594994686675)*x[1]
        arg[1,3,1,2]=(-0.059093568436)*x[2]
        arg[1,3,2,0]=(0.651004255104)*x[0]
        arg[1,3,2,1]=(-0.977880706193)*x[1]
        arg[1,3,2,2]=(0.370344651319)*x[2]
        arg[1,4,0,0]=(-0.503657215247)*x[0]
        arg[1,4,0,1]=(-0.170885297253)*x[1]
        arg[1,4,0,2]=(0.533424480956)*x[2]
        arg[1,4,1,0]=(-0.220533193308)*x[0]
        arg[1,4,1,1]=(0.344537611882)*x[1]
        arg[1,4,1,2]=(0.861473877282)*x[2]
        arg[1,4,2,0]=(-0.0923010438884)*x[0]
        arg[1,4,2,1]=(-0.338256780498)*x[1]
        arg[1,4,2,2]=(0.528567959345)*x[2]
        arg[2,0,0,0]=(-0.0423053381485)*x[0]
        arg[2,0,0,1]=(0.856798579151)*x[1]
        arg[2,0,0,2]=(0.383258153853)*x[2]
        arg[2,0,1,0]=(0.350994872736)*x[0]
        arg[2,0,1,1]=(-0.78055158106)*x[1]
        arg[2,0,1,2]=(-0.770876699915)*x[2]
        arg[2,0,2,0]=(-0.935133287106)*x[0]
        arg[2,0,2,1]=(0.618238076989)*x[1]
        arg[2,0,2,2]=(-0.846783087949)*x[2]
        arg[2,1,0,0]=(-0.657394511405)*x[0]
        arg[2,1,0,1]=(0.576218821654)*x[1]
        arg[2,1,0,2]=(0.0269446356493)*x[2]
        arg[2,1,1,0]=(-0.310710230949)*x[0]
        arg[2,1,1,1]=(0.425412515598)*x[1]
        arg[2,1,1,2]=(-0.225225290862)*x[2]
        arg[2,1,2,0]=(-0.539928589495)*x[0]
        arg[2,1,2,1]=(-0.348082121765)*x[1]
        arg[2,1,2,2]=(-0.0287274646233)*x[2]
        arg[2,2,0,0]=(0.746132865923)*x[0]
        arg[2,2,0,1]=(-0.0234203693548)*x[1]
        arg[2,2,0,2]=(0.517411821941)*x[2]
        arg[2,2,1,0]=(-0.183204217349)*x[0]
        arg[2,2,1,1]=(0.714988861836)*x[1]
        arg[2,2,1,2]=(0.829083318937)*x[2]
        arg[2,2,2,0]=(0.458067555841)*x[0]
        arg[2,2,2,1]=(0.639317125869)*x[1]
        arg[2,2,2,2]=(0.104611520408)*x[2]
        arg[2,3,0,0]=(-0.420513461135)*x[0]
        arg[2,3,0,1]=(0.888686162754)*x[1]
        arg[2,3,0,2]=(0.939305777879)*x[2]
        arg[2,3,1,0]=(0.856795132015)*x[0]
        arg[2,3,1,1]=(0.817593141895)*x[1]
        arg[2,3,1,2]=(0.962503342535)*x[2]
        arg[2,3,2,0]=(-0.334586700245)*x[0]
        arg[2,3,2,1]=(0.182696129528)*x[1]
        arg[2,3,2,2]=(-0.707271571206)*x[2]
        arg[2,4,0,0]=(-0.594795981069)*x[0]
        arg[2,4,0,1]=(0.451239168073)*x[1]
        arg[2,4,0,2]=(0.191212211556)*x[2]
        arg[2,4,1,0]=(-0.503465984944)*x[0]
        arg[2,4,1,1]=(0.725377884208)*x[1]
        arg[2,4,1,2]=(-0.40719255752)*x[2]
        arg[2,4,2,0]=(0.268071476451)*x[0]
        arg[2,4,2,1]=(0.85066639942)*x[1]
        arg[2,4,2,2]=(-0.906021406945)*x[2]
        arg[3,0,0,0]=(-0.922179152122)*x[0]
        arg[3,0,0,1]=(-0.0903841240007)*x[1]
        arg[3,0,0,2]=(-0.751482803516)*x[2]
        arg[3,0,1,0]=(0.960697809119)*x[0]
        arg[3,0,1,1]=(0.638878873158)*x[1]
        arg[3,0,1,2]=(0.390932234724)*x[2]
        arg[3,0,2,0]=(-0.925078301694)*x[0]
        arg[3,0,2,1]=(0.793590580665)*x[1]
        arg[3,0,2,2]=(0.535478366911)*x[2]
        arg[3,1,0,0]=(-0.431951993217)*x[0]
        arg[3,1,0,1]=(0.211750261417)*x[1]
        arg[3,1,0,2]=(-0.930706580442)*x[2]
        arg[3,1,1,0]=(0.330979313323)*x[0]
        arg[3,1,1,1]=(-0.838919076081)*x[1]
        arg[3,1,1,2]=(0.134250050168)*x[2]
        arg[3,1,2,0]=(0.414922811301)*x[0]
        arg[3,1,2,1]=(-0.663692878121)*x[1]
        arg[3,1,2,2]=(0.88499278543)*x[2]
        arg[3,2,0,0]=(0.0884662742233)*x[0]
        arg[3,2,0,1]=(-0.412630722821)*x[1]
        arg[3,2,0,2]=(-0.730850884928)*x[2]
        arg[3,2,1,0]=(0.722207547366)*x[0]
        arg[3,2,1,1]=(-0.260067950749)*x[1]
        arg[3,2,1,2]=(0.426259201494)*x[2]
        arg[3,2,2,0]=(0.0516111795322)*x[0]
        arg[3,2,2,1]=(0.922853710048)*x[1]
        arg[3,2,2,2]=(-0.991912758116)*x[2]
        arg[3,3,0,0]=(0.263933965905)*x[0]
        arg[3,3,0,1]=(0.840541758799)*x[1]
        arg[3,3,0,2]=(0.417658511125)*x[2]
        arg[3,3,1,0]=(-0.901745614723)*x[0]
        arg[3,3,1,1]=(-0.623608908699)*x[1]
        arg[3,3,1,2]=(0.0522167208784)*x[2]
        arg[3,3,2,0]=(-0.549431264931)*x[0]
        arg[3,3,2,1]=(0.919971855457)*x[1]
        arg[3,3,2,2]=(0.142757773397)*x[2]
        arg[3,4,0,0]=(-0.258875259824)*x[0]
        arg[3,4,0,1]=(-0.0373872187041)*x[1]
        arg[3,4,0,2]=(0.445989164864)*x[2]
        arg[3,4,1,0]=(0.0273711397038)*x[0]
        arg[3,4,1,1]=(-0.85522629)*x[1]
        arg[3,4,1,2]=(-0.835392581226)*x[2]
        arg[3,4,2,0]=(0.402929450189)*x[0]
        arg[3,4,2,1]=(0.549623033221)*x[1]
        arg[3,4,2,2]=(0.237803212109)*x[2]
        ref=sqrt((15.1448100099)+(3.61598249819))

      res=L2(arg)
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertAlmostEqual(res,ref,int(-log10(self.RES_TOL)),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_L2_onFunctionOnContactZero_fromSymbol_rank0(self):
      """
      tests L2-norm of Symbol on the FunctionOnContactZero

      assumptions: self.domain supports integration on FunctionOnContactZero
      """
      dim=self.domain.getDim()
      w=FunctionOnContactZero(self.domain)
      x=w.getX()
      if dim==2:
        arg=Symbol(shape=(),dim=dim)
        s=Data(0,(),w)
        s=(-0.833494674242)*x[0]
        ref=sqrt((0.0)+(0.173678342998))

      else:
        arg=Symbol(shape=(),dim=dim)
        s=Data(0,(),w)
        s=(-0.413475598947)*x[0]
        ref=sqrt((0.0)+(0.0427405177311))

      res=L2(arg)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertAlmostEqual(sub,ref,int(-log10(self.RES_TOL)),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_L2_onFunctionOnContactZero_fromSymbol_rank1(self):
      """
      tests L2-norm of Symbol on the FunctionOnContactZero

      assumptions: self.domain supports integration on FunctionOnContactZero
      """
      dim=self.domain.getDim()
      w=FunctionOnContactZero(self.domain)
      x=w.getX()
      if dim==2:
        arg=Symbol(shape=(2,),dim=dim)
        s=Data(0,(2,),w)
        s[0]=(-0.836300232979)*x[0]
        s[1]=(-0.855949069442)*x[1]
        ref=sqrt((0.244216269826)+(0.17484951992))

      else:
        arg=Symbol(shape=(3,),dim=dim)
        s=Data(0,(3,),w)
        s[0]=(0.604037678348)*x[0]
        s[1]=(0.173183346503)*x[1]
        s[2]=(0.0242607956405)*x[2]
        ref=sqrt((0.0101936859037)+(0.091215379216))

      res=L2(arg)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertAlmostEqual(sub,ref,int(-log10(self.RES_TOL)),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_L2_onFunctionOnContactZero_fromSymbol_rank2(self):
      """
      tests L2-norm of Symbol on the FunctionOnContactZero

      assumptions: self.domain supports integration on FunctionOnContactZero
      """
      dim=self.domain.getDim()
      w=FunctionOnContactZero(self.domain)
      x=w.getX()
      if dim==2:
        arg=Symbol(shape=(4, 2),dim=dim)
        s=Data(0,(4, 2),w)
        s[0,0]=(-0.482686145994)*x[0]
        s[0,1]=(-0.948811640283)*x[1]
        s[1,0]=(-0.992141136202)*x[0]
        s[1,1]=(0.966819328886)*x[1]
        s[2,0]=(0.948775134068)*x[0]
        s[2,1]=(-0.468349890651)*x[1]
        s[3,0]=(-0.671716588929)*x[0]
        s[3,1]=(-0.340484526166)*x[1]
        ref=sqrt((0.723421492026)+(0.642176845137))

      else:
        arg=Symbol(shape=(4, 3),dim=dim)
        s=Data(0,(4, 3),w)
        s[0,0]=(-0.891790655048)*x[0]
        s[0,1]=(-0.245798024334)*x[1]
        s[0,2]=(0.0537464591204)*x[2]
        s[1,0]=(0.230259637248)*x[0]
        s[1,1]=(0.0816863818806)*x[1]
        s[1,2]=(0.000519957155649)*x[2]
        s[2,0]=(0.0828558901056)*x[0]
        s[2,1]=(0.831425281201)*x[1]
        s[2,2]=(-0.601659198259)*x[2]
        s[3,0]=(-0.499606719218)*x[0]
        s[3,1]=(0.0962103643534)*x[1]
        s[3,2]=(0.997600764289)*x[2]
        ref=sqrt((0.709234598055)+(0.276195511347))

      res=L2(arg)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertAlmostEqual(sub,ref,int(-log10(self.RES_TOL)),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_L2_onFunctionOnContactZero_fromSymbol_rank3(self):
      """
      tests L2-norm of Symbol on the FunctionOnContactZero

      assumptions: self.domain supports integration on FunctionOnContactZero
      """
      dim=self.domain.getDim()
      w=FunctionOnContactZero(self.domain)
      x=w.getX()
      if dim==2:
        arg=Symbol(shape=(6, 2, 2),dim=dim)
        s=Data(0,(6, 2, 2),w)
        s[0,0,0]=(0.991681821415)*x[0]
        s[0,0,1]=(-0.224425650593)*x[1]
        s[0,1,0]=(-0.876361852829)*x[0]
        s[0,1,1]=(0.537837501136)*x[1]
        s[1,0,0]=(-0.54481895152)*x[0]
        s[1,0,1]=(0.593327791724)*x[1]
        s[1,1,0]=(0.418004915001)*x[0]
        s[1,1,1]=(-0.192957383127)*x[1]
        s[2,0,0]=(0.201546374554)*x[0]
        s[2,0,1]=(-0.265198464218)*x[1]
        s[2,1,0]=(-0.965663986948)*x[0]
        s[2,1,1]=(-0.323140398665)*x[1]
        s[3,0,0]=(0.233122332146)*x[0]
        s[3,0,1]=(-0.121823927196)*x[1]
        s[3,1,0]=(0.292287834978)*x[0]
        s[3,1,1]=(0.932977541215)*x[1]
        s[4,0,0]=(-0.345560503758)*x[0]
        s[4,0,1]=(0.192322796658)*x[1]
        s[4,1,0]=(0.0397082772329)*x[0]
        s[4,1,1]=(0.659346662851)*x[1]
        s[5,0,0]=(-0.195000663748)*x[0]
        s[5,0,1]=(-0.214222081491)*x[1]
        s[5,1,0]=(0.830046961039)*x[0]
        s[5,1,1]=(0.535960833948)*x[1]
        ref=sqrt((0.864605256793)+(1.04597420834))

      else:
        arg=Symbol(shape=(6, 2, 3),dim=dim)
        s=Data(0,(6, 2, 3),w)
        s[0,0,0]=(-0.660450094858)*x[0]
        s[0,0,1]=(-0.383241027824)*x[1]
        s[0,0,2]=(0.615453081929)*x[2]
        s[0,1,0]=(0.0679578237259)*x[0]
        s[0,1,1]=(0.801162277845)*x[1]
        s[0,1,2]=(0.741261458673)*x[2]
        s[1,0,0]=(0.0391969780398)*x[0]
        s[1,0,1]=(-0.400798945746)*x[1]
        s[1,0,2]=(-0.00301944444045)*x[2]
        s[1,1,0]=(-0.473988187262)*x[0]
        s[1,1,1]=(0.124770000361)*x[1]
        s[1,1,2]=(-0.65645884933)*x[2]
        s[2,0,0]=(-0.849984307849)*x[0]
        s[2,0,1]=(-0.992314728332)*x[1]
        s[2,0,2]=(-0.221196101266)*x[2]
        s[2,1,0]=(0.303587301796)*x[0]
        s[2,1,1]=(-0.276498999588)*x[1]
        s[2,1,2]=(0.341982161276)*x[2]
        s[3,0,0]=(-0.479181710128)*x[0]
        s[3,0,1]=(-0.799175074635)*x[1]
        s[3,0,2]=(-0.741790237186)*x[2]
        s[3,1,0]=(0.512279309628)*x[0]
        s[3,1,1]=(-0.346869946977)*x[1]
        s[3,1,2]=(-0.559402479335)*x[2]
        s[4,0,0]=(0.449912947237)*x[0]
        s[4,0,1]=(-0.0398661706497)*x[1]
        s[4,0,2]=(-0.856490275998)*x[2]
        s[4,1,0]=(-0.720500133511)*x[0]
        s[4,1,1]=(-0.97292914508)*x[1]
        s[4,1,2]=(-0.151959359468)*x[2]
        s[5,0,0]=(0.397527396462)*x[0]
        s[5,0,1]=(-0.151341467662)*x[1]
        s[5,0,2]=(-0.202710617619)*x[2]
        s[5,1,0]=(0.386974642401)*x[0]
        s[5,1,1]=(0.103900256892)*x[1]
        s[5,1,2]=(-0.152493955764)*x[2]
        ref=sqrt((2.3254122568)+(0.750754270358))

      res=L2(arg)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertAlmostEqual(sub,ref,int(-log10(self.RES_TOL)),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_L2_onFunctionOnContactZero_fromSymbol_rank4(self):
      """
      tests L2-norm of Symbol on the FunctionOnContactZero

      assumptions: self.domain supports integration on FunctionOnContactZero
      """
      dim=self.domain.getDim()
      w=FunctionOnContactZero(self.domain)
      x=w.getX()
      if dim==2:
        arg=Symbol(shape=(4, 5, 3, 2),dim=dim)
        s=Data(0,(4, 5, 3, 2),w)
        s[0,0,0,0]=(0.914300112399)*x[0]
        s[0,0,0,1]=(0.800216576034)*x[1]
        s[0,0,1,0]=(-0.396870583938)*x[0]
        s[0,0,1,1]=(0.867325478217)*x[1]
        s[0,0,2,0]=(-0.200425849404)*x[0]
        s[0,0,2,1]=(0.562512527677)*x[1]
        s[0,1,0,0]=(-0.26457813536)*x[0]
        s[0,1,0,1]=(-0.865366345944)*x[1]
        s[0,1,1,0]=(0.906255889925)*x[0]
        s[0,1,1,1]=(-0.144426923939)*x[1]
        s[0,1,2,0]=(-0.835643108487)*x[0]
        s[0,1,2,1]=(-0.644514669599)*x[1]
        s[0,2,0,0]=(0.516524292926)*x[0]
        s[0,2,0,1]=(0.825766812215)*x[1]
        s[0,2,1,0]=(-0.37867452414)*x[0]
        s[0,2,1,1]=(0.920377065217)*x[1]
        s[0,2,2,0]=(0.0937552904916)*x[0]
        s[0,2,2,1]=(0.378732372322)*x[1]
        s[0,3,0,0]=(0.496752465789)*x[0]
        s[0,3,0,1]=(0.427047604935)*x[1]
        s[0,3,1,0]=(0.750830919575)*x[0]
        s[0,3,1,1]=(0.576559278913)*x[1]
        s[0,3,2,0]=(-0.115508523662)*x[0]
        s[0,3,2,1]=(0.737623150949)*x[1]
        s[0,4,0,0]=(0.440274015173)*x[0]
        s[0,4,0,1]=(-0.450295347264)*x[1]
        s[0,4,1,0]=(0.170660831663)*x[0]
        s[0,4,1,1]=(-0.00668299893696)*x[1]
        s[0,4,2,0]=(0.443915340807)*x[0]
        s[0,4,2,1]=(-0.096469220433)*x[1]
        s[1,0,0,0]=(0.124319331592)*x[0]
        s[1,0,0,1]=(0.0209567989245)*x[1]
        s[1,0,1,0]=(-0.120682670913)*x[0]
        s[1,0,1,1]=(-0.0668286988997)*x[1]
        s[1,0,2,0]=(-0.622418696038)*x[0]
        s[1,0,2,1]=(0.690522460229)*x[1]
        s[1,1,0,0]=(-0.564034328652)*x[0]
        s[1,1,0,1]=(-0.488373582152)*x[1]
        s[1,1,1,0]=(0.37807858594)*x[0]
        s[1,1,1,1]=(0.607009462361)*x[1]
        s[1,1,2,0]=(0.357544289055)*x[0]
        s[1,1,2,1]=(0.89416048362)*x[1]
        s[1,2,0,0]=(0.180403414369)*x[0]
        s[1,2,0,1]=(-0.590910014416)*x[1]
        s[1,2,1,0]=(-0.0184755955417)*x[0]
        s[1,2,1,1]=(0.84738705179)*x[1]
        s[1,2,2,0]=(-0.677245742487)*x[0]
        s[1,2,2,1]=(-0.457219426905)*x[1]
        s[1,3,0,0]=(-0.454834820082)*x[0]
        s[1,3,0,1]=(-0.725915297395)*x[1]
        s[1,3,1,0]=(0.184608835513)*x[0]
        s[1,3,1,1]=(0.313259821447)*x[1]
        s[1,3,2,0]=(-0.252818794924)*x[0]
        s[1,3,2,1]=(-0.654170873581)*x[1]
        s[1,4,0,0]=(-0.315092745211)*x[0]
        s[1,4,0,1]=(-0.922091825394)*x[1]
        s[1,4,1,0]=(-0.386863810397)*x[0]
        s[1,4,1,1]=(0.430983629642)*x[1]
        s[1,4,2,0]=(-0.762809887523)*x[0]
        s[1,4,2,1]=(0.58896325465)*x[1]
        s[2,0,0,0]=(-0.456340727871)*x[0]
        s[2,0,0,1]=(-0.0187981167144)*x[1]
        s[2,0,1,0]=(-0.525249731866)*x[0]
        s[2,0,1,1]=(0.399070164526)*x[1]
        s[2,0,2,0]=(-0.201772101073)*x[0]
        s[2,0,2,1]=(-0.0968197655895)*x[1]
        s[2,1,0,0]=(0.191048791069)*x[0]
        s[2,1,0,1]=(0.434067010545)*x[1]
        s[2,1,1,0]=(0.848522099444)*x[0]
        s[2,1,1,1]=(0.696991804739)*x[1]
        s[2,1,2,0]=(0.220121221699)*x[0]
        s[2,1,2,1]=(0.301203065888)*x[1]
        s[2,2,0,0]=(0.766472384148)*x[0]
        s[2,2,0,1]=(-0.0714402172399)*x[1]
        s[2,2,1,0]=(0.492328493075)*x[0]
        s[2,2,1,1]=(0.558080277043)*x[1]
        s[2,2,2,0]=(0.469934590015)*x[0]
        s[2,2,2,1]=(0.758048072142)*x[1]
        s[2,3,0,0]=(0.085308155098)*x[0]
        s[2,3,0,1]=(0.297189197296)*x[1]
        s[2,3,1,0]=(0.978282022769)*x[0]
        s[2,3,1,1]=(0.682153031605)*x[1]
        s[2,3,2,0]=(-0.561125939181)*x[0]
        s[2,3,2,1]=(0.440224855986)*x[1]
        s[2,4,0,0]=(0.72050530483)*x[0]
        s[2,4,0,1]=(-0.206652409514)*x[1]
        s[2,4,1,0]=(0.387667449577)*x[0]
        s[2,4,1,1]=(0.493459617619)*x[1]
        s[2,4,2,0]=(0.688054409914)*x[0]
        s[2,4,2,1]=(0.971336356338)*x[1]
        s[3,0,0,0]=(0.861462391015)*x[0]
        s[3,0,0,1]=(0.0554843601356)*x[1]
        s[3,0,1,0]=(0.83937959439)*x[0]
        s[3,0,1,1]=(0.954697475987)*x[1]
        s[3,0,2,0]=(-0.997867671237)*x[0]
        s[3,0,2,1]=(-0.218317664783)*x[1]
        s[3,1,0,0]=(-0.262679977079)*x[0]
        s[3,1,0,1]=(0.0145063905029)*x[1]
        s[3,1,1,0]=(0.130108953527)*x[0]
        s[3,1,1,1]=(0.9110336683)*x[1]
        s[3,1,2,0]=(0.278623938991)*x[0]
        s[3,1,2,1]=(-0.374239642631)*x[1]
        s[3,2,0,0]=(0.720411588365)*x[0]
        s[3,2,0,1]=(0.794261999505)*x[1]
        s[3,2,1,0]=(0.098798147685)*x[0]
        s[3,2,1,1]=(-0.338610161254)*x[1]
        s[3,2,2,0]=(0.961927950853)*x[0]
        s[3,2,2,1]=(-0.307621697905)*x[1]
        s[3,3,0,0]=(-0.73836606888)*x[0]
        s[3,3,0,1]=(0.714826978574)*x[1]
        s[3,3,1,0]=(-0.394351029155)*x[0]
        s[3,3,1,1]=(0.408664173503)*x[1]
        s[3,3,2,0]=(0.844614972483)*x[0]
        s[3,3,2,1]=(-0.478702935986)*x[1]
        s[3,4,0,0]=(0.0447493337692)*x[0]
        s[3,4,0,1]=(0.317235470904)*x[1]
        s[3,4,1,0]=(-0.960879888735)*x[0]
        s[3,4,1,1]=(-0.349654948749)*x[1]
        s[3,4,2,0]=(0.15351062359)*x[0]
        s[3,4,2,1]=(-0.30302268499)*x[1]
        ref=sqrt((6.41155668825)+(4.53628477602))

      else:
        arg=Symbol(shape=(4, 5, 3, 3),dim=dim)
        s=Data(0,(4, 5, 3, 3),w)
        s[0,0,0,0]=(-0.89168364071)*x[0]
        s[0,0,0,1]=(0.656887229548)*x[1]
        s[0,0,0,2]=(0.0379592114434)*x[2]
        s[0,0,1,0]=(-0.499743077643)*x[0]
        s[0,0,1,1]=(0.433796934727)*x[1]
        s[0,0,1,2]=(-0.419639340675)*x[2]
        s[0,0,2,0]=(-0.229849090155)*x[0]
        s[0,0,2,1]=(0.48808994712)*x[1]
        s[0,0,2,2]=(0.424012703453)*x[2]
        s[0,1,0,0]=(0.498682171432)*x[0]
        s[0,1,0,1]=(0.623180282688)*x[1]
        s[0,1,0,2]=(0.467278343783)*x[2]
        s[0,1,1,0]=(-0.461240789999)*x[0]
        s[0,1,1,1]=(0.79579036671)*x[1]
        s[0,1,1,2]=(0.0336609748509)*x[2]
        s[0,1,2,0]=(-0.758182366212)*x[0]
        s[0,1,2,1]=(0.335354770639)*x[1]
        s[0,1,2,2]=(-0.43256653064)*x[2]
        s[0,2,0,0]=(0.267085562446)*x[0]
        s[0,2,0,1]=(0.74096852219)*x[1]
        s[0,2,0,2]=(0.884306535005)*x[2]
        s[0,2,1,0]=(0.839435822303)*x[0]
        s[0,2,1,1]=(0.981598566715)*x[1]
        s[0,2,1,2]=(0.178444224041)*x[2]
        s[0,2,2,0]=(0.28828503848)*x[0]
        s[0,2,2,1]=(-0.626210031155)*x[1]
        s[0,2,2,2]=(-0.69299554971)*x[2]
        s[0,3,0,0]=(-0.0218942570636)*x[0]
        s[0,3,0,1]=(0.542278046952)*x[1]
        s[0,3,0,2]=(0.953044269075)*x[2]
        s[0,3,1,0]=(-0.557505895397)*x[0]
        s[0,3,1,1]=(0.523921264225)*x[1]
        s[0,3,1,2]=(-0.786147057715)*x[2]
        s[0,3,2,0]=(-0.89329999988)*x[0]
        s[0,3,2,1]=(0.462215391096)*x[1]
        s[0,3,2,2]=(-0.374927543026)*x[2]
        s[0,4,0,0]=(-0.247805259481)*x[0]
        s[0,4,0,1]=(0.645841666564)*x[1]
        s[0,4,0,2]=(0.16650874055)*x[2]
        s[0,4,1,0]=(0.266137626404)*x[0]
        s[0,4,1,1]=(-0.288059047514)*x[1]
        s[0,4,1,2]=(-0.486812056856)*x[2]
        s[0,4,2,0]=(0.688344153316)*x[0]
        s[0,4,2,1]=(-0.655455791821)*x[1]
        s[0,4,2,2]=(-0.348241680636)*x[2]
        s[1,0,0,0]=(0.762345221451)*x[0]
        s[1,0,0,1]=(0.264502883884)*x[1]
        s[1,0,0,2]=(0.48788907144)*x[2]
        s[1,0,1,0]=(0.42005226242)*x[0]
        s[1,0,1,1]=(-0.825723492918)*x[1]
        s[1,0,1,2]=(0.797321195042)*x[2]
        s[1,0,2,0]=(-0.19639010863)*x[0]
        s[1,0,2,1]=(-0.702950524388)*x[1]
        s[1,0,2,2]=(0.235603192829)*x[2]
        s[1,1,0,0]=(-0.454504750183)*x[0]
        s[1,1,0,1]=(0.31590384254)*x[1]
        s[1,1,0,2]=(-0.19227935466)*x[2]
        s[1,1,1,0]=(0.0070439759207)*x[0]
        s[1,1,1,1]=(-0.293083678876)*x[1]
        s[1,1,1,2]=(-0.878745990375)*x[2]
        s[1,1,2,0]=(-0.192794025412)*x[0]
        s[1,1,2,1]=(0.648895386958)*x[1]
        s[1,1,2,2]=(0.775208361981)*x[2]
        s[1,2,0,0]=(-0.368309400642)*x[0]
        s[1,2,0,1]=(-0.917724017536)*x[1]
        s[1,2,0,2]=(0.920777364818)*x[2]
        s[1,2,1,0]=(-0.645052443283)*x[0]
        s[1,2,1,1]=(0.139033194163)*x[1]
        s[1,2,1,2]=(-0.0404331243874)*x[2]
        s[1,2,2,0]=(-0.61245787618)*x[0]
        s[1,2,2,1]=(0.0482137893562)*x[1]
        s[1,2,2,2]=(0.179049768973)*x[2]
        s[1,3,0,0]=(-0.896233852085)*x[0]
        s[1,3,0,1]=(-0.681461465854)*x[1]
        s[1,3,0,2]=(-0.446646723524)*x[2]
        s[1,3,1,0]=(-0.555003216598)*x[0]
        s[1,3,1,1]=(0.35239325809)*x[1]
        s[1,3,1,2]=(0.39507338896)*x[2]
        s[1,3,2,0]=(-0.13811362276)*x[0]
        s[1,3,2,1]=(-0.374436915851)*x[1]
        s[1,3,2,2]=(0.0513289236564)*x[2]
        s[1,4,0,0]=(-0.687683310284)*x[0]
        s[1,4,0,1]=(0.572359319345)*x[1]
        s[1,4,0,2]=(-0.814515142116)*x[2]
        s[1,4,1,0]=(-0.565292086642)*x[0]
        s[1,4,1,1]=(-0.191646702103)*x[1]
        s[1,4,1,2]=(-0.292049727092)*x[2]
        s[1,4,2,0]=(0.452055314722)*x[0]
        s[1,4,2,1]=(-0.457431088076)*x[1]
        s[1,4,2,2]=(-0.178361949498)*x[2]
        s[2,0,0,0]=(0.354814016099)*x[0]
        s[2,0,0,1]=(-0.384609805819)*x[1]
        s[2,0,0,2]=(-0.281337028672)*x[2]
        s[2,0,1,0]=(0.142919596773)*x[0]
        s[2,0,1,1]=(-0.57612422002)*x[1]
        s[2,0,1,2]=(-0.529853258262)*x[2]
        s[2,0,2,0]=(0.80636898712)*x[0]
        s[2,0,2,1]=(0.106993663032)*x[1]
        s[2,0,2,2]=(0.255600789365)*x[2]
        s[2,1,0,0]=(0.978145357315)*x[0]
        s[2,1,0,1]=(-0.901086815575)*x[1]
        s[2,1,0,2]=(0.648553017338)*x[2]
        s[2,1,1,0]=(-0.283065724087)*x[0]
        s[2,1,1,1]=(-0.0976255838643)*x[1]
        s[2,1,1,2]=(-0.518416419376)*x[2]
        s[2,1,2,0]=(0.572224523241)*x[0]
        s[2,1,2,1]=(0.676830468876)*x[1]
        s[2,1,2,2]=(0.347547333574)*x[2]
        s[2,2,0,0]=(-0.890366473527)*x[0]
        s[2,2,0,1]=(-0.202566992837)*x[1]
        s[2,2,0,2]=(-0.00994077956762)*x[2]
        s[2,2,1,0]=(-0.312288044658)*x[0]
        s[2,2,1,1]=(0.215695301526)*x[1]
        s[2,2,1,2]=(0.134196622741)*x[2]
        s[2,2,2,0]=(-0.564325028454)*x[0]
        s[2,2,2,1]=(-0.707031226029)*x[1]
        s[2,2,2,2]=(-0.793190569275)*x[2]
        s[2,3,0,0]=(0.28006826199)*x[0]
        s[2,3,0,1]=(0.743445147616)*x[1]
        s[2,3,0,2]=(0.57457392009)*x[2]
        s[2,3,1,0]=(-0.353169838541)*x[0]
        s[2,3,1,1]=(-0.353550104906)*x[1]
        s[2,3,1,2]=(0.26061713616)*x[2]
        s[2,3,2,0]=(0.360387490548)*x[0]
        s[2,3,2,1]=(0.532836358847)*x[1]
        s[2,3,2,2]=(0.517337142155)*x[2]
        s[2,4,0,0]=(0.969178027599)*x[0]
        s[2,4,0,1]=(-0.0671877983392)*x[1]
        s[2,4,0,2]=(0.692408837657)*x[2]
        s[2,4,1,0]=(0.988861077126)*x[0]
        s[2,4,1,1]=(0.987792613173)*x[1]
        s[2,4,1,2]=(-0.147075972258)*x[2]
        s[2,4,2,0]=(-0.440780900369)*x[0]
        s[2,4,2,1]=(0.390407807605)*x[1]
        s[2,4,2,2]=(-0.893694860437)*x[2]
        s[3,0,0,0]=(0.699550432817)*x[0]
        s[3,0,0,1]=(-0.787672358838)*x[1]
        s[3,0,0,2]=(-0.583257623239)*x[2]
        s[3,0,1,0]=(-0.157588638816)*x[0]
        s[3,0,1,1]=(-0.986238064621)*x[1]
        s[3,0,1,2]=(0.699443634206)*x[2]
        s[3,0,2,0]=(-0.222407346878)*x[0]
        s[3,0,2,1]=(-0.308189430231)*x[1]
        s[3,0,2,2]=(0.299965297762)*x[2]
        s[3,1,0,0]=(-0.412975772409)*x[0]
        s[3,1,0,1]=(0.717170353427)*x[1]
        s[3,1,0,2]=(0.555847828902)*x[2]
        s[3,1,1,0]=(0.301148971297)*x[0]
        s[3,1,1,1]=(0.556400276399)*x[1]
        s[3,1,1,2]=(0.473027697568)*x[2]
        s[3,1,2,0]=(-0.701958828234)*x[0]
        s[3,1,2,1]=(-0.965435432938)*x[1]
        s[3,1,2,2]=(0.226965032076)*x[2]
        s[3,2,0,0]=(0.548493161421)*x[0]
        s[3,2,0,1]=(-0.422082327803)*x[1]
        s[3,2,0,2]=(-0.64432755028)*x[2]
        s[3,2,1,0]=(-0.821784569582)*x[0]
        s[3,2,1,1]=(0.567506242312)*x[1]
        s[3,2,1,2]=(0.58945700083)*x[2]
        s[3,2,2,0]=(-0.31431442897)*x[0]
        s[3,2,2,1]=(-0.727570436185)*x[1]
        s[3,2,2,2]=(-0.234428528039)*x[2]
        s[3,3,0,0]=(-0.184478413951)*x[0]
        s[3,3,0,1]=(0.54667291001)*x[1]
        s[3,3,0,2]=(0.266086928294)*x[2]
        s[3,3,1,0]=(0.0279847154277)*x[0]
        s[3,3,1,1]=(-0.0851677217034)*x[1]
        s[3,3,1,2]=(-0.679725199609)*x[2]
        s[3,3,2,0]=(0.570361871278)*x[0]
        s[3,3,2,1]=(0.913879855699)*x[1]
        s[3,3,2,2]=(-0.036149215115)*x[2]
        s[3,4,0,0]=(0.430041266053)*x[0]
        s[3,4,0,1]=(-0.468708383807)*x[1]
        s[3,4,0,2]=(0.430318326847)*x[2]
        s[3,4,1,0]=(0.808876543295)*x[0]
        s[3,4,1,1]=(0.175574603952)*x[1]
        s[3,4,1,2]=(-0.982042631943)*x[2]
        s[3,4,2,0]=(0.730923035545)*x[0]
        s[3,4,2,1]=(0.65499693359)*x[1]
        s[3,4,2,2]=(0.678161392893)*x[2]
        ref=sqrt((12.3860143155)+(4.68566966204))

      res=L2(arg)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertAlmostEqual(sub,ref,int(-log10(self.RES_TOL)),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_L2_onReducedFunctionOnContactZero_fromData_rank0(self):
      """
      tests L2-norm of Data on the ReducedFunctionOnContactZero

      assumptions: self.domain supports integration on ReducedFunctionOnContactZero
      """
      dim=self.domain.getDim()
      w=ReducedFunctionOnContactZero(self.domain)
      x=w.getX()
      if dim==2:
        arg=Data(0,(),w)
        arg=(-0.908822474879)*sqrt(x[0])
        ref=sqrt((0.0)*3./2.+(0.206489572711)/0.5)

      else:
        arg=Data(0,(),w)
        arg=(-0.893168521371)*sqrt(x[0])
        ref=sqrt((0.0)*3./2.+(0.199437501892)/0.5)

      res=L2(arg)
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertAlmostEqual(res,ref,int(-log10(self.RES_TOL)),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_L2_onReducedFunctionOnContactZero_fromData_rank1(self):
      """
      tests L2-norm of Data on the ReducedFunctionOnContactZero

      assumptions: self.domain supports integration on ReducedFunctionOnContactZero
      """
      dim=self.domain.getDim()
      w=ReducedFunctionOnContactZero(self.domain)
      x=w.getX()
      if dim==2:
        arg=Data(0,(2,),w)
        arg[0]=(-0.30839170903)*sqrt(x[0])
        arg[1]=(0.108352504587)*sqrt(x[1])
        ref=sqrt((0.00391342175007)*3./2.+(0.0237763615497)/0.5)

      else:
        arg=Data(0,(3,),w)
        arg[0]=(-0.260572084359)*sqrt(x[0])
        arg[1]=(0.250696802346)*sqrt(x[1])
        arg[2]=(-0.498857619202)*sqrt(x[2])
        ref=sqrt((0.103902603647)*3./2.+(0.0169744527868)/0.5)

      res=L2(arg)
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertAlmostEqual(res,ref,int(-log10(self.RES_TOL)),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_L2_onReducedFunctionOnContactZero_fromData_rank2(self):
      """
      tests L2-norm of Data on the ReducedFunctionOnContactZero

      assumptions: self.domain supports integration on ReducedFunctionOnContactZero
      """
      dim=self.domain.getDim()
      w=ReducedFunctionOnContactZero(self.domain)
      x=w.getX()
      if dim==2:
        arg=Data(0,(4, 2),w)
        arg[0,0]=(0.0107777943992)*sqrt(x[0])
        arg[0,1]=(0.0188929341636)*sqrt(x[1])
        arg[1,0]=(-0.00268267834861)*sqrt(x[0])
        arg[1,1]=(-0.656039971477)*sqrt(x[1])
        arg[2,0]=(-0.997645982047)*sqrt(x[0])
        arg[2,1]=(0.993049748017)*sqrt(x[1])
        arg[3,0]=(0.938636169945)*sqrt(x[0])
        arg[3,1]=(-0.609892243528)*sqrt(x[1])
        ref=sqrt((0.596287245963)*3./2.+(0.46911468066)/0.5)

      else:
        arg=Data(0,(4, 3),w)
        arg[0,0]=(0.128987155564)*sqrt(x[0])
        arg[0,1]=(-0.582660210551)*sqrt(x[1])
        arg[0,2]=(-0.605040149822)*sqrt(x[2])
        arg[1,0]=(0.494540007291)*sqrt(x[0])
        arg[1,1]=(0.563900397935)*sqrt(x[1])
        arg[1,2]=(-0.806214220737)*sqrt(x[2])
        arg[2,0]=(0.878027993704)*sqrt(x[0])
        arg[2,1]=(0.597165310427)*sqrt(x[1])
        arg[2,2]=(-0.444798130742)*sqrt(x[2])
        arg[3,0]=(0.982067213342)*sqrt(x[0])
        arg[3,1]=(-0.160013302763)*sqrt(x[1])
        arg[3,2]=(-0.490830124654)*sqrt(x[2])
        ref=sqrt((0.831500595262)*3./2.+(0.49914916859)/0.5)

      res=L2(arg)
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertAlmostEqual(res,ref,int(-log10(self.RES_TOL)),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_L2_onReducedFunctionOnContactZero_fromData_rank3(self):
      """
      tests L2-norm of Data on the ReducedFunctionOnContactZero

      assumptions: self.domain supports integration on ReducedFunctionOnContactZero
      """
      dim=self.domain.getDim()
      w=ReducedFunctionOnContactZero(self.domain)
      x=w.getX()
      if dim==2:
        arg=Data(0,(6, 2, 2),w)
        arg[0,0,0]=(0.83264345248)*sqrt(x[0])
        arg[0,0,1]=(-0.29697614845)*sqrt(x[1])
        arg[0,1,0]=(-0.903150797089)*sqrt(x[0])
        arg[0,1,1]=(-0.569048463149)*sqrt(x[1])
        arg[1,0,0]=(0.317087339385)*sqrt(x[0])
        arg[1,0,1]=(0.0120740865344)*sqrt(x[1])
        arg[1,1,0]=(-0.221145803457)*sqrt(x[0])
        arg[1,1,1]=(-0.15362409313)*sqrt(x[1])
        arg[2,0,0]=(0.459428065932)*sqrt(x[0])
        arg[2,0,1]=(-0.495488213623)*sqrt(x[1])
        arg[2,1,0]=(0.919114285084)*sqrt(x[0])
        arg[2,1,1]=(0.54511942052)*sqrt(x[1])
        arg[3,0,0]=(0.442207797529)*sqrt(x[0])
        arg[3,0,1]=(0.958341188449)*sqrt(x[1])
        arg[3,1,0]=(-0.0915205976682)*sqrt(x[0])
        arg[3,1,1]=(-0.410964460336)*sqrt(x[1])
        arg[4,0,0]=(-0.243030095673)*sqrt(x[0])
        arg[4,0,1]=(-0.413661756935)*sqrt(x[1])
        arg[4,1,0]=(-0.0973014277464)*sqrt(x[0])
        arg[4,1,1]=(0.204130676874)*sqrt(x[1])
        arg[5,0,0]=(0.38083750131)*sqrt(x[0])
        arg[5,0,1]=(0.286331574579)*sqrt(x[1])
        arg[5,1,0]=(0.583320730408)*sqrt(x[0])
        arg[5,1,1]=(-0.0899419806181)*sqrt(x[1])
        ref=sqrt((0.789530406063)*3./2.+(0.868006693351)/0.5)

      else:
        arg=Data(0,(6, 2, 3),w)
        arg[0,0,0]=(-0.98411764268)*sqrt(x[0])
        arg[0,0,1]=(-0.180549567137)*sqrt(x[1])
        arg[0,0,2]=(0.357923974709)*sqrt(x[2])
        arg[0,1,0]=(0.67691778218)*sqrt(x[0])
        arg[0,1,1]=(-0.643267669153)*sqrt(x[1])
        arg[0,1,2]=(0.452374328301)*sqrt(x[2])
        arg[1,0,0]=(0.494303694439)*sqrt(x[0])
        arg[1,0,1]=(0.00767629822214)*sqrt(x[1])
        arg[1,0,2]=(0.232436713422)*sqrt(x[2])
        arg[1,1,0]=(-0.698568975983)*sqrt(x[0])
        arg[1,1,1]=(-0.08483720353)*sqrt(x[1])
        arg[1,1,2]=(-0.723935799719)*sqrt(x[2])
        arg[2,0,0]=(0.931313239646)*sqrt(x[0])
        arg[2,0,1]=(-0.0988070442341)*sqrt(x[1])
        arg[2,0,2]=(-0.166201798278)*sqrt(x[2])
        arg[2,1,0]=(-0.733172171406)*sqrt(x[0])
        arg[2,1,1]=(0.24542706584)*sqrt(x[1])
        arg[2,1,2]=(0.49170717592)*sqrt(x[2])
        arg[3,0,0]=(-0.718346096869)*sqrt(x[0])
        arg[3,0,1]=(0.297442410539)*sqrt(x[1])
        arg[3,0,2]=(0.0277708718174)*sqrt(x[2])
        arg[3,1,0]=(0.735100702194)*sqrt(x[0])
        arg[3,1,1]=(-0.808007526296)*sqrt(x[1])
        arg[3,1,2]=(-0.423023238313)*sqrt(x[2])
        arg[4,0,0]=(0.383188529876)*sqrt(x[0])
        arg[4,0,1]=(-0.190101601814)*sqrt(x[1])
        arg[4,0,2]=(-0.404380990215)*sqrt(x[2])
        arg[4,1,0]=(-0.31098204217)*sqrt(x[0])
        arg[4,1,1]=(-0.715247945432)*sqrt(x[1])
        arg[4,1,2]=(0.435774772982)*sqrt(x[2])
        arg[5,0,0]=(-0.0662056096206)*sqrt(x[0])
        arg[5,0,1]=(0.746710247367)*sqrt(x[1])
        arg[5,0,2]=(0.753267249301)*sqrt(x[2])
        arg[5,1,0]=(0.258102962518)*sqrt(x[0])
        arg[5,1,1]=(-0.0270028483187)*sqrt(x[1])
        arg[5,1,2]=(-0.971060107954)*sqrt(x[2])
        ref=sqrt((1.86493016409)*3./2.+(1.23371587938)/0.5)

      res=L2(arg)
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertAlmostEqual(res,ref,int(-log10(self.RES_TOL)),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_L2_onReducedFunctionOnContactZero_fromData_rank4(self):
      """
      tests L2-norm of Data on the ReducedFunctionOnContactZero

      assumptions: self.domain supports integration on ReducedFunctionOnContactZero
      """
      dim=self.domain.getDim()
      w=ReducedFunctionOnContactZero(self.domain)
      x=w.getX()
      if dim==2:
        arg=Data(0,(4, 5, 3, 2),w)
        arg[0,0,0,0]=(-0.144741217139)*sqrt(x[0])
        arg[0,0,0,1]=(-0.188187241012)*sqrt(x[1])
        arg[0,0,1,0]=(0.056339373733)*sqrt(x[0])
        arg[0,0,1,1]=(0.222177130759)*sqrt(x[1])
        arg[0,0,2,0]=(-0.437240978497)*sqrt(x[0])
        arg[0,0,2,1]=(-0.479213082014)*sqrt(x[1])
        arg[0,1,0,0]=(0.343204141181)*sqrt(x[0])
        arg[0,1,0,1]=(0.099683148208)*sqrt(x[1])
        arg[0,1,1,0]=(0.167657906331)*sqrt(x[0])
        arg[0,1,1,1]=(0.371064917656)*sqrt(x[1])
        arg[0,1,2,0]=(-0.175796118619)*sqrt(x[0])
        arg[0,1,2,1]=(-0.523477567832)*sqrt(x[1])
        arg[0,2,0,0]=(0.188832282277)*sqrt(x[0])
        arg[0,2,0,1]=(0.948618356423)*sqrt(x[1])
        arg[0,2,1,0]=(-0.3835066594)*sqrt(x[0])
        arg[0,2,1,1]=(0.872875257641)*sqrt(x[1])
        arg[0,2,2,0]=(-0.98414676194)*sqrt(x[0])
        arg[0,2,2,1]=(0.59170993756)*sqrt(x[1])
        arg[0,3,0,0]=(0.875820385471)*sqrt(x[0])
        arg[0,3,0,1]=(-0.209139794169)*sqrt(x[1])
        arg[0,3,1,0]=(0.783312979835)*sqrt(x[0])
        arg[0,3,1,1]=(-0.440048708153)*sqrt(x[1])
        arg[0,3,2,0]=(-0.906741480158)*sqrt(x[0])
        arg[0,3,2,1]=(0.63834196559)*sqrt(x[1])
        arg[0,4,0,0]=(0.0142616166715)*sqrt(x[0])
        arg[0,4,0,1]=(-0.0745581409622)*sqrt(x[1])
        arg[0,4,1,0]=(-0.779966728505)*sqrt(x[0])
        arg[0,4,1,1]=(0.734744791584)*sqrt(x[1])
        arg[0,4,2,0]=(0.0407627699446)*sqrt(x[0])
        arg[0,4,2,1]=(-0.629666423098)*sqrt(x[1])
        arg[1,0,0,0]=(-0.782291768803)*sqrt(x[0])
        arg[1,0,0,1]=(-0.197799867335)*sqrt(x[1])
        arg[1,0,1,0]=(-0.487323212409)*sqrt(x[0])
        arg[1,0,1,1]=(-0.789940409633)*sqrt(x[1])
        arg[1,0,2,0]=(-0.523871448728)*sqrt(x[0])
        arg[1,0,2,1]=(-0.622272488658)*sqrt(x[1])
        arg[1,1,0,0]=(0.370095832941)*sqrt(x[0])
        arg[1,1,0,1]=(-0.0683624906905)*sqrt(x[1])
        arg[1,1,1,0]=(-0.180583695959)*sqrt(x[0])
        arg[1,1,1,1]=(-0.0529097308814)*sqrt(x[1])
        arg[1,1,2,0]=(0.440088045764)*sqrt(x[0])
        arg[1,1,2,1]=(-0.207665675854)*sqrt(x[1])
        arg[1,2,0,0]=(-0.360120735448)*sqrt(x[0])
        arg[1,2,0,1]=(0.877797020694)*sqrt(x[1])
        arg[1,2,1,0]=(0.655192721419)*sqrt(x[0])
        arg[1,2,1,1]=(0.978781227463)*sqrt(x[1])
        arg[1,2,2,0]=(-0.205886258554)*sqrt(x[0])
        arg[1,2,2,1]=(-0.0050396100363)*sqrt(x[1])
        arg[1,3,0,0]=(-0.484332943177)*sqrt(x[0])
        arg[1,3,0,1]=(0.964697361018)*sqrt(x[1])
        arg[1,3,1,0]=(0.0815652906824)*sqrt(x[0])
        arg[1,3,1,1]=(-0.531163025802)*sqrt(x[1])
        arg[1,3,2,0]=(-0.229615479833)*sqrt(x[0])
        arg[1,3,2,1]=(0.139971291609)*sqrt(x[1])
        arg[1,4,0,0]=(0.960839193726)*sqrt(x[0])
        arg[1,4,0,1]=(-0.9744930023)*sqrt(x[1])
        arg[1,4,1,0]=(-0.0919000363634)*sqrt(x[0])
        arg[1,4,1,1]=(0.483909186003)*sqrt(x[1])
        arg[1,4,2,0]=(0.248597552464)*sqrt(x[0])
        arg[1,4,2,1]=(0.965872402486)*sqrt(x[1])
        arg[2,0,0,0]=(-0.935049188033)*sqrt(x[0])
        arg[2,0,0,1]=(-0.823239964421)*sqrt(x[1])
        arg[2,0,1,0]=(0.30901193713)*sqrt(x[0])
        arg[2,0,1,1]=(-0.0538396976182)*sqrt(x[1])
        arg[2,0,2,0]=(-0.449416473099)*sqrt(x[0])
        arg[2,0,2,1]=(-0.857785267819)*sqrt(x[1])
        arg[2,1,0,0]=(-0.167802578334)*sqrt(x[0])
        arg[2,1,0,1]=(-0.7016101186)*sqrt(x[1])
        arg[2,1,1,0]=(0.995848317929)*sqrt(x[0])
        arg[2,1,1,1]=(-0.824595018011)*sqrt(x[1])
        arg[2,1,2,0]=(-0.826030527686)*sqrt(x[0])
        arg[2,1,2,1]=(-0.856151725651)*sqrt(x[1])
        arg[2,2,0,0]=(0.342047605028)*sqrt(x[0])
        arg[2,2,0,1]=(-0.342370307931)*sqrt(x[1])
        arg[2,2,1,0]=(-0.540693890831)*sqrt(x[0])
        arg[2,2,1,1]=(-0.720754897785)*sqrt(x[1])
        arg[2,2,2,0]=(-0.0941215154824)*sqrt(x[0])
        arg[2,2,2,1]=(0.115887914141)*sqrt(x[1])
        arg[2,3,0,0]=(-0.0920059639279)*sqrt(x[0])
        arg[2,3,0,1]=(0.910948619784)*sqrt(x[1])
        arg[2,3,1,0]=(-0.40846609126)*sqrt(x[0])
        arg[2,3,1,1]=(0.542728733954)*sqrt(x[1])
        arg[2,3,2,0]=(0.451361830368)*sqrt(x[0])
        arg[2,3,2,1]=(0.0767538617936)*sqrt(x[1])
        arg[2,4,0,0]=(-0.431757008766)*sqrt(x[0])
        arg[2,4,0,1]=(0.258988103513)*sqrt(x[1])
        arg[2,4,1,0]=(-0.584559859464)*sqrt(x[0])
        arg[2,4,1,1]=(-0.776827606804)*sqrt(x[1])
        arg[2,4,2,0]=(-0.867078960637)*sqrt(x[0])
        arg[2,4,2,1]=(-0.0972960135129)*sqrt(x[1])
        arg[3,0,0,0]=(0.999511727022)*sqrt(x[0])
        arg[3,0,0,1]=(-0.753218704622)*sqrt(x[1])
        arg[3,0,1,0]=(0.506439354885)*sqrt(x[0])
        arg[3,0,1,1]=(-0.0845853282297)*sqrt(x[1])
        arg[3,0,2,0]=(-0.871475028801)*sqrt(x[0])
        arg[3,0,2,1]=(-0.69261642657)*sqrt(x[1])
        arg[3,1,0,0]=(0.549778399933)*sqrt(x[0])
        arg[3,1,0,1]=(0.246187536149)*sqrt(x[1])
        arg[3,1,1,0]=(-0.620454200676)*sqrt(x[0])
        arg[3,1,1,1]=(0.372738564315)*sqrt(x[1])
        arg[3,1,2,0]=(0.0544097075138)*sqrt(x[0])
        arg[3,1,2,1]=(-0.0883198944676)*sqrt(x[1])
        arg[3,2,0,0]=(-0.671057180545)*sqrt(x[0])
        arg[3,2,0,1]=(-0.118991797907)*sqrt(x[1])
        arg[3,2,1,0]=(-0.27196730081)*sqrt(x[0])
        arg[3,2,1,1]=(-0.458807068968)*sqrt(x[1])
        arg[3,2,2,0]=(-0.31485399083)*sqrt(x[0])
        arg[3,2,2,1]=(0.487291960328)*sqrt(x[1])
        arg[3,3,0,0]=(-0.750302723531)*sqrt(x[0])
        arg[3,3,0,1]=(-0.486428789771)*sqrt(x[1])
        arg[3,3,1,0]=(-0.295909324594)*sqrt(x[0])
        arg[3,3,1,1]=(0.325702372596)*sqrt(x[1])
        arg[3,3,2,0]=(-0.512109540631)*sqrt(x[0])
        arg[3,3,2,1]=(0.563284721908)*sqrt(x[1])
        arg[3,4,0,0]=(0.53579406798)*sqrt(x[0])
        arg[3,4,0,1]=(-0.468431927975)*sqrt(x[1])
        arg[3,4,1,0]=(0.570851401329)*sqrt(x[0])
        arg[3,4,1,1]=(0.107791149865)*sqrt(x[1])
        arg[3,4,2,0]=(-0.211543670157)*sqrt(x[0])
        arg[3,4,2,1]=(0.848189550468)*sqrt(x[1])
        ref=sqrt((6.54316712884)*3./2.+(4.39564064326)/0.5)

      else:
        arg=Data(0,(4, 5, 3, 3),w)
        arg[0,0,0,0]=(-0.392256494872)*sqrt(x[0])
        arg[0,0,0,1]=(0.271488478841)*sqrt(x[1])
        arg[0,0,0,2]=(-0.878268531987)*sqrt(x[2])
        arg[0,0,1,0]=(0.106903787643)*sqrt(x[0])
        arg[0,0,1,1]=(-0.117986434516)*sqrt(x[1])
        arg[0,0,1,2]=(0.912673598522)*sqrt(x[2])
        arg[0,0,2,0]=(-0.652598945824)*sqrt(x[0])
        arg[0,0,2,1]=(-0.984195895647)*sqrt(x[1])
        arg[0,0,2,2]=(-0.137845459589)*sqrt(x[2])
        arg[0,1,0,0]=(0.696713200721)*sqrt(x[0])
        arg[0,1,0,1]=(0.480337929223)*sqrt(x[1])
        arg[0,1,0,2]=(-0.628726114325)*sqrt(x[2])
        arg[0,1,1,0]=(-0.356503461347)*sqrt(x[0])
        arg[0,1,1,1]=(-0.64670584045)*sqrt(x[1])
        arg[0,1,1,2]=(-0.737787618908)*sqrt(x[2])
        arg[0,1,2,0]=(-0.14207595302)*sqrt(x[0])
        arg[0,1,2,1]=(-0.573485525334)*sqrt(x[1])
        arg[0,1,2,2]=(-0.955542178481)*sqrt(x[2])
        arg[0,2,0,0]=(0.371952883975)*sqrt(x[0])
        arg[0,2,0,1]=(0.114016178762)*sqrt(x[1])
        arg[0,2,0,2]=(0.552721218169)*sqrt(x[2])
        arg[0,2,1,0]=(0.318426742113)*sqrt(x[0])
        arg[0,2,1,1]=(0.862220239384)*sqrt(x[1])
        arg[0,2,1,2]=(0.887376889978)*sqrt(x[2])
        arg[0,2,2,0]=(0.203656742981)*sqrt(x[0])
        arg[0,2,2,1]=(0.350541335144)*sqrt(x[1])
        arg[0,2,2,2]=(-0.448401957447)*sqrt(x[2])
        arg[0,3,0,0]=(-0.349200084042)*sqrt(x[0])
        arg[0,3,0,1]=(-0.546012602125)*sqrt(x[1])
        arg[0,3,0,2]=(-0.931488270979)*sqrt(x[2])
        arg[0,3,1,0]=(0.207457831058)*sqrt(x[0])
        arg[0,3,1,1]=(0.557715840549)*sqrt(x[1])
        arg[0,3,1,2]=(-0.978130146744)*sqrt(x[2])
        arg[0,3,2,0]=(-0.55088967957)*sqrt(x[0])
        arg[0,3,2,1]=(0.0490328838057)*sqrt(x[1])
        arg[0,3,2,2]=(0.150209929122)*sqrt(x[2])
        arg[0,4,0,0]=(-0.484145614698)*sqrt(x[0])
        arg[0,4,0,1]=(0.393078411279)*sqrt(x[1])
        arg[0,4,0,2]=(0.0678631863917)*sqrt(x[2])
        arg[0,4,1,0]=(-0.350980464628)*sqrt(x[0])
        arg[0,4,1,1]=(-0.784203839564)*sqrt(x[1])
        arg[0,4,1,2]=(0.636960296147)*sqrt(x[2])
        arg[0,4,2,0]=(0.592799503581)*sqrt(x[0])
        arg[0,4,2,1]=(-0.672104833683)*sqrt(x[1])
        arg[0,4,2,2]=(0.0366914082467)*sqrt(x[2])
        arg[1,0,0,0]=(-0.147168019774)*sqrt(x[0])
        arg[1,0,0,1]=(-0.0823637938956)*sqrt(x[1])
        arg[1,0,0,2]=(-0.852729690176)*sqrt(x[2])
        arg[1,0,1,0]=(-0.612338430408)*sqrt(x[0])
        arg[1,0,1,1]=(-0.85820035747)*sqrt(x[1])
        arg[1,0,1,2]=(-0.463664966162)*sqrt(x[2])
        arg[1,0,2,0]=(0.274600720491)*sqrt(x[0])
        arg[1,0,2,1]=(-0.488508234093)*sqrt(x[1])
        arg[1,0,2,2]=(-0.28251466519)*sqrt(x[2])
        arg[1,1,0,0]=(-0.0196532462794)*sqrt(x[0])
        arg[1,1,0,1]=(0.239352528871)*sqrt(x[1])
        arg[1,1,0,2]=(-0.17054773873)*sqrt(x[2])
        arg[1,1,1,0]=(-0.2396627789)*sqrt(x[0])
        arg[1,1,1,1]=(0.868970323003)*sqrt(x[1])
        arg[1,1,1,2]=(0.401487430312)*sqrt(x[2])
        arg[1,1,2,0]=(-0.624411449783)*sqrt(x[0])
        arg[1,1,2,1]=(0.0036634266684)*sqrt(x[1])
        arg[1,1,2,2]=(0.736129120967)*sqrt(x[2])
        arg[1,2,0,0]=(0.183756511707)*sqrt(x[0])
        arg[1,2,0,1]=(-0.288651848639)*sqrt(x[1])
        arg[1,2,0,2]=(-0.0672121537447)*sqrt(x[2])
        arg[1,2,1,0]=(-0.323274725936)*sqrt(x[0])
        arg[1,2,1,1]=(0.298001016025)*sqrt(x[1])
        arg[1,2,1,2]=(-0.976052460675)*sqrt(x[2])
        arg[1,2,2,0]=(0.596504441096)*sqrt(x[0])
        arg[1,2,2,1]=(0.873776068983)*sqrt(x[1])
        arg[1,2,2,2]=(-0.994068273196)*sqrt(x[2])
        arg[1,3,0,0]=(-0.495387299681)*sqrt(x[0])
        arg[1,3,0,1]=(-0.123674756551)*sqrt(x[1])
        arg[1,3,0,2]=(0.581213818577)*sqrt(x[2])
        arg[1,3,1,0]=(0.146405749701)*sqrt(x[0])
        arg[1,3,1,1]=(-0.594994686675)*sqrt(x[1])
        arg[1,3,1,2]=(-0.059093568436)*sqrt(x[2])
        arg[1,3,2,0]=(0.651004255104)*sqrt(x[0])
        arg[1,3,2,1]=(-0.977880706193)*sqrt(x[1])
        arg[1,3,2,2]=(0.370344651319)*sqrt(x[2])
        arg[1,4,0,0]=(-0.503657215247)*sqrt(x[0])
        arg[1,4,0,1]=(-0.170885297253)*sqrt(x[1])
        arg[1,4,0,2]=(0.533424480956)*sqrt(x[2])
        arg[1,4,1,0]=(-0.220533193308)*sqrt(x[0])
        arg[1,4,1,1]=(0.344537611882)*sqrt(x[1])
        arg[1,4,1,2]=(0.861473877282)*sqrt(x[2])
        arg[1,4,2,0]=(-0.0923010438884)*sqrt(x[0])
        arg[1,4,2,1]=(-0.338256780498)*sqrt(x[1])
        arg[1,4,2,2]=(0.528567959345)*sqrt(x[2])
        arg[2,0,0,0]=(-0.0423053381485)*sqrt(x[0])
        arg[2,0,0,1]=(0.856798579151)*sqrt(x[1])
        arg[2,0,0,2]=(0.383258153853)*sqrt(x[2])
        arg[2,0,1,0]=(0.350994872736)*sqrt(x[0])
        arg[2,0,1,1]=(-0.78055158106)*sqrt(x[1])
        arg[2,0,1,2]=(-0.770876699915)*sqrt(x[2])
        arg[2,0,2,0]=(-0.935133287106)*sqrt(x[0])
        arg[2,0,2,1]=(0.618238076989)*sqrt(x[1])
        arg[2,0,2,2]=(-0.846783087949)*sqrt(x[2])
        arg[2,1,0,0]=(-0.657394511405)*sqrt(x[0])
        arg[2,1,0,1]=(0.576218821654)*sqrt(x[1])
        arg[2,1,0,2]=(0.0269446356493)*sqrt(x[2])
        arg[2,1,1,0]=(-0.310710230949)*sqrt(x[0])
        arg[2,1,1,1]=(0.425412515598)*sqrt(x[1])
        arg[2,1,1,2]=(-0.225225290862)*sqrt(x[2])
        arg[2,1,2,0]=(-0.539928589495)*sqrt(x[0])
        arg[2,1,2,1]=(-0.348082121765)*sqrt(x[1])
        arg[2,1,2,2]=(-0.0287274646233)*sqrt(x[2])
        arg[2,2,0,0]=(0.746132865923)*sqrt(x[0])
        arg[2,2,0,1]=(-0.0234203693548)*sqrt(x[1])
        arg[2,2,0,2]=(0.517411821941)*sqrt(x[2])
        arg[2,2,1,0]=(-0.183204217349)*sqrt(x[0])
        arg[2,2,1,1]=(0.714988861836)*sqrt(x[1])
        arg[2,2,1,2]=(0.829083318937)*sqrt(x[2])
        arg[2,2,2,0]=(0.458067555841)*sqrt(x[0])
        arg[2,2,2,1]=(0.639317125869)*sqrt(x[1])
        arg[2,2,2,2]=(0.104611520408)*sqrt(x[2])
        arg[2,3,0,0]=(-0.420513461135)*sqrt(x[0])
        arg[2,3,0,1]=(0.888686162754)*sqrt(x[1])
        arg[2,3,0,2]=(0.939305777879)*sqrt(x[2])
        arg[2,3,1,0]=(0.856795132015)*sqrt(x[0])
        arg[2,3,1,1]=(0.817593141895)*sqrt(x[1])
        arg[2,3,1,2]=(0.962503342535)*sqrt(x[2])
        arg[2,3,2,0]=(-0.334586700245)*sqrt(x[0])
        arg[2,3,2,1]=(0.182696129528)*sqrt(x[1])
        arg[2,3,2,2]=(-0.707271571206)*sqrt(x[2])
        arg[2,4,0,0]=(-0.594795981069)*sqrt(x[0])
        arg[2,4,0,1]=(0.451239168073)*sqrt(x[1])
        arg[2,4,0,2]=(0.191212211556)*sqrt(x[2])
        arg[2,4,1,0]=(-0.503465984944)*sqrt(x[0])
        arg[2,4,1,1]=(0.725377884208)*sqrt(x[1])
        arg[2,4,1,2]=(-0.40719255752)*sqrt(x[2])
        arg[2,4,2,0]=(0.268071476451)*sqrt(x[0])
        arg[2,4,2,1]=(0.85066639942)*sqrt(x[1])
        arg[2,4,2,2]=(-0.906021406945)*sqrt(x[2])
        arg[3,0,0,0]=(-0.922179152122)*sqrt(x[0])
        arg[3,0,0,1]=(-0.0903841240007)*sqrt(x[1])
        arg[3,0,0,2]=(-0.751482803516)*sqrt(x[2])
        arg[3,0,1,0]=(0.960697809119)*sqrt(x[0])
        arg[3,0,1,1]=(0.638878873158)*sqrt(x[1])
        arg[3,0,1,2]=(0.390932234724)*sqrt(x[2])
        arg[3,0,2,0]=(-0.925078301694)*sqrt(x[0])
        arg[3,0,2,1]=(0.793590580665)*sqrt(x[1])
        arg[3,0,2,2]=(0.535478366911)*sqrt(x[2])
        arg[3,1,0,0]=(-0.431951993217)*sqrt(x[0])
        arg[3,1,0,1]=(0.211750261417)*sqrt(x[1])
        arg[3,1,0,2]=(-0.930706580442)*sqrt(x[2])
        arg[3,1,1,0]=(0.330979313323)*sqrt(x[0])
        arg[3,1,1,1]=(-0.838919076081)*sqrt(x[1])
        arg[3,1,1,2]=(0.134250050168)*sqrt(x[2])
        arg[3,1,2,0]=(0.414922811301)*sqrt(x[0])
        arg[3,1,2,1]=(-0.663692878121)*sqrt(x[1])
        arg[3,1,2,2]=(0.88499278543)*sqrt(x[2])
        arg[3,2,0,0]=(0.0884662742233)*sqrt(x[0])
        arg[3,2,0,1]=(-0.412630722821)*sqrt(x[1])
        arg[3,2,0,2]=(-0.730850884928)*sqrt(x[2])
        arg[3,2,1,0]=(0.722207547366)*sqrt(x[0])
        arg[3,2,1,1]=(-0.260067950749)*sqrt(x[1])
        arg[3,2,1,2]=(0.426259201494)*sqrt(x[2])
        arg[3,2,2,0]=(0.0516111795322)*sqrt(x[0])
        arg[3,2,2,1]=(0.922853710048)*sqrt(x[1])
        arg[3,2,2,2]=(-0.991912758116)*sqrt(x[2])
        arg[3,3,0,0]=(0.263933965905)*sqrt(x[0])
        arg[3,3,0,1]=(0.840541758799)*sqrt(x[1])
        arg[3,3,0,2]=(0.417658511125)*sqrt(x[2])
        arg[3,3,1,0]=(-0.901745614723)*sqrt(x[0])
        arg[3,3,1,1]=(-0.623608908699)*sqrt(x[1])
        arg[3,3,1,2]=(0.0522167208784)*sqrt(x[2])
        arg[3,3,2,0]=(-0.549431264931)*sqrt(x[0])
        arg[3,3,2,1]=(0.919971855457)*sqrt(x[1])
        arg[3,3,2,2]=(0.142757773397)*sqrt(x[2])
        arg[3,4,0,0]=(-0.258875259824)*sqrt(x[0])
        arg[3,4,0,1]=(-0.0373872187041)*sqrt(x[1])
        arg[3,4,0,2]=(0.445989164864)*sqrt(x[2])
        arg[3,4,1,0]=(0.0273711397038)*sqrt(x[0])
        arg[3,4,1,1]=(-0.85522629)*sqrt(x[1])
        arg[3,4,1,2]=(-0.835392581226)*sqrt(x[2])
        arg[3,4,2,0]=(0.402929450189)*sqrt(x[0])
        arg[3,4,2,1]=(0.549623033221)*sqrt(x[1])
        arg[3,4,2,2]=(0.237803212109)*sqrt(x[2])
        ref=sqrt((15.1448100099)*3./2.+(3.61598249819)/0.5)

      res=L2(arg)
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertAlmostEqual(res,ref,int(-log10(self.RES_TOL)),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_L2_onReducedFunctionOnContactZero_fromSymbol_rank0(self):
      """
      tests L2-norm of Symbol on the ReducedFunctionOnContactZero

      assumptions: self.domain supports integration on ReducedFunctionOnContactZero
      """
      dim=self.domain.getDim()
      w=ReducedFunctionOnContactZero(self.domain)
      x=w.getX()
      if dim==2:
        arg=Symbol(shape=(),dim=dim)
        s=Data(0,(),w)
        s=(-0.833494674242)*sqrt(x[0])
        ref=sqrt((0.0)*3./2.+(0.173678342998)/0.5)

      else:
        arg=Symbol(shape=(),dim=dim)
        s=Data(0,(),w)
        s=(-0.413475598947)*sqrt(x[0])
        ref=sqrt((0.0)*3./2.+(0.0427405177311)/0.5)

      res=L2(arg)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertAlmostEqual(sub,ref,int(-log10(self.RES_TOL)),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_L2_onReducedFunctionOnContactZero_fromSymbol_rank1(self):
      """
      tests L2-norm of Symbol on the ReducedFunctionOnContactZero

      assumptions: self.domain supports integration on ReducedFunctionOnContactZero
      """
      dim=self.domain.getDim()
      w=ReducedFunctionOnContactZero(self.domain)
      x=w.getX()
      if dim==2:
        arg=Symbol(shape=(2,),dim=dim)
        s=Data(0,(2,),w)
        s[0]=(-0.836300232979)*sqrt(x[0])
        s[1]=(-0.855949069442)*sqrt(x[1])
        ref=sqrt((0.244216269826)*3./2.+(0.17484951992)/0.5)

      else:
        arg=Symbol(shape=(3,),dim=dim)
        s=Data(0,(3,),w)
        s[0]=(0.604037678348)*sqrt(x[0])
        s[1]=(0.173183346503)*sqrt(x[1])
        s[2]=(0.0242607956405)*sqrt(x[2])
        ref=sqrt((0.0101936859037)*3./2.+(0.091215379216)/0.5)

      res=L2(arg)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertAlmostEqual(sub,ref,int(-log10(self.RES_TOL)),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_L2_onReducedFunctionOnContactZero_fromSymbol_rank2(self):
      """
      tests L2-norm of Symbol on the ReducedFunctionOnContactZero

      assumptions: self.domain supports integration on ReducedFunctionOnContactZero
      """
      dim=self.domain.getDim()
      w=ReducedFunctionOnContactZero(self.domain)
      x=w.getX()
      if dim==2:
        arg=Symbol(shape=(4, 2),dim=dim)
        s=Data(0,(4, 2),w)
        s[0,0]=(-0.482686145994)*sqrt(x[0])
        s[0,1]=(-0.948811640283)*sqrt(x[1])
        s[1,0]=(-0.992141136202)*sqrt(x[0])
        s[1,1]=(0.966819328886)*sqrt(x[1])
        s[2,0]=(0.948775134068)*sqrt(x[0])
        s[2,1]=(-0.468349890651)*sqrt(x[1])
        s[3,0]=(-0.671716588929)*sqrt(x[0])
        s[3,1]=(-0.340484526166)*sqrt(x[1])
        ref=sqrt((0.723421492026)*3./2.+(0.642176845137)/0.5)

      else:
        arg=Symbol(shape=(4, 3),dim=dim)
        s=Data(0,(4, 3),w)
        s[0,0]=(-0.891790655048)*sqrt(x[0])
        s[0,1]=(-0.245798024334)*sqrt(x[1])
        s[0,2]=(0.0537464591204)*sqrt(x[2])
        s[1,0]=(0.230259637248)*sqrt(x[0])
        s[1,1]=(0.0816863818806)*sqrt(x[1])
        s[1,2]=(0.000519957155649)*sqrt(x[2])
        s[2,0]=(0.0828558901056)*sqrt(x[0])
        s[2,1]=(0.831425281201)*sqrt(x[1])
        s[2,2]=(-0.601659198259)*sqrt(x[2])
        s[3,0]=(-0.499606719218)*sqrt(x[0])
        s[3,1]=(0.0962103643534)*sqrt(x[1])
        s[3,2]=(0.997600764289)*sqrt(x[2])
        ref=sqrt((0.709234598055)*3./2.+(0.276195511347)/0.5)

      res=L2(arg)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertAlmostEqual(sub,ref,int(-log10(self.RES_TOL)),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_L2_onReducedFunctionOnContactZero_fromSymbol_rank3(self):
      """
      tests L2-norm of Symbol on the ReducedFunctionOnContactZero

      assumptions: self.domain supports integration on ReducedFunctionOnContactZero
      """
      dim=self.domain.getDim()
      w=ReducedFunctionOnContactZero(self.domain)
      x=w.getX()
      if dim==2:
        arg=Symbol(shape=(6, 2, 2),dim=dim)
        s=Data(0,(6, 2, 2),w)
        s[0,0,0]=(0.991681821415)*sqrt(x[0])
        s[0,0,1]=(-0.224425650593)*sqrt(x[1])
        s[0,1,0]=(-0.876361852829)*sqrt(x[0])
        s[0,1,1]=(0.537837501136)*sqrt(x[1])
        s[1,0,0]=(-0.54481895152)*sqrt(x[0])
        s[1,0,1]=(0.593327791724)*sqrt(x[1])
        s[1,1,0]=(0.418004915001)*sqrt(x[0])
        s[1,1,1]=(-0.192957383127)*sqrt(x[1])
        s[2,0,0]=(0.201546374554)*sqrt(x[0])
        s[2,0,1]=(-0.265198464218)*sqrt(x[1])
        s[2,1,0]=(-0.965663986948)*sqrt(x[0])
        s[2,1,1]=(-0.323140398665)*sqrt(x[1])
        s[3,0,0]=(0.233122332146)*sqrt(x[0])
        s[3,0,1]=(-0.121823927196)*sqrt(x[1])
        s[3,1,0]=(0.292287834978)*sqrt(x[0])
        s[3,1,1]=(0.932977541215)*sqrt(x[1])
        s[4,0,0]=(-0.345560503758)*sqrt(x[0])
        s[4,0,1]=(0.192322796658)*sqrt(x[1])
        s[4,1,0]=(0.0397082772329)*sqrt(x[0])
        s[4,1,1]=(0.659346662851)*sqrt(x[1])
        s[5,0,0]=(-0.195000663748)*sqrt(x[0])
        s[5,0,1]=(-0.214222081491)*sqrt(x[1])
        s[5,1,0]=(0.830046961039)*sqrt(x[0])
        s[5,1,1]=(0.535960833948)*sqrt(x[1])
        ref=sqrt((0.864605256793)*3./2.+(1.04597420834)/0.5)

      else:
        arg=Symbol(shape=(6, 2, 3),dim=dim)
        s=Data(0,(6, 2, 3),w)
        s[0,0,0]=(-0.660450094858)*sqrt(x[0])
        s[0,0,1]=(-0.383241027824)*sqrt(x[1])
        s[0,0,2]=(0.615453081929)*sqrt(x[2])
        s[0,1,0]=(0.0679578237259)*sqrt(x[0])
        s[0,1,1]=(0.801162277845)*sqrt(x[1])
        s[0,1,2]=(0.741261458673)*sqrt(x[2])
        s[1,0,0]=(0.0391969780398)*sqrt(x[0])
        s[1,0,1]=(-0.400798945746)*sqrt(x[1])
        s[1,0,2]=(-0.00301944444045)*sqrt(x[2])
        s[1,1,0]=(-0.473988187262)*sqrt(x[0])
        s[1,1,1]=(0.124770000361)*sqrt(x[1])
        s[1,1,2]=(-0.65645884933)*sqrt(x[2])
        s[2,0,0]=(-0.849984307849)*sqrt(x[0])
        s[2,0,1]=(-0.992314728332)*sqrt(x[1])
        s[2,0,2]=(-0.221196101266)*sqrt(x[2])
        s[2,1,0]=(0.303587301796)*sqrt(x[0])
        s[2,1,1]=(-0.276498999588)*sqrt(x[1])
        s[2,1,2]=(0.341982161276)*sqrt(x[2])
        s[3,0,0]=(-0.479181710128)*sqrt(x[0])
        s[3,0,1]=(-0.799175074635)*sqrt(x[1])
        s[3,0,2]=(-0.741790237186)*sqrt(x[2])
        s[3,1,0]=(0.512279309628)*sqrt(x[0])
        s[3,1,1]=(-0.346869946977)*sqrt(x[1])
        s[3,1,2]=(-0.559402479335)*sqrt(x[2])
        s[4,0,0]=(0.449912947237)*sqrt(x[0])
        s[4,0,1]=(-0.0398661706497)*sqrt(x[1])
        s[4,0,2]=(-0.856490275998)*sqrt(x[2])
        s[4,1,0]=(-0.720500133511)*sqrt(x[0])
        s[4,1,1]=(-0.97292914508)*sqrt(x[1])
        s[4,1,2]=(-0.151959359468)*sqrt(x[2])
        s[5,0,0]=(0.397527396462)*sqrt(x[0])
        s[5,0,1]=(-0.151341467662)*sqrt(x[1])
        s[5,0,2]=(-0.202710617619)*sqrt(x[2])
        s[5,1,0]=(0.386974642401)*sqrt(x[0])
        s[5,1,1]=(0.103900256892)*sqrt(x[1])
        s[5,1,2]=(-0.152493955764)*sqrt(x[2])
        ref=sqrt((2.3254122568)*3./2.+(0.750754270358)/0.5)

      res=L2(arg)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertAlmostEqual(sub,ref,int(-log10(self.RES_TOL)),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_L2_onReducedFunctionOnContactZero_fromSymbol_rank4(self):
      """
      tests L2-norm of Symbol on the ReducedFunctionOnContactZero

      assumptions: self.domain supports integration on ReducedFunctionOnContactZero
      """
      dim=self.domain.getDim()
      w=ReducedFunctionOnContactZero(self.domain)
      x=w.getX()
      if dim==2:
        arg=Symbol(shape=(4, 5, 3, 2),dim=dim)
        s=Data(0,(4, 5, 3, 2),w)
        s[0,0,0,0]=(0.914300112399)*sqrt(x[0])
        s[0,0,0,1]=(0.800216576034)*sqrt(x[1])
        s[0,0,1,0]=(-0.396870583938)*sqrt(x[0])
        s[0,0,1,1]=(0.867325478217)*sqrt(x[1])
        s[0,0,2,0]=(-0.200425849404)*sqrt(x[0])
        s[0,0,2,1]=(0.562512527677)*sqrt(x[1])
        s[0,1,0,0]=(-0.26457813536)*sqrt(x[0])
        s[0,1,0,1]=(-0.865366345944)*sqrt(x[1])
        s[0,1,1,0]=(0.906255889925)*sqrt(x[0])
        s[0,1,1,1]=(-0.144426923939)*sqrt(x[1])
        s[0,1,2,0]=(-0.835643108487)*sqrt(x[0])
        s[0,1,2,1]=(-0.644514669599)*sqrt(x[1])
        s[0,2,0,0]=(0.516524292926)*sqrt(x[0])
        s[0,2,0,1]=(0.825766812215)*sqrt(x[1])
        s[0,2,1,0]=(-0.37867452414)*sqrt(x[0])
        s[0,2,1,1]=(0.920377065217)*sqrt(x[1])
        s[0,2,2,0]=(0.0937552904916)*sqrt(x[0])
        s[0,2,2,1]=(0.378732372322)*sqrt(x[1])
        s[0,3,0,0]=(0.496752465789)*sqrt(x[0])
        s[0,3,0,1]=(0.427047604935)*sqrt(x[1])
        s[0,3,1,0]=(0.750830919575)*sqrt(x[0])
        s[0,3,1,1]=(0.576559278913)*sqrt(x[1])
        s[0,3,2,0]=(-0.115508523662)*sqrt(x[0])
        s[0,3,2,1]=(0.737623150949)*sqrt(x[1])
        s[0,4,0,0]=(0.440274015173)*sqrt(x[0])
        s[0,4,0,1]=(-0.450295347264)*sqrt(x[1])
        s[0,4,1,0]=(0.170660831663)*sqrt(x[0])
        s[0,4,1,1]=(-0.00668299893696)*sqrt(x[1])
        s[0,4,2,0]=(0.443915340807)*sqrt(x[0])
        s[0,4,2,1]=(-0.096469220433)*sqrt(x[1])
        s[1,0,0,0]=(0.124319331592)*sqrt(x[0])
        s[1,0,0,1]=(0.0209567989245)*sqrt(x[1])
        s[1,0,1,0]=(-0.120682670913)*sqrt(x[0])
        s[1,0,1,1]=(-0.0668286988997)*sqrt(x[1])
        s[1,0,2,0]=(-0.622418696038)*sqrt(x[0])
        s[1,0,2,1]=(0.690522460229)*sqrt(x[1])
        s[1,1,0,0]=(-0.564034328652)*sqrt(x[0])
        s[1,1,0,1]=(-0.488373582152)*sqrt(x[1])
        s[1,1,1,0]=(0.37807858594)*sqrt(x[0])
        s[1,1,1,1]=(0.607009462361)*sqrt(x[1])
        s[1,1,2,0]=(0.357544289055)*sqrt(x[0])
        s[1,1,2,1]=(0.89416048362)*sqrt(x[1])
        s[1,2,0,0]=(0.180403414369)*sqrt(x[0])
        s[1,2,0,1]=(-0.590910014416)*sqrt(x[1])
        s[1,2,1,0]=(-0.0184755955417)*sqrt(x[0])
        s[1,2,1,1]=(0.84738705179)*sqrt(x[1])
        s[1,2,2,0]=(-0.677245742487)*sqrt(x[0])
        s[1,2,2,1]=(-0.457219426905)*sqrt(x[1])
        s[1,3,0,0]=(-0.454834820082)*sqrt(x[0])
        s[1,3,0,1]=(-0.725915297395)*sqrt(x[1])
        s[1,3,1,0]=(0.184608835513)*sqrt(x[0])
        s[1,3,1,1]=(0.313259821447)*sqrt(x[1])
        s[1,3,2,0]=(-0.252818794924)*sqrt(x[0])
        s[1,3,2,1]=(-0.654170873581)*sqrt(x[1])
        s[1,4,0,0]=(-0.315092745211)*sqrt(x[0])
        s[1,4,0,1]=(-0.922091825394)*sqrt(x[1])
        s[1,4,1,0]=(-0.386863810397)*sqrt(x[0])
        s[1,4,1,1]=(0.430983629642)*sqrt(x[1])
        s[1,4,2,0]=(-0.762809887523)*sqrt(x[0])
        s[1,4,2,1]=(0.58896325465)*sqrt(x[1])
        s[2,0,0,0]=(-0.456340727871)*sqrt(x[0])
        s[2,0,0,1]=(-0.0187981167144)*sqrt(x[1])
        s[2,0,1,0]=(-0.525249731866)*sqrt(x[0])
        s[2,0,1,1]=(0.399070164526)*sqrt(x[1])
        s[2,0,2,0]=(-0.201772101073)*sqrt(x[0])
        s[2,0,2,1]=(-0.0968197655895)*sqrt(x[1])
        s[2,1,0,0]=(0.191048791069)*sqrt(x[0])
        s[2,1,0,1]=(0.434067010545)*sqrt(x[1])
        s[2,1,1,0]=(0.848522099444)*sqrt(x[0])
        s[2,1,1,1]=(0.696991804739)*sqrt(x[1])
        s[2,1,2,0]=(0.220121221699)*sqrt(x[0])
        s[2,1,2,1]=(0.301203065888)*sqrt(x[1])
        s[2,2,0,0]=(0.766472384148)*sqrt(x[0])
        s[2,2,0,1]=(-0.0714402172399)*sqrt(x[1])
        s[2,2,1,0]=(0.492328493075)*sqrt(x[0])
        s[2,2,1,1]=(0.558080277043)*sqrt(x[1])
        s[2,2,2,0]=(0.469934590015)*sqrt(x[0])
        s[2,2,2,1]=(0.758048072142)*sqrt(x[1])
        s[2,3,0,0]=(0.085308155098)*sqrt(x[0])
        s[2,3,0,1]=(0.297189197296)*sqrt(x[1])
        s[2,3,1,0]=(0.978282022769)*sqrt(x[0])
        s[2,3,1,1]=(0.682153031605)*sqrt(x[1])
        s[2,3,2,0]=(-0.561125939181)*sqrt(x[0])
        s[2,3,2,1]=(0.440224855986)*sqrt(x[1])
        s[2,4,0,0]=(0.72050530483)*sqrt(x[0])
        s[2,4,0,1]=(-0.206652409514)*sqrt(x[1])
        s[2,4,1,0]=(0.387667449577)*sqrt(x[0])
        s[2,4,1,1]=(0.493459617619)*sqrt(x[1])
        s[2,4,2,0]=(0.688054409914)*sqrt(x[0])
        s[2,4,2,1]=(0.971336356338)*sqrt(x[1])
        s[3,0,0,0]=(0.861462391015)*sqrt(x[0])
        s[3,0,0,1]=(0.0554843601356)*sqrt(x[1])
        s[3,0,1,0]=(0.83937959439)*sqrt(x[0])
        s[3,0,1,1]=(0.954697475987)*sqrt(x[1])
        s[3,0,2,0]=(-0.997867671237)*sqrt(x[0])
        s[3,0,2,1]=(-0.218317664783)*sqrt(x[1])
        s[3,1,0,0]=(-0.262679977079)*sqrt(x[0])
        s[3,1,0,1]=(0.0145063905029)*sqrt(x[1])
        s[3,1,1,0]=(0.130108953527)*sqrt(x[0])
        s[3,1,1,1]=(0.9110336683)*sqrt(x[1])
        s[3,1,2,0]=(0.278623938991)*sqrt(x[0])
        s[3,1,2,1]=(-0.374239642631)*sqrt(x[1])
        s[3,2,0,0]=(0.720411588365)*sqrt(x[0])
        s[3,2,0,1]=(0.794261999505)*sqrt(x[1])
        s[3,2,1,0]=(0.098798147685)*sqrt(x[0])
        s[3,2,1,1]=(-0.338610161254)*sqrt(x[1])
        s[3,2,2,0]=(0.961927950853)*sqrt(x[0])
        s[3,2,2,1]=(-0.307621697905)*sqrt(x[1])
        s[3,3,0,0]=(-0.73836606888)*sqrt(x[0])
        s[3,3,0,1]=(0.714826978574)*sqrt(x[1])
        s[3,3,1,0]=(-0.394351029155)*sqrt(x[0])
        s[3,3,1,1]=(0.408664173503)*sqrt(x[1])
        s[3,3,2,0]=(0.844614972483)*sqrt(x[0])
        s[3,3,2,1]=(-0.478702935986)*sqrt(x[1])
        s[3,4,0,0]=(0.0447493337692)*sqrt(x[0])
        s[3,4,0,1]=(0.317235470904)*sqrt(x[1])
        s[3,4,1,0]=(-0.960879888735)*sqrt(x[0])
        s[3,4,1,1]=(-0.349654948749)*sqrt(x[1])
        s[3,4,2,0]=(0.15351062359)*sqrt(x[0])
        s[3,4,2,1]=(-0.30302268499)*sqrt(x[1])
        ref=sqrt((6.41155668825)*3./2.+(4.53628477602)/0.5)

      else:
        arg=Symbol(shape=(4, 5, 3, 3),dim=dim)
        s=Data(0,(4, 5, 3, 3),w)
        s[0,0,0,0]=(-0.89168364071)*sqrt(x[0])
        s[0,0,0,1]=(0.656887229548)*sqrt(x[1])
        s[0,0,0,2]=(0.0379592114434)*sqrt(x[2])
        s[0,0,1,0]=(-0.499743077643)*sqrt(x[0])
        s[0,0,1,1]=(0.433796934727)*sqrt(x[1])
        s[0,0,1,2]=(-0.419639340675)*sqrt(x[2])
        s[0,0,2,0]=(-0.229849090155)*sqrt(x[0])
        s[0,0,2,1]=(0.48808994712)*sqrt(x[1])
        s[0,0,2,2]=(0.424012703453)*sqrt(x[2])
        s[0,1,0,0]=(0.498682171432)*sqrt(x[0])
        s[0,1,0,1]=(0.623180282688)*sqrt(x[1])
        s[0,1,0,2]=(0.467278343783)*sqrt(x[2])
        s[0,1,1,0]=(-0.461240789999)*sqrt(x[0])
        s[0,1,1,1]=(0.79579036671)*sqrt(x[1])
        s[0,1,1,2]=(0.0336609748509)*sqrt(x[2])
        s[0,1,2,0]=(-0.758182366212)*sqrt(x[0])
        s[0,1,2,1]=(0.335354770639)*sqrt(x[1])
        s[0,1,2,2]=(-0.43256653064)*sqrt(x[2])
        s[0,2,0,0]=(0.267085562446)*sqrt(x[0])
        s[0,2,0,1]=(0.74096852219)*sqrt(x[1])
        s[0,2,0,2]=(0.884306535005)*sqrt(x[2])
        s[0,2,1,0]=(0.839435822303)*sqrt(x[0])
        s[0,2,1,1]=(0.981598566715)*sqrt(x[1])
        s[0,2,1,2]=(0.178444224041)*sqrt(x[2])
        s[0,2,2,0]=(0.28828503848)*sqrt(x[0])
        s[0,2,2,1]=(-0.626210031155)*sqrt(x[1])
        s[0,2,2,2]=(-0.69299554971)*sqrt(x[2])
        s[0,3,0,0]=(-0.0218942570636)*sqrt(x[0])
        s[0,3,0,1]=(0.542278046952)*sqrt(x[1])
        s[0,3,0,2]=(0.953044269075)*sqrt(x[2])
        s[0,3,1,0]=(-0.557505895397)*sqrt(x[0])
        s[0,3,1,1]=(0.523921264225)*sqrt(x[1])
        s[0,3,1,2]=(-0.786147057715)*sqrt(x[2])
        s[0,3,2,0]=(-0.89329999988)*sqrt(x[0])
        s[0,3,2,1]=(0.462215391096)*sqrt(x[1])
        s[0,3,2,2]=(-0.374927543026)*sqrt(x[2])
        s[0,4,0,0]=(-0.247805259481)*sqrt(x[0])
        s[0,4,0,1]=(0.645841666564)*sqrt(x[1])
        s[0,4,0,2]=(0.16650874055)*sqrt(x[2])
        s[0,4,1,0]=(0.266137626404)*sqrt(x[0])
        s[0,4,1,1]=(-0.288059047514)*sqrt(x[1])
        s[0,4,1,2]=(-0.486812056856)*sqrt(x[2])
        s[0,4,2,0]=(0.688344153316)*sqrt(x[0])
        s[0,4,2,1]=(-0.655455791821)*sqrt(x[1])
        s[0,4,2,2]=(-0.348241680636)*sqrt(x[2])
        s[1,0,0,0]=(0.762345221451)*sqrt(x[0])
        s[1,0,0,1]=(0.264502883884)*sqrt(x[1])
        s[1,0,0,2]=(0.48788907144)*sqrt(x[2])
        s[1,0,1,0]=(0.42005226242)*sqrt(x[0])
        s[1,0,1,1]=(-0.825723492918)*sqrt(x[1])
        s[1,0,1,2]=(0.797321195042)*sqrt(x[2])
        s[1,0,2,0]=(-0.19639010863)*sqrt(x[0])
        s[1,0,2,1]=(-0.702950524388)*sqrt(x[1])
        s[1,0,2,2]=(0.235603192829)*sqrt(x[2])
        s[1,1,0,0]=(-0.454504750183)*sqrt(x[0])
        s[1,1,0,1]=(0.31590384254)*sqrt(x[1])
        s[1,1,0,2]=(-0.19227935466)*sqrt(x[2])
        s[1,1,1,0]=(0.0070439759207)*sqrt(x[0])
        s[1,1,1,1]=(-0.293083678876)*sqrt(x[1])
        s[1,1,1,2]=(-0.878745990375)*sqrt(x[2])
        s[1,1,2,0]=(-0.192794025412)*sqrt(x[0])
        s[1,1,2,1]=(0.648895386958)*sqrt(x[1])
        s[1,1,2,2]=(0.775208361981)*sqrt(x[2])
        s[1,2,0,0]=(-0.368309400642)*sqrt(x[0])
        s[1,2,0,1]=(-0.917724017536)*sqrt(x[1])
        s[1,2,0,2]=(0.920777364818)*sqrt(x[2])
        s[1,2,1,0]=(-0.645052443283)*sqrt(x[0])
        s[1,2,1,1]=(0.139033194163)*sqrt(x[1])
        s[1,2,1,2]=(-0.0404331243874)*sqrt(x[2])
        s[1,2,2,0]=(-0.61245787618)*sqrt(x[0])
        s[1,2,2,1]=(0.0482137893562)*sqrt(x[1])
        s[1,2,2,2]=(0.179049768973)*sqrt(x[2])
        s[1,3,0,0]=(-0.896233852085)*sqrt(x[0])
        s[1,3,0,1]=(-0.681461465854)*sqrt(x[1])
        s[1,3,0,2]=(-0.446646723524)*sqrt(x[2])
        s[1,3,1,0]=(-0.555003216598)*sqrt(x[0])
        s[1,3,1,1]=(0.35239325809)*sqrt(x[1])
        s[1,3,1,2]=(0.39507338896)*sqrt(x[2])
        s[1,3,2,0]=(-0.13811362276)*sqrt(x[0])
        s[1,3,2,1]=(-0.374436915851)*sqrt(x[1])
        s[1,3,2,2]=(0.0513289236564)*sqrt(x[2])
        s[1,4,0,0]=(-0.687683310284)*sqrt(x[0])
        s[1,4,0,1]=(0.572359319345)*sqrt(x[1])
        s[1,4,0,2]=(-0.814515142116)*sqrt(x[2])
        s[1,4,1,0]=(-0.565292086642)*sqrt(x[0])
        s[1,4,1,1]=(-0.191646702103)*sqrt(x[1])
        s[1,4,1,2]=(-0.292049727092)*sqrt(x[2])
        s[1,4,2,0]=(0.452055314722)*sqrt(x[0])
        s[1,4,2,1]=(-0.457431088076)*sqrt(x[1])
        s[1,4,2,2]=(-0.178361949498)*sqrt(x[2])
        s[2,0,0,0]=(0.354814016099)*sqrt(x[0])
        s[2,0,0,1]=(-0.384609805819)*sqrt(x[1])
        s[2,0,0,2]=(-0.281337028672)*sqrt(x[2])
        s[2,0,1,0]=(0.142919596773)*sqrt(x[0])
        s[2,0,1,1]=(-0.57612422002)*sqrt(x[1])
        s[2,0,1,2]=(-0.529853258262)*sqrt(x[2])
        s[2,0,2,0]=(0.80636898712)*sqrt(x[0])
        s[2,0,2,1]=(0.106993663032)*sqrt(x[1])
        s[2,0,2,2]=(0.255600789365)*sqrt(x[2])
        s[2,1,0,0]=(0.978145357315)*sqrt(x[0])
        s[2,1,0,1]=(-0.901086815575)*sqrt(x[1])
        s[2,1,0,2]=(0.648553017338)*sqrt(x[2])
        s[2,1,1,0]=(-0.283065724087)*sqrt(x[0])
        s[2,1,1,1]=(-0.0976255838643)*sqrt(x[1])
        s[2,1,1,2]=(-0.518416419376)*sqrt(x[2])
        s[2,1,2,0]=(0.572224523241)*sqrt(x[0])
        s[2,1,2,1]=(0.676830468876)*sqrt(x[1])
        s[2,1,2,2]=(0.347547333574)*sqrt(x[2])
        s[2,2,0,0]=(-0.890366473527)*sqrt(x[0])
        s[2,2,0,1]=(-0.202566992837)*sqrt(x[1])
        s[2,2,0,2]=(-0.00994077956762)*sqrt(x[2])
        s[2,2,1,0]=(-0.312288044658)*sqrt(x[0])
        s[2,2,1,1]=(0.215695301526)*sqrt(x[1])
        s[2,2,1,2]=(0.134196622741)*sqrt(x[2])
        s[2,2,2,0]=(-0.564325028454)*sqrt(x[0])
        s[2,2,2,1]=(-0.707031226029)*sqrt(x[1])
        s[2,2,2,2]=(-0.793190569275)*sqrt(x[2])
        s[2,3,0,0]=(0.28006826199)*sqrt(x[0])
        s[2,3,0,1]=(0.743445147616)*sqrt(x[1])
        s[2,3,0,2]=(0.57457392009)*sqrt(x[2])
        s[2,3,1,0]=(-0.353169838541)*sqrt(x[0])
        s[2,3,1,1]=(-0.353550104906)*sqrt(x[1])
        s[2,3,1,2]=(0.26061713616)*sqrt(x[2])
        s[2,3,2,0]=(0.360387490548)*sqrt(x[0])
        s[2,3,2,1]=(0.532836358847)*sqrt(x[1])
        s[2,3,2,2]=(0.517337142155)*sqrt(x[2])
        s[2,4,0,0]=(0.969178027599)*sqrt(x[0])
        s[2,4,0,1]=(-0.0671877983392)*sqrt(x[1])
        s[2,4,0,2]=(0.692408837657)*sqrt(x[2])
        s[2,4,1,0]=(0.988861077126)*sqrt(x[0])
        s[2,4,1,1]=(0.987792613173)*sqrt(x[1])
        s[2,4,1,2]=(-0.147075972258)*sqrt(x[2])
        s[2,4,2,0]=(-0.440780900369)*sqrt(x[0])
        s[2,4,2,1]=(0.390407807605)*sqrt(x[1])
        s[2,4,2,2]=(-0.893694860437)*sqrt(x[2])
        s[3,0,0,0]=(0.699550432817)*sqrt(x[0])
        s[3,0,0,1]=(-0.787672358838)*sqrt(x[1])
        s[3,0,0,2]=(-0.583257623239)*sqrt(x[2])
        s[3,0,1,0]=(-0.157588638816)*sqrt(x[0])
        s[3,0,1,1]=(-0.986238064621)*sqrt(x[1])
        s[3,0,1,2]=(0.699443634206)*sqrt(x[2])
        s[3,0,2,0]=(-0.222407346878)*sqrt(x[0])
        s[3,0,2,1]=(-0.308189430231)*sqrt(x[1])
        s[3,0,2,2]=(0.299965297762)*sqrt(x[2])
        s[3,1,0,0]=(-0.412975772409)*sqrt(x[0])
        s[3,1,0,1]=(0.717170353427)*sqrt(x[1])
        s[3,1,0,2]=(0.555847828902)*sqrt(x[2])
        s[3,1,1,0]=(0.301148971297)*sqrt(x[0])
        s[3,1,1,1]=(0.556400276399)*sqrt(x[1])
        s[3,1,1,2]=(0.473027697568)*sqrt(x[2])
        s[3,1,2,0]=(-0.701958828234)*sqrt(x[0])
        s[3,1,2,1]=(-0.965435432938)*sqrt(x[1])
        s[3,1,2,2]=(0.226965032076)*sqrt(x[2])
        s[3,2,0,0]=(0.548493161421)*sqrt(x[0])
        s[3,2,0,1]=(-0.422082327803)*sqrt(x[1])
        s[3,2,0,2]=(-0.64432755028)*sqrt(x[2])
        s[3,2,1,0]=(-0.821784569582)*sqrt(x[0])
        s[3,2,1,1]=(0.567506242312)*sqrt(x[1])
        s[3,2,1,2]=(0.58945700083)*sqrt(x[2])
        s[3,2,2,0]=(-0.31431442897)*sqrt(x[0])
        s[3,2,2,1]=(-0.727570436185)*sqrt(x[1])
        s[3,2,2,2]=(-0.234428528039)*sqrt(x[2])
        s[3,3,0,0]=(-0.184478413951)*sqrt(x[0])
        s[3,3,0,1]=(0.54667291001)*sqrt(x[1])
        s[3,3,0,2]=(0.266086928294)*sqrt(x[2])
        s[3,3,1,0]=(0.0279847154277)*sqrt(x[0])
        s[3,3,1,1]=(-0.0851677217034)*sqrt(x[1])
        s[3,3,1,2]=(-0.679725199609)*sqrt(x[2])
        s[3,3,2,0]=(0.570361871278)*sqrt(x[0])
        s[3,3,2,1]=(0.913879855699)*sqrt(x[1])
        s[3,3,2,2]=(-0.036149215115)*sqrt(x[2])
        s[3,4,0,0]=(0.430041266053)*sqrt(x[0])
        s[3,4,0,1]=(-0.468708383807)*sqrt(x[1])
        s[3,4,0,2]=(0.430318326847)*sqrt(x[2])
        s[3,4,1,0]=(0.808876543295)*sqrt(x[0])
        s[3,4,1,1]=(0.175574603952)*sqrt(x[1])
        s[3,4,1,2]=(-0.982042631943)*sqrt(x[2])
        s[3,4,2,0]=(0.730923035545)*sqrt(x[0])
        s[3,4,2,1]=(0.65499693359)*sqrt(x[1])
        s[3,4,2,2]=(0.678161392893)*sqrt(x[2])
        ref=sqrt((12.3860143155)*3./2.+(4.68566966204)/0.5)

      res=L2(arg)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertAlmostEqual(sub,ref,int(-log10(self.RES_TOL)),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_L2_onFunctionOnContactOne_fromData_rank0(self):
      """
      tests L2-norm of Data on the FunctionOnContactOne

      assumptions: self.domain supports integration on FunctionOnContactOne
      """
      dim=self.domain.getDim()
      w=FunctionOnContactOne(self.domain)
      x=w.getX()
      if dim==2:
        arg=Data(0,(),w)
        arg=(0.39039157828)*x[0]
        ref=sqrt((0.0)+(0.038101396098))

      else:
        arg=Data(0,(),w)
        arg=(0.893204658234)*x[0]
        ref=sqrt((0.0)+(0.199453640373))

      res=L2(arg)
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertAlmostEqual(res,ref,int(-log10(self.RES_TOL)),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_L2_onFunctionOnContactOne_fromData_rank1(self):
      """
      tests L2-norm of Data on the FunctionOnContactOne

      assumptions: self.domain supports integration on FunctionOnContactOne
      """
      dim=self.domain.getDim()
      w=FunctionOnContactOne(self.domain)
      x=w.getX()
      if dim==2:
        arg=Data(0,(2,),w)
        arg[0]=(0.697576620054)*x[0]
        arg[1]=(-0.302037273777)*x[1]
        ref=sqrt((0.0304088382502)+(0.121653285212))

      else:
        arg=Data(0,(3,),w)
        arg[0]=(0.649308272675)*x[0]
        arg[1]=(-0.408829699836)*x[1]
        arg[2]=(-0.984892977949)*x[2]
        ref=sqrt((0.37905196716)+(0.105400308241))

      res=L2(arg)
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertAlmostEqual(res,ref,int(-log10(self.RES_TOL)),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_L2_onFunctionOnContactOne_fromData_rank2(self):
      """
      tests L2-norm of Data on the FunctionOnContactOne

      assumptions: self.domain supports integration on FunctionOnContactOne
      """
      dim=self.domain.getDim()
      w=FunctionOnContactOne(self.domain)
      x=w.getX()
      if dim==2:
        arg=Data(0,(4, 2),w)
        arg[0,0]=(0.504133645896)*x[0]
        arg[0,1]=(0.728228500775)*x[1]
        arg[1,0]=(0.968237164633)*x[0]
        arg[1,1]=(-0.205351628957)*x[1]
        arg[2,0]=(-0.154008228702)*x[0]
        arg[2,1]=(-0.972088404566)*x[1]
        arg[3,0]=(0.585153521593)*x[0]
        arg[3,1]=(0.371861820001)*x[1]
        ref=sqrt((0.551907706774)+(0.389439279561))

      else:
        arg=Data(0,(4, 3),w)
        arg[0,0]=(-0.4847970605)*x[0]
        arg[0,1]=(-0.664433021479)*x[1]
        arg[0,2]=(-0.93730914676)*x[2]
        arg[1,0]=(0.712624065023)*x[0]
        arg[1,1]=(0.105431976424)*x[1]
        arg[1,2]=(-0.452636806104)*x[2]
        arg[2,0]=(0.488406064176)*x[0]
        arg[2,1]=(0.0594879976584)*x[1]
        arg[2,2]=(0.596963883202)*x[2]
        arg[3,0]=(0.619366395328)*x[0]
        arg[3,1]=(0.719965901041)*x[1]
        arg[3,2]=(-0.568971277169)*x[2]
        ref=sqrt((0.912666523048)+(0.341254115776))

      res=L2(arg)
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertAlmostEqual(res,ref,int(-log10(self.RES_TOL)),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_L2_onFunctionOnContactOne_fromData_rank3(self):
      """
      tests L2-norm of Data on the FunctionOnContactOne

      assumptions: self.domain supports integration on FunctionOnContactOne
      """
      dim=self.domain.getDim()
      w=FunctionOnContactOne(self.domain)
      x=w.getX()
      if dim==2:
        arg=Data(0,(6, 2, 2),w)
        arg[0,0,0]=(-0.481233278183)*x[0]
        arg[0,0,1]=(-0.883447576048)*x[1]
        arg[0,1,0]=(0.525938652633)*x[0]
        arg[0,1,1]=(-0.694971850651)*x[1]
        arg[1,0,0]=(-0.935880633291)*x[0]
        arg[1,0,1]=(0.970445286468)*x[1]
        arg[1,1,0]=(-0.302571977945)*x[0]
        arg[1,1,1]=(-0.237750319821)*x[1]
        arg[2,0,0]=(0.922940652883)*x[0]
        arg[2,0,1]=(-0.624756614078)*x[1]
        arg[2,1,0]=(-0.146839388764)*x[0]
        arg[2,1,1]=(-0.995551211092)*x[1]
        arg[3,0,0]=(0.719184541649)*x[0]
        arg[3,0,1]=(-0.629461063375)*x[1]
        arg[3,1,0]=(-0.925581215276)*x[0]
        arg[3,1,1]=(-0.564418658495)*x[1]
        arg[4,0,0]=(-0.724035884026)*x[0]
        arg[4,0,1]=(0.78393984221)*x[1]
        arg[4,1,0]=(-0.722375249317)*x[0]
        arg[4,1,1]=(0.27571990944)*x[1]
        arg[5,0,0]=(-0.197579392553)*x[0]
        arg[5,0,1]=(0.341717398275)*x[1]
        arg[5,1,0]=(0.926327822859)*x[0]
        arg[5,1,1]=(0.44114488285)*x[1]
        ref=sqrt((1.78665006238)+(1.41652558894))

      else:
        arg=Data(0,(6, 2, 3),w)
        arg[0,0,0]=(-0.348448140437)*x[0]
        arg[0,0,1]=(-0.530793589782)*x[1]
        arg[0,0,2]=(0.386511493376)*x[2]
        arg[0,1,0]=(0.00776536299593)*x[0]
        arg[0,1,1]=(-0.0696129074064)*x[1]
        arg[0,1,2]=(0.114179504216)*x[2]
        arg[1,0,0]=(-0.681009917406)*x[0]
        arg[1,0,1]=(0.763595403719)*x[1]
        arg[1,0,2]=(-0.54459342544)*x[2]
        arg[1,1,0]=(0.0839255723157)*x[0]
        arg[1,1,1]=(0.269389515003)*x[1]
        arg[1,1,2]=(0.612446509453)*x[2]
        arg[2,0,0]=(0.600070579238)*x[0]
        arg[2,0,1]=(-0.49585867531)*x[1]
        arg[2,0,2]=(-0.766857395141)*x[2]
        arg[2,1,0]=(-0.84515816441)*x[0]
        arg[2,1,1]=(-0.594337789049)*x[1]
        arg[2,1,2]=(0.774596009811)*x[2]
        arg[3,0,0]=(-0.833982234688)*x[0]
        arg[3,0,1]=(0.747667281218)*x[1]
        arg[3,0,2]=(-0.77025764051)*x[2]
        arg[3,1,0]=(0.0367023286497)*x[0]
        arg[3,1,1]=(-0.503080137993)*x[1]
        arg[3,1,2]=(-0.129478323797)*x[2]
        arg[4,0,0]=(0.304123530695)*x[0]
        arg[4,0,1]=(-0.873023368289)*x[1]
        arg[4,0,2]=(-0.712740806672)*x[2]
        arg[4,1,0]=(-0.0629132979843)*x[0]
        arg[4,1,1]=(0.096843438525)*x[1]
        arg[4,1,2]=(-0.608716559621)*x[2]
        arg[5,0,0]=(-0.642902822517)*x[0]
        arg[5,0,1]=(0.478134785533)*x[1]
        arg[5,0,2]=(0.757891512092)*x[2]
        arg[5,1,0]=(-0.144492755065)*x[0]
        arg[5,1,1]=(0.286775195938)*x[1]
        arg[5,1,2]=(-0.063580488638)*x[2]
        ref=sqrt((2.50835195543)+(0.723549067206))

      res=L2(arg)
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertAlmostEqual(res,ref,int(-log10(self.RES_TOL)),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_L2_onFunctionOnContactOne_fromData_rank4(self):
      """
      tests L2-norm of Data on the FunctionOnContactOne

      assumptions: self.domain supports integration on FunctionOnContactOne
      """
      dim=self.domain.getDim()
      w=FunctionOnContactOne(self.domain)
      x=w.getX()
      if dim==2:
        arg=Data(0,(4, 5, 3, 2),w)
        arg[0,0,0,0]=(0.523372259748)*x[0]
        arg[0,0,0,1]=(-0.732795389957)*x[1]
        arg[0,0,1,0]=(0.0571737128606)*x[0]
        arg[0,0,1,1]=(-0.627260438721)*x[1]
        arg[0,0,2,0]=(-0.945159375045)*x[0]
        arg[0,0,2,1]=(-0.145103328237)*x[1]
        arg[0,1,0,0]=(-0.0586985419455)*x[0]
        arg[0,1,0,1]=(-0.788101334389)*x[1]
        arg[0,1,1,0]=(0.615023802803)*x[0]
        arg[0,1,1,1]=(0.965381283357)*x[1]
        arg[0,1,2,0]=(0.173979432067)*x[0]
        arg[0,1,2,1]=(-0.980483156424)*x[1]
        arg[0,2,0,0]=(-0.345940909471)*x[0]
        arg[0,2,0,1]=(0.00162430279595)*x[1]
        arg[0,2,1,0]=(-0.0847413350082)*x[0]
        arg[0,2,1,1]=(-0.911248036753)*x[1]
        arg[0,2,2,0]=(-0.817295589942)*x[0]
        arg[0,2,2,1]=(-0.0514569401185)*x[1]
        arg[0,3,0,0]=(0.329704440159)*x[0]
        arg[0,3,0,1]=(0.580379057442)*x[1]
        arg[0,3,1,0]=(0.737014729605)*x[0]
        arg[0,3,1,1]=(0.535521740242)*x[1]
        arg[0,3,2,0]=(-0.943554721405)*x[0]
        arg[0,3,2,1]=(0.798294295959)*x[1]
        arg[0,4,0,0]=(-0.1814410847)*x[0]
        arg[0,4,0,1]=(0.968476629577)*x[1]
        arg[0,4,1,0]=(0.26595250332)*x[0]
        arg[0,4,1,1]=(-0.493516007909)*x[1]
        arg[0,4,2,0]=(0.374323739856)*x[0]
        arg[0,4,2,1]=(-0.914413601829)*x[1]
        arg[1,0,0,0]=(-0.701090812201)*x[0]
        arg[1,0,0,1]=(-0.403596866322)*x[1]
        arg[1,0,1,0]=(0.587183778075)*x[0]
        arg[1,0,1,1]=(-0.218129059558)*x[1]
        arg[1,0,2,0]=(-0.0205642762349)*x[0]
        arg[1,0,2,1]=(0.441382235525)*x[1]
        arg[1,1,0,0]=(0.909683859519)*x[0]
        arg[1,1,0,1]=(0.960088832754)*x[1]
        arg[1,1,1,0]=(0.0450355911985)*x[0]
        arg[1,1,1,1]=(0.366849576658)*x[1]
        arg[1,1,2,0]=(0.315490164497)*x[0]
        arg[1,1,2,1]=(0.533746946808)*x[1]
        arg[1,2,0,0]=(-0.552919155957)*x[0]
        arg[1,2,0,1]=(-0.647373009727)*x[1]
        arg[1,2,1,0]=(0.32617079311)*x[0]
        arg[1,2,1,1]=(-0.3424263412)*x[1]
        arg[1,2,2,0]=(0.848733118893)*x[0]
        arg[1,2,2,1]=(-0.65341712364)*x[1]
        arg[1,3,0,0]=(-0.698538235208)*x[0]
        arg[1,3,0,1]=(-0.176858090319)*x[1]
        arg[1,3,1,0]=(-0.956962782849)*x[0]
        arg[1,3,1,1]=(0.53041598553)*x[1]
        arg[1,3,2,0]=(-0.241715236701)*x[0]
        arg[1,3,2,1]=(0.0340608544103)*x[1]
        arg[1,4,0,0]=(-0.399279751219)*x[0]
        arg[1,4,0,1]=(0.383860128496)*x[1]
        arg[1,4,1,0]=(0.229189108307)*x[0]
        arg[1,4,1,1]=(0.963667605308)*x[1]
        arg[1,4,2,0]=(0.731912848998)*x[0]
        arg[1,4,2,1]=(0.0534970206048)*x[1]
        arg[2,0,0,0]=(0.422292915614)*x[0]
        arg[2,0,0,1]=(0.915607515426)*x[1]
        arg[2,0,1,0]=(0.451054723135)*x[0]
        arg[2,0,1,1]=(0.0400870534188)*x[1]
        arg[2,0,2,0]=(-0.851047453002)*x[0]
        arg[2,0,2,1]=(0.300957420969)*x[1]
        arg[2,1,0,0]=(0.636801515211)*x[0]
        arg[2,1,0,1]=(0.386798062533)*x[1]
        arg[2,1,1,0]=(-0.0501653606261)*x[0]
        arg[2,1,1,1]=(0.46515271421)*x[1]
        arg[2,1,2,0]=(-0.416436918527)*x[0]
        arg[2,1,2,1]=(0.214018855333)*x[1]
        arg[2,2,0,0]=(0.0516439357756)*x[0]
        arg[2,2,0,1]=(-0.237707451356)*x[1]
        arg[2,2,1,0]=(0.117559050121)*x[0]
        arg[2,2,1,1]=(0.164349348553)*x[1]
        arg[2,2,2,0]=(0.0524038362463)*x[0]
        arg[2,2,2,1]=(0.690751059081)*x[1]
        arg[2,3,0,0]=(-0.942079336547)*x[0]
        arg[2,3,0,1]=(0.340143899875)*x[1]
        arg[2,3,1,0]=(-0.630411753789)*x[0]
        arg[2,3,1,1]=(-0.722078100418)*x[1]
        arg[2,3,2,0]=(-0.310606406168)*x[0]
        arg[2,3,2,1]=(0.728069014178)*x[1]
        arg[2,4,0,0]=(0.981678768954)*x[0]
        arg[2,4,0,1]=(-0.273023706956)*x[1]
        arg[2,4,1,0]=(0.307187251211)*x[0]
        arg[2,4,1,1]=(-0.885309661176)*x[1]
        arg[2,4,2,0]=(-0.808022645995)*x[0]
        arg[2,4,2,1]=(-0.21488080753)*x[1]
        arg[3,0,0,0]=(-0.785416067695)*x[0]
        arg[3,0,0,1]=(0.344457091999)*x[1]
        arg[3,0,1,0]=(-0.845835855973)*x[0]
        arg[3,0,1,1]=(0.365474280353)*x[1]
        arg[3,0,2,0]=(-0.250535373488)*x[0]
        arg[3,0,2,1]=(-0.888004656815)*x[1]
        arg[3,1,0,0]=(-0.0652508248717)*x[0]
        arg[3,1,0,1]=(0.402069695186)*x[1]
        arg[3,1,1,0]=(-0.229714763188)*x[0]
        arg[3,1,1,1]=(-0.866938136156)*x[1]
        arg[3,1,2,0]=(-0.638603745536)*x[0]
        arg[3,1,2,1]=(-0.997108734664)*x[1]
        arg[3,2,0,0]=(-0.182784593825)*x[0]
        arg[3,2,0,1]=(0.426951207538)*x[1]
        arg[3,2,1,0]=(-0.958149736654)*x[0]
        arg[3,2,1,1]=(-0.58458370587)*x[1]
        arg[3,2,2,0]=(0.410243417149)*x[0]
        arg[3,2,2,1]=(0.764474507608)*x[1]
        arg[3,3,0,0]=(0.235623988334)*x[0]
        arg[3,3,0,1]=(-0.105137401903)*x[1]
        arg[3,3,1,0]=(-0.957743319072)*x[0]
        arg[3,3,1,1]=(0.285680328399)*x[1]
        arg[3,3,2,0]=(-0.525269717395)*x[0]
        arg[3,3,2,1]=(0.655974370871)*x[1]
        arg[3,4,0,0]=(0.926779313365)*x[0]
        arg[3,4,0,1]=(-0.111556488031)*x[1]
        arg[3,4,1,0]=(-0.672785588938)*x[0]
        arg[3,4,1,1]=(-0.687907843899)*x[1]
        arg[3,4,2,0]=(0.503984478085)*x[0]
        arg[3,4,2,1]=(0.213194091516)*x[1]
        ref=sqrt((6.92178561091)+(4.96610502195))

      else:
        arg=Data(0,(4, 5, 3, 3),w)
        arg[0,0,0,0]=(-0.641108959812)*x[0]
        arg[0,0,0,1]=(0.395837684631)*x[1]
        arg[0,0,0,2]=(0.36258240616)*x[2]
        arg[0,0,1,0]=(-0.0543293722691)*x[0]
        arg[0,0,1,1]=(0.850820297369)*x[1]
        arg[0,0,1,2]=(0.0910730209593)*x[2]
        arg[0,0,2,0]=(-0.831108730829)*x[0]
        arg[0,0,2,1]=(-0.653018722047)*x[1]
        arg[0,0,2,2]=(0.29052921548)*x[2]
        arg[0,1,0,0]=(-0.115846569271)*x[0]
        arg[0,1,0,1]=(-0.828080308748)*x[1]
        arg[0,1,0,2]=(0.715255255273)*x[2]
        arg[0,1,1,0]=(-0.0106077934044)*x[0]
        arg[0,1,1,1]=(0.358323295554)*x[1]
        arg[0,1,1,2]=(-0.916496266702)*x[2]
        arg[0,1,2,0]=(0.740969432327)*x[0]
        arg[0,1,2,1]=(0.0631507192754)*x[1]
        arg[0,1,2,2]=(0.440009320758)*x[2]
        arg[0,2,0,0]=(0.689964376385)*x[0]
        arg[0,2,0,1]=(0.58199412545)*x[1]
        arg[0,2,0,2]=(0.649058556936)*x[2]
        arg[0,2,1,0]=(0.811660544475)*x[0]
        arg[0,2,1,1]=(-0.121902248927)*x[1]
        arg[0,2,1,2]=(-0.0750232179247)*x[2]
        arg[0,2,2,0]=(0.779166450882)*x[0]
        arg[0,2,2,1]=(0.298560101475)*x[1]
        arg[0,2,2,2]=(-0.776198356206)*x[2]
        arg[0,3,0,0]=(0.79804128408)*x[0]
        arg[0,3,0,1]=(-0.378223308322)*x[1]
        arg[0,3,0,2]=(0.191534109359)*x[2]
        arg[0,3,1,0]=(-0.59973117885)*x[0]
        arg[0,3,1,1]=(0.149042766249)*x[1]
        arg[0,3,1,2]=(-0.357473233062)*x[2]
        arg[0,3,2,0]=(-0.343758339283)*x[0]
        arg[0,3,2,1]=(-0.140764743713)*x[1]
        arg[0,3,2,2]=(-0.506377878901)*x[2]
        arg[0,4,0,0]=(-0.967765607304)*x[0]
        arg[0,4,0,1]=(-0.121373931498)*x[1]
        arg[0,4,0,2]=(-0.300794608584)*x[2]
        arg[0,4,1,0]=(-0.604537150389)*x[0]
        arg[0,4,1,1]=(-0.290834102056)*x[1]
        arg[0,4,1,2]=(0.509331591828)*x[2]
        arg[0,4,2,0]=(0.643653985564)*x[0]
        arg[0,4,2,1]=(-0.599746831159)*x[1]
        arg[0,4,2,2]=(0.843193710828)*x[2]
        arg[1,0,0,0]=(-0.598249432719)*x[0]
        arg[1,0,0,1]=(0.840724468488)*x[1]
        arg[1,0,0,2]=(0.558893493847)*x[2]
        arg[1,0,1,0]=(0.843245438555)*x[0]
        arg[1,0,1,1]=(0.280919545563)*x[1]
        arg[1,0,1,2]=(-0.0549504830615)*x[2]
        arg[1,0,2,0]=(0.295812998973)*x[0]
        arg[1,0,2,1]=(0.513221249344)*x[1]
        arg[1,0,2,2]=(-0.432253600492)*x[2]
        arg[1,1,0,0]=(0.300696333449)*x[0]
        arg[1,1,0,1]=(0.964202717779)*x[1]
        arg[1,1,0,2]=(0.131730374204)*x[2]
        arg[1,1,1,0]=(0.209135189207)*x[0]
        arg[1,1,1,1]=(0.301389564979)*x[1]
        arg[1,1,1,2]=(0.378481289816)*x[2]
        arg[1,1,2,0]=(0.407413524533)*x[0]
        arg[1,1,2,1]=(0.184345610049)*x[1]
        arg[1,1,2,2]=(-0.884401323556)*x[2]
        arg[1,2,0,0]=(0.886883417187)*x[0]
        arg[1,2,0,1]=(-0.0524504232352)*x[1]
        arg[1,2,0,2]=(0.049116549681)*x[2]
        arg[1,2,1,0]=(-0.41240815391)*x[0]
        arg[1,2,1,1]=(-0.523033558623)*x[1]
        arg[1,2,1,2]=(-0.920238404199)*x[2]
        arg[1,2,2,0]=(-0.635169424461)*x[0]
        arg[1,2,2,1]=(0.773163888164)*x[1]
        arg[1,2,2,2]=(0.554400056145)*x[2]
        arg[1,3,0,0]=(0.573361031392)*x[0]
        arg[1,3,0,1]=(-0.107749750058)*x[1]
        arg[1,3,0,2]=(-0.971799077244)*x[2]
        arg[1,3,1,0]=(-0.103579714499)*x[0]
        arg[1,3,1,1]=(0.93329275867)*x[1]
        arg[1,3,1,2]=(0.994409555402)*x[2]
        arg[1,3,2,0]=(0.571961012279)*x[0]
        arg[1,3,2,1]=(-0.54961086377)*x[1]
        arg[1,3,2,2]=(0.563815465418)*x[2]
        arg[1,4,0,0]=(-0.820638948871)*x[0]
        arg[1,4,0,1]=(0.407750087109)*x[1]
        arg[1,4,0,2]=(0.0911636159786)*x[2]
        arg[1,4,1,0]=(0.371133349901)*x[0]
        arg[1,4,1,1]=(0.688167208194)*x[1]
        arg[1,4,1,2]=(0.188789217725)*x[2]
        arg[1,4,2,0]=(0.961439941063)*x[0]
        arg[1,4,2,1]=(0.524326385264)*x[1]
        arg[1,4,2,2]=(-0.0866064394717)*x[2]
        arg[2,0,0,0]=(0.392756517944)*x[0]
        arg[2,0,0,1]=(0.713723092455)*x[1]
        arg[2,0,0,2]=(0.68441988583)*x[2]
        arg[2,0,1,0]=(-0.471505369012)*x[0]
        arg[2,0,1,1]=(0.152712812211)*x[1]
        arg[2,0,1,2]=(-0.579106568625)*x[2]
        arg[2,0,2,0]=(0.637759469489)*x[0]
        arg[2,0,2,1]=(-0.986405270597)*x[1]
        arg[2,0,2,2]=(0.498109007475)*x[2]
        arg[2,1,0,0]=(0.870185247694)*x[0]
        arg[2,1,0,1]=(0.738095675991)*x[1]
        arg[2,1,0,2]=(-0.613836737976)*x[2]
        arg[2,1,1,0]=(-0.507009374688)*x[0]
        arg[2,1,1,1]=(0.00752595652646)*x[1]
        arg[2,1,1,2]=(-0.389118104751)*x[2]
        arg[2,1,2,0]=(-0.466733433552)*x[0]
        arg[2,1,2,1]=(-0.356829687828)*x[1]
        arg[2,1,2,2]=(0.691337110432)*x[2]
        arg[2,2,0,0]=(-0.684116912917)*x[0]
        arg[2,2,0,1]=(0.705774798227)*x[1]
        arg[2,2,0,2]=(0.522279733527)*x[2]
        arg[2,2,1,0]=(0.390315182318)*x[0]
        arg[2,2,1,1]=(-0.450165044241)*x[1]
        arg[2,2,1,2]=(0.436062806791)*x[2]
        arg[2,2,2,0]=(0.973600590724)*x[0]
        arg[2,2,2,1]=(-0.89116205465)*x[1]
        arg[2,2,2,2]=(-0.575262591791)*x[2]
        arg[2,3,0,0]=(0.842833949385)*x[0]
        arg[2,3,0,1]=(-0.69939817904)*x[1]
        arg[2,3,0,2]=(0.343679860971)*x[2]
        arg[2,3,1,0]=(-0.387030334358)*x[0]
        arg[2,3,1,1]=(0.901637622406)*x[1]
        arg[2,3,1,2]=(0.054184099786)*x[2]
        arg[2,3,2,0]=(0.652712575782)*x[0]
        arg[2,3,2,1]=(0.286474075274)*x[1]
        arg[2,3,2,2]=(-0.745166160145)*x[2]
        arg[2,4,0,0]=(-0.104591304669)*x[0]
        arg[2,4,0,1]=(0.417496371683)*x[1]
        arg[2,4,0,2]=(0.0579341684769)*x[2]
        arg[2,4,1,0]=(0.24486914915)*x[0]
        arg[2,4,1,1]=(-0.166211416717)*x[1]
        arg[2,4,1,2]=(0.651515760357)*x[2]
        arg[2,4,2,0]=(0.934693067557)*x[0]
        arg[2,4,2,1]=(0.628486602755)*x[1]
        arg[2,4,2,2]=(0.855313741494)*x[2]
        arg[3,0,0,0]=(0.2241978058)*x[0]
        arg[3,0,0,1]=(0.389284487657)*x[1]
        arg[3,0,0,2]=(0.711521372962)*x[2]
        arg[3,0,1,0]=(0.653184309521)*x[0]
        arg[3,0,1,1]=(0.270052840275)*x[1]
        arg[3,0,1,2]=(0.388881294549)*x[2]
        arg[3,0,2,0]=(0.973583452692)*x[0]
        arg[3,0,2,1]=(0.508175099792)*x[1]
        arg[3,0,2,2]=(-0.989999307019)*x[2]
        arg[3,1,0,0]=(0.935765996464)*x[0]
        arg[3,1,0,1]=(-0.186350920716)*x[1]
        arg[3,1,0,2]=(-0.809013452979)*x[2]
        arg[3,1,1,0]=(0.467966757364)*x[0]
        arg[3,1,1,1]=(-0.347420416508)*x[1]
        arg[3,1,1,2]=(0.686386725318)*x[2]
        arg[3,1,2,0]=(-0.8117641465)*x[0]
        arg[3,1,2,1]=(-0.357390874294)*x[1]
        arg[3,1,2,2]=(-0.0202262660381)*x[2]
        arg[3,2,0,0]=(-0.181594076044)*x[0]
        arg[3,2,0,1]=(0.0655758078072)*x[1]
        arg[3,2,0,2]=(0.926686460908)*x[2]
        arg[3,2,1,0]=(-0.700426183586)*x[0]
        arg[3,2,1,1]=(0.600692261192)*x[1]
        arg[3,2,1,2]=(0.757146807006)*x[2]
        arg[3,2,2,0]=(0.588097603826)*x[0]
        arg[3,2,2,1]=(-0.748732775295)*x[1]
        arg[3,2,2,2]=(0.628960378488)*x[2]
        arg[3,3,0,0]=(-0.0612899752684)*x[0]
        arg[3,3,0,1]=(0.567198903535)*x[1]
        arg[3,3,0,2]=(0.743996205698)*x[2]
        arg[3,3,1,0]=(0.333763157477)*x[0]
        arg[3,3,1,1]=(0.795773692731)*x[1]
        arg[3,3,1,2]=(-0.676301506816)*x[2]
        arg[3,3,2,0]=(-0.255504087739)*x[0]
        arg[3,3,2,1]=(-0.556263970561)*x[1]
        arg[3,3,2,2]=(0.721245074469)*x[2]
        arg[3,4,0,0]=(0.384551582302)*x[0]
        arg[3,4,0,1]=(0.849519144853)*x[1]
        arg[3,4,0,2]=(0.101881023262)*x[2]
        arg[3,4,1,0]=(-0.685891926653)*x[0]
        arg[3,4,1,1]=(0.390887672448)*x[1]
        arg[3,4,1,2]=(0.0911617386398)*x[2]
        arg[3,4,2,0]=(-0.889618312513)*x[0]
        arg[3,4,2,1]=(0.130310876981)*x[1]
        arg[3,4,2,2]=(0.958562278204)*x[2]
        ref=sqrt((12.9075635889)+(5.72789470359))

      res=L2(arg)
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertAlmostEqual(res,ref,int(-log10(self.RES_TOL)),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_L2_onFunctionOnContactOne_fromSymbol_rank0(self):
      """
      tests L2-norm of Symbol on the FunctionOnContactOne

      assumptions: self.domain supports integration on FunctionOnContactOne
      """
      dim=self.domain.getDim()
      w=FunctionOnContactOne(self.domain)
      x=w.getX()
      if dim==2:
        arg=Symbol(shape=(),dim=dim)
        s=Data(0,(),w)
        s=(-0.849531528726)*x[0]
        ref=sqrt((0.0)+(0.180425954575))

      else:
        arg=Symbol(shape=(),dim=dim)
        s=Data(0,(),w)
        s=(-0.708706581422)*x[0]
        ref=sqrt((0.0)+(0.125566254638))

      res=L2(arg)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertAlmostEqual(sub,ref,int(-log10(self.RES_TOL)),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_L2_onFunctionOnContactOne_fromSymbol_rank1(self):
      """
      tests L2-norm of Symbol on the FunctionOnContactOne

      assumptions: self.domain supports integration on FunctionOnContactOne
      """
      dim=self.domain.getDim()
      w=FunctionOnContactOne(self.domain)
      x=w.getX()
      if dim==2:
        arg=Symbol(shape=(2,),dim=dim)
        s=Data(0,(2,),w)
        s[0]=(-0.852476696951)*x[0]
        s[1]=(0.669311710064)*x[1]
        ref=sqrt((0.149326055076)+(0.181679129711))

      else:
        arg=Symbol(shape=(3,),dim=dim)
        s=Data(0,(3,),w)
        s[0]=(0.0886648134124)*x[0]
        s[1]=(-0.98107163405)*x[1]
        s[2]=(0.763548755951)*x[2]
        ref=sqrt((0.51516941795)+(0.00196536228436))

      res=L2(arg)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertAlmostEqual(sub,ref,int(-log10(self.RES_TOL)),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_L2_onFunctionOnContactOne_fromSymbol_rank2(self):
      """
      tests L2-norm of Symbol on the FunctionOnContactOne

      assumptions: self.domain supports integration on FunctionOnContactOne
      """
      dim=self.domain.getDim()
      w=FunctionOnContactOne(self.domain)
      x=w.getX()
      if dim==2:
        arg=Symbol(shape=(4, 2),dim=dim)
        s=Data(0,(4, 2),w)
        s[0,0]=(0.534469076507)*x[0]
        s[0,1]=(0.707717684613)*x[1]
        s[1,0]=(0.697608696253)*x[0]
        s[1,1]=(0.567450669898)*x[1]
        s[2,0]=(-0.622381718509)*x[0]
        s[2,1]=(0.546984463965)*x[1]
        s[3,0]=(-0.0825527152886)*x[0]
        s[3,1]=(-0.581446541532)*x[1]
        ref=sqrt((0.486712222787)+(0.291622260292))

      else:
        arg=Symbol(shape=(4, 3),dim=dim)
        s=Data(0,(4, 3),w)
        s[0,0]=(-0.860491143615)*x[0]
        s[0,1]=(0.140822783856)*x[1]
        s[0,2]=(-0.850233173189)*x[2]
        s[1,0]=(-0.0526405018976)*x[0]
        s[1,1]=(-0.620873428306)*x[1]
        s[1,2]=(-0.68323428604)*x[2]
        s[2,0]=(0.995631462645)*x[0]
        s[2,1]=(0.624478975925)*x[1]
        s[2,2]=(0.241456353598)*x[2]
        s[3,0]=(0.167743672846)*x[0]
        s[3,1]=(0.340565516735)*x[1]
        s[3,2]=(0.499048848748)*x[2]
        ref=sqrt((0.802776731843)+(0.440658994967))

      res=L2(arg)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertAlmostEqual(sub,ref,int(-log10(self.RES_TOL)),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_L2_onFunctionOnContactOne_fromSymbol_rank3(self):
      """
      tests L2-norm of Symbol on the FunctionOnContactOne

      assumptions: self.domain supports integration on FunctionOnContactOne
      """
      dim=self.domain.getDim()
      w=FunctionOnContactOne(self.domain)
      x=w.getX()
      if dim==2:
        arg=Symbol(shape=(6, 2, 2),dim=dim)
        s=Data(0,(6, 2, 2),w)
        s[0,0,0]=(0.53006006661)*x[0]
        s[0,0,1]=(0.855136875508)*x[1]
        s[0,1,0]=(0.965271929044)*x[0]
        s[0,1,1]=(0.00691131453267)*x[1]
        s[1,0,0]=(-0.244825630321)*x[0]
        s[1,0,1]=(-0.840591747582)*x[1]
        s[1,1,0]=(-0.0906837575187)*x[0]
        s[1,1,1]=(0.459130145496)*x[1]
        s[2,0,0]=(0.16143268528)*x[0]
        s[2,0,1]=(0.719843211037)*x[1]
        s[2,1,0]=(0.233952330698)*x[0]
        s[2,1,1]=(0.212086642323)*x[1]
        s[3,0,0]=(0.323764554006)*x[0]
        s[3,0,1]=(0.693684508228)*x[1]
        s[3,1,0]=(0.48296993334)*x[0]
        s[3,1,1]=(-0.13807549797)*x[1]
        s[4,0,0]=(-0.688641980918)*x[0]
        s[4,0,1]=(-0.98785692721)*x[1]
        s[4,1,0]=(0.987040832755)*x[0]
        s[4,1,1]=(-0.734764958687)*x[1]
        s[5,0,0]=(0.244402182319)*x[0]
        s[5,0,1]=(0.974686625391)*x[1]
        s[5,1,0]=(-0.277257927506)*x[0]
        s[5,1,1]=(0.0773106723551)*x[1]
        ref=sqrt((1.72795055403)+(0.821209030177))

      else:
        arg=Symbol(shape=(6, 2, 3),dim=dim)
        s=Data(0,(6, 2, 3),w)
        s[0,0,0]=(-0.61386584344)*x[0]
        s[0,0,1]=(-0.032040523373)*x[1]
        s[0,0,2]=(-0.723104197392)*x[2]
        s[0,1,0]=(0.665794765401)*x[0]
        s[0,1,1]=(-0.842083120109)*x[1]
        s[0,1,2]=(-0.642000380947)*x[2]
        s[1,0,0]=(-0.140120073896)*x[0]
        s[1,0,1]=(0.852883705123)*x[1]
        s[1,0,2]=(0.538615525775)*x[2]
        s[1,1,0]=(-0.268324107667)*x[0]
        s[1,1,1]=(0.723423091154)*x[1]
        s[1,1,2]=(-0.0210208139288)*x[2]
        s[2,0,0]=(-0.13072934339)*x[0]
        s[2,0,1]=(-0.287830796935)*x[1]
        s[2,0,2]=(-0.374274267746)*x[2]
        s[2,1,0]=(0.852351589285)*x[0]
        s[2,1,1]=(-0.16416141595)*x[1]
        s[2,1,2]=(0.218655720718)*x[2]
        s[3,0,0]=(0.0736657503857)*x[0]
        s[3,0,1]=(0.484109452084)*x[1]
        s[3,0,2]=(-0.68960733775)*x[2]
        s[3,1,0]=(0.866791078374)*x[0]
        s[3,1,1]=(-0.771121017091)*x[1]
        s[3,1,2]=(-0.643197172218)*x[2]
        s[4,0,0]=(0.00890571083899)*x[0]
        s[4,0,1]=(-0.762935399812)*x[1]
        s[4,0,2]=(-0.240832580447)*x[2]
        s[4,1,0]=(-0.657527623928)*x[0]
        s[4,1,1]=(0.558483255401)*x[1]
        s[4,1,2]=(-0.439397930536)*x[2]
        s[5,0,0]=(0.530029745037)*x[0]
        s[5,0,1]=(0.138779547908)*x[1]
        s[5,0,2]=(-0.0456719603355)*x[2]
        s[5,1,0]=(-0.751937380557)*x[0]
        s[5,1,1]=(-0.37055796529)*x[1]
        s[5,1,2]=(-0.285251484378)*x[2]
        ref=sqrt((2.19582819839)+(0.922713864342))

      res=L2(arg)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertAlmostEqual(sub,ref,int(-log10(self.RES_TOL)),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_L2_onFunctionOnContactOne_fromSymbol_rank4(self):
      """
      tests L2-norm of Symbol on the FunctionOnContactOne

      assumptions: self.domain supports integration on FunctionOnContactOne
      """
      dim=self.domain.getDim()
      w=FunctionOnContactOne(self.domain)
      x=w.getX()
      if dim==2:
        arg=Symbol(shape=(4, 5, 3, 2),dim=dim)
        s=Data(0,(4, 5, 3, 2),w)
        s[0,0,0,0]=(0.396322609451)*x[0]
        s[0,0,0,1]=(0.982533894141)*x[1]
        s[0,0,1,0]=(-0.129145024315)*x[0]
        s[0,0,1,1]=(-0.27837328441)*x[1]
        s[0,0,2,0]=(-0.731814618355)*x[0]
        s[0,0,2,1]=(0.989632262348)*x[1]
        s[0,1,0,0]=(-0.501434234651)*x[0]
        s[0,1,0,1]=(0.134247720647)*x[1]
        s[0,1,1,0]=(0.72638709986)*x[0]
        s[0,1,1,1]=(-0.66389079496)*x[1]
        s[0,1,2,0]=(-0.285431055287)*x[0]
        s[0,1,2,1]=(0.140502987885)*x[1]
        s[0,2,0,0]=(0.699721872914)*x[0]
        s[0,2,0,1]=(-0.232830090422)*x[1]
        s[0,2,1,0]=(0.322184745748)*x[0]
        s[0,2,1,1]=(0.725202642274)*x[1]
        s[0,2,2,0]=(0.398485732696)*x[0]
        s[0,2,2,1]=(0.230163541848)*x[1]
        s[0,3,0,0]=(-0.680544045823)*x[0]
        s[0,3,0,1]=(0.108130166525)*x[1]
        s[0,3,1,0]=(0.5077516367)*x[0]
        s[0,3,1,1]=(0.88975554254)*x[1]
        s[0,3,2,0]=(0.775170169413)*x[0]
        s[0,3,2,1]=(-0.275812588437)*x[1]
        s[0,4,0,0]=(0.348603150333)*x[0]
        s[0,4,0,1]=(-0.176482742037)*x[1]
        s[0,4,1,0]=(-0.927870151856)*x[0]
        s[0,4,1,1]=(-0.661067126232)*x[1]
        s[0,4,2,0]=(-0.208628316679)*x[0]
        s[0,4,2,1]=(0.383637018163)*x[1]
        s[1,0,0,0]=(0.88866687126)*x[0]
        s[1,0,0,1]=(0.260170060159)*x[1]
        s[1,0,1,0]=(0.126798500798)*x[0]
        s[1,0,1,1]=(0.264342178219)*x[1]
        s[1,0,2,0]=(0.662206500808)*x[0]
        s[1,0,2,1]=(-0.347569034957)*x[1]
        s[1,1,0,0]=(0.0960100811475)*x[0]
        s[1,1,0,1]=(-0.813364260291)*x[1]
        s[1,1,1,0]=(-0.128545578333)*x[0]
        s[1,1,1,1]=(0.557352804436)*x[1]
        s[1,1,2,0]=(0.177270533211)*x[0]
        s[1,1,2,1]=(-0.879697427458)*x[1]
        s[1,2,0,0]=(0.260811041962)*x[0]
        s[1,2,0,1]=(-0.0448529305202)*x[1]
        s[1,2,1,0]=(0.57598635298)*x[0]
        s[1,2,1,1]=(-0.899140376189)*x[1]
        s[1,2,2,0]=(-0.523456336484)*x[0]
        s[1,2,2,1]=(0.759703092295)*x[1]
        s[1,3,0,0]=(-0.262236560882)*x[0]
        s[1,3,0,1]=(-0.0829260777719)*x[1]
        s[1,3,1,0]=(0.327366286293)*x[0]
        s[1,3,1,1]=(-0.473163087927)*x[1]
        s[1,3,2,0]=(0.160705506648)*x[0]
        s[1,3,2,1]=(-0.347036912731)*x[1]
        s[1,4,0,0]=(0.343187252577)*x[0]
        s[1,4,0,1]=(-0.644467832838)*x[1]
        s[1,4,1,0]=(-0.95236021268)*x[0]
        s[1,4,1,1]=(0.475087073317)*x[1]
        s[1,4,2,0]=(0.808985496682)*x[0]
        s[1,4,2,1]=(-0.421357379281)*x[1]
        s[2,0,0,0]=(0.439969305501)*x[0]
        s[2,0,0,1]=(0.67689403622)*x[1]
        s[2,0,1,0]=(-0.754345274443)*x[0]
        s[2,0,1,1]=(0.881153579305)*x[1]
        s[2,0,2,0]=(0.825454096565)*x[0]
        s[2,0,2,1]=(-0.320622136485)*x[1]
        s[2,1,0,0]=(0.247206810015)*x[0]
        s[2,1,0,1]=(0.745173031392)*x[1]
        s[2,1,1,0]=(0.683694038926)*x[0]
        s[2,1,1,1]=(0.751191436507)*x[1]
        s[2,1,2,0]=(-0.535657258854)*x[0]
        s[2,1,2,1]=(0.832509218552)*x[1]
        s[2,2,0,0]=(0.397072754881)*x[0]
        s[2,2,0,1]=(-0.879089743252)*x[1]
        s[2,2,1,0]=(0.601245034596)*x[0]
        s[2,2,1,1]=(0.265236139375)*x[1]
        s[2,2,2,0]=(-0.0253945476718)*x[0]
        s[2,2,2,1]=(-0.976895844239)*x[1]
        s[2,3,0,0]=(-0.356083748879)*x[0]
        s[2,3,0,1]=(0.826339109837)*x[1]
        s[2,3,1,0]=(-0.594681379568)*x[0]
        s[2,3,1,1]=(-0.58277681276)*x[1]
        s[2,3,2,0]=(-0.408387409616)*x[0]
        s[2,3,2,1]=(0.602785352992)*x[1]
        s[2,4,0,0]=(-0.891295605242)*x[0]
        s[2,4,0,1]=(-0.750844606395)*x[1]
        s[2,4,1,0]=(-0.29493371455)*x[0]
        s[2,4,1,1]=(-0.0269046032275)*x[1]
        s[2,4,2,0]=(0.49367579924)*x[0]
        s[2,4,2,1]=(-0.671892240661)*x[1]
        s[3,0,0,0]=(0.526420934634)*x[0]
        s[3,0,0,1]=(0.43040092109)*x[1]
        s[3,0,1,0]=(-0.135568748195)*x[0]
        s[3,0,1,1]=(0.146386072934)*x[1]
        s[3,0,2,0]=(-0.609373566828)*x[0]
        s[3,0,2,1]=(0.763194941657)*x[1]
        s[3,1,0,0]=(0.511063874735)*x[0]
        s[3,1,0,1]=(0.894580623201)*x[1]
        s[3,1,1,0]=(-0.372632207973)*x[0]
        s[3,1,1,1]=(-0.819194915438)*x[1]
        s[3,1,2,0]=(0.258313934708)*x[0]
        s[3,1,2,1]=(-0.708534091752)*x[1]
        s[3,2,0,0]=(0.439636692996)*x[0]
        s[3,2,0,1]=(-0.996699172676)*x[1]
        s[3,2,1,0]=(0.24213697925)*x[0]
        s[3,2,1,1]=(0.0487822729335)*x[1]
        s[3,2,2,0]=(-0.747366711813)*x[0]
        s[3,2,2,1]=(0.533579125235)*x[1]
        s[3,3,0,0]=(0.590618210003)*x[0]
        s[3,3,0,1]=(-0.378862695527)*x[1]
        s[3,3,1,0]=(0.486532205417)*x[0]
        s[3,3,1,1]=(-0.568241987046)*x[1]
        s[3,3,2,0]=(-0.368983251697)*x[0]
        s[3,3,2,1]=(0.978959945336)*x[1]
        s[3,4,0,0]=(-0.944651128993)*x[0]
        s[3,4,0,1]=(0.449642532401)*x[1]
        s[3,4,1,0]=(-0.171120868333)*x[0]
        s[3,4,1,1]=(-0.985738801506)*x[1]
        s[3,4,2,0]=(0.39325635403)*x[0]
        s[3,4,2,1]=(-0.355332366915)*x[1]
        ref=sqrt((7.76922176438)+(4.20495359519))

      else:
        arg=Symbol(shape=(4, 5, 3, 3),dim=dim)
        s=Data(0,(4, 5, 3, 3),w)
        s[0,0,0,0]=(0.296145515325)*x[0]
        s[0,0,0,1]=(-0.112284888661)*x[1]
        s[0,0,0,2]=(0.0746491379222)*x[2]
        s[0,0,1,0]=(0.14756535943)*x[0]
        s[0,0,1,1]=(-0.653188513529)*x[1]
        s[0,0,1,2]=(-0.881965485447)*x[2]
        s[0,0,2,0]=(-0.645436136716)*x[0]
        s[0,0,2,1]=(-0.996242179756)*x[1]
        s[0,0,2,2]=(-0.613744842724)*x[2]
        s[0,1,0,0]=(0.543040571766)*x[0]
        s[0,1,0,1]=(0.172439667925)*x[1]
        s[0,1,0,2]=(0.134262077825)*x[2]
        s[0,1,1,0]=(0.206973085316)*x[0]
        s[0,1,1,1]=(0.78448343454)*x[1]
        s[0,1,1,2]=(-0.533409163778)*x[2]
        s[0,1,2,0]=(0.430560905893)*x[0]
        s[0,1,2,1]=(-0.821838520257)*x[1]
        s[0,1,2,2]=(-0.814853740301)*x[2]
        s[0,2,0,0]=(-0.763338805036)*x[0]
        s[0,2,0,1]=(0.973800200861)*x[1]
        s[0,2,0,2]=(-0.591166623249)*x[2]
        s[0,2,1,0]=(-0.338452805899)*x[0]
        s[0,2,1,1]=(-0.324236181738)*x[1]
        s[0,2,1,2]=(0.738595180918)*x[2]
        s[0,2,2,0]=(0.618927719575)*x[0]
        s[0,2,2,1]=(0.0632651077831)*x[1]
        s[0,2,2,2]=(0.415573120842)*x[2]
        s[0,3,0,0]=(-0.786194037231)*x[0]
        s[0,3,0,1]=(-0.0402223807021)*x[1]
        s[0,3,0,2]=(0.344479980368)*x[2]
        s[0,3,1,0]=(0.545021524016)*x[0]
        s[0,3,1,1]=(0.0563273477935)*x[1]
        s[0,3,1,2]=(-0.0659388897897)*x[2]
        s[0,3,2,0]=(0.152424684107)*x[0]
        s[0,3,2,1]=(-0.518122801248)*x[1]
        s[0,3,2,2]=(0.985522487899)*x[2]
        s[0,4,0,0]=(-0.830069953806)*x[0]
        s[0,4,0,1]=(0.571063785633)*x[1]
        s[0,4,0,2]=(0.904417376158)*x[2]
        s[0,4,1,0]=(0.687144774983)*x[0]
        s[0,4,1,1]=(-0.250243991657)*x[1]
        s[0,4,1,2]=(0.211438343321)*x[2]
        s[0,4,2,0]=(-0.887502165654)*x[0]
        s[0,4,2,1]=(0.342464350162)*x[1]
        s[0,4,2,2]=(-0.389354004393)*x[2]
        s[1,0,0,0]=(-0.00381480047885)*x[0]
        s[1,0,0,1]=(0.584208423663)*x[1]
        s[1,0,0,2]=(-0.781219608112)*x[2]
        s[1,0,1,0]=(-0.484192327306)*x[0]
        s[1,0,1,1]=(-0.0693964278254)*x[1]
        s[1,0,1,2]=(0.401920719606)*x[2]
        s[1,0,2,0]=(-0.964645208799)*x[0]
        s[1,0,2,1]=(-0.458496519567)*x[1]
        s[1,0,2,2]=(-0.837611873973)*x[2]
        s[1,1,0,0]=(0.837066643335)*x[0]
        s[1,1,0,1]=(0.220732404668)*x[1]
        s[1,1,0,2]=(-0.341146357851)*x[2]
        s[1,1,1,0]=(-0.432882962388)*x[0]
        s[1,1,1,1]=(0.924385436071)*x[1]
        s[1,1,1,2]=(-0.465249254821)*x[2]
        s[1,1,2,0]=(0.307318032655)*x[0]
        s[1,1,2,1]=(0.914827475386)*x[1]
        s[1,1,2,2]=(-0.830756418498)*x[2]
        s[1,2,0,0]=(-0.406562274215)*x[0]
        s[1,2,0,1]=(-0.717178070516)*x[1]
        s[1,2,0,2]=(0.513401821021)*x[2]
        s[1,2,1,0]=(0.0903563906255)*x[0]
        s[1,2,1,1]=(0.0714233348867)*x[1]
        s[1,2,1,2]=(-0.629397568181)*x[2]
        s[1,2,2,0]=(0.164139578343)*x[0]
        s[1,2,2,1]=(-0.59013912099)*x[1]
        s[1,2,2,2]=(-0.406224661607)*x[2]
        s[1,3,0,0]=(0.0778758168202)*x[0]
        s[1,3,0,1]=(0.738003702753)*x[1]
        s[1,3,0,2]=(0.881333264003)*x[2]
        s[1,3,1,0]=(-0.175166747001)*x[0]
        s[1,3,1,1]=(0.326269577833)*x[1]
        s[1,3,1,2]=(-0.340110356196)*x[2]
        s[1,3,2,0]=(-0.443150540915)*x[0]
        s[1,3,2,1]=(0.116960071142)*x[1]
        s[1,3,2,2]=(0.969893913777)*x[2]
        s[1,4,0,0]=(0.487572431401)*x[0]
        s[1,4,0,1]=(-0.915415755722)*x[1]
        s[1,4,0,2]=(0.0590936356226)*x[2]
        s[1,4,1,0]=(0.968491519638)*x[0]
        s[1,4,1,1]=(0.393515954209)*x[1]
        s[1,4,1,2]=(-0.95643405719)*x[2]
        s[1,4,2,0]=(0.645368604523)*x[0]
        s[1,4,2,1]=(-0.157884217538)*x[1]
        s[1,4,2,2]=(0.0497276310466)*x[2]
        s[2,0,0,0]=(0.79948016484)*x[0]
        s[2,0,0,1]=(0.404876280848)*x[1]
        s[2,0,0,2]=(-0.0600452523729)*x[2]
        s[2,0,1,0]=(0.128681372314)*x[0]
        s[2,0,1,1]=(0.398166908201)*x[1]
        s[2,0,1,2]=(-0.890799046442)*x[2]
        s[2,0,2,0]=(-0.43878626818)*x[0]
        s[2,0,2,1]=(0.295788703551)*x[1]
        s[2,0,2,2]=(0.380983964299)*x[2]
        s[2,1,0,0]=(-0.426682002183)*x[0]
        s[2,1,0,1]=(0.13781617784)*x[1]
        s[2,1,0,2]=(0.964133330513)*x[2]
        s[2,1,1,0]=(0.397029527608)*x[0]
        s[2,1,1,1]=(0.985177398513)*x[1]
        s[2,1,1,2]=(-0.827997515899)*x[2]
        s[2,1,2,0]=(0.730645004164)*x[0]
        s[2,1,2,1]=(-0.797420668382)*x[1]
        s[2,1,2,2]=(-0.300290718797)*x[2]
        s[2,2,0,0]=(-0.257811741709)*x[0]
        s[2,2,0,1]=(0.108526509889)*x[1]
        s[2,2,0,2]=(0.657942581233)*x[2]
        s[2,2,1,0]=(0.486944254347)*x[0]
        s[2,2,1,1]=(0.498753219399)*x[1]
        s[2,2,1,2]=(-0.392553093833)*x[2]
        s[2,2,2,0]=(0.570390526883)*x[0]
        s[2,2,2,1]=(-0.655828186915)*x[1]
        s[2,2,2,2]=(-0.886504590868)*x[2]
        s[2,3,0,0]=(0.786841433677)*x[0]
        s[2,3,0,1]=(0.510317771549)*x[1]
        s[2,3,0,2]=(-0.0755114998263)*x[2]
        s[2,3,1,0]=(0.823613342893)*x[0]
        s[2,3,1,1]=(-0.494194753878)*x[1]
        s[2,3,1,2]=(-0.659935864742)*x[2]
        s[2,3,2,0]=(0.640809074487)*x[0]
        s[2,3,2,1]=(0.304983735262)*x[1]
        s[2,3,2,2]=(0.316983109882)*x[2]
        s[2,4,0,0]=(0.726358084867)*x[0]
        s[2,4,0,1]=(0.559356161307)*x[1]
        s[2,4,0,2]=(-0.619050390356)*x[2]
        s[2,4,1,0]=(0.201203183195)*x[0]
        s[2,4,1,1]=(-0.882713341877)*x[1]
        s[2,4,1,2]=(-0.480400204958)*x[2]
        s[2,4,2,0]=(-0.140145093838)*x[0]
        s[2,4,2,1]=(0.145271336227)*x[1]
        s[2,4,2,2]=(-0.870632579447)*x[2]
        s[3,0,0,0]=(0.691022073815)*x[0]
        s[3,0,0,1]=(-0.901932486396)*x[1]
        s[3,0,0,2]=(-0.358787504481)*x[2]
        s[3,0,1,0]=(-0.79150290154)*x[0]
        s[3,0,1,1]=(0.440401080151)*x[1]
        s[3,0,1,2]=(-0.656596021663)*x[2]
        s[3,0,2,0]=(0.373265557264)*x[0]
        s[3,0,2,1]=(0.147981174911)*x[1]
        s[3,0,2,2]=(-0.254845607807)*x[2]
        s[3,1,0,0]=(0.199574501407)*x[0]
        s[3,1,0,1]=(-0.676397866417)*x[1]
        s[3,1,0,2]=(0.83161020476)*x[2]
        s[3,1,1,0]=(-0.887259353592)*x[0]
        s[3,1,1,1]=(-0.599855381787)*x[1]
        s[3,1,1,2]=(0.440852418479)*x[2]
        s[3,1,2,0]=(0.0363737283834)*x[0]
        s[3,1,2,1]=(-0.242329100485)*x[1]
        s[3,1,2,2]=(-0.57882645459)*x[2]
        s[3,2,0,0]=(0.208985671572)*x[0]
        s[3,2,0,1]=(0.753748554448)*x[1]
        s[3,2,0,2]=(0.175858719301)*x[2]
        s[3,2,1,0]=(0.821744269994)*x[0]
        s[3,2,1,1]=(-0.590861142533)*x[1]
        s[3,2,1,2]=(0.766179113979)*x[2]
        s[3,2,2,0]=(-0.299111001599)*x[0]
        s[3,2,2,1]=(-0.817223826668)*x[1]
        s[3,2,2,2]=(-0.17369899949)*x[2]
        s[3,3,0,0]=(-0.478253760517)*x[0]
        s[3,3,0,1]=(-0.4618406658)*x[1]
        s[3,3,0,2]=(-0.434688925412)*x[2]
        s[3,3,1,0]=(0.13832631548)*x[0]
        s[3,3,1,1]=(0.407639738669)*x[1]
        s[3,3,1,2]=(-0.870829708621)*x[2]
        s[3,3,2,0]=(0.564907773571)*x[0]
        s[3,3,2,1]=(-0.113149301306)*x[1]
        s[3,3,2,2]=(0.972631828884)*x[2]
        s[3,4,0,0]=(0.505964924785)*x[0]
        s[3,4,0,1]=(-0.567647029204)*x[1]
        s[3,4,0,2]=(-0.535422935572)*x[2]
        s[3,4,1,0]=(-0.832260433763)*x[0]
        s[3,4,1,1]=(0.52551900128)*x[1]
        s[3,4,1,2]=(0.0171198096245)*x[2]
        s[3,4,2,0]=(-0.675829280696)*x[0]
        s[3,4,2,1]=(0.297095606749)*x[1]
        s[3,4,2,2]=(0.99944710061)*x[2]
        ref=sqrt((13.8090623863)+(4.67224475394))

      res=L2(arg)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertAlmostEqual(sub,ref,int(-log10(self.RES_TOL)),"wrong result")

   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_L2_onReducedFunctionOnContactOne_fromData_rank0(self):
      """
      tests L2-norm of Data on the ReducedFunctionOnContactOne

      assumptions: self.domain supports integration on ReducedFunctionOnContactOne
      """
      dim=self.domain.getDim()
      w=ReducedFunctionOnContactOne(self.domain)
      x=w.getX()
      if dim==2:
        arg=Data(0,(),w)
        arg=(0.39039157828)*sqrt(x[0])
        ref=sqrt((0.0)*3./2.+(0.038101396098)/0.5)

      else:
        arg=Data(0,(),w)
        arg=(0.893204658234)*sqrt(x[0])
        ref=sqrt((0.0)*3./2.+(0.199453640373)/0.5)

      res=L2(arg)
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertAlmostEqual(res,ref,int(-log10(self.RES_TOL)),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_L2_onReducedFunctionOnContactOne_fromData_rank1(self):
      """
      tests L2-norm of Data on the ReducedFunctionOnContactOne

      assumptions: self.domain supports integration on ReducedFunctionOnContactOne
      """
      dim=self.domain.getDim()
      w=ReducedFunctionOnContactOne(self.domain)
      x=w.getX()
      if dim==2:
        arg=Data(0,(2,),w)
        arg[0]=(0.697576620054)*sqrt(x[0])
        arg[1]=(-0.302037273777)*sqrt(x[1])
        ref=sqrt((0.0304088382502)*3./2.+(0.121653285212)/0.5)

      else:
        arg=Data(0,(3,),w)
        arg[0]=(0.649308272675)*sqrt(x[0])
        arg[1]=(-0.408829699836)*sqrt(x[1])
        arg[2]=(-0.984892977949)*sqrt(x[2])
        ref=sqrt((0.37905196716)*3./2.+(0.105400308241)/0.5)

      res=L2(arg)
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertAlmostEqual(res,ref,int(-log10(self.RES_TOL)),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_L2_onReducedFunctionOnContactOne_fromData_rank2(self):
      """
      tests L2-norm of Data on the ReducedFunctionOnContactOne

      assumptions: self.domain supports integration on ReducedFunctionOnContactOne
      """
      dim=self.domain.getDim()
      w=ReducedFunctionOnContactOne(self.domain)
      x=w.getX()
      if dim==2:
        arg=Data(0,(4, 2),w)
        arg[0,0]=(0.504133645896)*sqrt(x[0])
        arg[0,1]=(0.728228500775)*sqrt(x[1])
        arg[1,0]=(0.968237164633)*sqrt(x[0])
        arg[1,1]=(-0.205351628957)*sqrt(x[1])
        arg[2,0]=(-0.154008228702)*sqrt(x[0])
        arg[2,1]=(-0.972088404566)*sqrt(x[1])
        arg[3,0]=(0.585153521593)*sqrt(x[0])
        arg[3,1]=(0.371861820001)*sqrt(x[1])
        ref=sqrt((0.551907706774)*3./2.+(0.389439279561)/0.5)

      else:
        arg=Data(0,(4, 3),w)
        arg[0,0]=(-0.4847970605)*sqrt(x[0])
        arg[0,1]=(-0.664433021479)*sqrt(x[1])
        arg[0,2]=(-0.93730914676)*sqrt(x[2])
        arg[1,0]=(0.712624065023)*sqrt(x[0])
        arg[1,1]=(0.105431976424)*sqrt(x[1])
        arg[1,2]=(-0.452636806104)*sqrt(x[2])
        arg[2,0]=(0.488406064176)*sqrt(x[0])
        arg[2,1]=(0.0594879976584)*sqrt(x[1])
        arg[2,2]=(0.596963883202)*sqrt(x[2])
        arg[3,0]=(0.619366395328)*sqrt(x[0])
        arg[3,1]=(0.719965901041)*sqrt(x[1])
        arg[3,2]=(-0.568971277169)*sqrt(x[2])
        ref=sqrt((0.912666523048)*3./2.+(0.341254115776)/0.5)

      res=L2(arg)
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertAlmostEqual(res,ref,int(-log10(self.RES_TOL)),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_L2_onReducedFunctionOnContactOne_fromData_rank3(self):
      """
      tests L2-norm of Data on the ReducedFunctionOnContactOne

      assumptions: self.domain supports integration on ReducedFunctionOnContactOne
      """
      dim=self.domain.getDim()
      w=ReducedFunctionOnContactOne(self.domain)
      x=w.getX()
      if dim==2:
        arg=Data(0,(6, 2, 2),w)
        arg[0,0,0]=(-0.481233278183)*sqrt(x[0])
        arg[0,0,1]=(-0.883447576048)*sqrt(x[1])
        arg[0,1,0]=(0.525938652633)*sqrt(x[0])
        arg[0,1,1]=(-0.694971850651)*sqrt(x[1])
        arg[1,0,0]=(-0.935880633291)*sqrt(x[0])
        arg[1,0,1]=(0.970445286468)*sqrt(x[1])
        arg[1,1,0]=(-0.302571977945)*sqrt(x[0])
        arg[1,1,1]=(-0.237750319821)*sqrt(x[1])
        arg[2,0,0]=(0.922940652883)*sqrt(x[0])
        arg[2,0,1]=(-0.624756614078)*sqrt(x[1])
        arg[2,1,0]=(-0.146839388764)*sqrt(x[0])
        arg[2,1,1]=(-0.995551211092)*sqrt(x[1])
        arg[3,0,0]=(0.719184541649)*sqrt(x[0])
        arg[3,0,1]=(-0.629461063375)*sqrt(x[1])
        arg[3,1,0]=(-0.925581215276)*sqrt(x[0])
        arg[3,1,1]=(-0.564418658495)*sqrt(x[1])
        arg[4,0,0]=(-0.724035884026)*sqrt(x[0])
        arg[4,0,1]=(0.78393984221)*sqrt(x[1])
        arg[4,1,0]=(-0.722375249317)*sqrt(x[0])
        arg[4,1,1]=(0.27571990944)*sqrt(x[1])
        arg[5,0,0]=(-0.197579392553)*sqrt(x[0])
        arg[5,0,1]=(0.341717398275)*sqrt(x[1])
        arg[5,1,0]=(0.926327822859)*sqrt(x[0])
        arg[5,1,1]=(0.44114488285)*sqrt(x[1])
        ref=sqrt((1.78665006238)*3./2.+(1.41652558894)/0.5)

      else:
        arg=Data(0,(6, 2, 3),w)
        arg[0,0,0]=(-0.348448140437)*sqrt(x[0])
        arg[0,0,1]=(-0.530793589782)*sqrt(x[1])
        arg[0,0,2]=(0.386511493376)*sqrt(x[2])
        arg[0,1,0]=(0.00776536299593)*sqrt(x[0])
        arg[0,1,1]=(-0.0696129074064)*sqrt(x[1])
        arg[0,1,2]=(0.114179504216)*sqrt(x[2])
        arg[1,0,0]=(-0.681009917406)*sqrt(x[0])
        arg[1,0,1]=(0.763595403719)*sqrt(x[1])
        arg[1,0,2]=(-0.54459342544)*sqrt(x[2])
        arg[1,1,0]=(0.0839255723157)*sqrt(x[0])
        arg[1,1,1]=(0.269389515003)*sqrt(x[1])
        arg[1,1,2]=(0.612446509453)*sqrt(x[2])
        arg[2,0,0]=(0.600070579238)*sqrt(x[0])
        arg[2,0,1]=(-0.49585867531)*sqrt(x[1])
        arg[2,0,2]=(-0.766857395141)*sqrt(x[2])
        arg[2,1,0]=(-0.84515816441)*sqrt(x[0])
        arg[2,1,1]=(-0.594337789049)*sqrt(x[1])
        arg[2,1,2]=(0.774596009811)*sqrt(x[2])
        arg[3,0,0]=(-0.833982234688)*sqrt(x[0])
        arg[3,0,1]=(0.747667281218)*sqrt(x[1])
        arg[3,0,2]=(-0.77025764051)*sqrt(x[2])
        arg[3,1,0]=(0.0367023286497)*sqrt(x[0])
        arg[3,1,1]=(-0.503080137993)*sqrt(x[1])
        arg[3,1,2]=(-0.129478323797)*sqrt(x[2])
        arg[4,0,0]=(0.304123530695)*sqrt(x[0])
        arg[4,0,1]=(-0.873023368289)*sqrt(x[1])
        arg[4,0,2]=(-0.712740806672)*sqrt(x[2])
        arg[4,1,0]=(-0.0629132979843)*sqrt(x[0])
        arg[4,1,1]=(0.096843438525)*sqrt(x[1])
        arg[4,1,2]=(-0.608716559621)*sqrt(x[2])
        arg[5,0,0]=(-0.642902822517)*sqrt(x[0])
        arg[5,0,1]=(0.478134785533)*sqrt(x[1])
        arg[5,0,2]=(0.757891512092)*sqrt(x[2])
        arg[5,1,0]=(-0.144492755065)*sqrt(x[0])
        arg[5,1,1]=(0.286775195938)*sqrt(x[1])
        arg[5,1,2]=(-0.063580488638)*sqrt(x[2])
        ref=sqrt((2.50835195543)*3./2.+(0.723549067206)/0.5)

      res=L2(arg)
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertAlmostEqual(res,ref,int(-log10(self.RES_TOL)),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_L2_onReducedFunctionOnContactOne_fromData_rank4(self):
      """
      tests L2-norm of Data on the ReducedFunctionOnContactOne

      assumptions: self.domain supports integration on ReducedFunctionOnContactOne
      """
      dim=self.domain.getDim()
      w=ReducedFunctionOnContactOne(self.domain)
      x=w.getX()
      if dim==2:
        arg=Data(0,(4, 5, 3, 2),w)
        arg[0,0,0,0]=(0.523372259748)*sqrt(x[0])
        arg[0,0,0,1]=(-0.732795389957)*sqrt(x[1])
        arg[0,0,1,0]=(0.0571737128606)*sqrt(x[0])
        arg[0,0,1,1]=(-0.627260438721)*sqrt(x[1])
        arg[0,0,2,0]=(-0.945159375045)*sqrt(x[0])
        arg[0,0,2,1]=(-0.145103328237)*sqrt(x[1])
        arg[0,1,0,0]=(-0.0586985419455)*sqrt(x[0])
        arg[0,1,0,1]=(-0.788101334389)*sqrt(x[1])
        arg[0,1,1,0]=(0.615023802803)*sqrt(x[0])
        arg[0,1,1,1]=(0.965381283357)*sqrt(x[1])
        arg[0,1,2,0]=(0.173979432067)*sqrt(x[0])
        arg[0,1,2,1]=(-0.980483156424)*sqrt(x[1])
        arg[0,2,0,0]=(-0.345940909471)*sqrt(x[0])
        arg[0,2,0,1]=(0.00162430279595)*sqrt(x[1])
        arg[0,2,1,0]=(-0.0847413350082)*sqrt(x[0])
        arg[0,2,1,1]=(-0.911248036753)*sqrt(x[1])
        arg[0,2,2,0]=(-0.817295589942)*sqrt(x[0])
        arg[0,2,2,1]=(-0.0514569401185)*sqrt(x[1])
        arg[0,3,0,0]=(0.329704440159)*sqrt(x[0])
        arg[0,3,0,1]=(0.580379057442)*sqrt(x[1])
        arg[0,3,1,0]=(0.737014729605)*sqrt(x[0])
        arg[0,3,1,1]=(0.535521740242)*sqrt(x[1])
        arg[0,3,2,0]=(-0.943554721405)*sqrt(x[0])
        arg[0,3,2,1]=(0.798294295959)*sqrt(x[1])
        arg[0,4,0,0]=(-0.1814410847)*sqrt(x[0])
        arg[0,4,0,1]=(0.968476629577)*sqrt(x[1])
        arg[0,4,1,0]=(0.26595250332)*sqrt(x[0])
        arg[0,4,1,1]=(-0.493516007909)*sqrt(x[1])
        arg[0,4,2,0]=(0.374323739856)*sqrt(x[0])
        arg[0,4,2,1]=(-0.914413601829)*sqrt(x[1])
        arg[1,0,0,0]=(-0.701090812201)*sqrt(x[0])
        arg[1,0,0,1]=(-0.403596866322)*sqrt(x[1])
        arg[1,0,1,0]=(0.587183778075)*sqrt(x[0])
        arg[1,0,1,1]=(-0.218129059558)*sqrt(x[1])
        arg[1,0,2,0]=(-0.0205642762349)*sqrt(x[0])
        arg[1,0,2,1]=(0.441382235525)*sqrt(x[1])
        arg[1,1,0,0]=(0.909683859519)*sqrt(x[0])
        arg[1,1,0,1]=(0.960088832754)*sqrt(x[1])
        arg[1,1,1,0]=(0.0450355911985)*sqrt(x[0])
        arg[1,1,1,1]=(0.366849576658)*sqrt(x[1])
        arg[1,1,2,0]=(0.315490164497)*sqrt(x[0])
        arg[1,1,2,1]=(0.533746946808)*sqrt(x[1])
        arg[1,2,0,0]=(-0.552919155957)*sqrt(x[0])
        arg[1,2,0,1]=(-0.647373009727)*sqrt(x[1])
        arg[1,2,1,0]=(0.32617079311)*sqrt(x[0])
        arg[1,2,1,1]=(-0.3424263412)*sqrt(x[1])
        arg[1,2,2,0]=(0.848733118893)*sqrt(x[0])
        arg[1,2,2,1]=(-0.65341712364)*sqrt(x[1])
        arg[1,3,0,0]=(-0.698538235208)*sqrt(x[0])
        arg[1,3,0,1]=(-0.176858090319)*sqrt(x[1])
        arg[1,3,1,0]=(-0.956962782849)*sqrt(x[0])
        arg[1,3,1,1]=(0.53041598553)*sqrt(x[1])
        arg[1,3,2,0]=(-0.241715236701)*sqrt(x[0])
        arg[1,3,2,1]=(0.0340608544103)*sqrt(x[1])
        arg[1,4,0,0]=(-0.399279751219)*sqrt(x[0])
        arg[1,4,0,1]=(0.383860128496)*sqrt(x[1])
        arg[1,4,1,0]=(0.229189108307)*sqrt(x[0])
        arg[1,4,1,1]=(0.963667605308)*sqrt(x[1])
        arg[1,4,2,0]=(0.731912848998)*sqrt(x[0])
        arg[1,4,2,1]=(0.0534970206048)*sqrt(x[1])
        arg[2,0,0,0]=(0.422292915614)*sqrt(x[0])
        arg[2,0,0,1]=(0.915607515426)*sqrt(x[1])
        arg[2,0,1,0]=(0.451054723135)*sqrt(x[0])
        arg[2,0,1,1]=(0.0400870534188)*sqrt(x[1])
        arg[2,0,2,0]=(-0.851047453002)*sqrt(x[0])
        arg[2,0,2,1]=(0.300957420969)*sqrt(x[1])
        arg[2,1,0,0]=(0.636801515211)*sqrt(x[0])
        arg[2,1,0,1]=(0.386798062533)*sqrt(x[1])
        arg[2,1,1,0]=(-0.0501653606261)*sqrt(x[0])
        arg[2,1,1,1]=(0.46515271421)*sqrt(x[1])
        arg[2,1,2,0]=(-0.416436918527)*sqrt(x[0])
        arg[2,1,2,1]=(0.214018855333)*sqrt(x[1])
        arg[2,2,0,0]=(0.0516439357756)*sqrt(x[0])
        arg[2,2,0,1]=(-0.237707451356)*sqrt(x[1])
        arg[2,2,1,0]=(0.117559050121)*sqrt(x[0])
        arg[2,2,1,1]=(0.164349348553)*sqrt(x[1])
        arg[2,2,2,0]=(0.0524038362463)*sqrt(x[0])
        arg[2,2,2,1]=(0.690751059081)*sqrt(x[1])
        arg[2,3,0,0]=(-0.942079336547)*sqrt(x[0])
        arg[2,3,0,1]=(0.340143899875)*sqrt(x[1])
        arg[2,3,1,0]=(-0.630411753789)*sqrt(x[0])
        arg[2,3,1,1]=(-0.722078100418)*sqrt(x[1])
        arg[2,3,2,0]=(-0.310606406168)*sqrt(x[0])
        arg[2,3,2,1]=(0.728069014178)*sqrt(x[1])
        arg[2,4,0,0]=(0.981678768954)*sqrt(x[0])
        arg[2,4,0,1]=(-0.273023706956)*sqrt(x[1])
        arg[2,4,1,0]=(0.307187251211)*sqrt(x[0])
        arg[2,4,1,1]=(-0.885309661176)*sqrt(x[1])
        arg[2,4,2,0]=(-0.808022645995)*sqrt(x[0])
        arg[2,4,2,1]=(-0.21488080753)*sqrt(x[1])
        arg[3,0,0,0]=(-0.785416067695)*sqrt(x[0])
        arg[3,0,0,1]=(0.344457091999)*sqrt(x[1])
        arg[3,0,1,0]=(-0.845835855973)*sqrt(x[0])
        arg[3,0,1,1]=(0.365474280353)*sqrt(x[1])
        arg[3,0,2,0]=(-0.250535373488)*sqrt(x[0])
        arg[3,0,2,1]=(-0.888004656815)*sqrt(x[1])
        arg[3,1,0,0]=(-0.0652508248717)*sqrt(x[0])
        arg[3,1,0,1]=(0.402069695186)*sqrt(x[1])
        arg[3,1,1,0]=(-0.229714763188)*sqrt(x[0])
        arg[3,1,1,1]=(-0.866938136156)*sqrt(x[1])
        arg[3,1,2,0]=(-0.638603745536)*sqrt(x[0])
        arg[3,1,2,1]=(-0.997108734664)*sqrt(x[1])
        arg[3,2,0,0]=(-0.182784593825)*sqrt(x[0])
        arg[3,2,0,1]=(0.426951207538)*sqrt(x[1])
        arg[3,2,1,0]=(-0.958149736654)*sqrt(x[0])
        arg[3,2,1,1]=(-0.58458370587)*sqrt(x[1])
        arg[3,2,2,0]=(0.410243417149)*sqrt(x[0])
        arg[3,2,2,1]=(0.764474507608)*sqrt(x[1])
        arg[3,3,0,0]=(0.235623988334)*sqrt(x[0])
        arg[3,3,0,1]=(-0.105137401903)*sqrt(x[1])
        arg[3,3,1,0]=(-0.957743319072)*sqrt(x[0])
        arg[3,3,1,1]=(0.285680328399)*sqrt(x[1])
        arg[3,3,2,0]=(-0.525269717395)*sqrt(x[0])
        arg[3,3,2,1]=(0.655974370871)*sqrt(x[1])
        arg[3,4,0,0]=(0.926779313365)*sqrt(x[0])
        arg[3,4,0,1]=(-0.111556488031)*sqrt(x[1])
        arg[3,4,1,0]=(-0.672785588938)*sqrt(x[0])
        arg[3,4,1,1]=(-0.687907843899)*sqrt(x[1])
        arg[3,4,2,0]=(0.503984478085)*sqrt(x[0])
        arg[3,4,2,1]=(0.213194091516)*sqrt(x[1])
        ref=sqrt((6.92178561091)*3./2.+(4.96610502195)/0.5)

      else:
        arg=Data(0,(4, 5, 3, 3),w)
        arg[0,0,0,0]=(-0.641108959812)*sqrt(x[0])
        arg[0,0,0,1]=(0.395837684631)*sqrt(x[1])
        arg[0,0,0,2]=(0.36258240616)*sqrt(x[2])
        arg[0,0,1,0]=(-0.0543293722691)*sqrt(x[0])
        arg[0,0,1,1]=(0.850820297369)*sqrt(x[1])
        arg[0,0,1,2]=(0.0910730209593)*sqrt(x[2])
        arg[0,0,2,0]=(-0.831108730829)*sqrt(x[0])
        arg[0,0,2,1]=(-0.653018722047)*sqrt(x[1])
        arg[0,0,2,2]=(0.29052921548)*sqrt(x[2])
        arg[0,1,0,0]=(-0.115846569271)*sqrt(x[0])
        arg[0,1,0,1]=(-0.828080308748)*sqrt(x[1])
        arg[0,1,0,2]=(0.715255255273)*sqrt(x[2])
        arg[0,1,1,0]=(-0.0106077934044)*sqrt(x[0])
        arg[0,1,1,1]=(0.358323295554)*sqrt(x[1])
        arg[0,1,1,2]=(-0.916496266702)*sqrt(x[2])
        arg[0,1,2,0]=(0.740969432327)*sqrt(x[0])
        arg[0,1,2,1]=(0.0631507192754)*sqrt(x[1])
        arg[0,1,2,2]=(0.440009320758)*sqrt(x[2])
        arg[0,2,0,0]=(0.689964376385)*sqrt(x[0])
        arg[0,2,0,1]=(0.58199412545)*sqrt(x[1])
        arg[0,2,0,2]=(0.649058556936)*sqrt(x[2])
        arg[0,2,1,0]=(0.811660544475)*sqrt(x[0])
        arg[0,2,1,1]=(-0.121902248927)*sqrt(x[1])
        arg[0,2,1,2]=(-0.0750232179247)*sqrt(x[2])
        arg[0,2,2,0]=(0.779166450882)*sqrt(x[0])
        arg[0,2,2,1]=(0.298560101475)*sqrt(x[1])
        arg[0,2,2,2]=(-0.776198356206)*sqrt(x[2])
        arg[0,3,0,0]=(0.79804128408)*sqrt(x[0])
        arg[0,3,0,1]=(-0.378223308322)*sqrt(x[1])
        arg[0,3,0,2]=(0.191534109359)*sqrt(x[2])
        arg[0,3,1,0]=(-0.59973117885)*sqrt(x[0])
        arg[0,3,1,1]=(0.149042766249)*sqrt(x[1])
        arg[0,3,1,2]=(-0.357473233062)*sqrt(x[2])
        arg[0,3,2,0]=(-0.343758339283)*sqrt(x[0])
        arg[0,3,2,1]=(-0.140764743713)*sqrt(x[1])
        arg[0,3,2,2]=(-0.506377878901)*sqrt(x[2])
        arg[0,4,0,0]=(-0.967765607304)*sqrt(x[0])
        arg[0,4,0,1]=(-0.121373931498)*sqrt(x[1])
        arg[0,4,0,2]=(-0.300794608584)*sqrt(x[2])
        arg[0,4,1,0]=(-0.604537150389)*sqrt(x[0])
        arg[0,4,1,1]=(-0.290834102056)*sqrt(x[1])
        arg[0,4,1,2]=(0.509331591828)*sqrt(x[2])
        arg[0,4,2,0]=(0.643653985564)*sqrt(x[0])
        arg[0,4,2,1]=(-0.599746831159)*sqrt(x[1])
        arg[0,4,2,2]=(0.843193710828)*sqrt(x[2])
        arg[1,0,0,0]=(-0.598249432719)*sqrt(x[0])
        arg[1,0,0,1]=(0.840724468488)*sqrt(x[1])
        arg[1,0,0,2]=(0.558893493847)*sqrt(x[2])
        arg[1,0,1,0]=(0.843245438555)*sqrt(x[0])
        arg[1,0,1,1]=(0.280919545563)*sqrt(x[1])
        arg[1,0,1,2]=(-0.0549504830615)*sqrt(x[2])
        arg[1,0,2,0]=(0.295812998973)*sqrt(x[0])
        arg[1,0,2,1]=(0.513221249344)*sqrt(x[1])
        arg[1,0,2,2]=(-0.432253600492)*sqrt(x[2])
        arg[1,1,0,0]=(0.300696333449)*sqrt(x[0])
        arg[1,1,0,1]=(0.964202717779)*sqrt(x[1])
        arg[1,1,0,2]=(0.131730374204)*sqrt(x[2])
        arg[1,1,1,0]=(0.209135189207)*sqrt(x[0])
        arg[1,1,1,1]=(0.301389564979)*sqrt(x[1])
        arg[1,1,1,2]=(0.378481289816)*sqrt(x[2])
        arg[1,1,2,0]=(0.407413524533)*sqrt(x[0])
        arg[1,1,2,1]=(0.184345610049)*sqrt(x[1])
        arg[1,1,2,2]=(-0.884401323556)*sqrt(x[2])
        arg[1,2,0,0]=(0.886883417187)*sqrt(x[0])
        arg[1,2,0,1]=(-0.0524504232352)*sqrt(x[1])
        arg[1,2,0,2]=(0.049116549681)*sqrt(x[2])
        arg[1,2,1,0]=(-0.41240815391)*sqrt(x[0])
        arg[1,2,1,1]=(-0.523033558623)*sqrt(x[1])
        arg[1,2,1,2]=(-0.920238404199)*sqrt(x[2])
        arg[1,2,2,0]=(-0.635169424461)*sqrt(x[0])
        arg[1,2,2,1]=(0.773163888164)*sqrt(x[1])
        arg[1,2,2,2]=(0.554400056145)*sqrt(x[2])
        arg[1,3,0,0]=(0.573361031392)*sqrt(x[0])
        arg[1,3,0,1]=(-0.107749750058)*sqrt(x[1])
        arg[1,3,0,2]=(-0.971799077244)*sqrt(x[2])
        arg[1,3,1,0]=(-0.103579714499)*sqrt(x[0])
        arg[1,3,1,1]=(0.93329275867)*sqrt(x[1])
        arg[1,3,1,2]=(0.994409555402)*sqrt(x[2])
        arg[1,3,2,0]=(0.571961012279)*sqrt(x[0])
        arg[1,3,2,1]=(-0.54961086377)*sqrt(x[1])
        arg[1,3,2,2]=(0.563815465418)*sqrt(x[2])
        arg[1,4,0,0]=(-0.820638948871)*sqrt(x[0])
        arg[1,4,0,1]=(0.407750087109)*sqrt(x[1])
        arg[1,4,0,2]=(0.0911636159786)*sqrt(x[2])
        arg[1,4,1,0]=(0.371133349901)*sqrt(x[0])
        arg[1,4,1,1]=(0.688167208194)*sqrt(x[1])
        arg[1,4,1,2]=(0.188789217725)*sqrt(x[2])
        arg[1,4,2,0]=(0.961439941063)*sqrt(x[0])
        arg[1,4,2,1]=(0.524326385264)*sqrt(x[1])
        arg[1,4,2,2]=(-0.0866064394717)*sqrt(x[2])
        arg[2,0,0,0]=(0.392756517944)*sqrt(x[0])
        arg[2,0,0,1]=(0.713723092455)*sqrt(x[1])
        arg[2,0,0,2]=(0.68441988583)*sqrt(x[2])
        arg[2,0,1,0]=(-0.471505369012)*sqrt(x[0])
        arg[2,0,1,1]=(0.152712812211)*sqrt(x[1])
        arg[2,0,1,2]=(-0.579106568625)*sqrt(x[2])
        arg[2,0,2,0]=(0.637759469489)*sqrt(x[0])
        arg[2,0,2,1]=(-0.986405270597)*sqrt(x[1])
        arg[2,0,2,2]=(0.498109007475)*sqrt(x[2])
        arg[2,1,0,0]=(0.870185247694)*sqrt(x[0])
        arg[2,1,0,1]=(0.738095675991)*sqrt(x[1])
        arg[2,1,0,2]=(-0.613836737976)*sqrt(x[2])
        arg[2,1,1,0]=(-0.507009374688)*sqrt(x[0])
        arg[2,1,1,1]=(0.00752595652646)*sqrt(x[1])
        arg[2,1,1,2]=(-0.389118104751)*sqrt(x[2])
        arg[2,1,2,0]=(-0.466733433552)*sqrt(x[0])
        arg[2,1,2,1]=(-0.356829687828)*sqrt(x[1])
        arg[2,1,2,2]=(0.691337110432)*sqrt(x[2])
        arg[2,2,0,0]=(-0.684116912917)*sqrt(x[0])
        arg[2,2,0,1]=(0.705774798227)*sqrt(x[1])
        arg[2,2,0,2]=(0.522279733527)*sqrt(x[2])
        arg[2,2,1,0]=(0.390315182318)*sqrt(x[0])
        arg[2,2,1,1]=(-0.450165044241)*sqrt(x[1])
        arg[2,2,1,2]=(0.436062806791)*sqrt(x[2])
        arg[2,2,2,0]=(0.973600590724)*sqrt(x[0])
        arg[2,2,2,1]=(-0.89116205465)*sqrt(x[1])
        arg[2,2,2,2]=(-0.575262591791)*sqrt(x[2])
        arg[2,3,0,0]=(0.842833949385)*sqrt(x[0])
        arg[2,3,0,1]=(-0.69939817904)*sqrt(x[1])
        arg[2,3,0,2]=(0.343679860971)*sqrt(x[2])
        arg[2,3,1,0]=(-0.387030334358)*sqrt(x[0])
        arg[2,3,1,1]=(0.901637622406)*sqrt(x[1])
        arg[2,3,1,2]=(0.054184099786)*sqrt(x[2])
        arg[2,3,2,0]=(0.652712575782)*sqrt(x[0])
        arg[2,3,2,1]=(0.286474075274)*sqrt(x[1])
        arg[2,3,2,2]=(-0.745166160145)*sqrt(x[2])
        arg[2,4,0,0]=(-0.104591304669)*sqrt(x[0])
        arg[2,4,0,1]=(0.417496371683)*sqrt(x[1])
        arg[2,4,0,2]=(0.0579341684769)*sqrt(x[2])
        arg[2,4,1,0]=(0.24486914915)*sqrt(x[0])
        arg[2,4,1,1]=(-0.166211416717)*sqrt(x[1])
        arg[2,4,1,2]=(0.651515760357)*sqrt(x[2])
        arg[2,4,2,0]=(0.934693067557)*sqrt(x[0])
        arg[2,4,2,1]=(0.628486602755)*sqrt(x[1])
        arg[2,4,2,2]=(0.855313741494)*sqrt(x[2])
        arg[3,0,0,0]=(0.2241978058)*sqrt(x[0])
        arg[3,0,0,1]=(0.389284487657)*sqrt(x[1])
        arg[3,0,0,2]=(0.711521372962)*sqrt(x[2])
        arg[3,0,1,0]=(0.653184309521)*sqrt(x[0])
        arg[3,0,1,1]=(0.270052840275)*sqrt(x[1])
        arg[3,0,1,2]=(0.388881294549)*sqrt(x[2])
        arg[3,0,2,0]=(0.973583452692)*sqrt(x[0])
        arg[3,0,2,1]=(0.508175099792)*sqrt(x[1])
        arg[3,0,2,2]=(-0.989999307019)*sqrt(x[2])
        arg[3,1,0,0]=(0.935765996464)*sqrt(x[0])
        arg[3,1,0,1]=(-0.186350920716)*sqrt(x[1])
        arg[3,1,0,2]=(-0.809013452979)*sqrt(x[2])
        arg[3,1,1,0]=(0.467966757364)*sqrt(x[0])
        arg[3,1,1,1]=(-0.347420416508)*sqrt(x[1])
        arg[3,1,1,2]=(0.686386725318)*sqrt(x[2])
        arg[3,1,2,0]=(-0.8117641465)*sqrt(x[0])
        arg[3,1,2,1]=(-0.357390874294)*sqrt(x[1])
        arg[3,1,2,2]=(-0.0202262660381)*sqrt(x[2])
        arg[3,2,0,0]=(-0.181594076044)*sqrt(x[0])
        arg[3,2,0,1]=(0.0655758078072)*sqrt(x[1])
        arg[3,2,0,2]=(0.926686460908)*sqrt(x[2])
        arg[3,2,1,0]=(-0.700426183586)*sqrt(x[0])
        arg[3,2,1,1]=(0.600692261192)*sqrt(x[1])
        arg[3,2,1,2]=(0.757146807006)*sqrt(x[2])
        arg[3,2,2,0]=(0.588097603826)*sqrt(x[0])
        arg[3,2,2,1]=(-0.748732775295)*sqrt(x[1])
        arg[3,2,2,2]=(0.628960378488)*sqrt(x[2])
        arg[3,3,0,0]=(-0.0612899752684)*sqrt(x[0])
        arg[3,3,0,1]=(0.567198903535)*sqrt(x[1])
        arg[3,3,0,2]=(0.743996205698)*sqrt(x[2])
        arg[3,3,1,0]=(0.333763157477)*sqrt(x[0])
        arg[3,3,1,1]=(0.795773692731)*sqrt(x[1])
        arg[3,3,1,2]=(-0.676301506816)*sqrt(x[2])
        arg[3,3,2,0]=(-0.255504087739)*sqrt(x[0])
        arg[3,3,2,1]=(-0.556263970561)*sqrt(x[1])
        arg[3,3,2,2]=(0.721245074469)*sqrt(x[2])
        arg[3,4,0,0]=(0.384551582302)*sqrt(x[0])
        arg[3,4,0,1]=(0.849519144853)*sqrt(x[1])
        arg[3,4,0,2]=(0.101881023262)*sqrt(x[2])
        arg[3,4,1,0]=(-0.685891926653)*sqrt(x[0])
        arg[3,4,1,1]=(0.390887672448)*sqrt(x[1])
        arg[3,4,1,2]=(0.0911617386398)*sqrt(x[2])
        arg[3,4,2,0]=(-0.889618312513)*sqrt(x[0])
        arg[3,4,2,1]=(0.130310876981)*sqrt(x[1])
        arg[3,4,2,2]=(0.958562278204)*sqrt(x[2])
        ref=sqrt((12.9075635889)*3./2.+(5.72789470359)/0.5)

      res=L2(arg)
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertAlmostEqual(res,ref,int(-log10(self.RES_TOL)),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_L2_onReducedFunctionOnContactOne_fromSymbol_rank0(self):
      """
      tests L2-norm of Symbol on the ReducedFunctionOnContactOne

      assumptions: self.domain supports integration on ReducedFunctionOnContactOne
      """
      dim=self.domain.getDim()
      w=ReducedFunctionOnContactOne(self.domain)
      x=w.getX()
      if dim==2:
        arg=Symbol(shape=(),dim=dim)
        s=Data(0,(),w)
        s=(-0.849531528726)*sqrt(x[0])
        ref=sqrt((0.0)*3./2.+(0.180425954575)/0.5)

      else:
        arg=Symbol(shape=(),dim=dim)
        s=Data(0,(),w)
        s=(-0.708706581422)*sqrt(x[0])
        ref=sqrt((0.0)*3./2.+(0.125566254638)/0.5)

      res=L2(arg)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertAlmostEqual(sub,ref,int(-log10(self.RES_TOL)),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_L2_onReducedFunctionOnContactOne_fromSymbol_rank1(self):
      """
      tests L2-norm of Symbol on the ReducedFunctionOnContactOne

      assumptions: self.domain supports integration on ReducedFunctionOnContactOne
      """
      dim=self.domain.getDim()
      w=ReducedFunctionOnContactOne(self.domain)
      x=w.getX()
      if dim==2:
        arg=Symbol(shape=(2,),dim=dim)
        s=Data(0,(2,),w)
        s[0]=(-0.852476696951)*sqrt(x[0])
        s[1]=(0.669311710064)*sqrt(x[1])
        ref=sqrt((0.149326055076)*3./2.+(0.181679129711)/0.5)

      else:
        arg=Symbol(shape=(3,),dim=dim)
        s=Data(0,(3,),w)
        s[0]=(0.0886648134124)*sqrt(x[0])
        s[1]=(-0.98107163405)*sqrt(x[1])
        s[2]=(0.763548755951)*sqrt(x[2])
        ref=sqrt((0.51516941795)*3./2.+(0.00196536228436)/0.5)

      res=L2(arg)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertAlmostEqual(sub,ref,int(-log10(self.RES_TOL)),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_L2_onReducedFunctionOnContactOne_fromSymbol_rank2(self):
      """
      tests L2-norm of Symbol on the ReducedFunctionOnContactOne

      assumptions: self.domain supports integration on ReducedFunctionOnContactOne
      """
      dim=self.domain.getDim()
      w=ReducedFunctionOnContactOne(self.domain)
      x=w.getX()
      if dim==2:
        arg=Symbol(shape=(4, 2),dim=dim)
        s=Data(0,(4, 2),w)
        s[0,0]=(0.534469076507)*sqrt(x[0])
        s[0,1]=(0.707717684613)*sqrt(x[1])
        s[1,0]=(0.697608696253)*sqrt(x[0])
        s[1,1]=(0.567450669898)*sqrt(x[1])
        s[2,0]=(-0.622381718509)*sqrt(x[0])
        s[2,1]=(0.546984463965)*sqrt(x[1])
        s[3,0]=(-0.0825527152886)*sqrt(x[0])
        s[3,1]=(-0.581446541532)*sqrt(x[1])
        ref=sqrt((0.486712222787)*3./2.+(0.291622260292)/0.5)

      else:
        arg=Symbol(shape=(4, 3),dim=dim)
        s=Data(0,(4, 3),w)
        s[0,0]=(-0.860491143615)*sqrt(x[0])
        s[0,1]=(0.140822783856)*sqrt(x[1])
        s[0,2]=(-0.850233173189)*sqrt(x[2])
        s[1,0]=(-0.0526405018976)*sqrt(x[0])
        s[1,1]=(-0.620873428306)*sqrt(x[1])
        s[1,2]=(-0.68323428604)*sqrt(x[2])
        s[2,0]=(0.995631462645)*sqrt(x[0])
        s[2,1]=(0.624478975925)*sqrt(x[1])
        s[2,2]=(0.241456353598)*sqrt(x[2])
        s[3,0]=(0.167743672846)*sqrt(x[0])
        s[3,1]=(0.340565516735)*sqrt(x[1])
        s[3,2]=(0.499048848748)*sqrt(x[2])
        ref=sqrt((0.802776731843)*3./2.+(0.440658994967)/0.5)

      res=L2(arg)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertAlmostEqual(sub,ref,int(-log10(self.RES_TOL)),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_L2_onReducedFunctionOnContactOne_fromSymbol_rank3(self):
      """
      tests L2-norm of Symbol on the ReducedFunctionOnContactOne

      assumptions: self.domain supports integration on ReducedFunctionOnContactOne
      """
      dim=self.domain.getDim()
      w=ReducedFunctionOnContactOne(self.domain)
      x=w.getX()
      if dim==2:
        arg=Symbol(shape=(6, 2, 2),dim=dim)
        s=Data(0,(6, 2, 2),w)
        s[0,0,0]=(0.53006006661)*sqrt(x[0])
        s[0,0,1]=(0.855136875508)*sqrt(x[1])
        s[0,1,0]=(0.965271929044)*sqrt(x[0])
        s[0,1,1]=(0.00691131453267)*sqrt(x[1])
        s[1,0,0]=(-0.244825630321)*sqrt(x[0])
        s[1,0,1]=(-0.840591747582)*sqrt(x[1])
        s[1,1,0]=(-0.0906837575187)*sqrt(x[0])
        s[1,1,1]=(0.459130145496)*sqrt(x[1])
        s[2,0,0]=(0.16143268528)*sqrt(x[0])
        s[2,0,1]=(0.719843211037)*sqrt(x[1])
        s[2,1,0]=(0.233952330698)*sqrt(x[0])
        s[2,1,1]=(0.212086642323)*sqrt(x[1])
        s[3,0,0]=(0.323764554006)*sqrt(x[0])
        s[3,0,1]=(0.693684508228)*sqrt(x[1])
        s[3,1,0]=(0.48296993334)*sqrt(x[0])
        s[3,1,1]=(-0.13807549797)*sqrt(x[1])
        s[4,0,0]=(-0.688641980918)*sqrt(x[0])
        s[4,0,1]=(-0.98785692721)*sqrt(x[1])
        s[4,1,0]=(0.987040832755)*sqrt(x[0])
        s[4,1,1]=(-0.734764958687)*sqrt(x[1])
        s[5,0,0]=(0.244402182319)*sqrt(x[0])
        s[5,0,1]=(0.974686625391)*sqrt(x[1])
        s[5,1,0]=(-0.277257927506)*sqrt(x[0])
        s[5,1,1]=(0.0773106723551)*sqrt(x[1])
        ref=sqrt((1.72795055403)*3./2.+(0.821209030177)/0.5)

      else:
        arg=Symbol(shape=(6, 2, 3),dim=dim)
        s=Data(0,(6, 2, 3),w)
        s[0,0,0]=(-0.61386584344)*sqrt(x[0])
        s[0,0,1]=(-0.032040523373)*sqrt(x[1])
        s[0,0,2]=(-0.723104197392)*sqrt(x[2])
        s[0,1,0]=(0.665794765401)*sqrt(x[0])
        s[0,1,1]=(-0.842083120109)*sqrt(x[1])
        s[0,1,2]=(-0.642000380947)*sqrt(x[2])
        s[1,0,0]=(-0.140120073896)*sqrt(x[0])
        s[1,0,1]=(0.852883705123)*sqrt(x[1])
        s[1,0,2]=(0.538615525775)*sqrt(x[2])
        s[1,1,0]=(-0.268324107667)*sqrt(x[0])
        s[1,1,1]=(0.723423091154)*sqrt(x[1])
        s[1,1,2]=(-0.0210208139288)*sqrt(x[2])
        s[2,0,0]=(-0.13072934339)*sqrt(x[0])
        s[2,0,1]=(-0.287830796935)*sqrt(x[1])
        s[2,0,2]=(-0.374274267746)*sqrt(x[2])
        s[2,1,0]=(0.852351589285)*sqrt(x[0])
        s[2,1,1]=(-0.16416141595)*sqrt(x[1])
        s[2,1,2]=(0.218655720718)*sqrt(x[2])
        s[3,0,0]=(0.0736657503857)*sqrt(x[0])
        s[3,0,1]=(0.484109452084)*sqrt(x[1])
        s[3,0,2]=(-0.68960733775)*sqrt(x[2])
        s[3,1,0]=(0.866791078374)*sqrt(x[0])
        s[3,1,1]=(-0.771121017091)*sqrt(x[1])
        s[3,1,2]=(-0.643197172218)*sqrt(x[2])
        s[4,0,0]=(0.00890571083899)*sqrt(x[0])
        s[4,0,1]=(-0.762935399812)*sqrt(x[1])
        s[4,0,2]=(-0.240832580447)*sqrt(x[2])
        s[4,1,0]=(-0.657527623928)*sqrt(x[0])
        s[4,1,1]=(0.558483255401)*sqrt(x[1])
        s[4,1,2]=(-0.439397930536)*sqrt(x[2])
        s[5,0,0]=(0.530029745037)*sqrt(x[0])
        s[5,0,1]=(0.138779547908)*sqrt(x[1])
        s[5,0,2]=(-0.0456719603355)*sqrt(x[2])
        s[5,1,0]=(-0.751937380557)*sqrt(x[0])
        s[5,1,1]=(-0.37055796529)*sqrt(x[1])
        s[5,1,2]=(-0.285251484378)*sqrt(x[2])
        ref=sqrt((2.19582819839)*3./2.+(0.922713864342)/0.5)

      res=L2(arg)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertAlmostEqual(sub,ref,int(-log10(self.RES_TOL)),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_L2_onReducedFunctionOnContactOne_fromSymbol_rank4(self):
      """
      tests L2-norm of Symbol on the ReducedFunctionOnContactOne

      assumptions: self.domain supports integration on ReducedFunctionOnContactOne
      """
      dim=self.domain.getDim()
      w=ReducedFunctionOnContactOne(self.domain)
      x=w.getX()
      if dim==2:
        arg=Symbol(shape=(4, 5, 3, 2),dim=dim)
        s=Data(0,(4, 5, 3, 2),w)
        s[0,0,0,0]=(0.396322609451)*sqrt(x[0])
        s[0,0,0,1]=(0.982533894141)*sqrt(x[1])
        s[0,0,1,0]=(-0.129145024315)*sqrt(x[0])
        s[0,0,1,1]=(-0.27837328441)*sqrt(x[1])
        s[0,0,2,0]=(-0.731814618355)*sqrt(x[0])
        s[0,0,2,1]=(0.989632262348)*sqrt(x[1])
        s[0,1,0,0]=(-0.501434234651)*sqrt(x[0])
        s[0,1,0,1]=(0.134247720647)*sqrt(x[1])
        s[0,1,1,0]=(0.72638709986)*sqrt(x[0])
        s[0,1,1,1]=(-0.66389079496)*sqrt(x[1])
        s[0,1,2,0]=(-0.285431055287)*sqrt(x[0])
        s[0,1,2,1]=(0.140502987885)*sqrt(x[1])
        s[0,2,0,0]=(0.699721872914)*sqrt(x[0])
        s[0,2,0,1]=(-0.232830090422)*sqrt(x[1])
        s[0,2,1,0]=(0.322184745748)*sqrt(x[0])
        s[0,2,1,1]=(0.725202642274)*sqrt(x[1])
        s[0,2,2,0]=(0.398485732696)*sqrt(x[0])
        s[0,2,2,1]=(0.230163541848)*sqrt(x[1])
        s[0,3,0,0]=(-0.680544045823)*sqrt(x[0])
        s[0,3,0,1]=(0.108130166525)*sqrt(x[1])
        s[0,3,1,0]=(0.5077516367)*sqrt(x[0])
        s[0,3,1,1]=(0.88975554254)*sqrt(x[1])
        s[0,3,2,0]=(0.775170169413)*sqrt(x[0])
        s[0,3,2,1]=(-0.275812588437)*sqrt(x[1])
        s[0,4,0,0]=(0.348603150333)*sqrt(x[0])
        s[0,4,0,1]=(-0.176482742037)*sqrt(x[1])
        s[0,4,1,0]=(-0.927870151856)*sqrt(x[0])
        s[0,4,1,1]=(-0.661067126232)*sqrt(x[1])
        s[0,4,2,0]=(-0.208628316679)*sqrt(x[0])
        s[0,4,2,1]=(0.383637018163)*sqrt(x[1])
        s[1,0,0,0]=(0.88866687126)*sqrt(x[0])
        s[1,0,0,1]=(0.260170060159)*sqrt(x[1])
        s[1,0,1,0]=(0.126798500798)*sqrt(x[0])
        s[1,0,1,1]=(0.264342178219)*sqrt(x[1])
        s[1,0,2,0]=(0.662206500808)*sqrt(x[0])
        s[1,0,2,1]=(-0.347569034957)*sqrt(x[1])
        s[1,1,0,0]=(0.0960100811475)*sqrt(x[0])
        s[1,1,0,1]=(-0.813364260291)*sqrt(x[1])
        s[1,1,1,0]=(-0.128545578333)*sqrt(x[0])
        s[1,1,1,1]=(0.557352804436)*sqrt(x[1])
        s[1,1,2,0]=(0.177270533211)*sqrt(x[0])
        s[1,1,2,1]=(-0.879697427458)*sqrt(x[1])
        s[1,2,0,0]=(0.260811041962)*sqrt(x[0])
        s[1,2,0,1]=(-0.0448529305202)*sqrt(x[1])
        s[1,2,1,0]=(0.57598635298)*sqrt(x[0])
        s[1,2,1,1]=(-0.899140376189)*sqrt(x[1])
        s[1,2,2,0]=(-0.523456336484)*sqrt(x[0])
        s[1,2,2,1]=(0.759703092295)*sqrt(x[1])
        s[1,3,0,0]=(-0.262236560882)*sqrt(x[0])
        s[1,3,0,1]=(-0.0829260777719)*sqrt(x[1])
        s[1,3,1,0]=(0.327366286293)*sqrt(x[0])
        s[1,3,1,1]=(-0.473163087927)*sqrt(x[1])
        s[1,3,2,0]=(0.160705506648)*sqrt(x[0])
        s[1,3,2,1]=(-0.347036912731)*sqrt(x[1])
        s[1,4,0,0]=(0.343187252577)*sqrt(x[0])
        s[1,4,0,1]=(-0.644467832838)*sqrt(x[1])
        s[1,4,1,0]=(-0.95236021268)*sqrt(x[0])
        s[1,4,1,1]=(0.475087073317)*sqrt(x[1])
        s[1,4,2,0]=(0.808985496682)*sqrt(x[0])
        s[1,4,2,1]=(-0.421357379281)*sqrt(x[1])
        s[2,0,0,0]=(0.439969305501)*sqrt(x[0])
        s[2,0,0,1]=(0.67689403622)*sqrt(x[1])
        s[2,0,1,0]=(-0.754345274443)*sqrt(x[0])
        s[2,0,1,1]=(0.881153579305)*sqrt(x[1])
        s[2,0,2,0]=(0.825454096565)*sqrt(x[0])
        s[2,0,2,1]=(-0.320622136485)*sqrt(x[1])
        s[2,1,0,0]=(0.247206810015)*sqrt(x[0])
        s[2,1,0,1]=(0.745173031392)*sqrt(x[1])
        s[2,1,1,0]=(0.683694038926)*sqrt(x[0])
        s[2,1,1,1]=(0.751191436507)*sqrt(x[1])
        s[2,1,2,0]=(-0.535657258854)*sqrt(x[0])
        s[2,1,2,1]=(0.832509218552)*sqrt(x[1])
        s[2,2,0,0]=(0.397072754881)*sqrt(x[0])
        s[2,2,0,1]=(-0.879089743252)*sqrt(x[1])
        s[2,2,1,0]=(0.601245034596)*sqrt(x[0])
        s[2,2,1,1]=(0.265236139375)*sqrt(x[1])
        s[2,2,2,0]=(-0.0253945476718)*sqrt(x[0])
        s[2,2,2,1]=(-0.976895844239)*sqrt(x[1])
        s[2,3,0,0]=(-0.356083748879)*sqrt(x[0])
        s[2,3,0,1]=(0.826339109837)*sqrt(x[1])
        s[2,3,1,0]=(-0.594681379568)*sqrt(x[0])
        s[2,3,1,1]=(-0.58277681276)*sqrt(x[1])
        s[2,3,2,0]=(-0.408387409616)*sqrt(x[0])
        s[2,3,2,1]=(0.602785352992)*sqrt(x[1])
        s[2,4,0,0]=(-0.891295605242)*sqrt(x[0])
        s[2,4,0,1]=(-0.750844606395)*sqrt(x[1])
        s[2,4,1,0]=(-0.29493371455)*sqrt(x[0])
        s[2,4,1,1]=(-0.0269046032275)*sqrt(x[1])
        s[2,4,2,0]=(0.49367579924)*sqrt(x[0])
        s[2,4,2,1]=(-0.671892240661)*sqrt(x[1])
        s[3,0,0,0]=(0.526420934634)*sqrt(x[0])
        s[3,0,0,1]=(0.43040092109)*sqrt(x[1])
        s[3,0,1,0]=(-0.135568748195)*sqrt(x[0])
        s[3,0,1,1]=(0.146386072934)*sqrt(x[1])
        s[3,0,2,0]=(-0.609373566828)*sqrt(x[0])
        s[3,0,2,1]=(0.763194941657)*sqrt(x[1])
        s[3,1,0,0]=(0.511063874735)*sqrt(x[0])
        s[3,1,0,1]=(0.894580623201)*sqrt(x[1])
        s[3,1,1,0]=(-0.372632207973)*sqrt(x[0])
        s[3,1,1,1]=(-0.819194915438)*sqrt(x[1])
        s[3,1,2,0]=(0.258313934708)*sqrt(x[0])
        s[3,1,2,1]=(-0.708534091752)*sqrt(x[1])
        s[3,2,0,0]=(0.439636692996)*sqrt(x[0])
        s[3,2,0,1]=(-0.996699172676)*sqrt(x[1])
        s[3,2,1,0]=(0.24213697925)*sqrt(x[0])
        s[3,2,1,1]=(0.0487822729335)*sqrt(x[1])
        s[3,2,2,0]=(-0.747366711813)*sqrt(x[0])
        s[3,2,2,1]=(0.533579125235)*sqrt(x[1])
        s[3,3,0,0]=(0.590618210003)*sqrt(x[0])
        s[3,3,0,1]=(-0.378862695527)*sqrt(x[1])
        s[3,3,1,0]=(0.486532205417)*sqrt(x[0])
        s[3,3,1,1]=(-0.568241987046)*sqrt(x[1])
        s[3,3,2,0]=(-0.368983251697)*sqrt(x[0])
        s[3,3,2,1]=(0.978959945336)*sqrt(x[1])
        s[3,4,0,0]=(-0.944651128993)*sqrt(x[0])
        s[3,4,0,1]=(0.449642532401)*sqrt(x[1])
        s[3,4,1,0]=(-0.171120868333)*sqrt(x[0])
        s[3,4,1,1]=(-0.985738801506)*sqrt(x[1])
        s[3,4,2,0]=(0.39325635403)*sqrt(x[0])
        s[3,4,2,1]=(-0.355332366915)*sqrt(x[1])
        ref=sqrt((7.76922176438)*3./2.+(4.20495359519)/0.5)

      else:
        arg=Symbol(shape=(4, 5, 3, 3),dim=dim)
        s=Data(0,(4, 5, 3, 3),w)
        s[0,0,0,0]=(0.296145515325)*sqrt(x[0])
        s[0,0,0,1]=(-0.112284888661)*sqrt(x[1])
        s[0,0,0,2]=(0.0746491379222)*sqrt(x[2])
        s[0,0,1,0]=(0.14756535943)*sqrt(x[0])
        s[0,0,1,1]=(-0.653188513529)*sqrt(x[1])
        s[0,0,1,2]=(-0.881965485447)*sqrt(x[2])
        s[0,0,2,0]=(-0.645436136716)*sqrt(x[0])
        s[0,0,2,1]=(-0.996242179756)*sqrt(x[1])
        s[0,0,2,2]=(-0.613744842724)*sqrt(x[2])
        s[0,1,0,0]=(0.543040571766)*sqrt(x[0])
        s[0,1,0,1]=(0.172439667925)*sqrt(x[1])
        s[0,1,0,2]=(0.134262077825)*sqrt(x[2])
        s[0,1,1,0]=(0.206973085316)*sqrt(x[0])
        s[0,1,1,1]=(0.78448343454)*sqrt(x[1])
        s[0,1,1,2]=(-0.533409163778)*sqrt(x[2])
        s[0,1,2,0]=(0.430560905893)*sqrt(x[0])
        s[0,1,2,1]=(-0.821838520257)*sqrt(x[1])
        s[0,1,2,2]=(-0.814853740301)*sqrt(x[2])
        s[0,2,0,0]=(-0.763338805036)*sqrt(x[0])
        s[0,2,0,1]=(0.973800200861)*sqrt(x[1])
        s[0,2,0,2]=(-0.591166623249)*sqrt(x[2])
        s[0,2,1,0]=(-0.338452805899)*sqrt(x[0])
        s[0,2,1,1]=(-0.324236181738)*sqrt(x[1])
        s[0,2,1,2]=(0.738595180918)*sqrt(x[2])
        s[0,2,2,0]=(0.618927719575)*sqrt(x[0])
        s[0,2,2,1]=(0.0632651077831)*sqrt(x[1])
        s[0,2,2,2]=(0.415573120842)*sqrt(x[2])
        s[0,3,0,0]=(-0.786194037231)*sqrt(x[0])
        s[0,3,0,1]=(-0.0402223807021)*sqrt(x[1])
        s[0,3,0,2]=(0.344479980368)*sqrt(x[2])
        s[0,3,1,0]=(0.545021524016)*sqrt(x[0])
        s[0,3,1,1]=(0.0563273477935)*sqrt(x[1])
        s[0,3,1,2]=(-0.0659388897897)*sqrt(x[2])
        s[0,3,2,0]=(0.152424684107)*sqrt(x[0])
        s[0,3,2,1]=(-0.518122801248)*sqrt(x[1])
        s[0,3,2,2]=(0.985522487899)*sqrt(x[2])
        s[0,4,0,0]=(-0.830069953806)*sqrt(x[0])
        s[0,4,0,1]=(0.571063785633)*sqrt(x[1])
        s[0,4,0,2]=(0.904417376158)*sqrt(x[2])
        s[0,4,1,0]=(0.687144774983)*sqrt(x[0])
        s[0,4,1,1]=(-0.250243991657)*sqrt(x[1])
        s[0,4,1,2]=(0.211438343321)*sqrt(x[2])
        s[0,4,2,0]=(-0.887502165654)*sqrt(x[0])
        s[0,4,2,1]=(0.342464350162)*sqrt(x[1])
        s[0,4,2,2]=(-0.389354004393)*sqrt(x[2])
        s[1,0,0,0]=(-0.00381480047885)*sqrt(x[0])
        s[1,0,0,1]=(0.584208423663)*sqrt(x[1])
        s[1,0,0,2]=(-0.781219608112)*sqrt(x[2])
        s[1,0,1,0]=(-0.484192327306)*sqrt(x[0])
        s[1,0,1,1]=(-0.0693964278254)*sqrt(x[1])
        s[1,0,1,2]=(0.401920719606)*sqrt(x[2])
        s[1,0,2,0]=(-0.964645208799)*sqrt(x[0])
        s[1,0,2,1]=(-0.458496519567)*sqrt(x[1])
        s[1,0,2,2]=(-0.837611873973)*sqrt(x[2])
        s[1,1,0,0]=(0.837066643335)*sqrt(x[0])
        s[1,1,0,1]=(0.220732404668)*sqrt(x[1])
        s[1,1,0,2]=(-0.341146357851)*sqrt(x[2])
        s[1,1,1,0]=(-0.432882962388)*sqrt(x[0])
        s[1,1,1,1]=(0.924385436071)*sqrt(x[1])
        s[1,1,1,2]=(-0.465249254821)*sqrt(x[2])
        s[1,1,2,0]=(0.307318032655)*sqrt(x[0])
        s[1,1,2,1]=(0.914827475386)*sqrt(x[1])
        s[1,1,2,2]=(-0.830756418498)*sqrt(x[2])
        s[1,2,0,0]=(-0.406562274215)*sqrt(x[0])
        s[1,2,0,1]=(-0.717178070516)*sqrt(x[1])
        s[1,2,0,2]=(0.513401821021)*sqrt(x[2])
        s[1,2,1,0]=(0.0903563906255)*sqrt(x[0])
        s[1,2,1,1]=(0.0714233348867)*sqrt(x[1])
        s[1,2,1,2]=(-0.629397568181)*sqrt(x[2])
        s[1,2,2,0]=(0.164139578343)*sqrt(x[0])
        s[1,2,2,1]=(-0.59013912099)*sqrt(x[1])
        s[1,2,2,2]=(-0.406224661607)*sqrt(x[2])
        s[1,3,0,0]=(0.0778758168202)*sqrt(x[0])
        s[1,3,0,1]=(0.738003702753)*sqrt(x[1])
        s[1,3,0,2]=(0.881333264003)*sqrt(x[2])
        s[1,3,1,0]=(-0.175166747001)*sqrt(x[0])
        s[1,3,1,1]=(0.326269577833)*sqrt(x[1])
        s[1,3,1,2]=(-0.340110356196)*sqrt(x[2])
        s[1,3,2,0]=(-0.443150540915)*sqrt(x[0])
        s[1,3,2,1]=(0.116960071142)*sqrt(x[1])
        s[1,3,2,2]=(0.969893913777)*sqrt(x[2])
        s[1,4,0,0]=(0.487572431401)*sqrt(x[0])
        s[1,4,0,1]=(-0.915415755722)*sqrt(x[1])
        s[1,4,0,2]=(0.0590936356226)*sqrt(x[2])
        s[1,4,1,0]=(0.968491519638)*sqrt(x[0])
        s[1,4,1,1]=(0.393515954209)*sqrt(x[1])
        s[1,4,1,2]=(-0.95643405719)*sqrt(x[2])
        s[1,4,2,0]=(0.645368604523)*sqrt(x[0])
        s[1,4,2,1]=(-0.157884217538)*sqrt(x[1])
        s[1,4,2,2]=(0.0497276310466)*sqrt(x[2])
        s[2,0,0,0]=(0.79948016484)*sqrt(x[0])
        s[2,0,0,1]=(0.404876280848)*sqrt(x[1])
        s[2,0,0,2]=(-0.0600452523729)*sqrt(x[2])
        s[2,0,1,0]=(0.128681372314)*sqrt(x[0])
        s[2,0,1,1]=(0.398166908201)*sqrt(x[1])
        s[2,0,1,2]=(-0.890799046442)*sqrt(x[2])
        s[2,0,2,0]=(-0.43878626818)*sqrt(x[0])
        s[2,0,2,1]=(0.295788703551)*sqrt(x[1])
        s[2,0,2,2]=(0.380983964299)*sqrt(x[2])
        s[2,1,0,0]=(-0.426682002183)*sqrt(x[0])
        s[2,1,0,1]=(0.13781617784)*sqrt(x[1])
        s[2,1,0,2]=(0.964133330513)*sqrt(x[2])
        s[2,1,1,0]=(0.397029527608)*sqrt(x[0])
        s[2,1,1,1]=(0.985177398513)*sqrt(x[1])
        s[2,1,1,2]=(-0.827997515899)*sqrt(x[2])
        s[2,1,2,0]=(0.730645004164)*sqrt(x[0])
        s[2,1,2,1]=(-0.797420668382)*sqrt(x[1])
        s[2,1,2,2]=(-0.300290718797)*sqrt(x[2])
        s[2,2,0,0]=(-0.257811741709)*sqrt(x[0])
        s[2,2,0,1]=(0.108526509889)*sqrt(x[1])
        s[2,2,0,2]=(0.657942581233)*sqrt(x[2])
        s[2,2,1,0]=(0.486944254347)*sqrt(x[0])
        s[2,2,1,1]=(0.498753219399)*sqrt(x[1])
        s[2,2,1,2]=(-0.392553093833)*sqrt(x[2])
        s[2,2,2,0]=(0.570390526883)*sqrt(x[0])
        s[2,2,2,1]=(-0.655828186915)*sqrt(x[1])
        s[2,2,2,2]=(-0.886504590868)*sqrt(x[2])
        s[2,3,0,0]=(0.786841433677)*sqrt(x[0])
        s[2,3,0,1]=(0.510317771549)*sqrt(x[1])
        s[2,3,0,2]=(-0.0755114998263)*sqrt(x[2])
        s[2,3,1,0]=(0.823613342893)*sqrt(x[0])
        s[2,3,1,1]=(-0.494194753878)*sqrt(x[1])
        s[2,3,1,2]=(-0.659935864742)*sqrt(x[2])
        s[2,3,2,0]=(0.640809074487)*sqrt(x[0])
        s[2,3,2,1]=(0.304983735262)*sqrt(x[1])
        s[2,3,2,2]=(0.316983109882)*sqrt(x[2])
        s[2,4,0,0]=(0.726358084867)*sqrt(x[0])
        s[2,4,0,1]=(0.559356161307)*sqrt(x[1])
        s[2,4,0,2]=(-0.619050390356)*sqrt(x[2])
        s[2,4,1,0]=(0.201203183195)*sqrt(x[0])
        s[2,4,1,1]=(-0.882713341877)*sqrt(x[1])
        s[2,4,1,2]=(-0.480400204958)*sqrt(x[2])
        s[2,4,2,0]=(-0.140145093838)*sqrt(x[0])
        s[2,4,2,1]=(0.145271336227)*sqrt(x[1])
        s[2,4,2,2]=(-0.870632579447)*sqrt(x[2])
        s[3,0,0,0]=(0.691022073815)*sqrt(x[0])
        s[3,0,0,1]=(-0.901932486396)*sqrt(x[1])
        s[3,0,0,2]=(-0.358787504481)*sqrt(x[2])
        s[3,0,1,0]=(-0.79150290154)*sqrt(x[0])
        s[3,0,1,1]=(0.440401080151)*sqrt(x[1])
        s[3,0,1,2]=(-0.656596021663)*sqrt(x[2])
        s[3,0,2,0]=(0.373265557264)*sqrt(x[0])
        s[3,0,2,1]=(0.147981174911)*sqrt(x[1])
        s[3,0,2,2]=(-0.254845607807)*sqrt(x[2])
        s[3,1,0,0]=(0.199574501407)*sqrt(x[0])
        s[3,1,0,1]=(-0.676397866417)*sqrt(x[1])
        s[3,1,0,2]=(0.83161020476)*sqrt(x[2])
        s[3,1,1,0]=(-0.887259353592)*sqrt(x[0])
        s[3,1,1,1]=(-0.599855381787)*sqrt(x[1])
        s[3,1,1,2]=(0.440852418479)*sqrt(x[2])
        s[3,1,2,0]=(0.0363737283834)*sqrt(x[0])
        s[3,1,2,1]=(-0.242329100485)*sqrt(x[1])
        s[3,1,2,2]=(-0.57882645459)*sqrt(x[2])
        s[3,2,0,0]=(0.208985671572)*sqrt(x[0])
        s[3,2,0,1]=(0.753748554448)*sqrt(x[1])
        s[3,2,0,2]=(0.175858719301)*sqrt(x[2])
        s[3,2,1,0]=(0.821744269994)*sqrt(x[0])
        s[3,2,1,1]=(-0.590861142533)*sqrt(x[1])
        s[3,2,1,2]=(0.766179113979)*sqrt(x[2])
        s[3,2,2,0]=(-0.299111001599)*sqrt(x[0])
        s[3,2,2,1]=(-0.817223826668)*sqrt(x[1])
        s[3,2,2,2]=(-0.17369899949)*sqrt(x[2])
        s[3,3,0,0]=(-0.478253760517)*sqrt(x[0])
        s[3,3,0,1]=(-0.4618406658)*sqrt(x[1])
        s[3,3,0,2]=(-0.434688925412)*sqrt(x[2])
        s[3,3,1,0]=(0.13832631548)*sqrt(x[0])
        s[3,3,1,1]=(0.407639738669)*sqrt(x[1])
        s[3,3,1,2]=(-0.870829708621)*sqrt(x[2])
        s[3,3,2,0]=(0.564907773571)*sqrt(x[0])
        s[3,3,2,1]=(-0.113149301306)*sqrt(x[1])
        s[3,3,2,2]=(0.972631828884)*sqrt(x[2])
        s[3,4,0,0]=(0.505964924785)*sqrt(x[0])
        s[3,4,0,1]=(-0.567647029204)*sqrt(x[1])
        s[3,4,0,2]=(-0.535422935572)*sqrt(x[2])
        s[3,4,1,0]=(-0.832260433763)*sqrt(x[0])
        s[3,4,1,1]=(0.52551900128)*sqrt(x[1])
        s[3,4,1,2]=(0.0171198096245)*sqrt(x[2])
        s[3,4,2,0]=(-0.675829280696)*sqrt(x[0])
        s[3,4,2,1]=(0.297095606749)*sqrt(x[1])
        s[3,4,2,2]=(0.99944710061)*sqrt(x[2])
        ref=sqrt((13.8090623863)*3./2.+(4.67224475394)/0.5)

      res=L2(arg)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertAlmostEqual(sub,ref,int(-log10(self.RES_TOL)),"wrong result")

   def test_integrate_onReducedFunctionOnContactZero_fromConstant_Solution_rank0(self):
      """
      test integral of constant rank 0 Data on ReducedFunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactZero(self.domain)
      w_arg = Solution(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = 2.525086e-01
      arg = Data(val,(),w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactZero_fromConstant_Solution_rank1(self):
      """
      test integral of constant rank 1 Data on ReducedFunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactZero(self.domain)
      w_arg = Solution(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([0.431528104184118, 0.7749914620249352, 0.6637636868277914] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactZero_fromConstant_Solution_rank2(self):
      """
      test integral of constant rank 2 Data on ReducedFunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactZero(self.domain)
      w_arg = Solution(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[0.42410556096312935, 0.6630332022794376], [0.27311751616354074, 0.3334891627211807], [0.30010259039114984, 0.7577542223044212], [0.973903218071565, 0.6511896321879975]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactZero_fromConstant_Solution_rank3(self):
      """
      test integral of constant rank 3 Data on ReducedFunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactZero(self.domain)
      w_arg = Solution(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[[0.3749949324549515, 0.6434797108022745, 0.09399855616084396, 0.0030048753646393367], [0.9266767776377741, 0.12293072299924857, 0.9935793575410728, 0.27647072963152397]], [[0.1407180827481208, 0.7584912735780073, 0.24765317214579674, 0.9894309402754766], [0.9981232901945974, 0.4424036553758578, 0.11269338591366262, 0.8349543959249837]]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 2, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactZero_fromConstant_Solution_rank4(self):
      """
      test integral of constant rank 4 Data on ReducedFunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactZero(self.domain)
      w_arg = Solution(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[[[0.5649510871215151, 0.7359580004984178, 0.8220630321854513, 0.5027233001589685], [0.7058357714534069, 0.1887736715798748, 0.42387373953228247, 0.43288881262060874], [0.7971791552636078, 0.0566242493127701, 0.5885921091448525, 0.08582107947036766]], [[0.008882972513422072, 0.5305028904431807, 0.41906080425670345, 0.7728487921330005], [0.2330765590868572, 0.9671105123434656, 0.4965201117134995, 0.49205749193731807], [0.4243650038646213, 0.9169814655474148, 0.625409621370019, 0.1412276632928665]]], [[[0.3716653363159309, 0.572939034042032, 0.6745436464329282, 0.8423412608453841], [0.7328887957841297, 0.6607548054090677, 0.6869132149143271, 0.031220043636201233], [0.6043634774305587, 0.9407433178818327, 0.9006643897174946, 0.9029699644215269]], [[0.8896320181677849, 0.1202335822079903, 0.09481173333938986, 0.06707344441077223], [0.8934517422227609, 0.2396139983511939, 0.19101853860665396, 0.4171276865370478], [0.22829020701999614, 0.4812258416018981, 0.9108647018398752, 0.6358083356281358]]], [[[0.22552162268644904, 0.47534616947762254, 0.06512477479564704, 0.6910937812572197], [0.2249203911196569, 0.6978557503878542, 0.368946598016913, 0.7755495386039052], [0.49108308762414443, 0.1695299897414928, 0.22630211599545969, 0.9597913422641011]], [[0.7264228563470575, 0.08546740015657561, 0.7246885976804309, 0.2584449750985891], [0.536594829236391, 0.1423608198387767, 0.13252325551729016, 0.8699762575324446], [0.08150858744193046, 0.49089574081931275, 0.11441394798629057, 0.7445579885460055]]]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactZero_fromConstant_ReducedSolution_rank0(self):
      """
      test integral of constant rank 0 Data on ReducedFunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactZero(self.domain)
      w_arg = ReducedSolution(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = 9.145496e-01
      arg = Data(val,(),w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactZero_fromConstant_ReducedSolution_rank1(self):
      """
      test integral of constant rank 1 Data on ReducedFunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactZero(self.domain)
      w_arg = ReducedSolution(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([0.6488403359176133, 0.43312146081756975, 0.5835821522820724] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactZero_fromConstant_ReducedSolution_rank2(self):
      """
      test integral of constant rank 2 Data on ReducedFunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactZero(self.domain)
      w_arg = ReducedSolution(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[0.25094433501222213, 0.9668073067298761], [0.5828562642327564, 0.5706916149344141], [0.6460329828664261, 0.16845621980623227], [0.5581878212339663, 0.6765017068377533]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactZero_fromConstant_ReducedSolution_rank3(self):
      """
      test integral of constant rank 3 Data on ReducedFunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactZero(self.domain)
      w_arg = ReducedSolution(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[[0.36025508078581103, 0.38059627481527014, 0.7163027493391224, 0.6775741297781259], [0.5720254839777164, 0.2254200611431153, 0.0024270397122935217, 0.23201620550487279]], [[0.24534780637468923, 0.31927445126537335, 0.1648299593377175, 0.42989984806485015], [0.4121753209599214, 0.39951276989507745, 0.3339733046765516, 0.17050189732976528]]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 2, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactZero_fromConstant_ReducedSolution_rank4(self):
      """
      test integral of constant rank 4 Data on ReducedFunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactZero(self.domain)
      w_arg = ReducedSolution(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[[[0.5151503491351145, 0.31376196005774526, 0.5038528508223795, 0.9127964032613619], [0.5102430713848369, 0.5568865641551866, 0.9538483975043232, 0.1228432147514682], [0.015148639985986034, 0.5402183855281764, 0.999285972760258, 0.07944414937541666]], [[0.7850024831974397, 0.6123097025371493, 0.43796242132323704, 0.538715764546832], [0.6044952307580033, 0.8932044836004265, 0.692946759879756, 0.3762129104696551], [0.6213753745788941, 0.8383947083143557, 0.7139492567784624, 0.8386050175542806]]], [[[0.5591314514242982, 0.3239965241899819, 0.21099246539658323, 0.1673683324190306], [0.2865551504358138, 0.8072557517835051, 0.21927835479431912, 0.03248749109896876], [0.47136579198331763, 0.8691870407910404, 0.18820928370224932, 0.7080227182616429]], [[0.44808335516227604, 0.7559176000169756, 0.673035935929749, 0.22612587268796502], [0.27926503793315194, 0.27670086102621716, 0.6423629143791996, 0.24593939030873546], [0.8890194370521015, 0.697652681663893, 0.37421904720900656, 0.2897157845796675]]], [[[0.6629215209608263, 0.4524944773859445, 0.5398726025914728, 0.2784353603275229], [0.3872553383985835, 0.31942784309887406, 0.07290410167348871, 0.25930474794652036], [0.8776018513287546, 0.5742960754296649, 0.9914579911424887, 0.2987912848910105]], [[0.01001459077049216, 0.19156734667362607, 0.9926616095401244, 0.7765511994210598], [0.8055335699570756, 0.4568151614277606, 0.5366500342803413, 0.7717813095218318], [0.0254833033009344, 0.17368781832596214, 0.7561513057402222, 0.4913334290393455]]]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactZero_fromConstant_ContinuousFunction_rank0(self):
      """
      test integral of constant rank 0 Data on ReducedFunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactZero(self.domain)
      w_arg = ContinuousFunction(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = 7.417632e-01
      arg = Data(val,(),w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactZero_fromConstant_ContinuousFunction_rank1(self):
      """
      test integral of constant rank 1 Data on ReducedFunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactZero(self.domain)
      w_arg = ContinuousFunction(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([0.7975013443779124, 0.44845632167537797, 0.09155291677857957] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactZero_fromConstant_ContinuousFunction_rank2(self):
      """
      test integral of constant rank 2 Data on ReducedFunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactZero(self.domain)
      w_arg = ContinuousFunction(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[0.4704310290232887, 0.382716052275478], [0.6829567302822172, 0.027020724373451488], [0.3054069882049997, 0.5369302834082202], [0.7845491995004163, 0.807581969806345]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactZero_fromConstant_ContinuousFunction_rank3(self):
      """
      test integral of constant rank 3 Data on ReducedFunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactZero(self.domain)
      w_arg = ContinuousFunction(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[[0.5330152510991164, 0.9199824564164583, 0.8207847959953989, 0.761717696692577], [0.9106612882105936, 0.7467849697774439, 0.0037183934772623273, 0.1498315437362655]], [[0.36804009217614764, 0.8833293665049883, 0.6853212083711471, 0.35265378978442696], [0.2180315417947929, 0.612513783589099, 0.773777551700871, 0.057421564295797145]]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 2, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactZero_fromConstant_ContinuousFunction_rank4(self):
      """
      test integral of constant rank 4 Data on ReducedFunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactZero(self.domain)
      w_arg = ContinuousFunction(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[[[0.8863524302153578, 0.5141021317770186, 0.7537735302695424, 0.8508085921477385], [0.4086519983672008, 0.23549947376508495, 0.7039155114055838, 0.5761782240881609], [0.9292336101279125, 0.3633707542304313, 0.577000387617159, 0.5299257949280989]], [[0.22397886787418286, 0.8798483548581983, 0.2747849998376978, 0.026576659653054224], [0.3030673633563975, 0.6548975278195708, 0.40799602538637103, 0.08771142334126913], [0.3329761874537872, 0.5246729681877698, 0.40698232299630244, 0.7426551916635709]]], [[[0.987870797626442, 0.3333029496661134, 0.7301892116966903, 0.4192629038678156], [0.4932023536932414, 0.026841627796700074, 0.023523455200852972, 0.9768314368430113], [0.6954810926781612, 0.2459361347142085, 0.9673844231154717, 0.52978184105355]], [[0.9032546300879507, 0.11518048221808563, 0.5552394459979392, 0.12700824202102545], [0.7297550761998094, 0.3881377644766276, 0.9629921914330017, 0.4505781475461933], [0.3679014211947874, 0.5642672771876244, 0.9703976676459037, 0.9777488809935937]]], [[[0.8035733135686045, 0.5626597091388919, 0.8655824522464319, 0.3781034482327684], [0.8471614620257096, 0.5426187605520886, 0.48607910670439636, 0.6565384089530854], [0.9013489632981162, 0.004450523429103792, 0.37627030140819095, 0.14231934711722183]], [[0.8385948931328291, 0.021696534497830378, 0.4961012637090927, 0.2692025033792481], [0.6918044533757262, 0.768347915598968, 0.9522382443138381, 0.49934694361291265], [0.5496404760362463, 0.09953883732162139, 0.35452444213193035, 0.057516644833046415]]]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactZero_fromConstant_ReducedContinuousFunction_rank0(self):
      """
      test integral of constant rank 0 Data on ReducedFunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactZero(self.domain)
      w_arg = ReducedContinuousFunction(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = 9.127844e-01
      arg = Data(val,(),w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactZero_fromConstant_ReducedContinuousFunction_rank1(self):
      """
      test integral of constant rank 1 Data on ReducedFunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactZero(self.domain)
      w_arg = ReducedContinuousFunction(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([0.686044159445106, 0.6859000925904821, 0.5085437825791668] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactZero_fromConstant_ReducedContinuousFunction_rank2(self):
      """
      test integral of constant rank 2 Data on ReducedFunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactZero(self.domain)
      w_arg = ReducedContinuousFunction(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[0.38642339954482186, 0.22093605108418446], [0.6560527751042998, 0.48101694185001476], [0.044053186041325354, 0.55365472242926], [0.8728242939804305, 0.9787760437170994]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactZero_fromConstant_ReducedContinuousFunction_rank3(self):
      """
      test integral of constant rank 3 Data on ReducedFunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactZero(self.domain)
      w_arg = ReducedContinuousFunction(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[[0.3490822753217583, 0.23823874426489722, 0.5746622422578912, 0.8535250260002284], [0.8038112306172642, 0.04380594518868375, 0.3603694287985125, 0.6385995336804273]], [[0.5845474735779229, 0.5123977082215297, 0.7961843678372816, 0.5172605209307137], [0.9142372760381393, 0.017481159926396983, 0.1114684172312348, 0.16951881730947316]]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 2, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactZero_fromConstant_ReducedContinuousFunction_rank4(self):
      """
      test integral of constant rank 4 Data on ReducedFunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactZero(self.domain)
      w_arg = ReducedContinuousFunction(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[[[0.8878382344534057, 0.13674890886415347, 0.7792990782571476, 0.09402029233952913], [0.04663076253939258, 0.3238678259304406, 0.28811058149949975, 0.11470329678624758], [0.6578417744257605, 0.41035755516104655, 0.22986399836306448, 0.7415776216770451]], [[0.9900663091309843, 0.5033328668968147, 0.7673205303148356, 0.1709449228181229], [0.4646332440040789, 0.4196792950355013, 0.43323792629729685, 0.05213855036603143], [0.37628677300187596, 0.3287754478610895, 0.9843100207447746, 0.5844935228888299]]], [[[0.09439341411197677, 0.28988436643314774, 0.878475453334073, 0.040225236688740784], [0.040127550897501085, 0.48392963447599546, 0.2630024392922711, 0.10966912765218595], [0.11565273565416956, 0.09183161284539587, 0.45589922961009643, 0.4902452484655726]], [[0.9628418766744591, 0.5787408805369061, 0.476319343458083, 0.6285633538071376], [0.3985010941762869, 0.47880471790943413, 0.28298418604517794, 0.31176144616492174], [0.25360607442920546, 0.4487940987530682, 0.863418879096857, 0.7629071391504596]]], [[[0.9163249136043641, 0.44531376748665785, 0.4129265480720671, 0.7834106060790957], [0.03201669519578254, 0.15049558959481257, 0.7206061772425282, 0.1612712789153674], [0.8070672122170596, 0.886480903667749, 0.6724491201103312, 0.28555057650793736]], [[0.7783236203118385, 0.9788631652156544, 0.03177126045123402, 0.12344749519119091], [0.029911510601437374, 0.05242047743998901, 0.5977930627346539, 0.6685537550680941], [0.9229371756181977, 0.6886472937234335, 0.13435058433927782, 0.5478385098404411]]]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactZero_fromConstant_ReducedFunctionOnContactZero_rank0(self):
      """
      test integral of constant rank 0 Data on ReducedFunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactZero(self.domain)
      w_arg = ReducedFunctionOnContactZero(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = 4.750773e-01
      arg = Data(val,(),w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactZero_fromConstant_ReducedFunctionOnContactZero_rank1(self):
      """
      test integral of constant rank 1 Data on ReducedFunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactZero(self.domain)
      w_arg = ReducedFunctionOnContactZero(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([0.21336301714324446, 0.4926579851725096, 0.03202956961913683] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactZero_fromConstant_ReducedFunctionOnContactZero_rank2(self):
      """
      test integral of constant rank 2 Data on ReducedFunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactZero(self.domain)
      w_arg = ReducedFunctionOnContactZero(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[0.9484987653320537, 0.20189258401547805], [0.9978726677022479, 0.41513331705222867], [0.7608402893428862, 0.46173110774383674], [0.4059076556015372, 0.8881569921753834]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactZero_fromConstant_ReducedFunctionOnContactZero_rank3(self):
      """
      test integral of constant rank 3 Data on ReducedFunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactZero(self.domain)
      w_arg = ReducedFunctionOnContactZero(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[[0.3632048531461828, 0.6514034516925535, 0.5642383295711927, 0.7208948153334722], [0.8907419677092967, 0.2543467312338745, 0.7031062223700405, 0.06143389460786197]], [[0.1033204954121818, 0.9673150103683781, 0.22383948335471626, 0.053862940893552036], [0.7837643380541005, 0.8335712597190302, 0.3061805911607115, 0.3838800128510724]]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 2, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactZero_fromConstant_ReducedFunctionOnContactZero_rank4(self):
      """
      test integral of constant rank 4 Data on ReducedFunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactZero(self.domain)
      w_arg = ReducedFunctionOnContactZero(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[[[0.46724330914007195, 0.7943116543887297, 0.44689304933912855, 0.106086850002311], [0.859974469452861, 0.42068348997881055, 0.11207010641633985, 0.7137337433618505], [0.9397555090228368, 0.059827261675371, 0.968557878153554, 0.24213916385901302]], [[0.3173082629354119, 0.7630875202019153, 0.21903191581080805, 0.37133162137270725], [0.28140965952295127, 0.08867026167809744, 0.05271963507580324, 0.17873840282560305], [0.25670900510408867, 0.4573130910367038, 0.1799492276857283, 0.30739834234119945]]], [[[0.142598236756997, 0.48943075965266636, 0.509590743758051, 0.6111692476535552], [0.6639417381139768, 0.9111679657868261, 0.02024047450186961, 0.8439022724471736], [0.3358221183138327, 0.25270842707362307, 0.5810989876048378, 0.7852820279614563]], [[0.03207042122215975, 0.4144127539307515, 0.638994561500725, 0.9093728558294354], [0.3204201738302107, 0.336317025785169, 0.9514624523985529, 0.8827099329631747], [0.27037757532531626, 0.09240697430369216, 0.46416491378560387, 0.4668442083554155]]], [[[0.07363471895536566, 0.6935228124813128, 0.11324662792080897, 0.5504965284255703], [0.07766336531041285, 0.4960087818840313, 0.9071436537043066, 0.44613683305780194], [0.5051862888675825, 0.43610790862102844, 0.40102233260783104, 0.2536279376132835]], [[0.3964698572768971, 0.0728296907585203, 0.4831915179798574, 0.616626346506119], [0.29356111031971055, 0.03465969501398236, 0.07475872946890483, 0.03837456922959648], [0.12336541598494255, 0.9360954692844571, 0.07324047352055929, 0.3917329344901287]]]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactZero_fromConstant_ReducedFunctionOnContactOne_rank0(self):
      """
      test integral of constant rank 0 Data on ReducedFunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactZero(self.domain)
      w_arg = ReducedFunctionOnContactOne(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = 3.672892e-01
      arg = Data(val,(),w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactZero_fromConstant_ReducedFunctionOnContactOne_rank1(self):
      """
      test integral of constant rank 1 Data on ReducedFunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactZero(self.domain)
      w_arg = ReducedFunctionOnContactOne(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([0.6182111000441537, 0.5148441424203023, 0.6655800796772579] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactZero_fromConstant_ReducedFunctionOnContactOne_rank2(self):
      """
      test integral of constant rank 2 Data on ReducedFunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactZero(self.domain)
      w_arg = ReducedFunctionOnContactOne(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[0.4988567575044003, 0.30938186516449917], [0.5320416623834278, 0.7231322113014568], [0.012746360901172205, 0.6889289526347554], [0.6575321678946946, 0.8461232121282896]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactZero_fromConstant_ReducedFunctionOnContactOne_rank3(self):
      """
      test integral of constant rank 3 Data on ReducedFunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactZero(self.domain)
      w_arg = ReducedFunctionOnContactOne(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[[0.911269147334316, 0.3505849565599999, 0.7577988546287429, 0.6824863367247404], [0.9624301598624073, 0.5219149331139439, 0.3694098151239805, 0.4750221034634994]], [[0.11185836249756542, 0.30431894864149334, 0.14883302162114365, 0.11808591492728227], [0.9870196176056787, 0.5754974071103578, 0.6054838533420094, 0.3801064044432697]]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 2, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactZero_fromConstant_ReducedFunctionOnContactOne_rank4(self):
      """
      test integral of constant rank 4 Data on ReducedFunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactZero(self.domain)
      w_arg = ReducedFunctionOnContactOne(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[[[0.2494636544590333, 0.9321641843638748, 0.6287576342833752, 0.9684825785793357], [0.9000076334442965, 0.8750898083747524, 0.639822615411278, 0.9927313187774435], [0.29552152782271524, 0.7280945992103874, 0.9377720901874705, 0.5663759788765188]], [[0.5385702209545004, 0.8713770642561344, 0.737813646182876, 0.8176612781568534], [0.2393632527601498, 0.47796284353943785, 0.6819459331150214, 0.37138014969117183], [0.336006951270017, 0.8320924714545408, 0.8506710397212738, 0.11304315538410947]]], [[[0.9314575752404489, 0.20698796841031097, 0.26863666738666225, 0.972465954452442], [0.8523407595907638, 0.11846171055287824, 0.7084581237554853, 0.6146061292180579], [0.07537035525572833, 0.12399645967146644, 0.3217088024514849, 0.5067881767424013]], [[0.1506487650822329, 0.5866013654210072, 0.1563312329587785, 0.58017246485903], [0.6096616664437833, 0.6738531279188033, 0.6018025745381165, 0.44342773470724717], [0.6049619252313234, 0.829503865562864, 0.6845164546459848, 0.7553235952438251]]], [[[0.3551428459783058, 0.389628352577993, 0.34637969114773615, 0.05433428412432517], [0.2903640469638441, 0.6097547735713603, 0.130224431035191, 0.38334232982417316], [0.07864498639572526, 0.4041664594016362, 0.21905219217488314, 0.6902464836206194]], [[0.05674605266760735, 0.5684804508948936, 0.9167073840503153, 0.861745249909916], [0.04162975946179426, 0.19533613820804208, 0.7483826267119846, 0.46403839093389687], [0.62985359414214, 0.18807503614182408, 0.9885079488524472, 0.4294217188999735]]]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactZero_fromConstant_FunctionOnContactZero_rank0(self):
      """
      test integral of constant rank 0 Data on ReducedFunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactZero(self.domain)
      w_arg = FunctionOnContactZero(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = 4.260137e-02
      arg = Data(val,(),w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactZero_fromConstant_FunctionOnContactZero_rank1(self):
      """
      test integral of constant rank 1 Data on ReducedFunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactZero(self.domain)
      w_arg = FunctionOnContactZero(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([0.4848166554451502, 0.1459856943610377, 0.7988037741995702] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactZero_fromConstant_FunctionOnContactZero_rank2(self):
      """
      test integral of constant rank 2 Data on ReducedFunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactZero(self.domain)
      w_arg = FunctionOnContactZero(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[0.17298173109084525, 0.5235470293102914], [0.26761746872127434, 0.7698296600141968], [0.5404448185582517, 0.345442749081986], [0.34005178248030954, 0.33362931187832334]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactZero_fromConstant_FunctionOnContactZero_rank3(self):
      """
      test integral of constant rank 3 Data on ReducedFunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactZero(self.domain)
      w_arg = FunctionOnContactZero(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[[0.009411259439403907, 0.4611130416948871, 0.9391898513407464, 0.29486370004048423], [0.6247372172074737, 0.6791541626339095, 0.2099443799436138, 0.8629585597447137]], [[0.5571296171240786, 0.7502944533232693, 0.9452952272671803, 0.4573691996222998], [0.952981596049467, 0.11118247661252811, 0.05392086992777301, 0.5376053043743428]]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 2, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactZero_fromConstant_FunctionOnContactZero_rank4(self):
      """
      test integral of constant rank 4 Data on ReducedFunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactZero(self.domain)
      w_arg = FunctionOnContactZero(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[[[0.43355673289733876, 0.11801260845521977, 0.7087375146604054, 0.2778221066896601], [0.5094126506995085, 0.07153783488450804, 0.5989124985373315, 0.8164794341708241], [0.654921863191017, 0.014287607118830814, 0.752918038277869, 0.6273480431669939]], [[0.47164537732814715, 0.20127599694733644, 0.5932255576822645, 0.9962804893392272], [0.4931915781903602, 0.14615189256953653, 0.7704397410944633, 0.656316205577235], [0.48394058342702784, 0.9991980903443893, 0.43317074946168954, 0.09108774044685086]]], [[[0.6621490014082553, 0.26156926820483073, 0.8945350850753389, 0.636445737354847], [0.124694789296521, 0.9480096151059947, 0.6349604098200311, 0.3526315720340535], [0.002079254352673421, 0.3405499748631644, 0.31735128859404327, 0.9282510648663834]], [[0.6731549191176112, 0.7550295640086269, 0.46797626216375865, 0.4177920723771473], [0.34659951068419526, 0.0947313750948724, 0.6136618055285832, 0.6499154499092002], [0.5379288912164735, 0.032348302011354635, 0.8775998388813242, 0.15881158287558161]]], [[[0.3352309459226639, 0.8128229011786033, 0.2992210557276648, 0.013937123142673591], [0.15705218982653468, 0.5660240183714428, 0.6840644888052033, 0.4104891753475105], [0.36021443687433785, 0.4480755940994, 0.23911046930370305, 0.434376854139499]], [[0.5058047679530762, 0.3461568996567741, 0.5026947199126742, 0.6237778200785412], [0.9854779455177672, 0.07330210563682327, 0.4856743510170125, 0.9791475218713811], [0.5585718307944592, 0.06239256373450108, 0.2539354326769475, 0.7695578717339137]]]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactZero_fromConstant_FunctionOnContactOne_rank0(self):
      """
      test integral of constant rank 0 Data on ReducedFunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactZero(self.domain)
      w_arg = FunctionOnContactOne(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = 5.591169e-01
      arg = Data(val,(),w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactZero_fromConstant_FunctionOnContactOne_rank1(self):
      """
      test integral of constant rank 1 Data on ReducedFunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactZero(self.domain)
      w_arg = FunctionOnContactOne(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([0.8601034214009924, 0.9413456797910883, 0.032080190203067294] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactZero_fromConstant_FunctionOnContactOne_rank2(self):
      """
      test integral of constant rank 2 Data on ReducedFunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactZero(self.domain)
      w_arg = FunctionOnContactOne(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[0.10995480000055458, 0.560875216930863], [0.8106554505246043, 0.5572924817831376], [0.122260482665836, 0.8673322892186142], [0.20526090480379633, 0.9192323862960567]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactZero_fromConstant_FunctionOnContactOne_rank3(self):
      """
      test integral of constant rank 3 Data on ReducedFunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactZero(self.domain)
      w_arg = FunctionOnContactOne(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[[0.33736986151429027, 0.9504787678637983, 0.17607657716915237, 0.6399451469131869], [0.6878771663479692, 0.2556907322208567, 0.033691045524990804, 0.9791415691570137]], [[0.43970771604211145, 0.3079485734016333, 0.3038595284836839, 0.37014956108118846], [0.4846613983607927, 0.05066654461996822, 0.31041927171451655, 0.30377683477885975]]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 2, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactZero_fromConstant_FunctionOnContactOne_rank4(self):
      """
      test integral of constant rank 4 Data on ReducedFunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactZero(self.domain)
      w_arg = FunctionOnContactOne(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[[[0.5632117628871797, 0.7569473351950018, 0.8172544357661247, 0.7238601183779925], [0.5004385007006551, 0.21193343863539504, 0.17777048686141306, 0.27641674318854126], [0.5710000575946498, 0.5008996884701732, 0.17008421416357256, 0.9987251228677066]], [[0.7941602212953656, 0.8736209362405195, 0.2762352260995191, 0.17697096218734942], [0.32811552143536926, 0.9052558721769968, 0.5997867228632167, 0.4609903385929982], [0.4779106057991179, 0.08619150478219229, 0.889985229199598, 0.5682850384167982]]], [[[0.4899562713509431, 0.3338798785861644, 0.4407345822014288, 0.14069031528716147], [0.8443848712818334, 0.6314635058826136, 0.36568769062483686, 0.739575741291722], [0.9818118869661545, 0.671919262484029, 0.3904044800809746, 0.8416231397391041]], [[0.680269712211979, 0.5394725196048809, 0.011607741842188601, 0.058544314258036634], [0.7074018394895647, 0.9030297283202042, 0.06995096530566947, 0.03136692691549681], [0.5762091008207191, 0.9388433280918118, 0.7941895853799287, 0.7263480104904657]]], [[[0.9344798560467055, 0.013264608447003723, 0.2810491417754114, 0.9994040446165507], [0.8097700006979895, 0.017920524488821976, 0.5335390719382904, 0.6970983962453248], [0.5748248618933889, 0.7332693222290667, 0.15603064530080157, 0.06024980866358742]], [[0.6543686799652171, 0.569381501694053, 0.7322202931001693, 0.5493696526370417], [0.6924511644547436, 0.2328718885348091, 0.9121867457225998, 0.727661015825948], [0.3503430606331268, 0.10229475639208574, 0.8956372180490341, 0.10750911407732344]]]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactOne_fromConstant_Solution_rank0(self):
      """
      test integral of constant rank 0 Data on ReducedFunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactOne(self.domain)
      w_arg = Solution(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = 4.635506e-01
      arg = Data(val,(),w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactOne_fromConstant_Solution_rank1(self):
      """
      test integral of constant rank 1 Data on ReducedFunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactOne(self.domain)
      w_arg = Solution(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([0.6591168973917837, 0.7093293050550054, 0.5812043244368388] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactOne_fromConstant_Solution_rank2(self):
      """
      test integral of constant rank 2 Data on ReducedFunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactOne(self.domain)
      w_arg = Solution(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[0.4481075552466961, 0.9263317891788495], [0.8178025586404991, 0.056880147459412056], [0.003971730260071027, 0.9750684215247315], [0.6013933938341001, 0.34411249674293976]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactOne_fromConstant_Solution_rank3(self):
      """
      test integral of constant rank 3 Data on ReducedFunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactOne(self.domain)
      w_arg = Solution(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[[0.5370405113095748, 0.26504207441058913, 0.5678630843986171, 0.5440231525045225], [0.37463641790052427, 0.3362126093750898, 0.820320579734166, 0.33491475242993385]], [[0.28672739475713105, 0.9797760512709587, 0.06077083964973595, 0.201320049419771], [0.6968728833464424, 0.6159900650505972, 0.06581917525496817, 0.33155386346596083]]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 2, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactOne_fromConstant_Solution_rank4(self):
      """
      test integral of constant rank 4 Data on ReducedFunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactOne(self.domain)
      w_arg = Solution(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[[[0.9672110621099455, 0.8577481930580224, 0.04479924014462744, 0.5469501199513037], [0.37286386922648795, 0.2396130613238362, 0.26114816988996203, 0.5806121293962747], [0.1949245411949485, 0.02152815773561001, 0.8440495356765813, 0.20300038448420377]], [[0.7157271535548353, 0.13243770689265577, 0.32019413774438754, 0.22274570644654978], [0.5591345426729251, 0.6648793075314693, 0.6996369047531661, 0.9391546088690959], [0.00025698168108434505, 0.31339622592065397, 0.49329340204920136, 0.5578717099178316]]], [[[0.042449316086755506, 0.46625680581415474, 0.4442712304626306, 0.9431647257324046], [0.4791056606946632, 0.05241887283829538, 0.42207804334750487, 0.8603713857545403], [0.47533255153398757, 0.22486375638026035, 0.14220689686470767, 0.3112410707661257]], [[0.18023914888154602, 0.4269417480566635, 0.059161233560148796, 0.807106719415425], [0.6561384294507512, 0.7021777922868789, 0.49142166448462254, 0.8987289711269718], [0.1251991806486864, 0.16783248010860485, 0.14306771329104295, 0.4192660284399553]]], [[[0.7983846319199841, 0.35286949345831287, 0.7449878173037571, 0.8007347311972667], [0.11633674682559803, 0.8041325012643269, 0.3594221968979959, 0.07236046795754258], [0.3002439886000323, 0.29837424942206336, 0.5188373365696166, 0.1719853640892709]], [[0.33696066011666936, 0.4161015782615255, 0.5485860700621579, 0.38324752467958134], [0.3678857792920902, 0.6749846969742358, 0.5273150667932907, 0.5025544248981302], [0.7695287711209781, 0.1685594066645928, 0.8387559541694748, 0.3461181720730645]]]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactOne_fromConstant_ReducedSolution_rank0(self):
      """
      test integral of constant rank 0 Data on ReducedFunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactOne(self.domain)
      w_arg = ReducedSolution(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = 3.904440e-01
      arg = Data(val,(),w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactOne_fromConstant_ReducedSolution_rank1(self):
      """
      test integral of constant rank 1 Data on ReducedFunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactOne(self.domain)
      w_arg = ReducedSolution(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([0.4890691966167301, 0.22650345580289422, 0.8943318876355936] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactOne_fromConstant_ReducedSolution_rank2(self):
      """
      test integral of constant rank 2 Data on ReducedFunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactOne(self.domain)
      w_arg = ReducedSolution(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[0.029274628818413584, 0.6483247263336201], [0.9675058005388452, 0.7034305783142056], [0.3145437287334534, 0.22371601695393295], [0.38315248081472253, 0.01132773446197477]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactOne_fromConstant_ReducedSolution_rank3(self):
      """
      test integral of constant rank 3 Data on ReducedFunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactOne(self.domain)
      w_arg = ReducedSolution(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[[0.5533451505618618, 0.48164923309197394, 0.2726182360572458, 0.489147991595708], [0.11404706301477652, 0.6683184527092706, 0.9984464023472918, 0.6952417980693619]], [[0.8418559087710236, 0.5565987760501447, 0.5252179446341174, 0.2979145338068392], [0.1794171877631764, 0.22669558213871033, 0.9159021981224137, 0.7677098814566673]]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 2, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactOne_fromConstant_ReducedSolution_rank4(self):
      """
      test integral of constant rank 4 Data on ReducedFunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactOne(self.domain)
      w_arg = ReducedSolution(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[[[0.7342555908758024, 0.30547171422301944, 0.9932166251183027, 0.23792123732861759], [0.8445053702654959, 0.9670657407254408, 0.9827679330459912, 0.2954784654841819], [0.9673190165481341, 0.5747735734661384, 0.823962884555036, 0.818810912708159]], [[0.2812655106635723, 0.6214047988136943, 0.9291629866419473, 0.041086640851140666], [0.511055224486917, 0.8208408978930605, 0.9495433216894433, 0.22254918356724318], [0.3281595096943798, 0.4228885377743763, 0.9758883829435414, 0.8614572099217364]]], [[[0.00436970011191995, 0.39274794253310197, 0.9563031215710485, 0.05460234037208267], [0.746353280886055, 0.37321992806157334, 0.11037211088302146, 0.6407989621368507], [0.7212250582950387, 0.074837391165964, 0.22873602337289778, 0.24194102703098186]], [[0.8820902379250833, 0.09527064858643775, 0.3849680080162564, 0.8074810013227114], [0.4857058804099731, 0.07579745817276673, 0.7614852963590033, 0.8528575302801255], [0.12529472327062618, 0.8838414539300405, 0.841526083040229, 0.03996681223912413]]], [[[0.5323790199002708, 0.8209035262337795, 0.029194947199732812, 0.2585812058721787], [0.28938398392705966, 0.05772649695799681, 0.2740288142338453, 0.7083527720230409], [0.9964885275925106, 0.954893537057956, 0.6784127455772663, 0.249566888345522]], [[0.9205358220441707, 0.5849746479236245, 0.9763213275122087, 0.6130852719211534], [0.6439497425502442, 0.8654015799109439, 0.4729750235585486, 0.7280782332548427], [0.42402769702719756, 0.2396449180135053, 0.7956334735449024, 0.9017019983898613]]]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactOne_fromConstant_ContinuousFunction_rank0(self):
      """
      test integral of constant rank 0 Data on ReducedFunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactOne(self.domain)
      w_arg = ContinuousFunction(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = 7.881088e-02
      arg = Data(val,(),w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactOne_fromConstant_ContinuousFunction_rank1(self):
      """
      test integral of constant rank 1 Data on ReducedFunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactOne(self.domain)
      w_arg = ContinuousFunction(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([0.057130194051177474, 0.6127832557331973, 0.14120294644312525] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactOne_fromConstant_ContinuousFunction_rank2(self):
      """
      test integral of constant rank 2 Data on ReducedFunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactOne(self.domain)
      w_arg = ContinuousFunction(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[0.741452035367966, 0.7629173412284856], [0.9671536403955171, 0.4418644245915956], [0.4275847096344141, 0.4006671404186476], [0.4357008155633185, 0.9060043251600465]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactOne_fromConstant_ContinuousFunction_rank3(self):
      """
      test integral of constant rank 3 Data on ReducedFunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactOne(self.domain)
      w_arg = ContinuousFunction(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[[0.4699068848814748, 0.5406065451293735, 0.40777283100470973, 0.1610842242747843], [0.6761192290314728, 0.116082143814077, 0.8869573075123034, 0.3614881681075002]], [[0.13075882495380808, 0.009282487629694636, 0.7522878491424495, 0.08972003759602243], [0.8495203866962844, 0.37642538376736034, 0.4156525317803559, 0.24371553855958672]]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 2, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactOne_fromConstant_ContinuousFunction_rank4(self):
      """
      test integral of constant rank 4 Data on ReducedFunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactOne(self.domain)
      w_arg = ContinuousFunction(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[[[0.4457481880529707, 0.7195803448552743, 0.37066555598311834, 0.8356974692283944], [0.6103174478383246, 0.13833306639238474, 0.6987623340603879, 0.6992460495889199], [0.9437908424373677, 0.09418773312114603, 0.9253934880835294, 0.4602278224274885]], [[0.10454938548640202, 0.6756149841036414, 0.7295458776356214, 0.9806333231779377], [0.6063923633911582, 0.2955101788116553, 0.8310380901491737, 0.014930743614127162], [0.6958601842210156, 0.2543310783320043, 0.8662853531600784, 0.5153816881459732]]], [[[0.40651208098603164, 0.07427516164105041, 0.1762103525856873, 0.9942925290493997], [0.7009470651464719, 0.8690134296747459, 0.9792891184558238, 0.05931682909690983], [0.18055229930105787, 0.430607991861238, 0.9352281082183351, 0.2856142471015045]], [[0.02491362841881417, 0.08628860569558006, 0.7668505440505008, 0.42441617871359827], [0.43919268690570723, 0.12354619167945569, 0.6009871671478554, 0.8875589398449725], [0.5533494750208501, 0.1634868050535777, 0.6363502983980578, 0.8859662237022607]]], [[[0.8119015875065023, 0.953550934072758, 0.2774687540353833, 0.9855281345842936], [0.36699375297143677, 0.7005345501443304, 0.02241878262307484, 0.996757893528923], [0.590281467235221, 0.7504128758671901, 0.31114491923097165, 0.6725956675372938]], [[0.9859348998210977, 0.18173087327102477, 0.9198226751025405, 0.788697717686816], [0.8511027029029439, 0.7395067554492902, 0.09433559444772377, 0.8256031465138104], [0.9261676204732905, 0.9023788160948374, 0.9280865573327655, 0.6436776085128718]]]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactOne_fromConstant_ReducedContinuousFunction_rank0(self):
      """
      test integral of constant rank 0 Data on ReducedFunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactOne(self.domain)
      w_arg = ReducedContinuousFunction(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = 2.346177e-01
      arg = Data(val,(),w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactOne_fromConstant_ReducedContinuousFunction_rank1(self):
      """
      test integral of constant rank 1 Data on ReducedFunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactOne(self.domain)
      w_arg = ReducedContinuousFunction(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([0.9459471766052175, 0.329712840120269, 0.13761073207408303] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactOne_fromConstant_ReducedContinuousFunction_rank2(self):
      """
      test integral of constant rank 2 Data on ReducedFunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactOne(self.domain)
      w_arg = ReducedContinuousFunction(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[0.7207921279448569, 0.3472115664627906], [0.4981882111511854, 0.6632151024751499], [0.8452396235498024, 0.9626039198534639], [0.3878305529789272, 0.43924801992958173]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactOne_fromConstant_ReducedContinuousFunction_rank3(self):
      """
      test integral of constant rank 3 Data on ReducedFunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactOne(self.domain)
      w_arg = ReducedContinuousFunction(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[[0.4326739762474251, 0.003455975709570791, 0.43710139123045355, 0.9739316114111745], [0.3846624389415574, 0.06798740664442937, 0.5663157359417933, 0.9854295810649004]], [[0.34085087848216356, 0.2796669039770846, 0.6902331436153027, 0.6971130051175111], [0.1602474886491423, 0.6555566634876816, 0.606973062809386, 0.045737813371095615]]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 2, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactOne_fromConstant_ReducedContinuousFunction_rank4(self):
      """
      test integral of constant rank 4 Data on ReducedFunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactOne(self.domain)
      w_arg = ReducedContinuousFunction(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[[[0.7293345594872522, 0.9203515486362716, 0.11615878234500165, 0.47142104687580444], [0.9168725302801237, 0.34820328358351205, 0.40232917627495657, 0.97424652343025], [0.4150861963357185, 0.5991240735102373, 0.795572555273179, 0.8867745943054096]], [[0.14680992586196262, 0.4478985831463368, 0.41559378039000816, 0.40722362760460906], [0.7116171576581489, 0.37782577846006293, 0.06431709027932286, 0.2192787523151668], [0.7259323622579246, 0.2166249011277246, 0.2638170045738898, 0.6958555689916919]]], [[[0.6864331095914465, 0.9199464719567179, 0.2828036847249886, 0.28118125750215905], [0.19419498413236036, 0.13890195553850948, 0.1760454738307532, 0.1313780856006468], [0.6550192769046749, 0.28106573640296717, 0.057368098974731496, 0.5517170715148233]], [[0.3921144880676458, 0.5266974848395647, 0.9064916202676432, 0.35097820627675713], [0.5622182882971267, 0.15881370277354523, 0.520969260044073, 0.562755059196888], [0.4049988357698662, 0.4361132662037802, 0.11751925218806014, 0.5592336477729075]]], [[[0.9679872167989797, 0.19190080960769473, 0.7255135027193516, 0.8421470709052287], [0.615480794389962, 0.25493406848941536, 0.34406064589160956, 0.07688844406110018], [0.901709491780357, 0.1429366300946583, 0.0641765892202184, 0.02980629316633643]], [[0.5285227357553478, 0.9851850949689838, 0.6861863696874184, 0.18784169169573461], [0.17461874367983232, 0.5167482928623075, 0.5829936714349406, 0.4168559333676286], [0.1218906091198283, 0.03301398396587796, 0.25002793371713405, 0.6769114640418931]]]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactOne_fromConstant_ReducedFunctionOnContactZero_rank0(self):
      """
      test integral of constant rank 0 Data on ReducedFunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactOne(self.domain)
      w_arg = ReducedFunctionOnContactZero(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = 2.471525e-01
      arg = Data(val,(),w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactOne_fromConstant_ReducedFunctionOnContactZero_rank1(self):
      """
      test integral of constant rank 1 Data on ReducedFunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactOne(self.domain)
      w_arg = ReducedFunctionOnContactZero(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([0.01043958906356135, 0.5846187706113661, 0.6399449816658811] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactOne_fromConstant_ReducedFunctionOnContactZero_rank2(self):
      """
      test integral of constant rank 2 Data on ReducedFunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactOne(self.domain)
      w_arg = ReducedFunctionOnContactZero(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[0.5400180123730902, 0.9405625959072397], [0.08951592492702032, 0.013733356223024296], [0.5122606931613205, 0.5238166820748772], [0.4998706323096298, 0.09245777329551241]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactOne_fromConstant_ReducedFunctionOnContactZero_rank3(self):
      """
      test integral of constant rank 3 Data on ReducedFunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactOne(self.domain)
      w_arg = ReducedFunctionOnContactZero(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[[0.7465977970717627, 0.5929814196189757, 0.4321446609597859, 0.5173937540804529], [0.6967163366300329, 0.18224875004205288, 0.02085918705682921, 0.3034859052924591]], [[0.7741445064340949, 0.5503871918310869, 0.561984538188848, 0.8134125128737905], [0.02471921990202164, 0.2700356170088699, 0.5052705205999801, 0.791087940501339]]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 2, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactOne_fromConstant_ReducedFunctionOnContactZero_rank4(self):
      """
      test integral of constant rank 4 Data on ReducedFunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactOne(self.domain)
      w_arg = ReducedFunctionOnContactZero(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[[[0.8191479444438201, 0.39893033708318026, 0.5515547838647816, 0.27485757638968467], [0.8155163211627533, 0.5426795364576336, 0.4742310482514541, 0.9586701218549546], [0.9731074824540649, 0.35774818106348116, 0.5551138796025206, 0.857978913010403]], [[0.8164517681640927, 0.07783764573946195, 0.012293043462458408, 0.8568217262013482], [0.18156470289337578, 0.0451648386971224, 0.8185122126411771, 0.12760634833185247], [0.016250891518352018, 0.2304874646392917, 0.43884779877924307, 0.0957752013249753]]], [[[0.32060570945340694, 0.8151504074659118, 0.8759667443934652, 0.8842176916947125], [0.8874237856690823, 0.1567709659006702, 0.32600916199683216, 0.1879442067957543], [0.6205511347366125, 0.09714358322531425, 0.247083122734298, 0.5335758444574608]], [[0.3162619114212887, 0.9141491186553203, 0.6086976555562243, 0.27561680856489523], [0.7664688012810315, 0.9397413529487998, 0.9720014247034141, 0.28024073214255585], [0.1692707066513458, 0.4823043754632649, 0.036838159526103254, 0.06872772486499334]]], [[[0.40509939085843794, 0.32925824648732416, 0.13483736485532216, 0.568393276000273], [0.0023280936671677965, 0.18988613431528356, 0.28138497475035784, 0.8875403708945667], [0.3001985275609391, 0.9261026838319929, 0.03648907640845578, 0.8938443426065736]], [[0.09601498618723348, 0.3539449856600062, 0.2590643200496431, 0.771498726686708], [0.8157143315532202, 0.7117756750069355, 0.8995757400007217, 0.11803229291464268], [0.12218584616234984, 0.8339280528536638, 0.30119472481870646, 0.6223064062311142]]]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactOne_fromConstant_ReducedFunctionOnContactOne_rank0(self):
      """
      test integral of constant rank 0 Data on ReducedFunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactOne(self.domain)
      w_arg = ReducedFunctionOnContactOne(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = 2.947275e-01
      arg = Data(val,(),w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactOne_fromConstant_ReducedFunctionOnContactOne_rank1(self):
      """
      test integral of constant rank 1 Data on ReducedFunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactOne(self.domain)
      w_arg = ReducedFunctionOnContactOne(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([0.9716448626746664, 0.030880086168497134, 0.6473499194205695] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactOne_fromConstant_ReducedFunctionOnContactOne_rank2(self):
      """
      test integral of constant rank 2 Data on ReducedFunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactOne(self.domain)
      w_arg = ReducedFunctionOnContactOne(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[0.824646210178544, 0.24325674389836438], [0.5502699426705583, 0.695819800331274], [0.26628922136157585, 0.8956608843380275], [0.3539145432895634, 0.6221992348052285]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactOne_fromConstant_ReducedFunctionOnContactOne_rank3(self):
      """
      test integral of constant rank 3 Data on ReducedFunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactOne(self.domain)
      w_arg = ReducedFunctionOnContactOne(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[[0.4810159573423819, 0.9602761661657525, 0.15467369104916473, 0.7449634841769645], [0.4830592916179749, 0.12011800949969598, 0.14665486766799962, 0.5347883910027665]], [[0.8161462721380852, 0.8953230131518285, 0.9186370652596926, 0.530400288055715], [0.9735786258609173, 0.46140286647540396, 0.2483542190198934, 0.45458627959165754]]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 2, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactOne_fromConstant_ReducedFunctionOnContactOne_rank4(self):
      """
      test integral of constant rank 4 Data on ReducedFunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactOne(self.domain)
      w_arg = ReducedFunctionOnContactOne(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[[[0.27852807189923867, 0.80414103594398, 0.38621616072952303, 0.3463424503620445], [0.015541544813967412, 0.29666053897060585, 0.981442731328498, 0.8721722316964071], [0.371669470055741, 0.7137396990008515, 0.8045344523778398, 0.1584829360138451]], [[0.23718061004633517, 0.03144169766176064, 0.7776039034279171, 0.09096925546980072], [0.6605416906983957, 0.6598736743107558, 0.393317136272588, 0.13941813042184836], [0.8378712772897127, 0.18661118478503658, 0.1922430288575674, 0.08821270428853845]]], [[[0.15370855367433534, 0.7359531308251214, 0.35454618211190425, 0.12090594188628467], [0.34489606793349137, 0.4338282811919164, 0.38670867280512045, 0.08812363723574201], [0.2932082102371014, 0.10965195146664208, 0.12899459699028093, 0.6418717199692334]], [[0.8866046160413344, 0.040749283816254867, 0.8754017864637519, 0.04156267008396175], [0.4977196282016577, 0.22374929245188868, 0.5721090964670347, 0.8826371029075082], [0.2321747741848238, 0.7731231373529478, 0.6462411337188285, 0.6894462537813224]]], [[[0.05724221765144444, 0.48101999067471335, 0.2994106078576728, 0.3505441175341375], [0.4974755469524368, 0.39966953181724185, 0.3926013347950764, 0.4065742827423504], [0.6279034412960182, 0.2882686047708628, 0.34595527413034044, 0.42730548731456264]], [[0.5917458246701541, 0.44075047905270137, 0.29330224086961787, 0.182709461505413], [0.5215469791240446, 0.18421368097737112, 0.1639603528416479, 0.9922448885541346], [0.17087302795043802, 0.9170873338599324, 0.4818605033266543, 0.969935965872987]]]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactOne_fromConstant_FunctionOnContactZero_rank0(self):
      """
      test integral of constant rank 0 Data on ReducedFunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactOne(self.domain)
      w_arg = FunctionOnContactZero(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = 3.727265e-01
      arg = Data(val,(),w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactOne_fromConstant_FunctionOnContactZero_rank1(self):
      """
      test integral of constant rank 1 Data on ReducedFunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactOne(self.domain)
      w_arg = FunctionOnContactZero(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([0.7421568149113168, 0.6265732892817735, 0.3354640988248244] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactOne_fromConstant_FunctionOnContactZero_rank2(self):
      """
      test integral of constant rank 2 Data on ReducedFunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactOne(self.domain)
      w_arg = FunctionOnContactZero(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[0.4769514811658414, 0.9474571789536855], [0.9867655624794561, 0.8208926144143623], [0.9694690369003416, 0.6312000964011596], [0.6655034419396955, 0.3573150121628542]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactOne_fromConstant_FunctionOnContactZero_rank3(self):
      """
      test integral of constant rank 3 Data on ReducedFunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactOne(self.domain)
      w_arg = FunctionOnContactZero(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[[0.5921084844987217, 0.33818086884669163, 0.5574762314179895, 0.7088671168474313], [0.9146005501676648, 0.6325016831456667, 0.567877745036038, 0.9900002400720543]], [[0.04388965916787324, 0.5212290550807717, 0.024946746005570675, 0.2612300560288914], [0.5882837615357712, 0.9652417094778742, 0.19019778824734812, 0.28497985558939865]]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 2, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactOne_fromConstant_FunctionOnContactZero_rank4(self):
      """
      test integral of constant rank 4 Data on ReducedFunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactOne(self.domain)
      w_arg = FunctionOnContactZero(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[[[0.8808136563283823, 0.018601229379717532, 0.8202417456363673, 0.9754605216642681], [0.38126524543891904, 0.28458719022766243, 0.4978783930867602, 0.8666849862856705], [0.31766654918010007, 0.7854752319943681, 0.30447824907407817, 0.07118942296979414]], [[0.2336784374018671, 0.3826013654863183, 0.783879061293046, 0.9268487149140385], [0.1965248965419064, 0.6498201285644261, 0.5459562622889068, 0.4845569184498856], [0.8832326217454061, 0.1581023014838181, 0.1671880316446831, 0.846928932532739]]], [[[0.8907762948873216, 0.6157581001546256, 0.9250238478070258, 0.4641311821449102], [0.3254590648697865, 0.6718041152301492, 0.09633104041519447, 0.7576459135734884], [0.8014521936270264, 0.4681834147657211, 0.3149365686269505, 0.6676476429551466]], [[0.5698181547840214, 0.9797846289123473, 0.4459048471545354, 0.5940194390847595], [0.5214185945163765, 0.3347189619492056, 0.7856740393414683, 0.5889363593124867], [0.5707926335538824, 0.6154113883510367, 0.8181009544602822, 0.19010053346663558]]], [[[0.1830831398525128, 0.06643569077826017, 0.17068811380830884, 0.8165813512331984], [0.5083991114384573, 0.02934928230827527, 0.3697877103517997, 0.7325621007035752], [0.7199774301277855, 0.18006716886347462, 0.2394528471732581, 0.21973138922097013]], [[0.09776249020703476, 0.19555736943060076, 0.8893096776647247, 0.941975600498078], [0.39997332127419616, 0.7839148752896193, 0.8923364293695838, 0.8070299534488198], [0.29373728165631974, 0.4652937402403884, 0.5476323143744777, 0.24892327237701584]]]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactOne_fromConstant_FunctionOnContactOne_rank0(self):
      """
      test integral of constant rank 0 Data on ReducedFunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactOne(self.domain)
      w_arg = FunctionOnContactOne(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = 7.249659e-01
      arg = Data(val,(),w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactOne_fromConstant_FunctionOnContactOne_rank1(self):
      """
      test integral of constant rank 1 Data on ReducedFunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactOne(self.domain)
      w_arg = FunctionOnContactOne(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([0.1265463937188822, 0.10424454612769407, 0.5033260406676746] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactOne_fromConstant_FunctionOnContactOne_rank2(self):
      """
      test integral of constant rank 2 Data on ReducedFunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactOne(self.domain)
      w_arg = FunctionOnContactOne(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[0.9193764935091446, 0.27462602942112013], [0.9024656988702725, 0.427745813186607], [0.955392081114512, 0.32553432549850503], [0.757465774427598, 0.7962248787084191]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactOne_fromConstant_FunctionOnContactOne_rank3(self):
      """
      test integral of constant rank 3 Data on ReducedFunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactOne(self.domain)
      w_arg = FunctionOnContactOne(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[[0.4825036114683723, 0.13390912007480404, 0.6824812540935531, 0.5365207709700576], [0.21660355828636513, 0.6592138675898028, 0.6854203248112931, 0.003803022905196296]], [[0.650555711618984, 0.4319890632817577, 0.5699191968447312, 0.39505927942858543], [0.6876750861713231, 0.03682112063442944, 0.5380332402118407, 0.231695887966476]]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 2, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onReducedFunctionOnContactOne_fromConstant_FunctionOnContactOne_rank4(self):
      """
      test integral of constant rank 4 Data on ReducedFunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_int = ReducedFunctionOnContactOne(self.domain)
      w_arg = FunctionOnContactOne(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[[[0.1745244010407937, 0.3329930860306627, 0.9290308872963652, 0.15506169081080823], [0.1281058198052022, 0.13316108197633714, 0.26781611967009045, 0.9611416046453726], [0.08492525145932539, 0.6771623085377849, 0.608205653182515, 0.664293587883617]], [[0.9406473307409655, 0.25207479117439957, 0.3343664343181043, 0.4135927503224247], [0.32073387657985497, 0.5554412185525595, 0.2737631829168148, 0.35750757007964895], [0.880410811922243, 0.7869245194537483, 0.6830786877229625, 0.34369781393509313]]], [[[0.4103689714181823, 0.03714184311739699, 0.9394789663377615, 0.9459995533178022], [0.8598761048065298, 0.690179062018881, 0.8044180848952837, 0.5884076843059306], [0.7788246258217072, 0.29686179813512825, 0.795910627057347, 0.2862774728905856]], [[0.63847213974572, 0.549820830216857, 0.6767550577245912, 0.13482534455462702], [0.5607659267588305, 0.018188114256662335, 0.18783379614820905, 0.39569089508248356], [0.43406046631514617, 0.5680301173195127, 0.9103627876485809, 0.13164971303228568]]], [[[0.6141307477860721, 0.1165093210285485, 0.2892425351933595, 0.8375796340282023], [0.08932196148275284, 0.6738339467766203, 0.8181612700184625, 0.667411393405352], [0.5958909608741012, 0.836077311805413, 0.7871188310796109, 0.5284945525928256]], [[0.775111876913437, 0.1681873102839322, 0.28196893046654325, 0.5284202970432588], [0.919532157608429, 0.13999598013981973, 0.48001714882212654, 0.46907569722310294], [0.9104363523208308, 0.8697304954805138, 0.5813268744378427, 0.009131927525251382]]]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onFunctionOnContactZero_fromConstant_Solution_rank0(self):
      """
      test integral of constant rank 0 Data on FunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_int = FunctionOnContactZero(self.domain)
      w_arg = Solution(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = 6.252582e-02
      arg = Data(val,(),w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onFunctionOnContactZero_fromConstant_Solution_rank1(self):
      """
      test integral of constant rank 1 Data on FunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_int = FunctionOnContactZero(self.domain)
      w_arg = Solution(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([0.18941038483969497, 0.5101817749538237, 0.7892153994352018] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onFunctionOnContactZero_fromConstant_Solution_rank2(self):
      """
      test integral of constant rank 2 Data on FunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_int = FunctionOnContactZero(self.domain)
      w_arg = Solution(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[0.8953751533088582, 0.47614751344708117], [0.7941752501244356, 0.24966648440109473], [0.49500811003819856, 0.35750226779809635], [0.43469458735970745, 0.9730533398448067]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onFunctionOnContactZero_fromConstant_Solution_rank3(self):
      """
      test integral of constant rank 3 Data on FunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_int = FunctionOnContactZero(self.domain)
      w_arg = Solution(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[[0.5562585074070735, 0.14157500663766487, 0.25141411313597206, 0.774010959457815], [0.26119435654146506, 0.974289221894602, 0.35092616130700494, 0.12633492704672333]], [[0.6969120247084782, 0.2288692916436772, 0.7288126758051586, 0.00016298040307560768], [0.02536896909913422, 0.472517450433666, 0.83309603067121, 0.25738358778869497]]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 2, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onFunctionOnContactZero_fromConstant_Solution_rank4(self):
      """
      test integral of constant rank 4 Data on FunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_int = FunctionOnContactZero(self.domain)
      w_arg = Solution(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[[[0.7996037557524834, 0.9564718589094513, 0.7238393212338419, 0.1841120772341035], [0.4180626721114411, 0.7792319479936879, 0.4796502168778223, 0.04330514302582256], [0.8177338411903335, 0.7335444472186222, 0.308278079500649, 0.9404714588437613]], [[0.8261470134411414, 0.5411895281024968, 0.15648610051268508, 0.7637995716394361], [0.11800845462524612, 0.2445936516520193, 0.07865771538020405, 0.25901873786439056], [0.04652754946489046, 0.46887422598832473, 0.40040518636268063, 0.6775538343377722]]], [[[0.07525233689800437, 0.7886012975165518, 0.7468767171241801, 0.4904726631645494], [0.8575876196431226, 0.7473848801014185, 0.9617263198219977, 0.5079864216218014], [0.00023009597373557789, 0.47428307106700685, 0.5898095425335739, 0.6640635783598545]], [[0.7660818862030147, 0.3353677576382589, 0.7317614924296681, 0.6058807507443603], [0.9806885902142718, 0.1793260360943092, 0.7901614994726793, 0.7089796079501444], [0.18608483869711967, 0.9805520005499045, 0.1588249197736351, 0.4280724836059]]], [[[0.04573908625302492, 0.02841346286938451, 0.17256618626421172, 0.6911993712416742], [0.9676776057389158, 0.5501769855581893, 0.7786896753013857, 0.2502198556301176], [0.7688426791382029, 0.9338657495886342, 0.6991273275643088, 0.13421063595763172]], [[0.6035872419038153, 0.311247405748445, 0.4053835356640053, 0.7352480481886007], [0.16725536773042726, 0.07423455327452477, 0.061765455427777005, 0.8947814168437433], [0.28940576237613336, 0.6441695001898311, 0.5958081560761066, 0.4471346931974288]]]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onFunctionOnContactZero_fromConstant_ReducedSolution_rank0(self):
      """
      test integral of constant rank 0 Data on FunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_int = FunctionOnContactZero(self.domain)
      w_arg = ReducedSolution(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = 1.189011e-01
      arg = Data(val,(),w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onFunctionOnContactZero_fromConstant_ReducedSolution_rank1(self):
      """
      test integral of constant rank 1 Data on FunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_int = FunctionOnContactZero(self.domain)
      w_arg = ReducedSolution(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([0.9610079028622128, 0.15360807327227455, 0.3510717406169743] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onFunctionOnContactZero_fromConstant_ReducedSolution_rank2(self):
      """
      test integral of constant rank 2 Data on FunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_int = FunctionOnContactZero(self.domain)
      w_arg = ReducedSolution(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[0.9446161802956483, 0.35689850341670215], [0.9907238161314846, 0.4197080196376063], [0.6132624335011285, 0.512136110285681], [0.9672951871966853, 0.4093100439631907]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onFunctionOnContactZero_fromConstant_ReducedSolution_rank3(self):
      """
      test integral of constant rank 3 Data on FunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_int = FunctionOnContactZero(self.domain)
      w_arg = ReducedSolution(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[[0.6123837823183088, 0.4054963398051865, 0.4563808369810384, 0.8740638404606298], [0.9334778643788213, 0.9678438669430208, 0.16388809607997923, 0.20581432001449873]], [[0.0389032957502885, 0.08025156236865638, 0.19070267487059356, 0.8832794386776329], [0.8031267949324358, 0.3838588728005533, 0.6802674882413772, 0.741813085203603]]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 2, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onFunctionOnContactZero_fromConstant_ReducedSolution_rank4(self):
      """
      test integral of constant rank 4 Data on FunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_int = FunctionOnContactZero(self.domain)
      w_arg = ReducedSolution(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[[[0.45703802355162015, 0.6107316032029978, 0.010476570558081377, 0.6132268578560847], [0.05037515273369664, 0.11002710584945774, 0.029304613174337857, 0.8248130257530087], [0.05283224659317143, 0.9172165326763596, 0.1030826998146287, 0.7120274665457664]], [[0.8131872346756213, 0.6428070867469238, 0.46875454378610304, 0.35458056503277946], [0.46336541920228225, 0.5087874684803969, 0.16640630939342727, 0.31169274257295754], [0.15822548780986578, 0.7773757686213744, 0.4522451054428872, 0.2693923152550117]]], [[[0.8931617650173607, 0.41531916037372474, 0.6105196888557188, 0.3208536617079114], [0.45982971277723406, 0.6137981932117671, 0.5840594544474969, 0.46226146672835633], [0.959368880333653, 0.41285142281232934, 0.8579643493368686, 0.11308529464334782]], [[0.47429940455441766, 0.40742365136268177, 0.9217188030068716, 0.6556198535518599], [0.7797632728317958, 0.013028689185771825, 0.1256004510417339, 0.5331626981763898], [0.6996013332952348, 0.16117382241930756, 0.643207669909556, 0.9486673382907617]]], [[[0.5371908314153044, 0.42467868717744495, 0.46218606947237906, 0.3394559693286957], [0.48575168396935386, 0.7704040285382424, 0.6966773614005307, 0.3800963820682497], [0.7535285323577607, 0.38074640485298217, 0.15481682353540438, 0.2773145067282272]], [[0.3400580309408633, 0.8606520600543534, 0.10895275343247779, 0.7984786414210868], [0.9320194798082381, 0.44110804214264854, 0.621985738303941, 0.20132359483259654], [0.8274467271444378, 0.47963831977075977, 0.7888594778123779, 0.3688629501566171]]]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onFunctionOnContactZero_fromConstant_ContinuousFunction_rank0(self):
      """
      test integral of constant rank 0 Data on FunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_int = FunctionOnContactZero(self.domain)
      w_arg = ContinuousFunction(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = 1.622218e-01
      arg = Data(val,(),w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onFunctionOnContactZero_fromConstant_ContinuousFunction_rank1(self):
      """
      test integral of constant rank 1 Data on FunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_int = FunctionOnContactZero(self.domain)
      w_arg = ContinuousFunction(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([0.3489836790520975, 0.9821562386993342, 0.5258903365351186] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onFunctionOnContactZero_fromConstant_ContinuousFunction_rank2(self):
      """
      test integral of constant rank 2 Data on FunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_int = FunctionOnContactZero(self.domain)
      w_arg = ContinuousFunction(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[0.3833206685898283, 0.8633950542053122], [0.35292440435044803, 0.10442481655226721], [0.5397981905636176, 0.9239149755673478], [0.19049130887809518, 0.9136706968847405]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onFunctionOnContactZero_fromConstant_ContinuousFunction_rank3(self):
      """
      test integral of constant rank 3 Data on FunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_int = FunctionOnContactZero(self.domain)
      w_arg = ContinuousFunction(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[[0.9317049672697743, 0.08941034796455327, 0.2570801880563319, 0.14736521080508413], [0.12687940900574635, 0.035771631938525617, 0.34535539292448636, 0.7587462252725657]], [[0.5475900419342478, 0.9485504898515629, 0.41270626996103554, 0.26147715065162014], [0.7812868779118942, 0.2912155899727582, 0.5527006074224675, 0.7559189816164062]]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 2, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onFunctionOnContactZero_fromConstant_ContinuousFunction_rank4(self):
      """
      test integral of constant rank 4 Data on FunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_int = FunctionOnContactZero(self.domain)
      w_arg = ContinuousFunction(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[[[0.024453201927246426, 0.17211677277909643, 0.5291145691354421, 0.40325995242347545], [0.5790994546860493, 0.35688115494248307, 0.4263407575957018, 0.6064818394380879], [0.3797057139242519, 0.13778290937196702, 0.8815763245484906, 0.4797410117865848]], [[0.7687583129595299, 0.4383120003617704, 0.6320541753347193, 0.0741227835458087], [0.12273006918646656, 0.8937186387983237, 0.7111916389987614, 0.7830643857208363], [0.025497341729896927, 0.5114288277799586, 0.33023147432659106, 0.8367774040910714]]], [[[0.5081612436166563, 0.19991079400918765, 0.9321150455169117, 0.6279359867321455], [0.6438677399168641, 0.3616098088664319, 0.20032761225692564, 0.38625035268623986], [0.22140562750260206, 0.6356789391921743, 0.29816369566574885, 0.07062613251112015]], [[0.5292823829319386, 0.2644257224162073, 0.10579197987238709, 0.06239941765314616], [0.004180840108474304, 0.40698081412649156, 0.005784247667595777, 0.36256135658166244], [0.12526672895822732, 0.47621103322417613, 0.9321039201423572, 0.3746444120687068]]], [[[0.678786506722394, 0.7113083553542261, 0.025795472421411447, 0.9392172010650739], [0.3611030494140073, 0.17272941746224324, 0.49342750869801355, 0.24921341851457857], [0.6853600556997634, 0.06500470897985289, 0.8898914748799315, 0.5073855482562482]], [[0.29096552892935124, 0.7157387110727949, 0.06170342210076274, 0.9544365651312128], [0.8037102947791191, 0.5277426563041598, 0.39612339541029007, 0.27988486967251447], [0.8454726036830532, 0.7417577557639863, 0.8541088833268192, 0.8346637732706557]]]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onFunctionOnContactZero_fromConstant_ReducedContinuousFunction_rank0(self):
      """
      test integral of constant rank 0 Data on FunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_int = FunctionOnContactZero(self.domain)
      w_arg = ReducedContinuousFunction(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = 3.691760e-01
      arg = Data(val,(),w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onFunctionOnContactZero_fromConstant_ReducedContinuousFunction_rank1(self):
      """
      test integral of constant rank 1 Data on FunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_int = FunctionOnContactZero(self.domain)
      w_arg = ReducedContinuousFunction(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([0.41643220536719894, 0.041431068878988064, 0.2493888957051118] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onFunctionOnContactZero_fromConstant_ReducedContinuousFunction_rank2(self):
      """
      test integral of constant rank 2 Data on FunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_int = FunctionOnContactZero(self.domain)
      w_arg = ReducedContinuousFunction(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[0.290600119863196, 0.2811569823337098], [0.13054482690493696, 0.024893493737391026], [0.15648310559583734, 0.24920064498999228], [0.9830546281033957, 0.5459155096211913]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onFunctionOnContactZero_fromConstant_ReducedContinuousFunction_rank3(self):
      """
      test integral of constant rank 3 Data on FunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_int = FunctionOnContactZero(self.domain)
      w_arg = ReducedContinuousFunction(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[[0.9648719559695005, 0.3684035567084364, 0.7960269396420092, 0.6158265735329564], [0.5714871342516227, 0.7703152712667152, 0.2106300794757926, 0.15957412866420184]], [[0.07571173929103658, 0.11387757937524556, 0.10410433319596568, 0.12482438320702216], [0.8746143561786558, 0.5823832437428643, 0.7351250095305907, 0.3485457883339419]]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 2, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onFunctionOnContactZero_fromConstant_ReducedContinuousFunction_rank4(self):
      """
      test integral of constant rank 4 Data on FunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_int = FunctionOnContactZero(self.domain)
      w_arg = ReducedContinuousFunction(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[[[0.12633449869893632, 0.16025545249576645, 0.32694415489913986, 0.36136690830324136], [0.9775590080677335, 0.9883441839901886, 0.30702521614686507, 0.21476784819003514], [0.45081787474871804, 0.890080705141968, 0.7111755839539737, 0.32881018290778086]], [[0.02464751931441156, 0.2604912212258014, 0.33344983215775836, 0.05447197983099339], [0.5450970941071034, 0.5828294070074191, 0.31912204036165703, 0.6405599714368841], [0.0698138086675093, 0.9358870718297323, 0.8113885136255008, 0.9793822859290824]]], [[[0.724074329912394, 0.7201921889955495, 0.5429782353129716, 0.41679075102997487], [0.04002571923819642, 0.8134243618137212, 0.9653369834125513, 0.39825479012153586], [0.648050583880924, 0.7066085053459181, 0.28917701509773597, 0.6855221833186605]], [[0.15368958549957223, 0.5154551247301358, 0.23120337436952476, 0.5662705420977866], [0.724170987911637, 0.6682761455324723, 0.7634240752014825, 0.29360561813587227], [0.2241053783655278, 0.984471833700543, 0.4738321578703193, 0.8093386185034039]]], [[[0.4688934151436225, 0.9861309510489927, 0.4825713607240394, 0.36653693284846334], [0.10272605918367239, 0.8928125718376746, 0.43637706753307404, 0.9428197149282543], [0.9234493411806026, 0.4465701842938603, 0.1944992431978666, 0.49996423420683533]], [[0.7220440010140734, 0.19733260800445862, 0.5792031813105083, 0.23290635819340622], [0.6199339167244267, 0.8813537753536951, 0.10773318283098954, 0.9700310733003268], [0.008297702138917784, 0.2762572168041715, 0.7838619562152003, 0.4475963844645179]]]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onFunctionOnContactZero_fromConstant_FunctionOnContactZero_rank0(self):
      """
      test integral of constant rank 0 Data on FunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_int = FunctionOnContactZero(self.domain)
      w_arg = FunctionOnContactZero(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = 7.660512e-01
      arg = Data(val,(),w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onFunctionOnContactZero_fromConstant_FunctionOnContactZero_rank1(self):
      """
      test integral of constant rank 1 Data on FunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_int = FunctionOnContactZero(self.domain)
      w_arg = FunctionOnContactZero(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([0.5504539543505367, 0.3547519854685537, 0.32408937550554895] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onFunctionOnContactZero_fromConstant_FunctionOnContactZero_rank2(self):
      """
      test integral of constant rank 2 Data on FunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_int = FunctionOnContactZero(self.domain)
      w_arg = FunctionOnContactZero(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[0.944168967212026, 0.5194393468212852], [0.8102718147433784, 0.8831549542278619], [0.9310965126198142, 0.957193664961087], [0.2891284710073757, 0.7832301037178383]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onFunctionOnContactZero_fromConstant_FunctionOnContactZero_rank3(self):
      """
      test integral of constant rank 3 Data on FunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_int = FunctionOnContactZero(self.domain)
      w_arg = FunctionOnContactZero(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[[0.15240432781570712, 0.8543139224471626, 0.5589243772124808, 0.5170241203564816], [0.9608975949031109, 0.3159038871641633, 0.22427011959019172, 0.6743452835336753]], [[0.07965710284408856, 0.9527658399348778, 0.9993844714564183, 0.4904237766078674], [0.03415770640681104, 0.1937692811162669, 0.7743125899174228, 0.5415310311211089]]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 2, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onFunctionOnContactZero_fromConstant_FunctionOnContactZero_rank4(self):
      """
      test integral of constant rank 4 Data on FunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_int = FunctionOnContactZero(self.domain)
      w_arg = FunctionOnContactZero(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[[[0.22517194005755892, 0.2800695266179398, 0.374737598867518, 0.8340611256567968], [0.5778477686315274, 0.004993633182563628, 0.3485400662107424, 0.14839600574950051], [0.6111309658336569, 0.8057337029598981, 0.41256400078047584, 0.9241496995880142]], [[0.5223445832579124, 0.7565380966255216, 0.430783932986465, 0.03707673490902441], [0.3987631431893919, 0.5194194967579775, 0.7124022398657499, 0.8348537352669492], [0.7269412288329141, 0.27613824641362406, 0.18468119880195766, 0.7168988938837504]]], [[[0.8480484838321047, 0.705116251162059, 0.2266639122961912, 0.8804879150499225], [0.23267247630195054, 0.42785285084254954, 0.11095052023671592, 0.4271938088083096], [0.8247953723971633, 0.2316955226399201, 0.24859979155272882, 0.5982996322783886]], [[0.8681412582403422, 0.0500016494359139, 0.5989596908785256, 0.7290287180103003], [0.8404516469941168, 0.26034422887367825, 0.7021299310691201, 0.16614259113198537], [0.6948807854773501, 0.8577558559560988, 0.9557485425223371, 0.23337077151037988]]], [[[0.1974340707814869, 0.9134379899647826, 0.6264613832552848, 0.5292910227270656], [0.39157967993934506, 0.6595327412689946, 0.360370051578181, 0.6369934958482596], [0.5441209871785585, 0.7262280815294339, 0.22496039744001184, 0.5176985029233008]], [[0.12425750543231973, 0.24835181122516203, 0.8276877711519136, 0.8311777056060491], [0.5295851771859572, 0.33050473018952775, 0.14023743714858372, 0.20111987170130796], [0.2581188468514469, 0.8973969188319553, 0.46614178948181506, 0.23002462393091927]]]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onFunctionOnContactZero_fromConstant_FunctionOnContactOne_rank0(self):
      """
      test integral of constant rank 0 Data on FunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_int = FunctionOnContactZero(self.domain)
      w_arg = FunctionOnContactOne(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = 3.047886e-02
      arg = Data(val,(),w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onFunctionOnContactZero_fromConstant_FunctionOnContactOne_rank1(self):
      """
      test integral of constant rank 1 Data on FunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_int = FunctionOnContactZero(self.domain)
      w_arg = FunctionOnContactOne(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([0.10016633693020527, 0.8990504330676673, 0.704438811289166] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onFunctionOnContactZero_fromConstant_FunctionOnContactOne_rank2(self):
      """
      test integral of constant rank 2 Data on FunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_int = FunctionOnContactZero(self.domain)
      w_arg = FunctionOnContactOne(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[0.7746716737141279, 0.8405421722082403], [0.6154645953753503, 0.31156470407921366], [0.5411903145369927, 0.47854283519284835], [0.8272026479403588, 0.0038058905588590486]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onFunctionOnContactZero_fromConstant_FunctionOnContactOne_rank3(self):
      """
      test integral of constant rank 3 Data on FunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_int = FunctionOnContactZero(self.domain)
      w_arg = FunctionOnContactOne(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[[0.06599279936634961, 0.8258562979243965, 0.7493440156737361, 0.1521173893969502], [0.840663205941576, 0.4151316184238327, 0.9648853718368803, 0.12308368215615484]], [[0.6444009698890206, 0.2893710066944951, 0.19443099725672963, 0.2767263820165854], [0.8878801745077394, 0.6854264681192274, 0.7966845518851969, 0.581289383381096]]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 2, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onFunctionOnContactZero_fromConstant_FunctionOnContactOne_rank4(self):
      """
      test integral of constant rank 4 Data on FunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_int = FunctionOnContactZero(self.domain)
      w_arg = FunctionOnContactOne(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[[[0.10699256937957913, 0.22343689625287744, 0.6919671181011046, 0.5329540816350643], [0.5740498600967272, 0.7941892280515279, 0.6751096581763518, 0.4249688500188261], [0.5374439542374496, 0.10795060907312648, 0.22011432702794076, 0.09046031407124422]], [[0.29335074000317174, 0.8957386738705885, 0.7668447233474855, 0.6705888447587168], [0.8684574729135286, 0.7569601099071328, 0.5314761057527467, 0.0029549406011877277], [0.13223835449122545, 0.7568776021494857, 0.0176825150713793, 0.28378281012168627]]], [[[0.8230776424927826, 0.4075458509958041, 0.6239268927803635, 0.8460405075920091], [0.201136111886966, 0.571670973867131, 0.7778349539966918, 0.6686108750008829], [0.7582168089981769, 0.1651354488387463, 0.7094447949561743, 0.2924175416553938]], [[0.577498441814413, 0.9581278566171317, 0.7524052144002791, 0.3281086261015397], [0.26864425556747507, 0.18800504828021247, 0.42694438852140204, 0.0715885411407633], [0.4822093736886903, 0.9468886835927811, 0.5678188116210047, 0.9537985726622287]]], [[[0.6662752576017211, 0.23440771091139168, 0.9174696392574011, 0.7663172898039327], [0.7084227672578721, 0.2737026734106154, 0.704506979909672, 0.19955061276183939], [0.8020344279921677, 0.6796166425057897, 0.5426544675783702, 0.15509678526452908]], [[0.730608250349834, 0.6735003755172071, 0.7540565796196795, 0.881322638169764], [0.017330706192941192, 0.5626032779867876, 0.27152458360354415, 0.6464488335722546], [0.5777581341066254, 0.23834774287947857, 0.7353728300339369, 0.04359970842691219]]]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onFunctionOnContactOne_fromConstant_Solution_rank0(self):
      """
      test integral of constant rank 0 Data on FunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_int = FunctionOnContactOne(self.domain)
      w_arg = Solution(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = 8.635209e-01
      arg = Data(val,(),w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onFunctionOnContactOne_fromConstant_Solution_rank1(self):
      """
      test integral of constant rank 1 Data on FunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_int = FunctionOnContactOne(self.domain)
      w_arg = Solution(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([0.6559633097272591, 0.303581913194529, 0.279639644528916] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onFunctionOnContactOne_fromConstant_Solution_rank2(self):
      """
      test integral of constant rank 2 Data on FunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_int = FunctionOnContactOne(self.domain)
      w_arg = Solution(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[0.7310652943921611, 0.10823411136782435], [0.6931402095986033, 0.1283541616559084], [0.5969952533207404, 0.3333813189432362], [0.620949262987432, 0.12872889936071497]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onFunctionOnContactOne_fromConstant_Solution_rank3(self):
      """
      test integral of constant rank 3 Data on FunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_int = FunctionOnContactOne(self.domain)
      w_arg = Solution(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[[0.7150280284562628, 0.8624480090101116, 0.8047376280355353, 0.6769511153867326], [0.6596729535822485, 0.7689401374571833, 0.3390023252785548, 0.36636928142738157]], [[0.22052258318101414, 0.4867207694571324, 0.7090854255588297, 0.8890936533177319], [0.5262080688726379, 0.9040877981112247, 0.4018857890307901, 0.15665437772789292]]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 2, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onFunctionOnContactOne_fromConstant_Solution_rank4(self):
      """
      test integral of constant rank 4 Data on FunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_int = FunctionOnContactOne(self.domain)
      w_arg = Solution(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[[[0.2552029057661205, 0.5989330087513955, 0.6512797134952418, 0.3063231742727046], [0.6252231009854976, 0.9024417362995216, 0.28153373165683215, 0.8000187322668615], [0.8229273643005427, 0.9870283521368438, 0.7932802008249986, 0.6879820501936008]], [[0.3621657599002577, 0.4353913912825754, 0.93170426175885, 0.16234565826154568], [0.2526245451494832, 0.9265463638908421, 0.012647318590888634, 0.16500769931116366], [0.31411925002052155, 0.012881656310726575, 0.42891476081551283, 0.12108804725726463]]], [[[0.8799826320126851, 0.3929730628636324, 0.8833703993123817, 0.5892077831571775], [0.5086240846820094, 0.3040947575364398, 0.1315673898464771, 0.7562793926220291], [0.9786190143251992, 0.8821366894642204, 0.8704945106044487, 0.8723990278653156]], [[0.10687980998126856, 0.32267061888605464, 0.0766947006574562, 0.6056145035811329], [0.9480058251323819, 0.30744922512366657, 0.8981752915273502, 0.8982700078822757], [0.6556604211593893, 0.6836623186419879, 0.18124496612756935, 0.5324879384553793]]], [[[0.8322424995289417, 0.6906698250081009, 0.4811375190239209, 0.44520541940798497], [0.6155426932673826, 0.005024719078883466, 0.03916735122391757, 0.605179990390979], [0.4014052684935492, 0.5952513217533352, 0.5603566874451659, 0.287376177471076]], [[0.8747609502379454, 0.5194677134873333, 0.9303647087116543, 0.1190645253069168], [0.5346433395708554, 0.44442794154472953, 0.9501420477424819, 0.19467466474271888], [0.6259332085141268, 0.19056297050408877, 0.46269059030117987, 0.8933885016837446]]]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onFunctionOnContactOne_fromConstant_ReducedSolution_rank0(self):
      """
      test integral of constant rank 0 Data on FunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_int = FunctionOnContactOne(self.domain)
      w_arg = ReducedSolution(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = 4.382558e-01
      arg = Data(val,(),w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onFunctionOnContactOne_fromConstant_ReducedSolution_rank1(self):
      """
      test integral of constant rank 1 Data on FunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_int = FunctionOnContactOne(self.domain)
      w_arg = ReducedSolution(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([0.9570617767635065, 0.36776097166286903, 0.21965920614259926] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onFunctionOnContactOne_fromConstant_ReducedSolution_rank2(self):
      """
      test integral of constant rank 2 Data on FunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_int = FunctionOnContactOne(self.domain)
      w_arg = ReducedSolution(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[0.9255137246009032, 0.3154442979318731], [0.7623308489103543, 0.6849068325259188], [0.09953717372573145, 0.056625520537969654], [0.9278006228115179, 0.45065003235929535]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onFunctionOnContactOne_fromConstant_ReducedSolution_rank3(self):
      """
      test integral of constant rank 3 Data on FunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_int = FunctionOnContactOne(self.domain)
      w_arg = ReducedSolution(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[[0.15754745222837552, 0.34097543821428544, 0.41332619000691995, 0.28975553137614574], [0.913495416835855, 0.8191660516535821, 0.9556462375432297, 0.043208267035229064]], [[0.11034858903540146, 0.32143019958360863, 0.9672022293753719, 0.30128484868758654], [0.41549191162971655, 0.4485171535377709, 0.8232647777291128, 0.6958676039224349]]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 2, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onFunctionOnContactOne_fromConstant_ReducedSolution_rank4(self):
      """
      test integral of constant rank 4 Data on FunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_int = FunctionOnContactOne(self.domain)
      w_arg = ReducedSolution(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[[[0.7083144994428189, 0.06288671501277387, 0.868886531756736, 0.39360824993892896], [0.14984237728791416, 0.5240555383362591, 0.6101010069949732, 0.8169408708167443], [0.5862739402264583, 0.21736420706813775, 0.11675052576172773, 0.5599423055979807]], [[0.8046393402589023, 0.4387212349785794, 0.8818965695096775, 0.6169183262871104], [0.7922122167881446, 0.41486256102084274, 0.5292675019383261, 0.8910457899826698], [0.2163687891938968, 0.737533717077667, 0.044663265571717736, 0.5700878093102094]]], [[[0.6907294025309161, 0.4004380069849166, 0.12136679941599571, 0.9454753580338768], [0.28959642702422317, 0.5757683525447995, 0.4591791827787698, 0.9067121397086528], [0.9983429611720162, 0.5893140152169142, 0.9253505119822287, 0.4194307896116566]], [[0.03331484273833951, 0.13143323591322686, 0.06566646103970564, 0.677024814935137], [0.6024687302587642, 0.09596918739784333, 0.9329275238263706, 0.33562934440836156], [0.1789378229772396, 0.20799981521094313, 0.5464505356602939, 0.5217151539476265]]], [[[0.07322614078300227, 0.29377995506101606, 0.9856436973236365, 0.859007324885978], [0.817401693002145, 0.9446012127764656, 0.7187703204581282, 0.7046805137925216], [0.4580217911400908, 0.40454008503963945, 0.7056286436948207, 0.3229881571760119]], [[0.30765362000939434, 0.8863066076875503, 0.22948693354027716, 0.5186374854723428], [0.8550033784877045, 0.04694310062518425, 0.5563620353342265, 0.06344121610781195], [0.7014942989140549, 0.6452465032726077, 0.09488801018543824, 0.1297571483290637]]]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onFunctionOnContactOne_fromConstant_ContinuousFunction_rank0(self):
      """
      test integral of constant rank 0 Data on FunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_int = FunctionOnContactOne(self.domain)
      w_arg = ContinuousFunction(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = 2.961034e-01
      arg = Data(val,(),w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onFunctionOnContactOne_fromConstant_ContinuousFunction_rank1(self):
      """
      test integral of constant rank 1 Data on FunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_int = FunctionOnContactOne(self.domain)
      w_arg = ContinuousFunction(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([0.4769215339282361, 0.13284755924441694, 0.011313279351570893] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onFunctionOnContactOne_fromConstant_ContinuousFunction_rank2(self):
      """
      test integral of constant rank 2 Data on FunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_int = FunctionOnContactOne(self.domain)
      w_arg = ContinuousFunction(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[0.815988313996075, 0.3586385151967657], [0.012631115818845351, 0.21902624543008864], [0.38972686758144304, 0.5389125168364511], [0.6489322310748054, 0.6793682404454512]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onFunctionOnContactOne_fromConstant_ContinuousFunction_rank3(self):
      """
      test integral of constant rank 3 Data on FunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_int = FunctionOnContactOne(self.domain)
      w_arg = ContinuousFunction(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[[0.1311146667583517, 0.9388550232545688, 0.40547699727725584, 0.7876397319176007], [0.9472970999584749, 0.8051385465709553, 0.07441132808096385, 0.3735226868650944]], [[0.6776877398924762, 0.6755643730703971, 0.526986405313971, 0.13048727274457095], [0.13590399476391557, 0.025258057745970786, 0.43654678292036664, 0.25062679491623074]]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 2, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onFunctionOnContactOne_fromConstant_ContinuousFunction_rank4(self):
      """
      test integral of constant rank 4 Data on FunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_int = FunctionOnContactOne(self.domain)
      w_arg = ContinuousFunction(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[[[0.1716820400341179, 0.6748394844499948, 0.4491327055643459, 0.28435142863715523], [0.8101614418567336, 0.05572377003884399, 0.24464478229237308, 0.7825617087303303], [0.4589670412766522, 0.31504297209523413, 0.814179484291148, 0.8346532682809373]], [[0.2708785891647635, 0.22688620364500267, 0.5369958726189362, 0.6009463726379835], [0.5375268375560918, 0.897621279413471, 0.8425671763165552, 0.3553226920347645], [0.6279287625905328, 0.4824699112091495, 0.8757802743526434, 0.3063189501985131]]], [[[0.18363138745121643, 7.852330646296313e-05, 0.23850592397323567, 0.8113745860509989], [0.5522490865792813, 0.7116901855106719, 0.36025152904154645, 0.6973690598553881], [0.7237158921438394, 0.7688208066922136, 0.039036084201188315, 0.9507743633547633]], [[0.39727179856289974, 0.704126460509093, 0.905595244595797, 0.5975861063785006], [0.8455403047220947, 0.7805267419852018, 0.3770478632510591, 0.7554354112618592], [0.005477180443195007, 0.4406334691148014, 0.2561621805351063, 0.16815027039436925]]], [[[0.6388952121654426, 0.9764369432309027, 0.9843697545440598, 0.2974471821646514], [0.6134427760201036, 0.07492907066807086, 0.6386524432712448, 0.2613204623677], [0.14176797908625272, 0.21081652982508303, 0.3150821292834699, 0.318813713302012]], [[0.41300963490262166, 0.10282640196186565, 0.2703286372490954, 0.3296317112071223], [0.9821366261082973, 0.6872820106037308, 0.5061425643939246, 0.5253713766460101], [0.09717366555099993, 0.38068964538571715, 0.878860543398503, 0.9996932577150508]]]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onFunctionOnContactOne_fromConstant_ReducedContinuousFunction_rank0(self):
      """
      test integral of constant rank 0 Data on FunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_int = FunctionOnContactOne(self.domain)
      w_arg = ReducedContinuousFunction(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = 7.161462e-01
      arg = Data(val,(),w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onFunctionOnContactOne_fromConstant_ReducedContinuousFunction_rank1(self):
      """
      test integral of constant rank 1 Data on FunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_int = FunctionOnContactOne(self.domain)
      w_arg = ReducedContinuousFunction(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([0.716629598519305, 0.7036451730655328, 0.9861794634046476] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onFunctionOnContactOne_fromConstant_ReducedContinuousFunction_rank2(self):
      """
      test integral of constant rank 2 Data on FunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_int = FunctionOnContactOne(self.domain)
      w_arg = ReducedContinuousFunction(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[0.4282795172215408, 0.10302473008577073], [0.2959530117486492, 0.0952278841452775], [0.45835136644866226, 0.7104476963578635], [0.698408401607586, 0.4384944370649585]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onFunctionOnContactOne_fromConstant_ReducedContinuousFunction_rank3(self):
      """
      test integral of constant rank 3 Data on FunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_int = FunctionOnContactOne(self.domain)
      w_arg = ReducedContinuousFunction(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[[0.4262029389621972, 0.23743318890428644, 0.05470034731878326, 0.5370389699665769], [0.11418571119482324, 0.7902205125058088, 0.7864468517746247, 0.6669227319501526]], [[0.6717642785780955, 0.5767321195239039, 0.4777086993548356, 0.756435489192914], [0.18476969214453942, 0.9593891747421411, 0.583792477934532, 0.97189082791806]]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 2, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onFunctionOnContactOne_fromConstant_ReducedContinuousFunction_rank4(self):
      """
      test integral of constant rank 4 Data on FunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_int = FunctionOnContactOne(self.domain)
      w_arg = ReducedContinuousFunction(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[[[0.09577685495310329, 0.1948250099739781, 0.7597409977266657, 0.004308769767824128], [0.6926111624180952, 0.235943069902111, 0.13599280606751196, 0.9964034498477996], [0.83430203110643, 0.6095291506170687, 0.19973085168791838, 0.1333733517561866]], [[0.06474965889762929, 0.852407655730855, 0.28143830510221335, 0.8093907121105646], [0.4491873575420475, 0.9886147504690581, 0.2856443297853133, 0.7081251599104883], [0.07683043940095202, 0.3594858718532451, 0.7151575678248232, 0.009405487261270662]]], [[[0.2967945601508234, 0.9263865403807459, 0.8442700467506427, 0.4456193835944363], [0.6209960470688396, 0.5557979282423033, 0.5594937206181152, 0.7282890768885019], [0.18217057667511194, 0.7857727187652334, 0.25970289063506846, 0.6359544252996424]], [[0.3008954412412288, 0.7323568372443957, 0.1869252562036593, 0.6827923231976256], [0.48502854036764886, 0.05603369434890049, 0.08942109548498012, 0.5014710368882586], [0.37099688958274957, 0.2954930234462, 0.46301705454434616, 0.9967781462706649]]], [[[0.004939324217456242, 0.4986414251462764, 0.25891375353568313, 0.5525510296164893], [0.4610763347484814, 0.7078624791415129, 0.5450009497081078, 0.12328640868204155], [0.6385320744699953, 0.06319217349120887, 0.19876079073474606, 0.9163392980105555]], [[0.019693437999968477, 0.1446787876300989, 0.4591779981609282, 0.4539218512097577], [0.1968981205264002, 0.6435417776316797, 0.9456581528399156, 0.863322116822602], [0.6150277755356808, 0.7346805993214411, 0.606167122843953, 0.37401222142897494]]]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onFunctionOnContactOne_fromConstant_FunctionOnContactZero_rank0(self):
      """
      test integral of constant rank 0 Data on FunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_int = FunctionOnContactOne(self.domain)
      w_arg = FunctionOnContactZero(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = 2.352467e-01
      arg = Data(val,(),w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onFunctionOnContactOne_fromConstant_FunctionOnContactZero_rank1(self):
      """
      test integral of constant rank 1 Data on FunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_int = FunctionOnContactOne(self.domain)
      w_arg = FunctionOnContactZero(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([0.0017245497114272146, 0.04942560566398435, 0.9142885222615725] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onFunctionOnContactOne_fromConstant_FunctionOnContactZero_rank2(self):
      """
      test integral of constant rank 2 Data on FunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_int = FunctionOnContactOne(self.domain)
      w_arg = FunctionOnContactZero(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[0.5116501236507419, 0.8599238807686773], [0.7389687559291916, 0.5346477336306447], [0.23173198626616665, 0.8427275177826711], [0.8647247152559212, 0.391603882187607]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onFunctionOnContactOne_fromConstant_FunctionOnContactZero_rank3(self):
      """
      test integral of constant rank 3 Data on FunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_int = FunctionOnContactOne(self.domain)
      w_arg = FunctionOnContactZero(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[[0.7365695572183082, 0.6639203607792771, 0.0032966203665132365, 0.2770933311857531], [0.9709763017469608, 0.2779826230972915, 0.7913048834896851, 0.6160046029404084]], [[0.025983388586837797, 0.2976952562479973, 0.6803930612266367, 0.5437175380603391], [0.08339609559303973, 0.693256269655246, 0.7885306679729965, 0.4733032468093127]]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 2, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onFunctionOnContactOne_fromConstant_FunctionOnContactZero_rank4(self):
      """
      test integral of constant rank 4 Data on FunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_int = FunctionOnContactOne(self.domain)
      w_arg = FunctionOnContactZero(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[[[0.2068069276136555, 0.4370745313602713, 0.26701929538644775, 0.24705795759028992], [0.6957995927849258, 0.6769374299999227, 0.8290363855974663, 0.8096285379324201], [0.6617077287675432, 0.7249191945888279, 0.5134779146053434, 0.7876574420447815]], [[0.35943801300319167, 0.48567121750499265, 0.37459218515081827, 0.361584068774382], [0.43910521928006174, 0.050522159574661996, 0.06805302738890029, 0.7106854939219162], [0.6019880105181219, 0.8025842319888481, 0.03207943511522671, 0.5965614769240216]]], [[[0.6729435775380143, 0.8823865812651432, 0.8161469194392513, 0.40975969799617895], [0.8594520319198373, 0.630221807126522, 0.7194765605397677, 0.775949555785766], [0.11222067213918319, 0.20479819740634475, 0.9246222068195888, 0.01778678018887392]], [[0.7819779471833652, 0.2759569895711007, 0.6059880672846061, 0.6911786094295835], [0.8476116814475815, 0.6229113665515917, 0.11250691015315428, 0.8417488376890832], [0.06281103966612844, 0.5708629513952843, 0.9528715153716371, 0.37605560884405487]]], [[[0.029125893983584783, 0.39988349258279254, 0.05479262202766799, 0.5628788995070214], [0.10108760652694782, 0.0944257507011681, 0.14948701249487173, 0.18191631716625867], [0.7270073417955892, 0.4266504910505764, 0.7048888003621471, 0.26651906086181887]], [[0.1034167300903478, 0.8962744854922687, 0.7475750699166294, 0.5377942586571324], [0.71756896364301, 0.8978170125803404, 0.2256559641465733, 0.7148162335948848], [0.9419656909770407, 0.6579344792045677, 0.749742809639675, 0.9183021013341917]]]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onFunctionOnContactOne_fromConstant_FunctionOnContactOne_rank0(self):
      """
      test integral of constant rank 0 Data on FunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_int = FunctionOnContactOne(self.domain)
      w_arg = FunctionOnContactOne(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = 6.252675e-01
      arg = Data(val,(),w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onFunctionOnContactOne_fromConstant_FunctionOnContactOne_rank1(self):
      """
      test integral of constant rank 1 Data on FunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_int = FunctionOnContactOne(self.domain)
      w_arg = FunctionOnContactOne(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([0.43204481575872433, 0.6863699605733442, 0.008326101442010514] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onFunctionOnContactOne_fromConstant_FunctionOnContactOne_rank2(self):
      """
      test integral of constant rank 2 Data on FunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_int = FunctionOnContactOne(self.domain)
      w_arg = FunctionOnContactOne(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[0.27561827238976544, 0.48906979553628993], [0.01380139145107595, 0.3864028343610827], [0.38910107521394643, 0.09532676827658726], [0.3554720216487369, 0.11405462530925037]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onFunctionOnContactOne_fromConstant_FunctionOnContactOne_rank3(self):
      """
      test integral of constant rank 3 Data on FunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_int = FunctionOnContactOne(self.domain)
      w_arg = FunctionOnContactOne(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[[0.8783025582086582, 0.08391515671332994, 0.5410937740388087, 0.6942188319282586], [0.6094621984282298, 0.8403735050591061, 0.5452855259826072, 0.10256897606511994]], [[0.36540533126603647, 0.7982230632811729, 0.43816942238842116, 0.10195021426685147], [0.4748775408262177, 0.972276523013585, 0.7917776163489717, 0.24538490961691828]]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 2, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   def test_integrate_onFunctionOnContactOne_fromConstant_FunctionOnContactOne_rank4(self):
      """
      test integral of constant rank 4 Data on FunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_int = FunctionOnContactOne(self.domain)
      w_arg = FunctionOnContactOne(self.domain)
      m = [ inf(self.domain.getX()[d]) for d in range(DIM) ]
      M = [ sup(self.domain.getX()[d]) for d in range(DIM) ]
      if DIM == 2 :
          V = (M[0]-m[0]) 
      else:
          V = (M[0]-m[0]) * (M[1]-m[1])
      val = numpy.array([[[[0.6152134345296085, 0.400803041581136, 0.8340499673598227, 0.7655571537758279], [0.17328654232161622, 0.20886144042655197, 0.5153328984698188, 0.05171246225399584], [0.5141792287129399, 0.583283500705354, 0.8606165853054311, 0.5400957336191635]], [[0.3040971971109807, 0.2756197369023887, 0.8784347193903734, 0.8012109644278425], [0.22639760033061396, 0.9740233759510917, 0.2172276644911606, 0.4291846769645854], [0.49759498425082516, 0.6659590643646194, 0.6606162587711154, 0.7711638256378444]]], [[[0.3209632619316679, 0.10488607601209976, 0.5398417295258359, 0.31036377248316926], [0.32631785687213477, 0.738591103552324, 0.4824409344836714, 0.19563639193549776], [0.115115059234144, 0.2665498136193878, 0.5891651533882257, 0.8444065677225355]], [[0.2687538694893077, 0.10777339870460045, 0.3892261060296086, 0.6369180446929883], [0.22474158826012824, 0.15401958206246325, 0.4763738539282064, 0.8818787221324483], [0.6621059096195536, 0.24653171428690523, 0.13560124988052324, 0.1677304780351253]]], [[[0.7309471920884236, 0.6600900093186649, 0.03657211885098899, 0.6229238458814739], [0.7680918577597312, 0.33650127794325124, 0.3036213787279619, 0.7471661760709509], [0.9197321814816789, 0.06121682265878492, 0.49530528644768657, 0.1611400576430566]], [[0.708408052204989, 0.3346297944187204, 0.5505988893143469, 0.6199648108869605], [0.6057255044368665, 0.6998570523924483, 0.2058841351570614, 0.34627446385817007], [0.28217966366826874, 0.2530884743335543, 0.7028397978966962, 0.9660128446445494]]]] ,numpy.float_)
      arg = Data(val,w_arg)
      ref =  V* val
      res=integrate(arg,where=w_int)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")

   def test_interpolate_onReducedFunctionOnContactZero_fromConstant_onSolution_rank0(self):
      """
      test interploation of constant rank 0 Data on ReducedFunctionOnContactZero
      """
      w_ref = ReducedFunctionOnContactZero(self.domain)
      w_arg = Solution(self.domain)
      val = 6.330481e-01
      arg = Data(val,(),w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactZero_fromConstant_onSolution_rank1(self):
      """
      test interploation of constant rank 1 Data on ReducedFunctionOnContactZero
      """
      w_ref = ReducedFunctionOnContactZero(self.domain)
      w_arg = Solution(self.domain)
      val = numpy.array([0.3847303032020608, 0.18147843998408453, 0.9940159412204554] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactZero_fromConstant_onSolution_rank2(self):
      """
      test interploation of constant rank 2 Data on ReducedFunctionOnContactZero
      """
      w_ref = ReducedFunctionOnContactZero(self.domain)
      w_arg = Solution(self.domain)
      val = numpy.array([[0.8208577150352943, 0.09873815987394452], [0.5093250544122111, 0.23094291531128197], [0.8514022708471113, 0.964575355993364], [0.1701391961561992, 0.5068419792686839]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactZero_fromConstant_onSolution_rank3(self):
      """
      test interploation of constant rank 3 Data on ReducedFunctionOnContactZero
      """
      w_ref = ReducedFunctionOnContactZero(self.domain)
      w_arg = Solution(self.domain)
      val = numpy.array([[[0.7491660335810698, 0.3781943908730171, 0.4830612377154747, 0.2071669494998487], [0.22960699147160069, 0.13733653174183047, 0.6244232542514038, 0.14146042232141587]], [[0.37818031217364223, 0.8943647335540977, 0.09036738509105202, 0.09622127906237554], [0.07210102540329866, 0.6388556126198095, 0.6769400669818662, 0.9143536767117854]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2, 2, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactZero_fromConstant_onSolution_rank4(self):
      """
      test interploation of constant rank 4 Data on ReducedFunctionOnContactZero
      """
      w_ref = ReducedFunctionOnContactZero(self.domain)
      w_arg = Solution(self.domain)
      val = numpy.array([[[[0.6718250462032066, 0.29986789507335887, 0.3110256285500069, 0.6505477317104773], [0.3425951754134493, 0.237344842051738, 0.8275585108656683, 0.24519009486586185], [0.4031511524830552, 0.3166140413376468, 0.10005471556181855, 0.3254197030312074]], [[0.40617481107025755, 0.44726801873832145, 0.09071143280787397, 0.883474185942764], [0.5237359517089172, 0.08126785760724764, 0.3009184406804586, 0.7160481900774361], [0.7655847125163144, 0.4249660271184704, 0.5008962498494155, 0.7249237060161606]]], [[[0.7360051119017891, 0.645888226793658, 0.26714392703105694, 0.6022966727656676], [0.287507561238987, 0.9930331748985254, 0.7232020632371701, 0.20292202319445463], [0.889796327571103, 0.9949184984621474, 0.24426073838085904, 0.09731117222288488]], [[0.35228314971731745, 0.9002459954269506, 0.3781967829699544, 0.7230201407340527], [0.5077124957966629, 0.43840369649254507, 0.6237226329227449, 0.8546983335504517], [0.42257658812665655, 0.5580163776126614, 0.9837258019585285, 0.8217156175026029]]], [[[0.06540846082131235, 0.976259392997154, 0.8071483996891965, 0.8526377776948062], [0.8573384175848723, 0.5268058810408864, 0.4368408388051713, 0.38593395831850486], [0.13005523827196253, 0.57482486773416, 0.22344636722953815, 0.62639297295493]], [[0.42475203937081163, 0.1052269050588911, 0.36255917905669044, 0.9859470428887305], [0.29801058959055027, 0.6325110092015462, 0.9093463197409053, 0.44900542105854613], [0.8555544210803229, 0.46929767874235195, 0.943608161714152, 0.4012514339786649]]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactZero_fromConstant_onReducedSolution_rank0(self):
      """
      test interploation of constant rank 0 Data on ReducedFunctionOnContactZero
      """
      w_ref = ReducedFunctionOnContactZero(self.domain)
      w_arg = ReducedSolution(self.domain)
      val = 6.060676e-01
      arg = Data(val,(),w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactZero_fromConstant_onReducedSolution_rank1(self):
      """
      test interploation of constant rank 1 Data on ReducedFunctionOnContactZero
      """
      w_ref = ReducedFunctionOnContactZero(self.domain)
      w_arg = ReducedSolution(self.domain)
      val = numpy.array([0.6906812771893213, 0.8080673054304435, 0.18950422823206092] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactZero_fromConstant_onReducedSolution_rank2(self):
      """
      test interploation of constant rank 2 Data on ReducedFunctionOnContactZero
      """
      w_ref = ReducedFunctionOnContactZero(self.domain)
      w_arg = ReducedSolution(self.domain)
      val = numpy.array([[0.9189228069867992, 0.6324602542639153], [0.2443545392607157, 0.3343650598287712], [0.8924484055230667, 0.17086476689686925], [0.35033597738746536, 0.8369454502618392]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactZero_fromConstant_onReducedSolution_rank3(self):
      """
      test interploation of constant rank 3 Data on ReducedFunctionOnContactZero
      """
      w_ref = ReducedFunctionOnContactZero(self.domain)
      w_arg = ReducedSolution(self.domain)
      val = numpy.array([[[0.1371143725861017, 0.27932715073173353, 0.8332259067661251, 0.5600063787568943], [0.15385825515609697, 0.1360435698355964, 0.03782448970361396, 0.8991865253916586]], [[0.9956674984525522, 0.10160961705635152, 0.10486501883304411, 0.49770198355059925], [0.9065287567680511, 0.02267047480629869, 0.6611001335973236, 0.5826742825885743]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2, 2, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactZero_fromConstant_onReducedSolution_rank4(self):
      """
      test interploation of constant rank 4 Data on ReducedFunctionOnContactZero
      """
      w_ref = ReducedFunctionOnContactZero(self.domain)
      w_arg = ReducedSolution(self.domain)
      val = numpy.array([[[[0.022691076620816264, 0.3533832857970205, 0.22963568350150954, 0.3119816493309603], [0.30749262016843926, 0.3802296216499901, 0.888625527211768, 0.4500410463056953], [0.1917849079730395, 0.8655154716792614, 0.3126308600286407, 0.5634837550587327]], [[0.2671124850384601, 0.26264477741416936, 0.2551979655454538, 0.6341046713691291], [0.8427325869234399, 0.5541237773217742, 0.6464530250228733, 0.34166173622784846], [0.4462343254076133, 0.07170831770550479, 0.6450594807332083, 0.5638453648893454]]], [[[0.6428418052354318, 0.7845310997782408, 0.22425876876725304, 0.6594252698861488], [0.0031929191359137965, 0.7385874585595147, 0.8927545724866874, 0.5366771586030736], [0.4144854731621478, 0.7120521093816342, 0.22654211118026113, 0.21291824748884214]], [[0.12295192321295256, 0.47592588369927835, 0.08777102342109055, 0.6272600801153061], [0.6632518810004022, 0.1554615488029818, 0.6344576003922896, 0.4871102916959503], [0.9563569625929726, 0.7356299835937146, 0.28505760780484, 0.7015914590499053]]], [[[0.2739320241578147, 0.17202057977629126, 0.6505355163093913, 0.08869773363864453], [0.09384858129857321, 0.7079227547938437, 0.12608403625539966, 0.19892160601287967], [0.020640043648576967, 0.6406729684511675, 0.060555290561925434, 0.2410308886677397]], [[0.45734559875986935, 0.15119106903657464, 0.8378657034068868, 0.7826598202086885], [0.23873198700457376, 0.11302876445582, 0.6799950963335898, 0.888270753284121], [0.9401794196258947, 0.12966317539066152, 0.2054096343050703, 0.033157530789681866]]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactZero_fromConstant_onContinuousFunction_rank0(self):
      """
      test interploation of constant rank 0 Data on ReducedFunctionOnContactZero
      """
      w_ref = ReducedFunctionOnContactZero(self.domain)
      w_arg = ContinuousFunction(self.domain)
      val = 2.953078e-01
      arg = Data(val,(),w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactZero_fromConstant_onContinuousFunction_rank1(self):
      """
      test interploation of constant rank 1 Data on ReducedFunctionOnContactZero
      """
      w_ref = ReducedFunctionOnContactZero(self.domain)
      w_arg = ContinuousFunction(self.domain)
      val = numpy.array([0.3845355597359712, 0.86963576072494, 0.860449577218653] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactZero_fromConstant_onContinuousFunction_rank2(self):
      """
      test interploation of constant rank 2 Data on ReducedFunctionOnContactZero
      """
      w_ref = ReducedFunctionOnContactZero(self.domain)
      w_arg = ContinuousFunction(self.domain)
      val = numpy.array([[0.2391570735164953, 0.13196946368084606], [0.8905912217245331, 0.2633768917539586], [0.9942298991861107, 0.04095865299879475], [0.5319312721065134, 0.30463112816560534]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactZero_fromConstant_onContinuousFunction_rank3(self):
      """
      test interploation of constant rank 3 Data on ReducedFunctionOnContactZero
      """
      w_ref = ReducedFunctionOnContactZero(self.domain)
      w_arg = ContinuousFunction(self.domain)
      val = numpy.array([[[0.49114265807832236, 0.5114581577573066, 0.07169960652055996, 0.5790704426790384], [0.24459129723452055, 0.9270936133699866, 0.23726899132346013, 0.0002064213290204897]], [[0.6672202726944024, 0.24120842266787224, 0.17381308263223705, 0.1101699489616379], [0.8828201656638118, 0.1382703833214055, 0.17282279637210474, 0.7178295790711469]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2, 2, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactZero_fromConstant_onContinuousFunction_rank4(self):
      """
      test interploation of constant rank 4 Data on ReducedFunctionOnContactZero
      """
      w_ref = ReducedFunctionOnContactZero(self.domain)
      w_arg = ContinuousFunction(self.domain)
      val = numpy.array([[[[0.4177653109650642, 0.022048657361918855, 0.32211678042604097, 0.9167709922903761], [0.6305143539043071, 0.02688412728450229, 0.06774737721874857, 0.3869922454029354], [0.5207547772063892, 0.9024539465085347, 0.19500996893906075, 0.0699550058918712]], [[0.5214720360088625, 0.8923763386480904, 0.873629151274173, 0.15404777664370894], [0.22574408035014037, 0.47684933810233454, 0.024085677923542437, 0.8462778314593252], [0.7068568127163424, 0.6099199461074133, 0.3367659761357047, 0.06252752040143872]]], [[[0.7192944587704798, 0.7154141744368239, 0.866664975241598, 0.08144122888135519], [0.5406658086312278, 0.390914122127785, 0.11957371108187709, 0.47500971947916004], [0.20659400046479626, 0.9089315863431751, 0.045064233162471545, 0.5061581898715457]], [[0.13414661038966558, 0.7044852073513079, 0.5472034417406056, 0.24382822233230228], [0.8671501210548861, 0.594835404447363, 0.07193521431193228, 0.45258925073459666], [0.816348999762677, 0.5448395518611996, 0.9073949904252755, 0.582188410102424]]], [[[0.9380939915342696, 0.4430838483114308, 0.520938271411763, 0.2902246651283391], [0.321815594116018, 0.9448965469327676, 0.3669063245652392, 0.3891845259433939], [0.37459193705443306, 0.43904226883456976, 0.9886164692558943, 0.8348602912755285]], [[0.599507447191334, 0.31989050757525916, 0.566426966823346, 0.851646110756947], [0.2308476866140663, 0.6053061279907493, 0.6691753057313028, 0.07258897990499413], [0.8255340902720096, 0.9386906408948554, 0.8584606721948733, 0.5226931993562006]]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactZero_fromConstant_onReducedContinuousFunction_rank0(self):
      """
      test interploation of constant rank 0 Data on ReducedFunctionOnContactZero
      """
      w_ref = ReducedFunctionOnContactZero(self.domain)
      w_arg = ReducedContinuousFunction(self.domain)
      val = 9.336493e-01
      arg = Data(val,(),w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactZero_fromConstant_onReducedContinuousFunction_rank1(self):
      """
      test interploation of constant rank 1 Data on ReducedFunctionOnContactZero
      """
      w_ref = ReducedFunctionOnContactZero(self.domain)
      w_arg = ReducedContinuousFunction(self.domain)
      val = numpy.array([0.9886775563609929, 0.7371308938062822, 0.21931795217365824] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactZero_fromConstant_onReducedContinuousFunction_rank2(self):
      """
      test interploation of constant rank 2 Data on ReducedFunctionOnContactZero
      """
      w_ref = ReducedFunctionOnContactZero(self.domain)
      w_arg = ReducedContinuousFunction(self.domain)
      val = numpy.array([[0.03840819862857736, 0.2863633250961184], [0.10559124679039344, 0.6597256040849194], [0.6834773541555171, 0.08984943899067521], [0.5554228448357418, 0.24623960618512575]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactZero_fromConstant_onReducedContinuousFunction_rank3(self):
      """
      test interploation of constant rank 3 Data on ReducedFunctionOnContactZero
      """
      w_ref = ReducedFunctionOnContactZero(self.domain)
      w_arg = ReducedContinuousFunction(self.domain)
      val = numpy.array([[[0.6998941906226313, 0.5445760547127386, 0.8240769290809731, 0.4930422827933304], [0.5495397542781163, 0.5481843020010052, 0.318488359313044, 0.27990975787465844]], [[0.7934430793601053, 0.5865665319041602, 0.48807037742756476, 0.12129081360504934], [0.5387314642640326, 0.6309147992160005, 0.24912452158413756, 0.03709618533286696]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2, 2, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactZero_fromConstant_onReducedContinuousFunction_rank4(self):
      """
      test interploation of constant rank 4 Data on ReducedFunctionOnContactZero
      """
      w_ref = ReducedFunctionOnContactZero(self.domain)
      w_arg = ReducedContinuousFunction(self.domain)
      val = numpy.array([[[[0.9715664343159572, 0.32752605420346126, 0.2824848280267871, 0.4818122416757301], [0.6879926771276003, 0.8237721056402665, 0.7808412589457396, 0.837291221343629], [0.5376960220803515, 0.8923558915759796, 0.1548666220868764, 0.3001463620657945]], [[0.23778938614690148, 0.3385349525635021, 0.5784384199497421, 0.7622321212352097], [0.17022291578508597, 0.9741360537357493, 0.9096895999046625, 0.7403275182578618], [0.7278428470859918, 0.4778837898750473, 0.6244526122200538, 0.0347146372826983]]], [[[0.8064459100141096, 0.24752283512140105, 0.9194679072205766, 0.7831085881551937], [0.7712832368840539, 0.04759291077119143, 0.1873206066440508, 0.20730513192178146], [0.9557277074068693, 0.49247458874729144, 0.6541002686400001, 0.5666224264597615]], [[0.26193614602389503, 0.4931898599678237, 0.37706513558147137, 0.22420559658361006], [0.8441673464097529, 0.23839173726723328, 0.8498057027761104, 0.912847323167645], [0.20340678621086516, 0.5760352923021739, 0.8840542659302092, 0.3972551752198177]]], [[[0.9150643763361181, 0.8072203610679646, 0.7331420529781646, 0.8875565620599876], [0.04532661462352716, 0.03084912626985492, 0.6677352277139321, 0.7952195557705751], [0.865479095047719, 0.5498754718191168, 0.7595284821539301, 0.8943334530638315]], [[0.854073454824556, 0.8243569907298768, 0.6091993157376407, 0.5725659926415532], [0.9001563151081481, 0.11206572300738216, 0.7588389019081724, 0.32565137107019126], [0.6586759823342888, 0.7926285686252127, 0.7849075359860759, 0.07231631299643582]]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactZero_fromConstant_onReducedFunctionOnContactZero_rank0(self):
      """
      test interploation of constant rank 0 Data on ReducedFunctionOnContactZero
      """
      w_ref = ReducedFunctionOnContactZero(self.domain)
      w_arg = ReducedFunctionOnContactZero(self.domain)
      val = 1.145868e-01
      arg = Data(val,(),w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactZero_fromConstant_onReducedFunctionOnContactZero_rank1(self):
      """
      test interploation of constant rank 1 Data on ReducedFunctionOnContactZero
      """
      w_ref = ReducedFunctionOnContactZero(self.domain)
      w_arg = ReducedFunctionOnContactZero(self.domain)
      val = numpy.array([0.014598946816728353, 0.5388924052004451, 0.11653148663685087] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactZero_fromConstant_onReducedFunctionOnContactZero_rank2(self):
      """
      test interploation of constant rank 2 Data on ReducedFunctionOnContactZero
      """
      w_ref = ReducedFunctionOnContactZero(self.domain)
      w_arg = ReducedFunctionOnContactZero(self.domain)
      val = numpy.array([[0.4687889567219372, 0.8332212565567162], [0.3182681090046857, 0.9832559077365997], [0.11014662238585038, 0.13385077568540205], [0.08981551850674097, 0.5316857454980802]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactZero_fromConstant_onReducedFunctionOnContactZero_rank3(self):
      """
      test interploation of constant rank 3 Data on ReducedFunctionOnContactZero
      """
      w_ref = ReducedFunctionOnContactZero(self.domain)
      w_arg = ReducedFunctionOnContactZero(self.domain)
      val = numpy.array([[[0.05136782848478505, 0.3616174604014717, 0.6697031846273365, 0.860200631168688], [0.1921153223733585, 0.35808933704543067, 0.8931558333595058, 0.2477142901806817]], [[0.4019306698584463, 0.10089461702836311, 0.2579939066532463, 0.3591614996527942], [0.6164648682436485, 0.05735063809883212, 0.9986841542843176, 0.6569099434982767]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2, 2, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactZero_fromConstant_onReducedFunctionOnContactZero_rank4(self):
      """
      test interploation of constant rank 4 Data on ReducedFunctionOnContactZero
      """
      w_ref = ReducedFunctionOnContactZero(self.domain)
      w_arg = ReducedFunctionOnContactZero(self.domain)
      val = numpy.array([[[[0.5669971366164621, 0.01580416307767185, 0.06630517589952745, 0.20221007560758097], [0.6462771999580642, 0.49763664243966543, 0.3130784507172122, 0.6131089351944367], [0.36957178626595055, 0.196712238820288, 0.7046759237509974, 0.4674503083951723]], [[0.19678777278077786, 0.46279123374341846, 0.08371063082219199, 0.8630282074251086], [0.010862230782281768, 0.5353974080065692, 0.668479523216562, 0.5872003221351528], [0.24333529376096674, 0.6820999275882298, 0.7379304485334041, 0.8634803176515862]]], [[[0.30704201191215263, 0.4846064810266443, 0.8579185986412746, 0.6619678720765588], [0.13049096224896362, 0.36738583684508674, 0.777977515981437, 0.5474162683860738], [0.7323121470823193, 0.35510805665931977, 0.551877783225242, 0.4074331295993565]], [[0.9057579970974661, 0.12898104422287127, 0.3694441975069761, 0.17720135222892186], [0.24507243089248587, 0.5564811310765988, 0.7601307740503912, 0.6152450121219499], [0.0014042221665896415, 0.9888021516097056, 0.7681816977068601, 0.9482445981610248]]], [[[0.30785109330412164, 0.10846490575464107, 0.24131043343856173, 0.1060095502125532], [0.7187153578512048, 0.19493814498017836, 0.755229006295177, 0.7805905502479094], [0.6863504395822222, 0.4127326393501052, 0.6796023652441215, 0.3444834871632124]], [[0.33534234836702137, 0.14493680932633346, 0.9129475017077021, 0.5247811551194456], [0.9324156260291195, 0.7598352779583928, 0.18700945199655483, 0.5917312925076152], [0.8358592737177373, 0.24182847710499955, 0.9192976791320597, 0.7813867633950601]]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactZero_fromConstant_onReducedFunctionOnContactOne_rank0(self):
      """
      test interploation of constant rank 0 Data on ReducedFunctionOnContactZero
      """
      w_ref = ReducedFunctionOnContactZero(self.domain)
      w_arg = ReducedFunctionOnContactOne(self.domain)
      val = 6.943884e-01
      arg = Data(val,(),w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactZero_fromConstant_onReducedFunctionOnContactOne_rank1(self):
      """
      test interploation of constant rank 1 Data on ReducedFunctionOnContactZero
      """
      w_ref = ReducedFunctionOnContactZero(self.domain)
      w_arg = ReducedFunctionOnContactOne(self.domain)
      val = numpy.array([0.8372061535054994, 0.40521616915850245, 0.2114836970531737] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactZero_fromConstant_onReducedFunctionOnContactOne_rank2(self):
      """
      test interploation of constant rank 2 Data on ReducedFunctionOnContactZero
      """
      w_ref = ReducedFunctionOnContactZero(self.domain)
      w_arg = ReducedFunctionOnContactOne(self.domain)
      val = numpy.array([[0.7256493770622505, 0.1482312462529609], [0.7360015002282804, 0.8691433215929194], [0.4689098845451053, 0.7848275761816804], [0.8181516879866743, 0.14911338186871803]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactZero_fromConstant_onReducedFunctionOnContactOne_rank3(self):
      """
      test interploation of constant rank 3 Data on ReducedFunctionOnContactZero
      """
      w_ref = ReducedFunctionOnContactZero(self.domain)
      w_arg = ReducedFunctionOnContactOne(self.domain)
      val = numpy.array([[[0.7741108313841725, 0.07867597832568796, 0.8507932534778745, 0.6665074031633893], [0.877758564384925, 0.674167604795121, 0.2302869285324297, 0.8469510858777648]], [[0.37767868542544036, 0.3228960140301764, 0.8278726185576784, 0.013272790670921353], [0.5325634136266355, 0.9851843369303385, 0.24910698717180257, 0.004955611133203752]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2, 2, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactZero_fromConstant_onReducedFunctionOnContactOne_rank4(self):
      """
      test interploation of constant rank 4 Data on ReducedFunctionOnContactZero
      """
      w_ref = ReducedFunctionOnContactZero(self.domain)
      w_arg = ReducedFunctionOnContactOne(self.domain)
      val = numpy.array([[[[0.8611161439388498, 0.32155962199210253, 0.8254098053890913, 0.12982886989501508], [0.5249916045648777, 0.35397017216685167, 0.83020214979672, 0.1589582377841191], [0.6040215924739342, 0.13679638110767178, 0.06346565279053529, 0.324727569733824]], [[0.12496625827234364, 0.39366045280136186, 0.906850882184089, 0.7599170488161139], [0.19052455076162222, 0.017538106911885176, 0.40142412278443895, 0.9441545680896254], [0.763718658605361, 0.9796213536747616, 0.9228333540472967, 0.5125569156059232]]], [[[0.026040922654592014, 0.6229045466950793, 0.7167729239585867, 0.3048067903231889], [0.13339723408694915, 0.9574635065237797, 0.8149968377125104, 0.12997839321044324], [0.14467723173737834, 0.1327221075281273, 0.2750435157159411, 0.06106304417268327]], [[0.9163440024478942, 0.21428906478440846, 0.6894155773738542, 0.6751612279093356], [0.29511969374178804, 0.11568943040607904, 0.32596884148848015, 0.9537932945409335], [0.6708895914632708, 0.8346295524244489, 0.21707919011589005, 0.14509870246259016]]], [[[0.3361158554518662, 0.838602001020879, 0.8520665989089473, 0.30012919469780885], [0.9734924655326161, 0.7647787217669181, 0.34815949319209394, 0.34318362958021953], [0.31917662712988804, 0.6256910108499081, 0.7498603661227391, 0.5580523841250132]], [[0.28899718364899596, 0.648020666539341, 0.7102451804425935, 0.3019210722641553], [0.20501813980520156, 0.6712331681288656, 0.927304593813167, 0.3750259080970749], [0.07315463714671933, 0.37788062980801895, 0.3300656776241899, 0.23522476925820313]]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactZero_fromConstant_onFunctionOnContactZero_rank0(self):
      """
      test interploation of constant rank 0 Data on ReducedFunctionOnContactZero
      """
      w_ref = ReducedFunctionOnContactZero(self.domain)
      w_arg = FunctionOnContactZero(self.domain)
      val = 4.845049e-01
      arg = Data(val,(),w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactZero_fromConstant_onFunctionOnContactZero_rank1(self):
      """
      test interploation of constant rank 1 Data on ReducedFunctionOnContactZero
      """
      w_ref = ReducedFunctionOnContactZero(self.domain)
      w_arg = FunctionOnContactZero(self.domain)
      val = numpy.array([0.6659190337943574, 0.8425843366524217, 0.86660804623725] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactZero_fromConstant_onFunctionOnContactZero_rank2(self):
      """
      test interploation of constant rank 2 Data on ReducedFunctionOnContactZero
      """
      w_ref = ReducedFunctionOnContactZero(self.domain)
      w_arg = FunctionOnContactZero(self.domain)
      val = numpy.array([[0.25217421622529224, 0.09102472535115458], [0.5729982223940684, 0.8740105387167216], [0.21524015232674165, 0.44837802824313044], [0.7327348938019275, 0.05221915361280771]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactZero_fromConstant_onFunctionOnContactZero_rank3(self):
      """
      test interploation of constant rank 3 Data on ReducedFunctionOnContactZero
      """
      w_ref = ReducedFunctionOnContactZero(self.domain)
      w_arg = FunctionOnContactZero(self.domain)
      val = numpy.array([[[0.6799229601774239, 0.3929325398855149, 0.6847673145434182, 0.3724803911748573], [0.711887207769998, 0.22194916796210573, 0.6712549188704774, 0.28258321841582257]], [[0.7484111828715916, 0.9631433363244414, 0.7733569056267277, 0.7476954837264104], [0.45659970344320955, 0.7774148192295013, 0.21085249644732107, 0.25407762009731116]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2, 2, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactZero_fromConstant_onFunctionOnContactZero_rank4(self):
      """
      test interploation of constant rank 4 Data on ReducedFunctionOnContactZero
      """
      w_ref = ReducedFunctionOnContactZero(self.domain)
      w_arg = FunctionOnContactZero(self.domain)
      val = numpy.array([[[[0.21768148774428764, 0.6640555066349386, 0.593573127369806, 0.7030216681373775], [0.2655388194828342, 0.649638599701596, 0.4285054262482364, 0.20160339715342557], [0.20056970892555293, 0.5491660946360606, 0.04692162190468818, 0.4005442872519722]], [[0.16656078389952345, 0.6599981034016131, 0.037315941635561733, 0.9957549771401615], [0.04810148566846428, 0.46551096251388935, 0.6569628052537012, 0.9808626268499148], [0.39771971513048987, 0.008068708899264698, 0.1090751226259179, 0.05099323999588268]]], [[[0.04348861256016889, 0.365442326678836, 0.9414415283820283, 0.37394714916448946], [0.31486594364767595, 0.6226307984357337, 0.7214028747650802, 0.921775907496978], [0.12317126972460779, 0.1557646890316161, 0.9335148870908166, 0.7009875963822612]], [[0.553128646781664, 0.44492668999737783, 0.4667060222591197, 0.30351659012079457], [0.8317969480524376, 0.01917110870680916, 0.9940531811835266, 0.003450315454359476], [0.24015505103792256, 0.19041797883314648, 0.9014045204901404, 0.04379457314930135]]], [[[0.19618751830791958, 0.9558857598481447, 0.1462665927598613, 0.1886424867437131], [0.48965060806051286, 0.8321292490729796, 0.4151671924835313, 0.34062434911427986], [0.5365857102463347, 0.2732600262580188, 0.5158947896319159, 0.9939782588196867]], [[0.778451531491623, 0.9571672102046037, 0.7302482421416558, 0.7378134842780778], [0.38612725697484807, 0.7338263620852454, 0.8400804975696317, 0.42841324502886136], [0.35932744228731806, 0.31504182302593486, 0.7663613791571788, 0.9781536043256535]]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactZero_fromConstant_onFunctionOnContactOne_rank0(self):
      """
      test interploation of constant rank 0 Data on ReducedFunctionOnContactZero
      """
      w_ref = ReducedFunctionOnContactZero(self.domain)
      w_arg = FunctionOnContactOne(self.domain)
      val = 5.724791e-01
      arg = Data(val,(),w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactZero_fromConstant_onFunctionOnContactOne_rank1(self):
      """
      test interploation of constant rank 1 Data on ReducedFunctionOnContactZero
      """
      w_ref = ReducedFunctionOnContactZero(self.domain)
      w_arg = FunctionOnContactOne(self.domain)
      val = numpy.array([0.5698368155523524, 0.5011314959919055, 0.32171099261248726] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactZero_fromConstant_onFunctionOnContactOne_rank2(self):
      """
      test interploation of constant rank 2 Data on ReducedFunctionOnContactZero
      """
      w_ref = ReducedFunctionOnContactZero(self.domain)
      w_arg = FunctionOnContactOne(self.domain)
      val = numpy.array([[0.9238376342878144, 0.9999212701723688], [0.9221935327868198, 0.8690211330196875], [0.3981760585605262, 0.3799044077748108], [0.14232892443529388, 0.31344836746195603]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactZero_fromConstant_onFunctionOnContactOne_rank3(self):
      """
      test interploation of constant rank 3 Data on ReducedFunctionOnContactZero
      """
      w_ref = ReducedFunctionOnContactZero(self.domain)
      w_arg = FunctionOnContactOne(self.domain)
      val = numpy.array([[[0.06056382123169868, 0.359817046935317, 0.28381043003603423, 0.4816695237021398], [0.9241695552382608, 0.22045482375842562, 0.9939653621100799, 0.31314176920992887]], [[0.3817408616497908, 0.22369390831913782, 0.527385528624288, 0.22576418268348042], [0.26244996773731366, 0.11697331204136951, 0.7509837159079806, 0.9346177813221138]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2, 2, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactZero_fromConstant_onFunctionOnContactOne_rank4(self):
      """
      test interploation of constant rank 4 Data on ReducedFunctionOnContactZero
      """
      w_ref = ReducedFunctionOnContactZero(self.domain)
      w_arg = FunctionOnContactOne(self.domain)
      val = numpy.array([[[[0.9611056473977208, 0.9625252433932414, 0.9701210392672508, 0.7392339432713271], [0.8125183894981813, 0.20490337916335266, 0.5313678844562993, 0.7502017282131984], [0.6560708268845925, 0.17454924487736734, 0.7687959897406307, 0.9565415557844211]], [[0.8944855699395525, 0.7792395779626226, 0.8708531279970064, 0.9620073237823549], [0.8838912214913078, 0.5215520015606976, 0.5586945994230323, 0.921579825455942], [0.6301087631520504, 0.035212743170049965, 0.5389854926876433, 0.29091508887432127]]], [[[0.3297109672452041, 0.9339653472582714, 0.3259050675319838, 0.9314065435929958], [0.4429931962783342, 0.23694021826557976, 0.12858004366485876, 0.6882459597499574], [0.08209003947879734, 0.9960079105665074, 0.2841255462316885, 0.5592870479239467]], [[0.8418287665306695, 0.9432765765026192, 0.6551290626281849, 0.44728268160156304], [0.21151964485242125, 0.1833556472481973, 0.07247562741733071, 0.005213331928283549], [0.9838906601539259, 0.04065411088930826, 0.7965971010299203, 0.7068566837958828]]], [[[0.2073063316702618, 0.5503845897620818, 0.6989624559798674, 0.8464504221293961], [0.0761415094247948, 0.6427663541749622, 0.6114978020573574, 0.38368907229409777], [0.6624151078407227, 0.40744011855276563, 0.9943285398742499, 0.7130099498829475]], [[0.9333858863734091, 0.8705579491142914, 0.7183669180540276, 0.5336980442576607], [0.722637631155766, 0.15914988496070648, 0.609464425883208, 0.8625554848316861], [0.6003406314835725, 0.5169900024490881, 0.4111695760667604, 0.8117933242175697]]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactOne_fromConstant_onSolution_rank0(self):
      """
      test interploation of constant rank 0 Data on ReducedFunctionOnContactOne
      """
      w_ref = ReducedFunctionOnContactOne(self.domain)
      w_arg = Solution(self.domain)
      val = 2.329112e-01
      arg = Data(val,(),w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactOne_fromConstant_onSolution_rank1(self):
      """
      test interploation of constant rank 1 Data on ReducedFunctionOnContactOne
      """
      w_ref = ReducedFunctionOnContactOne(self.domain)
      w_arg = Solution(self.domain)
      val = numpy.array([0.2711886947991414, 0.8151355217815176, 0.548802974879618] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactOne_fromConstant_onSolution_rank2(self):
      """
      test interploation of constant rank 2 Data on ReducedFunctionOnContactOne
      """
      w_ref = ReducedFunctionOnContactOne(self.domain)
      w_arg = Solution(self.domain)
      val = numpy.array([[0.8570748126811828, 0.6716856327358565], [0.21482736306984695, 0.06808127712181944], [0.9612468983613089, 0.28244457685273383], [0.7948157169365453, 0.6984681635200359]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactOne_fromConstant_onSolution_rank3(self):
      """
      test interploation of constant rank 3 Data on ReducedFunctionOnContactOne
      """
      w_ref = ReducedFunctionOnContactOne(self.domain)
      w_arg = Solution(self.domain)
      val = numpy.array([[[0.45165079289227006, 0.6289081084686083, 0.0026816585529017, 0.6272694446815826], [0.3010372661670382, 0.41395307175135165, 0.6042854301060929, 0.5142571738122232]], [[0.5008637439715871, 0.9353749115529721, 0.8461283387858052, 0.7786733015801677], [0.5645640680484281, 0.029207927788641452, 0.9245927728277314, 0.11215605749828972]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2, 2, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactOne_fromConstant_onSolution_rank4(self):
      """
      test interploation of constant rank 4 Data on ReducedFunctionOnContactOne
      """
      w_ref = ReducedFunctionOnContactOne(self.domain)
      w_arg = Solution(self.domain)
      val = numpy.array([[[[0.6667443235973334, 0.5942110361407772, 0.912954606463574, 0.2561427231087743], [0.03600309973338811, 0.5268165160971201, 0.867185955246074, 0.1239484471139769], [0.44253560243971357, 0.43847765297702945, 0.7821567693309968, 0.43443532577956356]], [[0.3984396145874014, 0.38981536734729494, 0.1087483198166681, 0.31180401854591444], [0.889056982006137, 0.48548744067116734, 0.906615382695575, 0.3407086005104446], [0.9198134373422651, 0.9231091093031504, 0.1270515255589374, 0.5067823228534746]]], [[[0.3617513525269881, 0.9574600969184808, 0.11483159594978753, 0.45717952457785926], [0.6916384030503141, 0.047365934376841334, 0.3084139015584185, 0.34364384139996695], [0.05537078077656177, 0.7810234521541862, 0.7306151948170092, 0.7704150321221694]], [[0.784746089405918, 0.8707184031358675, 0.47187684294608834, 0.9226435647241911], [0.6589102270951339, 0.9003100870465465, 0.4302446269703575, 0.8289555515035888], [0.36657797042557505, 0.3740237638520778, 0.9241858074329256, 0.20129929703134763]]], [[[0.032220216298067705, 0.9702497099141327, 0.15627179451850504, 0.5276175837577495], [0.679992562773294, 0.3937022020883134, 0.39634005303659947, 0.08166191158104608], [0.489219423966899, 0.1139517718170946, 0.2190339095220224, 0.7646487210991467]], [[0.6907510516138318, 0.5855936531863534, 0.5457891503259775, 0.34046708951156646], [0.9954466744953929, 0.7578070949471264, 0.05861984555028843, 0.4050603537075873], [0.3839626297766049, 0.45543971152592133, 0.6354353285261648, 0.5529225073133225]]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactOne_fromConstant_onReducedSolution_rank0(self):
      """
      test interploation of constant rank 0 Data on ReducedFunctionOnContactOne
      """
      w_ref = ReducedFunctionOnContactOne(self.domain)
      w_arg = ReducedSolution(self.domain)
      val = 1.264995e-01
      arg = Data(val,(),w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactOne_fromConstant_onReducedSolution_rank1(self):
      """
      test interploation of constant rank 1 Data on ReducedFunctionOnContactOne
      """
      w_ref = ReducedFunctionOnContactOne(self.domain)
      w_arg = ReducedSolution(self.domain)
      val = numpy.array([0.00606243243505622, 0.873034020260163, 0.6841164021678631] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactOne_fromConstant_onReducedSolution_rank2(self):
      """
      test interploation of constant rank 2 Data on ReducedFunctionOnContactOne
      """
      w_ref = ReducedFunctionOnContactOne(self.domain)
      w_arg = ReducedSolution(self.domain)
      val = numpy.array([[0.31093865621812533, 0.39058400322997067], [0.1567669755282769, 0.03514071146133979], [0.9500533629353507, 0.948835882084125], [0.939378083220056, 0.5099242977624797]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactOne_fromConstant_onReducedSolution_rank3(self):
      """
      test interploation of constant rank 3 Data on ReducedFunctionOnContactOne
      """
      w_ref = ReducedFunctionOnContactOne(self.domain)
      w_arg = ReducedSolution(self.domain)
      val = numpy.array([[[0.45007016486443874, 0.6124223809428949, 0.7119569531022489, 0.9442018514201687], [0.36356781159602614, 0.27723610150765865, 0.7946173432983135, 0.9092945948072412]], [[0.1759860846095892, 0.6678468246950318, 0.17838954927513861, 0.8153494705492041], [0.9824334501398573, 0.900923805295426, 0.4704772535790611, 0.9921140597526442]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2, 2, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactOne_fromConstant_onReducedSolution_rank4(self):
      """
      test interploation of constant rank 4 Data on ReducedFunctionOnContactOne
      """
      w_ref = ReducedFunctionOnContactOne(self.domain)
      w_arg = ReducedSolution(self.domain)
      val = numpy.array([[[[0.11855174064114682, 0.1451392781947035, 0.6160462460663021, 0.9435709224990018], [0.5778251588350022, 0.35345122476635427, 0.15737472936970198, 0.5068053965783152], [0.6496370130999982, 0.7565725953485603, 0.8877311648144324, 0.6779183819088433]], [[0.19499210597476802, 0.05963196379949376, 0.8471924161676365, 0.6796323912421396], [0.6879746956169507, 0.31493633190401105, 0.10395651021404007, 0.7566677619630795], [0.8219461421678597, 0.7427790065801423, 0.4898599644283754, 0.09731680329940562]]], [[[0.8578179515616177, 0.017653922457677496, 0.5920299881752641, 0.6540800346765566], [0.08108154104941678, 0.5492952546810023, 0.327341995902085, 0.008431680039213907], [0.43869582655314643, 0.5031428199029038, 0.23772964965025756, 0.56411232787934]], [[0.8338427483017984, 0.2784845639882001, 0.684750809513023, 0.5771865103410377], [0.5383837799814473, 0.4326295086578277, 0.5415153722697578, 0.42890048909447354], [0.9176178153213768, 0.3097673125495437, 0.029148740941863593, 0.5698778269902484]]], [[[0.6199790422663453, 0.5454395501524304, 0.2559067601655193, 0.3723918906555914], [0.43061378209477463, 0.5209802274257546, 0.9632587198410986, 0.7212958354714519], [0.0026623677260552148, 0.8128428713463965, 0.9704429257805324, 0.7749663740105225]], [[0.28521348143121894, 0.04008708758840007, 0.2304032047764707, 0.14563590389591285], [0.9271821441390897, 0.5912086707216369, 0.6719091239917508, 0.07848234072199611], [0.39376430262958106, 0.15375603049356268, 0.1955391476314452, 0.610816803461706]]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactOne_fromConstant_onContinuousFunction_rank0(self):
      """
      test interploation of constant rank 0 Data on ReducedFunctionOnContactOne
      """
      w_ref = ReducedFunctionOnContactOne(self.domain)
      w_arg = ContinuousFunction(self.domain)
      val = 5.179575e-01
      arg = Data(val,(),w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactOne_fromConstant_onContinuousFunction_rank1(self):
      """
      test interploation of constant rank 1 Data on ReducedFunctionOnContactOne
      """
      w_ref = ReducedFunctionOnContactOne(self.domain)
      w_arg = ContinuousFunction(self.domain)
      val = numpy.array([0.9575768192361481, 0.9644800915878442, 0.23982841164781] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactOne_fromConstant_onContinuousFunction_rank2(self):
      """
      test interploation of constant rank 2 Data on ReducedFunctionOnContactOne
      """
      w_ref = ReducedFunctionOnContactOne(self.domain)
      w_arg = ContinuousFunction(self.domain)
      val = numpy.array([[0.9718619518517126, 0.6327758010877947], [0.358183029640899, 0.6439584235904635], [0.01076934009736974, 0.3920209701539201], [0.3271826942675151, 0.12500826265093368]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactOne_fromConstant_onContinuousFunction_rank3(self):
      """
      test interploation of constant rank 3 Data on ReducedFunctionOnContactOne
      """
      w_ref = ReducedFunctionOnContactOne(self.domain)
      w_arg = ContinuousFunction(self.domain)
      val = numpy.array([[[0.23389693596510674, 0.34876444091356096, 0.08744141097388891, 0.4860415807991585], [0.908103324472909, 0.4876573256899095, 0.7320153900343996, 0.27328200078230325]], [[0.2885358653030069, 0.8225359508790787, 0.3719228117362148, 0.8599909728511795], [0.07608006606226236, 0.3585371367277932, 0.9832606683200212, 0.4231855648978766]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2, 2, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactOne_fromConstant_onContinuousFunction_rank4(self):
      """
      test interploation of constant rank 4 Data on ReducedFunctionOnContactOne
      """
      w_ref = ReducedFunctionOnContactOne(self.domain)
      w_arg = ContinuousFunction(self.domain)
      val = numpy.array([[[[0.818535980265798, 0.6156101394672567, 0.9411922390783354, 0.26268772801813944], [0.19860764368424977, 0.06682526098172914, 0.5729195351645129, 0.7939987998645818], [0.45296785676428875, 0.3574723572222631, 0.05507517619000457, 0.37615849163587134]], [[0.24889441716570793, 0.8269423706543342, 0.8689218475567405, 0.862901595496822], [0.5113018931092743, 0.4729113909263415, 0.8640787989842668, 0.6144202571856222], [0.059352354383009454, 0.8841351505399457, 0.4655495168602738, 0.04588429517329806]]], [[[0.9976406856864566, 0.19979008945652021, 0.37575561777196065, 0.5114301121118882], [0.5087730755067255, 0.7445349695232956, 0.20158996669666251, 0.17416591891247846], [0.626996463374618, 0.4717095353175206, 0.4127681790678299, 0.2267064023100076]], [[0.9797121620388637, 0.2506646833379459, 0.27849080014984273, 0.10827692144418677], [0.9282022116283123, 0.463766999230508, 0.5036225568963277, 0.6711495953323894], [0.8734368271422581, 0.3482094575282234, 0.5495193518240209, 0.3423976284311623]]], [[[0.14344456903470693, 0.4018605515532431, 0.3045836878855571, 0.7619929766169216], [0.7863650744050896, 0.3215891898594264, 0.9390648313351854, 0.6492326307978694], [0.2796544369091747, 0.8007304655227839, 0.42333187150567964, 0.09960921173211457]], [[0.4679857067545191, 0.4006626029265893, 0.24884973672213073, 0.52548623594872], [0.7537325455397446, 0.2631311984677541, 0.24602201916926036, 0.9387862828976604], [0.7387343021186891, 0.960930261846333, 0.22593033956112607, 0.834975286046687]]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactOne_fromConstant_onReducedContinuousFunction_rank0(self):
      """
      test interploation of constant rank 0 Data on ReducedFunctionOnContactOne
      """
      w_ref = ReducedFunctionOnContactOne(self.domain)
      w_arg = ReducedContinuousFunction(self.domain)
      val = 3.875819e-01
      arg = Data(val,(),w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactOne_fromConstant_onReducedContinuousFunction_rank1(self):
      """
      test interploation of constant rank 1 Data on ReducedFunctionOnContactOne
      """
      w_ref = ReducedFunctionOnContactOne(self.domain)
      w_arg = ReducedContinuousFunction(self.domain)
      val = numpy.array([0.18278843949565338, 0.03443967023834149, 0.004760660997311761] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactOne_fromConstant_onReducedContinuousFunction_rank2(self):
      """
      test interploation of constant rank 2 Data on ReducedFunctionOnContactOne
      """
      w_ref = ReducedFunctionOnContactOne(self.domain)
      w_arg = ReducedContinuousFunction(self.domain)
      val = numpy.array([[0.2578876581313929, 0.1138047786434172], [0.2542456971866308, 0.13941180663674912], [0.3216905040850888, 0.6388738438409454], [0.6135317788608333, 0.28093463385456574]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactOne_fromConstant_onReducedContinuousFunction_rank3(self):
      """
      test interploation of constant rank 3 Data on ReducedFunctionOnContactOne
      """
      w_ref = ReducedFunctionOnContactOne(self.domain)
      w_arg = ReducedContinuousFunction(self.domain)
      val = numpy.array([[[0.199315997451887, 0.45486284065809934, 0.405439889990851, 0.29442748313818545], [0.26863720801322866, 0.17106342510663386, 0.9508530974979329, 0.2883315892815461]], [[0.7952214446491916, 0.9355296046333761, 0.5338441039561624, 0.02321050156131721], [0.09269655172112112, 0.994349126977387, 0.062363748803044405, 0.703105591771991]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2, 2, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactOne_fromConstant_onReducedContinuousFunction_rank4(self):
      """
      test interploation of constant rank 4 Data on ReducedFunctionOnContactOne
      """
      w_ref = ReducedFunctionOnContactOne(self.domain)
      w_arg = ReducedContinuousFunction(self.domain)
      val = numpy.array([[[[0.27195814824192577, 0.029726316922277896, 0.49277870766075105, 0.6882715721882958], [0.06530184102907244, 0.3107327502824384, 0.3250986030626254, 0.5322683855671014], [0.9999187287127889, 0.04262557739347317, 0.47022418238457353, 0.5979396856062474]], [[0.5375737583080935, 0.6972017617080326, 0.4860487017535191, 0.5746615797843607], [0.081808161048184, 0.19623554896756146, 0.7644374545278538, 0.4558595129288673], [0.027786048701642918, 0.880508951040538, 0.8454412050462895, 0.7035264702988474]]], [[[0.06264870669390299, 0.9992877495547031, 0.6237397672951204, 0.11013805728287651], [0.11659585486148294, 0.7721732639128884, 0.12789435384035763, 0.8719995775452345], [0.6235605837278547, 0.8881807315866083, 0.6847908121857974, 0.9536927911310386]], [[0.7475667499188174, 0.17340704220159198, 0.9377964314689402, 0.2619685096798563], [0.11142872847409657, 0.372971012113405, 0.5717009335101155, 0.3046912409554343], [0.45706794587981936, 0.7074317314588832, 0.5229389820159638, 0.9345077481625098]]], [[[0.24417132911143768, 0.6103183235574506, 0.2150951867132428, 0.10646173490686373], [0.3181942631910315, 0.6191198150724172, 0.35592507092630776, 0.9273583808730407], [0.19175170549128606, 0.9483615422865661, 0.00693853046549997, 0.16967301609688423]], [[0.3668426329153802, 0.539203966787359, 0.4097258359307496, 0.5090384595876427], [0.38135386133641624, 0.9141255977853077, 0.19462182339311707, 0.017402039812049686], [0.10747602911816012, 0.5265530738832295, 0.5693492483053987, 0.15872280601063737]]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactOne_fromConstant_onReducedFunctionOnContactZero_rank0(self):
      """
      test interploation of constant rank 0 Data on ReducedFunctionOnContactOne
      """
      w_ref = ReducedFunctionOnContactOne(self.domain)
      w_arg = ReducedFunctionOnContactZero(self.domain)
      val = 7.955394e-01
      arg = Data(val,(),w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactOne_fromConstant_onReducedFunctionOnContactZero_rank1(self):
      """
      test interploation of constant rank 1 Data on ReducedFunctionOnContactOne
      """
      w_ref = ReducedFunctionOnContactOne(self.domain)
      w_arg = ReducedFunctionOnContactZero(self.domain)
      val = numpy.array([0.03765147921520662, 0.2859074148555407, 0.8652615609153638] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactOne_fromConstant_onReducedFunctionOnContactZero_rank2(self):
      """
      test interploation of constant rank 2 Data on ReducedFunctionOnContactOne
      """
      w_ref = ReducedFunctionOnContactOne(self.domain)
      w_arg = ReducedFunctionOnContactZero(self.domain)
      val = numpy.array([[0.36341448566303136, 0.17964069584166464], [0.356752488191608, 0.46543665302504234], [0.7406980183987848, 0.9808612211679858], [0.7644979900921803, 0.5266073649506665]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactOne_fromConstant_onReducedFunctionOnContactZero_rank3(self):
      """
      test interploation of constant rank 3 Data on ReducedFunctionOnContactOne
      """
      w_ref = ReducedFunctionOnContactOne(self.domain)
      w_arg = ReducedFunctionOnContactZero(self.domain)
      val = numpy.array([[[0.7870982948344794, 0.11262360158133167, 0.21737213085331708, 0.01180789807172311], [0.49899743378700356, 0.10070842951855086, 0.6307896822003546, 0.13868983966975845]], [[0.7455103088049124, 0.534381249407766, 0.31885226909664643, 0.5252376541572474], [0.5434115143323359, 0.44547596461106564, 0.36893165457783017, 0.3068600727665063]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2, 2, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactOne_fromConstant_onReducedFunctionOnContactZero_rank4(self):
      """
      test interploation of constant rank 4 Data on ReducedFunctionOnContactOne
      """
      w_ref = ReducedFunctionOnContactOne(self.domain)
      w_arg = ReducedFunctionOnContactZero(self.domain)
      val = numpy.array([[[[0.8189704356276197, 0.3749050832248578, 0.27389450562415263, 0.5383418333545886], [0.21503535733189094, 0.1330764764687099, 0.23537866042369227, 0.08439743856859616], [0.25695331975720415, 0.1611568135103344, 0.2120386564651976, 0.2759536169148714]], [[0.21258951350562738, 0.8613499384201753, 0.47317365415525503, 0.7355470830751574], [0.9894693607386535, 0.4657250277922349, 0.7367602483666915, 0.18317427209088943], [0.32843011883353856, 0.0012833284516546861, 0.11094884486888767, 0.06499518950184302]]], [[[0.1589764113768818, 0.5424115098290392, 0.5756546771331035, 0.5891081919596415], [0.9971395232800137, 0.5127953175775566, 0.9493819905561655, 0.7617398358785147], [0.9553688130117116, 0.22687512684115074, 0.5001452046414812, 0.688567440068334]], [[0.5918445549691579, 0.2967613273527532, 0.5601946369187094, 0.3399745963621382], [0.8780144653070053, 0.6966757894030086, 0.6500925389906903, 0.9917461476498427], [0.5108217455443833, 0.6330537789188553, 0.4617614480677317, 0.25943269372998146]]], [[[0.2813712889523716, 0.8022918169474674, 0.6790872615412888, 0.7551331928170524], [0.8951817149895257, 0.9257445233242998, 0.7409626782849315, 0.5222813180591156], [0.7954260297545448, 0.9643852812336107, 0.7876470759398919, 0.4702114267889068]], [[0.9020950849353047, 0.7488231531166031, 0.12645975496382733, 0.8684049309189978], [0.5665838087497248, 0.6346594570923372, 0.6834979216607802, 0.970333455895161], [0.4172392831795092, 0.7619652663623792, 0.8219136442872614, 0.11204388928270648]]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactOne_fromConstant_onReducedFunctionOnContactOne_rank0(self):
      """
      test interploation of constant rank 0 Data on ReducedFunctionOnContactOne
      """
      w_ref = ReducedFunctionOnContactOne(self.domain)
      w_arg = ReducedFunctionOnContactOne(self.domain)
      val = 8.306100e-01
      arg = Data(val,(),w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactOne_fromConstant_onReducedFunctionOnContactOne_rank1(self):
      """
      test interploation of constant rank 1 Data on ReducedFunctionOnContactOne
      """
      w_ref = ReducedFunctionOnContactOne(self.domain)
      w_arg = ReducedFunctionOnContactOne(self.domain)
      val = numpy.array([0.8126865327966136, 0.9829268967367245, 0.20673602560938897] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactOne_fromConstant_onReducedFunctionOnContactOne_rank2(self):
      """
      test interploation of constant rank 2 Data on ReducedFunctionOnContactOne
      """
      w_ref = ReducedFunctionOnContactOne(self.domain)
      w_arg = ReducedFunctionOnContactOne(self.domain)
      val = numpy.array([[0.35008005617383176, 0.8142512701460992], [0.45103962710001466, 0.0424878627433154], [0.5343025439071445, 0.33656728140986336], [0.21703155524253026, 0.9737966724769462]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactOne_fromConstant_onReducedFunctionOnContactOne_rank3(self):
      """
      test interploation of constant rank 3 Data on ReducedFunctionOnContactOne
      """
      w_ref = ReducedFunctionOnContactOne(self.domain)
      w_arg = ReducedFunctionOnContactOne(self.domain)
      val = numpy.array([[[0.6178796961055507, 0.2693940062385778, 0.11788134151880392, 0.9743629728341656], [0.4586535139254899, 0.6020370252232854, 0.37269584333047734, 0.9587587193475751]], [[0.7282684231616958, 0.5293756930533948, 0.8154839499936752, 0.030517350536140797], [0.07446755618228962, 0.9959175880156744, 0.8685115095234088, 0.9444178438621647]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2, 2, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactOne_fromConstant_onReducedFunctionOnContactOne_rank4(self):
      """
      test interploation of constant rank 4 Data on ReducedFunctionOnContactOne
      """
      w_ref = ReducedFunctionOnContactOne(self.domain)
      w_arg = ReducedFunctionOnContactOne(self.domain)
      val = numpy.array([[[[0.8229293340116391, 0.04456583018757365, 0.6715510688742183, 0.8910519958413956], [0.19894162523286274, 0.4552004293501739, 0.7233022966267526, 0.6399722753245947], [0.2183475810904214, 0.030019934703234386, 0.4559960877367557, 0.790602623163557]], [[0.23517742840085265, 0.86099844962103, 0.41449046709133874, 0.7974807011831332], [0.09498631833081073, 0.6969597482520815, 0.1655532927738047, 0.0012090022895088204], [0.3679632949225038, 0.8073400302714234, 0.06002516215853826, 0.13965166901741777]]], [[[0.805510627229193, 0.6641922758413422, 0.907076465037853, 0.609870550901237], [0.3549098061568603, 0.44333040318254247, 0.9941570771095157, 0.8857533103519696], [0.7460339094785616, 0.3546039138396384, 0.6868416067314108, 0.21085851968104685]], [[0.24325335870791442, 0.7587127481051399, 0.7399607443828173, 0.4719188132354729], [0.7684370047303896, 0.7074337577840932, 0.8311949866565859, 0.4281633523538112], [0.6496731595028286, 0.3157488953129065, 0.41456560304268386, 0.9990573393495944]]], [[[0.8591204685291776, 0.11832932081662917, 0.9954904508141627, 0.8274744901031881], [0.876832009144863, 0.18439293339631913, 0.9225427810054083, 0.9030760584934312], [0.8545293219699139, 0.7383696332456177, 0.5093167813355622, 0.2180258648280875]], [[0.9123825889878454, 0.894749826362633, 0.7911943997179549, 0.3490818123417232], [0.33556337999303076, 0.7128501122208191, 0.6593788805767017, 0.4140054483336142], [0.30654568343166866, 0.36240259803332264, 0.5521850257261249, 0.2572440770617319]]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactOne_fromConstant_onFunctionOnContactZero_rank0(self):
      """
      test interploation of constant rank 0 Data on ReducedFunctionOnContactOne
      """
      w_ref = ReducedFunctionOnContactOne(self.domain)
      w_arg = FunctionOnContactZero(self.domain)
      val = 8.572442e-02
      arg = Data(val,(),w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactOne_fromConstant_onFunctionOnContactZero_rank1(self):
      """
      test interploation of constant rank 1 Data on ReducedFunctionOnContactOne
      """
      w_ref = ReducedFunctionOnContactOne(self.domain)
      w_arg = FunctionOnContactZero(self.domain)
      val = numpy.array([0.6339958903951071, 0.3834031163077566, 0.28960145769538304] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactOne_fromConstant_onFunctionOnContactZero_rank2(self):
      """
      test interploation of constant rank 2 Data on ReducedFunctionOnContactOne
      """
      w_ref = ReducedFunctionOnContactOne(self.domain)
      w_arg = FunctionOnContactZero(self.domain)
      val = numpy.array([[0.30281019536341214, 0.4061960862960112], [0.9469282733503931, 0.7211459631770277], [0.7915727058455629, 0.00531635681177367], [0.23027833431083422, 0.24603244345161057]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactOne_fromConstant_onFunctionOnContactZero_rank3(self):
      """
      test interploation of constant rank 3 Data on ReducedFunctionOnContactOne
      """
      w_ref = ReducedFunctionOnContactOne(self.domain)
      w_arg = FunctionOnContactZero(self.domain)
      val = numpy.array([[[0.23230818858420033, 0.546564545487729, 0.32266159961977714, 0.603316962320113], [0.2961725195272631, 0.43933328167426966, 0.844785166976214, 0.4881408035048409]], [[0.2607395253886762, 0.5423934580557075, 0.7619733519017275, 0.5812060614648623], [0.0613130772917484, 0.05223867445369368, 0.6802367998056611, 0.08920080302295674]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2, 2, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactOne_fromConstant_onFunctionOnContactZero_rank4(self):
      """
      test interploation of constant rank 4 Data on ReducedFunctionOnContactOne
      """
      w_ref = ReducedFunctionOnContactOne(self.domain)
      w_arg = FunctionOnContactZero(self.domain)
      val = numpy.array([[[[0.5359541100352402, 0.6492481992248392, 0.627809876175268, 0.032183035161362894], [0.6855392752408003, 0.38386506246246577, 0.6877496796830016, 0.9244294054159945], [0.779266018879794, 0.5454157741507168, 0.6076136704393503, 0.34098977241628126]], [[0.6725574014492601, 0.9924554727497924, 0.11265712415266071, 0.6509645338461001], [0.6566058880315083, 0.05085750216442875, 0.25050214797123427, 0.11913979139451825], [0.6011271462578217, 0.6240268129488139, 0.26460794633119367, 0.05733546763708264]]], [[[0.607769674684382, 0.10598819875815713, 0.8776962546840539, 0.8266601773987386], [0.8631846579822211, 0.6879685601086099, 0.1359518845716131, 0.4685454581791084], [0.3209883310787902, 0.4737321179870807, 0.6413260281499608, 0.060790725664807366]], [[0.5956883363196818, 0.07328419173849654, 0.8490340074264021, 0.12305983025717881], [0.9629571076901358, 0.8103596371491041, 0.14296043169605432, 0.8455997248569967], [0.566533587998251, 0.4079548923236669, 0.09480821134398254, 0.8453438861794155]]], [[[0.36555150410471093, 0.3009926174449653, 0.9326518297055033, 0.452395179058795], [0.045232604076099636, 0.8196996070484879, 0.4118988069767353, 0.5721464132856435], [0.1023427638050095, 0.7868280835389061, 0.25359288064036933, 0.44768371695910847]], [[0.10293864817984721, 0.43875987030641805, 0.22119598335766544, 0.5248768126420456], [0.5996872812495732, 0.07227556382845923, 0.18190141013260508, 0.6082154732630453], [0.2897180173479402, 0.17483245717532458, 0.9368027518821712, 0.416391916226614]]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactOne_fromConstant_onFunctionOnContactOne_rank0(self):
      """
      test interploation of constant rank 0 Data on ReducedFunctionOnContactOne
      """
      w_ref = ReducedFunctionOnContactOne(self.domain)
      w_arg = FunctionOnContactOne(self.domain)
      val = 5.976848e-01
      arg = Data(val,(),w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactOne_fromConstant_onFunctionOnContactOne_rank1(self):
      """
      test interploation of constant rank 1 Data on ReducedFunctionOnContactOne
      """
      w_ref = ReducedFunctionOnContactOne(self.domain)
      w_arg = FunctionOnContactOne(self.domain)
      val = numpy.array([0.9979220795470324, 0.913058303488773, 0.9238045481549146] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactOne_fromConstant_onFunctionOnContactOne_rank2(self):
      """
      test interploation of constant rank 2 Data on ReducedFunctionOnContactOne
      """
      w_ref = ReducedFunctionOnContactOne(self.domain)
      w_arg = FunctionOnContactOne(self.domain)
      val = numpy.array([[0.3760218998871382, 0.5495501353286991], [0.7733345507339668, 0.6655487841256854], [0.6270627445510198, 0.5850690403436837], [0.8377177857529043, 0.37950806864436704]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactOne_fromConstant_onFunctionOnContactOne_rank3(self):
      """
      test interploation of constant rank 3 Data on ReducedFunctionOnContactOne
      """
      w_ref = ReducedFunctionOnContactOne(self.domain)
      w_arg = FunctionOnContactOne(self.domain)
      val = numpy.array([[[0.8125672122559341, 0.1469570456460404, 0.2593339404377025, 0.07208068660362044], [0.8069143731585875, 0.6618452104435641, 0.35155373733752715, 0.16464730345004108]], [[0.9621221120677534, 0.0765861810496069, 0.6277671674055675, 0.963676802059102], [0.10360342520174248, 0.28692024730085797, 0.987611605315119, 0.07931181270683207]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2, 2, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onReducedFunctionOnContactOne_fromConstant_onFunctionOnContactOne_rank4(self):
      """
      test interploation of constant rank 4 Data on ReducedFunctionOnContactOne
      """
      w_ref = ReducedFunctionOnContactOne(self.domain)
      w_arg = FunctionOnContactOne(self.domain)
      val = numpy.array([[[[0.25191897836414423, 0.42126915999112624, 0.7154530949286638, 0.9952365016642971], [0.14662642318396835, 0.7041962973650627, 0.78731215708391, 0.6924357889858904], [0.22827464902688777, 0.7586553848074509, 0.032078529068980566, 0.3212866838599123]], [[0.8731211780918547, 0.6196786661571002, 0.1727227903088394, 0.6346700793467431], [0.015548189946719582, 0.368431899850422, 0.0976598151032797, 0.8252867801788962], [0.9103820506794112, 0.3008037192898215, 0.30540005256297176, 0.27780553926348106]]], [[[0.4757913279940317, 0.2987358078874637, 0.22024292330759987, 0.6229773193005934], [0.922087290469399, 0.892782366331258, 0.5013328004922624, 0.10353773684686463], [0.4895535641548977, 0.8859070541418946, 0.3441366078907462, 0.43419961139201135]], [[0.3088896761180112, 0.5396803247776865, 0.5177493168768168, 0.2875982176137747], [0.8998467954699237, 0.30598290872935097, 0.7523320401608645, 0.6906490971864888], [0.032724534976539266, 0.6958518818724602, 0.1189430874786982, 0.05054464101685496]]], [[[0.2601400928708387, 0.6672691969164184, 0.4520504631150756, 0.5050320420635327], [0.08433613825917274, 0.048789921774640144, 0.12160750905727158, 0.8856044469408108], [0.38082038045179945, 0.4684120562278291, 0.6266534901363282, 0.010003341349897221]], [[0.5532664995018765, 0.7699851130858237, 0.4078565149425506, 0.4962712401556637], [0.1300875433267774, 0.8378778498446537, 0.4411427823165137, 0.3336098908203802], [0.800484509122118, 0.9218586595145473, 0.7032447055420981, 0.355658645314631]]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onFunctionOnContactZero_fromConstant_onSolution_rank0(self):
      """
      test interploation of constant rank 0 Data on FunctionOnContactZero
      """
      w_ref = FunctionOnContactZero(self.domain)
      w_arg = Solution(self.domain)
      val = 4.416713e-01
      arg = Data(val,(),w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onFunctionOnContactZero_fromConstant_onSolution_rank1(self):
      """
      test interploation of constant rank 1 Data on FunctionOnContactZero
      """
      w_ref = FunctionOnContactZero(self.domain)
      w_arg = Solution(self.domain)
      val = numpy.array([0.2397685947326934, 0.7097890472485217, 0.9189098860849187] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onFunctionOnContactZero_fromConstant_onSolution_rank2(self):
      """
      test interploation of constant rank 2 Data on FunctionOnContactZero
      """
      w_ref = FunctionOnContactZero(self.domain)
      w_arg = Solution(self.domain)
      val = numpy.array([[0.17602046944924, 0.7956413543116365], [0.15543862150310883, 0.2482828822749814], [0.1538718613145098, 0.6530094362725264], [0.1514329050794636, 0.5257034960240661]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onFunctionOnContactZero_fromConstant_onSolution_rank3(self):
      """
      test interploation of constant rank 3 Data on FunctionOnContactZero
      """
      w_ref = FunctionOnContactZero(self.domain)
      w_arg = Solution(self.domain)
      val = numpy.array([[[0.4902557912844012, 0.440304985891957, 0.9463253262661137, 0.5715680435601652], [0.8140505952959916, 0.47358128328833893, 0.996854593429773, 0.13513348645448808]], [[0.22438828201655725, 0.6934202420714549, 0.5177454252945897, 0.1372828042658516], [0.6665024488555925, 0.8896168906819952, 0.4059514060217976, 0.48127708062411934]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2, 2, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onFunctionOnContactZero_fromConstant_onSolution_rank4(self):
      """
      test interploation of constant rank 4 Data on FunctionOnContactZero
      """
      w_ref = FunctionOnContactZero(self.domain)
      w_arg = Solution(self.domain)
      val = numpy.array([[[[0.5198503742148101, 0.23333916232844198, 0.45481213788554153, 0.6641616556715114], [0.5025725242121142, 0.7338599485235121, 0.8027874427683204, 0.04573894917401766], [0.06675459781397808, 0.04327693142551525, 0.9214794390300253, 0.673903707702856]], [[0.0021792127542202833, 0.36032368666476733, 0.8700110328075396, 0.3093369106906919], [0.22764719174857673, 0.10616278045101124, 0.1325575760664841, 0.46319086008170507], [0.9828152112752936, 0.6431208639782472, 0.7605833063626741, 0.6791836108384837]]], [[[0.9706310538066029, 0.6736311634646226, 0.20946760175041101, 0.2186355830532991], [0.030648945491146273, 0.12217926635715637, 0.7531427688395137, 0.07597018308112102], [0.9798259336546012, 0.4127985068978618, 0.17669965605585392, 0.22457811769077984]], [[0.6626413306808323, 0.7112410623943974, 0.2822240910976339, 0.2978993862038072], [0.3575751872844851, 0.1718195504274722, 0.09946553575943917, 0.31405656569563933], [0.15235623461362402, 0.6764695857236684, 0.4415189794350217, 0.4692313872286529]]], [[[0.5571194011015072, 0.13980736379490633, 0.828462961394972, 0.6520829063879348], [0.18968066803128192, 0.30163313060691166, 0.3329757076456661, 0.983952227909519], [0.3135321066461725, 0.3773481332917946, 0.9420848004040188, 0.08147895844855246]], [[0.481955104520406, 0.5055841393922007, 0.3444043816788931, 0.027329674792177294], [0.7097385615922654, 0.6189655499331437, 0.1308682118965171, 0.515859974532883], [0.05355114050128118, 0.18673031877411306, 0.19931819211666413, 0.9013240937315466]]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onFunctionOnContactZero_fromConstant_onReducedSolution_rank0(self):
      """
      test interploation of constant rank 0 Data on FunctionOnContactZero
      """
      w_ref = FunctionOnContactZero(self.domain)
      w_arg = ReducedSolution(self.domain)
      val = 7.510911e-01
      arg = Data(val,(),w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onFunctionOnContactZero_fromConstant_onReducedSolution_rank1(self):
      """
      test interploation of constant rank 1 Data on FunctionOnContactZero
      """
      w_ref = FunctionOnContactZero(self.domain)
      w_arg = ReducedSolution(self.domain)
      val = numpy.array([0.630645745721905, 0.8939684421289026, 0.9601162395275649] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onFunctionOnContactZero_fromConstant_onReducedSolution_rank2(self):
      """
      test interploation of constant rank 2 Data on FunctionOnContactZero
      """
      w_ref = FunctionOnContactZero(self.domain)
      w_arg = ReducedSolution(self.domain)
      val = numpy.array([[0.02126952766669543, 0.4458741782828476], [0.3809314199301166, 0.11701990827994091], [0.873777780491416, 0.8230600714159015], [0.5028610747815478, 0.09887432015946263]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onFunctionOnContactZero_fromConstant_onReducedSolution_rank3(self):
      """
      test interploation of constant rank 3 Data on FunctionOnContactZero
      """
      w_ref = FunctionOnContactZero(self.domain)
      w_arg = ReducedSolution(self.domain)
      val = numpy.array([[[0.8966507131291467, 0.286358857446412, 0.4800362539759593, 0.6495513851425094], [0.5178427919521975, 0.01531019627616903, 0.5689211306550106, 0.36217620346944623]], [[0.8502537388147945, 0.6857268648635135, 0.27215688726081644, 0.7716588833363628], [0.4473005431632585, 0.5635508146243543, 0.18066137241072755, 0.8482767005377161]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2, 2, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onFunctionOnContactZero_fromConstant_onReducedSolution_rank4(self):
      """
      test interploation of constant rank 4 Data on FunctionOnContactZero
      """
      w_ref = FunctionOnContactZero(self.domain)
      w_arg = ReducedSolution(self.domain)
      val = numpy.array([[[[0.2750355293806127, 0.5390467395895852, 0.8439927541767266, 0.5090111628883961], [0.27350493869815096, 0.37273228754252774, 0.1662453090753061, 0.13560569579061832], [0.7354386331965185, 0.051108003331235885, 0.06031589755776645, 0.30154602205947545]], [[0.6920775172275642, 0.8501136792107697, 0.21023395859900362, 0.956730824374313], [0.1859320728226913, 0.07787267769488615, 0.5130302327416403, 0.3400327980674329], [0.042897263695016785, 0.3818048048698155, 0.6392209318676889, 0.12165560206540982]]], [[[0.18344558842091896, 0.7619916973943744, 0.4909055613828005, 0.07804749944815503], [0.07929391923541218, 0.716420484177726, 0.2581418542426863, 0.07807784079948155], [0.9950388762434933, 0.4004671220915821, 0.6829278998786378, 0.38203800041773084]], [[0.5575655055312282, 0.12066084517531162, 0.8239805798960135, 0.2330351600266607], [0.2764800102169849, 0.5290179635749851, 0.3774607516731834, 0.22767364978997406], [0.6006583474371304, 0.06202361570967674, 0.6882843425453093, 0.18431133952073864]]], [[[0.5655563514285874, 0.78224967695824, 0.5842534247879523, 0.3436621666271471], [0.27003028864710354, 0.13333711261716463, 0.8666408847862285, 0.9839257744154463], [0.4637395857695982, 0.07962639384265635, 0.18918116101042848, 0.7291141678302442]], [[0.35801260833911974, 0.07696594370037868, 0.8058358324912938, 0.9954433534714624], [0.6075857423675662, 0.2182879784226538, 0.8181979449596574, 0.4222063110465062], [0.47275862738828045, 0.060720581084092395, 0.004203445385625226, 0.2439379081924069]]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onFunctionOnContactZero_fromConstant_onContinuousFunction_rank0(self):
      """
      test interploation of constant rank 0 Data on FunctionOnContactZero
      """
      w_ref = FunctionOnContactZero(self.domain)
      w_arg = ContinuousFunction(self.domain)
      val = 2.992490e-01
      arg = Data(val,(),w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onFunctionOnContactZero_fromConstant_onContinuousFunction_rank1(self):
      """
      test interploation of constant rank 1 Data on FunctionOnContactZero
      """
      w_ref = FunctionOnContactZero(self.domain)
      w_arg = ContinuousFunction(self.domain)
      val = numpy.array([0.9084081458069398, 0.9610955917455746, 0.8974375039525531] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onFunctionOnContactZero_fromConstant_onContinuousFunction_rank2(self):
      """
      test interploation of constant rank 2 Data on FunctionOnContactZero
      """
      w_ref = FunctionOnContactZero(self.domain)
      w_arg = ContinuousFunction(self.domain)
      val = numpy.array([[0.36406610333741973, 0.7193115171409343], [0.49520067354065733, 0.4648841623168536], [0.5691510386664795, 0.3835616339040089], [0.2894998741692758, 0.37981618394481365]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onFunctionOnContactZero_fromConstant_onContinuousFunction_rank3(self):
      """
      test interploation of constant rank 3 Data on FunctionOnContactZero
      """
      w_ref = FunctionOnContactZero(self.domain)
      w_arg = ContinuousFunction(self.domain)
      val = numpy.array([[[0.33657633210799043, 0.8515767774529702, 0.8583862148104472, 0.6285488042554537], [0.11901978484130082, 0.04043558134072045, 0.11153632753267007, 0.008877017782832008]], [[0.8754297146483313, 0.7855330600535763, 0.8609253136976776, 0.09305497491150616], [0.22851466737323767, 0.8538173044233048, 0.3508007751425648, 0.05235128240361597]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2, 2, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onFunctionOnContactZero_fromConstant_onContinuousFunction_rank4(self):
      """
      test interploation of constant rank 4 Data on FunctionOnContactZero
      """
      w_ref = FunctionOnContactZero(self.domain)
      w_arg = ContinuousFunction(self.domain)
      val = numpy.array([[[[0.37153387003771776, 0.8493606594813612, 0.054706437487670456, 0.0924185624762155], [0.7226040985185594, 0.3132258446969316, 0.07404727361065322, 0.006450500963350092], [0.3815230607784271, 0.4018203119540481, 0.7616728010837144, 0.5850116319392834]], [[0.8925192223767027, 0.15133391013460795, 0.6984644061929081, 0.09425477063877874], [0.003491070140467567, 0.060347953448054326, 0.47564528258681327, 0.8066401750422154], [0.5600675174531321, 0.24591977989820601, 0.8690581332718511, 0.21111001513338168]]], [[[0.1281761411140654, 0.07227128229669033, 0.13401495978275313, 0.3413758890063746], [0.9174989644207403, 0.745648533903282, 0.7828693889445822, 0.5933107087595659], [0.7497147035078707, 0.45508225240513633, 0.03457097069305781, 0.9104863861267437]], [[0.9793179113174781, 0.6727202614912273, 0.06399081591456579, 0.2222167201613352], [0.7271571253320678, 0.08183536821421089, 0.816260427759079, 0.3457936525026265], [0.4868492856427924, 0.32433352456137887, 0.21167787570248076, 0.0581860749528339]]], [[[0.9831713825535134, 0.07509969410304307, 0.07490358211827342, 0.03282166725581881], [0.12044168486481666, 0.515154457439558, 0.2839219961190307, 0.5630409332887808], [0.06183068822143256, 0.31030913114218295, 0.16646500351640292, 0.5675143797937996]], [[0.4191681625715874, 0.8074590639967545, 0.07825860740935309, 0.15450090906733305], [0.720588407598866, 0.3649309179206196, 0.9216552026450284, 0.4021699442404362], [0.5983140824706527, 0.05297429697060296, 0.5158481932356002, 0.4119755592458536]]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onFunctionOnContactZero_fromConstant_onReducedContinuousFunction_rank0(self):
      """
      test interploation of constant rank 0 Data on FunctionOnContactZero
      """
      w_ref = FunctionOnContactZero(self.domain)
      w_arg = ReducedContinuousFunction(self.domain)
      val = 8.252940e-03
      arg = Data(val,(),w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onFunctionOnContactZero_fromConstant_onReducedContinuousFunction_rank1(self):
      """
      test interploation of constant rank 1 Data on FunctionOnContactZero
      """
      w_ref = FunctionOnContactZero(self.domain)
      w_arg = ReducedContinuousFunction(self.domain)
      val = numpy.array([0.3448076505076355, 0.6900313488599504, 0.10567828073717866] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onFunctionOnContactZero_fromConstant_onReducedContinuousFunction_rank2(self):
      """
      test interploation of constant rank 2 Data on FunctionOnContactZero
      """
      w_ref = FunctionOnContactZero(self.domain)
      w_arg = ReducedContinuousFunction(self.domain)
      val = numpy.array([[0.35470766202138115, 0.09986094598261086], [0.25266424864387704, 0.8779949525205841], [0.25042177170139934, 0.9036032068029195], [0.2008528682884816, 0.2378173838456351]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onFunctionOnContactZero_fromConstant_onReducedContinuousFunction_rank3(self):
      """
      test interploation of constant rank 3 Data on FunctionOnContactZero
      """
      w_ref = FunctionOnContactZero(self.domain)
      w_arg = ReducedContinuousFunction(self.domain)
      val = numpy.array([[[0.3273639347586249, 0.9298261012414404, 0.913701750025211, 0.4736803614698052], [0.7635912130688315, 0.13183737655423222, 0.7760816387793189, 0.4818779091034959]], [[0.7352230033431076, 0.7815834720194401, 0.6304488851014184, 0.8742794921950976], [0.5281738455853388, 0.2752856031288444, 0.377977956892526, 0.013672625494558499]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2, 2, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onFunctionOnContactZero_fromConstant_onReducedContinuousFunction_rank4(self):
      """
      test interploation of constant rank 4 Data on FunctionOnContactZero
      """
      w_ref = FunctionOnContactZero(self.domain)
      w_arg = ReducedContinuousFunction(self.domain)
      val = numpy.array([[[[0.9806721863738455, 0.9420397905945826, 0.4759653563480256, 0.6838085617199985], [0.7235282182444596, 0.43366565885057873, 0.8112074102767002, 0.8284908996938819], [0.02974965437016075, 0.697613323352396, 0.5186387902996138, 0.04365161154110808]], [[0.5832773206987517, 0.9810209479115547, 0.6061630212424512, 0.46604203254036913], [0.8135907770650561, 0.8031912402292878, 0.29726297081314557, 0.21332985568864593], [0.42801658535045817, 0.7534302538432589, 0.6211769880606737, 0.9192451559580421]]], [[[0.6844579365139096, 0.20548191610037114, 0.7468354520285635, 0.7630304942291364], [0.5313483043271173, 0.25313222404919977, 0.9073834488147124, 0.0715532276750892], [0.7732593974641134, 0.14943053384767535, 0.7162983381990314, 0.49004571909471895]], [[0.8198603739899627, 0.36728047400449204, 0.5123151807849293, 0.9828245159299462], [0.35736608493209954, 0.9939711062328335, 0.12255412576671498, 0.4431013295983658], [0.47455302203504757, 0.9657076599292064, 0.9215547472021081, 0.5422555904839185]]], [[[0.7192153597996084, 0.9007403952562474, 0.579916428899817, 0.43747395368978603], [0.5733828993717659, 0.7961898331137294, 0.7205227665391735, 0.09394967176864544], [0.43995237266279985, 0.1339366322472404, 0.7840868152248345, 0.8102066218380822]], [[0.06692787457453286, 0.5475617072862115, 0.9191194246666381, 0.6476330805729692], [0.21633461620688066, 0.5717077212234045, 0.3388888775576444, 0.10502341187582986], [0.5909013124923022, 0.8178615500689735, 0.34284763698586973, 0.38611240571854644]]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onFunctionOnContactZero_fromConstant_onFunctionOnContactZero_rank0(self):
      """
      test interploation of constant rank 0 Data on FunctionOnContactZero
      """
      w_ref = FunctionOnContactZero(self.domain)
      w_arg = FunctionOnContactZero(self.domain)
      val = 7.327513e-01
      arg = Data(val,(),w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onFunctionOnContactZero_fromConstant_onFunctionOnContactZero_rank1(self):
      """
      test interploation of constant rank 1 Data on FunctionOnContactZero
      """
      w_ref = FunctionOnContactZero(self.domain)
      w_arg = FunctionOnContactZero(self.domain)
      val = numpy.array([0.7231166515226765, 0.5841319152801158, 0.8902160789797313] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onFunctionOnContactZero_fromConstant_onFunctionOnContactZero_rank2(self):
      """
      test interploation of constant rank 2 Data on FunctionOnContactZero
      """
      w_ref = FunctionOnContactZero(self.domain)
      w_arg = FunctionOnContactZero(self.domain)
      val = numpy.array([[0.31666674128214967, 0.8082352527788402], [0.5527436152270833, 0.9022399259157667], [0.09592541580574276, 0.2921831081543108], [0.4304143124615589, 0.10331014717749942]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onFunctionOnContactZero_fromConstant_onFunctionOnContactZero_rank3(self):
      """
      test interploation of constant rank 3 Data on FunctionOnContactZero
      """
      w_ref = FunctionOnContactZero(self.domain)
      w_arg = FunctionOnContactZero(self.domain)
      val = numpy.array([[[0.15595661516165404, 0.7833734244427264, 0.24540644204391682, 0.1542925747281464], [0.15838583560617558, 0.7395401696574047, 0.7346352689491014, 0.24458004081631357]], [[0.19927127813017875, 0.8777192033205472, 0.8491994511001953, 0.4703062874568794], [0.980759804700044, 0.9682583138081405, 0.3790846035822203, 0.38138319170544466]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2, 2, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onFunctionOnContactZero_fromConstant_onFunctionOnContactZero_rank4(self):
      """
      test interploation of constant rank 4 Data on FunctionOnContactZero
      """
      w_ref = FunctionOnContactZero(self.domain)
      w_arg = FunctionOnContactZero(self.domain)
      val = numpy.array([[[[0.20419114009277073, 0.647175983842068, 0.5093547151876396, 0.9216011526617293], [0.42295910976817985, 0.7188670182816705, 0.7741150536748513, 0.45558965220901193], [0.03436003707429747, 0.07563143784432569, 0.48620344230809576, 0.8013645733575203]], [[0.7771578779880213, 0.19603575918509386, 0.5985466948715691, 0.828958278366855], [0.0938459606708103, 0.19187906528756593, 0.01662095587451151, 0.15028937565734368], [0.034012959068946524, 0.35696919619085865, 0.12750367788423345, 0.9458667600166505]]], [[[0.22776111844779323, 0.74809180845036, 0.06489892587426582, 0.8765495727006761], [0.5693912992359508, 0.41255204446644667, 0.7066014359982783, 0.9717171105845629], [0.6876321588836807, 0.201805999400562, 0.5208661607283092, 0.26638579370241744]], [[0.8755482739394753, 0.20448437653898544, 0.4944880065113454, 0.03192709625044665], [0.6751522218320636, 0.6008625798947532, 0.6395416148567048, 0.3754171935071611], [0.33160648108815993, 0.02753911680324972, 0.3281591135689743, 0.8736536966446298]]], [[[0.33033444631590414, 0.9802368565740771, 0.3604057339737867, 0.4530725735113721], [0.20764202465902182, 0.9977143081423786, 0.9781748640481389, 0.28129650096176884], [0.3811869625421732, 0.9110908868512202, 0.8997281186553494, 0.8398212773868227]], [[0.8337359476242638, 0.24012391654438614, 0.30322675360573226, 0.4998867082796524], [0.11277484849791797, 0.7619896431886289, 0.4162911610580866, 0.5064994338014415], [0.8550867003183041, 0.4168470403089801, 0.18660226726284013, 0.37137042986275126]]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onFunctionOnContactZero_fromConstant_onFunctionOnContactOne_rank0(self):
      """
      test interploation of constant rank 0 Data on FunctionOnContactZero
      """
      w_ref = FunctionOnContactZero(self.domain)
      w_arg = FunctionOnContactOne(self.domain)
      val = 8.486111e-01
      arg = Data(val,(),w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onFunctionOnContactZero_fromConstant_onFunctionOnContactOne_rank1(self):
      """
      test interploation of constant rank 1 Data on FunctionOnContactZero
      """
      w_ref = FunctionOnContactZero(self.domain)
      w_arg = FunctionOnContactOne(self.domain)
      val = numpy.array([0.19349620530877265, 0.5866471986054227, 0.6787889731753596] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onFunctionOnContactZero_fromConstant_onFunctionOnContactOne_rank2(self):
      """
      test interploation of constant rank 2 Data on FunctionOnContactZero
      """
      w_ref = FunctionOnContactZero(self.domain)
      w_arg = FunctionOnContactOne(self.domain)
      val = numpy.array([[0.2822602679507009, 0.40533825534315016], [0.7258825121048694, 0.5324694570640633], [0.36909609843304925, 0.12198043686742], [0.12075237403480787, 0.5785326226174939]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onFunctionOnContactZero_fromConstant_onFunctionOnContactOne_rank3(self):
      """
      test interploation of constant rank 3 Data on FunctionOnContactZero
      """
      w_ref = FunctionOnContactZero(self.domain)
      w_arg = FunctionOnContactOne(self.domain)
      val = numpy.array([[[0.5346208611352831, 0.4469842740453335, 0.8051916060864622, 0.48733380565029016], [0.8307328127369256, 0.9393115411216822, 0.4398715323854856, 0.29522037647769916]], [[0.5870016848993143, 0.019098167430426027, 0.6116361915920812, 0.8720224112887579], [0.572976558198275, 0.49941509000507023, 0.5946267689626442, 0.6475179416827223]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2, 2, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onFunctionOnContactZero_fromConstant_onFunctionOnContactOne_rank4(self):
      """
      test interploation of constant rank 4 Data on FunctionOnContactZero
      """
      w_ref = FunctionOnContactZero(self.domain)
      w_arg = FunctionOnContactOne(self.domain)
      val = numpy.array([[[[0.9274001658794875, 0.28129554243035704, 0.8781639418523312, 0.2502775019483543], [0.7044955363081367, 0.9041118440883114, 0.3602052270922288, 0.9985565147392614], [0.5972379373665277, 0.06713057365947417, 0.3620165408330318, 0.9772450466979303]], [[0.07320303071616574, 0.9460091082631562, 0.6608561394784608, 0.896991807046738], [0.7057319905818371, 0.8650091980891809, 0.5950323580975542, 0.8086663075937696], [0.5485181344188715, 0.9445866937441508, 0.3153107315109378, 0.36858289115803966]]], [[[0.7786183386104515, 0.3609547326506787, 0.02311328299808979, 0.9530050392474433], [0.1446153236837675, 0.9190223271985497, 0.24760153477578672, 0.2796250506324369], [0.901051983379246, 0.11340072399800483, 0.39826787824937204, 0.8644036183549159]], [[0.0030006812566425456, 0.5197289474173273, 0.12424284697360144, 0.9422373979654418], [0.04495511401683372, 0.03315171261125649, 0.18692678365674031, 0.056481667996537754], [0.9377011841037844, 0.4731040233528967, 0.5373841539527175, 0.25562355372378565]]], [[[0.2537851697115757, 0.14094081127476776, 0.010205458967384562, 0.20765140118592673], [0.02999979580974721, 0.11038237323218791, 0.8244473412279174, 0.5810733992018395], [0.9967498428234133, 0.7796913893645318, 0.4926493661339262, 0.1907508420561247]], [[0.5485212280712575, 0.6002896061014056, 0.47343878848488685, 0.6739775413957486], [0.22382265206362462, 0.5212412074684933, 0.5298640334376036, 0.3071447604012596], [0.42094268186296824, 0.8827363291256063, 0.9730196003317173, 0.5689645278272888]]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onFunctionOnContactOne_fromConstant_onSolution_rank0(self):
      """
      test interploation of constant rank 0 Data on FunctionOnContactOne
      """
      w_ref = FunctionOnContactOne(self.domain)
      w_arg = Solution(self.domain)
      val = 5.767294e-01
      arg = Data(val,(),w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onFunctionOnContactOne_fromConstant_onSolution_rank1(self):
      """
      test interploation of constant rank 1 Data on FunctionOnContactOne
      """
      w_ref = FunctionOnContactOne(self.domain)
      w_arg = Solution(self.domain)
      val = numpy.array([0.9393272777113492, 0.3566768496456404, 0.5950175292092031] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onFunctionOnContactOne_fromConstant_onSolution_rank2(self):
      """
      test interploation of constant rank 2 Data on FunctionOnContactOne
      """
      w_ref = FunctionOnContactOne(self.domain)
      w_arg = Solution(self.domain)
      val = numpy.array([[0.24432369184270464, 0.51048589333877], [0.651880378266647, 0.7871665354232306], [0.8156590150712001, 0.607738119865395], [0.7271726943916907, 0.1811763770522743]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onFunctionOnContactOne_fromConstant_onSolution_rank3(self):
      """
      test interploation of constant rank 3 Data on FunctionOnContactOne
      """
      w_ref = FunctionOnContactOne(self.domain)
      w_arg = Solution(self.domain)
      val = numpy.array([[[0.4893404027379529, 0.7143945993639663, 0.2800095632387746, 0.03077209934310221], [0.09741763000806669, 0.9523828574035264, 0.6150161165584151, 0.07683227286346894]], [[0.41437984289378427, 0.8512176451950799, 0.19676008983148308, 0.6302950649094535], [0.2566098357459874, 0.8458667868726177, 0.726098072274147, 0.22583237649855714]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2, 2, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onFunctionOnContactOne_fromConstant_onSolution_rank4(self):
      """
      test interploation of constant rank 4 Data on FunctionOnContactOne
      """
      w_ref = FunctionOnContactOne(self.domain)
      w_arg = Solution(self.domain)
      val = numpy.array([[[[0.33039432387156287, 0.8483347767807963, 0.3038197358056276, 0.3173098177329524], [0.2617638288301254, 0.2379318906112151, 0.9277003165528401, 0.26256000875539154], [0.49140753839399287, 0.7944876328294765, 0.5673496116390729, 0.7484953495934313]], [[0.15357034592966234, 0.22551372099204892, 0.44522517076017576, 0.7589320701726646], [0.020031921785506612, 0.8980125272861668, 0.5927958056182403, 0.6626111290187529], [0.16366391987541307, 0.7866885462330552, 0.2519000928757915, 0.6237889465762602]]], [[[0.3265396885581284, 0.40298064992746185, 0.9851664096048879, 0.7456683000131761], [0.4044007732457938, 0.9318879431120379, 0.49611085891547846, 0.9086679158503379], [0.7819868747485897, 0.37812640508932105, 0.08203140235780115, 0.4091759221026524]], [[0.2638073355435474, 0.7097331633234422, 0.11199217920819804, 0.8332891900252846], [0.7289120583934607, 0.8044279698358008, 0.6202194678103693, 0.9047066999710591], [0.7347242082064213, 0.6815120792749805, 0.5714184226956788, 0.9289316135807777]]], [[[0.5843985120672339, 0.3585269600730254, 0.849389223566183, 0.5846480341029401], [0.9290824575336621, 0.27583542447387044, 0.028879475459195447, 0.38621936577061455], [0.2451827909850518, 0.5787995107442441, 0.2579913053469185, 0.24571157118058595]], [[0.745288740109279, 0.8716589775031495, 0.9306469293450405, 0.9614185299822352], [0.3867084350612561, 0.053525113091472676, 0.15825875323679062, 0.5391251968145623], [0.3222790115730473, 0.9376764774926398, 0.2243389246548435, 0.6395499141302641]]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onFunctionOnContactOne_fromConstant_onReducedSolution_rank0(self):
      """
      test interploation of constant rank 0 Data on FunctionOnContactOne
      """
      w_ref = FunctionOnContactOne(self.domain)
      w_arg = ReducedSolution(self.domain)
      val = 1.909980e-02
      arg = Data(val,(),w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onFunctionOnContactOne_fromConstant_onReducedSolution_rank1(self):
      """
      test interploation of constant rank 1 Data on FunctionOnContactOne
      """
      w_ref = FunctionOnContactOne(self.domain)
      w_arg = ReducedSolution(self.domain)
      val = numpy.array([0.9310097157024845, 0.9598516961599882, 0.5426786184009288] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onFunctionOnContactOne_fromConstant_onReducedSolution_rank2(self):
      """
      test interploation of constant rank 2 Data on FunctionOnContactOne
      """
      w_ref = FunctionOnContactOne(self.domain)
      w_arg = ReducedSolution(self.domain)
      val = numpy.array([[0.05119923247920832, 0.41079030463174626], [0.7601386446015862, 0.850554845330653], [0.4423350538349472, 0.1180775929815242], [0.9840637068902307, 0.5097944715854054]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onFunctionOnContactOne_fromConstant_onReducedSolution_rank3(self):
      """
      test interploation of constant rank 3 Data on FunctionOnContactOne
      """
      w_ref = FunctionOnContactOne(self.domain)
      w_arg = ReducedSolution(self.domain)
      val = numpy.array([[[0.6929617892145945, 0.8564860009098837, 0.09862451701848529, 0.9927354003452259], [0.618567040058047, 0.8109588623837427, 0.43985230793568897, 0.696936365283216]], [[0.7713616009191991, 0.7370842239587223, 0.5144762584826544, 0.6902963812144104], [0.08556994290820097, 0.290021600059007, 0.0675561465235971, 0.5666089364555084]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2, 2, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onFunctionOnContactOne_fromConstant_onReducedSolution_rank4(self):
      """
      test interploation of constant rank 4 Data on FunctionOnContactOne
      """
      w_ref = FunctionOnContactOne(self.domain)
      w_arg = ReducedSolution(self.domain)
      val = numpy.array([[[[0.8040132408324383, 0.05740157341271068, 0.9954811313394446, 0.9307313044733667], [0.8832816764924746, 0.9088044114136531, 0.09793082858191304, 0.7900115380295009], [0.44859461087747887, 0.7701805309926891, 0.8976019059724086, 0.6149980378552519]], [[0.9040711526397776, 0.7345393514222155, 0.36130436244331954, 0.0750344731625231], [0.10125172271598892, 0.23032718083162607, 0.32180275306924067, 0.2731072430360305], [0.5309712933425926, 0.5194240836074977, 0.2094979496910092, 0.06582580878343891]]], [[[0.15170388765293186, 0.1831511098441868, 0.1918488201791595, 0.918359165512001], [0.250085591384253, 0.4744329151161065, 0.36773681691550497, 0.17016247242238114], [0.7036352886702043, 0.8125491445623759, 0.6720672838705962, 0.0465285745172509]], [[0.09433260987485459, 0.55826558565799, 0.996072681664486, 0.4443346482960586], [0.3951568444418436, 0.138621122057483, 0.7198860321672503, 0.8962949629973527], [0.2681882998995653, 0.05745511213300669, 0.4605882554882842, 0.8685845444503997]]], [[[0.6421953408785401, 0.1757052625120029, 0.21783918421892112, 0.4200436710487724], [0.08834583002749208, 0.19910931327412618, 0.4102105366367709, 0.23533298054459706], [0.5375102589162664, 0.9755080674596786, 0.608682365886696, 0.15439981768265232]], [[0.7088614099110924, 0.29990237337124237, 0.0331339120812264, 0.7154129057225462], [0.7536666092699825, 0.43789075473153205, 0.32489584680417394, 0.03214215964398259], [0.12473044124285826, 0.041151436003503905, 0.9143708537634556, 0.05046875283984975]]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onFunctionOnContactOne_fromConstant_onContinuousFunction_rank0(self):
      """
      test interploation of constant rank 0 Data on FunctionOnContactOne
      """
      w_ref = FunctionOnContactOne(self.domain)
      w_arg = ContinuousFunction(self.domain)
      val = 4.772256e-01
      arg = Data(val,(),w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onFunctionOnContactOne_fromConstant_onContinuousFunction_rank1(self):
      """
      test interploation of constant rank 1 Data on FunctionOnContactOne
      """
      w_ref = FunctionOnContactOne(self.domain)
      w_arg = ContinuousFunction(self.domain)
      val = numpy.array([0.13970398691552355, 0.9685019005154762, 0.5091845224613416] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onFunctionOnContactOne_fromConstant_onContinuousFunction_rank2(self):
      """
      test interploation of constant rank 2 Data on FunctionOnContactOne
      """
      w_ref = FunctionOnContactOne(self.domain)
      w_arg = ContinuousFunction(self.domain)
      val = numpy.array([[0.7871247247714709, 0.4695168272735891], [0.08315880520569163, 0.26457851850854774], [0.8627336131338595, 0.39168832860705083], [0.84345293538415, 0.044640616676462574]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onFunctionOnContactOne_fromConstant_onContinuousFunction_rank3(self):
      """
      test interploation of constant rank 3 Data on FunctionOnContactOne
      """
      w_ref = FunctionOnContactOne(self.domain)
      w_arg = ContinuousFunction(self.domain)
      val = numpy.array([[[0.482642338632141, 0.8133489743365346, 0.7090061264352878, 0.674660169309214], [0.7792521170475486, 0.9375790250660871, 0.5621369310673493, 0.14196279393188171]], [[0.84621492908988, 0.4315664433371823, 0.7105047101494437, 0.9630127116493553], [0.0017345619067646867, 0.14408799614484125, 0.8672034081224578, 0.40599645665431994]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2, 2, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onFunctionOnContactOne_fromConstant_onContinuousFunction_rank4(self):
      """
      test interploation of constant rank 4 Data on FunctionOnContactOne
      """
      w_ref = FunctionOnContactOne(self.domain)
      w_arg = ContinuousFunction(self.domain)
      val = numpy.array([[[[0.5025528596467772, 0.6639044803784455, 0.7321936771945932, 0.34396887853487434], [0.3662036100865744, 0.5936347591405934, 0.014391334956976864, 0.1760581286699141], [0.35159192836421216, 0.6131387874422899, 0.8423297697861831, 0.6837804146854264]], [[0.6634900483232942, 0.5239579363473771, 0.4395513950457304, 0.8118070231181496], [0.45523155926102055, 0.7812539148111305, 0.34628795056330886, 0.4809875858330689], [0.6439429777579342, 0.6442238897350978, 0.7881666686221916, 0.8828193937940547]]], [[[0.6412203008644388, 0.6741491134777143, 0.3563794759848351, 0.08352673628003027], [0.6210802083734595, 0.10126987104907625, 0.571281771572459, 0.9818461369488507], [0.15981768383543915, 0.515869829097547, 0.3177613527023123, 0.2270022094614812]], [[0.5225431656130042, 0.948241003786963, 0.8906849122716235, 0.7971527713971847], [0.43011343017143766, 0.8595959184508085, 0.4542020402828335, 0.5253599956035548], [0.12182693232141639, 0.8323040539160632, 0.8244243763405522, 0.10774558353482466]]], [[[0.5531251667837781, 0.6896056884916075, 0.4074899302049373, 0.07062460757672595], [0.5003130136895001, 0.7714971613073536, 0.7820503331058146, 0.474015037770826], [0.6983050632376289, 0.4512337129830044, 0.8416082320993941, 0.31337018427982755]], [[0.15284708758251342, 0.13715548456750493, 0.7545244238170862, 0.03940687094955919], [0.03550432950140048, 0.43858877707216637, 0.015608392276873162, 0.21322764326121024], [0.8286592986359806, 0.02604881256602587, 0.7560447750002263, 0.5518618067515372]]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onFunctionOnContactOne_fromConstant_onReducedContinuousFunction_rank0(self):
      """
      test interploation of constant rank 0 Data on FunctionOnContactOne
      """
      w_ref = FunctionOnContactOne(self.domain)
      w_arg = ReducedContinuousFunction(self.domain)
      val = 4.366381e-01
      arg = Data(val,(),w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onFunctionOnContactOne_fromConstant_onReducedContinuousFunction_rank1(self):
      """
      test interploation of constant rank 1 Data on FunctionOnContactOne
      """
      w_ref = FunctionOnContactOne(self.domain)
      w_arg = ReducedContinuousFunction(self.domain)
      val = numpy.array([0.8729896511743298, 0.8592284116201467, 0.7249011457001252] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onFunctionOnContactOne_fromConstant_onReducedContinuousFunction_rank2(self):
      """
      test interploation of constant rank 2 Data on FunctionOnContactOne
      """
      w_ref = FunctionOnContactOne(self.domain)
      w_arg = ReducedContinuousFunction(self.domain)
      val = numpy.array([[0.9592621284831397, 0.43537457044602057], [0.22424733361843718, 0.3967925848358177], [0.8816116559042844, 0.6277335225763199], [0.609653528118983, 0.5582539018213913]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onFunctionOnContactOne_fromConstant_onReducedContinuousFunction_rank3(self):
      """
      test interploation of constant rank 3 Data on FunctionOnContactOne
      """
      w_ref = FunctionOnContactOne(self.domain)
      w_arg = ReducedContinuousFunction(self.domain)
      val = numpy.array([[[0.8408978798560507, 0.9211923180120242, 0.23506769537534655, 0.14874650599058048], [0.6836138689798581, 0.249682325647547, 0.27166576825803423, 0.08586679061342994]], [[0.5615086339973592, 0.8632454549187649, 0.815910076982812, 0.33757568528084614], [0.1341937252940094, 0.039849570412994306, 0.603971777156599, 0.5051028816540584]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2, 2, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onFunctionOnContactOne_fromConstant_onReducedContinuousFunction_rank4(self):
      """
      test interploation of constant rank 4 Data on FunctionOnContactOne
      """
      w_ref = FunctionOnContactOne(self.domain)
      w_arg = ReducedContinuousFunction(self.domain)
      val = numpy.array([[[[0.39042920883550436, 0.09654008742627618, 0.23459468057374977, 0.4997760056783489], [0.9803093359121334, 0.25420567781552883, 0.1384271894889244, 0.652054174046505], [0.6420115372187916, 0.4680093623067527, 0.5545188168878056, 0.8648494463441425]], [[0.2128663216940293, 0.07161356306800726, 0.4159331514787875, 0.42787781855117224], [0.8955262758183068, 0.9575458214197974, 0.4640998721416092, 0.7518401550693129], [0.2724439565287011, 0.24540644244699927, 0.5825295453488123, 0.493757576415915]]], [[[0.2912381581584985, 0.22265719076874357, 0.045927255048914706, 0.502348793920519], [0.14271563481258098, 0.5497283215674386, 0.5011592561560229, 0.3846001208453744], [0.41602958815663127, 0.642881597452492, 0.6389435318509104, 0.7841619838979582]], [[0.822288808976087, 0.9116382463298777, 0.2097320377905011, 0.7615548009091255], [0.5556861573959306, 0.19196072042728163, 0.17980432324983586, 0.636290474090794], [0.1150202543227441, 0.31489520423947726, 0.009489938552266208, 0.3706298964935535]]], [[[0.22628832098316465, 0.06939177618540027, 0.3969239286087368, 0.6363514673381082], [0.9514645526648707, 0.5461826847258049, 0.8724483901590938, 0.15251094070894067], [0.3532567056040524, 0.9447858094976036, 0.5272639786528185, 0.8567212545183848]], [[0.979529498684061, 0.2396888342803456, 0.6268859620853611, 0.8435251665352165], [0.08638429623459332, 0.39944184404828165, 0.7730346446831876, 0.6139768824821263], [0.6467634930821072, 0.7952116992062654, 0.3068758449420531, 0.599335205309747]]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onFunctionOnContactOne_fromConstant_onFunctionOnContactZero_rank0(self):
      """
      test interploation of constant rank 0 Data on FunctionOnContactOne
      """
      w_ref = FunctionOnContactOne(self.domain)
      w_arg = FunctionOnContactZero(self.domain)
      val = 5.001400e-01
      arg = Data(val,(),w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onFunctionOnContactOne_fromConstant_onFunctionOnContactZero_rank1(self):
      """
      test interploation of constant rank 1 Data on FunctionOnContactOne
      """
      w_ref = FunctionOnContactOne(self.domain)
      w_arg = FunctionOnContactZero(self.domain)
      val = numpy.array([0.3446244000495903, 0.593446268134426, 0.49417928509040543] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onFunctionOnContactOne_fromConstant_onFunctionOnContactZero_rank2(self):
      """
      test interploation of constant rank 2 Data on FunctionOnContactOne
      """
      w_ref = FunctionOnContactOne(self.domain)
      w_arg = FunctionOnContactZero(self.domain)
      val = numpy.array([[0.7396858665244294, 0.2591321450931059], [0.9138555289206731, 0.5655255068410112], [0.42702057599396614, 0.5538374647017625], [0.30882993796761027, 0.25520699740792707]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onFunctionOnContactOne_fromConstant_onFunctionOnContactZero_rank3(self):
      """
      test interploation of constant rank 3 Data on FunctionOnContactOne
      """
      w_ref = FunctionOnContactOne(self.domain)
      w_arg = FunctionOnContactZero(self.domain)
      val = numpy.array([[[0.7814574177959093, 0.2885240146553637, 0.07602748917796198, 0.9434537117594162], [0.11453942642341597, 0.6536688083758768, 0.6652350210110864, 0.7007107129601465]], [[0.5159704693548254, 0.08926010595869083, 0.2951171897121806, 0.6056882959408596], [0.977098129471714, 0.3353343460652275, 0.07160809848615957, 0.2081406400615552]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2, 2, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onFunctionOnContactOne_fromConstant_onFunctionOnContactZero_rank4(self):
      """
      test interploation of constant rank 4 Data on FunctionOnContactOne
      """
      w_ref = FunctionOnContactOne(self.domain)
      w_arg = FunctionOnContactZero(self.domain)
      val = numpy.array([[[[0.5744208457569366, 0.6276911535940791, 0.4890217568852586, 0.4453990558743598], [0.7413339898808041, 0.3080648676735249, 0.2292644941988884, 0.5558197054430838], [0.27257577986078196, 0.709889311378699, 0.6676133429451673, 0.059791577291348874]], [[0.3321024058463743, 0.5548229915715125, 0.9071852420920024, 0.5798147595802225], [0.06682655216594979, 0.768613476658211, 0.8713989894148532, 0.3730797680034109], [0.3211194675035839, 0.23320985956220253, 0.4834356403659402, 0.4634951292311712]]], [[[0.6196225298971869, 0.7470473319754436, 0.45984942093693193, 0.11062821685289281], [0.7537048674141694, 0.01801585170950748, 0.9888570901074166, 0.7355681564250827], [0.3076423493263659, 0.617722273682797, 0.11707057330441095, 0.7523131779840234]], [[0.23172405807175178, 0.1577333122816379, 0.2485127649785972, 0.720628815651124], [0.936358956992718, 0.282849981634144, 0.7363872911265287, 0.6015518382081952], [0.8752776813276878, 0.2996279485812928, 0.13888614408354494, 0.08344348318416006]]], [[[0.34750149160171884, 0.7564914031965304, 0.027245770926107582, 0.5757061321651941], [0.4958948557109324, 0.7598995318359827, 0.42848139698339205, 0.016097918927411303], [0.9262882998672358, 0.7883925724033938, 0.6080922556620691, 0.6129944159801674]], [[0.9537961812064384, 0.134407738154536, 0.5126105369886191, 0.7111473155163776], [0.9153864068679111, 0.9074431910467052, 0.5483818056916192, 0.9309973883497766], [0.492291954922345, 0.35451124254040933, 0.3561789012121256, 0.4475198980036986]]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onFunctionOnContactOne_fromConstant_onFunctionOnContactOne_rank0(self):
      """
      test interploation of constant rank 0 Data on FunctionOnContactOne
      """
      w_ref = FunctionOnContactOne(self.domain)
      w_arg = FunctionOnContactOne(self.domain)
      val = 3.346270e-01
      arg = Data(val,(),w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onFunctionOnContactOne_fromConstant_onFunctionOnContactOne_rank1(self):
      """
      test interploation of constant rank 1 Data on FunctionOnContactOne
      """
      w_ref = FunctionOnContactOne(self.domain)
      w_arg = FunctionOnContactOne(self.domain)
      val = numpy.array([0.19203830022700397, 0.29393614853109507, 0.2927788998014659] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onFunctionOnContactOne_fromConstant_onFunctionOnContactOne_rank2(self):
      """
      test interploation of constant rank 2 Data on FunctionOnContactOne
      """
      w_ref = FunctionOnContactOne(self.domain)
      w_arg = FunctionOnContactOne(self.domain)
      val = numpy.array([[0.17099980479408938, 0.16660487899571363], [0.2818948355524076, 0.9580210257403371], [0.17920860817050355, 0.2286630073717001], [0.9643645186245067, 0.8783395586003125]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onFunctionOnContactOne_fromConstant_onFunctionOnContactOne_rank3(self):
      """
      test interploation of constant rank 3 Data on FunctionOnContactOne
      """
      w_ref = FunctionOnContactOne(self.domain)
      w_arg = FunctionOnContactOne(self.domain)
      val = numpy.array([[[0.9393739922305128, 0.19033728614543044, 0.9770734231776671, 0.610912509126845], [0.6461057892571009, 0.14149710667856086, 0.1682075722230867, 0.9403474792723889]], [[0.8705275436933966, 0.5929405985046501, 0.6542425702772899, 0.878195309490558], [0.2812100926557496, 0.3801011519043189, 0.8529775975930917, 0.7474370026506547]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2, 2, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")
   def test_interpolate_onFunctionOnContactOne_fromConstant_onFunctionOnContactOne_rank4(self):
      """
      test interploation of constant rank 4 Data on FunctionOnContactOne
      """
      w_ref = FunctionOnContactOne(self.domain)
      w_arg = FunctionOnContactOne(self.domain)
      val = numpy.array([[[[0.4230767108903867, 0.4194336862093251, 0.688756986286944, 0.9701998352992696], [0.3599725018981642, 0.4395651214979335, 0.39130830157406016, 0.35147621827791853], [0.06676920163915534, 0.22791986401980824, 0.475673299314337, 0.4026017674132487]], [[0.056366134507255916, 0.5741273447458412, 0.9519753634152845, 0.12822909211098554], [0.250119203532574, 0.6873115416399119, 0.5464406708874457, 0.7048880010667673], [0.635829361569282, 0.9342534453355489, 0.4911655802655164, 0.11099925917174758]]], [[[0.3131259925504235, 0.33032635347931416, 0.32002829136391986, 0.7341514812029089], [0.6248535441990961, 0.6864103652729907, 0.12363851297631567, 0.5799878793698392], [0.4708522022126608, 0.05850695501535841, 0.9175964491583588, 0.4316970415686543]], [[0.610380902656425, 0.7211048369384372, 0.7224220277694913, 0.9191863141386084], [0.4186251049927596, 0.7684041665858556, 0.14007754319815924, 0.8225602727877354], [0.5176881718188399, 0.9338567942806608, 0.4955060506872614, 0.504093908616572]]], [[[0.5617998823403688, 0.7395782993178911, 0.2832526237426829, 0.9167996032937219], [0.6024532952820588, 0.35446724141697894, 0.17631123315292152, 0.9664887509724667], [0.9637263605470135, 0.8739945809801134, 0.38677356757481307, 0.0813036805382904]], [[0.43864783261193485, 0.7036563752902615, 0.04635412572939557, 0.2755758826348347], [0.27150481049277986, 0.28020769195516304, 0.9719022050658143, 0.21077693846298318], [0.40391255123233794, 0.38188301804600777, 0.23516998176641224, 0.23820706739800213]]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=interpolate(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-val)<=self.RES_TOL*Lsup(val),"wrong result")

